(function(/*! Brunch !*/) {
  'use strict';

  var globals = typeof window !== 'undefined' ? window : global;
  if (typeof globals.require === 'function') return;

  var modules = {};
  var cache = {};

  var has = function(object, name) {
    return ({}).hasOwnProperty.call(object, name);
  };

  var expand = function(root, name) {
    var results = [], parts, part;
    if (/^\.\.?(\/|$)/.test(name)) {
      parts = [root, name].join('/').split('/');
    } else {
      parts = name.split('/');
    }
    for (var i = 0, length = parts.length; i < length; i++) {
      part = parts[i];
      if (part === '..') {
        results.pop();
      } else if (part !== '.' && part !== '') {
        results.push(part);
      }
    }
    return results.join('/');
  };

  var dirname = function(path) {
    return path.split('/').slice(0, -1).join('/');
  };

  var localRequire = function(path) {
    return function(name) {
      var dir = dirname(path);
      var absolute = expand(dir, name);
      return globals.require(absolute, path);
    };
  };

  var initModule = function(name, definition) {
    var module = {id: name, exports: {}};
    cache[name] = module;
    definition(module.exports, localRequire(name), module);
    return module.exports;
  };

  var require = function(name, loaderPath) {
    var path = expand(name, '.');
    if (loaderPath == null) loaderPath = '/';

    if (has(cache, path)) return cache[path].exports;
    if (has(modules, path)) return initModule(path, modules[path]);

    var dirIndex = expand(path, './index');
    if (has(cache, dirIndex)) return cache[dirIndex].exports;
    if (has(modules, dirIndex)) return initModule(dirIndex, modules[dirIndex]);

    throw new Error('Cannot find module "' + name + '" from '+ '"' + loaderPath + '"');
  };

  var define = function(bundle, fn) {
    if (typeof bundle === 'object') {
      for (var key in bundle) {
        if (has(bundle, key)) {
          modules[key] = bundle[key];
        }
      }
    } else {
      modules[bundle] = fn;
    }
  };

  var list = function() {
    var result = [];
    for (var item in modules) {
      if (has(modules, item)) {
        result.push(item);
      }
    }
    return result;
  };

  globals.require = require;
  globals.require.define = define;
  globals.require.register = define;
  globals.require.list = list;
  globals.require.brunch = true;
})();
require.register("app", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

if (Ember.$.uuid === undefined) {
  Ember.$.uuid = 0;
}

// Application bootstrapper
require('utils/bootstrap_reopen');
require('utils/ember_reopen');
require('utils/ember_computed');
var stringUtils = require('utils/string_utils');
var stompClientClass = require('utils/stomp_client');

module.exports = Em.Application.create({
  name: 'Ambari Web',
  rootElement: '#wrapper',

  store: DS.Store.create({
    revision: 4,
    adapter: DS.FixtureAdapter.create({
      simulateRemoteResponse: false
    }),
    typeMaps: {},
    recordCache: []
  }),
  StompClient: stompClientClass.create(),
  isAdmin: false,
  isOperator: false,
  isClusterUser: false,
  isPermissionDataLoaded: false,
  auth: undefined,
  isOnlyViewUser: function () {
    return App.auth && (App.auth.length == 0 || App.isAuthorized('VIEW.USE') && App.auth.length == 1);
  }.property('auth'),

  /**
   * @type {boolean}
   * @default false
   */
  isKerberosEnabled: false,

  /**
   * state of stack upgrade process
   * states:
   *  - NOT_REQUIRED
   *  - PENDING
   *  - IN_PROGRESS
   *  - HOLDING
   *  - COMPLETED
   *  - ABORTED
   *  - HOLDING_FAILED
   *  - HOLDING_TIMEDOUT
   * @type {String}
   */
  upgradeState: 'NOT_REQUIRED',

  /**
   * Check if upgrade is in INIT state
   * 'INIT' is set on upgrade start and when it's finished
   * @type {boolean}
   */
  upgradeInit: Em.computed.equal('upgradeState', 'NOT_REQUIRED'),

  /**
   * flag is true when upgrade process is running
   * @returns {boolean}
   */
  upgradeInProgress: Em.computed.equal('upgradeState', 'IN_PROGRESS'),

  /**
   * Checks if update process is completed
   * @type {boolean}
   */
  upgradeCompleted: Em.computed.equal('upgradeState', 'COMPLETED'),

  /**
   * flag is true when upgrade process is waiting for user action
   * to proceed, retry, perform manual steps etc.
   * @returns {boolean}
   */
  upgradeHolding: function () {
    return this.get('upgradeState').contains("HOLDING") || this.get('upgradeAborted');
  }.property('upgradeState', 'upgradeAborted'),

  /**
   * flag is true when upgrade process is aborted
   * SHOULD behave similar to HOLDING_FAILED state
   * @returns {boolean}
   */
  upgradeAborted: function () {
    return this.get('upgradeState') === "ABORTED" && !App.router.get('mainAdminStackAndUpgradeController.isSuspended');
  }.property('upgradeState', 'router.mainAdminStackAndUpgradeController.isSuspended'),

  /**
   * flag is true when upgrade process is suspended
   * @returns {boolean}
   */
  upgradeSuspended: function () {
    return this.get('upgradeState') === "ABORTED" && App.router.get('mainAdminStackAndUpgradeController.isSuspended');
  }.property('upgradeState', 'router.mainAdminStackAndUpgradeController.isSuspended'),

  /**
   * RU is running
   * @type {boolean}
   */
  upgradeIsRunning: Em.computed.or('upgradeInProgress', 'upgradeHolding'),

  /**
   * flag is true when upgrade process is running or suspended
   * or wizard used by another user
   * @returns {boolean}
   */
  wizardIsNotFinished: function () {
    return this.get('upgradeIsRunning') || this.get('upgradeSuspended') || App.router.get('wizardWatcherController.isNonWizardUser');
  }.property('upgradeIsRunning', 'upgradeAborted', 'router.wizardWatcherController.isNonWizardUser', 'upgradeSuspended'),

  /**
   * @param {string} authRoles
   * @returns {boolean}
   */
  havePermissions: function havePermissions(authRoles) {
    var result = false;
    authRoles = $.map(authRoles.split(","), $.trim);

    // When Upgrade running(not suspended) only operations related to upgrade should be allowed
    if (!this.get('upgradeSuspended') && !authRoles.contains('CLUSTER.UPGRADE_DOWNGRADE_STACK') && !authRoles.contains('CLUSTER.MANAGE_USER_PERSISTED_DATA') && !App.get('supports.opsDuringRollingUpgrade') && !['NOT_REQUIRED', 'COMPLETED'].contains(this.get('upgradeState')) || !App.auth) {
      return false;
    }

    authRoles.forEach(function (auth) {
      result = result || App.auth.contains(auth);
    });

    return result;
  },
  /**
   * @param {string} authRoles
   * @returns {boolean}
   */
  isAuthorized: function isAuthorized(authRoles) {
    return this.havePermissions(authRoles) && !App.router.get('wizardWatcherController.isNonWizardUser');
  },

  isStackServicesLoaded: false,
  /**
   * return url prefix with number value of version of HDP stack
   */
  stackVersionURL: function () {
    return '/stacks/{0}/versions/{1}'.format(this.get('currentStackName') || 'HDP', this.get('currentStackVersionNumber'));
  }.property('currentStackName', 'currentStackVersionNumber'),

  falconServerURL: function () {
    var falconService = this.Service.find().findProperty('serviceName', 'FALCON');
    if (falconService) {
      return falconService.get('hostComponents').findProperty('componentName', 'FALCON_SERVER').get('hostName');
    }
    return '';
  }.property().volatile(),

  /* Determine if Application Timeline Service supports Kerberization.
   * Because this value is retrieved from the cardinality of the component, it is safe to keep in app.js
   * since its value will not change during the lifetime of the application.
   */
  doesATSSupportKerberos: function () {
    var YARNService = App.StackServiceComponent.find().filterProperty('serviceName', 'YARN');
    if (YARNService.length) {
      var ATS = App.StackServiceComponent.find().findProperty('componentName', 'APP_TIMELINE_SERVER');
      return !!ATS && !!ATS.get('minToInstall');
    }
    return false;
  }.property('router.clusterController.isLoaded'),

  clusterId: null,
  clusterName: null,
  clockDistance: null, // server clock - client clock
  currentStackVersion: '',
  currentStackName: function () {
    return Em.get((this.get('currentStackVersion') || this.get('defaultStackVersion')).match(/(.+)-\d.+/), '1');
  }.property('currentStackVersion', 'defaultStackVersion'),

  /**
   * true if cluster has only 1 host
   * for now is used to disable move/HA actions
   * @type {boolean}
   */
  isSingleNode: Em.computed.equal('allHostNames.length', 1),

  allHostNames: [],

  /**
   * This object is populated to keep track of uninstalled components to be included in the layout for recommendation/validation call
   * @type {object}
   * keys = componentName, hostName
   */
  componentToBeAdded: {},

  /**
   * This object is populated to keep track of installed components to be excluded in the layout for recommendation/validation call
   * @type {object}
   * keys = componentName, hostName
   */
  componentToBeDeleted: {},

  uiOnlyConfigDerivedFromTheme: [],

  currentStackVersionNumber: function () {
    var regExp = new RegExp(this.get('currentStackName') + '-');
    return (this.get('currentStackVersion') || this.get('defaultStackVersion')).replace(regExp, '');
  }.property('currentStackVersion', 'defaultStackVersion', 'currentStackName'),

  isHadoopWindowsStack: Em.computed.equal('currentStackName', 'HDPWIN'),

  /**
   * If NameNode High Availability is enabled
   * Based on <code>clusterStatus.isInstalled</code>, stack version, <code>SNameNode</code> availability
   *
   * @type {bool}
   */
  isHaEnabled: function () {
    return App.Service.find('HDFS').get('isLoaded') && !App.HostComponent.find().someProperty('componentName', 'SECONDARY_NAMENODE');
  }.property('router.clusterController.dataLoadList.services', 'router.clusterController.isServiceContentFullyLoaded'),

  hasNameNodeFederation: function () {
    return App.HDFSService.find('HDFS').get('masterComponentGroups.length') > 1;
  }.property('router.clusterController.isHostComponentMetricsLoaded', 'router.clusterController.isHDFSNameSpacesLoaded'),

  /**
   * If ResourceManager High Availability is enabled
   * Based on number of ResourceManager host components installed
   *
   * @type {bool}
   */
  isRMHaEnabled: function () {
    var result = false;
    var rmStackComponent = App.StackServiceComponent.find().findProperty('componentName', 'RESOURCEMANAGER');
    if (rmStackComponent && rmStackComponent.get('isMultipleAllowed')) {
      result = this.HostComponent.find().filterProperty('componentName', 'RESOURCEMANAGER').length > 1;
    }
    return result;
  }.property('router.clusterController.isLoaded', 'isStackServicesLoaded'),

  /**
   * If Ranger Admin High Availability is enabled
   * Based on number of Ranger Admin host components installed
   *
   * @type {bool}
   */
  isRAHaEnabled: function () {
    var result = false;
    var raStackComponent = App.StackServiceComponent.find().findProperty('componentName', 'RANGER_ADMIN');
    if (raStackComponent && raStackComponent.get('isMultipleAllowed')) {
      result = App.HostComponent.find().filterProperty('componentName', 'RANGER_ADMIN').length > 1;
    }
    return result;
  }.property('router.clusterController.isLoaded', 'isStackServicesLoaded'),

  /**
   * Object with utility functions for list of service names with similar behavior
   */
  services: Em.Object.create({
    all: function () {
      return App.StackService.find().mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    clientOnly: function () {
      return App.StackService.find().filterProperty('isClientOnlyService').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    hasClient: function () {
      return App.StackService.find().filterProperty('hasClient').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    hasMaster: function () {
      return App.StackService.find().filterProperty('hasMaster').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    hasSlave: function () {
      return App.StackService.find().filterProperty('hasSlave').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    noConfigTypes: function () {
      return App.StackService.find().filterProperty('isNoConfigTypes').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    servicesWithHeatmapTab: function () {
      return App.StackService.find().filterProperty('hasHeatmapSection').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    monitoring: function () {
      return App.StackService.find().filterProperty('isMonitoringService').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    hostMetrics: function () {
      return App.StackService.find().filterProperty('isHostMetricsService').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    serviceMetrics: function () {
      return App.StackService.find().filterProperty('isServiceMetricsService').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    supportsServiceCheck: function () {
      return App.StackService.find().filterProperty('serviceCheckSupported').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded'),

    supportsDeleteViaUI: function () {
      return App.StackService.find().filterProperty('supportDeleteViaUi').mapProperty('serviceName');
    }.property('App.router.clusterController.isLoaded')
  }),

  /**
   * List of components with allowed action for them
   * @type {Em.Object}
   */
  components: Em.Object.create({
    isMasterAddableOnlyOnHA: function () {
      return App.StackServiceComponent.find().filterProperty('isMasterAddableOnlyOnHA').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    allComponents: function () {
      return App.StackServiceComponent.find().mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    reassignable: function () {
      return App.StackServiceComponent.find().filterProperty('isReassignable').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    restartable: function () {
      return App.StackServiceComponent.find().filterProperty('isRestartable').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    deletable: function () {
      return App.StackServiceComponent.find().filterProperty('isDeletable').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    rollinRestartAllowed: function () {
      return App.StackServiceComponent.find().filterProperty('isRollinRestartAllowed').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    decommissionAllowed: function () {
      return App.StackServiceComponent.find().filterProperty('isDecommissionAllowed').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    refreshConfigsAllowed: function () {
      return App.StackServiceComponent.find().filterProperty('isRefreshConfigsAllowed').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    addableToHost: function () {
      return App.StackServiceComponent.find().filterProperty('isAddableToHost').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    addableMasterInstallerWizard: function () {
      return App.StackServiceComponent.find().filterProperty('isMasterAddableInstallerWizard').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    multipleMasters: function () {
      return App.StackServiceComponent.find().filterProperty('isMasterWithMultipleInstances').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    slaves: function () {
      return App.StackServiceComponent.find().filterProperty('isSlave').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    masters: function () {
      return App.StackServiceComponent.find().filterProperty('isMaster').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    clients: function () {
      return App.StackServiceComponent.find().filterProperty('isClient').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded'),

    nonHDP: function () {
      return App.StackServiceComponent.find().filterProperty('isNonHDPComponent').mapProperty('componentName');
    }.property('App.router.clusterController.isLoaded')
  })
});

});

require.register("config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.version = '2.7.7.0'; // filled out by set-ambari-version.sh script
App.testMode = location.port == '3333'; // test mode is automatically enabled if running on brunch server
App.testModeDelayForActions = 10000;
App.skipBootstrap = false;
App.alwaysGoToInstaller = false;
App.testEnableSecurity = true; // By default enable security is tested; turning it false tests disable security
App.testNameNodeHA = true;
App.appURLRoot = '{proxy_root}/'.replace(/\{.+\}/g, ''); // determines application root path name, not related to hash route
App.apiPrefix = '/api/v1';
App.defaultStackVersion = 'HDP-2.3';
App.defaultWindowsStackVersion = 'HDPWIN-2.1';

App.defaultJavaHome = '/usr/jdk/jdk1.6.0_31';
App.timeout = 300000; // default AJAX timeout
App.maxRetries = 3; // max number of retries for certain AJAX calls
App.sessionKeepAliveInterval = 60000;
App.bgOperationsUpdateInterval = 6000;
App.componentsUpdateInterval = 6000;
App.contentUpdateInterval = 15000;
App.hostStatusCountersUpdateInterval = 10000;
App.alertDefinitionsUpdateInterval = 10000;
App.alertInstancesUpdateInterval = 10000;
App.alertGroupsUpdateInterval = 10000;
App.clusterEnvUpdateInterval = 10000;
App.pageReloadTime = 3600000;
App.nnCheckpointAgeAlertThreshold = 12; // in hours
App.minDiskSpace = 2.0; // minimum disk space required for '/' for each host before install, unit GB
App.minDiskSpaceUsrLib = 1.0; // minimum disk space for '/usr/lib' for each host before install, unit GB
App.healthIconClassGreen = 'glyphicon glyphicon-ok-sign'; // bootstrap icon class for healthy/started service/host/host-component
App.healthIconClassRed = 'glyphicon glyphicon-warning-sign'; // bootstrap icon class for master down/stopped service/host/host-component
App.healthIconClassOrange = 'glyphicon glyphicon-minus-sign'; // bootstrap icon class for slave down/decommissioned host/host-component
App.healthIconClassYellow = 'glyphicon glyphicon-question-sign'; // bootstrap icon class for heartbeat lost service/host/host-component
App.isManagedMySQLForHiveEnabled = false;
App.isStormMetricsSupported = true;
App.healthStatusRed = '#EF6162';
App.healthStatusGreen = '#1EB475';
App.healthStatusOrange = '#E98A40';
App.widgetContentColor = '#666666';
App.inactivityRemainTime = 60; // in seconds
App.enableLogger = true;
App.stackVersionsAvailable = true;
App.upgradeHistoryAvailable = false;
App.enableDigitalClock = false;

// experimental features are automatically enabled if running on brunch server
App.enableExperimental = false;

App.supports = {
  preUpgradeCheck: true,
  displayOlderVersions: false,
  autoRollbackHA: false,
  alwaysEnableManagedMySQLForHive: false,
  preKerberizeCheck: false,
  customizeAgentUserAccount: false,
  installGanglia: false,
  opsDuringRollingUpgrade: false,
  customizedWidgetLayout: false,
  showPageLoadTime: false,
  skipComponentStartAfterInstall: false,
  preInstallChecks: false,
  serviceAutoStart: true,
  logSearch: true,
  redhatSatellite: false,
  addingNewRepository: false,
  kerberosStackAdvisor: true,
  logCountVizualization: false,
  createAlerts: false,
  enabledWizardForHostOrderedUpgrade: true,
  manageJournalNode: true,
  enableToggleKerberos: true,
  enableAddDeleteServices: true,
  regenerateKeytabsOnSingleHost: false,
  disableCredentialsAutocompleteForRepoUrls: false
};

if (App.enableExperimental) {
  for (var support in App.supports) {
    App.supports[support] = true;
  }
}

// this is to make sure that IE does not cache data when making AJAX calls to the server
if (!$.mocho) {
  $.ajaxSetup({
    cache: false,
    headers: { "X-Requested-By": "X-Requested-By" }
  });
}

/**
 * Test Mode values
 */
App.test_hostname = 'hostname';

});

require.register("controllers", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load all controllers here

require('controllers/application');
require('controllers/login_controller');
require('controllers/wizard');
require('controllers/installer');
require('controllers/experimental');
require('controllers/global/background_operations_controller');
require('controllers/global/wizard_watcher_controller');
require('controllers/global/user_settings_controller');
require('controllers/global/errors_handler_controller');
require('controllers/main');
require('controllers/main/dashboard');
require('controllers/main/dashboard/config_history_controller');
require('controllers/main/admin');
require('controllers/main/admin/service_auto_start');
require('controllers/main/admin/highAvailability_controller');
require('controllers/main/admin/highAvailability/nameNode/wizard_controller');
require('controllers/main/admin/highAvailability/progress_controller');
require('controllers/main/admin/highAvailability/progress_popup_controller');
require('controllers/main/admin/highAvailability/nameNode/rollback_controller');
require('controllers/main/admin/highAvailability/nameNode/step1_controller');
require('controllers/main/admin/highAvailability/nameNode/step2_controller');
require('controllers/main/admin/highAvailability/nameNode/step3_controller');
require('controllers/main/admin/highAvailability/nameNode/step4_controller');
require('controllers/main/admin/highAvailability/nameNode/step5_controller');
require('controllers/main/admin/highAvailability/nameNode/step6_controller');
require('controllers/main/admin/highAvailability/nameNode/step7_controller');
require('controllers/main/admin/highAvailability/nameNode/step8_controller');
require('controllers/main/admin/highAvailability/nameNode/step9_controller');
require('controllers/main/admin/highAvailability/nameNode/rollbackHA/step1_controller');
require('controllers/main/admin/highAvailability/nameNode/rollbackHA/step2_controller');
require('controllers/main/admin/highAvailability/nameNode/rollbackHA/step3_controller');
require('controllers/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard_controller');
require('controllers/main/admin/highAvailability/resourceManager/wizard_controller');
require('controllers/main/admin/highAvailability/resourceManager/step1_controller');
require('controllers/main/admin/highAvailability/resourceManager/step2_controller');
require('controllers/main/admin/highAvailability/resourceManager/step3_controller');
require('controllers/main/admin/highAvailability/resourceManager/step4_controller');
require('controllers/main/admin/federation/wizard_controller');
require('controllers/main/admin/federation/step1_controller');
require('controllers/main/admin/federation/step2_controller');
require('controllers/main/admin/federation/step3_controller');
require('controllers/main/admin/federation/step4_controller');
require('controllers/main/admin/highAvailability/hawq/addStandby/wizard_controller');
require('controllers/main/admin/highAvailability/hawq/addStandby/step1_controller');
require('controllers/main/admin/highAvailability/hawq/addStandby/step2_controller');
require('controllers/main/admin/highAvailability/hawq/addStandby/step3_controller');
require('controllers/main/admin/highAvailability/hawq/addStandby/step4_controller');
require('controllers/main/admin/highAvailability/hawq/removeStandby/wizard_controller');
require('controllers/main/admin/highAvailability/hawq/removeStandby/step1_controller');
require('controllers/main/admin/highAvailability/hawq/removeStandby/step2_controller');
require('controllers/main/admin/highAvailability/hawq/removeStandby/step3_controller');
require('controllers/main/admin/highAvailability/hawq/activateStandby/wizard_controller');
require('controllers/main/admin/highAvailability/hawq/activateStandby/step1_controller');
require('controllers/main/admin/highAvailability/hawq/activateStandby/step2_controller');
require('controllers/main/admin/highAvailability/hawq/activateStandby/step3_controller');
require('controllers/main/admin/highAvailability/rangerAdmin/wizard_controller');
require('controllers/main/admin/highAvailability/rangerAdmin/step1_controller');
require('controllers/main/admin/highAvailability/rangerAdmin/step2_controller');
require('controllers/main/admin/highAvailability/rangerAdmin/step3_controller');
require('controllers/main/admin/highAvailability/rangerAdmin/step4_controller');
require('controllers/main/admin/highAvailability/journalNode/wizard_controller');
require('controllers/main/admin/highAvailability/journalNode/progress_controller');
require('controllers/main/admin/highAvailability/journalNode/step1_controller');
require('controllers/main/admin/highAvailability/journalNode/step2_controller');
require('controllers/main/admin/highAvailability/journalNode/step3_controller');
require('controllers/main/admin/highAvailability/journalNode/step4_controller');
require('controllers/main/admin/highAvailability/journalNode/step5_controller');
require('controllers/main/admin/highAvailability/journalNode/step6_controller');
require('controllers/main/admin/highAvailability/journalNode/step7_controller');
require('controllers/main/admin/stack_and_upgrade_controller');
require('controllers/main/admin/stack_upgrade_history_controller');
require('controllers/main/admin/serviceAccounts_controller');
require('utils/polling');
require('controllers/main/admin/kerberos');
require('controllers/main/admin/kerberos/wizard_controller');
require('controllers/main/admin/kerberos/disable_controller');
require('controllers/main/admin/kerberos/progress_controller');
require('controllers/main/admin/kerberos/step1_controller');
require('controllers/main/admin/kerberos/step2_controller');
require('controllers/main/admin/kerberos/step3_controller');
require('controllers/main/admin/kerberos/step4_controller');
require('controllers/main/admin/kerberos/step5_controller');
require('controllers/main/admin/kerberos/step6_controller');
require('controllers/main/admin/kerberos/step7_controller');
require('controllers/main/admin/kerberos/step8_controller');
require('controllers/main/alert_definitions_controller');
require('controllers/main/alerts/alert_definitions_actions_controller');
require('controllers/main/alerts/add_alert_definition/add_alert_definition_controller');
require('controllers/main/alerts/add_alert_definition/step1_controller');
require('controllers/main/alerts/add_alert_definition/step2_controller');
require('controllers/main/alerts/add_alert_definition/step3_controller');
require('controllers/main/alerts/definition_details_controller');
require('controllers/main/alerts/definition_configs_controller');
require('controllers/main/alerts/alert_instances_controller');
require('controllers/main/alerts/manage_alert_groups_controller');
require('controllers/main/alerts/manage_alert_notifications_controller');
require('controllers/main/service');
require('controllers/main/service/item');
require('controllers/main/service/info/summary');
require('controllers/main/service/info/configs');
require('controllers/main/service/info/audit');
require('controllers/main/service/add_controller');
require('controllers/main/service/reassign_controller');
require('controllers/main/service/reassign/step1_controller');
require('controllers/main/service/reassign/step2_controller');
require('controllers/main/service/reassign/step3_controller');
require('controllers/main/service/reassign/step4_controller');
require('controllers/main/service/reassign/step5_controller');
require('controllers/main/service/reassign/step6_controller');
require('controllers/main/service/manage_config_groups_controller');
require('controllers/main/service/widgets/create/wizard_controller');
require('controllers/main/service/widgets/create/step1_controller');
require('controllers/main/service/widgets/create/step2_controller');
require('controllers/main/service/widgets/create/step3_controller');
require('controllers/main/service/widgets/edit_controller');
require('controllers/main/host');
require('controllers/main/host/bulk_operations_controller');
require('controllers/main/host/details');
require('controllers/main/host/configs_service');
require('controllers/main/host/add_controller');
require('controllers/main/host/combo_search_box');
require('controllers/main/host/addHost/step4_controller');
require('controllers/main/host/host_alerts_controller');
require('controllers/main/charts');
require('controllers/main/charts/heatmap_metrics/heatmap_metric');
require('controllers/main/charts/heatmap');
require('controllers/main/service/info/heatmap');
require('controllers/main/service/info/metric');
require('controllers/main/views_controller');
require('controllers/main/views/details_controller');
require('controllers/wizard/step0_controller');
require('controllers/wizard/step1_controller');
require('controllers/wizard/step2_controller');
require('controllers/wizard/step3_controller');
require('controllers/wizard/step4_controller');
require('controllers/wizard/step5_controller');
require('controllers/wizard/step6_controller');
require('controllers/wizard/step7_controller');
require('controllers/wizard/step7/assign_master_controller');
require('controllers/wizard/step7/pre_install_checks_controller');
require('controllers/wizard/step8_controller');
require('controllers/wizard/step9_controller');
require('controllers/wizard/step10_controller');
require('controllers/global/cluster_controller');
require('controllers/global/update_controller');
require('controllers/global/configuration_controller');
require('controllers/main/service/reassign/step7_controller');

});

require.register("controllers/application", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ApplicationController = Em.Controller.extend(App.Persist, {

  name: 'applicationController',

  isPollerRunning: false,

  clusterName: Em.computed.alias('App.router.clusterController.clusterName'),

  /**
   * set ambari server version from installerController or mainController, making sure version shown up all the time
   */
  ambariVersion: function () {
    return App.router.get('installerController.ambariServerVersion') || App.router.get('mainController.ambariServerVersion') || Em.I18n.t('common.notAvailable');
  }.property('App.router.installerController.ambariServerVersion', 'App.router.mainController.ambariServerVersion'),

  clusterDisplayName: Em.computed.truncate('clusterName', 13, 10),

  isClusterDataLoaded: Em.computed.and('App.router.clusterController.isLoaded', 'App.router.loggedIn'),

  isExistingClusterDataLoaded: Em.computed.and('App.router.clusterInstallCompleted', 'isClusterDataLoaded'),

  enableLinks: Em.computed.and('isExistingClusterDataLoaded', '!App.isOnlyViewUser'),

  /**
   * Determines if "Exit" menu-item should be shown
   * It should if cluster isn't installed
   * If cluster is installer, <code>isClusterDataLoaded</code> is checked
   * @type {boolean}
   */
  showExitLink: function () {
    if (App.router.get('clusterInstallCompleted')) {
      return this.get('isClusterDataLoaded');
    }
    return true;
  }.property('App.router.clusterInstallCompleted', 'isClusterDataLoaded'),

  /**
   * Determines if "Manage Ambari" menu-item should be shown
   *
   * @type {boolean}
   */
  showManageAmbari: function () {
    if (App.router.get('clusterInstallCompleted')) {
      return this.get('isClusterDataLoaded');
    }
    return App.get('isPermissionDataLoaded');
  }.property('App.router.clusterInstallCompleted', 'isClusterDataLoaded', 'App.isPermissionDataLoaded'),

  /**
   * Determines if upgrade label should be shown
   *
   * @type {boolean}
   */
  showUpgradeLabel: Em.computed.or('App.upgradeInProgress', 'App.upgradeHolding', 'App.upgradeSuspended'),

  /**
   * @return {{msg: string, cls: string, icon: string}}
   */
  upgradeMap: function () {
    var upgradeInProgress = App.get('upgradeInProgress');
    var upgradeHolding = App.get('upgradeHolding');
    var upgradeSuspended = App.get('upgradeSuspended');
    var isDowngrade = App.router.get('mainAdminStackAndUpgradeController.isDowngrade');
    var typeSuffix = isDowngrade ? 'downgrade' : 'upgrade';
    var hasUpgradePrivilege = App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK');
    var wizardWatcherController = App.router.get('wizardWatcherController');
    var isNotWizardUser;
    var wizardUserName;
    if (upgradeInProgress) {
      isNotWizardUser = wizardWatcherController.get('isNonWizardUser');
      wizardUserName = wizardWatcherController.get('wizardUser');
      return {
        cls: hasUpgradePrivilege ? 'upgrade-in-progress' : 'upgrade-in-progress not-allowed-cursor',
        icon: 'glyphicon-cog',
        msg: isNotWizardUser ? Em.I18n.t('admin.stackVersions.version.' + typeSuffix + '.running.nonWizard').format(wizardUserName) : Em.I18n.t('admin.stackVersions.version.' + typeSuffix + '.running')
      };
    }
    if (upgradeHolding) {
      return {
        cls: hasUpgradePrivilege ? 'upgrade-holding' : 'upgrade-holding not-allowed-cursor',
        icon: 'glyphicon-pause',
        msg: Em.I18n.t('admin.stackVersions.version.' + typeSuffix + '.pause')
      };
    }
    if (upgradeSuspended) {
      return {
        cls: hasUpgradePrivilege ? 'upgrade-aborted' : 'upgrade-aborted not-allowed-cursor',
        icon: 'glyphicon-pause',
        msg: Em.I18n.t('admin.stackVersions.version.' + typeSuffix + '.suspended')
      };
    }
    return {};
  }.property('App.upgradeInProgress', 'App.upgradeHolding', 'App.upgradeSuspended', 'App.router.mainAdminStackAndUpgradeController.isDowngrade'),

  startKeepAlivePoller: function startKeepAlivePoller() {
    if (!this.get('isPollerRunning')) {
      this.set('isPollerRunning', true);
      App.updater.run(this, 'getStack', 'isPollerRunning', App.sessionKeepAliveInterval);
    }
  },

  getStack: function getStack(callback) {
    App.ajax.send({
      name: 'router.login.clusters',
      sender: this,
      callback: callback
    });
  },

  goToAdminView: function goToAdminView() {
    App.router.route("adminView");
  },

  goToDashboard: function goToDashboard() {
    if (this.get('enableLinks')) {
      App.router.route("main/dashboard");
    }
  },

  showAboutPopup: function showAboutPopup() {
    App.ModalPopup.show({
      header: Em.I18n.t('common.aboutAmbari'),
      secondary: false,
      bodyClass: Em.View.extend({
        templateName: require('templates/common/about'),
        ambariVersion: this.get('ambariVersion')
      })
    });
  }

});

});

require.register("controllers/experimental", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.ExperimentalController = Em.Controller.extend(App.Persist, {
  name: 'experimentalController',
  supports: function () {
    return Em.keys(App.get('supports')).map(function (sup) {
      return Ember.Object.create({
        name: sup,
        selected: App.get('supports')[sup]
      });
    });
  }.property('App.supports'),

  loadSupports: function loadSupports() {
    return this.getUserPref('user-pref-' + App.router.get('loginName') + '-supports');
  },

  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response) {
    if (response) {
      App.set('supports', $.extend({}, App.get('supports'), response));
    }
  },

  doSave: function doSave() {
    var supports = this.get('supports');
    supports.forEach(function (s) {
      var propName = 'App.supports.' + s.get('name');
      var propValue = s.get('selected');
      Ember.set(propName, propValue);
    });
    this.postUserPref('user-pref-' + App.router.get('loginName') + '-supports', App.get('supports')).complete(function () {
      App.router.transitionTo('root.index');
    });
  },

  doCancel: function doCancel() {
    App.router.transitionTo('root.index');
  },

  doResetUIStates: function doResetUIStates() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('reset.ui.states'),
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile(Em.I18n.t('reset.ui.states.body'))
      }),
      primary: Em.I18n.t('yes'),
      context: self,
      onPrimary: function onPrimary() {
        var router = App.router;
        App.db.cleanUp();
        router.clearAllSteps();
        App.cache.clear();
        App.clusterStatus.setClusterStatus({});
        this.context.postUserPref('wizard-data', {});
        this.hide();
        router.transitionTo('root.index');
      }
    });
  }
});

});

require.register("controllers/global/background_operations_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.BackgroundOperationsController = Em.Controller.extend({
  name: 'backgroundOperationsController',

  runningOperationsCount: function () {
    return this.get('services').filterProperty('isRunning').length;
  }.property('services.@each.isRunning'),

  /**
   * List of requests
   */
  services: Em.A([]),
  serviceTimestamp: null,

  /**
   * Number of operation to load
   */
  operationsCount: 10,

  /**
   * Possible levels:
   * REQUESTS_LIST
   * HOSTS_LIST
   * TASKS_LIST
   * TASK_DETAILS
   */
  levelInfo: Em.Object.create({
    name: 'REQUESTS_LIST',
    requestId: null,
    taskId: null
  }),

  subscribeToUpdates: function subscribeToUpdates() {
    var _this = this;

    this.requestMostRecent(function () {
      App.StompClient.subscribe('/events/requests', _this.updateRequests.bind(_this));
    });
  },

  updateRequests: function updateRequests(event) {
    if (this.isUpgradeRequest({ Requests: { request_context: event.requestContext } })) {
      return;
    }
    var request = this.get('services').findProperty('id', event.requestId);
    var context = this.parseRequestContext(event.requestContext);
    var visibleOperationsCount = this.get('operationsCount');
    var map = this.generateTasksMapOfRequest(event, request);
    var updatedState = {
      progress: Math.floor(event.progressPercent),
      status: event.requestStatus,
      userName: event.userName,
      isRunning: this.isRunning(event.requestStatus),
      startTime: App.dateTimeWithTimeZone(event.startTime),
      endTime: event.endTime > 0 ? App.dateTimeWithTimeZone(event.endTime) : event.endTime,
      previousTaskStatusMap: map.currentTaskStatusMap,
      hostsMap: map.hostsMap
    };

    if (request) {
      request.setProperties(updatedState);
    } else {
      this.get('services').unshift(Em.Object.create(updatedState, {
        id: event.requestId,
        name: context.requestContext,
        displayName: context.requestContext,
        tasks: event.Tasks
      }));
      if (this.get('services').length >= visibleOperationsCount) {
        this.set('isShowMoreAvailable', true);
        this.get('services').pop();
      }
    }
    this.set('serviceTimestamp', App.dateTime());
    this.propertyDidChange('services');
  },

  /**
   *
   * @param {object} event
   * @param {Em.Object} request
   * @returns {{}}
   */
  generateTasksMapOfRequest: function generateTasksMapOfRequest(event, request) {
    var _this2 = this;

    var hostsMap = request ? request.get('hostsMap') : {};
    var previousTaskStatusMap = request ? request.get('previousTaskStatusMap') : {};
    var currentTaskStatusMap = {};
    event.Tasks.forEach(function (task) {
      var host = hostsMap[task.hostName];
      if (host) {
        var existedTask = host.logTasks.findProperty('Tasks.id', task.id);
        if (existedTask) {
          existedTask.Tasks.status = task.status;
        } else {
          host.logTasks.push(_this2.convertTaskFromEventToApi(task));
        }
        host.isModified = host.isModified ? true : previousTaskStatusMap[task.id] !== task.status;
      } else {
        hostsMap[task.hostName] = {
          name: task.hostName,
          publicName: task.hostName,
          logTasks: [_this2.convertTaskFromEventToApi(task)],
          isModified: previousTaskStatusMap[task.id] !== task.status
        };
      }
      currentTaskStatusMap[task.id] = task.status;
    }, this);
    return {
      currentTaskStatusMap: currentTaskStatusMap,
      hostsMap: hostsMap
    };
  },

  convertTaskFromEventToApi: function convertTaskFromEventToApi(task) {
    return {
      Tasks: {
        status: task.status,
        host_name: task.hostName,
        id: task.id,
        request_id: task.requestId
      }
    };
  },

  handleTaskUpdates: function () {
    var _this3 = this;

    var levelInfo = this.get('levelInfo');
    if (!levelInfo.get('requestId') || !levelInfo.get('taskId')) {
      return;
    }
    var request = this.get('services').findProperty('id', levelInfo.get('requestId'));
    var taskStatus = request.get('previousTaskStatusMap')[levelInfo.get('taskId')];
    if (levelInfo.get('name') === 'TASK_DETAILS' && !this.isFinished(taskStatus)) {
      App.StompClient.subscribe('/events/tasks/' + levelInfo.get('taskId'), function (updatedTask) {
        _this3.updateTask(updatedTask);
        if (_this3.isFinished(updatedTask.status)) {
          App.StompClient.unsubscribe('/events/tasks/' + updatedTask.id);
        }
      });
    }
  }.observes('levelInfo.name'),

  updateTask: function updateTask(updatedTask) {
    var request = this.get('services').findProperty('id', updatedTask.requestId);
    var host = request.get('hostsMap')[updatedTask.hostName];
    var task = host.logTasks.findProperty('Tasks.id', updatedTask.id);
    task.Tasks.status = updatedTask.status;
    task.Tasks.stdout = updatedTask.stdout;
    task.Tasks.stderr = updatedTask.stderr;
    task.Tasks.structured_out = updatedTask.structured_out;
    task.Tasks.output_log = updatedTask.outLog;
    task.Tasks.error_log = updatedTask.errorLog;
    this.set('serviceTimestamp', App.dateTime());
  },

  /**
   * Get requests data from server
   * @param callback
   */
  requestMostRecent: function requestMostRecent(callback) {
    var queryParams = this.getQueryParams();
    App.ajax.send({
      'name': queryParams.name,
      'sender': this,
      'success': queryParams.successCallback,
      'callback': callback,
      'data': queryParams.data
    });
    return !this.isInitLoading();
  },

  /**
   * indicate whether data for current level has already been loaded or not
   * @return {Boolean}
   */
  isInitLoading: function isInitLoading() {
    var levelInfo = this.get('levelInfo');
    var request = this.get('services').findProperty('id', levelInfo.get('requestId'));

    if (levelInfo.get('name') === 'HOSTS_LIST') {
      return Boolean(request && !request.get('hostsLevelLoaded'));
    }
    return false;
  },
  /**
   * construct params of ajax query regarding displayed level
   */
  getQueryParams: function getQueryParams() {
    var levelInfo = this.get('levelInfo');
    var count = this.get('operationsCount');
    var result = {
      name: 'background_operations.get_most_recent',
      successCallback: 'callBackForMostRecent',
      data: {
        'operationsCount': count
      }
    };
    if (levelInfo.get('name') === 'TASK_DETAILS') {
      result.name = 'background_operations.get_by_task';
      result.successCallback = 'callBackFilteredByTask';
      result.data = {
        'taskId': levelInfo.get('taskId'),
        'requestId': levelInfo.get('requestId')
      };
    } else if (levelInfo.get('name') === 'TASKS_LIST' || levelInfo.get('name') === 'HOSTS_LIST') {
      result.name = 'background_operations.get_by_request';
      result.successCallback = 'callBackFilteredByRequest';
      result.data = {
        'requestId': levelInfo.get('requestId')
      };
    }
    return result;
  },

  /**
   * Push hosts and their tasks to request
   * @param data
   */
  callBackFilteredByRequest: function callBackFilteredByRequest(data) {
    var requestId = data.Requests.id;
    var requestInputs = data.Requests.inputs;
    var request = this.get('services').findProperty('id', requestId);
    var hostsMap = {};
    var previousTaskStatusMap = request.get('previousTaskStatusMap');
    var currentTaskStatusMap = {};
    data.tasks.forEach(function (task) {
      var host = hostsMap[task.Tasks.host_name];
      task.Tasks.request_id = requestId;
      task.Tasks.request_inputs = requestInputs;
      if (host) {
        host.logTasks.push(task);
        host.isModified = true;
      } else {
        hostsMap[task.Tasks.host_name] = {
          name: task.Tasks.host_name,
          publicName: task.Tasks.host_name,
          logTasks: [task],
          isModified: true
        };
      }
      currentTaskStatusMap[task.Tasks.id] = task.Tasks.status;
    }, this);
    request.set('previousTaskStatusMap', currentTaskStatusMap);
    request.set('hostsMap', hostsMap);
    request.set('hostsLevelLoaded', true);
    this.set('serviceTimestamp', App.dateTime());
  },
  /**
   * Update task, with uploading two additional properties: stdout and stderr
   * @param data
   * @param ajaxQuery
   * @param params
   */
  callBackFilteredByTask: function callBackFilteredByTask(data, ajaxQuery, params) {
    var request = this.get('services').findProperty('id', data.Tasks.request_id);
    var host = request.get('hostsMap')[data.Tasks.host_name];
    var task = host.logTasks.findProperty('Tasks.id', data.Tasks.id);
    task.Tasks.status = data.Tasks.status;
    task.Tasks.stdout = data.Tasks.stdout;
    task.Tasks.stderr = data.Tasks.stderr;

    // Put some command information to task object
    task.Tasks.command = data.Tasks.command;
    task.Tasks.custom_command_name = data.Tasks.custom_command_name;
    task.Tasks.structured_out = data.Tasks.structured_out;

    task.Tasks.output_log = data.Tasks.output_log;
    task.Tasks.error_log = data.Tasks.error_log;
    this.set('serviceTimestamp', App.dateTime());
  },

  /**
   * returns true if it's upgrade equest
   * use this flag to exclude upgrade requests from bgo
   * @param {object} request
   * @returns {boolean}
   */
  isUpgradeRequest: function isUpgradeRequest(request) {
    var context = Em.get(request, 'Requests.request_context');
    return context ? /(upgrading|downgrading)/.test(context.toLowerCase()) : false;
  },
  /**
   * Prepare, received from server, requests for host component popup
   * @param data
   */
  callBackForMostRecent: function callBackForMostRecent(data) {
    var currentRequestIds = [];
    var countIssued = this.get('operationsCount');
    var countGot = data.itemTotal;

    data.items.forEach(function (request) {
      if (this.isUpgradeRequest(request)) {
        return;
      }
      var rq = this.get("services").findProperty('id', request.Requests.id);
      var isRunning = this.isRunning(request.Requests.request_status);
      var requestParams = this.parseRequestContext(request.Requests.request_context);
      var requestState = {
        progress: Math.floor(request.Requests.progress_percent),
        status: request.Requests.request_status,
        userName: request.Requests.user_name || Em.I18n.t('hostPopup.default.userName'),
        isRunning: isRunning,
        startTime: App.dateTimeWithTimeZone(request.Requests.start_time),
        endTime: request.Requests.end_time > 0 ? App.dateTimeWithTimeZone(request.Requests.end_time) : request.Requests.end_time
      };
      this.assignScheduleId(request, requestParams);
      currentRequestIds.push(request.Requests.id);

      if (rq) {
        rq.setProperties(requestState);
      } else {
        rq = Em.Object.create(requestState, {
          id: request.Requests.id,
          name: requestParams.requestContext,
          displayName: requestParams.requestContext,
          hostsMap: {},
          tasks: [],
          dependentService: requestParams.dependentService,
          sourceRequestScheduleId: request.Requests.request_schedule && request.Requests.request_schedule.schedule_id,
          previousTaskStatusMap: {},
          contextCommand: requestParams.contextCommand
        });
        this.get("services").unshift(rq);
        //To sort DESC by request id
        this.set("services", this.get("services").sortProperty('id').reverse());
      }
    }, this);
    this.removeOldRequests(currentRequestIds);
    this.set('isShowMoreAvailable', countGot >= countIssued);
    this.set('serviceTimestamp', App.dateTimeWithTimeZone());
  },

  isShowMoreAvailable: null,

  /**
   * remove old requests
   * as API returns 10, or  20 , or 30 ...etc latest request, the requests that absent in response should be removed
   * @param currentRequestIds
   */
  removeOldRequests: function removeOldRequests(currentRequestIds) {
    var services = this.get('services');

    for (var i = 0, l = services.length; i < l; i++) {
      if (!currentRequestIds.contains(services[i].id)) {
        services.splice(i, 1);
        i--;
        l--;
      }
    }
  },

  /**
   * identify whether request or task is running by status
   * @param status
   * @return {Boolean}
   */
  isRunning: function isRunning(status) {
    return ['IN_PROGRESS', 'QUEUED', 'PENDING'].contains(status);
  },

  /**
   * identify whether request or task is finished by status
   * @param status
   * @return {Boolean}
   */
  isFinished: function isFinished(status) {
    return ['FAILED', 'ABORTED', 'COMPLETED'].contains(status);
  },

  /**
   * identify whether there is only one host in request
   * @param inputs
   * @return {Boolean}
   */
  isOneHost: function isOneHost(inputs) {
    if (!inputs) {
      return false;
    }
    inputs = JSON.parse(inputs);
    if (inputs && inputs.included_hosts) {
      return inputs.included_hosts.split(',').length < 2;
    }
    return false;
  },
  /**
   * assign schedule_id of request to null if it's Recommission operation
   * @param request
   * @param requestParams
   */
  assignScheduleId: function assignScheduleId(request, requestParams) {
    var oneHost = this.isOneHost(request.Requests.inputs);
    if (request.Requests.request_schedule && oneHost && /Recommission/.test(requestParams.requestContext)) {
      request.Requests.request_schedule.schedule_id = null;
    }
  },

  /**
   * parse request context and if keyword "_PARSE_" is present then format it
   * @param {string} requestContext
   * @return {Object}
   */
  parseRequestContext: function parseRequestContext(requestContext) {
    var context = {};
    if (requestContext) {
      if (requestContext.indexOf(App.BackgroundOperationsController.CommandContexts.PREFIX) !== -1) {
        context = this.getRequestContextWithPrefix(requestContext);
      } else {
        context.requestContext = requestContext;
      }
    } else {
      context.requestContext = Em.I18n.t('requestInfo.unspecified');
    }
    return context;
  },

  /**
   *
   * @param {string} requestContext
   * @returns {{requestContext: *, dependentService: *, contextCommand: *}}
   */
  getRequestContextWithPrefix: function getRequestContextWithPrefix(requestContext) {
    var contextSplits = requestContext.split('.'),
        parsedRequestContext,
        contextCommand = contextSplits[1],
        service = contextSplits[2];

    switch (contextCommand) {
      case "STOP":
      case "START":
        if (service === 'ALL_SERVICES') {
          parsedRequestContext = Em.I18n.t("requestInfo." + contextCommand.toLowerCase()).format(Em.I18n.t('common.allServices'));
        } else {
          parsedRequestContext = Em.I18n.t("requestInfo." + contextCommand.toLowerCase()).format(App.format.role(service, true));
        }
        break;
      case "ROLLING-RESTART":
        parsedRequestContext = Em.I18n.t("rollingrestart.rest.context").format(App.format.role(service, true), contextSplits[3], contextSplits[4]);
        break;
    }
    return {
      requestContext: parsedRequestContext,
      dependentService: service,
      contextCommand: contextCommand
    };
  },

  popupView: null,

  /**
   * Onclick handler for background operations number located right to logo
   */
  showPopup: function showPopup() {
    // load the checkbox on footer first, then show popup.
    var self = this;
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      App.updater.immediateRun('requestMostRecent');

      App.HostPopup.set("breadcrumbs", [App.HostPopup.get("rootBreadcrumb")]);

      if (self.get('popupView') && App.HostPopup.get('isBackgroundOperations')) {
        self.set('popupView.isNotShowBgChecked', !initValue);
        self.set('popupView.isOpen', true);
        var el = $(self.get('popupView.element'));
        el.appendTo('#wrapper');
        el.find('.modal').show();
      } else {
        self.set('popupView', App.HostPopup.initPopup("", self, true));
        self.set('popupView.isNotShowBgChecked', !initValue);
      }
    });
  },

  /**
   * Called on logout
   */
  clear: function clear() {
    // set operations count to default value
    this.set('operationsCount', 10);
  }

});

/**
 * Each background operation has a context in which it operates.
 * Generally these contexts are fixed messages. However, we might
 * want to associate semantics to this context - like showing, disabling
 * buttons when certain operations are in progress.
 *
 * To make this possible we have command contexts where the context
 * is not a human readable string, but a pattern indicating the command
 * it is running. When UI shows these, they are translated into human
 * readable strings.
 *
 * General pattern of context names is "_PARSE_.{COMMAND}.{ID}[.{Additional-Data}...]"
 */
App.BackgroundOperationsController.CommandContexts = {
  PREFIX: "_PARSE_",
  /**
   * Stops all services
   */
  STOP_ALL_SERVICES: "_PARSE_.STOP.ALL_SERVICES",
  /**
   * Starts all services
   */
  START_ALL_SERVICES: "_PARSE_.START.ALL_SERVICES",
  /**
   * Starts service indicated by serviceID.
   * @param {String} serviceID Parameter {0}. Example: HDFS
   */
  START_SERVICE: "_PARSE_.START.{0}",
  /**
   * Stops service indicated by serviceID.
   * @param {String} serviceID Parameter {0}. Example: HDFS
   */
  STOP_SERVICE: "_PARSE_.STOP.{0}",
  /**
   * Performs rolling restart of componentID in batches.
   * This context is the batchNumber batch out of totalBatchCount batches.
   * @param {String} componentID Parameter {0}. Example "DATANODE"
   * @param {Number} batchNumber Parameter {1}. Batch number of this batch. Example 3.
   * @param {Number} totalBatchCount Parameter {2}. Total number of batches. Example 10.
   */
  ROLLING_RESTART: "_PARSE_.ROLLING-RESTART.{0}.{1}.{2}"
};

});

require.register("controllers/global/cluster_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');
var credentialUtils = require('utils/credentials');

App.ClusterController = Em.Controller.extend(App.ReloadPopupMixin, {
  name: 'clusterController',
  isLoaded: false,
  ambariProperties: null,
  clusterEnv: null,
  clusterDataLoadedPercent: 'width:0', // 0 to 1

  isClusterNameLoaded: false,

  isAlertsLoaded: false,

  isComponentsStateLoaded: false,

  isHostsLoaded: false,

  isConfigsPropertiesLoaded: false,

  isComponentsConfigLoaded: false,

  isStackConfigsLoaded: false,

  isServiceMetricsLoaded: false,

  /**
   * @type {boolean}
   */
  isHostComponentMetricsLoaded: false,

  isHDFSNameSpacesLoaded: false,

  /**
   * This counter used as event trigger to notify that quick links should be changed.
   */
  quickLinksUpdateCounter: 0,

  /**
   * Ambari uses custom jdk.
   * @type {Boolean}
   */
  isCustomJDK: false,

  isHostContentLoaded: Em.computed.and('isHostsLoaded', 'isComponentsStateLoaded'),

  isServiceContentFullyLoaded: Em.computed.and('isServiceMetricsLoaded', 'isComponentsStateLoaded', 'isComponentsConfigLoaded'),

  isStackVersionsLoaded: false,

  clusterName: Em.computed.alias('App.clusterName'),

  updateLoadStatus: function updateLoadStatus(item) {
    var loadList = this.get('dataLoadList');
    var loaded = true;
    var numLoaded = 0;
    var loadListLength = 0;
    loadList.set(item, true);
    for (var i in loadList) {
      if (loadList.hasOwnProperty(i)) {
        loadListLength++;
        if (!loadList[i] && loaded) {
          loaded = false;
        }
      }
      // calculate the number of true
      if (loadList.hasOwnProperty(i) && loadList[i]) {
        numLoaded++;
      }
    }
    this.set('isLoaded', loaded);
    this.set('clusterDataLoadedPercent', 'width:' + Math.floor(numLoaded / loadListLength * 100).toString() + '%');
  },

  dataLoadList: Em.Object.create({
    'stackComponents': false,
    'services': false
  }),

  /**
   * load cluster name
   */
  loadClusterName: function loadClusterName(reload, deferred) {
    var dfd = deferred || $.Deferred();

    if (App.get('clusterName') && !reload) {
      App.set('clusterName', this.get('clusterName'));
      this.set('isClusterNameLoaded', true);
      dfd.resolve();
    } else {
      App.ajax.send({
        name: 'cluster.load_cluster_name',
        sender: this,
        data: {
          reloadPopupText: Em.I18n.t('app.reloadPopup.noClusterName.text'),
          errorLogMessage: 'failed on loading cluster name',
          callback: this.loadClusterName,
          args: [reload, dfd],
          shouldUseDefaultHandler: true
        },
        success: 'reloadSuccessCallback',
        error: 'reloadErrorCallback',
        callback: function callback() {
          if (!App.get('currentStackVersion')) {
            App.set('currentStackVersion', App.defaultStackVersion);
          }
        }
      }).then(function () {
        dfd.resolve();
      }, null);
    }
    return dfd.promise();
  },

  reloadSuccessCallback: function reloadSuccessCallback(data) {
    this._super();
    if (data.items && data.items.length > 0) {
      App.setProperties({
        clusterId: data.items[0].Clusters.cluster_id,
        clusterName: data.items[0].Clusters.cluster_name,
        currentStackVersion: data.items[0].Clusters.version,
        isKerberosEnabled: data.items[0].Clusters.security_type === 'KERBEROS'
      });
      this.set('isClusterNameLoaded', true);
    }
  },

  setServerClock: function setServerClock(data) {
    var clientClock = new Date().getTime();
    var serverClock = Em.getWithDefault(data, 'RootServiceComponents.server_clock', '').toString();
    serverClock = serverClock.length < 13 ? serverClock + '000' : serverClock;
    App.set('clockDistance', serverClock - clientClock);
    App.set('currentServerTime', parseInt(serverClock));
  },

  getServerClockErrorCallback: Em.K,

  getUrl: function getUrl(testUrl, url) {
    return App.get('testMode') ? testUrl : App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + url;
  },

  /**
   *  load all data and update load status
   */
  loadClusterData: function loadClusterData() {
    this.loadAuthorizations();
    this.getAllHostNames();

    if (!App.get('clusterName')) {
      return;
    }

    if (this.get('isLoaded')) {
      // do not load data repeatedly
      App.router.get('mainController').startPolling();
      return;
    }
    App.router.get('userSettingsController').getAllUserSettings();
    App.router.get('errorsHandlerController').loadErrorLogs();

    this.loadClusterInfo();
    this.restoreUpgradeState();
    App.router.get('wizardWatcherController').getUser();

    this.loadClusterDataToModel();

    //force clear filters  for hosts page to load all data
    App.db.setFilterConditions('mainHostController', null);
  },

  loadClusterInfo: function loadClusterInfo() {
    var clusterUrl = this.getUrl('/data/clusters/cluster.json', '?fields=Clusters');
    App.HttpClient.get(clusterUrl, App.clusterMapper, {
      complete: function complete(jqXHR, textStatus) {
        App.set('isCredentialStorePersistent', Em.getWithDefault(App.Cluster.find().findProperty('clusterName', App.get('clusterName')), 'isCredentialStorePersistent', false));
      }
    }, Em.K);
  },

  /**
   * Order of loading:
   * 1. load all created service components
   * 2. request for service components supported by stack
   * 3. load stack components to model
   * 4. request for services
   * 5. put services in cache
   * 6. request for hosts and host-components (single call)
   * 7. request for service metrics
   * 8. load host-components to model
   * 9. load services from cache with metrics to model
   */
  loadClusterDataToModel: function loadClusterDataToModel() {
    var self = this;

    this.loadStackServiceComponents(function (data) {
      data.items.forEach(function (service) {
        service.StackServices.is_selected = true;
        service.StackServices.is_installed = false;
      }, self);
      App.stackServiceMapper.mapStackServices(data);
      App.config.setPreDefinedServiceConfigs(true);
      self.updateLoadStatus('stackComponents');
      self.loadServicesAndComponents();
    });
  },

  loadServicesAndComponents: function loadServicesAndComponents() {
    var updater = App.router.get('updateController');
    var self = this;

    updater.updateServices(function () {
      self.updateLoadStatus('services');

      //hosts should be loaded after services in order to properly populate host-component relation in App.cache.services
      updater.updateHost(function () {
        self.set('isHostsLoaded', true);
        self.loadAlerts();
      });
      self.loadConfigProperties();
      // components state loading doesn't affect overall progress
      updater.updateComponentsState(function () {
        self.set('isComponentsStateLoaded', true);
        // service metrics should be loaded after components state for mapping service components to service in the DS model
        // service metrics loading doesn't affect overall progress
        updater.updateServiceMetric(function () {
          self.set('isServiceMetricsLoaded', true);
          // make second call, because first is light since it doesn't request host-component metrics
          updater.updateServiceMetric(function () {
            self.set('isHostComponentMetricsLoaded', true);
            updater.updateHDFSNameSpaces();
          });
          // components config loading doesn't affect overall progress
          self.loadComponentWithStaleConfigs(function () {
            self.set('isComponentsConfigLoaded', true);
          });
        });
      });
    });
  },

  loadComponentWithStaleConfigs: function loadComponentWithStaleConfigs(callback) {
    return App.ajax.send({
      name: 'components.get.staleConfigs',
      sender: this,
      success: 'loadComponentWithStaleConfigsSuccessCallback',
      callback: callback
    });
  },

  loadComponentWithStaleConfigsSuccessCallback: function loadComponentWithStaleConfigsSuccessCallback(json) {
    json.items.forEach(function (item) {
      var componentName = item.ServiceComponentInfo.component_name;
      var hosts = item.host_components.mapProperty('HostRoles.host_name') || [];
      App.componentsStateMapper.updateStaleConfigsHosts(componentName, hosts);
    });
  },

  loadConfigProperties: function loadConfigProperties() {
    var self = this;

    App.config.loadConfigsFromStack(App.Service.find().mapProperty('serviceName')).always(function () {
      App.config.loadClusterConfigsFromStack().always(function () {
        App.router.get('configurationController').updateConfigTags().always(function () {
          App.router.get('updateController').updateClusterEnv().always(function () {
            self.set('isConfigsPropertiesLoaded', true);
          });
        });
      });
    });
  },

  loadAlerts: function loadAlerts() {
    var updater = App.router.get('updateController');
    var self = this;

    console.time('Overall alerts loading time');
    updater.updateAlertGroups(function () {
      updater.updateAlertDefinitions(function () {
        updater.updateAlertDefinitionSummary(function () {
          updater.updateUnhealthyAlertInstances(function () {
            console.timeEnd('Overall alerts loading time');
            self.set('isAlertsLoaded', true);
          });
        });
      });
    });
  },

  /**
   * restore upgrade status from server
   * and make call to get latest status from server
   * Also loading all upgrades to App.StackUpgradeHistory model
   */
  restoreUpgradeState: function restoreUpgradeState() {
    var self = this;
    return this.getAllUpgrades().done(function (data) {
      var upgradeController = App.router.get('mainAdminStackAndUpgradeController');
      var allUpgrades = data.items.sortProperty('Upgrade.request_id');
      var lastUpgradeData = allUpgrades.pop();
      if (lastUpgradeData) {
        var status = lastUpgradeData.Upgrade.request_status;
        var lastUpgradeNotFinished = self.isSuspendedState(status) || self.isRunningState(status);
        if (lastUpgradeNotFinished) {
          /**
           * No need to display history if there is only one running or suspended upgrade.
           * Because UI still needs to provide user the option to resume the upgrade via the Upgrade Wizard UI.
           * If there is more than one upgrade. Show/Hive the tab based on the status.
           */
          var hasFinishedUpgrades = allUpgrades.some(function (item) {
            var status = item.Upgrade.request_status;
            if (!self.isRunningState(status)) {
              return true;
            }
          }, self);
          App.set('upgradeHistoryAvailable', hasFinishedUpgrades);
        } else {
          //There is at least one finished upgrade. Display it.
          App.set('upgradeHistoryAvailable', true);
        }
      } else {
        //There is no upgrades at all.
        App.set('upgradeHistoryAvailable', false);
      }

      //completed upgrade shouldn't be restored
      if (lastUpgradeData) {
        if (lastUpgradeData.Upgrade.request_status !== "COMPLETED") {
          upgradeController.restoreLastUpgrade(lastUpgradeData);
        }
      } else {
        upgradeController.initDBProperties();
      }

      App.stackUpgradeHistoryMapper.map(data);
      upgradeController.loadStackVersionsToModel(true).done(function () {
        upgradeController.loadCompatibleVersions();
        upgradeController.updateCurrentStackVersion();
        App.set('stackVersionsAvailable', App.StackVersion.find().content.length > 0);
        self.set('isStackVersionsLoaded', true);
      });
    });
  },

  isRunningState: function isRunningState(status) {
    if (status) {
      return "IN_PROGRESS" === status || "PENDING" === status || status.contains("HOLDING");
    } else {
      //init state
      return true;
    }
  },

  /**
   * ABORTED should be handled as SUSPENDED for the lastUpgradeItem
   * */
  isSuspendedState: function isSuspendedState(status) {
    return "ABORTED" === status;
  },

  loadRootService: function loadRootService() {
    return App.ajax.send({
      name: 'service.ambari',
      sender: this
    });
  },

  requestHosts: function requestHosts(realUrl, callback) {
    var testHostUrl = '/data/hosts/HDP2/hosts.json';
    var url = this.getUrl(testHostUrl, realUrl);
    App.HttpClient.get(url, App.hostsMapper, {
      complete: callback
    }, callback);
  },

  /**
   *
   * @param callback
   * @returns {?object}
   */
  loadStackServiceComponents: function loadStackServiceComponents(callback) {
    var callbackObj = {
      loadStackServiceComponentsSuccess: callback
    };
    return App.ajax.send({
      name: 'wizard.service_components',
      data: {
        stackUrl: App.get('stackVersionURL'),
        stackVersion: App.get('currentStackVersionNumber')
      },
      sender: callbackObj,
      success: 'loadStackServiceComponentsSuccess'
    });
  },

  loadAmbariProperties: function loadAmbariProperties() {
    return App.ajax.send({
      name: 'ambari.service',
      sender: this,
      success: 'loadAmbariPropertiesSuccess',
      error: 'loadAmbariPropertiesError'
    });
  },

  loadAuthorizations: function loadAuthorizations() {
    return App.ajax.send({
      name: 'router.user.authorizations',
      sender: this,
      data: { userName: App.db.getLoginName() },
      success: 'loadAuthorizationsSuccessCallback'
    });
  },

  loadAuthorizationsSuccessCallback: function loadAuthorizationsSuccessCallback(response) {
    if (response && response.items) {
      App.set('auth', response.items.mapProperty('AuthorizationInfo.authorization_id').uniq());
      App.db.setAuth(App.get('auth'));
    }
  },

  loadAmbariPropertiesSuccess: function loadAmbariPropertiesSuccess(data) {
    var mainController = App.router.get('mainController');
    this.set('ambariProperties', data.RootServiceComponents.properties);
    // Absence of 'jdk.name' and 'jce.name' properties says that ambari configured with custom jdk.
    this.set('isCustomJDK', App.isEmptyObject(App.permit(data.RootServiceComponents.properties, ['jdk.name', 'jce.name'])));
    this.setServerClock(data);
    mainController.setAmbariServerVersion.call(mainController, data);
    mainController.monitorInactivity();
  },

  loadAmbariPropertiesError: Em.K,

  updateClusterData: function updateClusterData() {
    var testUrl = '/data/clusters/HDP2/cluster.json';
    var clusterUrl = this.getUrl(testUrl, '?fields=Clusters');
    App.HttpClient.get(clusterUrl, App.clusterMapper, {
      complete: function complete() {}
    });
  },

  /**
   *
   * @returns {*|Transport|$.ajax|boolean|ServerResponse}
   */
  getAllHostNames: function getAllHostNames() {
    return App.ajax.send({
      name: 'hosts.all',
      sender: this,
      success: 'getHostNamesSuccess',
      error: 'getHostNamesError'
    });
  },

  getHostNamesSuccess: function getHostNamesSuccess(data) {
    App.set("allHostNames", data.items.mapProperty("Hosts.host_name"));
  },

  getHostNamesError: Em.K,

  /**
   * puts kerberos admin credentials in the live cluster session
   * and resend ajax request
   * @param {credentialResourceObject} credentialResource
   * @param {object} ajaxOpt
   * @returns {$.ajax}
   */
  createKerberosAdminSession: function createKerberosAdminSession(credentialResource, ajaxOpt) {
    return credentialUtils.createOrUpdateCredentials(App.get('clusterName'), credentialUtils.ALIAS.KDC_CREDENTIALS, credentialResource).then(function () {
      if (ajaxOpt) {
        $.ajax(ajaxOpt);
      }
    });
  },

  //TODO Replace this check with any other which is applicable to non-HDP stack
  /**
   * Check if HDP stack version is more or equal than 2.2.2 to determine if pluggable metrics for Storm are supported
   * @method checkDetailedRepoVersion
   * @returns {promise|*|promise|promise|HTMLElement|promise}
   */
  checkDetailedRepoVersion: function checkDetailedRepoVersion() {
    var dfd;
    var currentStackName = App.get('currentStackName');
    var currentStackVersionNumber = App.get('currentStackVersionNumber');
    if (currentStackName == 'HDP' && currentStackVersionNumber == '2.2') {
      dfd = App.ajax.send({
        name: 'cluster.load_detailed_repo_version',
        sender: this,
        success: 'checkDetailedRepoVersionSuccessCallback',
        error: 'checkDetailedRepoVersionErrorCallback'
      });
    } else {
      dfd = $.Deferred();
      App.set('isStormMetricsSupported', currentStackName != 'HDP' || stringUtils.compareVersions(currentStackVersionNumber, '2.2') == 1);
      dfd.resolve();
    }
    return dfd.promise();
  },

  checkDetailedRepoVersionSuccessCallback: function checkDetailedRepoVersionSuccessCallback(data) {
    var rv = (Em.getWithDefault(data, 'items', []) || []).filter(function (i) {
      return Em.getWithDefault(i || {}, 'ClusterStackVersions.stack', null) === App.get('currentStackName') && Em.getWithDefault(i || {}, 'ClusterStackVersions.version', null) === App.get('currentStackVersionNumber');
    })[0];
    var version = Em.getWithDefault(rv || {}, 'repository_versions.0.RepositoryVersions.repository_version', false);
    App.set('isStormMetricsSupported', stringUtils.compareVersions(version, '2.2.2') > -1 || !version);
  },
  checkDetailedRepoVersionErrorCallback: function checkDetailedRepoVersionErrorCallback() {
    App.set('isStormMetricsSupported', true);
  },

  /**
   * Load required data for all upgrades from API
   * @returns {$.ajax}
   */
  getAllUpgrades: function getAllUpgrades() {
    return App.ajax.send({
      name: 'cluster.load_last_upgrade',
      sender: this
    });
  },

  triggerQuickLinksUpdate: function triggerQuickLinksUpdate() {
    this.incrementProperty('quickLinksUpdateCounter');
  }
});

});

require.register("controllers/global/configuration_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigurationController = Em.Controller.extend({
  name: 'configurationController',

  /**
   * get configs by tags
   * return Deferred object with configs as argument
   * @param tags {Object}
   * ** siteName
   * ** tagName (optional)
   * @return {object}
   */
  getConfigsByTags: function getConfigsByTags(tags) {
    var storedTags = [];
    App.db.getConfigs().forEach(function (site) {
      storedTags.push({
        siteName: site.type,
        tagName: site.tag
      });
    });
    if (this.checkTagsChanges(tags, storedTags)) {
      return this.loadFromServer(tags);
    } else {
      return this.loadFromDB(tags.mapProperty('siteName'));
    }
  },

  /**
   * if no sites specified then configs from all sites will be fetched
   * @param {Array} sites
   * @returns {$.Deferred}
   */
  getCurrentConfigsBySites: function getCurrentConfigsBySites() {
    var _this = this;

    var sites = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    var dfd = $.Deferred();
    this.getConfigTags(sites).done(function (tags) {
      _this.getConfigsByTags(tags).done(dfd.resolve);
    });
    return dfd.promise();
  },

  /**
   * check whether tag versions have been changed
   * if they are different then return true
   * otherwise false
   * @param tags
   * @param storedTags
   * @return {Boolean}
   */
  checkTagsChanges: function checkTagsChanges(tags, storedTags) {
    var isDifferent = false;
    var i = 0;
    while (i < tags.length && !isDifferent) {
      var storedTag = storedTags.findProperty('siteName', tags[i].siteName);
      isDifferent = !storedTag || !tags[i].tagName || storedTag.tagName !== tags[i].tagName;
      i++;
    }
    return isDifferent;
  },

  /**
   *
   * @param {Array} siteNames
   * @returns {*}
   */
  loadFromDB: function loadFromDB(siteNames) {
    var dfd = $.Deferred();
    var configs = App.db.getConfigs().filter(function (site) {
      return siteNames.contains(site.type);
    });
    dfd.resolve(configs);
    return dfd.promise();
  },
  /**
   * load configs from server
   * and update them in local DB
   * @param tags
   * @return {Array}
   */
  loadFromServer: function loadFromServer(tags) {
    var self = this;
    var dfd = $.Deferred();
    if (!tags.everyProperty('tagName')) {
      var configTags;
      var jqXhr = this.loadConfigTags();
      jqXhr.done(function (data) {
        configTags = data.Clusters.desired_configs;
        tags.forEach(function (_tag) {
          if (_tag.siteName && configTags[_tag.siteName] && !_tag.tagName) {
            _tag.tagName = configTags[_tag.siteName].tag;
          }
        }, self);
        self.loadConfigsByTags(tags, dfd);
      });
    } else {
      self.loadConfigsByTags(tags, dfd);
    }
    return dfd.promise();
  },

  /**
   *  loadConfigsByTags: Loads properties for a config tag
   *  @params tags
   *  @params dfd jqXhr promise
   */
  loadConfigsByTags: function loadConfigsByTags(tags, dfd) {
    var self = this;
    var loadedConfigs = [];
    App.config.loadConfigsByTags(tags).done(function (data) {
      if (data.items) {
        data.items.forEach(function (item) {
          loadedConfigs.push(item);
        });
      }
    }).complete(function () {
      self.saveToDB(loadedConfigs);
      dfd.resolve(loadedConfigs);
    });
  },

  /**
   * loadConfigTags: Loads all config tags applied to the cluster
   * @return: jqXhr promise
   */
  loadConfigTags: function loadConfigTags() {
    return App.ajax.send({
      name: 'config.tags',
      sender: this
    });
  },

  /**
   * save properties obtained from server to local DB
   * @param loadedConfigs
   */
  saveToDB: function saveToDB(loadedConfigs) {
    var storedConfigs = App.db.getConfigs();
    loadedConfigs.forEach(function (loadedSite) {
      var storedSite = storedConfigs.findProperty('type', loadedSite.type);
      if (storedSite) {
        storedSite.tag = loadedSite.tag;
        storedSite.properties = loadedSite.properties;
        storedSite.properties_attributes = loadedSite.properties_attributes;
      } else {
        storedConfigs.push(loadedSite);
      }
    });
    App.db.setConfigs(storedConfigs);
  },

  /**
   * @param {Array} sites
   * @return {Array}
   */
  getConfigTags: function getConfigTags() {
    var _this2 = this;

    var sites = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    var dfd = $.Deferred();
    if (App.db.getTags().length > 0) {
      dfd.resolve(this.extractTagsFromLocalDB(sites));
    } else {
      this.updateConfigTags().always(function () {
        dfd.resolve(_this2.extractTagsFromLocalDB(sites));
      });
    }
    return dfd.promise();
  },

  /**
   *
   * @param {Array} sites
   * @return {Array}
   */
  extractTagsFromLocalDB: function extractTagsFromLocalDB(sites) {
    return App.db.getTags().filter(function (tag) {
      if (sites.length > 0) {
        return sites.contains(tag.siteName);
      } else {
        return true;
      }
    });
  },

  /**
   * update configs' tags from server
   */
  updateConfigTags: function updateConfigTags() {
    return this.loadConfigTags().done(function (data) {
      var tags = [];
      for (var site in data.Clusters.desired_configs) {
        tags.push({
          siteName: site,
          tagName: data.Clusters.desired_configs[site].tag
        });
      }
      App.db.setTags(tags);
    });
  }
});

});

require.register("controllers/global/errors_handler_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ErrorsHandlerController = Em.Controller.extend(App.Persist, {

  name: 'errorsHandlerController',

  /**
  * @const
  */
  ERROR_STORAGE_SIZE: 500000,

  /**
  * @const
  */
  MAX_TRACE_LENGTH: 1000,

  init: function init() {
    var oldError = window.onerror || Em.K;
    var self = this;
    window.onerror = function (err, url, lineNumber, colNumber, Err) {
      oldError.call(this, err, url, lineNumber, colNumber, Err);
      self.saveErrorLogs(err, url, lineNumber, colNumber, Err);
    };
    return this._super();
  },

  /**
   * load logs from server
   */
  loadErrorLogs: function loadErrorLogs() {
    this.getUserPref('errors');
  },

  /**
  * @method getUserPrefSuccessCallback
  * @param {object|null} data
  */
  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(data) {
    if (data) {
      localStorage.setObject('errors', data);
    }
  },

  /**
   * save error logs to localStorage and server
   * @param {string} err
   * @param {string} url
   * @param {number} lineNumber
   * @param {number} colNumber
   * @param {Error} Err
   */
  saveErrorLogs: function saveErrorLogs(err, url, lineNumber, colNumber, Err) {
    var ls = localStorage.getObject('errors') || {};
    var key = new Date().getTime();
    var stackTrace = Em.get(Err || {}, 'stack');

    if (stackTrace) {
      var origin = location.origin || location.protocol + '//' + location.host,
          path = origin + location.pathname + 'javascripts',
          pattern = new RegExp(path, 'g');
      stackTrace = stackTrace.replace(pattern, '').substr(0, this.MAX_TRACE_LENGTH);
    }

    var val = {
      file: url,
      line: lineNumber,
      col: colNumber,
      error: err,
      stackTrace: stackTrace
    };

    //overwrite errors if storage full
    if (JSON.stringify(ls).length > this.ERROR_STORAGE_SIZE) {
      delete ls[Object.keys(ls).sort()[0]];
    }

    ls[key] = val;
    localStorage.setObject('errors', ls);
    this.postUserPref('errors', ls);
  }
});

});

require.register("controllers/global/update_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.UpdateController = Em.Controller.extend({
  name: 'updateController',
  isUpdated: false,
  cluster: null,
  isWorking: false,
  updateAlertInstances: Em.computed.and('isWorking', '!App.router.mainAlertInstancesController.isUpdating'),
  timeIntervalId: null,
  clusterName: Em.computed.alias('App.router.clusterController.clusterName'),

  paginationKeys: ['page_size', 'from'],

  /**
   * @type {Array}
   */
  serviceComponentMetrics: ['host_components/metrics/jvm/memHeapUsedM', 'host_components/metrics/jvm/HeapMemoryMax', 'host_components/metrics/jvm/HeapMemoryUsed', 'host_components/metrics/jvm/memHeapCommittedM', 'host_components/metrics/mapred/jobtracker/trackers_decommissioned', 'host_components/metrics/cpu/cpu_wio', 'host_components/metrics/rpc/client/RpcQueueTime_avg_time', 'host_components/metrics/dfs/FSNamesystem/*', 'host_components/metrics/dfs/namenode/Version', 'host_components/metrics/dfs/namenode/LiveNodes', 'host_components/metrics/dfs/namenode/DeadNodes', 'host_components/metrics/dfs/namenode/DecomNodes', 'host_components/metrics/dfs/namenode/TotalFiles', 'host_components/metrics/dfs/namenode/UpgradeFinalized', 'host_components/metrics/dfs/namenode/Safemode', 'host_components/metrics/runtime/StartTime'],

  /**
   * @type {object}
   */
  serviceSpecificParams: {
    'FLUME': "host_components/processes/HostComponentProcess",
    'YARN': "host_components/metrics/yarn/Queue," + "host_components/metrics/yarn/ClusterMetrics/NumActiveNMs," + "host_components/metrics/yarn/ClusterMetrics/NumLostNMs," + "host_components/metrics/yarn/ClusterMetrics/NumUnhealthyNMs," + "host_components/metrics/yarn/ClusterMetrics/NumRebootedNMs," + "host_components/metrics/yarn/ClusterMetrics/NumDecommissionedNMs",
    'HBASE': "host_components/metrics/hbase/master/IsActiveMaster," + "host_components/metrics/hbase/master/MasterStartTime," + "host_components/metrics/hbase/master/MasterActiveTime," + "host_components/metrics/hbase/master/AverageLoad," + "host_components/metrics/master/AssignmentManager/ritCount",
    'STORM': 'metrics/api/v1/cluster/summary,metrics/api/v1/topology/summary,metrics/api/v1/nimbus/summary',
    'HDFS': 'host_components/metrics/dfs/namenode/ClusterId'
  },

  nameNodeMetricsModelProperties: ['jvm_memory_heap_max_values', 'jvm_memory_heap_used_values', 'capacity_used', 'capacity_total', 'capacity_remaining', 'capacity_non_dfs_used', 'name_node_rpc_values', 'name_node_start_time_values'],

  /**
   * @type {string}
   */
  HOSTS_TEST_URL: '/data/hosts/HDP2/hosts.json',

  /**
   * map which track status of requests, whether it's running or completed
   * @type {object}
   */
  requestsRunningStatus: {
    "updateServiceMetric": false
  },

  getUrl: function getUrl(testUrl, url) {
    return App.get('testMode') ? testUrl : App.apiPrefix + '/clusters/' + this.get('clusterName') + url;
  },

  /**
   * construct URL from real URL and query parameters
   * @param realUrl
   * @param queryParams
   * @return {String}
   */
  getComplexUrl: function getComplexUrl(realUrl, queryParams) {
    var prefix = App.get('apiPrefix') + '/clusters/' + App.get('clusterName'),
        params = '';

    if (queryParams) {
      params = this.computeParameters(queryParams);
    }
    params = params.length > 0 ? params + "&" : params;
    return prefix + realUrl.replace('<parameters>', params);
  },

  /**
   * compute parameters according to their type
   * @param queryParams
   * @return {String}
   */
  computeParameters: function computeParameters(queryParams) {
    var params = '';

    queryParams.forEach(function (param) {
      var customKey = param.key;

      switch (param.type) {
        case 'EQUAL':
          if (Em.isArray(param.value)) {
            params += param.key + '.in(' + param.value.join(',') + ')';
          } else {
            params += param.key + '=' + param.value;
          }
          break;
        case 'LESS':
          params += param.key + '<' + param.value;
          break;
        case 'MORE':
          params += param.key + '>' + param.value;
          break;
        case 'MATCH':
          if (Em.isArray(param.value)) {
            params += '(' + param.value.map(function (v) {
              return param.key + '.matches(' + v + ')';
            }).join('|') + ')';
          } else {
            params += param.key + '.matches(' + param.value + ')';
          }
          break;
        case 'MULTIPLE':
          params += param.key + '.in(' + param.value.join(',') + ')';
          break;
        case 'SORT':
          params += 'sortBy=' + param.key + '.' + param.value;
          break;
        case 'CUSTOM':
          param.value.forEach(function (item, index) {
            customKey = customKey.replace('{' + index + '}', item);
          }, this);
          params += customKey;
          break;
        case 'COMBO':
          params += App.router.get('mainHostComboSearchBoxController').generateQueryParam(param);
          break;
      }
      params += '&';
    });
    return params.substring(0, params.length - 1);
  },

  /**
   * depict query parameters of table
   */
  queryParams: Em.Object.create({
    'Hosts': []
  }),

  /**
   * Pagination query-parameters for unhealthy alerts request
   * @type {{from: Number, page_size: Number}}
   */
  queryParamsForUnhealthyAlertInstances: {
    from: 0,
    page_size: 10
  },

  /**
   * map describes relations between updater function and table
   */
  tableUpdaterMap: {
    'Hosts': 'updateHost'
  },

  /**
   * Start polling, when <code>isWorking</code> become true
   */
  updateAll: function () {
    if (this.get('isWorking') && !App.get('isOnlyViewUser')) {
      App.updater.run(this, 'updateHostsMetrics', 'isWorking', App.contentUpdateInterval, '\/main\/(hosts).*');
      App.updater.run(this, 'updateServiceMetric', 'isWorking', App.componentsUpdateInterval, '\/main\/(dashboard|services).*');
      App.updater.run(this, 'graphsUpdate', 'isWorking');

      if (!App.get('router.mainAlertInstancesController.isUpdating')) {
        App.updater.run(this, 'updateUnhealthyAlertInstances', 'updateAlertInstances', App.alertInstancesUpdateInterval, '\/main\/alerts.*');
      }
      App.updater.run(this, 'updateWizardWatcher', 'isWorking', App.bgOperationsUpdateInterval);
      App.updater.run(this, 'updateHDFSNameSpaces', 'isWorking', App.componentsUpdateInterval, '\/main\/(dashboard|services\/HDFS|hosts).*');
    }
  }.observes('isWorking', 'App.router.mainAlertInstancesController.isUpdating'),

  startSubscriptions: function startSubscriptions() {
    App.StompClient.subscribe('/events/hostcomponents', App.hostComponentStatusMapper.map.bind(App.hostComponentStatusMapper));
    App.StompClient.subscribe('/events/alerts', App.alertSummaryMapper.map.bind(App.alertSummaryMapper));
    App.StompClient.subscribe('/events/ui_topologies', App.topologyMapper.map.bind(App.topologyMapper));
    App.StompClient.subscribe('/events/configs', this.configsChangedHandler.bind(this));
    App.StompClient.subscribe('/events/services', App.serviceStateMapper.map.bind(App.serviceStateMapper));
    App.StompClient.subscribe('/events/hosts', App.hostStateMapper.map.bind(App.hostStateMapper));
    App.StompClient.subscribe('/events/alert_definitions', App.alertDefinitionsMapperAdapter.map.bind(App.alertDefinitionsMapperAdapter));
    App.StompClient.subscribe('/events/alert_group', App.alertGroupsMapperAdapter.map.bind(App.alertGroupsMapperAdapter));
    App.StompClient.subscribe('/events/upgrade', App.upgradeStateMapper.map.bind(App.upgradeStateMapper));
    App.router.get('backgroundOperationsController').subscribeToUpdates();
  },

  /**
   *
   * @param {boolean} loadMetricsSeparately
   * @returns {string|*}
   */
  getUpdateHostUrlWithParams: function getUpdateHostUrlWithParams(loadMetricsSeparately) {
    var url = '/hosts?fields=Hosts/rack_info,Hosts/host_name,Hosts/maintenance_state,Hosts/public_host_name,' + 'Hosts/cpu_count,Hosts/ph_cpu_count,<lastAgentEnv>alerts_summary,Hosts/host_status,Hosts/host_state,' + 'Hosts/last_heartbeat_time,Hosts/ip,host_components/HostRoles/state,' + 'host_components/HostRoles/maintenance_state,host_components/HostRoles/stale_configs,' + 'host_components/HostRoles/service_name,host_components/HostRoles/display_name,' + 'host_components/HostRoles/desired_admin_state,<nameNodeMetrics>' + '<metrics>Hosts/total_mem<hostDetailsParams><stackVersions>&minimal_response=true';
    var stackVersionInfo = ',stack_versions/HostStackVersions,' + 'stack_versions/repository_versions/RepositoryVersions/repository_version,' + 'stack_versions/repository_versions/RepositoryVersions/id,' + 'stack_versions/repository_versions/RepositoryVersions/display_name',
        loggingResource = ',host_components/logging',
        isHostDetailPage = App.router.get('currentState.parentState.name') === 'hostDetails',
        isHostsPage = App.router.get('currentState.parentState.name') === 'hosts',
        hostDetailsParams = ',Hosts/os_arch,Hosts/os_type,metrics/cpu/cpu_system,metrics/cpu/cpu_user,' + 'metrics/memory/mem_total,metrics/memory/mem_free',
        nameNodeMetrics = 'host_components/metrics/dfs/namenode/ClusterId,' + 'host_components/metrics/dfs/FSNamesystem/HAState,';

    url = url.replace("<stackVersions>", stackVersionInfo);
    url = url.replace("<metrics>", loadMetricsSeparately ? "" : "metrics/disk,metrics/load/load_one,");
    url = url.replace('<hostDetailsParams>', isHostsPage ? '' : hostDetailsParams);
    url = url.replace('<lastAgentEnv>', isHostDetailPage ? 'Hosts/last_agent_env,' : '');
    url = url.replace('<nameNodeMetrics>', App.Service.find('HDFS').get('isLoaded') ? nameNodeMetrics : '');
    url = App.get('supports.logSearch') ? url + loggingResource : url;
    return url;
  },

  /**
   *
   * @param {Function} callback
   * @param {Function} error
   * @param {boolean} lazyLoadMetrics
   */
  updateHost: function updateHost(callback, error, lazyLoadMetrics) {
    var testUrl = this.get('HOSTS_TEST_URL'),
        self = this,
        hostDetailsFilter = '',
        mainHostController = App.router.get('mainHostController'),
        sortProperties = mainHostController.getSortProps(),
        isHostsLoaded = false,

    // load hosts metrics separately of lazyLoadMetrics=true, but metrics in current request if we are sorting
    loadMetricsSeparately = lazyLoadMetrics && !(sortProperties.length && ['loadAvg', 'diskUsage'].contains(sortProperties[0].name));
    this.get('queryParams').set('Hosts', mainHostController.getQueryParameters(true));
    if (App.router.get('currentState.parentState.name') === 'hosts') {
      App.updater.updateInterval('updateHost', App.get('contentUpdateInterval'));
    } else {
      if (App.router.get('currentState.parentState.name') === 'hostDetails') {
        hostDetailsFilter = App.router.get('location.lastSetURL').match(/\/hosts\/(.*)\/(summary|configs|alerts|stackVersions|logs)/)[1];
        App.updater.updateInterval('updateHost', App.get('componentsUpdateInterval'));
        //if host details page opened then request info only of one displayed host
        this.get('queryParams').set('Hosts', [{
          key: 'Hosts/host_name',
          value: [hostDetailsFilter],
          type: 'MULTIPLE',
          isHostDetails: true
        }]);
      } else {
        // On pages except for hosts/hostDetails, making sure hostsMapper loaded only once on page load, no need to update, but at least once
        isHostsLoaded = App.router.get('clusterController.isHostsLoaded');
        if (isHostsLoaded) {
          callback();
          return;
        }
      }
    }

    var realUrl = this.getUpdateHostUrlWithParams(loadMetricsSeparately);

    var clientCallback = function clientCallback(skipCall, queryParams, itemTotal) {
      var completeCallback = function completeCallback() {
        callback();
        if (loadMetricsSeparately) {
          self.loadHostsMetric(queryParams);
        }
      };
      if (skipCall) {
        //no hosts match filter by component
        App.hostsMapper.map({
          items: [],
          itemTotal: '0'
        });
        callback();
      } else {
        if (App.get('testMode')) {
          realUrl = testUrl;
        } else {
          realUrl = self.addParamsToHostsUrl(queryParams, sortProperties, realUrl);
        }

        App.HttpClient.get(realUrl, App.hostsMapper, {
          complete: completeCallback,
          beforeMap: function beforeMap(response) {
            if (itemTotal) {
              response.itemTotal = itemTotal;
            }
          },
          doGetAsPost: true,
          params: self.computeParameters(queryParams),
          error: error
        });
      }
    };

    if (!this.preLoadHosts(clientCallback)) {
      clientCallback(false, self.get('queryParams.Hosts'));
    }
  },

  updateHostsMetrics: function updateHostsMetrics(callback) {
    var queryParams = App.router.get('mainHostController').getQueryParameters(true);
    if (App.router.get('currentState.parentState.name') === 'hostDetails') {
      var currentHostname = App.router.get('location.lastSetURL').match(/\/hosts\/(.*)\/(summary|configs|alerts|stackVersions|logs)/)[1];
      queryParams = [{
        key: 'Hosts/host_name',
        value: [currentHostname],
        type: 'MULTIPLE',
        isHostDetails: true
      }];
    }
    this.loadHostsMetric(queryParams).always(callback);
  },

  /**
   *
   * @param {Array} queryParams
   * @param {Array} sortProperties
   * @param {string} realUrl
   * @returns {string}
   */
  addParamsToHostsUrl: function addParamsToHostsUrl(queryParams, sortProperties, realUrl) {
    var paginationProps = this.computeParameters(queryParams.filter(function (param) {
      return this.get('paginationKeys').contains(param.key);
    }, this));
    var sortProps = this.computeParameters(sortProperties);

    return App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + realUrl + (paginationProps.length > 0 ? '&' + paginationProps : '') + (sortProps.length > 0 ? '&' + sortProps : '');
  },

  /**
   * lazy load metrics of hosts
   * @param {Array} queryParams
   * @returns {$.ajax|null}
   */
  loadHostsMetric: function loadHostsMetric(queryParams) {
    var isAmbariMetricsStarted = App.Service.find('AMBARI_METRICS').get('isStarted'),
        hostDetailsParam = queryParams.findProperty('isHostDetails'),
        currentHostName = hostDetailsParam && hostDetailsParam.value[0],
        isHostWithNameNode = currentHostName && App.HostComponent.find('NAMENODE_' + currentHostName).get('isLoaded');
    if (isAmbariMetricsStarted || isHostWithNameNode) {
      var realUrl = '/hosts?fields=',
          realUrlFields = [];
      if (isAmbariMetricsStarted) {
        realUrlFields.push('metrics/disk/disk_free', 'metrics/disk/disk_total', 'metrics/load/load_one');
      }
      if (isHostWithNameNode) {
        realUrlFields.push('host_components/metrics/dfs/namenode/ClusterId', 'host_components/metrics/jvm/HeapMemoryMax', 'host_components/metrics/jvm/HeapMemoryUsed', 'host_components/metrics/dfs/FSNamesystem/CapacityUsed', 'host_components/metrics/dfs/FSNamesystem/CapacityTotal', 'host_components/metrics/dfs/FSNamesystem/CapacityRemaining', 'host_components/metrics/dfs/FSNamesystem/CapacityNonDFSUsed', 'host_components/metrics/rpc/client/RpcQueueTime_avg_time', 'host_components/metrics/runtime/StartTime');
      }
      realUrl += realUrlFields.join(',') + '&minimal_response=true';
      return App.ajax.send({
        name: 'hosts.metrics.lazy_load',
        sender: this,
        data: {
          url: this.addParamsToHostsUrl(queryParams, [], realUrl),
          parameters: this.computeParameters(queryParams)
        },
        success: 'loadHostsMetricSuccessCallback'
      });
    }
    return $.Deferred().resolve().promise();
  },

  /**
   * success callback of <code>loadHostsMetric</code>
   * @param {object} data
   */
  loadHostsMetricSuccessCallback: function loadHostsMetricSuccessCallback(data) {
    App.hostsMapper.setMetrics(data);
    if (App.router.get('currentState.parentState.name') === 'hostDetails' && data) {
      var hostComponentsData = Em.get(data, 'items.0.host_components');
      if (hostComponentsData) {
        var nameNodeData = hostComponentsData.findProperty('HostRoles.component_name', 'NAMENODE');
        if (nameNodeData) {
          var hostName = Em.get(data, 'items.0.Hosts.host_name'),
              nameNodeModelMap = App.serviceMetricsMapper.activeNameNodeConfig,
              processedModelProperties = this.nameNodeMetricsModelProperties,
              hdfsModel = App.HDFSService.find('HDFS'),
              componentModel = App.HostComponent.find('NAMENODE_' + hostName);
          Object.keys(nameNodeModelMap).forEach(function (key) {
            if (processedModelProperties.contains(key)) {
              var modelKey = stringUtils.underScoreToCamelCase(key);
              hdfsModel.get(modelKey)[hostName] = Em.get(nameNodeData, nameNodeModelMap[key]);
              hdfsModel.propertyDidChange(modelKey);
            }
          });
          componentModel.setProperties({
            clusterIdValue: Em.get(nameNodeData, 'metrics.dfs.namenode.ClusterId')
          });
        }
      }
    }
  },

  /**
   * identify if any filter by host-component is active
   * if so run @getHostByHostComponents
   *
   * @param callback
   * @return {Boolean}
   */
  preLoadHosts: function preLoadHosts(callback) {
    if (this.get('queryParams.Hosts').length > 0 && this.get('queryParams.Hosts').filter(function (param) {
      return param.isComponentRelatedFilter;
    }, this).length > 0) {
      this.getHostByHostComponents(callback);
      return true;
    }
    return false;
  },

  /**
   * get hosts' names which match filter by host-component
   * @param callback
   */
  getHostByHostComponents: function getHostByHostComponents(callback) {
    var realUrl = '/hosts?<parameters>minimal_response=true';

    App.ajax.send({
      name: 'hosts.host_components.pre_load',
      sender: this,
      data: {
        url: this.getComplexUrl(realUrl, this.get('queryParams.Hosts')),
        callback: callback
      },
      success: 'getHostByHostComponentsSuccessCallback',
      error: 'getHostByHostComponentsErrorCallback'
    });
  },
  getHostByHostComponentsSuccessCallback: function getHostByHostComponentsSuccessCallback(data, opt, params) {
    var queryParams = this.get('queryParams.Hosts');
    var hostNames = data.items.mapProperty('Hosts.host_name');
    var skipCall = hostNames.length === 0;

    var itemTotal = parseInt(data.itemTotal, 10);
    if (!isNaN(itemTotal)) {
      App.router.set('mainHostController.filteredCount', itemTotal);
    }

    if (skipCall) {
      params.callback(skipCall);
    } else {
      queryParams = [{
        key: 'Hosts/host_name',
        value: hostNames,
        type: 'MULTIPLE'
      }];
      params.callback(skipCall, queryParams, itemTotal);
    }
  },
  getHostByHostComponentsErrorCallback: Em.K,

  graphs: [],

  graphsUpdate: function graphsUpdate(callback) {
    var existedGraphs = [];
    this.get('graphs').forEach(function (_graph) {
      var view = Em.View.views[_graph.id];
      if (view) {
        existedGraphs.push(_graph);
        if (!view.get('isRequestRunning')) {
          //console.log('updated graph', _graph.name);
          view.loadData();
          //if graph opened as modal popup update it to
          if ($(".modal-graph-line .modal-body #" + _graph.popupId + "-container-popup").length) {
            view.loadData();
          }
        }
      }
    });
    callback();
    this.set('graphs', existedGraphs);
  },

  /**
   * Updates the services information.
   *
   * @param callback
   */
  updateServiceMetric: function updateServiceMetric(callback) {
    var self = this;
    self.set('isUpdated', false);
    var isATSPresent = App.StackServiceComponent.find().findProperty('componentName', 'APP_TIMELINE_SERVER');

    var conditionalFields = this.getConditionalFields(),
        requestsRunningStatus = this.get('requestsRunningStatus'),
        conditionalFieldsString = conditionalFields.length > 0 ? ',' + conditionalFields.join(',') : '',
        testUrl = '/data/dashboard/HDP2/master_components.json',
        isFlumeInstalled = App.cache.services.mapProperty('ServiceInfo.service_name').contains('FLUME'),
        isATSInstalled = App.cache.services.mapProperty('ServiceInfo.service_name').contains('YARN') && isATSPresent,
        flumeHandlerParam = isFlumeInstalled ? 'ServiceComponentInfo/component_name=FLUME_HANDLER|' : '',
        atsHandlerParam = isATSInstalled ? 'ServiceComponentInfo/component_name=APP_TIMELINE_SERVER|' : '',
        haComponents = App.get('isHaEnabled') ? 'ServiceComponentInfo/component_name=JOURNALNODE|ServiceComponentInfo/component_name=ZKFC|' : '',
        realUrl = '/components/?' + flumeHandlerParam + atsHandlerParam + haComponents + 'ServiceComponentInfo/category.in(MASTER,CLIENT)&fields=' + 'ServiceComponentInfo/service_name,' + 'host_components/HostRoles/display_name,' + 'host_components/HostRoles/host_name,' + 'host_components/HostRoles/public_host_name,' + 'host_components/HostRoles/state,' + 'host_components/HostRoles/maintenance_state,' + 'host_components/HostRoles/stale_configs,' + 'host_components/HostRoles/ha_state,' + 'host_components/HostRoles/desired_admin_state,' + conditionalFieldsString + '&minimal_response=true';

    var servicesUrl = this.getUrl(testUrl, realUrl);
    callback = callback || function () {
      self.set('isUpdated', true);
    };

    if (!requestsRunningStatus.updateServiceMetric) {
      requestsRunningStatus.updateServiceMetric = true;
      App.HttpClient.get(servicesUrl, App.serviceMetricsMapper, {
        complete: function complete() {
          App.set('router.mainServiceItemController.isServicesInfoLoaded', App.get('router.clusterController.isLoaded'));
          requestsRunningStatus.updateServiceMetric = false;
          callback();
        }
      });
    } else {
      callback();
    }
  },
  /**
   * construct conditional parameters of query, depending on which services are installed
   * @return {Array}
   */
  getConditionalFields: function getConditionalFields() {
    var conditionalFields = this.get('serviceComponentMetrics').slice(0);
    var serviceSpecificParams = $.extend({}, this.get('serviceSpecificParams'));
    var metricsKey = 'metrics/';

    if (/^2.1/.test(App.get('currentStackVersionNumber'))) {
      serviceSpecificParams.STORM = 'metrics/api/cluster/summary';
    } else if (/^2.2/.test(App.get('currentStackVersionNumber'))) {
      serviceSpecificParams.STORM = 'metrics/api/v1/cluster/summary,metrics/api/v1/topology/summary';
    }
    serviceSpecificParams.ONEFS = 'metrics/*,';

    App.cache.services.forEach(function (service) {
      var urlParams = serviceSpecificParams[service.ServiceInfo.service_name];
      if (urlParams) {
        conditionalFields.push(urlParams);
      }
    });

    //first load shouldn't contain metrics in order to make call lighter
    if (!App.get('router.clusterController.isServiceMetricsLoaded')) {
      return conditionalFields.filter(function (condition) {
        return condition.indexOf(metricsKey) === -1;
      });
    }

    return conditionalFields;
  },

  updateServices: function updateServices(callback) {
    var testUrl = '/data/services/HDP2/services.json';
    var componentConfigUrl = this.getUrl(testUrl, '/services?fields=ServiceInfo/state,ServiceInfo/maintenance_state,ServiceInfo/desired_repository_version_id,components/ServiceComponentInfo/component_name&minimal_response=true');
    App.HttpClient.get(componentConfigUrl, App.serviceMapper, {
      complete: callback
    });
  },

  updateComponentsState: function updateComponentsState(callback) {
    var testUrl = '/data/services/HDP2/components_state.json';
    var realUrl = '/components/?fields=ServiceComponentInfo/service_name,' + 'ServiceComponentInfo/category,ServiceComponentInfo/installed_count,ServiceComponentInfo/started_count,ServiceComponentInfo/init_count,ServiceComponentInfo/install_failed_count,ServiceComponentInfo/unknown_count,ServiceComponentInfo/total_count,ServiceComponentInfo/display_name,host_components/HostRoles/host_name&minimal_response=true';
    var url = this.getUrl(testUrl, realUrl);

    App.HttpClient.get(url, App.componentsStateMapper, {
      complete: callback
    });
  },

  updateAlertDefinitions: function updateAlertDefinitions(callback) {
    var testUrl = '/data/alerts/alertDefinitions.json';
    var realUrl = '/alert_definitions?fields=' + 'AlertDefinition/component_name,AlertDefinition/description,AlertDefinition/enabled,AlertDefinition/repeat_tolerance,AlertDefinition/repeat_tolerance_enabled,' + 'AlertDefinition/id,AlertDefinition/ignore_host,AlertDefinition/interval,AlertDefinition/label,AlertDefinition/name,' + 'AlertDefinition/scope,AlertDefinition/service_name,AlertDefinition/source,AlertDefinition/help_url';
    var url = this.getUrl(testUrl, realUrl);

    App.HttpClient.get(url, App.alertDefinitionsMapper, {
      complete: callback
    });
  },

  updateUnhealthyAlertInstances: function updateUnhealthyAlertInstances(callback) {
    var testUrl = '/data/alerts/alert_instances.json';
    var queryParams = this.get('queryParamsForUnhealthyAlertInstances');
    var realUrl = '/alerts?fields=' + 'Alert/component_name,Alert/definition_id,Alert/definition_name,Alert/host_name,Alert/id,Alert/instance,' + 'Alert/label,Alert/latest_timestamp,Alert/maintenance_state,Alert/original_timestamp,Alert/scope,' + 'Alert/service_name,Alert/state,Alert/text,Alert/repeat_tolerance,Alert/repeat_tolerance_remaining' + '&Alert/state.in(CRITICAL,WARNING)&Alert/maintenance_state.in(OFF)&from=' + queryParams.from + '&page_size=' + queryParams.page_size;
    var url = this.getUrl(testUrl, realUrl);

    App.HttpClient.get(url, App.alertInstanceMapper, {
      complete: callback
    });
  },

  updateAlertDefinitionSummary: function updateAlertDefinitionSummary(callback) {
    //TODO move to clusterController
    var testUrl = '/data/alerts/alert_summary.json';
    var realUrl = '/alerts?format=groupedSummary';
    var url = this.getUrl(testUrl, realUrl);

    App.HttpClient.get(url, App.alertDefinitionSummaryMapper, {
      complete: callback
    });
  },

  updateAlertGroups: function updateAlertGroups(callback) {
    var testUrl = '/data/alerts/alertGroups.json';
    var realUrl = '/alert_groups?fields=' + 'AlertGroup/default,AlertGroup/definitions,AlertGroup/id,AlertGroup/name,AlertGroup/targets';
    var url = this.getUrl(testUrl, realUrl);

    App.HttpClient.get(url, App.alertGroupsMapper, {
      complete: callback
    });
  },

  updateAlertNotifications: function updateAlertNotifications(callback) {
    App.HttpClient.get(App.get('apiPrefix') + '/alert_targets?fields=*', App.alertNotificationMapper, {
      complete: callback
    });
  },

  configsChangedHandler: function configsChangedHandler(event) {
    if (event.configs && event.configs.someProperty('type', 'cluster-env')) {
      this.updateClusterEnv();
    }
    App.router.get('configurationController').updateConfigTags();
  },

  //TODO - update service auto-start to use this
  updateClusterEnv: function updateClusterEnv() {
    return App.router.get('configurationController').getCurrentConfigsBySites(['cluster-env']).done(function (config) {
      App.router.get('clusterController').set('clusterEnv', config[0]);
    });
  },

  loadClusterConfig: function loadClusterConfig(callback) {
    return App.ajax.send({
      name: 'config.tags.site',
      sender: this,
      data: {
        site: 'cluster-env'
      },
      callback: callback
    });
  },

  updateWizardWatcher: function updateWizardWatcher(callback) {
    App.router.get('wizardWatcherController').getUser().complete(callback);
  },

  /**
   * Request to fetch logging info for specified host.
   *
   * @method updateLogging
   * @param {string} hostName
   * @param {string[]|boolean} fields additional fields to request e.g. ['field1', 'field2']
   * @param {function} callback execute on when request succeed, json response will be passed to first argument
   * @returns {$.Deferred.promise()}
   */
  updateLogging: function updateLogging(hostName, fields, callback) {
    var flds = !!fields ? "," + fields.join(',') : "";

    return App.ajax.send({
      name: 'host.logging',
      sender: this,
      data: {
        hostName: hostName,
        callback: callback,
        fields: flds
      },
      success: 'updateLoggingSuccess'
    });
  },

  updateLoggingSuccess: function updateLoggingSuccess(data, opt, params) {
    var clbk = params.callback || function () {};
    clbk(data);
  },

  updateHDFSNameSpaces: function updateHDFSNameSpaces() {
    if (App.Service.find('HDFS').get('isLoaded') && App.get('isHaEnabled')) {
      App.router.get('configurationController').getCurrentConfigsBySites(['hdfs-site']).done(function (configs) {
        var properties = configs && configs[0] && configs[0].properties;
        if (properties) {
          var nameSpaceProperty = properties['dfs.nameservices'];
          if (nameSpaceProperty) {
            var nameSpaces = nameSpaceProperty.split(',').map(function (nameSpace) {
              var nameNodeIdsProperty = properties['dfs.ha.namenodes.' + nameSpace];
              if (nameNodeIdsProperty) {
                var nameNodeIds = nameNodeIdsProperty.split(','),
                    hostNames = nameNodeIds.map(function (id) {
                  var propertyValue = properties['dfs.namenode.http-address.' + nameSpace + '.' + id],
                      matches = propertyValue && propertyValue.match(/([\D\d]+)\:\d+$/),
                      hostName = matches && matches[1];
                  return hostName;
                });
                return {
                  nameSpace: nameSpace,
                  hostNames: hostNames
                };
              }
            }),
                allNameNodes = App.HDFSService.find().objectAt(0).get('hostComponents').filterProperty('componentName', 'NAMENODE');
            allNameNodes.forEach(function (component) {
              var nameSpaceObject = nameSpaces.find(function (ns) {
                return ns && ns.hostNames && ns.hostNames.contains(component.get('hostName'));
              });
              if (nameSpaceObject) {
                component.set('haNameSpace', nameSpaceObject.nameSpace);
              }
            });
            App.set('router.clusterController.isHDFSNameSpacesLoaded', true);
          }
        }
      });
    } else {
      App.set('router.clusterController.isHDFSNameSpacesLoaded', true);
    }
  }
});

});

require.register("controllers/global/user_settings_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var timezoneUtils = require('utils/date/timezone');

/**
 * Controller for user settings
 * Allows to get them from persist and update them to the persist
 *
 * @class UserSettingsController
 */
App.UserSettingsController = Em.Controller.extend(App.Persist, {

  name: 'userSettingsController',

  /**
   * @type {object}
   */
  userSettings: {},

  /**
   * Each property's type is {name: string, defaultValue: *, formatter: function}
   *
   * @type {object}
   */
  userSettingsKeys: function () {
    var loginName = App.router.get('loginName');
    var prefix = 'admin-settings-';
    return {
      show_bg: {
        name: prefix + 'show-bg-' + loginName,
        defaultValue: true
      },
      timezone: {
        name: prefix + 'timezone-' + loginName,
        defaultValue: timezoneUtils.detectUserTimezone(),
        formatter: function formatter(v) {
          return timezoneUtils.get('timezonesMappedByValue')[v];
        }
      }
    };
  }.property('App.router.loginName'),

  /**
   * Load some user's setting from the persist
   * If <code>persistKey</code> is not provided, all settings are loaded
   *
   * @param {string} [persistKey]
   * @method dataLoading
   * @returns {$.Deferred.promise}
   */
  dataLoading: function dataLoading(persistKey) {
    var key = persistKey ? this.get('userSettingsKeys.' + persistKey + '.name') : '';
    var dfd = $.Deferred();
    var self = this;
    this.getUserPref(key).complete(function () {
      var curPref = self.get('currentPrefObject');
      self.set('currentPrefObject', null);
      dfd.resolve(curPref);
    });
    return dfd.promise();
  },

  /**
   * Success-callback for user pref
   *
   * @param {?object} response
   * @param {object} opt
   * @returns {?object}
   * @method getUserPrefSuccessCallback
   */
  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response, opt) {
    var getAllRequest = opt.url.endsWith('persist/');
    if (Em.isNone(response)) {
      this.updateUserPrefWithDefaultValues(response, getAllRequest);
    }
    this.set('currentPrefObject', response);
    return response;
  },

  /**
   * Error-callback for user-pref request
   * Update user pref with default values if user firstly login
   *
   * @param {object} request
   * @method getUserPrefErrorCallback
   */
  getUserPrefErrorCallback: function getUserPrefErrorCallback(request) {
    // this user is first time login
    if (404 === request.status) {
      this.updateUserPrefWithDefaultValues();
    }
  },

  /**
   * Load all current user's settings to the <code>userSettings</code>
   *
   * @method getAllUserSettings
   */
  getAllUserSettings: function getAllUserSettings() {
    var userSettingsKeys = this.get('userSettingsKeys');
    var userSettings = {};
    var self = this;
    this.dataLoading().done(function (json) {
      if (!json) {
        return;
      }
      Object.keys(userSettingsKeys).forEach(function (k) {
        var value = userSettingsKeys[k].defaultValue;
        if (undefined === json[userSettingsKeys[k].name]) {
          self.postUserPref(k, userSettingsKeys[k].defaultValue);
        } else {
          value = JSON.parse(json[userSettingsKeys[k].name]);
        }
        if ('function' === Em.typeOf(userSettingsKeys[k].formatter)) {
          value = userSettingsKeys[k].formatter(value);
        }
        userSettings[k] = value;
      });
    });
    this.set('userSettings', userSettings);
  },

  /**
   * If user doesn't have any settings stored in the persist,
   * default values should be populated there
   *
   * @param {object} [response]
   * @param {boolean} [getAllRequest] determines, if user tried to get one field or all fields
   * @method updateUserPrefWithDefaultValues
   */
  updateUserPrefWithDefaultValues: function updateUserPrefWithDefaultValues(response, getAllRequest) {
    var r = response || {};
    var keys = this.get('userSettingsKeys');
    var self = this;
    if (getAllRequest) {
      Object.keys(keys).forEach(function (key) {
        if (Em.isNone(r[keys[key].name])) {
          self.postUserPref(key, keys[key].defaultValue);
        }
      });
    }
  },

  /**
   * "Short"-key method for post user settings to the persist
   * Example:
   *  real key is something like 'userSettingsKeys.timezone.name'
   *  but user should call this method with 'timezone'
   *
   * @method postUserPref
   * @param {string} key
   * @param {*} value
   * @returns {*}
   */
  postUserPref: function postUserPref(key, value) {
    var normalizedKey = key.startsWith('userSettingsKeys.') ? key : 'userSettingsKeys.' + key + '.name';
    var shortKey = normalizedKey.replace('userSettingsKeys.', '').replace('.name', '');
    this.set('userSettings.' + shortKey, value);
    return this._super(this.get(normalizedKey), value);
  },

  /**
   * Open popup with user settings after settings-request is complete
   *
   * @method showSettingsPopup
   */
  showSettingsPopup: function showSettingsPopup() {
    var self = this;
    // Settings only for admins
    if (!App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK')) {
      return;
    }

    this.dataLoading().done(function (response) {
      self.loadPrivileges().complete(function () {
        self._showSettingsPopup(response);
      });
    });
  },

  loadPrivileges: function loadPrivileges() {
    return App.ajax.send({
      name: 'router.user.privileges',
      sender: this,
      data: {
        userName: App.db.getLoginName()
      },
      success: 'loadPrivilegesSuccessCallback'
    });
  },

  loadPrivilegesSuccessCallback: function loadPrivilegesSuccessCallback(data) {
    var key,
        privileges = this.parsePrivileges(data),
        clusters = [],
        views = [];

    for (key in privileges.clusters) {
      clusters.push({
        name: key,
        privileges: privileges.clusters[key]
      });
    }
    for (key in privileges.views) {
      views.push({
        instance_name: key,
        privileges: privileges.views[key].privileges,
        version: privileges.views[key].version,
        view_name: privileges.views[key].view_name
      });
    }
    privileges.clusters = clusters;
    privileges.views = views;
    this.set('privileges', data.items.length ? privileges : null);
    this.set('noClusterPriv', Em.isEmpty(clusters));
    this.set('noViewPriv', Em.isEmpty(views));
    this.set('hidePrivileges', this.get('noClusterPriv') && this.get('noViewPriv'));
  },

  /**
   *
   * @param {?object} data
   * @returns {{clusters: {}, views: {}}}
   */
  parsePrivileges: function parsePrivileges(data) {
    var privileges = {
      clusters: {},
      views: {}
    };
    data.items.forEach(function (privilege) {
      privilege = privilege.PrivilegeInfo;
      if (privilege.type === 'CLUSTER') {
        // This is cluster
        privileges.clusters[privilege.cluster_name] = privileges.clusters[privilege.cluster_name] || [];
        privileges.clusters[privilege.cluster_name].push(privilege.permission_label);
      } else if (privilege.type === 'VIEW') {
        privileges.views[privilege.instance_name] = privileges.views[privilege.instance_name] || { privileges: [] };
        privileges.views[privilege.instance_name].version = privilege.version;
        privileges.views[privilege.instance_name].view_name = privilege.view_name;
        privileges.views[privilege.instance_name].privileges.push(privilege.permission_label);
      }
    });
    return privileges;
  },

  /**
   * Show popup with settings for user
   * Don't call this method directly! Use <code>showSettingsPopup</code>
   *
   * @param {object} response
   * @returns {App.ModalPopup}
   * @method _showSettingsPopup
   * @private
   */
  _showSettingsPopup: function _showSettingsPopup(response) {
    var keys = this.get('userSettingsKeys');
    var curValue, self, timezonesFormatted, initValue, initTimezone;
    if (response[keys.show_bg.name]) {
      curValue = null;
      self = this;
      timezonesFormatted = timezoneUtils.get('timezones');
      initValue = JSON.parse(response[keys.show_bg.name]);
      initTimezone = timezonesFormatted.findProperty('value', JSON.parse(response[keys.timezone.name]));
      return App.ModalPopup.show({

        header: Em.I18n.t('common.userSettings'),

        bodyClass: Em.View.extend({

          templateName: require('templates/common/settings'),

          isNotShowBgChecked: !initValue,

          updateValue: function () {
            curValue = !this.get('isNotShowBgChecked');
          }.observes('isNotShowBgChecked'),

          timezonesList: timezonesFormatted,

          privileges: self.get('privileges'),

          isAdmin: App.get('isAdmin'),

          noClusterPriv: self.get('noClusterPriv'),

          noViewPriv: self.get('noViewPriv'),

          hidePrivileges: self.get('hidePrivileges') || App.get('isAdmin')
        }),

        /**
         * @type {string}
         */
        selectedTimezone: initTimezone,

        primary: Em.I18n.t('common.save'),

        onPrimary: function onPrimary() {
          if (Em.isNone(curValue)) {
            curValue = initValue;
          }
          var tz = this.get('selectedTimezone.value');
          var popup = this;
          if (!App.get('testMode')) {
            self.postUserPref('show_bg', curValue).always(function () {
              self.postUserPref('timezone', tz).always(function () {
                if (popup.needsPageRefresh()) {
                  location.reload();
                }
              });
            });
          }
          this._super();
        },

        /**
         * Determines if page should be refreshed after user click "Save"
         *
         * @returns {boolean}
         */
        needsPageRefresh: function needsPageRefresh() {
          return initTimezone !== this.get('selectedTimezone');
        }
      });
    } else {
      App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('app.settings.noData'));
    }
  }

});

});

require.register("controllers/global/wizard_watcher_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardWatcherController = Em.Controller.extend(App.Persist, {
  name: 'wizardWatcherController',

  /**
   * @const
   */
  PREF_KEY: 'wizard-data',

  /**
   * name of user who working with wizard
   * @type {string|null}
   */
  wizardUser: null,

  /**
   * @type {string|null}
   */
  controllerName: null,

  /**
   * define whether Wizard is running
   * @type {boolean}
   */
  isWizardRunning: Em.computed.bool('wizardUser'),

  /**
   * @type {string}
   */
  wizardDisplayName: function () {
    var controllerName = this.get('controllerName');
    return controllerName ? Em.I18n.t('wizard.inProgress').format(App.router.get(controllerName).get('displayName'), this.get('wizardUser')) : '';
  }.property('controllerName'),

  /**
   * define whether logged in user is the one who started wizard
   * @type {boolean}
   */
  isNonWizardUser: function () {
    return this.get('isWizardRunning') && this.get('wizardUser') !== App.router.get('loginName');
  }.property('App.router.loginName', 'wizardUser').volatile(),

  /**
   * set user who launched wizard
   * @returns {$.ajax}
   */
  setUser: function setUser(controllerName) {
    return this.postUserPref(this.get('PREF_KEY'), {
      userName: App.router.get('loginName'),
      controllerName: controllerName
    });
  },

  /**
   * reset user who launched wizard
   * @returns {$.ajax}
   */
  resetUser: function resetUser() {
    return this.postUserPref(this.get('PREF_KEY'), null);
  },

  /**
   * get user who launched wizard
   * @returns {$.ajax}
   */
  getUser: function getUser() {
    return this.getUserPref(this.get('PREF_KEY'));
  },

  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(data) {
    if (Em.isNone(data)) {
      this.set('wizardUser', null);
      this.set('controllerName', null);
    } else {
      this.set('wizardUser', data.userName);
      this.set('controllerName', data.controllerName);
    }
  },

  getUserPrefErrorCallback: function getUserPrefErrorCallback() {
    this.resetUser();
  }
});

});

require.register("controllers/installer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');
var validator = require('utils/validator');

App.InstallerController = App.WizardController.extend(App.Persist, {

  name: 'installerController',

  isCheckInProgress: false,

  totalSteps: 11,

  isInstallerWizard: true,

  content: Em.Object.create({
    cluster: null,
    installOptions: null,
    hosts: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: null,
    advancedServiceConfig: null,
    configGroups: [],
    slaveGroupProperties: null,
    stacks: null,
    clients: [],
    // list of components, that was added from configs page via AssignMasterOnStep7Controller
    componentsFromConfigs: [],
    /**
     * recommendations for host groups loaded from server
     */
    recommendations: null,
    /**
     * recommendationsHostGroups - current component assignment after 5 and 6 steps, 
     * or adding hiveserver2 interactive on "configure services" page
     * (uses for host groups validation and to load recommended configs)
     */
    recommendationsHostGroups: null,
    controllerName: 'installerController'
  }),

  /**
   * Wizard properties in local storage, which should be cleaned right after wizard has been finished
   */
  dbPropertiesToClean: ['service', 'hosts', 'masterComponentHosts', 'slaveComponentHosts', 'cluster', 'allHostNames', 'installOptions', 'allHostNamesPattern', 'serviceComponents', 'clientInfo', 'selectedServiceNames', 'serviceConfigGroups', 'serviceConfigProperties', 'fileNamesToUpdate', 'bootStatus', 'stacksVersions', 'currentStep', 'serviceInfo', 'hostInfo', 'recommendations', 'recommendationsHostGroups', 'recommendationsConfigs', 'componentsFromConfigs', 'operatingSystems', 'repositories'],

  init: function init() {
    this._super();
    this.get('isStepDisabled').setEach('value', true);
    this.get('isStepDisabled').pushObject(Ember.Object.create({
      step: 0,
      value: true
    }));
  },
  /**
   * redefined connectOutlet method to avoid view loading by unauthorized user
   * @param view
   * @param content
   */
  connectOutlet: function connectOutlet(view, content) {
    if (App.db.getAuthenticated()) {
      this._super(view, content);
    }
  },

  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'));
  },

  getHosts: function getHosts() {
    return [];
  },

  /**
   * Remove host from model. Used at <code>Confirm hosts(step2)</code> step
   * @param hosts Array of hosts, which we want to delete
   */
  removeHosts: function removeHosts(hosts) {
    var dbHosts = this.getDBProperty('hosts');
    hosts.forEach(function (_hostInfo) {
      var host = _hostInfo.name;
      delete dbHosts[host];
    });
    this.setDBProperty('hosts', dbHosts);
  },

  cancelInstall: function cancelInstall() {
    return App.showConfirmationPopup(function () {
      App.router.get('applicationController').goToAdminView();
    });
  },

  /**
   * Load services data. Will be used at <code>Select services(step4)</code> step
   */
  loadServices: function loadServices() {
    var dfd = $.Deferred();
    var self = this;
    var stackServices = App.StackService.find().mapProperty('serviceName');
    if (!(stackServices.length && App.StackService.find().objectAt(0).get('stackVersion') === App.get('currentStackVersionNumber'))) {
      this.loadServiceComponents().complete(function () {
        self.set('content.services', App.StackService.find().forEach(function (item) {
          // user the service version from VersionDefinition
          var serviceInStack = App.Stack.find().findProperty('isSelected').get('stackServices').findProperty('name', item.get('serviceName'));
          var serviceVersionDisplay = serviceInStack ? serviceInStack.get('latestVersion') : item.get('serviceVersion');
          item.set('serviceVersionDisplay', serviceVersionDisplay);
        }));
        dfd.resolve();
      });
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  /**
   * total set of hosts registered to cluster, analog of App.Host model,
   * used in Installer wizard until hosts are installed
   */
  allHosts: function () {
    var rawHosts = this.get('content.hosts');
    var masterComponents = this.get('content.masterComponentHosts');
    var slaveComponents = this.get('content.slaveComponentHosts');
    var hosts = [];
    masterComponents.forEach(function (component) {
      var host = rawHosts[component.hostName];
      if (host.hostComponents) {
        host.hostComponents.push(Em.Object.create({
          componentName: component.component,
          displayName: component.display_name
        }));
      } else {
        rawHosts[component.hostName].hostComponents = [Em.Object.create({
          componentName: component.component,
          displayName: component.display_name
        })];
      }
    });
    slaveComponents.forEach(function (component) {
      component.hosts.forEach(function (rawHost) {
        var host = rawHosts[rawHost.hostName];
        if (host.hostComponents) {
          host.hostComponents.push(Em.Object.create({
            componentName: component.componentName,
            displayName: component.displayName
          }));
        } else {
          rawHosts[rawHost.hostName].hostComponents = [Em.Object.create({
            componentName: component.componentName,
            displayName: component.displayName
          })];
        }
      });
    });

    for (var hostName in rawHosts) {
      var host = rawHosts[hostName];
      hosts.pushObject(Em.Object.create({
        id: host.name,
        hostName: host.name,
        hostComponents: host.hostComponents || []
      }));
    }
    return hosts;
  }.property('content.hosts'),

  stacks: [],

  /**
   * stack names used as auxiliary data to query stacks by name
   */
  stackNames: [],

  /**
   * Load stacks data from server or take exist data from in memory variable {{content.stacks}}
   * The series of API calls will be called  When landing first time on Select Stacks page
   * or on hitting refresh post select stacks page in installer wizard
   */
  loadStacks: function loadStacks() {
    var stacks = this.get('content.stacks');
    var dfd = $.Deferred();
    if (stacks && stacks.get('length')) {
      App.set('currentStackVersion', App.Stack.find().findProperty('isSelected').get('stackNameVersion'));
      dfd.resolve(true);
    } else {
      App.ajax.send({
        name: 'wizard.stacks',
        sender: this,
        success: 'loadStacksSuccessCallback',
        error: 'loadStacksErrorCallback'
      }).complete(function () {
        dfd.resolve(false);
      });
    }
    return dfd.promise();
  },

  /**
   * Send queries to load versions for each stack
   */
  loadStacksSuccessCallback: function loadStacksSuccessCallback(data) {
    this.get('stacks').clear();
    this.set('stackNames', data.items.mapProperty('Stacks.stack_name'));
  },

  /**
   * onError callback for loading stacks data
   */
  loadStacksErrorCallback: function loadStacksErrorCallback() {},

  /**
   * query every stack names from server
   * @return {Array}
   */
  loadStacksVersions: function loadStacksVersions() {
    var requests = [];
    var dfd = $.Deferred();
    this.get('stackNames').forEach(function (stackName) {
      requests.push(App.ajax.send({
        name: 'wizard.stacks_versions_definitions',
        sender: this,
        data: {
          stackName: stackName,
          dfd: dfd
        },
        success: 'loadStacksVersionsDefinitionsSuccessCallback',
        error: 'loadStacksVersionsErrorCallback'
      }));
    }, this);
    this.set('loadStacksRequestsCounter', requests.length);
    return dfd.promise();
  },

  /**
   * Counter for counting number of successful requests to load stack versions
   */
  loadStacksRequestsCounter: 0,

  /**
   * Parse loaded data and create array of stacks objects
   */
  loadStacksVersionsDefinitionsSuccessCallback: function loadStacksVersionsDefinitionsSuccessCallback(data, opt, params) {
    var stacks = App.db.getStacks();
    var oses = App.db.getOses();
    var repos = App.db.getRepos();
    this.decrementProperty('loadStacksRequestsCounter');
    var isStacksExistInDb = stacks && stacks.length;
    if (isStacksExistInDb) {
      stacks.forEach(function (_stack) {
        var stack = data.items.findProperty('VersionDefinition.id', _stack.id);
        if (stack) {
          stack.VersionDefinition.is_selected = _stack.is_selected;
        }
      }, this);
    }
    if (!data.items || !data.items.length) {
      this.setSelected(true, params.dfd);
    }
    data.items.sortProperty('VersionDefinition.stack_version').reverse().forEach(function (versionDefinition) {
      // to display repos panel, should map all available operating systems including empty ones
      var stackInfo = {};
      stackInfo.isStacksExistInDb = isStacksExistInDb;
      stackInfo.stacks = stacks;
      stackInfo.oses = oses;
      stackInfo.repos = repos;
      this.getSupportedOSList(versionDefinition, stackInfo, params.dfd);
    }, this);
  },

  mergeChanges: function mergeChanges(repos, oses, stacks) {
    var _repos = repos || [];
    var _oses = oses || [];
    var _stacks = stacks || [];
    _repos.forEach(function (repo) {
      if (App.Repository.find(repo.id).get('isLoaded')) {
        App.Repository.find(repo.id).set('baseUrl', repo.base_url);
      }
    });
    _oses.forEach(function (os) {
      if (App.OperatingSystem.find().findProperty('id', os.id)) {
        App.OperatingSystem.find().findProperty('id', os.id).set('isSelected', os.is_selected);
      }
    });
    //should delete the record on going to step 2, on going back to step 1, still need the record
    if (App.router.get('currentState.name') != "step1") {
      App.OperatingSystem.find().filterProperty('isSelected', false).forEach(function (os) {
        App.stackMapper.deleteRecord(os);
      });
    }
    _stacks.forEach(function (_stack) {
      var stack = App.Stack.find().findProperty('id', _stack.id);
      if (stack) {
        stack.set('useRedhatSatellite', _stack.use_redhat_satellite);
      }
    });
  },

  setSelected: function setSelected(isStacksExistInDb, dfd) {
    if (!isStacksExistInDb) {
      var stacks = App.Stack.find();
      stacks.setEach('isSelected', false);
      stacks.sortProperty('id').set('lastObject.isSelected', true);
    }
    this.set('content.stacks', App.Stack.find());
    var selected = App.Stack.find().findProperty('isSelected');
    App.set('currentStackVersion', selected ? selected.get('stackNameVersion') : null);
    dfd.resolve();
  },

  /**
   * Get the the repo version (to install) info, this data will be POST
   * @method startDeploy
   */
  getSelectedRepoVersionData: function getSelectedRepoVersionData() {
    var vdfData = App.db.getLocalRepoVDFData();
    var selectedStack = App.Stack.find().findProperty('isSelected', true);
    var isXMLdata = false;
    var data = {};
    if (selectedStack && selectedStack.get('showAvailable')) {
      //meaning user selected a public repo
      data = {
        "VersionDefinition": {
          "available": selectedStack.get('id')
        }
      };
      isXMLdata = false;
    } else if (vdfData && validator.isValidURL(vdfData)) {
      // meaning user uploaded a VDF via entering URL
      data = {
        "VersionDefinition": {
          "version_url": vdfData
        }
      };
      isXMLdata = false;
    } else if (vdfData) {
      // meaning user uploaded a local VDF.xml file
      isXMLdata = true;
      data = vdfData;
    } else {
      return null;
    }
    return {
      isXMLdata: isXMLdata,
      data: data
    };
  },

  /**
   * onError callback for loading stacks data
   */
  loadStacksVersionsErrorCallback: function loadStacksVersionsErrorCallback() {},

  /**
   * check server version and web client version
   */
  checkServerClientVersion: function checkServerClientVersion() {
    var dfd = $.Deferred();
    var self = this;
    self.getServerVersion().done(function () {
      dfd.resolve();
    });
    return dfd.promise();
  },
  getServerVersion: function getServerVersion() {
    return App.ajax.send({
      name: 'ambari.service',
      sender: this,
      data: {
        fields: '?fields=RootServiceComponents/component_version,RootServiceComponents/properties/server.os_family&minimal_response=true'
      },
      success: 'getServerVersionSuccessCallback',
      error: 'getServerVersionErrorCallback'
    });
  },
  getServerVersionSuccessCallback: function getServerVersionSuccessCallback(data) {
    var clientVersion = App.get('version');
    var serverVersion = data.RootServiceComponents.component_version.toString();
    this.set('ambariServerVersion', serverVersion);
    if (clientVersion) {
      this.set('versionConflictAlertBody', Em.I18n.t('app.versionMismatchAlert.body').format(serverVersion, clientVersion));
      this.set('isServerClientVersionMismatch', clientVersion !== serverVersion);
    } else {
      this.set('isServerClientVersionMismatch', false);
    }
    App.set('isManagedMySQLForHiveEnabled', App.config.isManagedMySQLForHiveAllowed(data.RootServiceComponents.properties['server.os_family']));
  },
  getServerVersionErrorCallback: function getServerVersionErrorCallback() {},

  /**
   * set stacks from server to content and local DB
   */
  setStacks: function setStacks() {
    App.db.setStacks(App.Stack.find().slice());
    this.set('content.stacks', App.Stack.find());
    App.db.setOses(App.OperatingSystem.find().slice());
    App.db.setRepos(App.Repository.find().slice());
  },

  /**
   * Save data to model
   * @param stepController App.WizardStep4Controller
   */
  saveServices: function saveServices(stepController) {
    var selectedServiceNames = [];
    var installedServiceNames = [];
    stepController.filterProperty('isSelected').forEach(function (item) {
      selectedServiceNames.push(item.get('serviceName'));
    });
    stepController.filterProperty('isInstalled').forEach(function (item) {
      installedServiceNames.push(item.get('serviceName'));
    });
    this.set('content.services', App.StackService.find());
    this.set('content.selectedServiceNames', selectedServiceNames);
    this.set('content.installedServiceNames', installedServiceNames);
    this.setDBProperties({
      selectedServiceNames: selectedServiceNames,
      installedServiceNames: installedServiceNames
    });
  },

  /**
   * Save Master Component Hosts data to Main Controller
   * @param stepController App.WizardStep5Controller
   * @param  skip  {Boolean}
   */
  saveMasterComponentHosts: function saveMasterComponentHosts(stepController, skip) {
    var obj = stepController.get('selectedServicesMasters'),
        hosts = this.getDBProperty('hosts');

    var masterComponentHosts = [];
    obj.forEach(function (_component) {
      masterComponentHosts.push({
        display_name: _component.get('display_name'),
        component: _component.get('component_name'),
        serviceId: _component.get('serviceId'),
        isInstalled: false,
        host_id: hosts[_component.get('selectedHost')].id
      });
    });

    this.set('content.masterComponentHosts', masterComponentHosts);
    if (!skip) {
      this.setDBProperty('masterComponentHosts', masterComponentHosts);
    }
  },

  /**
   * Load master component hosts data for using in required step controllers
   * @param inMemory {Boolean}: Load master component hosts from memory
   */
  loadMasterComponentHosts: function loadMasterComponentHosts(inMemory) {
    var props = this.getDBProperties(['masterComponentHosts', 'hosts']);
    var masterComponentHosts = !!inMemory ? this.get("content.masterComponentHosts") : props.masterComponentHosts,
        hosts = props.hosts || {},
        hostNames = Em.keys(hosts);
    if (Em.isNone(masterComponentHosts)) {
      masterComponentHosts = [];
    } else {
      masterComponentHosts.forEach(function (component) {
        for (var i = 0; i < hostNames.length; i++) {
          if (hosts[hostNames[i]].id === component.host_id) {
            component.hostName = hostNames[i];
            break;
          }
        }
      });
    }
    this.set("content.masterComponentHosts", masterComponentHosts);
  },

  loadCurrentHostGroups: function loadCurrentHostGroups() {
    this.set("content.recommendationsHostGroups", this.getDBProperty('recommendationsHostGroups'));
  },

  loadRecommendationsConfigs: function loadRecommendationsConfigs() {
    App.router.set("wizardStep7Controller.recommendationsConfigs", this.getDBProperty('recommendationsConfigs'));
  },

  /**
   * Load master component hosts data for using in required step controllers
   */
  loadSlaveComponentHosts: function loadSlaveComponentHosts() {
    var props = this.getDBProperties(['slaveComponentHosts', 'hosts']);
    var slaveComponentHosts = props.slaveComponentHosts,
        hosts = props.hosts || {},
        hostNames = Em.keys(hosts);
    if (!Em.isNone(slaveComponentHosts)) {
      slaveComponentHosts.forEach(function (component) {
        component.hosts.forEach(function (host) {
          for (var i = 0; i < hostNames.length; i++) {
            if (hosts[hostNames[i]].id === host.host_id) {
              host.hostName = hostNames[i];
              break;
            }
          }
        });
      });
    }
    this.set("content.slaveComponentHosts", slaveComponentHosts);
  },

  /**
   * Generate clients list for selected services and save it to model
   * @param stepController step4WizardController
   */
  saveClients: function saveClients(stepController) {
    var clients = [];
    stepController.get('content').filterProperty('isSelected', true).forEach(function (_service) {
      var client = _service.get('serviceComponents').filterProperty('isClient', true);
      client.forEach(function (clientComponent) {
        clients.pushObject({
          component_name: clientComponent.get('componentName'),
          display_name: clientComponent.get('displayName'),
          isInstalled: false
        });
      }, this);
    }, this);
    this.setDBProperty('clientInfo', clients);
    this.set('content.clients', clients);
  },

  /*
   * Post version definition file (.xml) to server, DRY_RUN = TRUE
   */
  postVersionDefinitionFile: function postVersionDefinitionFile(isXMLdata, data) {
    var dfd = $.Deferred();
    var name = isXMLdata ? 'wizard.step1.post_version_definition_file.xml' : 'wizard.step1.post_version_definition_file.url';

    App.ajax.send({
      name: name,
      sender: this,
      data: {
        dfd: dfd,
        data: data
      },
      success: 'postVersionDefinitionFileSuccessCallback',
      error: 'postVersionDefinitionFileErrorCallback'
    });
    return dfd.promise();
  },

  /**
   * onSuccess callback for postVersionDefinitionFile.
   */
  postVersionDefinitionFileSuccessCallback: function postVersionDefinitionFileSuccessCallback(_data, request, dataInfo) {
    if (_data.resources.length && _data.resources[0].VersionDefinition) {
      var data = _data.resources[0];
      // load the data info to display for details and contents panel
      data.VersionDefinition.id = Em.get(dataInfo, 'data.VersionDefinition.available') || data.VersionDefinition.id;
      var response = {
        id: data.VersionDefinition.id,
        stackVersion: data.VersionDefinition.stack_version,
        stackName: data.VersionDefinition.stack_name,
        type: data.VersionDefinition.type,
        stackNameVersion: data.VersionDefinition.stack_name + '-' + data.VersionDefinition.stack_version, /// HDP-2.3
        actualVersion: data.VersionDefinition.repository_version, /// 2.3.4.0-3846
        version: data.VersionDefinition.release ? data.VersionDefinition.release.version : null, /// 2.3.4.0
        releaseNotes: data.VersionDefinition.release ? data.VersionDefinition.release.notes : null,
        displayName: data.VersionDefinition.release ? data.VersionDefinition.stack_name + '-' + data.VersionDefinition.release.version : data.VersionDefinition.stack_name + '-' + data.VersionDefinition.repository_version, //HDP-2.3.4.0
        repoVersionFullName: data.VersionDefinition.stack_name + '-' + data.VersionDefinition.repository_version,
        osList: data.operating_systems,
        updateObj: data
      };
      var services = [];
      data.VersionDefinition.services.forEach(function (service) {
        services.push({
          name: service.name,
          version: service.versions[0].version,
          components: service.versions[0].components
        });
      });
      response.services = services;

      // to display repos panel, should map all available operating systems including empty ones
      var stackInfo = {};
      stackInfo.dfd = dataInfo.dfd;
      stackInfo.response = response;
      this.incrementProperty('loadStacksRequestsCounter');
      this.getSupportedOSListSuccessCallback(data, null, {
        stackName: data.VersionDefinition.stack_name,
        stackVersion: data.VersionDefinition.stack_version,
        versionDefinition: data,
        stackInfo: stackInfo
      });
    }
  },

  /*
   * Post version definition file (.xml) to server in step 8
   */
  postVersionDefinitionFileStep8: function postVersionDefinitionFileStep8(isXMLdata, data) {
    var dfd = $.Deferred();
    var name = isXMLdata == true ? 'wizard.step8.post_version_definition_file.xml' : 'wizard.step8.post_version_definition_file';
    App.ajax.send({
      name: name,
      sender: this,
      data: {
        dfd: dfd,
        data: data
      },
      success: 'postVersionDefinitionFileStep8SuccessCallback',
      error: 'postVersionDefinitionFileErrorCallback'
    });
    return dfd.promise();
  },
  /**
   * onSuccess callback for postVersionDefinitionFile.
   */
  postVersionDefinitionFileStep8SuccessCallback: function postVersionDefinitionFileStep8SuccessCallback(response, request, data) {
    if (response.resources.length && response.resources[0].VersionDefinition) {
      data.dfd.resolve({
        stackName: response.resources[0].VersionDefinition.stack_name,
        id: response.resources[0].VersionDefinition.id,
        stackVersion: response.resources[0].VersionDefinition.stack_version
      });
    }
  },

  /**
   * onError callback for postVersionDefinitionFile.
   */
  postVersionDefinitionFileErrorCallback: function postVersionDefinitionFileErrorCallback(request, ajaxOptions, error, data, params) {
    params.dfd.reject(data);
    var header = Em.I18n.t('installer.step1.useLocalRepo.uploadFile.error.title');
    var body = '';
    if (request && request.responseText) {
      try {
        var json = $.parseJSON(request.responseText);
        body = json.message;
      } catch (err) {}
    }
    App.db.setLocalRepoVDFData(undefined);
    App.showAlertPopup(header, body);
  },

  getSupportedOSList: function getSupportedOSList(versionDefinition, stackInfo, dfd) {
    this.incrementProperty('loadStacksRequestsCounter');
    return App.ajax.send({
      name: 'wizard.step1.get_supported_os_types',
      sender: this,
      data: {
        stackName: versionDefinition.VersionDefinition.stack_name,
        stackVersion: versionDefinition.VersionDefinition.stack_version,
        versionDefinition: versionDefinition,
        stackInfo: stackInfo,
        dfd: dfd
      },
      success: 'getSupportedOSListSuccessCallback',
      error: 'getSupportedOSListErrorCallback'
    });
  },

  /**
   * onSuccess callback for getSupportedOSList.
   */
  getSupportedOSListSuccessCallback: function getSupportedOSListSuccessCallback(response, request, data) {
    var self = this;
    var stack_default = data.versionDefinition.VersionDefinition.stack_default;
    var existedOS = data.versionDefinition.operating_systems;
    var existedMap = {};
    existedOS.map(function (existedOS) {
      existedOS.isSelected = true;
      existedMap[existedOS.OperatingSystems.os_type] = existedOS;
    });
    response.operating_systems.forEach(function (supportedOS) {
      if (!existedMap[supportedOS.OperatingSystems.os_type]) {
        supportedOS.isSelected = false;
        existedOS.push(supportedOS);
      } else {
        existedMap[supportedOS.OperatingSystems.os_type].repositories.forEach(function (repo) {
          supportedOS.repositories.forEach(function (supportedRepo) {
            if (supportedRepo.Repositories.repo_id == repo.Repositories.repo_id) {
              repo.Repositories.components = supportedRepo.Repositories.components;
              repo.Repositories.distribution = supportedRepo.Repositories.distribution;
            }
          });
        });
      }
    });

    App.stackMapper.map(data.versionDefinition);

    if (!this.decrementProperty('loadStacksRequestsCounter')) {
      if (data.stackInfo.dfd) {
        data.stackInfo.dfd.resolve(data.stackInfo.response);
      } else {
        var versionData = this.getSelectedRepoVersionData();
        if (versionData) {
          this.postVersionDefinitionFile(versionData.isXMLdata, versionData.data).done(function (versionInfo) {
            self.mergeChanges(data.stackInfo.repos, data.stackInfo.oses, data.stackInfo.stacks);
            App.Stack.find().setEach('isSelected', false);
            var stackId = Em.get(versionData, 'data.VersionDefinition.available') || versionInfo.stackNameVersion + "-" + versionInfo.actualVersion;
            App.Stack.find().findProperty('id', stackId).set('isSelected', true);
            self.setSelected(data.stackInfo.isStacksExistInDb, data.dfd);
          }).fail(function () {
            self.setSelected(data.stackInfo.isStacksExistInDb, data.dfd);
          });
        } else {
          this.setSelected(data.stackInfo.isStacksExistInDb, data.dfd);
        }
      }
    }
  },

  /**
   * onError callback for getSupportedOSList
   */
  getSupportedOSListErrorCallback: function getSupportedOSListErrorCallback(request, ajaxOptions, error, data, params) {
    var header = Em.I18n.t('installer.step1.useLocalRepo.getSurpottedOs.error.title');
    var body = "";
    if (request && request.responseText) {
      try {
        var json = $.parseJSON(request.responseText);
        body = json.message;
      } catch (err) {}
    }
    App.showAlertPopup(header, body);
  },

  updateRepoOSInfo: function updateRepoOSInfo(repoToUpdate, repo) {
    var deferred = $.Deferred();
    var repoVersion = this.prepareRepoForSaving(repo);
    App.ajax.send({
      name: 'admin.stack_versions.edit.repo',
      sender: this,
      data: {
        stackName: repoToUpdate.stackName,
        stackVersion: repoToUpdate.stackVersion,
        repoVersionId: repoToUpdate.id,
        repoVersion: repoVersion
      }
    }).success(function () {
      deferred.resolve([]);
    }).error(function () {
      deferred.resolve([]);
    });
    return deferred.promise();
  },

  /**
   * transform repo data into json for
   * saving changes to repository version
   * @param {Em.Object} repo
   * @returns {{operating_systems: Array}}
   */
  prepareRepoForSaving: function prepareRepoForSaving(repo) {
    var repoVersion = { "operating_systems": [] };
    var ambariManagedRepositories = !repo.get('useRedhatSatellite');
    var k = 0;
    repo.get('operatingSystems').forEach(function (os) {
      if (os.get('isSelected')) {
        repoVersion.operating_systems.push({
          "OperatingSystems": {
            "os_type": os.get("osType"),
            "ambari_managed_repositories": ambariManagedRepositories
          },
          "repositories": []
        });
        os.get('repositories').forEach(function (repository) {
          if (!(repository.get('isGPL') && _.isEmpty(repository.get('baseUrl')))) {
            repoVersion.operating_systems[k].repositories.push({
              "Repositories": {
                "base_url": repository.get('baseUrl'),
                "repo_id": repository.get('repoId'),
                "repo_name": repository.get('repoName'),
                "components": repository.get('components'),
                "tags": repository.get('tags'),
                "distribution": repository.get('distribution'),
                "applicable_services": repository.get('applicable_services')
              }
            });
          }
        });
        k++;
      }
    });
    return repoVersion;
  },

  /**
   * Check validation of the customized local urls
   */
  checkRepoURL: function checkRepoURL(wizardStep1Controller) {
    var selectedStack = this.get('content.stacks').findProperty('isSelected', true);
    selectedStack.set('reload', true);
    var nameVersionCombo = selectedStack.get('stackNameVersion');
    var stackName = nameVersionCombo.split('-')[0];
    var stackVersion = nameVersionCombo.split('-')[1];
    var dfd = $.Deferred();
    if (selectedStack && selectedStack.get('operatingSystems')) {
      this.set('validationCnt', selectedStack.get('operatingSystems').filterProperty('isSelected').filterProperty('isEmpty', false).map(function (os) {
        return os.get('repositories').filterProperty('showRepo', true).length;
      }).reduce(Em.sum, 0));
      var verifyBaseUrl = !wizardStep1Controller.get('skipValidationChecked') && !wizardStep1Controller.get('selectedStack.useRedhatSatellite');
      if (!verifyBaseUrl) {
        dfd.resolve();
      }
      //for redhat satellite/spacewalk the os urls will be empty
      var useRedhatSatellite = wizardStep1Controller.get('selectedStack.useRedhatSatellite');
      selectedStack.get('operatingSystems').forEach(function (os) {
        if (os.get('isSelected') && (useRedhatSatellite || !os.get('isEmpty'))) {
          os.get('repositories').forEach(function (repo) {
            if (repo.get('showRepo')) {
              repo.setProperties({
                errorTitle: '',
                errorContent: '',
                validation: 'INPROGRESS'
              });
              this.set('content.isCheckInProgress', true);
              App.ajax.send({
                name: 'wizard.advanced_repositories.valid_url',
                sender: this,
                data: {
                  stackName: stackName,
                  stackVersion: stackVersion,
                  repoId: repo.get('repoId'),
                  osType: os.get('osType'),
                  osId: os.get('id'),
                  dfd: dfd,
                  data: {
                    'Repositories': {
                      'base_url': repo.get('baseUrl'),
                      'repo_name': repo.get('repoName'),
                      "verify_base_url": verifyBaseUrl
                    }
                  }
                },
                success: 'checkRepoURLSuccessCallback',
                error: 'checkRepoURLErrorCallback'
              });
            }
          }, this);
        } else if (os.get('isSelected') && os.get('isEmpty')) {
          os.set('isSelected', false);
        }
      }, this);
    }
    return dfd.promise();
  },
  /**
   * onSuccess callback for check Repo URL.
   */
  checkRepoURLSuccessCallback: function checkRepoURLSuccessCallback(response, request, data) {
    var selectedStack = this.get('content.stacks').findProperty('isSelected');
    if (selectedStack && selectedStack.get('operatingSystems')) {
      var os = selectedStack.get('operatingSystems').findProperty('id', data.osId);
      var repo = os.get('repositories').findProperty('repoId', data.repoId);
      if (repo) {
        repo.set('validation', 'OK');
      }
    }
    this.set('validationCnt', this.get('validationCnt') - 1);
    if (!this.get('validationCnt')) {
      this.set('content.isCheckInProgress', false);
      data.dfd.resolve();
    }
  },

  /**
   * onError callback for check Repo URL.
   */
  checkRepoURLErrorCallback: function checkRepoURLErrorCallback(request, ajaxOptions, error, data, params) {
    var selectedStack = this.get('content.stacks').findProperty('isSelected', true);
    if (selectedStack && selectedStack.get('operatingSystems')) {
      var os = selectedStack.get('operatingSystems').findProperty('id', params.osId);
      var repo = os.get('repositories').findProperty('repoId', params.repoId);
      if (repo) {
        var title = Ember.Handlebars.Utils.escapeExpression(request.status + ":" + request.statusText);
        var content = Ember.Handlebars.Utils.escapeExpression($.parseJSON(request.responseText) ? $.parseJSON(request.responseText).message : "");
        repo.setProperties({
          validation: 'INVALID',
          errorTitle: title,
          errorContent: content
        });
      }
    }
    this.set('content.isCheckInProgress', false);
    params.dfd.reject();
  },

  loadMap: {
    '0': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '1': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();

        this.loadStacks().done(function (stacksLoaded) {
          App.router.get('clusterController').loadAmbariProperties().always(function () {
            dfd.resolve(stacksLoaded);
          });
        });

        return dfd.promise();
      }
    }, {
      type: 'async',
      callback: function callback(stacksLoaded) {
        var dfd = $.Deferred();

        if (!stacksLoaded) {
          this.loadStacksVersions().done(function () {
            dfd.resolve(true);
          });
        } else {
          dfd.resolve(stacksLoaded);
        }

        return dfd.promise();
      }
    }],
    '2': [{
      type: 'sync',
      callback: function callback() {
        this.load('installOptions');
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadConfirmedHosts();
      }
    }],
    '4': [{
      type: 'async',
      callback: function callback() {
        return this.loadServices();
      }
    }],
    '5': [{
      type: 'sync',
      callback: function callback() {
        this.setSkipSlavesStep(App.StackService.find().filterProperty('isSelected'), 6);
        this.loadMasterComponentHosts();
        this.loadConfirmedHosts();
        this.loadComponentsFromConfigs();
        this.loadRecommendations();
      }
    }],
    '6': [{
      type: 'sync',
      callback: function callback() {
        this.loadSlaveComponentHosts();
        this.loadClients();
        this.loadComponentsFromConfigs();
        this.loadRecommendations();
      }
    }],
    '7': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        var self = this;
        this.loadServiceConfigGroups();
        this.loadCurrentHostGroups();
        this.loadRecommendationsConfigs();
        this.loadComponentsFromConfigs();
        this.loadConfigThemes().then(function () {
          self.loadServiceConfigProperties();
          dfd.resolve();
        });
        return dfd.promise();
      }
    }]
  },

  clearConfigActionComponents: function clearConfigActionComponents() {
    var masterComponentHosts = this.get('content.masterComponentHosts');
    var componentsAddedFromConfigAction = this.get('content.componentsFromConfigs');

    if (componentsAddedFromConfigAction && componentsAddedFromConfigAction.length) {
      componentsAddedFromConfigAction.forEach(function (_masterComponent) {
        masterComponentHosts = masterComponentHosts.rejectProperty('component', _masterComponent);
      });
    }
    this.set('content.masterComponentHosts', masterComponentHosts);
    this.setDBProperty('masterComponentHosts', masterComponentHosts);
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.setCurrentStep('0');
    this.clearStorageData();
    this.clearServiceConfigProperties();
    App.router.get('userSettingsController').postUserPref('show_bg', true);
    App.themesMapper.resetModels();
  },

  /**
   * Save cluster provisioning state to the server
   * @param state cluster provisioning state
   */
  setClusterProvisioningState: function setClusterProvisioningState(state) {
    return App.ajax.send({
      name: 'cluster.save_provisioning_state',
      sender: this,
      data: {
        state: state
      }
    });
  },

  setStepsEnable: function () {
    for (var i = 0; i <= this.totalSteps; i++) {
      this.get('isStepDisabled').findProperty('step', i).set('value', i > this.get('currentStep'));
    }
  }.observes('currentStep'),

  setLowerStepsDisable: function setLowerStepsDisable(stepNo) {
    for (var i = 0; i < stepNo; i++) {
      var step = this.get('isStepDisabled').findProperty('step', i);
      step.set('value', true);
    }
  },

  /**
   * Compare jdk versions used for ambari and selected stack.
   * Validation check will fire only for non-custom jdk configuration.
   *
   * @param {Function} successCallback
   * @param {Function} failCallback
   */
  validateJDKVersion: function validateJDKVersion(successCallback, failCallback) {
    var selectedStack = App.Stack.find().findProperty('isSelected', true),
        currentJDKVersion = App.router.get('clusterController.ambariProperties')['java.version'],

    // use min as max, or max as min version, in case when some of them missed
    minJDKVersion = selectedStack.get('minJdkVersion') || selectedStack.get('maxJdkVersion'),
        maxJDKVersion = selectedStack.get('maxJdkVersion') || selectedStack.get('minJdkVersion'),
        t = Em.I18n.t,
        fCallback = failCallback || function () {},
        sCallback = successCallback || function () {};

    // Skip jdk check if min and max required version not set in stack definition.
    if (!minJDKVersion && !maxJDKVersion) {
      sCallback();
      return;
    }

    if (currentJDKVersion) {
      if (stringUtils.compareVersions(currentJDKVersion, minJDKVersion) < 0 || stringUtils.compareVersions(maxJDKVersion, currentJDKVersion) < 0) {
        // checks and process only minor part for now
        var versionDistance = parseInt(maxJDKVersion.split('.')[1], 10) - parseInt(minJDKVersion.split('.')[1], 10);
        var versionsList = [minJDKVersion];
        for (var i = 1; i < versionDistance + 1; i++) {
          versionsList.push("" + minJDKVersion.split('.')[0] + '.' + (+minJDKVersion.split('.')[1] + i));
        }
        var versionsString = stringUtils.getFormattedStringFromArray(versionsList, t('or'));
        var popupBody = t('popup.jdkValidation.body').format(selectedStack.get('stackName') + ' ' + selectedStack.get('stackVersion'), versionsString, currentJDKVersion);
        App.showConfirmationPopup(sCallback, popupBody, fCallback, t('popup.jdkValidation.header'), t('common.proceedAnyway'), 'danger');
        return;
      }
    }
    sCallback();
  }

});

});

require.register("controllers/login_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.LoginController = Em.Object.extend({

  name: 'loginController',

  loginName: '',
  password: '',

  errorMessage: '',

  isSubmitDisabled: false,

  submit: function submit(e) {
    this.set('errorMessage', '');
    this.set('isSubmitDisabled', true);
    App.get('router').login();
  },

  postLogin: function postLogin(isConnected, isAuthenticated, responseText) {
    var errorMessage = "";
    if (!isConnected) {
      this.set('errorMessage', responseText || Em.I18n.t('login.error.bad.connection'));
    } else if (!isAuthenticated) {
      if (responseText === "User is disabled") {
        errorMessage = Em.I18n.t('login.error.disabled');
      } else if (responseText === "Authentication required" || Em.isNone(responseText)) {
        errorMessage = Em.I18n.t('login.error.bad.credentials');
      } else {
        errorMessage = responseText;
      }
      this.set('errorMessage', errorMessage);
    }
    this.set('isSubmitDisabled', false);
  }

});

});

require.register("controllers/main", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('models/background_operation');

App.MainController = Em.Controller.extend({
  name: 'mainController',
  isUserActive: true,
  checkActivenessInterval: null,
  lastUserActiveTime: null,
  userTimeOut: 0,
  userTimeOutModal: null,

  updateTitle: function () {
    var name = App.router.get('clusterController.clusterName');
    if (App.router.get('clusterInstallCompleted')) {
      if (name && App.router.get('clusterController').get('isLoaded')) {
        name = name.length > 13 ? name.substr(0, 10) + "..." : name;
      } else {
        name = Em.I18n.t('common.loading');
      }
      $('title').text(Em.I18n.t('app.name.subtitle').format(name));
    }
  }.observes('App.router.clusterController.clusterName', 'App.router.clusterInstallCompleted', 'App.router.clusterController.isLoaded'),

  isClusterDataLoaded: Em.computed.alias('App.router.clusterController.isLoaded'),

  clusterDataLoadedPercent: Em.computed.alias('App.router.clusterController.clusterDataLoadedPercent'),
  /**
   * run all processes and cluster's data loading
   */
  initialize: function initialize() {
    // Since we use only defaultTransaction, we can stub <code>removeCleanRecords</code> method,
    // because it would remove from and add records to the same (default) transaction
    App.store.defaultTransaction.reopen({
      removeCleanRecords: Em.K
    });
    var startSubscription = App.router.get('updateController').startSubscriptions.bind(App.router.get('updateController'));
    App.StompClient.connect().done(startSubscription).fail(function (dfd) {
      dfd.always(startSubscription);
    });
    App.router.get('clusterController').loadClusterData();
  },

  dataLoading: function dataLoading() {
    var self = this;
    var dfd = $.Deferred();
    if (App.router.get('clusterController.isLoaded')) {
      dfd.resolve();
    } else {
      var interval = setInterval(function () {
        if (self.get('isClusterDataLoaded')) {
          dfd.resolve();
          clearInterval(interval);
        }
      }, 50);
    }
    return dfd.promise();
  },

  /**
   *
   * @param isLoaded {Boolean}
   * @param opts {Object}
   * {
   *   period {Number}
   * }
   * @return {*|{then}}
   */
  isLoading: function isLoading(isLoaded, opts) {
    var dfd = $.Deferred();
    var self = this;
    opts = opts || {};
    var period = opts.period || 20;
    if (this.get(isLoaded)) {
      dfd.resolve();
    } else {
      var interval = setInterval(function () {
        if (self.get(isLoaded)) {
          dfd.resolve();
          clearInterval(interval);
        }
      }, period);
    }
    return dfd.promise();
  },

  startPolling: function () {
    if (App.router.get('applicationController.isExistingClusterDataLoaded')) {
      App.router.get('updateController').set('isWorking', true);
    }
  }.observes('App.router.applicationController.isExistingClusterDataLoaded'),

  stopPolling: function stopPolling() {
    App.router.get('updateController').set('isWorking', false);
  },

  reloadTimeOut: null,

  pageReload: function () {

    clearTimeout(this.get("reloadTimeOut"));

    this.set('reloadTimeOut', setTimeout(function () {
      if (App.clusterStatus.get('isInstalled')) {
        location.reload();
      }
    }, App.pageReloadTime));
  }.observes("App.router.location.lastSetURL", "App.clusterStatus.isInstalled"),

  setAmbariServerVersion: function setAmbariServerVersion(data) {
    var clientVersion = App.get('version');
    var serverVersion = data.RootServiceComponents.component_version.toString();
    this.set('ambariServerVersion', serverVersion);
    if (clientVersion) {
      this.set('versionConflictAlertBody', Em.I18n.t('app.versionMismatchAlert.body').format(serverVersion, clientVersion));
      this.set('isServerClientVersionMismatch', clientVersion !== serverVersion);
    } else {
      this.set('isServerClientVersionMismatch', false);
    }
    App.set('isManagedMySQLForHiveEnabled', App.config.isManagedMySQLForHiveAllowed(data.RootServiceComponents.properties['server.os_family']));
  },

  monitorInactivity: function monitorInactivity() {
    var timeout = Number(App.router.get('clusterController.ambariProperties')['user.inactivity.timeout.default']);
    var readonly_timeout = Number(App.router.get('clusterController.ambariProperties')['user.inactivity.timeout.role.readonly.default']);
    var isAdmin = App.get('isAdmin');
    if (isAdmin && timeout > 0) {
      this.set('userTimeOut', timeout * 1000);
    } else if (!isAdmin && readonly_timeout > 0) {
      this.set('userTimeOut', readonly_timeout * 1000);
    }
    if (this.get('userTimeOut') > 0) {
      this.startMonitorInactivity();
    }
  },

  startMonitorInactivity: function startMonitorInactivity() {
    this.set('isUserActive', true);
    this.set('lastUserActiveTime', Date.now());

    this.rebindActivityEventMonitors();
    if (!this.get('checkActivenessInterval')) {
      this.set('checkActivenessInterval', window.setInterval(this.checkActiveness, 1000));
    }
  },

  /* this will be triggerred by user driven events: 'mousemove', 'keypress' and 'click' */
  keepActive: function keepActive() {
    var scope = App.router.get('mainController');
    if (scope.get('isUserActive')) {
      scope.set('lastUserActiveTime', Date.now());
    }
  },

  checkActiveness: function checkActiveness() {
    var scope = App.router.get('mainController');
    if (!scope.isOnWizard()) {
      var remainTime = scope.get('userTimeOut') - (Date.now() - scope.get('lastUserActiveTime'));
      if (remainTime < 0) {
        scope.set('isUserActive', false);
        scope.unbindActivityEventMonitors();
        clearInterval(scope.get('checkActivenessInterval'));
        App.router.logOff({});
      } else if (remainTime < App.inactivityRemainTime * 1000 && !scope.userTimeOutModal) {
        // show alert 60 seconds before logging user out
        scope.userTimeOutModal = App.ModalPopup.show({
          primary: Em.I18n.t('common.timeout.warning.popup.primary'),
          secondary: Em.I18n.t('common.timeout.warning.popup.secondary'),
          third: false,
          header: Em.I18n.t('common.timeout.warning.popup.header'),
          showCloseButton: false,
          bodyClass: Ember.View.extend({
            template: Ember.Handlebars.compile('<p>{{view.beforeMsg}}<b>{{view.remainTime}}</b>{{view.afterMsg}}</p>'),
            beforeMsg: Em.I18n.t('common.timeout.warning.popup.body.before'),
            afterMsg: Em.I18n.t('common.timeout.warning.popup.body.after'),
            remainTime: App.inactivityRemainTime,
            didInsertElement: function didInsertElement() {
              var self = this;
              setInterval(function () {
                self.countDown();
              }, 1000);
            },
            countDown: function countDown() {
              if (this.get('remainTime') > 0) {
                this.set('remainTime', this.get('remainTime') - 1);
              }
              if (this.get('remainTime') == 0) {
                App.router.logOff({});
              }
            }
          }),
          onPrimary: function onPrimary() {
            scope.keepActive();
            scope.userTimeOutModal.hide();
            delete scope.userTimeOutModal;
          },
          onSecondary: function onSecondary() {
            scope.userTimeOutModal.hide();
            delete scope.userTimeOutModal;
            App.router.logOff({});
          }
        });
      }
    }
  },

  rebindActivityEventMonitors: function rebindActivityEventMonitors() {
    this.unbindActivityEventMonitors();
    this.bindActivityEventMonitors();
  },

  isOnWizard: function isOnWizard() {
    var isWizard = window.location.href.indexOf('/step') != -1;
    var isUpgrade = window.location.href.indexOf('/stack/upgrade') != -1;
    return isWizard || isUpgrade;
  },

  bindActivityEventMonitors: function bindActivityEventMonitors() {
    $(window).bind('mousemove', this.keepActive);
    $(window).bind('keypress', this.keepActive);
    $(window).bind('click', this.keepActive);
    // iframes need to be monitored as well
    var iframes = $('iframe');
    if (iframes.length > 0) {
      for (var i = 0; i < iframes.length; i++) {
        var iframe = iframes[i];
        $(iframe.contentWindow).bind('mousemove', this.keepActive);
        $(iframe.contentWindow).bind('keypress', this.keepActive);
        $(iframe.contentWindow).bind('click', this.keepActive);
      }
    }
  },

  unbindActivityEventMonitors: function unbindActivityEventMonitors() {
    $(window).unbind('mousemove', this.keepActive);
    $(window).unbind('keypress', this.keepActive);
    $(window).unbind('click', this.keepActive);
    // iframes need to be monitored as well
    var iframes = $('iframe');
    if (iframes.length > 0) {
      for (var i = 0; i < iframes.length; i++) {
        var iframe = iframes[i];
        $(iframe.contentWindow).unbind('mousemove', this.keepActive);
        $(iframe.contentWindow).unbind('keypress', this.keepActive);
        $(iframe.contentWindow).unbind('click', this.keepActive);
      }
    }
  }
});

});

require.register("controllers/main/admin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminController = Em.Controller.extend({
  name: 'mainAdminController',

  /**
   * @type {string}
   * @default null
   */
  category: null,

  /**
   * Check if access page available.
   * Turn on if YARN service is installed with Application Timeline Server component and TEZ installed too.
   *
   * @type {Boolean}
   */
  isAccessAvailable: function () {
    var dependencies = {
      services: ['YARN', 'TEZ']
    };
    var serviceNames = App.Service.find().mapProperty('serviceName').filter(function (serviceName) {
      return dependencies.services.contains(serviceName);
    });
    var isAppTimelineServerAvailable = App.HostComponent.find().someProperty('componentName', 'APP_TIMELINE_SERVER');
    return dependencies.services.length === serviceNames.length && isAppTimelineServerAvailable;
  }.property()
});

});

require.register("controllers/main/admin/federation/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.NameNodeFederationWizardStep1Controller = Em.Controller.extend({
  name: "nameNodeFederationWizardStep1Controller",

  existingNameServices: function () {
    var isMetricsLoaded = App.router.get('clusterController.isHostComponentMetricsLoaded');
    return isMetricsLoaded ? App.HDFSService.find().objectAt(0).get('masterComponentGroups').mapProperty('name') : [];
  }.property('App.router.clusterController.isHDFSNameSpacesLoaded'),

  existingNameServicesString: function () {
    return this.get('existingNameServices').join(', ');
  }.property('existingNameServices.length'),

  isNameServiceIdError: function () {
    var nameServiceId = this.get('content.nameServiceId');
    return !nameServiceId || this.get('existingNameServices').contains(nameServiceId) || !validator.isValidNameServiceId(nameServiceId);
  }.property('content.nameServiceId', 'existingNameServices.length'),

  next: function next() {
    if (!this.get('isNameServiceIdError')) {
      App.router.send('next');
    }
  }
});

});

require.register("controllers/main/admin/federation/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep2Controller = Em.Controller.extend(App.AssignMasterComponents, {

  name: "nameNodeFederationWizardStep2Controller",

  useServerValidation: false,

  mastersToShow: ['NAMENODE'],

  mastersToAdd: ['NAMENODE', 'NAMENODE'],

  showCurrentPrefix: ['NAMENODE'],

  showAdditionalPrefix: ['NAMENODE'],

  showInstalledMastersFirst: true

});

});

require.register("controllers/main/admin/federation/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep3Controller = Em.Controller.extend(App.BlueprintMixin, {
  name: "nameNodeFederationWizardStep3Controller",
  selectedService: null,
  stepConfigs: [],
  serverConfigData: {},
  once: false,
  isLoaded: false,
  isConfigsLoaded: false,
  versionLoaded: true,
  hideDependenciesInfoBar: true,

  /**
   * Map of sites and properties to delete
   * @type Object
   */
  configsToRemove: {
    'hdfs-site': ['dfs.namenode.shared.edits.dir', 'dfs.journalnode.edits.dir']
  },

  clearStep: function clearStep() {
    this.get('stepConfigs').clear();
    this.set('serverConfigData', {});
    this.set('isConfigsLoaded', false);
    this.set('isLoaded', false);
  },

  loadStep: function loadStep() {
    this.clearStep();
    this.loadConfigsTags();
  },

  loadConfigsTags: function loadConfigsTags() {
    return App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadConfigsTags'
    });
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var servicesModel = App.Service.find();
    var urlParams = '(type=hdfs-site&tag=' + data.Clusters.desired_configs['hdfs-site'].tag + ')';
    if (servicesModel.someProperty('serviceName', 'RANGER')) {
      urlParams += '|(type=core-site&tag=' + data.Clusters.desired_configs['core-site'].tag + ')' + '|(type=ranger-tagsync-site&tag=' + data.Clusters.desired_configs['ranger-tagsync-site'].tag + ')' + '|(type=ranger-hdfs-security&tag=' + data.Clusters.desired_configs['ranger-hdfs-security'].tag + ')';
    }
    if (servicesModel.someProperty('serviceName', 'ACCUMULO')) {
      urlParams += '|(type=accumulo-site&tag=' + data.Clusters.desired_configs['accumulo-site'].tag + ')';
    }
    App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: urlParams
      },
      success: 'onLoadConfigs'
    });
  },

  onLoadConfigs: function onLoadConfigs(data) {
    this.set('serverConfigData', data);
    this.set('isConfigsLoaded', true);
  },

  onLoad: function () {
    if (this.get('isConfigsLoaded') && App.router.get('clusterController.isHDFSNameSpacesLoaded')) {
      var federationConfig = $.extend(true, {}, require('data/configs/wizards/federation_properties').federationConfig);
      if (App.get('hasNameNodeFederation')) {
        federationConfig.configs = federationConfig.configs.rejectProperty('firstRun');
      }
      federationConfig.configs = this.tweakServiceConfigs(federationConfig.configs);
      this.removeConfigs(this.get('configsToRemove'), this.get('serverConfigData'));
      this.renderServiceConfigs(federationConfig);
      this.set('isLoaded', true);
    }
  }.observes('isConfigsLoaded', 'App.router.clusterController.isHDFSNameSpacesLoaded'),

  prepareDependencies: function prepareDependencies() {
    var ret = {};
    var configsFromServer = this.get('serverConfigData.items');
    var journalNodes = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE');
    var nameNodes = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE');
    var hdfsSiteConfigs = configsFromServer.findProperty('type', 'hdfs-site').properties;
    var nameServices = App.HDFSService.find().objectAt(0).get('masterComponentGroups').mapProperty('name');
    ret.nameServicesList = nameServices.join(',');
    ret.nameservice1 = nameServices.find(function (ns) {
      return hdfsSiteConfigs['dfs.namenode.rpc-address.' + ns + '.nn1'];
    });
    ret.newNameservice = this.get('content.nameServiceId');
    ret.namenode1 = hdfsSiteConfigs['dfs.namenode.rpc-address.' + ret.nameservice1 + '.nn1'].split(':')[0];
    ret.namenode2 = hdfsSiteConfigs['dfs.namenode.rpc-address.' + ret.nameservice1 + '.nn2'].split(':')[0];
    ret.newNameNode1Index = 'nn' + (nameNodes.length - 1);
    ret.newNameNode2Index = 'nn' + nameNodes.length;
    ret.newNameNode1 = nameNodes.filterProperty('isInstalled', false).mapProperty('hostName')[0];
    ret.newNameNode2 = nameNodes.filterProperty('isInstalled', false).mapProperty('hostName')[1];
    ret.journalnodes = journalNodes.map(function (c) {
      return c.get('hostName') + ':8485';
    }).join(';');
    ret.clustername = App.get('clusterName');

    var dfsHttpA = hdfsSiteConfigs['dfs.namenode.http-address'];
    ret.nnHttpPort = dfsHttpA ? dfsHttpA.split(':')[1] : 50070;

    var dfsHttpsA = hdfsSiteConfigs['dfs.namenode.https-address'];
    ret.nnHttpsPort = dfsHttpsA ? dfsHttpsA.split(':')[1] : 50470;

    var dfsRpcA = hdfsSiteConfigs['dfs.namenode.rpc-address'];
    ret.nnRpcPort = dfsRpcA ? dfsRpcA.split(':')[1] : 8020;

    ret.journalnode_edits_dir = hdfsSiteConfigs['dfs.journalnode.edits.dir'];

    return ret;
  },

  tweakServiceConfigs: function tweakServiceConfigs(configs) {
    var servicesModel = App.Service.find();
    var dependencies = this.prepareDependencies();
    var nameServices = App.HDFSService.find().objectAt(0).get('masterComponentGroups').mapProperty('name');
    nameServices.push(dependencies.newNameservice);
    var result = [];
    var configsToRemove = [];
    var hdfsSiteConfigs = this.get('serverConfigData').items.findProperty('type', 'hdfs-site').properties;
    var wizardController = App.router.get(this.get('content.controllerName'));

    if (!hdfsSiteConfigs['dfs.namenode.servicerpc-address.' + dependencies.nameservice1 + '.nn1'] && !hdfsSiteConfigs['dfs.namenode.servicerpc-address.' + dependencies.nameservice1 + '.nn2']) {
      configsToRemove = configsToRemove.concat(['dfs.namenode.servicerpc-address.{{nameservice1}}.nn1', 'dfs.namenode.servicerpc-address.{{nameservice1}}.nn2', 'dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode1Index}}', 'dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode2Index}}']);
    }

    if (servicesModel.someProperty('serviceName', 'RANGER')) {
      var hdfsRangerConfigs = this.get('serverConfigData').items.findProperty('type', 'ranger-hdfs-security').properties;
      var reponamePrefix = hdfsRangerConfigs['ranger.plugin.hdfs.service.name'] === '{{repo_name}}' ? dependencies.clustername + '_hadoop_' : hdfsRangerConfigs['ranger.plugin.hdfs.service.name'] + '_';
      var coreSiteConfigs = this.get('serverConfigData').items.findProperty('type', 'core-site').properties;
      var defaultFSNS = coreSiteConfigs['fs.defaultFS'].split('hdfs://')[1];

      nameServices.forEach(function (nameService) {
        configs.push(this.createRangerServiceProperty(nameService, reponamePrefix, "ranger.tagsync.atlas.hdfs.instance." + App.get('clusterName') + ".nameservice." + nameService + ".ranger.service"));
        configs.push(this.createRangerServiceProperty(defaultFSNS, reponamePrefix, "ranger.tagsync.atlas.hdfs.instance." + App.get('clusterName') + ".ranger.service"));
      }, this);
    }

    if (servicesModel.someProperty('serviceName', 'ACCUMULO')) {
      var hdfsNameSpacesModel = App.HDFSService.find().objectAt(0).get('masterComponentGroups');
      var newNameSpace = this.get('content.nameServiceId');
      var volumesValue = nameServices.map(function (ns) {
        return 'hdfs://' + ns + '/apps/accumulo/data';
      }).join();
      var replacementsValue = nameServices.map(function (ns) {
        var hostName;
        if (ns === newNameSpace) {
          var hostNames = this.get('content.masterComponentHosts').filter(function (hc) {
            return hc.component === 'NAMENODE' && !hc.isInstalled;
          }).mapProperty('hostName');
          hostName = hostNames[0];
        } else {
          var nameSpaceObject = hdfsNameSpacesModel.findProperty('name', ns);
          hostName = nameSpaceObject && nameSpaceObject.hosts[0];
        }
        return 'hdfs://' + hostName + ':8020/apps/accumulo/data hdfs://' + ns + '/apps/accumulo/data';
      }, this).join();
      configs.push({
        name: 'instance.volumes',
        displayName: 'instance.volumes',
        isReconfigurable: false,
        value: volumesValue,
        recommendedValue: volumesValue,
        category: 'ACCUMULO',
        filename: 'accumulo-site',
        serviceName: 'MISC'
      }, {
        name: 'instance.volumes.replacements',
        displayName: 'instance.volumes.replacements',
        isReconfigurable: false,
        value: replacementsValue,
        recommendedValue: replacementsValue,
        category: 'ACCUMULO',
        filename: 'accumulo-site',
        serviceName: 'MISC'
      });
    }

    configs.forEach(function (config) {
      if (!configsToRemove.contains(config.name)) {
        config.isOverridable = false;
        config.name = wizardController.replaceDependencies(config.name, dependencies);
        config.displayName = wizardController.replaceDependencies(config.displayName, dependencies);
        config.value = wizardController.replaceDependencies(config.value, dependencies);
        config.recommendedValue = wizardController.replaceDependencies(config.recommendedValue, dependencies);
        result.push(config);
      }
    }, this);

    return result;
  },

  createRangerServiceProperty: function createRangerServiceProperty(nameservice, reponamePrefix, propertyName) {
    return {
      "name": propertyName,
      "displayName": propertyName,
      "isReconfigurable": false,
      "recommendedValue": reponamePrefix + nameservice,
      "value": reponamePrefix + nameservice,
      "category": "RANGER",
      "filename": "ranger-tagsync-site",
      "serviceName": 'MISC'
    };
  },

  removeConfigs: function removeConfigs(configsToRemove, configs) {
    Em.keys(configsToRemove).forEach(function (site) {
      var siteConfigs = configs.items.findProperty('type', site);
      if (siteConfigs) {
        configsToRemove[site].forEach(function (property) {
          delete siteConfigs.properties[property];
        });
      }
    });
    return configs;
  },

  renderServiceConfigs: function renderServiceConfigs(_serviceConfig) {
    var serviceConfig = App.ServiceConfig.create({
      serviceName: _serviceConfig.serviceName,
      displayName: _serviceConfig.displayName,
      configCategories: [],
      showConfig: true,
      configs: []
    });

    _serviceConfig.configCategories.forEach(function (_configCategory) {
      if (App.Service.find().someProperty('serviceName', _configCategory.name)) {
        serviceConfig.configCategories.pushObject(_configCategory);
      }
    }, this);

    this.loadComponentConfigs(_serviceConfig, serviceConfig);

    this.get('stepConfigs').pushObject(serviceConfig);
    this.set('selectedService', this.get('stepConfigs').objectAt(0));
    this.set('once', true);
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  loadComponentConfigs: function loadComponentConfigs(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  },

  isNextDisabled: function () {
    return !this.get('isLoaded') || this.get('isLoaded') && this.get('selectedService.configs').someProperty('isValid', false);
  }.property('selectedService.configs.@each.isValid', 'isLoaded')
});

});

require.register("controllers/main/admin/federation/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep4Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "nameNodeFederationWizardStep4Controller",

  commands: ['stopRequiredServices', 'reconfigureServices', 'installNameNode', 'installZKFC', 'startJournalNodes', 'startInfraSolr', 'startRangerAdmin', 'startRangerUsersync', 'startNameNodes', 'startZKFCs', 'formatNameNode', 'formatZKFC', 'startZKFC', 'startNameNode', 'bootstrapNameNode', 'startZKFC2', 'startNameNode2', 'restartAllServices'],

  tasksMessagesPrefix: 'admin.nameNodeFederation.wizard.step',

  initializeTasks: function initializeTasks() {
    this._super();
    this.removeUnneededTasks();
  },

  removeUnneededTasks: function removeUnneededTasks() {
    var installedServices = App.Service.find().mapProperty('serviceName');
    if (!installedServices.contains('RANGER')) {
      this.removeTasks(['startInfraSolr', 'startRangerAdmin', 'startRangerUsersync']);
    }
    if (!installedServices.contains('AMBARI_INFRA_SOLR')) {
      this.removeTasks(['startInfraSolr']);
    }
  },

  newNameNodeHosts: function () {
    return this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').filterProperty('isInstalled', false).mapProperty('hostName');
  }.property('content.masterComponentHosts.@each.hostName'),

  stopRequiredServices: function stopRequiredServices() {
    this.stopServices(["ZOOKEEPER"]);
  },

  reconfigureServices: function reconfigureServices() {
    var servicesModel = App.Service.find();
    var configs = [];
    var data = this.get('content.serviceConfigProperties');
    var note = Em.I18n.t('admin.nameNodeFederation.wizard,step4.save.configuration.note');
    configs.push({
      Clusters: {
        desired_config: this.reconfigureSites(['hdfs-site'], data, note)
      }
    });
    if (servicesModel.someProperty('serviceName', 'RANGER')) {
      configs.push({
        Clusters: {
          desired_config: this.reconfigureSites(['ranger-tagsync-site'], data, note)
        }
      });
    }
    if (servicesModel.someProperty('serviceName', 'ACCUMULO')) {
      configs.push({
        Clusters: {
          desired_config: this.reconfigureSites(['accumulo-site'], data, note)
        }
      });
    }
    return App.ajax.send({
      name: 'common.service.multiConfigurations',
      sender: this,
      data: {
        configs: configs
      },
      error: 'onTaskError',
      success: 'installHDFSClients'
    });
  },

  installHDFSClients: function installHDFSClients() {
    var nnHostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    var jnHostNames = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
    var hostNames = nnHostNames.concat(jnHostNames).uniq();
    this.createInstallComponentTask('HDFS_CLIENT', hostNames, 'HDFS');
  },

  installNameNode: function installNameNode() {
    this.createInstallComponentTask('NAMENODE', this.get('newNameNodeHosts'), "HDFS");
  },

  installZKFC: function installZKFC() {
    this.createInstallComponentTask('ZKFC', this.get('newNameNodeHosts'), "HDFS");
  },

  startJournalNodes: function startJournalNodes() {
    var hostNames = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
    this.updateComponent('JOURNALNODE', hostNames, "HDFS", "Start");
  },

  startNameNodes: function startNameNodes() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').filterProperty('isInstalled').mapProperty('hostName');
    this.updateComponent('NAMENODE', hostNames, "HDFS", "Start");
  },

  startZKFCs: function startZKFCs() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').filterProperty('isInstalled').mapProperty('hostName');
    this.updateComponent('ZKFC', hostNames, "HDFS", "Start");
  },

  formatNameNode: function formatNameNode() {
    App.ajax.send({
      name: 'nameNode.federation.formatNameNode',
      sender: this,
      data: {
        host: this.get('newNameNodeHosts')[0]
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  formatZKFC: function formatZKFC() {
    App.ajax.send({
      name: 'nameNode.federation.formatZKFC',
      sender: this,
      data: {
        host: this.get('newNameNodeHosts')[0]
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  startZKFC: function startZKFC() {
    this.updateComponent('ZKFC', this.get('newNameNodeHosts')[0], "HDFS", "Start");
  },

  startInfraSolr: function startInfraSolr() {
    this.startServices(false, ['AMBARI_INFRA_SOLR'], true);
  },

  startRangerAdmin: function startRangerAdmin() {
    var hostNames = App.HostComponent.find().filterProperty('componentName', 'RANGER_ADMIN').mapProperty('hostName');
    this.updateComponent('RANGER_ADMIN', hostNames, "RANGER", "Start");
  },

  startRangerUsersync: function startRangerUsersync() {
    var hostNames = App.HostComponent.find().filterProperty('componentName', 'RANGER_USERSYNC').mapProperty('hostName');
    this.updateComponent('RANGER_USERSYNC', hostNames, "RANGER", "Start");
  },

  startNameNode: function startNameNode() {
    this.updateComponent('NAMENODE', this.get('newNameNodeHosts')[0], "HDFS", "Start");
  },

  bootstrapNameNode: function bootstrapNameNode() {
    App.ajax.send({
      name: 'nameNode.federation.bootstrapNameNode',
      sender: this,
      data: {
        host: this.get('newNameNodeHosts')[1]
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  startZKFC2: function startZKFC2() {
    this.updateComponent('ZKFC', this.get('newNameNodeHosts')[1], "HDFS", "Start");
  },

  startNameNode2: function startNameNode2() {
    this.updateComponent('NAMENODE', this.get('newNameNodeHosts')[1], "HDFS", "Start");
  },

  restartAllServices: function restartAllServices() {
    App.ajax.send({
      name: 'restart.custom.filter',
      sender: this,
      data: {
        filter: "HostRoles/component_name!=NAMENODE&HostRoles/component_name!=JOURNALNODE&HostRoles/component_name!=ZKFC&HostRoles/component_name!=RANGER_ADMIN&HostRoles/component_name!=RANGER_USERSYNC&HostRoles/cluster_name=" + App.get('clusterName'),
        context: "Restart Required Services"
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  }
});

});

require.register("controllers/main/admin/federation/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardController = App.WizardController.extend({

  name: 'nameNodeFederationWizardController',

  totalSteps: 4,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.nameNodeFederation.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'nameNodeFederationWizardController'
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var self = this,
            dfd = $.Deferred();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          self.loadConfirmedHosts();
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadNameServiceId();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadServiceConfigProperties();
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
      }
    }]
  },

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'nameNodeFederationWizardController',
      localdb: App.db.data
    });
  },

  saveNameServiceId: function saveNameServiceId(nameServiceId) {
    this.setDBProperty('nameServiceId', nameServiceId);
    this.set('content.nameServiceId', nameServiceId);
  },

  loadNameServiceId: function loadNameServiceId() {
    var nameServiceId = this.getDBProperty('nameServiceId');
    this.set('content.nameServiceId', nameServiceId);
  },

  saveServiceConfigProperties: function saveServiceConfigProperties(stepController) {
    var serviceConfigProperties = [];
    var data = stepController.get('serverConfigData');

    var _content = stepController.get('stepConfigs')[0];
    _content.get('configs').forEach(function (_configProperties) {
      var siteObj = data.items.findProperty('type', _configProperties.get('filename'));
      if (siteObj) {
        siteObj.properties[_configProperties.get('name')] = _configProperties.get('value');
      }
    }, this);
    this.setDBProperty('serviceConfigProperties', data);
    this.set('content.serviceConfigProperties', data);
  },

  /**
   * Load serviceConfigProperties to model
   */
  loadServiceConfigProperties: function loadServiceConfigProperties() {
    this.set('content.serviceConfigProperties', this.getDBProperty('serviceConfigProperties'));
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }
});

});

require.register("controllers/main/admin/highAvailability/hawq/activateStandby/step1_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardStep1Controller = Em.Controller.extend({
  name: "activateHawqStandbyWizardStep1Controller"
});

});

require.register("controllers/main/admin/highAvailability/hawq/activateStandby/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} hawqActivateStandbyConfigDependencies
 */

var App = require('app');
require('utils/configs/hawq_activate_standby_config_initializer');

App.ActivateHawqStandbyWizardStep2Controller = Em.Controller.extend({
  name: "activateHawqStandbyWizardStep2Controller",

  selectedService: null,

  versionLoaded: true,

  hideDependenciesInfoBar: true,

  isLoaded: false,

  isSubmitDisabled: Em.computed.not('isLoaded'),

  loadStep: function loadStep() {
    this.renderConfigs();
  },

  /**
   * Render configs to show them in <code>App.ServiceConfigView</code>
   */
  renderConfigs: function renderConfigs() {
    newHawqMaster = App.HostComponent.find().findProperty('componentName', 'HAWQSTANDBY').get('hostName');

    var configs = require('data/configs/wizards/hawq_activate_standby_properties').haConfig;

    var serviceConfig = App.ServiceConfig.create({
      serviceName: configs.serviceName,
      displayName: configs.displayName,
      configCategories: [],
      showConfig: true,
      configs: []
    });

    configs.configCategories.forEach(function (configCategory) {
      if (App.Service.find().someProperty('serviceName', configCategory.name)) {
        serviceConfig.configCategories.pushObject(configCategory);
      }
    }, this);

    this.renderConfigProperties(configs, serviceConfig);
    this.setDynamicConfigValues(serviceConfig);
    this.setProperties({
      selectedService: serviceConfig,
      isLoaded: true
    });
  },

  setDynamicConfigValues: function setDynamicConfigValues(configs) {
    var topologyLocalDB = this.get('content').getProperties(['masterComponentHosts']);
    configs.configs.forEach(function (config) {
      App.HawqActivateStandbyConfigInitializer.initialValue(config, topologyLocalDB);
    });
    App.HawqActivateStandbyConfigInitializer.cleanup();
    return configs;
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  renderConfigProperties: function renderConfigProperties(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  },

  submit: function submit() {
    if (!this.get('isSubmitDisabled')) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        App.router.send("next");
      });
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/activateStandby/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/main/admin/serviceAccounts_controller');

App.ActivateHawqStandbyWizardStep3Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "activateHawqStandbyWizardStep3Controller",

  clusterDeployState: 'ACTIVATE_HAWQ_STANDBY',

  hawqActivateStandbyCustomCommand: "ACTIVATE_HAWQ_STANDBY",

  hawqServiceName: "HAWQ",

  hawqStandbyComponentName: "HAWQSTANDBY",

  hawqMasterComponentName: "HAWQMASTER",

  commands: ['activateStandby', 'stopRequiredServices', 'reconfigureHAWQ', 'installHawqMaster', 'deleteOldHawqMaster', 'deleteHawqStandby', 'startRequiredServices'],

  tasksMessagesPrefix: 'admin.activateHawqStandby.wizard.step',

  activateStandby: function activateStandby() {
    App.ajax.send({
      name: 'service.item.executeCustomCommand',
      sender: this,
      data: {
        command: this.hawqActivateStandbyCustomCommand,
        context: Em.I18n.t('admin.activateHawqStandby.wizard.step3.activateHawqStandbyCommand.context'),
        hosts: this.get('content.hawqHosts.hawqStandby'),
        serviceName: this.hawqServiceName,
        componentName: this.hawqStandbyComponentName
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  stopRequiredServices: function stopRequiredServices() {
    this.stopServices([this.hawqServiceName], true);
  },

  reconfigureHAWQ: function reconfigureHAWQ() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadHawqConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadHawqConfigsTags: function onLoadHawqConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=hawq-site&tag=' + data.Clusters.desired_configs['hawq-site'].tag + ')',
        type: 'hawq-site'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data, opt, params) {
    delete data.items[0].properties['hawq_standby_address_host'];

    var propertiesToAdd = this.get('content.configs').filterProperty('filename', params.type);
    propertiesToAdd.forEach(function (property) {
      data.items[0].properties[property.name] = property.value;
    });

    var configData = this.reconfigureSites([params.type], data, Em.I18n.t('admin.activateHawqStandby.step4.save.configuration.note').format(App.format.role('HAWQSTANDBY', false)));

    App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },
  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  installHawqMaster: function installHawqMaster() {
    var hostName = this.get('content.hawqHosts.hawqStandby');
    this.createInstallComponentTask(this.hawqMasterComponentName, hostName, this.hawqServiceName);
  },

  deleteOldHawqMaster: function deleteOldHawqMaster() {
    var hostName = this.get('content.hawqHosts.hawqMaster');
    this.deleteComponent(this.hawqMasterComponentName, hostName);
  },

  deleteHawqStandby: function deleteHawqStandby() {
    var hostName = this.get('content.hawqHosts.hawqStandby');
    this.deleteComponent(this.hawqStandbyComponentName, hostName);
  },

  startRequiredServices: function startRequiredServices() {
    this.startServices(false, [this.hawqServiceName], true);
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/activateStandby/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardController = App.WizardController.extend({

  name: 'activateHawqStandbyWizardController',

  totalSteps: 3,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.activateHawqStandby.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'activateHawqStandbyWizardController'
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        this.loadHawqHosts();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
        this.loadConfigs();
      }
    }]
  },

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: this.name,
      localdb: App.db.data
    });
  },

  /**
   * Save hosts for Hawq Master and Hawq Standby<code>controller.content</code>
   * @param hawqHosts
   */
  saveHawqHosts: function saveHawqHosts(hawqHosts) {
    this.set('content.hawqHosts', hawqHosts);
    this.setDBProperty('hawqHosts', hawqHosts);
  },

  /**
   * Load hosts for additional and current ResourceManagers from local db to <code>controller.content</code>
   */
  loadHawqHosts: function loadHawqHosts() {
    var hawqHosts = this.getDBProperty('hawqHosts');
    this.set('content.hawqHosts', hawqHosts);
  },

  /**
   * Save configs to load and apply them on Configure Components step
   * @param configs
   */
  saveConfigs: function saveConfigs(configs) {
    this.set('content.configs', configs);
    this.setDBProperty('configs', configs);
  },

  /**
   * Load configs to apply them on Configure Components step
   */
  loadConfigs: function loadConfigs() {
    var configs = this.getDBProperty('configs');
    this.set('content.configs', configs);
  },

  /**
   * Remove all loaded data.
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }
});

});

require.register("controllers/main/admin/highAvailability/hawq/addStandby/step1_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep1Controller = Em.Controller.extend({
  name: "addHawqStandbyWizardStep1Controller"
});

});

require.register("controllers/main/admin/highAvailability/hawq/addStandby/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep2Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {

  name: "addHawqStandbyWizardStep2Controller",

  useServerValidation: true,

  mastersToShow: ['HAWQMASTER', 'HAWQSTANDBY'],

  mastersToAdd: ['HAWQSTANDBY'],

  showCurrentPrefix: ['HAWQMASTER'],

  showInstalledMastersFirst: true
});

});

require.register("controllers/main/admin/highAvailability/hawq/addStandby/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} hawqHaConfigDependencies
 */

var App = require('app');
require('utils/configs/hawq_ha_config_initializer');

App.AddHawqStandbyWizardStep3Controller = Em.Controller.extend({
  name: "addHawqStandbyWizardStep3Controller",

  // Used in service_config.hbs
  // selectedService
  // hideDependenciesInfoBar
  // versionLoaded

  selectedService: null,

  hawqProps: null,

  hideDependenciesInfoBar: true,

  versionLoaded: true,

  // Used in step3.hbs
  // isLoaded
  // isSubmitDisabled
  // hawqMasterDirectoryCleanUpMessage
  isLoaded: false,

  isSubmitDisabled: Em.computed.not('isLoaded'),

  loadStep: function loadStep() {
    this.renderConfigs();
  },

  content: Em.Object.create({
    controllerName: 'addHawqStandbyWizardStep3Controller'
  }),

  /**
   * Render configs to show them in <code>App.ServiceConfigView</code>
   */
  renderConfigs: function renderConfigs() {

    var configs = require('data/configs/wizards/hawq_ha_properties').haConfig;

    var serviceConfig = App.ServiceConfig.create({
      serviceName: configs.serviceName,
      displayName: configs.displayName,
      configCategories: [],
      showConfig: true,

      configs: []
    });

    configs.configCategories.forEach(function (configCategory) {
      if (App.Service.find().someProperty('serviceName', configCategory.name)) {
        serviceConfig.configCategories.pushObject(configCategory);
      }
    }, this);

    this.renderConfigProperties(configs, serviceConfig);
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'loadConfigTagsSuccessCallback',
      error: '',
      data: {
        serviceConfig: serviceConfig
      }
    });
  },

  loadConfigTagsSuccessCallback: function loadConfigTagsSuccessCallback(data, opt, params) {
    var urlParams = '(type=hawq-site&tag=' + data.Clusters.desired_configs['hawq-site'].tag + ')';
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: urlParams,
        serviceConfig: params.serviceConfig
      },
      success: 'loadConfigsSuccessCallback',
      error: 'loadConfigsSuccessCallback'
    });
  },

  loadConfigsSuccessCallback: function loadConfigsSuccessCallback(data, opt, params) {
    params = params.serviceConfig ? params.serviceConfig : {};
    this.setDynamicConfigValues(params, data);
    this.setProperties({
      selectedService: params,
      isLoaded: true,
      hawqProps: data
    });
  },

  setDynamicConfigValues: function setDynamicConfigValues(configs, data) {
    var topologyLocalDB = this.get('content').getProperties(['masterComponentHosts']);
    configs.configs.forEach(function (config) {
      App.HawqHaConfigInitializer.initialValue(config, topologyLocalDB);
    });
    App.HawqHaConfigInitializer.cleanup();
    return configs;
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  renderConfigProperties: function renderConfigProperties(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  },

  submit: function submit() {
    if (!this.get('isSubmitDisabled')) {
      dataDir = this.get('hawqProps').items[0].properties['hawq_master_directory'];
      hawqStandby = this.get('content.hawqHosts.newHawqStandby');
      App.showConfirmationPopup(function () {
        App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
          App.router.send("next");
        });
      }, Em.I18n.t('admin.addHawqStandby.wizard.step3.confirm.dataDir.body').format(dataDir, hawqStandby), null, Em.I18n.t('admin.addHawqStandby.wizard.step3.confirm.dataDir.title'), "Confirm");
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/addStandby/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/main/admin/serviceAccounts_controller');

App.AddHawqStandbyWizardStep4Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "addHawqStandbyWizardStep4Controller",

  clusterDeployState: 'ADD_HAWQ_STANDBY',

  commands: ['stopRequiredServices', 'installHawqStandbyMaster', 'reconfigureHAWQ', 'startRequiredServices'],

  tasksMessagesPrefix: 'admin.addHawqStandby.wizard.step',

  stopRequiredServices: function stopRequiredServices() {
    this.stopServices(['HAWQ'], true);
  },

  installHawqStandbyMaster: function installHawqStandbyMaster() {
    var hostName = this.get('content.hawqHosts.newHawqStandby');
    this.createInstallComponentTask('HAWQSTANDBY', hostName, "HAWQ");
  },

  reconfigureHAWQ: function reconfigureHAWQ() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadHawqConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadHawqConfigsTags: function onLoadHawqConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=hawq-site&tag=' + data.Clusters.desired_configs['hawq-site'].tag + ')',
        type: 'hawq-site'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data, opt, params) {
    var propertiesToAdd = this.get('content.configs').filterProperty('filename', params.type);
    propertiesToAdd.forEach(function (property) {
      data.items[0].properties[property.name] = property.value;
    });

    var configData = this.reconfigureSites([params.type], data, Em.I18n.t('admin.addHawqStandby.step4.save.configuration.note').format(App.format.role('HAWQSTANDBY', false)));

    App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },
  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  startRequiredServices: function startRequiredServices() {
    this.startServices(false, ["HAWQ"], true);
  }
});

});

require.register("controllers/main/admin/highAvailability/hawq/addStandby/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardController = App.WizardController.extend({

  name: 'addHawqStandbyWizardController',

  totalSteps: 4,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.addHawqStandby.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'addHawqStandbyWizardController'
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var self = this,
            dfd = $.Deferred();
        this.loadHawqHosts();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          self.loadConfirmedHosts();
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
        this.loadConfigs();
      }
    }]
  },

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'addHawqStandbyWizardController',
      localdb: App.db.data
    });
  },

  /**
   * Save hosts for hawq master and hawq standby to local db and <code>controller.content</code>
   * @param hawqHosts
   */
  saveHawqHosts: function saveHawqHosts(hawqHosts) {
    this.set('content.hawqHosts', hawqHosts);
    this.setDBProperty('hawqHosts', hawqHosts);
  },

  /**
   * Load hosts for hawq master and hawq standby from local db to <code>controller.content</code>
   */
  loadHawqHosts: function loadHawqHosts() {
    this.set('content.hawqHosts', this.getDBProperty('hawqHosts'));
  },

  /**
   * Save configs to load and apply them on Configure Components step
   * @param configs
   */
  saveConfigs: function saveConfigs(configs) {
    this.set('content.configs', configs);
    this.setDBProperty('configs', configs);
  },

  /**
   * Load configs to apply them on Configure Components step
   */
  loadConfigs: function loadConfigs() {
    this.set('content.configs', this.getDBProperty('configs'));
  },

  /**
   * Remove all loaded data.
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/removeStandby/step1_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardStep1Controller = Em.Controller.extend({
  name: "removeHawqStandbyWizardStep1Controller"
});

});

require.register("controllers/main/admin/highAvailability/hawq/removeStandby/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} hawqRemoveStandbyConfigDependencies
 */

var App = require('app');

App.RemoveHawqStandbyWizardStep2Controller = Em.Controller.extend({
  name: "removeHawqStandbyWizardStep2Controller",

  selectedService: null,

  versionLoaded: true,

  hideDependenciesInfoBar: true,

  isLoaded: true,

  isSubmitDisabled: Em.computed.not('isLoaded'),

  submit: function submit() {
    if (!this.get('isSubmitDisabled')) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        App.router.send("next");
      });
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/removeStandby/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/main/admin/serviceAccounts_controller');

App.RemoveHawqStandbyWizardStep3Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "removeHawqStandbyWizardStep3Controller",

  clusterDeployState: 'REMOVE_HAWQ_STANDBY',

  hawqRemoveStandbyCustomCommand: "REMOVE_HAWQ_STANDBY",

  hawqServiceName: "HAWQ",

  hawqMasterComponentName: "HAWQMASTER",

  hawqStandbyComponentName: "HAWQSTANDBY",

  commands: ['removeStandby', 'stopRequiredServices', 'reconfigureHAWQ', 'deleteHawqStandbyComponent', 'startRequiredServices'],

  tasksMessagesPrefix: 'admin.removeHawqStandby.wizard.step',

  removeStandby: function removeStandby() {
    App.ajax.send({
      name: 'service.item.executeCustomCommand',
      sender: this,
      data: {
        command: this.hawqRemoveStandbyCustomCommand,
        context: Em.I18n.t('admin.removeHawqStandby.wizard.step3.removeHawqStandbyCommand.context'),
        hosts: this.get('content.hawqHosts.hawqMaster'),
        serviceName: this.hawqServiceName,
        componentName: this.hawqMasterComponentName
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  stopRequiredServices: function stopRequiredServices() {
    this.stopServices([this.hawqServiceName], true);
  },

  reconfigureHAWQ: function reconfigureHAWQ() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadHawqConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadHawqConfigsTags: function onLoadHawqConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=hawq-site&tag=' + data.Clusters.desired_configs['hawq-site'].tag + ')',
        type: 'hawq-site'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data, opt, params) {
    delete data.items[0].properties['hawq_standby_address_host'];

    var configData = this.reconfigureSites([params.type], data, Em.I18n.t('admin.removeHawqStandby.wizard.step3.save.configuration.note').format(App.format.role('HAWQSTANDBY', false)));

    App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },
  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  deleteHawqStandbyComponent: function deleteHawqStandbyComponent() {
    var hostName = this.get('content.hawqHosts.hawqStandby');
    this.deleteComponent(this.hawqStandbyComponentName, hostName);
  },

  startRequiredServices: function startRequiredServices() {
    this.startServices(false, [this.hawqServiceName], true);
  }

});

});

require.register("controllers/main/admin/highAvailability/hawq/removeStandby/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardController = App.WizardController.extend({

  name: 'removeHawqStandbyWizardController',

  totalSteps: 3,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.removeHawqStandby.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'removeHawqStandbyWizardController'
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        this.loadHawqHosts();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
        this.loadConfigs();
      }
    }]
  },

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'removeHawqStandbyWizardController',
      localdb: App.db.data
    });
  },

  /**
   * Save hosts for Hawq Master and Hawq Standby to local db<code>controller.content</code>
   * @param hawqHosts
   */
  saveHawqHosts: function saveHawqHosts(hawqHosts) {
    this.set('content.hawqHosts', hawqHosts);
    this.setDBProperty('hawqHosts', hawqHosts);
  },

  /**
   * Load hosts for Hawq Master and Hawq Standby from local db to <code>controller.content</code>
   */
  loadHawqHosts: function loadHawqHosts() {
    var hawqHosts = this.getDBProperty('hawqHosts');
    this.set('content.hawqHosts', hawqHosts);
  },

  /**
   * Save configs to load and apply them on Configure Components step
   * @param configs
   */
  saveConfigs: function saveConfigs(configs) {
    this.set('content.configs', configs);
    this.setDBProperty('configs', configs);
  },

  /**
   * Load configs to apply them on Configure Components step
   */
  loadConfigs: function loadConfigs() {
    var configs = this.getDBProperty('configs');
    this.set('content.configs', configs);
  },

  /**
   * Remove all loaded data.
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/progress_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeProgressPageController = App.ManageJournalNodeWizardController.extend(App.wizardProgressPageControllerMixin, {
  name: 'manageJournalNodeProgressPageController',
  clusterDeployState: 'JOURNALNODE_MANAGEMENT',
  tasksMessagesPrefix: 'admin.manageJournalNode.wizard.step',
  isRollback: false,

  /**
   * Prepare object to send to the server to save configs
   * Split all configs by site names and note
   * @param siteNames Array
   * @param data Object
   * @param note String
   */
  reconfigureSites: function reconfigureSites(siteNames, data, note) {

    return siteNames.map(function (_siteName) {
      var config = data.items.findProperty('type', _siteName);
      var configToSave = {
        type: _siteName,
        properties: config && config.properties,
        service_config_version_note: note || ''
      };
      if (config && config.properties_attributes) {
        configToSave.properties_attributes = config.properties_attributes;
      }
      return configToSave;
    });
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/wizard/step5_controller');

App.ManageJournalNodeWizardStep1Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {

  name: "manageJournalNodeWizardStep1Controller",

  useServerValidation: false,

  mastersToShow: ['JOURNALNODE'],

  showInstalledMastersFirst: true,

  JOURNALNODES_COUNT_MINIMUM: 3, // TODO get this from stack

  nextButtonCheckTrigger: true,

  /**
   * On initial rendering, load equivalent number of existing JournalNodes to masterToShow
   * @param masterComponents
   */
  renderComponents: function renderComponents(masterComponents) {
    //check if we are restoring components assignment by checking existing of JOURNALNODE component in array
    var restoringComponents = masterComponents.someProperty('component_name', 'JOURNALNODE');
    masterComponents = restoringComponents ? masterComponents : masterComponents.concat(this.generateJournalNodeComponents());
    this._super(masterComponents);
    this.updateJournalNodeInfo();
    this.showHideJournalNodesAddRemoveControl();
    this.toggleProperty('nextButtonCheckTrigger');
  },

  /**
   * Create JOURNALNODE components to add them to masters array
   */
  generateJournalNodeComponents: function generateJournalNodeComponents() {
    var journalNodes = [];
    App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').forEach(function (jn) {
      var jnComponent = this.createComponentInstallationObject(Em.Object.create({
        serviceName: jn.get('service.serviceName'),
        componentName: jn.get('componentName')
      }), jn.get('hostName'));
      jnComponent.isInstalled = true;
      journalNodes.push(jnComponent);
    }, this);
    return journalNodes;
  },

  /**
   * Override of method from <code>App.AssignMasterComponents</code>
   * Added to satisfy the requirement that maximum number of new JournalNodes to be added
   * should be 1 less that total of the existing JournalNodes
   * @returns {number}
   */
  getMaxNumberOfMasters: function getMaxNumberOfMasters() {
    var defaultLimitation = this._super('JOURNALNODE'),
        installedJournalNodesCount = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').length;
    return Math.min(defaultLimitation, installedJournalNodesCount * 2 - 1);
  },

  /**
   * Enable/Disable show/hide operation for each JournalNode
   */
  showHideJournalNodesAddRemoveControl: function () {
    var masterComponents = this.get('selectedServicesMasters');
    var jns = masterComponents.filterProperty('component_name', 'JOURNALNODE');
    var maxNumMasters = this.getMaxNumberOfMasters('JOURNALNODE');
    var showRemoveControl = jns.get('length') > this.get('JOURNALNODES_COUNT_MINIMUM');
    var showAddControl = jns.get('length') < maxNumMasters;
    jns.forEach(function (item) {
      item.set('showAddControl', false);
      item.set('showRemoveControl', showRemoveControl);
    });
    jns.set('lastObject.showAddControl', showAddControl);
  }.observes('hostNameCheckTrigger'),

  /**
   * Mark existing JournalNodes 'isInstalled' and 'showCurrentPrefix'
   */
  updateJournalNodeInfo: function () {
    var jns = this.get('selectedServicesMasters').filterProperty('component_name', 'JOURNALNODE');
    var hosts = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
    hosts.forEach(function (host) {
      var jn = jns.findProperty('selectedHost', host);
      if (jn) {
        jn.set('isInstalled', true);
        jn.set('showCurrentPrefix', true);
      }
    });
  }.observes('hostNameCheckTrigger'),

  /**
   * Callback after load controller data (hosts, host components etc)
   * @method loadStepCallback
   */
  loadStepCallback: function loadStepCallback(components, self) {
    self.renderComponents(components);

    self.get('addableComponents').forEach(function (componentName) {
      self.updateComponent(componentName);
    }, self);
    self.set('isRecommendationsLoaded', true);
  },

  /**
   * Next button is disabled when there is any change to the original JournalNode hosts
   */
  nextButtonDisabled: function () {
    var currentHosts = this.get('selectedServicesMasters').filterProperty('component_name', 'JOURNALNODE').mapProperty('selectedHost');
    var originalHosts = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
    return currentHosts.sort().join() === originalHosts.sort().join();
  }.property('hostNameCheckTrigger', 'nextButtonCheckTrigger')

});

});

require.register("controllers/main/admin/highAvailability/journalNode/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} nnHaConfigDependencies
 * @property {string} namespaceId
 * @property {object} serverConfigs
 * @property {string|number} nnHttpPort
 * @property {string|number} nnHttpsPort
 * @property {string|number} nnRpcPort
 * @property {string|number} zkClientPort
 */

var App = require('app');

require('utils/configs/nn_ha_config_initializer');

App.ManageJournalNodeWizardStep2Controller = Em.Controller.extend({
  name: "manageJournalNodeWizardStep2Controller",
  selectedService: null,
  stepConfigs: [],
  serverConfigData: {},
  moveJNConfig: $.extend(true, {}, require('data/configs/wizards/move_journal_node_properties').moveJNConfig),
  once: false,
  isLoaded: false,
  versionLoaded: true,
  hideDependenciesInfoBar: true,

  isNextDisabled: Em.computed.not('isLoaded'),

  clearStep: function clearStep() {
    this.get('stepConfigs').clear();
    this.set('serverConfigData', {});
  },

  loadStep: function loadStep() {
    this.clearStep();
    this.loadConfigsTags();
  },

  loadConfigsTags: function loadConfigsTags() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var urlParams = [];
    var hdfsSiteTag = data.Clusters.desired_configs['hdfs-site'].tag;
    urlParams.push('(type=hdfs-site&tag=' + hdfsSiteTag + ')');
    this.set("hdfsSiteTag", { name: "hdfsSiteTag", value: hdfsSiteTag });

    App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: urlParams.join('|')
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data) {
    this.set('serverConfigData', data);
    this.set('content.nameServiceIds', data.items[0].properties['dfs.nameservices'].split(','));
    this.tweakServiceConfigs(this.get('moveJNConfig.configs'));
    this.renderServiceConfigs(this.get('moveJNConfig'));
    this.set('isLoaded', true);
  },

  /**
   * Generate set of data used to correctly initialize config values and names
   */
  _prepareDependencies: function _prepareDependencies(nameServiceId) {
    var ret = {};
    var configsFromServer = this.get('serverConfigData.items');
    ret.namespaceId = nameServiceId || this.get('content.nameServiceIds')[0];
    ret.serverConfigs = configsFromServer;
    return ret;
  },

  /**
   * Generate set of data with information about cluster topology
   * Used in the configs' initialization process
   *
   * @returns {extendedTopologyLocalDB}
   * @private
   * @method _prepareLocalDB
   */
  _prepareLocalDB: function _prepareLocalDB() {
    var localDB = this.get('content').getProperties(['masterComponentHosts', 'slaveComponentHosts', 'hosts']);
    localDB.installedServices = App.Service.find().mapProperty('serviceName');
    return localDB;
  },

  tweakServiceConfigs: function tweakServiceConfigs(allConfigsDescriptor) {
    var hasNameNodeFederation = App.get('hasNameNodeFederation');
    var configs = hasNameNodeFederation ? allConfigsDescriptor.filterProperty('presentForFederatedHDFS') : allConfigsDescriptor.filterProperty('presentForNonFederatedHDFS');
    var nameSpaceDependentConfigs = configs.filterProperty('dependsOnNameServiceId');
    var nameSpaceIndependentConfigs = configs.rejectProperty('dependsOnNameServiceId');
    var localDB = this._prepareLocalDB();
    var commonDependencies = this._prepareDependencies();
    var generatedConfigs = [];
    var wizardController = App.router.get(this.get('content.controllerName'));
    var journalNodes = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE');

    nameSpaceIndependentConfigs.forEach(function (config) {
      App.NnHaConfigInitializer.initialValue(config, localDB, commonDependencies);
      config.isOverridable = false;
      generatedConfigs.push(config);
    });

    this.get('content.nameServiceIds').forEach(function (nameServiceId) {
      var dependencies = this._prepareDependencies(nameServiceId);
      dependencies.journalnodes = journalNodes.map(function (c) {
        return c.hostName + ':8485';
      }).join(';');
      nameSpaceDependentConfigs.forEach(function (config) {
        var generatedConfig = $.extend({}, config, {
          isOverridable: false,
          name: wizardController.replaceDependencies(config.name, dependencies),
          displayName: wizardController.replaceDependencies(config.displayName, dependencies),
          value: wizardController.replaceDependencies(config.value, dependencies),
          recommendedValue: wizardController.replaceDependencies(config.recommendedValue, dependencies)
        });
        generatedConfigs.push(generatedConfig);
      }, this);
    }, this);

    this.set('moveJNConfig.configs', generatedConfigs);

    return generatedConfigs;
  },

  renderServiceConfigs: function renderServiceConfigs(_serviceConfig) {
    var serviceConfig = App.ServiceConfig.create({
      serviceName: _serviceConfig.serviceName,
      displayName: _serviceConfig.displayName,
      configCategories: [],
      showConfig: true,
      configs: []
    });

    _serviceConfig.configCategories.forEach(function (_configCategory) {
      if (App.Service.find().someProperty('serviceName', _configCategory.name)) {
        serviceConfig.configCategories.pushObject(_configCategory);
      }
    }, this);

    this.loadComponentConfigs(_serviceConfig, serviceConfig);

    this.get('stepConfigs').pushObject(serviceConfig);
    this.set('selectedService', this.get('stepConfigs').objectAt(0));
    this.set('once', true);
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  loadComponentConfigs: function loadComponentConfigs(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep3Controller = App.HighAvailabilityWizardStep4Controller.extend({
  name: 'manageJournalNodeWizardStep3Controller',

  isActiveNameNodesStarted: true,

  isHDFSNameSpacesLoaded: Em.computed.alias('App.router.clusterController.isHDFSNameSpacesLoaded'),

  isDataLoadedAndNextEnabled: Em.computed.and('isNextEnabled', 'isHDFSNameSpacesLoaded'),

  pullCheckPointsStatuses: function pullCheckPointsStatuses() {
    if (this.get('isHDFSNameSpacesLoaded')) {
      this.removeObserver('isHDFSNameSpacesLoaded', this, 'pullCheckPointsStatuses');
      var hdfsModel = App.HDFSService.find('HDFS'),
          nameSpaces = hdfsModel.get('masterComponentGroups'),
          nameSpacesCount = nameSpaces.length;
      if (nameSpacesCount > 1) {
        var hostNames = hdfsModel.get('activeNameNodes').mapProperty('hostName');
        if (hostNames.length < nameSpacesCount) {
          nameSpaces.forEach(function (nameSpace) {
            var hosts = nameSpace.hosts,
                hasActiveNameNode = hosts.some(function (hostName) {
              return hostNames.contains(hostName);
            });

            if (!hasActiveNameNode) {
              var hostForNameSpace = hosts.find(function (hostName) {
                return App.HostComponent.find('NAMENODE_' + hostName).get('workStatus') === 'STARTED';
              }) || hosts[0];
              hostNames.push(hostForNameSpace);
            }
          });
        }
        App.ajax.send({
          name: 'admin.high_availability.getNnCheckPointsStatuses',
          sender: this,
          data: {
            hostNames: hostNames
          },
          success: 'checkNnCheckPointsStatuses'
        });
      } else {
        this.pullCheckPointStatus();
      }
    } else {
      this.addObserver('isHDFSNameSpacesLoaded', this, 'pullCheckPointsStatuses');
    }
  },

  checkNnCheckPointsStatuses: function checkNnCheckPointsStatuses(data) {
    var _this = this;

    var items = Em.getWithDefault(data, 'items', []),
        isNextEnabled = items.length && items.every(this.getNnCheckPointStatus);
    this.setProperties({
      isActiveNameNodesStarted: items.length && items.everyProperty('HostRoles.desired_state', 'STARTED'),
      isNextEnabled: isNextEnabled
    });
    if (!isNextEnabled) {
      window.setTimeout(function () {
        _this.pullCheckPointsStatuses();
      }, this.POLL_INTERVAL);
    }
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep4Controller = App.ManageJournalNodeProgressPageController.extend({
  name: 'manageJournalNodeWizardStep4Controller',
  clusterDeployState: 'JOURNALNODE_MANAGEMENT',
  tasksMessagesPrefix: 'admin.manageJournalNode.wizard.step',

  commands: ['stopStandbyNameNode', 'stopAllServices', 'installJournalNodes', 'deleteJournalNodes', 'reconfigureHDFS'],

  hdfsSiteTag: "",

  stopStandbyNameNode: function stopStandbyNameNode() {
    // save who's active and who's standby at this point in time
    var hostName = this.get('content.standByNN.host_name');
    this.updateComponent('NAMENODE', hostName, 'HDFS', 'INSTALLED');
  },

  stopAllServices: function stopAllServices() {
    this.stopServices([], true, true);
  },

  installJournalNodes: function installJournalNodes() {
    var hostNames = App.router.get('manageJournalNodeWizardController').getJournalNodesToAdd();
    if (hostNames && hostNames.length > 0) {
      this.createInstallComponentTask('JOURNALNODE', hostNames, "HDFS");
    } else {
      this.onTaskCompleted();
    }
  },

  deleteJournalNodes: function deleteJournalNodes() {
    var hosts = App.router.get('manageJournalNodeWizardController').getJournalNodesToDelete();
    if (hosts && hosts.length > 0) {
      hosts.forEach(function (host) {
        this.deleteComponent('JOURNALNODE', host);
      }, this);
    } else {
      this.onTaskCompleted();
    }
  },

  reconfigureHDFS: function reconfigureHDFS() {
    this.updateConfigProperties(this.get('content.serviceConfigProperties'));
  },

  /**
   * Update service configurations
   * @param {Object} data - config object to update
   */
  updateConfigProperties: function updateConfigProperties(data) {
    var siteNames = ['hdfs-site'];
    var configData = this.reconfigureSites(siteNames, data, Em.I18n.t('admin.manageJournalNode.step4.save.configuration.note').format(App.format.role('NAMENODE', false)));
    App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'installHDFSClients',
      error: 'onTaskError'
    });
  },

  installHDFSClients: function installHDFSClients() {
    var nnHostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    var jnHostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    var hostNames = nnHostNames.concat(jnHostNames).uniq();
    this.createInstallComponentTask('HDFS_CLIENT', hostNames, 'HDFS');
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'JOURNALNODE_MANAGEMENT',
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    });
  }

});

});

require.register("controllers/main/admin/highAvailability/journalNode/step5_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep5Controller = Em.Controller.extend({
  name: 'manageJournalNodeWizardStep5Controller',
  isHDFSNameSpacesLoaded: Em.computed.alias('App.router.clusterController.isHDFSNameSpacesLoaded'),
  isNextEnabled: Em.computed.alias('isHDFSNameSpacesLoaded'),
  done: function done() {
    if (this.get('isNextEnabled')) {
      App.router.send('next');
    }
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/step6_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep6Controller = App.ManageJournalNodeProgressPageController.extend({
  name: 'manageJournalNodeWizardStep6Controller',
  clusterDeployState: 'JOURNALNODE_MANAGEMENT',
  tasksMessagesPrefix: 'admin.manageJournalNode.wizard.step',

  commands: ['startJournalNodes'],

  startJournalNodes: function startJournalNodes() {
    var hostNames = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
    this.updateComponent('JOURNALNODE', hostNames, 'HDFS', 'Start');
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/step7_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep7Controller = App.ManageJournalNodeProgressPageController.extend(App.WizardEnableDone, {
  name: 'manageJournalNodeWizardStep7Controller',
  clusterDeployState: 'JOURNALNODE_MANAGEMENT',
  tasksMessagesPrefix: 'admin.manageJournalNode.wizard.step',

  commands: ['startAllServices'],

  startAllServices: function startAllServices() {
    this.startServices(false);
  }
});

});

require.register("controllers/main/admin/highAvailability/journalNode/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardController = App.WizardController.extend({

  name: 'manageJournalNodeWizardController',

  totalSteps: 7,

  displayName: Em.I18n.t('admin.manageJournalNode.wizard.header'),

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  content: Em.Object.create({
    controllerName: 'manageJournalNodeWizardController',
    cluster: null,
    hosts: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: [],
    serviceName: 'MISC',
    hdfsUser: "hdfs",
    nameServiceIds: [],
    failedTask: null,
    requestIds: null
  }),

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'JOURNALNODE_MANAGEMENT',
      wizardControllerName: 'manageJournalNodeWizardController',
      localdb: App.db.data
    });
  },

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.router.getClusterName() });
  },

  loadMap: {
    '1': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred(),
            self = this,
            usersLoadingCallback = function usersLoadingCallback() {
          self.save('hdfsUser');
          self.load('cluster');
          self.loadHosts().done(function () {
            self.loadServicesFromServer();
            self.loadMasterComponentHosts().done(function () {
              self.load('hdfsUser');
              if (!self.getDBProperty('activeNN')) {
                self.saveNNs();
              } else {
                self.loadNNs();
              }
              dfd.resolve();
            });
          });
        };
        if (self.getDBProperty('hdfsUser')) {
          usersLoadingCallback();
        } else {
          this.loadHdfsUserFromServer().done(function (data) {
            self.set('content.hdfsUser', Em.get(data, '0.properties.hdfs_user'));
            usersLoadingCallback();
          });
        }
        return dfd.promise();
      }
    }],
    2: [{
      type: 'sync',
      callback: function callback() {
        this.loadNameServiceIds();
        this.loadServiceConfigProperties();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
      }
    }]
  },

  getJournalNodesToAdd: function getJournalNodesToAdd() {
    var result = [];
    var masterComponentHosts = this.get('content.masterComponentHosts');
    if (masterComponentHosts) {
      result = masterComponentHosts.filterProperty('component', 'JOURNALNODE').filterProperty('isInstalled', false).mapProperty('hostName');
    }
    return result;
  },

  getJournalNodesToDelete: function getJournalNodesToDelete() {
    var result = [];
    var masterComponentHosts = this.get('content.masterComponentHosts');
    if (masterComponentHosts) {
      var currentJNs = masterComponentHosts.filterProperty('component', 'JOURNALNODE');
      var existingHosts = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').mapProperty('hostName');
      result = existingHosts.filter(function (host) {
        return currentJNs.filterProperty('hostName', host).length === 0;
      });
    }
    return result;
  },

  isDeleteOnly: function () {
    return this.get('currentStep') > 1 && this.getJournalNodesToAdd().length === 0 && this.getJournalNodesToDelete().length > 0;
  }.property('content.masterComponentHosts', 'App.router.clusterController.isHostsLoaded', 'currentStep'),

  /**
   * Save config properties
   * @param stepController ManageJournalNodeWizardStep3Controller
   */
  saveServiceConfigProperties: function saveServiceConfigProperties(stepController) {
    var serviceConfigProperties = [];
    var data = stepController.get('serverConfigData');

    var _content = stepController.get('stepConfigs')[0];
    _content.get('configs').forEach(function (_configProperties) {
      var siteObj = data.items.findProperty('type', _configProperties.get('filename'));
      if (siteObj) {
        siteObj.properties[_configProperties.get('name')] = _configProperties.get('value');
      }
    }, this);
    this.setDBProperty('serviceConfigProperties', data);
    this.set('content.serviceConfigProperties', data);
  },

  /**
   * Load serviceConfigProperties to model
   */
  loadServiceConfigProperties: function loadServiceConfigProperties() {
    this.set('content.serviceConfigProperties', this.getDBProperty('serviceConfigProperties'));
  },

  saveNNs: function saveNNs() {
    var activeNN = App.HostComponent.find().findProperty('displayNameAdvanced', 'Active NameNode');
    var standByNN = App.HostComponent.find().findProperty('displayNameAdvanced', 'Standby NameNode');
    this.set('content.activeNN', activeNN);
    this.set('content.standByNN', standByNN);
    this.setDBProperty('activeNN', activeNN);
    this.setDBProperty('standByNN', standByNN);
  },

  loadNNs: function loadNNs() {
    var activeNN = this.getDBProperty('activeNN');
    var standByNN = this.getDBProperty('standByNN');
    this.set('content.activeNN', activeNN);
    this.set('content.standByNN', standByNN);
  },

  saveConfigTag: function saveConfigTag(tag) {
    App.db.setManageJournalNodeWizardConfigTag(tag);
    this.set('content.' + tag.name, tag.value);
  },

  loadConfigTag: function loadConfigTag(tag) {
    var tagVal = App.db.getManageJournalNodeWizardConfigTag(tag);
    this.set('content.' + tag, tagVal);
  },

  saveNameServiceIds: function saveNameServiceIds(nameServiceIds) {
    this.setDBProperty('nameServiceIds', nameServiceIds);
    this.set('content.nameServiceIds', nameServiceIds);
  },

  loadNameServiceIds: function loadNameServiceIds() {
    var nameServiceIds = this.getDBProperty('nameServiceIds');
    this.set('content.nameServiceIds', nameServiceIds);
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearTasksData: function clearTasksData() {
    this.saveTasksStatuses(undefined);
    this.saveRequestIds(undefined);
    this.saveTasksRequestIds(undefined);
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    App.db.data.Installer = {};
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardController = App.WizardController.extend({

  name: 'rollbackHighAvailabilityWizardController',

  totalSteps: 3,

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  content: Em.Object.create({
    controllerName: 'RollbackHighAvailabilityWizardController',
    cluster: null,
    masterComponentHosts: null,
    serviceName: 'MISC',
    hdfsUser: "hdfs",
    nameServiceId: '',
    selectedAddNNHost: null,
    selectedSNNHost: null,
    activeNNHost: null
  }),

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'ROLLBACK_HIGH_AVAILABILITY',
      wizardControllerName: 'rollbackHighAvailabilityWizardController',
      localdb: App.db.data
    });
  },

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.router.getClusterName() });
  },

  /**
   * save status of the cluster.
   * @param clusterStatus object with status,requestId fields.
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    var oldStatus = this.toObject(this.get('content.cluster'));
    clusterStatus = jQuery.extend(oldStatus, clusterStatus);
    if (clusterStatus.requestId) {
      clusterStatus.requestId.forEach(function (requestId) {
        if (clusterStatus.oldRequestsId.indexOf(requestId) === -1) {
          clusterStatus.oldRequestsId.push(requestId);
        }
      }, this);
    }
    this.set('content.cluster', clusterStatus);
    this.save('cluster');
  },

  saveTasksStatuses: function saveTasksStatuses(statuses) {
    App.db.setRollbackHighAvailabilityWizardTasksStatuses(statuses);
    this.set('content.tasksStatuses', statuses);
  },

  saveRequestIds: function saveRequestIds(requestIds) {
    App.db.setRollbackHighAvailabilityWizardRequestIds(requestIds);
    this.set('content.requestIds', requestIds);
  },

  saveSelectedSNN: function saveSelectedSNN(addNN) {
    App.db.setRollBackHighAvailabilityWizardSelectedSNN(addNN);
    this.set('content.selectedAddNN', addNN);
  },

  saveSelectedAddNN: function saveSelectedAddNN(sNN) {
    App.db.setRollBackHighAvailabilityWizardSelectedAddNN(sNN);
    this.set('content.selectedSNN', sNN);
  },

  loadAddNNHost: function loadAddNNHost() {
    var addNNHost = App.db.getRollBackHighAvailabilityWizardAddNNHost();
    this.set('content.addNNHost', addNNHost);
  },

  loadSNNHost: function loadSNNHost() {
    var sNNHost = App.db.getRollBackHighAvailabilityWizardSNNHost();
    this.set('content.sNNHost', sNNHost);
  },

  loadTasksStatuses: function loadTasksStatuses() {
    var sNNHost = App.db.getRollbackHighAvailabilityWizardTasksStatuses();
    this.set('content.tasksStatuses', sNNHost);
  },

  loadRequestIds: function loadRequestIds() {
    var requestIds = App.db.getRollbackHighAvailabilityWizardRequestIds();
    this.set('content.requestIds', requestIds);
  },

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadAllPriorSteps: function loadAllPriorSteps() {
    var step = this.get('currentStep');
    switch (step) {
      case '3':
      case '2':
      case '1':
        this.loadSNNHost();
        this.loadAddNNHost();
        this.load('cluster');
    }
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearTasksData: function clearTasksData() {
    this.saveTasksStatuses(undefined);
    this.saveRequestIds(undefined);
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.setCurrentStep('1');
    this.clearAllSteps();
    App.router.get('updateController').updateAll();
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/rollbackHA/step1_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep1Controller = Em.Controller.extend({

  name: "rollbackHighAvailabilityWizardStep1Controller"

});

});

require.register("controllers/main/admin/highAvailability/nameNode/rollbackHA/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep2Controller = App.HighAvailabilityWizardStep4Controller.extend({
  name: "rollbackHighAvailabilityWizardStep2Controller",

  pullCheckPointStatus: function pullCheckPointStatus() {
    var hostName = this.get('content.activeNNHost');
    App.ajax.send({
      name: 'admin.high_availability.getNnCheckPointStatus',
      sender: this,
      data: {
        hostName: hostName
      },
      success: 'checkNnCheckPointStatus'
    });
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/rollbackHA/step3_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep3Controller = Em.Controller.extend({
  name: "rollbackHighAvailabilityWizardStep3Controller"
});

});

require.register("controllers/main/admin/highAvailability/nameNode/rollback_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/main/admin/highAvailability/progress_controller');

App.HighAvailabilityRollbackController = App.HighAvailabilityProgressPageController.extend({

  name: "highAvailabilityRollbackController",

  failedTask: null,
  configsSaved: false,
  deletedHdfsClients: 0,
  numOfDelOperations: 0,
  isRollback: true,
  hostsToPerformDel: [],

  commands: ['stopAllServices', 'restoreHBaseConfigs', 'restoreAccumuloConfigs', 'restoreHawqConfigs', 'stopFailoverControllers', 'deleteFailoverControllers', 'deletePXF', 'stopStandbyNameNode', 'stopNameNode', 'restoreHDFSConfigs', 'enableSecondaryNameNode', 'stopJournalNodes', 'deleteJournalNodes', 'deleteAdditionalNameNode', 'startAllServices'],

  loadStep: function loadStep() {
    var self = this;
    this.initData().done(function () {
      self.clearStep();
      self.loadTasks();
      self.addObserver('tasks.@each.status', self, 'onTaskStatusChange');
      self.onTaskStatusChange();
    });
  },

  initData: function initData() {
    var self = this,
        dfd = $.Deferred();
    this.loadMasterComponentHosts().done(function () {
      self.loadFailedTask();
      self.loadHdfsClientHosts();
      dfd.resolve();
    });
    return dfd.promise();
  },

  setCommandsAndTasks: function setCommandsAndTasks(tmpTasks) {
    var fTask = this.get('failedTask');
    var commandsArray = ['deleteSNameNode', 'startAllServices', 'reconfigureHBase', 'reconfigureAMS', 'reconfigureAccumulo', 'reconfigureHawq', 'installPXF', 'startZKFC', 'installZKFC', 'startSecondNameNode', 'startNameNode', 'startZooKeeperServers', 'reconfigureHDFS', 'disableSNameNode', 'startJournalNodes', 'installJournalNodes', 'installNameNode', 'stopAllServices'];
    var index = commandsArray.indexOf(fTask.command);

    if (index > commandsArray.indexOf('startSecondNameNode')) {
      --index;
    }
    var newCommands = this.get('commands').splice(index);
    this.set('commands', newCommands);
    var newTasks = tmpTasks.splice(index);
    for (var i = 0; i < newTasks.length; i++) {
      newTasks[i].id = i;
    }
    this.set('tasks', newTasks);
    var pxfTask = this.get('tasks').findProperty('command', 'deletePXF');
    if (!App.Service.find().someProperty('serviceName', 'PXF') && pxfTask) {
      this.get('tasks').splice(pxfTask.get('id'), 1);
    }
    var hbaseTask = this.get('tasks').findProperty('command', 'restoreHBaseConfigs');
    if (!App.Service.find().someProperty('serviceName', 'HBASE') && hbaseTask) {
      this.get('tasks').splice(hbaseTask.get('id'), 1);
    }
    var accumuloTask = this.get('tasks').findProperty('command', 'restoreAccumuloConfigs');
    if (!App.Service.find().someProperty('serviceName', 'ACCUMULO') && accumuloTask) {
      this.get('tasks').splice(accumuloTask.get('id'), 1);
    }
    var hawqTask = this.get('tasks').findProperty('command', 'restoreHawqConfigs');
    if (!App.Service.find().someProperty('serviceName', 'HAWQ') && hawqTask) {
      this.get('tasks').splice(hawqTask.get('id'), 1);
    }
  },

  clearStep: function clearStep() {
    this.set('isSubmitDisabled', true);
    this.set('tasks', []);
    this.set('logs', []);
    this.set('currentRequestIds', []);
    var commands = this.get('commands');
    var tmpTasks = [];
    for (var i = 0; i < commands.length; i++) {
      tmpTasks.pushObject(Ember.Object.create({
        title: Em.I18n.t('admin.highAvailability.rollback.task' + i + '.title'),
        status: 'PENDING',
        id: i,
        command: commands[i],
        showRetry: false,
        showRollback: false,
        showSkip: false,
        name: Em.I18n.t('admin.highAvailability.rollback.task' + i + '.title'),
        displayName: Em.I18n.t('admin.highAvailability.rollback.task' + i + '.title'),
        progress: 0,
        isRunning: false,
        hosts: []
      }));
    }
    this.setCommandsAndTasks(tmpTasks);
  },

  onTaskStatusChange: function onTaskStatusChange() {
    if (!this.get('tasks').someProperty('status', 'IN_PROGRESS') && !this.get('tasks').someProperty('status', 'QUEUED') && !this.get('tasks').someProperty('status', 'FAILED')) {
      var nextTask = this.get('tasks').findProperty('status', 'PENDING');
      if (nextTask) {
        this.set('status', 'IN_PROGRESS');
        this.setTaskStatus(nextTask.get('id'), 'QUEUED');
        this.set('currentTaskId', nextTask.get('id'));
        this.runTask(nextTask.get('id'));
      } else {
        this.set('status', 'COMPLETED');
        this.set('isSubmitDisabled', false);
      }
    } else if (this.get('tasks').someProperty('status', 'FAILED')) {
      this.set('status', 'FAILED');
      this.get('tasks').findProperty('status', 'FAILED').set('showRetry', true);
      this.get('tasks').findProperty('status', 'FAILED').set('showSkip', true);
    }
    this.get('tasks').filterProperty('status', 'COMPLETED').setEach('showRetry', false);
    this.get('tasks').filterProperty('status', 'COMPLETED').setEach('showSkip', false);

    var statuses = this.get('tasks').mapProperty('status');
    var logs = this.get('tasks').mapProperty('hosts');
    var requestIds = this.get('currentRequestIds');
    this.saveTasksStatuses(statuses);
    this.saveRequestIds(requestIds);
    this.saveLogs(logs);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'HIGH_AVAILABILITY_ROLLBACK',
      wizardControllerName: 'highAvailabilityRollbackController',
      localdb: App.db.data
    });
  },

  skipTask: function skipTask() {
    var task = this.get('tasks').findProperty('status', 'FAILED');
    task.set('showRetry', false);
    task.set('showSkip', false);
    task.set('status', 'COMPLETED');
  },

  retryTask: function retryTask() {
    var task = this.get('tasks').findProperty('status', 'FAILED');
    task.set('showRetry', false);
    task.set('showSkip', false);
    task.set('status', 'PENDING');
  },

  onTaskCompleted: function onTaskCompleted() {
    var curTaskStatus = this.getTaskStatus(this.get('currentTaskId'));
    if (curTaskStatus != 'FAILED' && curTaskStatus != 'TIMEDOUT' && curTaskStatus != 'ABORTED') {
      this.setTaskStatus(this.get('currentTaskId'), 'COMPLETED');
    }
  },

  getTaskStatus: function getTaskStatus(taskId) {
    return this.get('tasks').findProperty('id', taskId).get('status');
  },

  loadFailedTask: function loadFailedTask() {
    var failedTask = App.db.getHighAvailabilityWizardFailedTask();
    this.set('failedTask', failedTask);
  },

  done: function done() {
    if (!this.get('isSubmitDisabled')) {
      this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
      this.popup.proceedOnClose();
    }
  },

  stopAllServices: function stopAllServices() {
    App.ajax.send({
      name: 'common.services.update',
      data: {
        context: "Stop all services",
        "ServiceInfo": {
          "state": "INSTALLED"
        }
      },
      sender: this,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },
  restoreHBaseConfigs: function restoreHBaseConfigs() {
    this.loadConfigTag("hbaseSiteTag");
    var hbaseSiteTag = this.get("content.hbaseSiteTag");
    App.ajax.send({
      name: 'admin.high_availability.load_hbase_configs',
      sender: this,
      data: {
        hbaseSiteTag: hbaseSiteTag
      },
      success: 'onLoadHbaseConfigs',
      error: 'onTaskError'
    });
  },
  restoreAccumuloConfigs: function restoreAccumuloConfigs() {
    this.loadConfigTag("accumuloSiteTag");
    var accumuloSiteTag = this.get("content.accumuloSiteTag");
    App.ajax.send({
      name: 'admin.high_availability.load_accumulo_configs',
      sender: this,
      data: {
        accumuloSiteTag: accumuloSiteTag
      },
      success: 'onLoadAccumuloConfigs',
      error: 'onTaskError'
    });
  },
  restoreHawqConfigs: function restoreHawqConfigs() {
    var tags = ['hawqSiteTag', 'hdfsClientTag'];
    tags.forEach(function (tagName) {
      var tag = this.get("content." + tagName);
      App.ajax.send({
        name: 'admin.high_availability.load_hawq_configs',
        sender: this,
        data: {
          tagName: tag
        },
        success: 'onLoadHawqConfigs',
        error: 'onTaskError'
      });
    }, this);
  },

  deletePXF: function deletePXF() {
    var secondNameNodeHost = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).mapProperty('hostName');
    var pxfComponent = this.getSlaveComponentHosts().findProperty('componentName', 'PXF');
    var dataNodeComponent = this.getSlaveComponentHosts().findProperty('componentName', 'DATANODE');

    var host, i;

    // check if PXF is already installed on the host assigned for additional NameNode
    var pxfComponentInstalled = false;
    for (i = 0; i < pxfComponent.hosts.length; i++) {
      host = pxfComponent.hosts[i];
      if (host.hostName === secondNameNodeHost) {
        pxfComponentInstalled = true;
        break;
      }
    }

    // check if DATANODE is already installed on the host assigned for additional NameNode
    var dataNodeComponentInstalled = false;
    for (i = 0; i < dataNodeComponent.hosts.length; i++) {
      host = dataNodeComponent.hosts[i];
      if (host.hostName === secondNameNodeHost) {
        dataNodeComponentInstalled = true;
        break;
      }
    }

    // if no DATANODE exists on that host, remove PXF
    if (!dataNodeComponentInstalled && pxfComponentInstalled) {
      this.updateComponent('PXF', secondNameNodeHost, "PXF", "Stop");
      this.checkBeforeDelete('PXF', secondNameNodeHost);
    }
  },

  stopFailoverControllers: function stopFailoverControllers() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    this.updateComponent('ZKFC', hostNames, "HDFS", "Stop");
  },
  deleteFailoverControllers: function deleteFailoverControllers() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    this.checkBeforeDelete('ZKFC', hostNames);
  },
  stopStandbyNameNode: function stopStandbyNameNode() {
    var hostName = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).hostName;
    this.updateComponent('NAMENODE', hostName, "HDFS", "Stop");
  },
  stopNameNode: function stopNameNode() {
    var hostName = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true).hostName;
    this.updateComponent('NAMENODE', hostName, "HDFS", "Stop");
  },
  restoreHDFSConfigs: function restoreHDFSConfigs() {
    this.unInstallHDFSClients();
  },
  enableSecondaryNameNode: function enableSecondaryNameNode() {
    var hostName = this.get('content.masterComponentHosts').findProperty('component', 'SECONDARY_NAMENODE').hostName;
    this.updateComponent('SECONDARY_NAMENODE', hostName, "HDFS", "Install", hostName.length);
  },
  stopJournalNodes: function stopJournalNodes() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    this.updateComponent('JOURNALNODE', hostNames, "HDFS", "Stop");
  },
  deleteJournalNodes: function deleteJournalNodes() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    this.unInstallComponent('JOURNALNODE', hostNames);
  },
  deleteAdditionalNameNode: function deleteAdditionalNameNode() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).mapProperty('hostName');
    this.unInstallComponent('NAMENODE', hostNames);
  },
  startAllServices: function startAllServices() {
    App.ajax.send({
      name: 'common.services.update',
      data: {
        context: "Start all services",
        "ServiceInfo": {
          "state": "STARTED"
        }
      },
      sender: this,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  onLoadHbaseConfigs: function onLoadHbaseConfigs(data) {
    var hbaseSiteProperties = data.items.findProperty('type', 'hbase-site').properties;
    App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'hbase-site',
        properties: hbaseSiteProperties
      },
      success: 'onTaskCompleted',
      error: 'onTaskError'
    });
  },
  onLoadAccumuloConfigs: function onLoadAccumuloConfigs(data) {
    var accumuloSiteProperties = data.items.findProperty('type', 'accumulo-site').properties;
    App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'accumulo-site',
        properties: accumuloSiteProperties
      },
      success: 'onTaskCompleted',
      error: 'onTaskError'
    });
  },

  onLoadHawqConfigs: function onLoadHawqConfigs(data) {
    var hawqSiteProperties = data.items.findProperty('type', 'hawq-site').properties;
    App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'hawq-site',
        properties: hawqSiteProperties
      },
      success: 'onTaskCompleted',
      error: 'onTaskError'
    });
  },
  onDeletedHDFSClient: function onDeletedHDFSClient() {
    var deletedHdfsClients = this.get('deletedHdfsClients');
    var hostName = this.get("content.hdfsClientHostNames");
    var notDeletedHdfsClients = hostName.length - deletedHdfsClients;
    if (notDeletedHdfsClients > 1 && hostName.length != 1) {
      this.set('deletedHdfsClients', deletedHdfsClients + 1);
      return;
    }
    this.loadConfigTag("hdfsSiteTag");
    this.loadConfigTag("coreSiteTag");
    var hdfsSiteTag = this.get("content.hdfsSiteTag");
    var coreSiteTag = this.get("content.coreSiteTag");
    App.ajax.send({
      name: 'admin.high_availability.load_configs',
      sender: this,
      data: {
        hdfsSiteTag: hdfsSiteTag,
        coreSiteTag: coreSiteTag
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data) {
    this.set('configsSaved', false);
    App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'hdfs-site',
        properties: data.items.findProperty('type', 'hdfs-site').properties
      },
      success: 'onHdfsConfigsSaved',
      error: 'onTaskError'
    });
    App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'core-site',
        properties: data.items.findProperty('type', 'core-site').properties
      },
      success: 'onHdfsConfigsSaved',
      error: 'onTaskError'
    });
  },

  onHdfsConfigsSaved: function onHdfsConfigsSaved() {
    if (!this.get('configsSaved')) {
      this.set('configsSaved', true);
      return;
    }
    this.onTaskCompleted();
  },

  unInstallHDFSClients: function unInstallHDFSClients() {
    var hostName = this.get("content.hdfsClientHostNames");
    for (var i = 0; i < hostName.length; i++) {
      App.ajax.send({
        name: 'common.delete.host_component',
        sender: this,
        data: {
          componentName: 'HDFS_CLIENT',
          hostName: hostName[i]
        },
        success: 'onDeletedHDFSClient',
        error: 'onTaskError'
      });
    }
  },

  unInstallComponent: function unInstallComponent(componentName, hostName) {
    if (!(hostName instanceof Array)) {
      hostName = [hostName];
    }
    for (var i = 0; i < hostName.length; i++) {
      App.ajax.send({
        name: 'common.host.host_component.passive',
        sender: this,
        data: {
          hostName: hostName[i],
          componentName: componentName,
          passive_state: "ON",
          taskNum: hostName.length,
          callback: 'checkBeforeDelete'
        },
        success: 'checkResult',
        error: 'checkResult'
      });
    }
  },

  checkBeforeDelete: function checkBeforeDelete(componentName, hostName) {
    this.set('hostsToPerformDel', []);
    if (!(hostName instanceof Array)) {
      hostName = [hostName];
    }
    for (var i = 0; i < hostName.length; i++) {
      App.ajax.send({
        name: 'admin.high_availability.getHostComponent',
        sender: this,
        data: {
          componentName: componentName,
          hostName: hostName[i],
          taskNum: hostName.length,
          callback: 'deleteComponent'
        },
        success: 'checkResult',
        error: 'checkResult'
      });
    }
  },

  checkResult: function checkResult() {
    var callback = arguments[2].callback;
    var hostName = arguments[2].hostName;
    var componentName = arguments[2].componentName;
    var taskNum = arguments[2].taskNum;
    var hostsToPerformDel = this.get('hostsToPerformDel');
    if (arguments[1] != 'error') {
      hostsToPerformDel.push({
        hostName: hostName,
        isOnHost: true
      });
    } else {
      hostsToPerformDel.push({
        hostName: 'error',
        isOnHost: false
      });
    }
    if (hostsToPerformDel.length == taskNum) {
      var hostsForDel = hostsToPerformDel.filterProperty('isOnHost', true).mapProperty('hostName');
      this.set('hostsToPerformDel', []);
      if (hostsForDel.length == 0) {
        this.onTaskCompleted();
        return;
      }
      this[callback](componentName, hostsForDel);
    }
  },

  deleteComponent: function deleteComponent(componentName, hostName) {
    if (!(hostName instanceof Array)) {
      hostName = [hostName];
    }
    this.set('numOfDelOperations', hostName.length);
    for (var i = 0; i < hostName.length; i++) {
      App.ajax.send({
        name: 'common.delete.host_component',
        sender: this,
        data: {
          componentName: componentName,
          hostName: hostName[i]
        },
        success: 'onDeleteComplete',
        error: 'onTaskError'
      });
    }
  },

  onDeleteComplete: function onDeleteComplete() {
    var leftOp = this.get('numOfDelOperations');
    if (leftOp > 1) {
      this.set('numOfDelOperations', leftOp - 1);
      return;
    }
    this.onTaskCompleted();
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

require('controllers/main/admin/serviceAccounts_controller');

App.HighAvailabilityWizardStep1Controller = Em.Controller.extend({
  name: "highAvailabilityWizardStep1Controller",

  isHawqInstalled: false,

  isNameServiceIdValid: function () {
    return validator.isValidNameServiceId(this.get('content.nameServiceId'));
  }.property('content.nameServiceId'),

  setHawqInstalled: function setHawqInstalled() {
    this.set('isHawqInstalled', App.Service.find().someProperty('serviceName', 'HAWQ'));
  },

  next: function next() {
    if (this.get('isNameServiceIdValid')) {
      App.router.send('next');
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/wizard/step5_controller');

App.HighAvailabilityWizardStep2Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {

  name: "highAvailabilityWizardStep2Controller",

  useServerValidation: false,

  mastersToShow: ['NAMENODE', 'JOURNALNODE'],

  mastersToAdd: ['NAMENODE', 'JOURNALNODE', 'JOURNALNODE', 'JOURNALNODE'],

  showCurrentPrefix: ['NAMENODE'],

  showAdditionalPrefix: ['NAMENODE'],

  showInstalledMastersFirst: true,

  JOURNALNODES_COUNT_MINIMUM: 3, // TODO get this from stack

  renderComponents: function renderComponents(masterComponents) {
    this._super(masterComponents);
    this.showHideJournalNodesAddRemoveControl();
  },

  addComponent: function addComponent(componentName) {
    this._super(componentName);
    this.showHideJournalNodesAddRemoveControl();
  },

  removeComponent: function removeComponent(componentName, serviceComponentId) {
    this._super(componentName, serviceComponentId);
    this.showHideJournalNodesAddRemoveControl();
  },

  showHideJournalNodesAddRemoveControl: function showHideJournalNodesAddRemoveControl() {
    var jns = this.get('selectedServicesMasters').filterProperty('component_name', 'JOURNALNODE');
    var maxNumMasters = this.getMaxNumberOfMasters('JOURNALNODE');
    var showRemoveControl = jns.get('length') > this.get('JOURNALNODES_COUNT_MINIMUM');
    var showAddControl = jns.get('length') < maxNumMasters;
    jns.forEach(function (item) {
      item.set('showAddControl', false);
      item.set('showRemoveControl', showRemoveControl);
    });
    jns.set('lastObject.showAddControl', showAddControl);
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} nnHaConfigDependencies
 * @property {string} namespaceId
 * @property {object} serverConfigs
 * @property {string|number} nnHttpPort
 * @property {string|number} nnHttpsPort
 * @property {string|number} nnRpcPort
 * @property {string|number} zkClientPort
 */

var App = require('app');

require('utils/configs/nn_ha_config_initializer');

App.HighAvailabilityWizardStep3Controller = Em.Controller.extend({
  name: "highAvailabilityWizardStep3Controller",
  selectedService: null,
  stepConfigs: [],
  serverConfigData: {},
  haConfig: $.extend(true, {}, require('data/configs/wizards/ha_properties').haConfig),
  once: false,
  isLoaded: false,
  isNextDisabled: Em.computed.not('isLoaded'),
  versionLoaded: true,
  hideDependenciesInfoBar: true,

  /**
   * Map of sites and properties to delete
   * @type Object
   */
  configsToRemove: {
    'hdfs-site': ['dfs.namenode.secondary.http-address', 'dfs.namenode.rpc-address', 'dfs.namenode.http-address', 'dfs.namenode.https-address']
  },

  clearStep: function clearStep() {
    this.get('stepConfigs').clear();
    this.set('serverConfigData', {});
  },

  loadStep: function loadStep() {
    this.clearStep();
    this.loadConfigsTags();
  },

  loadConfigsTags: function loadConfigsTags() {
    return App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var urlParams = [];
    var hdfsSiteTag = data.Clusters.desired_configs['hdfs-site'].tag;
    var coreSiteTag = data.Clusters.desired_configs['core-site'].tag;
    var zkSiteTag = data.Clusters.desired_configs['zoo.cfg'].tag;

    urlParams.push('(type=hdfs-site&tag=' + hdfsSiteTag + ')');
    urlParams.push('(type=core-site&tag=' + coreSiteTag + ')');
    urlParams.push('(type=zoo.cfg&tag=' + zkSiteTag + ')');
    this.set("hdfsSiteTag", { name: "hdfsSiteTag", value: hdfsSiteTag });
    this.set("coreSiteTag", { name: "coreSiteTag", value: coreSiteTag });
    this.set("zkSiteTag", { name: "zkSiteTag", value: zkSiteTag });

    if (App.Service.find().someProperty('serviceName', 'HBASE')) {
      var hbaseSiteTag = data.Clusters.desired_configs['hbase-site'].tag;
      urlParams.push('(type=hbase-site&tag=' + hbaseSiteTag + ')');
      this.set("hbaseSiteTag", { name: "hbaseSiteTag", value: hbaseSiteTag });
    }
    if (App.Service.find().someProperty('serviceName', 'ACCUMULO')) {
      var accumuloSiteTag = data.Clusters.desired_configs['accumulo-site'].tag;
      urlParams.push('(type=accumulo-site&tag=' + accumuloSiteTag + ')');
      this.set("accumuloSiteTag", { name: "accumuloSiteTag", value: accumuloSiteTag });
    }
    if (App.Service.find().someProperty('serviceName', 'AMBARI_METRICS')) {
      var amsHbaseSiteTag = data.Clusters.desired_configs['ams-hbase-site'].tag;
      urlParams.push('(type=ams-hbase-site&tag=' + amsHbaseSiteTag + ')');
      this.set("amsHbaseSiteTag", { name: "amsHbaseSiteTag", value: amsHbaseSiteTag });
    }
    if (App.Service.find().someProperty('serviceName', 'HAWQ')) {
      var hawqSiteTag = data.Clusters.desired_configs['hawq-site'].tag;
      urlParams.push('(type=hawq-site&tag=' + hawqSiteTag + ')');
      this.set("hawqSiteTag", { name: "hawqSiteTag", value: hawqSiteTag });
      var hdfsClientTag = data.Clusters.desired_configs['hdfs-client'].tag;
      urlParams.push('(type=hdfs-client&tag=' + hdfsClientTag + ')');
      this.set("hdfsClientTag", { name: "hdfsClientTag", value: hdfsClientTag });
    }
    if (App.Service.find().someProperty('serviceName', 'RANGER')) {
      var rangerEnvTag = data.Clusters.desired_configs['ranger-env'].tag;
      urlParams.push('(type=ranger-env&tag=' + rangerEnvTag + ')');
      this.set("rangerEnvTag", { name: "rangerEnvTag", value: rangerEnvTag });
      if ('ranger-hdfs-plugin-properties' in data.Clusters.desired_configs) {
        var rangerHdfsPluginPropertiesTag = data.Clusters.desired_configs['ranger-hdfs-plugin-properties'].tag;
        urlParams.push('(type=ranger-hdfs-plugin-properties&tag=' + rangerHdfsPluginPropertiesTag + ')');
        this.set("rangerHdfsPluginPropertiesTag", {
          name: "rangerHdfsPluginPropertiesTag",
          value: rangerHdfsPluginPropertiesTag
        });
      }
      if ('ranger-hdfs-audit' in data.Clusters.desired_configs) {
        var rangerHdfsAuditTag = data.Clusters.desired_configs['ranger-hdfs-audit'].tag;
        urlParams.push('(type=ranger-hdfs-audit&tag=' + rangerHdfsAuditTag + ')');
        this.set("rangerHdfsAuditTag", { name: "rangerHdfsAuditTag", value: rangerHdfsAuditTag });
      }
      if ('ranger-yarn-audit' in data.Clusters.desired_configs) {
        var yarnAuditTag = data.Clusters.desired_configs['ranger-yarn-audit'].tag;
        urlParams.push('(type=ranger-yarn-audit&tag=' + yarnAuditTag + ')');
        this.set("yarnAuditTag", { name: "yarnAuditTag", value: yarnAuditTag });
      }
      if (App.Service.find().someProperty('serviceName', 'HBASE')) {
        if ('ranger-hbase-audit' in data.Clusters.desired_configs) {
          var rangerHbaseAuditTag = data.Clusters.desired_configs['ranger-hbase-audit'].tag;
          urlParams.push('(type=ranger-hbase-audit&tag=' + rangerHbaseAuditTag + ')');
          this.set("rangerHbaseAuditTag", { name: "rangerHbaseAuditTag", value: rangerHbaseAuditTag });
        }
        if ('ranger-hbase-plugin-properties' in data.Clusters.desired_configs) {
          var rangerHbasePluginPropertiesTag = data.Clusters.desired_configs['ranger-hbase-plugin-properties'].tag;
          urlParams.push('(type=ranger-hbase-plugin-properties&tag=' + rangerHbasePluginPropertiesTag + ')');
          this.set("rangerHbasePluginPropertiesTag", {
            name: "rangerHbasePluginPropertiesTag",
            value: rangerHbasePluginPropertiesTag
          });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'KAFKA')) {
        if ('ranger-kafka-audit' in data.Clusters.desired_configs) {
          var rangerKafkaAuditTag = data.Clusters.desired_configs['ranger-kafka-audit'].tag;
          urlParams.push('(type=ranger-kafka-audit&tag=' + rangerKafkaAuditTag + ')');
          this.set("rangerKafkaAuditTag", { name: "rangerKafkaAuditTag", value: rangerKafkaAuditTag });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'KNOX')) {
        if ('ranger-knox-audit' in data.Clusters.desired_configs) {
          var rangerKnoxAuditTag = data.Clusters.desired_configs['ranger-knox-audit'].tag;
          urlParams.push('(type=ranger-knox-audit&tag=' + rangerKnoxAuditTag + ')');
          this.set("rangerKnoxAuditTag", { name: "rangerKnoxAuditTag", value: rangerKnoxAuditTag });
        }
        if ('ranger-knox-plugin-properties' in data.Clusters.desired_configs) {
          var rangerKnoxPluginPropertiesTag = data.Clusters.desired_configs['ranger-knox-plugin-properties'].tag;
          urlParams.push('(type=ranger-knox-plugin-properties&tag=' + rangerKnoxPluginPropertiesTag + ')');
          this.set("rangerKnoxPluginPropertiesTag", {
            name: "rangerKnoxPluginPropertiesTag",
            value: rangerKnoxPluginPropertiesTag
          });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'STORM')) {
        if ('ranger-storm-audit' in data.Clusters.desired_configs) {
          var rangerStormAuditTag = data.Clusters.desired_configs['ranger-storm-audit'].tag;
          urlParams.push('(type=ranger-storm-audit&tag=' + rangerStormAuditTag + ')');
          this.set("rangerStormAuditTag", { name: "rangerStormAuditTag", value: rangerStormAuditTag });
        }
        if ('ranger-storm-plugin-properties' in data.Clusters.desired_configs) {
          var rangerStormPluginPropertiesTag = data.Clusters.desired_configs['ranger-storm-plugin-properties'].tag;
          urlParams.push('(type=ranger-storm-plugin-properties&tag=' + rangerStormPluginPropertiesTag + ')');
          this.set("rangerStormPluginPropertiesTag", {
            name: "rangerStormPluginPropertiesTag",
            value: rangerStormPluginPropertiesTag
          });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'ATLAS')) {
        if ('ranger-atlas-audit' in data.Clusters.desired_configs) {
          var rangerAtlasAuditTag = data.Clusters.desired_configs['ranger-atlas-audit'].tag;
          urlParams.push('(type=ranger-atlas-audit&tag=' + rangerAtlasAuditTag + ')');
          this.set("rangerAtlasAuditTag", { name: "rangerAtlasAuditTag", value: rangerAtlasAuditTag });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'HIVE')) {
        if ('ranger-hive-audit' in data.Clusters.desired_configs) {
          var rangerHiveAuditTag = data.Clusters.desired_configs['ranger-hive-audit'].tag;
          urlParams.push('(type=ranger-hive-audit&tag=' + rangerHiveAuditTag + ')');
          this.set("rangerHiveAuditTag", { name: "rangerHiveAuditTag", value: rangerHiveAuditTag });
        }
        if ('ranger-hive-plugin-properties' in data.Clusters.desired_configs) {
          var rangerHivePluginPropertiesTag = data.Clusters.desired_configs['ranger-hive-plugin-properties'].tag;
          urlParams.push('(type=ranger-hive-plugin-properties&tag=' + rangerHivePluginPropertiesTag + ')');
          this.set("rangerHivePluginPropertiesTag", {
            name: "rangerHivePluginPropertiesTag",
            value: rangerHivePluginPropertiesTag
          });
        }
      }
      if (App.Service.find().someProperty('serviceName', 'RANGER_KMS')) {
        if ('ranger-kms-audit' in data.Clusters.desired_configs) {
          var rangerKMSAuditTag = data.Clusters.desired_configs['ranger-kms-audit'].tag;
          urlParams.push('(type=ranger-kms-audit&tag=' + rangerKMSAuditTag + ')');
          this.set("rangerKMSAuditTag", { name: "rangerKMSAuditTag", value: rangerKMSAuditTag });
        }
      }
    }
    App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: urlParams.join('|')
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data) {
    this.set('serverConfigData', data);
    this.removeConfigs(this.get('configsToRemove'), data);
    this.tweakServiceConfigs(this.get('haConfig.configs'));
    this.renderServiceConfigs(this.get('haConfig'));
    this.set('isLoaded', true);
  },

  /**
   * Generate set of data used to correctly initialize config values and names
   *
   * @returns {nnHaConfigDependencies}
   * @private
   * @method _prepareDependencies
   */
  _prepareDependencies: function _prepareDependencies() {
    var ret = {};
    var configsFromServer = this.get('serverConfigData.items');
    ret.namespaceId = this.get('content.nameServiceId');
    ret.serverConfigs = configsFromServer;
    var hdfsConfigs = configsFromServer.findProperty('type', 'hdfs-site').properties;
    var zkConfigs = configsFromServer.findProperty('type', 'zoo.cfg').properties;

    var dfsHttpA = hdfsConfigs['dfs.namenode.http-address'];
    ret.nnHttpPort = dfsHttpA ? dfsHttpA.split(':')[1] : 50070;

    var dfsHttpsA = hdfsConfigs['dfs.namenode.https-address'];
    ret.nnHttpsPort = dfsHttpsA ? dfsHttpsA.split(':')[1] : 50470;

    var dfsRpcA = hdfsConfigs['dfs.namenode.rpc-address'];
    ret.nnRpcPort = dfsRpcA ? dfsRpcA.split(':')[1] : 8020;

    ret.zkClientPort = zkConfigs['clientPort'] ? zkConfigs['clientPort'] : 2181;

    return ret;
  },

  /**
   * Generate set of data with information about cluster topology
   * Used in the configs' initialization process
   *
   * @returns {extendedTopologyLocalDB}
   * @private
   * @method _prepareLocalDB
   */
  _prepareLocalDB: function _prepareLocalDB() {
    var localDB = this.get('content').getProperties(['masterComponentHosts', 'slaveComponentHosts', 'hosts']);
    localDB.installedServices = App.Service.find().mapProperty('serviceName');
    return localDB;
  },

  tweakServiceConfigs: function tweakServiceConfigs(configs) {
    var localDB = this._prepareLocalDB();
    var dependencies = this._prepareDependencies();

    configs.forEach(function (config) {
      App.NnHaConfigInitializer.initialValue(config, localDB, dependencies);
      config.isOverridable = false;
    });

    return configs;
  },

  /**
   * Find and remove config properties in <code>serverConfigData</code>
   * @param configsToRemove - map of config sites and properties to remove
   * @param configs - configuration object
   * @returns {Object}
   */
  removeConfigs: function removeConfigs(configsToRemove, configs) {
    Em.keys(configsToRemove).forEach(function (site) {
      var siteConfigs = configs.items.findProperty('type', site);
      if (siteConfigs) {
        configsToRemove[site].forEach(function (property) {
          delete siteConfigs.properties[property];
        });
      }
    });
    return configs;
  },

  renderServiceConfigs: function renderServiceConfigs(_serviceConfig) {
    var serviceConfig = App.ServiceConfig.create({
      serviceName: _serviceConfig.serviceName,
      displayName: _serviceConfig.displayName,
      configCategories: [],
      showConfig: true,
      configs: []
    });

    _serviceConfig.configCategories.forEach(function (_configCategory) {
      if (App.Service.find().someProperty('serviceName', _configCategory.name)) {
        serviceConfig.configCategories.pushObject(_configCategory);
      }
    }, this);

    this.loadComponentConfigs(_serviceConfig, serviceConfig);

    this.get('stepConfigs').pushObject(serviceConfig);
    this.set('selectedService', this.get('stepConfigs').objectAt(0));
    this.set('once', true);
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  loadComponentConfigs: function loadComponentConfigs(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/main/admin/serviceAccounts_controller');

App.HighAvailabilityWizardStep4Controller = Em.Controller.extend({

  name: "highAvailabilityWizardStep4Controller",

  POLL_INTERVAL: 1000,

  isNextEnabled: false,

  isNameNodeStarted: true,

  pullCheckPointStatus: function pullCheckPointStatus() {
    var hostName = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true).hostName;
    return App.ajax.send({
      name: 'admin.high_availability.getNnCheckPointStatus',
      sender: this,
      data: {
        hostName: hostName
      },
      success: 'checkNnCheckPointStatus'
    });
  },

  checkNnCheckPointStatus: function checkNnCheckPointStatus(data) {
    var _this = this;

    this.set('isNameNodeStarted', data.HostRoles.desired_state === 'STARTED');
    var isNextEnabled = this.getNnCheckPointStatus(data);
    if (isNextEnabled) {
      this.set('isNextEnabled', true);
      return;
    }

    window.setTimeout(function () {
      _this.pullCheckPointStatus();
    }, this.POLL_INTERVAL);
  },

  getNnCheckPointStatus: function getNnCheckPointStatus(data) {
    var isInSafeMode = !Em.isEmpty(Em.get(data, 'metrics.dfs.namenode.Safemode'));
    var journalTransactionInfo = $.parseJSON(Em.get(data, 'metrics.dfs.namenode.JournalTransactionInfo'));
    // in case when transaction info absent or invalid return 2 which will return false in next `if` statement
    journalTransactionInfo = !!journalTransactionInfo ? parseInt(journalTransactionInfo.LastAppliedOrWrittenTxId) - parseInt(journalTransactionInfo.MostRecentCheckpointTxId) : 2;
    return journalTransactionInfo <= 1 && isInSafeMode;
  },

  done: function done() {
    if (this.get('isNextEnabled')) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        App.router.send("next");
      });
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step5_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep5Controller = App.HighAvailabilityProgressPageController.extend({

  name: "highAvailabilityWizardStep5Controller",

  commands: ['stopAllServices', 'installNameNode', 'installJournalNodes', 'reconfigureHDFS', 'startJournalNodes', 'disableSNameNode'],

  hdfsSiteTag: "",
  coreSiteTag: "",

  stopAllServices: function stopAllServices() {
    this.stopServices([], true, true);
  },

  installNameNode: function installNameNode() {
    var hostName = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).hostName;
    this.createInstallComponentTask('NAMENODE', hostName, "HDFS");
  },

  installJournalNodes: function installJournalNodes() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    this.createInstallComponentTask('JOURNALNODE', hostNames, "HDFS");
  },

  startJournalNodes: function startJournalNodes() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    this.updateComponent('JOURNALNODE', hostNames, "HDFS", "Start");
  },

  disableSNameNode: function disableSNameNode() {
    var hostName = this.get('content.masterComponentHosts').findProperty('component', 'SECONDARY_NAMENODE').hostName;
    return App.ajax.send({
      name: 'common.host.host_component.passive',
      sender: this,
      data: {
        hostName: hostName,
        passive_state: "ON",
        componentName: 'SECONDARY_NAMENODE'
      },
      success: 'onTaskCompleted',
      error: 'onTaskError'
    });
  },

  reconfigureHDFS: function reconfigureHDFS() {
    var data = this.get('content.serviceConfigProperties');
    if (App.get('isKerberosEnabled')) {
      this.reconfigureSecureHDFS();
    } else {
      this.updateConfigProperties(data);
    }
  },

  /**
   * Update service configurations
   * @param {Object} data - config object to update
   */
  updateConfigProperties: function updateConfigProperties(data) {
    var siteNames = ['hdfs-site', 'core-site'].concat(this.getRangerSiteNames(data));
    var note = Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false));
    var configData = this.reconfigureSites(siteNames, data, note);
    return App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      error: 'onTaskError',
      success: 'installHDFSClients'
    });
  },

  getRangerSiteNames: function getRangerSiteNames(data) {
    var siteNames = [];
    if (App.Service.find().someProperty('serviceName', 'RANGER')) {
      var hdfsPluginConfig = data.items.findProperty('type', 'ranger-hdfs-plugin-properties');
      if (hdfsPluginConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hdfsPluginConfig.properties) {
          siteNames.push('ranger-hdfs-plugin-properties');
        }
      }
      var hdfsAuditConfig = data.items.findProperty('type', 'ranger-hdfs-audit');
      if (hdfsAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hdfsAuditConfig.properties) {
          siteNames.push('ranger-hdfs-audit');
        }
      }
    }
    return siteNames;
  },

  installHDFSClients: function installHDFSClients() {
    var nnHostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    var jnHostNames = this.get('content.masterComponentHosts').filterProperty('component', 'JOURNALNODE').mapProperty('hostName');
    var hostNames = nnHostNames.concat(jnHostNames).uniq();
    this.createInstallComponentTask('HDFS_CLIENT', hostNames, 'HDFS');
    App.router.get(this.get('content.controllerName')).saveHdfsClientHosts(hostNames);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'HIGH_AVAILABILITY_DEPLOY',
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    });
  },

  /**
   * Process configurations for hdfs on kerberized cluster.
   * Secure properties will be applied to hdfs and core site after installing JournalNode.
   * For this case we need to pull updated configurations from API, merge them with configurations created
   * during `Review` step and store new configurations.
   */
  reconfigureSecureHDFS: function reconfigureSecureHDFS() {
    App.router.get('highAvailabilityWizardStep3Controller').loadConfigsTags.call(this);
  },

  onLoadConfigsTags: function onLoadConfigsTags() {
    App.router.get('highAvailabilityWizardStep3Controller').onLoadConfigsTags.apply(this, [].slice.call(arguments));
  },

  onLoadConfigs: function onLoadConfigs(data) {
    var self = this;
    var configController = App.router.get('highAvailabilityWizardStep3Controller');
    var configItems = data.items.map(function (item) {
      var fileName = Em.get(item, 'type');
      var configTypeObject = self.get('content.serviceConfigProperties').items.findProperty('type', fileName);
      if (configTypeObject) {
        $.extend(item.properties, configTypeObject.properties);
      }
      return item;
    });
    configItems = configController.removeConfigs(configController.get('configsToRemove'), { items: configItems });
    this.updateConfigProperties(configItems);
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/step6_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep6Controller = Em.Controller.extend({

  name: "highAvailabilityWizardStep6Controller",

  POLL_INTERVAL: 1000,

  MINIMAL_JOURNALNODE_COUNT: 3,

  /**
   * Define whether next button should be enabled
   * @type {Boolean}
   */
  isNextEnabled: false,

  /**
   * Counter of initialized JournalNodes
   * @type {Number}
   */
  initJnCounter: 0,

  /**
   * Counter of completed requests
   * @type {Number}
   */
  requestsCounter: 0,

  /**
   * Define whether are some not started JournalNodes
   * @type {Boolean}
   */
  hasStoppedJNs: false,

  /**
   * Current status for step.
   * waiting - for waiting for initializing
   * done - for completed initializing check
   * journalnode_stopped - if there are stopped JournalNode
   * @type {String}
   */
  status: 'waiting',

  loadStep: function loadStep() {
    this.set('status', 'waiting');
    this.set('isNextEnabled', false);
    this.pullCheckPointStatus();
  },

  pullCheckPointStatus: function pullCheckPointStatus() {
    this.set('initJnCounter', 0);
    this.set('requestsCounter', 0);
    this.set('hasStoppedJNs', false);
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', "JOURNALNODE").mapProperty('hostName');
    hostNames.forEach(function (hostName) {
      this.pullEachJnStatus(hostName);
    }, this);
  },

  pullEachJnStatus: function pullEachJnStatus(hostName) {
    return App.ajax.send({
      name: 'admin.high_availability.getJnCheckPointStatus',
      sender: this,
      data: {
        hostName: hostName
      },
      success: 'checkJnCheckPointStatus'
    });
  },

  checkJnCheckPointStatus: function checkJnCheckPointStatus(data) {
    var journalStatusInfo;
    var initJnCounter = 0;

    if (data.metrics && data.metrics.dfs) {
      journalStatusInfo = $.parseJSON(data.metrics.dfs.journalnode.journalsStatus);
      if (journalStatusInfo[this.get('content.nameServiceId')] && journalStatusInfo[this.get('content.nameServiceId')].Formatted === "true") {
        initJnCounter = this.incrementProperty('initJnCounter');
      }
    } else {
      this.set('hasStoppedJNs', true);
    }

    if (this.incrementProperty('requestsCounter') === this.MINIMAL_JOURNALNODE_COUNT) {
      this.resolveJnCheckPointStatus(initJnCounter);
    }
  },

  resolveJnCheckPointStatus: function resolveJnCheckPointStatus(initJnCounter) {
    var self = this;
    if (initJnCounter === this.MINIMAL_JOURNALNODE_COUNT) {
      this.set('status', 'done');
      this.set('isNextEnabled', true);
    } else {
      if (this.get('hasStoppedJNs')) {
        this.set('status', 'journalnode_stopped');
      } else {
        this.set('status', 'waiting');
      }
      window.setTimeout(function () {
        self.pullCheckPointStatus();
      }, self.POLL_INTERVAL);
    }
  },

  done: function done() {
    if (this.get('isNextEnabled')) {
      App.router.send('next');
    }
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step7_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep7Controller = App.HighAvailabilityProgressPageController.extend({

  name: "highAvailabilityWizardStep7Controller",

  commands: ['startZooKeeperServers', 'startAmbariInfra', 'startMysqlServer', 'startRanger', 'startNameNode'],

  initializeTasks: function initializeTasks() {
    this._super();
    var tasksToRemove = [];

    if (!App.Service.find('AMBARI_INFRA_SOLR').get('isLoaded')) {
      tasksToRemove.push('startAmbariInfra');
    }

    if (App.ClientComponent.getModelByComponentName('RANGER_ADMIN').get('installedCount') === 0) {
      tasksToRemove.push('startRanger');
    }

    if (App.ClientComponent.getModelByComponentName('MYSQL_SERVER').get('installedCount') === 0) {
      tasksToRemove.push('startMysqlServer');
    }

    this.removeTasks(tasksToRemove);
  },

  startAmbariInfra: function startAmbariInfra() {
    this.startServices(false, ['AMBARI_INFRA_SOLR'], true);
  },

  startRanger: function startRanger() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'RANGER_ADMIN').mapProperty('hostName');
    if (hostNames.length) {
      this.updateComponent('RANGER_ADMIN', hostNames, "RANGER", "Start");
    }
  },

  startMysqlServer: function startMysqlServer() {
    var hostNames = App.MasterComponent.find('MYSQL_SERVER').get('hostNames');
    this.updateComponent('MYSQL_SERVER', hostNames, "HIVE", "Start");
  },

  startZooKeeperServers: function startZooKeeperServers() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'ZOOKEEPER_SERVER').mapProperty('hostName');
    this.updateComponent('ZOOKEEPER_SERVER', hostNames, "ZOOKEEPER", "Start");
  },

  startNameNode: function startNameNode() {
    var hostName = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true).hostName;
    this.updateComponent('NAMENODE', hostName, "HDFS", "Start");
  }
});

});

require.register("controllers/main/admin/highAvailability/nameNode/step8_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep8Controller = Em.Controller.extend({

  name: "highAvailabilityWizardStep8Controller",

  done: function done() {
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      App.router.send("next");
    });
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/step9_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep9Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "highAvailabilityWizardStep9Controller",

  commands: ['startSecondNameNode', 'installZKFC', 'startZKFC', 'installPXF', 'reconfigureRanger', 'reconfigureHBase', 'reconfigureAMS', 'reconfigureAccumulo', 'reconfigureHawq', 'deleteSNameNode', 'stopHDFS', 'startAllServices'],

  hbaseSiteTag: "",
  accumuloSiteTag: "",
  hawqSiteTag: "",
  secondNameNodeHost: "",

  initializeTasks: function initializeTasks() {
    this._super();
    var tasksToRemove = [];

    // find hostname where second namenode will be installed
    this.set('secondNameNodeHost', this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).hostName);

    if (!App.Service.find().someProperty('serviceName', 'PXF') || this.isPxfComponentInstalled()) {
      tasksToRemove.push('installPXF');
    }
    if (!App.Service.find().someProperty('serviceName', 'RANGER')) {
      tasksToRemove.push('reconfigureRanger');
    }
    if (!App.Service.find().someProperty('serviceName', 'HBASE')) {
      tasksToRemove.push('reconfigureHBase');
    }
    if (!App.Service.find().someProperty('serviceName', 'AMBARI_METRICS')) {
      tasksToRemove.push('reconfigureAMS');
    }
    if (!App.Service.find().someProperty('serviceName', 'ACCUMULO')) {
      tasksToRemove.push('reconfigureAccumulo');
    }
    if (!App.Service.find().someProperty('serviceName', 'HAWQ')) {
      tasksToRemove.push('reconfigureHawq');
    }
    this.removeTasks(tasksToRemove);
  },

  startSecondNameNode: function startSecondNameNode() {
    var hostName = this.get('secondNameNodeHost');
    this.updateComponent('NAMENODE', hostName, "HDFS", "Start");
  },

  installZKFC: function installZKFC() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    this.createInstallComponentTask('ZKFC', hostNames, "HDFS");
  },

  startZKFC: function startZKFC() {
    var hostNames = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName');
    this.updateComponent('ZKFC', hostNames, "HDFS", "Start");
  },

  isPxfComponentInstalled: function isPxfComponentInstalled() {
    var pxfComponent = this.getSlaveComponentHosts().findProperty('componentName', 'PXF');

    if (pxfComponent !== undefined) {
      var host;
      // check if PXF is already installed on the host assigned for additional NameNode
      for (var i = 0; i < pxfComponent.hosts.length; i++) {
        host = pxfComponent.hosts[i];
        if (host.hostName === this.get('secondNameNodeHost')) return true;
      }
    }

    return false;
  },

  installPXF: function installPXF() {
    this.createInstallComponentTask('PXF', this.get('secondNameNodeHost'), "PXF");
  },

  reconfigureRanger: function reconfigureRanger() {
    var data = this.get('content.serviceConfigProperties');
    var siteNames = ['ranger-env'];
    var configs = [];
    configs.push({
      Clusters: {
        desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
      }
    });
    if (App.Service.find().someProperty('serviceName', 'YARN')) {
      siteNames = [];
      var yarnAuditConfig = data.items.findProperty('type', 'ranger-yarn-audit');
      if (yarnAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in yarnAuditConfig.properties) {
          siteNames.push('ranger-yarn-audit');
          configs.push({
            Clusters: {
              desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
            }
          });
        }
      }
    }
    if (App.Service.find().someProperty('serviceName', 'STORM')) {
      siteNames = [];
      var stormPluginConfig = data.items.findProperty('type', 'ranger-storm-plugin-properties');
      if (stormPluginConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in stormPluginConfig.properties) {
          siteNames.push('ranger-storm-plugin-properties');
        }
      }
      var stormAuditConfig = data.items.findProperty('type', 'ranger-storm-audit');
      if (stormAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in stormAuditConfig.properties) {
          siteNames.push('ranger-storm-audit');
        }
      }
      if (siteNames.length) {
        configs.push({
          Clusters: {
            desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
          }
        });
      }
    }
    if (App.Service.find().someProperty('serviceName', 'KAFKA')) {
      siteNames = [];
      var kafkaAuditConfig = data.items.findProperty('type', 'ranger-kafka-audit');
      if (kafkaAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in kafkaAuditConfig.properties) {
          siteNames.push('ranger-kafka-audit');
          configs.push({
            Clusters: {
              desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
            }
          });
        }
      }
    }
    if (App.Service.find().someProperty('serviceName', 'KNOX')) {
      siteNames = [];
      var knoxPluginConfig = data.items.findProperty('type', 'ranger-knox-plugin-properties');
      if (knoxPluginConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in knoxPluginConfig.properties) {
          siteNames.push('ranger-knox-plugin-properties');
        }
      }
      var knoxAuditConfig = data.items.findProperty('type', 'ranger-knox-audit');
      if (knoxAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in knoxAuditConfig.properties) {
          siteNames.push('ranger-knox-audit');
        }
      }
      if (siteNames.length) {
        configs.push({
          Clusters: {
            desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
          }
        });
      }
    }
    if (App.Service.find().someProperty('serviceName', 'ATLAS')) {
      siteNames = [];
      var atlasAuditConfig = data.items.findProperty('type', 'ranger-atlas-audit');
      if (atlasAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in atlasAuditConfig.properties) {
          siteNames.push('ranger-atlas-audit');
          configs.push({
            Clusters: {
              desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
            }
          });
        }
      }
    }
    if (App.Service.find().someProperty('serviceName', 'HIVE')) {
      siteNames = [];
      var hivePluginConfig = data.items.findProperty('type', 'ranger-hive-plugin-properties');
      if (hivePluginConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hivePluginConfig.properties) {
          siteNames.push('ranger-hive-plugin-properties');
        }
      }
      var hiveAuditConfig = data.items.findProperty('type', 'ranger-hive-audit');
      if (hiveAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hiveAuditConfig.properties) {
          siteNames.push('ranger-hive-audit');
        }
      }
      if (siteNames.length) {
        configs.push({
          Clusters: {
            desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
          }
        });
      }
    }
    if (App.Service.find().someProperty('serviceName', 'RANGER_KMS')) {
      siteNames = [];
      var rangerKMSConfig = data.items.findProperty('type', 'ranger-kms-audit');
      if (rangerKMSConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in rangerKMSConfig.properties) {
          siteNames.push('ranger-kms-audit');
          configs.push({
            Clusters: {
              desired_config: this.reconfigureSites(siteNames, data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false)))
            }
          });
        }
      }
    }

    App.ajax.send({
      name: 'common.service.multiConfigurations',
      sender: this,
      data: {
        configs: configs
      },
      success: 'saveConfigTag',
      error: 'onTaskError'
    });
  },

  /**
   *
   * @param {array} siteNames
   * @param {object} data
   */
  saveReconfiguredConfigs: function saveReconfiguredConfigs(siteNames, data) {
    var note = Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('NAMENODE', false));
    var configData = this.reconfigureSites(siteNames, data, note);
    return App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'saveConfigTag',
      error: 'onTaskError'
    });
  },

  reconfigureHBase: function reconfigureHBase() {
    var data = this.get('content.serviceConfigProperties');
    var siteNames = ['hbase-site'];
    if (App.Service.find().someProperty('serviceName', 'RANGER')) {
      var hbasePluginConfig = data.items.findProperty('type', 'ranger-hbase-plugin-properties');
      if (hbasePluginConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hbasePluginConfig.properties) {
          siteNames.push('ranger-hbase-plugin-properties');
        }
      }
      var hbaseAuditConfig = data.items.findProperty('type', 'ranger-hbase-audit');
      if (hbaseAuditConfig) {
        if ('xasecure.audit.destination.hdfs.dir' in hbaseAuditConfig.properties) {
          siteNames.push('ranger-hbase-audit');
        }
      }
    }
    this.saveReconfiguredConfigs(siteNames, data);
  },

  reconfigureAMS: function reconfigureAMS() {
    this.saveReconfiguredConfigs(['ams-hbase-site'], this.get('content.serviceConfigProperties'));
  },

  reconfigureAccumulo: function reconfigureAccumulo() {
    this.saveReconfiguredConfigs(['accumulo-site'], this.get('content.serviceConfigProperties'));
  },

  reconfigureHawq: function reconfigureHawq() {
    this.saveReconfiguredConfigs(['hawq-site', 'hdfs-client'], this.get('content.serviceConfigProperties'));
  },

  saveConfigTag: function saveConfigTag() {
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'HIGH_AVAILABILITY_DEPLOY',
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    });
    this.onTaskCompleted();
  },

  startAllServices: function startAllServices() {
    this.startServices(false);
  },

  stopHDFS: function stopHDFS() {
    this.stopServices(["HDFS"], true);
  },

  deleteSNameNode: function deleteSNameNode() {
    var hostName = this.get('content.masterComponentHosts').findProperty('component', 'SECONDARY_NAMENODE').hostName;
    return App.ajax.send({
      name: 'common.delete.host_component',
      sender: this,
      data: {
        componentName: 'SECONDARY_NAMENODE',
        hostName: hostName
      },
      success: 'onTaskCompleted',
      error: 'onTaskError'
    });
  }

});

});

require.register("controllers/main/admin/highAvailability/nameNode/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardController = App.WizardController.extend({

  name: 'highAvailabilityWizardController',

  totalSteps: 9,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.highAvailability.wizard.header'),

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  content: Em.Object.create({
    controllerName: 'highAvailabilityWizardController',
    cluster: null,
    hosts: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: [],
    serviceName: 'MISC',
    hdfsUser: "hdfs",
    nameServiceId: '',
    failedTask: null,
    requestIds: null
  }),

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'HIGH_AVAILABILITY_DEPLOY',
      wizardControllerName: 'highAvailabilityWizardController',
      localdb: App.db.data
    });
  },

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.router.getClusterName() });
  },

  /**
   * save status of the cluster.
   * @param clusterStatus object with status,requestId fields.
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    var oldStatus = this.toObject(this.get('content.cluster'));
    clusterStatus = jQuery.extend(oldStatus, clusterStatus);
    if (clusterStatus.requestId) {
      clusterStatus.requestId.forEach(function (requestId) {
        if (clusterStatus.oldRequestsId.indexOf(requestId) === -1) {
          clusterStatus.oldRequestsId.push(requestId);
        }
      }, this);
    }
    this.set('content.cluster', clusterStatus);
    this.save('cluster');
  },

  saveHdfsUser: function saveHdfsUser() {
    App.db.setHighAvailabilityWizardHdfsUser(this.get('content.hdfsUser'));
  },

  saveTasksStatuses: function saveTasksStatuses(statuses) {
    App.db.setHighAvailabilityWizardTasksStatuses(statuses);
    this.set('content.tasksStatuses', statuses);
  },

  saveConfigTag: function saveConfigTag(tag) {
    App.db.setHighAvailabilityWizardConfigTag(tag);
    this.set('content.' + [tag.name], tag.value);
  },

  saveHdfsClientHosts: function saveHdfsClientHosts(hostNames) {
    App.db.setHighAvailabilityWizardHdfsClientHosts(hostNames);
    this.set('content.hdfsClientHostNames', hostNames);
  },

  /**
   * Save config properties
   * @param stepController HighAvailabilityWizardStep3Controller
   */
  saveServiceConfigProperties: function saveServiceConfigProperties(stepController) {
    var serviceConfigProperties = [];
    var data = stepController.get('serverConfigData');

    var _content = stepController.get('stepConfigs')[0];
    _content.get('configs').forEach(function (_configProperties) {
      var siteObj = data.items.findProperty('type', _configProperties.get('filename'));
      if (siteObj) {
        if (_configProperties.get('name') == 'xasecure.audit.destination.hdfs.dir') {
          if ('xasecure.audit.destination.hdfs.dir' in siteObj.properties) {
            siteObj.properties[_configProperties.get('name')] = _configProperties.get('value');
          }
        } else {
          siteObj.properties[_configProperties.get('name')] = _configProperties.get('value');
        }
      }
    }, this);
    this.setDBProperty('serviceConfigProperties', data);
    this.set('content.serviceConfigProperties', data);
  },

  loadHdfsClientHosts: function loadHdfsClientHosts() {
    var hostNames = App.db.getHighAvailabilityWizardHdfsClientHosts();
    if (!(hostNames instanceof Array)) {
      hostNames = [hostNames];
    }
    this.set('content.hdfsClientHostNames', hostNames);
  },

  loadConfigTag: function loadConfigTag(tag) {
    var tagVal = App.db.getHighAvailabilityWizardConfigTag(tag);
    this.set('content.' + tag, tagVal);
  },

  loadHdfsUser: function loadHdfsUser() {
    var hdfsUser = App.db.getHighAvailabilityWizardHdfsUser();
    this.set('content.hdfsUser', hdfsUser);
  },

  loadTasksStatuses: function loadTasksStatuses() {
    var statuses = App.db.getHighAvailabilityWizardTasksStatuses();
    this.set('content.tasksStatuses', statuses);
  },

  /**
   * Load serviceConfigProperties to model
   */
  loadServiceConfigProperties: function loadServiceConfigProperties() {
    var serviceConfigProperties = this.getDBProperty('serviceConfigProperties');
    this.set('content.serviceConfigProperties', serviceConfigProperties);
  },

  saveRequestIds: function saveRequestIds(requestIds) {
    App.db.setHighAvailabilityWizardRequestIds(requestIds);
    this.set('content.requestIds', requestIds);
  },

  loadRequestIds: function loadRequestIds() {
    var requestIds = App.db.getHighAvailabilityWizardRequestIds();
    this.set('content.requestIds', requestIds);
  },

  saveNameServiceId: function saveNameServiceId(nameServiceId) {
    App.db.setHighAvailabilityWizardNameServiceId(nameServiceId);
    this.set('content.nameServiceId', nameServiceId);
  },

  loadNameServiceId: function loadNameServiceId() {
    var nameServiceId = App.db.getHighAvailabilityWizardNameServiceId();
    this.set('content.nameServiceId', nameServiceId);
  },

  saveTasksRequestIds: function saveTasksRequestIds(requestIds) {
    App.db.setHighAvailabilityWizardTasksRequestIds(requestIds);
    this.set('content.tasksRequestIds', requestIds);
  },

  loadTasksRequestIds: function loadTasksRequestIds() {
    var requestIds = App.db.getHighAvailabilityWizardTasksRequestIds();
    this.set('content.tasksRequestIds', requestIds);
  },

  loadMap: {
    '1': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred(),
            self = this,
            usersLoadingCallback = function usersLoadingCallback() {
          self.saveHdfsUser();
          self.load('cluster');
          dfd.resolve();
        };
        if (App.db.getHighAvailabilityWizardHdfsUser()) {
          usersLoadingCallback();
        } else {
          this.loadHdfsUserFromServer().done(function (data) {
            self.set('content.hdfsUser', Em.get(data, '0.properties.hdfs_user'));
            usersLoadingCallback();
          });
        }
        return dfd.promise();
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        var self = this;
        this.loadHosts().done(function () {
          self.loadServicesFromServer();
          self.loadMasterComponentHosts().done(function () {
            self.loadHdfsUser();
            dfd.resolve();
          });
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadNameServiceId();
        this.loadServiceConfigProperties();
      }
    }],
    '5': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
      }
    }]
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearTasksData: function clearTasksData() {
    this.saveTasksStatuses(undefined);
    this.saveRequestIds(undefined);
    this.saveTasksRequestIds(undefined);
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    App.db.data.Installer = {};
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
  }
});

});

require.register("controllers/main/admin/highAvailability/progress_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityProgressPageController = App.HighAvailabilityWizardController.extend(App.wizardProgressPageControllerMixin, {

  name: 'highAvailabilityProgressPageController',
  clusterDeployState: 'HIGH_AVAILABILITY_DEPLOY',
  tasksMessagesPrefix: 'admin.highAvailability.wizard.step',
  isRollback: false,

  manualRollback: function manualRollback() {
    App.ModalPopup.show({
      header: Em.I18n.t('admin.highAvailability.confirmRollbackHeader'),
      primary: Em.I18n.t('yes'),
      showCloseButton: false,
      onPrimary: function onPrimary() {
        var self = this;
        var controller = App.router.get('highAvailabilityWizardController');
        controller.clearTasksData();
        controller.clearStorageData();
        controller.resetOnClose(controller, 'main.services.index');
        this.hide();
      },
      secondary: Em.I18n.t('no'),
      onSecondary: function onSecondary() {
        this.hide();
      },
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile(Em.I18n.t('admin.highAvailability.confirmManualRollbackBody'))
      })
    });
  },

  rollback: function rollback() {
    var task = this.get('tasks').findProperty('status', 'FAILED');
    App.router.get(this.get('content.controllerName')).saveFailedTask(task);
    App.ModalPopup.show({
      header: Em.I18n.t('admin.highAvailability.confirmRollbackHeader'),
      primary: Em.I18n.t('common.confirm'),
      showCloseButton: false,
      onPrimary: function onPrimary() {
        App.router.get('highAvailabilityWizardController').clearTasksData();
        App.router.transitionTo('main.admin.highAvailabilityRollback');
        this.hide();
      },
      secondary: Em.I18n.t('common.cancel'),
      body: Em.I18n.t('admin.highAvailability.confirmRollbackBody')
    });
  },

  /**
   * Prepare object to send to the server to save configs
   * Split all configs by site names and note
   * @param siteNames Array
   * @param data Object
   * @param note String
   */
  reconfigureSites: function reconfigureSites(siteNames, data, note) {
    return siteNames.map(function (_siteName) {
      var config = data.items.findProperty('type', _siteName);
      var configToSave = {
        type: _siteName,
        properties: config && config.properties,
        service_config_version_note: note || ''
      };
      if (config && config.properties_attributes) {
        configToSave.properties_attributes = config.properties_attributes;
      }
      return configToSave;
    });
  }
});

});

require.register("controllers/main/admin/highAvailability/progress_popup_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.HighAvailabilityProgressPopupController = Ember.Controller.extend({

  name: 'highAvailabilityProgressPopupController',

  /**
   * Id of current request
   * @type {Array}
   */
  requestIds: [],

  /**
   * Title for popup header
   * @type {String}
   */
  popupTitle: '',

  /**
   * Array with Hosts tasks data used in <code>App.HostPopup</code>
   * @type {Array}
   */
  services: [],

  /**
   * Timestamp used in <code>App.HostPopup</code>
   * @type {Number}
   */
  serviceTimestamp: null,

  /**
   * Progress controller. Used to get tasks data.
   * @type {App.HighAvailabilityProgressPageController}
   */
  progressController: null,

  /**
   * Requests data with tasks
   * @type {Array}
   */
  hostsData: [],

  /**
   *  StageId for the command.
   *  @type {Number}
   */
  stageId: null,

  /**
   * During loading and calculations show popup with spinner
   * @type {Object}
   */
  spinnerPopup: null,

  isTaskPolling: false,

  taskInfo: null,

  /**
   * Get info for <code>requestIds</code> and initialize <code>App.HostPopup</code>
   * @param popupTitle {String}
   * @param requestIds {Array}
   * @param progressController {App.HighAvailabilityProgressPageController}
   * @param showSpinner {Boolean}
   * @param stageId {Number}
   */
  initPopup: function initPopup(popupTitle, requestIds, progressController, showSpinner, stageId) {
    if (showSpinner) {
      var loadingPopup = App.ModalPopup.show({
        header: Em.I18n.t('common.loading.eclipses'),
        primary: false,
        secondary: false,
        bodyClass: Ember.View.extend({
          template: Ember.Handlebars.compile('{{view App.SpinnerView}}')
        })
      });
      this.set('spinnerPopup', loadingPopup);
    }
    this.setProperties({
      progressController: progressController,
      popupTitle: popupTitle,
      requestIds: requestIds,
      hostsData: [],
      stageId: stageId
    });
    this.getHosts();
  },

  /**
   * Send AJAX request to get hosts tasks data
   */
  getHosts: function getHosts(successCallback) {
    var requestIds = this.get('requestIds');
    var stageId = this.get('stageId');
    var name = 'background_operations.get_by_request';
    if (!Em.isNone(stageId)) {
      name = 'common.request.polling';
      if (stageId === 0) {
        stageId = '0';
      }
    }
    if (Em.isNone(successCallback)) {
      successCallback = 'onGetHostsSuccess';
    }
    requestIds.forEach(function (requestId) {
      App.ajax.send({
        name: name,
        sender: this,
        data: {
          requestId: requestId,
          stageId: stageId
        },
        success: successCallback
      });
    }, this);
  },

  doPolling: function doPolling() {
    var self = this;
    this.set('progressController.logs', []);
    setTimeout(function () {
      self.getHosts('doPollingSuccessCallback');
    }, App.bgOperationsUpdateInterval);
  },

  doPollingSuccessCallback: function doPollingSuccessCallback(data) {
    this.set('hostsData', [data]);
    var hostsData = this.get('hostsData');
    this.set('progressController.logs', data.tasks);
    if (this.isRequestRunning(hostsData)) {
      this.doPolling();
    }
  },

  /**
   * Callback for <code>getHosts</code> request
   * @param data
   */
  onGetHostsSuccess: function onGetHostsSuccess(data) {
    var hostsData = this.get('hostsData');
    hostsData.push(data);
    if (this.get('requestIds.length') === this.get('hostsData.length')) {
      var popupTitle = this.get('popupTitle');
      this.calculateHostsData(hostsData);
      App.HostPopup.initPopup(popupTitle, this, false, this.get("requestIds")[0]);
      if (this.isRequestRunning(hostsData)) {
        if (this.get('progressController.name') === 'mainAdminStackAndUpgradeController') {
          this.doPolling();
        }
        this.addObserver('progressController.logs.length', this, 'getDataFromProgressController');
      }
    }
    if (this.get('spinnerPopup')) {
      this.get('spinnerPopup').hide();
      this.set('spinnerPopup', null);
    }
  },

  /**
   * Convert data to format used in <code>App.HostPopup</code>
   * @param data {Array}
   */
  calculateHostsData: function calculateHostsData(data) {
    var hosts = [];
    var hostsMap = {};
    var popupTitle = this.get('popupTitle');
    var id = this.get('requestIds')[0];

    data.forEach(function (request) {
      request.tasks.forEach(function (task) {
        var host = task.Tasks.host_name;
        if (hostsMap[host]) {
          hostsMap[host].logTasks.push(task);
        } else {
          hostsMap[host] = {
            name: task.Tasks.host_name,
            publicName: task.Tasks.host_name,
            logTasks: [task]
          };
        }
      });
    });
    for (var host in hostsMap) {
      hosts.push(hostsMap[host]);
    }
    this.set('services', [{ id: id, name: popupTitle, hosts: hosts }]);
    this.set('serviceTimestamp', App.dateTime());
    if (!this.isRequestRunning(data)) {
      this.removeObserver('progressController.logs.length', this, 'getDataFromProgressController');
    }
  },

  /**
   * Get hosts tasks data from <code>progressController</code>
   */
  getDataFromProgressController: function getDataFromProgressController() {
    var data = this.get('hostsData');
    var tasksData = [];
    var stageId = this.get('stageId');
    // If the progress page is broken into stages then update host with only stage's tasks
    if (!!stageId) {
      tasksData = this.get('progressController.logs').filterProperty('Tasks.stage_id', stageId);
    } else {
      tasksData = this.get('progressController.logs');
    }
    if (tasksData.length) {
      var tasks = [];
      tasksData.forEach(function (logs) {
        tasks.pushObjects(logs);
      }, this);
      data.forEach(function (request) {
        tasks = tasks.filterProperty('Tasks.request_id', request.Requests.id);
        request.tasks = tasks;
      });
      this.calculateHostsData(data);
    }
  },

  /**
   * Identify whether request is running by task counters
   * @param requests {Array}
   * @return {Boolean}
   */
  isRequestRunning: function isRequestRunning(requests) {
    var result = false;
    requests.forEach(function (request) {
      if (request.Requests.task_count - (request.Requests.aborted_task_count + request.Requests.completed_task_count + request.Requests.failed_task_count + request.Requests.timed_out_task_count - request.Requests.queued_task_count) > 0) {
        result = true;
      }
    });
    return result;
  },

  startTaskPolling: function startTaskPolling(requestId, taskId) {
    this.setProperties({
      isTaskPolling: true,
      taskInfo: {
        id: taskId,
        requestId: requestId
      }
    });
    App.updater.run(this, 'updateTask', 'isTaskPolling', App.bgOperationsUpdateInterval);
    App.updater.immediateRun('updateTask');
  },

  stopTaskPolling: function stopTaskPolling() {
    this.set('isTaskPolling', false);
  },

  updateTask: function updateTask(callback) {
    App.ajax.send({
      name: 'background_operations.get_by_task',
      sender: this,
      data: {
        requestId: this.get('taskInfo.requestId'),
        taskId: this.get('taskInfo.id')
      },
      success: 'updateTaskSuccessCallback',
      callback: callback
    });
  },

  updateTaskSuccessCallback: function updateTaskSuccessCallback(data) {
    this.setProperties({
      'taskInfo.stderr': data.Tasks.stderr,
      'taskInfo.stdout': data.Tasks.stdout,
      'taskInfo.outputLog': data.Tasks.output_log,
      'taskInfo.errorLog': data.Tasks.error_log,
      'isTaskPolling': !['FAILED', 'COMPLETED', 'TIMEDOUT', 'ABORTED'].contains(data.Tasks.status)
    });
  }

});

});

require.register("controllers/main/admin/highAvailability/rangerAdmin/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.RAHighAvailabilityWizardStep1Controller = Em.Controller.extend({
  name: "rAHighAvailabilityWizardStep1Controller",

  /**
   * Define if typed load balancer URL is valid URL
   * @type {Boolean}
   */
  isloadBalancerURLValid: function () {
    return validator.isValidURL(this.get('content.loadBalancerURL'));
  }.property('content.loadBalancerURL'),

  /**
   * Define if show load balancer URL error
   * do not show is input-field is empty
   * @type {Boolean}
   */
  showloadBalancerURLError: Em.computed.and('content.loadBalancerURL', '!isloadBalancerURLValid'),

  /**
   * Define either Submit is disabled or enabled
   * @type {Boolean}
   */
  isSubmitDisabled: Em.computed.not('isloadBalancerURLValid')
});

});

require.register("controllers/main/admin/highAvailability/rangerAdmin/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep2Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {
  name: "rAHighAvailabilityWizardStep2Controller",

  useServerValidation: false,

  mastersToShow: ['RANGER_ADMIN'],

  mastersToAdd: ['RANGER_ADMIN'],

  showCurrentPrefix: ['RANGER_ADMIN'],

  showAdditionalPrefix: ['RANGER_ADMIN'],

  mastersAddableInHA: ['RANGER_ADMIN'],

  showInstalledMastersFirst: true,

  generalWarningMessages: [Em.I18n.t('admin.ra_highAvailability.wizard.step2.warning')]

});

});

require.register("controllers/main/admin/highAvailability/rangerAdmin/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep3Controller = Em.Controller.extend({
  name: 'rAHighAvailabilityWizardStep3Controller',

  isLoaded: false,

  versionLoaded: true,

  hideDependenciesInfoBar: true,

  stepConfigs: [App.ServiceConfig.create({
    serviceName: 'MISC',
    showConfig: true
  })],

  loadStep: function loadStep() {
    var self = this;
    App.get('router.mainController.isLoading').call(App.get('router.clusterController'), 'isConfigsPropertiesLoaded').done(function () {
      var stepConfig = self.get('stepConfigs.firstObject'),
          configs = [],
          configCategories = [],
          installedServices = App.Service.find().mapProperty('serviceName');
      self.get('wizardController.configs').forEach(function (config) {
        var service = App.config.get('serviceByConfigTypeMap')[config.siteName];
        if (service) {
          var serviceName = service.get('serviceName'),
              serviceDisplayName = service.get('displayName');
          if (installedServices.contains(serviceName)) {
            var property = App.configsCollection.getConfigByName(config.propertyName, config.siteName) || {};
            if (!configCategories.someProperty('name'), serviceName) {
              configCategories.push(App.ServiceConfigCategory.create({
                name: serviceName,
                displayName: serviceDisplayName
              }));
            }
            configs.push(App.ServiceConfigProperty.create(property, {
              category: serviceName,
              value: self.get('content.loadBalancerURL'),
              isEditable: false
            }));
          }
        }
      });
      stepConfig.setProperties({
        configs: configs,
        configCategories: configCategories
      });
      self.setProperties({
        isLoaded: true,
        selectedService: stepConfig
      });
    });
  }
});

});

require.register("controllers/main/admin/highAvailability/rangerAdmin/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep4Controller = App.HighAvailabilityProgressPageController.extend({

  name: "rAHighAvailabilityWizardStep4Controller",

  clusterDeployState: 'RA_HIGH_AVAILABILITY_DEPLOY',

  commands: ['stopAllServices', 'installRangerAdmin', 'reconfigureRanger', 'startAllServices'],

  tasksMessagesPrefix: 'admin.ra_highAvailability.wizard.step',

  stopAllServices: function stopAllServices() {
    this.stopServices([], true, true);
  },

  installRangerAdmin: function installRangerAdmin() {
    var hostNames = this.get('content.raHosts.additionalRA');
    this.createInstallComponentTask('RANGER_ADMIN', hostNames, "RANGER");
  },

  reconfigureRanger: function reconfigureRanger() {
    this.loadConfigsTags();
  },

  loadConfigsTags: function loadConfigsTags() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadConfigsTags',
      error: 'onTaskError'
    });
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var urlParams = [];
    var siteNamesToFetch = this.get('wizardController.configs').mapProperty('siteName');
    siteNamesToFetch.map(function (siteName) {
      if (siteName in data.Clusters.desired_configs) {
        urlParams.push('(type=' + siteName + '&tag=' + data.Clusters.desired_configs[siteName].tag + ')');
      }
    });
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: urlParams.join('|')
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data) {
    var configs = [];

    this.get('wizardController.configs').map(function (item) {
      var config = data.items.findProperty('type', item.siteName);
      if (config) {
        config.properties[item.propertyName] = this.get('content.loadBalancerURL');
        configs.push({
          Clusters: {
            desired_config: this.reconfigureSites([item.siteName], data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('RANGER_ADMIN', false)))
          }
        });
      }
    }, this);

    App.ajax.send({
      name: 'common.service.multiConfigurations',
      sender: this,
      data: {
        configs: configs
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },

  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  startAllServices: function startAllServices() {
    this.startServices(true);
  }
});

});

require.register("controllers/main/admin/highAvailability/rangerAdmin/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardController = App.WizardController.extend({

  name: 'rAHighAvailabilityWizardController',

  totalSteps: 4,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.ra_highAvailability.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'rAHighAvailabilityWizardController',
    cluster: null,
    loadBalancerURL: null,
    hosts: null,
    services: null,
    masterComponentHosts: null
  }),

  configs: [{
    siteName: 'admin-properties',
    propertyName: 'policymgr_external_url'
  }, {
    siteName: 'ranger-hdfs-security',
    propertyName: 'ranger.plugin.hdfs.policy.rest.url'
  }, {
    siteName: 'ranger-yarn-security',
    propertyName: 'ranger.plugin.yarn.policy.rest.url'
  }, {
    siteName: 'ranger-hbase-security',
    propertyName: 'ranger.plugin.hbase.policy.rest.url'
  }, {
    siteName: 'ranger-hive-security',
    propertyName: 'ranger.plugin.hive.policy.rest.url'
  }, {
    siteName: 'ranger-knox-security',
    propertyName: 'ranger.plugin.knox.policy.rest.url'
  }, {
    siteName: 'ranger-kafka-security',
    propertyName: 'ranger.plugin.kafka.policy.rest.url'
  }, {
    siteName: 'ranger-kms-security',
    propertyName: 'ranger.plugin.kms.policy.rest.url'
  }, {
    siteName: 'ranger-storm-security',
    propertyName: 'ranger.plugin.storm.policy.rest.url'
  }, {
    siteName: 'ranger-atlas-security',
    propertyName: 'ranger.plugin.atlas.policy.rest.url'
  }],

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'rAHighAvailabilityWizardController',
      localdb: App.db.data
    });
  },

  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
        this.load('loadBalancerURL');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        var self = this;
        this.loadHosts().done(function () {
          self.loadServicesFromServer();
          self.loadMasterComponentHosts().done(function () {
            dfd.resolve();
          });
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.load('raHosts');
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
      }
    }]
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }
});

});

require.register("controllers/main/admin/highAvailability/resourceManager/step1_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep1Controller = Em.Controller.extend({
  name: "rMHighAvailabilityWizardStep1Controller"
});

});

require.register("controllers/main/admin/highAvailability/resourceManager/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep2Controller = Em.Controller.extend(App.AssignMasterComponents, {

  name: "rMHighAvailabilityWizardStep2Controller",

  useServerValidation: false,

  mastersToShow: ['RESOURCEMANAGER'],

  mastersToAdd: ['RESOURCEMANAGER'],

  showCurrentPrefix: ['RESOURCEMANAGER'],

  showAdditionalPrefix: ['RESOURCEMANAGER'],

  showInstalledMastersFirst: true
});

});

require.register("controllers/main/admin/highAvailability/resourceManager/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} rmHaConfigDependencies
 * @property {string|number} webAddressPort
 * @property {string|number} httpsWebAddressPort
 * @property {string|number} zkClientPort
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');
require('utils/configs/rm_ha_config_initializer');

App.RMHighAvailabilityWizardStep3Controller = Em.Controller.extend(App.BlueprintMixin, {
  name: "rMHighAvailabilityWizardStep3Controller",

  selectedService: null,

  versionLoaded: true,

  hideDependenciesInfoBar: true,

  isLoaded: false,

  isSubmitDisabled: Em.computed.not('isLoaded'),

  loadStep: function loadStep() {
    this.renderConfigs();
  },

  /**
   * Render configs to show them in <code>App.ServiceConfigView</code>
   */
  renderConfigs: function renderConfigs() {

    var configs = $.extend(true, {}, require('data/configs/wizards/rm_ha_properties').haConfig);

    var serviceConfig = App.ServiceConfig.create({
      serviceName: configs.serviceName,
      displayName: configs.displayName,
      configCategories: [],
      showConfig: true,
      configs: []
    });

    configs.configCategories.forEach(function (configCategory) {
      if (App.Service.find().someProperty('serviceName', configCategory.name)) {
        serviceConfig.configCategories.pushObject(configCategory);
      }
    }, this);

    this.renderConfigProperties(configs, serviceConfig);
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'loadConfigTagsSuccessCallback',
      error: 'loadConfigsErrorCallback',
      data: {
        serviceConfig: serviceConfig
      }
    });
  },

  loadConfigTagsSuccessCallback: function loadConfigTagsSuccessCallback(data, opt, params) {
    var urlParams = '(type=zoo.cfg&tag=' + data.Clusters.desired_configs['zoo.cfg'].tag + ')|' + '(type=yarn-site&tag=' + data.Clusters.desired_configs['yarn-site'].tag + ')|' + '(type=yarn-env&tag=' + data.Clusters.desired_configs['yarn-env'].tag + ')';
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: urlParams,
        serviceConfig: params.serviceConfig
      },
      success: 'loadConfigsSuccessCallback',
      error: 'loadConfigsSuccessCallback'
    });
  },

  loadConfigsSuccessCallback: function loadConfigsSuccessCallback(data, opt, params) {
    var self = this;
    var blueprintConfigurations = Em.getWithDefault(data || {}, 'items', []).reduce(function (prev, cur) {
      prev[cur.type] = { properties: cur.properties };
      return prev;
    }, {});
    params = params.serviceConfig ? params.serviceConfig : arguments[4].serviceConfig;
    this.setDynamicConfigValues(params, data);
    this.loadRecommendations(blueprintConfigurations).always(function (recommendations) {
      self.applyRecommendedConfigurations(recommendations, data, params);
      self.setProperties({
        selectedService: params,
        isLoaded: true
      });
    });
  },

  /**
   * Get dependencies for new configs
   *
   * @param {{items: object[]}} data
   * @returns {rmHaConfigDependencies}
   * @private
   * @method _prepareDependencies
   */
  _prepareDependencies: function _prepareDependencies(data) {
    var ret = {};
    var zooCfg = data && data.items ? data.items.findProperty('type', 'zoo.cfg') : null;
    var yarnSite = data && data.items ? data.items.findProperty('type', 'yarn-site') : null;
    var portValue = zooCfg && Em.get(zooCfg, 'properties.clientPort');
    var webAddressPort = yarnSite && yarnSite.properties ? yarnSite.properties['yarn.resourcemanager.webapp.address'] : '';
    var httpsWebAddressPort = yarnSite && yarnSite.properties ? yarnSite.properties['yarn.resourcemanager.webapp.https.address'] : '';
    var trackerAddressPort = yarnSite && yarnSite.properties ? yarnSite.properties['yarn.resourcemanager.resource-tracker.address'] : '';

    ret.webAddressPort = webAddressPort && webAddressPort.contains(':') ? webAddressPort.split(':')[1] : '8088';
    ret.httpsWebAddressPort = httpsWebAddressPort && httpsWebAddressPort.contains(':') ? httpsWebAddressPort.split(':')[1] : '8090';
    ret.trackerAddressPort = trackerAddressPort && trackerAddressPort.contains(':') ? trackerAddressPort.split(':')[1] : '8025';
    ret.zkClientPort = portValue ? portValue : '2181';
    return ret;
  },

  /**
   * Set values to the new configs
   *
   * @param {object} configs
   * @param {object} data
   * @returns {object}
   * @method setDynamicConfigValues
   */
  setDynamicConfigValues: function setDynamicConfigValues(configs, data) {
    var topologyLocalDB = this.get('content').getProperties(['masterComponentHosts', 'slaveComponentHosts', 'hosts']);
    var yarnUser = data.items.findProperty('type', 'yarn-env').properties.yarn_user;
    App.RmHaConfigInitializer.setup({
      yarnUser: yarnUser
    });
    var dependencies = this._prepareDependencies(data);
    // /** add dynamic property 'hadoop.proxyuser.' + yarnUser + '.hosts' **/
    // var proxyUserConfig = App.ServiceConfigProperty.create(App.config.createDefaultConfig('hadoop.proxyuser.' + yarnUser + '.hosts',
    //   'core-site', false,  {category : "HDFS", isUserProperty: false, isEditable: false, isOverridable: false, serviceName: 'MISC'}));
    // configs.configs.pushObject(proxyUserConfig);

    configs.configs.forEach(function (config) {
      App.RmHaConfigInitializer.initialValue(config, topologyLocalDB, dependencies);
    });
    App.RmHaConfigInitializer.cleanup();
    return configs;
  },

  /**
   * Load child components to service config object
   * @param _componentConfig
   * @param componentConfig
   */
  renderConfigProperties: function renderConfigProperties(_componentConfig, componentConfig) {
    _componentConfig.configs.forEach(function (_serviceConfigProperty) {
      var serviceConfigProperty = App.ServiceConfigProperty.create(_serviceConfigProperty);
      componentConfig.configs.pushObject(serviceConfigProperty);
      serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isReconfigurable'));
    }, this);
  },

  submit: function submit() {
    if (!this.get('isSubmitDisabled')) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        App.router.send("next");
      });
    }
  },

  loadRecommendations: function loadRecommendations(blueprintConfigurations) {
    var blueprint = this.getCurrentMasterSlaveBlueprint();
    // host group where new ResourceManager will be added
    var hostGroupName = blueprintUtils.getHostGroupByFqdn(blueprint, this.get('content.rmHosts.additionalRM'));
    var dataToSend = {
      recommend: 'configurations',
      hosts: App.get('allHostNames'),
      services: App.Service.find().mapProperty('serviceName').uniq(),
      recommendations: {}
    };
    if (!!hostGroupName) {
      blueprintUtils.addComponentToHostGroup(blueprint, 'RESOURCEMANAGER', hostGroupName);
    }
    blueprint.blueprint.configurations = blueprintConfigurations;
    dataToSend.recommendations = blueprint;
    return App.ajax.send({
      name: 'config.recommendations',
      sender: this,
      data: {
        stackVersionUrl: App.get('stackVersionURL'),
        dataToSend: dataToSend
      }
    });
  },

  applyRecommendedConfigurations: function applyRecommendedConfigurations(recommendations, configurations, stepConfigs) {
    var yarnEnv = Em.getWithDefault(configurations || {}, 'items', []).findProperty('type', 'yarn-env') || {},
        yarnUser = Em.getWithDefault(yarnEnv, 'properties.yarn_user', false),
        coreSite = Em.getWithDefault(recommendations, 'resources.0.recommendations.blueprint.configurations.core-site.properties', {}),
        proxyHostName = 'hadoop.proxyuser.' + yarnUser + '.hosts',
        recommendedHosts = coreSite[proxyHostName] || false,
        newProp;

    if (yarnUser && recommendedHosts) {
      if (stepConfigs.get('configs').someProperty('name', proxyHostName)) {
        stepConfigs.get('configs').findProperty('name', proxyHostName).setProperties({
          recommendedValue: recommendedHosts,
          value: recommendedHosts
        });
      } else {
        newProp = App.config.createDefaultConfig(proxyHostName, 'core-site', false, {
          category: "HDFS",
          isUserProperty: false,
          isEditable: false,
          isOverridable: false,
          serviceName: 'MISC',
          value: recommendedHosts,
          recommendedValue: recommendedHosts
        });
        newProp.filename = App.config.getConfigTagFromFileName(newProp.filename);
        stepConfigs.get('configs').pushObject(App.ServiceConfigProperty.create(newProp));
      }
    }
  }
});

});

require.register("controllers/main/admin/highAvailability/resourceManager/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/main/admin/serviceAccounts_controller');

App.RMHighAvailabilityWizardStep4Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "rMHighAvailabilityWizardStep4Controller",

  clusterDeployState: 'RM_HIGH_AVAILABILITY_DEPLOY',

  commands: ['stopRequiredServices', 'installResourceManager', 'reconfigureYARN', 'reconfigureHAWQ', 'reconfigureHDFS', 'startAllServices'],

  tasksMessagesPrefix: 'admin.rm_highAvailability.wizard.step',

  initializeTasks: function initializeTasks() {
    this._super();
    var tasksToRemove = [];
    if (!App.Service.find().someProperty('serviceName', 'HAWQ')) {
      tasksToRemove.push('reconfigureHAWQ');
    }
    this.removeTasks(tasksToRemove);
  },

  stopRequiredServices: function stopRequiredServices() {
    this.stopServices(['HDFS']);
  },

  installResourceManager: function installResourceManager() {
    var hostName = this.get('content.rmHosts.additionalRM');
    this.createInstallComponentTask('RESOURCEMANAGER', hostName, "YARN");
  },

  reconfigureYARN: function reconfigureYARN() {
    this.loadConfigsTags("Yarn");
  },

  reconfigureHAWQ: function reconfigureHAWQ() {
    this.loadConfigsTags("Hawq");
  },

  reconfigureHDFS: function reconfigureHDFS() {
    this.loadConfigsTags("Hdfs");
  },

  loadConfigsTags: function loadConfigsTags(service) {
    var onLoadServiceConfigsTags = 'onLoad' + service + "ConfigsTags";
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: onLoadServiceConfigsTags,
      error: 'onTaskError'
    });
  },

  onLoadYarnConfigsTags: function onLoadYarnConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=yarn-site&tag=' + data.Clusters.desired_configs['yarn-site'].tag + ')',
        type: 'yarn-site'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadHawqConfigsTags: function onLoadHawqConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=yarn-client&tag=' + data.Clusters.desired_configs['yarn-client'].tag + ')',
        type: 'yarn-client'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadHdfsConfigsTags: function onLoadHdfsConfigsTags(data) {
    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: '(type=core-site&tag=' + data.Clusters.desired_configs['core-site'].tag + ')',
        type: 'core-site'
      },
      success: 'onLoadConfigs',
      error: 'onTaskError'
    });
  },

  onLoadConfigs: function onLoadConfigs(data, opt, params) {
    var propertiesToAdd = this.get('content.configs').filterProperty('filename', params.type);
    propertiesToAdd.forEach(function (property) {
      data.items[0].properties[property.name] = property.value;
    });

    var configData = this.reconfigureSites([params.type], data, Em.I18n.t('admin.highAvailability.step4.save.configuration.note').format(App.format.role('RESOURCEMANAGER', false)));

    App.ajax.send({
      name: 'common.service.configurations',
      sender: this,
      data: {
        desired_config: configData
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },
  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  startAllServices: function startAllServices() {
    this.startServices(true);
  }
});

});

require.register("controllers/main/admin/highAvailability/resourceManager/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardController = App.WizardController.extend({

  name: 'rMHighAvailabilityWizardController',

  totalSteps: 4,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.rm_highAvailability.wizard.header'),

  isFinished: false,

  content: Em.Object.create({
    controllerName: 'rMHighAvailabilityWizardController'
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var self = this,
            dfd = $.Deferred();
        this.loadRmHosts();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          self.loadConfirmedHosts();
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
        this.loadConfigs();
      }
    }]
  },

  init: function init() {
    this._super();
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('isFinished', false);
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'rMHighAvailabilityWizardController',
      localdb: App.db.data
    });
  },

  /**
   * Save hosts for additional and current ResourceManagers to local db and <code>controller.content</code>
   * @param rmHosts
   */
  saveRmHosts: function saveRmHosts(rmHosts) {
    this.set('content.rmHosts', rmHosts);
    this.setDBProperty('rmHosts', rmHosts);
  },

  /**
   * Load hosts for additional and current ResourceManagers from local db to <code>controller.content</code>
   */
  loadRmHosts: function loadRmHosts() {
    var rmHosts = this.getDBProperty('rmHosts');
    this.set('content.rmHosts', rmHosts);
  },

  /**
   * Save configs to load and apply them on Configure Components step
   * @param configs
   */
  saveConfigs: function saveConfigs(configs) {
    this.set('content.configs', configs);
    this.setDBProperty('configs', configs);
  },

  /**
   * Load configs to apply them on Configure Components step
   */
  loadConfigs: function loadConfigs() {
    var configs = this.getDBProperty('configs');
    this.set('content.configs', configs);
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
    this.set('isFinished', true);
  }
});

});

require.register("controllers/main/admin/highAvailability_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminHighAvailabilityController = App.WizardController.extend({
  name: 'mainAdminHighAvailabilityController',

  tag: null,

  dataIsLoaded: false,

  /**
   * enable High Availability
   * @return {Boolean}
   */
  enableHighAvailability: function enableHighAvailability() {
    var message = [];
    var hostComponents = App.HostComponent.find();

    if (hostComponents.findProperty('componentName', 'NAMENODE').get('workStatus') !== 'STARTED') {
      message.push(Em.I18n.t('admin.highAvailability.error.namenodeStarted'));
    }
    if (hostComponents.filterProperty('componentName', 'ZOOKEEPER_SERVER').length < 3) {
      message.push(Em.I18n.t('admin.highAvailability.error.zooKeeperNum'));
    }
    if (hostComponents.filterProperty('isMaster', true).someProperty('passiveState', "ON") || hostComponents.filterProperty('isMaster', true).someProperty('isImpliedState', true)) {
      message.push(Em.I18n.t('admin.highAvailability.error.maintenanceMode'));
    }

    if (App.get('allHostNames.length') < 3) {
      message.push(Em.I18n.t('admin.highAvailability.error.hostsNum'));
    }
    if (message.length > 0) {
      this.showErrorPopup(message);
      return false;
    }
    App.router.transitionTo('main.services.enableHighAvailability');
    return true;
  },

  disableHighAvailability: function disableHighAvailability() {
    App.router.transitionTo('main.admin.rollbackHighAvailability');
  },

  /**
   * enable ResourceManager High Availability
   * @return {Boolean}
   */
  enableRMHighAvailability: function enableRMHighAvailability() {
    //Prerequisite Checks
    var message = [];

    if (App.HostComponent.find().findProperty('componentName', 'RESOURCEMANAGER').get('workStatus') !== 'STARTED') {
      message.push(Em.I18n.t('admin.rm_highAvailability.error.resourceManagerStarted'));
    }
    if (App.HostComponent.find().filterProperty('componentName', 'ZOOKEEPER_SERVER').length < 3) {
      message.push(Em.I18n.t('admin.rm_highAvailability.error.zooKeeperNum'));
    }

    if (App.get('allHostNames.length') < 3) {
      message.push(Em.I18n.t('admin.rm_highAvailability.error.hostsNum'));
    }
    if (message.length > 0) {
      this.showErrorPopup(message);
      return false;
    }
    App.router.transitionTo('main.services.enableRMHighAvailability');
    return true;
  },

  /**
   * add Hawq Standby
   * @return {Boolean}
   */
  addHawqStandby: function addHawqStandby() {
    App.router.transitionTo('main.services.addHawqStandby');
    return true;
  },

  /**
   * remove Hawq Standby
   * @return {Boolean}
   */
  removeHawqStandby: function removeHawqStandby() {
    App.router.transitionTo('main.services.removeHawqStandby');
    return true;
  },

  /**
   * activate Hawq Standby
   * @return {Boolean}
   */
  activateHawqStandby: function activateHawqStandby() {
    App.router.transitionTo('main.services.activateHawqStandby');
    return true;
  },

  /**
   * enable Ranger Admin High Availability
   * @return {Boolean}
   */
  enableRAHighAvailability: function enableRAHighAvailability() {
    App.router.transitionTo('main.services.enableRAHighAvailability');
    return true;
  },

  /**
   * enable NameNode Federation
   * @return {Boolean}
   */
  enableNameNodeFederation: function enableNameNodeFederation() {
    //Prerequisite Checks
    var message = [];
    if (!App.HostComponent.find().filterProperty('componentName', 'ZOOKEEPER_SERVER').everyProperty('workStatus', 'STARTED')) {
      message.push(Em.I18n.t('admin.nameNodeFederation.wizard.required.zookeepers'));
    }

    if (!App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').everyProperty('workStatus', 'STARTED')) {
      message.push(Em.I18n.t('admin.nameNodeFederation.wizard.required.journalnodes'));
    }
    if (message.length > 0) {
      this.showErrorPopup(message);
      return false;
    }
    App.router.transitionTo('main.services.enableNameNodeFederation');
    return true;
  },

  /**
   * open Manage JournalNode Wizard if there are two started NameNodes with defined active/standby state
   * @returns {boolean}
   */
  manageJournalNode: function manageJournalNode() {
    var nameNodes = App.HostComponent.find().filterProperty('componentName', 'NAMENODE');
    if (nameNodes.someProperty('displayNameAdvanced', 'Active NameNode') && nameNodes.someProperty('displayNameAdvanced', 'Standby NameNode')) {
      App.router.transitionTo('main.services.manageJournalNode');
      return true;
    }
    this.showErrorPopup(Em.I18n.t('admin.manageJournalNode.warning'));
    return false;
  },

  /**
   * join or wrap message depending on whether it is array or string
   * @param message
   * @return {*}
   */
  joinMessage: function joinMessage(message) {
    if (Array.isArray(message)) {
      return message.join('<br/>');
    } else {
      return '<p>' + message + '</p>';
    }
  },

  showErrorPopup: function showErrorPopup(message) {
    message = this.joinMessage(message);
    App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile(message)
      }),
      secondary: false
    });
  }
});

});

require.register("controllers/main/admin/kerberos", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/main/admin/kerberos/step4_controller');

App.MainAdminKerberosController = App.KerberosWizardStep4Controller.extend({
  name: 'mainAdminKerberosController',

  /**
   * @type {boolean}
   * @deafult false
   */
  securityEnabled: false,

  /**
   * @type {boolean}
   * @default false
   */
  defaultKerberosLoaded: false,

  /**
   * @type {boolean}
   * @default false
   */
  dataIsLoaded: false,

  /**
   * @type {boolean}
   * @default true
   */
  isRecommendedLoaded: true,

  /**
   * @type {boolean}
   * @default false
   */
  isEditMode: false,

  /**
   * @type {string}
   */
  kdc_type: '',

  kdcTypesValues: {
    'mit-kdc': Em.I18n.t('admin.kerberos.wizard.step1.option.kdc'),
    'active-directory': Em.I18n.t('admin.kerberos.wizard.step1.option.ad'),
    'ipa': Em.I18n.t('admin.kerberos.wizard.step1.option.ipa'),
    'none': Em.I18n.t('admin.kerberos.wizard.step1.option.manual')
  },

  getAddSecurityWizardStatus: function getAddSecurityWizardStatus() {
    return App.db.getSecurityWizardStatus();
  },
  setAddSecurityWizardStatus: function setAddSecurityWizardStatus(status) {
    App.db.setSecurityWizardStatus(status);
  },

  setDisableSecurityStatus: function setDisableSecurityStatus(status) {
    App.db.setDisableSecurityStatus(status);
  },
  getDisableSecurityStatus: function getDisableSecurityStatus(status) {
    return App.db.getDisableSecurityStatus();
  },

  notifySecurityOff: false,
  notifySecurityAdd: false,

  notifySecurityOffPopup: function notifySecurityOffPopup() {
    var self = this;
    this.checkServiceWarnings().then(function () {
      App.ModalPopup.show({
        header: Em.I18n.t('popup.confirmation.commonHeader'),
        primary: Em.I18n.t('ok'),
        onPrimary: function onPrimary() {
          App.db.setSecurityDeployCommands(undefined);
          self.setDisableSecurityStatus("RUNNING");
          App.router.transitionTo('disableSecurity');
          this.hide();
        },
        bodyClass: Ember.View.extend({
          templateName: require('templates/main/admin/kerberos/notify_security_off_popup')
        })
      });
    });
  },

  /**
   * Show confirmation popup for regenerate keytabs
   * @method regenerateKeytabs
   * @param callback function (optional)
   * @return {App.ModalPopup}
   */
  regenerateKeytabs: function regenerateKeytabs(callback) {
    var self = this;

    return App.ModalPopup.show({

      /**
       * True - regenerate keytabs only for missing hosts and components, false - regenerate for all hosts and components
       * @type {boolean}
       */
      regenerateKeytabsOnlyForMissing: false,

      header: Em.I18n.t('admin.kerberos.button.regenerateKeytabs'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/admin/kerberos/regenerate_keytabs_popup_body')
      }),

      onPrimary: function onPrimary() {
        this._super();
        return self.restartServicesAfterRegenerate(this.get('regenerateKeytabsOnlyForMissing'), callback);
      }
    });
  },

  /**
   * Show confirmation popup for restarting all services and after confirmation regenerate keytabs
   *
   * @param regenerateKeytabsOnlyForMissing {Boolean}
   * @param callback (optional)
   * @returns {*}
   */
  restartServicesAfterRegenerate: function restartServicesAfterRegenerate(regenerateKeytabsOnlyForMissing, callback) {
    var self = this;

    return App.ModalPopup.show({

      /**
       * True - automatically restart services, false - user will have to restart required services manually
       * @type {boolean}
       */
      restartComponents: false,

      header: Em.I18n.t('admin.kerberos.button.regenerateKeytabs'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/admin/kerberos/restart_services_after_regenerate_body')
      }),

      onPrimary: function onPrimary() {
        this._super();
        var popupContext = this;
        // Keytabs can either be regenerated directly or after updating kerberos descriptor in the callback function
        if (Em.typeOf(callback) === 'function') {
          callback().done(function () {
            self.regenerateKeytabsRequest(regenerateKeytabsOnlyForMissing, popupContext.get('restartComponents'));
          });
        } else {
          self.regenerateKeytabsRequest(regenerateKeytabsOnlyForMissing, popupContext.get('restartComponents'));
        }
      }
    });
  },

  /**
   * Send request to regenerate keytabs
   * @param {boolean} missingOnly determines type of regeneration - missing|all
   * @param {boolean} withAutoRestart determines if the system should automatically restart all services or not after regeneration
   * @returns {$.ajax}
   */
  regenerateKeytabsRequest: function regenerateKeytabsRequest(missingOnly, withAutoRestart) {
    missingOnly = missingOnly || false;

    return App.ajax.send({
      name: "admin.kerberos_security.regenerate_keytabs",
      sender: this,
      data: {
        type: missingOnly ? 'missing' : 'all',
        withAutoRestart: withAutoRestart || false
      },
      success: "regenerateKeytabsSuccess"
    });
  },

  /**
   * Success callback of <code>regenerateKeytabs</code>
   * show background operations popup if appropriate option is set
   *
   * @param data
   * @param opt
   * @param params
   * @param request
   */
  regenerateKeytabsSuccess: function regenerateKeytabsSuccess(data, opt, params, request) {
    var self = this;
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
      self.set('needsRestartAfterRegenerate', params.withAutoRestart);
    });
  },

  /**
   * Do request to server for restarting all services
   * @method restartAllServices
   * @return {$.ajax}
   */
  restartAllServices: function () {
    if (!App.router.get('backgroundOperationsController.runningOperationsCount')) {
      if (this.get('needsRestartAfterRegenerate')) {
        this.set('needsRestartAfterRegenerate', false);
        App.router.get('mainServiceController').restartAllServices();
      }
    }
  }.observes('controllers.backgroundOperationsController.runningOperationsCount'),

  /**
   * performs cluster check before kerbefos security
   * wizard starts if <code>preKerberizeCheck<code> supports is true
   * otherwise runs <code>startKerberosWizard<code>
   * @method checkAndStartKerberosWizard
   */
  checkAndStartKerberosWizard: function checkAndStartKerberosWizard() {
    if (App.get('supports.preKerberizeCheck')) {
      App.ajax.send({
        name: "admin.kerberos_security.checks",
        sender: this,
        success: "runSecurityCheckSuccess"
      });
    } else {
      this.startKerberosWizard();
    }
  },

  /**
   * success callback of <code>checkAndStartKerberosWizard()</code>
   * if there are some fails - it shows popup else open security wizard
   * @param data {object}
   * @param opt {object}
   * @param params {object}
   */
  runSecurityCheckSuccess: function runSecurityCheckSuccess(data, opt, params) {
    //TODO correct check
    if (data.items.someProperty('UpgradeChecks.status', "FAIL")) {
      var hasFails = data.items.someProperty('UpgradeChecks.status', 'FAIL'),
          header = Em.I18n.t('popup.clusterCheck.Security.header').format(params.label),
          title = Em.I18n.t('popup.clusterCheck.Security.title'),
          alert = Em.I18n.t('popup.clusterCheck.Security.alert');

      App.showClusterCheckPopup(data, {
        header: header,
        failTitle: title,
        failAlert: alert,
        noCallbackCondition: hasFails
      });
    } else {
      this.startKerberosWizard();
    }
  },

  startKerberosWizard: function startKerberosWizard() {
    var self = this;
    this.checkServiceWarnings().then(function () {
      self.setAddSecurityWizardStatus('RUNNING');
      App.router.get('kerberosWizardController').setDBProperty('onClosePath', 'main.admin.adminKerberos.index');
      App.router.transitionTo('adminKerberos.adminAddKerberos');
    });
  },

  /**
   * Loads the security status from server (security_enabled property in cluster-env configuration)
   */
  loadSecurityStatusFromServer: function loadSecurityStatusFromServer() {
    if (App.get('testMode')) {
      this.set('securityEnabled', !App.get('testEnableSecurity'));
      this.set('dataIsLoaded', true);
    } else {
      //get Security Status From Server
      this.getSecurityType();
      return this.getSecurityStatus();
    }
  },

  /**
   * Load security status from server.
   * @returns {$.Deferred}
   */
  getSecurityStatus: function getSecurityStatus() {
    var self = this;
    var dfd = $.Deferred();
    if (App.get('testMode')) {
      this.set('securityEnabled', !App.get('testEnableSecurity'));
      this.set('dataIsLoaded', true);
      dfd.resolve();
    } else {
      //get Security Status From Server
      App.ajax.send({
        name: 'admin.security_status',
        sender: this,
        success: 'getSecurityStatusSuccessCallback',
        error: 'errorCallback'
      }).always(function () {
        self.getSecurityType(function () {
          dfd.resolve();
        });
      });
    }
    return dfd.promise();
  },

  getSecurityStatusSuccessCallback: function getSecurityStatusSuccessCallback(data) {
    this.set('dataIsLoaded', true);
    var securityType = data.Clusters.security_type;
    this.set('securityEnabled', securityType === 'KERBEROS');
  },

  errorCallback: function errorCallback(jqXHR) {
    this.set('dataIsLoaded', true);
    // Show the error popup if the API call received a response from the server.
    // jqXHR.status will be empty when browser cancels the request. Refer to AMBARI-5921 for more info
    if (!!jqXHR.status) {
      this.showSecurityErrorPopup();
    }
  },

  showSecurityErrorPopup: function showSecurityErrorPopup() {
    App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile('<p>{{t admin.security.status.error}}</p>')
      })
    });
  },

  /**
   * Override <code>App.KerberosWizardStep4Controller</code>
   */
  clearStep: function clearStep() {
    this.set('isEditMode', false);
    this._super();
  },

  /**
   * Override <code>App.KerberosWizardStep4Controller</code>
   *
   * @param {App.ServiceConfigProperty[]} properties
   */
  setStepConfigs: function setStepConfigs(properties) {
    this.get('stepConfigs').clear();
    this._super(properties);
    this.set('selectedService', this.get('stepConfigs')[0]);
    this.get('stepConfigs').forEach(function (serviceConfig) {
      serviceConfig.set('initConfigsLength', serviceConfig.get('configs.length'));
    });
  },

  /**
   * Override <code>App.KerberosWizardStep4Controller</code>
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @returns {App.ServiceConfigProperty[]}
   */
  prepareConfigProperties: function prepareConfigProperties(configs) {
    var self = this;
    var configProperties = configs.slice(0);
    var siteProperties = App.configsCollection.getAll();
    var installedServiceNames = ['Cluster'].concat(App.Service.find().mapProperty('serviceName'));
    configProperties = configProperties.filter(function (item) {
      return installedServiceNames.contains(item.get('serviceName'));
    });
    configProperties.setEach('isSecureConfig', false);
    configProperties.forEach(function (property, item, allConfigs) {
      if (['spnego_keytab', 'spnego_principal'].contains(property.get('name'))) {
        property.addObserver('value', self, 'spnegoPropertiesObserver');
      }
      if (property.get('observesValueFrom')) {
        var observedValue = allConfigs.findProperty('name', property.get('observesValueFrom')).get('value');
        property.set('value', observedValue);
        property.set('recommendedValue', observedValue);
      }
      if (property.get('serviceName') == 'Cluster') {
        property.set('category', 'Global');
      } else {
        property.set('category', property.get('serviceName'));
      }
      // All user identity should be grouped under "Ambari Principals" category
      if (property.get('identityType') == 'user') property.set('category', 'Ambari Principals');
      var siteProperty = siteProperties.findProperty('name', property.get('name'));
      if (siteProperty) {
        if (siteProperty.category === property.get('category')) {
          property.set('displayName', siteProperty.displayName);
          if (siteProperty.index) {
            property.set('index', siteProperty.index);
          }
        }
        if (siteProperty.displayType) {
          property.set('displayType', siteProperty.displayType);
        }
      }
    });
    configProperties.setEach('isEditable', false);
    return configProperties;
  },

  getKDCSessionState: function getKDCSessionState(callback, kdcCancelHandler) {
    var self = this;
    if (this.get('securityEnabled') || App.get('isKerberosEnabled')) {
      this.getSecurityType(function () {
        if (!self.get('isManualKerberos')) {
          App.ajax.send({
            name: 'kerberos.session.state',
            sender: self,
            data: {
              callback: callback
            },
            success: 'checkState',
            kdcCancelHandler: kdcCancelHandler
          });
        } else {
          callback();
        }
      });
    } else {
      callback();
    }
  },

  /**
   * Determines security type.
   *
   * @param {function} [callback] callback function to execute
   * @returns {$.Deferred|null}
   */
  getSecurityType: function getSecurityType(callback) {
    if (this.get('securityEnabled') || App.get('isKerberosEnabled')) {
      if (!this.get('kdc_type')) {
        return App.ajax.send({
          name: 'admin.security.cluster_configs.kerberos',
          sender: this,
          data: {
            clusterName: App.get('clusterName'),
            additionalCallback: callback
          },
          success: 'getSecurityTypeSuccess'
        });
      } else {
        if (Em.typeOf(callback) === 'function') {
          callback();
        }
        return $.Deferred().resolve().promise;
      }
    } else if (Em.typeOf(callback) === 'function') {
      callback();
    } else {
      return $.Deferred().resolve().promise;
    }
  },

  getSecurityTypeSuccess: function getSecurityTypeSuccess(data, opt, params) {
    var kdcType = data.items && data.items[0] && Em.getWithDefault(Em.getWithDefault(data.items[0], 'configurations', []).findProperty('type', 'kerberos-env') || {}, 'properties.kdc_type', 'none') || 'none';
    this.set('kdc_type', kdcType);
    if (Em.typeOf(params.additionalCallback) === 'function') {
      params.additionalCallback();
    }
  },

  isManualKerberos: Em.computed.equal('kdc_type', 'none'),

  checkState: function checkState(data, opt, params) {
    var res = Em.get(data, 'Services.attributes.kdc_validation_result');
    var message = Em.get(data, 'Services.attributes.kdc_validation_failure_details');
    if (res.toUpperCase() === "OK") {
      params.callback();
    } else {
      App.showInvalidKDCPopup(opt, App.format.kdcErrorMsg(message, false));
    }
  },

  /**
   * Determines if some config value is changed
   * @type {boolean}
   */
  isPropertiesChanged: Em.computed.someBy('stepConfigs', 'isPropertiesChanged', true),

  /**
   * Determines if the save button is disabled
   */
  isSaveButtonDisabled: Em.computed.or('isSubmitDisabled', '!isPropertiesChanged'),

  /**
   * Determines if the `Disbale Kerberos` and `Regenerate Keytabs` button are disabled
   */
  isKerberosButtonsDisabled: Em.computed.not('isSaveButtonDisabled'),

  makeConfigsEditable: function makeConfigsEditable() {
    if (this.get('stepConfigs') && this.get('stepConfigs.length')) {
      this.set('isEditMode', true);
      this.get('stepConfigs').forEach(function (_stepConfig) {
        _stepConfig.get('configs').setEach('isEditable', true);
        _stepConfig.get('configs').forEach(function (_config) {
          _config.set('isEditable', _config.get('name') != 'realm');
        });
      }, this);
    }
  },

  _updateConfigs: function _updateConfigs() {
    this.makeConfigsUneditable(true);
  },

  /**
   * @method makeConfigsUneditable
   * @param configsUpdated
   */
  makeConfigsUneditable: function makeConfigsUneditable(configsUpdated) {
    this.set('isEditMode', false);
    this.get('stepConfigs').forEach(function (_stepConfig) {
      _stepConfig.get('configs').forEach(function (_config) {
        if (configsUpdated === true) {
          // configsUpdated should be checked for boolean true
          _config.set('savedValue', _config.get('value'));
          _config.set('defaultValue', _config.get('value'));
        } else {
          _config.set('value', _config.get('savedValue') || _config.get('defaultValue'));
        }
        _config.set('isEditable', false);
      });
    }, this);
  },

  /**
   * Update kerberos descriptor and regenerate keytabs
   */
  submit: function submit(context) {
    var callback;
    var self = this;
    var kerberosDescriptor = this.get('kerberosDescriptor');
    var configs = [];
    this.get('stepConfigs').forEach(function (_stepConfig) {
      configs = configs.concat(_stepConfig.get('configs'));
    });
    callback = function callback() {
      return App.ajax.send({
        name: 'admin.kerberos.cluster.artifact.update',
        sender: self,
        data: {
          artifactName: 'kerberos_descriptor',
          data: {
            artifact_data: kerberosDescriptor
          }
        },
        success: '_updateConfigs',
        error: 'createKerberosDescriptor'
      });
    };
    this.updateKerberosDescriptor(kerberosDescriptor, configs);
    if (this.get('isManualKerberos')) {
      callback().done(function () {
        self.regenerateKeytabsRequest(false, false);
      });
    } else {
      this.restartServicesAfterRegenerate(false, callback);
    }
  },

  createKerberosDescriptor: function createKerberosDescriptor(requestData, ajaxOptions, error, opt, params) {
    if (requestData && requestData.status === 404) {
      var artifactName = params.artifactName,
          data = params.data;

      App.ajax.send({
        name: 'admin.kerberos.cluster.artifact.create',
        sender: self,
        data: {
          artifactName: artifactName,
          data: data
        },
        success: '_updateConfigs'
      });
    }
  },

  /**
   * List of the warnings regarding specific services before enabling/disabling Kerberos.
   *
   * @type {String[]}
   */
  serviceAlerts: function () {
    var messages = [];
    var serviceAlertMap = {
      YARN: Em.I18n.t('admin.kerberos.service.alert.yarn')
    };
    var installedServices = App.Service.find().mapProperty('serviceName');
    Em.keys(serviceAlertMap).forEach(function (serviceName) {
      if (installedServices.contains(serviceName)) {
        messages.push(serviceAlertMap[serviceName]);
      }
    });
    return messages;
  }.property(),

  /**
   * Check for additional info to display before enabling/disabling kerberos and show appropriate
   * messages in popup if needed.
   * @returns {$.Deferred} - promise
   */
  checkServiceWarnings: function checkServiceWarnings() {
    var dfd = $.Deferred();
    this.displayServiceWarnings(this.get('serviceAlerts'), dfd);
    return dfd.promise();
  },

  /**
   * Show appropriate message regarding changes affected after enabling/disabling Kerberos
   *
   * @param {String[]} messages - list of the messages to display
   * @param {$.Deferred} dfd - used to break recursive calls and reject/resolve promise returned by <code>checkServiceWarnings</code>
   */
  displayServiceWarnings: function displayServiceWarnings(messages, dfd) {
    var self = this;
    if (!messages.get('length')) {
      dfd.resolve();
    } else {
      App.showConfirmationPopup(function () {
        self.displayServiceWarnings(messages.slice(1), dfd);
      }, messages[0], function () {
        dfd.reject();
      }, Em.I18n.t('common.warning'), Em.I18n.t('common.proceedAnyway'));
    }
  },

  showManageKDCCredentialsPopup: function showManageKDCCredentialsPopup() {
    return App.showManageCredentialsPopup();
  },

  loadStep: function loadStep() {
    var self = this;
    if (this.get('isRecommendedLoaded') === false) {
      return;
    }
    this.clearStep();
    this.getDescriptor().then(function (properties) {
      self.setStepConfigs(self.createServicesStackDescriptorConfigs(properties));
    }).always(function () {
      self.set('isRecommendedLoaded', true);
    });
  },

  showDownloadCsv: function () {
    var hasUpgradePrivilege = App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK');
    return hasUpgradePrivilege;
  }.property(),

  downloadCSV: function downloadCSV() {
    App.router.get('kerberosWizardStep5Controller').getCSVData(false);
  }

});

});

require.register("controllers/main/admin/kerberos/disable_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/main/admin/kerberos/progress_controller');

App.KerberosDisableController = App.KerberosProgressPageController.extend(App.WizardEnableDone, {

  name: 'kerberosDisableController',
  clusterDeployState: 'DEFAULT',
  commands: ['startZookeeper', 'stopAllButZookeeper', 'unkerberize', 'deleteKerberos', 'startAllServices'],

  tasksMessagesPrefix: 'admin.kerberos.disable.step',

  loadStep: function loadStep() {
    this.set('content.controllerName', 'kerberosDisableController');
    this.loadTasksStatuses();
    this.loadTasksRequestIds();
    this.loadRequestIds();
    this._super();
  },

  startZookeeper: function startZookeeper() {
    this.startServices(false, ["ZOOKEEPER"], true);
  },

  stopAllButZookeeper: function stopAllButZookeeper() {
    this.stopServices(["ZOOKEEPER"], false);
  },

  unkerberize: function unkerberize() {
    return App.ajax.send({
      name: 'admin.unkerberize.cluster',
      sender: this,
      success: 'startPolling',
      error: 'onTaskErrorWithSkip'
    });
  },

  skipTask: function skipTask() {
    return App.ajax.send({
      name: 'admin.unkerberize.cluster.skip',
      sender: this,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  deleteKerberos: function deleteKerberos() {
    return App.ajax.send({
      name: 'common.delete.service',
      sender: this,
      data: {
        serviceName: 'KERBEROS'
      },
      success: 'onTaskCompleted',
      error: 'onTaskCompleted'
    });
  },

  startAllServices: function startAllServices() {
    this.startServices(true);
  }

});

});

require.register("controllers/main/admin/kerberos/progress_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.KerberosProgressPageController = App.KerberosWizardController.extend(App.wizardProgressPageControllerMixin, {

  name: 'kerberosProgressPageController',
  clusterDeployState: 'KERBEROS__DEPLOY',
  tasksMessagesPrefix: 'admin.kerberos.wizard.step',
  isRollback: false
});

});

require.register("controllers/main/admin/kerberos/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var PreCondition = Ember.Object.extend({
  displayText: null,
  checked: false,
  visibilityCriteria: function visibilityCriteria() {
    return true;
  },

  hidden: function () {
    return !this.get('visibilityCriteria')();
  }.property('visibilityCriteria'),

  satisfied: function () {
    return this.get('checked') || this.get('hidden');
  }.property('checked', 'hidden')
});

App.KerberosWizardStep1Controller = Em.Controller.extend({
  name: "kerberosWizardStep1Controller",

  selectedItem: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc'),

  isSubmitDisabled: Em.computed.someBy('selectedOption.preConditions', 'satisfied', false),

  options: Em.A([Em.Object.create({
    displayName: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc'),
    value: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc'),
    preConditions: [PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc.condition.1')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc.condition.2')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc.condition.3')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc.condition.4'),
      visibilityCriteria: function visibilityCriteria() {
        return App.Service.find().someProperty('serviceName', 'ONEFS');
      }
    })]
  }), Em.Object.create({
    displayName: Em.I18n.t('admin.kerberos.wizard.step1.option.ad'),
    value: Em.I18n.t('admin.kerberos.wizard.step1.option.ad'),
    preConditions: [PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ad.condition.1')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ad.condition.2')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ad.condition.3')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ad.condition.4')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ad.condition.5')
    })]
  }), Em.Object.create({
    displayName: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa'),
    value: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa'),
    preConditions: [PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa.condition.1')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa.condition.2')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa.condition.3')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa.condition.4')
    })]
  }), Em.Object.create({
    displayName: Em.I18n.t('admin.kerberos.wizard.step1.option.manual'),
    value: Em.I18n.t('admin.kerberos.wizard.step1.option.manual'),
    preConditions: [PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.manual.condition.1')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.manual.condition.2')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.manual.condition.3')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.manual.condition.4')
    }), PreCondition.create({
      displayText: Em.I18n.t('admin.kerberos.wizard.step1.option.manual.condition.5')
    })]
  })]),

  /**
   * precondition for the selected KDC option
   * whenever the KDC type is changed, all checkboxes for the precondition should be unchecked
   */
  selectedOption: function () {
    var options = this.get('options');
    options.forEach(function (option) {
      option.preConditions.setEach('checked', false);
    });
    return this.get('options').findProperty('value', this.get('selectedItem'));
  }.property('selectedItem'),

  loadStep: function loadStep() {},

  submit: function submit() {
    if (!this.get('isSubmitDisabled')) {
      App.router.send('next');
    }
  }
});

});

require.register("controllers/main/admin/kerberos/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/wizard/step7_controller');

App.KerberosWizardStep2Controller = App.WizardStep7Controller.extend(App.KDCCredentialsControllerMixin, {
  name: "kerberosWizardStep2Controller",

  isKerberosWizard: true,

  selectedServiceNames: ['KERBEROS'],

  allSelectedServiceNames: ['KERBEROS'],

  componentName: 'KERBEROS_CLIENT',

  installedServiceNames: [],

  servicesInstalled: false,

  addMiscTabToPage: false,

  kerberosConfigMap: {
    'ad': {
      configNames: ['ldap_url', 'container_dn', 'ad_create_attributes_template'],
      type: Em.I18n.t('admin.kerberos.wizard.step1.option.ad')
    },
    'mit': {
      configNames: ['kdc_create_attributes'],
      type: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc')
    },
    'ipa': {
      configNames: ['ipa_user_group'],
      type: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa')
    }
  },

  /**
   * @type {boolean} true if test connection to hosts is in progress
   */
  testConnectionInProgress: false,

  /**
   * Should Back-button be disabled
   * @type {boolean}
   */
  isBackBtnDisabled: Em.computed.or('testConnectionInProgress', 'App.router.nextBtnClickInProgress'),

  /**
   * Should Next-button be disabled
   * @type {boolean}
   */
  isSubmitDisabled: function () {
    if (!this.get('stepConfigs.length') || this.get('testConnectionInProgress') || this.get('submitButtonClicked') || App.get('router.nextBtnClickInProgress')) return true;
    return !this.get('stepConfigs').filterProperty('showConfig', true).everyProperty('errorCount', 0) || this.get("miscModalVisible");
  }.property('stepConfigs.@each.errorCount', 'miscModalVisible', 'submitButtonClicked', 'testConnectionInProgress', 'App.router.nextBtnClickInProgress'),

  hostNames: Em.computed.alias('App.allHostNames'),

  serviceConfigTags: [],

  clearStep: function clearStep() {
    this._super();
    this.set('configs', []);
    this.get('serviceConfigTags').clear();
    this.set('servicesInstalled', false);
  },

  isConfigsLoaded: Em.computed.alias('wizardController.stackConfigsLoaded'),

  /**
   * On load function
   * @method loadStep
   */

  loadStep: function loadStep() {
    if (!App.StackService.find().someProperty('serviceName', 'KERBEROS') || !this.get('isConfigsLoaded')) {
      return false;
    }
    this.clearStep();
    App.config.setPreDefinedServiceConfigs(this.get('addMiscTabToPage'));
    var stored = this.get('content.serviceConfigProperties');

    this.set('configs', stored ? App.config.mergeStoredValue(this.getKerberosConfigs(), stored) : this.getKerberosConfigs());

    this.filterConfigs(this.get('configs'));
    if (!this.get('wizardController.skipClientInstall')) {
      this.initializeKDCStoreProperties(this.get('configs'));
    }
    this.applyServicesConfigs(this.get('configs'));
    if (!this.get('wizardController.skipClientInstall')) {
      this.updateKDCStoreProperties(this.get('stepConfigs').findProperty('serviceName', 'KERBEROS').get('configs'));
    }
  },

  /**
   * @method getKerberosConfigs
   * @returns {Array.<T>|*}
   */
  getKerberosConfigs: function getKerberosConfigs() {
    var kerberosConfigTypes = Em.keys(App.config.get('preDefinedServiceConfigs').findProperty('serviceName', 'KERBEROS').get('configTypes'));

    return App.configsCollection.getAll().filter(function (configProperty) {
      var fileName = Em.getWithDefault(configProperty, 'fileName', false);
      var isService = ['KERBEROS'].contains(Em.get(configProperty, 'serviceName'));
      var isFileName = fileName && kerberosConfigTypes.contains(App.config.getConfigTagFromFileName(fileName));
      return isService || isFileName;
    });
  },

  /**
   * Make Active Directory or IPA specific configs visible if user has selected AD or IPA option
   * @param configs
   */
  filterConfigs: function filterConfigs(configs) {
    var kdcType = this.get('content.kerberosOption');
    var kerberosWizardController = this.get('controllers.kerberosWizardController');
    var manageIdentitiesConfig = configs.findProperty('name', 'manage_identities');
    configs.filterProperty('serviceName', 'KERBEROS').setEach('isVisible', true);
    this.setKDCTypeProperty(configs);
    if (kdcType !== Em.I18n.t('admin.kerberos.wizard.step1.option.ad')) {
      kerberosWizardController.overrideVisibility(configs, false, kerberosWizardController.get('exceptionsForNonAdOption'), true);
    }
    if (kdcType === Em.I18n.t('admin.kerberos.wizard.step1.option.manual')) {
      if (kerberosWizardController.get('skipClientInstall')) {
        kerberosWizardController.overrideVisibility(configs, false, kerberosWizardController.get('exceptionsOnSkipClient'));
      }
      return;
    } else if (manageIdentitiesConfig) {
      manageIdentitiesConfig.isVisible = false;
      manageIdentitiesConfig.value = 'true';
    }

    this.setConfigVisibility('ad', configs, kdcType);
    this.setConfigVisibility('mit', configs, kdcType);
    this.setConfigVisibility('ipa', configs, kdcType);
  },

  /**
   *
   * @param {string} type
   * @param {Array} configs
   * @param {string} kdcType
   */
  setConfigVisibility: function setConfigVisibility(type, configs, kdcType) {
    var typeSettings = this.get('kerberosConfigMap')[type];

    typeSettings.configNames.forEach(function (_configName) {
      var config = configs.findProperty('name', _configName);
      if (config) {
        config.isVisible = kdcType === typeSettings.type;
      }
    }, this);
  },

  submit: function submit() {
    var self = this;
    if (this.get('isSubmitDisabled')) return false;
    App.set('router.nextBtnClickInProgress', true);
    this.get('wizardController').deleteKerberosService().always(function () {
      self.configureKerberos();
    });
  },

  configureKerberos: function configureKerberos() {
    var self = this;
    var wizardController = App.router.get(this.get('content.controllerName'));
    var callback = function callback() {
      self.createConfigurations().done(function () {
        self.createKerberosAdminSession().done(function () {
          App.set('router.nextBtnClickInProgress', false);
          App.router.send('next');
        });
      });
    };
    if (wizardController.get('skipClientInstall')) {
      callback();
    } else {
      wizardController.createKerberosResources(callback);
    }
  },

  createConfigurations: function createConfigurations() {
    var service = App.StackService.find().findProperty('serviceName', 'KERBEROS'),
        serviceConfigTags = [],
        allConfigData = [],
        serviceConfigData = [];

    Object.keys(service.get('configTypes')).forEach(function (type) {
      if (!serviceConfigTags.someProperty('type', type)) {
        var obj = this.createKerberosSiteObj(type);
        obj.service_config_version_note = Em.I18n.t('admin.kerberos.wizard.configuration.note');
        serviceConfigTags.pushObject(obj);
      }
    }, this);

    Object.keys(service.get('configTypesRendered')).forEach(function (type) {
      var serviceConfigTag = serviceConfigTags.findProperty('type', type);
      if (serviceConfigTag) {
        serviceConfigData.pushObject(serviceConfigTag);
      }
    }, this);
    if (serviceConfigData.length) {
      allConfigData.pushObject(JSON.stringify({
        Clusters: {
          desired_config: serviceConfigData
        }
      }));
    }
    return App.ajax.send({
      name: 'common.across.services.configurations',
      sender: this,
      data: {
        data: '[' + allConfigData.toString() + ']'
      }
    });
  },

  createKerberosSiteObj: function createKerberosSiteObj(site) {
    var properties = {};
    var content = this.get('stepConfigs')[0].get('configs');
    var configs = content.filterProperty('filename', site + '.xml');
    // properties that should be formated as hosts
    var hostProperties = ['kdc_hosts', 'realm'];

    configs.forEach(function (_configProperty) {
      // do not pass any globals whose name ends with _host or _hosts
      if (_configProperty.isRequiredByAgent !== false) {
        if (hostProperties.contains(_configProperty.name)) {
          properties[_configProperty.name] = App.config.trimProperty({ displayType: 'host', value: _configProperty.value });
        } else {
          properties[_configProperty.name] = App.config.trimProperty(_configProperty);
        }
      }
    }, this);
    this.tweakKdcTypeValue(properties);
    this.tweakManualKdcProperties(properties);
    this.tweakIpaKdcProperties(properties);
    return { "type": site, "properties": properties };
  },

  tweakKdcTypeValue: function tweakKdcTypeValue(properties) {
    var kdcTypesValues = App.router.get('mainAdminKerberosController.kdcTypesValues');
    for (var prop in kdcTypesValues) {
      if (kdcTypesValues.hasOwnProperty(prop)) {
        if (kdcTypesValues[prop] === properties['kdc_type']) {
          properties['kdc_type'] = prop;
        }
      }
    }
  },

  tweakManualKdcProperties: function tweakManualKdcProperties(properties) {
    var kerberosWizardController = this.get('controllers.kerberosWizardController');
    if (properties['kdc_type'] === 'none' || kerberosWizardController.get('skipClientInstall')) {
      if (properties.hasOwnProperty('manage_identities')) {
        properties['manage_identities'] = 'false';
      }
      if (properties.hasOwnProperty('install_packages')) {
        properties['install_packages'] = 'false';
      }
      if (properties.hasOwnProperty('manage_krb5_conf')) {
        properties['manage_krb5_conf'] = 'false';
      }
    }
  },

  tweakIpaKdcProperties: function tweakIpaKdcProperties(properties) {
    if (typeof properties['kdc_type'] === 'undefined') {
      return;
    }
    if (this.get('content.kerberosOption') === App.router.get('mainAdminKerberosController.kdcTypesValues')['ipa']) {
      if (properties.hasOwnProperty('install_packages')) {
        properties['install_packages'] = 'false';
      }
      if (properties.hasOwnProperty('manage_krb5_conf')) {
        properties['manage_krb5_conf'] = 'false';
      }
    }
  },

  /**
   * puts kerberos admin credentials in the live cluster session
   * @returns {*} jqXHr
   */
  createKerberosAdminSession: function createKerberosAdminSession(configs) {
    configs = configs || this.get('stepConfigs')[0].get('configs');
    if (!this.get('wizardController.skipClientInstall')) {
      return this.createKDCCredentials(configs);
    }

    var adminPrincipalValue = configs.findProperty('name', 'admin_principal').value;
    var adminPasswordValue = configs.findProperty('name', 'admin_password').value;
    return App.ajax.send({
      name: 'common.cluster.update',
      sender: this,
      data: {
        clusterName: App.get('clusterName'),
        data: [{
          session_attributes: {
            kerberos_admin: { principal: adminPrincipalValue, password: adminPasswordValue }
          }
        }]
      }
    });
  },

  /**
   * shows popup with to warn user
   * @param {Function} primary
   */
  showConnectionInProgressPopup: function showConnectionInProgressPopup(primary) {
    var primaryText = Em.I18n.t('common.exitAnyway');
    var msg = Em.I18n.t('services.service.config.connection.exitPopup.msg');
    App.showConfirmationPopup(primary, msg, null, null, primaryText);
  },

  setKDCTypeProperty: function setKDCTypeProperty(configs) {
    var selectedOption = this.get('content.kerberosOption');
    var kdcTypeProperty = configs.filterProperty('filename', 'kerberos-env.xml').findProperty('name', 'kdc_type');
    var kdcValuesMap = App.router.get('mainAdminKerberosController.kdcTypesValues');
    var kdcTypeValue = Em.keys(kdcValuesMap).filter(function (typeAlias) {
      return Em.get(kdcValuesMap, typeAlias) === selectedOption;
    })[0];
    if (kdcTypeProperty) {
      Em.set(kdcTypeProperty, 'value', kdcValuesMap[kdcTypeValue]);
    }
  },

  /**
   * Override App.WizardStep7Controller#overrideConfigIsRequired
   *
   * @override
   */
  overrideConfigIsRequired: function overrideConfigIsRequired() {}
});

});

require.register("controllers/main/admin/kerberos/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.KerberosWizardStep3Controller = App.KerberosProgressPageController.extend({
  name: 'kerberosWizardStep3Controller',
  clusterDeployState: 'KERBEROS_DEPLOY',
  serviceName: 'KERBEROS',
  componentName: 'KERBEROS_CLIENT',
  ignore: undefined,
  heartBeatLostHosts: [],
  commands: ['installKerberos', 'testKerberos'],

  loadStep: function loadStep() {
    this._super();
    this.enableDisablePreviousSteps();
  },

  clearStep: function clearStep() {
    this.get('heartBeatLostHosts').clear();
    this._super();
  },

  installKerberos: function installKerberos() {
    var self = this;
    this.getKerberosClientState().done(function (data) {
      if (data.ServiceComponentInfo.state === 'INIT') {
        App.ajax.send({
          name: 'common.services.update',
          sender: self,
          data: {
            context: Em.I18n.t('requestInfo.kerberosService'),
            ServiceInfo: { "state": "INSTALLED" },
            urlParams: "ServiceInfo/state=INSTALLED&ServiceInfo/service_name=KERBEROS"
          },
          success: 'startPolling',
          error: 'onTaskError'
        });
      } else {
        var hostNames = App.get('allHostNames');
        self.updateComponent('KERBEROS_CLIENT', hostNames, "KERBEROS", "Install");
      }
    });
  },

  /**
   * Get hosts with HEARTBEAT_LOST state.
   *
   * @return {$.Deferred.promise} promise
   */
  getHeartbeatLostHosts: function getHeartbeatLostHosts() {
    return App.ajax.send({
      name: 'hosts.heartbeat_lost',
      sender: this,
      data: {
        clusterName: App.get('clusterName')
      }
    });
  },

  getKerberosClientState: function getKerberosClientState() {
    return App.ajax.send({
      name: 'common.service_component.info',
      sender: this,
      data: {
        serviceName: this.get('serviceName'),
        componentName: this.get('componentName'),
        urlParams: "fields=ServiceComponentInfo/state"
      }
    });
  },

  testKerberos: function testKerberos() {
    var self = this;
    App.ajax.send({
      'name': 'service.item.smoke',
      'sender': this,
      'success': 'startPolling',
      'error': 'onTestKerberosError',
      'kdcCancelHandler': function kdcCancelHandler() {
        App.router.get(self.get('content.controllerName')).setStepsEnable();
        self.get('tasks').objectAt(self.get('currentTaskId')).set('status', 'FAILED');
      },
      'data': {
        'serviceName': this.serviceName,
        'displayName': App.format.role(this.serviceName, true),
        'actionName': this.serviceName + '_SERVICE_CHECK',
        'operationLevel': {
          "level": "CLUSTER",
          "cluster_name": App.get('clusterName')
        }
      }
    });
  },

  onTestKerberosError: function onTestKerberosError(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
    this.onTaskError(jqXHR, ajaxOptions, error, opt);
  },

  /**
   * Enable or disable previous steps according to tasks statuses
   */
  enableDisablePreviousSteps: function () {
    var wizardController = App.router.get(this.get('content.controllerName'));
    if (this.get('tasks').someProperty('status', 'FAILED')) {
      wizardController.setStepsEnable();
    } else {
      wizardController.setLowerStepsDisable(3);
    }
  }.observes('tasks.@each.status'),

  /**
   * Show or hide warning to ignore errors and continue with the install
   */
  showIgnore: Em.computed.someBy('tasks', 'showRetry', true),

  /**
   * Enable or disable next button if ignore checkbox ticked
   */
  ignoreAndProceed: function () {
    if (this.get('showIgnore')) {
      this.set('isSubmitDisabled', !this.get('ignore'));
    }
  }.observes('ignore', 'showIgnore'),

  retryTask: function retryTask() {
    this._super();
    // retry from the first task (installKerberos) if there is any host in HEARTBEAT_LOST state.
    if (this.get('heartBeatLostHosts').length) {
      this.get('tasks').setEach('status', 'PENDING');
      this.get('tasks').setEach('showRetry', false);
      this.get('heartBeatLostHosts').clear();
    }
  },

  /**
   * Check for complete status and determines:
   *  - if there are any hosts in HEARTBEAT_LOST state. In this case warn about hosts and make step FAILED.
   *
   * @return {undefined}
   */
  statusDidChange: function () {
    var self = this;
    if (this.get('completedStatuses').contains(this.get('status'))) {
      this.getHeartbeatLostHosts().then(function (data) {
        var hostNames = Em.getWithDefault(data || {}, 'items', []).mapProperty('Hosts.host_name');
        if (hostNames.length) {
          self.set('heartBeatLostHosts', hostNames.uniq());
          self.get('tasks').objectAt(0).set('status', 'FAILED');
        }
      });
    }
  }.observes('status')
});

});

require.register("controllers/main/admin/kerberos/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('controllers/wizard/step7_controller');

App.KerberosWizardStep4Controller = App.WizardStep7Controller.extend(App.AddSecurityConfigs, App.ToggleIsRequiredMixin, App.KDCCredentialsControllerMixin, {
  name: 'kerberosWizardStep4Controller',

  // stores configurations loaded by ConfigurationsController.getConfigsByTags
  servicesConfigurations: null,

  clearStep: function clearStep() {
    this.set('isRecommendedLoaded', false);
    this.set('submitButtonClicked', false);
    this.set('selectedService', null);
    this.set('stepConfigs', []);
  },

  loadStep: function loadStep() {
    var self, stored;
    self = this;
    this.clearStep();
    stored = this.get('wizardController').loadCachedStepConfigValues(this) || [];
    this.getDescriptor().then(function (kerberosDescriptor) {
      var stepConfigs = self.setStepConfigs(self.createServicesStackDescriptorConfigs(kerberosDescriptor));
      self.set('stepConfigs', stepConfigs);
      // when configurations were stored no need to apply recommendations again
      if (App.get('supports.kerberosStackAdvisor') && !stored.length) {
        self.bootstrapRecommendationPayload(kerberosDescriptor).then(function (recommendations) {
          self.loadServerSideConfigsRecommendations(recommendations).always(function () {
            self.applyServiceConfigs(stepConfigs);
          });
        });
      } else {
        self.applyServiceConfigs(stepConfigs);
      }
    }, function () {
      self.set('isRecommendedLoaded', true);
    });
  },

  /**
   * Get descriptor configs from API endpoint.
   * On <b>Enable Kerberos</b> loads descriptor from cluster STACK resource.
   * On <b>Add Service Wizard</b> first check for cluster's artifacts descriptor and
   * save it presence status, then loads from cluster COMPOSITE resource.
   * Check for cluster/artifacts/kerberos_descriptor is necessary to determine updating or creation
   * kerberos descriptor.
   *
   * @returns {$.Deferred}
   */
  getDescriptor: function getDescriptor() {
    var dfd = $.Deferred();
    var successCallback = function successCallback(data) {
      dfd.resolve(data);
    };

    this.loadClusterDescriptorConfigs(false).then(successCallback);
    return dfd.promise();
  },

  /**
   * Create service config object for Kerberos service.
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @returns {Em.Object}
   */
  createServiceConfig: function createServiceConfig(configs) {
    // Identity configs related to user principal
    var clusterConfigs = configs.filterProperty('serviceName', 'Cluster');
    // storm user principal is not required for ambari operation
    var userConfigs = configs.filterProperty('identityType', 'user');
    var generalConfigs = clusterConfigs.concat(userConfigs).uniq('name');
    var advancedConfigs = configs.filter(function (element) {
      return !generalConfigs.findProperty('name', element.get('name'));
    });
    var categoryForGeneralConfigs = [App.ServiceConfigCategory.create({ name: 'Global', displayName: 'Global' }), App.ServiceConfigCategory.create({ name: 'Ambari Principals', displayName: 'Ambari Principals' })];
    var categoryForAdvancedConfigs = this.createCategoryForServices();
    return [App.ServiceConfig.create({
      displayName: 'General',
      name: 'GENERAL',
      serviceName: 'KERBEROS_GENERAL',
      configCategories: categoryForGeneralConfigs,
      configs: generalConfigs,
      configGroups: [],
      showConfig: true
    }), App.ServiceConfig.create({
      displayName: 'Advanced',
      name: 'ADVANCED',
      serviceName: 'KERBEROS_ADVANCED',
      configCategories: categoryForAdvancedConfigs,
      configs: advancedConfigs,
      configGroups: [],
      showConfig: true
    })];
  },

  /**
   * creates categories for advanced secure configs
   * @returns {[App.ServiceConfigCategory]}
   */
  createCategoryForServices: function createCategoryForServices() {
    var services = [];
    if (this.get('wizardController.name') === 'addServiceController') {
      services = App.StackService.find().filter(function (item) {
        return item.get('isInstalled') || item.get('isSelected');
      });
    } else {
      services = App.Service.find();
    }
    return services.map(function (item) {
      return App.ServiceConfigCategory.create({ name: item.get('serviceName'), displayName: item.get('displayName'), collapsedByDefault: true });
    });
  },

  /**
   * Prepare step configs using stack descriptor properties.
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @param {App.ServiceConfigProperty[]} stackConfigs converted kerberos descriptor
   */
  setStepConfigs: function setStepConfigs(configs, stackConfigs) {
    var configProperties = this.prepareConfigProperties(configs),
        stackConfigProperties = stackConfigs ? this.prepareConfigProperties(stackConfigs) : [],
        alterProperties = ['value', 'initialValue', 'defaultValue'];
    if (this.get('wizardController.name') === 'addServiceController') {
      // config properties for installed services should be disabled on Add Service Wizard
      configProperties.forEach(function (item) {
        if (this.get('installedServiceNames').contains(item.get('serviceName')) || item.get('serviceName') == 'Cluster') {
          item.set('isEditable', false);
        } else if (stackConfigs) {
          var stackConfigProperty = stackConfigProperties.filterProperty('filename', item.get('filename')).findProperty('name', item.get('name'));
          if (stackConfigProperty) {
            alterProperties.forEach(function (alterProperty) {
              item.set(alterProperty, stackConfigProperty.get(alterProperty));
            });
          }
        }
      }, this);
      // Concat properties that are present in the stack's kerberos  descriptor but not in the cluster kerberos descriptor
      stackConfigProperties.forEach(function (_stackConfigProperty) {
        var isPropertyInClusterDescriptor = configProperties.filterProperty('filename', _stackConfigProperty.get('filename')).someProperty('name', _stackConfigProperty.get('name'));
        if (!isPropertyInClusterDescriptor) {
          if (this.get('installedServiceNames').contains(_stackConfigProperty.get('serviceName')) || _stackConfigProperty.get('serviceName') === 'Cluster') {
            _stackConfigProperty.set('isEditable', false);
          }
          configProperties.pushObject(_stackConfigProperty);
        }
      }, this);
    }
    configProperties = App.config.sortConfigs(configProperties);
    var stepConfigs = this.createServiceConfig(configProperties);
    this.set('selectedService', stepConfigs[0]);
    this.get('stepConfigs').pushObjects(stepConfigs);
    return stepConfigs;
  },

  /**
   * Filter configs by installed services for Kerberos Wizard or by installed + selected services
   * for Add Service Wizard.
   * Set property value observer.
   * Set realm property with value from previous configuration step.
   * Set appropriate category for all configs.
   * Hide KDC related credentials properties if kerberos was manually enabled.
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @returns {App.ServiceConfigProperty[]}
   */
  prepareConfigProperties: function prepareConfigProperties(configs) {
    var self = this;
    // stored configs from previous steps (Configure Kerberos or Customize Services for ASW)
    var storedServiceConfigs = this.get('wizardController.content.serviceConfigProperties');
    var installedServiceNames = ['Cluster', 'AMBARI'].concat(App.Service.find().mapProperty('serviceName'));
    var configProperties = configs.slice(0);
    var siteProperties = App.configsCollection.getAll();
    var realmValue;
    // override stored values
    App.config.mergeStoredValue(configProperties, this.get('wizardController').loadCachedStepConfigValues(this));

    // show admin properties in add service wizard
    if (this.get('isWithinAddService')) {
      installedServiceNames = installedServiceNames.concat(this.get('selectedServiceNames'));
    }
    configProperties = configProperties.filter(function (item) {
      return installedServiceNames.contains(item.get('serviceName'));
    });
    if (this.get('wizardController.name') !== 'addServiceController') {
      realmValue = storedServiceConfigs.findProperty('name', 'realm').value;
      configProperties.findProperty('name', 'realm').set('value', realmValue);
      configProperties.findProperty('name', 'realm').set('savedValue', realmValue);
      configProperties.findProperty('name', 'realm').set('recommendedValue', realmValue);
    }

    configProperties.setEach('isSecureConfig', false);
    configProperties.forEach(function (property, item, allConfigs) {
      if (['spnego_keytab', 'spnego_principal'].contains(property.get('name'))) {
        property.addObserver('value', self, 'spnegoPropertiesObserver');
      }
      if (property.get('observesValueFrom') && allConfigs.someProperty('name', property.get('observesValueFrom'))) {
        var observedValue = Em.get(allConfigs.findProperty('name', property.get('observesValueFrom')), 'value');
        property.set('value', observedValue);
        property.set('recommendedValue', observedValue);
        property.set('isVisible', true);
      }
      if (property.get('serviceName') === 'Cluster') {
        property.set('category', 'Global');
      } else {
        property.set('category', property.get('serviceName'));
      }
      // All user identity except storm should be grouped under "Ambari Principals" category
      if (property.get('identityType') == 'user') property.set('category', 'Ambari Principals');
      var siteProperty = siteProperties.findProperty('name', property.get('name'));
      if (siteProperty) {
        if (siteProperty.category === property.get('category')) {
          property.set('displayName', siteProperty.displayName);
          if (siteProperty.index) {
            property.set('index', siteProperty.index);
          }
        }
        if (siteProperty.displayType) {
          property.set('displayType', siteProperty.displayType);
        }
      }
      this.tweakConfigProperty(property);
    }, this);

    return configProperties;
  },

  /**
   * Function to override kerberos descriptor's property values
   */
  tweakConfigProperty: function tweakConfigProperty(config) {
    var defaultHiveMsPort = "9083",
        hiveMSHosts,
        port,
        hiveMSHostNames,
        configValue;
    if (config.name === 'templeton.hive.properties') {
      hiveMSHosts = App.HostComponent.find().filterProperty('componentName', 'HIVE_METASTORE');
      if (hiveMSHosts.length > 1) {
        hiveMSHostNames = hiveMSHosts.mapProperty('hostName');
        port = config.value.match(/:[0-9]{2,4}/);
        port = port ? port[0].slice(1) : defaultHiveMsPort;
        for (var i = 0; i < hiveMSHostNames.length; i++) {
          hiveMSHostNames[i] = "thrift://" + hiveMSHostNames[i] + ":" + port;
        }
        configValue = config.value.replace(/thrift.+[0-9]{2,},/i, hiveMSHostNames.join('\\,') + ",");
        config.set('value', configValue);
        config.set('recommendedValue', configValue);
      }
    }
  },

  /**
   * Sync up values between inherited property and its reference.
   *
   * @param {App.ServiceConfigProperty} configProperty
   */
  spnegoPropertiesObserver: function spnegoPropertiesObserver(configProperty) {
    var stepConfig = this.get('stepConfigs').findProperty('name', 'ADVANCED');

    stepConfig.get('configs').forEach(function (config) {
      if (config.get('observesValueFrom') === configProperty.get('name')) {
        Em.run.once(this, function () {
          config.set('value', configProperty.get('value'));
          config.set('recommendedValue', configProperty.get('value'));
        });
      }
    }, this);
  },

  submit: function submit() {
    this.set('submitButtonClicked', true);
    this.saveConfigurations();
    App.router.send('next');
  },

  saveConfigurations: function saveConfigurations() {
    var kerberosDescriptor = this.get('kerberosDescriptor');
    var configs = [];

    this.get('stepConfigs').forEach(function (_stepConfig) {
      configs = configs.concat(_stepConfig.get('configs'));
    });
    this.updateKerberosDescriptor(kerberosDescriptor, configs);
    App.get('router.kerberosWizardController').saveKerberosDescriptorConfigs(kerberosDescriptor);
  },

  /**
   * Add/update property in `properties` object for each config type with
   * associated kerberos descriptor config value.
   *
   * @private
   * @param {object[]} configurations
   * @param {App.ServiceConfigProperty[]} kerberosDescriptor
   * @returns {object[]}
   */
  mergeDescriptorToConfigurations: function mergeDescriptorToConfigurations(configurations, kerberosDescriptor) {
    return configurations.map(function (configType) {
      var properties = $.extend({}, configType.properties);
      var filteredDescriptor = kerberosDescriptor.filterProperty('filename', configType.type);
      if (filteredDescriptor.length) {
        filteredDescriptor.forEach(function (descriptorConfig) {
          var configName = Em.get(descriptorConfig, 'name');
          properties[configName] = Em.get(descriptorConfig, 'value');
        });
      }
      return {
        type: configType.type,
        version: configType.version,
        tag: configType.tag,
        properties: properties
      };
    });
  },

  loadServerSideConfigsRecommendations: function loadServerSideConfigsRecommendations(recommendations) {
    return App.ajax.send({
      'name': 'config.recommendations',
      'sender': this,
      'data': {
        stackVersionUrl: App.get('stackVersionURL'),
        dataToSend: {
          recommend: 'configurations',
          hosts: App.get('allHostNames'),
          services: this.get('serviceNames'),
          recommendations: recommendations
        }
      },
      'success': 'loadRecommendationsSuccess',
      'error': 'loadRecommendationsError'
    });
  },

  loadRecommendationsError: function loadRecommendationsError(req, ajaxOpts, error, opt) {
    var resp;
    try {
      resp = $.parseJSON(req.responseText);
    } catch (e) {}
    return App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      bodyClass: App.AjaxDefaultErrorPopupBodyView.extend({
        type: opt.type || 'GET',
        url: opt.url,
        status: req.status,
        message: resp && resp.message || req.responseText
      })
    });
  },

  applyServiceConfigs: function applyServiceConfigs(stepConfigs) {
    this.set('isRecommendedLoaded', true);
    this.set('selectedService', stepConfigs[0]);
  },

  /**
   * Callback executed when all configs specified by tags are loaded.
   * Here we handle configurations for instlled services and Kerberos.
   * Gather needed info for recommendation request such as configurations object.
   *
   * @override
   */
  getConfigTagsSuccess: function getConfigTagsSuccess(data) {
    // here we get all installed services including KERBEROS
    var serviceNames = App.Service.find().mapProperty('serviceName').concat(['KERBEROS']).uniq();
    // collect all config types for selected services
    var installedServiceSites = Array.prototype.concat.apply([], App.config.get('preDefinedServiceConfigs').filter(function (serviceConfig) {
      return serviceNames.contains(Em.get(serviceConfig, 'serviceName'));
    }).map(function (service) {
      // when service have no configs return <code>null</code> instead return config types
      if (!service.get('configTypes')) return null;
      return Object.keys(service.get('configTypes'));
    }, this).compact()).uniq(); // cleanup <code>null</code>

    // take all configs for selected services by config types recieved from API response
    var serviceConfigTags = Em.keys(data.Clusters.desired_configs).reduce(function (tags, site) {
      if (data.Clusters.desired_configs.hasOwnProperty(site)) {
        // push cluster-env.xml also since it not associated with any service but need to further processing
        if (installedServiceSites.contains(site) || site === 'cluster-env') {
          tags.push({
            siteName: site,
            tagName: data.Clusters.desired_configs[site].tag,
            newTagName: null
          });
        }
      }
      return tags;
    }, []);
    // store configurations
    this.set('serviceConfigTags', serviceConfigTags);
    this.set('isAppliedConfigLoaded', true);
  },

  /**
   * Prepare all necessary data for recommendations payload.
   *
   * #mutates initialConfigValues
   * @returns {$.Deferred.promise()}
   */
  bootstrapRecommendationPayload: function bootstrapRecommendationPayload(kerberosDescriptor) {
    var dfd = $.Deferred();
    var self = this;

    this.getServicesConfigurations().then(function (configurations) {
      var recommendations = self.getBlueprintPayloadObject(configurations, kerberosDescriptor);
      self.set('servicesConfigurations', configurations);
      self.set('initialConfigValues', recommendations.blueprint.configurations);
      dfd.resolve(recommendations);
    });
    return dfd.promise();
  },

  getServicesConfigurations: function getServicesConfigurations() {
    var dfd = $.Deferred();
    var self = this;
    var configs, servicesConfigurations;
    if (this.get('isWithinAddService')) {
      configs = this.get('content.serviceConfigProperties');
      servicesConfigurations = configs.reduce(function (configTags, property) {
        var fileName = App.config.getConfigTagFromFileName(property.filename),
            configType;
        if (!configTags.someProperty('type', fileName)) {
          configTags.push({
            type: fileName,
            properties: {}
          });
        }
        configType = configTags.findProperty('type', fileName);
        configType.properties[property.name] = property.value;
        return configTags;
      }, []);
      dfd.resolve(servicesConfigurations);
    } else {
      this.getConfigTags().then(function () {
        App.router.get('configurationController').getConfigsByTags(self.get('serviceConfigTags')).done(function (configurations) {
          dfd.resolve(configurations);
        });
      });
    }

    return dfd.promise();
  },

  /**
   * @override
   */
  _saveRecommendedValues: function _saveRecommendedValues(data) {
    var recommendedConfigurations = Em.getWithDefault(data, 'resources.0.recommendations.blueprint.configurations', {});
    var allConfigs = Array.prototype.concat.apply([], this.get('stepConfigs').mapProperty('configs'));
    var self = this;
    // iterate by each config file name e.g. hdfs-site
    var groupedProperties = this.groupRecommendationProperties(recommendedConfigurations, this.get('servicesConfigurations'), allConfigs);
    var newProperties = [];
    Em.keys(groupedProperties.add).forEach(function (fileName) {
      var serviceName = self.getServiceByFilename(fileName);
      Em.keys(groupedProperties.add[fileName]).forEach(function (propertyName) {
        var property = self._createNewProperty(propertyName, fileName, serviceName, groupedProperties.add[fileName][propertyName]);
        property.set('category', serviceName);
        property.set('isOverridable', false);
        property.set('supportsFinal', false);
        property.set('isUserProperty', false);
        property.set('filename', fileName);
        newProperties.push(property);
      });
    });
    Array.prototype.push.apply(self.getServicesConfigObject().get('configs'), newProperties);
    Em.keys(groupedProperties.update).forEach(function (fileName) {
      Em.keys(groupedProperties.update[fileName]).forEach(function (propertyName) {
        var configProperty = allConfigs.filterProperty('filename', fileName).findProperty('name', propertyName);
        if (configProperty) {
          self._updateConfigByRecommendation(configProperty, groupedProperties.update[fileName][propertyName], true, false);
        }
      });
    });
    Em.keys(groupedProperties.delete).forEach(function (fileName) {
      Em.keys(groupedProperties.delete[fileName]).forEach(function (propertyName) {
        var servicesConfigs = self.getServicesConfigObject().get('configs');
        servicesConfigs.removeObject(servicesConfigs.filterProperty('filename', fileName).findProperty('name', propertyName));
      });
    });
  },

  /**
   * Returns category where services' configuration located.
   *
   * @returns {App.ServiceConfig}
   */
  getServicesConfigObject: function getServicesConfigObject() {
    return this.get('stepConfigs').findProperty('name', 'ADVANCED');
  }

});

});

require.register("controllers/main/admin/kerberos/step5_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
var stringUtils = require('utils/string_utils');
var fileUtils = require('utils/file_utils');

App.KerberosWizardStep5Controller = App.KerberosProgressPageController.extend(App.AddSecurityConfigs, {
  name: 'kerberosWizardStep5Controller',

  /**
   * @type {Array}
   */
  csvData: [],

  /**
   * @type {Array}
   */
  kdcProperties: [{
    key: Em.I18n.t('admin.kerberos.wizard.step1.option.kdc'),
    properties: ['kdc_type', 'kdc_hosts', 'realm', 'executable_search_paths']
  }, {
    key: Em.I18n.t('admin.kerberos.wizard.step1.option.ad'),
    properties: ['kdc_type', 'kdc_hosts', 'realm', 'ldap_url', 'container_dn', 'executable_search_paths']
  }, {
    key: Em.I18n.t('admin.kerberos.wizard.step1.option.ipa'),
    properties: ['kdc_type', 'kdc_hosts', 'realm', 'executable_search_paths']
  }, {
    key: Em.I18n.t('admin.kerberos.wizard.step1.option.manual'),
    properties: ['kdc_type', 'realm', 'executable_search_paths']
  }],

  isCSVRequestInProgress: false,

  submit: function submit() {
    App.router.send('next');
  },

  /**
   * get CSV data from the server
   */
  getCSVData: function getCSVData(skipDownload) {
    this.set('isCSVRequestInProgress', true);
    return App.ajax.send({
      name: 'admin.kerberos.cluster.csv',
      sender: this,
      data: {
        'skipDownload': skipDownload
      },
      success: 'getCSVDataSuccessCallback',
      error: 'getCSVDataSuccessCallback'
    });
  },

  /**
   * get CSV data from server success callback
   * @param data {string}
   * @param opt {object}
   * @param params {object}
   */
  getCSVDataSuccessCallback: function getCSVDataSuccessCallback(data, opt, params) {
    this.set('csvData', this.prepareCSVData(data.split('\n')));
    this.set('isCSVRequestInProgress', false);
    if (!Em.get(params, 'skipDownload')) {
      fileUtils.downloadTextFile(stringUtils.arrayToCSV(this.get('csvData')), 'csv', 'kerberos.csv');
    }
  },

  prepareCSVData: function prepareCSVData(array) {
    for (var i = 0; i < array.length; i += 1) {
      array[i] = array[i].split(',');
    }

    return array;
  },

  /**
   * Send request to unkerberisze cluster
   * @returns {$.ajax}
   */
  unkerberizeCluster: function unkerberizeCluster() {
    return App.ajax.send({
      name: 'admin.unkerberize.cluster',
      sender: this,
      success: 'goToNextStep',
      error: 'goToNextStep'
    });
  },

  goToNextStep: function goToNextStep() {
    this.clearStage();
    App.router.transitionTo('step5');
  },

  isSubmitDisabled: Em.computed.notExistsIn('status', ['COMPLETED', 'FAILED']),

  confirmProperties: function () {
    var wizardController = App.router.get('kerberosWizardController');
    var kdc_type = wizardController.get('content.kerberosOption'),
        kdcTypeProperties = this.get('kdcProperties').filter(function (item) {
      return item.key === kdc_type;
    }),
        properties = kdcTypeProperties.length ? kdcTypeProperties[0].properties : [];

    return wizardController.get('content.serviceConfigProperties').filter(function (item) {
      return properties.contains(item.name);
    }).map(function (item) {
      item['label'] = Em.I18n.t('admin.kerberos.wizard.step5.' + item['name'] + '.label');
      return item;
    });
  }.property('App.router.kerberosWizardController.content.@each.serviceConfigProperties')
});

});

require.register("controllers/main/admin/kerberos/step6_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.KerberosWizardStep6Controller = App.KerberosProgressPageController.extend({
  name: 'kerberosWizardStep6Controller',
  clusterDeployState: 'KERBEROS_DEPLOY',
  commands: ['stopServices'],

  stopServices: function stopServices() {
    App.ajax.send({
      name: 'common.services.update',
      data: {
        context: "Stop services",
        "ServiceInfo": {
          "state": "INSTALLED"
        }
      },
      sender: this,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  loadStep: function loadStep() {
    this.checkComponentsRemoval();
    this._super();
  },

  /**
   * remove Application Timeline Server component if needed.
   */
  checkComponentsRemoval: function checkComponentsRemoval() {
    if (App.Service.find().someProperty('serviceName', 'YARN') && !App.get('doesATSSupportKerberos') && !this.get('commands').contains('deleteATS') && App.HostComponent.find().findProperty('componentName', 'APP_TIMELINE_SERVER')) {
      this.get('commands').pushObject('deleteATS');
    }
  },

  /**
   * Remove Application Timeline Server from the host.
   * @returns {$.Deferred}
   */
  deleteATS: function deleteATS() {
    return App.ajax.send({
      name: 'common.delete.host_component',
      sender: this,
      data: {
        componentName: 'APP_TIMELINE_SERVER',
        hostName: App.HostComponent.find().findProperty('componentName', 'APP_TIMELINE_SERVER').get('hostName')
      },
      success: 'onDeleteATSSuccess',
      error: 'onDeleteATSError'
    });
  },

  onDeleteATSSuccess: function onDeleteATSSuccess() {
    this.onTaskCompleted();
  },

  onDeleteATSError: function onDeleteATSError(error) {
    if (error.responseText.indexOf('org.apache.ambari.server.controller.spi.NoSuchResourceException') !== -1) {
      this.onDeleteATSSuccess();
    }
  }
});

});

require.register("controllers/main/admin/kerberos/step7_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep7Controller = App.KerberosProgressPageController.extend(App.AddSecurityConfigs, {
  name: 'kerberosWizardStep7Controller',
  clusterDeployState: 'KERBEROS_DEPLOY',
  isSingleRequestPage: true,
  request: {},
  commands: [],
  contextForPollingRequest: Em.I18n.t('requestInfo.kerberizeCluster'),

  /**
   * Define whether show Back button
   * @type {Boolean}
   */
  isBackButtonDisabled: true,

  /**
   * Start cluster kerberization. On retry just unkerberize and kerbrize cluster.
   * @param {bool} isRetry
   */
  setRequest: function setRequest(isRetry) {
    var kerberizeRequest = {
      name: 'KERBERIZE_CLUSTER',
      ajaxName: 'admin.kerberize.cluster',
      ajaxData: {
        data: {
          Clusters: {
            security_type: "KERBEROS"
          }
        }
      }
    };
    if (isRetry) {
      // on retry send force update
      this.set('request', {
        name: 'KERBERIZE_CLUSTER',
        ajaxName: 'admin.kerberize.cluster.force'
      });
      this.clearStage();
      this.loadStep();
    } else {
      this.set('request', kerberizeRequest);
    }
  },

  /**
   * Send request to unkerberisze cluster
   * @returns {$.ajax}
   */
  unkerberizeCluster: function unkerberizeCluster() {
    return App.ajax.send({
      name: 'admin.unkerberize.cluster',
      sender: this,
      success: 'goToNextStep',
      error: 'goToNextStep'
    });
  },

  goToNextStep: function goToNextStep() {
    this.clearStage();
    App.router.transitionTo('step7');
  },

  retry: function retry() {
    this.set('showRetry', false);
    this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
    this.set('status', 'IN_PROGRESS');
    this.get('tasks').setEach('status', 'PENDING');
    this.setRequest(true);
  },

  /**
   * Enable or disable previous steps according to tasks statuses
   */
  enableDisablePreviousSteps: function () {
    var wizardController = App.router.get(this.get('content.controllerName'));
    if (this.get('tasks').someProperty('status', 'FAILED')) {
      wizardController.enableStep(4);
      this.set('isBackButtonDisabled', false);
    } else {
      wizardController.setLowerStepsDisable(6);
      this.set('isBackButtonDisabled', true);
    }
  }.observes('tasks.@each.status')
});

});

require.register("controllers/main/admin/kerberos/step8_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.KerberosWizardStep8Controller = App.KerberosProgressPageController.extend({
  name: 'kerberosWizardStep8Controller',
  clusterDeployState: 'KERBEROS_DEPLOY',
  commands: ['startServices'],

  startServices: function startServices() {
    var skipServiceCheck = App.router.get('clusterController.ambariProperties')['skip.service.checks'] === "true";
    App.ajax.send({
      name: 'common.services.update',
      sender: this,
      data: {
        "context": "Start services",
        "ServiceInfo": {
          "state": "STARTED"
        },
        urlParams: "params/run_smoke_test=" + !skipServiceCheck
      },
      success: 'startPolling',
      error: 'startServicesErrorCallback'
    });
  },

  isSubmitDisabled: Em.computed.notExistsIn('status', ['COMPLETED', 'FAILED'])

});

});

require.register("controllers/main/admin/kerberos/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardController = App.WizardController.extend(App.InstallComponent, {

  exceptionsOnSkipClient: [{ 'KDC': 'realm' }, { 'KDC': 'kdc_type' }, { 'Advanced kerberos-env': 'executable_search_paths' }],

  exceptionsForNonAdOption: [{ "Advanced kerberos-env": "password_length" }, { "Advanced kerberos-env": "password_min_digits" }, { "Advanced kerberos-env": "password_min_lowercase_letters" }, { "Advanced kerberos-env": "password_min_punctuation" }, { "Advanced kerberos-env": "password_min_uppercase_letters" }, { "Advanced kerberos-env": "password_min_whitespace" }],

  name: 'kerberosWizardController',

  totalSteps: 8,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('admin.kerberos.wizard.header'),

  isKerberosWizard: true,

  stackConfigsLoaded: false,
  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  /**
   * Check if Kerberos Clients should be installed.
   * Clients installation will be skipped if Manual Kerberization was selected.
   *
   * @type {Boolean}
   */
  skipClientInstall: function () {
    if (this.get('content.kerberosOption')) {
      return this.get('content.kerberosOption') === Em.I18n.t('admin.kerberos.wizard.step1.option.manual');
    }
    return false;
  }.property('content.kerberosOption'),

  kerberosDescriptorConfigs: null,

  content: Em.Object.create({
    controllerName: 'kerberosWizardController',
    serviceName: 'KERBEROS',
    kerberosOption: null,
    cluster: null,
    services: [],
    advancedServiceConfig: null,
    serviceConfigProperties: [],
    failedTask: null
  }),

  /**
   * set current step
   * @param {string} currentStep
   * @param {boolean} completed
   * @param {boolean} skipStateSave
   */
  setCurrentStep: function setCurrentStep(currentStep, completed, skipStateSave) {
    this._super(currentStep, completed);
    if (App.get('testMode') || skipStateSave) {
      return;
    }
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: 'KERBEROS_DEPLOY',
      wizardControllerName: 'kerberosWizardController',
      localdb: App.db.data
    });
  },

  setStepsEnable: function () {
    for (var i = 1; i <= this.get('totalSteps'); i++) {
      var step = this.get('isStepDisabled').findProperty('step', i);
      if (i <= this.get('currentStep') && App.get('router.clusterController.isLoaded')) {
        step.set('value', false);
      } else {
        step.set('value', i != this.get('currentStep'));
      }
    }
  }.observes('currentStep', 'App.router.clusterController.isLoaded'),

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.get('clusterName') });
  },

  updateClusterEnvData: function updateClusterEnvData(configs) {
    var kerberosDescriptor = this.get('kerberosDescriptorConfigs');
    configs['security_enabled'] = true;
    configs['kerberos_domain'] = kerberosDescriptor.properties.realm;
    return configs;
  },

  dataLoading: function dataLoading() {
    var dfd = $.Deferred();
    this.connectOutlet('loading');
    if (App.router.get('clusterController.isLoaded') && App.router.get('clusterController.isComponentsStateLoaded')) {
      dfd.resolve();
    } else {
      var interval = setInterval(function () {
        if (App.router.get('clusterController.isLoaded') && App.router.get('clusterController.isComponentsStateLoaded')) {
          dfd.resolve();
          clearInterval(interval);
        }
      }, 50);
    }
    return dfd.promise();
  },
  /**
   * save status of the cluster.
   * @param clusterStatus object with status,requestId fields.
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    var oldStatus = this.toObject(this.get('content.cluster'));
    clusterStatus = jQuery.extend(oldStatus, clusterStatus);
    if (clusterStatus.requestId) {
      clusterStatus.requestId.forEach(function (requestId) {
        if (clusterStatus.oldRequestsId.indexOf(requestId) === -1) {
          clusterStatus.oldRequestsId.push(requestId);
        }
      }, this);
    }
    this.set('content.cluster', clusterStatus);
    this.save('cluster');
  },

  saveConfigTag: function saveConfigTag(tag) {
    App.db.setKerberosWizardConfigTag(tag);
    this.set('content.' + [tag.name], tag.value);
  },

  saveKerberosOption: function saveKerberosOption(stepController) {
    this.setDBProperty('kerberosOption', stepController.get('selectedItem'));
    this.set('content.kerberosOption', stepController.get('selectedItem'));
  },

  /**
   * Override the visibility of a list of form items with a new value
   *
   * @param {Array} itemsArray
   * @param {boolean} newValue
   * @param {Array} exceptions
   */
  overrideVisibility: function overrideVisibility(itemsArray, newValue, exceptions, inverse) {
    newValue = newValue || false;

    for (var i = 0, len = itemsArray.length; i < len; i += 1) {
      if (!App.isEmptyObject(itemsArray[i])) {
        var isException = exceptions.filterProperty(itemsArray[i].category, itemsArray[i].name);
        if (!isException.length && !inverse) {
          itemsArray[i].isVisible = newValue;
        }
        if (isException.length && inverse) {
          itemsArray[i].isVisible = newValue;
        }
      }
    }
  },

  loadKerberosOption: function loadKerberosOption() {
    this.set('content.kerberosOption', this.getDBProperty('kerberosOption'));
  },

  createKerberosResources: function createKerberosResources(callback) {
    var self = this;
    this.createKerberosService().done(function () {
      self.updateAndCreateServiceComponent('KERBEROS_CLIENT').done(function () {
        self.createKerberosHostComponents().done(callback);
      });
    });
  },

  createKerberosService: function createKerberosService() {
    return App.ajax.send({
      name: 'wizard.step8.create_selected_services',
      sender: this,
      data: {
        data: '{"ServiceInfo": { "service_name": "KERBEROS"}}',
        cluster: App.get('clusterName')
      }
    });
  },

  /**
   * Delete Kerberos service if it exists
   *
   * @returns {$.Deferred}
   */
  deleteKerberosService: function deleteKerberosService() {
    var serviceName = 'KERBEROS',
        cachedService = App.cache.services.findProperty('ServiceInfo.service_name', serviceName),
        modelService = App.Service.find(serviceName);

    if (cachedService) {
      App.cache.services.removeObject(cachedService);
    }
    if (modelService.get('isLoaded')) {
      App.serviceMapper.deleteRecord(modelService);
    }
    return App.ajax.send({
      name: 'common.delete.service',
      sender: this,
      data: {
        serviceName: serviceName
      }
    });
  },

  /**
   * Unkerberize cluster. Set cluster `security_type` to "NONE".
   *
   * @returns {$.Deferred}
   */
  unkerberize: function unkerberize() {
    return App.ajax.send({
      name: 'admin.unkerberize.cluster',
      sender: this
    });
  },

  createKerberosHostComponents: function createKerberosHostComponents() {
    var hostNames = App.get('allHostNames');
    var queryStr = '';
    hostNames.forEach(function (hostName) {
      queryStr += 'Hosts/host_name=' + hostName + '|';
    });
    //slice off last symbol '|'
    queryStr = queryStr.slice(0, -1);

    var data = {
      "RequestInfo": {
        "query": queryStr
      },
      "Body": {
        "host_components": [{
          "HostRoles": {
            "component_name": 'KERBEROS_CLIENT'
          }
        }]
      }
    };

    return App.ajax.send({
      name: 'wizard.step8.register_host_to_component',
      sender: this,
      data: {
        cluster: App.get('clusterName'),
        data: JSON.stringify(data)
      }
    });
  },

  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.loadKerberosOption();
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var self = this;
        var dfd = $.Deferred();

        if (!self.get('stackConfigsLoaded')) {
          App.config.loadConfigsFromStack(['KERBEROS']).always(function () {
            self.loadServiceConfigProperties();
            self.set('stackConfigsLoaded', true);
            dfd.resolve();
          });
        } else {
          this.loadServiceConfigProperties();
          dfd.resolve();
        }
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadKerberosDescriptorConfigs();
      }
    }],
    '6': [{
      type: 'sync',
      callback: function callback() {
        this.loadKerberosDescriptorConfigs();
      }
    }]
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearTasksData: function clearTasksData() {
    this.saveTasksStatuses(undefined);
    this.saveRequestIds(undefined);
    this.saveTasksRequestIds(undefined);
  },

  /**
   * shows popup with to warn user
   * @param {Function} primary
   * @param {boolean} isCritical
   */
  warnBeforeExitPopup: function warnBeforeExitPopup(primary, isCritical) {
    var primaryText = Em.I18n.t('common.exitAnyway');
    var msg = isCritical ? Em.I18n.t('admin.kerberos.wizard.exit.critical.msg') : Em.I18n.t('admin.kerberos.wizard.exit.warning.msg');
    return App.showConfirmationPopup(primary, msg, null, null, primaryText, isCritical ? 'danger' : 'success');
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    // The in-memory variable for current step should be reset to 1st step.
    this.setCurrentStep('1', false, true);
    // kerberos wizard namespace in the localStorage should be emptied
    this.resetDbNamespace();
  },

  /**
   * Discard changes affected by wizard:
   *   - Unkerberize cluster
   *   - Remove Kerberos service
   *
   * @returns {$.Deferred}
   */
  discardChanges: function discardChanges() {
    var dfd = $.Deferred();
    var self = this;

    this.unkerberize().always(function () {
      self.deleteKerberosService().always(function () {
        dfd.resolve();
      });
    });

    return dfd.promise();
  }
});

});

require.register("controllers/main/admin/serviceAccounts_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('controllers/main/service/info/configs');

App.MainAdminServiceAccountsController = App.MainServiceInfoConfigsController.extend({
  name: 'mainAdminServiceAccountsController',
  users: null,
  serviceConfigTags: [],
  content: Em.Object.create({
    serviceName: 'MISC'
  }),
  loadUsers: function loadUsers() {
    this.set('selectedService', this.get('content.serviceName') ? this.get('content.serviceName') : "MISC");
    this.loadServiceConfig();
  },
  loadServiceConfig: function loadServiceConfig() {
    var _this = this;

    App.router.get('configurationController').getCurrentConfigsBySites().done(function (serverConfigs) {
      _this.createConfigObject(serverConfigs);
    });
  },

  /**
   * Generate configuration object that will be rendered
   *
   * @param {Object[]} serverConfigs
   */
  createConfigObject: function createConfigObject(serverConfigs) {
    var configs = [];
    serverConfigs.forEach(function (configObject) {
      configs = configs.concat(App.config.getConfigsFromJSON(configObject, true));
    });
    var miscConfigs = configs.filterProperty('displayType', 'user').filterProperty('category', 'Users and Groups');
    miscConfigs.setEach('isVisible', true);
    this.set('users', miscConfigs);
    this.set('dataIsLoaded', true);
  },

  /**
   * sort miscellaneous configs by specific order
   * @param sortOrder
   * @param arrayToSort
   * @return {Array}
   */
  sortByOrder: function sortByOrder(sortOrder, arrayToSort) {
    var sorted = [];
    if (sortOrder && sortOrder.length > 0) {
      sortOrder.forEach(function (name) {
        var user = arrayToSort.findProperty('name', name);
        if (user) {
          sorted.push({
            isVisible: user.get('isVisible'),
            displayName: user.get('displayName'),
            value: user.get('value')
          });
        }
      });
      return sorted;
    } else {
      return arrayToSort;
    }
  }
});

});

require.register("controllers/main/admin/service_auto_start", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminServiceAutoStartController = Em.Controller.extend({
  name: 'mainAdminServiceAutoStartController',

  /**
   * @type {?object}
   * @default null
   */
  clusterConfigs: null,

  /**
   * @type {Array}
   */
  componentsConfigsCached: [],

  /**
   * @type {object}
   */
  componentsConfigsCachedMap: function () {
    var map = {};
    this.get('componentsConfigsCached').mapProperty('ServiceComponentInfo').forEach(function (component) {
      map[component.component_name] = component.recovery_enabled === 'true';
    });
    return map;
  }.property('componentsConfigsCached.@each.ServiceComponentInfo.recovery_enabled'),

  /**
   * @type {Array}
   */
  componentsConfigsGrouped: [],

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * @type {boolean}
   */
  isGeneralRecoveryEnabled: false,

  /**
   * @type {boolean}
   */
  isGeneralRecoveryEnabledCached: false,

  /**
   * @type {boolean}
   */
  saveInProgress: false,

  /**
   * @type {boolean}
   */
  isModified: function () {
    return this.get('isGeneralModified') || this.get('isComponentModified');
  }.property('isGeneralModified', 'isComponentModified'),

  /**
   * @type {boolean}
   */
  isGeneralModified: function () {
    return this.get('isGeneralRecoveryEnabled') !== this.get('isGeneralRecoveryEnabledCached');
  }.property('isGeneralRecoveryEnabled', 'isGeneralRecoveryEnabledCached'),

  /**
   * @type {boolean}
   */
  isComponentModified: function () {
    var componentsConfigsCachedMap = this.get('componentsConfigsCachedMap');

    return this.get('componentsConfigsGrouped').some(function (component) {
      return component.get('recoveryEnabled') !== componentsConfigsCachedMap[component.get('componentName')];
    });
  }.property('componentsConfigsGrouped.@each.recoveryEnabled', 'componentsConfigsCachedMap'),

  parseComponentConfigs: function parseComponentConfigs(componentsConfigsCached) {
    componentsConfigsCached.sortPropertyLight('ServiceComponentInfo.service_name');
    var componentsConfigsGrouped = [];
    var servicesMap = componentsConfigsCached.mapProperty('ServiceComponentInfo.service_name').uniq().toWickMap();

    componentsConfigsCached.mapProperty('ServiceComponentInfo').forEach(function (component) {
      componentsConfigsGrouped.push(Em.Object.create({
        serviceDisplayName: App.format.role(component.service_name, true),
        isFirst: servicesMap[component.service_name],
        componentName: component.component_name,
        displayName: App.format.role(component.component_name, false),
        recoveryEnabled: component.recovery_enabled === 'true'
      }));
      servicesMap[component.service_name] = false;
    });
    return componentsConfigsGrouped;
  },

  load: function load() {
    var self = this;
    var clusterConfigController = App.router.get('configurationController');
    clusterConfigController.updateConfigTags().always(function () {
      clusterConfigController.getCurrentConfigsBySites(['cluster-env']).done(function (data) {
        self.set('clusterConfigs', data[0].properties);
        self.set('isGeneralRecoveryEnabled', data[0].properties.recovery_enabled === 'true');
        self.set('isGeneralRecoveryEnabledCached', self.get('isGeneralRecoveryEnabled'));
        self.loadComponentsConfigs().then(function () {
          self.set('isLoaded', true);
        });
      });
    });
  },

  loadComponentsConfigs: function loadComponentsConfigs() {
    return App.ajax.send({
      name: 'components.get_category',
      sender: this,
      success: 'loadComponentsConfigsSuccess'
    });
  },

  loadComponentsConfigsSuccess: function loadComponentsConfigsSuccess(data) {
    this.set('componentsConfigsCached', data.items);
    this.set('componentsConfigsGrouped', this.parseComponentConfigs(data.items));
  },

  saveClusterConfigs: function saveClusterConfigs(clusterConfigs, recoveryEnabled) {
    clusterConfigs.recovery_enabled = String(recoveryEnabled);
    return App.ajax.send({
      name: 'admin.save_configs',
      sender: this,
      data: {
        siteName: 'cluster-env',
        properties: clusterConfigs
      }
    });
  },

  saveComponentSettingsCall: function saveComponentSettingsCall(recoveryEnabled, components) {
    return App.ajax.send({
      name: 'components.update',
      sender: this,
      data: {
        ServiceComponentInfo: {
          recovery_enabled: recoveryEnabled
        },
        query: 'ServiceComponentInfo/component_name.in(' + components.join(',') + ')'
      }
    });
  },

  syncStatus: function syncStatus() {
    var componentsConfigsGrouped = this.get('componentsConfigsGrouped');
    this.set('isGeneralRecoveryEnabledCached', this.get('isGeneralRecoveryEnabled'));
    this.get('componentsConfigsCached').mapProperty('ServiceComponentInfo').forEach(function (component) {
      var actualComponent = componentsConfigsGrouped.findProperty('componentName', component.component_name);
      component.recovery_enabled = String(actualComponent.get('recoveryEnabled'));
    });
    this.propertyDidChange('componentsConfigsCached');
  },

  restoreCachedValues: function restoreCachedValues() {
    this.set('isGeneralRecoveryEnabled', this.get('isGeneralRecoveryEnabledCached'));
    this.set('componentsConfigsGrouped', this.parseComponentConfigs(this.get('componentsConfigsCached')));
  },

  filterComponentsByChange: function filterComponentsByChange(components, value) {
    var map = this.get('componentsConfigsCachedMap');

    return components.filter(function (component) {
      return component.get('recoveryEnabled') !== map[component.get('componentName')] && component.get('recoveryEnabled') === value;
    }).mapProperty('componentName');
  },

  /**
   * If some configs are changed and user navigates away or select another config-group, show this popup with propose to save changes
   * @param {object} transitionCallback - callback with action to change configs view
   * @return {App.ModalPopup}
   * @method showSavePopup
   */
  showSavePopup: function showSavePopup(transitionCallback) {
    var self = this;
    var title = '';
    var body = '';
    if (typeof transitionCallback === 'function') {
      title = Em.I18n.t('admin.serviceAutoStart.save.popup.transition.title');
      body = Em.I18n.t('admin.serviceAutoStart.save.popup.transition.body');
    } else {
      title = Em.I18n.t('admin.serviceAutoStart.save.popup.title');
      body = Em.I18n.t('admin.serviceAutoStart.save.popup.body');
    }
    return App.ModalPopup.show({
      header: title,
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile(body)
      }),
      footerClass: Em.View.extend({
        templateName: require('templates/main/service/info/save_popup_footer')
      }),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.cancel'),
      onSave: function onSave() {
        var clusterConfigsCall = void 0,
            enabledComponentsCall = void 0,
            disabledComponentsCall = void 0;

        if (self.get('isGeneralModified')) {
          clusterConfigsCall = self.saveClusterConfigs(self.get('clusterConfigs'), self.get('isGeneralRecoveryEnabled'));
        }

        var enabledComponents = self.filterComponentsByChange(self.get('componentsConfigsGrouped'), true);
        var disabledComponents = self.filterComponentsByChange(self.get('componentsConfigsGrouped'), false);

        if (enabledComponents.length) {
          enabledComponentsCall = self.saveComponentSettingsCall('true', enabledComponents);
        }
        if (disabledComponents.length) {
          disabledComponentsCall = self.saveComponentSettingsCall('false', disabledComponents);
        }
        self.set('saveInProgress', true);
        $.when(clusterConfigsCall, enabledComponentsCall, disabledComponentsCall).done(function () {
          if (typeof transitionCallback === 'function') {
            transitionCallback();
          }
          self.syncStatus();
        }).always(function () {
          self.set('saveInProgress', false);
        });
        this.hide();
      },
      onDiscard: function onDiscard() {
        self.restoreCachedValues();
        if (typeof transitionCallback === 'function') {
          transitionCallback();
        }
        this.hide();
      },
      onCancel: function onCancel() {
        this.hide();
      }
    });
  }
});

});

require.register("controllers/main/admin/stack_and_upgrade_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.MainAdminStackAndUpgradeController = Em.Controller.extend(App.LocalStorage, {
  name: 'mainAdminStackAndUpgradeController',

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * @type {object}
   * @default null
   */
  upgradeData: null,

  /**
   * @type {number}
   * @default null
   */
  upgradeId: null,

  /**
   * Start version of upgrade
   * @type {string}
   * @default null
   */
  fromVersion: null,

  /**
   * @type {string}
   * @default null
   */
  upgradeVersion: null,

  /**
   * @type {string}
   * @default null
   */
  upgradeType: null,

  /**
   * @type {Em.Object}
   */
  upgradeTypeConfig: Em.computed.findByKey('upgradeMethods', 'type', 'upgradeType'),

  /**
   * @type {boolean}
   */
  cantBeStarted: Em.computed.alias('upgradeTypeConfig.cantBeStarted'),

  /**
   * @type {boolean}
   */
  showPauseButton: Em.computed.and('!App.upgradeSuspended', '!App.upgradeCompleted', '!App.upgradeInit'),

  /**
   * @type {boolean}
   * @default true
   */
  downgradeAllowed: true,

  /**
   * @type {string}
   * @default null
   */
  upgradeTypeDisplayName: null,

  /**
   * @type {object}
   * @default null
   */
  failuresTolerance: null,

  /**
   * @type {boolean}
   * @default false
   */
  isDowngrade: false,

  /**
   * flag which indicate that upgrade suspended
   * @type {boolean}
   * @default false
   */
  isSuspended: false,

  /**
   * version that currently applied to server
   * should be plain object, because stored to localStorage
   * @type {object|null}
   */
  currentVersion: null,

  /**
   * versions to which cluster could be upgraded
   * @type {Array}
   */
  targetVersions: [],

  /**
   * @type {object}
   * @default null
   */
  slaveComponentStructuredInfo: null,

  /**
   * @type {Array}
   */
  serviceCheckFailuresServicenames: [],

  /**
   * @type {boolean}
   * @default false
   */
  isUpgradeTypesLoaded: false,

  /**
   * @type {string}
   */
  getSupportedUpgradeError: '',

  /**
   * Restricted type of upgrade, can't be viewed in wizard.
   * It's status visible only in upgrade status label
   * @type {boolean}
   * @default false
   */
  isWizardRestricted: false,

  /**
   * @type {string}
   */
  wizardModalTitle: function () {
    var repoVersion = App.RepositoryVersion.find().findProperty('repositoryVersion', this.get('toVersion'));
    return this.getUpgradeDowngradeHeader(this.get('upgradeTypeDisplayName'), this.get('upgradeVersion'), this.get('isDowngrade'), repoVersion);
  }.property('upgradeTypeDisplayName', 'upgradeVersion', 'isDowngrade'),

  /**
   * @param {string} upgradeType
   * @param {string} upgradeVersion
   * @param {boolean} isDowngrade
   * @param {boolean} isPatch
   * @returns {string}
   */
  getUpgradeDowngradeHeader: function getUpgradeDowngradeHeader(upgradeType, upgradeVersion, isDowngrade, repoVersion) {
    if (isDowngrade) {
      return Em.I18n.t('admin.stackUpgrade.dialog.downgrade.header').format(upgradeVersion);
    }
    if (repoVersion && repoVersion.get('isPatch')) {
      return Em.I18n.t('admin.stackUpgrade.dialog.upgrade.patch.header').format(upgradeType, upgradeVersion);
    }
    if (repoVersion && repoVersion.get('isMaint')) {
      return Em.I18n.t('admin.stackUpgrade.dialog.upgrade.maint.header').format(upgradeType, upgradeVersion);
    }
    return Em.I18n.t('admin.stackUpgrade.dialog.upgrade.header').format(upgradeType, upgradeVersion);
  },

  /**
   * methods through which cluster could be upgraded, "allowed" indicated if the method is allowed
   * by stack upgrade path
   * @type {Array}
   */
  upgradeMethods: [Em.Object.create({
    displayName: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.RU.title'),
    type: 'ROLLING',
    icon: "glyphicon glyphicon-dashboard",
    description: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.RU.description'),
    selected: false,
    allowed: true,
    isCheckComplete: false,
    isCheckRequestInProgress: false,
    precheckResultsMessage: '',
    precheckResultsTitle: '',
    action: '',
    isWizardRestricted: false
  }), Em.Object.create({
    displayName: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.EU.title'),
    type: 'NON_ROLLING',
    icon: "icon-bolt",
    description: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.EU.description'),
    selected: false,
    allowed: true,
    isCheckComplete: false,
    isCheckRequestInProgress: false,
    precheckResultsMessage: '',
    precheckResultsTitle: '',
    action: '',
    isWizardRestricted: false
  }), Em.Object.create({
    displayName: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.HOU.title'),
    type: 'HOST_ORDERED',
    icon: "icon-bolt",
    description: '',
    selected: false,
    allowed: false,
    isCheckComplete: false,
    isCheckRequestInProgress: false,
    precheckResultsMessage: '',
    precheckResultsTitle: '',
    action: '',
    isWizardRestricted: !App.supports.enabledWizardForHostOrderedUpgrade,
    cantBeStarted: true
  })],

  /**
   * pre-check messages map
   * @type {object}
   */
  preCheckMessages: {
    'WARNING': {
      template: 'admin.stackUpgrade.preCheck.warning.message',
      precheckResultsMessageClass: 'ORANGE',
      isPrecheckFailed: false,
      precheckResultsMessageIconClass: 'glyphicon glyphicon-warning-sign'
    },
    'BYPASS': {
      template: 'admin.stackUpgrade.preCheck.bypass.message',
      precheckResultsMessageClass: 'RED',
      isPrecheckFailed: false,
      precheckResultsMessageIconClass: 'glyphicon glyphicon-remove',
      bypassedFailures: true
    },
    'FAIL': {
      template: 'admin.stackUpgrade.preCheck.fail.message',
      precheckResultsMessageClass: 'RED',
      isPrecheckFailed: true,
      precheckResultsMessageIconClass: 'glyphicon glyphicon-remove'
    }
  },

  runningCheckRequests: [],

  /**
   * @type {boolean} true if some request that should disable actions is in progress
   */
  requestInProgress: false,

  /**
   * @type {number} repo id, request for which is currently in progress
   */
  requestInProgressRepoId: null,

  /**
   * @type {boolean} true while no updated upgrade info is loaded after retry
   */
  isRetryPending: false,

  /**
   * properties that stored to localStorage to resume wizard progress
   */
  wizardStorageProperties: ['fromVersion', 'upgradeId', 'upgradeVersion', 'toVersion', 'currentVersion', 'upgradeTypeDisplayName', 'upgradeType', 'failuresTolerance', 'isDowngrade', 'downgradeAllowed', 'isSuspended', 'isWizardRestricted'],

  /**
   * mutable properties of Upgrade Task
   * @type {Array}
   */
  taskDetailsProperties: ['status', 'stdout', 'stderr', 'error_log', 'host_name', 'output_log'],

  /**
   * Context for Finalize item
   * @type {string}
   */
  finalizeContext: 'Confirm Finalize',

  /**
   * Context for Slave component failures manual item
   * @type {string}
   */
  slaveFailuresContext: "Check Component Versions",

  /**
   * Context for Service check (may include slave component) failures manual item
   * @type {string}
   */
  serviceCheckFailuresContext: "Verifying Skipped Failures",

  /**
   * Check if current item is Finalize
   * @type {boolean}
   */
  isFinalizeItem: false,

  isLoadUpgradeDataPending: false,

  /**
   * path to the mock json
   * @type {String}
   */
  mockRepoUrl: '/data/stack_versions/repo_versions_all.json',

  /**
   * api to get RepoVersions
   * @type {String}
   */
  realRepoUrl: function () {
    return App.get('apiPrefix') + '/stacks?fields=versions/repository_versions/RepositoryVersions,' + 'versions/repository_versions/operating_systems/*,versions/repository_versions/operating_systems/repositories/*';
  }.property('App.stackVersionURL'),

  /**
   * path to the mock json
   * @type {String}
   */
  mockStackUrl: '/data/stack_versions/stack_version_all.json',

  /**
   * api to get ClusterStackVersions with repository_versions (use to init data load)
   * @type {String}
   */
  realStackUrl: function () {
    return App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + '/stack_versions?fields=*,repository_versions/*,repository_versions/operating_systems/OperatingSystems/*,repository_versions/operating_systems/repositories/*';
  }.property('App.clusterName'),

  /**
   * api to get ClusterStackVersions without repository_versions (use to update data)
   * @type {String}
   */
  realUpdateUrl: function () {
    return App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + '/stack_versions?fields=ClusterStackVersions/*';
  }.property('App.clusterName'),

  /**
   * Determines if list of services with checks that failed and were skipped by user during the upgrade is loaded
   * @type {boolean}
   */
  areSkippedServiceChecksLoaded: false,

  /**
   * List of services with checks that failed and were skipped by user during the upgrade
   * @type {array}
   */
  skippedServiceChecks: [],

  /**
   * status of tasks/items/groups which should be grayed out and disabled
   * @type {Array}
   */
  nonActiveStates: ['PENDING', 'ABORTED'],

  /**
   * status of Upgrade request
   * @type {string}
   */
  requestStatus: function () {
    if (this.get('upgradeData.Upgrade') && App.get('upgradeSuspended')) {
      return 'SUSPENDED';
    } else if (this.get('upgradeData.Upgrade')) {
      return this.get('upgradeData.Upgrade.request_status');
    } else {
      return 'INIT';
    }
  }.property('upgradeData.Upgrade.request_status', 'App.upgradeSuspended'),

  init: function init() {
    this.initDBProperties();
  },

  /**
   * restore data from localStorage
   */
  initDBProperties: function initDBProperties() {
    var props = this.getDBProperties(this.get('wizardStorageProperties'));
    Em.keys(props).forEach(function (k) {
      if (!Em.isNone(props[k])) {
        this.set(k, props[k]);
      }
    }, this);
  },

  /**
   * load all data:
   * - upgrade data
   * - stack versions
   * - repo versions
   */
  load: function load() {
    var dfd = $.Deferred();
    var self = this;

    this.loadStackVersionsToModel(true).done(function () {
      self.loadRepoVersionsToModel().done(function () {
        self.loadCompatibleVersions().done(function () {
          self.updateCurrentStackVersion();
          dfd.resolve();
        });
      });
    });
    return dfd.promise();
  },

  updateCurrentStackVersion: function updateCurrentStackVersion() {
    var currentVersion = App.StackVersion.find().findProperty('state', 'CURRENT');
    if (currentVersion) {
      this.set('currentVersion', {
        stack_name: currentVersion.get('repositoryVersion.stackVersionType'),
        repository_version: currentVersion.get('repositoryVersion.repositoryVersion'),
        repository_name: currentVersion.get('repositoryVersion.displayName'),
        id: currentVersion.get('repositoryVersion.id')
      });
    }
  },

  /**
   * load upgrade tasks by upgrade id
   * @return {$.Deferred}
   * @param {boolean} onlyState
   */
  loadUpgradeData: function loadUpgradeData(onlyState) {
    var upgradeId = this.get('upgradeId'),
        deferred = $.Deferred(),
        self = this;

    if (Em.isNone(upgradeId)) {
      deferred.resolve();
    } else {
      this.set('isLoadUpgradeDataPending', true);
      App.ajax.send({
        name: onlyState ? 'admin.upgrade.state' : 'admin.upgrade.data',
        sender: this,
        data: {
          id: upgradeId
        },
        success: 'loadUpgradeDataSuccessCallback'
      }).then(deferred.resolve).always(function () {
        self.set('isLoadUpgradeDataPending', false);
      });
    }
    return deferred.promise();
  },

  /**
   * parse and push upgrade tasks to controller
   * @param data
   */
  loadUpgradeDataSuccessCallback: function loadUpgradeDataSuccessCallback(data) {
    if (Em.isNone(data)) return;
    App.set('upgradeState', data.Upgrade.request_status);
    this.setDBProperty('upgradeState', data.Upgrade.request_status);
    this.set('isSuspended', data.Upgrade.suspended);
    this.setDBProperty('isSuspended', data.Upgrade.suspended);
    if (data.upgrade_groups) {
      this.updateUpgradeData(data);
    }
    if (this.get('isRetryPending') && data.Upgrade.request_status !== 'ABORTED') {
      this.setProperties({
        requestInProgress: false,
        isRetryPending: false
      });
    }
    if (data.Upgrade.request_status === 'COMPLETED') {
      this.finish();
    }
  },

  loadCompatibleVersions: function loadCompatibleVersions() {
    return App.ajax.send({
      name: 'admin.upgrade.get_compatible_versions',
      sender: this,
      data: {
        stackName: App.get('currentStackName'),
        stackVersion: App.get('currentStackVersionNumber')
      },
      success: 'loadCompatibleVersionsSuccessCallback'
    });
  },

  /**
   *
   * @param {object} data
   */
  loadCompatibleVersionsSuccessCallback: function loadCompatibleVersionsSuccessCallback(data) {
    App.RepositoryVersion.find().forEach(function (repo) {
      var version = repo.get('repositoryVersion');
      repo.set('isCompatible', data.items.someProperty('CompatibleRepositoryVersions.repository_version', version));
    });
  },

  /**
   * update data of Upgrade
   * @param {object} newData
   */
  updateUpgradeData: function updateUpgradeData(newData) {
    var oldData = this.get('upgradeData'),
        nonActiveStates = this.get('nonActiveStates'),
        groupsMap = {},
        itemsMap = {};

    if (Em.isNone(oldData) || newData.upgrade_groups.length !== oldData.upgradeGroups.length) {
      this.initUpgradeData(newData);
    } else {
      //create entities maps
      newData.upgrade_groups.forEach(function (newGroup) {
        groupsMap[newGroup.UpgradeGroup.group_id] = newGroup.UpgradeGroup;
        newGroup.upgrade_items.forEach(function (item) {
          itemsMap[item.UpgradeItem.stage_id] = item.UpgradeItem;
        });
      });

      //update existed entities with new data
      oldData.upgradeGroups.forEach(function (oldGroup) {
        oldGroup.set('status', groupsMap[oldGroup.get('group_id')].status);
        oldGroup.set('display_status', groupsMap[oldGroup.get('group_id')].display_status);
        oldGroup.set('progress_percent', groupsMap[oldGroup.get('group_id')].progress_percent);
        oldGroup.set('completed_task_count', groupsMap[oldGroup.get('group_id')].completed_task_count);
        oldGroup.upgradeItems.forEach(function (item) {
          item.set('status', itemsMap[item.get('stage_id')].status);
          item.set('display_status', itemsMap[item.get('stage_id')].display_status);
          item.set('progress_percent', itemsMap[item.get('stage_id')].progress_percent);
        });
        var hasExpandableItems = oldGroup.upgradeItems.some(function (item) {
          return !nonActiveStates.contains(item.get('status'));
        });
        oldGroup.set('hasExpandableItems', hasExpandableItems);
      });
      oldData.set('Upgrade', newData.Upgrade);
    }
  },

  /**
   * change structure of Upgrade
   * In order to maintain nested views in template object should have direct link to its properties, for example
   * item.UpgradeItem.<properties> -> item.<properties>
   * @param {object} newData
   */
  initUpgradeData: function initUpgradeData(newData) {
    var upgradeGroups = [],
        nonActiveStates = this.get('nonActiveStates');

    //wrap all entities into App.upgradeEntity
    newData.upgrade_groups.forEach(function (newGroup) {
      var hasExpandableItems = newGroup.upgrade_items.some(function (item) {
        return !nonActiveStates.contains(item.UpgradeItem.status);
      }),
          oldGroup = App.upgradeEntity.create({ type: 'GROUP', hasExpandableItems: hasExpandableItems }, newGroup.UpgradeGroup),
          upgradeItems = [];
      newGroup.upgrade_items.forEach(function (item) {
        var oldItem = App.upgradeEntity.create({ type: 'ITEM' }, item.UpgradeItem);
        this.formatMessages(oldItem);
        oldItem.set('tasks', []);
        upgradeItems.pushObject(oldItem);
      }, this);
      upgradeItems.reverse();
      oldGroup.set('upgradeItems', upgradeItems);
      upgradeGroups.pushObject(oldGroup);
    }, this);
    upgradeGroups.reverse();
    this.set('upgradeData', Em.Object.create({
      upgradeGroups: upgradeGroups,
      Upgrade: newData.Upgrade
    }));
    this.set('downgradeAllowed', newData.Upgrade.downgrade_allowed);
    this.setDBProperty('downgradeAllowed', newData.Upgrade.downgrade_allowed);
  },

  /**
   * format upgrade item text
   * @param {App.upgradeEntity} oldItem
   */
  formatMessages: function formatMessages(oldItem) {
    var text = oldItem.get('text');
    var messages = [];

    try {
      var messageArray = JSON.parse(text);
      for (var i = 0; i < messageArray.length; i++) {
        messages.push(messageArray[i].message);
      }
      oldItem.set('text', messages.join(' '));
    } catch (err) {
      console.warn('Upgrade Item has malformed text');
    }
    oldItem.set('messages', messages);
  },

  /**
   * request Upgrade Item and its tasks from server
   * @param {Em.Object} item
   * @param {Function} customCallback
   * @return {$.ajax}
   */
  getUpgradeItem: function getUpgradeItem(item, customCallback) {
    return App.ajax.send({
      name: 'admin.upgrade.upgrade_item',
      sender: this,
      data: {
        upgradeId: item.get('request_id'),
        groupId: item.get('group_id'),
        stageId: item.get('stage_id')
      },
      success: customCallback || 'getUpgradeItemSuccessCallback'
    });
  },

  /**
   * success callback of <code>getTasks</code>
   * @param {object} data
   */
  getUpgradeItemSuccessCallback: function getUpgradeItemSuccessCallback(data) {
    this.get('upgradeData.upgradeGroups').forEach(function (group) {
      if (group.get('group_id') === data.UpgradeItem.group_id) {
        group.get('upgradeItems').forEach(function (item) {
          if (item.get('stage_id') === data.UpgradeItem.stage_id) {
            if (item.get('tasks.length')) {
              data.tasks.forEach(function (task) {
                var currentTask = item.get('tasks').findProperty('id', task.Tasks.id);
                this.get('taskDetailsProperties').forEach(function (property) {
                  if (!Em.isNone(task.Tasks[property])) {
                    currentTask.set(property, task.Tasks[property]);
                  }
                }, this);
              }, this);
            } else {
              var tasks = [];
              data.tasks.forEach(function (task) {
                tasks.pushObject(App.upgradeEntity.create({
                  type: 'TASK',
                  group_id: data.UpgradeItem.group_id
                }, task.Tasks));
              });
              item.set('tasks', tasks);
            }
            item.set('isTasksLoaded', true);
          }
        }, this);
      }
    }, this);
  },

  /**
   * request Upgrade Task
   * @param {Em.Object} task
   * @return {$.ajax}
   */
  getUpgradeTask: function getUpgradeTask(task) {
    return App.ajax.send({
      name: 'admin.upgrade.upgrade_task',
      sender: this,
      data: {
        upgradeId: task.get('request_id'),
        groupId: task.get('group_id'),
        stageId: task.get('stage_id'),
        taskId: task.get('id'),
        task: task
      },
      success: 'getUpgradeTaskSuccessCallback'
    });
  },

  getUpgradeTaskSuccessCallback: function getUpgradeTaskSuccessCallback(data, xhr, params) {
    this.get('taskDetailsProperties').forEach(function (property) {
      params.task.set(property, data.Tasks[property]);
    }, this);
  },

  /**
   * Failures info may includes service_check and host_component failures. These two types should be displayed separately.
   */
  getServiceCheckItemSuccessCallback: function getServiceCheckItemSuccessCallback(data) {
    var task = data.tasks[0];
    var info = {
      hosts: [],
      host_detail: {}
    };

    if (task && task.Tasks && task.Tasks.structured_out && task.Tasks.structured_out.failures) {
      this.set('serviceCheckFailuresServicenames', task.Tasks.structured_out.failures.service_check || []);
      if (task.Tasks.structured_out.failures.host_component) {
        for (var hostname in task.Tasks.structured_out.failures.host_component) {
          info.hosts.push(hostname);
        }
        info.host_detail = task.Tasks.structured_out.failures.host_component;
      }
      this.set('slaveComponentStructuredInfo', info);
    }
  },

  getSlaveComponentItemSuccessCallback: function getSlaveComponentItemSuccessCallback(data) {
    var info = data.tasks[0];
    if (info && info.Tasks && info.Tasks.structured_out) {
      this.set('slaveComponentStructuredInfo', info.Tasks.structured_out);
    }
  },

  /**
   * downgrade confirmation popup
   * @param {object} event
   */
  confirmDowngrade: function confirmDowngrade(event) {
    var self = this;
    if (!this.get('currentVersion')) {
      this.updateCurrentStackVersion();
    }
    var currentVersion = this.get('currentVersion');
    return App.showConfirmationPopup(function () {
      self.downgrade.call(self, currentVersion, event);
    }, Em.I18n.t('admin.stackUpgrade.downgrade.body').format(currentVersion.repository_name), null, Em.I18n.t('admin.stackUpgrade.dialog.downgrade.header').format(this.get('upgradeVersion')), Em.I18n.t('admin.stackUpgrade.downgrade.proceed'));
  },

  /**
   * make call to start downgrade process
   * @param {object} currentVersion
   * @param {object} event
   */
  downgrade: function downgrade(currentVersion, event) {
    var self = this;
    this.set('requestInProgress', true);
    this.abortUpgrade().done(function () {
      var interval = setInterval(function () {
        if (self.get('upgradeData.Upgrade.request_status') == 'ABORTED') {
          clearInterval(interval);
          self.startDowngrade(currentVersion);
        }
      }, 1000);
    });
  },

  /**
   * abort upgrade (in order to start Downgrade)
   */
  abortUpgrade: function abortUpgrade() {
    var errorCallback = this.get('isDowngrade') ? 'abortDowngradeErrorCallback' : 'abortUpgradeErrorCallback';
    return App.ajax.send({
      name: 'admin.upgrade.abort',
      sender: this,
      data: {
        upgradeId: this.get('upgradeId'),
        isDowngrade: this.get('isDowngrade')
      },
      error: errorCallback
    });
  },

  /**
   * just request ro start downgrade,
   * should be performed only if <code>abortUpgrade<code> was completed
   */
  startDowngrade: function startDowngrade(currentVersion) {
    App.ajax.send({
      name: 'admin.downgrade.start',
      sender: this,
      data: {
        value: currentVersion.repository_version,
        label: this.get('upgradeVersion'),
        id: currentVersion.id,
        isDowngrade: true,
        upgradeType: this.get('upgradeType')
      },
      success: 'upgradeSuccessCallback',
      callback: function callback() {
        this.sender.set('requestInProgress', false);
      }
    });
  },

  /**
   * suspend upgrade (in order to restart it later)
   */
  abortUpgradeWithSuspend: function abortUpgradeWithSuspend() {
    var errorCallback = this.get('isDowngrade') ? 'abortDowngradeErrorCallback' : 'abortUpgradeErrorCallback';
    return App.ajax.send({
      name: 'admin.upgrade.suspend',
      sender: this,
      data: {
        upgradeId: this.get('upgradeId'),
        isDowngrade: this.get('isDowngrade')
      },
      error: errorCallback
    });
  },

  /**
   * error callback of <code>abortUpgrade()</code>
   * @param {object} data
   */
  abortUpgradeErrorCallback: function abortUpgradeErrorCallback(data) {
    var header = Em.I18n.t('admin.stackUpgrade.state.paused.fail.header');
    var body = Em.I18n.t('admin.stackUpgrade.state.paused.fail.body');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        body = body + ' ' + json.message;
      } catch (err) {}
    }
    App.showAlertPopup(header, body);
  },

  /**
   * error callback of <code>abortDowngrade()</code>
   * @param {object} data
   */
  abortDowngradeErrorCallback: function abortDowngradeErrorCallback(data) {
    var header = Em.I18n.t('admin.stackDowngrade.state.paused.fail.header');
    var body = Em.I18n.t('admin.stackDowngrade.state.paused.fail.body');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        body = body + ' ' + json.message;
      } catch (err) {}
    }
    App.showAlertPopup(header, body);
  },

  retryUpgrade: function retryUpgrade() {
    this.setProperties({
      requestInProgress: true,
      isRetryPending: true
    });
    return App.ajax.send({
      name: 'admin.upgrade.retry',
      sender: this,
      data: {
        upgradeId: this.get('upgradeId')
      }
    });
  },

  /**
   * make call to start upgrade process and show popup with current progress
   * @param {object} version
   */
  upgrade: function upgrade(version) {
    this.set('requestInProgress', true);
    App.ajax.send({
      name: 'admin.upgrade.start',
      sender: this,
      data: version,
      success: 'upgradeSuccessCallback',
      error: 'upgradeErrorCallback',
      callback: function callback() {
        this.sender.set('requestInProgress', false);
      }
    });
    this.setDBProperty('currentVersion', this.get('currentVersion'));

    // Show a "preparing the upgrade..." dialog in case the api call returns too slow
    if (App.router.get('currentState.name') != 'stackUpgrade') {
      this.showPreparingUpgradeIndicator();
    }
  },

  /**
   * Should progress bar be displayed when preparing upgrade,
   * should show after Upgrade Options window and before Upgrade Wizard
   * @method showPreparingUpgradeIndicator
   */
  showPreparingUpgradeIndicator: function showPreparingUpgradeIndicator() {
    return App.ModalPopup.show({
      header: '',
      showFooter: false,
      showCloseButton: false,
      bodyClass: Em.View.extend({
        templateName: require('templates/wizard/step8/step8_log_popup'),
        controllerBinding: 'App.router.mainAdminStackAndUpgradeController',

        /**
         * Css-property for progress-bar
         * @type {string}
         */
        barWidth: 'width: 100%;',
        progressBarClass: 'progress log_popup',

        /**
         * Popup-message
         * @type {string}
         */
        message: Em.I18n.t('admin.stackUpgrade.dialog.prepareUpgrade.header'),

        /**
         * Hide popup when upgrade wizard is open
         * @method autoHide
         */
        autoHide: function () {
          if (!this.get('controller.requestInProgress')) {
            this.get('parentView').hide();
          }
        }.observes('controller.requestInProgress')
      })
    });
  },

  /**
   * error callback of <code>upgrade()</code>
   * @param {object} data
   */
  upgradeErrorCallback: function upgradeErrorCallback(data) {
    var header = Em.I18n.t('admin.stackVersions.upgrade.start.fail.title');
    var body = "";
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        body = json.message;
      } catch (err) {}
    }
    App.showAlertPopup(header, body);
  },

  /**
   * success callback of <code>upgrade()</code>
   * @param {object} data
   */
  upgradeSuccessCallback: function upgradeSuccessCallback(data, opt, params) {
    this.set('upgradeData', null);
    this.set('upgradeId', data.resources[0].Upgrade.request_id);
    this.set('toVersion', params.value);
    this.set('upgradeVersion', params.label);
    this.set('isDowngrade', !!params.isDowngrade);
    var upgradeMethod = this.get('upgradeMethods').findProperty('type', params.type),
        upgradeTypeDisplayName = null,
        upgradeType = null,
        isWizardRestricted = false;

    if (upgradeMethod) {
      upgradeTypeDisplayName = upgradeMethod.get('displayName');
      upgradeType = upgradeMethod.get('type');
      isWizardRestricted = upgradeMethod.get('isWizardRestricted');
    }

    this.set('isWizardRestricted', isWizardRestricted);
    this.set('upgradeType', upgradeType);
    this.set('upgradeTypeDisplayName', upgradeTypeDisplayName);
    this.set('failuresTolerance', Em.Object.create({
      skipComponentFailures: params.skipComponentFailures == 'true',
      skipSCFailures: params.skipSCFailures == 'true'
    }));
    this.setDBProperties({
      upgradeVersion: params.label,
      upgradeId: data.resources[0].Upgrade.request_id,
      toVersion: params.value,
      upgradeState: 'PENDING',
      isDowngrade: !!params.isDowngrade,
      upgradeType: upgradeType,
      isWizardRestricted: isWizardRestricted,
      upgradeTypeDisplayName: upgradeTypeDisplayName,
      failuresTolerance: Em.Object.create({
        skipComponentFailures: params.skipComponentFailures == 'true',
        skipSCFailures: params.skipSCFailures == 'true'
      })
    });
    App.set('upgradeState', 'PENDING');
    App.clusterStatus.setClusterStatus({
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
    this.load();
    App.router.get('wizardWatcherController').setUser(App.router.get('mainAdminStackAndUpgradeController').get('name'));
    this.openUpgradeDialog();
  },

  /**
   * success callback of updating upgrade options including failures tolerance. etc
   * @param {object} data
   */
  updateOptionsSuccessCallback: function updateOptionsSuccessCallback(data, opt, params) {
    this.set('failuresTolerance', Em.Object.create({
      skipComponentFailures: params.skipComponentFailures == 'true',
      skipSCFailures: params.skipSCFailures == 'true'
    }));
  },

  /**
   * run upgrade checks and add results to each method object and set selected method
   * @param {Em.Object} version
   */
  runUpgradeMethodChecks: function runUpgradeMethodChecks(version) {
    this.get('upgradeMethods').forEach(function (method) {
      if (method.get('allowed')) {
        this.runPreUpgradeCheckOnly({
          id: version.get('id'),
          label: version.get('displayName'),
          type: method.get('type')
        });
      } else {
        //if method not supported in current stack version, mark as check completed
        method.setProperties({
          isCheckComplete: false,
          isCheckRequestInProgress: false,
          action: ''
        });
      }
    }, this);
  },

  getConfigsWarnings: function getConfigsWarnings(configsMergeWarning) {
    var configs = [];
    if (configsMergeWarning && Em.get(configsMergeWarning, 'UpgradeChecks.status') === 'WARNING') {
      var configsMergeCheckData = Em.get(configsMergeWarning, 'UpgradeChecks.failed_detail');
      if (configsMergeCheckData && Em.isArray(configsMergeCheckData)) {
        configs = configsMergeCheckData.reduce(function (allConfigs, item) {
          var isDeprecated = Em.isNone(item.new_stack_value),
              willBeRemoved = Em.isNone(item.result_value);

          return allConfigs.concat({
            type: item.type,
            name: item.property,
            wasModified: !isDeprecated && !willBeRemoved && Em.compare(item.current, item.result_value) === 0,
            currentValue: item.current,
            recommendedValue: isDeprecated ? Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.deprecated') : item.new_stack_value,
            isDeprecated: isDeprecated,
            resultingValue: willBeRemoved ? Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.willBeRemoved') : item.result_value,
            willBeRemoved: willBeRemoved
          });
        }, []);
      }
    }
    return configs;
  },

  showUpgradeOptions: function showUpgradeOptions(version) {
    this.upgradeOptions(false, version, true);
  },

  /**
   * Open upgrade options window: upgrade type and failures tolerance
   * @param {boolean} isInUpgradeWizard
   * @param {object} version
   * @return App.ModalPopup
   */
  upgradeOptions: function upgradeOptions(isInUpgradeWizard, version, preUpgradeShow) {
    var self = this,
        upgradeMethods = this.get('upgradeMethods'),
        runningCheckRequests = this.get('runningCheckRequests');
    if (!isInUpgradeWizard) {
      upgradeMethods.setEach('isCheckRequestInProgress', true);
      upgradeMethods.setEach('selected', false);
      var request = this.getSupportedUpgradeTypes(Ember.Object.create({
        stackName: App.get('currentStackVersion').split('-')[0],
        stackVersion: App.get('currentStackVersion').split('-')[1],
        toVersion: version.get('repositoryVersion')
      })).done(function () {
        if (App.get('router.currentState.name') === 'versions' && App.get('router.currentState.parentState.name') === 'stackAndUpgrade') {
          self.runUpgradeMethodChecks(version);
        }
      }).always(function () {
        self.set('isUpgradeTypesLoaded', true);
        self.set('runningCheckRequests', runningCheckRequests.rejectProperty('type', 'ALL'));
      });
      request.type = 'ALL';
      this.get('runningCheckRequests').push(request);
    } else {
      this.set('isUpgradeTypesLoaded', true);
    }

    return App.ModalPopup.show({
      encodeBody: false,
      primary: function () {
        if (preUpgradeShow) return false;
        if (isInUpgradeWizard || this.get('controller.getSupportedUpgradeError')) return Em.I18n.t('ok');
        return Em.I18n.t('common.proceed');
      }.property('controller.getSupportedUpgradeError'),
      secondary: function () {
        if (preUpgradeShow) return Em.I18n.t('common.dismiss');
        if (this.get('controller.getSupportedUpgradeError')) return null;
        return Em.I18n.t('common.cancel');
      }.property('controller.getSupportedUpgradeError'),
      secondaryClass: preUpgradeShow ? 'btn-success' : '',
      classNames: ['upgrade-options-popup'],
      header: preUpgradeShow ? Em.I18n.t('admin.stackVersions.version.preUpgrade.header') : Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.header'),
      controller: this,
      showFooter: function () {
        return this.get('controller.isUpgradeTypesLoaded') || preUpgradeShow;
      }.property('controller.isUpgradeTypesLoaded'),
      bodyClass: Em.View.extend({
        templateName: require('templates/main/admin/stack_upgrade/upgrade_options'),
        didInsertElement: function didInsertElement() {
          App.tooltip($(".failure-tolerance-tooltip"), {
            placement: "top",
            title: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.tolerance.tooltip')
          });
          Em.run.later(this, function () {
            App.tooltip($(".img-thumbnail.check-failed"), {
              placement: "bottom",
              title: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.failed.tooltip')
            });
            App.tooltip($(".not-allowed-by-version"), {
              placement: "bottom",
              title: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.notAllowed')
            });
          }, 1000);
        },
        upgradeMethods: function () {
          self.updateSelectedMethod(isInUpgradeWizard);
          return self.get('upgradeMethods');
        }.property().volatile(),
        isInUpgradeWizard: isInUpgradeWizard,
        showPreUpgradeChecks: App.get('supports.preUpgradeCheck') && !isInUpgradeWizard,
        versionText: function () {
          if (preUpgradeShow) return Em.I18n.t('admin.stackVersions.version.preUpgrade.bodyMsg.version').format(version.get('displayName'));
          if (isInUpgradeWizard) return '';
          return Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.version').format(version.get('displayName'));
        }(),
        selectMethod: function selectMethod(event) {
          if (isInUpgradeWizard || !event.context.get('allowed') || event.context.get('isPrecheckFailed')) return;
          var selectedMethod = event.context;
          self.updateSelectedMethod(isInUpgradeWizard);
          self.get('upgradeMethods').forEach(function (method) {
            method.set('selected', false);
          });
          selectedMethod.set('selected', true);
          this.set('parentView.selectedMethod', selectedMethod);
        },
        runAction: function runAction(event) {
          var method = event.context,
              action = method.get('action');
          if (action) {
            this.get(action)(event);
          }
        },
        rerunCheck: function rerunCheck(event) {
          self.runPreUpgradeCheckOnly({
            id: version.get('id'),
            label: version.get('displayName'),
            type: event.context.get('type')
          });
        },
        openMessage: function openMessage(event) {
          if (isInUpgradeWizard || !event.context.get('allowed')) return;
          var data = event.context.get('precheckResultsData');

          var failTitle = Em.I18n.t('popup.clusterCheck.Upgrade.fail.title');
          var failAlert = new Em.Handlebars.SafeString(Em.I18n.t('popup.clusterCheck.Upgrade.fail.alert'));
          var bypassedFailures = data.items.filterProperty('UpgradeChecks.status', 'BYPASS').length > 0;
          if (data.items.filterProperty('UpgradeChecks.status', 'ERROR').length == 0 && bypassedFailures) {
            failTitle = Em.I18n.t('popup.clusterCheck.Upgrade.bypassed-failures.title');
            failAlert = new Em.Handlebars.SafeString(Em.I18n.t('popup.clusterCheck.Upgrade.bypassed-failures.alert'));
          }

          var header = Em.I18n.t('popup.clusterCheck.Upgrade.header').format(version.get('displayName')),
              warningTitle = Em.I18n.t('popup.clusterCheck.Upgrade.warning.title'),
              warningAlert = new Em.Handlebars.SafeString(Em.I18n.t('popup.clusterCheck.Upgrade.warning.alert')),
              configsMergeWarning = data.items.findProperty('UpgradeChecks.id', "CONFIG_MERGE"),
              popupData = {
            items: data.items.rejectProperty('UpgradeChecks.id', 'CONFIG_MERGE')
          },
              configs = self.getConfigsWarnings(configsMergeWarning);
          App.showClusterCheckPopup(popupData, {
            header: header,
            failTitle: failTitle,
            failAlert: failAlert,
            warningTitle: warningTitle,
            warningAlert: warningAlert,
            primary: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.rerun'),
            secondary: Em.I18n.t('common.cancel'),
            bypassedFailures: bypassedFailures,
            callback: function callback() {
              self.runPreUpgradeCheckOnly.call(self, {
                id: version.get('id'),
                label: version.get('displayName'),
                type: event.context.get('type')
              });
            }
          }, configs);
        },
        upgradeShow: !preUpgradeShow
      }),

      /**
       * @type {Em.Object}
       * @default null
       */
      selectedMethod: null,
      skipComponentFailures: self.get('failuresTolerance.skipComponentFailures'),
      skipSCFailures: self.get('failuresTolerance.skipSCFailures'),
      disablePrimary: function () {
        if (isInUpgradeWizard || this.get('controller.getSupportedUpgradeError')) return false;
        var selectedMethod = this.get('selectedMethod');
        if (selectedMethod) {
          if (App.get('supports.preUpgradeCheck')) {
            return selectedMethod.get('isPrecheckFailed') || selectedMethod.get('isCheckRequestInProgress');
          } else {
            return false;
          }
        } else {
          return true;
        }
      }.property('selectedMethod', 'selectedMethod.isPrecheckFailed', 'selectedMethod.isCheckRequestInProgress'),
      onPrimary: function onPrimary() {
        this.hide();
        if (isInUpgradeWizard) {
          return App.ajax.send({
            name: 'admin.upgrade.update.options',
            sender: self,
            data: {
              upgradeId: self.get('upgradeId'),
              skipComponentFailures: Boolean(this.get('skipComponentFailures')).toString(),
              skipSCFailures: Boolean(this.get('skipSCFailures')).toString()
            },
            success: 'updateOptionsSuccessCallback'
          });
        } else {
          var upgradeMethod = self.get('upgradeMethods').findProperty('selected');
          version.upgradeType = upgradeMethod.get('type');
          version.upgradeTypeDisplayName = upgradeMethod.get('displayName');
          version.skipComponentFailures = this.get('skipComponentFailures');
          version.skipSCFailures = this.get('skipSCFailures');

          var toVersion = version.get('displayName');
          var bodyMessage = Em.Object.create({
            confirmButton: Em.I18n.t('yes'),
            confirmMsg: upgradeMethod.get('type') === 'ROLLING' ? Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.RU.confirm.msg').format(toVersion) : Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.EU.confirm.msg').format(toVersion)
          });
          return App.showConfirmationFeedBackPopup(function (query) {
            return self.runPreUpgradeCheck.call(self, version);
          }, bodyMessage);
        }
      }
    });
  },

  /**
   * open upgrade options from upgrade wizard
   */
  openUpgradeOptions: function openUpgradeOptions() {
    if (this.get('isDowngrade')) return;
    this.upgradeOptions(true, null);
  },

  /**
   * upgrade confirmation popup including upgrade options: upgrade type and failures tolerance
   * @param {object} version
   * @return App.ModalPopup
   */
  confirmUpgrade: function confirmUpgrade(version) {
    this.upgradeOptions(false, version);
  },

  /**
   * send request for pre upgrade check only
   */
  runPreUpgradeCheckOnly: function runPreUpgradeCheckOnly(data) {
    if (App.get('supports.preUpgradeCheck')) {
      var method = this.get('upgradeMethods').findProperty('type', data.type);
      method.setProperties({
        isCheckComplete: false,
        isCheckRequestInProgress: true,
        action: ''
      });
      var request = App.ajax.send({
        name: "admin.upgrade.pre_upgrade_check",
        sender: this,
        data: data,
        success: 'runPreUpgradeCheckOnlySuccess',
        error: 'runPreUpgradeCheckOnlyError',
        callback: function callback() {
          var runningCheckRequests = this.sender.get('runningCheckRequests');
          method.set('isCheckRequestInProgress', false);
          this.sender.set('runningCheckRequests', runningCheckRequests.rejectProperty('type', this.data.type));
        }
      });
      request.type = data.type;
      this.get('runningCheckRequests').push(request);
    }
  },

  /**
   * send request to get available upgrade tye names
   */
  getSupportedUpgradeTypes: function getSupportedUpgradeTypes(data) {
    this.set('isUpgradeTypesLoaded', false);
    this.set('getSupportedUpgradeError', '');
    return App.ajax.send({
      name: "admin.upgrade.get_supported_upgradeTypes",
      sender: this,
      data: data,
      success: "getSupportedUpgradeTypesSuccess",
      error: "getSupportedUpgradeTypesError"
    });
  },

  /**
   * success callback of <code>getSupportedUpgradeTypes()</code>
   * @param data {object}
   */
  getSupportedUpgradeTypesSuccess: function getSupportedUpgradeTypesSuccess(data) {
    var supportedUpgradeTypes = data.items[0] && data.items[0].CompatibleRepositoryVersions.upgrade_types;
    this.get('upgradeMethods').forEach(function (method) {
      method.set('allowed', Boolean(supportedUpgradeTypes && supportedUpgradeTypes.contains(method.get('type'))));
    });
  },

  /**
   * error callback of <code>getSupportedUpgradeTypes()</code>
   * @param xhr {object}
   */
  getSupportedUpgradeTypesError: function getSupportedUpgradeTypesError(xhr) {
    var response;
    try {
      response = JSON.parse(xhr.responseText);
    } catch (e) {
      response = { message: xhr.statusText };
    }
    this.set('getSupportedUpgradeError', response.message);
  },

  /**
   * success callback of <code>runPreUpgradeCheckOnly()</code>
   * Show a message how many fails/warnings/bypass/passed
   * on clicking that message a popup window show up
   * @param data {object}
   * @param opt {object}
   * @param params {object}
   */
  runPreUpgradeCheckOnlySuccess: function runPreUpgradeCheckOnlySuccess(data, opt, params) {
    var properties = {
      precheckResultsTitle: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.title'),
      precheckResultsData: data,
      isCheckComplete: true,
      action: 'openMessage',
      precheckResultsMessage: '',
      recheckResultsMessageClass: 'GREEN',
      isPrecheckFailed: false,
      precheckResultsMessageIconClass: 'glyphicon glyphicon-ok',
      bypassedFailures: false
    };

    Object.keys(this.get('preCheckMessages')).forEach(function (status) {
      if (data.items.someProperty('UpgradeChecks.status', status)) {
        properties = this.formatPreCheckMessage(status, data, properties);
      }
    }, this);

    if (!properties.precheckResultsMessage) {
      properties.precheckResultsMessage = Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.allPassed');
    }
    this.get('upgradeMethods').findProperty('type', params.type).setProperties(properties);
    this.updateSelectedMethod(false);
    this.addPrecheckMessageTooltip();
  },

  /**
   * @method formatPreCheckMessage
   * @param {string} type
   * @param {object} data
   * @param {object} defaults
   * @returns {object}
   */
  formatPreCheckMessage: function formatPreCheckMessage(type, data, defaults) {
    var length = data.items.filterProperty('UpgradeChecks.status', type).length;
    var properties = this.get('preCheckMessages')[type] || {};
    var message = Em.I18n.t(properties.template).format(length, defaults.precheckResultsMessage);
    defaults = $.extend(defaults, properties);
    delete defaults.template;
    defaults.precheckResultsMessage = message;
    return defaults;
  },

  addPrecheckMessageTooltip: function addPrecheckMessageTooltip() {
    Em.run.later(this, function () {
      // add tooltip for the type with preCheck errors
      App.tooltip($(".img-thumbnail.check-failed"), {
        placement: "bottom",
        title: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.failed.tooltip')
      });
      // destroy the tooltip for the type wo preCheck errors
      $(".img-thumbnail").not(".check-failed").not(".not-allowed-by-version").tooltip("destroy");
    }, 1000);
  },

  runPreUpgradeCheckOnlyError: function runPreUpgradeCheckOnlyError(request, ajaxOptions, error, data, params) {
    var method = this.get('upgradeMethods').findProperty('type', params.type);
    method.setProperties({
      precheckResultsMessage: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.failed.link'),
      precheckResultsTitle: Em.I18n.t('admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.failed.title'),
      precheckResultsMessageClass: 'RED',
      isPrecheckFailed: true,
      precheckResultsMessageIconClass: 'glyphicon glyphicon-warning-sign',
      action: 'rerunCheck'
    });
  },

  /**
   * In Upgrade Wizard: update which method already been selected on open
   * Not in upgrade wizard: de-select the method with pre-check errors
   * @param isInUpgradeWizard {boolean}
   */
  updateSelectedMethod: function updateSelectedMethod(isInUpgradeWizard) {
    if (isInUpgradeWizard) {
      this.get('upgradeMethods').forEach(function (method) {
        method.set('selected', method.get('type') === this.get('upgradeType'));
      }, this);
    } else {
      var ruMethod = this.get('upgradeMethods').findProperty('type', 'ROLLING');
      var euMethod = this.get('upgradeMethods').findProperty('type', 'NON_ROLLING');
      if (ruMethod && ruMethod.get('isPrecheckFailed')) ruMethod.set('selected', false);
      if (euMethod && euMethod.get('isPrecheckFailed')) euMethod.set('selected', false);
    }
  },

  /**
   * send request for pre upgrade check
   * @param version
   */
  runPreUpgradeCheck: function runPreUpgradeCheck(version) {
    var params = {
      value: version.get('repositoryVersion'),
      label: version.get('displayName'),
      type: version.get('upgradeType'),
      skipComponentFailures: version.get('skipComponentFailures') ? 'true' : 'false',
      skipSCFailures: version.get('skipSCFailures') ? 'true' : 'false',
      id: version.get('id'),
      targetStack: version.get('displayName')
    };
    if (App.get('supports.preUpgradeCheck')) {
      this.set('requestInProgress', true);
      App.ajax.send({
        name: "admin.upgrade.pre_upgrade_check",
        sender: this,
        data: params,
        success: "runPreUpgradeCheckSuccess",
        error: "runPreUpgradeCheckError"
      });
    } else {
      this.upgrade(params);
    }
  },

  /**
   * success callback of <code>runPreUpgradeCheckSuccess()</code>
   * if there are some fails - it shows popup else run upgrade
   * @param data {object}
   * @param opt {object}
   * @param params {object}
   * @returns {App.ModalPopup|undefined}
   */
  runPreUpgradeCheckSuccess: function runPreUpgradeCheckSuccess(data, opt, params) {
    var self = this;
    if (data.items.someProperty('UpgradeChecks.status', 'FAIL') || data.items.someProperty('UpgradeChecks.status', 'WARNING') || data.items.someProperty('UpgradeChecks.status', 'BYPASS')) {
      this.set('requestInProgress', false);
      var hasFails = data.items.someProperty('UpgradeChecks.status', 'FAIL'),
          header = Em.I18n.t('popup.clusterCheck.Upgrade.header').format(params.label),
          failTitle = Em.I18n.t('popup.clusterCheck.Upgrade.fail.title'),
          failAlert = new Em.Handlebars.SafeString(Em.I18n.t('popup.clusterCheck.Upgrade.fail.alert')),
          warningTitle = Em.I18n.t('popup.clusterCheck.Upgrade.warning.title'),
          warningAlert = new Em.Handlebars.SafeString(Em.I18n.t('popup.clusterCheck.Upgrade.warning.alert')),
          bypassedFailures = data.items.someProperty('UpgradeChecks.status', 'BYPASS').length > 0,
          configsMergeWarning = data.items.findProperty('UpgradeChecks.id', 'CONFIG_MERGE'),
          popupData = {
        items: data.items.rejectProperty('UpgradeChecks.id', 'CONFIG_MERGE')
      },
          configs = this.getConfigsWarnings(configsMergeWarning);
      App.showClusterCheckPopup(popupData, {
        header: header,
        failTitle: failTitle,
        failAlert: failAlert,
        warningTitle: warningTitle,
        warningAlert: warningAlert,
        bypassedFailures: bypassedFailures,
        noCallbackCondition: hasFails,
        callback: function callback() {
          self.upgrade(params);
        }
      }, configs, params.label);
    } else {
      this.upgrade(params);
    }
  },

  runPreUpgradeCheckError: function runPreUpgradeCheckError() {
    this.set('requestInProgress', false);
  },

  confirmRetryUpgrade: function confirmRetryUpgrade(version) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.retryUpgrade();
    }, Em.I18n.t('admin.stackUpgrade.upgrade.retry.confirm.body').format(version.get('displayName')), null, this.getUpgradeDowngradeHeader(version.get('upgradeTypeDislayName'), version.get('displayName')));
  },

  confirmRetryDowngrade: function confirmRetryDowngrade() {
    var self = this,
        currentVersion = this.get('currentVersion');
    return App.showConfirmationPopup(function () {
      self.retryUpgrade();
    }, Em.I18n.t('admin.stackUpgrade.downgrade.retry.body').format(currentVersion.repository_name), null, Em.I18n.t('admin.stackUpgrade.dialog.downgrade.header').format(currentVersion.repository_name), Em.I18n.t('admin.stackUpgrade.downgrade.proceed'));
  },

  /**
   * confirmation popup before install repository version
   */
  installRepoVersionPopup: function installRepoVersionPopup(repo) {
    var availableServices = repo.get('stackServices').filter(function (service) {
      return App.Service.find(service.get('name')).get('isLoaded') && service.get('isAvailable') && service.get('isUpgradable');
    }, this);
    if (!availableServices.length && !repo.get('isStandard')) {
      return App.showAlertPopup(Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.title'), Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.noAvailableServices').format(repo.get('displayName')));
    }
    var self = this;
    var bodyContent = repo.get('isPatch') ? Em.I18n.t('admin.stackVersions.version.install.patch.confirm') : Em.I18n.t('admin.stackVersions.version.install.confirm');
    return App.ModalPopup.show({
      header: Em.I18n.t('popup.confirmation.commonHeader'),
      popupBody: bodyContent.format(repo.get('displayName')),
      skipDependencyCheck: false,
      bodyClass: Em.View.extend({
        classNames: ['install-repo-confirmation'],
        content: availableServices,
        showAvailableServices: repo.get('isPatch') || repo.get('isMaint'),
        templateName: require('templates/common/modal_popups/install_repo_confirmation')
      }),
      onPrimary: function onPrimary() {
        self.installRepoVersion(repo);
        this._super();
      }
    });
  },

  /**
   * sends request to install repoVersion to the cluster
   * and create clusterStackVersion resourse
   * @param {Em.Object} repo
   * @return {$.ajax}
   * @method installRepoVersion
   */
  installRepoVersion: function installRepoVersion(repo) {
    this.set('requestInProgress', true);
    this.set('requestInProgressRepoId', repo.get('id'));

    var data = {
      ClusterStackVersions: {
        stack: repo.get('stackVersionType'),
        version: repo.get('stackVersionNumber'),
        repository_version: repo.get('repositoryVersion')
      },
      id: repo.get('id')
    };
    return App.ajax.send({
      name: 'admin.stack_version.install.repo_version',
      sender: this,
      data: data,
      success: 'installRepoVersionSuccess',
      error: 'installRepoVersionError',
      callback: function callback() {
        this.sender.set('requestInProgress', false);
        this.sender.set('requestInProgressRepoId', null);
      }
    });
  },

  /**
   * TODO unify and move modal view into common/modal_popups
   * @param {string[]} hosts
   * @param {string} title
   * @param {string} operation
   * @param {Function} primary
   */
  showReinstallRemoveConfirmation: function showReinstallRemoveConfirmation(_ref) {
    var hosts = _ref.hosts,
        title = _ref.title,
        operation = _ref.operation,
        _ref$primary = _ref.primary,
        primary = _ref$primary === undefined ? function () {} : _ref$primary;

    return App.ModalPopup.show({
      header: title,
      visibleHosts: hosts.join("\n"),
      expanded: true,
      onPrimary: function onPrimary() {
        primary(hosts);
        this._super();
      },

      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/bulk_operation_confirm_popup'),
        message: Em.I18n.t('hosts.bulkOperation.confirmation.hosts').format(operation, hosts.length),
        textareaVisible: false,
        textTrigger: function textTrigger() {
          this.toggleProperty('textareaVisible');
        },
        putHostNamesToTextarea: function () {
          var hostNames = this.get('parentView.visibleHosts');
          if (this.get('textareaVisible')) {
            var wrapper = $(".task-detail-log-maintext");
            $('.task-detail-log-clipboard').html(hostNames).width(wrapper.width()).height(250);
            Em.run.next(function () {
              $('.task-detail-log-clipboard').select();
            });
          }
        }.observes('textareaVisible')
      })
    });
  },

  removeOutOfSyncComponents: function removeOutOfSyncComponents(event) {
    var _this = this;

    var hosts = App.RepositoryVersion.find(event.context.repoId).get('stackVersion.outOfSyncHosts');
    return this.showReinstallRemoveConfirmation({
      hosts: hosts,
      title: Em.I18n.t('admin.stackVersions.version.errors.outOfSync.remove.title'),
      operation: Em.I18n.t('hosts.host.maintainance.removeFailedComponents.context'),
      primary: function primary() {
        App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
          App.ajax.send({
            name: 'host.host_component.delete_components',
            sender: _this,
            data: {
              hosts: hosts,
              data: JSON.stringify({
                RequestInfo: {
                  query: 'HostRoles/host_name.in(' + hosts.join(',') + ')&HostRoles/state=INSTALL_FAILED'
                }
              })
            }
          });
        });
      }
    });
  },

  reinstallOutOfSyncComponents: function reinstallOutOfSyncComponents(event) {
    var _this2 = this;

    var hosts = App.RepositoryVersion.find(event.context.repoId).get('stackVersion.outOfSyncHosts');
    return this.showReinstallRemoveConfirmation({
      hosts: hosts,
      title: Em.I18n.t('admin.stackVersions.version.errors.outOfSync.reinstall.title'),
      operation: Em.I18n.t('hosts.host.maintainance.reinstallFailedComponents.context'),
      primary: function primary() {
        App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
          App.ajax.send({
            name: 'common.host_components.update',
            sender: _this2,
            data: {
              HostRoles: {
                state: 'INSTALLED'
              },
              query: 'HostRoles/host_name.in(' + hosts.join(',') + ')&HostRoles/state=INSTALL_FAILED',
              context: Em.I18n.t('hosts.host.maintainance.reinstallFailedComponents.context')
            },
            success: 'reinstallOutOfSyncComponentsSuccessCallback'
          });
        });
      }
    });
  },

  reinstallOutOfSyncComponentsSuccessCallback: function reinstallOutOfSyncComponentsSuccessCallback(data, opt, params, req) {
    if (!data && req.status == 200) {
      return App.ModalPopup.show({
        header: Em.I18n.t('rolling.nothingToDo.header'),
        body: Em.I18n.t('rolling.nothingToDo.body').format(params.noOpsMessage || Em.I18n.t('hosts.host.maintainance.allComponents.context')),
        secondary: false
      });
    }
    return App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  /**
   * transform repo data into json for
   * saving changes to repository version
   * @param {Em.Object} repo
   * @returns {{operating_systems: Array}}
   */
  prepareRepoForSaving: function prepareRepoForSaving(repo) {
    var repoVersion = { "operating_systems": [] };
    var ambari_managed_repositories = !repo.get('useRedhatSatellite');
    repo.get('operatingSystems').forEach(function (os, k) {
      repoVersion.operating_systems.push({
        "OperatingSystems": {
          "os_type": os.get("osType"),
          "ambari_managed_repositories": ambari_managed_repositories
        },
        "repositories": []
      });
      os.get('repositories').forEach(function (repository) {
        repoVersion.operating_systems[k].repositories.push({
          "Repositories": {
            "base_url": repository.get('baseUrl'),
            "repo_id": repository.get('repoId'),
            "repo_name": repository.get('repoName')
          }
        });
      });
    });
    return repoVersion;
  },

  /**
   * Return stack version for the repo object
   * @param {Em.Object} repo
   * */
  getStackVersionNumber: function getStackVersionNumber(repo) {
    var stackVersionNumber = repo.get('stackVersion');
    if (null == stackVersionNumber) stackVersionNumber = App.get('currentStackVersion');
    return stackVersionNumber;
  },

  /**
   * perform validation if <code>skip<code> is  false and run save if
   * validation successfull or run save without validation is <code>skip<code> is true
   * @param {Em.Object} repo
   * @param {boolean} skip
   * @returns {$.Deferred}
   */
  saveRepoOS: function saveRepoOS(repo, skip) {
    var self = this;
    var deferred = $.Deferred();
    this.validateRepoVersions(repo, skip).done(function (data) {
      if (data.length > 0) {
        deferred.resolve(data);
      } else {
        var repoVersion = self.prepareRepoForSaving(repo);
        var stackVersionNumber = self.getStackVersionNumber(repo);

        App.ajax.send({
          name: 'admin.stack_versions.edit.repo',
          sender: this,
          data: {
            stackName: App.get('currentStackName'),
            stackVersion: stackVersionNumber,
            repoVersionId: repo.get('repoVersionId'),
            repoVersion: repoVersion
          }
        }).success(function () {
          deferred.resolve([]);
        });
      }
    });
    return deferred.promise();
  },

  /**
   * send request for validation for each repository
   * @param {Em.Object} repo
   * @param {boolean} skip
   * @returns {*}
   */
  validateRepoVersions: function validateRepoVersions(repo, skip) {
    var deferred = $.Deferred(),
        totalCalls = 0,
        invalidUrls = [];

    if (skip) {
      deferred.resolve(invalidUrls);
    } else {
      var stackVersionNumber = this.getStackVersionNumber(repo);
      repo.get('operatingSystems').forEach(function (os) {
        if (os.get('isSelected')) {
          os.get('repositories').forEach(function (repo) {
            totalCalls++;
            this.validationCall(repo, os, stackVersionNumber).success(function () {
              totalCalls--;
              if (totalCalls === 0) deferred.resolve(invalidUrls);
            }).error(function () {
              repo.set('hasError', true);
              invalidUrls.push(repo);
              totalCalls--;
              if (totalCalls === 0) deferred.resolve(invalidUrls);
            });
          }, this);
        } else {
          return deferred.resolve(invalidUrls);
        }
      }, this);
    }
    return deferred.promise();
  },

  /**
   *
   * @param {Em.Object} repo
   * @param {Em.Object} os
   * @param {string} stackVersionNumber
   */
  validationCall: function validationCall(repo, os, stackVersionNumber) {
    return App.ajax.send({
      name: 'admin.stack_versions.validate.repo',
      sender: this,
      data: {
        repo: repo,
        repoName: repo.get('repoName'),
        repoId: repo.get('repoId'),
        baseUrl: repo.get('baseUrl'),
        osType: os.get('osType'),
        stackName: App.get('currentStackName'),
        stackVersion: stackVersionNumber
      }
    });
  },

  /**
   * success callback for <code>installRepoVersion()<code>
   * saves request id to the db
   * @param data
   * @param opt
   * @param params
   * @method installStackVersionSuccess
   */
  installRepoVersionSuccess: function installRepoVersionSuccess(data, opt, params) {
    if (data && data.statusText === "timeout") {
      App.showAlertPopup(Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.title'), Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.timeout'));
      return false;
    }
    var version = App.RepositoryVersion.find(params.id);
    App.db.set('repoVersionInstall', 'id', [data.Requests.id]);
    App.clusterStatus.setClusterStatus({
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
    version.set('defaultStatus', 'INSTALLING');
    if (version.get('stackVersion')) {
      version.set('stackVersion.state', 'INSTALLING');
    }
  },

  /**
   * error callback for <code>installRepoVersion()<code>
   * show the error message
   * @param data
   * @method installStackVersionSuccess
   */
  installRepoVersionError: function installRepoVersionError(data, opt, params) {
    var header = Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.title');
    var body = "";
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        body = json.message;
      } catch (err) {}
    }
    if (data && data.statusText === "timeout") {
      body = Em.I18n.t('admin.stackVersions.upgrade.installPackage.fail.timeout');
    }
    var version = App.RepositoryVersion.find(params.id);
    version.set('defaultStatus', 'INSTALL_FAILED');
    if (version.get('stackVersion')) {
      version.set('stackVersion.state', 'INSTALL_FAILED');
    }
    App.showAlertPopup(header, body);
  },

  /**
   * opens a popup with installations state per host
   * @param {Em.Object} version
   * @method showProgressPopup
   */
  showProgressPopup: function showProgressPopup(version) {
    var popupTitle = Em.I18n.t('admin.stackVersions.details.install.hosts.popup.title').format(version.get('displayName'));
    var requestIds = this.getRepoVersionInstallId();
    var hostProgressPopupController = App.router.get('highAvailabilityProgressPopupController');
    hostProgressPopupController.initPopup(popupTitle, requestIds, this);
  },

  getRepoVersionInstallId: function getRepoVersionInstallId() {
    if (App.get('testMode')) return [1];

    var requestIds = App.db.get('repoVersionInstall', 'id');
    var lastRepoVersionInstall = App.router.get('backgroundOperationsController.services').find(function (request) {
      return request.get('name').startsWith('Install version');
    });
    if (lastRepoVersionInstall && (!requestIds || !requestIds.contains(lastRepoVersionInstall.get('id')))) {
      requestIds = [lastRepoVersionInstall.get('id')];
    }
    return requestIds || [];
  },

  /**
   * reset upgradeState to NOT_REQUIRED when upgrade is COMPLETED
   * and clean auxiliary data
   */
  finish: function finish() {
    var upgradeVersion = this.get('upgradeVersion') && this.get('upgradeVersion').match(/[a-zA-Z]+\-\d+\.\d+/);
    this.setDBProperties({
      fromVersion: undefined,
      upgradeId: undefined,
      upgradeState: 'NOT_REQUIRED',
      upgradeVersion: undefined,
      currentVersion: undefined,
      upgradeTypeDisplayName: undefined,
      upgradeType: undefined,
      isWizardRestricted: false,
      failuresTolerance: undefined,
      isDowngrade: undefined,
      downgradeAllowed: undefined
    });
    this.initDBProperties();
    App.clusterStatus.setClusterStatus({
      localdb: App.db.data
    });
    if (upgradeVersion && upgradeVersion[0]) {
      App.set('currentStackVersion', upgradeVersion[0]);
    }
    App.set('upgradeState', 'NOT_REQUIRED');
  },

  /**
   * Check <code>App.upgradeState</code> for HOLDING
   * If it is, send request to check if current item is Finalize
   * @method updateFinalize
   */
  updateFinalize: function () {
    var upgradeState = App.get('upgradeState');
    if (upgradeState === 'HOLDING') {
      return App.ajax.send({
        name: 'admin.upgrade.finalizeContext',
        sender: this,
        success: 'updateFinalizeSuccessCallback',
        error: 'updateFinalizeErrorCallback'
      });
    } else {
      this.set('isFinalizeItem', false);
    }
  }.observes('App.upgradeState'),

  /**
   *
   * @param {object|null} data
   * @method updateFinalizeSuccessCallback
   */
  updateFinalizeSuccessCallback: function updateFinalizeSuccessCallback(data) {
    var context = data ? Em.get(data, 'items.firstObject.upgrade_groups.firstObject.upgrade_items.firstObject.UpgradeItem.context') : '';
    this.set('isFinalizeItem', context === this.get('finalizeContext'));
  },

  updateFinalizeErrorCallback: function updateFinalizeErrorCallback() {
    this.set('isFinalizeItem', false);
  },

  /**
   * show dialog with tasks of upgrade
   * @return {App.ModalPopup}
   */
  openUpgradeDialog: function openUpgradeDialog() {
    if (App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK') && !this.get('isWizardRestricted')) {
      App.propertyDidChange('upgradeSuspended');
      App.router.transitionTo('admin.stackUpgrade');
    }
  },

  /**
   * returns url to get data for repoVersion or clusterStackVersion
   * @param {Boolean} stack true if load clusterStackVersion
   * @param {Boolean} fullLoad true if load all data
   * @returns {String}
   * @method getUrl
   */
  getUrl: function getUrl(stack, fullLoad) {
    if (App.get('testMode')) {
      return stack ? this.get('mockStackUrl') : this.get('mockRepoUrl');
    } else {
      if (fullLoad) {
        return stack ? this.get('realStackUrl') : this.get('realRepoUrl');
      } else {
        return this.get('realUpdateUrl');
      }
    }
  },

  /**
   * get stack versions from server and push it to model
   * @return {*}
   * @method loadStackVersionsToModel
   */
  loadStackVersionsToModel: function loadStackVersionsToModel(fullLoad) {
    var dfd = $.Deferred();
    App.HttpClient.get(this.getUrl(true, fullLoad), App.stackVersionMapper, {
      complete: function complete() {
        dfd.resolve();
      }
    });
    return dfd.promise();
  },

  /**
   * get repo versions from server and push it to model
   * @return {*}
   * @params {Boolean} isUpdate - if true loads part of data that need to be updated
   * @method loadRepoVersionsToModel()
   */
  loadRepoVersionsToModel: function loadRepoVersionsToModel() {
    var dfd = $.Deferred();
    App.HttpClient.get(this.getUrl(false, true), App.repoVersionMapper, {
      complete: function complete() {
        dfd.resolve();
      }
    });
    return dfd.promise();
  },

  /**
   * set status to Upgrade item
   * @param item
   * @param status
   */
  setUpgradeItemStatus: function setUpgradeItemStatus(item, status) {
    this.set('requestInProgress', true);
    return App.ajax.send({
      name: 'admin.upgrade.upgradeItem.setState',
      sender: this,
      data: {
        upgradeId: item.get('request_id'),
        itemId: item.get('stage_id'),
        groupId: item.get('group_id'),
        status: status
      },
      callback: function callback() {
        this.sender.set('requestInProgress', false);
      }
    }).done(function () {
      item.set('status', status);
    });
  },

  currentVersionObserver: function () {
    var versionNumber = this.get('currentVersion.repository_version');
    var currentVersionObject = App.RepositoryVersion.find().findProperty('status', 'CURRENT');
    var versionName = currentVersionObject && currentVersionObject.get('stackVersionType');
    App.set('isStormMetricsSupported', versionName != 'HDP' || stringUtils.compareVersions(versionNumber, '2.2.2') > -1 || !versionNumber);
  }.observes('currentVersion.repository_version'),

  /**
   * get the installed repositories of HDP from server
   */
  loadRepositories: function () {
    if (App.router.get('clusterController.isLoaded') && App.get('currentStackVersion')) {
      var nameVersionCombo = App.get('currentStackVersion');
      var stackName = nameVersionCombo.split('-')[0];
      var stackVersion = nameVersionCombo.split('-')[1];
      App.ajax.send({
        name: 'cluster.load_repositories',
        sender: this,
        data: {
          stackName: stackName,
          stackVersion: stackVersion
        },
        success: 'loadRepositoriesSuccessCallback',
        error: 'loadRepositoriesErrorCallback'
      });
    }
  }.observes('App.router.clusterController.isLoaded'),

  loadRepositoriesSuccessCallback: function loadRepositoriesSuccessCallback(data) {
    var allRepos = [];
    data.items.forEach(function (os) {
      os.repositories.forEach(function (repository) {
        var osType = repository.Repositories.os_type;
        var repo = Em.Object.create({
          baseUrl: repository.Repositories.base_url,
          osType: osType,
          repoId: repository.Repositories.repo_id,
          repoName: repository.Repositories.repo_name,
          stackName: repository.Repositories.stack_name,
          stackVersion: repository.Repositories.stack_version,
          isFirst: false
        });
        var group = allRepos.findProperty('name', osType);
        if (!group) {
          group = {
            name: osType,
            repositories: []
          };
          repo.set('isFirst', true);
          allRepos.push(group);
        }
        group.repositories.push(repo);
      });
    }, this);
    allRepos.stackVersion = App.get('currentStackVersionNumber');
    this.set('allRepos', allRepos);
  },

  loadRepositoriesErrorCallback: function loadRepositoriesErrorCallback(request, ajaxOptions, error) {},

  /**
   * @returns {$.ajax}
   */
  suspendUpgrade: function suspendUpgrade() {
    var self = this;
    return this.abortUpgradeWithSuspend().done(function () {
      App.set('upgradeState', 'ABORTED');
      self.setDBProperty('upgradeState', 'ABORTED');
      App.clusterStatus.setClusterStatus({
        wizardControllerName: self.get('name'),
        localdb: App.db.data
      });
    });
  },

  /**
   * @returns {$.ajax}
   */
  resumeUpgrade: function resumeUpgrade() {
    var self = this;
    this.retryUpgrade().done(function () {
      App.set('upgradeState', 'PENDING');
      App.propertyDidChange('upgradeSuspended');
      self.setDBProperty('upgradeState', 'PENDING');
      App.clusterStatus.setClusterStatus({
        wizardControllerName: self.get('name'),
        localdb: App.db.data
      });
    });
  },

  /**
   *
   * @param {Em.Object} version
   */
  confirmRevertPatchUpgrade: function confirmRevertPatchUpgrade(version) {
    var self = this;
    var currentStack = App.RepositoryVersion.find(this.get('currentVersion.id'));
    App.ModalPopup.show({
      header: Em.I18n.t('popup.confirmation.commonHeader'),
      popupBody: Em.I18n.t('admin.stackVersions.upgrade.patch.revert.confirmation'),
      bodyClass: Em.View.extend({
        classNames: ['revert-patch-upgrade-confirmation'],
        servicesToBeReverted: this.getServicesToBeReverted(version, currentStack),
        stackFromVersion: version.get('displayName'),
        stackToVersion: currentStack.get('displayNameSimple'),
        templateName: require('templates/common/modal_popups/revert_patch_upgrade_confirmation')
      }),
      onPrimary: function onPrimary() {
        self.revertPatchUpgrade(version);
        this._super();
      }
    });
  },

  /**
   *
   * @param {Em.Object} version
   * @param {Em.Object} currentStack
   * @returns {Array}
   */
  getServicesToBeReverted: function getServicesToBeReverted(version, currentStack) {
    return version.get('stackServices').filter(function (_service) {
      var originalService = App.Service.find(_service.get('name'));
      var isOriginal = originalService.get('desiredRepositoryVersionId') === version.get('id');
      return originalService.get('isLoaded') && _service.get('isAvailable') && isOriginal;
    }).map(function (_service) {
      return {
        displayName: _service.get('displayName'),
        fromVersion: _service.get('latestVersion'),
        toVersion: currentStack.get('stackServices').findProperty('name', _service.get('name')).get('latestVersion')
      };
    });
  },

  /**
   *
   * @param {Em.Object} version
   * @returns {$.ajax}
   */
  revertPatchUpgrade: function revertPatchUpgrade(version) {
    this.set('requestInProgress', true);
    return App.ajax.send({
      name: 'admin.upgrade.revert',
      sender: this,
      success: 'upgradeSuccessCallback',
      error: 'upgradeErrorCallback',
      callback: function callback() {
        this.sender.set('requestInProgress', false);
      },
      data: {
        upgradeId: version.get('stackVersion').get('revertUpgradeId'),
        id: version.get('id'),
        value: version.get('repositoryVersion'),
        label: version.get('displayName'),
        isDowngrade: true
      }
    });
  },

  /**
   * @param {App.RepositoryVersion} version
   * @returns {Em.Object}
   */
  confirmDiscardRepoVersion: function confirmDiscardRepoVersion(version) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.discardRepoVersion(version);
    });
  },

  /**
   * @param {App.RepositoryVersion} version
   * @returns {$.ajax}
   */
  discardRepoVersion: function discardRepoVersion(version) {
    this.set('requestInProgress', true);
    return App.ajax.send({
      name: 'admin.stack_versions.discard',
      sender: this,
      callback: function callback() {
        this.sender.set('requestInProgress', false);
      },
      data: {
        id: version.get('id'),
        stackName: version.get('stackVersionType'),
        stackVersion: version.get('stackVersionNumber')
      }
    });
  },

  /**
   * restore last Upgrade data
   * @param {object} lastUpgradeData
   */
  restoreLastUpgrade: function restoreLastUpgrade(lastUpgradeData) {
    var self = this;
    var upgradeType = this.get('upgradeMethods').findProperty('type', lastUpgradeData.Upgrade.upgrade_type);
    var isDowngrade = lastUpgradeData.Upgrade.direction === 'DOWNGRADE';
    this.setDBProperties({
      toVersion: lastUpgradeData.Upgrade.associated_version,
      upgradeId: lastUpgradeData.Upgrade.request_id,
      isDowngrade: isDowngrade,
      upgradeState: lastUpgradeData.Upgrade.request_status,
      upgradeType: lastUpgradeData.Upgrade.upgrade_type,
      isWizardRestricted: upgradeType.get('isWizardRestricted'),
      downgradeAllowed: lastUpgradeData.Upgrade.downgrade_allowed,
      upgradeTypeDisplayName: upgradeType.get('displayName'),
      isSuspended: lastUpgradeData.Upgrade.suspended,
      failuresTolerance: {
        skipComponentFailures: lastUpgradeData.Upgrade.skip_failures,
        skipSCFailures: lastUpgradeData.Upgrade.skip_service_check_failures
      }
    });
    this.initDBProperties();
    App.set('upgradeState', lastUpgradeData.Upgrade.request_status);
    this.loadRepoVersionsToModel().done(function () {
      var version = App.RepositoryVersion.find().findProperty('repositoryVersion', lastUpgradeData.Upgrade.associated_version);
      var upgradeVersion = version && version.get('displayName');
      self.setDBProperty('upgradeVersion', upgradeVersion);
      self.set('upgradeVersion', upgradeVersion);
    });
  },

  /**
   * Build table from configs list and open new window to show this table
   * @param configs
   */
  openConfigsInNewWindow: function openConfigsInNewWindow(configs) {
    var newWindow;
    var output = '';

    output += '<table style="text-align: left;"><thead><tr>' + '<th>' + Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.configType') + '</th>' + '<th>' + Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.propertyName') + '</th>' + '<th>' + Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.currentValue') + '</th>' + '<th>' + Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.recommendedValue') + '</th>' + '<th>' + Em.I18n.t('popup.clusterCheck.Upgrade.configsMerge.resultingValue') + '</th>' + '</tr></thead><tbody>';

    configs.context.forEach(function (config) {
      output += '<tr>' + '<td>' + config.type + '</td>' + '<td>' + config.name + '</td>' + '<td>' + config.currentValue + '</td>' + '<td>' + config.recommendedValue + '</td>' + '<td>' + config.resultingValue + '</td>' + '</tr>';
    });

    output += '</tbody></table>';
    newWindow = window.open();
    newWindow.document.write(output);
    newWindow.focus();
  },

  serviceVersionsMap: {},

  /**
   * load version for services to display on admin service page
   * it should be fetched from the repo which corresponds to the desiredRepositoryVersionId of the service
   */
  getServiceVersionFromRepo: function () {
    if (!App.router.get('clusterController.isLoaded') || !App.router.get('clusterController.isStackVersionsLoaded')) {
      return;
    }

    var currentStackName = App.get('currentStackName');
    var currentStackVersionNumber = App.get('currentStackVersionNumber');
    var map = this.get('serviceVersionsMap');

    var stackServices = App.StackService.find().filter(function (service) {
      return service.get('stackName') === currentStackName && service.get('stackVersion') === currentStackVersionNumber;
    });

    stackServices.forEach(function (service) {
      var serviceName = service.get('serviceName');

      var installedService = App.Service.find().findProperty('serviceName', serviceName);
      var desiredRepositoryVersionId = installedService ? installedService.get('desiredRepositoryVersionId') : null;
      var serviceVersion = "";
      var currentStackRepoVersions = App.RepositoryVersion.find().filter(function (repoVersion) {
        return repoVersion.get('stackVersionType') === currentStackName && repoVersion.get('stackVersionNumber') === currentStackVersionNumber;
      });

      //Get service version from the current standard repo for stack services which are not installed else search by id
      if (!desiredRepositoryVersionId) {
        currentStackRepoVersions.forEach(function (repoVersion) {
          if (repoVersion.get('isCurrent') && repoVersion.get('isStandard')) {
            serviceVersion = repoVersion.get('stackServices').findProperty('name', serviceName).get('latestVersion');
          }
        });
      } else {
        var repoVersion = currentStackRepoVersions.findProperty('id', desiredRepositoryVersionId);
        serviceVersion = repoVersion.get('stackServices').findProperty('name', serviceName).get('latestVersion');
      }
      map[serviceName] = serviceVersion;
    });
  }.observes('App.router.clusterController.isLoaded', 'App.router.clusterController.isStackVersionsLoaded')

});

});

require.register("controllers/main/admin/stack_upgrade_history_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.MainAdminStackUpgradeHistoryController = Em.ArrayController.extend({
  name: 'mainAdminStackUpgradeHistoryController',

  startIndex: 1,

  resetStartIndex: false,

  /**
   * status of tasks/items/groups which should be grayed out and disabled
   * @type {Array}
   */
  nonActiveStates: ['PENDING'],

  /**
   * mutable properties of Upgrade Task
   * @type {Array}
   */
  taskDetailsProperties: ['status', 'stdout', 'stderr', 'error_log', 'host_name', 'output_log'],

  /**
   * Current upgrade record clicked on the UI
   * @type {App.StackUpgradeHistory|null}
   * */
  currentUpgradeRecord: null,

  isDowngrade: false,

  upgradeData: null,

  /**
   * List of all <code>App.StackUpgradeHistory</code>. Latest one at the beginning
   * @type {App.StackUpgradeHistory[]}
   */
  content: App.StackUpgradeHistory.find(),

  upgradeHistoryUrl: function () {
    return App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + '/upgrades?fields=Upgrade';
  }.property('App.clusterName'),

  upgradeUpgradeRecordUrl: function () {
    var record = this.get('currentUpgradeRecord');
    return App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + '/upgrades/' + record.get('requestId');
  }.property('App.clusterName', 'currentUpgradeRecord'),

  loadStackUpgradeHistoryToModel: function loadStackUpgradeHistoryToModel() {
    console.log('Load stack upgrade history');
    var dfd = $.Deferred();
    App.HttpClient.get(this.get('upgradeHistoryUrl'), App.stackUpgradeHistoryMapper, {
      complete: function complete() {
        dfd.resolve();
      }
    });
    return dfd.promise();
  },

  loadStackUpgradeRecord: function loadStackUpgradeRecord() {
    var record = this.get('currentUpgradeRecord');
    this.set('isDowngrade', 'DOWNGRADE' == record.get('direction'));
    var dfd = $.Deferred();
    var self = this;
    if (record != null) {
      App.ajax.send({
        name: 'admin.upgrade.data',
        sender: this,
        data: {
          id: record.get('requestId')
        },
        success: 'loadUpgradeRecordSuccessCallback'
      }).then(dfd.resolve).always(function () {});
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  loadUpgradeRecordSuccessCallback: function loadUpgradeRecordSuccessCallback(newData) {
    if (Em.isNone(newData)) {
      var record = this.get('currentUpgradeRecord');
      console.debug('No data returned for upgrad record ' + record.get('requestId'));
      return;
    }
    var upgradeGroups = [];
    if (newData.upgrade_groups) {
      var nonActiveStates = this.get('nonActiveStates');
      //wrap all entities into App.finishedUpgradeEntity
      newData.upgrade_groups.forEach(function (newGroup) {
        var hasExpandableItems = newGroup.upgrade_items.some(function (item) {
          return !nonActiveStates.contains(item.UpgradeItem.status);
        }),
            oldGroup = App.finishedUpgradeEntity.create({ type: 'GROUP', hasExpandableItems: hasExpandableItems }, newGroup.UpgradeGroup),
            upgradeItems = [];
        newGroup.upgrade_items.forEach(function (item) {
          var oldItem = App.finishedUpgradeEntity.create({ type: 'ITEM' }, item.UpgradeItem);
          this.formatMessages(oldItem);
          oldItem.set('tasks', []);
          upgradeItems.pushObject(oldItem);
        }, this);
        upgradeItems.reverse();
        oldGroup.set('upgradeItems', upgradeItems);
        upgradeGroups.pushObject(oldGroup);
      }, this);
      upgradeGroups.reverse();
      this.set('upgradeData', Em.Object.create({
        upgradeGroups: upgradeGroups,
        upgrade_groups: newData.upgrade_groups,
        Upgrade: newData.Upgrade
      }));
    }
  },

  /**
   * format upgrade item text
   * @param {App.finishedUpgradeEntity} oldItem
   */
  formatMessages: function formatMessages(oldItem) {
    var text = oldItem.get('text');
    var messages = [];

    try {
      var messageArray = JSON.parse(text);
      for (var i = 0; i < messageArray.length; i++) {
        messages.push(messageArray[i].message);
      }
      oldItem.set('text', messages.join(' '));
    } catch (err) {
      console.warn('Upgrade Item has malformed text');
    }
    oldItem.set('messages', messages);
  },

  /**
   * request Upgrade Item and its tasks from server
   * @param {Em.Object} item
   * @param {Function} customCallback
   * @return {$.ajax}
   */
  getUpgradeItem: function getUpgradeItem(item) {
    return App.ajax.send({
      name: 'admin.upgrade.upgrade_item',
      sender: this,
      data: {
        upgradeId: item.get('request_id'),
        groupId: item.get('group_id'),
        stageId: item.get('stage_id')
      },
      success: 'getUpgradeItemSuccessCallback'
    });
  },

  /**
   * success callback of <code>getTasks</code>
   * @param {object} data
   */
  getUpgradeItemSuccessCallback: function getUpgradeItemSuccessCallback(data) {
    this.get('upgradeData.upgradeGroups').forEach(function (group) {
      if (group.get('group_id') === data.UpgradeItem.group_id) {
        group.get('upgradeItems').forEach(function (item) {
          if (item.get('stage_id') === data.UpgradeItem.stage_id) {
            if (item.get('tasks.length')) {
              data.tasks.forEach(function (task) {
                var currentTask = item.get('tasks').findProperty('id', task.Tasks.id);
                this.get('taskDetailsProperties').forEach(function (property) {
                  if (!Em.isNone(task.Tasks[property])) {
                    currentTask.set(property, task.Tasks[property]);
                  }
                }, this);
              }, this);
            } else {
              var tasks = [];
              data.tasks.forEach(function (task) {
                tasks.pushObject(App.finishedUpgradeEntity.create({
                  type: 'TASK',
                  group_id: data.UpgradeItem.group_id
                }, task.Tasks));
              });
              item.set('tasks', tasks);
            }
            item.set('isTasksLoaded', true);
          }
        }, this);
      }
    }, this);
  },

  /**
   * request Upgrade Task
   * @param {Em.Object} task
   * @return {$.ajax}
   */
  getUpgradeTask: function getUpgradeTask(task) {
    return App.ajax.send({
      name: 'admin.upgrade.upgrade_task',
      sender: this,
      data: {
        upgradeId: task.get('request_id'),
        groupId: task.get('group_id'),
        stageId: task.get('stage_id'),
        taskId: task.get('id'),
        task: task
      },
      success: 'getUpgradeTaskSuccessCallback'
    });
  },

  getUpgradeTaskSuccessCallback: function getUpgradeTaskSuccessCallback(data, xhr, params) {
    this.get('taskDetailsProperties').forEach(function (property) {
      params.task.set(property, data.Tasks[property]);
    }, this);
  },

  /**
   * status of Upgrade request
   * @type {string}
   */
  requestStatus: function () {
    if (this.get('upgradeData')) {
      if (this.get('upgradeData.Upgrade')) {
        return this.get('upgradeData.Upgrade.request_status');
      } else {
        return '';
      }
    } else {
      return '';
    }
  }.property('upgradeData.Upgrade.request_status')
});

});

require.register("controllers/main/alert_definitions_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAlertDefinitionsController = Em.ArrayController.extend({

  name: 'mainAlertDefinitionsController',

  /**
   * Define whether restore filter conditions from local db
   * @type {Boolean}
   */
  showFilterConditionsFirstLoad: false,

  contentUpdater: null,

  /**
   * List of all <code>App.AlertDefinition</code>
   * @type {App.AlertDefinition[]}
   */
  content: App.AlertDefinition.find(),

  /**
   * Generates key for alert summary that represents current state
   */
  getSummaryCache: function getSummaryCache() {
    var res = '';
    this.get('content').forEach(function (o) {
      var summary = o.get('summary');
      o.get('order').forEach(function (state) {
        res += summary[state] ? summary[state].count + summary[state].maintenanceCount : 0;
      });
    });

    return res;
  },

  generateCacheByKey: function generateCacheByKey(key) {
    if (key === 'summary') {
      return this.getSummaryCache();
    }

    return this.get('content').mapProperty(key).join('');
  },

  contentWasChanged: function contentWasChanged(key) {
    var updatedCache = this.generateCacheByKey(key);
    if (this.get('cache.' + key) !== updatedCache) {
      this.set('cache.' + key, updatedCache);
      this.propertyDidChange('contentUpdater');
    }
  },

  cache: {
    'label': '',
    'summary': '',
    'serviceName': '',
    'lastTriggered': '',
    'enabled': ''
  },

  /**
   * Enable/disable alertDefinition confirmation popup
   * @param {object} event
   * @method toggleState
   * @return {App.ModalPopup}
   */
  toggleState: function toggleState(event) {
    var alertDefinition = event.context;
    var self = this;
    var bodyMessage = Em.Object.create({
      confirmMsg: alertDefinition.get('enabled') ? Em.I18n.t('alerts.table.state.enabled.confirm.msg') : Em.I18n.t('alerts.table.state.disabled.confirm.msg'),
      confirmButton: alertDefinition.get('enabled') ? Em.I18n.t('alerts.table.state.enabled.confirm.btn') : Em.I18n.t('alerts.table.state.disabled.confirm.btn')
    });

    return App.showConfirmationFeedBackPopup(function (query) {
      self.toggleDefinitionState(alertDefinition);
    }, bodyMessage);
  },

  /**
   * Enable/disable alertDefinition
   * @param {object} alertDefinition
   * @returns {$.ajax}
   * @method toggleDefinitionState
   */
  toggleDefinitionState: function toggleDefinitionState(alertDefinition) {
    var newState = !alertDefinition.get('enabled');
    alertDefinition.set('enabled', newState);
    Em.run.next(function () {
      App.tooltip($('.enable-disable-button'));
    });
    return App.ajax.send({
      name: 'alerts.update_alert_definition',
      sender: this,
      data: {
        id: alertDefinition.get('id'),
        data: {
          "AlertDefinition/enabled": newState
        }
      }
    });
  },

  /**
   *  ========================== alerts notifications dropdown dialog =========================
   */

  /**
   * Number of all critical and warning alert instances
   * Calculation is based on each <code>alertDefinitions.summary</code>
   * @type {Number}
   */
  unhealthyAlertInstancesCount: function () {
    return this.get('criticalAlertInstancesCount') + this.get('warningAlertInstancesCount');
  }.property('criticalAlertInstancesCount', 'warningAlertInstancesCount'),

  criticalAlertInstancesCount: function () {
    return this.get('content').map(function (alertDefinition) {
      return alertDefinition.getWithDefault('summary.CRITICAL.count', 0);
    }).reduce(Em.sum, 0);
  }.property('content.@each.summary'),

  warningAlertInstancesCount: function () {
    return this.get('content').map(function (alertDefinition) {
      return alertDefinition.getWithDefault('summary.WARNING.count', 0);
    }).reduce(Em.sum, 0);
  }.property('content.@each.summary'),

  /**
   * if critical alerts exist, the alert badge should be red.
   * @type {Boolean}
   */
  isCriticalAlerts: function () {
    return this.get('content').invoke('getWithDefault', 'summary.CRITICAL.count', 0).reduce(Em.sum, 0) !== 0;
  }.property('content.@each.summary')
});

});

require.register("controllers/main/alerts/add_alert_definition/add_alert_definition_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionController = App.WizardController.extend({

  name: 'addAlertDefinitionController',

  totalSteps: 3,

  content: {
    selectedType: null
  },

  /**
   * Do request to create new alert definition
   * @param {object} newDefinitionData
   * @returns {$.ajax}
   */
  createNewAlertDefinition: function createNewAlertDefinition(newDefinitionData) {
    return App.ajax.send({
      name: 'alerts.create_alert_definition',
      sender: this,
      data: {
        data: newDefinitionData
      }
    });
  },

  finish: function finish() {
    this.clear();
  }

});

});

require.register("controllers/main/alerts/add_alert_definition/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep1Controller = Em.Controller.extend({

  name: 'addAlertDefinitionStep1',

  /**
   * List of available alert definition types
   * @type {{name: string, isActive: boolean}[]}
   */
  alertDefinitionsTypes: function () {
    return App.AlertType.find().map(function (option) {
      return Em.Object.create({
        name: option.get('name'),
        displayName: option.get('displayName'),
        icon: option.get('iconPath'),
        description: option.get('description')
      });
    });
  }.property(),

  /**
   * Set selectedType if it exists in the wizard controller
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.set('content.selectedType', '');
  },

  /**
   * Handler for select alert definition type selection
   * @param {object} e
   * @method selectType
   */
  selectType: function selectType(e) {
    var type = e.context,
        types = this.get('alertDefinitionsTypes');
    this.set('content.selectedType', type.name);
    $("[rel='selectable-tooltip']").trigger('mouseleave');
    App.router.send('next');
  }

});

});

require.register("controllers/main/alerts/add_alert_definition/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep2Controller = Em.Controller.extend({

  name: 'addAlertDefinitionStep2Controller'

});

});

require.register("controllers/main/alerts/add_alert_definition/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep3Controller = Em.Controller.extend({

  name: 'addAlertDefinitionStep3Controller'

});

});

require.register("controllers/main/alerts/alert_definitions_actions_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var validator = require('utils/validator');

App.MainAlertDefinitionActionsController = Em.ArrayController.extend({

  name: 'mainAlertDefinitionActionsController',

  /**
   * List of available actions for alert definitions
   * @type {{title: string, icon: string, action: string, showDivider: boolean}[]}
   */
  content: function () {
    var content = [];
    if (App.supports.createAlerts) {
      content.push({
        title: Em.I18n.t('alerts.actions.create'),
        icon: 'glyphicon glyphicon-plus',
        action: 'createNewAlertDefinition',
        showDivider: true
      });
    }
    content.push({
      title: Em.I18n.t('alerts.actions.manageGroups'),
      icon: 'glyphicon glyphicon-th-large',
      action: 'manageAlertGroups',
      showDivider: false
    });
    if (App.isAuthorized('CLUSTER.MANAGE_ALERT_NOTIFICATIONS')) {
      content.push({
        title: Em.I18n.t('alerts.actions.manageNotifications'),
        icon: 'glyphicon glyphicon-envelope',
        action: 'manageNotifications',
        showDivider: false
      });
    }
    content.push({
      title: Em.I18n.t('alerts.actions.manageSettings'),
      icon: 'glyphicon glyphicon-cog',
      action: 'manageSettings',
      showDivider: false
    });
    return content;
  }.property('App.supports.createAlerts', 'App.auth'),

  /**
   * Common handler for menu item click
   * Call proper controller's method described in <code>action</code>-field (see <code>content</code>)
   * @param {object} event
   * @method actionHandler
   */
  actionHandler: function actionHandler(event) {
    var menuElement = event.context,
        action = menuElement.action;
    if ('function' === Em.typeOf(Em.get(this, action))) {
      this[action]();
    } else {
      Em.assert('Invalid action provided - ' + action, false);
    }
  },

  /**
   * Start "Create Alert Definition" wizard
   * @method createNewAlertDefinition
   */
  createNewAlertDefinition: function createNewAlertDefinition() {
    App.router.transitionTo('alertAdd');
  },

  /**
   * Handler when clicking on "Manage Alert Groups", a popup will show up
   * @method manageAlertGroups
   * @return {App.ModalPopup}
   */
  manageAlertGroups: function manageAlertGroups() {

    return App.ModalPopup.show({

      header: Em.I18n.t('alerts.actions.manage_alert_groups_popup.header'),

      bodyClass: App.MainAlertsManageAlertGroupView.extend({
        controllerBinding: 'App.router.manageAlertGroupsController'
      }),

      classNames: ['common-modal-wrapper', 'manage-alert-group-popup'],
      modalDialogClasses: ['modal-lg'],
      primary: Em.I18n.t('common.save'),

      onPrimary: function onPrimary() {
        var modifiedAlertGroups = this.get('subViewController.defsModifiedAlertGroups');
        var dataForSuccessPopup = {
          created: modifiedAlertGroups.toCreate.length,
          deleted: modifiedAlertGroups.toDelete.length,
          updated: modifiedAlertGroups.toSet.length
        };
        var showSuccessPopup = dataForSuccessPopup.created + dataForSuccessPopup.deleted + dataForSuccessPopup.updated > 0;
        // Save modified Alert-groups
        var self = this;
        var errors = [];
        var deleteQueriesCounter = modifiedAlertGroups.toDelete.length;
        var createQueriesCounter = modifiedAlertGroups.toCreate.length;
        var deleteQueriesRun = false;
        var createQueriesRun = false;
        var runNextQuery = function runNextQuery() {
          if (!deleteQueriesRun && deleteQueriesCounter > 0) {
            deleteQueriesRun = true;
            modifiedAlertGroups.toDelete.forEach(function (group) {
              self.get('subViewController').removeAlertGroup(group, finishFunction, finishFunction);
            }, this);
          } else if (!createQueriesRun && deleteQueriesCounter < 1) {
            createQueriesRun = true;
            modifiedAlertGroups.toSet.forEach(function (group) {
              self.get('subViewController').updateAlertGroup(group, finishFunction, finishFunction);
            }, this);
            modifiedAlertGroups.toCreate.forEach(function (group) {
              self.get('subViewController').postNewAlertGroup(group, finishFunction);
            }, this);
          }
        };
        var finishFunction = function finishFunction(xhr, text, errorThrown) {
          if (xhr && errorThrown) {
            var error = xhr.status + "(" + errorThrown + ") ";
            try {
              var json = $.parseJSON(xhr.responseText);
              error += json.message;
            } catch (err) {}
            errors.push(error);
          }
          if (createQueriesRun) {
            createQueriesCounter--;
          } else {
            deleteQueriesCounter--;
          }
          if (deleteQueriesCounter + createQueriesCounter < 1) {
            if (errors.length > 0) {
              self.get('subViewController').set('errorMessage', errors.join(". "));
            } else {
              self.hide();
              if (showSuccessPopup) {
                App.ModalPopup.show({
                  secondary: null,
                  header: Em.I18n.t('alerts.groups.successPopup.header'),
                  bodyClass: Em.View.extend({
                    dataForSuccessPopup: dataForSuccessPopup,
                    templateName: require('templates/main/alerts/alert_groups/success_popup_body')
                  })
                });
              }
              App.router.get('updateController').updateAlertNotifications(Em.K);
            }
          } else {
            runNextQuery();
          }
        };
        runNextQuery();
      },

      subViewController: function () {
        return App.router.get('manageAlertGroupsController');
      }.property('App.router.manageAlertGroupsController'),

      disablePrimary: Em.computed.not('subViewController.isDefsModified')

    });
  },

  /**
   * "Manage Alert Notifications" handler
   * @method manageNotifications
   * @return {App.ModalPopup}
   */
  manageNotifications: function manageNotifications() {

    return App.ModalPopup.show({

      header: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.header'),

      bodyClass: App.ManageAlertNotificationsView.extend({
        controllerBinding: 'App.router.manageAlertNotificationsController'
      }),

      classNames: ['common-modal-wrapper', 'manage-configuration-group-popup'],
      modalDialogClasses: ['modal-lg'],

      secondary: null,

      primary: Em.I18n.t('common.close'),

      autoHeight: false

    });
  },

  /**
   * "Manage Alert Settings" handler
   * @method manageSettings
   * @return {App.ModalPopup}
   */
  manageSettings: function manageSettings() {
    var controller = this;
    var configProperties = App.router.get('clusterController.clusterEnv.properties');

    return App.ModalPopup.show({
      classNames: ['forty-percent-width-modal'],
      header: Em.I18n.t('alerts.actions.manageSettings'),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.cancel'),
      inputValue: configProperties.alerts_repeat_tolerance || '1',
      errorMessage: Em.I18n.t('alerts.actions.editRepeatTolerance.error'),
      isInvalid: function () {
        var intValue = Number(this.get('inputValue'));
        return this.get('inputValue') !== 'DEBUG' && (!validator.isValidInt(intValue) || intValue < 1 || intValue > 99);
      }.property('inputValue'),
      disablePrimary: Em.computed.alias('isInvalid'),
      onPrimary: function onPrimary() {
        if (this.get('isInvalid')) {
          return;
        }
        configProperties.alerts_repeat_tolerance = this.get('inputValue');
        App.ajax.send({
          name: 'admin.save_configs',
          sender: controller,
          data: {
            siteName: 'cluster-env',
            properties: configProperties
          },
          error: 'manageSettingsErrorCallback'
        });
        this.hide();
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/prompt_popup'),
        title: Em.I18n.t('alerts.actions.editRepeatTolerance.title'),
        description: Em.I18n.t('alerts.actions.editRepeatTolerance.description'),
        label: Em.I18n.t('alerts.actions.editRepeatTolerance.label')
      })
    });
  },

  manageSettingsErrorCallback: function manageSettingsErrorCallback(data) {
    var error = Em.I18n.t('alerts.actions.manageSettings.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('alerts.actions.manageSettings.error'), error);
  }

});

});

require.register("controllers/main/alerts/alert_instances_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAlertInstancesController = Em.Controller.extend({

  name: 'mainAlertInstancesController',

  content: App.AlertInstance.find(),

  /**
   * @type {App.AlertInstance[]}
   */
  unhealthyAlertInstances: [],

  updateUnhealthyAlertInstances: function () {
    Em.run.once(this, this.updateUnhealthyAlertInstancesOnce);
  }.observes('content.[]'),

  updateUnhealthyAlertInstancesOnce: function updateUnhealthyAlertInstancesOnce() {
    var alertInstances = App.AlertInstance.find().filter(function (item) {
      return ['CRITICAL', 'WARNING'].contains(item.get('state'));
    });
    this.set('unhealthyAlertInstances', alertInstances);
  },

  /**
   * Are alertInstances loaded
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * A flag to reload alert instances table every 10 seconds
   * @type {boolean}
   */
  reload: false,

  /**
   * Causes automatic updates of content if set to true
   * @type {boolean}
   */
  isUpdating: false,

  /**
   * Times for alert instances updater
   * Used in <code>scheduleUpdate</code>
   * @type {number|null}
   */
  updateTimer: null,

  /**
   * @type {string|null} sourceName - hostName or alertDefinitionId
   */
  sourceName: null,

  /**
   * @type {string|null} sourceType - 'HOST'|'ALERT_DEFINITION'
   */
  sourceType: null,

  /**
   * Load alert instances from server (all, for selected host, for selected alert definition)
   * @returns {$.ajax}
   * @method fetchAlertInstances
   */
  fetchAlertInstances: function fetchAlertInstances() {
    var sourceType = this.get('sourceType'),
        sourceName = this.get('sourceName'),
        ajaxData = {
      sender: this,
      success: 'getAlertInstancesSuccessCallback',
      error: 'getAlertInstancesErrorCallback'
    };

    switch (sourceType) {
      case 'HOST':
        $.extend(ajaxData, {
          name: 'alerts.instances.by_host',
          data: {
            hostName: sourceName
          }
        });
        break;

      case 'ALERT_DEFINITION':
        $.extend(ajaxData, {
          name: 'alerts.instances.by_definition',
          data: {
            definitionId: sourceName
          }
        });
        break;

      default:
        $.extend(ajaxData, {
          name: 'alerts.instances'
        });
        break;
    }

    return App.ajax.send(ajaxData);
  },

  /**
   * Pseudo for <code>fetchAlertInstances</code>
   * Used to get all alert instances
   * @method loadAlertInstances
   */
  loadAlertInstances: function loadAlertInstances() {
    this.setProperties({
      isLoaded: false,
      sourceType: null,
      sourceName: null
    });
    this.fetchAlertInstances();
  },

  /**
   * Pseudo for <code>fetchAlertInstances</code>
   * Used to get alert instances for some host
   * @param {string} hostName
   * @method loadAlertInstancesByHost
   */
  loadAlertInstancesByHost: function loadAlertInstancesByHost(hostName) {
    this.setProperties({
      isLoaded: false,
      sourceType: 'HOST',
      sourceName: hostName
    });
    this.fetchAlertInstances();
  },

  /**
   * Pseudo for <code>fetchAlertInstances</code>
   * Used to get alert instances for some alert definition
   * @param {string} definitionId
   * @method loadAlertInstancesByAlertDefinition
   */
  loadAlertInstancesByAlertDefinition: function loadAlertInstancesByAlertDefinition(definitionId) {
    this.setProperties({
      isLoaded: false,
      sourceType: 'ALERT_DEFINITION',
      sourceName: definitionId
    });
    this.fetchAlertInstances();
  },

  scheduleUpdate: function () {
    var self = this;
    if (this.get('isUpdating')) {
      this.set('updateTimer', setTimeout(function () {
        self.fetchAlertInstances().complete(function () {
          self.scheduleUpdate();
        });
      }, App.get('alertInstancesUpdateInterval')));
    } else {
      clearTimeout(this.get('updateTimer'));
    }
  }.observes('isUpdating'),

  /**
   * Success-callback for alert instances request
   * @param {object} json
   * @method getAlertInstancesSuccessCallback
   */
  getAlertInstancesSuccessCallback: function getAlertInstancesSuccessCallback(json) {
    App.alertInstanceMapper.mapLocal(json);
    this.set('isLoaded', true);
    this.toggleProperty('reload');
  },

  /**
   * Error-callback for alert instances request
   * @method getAlertInstancesErrorCallback
   */
  getAlertInstancesErrorCallback: function getAlertInstancesErrorCallback() {
    this.set('isLoaded', true);
  }

});

});

require.register("controllers/main/alerts/definition_configs_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var numericUtils = require('utils/number_utils');

App.MainAlertDefinitionConfigsController = Em.Controller.extend({

  name: 'mainAlertDefinitionConfigsController',

  /**
   * All configurable properties of alert definition
   * @type {Array}
   */
  configs: [],

  /**
   * Define whether configs are editable
   * binds to property populated in template
   * @type {Boolean}
   */
  canEdit: true,

  /**
   * Define configs view mode (Wizard or Definition Details page)
   * @type {Boolean}
   */
  isWizard: false,

  /**
   * Alert Definition type
   * binding is set in template
   * @type {String}
   */
  alertDefinitionType: '',

  /**
   * Array of displayNames of all services
   * is used for "Service" config options
   * @type {Array}
   */
  allServices: function () {
    return App.Service.find().mapProperty('displayName').concat('CUSTOM');
  }.property(),

  /**
   * All possible values for scope propery
   * @type {Array}
   */
  allScopes: ['Any', 'Host', 'Service'],

  /**
   * Array of all aggregate-alerts names
   * @type {Array}
   */
  aggregateAlertNames: function () {
    return App.AggregateAlertDefinition.find().mapProperty('name');
  }.property(),

  /**
   * If any service selected, show related components, hide PORT and HOST field
   * If CUSTOM was selected, show no component, and show PORT and HOST field
   * @method onServiceSelect
   */
  onServiceSelect: function () {
    var serviceProperty = this.get('configs').findProperty('name', 'service');
    var componentsProperty = this.get('configs').findProperty('name', 'component');
    var defaultPortProperty = this.get('configs').findProperty('name', 'default_port');
    var uriProperty = this.get('configs').findProperty('name', 'uri');
    if (serviceProperty && serviceProperty.get('value') == 'CUSTOM') {
      componentsProperty.set('options', ['No component']);
      componentsProperty.set('canEdit', false);
      var results = this.get('configs');
      if (defaultPortProperty && uriProperty) {
        results = results.filter(function (config) {
          return config.name != 'default_port' && config.name != 'uri';
        });
      }
      if (!defaultPortProperty) {
        results = results.concat([App.AlertConfigProperties.DefaultPort.create({
          value: '8050'
        }), App.AlertConfigProperties.URI.create({
          value: ''
        })]);
        this.set('configs', results);
      }
    } else if (serviceProperty && serviceProperty.get('value') !== 'CUSTOM' && componentsProperty.get('value') && componentsProperty.get('value') != 'No component') {
      componentsProperty.set('options', ['No component'].concat(App.HostComponent.find().filterProperty('service.displayName', serviceProperty.get('value')).mapProperty('displayName').uniq()));
      if (!defaultPortProperty) {
        var results = this.get('configs').concat([App.AlertConfigProperties.DefaultPort.create({
          value: '8060'
        }), App.AlertConfigProperties.URI.create({
          value: ''
        })]);
        this.set('configs', results);
      }
    } else if (serviceProperty && serviceProperty.get('value') !== 'CUSTOM') {
      componentsProperty.set('options', ['No component'].concat(App.HostComponent.find().filterProperty('service.displayName', serviceProperty.get('value')).mapProperty('displayName').uniq()));
      if (defaultPortProperty && uriProperty) {
        var results = this.get('configs').filter(function (config) {
          return config.name != 'default_port' && config.name != 'uri';
        });
        this.set('configs', results);
      }
    }
  }.observes('configs.@each.value'),

  /**
   * @return {string|Null}
   * @method getThresholdsProperty
   */
  getThresholdsProperty: function getThresholdsProperty(type, property) {
    var warning = this.get('content.reporting').findProperty('type', type);
    return warning && !Ember.isNone(warning.get(property)) ? warning.get(property) : null;
  },

  /**
   * Render array of configs for appropriate alert definition type
   * @method renderConfigs
   */
  renderConfigs: function renderConfigs() {
    var self = this;
    var alertDefinitionType = this.get('alertDefinitionType');
    var configs = [];
    switch (alertDefinitionType) {
      case 'PORT':
        configs = this.renderPortConfigs();
        break;
      case 'METRIC':
        configs = this.renderMetricConfigs();
        break;
      case 'WEB':
        configs = this.renderWebConfigs();
        break;
      case 'SCRIPT':
        configs = this.renderScriptConfigs();
        break;
      case 'AGGREGATE':
        configs = this.renderAggregateConfigs();
        break;
      case 'SERVER':
        configs = this.renderServerConfigs();
        break;
      case 'RECOVERY':
        configs = this.renderRecoveryConfigs();
        break;
      case 'AMS':
        configs = this.renderAmsConfigs();
        break;
      default:
    }

    configs.forEach(function (config) {
      config.set('isDisabled', !self.get('canEdit') || config.get('readonly'));
    });

    this.set('configs', configs);
  },

  /**
   * Render config properties for port-type alert definition
   * @method renderPortConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderPortConfigs: function renderPortConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');

    if (this.get('isWizard')) {
      result = result.concat([App.AlertConfigProperties.AlertName.create({
        value: ''
      }), App.AlertConfigProperties.Service.create({
        options: this.get('allServices'),
        value: this.get('allServices')[0],
        isShifted: true
      }), App.AlertConfigProperties.Component.create({
        options: this.get('allComponents'),
        value: 'No component',
        isShifted: true
      }),

      //should be on next step
      App.AlertConfigProperties.Interval.create({
        value: isWizard ? '' : alertDefinition.get('interval')
      }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
        label: 'Thresholds',
        showInputForValue: false,
        text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
      }), App.AlertConfigProperties.Thresholds.WarningThreshold.create(App.AlertConfigProperties.Thresholds.PositiveMixin, {
        valueMetric: 'Seconds',
        text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('warning', 'value')
      }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create(App.AlertConfigProperties.Thresholds.PositiveMixin, {
        valueMetric: 'Seconds',
        text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('critical', 'value')
      })]);
    } else {
      result = result.concat([App.AlertConfigProperties.Description.create({
        value: isWizard ? '' : alertDefinition.get('description')
      }), App.AlertConfigProperties.Interval.create({
        value: isWizard ? '' : alertDefinition.get('interval')
      }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
        label: 'Thresholds',
        showInputForValue: false,
        text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
      }), App.AlertConfigProperties.Thresholds.WarningThreshold.create(App.AlertConfigProperties.Thresholds.PositiveMixin, {
        valueMetric: 'Seconds',
        text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('warning', 'value')
      }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create(App.AlertConfigProperties.Thresholds.PositiveMixin, {
        valueMetric: 'Seconds',
        text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
        value: isWizard ? '' : this.getThresholdsProperty('critical', 'value')
      })]);
    }

    return result;
  },

  /**
   * Render config properties for metric-type alert definition
   * @method renderMetricConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderMetricConfigs: function renderMetricConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');
    var units = this.get('content.reporting') && this.get('content.reporting').findProperty('type', 'units') ? this.get('content.reporting').findProperty('type', 'units').get('text') : null;

    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
      label: 'Thresholds',
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
    }), App.AlertConfigProperties.Thresholds.WarningThreshold.create({
      valueMetric: units,
      text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('warning', 'value')
    }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create({
      valueMetric: units,
      text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('critical', 'value')
    }), App.AlertConfigProperties.Parameter.create({
      value: isWizard ? '' : alertDefinition.get('uri.connectionTimeout'),
      threshold: "CRITICAL",
      name: 'connection_timeout',
      label: 'Connection Timeout',
      displayType: 'parameter',
      apiProperty: 'source.uri.connection_timeout',
      units: 'Seconds',
      isValid: function () {
        var value = this.get('value');
        return numericUtils.isPositiveNumber(value);
      }.property('value')
    })]);

    return result;
  },

  /**
   * Render config properties for web-type alert definition
   * @method renderWebConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderWebConfigs: function renderWebConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');

    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
      label: 'Thresholds',
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
    }), App.AlertConfigProperties.Thresholds.WarningThreshold.create({
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('warning', 'value')
    }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create({
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('critical', 'value')
    }), App.AlertConfigProperties.Parameter.create({
      value: isWizard ? '' : alertDefinition.get('uri.connectionTimeout'),
      threshold: "CRITICAL",
      name: 'connection_timeout',
      label: 'Connection Timeout',
      displayType: 'parameter',
      apiProperty: 'source.uri.connection_timeout',
      units: 'Seconds',
      isValid: function () {
        var value = this.get('value');
        return numericUtils.isPositiveNumber(value);
      }.property('value')
    })]);

    return result;
  },

  /**
   * Render config properties for recovery-type alert definition
   * @method renderRecoveryConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderRecoveryConfigs: function renderRecoveryConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');

    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
      label: 'Thresholds',
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
    }), App.AlertConfigProperties.Thresholds.WarningThreshold.create({
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('warning', 'value')
    }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create({
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('critical', 'value')
    })]);

    return result;
  },

  renderAmsConfigs: function renderAmsConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');
    var units = this.get('content.reporting').findProperty('type', 'units') ? this.get('content.reporting').findProperty('type', 'units').get('text') : null;
    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
      label: 'Thresholds',
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
    }), App.AlertConfigProperties.Thresholds.WarningThreshold.create(App.AlertConfigProperties.Thresholds.PercentageMixin, {
      text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('warning', 'value'),
      valueMetric: units
    }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create(App.AlertConfigProperties.Thresholds.PercentageMixin, {
      text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('critical', 'value'),
      valueMetric: units
    }), App.AlertConfigProperties.Parameter.create({
      value: isWizard ? '' : alertDefinition.get('uri.connectionTimeout'),
      name: 'connection_timeout',
      label: 'Connection Timeout',
      displayType: 'parameter',
      apiProperty: 'source.uri.connection_timeout',
      units: 'Seconds',
      isValid: function () {
        var value = this.get('value');
        return numericUtils.isPositiveNumber(value);
      }.property('value')
    })]);
    return result;
  },

  /**
   * Render config properties for script-type alert definition
   * @method renderScriptConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderScriptConfigs: function renderScriptConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');

    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    })]);

    var mixins = {
      STRING: App.AlertConfigProperties.Parameters.StringMixin,
      NUMERIC: App.AlertConfigProperties.Parameters.NumericMixin,
      PERCENT: App.AlertConfigProperties.Parameters.PercentageMixin
    };
    if (alertDefinition) {
      alertDefinition.get('parameters').forEach(function (parameter) {
        var mixin = mixins[parameter.get('type')] || {}; // validation depends on parameter-type
        result.push(App.AlertConfigProperties.Parameter.create(mixin, {
          value: isWizard ? '' : parameter.get('value'),
          apiProperty: parameter.get('name'),
          description: parameter.get('description'),
          label: isWizard ? '' : parameter.get('displayName'),
          threshold: isWizard ? '' : parameter.get('threshold'),
          units: isWizard ? '' : parameter.get('units'),
          type: isWizard ? '' : parameter.get('type'),
          hidden: parameter.get('visibility') === "HIDDEN",
          readonly: parameter.get('visibility') === "READ_ONLY"
        }));
      });
    }

    return result;
  },

  /**
   * Render config properties for aggregate-type alert definition
   * @method renderAggregateConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderAggregateConfigs: function renderAggregateConfigs() {
    var isWizard = this.get('isWizard');
    var alertDefinition = this.get('content');
    var units = this.get('content.reporting') && this.get('content.reporting').findProperty('type', 'units') ? this.get('content.reporting').findProperty('type', 'units').get('text') : null;
    return [App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    }), App.AlertConfigProperties.Thresholds.OkThreshold.create({
      label: 'Thresholds',
      showInputForValue: false,
      text: isWizard ? '' : this.getThresholdsProperty('ok', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('ok', 'value')
    }), App.AlertConfigProperties.Thresholds.WarningThreshold.create(App.AlertConfigProperties.Thresholds.PercentageMixin, {
      text: isWizard ? '' : this.getThresholdsProperty('warning', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('warning', 'value'),
      valueMetric: units
    }), App.AlertConfigProperties.Thresholds.CriticalThreshold.create(App.AlertConfigProperties.Thresholds.PercentageMixin, {
      text: isWizard ? '' : this.getThresholdsProperty('critical', 'text'),
      value: isWizard ? '' : this.getThresholdsProperty('critical', 'value'),
      valueMetric: units
    })];
  },

  /**
   * Render config properties for server-type alert definition
   * @method renderScriptConfigs
   * @returns {App.AlertConfigProperty[]}
   */
  renderServerConfigs: function renderServerConfigs() {
    var result = [];
    var alertDefinition = this.get('content');
    var isWizard = this.get('isWizard');

    if (this.get('isWizard')) {
      result = result.concat(this.renderCommonWizardConfigs());
    }

    result = result.concat([App.AlertConfigProperties.Description.create({
      value: isWizard ? '' : alertDefinition.get('description')
    }), App.AlertConfigProperties.Interval.create({
      value: isWizard ? '' : alertDefinition.get('interval')
    })]);

    var mixins = {
      STRING: App.AlertConfigProperties.Parameters.StringMixin,
      NUMERIC: App.AlertConfigProperties.Parameters.NumericMixin,
      PERCENT: App.AlertConfigProperties.Parameters.PercentageMixin
    };
    alertDefinition.get('parameters').forEach(function (parameter) {
      var mixin = mixins[parameter.get('type')] || {}; // validation depends on parameter-type
      result.push(App.AlertConfigProperties.Parameter.create(mixin, {
        value: isWizard ? '' : parameter.get('value'),
        apiProperty: parameter.get('name'),
        description: parameter.get('description'),
        label: isWizard ? '' : parameter.get('displayName'),
        threshold: isWizard ? '' : parameter.get('threshold'),
        units: isWizard ? '' : parameter.get('units'),
        type: isWizard ? '' : parameter.get('type'),
        hidden: parameter.get('visibility') === "HIDDEN",
        readonly: parameter.get('visibility') === "READ_ONLY"
      }));
    });

    return result;
  },

  /**
   * Render common list of configs used in almost all alert types in wizard
   * @returns {App.AlertConfigProperty[]}
   */
  renderCommonWizardConfigs: function renderCommonWizardConfigs() {
    return [App.AlertConfigProperties.AlertName.create({
      value: ''
    }), App.AlertConfigProperties.Service.create({
      options: this.get('allServices'),
      value: this.get('allServices')[0],
      isShifted: true
    }), App.AlertConfigProperties.Component.create({
      options: this.get('allComponents'),
      value: 'No component',
      isShifted: true
    }), App.AlertConfigProperties.Scope.create({
      options: this.get('allScopes'),
      isShifted: true
    })];
  },

  /**
   * Edit configs button handler
   * @method editConfigs
   */
  editConfigs: function editConfigs() {
    this.get('configs').forEach(function (property) {
      property.set('previousValue', property.get('value'));
      property.set('previousText', property.get('text'));
    });
    this.get('configs').forEach(function (config) {
      config.set('isDisabled', config.get('readonly'));
    });
    this.set('canEdit', true);
  },

  /**
   * Cancel edit configs button handler
   * @method cancelEditConfigs
   */
  cancelEditConfigs: function cancelEditConfigs() {
    this.get('configs').forEach(function (property) {
      property.set('value', property.get('previousValue'));
      property.set('text', property.get('previousText'));
    });
    this.get('configs').setEach('isDisabled', true);
    this.set('canEdit', false);
  },

  /**
   * Save edit configs button handler
   * @method saveConfigs
   * @return {$.ajax}
   */
  saveConfigs: function saveConfigs() {
    this.get('configs').setEach('isDisabled', true);
    this.set('canEdit', false);

    return App.ajax.send({
      name: 'alerts.update_alert_definition',
      sender: this,
      data: {
        id: this.get('content.id'),
        data: this.getPropertiesToUpdate(true)
      }
    });
  },

  /**
   * Create object with new values to put it on server
   * @param {boolean} onlyChanged
   * @method getPropertiesToUpdate
   * @returns {Object}
   */
  getPropertiesToUpdate: function getPropertiesToUpdate(onlyChanged) {
    var propertiesToUpdate = {};
    var configs = onlyChanged ? this.get('configs').filterProperty('wasChanged') : this.get('configs');
    configs = configs.filter(function (c) {
      return c.get('name') !== 'parameter';
    });
    configs.forEach(function (property) {
      var apiProperties = Em.makeArray(property.get('apiProperty'));
      var apiFormattedValues = Em.makeArray(property.get('apiFormattedValue'));
      apiProperties.forEach(function (apiProperty, i) {
        if (apiProperty.contains('source.')) {
          if (!propertiesToUpdate['AlertDefinition/source']) {
            if (this.get('content.rawSourceData')) {
              propertiesToUpdate['AlertDefinition/source'] = this.get('content.rawSourceData');
            }
          }

          if (this.get('content.rawSourceData')) {
            // use rawSourceData to populate propertiesToUpdate
            var sourcePath = propertiesToUpdate['AlertDefinition/source'];
            apiProperty.replace('source.', '').split('.').forEach(function (path, index, array) {
              // check if it is last path
              if (array.length - index === 1) {
                sourcePath[path] = apiFormattedValues[i];
              } else {
                sourcePath = sourcePath[path];
              }
            });
          } else {
            if (!propertiesToUpdate['AlertDefinition/source']) {
              propertiesToUpdate['AlertDefinition/source'] = {};
            }
            Ember.setFullPath(propertiesToUpdate['AlertDefinition/source'], apiProperty.replace('source.', ''), apiFormattedValues[i]);
          }
        } else {
          if (apiProperty) {
            propertiesToUpdate['AlertDefinition/' + apiProperty] = apiFormattedValues[i];
          }
        }
      }, this);
    }, this);

    if (Em.get(propertiesToUpdate, 'AlertDefinition/source.uri.id')) {
      delete propertiesToUpdate['AlertDefinition/source'].uri.id;
    }
    if (Em.get(propertiesToUpdate, 'AlertDefinition/source.ams.id')) {
      delete propertiesToUpdate['AlertDefinition/source'].ams.id;
    }

    // `source.parameters` is an array and should be updated separately from other configs
    if (this.get('content.parameters.length')) {
      propertiesToUpdate['AlertDefinition/source'] = this.get('content.rawSourceData');
      var parameterConfigs = this.get('configs').filterProperty('name', 'parameter');
      parameterConfigs.forEach(function (parameter) {
        propertiesToUpdate['AlertDefinition/source'].parameters.findProperty('name', parameter.get('apiProperty')).value = parameter.get('apiFormattedValue');
      });
    }

    return propertiesToUpdate;
  },

  /**
   * Return array of all config values
   * used to save configs to local db in wizard
   * @method getConfigsValues
   * @returns {Array}
   */
  getConfigsValues: function getConfigsValues() {
    return this.get('configs').map(function (property) {
      return {
        name: property.get('name'),
        value: property.get('value')
      };
    });
  },

  /**
   * Define whether critical threshold >= critical threshold
   * @type {Boolean}
   */
  hasThresholdsError: function () {
    var smallValue, smallValid, largeValue, largeValid;
    if (this.get('configs').findProperty('name', 'warning_threshold')) {
      smallValue = Em.get(this.get('configs').findProperty('name', 'warning_threshold'), 'value');
      smallValid = Em.get(this.get('configs').findProperty('name', 'warning_threshold'), 'isValid');
    }
    if (this.get('configs').findProperty('name', 'critical_threshold')) {
      largeValue = Em.get(this.get('configs').findProperty('name', 'critical_threshold'), 'value');
      largeValid = Em.get(this.get('configs').findProperty('name', 'critical_threshold'), 'isValid');
    }
    return smallValid && largeValid ? Number(smallValue) > Number(largeValue) : false;
  }.property('configs.@each.value'),

  someConfigIsInvalid: Em.computed.someBy('configs', 'isValid', false),

  /**
   * Define whether all configs are valid
   * @type {Boolean}
   */
  hasErrors: Em.computed.or('someConfigIsInvalid', 'hasThresholdsError')

});

});

require.register("controllers/main/alerts/definition_details_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var validator = require('utils/validator');

App.MainAlertDefinitionDetailsController = Em.Controller.extend({

  name: 'mainAlertDefinitionDetailsController',

  alerts: function () {
    return App.AlertInstanceLocal.find().toArray().filterProperty('definitionId', this.get('content.id'));
  }.property('App.router.mainAlertInstancesController.isLoaded', 'App.router.mainAlertInstancesController.reload'),

  // stores object with editing form data (label)
  editing: Em.Object.create({
    label: Em.Object.create({
      name: 'label',
      isEditing: false,
      value: '',
      originalValue: '',
      isError: false,
      bindingValue: 'content.label'
    })
  }),

  /**
   * Host to count of alerts on this host during last day map
   * @type {Object}
   */
  lastDayAlertsCount: null,

  /**
   * List of all group names related to alert definition
   * @type {Array}
   */
  groupsList: Em.computed.mapBy('content.groups', 'displayName'),

  /**
   * Validation function to define if label field populated correctly
   * @method labelValidation
   */
  labelValidation: function () {
    this.set('editing.label.isError', !this.get('editing.label.value').trim());
  }.observes('editing.label.value'),

  /**
   * Set init values for variables
   */
  clearStep: function clearStep() {
    var editing = this.get('editing');
    Em.keys(editing).forEach(function (key) {
      editing.get(key).set('isEditing', false);
    });
  },

  /**
   * Load alert instances for current alertDefinition
   * Start updating loaded data
   * @method loadAlertInstances
   */
  loadAlertInstances: function loadAlertInstances() {
    App.router.get('mainAlertInstancesController').loadAlertInstancesByAlertDefinition(this.get('content.id'));
    App.router.set('mainAlertInstancesController.isUpdating', true);
    this.loadAlertInstancesHistory();
  },

  /**
   * Load alert instances history data
   * used to count instances number of the last 24 hour
   * @method loadAlertInstancesHistory
   */
  loadAlertInstancesHistory: function loadAlertInstancesHistory() {
    this.set('lastDayAlertsCount', null);
    return App.ajax.send({
      name: 'alerts.get_instances_history',
      sender: this,
      data: {
        definitionName: this.get('content.name'),
        timestamp: App.dateTime() - 86400000 // timestamp for time 24-hours ago
      },
      success: 'loadAlertInstancesHistorySuccess'
    });
  },

  /**
   * Success-callback for <code>loadAlertInstancesHistory</code>
   */
  loadAlertInstancesHistorySuccess: function loadAlertInstancesHistorySuccess(data) {
    var lastDayAlertsCount = {};
    data.items.forEach(function (alert) {
      if (!lastDayAlertsCount[alert.AlertHistory.host_name]) {
        lastDayAlertsCount[alert.AlertHistory.host_name] = 0;
      }
      lastDayAlertsCount[alert.AlertHistory.host_name]++;
    });
    this.set('lastDayAlertsCount', lastDayAlertsCount);
  },

  /**
   * Edit button handler
   * @param {object} event
   * @method edit
   */
  edit: function edit(event) {
    var element = event.context;
    var value = this.get(element.get('bindingValue'));
    element.set('originalValue', value);
    element.set('value', value);
    element.set('isEditing', true);
  },

  /**
   * Cancel button handler
   * @param {object} event
   * @method cancelEdit
   */
  cancelEdit: function cancelEdit(event) {
    var element = event.context;
    element.set('value', element.get('originalValue'));
    element.set('isEditing', false);
  },

  /**
   * Save button handler, could save label of alert definition
   * @param {object} event
   * @returns {$.ajax}
   * @method saveEdit
   */
  saveEdit: function saveEdit(event) {
    var element = event.context;
    this.set(element.get('bindingValue'), element.get('value'));
    element.set('isEditing', false);

    var data = Em.Object.create({});
    var propertyName = "AlertDefinition/" + element.get('name');
    data.set(propertyName, element.get('value'));
    var alertDefinitionId = this.get('content.id');
    return App.ajax.send({
      name: 'alerts.update_alert_definition',
      sender: this,
      data: {
        id: alertDefinitionId,
        data: data
      }
    });
  },

  /**
   * Onclick handler for save button on Save popup
   * Save changes of label and configs
   */
  saveLabelAndConfigs: function saveLabelAndConfigs() {
    var configsController = App.router.get('mainAlertDefinitionConfigsController');
    if (configsController.get('canEdit')) {
      configsController.saveConfigs();
    }
    if (this.get('editing.label.isEditing')) {
      this.saveEdit({
        context: this.get('editing.label')
      });
    }
  },

  /**
   * "Delete" button handler
   * @method deleteAlertDefinition
   */
  deleteAlertDefinition: function deleteAlertDefinition() {
    var alertDefinition = this.get('content');
    var self = this;
    App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'alerts.delete_alert_definition',
        sender: self,
        success: 'deleteAlertDefinitionSuccess',
        error: 'deleteAlertDefinitionError',
        data: {
          id: alertDefinition.get('id')
        }
      });
    }, null, function () {});
  },

  /**
   * Success-callback for <code>deleteAlertDefinition</code>
   * @method deleteAlertDefinitionSuccess
   */
  deleteAlertDefinitionSuccess: function deleteAlertDefinitionSuccess() {
    App.router.transitionTo('main.alerts.index');
  },

  /**
   * Error-callback for <code>deleteAlertDefinition</code>
   * @method deleteAlertDefinitionError
   */
  deleteAlertDefinitionError: function deleteAlertDefinitionError(xhr, textStatus, errorThrown, opt) {
    xhr.responseText = "{\"message\": \"" + xhr.statusText + "\"}";
    App.ajax.defaultErrorHandler(xhr, opt.url, 'DELETE', xhr.status);
  },

  /**
   * "Disable / Enable" button handler
   * @method toggleState
   */
  toggleState: function toggleState() {
    var alertDefinition = this.get('content');
    var self = this;
    var bodyMessage = Em.Object.create({
      confirmMsg: alertDefinition.get('enabled') ? Em.I18n.t('alerts.table.state.enabled.confirm.msg') : Em.I18n.t('alerts.table.state.disabled.confirm.msg'),
      confirmButton: alertDefinition.get('enabled') ? Em.I18n.t('alerts.table.state.enabled.confirm.btn') : Em.I18n.t('alerts.table.state.disabled.confirm.btn')
    });

    return App.showConfirmationFeedBackPopup(function () {
      self.toggleDefinitionState(alertDefinition);
    }, bodyMessage);
  },

  /**
   * Enable/disable alertDefinition
   * @param {object} alertDefinition
   * @param {string} property
   * @returns {$.ajax}
   * @method toggleDefinitionState
   */
  toggleDefinitionState: function toggleDefinitionState(alertDefinition) {
    var newState = !alertDefinition.get('enabled');
    alertDefinition.set('enabled', newState);
    return App.ajax.send({
      name: 'alerts.update_alert_definition',
      sender: this,
      data: {
        id: alertDefinition.get('id'),
        data: {
          "AlertDefinition/enabled": newState
        }
      }
    });
  },

  globalAlertsRepeatTolerance: function () {
    return App.router.get('clusterController.clusterEnv.properties.alerts_repeat_tolerance') || "1";
  }.property('App.router.clusterController.clusterEnv'),

  enableRepeatTolerance: function enableRepeatTolerance(enable) {
    var alertDefinition = this.get('content');
    alertDefinition.set('repeat_tolerance_enabled', enable);
    return App.ajax.send({
      name: 'alerts.update_alert_definition',
      sender: this,
      data: {
        id: alertDefinition.get('id'),
        data: {
          "AlertDefinition/repeat_tolerance_enabled": enable
        }
      }
    });
  },

  editRepeatTolerance: function editRepeatTolerance() {
    var self = this;
    var alertDefinition = this.get('content');

    var alertsRepeatTolerance = App.router.get('clusterController.clusterEnv.properties.alerts_repeat_tolerance') || "1";

    return App.ModalPopup.show({
      classNames: ['forty-percent-width-modal'],
      header: Em.I18n.t('alerts.actions.editRepeatTolerance.header'),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.cancel'),
      inputValue: self.get('content.repeat_tolerance_enabled') ? self.get('content.repeat_tolerance') || 1 : alertsRepeatTolerance,
      errorMessage: Em.I18n.t('alerts.actions.editRepeatTolerance.error'),
      isInvalid: function () {
        var intValue = Number(this.get('inputValue'));
        return this.get('inputValue') !== 'DEBUG' && (!validator.isValidInt(intValue) || intValue < 1 || intValue > 99);
      }.property('inputValue'),
      isChanged: function () {
        return Number(this.get('inputValue')) !== alertsRepeatTolerance;
      }.property('inputValue'),
      doRestoreDefaultValue: function doRestoreDefaultValue() {
        this.set('inputValue', alertsRepeatTolerance);
        this.$('[data-toggle=tooltip]').tooltip('destroy');
      },
      disablePrimary: Em.computed.alias('isInvalid'),
      onPrimary: function onPrimary() {
        if (this.get('isInvalid')) {
          return;
        }
        var input = this.get('inputValue');
        self.set('content.repeat_tolerance', input);
        self.enableRepeatTolerance(input !== alertsRepeatTolerance);
        App.ajax.send({
          name: 'alerts.update_alert_definition',
          sender: self,
          data: {
            id: alertDefinition.get('id'),
            data: {
              "AlertDefinition/repeat_tolerance": input
            }
          }
        });
        this.hide();
      },
      didInsertElement: function didInsertElement() {
        this._super();
        App.tooltip(this.$('[data-toggle=tooltip]'));
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/prompt_popup'),
        title: Em.I18n.t('alerts.actions.editRepeatTolerance.title'),
        description: Em.I18n.t('alerts.actions.editRepeatTolerance.description'),
        label: Em.I18n.t('alerts.actions.editRepeatTolerance.label')
      })
    });
  },

  /**
   * Define if label or configs are in edit mode
   * @type {Boolean}
   */
  isEditing: Em.computed.or('editing.label.isEditing', 'App.router.mainAlertDefinitionConfigsController.canEdit'),

  /**
   * If some configs or label are changed and user navigates away, show this popup with propose to save changes
   * @param {String} path
   * @method showSavePopup
   */
  showSavePopup: function showSavePopup(callback) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      bodyClass: Em.View.extend({
        template: Ember.Handlebars.compile('{{t alerts.saveChanges}}')
      }),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.discard'),
      third: Em.I18n.t('common.cancel'),
      disablePrimary: Em.computed.or('App.router.mainAlertDefinitionDetailsController.editing.label.isError', 'App.router.mainAlertDefinitionConfigsController.hasErrors'),
      onPrimary: function onPrimary() {
        self.saveLabelAndConfigs();
        callback();
        this.hide();
      },
      onSecondary: function onSecondary() {
        callback();
        this.hide();
      },
      onThird: function onThird() {
        this.hide();
      }
    });
  }

});

});

require.register("controllers/main/alerts/manage_alert_groups_controller", function(exports, require, module) {
'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var validator = require('utils/validator');

function stringify(obj, property) {
  return JSON.stringify(obj.get(property).slice().sort());
}

function groupsAreNotEqual(group1, group2) {
  return stringify(group1, 'definitions') !== stringify(group2, 'definitions') || stringify(group1, 'notifications') !== stringify(group2, 'notifications');
}

function mapToEmObjects(collection, fields, renamedFields) {
  var _renamedFields = arguments.length === 3 ? renamedFields : [];
  return collection.map(function (item) {
    var ret = Em.Object.create(Em.getProperties(item, fields));
    _renamedFields.forEach(function (renamedField) {
      var _renamedField$split = renamedField.split(':'),
          _renamedField$split2 = _slicedToArray(_renamedField$split, 2),
          realName = _renamedField$split2[0],
          newName = _renamedField$split2[1];

      Em.set(ret, newName, Em.get(item, realName));
    });
    return ret;
  });
}

var AlertGroupClone = Em.Object.extend({
  displayName: function () {
    var name = App.config.truncateGroupName(this.get('name'));
    return this.get('default') ? name + ' Default' : name;
  }.property('name', 'default'),
  label: Em.computed.format('{0} ({1})', 'displayName', 'definitions.length')
});

App.ManageAlertGroupsController = Em.Controller.extend({

  name: 'manageAlertGroupsController',

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * Property used to trigger Alert Groups Filter content updating
   * @type {Boolean}
   */
  changeTrigger: false,

  /**
   * @type {App.AlertGroup[]}
   */
  alertGroups: [],

  /**
   * @type {App.AlertGroup[]}
   */
  originalAlertGroups: [],

  /**
   * @type {App.AlertGroup}
   */
  selectedAlertGroup: null,

  /**
   * @type {App.AlertDefinition[]}
   */
  selectedDefinitions: [],

  /**
   * List of all Alert Notifications
   * @type {App.AlertNotification[]}
   */
  alertNotifications: function () {
    return this.get('isLoaded') ? mapToEmObjects(App.AlertNotification.find(), ['id', 'name', 'description', 'type', 'global']) : [];
  }.property('isLoaded'),

  /**
   * List of all global Alert Notifications
   * @type {App.AlertNotification[]}
   */
  alertGlobalNotifications: Em.computed.filterBy('alertNotifications', 'global', true),

  /**
   * @type {boolean}
   */
  isRemoveButtonDisabled: true,

  /**
   * @type {boolean}
   */
  isRenameButtonDisabled: true,

  /**
   * @type {boolean}
   */
  isDuplicateButtonDisabled: true,

  /**
   * @type {boolean}
   */
  isDeleteDefinitionsDisabled: function () {
    var selectedGroup = this.get('selectedAlertGroup');
    return selectedGroup ? selectedGroup.default || this.get('selectedDefinitions').length === 0 : true;
  }.property('selectedAlertGroup.definitions.length', 'selectedDefinitions.length'),

  /**
   * observes if any group changed including: group name, newly created group, deleted group, group with definitions/notifications changed
   * @type {{toDelete: App.AlertGroup[], toSet: App.AlertGroup[], toCreate: App.AlertGroup[]}}
   */
  defsModifiedAlertGroups: {},

  /**
   * Determines if some group was edited/created/deleted
   * @type {boolean}
   */
  isDefsModified: Em.computed.and('isLoaded', 'isDefsModifiedAlertGroups'),

  isDefsModifiedAlertGroups: Em.computed.or('defsModifiedAlertGroups.toSet.length', 'defsModifiedAlertGroups.toCreate.length', 'defsModifiedAlertGroups.toDelete.length'),

  /**
   * Check when some config group was changed and updates <code>defsModifiedAlertGroups</code> once
   * @method defsModifiedAlertGroupsObs
   */
  defsModifiedAlertGroupsObs: function () {
    Em.run.once(this, this.defsModifiedAlertGroupsObsOnce);
  }.observes('selectedAlertGroup.definitions.[]', 'selectedAlertGroup.notifications.[]', 'alertGroups', 'isLoaded'),

  /**
   * Update <code>defsModifiedAlertGroups</code>-value
   * Called once in the <code>defsModifiedAlertGroupsObs</code>
   * @method defsModifiedAlertGroupsObsOnce
   * @returns {boolean}
   */
  defsModifiedAlertGroupsObsOnce: function defsModifiedAlertGroupsObsOnce() {
    if (!this.get('isLoaded')) {
      return;
    }
    var groupsToDelete = [];
    var groupsToSet = [];
    var groupsToCreate = [];
    var groups = this.get('alertGroups'); //current alert groups
    var originalGroups = this.get('originalAlertGroups'); // original alert groups
    var mappedOriginalGroups = {}; // map is faster than `originalGroups.findProperty('id', ...)`
    originalGroups.forEach(function (group) {
      mappedOriginalGroups[group.get('id')] = group;
    });
    var originalGroupsIds = originalGroups.mapProperty('id');

    groups.forEach(function (group) {
      var originalGroup = mappedOriginalGroups[group.get('id')];
      if (originalGroup) {
        // should update definitions or notifications
        if (groupsAreNotEqual(group, originalGroup) || group.get('name') !== originalGroup.get('name')) {
          // should update name
          groupsToSet.push(group.set('id', originalGroup.get('id')));
        }
        originalGroupsIds = originalGroupsIds.without(group.get('id'));
      } else {
        // should add new group
        groupsToCreate.push(group);
      }
    });
    // should delete groups
    originalGroupsIds.forEach(function (id) {
      groupsToDelete.push(originalGroups.findProperty('id', id));
    });

    this.set('defsModifiedAlertGroups', {
      toDelete: groupsToDelete,
      toSet: groupsToSet,
      toCreate: groupsToCreate
    });
  },

  /**
   * Load all Alert Notifications from server
   * @returns {$.ajax}
   * @method loadAlertNotifications
   */
  loadAlertNotifications: function loadAlertNotifications() {
    this.setProperties({
      isLoaded: false,
      alertGroups: [],
      originalAlertGroups: [],
      selectedAlertGroup: null,
      isRemoveButtonDisabled: true,
      isRenameButtonDisabled: true,
      isDuplicateButtonDisabled: true
    });
    return App.ajax.send({
      name: 'alerts.notifications',
      sender: this,
      success: 'getAlertNotificationsSuccessCallback',
      error: 'getAlertNotificationsErrorCallback'
    });
  },

  /**
   * Success-callback for load alert notifications request
   * @param {object} json
   * @method getAlertNotificationsSuccessCallback
   */
  getAlertNotificationsSuccessCallback: function getAlertNotificationsSuccessCallback(json) {
    App.alertNotificationMapper.map(json);
    this.loadAlertGroups();
  },

  /**
   * Error-callback for load alert notifications request
   * @method getAlertNotificationsErrorCallback
   */
  getAlertNotificationsErrorCallback: function getAlertNotificationsErrorCallback() {
    this.set('isLoaded', true);
  },

  /**
   * Load all alert groups from alert group model
   * @method loadAlertGroups
   */
  loadAlertGroups: function loadAlertGroups() {
    var alertGroups = App.AlertGroup.find().map(function (group) {
      var definitions = mapToEmObjects(group.get('definitions'), ['name', 'serviceName', 'componentName', 'label', 'id'], ['service.displayName:serviceNameDisplay', 'componentNameFormatted:componentNameDisplay']);

      var targets = mapToEmObjects(group.get('targets'), ['name', 'id', 'description', 'type', 'global']);

      var hash = Em.getProperties(group, ['id', 'name', 'default', 'isAddDefinitionsDisabled']);
      hash.definitions = definitions;
      hash.notifications = targets;
      return AlertGroupClone.create(hash);
    });
    this.setProperties({
      alertGroups: alertGroups,
      isLoaded: true,
      originalAlertGroups: this.copyAlertGroups(alertGroups),
      selectedAlertGroup: alertGroups[0]
    });
  },

  /**
   * Enable/disable "Remove"/"Rename"/"Duplicate" buttons basing on <code>controller.selectedAlertGroup</code>
   * @method buttonObserver
   */
  buttonObserver: function () {
    var selectedAlertGroup = this.get('selectedAlertGroup');
    var flag = selectedAlertGroup && selectedAlertGroup.get('default');
    this.setProperties({
      isRemoveButtonDisabled: flag,
      isRenameButtonDisabled: flag,
      isDuplicateButtonDisabled: false
    });
  }.observes('selectedAlertGroup'),

  /**
   * @method resortAlertGroup
   */
  resortAlertGroup: function () {
    var alertGroups = Em.copy(this.get('alertGroups'));
    if (alertGroups.length < 2) {
      return;
    }
    var defaultGroups = alertGroups.filterProperty('default');
    defaultGroups.forEach(function (defaultGroup) {
      alertGroups.removeObject(defaultGroup);
    });
    var sorted = defaultGroups.sortProperty('name').concat(alertGroups.sortProperty('name'));

    this.removeObserver('alertGroups.@each.name', this, 'resortAlertGroup');
    this.set('alertGroups', sorted);
    this.addObserver('alertGroups.@each.name', this, 'resortAlertGroup');
  }.observes('alertGroups.@each.name'),

  /**
   * remove definitions from group
   * @method deleteDefinitions
   */
  deleteDefinitions: function deleteDefinitions() {
    if (this.get('isDeleteDefinitionsDisabled')) {
      return;
    }
    var groupDefinitions = this.get('selectedAlertGroup.definitions');
    this.get('selectedDefinitions').slice().forEach(function (defObj) {
      groupDefinitions.removeObject(defObj);
    }, this);
    this.set('selectedDefinitions', []);
  },

  /**
   * Provides alert definitions which are available for inclusion in
   * non-default alert groups.
   * @param {App.AlertGroup} selectedAlertGroup
   * @method getAvailableDefinitions
   * @return {{name: string, serviceName: string, componentName: string, serviceNameDisplay: string, componentNameDisplay: string, label: string, id: number}[]}
   */
  getAvailableDefinitions: function getAvailableDefinitions(selectedAlertGroup) {
    if (selectedAlertGroup.get('default')) return [];
    var usedDefinitionsMap = {};
    var availableDefinitions = [];
    var sharedDefinitions = App.AlertDefinition.find();

    usedDefinitionsMap = selectedAlertGroup.get('definitions').toWickMapByProperty('name');

    selectedAlertGroup.get('definitions').forEach(function (def) {
      usedDefinitionsMap[def.name] = true;
    });
    sharedDefinitions.forEach(function (sharedDef) {
      if (!usedDefinitionsMap[sharedDef.get('name')]) {
        availableDefinitions.pushObject(sharedDef);
      }
    });
    return mapToEmObjects(availableDefinitions, ['name', 'serviceName', 'componentName', 'label', 'id'], ['service.displayName:serviceNameDisplay', 'componentNameFormatted:componentNameDisplay']);
  },

  /**
   * add alert definitions to a group
   * @method addDefinitions
   */
  addDefinitions: function addDefinitions() {
    if (this.get('selectedAlertGroup.isAddDefinitionsDisabled')) {
      return false;
    }
    var availableDefinitions = this.getAvailableDefinitions(this.get('selectedAlertGroup'));
    var validComponents = App.StackServiceComponent.find().map(function (component) {
      return Em.Object.create({
        componentName: component.get('componentName'),
        displayName: App.format.role(component.get('componentName'), false),
        selected: false
      });
    });
    var validServices = App.Service.find().map(function (service) {
      return Em.Object.create({
        serviceName: service.get('serviceName'),
        displayName: App.format.role(service.get('serviceName'), true),
        selected: false
      });
    });
    return this.launchDefsSelectionDialog(availableDefinitions, [], validServices, validComponents);
  },

  /**
   * Launch a table view of all available definitions to choose
   * @method launchDefsSelectionDialog
   * @return {App.ModalPopup}
   */
  launchDefsSelectionDialog: function launchDefsSelectionDialog(initialDefs, selectedDefs, validServices, validComponents) {
    var self = this;
    return App.ModalPopup.show({

      classNames: ['common-modal-wrapper', 'full-height-modal'],

      modalDialogClasses: ['modal-lg'],

      header: Em.I18n.t('alerts.actions.manage_alert_groups_popup.selectDefsDialog.title'),

      /**
       * @type {string}
       */
      dialogMessage: Em.I18n.t('alerts.actions.manage_alert_groups_popup.selectDefsDialog.message').format(this.get('selectedAlertGroup.displayName')),

      /**
       * @type {string|null}
       */
      warningMessage: null,

      /**
       * @type {App.AlertDefinition[]}
       */
      availableDefs: [],

      onPrimary: function onPrimary() {
        this.set('warningMessage', null);
        var arrayOfSelectedDefs = this.get('availableDefs').filterProperty('selected', true);
        if (arrayOfSelectedDefs.length < 1) {
          this.set('warningMessage', Em.I18n.t('alerts.actions.manage_alert_groups_popup.selectDefsDialog.message.warning'));
          return;
        }
        self.addDefinitionsCallback(arrayOfSelectedDefs);
        this.hide();
      },

      /**
       * Primary button should be disabled while alert definitions are not loaded
       * @type {boolean}
       */
      disablePrimary: Em.computed.not('isLoaded'),

      onSecondary: function onSecondary() {
        self.addDefinitionsCallback(null);
        this.hide();
      },

      bodyClass: App.SelectDefinitionsPopupBodyView.extend({

        filterComponents: validComponents,

        filterServices: validServices,

        initialDefs: initialDefs

      })

    });
  },

  /**
   * add alert definitions callback
   * @method addDefinitionsCallback
   */
  addDefinitionsCallback: function addDefinitionsCallback(selectedDefs) {
    var group = this.get('selectedAlertGroup');
    if (selectedDefs) {
      group.get('definitions').pushObjects(selectedDefs);
    }
  },

  /**
   * copy alert groups for backup, to compare with current alert groups, so will know if some groups changed/added/deleted
   * @param {App.AlertGroup[]} originGroups
   * @return {App.AlertGroup[]}
   * @method copyAlertGroups
   */
  copyAlertGroups: function copyAlertGroups(originGroups) {
    var alertGroups = [];
    originGroups.forEach(function (alertGroup) {
      var copiedGroup = Em.Object.create($.extend(true, {}, alertGroup));
      alertGroups.pushObject(copiedGroup);
    });
    return alertGroups;
  },

  /**
   * Create a new alert group
   * @param {Em.Object} newAlertGroupData
   * @param {callback} callback Callback function for Success or Error handling
   * @return {App.AlertGroup} Returns the created alert group
   * @method postNewAlertGroup
   */
  postNewAlertGroup: function postNewAlertGroup(newAlertGroupData, callback) {
    // create a new group with name , definition and notifications
    var data = {
      name: newAlertGroupData.get('name')
    };
    if (newAlertGroupData.get('definitions').length > 0) {
      data.definitions = newAlertGroupData.get('definitions').mapProperty('id');
    }
    if (newAlertGroupData.get('notifications').length > 0) {
      data.targets = newAlertGroupData.get('notifications').mapProperty('id');
    }
    var sendData = {
      name: 'alert_groups.create',
      data: data,
      success: 'successFunction',
      error: 'errorFunction',
      successFunction: function successFunction() {
        if (callback) {
          callback();
        }
      },
      errorFunction: function errorFunction(xhr, text, errorThrown) {
        if (callback) {
          callback(xhr, text, errorThrown);
        }
      }
    };
    sendData.sender = sendData;
    App.ajax.send(sendData);
    return newAlertGroupData;
  },

  /**
   * PUTs the new alert group information on the server.
   * Changes possible here are the name, definitions, notifications
   *
   * @param {App.AlertGroup} alertGroup
   * @param {Function} successCallback
   * @param {Function} errorCallback
   * @method updateAlertGroup
   */
  updateAlertGroup: function updateAlertGroup(alertGroup, successCallback, errorCallback) {
    var sendData = {
      name: 'alert_groups.update',
      data: {
        group_id: alertGroup.id,
        name: alertGroup.get('name'),
        definitions: alertGroup.get('definitions').mapProperty('id'),
        targets: alertGroup.get('notifications').mapProperty('id')
      },
      success: 'successFunction',
      error: 'errorFunction',
      successFunction: function successFunction() {
        if (successCallback) {
          successCallback();
        }
      },
      errorFunction: function errorFunction(xhr, text, errorThrown) {
        if (errorCallback) {
          errorCallback(xhr, text, errorThrown);
        }
      }
    };
    sendData.sender = sendData;
    App.ajax.send(sendData);
  },

  /**
   * Request for deleting alert group
   * @param {App.AlertGroup} alertGroup
   * @param {callback} successCallback
   * @param {callback} errorCallback
   * @method removeAlertGroup
   */
  removeAlertGroup: function removeAlertGroup(alertGroup, successCallback, errorCallback) {
    var sendData = {
      name: 'alert_groups.delete',
      data: {
        group_id: alertGroup.id
      },
      success: 'successFunction',
      error: 'errorFunction',
      successFunction: function successFunction() {
        if (successCallback) {
          successCallback();
        }
      },
      errorFunction: function errorFunction(xhr, text, errorThrown) {
        if (errorCallback) {
          errorCallback(xhr, text, errorThrown);
        }
      }
    };
    sendData.sender = sendData;
    App.ajax.send(sendData);
  },

  /**
   * confirm delete alert group
   * @method confirmDelete
   */
  confirmDelete: function confirmDelete() {
    if (this.get('isRemoveButtonDisabled')) return;
    var self = this;
    App.showConfirmationPopup(function () {
      self.deleteAlertGroup();
    });
  },

  /**
   * delete selected alert group
   * @method deleteAlertGroup
   */
  deleteAlertGroup: function deleteAlertGroup() {
    var selectedAlertGroup = this.get('selectedAlertGroup');
    if (this.get('isDeleteAlertDisabled')) {
      return;
    }
    this.get('alertGroups').removeObject(selectedAlertGroup);
    this.set('selectedAlertGroup', this.get('alertGroups')[0]);
  },

  /**
   * Rename non-default alert group
   * @method renameAlertGroup
   */
  renameAlertGroup: function renameAlertGroup() {

    if (this.get('selectedAlertGroup.default')) {
      return;
    }
    var self = this;
    var popup = App.ModalPopup.show({

      header: Em.I18n.t('alerts.actions.manage_alert_groups_popup.renameButton'),

      bodyClass: Ember.View.extend({
        templateName: require('templates/main/alerts/create_new_alert_group')
      }),

      /**
       * @type {string}
       */
      alertGroupName: self.get('selectedAlertGroup.name'),

      alertGroupNameIsEmpty: function () {
        return !this.get('alertGroupName').trim().length;
      }.property('alertGroupName'),

      /**
       * @type {string|null}
       */
      warningMessage: function () {
        var originalGroup = self.get('selectedAlertGroup');
        var groupName = this.get('alertGroupName').trim();

        if (originalGroup.get('name').trim() === groupName) {
          return Em.I18n.t("alerts.actions.manage_alert_groups_popup.addGroup.exist");
        }
        if (self.get('alertGroups').mapProperty('displayName').contains(groupName)) {
          return Em.I18n.t("alerts.actions.manage_alert_groups_popup.addGroup.exist");
        }
        if (groupName && !validator.isValidAlertGroupName(groupName)) {
          return Em.I18n.t("form.validator.alertGroupName");
        }
        return '';
      }.property('alertGroupName'),

      /**
       * Primary button is disabled while user doesn't input valid group name
       * @type {boolean}
       */
      disablePrimary: Em.computed.or('alertGroupNameIsEmpty', 'warningMessage'),

      onPrimary: function onPrimary() {
        self.set('selectedAlertGroup.name', this.get('alertGroupName'));
        this._super();
      }

    });
    this.set('renameGroupPopup', popup);
  },

  /**
   * Create new alert group
   * @param {boolean} duplicated is new group a copy of the existing group
   * @method addAlertGroup
   */
  addAlertGroup: function addAlertGroup(duplicated) {
    duplicated = duplicated === true;
    var self = this;
    var popup = App.ModalPopup.show({

      header: Em.I18n.t('alerts.actions.manage_alert_groups_popup.addButton'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/alerts/create_new_alert_group')
      }),

      /**
       * Name for new alert group
       * @type {string}
       */
      alertGroupName: duplicated ? self.get('selectedAlertGroup.name') + ' Copy' : "",

      alertGroupNameIsEmpty: function () {
        return !this.get('alertGroupName').trim().length;
      }.property('alertGroupName'),

      /**
       * @type {string}
       */
      warningMessage: function () {
        var groupName = this.get('alertGroupName').trim();
        if (self.get('alertGroups').mapProperty('displayName').contains(groupName)) {
          return Em.I18n.t("alerts.actions.manage_alert_groups_popup.addGroup.exist");
        }
        if (groupName && !validator.isValidAlertGroupName(groupName)) {
          return Em.I18n.t("form.validator.alertGroupName");
        }
        return '';
      }.property('alertGroupName'),

      /**
       * Primary button is disabled while user doesn't input valid group name
       * @type {boolean}
       */
      disablePrimary: Em.computed.or('alertGroupNameIsEmpty', 'warningMessage'),

      onPrimary: function onPrimary() {
        var newAlertGroup = AlertGroupClone.create({
          name: this.get('alertGroupName').trim(),
          default: false,
          definitions: duplicated ? self.get('selectedAlertGroup.definitions').slice(0) : [],
          notifications: self.get('alertGlobalNotifications'),
          isAddDefinitionsDisabled: false
        });
        self.get('alertGroups').pushObject(newAlertGroup);
        self.set('selectedAlertGroup', newAlertGroup);
        this.hide();
      }

    });
    this.set('addGroupPopup', popup);
  },

  /**
   * @method duplicateAlertGroup
   */
  duplicateAlertGroup: function duplicateAlertGroup() {
    this.addAlertGroup(true);
  }

});

});

require.register("controllers/main/alerts/manage_alert_notifications_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.ManageAlertNotificationsController = Em.Controller.extend({

  name: 'manageAlertNotificationsController',

  /**
   * Are alert notifications loaded
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * Create/Edit modal popup object
   * used to hide popup
   * @type {App.ModalPopup}
   */
  createEditPopup: null,

  /**
   * Map of edit inputs shown in Create/Edit Notification popup
   * @type {Object}
   */
  inputFields: Em.Object.create({
    name: {
      label: Em.I18n.t('common.name'),
      value: '',
      defaultValue: ''
    },
    groups: {
      label: Em.I18n.t('common.groups'),
      value: [],
      defaultValue: []
    },
    global: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.global'),
      value: false,
      defaultValue: false,
      disabled: false
    },
    allGroups: Em.Object.create({
      value: '',
      defaultValue: 'custom',
      disabled: false,
      isAll: Em.computed.equal('value', 'all')
    }),
    method: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.method'),
      value: '',
      defaultValue: ''
    },
    email: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.email'),
      value: '',
      defaultValue: ''
    },
    SMTPServer: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPServer'),
      value: '',
      defaultValue: ''
    },
    SMTPPort: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPPort'),
      value: '',
      defaultValue: ''
    },
    SMTPUseAuthentication: Em.Object.create({
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPUseAuthentication'),
      value: false,
      defaultValue: false,
      invertedValue: Em.computed.not('value')
    }),
    SMTPUsername: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPUsername'),
      value: '',
      defaultValue: ''
    },
    SMTPPassword: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPPassword'),
      value: '',
      defaultValue: ''
    },
    retypeSMTPPassword: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.retypeSMTPPassword'),
      value: '',
      defaultValue: ''
    },
    SMTPSTARTTLS: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.SMTPSTARTTLS'),
      value: false,
      defaultValue: false
    },
    emailFrom: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.emailFrom'),
      value: '',
      defaultValue: ''
    },
    version: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.version'),
      value: '',
      defaultValue: ''
    },
    OIDs: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.OIDs'),
      value: '',
      defaultValue: ''
    },
    community: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.community'),
      value: '',
      defaultValue: ''
    },
    host: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.host'),
      value: '',
      defaultValue: ''
    },
    port: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.port'),
      value: '',
      defaultValue: ''
    },
    severityFilter: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.severityFilter'),
      value: [],
      defaultValue: []
    },
    description: {
      label: Em.I18n.t('common.description'),
      value: '',
      defaultValue: ''
    },
    scriptDispatchProperty: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.scriptDispatchProperty'),
      value: '',
      defaultValue: ''
    },
    scriptFileName: {
      label: Em.I18n.t('alerts.actions.manage_alert_notifications_popup.scriptFileName'),
      value: '',
      defaultValue: ''
    },
    customProperties: Em.A([])
  }),

  /**
   * List of available options for Enable or Disable
   * used in settings of SelectedAlertNotification
   * @type {Object}
   */
  enableOrDisable: {
    enable: "enable",
    disable: "disable"
  },

  /**
   * List of available Notification types
   * used in Type combobox
   * @type {Array}
   */
  methods: ['EMAIL', 'SNMP', 'Custom SNMP', 'Alert Script'],

  /**
   * List of available value for Severity Filter
   * used in Severity Filter combobox
   * @type {Array}
   */
  severities: ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN'],

  /**
   * List of available SNMP versions
   * @type {Array}
   */
  SNMPVersions: ['SNMPv1', 'SNMPv2c'],

  /**
   * List of all Alert Notifications
   * @type {App.AlertNotification[]}
   */
  alertNotifications: function () {
    return this.get('isLoaded') ? App.AlertNotification.find().toArray() : [];
  }.property('isLoaded'),

  /**
   * List of all Alert Groups
   * @type {App.AlertGroup[]}
   */
  allAlertGroups: function () {
    return this.get('isLoaded') ? App.AlertGroup.find().toArray() : [];
  }.property('isLoaded'),

  /**
   * Selected Alert Notification
   * @type {App.AlertNotification}
   */
  selectedAlertNotification: null,

  /**
   * Addable to <code>selectedAlertNotification.properties</code> custom property
   * @type {{name: string, value: string}}
   */
  newCustomProperty: { name: '', value: '' },

  /**
   * @type {string[]}
   */
  customPropertyNames: Em.computed.mapBy('inputFields.customProperties', 'name'),

  /**
   * Check if custom property name exists in the <code>inputFields.customProperties</code>
   *
   * @type {boolean}
   */
  isNewCustomPropertyExists: Em.computed.existsInByKey('newCustomProperty.name', 'customPropertyNames'),

  /**
   * Check if custom property name exists in the <code>ignoredCustomProperties</code>
   *
   * @type {boolean}
   */
  isNewCustomPropertyIgnored: Em.computed.existsInByKey('newCustomProperty.name', 'ignoredCustomProperties'),

  /**
   * Check if custom property name is valid according to the <code>validator.isValidConfigKey</code>
   *
   * @type {boolean}
   */
  isNewCustomPropertyNameValid: function () {
    return validator.isValidConfigKey(this.get('newCustomProperty.name'));
  }.property('newCustomProperty.name'),

  /**
   * Error message for the new custom property name
   *
   * @type {string}
   */
  errorMessageForNewCustomPropertyName: function () {
    var isNewCustomPropertyIgnored = this.get('isNewCustomPropertyIgnored');
    var isNewCustomPropertyExists = this.get('isNewCustomPropertyExists');
    var flag = this.get('isNewCustomPropertyNameValid');
    if (flag) {
      if (isNewCustomPropertyExists || isNewCustomPropertyIgnored) {
        return Em.I18n.t('alerts.notifications.addCustomPropertyPopup.error.propertyExists');
      }
    } else {
      return Em.I18n.t('alerts.notifications.addCustomPropertyPopup.error.invalidPropertyName');
    }
    return '';
  }.property('isNewCustomPropertyNameValid', 'isNewCustomPropertyExists', 'isNewCustomPropertyIgnored'),

  /**
   * If some error with new custom property
   *
   * @type {boolean}
   */
  isErrorWithNewCustomPropertyName: Em.computed.bool('errorMessageForNewCustomPropertyName'),

  /**
   * List custom property names that shouldn't be displayed on Edit page
   * @type {string[]}
   */
  ignoredCustomProperties: ['ambari.dispatch.credential.password', 'ambari.dispatch.credential.username', 'ambari.dispatch.recipients', 'ambari.dispatch.snmp.community', 'ambari.dispatch.snmp.oids.trap', 'ambari.dispatch.snmp.oids.subject', 'ambari.dispatch.snmp.oids.body', 'ambari.dispatch.snmp.port', 'ambari.dispatch.snmp.version', 'mail.smtp.auth', 'mail.smtp.from', 'mail.smtp.host', 'mail.smtp.port', 'mail.smtp.starttls.enable', 'ambari.dispatch-property.script', 'ambari.dispatch-property.script.filename'],

  validationMap: {
    EMAIL: [{
      errorKey: 'emailToError',
      validator: 'emailToValidation'
    }, {
      errorKey: 'emailFromError',
      validator: 'emailFromValidation'
    }, {
      errorKey: 'smtpPortError',
      validator: 'smtpPortValidation'
    }, {
      errorKey: 'smtpUsernameError',
      validator: 'smtpUsernameValidation'
    }, {
      errorKey: 'smtpPasswordError',
      validator: 'smtpPasswordValidation'
    }, {
      errorKey: 'passwordError',
      validator: 'retypePasswordValidation'
    }],
    SNMP: [{
      errorKey: 'portError',
      validator: 'portValidation'
    }, {
      errorKey: 'hostError',
      validator: 'hostsValidation'
    }],
    CustomSNMP: [{
      errorKey: 'portError',
      validator: 'portValidation'
    }, {
      errorKey: 'hostError',
      validator: 'hostsValidation'
    }],
    AlertScript: [{
      errorKey: 'scriptFileNameError',
      validator: 'scriptFileNameValidation'
    }]
  },

  /**
   * Load all Alert Notifications from server
   * @method loadAlertNotifications
   */
  loadAlertNotifications: function loadAlertNotifications() {
    this.set('isLoaded', false);
    return App.ajax.send({
      name: 'alerts.notifications',
      sender: this,
      success: 'getAlertNotificationsSuccessCallback',
      error: 'getAlertNotificationsErrorCallback'
    });
  },

  /**
   * Success-callback for load alert notifications request
   * @param {object} json
   * @method getAlertNotificationsSuccessCallback
   */
  getAlertNotificationsSuccessCallback: function getAlertNotificationsSuccessCallback(json) {
    App.alertNotificationMapper.map(json);
    this.set('isLoaded', true);
  },

  /**
   * Error-callback for load alert notifications request
   * @method getAlertNotificationsErrorCallback
   */
  getAlertNotificationsErrorCallback: function getAlertNotificationsErrorCallback() {
    this.set('isLoaded', true);
  },

  /**
   * Add Notification button handler
   * @method addAlertNotification
   */
  addAlertNotification: function addAlertNotification() {
    var inputFields = this.get('inputFields');
    inputFields.setProperties({
      'global.disabled': false
    });
    Em.keys(inputFields).forEach(function (key) {
      inputFields.set(key + '.value', inputFields.get(key + '.defaultValue'));
    });
    inputFields.set('severityFilter.value', ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN']);
    inputFields.set('customProperties', Em.A([]));
    this.showCreateEditPopup(false);
  },

  /**
   * Edit Notification button handler
   * @method editAlertNotification
   */
  editAlertNotification: function editAlertNotification() {
    this.fillEditCreateInputs();
    this.showCreateEditPopup(true);
  },

  /**
   * Fill inputs of Create/Edit popup form
   * @param addCopyToName define whether add 'Copy of ' to name
   * @method fillEditCreateInputs
   */
  fillEditCreateInputs: function fillEditCreateInputs(addCopyToName) {
    var inputFields = this.get('inputFields');
    var selectedAlertNotification = this.get('selectedAlertNotification');
    var methodValue = this.getNotificationTypeText(selectedAlertNotification.get('type'));
    var properties = selectedAlertNotification.get('properties');
    inputFields.set('name.value', (addCopyToName ? 'Copy of ' : '') + selectedAlertNotification.get('name'));
    inputFields.set('groups.value', selectedAlertNotification.get('groups').toArray());
    inputFields.set('email.value', properties['ambari.dispatch.recipients'] ? properties['ambari.dispatch.recipients'].join(', ') : '');
    inputFields.set('SMTPServer.value', properties['mail.smtp.host']);
    inputFields.set('SMTPPort.value', properties['mail.smtp.port']);
    inputFields.set('SMTPUseAuthentication.value', properties['mail.smtp.auth'] !== "false");
    inputFields.set('SMTPUsername.value', properties['ambari.dispatch.credential.username']);
    inputFields.set('SMTPPassword.value', properties['ambari.dispatch.credential.password']);
    inputFields.set('retypeSMTPPassword.value', properties['ambari.dispatch.credential.password']);
    inputFields.set('SMTPSTARTTLS.value', properties['mail.smtp.starttls.enable'] !== "false");
    inputFields.set('emailFrom.value', properties['mail.smtp.from']);
    inputFields.set('version.value', properties['ambari.dispatch.snmp.version']);
    inputFields.set('OIDs.value', properties['ambari.dispatch.snmp.oids.trap']);
    inputFields.set('community.value', properties['ambari.dispatch.snmp.community']);
    inputFields.set('host.value', properties['ambari.dispatch.recipients'] ? properties['ambari.dispatch.recipients'].join(', ') : '');
    inputFields.set('port.value', properties['ambari.dispatch.snmp.port']);
    inputFields.set('severityFilter.value', selectedAlertNotification.get('alertStates'));
    inputFields.set('global.value', selectedAlertNotification.get('global'));
    inputFields.set('allGroups.value', selectedAlertNotification.get('global') ? 'all' : 'custom');
    inputFields.set('scriptDispatchProperty.value', properties['ambari.dispatch-property.script'] || '');
    inputFields.set('scriptFileName.value', properties['ambari.dispatch-property.script.filename'] || '');
    // not allow to edit global field
    inputFields.set('global.disabled', true);
    inputFields.set('description.value', selectedAlertNotification.get('description'));
    inputFields.set('method.value', methodValue);
    inputFields.get('customProperties').clear();
    var ignoredCustomProperties = this.get('ignoredCustomProperties');
    Em.keys(properties).forEach(function (k) {
      if (ignoredCustomProperties.contains(k)) return;
      inputFields.get('customProperties').pushObject({
        name: k,
        value: properties[k],
        defaultValue: properties[k]
      });
    });
  },

  /**
   * Show Edit or Create Notification popup
   * @param {boolean} isEdit true - edit, false - create
   * @returns {App.ModalPopup}
   * @method showCreateEditPopup
   */
  showCreateEditPopup: function showCreateEditPopup(isEdit) {
    var self = this;
    var createEditPopup = App.ModalPopup.show({
      header: isEdit ? Em.I18n.t('alerts.actions.manage_alert_notifications_popup.editHeader') : Em.I18n.t('alerts.actions.manage_alert_notifications_popup.addHeader'),
      classNames: ['create-edit-alert-notification-popup'],
      marginBottom: 130,
      bodyClass: Em.View.extend({
        controller: this,
        templateName: require('templates/main/alerts/create_alert_notification'),

        didInsertElement: function didInsertElement() {
          App.tooltip($('.checkbox-tooltip'));
          this.nameValidation();
          this.emailToValidation();
          this.emailFromValidation();
          this.smtpPortValidation();
          this.hostsValidation();
          this.portValidation();
          this.smtpUsernameValidation();
          this.smtpPasswordValidation();
          this.retypePasswordValidation();
          this.scriptFileNameValidation();
        },

        isEmailMethodSelected: Em.computed.equal('controller.inputFields.method.value', 'EMAIL'),

        isSNMPMethodSelected: Em.computed.equal('controller.inputFields.method.value', 'SNMP'),

        isCustomSNMPMethodSelected: Em.computed.equal('controller.inputFields.method.value', 'Custom SNMP'),

        isAlertScriptMethodSelected: Em.computed.equal('controller.inputFields.method.value', 'Alert Script'),

        methodObserver: function () {
          var currentMethod = this.get('controller.inputFields.method.value'),
              validationMap = self.get('validationMap');
          self.get('methods').forEach(function (method) {
            // Replace blank spaces with empty character
            var mapKey = method.replace(/\s/g, "");
            var validations = validationMap[mapKey];
            if (method === currentMethod) {
              validations.mapProperty('validator').forEach(function (key) {
                this.get(key).call(this);
              }, this);
            } else {
              validations.mapProperty('errorKey').forEach(function (key) {
                this.set(key, false);
              }, this);
            }
          }, this);
        }.observes('controller.inputFields.method.value'),

        nameValidation: function () {
          var newName = this.get('controller.inputFields.name.value').trim();
          var errorMessage = '';
          // on editing, save current notification name
          if (newName && !this.get('currentName')) {
            this.set('currentName', newName);
          }
          var nameExistsCondition = isEdit ? newName && newName !== this.get('currentName') : !!newName;
          if (!newName) {
            this.set('nameError', true);
            errorMessage = Em.I18n.t('alerts.actions.manage_alert_notifications_popup.error.name.empty');
          } else if (nameExistsCondition && self.get('alertNotifications').mapProperty('name').contains(newName)) {
            this.set('nameError', true);
            errorMessage = Em.I18n.t('alerts.actions.manage_alert_notifications_popup.error.name.existed');
          } else if (newName && !validator.isValidAlertNotificationName(newName)) {
            this.set('nameError', true);
            errorMessage = Em.I18n.t('form.validator.alertNotificationName');
          } else {
            this.set('nameError', false);
          }
          this.set('controller.inputFields.name.errorMsg', errorMessage);
        }.observes('controller.inputFields.name.value'),

        emailToValidation: function () {
          var emailToError = false;
          if (this.get('isEmailMethodSelected')) {
            var inputValues = this.get('controller.inputFields.email.value').trim().split(',');
            emailToError = inputValues.some(function (emailTo) {
              return emailTo && !validator.isValidEmail(emailTo.trim());
            });
          }
          this.set('emailToError', emailToError);
          this.set('controller.inputFields.email.errorMsg', emailToError ? Em.I18n.t('alerts.notifications.error.email') : null);
        }.observes('controller.inputFields.email.value'),

        emailFromValidation: function () {
          var emailFrom = this.get('controller.inputFields.emailFrom.value');
          if (emailFrom && !validator.isValidEmail(emailFrom)) {
            this.set('emailFromError', true);
            this.set('controller.inputFields.emailFrom.errorMsg', Em.I18n.t('alerts.notifications.error.email'));
          } else {
            this.set('emailFromError', false);
            this.set('controller.inputFields.emailFrom.errorMsg', null);
          }
        }.observes('controller.inputFields.emailFrom.value'),

        smtpPortValidation: function () {
          var value = this.get('controller.inputFields.SMTPPort.value');
          if (value && (!validator.isValidInt(value) || value < 0)) {
            this.set('smtpPortError', true);
            this.set('controller.inputFields.SMTPPort.errorMsg', Em.I18n.t('alerts.notifications.error.integer'));
          } else {
            this.set('smtpPortError', false);
            this.set('controller.inputFields.SMTPPort.errorMsg', null);
          }
        }.observes('controller.inputFields.SMTPPort.value'),

        hostsValidation: function () {
          var inputValue = this.get('controller.inputFields.host.value').trim(),
              hostError = false;
          if (!this.get('isEmailMethodSelected') && !this.get('isAlertScriptMethodSelected')) {
            var array = inputValue.split(',');
            hostError = array.some(function (hostname) {
              return hostname && !validator.isHostname(hostname.trim());
            });
            hostError = hostError || inputValue === '';
          }
          this.set('hostError', hostError);
          this.set('controller.inputFields.host.errorMsg', hostError ? Em.I18n.t('alerts.notifications.error.host') : null);
        }.observes('controller.inputFields.host.value'),

        portValidation: function () {
          var value = this.get('controller.inputFields.port.value');
          if (value && (!validator.isValidInt(value) || value < 0)) {
            this.set('portError', true);
            this.set('controller.inputFields.port.errorMsg', Em.I18n.t('alerts.notifications.error.integer'));
          } else {
            this.set('portError', false);
            this.set('controller.inputFields.port.errorMsg', null);
          }
        }.observes('controller.inputFields.port.value'),

        smtpUsernameValidation: function () {
          var smtpUsernameError = false;
          var errorMessage = null;
          if (this.get('isEmailMethodSelected')) {
            if (this.get('controller.inputFields.SMTPUseAuthentication.value')) {
              if (Em.isBlank(this.get('controller.inputFields.SMTPUsername.value'))) {
                smtpUsernameError = true;
                errorMessage = Em.I18n.t('alerts.notifications.error.SMTPUsername');
              }
            }
          }
          this.set('smtpUsernameError', smtpUsernameError);
          this.set('controller.inputFields.SMTPUsername.errorMsg', errorMessage);
        }.observes('controller.inputFields.SMTPUsername.value', 'controller.inputFields.SMTPUseAuthentication.value'),

        smtpPasswordValidation: function () {
          var smtpPasswordError = false;
          var errorMessage = null;
          if (this.get('isEmailMethodSelected')) {
            if (this.get('controller.inputFields.SMTPUseAuthentication.value')) {
              if (Em.isBlank(this.get('controller.inputFields.SMTPPassword.value'))) {
                smtpPasswordError = true;
                errorMessage = Em.I18n.t('alerts.notifications.error.SMTPPassword');
              }
            }
          }
          this.set('smtpPasswordError', smtpPasswordError);
          this.set('controller.inputFields.SMTPPassword.errorMsg', errorMessage);
        }.observes('controller.inputFields.SMTPPassword.value', 'controller.inputFields.SMTPUseAuthentication.value'),

        retypePasswordValidation: function () {
          var passwordValue = this.get('controller.inputFields.SMTPPassword.value');
          var retypePasswordValue = this.get('controller.inputFields.retypeSMTPPassword.value');
          if (passwordValue !== retypePasswordValue) {
            this.set('passwordError', true);
            this.set('controller.inputFields.retypeSMTPPassword.errorMsg', Em.I18n.t('alerts.notifications.error.retypePassword'));
          } else {
            this.set('passwordError', false);
            this.set('controller.inputFields.retypeSMTPPassword.errorMsg', null);
          }
        }.observes('controller.inputFields.retypeSMTPPassword.value', 'controller.inputFields.SMTPPassword.value'),

        scriptFileNameValidation: function () {
          var scriptFileNameValue = this.get('controller.inputFields.scriptFileName.value').trim();
          if (!Em.isBlank(scriptFileNameValue) && !validator.isValidFileName(scriptFileNameValue)) {
            this.set('scriptFileNameError', true);
            this.set('controller.inputFields.scriptFileName.errorMsg', Em.I18n.t('alerts.actions.manage_alert_notifications_popup.error.scriptFileName.invalid'));
          } else {
            this.set('scriptFileNameError', false);
            this.set('controller.inputFields.scriptFileName.errorMsg', null);
          }
        }.observes('controller.inputFields.scriptFileName.value'),

        someErrorExists: Em.computed.or('nameError', 'emailToError', 'emailFromError', 'smtpPortError', 'hostError', 'portError', 'smtpUsernameError', 'smtpPasswordError', 'passwordError', 'scriptFileNameError'),

        setParentErrors: function () {
          this.set('parentView.hasErrors', this.get('someErrorExists'));
        }.observes('someErrorExists'),

        groupsSelectView: Em.Select.extend({
          attributeBindings: ['disabled'],
          init: function init() {
            this._super();
            this.set('parentView.groupSelect', this);
          }
        }),

        groupSelect: null,

        /**
         * Select all alert-groups if <code>allGroups.value</code> is 'custom'
         * @method selectAllGroups
         */
        selectAllGroups: function selectAllGroups() {
          if (this.get('controller.inputFields.allGroups.value') === 'custom') {
            this.set('groupSelect.selection', this.get('groupSelect.content').slice());
          }
        },

        /**
         * Deselect all alert-groups if <code>allGroups.value</code> is 'custom'
         * @method clearAllGroups
         */
        clearAllGroups: function clearAllGroups() {
          if (this.get('controller.inputFields.allGroups.value') === 'custom') {
            this.set('groupSelect.selection', []);
          }
        },

        severitySelectView: Em.Select.extend({
          init: function init() {
            this._super();
            this.set('parentView.severitySelect', this);
          }
        }),

        severitySelect: null,

        /**
         * Determines if all alert-groups are selected
         * @type {boolean}
         */
        allGroupsSelected: Em.computed.equalProperties('groupSelect.selection.length', 'groupSelect.content.length'),

        /**
         * Determines if no one alert-group is selected
         * @type {boolean}
         */
        noneGroupsSelected: Em.computed.empty('groupSelect.selection'),

        /**
         * Determines if all severities are selected
         * @type {boolean}
         */
        allSeveritySelected: Em.computed.equalProperties('severitySelect.selection.length', 'severitySelect.content.length'),

        /**
         * Determines if no one severity is selected
         * @type {boolean}
         */
        noneSeveritySelected: Em.computed.empty('severitySelect.selection'),

        /**
         * Select all severities
         * @method selectAllSeverity
         */
        selectAllSeverity: function selectAllSeverity() {
          this.set('severitySelect.selection', this.get('severitySelect.content').slice());
        },

        /**
         * Deselect all severities
         * @method clearAllSeverity
         */
        clearAllSeverity: function clearAllSeverity() {
          this.set('severitySelect.selection', []);
        }
      }),

      isSaving: false,

      hasErrors: false,

      primary: Em.I18n.t('common.save'),

      disablePrimary: Em.computed.or('isSaving', 'hasErrors'),

      onPrimary: function onPrimary() {
        this.set('isSaving', true);
        var apiObject = self.formatNotificationAPIObject();
        if (isEdit) {
          self.updateAlertNotification(apiObject);
        } else {
          self.createAlertNotification(apiObject);
        }
      },
      hide: function hide() {
        self.set('createEditPopup', null);
        return this._super.apply(this, arguments);
      }
    });
    this.set('createEditPopup', createEditPopup);
    return createEditPopup;
  },

  /**
   * Create API-formatted object from data populate by user
   * @returns {Object}
   * @method formatNotificationAPIObject
   */
  formatNotificationAPIObject: function formatNotificationAPIObject() {
    var inputFields = this.get('inputFields');
    var properties = {};
    if (inputFields.get('method.value') === 'EMAIL') {
      properties['ambari.dispatch.recipients'] = inputFields.get('email.value').replace(/\s/g, '').split(',');
      properties['mail.smtp.host'] = inputFields.get('SMTPServer.value');
      properties['mail.smtp.port'] = inputFields.get('SMTPPort.value');
      properties['mail.smtp.from'] = inputFields.get('emailFrom.value');
      properties['mail.smtp.auth'] = inputFields.get('SMTPUseAuthentication.value');
      if (inputFields.get('SMTPUseAuthentication.value')) {
        properties['ambari.dispatch.credential.username'] = inputFields.get('SMTPUsername.value');
        properties['ambari.dispatch.credential.password'] = inputFields.get('SMTPPassword.value');
        properties['mail.smtp.starttls.enable'] = inputFields.get('SMTPSTARTTLS.value');
      }
    } else if (inputFields.get('method.value') === 'SNMP') {
      properties['ambari.dispatch.snmp.version'] = inputFields.get('version.value');
      properties['ambari.dispatch.snmp.community'] = inputFields.get('community.value');
      properties['ambari.dispatch.recipients'] = inputFields.get('host.value').replace(/\s/g, '').split(',');
      properties['ambari.dispatch.snmp.port'] = inputFields.get('port.value');
    } else if (inputFields.get('method.value') === 'Custom SNMP') {
      properties['ambari.dispatch.snmp.version'] = inputFields.get('version.value');
      properties['ambari.dispatch.snmp.oids.trap'] = inputFields.get('OIDs.value');
      properties['ambari.dispatch.snmp.oids.subject'] = inputFields.get('OIDs.value');
      properties['ambari.dispatch.snmp.oids.body'] = inputFields.get('OIDs.value');
      properties['ambari.dispatch.snmp.community'] = inputFields.get('community.value');
      properties['ambari.dispatch.recipients'] = inputFields.get('host.value').replace(/\s/g, '').split(',');
      properties['ambari.dispatch.snmp.port'] = inputFields.get('port.value');
    } else if (inputFields.get('method.value') === 'Alert Script') {
      var scriptDispatchProperty = inputFields.get('scriptDispatchProperty.value').trim();
      if (scriptDispatchProperty != '') properties['ambari.dispatch-property.script'] = scriptDispatchProperty;

      var scriptFileName = inputFields.get('scriptFileName.value').trim();
      if (scriptFileName != '') properties['ambari.dispatch-property.script.filename'] = scriptFileName;
    }
    inputFields.get('customProperties').forEach(function (customProperty) {
      properties[customProperty.name] = customProperty.value;
    });
    var apiObject = {
      AlertTarget: {
        name: inputFields.get('name.value'),
        description: inputFields.get('description.value'),
        global: inputFields.get('allGroups.value') === 'all',
        notification_type: this.getNotificationType(inputFields.get('method.value')),
        alert_states: inputFields.get('severityFilter.value'),
        properties: properties
      }
    };
    if (inputFields.get('allGroups.value') === 'custom') {
      apiObject.AlertTarget.groups = inputFields.get('groups.value').mapProperty('id');
    }
    return apiObject;
  },

  getNotificationType: function getNotificationType(text) {
    var notificationType = text;
    if (notificationType === "Custom SNMP") {
      notificationType = "SNMP";
    } else if (notificationType === "SNMP") {
      notificationType = "AMBARI_SNMP";
    } else if (notificationType === "Alert Script") {
      notificationType = "ALERT_SCRIPT";
    }
    return notificationType;
  },

  getNotificationTypeText: function getNotificationTypeText(notificationType) {
    var notificationTypeText = notificationType;
    if (notificationType === "SNMP") {
      notificationTypeText = "Custom SNMP";
    } else if (notificationType === "AMBARI_SNMP") {
      notificationTypeText = "SNMP";
    } else if (notificationType === "ALERT_SCRIPT") {
      notificationTypeText = "Alert Script";
    }
    return notificationTypeText;
  },

  /**
   * Send request to server to create Alert Notification
   * @param {object} apiObject (@see formatNotificationAPIObject)
   * @returns {$.ajax}
   * @method createAlertNotification
   */
  createAlertNotification: function createAlertNotification(apiObject) {
    return App.ajax.send({
      name: 'alerts.create_alert_notification',
      sender: this,
      data: {
        data: apiObject
      },
      success: 'createAlertNotificationSuccessCallback',
      error: 'saveErrorCallback'
    });
  },

  /**
   * Success callback for <code>createAlertNotification</code>
   * @method createAlertNotificationSuccessCallback
   */
  createAlertNotificationSuccessCallback: function createAlertNotificationSuccessCallback() {
    this.loadAlertNotifications();
    var createEditPopup = this.get('createEditPopup');
    if (createEditPopup) {
      createEditPopup.hide();
    }
  },

  /**
   * Send request to server to update Alert Notification
   * @param {object} apiObject (@see formatNotificationAPIObject)
   * @returns {$.ajax}
   * @method updateAlertNotification
   */
  updateAlertNotification: function updateAlertNotification(apiObject) {
    if (apiObject != null) {
      if (apiObject.AlertTarget.global == false) {
        this.get('selectedAlertNotification').set('global', false);
      } else {
        this.get('selectedAlertNotification').set('global', true);
      }
    }
    return App.ajax.send({
      name: 'alerts.update_alert_notification',
      sender: this,
      data: {
        data: apiObject,
        id: this.get('selectedAlertNotification.id')
      },
      success: 'updateAlertNotificationSuccessCallback',
      error: 'saveErrorCallback'
    });
  },

  /**
   * Success callback for <code>updateAlertNotification</code>
   * @method updateAlertNotificationSuccessCallback
   */
  updateAlertNotificationSuccessCallback: function updateAlertNotificationSuccessCallback() {
    this.loadAlertNotifications();
    var createEditPopup = this.get('createEditPopup');
    if (createEditPopup) {
      createEditPopup.hide();
    }
  },

  /**
   * Error callback for <code>createAlertNotification</code> and <code>updateAlertNotification</code>
   * @method saveErrorCallback
   */
  saveErrorCallback: function saveErrorCallback() {
    this.set('createEditPopup.isSaving', false);
  },

  /**
   * Delete Notification button handler
   * @return {App.ModalPopup}
   * @method deleteAlertNotification
   */
  deleteAlertNotification: function deleteAlertNotification() {
    var self = this;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'alerts.delete_alert_notification',
        sender: self,
        data: {
          id: self.get('selectedAlertNotification.id')
        },
        success: 'deleteAlertNotificationSuccessCallback'
      });
    }, Em.I18n.t('alerts.actions.manage_alert_notifications_popup.confirmDeleteBody').format(this.get('selectedAlertNotification.name')), null, Em.I18n.t('alerts.actions.manage_alert_notifications_popup.confirmDeleteHeader'), Em.I18n.t('common.delete'));
  },

  /**
   * Success callback for <code>deleteAlertNotification</code>
   * @method deleteAlertNotificationSuccessCallback
   */
  deleteAlertNotificationSuccessCallback: function deleteAlertNotificationSuccessCallback() {
    this.loadAlertNotifications();
    var selectedAlertNotification = this.get('selectedAlertNotification');
    selectedAlertNotification.deleteRecord();
    this.set('selectedAlertNotification', null);
  },

  /**
   * Duplicate Notification button handler
   * @method duplicateAlertNotification
   */
  duplicateAlertNotification: function duplicateAlertNotification() {
    this.fillEditCreateInputs(true);
    this.showCreateEditPopup();
  },

  /**
   * Enable or Disable Notification button handler
   * @method enableOrDisableAlertNotification
   */
  enableOrDisableAlertNotification: function enableOrDisableAlertNotification(e) {
    return App.ajax.send({
      name: 'alerts.update_alert_notification',
      sender: this,
      data: {
        data: {
          AlertTarget: {
            enabled: e.context !== "disable"
          }
        },
        id: this.get('selectedAlertNotification.id')
      },
      success: 'enableOrDisableAlertNotificationSuccessCallback',
      error: 'saveErrorCallback'
    });
  },

  /**
   * Success callback for <code>enableOrDisableAlertNotification</code>
   * @method enableOrDisableAlertNotificationSuccessCallback
   */
  enableOrDisableAlertNotificationSuccessCallback: function enableOrDisableAlertNotificationSuccessCallback() {
    this.loadAlertNotifications();
    var createEditPopup = this.get('createEditPopup');
    if (createEditPopup) {
      createEditPopup.hide();
    }
  },

  /**
   * Show popup with form for new custom property
   * @method addCustomPropertyHandler
   * @return {App.ModalPopup}
   */
  addCustomPropertyHandler: function addCustomPropertyHandler() {
    var self = this;

    return App.ModalPopup.show({

      controllerBinding: 'App.router.manageAlertNotificationsController',

      header: Em.I18n.t('alerts.notifications.addCustomPropertyPopup.header'),

      primary: Em.I18n.t('common.add'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/alerts/add_custom_config_to_alert_notification_popup')
      }),

      disablePrimary: Em.computed.alias('controller.isErrorWithNewCustomPropertyName'),

      onPrimary: function onPrimary() {
        self.addCustomProperty();
        self.set('newCustomProperty', { name: '', value: '' }); // cleanup
        this.hide();
      }

    });
  },

  /**
   * Add Custom Property to <code>selectedAlertNotification</code> (push it to <code>properties</code>-field)
   * @method addCustomProperty
   */
  addCustomProperty: function addCustomProperty() {
    var newCustomProperty = this.get('newCustomProperty');
    this.get('inputFields.customProperties').pushObject({
      name: newCustomProperty.name,
      value: newCustomProperty.value,
      defaultValue: newCustomProperty.value
    });
  },

  /**
   * "Remove"-button click handler
   * Delete customProperty from <code>inputFields.customProperties</code>
   * @param {object} e
   * @method removeCustomProperty
   */
  removeCustomPropertyHandler: function removeCustomPropertyHandler(e) {
    var customProperties = this.get('inputFields.customProperties');
    this.set('inputFields.customProperties', customProperties.without(e.context));
  }

});

});

require.register("controllers/main/charts", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainChartsController = Em.ArrayController.extend({
  name: 'mainChartsController'
});

});

require.register("controllers/main/charts/heatmap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainChartsHeatmapController = Em.Controller.extend(App.WidgetSectionMixin, {
  name: 'mainChartsHeatmapController',
  rackMap: {},
  racks: [],
  rackViews: [],

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * Heatmap metrics that are available choices  on the page
   */
  heatmapCategories: [],

  allHeatmaps: [],

  layoutNameSuffix: "_heatmap",

  sectionNameSuffix: "_HEATMAPS",

  loadRacksUrlParams: 'fields=Hosts/rack_info,Hosts/host_name,Hosts/public_host_name,Hosts/os_type,Hosts/ip,host_components,metrics/disk,metrics/cpu/cpu_system,metrics/cpu/cpu_user,metrics/memory/mem_total,metrics/memory/mem_free&minimal_response=true',

  /**
   * @type {string}
   */
  loadHeatmapsUrlParams: function () {
    var serviceName = this.get('content.serviceName');
    if (serviceName) {
      return 'WidgetInfo/widget_type=HEATMAP&WidgetInfo/scope=CLUSTER&WidgetInfo/metrics.matches(.*\"service_name\":\"' + serviceName + '\".*)&fields=WidgetInfo/metrics';
    } else {
      return 'WidgetInfo/widget_type=HEATMAP&WidgetInfo/scope=CLUSTER&fields=WidgetInfo/metrics';
    }
  }.property('content.serviceName'),

  selectedMetric: null,

  inputMaximum: '',

  /**
   * Heatmap widget currently shown on the page
   */
  activeWidget: Em.computed.alias('widgets.firstObject'),

  /**
   * This function is called from the bound view of the controller
   */
  loadPageData: function loadPageData() {
    var self = this;

    this.loadRacks().always(function () {
      self.resetPageData();
      self.getAllHeatMaps().done(function (allHeatmapData) {
        self.set('isLoaded', true);
        allHeatmapData.items.forEach(function (_allHeatmapData) {
          self.get('allHeatmaps').pushObject(_allHeatmapData.WidgetInfo);
        });
        var categories = self.categorizeByServiceName(self.get('allHeatmaps'));
        self.set('heatmapCategories', categories);
        self.getActiveWidgetLayout();
      });
    });
  },

  /**
   * categorize heatmaps with respect to service names
   * @param {Array} allHeatmaps
   * @return {Array}
   */
  categorizeByServiceName: function categorizeByServiceName(allHeatmaps) {
    var categories = [];

    allHeatmaps.forEach(function (_heatmap) {
      var serviceNames = JSON.parse(_heatmap.metrics).mapProperty('service_name').uniq();
      serviceNames.forEach(function (_serviceName) {
        var category = categories.findProperty('serviceName', _serviceName);
        if (!category) {
          categories.pushObject(Em.Object.create({
            serviceName: _serviceName,
            displayName: _serviceName === 'STACK' ? 'Host' : App.format.role(_serviceName, true),
            heatmaps: [_heatmap]
          }));
        } else {
          category.get('heatmaps').pushObject(_heatmap);
        }
      }, this);
    }, this);
    return categories;
  },

  /**
   * clears/resets the data. This function should be called every time user navigates to heatmap page
   */
  resetPageData: function resetPageData() {
    this.get('heatmapCategories').clear();
    this.get('allHeatmaps').clear();
  },

  /**
   *  Gets all heatmap widgets that should be available in select metrics dropdown on heatmap page
   * @return {$.ajax}
   */
  getAllHeatMaps: function getAllHeatMaps() {
    var urlParams = this.get('loadHeatmapsUrlParams');

    return App.ajax.send({
      name: 'widgets.get',
      sender: this,
      data: {
        urlParams: urlParams,
        sectionName: this.get('sectionName')
      }
    });
  },

  /**
   * get hosts from server
   */
  loadRacks: function loadRacks() {
    this.get('racks').clear();
    this.set('rackMap', {});
    var urlParams = this.get('loadRacksUrlParams');
    return App.ajax.send({
      name: 'hosts.heatmaps',
      sender: this,
      data: {
        urlParams: urlParams
      },
      success: 'loadRacksSuccessCallback'
    });
  },

  loadRacksSuccessCallback: function loadRacksSuccessCallback(data, opt, params) {
    var hosts = [];
    data.items.forEach(function (item) {
      hosts.push({
        hostName: item.Hosts.host_name,
        publicHostName: item.Hosts.public_host_name,
        osType: item.Hosts.os_type,
        ip: item.Hosts.ip,
        rack: item.Hosts.rack_info,
        diskTotal: item.metrics ? item.metrics.disk.disk_total : 0,
        diskFree: item.metrics ? item.metrics.disk.disk_free : 0,
        cpuSystem: item.metrics ? item.metrics.cpu.cpu_system : 0,
        cpuUser: item.metrics ? item.metrics.cpu.cpu_user : 0,
        memTotal: item.metrics ? item.metrics.memory.mem_total : 0,
        memFree: item.metrics ? item.metrics.memory.mem_free : 0,
        hostComponents: item.host_components.mapProperty('HostRoles.component_name')
      });
    });
    var rackMap = this.indexByRackId(hosts);
    var racks = this.toList(rackMap);
    //this list has an empty host array property
    this.set('rackMap', rackMap);
    this.set('racks', racks);
  },

  /**
   * @param {Array} hosts
   * @returns {Object} rackMap
   */
  indexByRackId: function indexByRackId(hosts) {
    var rackMap = {};

    hosts.forEach(function (host) {
      var rackId = host.rack;
      if (!rackMap[rackId]) {
        rackMap[rackId] = Em.Object.create({
          name: rackId,
          rackId: rackId,
          hosts: [host]
        });
      } else {
        rackMap[rackId].hosts.push(host);
      }
    });
    return rackMap;
  },

  /**
   *
   * @param {Object} rackMap
   * @returns {Array} racks
   */
  toList: function toList(rackMap) {
    var racks = [];
    var i = 0;
    for (var rackKey in rackMap) {
      if (rackMap.hasOwnProperty(rackKey)) {
        racks.push(Em.Object.create({
          name: rackKey,
          rackId: rackKey,
          hosts: rackMap[rackKey].hosts,
          isLoaded: false,
          index: i++
        }));
      }
    }
    return racks;
  },

  validation: function () {
    if (this.get('selectedMetric')) {
      if (/^\d+$/.test(this.get('inputMaximum'))) {
        $('#inputMaximum').removeClass('error');
        this.set('selectedMetric.maximumValue', this.get('inputMaximum'));
      } else {
        $('#inputMaximum').addClass('error');
      }
    }
  }.observes('inputMaximum'),

  addRackView: function addRackView(view) {
    this.get('rackViews').push(view);
    if (this.get('rackViews').length == this.get('racks').length) {
      this.displayAllRacks();
    }
  },

  displayAllRacks: function displayAllRacks() {
    if (this.get('rackViews').length) {
      this.get('rackViews').pop().displayHosts();
      this.displayAllRacks();
    }
  },

  showHeatMapMetric: function showHeatMapMetric(event) {
    var self = this;
    var metricItem = Em.Object.create(event.context);
    this.saveWidgetLayout([metricItem]).done(function () {
      self.getActiveWidgetLayout();
    });
  },

  hostToSlotMap: Em.computed.alias('selectedMetric.hostToSlotMap'),

  /**
   * return class name for to be used for containing each rack.
   *
   * @this App.MainChartsHeatmapController
   */
  rackClass: function () {
    var rackCount = this.get('racks.length');
    if (rackCount < 2) {
      return "col-md-12";
    }
    if (rackCount === 2) {
      return "col-md-6";
    }
    return "col-md-4";
  }.property('racks.length')
});

});

require.register("controllers/main/charts/heatmap_metrics/heatmap_metric", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var date = require('utils/date/date');

var numberUtils = require('utils/number_utils');

/**
 * Base class for any heatmap metric.
 * 
 * This class basically provides the following for each heatmap metric.
 * <ul>
 * <li> Provides number of slots in which temperature can fall.
 * <li> Maintains the maximum value so as to scale slot ranges.
 * <li> Gets JSON data from server and maps response for all hosts into above
 * slots.
 * </ul>
 * 
 */
App.MainChartHeatmapMetric = Em.Object.extend({
  /**
   * Name of this metric
   */
  name: null,

  /**
   * Number of slots this metric will be mapped into. When changing this value,
   * the color count in 'slotColors' should also be changed.
   */
  numberOfSlots: 5,

  /**
   * Colors for the each of the number of slots defined above. When changing the
   * number of slots, the number of colors also should be updated.
   * 
   * @type {Array}
   */
  slotColors: ['#1EB475', '#1FB418', '#E9E23F', '#E9A840', '#EF6162'],

  /**
   * Minimum value of this metric. Default is 0.
   */
  minimumValue: 0,

  /**
   * Maximum value of this metric. This has to be specified by extending classes
   * so that the range from 'minimumValue' to 'maximumValue' can be split among
   * 'numberOfSlots'. It is recommended that this value be a multiple of
   * 'numberOfSlots'.
   */
  maximumValue: 100,

  /**
   * Units of the maximum value which is shown in UI {String}
   */
  units: '',

  /**
   * Provides following information about slots in an array of objects.
   * <ul>
   * <li> from: {number} Slot starts from this value
   * <li> to: {number} Slot ends at this value (inclusive)
   * <li> label: {String} Slot name to be shown
   * <li> cssStyle: {String} style to be embedded on hosts which fall into this
   * slot.
   * </ul>
   * 
   * Slot count will be the same as specified in 'numberOfSlots'. Slot
   * definitions will be given in increasing temperature from 'minimumValue' to
   * 'maximumValue'.
   * 
   */
  slotDefinitions: function () {
    var max = this.getMaximumValue();
    var slotCount = this.get('numberOfSlots');
    var units = this.get('units');
    var delta = (max - this.get('minimumValue')) / slotCount;
    var defs = [];
    var slotColors = this.get('slotColors');
    var slotColorIndex = 0;
    for (var c = 0; c < slotCount - 1; c++) {
      var slotColor = slotColors[slotColorIndex++];
      var start = c * delta;
      var end = (c + 1) * delta;
      defs.push(this.generateSlot(start, end, units, slotColor));
    }
    slotColor = slotColors[slotColorIndex++];
    start = (slotCount - 1) * delta;
    defs.push(this.generateSlot(start, max, units, slotColor));

    defs.push(Em.Object.create({
      invalidData: true,
      index: defs.length,
      label: Em.I18n.t('charts.heatmap.label.invalidData'),
      cssStyle: this.getHatchStyle()
    }));
    defs.push(Em.Object.create({
      notAvailable: true,
      index: defs.length,
      label: Em.I18n.t('charts.heatmap.label.notAvailable'),
      cssStyle: "background-color: #666"
    }));
    defs.push(Em.Object.create({
      notApplicable: true,
      index: defs.length,
      label: Em.I18n.t('charts.heatmap.label.notApplicable'),
      cssStyle: "background-color:#ccc"
    }));
    return defs;
  }.property('minimumValue', 'maximumValue', 'numberOfSlots'),
  /**
   * genarate slot by given parameters
   *
   * @param min
   * @param max
   * @param units
   * @param slotColor
   * @return {object}
   * @method generateSlot
   */
  generateSlot: function generateSlot(min, max, units, slotColor) {
    var fromLabel = this.formatLegendLabel(min, units);
    var toLabel = this.formatLegendLabel(max, units);
    var from = this.convertNumber(min, units);
    var to = this.convertNumber(max, units);

    return Em.Object.create({
      hasBoundaries: true,
      from: from,
      to: to,
      label: fromLabel + " - " + toLabel,
      cssStyle: "background-color:" + slotColor
    });
  },

  /**
   * calculate hatch style of slot according to browser version used
   * @return {String}
   */
  getHatchStyle: function getHatchStyle() {
    var hatchStyle = "background-color:#666";
    if (jQuery.browser.webkit) {
      hatchStyle = "background-image:-webkit-repeating-linear-gradient(-45deg, #666, #666 6px, #fff 6px, #fff 7px)";
    } else if (jQuery.browser.mozilla) {
      hatchStyle = "background-image:repeating-linear-gradient(-45deg, #666, #666 6px, #fff 6px, #fff 7px)";
    } else if (jQuery.browser.msie && jQuery.browser.version) {
      var majorVersion = parseInt(jQuery.browser.version.split('.')[0]);
      if (majorVersion > 9) {
        hatchStyle = "background-image:repeating-linear-gradient(-45deg, #666, #666 6px, #fff 6px, #fff 7px)";
      }
    }
    return hatchStyle;
  },

  /**
   * compatible with special input value, such as negative float number or string
   * @return {float}
   */
  getMaximumValue: function getMaximumValue() {
    var max = this.get('maximumValue') ? parseFloat(this.get('maximumValue')) : 0;
    return isNaN(max) ? 0 : Math.max(0, max);
  },

  /**
   * In slot definitions this value is used to construct the label by appending
   * it to slot min-max values. For example giving '%' here would result in slot
   * definition label being '0% - 10%'.
   */
  slotDefinitionLabelSuffix: '',

  hostToValueMap: null,

  hostToSlotMap: function () {
    var hostToValueMap = this.get('hostToValueMap');
    var hostNames = this.get('hostNames');
    var hostToSlotMap = {};
    if (hostToValueMap && hostNames) {
      hostNames.forEach(function (hostName) {
        var slot = this.calculateSlot(hostToValueMap, hostName);
        if (slot > -1) {
          hostToSlotMap[hostName] = slot;
        }
      }, this);
    }
    return hostToSlotMap;
  }.property('hostToValueMap', 'slotDefinitions'),

  /**
   * calculate slot position in hostToSlotMap by hostname
   * @param hostToValueMap
   * @param hostName
   * @return {Number}
   */
  calculateSlot: function calculateSlot(hostToValueMap, hostName) {
    var slotDefinitions = this.get('slotDefinitions');
    var slotWithBoundaries = slotDefinitions.filterProperty('hasBoundaries');
    var invalidDataSlot = slotDefinitions.findProperty('invalidData').get('index');
    var notAvailableDataSlot = slotDefinitions.findProperty('notAvailable').get('index');
    var slot = slotDefinitions.findProperty('notApplicable').get('index');

    if (hostName in hostToValueMap) {
      var value = hostToValueMap[hostName];
      if (Em.isNone(value)) {
        slot = notAvailableDataSlot;
      } else if (isNaN(value) || !isFinite(value) || value < 0) {
        slot = invalidDataSlot;
      } else {
        value = Number(value);
        slotWithBoundaries.forEach(function (slotDef, slotIndex, array) {

          if (value >= slotDef.from && value <= slotDef.to ||
          // If value exceeds maximum then it pushed to the last/maximal slot
          value > slotDef.to && slotIndex === array.length - 1) {
            slot = slotIndex;
          }
        });
      }
    }
    return slot;
  },

  /**
   * Turns numbers into displayable values. For example 24.345432425 into 24.3
   * etc.
   * 
   * @private
   */
  formatLegendLabel: function formatLegendLabel(num, units) {
    var fraction = num % 1;
    if (num >= 100) {
      num = Math.round(num);
    } else if (num >= 10 && fraction > 0) {
      num = parseFloat(num.toFixed(1));
    } else if (fraction > 0) {
      num = parseFloat(num.toFixed(2));
    }
    if (units === 'ms') {
      return date.timingFormat(num);
    }
    return num + units;
  },

  /**
   * return converted value
   *
   * @param {number} number
   * @param {string} units
   */
  convertNumber: function convertNumber(number, units) {
    if (units === 'MB') {
      return Math.round(number * numberUtils.BYTES_IN_MB);
    }
    return number;
  }

});

});

require.register("controllers/main/dashboard", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainDashboardController = Em.Controller.extend({
  name: 'mainDashboardController',
  categorySelected: 'widgets'
});

});

require.register("controllers/main/dashboard/config_history_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainConfigHistoryController = Em.ArrayController.extend(App.TableServerMixin, {
  name: 'mainConfigHistoryController',

  content: App.ServiceConfigVersion.find(),
  totalCount: 0,
  filteredCount: 0,
  resetStartIndex: true,
  mockUrl: '/data/configurations/service_versions.json',
  realUrl: function () {
    return App.apiPrefix + '/clusters/' + App.get('clusterName') + '/configurations/service_config_versions?<parameters>fields=service_config_version,user,group_id,group_name,is_current,createtime,service_name,hosts,service_config_version_note,is_cluster_compatible,stack_id&minimal_response=true';
  }.property('App.clusterName'),

  /**
   * @type {boolean}
   * @default false
   */
  showFilterConditionsFirstLoad: false,

  /**
   * associations between host property and column index
   * @type {Array}
   */
  colPropAssoc: function () {
    var associations = [];
    associations[1] = 'serviceVersion';
    associations[2] = 'configGroup';
    associations[3] = 'createTime';
    associations[4] = 'author';
    associations[5] = 'notes';
    return associations;
  }.property(),

  filterProps: [{
    name: 'serviceVersion',
    key: 'service_name',
    type: 'EQUAL'
  }, {
    name: 'configGroup',
    key: 'group_name',
    type: 'EQUAL'
  }, {
    name: 'createTime',
    key: 'createtime',
    type: 'MORE'
  }, {
    name: 'author',
    key: 'user',
    type: 'MATCH'
  }, {
    name: 'notes',
    key: 'service_config_version_note',
    type: 'MATCH'
  }],

  sortProps: [{
    name: 'serviceVersion',
    key: 'service_name'
  }, {
    name: 'configGroup',
    key: 'group_name'
  }, {
    name: 'createTime',
    key: 'createtime'
  }, {
    name: 'author',
    key: 'user'
  }, {
    name: 'notes',
    key: 'service_config_version_note'
  }],

  /**
   * load all data components required by config history table
   *  - total counter of service config versions(called in parallel)
   *  - current versions
   *  - filtered versions
   * @param {boolean} shouldUpdateCounter
   * @return {*}
   */
  load: function load() {
    var _this = this;

    var shouldUpdateCounter = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    var dfd = $.Deferred();
    this.loadConfigVersionsToModel().done(function () {
      if (shouldUpdateCounter) {
        _this.updateTotalCounter();
      }
      dfd.resolve();
    });
    return dfd.promise();
  },

  /**
   * get filtered service config versions from server and push it to model
   * @return {*}
   */
  loadConfigVersionsToModel: function loadConfigVersionsToModel() {
    var dfd = $.Deferred();
    var queryParams = this.getQueryParameters();

    App.HttpClient.get(this.getUrl(queryParams), App.serviceConfigVersionsMapper, {
      complete: function complete() {
        dfd.resolve();
      }
    });
    return dfd.promise();
  },

  updateTotalCounter: function updateTotalCounter() {
    return App.ajax.send({
      name: 'service.serviceConfigVersions.get.total',
      sender: this,
      data: {},
      success: 'updateTotalCounterSuccess'
    });
  },

  updateTotalCounterSuccess: function updateTotalCounterSuccess(data, opt, params) {
    this.set('totalCount', Number(data.itemTotal));
  },

  getUrl: function getUrl(queryParams) {
    var params = '';
    if (App.get('testMode')) {
      return this.get('mockUrl');
    } else {
      if (queryParams) {
        params = App.router.get('updateController').computeParameters(queryParams);
      }
      params = params.length > 0 ? params + "&" : params;
      return this.get('realUrl').replace('<parameters>', params);
    }
  },

  subscribeToUpdates: function subscribeToUpdates() {
    App.StompClient.addHandler('/events/configs', 'history', this.load.bind(this, true));
  },

  unsubscribeOfUpdates: function unsubscribeOfUpdates() {
    App.StompClient.removeHandler('/events/configs', 'history');
  },

  /**
   * get sort properties from local db. A overridden funtion from parent
   * @return {Array}
   */
  getSortProps: function getSortProps() {
    var savedSortConditions = App.db.getSortingStatuses(this.get('name')) || [],
        sortProperties = this.get('sortProps'),
        sortParams = [];

    savedSortConditions.forEach(function (sort) {
      var property = sortProperties.findProperty('name', sort.name);
      if (property && (sort.status === 'sorting_asc' || sort.status === 'sorting_desc')) {
        property.value = sort.status.replace('sorting_', '');
        property.type = 'SORT';
        if (property.name == 'serviceVersion') {
          property.key = "service_name." + sort.status.replace('sorting_', '') + ",service_config_version";
          property.value = "desc";
        }
        if (property.name == 'configGroup') {
          property.key = "group_name." + sort.status.replace('sorting_', '') + ",service_config_version";
          property.value = "desc";
        }

        sortParams.push(property);
      }
    });
    return sortParams;
  },

  /**
   *
   * @param {string} name
   */
  getSearchBoxSuggestions: function getSearchBoxSuggestions(name) {
    var dfd = $.Deferred();
    var key = this.get('filterProps').findProperty('name', name).key;
    App.ajax.send({
      name: 'service.serviceConfigVersions.get.suggestions',
      sender: this,
      data: {
        key: key
      }
    }).done(function (data) {
      dfd.resolve(data.items.mapProperty(key).uniq());
    }).fail(function () {
      dfd.resolve([]);
    });
    return dfd.promise();
  }
});

});

require.register("controllers/main/host", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.MainHostController = Em.ArrayController.extend(App.TableServerMixin, {
  name: 'mainHostController',

  clearFilters: null,

  filteredCount: 0,

  /**
   * total number of installed hosts
   * @type {number}
   */
  totalCount: Em.computed.alias('App.allHostNames.length'),

  /**
   * @type {boolean}
   * @default false
   */
  resetStartIndex: false,

  startIndex: 1,

  /**
   * true if any host page filter changed
   */
  filterChangeHappened: false,

  /**
   * if true, do not clean stored filter before hosts page rendering.
   */
  showFilterConditionsFirstLoad: false,

  saveSelection: false,

  content: App.Host.find(),

  allHostStackVersions: App.HostStackVersion.find(),
  /**
   * filterProperties support follow types of filter:
   * MATCH - match of RegExp
   * EQUAL - equality "="
   * LESS - "<"
   * MORE - ">"
   * MULTIPLE - multiple values to compare
   * CUSTOM - substitute values with keys "{#}" in alias
   */
  filterProperties: [{
    name: 'hostName',
    key: 'Hosts/host_name',
    type: 'MATCH'
  }, {
    name: 'ip',
    key: 'Hosts/ip',
    type: 'MATCH'
  }, {
    name: 'cpu',
    key: 'Hosts/cpu_count',
    type: 'EQUAL'
  }, {
    name: 'memoryFormatted',
    key: 'Hosts/total_mem',
    type: 'EQUAL'
  }, {
    name: 'loadAvg',
    key: 'metrics/load/load_one',
    type: 'EQUAL'
  }, {
    name: 'rack',
    key: 'Hosts/rack_info',
    type: 'MATCH'
  }, {
    name: 'hostComponents',
    key: 'host_components/HostRoles/component_name',
    type: 'EQUAL',
    isComponentRelatedFilter: true
  }, {
    name: 'services',
    key: 'host_components/HostRoles/service_name',
    type: 'MATCH',
    isComponentRelatedFilter: true
  }, {
    name: 'state',
    key: 'host_components/HostRoles/state',
    type: 'MATCH',
    isComponentRelatedFilter: true
  }, {
    name: 'healthClass',
    key: 'Hosts/host_status',
    type: 'EQUAL'
  }, {
    name: 'criticalWarningAlertsCount',
    key: '(alerts_summary/CRITICAL{0}|alerts_summary/WARNING{1})',
    type: 'CUSTOM'
  }, {
    name: 'componentsWithStaleConfigsCount',
    key: 'host_components/HostRoles/stale_configs',
    type: 'EQUAL',
    isComponentRelatedFilter: true
  }, {
    name: 'componentsInPassiveStateCount',
    key: 'host_components/HostRoles/maintenance_state',
    type: 'MULTIPLE',
    isComponentRelatedFilter: true
  }, {
    name: 'selected',
    key: 'Hosts/host_name',
    type: 'MULTIPLE'
  }, {
    name: 'version',
    key: 'stack_versions/repository_versions/RepositoryVersions/display_name',
    type: 'EQUAL'
  }, {
    name: 'versionState',
    key: 'stack_versions/HostStackVersions/state',
    type: 'EQUAL'
  }, {
    name: 'hostStackVersion',
    key: 'stack_versions',
    type: 'EQUAL'
  }, {
    name: 'componentState',
    key: ['(host_components/HostRoles/component_name={0})', '(host_components/HostRoles/component_name={0}&host_components/HostRoles/state={1})', '(host_components/HostRoles/component_name={0}&host_components/HostRoles/desired_admin_state={1})', '(host_components/HostRoles/component_name={0}&host_components/HostRoles/maintenance_state={1})'],
    type: 'COMBO',
    isComponentRelatedFilter: true
  }],

  sortProps: [{
    name: 'hostName',
    key: 'Hosts/host_name'
  }, {
    name: 'ip',
    key: 'Hosts/ip'
  }, {
    name: 'cpu',
    key: 'Hosts/cpu_count'
  }, {
    name: 'memoryFormatted',
    key: 'Hosts/total_mem'
  }, {
    name: 'diskUsage',
    //TODO disk_usage is relative property and need support from API, metrics/disk/disk_free used temporarily
    key: 'metrics/disk/disk_free'
  }, {
    name: 'rack',
    key: 'Hosts/rack_info'
  }, {
    name: 'loadAvg',
    key: 'metrics/load/load_one'
  }],

  /**
   * Validate and convert input string to valid url parameter.
   * Detect if user have passed string as regular expression or extend
   * string to regexp.
   *
   * @param {String} value
   * @return {String}
   **/
  getRegExp: function getRegExp(value) {
    value = validator.isValidMatchesRegexp(value) ? value.replace(/(\.+\*?|(\.\*)+)$/, '') + '.*' : '^$';
    value = /^\.\*/.test(value) || value == '^$' ? value : '.*' + value;
    return value;
  },

  /**
   * Sort by host_name by default
   * @method getSortProps
   * @returns {{value: 'asc|desc', name: string, type: 'SORT'}[]}
   */
  getSortProps: function getSortProps() {
    var controllerName = this.get('name'),
        db = App.db.getSortingStatuses(controllerName);
    if (db && db.everyProperty('status', 'sorting')) {
      App.db.setSortingStatuses(controllerName, {
        name: 'hostName',
        status: 'sorting_asc'
      });
    }
    return this._super();
  },

  /**
   * get query parameters computed from filter properties, sort properties and custom properties of view
   * @param {boolean} [skipNonFilterProperties]
   * @return {Array}
   * @method getQueryParameters
   */
  getQueryParameters: function getQueryParameters(skipNonFilterProperties) {
    skipNonFilterProperties = skipNonFilterProperties || false;
    var queryParams = [],
        savedFilterConditions = App.db.getFilterConditions(this.get('name')) || [],
        colPropAssoc = this.get('colPropAssoc'),
        filterProperties = this.get('filterProperties'),
        sortProperties = this.get('sortProps'),
        oldProperties = App.router.get('updateController.queryParams.Hosts');

    this.set('resetStartIndex', false);

    queryParams.pushObjects(this.getPaginationProps());

    savedFilterConditions.forEach(function (filter) {
      var property = filterProperties.findProperty('name', colPropAssoc[filter.iColumn]);
      if (property && filter.value.length > 0 && !filter.skipFilter) {
        var result = {
          key: property.key,
          value: filter.value,
          type: property.type,
          isFilter: true,
          isComponentRelatedFilter: property.isComponentRelatedFilter
        };
        if (filter.type === 'string' && sortProperties.someProperty('name', colPropAssoc[filter.iColumn])) {
          if (Em.isArray(filter.value)) {
            for (var i = 0; i < filter.value.length; i++) {
              filter.value[i] = this.getRegExp(filter.value[i]);
            }
          } else {
            result.value = this.getRegExp(filter.value);
          }
        }
        if (filter.type === 'number' || filter.type === 'ambari-bandwidth') {
          result.type = this.getComparisonType(filter.value);
          result.value = this.getProperValue(filter.value);
        }
        // enter an exact number for RAM filter, need to do a range number match for this
        if (filter.type === 'ambari-bandwidth' && result.type == 'EQUAL' && result.value) {
          var valuePair = this.convertMemoryToRange(filter.value);
          queryParams.push({
            key: result.key,
            value: valuePair[0],
            type: 'MORE'
          });
          queryParams.push({
            key: result.key,
            value: valuePair[1],
            type: 'LESS'
          });
        } else if (filter.type === 'ambari-bandwidth' && result.type != 'EQUAL' && result.value) {
          // enter a comparison type, eg > 1, just do regular match
          result.value = this.convertMemory(filter.value);
          queryParams.push(result);
        } else if (filter.type === 'sub-resource') {
          filter.value.forEach(function (item) {
            queryParams.push({
              key: result.key + "/" + item.property,
              value: item.value,
              type: 'EQUAL'
            });
          }, this);
        } else {
          queryParams.push(result);
        }
      }
    }, this);

    if (!oldProperties.findProperty('isHostDetails')) {
      // shouldn't reset start index after coming back from Host Details page
      if (queryParams.filterProperty('isFilter').length !== oldProperties.filterProperty('isFilter').length) {
        queryParams.findProperty('key', 'from').value = 0;
        this.set('resetStartIndex', true);
      } else {
        queryParams.filterProperty('isFilter').forEach(function (queryParam) {
          var oldProperty = oldProperties.filterProperty('isFilter').findProperty('key', queryParam.key);
          if (!oldProperty || JSON.stringify(oldProperty.value) !== JSON.stringify(queryParam.value)) {
            queryParams.findProperty('key', 'from').value = 0;
            this.set('resetStartIndex', true);
          }
        }, this);
      }
    }

    if (!skipNonFilterProperties) {
      queryParams.pushObjects(this.getSortProps());
    }

    return queryParams;
  },

  /**
   * Return value without predicate
   * @param {String} value
   * @return {String}
   */
  getProperValue: function getProperValue(value) {
    return ['>', '<', '='].contains(value.charAt(0)) ? value.substr(1, value.length) : value;
  },

  /**
   * Return value converted to kilobytes
   * @param {String} value
   * @return {number}
   */
  convertMemory: function convertMemory(value) {
    var scale = value.charAt(value.length - 1);
    // first char may be predicate for comparison
    value = this.getProperValue(value);
    var parsedValue = parseFloat(value);

    if (isNaN(parsedValue)) {
      return value;
    }

    switch (scale) {
      case 'g':
        parsedValue *= 1048576;
        break;
      case 'm':
        parsedValue *= 1024;
        break;
      case 'k':
        break;
      default:
        //default value in GB
        parsedValue *= 1048576;
    }
    return Math.round(parsedValue);
  },

  /**
   * Return value converted to a range of kilobytes
   * @param {String} value
   * @return {Array}
   */
  convertMemoryToRange: function convertMemoryToRange(value) {
    var scale = value.charAt(value.length - 1);
    // first char may be predicate for comparison
    value = this.getProperValue(value);
    var parsedValue = parseFloat(value);
    if (isNaN(parsedValue)) {
      return [0, 0];
    }
    var parsedValuePair = this.rangeConvertNumber(parsedValue, scale);
    var multiplyingFactor = 1;
    switch (scale) {
      case 'g':
        multiplyingFactor = 1048576;
        break;
      case 'm':
        multiplyingFactor = 1024;
        break;
      case 'k':
        break;
      default:
        //default value in GB
        multiplyingFactor = 1048576;
    }
    parsedValuePair[0] = Math.round(parsedValuePair[0] * multiplyingFactor);
    parsedValuePair[1] = Math.round(parsedValuePair[1] * multiplyingFactor);
    return parsedValuePair;
  },

  /**
   * Return value converted to a range of kilobytes
   * eg, return value 1.83 g will target 1.82500 ~ 1.83499 g
   * eg, return value 1.8 k will target 1.7500 ~ 1.8499 k
   * eg, return value 1.8 m will target 1.7500 ~ 1.8499 m
   * @param {number} value
   * @param {String} scale
   * @return {Array}
   */
  rangeConvertNumber: function rangeConvertNumber(value, scale) {
    if (isNaN(value)) {
      return [0, 0];
    }
    var valuePair = [];
    switch (scale) {
      case 'g':
        valuePair = [value - 0.005000, value + 0.004999999];
        break;
      case 'm':
      case 'k':
        valuePair = [value - 0.05000, value + 0.04999];
        break;
      default:
        //default value in GB
        valuePair = [value - 0.005000, value + 0.004999999];
    }
    return valuePair;
  },

  /**
   * Return comparison type depending on populated predicate
   * @param {string} value
   * @return {string}
   */
  getComparisonType: function getComparisonType(value) {
    var comparisonChar = value.charAt(0);
    var result = 'EQUAL';
    if (isNaN(comparisonChar)) {
      switch (comparisonChar) {
        case '>':
          result = 'MORE';
          break;
        case '<':
          result = 'LESS';
          break;
      }
    }
    return result;
  },

  labelValueMap: {},

  /**
   * Filter hosts by componentName of <code>component</code>
   * @param {App.HostComponent} component
   */
  filterByComponent: function filterByComponent(component) {
    if (!component) return;
    var componentName = component.get('componentName');
    var displayName = App.format.role(componentName, false);
    var colPropAssoc = this.get('colPropAssoc');
    var map = this.get('labelValueMap');

    var filterForComponent = {
      iColumn: 15,
      value: componentName + ':ALL',
      type: 'string'
    };
    map[displayName] = componentName;
    map['All'] = 'ALL';
    var filterStr = '"' + displayName + '"' + ': "All"';
    App.db.setFilterConditions(this.get('name'), [filterForComponent]);
    App.db.setComboSearchQuery(this.get('name'), filterStr);
  },

  /**
   * Filter hosts by stack version and state
   * @param {String} displayName
   * @param {Array} states
   */
  filterByStack: function filterByStack(displayName, states) {
    if (Em.isNone(displayName) || Em.isNone(states) || !states.length) return;
    var colPropAssoc = this.get('colPropAssoc');
    var map = this.get('labelValueMap');
    var stateFilterStrs = [];

    var versionFilter = {
      iColumn: 16,
      value: displayName,
      type: 'string'
    };
    var stateFilter = {
      iColumn: 17,
      value: states,
      type: 'string'
    };
    map["Stack Version"] = colPropAssoc[versionFilter.iColumn];
    map["Version State"] = colPropAssoc[stateFilter.iColumn];
    stateFilter.value.forEach(function (state) {
      map[App.HostStackVersion.formatStatus(state)] = state;
      stateFilterStrs.push('"Version State": "' + App.HostStackVersion.formatStatus(state) + '"');
    });
    var versionFilterStr = '"Stack Version": "' + versionFilter.value + '"';
    App.db.setFilterConditions(this.get('name'), [versionFilter, stateFilter]);
    App.db.setComboSearchQuery(this.get('name'), [versionFilterStr, stateFilterStrs.join(' ')].join(' '));
  },

  goToHostAlerts: function goToHostAlerts(event) {
    var host = event && event.context;
    if (host) {
      App.router.transitionTo('main.hosts.hostDetails.alerts', host);
    }
  },

  /**
   * remove selected hosts
   */
  removeHosts: function removeHosts() {
    var hosts = this.get('content');
    var selectedHosts = hosts.filterProperty('isChecked');
    this.get('fullContent').removeObjects(selectedHosts);
  },

  /**
   * remove hosts with id equal host_id
   * @param {String} host_id
   */
  checkRemoved: function checkRemoved(host_id) {
    var hosts = this.get('content');
    var selectedHosts = hosts.filterProperty('id', host_id);
    this.get('fullContent').removeObjects(selectedHosts);
  },

  /**
   * associations between host property and column index
   * @type {Array}
   */
  colPropAssoc: function () {
    var associations = [];
    associations[0] = 'healthClass';
    associations[1] = 'hostName';
    associations[2] = 'ip';
    associations[3] = 'cpu';
    associations[4] = 'memoryFormatted';
    associations[5] = 'loadAvg';
    associations[6] = 'hostComponents';
    associations[7] = 'criticalWarningAlertsCount';
    associations[8] = 'componentsWithStaleConfigsCount';
    associations[9] = 'componentsInPassiveStateCount';
    associations[10] = 'selected';
    associations[11] = 'hostStackVersion';
    associations[12] = 'rack';
    associations[13] = 'services';
    associations[14] = 'state';
    associations[15] = 'componentState';
    associations[16] = 'version';
    associations[17] = 'versionState';
    return associations;
  }.property()

});

});

require.register("controllers/main/host/addHost/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHostStep4Controller = Em.Controller.extend({

  name: "addHostStep4Controller",
  isConfigGroupLoaded: false,

  loadConfigGroups: function loadConfigGroups() {
    App.ajax.send({
      name: 'config_groups.all_fields',
      sender: this,
      success: 'successLoadingConfigGroup',
      error: 'errorLoadingConfigGroup'
    });
  },

  successLoadingConfigGroup: function successLoadingConfigGroup(data) {
    App.router.get('addHostController.content').set('configGroups', data.items);
    this.set('isConfigGroupLoaded', true);
  },

  errorLoadingConfigGroup: function errorLoadingConfigGroup(data) {
    App.router.get('addHostController.content').set('configGroups', []);
    this.set('isConfigGroupLoaded', true);
  },

  configGroupsLoading: function configGroupsLoading() {
    var df = $.Deferred();
    if (this.get('isConfigGroupLoaded')) {
      df.resolve();
    } else {
      this.interval = setInterval(function () {
        if (this.get('isConfigGroupLoaded')) {
          df.resolve();
          clearInterval(this.interval);
        }
      }.bind(this), 55);
    }
    return df.promise();
  }

});

});

require.register("controllers/main/host/add_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHostController = App.WizardController.extend({

  name: 'addHostController',

  totalSteps: 7,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('hosts.add.header'),

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  /**
   * All wizards data will be stored in this variable
   *
   * cluster - cluster name
   * hosts - hosts, ssh key, repo info, etc.
   * services - services list
   * hostsInfo - list of selected hosts
   * slaveComponentHosts, hostSlaveComponents - info about slave hosts
   * masterComponentHosts - info about master hosts
   * serviceConfigGroups - info about selected config group for service
   * configGroups - all config groups
   * config??? - to be described later
   */
  content: Em.Object.create({
    cluster: null,
    hosts: null,
    installOptions: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: null,
    advancedServiceConfig: null,
    controllerName: 'addHostController',
    serviceConfigGroups: null,
    configGroups: null
  }),

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('hosts');
        this.load('installOptions');
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'sync',
      callback: function callback() {
        this.loadServices();
      }
    }],
    '3': [{
      type: 'async',
      callback: function callback() {
        var self = this,
            dfd = $.Deferred();
        this.loadClients();
        this.loadServices();
        this.loadMasterComponentHosts().done(function () {
          self.loadSlaveComponentHosts();
          self.load('hosts');
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '5': [{
      type: 'sync',
      callback: function callback() {
        this.loadServiceConfigProperties();
        this.getServiceConfigGroups();
      }
    }]
  },

  /**
   * save info about wizard progress, particularly current step of wizard
   * @param currentStep
   * @param completed
   */
  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
    var self = this;
    Em.run.next(function () {
      if (self.isConfigGroupsEmpty()) {
        self.disableStep(4);
      }
    });
  },

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.router.getClusterName() });
  },

  /**
   * Remove host from model. Used at <code>Confirm hosts</code> step
   * @param hosts Array of hosts, which we want to delete
   */
  removeHosts: function removeHosts(hosts) {
    var dbHosts = this.getDBProperty('hosts');
    hosts.forEach(function (_hostInfo) {
      var host = _hostInfo.name;
      delete dbHosts[host];
    });
    this.setDBProperty('hosts', dbHosts);
  },

  disableStep: function disableStep(step) {
    this.get('isStepDisabled').findProperty('step', step).set('value', true);
  },

  isConfigGroupsEmpty: function isConfigGroupsEmpty() {
    return !this.get('content.configGroups') || !this.get('content.configGroups').length;
  },

  /**
   * Load services data. Will be used at <code>Select services(step4)</code> step
   */
  loadServices: function loadServices() {
    var services = this.getDBProperty('services');
    if (!services) {
      services = {
        selectedServices: [],
        installedServices: []
      };
      App.StackService.find().forEach(function (item) {
        var isInstalled = App.Service.find().someProperty('serviceName', item.get('serviceName'));
        item.set('isSelected', isInstalled);
        item.set('isInstalled', isInstalled);
        if (isInstalled) {
          services.selectedServices.push(item.get('serviceName'));
          services.installedServices.push(item.get('serviceName'));
        }
      }, this);
      this.setDBProperty('services', services);
    } else {
      App.StackService.find().forEach(function (item) {
        var isSelected = services.selectedServices.contains(item.get('serviceName'));
        var isInstalled = services.installedServices.contains(item.get('serviceName'));
        item.set('isSelected', isSelected);
        item.set('isInstalled', isInstalled);
      }, this);
    }
    this.set('content.services', App.StackService.find());
  },

  /**
    * Load slave component hosts data for using in required step controllers
    * TODO move to mixin
    */
  loadSlaveComponentHosts: function loadSlaveComponentHosts() {
    var props = this.getDBProperties(['slaveComponentHosts', 'hosts']);
    var slaveComponentHosts = props.slaveComponentHosts || [];
    if (slaveComponentHosts.length) {
      var hosts = props.hosts || {},
          host_names = Em.keys(hosts);
      slaveComponentHosts.forEach(function (component) {
        component.hosts.forEach(function (host) {
          //Em.set(host, 'hostName', hosts[host.host_id].name);
          for (var i = 0; i < host_names.length; i++) {
            if (hosts[host_names[i]].id === host.host_id) {
              host.hostName = host_names[i];
              break;
            }
          }
        });
      });
    }
    this.set("content.slaveComponentHosts", slaveComponentHosts);
  },

  /**
   * Generate clients list for selected services and save it to model
   */
  saveClients: function saveClients() {
    var serviceComponents = App.StackServiceComponent.find();
    var services = this.get('content.services').filterProperty('isInstallable').filterProperty('isSelected');
    var clients = this.getClientsToInstall(services, serviceComponents);
    this.setDBProperty('clientInfo', clients);
    this.set('content.clients', clients);
  },

  /**
   * get list of clients which will be installed on host
   * @param services {Array} of service objects
   * @param components {Array} of component objects
   * @returns {Array} returns array of clients
   * @method getClientsToInstall;
   */
  getClientsToInstall: function getClientsToInstall(services, components) {
    var clients = [];
    services.forEach(function (_service) {
      var serviceClients = components.filter(function (component) {
        return component.get('serviceName') == _service.get('serviceName') && component.get('isClient');
      });
      if (serviceClients.length) {
        serviceClients.forEach(function (client) {
          clients.push({
            component_name: client.get('componentName'),
            display_name: client.get('displayName'),
            isInstalled: false
          });
        });
      }
    }, this);
    return clients;
  },
  /**
   *  Apply config groups from step4 Configurations
   */
  applyConfigGroup: function applyConfigGroup() {
    var serviceConfigGroups = this.get('content.configGroups');
    serviceConfigGroups.forEach(function (group) {
      if (group.configGroups.someProperty('ConfigGroup.group_name', group.selectedConfigGroup)) {
        var configGroup = group.configGroups.findProperty('ConfigGroup.group_name', group.selectedConfigGroup);
        group.hosts.forEach(function (host) {
          configGroup.ConfigGroup.hosts.push({
            host_name: host
          });
        }, this);
        delete configGroup.href;
        App.ajax.send({
          name: 'config_groups.update_config_group',
          sender: this,
          data: {
            id: configGroup.ConfigGroup.id,
            configGroup: configGroup
          }
        });
      }
    }, this);
  },

  /**
   * Load information about selected config groups
   */
  getServiceConfigGroups: function getServiceConfigGroups() {
    var serviceConfigGroups = this.getDBProperty('serviceConfigGroups');
    this.set('content.configGroups', serviceConfigGroups);
  },

  /**
   * Save information about selected config groups
   */
  saveServiceConfigGroups: function saveServiceConfigGroups() {
    this.setDBProperty('serviceConfigGroups', this.get('content.configGroups'));
  },

  /**
   * Set content.configGroups for step4
   */
  loadServiceConfigGroups: function loadServiceConfigGroups() {
    var selectedServices = [];
    this.loadServiceConfigGroupsBySlaves(selectedServices);
    this.loadServiceConfigGroupsByClients(selectedServices);
    this.sortServiceConfigGroups(selectedServices);
    this.set('content.configGroups', selectedServices);
  },
  /**
   * sort config groups by name
   * @param selectedServices
   */
  sortServiceConfigGroups: function sortServiceConfigGroups(selectedServices) {
    selectedServices.forEach(function (selectedService) {
      selectedService.configGroups.sort(function (cfgA, cfgB) {
        if (cfgA.ConfigGroup.group_name < cfgB.ConfigGroup.group_name) return -1;
        if (cfgA.ConfigGroup.group_name > cfgB.ConfigGroup.group_name) return 1;
        return 0;
      });
    });
  },
  /**
   * load service config groups by slave components,
   * push them into selectedServices
   * @param selectedServices
   */
  loadServiceConfigGroupsBySlaves: function loadServiceConfigGroupsBySlaves(selectedServices) {
    var slaveComponentHosts = this.get('content.slaveComponentHosts');
    if (slaveComponentHosts && slaveComponentHosts.length > 0) {
      slaveComponentHosts.forEach(function (slave) {
        if (slave.hosts.length > 0) {
          if (slave.componentName !== "CLIENT") {
            var service = App.StackServiceComponent.find(slave.componentName).get('stackService');
            var serviceName = service.get('serviceName');
            var configGroups = this.get('content.configGroups').filterProperty('ConfigGroup.tag', serviceName);
            var configGroupsNames = configGroups.mapProperty('ConfigGroup.group_name');
            var defaultGroupName = 'Default';
            var selectedService = selectedServices.findProperty('serviceId', serviceName);
            configGroupsNames.unshift(defaultGroupName);
            if (selectedService) {
              Em.set(selectedService, 'hosts', Em.getWithDefault(selectedService, 'hosts', []).concat(slave.hosts.mapProperty('hostName')).uniq());
            } else {
              selectedServices.push({
                serviceId: serviceName,
                displayName: service.get('displayName'),
                hosts: slave.hosts.mapProperty('hostName'),
                configGroupsNames: configGroupsNames,
                configGroups: configGroups,
                selectedConfigGroup: defaultGroupName
              });
            }
          }
        }
      }, this);
      return true;
    }
    return false;
  },
  /**
   * load service config groups by clients,
   * push them into selectedServices
   * @param selectedServices
   */
  loadServiceConfigGroupsByClients: function loadServiceConfigGroupsByClients(selectedServices) {
    var slaveComponentHosts = this.get('content.slaveComponentHosts');
    var clients = this.get('content.clients');
    var client = slaveComponentHosts && slaveComponentHosts.findProperty('componentName', 'CLIENT');
    var selectedClientHosts = client && client.hosts.mapProperty('hostName');
    if (clients && selectedClientHosts && clients.length > 0 && selectedClientHosts.length > 0) {
      this.loadClients();
      clients.forEach(function (client) {
        var service = App.StackServiceComponent.find(client.component_name).get('stackService');
        var serviceName = service.get('serviceName');
        var serviceMatch = selectedServices.findProperty('serviceId', serviceName);
        if (serviceMatch) {
          serviceMatch.hosts = serviceMatch.hosts.concat(selectedClientHosts).uniq();
        } else {
          var configGroups = this.get('content.configGroups').filterProperty('ConfigGroup.tag', serviceName);
          var configGroupsNames = configGroups.mapProperty('ConfigGroup.group_name').sort();
          var defaultGroupName = 'Default';
          configGroupsNames.unshift(defaultGroupName);
          selectedServices.push({
            serviceId: serviceName,
            displayName: service.get('displayName'),
            hosts: selectedClientHosts,
            configGroupsNames: configGroupsNames,
            configGroups: configGroups,
            selectedConfigGroup: defaultGroupName
          });
        }
      }, this);
      return true;
    }
    return false;
  },

  loadServiceConfigProperties: function loadServiceConfigProperties() {
    var serviceConfigProperties = App.db.get('AddService', 'serviceConfigProperties');
    if (!serviceConfigProperties || !serviceConfigProperties.length) {
      serviceConfigProperties = App.db.get('Installer', 'serviceConfigProperties');
    }
    this.set('content.serviceConfigProperties', serviceConfigProperties);
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearStorageData: function clearStorageData() {
    this._super();
    this.resetDbNamespace();
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.clearAllSteps();
    this.clearStorageData();
  },

  /**
   * send request to server in order to install services
   * @param isRetry
   * @param callback
   * @param errorCallback
   */
  installServices: function installServices(isRetry, callback, errorCallback) {
    callback = callback || Em.K;
    this.set('content.cluster.oldRequestsId', []);
    this.set('content.cluster.status', 'PENDING');
    var clusterName = this.get('content.cluster.name');
    var hostNames = [];
    var hosts = this.getDBProperty('hosts');
    for (var hostname in hosts) {
      if (!hosts[hostname].isInstalled) {
        hostNames.push(hostname);
      }
    }
    if (!clusterName || hostNames.length === 0) return false;

    App.ajax.send({
      name: "common.host_components.update",
      sender: this,
      data: {
        "context": Em.I18n.t('requestInfo.installComponents'),
        "query": "HostRoles/host_name.in(" + hostNames.join(',') + ")",
        "HostRoles": { "state": "INSTALLED" },
        "level": "HOST_COMPONENT"
      },
      success: 'installServicesSuccessCallback',
      error: 'installServicesErrorCallback'
    }).then(callback, errorCallback || callback);
    return true;
  }
});

});

require.register("controllers/main/host/bulk_operations_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var hostsManagement = require('utils/hosts');
var O = Em.Object;

/**
 * @class BulkOperationsController
 */
App.BulkOperationsController = Em.Controller.extend({

  name: 'bulkOperationsController',

  /**
   * Bulk operation wrapper
   * @param {Object} operationData - data about bulk operation (action, hosts or hostComponents etc)
   * @param {Array} hosts - list of affected hosts
   * @method bulkOperation
   */
  bulkOperation: function bulkOperation(operationData, hosts) {
    if (operationData.componentNameFormatted) {
      if (operationData.action === 'RESTART') {
        this.bulkOperationForHostComponentsRestart(operationData, hosts);
      } else if (operationData.action === 'ADD') {
        this.bulkOperationForHostComponentsAdd(operationData, hosts);
      } else if (operationData.action === 'DELETE') {
        this.bulkOperationForHostComponentsDelete(operationData, hosts);
      } else {
        if (operationData.action.indexOf('DECOMMISSION') == -1) {
          this.bulkOperationForHostComponents(operationData, hosts);
        } else {
          this.bulkOperationForHostComponentsDecommission(operationData, hosts);
        }
      }
    } else {
      if (operationData.action === 'SET_RACK_INFO') {
        this.bulkOperationForHostsSetRackInfo(operationData, hosts);
      } else {
        if (operationData.action === 'RESTART') {
          this.bulkOperationForHostsRestart(operationData, hosts);
        } else if (operationData.action === 'REINSTALL') {
          this.bulkOperationForHostsReinstall(operationData, hosts);
        } else if (operationData.action === 'DELETE') {
          this._bulkOperationForHostsDelete(hosts);
        } else if (operationData.action === 'CONFIGURE') {
          this.bulkOperationForHostsRefreshConfig(operationData, hosts);
        } else {
          if (operationData.action === 'PASSIVE_STATE') {
            this.bulkOperationForHostsPassiveState(operationData, hosts);
          } else {
            this.bulkOperationForHosts(operationData, hosts);
          }
        }
      }
    }
  },

  /**
   * Bulk operation (start/stop all) for selected hosts
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Array} hosts - list of affected hosts
   * @return {$.ajax}
   */
  bulkOperationForHosts: function bulkOperationForHosts(operationData, hosts) {
    var self = this;
    return batchUtils.getComponentsFromServer({
      hosts: hosts.mapProperty('hostName'),
      passiveState: 'OFF',
      displayParams: ['host_components/HostRoles/component_name']
    }, function (data) {
      return self._getComponentsFromServerForHostsCallback(operationData, data);
    });
  },

  /**
   * run Bulk operation (start/stop all) for selected hosts
   * after host and components are loaded
   * @param operationData
   * @param data
   */
  _getComponentsFromServerForHostsCallback: function _getComponentsFromServerForHostsCallback(operationData, data) {
    var query = [];
    var hostNames = [];
    var hostsMap = {};
    var clients = App.components.get('clients');

    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        if (!clients.contains(hostComponent.HostRoles.component_name)) {
          if (hostsMap[host.Hosts.host_name]) {
            hostsMap[host.Hosts.host_name].push(hostComponent.HostRoles.component_name);
          } else {
            hostsMap[host.Hosts.host_name] = [hostComponent.HostRoles.component_name];
          }
        }
      });
    });

    var nn_hosts = [];
    for (var hostName in hostsMap) {
      if (hostsMap.hasOwnProperty(hostName)) {
        var subQuery = '(HostRoles/component_name.in(%@)&HostRoles/host_name=' + hostName + ')';
        var components = hostsMap[hostName];

        if (components.length) {
          if (components.contains('NAMENODE')) {
            nn_hosts.push(hostName);
          }
          query.push(subQuery.fmt(components.join(',')));
        }
        hostNames.push(hostName);
      }
    }
    hostNames = hostNames.join(",");
    if (query.length) {
      query = query.join('|');
      var self = this;
      // if NameNode included, check HDFS NameNode checkpoint before stop NN
      var isHDFSStarted = 'STARTED' === App.Service.find('HDFS').get('workStatus');

      var request = function request() {
        return App.ajax.send({
          name: 'common.host_components.update',
          sender: self,
          data: {
            query: query,
            HostRoles: {
              state: operationData.action
            },
            context: operationData.message,
            hostName: hostNames,
            noOpsMessage: Em.I18n.t('hosts.host.maintainance.allComponents.context')
          },
          success: 'bulkOperationForHostComponentsSuccessCallback'
        });
      };

      if (operationData.action === 'INSTALLED' && isHDFSStarted) {
        if (nn_hosts.length == 1) {
          return App.router.get('mainHostDetailsController').checkNnLastCheckpointTime(request, nn_hosts[0]);
        }
        if (nn_hosts.length > 1) {
          // HA or federation enabled
          return App.router.get('mainServiceItemController').checkNnLastCheckpointTime(request);
        }
      }
      return request();
    } else {
      return App.ModalPopup.show({
        header: Em.I18n.t('rolling.nothingToDo.header'),
        body: Em.I18n.t('rolling.nothingToDo.body').format(Em.I18n.t('hosts.host.maintainance.allComponents.context')),
        secondary: false
      });
    }
  },

  bulkOperationForHostsSetRackInfo: function bulkOperationForHostsSetRackInfo(operationData, hosts) {
    return hostsManagement.setRackInfo(operationData, hosts);
  },

  /**
   * Bulk restart for selected hosts
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Ember.Enumerable} hosts - list of affected hosts
   */
  bulkOperationForHostsRestart: function bulkOperationForHostsRestart(operationData, hosts) {
    return batchUtils.getComponentsFromServer({
      passiveState: 'OFF',
      hosts: hosts.mapProperty('hostName'),
      displayParams: ['host_components/HostRoles/component_name']
    }, this._getComponentsFromServerForRestartCallback);
  },

  /**
   *
   * @param {object} data
   * @private
   * @method _getComponentsFromServerCallback
   */
  _getComponentsFromServerForRestartCallback: function _getComponentsFromServerForRestartCallback(data) {
    var hostComponents = [];
    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        hostComponents.push(O.create({
          componentName: hostComponent.HostRoles.component_name,
          hostName: host.Hosts.host_name
        }));
      });
    });
    // if NameNode included, check HDFS NameNode checkpoint before restart NN
    var isHDFSStarted = 'STARTED' === App.Service.find('HDFS').get('workStatus');
    var namenodes = hostComponents.filterProperty('componentName', 'NAMENODE');
    var nn_count = namenodes.get('length');

    if (nn_count == 1 && isHDFSStarted) {
      var hostName = namenodes.get('firstObject.hostName');
      App.router.get('mainHostDetailsController').checkNnLastCheckpointTime(function () {
        batchUtils.restartHostComponents(hostComponents, Em.I18n.t('rollingrestart.context.allOnSelectedHosts'), "HOST");
      }, hostName);
    } else {
      if (nn_count > 1 && isHDFSStarted) {
        // HA or federation enabled
        App.router.get('mainServiceItemController').checkNnLastCheckpointTime(function () {
          batchUtils.restartHostComponents(hostComponents, Em.I18n.t('rollingrestart.context.allOnSelectedHosts'), "HOST");
        });
      } else {
        batchUtils.restartHostComponents(hostComponents, Em.I18n.t('rollingrestart.context.allOnSelectedHosts'), "HOST");
      }
    }
  },

  /**
   * Bulk reinstall failed components for selected hosts
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Ember.Enumerable} hosts - list of affected hosts
   */
  bulkOperationForHostsReinstall: function bulkOperationForHostsReinstall(operationData, hosts) {
    var self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      return App.ajax.send({
        name: 'common.host_components.update',
        sender: self,
        data: {
          HostRoles: {
            state: 'INSTALLED'
          },
          query: 'HostRoles/host_name.in(' + hosts.mapProperty('hostName').join(',') + ')&HostRoles/state=INSTALL_FAILED',
          context: operationData.message,
          noOpsMessage: Em.I18n.t('hosts.host.maintainance.reinstallFailedComponents.context')
        },
        success: 'bulkOperationForHostComponentsSuccessCallback',
        showLoadingPopup: true
      });
    });
  },

  /**
  * Check which hosts can be deleted and warn the user about it in advance
  * @param {Ember.Enumerable} hosts - list of affected hosts
  */
  _bulkOperationForHostsDelete: function _bulkOperationForHostsDelete(hosts) {
    var self = this,
        hostNamesToDelete = [],
        hostsNotToDelete = [];
    var createNonDeletableComponents = function createNonDeletableComponents(hostName, message) {
      return Em.Object.create({
        error: {
          key: hostName,
          message: message
        },
        isCollapsed: true,
        isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
      });
    };
    hosts.forEach(function (host) {
      var hostComponents = App.HostComponent.find().filterProperty('hostName', host.hostName);
      var hostInfo = App.router.get('mainHostDetailsController').getHostComponentsInfo(hostComponents);
      console.dir(hostInfo);
      if (hostInfo.nonDeletableComponents.length > 0) {
        hostsNotToDelete.push(createNonDeletableComponents(host.hostName, Em.I18n.t('hosts.bulkOperation.deleteHosts.nonDeletableComponents').format(hostInfo.nonDeletableComponents.join(", "))));
      } else if (hostInfo.nonAddableMasterComponents.length > 0) {
        hostsNotToDelete.push(createNonDeletableComponents(host.hostName, Em.I18n.t('hosts.bulkOperation.deleteHosts.nonAddableMasterComponents').format(hostInfo.nonAddableMasterComponents.join(", "))));
      } else if (hostInfo.lastMasterComponents.length > 0) {
        hostsNotToDelete.push(createNonDeletableComponents(host.hostName, Em.I18n.t('hosts.bulkOperation.deleteHosts.lastMasterComponents').format(hostInfo.lastMasterComponents.join(", "))));
      } else if (hostInfo.runningComponents.length > 0) {
        hostsNotToDelete.push(createNonDeletableComponents(host.hostName, Em.I18n.t('hosts.bulkOperation.deleteHosts.runningComponents').format(hostInfo.runningComponents.join(", "))));
      } else {
        hostNamesToDelete.push(host.hostName);
      }
    });

    return App.ModalPopup.show({
      header: hostNamesToDelete.length ? Em.I18n.t('hosts.bulkOperation.deleteHosts.confirm.header') : Em.I18n.t('rolling.nothingToDo.header'),
      primary: hostNamesToDelete.length ? Em.I18n.t('common.next') : null,
      primaryClass: 'btn-default',
      onPrimary: function onPrimary() {
        this._super();
        self.bulkOperationForHostsDelete(hostNamesToDelete);
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/bulk_add_delete_confirm_popup'),
        modifyMessage: Em.I18n.t('hosts.bulkOperation.deleteHosts.confirm.delete'),
        skipMessage: hostNamesToDelete.length ? Em.I18n.t('hosts.bulkOperation.deleteHosts.cannot.delete1') : Em.I18n.t('hosts.bulkOperation.deleteHosts.cannot.delete2'),
        skippedHosts: hostsNotToDelete.length ? hostsNotToDelete : null,
        hostsToModify: hostNamesToDelete.length ? hostNamesToDelete.join("\n") : null,
        onToggleHost: function onToggleHost(host) {
          host.contexts[0].toggleProperty('isCollapsed');
        }
      })
    });
  },

  /**
   * Bulk refresh configs for selected hosts
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Ember.Enumerable} hosts - list of affected/selected hosts
   */
  bulkOperationForHostsRefreshConfig: function bulkOperationForHostsRefreshConfig(operationData, hosts) {
    return batchUtils.getComponentsFromServer({
      passiveState: 'OFF',
      hosts: hosts.mapProperty('hostName'),
      displayParams: ['host_components/HostRoles/component_name']
    }, this._getComponentsFromServerForRefreshConfigsCallback);
  },

  /**
   *
   * @param {object} data
   * @private
   * @method _getComponentsFromServerForRefreshConfigsCallback
   */
  _getComponentsFromServerForRefreshConfigsCallback: function _getComponentsFromServerForRefreshConfigsCallback(data) {
    var hostComponents = [];
    var clients = App.components.get('clients');
    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        if (clients.contains(hostComponent.HostRoles.component_name)) {
          hostComponents.push(O.create({
            componentName: hostComponent.HostRoles.component_name,
            hostName: host.Hosts.host_name
          }));
        }
      });
    });
    batchUtils.restartHostComponents(hostComponents, Em.I18n.t('rollingrestart.context.configs.allOnSelectedHosts'), "HOST");
  },

  /**
   * Bulk delete selected hosts
   * @param {String} hosts - list of affected host names
   */
  bulkOperationForHostsDelete: function bulkOperationForHostsDelete(hosts) {
    var confirmKey = 'delete',
        self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      return App.ModalPopup.show({
        header: Em.I18n.t('hosts.bulkOperation.deleteHosts.confirmation.header'),
        confirmInput: '',
        disablePrimary: Em.computed.notEqual('confirmInput', confirmKey),
        primary: Em.I18n.t('common.confirm'),
        primaryClass: 'btn-warning',
        onPrimary: function onPrimary() {
          this._super();
          return App.ajax.send({
            name: 'common.hosts.delete',
            sender: self,
            data: {
              query: 'Hosts/host_name.in(' + hosts.join(',') + ')',
              hosts: hosts
            },
            success: 'bulkOperationForHostsDeleteCallback',
            error: 'bulkOperationForHostsDeleteCallback',
            showLoadingPopup: true
          });
        },
        bodyClass: Em.View.extend({
          templateName: require('templates/main/host/delete_hosts_popup'),
          hostNames: hosts,
          typeMessage: Em.I18n.t('services.service.confirmDelete.popup.body.type').format(confirmKey)
        })
      });
    });
  },

  /**
   * Show popup after bulk delete hosts
   * @method bulkOperationForHostsDeleteCallback
   */
  bulkOperationForHostsDeleteCallback: function bulkOperationForHostsDeleteCallback(arg0, arg1, arg2, arg3, arg4) {
    var deletedHosts = [];
    var undeletableHosts = [];
    if (arg1 == "error") {
      var request = arg0;
      var params = arg4;
      var response = JSON.parse(request.responseText);
      var host = Ember.Object.create({
        error: {
          key: params.hosts[0],
          code: response.status,
          message: response.message
        },
        isCollapsed: true,
        isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
      });
      undeletableHosts.push(host);
    } else {
      var data = arg0;
      var params = arg2;
      if (data) {
        data.deleteResult.forEach(function (host) {
          if (!host.deleted) {
            var _host = Ember.Object.create({
              error: host.error,
              isCollapsed: true,
              isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
            });
            undeletableHosts.push(_host);
          }
        });
      }
      deletedHosts = params.hosts;
    }

    return App.ModalPopup.show({
      header: Em.I18n.t('hosts.bulkOperation.deleteHosts.result.header'),

      secondary: null,

      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/delete_hosts_result_popup'),
        message: Em.I18n.t('hosts.bulkOperation.deleteHosts.dryRun.message').format(undeletableHosts.length),
        undeletableHosts: undeletableHosts,
        deletedHosts: deletedHosts,
        onToggleHost: function onToggleHost(host) {
          host.contexts[0].toggleProperty('isCollapsed');
        }
      }),

      completeDelete: function completeDelete() {
        if (arg1 !== 'error') {
          App.db.unselectHosts(arg2.hosts);
        }
        location.reload();
      },


      onPrimary: function onPrimary() {
        this.completeDelete();
        this._super();
      },

      onClose: function onClose() {
        this.completeDelete();
        this._super();
      }
    });
  },

  /**
   * Bulk turn on/off passive state for selected hosts
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Array} hosts - list of affected hosts
   */
  bulkOperationForHostsPassiveState: function bulkOperationForHostsPassiveState(operationData, hosts) {
    var self = this;

    return batchUtils.getComponentsFromServer({
      hosts: hosts.mapProperty('hostName'),
      displayParams: ['Hosts/maintenance_state']
    }, function (data) {
      return self._getComponentsFromServerForPassiveStateCallback(operationData, data);
    });
  },

  /**
   *
   * @param {object} operationData
   * @param {object} data
   * @returns {$.ajax|App.ModalPopup}
   * @private
   * @method _getComponentsFromServerForPassiveStateCallback
   */
  _getComponentsFromServerForPassiveStateCallback: function _getComponentsFromServerForPassiveStateCallback(operationData, data) {
    var hostNames = [];

    data.items.forEach(function (host) {
      if (host.Hosts.maintenance_state !== operationData.state) {
        hostNames.push(host.Hosts.host_name);
      }
    });
    if (hostNames.length) {
      return App.ajax.send({
        name: 'bulk_request.hosts.passive_state',
        sender: this,
        data: {
          hostNames: hostNames.join(','),
          passive_state: operationData.state,
          requestInfo: operationData.message
        },
        success: 'updateHostPassiveState'
      });
    }
    return App.ModalPopup.show({
      header: Em.I18n.t('rolling.nothingToDo.header'),
      body: Em.I18n.t('hosts.bulkOperation.passiveState.nothingToDo.body'),
      secondary: false
    });
  },

  updateHostPassiveState: function updateHostPassiveState(data, opt, params) {
    return batchUtils.infoPassiveState(params.passive_state);
  },

  /**
   * bulk add for selected hostComponent
   * @param {Object} operationData - data about bulk operation (action, hostComponent etc)
   * @param {Array} hosts - list of affected hosts
   */
  bulkOperationForHostComponentsAdd: function bulkOperationForHostComponentsAdd(operationData, hosts) {
    var self = this;
    return batchUtils.getComponentsFromServer({
      components: [operationData.componentName],
      hosts: hosts.mapProperty('hostName')
    }, function (data) {
      return self._getComponentsFromServerForHostComponentsAddCallback(operationData, data, hosts);
    });
  },

  _getComponentsFromServerForHostComponentsAddCallback: function _getComponentsFromServerForHostComponentsAddCallback(operationData, data, hosts) {
    var self = this;

    var allHostsWithComponent = data.items.mapProperty('Hosts.host_name');
    var hostsWithComponent = [];
    hosts.forEach(function (host) {
      var isNotHeartBeating = host.state === 'HEARTBEAT_LOST';
      if (allHostsWithComponent.contains(host.hostName) || isNotHeartBeating) {
        hostsWithComponent.push(Em.Object.create({
          error: {
            key: host.hostName,
            message: isNotHeartBeating ? Em.I18n.t('hosts.bulkOperation.confirmation.add.component.noHeartBeat.skip') : Em.I18n.t('hosts.bulkOperation.confirmation.add.component.skip').format(operationData.componentNameFormatted)
          },
          isCollapsed: true,
          isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
        }));
      }
    });
    var hostsWithOutComponent = hosts.filter(function (host) {
      return !hostsWithComponent.findProperty('error.key', host.hostName);
    });

    hostsWithOutComponent = hostsWithOutComponent.mapProperty('hostName');

    return App.ModalPopup.show({
      header: hostsWithOutComponent.length ? Em.I18n.t('hosts.bulkOperation.confirmation.header') : Em.I18n.t('rolling.nothingToDo.header'),
      primary: hostsWithOutComponent.length ? Em.I18n.t('hosts.host.addComponent.popup.confirm') : null,

      onPrimary: function onPrimary() {
        self.bulkAddHostComponents(operationData, hostsWithOutComponent);
        this._super();
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/bulk_add_delete_confirm_popup'),
        modifyMessage: Em.I18n.t('hosts.bulkOperation.confirmation.add.component').format(operationData.componentNameFormatted),
        skipMessage: hostsWithOutComponent.length ? Em.I18n.t('hosts.bulkOperation.confirmation.cannot.add1') : Em.I18n.t('hosts.bulkOperation.confirmation.cannot.add2').format(operationData.componentNameFormatted),
        hostsToModify: hostsWithOutComponent.length ? hostsWithOutComponent.join("\n") : null,
        skippedHosts: hostsWithComponent.length ? hostsWithComponent : null,
        onToggleHost: function onToggleHost(host) {
          host.contexts[0].toggleProperty('isCollapsed');
        }
      })
    });
  },
  /**
   * Bulk add for selected hostComponent
   * @param {Object} operationData - data about bulk operation (action, hostComponent etc)
   * @param {Array} hostNames - list of affected hosts' names
   */
  bulkAddHostComponents: function bulkAddHostComponents(operationData, hostNames) {
    var self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      App.ajax.send({
        name: 'host.host_component.add_new_components',
        sender: self,
        data: {
          data: JSON.stringify({
            RequestInfo: {
              query: 'Hosts/host_name.in(' + hostNames.join(',') + ')'
            },
            Body: {
              host_components: [{
                HostRoles: {
                  component_name: operationData.componentName
                }
              }]
            }
          }),
          context: operationData.message + ' ' + operationData.componentNameFormatted
        },
        success: 'bulkOperationForHostComponentsAddSuccessCallback',
        showLoadingPopup: true
      });
    });
  },

  bulkOperationForHostComponentsAddSuccessCallback: function bulkOperationForHostComponentsAddSuccessCallback(data, opt, params) {
    App.ajax.send({
      name: 'common.host_components.update',
      sender: this,
      data: {
        query: 'HostRoles/state=INIT',
        HostRoles: {
          state: 'INSTALLED'
        },
        context: params.context
      },
      success: 'bulkOperationForHostComponentsSuccessCallback'
    });
  },

  /**
   * Confirm bulk delete for selected hostComponent
   * @param {Object} operationData - data about bulk operation (action, hostComponent etc)
   * @param {Array} hosts - list of affected hosts
   */
  bulkOperationForHostComponentsDelete: function bulkOperationForHostComponentsDelete(operationData, hosts) {
    var self = this;
    return batchUtils.getComponentsFromServer({
      components: [operationData.componentName],
      hosts: hosts.mapProperty('hostName'),
      displayParams: ['host_components/HostRoles/state']
    }, function (data) {
      return self._getComponentsFromServerForHostComponentsDeleteCallback(operationData, data, hosts);
    });
  },

  _getComponentsFromServerForHostComponentsDeleteCallback: function _getComponentsFromServerForHostComponentsDeleteCallback(operationData, data, requestedHosts) {
    var self = this;
    var minToInstall = App.StackServiceComponent.find(operationData.componentName).get('minToInstall');
    var installedCount = App.HostComponent.getCount(operationData.componentName, 'totalCount');
    var installedHosts = data.items.mapProperty('Hosts.host_name');
    var hostsToDelete = data.items.filter(function (host) {
      var state = host.host_components[0].HostRoles.state;
      return [App.HostComponentStatus.stopped, App.HostComponentStatus.unknown, App.HostComponentStatus.install_failed, App.HostComponentStatus.upgrade_failed, App.HostComponentStatus.init].contains(state);
    }).mapProperty('Hosts.host_name');

    if (installedCount - hostsToDelete.length < minToInstall) {
      return App.ModalPopup.show({
        header: Em.I18n.t('rolling.nothingToDo.header'),
        body: Em.I18n.t('hosts.bulkOperation.confirmation.delete.component.minimum.body').format(minToInstall, operationData.componentNameFormatted),
        secondary: false
      });
    }

    var hostsNotToDelete = [];

    requestedHosts.mapProperty('hostName').forEach(function (host) {
      if (!hostsToDelete.contains(host)) {
        var hostToSkip = Em.Object.create({
          error: {
            key: host,
            message: null
          },
          isCollapsed: true,
          isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
        });
        if (installedHosts.contains(host)) {
          hostToSkip.error.message = Em.I18n.t('hosts.bulkOperation.confirmation.delete.component.notStopped').format(operationData.componentNameFormatted);
        } else {
          hostToSkip.error.message = Em.I18n.t('hosts.bulkOperation.confirmation.delete.component.notInstalled').format(operationData.componentNameFormatted);
        }
        hostsNotToDelete.push(hostToSkip);
      }
    });

    return App.ModalPopup.show({
      header: hostsToDelete.length ? Em.I18n.t('hosts.bulkOperation.confirmation.header') : Em.I18n.t('rolling.nothingToDo.header'),
      primary: hostsToDelete.length ? Em.I18n.t('hosts.host.deleteComponent.popup.confirm') : null,
      primaryClass: 'btn-warning',

      onPrimary: function onPrimary() {
        self.bulkDeleteHostComponents(operationData, hostsToDelete);
        this._super();
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/bulk_add_delete_confirm_popup'),
        modifyMessage: Em.I18n.t('hosts.bulkOperation.confirmation.delete.component').format(operationData.componentNameFormatted),
        skipMessage: hostsToDelete.length ? Em.I18n.t('hosts.bulkOperation.confirmation.delete.component.cannot1') : Em.I18n.t('hosts.bulkOperation.confirmation.delete.component.cannot2').format(operationData.componentNameFormatted),
        hostsToModify: hostsToDelete.length ? hostsToDelete.join("\n") : null,
        skippedHosts: hostsNotToDelete.length ? hostsNotToDelete : null,
        onToggleHost: function onToggleHost(host) {
          host.contexts[0].toggleProperty('isCollapsed');
        }
      })
    });
  },

  /**
   * Bulk delete for selected hostComponent
   * @param {Object} operationData - data about bulk operation (action, hostComponent etc)
   * @param {Array} hostNames - list of affected hosts' names
   */
  bulkDeleteHostComponents: function bulkDeleteHostComponents(operationData, hostNames) {
    var self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      App.ajax.send({
        name: 'host.host_component.delete_components',
        sender: self,
        data: {
          hostNames: hostNames,
          componentName: operationData.componentName,
          data: JSON.stringify({
            RequestInfo: {
              query: 'HostRoles/host_name.in(' + hostNames.join(',') + ')&HostRoles/component_name.in(' + operationData.componentName + ')'
            }
          })
        },
        success: 'bulkOperationForHostComponentsDeleteCallback'
      });
    });
  },

  /**
   * Show popup after bulk delete host_components
   * @method bulkOperationForHostComponentsDeleteCallback
   */
  bulkOperationForHostComponentsDeleteCallback: function bulkOperationForHostComponentsDeleteCallback(arg0, arg1, arg2, arg3, arg4) {
    var deletedHosts = [];
    var undeletableHosts = [];
    var componentName = arg2.componentName;
    if (arg1 == "error") {
      var request = arg0;
      var params = arg4;
      var response = JSON.parse(request.responseText);
      var host = Ember.Object.create({
        error: {
          key: params.hosts[0],
          code: response.status,
          message: response.message
        },
        isCollapsed: true,
        isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
      });
      undeletableHosts.push(host);
    } else {
      var data = arg0;
      var _params = arg2;
      if (data) {
        data.deleteResult.forEach(function (host) {
          if (!host.deleted) {
            var _host = Ember.Object.create({
              error: host.error,
              isCollapsed: true,
              isBodyVisible: Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;')
            });
            undeletableHosts.push(_host);
          }
        });
      }
      deletedHosts = _params.hostNames;
    }

    return App.ModalPopup.show({
      header: Em.I18n.t('hosts.bulkOperation.delete.component.result.header'),

      secondary: null,

      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/delete_hosts_result_popup'),
        message: Em.I18n.t('hosts.bulkOperation.delete.component.dryRun.message').format(componentName),
        componentName: componentName,
        undeletableHosts: undeletableHosts,
        deletedHosts: deletedHosts,
        deleteComponents: true,
        onToggleHost: function onToggleHost(host) {
          host.contexts[0].toggleProperty('isCollapsed');
        }
      }),

      onPrimary: function onPrimary() {
        location.reload();
        this._super();
      },

      onClose: function onClose() {
        location.reload();
        this._super();
      }
    });
  },

  /**
   * Bulk operation for selected hostComponents
   * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
   * @param {Array} hosts - list of affected hosts
   */
  bulkOperationForHostComponents: function bulkOperationForHostComponents(operationData, hosts) {
    var self = this;

    return batchUtils.getComponentsFromServer({
      components: [operationData.componentName],
      hosts: hosts.mapProperty('hostName'),
      passiveState: 'OFF'
    }, function (data) {
      return self._getComponentsFromServerForHostComponentsCallback(operationData, data);
    });
  },

  /**
   *
   * @param {object} operationData
   * @param {object} data
   * @returns {$.ajax|App.ModalPopup}
   * @private
   */
  _getComponentsFromServerForHostComponentsCallback: function _getComponentsFromServerForHostComponentsCallback(operationData, data) {
    if (data.items) {
      var hostsWithComponentInProperState = data.items.mapProperty('Hosts.host_name');
      return App.ajax.send({
        name: 'common.host_components.update',
        sender: this,
        data: {
          HostRoles: {
            state: operationData.action
          },
          query: 'HostRoles/component_name=' + operationData.componentName + '&HostRoles/host_name.in(' + hostsWithComponentInProperState.join(',') + ')&HostRoles/maintenance_state=OFF',
          context: operationData.message + ' ' + operationData.componentNameFormatted,
          level: 'SERVICE',
          noOpsMessage: operationData.componentNameFormatted
        },
        success: 'bulkOperationForHostComponentsSuccessCallback'
      });
    }
    return App.ModalPopup.show({
      header: Em.I18n.t('rolling.nothingToDo.header'),
      body: Em.I18n.t('rolling.nothingToDo.body').format(operationData.componentNameFormatted),
      secondary: false
    });
  },

  /**
   * Bulk decommission/recommission for selected hostComponents
   * @param {Object} operationData
   * @param {Array} hosts
   */
  bulkOperationForHostComponentsDecommission: function bulkOperationForHostComponentsDecommission(operationData, hosts) {
    var self = this;

    return batchUtils.getComponentsFromServer({
      components: [operationData.realComponentName],
      hosts: hosts.mapProperty('hostName'),
      passiveState: 'OFF',
      displayParams: ['host_components/HostRoles/state']
    }, function (data) {
      return self._getComponentsFromServerForHostComponentsDecommissionCallBack(operationData, data);
    });
  },

  /**
   * run Bulk decommission/recommission for selected hostComponents
   * after host and components are loaded
   * @param operationData
   * @param data
   * @method _getComponentsFromServerForHostComponentsDecommissionCallBack
   */
  _getComponentsFromServerForHostComponentsDecommissionCallBack: function _getComponentsFromServerForHostComponentsDecommissionCallBack(operationData, data) {
    var service = App.Service.find(operationData.serviceName);
    var components = [];

    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        components.push(O.create({
          componentName: hostComponent.HostRoles.component_name,
          hostName: host.Hosts.host_name,
          workStatus: hostComponent.HostRoles.state
        }));
      });
    });

    if (components.length) {
      var hostsWithComponentInProperState = components.mapProperty('hostName');
      var turn_off = operationData.action.indexOf('OFF') !== -1;
      var svcName = operationData.serviceName;
      var masterName = operationData.componentName;
      var slaveName = operationData.realComponentName;
      var hostNames = hostsWithComponentInProperState.join(',');
      if (turn_off) {
        // For recommession
        if (svcName === "YARN" || svcName === "HBASE" || svcName === "HDFS") {
          App.router.get('mainHostDetailsController').doRecommissionAndStart(hostNames, svcName, masterName, slaveName);
        }
      } else {
        hostsWithComponentInProperState = components.filterProperty('workStatus', 'STARTED').mapProperty('hostName');
        //For decommession
        if (svcName == "HBASE") {
          // HBASE service, decommission RegionServer in batch requests
          this.warnBeforeDecommission(hostNames);
        } else {
          var parameters = {
            "slave_type": slaveName
          };
          var contextString = turn_off ? 'hosts.host.' + slaveName.toLowerCase() + '.recommission' : 'hosts.host.' + slaveName.toLowerCase() + '.decommission';
          if (turn_off) {
            parameters['included_hosts'] = hostsWithComponentInProperState.join(',');
          } else {
            parameters['excluded_hosts'] = hostsWithComponentInProperState.join(',');
          }
          App.ajax.send({
            name: 'bulk_request.decommission',
            sender: this,
            data: {
              context: Em.I18n.t(contextString),
              serviceName: service.get('serviceName'),
              componentName: operationData.componentName,
              parameters: parameters,
              noOpsMessage: operationData.componentNameFormatted
            },
            success: 'bulkOperationForHostComponentsSuccessCallback'
          });
        }
      }
    } else {
      App.ModalPopup.show({
        header: Em.I18n.t('rolling.nothingToDo.header'),
        body: Em.I18n.t('rolling.nothingToDo.body').format(operationData.componentNameFormatted),
        secondary: false
      });
    }
  },

  /**
   * get info about regionserver passive_state
   * @method warnBeforeDecommission
   * @param {String} hostNames
   * @return {$.ajax}
   */
  warnBeforeDecommission: function warnBeforeDecommission(hostNames) {
    return App.ajax.send({
      'name': 'host_components.hbase_regionserver.active',
      'sender': this,
      'data': {
        hostNames: hostNames
      },
      success: 'warnBeforeDecommissionSuccess'
    });
  },

  /**
   * check is hbase regionserver in mm. If so - run decommission
   * otherwise shows warning
   * @method warnBeforeDecommission
   * @param {Object} data
   * @param {Object} opt
   * @param {Object} params
   */
  warnBeforeDecommissionSuccess: function warnBeforeDecommissionSuccess(data, opt, params) {
    if (Em.get(data, 'items.length')) {
      return App.router.get('mainHostDetailsController').showHbaseActiveWarning();
    }
    return App.router.get('mainHostDetailsController').checkRegionServerState(params.hostNames);
  },

  /**
   * Bulk restart for selected hostComponents
   * @param {Object} operationData
   * @param {Array} hosts
   */
  bulkOperationForHostComponentsRestart: function bulkOperationForHostComponentsRestart(operationData, hosts) {
    var self = this;
    return batchUtils.getComponentsFromServer({
      components: [operationData.componentName],
      hosts: hosts.mapProperty('hostName'),
      passiveState: 'OFF',
      displayParams: ['Hosts/maintenance_state', 'host_components/HostRoles/stale_configs', 'host_components/HostRoles/maintenance_state']
    }, function (data) {
      return self._getComponentsFromServerForHostComponentsRestartCallback(operationData, data);
    });
  },

  _getComponentsFromServerForHostComponentsRestartCallback: function _getComponentsFromServerForHostComponentsRestartCallback(operationData, data) {
    var wrappedHostComponents = [];
    var service = App.Service.find(operationData.serviceName);

    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        wrappedHostComponents.push(O.create({
          componentName: hostComponent.HostRoles.component_name,
          serviceName: operationData.serviceName,
          hostName: host.Hosts.host_name,
          hostPassiveState: host.Hosts.maintenance_state,
          staleConfigs: hostComponent.HostRoles.stale_configs,
          passiveState: hostComponent.HostRoles.maintenance_state
        }));
      });
    });

    if (wrappedHostComponents.length) {
      return batchUtils.showRollingRestartPopup(wrappedHostComponents.objectAt(0).get('componentName'), service.get('displayName'), service.get('passiveState') === "ON", false, wrappedHostComponents);
    }
    return App.ModalPopup.show({
      header: Em.I18n.t('rolling.nothingToDo.header'),
      body: Em.I18n.t('rolling.nothingToDo.body').format(operationData.componentNameFormatted),
      secondary: false
    });
  },

  updateHostComponentsPassiveState: function updateHostComponentsPassiveState(data, opt, params) {
    return batchUtils.infoPassiveState(params.passive_state);
  },

  /**
   * Show BO popup after bulk request
   * @method bulkOperationForHostComponentsSuccessCallback
   */
  bulkOperationForHostComponentsSuccessCallback: function bulkOperationForHostComponentsSuccessCallback(data, opt, params, req) {
    if (!data && req.status == 200) {
      return App.ModalPopup.show({
        header: Em.I18n.t('rolling.nothingToDo.header'),
        body: Em.I18n.t('rolling.nothingToDo.body').format(params.noOpsMessage || Em.I18n.t('hosts.host.maintainance.allComponents.context')),
        secondary: false
      });
    }
    return App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  /**
   * Returns all hostNames if amount is less than {minShown} or
   * first elements of array (number of elements - {minShown}) converted to string
   * @param {Array} hostNames - array of all listed hostNames
   * @param {String} divider - string to separate hostNames
   * @param {Number} minShown - min amount of hostName to be shown
   * @returns {String} hostNames
   * @method _showHostNames
   * @private
   */
  _showHostNames: function _showHostNames(hostNames, divider, minShown) {
    if (hostNames.length > minShown) {
      return hostNames.slice(0, minShown).join(divider) + divider + Em.I18n.t("installer.step8.other").format(hostNames.length - minShown);
    }
    return hostNames.join(divider);
  },

  /**
   * Confirmation Popup for bulk Operations
   */
  bulkOperationConfirm: function bulkOperationConfirm(operationData, selection) {
    var hostsNames = [],
        queryParams = [];
    // @todo remove using external controller
    switch (selection) {
      case 's':
        hostsNames = App.db.getSelectedHosts('mainHostController');
        if (hostsNames.length > 0) {
          queryParams.push({
            key: 'Hosts/host_name',
            value: hostsNames,
            type: 'MULTIPLE'
          });
        }
        break;
      case 'f':
        queryParams = App.router.get('mainHostController').getQueryParameters(true).filter(function (obj) {
          return !(obj.key == 'page_size' || obj.key == 'from');
        });
        break;
    }

    if (operationData.action === 'SET_RACK_INFO') {
      this.getHostsForBulkOperations(queryParams, operationData, null);
      return;
    }

    this.getHostsForBulkOperations(queryParams, operationData);
  },

  getHostsForBulkOperations: function getHostsForBulkOperations(queryParams, operationData) {
    return App.ajax.send({
      name: 'hosts.bulk.operations',
      sender: this,
      data: {
        parameters: App.router.get('updateController').computeParameters(queryParams),
        operationData: operationData
      },
      success: 'getHostsForBulkOperationSuccessCallback',
      showLoadingPopup: true
    });
  },

  _convertHostsObjects: function _convertHostsObjects(hosts) {
    return hosts.map(function (host) {
      return {
        index: host.index,
        id: host.id,
        clusterId: host.cluster_id,
        passiveState: host.passive_state,
        state: host.state,
        hostName: host.host_name,
        hostComponents: host.host_components
      };
    });
  },

  getHostsForBulkOperationSuccessCallback: function getHostsForBulkOperationSuccessCallback(json, opt, param) {
    var self = this;
    var operationData = param.operationData;
    var hosts = this._convertHostsObjects(App.hostsMapper.map(json, true));
    var repoVersion = null;
    // no hosts - no actions
    if (!hosts.length) {
      return;
    }

    if (['SET_RACK_INFO', 'ADD', 'DELETE'].contains(operationData.action)) {
      return self.bulkOperation(operationData, hosts);
    }

    var hostNames = hosts.mapProperty('hostName');
    var hostNamesSkipped = [];
    if ('DECOMMISSION' === operationData.action) {
      hostNamesSkipped = this._getSkippedForDecommissionHosts(json, hosts, operationData);
    }
    if ('PASSIVE_STATE' === operationData.action) {
      repoVersion = App.StackVersion.find().findProperty('isCurrent');
      if (!repoVersion && App.StackVersion.find().toArray().length === 1) {
        repoVersion = App.StackVersion.find().findProperty('isOutOfSync');
      }
      if (!repoVersion) {
        console.error("CLUSTER STACK VERSIONS ERROR: multiple clusters in OUT_OF_SYNC state OR none in CURRENT or OUT_OF_SYNC state");
        return;
      }
      hostNamesSkipped = this._getSkippedForPassiveStateHosts(hosts, repoVersion);
    }

    var message = "";
    if (operationData.componentNameFormatted) {
      message = Em.I18n.t('hosts.bulkOperation.confirmation.hostComponents').format(operationData.message, operationData.componentNameFormatted, hostNames.length);
    } else {
      message = Em.I18n.t('hosts.bulkOperation.confirmation.hosts').format(operationData.message, hostNames.length);
    }

    return App.ModalPopup.show({
      header: Em.I18n.t('hosts.bulkOperation.confirmation.header'),
      hostNames: hostNames.join("\n"),
      visibleHosts: self._showHostNames(hostNames, "\n", 3),
      hostNamesSkippedVisible: self._showHostNames(hostNamesSkipped, "\n", 3),
      expanded: false,

      hostNamesSkipped: function () {
        return hostNamesSkipped.length ? hostNamesSkipped.join("\n") : false;
      }.property(),

      didInsertElement: function didInsertElement() {
        this._super();
        this.set('expanded', hostNames.length <= 3);
      },
      onPrimary: function onPrimary() {
        self.bulkOperation(operationData, hosts);
        this._super();
      },

      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/bulk_operation_confirm_popup'),
        message: message,
        textareaVisible: false,

        warningInfo: function () {
          switch (operationData.action) {
            case "DECOMMISSION":
              return Em.I18n.t('hosts.bulkOperation.warningInfo.body');
            case "PASSIVE_STATE":
              return operationData.state === 'OFF' ? Em.I18n.t('hosts.passiveMode.popup.version.mismatch.multiple').format(repoVersion.get('repositoryVersion.repositoryVersion')) : "";
            default:
              return "";
          }
        }.property(),

        textTrigger: function textTrigger() {
          this.toggleProperty('textareaVisible');
        },

        showAll: function showAll() {
          this.set('parentView.visibleHosts', this.get('parentView.hostNames'));
          this.set('parentView.hostNamesSkippedVisible', this.get('parentView.hostNamesSkipped'));
          this.set('parentView.expanded', true);
        },

        putHostNamesToTextarea: function () {
          var hostNames = this.get('parentView.hostNames');
          if (this.get('textareaVisible')) {
            var wrapper = $(".task-detail-log-maintext");
            $('.task-detail-log-clipboard').html(hostNames).width(wrapper.width()).height(250);
            Em.run.next(function () {
              $('.task-detail-log-clipboard').select();
            });
          }
        }.observes('textareaVisible')

      })
    });
  },

  /**
   * @param {object} json
   * @param {object[]} hosts
   * @param {object} operationData
   * @returns {string[]}
   * @private
   * @method _getSkippedForDecommissionHosts
   */
  _getSkippedForDecommissionHosts: function _getSkippedForDecommissionHosts(json, hosts, operationData) {
    var hostComponentStatusMap = {}; // "DATANODE_c6401.ambari.apache.org" => "STARTED"
    var hostComponentIdMap = {}; // "DATANODE_c6401.ambari.apache.org" => "DATANODE"
    if (json.items) {
      json.items.forEach(function (host) {
        if (host.host_components) {
          host.host_components.forEach(function (component) {
            hostComponentStatusMap[component.id] = component.HostRoles.state;
            hostComponentIdMap[component.id] = component.HostRoles.component_name;
          });
        }
      });
    }
    return hosts.filter(function (host) {
      return host.hostComponents.filter(function (component) {
        return hostComponentIdMap[component] == operationData.realComponentName && hostComponentStatusMap[component] == 'INSTALLED';
      }).length > 0;
    }).mapProperty('hostName');
  },

  /**
   * Exclude <code>outOfSyncHosts</code> hosts for PASSIVE request
   *
   * @param {object[]} hosts
   * @returns {string[]}
   * @private
   * @method _getSkippedForPassiveStateHosts
   */
  _getSkippedForPassiveStateHosts: function _getSkippedForPassiveStateHosts(hosts, repoVersion) {
    var hostNames = hosts.mapProperty('hostName');
    var hostNamesSkipped = [];
    var outOfSyncHosts = repoVersion.get('outOfSyncHosts') || [];
    for (var i = 0; i < outOfSyncHosts.length; i++) {
      if (hostNames.contains(outOfSyncHosts[i])) {
        hostNamesSkipped.push(outOfSyncHosts[i]);
      }
    }
    return hostNamesSkipped;
  }

});

});

require.register("controllers/main/host/combo_search_box", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostComboSearchBoxController = Em.Controller.extend({
  name: 'mainHostComboSearchBoxController',
  currentSuggestion: [],
  page_size: 10,
  getPropertySuggestions: function getPropertySuggestions(facet, searchTerm) {
    facet = facet == 'hostName' ? 'host_name' : facet;
    return App.ajax.send({
      name: 'hosts.with_searchTerm',
      sender: this,
      data: {
        facet: facet,
        searchTerm: searchTerm,
        page_size: this.get('page_size')
      },
      success: 'getPropertySuggestionsSuccess',
      error: 'commonSuggestionErrorCallback'
    });
  },

  getPropertySuggestionsSuccess: function getPropertySuggestionsSuccess(data, opt, params) {
    this.updateSuggestion(data.items.map(function (item) {
      return item.Hosts[params.facet];
    }));
  },

  updateSuggestion: function updateSuggestion(data) {
    var controller = App.router.get('mainHostComboSearchBoxController');
    controller.set('currentSuggestion', data);
  },

  commonSuggestionErrorCallback: function commonSuggestionErrorCallback() {
    // handle suggestion error
  },

  isComponentStateFacet: function isComponentStateFacet(facet) {
    return App.StackServiceComponent.find(facet).get('isLoaded');
  },

  isComplexHealthStatusFacet: function isComplexHealthStatusFacet(facet) {
    return ['health-status-WITH-ALERTS', 'health-status-RESTART', 'health-status-PASSIVE_STATE'].contains(facet);
  },

  generateQueryParam: function generateQueryParam(param) {
    var expressions = param.key;
    var pHash = this.createComboParamHash(param);
    return this.createComboParamURL(pHash, expressions);
  },

  /**
   * @param pHash {k1:v1, k2:[v1,v2], ...}
   * @param expression
   * @returns {string} 'k1=v1&(k2=v1|k2=v2)'
   */
  createComboParamURL: function createComboParamURL(pHash, expressions) {
    var self = this;
    var result = '';
    for (var key in pHash) {
      var v = pHash[key];
      if (Em.isArray(v)) {
        var ex = '(';
        v.forEach(function (item) {
          var expression = self.getComboParamURL(item, expressions);
          var toAdd = expression.replace('{0}', key);
          toAdd = toAdd.replace('{1}', item);
          ex += toAdd + '|';
        });
        ex = ex.substring(0, ex.length - 1);
        result += ex + ')';
      } else {
        var expression = self.getComboParamURL(v, expressions);
        var ex = expression.replace('{0}', key);
        ex = ex.replace('{1}', v);
        result += ex;
      }
      result += '|';
    }

    return result.substring(0, result.length - 1);
  },

  /**
   * @param value //value of component state
   * @returns expression //url of query of state
   */
  getComboParamURL: function getComboParamURL(value, expressions) {
    var expression = expressions[1];
    switch (value) {
      case 'ALL':
        expression = expressions[0];
        break;
      case 'STARTED':
      case 'STARTING':
      case 'INSTALLED':
      case 'STOPPING':
      case 'INSTALL_FAILED':
      case 'INSTALLING':
      case 'UPGRADE_FAILED':
      case 'UNKNOWN':
      case 'DISABLED':
      case 'INIT':
        break;
      case 'INSERVICE':
      case 'DECOMMISSIONING':
      case 'DECOMMISSIONED':
      case 'RS_DECOMMISSIONED':
        expression = expressions[2];
        break;
      case 'ON':
      case 'OFF':
        expression = expressions[3];
        break;
    }
    return expression;
  },

  /**
   * @param param ['k1:v1','k2:v1', 'k2:v2'] or 'k1:v1'
   * @returns {k1:v1, k2:[v1,v2], ...}
   */
  createComboParamHash: function createComboParamHash(param) {
    var pHash = {};
    if (Em.isArray(param.value)) {
      param.value.forEach(function (item) {
        var values = item.split(':');
        var k = values[0];
        var v = values[1];
        if (!pHash[k]) {
          pHash[k] = v;
        } else {
          if (Em.isArray(pHash[k])) {
            if (pHash[k].indexOf(v) == -1) {
              pHash[k].push(v);
            }
          } else {
            pHash[k] = [pHash[k], v];
          }
        }
      });
    } else {
      var values = param.value.split(':');
      pHash[values[0]] = values[1];
    }
    return pHash;
  }
});

});

require.register("controllers/main/host/configs_service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainHostServiceConfigsController = App.MainServiceInfoConfigsController.extend(App.ConfigOverridable, {
  name: 'mainHostServiceConfigsController',
  host: null,
  isHostsConfigsPage: true,
  typeTagToHostMap: null,
  configKeyToConfigMap: null,

  canEdit: false,

  /**
   * On load function
   */
  loadStep: function loadStep() {
    var content = this.get('content');
    this.set('host', content.host);
    this._super();
  },

  /**
   * Removes categories which are not valid for this host. Ex: Remove JOBTRACKER
   * category on host which does not have it installed.
   */
  renderServiceConfigs: function renderServiceConfigs(serviceConfigs) {
    var newServiceConfigs = jQuery.extend({}, serviceConfigs);
    newServiceConfigs.configCategories = this.filterServiceConfigs(serviceConfigs.configCategories);
    this._super(newServiceConfigs);
  },
  /**
   * filter config categories by host-component of host
   * @param configCategories
   * @return {Array}
   */
  filterServiceConfigs: function filterServiceConfigs(configCategories) {
    var hostComponents = this.get('host.hostComponents');
    var hostHostComponentNames = hostComponents ? hostComponents.mapProperty('componentName') : [];

    return configCategories.filter(function (category) {
      var hcNames = category.get('hostComponentNames');
      if (hcNames && hcNames.length > 0) {
        for (var i = 0, l = hcNames.length; i < l; i++) {
          if (hostHostComponentNames.contains(hcNames[i])) {
            return true;
          }
        }
        return false;
      }
      return true;
    });
  },

  /**
   * invoke dialog for switching group of host
   */
  switchHostGroup: function switchHostGroup() {
    var self = this;
    this.launchSwitchConfigGroupOfHostDialog(this.get('selectedConfigGroup'), this.get('configGroups'), this.get('host.hostName'), function (newGroup) {
      self.set('selectedConfigGroup', newGroup);
    });
  }
});

});

require.register("controllers/main/host/details", function(exports, require, module) {
'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var hostsManagement = require('utils/hosts');
var stringUtils = require('utils/string_utils');
require('utils/configs/add_component_config_initializer');

App.MainHostDetailsController = Em.Controller.extend(App.SupportClientConfigsDownload, App.InstallComponent, App.InstallNewVersion, App.CheckHostMixin, App.TrackRequestMixin, {

  name: 'mainHostDetailsController',

  /**
   * Viewed host
   * @type {App.Host|null}
   */
  content: null,

  /**
   * Is check host procedure finished
   * @type {bool}
   */
  checkHostFinished: null,

  /**
   * Does user come from hosts page
   * @type {bool}
   */
  isFromHosts: false,

  /**
   * Determines whether we are adding Hive Server2 component
   * @type {bool}
   */
  addHiveServer: false,

  /**
   * Determines whether we are adding ZooKeeper Server component
   * @type {bool}
   */
  addZooKeeperServer: false,

  /**
   * path to page visited before
   * @type {string}
   */
  referer: '',

  /**
   *  Host on which Hive Metastore will be added
   * @type {string}
   */
  hiveMetastoreHost: '',

  /**
   * Deferred object will be resolved when Oozie configs are downloaded
   * @type {object}
   */
  isOozieConfigLoaded: $.Deferred(),

  /**
   * @type {bool}
   */
  isOozieServerAddable: true,

  isConfigsLoadingInProgress: false,

  addDeleteComponentsMap: {
    'ZOOKEEPER_SERVER': {
      addPropertyName: 'addZooKeeperServer',
      deletePropertyName: 'fromDeleteZkServer',
      configTagsCallbackName: 'loadZookeeperConfigs',
      configsCallbackName: 'saveZkConfigs'
    },
    'HIVE_METASTORE': {
      deletePropertyName: 'deleteHiveMetaStore',
      hostPropertyName: 'hiveMetastoreHost',
      configTagsCallbackName: 'loadHiveConfigs',
      configsCallbackName: 'onLoadHiveConfigs'
    },
    'WEBHCAT_SERVER': {
      deletePropertyName: 'deleteWebHCatServer',
      hostPropertyName: 'webhcatServerHost',
      configTagsCallbackName: 'loadWebHCatConfigs',
      configsCallbackName: 'onLoadHiveConfigs'
    },
    'HIVE_SERVER': {
      addPropertyName: 'addHiveServer',
      deletePropertyName: 'deleteHiveServer',
      configTagsCallbackName: 'loadHiveConfigs',
      configsCallbackName: 'onLoadHiveConfigs'
    },
    'NIMBUS': {
      deletePropertyName: 'deleteNimbusHost',
      hostPropertyName: 'nimbusHost',
      configTagsCallbackName: 'loadStormConfigs',
      configsCallbackName: 'onLoadStormConfigs'
    },
    'ATLAS_SERVER': {
      deletePropertyName: 'deleteAtlasServer',
      hostPropertyName: 'atlasServer',
      configTagsCallbackName: 'loadAtlasConfigs',
      configsCallbackName: 'onLoadAtlasConfigs'
    },
    'RANGER_KMS_SERVER': {
      deletePropertyName: 'deleteRangerKMSServer',
      hostPropertyName: 'rangerKMSServerHost',
      configTagsCallbackName: 'loadRangerConfigs',
      configsCallbackName: 'onLoadRangerConfigs'
    }
  },

  zooKeeperRelatedServices: [{
    serviceName: 'HIVE',
    typesToLoad: ['hive-site', 'webhcat-site'],
    typesToSave: ['hive-site', 'webhcat-site']
  }, {
    serviceName: 'YARN',
    typesToLoad: ['yarn-site', 'zoo.cfg'],
    typesToSave: ['yarn-site']
  }, {
    serviceName: 'HBASE',
    typesToLoad: ['hbase-site'],
    typesToSave: ['hbase-site']
  }, {
    serviceName: 'ACCUMULO',
    typesToLoad: ['accumulo-site'],
    typesToSave: ['accumulo-site']
  }, {
    serviceName: 'KAFKA',
    typesToLoad: ['kafka-broker'],
    typesToSave: ['kafka-broker']
  }, {
    serviceName: 'ATLAS',
    typesToLoad: ['application-properties', 'infra-solr-env'],
    typesToSave: ['application-properties']
  }, {
    serviceName: 'STORM',
    typesToLoad: ['storm-site'],
    typesToSave: ['storm-site']
  }],

  /**
   * Determines whether adding/deleting host component requires configs changes
   * @type {Boolean}
   */
  isReconfigureRequired: false,

  /**
   * Contains component-related config properties loaded from server
   * @type {Object|null}
   */
  configs: null,

  /**
   * Array of all properties affected by adding/deleting host component
   * @type {Array}
   */
  allPropertiesToChange: [],

  /**
   * Array of editable properties affected by adding/deleting host component
   * @type {Array}
   */
  recommendedPropertiesToChange: [],

  /**
   * Array of non-editable properties affected by adding/deleting host component
   * @type {Array}
   */
  requiredPropertiesToChange: [],

  /**
   * Properties affected by adding/deleting host component, grouped by service, formatted for PUT call
   * @type {Array}
   */
  groupedPropertiesToChange: [],

  hasPropertiesToChange: Em.computed.or('recommendedPropertiesToChange.length', 'requiredPropertiesToChange.length'),

  addDeleteComponentPopupBody: Em.View.extend({
    templateName: require('templates/main/host/details/addDeleteComponentPopup'),
    commonMessage: '',
    manualKerberosWarning: App.get('router.mainAdminKerberosController.isManualKerberos') ? Em.I18n.t('hosts.host.manualKerberosWarning') : '',
    lastComponent: false,
    lastComponentError: '',
    fromServiceSummary: false,
    selectedHost: null,
    anyHostsWithoutComponent: true
  }),

  saveLoadedConfigs: function saveLoadedConfigs(data) {
    this.set('configs', {
      items: data.items.map(function (item) {
        return {
          type: item.type,
          properties_attributes: item.properties_attributes,
          properties: Em.copy(item.properties)
        };
      })
    });
  },

  clearConfigsChanges: function clearConfigsChanges(shouldKeepLoadedConfigs) {
    var arrayNames = ['allPropertiesToChange', 'recommendedPropertiesToChange', 'requiredPropertiesToChange', 'groupedPropertiesToChange'];
    this.abortRequests();
    arrayNames.forEach(function (arrayName) {
      this.get(arrayName).clear();
    }, this);
    this.set('isReconfigureRequired', false);
    if (!shouldKeepLoadedConfigs) {
      this.set('configs', null);
    }
  },

  applyConfigsCustomization: function applyConfigsCustomization() {
    this.get('recommendedPropertiesToChange').forEach(function (property) {
      var value = property.saveRecommended ? property.recommendedValue : property.initialValue,
          filename = property.propertyFileName;
      if (this.get('groupedPropertiesToChange.length')) {
        var group = this.get('groupedPropertiesToChange').find(function (item) {
          return item.properties.hasOwnProperty(filename);
        });
        if (group) {
          group.properties[filename][property.propertyName] = value;
        }
      }
    }, this);
  },

  /**
   * Open dashboard page
   * @method routeHome
   */
  routeHome: function routeHome() {
    App.router.transitionTo('main.dashboard.index');
  },

  /**
   * List of active (not in passive state) host components
   * @type {Ember.Enumerable}
   */
  serviceActiveComponents: function () {
    return this.get('content.hostComponents').filterProperty('service.isInPassive', false);
  }.property('content.hostComponents'),

  /**
   * List of active host components which aren't clients
   * @type {Ember.Enumerable}
   */
  serviceNonClientActiveComponents: Em.computed.filterBy('serviceActiveComponents', 'isClient', false),

  /**
   * send command to server to start selected host component
   * @param {object} event
   * @method startComponent
   */
  startComponent: function startComponent(event) {
    var self = this;
    var component = event.context;
    return App.showConfirmationPopup(function () {
      var context = Em.I18n.t('requestInfo.startHostComponent') + " " + component.get('displayName');
      self.sendComponentCommand(component, context, App.HostComponentStatus.started);
    }, Em.I18n.t('question.sure.start').format(component.get('displayName')));
  },

  /**
   * send command to server to stop selected host component
   * @param {object} event
   * @method startComponent
   */
  stopComponent: function stopComponent(event) {
    var self = this;
    var component = event.context;
    if (event.context.get('componentName') === 'NAMENODE') {
      this.checkNnLastCheckpointTime(function () {
        return App.showConfirmationPopup(function () {
          var context = Em.I18n.t('requestInfo.stopHostComponent') + " " + component.get('displayName');
          self.sendComponentCommand(component, context, App.HostComponentStatus.stopped);
        }, Em.I18n.t('question.sure.stop').format(component.get('displayName')));
      });
    } else {
      return App.showConfirmationPopup(function () {
        var context = Em.I18n.t('requestInfo.stopHostComponent') + " " + component.get('displayName');
        self.sendComponentCommand(component, context, App.HostComponentStatus.stopped);
      }, Em.I18n.t('question.sure.stop').format(component.get('displayName')));
    }
  },
  /**
   * PUTs a command to server to start/stop a component. If no
   * specific component is provided, all components are started.
   * @param {object} component  When <code>null</code> all startable components are started.
   * @param {String} context  Context under which this command is beign sent.
   * @param {String} state - desired state of component can be 'STARTED' or 'STOPPED'
   * @method sendComponentCommand
   */
  sendComponentCommand: function sendComponentCommand(component, context, state) {
    var data = {
      hostName: this.get('content.hostName'),
      context: context,
      component: component,
      HostRoles: {
        state: state
      }
    };
    if (Array.isArray(component)) {
      data.query = "HostRoles/component_name.in(" + component.mapProperty('componentName').join(',') + ")";
    } else {
      data.componentName = component.get('componentName');
      data.serviceName = component.get('service.serviceName');
    }
    App.ajax.send({
      name: Array.isArray(component) ? 'common.host.host_components.update' : 'common.host.host_component.update',
      sender: this,
      data: data,
      success: 'sendComponentCommandSuccessCallback',
      error: 'ajaxErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Success callback for stop/start host component request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method stopComponentSuccessCallback
   */
  sendComponentCommandSuccessCallback: function sendComponentCommandSuccessCallback(data, opt, params) {
    var running = params.HostRoles.state === App.HostComponentStatus.stopped ? App.HostComponentStatus.stopping : App.HostComponentStatus.starting;
    params.component.set('workStatus', running);
    if (App.get('testMode')) {
      this.mimicWorkStatusChange(params.component, running, params.HostRoles.state);
    }
    this.showBackgroundOperationsPopup();
  },

  /**
   * Return true if hdfs user data is loaded via App.MainServiceInfoConfigsController
   */
  getHdfsUser: function getHdfsUser() {
    var self = this;
    var dfd = $.Deferred();
    var miscController = App.MainAdminServiceAccountsController.create();
    miscController.loadUsers();
    miscController.addObserver('dataIsLoaded', this, function () {
      if (miscController.get('dataIsLoaded') && miscController.get('users')) {
        if (miscController.get('users').someProperty('name', 'hdfs_user')) {
          self.set('hdfsUser', miscController.get('users').findProperty('name', 'hdfs_user').get('value'));
        } else {
          self.set('hdfsUser', '&lt;hdfs-user&gt;');
        }
        dfd.resolve();
        miscController.destroy();
      }
    });
    return dfd.promise();
  },

  /**
   * this function will be called from :1) stop NN 2) restart NN 3) stop all components
   * @param callback - callback function to continue next operation
   * @param hostname - namenode host (by default is current host)
   */
  checkNnLastCheckpointTime: function checkNnLastCheckpointTime(callback, hostName) {
    var self = this;
    this.pullNnCheckPointTime(hostName).complete(function () {
      var isNNCheckpointTooOld = self.get('isNNCheckpointTooOld');
      self.set('isNNCheckpointTooOld', null);
      if (isNNCheckpointTooOld) {
        // too old
        self.getHdfsUser().done(function () {
          var msg = Em.Object.create({
            confirmMsg: Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld').format(App.nnCheckpointAgeAlertThreshold) + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.makeSure') + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions.singleHost.login').format(isNNCheckpointTooOld) + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions').format(self.get('hdfsUser')),
            confirmButton: Em.I18n.t('common.next')
          });
          return App.showConfirmationFeedBackPopup(callback, msg);
        });
      } else if (isNNCheckpointTooOld == null) {
        // not available
        return App.showConfirmationPopup(callback, Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointNA'), null, Em.I18n.t('common.warning'), Em.I18n.t('common.proceedAnyway'), 'danger');
      } else {
        // still young
        callback();
      }
    });
  },

  pullNnCheckPointTime: function pullNnCheckPointTime(hostName) {
    return App.ajax.send({
      name: 'common.host_component.getNnCheckPointTime',
      sender: this,
      data: {
        host: hostName || this.get('content.hostName')
      },
      success: 'parseNnCheckPointTime'
    });
  },

  parseNnCheckPointTime: function parseNnCheckPointTime(data) {
    var lastCheckpointTime = Em.get(data, 'metrics.dfs.FSNamesystem.LastCheckpointTime');
    var hostName = Em.get(data, 'HostRoles.host_name');

    if (Em.get(data, 'metrics.dfs.FSNamesystem.HAState') === 'active') {
      if (!lastCheckpointTime) {
        this.set("isNNCheckpointTooOld", null);
      } else {
        var time_criteria = App.nnCheckpointAgeAlertThreshold; // time in hours to define how many hours ago is too old
        var time_ago = (Math.round(App.dateTime() / 1000) - time_criteria * 3600) * 1000;
        if (lastCheckpointTime <= time_ago) {
          // too old, set the effected hostName
          this.set("isNNCheckpointTooOld", hostName);
        } else {
          // still young
          this.set("isNNCheckpointTooOld", false);
        }
      }
    } else if (Em.get(data, 'metrics.dfs.FSNamesystem.HAState') === 'standby') {
      this.set("isNNCheckpointTooOld", false);
    }
  },

  /**
   * mimic status transition in test mode
   * @param entity
   * @param transitionalState
   * @param finalState
   */
  mimicWorkStatusChange: function mimicWorkStatusChange(entity, transitionalState, finalState) {
    if (Em.isArray(entity)) {
      entity.forEach(function (item) {
        item.set('workStatus', transitionalState);
        setTimeout(function () {
          item.set('workStatus', finalState);
        }, App.testModeDelayForActions);
      });
    } else {
      entity.set('workStatus', transitionalState);
      setTimeout(function () {
        entity.set('workStatus', finalState);
      }, App.testModeDelayForActions);
    }
  },

  /**
   * load data (if we need to show this background operations popup) from persist
   * @param callback
   */
  showBackgroundOperationsPopup: function showBackgroundOperationsPopup(callback) {
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
      if (typeof callback === 'function') {
        callback();
      }
    });
  },

  /**
   * Send command to server to delete selected host component
   * @param {object} event
   * @method deleteComponent
   */
  deleteComponent: function deleteComponent(event) {
    var _this2 = this;

    if ($(event.target).closest('li').hasClass('disabled')) {
      return;
    }
    var component = event.context;
    var componentName = component.get('componentName');
    var componentsMapItem = this.get('addDeleteComponentsMap')[componentName];

    if (componentsMapItem) {
      this.deleteAndReconfigureComponent(componentsMapItem, component);
    } else if (componentName === 'JOURNALNODE') {
      return App.showConfirmationPopup(function () {
        App.router.transitionTo('main.services.manageJournalNode');
      }, Em.I18n.t('hosts.host.deleteComponent.popup.deleteJournalNodeMsg'));
    } else {
      return this.showReconfigurationPopupPreDelete(component, function () {
        return _this2._doDeleteHostComponent(componentName);
      });
    }
  },

  /**
   *
   * @param {object} componentsMapItem
   * @param {App.HostComponent} component
   * @returns {App.ModalPopup}
   */
  deleteAndReconfigureComponent: function deleteAndReconfigureComponent(componentsMapItem, component) {
    var _this3 = this;

    if (componentsMapItem.deletePropertyName) {
      this.set(componentsMapItem.deletePropertyName, true);
    }
    this.loadComponentRelatedConfigs(componentsMapItem.configTagsCallbackName, componentsMapItem.configsCallbackName);
    return this.showReconfigurationPopupPreDelete(component, function () {
      _this3._doDeleteHostComponent(component.get('componentName')).done(function () {
        _this3.applyConfigsCustomization();
        _this3.putConfigsToServer(_this3.get('groupedPropertiesToChange'), component.get('componentName'));
        _this3.clearConfigsChanges();
      });
    });
  },

  showReconfigurationPopupPreDelete: function showReconfigurationPopupPreDelete(component) {
    var primary = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Em.K;
    var commonMessage = arguments[2];

    var isLastComponent = this.getTotalComponent(component) === 1,
        componentDisplayName = component.get('displayName');

    return App.ModalPopup.show({
      header: Em.I18n.t('popup.confirmation.commonHeader'),
      controller: this,
      hasPropertiesToChange: false,
      classNameBindings: ['controller.hasPropertiesToChange:common-modal-wrapper'],
      modalDialogClasses: function () {
        return this.get('controller.hasPropertiesToChange') ? ['modal-xlg'] : [];
      }.property('controller.hasPropertiesToChange'),
      primary: Em.I18n.t('hosts.host.deleteComponent.popup.confirm'),
      bodyClass: this.get('addDeleteComponentPopupBody').extend({
        commonMessage: commonMessage || Em.I18n.t('hosts.host.deleteComponent.popup.msg1').format(componentDisplayName),
        recommendedPropertiesToChange: this.get('recommendedPropertiesToChange'),
        requiredPropertiesToChange: this.get('requiredPropertiesToChange'),
        lastComponentError: Em.I18n.t('hosts.host.deleteComponent.popup.warning').format(componentDisplayName),
        lastComponent: isLastComponent
      }),
      isChecked: !isLastComponent,
      disablePrimary: function () {
        return this.get('controller.isReconfigureRequired') && this.get('controller.isConfigsLoadingInProgress') || !this.get('isChecked');
      }.property('controller.isReconfigureRequired', 'controller.isConfigsLoadingInProgress', 'isChecked'),
      onPrimary: function onPrimary() {
        this._super();
        primary();
      },
      onSecondary: function onSecondary() {
        this._super();
        this.get('controller').clearConfigsChanges();
      },
      onClose: function onClose() {
        this._super();
        this.get('controller').clearConfigsChanges();
      }
    });
  },

  /**
   * get total count of host-components
   * @method getTotalComponent
   * @param component
   * @return {Number}
   */
  getTotalComponent: function getTotalComponent(component) {
    var count;
    if (component.get('isSlave')) {
      count = App.SlaveComponent.find(component.get('componentName')).get('totalCount');
    } else if (component.get('isClient')) {
      count = App.ClientComponent.find(component.get('componentName')).get('totalCount');
    } else {
      count = App.HostComponent.find().filterProperty('componentName', component.get('componentName')).get('length');
    }
    return count || 0;
  },

  /**
   * Trigger to reset list of master/slaves components on the view
   * @type {bool}
   */
  redrawComponents: false,

  /**
   * Deletes the given host component, or all host components.
   *
   * @param {string} componentName
   * @return  {$.ajax}
   * @method _doDeleteHostComponent
   */
  _doDeleteHostComponent: function _doDeleteHostComponent(componentName) {
    return App.ajax.send({
      name: Em.isNone(componentName) ? 'common.delete.host' : 'common.delete.host_component',
      sender: this,
      data: {
        componentName: componentName || '',
        hostName: this.get('content.hostName')
      },
      success: '_doDeleteHostComponentSuccessCallback',
      error: '_doDeleteHostComponentErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Error of delete component(s) request
   * @type {object}
   */
  _deletedHostComponentError: null,

  /**
   * Success callback for delete host component request
   * @method _doDeleteHostComponentSuccessCallback
   */
  _doDeleteHostComponentSuccessCallback: function _doDeleteHostComponentSuccessCallback(response, request, data) {
    this.set('_deletedHostComponentError', null);
  },

  /**
   * Error-callback for delete host component request
   * @param {object} xhr
   * @param {string} textStatus
   * @param {object} errorThrown
   * @param {object} data
   * @method _doDeleteHostComponentErrorCallback
   */
  _doDeleteHostComponentErrorCallback: function _doDeleteHostComponentErrorCallback(xhr, textStatus, errorThrown, data) {
    this.set('_deletedHostComponentError', { xhr: xhr, url: data.url, method: 'DELETE' });
  },

  /**
   * Send command to server to upgrade selected host component
   * @param {object} event
   * @method upgradeComponent
   */
  upgradeComponent: function upgradeComponent(event) {
    var self = this;
    var component = event.context;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'host.host_component.upgrade',
        sender: self,
        data: {
          component: component,
          hostName: self.get('content.hostName'),
          componentName: component.get('componentName'),
          data: JSON.stringify({
            RequestInfo: {
              "context": Em.I18n.t('requestInfo.upgradeHostComponent') + " " + component.get('displayName')
            },
            Body: {
              HostRoles: {
                stack_id: 'HDP-1.2.2',
                state: 'INSTALLED'
              }
            }
          })
        },
        success: 'upgradeComponentSuccessCallback',
        error: 'ajaxErrorCallback'
      });
    }, Em.I18n.t('question.sure.upgrade').format(component.get('displayName')));
  },

  /**
   * Success callback for upgrade host component request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method upgradeComponentSuccessCallback
   */
  upgradeComponentSuccessCallback: function upgradeComponentSuccessCallback(data, opt, params) {
    if (App.get('testMode')) {
      this.mimicWorkStatusChange(params.component, App.HostComponentStatus.starting, App.HostComponentStatus.started);
    }
    this.showBackgroundOperationsPopup();
  },

  /**
   * Send command to server to restart selected components
   * @param {object} event
   * @method restartComponent
   */
  restartComponent: function restartComponent(event) {
    var component = event.context;
    if (event.context.get('componentName') === 'NAMENODE') {
      this.checkNnLastCheckpointTime(function () {
        return App.showConfirmationPopup(function () {
          batchUtils.restartHostComponents([component], Em.I18n.t('rollingrestart.context.selectedComponentOnSelectedHost').format(component.get('displayName')), "HOST_COMPONENT");
        }, Em.I18n.t('question.sure.restart').format(component.get('displayName')));
      });
    } else {
      return App.showConfirmationPopup(function () {
        batchUtils.restartHostComponents([component], Em.I18n.t('rollingrestart.context.selectedComponentOnSelectedHost').format(component.get('displayName')), "HOST_COMPONENT");
      }, Em.I18n.t('question.sure.restart').format(component.get('displayName')));
    }
  },

  /**
   * add component as <code>addComponent<code> method but perform
   * kdc session state if cluster is secure;
   * @param event
   */
  addComponentWithCheck: function addComponentWithCheck(event) {
    App.get('router.mainAdminKerberosController').getSecurityType(function (event) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(this.addComponent.bind(this, event));
    }.bind(this, event));
  },
  /**
   * Send command to server to install selected host component
   * @param {object} event
   * @method addComponent
   */
  addComponent: function addComponent(event) {
    var _this4 = this;

    var component = event.context,
        hostName = this.get('content.hostName'),
        componentName = component.get('componentName'),
        missedComponents = event.fromServiceSummary ? [] : this.checkComponentDependencies(componentName, {
      scope: 'host',
      installedComponents: this.get('content.hostComponents').mapProperty('componentName')
    }),
        isManualKerberos = App.get('router.mainAdminKerberosController.isManualKerberos'),
        manualKerberosWarning = isManualKerberos ? Em.I18n.t('hosts.host.manualKerberosWarning') : '',
        componentsMapItem = this.get('addDeleteComponentsMap')[componentName];

    if (missedComponents.length) {
      var popupMessage = Em.I18n.t('host.host.addComponent.popup.dependedComponents.body').format(component.get('displayName'), stringUtils.getFormattedStringFromArray(missedComponents.map(function (cName) {
        return App.StackServiceComponent.find(cName).get('displayName');
      })));
      return App.showAlertPopup(Em.I18n.t('host.host.addComponent.popup.dependedComponents.header'), popupMessage);
    }

    if (componentsMapItem) {
      this.addAndReconfigureComponent(componentsMapItem, hostName, component, event.fromServiceSummary);
    } else if (componentName === 'JOURNALNODE') {
      return App.showConfirmationPopup(function () {
        App.router.transitionTo('main.services.manageJournalNode');
      }, Em.I18n.t('hosts.host.addComponent.' + componentName) + manualKerberosWarning);
    } else {
      return this.showAddComponentPopup(component, hostName, function () {
        // hostname should be taken via this.get('content.hostName') because the host could be selected later
        _this4.installHostComponentCall(_this4.get('content.hostName'), component);
      }, event.fromServiceSummary);
    }
  },

  /**
   *
   * @param {object} componentsMapItem
   * @param {string} hostName
   * @param {App.HostComponent} component
   * @param {boolean} fromServiceSummary
   * @returns {*}
   */
  addAndReconfigureComponent: function addAndReconfigureComponent(componentsMapItem, hostName, component, fromServiceSummary) {
    var _this5 = this;

    if (componentsMapItem.hostPropertyName) {
      this.set(componentsMapItem.hostPropertyName, hostName);
    }
    if (componentsMapItem.addPropertyName) {
      this.set(componentsMapItem.addPropertyName, true);
    }
    this.loadComponentRelatedConfigs(componentsMapItem.configTagsCallbackName, componentsMapItem.configsCallbackName);
    return this.showAddComponentPopup(component, hostName, function () {
      _this5.installAndReconfigureComponent(_this5.get('content.hostName'), component, componentsMapItem);
    }, fromServiceSummary);
  },

  /**
   *
   * @param {string} hostName
   * @param {App.HostComponent} component
   * @param {object} componentsMapItem
   */
  installAndReconfigureComponent: function installAndReconfigureComponent(hostName, component, componentsMapItem) {
    var _this6 = this;

    this.applyConfigsCustomization();
    this.installHostComponentCall(hostName, component).done(function () {
      _this6.putConfigsToServer(_this6.get('groupedPropertiesToChange'), component.get('componentName'));
    }).always(function () {
      _this6.set(componentsMapItem.addPropertyName, false);
      _this6.clearConfigsChanges();
    });
  },

  showAddComponentPopup: function showAddComponentPopup(component, hostName, primary, fromServiceSummary) {
    var self = this,
        componentName = component.get('componentName'),
        componentDisplayName = component.get('displayName'),
        componentsMapItem = this.get('addDeleteComponentsMap')[componentName],
        commonMessage = Em.I18n.t('hosts.host.addComponent.msg').format(componentDisplayName);

    return App.ModalPopup.show({
      header: Em.I18n.t('popup.confirmation.commonHeader'),
      controller: self,
      hasPropertiesToChange: false,
      classNameBindings: ['hasPropertiesToChange:common-modal-wrapper'],
      modalDialogClasses: function () {
        return this.get('controller.hasPropertiesToChange') ? ['modal-xlg'] : [];
      }.property('controller.hasPropertiesToChange'),
      primary: Em.I18n.t('hosts.host.addComponent.popup.confirm'),
      bodyClass: self.get('addDeleteComponentPopupBody').extend({
        commonMessage: commonMessage,
        fromServiceSummary: fromServiceSummary,
        addComponentMsg: Em.I18n.t('hosts.host.addComponent.msg').format(componentDisplayName),
        selectHostMsg: Em.I18n.t('services.summary.selectHostForComponent').format(componentDisplayName),
        thereIsNoHostsMsg: Em.I18n.t('services.summary.allHostsAlreadyRunComponent').format(componentDisplayName),
        hostsWithoutComponent: function () {
          if (this.get('fromServiceSummary')) {
            var hostsWithComponent = App.HostComponent.find().filterProperty('componentName', componentName).mapProperty('hostName');
            return App.get('allHostNames').filter(function (hostname) {
              return !hostsWithComponent.contains(hostname);
            });
          } else {
            return [];
          }
        }.property('fromServiceSummary'),
        anyHostsWithoutComponent: Em.computed.or('!fromServiceSummary', 'hostsWithoutComponent.length'),
        selectedHostObserver: function () {
          var selectedHostName = this.get('selectedHost');
          if (!self.get('content')) {
            self.set('content', {});
          }
          self.setProperties({
            'content.hostName': selectedHostName
          });
          if (componentsMapItem) {
            var configs = self.get('configs');
            var params = configs && configs.params || {};
            if (componentsMapItem.hostPropertyName) {
              self.set(componentsMapItem.hostPropertyName, selectedHostName);
            }
            if (componentsMapItem.addPropertyName) {
              self.set(componentsMapItem.addPropertyName, true);
            }
            if (configs) {
              self.clearConfigsChanges(true);
              self.set('isReconfigureRequired', true);
              self.set('isConfigsLoadingInProgress', true);
              self[componentsMapItem.configsCallbackName](configs, null, params);
            } else {
              self.loadComponentRelatedConfigs(componentsMapItem.configTagsCallbackName, componentsMapItem.configsCallbackName);
            }
          }
        }.observes('selectedHost')
      }),
      disablePrimary: Em.computed.and('controller.isReconfigureRequired', 'controller.isConfigsLoadingInProgress'),
      onPrimary: function onPrimary() {
        this._super();
        if (primary) {
          primary();
        }
      },
      onSecondary: function onSecondary() {
        this._super();
        self.clearConfigsChanges();
      },
      onClose: function onClose() {
        this._super();
        self.clearConfigsChanges();
      }
    });
  },

  loadComponentRelatedConfigs: function loadComponentRelatedConfigs(configTagsCallbackName, configsCallbackName) {
    var _this7 = this;

    this.clearConfigsChanges();
    this.set('isReconfigureRequired', true);
    this.set('isConfigsLoadingInProgress', true);
    this.isServiceMetricsLoaded(function () {
      _this7.loadConfigs(configTagsCallbackName, configsCallbackName);
    });
  },

  /**
   * Success callback for install host component request (sent in <code>addNewComponentSuccessCallback</code>)
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method installNewComponentSuccessCallback
   */
  installNewComponentSuccessCallback: function installNewComponentSuccessCallback(data, opt, params) {
    if (!data || !data.Requests || !data.Requests.id) {
      return false;
    }

    if (App.get('testMode')) {
      this.mimicWorkStatusChange(params.component, App.HostComponentStatus.installing, App.HostComponentStatus.stopped);
    }

    this.showBackgroundOperationsPopup();
    return true;
  },

  /**
   * Call <code>setRackInfo</code> function to show Set Rack Id popup
   * @param data
   */
  setRackId: function setRackId(data) {
    var rack = data.context.get('rack');
    var hosts = [data.context];
    var operationData = { message: Em.I18n.t('hosts.host.details.setRackId') };
    hostsManagement.setRackInfo(operationData, hosts, rack);
  },

  /**
   * Success callback for load configs request
   * @param {object} data
   * @method loadOozieConfigs
   */
  loadOozieConfigs: function loadOozieConfigs(data) {
    return App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: '(type=oozie-env&tag=' + data.Clusters.desired_configs['oozie-env'].tag + ')'
      },
      success: 'onLoadOozieConfigs',
      error: 'onLoadConfigsErrorCallback'
    });
  },

  /**
   * get Oozie database config and set databaseType
   * @param {object} data
   * @method onLoadOozieConfigs
   */
  onLoadOozieConfigs: function onLoadOozieConfigs(data) {
    var configs = {};
    data.items.forEach(function (item) {
      $.extend(configs, item.properties);
    });
    this.set('isOozieServerAddable', !(Em.isEmpty(configs["oozie_database"]) || configs["oozie_database"] === 'New Derby Database'));
    this.get('isOozieConfigLoaded').resolve();
  },

  /**
   * Success callback for Storm load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadStormConfigs
   */
  loadStormConfigs: function loadStormConfigs(data, opt, params) {
    var request = App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: '(type=storm-site&tag=' + data.Clusters.desired_configs['storm-site'].tag + ')'
      },
      success: params.callback
    });
    this.trackRequest(request);
  },

  /**
   * Success callback for Atlas load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadAtlasConfigs
   */
  loadAtlasConfigs: function loadAtlasConfigs(data, opt, params) {
    var request = App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: '(type=application-properties&tag=' + data.Clusters.desired_configs['application-properties'].tag + ')'
      },
      success: params.callback
    });
    this.trackRequest(request);
  },

  /**
   * Update zk configs
   * @param {object} configs
   * @method updateZkConfigs
   */
  updateZkConfigs: function updateZkConfigs(configs) {
    var portValue = configs['zoo.cfg'] && Em.get(configs['zoo.cfg'], 'clientPort'),
        zkPort = typeof portValue === 'undefined' ? '2181' : portValue,
        infraSolrZnode = configs['infra-solr-env'] ? Em.get(configs['infra-solr-env'], 'infra_solr_znode') : '/ambari-solr',
        initializer = App.AddZooKeeperComponentsInitializer,
        hostComponentsTopology = {
      masterComponentHosts: []
    },
        propertiesToChange = this.get('allPropertiesToChange'),
        masterComponents = this.bootstrapHostsMapping('ZOOKEEPER_SERVER');
    if (this.get('fromDeleteHost') || this.get('fromDeleteZkServer')) {
      this.set('fromDeleteHost', false);
      this.set('fromDeleteZkServer', false);
      var removedHost = masterComponents.findProperty('hostName', this.get('content.hostName'));
      if (!Em.isNone(removedHost)) {
        Em.set(removedHost, 'isInstalled', false);
      }
    } else if (this.get('addZooKeeperServer')) {
      this.set('addZooKeeperServer', false);
      masterComponents.push({
        component: 'ZOOKEEPER_SERVER',
        hostName: this.get('content.hostName'),
        isInstalled: true
      });
    }
    var dependencies = {
      zkClientPort: zkPort,
      infraSolrZnode: infraSolrZnode
    };
    hostComponentsTopology.masterComponentHosts = masterComponents;
    Em.keys(configs).forEach(function (fileName) {
      var properties = configs[fileName];
      Em.keys(properties).forEach(function (propertyName) {
        var currentValue = properties[propertyName],
            propertyDef = {
          fileName: fileName,
          name: propertyName,
          value: currentValue
        },
            configProperty = initializer.initialValue(propertyDef, hostComponentsTopology, dependencies);
        initializer.updateSiteObj(configs[fileName], configProperty);
        if (currentValue !== configs[fileName][propertyName]) {
          var service = App.config.get('serviceByConfigTypeMap')[fileName],
              configObject = App.configsCollection.getConfigByName(propertyName, fileName),
              displayName = configObject && configObject.displayName;
          propertiesToChange.pushObject({
            propertyFileName: fileName,
            propertyName: propertyName,
            propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === propertyName ? '' : propertyName),
            propertyDescription: configObject && configObject.description,
            serviceDisplayName: service && service.get('displayName'),
            initialValue: currentValue,
            recommendedValue: propertyDef.value
          });
        }
      });
    });
  },

  /**
   *
   * @param {string} componentName
   * @param {string[]} [hostNames]
   * @returns {}
   */
  bootstrapHostsMapping: function bootstrapHostsMapping(componentName, hostNames) {
    if (Em.isNone(hostNames)) {
      hostNames = App.HostComponent.find().filterProperty('componentName', componentName).mapProperty('hostName');
    }
    return hostNames.map(function (hostName) {
      return {
        component: componentName,
        hostName: hostName,
        isInstalled: true
      };
    });
  },

  /**
   * update and save Storm related configs to server
   * @param {object} data
   * @method onLoadStormConfigs
   */
  onLoadStormConfigs: function onLoadStormConfigs(data) {
    var nimbusHost = this.get('nimbusHost'),
        stormNimbusHosts = this.getStormNimbusHosts(),
        configs = {},
        attributes = {},
        propertiesToChange = this.get('allPropertiesToChange');

    this.saveLoadedConfigs(data);
    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
      attributes[item.type] = item.properties_attributes || {};
    });

    this.updateZkConfigs(configs);

    var propertyName = 'nimbus.seeds',
        propertyFileName = 'storm-site',
        nimbusSeedsInit = configs[propertyFileName][propertyName],
        nimbusSeedsRecommended = JSON.stringify(stormNimbusHosts).replace(/"/g, "'");
    configs[propertyFileName][propertyName] = nimbusSeedsRecommended;
    if (this.get('isReconfigureRequired') && nimbusSeedsInit !== nimbusSeedsRecommended) {
      var service = App.config.get('serviceByConfigTypeMap')[propertyFileName],
          configObject = App.configsCollection.getConfigByName(propertyName, propertyFileName),
          displayName = configObject && configObject.displayName;
      propertiesToChange.pushObject({
        propertyFileName: propertyFileName,
        propertyName: propertyName,
        propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === propertyName ? '' : propertyName),
        propertyDescription: configObject && configObject.description,
        serviceDisplayName: service && service.get('displayName'),
        initialValue: nimbusSeedsInit,
        recommendedValue: nimbusSeedsRecommended
      });
    }
    var groups = [{
      properties: _defineProperty({}, propertyFileName, configs[propertyFileName]),
      properties_attributes: _defineProperty({}, propertyFileName, attributes[propertyFileName])
    }];
    this.setConfigsChanges(groups);
  },

  onLoadAtlasConfigs: function onLoadAtlasConfigs(data) {
    var atlasServer = this.get('atlasServer'),
        atlasServerHosts = this.getAtlasServerHosts(),
        configs = {},
        attributes = {},
        propertiesToChange = this.get('allPropertiesToChange');

    this.saveLoadedConfigs(data);
    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
      attributes[item.type] = item.properties_attributes || {};
    });

    var propertyFileName = 'application-properties',
        propertyName = 'atlas.rest.address',
        atlasAddresses = configs[propertyFileName][propertyName],
        hostMask = atlasAddresses.split(',')[0].replace(/([https|http]*\:\/\/)(.*?)(:[0-9]+)/, '$1{hostname}$3'),
        atlasAddressesRecommended = atlasServerHosts.map(function (hostName) {
      return hostMask.replace('{hostname}', hostName);
    }).join(',');
    configs[propertyFileName][propertyName] = atlasAddressesRecommended;
    if (this.get('isReconfigureRequired') && atlasAddresses !== atlasAddressesRecommended) {
      var service = App.config.get('serviceByConfigTypeMap')[propertyFileName],
          configObject = App.configsCollection.getConfigByName(propertyName, propertyFileName),
          displayName = configObject && configObject.displayName;
      propertiesToChange.pushObject({
        propertyFileName: propertyFileName,
        propertyName: propertyName,
        propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === propertyName ? '' : propertyName),
        propertyDescription: configObject && configObject.description,
        serviceDisplayName: service && service.get('displayName'),
        initialValue: atlasAddresses,
        recommendedValue: atlasAddressesRecommended
      });
    }
    var groups = [{
      properties: _defineProperty({}, propertyFileName, configs[propertyFileName]),
      properties_attributes: _defineProperty({}, propertyFileName, attributes[propertyFileName])
    }];
    this.setConfigsChanges(groups);
  },

  /**
   * Success callback for load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadWebHCatConfigs
   */
  loadWebHCatConfigs: function loadWebHCatConfigs(data, opt, params) {
    var urlParams = this.getUrlParamsForConfigsRequest(data, ['hive-site', 'webhcat-site', 'hive-env', 'core-site']),
        request = App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        webHCat: true,
        urlParams: urlParams
      },
      success: params.callback
    });
    this.trackRequest(request);
    return request;
  },

  /**
   * Success callback for load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadHiveConfigs
   */
  loadHiveConfigs: function loadHiveConfigs(data, opt, params) {
    var urlParams = this.getUrlParamsForConfigsRequest(data, ['hive-site', 'webhcat-site', 'hive-env', 'core-site']),
        request = App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: urlParams
      },
      success: params.callback
    });
    this.trackRequest(request);
    return request;
  },

  /**
   * update and save Hive related configs to server
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method onLoadHiveConfigs
   */
  onLoadHiveConfigs: function onLoadHiveConfigs(data, opt, params) {
    var _this8 = this;

    var port = "";
    var configs = {},
        attributes = {},
        userSetup = {},
        localDB = {
      masterComponentHosts: this.getHiveHosts()
    },
        dependencies = {
      hiveMetastorePort: ""
    },
        initializer = params.webHCat ? App.AddWebHCatComponentsInitializer : App.AddHiveComponentsInitializer;
    this.saveLoadedConfigs(data);
    this.set('configs.params', {
      webHCat: params.webHCat
    });
    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
      attributes[item.type] = item.properties_attributes || {};
    });
    var propertiesToChange = this.get('allPropertiesToChange');

    port = configs['hive-site']['hive.metastore.uris'].match(/:[0-9]{2,4}/);
    port = port ? port[0].slice(1) : "9083";

    dependencies.hiveMetastorePort = port;

    if (params.webHCat) {
      userSetup.webhcatUser = configs['hive-env']['webhcat_user'];
    } else {
      userSetup.hiveUser = configs['hive-env']['hive_user'];
    }

    initializer.setup(userSetup);

    ['hive-site', 'webhcat-site', 'hive-env', 'core-site'].forEach(function (fileName) {
      if (configs[fileName]) {
        Em.keys(configs[fileName]).forEach(function (propertyName) {
          var currentValue = configs[fileName][propertyName],
              propertyDef = {
            fileName: fileName,
            name: propertyName,
            value: currentValue
          },
              configProperty = initializer.initialValue(propertyDef, localDB, dependencies);
          initializer.updateSiteObj(configs[fileName], configProperty);
          if (_this8.get('isReconfigureRequired') && currentValue !== configs[fileName][propertyName]) {
            var service = App.config.get('serviceByConfigTypeMap')[fileName],
                configObject = App.configsCollection.getConfigByName(propertyName, fileName),
                displayName = configObject && configObject.displayName;
            propertiesToChange.pushObject({
              propertyFileName: fileName,
              propertyName: propertyName,
              propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === propertyName ? '' : propertyName),
              propertyDescription: configObject && configObject.description,
              serviceDisplayName: service && service.get('displayName'),
              initialValue: currentValue,
              recommendedValue: propertyDef.value
            });
          }
        });
      }
    });

    initializer.cleanup();

    var groups = [{
      properties: {
        'hive-site': configs['hive-site'],
        'webhcat-site': configs['webhcat-site'],
        'hive-env': configs['hive-env']
      },
      properties_attributes: {
        'hive-site': attributes['hive-site'],
        'webhcat-site': attributes['webhcat-site'],
        'hive-env': attributes['hive-env']
      }
    }, {
      properties: {
        'core-site': configs['core-site']
      },
      properties_attributes: {
        'core-site': attributes['core-site']
      }
    }];
    this.setConfigsChanges(groups);
  },

  /**
   * save configs' sites in batch
   * @param groups
   * @param componentName
   */
  putConfigsToServer: function putConfigsToServer(groups, componentName) {
    var dfd = $.Deferred();
    var requests = [];
    if (groups.length) {
      groups.forEach(function (group) {
        var desiredConfigs = [];
        var properties = group.properties;

        for (var site in properties) {
          if (!properties.hasOwnProperty(site) || Em.isNone(properties[site])) continue;
          desiredConfigs.push({
            "type": site,
            "properties": properties[site],
            "properties_attributes": group.properties_attributes[site],
            "service_config_version_note": Em.I18n.t('hosts.host.configs.save.note').format(App.format.role(componentName, false))
          });
        }
        if (desiredConfigs.length > 0) {
          requests.push(App.ajax.send({
            name: 'common.service.configurations',
            sender: this,
            data: {
              desired_config: desiredConfigs,
              componentName: componentName
            }
          }));
        }
      }, this);
      $.when(requests).done(dfd.resolve).fail(dfd.reject).then(function () {
        // If user adding component which require config changes from Configs page then configs should be reloaded
        if (App.router.get('location.location.hash').contains('configs')) {
          App.router.get('mainServiceInfoConfigsController').loadStep();
        }
      });
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  /**
   * Delete Hive Metastore is performed
   * @type {bool}
   */
  deleteHiveMetaStore: false,

  /**
   * Delete WebHCat Server is performed
   *
   * @type {bool}
   */
  deleteWebHCatServer: false,

  getHiveHosts: function getHiveHosts() {
    var self = this;
    var removePerformed = this.get('fromDeleteHost') || this.get('deleteHiveMetaStore') || this.get('deleteHiveServer') || this.get('deleteWebHCatServer');
    var hiveMasterComponents = ['WEBHCAT_SERVER', 'HIVE_METASTORE', 'HIVE_SERVER'];
    var masterComponentsMap = hiveMasterComponents.map(function (componentName) {
      return self.bootstrapHostsMapping(componentName);
    }).reduce(function (p, c) {
      return p.concat(c);
    });

    if (removePerformed) {
      self.setProperties({
        deleteHiveMetaStore: false,
        deleteHiveServer: false,
        deleteWebHCatServer: false,
        fromDeleteHost: false
      });
      masterComponentsMap = masterComponentsMap.map(function (masterComponent) {
        masterComponent.isInstalled = masterComponent.hostName !== self.get('content.hostName');
        return masterComponent;
      });
    }

    if (!!this.get('hiveMetastoreHost')) {
      masterComponentsMap.push({
        component: 'HIVE_METASTORE',
        hostName: this.get('hiveMetastoreHost'),
        isInstalled: !removePerformed
      });
      this.set('hiveMetastoreHost', '');
    }

    if (!!this.get('webhcatServerHost')) {
      masterComponentsMap.push({
        component: 'WEBHCAT_SERVER',
        hostName: this.get('webhcatServerHost'),
        isInstalled: !removePerformed
      });
      this.set('webhcatServerHost', '');
    }

    return masterComponentsMap;
  },

  /**
   * Success callback for load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadRangerConfigs
   */
  loadRangerConfigs: function loadRangerConfigs(data, opt, params) {
    var urlParams = this.getUrlParamsForConfigsRequest(data, ['core-site', 'hdfs-site', 'kms-env', 'kms-site']),
        request = App.ajax.send({
      name: 'admin.get.all_configurations',
      sender: this,
      data: {
        urlParams: urlParams
      },
      success: params.callback
    });
    this.trackRequest(request);
  },

  /**
   * update and save Hive hive.metastore.uris config to server
   * @param {object} data
   * @method onLoadRangerConfigs
   */
  onLoadRangerConfigs: function onLoadRangerConfigs(data) {
    var _this9 = this;

    var hdfsProperties = [{
      type: 'core-site',
      name: 'hadoop.security.key.provider.path'
    }, {
      type: 'hdfs-site',
      name: 'dfs.encryption.key.provider.uri'
    }],
        kmsSiteProperties = [{
      name: 'hadoop.kms.cache.enable',
      notHaValue: 'true',
      haValue: 'false'
    }, {
      name: 'hadoop.kms.authentication.zk-dt-secret-manager.enable',
      notHaValue: 'false',
      haValue: 'true'
    }, {
      name: 'hadoop.kms.cache.timeout.ms',
      notHaValue: '600000',
      haValue: '0'
    }, {
      name: 'hadoop.kms.current.key.cache.timeout.ms',
      notHaValue: '30000',
      haValue: '0'
    }, {
      name: 'hadoop.kms.authentication.signer.secret.provider',
      notHaValue: 'random',
      haValue: 'zookeeper'
    }, {
      name: 'hadoop.kms.authentication.signer.secret.provider.zookeeper.auth.type',
      notHaValue: 'kerberos',
      haValue: 'none'
    }, {
      name: 'hadoop.kms.authentication.signer.secret.provider.zookeeper.connection.string',
      notHaValue: '#HOSTNAME#:#PORT#,...',
      haValue: this.getZookeeperConnectionString()
    }],
        rkmsHosts = this.getRangerKMSServerHosts(),
        rkmsHostsStr = rkmsHosts.join(';'),
        isHA = rkmsHosts.length > 1,
        rkmsPort = data.items.findProperty('type', 'kms-env').properties['kms_port'],
        newValue = 'kms://http@' + rkmsHostsStr + ':' + rkmsPort + '/kms',
        coreSiteConfigs = data.items.findProperty('type', 'core-site'),
        hdfsSiteConfigs = data.items.findProperty('type', 'hdfs-site'),
        kmsSiteConfigs = data.items.findProperty('type', 'kms-site'),
        groups = [{
      properties: {
        'core-site': coreSiteConfigs.properties,
        'hdfs-site': hdfsSiteConfigs.properties
      },
      properties_attributes: {
        'core-site': coreSiteConfigs.properties_attributes,
        'hdfs-site': hdfsSiteConfigs.properties_attributes
      }
    }, {
      properties: {
        'kms-site': kmsSiteConfigs.properties
      },
      properties_attributes: {
        'kms-site': kmsSiteConfigs.properties_attributes
      }
    }],
        propertiesToChange = this.get('allPropertiesToChange');

    this.saveLoadedConfigs(data);

    hdfsProperties.forEach(function (property) {
      var typeConfigs = data.items.findProperty('type', property.type).properties,
          currentValue = typeConfigs[property.name],
          pattern = new RegExp('^kms:\\/\\/http@(.+):' + rkmsPort + '\\/kms$'),
          patternMatch = currentValue && currentValue.match(pattern),
          currentHostsList = patternMatch && patternMatch[1].split(';').sort().join(';');
      if (currentHostsList !== rkmsHostsStr) {
        typeConfigs[property.name] = newValue;
        if (_this9.get('isReconfigureRequired')) {
          var propertyFileName = property.type,
              propertyName = property.name,
              service = App.config.get('serviceByConfigTypeMap')[propertyFileName],
              configObject = App.configsCollection.getConfigByName(propertyName, propertyFileName),
              displayName = configObject && configObject.displayName;
          propertiesToChange.pushObject({
            propertyFileName: propertyFileName,
            propertyName: propertyName,
            propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === propertyName ? '' : propertyName),
            propertyDescription: configObject && configObject.description,
            serviceDisplayName: service && service.get('displayName'),
            initialValue: currentValue,
            recommendedValue: newValue,
            saveRecommended: true
          });
        }
      }
    });

    kmsSiteProperties.forEach(function (property) {
      var currentValue = kmsSiteConfigs.properties[property.name];
      var newValue = isHA ? property.haValue : property.notHaValue;
      kmsSiteConfigs.properties[property.name] = newValue;

      propertiesToChange.pushObject({
        propertyFileName: 'kms-site',
        propertyName: property.name,
        serviceDisplayName: App.Service.find().findProperty('serviceName', 'RANGER_KMS').get('displayName'),
        initialValue: currentValue,
        recommendedValue: newValue,
        saveRecommended: true
      });
    });
    this.setConfigsChanges(groups);
  },

  /**
   * Delete Hive Metastore is performed
   * @type {bool}
   */
  deleteRangerKMSServer: false,

  getRangerKMSServerHosts: function getRangerKMSServerHosts() {
    var rkmsHosts = App.HostComponent.find().filterProperty('componentName', 'RANGER_KMS_SERVER').mapProperty('hostName');
    var rangerKMSServerHost = this.get('rangerKMSServerHost');

    if (!!rangerKMSServerHost) {
      rkmsHosts.push(rangerKMSServerHost);
    }

    if (this.get('fromDeleteHost') || this.get('deleteRangerKMSServer')) {
      return rkmsHosts.without(this.get('content.hostName'));
    }
    return rkmsHosts.sort();
  },

  getZookeeperConnectionString: function getZookeeperConnectionString() {
    var zookeeperHosts = App.MasterComponent.find('ZOOKEEPER_SERVER').get('hostNames');
    return zookeeperHosts.map(function (host) {
      return host + ':2181';
    }).join(',');
  },

  /**
   * Delete Storm Nimbus is performed
   * @type {bool}
   */
  deleteNimbusHost: false,

  getStormNimbusHosts: function getStormNimbusHosts() {
    var stormNimbusHosts = App.HostComponent.find().filterProperty('componentName', 'NIMBUS').mapProperty('hostName'),
        nimbusHost = this.get('nimbusHost');

    if (!!nimbusHost) {
      stormNimbusHosts.push(nimbusHost);
      this.set('nimbusHost', '');
    }

    if (this.get('fromDeleteHost') || this.get('deleteNimbusHost')) {
      this.set('deleteNimbusHost', false);
      this.set('fromDeleteHost', false);
      return stormNimbusHosts.without(this.get('content.hostName'));
    }
    return stormNimbusHosts.sort();
  },

  getAtlasServerHosts: function getAtlasServerHosts() {
    var atlasServerHosts = App.HostComponent.find().filterProperty('componentName', 'ATLAS_SERVER').mapProperty('hostName'),
        atlasServer = this.get('atlasServer');

    if (!!atlasServer) {
      atlasServerHosts.push(atlasServer);
      this.set('atlasServer', '');
    }

    if (this.get('fromDeleteHost') || this.get('deleteAtlasServer')) {
      this.set('deleteAtlasServer', false);
      this.set('fromDeleteHost', false);
      return atlasServerHosts.without(this.get('content.hostName'));
    }
    return atlasServerHosts.sort();
  },

  /**
   * Send command to server to resfresh configs of selected component
   * @param {object} event
   * @method refreshComponentConfigs
   */
  refreshComponentConfigs: function refreshComponentConfigs(event) {
    var self = this;
    return App.showConfirmationPopup(function () {
      var component = event.context;
      var context = Em.I18n.t('requestInfo.refreshComponentConfigs').format(component.get('displayName'));
      self.sendRefreshComponentConfigsCommand(component, context);
    });
  },

  /**
   * PUTs a command to server to refresh configs of host component.
   * @param {object} component
   * @param {object} context Context under which this command is beign sent.
   * @method sendRefreshComponentConfigsCommand
   */
  sendRefreshComponentConfigsCommand: function sendRefreshComponentConfigsCommand(component, context) {
    var resource_filters = [{
      service_name: component.get('service.serviceName'),
      component_name: component.get('componentName'),
      hosts: component.get('host.hostName')
    }];
    App.ajax.send({
      name: 'host.host_component.refresh_configs',
      sender: this,
      data: {
        resource_filters: resource_filters,
        context: context
      },
      success: 'refreshComponentConfigsSuccessCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Success callback for refresh host component configs request
   * @method refreshComponentConfigsSuccessCallback
   */
  refreshComponentConfigsSuccessCallback: function refreshComponentConfigsSuccessCallback() {
    this.showBackgroundOperationsPopup();
  },

  /**
   * Load configs
   * This function when used without a callback should be always used from successcallback function of the promise `App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(promise)`
   * This is required to make sure that service metrics API determining the HA state of components is loaded
   * @method loadConfigs
   */
  loadConfigs: function loadConfigs(configTagsCallback, configsCallback) {
    var request = App.ajax.send({
      name: 'config.tags',
      sender: this,
      data: {
        callback: configsCallback
      },
      success: configTagsCallback,
      error: 'onLoadConfigsErrorCallback'
    });
    this.trackRequest(request);
  },

  /**
   * onLoadConfigsErrorCallback
   * @method onLoadConfigsErrorCallback
   */
  onLoadConfigsErrorCallback: function onLoadConfigsErrorCallback() {
    this.get('isOozieConfigLoaded').reject();
  },

  /**
   * Success callback for load configs request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method loadZookeeperConfigs
   */
  loadZookeeperConfigs: function loadZookeeperConfigs(data, opt, params) {
    var urlParams = this.constructZookeeperConfigUrlParams(data);
    if (urlParams.length > 0) {
      var request = App.ajax.send({
        name: 'reassign.load_configs',
        sender: this,
        data: {
          urlParams: urlParams.join('|')
        },
        success: params.callback
      });
      this.trackRequest(request);
      return true;
    }
    this.set('isConfigsLoadingInProgress', false);
    return false;
  },

  /**
   * construct URL params for query, that load configs
   * @param data {Object}
   * @return {Array}
   */
  constructZookeeperConfigUrlParams: function constructZookeeperConfigUrlParams(data) {
    var urlParams = [];
    var services = App.Service.find();
    var zooKeeperRelatedServices = this.get('zooKeeperRelatedServices').slice(0);
    if (App.get('isHaEnabled')) {
      zooKeeperRelatedServices.push({
        serviceName: 'HDFS',
        typesToLoad: ['core-site'],
        typesToSave: ['core-site']
      });
    }
    zooKeeperRelatedServices.forEach(function (service) {
      if (services.someProperty('serviceName', service.serviceName)) {
        service.typesToLoad.forEach(function (type) {
          if (data.Clusters.desired_configs[type]) {
            urlParams.push('(type=' + type + '&tag=' + data.Clusters.desired_configs[type].tag + ')');
          }
        });
      }
    });
    return urlParams;
  },

  /**
   * save new ZooKeeper configs to server
   * @param {object} data
   * @method saveZkConfigs
   */
  saveZkConfigs: function saveZkConfigs(data) {
    var configs = {};
    var attributes = {};
    this.saveLoadedConfigs(data);
    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
      attributes[item.type] = item.properties_attributes || {};
    }, this);

    this.updateZkConfigs(configs);
    var groups = [];
    var installedServiceNames = App.Service.find().mapProperty('serviceName');
    var zooKeeperRelatedServices = this.get('zooKeeperRelatedServices').slice(0);
    if (App.get('isHaEnabled')) {
      zooKeeperRelatedServices.push({
        serviceName: 'HDFS',
        typesToLoad: ['core-site'],
        typesToSave: ['core-site']
      });
    }
    zooKeeperRelatedServices.forEach(function (service) {
      if (installedServiceNames.contains(service.serviceName)) {
        var group = {
          properties: {},
          properties_attributes: {}
        };
        service.typesToSave.forEach(function (type) {
          group.properties[type] = configs[type];
          group.properties_attributes[type] = attributes[type];
        });
        groups.push(group);
      }
    });
    this.setConfigsChanges(groups);
  },

  /**
   * Is deleteHost action id fired
   * @type {bool}
   */
  fromDeleteHost: false,

  /**
   * Is ZooKeeper Server being deleted from host
   * @type {bool}
   */
  fromDeleteZkServer: false,

  /**
   * Send command to server to install selected host component
   * @param {Object} event
   * @method installComponent
   */
  installComponent: function installComponent(event) {
    var self = this;
    var component = event.context;
    var componentName = component.get('componentName');
    var displayName = component.get('displayName');

    return App.ModalPopup.show({
      primary: Em.I18n.t('hosts.host.installComponent.popup.confirm'),
      header: Em.I18n.t('popup.confirmation.commonHeader'),
      installComponentMessage: Em.I18n.t('hosts.host.installComponent.msg').format(displayName),
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/details/installComponentPopup')
      }),
      onPrimary: function onPrimary() {
        var _this = this;
        App.get('router.mainAdminKerberosController').getSecurityType(function () {
          App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
            _this.hide();

            App.ajax.send({
              name: 'common.host.host_component.update',
              sender: self,
              data: {
                hostName: self.get('content.hostName'),
                serviceName: component.get('service.serviceName'),
                componentName: componentName,
                component: component,
                context: Em.I18n.t('requestInfo.installHostComponent') + " " + displayName,
                HostRoles: {
                  state: 'INSTALLED'
                }
              },
              success: 'installComponentSuccessCallback',
              error: 'ajaxErrorCallback',
              showLoadingPopup: true
            });
          });
        });
      }
    });
  },

  /**
   * Success callback for install component request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method installComponentSuccessCallback
   */
  installComponentSuccessCallback: function installComponentSuccessCallback(data, opt, params) {
    if (App.get('testMode')) {
      this.mimicWorkStatusChange(params.component, App.HostComponentStatus.installing, App.HostComponentStatus.stopped);
    }
    this.showBackgroundOperationsPopup();
  },

  /**
   * Send command to server to run decommission on DATANODE, TASKTRACKER, NODEMANAGER, REGIONSERVER
   * @param {App.HostComponent} component
   * @param {callback} callback
   * @method decommission
   */
  decommission: function decommission(component, callback) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.runDecommission.call(self, self.get('content.hostName'), component.get('service.serviceName'));
      if (callback) {
        callback();
      }
    }, Em.I18n.t('question.sure.decommission').format(component.get('service.serviceName')));
  },
  /**
   * identify correct component to run decommission on them by service name,
   * in result run proper decommission method
   * @param hostName
   * @param svcName
   */
  runDecommission: function runDecommission(hostName, svcName) {
    switch (svcName) {
      case 'HDFS':
        this.doDecommission(hostName, svcName, "NAMENODE", "DATANODE");
        break;
      case 'YARN':
        this.doDecommission(hostName, svcName, "RESOURCEMANAGER", "NODEMANAGER");
        break;
      case 'HBASE':
        this.warnBeforeDecommission(hostName);
    }
  },

  /**
   * Send command to server to run recommission on DATANODE, TASKTRACKER, NODEMANAGER
   * @param {App.HostComponent} component
   * @param {callback} callback
   * @method recommission
   */
  recommission: function recommission(component, callback) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.runRecommission.call(self, self.get('content.hostName'), component.get('service.serviceName'));
      if (callback) {
        callback();
      }
    }, Em.I18n.t('question.sure.recommission').format(component.get('service.serviceName')));
  },
  /**
   * identify correct component to run recommission on them by service name,
   * in result run proper recommission method
   * @param hostName
   * @param svcName
   */
  runRecommission: function runRecommission(hostName, svcName) {
    switch (svcName) {
      case 'HDFS':
        this.doRecommissionAndStart(hostName, svcName, "NAMENODE", "DATANODE");
        break;
      case 'YARN':
        this.doRecommissionAndStart(hostName, svcName, "RESOURCEMANAGER", "NODEMANAGER");
        break;
      case 'HBASE':
        this.doRecommissionAndStart(hostName, svcName, "HBASE_MASTER", "HBASE_REGIONSERVER");
    }
    this.showBackgroundOperationsPopup();
  },

  /**
   * Performs Decommission (for DN, TT and NM)
   * @param {string} hostName
   * @param {string} serviceName
   * @param {string} componentName
   * @param {string} slaveType
   * @method doDecommission
   */
  doDecommission: function doDecommission(hostName, serviceName, componentName, slaveType) {
    var contextNameString = 'hosts.host.' + slaveType.toLowerCase() + '.decommission';
    var context = Em.I18n.t(contextNameString);
    App.ajax.send({
      name: 'host.host_component.decommission_slave',
      sender: this,
      data: {
        context: context,
        command: 'DECOMMISSION',
        hostName: hostName,
        serviceName: serviceName,
        componentName: componentName,
        slaveType: slaveType
      },
      success: 'decommissionSuccessCallback',
      error: 'decommissionErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * check is hbase regionserver in mm. If so - run decommission
   * otherwise shows warning
   * @method warnBeforeDecommission
   * @param {string} hostNames - list of host when run from bulk operations or current host
   */
  warnBeforeDecommission: function warnBeforeDecommission(hostNames) {
    if (this.get('content.hostComponents').findProperty('componentName', 'HBASE_REGIONSERVER').get('passiveState') == "OFF") {
      this.showHbaseActiveWarning();
    } else {
      this.checkRegionServerState(hostNames);
    }
  },

  /**
   *  send call to check is this regionserver last in cluster which has desired_admin_state property "INSERVICE"
   * @method checkRegionServerState
   * @param hostNames
   */
  checkRegionServerState: function checkRegionServerState(hostNames) {
    return App.ajax.send({
      name: 'host.region_servers.in_inservice',
      sender: this,
      data: {
        hostNames: hostNames
      },
      success: 'checkRegionServerStateSuccessCallback'
    });
  },

  /**
   * check is this regionserver last in cluster which has desired_admin_state property "INSERVICE"
   * @method checkRegionServerStateSuccessCallback
   * @param data
   * @param opt
   * @param params
   */
  checkRegionServerStateSuccessCallback: function checkRegionServerStateSuccessCallback(data, opt, params) {
    var hostArray = params.hostNames.split(",");
    var decommissionPossible = data.items.mapProperty('HostRoles.host_name').filter(function (hostName) {
      return !hostArray.contains(hostName);
    }, this).length >= 1;
    if (decommissionPossible) {
      this.doDecommissionRegionServer(params.hostNames, "HBASE", "HBASE_MASTER", "HBASE_REGIONSERVER");
    } else {
      this.showRegionServerWarning();
    }
  },

  /**
   * show warning that regionserver is last in cluster which has desired_admin_state property "INSERVICE"
   * @method showRegionServerWarning
   * @param hostNames
   */
  showRegionServerWarning: function showRegionServerWarning() {
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      message: Em.I18n.t('hosts.host.hbase_regionserver.decommission.warning'),
      bodyClass: Ember.View.extend({
        template: Em.Handlebars.compile('<div class="alert alert-warning">{{message}}</div>')
      }),
      secondary: false
    });
  },

  /**
   * shows warning: put hbase regionserver in passive state
   * @method showHbaseActiveWarning
   * @return {App.ModalPopup}
   */
  showHbaseActiveWarning: function showHbaseActiveWarning() {
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      message: Em.I18n.t('hostPopup.recommendation.beforeDecommission').format(App.format.components["HBASE_REGIONSERVER"]),
      bodyClass: Ember.View.extend({
        template: Em.Handlebars.compile('<div class="alert alert-warning">{{message}}</div>')
      }),
      secondary: false
    });
  },

  /**
   * Performs Decommission (for RegionServer)
   * @method doDecommissionRegionServer
   * @param {string} hostNames - list of host when run from bulk operations or current host
   * @param {string} serviceName - serviceName
   * @param {string} componentName - master compoent name
   * @param {string} slaveType - slave component name
   */
  doDecommissionRegionServer: function doDecommissionRegionServer(hostNames, serviceName, componentName, slaveType) {
    var batches = [{
      "order_id": 1,
      "type": "POST",
      "uri": "/clusters/" + App.get('clusterName') + "/requests",
      "RequestBodyInfo": {
        "RequestInfo": {
          "context": Em.I18n.t('hosts.host.regionserver.decommission.batch1'),
          "command": "DECOMMISSION",
          "exclusive": "true",
          "parameters": {
            "slave_type": slaveType,
            "excluded_hosts": hostNames
          },
          'operation_level': {
            level: "HOST_COMPONENT",
            cluster_name: App.get('clusterName'),
            host_name: hostNames,
            service_name: serviceName
          }
        },
        "Requests/resource_filters": [{ "service_name": serviceName, "component_name": componentName }]
      }
    }];
    var id = 2;
    var hAray = hostNames.split(",");
    for (var i = 0; i < hAray.length; i++) {
      batches.push({
        "order_id": id,
        "type": "PUT",
        "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hAray[i] + "/host_components/" + slaveType,
        "RequestBodyInfo": {
          "RequestInfo": {
            context: Em.I18n.t('hosts.host.regionserver.decommission.batch2'),
            exclusive: true,
            operation_level: {
              level: "HOST_COMPONENT",
              cluster_name: App.get('clusterName'),
              host_name: hostNames,
              service_name: serviceName || null
            }
          },
          "Body": {
            HostRoles: {
              state: "INSTALLED"
            }
          }
        }
      });
      id++;
    }
    batches.push({
      "order_id": id,
      "type": "POST",
      "uri": "/clusters/" + App.get('clusterName') + "/requests",
      "RequestBodyInfo": {
        "RequestInfo": {
          "context": Em.I18n.t('hosts.host.regionserver.decommission.batch3'),
          "command": "DECOMMISSION",
          "service_name": serviceName,
          "component_name": componentName,
          "parameters": {
            "slave_type": slaveType,
            "excluded_hosts": hostNames,
            "mark_draining_only": true
          },
          'operation_level': {
            level: "HOST_COMPONENT",
            cluster_name: App.get('clusterName'),
            host_name: hostNames,
            service_name: serviceName
          }
        },
        "Requests/resource_filters": [{ "service_name": serviceName, "component_name": componentName }]
      }
    });
    App.ajax.send({
      name: 'common.batch.request_schedules',
      sender: this,
      data: {
        intervalTimeSeconds: 1,
        tolerateSize: 0,
        batches: batches
      },
      success: 'decommissionSuccessCallback',
      error: 'decommissionErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Error callback for decommission requests
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @method decommissionErrorCallback
   */
  decommissionErrorCallback: function decommissionErrorCallback(request, ajaxOptions, error) {},

  /**
   * Success ajax response for Recommission/Decommission slaves
   * @param {object} data
   * @method decommissionSuccessCallback
   * @return {Boolean}
   */
  decommissionSuccessCallback: function decommissionSuccessCallback(data) {
    if (data && (data.Requests || data.resources[0].RequestSchedule)) {
      this.showBackgroundOperationsPopup();
      return true;
    } else {
      return false;
    }
  },

  /**
   * Performs Recommission and Start
   * @param {string} hostNames
   * @param {string} serviceName
   * @param {string} componentName
   * @param {string} slaveType
   * @method doRecommissionAndStart
   */
  doRecommissionAndStart: function doRecommissionAndStart(hostNames, serviceName, componentName, slaveType) {
    var contextNameString_1 = 'hosts.host.' + slaveType.toLowerCase() + '.recommission';
    var context_1 = Em.I18n.t(contextNameString_1);
    var contextNameString_2 = 'requestInfo.startHostComponent.' + slaveType.toLowerCase();
    var startContext = Em.I18n.t(contextNameString_2);
    var params = {
      "slave_type": slaveType,
      "included_hosts": hostNames
    };
    if (serviceName == "HBASE") {
      params.mark_draining_only = true;
    }
    var batches = [{
      "order_id": 1,
      "type": "POST",
      "uri": "/clusters/" + App.get('clusterName') + "/requests",
      "RequestBodyInfo": {
        "RequestInfo": {
          "context": context_1,
          "command": "DECOMMISSION",
          "exclusive": "true",
          "parameters": params,
          'operation_level': {
            level: "HOST_COMPONENT",
            cluster_name: App.get('clusterName'),
            host_name: hostNames,
            service_name: serviceName
          }
        },
        "Requests/resource_filters": [{ "service_name": serviceName, "component_name": componentName }]
      }
    }];
    var id = 2;
    var hAray = hostNames.split(",");
    for (var i = 0; i < hAray.length; i++) {
      batches.push({
        "order_id": id,
        "type": "PUT",
        "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hAray[i] + "/host_components/" + slaveType,
        "RequestBodyInfo": {
          "RequestInfo": {
            context: startContext,
            operation_level: {
              level: "HOST_COMPONENT",
              cluster_name: App.get('clusterName'),
              host_name: hostNames,
              service_name: serviceName || null
            }
          },
          "Body": {
            HostRoles: {
              state: "STARTED"
            }
          }
        }
      });
      id++;
    }
    App.ajax.send({
      name: 'common.batch.request_schedules',
      sender: this,
      data: {
        intervalTimeSeconds: 1,
        tolerateSize: 1,
        batches: batches
      },
      success: 'decommissionSuccessCallback',
      error: 'decommissionErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Handler for host-menu items actions
   * @param {object} option
   * @method doAction
   */
  doAction: function doAction(option) {
    switch (option.context.action) {
      case "deleteHost":
        this.validateAndDeleteHost();
        break;
      case "startAllComponents":
        if (!this.get('content.isNotHeartBeating')) this.doStartAllComponents();
        break;
      case "stopAllComponents":
        if (!this.get('content.isNotHeartBeating')) this.doStopAllComponents();
        break;
      case "restartAllComponents":
        if (!this.get('content.isNotHeartBeating')) this.doRestartAllComponents();
        break;
      case "onOffPassiveModeForHost":
        this.onOffPassiveModeForHost(option.context);
        break;
      case "setRackId":
        this.setRackIdForHost();
        break;
      case "checkHost":
        this.runHostCheckConfirmation();
        break;
      case "regenerateKeytabFileOperations":
        this.regenerateKeytabFileOperations();
        break;
    }
  },

  /**
   * Turn On/Off Passive Mode for host
   * @param {object} context
   * @method onOffPassiveModeForHost
   */
  onOffPassiveModeForHost: function onOffPassiveModeForHost(context) {
    var state = context.active ? 'ON' : 'OFF';
    var self = this;
    var message = Em.I18n.t('hosts.host.details.for.postfix').format(context.label);
    var popupInfo = Em.I18n.t('hosts.passiveMode.popup').format(context.active ? 'On' : 'Off', this.get('content.hostName'));
    if (state === 'OFF') {
      var currentHostVersion = this.get('content.stackVersions') && this.get('content.stackVersions').findProperty('isCurrent'),
          hostVersion = currentHostVersion && currentHostVersion.get('repoVersion'),
          currentVersion = App.StackVersion.find().findProperty('isCurrent'),
          clusterVersion = currentVersion && currentVersion.get('repositoryVersion.repositoryVersion');
      if (hostVersion !== clusterVersion) {
        var msg = Em.I18n.t("hosts.passiveMode.popup.version.mismatch").format(this.get('content.hostName'), clusterVersion);
        popupInfo += '<br/><div class="alert alert-warning">' + msg + '</div>';
      }
    }
    return App.showConfirmationPopup(function () {
      self.hostPassiveModeRequest(state, message);
    }, popupInfo);
  },

  /**
   * Set rack id for host
   * @method setRackIdForHost
   */
  setRackIdForHost: function setRackIdForHost() {
    var hostNames = [{ hostName: this.get('content.hostName') }];
    var rack = this.get('content.rack');
    var operationData = { message: Em.I18n.t('hosts.host.details.setRackId') };
    hostsManagement.setRackInfo(operationData, hostNames, rack);
  },

  /**
   * Send request to get passive state for host
   * @param {string} state
   * @param {string} message
   * @method hostPassiveModeRequest
   */
  hostPassiveModeRequest: function hostPassiveModeRequest(state, message) {
    App.ajax.send({
      name: 'bulk_request.hosts.passive_state',
      sender: this,
      data: {
        hostNames: this.get('content.hostName'),
        passive_state: state,
        requestInfo: message
      },
      success: 'updateHost'
    });
  },

  /**
   * Success callback for receiving host passive state
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method updateHost
   */
  updateHost: function updateHost(data, opt, params) {
    this.set('content.passiveState', params.passive_state);
    batchUtils.infoPassiveState(params.passive_state);
  },

  /**
   * Send request to get passive state for hostComponent
   * @param {object} component - hostComponentn object
   * @param {string} state
   * @param {string} message
   * @method hostPassiveModeRequest
   */
  updateComponentPassiveState: function updateComponentPassiveState(component, state, message) {
    App.ajax.send({
      name: 'common.host.host_component.passive',
      sender: this,
      data: {
        hostName: this.get('content.hostName'),
        componentName: component.get('componentName'),
        component: component,
        passive_state: state,
        context: message
      },
      success: 'updateHostComponent'
    });
  },

  /**
   * Success callback for receiving hostComponent passive state
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method updateHost
   */
  updateHostComponent: function updateHostComponent(data, opt, params) {
    params.component.set('passiveState', params.passive_state);
    batchUtils.infoPassiveState(params.passive_state);
  },

  /**
   * Show confirmation popup for action "start all components"
   * @method doStartAllComponents
   */
  doStartAllComponents: function doStartAllComponents() {
    var self = this;
    var components = this.get('serviceNonClientActiveComponents');

    if (components && components.get('length')) {
      return App.showConfirmationPopup(function () {
        self.sendComponentCommand(components, Em.I18n.t('hosts.host.maintainance.startAllComponents.context'), App.HostComponentStatus.started);
      }, Em.I18n.t('question.sure.startAll'));
    }
  },

  /**
   * Show confirmation popup for action "stop all components"
   * @method doStopAllComponents
   */
  doStopAllComponents: function doStopAllComponents() {
    var self = this;
    var components = this.get('serviceNonClientActiveComponents');

    if (components && components.get('length')) {
      if (components.someProperty('componentName', 'NAMENODE') && this.get('content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
        this.checkNnLastCheckpointTime(function () {
          App.showConfirmationPopup(function () {
            self.sendComponentCommand(components, Em.I18n.t('hosts.host.maintainance.stopAllComponents.context'), App.HostComponentStatus.stopped);
          }, Em.I18n.t('question.sure.stopAll'));
        });
      } else {
        return App.showConfirmationPopup(function () {
          self.sendComponentCommand(components, Em.I18n.t('hosts.host.maintainance.stopAllComponents.context'), App.HostComponentStatus.stopped);
        }, Em.I18n.t('question.sure.stopAll'));
      }
    }
  },

  /**
   * Show confirmation popup for action "restart all components"
   * @method doRestartAllComponents
   */
  doRestartAllComponents: function doRestartAllComponents() {
    var self = this;
    var components = this.get('serviceActiveComponents');

    if (components && components.get('length')) {
      if (components.someProperty('componentName', 'NAMENODE') && this.get('content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
        this.checkNnLastCheckpointTime(function () {
          App.showConfirmationPopup(function () {
            batchUtils.restartHostComponents(components, Em.I18n.t('rollingrestart.context.allOnSelectedHost').format(self.get('content.hostName')), "HOST");
          }, Em.I18n.t('question.sure.restartAll'));
        });
      } else {
        return App.showConfirmationPopup(function () {
          batchUtils.restartHostComponents(components, Em.I18n.t('rollingrestart.context.allOnSelectedHost').format(self.get('content.hostName')), "HOST");
        }, Em.I18n.t('question.sure.restartAll'));
      }
    }
  },

  /**
   * get info about host-components, exactly:
   *  - host-components grouped by status, features
   *  - flag, that indicate whether ZooKeeper Server is installed
   * @return {Object}
   */
  getHostComponentsInfo: function getHostComponentsInfo(hostComponents) {
    var componentsOnHost = hostComponents || this.get('content.hostComponents');
    var stoppedStates = [App.HostComponentStatus.stopped, App.HostComponentStatus.install_failed, App.HostComponentStatus.upgrade_failed, App.HostComponentStatus.init, App.HostComponentStatus.unknown];
    var container = {
      isReconfigureRequired: false,
      lastComponents: [],
      masterComponents: [],
      nonAddableMasterComponents: [],
      lastMasterComponents: [],
      runningComponents: [],
      nonDeletableComponents: [],
      unknownComponents: [],
      toDecommissionComponents: []
    };
    var addDeleteComponentsMap = this.get('addDeleteComponentsMap');

    if (componentsOnHost && componentsOnHost.get('length') > 0) {
      componentsOnHost.forEach(function (cInstance) {
        if (addDeleteComponentsMap[cInstance.get('componentName')]) {
          container.isReconfigureRequired = true;
        }
        var isLastComponent = false;
        if (this.getTotalComponent(cInstance) === 1) {
          container.lastComponents.push(cInstance.get('displayName'));
          isLastComponent = true;
        }
        var workStatus = cInstance.get('workStatus');

        if (cInstance.get('isMaster')) {
          var displayName = cInstance.get('displayName');
          container.masterComponents.push(displayName);
          if (!App.StackServiceComponent.find(cInstance.get('componentName')).get('isMasterAddableInstallerWizard')) {
            container.nonAddableMasterComponents.push(displayName);
          }
          if (isLastComponent) {
            container.lastMasterComponents.push(displayName);
          }
        }
        if (stoppedStates.indexOf(workStatus) < 0) {
          container.runningComponents.push(cInstance.get('displayName'));
        }
        if (!cInstance.get('isDeletable')) {
          container.nonDeletableComponents.push(cInstance.get('displayName'));
        }
        if (workStatus === App.HostComponentStatus.unknown) {
          container.unknownComponents.push(cInstance.get('displayName'));
        }
        if (App.get('components.decommissionAllowed').contains(cInstance.get('componentName')) && !cInstance.get('view.isComponentRecommissionAvailable')) {
          container.toDecommissionComponents.push(cInstance.get('displayName'));
        }
      }, this);
    }
    return container;
  },

  /**
   * Run host check confirmation
   * @method runHostCheckConfirmation
   */
  runHostCheckConfirmation: function runHostCheckConfirmation() {
    var self = this;
    var popupInfo = Em.I18n.t('hosts.checkHost.popup').format(this.get('content.hostName'));

    return App.showConfirmationPopup(function () {
      self.runHostCheck();
    }, popupInfo);
  },

  getDataForHostCheck: function getDataForHostCheck() {
    var hostName = this.get('content.hostName');
    var jdk_location = App.router.get('clusterController.ambariProperties.jdk_location');
    var RequestInfo = {
      "action": "check_host",
      "context": "Check host",
      "parameters": {
        "hosts": hostName,
        "check_execute_list": "last_agent_env_check,installed_packages,existing_repos,transparentHugePage",
        "jdk_location": jdk_location,
        "threshold": "20"
      }
    };

    return {
      RequestInfo: RequestInfo,
      resource_filters: { "hosts": hostName }
    };
  },

  /**
   * Callback for runHostCheckConfirmation
   * @method runHostCheck
   */
  runHostCheck: function runHostCheck() {
    var dataForCheckHostRequest = this.getDataForHostCheck();

    this.setProperties({
      stopChecking: false,
      checkHostFinished: false,
      isRerun: false
    });
    this.setBootHostsProp();
    this.showHostWarningsPopup();
    this.requestToPerformHostCheck(dataForCheckHostRequest);
  },

  /**
   * Shape controller's bootHosts property needed to host check
   * @method setBootHostsProp
   */
  setBootHostsProp: function setBootHostsProp() {
    var host = this.get('content');
    var bootHosts = [];

    host.name = host.get('hostName');
    bootHosts.push(host);

    this.set('bootHosts', bootHosts);
  },

  /**
   * Open popup that contain hosts' warnings
   * @return {App.ModalPopup}
   * @method showHostWarningsPopup
   */
  showHostWarningsPopup: function showHostWarningsPopup() {
    var self = this;

    return App.ModalPopup.show({

      header: Em.I18n.t('installer.step3.warnings.popup.header'),

      secondary: Em.I18n.t('installer.step3.hostWarningsPopup.rerunChecks'),

      primary: Em.I18n.t('common.close'),

      autoHeight: false,

      onPrimary: function onPrimary() {
        self.set('checksUpdateStatus', null);
        this.hide();
      },

      onClose: function onClose() {
        self.set('checksUpdateStatus', null);
        this.hide();
      },

      onSecondary: function onSecondary() {
        self.set('checkHostFinished', false);
        self.rerunChecks();
      },

      didInsertElement: function didInsertElement() {
        this._super();
        this.fitHeight();
      },

      footerClass: App.WizardStep3HostWarningPopupFooter.reopen({
        footerControllerBinding: 'App.router.mainHostDetailsController',
        checkHostFinished: function () {
          return this.get('footerController.checkHostFinished');
        }.property('footerController.checkHostFinished')
      }),

      bodyClass: App.WizardStep3HostWarningPopupBody.reopen({
        bodyControllerBinding: 'App.router.mainHostDetailsController',
        checkHostFinished: function () {
          return this.get('bodyController.checkHostFinished');
        }.property('bodyController.checkHostFinished')
      })
    });
  },

  /**
   * Deletion of hosts not supported for this version
   * @method validateAndDeleteHost
   */
  validateAndDeleteHost: function validateAndDeleteHost() {
    var container = this.getHostComponentsInfo();

    if (container.nonDeletableComponents.length > 0) {
      this.raiseDeleteComponentsError(container, 'nonDeletableList');
      return;
    } else if (container.nonAddableMasterComponents.length > 0) {
      this.raiseDeleteComponentsError(container, 'masterList');
      return;
    } else if (container.runningComponents.length > 0) {
      this.raiseDeleteComponentsError(container, 'runningList');
      return;
    } else if (container.lastMasterComponents.length > 0) {
      this.raiseDeleteComponentsError(container, 'lastMasterList');
      return;
    }

    this.set('fromDeleteHost', true);

    if (container.isReconfigureRequired) {
      this.reconfigureAndDeleteHost(container);
    } else {
      this.confirmDeleteHost(container);
    }
  },

  /**
   *
   * @param {object} container
   */
  reconfigureAndDeleteHost: function reconfigureAndDeleteHost(container) {
    var _this10 = this;

    var addDeleteComponentsMap = this.get('addDeleteComponentsMap');
    var hostName = this.get('content.hostName');
    var reconfiguredComponents = [];
    var componentStub = Em.Object.create();

    this.get('content.hostComponents').forEach(function (component) {
      var componentsMapItem = addDeleteComponentsMap[component.get('componentName')];
      if (componentsMapItem) {
        reconfiguredComponents.push(component.get('displayName'));
        if (componentsMapItem.hostPropertyName) {
          _this10.set(componentsMapItem.hostPropertyName, hostName);
        }
        if (componentsMapItem.addPropertyName) {
          _this10.set(componentsMapItem.addPropertyName, true);
        }
        _this10.loadComponentRelatedConfigs(componentsMapItem.configTagsCallbackName, componentsMapItem.configsCallbackName);
      }
    });
    this.showReconfigurationPopupPreDelete(componentStub, function () {
      _this10.confirmDeleteHost(container);
    }, Em.I18n.t('hosts.host.delete.componentsRequireReconfigure').format(reconfiguredComponents.join(', ')));
  },

  /**
   * Show popup with info about reasons why host can't be deleted
   * @param {string[]} components
   * @param {string} type
   * @method raiseDeleteComponentsError
   */
  raiseDeleteComponentsError: function raiseDeleteComponentsError(container, type) {
    App.ModalPopup.show({
      header: Em.I18n.t('hosts.cant.do.popup.title'),
      type: type,
      showBodyEnd: Em.computed.existsIn('type', ['runningList', 'masterList', 'lastMasterList']),
      container: container,
      components: function () {
        var container = this.get('container');
        switch (this.get('type')) {
          case 'masterList':
            return container.nonAddableMasterComponents;
          case 'nonDeletableList':
            return container.nonDeletableComponents;
          case 'runningList':
            return container.runningComponents;
          case 'lastMasterList':
            return container.lastMasterComponents;
        }
      }.property('type'),
      componentsStr: function () {
        return this.get('components').join(", ");
      }.property('components.[]'),
      componentsBody: Em.computed.i18nFormat('hosts.cant.do.popup.' + type + '.body', 'components.length'),
      componentsBodyEnd: function () {
        if (this.get('showBodyEnd')) {
          return Em.I18n.t('hosts.cant.do.popup.' + type + '.body.end').format(App.get('components.decommissionAllowed').map(function (c) {
            return App.format.role(c, false);
          }).join(", "));
        }
        return '';
      }.property(),
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/details/raiseDeleteComponentErrorPopup')
      }),
      secondary: null
    });
  },

  /**
   * Show confirmation popup to delete host
   * @param {Object} container
   * @method confirmDeleteHost
   */
  confirmDeleteHost: function confirmDeleteHost(container) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('hosts.delete.popup.title'),
      deletePopupBody: Em.I18n.t('hosts.delete.popup.body').format(self.get('content.publicHostName')),
      lastComponent: function () {
        if (container.lastComponents && container.lastComponents.length) {
          this.set('isChecked', false);
          return true;
        } else {
          this.set('isChecked', true);
          return false;
        }
      }.property(),
      disablePrimary: Em.computed.not('isChecked'),
      isChecked: false,
      lastComponentError: Em.View.extend({
        template: Em.Handlebars.compile(Em.I18n.t('hosts.delete.popup.body.msg4').format(container.lastComponents))
      }),
      unknownComponents: function () {
        if (container.unknownComponents && container.unknownComponents.length) {
          return container.unknownComponents.join(", ");
        }
        return '';
      }.property(),
      decommissionWarning: Em.View.extend({
        template: Em.Handlebars.compile(Em.I18n.t('hosts.delete.popup.body.msg7').format(container.toDecommissionComponents.join(', ')))
      }),
      toDecommissionComponents: container.toDecommissionComponents,
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/details/doDeleteHostPopup')
      }),
      onPrimary: function onPrimary() {
        this.hide();
        self.doDeleteHost();
      }
    });
  },

  /**
   * send DELETE calls to components of host and after delete host itself
   * @method doDeleteHost
   */
  doDeleteHost: function doDeleteHost() {
    var _this11 = this;

    var allComponents = this.get('content.hostComponents');
    var addDeleteComponentsMap = this.get('addDeleteComponentsMap');
    var deleteRequests = [];
    var deleteError = null;
    var dfd = $.Deferred();

    if (allComponents.get('length') > 0) {
      allComponents.forEach(function (component) {
        deleteRequests.push(this._doDeleteHostComponent(component.get('componentName')));
      }, this);
      $.when(deleteRequests).done(function () {
        if (_this11.get('isReconfigureRequired')) {
          var reconfiguredComponents = allComponents.filter(function (component) {
            return addDeleteComponentsMap[component.get('componentName')];
          }).mapProperty('displayName').join(', ');
          _this11.applyConfigsCustomization();
          _this11.putConfigsToServer(_this11.get('groupedPropertiesToChange'), reconfiguredComponents);
          _this11.clearConfigsChanges();
        }
        _this11.deleteHostCall();
      }).fail(function () {
        deleteError = _this11.get('_deletedHostComponentError');
        deleteError.xhr.responseText = "{\"message\": \"" + deleteError.xhr.statusText + "\"}";
        App.ajax.defaultErrorHandler(deleteError.xhr, deleteError.url, deleteError.type, deleteError.xhr.status);
      }).always(dfd.resolve);
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  deleteHostCall: function deleteHostCall() {
    return App.ajax.send({
      name: 'common.delete.host',
      sender: this,
      data: {
        hostName: this.get('content.hostName')
      },
      success: 'deleteHostCallSuccessCallback',
      error: 'deleteHostCallErrorCallback',
      showLoadingPopup: true
    });
  },

  deleteHostCallSuccessCallback: function deleteHostCallSuccessCallback(data, rq, requestBody) {
    App.router.get('updateController').updateHost(function () {
      App.router.transitionTo('hosts.index');
    });
    if (!!(requestBody && requestBody.hostName)) {
      var remainingHosts = App.db.getSelectedHosts().removeObject(requestBody.hostName);
      App.db.setSelectedHosts(remainingHosts);
      App.hostsMapper.deleteRecord(App.Host.find().findProperty('hostName', requestBody.hostName));
    }
    App.router.get('clusterController').getAllHostNames();
  },
  deleteHostCallErrorCallback: function deleteHostCallErrorCallback(xhr, textStatus, errorThrown, opt) {
    xhr.responseText = "{\"message\": \"" + xhr.statusText + "\"}";
    App.ajax.defaultErrorHandler(xhr, opt.url, 'DELETE', xhr.status);
  },

  /**
   * Send command to server to restart all host components with stale configs
   * @method restartAllStaleConfigComponents
   */
  restartAllStaleConfigComponents: function restartAllStaleConfigComponents() {
    var self = this;
    var staleComponents = self.get('content.componentsWithStaleConfigs');
    if (staleComponents.someProperty('componentName', 'NAMENODE') && this.get('content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
      this.checkNnLastCheckpointTime(function () {
        App.showConfirmationPopup(function () {
          batchUtils.restartHostComponents(staleComponents, Em.I18n.t('rollingrestart.context.allWithStaleConfigsOnSelectedHost').format(self.get('content.hostName')), "HOST");
        });
      });
    } else {
      return App.showConfirmationPopup(function () {
        batchUtils.restartHostComponents(staleComponents, Em.I18n.t('rollingrestart.context.allWithStaleConfigsOnSelectedHost').format(self.get('content.hostName')), "HOST");
      });
    }
  },

  /**
   * open Reassign Master Wizard with selected component
   * @param {object} event
   * @method moveComponent
   */
  moveComponent: function moveComponent(event) {
    var component = event.context;
    if ($(event.target).closest('li').hasClass('disabled')) {
      return;
    }
    return App.showConfirmationPopup(function () {
      var reassignMasterController = App.router.get('reassignMasterController');
      reassignMasterController.saveComponentToReassign(component);
      reassignMasterController.setCurrentStep('1');
      App.router.transitionTo('reassign');
    }, Em.I18n.t('question.sure.move').format(component.get('displayName')));
  },

  /**
   * Restart clients host components to apply config changes
   * @param {object} event
   * @method refreshConfigs
   */
  refreshConfigs: function refreshConfigs(event) {
    var self = this;
    var component = event.context;
    if (!Em.isNone(component)) {
      return App.showConfirmationPopup(function () {
        var message = Em.I18n.t('rollingrestart.context.ClientOnSelectedHost').format(component.get('displayName'), self.get('content.hostName'));
        batchUtils.restartHostComponents([component], message, "HOST");
      }, Em.I18n.t('question.sure.refresh').format(component.get('displayName'), self.get('content.hostName')));
    }
  },

  toggleMaintenanceMode: function toggleMaintenanceMode(event) {
    var state,
        message,
        self = this;
    if (event.context.get('isImpliedState')) return null;
    state = event.context.get('passiveState') === "ON" ? "OFF" : "ON";
    message = Em.I18n.t('passiveState.turn' + state.toCapital() + 'For').format(event.context.get('displayName'));
    return App.showConfirmationPopup(function () {
      self.updateComponentPassiveState(event.context, state, message);
    }, Em.I18n.t('question.sure.maintenance').format(state.toLowerCase(), event.context.get('displayName')));
  },

  downloadClientConfigs: function downloadClientConfigs(event) {
    this.downloadClientConfigsCall({
      hostName: event.context.get('hostName'),
      componentName: event.context.get('componentName'),
      resourceType: this.resourceTypeEnum.HOST_COMPONENT
    });
  },

  /**
   *  This controller action is called from the template when user clicks to download configs for "All Clients On Host"
   */
  downloadAllClientConfigs: function downloadAllClientConfigs() {
    var self = this;
    this.downloadClientConfigsCall({
      hostName: self.get('content.hostName'),
      resourceType: this.resourceTypeEnum.HOST
    });
  },

  installClients: function installClients(components) {
    var clientsToInstall = [],
        clientsToAdd = [],
        missedComponents = [],
        dependentComponents = [],
        self = this;
    components.forEach(function (component) {
      if (['INIT', 'INSTALL_FAILED'].contains(component.get('workStatus'))) {
        clientsToInstall.push(component);
      } else if (typeof component.get('workStatus') == 'undefined') {
        clientsToAdd.push(component);
      }
    });
    clientsToAdd.forEach(function (component, index, array) {
      var dependencies = this.checkComponentDependencies(component.get('componentName'), {
        scope: 'host',
        installedComponents: this.get('content.hostComponents').mapProperty('componentName')
      }).reject(function (componentName) {
        return array.mapProperty('componentName').contains(componentName);
      });
      if (dependencies.length) {
        missedComponents.pushObjects(dependencies);
        dependentComponents.push(component.get('displayName'));
      }
    }, this);
    missedComponents = missedComponents.uniq();
    if (missedComponents.length) {
      var popupMessage = Em.I18n.t('host.host.addComponent.popup.clients.dependedComponents.body').format(stringUtils.getFormattedStringFromArray(dependentComponents), stringUtils.getFormattedStringFromArray(missedComponents.map(function (componentName) {
        return App.StackServiceComponent.find(componentName).get('displayName');
      })));
      App.showAlertPopup(Em.I18n.t('host.host.addComponent.popup.dependedComponents.header'), popupMessage);
    } else {
      App.get('router.mainAdminKerberosController').getSecurityType(function () {
        App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
          var sendInstallCommand = function sendInstallCommand() {
            if (clientsToInstall.length) {
              self.sendComponentCommand(clientsToInstall, Em.I18n.t('host.host.details.installClients'), 'INSTALLED');
            }
          };
          if (clientsToAdd.length) {
            var message = stringUtils.getFormattedStringFromArray(clientsToAdd.mapProperty('displayName')),
                componentObject = Em.Object.create({
              displayName: message
            });
            self.showAddComponentPopup(componentObject, self.get('content.hostName'), function () {
              sendInstallCommand();
              clientsToAdd.forEach(function (component) {
                self.installHostComponentCall(self.get('content.hostName'), component);
              });
            });
          } else {
            sendInstallCommand();
          }
        });
      }.bind(this));
    }
  },

  /**
   * Check if all required components are installed on host.
   * Available options:
   *  scope: 'host' - dependency level `host`,`cluster` or `*`.
   *  hostName: 'example.com' - host name to search installed components
   *  installedComponents: ['A', 'B'] - names of installed components
   *
   * By default scope level is `*`
   * For host level dependency you should specify at least `hostName` or `installedComponents` attribute.
   *
   * @param {String} componentName
   * @param {Object} opt - options. Allowed options are `hostName`, `installedComponents`, `scope`.
   * @return {Array} - names of missed components
   */
  checkComponentDependencies: function checkComponentDependencies(componentName, opt) {
    opt = opt || {};
    opt.scope = opt.scope || '*';
    var installedComponents;
    switch (opt.scope) {
      case 'host':
        Em.assert("You should pass at least `hostName` or `installedComponents` to options.", opt.hostName || opt.installedComponents);
        installedComponents = opt.installedComponents || App.HostComponent.find().filterProperty('hostName', opt.hostName).mapProperty('componentName').uniq();
        break;
      default:
        // @todo: use more appropriate value regarding installed components
        installedComponents = opt.installedComponents || App.HostComponent.find().mapProperty('componentName').uniq();
        break;
    }
    var component = App.StackServiceComponent.find(componentName);
    return component.missingDependencies(installedComponents, opt).map(function (componentDependency) {
      return componentDependency.chooseCompatible();
    });
  },

  /**
   * On click handler for custom command from items menu
   * @param context
   */
  executeCustomCommand: function executeCustomCommand(event) {
    var controller = this;
    var context = event.context;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'service.item.executeCustomCommand',
        sender: controller,
        data: {
          command: context.command,
          context: context.context || Em.I18n.t('services.service.actions.run.executeCustomCommand.context').format(context.command),
          hosts: context.hosts,
          serviceName: context.service,
          componentName: context.component
        },
        success: 'executeCustomCommandSuccessCallback',
        error: 'executeCustomCommandErrorCallback'
      });
    });
  },

  executeCustomCommandSuccessCallback: function executeCustomCommandSuccessCallback(data, ajaxOptions, params) {
    if (data.Requests.id) {
      App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
        if (initValue) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      });
    }
  },
  executeCustomCommandErrorCallback: function executeCustomCommandErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.executeCustomCommand.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.executeCustomCommand.error'), error);
  },

  /**
   * Call callback after loading service metrics
   * @param callback
   */
  isServiceMetricsLoaded: function isServiceMetricsLoaded(callback) {
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(callback);
  },

  setConfigsChangesForDisplay: function setConfigsChangesForDisplay() {
    if (App.get('router.clusterController.isConfigsPropertiesLoaded')) {
      this.get('allPropertiesToChange').forEach(function (property) {
        var stackProperty = App.configsCollection.getConfigByName(property.propertyName, property.propertyFileName);
        if (stackProperty && (!stackProperty.isEditable || !stackProperty.isReconfigurable)) {
          this.get('requiredPropertiesToChange').pushObject(property);
        } else {
          Em.set(property, 'saveRecommended', true);
          this.get('recommendedPropertiesToChange').pushObject(property);
        }
      }, this);
      this.set('isConfigsLoadingInProgress', false);
      this.removeObserver('App.router.clusterController.isConfigsPropertiesLoaded', this, 'setConfigsChangesForDisplay');
    }
  },

  setConfigsChanges: function setConfigsChanges(groups) {
    this.get('groupedPropertiesToChange').pushObjects(groups);
    if (this.get('allPropertiesToChange.length')) {
      if (App.get('router.clusterController.isConfigsPropertiesLoaded')) {
        this.setConfigsChangesForDisplay();
      } else {
        this.addObserver('App.router.clusterController.isConfigsPropertiesLoaded', this, 'setConfigsChangesForDisplay');
      }
    } else {
      this.set('isConfigsLoadingInProgress', false);
    }
  },

  recoverHost: function recoverHost() {
    var components = this.get('content.hostComponents');
    var hostName = this.get('content.hostName');
    var self = this;
    var batches = [{
      "order_id": 1,
      "type": "PUT",
      "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hostName + "/host_components",
      "RequestBodyInfo": {
        "RequestInfo": {
          context: Em.I18n.t('hosts.host.recover.initAllComponents.context'),
          operation_level: {
            level: "HOST",
            cluster_name: App.get('clusterName'),
            host_name: hostName
          },
          query: "HostRoles/component_name.in(" + components.mapProperty('componentName').join(',') + ")"
        },
        "Body": {
          HostRoles: {
            state: "INIT"
          }
        }
      }
    }];
    batches.push({
      "order_id": 2,
      "type": "PUT",
      "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hostName + "/host_components",
      "RequestBodyInfo": {
        "RequestInfo": {
          context: Em.I18n.t('hosts.host.recover.installAllComponents.context'),
          operation_level: {
            level: "HOST",
            cluster_name: App.get('clusterName'),
            host_name: hostName
          },
          query: "HostRoles/component_name.in(" + components.mapProperty('componentName').join(',') + ")"
        },
        "Body": {
          HostRoles: {
            state: "INSTALLED"
          }
        }
      }
    });

    if (App.get('isKerberosEnabled')) {
      batches.push({
        "order_id": 3,
        "type": "PUT",
        "uri": "/clusters/" + App.get('clusterName'),
        "RequestBodyInfo": {
          "RequestInfo": {
            context: Em.I18n.t('hosts.host.recover.regenerateKeytabs.context'),
            query: "regenerate_keytabs=all&regenerate_hosts=" + hostName + "&config_update_policy=none"
          },
          "Body": {
            Clusters: {
              security_type: "KERBEROS"
            }
          }
        }
      });
    }
    App.get('router.mainAdminKerberosController').getSecurityType(function () {
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        self._doRecoverHost(batches);
      });
    });
  },

  _doRecoverHost: function _doRecoverHost(batches) {
    App.ajax.send({
      name: 'common.batch.request_schedules',
      sender: this,
      data: {
        intervalTimeSeconds: 1,
        tolerateSize: 0,
        batches: batches
      },
      success: 'recoverHostSuccessCallback',
      showLoadingPopup: true
    });
  },

  recoverHostSuccessCallback: function recoverHostSuccessCallback(data) {
    if (data && (data.Requests || data.resources[0].RequestSchedule)) {
      this.showBackgroundOperationsPopup();
      return true;
    } else {
      return false;
    }
  },

  recoverHostDisabled: function () {

    var isDisabled = false;
    var allowedStates = [App.HostComponentStatus.stopped, App.HostComponentStatus.install_failed, App.HostComponentStatus.init];
    this.get('content.hostComponents').forEach(function (component) {
      isDisabled = isDisabled ? true : !allowedStates.contains(component.get('workStatus'));
    });
    return isDisabled;
  }.property('content.hostComponents.@each.workStatus'),

  confirmRecoverHost: function confirmRecoverHost() {
    var self = this;
    var componentsNotStopped = [];
    var allowedStates = [App.HostComponentStatus.stopped, App.HostComponentStatus.install_failed, App.HostComponentStatus.init];
    this.get('content.hostComponents').forEach(function (component) {
      if (!allowedStates.contains(component.get('workStatus'))) {
        componentsNotStopped.push(component.get('componentName'));
      }
    });
    if (componentsNotStopped.length) {
      App.ModalPopup.show({
        header: Em.I18n.t('hosts.recover.error.popup.title'),
        recoverErrorPopupBody: Em.I18n.t('hosts.recover.error.popup.body').format(componentsNotStopped.toString()),
        componentsStr: componentsNotStopped.toString(),
        bodyClass: Em.View.extend({
          templateName: require('templates/main/host/details/recoverHostErrorPopup')
        }),
        secondary: false
      });
    } else {
      App.ModalPopup.show({
        header: Em.I18n.t('hosts.recover.popup.title'),
        bodyClass: Em.View.extend({
          templateName: require('templates/main/host/details/recoverHostPopup')
        }),
        primary: Em.I18n.t('yes'),
        secondary: Em.I18n.t('no'),
        onPrimary: function onPrimary() {
          self.recoverHost();
          this.hide();
        }
      });
    }
  },

  regenerateKeytabFileOperations: function regenerateKeytabFileOperations() {
    var self = this;
    var hostName = this.content.get('hostName');
    var clusterName = App.get('clusterName');
    return App.showConfirmationPopup(function () {
      return App.ajax.send({
        name: "admin.kerberos_security.regenerate_keytabs.host",
        sender: self,
        data: {
          clusterName: clusterName,
          hostName: hostName
        },
        success: 'regenerateKeytabFileOperationsRequestSuccess',
        error: 'regenerateKeytabFileOperationsRequestError'
      });
    }, Em.I18n.t('question.sure.regenerateKeytab.host').format(hostName));
  },

  regenerateKeytabFileOperationsRequestSuccess: function regenerateKeytabFileOperationsRequestSuccess() {
    App.router.get('backgroundOperationsController').showPopup();
  },

  regenerateKeytabFileOperationsRequestError: function regenerateKeytabFileOperationsRequestError() {
    App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('alerts.notifications.regenerateKeytab.host.error').format(this.content.get('hostName')));
  },

  /**
   * Returns URL parameters for configs request by certain tags
   * @param {object} data - object with desired configs tags received from API
   * @param {string[]} configTypes - list of config types
   * @returns {string}
   */
  getUrlParamsForConfigsRequest: function getUrlParamsForConfigsRequest(data, configTypes) {
    return configTypes.map(function (type) {
      var tag = Em.get(data, 'Clusters.desired_configs.' + type + '.tag');
      return tag ? '(type=' + type + '&tag=' + tag + ')' : null;
    }).compact().join('|');
  }

});

});

require.register("controllers/main/host/host_alerts_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostAlertsController = Em.ArrayController.extend({
  name: 'mainHostAlertsController',

  selectedHost: Em.computed.alias('App.router.mainHostDetailsController.content'),

  /**
   * List of all <code>App.AlertInstance</code> by Host
   * @type {App.AlertInstance[]}
   */
  content: function () {
    return App.AlertInstanceLocal.find().toArray().filterProperty('host', this.get('selectedHost'));
  }.property('App.router.mainAlertInstancesController.isLoaded', 'selectedHost'),

  /**
   * Open details page of the selected alertDefinition
   * @param {object} event
   * @method routeToAlertDefinition
   */
  routeToAlertDefinition: function routeToAlertDefinition(event) {
    var alertDefinition = App.AlertDefinition.find(event.context);
    App.router.transitionTo('main.alerts.alertDetails', alertDefinition);
  }
});

});

require.register("controllers/main/service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var misc = require('utils/misc');

App.MainServiceController = Em.ArrayController.extend(App.SupportClientConfigsDownload, {

  name: 'mainServiceController',

  /**
   * @type {Ember.Object[]}
   */
  content: function () {
    if (!App.router.get('clusterController.isLoaded')) {
      return [];
    }
    return misc.sortByOrder(App.StackService.find().mapProperty('serviceName'), App.Service.find().toArray());
  }.property('App.router.clusterController.isLoaded').volatile(),

  /**
   * Current cluster
   * @type {Ember.Object}
   */
  cluster: function () {
    if (!App.router.get('clusterController.isClusterDataLoaded')) {
      return null;
    }
    return App.Cluster.find().objectAt(0);
  }.property('App.router.clusterController.isClusterDataLoaded'),

  /**
   * Check if all services are installed
   * true - all installed, false - not all
   * @type {bool}
   */
  isAllServicesInstalled: function () {
    if (!this.get('content')) return false;
    var notAvailableServices = App.ServiceSimple.find().filterProperty('doNotShowAndInstall').mapProperty('name');
    var availableServices = App.ServiceSimple.find().filterProperty('doNotShowAndInstall', false);
    var installedServices = this.get('content').filter(function (service) {
      return !notAvailableServices.contains(service.get('serviceName'));
    });
    return installedServices.length == availableServices.length;
  }.property('content.@each', 'content.length'),

  /**
   * Should "Start All"-button be disabled
   * @type {bool}
   */
  isStartAllDisabled: function () {
    if (this.get('isStartStopAllClicked') == true) {
      return true;
    }
    var stoppedServices = this.get('content').filter(function (_service) {
      return _service.get('healthStatus') === 'red' && !App.get('services.clientOnly').contains(_service.get('serviceName'));
    });
    return stoppedServices.length === 0; // all green status
  }.property('isStartStopAllClicked', 'content.@each.healthStatus'),

  /**
   * Should "Stop All"-button be disabled
   * @type {bool}
   */
  isStopAllDisabled: function () {
    if (this.get('isStartStopAllClicked') == true) {
      return true;
    }
    return !this.get('content').someProperty('healthStatus', 'green');
  }.property('isStartStopAllClicked', 'content.@each.healthStatus'),

  /**
   * Should "Refresh All"-button be disabled
   * @type {bool}
   */
  isRestartAllRequiredDisabled: Em.computed.everyBy('content', 'isRestartRequired', false),

  /**
   * @type {bool}
   */
  isStartStopAllClicked: Em.computed.notEqual('App.router.backgroundOperationsController.runningOperationsCount', 0),

  /**
   * Callback for <code>start all service</code> button
   * @return {App.ModalPopup|null}
   * @method startAllService
   */
  startAllService: function startAllService(event) {
    return this.startStopAllService(event, 'STARTED');
  },

  /**
   * Callback for <code>stop all service</code> button
   * @return {App.ModalPopup|null}
   * @method stopAllService
   */
  stopAllService: function stopAllService(event) {
    return this.startStopAllService(event, 'INSTALLED');
  },

  /**
   * Common method for "start-all", "stop-all" calls
   * @param {object} event
   * @param {string} state 'STARTED|INSTALLED'
   * @returns {App.ModalPopup|null}
   * @method startStopAllService
   */
  startStopAllService: function startStopAllService(event, state) {
    if ($(event.target).hasClass('disabled') || $(event.target.parentElement).hasClass('disabled')) {
      return null;
    }
    var self = this;
    var bodyMessage = Em.Object.create({
      confirmMsg: state == 'INSTALLED' ? Em.I18n.t('services.service.stopAll.confirmMsg') : Em.I18n.t('services.service.startAll.confirmMsg'),
      confirmButton: state == 'INSTALLED' ? Em.I18n.t('services.service.stop.confirmButton') : Em.I18n.t('services.service.start.confirmButton')
    });

    if (state == 'INSTALLED' && App.Service.find().filterProperty('serviceName', 'HDFS').someProperty('workStatus', App.HostComponentStatus.started)) {
      App.router.get('mainServiceItemController').checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query) {
          self.allServicesCall(state, query);
        }, bodyMessage);
      });
    } else {
      return App.showConfirmationFeedBackPopup(function (query) {
        self.allServicesCall(state, query);
      }, bodyMessage);
    }
  },

  /**
   * Download client configs for all services
   */
  downloadAllClientConfigs: function downloadAllClientConfigs() {
    this.downloadClientConfigsCall({ resourceType: this.resourceTypeEnum.CLUSTER });
  },

  /**
   * Do request to server for "start|stop" all services
   * @param {string} state "STARTED|INSTALLED"
   * @param {object} query
   * @method allServicesCall
   * @return {$.ajax}
   */
  allServicesCall: function allServicesCall(state, query) {
    var context = state == 'INSTALLED' ? App.BackgroundOperationsController.CommandContexts.STOP_ALL_SERVICES : App.BackgroundOperationsController.CommandContexts.START_ALL_SERVICES;
    return App.ajax.send({
      name: 'common.services.update',
      sender: this,
      data: {
        context: context,
        ServiceInfo: {
          state: state
        },
        query: query
      },
      success: 'allServicesCallSuccessCallback',
      error: 'allServicesCallErrorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Restart all services - restarts by sending one RESTART command
   */
  restartAllServices: function restartAllServices() {
    App.ajax.send({
      name: 'restart.allServices',
      sender: this,
      showLoadingPopup: true
    });
  },

  /**
   * Restart all services - stops all services, then starts them back
   */
  stopAndStartAllServices: function stopAndStartAllServices() {
    this.silentStopAllServices();
  },

  /**
   * Silent stop all services - without user confirmation
   * @returns {$.ajax}
   */
  silentStopAllServices: function silentStopAllServices() {
    return App.ajax.send({
      name: 'common.services.update',
      sender: this,
      data: {
        context: App.BackgroundOperationsController.CommandContexts.STOP_ALL_SERVICES,
        ServiceInfo: {
          state: 'INSTALLED'
        }
      },
      success: 'silentStopSuccess',
      showLoadingPopup: true
    });
  },

  isStopAllServicesFailed: function isStopAllServicesFailed() {
    var workStatuses = App.Service.find().mapProperty('workStatus');
    for (var i = 0; i < workStatuses.length; i++) {
      if (workStatuses[i] !== 'INSTALLED' && workStatuses[i] !== 'STOPPING') {
        return true;
      }
    }
    return false;
  },

  /**
   * Success callback for silent stop
   */
  silentStopSuccess: function silentStopSuccess() {
    var self = this;

    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }

      Em.run.later(function () {
        self.set('shouldStart', true);
      }, App.bgOperationsUpdateInterval);
    });
  },

  /**
   * Silent start all services - without user confirmation
   */
  silentStartAllServices: function () {
    if (!App.router.get('backgroundOperationsController').get('runningOperationsCount') && this.get('shouldStart') && !this.isStopAllServicesFailed()) {
      this.set('shouldStart', false);
      return App.ajax.send({
        name: 'common.services.update',
        sender: this,
        data: {
          context: App.BackgroundOperationsController.CommandContexts.START_ALL_SERVICES,
          ServiceInfo: {
            state: 'STARTED'
          }
        },
        success: 'silentCallSuccessCallback',
        showLoadingPopup: true
      });
    }
  }.observes('shouldStart', 'controllers.backgroundOperationsController.runningOperationsCount'),

  /**
   * Success callback for silent start
   */
  silentCallSuccessCallback: function silentCallSuccessCallback() {
    // load data (if we need to show this background operations popup) from persist
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  /**
   * Success-callback for all-services request
   * @param {object} data
   * @param {object} xhr
   * @param {object} params
   * @method allServicesCallSuccessCallback
   */
  allServicesCallSuccessCallback: function allServicesCallSuccessCallback(data, xhr, params) {
    params.query.set('status', 'SUCCESS');

    // load data (if we need to show this background operations popup) from persist
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  /**
   * Error-callback for all-services request
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   * @method allServicesCallErrorCallback
   */
  allServicesCallErrorCallback: function allServicesCallErrorCallback(request, ajaxOptions, error, opt, params) {
    params.query.set('status', 'FAIL');
    App.ajax.defaultErrorHandler(request, opt.url, opt.type, request.status);
  },

  /**
   * "Add-service"-click handler
   * @method gotoAddService
   */
  gotoAddService: function gotoAddService() {
    if (this.get('isAllServicesInstalled')) {
      return;
    }
    App.router.get('addServiceController').setDBProperty('onClosePath', 'main.services.index');
    App.router.transitionTo('main.serviceAdd');
  },

  /**
   * Show confirmation popup and send request to restart all host components with stale_configs=true
   */
  restartAllRequired: function restartAllRequired() {
    var self = this;
    if (!this.get('isRestartAllRequiredDisabled')) {
      return App.showConfirmationPopup(function () {
        self.restartHostComponents();
      }, Em.I18n.t('services.service.refreshAll.confirmMsg').format(App.Service.find().filterProperty('isRestartRequired').mapProperty('displayName').uniq().join(', ')), null, null, Em.I18n.t('services.service.restartAll.confirmButton'));
    } else {
      return null;
    }
  },

  /**
   * Send request restart host components from hostComponentsToRestart
   * @returns {$.ajax}
   */
  restartHostComponents: function restartHostComponents() {
    App.ajax.send({
      name: 'restart.staleConfigs',
      sender: this,
      success: 'restartAllRequiredSuccessCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Success callback for restartAllRequired
   */
  restartAllRequiredSuccessCallback: function restartAllRequiredSuccessCallback() {
    // load data (if we need to show this background operations popup) from persist
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  }
});

});

require.register("controllers/main/service/add_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
App.AddServiceController = App.WizardController.extend(App.AddSecurityConfigs, {

  name: 'addServiceController',

  totalSteps: 7,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('services.add.header'),

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  /**
   * @type {string}
   * @default null
   */
  serviceToInstall: null,

  /**
   *
   */
  installClientQueueLength: 0,

  areInstalledConfigGroupsLoaded: false,

  /**
   * All wizards data will be stored in this variable
   *
   * cluster - cluster name
   * installOptions - ssh key, repo info, etc.
   * services - services list
   * hosts - list of selected hosts
   * slaveComponentHosts, - info about slave hosts
   * masterComponentHosts - info about master hosts
   * config??? - to be described later
   */
  content: Em.Object.create({
    cluster: null,
    hosts: null,
    installOptions: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: null,
    advancedServiceConfig: null,
    controllerName: 'addServiceController',
    configGroups: [],
    clients: [],
    additionalClients: [],
    installedHosts: {}
  }),

  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.loadServices();
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        var self = this;
        this.loadHosts().done(function () {
          self.loadMasterComponentHosts().done(function () {
            self.load('hosts');
            self.loadRecommendations();
            dfd.resolve();
          });
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'async',
      callback: function callback() {
        var dfd = $.Deferred();
        var self = this;
        this.loadHosts().done(function () {
          self.loadServices();
          self.loadClients();
          self.loadSlaveComponentHosts(); //depends on loadServices
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '4': [{
      type: 'async',
      callback: function callback() {
        var self = this;
        var dfd = $.Deferred();
        this.load('cluster');
        this.set('content.additionalClients', []);
        this.set('installClientQueueLength', 0);
        this.set('installClietsQueue', App.ajaxQueue.create({ abortOnError: false }));
        this.loadKerberosDescriptorConfigs().done(function () {
          self.loadServiceConfigGroups();
          self.loadConfigThemes().then(function () {
            self.loadServiceConfigProperties();
            self.loadCurrentHostGroups();
            dfd.resolve();
          });
        });
        return dfd.promise();
      }
    }]
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
  },

  loadCurrentHostGroups: function loadCurrentHostGroups() {
    this.set("content.recommendationsHostGroups", this.getDBProperty('recommendationsHostGroups'));
  },

  /**
   * Load services data. Will be used at <code>Select services(step4)</code> step
   */
  loadServices: function loadServices() {
    var services = this.getDBProperty('services');
    var stackServices = App.StackService.find();
    if (services) {
      stackServices.forEach(function (item) {
        var isSelected = services.selectedServices.contains(item.get('serviceName')) || item.get('serviceName') == this.get('serviceToInstall');
        var isInstalled = services.installedServices.contains(item.get('serviceName'));
        item.set('isSelected', isSelected || (this.get("currentStep") == "1" ? isInstalled : isSelected));
        item.set('isInstalled', isInstalled);
      }, this);
      this.setSkipSlavesStep(App.StackService.find().filterProperty('isSelected').filterProperty('isInstalled', false), 3);
    } else {
      services = {
        selectedServices: [],
        installedServices: []
      };
      stackServices.forEach(function (item) {
        var isInstalled = App.Service.find().someProperty('id', item.get('serviceName'));
        var isSelected = item.get('serviceName') === this.get('serviceToInstall') || item.get('coSelectedServices').contains(this.get('serviceToInstall'));
        item.set('isSelected', isInstalled || isSelected);
        item.set('isInstalled', isInstalled);
        var serviceName = item.get('serviceName');
        if (isInstalled) {
          services.selectedServices.push(serviceName);
          services.installedServices.push(serviceName);
        } else {
          if (isSelected) {
            services.selectedServices.push(serviceName);
          }
        }
      }, this);
      this.setDBProperty('services', services);
    }
    App.store.fastCommit();
    this.set('serviceToInstall', null);
    this.set('content.services', stackServices);
    var self = this;
    this.loadServiceVersionFromVersionDefinitions().complete(function () {
      var serviceVersionsMap = self.get('serviceVersionsMap');
      stackServices.forEach(function (stackService) {
        var serviceName = stackService.get('serviceName');
        Em.set(stackService, 'serviceVersionDisplay', serviceVersionsMap[serviceName]);
      });
      self.set('content.services', stackServices);
    });
  },

  /**
   * Save data to model
   * @param stepController App.WizardStep4Controller
   */
  saveServices: function saveServices(stepController) {
    var services = {
      selectedServices: [],
      installedServices: []
    };
    var selectedServices = stepController.get('content').filterProperty('isSelected', true).filterProperty('isInstalled', false);
    var selectedServiceNames = selectedServices.mapProperty('serviceName');
    services.selectedServices.pushObjects(selectedServiceNames);
    services.installedServices.pushObjects(stepController.get('content').filterProperty('isInstalled', true).mapProperty('serviceName'));
    // save services that already installed but ignored on choose services page
    // these services marked by `isInstallable` flag with value `false`, for example `Kerberos` service
    services.installedServices.pushObjects(App.Service.find().mapProperty('serviceName').filter(function (serviceName) {
      return !services.installedServices.contains(serviceName);
    }));
    this.setDBProperty('services', services);

    this.set('content.selectedServiceNames', selectedServiceNames);
    this.setDBProperty('selectedServiceNames', selectedServiceNames);
    this.setSkipSlavesStep(selectedServices, 3);
  },

  /**
   * Save Master Component Hosts data to Main Controller
   * @param stepController App.WizardStep5Controller
   */
  saveMasterComponentHosts: function saveMasterComponentHosts(stepController) {
    var masterComponentHosts = [];
    var installedComponentsMap = App.HostComponent.find().toArray().toMapByProperty('componentName');

    stepController.get('selectedServicesMasters').forEach(function (_component) {
      var installedComponent = installedComponentsMap[_component.component_name];
      masterComponentHosts.push({
        display_name: _component.display_name,
        component: _component.component_name,
        hostName: _component.selectedHost,
        serviceId: _component.serviceId,
        isInstalled: !!installedComponent,
        workStatus: installedComponent && installedComponent.get('workStatus')
      });
    });

    this.setDBProperty('masterComponentHosts', masterComponentHosts);
    this.set('content.masterComponentHosts', masterComponentHosts);

    this.set('content.skipMasterStep', masterComponentHosts.everyProperty('isInstalled', true));
    this.get('isStepDisabled').findProperty('step', 2).set('value', this.get('content.skipMasterStep'));
  },

  /**
   * Load master component hosts data for using in required step controllers
   */
  loadMasterComponentHosts: function loadMasterComponentHosts() {
    var self = this,
        dfd = $.Deferred(),
        ASSIGN_MASTER_STEP = 2,
        DEPLOYMENT_STEPS = ['6', '7', '8'];
    this._super().done(function () {
      self.set('content.skipMasterStep', App.StackService.find().filterProperty('isSelected').filterProperty('hasMaster').everyProperty('isInstalled', true));
      self.get('isStepDisabled').findProperty('step', ASSIGN_MASTER_STEP).set('value', self.get('content.skipMasterStep') || DEPLOYMENT_STEPS.contains(self.get('currentStep')));
      dfd.resolve();
    });
    return dfd.promise();
  },

  /**
   * Does service have any configs
   * @param {string} serviceName
   * @returns {boolean}
   */
  isServiceNotConfigurable: function isServiceNotConfigurable(serviceName) {
    return App.get('services.noConfigTypes').contains(serviceName);
  },

  /**
   * Should Config Step be skipped (based on selected services list)
   * @returns {boolean}
   */
  skipConfigStep: function skipConfigStep() {
    var skipConfigStep = true;
    var selectedServices = this.get('content.services').filterProperty('isSelected', true).filterProperty('isInstalled', false).mapProperty('serviceName');
    selectedServices.map(function (serviceName) {
      skipConfigStep = skipConfigStep && this.isServiceNotConfigurable(serviceName);
    }, this);
    return skipConfigStep;
  },

  loadServiceConfigProperties: function loadServiceConfigProperties() {
    this._super();
    if (!this.get('content.services')) {
      this.loadServices();
    }
    if (this.get('currentStep') > 1 && this.get('currentStep') < 6) {
      this.set('content.skipConfigStep', this.skipConfigStep());
      this.get('isStepDisabled').findProperty('step', 4).set('value', this.get('content.skipConfigStep'));
    }
  },

  /**
   * Load kerberos descriptor configuration
   * @returns {$.Deferred}
   */
  loadKerberosDescriptorConfigs: function loadKerberosDescriptorConfigs() {
    var self = this,
        dfd = $.Deferred(),
        mergedDescriptorConfigs;
    if (App.get('isKerberosEnabled')) {
      this.loadClusterDescriptorStackConfigs().then(function (stackProperties) {
        self.loadClusterDescriptorConfigs().then(function (properties) {
          self.set('kerberosDescriptorData', properties);
          mergedDescriptorConfigs = self.mergeDescriptorStackWithConfigs(stackProperties, properties);
          self.set('kerberosDescriptorConfigs', mergedDescriptorConfigs);
        }).always(function () {
          dfd.resolve();
        });
      });
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  saveServiceConfigProperties: function saveServiceConfigProperties(stepController) {
    this._super(stepController);
    if (this.get('currentStep') > 1 && this.get('currentStep') < 6) {
      this.set('content.skipConfigStep', this.skipConfigStep());
      this.get('isStepDisabled').findProperty('step', 4).set('value', this.get('content.skipConfigStep'));
    }
  },

  /**
   * Load master component hosts data for using in required step controllers
   */
  loadSlaveComponentHosts: function loadSlaveComponentHosts() {
    var props = this.getDBProperties(['slaveComponentHosts', 'hosts']);
    var slaveComponentHosts = props.slaveComponentHosts,
        hosts = props.hosts || {},
        hostNames = Em.keys(hosts);
    if (!Em.isNone(slaveComponentHosts)) {
      slaveComponentHosts.forEach(function (component) {
        component.hosts.forEach(function (host) {
          //Em.set(host, 'hostName', hosts[host.host_id].name);
          for (var i = 0; i < hostNames.length; i++) {
            if (hosts[hostNames[i]].id === host.host_id) {
              host.hostName = hostNames[i];
              break;
            }
          }
        });
      });
    }

    this.set('content.installedHosts', this.getDBProperty('hosts') || this.get('content.hosts'));
    this.set("content.slaveComponentHosts", slaveComponentHosts);
  },

  /**
   * Generate clients list for selected services and save it to model
   */
  saveClients: function saveClients() {
    var clients = [];
    var serviceComponents = App.StackServiceComponent.find();
    this.get('content.services').filterProperty('isSelected').filterProperty('isInstalled', false).forEach(function (_service) {
      var serviceClients = serviceComponents.filterProperty('serviceName', _service.get('serviceName')).filterProperty('isClient');
      serviceClients.forEach(function (client) {
        clients.push({
          component_name: client.get('componentName'),
          display_name: client.get('displayName'),
          isInstalled: false
        });
      }, this);
    }, this);

    this.setDBProperty('clientInfo', clients);
    this.set('content.clients', clients);
  },

  /**
   * Load information about hosts with clients components
   */
  loadClients: function loadClients() {
    var clients = this.getDBProperty('clientInfo');
    if (clients) {
      this.set('content.clients', clients);
    } else {
      this.saveClients();
    }
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.clearAllSteps();
    this.clearStorageData();
    this.clearServiceConfigProperties();
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
  },

  /**
   * genarates data for ajax request to launch install services
   * @method generateDataForInstallServices
   * @param {Array} selectedServices
   * @returns {{context: *, ServiceInfo: {state: string}, urlParams: string}}
   */
  generateDataForInstallServices: function generateDataForInstallServices(selectedServices) {
    if (selectedServices.contains('OOZIE')) {
      selectedServices = selectedServices.concat(['HDFS', 'YARN', 'MAPREDUCE2']);
    }
    return {
      "context": Em.I18n.t('requestInfo.installServices'),
      "ServiceInfo": { "state": "INSTALLED" },
      "urlParams": "ServiceInfo/service_name.in(" + selectedServices.join(',') + ")"
    };
  },

  /**
   * main method for installing additional clients and services
   * @param {function} callback
   * @method installServices
   */
  installServices: function installServices(callback) {
    var self = this;
    this.set('content.cluster.oldRequestsId', []);
    this.installAdditionalClients().done(function () {
      self.installSelectedServices(callback);
    });
  },

  /**
   * method to install added services
   * @param {function} callback
   * @method installSelectedServices
   */
  installSelectedServices: function installSelectedServices(callback) {
    var name = 'common.services.update';
    var selectedServices = this.get('content.services').filterProperty('isInstalled', false).filterProperty('isSelected', true).mapProperty('serviceName');
    var dependentServices = this.getServicesBySelectedSlaves();
    var data = this.generateDataForInstallServices(selectedServices.concat(dependentServices));
    this.installServicesRequest(name, data, callback.bind(this));
  },

  installServicesRequest: function installServicesRequest(name, data, callback) {
    callback = callback || Em.K;
    App.ajax.send({
      name: name,
      sender: this,
      data: data,
      success: 'installServicesSuccessCallback',
      error: 'installServicesErrorCallback'
    }).then(callback, callback);
  },

  /**
   * return list of services by selected and not installed slave components
   * @returns {Array}
   */
  getServicesBySelectedSlaves: function getServicesBySelectedSlaves() {
    var result = [];
    this.get('content.slaveComponentHosts').forEach(function (slaveComponent) {
      if (slaveComponent.hosts.someProperty('isInstalled', false)) {
        var stackComponent = App.StackServiceComponent.find().findProperty('componentName', slaveComponent.componentName);
        if (stackComponent) {
          result.push(stackComponent.get('serviceName'));
        }
      }
    });
    return result.uniq();
  },

  /**
   * installs clients before install new services
   * on host where some components require this
   * @method installAdditionalClients
   */
  installAdditionalClients: function installAdditionalClients() {
    var dfd = $.Deferred();
    var count = 0;
    if (this.get('content.additionalClients.length') > 0) {
      this.get('content.additionalClients').forEach(function (c) {
        if (c.hostNames.length > 0) {
          var queryStr = 'HostRoles/component_name=' + c.componentName + '&HostRoles/host_name.in(' + c.hostNames.join() + ')';
          this.get('installClietsQueue').addRequest({
            name: 'common.host_component.update',
            sender: this,
            data: {
              query: queryStr,
              context: 'Install ' + App.format.role(c.componentName, false),
              HostRoles: {
                state: 'INSTALLED'
              },
              counter: count++,
              deferred: dfd
            },
            success: 'installClientSuccess',
            error: 'installClientError'
          });
        }
      }, this);
      if (!this.get('installClietsQueue.queue.length')) {
        return dfd.resolve();
      }
      this.set('installClientQueueLength', this.get('installClietsQueue.queue.length'));
      App.get('router.wizardStep8Controller').set('servicesInstalled', true);
      this.get('installClietsQueue').start();
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  /**
   * callback for when install clients success
   * @param data
   * @param opt
   * @param params
   * @method installClientComplete
   */
  installClientSuccess: function installClientSuccess(data, opt, params) {
    if (this.get('installClientQueueLength') - 1 === params.counter) {
      params.deferred.resolve();
    }
  },

  /**
   * callback for when install clients fail
   * @param request
   * @param ajaxOptions
   * @param error
   * @param opt
   * @param params
   */
  installClientError: function installClientError(request, ajaxOptions, error, opt, params) {
    if (this.get('installClientQueueLength') - 1 === params.counter) {
      params.deferred.resolve();
    }
  },

  loadServiceConfigGroups: function loadServiceConfigGroups() {
    this._super();
    this.set('areInstalledConfigGroupsLoaded', !Em.isNone(this.getDBProperty('serviceConfigGroups')));
  },

  clearStorageData: function clearStorageData() {
    this._super();
    this.set('areInstalledConfigGroupsLoaded', false);
  }

});

});

require.register("controllers/main/service/info/audit", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceInfoAuditController = Em.Controller.extend({
  name: 'mainServiceInfoAuditController'
});

});

require.register("controllers/main/service/info/configs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');

App.MainServiceInfoConfigsController = Em.Controller.extend(App.AddSecurityConfigs, App.ConfigsLoader, App.ServerValidatorMixin, App.EnhancedConfigsMixin, App.ThemesMappingMixin, App.ConfigsSaverMixin, App.ConfigsComparator, App.ComponentActionsByConfigs, {

  name: 'mainServiceInfoConfigsController',

  /**
   * Recommendations data will be completed on server side,
   * UI doesn't have to send all cluster data as hosts, configurations, config-groups, etc.
   */
  isRecommendationsAutoComplete: true,

  isHostsConfigsPage: false,

  isRecommendedLoaded: true,

  /**
   * Is true while request to recommendations is in progress
   * @type {Boolean}
   */
  recommendationsInProgress: false,

  dataIsLoaded: false,

  stepConfigs: [], //contains all field properties that are viewed in this service

  selectedService: null,

  selectedConfigGroup: null,

  groupsStore: App.ServiceConfigGroup.find(),

  /**
   * Configs tab for current service
   *
   * @type {App.Tab[]}
   */
  activeServiceTabs: function () {
    var selectedServiceName = this.get('selectedService.serviceName');
    return selectedServiceName ? App.Tab.find().rejectProperty('isCategorized').filterProperty('serviceName', selectedServiceName) : [];
  }.property('selectedService.serviceName'),

  /**
   * Currently opened configs tab
   *
   * @type {App.Tab}
   */
  activeTab: Em.computed.findBy('activeServiceTabs', 'isActive', true),

  /**
   * config groups for current service
   * @type {App.ConfigGroup[]}
   */
  configGroups: function () {
    return this.get('groupsStore').filterProperty('serviceName', this.get('content.serviceName'));
  }.property('content.serviceName', 'groupsStore.@each.serviceName'),

  defaultGroup: function () {
    return this.get('configGroups').findProperty('isDefault');
  }.property('configGroups'),

  isNonDefaultGroupSelectedInCompare: function () {
    return this.get('isCompareMode') && this.get('selectedConfigGroup') && !this.get('selectedConfigGroup.isDefault');
  }.property('selectedConfigGroup', 'isCompareMode'),

  dependentConfigGroups: function () {
    if (this.get('dependentServiceNames.length') === 0) return [];
    return this.get('groupsStore').filter(function (group) {
      return this.get('dependentServiceNames').contains(group.get('serviceName'));
    }, this);
  }.property('content.serviceName', 'dependentServiceNames', 'groupsStore.length', 'groupsStore.@each.name'),

  allConfigs: [],

  /**
   * Determines if save configs is in progress
   * @type {boolean}
   */
  saveInProgress: false,

  isCompareMode: false,

  preSelectedConfigVersion: null,

  /**
   * contain Service Config Property, when user proceed from Select Config Group dialog
   */
  overrideToAdd: null,

  /**
   * version selected to view
   */
  selectedVersion: null,

  /**
   * currently displayed service config version
   * @type {App.ServiceConfigVersion}
   */
  selectedVersionRecord: function () {
    var id = App.serviceConfigVersionsMapper.makeId(this.get('content.serviceName'), this.get('selectedVersion'));
    return App.ServiceConfigVersion.find(id);
  }.property('selectedVersion'),

  /**
   * note passed on configs save
   * @type {string}
   */
  serviceConfigVersionNote: '',

  versionLoaded: false,

  /**
   * Determines when data about config groups is loaded
   * Including recommendations with information about hosts in the each group
   * @type {boolean}
   */
  configGroupsAreLoaded: false,

  dependentServiceNames: [],
  /**
   * defines which service configs need to be loaded to stepConfigs
   * @type {string[]}
   */
  servicesToLoad: function () {
    return [this.get('content.serviceName')].concat(this.get('dependentServiceNames')).uniq();
  }.property('content.serviceName', 'dependentServiceNames.length'),

  /**
   * @type {boolean}
   */
  isCurrentSelected: function () {
    return App.ServiceConfigVersion.find(this.get('content.serviceName') + "_" + this.get('selectedVersion')).get('isCurrent');
  }.property('selectedVersion', 'content.serviceName', 'dataIsLoaded', 'versionLoaded'),

  /**
   * @type {boolean}
   */
  canEdit: function () {
    return (this.get('selectedVersion') == this.get('currentDefaultVersion') || !this.get('selectedConfigGroup.isDefault')) && !this.get('isCompareMode') && App.isAuthorized('SERVICE.MODIFY_CONFIGS');
  }.property('selectedVersion', 'isCompareMode', 'currentDefaultVersion', 'selectedConfigGroup.isDefault'),

  serviceConfigs: Em.computed.alias('App.config.preDefinedServiceConfigs'),

  /**
   * Number of errors in the configs in the selected service (only for AdvancedTab if App supports Enhanced Configs)
   * @type {number}
   */
  errorsCount: function () {
    return this.get('selectedService.configsWithErrors').filter(function (c) {
      return Em.isNone(c.get('isInDefaultTheme'));
    }).length;
  }.property('selectedService.configsWithErrors'),

  /**
   * Determines if Save-button should be disabled
   * Disabled if some configs have invalid values for selected service
   * or save-process currently in progress
   *
   * @type {boolean}
   */
  isSubmitDisabled: function () {
    if (!this.get('selectedService')) return true;
    return this.get('selectedService').get('errorCount') !== 0 || this.get('saveInProgress') || this.get('recommendationsInProgress');
  }.property('selectedService.errorCount', 'saveInProgress', 'recommendationsInProgress'),

  /**
   * Determines if some config value is changed
   * @type {boolean}
   */
  isPropertiesChanged: Em.computed.alias('selectedService.isPropertiesChanged'),

  /**
   * Filter text will be located here
   * @type {string}
   */
  filter: '',

  /**
   * List of filters for config properties to populate filter combobox
   * @type {{attributeName: string, attributeValue: boolean, caption: string}[]}
   */
  propertyFilters: [{
    attributeName: 'isOverridden',
    attributeValue: true,
    caption: 'common.combobox.dropdown.overridden',
    dependentOn: 'isNonDefaultGroupSelectedInCompare',
    disabledOnCondition: 'isNonDefaultGroupSelectedInCompare'
  }, {
    attributeName: 'isFinal',
    attributeValue: true,
    caption: 'common.combobox.dropdown.final'
  }, {
    attributeName: 'hasCompareDiffs',
    attributeValue: true,
    caption: 'common.combobox.dropdown.changed',
    dependentOn: 'isCompareMode',
    canBeExcluded: true
  }, {
    attributeName: 'hasIssues',
    attributeValue: true,
    caption: 'common.combobox.dropdown.issues'
  }],

  /**
   * Dropdown menu items in filter combobox
   * @type {{attributeName: string, attributeValue: string, name: string, selected: boolean}[]}
   */
  filterColumns: function () {
    var filterColumns = [];

    this.get('propertyFilters').forEach(function (filter) {
      if (filter.canBeExcluded && !(Em.isNone(filter.dependentOn) || this.get(filter.dependentOn))) {
        return; // exclude column
      }
      filterColumns.push(Ember.Object.create({
        attributeName: filter.attributeName,
        attributeValue: filter.attributeValue,
        name: this.t(filter.caption),
        selected: filter.dependentOn ? this.get(filter.dependentOn) : false,
        isDisabled: filter.disabledOnCondition ? this.get(filter.disabledOnCondition) : false
      }));
    }, this);
    return filterColumns;
  }.property('propertyFilters', 'isCompareMode', 'isNonDefaultGroupSelectedInCompare'),

  /**
   * Detects of some of the `password`-configs has not default value
   *
   * @type {boolean}
   */
  passwordConfigsAreChanged: function () {
    return this.get('stepConfigs').findProperty('serviceName', this.get('selectedService.serviceName')).get('configs').filterProperty('displayType', 'password').someProperty('isNotDefaultValue');
  }.property('stepConfigs.@each.configs', 'selectedService.serviceName'),

  /**
   * indicate whether service config version belongs to default config group
   * @param {object} version
   * @return {Boolean}
   * @private
   * @method isVersionDefault
   */
  isVersionDefault: function isVersionDefault(version) {
    return App.ServiceConfigVersion.find(this.get('content.serviceName') + "_" + version).get('groupName') === App.ServiceConfigGroup.defaultGroupName;
  },

  /**
   * clear and set properties to default value
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.abortRequests();
    App.set('componentToBeAdded', {});
    App.set('componentToBeDeleted', {});
    this.clearLoadInfo();
    this.clearSaveInfo();
    this.clearRecommendations();
    this.setProperties({
      saveInProgress: false,
      isInit: true,
      hash: null,
      dataIsLoaded: false,
      versionLoaded: false,
      filter: '',
      serviceConfigVersionNote: '',
      dependentServiceNames: [],
      configGroupsAreLoaded: false
    });
    this.get('filterColumns').setEach('selected', false);
    this.clearConfigs();
  },

  clearConfigs: function clearConfigs() {
    this.get('selectedConfigGroup', null);
    this.get('allConfigs').invoke('destroy');
    this.get('stepConfigs').invoke('destroy');
    this.set('stepConfigs', []);
    this.set('allConfigs', []);
    this.set('selectedService', null);
  },

  /**
   * "Finger-print" of the <code>stepConfigs</code>. Filled after first configGroup selecting
   * Used to determine if some changes were made (when user navigates away from this page)
   * @type {String|null}
   */
  hash: null,

  /**
   * Is this initial config group changing
   * @type {Boolean}
   */
  isInit: true,

  /**
   * Returns dependencies at all levels for service including dependencies for its childs, children dependencies
   * and so on.
   *
   * @param  {String} serviceName name of services to get dependencies
   * @returns {String[]}
   */
  getServicesDependencies: function getServicesDependencies(serviceName) {
    var dependencies = Em.getWithDefault(App.StackService.find(serviceName), 'dependentServiceNames', []);
    var loop = function loop(dependentServices, allDependencies) {
      return dependentServices.reduce(function (all, name) {
        var service = App.StackService.find(name);
        if (!service) {
          return all;
        }
        var serviceDependencies = service.get('dependentServiceNames');
        if (!serviceDependencies.length) {
          return all.concat(name);
        }
        var missed = _.intersection(_.difference(serviceDependencies, all), serviceDependencies);
        if (missed.length) {
          return loop(missed, all.concat(missed));
        }
        return all;
      }, allDependencies || dependentServices);
    };

    return loop(dependencies).uniq().without(serviceName).toArray();
  },

  /**
   * On load function
   * @method loadStep
   */
  loadStep: function loadStep() {
    var serviceName = this.get('content.serviceName'),
        self = this;
    this.clearStep();
    this.set('dependentServiceNames', this.getServicesDependencies(serviceName));
    this.trackRequestChain(this.loadConfigTheme(serviceName).always(function () {
      if (self.get('preSelectedConfigVersion')) {
        self.loadPreSelectedConfigVersion();
      } else {
        self.loadCurrentVersions();
      }
      self.trackRequest(self.loadServiceConfigVersions());
    }));
  },

  saveConfigs: function saveConfigs() {
    var newVersionToBeCreated = Math.max.apply(null, App.ServiceConfigVersion.find().mapProperty('version')) + 1;
    var isDefault = this.get('selectedConfigGroup.name') === App.ServiceConfigGroup.defaultGroupName;
    this.set('currentDefaultVersion', isDefault ? newVersionToBeCreated : this.get('currentDefaultVersion'));
    this._super();
  },

  /**
   * Generate "finger-print" for current <code>stepConfigs[0]</code>
   * Used to determine, if user has some unsaved changes (comparing with <code>hash</code>)
   * @returns {string|null}
   * @method getHash
   */
  getHash: function getHash() {
    if (!this.get('selectedService.configs.length')) {
      return null;
    }
    var hash = {};
    var sortedProperties = this.get('selectedService.configs').slice().sort(function (a, b) {
      var first = a.get('id') || App.config.configId(a.get('name'), a.get('filename'));
      var second = b.get('id') || App.config.configId(b.get('name'), b.get('filename'));
      if (first < second) return -1;
      if (first > second) return 1;
      return 0;
    });
    sortedProperties.forEach(function (config) {
      var configId = '';
      if (config.isRequiredByAgent) {
        configId = config.get('id') || App.config.configId(config.get('name'), config.get('filename'));
        hash[configId] = {
          value: App.config.formatPropertyValue(config),
          overrides: [],
          isFinal: config.get('isFinal')
        };
        if (!config.get('overrides')) return;
        if (!config.get('overrides.length')) return;

        config.get('overrides').forEach(function (override) {
          hash[configId].overrides.push(App.config.formatPropertyValue(override));
        });
      }
    });
    return JSON.stringify(hash);
  },

  parseConfigData: function parseConfigData(data) {
    var _this = this;

    this.loadKerberosIdentitiesConfigs().done(function (identityConfigs) {
      _this.prepareConfigObjects(data, identityConfigs);
      _this.loadCompareVersionConfigs(_this.get('allConfigs')).done(function () {
        _this.addOverrides(data, _this.get('allConfigs'));
        _this.onLoadOverrides(_this.get('allConfigs'));
        _this.updateAttributesFromTheme(_this.get('content.serviceName'));
      });
    });
  },

  prepareConfigObjects: function prepareConfigObjects(data, identitiesMap) {
    this.get('stepConfigs').clear();

    var configs = [];
    data.items.forEach(function (version) {
      if (version.group_name === App.ServiceConfigGroup.defaultGroupName) {
        version.configurations.forEach(function (configObject) {
          configs = configs.concat(App.config.getConfigsFromJSON(configObject, true));
        });
      }
    });

    configs = App.config.sortConfigs(configs);
    /**
     * if property defined in stack but somehow it missed from cluster properties (can be after stack upgrade)
     * ui should add this properties to step configs
     */
    configs = this.mergeWithStackProperties(configs);

    var filenames = configs.mapProperty('fileName').uniq();
    //put properties from capacity-scheduler.xml into one config with textarea view
    if (filenames.contains('capacity-scheduler.xml')) {
      configs = App.config.addYarnCapacityScheduler(configs);
    }

    this.setPropertyIsVisible(configs);
    this.setPropertyIsEditable(configs, identitiesMap);
    this.set('allConfigs', configs);
  },

  setPropertyIsVisible: function setPropertyIsVisible(configs) {
    if (this.get('content.serviceName') === 'KERBEROS') {
      var kdc_type = configs.findProperty('name', 'kdc_type');
      if (kdc_type.get('value') === 'none') {
        configs.findProperty('name', 'kdc_hosts').set('isVisible', false);
        configs.findProperty('name', 'admin_server_host').set('isVisible', false);
        configs.findProperty('name', 'domains').set('isVisible', false);
      } else if (kdc_type.get('value') === 'active-directory') {
        configs.findProperty('name', 'container_dn').set('isVisible', true);
        configs.findProperty('name', 'ldap_url').set('isVisible', true);
      } else if (kdc_type.get('value') === 'ipa') {
        configs.findProperty('name', 'group').set('isVisible', true);
        configs.findProperty('name', 'manage_krb5_conf').set('value', false); //TODO
        configs.findProperty('name', 'install_packages').set('value', false); //TODO
        configs.findProperty('name', 'admin_server_host').set('isVisible', false);
        configs.findProperty('name', 'domains').set('isVisible', false);
      }
    }
  },

  /**
   * Set <code>isEditable<code> property based on selected group, security
   * and controller restriction
   * @param configs
   * @param identitiesMap
   */
  setPropertyIsEditable: function setPropertyIsEditable(configs, identitiesMap) {
    if (!this.get('selectedConfigGroup.isDefault') || !this.get('canEdit')) {
      configs.setEach('isEditable', false);
    } else if (App.get('isKerberosEnabled')) {
      configs.forEach(function (c) {
        if (identitiesMap[c.get('id')]) {
          c.set('isConfigIdentity', true);
          c.set('isEditable', false);
          c.set('isSecureConfig', true);
          c.set('description', App.config.kerberosIdentitiesDescription(c.get('description')));
        }
      });
    }
  },

  /**
   * Load config Identities
   *
   * @returns {*}
   */
  loadKerberosIdentitiesConfigs: function loadKerberosIdentitiesConfigs() {
    var dfd = $.Deferred();
    if (App.get('isKerberosEnabled')) {
      this.loadClusterDescriptorConfigs().then(function (kerberosDescriptor) {
        dfd.resolve(App.config.parseDescriptor(kerberosDescriptor));
      });
    } else {
      dfd.resolve(true);
    }
    return dfd.promise();
  },

  /**
   * adds properties form stack that doesn't belong to cluster
   * to step configs
   * also set recommended value if isn't exists
   *
   * @return {App.ServiceConfigProperty[]}
   * @method mergeWithStackProperties
   */
  mergeWithStackProperties: function mergeWithStackProperties(configs) {
    App.config.getPropertiesFromTheme(this.get('content.serviceName')).forEach(function (advanced_id) {
      if (!configs.someProperty('id', advanced_id)) {
        var advanced = App.configsCollection.getConfig(advanced_id);
        if (advanced) {
          advanced.savedValue = null;
          advanced.isNotSaved = advanced.isRequiredByAgent;
          configs.pushObject(App.ServiceConfigProperty.create(advanced));
        }
      }
    });
    return configs;
  },

  addOverrides: function addOverrides(data, allConfigs) {
    var self = this;
    data.items.forEach(function (group) {
      if (![App.ServiceConfigGroup.defaultGroupName, App.ServiceConfigGroup.deletedGroupName].contains(group.group_name)) {
        var configGroup = App.ServiceConfigGroup.find().filterProperty('serviceName', group.service_name).findProperty('name', group.group_name);
        group.configurations.forEach(function (config) {
          for (var prop in config.properties) {
            var fileName = App.config.getOriginalFileName(config.type);
            var serviceConfig = allConfigs.filterProperty('name', prop).findProperty('filename', fileName);
            if (serviceConfig) {
              var value = App.config.formatPropertyValue(serviceConfig, config.properties[prop]);
              var isFinal = !!(config.properties_attributes && config.properties_attributes.final && config.properties_attributes.final[prop]);
              var overridePlainObject = {
                "value": value,
                "isVisible": self.get('selectedConfigGroup.isDefault') || configGroup.get('name') === self.get('selectedConfigGroup.name'),
                "savedValue": value,
                "isFinal": isFinal,
                "savedIsFinal": isFinal,
                "isEditable": self.get('canEdit') && configGroup.get('name') === self.get('selectedConfigGroup.name')
              };
              App.config.createOverride(serviceConfig, overridePlainObject, configGroup);
            } else {
              var isEditable = self.get('canEdit') && configGroup.get('name') === self.get('selectedConfigGroup.name');
              var propValue = config.properties[prop];
              var newConfig = App.ServiceConfigProperty.create(App.config.createDefaultConfig(prop, fileName, false, {
                overrides: [],
                displayType: 'label',
                value: 'Undefined',
                isPropertyOverridable: false,
                overrideValues: [propValue],
                overrideIsFinalValues: [false]
              }));
              newConfig.get('overrides').push(App.config.createCustomGroupConfig({
                propertyName: prop,
                filename: fileName,
                value: propValue,
                savedValue: propValue,
                isEditable: isEditable
              }, configGroup));
              allConfigs.push(newConfig);
            }
          }
        });
      }
    });
  },

  /**
   * @param allConfigs
   * @private
   * @method onLoadOverrides
   */
  onLoadOverrides: function onLoadOverrides(allConfigs) {
    this.get('servicesToLoad').forEach(function (serviceName) {
      var configGroups = serviceName === this.get('content.serviceName') ? this.get('configGroups') : this.get('dependentConfigGroups').filterProperty('serviceName', serviceName);
      var configTypes = App.StackService.find(serviceName).get('configTypeList');
      var configsByService = this.get('allConfigs').filter(function (c) {
        return configTypes.contains(App.config.getConfigTagFromFileName(c.get('filename')));
      });
      var serviceConfig = App.config.createServiceConfig(serviceName, configGroups, configsByService, configsByService.length);
      this.addHostNamesToConfigs(serviceConfig);
      this.get('stepConfigs').pushObject(serviceConfig);
    }, this);

    var selectedService = this.get('stepConfigs').findProperty('serviceName', this.get('content.serviceName'));
    this.set('selectedService', selectedService);
    this.checkOverrideProperty(selectedService);

    var isRangerPresent = App.Service.find().someProperty('serviceName', 'RANGER');
    if (isRangerPresent) {
      App.router.get('mainServiceInfoSummaryController').updateRangerPluginsStatus();
      this.setVisibilityForRangerProperties(selectedService);
      this.loadConfigRecommendations(null, this._onLoadComplete.bind(this));
      App.loadTimer.finish('Service Configs Page');
    } else {
      var mainController = App.get('router.mainController');
      var clusterController = App.get('router.clusterController');
      var self = this;
      mainController.isLoading.call(clusterController, 'clusterEnv').done(function () {
        var isExternalRangerSetup = clusterController.get("clusterEnv")["properties"]["enable_external_ranger"];
        if (isExternalRangerSetup) {
          self.setVisibilityForRangerProperties(selectedService);
        } else {
          App.config.removeRangerConfigs(self.get('stepConfigs'));
        }
        self.loadConfigRecommendations(null, self._onLoadComplete.bind(self));
        App.loadTimer.finish('Service Configs Page');
      });
    }
  },

  /**
   * @method _getRecommendationsForDependenciesCallback
   */
  _onLoadComplete: function _onLoadComplete() {
    this.get('stepConfigs').forEach(function (serviceConfig) {
      serviceConfig.set('initConfigsLength', serviceConfig.get('configs.length'));
    });
    this.setProperties({
      dataIsLoaded: true,
      versionLoaded: true,
      isInit: false,
      hash: this.getHash()
    });
  },

  /**
   * hide properties from Advanced ranger category that match pattern
   * if property with dependentConfigPattern is false otherwise don't hide
   * @param serviceConfig
   * @private
   * @method setVisibilityForRangerProperties
   */
  setVisibilityForRangerProperties: function setVisibilityForRangerProperties(serviceConfig) {
    var category = "Advanced ranger-{0}-plugin-properties".format(this.get('content.serviceName').toLowerCase());
    if (serviceConfig.configCategories.findProperty('name', category)) {
      var patternConfig = serviceConfig.configs.findProperty('dependentConfigPattern');
      if (patternConfig) {
        var value = patternConfig.get('value') === true || ["yes", "true"].contains(patternConfig.get('value').toLowerCase());

        serviceConfig.configs.filter(function (c) {
          if (c.get('category') === category && c.get('name').match(patternConfig.get('dependentConfigPattern')) && c.get('name') !== patternConfig.get('name')) c.set('isVisible', value);
        });
      }
    }
  },

  /**
   * Allow update property if recommendations
   * is based on changing property
   *
   * @param parentProperties
   * @returns {boolean}
   * @override
   */
  allowUpdateProperty: function allowUpdateProperty(parentProperties) {
    return !!(parentProperties && parentProperties.length);
  },

  /**
   * trigger App.config.createOverride
   * @param {Object[]} stepConfig
   * @private
   * @method checkOverrideProperty
   */
  checkOverrideProperty: function checkOverrideProperty(stepConfig) {
    var overrideToAdd = this.get('overrideToAdd');
    var value = !!this.get('overrideToAdd.widget') ? Em.get(overrideToAdd, 'value') : '';
    if (overrideToAdd) {
      overrideToAdd = stepConfig.configs.filter(function (c) {
        return c.name == overrideToAdd.name && c.filename === overrideToAdd.filename;
      });
      if (overrideToAdd[0]) {
        App.config.createOverride(overrideToAdd[0], { "isEditable": true, "value": value }, this.get('selectedConfigGroup'));
        this.set('overrideToAdd', null);
      }
    }
  },

  /**
   *
   * @param serviceConfig
   */
  addHostNamesToConfigs: function addHostNamesToConfigs(serviceConfig) {
    serviceConfig.get('configCategories').forEach(function (c) {
      if (c.showHost) {
        var stackComponent = App.StackServiceComponent.find(c.name),
            value = this.getComponentHostValue(c.name);
        var hProperty = App.config.createHostNameProperty(serviceConfig.get('serviceName'), c.name, value, stackComponent);
        serviceConfig.get('configs').push(App.ServiceConfigProperty.create(hProperty));
      }
    }, this);
  },

  /**
   * Method to get host for master or slave component
   *
   * @param componentName
   * @returns {Array}
   */
  getComponentHostValue: function getComponentHostValue(componentName) {
    var stackComponent = App.StackServiceComponent.find(componentName);
    var component = stackComponent.get('isMaster') ? App.MasterComponent.find(componentName) : App.SlaveComponent.find(componentName);
    return component.get('hostNames') || [];
  },

  /**
   * Trigger loadSelectedVersion
   * @method doCancel
   */
  doCancel: function doCancel() {
    this.set('preSelectedConfigVersion', null);
    App.set('componentToBeAdded', {});
    App.set('componentToBeDeleted', {});
    this.clearRecommendations();
    this.loadSelectedVersion(this.get('selectedVersion'), this.get('selectedConfigGroup'));
  },

  /**
   * trigger restartAllServiceHostComponents(batchUtils) if confirmed in popup
   * @method restartAllStaleConfigComponents
   * @return App.showConfirmationFeedBackPopup
   */
  restartAllStaleConfigComponents: function restartAllStaleConfigComponents() {
    var self = this;
    var serviceDisplayName = this.get('content.displayName');
    var bodyMessage = Em.Object.create({
      confirmMsg: Em.I18n.t('services.service.restartAll.confirmMsg').format(serviceDisplayName),
      confirmButton: Em.I18n.t('services.service.restartAll.confirmButton'),
      additionalWarningMsg: this.get('content.passiveState') === 'OFF' ? Em.I18n.t('services.service.restartAll.warningMsg.turnOnMM').format(serviceDisplayName) : null
    });

    var isNNAffected = false;
    var restartRequiredHostsAndComponents = this.get('content.restartRequiredHostsAndComponents');
    for (var hostName in restartRequiredHostsAndComponents) {
      restartRequiredHostsAndComponents[hostName].forEach(function (hostComponent) {
        if (hostComponent === 'NameNode') isNNAffected = true;
      });
    }
    if (this.get('content.serviceName') === 'HDFS' && isNNAffected && this.get('content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
      App.router.get('mainServiceItemController').checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query) {
          var selectedService = self.get('content.id');
          batchUtils.restartAllServiceHostComponents(serviceDisplayName, selectedService, true, query);
        }, bodyMessage);
      });
    } else {
      return App.showConfirmationFeedBackPopup(function (query) {
        var selectedService = self.get('content.id');
        batchUtils.restartAllServiceHostComponents(serviceDisplayName, selectedService, true, query);
      }, bodyMessage);
    }
  },

  /**
   * trigger launchHostComponentRollingRestart(batchUtils)
   * @method rollingRestartStaleConfigSlaveComponents
   */
  rollingRestartStaleConfigSlaveComponents: function rollingRestartStaleConfigSlaveComponents(componentName) {
    batchUtils.launchHostComponentRollingRestart(componentName.context, this.get('content.displayName'), this.get('content.passiveState') === "ON", true);
  },

  /**
   * trigger showItemsShouldBeRestarted popup with hosts that requires restart
   * @param {{context: object}} event
   * @method showHostsShouldBeRestarted
   */
  showHostsShouldBeRestarted: function showHostsShouldBeRestarted(event) {
    var restartRequiredHostsAndComponents = event.context.restartRequiredHostsAndComponents;
    var hosts = [];
    for (var hostName in restartRequiredHostsAndComponents) {
      hosts.push(hostName);
    }
    var hostsText = hosts.length === 1 ? Em.I18n.t('common.host') : Em.I18n.t('common.hosts');
    hosts = hosts.join(', ');
    this.showItemsShouldBeRestarted(hosts, Em.I18n.t('service.service.config.restartService.shouldBeRestarted').format(hostsText));
  },

  /**
   * trigger showItemsShouldBeRestarted popup with components that requires restart
   * @param {{context: object}} event
   * @method showComponentsShouldBeRestarted
   */
  showComponentsShouldBeRestarted: function showComponentsShouldBeRestarted(event) {
    var restartRequiredHostsAndComponents = event.context.restartRequiredHostsAndComponents;
    var hostsComponets = [];
    var componentsObject = {};
    for (var hostName in restartRequiredHostsAndComponents) {
      restartRequiredHostsAndComponents[hostName].forEach(function (hostComponent) {
        hostsComponets.push(hostComponent);
        if (componentsObject[hostComponent] != undefined) {
          componentsObject[hostComponent]++;
        } else {
          componentsObject[hostComponent] = 1;
        }
      });
    }
    var componentsList = [];
    for (var obj in componentsObject) {
      var componentDisplayName = componentsObject[obj] > 1 ? obj + 's' : obj;
      componentsList.push(componentsObject[obj] + ' ' + componentDisplayName);
    }
    var componentsText = componentsList.length === 1 ? Em.I18n.t('common.component') : Em.I18n.t('common.components');
    hostsComponets = componentsList.join(', ');
    this.showItemsShouldBeRestarted(hostsComponets, Em.I18n.t('service.service.config.restartService.shouldBeRestarted').format(componentsText));
  },

  /**
   * Show popup with selectable (@see App.SelectablePopupBodyView) list of items
   * @param {string} content string with comma-separated list of hostNames or componentNames
   * @param {string} header popup header
   * @returns {App.ModalPopup}
   * @method showItemsShouldBeRestarted
   */
  showItemsShouldBeRestarted: function showItemsShouldBeRestarted(content, header) {
    return App.ModalPopup.show({
      content: content,
      header: header,
      bodyClass: App.SelectablePopupBodyView,
      secondary: null
    });
  },

  /**
   * trigger manageConfigurationGroups
   * @method manageConfigurationGroup
   */
  manageConfigurationGroup: function manageConfigurationGroup() {
    App.router.get('manageConfigGroupsController').manageConfigurationGroups(null, this.get('content'));
  },

  /**
   * If user changes cfg group if some configs was changed popup with propose to save changes must be shown
   * @param {object} event - triggered event for selecting another config-group
   * @method selectConfigGroup
   */
  selectConfigGroup: function selectConfigGroup(event) {
    var self = this;

    function callback() {
      self.doSelectConfigGroup(event);
    }

    if (!this.get('isInit')) {
      if (this.hasUnsavedChanges()) {
        this.showSavePopup(null, callback);
        return;
      }
    }
    callback();
  },

  /**
   * switch view to selected group
   * @param event
   * @method selectConfigGroup
   */
  doSelectConfigGroup: function doSelectConfigGroup(event) {
    App.loadTimer.start('Service Configs Page');
    var configGroupVersions = App.ServiceConfigVersion.find().filterProperty('groupId', event.context.get('id'));
    //check whether config group has config versions
    if (event.context.get('isDefault')) {
      this.loadCurrentVersions();
    } else if (configGroupVersions.length > 0) {
      this.loadSelectedVersion(configGroupVersions.findProperty('isCurrent').get('version'), event.context);
    } else {
      this.loadSelectedVersion(null, event.context);
    }
  }
});

});

require.register("controllers/main/service/info/heatmap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
App.MainServiceInfoHeatmapController = App.MainChartsHeatmapController.extend({
  name: 'mainServiceInfoHeatmapController'
});

});

require.register("controllers/main/service/info/metric", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainServiceInfoMetricsController = Em.Controller.extend(App.WidgetSectionMixin, {
  name: 'mainServiceInfoMetricsController',

  layoutNameSuffix: "_dashboard",

  sectionNameSuffix: "_SUMMARY",

  /**
   * Some widget has type `GRAPH`
   *
   * @type {boolean}
   */
  someWidgetGraphExists: Em.computed.someBy('widgets', 'widgetType', 'GRAPH'),

  /**
   * @type {boolean}
   */
  showTimeRangeControl: Em.computed.or('!isServiceWithEnhancedWidgets', 'someWidgetGraphExists'),

  /**
   * @type {boolean}
   */
  isWidgetLayoutsLoaded: false,

  /**
   * @type {boolean}
   */
  isAllSharedWidgetsLoaded: false,

  /**
   * @type {boolean}
   */
  isMineWidgetsLoaded: false,

  /**
   * load widget layouts across all users in CLUSTER scope
   * @returns {$.ajax}
   */
  loadWidgetLayouts: function loadWidgetLayouts() {
    this.set('isWidgetLayoutsLoaded', false);
    return App.ajax.send({
      name: 'widgets.layouts.get',
      sender: this,
      data: {
        sectionName: this.get('sectionName')
      },
      success: 'loadWidgetLayoutsSuccessCallback'
    });
  },

  loadWidgetLayoutsSuccessCallback: function loadWidgetLayoutsSuccessCallback(data) {
    App.widgetLayoutMapper.map(data);
    this.set('isWidgetLayoutsLoaded', true);
  },

  /**
   * load all shared widgets to show on widget browser
   * @returns {$.ajax}
   */
  loadAllSharedWidgets: function loadAllSharedWidgets() {
    this.set('isAllSharedWidgetsLoaded', false);
    return App.ajax.send({
      name: 'widgets.all.shared.get',
      sender: this,
      success: 'loadAllSharedWidgetsSuccessCallback'
    });
  },

  /**
   * success callback of <code>loadAllSharedWidgets</code>
   * @param {object|null} data
   */
  loadAllSharedWidgetsSuccessCallback: function loadAllSharedWidgetsSuccessCallback(data) {
    var widgetIds = this.get('widgets').mapProperty('id');
    var activeNSWidgetLayouts = this.get('activeNSWidgetLayouts');
    if (data.items[0] && data.items.length) {
      this.set("allSharedWidgets", data.items.filter(function (widget) {
        return widget.WidgetInfo.widget_type != "HEATMAP";
      }).map(function (widget) {
        var widgetType = widget.WidgetInfo.widget_type;
        var widgetName = widget.WidgetInfo.widget_name;
        var widgetId = widget.WidgetInfo.id;
        var widgetTag = widget.WidgetInfo.tag;
        var inNSLayouts = false;
        if (widgetTag) {
          inNSLayouts = activeNSWidgetLayouts.findProperty('nameServiceId', widgetTag).get('widgets').someProperty('id', widgetId) && activeNSWidgetLayouts.findProperty('nameServiceId', 'all').get('widgets').someProperty('id', widgetId);
        }
        return Em.Object.create({
          id: widgetId,
          widgetName: widgetName,
          tag: widgetTag,
          metrics: widget.WidgetInfo.metrics,
          description: widget.WidgetInfo.description,
          widgetType: widgetType,
          iconPath: "/img/widget-" + widgetType.toLowerCase() + ".png",
          serviceName: JSON.parse(widget.WidgetInfo.metrics).mapProperty('service_name').uniq().join('-'),
          added: widgetTag ? inNSLayouts : widgetIds.contains(widgetId),
          isShared: widget.WidgetInfo.scope == "CLUSTER"
        });
      }));
    }
    this.set('isAllSharedWidgetsLoaded', true);
  },

  allSharedWidgets: [],
  mineWidgets: [],

  /**
   * load all mine widgets of current user to show on widget browser
   * @returns {$.ajax}
   */
  loadMineWidgets: function loadMineWidgets() {
    this.set('isMineWidgetsLoaded', false);
    return App.ajax.send({
      name: 'widgets.all.mine.get',
      sender: this,
      data: {
        loginName: App.router.get('loginName')
      },
      success: 'loadMineWidgetsSuccessCallback'
    });
  },

  /**
   * success callback of <code>loadMineWidgets</code>
   * @param {object|null} data
   */
  loadMineWidgetsSuccessCallback: function loadMineWidgetsSuccessCallback(data) {
    var widgetIds = this.get('widgets').mapProperty('id');
    var activeNSWidgetLayouts = this.get('activeNSWidgetLayouts');
    if (data.items[0] && data.items.length) {
      this.set("mineWidgets", data.items.filter(function (widget) {
        return widget.WidgetInfo.widget_type != "HEATMAP";
      }).map(function (widget) {
        var widgetType = widget.WidgetInfo.widget_type;
        var widgetName = widget.WidgetInfo.widget_name;
        var widgetId = widget.WidgetInfo.id;
        var widgetTag = widget.WidgetInfo.tag;
        var inNSLayouts = false;
        if (widgetTag) {
          inNSLayouts = activeNSWidgetLayouts.findProperty('nameServiceId', widgetTag).get('widgets').someProperty('id', widgetId) && activeNSWidgetLayouts.findProperty('nameServiceId', 'all').get('widgets').someProperty('id', widgetId);
        }
        return Em.Object.create({
          id: widget.WidgetInfo.id,
          widgetName: widgetName,
          tag: widgetTag,
          metrics: widget.WidgetInfo.metrics,
          description: widget.WidgetInfo.description,
          widgetType: widgetType,
          iconPath: "/img/widget-" + widgetType.toLowerCase() + ".png",
          serviceName: JSON.parse(widget.WidgetInfo.metrics).mapProperty('service_name').uniq().join('-'),
          added: widgetTag ? inNSLayouts : widgetIds.contains(widgetId),
          isShared: widget.WidgetInfo.scope == "CLUSTER"
        });
      }));
    } else {
      this.set("mineWidgets", []);
    }
    this.set('isMineWidgetsLoaded', true);
  },

  /**
   * add widgets, on click handler for "Add"
   */
  addWidget: function addWidget(event) {
    var self = this;
    var widgetToAdd = event.context;
    var activeLayouts = widgetToAdd.tag ? this.get('activeNSWidgetLayouts').filter(function (l) {
      return ['all', widgetToAdd.tag].contains(l.get('nameServiceId'));
    }) : [this.get('activeWidgetLayout')];
    activeLayouts.forEach(function (activeLayout) {
      var widgetIds = activeLayout.get('widgets').map(function (widget) {
        return {
          "id": widget.get("id")
        };
      });
      if (!widgetIds.mapProperty('id').contains(widgetToAdd.id)) {
        widgetIds.pushObject({
          "id": widgetToAdd.id
        });
        var data = {
          "WidgetLayoutInfo": {
            "display_name": activeLayout.get("displayName"),
            "id": activeLayout.get("id"),
            "layout_name": activeLayout.get("layoutName"),
            "scope": activeLayout.get("scope"),
            "section_name": activeLayout.get("sectionName"),
            "widgets": widgetIds
          }
        };

        return App.ajax.send({
          name: 'widget.layout.edit',
          sender: self,
          data: {
            layoutId: activeLayout.get("id"),
            data: data
          },
          success: 'updateActiveLayout'
        });
      }
    });

    widgetToAdd.set('added', true);
  },

  /**
   * find and hide widgets from all layouts
   * @param event
   */
  hideWidgetBrowser: function hideWidgetBrowser(event) {
    var activeLayouts = event.context.tag ? this.get('activeNSWidgetLayouts').filter(function (l) {
      return l.get('widgets').mapProperty('id').contains(event.context.id);
    }) : [this.get('activeWidgetLayout')];
    activeLayouts.forEach(function (layout) {
      event.context.nsLayout = layout;
      this.hideWidget(event);
    }, this);
  },

  /**
   * hide widgets, on click handler for "Added"
   */
  hideWidget: function hideWidget(event) {
    var widgetToHide = event.context;
    var widgetLayout = event.context.nsLayout;
    var activeLayout = widgetLayout || this.get('activeWidgetLayout');
    var widgetIds = activeLayout.get('widgets').map(function (widget) {
      return {
        "id": widget.get("id")
      };
    });
    var data = {
      "WidgetLayoutInfo": {
        "display_name": activeLayout.get("displayName"),
        "id": activeLayout.get("id"),
        "layout_name": activeLayout.get("layoutName"),
        "scope": activeLayout.get("scope"),
        "section_name": activeLayout.get("sectionName"),
        "widgets": widgetIds.filter(function (widget) {
          return widget.id !== widgetToHide.id;
        })
      }
    };

    widgetToHide.set('added', false);
    return App.ajax.send({
      name: 'widget.layout.edit',
      sender: this,
      data: {
        layoutId: activeLayout.get("id"),
        data: data
      },
      success: 'hideWidgetSuccessCallback'
    });
  },

  /**
   * @param {object|null} data
   * @param {object} opt
   * @param {object} params
   */
  hideWidgetSuccessCallback: function hideWidgetSuccessCallback(data, opt, params) {
    params.data.WidgetLayoutInfo.widgets = params.data.WidgetLayoutInfo.widgets.map(function (widget) {
      return {
        WidgetInfo: {
          id: widget.id
        }
      };
    });
    App.widgetLayoutMapper.map({ items: [params.data] });
    this.propertyDidChange('widgets');
  },

  /**
   * update current active widget layout
   */
  updateActiveLayout: function updateActiveLayout() {
    this.getActiveWidgetLayout();
  },

  /**
   * delete widgets, on click handler for "Delete"
   */
  deleteWidget: function deleteWidget(event) {
    var widget = event.context;
    var self = this;
    var confirmMsg = widget.get('isShared') ? Em.I18n.t('dashboard.widgets.browser.action.delete.shared.bodyMsg').format(widget.widgetName) : Em.I18n.t('dashboard.widgets.browser.action.delete.mine.bodyMsg').format(widget.widgetName);
    var bodyMessage = Em.Object.create({
      confirmMsg: confirmMsg,
      confirmButton: Em.I18n.t('dashboard.widgets.browser.action.delete.btnMsg')
    });
    return App.showConfirmationFeedBackPopup(function (query) {
      return App.ajax.send({
        name: 'widget.action.delete',
        sender: self,
        data: {
          id: widget.id
        },
        success: 'updateWidgetBrowser'
      });
    }, bodyMessage);
  },

  /**
   * update widget browser content after deleted some widget
   */
  updateWidgetBrowser: function updateWidgetBrowser() {
    this.loadAllSharedWidgets();
    this.loadMineWidgets();
  },

  /**
   * Share widgets, on click handler for "Share"
   */
  shareWidget: function shareWidget(event) {
    var widget = event.context;
    var self = this;
    var bodyMessage = Em.Object.create({
      confirmMsg: Em.I18n.t('dashboard.widgets.browser.action.share.confirmation'),
      confirmButton: Em.I18n.t('dashboard.widgets.browser.action.share')
    });
    return App.showConfirmationFeedBackPopup(function (query) {
      return App.ajax.send({
        name: 'widgets.wizard.edit',
        sender: self,
        data: {
          data: {
            "WidgetInfo": {
              "widget_name": widget.get("widgetName"),
              "scope": "CLUSTER"
            }
          },
          widgetId: widget.get("id")
        },
        success: 'updateWidgetBrowser'
      });
    }, bodyMessage);
  },

  /**
   * create widget
   */
  createWidget: function createWidget() {
    App.router.send('createServiceWidget', Em.Object.create({
      layout: this.get('activeWidgetLayout'),
      serviceName: this.get('content.serviceName')
    }));
  },

  /**
   * edit widget
   * @param {App.Widget} content
   */
  editWidget: function editWidget(content) {
    content.set('serviceName', this.get('content.serviceName'));
    App.router.send('editServiceWidget', content);
  },

  /**
   * launch Widgets Browser popup
   * @method showPopup
   * @return {App.ModalPopup}
   */
  goToWidgetsBrowser: function goToWidgetsBrowser() {
    var self = this;

    return App.ModalPopup.show({
      header: Em.I18n.t('dashboard.widgets.browser.header'),

      classNames: ['common-modal-wrapper', 'widgets-browser-popup'],
      modalDialogClasses: ['modal-lg'],
      onPrimary: function onPrimary() {
        this.hide();
        self.set('isAllSharedWidgetsLoaded', false);
        self.set('allSharedWidgets', []);
        self.set('isMineWidgetsLoaded', false);
        self.set('mineWidgets', []);
      },
      autoHeight: false,
      isHideBodyScroll: false,
      footerClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/widget_browser_footer'),
        isShowMineOnly: false,
        onPrimary: function onPrimary() {
          this.get('parentView').onPrimary();
        }
      }),
      isShowMineOnly: false,
      showTopShadow: false,

      didInsertElement: function didInsertElement() {
        this._super();
        this.$().find('.modal-body').on('scroll', function (event) {
          var modalBody = $(event.currentTarget);
          if (modalBody.scrollTop() > 0) {
            modalBody.addClass('top-shadow');
          } else {
            modalBody.removeClass('top-shadow');
          }
        });
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/widget_browser_popup'),
        controller: self,
        willInsertElement: function willInsertElement() {
          this.get('controller').loadAllSharedWidgets();
          this.get('controller').loadMineWidgets();
        },

        isLoaded: Em.computed.and('controller.isAllSharedWidgetsLoaded', 'controller.isMineWidgetsLoaded'),

        isWidgetEmptyList: Em.computed.empty('filteredContent'),

        activeService: '',
        activeStatus: '',

        content: function () {
          var content = [];
          if (this.get('parentView.isShowMineOnly')) {
            content = this.get('controller.mineWidgets');
          } else {
            // merge my widgets and all shared widgets, no duplicated is allowed
            var widgetMap = {};
            var allWidgets = this.get('controller.allSharedWidgets').concat(this.get('controller.mineWidgets'));
            allWidgets.forEach(function (widget) {
              if (!widgetMap[widget.get("id")]) {
                content.pushObject(widget);
                widgetMap[widget.get("id")] = true;
              }
            });
          }

          //remove NameNode widgets with no tag if federation is enabled
          if (App.get('hasNameNodeFederation')) {
            content = content.filter(function (w) {
              var parsedMetric;
              try {
                parsedMetric = JSON.parse(w.metrics);
              } catch (e) {}
              return w.tag || !(parsedMetric && parsedMetric.someProperty('component_name', 'NAMENODE'));
            });
          }

          return content;
        }.property('controller.allSharedWidgets.length', 'controller.isAllSharedWidgetsLoaded', 'controller.mineWidgets.length', 'controller.isMineWidgetsLoaded', 'parentView.isShowMineOnly'),

        /**
         * displaying content filtered by service name and status.
         */
        filteredContent: function () {
          var activeService = this.get('activeService') ? this.get('activeService') : this.get('controller.content.serviceName');
          var result = [];
          this.get('content').forEach(function (widget) {
            if (widget.get('serviceName').indexOf(activeService) >= 0) {
              result.pushObject(widget);
            }
          });
          return result;
        }.property('content', 'activeService', 'activeStatus'),

        /**
         * service name filter
         */
        services: function () {
          var view = this;
          var services = App.Service.find().filter(function (item) {
            var stackService = App.StackService.find().findProperty('serviceName', item.get('serviceName'));
            return stackService.get('isServiceWithWidgets');
          });
          return services.map(function (service) {
            return Em.Object.create({
              value: service.get('serviceName'),
              label: service.get('displayName'),
              isActive: function () {
                var activeService = view.get('activeService') ? view.get('activeService') : view.get('controller.content.serviceName');
                return this.get('value') == activeService;
              }.property('value', 'view.activeService')
            });
          });
        }.property('activeService'),

        filterByService: function filterByService(event) {
          this.set('activeService', event.context);
        },

        createWidget: function createWidget() {
          this.get('parentView').onPrimary();
          this.get('controller').createWidget();
        },

        ensureTooltip: function () {
          Em.run.later(this, function () {
            App.tooltip($("[rel='shared-icon-tooltip']"));
          }, 1000);
        }.observes('activeService', 'parentView.isShowMineOnly'),

        didInsertElement: function didInsertElement() {
          this.ensureTooltip();
        }
      })
    });
  }

});

});

require.register("controllers/main/service/info/summary", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainServiceInfoSummaryController = Em.Controller.extend({
  name: 'mainServiceInfoSummaryController',

  selectedFlumeAgent: null,

  /**
   * Indicates whether Ranger plugins status update polling is active
   * @type {boolean}
   */
  isRangerUpdateWorking: false,

  /**
   * Indicates whether array with initial Ranger plugins data is set
   * @type {boolean}
   */
  isRangerPluginsArraySet: false,

  /**
   * Indicates whether previous AJAX request for Ranger plugins config properties has failed
   * @type {boolean}
   */
  isPreviousRangerConfigsCallFailed: false,

  /**
   * HiveServer2 JDBC connection endpoint data
   * @type {array}
   */
  hiveServerEndPoints: [],

  /**
   * Ranger plugins data
   * @type {array}
   */
  rangerPlugins: [{
    serviceName: 'HDFS',
    type: 'ranger-hdfs-plugin-properties',
    propertyName: 'ranger-hdfs-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'YARN',
    type: 'ranger-yarn-plugin-properties',
    propertyName: 'ranger-yarn-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'HBASE',
    type: 'ranger-hbase-plugin-properties',
    propertyName: 'ranger-hbase-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'HIVE',
    type: 'hive-env',
    propertyName: 'hive_security_authorization',
    valueForEnable: 'Ranger'
  }, {
    serviceName: 'KNOX',
    type: 'ranger-knox-plugin-properties',
    propertyName: 'ranger-knox-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'STORM',
    type: 'ranger-storm-plugin-properties',
    propertyName: 'ranger-storm-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'KAFKA',
    type: 'ranger-kafka-plugin-properties',
    propertyName: 'ranger-kafka-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'ATLAS',
    type: 'ranger-atlas-plugin-properties',
    propertyName: 'ranger-atlas-plugin-enabled',
    valueForEnable: 'Yes'
  }, {
    serviceName: 'NIFI',
    type: 'ranger-nifi-plugin-properties',
    propertyName: 'ranger-nifi-plugin-enabled',
    valueForEnable: 'Yes'
  }],

  /**
   * Set initial Ranger plugins data
   * @method setRangerPlugins
   */
  setRangerPlugins: function () {
    if (App.get('router.clusterController.isLoaded') && !this.get('isRangerPluginsArraySet')) {
      // Display order of ranger plugin for services should be decided from  App.StackService.displayOrder to keep consistency
      // with display order of services at other places in the application like `select service's page` and `service menu bar`
      var displayOrderLength = App.StackService.displayOrder.length;
      var rangerPlugins = this.get('rangerPlugins').map(function (item, index) {
        var displayOrderIndex = App.StackService.displayOrder.indexOf(item.serviceName);
        return $.extend(item, {
          index: displayOrderIndex == -1 ? displayOrderLength + index : displayOrderIndex
        });
      }).sortProperty('index');

      this.setProperties({
        rangerPlugins: rangerPlugins.map(function (item) {
          var stackService = App.StackService.find().findProperty('serviceName', item.serviceName);
          var displayName = stackService ? stackService.get('displayName') : item.serviceName;
          return $.extend(item, {
            pluginTitle: Em.I18n.t('services.service.summary.ranger.plugin.title').format(displayName),
            isDisplayed: App.Service.find().someProperty('serviceName', item.serviceName) && stackService.get('configTypes').hasOwnProperty(item.type),
            status: Em.I18n.t('services.service.summary.ranger.plugin.loadingStatus')
          });
        }),
        isRangerPluginsArraySet: true
      });
    }
  }.observes('App.router.clusterController.isLoaded'),

  /**
   * Get latest config tags
   * @method updateRangerPluginsStatus
   * @param callback
   */
  updateRangerPluginsStatus: function updateRangerPluginsStatus(callback) {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'getRangerPluginsStatus',
      callback: callback
    });
  },

  /**
   * Get latest Ranger plugins config properties
   * @method getRangerPluginsStatus
   * @param data
   */
  getRangerPluginsStatus: function getRangerPluginsStatus(data) {
    var urlParams = [];
    this.get('rangerPlugins').forEach(function (item) {
      if (App.Service.find().someProperty('serviceName', item.serviceName) && data.Clusters.desired_configs.hasOwnProperty(item.type)) {
        var currentTag = data.Clusters.desired_configs[item.type].tag;
        var isTagChanged = item.tag != currentTag;
        Em.set(item, 'isDisplayed', true);
        //Request for properties should be sent either if configs have changed or if previous Ranger plugins config properties has failed
        if (isTagChanged || this.get('isPreviousRangerConfigsCallFailed')) {
          Em.set(item, 'tag', currentTag);
          urlParams.push('(type=' + item.type + '&tag=' + currentTag + ')');
        }
      } else {
        Em.set(item, 'isDisplayed', false);
      }
    }, this);
    if (urlParams.length) {
      App.ajax.send({
        name: 'reassign.load_configs',
        sender: this,
        data: {
          urlParams: urlParams.join('|')
        },
        success: 'getRangerPluginsStatusSuccess',
        error: 'getRangerPluginsStatusError'
      });
    }
  },

  /**
   * Set Ranger plugins statuses
   * @method getRangerPluginsStatusSuccess
   * @param data
   */
  getRangerPluginsStatusSuccess: function getRangerPluginsStatusSuccess(data) {
    this.set('isPreviousRangerConfigsCallFailed', false);
    data.items.forEach(function (item) {
      var serviceName = this.get('rangerPlugins').findProperty('type', item.type).serviceName;
      var propertyName = this.get('rangerPlugins').findProperty('type', item.type).propertyName;
      var propertyValue = this.get('rangerPlugins').findProperty('type', item.type).valueForEnable;
      var statusString;

      if (item.properties[propertyName]) {
        statusString = item.properties[propertyName] == propertyValue ? 'alerts.table.state.enabled' : 'alerts.table.state.disabled';
      } else {
        statusString = 'common.unknown';
      }
      Em.set(this.get('rangerPlugins').findProperty('serviceName', serviceName), 'status', Em.I18n.t(statusString));
    }, this);
  },

  /**
   * Method executed if Ranger plugins config properties request has failed
   * @method getRangerPluginsStatusError
   */
  getRangerPluginsStatusError: function getRangerPluginsStatusError() {
    this.set('isPreviousRangerConfigsCallFailed', true);
  },

  /**
   * This method is invoked when hive view is rendered to fetch and display
   * information for JDBC connect string for HiveServer2 instances
   * @method  setHiveEndPointsValue
   * @public
   */
  setHiveEndPointsValue: function setHiveEndPointsValue() {
    var self = this;
    var sites = ['hive-site', 'hive-interactive-site'];

    var siteToComponentMap = {
      'hive-site': 'HIVE_SERVER',
      'hive-interactive-site': 'HIVE_SERVER_INTERACTIVE'
    };

    App.router.get('configurationController').getCurrentConfigsBySites(sites).done(function (configs) {

      var hiveSiteIndex = configs.map(function (item) {
        return item.type;
      }).indexOf('hive-site');

      // if hive-site is not first item then rotate the array to make it first
      if (!!hiveSiteIndex) {
        configs.push(configs.shift());
      }

      var hiveSiteDynamicDiscovery = configs[0].properties['hive.server2.support.dynamic.service.discovery'];
      var hiveSiteZkQuorom = configs[0].properties['hive.zookeeper.quorum'];
      var hiveSiteServiceDiscorveryMode = 'zooKeeper';
      var hiveSiteZkNameSpace = configs[0].properties['hive.server2.zookeeper.namespace'];

      configs.forEach(function (_config) {
        var masterComponent = App.MasterComponent.find().findProperty('componentName', siteToComponentMap[_config.type]);
        if (_config.type === 'hive-interactive-site') {
          hiveSiteDynamicDiscovery = _config.properties['hive.server2.support.dynamic.service.discovery'] || hiveSiteDynamicDiscovery;
          hiveSiteZkQuorom = _config.properties['hive.zookeeper.quorum'] || hiveSiteZkQuorom;
          hiveSiteZkNameSpace = _config.properties['hive.server2.zookeeper.namespace'] || hiveSiteZkNameSpace;
          if (App.HostComponent.find().filterProperty('componentName', 'HIVE_SERVER_INTERACTIVE').length > 1) {
            hiveSiteServiceDiscorveryMode = 'zooKeeperHA';
            hiveSiteZkNameSpace = _config.properties['hive.server2.active.passive.ha.registry.namespace'];
          }
        }
        if (masterComponent && !!masterComponent.get('totalCount')) {
          var hiveEndPoint = {
            isVisible: hiveSiteDynamicDiscovery,
            componentName: masterComponent.get('componentName'),
            label: masterComponent.get('displayName') + Em.I18n.t('services.service.summary.hiveserver2.jdbc.url.text'),
            value: Em.I18n.t('services.service.summary.hiveserver2.endpoint.value').format(hiveSiteZkQuorom, hiveSiteServiceDiscorveryMode, hiveSiteZkNameSpace),
            tooltipText: Em.I18n.t('services.service.summary.hiveserver2.endpoint.tooltip.text').format(masterComponent.get('displayName'))
          };
          self.get('hiveServerEndPoints').pushObject(Em.Object.create(hiveEndPoint));
        }
      });
    });
  },

  /**
   * Send start command for selected Flume Agent
   * @method startFlumeAgent
   */
  startFlumeAgent: function startFlumeAgent() {
    var selectedFlumeAgent = arguments[0].context;
    if (selectedFlumeAgent && selectedFlumeAgent.get('status') === 'NOT_RUNNING') {
      var self = this;
      App.showConfirmationPopup(function () {
        var state = 'STARTED';
        var context = Em.I18n.t('services.service.summary.flume.start.context').format(selectedFlumeAgent.get('name'));
        self.sendFlumeAgentCommandToServer(state, context, selectedFlumeAgent);
      });
    }
  },

  /**
   * Send stop command for selected Flume Agent
   * @method stopFlumeAgent
   */
  stopFlumeAgent: function stopFlumeAgent() {
    var selectedFlumeAgent = arguments[0].context;
    if (selectedFlumeAgent && selectedFlumeAgent.get('status') === 'RUNNING') {
      var self = this;
      App.showConfirmationPopup(function () {
        var state = 'INSTALLED';
        var context = Em.I18n.t('services.service.summary.flume.stop.context').format(selectedFlumeAgent.get('name'));
        self.sendFlumeAgentCommandToServer(state, context, selectedFlumeAgent);
      });
    }
  },

  /**
   * Send command for Flume Agent to server
   * @param {string} state
   * @param {string} context
   * @param {Object} agent
   * @method sendFlumeAgentCommandToServer
   */
  sendFlumeAgentCommandToServer: function sendFlumeAgentCommandToServer(state, context, agent) {
    App.ajax.send({
      name: 'service.flume.agent.command',
      sender: this,
      data: {
        state: state,
        context: context,
        agentName: agent.get('name'),
        host: agent.get('hostName')
      },
      success: 'commandSuccessCallback'
    });
  },

  /**
   * Callback, that shows Background operations popup if request was successful
   */
  commandSuccessCallback: function commandSuccessCallback() {
    // load data (if we need to show this background operations popup) from persist
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (showPopup) {
      if (showPopup) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  gotoConfigs: function gotoConfigs() {
    App.router.get('mainServiceItemController').set('routeToConfigs', true);
    App.router.transitionTo('main.services.service.configs', this.get('content'));
    App.router.get('mainServiceItemController').set('routeToConfigs', false);
  },

  showServiceAlertsPopup: function showServiceAlertsPopup(event) {
    var context = event.context;
    return App.ModalPopup.show({
      header: Em.I18n.t('services.service.summary.alerts.popup.header').format(context.get('displayName')),
      autoHeight: false,
      classNames: ['sixty-percent-width-modal', 'service-alerts-popup'],
      bodyClass: Em.View.extend({
        templateName: require('templates/main/service/info/service_alert_popup'),
        classNames: ['service-alerts'],
        controllerBinding: 'App.router.mainAlertDefinitionsController',
        didInsertElement: function didInsertElement() {
          Em.run.next(this, function () {
            App.tooltip(this.$(".timeago"));
            App.tooltip(this.$(".definition-latest-text"), { placement: 'bottom' });
          });
        },
        willDestroyElement: function willDestroyElement() {
          this.$(".timeago").tooltip('destroy');
          this.$(".definition-latest-text").tooltip('destroy');
        },
        alerts: function () {
          var property = context.get('componentName') ? 'componentName' : 'serviceName';
          var serviceDefinitions = this.get('controller.content').filterProperty(property, context.get(property));
          // definitions should be sorted in order: critical, warning, ok, unknown, other
          var criticalDefinitions = [],
              warningDefinitions = [],
              okDefinitions = [],
              unknownDefinitions = [];
          serviceDefinitions.forEach(function (definition) {
            if (definition.get('isCritical')) {
              criticalDefinitions.push(definition);
              serviceDefinitions = serviceDefinitions.without(definition);
            } else if (definition.get('isWarning')) {
              warningDefinitions.push(definition);
              serviceDefinitions = serviceDefinitions.without(definition);
            } else if (definition.get('isOK')) {
              okDefinitions.push(definition);
              serviceDefinitions = serviceDefinitions.without(definition);
            } else if (definition.get('isUnknown')) {
              unknownDefinitions.push(definition);
              serviceDefinitions = serviceDefinitions.without(definition);
            }
          });
          serviceDefinitions = criticalDefinitions.concat(warningDefinitions, okDefinitions, unknownDefinitions, serviceDefinitions);
          return serviceDefinitions;
        }.property('controller.content'),
        gotoAlertDetails: function gotoAlertDetails(e) {
          if (e && e.context) {
            this.get('parentView').hide();
            App.router.transitionTo('main.alerts.alertDetails', e.context);
          }
          return false;
        },
        closePopup: function closePopup() {
          this.get('parentView').hide();
        }
      }),
      isHideBodyScroll: false,
      primary: Em.I18n.t('common.close'),
      primaryClass: 'btn-default',
      secondary: null
    });
  },

  goToView: function goToView(event) {
    App.router.route(event.context.get('internalAmbariUrl'));
  }

});

});

require.register("controllers/main/service/item", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var blueprintUtils = require('utils/blueprint');
var stringUtils = require('utils/string_utils');

App.MainServiceItemController = Em.Controller.extend(App.SupportClientConfigsDownload, App.InstallComponent, App.ConfigsSaverMixin, App.EnhancedConfigsMixin, App.GroupsMappingMixin, {
  name: 'mainServiceItemController',

  /**
   * Callback functions for start and stop service have few differences
   *
   * Used with currentCallBack property
   */
  callBackConfig: {
    'STARTED': {
      'c': 'STARTING',
      'f': 'starting',
      'c2': 'live',
      'hs': 'started',
      's': 'start'
    },
    'INSTALLED': {
      'c': 'STOPPING',
      'f': 'stopping',
      'c2': 'dead',
      'hs': 'stopped',
      's': 'stop'
    }
  },

  /**
   * Map of service names and lists of sites they need to load
   */
  serviceConfigsMap: {
    'OOZIE': ['oozie-env']
  },

  /**
   * Configs loaded to use for service actions menu
   *
   * format: {config-type: {property-name1: property-value1, property-name2: property-value2, ...}}
   */
  configs: {},

  /**
   * @type {boolean}
   * @default true
   */
  isPending: true,

  /**
   * @type {boolean}
   * @default false
   */
  isServicesInfoLoaded: false,

  /**
   * Define whether configs for service actions menu were loaded
   * @type {Boolean}
   */
  isServiceConfigsLoaded: false,

  /**
   * flag to control router switch between service summary and configs
   * @type {boolean}
   */
  routeToConfigs: false,

  deleteServiceProgressPopup: null,

  isRecommendationInProgress: false,

  nameNodesWithOldCheckpoints: [],

  isNameNodeCheckpointUnavailable: false,

  isClientsOnlyService: function () {
    return App.get('services.clientOnly').contains(this.get('content.serviceName'));
  }.property('content.serviceName'),

  isConfigurable: function () {
    return !App.get('services.noConfigTypes').contains(this.get('content.serviceName'));
  }.property('App.services.noConfigTypes', 'content.serviceName'),

  clientComponents: function () {
    var clientNames = [{
      action: 'downloadAllClientConfigs',
      context: {
        label: Em.I18n.t('common.all.clients')
      }
    }];
    var clients = App.StackServiceComponent.find().filterProperty('serviceName', this.get('content.serviceName')).filterProperty('isClient');
    clients.forEach(function (item) {
      clientNames.push({
        action: 'downloadClientConfigs',
        context: {
          name: item.get('componentName'),
          label: item.get('displayName')
        }
      });
    });
    return clientNames;
  }.property('content.serviceName'),

  /**
   * Returns interdependent services
   *
   * @returns {string[]}
   */
  interDependentServices: function () {
    var serviceName = this.get('content.serviceName'),
        interDependentServices = [];
    App.StackService.find(serviceName).get('requiredServices').forEach(function (requiredService) {
      if (App.StackService.find(requiredService).get('requiredServices').contains(serviceName)) {
        interDependentServices.push(requiredService);
      }
    });
    return interDependentServices;
  }.property('content.serviceName'),

  /**
   * collection of serviceConfigs
   *
   * @type {Object[]}
   */
  stepConfigs: [],

  /**
   * List of service names that have configs dependent on current service configs
   *
   * @type {String[]}
   */
  dependentServiceNames: function () {
    return App.get('router.clusterController.isConfigsPropertiesLoaded') ? App.StackService.find(this.get('content.serviceName')).get('dependentServiceNames') : [];
  }.property('content.serviceName', 'App.router.clusterController.isConfigsPropertiesLoaded'),

  configDependentServiceNames: function () {
    return this.get('dependentServiceNames').concat(App.StackService.find(this.get('content.serviceName')).get('requiredServices'));
  }.property('dependentServiceNames'),

  /**
   * List of service names that could be deleted
   * Common case when there is only current service should be removed
   * But for some services there is <code>interDependentServices<code> services
   * Like 'YARN' depends on 'MAPREDUCE2' and 'MAPREDUCE2' depends on 'YARN'
   * So these services can be removed only together
   *
   * @type {String[]}
   */
  serviceNamesToDelete: function () {
    return [this.get('content.serviceName')].concat(this.get('interDependentServices'));
  }.property('content.serviceName'),

  /**
   * List of config types that should be loaded
   * Includes
   * 1. Dependent services config-types
   * 2. Some special cases from <code>serviceConfigsMap<code>
   * 3. 'cluster-env'
   *
   * @type {String[]}
   */
  sitesToLoad: function () {
    var services = this.get('configDependentServiceNames'),
        configTypeList = [];
    if (services.length) {
      configTypeList = App.StackService.find().filter(function (s) {
        return services.contains(s.get('serviceName'));
      }).mapProperty('configTypeList').reduce(function (p, v) {
        return p.concat(v);
      });
    }
    if (this.get('serviceConfigsMap')[this.get('content.serviceName')]) {
      configTypeList = configTypeList.concat(this.get('serviceConfigsMap')[this.get('content.serviceName')]);
    }
    configTypeList.push('cluster-env');
    return configTypeList.uniq();
  }.property('content.serviceName'),

  /**
   * Load configurations for implicit usage for service actions
   * not related to configs displayed on Configs page
   */
  loadConfigs: function loadConfigs() {
    var _this = this;

    this.set('isServiceConfigsLoaded', false);
    this.set('stepConfigs', []);
    App.get('router.mainController.isLoading').call(App.get('router.clusterController'), 'isConfigsPropertiesLoaded').done(function () {
      App.router.get('configurationController').getCurrentConfigsBySites(_this.get('sitesToLoad')).done(function (configs) {
        var allConfigs = [];
        configs.forEach(function (site) {
          _this.get('configs')[site.type] = site.properties;
          allConfigs = allConfigs.concat(App.config.getConfigsFromJSON(site, true));
        });

        _this.get('configDependentServiceNames').forEach(function (serviceName) {
          var configTypes = App.StackService.find(serviceName).get('configTypeList');
          var configsByService = allConfigs.filter(function (c) {
            return configTypes.contains(App.config.getConfigTagFromFileName(c.get('filename')));
          });
          if (App.config.get('preDefinedServiceConfigs').someProperty('serviceName', serviceName)) {
            _this.get('stepConfigs').pushObject(App.config.createServiceConfig(serviceName, [], configsByService));
          }
        });

        _this.set('isServiceConfigsLoaded', true);
      });
    });
  },

  /**
   * Common method for ajax (start/stop service) responses
   * @param data
   * @param ajaxOptions
   * @param params
   */
  startStopPopupSuccessCallback: function startStopPopupSuccessCallback(data, ajaxOptions, params) {
    var _this2 = this;

    if (data && data.Requests) {
      params.query.set('status', 'SUCCESS');
      if (App.get('testMode')) {
        var requestData = JSON.parse(ajaxOptions.data),
            state = requestData.Body.ServiceInfo.state || requestData.Body.HostRoles.state,
            config = this.get('callBackConfig')[state];
        this.set('content.workStatus', App.Service.Health[config.f]);
        this.get('content.hostComponents').setEach('workStatus', App.HostComponentStatus[config.f]);
        setTimeout(function () {
          _this2.set('content.workStatus', App.Service.Health[config.c2]);
          _this2.get('content.hostComponents').setEach('workStatus', App.HostComponentStatus[config.hs]);
        }, App.get('testModeDelayForActions'));
      }
      // load data (if we need to show this background operations popup) from persist
      App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
        if (initValue) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      });
    } else {
      params.query.set('status', 'FAIL');
    }
  },
  startStopPopupErrorCallback: function startStopPopupErrorCallback(request, ajaxOptions, error, opt, params) {
    App.ajax.defaultErrorHandler(request, opt.url, opt.type, request.status);
    params.query.set('status', 'FAIL');
  },
  /**
   * Confirmation popup for start/stop services
   * @param serviceHealth - 'STARTED' or 'INSTALLED'
   */
  startStopPopup: function startStopPopup(serviceHealth) {
    var _this3 = this;

    var serviceDisplayName = this.get('content.displayName'),
        isMaintenanceOFF = this.get('content.passiveState') === 'OFF';

    var msg = isMaintenanceOFF && serviceHealth === 'INSTALLED' ? Em.I18n.t('services.service.stop.warningMsg.turnOnMM').format(serviceDisplayName) : null;
    msg = this.addAdditionalWarningMessage(serviceHealth, msg, serviceDisplayName);

    var bodyMessage = Em.Object.create({
      putInMaintenance: serviceHealth === 'INSTALLED' && isMaintenanceOFF || serviceHealth === 'STARTED' && !isMaintenanceOFF,
      turnOnMmMsg: serviceHealth === 'INSTALLED' ? Em.I18n.t('passiveState.turnOnFor').format(serviceDisplayName) : Em.I18n.t('passiveState.turnOffFor').format(serviceDisplayName),
      confirmMsg: serviceHealth === 'INSTALLED' ? Em.I18n.t('services.service.stop.confirmMsg').format(serviceDisplayName) : Em.I18n.t('services.service.start.confirmMsg').format(serviceDisplayName),
      confirmButton: serviceHealth === 'INSTALLED' ? Em.I18n.t('services.service.stop.confirmButton') : Em.I18n.t('services.service.start.confirmButton'),
      additionalWarningMsg: msg
    });

    // check HDFS NameNode checkpoint before stop service
    if (serviceHealth === 'INSTALLED' && this.hasStartedNameNode()) {
      this.checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
          _this3.set('isPending', true);
          _this3.startStopWithMmode(serviceHealth, query, runMmOperation);
        }, bodyMessage);
      });
    } else {
      return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
        _this3.set('isPending', true);
        _this3.startStopWithMmode(serviceHealth, query, runMmOperation);
      }, bodyMessage);
    }
  },

  /**
   * Confirmation popup for start/stop services
   * @param context
   * @param serviceHealth - 'STARTED' or 'INSTALLED'
   */
  startStopCertainPopup: function startStopCertainPopup(context, serviceHealth) {
    var _this4 = this;

    var components = context.components,
        hosts = context.hosts,
        label = context.label,
        serviceDisplayName = this.get('content.displayName'),
        isMaintenanceOFF = this.get('content.passiveState') === 'OFF',
        confirmDisplayName = Em.I18n.t('services.service.componentsInNameSpace').format(label);

    var msg = isMaintenanceOFF && serviceHealth == 'INSTALLED' ? Em.I18n.t('services.service.stopCertain.warningMsg.turnOnMM').format(serviceDisplayName) : null;
    msg = this.addAdditionalWarningMessage(serviceHealth, msg, serviceDisplayName);
    var bodyMessage = Em.Object.create({
      putInMaintenance: serviceHealth === 'INSTALLED' && isMaintenanceOFF || serviceHealth === 'STARTED' && !isMaintenanceOFF,
      turnOnMmMsg: serviceHealth === 'INSTALLED' ? Em.I18n.t('passiveState.turnOnFor').format(serviceDisplayName) : Em.I18n.t('passiveState.turnOffFor').format(serviceDisplayName),
      confirmMsg: serviceHealth === 'INSTALLED' ? Em.I18n.t('services.service.stop.confirmMsg').format(confirmDisplayName) : Em.I18n.t('services.service.start.confirmMsg').format(confirmDisplayName),
      confirmButton: serviceHealth === 'INSTALLED' ? Em.I18n.t('services.service.stop.confirmButton') : Em.I18n.t('services.service.start.confirmButton'),
      additionalWarningMsg: msg
    });
    // check HDFS NameNode checkpoint before stop components
    if (serviceHealth === 'INSTALLED' && this.hasStartedNameNode(label)) {
      this.checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
          _this4.set('isPending', true);
          _this4.startStopWithMmode(serviceHealth, query, runMmOperation, components, hosts, label);
        }, bodyMessage);
      }, label);
    } else {
      return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
        _this4.set('isPending', true);
        _this4.startStopWithMmode(serviceHealth, query, runMmOperation, components, hosts, label);
      }, bodyMessage);
    }
  },

  /**
   * this function will be called from :1) stop HDFS 2) restart all for HDFS 3) restart all affected for HDFS
   * @param callback - callback function to continue next operation
   * @param [groupName] - id of HA namespace for enabled NameNode federation
   */
  checkNnLastCheckpointTime: function checkNnLastCheckpointTime(callback, groupName) {
    var _this5 = this;

    this.pullNnCheckPointTime(groupName).complete(function () {
      var nameNodesWithOldCheckpoints = _this5.get('nameNodesWithOldCheckpoints').slice(),
          isNameNodeCheckpointUnavailable = _this5.get('isNameNodeCheckpointUnavailable');
      _this5.get('nameNodesWithOldCheckpoints').clear();
      _this5.set('isNameNodeCheckpointUnavailable', false);
      if (nameNodesWithOldCheckpoints.length) {
        // too old
        _this5.getHdfsUser().done(function () {
          var maxAge = App.nnCheckpointAgeAlertThreshold,
              hdfsUser = _this5.get('hdfsUser'),
              confirmButton = Em.I18n.t('common.next');
          var msg = void 0;
          if (App.get('hasNameNodeFederation') && !groupName) {
            var oldCheckpointNameSpacesList = nameNodesWithOldCheckpoints.map(function (nn) {
              return '<li>' + nn.haNameSpace + '</li>';
            }),
                nameSpacesString = '<ul>' + oldCheckpointNameSpacesList.join('') + '</ul>',
                hostNamesList = nameNodesWithOldCheckpoints.map(function (nn) {
              return '<b>' + nn.hostName + '</b>';
            }).join(', ');
            msg = Em.Object.create({
              confirmMsg: Em.I18n.t('services.service.stop.HDFS.warningMsg.nameSpaces.checkPointTooOld').format(maxAge) + nameSpacesString + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.makeSure') + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions.multipleHosts.login').format(hostNamesList) + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions').format(hdfsUser),
              confirmButton: confirmButton
            });
          } else {
            var hostName = nameNodesWithOldCheckpoints[0].hostName;
            msg = Em.Object.create({
              confirmMsg: Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld').format(maxAge) + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.makeSure') + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions.singleHost.login').format(hostName) + Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions').format(hdfsUser),
              confirmButton: confirmButton
            });
          }
          return App.showConfirmationFeedBackPopup(callback, msg);
        });
      } else if (isNameNodeCheckpointUnavailable) {
        // not available
        return App.showConfirmationPopup(callback, Em.I18n.t('services.service.stop.HDFS.warningMsg.checkPointNA'), null, Em.I18n.t('common.warning'), Em.I18n.t('common.proceedAnyway'), 'danger');
      } else {
        // still young
        callback();
      }
    });
  },

  pullNnCheckPointTime: function pullNnCheckPointTime(haNameSpace) {
    var clusterIdValue = void 0;
    if (haNameSpace) {
      var hostComponents = App.HDFSService.find().objectAt(0).get('hostComponents'),
          correspondingComponent = hostComponents && hostComponents.findProperty('haNameSpace', haNameSpace);
      clusterIdValue = correspondingComponent && correspondingComponent.get('clusterIdValue');
    }
    return App.ajax.send({
      name: 'common.service.hdfs.getNnCheckPointTime',
      sender: this,
      data: {
        clusterIdValue: clusterIdValue
      },
      success: 'parseNnCheckPointTime'
    });
  },

  parseNnCheckPointTime: function parseNnCheckPointTime(data, opt, params) {
    var nameNodesStatus = {},
        nameNodesWithCheckpoints = [],
        hostComponents = params.clusterIdValue ? data.host_components.filterProperty('metrics.dfs.namenode.ClusterId', params.clusterIdValue) : data.host_components;
    if (hostComponents.length <= 1) {
      nameNodesWithCheckpoints.push({
        lastCheckpointTime: Em.get(hostComponents[0], 'metrics.dfs.FSNamesystem.LastCheckpointTime'),
        hostName: Em.get(hostComponents[0], 'HostRoles.host_name')
      });
    } else {
      // HA or federation enabled
      var nameNodeComponents = App.HDFSService.find().objectAt(0).get('hostComponents').filterProperty('componentName', 'NAMENODE'),
          _hostComponents = data.host_components;
      _hostComponents.forEach(function (namenode) {
        var clusterIdValue = Em.get(namenode, 'metrics.dfs.namenode.ClusterId'),
            existingArray = nameNodesStatus[clusterIdValue],
            currentArray = existingArray || [],
            correspondingComponent = nameNodeComponents.findProperty('clusterIdValue', clusterIdValue);
        if (!existingArray) {
          nameNodesStatus[clusterIdValue] = currentArray;
        }
        currentArray.pushObject(Em.Object.create({
          LastCheckpointTime: Em.get(namenode, 'metrics.dfs.FSNamesystem.LastCheckpointTime'),
          HAState: Em.get(namenode, 'metrics.dfs.FSNamesystem.HAState'),
          hostName: Em.get(namenode, 'HostRoles.host_name'),
          haNameSpace: correspondingComponent && correspondingComponent.get('haNameSpace')
        }));
      });
      Object.keys(nameNodesStatus).forEach(function (key) {
        var nameSpace = nameNodesStatus[key],
            activeNameNode = nameSpace.findProperty('HAState', 'active'),
            standbyNameNode = nameSpace.findProperty('HAState', 'standby');
        if (activeNameNode) {
          var lastActiveNameNodeCheckpointTime = activeNameNode.get('LastCheckpointTime'),
              nameNodeWithCheckpoint = nameSpace.findProperty('LastCheckpointTime');
          if (lastActiveNameNodeCheckpointTime) {
            nameNodesWithCheckpoints.push({
              lastCheckpointTime: lastActiveNameNodeCheckpointTime,
              hostName: activeNameNode.get('hostName'),
              haNameSpace: activeNameNode.get('haNameSpace')
            });
          } else if (nameNodeWithCheckpoint) {
            nameNodesWithCheckpoints.push({
              lastCheckpointTime: nameNodeWithCheckpoint.get('LastCheckpointTime'),
              hostName: nameNodeWithCheckpoint.get('hostName'),
              haNameSpace: nameNodeWithCheckpoint.get('haNameSpace')
            });
          }
        } else if (standbyNameNode) {
          nameNodesWithCheckpoints.push({
            lastCheckpointTime: standbyNameNode.get('LastCheckpointTime'),
            hostName: standbyNameNode.get('hostName'),
            haNameSpace: standbyNameNode.get('haNameSpace')
          });
        }
      });
    }
    var timeCriteria = App.nnCheckpointAgeAlertThreshold,
        // time in hours to define how many hours ago is too old
    timeAgo = (Math.round(App.dateTime() / 1000) - timeCriteria * 3600) * 1000,
        nameNodesWithOldCheckpoints = nameNodesWithCheckpoints.filter(function (nameNode) {
      return nameNode.lastCheckpointTime <= timeAgo;
    });

    this.setProperties({
      nameNodesWithOldCheckpoints: nameNodesWithOldCheckpoints,
      isNameNodeCheckpointUnavailable: !nameNodesWithCheckpoints.length
    });
  },

  /**
   * Return true if hdfs user data is loaded via App.MainServiceInfoConfigsController
   */
  getHdfsUser: function getHdfsUser() {
    var self = this;
    var dfd = $.Deferred();
    var miscController = App.MainAdminServiceAccountsController.create();
    miscController.loadUsers();
    var interval = setInterval(function () {
      if (miscController.get('dataIsLoaded') && miscController.get('users')) {
        self.set('hdfsUser', miscController.get('users').findProperty('name', 'hdfs_user').get('value'));
        dfd.resolve();
        clearInterval(interval);
      }
    }, 10);
    return dfd.promise();
  },

  addAdditionalWarningMessage: function addAdditionalWarningMessage(serviceHealth, msg, serviceDisplayName) {
    var servicesAffectedDisplayNames = [];
    var servicesAffected = [];

    if (serviceHealth == 'INSTALLED') {
      //To stop a service, display dependencies message...
      var currentService = this.get('content.serviceName');

      var stackServices = App.StackService.find();
      stackServices.forEach(function (service) {
        if (service.get('isInstalled') || service.get('isSelected')) {
          //only care about services installed...
          var stackServiceDisplayName = service.get("displayName");
          var requiredServices = service.get('requiredServices'); //services required in order to have the current service be functional...
          if (!!requiredServices && requiredServices.length) {
            //only care about services with a non-empty requiredServices list.

            requiredServices.forEach(function (_requiredService) {
              if (currentService === _requiredService) {
                //the service to be stopped is a required service by some other services...
                if (servicesAffected.indexOf(service) == -1) {
                  servicesAffected.push(service);
                  servicesAffectedDisplayNames.push(stackServiceDisplayName);
                }
              }
            }, this);
          }
        }
      }, this);

      var names = servicesAffectedDisplayNames.join();
      if (names) {
        //only display this line with a non-empty dependency list
        var dependenciesMsg = Em.I18n.t('services.service.stop.warningMsg.dependent.services').format(serviceDisplayName, names);
        msg = msg ? msg + " " + dependenciesMsg : dependenciesMsg;
      }
    }

    return msg;
  },

  startStopWithMmode: function startStopWithMmode(serviceHealth, query, runMmOperation, components, hosts, label) {
    var _this6 = this;

    if (runMmOperation) {
      if (serviceHealth === 'STARTED') {
        this.startStopPopupPrimary(serviceHealth, query, components, hosts, label).complete(function () {
          batchUtils.turnOnOffPassiveRequest('OFF', Em.I18n.t('passiveState.turnOff'), _this6.get('content.serviceName').toUpperCase());
        });
      } else {
        batchUtils.turnOnOffPassiveRequest('ON', Em.I18n.t('passiveState.turnOn'), this.get('content.serviceName').toUpperCase()).complete(function () {
          _this6.startStopPopupPrimary(serviceHealth, query, components, hosts, label);
        });
      }
    } else {
      this.startStopPopupPrimary(serviceHealth, query, components, hosts, label);
    }
  },

  startStopPopupPrimary: function startStopPopupPrimary(serviceHealth, query, components, hosts, label) {
    var _this7 = this;

    var isStart = serviceHealth === 'STARTED',
        serviceName = this.get('content.serviceName');
    if (components || hosts) {
      batchUtils.getComponentsFromServer({
        hosts: hosts,
        components: components,
        passiveState: 'OFF',
        displayParams: ['host_components/HostRoles/component_name']
      }, function (data) {
        var contextKey = isStart ? 'services.service.startAllComponents' : 'services.service.stopAllComponents',
            context = Em.I18n.t(contextKey).format(label),
            requestQuery = data.items.map(function (host) {
          var hostName = host.Hosts.host_name,
              componentNames = host.host_components.mapProperty('HostRoles.component_name').join(',');
          return '(HostRoles/component_name.in(' + componentNames + ')&HostRoles/host_name=' + hostName + ')';
        }).join('|');
        App.ajax.send({
          name: 'common.service.host_component.update',
          sender: _this7,
          data: {
            context: context,
            serviceName: serviceName,
            state: serviceHealth,
            query: requestQuery
          },
          showLoadingPopup: true
        });
      });
    } else {
      var context = isStart ? App.BackgroundOperationsController.CommandContexts.START_SERVICE.format(serviceName) : App.BackgroundOperationsController.CommandContexts.STOP_SERVICE.format(serviceName),
          data = {
        context: context,
        serviceName: serviceName.toUpperCase(),
        ServiceInfo: {
          state: serviceHealth
        },
        query: query,
        success: 'startStopPopupSuccessCallback',
        error: 'startStopPopupErrorCallback'
      };

      return App.ajax.send({
        name: 'common.service.update',
        sender: this,
        success: 'startStopPopupSuccessCallback',
        error: 'startStopPopupErrorCallback',
        data: data,
        showLoadingPopup: true
      });
    }
  },

  /**
   * On click callback for <code>start service</code> button
   */
  startService: function startService() {
    this.startStopPopup(App.HostComponentStatus.started);
  },

  /**
   * On click callback for <code>stop service</code> button
   */
  stopService: function stopService() {
    this.startStopPopup(App.HostComponentStatus.stopped);
  },

  /**
   * On click callback for <code>run rebalancer</code> button
   * @param event
   */
  runRebalancer: function runRebalancer(event) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.set("content.runRebalancer", true);
      // load data (if we need to show this background operations popup) from persist
      App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
        if (initValue) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      });
    });
  },
  /**
  * On click handler for Yarn Refresh Queues command from items menu
  * @param event
  */
  refreshYarnQueues: function refreshYarnQueues(event) {
    var controller = this;
    var hosts = App.Service.find('YARN').get('hostComponents').filterProperty('componentName', 'RESOURCEMANAGER').mapProperty('hostName');
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'service.item.refreshQueueYarnRequest',
        sender: controller,
        data: {
          command: "REFRESHQUEUES",
          context: Em.I18n.t('services.service.actions.run.yarnRefreshQueues.context'),
          hosts: hosts.join(','),
          serviceName: "YARN",
          componentName: "RESOURCEMANAGER",
          forceRefreshConfigTags: "capacity-scheduler"
        },
        success: 'refreshYarnQueuesSuccessCallback',
        error: 'refreshYarnQueuesErrorCallback',
        showLoadingPopup: true
      });
    });
  },
  refreshYarnQueuesSuccessCallback: function refreshYarnQueuesSuccessCallback(data, ajaxOptions, params) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },
  refreshYarnQueuesErrorCallback: function refreshYarnQueuesErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error'), error);
  },

  startLdapKnox: function startLdapKnox(event) {
    var context = Em.I18n.t('services.service.actions.run.startLdapKnox.context');
    this.startStopLdapKnox('STARTDEMOLDAP', context);
  },

  stopLdapKnox: function stopLdapKnox(event) {
    var context = Em.I18n.t('services.service.actions.run.stopLdapKnox.context');
    this.startStopLdapKnox('STOPDEMOLDAP', context);
  },

  startStopLdapKnox: function startStopLdapKnox(command, context) {
    var controller = this;
    var host = App.HostComponent.find().findProperty('componentName', 'KNOX_GATEWAY').get('hostName');
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'service.item.startStopLdapKnox',
        sender: controller,
        data: {
          command: command,
          context: context,
          host: host,
          serviceName: "KNOX",
          componentName: "KNOX_GATEWAY"
        },
        success: 'startStopLdapKnoxSuccessCallback',
        error: 'startStopLdapKnoxErrorCallback',
        showLoadingPopup: true
      });
    });
  },

  startStopLdapKnoxSuccessCallback: function startStopLdapKnoxSuccessCallback(data, ajaxOptions, params) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },
  startStopLdapKnoxErrorCallback: function startStopLdapKnoxErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.startStopLdapKnox.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error'), error);
  },

  restartLLAP: function restartLLAP() {
    var isRefreshQueueRequired,
        self = this;
    return App.showConfirmationPopup(function () {
      // regresh queue request is sending only if YARN service has stale configs
      isRefreshQueueRequired = App.Service.find().findProperty('serviceName', 'YARN').get('isRestartRequired');
      if (isRefreshQueueRequired) {
        self.restartLLAPAndRefreshQueueRequest();
      } else {
        self.restartLLAPRequest();
      }
    });
  },

  restartLLAPRequest: function restartLLAPRequest() {
    var host = App.HostComponent.find().findProperty('componentName', 'HIVE_SERVER_INTERACTIVE').get('hostName');
    App.ajax.send({
      name: 'service.item.executeCustomCommand',
      sender: this,
      data: {
        command: 'RESTART_LLAP',
        context: Em.I18n.t('services.service.actions.run.restartLLAP'),
        hosts: host,
        serviceName: "HIVE",
        componentName: "HIVE_SERVER_INTERACTIVE"
      },
      success: 'requestSuccessCallback',
      error: 'requestErrorCallback',
      showLoadingPopup: true
    });
  },

  restartLLAPAndRefreshQueueRequest: function restartLLAPAndRefreshQueueRequest() {
    var hiveServerInteractiveHost = App.HostComponent.find().findProperty('componentName', 'HIVE_SERVER_INTERACTIVE').get('hostName');
    var resourceManagerHost = App.HostComponent.find().findProperty('componentName', 'RESOURCEMANAGER').get('hostName');
    var batches = [{
      "order_id": 1,
      "type": "POST",
      "uri": "/clusters/" + App.get('clusterName') + "/requests",
      "RequestBodyInfo": {
        "RequestInfo": {
          "context": "Refresh YARN Capacity Scheduler",
          "command": "REFRESHQUEUES",
          "parameters/forceRefreshConfigTags": "capacity-scheduler"
        },
        "Requests/resource_filters": [{
          "service_name": "YARN",
          "component_name": "RESOURCEMANAGER",
          "hosts": resourceManagerHost
        }]
      }
    }, {
      "order_id": 2,
      "type": "POST",
      "uri": "/clusters/" + App.get('clusterName') + "/requests",
      "RequestBodyInfo": {
        "RequestInfo": { "context": "Restart LLAP", "command": "RESTART_LLAP" },
        "Requests/resource_filters": [{
          "service_name": "HIVE",
          "component_name": "HIVE_SERVER_INTERACTIVE",
          "hosts": hiveServerInteractiveHost
        }]
      }
    }];
    App.ajax.send({
      name: 'common.batch.request_schedules',
      sender: this,
      data: {
        intervalTimeSeconds: 1,
        tolerateSize: 0,
        batches: batches
      },
      success: 'requestSuccessCallback',
      error: 'requestErrorCallback',
      showLoadingPopup: true
    });
  },

  requestSuccessCallback: function requestSuccessCallback() {
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      if (initValue) {
        App.router.get('backgroundOperationsController').showPopup();
      }
    });
  },

  requestErrorCallback: function requestErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.executeCustomCommand.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('common.error'), error);
  },

  /**
   * On click handler for rebalance Hdfs command from items menu
   */
  rebalanceHdfsNodes: function rebalanceHdfsNodes() {
    var controller = this;
    App.ModalPopup.show({
      classNames: ['forty-percent-width-modal'],
      header: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.context'),
      primary: Em.I18n.t('common.start'),
      secondary: Em.I18n.t('common.cancel'),
      inputValue: 10,
      errorMessage: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.promptError'),
      isInvalid: function () {
        var intValue = Number(this.get('inputValue'));
        return this.get('inputValue') !== 'DEBUG' && (isNaN(intValue) || intValue < 1 || intValue > 100);
      }.property('inputValue'),
      disablePrimary: Em.computed.alias('isInvalid'),
      onPrimary: function onPrimary() {
        if (this.get('isInvalid')) {
          return;
        }
        App.ajax.send({
          name: 'service.item.rebalanceHdfsNodes',
          sender: controller,
          data: {
            hosts: App.Service.find('HDFS').get('hostComponents').findProperty('componentName', 'NAMENODE').get('hostName'),
            threshold: this.get('inputValue')
          },
          success: 'rebalanceHdfsNodesSuccessCallback',
          error: 'rebalanceHdfsNodesErrorCallback',
          showLoadingPopup: true
        });
        this.hide();
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/prompt_popup'),
        text: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.prompt'),
        didInsertElement: function didInsertElement() {
          App.tooltip(this.$(".prompt-input"), {
            placement: "bottom",
            title: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.promptTooltip')
          });
        }
      })
    });
  },
  rebalanceHdfsNodesSuccessCallback: function rebalanceHdfsNodesSuccessCallback(data) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },
  rebalanceHdfsNodesErrorCallback: function rebalanceHdfsNodesErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.error'), error);
  },

  regenerateKeytabFileOperations: function regenerateKeytabFileOperations() {
    var self = this;
    var serviceName = this.content.get('serviceName');
    var clusterName = App.get('clusterName');
    return App.showConfirmationPopup(function () {
      return App.ajax.send({
        name: "admin.kerberos_security.regenerate_keytabs.service",
        sender: self,
        data: {
          clusterName: clusterName,
          serviceName: serviceName
        },
        success: 'regenerateKeytabFileOperationsRequestSuccess',
        error: 'regenerateKeytabFileOperationsRequestError'
      });
    }, Em.I18n.t('question.sure.regenerateKeytab.service').format(serviceName));
  },

  regenerateKeytabFileOperationsRequestSuccess: function regenerateKeytabFileOperationsRequestSuccess() {
    App.router.get('backgroundOperationsController').showPopup();
  },

  regenerateKeytabFileOperationsRequestError: function regenerateKeytabFileOperationsRequestError() {
    App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('alerts.notifications.regenerateKeytab.service.error').format(this.content.get('serviceName')));
  },
  /**
   * On click callback for <code>run compaction</code> button
   * @param event
   */
  runCompaction: function runCompaction(event) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.set("content.runCompaction", true);
      // load data (if we need to show this background operations popup) from persist
      App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
        if (initValue) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      });
    });
  },

  /**
   * On click callback for <code>run smoke test</code> button
   * @param event
   */
  runSmokeTest: function runSmokeTest(event) {
    var self = this;
    if (this.get('content.serviceName') === 'MAPREDUCE2' && !App.Service.find('YARN').get('isStarted')) {
      return App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('services.mapreduce2.smokeTest.requirement'));
    }
    return App.showConfirmationFeedBackPopup(function (query) {
      self.runSmokeTestPrimary(query);
    });
  },

  /**
   * On click handler for Update HBase Replication command from items menu
   */
  updateHBaseReplication: function updateHBaseReplication() {
    var controller = this;
    App.ModalPopup.show({
      classNames: ['sixty-percent-width-modal', 'service-params-popup'],
      header: Em.I18n.t('services.service.actions.run.updateHBaseReplication.context'),
      primary: Em.I18n.t('common.enable'),
      secondary: Em.I18n.t('common.cancel'),
      peerId: '',
      parentzkey: '/hbase',
      zkport: '2181',
      zkquorum: '',
      errorMessage: Em.I18n.t('services.service.actions.run.updateHBaseReplication.promptError'),
      isInvalid: function () {
        var zkquorum = this.get('zkquorum');
        var zkport = this.get('zkport');
        var parentzkey = this.get('parentzkey');
        var peerId = this.get('peerId');
        if (zkquorum && zkport && parentzkey && peerId) {
          if (isNaN(zkport) || isNaN(peerId)) {
            return true;
          }
          var zkquorumArray = zkquorum.split(',');
          return zkquorumArray.length < 2;
        } else {
          return true;
        }
      }.property('zkquorum', 'zkport', 'parentzkey', 'peerId'),
      disablePrimary: Em.computed.alias('isInvalid'),
      onPrimary: function onPrimary() {
        if (this.get('isInvalid')) {
          return;
        }
        App.ajax.send({
          name: 'service.item.updateHBaseReplication',
          sender: controller,
          data: {
            hosts: App.Service.find('HBASE').get('hostComponents').findProperty('componentName', 'HBASE_MASTER').get('hostName'),
            replication_peers: this.get('peerId'),
            replication_cluster_keys: this.get('zkquorum') + ':' + this.get('zkport') + ":" + this.get('parentzkey')
          },
          success: 'updateHBaseReplicationSuccessCallback',
          error: 'updateHBaseReplicationErrorCallback',
          showLoadingPopup: true
        });
        this.hide();
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/update_replication_popup'),
        zkquorumText: Em.I18n.t('services.service.actions.run.updateHBaseReplication.zkquorumText.prompt'),
        zkportText: Em.I18n.t('services.service.actions.run.updateHBaseReplication.zkportText.prompt'),
        parentzkeyText: Em.I18n.t('services.service.actions.run.updateHBaseReplication.parentzkeyText.prompt'),
        peerIdText: Em.I18n.t('services.service.actions.run.updateHBaseReplication.peerIdText.prompt')
      })
    });
  },

  updateHBaseReplicationSuccessCallback: function updateHBaseReplicationSuccessCallback(data) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },

  updateHBaseReplicationErrorCallback: function updateHBaseReplicationErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.updateHBaseReplication.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {
        console.log(err);
      }
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.updateHBaseReplication.error'), error);
  },

  /**
   * On click handler for Stop HBase Replication command from items menu
   */
  stopHBaseReplication: function stopHBaseReplication() {
    var controller = this;
    App.ModalPopup.show({
      classNames: ['forty-percent-width-modal'],
      header: Em.I18n.t('services.service.actions.run.stopHBaseReplication.context'),
      primary: Em.I18n.t('common.disable'),
      secondary: Em.I18n.t('common.cancel'),
      inputValue: '',
      errorMessage: Em.I18n.t('services.service.actions.run.stopHBaseReplication.promptError'),
      isInvalid: function () {
        var inputValue = this.get('inputValue');
        return !inputValue || isNaN(inputValue);
      }.property('inputValue'),
      disablePrimary: Em.computed.alias('isInvalid'),
      onPrimary: function onPrimary() {
        if (this.get('isInvalid')) {
          return;
        }
        App.ajax.send({
          name: 'service.item.stopHBaseReplication',
          sender: controller,
          data: {
            hosts: App.Service.find('HBASE').get('hostComponents').findProperty('componentName', 'HBASE_MASTER').get('hostName'),
            replication_peers: this.get('inputValue')
          },
          success: 'stopHBaseReplicationSuccessCallback',
          error: 'stopHBaseReplicationErrorCallback',
          showLoadingPopup: true
        });
        this.hide();
      },
      bodyClass: Ember.View.extend({
        templateName: require('templates/common/modal_popups/prompt_popup'),
        text: Em.I18n.t('services.service.actions.run.stopHBaseReplication.prompt')
      })
    });
  },

  stopHBaseReplicationSuccessCallback: function stopHBaseReplicationSuccessCallback(data) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },

  stopHBaseReplicationErrorCallback: function stopHBaseReplicationErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.stopHBaseReplication.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {
        console.log(err);
      }
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.stopHBaseReplication.error'), error);
  },

  restartAllHostComponents: function restartAllHostComponents(serviceName) {
    var serviceDisplayName = this.get('content.displayName'),
        bodyMessage = Em.Object.create({
      putInMaintenance: this.get('content.passiveState') === 'OFF',
      turnOnMmMsg: Em.I18n.t('passiveState.turnOnFor').format(serviceDisplayName),
      confirmMsg: Em.I18n.t('services.service.restartAll.confirmMsg').format(serviceDisplayName),
      confirmButton: Em.I18n.t('services.service.restartAll.confirmButton'),
      additionalWarningMsg: this.get('content.passiveState') === 'OFF' ? Em.I18n.t('services.service.restartAll.warningMsg.turnOnMM').format(serviceDisplayName) : null
    });

    // check HDFS NameNode checkpoint before stop service
    if (this.hasStartedNameNode()) {
      this.checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
          batchUtils.restartAllServiceHostComponents(serviceDisplayName, serviceName, false, query, runMmOperation);
        }, bodyMessage);
      });
    } else {
      return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
        batchUtils.restartAllServiceHostComponents(serviceDisplayName, serviceName, false, query, runMmOperation);
      }, bodyMessage);
    }
  },

  restartCertainHostComponents: function restartCertainHostComponents(context) {
    var serviceDisplayName = this.get('content.displayName'),
        components = context.components,
        hosts = context.hosts,
        label = context.label,
        serviceName = context.serviceName;

    if (hosts) {
      var bodyMessage = Em.Object.create({
        putInMaintenance: this.get('content.passiveState') === 'OFF',
        turnOnMmMsg: Em.I18n.t('passiveState.turnOnFor').format(serviceDisplayName),
        confirmMsg: Em.I18n.t('services.service.restartAll.confirmMsg').format(Em.I18n.t('services.service.componentsInNameSpace').format(label)),
        confirmButton: Em.I18n.t('services.service.restartAll.confirmButton'),
        additionalWarningMsg: this.get('content.passiveState') === 'OFF' ? Em.I18n.t('services.service.restartCertain.warningMsg.turnOnMM').format(serviceDisplayName) : null
      });
      if (this.hasStartedNameNode(label)) {
        this.checkNnLastCheckpointTime(function () {
          return App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
            batchUtils.restartCertainServiceHostComponents(serviceName, components, hosts, label, query, runMmOperation);
          }, bodyMessage);
        }, label);
      } else {
        App.showConfirmationFeedBackPopup(function (query, runMmOperation) {
          batchUtils.restartCertainServiceHostComponents(serviceName, components, hosts, label, query, runMmOperation);
        }, bodyMessage);
      }
    } else {
      this.restartAllHostComponents(serviceName);
    }
  },

  startCertainHostComponents: function startCertainHostComponents(context) {
    if (context.hosts) {
      this.startStopCertainPopup(context, App.HostComponentStatus.started);
    } else {
      this.startService();
    }
  },

  stopCertainHostComponents: function stopCertainHostComponents(context) {
    if (context.hosts) {
      this.startStopCertainPopup(context, App.HostComponentStatus.stopped);
    } else {
      this.stopService();
    }
  },

  hasStartedNameNode: function hasStartedNameNode(nameSpace) {
    return this.get('content.serviceName') == 'HDFS' && this.get('content.hostComponents').some(function (component) {
      return component.get('componentName') === 'NAMENODE' && (!nameSpace || component.get('haNameSpace') === nameSpace) && component.get('workStatus') === App.HostComponentStatus.started;
    });
  },

  turnOnOffPassive: function turnOnOffPassive(label) {
    var self = this;
    var state = this.get('content.passiveState') == 'OFF' ? 'ON' : 'OFF';
    var onOff = state === 'ON' ? "On" : "Off";
    return App.showConfirmationPopup(function () {
      batchUtils.turnOnOffPassiveRequest(state, label, self.get('content.serviceName').toUpperCase(), function (data, opt, params) {
        self.set('content.passiveState', params.passive_state);
        batchUtils.infoPassiveState(params.passive_state);
      });
    }, Em.I18n.t('hosts.passiveMode.popup').format(onOff, self.get('content.displayName')));
  },

  rollingRestart: function rollingRestart(hostComponentName) {
    batchUtils.launchHostComponentRollingRestart(hostComponentName, this.get('content.displayName'), this.get('content.passiveState') === "ON", false, this.get('content.passiveState') === "ON");
  },

  runSmokeTestPrimary: function runSmokeTestPrimary(query) {
    var clusterLevelRequired = ['KERBEROS'];
    var requestData = {
      'serviceName': this.get('content.serviceName'),
      'displayName': this.get('content.displayName'),
      'actionName': this.get('content.serviceName') === 'ZOOKEEPER' ? 'ZOOKEEPER_QUORUM_SERVICE_CHECK' : this.get('content.serviceName') + '_SERVICE_CHECK',
      'query': query
    };
    if (clusterLevelRequired.contains(this.get('content.serviceName'))) {
      requestData.operationLevel = {
        "level": "CLUSTER",
        "cluster_name": App.get('clusterName')
      };
    }
    App.ajax.send({
      'name': 'service.item.smoke',
      'sender': this,
      'success': 'runSmokeTestSuccessCallBack',
      'error': 'runSmokeTestErrorCallBack',
      'data': requestData,
      showLoadingPopup: true
    });
  },

  runSmokeTestSuccessCallBack: function runSmokeTestSuccessCallBack(data, ajaxOptions, params) {
    if (data.Requests.id) {
      // load data (if we need to show this background operations popup) from persist
      App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
        params.query.set('status', 'SUCCESS');
        if (initValue) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      });
    } else {
      params.query.set('status', 'FAIL');
    }
  },
  runSmokeTestErrorCallBack: function runSmokeTestErrorCallBack(request, ajaxOptions, error, opt, params) {
    params.query.set('status', 'FAIL');
  },

  /**
   * On click callback for <code>Reassign <master component></code> button
   * @param hostComponent
   */
  reassignMaster: function reassignMaster(hostComponent) {
    var component = App.HostComponent.find().findProperty('componentName', hostComponent);
    if (component) {
      var reassignMasterController = App.router.get('reassignMasterController');
      reassignMasterController.saveComponentToReassign(component);
      reassignMasterController.setCurrentStep('1');
      App.router.transitionTo('reassign');
    }
  },

  /**
   * On click callback for <code>action</code> dropdown menu
   * Calls runSmokeTest, runRebalancer, runCompaction or reassignMaster depending on context
   * @param event
   */
  doAction: function doAction(event) {
    if ($(event.target).hasClass('disabled') || $(event.target.parentElement).hasClass('disabled')) {
      return;
    }
    var methodName = event.context.action;
    var context = event.context.context;
    if (methodName) {
      this[methodName](context);
    }
  },

  /**
   * Restart clients host components to apply config changes
   */
  refreshConfigs: function refreshConfigs() {
    var self = this;
    if (this.get('isClientsOnlyService') || this.get('content.serviceName') == "FLUME") {
      return App.showConfirmationFeedBackPopup(function (query) {
        batchUtils.getComponentsFromServer({
          services: [self.get('content.serviceName')]
        }, function (data) {
          var hostComponents = [];

          data.items.forEach(function (host) {
            host.host_components.forEach(function (hostComponent) {
              hostComponents.push(Em.Object.create({
                componentName: hostComponent.HostRoles.component_name,
                hostName: host.Hosts.host_name
              }));
            });
          });
          batchUtils.restartHostComponents(hostComponents, Em.I18n.t('rollingrestart.context.allForSelectedService').format(self.get('content.serviceName')), "SERVICE", query);
        });
      });
    }
  },

  /**
   * Send command to server to install client on selected host
   * @param componentName
   */
  addComponent: function addComponent(componentName) {
    var component = App.StackServiceComponent.find().findProperty('componentName', componentName);

    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      App.router.get('mainHostDetailsController').addComponentWithCheck({
        context: component,
        fromServiceSummary: true
      });
    });
  },

  /**
   * set property isPending (if this property is true - means that service has task in BGO)
   * and this makes start/stop button disabled
   */
  setStartStopState: function () {
    var serviceName = this.get('content.serviceName');
    var backgroundOperations = App.router.get('backgroundOperationsController.services');
    if (backgroundOperations && backgroundOperations.length > 0) {
      for (var i = 0; i < backgroundOperations.length; i++) {
        if (backgroundOperations[i].isRunning && (backgroundOperations[i].dependentService === "ALL_SERVICES" || backgroundOperations[i].dependentService === serviceName)) {
          this.set('isPending', true);
          return;
        }
      }
      this.set('isPending', false);
    } else {
      this.set('isPending', true);
    }
  }.observes('App.router.backgroundOperationsController.serviceTimestamp'),

  nonClientServiceComponents: function () {
    return App.MasterComponent.find().toArray().concat(App.SlaveComponent.find().toArray()).filterProperty('service.serviceName', this.get('content.serviceName'));
  }.property('content.serviceName'),

  isStartDisabled: function () {
    var allComponentsStarted = true;
    if (this.get('isPending')) return true;
    this.get('nonClientServiceComponents').forEach(function (component) {
      if (component.get('installedCount') > 0) allComponentsStarted = false;
    });
    return allComponentsStarted && this.get('content.isStarted');
  }.property('content.isStarted', 'isPending'),

  isStopDisabled: function () {
    var allComponentsStopped = true;
    if (this.get('isPending')) return true;

    if (App.get('isHaEnabled') && this.get('content.serviceName') == 'HDFS' && this.get('content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
      return false;
    }
    if (this.get('content.serviceName') == 'HAWQ' && this.get('content.hostComponents').filterProperty('componentName', 'HAWQMASTER').someProperty('workStatus', App.HostComponentStatus.started)) {
      return false;
    }
    if (this.get('content.serviceName') == 'PXF' && App.HostComponent.find().filterProperty('componentName', 'PXF').someProperty('workStatus', App.HostComponentStatus.started)) {
      return false;
    }

    this.get('nonClientServiceComponents').forEach(function (component) {
      if (component.get('startedCount') > 0) allComponentsStopped = false;
    });

    return allComponentsStopped && !this.get('content.isStarted');
  }.property('content.isStarted', 'isPending', 'App.isHaEnabled'),

  isSmokeTestDisabled: function () {
    if (this.get('isClientsOnlyService')) return false;
    // Disable PXF service check if at least one PXF is down
    if (this.get('content.serviceName') === 'PXF') return App.HostComponent.find().filterProperty('componentName', 'PXF').someProperty('workStatus', 'INSTALLED');
    return this.get('isStopDisabled');
  }.property('content.serviceName'),

  /**
   * Determine if service has than one service client components
   */
  isSeveralClients: function () {
    return App.StackServiceComponent.find().filterProperty('serviceName', this.get('content.serviceName')).filterProperty('isClient').length > 1;
  }.property('content.serviceName'),

  enableHighAvailability: function enableHighAvailability() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.enableHighAvailability();
  },

  manageJournalNode: function manageJournalNode() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.manageJournalNode();
  },

  disableHighAvailability: function disableHighAvailability() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.disableHighAvailability();
  },

  enableRMHighAvailability: function enableRMHighAvailability() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.enableRMHighAvailability();
  },

  addHawqStandby: function addHawqStandby() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.addHawqStandby();
  },

  removeHawqStandby: function removeHawqStandby() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.removeHawqStandby();
  },

  activateHawqStandby: function activateHawqStandby() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.activateHawqStandby();
  },

  enableRAHighAvailability: function enableRAHighAvailability() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.enableRAHighAvailability();
  },

  downloadClientConfigs: function downloadClientConfigs(event) {
    var component = this.get('content.clientComponents').rejectProperty('totalCount', 0)[0];
    this.downloadClientConfigsCall({
      serviceName: this.get('content.serviceName'),
      componentName: event && event.name || component.get('componentName'),
      resourceType: this.resourceTypeEnum.SERVICE_COMPONENT
    });
  },

  openNameNodeFederationWizard: function openNameNodeFederationWizard() {
    var highAvailabilityController = App.router.get('mainAdminHighAvailabilityController');
    highAvailabilityController.enableNameNodeFederation();
  },

  /**
   * This method is called when user event to download configs for "All Clients"
   * is made from service action menu
   */
  downloadAllClientConfigs: function downloadAllClientConfigs() {
    this.downloadClientConfigsCall({
      serviceName: this.get('content.serviceName'),
      resourceType: this.resourceTypeEnum.SERVICE
    });
  },

  /**
   * On click handler for custom hawq command from items menu
   * @param context
   */
  executeHawqCustomCommand: function executeHawqCustomCommand(context) {
    var controller = this;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'service.item.executeCustomCommand',
        sender: controller,
        data: {
          command: context.command,
          context: context.label,
          hosts: App.Service.find(context.service).get('hostComponents').findProperty('componentName', context.component).get('hostName'),
          serviceName: context.service,
          componentName: context.component
        },
        success: 'executeCustomCommandSuccessCallback',
        error: 'executeCustomCommandErrorCallback',
        showLoadingPopup: true
      });
    });
  },

  /**
   * On click handler for custom command from items menu
   * @param context
   */
  executeCustomCommand: function executeCustomCommand(context) {
    var controller = this;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'service.item.executeCustomCommand',
        sender: controller,
        data: {
          command: context.command,
          context: 'Execute ' + context.command,
          hosts: App.Service.find(context.service).get('hostComponents').findProperty('componentName', context.component).get('hostName'),
          serviceName: context.service,
          componentName: context.component,
          forceRefreshConfigTags: "capacity-scheduler"
        },
        success: 'executeCustomCommandSuccessCallback',
        error: 'executeCustomCommandErrorCallback',
        showLoadingPopup: true
      });
    });
  },

  executeCustomCommandSuccessCallback: function executeCustomCommandSuccessCallback(data, ajaxOptions, params) {
    if (data.Requests.id) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  },

  executeCustomCommandErrorCallback: function executeCustomCommandErrorCallback(data) {
    var error = Em.I18n.t('services.service.actions.run.executeCustomCommand.error');
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.executeCustomCommand.error'), error);
  },

  /**
   * find dependent services
   * @param {string[]} serviceNamesToDelete
   * @returns {Array}
   */
  findDependentServices: function findDependentServices(serviceNamesToDelete) {
    var dependentServices = [];

    App.Service.find().forEach(function (service) {
      if (!serviceNamesToDelete.contains(service.get('serviceName'))) {
        var requiredServices = App.StackService.find(service.get('serviceName')).get('requiredServices');
        serviceNamesToDelete.forEach(function (dependsOnService) {
          if (requiredServices.contains(dependsOnService)) {
            dependentServices.push(service.get('serviceName'));
          }
        });
      }
    }, this);
    return dependentServices;
  },

  /**
   * @param serviceNames
   * @returns {string}
   */
  servicesDisplayNames: function servicesDisplayNames(serviceNames) {
    return serviceNames.map(function (serviceName) {
      return App.format.role(serviceName, true);
    }).join(',');
  },

  /**
   * Is services can be removed based on work status
   * @param serviceNames
   */
  allowUninstallServices: function allowUninstallServices(serviceNames) {
    return App.Service.find().filter(function (service) {
      return serviceNames.contains(service.get('serviceName'));
    }).everyProperty('allowToDelete');
  },

  /**
   * delete service action
   * @param {string} serviceName
   */
  deleteService: function deleteService(serviceName) {
    var self = this,
        interDependentServices = this.get('interDependentServices'),
        serviceNamesToDelete = this.get('serviceNamesToDelete'),
        dependentServices = this.findDependentServices(serviceNamesToDelete),
        displayName = App.format.role(serviceName, true),
        popupHeader = Em.I18n.t('services.service.delete.popup.header'),
        dependentServicesToDeleteFmt = this.servicesDisplayNames(interDependentServices);

    if (serviceName === 'KERBEROS') {
      this.kerberosDeleteWarning(popupHeader);
      return;
    }
    if (serviceName === 'RANGER' && this.isRangerPluginEnabled()) {
      App.ModalPopup.show({
        secondary: null,
        header: popupHeader,
        encodeBody: false,
        body: Em.I18n.t('services.service.delete.popup.ranger')
      });
      return;
    }

    if (App.Service.find().get('length') === 1) {
      //at least one service should be installed
      App.ModalPopup.show({
        secondary: null,
        header: popupHeader,
        encodeBody: false,
        body: Em.I18n.t('services.service.delete.lastService.popup.body').format(displayName)
      });
    } else if (dependentServices.length > 0) {
      this.dependentServicesWarning(serviceName, dependentServices);
    } else {
      var isServiceInRemovableState = this.allowUninstallServices(serviceNamesToDelete);
      if (isServiceInRemovableState) {
        if (serviceName === 'RANGER_KMS') {
          App.showConfirmationPopup(function () {
            self.showLastWarning(serviceName, interDependentServices, dependentServicesToDeleteFmt);
          }, Em.I18n.t('services.service.delete.popup.warning.ranger_kms'), null, popupHeader, Em.I18n.t('common.delete'), 'danger');
        } else {
          this.showLastWarning(serviceName, interDependentServices, dependentServicesToDeleteFmt);
        }
      } else {
        var body = Em.I18n.t('services.service.delete.popup.mustBeStopped').format(displayName);
        if (interDependentServices.length) {
          body += Em.I18n.t('services.service.delete.popup.mustBeStopped.dependent').format(dependentServicesToDeleteFmt);
        }
        App.ModalPopup.show({
          secondary: null,
          header: popupHeader,
          encodeBody: false,
          body: body
        });
      }
    }
  },

  /**
   * show dialog with Kerberos warning prior to service delete
   * @param {string} header
   * @returns {App.ModalPopup}
   */
  kerberosDeleteWarning: function kerberosDeleteWarning(header) {
    return App.ModalPopup.show({
      primary: Em.I18n.t('ok'),
      secondary: Em.I18n.t('services.alerts.goTo').format('Kerberos'),
      header: header,
      encodeBody: false,
      body: Em.I18n.t('services.service.delete.popup.kerberos'),
      onSecondary: function onSecondary() {
        this._super();
        App.router.transitionTo('main.admin.adminKerberos.index');
      }
    });
  },

  /**
   * @returns {Boolean}
   */
  isRangerPluginEnabled: function isRangerPluginEnabled() {
    return App.router.get('mainServiceInfoSummaryController.rangerPlugins').filterProperty('isDisplayed').someProperty('status', 'Enabled');
  },

  /**
   * warning that show dependent services which must be deleted prior to chosen service deletion
   * @param {string} origin
   * @param {string[]} dependent
   * @returns {App.ModalPopup}
   */
  dependentServicesWarning: function dependentServicesWarning(origin, dependent) {
    return App.ModalPopup.show({
      secondary: null,
      header: Em.I18n.t('services.service.delete.popup.header'),
      dependentMessage: Em.I18n.t('services.service.delete.popup.dependentServices').format(App.format.role(origin, true)),
      dependentServices: dependent,
      bodyClass: Em.View.extend({
        templateName: require('templates/main/service/info/dependent_services_warning')
      })
    });
  },

  showLastWarning: function showLastWarning(serviceName, interDependentServices, dependentServicesToDeleteFmt) {
    var self = this,
        displayName = App.format.role(serviceName, true),
        popupHeader = Em.I18n.t('services.service.delete.popup.header'),
        popupPrimary = Em.I18n.t('common.delete'),
        warningMessage = Em.I18n.t('services.service.delete.popup.warning').format(displayName) + (interDependentServices.length ? Em.I18n.t('services.service.delete.popup.warning.dependent').format(dependentServicesToDeleteFmt) : ''),
        callback = this.loadConfigRecommendations.bind(this, null, function () {
      var serviceNames = self.get('changedProperties').mapProperty('serviceName').uniq();
      self.loadConfigGroups(serviceNames).done(function () {
        self.set('isRecommendationInProgress', false);
      });
    });
    this.clearRecommendations();
    this.setProperties({
      isRecommendationInProgress: true,
      selectedConfigGroup: Em.Object.create({
        isDefault: true
      })
    });
    this.loadConfigs();
    App.get('router.mainController.isLoading').call(this, 'isServiceConfigsLoaded').done(callback);
    return App.ModalPopup.show({
      controller: self,
      header: popupHeader,
      primary: popupPrimary,
      primaryClass: 'btn-danger',
      disablePrimary: Em.computed.alias('controller.isRecommendationInProgress'),
      classNameBindings: ['controller.changedProperties.length:common-modal-wrapper'],
      modalDialogClasses: Em.computed.ifThenElse('controller.changedProperties.length', ['modal-xlg'], []),
      bodyClass: Em.View.extend({
        templateName: require('templates/main/service/info/delete_service_warning_popup'),
        warningMessage: new Em.Handlebars.SafeString(warningMessage)
      }),
      onPrimary: function onPrimary() {
        self.confirmDeleteService(serviceName, interDependentServices, dependentServicesToDeleteFmt);
        this._super();
      },
      onSecondary: function onSecondary() {
        self.clearRecommendations();
        this._super();
      },
      onClose: function onClose() {
        self.clearRecommendations();
        this._super();
      }
    });
  },

  /**
   * Confirmation popup of service deletion
   * @param {string} serviceName
   * @param {string[]} [dependentServiceNames]
   * @param {string} [servicesToDeleteFmt]
   */
  confirmDeleteService: function confirmDeleteService(serviceName, dependentServiceNames, servicesToDeleteFmt) {
    var confirmKey = 'delete',
        self = this,
        message = Em.I18n.t('services.service.confirmDelete.popup.body').format(App.format.role(serviceName, true), confirmKey);

    if (dependentServiceNames.length > 0) {
      message = Em.I18n.t('services.service.confirmDelete.popup.body.dependent').format(App.format.role(serviceName, true), servicesToDeleteFmt, confirmKey);
    }

    App.ModalPopup.show({

      /**
       * @function onPrimary
       */
      onPrimary: function onPrimary() {
        var serviceNames = [serviceName].concat(dependentServiceNames),
            serviceDisplayNames = serviceNames.map(function (serviceName) {
          return App.Service.find(serviceName).get('displayName');
        }),
            progressPopup = App.ModalPopup.show({
          classNames: ['delete-service-progress'],
          header: Em.I18n.t('services.service.delete.popup.header'),
          showFooter: false,
          message: Em.I18n.t('services.service.delete.progressPopup.message').format(stringUtils.getFormattedStringFromArray(serviceDisplayNames)),
          bodyClass: Em.View.extend({
            classNames: ['delete-service-progress-body'],
            template: Em.Handlebars.compile('{{view App.SpinnerView}}<div class="progress-message">{{message}}</div>')
          }),
          onClose: function onClose() {
            self.set('deleteServiceProgressPopup', null);
            this._super();
          }
        });
        App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
          self.set('deleteServiceProgressPopup', progressPopup);
          self.deleteServiceCall(serviceNames);
        });
        this._super();
      },

      /**
       * @type {string}
       */
      primary: Em.I18n.t('common.delete'),

      /**
       * @type {string}
       */
      primaryClass: 'btn-danger',

      /**
       * @type {string}
       */
      header: Em.I18n.t('services.service.confirmDelete.popup.header'),

      /**
       * @type {string}
       */
      confirmInput: '',

      /**
       * @type {boolean}
       */
      disablePrimary: Em.computed.notEqual('confirmInput', confirmKey),

      message: message,

      /**
       * @type {Em.View}
       */
      bodyClass: Em.View.extend({
        confirmKey: confirmKey,
        typeMessage: Em.I18n.t('services.service.confirmDelete.popup.body.type').format(confirmKey),
        templateName: require('templates/main/service/info/confirm_delete_service')
      }),

      enterKeyPressed: function enterKeyPressed() {
        if (this.get('disablePrimary')) return;
        this.onPrimary();
      }
    });
  },

  /**
   * All host names
   * This property required for request for recommendations
   *
   * @type {String[]}
   * @override
   */
  hostNames: Em.computed.alias('App.allHostNames'),

  /**
   * Recommendation object
   * This property required for request for recommendations
   *
   * @type {Object}
   * @override
   */
  hostGroups: function () {
    var hostGroup = blueprintUtils.generateHostGroups(App.get('allHostNames'));
    return blueprintUtils.removeDeletedComponents(hostGroup, [this.get('serviceNamesToDelete')]);
  }.property('serviceNamesToDelete', 'App.allHostNames', 'App.componentToBeAdded', 'App.componentToBeDeleted'),

  /**
   * List of services without removed
   * This property required for request for recommendations
   *
   * @type {String[]}
   * @override
   */
  serviceNames: function () {
    return App.Service.find().filter(function (s) {
      return !this.get('serviceNamesToDelete').contains(s.get('serviceName'));
    }, this).mapProperty('serviceName');
  }.property('serviceNamesToDelete'),

  /**
   * This property required for request for recommendations
   *
   * @return {Boolean}
   * @override
   */
  isConfigHasInitialState: function isConfigHasInitialState() {
    return false;
  },

  /**
   * Describes condition when recommendation should be applied
   * Unfortunately for removing services it's not always true
   * Property should be updated only if it depends on service that will be removed
   * (similar as on add service)
   *
   * @return {Boolean}
   * @override
   */
  allowUpdateProperty: function allowUpdateProperty(parentProperties, name, fileName, configGroup, savedValue) {
    var stackProperty = App.configsCollection.getConfigByName(name, fileName);
    if (!stackProperty || stackProperty.serviceName === this.get('content.serviceName')) {
      /**
       * update properties for current service (in case will be used not only for removing service)
       * and properties that are not defined in stack
       */
      return true;
    }
    if (stackProperty.propertyDependsOn.length) {
      /**
       * update properties that depends on current service
       */
      return stackProperty.propertyDependsOn.some(function (p) {
        var service = App.config.get('serviceByConfigTypeMap')[p.type];
        return service && this.get('content.serviceName') === service.get('serviceName');
      }, this);
    }
    return !Em.isNone(savedValue) && stackProperty.recommendedValue === savedValue;
  },

  /**
   * Just config version note
   *
   * @type {String}
   */
  serviceConfigVersionNote: function () {
    var services = this.get('serviceNamesToDelete').join(',');
    if (this.get('serviceNamesToDelete.length') === 1) {
      return Em.I18n.t('services.service.delete.configVersionNote').format(services);
    }
    return Em.I18n.t('services.service.delete.configVersionNote.plural').format(services);
  }.property('serviceNamesToDelete'),

  /**
   * Method ot save configs after service have been removed
   * @override
   */
  saveConfigs: function saveConfigs() {
    var data = [],
        progressPopup = this.get('deleteServiceProgressPopup'),
        stepConfigs = this.get('stepConfigs');

    this.applyRecommendedValues(stepConfigs);

    stepConfigs.forEach(function (stepConfig) {
      var serviceConfig = this.getServiceConfigToSave(stepConfig.get('serviceName'), stepConfig.get('configs'));

      if (serviceConfig) {
        data.push(serviceConfig);
      }
    }, this);

    if (Em.isArray(data) && data.length) {
      this.putChangedConfigurations(data, 'confirmServiceDeletion', function () {
        if (progressPopup) {
          progressPopup.onClose();
        }
      });
    } else {
      this.confirmServiceDeletion();
    }
  },

  applyRecommendedValues: function applyRecommendedValues(stepConfigs) {
    var changedProperties = this.get('changedProperties');
    changedProperties.forEach(function (property) {
      var serviceConfigs = stepConfigs.findProperty('serviceName', property.serviceName);
      if (serviceConfigs) {
        var prop = serviceConfigs.get('configs').findProperty('name', property.propertyName);
        if (prop) {
          prop.set('value', property.saveRecommended ? property.recommendedValue : property.initialValue);
        }
      }
    });
    return stepConfigs;
  },

  confirmServiceDeletion: function confirmServiceDeletion() {
    var serviceNames = void 0,
        msg = void 0;
    if (this.get('interDependentServices.length')) {
      serviceNames = this.get('serviceNamesToDelete').map(function (serviceName) {
        return App.format.role(serviceName, true);
      }).join(', ');
      msg = Em.I18n.t('services.service.delete.service.success.confirmation.plural').format(serviceNames);
    } else {
      serviceNames = App.format.role(this.get('content.serviceName'), true);
      msg = Em.I18n.t('services.service.delete.service.success.confirmation').format(serviceNames);
    }

    var progressPopup = this.get('deleteServiceProgressPopup');
    if (progressPopup) {
      progressPopup.onClose();
    }
    return App.showAlertPopup(Em.I18n.t('popup.confirmation.commonHeader'), msg, function () {
      window.location.reload();
    });
  },

  /**
   * Ajax call to delete service
   * @param {string[]} serviceNames
   * @returns {$.ajax}
   */
  deleteServiceCall: function deleteServiceCall(serviceNames) {
    var serviceToDeleteNow = serviceNames[0];
    if (serviceNames.length > 1) {
      var servicesToDeleteNext = serviceNames.slice(1);
    }
    App.Service.find().findProperty('serviceName', serviceToDeleteNow).set('deleteInProgress', true);
    return App.ajax.send({
      name: 'common.delete.service',
      sender: this,
      data: {
        serviceName: serviceToDeleteNow,
        servicesToDeleteNext: servicesToDeleteNext
      },
      success: 'deleteServiceCallSuccessCallback',
      error: 'deleteServiceCallErrorCallback',
      showLoadingPopup: true
    });
  },

  deleteServiceCallSuccessCallback: function deleteServiceCallSuccessCallback(data, ajaxOptions, params) {
    if (params.servicesToDeleteNext) {
      this.deleteServiceCall(params.servicesToDeleteNext);
    } else {
      this.saveConfigs();
    }
  },

  deleteServiceCallErrorCallback: function deleteServiceCallErrorCallback(jqXHR, ajaxOptions, error, opt) {
    var progressPopup = this.get('deleteServiceProgressPopup');
    if (progressPopup) {
      progressPopup.onClose();
    }
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
  }

});

});

require.register("controllers/main/service/manage_config_groups_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');
var hostsManagement = require('utils/hosts');
var numberUtils = require('utils/number_utils');

App.ManageConfigGroupsController = Em.Controller.extend(App.ConfigOverridable, {

  name: 'manageConfigGroupsController',

  /**
   * Determines if needed data is already loaded
   * Loading chain starts at <code>loadHosts</code> and is complete on the <code>loadConfigGroups</code> (if user on
   * the Installer) or on the <code>_onLoadConfigGroupsSuccess</code> (otherwise)
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * Determines if user currently is on the Cluster Installer
   * @type {boolean}
   */
  isInstaller: false,

  /**
   * Determines if user currently is on the Add Service Wizard
   * @type {boolean}
   */
  isAddService: false,

  /**
   * Current service name
   * @type {string}
   */
  serviceName: null,

  /**
   * @type {App.ConfigGroup[]}
   */
  configGroups: [],

  /**
   * @type {App.ConfigGroup[]}
   */
  originalConfigGroups: [],

  /**
   * @type {App.ConfigGroup}
   */
  selectedConfigGroup: null,

  /**
   * @type {string[]}
   */
  selectedHosts: [],

  /**
   * List of all hosts in the cluster
   * @type {{
   *  id: string,
   *  ip: string,
   *  osType: string,
   *  osArch: string,
   *  hostName: string,
   *  publicHostName: string,
   *  cpu: number,
   *  memory: number,
   *  diskTotal: string,
   *  diskFree: string,
   *  disksMounted: number,
   *  hostComponents: {
   *    componentName: string,
   *    displayName: string
   *  }[]
   * }[]}
   */
  clusterHosts: [],

  /**
   * trigger <code>selectDefaultGroup</code> after group delete
   * @type {null}
   */
  groupDeleteTrigger: null,

  /**
   * List of available service components for <code>serviceName</code>
   * @type {{componentName: string, displayName: string, selected: boolean}[]}
   */
  componentsForFilter: function () {
    return App.StackServiceComponent.find().filterProperty('serviceName', this.get('serviceName')).map(function (component) {
      return Em.Object.create({
        componentName: component.get('componentName'),
        displayName: App.format.role(component.get('componentName'), false),
        selected: false
      });
    });
  }.property('serviceName'),

  /**
   * Determines when host may be deleted from config group
   * @type {boolean}
   */
  isDeleteHostsDisabled: function () {
    var selectedConfigGroup = this.get('selectedConfigGroup');
    if (selectedConfigGroup) {
      return selectedConfigGroup.get('isDefault') || this.get('selectedHosts').length === 0;
    }
    return true;
  }.property('selectedConfigGroup', 'selectedConfigGroup.hosts.length', 'selectedHosts.length'),

  /**
   * Map with modified/deleted/created config groups
   * @type {{
   *  toClearHosts: App.ConfigGroup[],
   *  toDelete: App.ConfigGroup[],
   *  toSetHosts: App.ConfigGroup[],
   *  toCreate: App.ConfigGroup[]
   * }}
   */
  hostsModifiedConfigGroups: {},

  /**
   * Trim the tooltip text to show first 500 characters of properties list
   * @type {string}
   */
  tooltipText: function () {
    var selectedConfigGroup = this.get('selectedConfigGroup'),
        propertiesList = selectedConfigGroup.get('propertiesList'),
        trimLength = 500,
        trimmedText = "",
        noOfRemainingProperties = 0,
        index = 0,
        propertyText = "",
        addDots = false;
    if (propertiesList.length > trimLength) {
      // Adjust trim length based on occurrence of <br/> around trim length
      index = propertiesList.substring(trimLength - 10, trimLength + 10).indexOf("<br/>");
      if (index > -1) {
        trimLength = trimLength - 10 + index;
      } else {
        addDots = true;
      }
      trimmedText = propertiesList.substring(0, trimLength);
      if (addDots) {
        trimmedText += " ...";
      }
      noOfRemainingProperties = (propertiesList.substring(trimLength).match(new RegExp("<br/>", "g")) || []).length - 1;
      if (noOfRemainingProperties > 0) {
        propertyText = noOfRemainingProperties > 1 ? "properties" : "property";
        trimmedText += "<br/> and " + noOfRemainingProperties + " more " + propertyText;
      }
    } else {
      trimmedText = propertiesList;
    }
    return trimmedText;
  }.property('selectedConfigGroup.propertiesList'),

  /**
   * Check when some config group was changed and updates <code>hostsModifiedConfigGroups</code> once
   * @method hostsModifiedConfigGroupsObs
   */
  hostsModifiedConfigGroupsObs: function () {
    Em.run.once(this, this.hostsModifiedConfigGroupsObsOnce);
  }.observes('selectedConfigGroup.hosts.@each', 'selectedConfigGroup.hosts.length', 'selectedConfigGroup.description', 'configGroups', 'isLoaded'),

  /**
   * Update <code>hostsModifiedConfigGroups</code>-value
   * Called once in the <code>hostsModifiedConfigGroupsObs</code>
   * @method hostsModifiedConfigGroupsObsOnce
   * @returns {boolean}
   */
  hostsModifiedConfigGroupsObsOnce: function hostsModifiedConfigGroupsObsOnce() {
    if (!this.get('isLoaded')) {
      return false;
    }
    var groupsToClearHosts = [];
    var groupsToDelete = [];
    var groupsToSetHosts = [];
    var groupsToCreate = [];
    var groups = this.get('configGroups');
    var originalGroups = [];
    var originalGroupsMap = {};

    this.get('originalConfigGroups').forEach(function (item) {
      if (!item.is_default) {
        originalGroupsMap[item.id] = item;
        originalGroups.push(item);
      }
    }, this);

    groups.forEach(function (groupRecord) {
      if (!groupRecord.get('isDefault')) {
        var originalGroup = originalGroupsMap[groupRecord.get('id')];
        if (originalGroup) {
          if (!(JSON.stringify(groupRecord.get('hosts').slice().sort()) === JSON.stringify(originalGroup.hosts.sort()))) {
            groupsToClearHosts.push(groupRecord);
            if (groupRecord.get('hosts').length) {
              groupsToSetHosts.push(groupRecord);
            }
            // should update name or description
          } else if (groupRecord.get('description') !== originalGroup.description || groupRecord.get('name') !== originalGroup.name) {
            groupsToSetHosts.push(groupRecord);
          }
          delete originalGroupsMap[groupRecord.get('id')];
        } else {
          groupsToCreate.push({
            id: groupRecord.get('id'),
            name: groupRecord.get('name'),
            description: groupRecord.get('description'),
            hosts: groupRecord.get('hosts').slice(0),
            service_id: groupRecord.get('serviceName'),
            desired_configs: groupRecord.get('desiredConfigs'),
            properties: groupRecord.get('properties')
          });
        }
      }
    });

    //groups to delete
    for (var id in originalGroupsMap) {
      groupsToDelete.push(App.ServiceConfigGroup.find(id));
    }

    this.set('hostsModifiedConfigGroups', {
      toClearHosts: groupsToClearHosts,
      toDelete: groupsToDelete,
      toSetHosts: groupsToSetHosts,
      toCreate: groupsToCreate,
      initialGroups: originalGroups
    });
  },

  /**
   * Determines if some changes were done with config groups
   * @use hostsModifiedConfigGroups
   * @type {boolean}
   */
  isHostsModified: function () {
    if (!this.get('isLoaded')) {
      return false;
    }
    var ignoreKeys = ['initialGroups'];
    var modifiedGroups = this.get('hostsModifiedConfigGroups');
    return Em.keys(modifiedGroups).map(function (key) {
      return ignoreKeys.contains(key) ? 0 : Em.get(modifiedGroups[key], 'length');
    }).reduce(Em.sum, 0) > 0;
  }.property('hostsModifiedConfigGroups'),

  /**
   * Resort config groups according to order:
   * default group first, other - last
   * @method resortConfigGroup
   */
  resortConfigGroup: function () {
    var configGroups = Em.copy(this.get('configGroups'));
    if (configGroups.length < 2) return;
    var defaultConfigGroup = configGroups.findProperty('isDefault');
    configGroups.removeObject(defaultConfigGroup);
    var sorted = [defaultConfigGroup].concat(configGroups.sortProperty('name'));

    this.removeObserver('configGroups.@each.name', this, 'resortConfigGroup');
    this.set('configGroups', sorted);
    this.addObserver('configGroups.@each.name', this, 'resortConfigGroup');
  }.observes('configGroups.@each.name'),

  /**
   * Load hosts from server or
   *  get them from installerController if user on the install wizard
   *  get them from isAddServiceController if user on the add service wizard
   * @method loadHosts
   */
  loadHosts: function loadHosts() {
    this.set('isLoaded', false);
    if (this.get('isInstaller') && !this.get('isAddService')) {
      var hostNames = App.router.get('installerController').get('allHosts').mapProperty('hostName').join();
      this.loadInstallerHostsFromServer(hostNames);
    } else {
      this.loadHostsFromServer();
    }
    this.loadConfigGroups(this.get('serviceName'));
  },

  /**
   * Request all hosts directly from server
   * @method loadHostsFromServer
   * @return {$.ajax}
   */
  loadHostsFromServer: function loadHostsFromServer() {
    return App.ajax.send({
      name: 'hosts.config_groups',
      sender: this,
      data: {},
      success: '_loadHostsFromServerSuccessCallback',
      error: '_loadHostsFromServerErrorCallback'
    });
  },

  /**
   * Success-callback for <code>loadHostsFromServer</code>
   * Parse hosts response and wrap them into Ember.Object
   * @param {object} data
   * @method _loadHostsFromServerSuccessCallback
   * @private
   */
  _loadHostsFromServerSuccessCallback: function _loadHostsFromServerSuccessCallback(data) {
    var wrappedHosts = [],
        newlyAddedHostComponentsMap = this.getNewlyAddedHostComponentsMap();

    data.items.forEach(function (host) {
      var hostComponents = [];
      var diskInfo = host.Hosts.disk_info.filter(function (item) {
        return (/^ext|^ntfs|^fat|^xfs/i.test(item.type)
        );
      });
      if (diskInfo.length) {
        diskInfo = diskInfo.reduce(function (a, b) {
          return {
            available: parseInt(a.available) + parseInt(b.available),
            size: parseInt(a.size) + parseInt(b.size)
          };
        });
      }
      host.host_components.forEach(function (hostComponent) {
        hostComponents.push(Em.Object.create({
          componentName: hostComponent.HostRoles.component_name,
          displayName: App.format.role(hostComponent.HostRoles.component_name, false)
        }));
      }, this);
      if (this.get('isAddService') && newlyAddedHostComponentsMap[host.Hosts.host_name]) {
        hostComponents.pushObjects(newlyAddedHostComponentsMap[host.Hosts.host_name]);
      }
      wrappedHosts.pushObject(Em.Object.create({
        id: host.Hosts.host_name,
        ip: host.Hosts.ip,
        osType: host.Hosts.os_type,
        osArch: host.Hosts.os_arch,
        hostName: host.Hosts.host_name,
        publicHostName: host.Hosts.public_host_name,
        cpu: host.Hosts.cpu_count,
        memory: host.Hosts.total_mem,
        diskTotal: numberUtils.bytesToSize(diskInfo.size, 0, undefined, 1024),
        diskFree: numberUtils.bytesToSize(diskInfo.available, 0, undefined, 1024),
        disksMounted: host.Hosts.disk_info.length,
        hostComponents: hostComponents
      }));
    }, this);

    this.set('clusterHosts', wrappedHosts);
  },

  /**
   * Error-callback for <code>loadHostsFromServer</code>
   * @method _loadHostsFromServerErrorCallback
   * @private
   */
  _loadHostsFromServerErrorCallback: function _loadHostsFromServerErrorCallback() {
    this.set('clusterHosts', []);
  },

  /**
   *
   * @returns {{}}
   */
  getNewlyAddedHostComponentsMap: function getNewlyAddedHostComponentsMap() {
    var newlyAddedHostComponentsMap = {};
    var masters = App.router.get('addServiceController.content.masterComponentHosts') || [];
    var slaves = App.router.get('addServiceController.content.slaveComponentHosts') || [];
    var clients = App.router.get('addServiceController.content.clients') || [];

    clients = clients.filterProperty('isInstalled', false).map(function (component) {
      return Em.Object.create({
        componentName: component.component_name,
        displayName: component.display_name
      });
    });

    masters.forEach(function (component) {
      if (!component.isInstalled) {
        if (!newlyAddedHostComponentsMap[component.hostName]) {
          newlyAddedHostComponentsMap[component.hostName] = [];
        }
        newlyAddedHostComponentsMap[component.hostName].push(Em.Object.create({
          componentName: component.component,
          displayName: component.display_name
        }));
      }
    });

    slaves.forEach(function (component) {
      component.hosts.forEach(function (host) {
        if (!host.isInstalled) {
          if (!newlyAddedHostComponentsMap[host.hostName]) {
            newlyAddedHostComponentsMap[host.hostName] = [];
          }
          if (component.componentName === 'CLIENT') {
            newlyAddedHostComponentsMap[host.hostName].pushObjects(clients);
          } else {
            newlyAddedHostComponentsMap[host.hostName].push(Em.Object.create({
              componentName: component.componentName,
              displayName: component.displayName
            }));
          }
        }
      });
    });
    return newlyAddedHostComponentsMap;
  },

  /**
   * Load config groups from server if user is on the already installed cluster
   * If not - use loaded data form wizardStep7Controller
   * @param {string} serviceName
   * @method loadConfigGroups
   */
  loadConfigGroups: function loadConfigGroups(serviceName) {
    if (this.get('isInstaller')) {
      var configGroups = App.router.get('wizardStep7Controller.selectedService.configGroups').slice(0);
      var originalConfigGroups = this.generateOriginalConfigGroups(configGroups);
      this.setProperties({
        configGroups: configGroups,
        originalConfigGroups: originalConfigGroups,
        isLoaded: true
      });
    } else {
      this.set('serviceName', serviceName);
      App.ajax.send({
        name: 'service.load_config_groups',
        data: {
          serviceName: serviceName
        },
        sender: this,
        success: '_onLoadConfigGroupsSuccess'
      });
    }
  },

  /**
   * Success-callback for <code>loadConfigGroups</code>
   * @param {object} data
   * @private
   * @method _onLoadConfigGroupsSuccess
   */
  _onLoadConfigGroupsSuccess: function _onLoadConfigGroupsSuccess(data) {
    var serviceName = this.get('serviceName');

    App.configGroupsMapper.map(data, false, [serviceName]);

    var configGroups = App.ServiceConfigGroup.find().filterProperty('serviceName', serviceName);
    var rawConfigGroups = this.generateOriginalConfigGroups(configGroups);
    var groupToTypeToTagMap = {};

    rawConfigGroups.forEach(function (item) {
      if (Array.isArray(item.desired_configs)) {
        item.desired_configs.forEach(function (config) {
          if (!groupToTypeToTagMap[item.name]) {
            groupToTypeToTagMap[item.name] = {};
          }
          groupToTypeToTagMap[item.name][config.type] = config.tag;
        });
      }
    });

    this.set('configGroups', configGroups);
    this.set('originalConfigGroups', rawConfigGroups);
    this.loadProperties(groupToTypeToTagMap);
    this.set('isLoaded', true);
  },

  /**
   *
   * @param {Array} configGroups
   * @returns {Array}
   */
  generateOriginalConfigGroups: function generateOriginalConfigGroups(configGroups) {
    var self = this;
    return configGroups.map(function (item) {
      return self.createOriginalRecord(item);
    });
  },

  /**
   *  Return object to use for loading to model with correct names for object keys
   * @param configGroup - config group object from model
   * @returns {Object}
   */
  createOriginalRecord: function createOriginalRecord(configGroup) {
    return {
      id: configGroup.get('id'),
      name: configGroup.get('name'),
      service_name: configGroup.get('serviceName'),
      description: configGroup.get('description'),
      hosts: configGroup.get('hosts').slice(0),
      service_id: configGroup.get('serviceName'),
      desired_configs: configGroup.get('desiredConfigs'),
      is_default: configGroup.get('isDefault'),
      child_config_groups: configGroup.get('childConfigGroups') ? configGroup.get('childConfigGroups').mapProperty('id') : [],
      parent_config_group_id: configGroup.get('parentConfigGroup.id'),
      properties: configGroup.get('properties')
    };
  },

  /**
   *
   * @param {object} groupToTypeToTagMap
   * @method loadProperties
   */
  loadProperties: function loadProperties(groupToTypeToTagMap) {
    var typeTagToGroupMap = {};
    var urlParams = [];
    for (var group in groupToTypeToTagMap) {
      var overrideTypeTags = groupToTypeToTagMap[group];
      for (var type in overrideTypeTags) {
        var tag = overrideTypeTags[type];
        typeTagToGroupMap[type + "///" + tag] = group;
        urlParams.push('(type=' + type + '&tag=' + tag + ')');
      }
    }
    var params = urlParams.join('|');
    if (urlParams.length) {
      App.ajax.send({
        name: 'config.host_overrides',
        sender: this,
        data: {
          params: params,
          typeTagToGroupMap: typeTagToGroupMap
        },
        success: '_onLoadPropertiesSuccess'
      });
    }
  },

  /**
   * Success-callback for <code>loadProperties</code>
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @private
   * @method _onLoadPropertiesSuccess
   */
  _onLoadPropertiesSuccess: function _onLoadPropertiesSuccess(data, opt, params) {
    var groupToPropertiesMap = {};
    data.items.forEach(function (configs) {
      var group = params.typeTagToGroupMap[configs.type + "///" + configs.tag];
      if (!groupToPropertiesMap[group]) {
        groupToPropertiesMap[group] = [];
      }
      for (var config in configs.properties) {
        groupToPropertiesMap[group].push({
          name: config,
          value: configs.properties[config],
          type: configs.type
        });
      }
    }, this);
    for (var g in groupToPropertiesMap) {
      this.get('configGroups').findProperty('name', g).set('properties', groupToPropertiesMap[g]);
    }
  },

  /**
   * Show popup with properties overridden in the selected config group
   * @method showProperties
   */
  showProperties: function showProperties() {
    var properties = this.get('selectedConfigGroup.propertiesList').htmlSafe();
    if (properties) {
      App.showAlertPopup(Em.I18n.t('services.service.config_groups_popup.properties'), properties);
    }
  },

  /**
   * Show popup with hosts to add to the selected config group
   * @returns {boolean}
   * @method addHosts
   */
  addHosts: function addHosts() {
    if (this.get('selectedConfigGroup.isAddHostsDisabled')) {
      return false;
    }
    var availableHosts = this.get('selectedConfigGroup.availableHosts');
    var popupDescription = {
      header: Em.I18n.t('hosts.selectHostsDialog.title'),
      dialogMessage: Em.I18n.t('hosts.selectHostsDialog.message').format(this.get('selectedConfigGroup.displayName'))
    };
    hostsManagement.launchHostsSelectionDialog(availableHosts, [], false, this.get('componentsForFilter'), this.addHostsCallback.bind(this), popupDescription);
  },

  /**
   * Remove selected hosts from default group (<code>selectedConfigGroup.parentConfigGroup</code>) and add them to the <code>selectedConfigGroup</code>
   * @param {string[]} selectedHosts
   * @method addHostsCallback
   */
  addHostsCallback: function addHostsCallback(selectedHosts) {
    if (selectedHosts) {
      var sortedHosts;
      var group = this.get('selectedConfigGroup');
      var parentGroupHosts = group.get('parentConfigGroup.hosts');
      var newHostsForParentGroup = parentGroupHosts.filter(function (hostName) {
        return !selectedHosts.contains(hostName);
      });
      group.get('hosts').pushObjects(selectedHosts);
      sortedHosts = group.get('hosts').sort();
      group.set('hosts', sortedHosts);
      group.set('parentConfigGroup.hosts', newHostsForParentGroup);
    }
  },

  /**
   * Delete hosts from <code>selectedConfigGroup</code> and move them to the Default group (<code>selectedConfigGroup.parentConfigGroup</code>)
   * @method deleteHosts
   */
  deleteHosts: function deleteHosts() {
    if (this.get('isDeleteHostsDisabled')) {
      return;
    }
    var hosts = this.get('selectedHosts').slice();
    var newHosts = [];
    var selectedGroup = this.get('selectedConfigGroup');
    var parentGroup = this.get('selectedConfigGroup.parentConfigGroup');
    selectedGroup.get('hosts').forEach(function (host) {
      if (!hosts.contains(host)) {
        newHosts.pushObject(host);
      }
    });
    selectedGroup.set('hosts', newHosts);
    parentGroup.set('hosts', parentGroup.get('hosts').pushObjects(hosts).slice().sort());
    this.set('selectedHosts', []);
  },

  /**
   * show popup for confirmation delete config group
   * @method confirmDelete
   */
  confirmDelete: function confirmDelete() {
    var self = this;
    App.showConfirmationPopup(function () {
      self.deleteConfigGroup();
    });
  },

  /**
   * delete selected config group (stored in the <code>selectedConfigGroup</code>)
   * then select default config group
   * @method deleteConfigGroup
   */
  deleteConfigGroup: function deleteConfigGroup() {
    var selectedConfigGroup = this.get('selectedConfigGroup');
    if (this.get('isDeleteGroupDisabled')) {
      return;
    }
    //move hosts of group to default group (available hosts)
    this.set('selectedHosts', selectedConfigGroup.get('hosts'));
    this.deleteHosts();
    this.get('configGroups').removeObject(selectedConfigGroup);
    App.configGroupsMapper.deleteRecord(selectedConfigGroup);
    this.set('selectedConfigGroup', this.get('configGroups').findProperty('isDefault'));
    this.propertyDidChange('groupDeleteTrigger');
  },

  /**
   * rename new config group (not allowed for default group)
   * @method renameConfigGroup
   */
  renameConfigGroup: function renameConfigGroup() {
    if (this.get('selectedConfigGroup.isDefault')) {
      return;
    }
    var self = this;
    var renameGroupPopup = App.ModalPopup.show({
      header: Em.I18n.t('services.service.config_groups.rename_config_group_popup.header'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/service/new_config_group')
      }),

      configGroupName: self.get('selectedConfigGroup.name'),

      configGroupDesc: self.get('selectedConfigGroup.description'),

      warningMessage: null,

      isDescriptionDirty: false,

      validate: function () {
        var warningMessage = '';
        var originalGroup = self.get('selectedConfigGroup');
        var groupName = this.get('configGroupName').trim();
        if (originalGroup.get('description') !== this.get('configGroupDesc') && !this.get('isDescriptionDirty')) {
          this.set('isDescriptionDirty', true);
        }
        if (originalGroup.get('name').trim() === groupName) {
          if (this.get('isDescriptionDirty')) {
            warningMessage = '';
          } else {
            warningMessage = Em.I18n.t("config.group.selection.dialog.err.name.exists");
          }
        } else {
          if (self.get('configGroups').mapProperty('name').contains(groupName)) {
            warningMessage = Em.I18n.t("config.group.selection.dialog.err.name.exists");
          } else if (groupName && !validator.isValidConfigGroupName(groupName)) {
            warningMessage = Em.I18n.t("form.validator.configGroupName");
          }
        }
        this.set('warningMessage', warningMessage);
      }.observes('configGroupName', 'configGroupDesc'),

      disablePrimary: function () {
        return !(this.get('configGroupName').trim().length > 0 && this.get('warningMessage') !== null && !this.get('warningMessage'));
      }.property('warningMessage', 'configGroupName', 'configGroupDesc'),

      onPrimary: function onPrimary() {
        self.get('selectedConfigGroup').setProperties({
          name: this.get('configGroupName'),
          description: this.get('configGroupDesc')
        });
        App.store.fastCommit();
        this.hide();
      }
    });
    this.set('renameGroupPopup', renameGroupPopup);
  },

  /**
   * add new config group (or copy existing)
   * @param {boolean} duplicated true - copy <code>selectedConfigGroup</code>, false - create a new one
   * @method addConfigGroup
   */
  addConfigGroup: function addConfigGroup(duplicated) {
    duplicated = duplicated === true;

    var self = this;

    var addGroupPopup = App.ModalPopup.show({
      header: Em.I18n.t('services.service.config_groups.add_config_group_popup.header'),

      bodyClass: Em.View.extend({
        templateName: require('templates/main/service/new_config_group')
      }),

      configGroupName: duplicated ? self.get('selectedConfigGroup.name') + ' Copy' : "",

      configGroupDesc: duplicated ? self.get('selectedConfigGroup.description') + ' (Copy)' : "",

      warningMessage: '',

      didInsertElement: function didInsertElement() {
        this._super();
        this.validate();
        this.$('input').focus();
      },

      validate: function () {
        var warningMessage = '';
        var groupName = this.get('configGroupName').trim();
        if (self.get('configGroups').mapProperty('name').contains(groupName)) {
          warningMessage = Em.I18n.t("config.group.selection.dialog.err.name.exists");
        } else if (groupName && !validator.isValidConfigGroupName(groupName)) {
          warningMessage = Em.I18n.t("form.validator.configGroupName");
        }
        this.set('warningMessage', warningMessage);
      }.observes('configGroupName'),

      disablePrimary: function () {
        return !(this.get('configGroupName').trim().length > 0 && !this.get('warningMessage'));
      }.property('warningMessage', 'configGroupName'),

      onPrimary: function onPrimary() {
        var defaultConfigGroup = self.get('configGroups').findProperty('isDefault'),
            properties = [],
            serviceName = self.get('serviceName'),
            groupName = this.get('configGroupName').trim(),
            newGroupId = new Date().getTime();

        if (duplicated) {
          self.get('selectedConfigGroup.properties').forEach(function (item) {
            var property = App.ServiceConfigProperty.create($.extend(false, {}, item));
            property.set('group', App.ServiceConfigGroup.find(newGroupId));
            properties.push(property);
          });
        }

        App.store.safeLoad(App.ServiceConfigGroup, {
          id: newGroupId,
          name: groupName,
          description: this.get('configGroupDesc'),
          isDefault: false,
          parent_config_group_id: App.ServiceConfigGroup.getParentConfigGroupId(serviceName),
          service_id: serviceName,
          service_name: serviceName,
          hosts: [],
          desired_configs: duplicated ? self.get('selectedConfigGroup.desiredConfigs') : [],
          properties: duplicated ? properties : [],
          is_temporary: true
        });
        App.store.fastCommit();
        var childConfigGroups = defaultConfigGroup.get('childConfigGroups').mapProperty('id');
        childConfigGroups.push(newGroupId);
        App.store.safeLoad(App.ServiceConfigGroup, App.configGroupsMapper.generateDefaultGroup(self.get('serviceName'), defaultConfigGroup.get('hosts'), childConfigGroups));
        App.store.fastCommit();
        self.get('configGroups').pushObject(App.ServiceConfigGroup.find(newGroupId));
        this.hide();
      }
    });
    this.set('addGroupPopup', addGroupPopup);
  },

  /**
   * Duplicate existing config group
   * @method duplicateConfigGroup
   */
  duplicateConfigGroup: function duplicateConfigGroup() {
    this.addConfigGroup(true);
  },

  /**
   * Show popup with config groups
   * User may edit/create/delete them
   * @param {Em.Controller} controller
   * @param {App.Service} service
   * @returns {App.ModalPopup}
   * @method manageConfigurationGroups
   */
  manageConfigurationGroups: function manageConfigurationGroups(controller, service) {
    var configsController = this;
    var serviceData = controller && controller.get('selectedService') || service;
    var serviceName = serviceData.get('serviceName');
    var displayName = serviceData.get('displayName');
    this.setProperties({
      isInstaller: !!controller,
      serviceName: serviceName
    });
    if (controller) {
      configsController.set('isAddService', controller.get('content.controllerName') == 'addServiceController');
    }
    return App.ModalPopup.show({

      header: Em.I18n.t('services.service.config_groups_popup.header').format(displayName),

      bodyClass: App.MainServiceManageConfigGroupView.extend({
        serviceName: serviceName,
        displayName: displayName,
        controller: configsController
      }),

      classNames: ['common-modal-wrapper', 'manage-configuration-group-popup'],
      modalDialogClasses: ['modal-lg'],

      primary: Em.I18n.t('common.save'),

      autoHeight: false,

      subViewController: configsController,

      /**
       * handle onPrimary action particularly in wizard
       * @param {Em.Controller} controller
       * @param {object} modifiedConfigGroups
       */
      onPrimaryWizard: function onPrimaryWizard(controller, modifiedConfigGroups) {
        controller.set('selectedService.configGroups', configsController.get('configGroups'));
        controller.selectedServiceObserver();
        if (controller.get('name') == "wizardStep7Controller") {
          if (controller.get('selectedService.selected') === false && modifiedConfigGroups.toDelete.length > 0) {
            controller.setGroupsToDelete(modifiedConfigGroups.toDelete);
          }
          configsController.persistConfigGroups();
          this.updateConfigGroupOnServicePage();
        }
        this.hide();
      },

      onClose: function onClose() {
        //<code>_super</code> has to be called before <code>resetGroupChanges</code>
        var originalGroups = this.get('subViewController.originalConfigGroups').slice(0);
        this._super();
        this.resetGroupChanges(originalGroups);
      },

      onSecondary: function onSecondary() {
        this.onClose();
      },

      /**
       * reset group changes made by user
       * @param {Array} originalGroups
       */
      resetGroupChanges: function resetGroupChanges(originalGroups) {
        if (this.get('subViewController.isHostsModified')) {
          App.ServiceConfigGroup.find().clear();
          App.store.safeLoadMany(App.ServiceConfigGroup, originalGroups);
        }
      },

      /**
       * run requests which delete config group and clear its hosts
       * @param {Function} finishFunction
       * @param {object} modifiedConfigGroups
       */
      runClearCGQueue: function runClearCGQueue(finishFunction, modifiedConfigGroups) {
        var counter = 0;
        var dfd = $.Deferred();
        var doneFunction = function doneFunction(xhr, text, errorThrown) {
          counter--;
          if (counter === 0) dfd.resolve();
          finishFunction(xhr, text, errorThrown);
        };

        modifiedConfigGroups.toClearHosts.forEach(function (cg) {
          counter++;
          configsController.updateConfigurationGroup(cg, doneFunction, doneFunction);
        }, this);
        modifiedConfigGroups.toDelete.forEach(function (cg) {
          counter++;
          configsController.deleteConfigurationGroup(cg, doneFunction, doneFunction);
        }, this);
        if (counter === 0) dfd.resolve();
        return dfd.promise();
      },

      /**
       * run requests which change properties of config group
       * @param {Function} finishFunction
       * @param {object} modifiedConfigGroups
       */
      runModifyCGQueue: function runModifyCGQueue(finishFunction, modifiedConfigGroups) {
        var counter = 0;
        var dfd = $.Deferred();
        var doneFunction = function doneFunction(xhr, text, errorThrown) {
          counter--;
          if (counter === 0) dfd.resolve();
          finishFunction(xhr, text, errorThrown);
        };

        modifiedConfigGroups.toSetHosts.forEach(function (cg) {
          counter++;
          configsController.updateConfigurationGroup(cg, doneFunction, doneFunction);
        }, this);
        if (counter === 0) dfd.resolve();
        return dfd.promise();
      },

      /**
       * run requests which create new config group
       * @param {Function} finishFunction
       * @param {object} modifiedConfigGroups
       */
      runCreateCGQueue: function runCreateCGQueue(finishFunction, modifiedConfigGroups) {
        var counter = 0;
        var dfd = $.Deferred();
        var doneFunction = function doneFunction(xhr, text, errorThrown) {
          counter--;
          if (counter === 0) dfd.resolve();
          finishFunction(xhr, text, errorThrown);
        };

        modifiedConfigGroups.toCreate.forEach(function (cg) {
          counter++;
          configsController.postNewConfigurationGroup(cg, doneFunction);
        }, this);
        if (counter === 0) dfd.resolve();
        return dfd.promise();
      },

      onPrimary: function onPrimary() {
        var modifiedConfigGroups = configsController.get('hostsModifiedConfigGroups');
        var errors = [];
        var self = this;
        var finishFunction = function finishFunction(xhr, text, errorThrown) {
          if (xhr && typeof errorThrown === 'string') {
            var error = xhr.status + "(" + errorThrown + ") ";
            try {
              var json = $.parseJSON(xhr.responseText);
              error += json.message;
            } catch (err) {}
            errors.push(error);
          }
        };

        // Save modified config-groups
        if (controller) {
          //called only in Wizard
          return this.onPrimaryWizard(controller, modifiedConfigGroups);
        }

        this.runClearCGQueue(finishFunction, modifiedConfigGroups).done(function () {
          self.runModifyCGQueue(finishFunction, modifiedConfigGroups).done(function () {
            self.runCreateCGQueue(finishFunction, modifiedConfigGroups).done(function () {
              if (errors.length > 0) {
                self.get('subViewController').set('errorMessage', errors.join(". "));
              } else {
                if (!self.get('isAddService') && !self.get('isInstaller')) {
                  //update service config versions only if it is service configs page
                  App.router.get('mainServiceInfoConfigsController').loadServiceConfigVersions().done(function () {
                    self.updateConfigGroupOnServicePage();
                    self.hide();
                  });
                } else {
                  self.updateConfigGroupOnServicePage();
                  self.hide();
                }
              }
            });
          });
        });
      },

      updateConfigGroupOnServicePage: function updateConfigGroupOnServicePage() {
        var selectedConfigGroup = configsController.get('selectedConfigGroup');
        var managedConfigGroups = configsController.get('configGroups').slice(0);
        if (!controller) {
          controller = App.router.get('mainServiceInfoConfigsController');
          //controller.set('configGroups', managedConfigGroups);
          controller.loadConfigGroups([controller.get('content.serviceName')]);
        } else {
          controller.set('selectedService.configGroups', managedConfigGroups);
        }

        var selectEventObject = {};
        //check whether selectedConfigGroup exists
        if (selectedConfigGroup && controller.get('configGroups').someProperty('name', selectedConfigGroup.get('name'))) {
          selectEventObject.context = selectedConfigGroup;
        } else {
          selectEventObject.context = managedConfigGroups.findProperty('isDefault', true);
        }
        controller.selectConfigGroup(selectEventObject);
      },

      updateButtons: function () {
        var modified = this.get('subViewController.isHostsModified');
        this.set('disablePrimary', !modified);
      }.observes('subViewController.isHostsModified')
    });
  },

  loadInstallerHostsFromServer: function loadInstallerHostsFromServer(hostNames) {
    return App.ajax.send({
      name: 'hosts.info.install',
      sender: this,
      data: {
        hostNames: hostNames
      },
      success: 'loadInstallerHostsSuccessCallback'
    });
  },

  loadInstallerHostsSuccessCallback: function loadInstallerHostsSuccessCallback(data) {
    var allHosts = App.router.get('installerController.allHosts').toMapByProperty('hostName'),
        slaveComponents = App.router.get('installerController.content.slaveComponentHosts'),
        clientComponents = App.router.get('installerController.content.clients'),
        clients = clientComponents.map(function (client) {
      return {
        componentName: client.component_name,
        displayName: client.display_name
      };
    });
    var hosts = [];

    slaveComponents.forEach(function (component) {
      component.hosts.forEach(function (rawHost) {
        var host = allHosts[rawHost.hostName];
        if (!host.hostComponents) {
          host.hostComponents = [];
        }
        if (component.componentName === 'CLIENT') {
          host.hostComponents.pushObjects(clients);
        }
      });
    });

    data.items.forEach(function (host) {
      var disksOverallCapacity = 0,
          diskFree = 0;
      host.Hosts.disk_info.forEach(function (disk) {
        disksOverallCapacity += parseFloat(disk.size);
        diskFree += parseFloat(disk.available);
      });
      hosts.pushObject(Em.Object.create({
        id: host.Hosts.host_name,
        ip: host.Hosts.ip,
        osType: host.Hosts.os_type,
        osArch: host.Hosts.os_arch,
        hostName: host.Hosts.host_name,
        publicHostName: host.Hosts.public_host_name,
        cpu: host.Hosts.cpu_count,
        memory: host.Hosts.total_mem.toFixed(2),
        diskInfo: host.Hosts.disk_info,
        diskTotal: disksOverallCapacity / (1024 * 1024),
        diskFree: diskFree / (1024 * 1024),
        hostComponents: allHosts[host.Hosts.host_name] && allHosts[host.Hosts.host_name].hostComponents || []
      }));
    });

    this.set('clusterHosts', hosts);
  }

});

});

require.register("controllers/main/service/reassign/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep1Controller = Em.Controller.extend({
  name: 'reassignMasterWizardStep1Controller',
  databaseType: null,

  /**
   * @type {object}
   */
  dbPropertyMap: {
    'HIVE_SERVER': {
      type: 'hive-env',
      name: 'hive_database_type'
    },
    'HIVE_METASTORE': {
      type: 'hive-env',
      name: 'hive_database_type'
    },
    'OOZIE_SERVER': {
      type: 'oozie-site',
      name: 'oozie.service.JPAService.jdbc.driver'
    }
  },

  loadConfigsTags: function loadConfigsTags() {
    App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'onLoadConfigsTags',
      error: ''
    });
  },

  /**
   * construct URL parameters for config call
   * @param componentName
   * @param data
   * @return {Array}
   */
  getConfigUrlParams: function getConfigUrlParams(componentName, data) {
    var urlParams = [];
    switch (componentName) {
      case 'OOZIE_SERVER':
        urlParams.push('(type=oozie-site&tag=' + data.Clusters.desired_configs['oozie-site'].tag + ')');
        urlParams.push('(type=oozie-env&tag=' + data.Clusters.desired_configs['oozie-env'].tag + ')');
        break;
      case 'HIVE_SERVER':
      case 'HIVE_METASTORE':
        urlParams.push('(type=hive-site&tag=' + data.Clusters.desired_configs['hive-site'].tag + ')');
        urlParams.push('(type=hive-env&tag=' + data.Clusters.desired_configs['hive-env'].tag + ')');
        break;
    }
    return urlParams;
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var urlParams = this.getConfigUrlParams(this.get('content.reassign.component_name'), data);

    if (urlParams.length > 0) {
      App.ajax.send({
        name: 'reassign.load_configs',
        sender: this,
        data: {
          urlParams: urlParams.join('|')
        },
        success: 'onLoadConfigs',
        error: ''
      });
    }
  },

  onLoadConfigs: function onLoadConfigs(data) {
    var databaseProperty,
        databaseType = null,
        databaseTypeMatch,
        properties = {},
        configs = {},
        dbPropertyMapItem = Em.getWithDefault(this.get('dbPropertyMap'), this.get('content.reassign.component_name'), null);

    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
    });

    this.get('content').setProperties({
      serviceProperties: properties,
      configs: configs
    });

    if (dbPropertyMapItem) {
      databaseProperty = Em.getWithDefault(configs, dbPropertyMapItem.type, {})[dbPropertyMapItem.name];
      databaseTypeMatch = databaseProperty && databaseProperty.match(/MySQL|PostgreS|Oracle|Derby|MSSQL|Anywhere/gi);
      if (databaseTypeMatch) {
        databaseType = databaseTypeMatch[0];
      }
    }
    this.set('databaseType', databaseType);

    if (this.get('content.reassign.component_name') == 'OOZIE_SERVER' && databaseType !== 'derby') {
      App.router.reassignMasterController.set('content.hasManualSteps', false);
    }

    properties['is_remote_db'] = this.isExistingDb(configs);
    properties['database_hostname'] = this.getDatabaseHost();

    this.saveDatabaseType(databaseType);
    this.saveServiceProperties(properties);
    this.saveConfigs(configs);
  },

  saveDatabaseType: function saveDatabaseType(type) {
    if (type) {
      App.router.get(this.get('content.controllerName')).saveDatabaseType(type);
    }
  },

  saveServiceProperties: function saveServiceProperties(properties) {
    if (properties) {
      App.router.get(this.get('content.controllerName')).saveServiceProperties(properties);
    }
  },

  saveConfigs: function saveConfigs(configs) {
    if (configs) {
      App.router.get(this.get('content.controllerName')).saveConfigs(configs);
    }
  },

  isExistingDb: function isExistingDb(configs) {
    var serviceName = this.get('content.reassign.service_id').toLowerCase();
    var serviceDbSite = serviceName + '-env';
    var serviceDbConfig = serviceName + '_database';
    return (/Existing/ig.test(configs[serviceDbSite][serviceDbConfig])
    );
  },

  getDatabaseHost: function getDatabaseHost() {
    var db_type = this.get('databaseType'),
        connectionURLProps = {
      'HIVE': {
        type: 'hive-site',
        name: 'javax.jdo.option.ConnectionURL'
      },
      'OOZIE': {
        type: 'oozie-site',
        name: 'oozie.service.JPAService.jdbc.url'
      }
    },
        service = this.get('content.reassign.service_id'),
        connectionURLPropsItem = connectionURLProps[service],
        connectionURL = Em.getWithDefault(this.get('content.configs'), connectionURLPropsItem.type, {})[connectionURLPropsItem.name];

    connectionURL = connectionURL.replace("jdbc:" + db_type + "://", "");
    connectionURL = connectionURL.replace("/hive?createDatabaseIfNotExist=true", "");
    connectionURL = connectionURL.replace("/oozie", "");

    return connectionURL;
  }
});

});

require.register("controllers/main/service/reassign/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep2Controller = Em.Controller.extend(App.AssignMasterComponents, {

  name: "reassignMasterWizardStep2Controller",

  useServerValidation: false,

  mastersToShow: function () {
    return [this.get('content.reassign.component_name')];
  }.property('content.reassign.component_name'),

  mastersToMove: function () {
    return [this.get('content.reassign.component_name')];
  }.property('content.reassign.component_name'),

  /**
   * Show 'Current: <host>' for masters with single instance
   */
  additionalHostsList: function () {
    if (this.get('servicesMastersToShow.length') === 1) {
      return [{
        label: Em.I18n.t('services.reassign.step2.currentHost'),
        host: App.HostComponent.find().findProperty('componentName', this.get('content.reassign.component_name')).get('hostName')
      }];
    }
    return [];
  }.property('content.reassign.component_name', 'servicesMastersToShow.length'),

  /**
   * Assignment is valid only if for one master component host was changed
   * @returns {boolean}
   */
  customClientSideValidation: function customClientSideValidation() {
    var reassigned = 0;
    var existedComponents = App.HostComponent.find().filterProperty('componentName', this.get('content.reassign.component_name')).mapProperty('hostName');
    var newComponents = this.get('servicesMasters').filterProperty('component_name', this.get('content.reassign.component_name')).mapProperty('selectedHost');
    existedComponents.forEach(function (host) {
      if (!newComponents.contains(host)) {
        reassigned++;
      }
    }, this);
    return reassigned === 1;
  }

});

});

require.register("controllers/main/service/reassign/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep3Controller = Em.Controller.extend({
  name: 'reassignMasterWizardStep3Controller',

  componentSpecificTypesMap: {
    'NAMENODE': [{
      serviceName: 'HBASE',
      configTypes: ['hbase-site']
    }, {
      serviceName: 'ACCUMULO',
      configTypes: ['accumulo-site']
    }, {
      serviceName: 'HAWQ',
      configTypes: ['hawq-site', 'hdfs-client']
    }],
    'RESOURCEMANAGER': [{
      serviceName: 'HAWQ',
      configTypes: ['hawq-site', 'yarn-client']
    }]
  },

  /**
   * additional configs with template values
   * Part of value to substitute has following format: "<replace-value>"
   */
  additionalConfigsMap: [{
    componentName: 'RESOURCEMANAGER',
    configs: {
      'yarn-site': {
        'yarn.resourcemanager.address': '<replace-value>:8050',
        'yarn.resourcemanager.admin.address': '<replace-value>:8141',
        'yarn.resourcemanager.resource-tracker.address': '<replace-value>:8025',
        'yarn.resourcemanager.scheduler.address': '<replace-value>:8030',
        'yarn.resourcemanager.webapp.address': '<replace-value>:8088',
        'yarn.resourcemanager.webapp.https.address': '<replace-value>:8090',
        'yarn.resourcemanager.hostname': '<replace-value>'
      }
    }
  }, {
    componentName: 'JOBTRACKER',
    configs: {
      'mapred-site': {
        'mapred.job.tracker.http.address': '<replace-value>:50030',
        'mapred.job.tracker': '<replace-value>:50300'
      }
    }
  }, {
    componentName: 'SECONDARY_NAMENODE',
    configs: {
      'hdfs-site': {
        'dfs.secondary.http.address': '<replace-value>:50090'
      }
    },
    configs_Hadoop2: {
      'hdfs-site': {
        'dfs.namenode.secondary.http-address': '<replace-value>:50090'
      }
    }
  }, {
    componentName: 'NAMENODE',
    configs: {
      'hdfs-site': {
        'dfs.http.address': '<replace-value>:50070',
        'dfs.https.address': '<replace-value>:50470'
      },
      'core-site': {
        'fs.default.name': 'hdfs://<replace-value>:8020'
      }
    },
    configs_Hadoop2: {
      'hdfs-site': {
        'dfs.namenode.rpc-address': '<replace-value>:8020',
        'dfs.namenode.http-address': '<replace-value>:50070',
        'dfs.namenode.https-address': '<replace-value>:50470'
      },
      'core-site': {
        'fs.defaultFS': 'hdfs://<replace-value>:8020'
      }
    }
  }, {
    componentName: 'APP_TIMELINE_SERVER',
    configs: {
      'yarn-site': {
        'yarn.timeline-service.webapp.address': '<replace-value>:8188',
        'yarn.timeline-service.webapp.https.address': '<replace-value>:8190',
        'yarn.timeline-service.address': '<replace-value>:10200'
      }
    }
  }, {
    componentName: 'TIMELINE_READER',
    configs: {
      'yarn-site': {
        'yarn.timeline-service.reader.webapp.address': '<replace-value>:8198',
        'yarn.timeline-service.reader.webapp.https.address': '<replace-value>:8199'
      }
    }
  }, {
    componentName: 'OOZIE_SERVER',
    configs: {
      'oozie-site': {
        'oozie.base.url': 'http://<replace-value>:11000/oozie'
      },
      'core-site': {
        'hadoop.proxyuser.oozie.hosts': '<replace-value>'
      }
    }
  }, {
    componentName: 'HIVE_METASTORE',
    configs: {
      'hive-site': {}
    }
  }, {
    componentName: 'MYSQL_SERVER',
    configs: {
      'hive-site': {
        'javax.jdo.option.ConnectionURL': 'jdbc:mysql://<replace-value>/hive?createDatabaseIfNotExist=true'
      }
    }
  }, {
    componentName: 'HISTORYSERVER',
    configs: {
      'mapred-site': {
        'mapreduce.jobhistory.webapp.address': '<replace-value>:19888',
        'mapreduce.jobhistory.address': '<replace-value>:10020'
      }
    }
  }],

  secureConfigsMap: [{
    componentName: 'NAMENODE',
    configs: [{
      site: 'hdfs-site',
      keytab: 'dfs.namenode.keytab.file',
      principal: 'dfs.namenode.kerberos.principal'
    }, {
      site: 'hdfs-site',
      keytab: 'dfs.web.authentication.kerberos.keytab',
      principal: 'dfs.web.authentication.kerberos.principal'
    }]
  }, {
    componentName: 'SECONDARY_NAMENODE',
    configs: [{
      site: 'hdfs-site',
      keytab: 'dfs.secondary.namenode.keytab.file',
      principal: 'dfs.secondary.namenode.kerberos.principal'
    }, {
      site: 'hdfs-site',
      keytab: 'dfs.web.authentication.kerberos.keytab',
      principal: 'dfs.web.authentication.kerberos.principal'
    }]
  }, {
    componentName: 'RESOURCEMANAGER',
    configs: [{
      site: 'yarn-site',
      keytab: 'yarn.resourcemanager.keytab',
      principal: 'yarn.resourcemanager.principal'
    }, {
      site: 'yarn-site',
      keytab: 'yarn.resourcemanager.webapp.spnego-keytab-file',
      principal: 'yarn.resourcemanager.webapp.spnego-principal'
    }]
  }, {
    componentName: 'OOZIE_SERVER',
    configs: [{
      site: 'oozie-site',
      keytab: 'oozie.authentication.kerberos.keytab',
      principal: 'oozie.authentication.kerberos.principal'
    }, {
      site: 'oozie-site',
      keytab: 'oozie.service.HadoopAccessorService.keytab.file',
      principal: 'oozie.service.HadoopAccessorService.kerberos.principal'
    }]
  }, {
    componentName: 'WEBHCAT_SERVER',
    configs: [{
      site: 'webhcat-site',
      keytab: 'templeton.kerberos.keytab',
      principal: 'templeton.kerberos.principal'
    }]
  }, {
    componentName: 'HIVE_SERVER',
    configs: [{
      site: 'hive-site',
      keytab: 'hive.server2.authentication.kerberos.keytab',
      principal: 'hive.server2.authentication.kerberos.principal'
    }, {
      site: 'hive-site',
      keytab: 'hive.server2.authentication.spnego.keytab',
      principal: 'hive.server2.authentication.spnego.principal'
    }]
  }, {
    componentName: 'HIVE_METASTORE',
    configs: [{
      site: 'hive-site',
      keytab: 'hive.metastore.kerberos.keytab.file',
      principal: 'hive.metastore.kerberos.principal'
    }]
  }],

  isLoaded: false,

  versionLoaded: true,

  hideDependenciesInfoBar: true,

  configs: null,

  configsAttributes: null,

  secureConfigs: [],

  stepConfigs: [],

  propertiesToChange: {},

  isSubmitDisabled: Em.computed.or('!isLoaded', 'submitButtonClicked'),

  /**
   * Is Submit-click processing now
   * @type {bool}
   */
  submitButtonClicked: false,

  loadStep: function loadStep() {
    this.set('submitButtonClicked', false);
    if (this.get('wizardController.isComponentWithReconfiguration')) {
      this.set('isLoaded', false);
      App.ajax.send({
        name: 'config.tags',
        sender: this,
        success: 'onLoadConfigsTags'
      });
    } else {
      this.set('isLoaded', true);
    }
  },

  clearStep: function clearStep() {
    this.setProperties({
      configs: null,
      configsAttributes: null,
      secureConfigs: [],
      propertiesToChange: {}
    });
  },

  onLoadConfigsTags: function onLoadConfigsTags(data) {
    var urlParams = this.getConfigUrlParams(this.get('content.reassign.component_name'), data);

    App.ajax.send({
      name: 'reassign.load_configs',
      sender: this,
      data: {
        urlParams: urlParams.join('|')
      },
      success: 'onLoadConfigs'
    });
  },

  getConfigUrlParams: function getConfigUrlParams(componentName, data) {
    var urlParams = [];

    this.get('wizardController.serviceToConfigSiteMap')[componentName].forEach(function (site) {
      if (data.Clusters.desired_configs[site]) {
        urlParams.push('(type=' + site + '&tag=' + data.Clusters.desired_configs[site].tag + ')');
      }
    });

    // specific cases for certain components
    var specificTypes = this.get('componentSpecificTypesMap')[componentName];
    if (specificTypes) {
      var services = App.Service.find();
      specificTypes.forEach(function (service) {
        if (services.someProperty('serviceName', service.serviceName)) {
          service.configTypes.forEach(function (site) {
            urlParams.push('(type=' + site + '&tag=' + data.Clusters.desired_configs[site].tag + ')');
          });
        }
      });
    }

    return urlParams;
  },

  renderServiceConfigs: function renderServiceConfigs(configs) {
    var self = this,
        configCategories = [],
        displayedConfigs = [],
        serviceConfig = App.ServiceConfig.create({
      serviceName: 'MISC',
      configCategories: configCategories,
      showConfig: true,
      configs: displayedConfigs
    });
    App.get('router.mainController.isLoading').call(App.get('router.clusterController'), 'isConfigsPropertiesLoaded').done(function () {
      Em.keys(self.get('propertiesToChange')).forEach(function (type) {
        var service = App.config.get('serviceByConfigTypeMap')[type];
        if (service) {
          var serviceName = service.get('serviceName');
          if (!configCategories.someProperty('name', serviceName)) {
            configCategories.push(App.ServiceConfigCategory.create({
              name: serviceName,
              displayName: service.get('displayName')
            }));
          }
          this.get('propertiesToChange')[type].forEach(function (property) {
            var propertyName = property.name,
                stackProperty = App.configsCollection.getConfigByName(propertyName, type) || {},
                displayName = self.getDisplayName(stackProperty.displayName, propertyName, type, serviceName),
                displayedProperty = App.ServiceConfigProperty.create({
              name: propertyName,
              fileName: type
            }, stackProperty, {
              value: configs[type][propertyName],
              category: serviceName,
              displayName: displayName,
              isEditable: Boolean(stackProperty.isEditable !== false && !property.isSecure)
            });
            displayedConfigs.push(displayedProperty);
          });
        }
      }, self);
      self.setProperties({
        stepConfigs: [serviceConfig],
        selectedService: serviceConfig,
        isLoaded: true
      });
    });
  },

  getDisplayName: function getDisplayName(stackDisplayName, propertyName, type, serviceName) {
    var displayName = stackDisplayName || propertyName;
    var keys = Em.keys(this.get('propertiesToChange'));
    for (var i = 0; i < keys.length; i++) {
      var fileName = keys[i],
          service = App.config.get('serviceByConfigTypeMap')[fileName];
      if (fileName !== type && service && service.get('serviceName') === serviceName) {
        var configs = this.get('propertiesToChange')[fileName];
        if (configs.someProperty('name', propertyName)) {
          displayName = type + '/' + propertyName;
          break;
        }
      }
    }
    return displayName;
  },

  onLoadConfigs: function onLoadConfigs(data) {
    // Find hawq-site.xml location
    var hawqSiteIndex = -1;
    for (var i = 0; i < data.items.length; i++) {
      if (data.items[i].type == 'hawq-site') {
        hawqSiteIndex = i;
        break;
      }
    }

    // if certain services are deployed, include related site files to additionalConfigsMap and relatedServicesMap.
    if (hawqSiteIndex >= 0) {
      // if HAWQ is deployed
      var hawqSiteProperties = {
        'hawq_rm_yarn_address': '<replace-value>:8050',
        'hawq_rm_yarn_scheduler_address': '<replace-value>:8030'
      };

      var rmComponent = this.get('additionalConfigsMap').findProperty('componentName', "RESOURCEMANAGER");
      rmComponent.configs["hawq-site"] = hawqSiteProperties;

      if (data.items[hawqSiteIndex].properties["hawq_global_rm_type"].toLowerCase() === "yarn") {
        this.get('wizardController.relatedServicesMap')['RESOURCEMANAGER'].append('HAWQ');
      }
    }

    var componentName = this.get('content.reassign.component_name');
    var targetHostName = this.get('content.reassignHosts.target');
    var configs = {};
    var attributes = {};
    var secureConfigs = [];

    data.items.forEach(function (item) {
      configs[item.type] = item.properties;
      if (item.properties_attributes) {
        attributes[item.type] = item.properties_attributes;
      }
    });

    this.set('configsAttributes', attributes);

    this.setAdditionalConfigs(configs, componentName, targetHostName);
    this.setSecureConfigs(secureConfigs, configs, componentName);

    this.set('secureConfigs', secureConfigs);

    switch (componentName) {
      case 'NAMENODE':
        App.MoveNameNodeConfigInitializer.setup(this._getNnInitializerSettings(configs));
        configs = this.setDynamicConfigs(configs, App.MoveNameNodeConfigInitializer);
        App.MoveNameNodeConfigInitializer.cleanup();
        break;
      case 'RESOURCEMANAGER':
        App.MoveRmConfigInitializer.setup(this._getRmInitializerSettings(configs));
        var additionalDependencies = this._getRmAdditionalDependencies(configs);
        configs = this.setDynamicConfigs(configs, App.MoveRmConfigInitializer, additionalDependencies);
        App.MoveRmConfigInitializer.cleanup();
        break;
      case 'HIVE_METASTORE':
        App.MoveHmConfigInitializer.setup(this._getHiveInitializerSettings(configs));
        configs = this.setDynamicConfigs(configs, App.MoveHmConfigInitializer);
        App.MoveHmConfigInitializer.cleanup();
        break;
      case 'HIVE_SERVER':
        App.MoveHsConfigInitializer.setup(this._getHiveInitializerSettings(configs));
        configs = this.setDynamicConfigs(configs, App.MoveHsConfigInitializer);
        App.MoveHsConfigInitializer.cleanup();
        break;
      case 'WEBHCAT_SERVER':
        App.MoveWsConfigInitializer.setup(this._getWsInitializerSettings(configs));
        configs = this.setDynamicConfigs(configs, App.MoveWsConfigInitializer);
        App.MoveWsConfigInitializer.cleanup();
        break;
      case 'OOZIE_SERVER':
        App.MoveOSConfigInitializer.setup(this._getOsInitializerSettings(configs));
        configs = this.setDynamicConfigs(configs, App.MoveOSConfigInitializer);
        App.MoveOSConfigInitializer.cleanup();
    }

    this.renderServiceConfigs(configs);
    this.set('configs', configs);
  },

  /**
   * set additional configs
   * configs_Hadoop2 - configs which belongs to Hadoop 2 stack only
   * @param configs
   * @param componentName
   * @param replaceValue
   * @return {Boolean}
   */
  setAdditionalConfigs: function setAdditionalConfigs(configs, componentName, replaceValue) {
    var component = this.get('additionalConfigsMap').findProperty('componentName', componentName);

    if (Em.isNone(component)) return false;
    var additionalConfigs = component.configs_Hadoop2 ? component.configs_Hadoop2 : component.configs;

    for (var site in additionalConfigs) {
      if (additionalConfigs.hasOwnProperty(site)) {
        for (var property in additionalConfigs[site]) {
          if (additionalConfigs[site].hasOwnProperty(property)) {
            if (App.get('isHaEnabled') && componentName === 'NAMENODE' && ['fs.defaultFS', 'dfs.namenode.rpc-address', 'dfs.namenode.http-address', 'dfs.namenode.https-address'].contains(property)) continue;

            configs[site][property] = additionalConfigs[site][property].replace('<replace-value>', replaceValue);
            if (!this.get('propertiesToChange').hasOwnProperty(site)) {
              this.get('propertiesToChange')[site] = [];
            }
            this.get('propertiesToChange')[site].push({
              name: property
            });
          }
        }
      }
    }
    return true;
  },

  /**
   * set secure configs for component
   * @param secureConfigs
   * @param configs
   * @param componentName
   * @return {Boolean}
   */
  setSecureConfigs: function setSecureConfigs(secureConfigs, configs, componentName) {
    var securityEnabled = App.get('isKerberosEnabled');
    var component = this.get('secureConfigsMap').findProperty('componentName', componentName);
    if (Em.isNone(component) || !securityEnabled) return false;

    component.configs.forEach(function (config) {
      secureConfigs.push({
        keytab: configs[config.site][config.keytab],
        principal: configs[config.site][config.principal]
      });
      if (!this.get('propertiesToChange').hasOwnProperty(config.site)) {
        this.get('propertiesToChange')[config.site] = [];
      }
      this.get('propertiesToChange')[config.site].push({
        name: config.keytab,
        isSecure: true
      }, {
        name: config.principal,
        isSecure: true
      });
    }, this);
    return true;
  },

  /**
   * Get additional dependencies-data for App.MoveNameNodeConfigInitializer
   *
   * @param {object} configs
   * @returns {object}
   * @private
   * @method _getNnInitializerSettings
   */
  _getNnInitializerSettings: function _getNnInitializerSettings(configs) {
    var _this = this;

    var ret = {};
    if (App.get('isHaEnabled')) {
      (function () {
        var configsObject = configs['hdfs-site'],
            nameSpaces = configsObject['dfs.nameservices'].split(','),
            nameSpacesCount = nameSpaces.length,
            propertyNames = Object.keys(configsObject);

        var _loop = function _loop(i) {
          var nameSpace = nameSpaces[i],
              propertyNameStart = 'dfs.namenode.http-address.' + nameSpace + '.',
              httpAddressPropertiesNames = propertyNames.filter(function (propertyName) {
            return propertyName.startsWith(propertyNameStart);
          }),
              matchingPropertyName = httpAddressPropertiesNames.find(function (propertyName) {
            return configsObject[propertyName].startsWith(_this.get('content.reassignHosts.source'));
          });
          if (matchingPropertyName) {
            var nameNodeSuffixMatch = matchingPropertyName.match(new RegExp(propertyNameStart + '(\\w+)'));
            ret.namespaceId = nameSpace;
            ret.suffix = nameNodeSuffixMatch && nameNodeSuffixMatch[1];
            return 'break';
          }
        };

        for (var i = 0; i < nameSpacesCount; i++) {
          var _ret2 = _loop(i);

          if (_ret2 === 'break') break;
        }
      })();
    }
    return ret;
  },

  /**
   * Settings used to the App.MoveRmConfigInitializer setup
   *
   * @param {object} configs
   * @returns {{suffix: string}}
   * @private
   * @method _getRmInitializerSettings
   */
  _getRmInitializerSettings: function _getRmInitializerSettings(configs) {
    return {
      suffix: configs['yarn-site']['yarn.resourcemanager.hostname.rm1'] === this.get('content.reassignHosts.source') ? 'rm1' : 'rm2'
    };
  },

  /**
   * Get additional dependencies-data for App.MoveRmConfigInitializer
   *
   * @param {object} configs
   * @returns {object}
   * @private
   * @method _getRmAdditionalDependencies
   */
  _getRmAdditionalDependencies: function _getRmAdditionalDependencies(configs) {
    var ret = {};
    var rm1 = configs['yarn-site']['yarn.resourcemanager.hostname.rm1'];
    if (rm1) {
      ret.rm1 = rm1;
    }
    var rm2 = configs['yarn-site']['yarn.resourcemanager.hostname.rm2'];
    if (rm2) {
      ret.rm2 = rm2;
    }
    return ret;
  },

  /**
   * Settings used to the App.MoveHsConfigInitializer and App.MoveHmConfigInitializer setup
   *
   * @param {object} configs
   * @returns {{hiveUser: string}}
   * @private
   * @method _getHiveInitializerSettings
   */
  _getHiveInitializerSettings: function _getHiveInitializerSettings(configs) {
    return {
      hiveUser: configs['hive-env']['hive_user']
    };
  },

  /**
   * Settings used to the App.MoveWsConfigInitializer setup
   *
   * @param {object} configs
   * @returns {{webhcatUser: string}}
   * @private
   * @method _getWsInitializerSettings
   */
  _getWsInitializerSettings: function _getWsInitializerSettings(configs) {
    return {
      webhcatUser: configs['hive-env']['webhcat_user']
    };
  },

  /**
   * Settings used to the App.MoveOSConfigInitializer setup
   *
   * @param {object} configs
   * @returns {object}
   * @private
   * @method _getOsInitializerSettings
   */
  _getOsInitializerSettings: function _getOsInitializerSettings(configs) {
    var ret = {};
    var cfg = configs['oozie-env']['oozie_user'];
    if (cfg) {
      ret.oozieUser = cfg;
    }
    return ret;
  },

  /**
   * Set config values according to the new cluster topology
   *
   * @param {object} configs
   * @param {MoveComponentConfigInitializerClass} initializer
   * @param {object} [additionalDependencies={}]
   * @returns {object}
   * @method setDynamicConfigs
   */
  setDynamicConfigs: function setDynamicConfigs(configs, initializer, additionalDependencies) {
    additionalDependencies = additionalDependencies || {};
    var topologyDB = this._prepareTopologyDB(),
        dependencies = this._prepareDependencies(additionalDependencies),
        initializerObjects = initializer.get('initializers'),
        uniqueInitializerObjects = initializer.get('uniqueInitializers');
    Em.keys(configs).forEach(function (site) {
      Em.keys(configs[site]).forEach(function (config) {
        // temporary object for initializer
        var cfg = {
          name: config,
          filename: site,
          value: configs[site][config]
        };
        configs[site][config] = initializer.initialValue(cfg, topologyDB, dependencies).value;
        if (initializerObjects[config] || uniqueInitializerObjects[config]) {
          if (!this.get('propertiesToChange').hasOwnProperty(site)) {
            this.get('propertiesToChange')[site] = [];
          }
          this.get('propertiesToChange')[site].push({
            name: config
          });
        }
      }, this);
    }, this);
    return configs;
  },

  /**
   *
   * @returns {extendedTopologyLocalDB}
   * @private
   * @method _prepareTopologyDB
   */
  _prepareTopologyDB: function _prepareTopologyDB() {
    var ret = this.get('content').getProperties(['masterComponentHosts', 'slaveComponentHosts', 'hosts']);
    ret.installedServices = App.Service.find().mapProperty('serviceName');
    return ret;
  },

  /**
   * Create dependencies for Config Initializers
   *
   * @param {object} additionalDependencies  some additional information that should be added
   * @returns {reassignComponentDependencies}
   * @private
   * @method _prepareDependencies
   */
  _prepareDependencies: function _prepareDependencies(additionalDependencies) {
    additionalDependencies = additionalDependencies || {};
    var ret = {};
    ret.sourceHostName = this.get('content.reassignHosts.source');
    ret.targetHostName = this.get('content.reassignHosts.target');
    return Em.merge(ret, additionalDependencies);
  },

  updateServiceConfigs: function updateServiceConfigs() {
    var configs = this.get('configs');
    if (configs) {
      this.get('selectedService.configs').forEach(function (property) {
        var type = App.config.getConfigTagFromFileName(property.fileName);
        configs[type][property.name] = property.value;
      }, this);
    }
  },

  submit: function submit() {
    if (!this.get('submitButtonClicked')) {
      this.set('submitButtonClicked', true);
      App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
        App.router.send("next");
      });
    }
  }
});

});

require.register("controllers/main/service/reassign/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Additional data that is used in the `Move Component Initializers`
 *
 * @typedef {object} reassignComponentDependencies
 * @property {string} sourceHostName host where component was before moving
 * @property {string} targetHostName host where component will be after moving
 */

App.ReassignMasterWizardStep4Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "reassignMasterWizardStep4Controller",

  commands: ['stopRequiredServices', 'cleanMySqlServer', 'createHostComponents', 'putHostComponentsInMaintenanceMode', 'reconfigure', 'installHostComponents', 'startZooKeeperServers', 'startNameNode', 'stopHostComponentsInMaintenanceMode', 'deleteHostComponents', 'configureMySqlServer', 'startMySqlServer', 'startNewMySqlServer', 'startRequiredServices'],

  // custom commands for Components with DB Configuration and Check
  commandsForDB: ['createHostComponents', 'installHostComponents', 'configureMySqlServer', 'restartMySqlServer', 'testDBConnection', 'stopRequiredServices', 'cleanMySqlServer', 'putHostComponentsInMaintenanceMode', 'reconfigure', 'stopHostComponentsInMaintenanceMode', 'deleteHostComponents', 'configureMySqlServer', 'startRequiredServices'],

  clusterDeployState: 'REASSIGN_MASTER_INSTALLING',

  multiTaskCounter: 0,

  hostComponents: [],

  dependentHostComponents: [],

  /**
   * load step info
   */
  loadStep: function loadStep() {
    var componentName = this.get('content.reassign.component_name');
    if (componentName === 'NAMENODE' && App.get('isHaEnabled')) {
      this.set('hostComponents', ['NAMENODE', 'ZKFC']);
    } else {
      this.set('hostComponents', [componentName]);
    }
    this.setDependentHostComponents(componentName);
    this.set('serviceName', [this.get('content.reassign.service_id')]);
    this._super();
  },

  /**
   * Set dependent host-components to <code>dependentHostComponents</code>
   * @param {string} componentName
   */
  setDependentHostComponents: function setDependentHostComponents(componentName) {
    var installedServices = App.Service.find().mapProperty('serviceName');
    var hostInstalledComponents = App.Host.find(this.get('content.reassignHosts.target')).get('hostComponents').mapProperty('componentName');
    var clusterInstalledComponents = App.MasterComponent.find().toArray().concat(App.ClientComponent.find().toArray()).concat(App.SlaveComponent.find().toArray()).filter(function (service) {
      return service.get("totalCount") > 0;
    }).mapProperty('componentName');
    var dependenciesToInstall = App.StackServiceComponent.find(componentName).get('dependencies').filter(function (component) {
      return !(component.scope == 'host' ? hostInstalledComponents : clusterInstalledComponents).contains(component.componentName) && installedServices.contains(component.serviceName) && (componentName === 'NAMENODE' ? App.get('isHaEnabled') : true);
    }).mapProperty('componentName');
    this.set('dependentHostComponents', dependenciesToInstall);
  },

  /**
   * concat host-component names into string
   * @return {String}
   */
  getHostComponentsNames: function getHostComponentsNames() {
    var hostComponentsNames = '';
    this.get('hostComponents').forEach(function (comp, index) {
      hostComponentsNames += index ? '+' : '';
      hostComponentsNames += comp === 'ZKFC' ? comp : App.format.role(comp, false);
    }, this);
    return hostComponentsNames;
  },

  /**
   * remove unneeded tasks
   */
  removeUnneededTasks: function removeUnneededTasks() {
    var componentName = this.get('content.reassign.component_name');
    if (this.isComponentWithDB()) {
      var db_type = this.get('content.databaseType');
      var is_remote_db = this.get('content.serviceProperties.is_remote_db');

      if (is_remote_db || db_type !== 'mysql') {
        this.removeTasks(['configureMySqlServer', 'startMySqlServer', 'restartMySqlServer', 'cleanMySqlServer', 'configureMySqlServer']);
      }

      if (db_type === 'derby') {
        this.removeTasks(['testDBConnection']);
      }
    }

    if (componentName !== 'MYSQL_SERVER' && !this.isComponentWithDB()) {
      this.removeTasks(['configureMySqlServer', 'startMySqlServer', 'restartMySqlServer', 'cleanMySqlServer', 'startNewMySqlServer', 'configureMySqlServer']);
    }

    if (componentName === 'MYSQL_SERVER') {
      this.removeTasks(['cleanMySqlServer']);
    }

    if (this.get('content.hasManualSteps')) {
      if (componentName === 'NAMENODE' && App.get('isHaEnabled')) {
        // Only for reassign NameNode with HA enabled
        this.removeTasks(['stopHostComponentsInMaintenanceMode', 'deleteHostComponents', 'startRequiredServices']);
      } else {
        this.removeTasks(['startZooKeeperServers', 'startNameNode', 'stopHostComponentsInMaintenanceMode', 'deleteHostComponents', 'startRequiredServices']);
      }
    } else {
      this.removeTasks(['startZooKeeperServers', 'startNameNode']);
    }

    if (!this.get('wizardController.isComponentWithReconfiguration')) {
      this.removeTasks(['reconfigure']);
    }

    if (!this.get('content.reassignComponentsInMM.length')) {
      this.removeTasks(['stopHostComponentsInMaintenanceMode']);
    }
  },

  /**
   * initialize tasks
   */
  initializeTasks: function initializeTasks() {
    var commands = this.get('commands');
    var currentStep = App.router.get('reassignMasterController.currentStep');
    var hostComponentsNames = this.getHostComponentsNames();

    if (this.isComponentWithDB()) {
      commands = this.get('commandsForDB');
    }

    for (var i = 0; i < commands.length; i++) {
      var TaskLabel = i === 3 ? this.get('serviceName') : hostComponentsNames; //For Reconfigure task, show serviceName
      var title = Em.I18n.t('services.reassign.step4.tasks.' + commands[i] + '.title').format(TaskLabel);
      this.get('tasks').pushObject(Ember.Object.create({
        title: title,
        status: 'PENDING',
        id: i,
        command: commands[i],
        showRetry: false,
        showRollback: false,
        name: title,
        displayName: title,
        progress: 0,
        isRunning: false,
        hosts: []
      }));
    }
    this.removeUnneededTasks();
    this.set('isLoaded', true);
  },

  hideRollbackButton: function () {
    var failedTask = this.get('tasks').findProperty('showRollback');
    if (failedTask) {
      failedTask.set('showRollback', false);
    }
  }.observes('tasks.@each.showRollback'),

  onComponentsTasksSuccess: function onComponentsTasksSuccess() {
    this.decrementProperty('multiTaskCounter');
    if (this.get('multiTaskCounter') <= 0) {
      this.onTaskCompleted();
    }
  },

  /**
   * make server call to stop services
   */
  stopRequiredServices: function stopRequiredServices() {
    var componentName = this.get('content.reassign.component_name');
    var servicesToStop = this.get('wizardController.relatedServicesMap')[componentName];
    if (this.get('content.componentsToStopAllServices').contains(componentName)) {
      this.stopServices(servicesToStop, true, true);
    } else {
      this.stopServices(servicesToStop, true);
    }
  },

  createHostComponents: function createHostComponents() {
    var hostComponents = this.get('hostComponents').concat(this.get('dependentHostComponents'));
    var hostName = this.get('content.reassignHosts.target');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      this.createComponent(hostComponents[i], hostName, this.get('content.reassign.service_id'));
    }
  },

  onCreateComponent: function onCreateComponent() {
    this.onComponentsTasksSuccess();
  },

  putHostComponentsInMaintenanceMode: function putHostComponentsInMaintenanceMode() {
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.source');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.host.host_component.passive',
        sender: this,
        data: {
          hostName: hostName,
          passive_state: "ON",
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onTaskError'
      });
    }
  },

  installHostComponents: function installHostComponents() {
    var hostComponents = this.get('hostComponents').concat(this.get('dependentHostComponents'));
    var hostName = this.get('content.reassignHosts.target');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      this.updateComponent(hostComponents[i], hostName, this.get('content.reassign.service_id'), "Install", hostComponents.length);
    }
  },

  reconfigure: function reconfigure() {
    var configs = this.get('content.configs'),
        attributes = this.get('content.configsAttributes'),
        secureConfigs = this.get('content.secureConfigs'),
        componentName = this.get('content.reassign.component_name');
    this.saveClusterStatus(secureConfigs, this.getComponentDir(configs, componentName));
    this.saveConfigsToServer(configs, attributes);
  },

  /**
   * make PUT call to save configs to server
   * @param configs
   * @param attributes
   */
  saveConfigsToServer: function saveConfigsToServer(configs, attributes) {
    App.ajax.send({
      name: 'common.across.services.configurations',
      sender: this,
      data: {
        data: '[' + this.getServiceConfigData(configs, attributes).toString() + ']'
      },
      success: 'onSaveConfigs',
      error: 'onTaskError'
    });
  },
  /**
   * gather and format config data before sending to server
   * @param configs
   * @param attributes
   * @return {Array}
   * @method getServiceConfigData
   */
  getServiceConfigData: function getServiceConfigData(configs, attributes) {
    var componentName = this.get('content.reassign.component_name');
    var configData = Object.keys(configs).map(function (_siteName) {
      return {
        type: _siteName,
        properties: configs[_siteName],
        properties_attributes: attributes[_siteName] || {},
        service_config_version_note: Em.I18n.t('services.reassign.step4.save.configuration.note').format(App.format.role(componentName, false))
      };
    });
    var allConfigData = [];

    App.Service.find().forEach(function (service) {
      var stackService = App.StackService.find().findProperty('serviceName', service.get('serviceName'));
      if (stackService) {
        var serviceConfigData = [];
        Object.keys(stackService.get('configTypesRendered')).forEach(function (type) {
          var serviceConfigTag = configData.findProperty('type', type);
          if (serviceConfigTag) {
            serviceConfigData.pushObject(serviceConfigTag);
          }
        }, this);
        allConfigData.pushObject(JSON.stringify({
          Clusters: {
            desired_config: serviceConfigData
          }
        }));
      }
    }, this);
    return allConfigData;
  },

  /**
   * Get the web address port when RM HA is enabled.
   * @param configs
   * @param webAddressKey (http vs https)
   * */
  getWebAddressPort: function getWebAddressPort(configs, webAddressKey) {
    var result = null;
    var rmWebAddressValue = configs['yarn-site'][webAddressKey];
    if (rmWebAddressValue) {
      var tokens = rmWebAddressValue.split(":");
      if (tokens.length > 1) {
        result = tokens[1];
        result = result.replace(/^\s+|\s+$/g, '');
      }
    }

    if (result) //only return non-empty result
      return result;else return null;
  },

  /**
   * derive component directory from configurations
   * @param configs
   * @param componentName
   * @return {String}
   */
  getComponentDir: function getComponentDir(configs, componentName) {
    if (componentName === 'NAMENODE') {
      return configs['hdfs-site']['dfs.namenode.name.dir'];
    } else if (componentName === 'SECONDARY_NAMENODE') {
      return configs['hdfs-site']['dfs.namenode.checkpoint.dir'];
    }
    return '';
  },

  /**
   * save cluster status to server
   *
   * @param secureConfigs
   * @param componentDir
   * @return {Boolean}
   */
  saveClusterStatus: function saveClusterStatus(secureConfigs, componentDir) {
    if (componentDir || secureConfigs.length) {
      App.router.get(this.get('content.controllerName')).saveComponentDir(componentDir);
      App.router.get(this.get('content.controllerName')).saveSecureConfigs(secureConfigs);
      App.clusterStatus.setClusterStatus({
        clusterName: this.get('content.cluster.name'),
        clusterState: this.get('clusterDeployState'),
        wizardControllerName: this.get('content.controllerName'),
        localdb: App.db.data
      });
      return true;
    }
    return false;
  },

  onSaveConfigs: function onSaveConfigs() {
    this.onTaskCompleted();
  },

  startZooKeeperServers: function startZooKeeperServers() {
    var components = this.get('content.masterComponentHosts').filterProperty('component', 'ZOOKEEPER_SERVER');
    this.updateComponent('ZOOKEEPER_SERVER', components.mapProperty('hostName'), "ZOOKEEPER", "Start");
  },

  /**
   * Start the namenode that is not being moved (applicable only in NameNode HA environment)
   * @private {startNameNode}
   */
  startNameNode: function startNameNode() {
    var components = this.get('content.masterComponentHosts').filterProperty('component', 'NAMENODE');
    var sourceHost = this.get('content.reassignHosts.source');
    var targetHost = this.get('content.reassignHosts.target');
    var hostname = components.mapProperty('hostName').without(sourceHost).without(targetHost);
    this.updateComponent('NAMENODE', hostname, "HDFS", "Start");
  },

  /**
   * make server call to start services
   */
  startRequiredServices: function startRequiredServices() {
    var relatedServices = this.get('wizardController.relatedServicesMap')[this.get('content.reassign.component_name')];
    if (relatedServices) {
      this.startServices(false, relatedServices, true);
    } else {
      this.startServices(true);
    }
  },

  /**
   * make DELETE call for each host component on host
   */
  deleteHostComponents: function deleteHostComponents() {
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.source');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.delete.host_component',
        sender: this,
        data: {
          hostName: hostName,
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onDeleteHostComponentsError'
      });
    }
  },

  onDeleteHostComponentsError: function onDeleteHostComponentsError(error) {
    if (error.responseText.indexOf('org.apache.ambari.server.controller.spi.NoSuchResourceException') !== -1) {
      this.onComponentsTasksSuccess();
    } else {
      this.onTaskError();
    }
  },

  done: function done() {
    if (!this.get('isSubmitDisabled')) {
      this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
      if (this.get('content.hasManualSteps')) {
        App.router.send('next');
      } else {
        App.router.send('complete');
      }
    }
  },

  /**
   * make server call to clean MYSQL
   */
  cleanMySqlServer: function cleanMySqlServer() {
    var hostname = App.HostComponent.find().findProperty('componentName', 'MYSQL_SERVER').get('hostName');

    if (this.get('content.reassign.component_name') === 'MYSQL_SERVER') {
      hostname = this.get('content.reassignHosts.target');
    }

    App.ajax.send({
      name: 'service.mysql.clean',
      sender: this,
      data: {
        host: hostname
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  /**
   * make server call to configure MYSQL
   */
  configureMySqlServer: function configureMySqlServer() {
    var hostname = App.HostComponent.find().findProperty('componentName', 'MYSQL_SERVER').get('hostName');

    if (this.get('content.reassign.component_name') === 'MYSQL_SERVER') {
      hostname = this.get('content.reassignHosts.target');
    }

    App.ajax.send({
      name: 'service.mysql.configure',
      sender: this,
      data: {
        host: hostname
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  startMySqlServer: function startMySqlServer() {
    App.ajax.send({
      name: 'common.host.host_component.update',
      sender: this,
      data: {
        context: "Start MySQL Server",
        hostName: App.HostComponent.find().findProperty('componentName', 'MYSQL_SERVER').get('hostName'),
        serviceName: "HIVE",
        componentName: "MYSQL_SERVER",
        HostRoles: {
          state: "STARTED"
        }
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  restartMySqlServer: function restartMySqlServer() {
    var context = "Restart MySql Server";

    var resource_filters = {
      component_name: "MYSQL_SERVER",
      hosts: App.HostComponent.find().filterProperty('componentName', 'MYSQL_SERVER').get('firstObject.hostName'),
      service_name: "HIVE"
    };

    var operation_level = {
      level: "HOST_COMPONENT",
      cluster_name: this.get('content.cluster.name'),
      service_name: "HIVE",
      hostcomponent_name: "MYSQL_SERVER"
    };

    App.ajax.send({
      name: 'restart.hostComponents',
      sender: this,
      data: {
        context: context,
        resource_filters: [resource_filters],
        operation_level: operation_level
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  startNewMySqlServer: function startNewMySqlServer() {
    App.ajax.send({
      name: 'common.host.host_component.update',
      sender: this,
      data: {
        context: "Start MySQL Server",
        hostName: this.get('content.reassignHosts.target'),
        serviceName: "HIVE",
        componentName: "MYSQL_SERVER",
        HostRoles: {
          state: "STARTED"
        }
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  testDBConnection: function testDBConnection() {
    this.prepareDBCheckAction();
  },

  isComponentWithDB: function isComponentWithDB() {
    return ['HIVE_SERVER', 'HIVE_METASTORE', 'OOZIE_SERVER'].contains(this.get('content.reassign.component_name'));
  },

  /** @property {Object} propertiesPattern - check pattern according to type of connection properties **/
  propertiesPattern: function () {
    return {
      user_name: /(username|dblogin)$/ig,
      user_passwd: /(dbpassword|password)$/ig,
      db_connection_url: /jdbc\.url|connectionurl/ig,
      driver_class: /ConnectionDriverName|jdbc\.driver/ig,
      schema_name: /db\.schema\.name/ig
    };
  }.property(),

  /** @property {Object} connectionProperties - service specific config values mapped for custom action request **/
  connectionProperties: function () {
    var propObj = {};
    for (var key in this.get('propertiesPattern')) {
      propObj[key] = this.getConnectionProperty(this.get('propertiesPattern')[key]);
    }
    return propObj;
  }.property('propertiesPattern'),

  getConnectionProperty: function getConnectionProperty(regexp) {
    var configType = this.get('requiredProperties.type'),
        propertyName = this.get('requiredProperties.names').filter(function (item) {
      return regexp.test(item);
    })[0];
    return Em.getWithDefault(this.get('content.configs'), configType, {})[propertyName];
  },

  /**
   * Properties that stores in local storage used for handling
   * last success connection.
   *
   * @property {Object} preparedDBProperties
   **/
  preparedDBProperties: function () {
    var propObj = {};
    for (var key in this.get('propertiesPattern')) {
      var propValue = this.getConnectionProperty(this.get('propertiesPattern')[key]);
      propObj[key] = propValue;
    }
    return propObj;
  }.property(),

  /** @property {object} requiredProperties - properties that necessary for database connection **/
  requiredProperties: function () {
    var propertiesMap = {
      OOZIE: {
        type: 'oozie-site',
        names: ['oozie.db.schema.name', 'oozie.service.JPAService.jdbc.username', 'oozie.service.JPAService.jdbc.password', 'oozie.service.JPAService.jdbc.driver', 'oozie.service.JPAService.jdbc.url']
      },
      HIVE: {
        type: 'hive-site',
        names: ['ambari.hive.db.schema.name', 'javax.jdo.option.ConnectionUserName', 'javax.jdo.option.ConnectionPassword', 'javax.jdo.option.ConnectionDriverName', 'javax.jdo.option.ConnectionURL']
      }
    };

    return propertiesMap[this.get('content.reassign.service_id')];
  }.property(),

  prepareDBCheckAction: function prepareDBCheckAction() {
    var params = this.get('preparedDBProperties');

    var ambariProperties = App.router.get('clusterController.ambariProperties');

    params['db_name'] = this.get('content.databaseType');
    params['jdk_location'] = ambariProperties['jdk_location'];
    params['jdk_name'] = ambariProperties['jdk.name'];
    params['java_home'] = ambariProperties['java.home'];

    params['threshold'] = 60;
    params['ambari_server_host'] = location.hostname;
    params['check_execute_list'] = "db_connection_check";

    App.ajax.send({
      name: 'cluster.custom_action.create',
      sender: this,
      data: {
        requestInfo: {
          "context": "Check host",
          "action": "check_host",
          "parameters": params
        },
        filteredHosts: [this.get('content.reassignHosts.target')]
      },
      success: 'onCreateActionSuccess',
      error: 'onTaskError'
    });
  },

  onCreateActionSuccess: function onCreateActionSuccess(data) {
    this.set('checkDBRequestId', data.Requests.id);
    App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('checkDBRequestId')
      },
      success: 'setCheckDBTaskId'
    });
  },

  setCheckDBTaskId: function setCheckDBTaskId(data) {
    this.set('checkDBTaskId', data.items[0].Tasks.id);
    this.startDBCheckPolling();
  },

  startDBCheckPolling: function startDBCheckPolling() {
    this.getDBConnTaskInfo();
  },

  getDBConnTaskInfo: function getDBConnTaskInfo() {
    this.setTaskStatus(this.get('currentTaskId'), 'IN_PROGRESS');
    this.get('tasks').findProperty('id', this.get('currentTaskId')).set('progress', 100);

    this.set('logs', []);
    App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('checkDBRequestId'),
        taskId: this.get('checkDBTaskId')
      },
      success: 'getDBConnTaskInfoSuccess'
    });
  },

  getDBConnTaskInfoSuccess: function getDBConnTaskInfoSuccess(data) {
    var task = data.Tasks;
    if (task.status === 'COMPLETED') {
      var structuredOut = task.structured_out.db_connection_check;
      if (structuredOut.exit_code != 0) {
        this.showConnectionErrorPopup(structuredOut.message);
        this.onTaskError();
      } else {
        this.onTaskCompleted();
      }
    }

    if (task.status === 'FAILED') {
      this.onTaskError();
    }

    if (/PENDING|QUEUED|IN_PROGRESS/.test(task.status)) {
      Em.run.later(this, function () {
        this.startDBCheckPolling();
      }, 3000);
    }
  },

  showConnectionErrorPopup: function showConnectionErrorPopup(error) {
    var popup = App.showAlertPopup('Database Connection Error');
    popup.set('body', error);
  },

  testDBRetryTooltip: function () {
    var db_host = this.get('content.serviceProperties.database_hostname');
    var db_type = this.get('content.databaseType');
    var db_props = this.get('preparedDBProperties');

    return Em.I18n.t('services.reassign.step4.tasks.testDBConnection.tooltip').format(db_host, db_type, db_props['schema_name'], db_props['user_name'], db_props['user_passwd'], db_props['driver_class'], db_props['db_connection_url']);
  }.property('dbProperties'),

  saveServiceProperties: function saveServiceProperties(configs) {
    App.router.get(this.get('content.controllerName')).saveServiceProperties(configs);
  },

  stopHostComponentsInMaintenanceMode: function stopHostComponentsInMaintenanceMode() {
    var hostComponentsInMM = this.get('content.reassignComponentsInMM');
    var hostName = this.get('content.reassignHosts.source');
    var serviceName = this.get('content.reassign.service_id');
    hostComponentsInMM = hostComponentsInMM.map(function (componentName) {
      return {
        hostName: hostName,
        serviceName: serviceName,
        componentName: componentName
      };
    });
    this.set('multiTaskCounter', hostComponentsInMM.length);
    this.updateComponentsState(hostComponentsInMM, 'INSTALLED');
  }

});

});

require.register("controllers/main/service/reassign/step5_controller", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep5Controller = Em.Controller.extend({
  name: "reassignMasterWizardStep5Controller"
});

});

require.register("controllers/main/service/reassign/step6_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep6Controller = App.HighAvailabilityProgressPageController.extend(App.WizardEnableDone, {

  name: "reassignMasterWizardStep6Controller",

  commands: ['stopMysqlService', 'putHostComponentsInMaintenanceMode', 'stopHostComponentsInMaintenanceMode', 'deleteHostComponents', 'startDatanodes', 'startAllServices'],

  clusterDeployState: 'REASSIGN_MASTER_INSTALLING',

  multiTaskCounter: 0,

  hostComponents: [],

  loadStep: function loadStep() {
    if (this.get('content.reassign.component_name') === 'NAMENODE' && App.get('isHaEnabled')) {
      this.set('hostComponents', ['NAMENODE', 'ZKFC']);
    } else {
      this.set('hostComponents', [this.get('content.reassign.component_name')]);
    }

    if (App.Service.find().someProperty('serviceName', 'PXF') && this.get('content.reassign.component_name') === 'NAMENODE') {
      var pxfHosts = App.HostComponent.find().filterProperty('componentName', 'PXF').mapProperty('hostName');
      var dataNodeHosts = App.HostComponent.find().filterProperty('componentName', 'DATANODE').mapProperty('hostName');

      // If NAMENODE is being moved and source host does not have DATANODE, PXF should be removed from source host
      if (pxfHosts.contains(this.get('content.reassignHosts.source')) && !dataNodeHosts.contains(this.get('content.reassignHosts.source'))) this.get('hostComponents').push('PXF');

      // If NAMENODE is being moved and target host does not have PXF, PXF should be added to target host
      if (!pxfHosts.contains(this.get('content.reassignHosts.target'))) this.get('commands').splice(this.get('commands').indexOf('startAllServices'), 0, 'installPxf');
    }

    this._super();
  },

  initializeTasks: function initializeTasks() {
    var commands = this.get('commands');
    var hostComponentsNames = '';
    this.get('hostComponents').forEach(function (comp, index) {
      hostComponentsNames += index ? '+' : '';
      hostComponentsNames += comp === 'ZKFC' ? comp : App.format.role(comp, false);
    }, this);
    var currentStep = App.router.get('reassignMasterController.currentStep');
    for (var i = 0; i < commands.length; i++) {
      var title = Em.I18n.t('services.reassign.step6.tasks.' + commands[i] + '.title').format(hostComponentsNames);
      this.get('tasks').pushObject(Ember.Object.create({
        title: title,
        status: 'PENDING',
        id: i,
        command: commands[i],
        showRetry: false,
        showRollback: false,
        name: title,
        displayName: title,
        progress: 0,
        isRunning: false,
        hosts: []
      }));
    }

    this.removeUnneededTasks();
    this.set('isLoaded', true);
  },

  removeUnneededTasks: function removeUnneededTasks() {
    if (this.get('content.reassign.component_name') !== 'MYSQL_SERVER') {
      this.removeTasks(['putHostComponentsInMaintenanceMode', 'stopMysqlService']);
      if (!this.get('content.reassignComponentsInMM.length')) {
        this.removeTasks(['stopHostComponentsInMaintenanceMode']);
      }
    } else {
      this.removeTasks(['stopHostComponentsInMaintenanceMode']);
    }
    if (!(this.get('content.reassign.component_name') === 'NAMENODE' && App.get('isHaEnabled'))) {
      this.removeTasks(['startDatanodes']);
    }
  },

  hideRollbackButton: function () {
    var failedTask = this.get('tasks').findProperty('showRollback');
    if (failedTask) {
      failedTask.set('showRollback', false);
    }
  }.observes('tasks.@each.showRollback'),

  onComponentsTasksSuccess: function onComponentsTasksSuccess() {
    this.decrementProperty('multiTaskCounter');
    if (this.get('multiTaskCounter') <= 0) {
      this.onTaskCompleted();
    }
  },

  installPxf: function installPxf() {
    this.createInstallComponentTask('PXF', this.get('content.reassignHosts.target'), "PXF");
  },

  startAllServices: function startAllServices() {
    this.startServices(true);
  },

  deleteHostComponents: function deleteHostComponents() {
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.source');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.delete.host_component',
        sender: this,
        data: {
          hostName: hostName,
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onDeleteHostComponentsError'
      });
    }
  },

  onDeleteHostComponentsError: function onDeleteHostComponentsError(error) {
    if (error.responseText.indexOf('org.apache.ambari.server.controller.spi.NoSuchResourceException') !== -1) {
      this.onComponentsTasksSuccess();
    } else {
      this.onTaskError();
    }
  },

  putHostComponentsInMaintenanceMode: function putHostComponentsInMaintenanceMode() {
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.source');
    this.set('multiTaskCounter', hostComponents.length);
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.host.host_component.passive',
        sender: this,
        data: {
          hostName: hostName,
          passive_state: "ON",
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onTaskError'
      });
    }
  },

  stopHostComponentsInMaintenanceMode: function stopHostComponentsInMaintenanceMode() {
    var hostComponentsInMM = this.get('content.reassignComponentsInMM');
    var hostName = this.get('content.reassignHosts.source');
    var serviceName = this.get('content.reassign.service_id');
    hostComponentsInMM = hostComponentsInMM.map(function (componentName) {
      return {
        hostName: hostName,
        serviceName: serviceName,
        componentName: componentName
      };
    });
    this.set('multiTaskCounter', hostComponentsInMM.length);
    this.updateComponentsState(hostComponentsInMM, 'INSTALLED');
  },

  /**
   * make server call to stop services
   */
  stopMysqlService: function stopMysqlService() {
    var data = {};

    data.context = "Stop required services";
    data.hostName = this.get('content.reassignHosts.source');
    data.serviceName = 'HIVE';
    data.HostRoles = { "state": "INSTALLED" };
    data.componentName = "MYSQL_SERVER";

    App.ajax.send({
      name: 'common.host.host_component.update',
      sender: this,
      data: data,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  startDatanodes: function startDatanodes() {
    this.updateComponent('DATANODE', null, 'HDFS', 'Start');
  }

});

});

require.register("controllers/main/service/reassign/step7_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep7Controller = App.ReassignMasterWizardStep4Controller.extend({
  name: 'reassignMasterWizardStep7Controller',
  commands: ['putHostComponentsInMaintenanceMode', 'deleteHostComponents', 'cleanMySqlServer', 'configureMySqlServer', 'startServices'],
  clusterDeployState: 'REASSIGN_MASTER_INSTALLING',
  multiTaskCounter: 0,

  initializeTasks: function initializeTasks() {

    var commands = this.get('commands');
    var hostComponentsNames = this.getHostComponentsNames();

    for (var i = 0; i < commands.length; i++) {
      var TaskLabel = i === 3 ? this.get('serviceName') : hostComponentsNames; //For Reconfigure task, show serviceName
      var title = Em.I18n.t('services.reassign.step4.tasks.' + commands[i] + '.title').format(TaskLabel);
      this.get('tasks').pushObject(Ember.Object.create({
        title: title,
        status: 'PENDING',
        id: i,
        command: commands[i],
        showRetry: false,
        showRollback: false,
        name: title,
        displayName: title,
        progress: 0,
        isRunning: false,
        hosts: []
      }));
    }

    this.removeUnneededTasks();
    this.set('isLoaded', true);
  },

  putHostComponentsInMaintenanceMode: function putHostComponentsInMaintenanceMode() {
    this.set('multiTaskCounter', 0);
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.target');
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.host.host_component.passive',
        sender: this,
        data: {
          hostName: hostName,
          passive_state: "ON",
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onTaskError'
      });
    }
  },

  deleteHostComponents: function deleteHostComponents() {
    this.set('multiTaskCounter', 0);
    var hostComponents = this.get('hostComponents');
    var hostName = this.get('content.reassignHosts.target');
    for (var i = 0; i < hostComponents.length; i++) {
      App.ajax.send({
        name: 'common.delete.host_component',
        sender: this,
        data: {
          hostName: hostName,
          componentName: hostComponents[i]
        },
        success: 'onComponentsTasksSuccess',
        error: 'onDeleteHostComponentsError'
      });
    }
  }
});

});

require.register("controllers/main/service/reassign_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterController = App.WizardController.extend({

  name: 'reassignMasterController',

  totalSteps: 6,

  /**
   * @type {string}
   */
  displayName: Em.I18n.t('services.reassign.header'),

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  /**
   * All wizards data will be stored in this variable
   *
   * cluster - cluster name
   * installOptions - ssh key, repo info, etc.
   * services - services list
   * hosts - list of selected hosts
   * slaveComponentHosts, - info about slave hosts
   * masterComponentHosts - info about master hosts
   * config??? - to be described later
   */
  content: Em.Object.create({
    cluster: null,
    hosts: null,
    installOptions: null,
    services: null,
    slaveComponentHosts: null,
    masterComponentHosts: null,
    serviceConfigProperties: null,
    advancedServiceConfig: null,
    controllerName: 'reassignMasterController',
    serviceName: 'MISC',
    hdfsUser: "hdfs",
    group: "hadoop",
    reassign: null,
    componentsWithManualCommands: ['NAMENODE', 'SECONDARY_NAMENODE', 'OOZIE_SERVER', 'MYSQL_SERVER', 'APP_TIMELINE_SERVER'],
    hasManualSteps: false,
    hasCheckDBStep: false,
    componentsWithCheckDBStep: ['HIVE_METASTORE', 'HIVE_SERVER', 'OOZIE_SERVER'],
    componentsWithoutSecurityConfigs: ['MYSQL_SERVER'],
    componentsToStopAllServices: ['NAMENODE', 'SECONDARY_NAMENODE'],
    reassignComponentsInMM: [],
    configs: null,
    configsAttributes: null
  }),

  /**
   * Wizard properties in local storage, which should be cleaned right after wizard has been finished
   */
  dbPropertiesToClean: ['cluster', 'hosts', 'installOptions', 'masterComponentHosts', 'serviceComponents', 'masterComponent', 'currentStep', 'reassignHosts', 'tasksStatuses', 'tasksRequestIds', 'requestIds'],

  /**
   * Load data for all steps until <code>current step</code>
   */
  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.loadComponentToReassign();
        this.loadDatabaseType();
        this.loadServiceProperties();
        this.load('cluster');
      }
    }],
    '2': [{
      type: 'async',
      callback: function callback() {
        var self = this,
            dfd = $.Deferred();
        this.loadServicesFromServer();
        this.loadMasterComponentHosts().done(function () {
          self.loadConfirmedHosts();
          dfd.resolve();
        });
        return dfd.promise();
      }
    }],
    '3': [{
      type: 'sync',
      callback: function callback() {
        this.loadReassignHosts();
      }
    }],
    '4': [{
      type: 'sync',
      callback: function callback() {
        this.loadTasksStatuses();
        this.loadTasksRequestIds();
        this.loadRequestIds();
        this.loadReassignComponentsInMM();
        this.loadConfigs();
        this.loadSecureConfigs();
      }
    }],
    '5': [{
      type: 'sync',
      callback: function callback() {
        this.loadComponentDir();
      }
    }]
  },

  serviceToConfigSiteMap: {
    'NAMENODE': ['hdfs-site', 'core-site'],
    'SECONDARY_NAMENODE': ['hdfs-site', 'core-site'],
    'JOBTRACKER': ['mapred-site'],
    'RESOURCEMANAGER': ['yarn-site'],
    'WEBHCAT_SERVER': ['hive-env', 'webhcat-site', 'core-site'],
    'APP_TIMELINE_SERVER': ['yarn-site', 'yarn-env'],
    'OOZIE_SERVER': ['oozie-site', 'core-site', 'oozie-env'],
    'HIVE_SERVER': ['hive-site', 'webhcat-site', 'hive-env', 'core-site'],
    'HIVE_METASTORE': ['hive-site', 'webhcat-site', 'hive-env', 'core-site', 'hive-interactive-site'],
    'MYSQL_SERVER': ['hive-site'],
    'HISTORYSERVER': ['mapred-site'],
    'TIMELINE_READER': ['yarn-site']
  },

  /**
   * Map with lists of related services.
   * Used to define list of services to stop/start.
   */
  relatedServicesMap: {
    'RESOURCEMANAGER': ['YARN', 'MAPREDUCE2', 'TEZ', 'HIVE', 'PIG', 'OOZIE', 'SLIDER', 'SPARK'],
    'APP_TIMELINE_SERVER': ['YARN', 'MAPREDUCE2', 'TEZ', 'HIVE', 'OOZIE', 'SLIDER', 'SPARK'],
    'HISTORYSERVER': ['MAPREDUCE2', 'HIVE', 'PIG', 'OOZIE'],
    'HIVE_SERVER': ['HIVE', 'FALCON', 'ATLAS', 'OOZIE'],
    'HIVE_METASTORE': ['HIVE', 'PIG', 'FALCON', 'ATLAS', 'OOZIE'],
    'WEBHCAT_SERVER': ['HIVE'],
    'OOZIE_SERVER': ['OOZIE', 'FALCON', 'KNOX'],
    'MYSQL_SERVER': ['HIVE', 'OOZIE', 'RANGER', 'RANGER_KMS'],
    'METRICS_COLLECTOR': ['AMBARI_METRICS']
  },

  isComponentWithReconfiguration: function () {
    return this.get('serviceToConfigSiteMap').hasOwnProperty(this.get('content.reassign.component_name'));
  }.property('content.reassign.component_name'),

  addManualSteps: function () {
    var hasManualSteps = this.get('content.componentsWithManualCommands').contains(this.get('content.reassign.component_name'));
    this.set('content.hasManualSteps', hasManualSteps);
    this.set('totalSteps', hasManualSteps ? 6 : 4);
  }.observes('content.reassign.component_name'),

  addCheckDBStep: function () {
    this.set('content.hasCheckDBStep', this.get('content.componentsWithCheckDBStep').contains(this.get('content.reassign.component_name')));
  }.observes('content.reassign.component_name'),

  /**
   * Load tasks statuses for step5 of Reassign Master Wizard to restore installation
   */
  loadTasksStatuses: function loadTasksStatuses() {
    var statuses = App.db.getReassignTasksStatuses();
    this.set('content.tasksStatuses', statuses);
  },

  /**
   * save status of the cluster.
   * @param clusterStatus object with status,requestId fields.
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    var oldStatus = this.toObject(this.get('content.cluster'));
    clusterStatus = jQuery.extend(oldStatus, clusterStatus);
    if (clusterStatus.requestId) {
      clusterStatus.requestId.forEach(function (requestId) {
        if (clusterStatus.oldRequestsId.indexOf(requestId) === -1) {
          clusterStatus.oldRequestsId.push(requestId);
        }
      }, this);
    }
    this.set('content.cluster', clusterStatus);
    this.save('cluster');
  },

  /**
   * Save Master Component Hosts data to Main Controller
   * @param stepController App.WizardStep5Controller
   */
  saveMasterComponentHosts: function saveMasterComponentHosts(stepController) {
    var obj = stepController.get('selectedServicesMasters');

    var masterComponentHosts = [];
    obj.forEach(function (_component) {
      masterComponentHosts.push({
        display_name: _component.get('display_name'),
        component: _component.get('component_name'),
        hostName: _component.get('selectedHost'),
        serviceId: _component.get('serviceId'),
        isInstalled: true
      });
    });
    App.db.setMasterComponentHosts(masterComponentHosts);
    this.set('content.masterComponentHosts', masterComponentHosts);
    this.setDBProperty('masterComponentHosts', masterComponentHosts);
  },

  loadComponentToReassign: function loadComponentToReassign() {
    var masterComponent = App.db.getMasterToReassign();
    if (masterComponent) {
      this.set('content.reassign', masterComponent);
    }
  },

  saveComponentToReassign: function saveComponentToReassign(masterComponent) {
    var component = {
      component_name: masterComponent.get('componentName'),
      display_name: masterComponent.get('displayName'),
      service_id: masterComponent.get('service.serviceName'),
      host_id: masterComponent.get('hostName')
    };
    App.db.setMasterToReassign(component);
  },

  saveTasksStatuses: function saveTasksStatuses(statuses) {
    App.db.setReassignTasksStatuses(statuses);
    this.set('content.tasksStatuses', statuses);
  },

  loadTasksRequestIds: function loadTasksRequestIds() {
    var requestIds = App.db.getReassignTasksRequestIds();
    this.set('content.tasksRequestIds', requestIds);
  },

  saveTasksRequestIds: function saveTasksRequestIds(requestIds) {
    App.db.setReassignTasksRequestIds(requestIds);
    this.set('content.tasksRequestIds', requestIds);
  },

  loadRequestIds: function loadRequestIds() {
    var requestIds = App.db.getReassignMasterWizardRequestIds();
    this.set('content.requestIds', requestIds);
  },

  saveRequestIds: function saveRequestIds(requestIds) {
    App.db.setReassignMasterWizardRequestIds(requestIds);
    this.set('content.requestIds', requestIds);
  },

  saveComponentDir: function saveComponentDir(componentDir) {
    App.db.setReassignMasterWizardComponentDir(componentDir);
    this.set('content.componentDir', componentDir);
  },

  loadComponentDir: function loadComponentDir() {
    var componentDir = App.db.getReassignMasterWizardComponentDir();
    this.set('content.componentDir', componentDir);
  },

  saveReassignHosts: function saveReassignHosts(reassignHosts) {
    App.db.setReassignMasterWizardReassignHosts(reassignHosts);
    this.set('content.reassignHosts', reassignHosts);
  },

  loadReassignHosts: function loadReassignHosts() {
    var reassignHosts = App.db.getReassignMasterWizardReassignHosts();
    this.set('content.reassignHosts', reassignHosts);
  },

  saveSecureConfigs: function saveSecureConfigs(secureConfigs) {
    this.setDBProperty('secureConfigs', secureConfigs);
    this.set('content.secureConfigs', secureConfigs);
  },

  loadSecureConfigs: function loadSecureConfigs() {
    var secureConfigs = this.getDBProperty('secureConfigs');
    this.set('content.secureConfigs', secureConfigs);
  },

  saveServiceProperties: function saveServiceProperties(properties) {
    this.setDBProperty('serviceProperties', properties);
    this.set('content.serviceProperties', properties);
  },

  loadServiceProperties: function loadServiceProperties() {
    var serviceProperties = this.getDBProperty('serviceProperties');
    this.set('content.serviceProperties', serviceProperties);
  },

  saveConfigs: function saveConfigs(configs, attributes) {
    var configsObject = {
      configs: configs,
      configsAttributes: attributes
    };
    this.setDBProperties(configsObject);
    this.get('content').setProperties(configsObject);
  },

  loadConfigs: function loadConfigs() {
    var configsObject = this.getDBProperties(['configs', 'configsAttributes']);
    this.get('content').setProperties(configsObject);
  },

  saveDatabaseType: function saveDatabaseType(type) {
    this.setDBProperty('databaseType', type);
    this.set('content.databaseType', type);
  },

  loadDatabaseType: function loadDatabaseType() {
    var databaseType = this.getDBProperty('databaseType');
    this.set('content.databaseType', databaseType);
    var component = this.get('content.reassign.component_name');

    if (component === 'OOZIE_SERVER') {
      if (this.get('content.hasCheckDBStep') && databaseType && databaseType !== 'derby') {
        // components with manual commands
        var manual = App.router.reassignMasterController.get('content.componentsWithManualCommands').without('OOZIE_SERVER');
        App.router.reassignMasterController.set('content.hasManualSteps', false);
        App.router.reassignMasterController.set('content.componentsWithManualCommands', manual);
        this.set('totalSteps', 4);
      }
    }
  },

  loadReassignComponentsInMM: function loadReassignComponentsInMM() {
    var reassignComponentsInMM = this.getDBProperty('reassignComponentsInMM');
    this.set('content.reassignComponentsInMM', reassignComponentsInMM);
  },

  saveReassignComponentsInMM: function saveReassignComponentsInMM(reassignComponentsInMM) {
    this.setDBProperty('reassignComponentsInMM', reassignComponentsInMM);
    this.set('content.reassignComponentsInMM', reassignComponentsInMM);
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this._super(currentStep, completed);
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      wizardControllerName: 'reassignMasterController',
      localdb: App.db.data
    });
  },

  getReassignComponentsInMM: function getReassignComponentsInMM() {
    var hostComponentsInMM = [];
    var sourceHostComponents = App.HostComponent.find().filterProperty('hostName', this.get('content.reassignHosts.source'));
    var reassignComponents = this.get('content.reassign.component_name') === 'NAMENODE' && App.get('isHaEnabled') ? ['NAMENODE', 'ZKFC'] : [this.get('content.reassign.component_name')];
    reassignComponents.forEach(function (hostComponent) {
      var componentToReassign = sourceHostComponents.findProperty('componentName', hostComponent);
      if (componentToReassign && !componentToReassign.get('isActive') && componentToReassign.get('workStatus') === 'STARTED') {
        hostComponentsInMM.push(hostComponent);
      }
    });
    return hostComponentsInMM;
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.clearAllSteps();
    this.clearStorageData();
    this.resetDbNamespace();
    App.router.get('updateController').updateAll();
  }

});

});

require.register("controllers/main/service/widgets/create/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetWizardStep1Controller = Em.Controller.extend({
  name: "widgetWizardStep1Controller",

  /**
   * Types:
   * - GAUGE
   * - NUMBER
   * - GRAPH
   * - TEMPLATE
   * @type {string}
   */
  widgetType: '',

  /**
   * @type {boolean}
   */
  isSubmitDisabled: Em.computed.not('widgetType'),

  /**
   * @type {App.WidgetType}
   */
  options: function () {
    var selectedType = this.get('widgetType');
    return App.WidgetType.find().map(function (option) {
      return {
        name: option.get('name'),
        displayName: option.get('displayName'),
        iconPath: option.get('iconPath'),
        description: option.get('description')
      };
    });
  }.property('widgetType'),

  /**
   * choose widget type and proceed to the next step
   * @param {object} event
   */
  chooseOption: function chooseOption(event) {
    this.set('widgetType', event.context);
    $("[rel='selectable-tooltip']").trigger('mouseleave');
    this.next();
  },

  loadStep: function loadStep() {
    this.clearStep();
  },

  clearStep: function clearStep() {
    this.set('widgetType', '');
  },

  next: function next() {
    App.router.send('next');
  }
});

});

require.register("controllers/main/service/widgets/create/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.WidgetWizardStep2Controller = Em.Controller.extend({
  name: "widgetWizardStep2Controller",

  EXPRESSION_PREFIX: 'Expression',

  /**
   * @type {RegExp}
   * @const
   */
  EXPRESSION_REGEX: /\$\{([\w\s\.\,\+\-\*\/\(\)\:\=\[\]]*)\}/g,

  /**
   * list of operators that can be used in expression
   * @type {Array}
   * @constant
   */
  OPERATORS: ["+", "-", "*", "/", "(", ")"],

  /**
   * actual values of properties in API format
   * @type {object}
   */
  widgetProperties: {},

  /**
   * @type {Array}
   */
  widgetValues: [],

  /**
   * @type {Array}
   */
  widgetMetrics: [],

  /**
   * @type {Array}
   */
  expressions: [],

  /**
   * used only for GRAPH widget
   * @type {Array}
   */
  dataSets: [],

  /**
   * content of template of Template widget
   * @type {string}
   */
  templateValue: '',

  /**
   * views of properties
   * @type {Array}
   */
  widgetPropertiesViews: [],

  /**
   * @type {boolean}
   */
  isEditWidget: Em.computed.equal('content.controllerName', 'widgetEditController'),

  /**
   * metrics filtered by type
   * @type {Array}
   */
  filteredMetrics: function () {
    var type = this.get('content.widgetType');
    return this.get('content.allMetrics').filter(function (metric) {
      if (type === 'GRAPH') {
        return metric.temporal;
      } else {
        return metric.point_in_time;
      }
    }, this);
  }.property('content.allMetrics'),

  /**
   * @type {boolean}
   */
  isSubmitDisabled: function () {
    if (this.get('widgetPropertiesViews').someProperty('isValid', false)) {
      return true;
    }
    switch (this.get('content.widgetType')) {
      case "NUMBER":
      case "GAUGE":
        var expression = this.get('expressions')[0];
        return !(this.isExpressionComplete(expression) && this.isExpressionWithMetrics(expression));
      case "GRAPH":
        return !this.isGraphDataComplete(this.get('dataSets'));
      case "TEMPLATE":
        return this.get('isTemplateInvalid') || !this.isTemplateDataComplete(this.get('expressions'), this.get('templateValue'));
    }
    return false;
  }.property('widgetPropertiesViews.@each.isValid', 'dataSets.@each.label', 'templateValue', 'isTemplateInvalid'),

  /**
   * check whether data of expression is complete
   * @param {Em.Object} expression
   * @returns {boolean}
   */
  isExpressionComplete: function isExpressionComplete(expression) {
    return Boolean(expression && !expression.get('isInvalid') && !expression.get('isEmpty'));
  },

  /**
   * check whether data of expression contains metrics
   * @param {Em.Object} expression
   * @returns {boolean}
   */
  isExpressionWithMetrics: function isExpressionWithMetrics(expression) {
    return Boolean(expression && expression.get('data') && expression.get('data').someProperty('isMetric'));
  },

  /**
   * check whether any of the expressions is incomplete or invalid
   * @returns {boolean}
   */
  isAnyExpressionInvalid: function () {
    var isAnyExpressionInvalid = false;
    switch (this.get('content.widgetType')) {
      case "NUMBER":
      case "GAUGE":
      case "TEMPLATE":
        isAnyExpressionInvalid = this.get('isSubmitDisabled') && this.get('expressions').someProperty('isEmpty', false);
        break;
      case "GRAPH":
        var dataSets = this.get('dataSets'),
            isNotEmpty = false;
        for (var i = 0; i < dataSets.length; i++) {
          if (dataSets[i].get('expression.data').length > 0) {
            isNotEmpty = true;
            break;
          }
        }
        isAnyExpressionInvalid = this.get('isSubmitDisabled') && isNotEmpty;
    }
    return isAnyExpressionInvalid;
  }.property('isSubmitDisabled'),

  /**
   * check whether data of graph widget is complete
   * @param dataSets
   * @returns {boolean} isComplete
   */
  isGraphDataComplete: function isGraphDataComplete(dataSets) {
    var isComplete = Boolean(dataSets.length),
        expressions = dataSets.mapProperty('expression'),
        isMetricsIncluded = expressions.some(this.isExpressionWithMetrics);

    for (var i = 0; i < dataSets.length; i++) {
      if (!this.checkIfIsLabelValid(dataSets[i]) || !this.isExpressionComplete(dataSets[i].get('expression'))) {
        isComplete = false;
        break;
      }
    }
    return isComplete && isMetricsIncluded;
  },

  /**
   * if label is valid
   * @param dataset
   * @returns {boolean} isValid
   */
  checkIfIsLabelValid: function checkIfIsLabelValid(dataset) {
    var label = dataset.get('label');
    var isValid = label.trim() !== '' && validator.isValidChartWidgetDatasetLabel(label);
    dataset.set('isInvalidLabel', !isValid);
    return isValid;
  },

  /**
   * check whether data of template widget is complete
   * @param {Array} expressions
   * @param {string} templateValue
   * @returns {boolean} isComplete
   */
  isTemplateDataComplete: function isTemplateDataComplete(expressions, templateValue) {
    var isComplete = Boolean(expressions.length > 0 && templateValue.trim() !== ''),
        isMetricsIncluded = expressions.some(this.isExpressionWithMetrics);

    if (isComplete) {
      for (var i = 0; i < expressions.length; i++) {
        if (!this.isExpressionComplete(expressions[i])) {
          isComplete = false;
          break;
        }
      }
    }
    return isComplete && isMetricsIncluded;
  },

  /**
   * Add data set
   * @param {object|null} event
   * @param {boolean} isDefault
   * @returns {number} id
   */
  addDataSet: function addDataSet(event, isDefault) {
    var id = isDefault ? 1 : Math.max.apply(this, this.get('dataSets').mapProperty('id')) + 1;

    this.get('dataSets').pushObject(Em.Object.create({
      id: id,
      label: Em.I18n.t('dashboard.widgets.wizard.step2.dataSeries').format(id),
      isRemovable: !isDefault,
      expression: Em.Object.create({
        id: id,
        data: [],
        isInvalid: false,
        isEmpty: Em.computed.equal('data.length', 0)
      })
    }));
    return id;
  },

  /**
   * Remove data set
   * @param {object} event
   */
  removeDataSet: function removeDataSet(event) {
    this.get('dataSets').removeObject(event.context);
  },

  /**
   * Add expression
   * @param {object|null} event
   * @param {boolean} isDefault
   * @returns {number} id
   */
  addExpression: function addExpression(event, isDefault) {
    var id = isDefault ? 1 : Math.max.apply(this, this.get('expressions').mapProperty('id')) + 1;

    this.get('expressions').pushObject(Em.Object.create({
      id: id,
      isRemovable: !isDefault,
      data: [],
      alias: '{{' + this.get('EXPRESSION_PREFIX') + id + '}}',
      isInvalid: false,
      isEmpty: Em.computed.equal('data.length', 0)
    }));
    return id;
  },

  /**
   * Remove expression
   * @param {object} event
   */
  removeExpression: function removeExpression(event) {
    this.get('expressions').removeObject(event.context);
  },

  /**
   * initialize data
   * widget should have at least one expression or dataSet
   */
  initWidgetData: function initWidgetData() {
    this.set('widgetProperties', this.get('content.widgetProperties'));
    this.set('widgetValues', this.get('content.widgetValues'));
    this.set('widgetMetrics', this.get('content.widgetMetrics'));
    if (this.get('expressions.length') === 0) {
      this.addExpression(null, true);
    }
    if (this.get('dataSets.length') === 0) {
      this.addDataSet(null, true);
    }
  },

  /**
   * update preview widget with latest expression data
   * Note: in order to draw widget it should be converted to API format of widget
   */
  updateExpressions: function () {
    var widgetType = this.get('content.widgetType');
    var expressionData = {
      values: [],
      metrics: []
    };

    if (this.get('expressions').length > 0 && this.get('dataSets').length > 0) {
      switch (widgetType) {
        case 'GAUGE':
        case 'NUMBER':
          expressionData = this.parseExpression(this.get('expressions')[0]);
          expressionData.values = [{
            name: "",
            value: expressionData.value
          }];
          break;
        case 'TEMPLATE':
          expressionData = this.parseTemplateExpression(this.get('templateValue'), this.get('expressions'));
          break;
        case 'GRAPH':
          expressionData = this.parseGraphDataset(this.get('dataSets'));
          break;
      }
    }
    this.set('widgetValues', expressionData.values);
    this.set('widgetMetrics', expressionData.metrics);
  }.observes('templateValue', 'dataSets.@each.label'),

  /**
   * parse Graph data set
   * @param {Array} dataSets
   * @returns {{metrics: Array, values: Array}}
   */
  parseGraphDataset: function parseGraphDataset(dataSets) {
    var metrics = [];
    var values = [];

    dataSets.forEach(function (dataSet) {
      var result = this.parseExpression(dataSet.get('expression'));
      metrics.pushObjects(result.metrics);
      values.push({
        name: dataSet.get('label'),
        value: result.value
      });
    }, this);

    return {
      metrics: metrics,
      values: values
    };
  },

  /**
   * parse expression from template
   * @param {string} templateValue
   * @param {Array} expressions
   * @returns {{metrics: Array, values: {value: *}[]}}
   */
  parseTemplateExpression: function parseTemplateExpression(templateValue, expressions) {
    var metrics = [];
    var self = this;

    // check if there is invalid expression name eg. {{myExpre}}
    var isTemplateInvalid = false;
    var validExpressionName = /\{\{(Expression[\d])\}\}/g;
    var expressionName = /\{\{((?!}}).)*\}\}/g;
    if (templateValue) {
      var expressionNames = templateValue.match(expressionName);
      if (expressionNames) {
        expressionNames.forEach(function (name) {
          if (!name.match(validExpressionName)) {
            isTemplateInvalid = true;
          }
        });
      }
    }
    this.set('isTemplateInvalid', isTemplateInvalid);

    var expression = templateValue.replace(/\{\{Expression[\d]\}\}/g, function (exp) {
      var result;
      if (expressions.someProperty('alias', exp)) {
        result = self.parseExpression(expressions.findProperty('alias', exp));
        metrics.pushObjects(result.metrics);
        return result.value;
      }
      return exp;
    });
    return {
      metrics: metrics,
      values: [{
        value: expression
      }]
    };
  },

  /**
   *
   * @param {object} expression
   * @returns {{metrics: Array, value: string}}
   */
  parseExpression: function parseExpression(expression) {
    var value = '';
    var metrics = [];

    if (expression.data.length > 0) {
      value = '${';
      expression.data.forEach(function (element) {
        if (element.isMetric) {
          var metricObj = {
            "name": element.name,
            "service_name": element.serviceName,
            "component_name": element.componentName,
            "metric_path": element.metricPath,
            "tag": element.tag
          };
          if (element.hostComponentCriteria) {
            metricObj.host_component_criteria = element.hostComponentCriteria;
          }
          metrics.push(metricObj);
        }
        if (element.isOperator) {
          // operators should have spaces around in order to differentiate them when symbol is a part of metric name
          // e.g "metric-a" and "metric1 - metric2"
          value += " " + element.name + " ";
        } else {
          value += element.name;
        }
      }, this);
      value += '}';
    }

    return {
      metrics: metrics,
      value: value
    };
  },

  /**
   * update properties of preview widget
   */
  updateProperties: function () {
    var result = {};

    this.get('widgetPropertiesViews').forEach(function (property) {
      for (var key in property.valueMap) {
        result[property.valueMap[key]] = property.get(key);
      }
    });
    this.set('widgetProperties', result);
  }.observes('widgetPropertiesViews.@each.value', 'widgetPropertiesViews.@each.bigValue', 'widgetPropertiesViews.@each.smallValue'),

  /*
   * Generate the thresholds, unit, time range.etc object based on the widget type selected in previous step.
   */
  renderProperties: function renderProperties() {
    var widgetProperties = App.WidgetType.find(this.get('content.widgetType')).get('properties');
    var propertiesData = this.get('widgetProperties');
    var result = [];

    widgetProperties.forEach(function (property) {
      var definition = App.WidgetPropertyTypes.findProperty('name', property.name);
      property = $.extend({}, property);

      //restore previous values
      for (var key in definition.valueMap) {
        if (propertiesData[definition.valueMap[key]]) {
          property[key] = propertiesData[definition.valueMap[key]];
        }
      }
      result.pushObject(App.WidgetProperty.create($.extend(definition, property)));
    });

    this.set('widgetPropertiesViews', result);
  },

  /**
   * convert data with model format to editable format
   * Note: in order to edit widget expression it should be converted to editable format
   */
  convertData: function convertData() {
    var self = this;
    var expressionId = 0;
    var widgetValues = this.get('content.widgetValues');
    var widgetMetrics = this.get('content.widgetMetrics');

    this.get('expressions').clear();
    this.get('dataSets').clear();

    if (widgetValues.length > 0) {
      switch (this.get('content.widgetType')) {
        case 'NUMBER':
        case 'GAUGE':
          var id = this.addExpression(null, true);
          this.get('expressions').findProperty('id', id).set('data', this.parseValue(widgetValues[0].value, widgetMetrics)[0]);
          break;
        case 'TEMPLATE':
          this.parseValue(widgetValues[0].value, widgetMetrics).forEach(function (item, index) {
            var id = this.addExpression(null, index === 0);
            this.get('expressions').findProperty('id', id).set('data', item);
          }, this);
          this.set('templateValue', widgetValues[0].value.replace(this.get('EXPRESSION_REGEX'), function () {
            return '{{' + self.get('EXPRESSION_PREFIX') + ++expressionId + '}}';
          }));
          break;
        case 'GRAPH':
          widgetValues.forEach(function (value, index) {
            var id = this.addDataSet(null, index === 0);
            var dataSet = this.get('dataSets').findProperty('id', id);
            dataSet.set('label', value.name);
            dataSet.set('expression.data', this.parseValue(value.value, widgetMetrics)[0]);
          }, this);
          break;
      }
    }
  },

  /**
   * parse value
   * @param value
   * @param metrics
   * @returns {Array}
   */
  parseValue: function parseValue(value, metrics) {
    var pattern = this.get('EXPRESSION_REGEX'),
        expressions = [],
        match;

    while (match = pattern.exec(value)) {
      expressions.push(this.getExpressionData(match[1], metrics));
    }

    return expressions;
  },

  /**
   * format values into expression data objects
   * @param {string} expression
   * @param {Array} metrics
   * @returns {Array}
   */
  getExpressionData: function getExpressionData(expression, metrics) {
    var str = '';
    var data = [];
    var id = 0;

    for (var i = 0, l = expression.length; i < l; i++) {
      if (this.get('OPERATORS').contains(expression[i])) {
        if (str.trim().length > 0) {
          data.pushObject(this.getExpressionVariable(str.trim(), ++id, metrics));
          str = '';
        }
        data.pushObject(Em.Object.create({
          id: ++id,
          name: expression[i],
          isOperator: true
        }));
      } else {
        str += expression[i];
      }
    }
    if (str.trim().length > 0) {
      data.pushObject(this.getExpressionVariable(str.trim(), ++id, metrics));
    }
    return data;
  },

  /**
   * get variable of expression
   * could be name of metric "m1" or constant number "1"
   * @param {string} name
   * @param {Array} metrics
   * @param {number} id
   * @returns {Em.Object}
   */
  getExpressionVariable: function getExpressionVariable(name, id, metrics) {
    var metric;

    if (isNaN(Number(name))) {
      metric = metrics.findProperty('name', name);
      return Em.Object.create({
        id: id,
        name: metric.name,
        isMetric: true,
        componentName: metric.component_name,
        serviceName: metric.service_name,
        metricPath: metric.metric_path,
        hostComponentCriteria: metric.host_component_criteria
      });
    } else {
      return Em.Object.create({
        id: id,
        name: name,
        isNumber: true
      });
    }
  },

  next: function next() {
    App.router.send('next');
  }
});

});

require.register("controllers/main/service/widgets/create/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var validator = require('utils/validator');

App.WidgetWizardStep3Controller = Em.Controller.extend({
  name: "widgetWizardStep3Controller",

  isEditController: Em.computed.equal('content.controllerName', 'widgetEditController'),

  /**
   * @type {string}
   */
  widgetName: '',

  /**
   * @type {boolean}
   */
  isNameInvalid: false,

  /**
   * @type {string}
   */
  widgetAuthor: '',

  /**
   * @type {string}
   */
  widgetNameErrorMessage: '',

  /**
   * @type {string}
   */
  descriptionErrorMessage: '',

  /**
   * @type {boolean}
   */
  isSharedChecked: false,

  /**
   * @type {boolean}
   */
  isSharedCheckboxDisabled: false,

  /**
   * @type {string}
   */
  widgetScope: Em.computed.ifThenElse('isSharedChecked', 'Cluster', 'User'),

  /**
   * @type {string}
   */
  widgetDescription: '',

  /**
   * @type {boolean}
   */
  isDescriptionInvalid: false,

  /**
   * actual values of properties in API format
   * @type {object}
   */
  widgetProperties: {},

  /**
   * @type {Array}
   */
  widgetValues: [],

  /**
   * @type {Array}
   */
  widgetMetrics: [],

  /**
   * @type {bool}
   */
  widgetNameEmpty: function () {
    return this.get('widgetName') ? !Boolean(this.get('widgetName').trim()) : true;
  }.property('widgetName'),

  /**
   * @type {boolean}
   */
  isSubmitDisabled: Em.computed.or('widgetNameEmpty', 'isNameInvalid', 'isDescriptionInvalid'),

  /**
   * validates the name on 3rd step
   */
  validateName: function () {
    var errorMessage = '';
    var widgetName = this.get('widgetName');
    this.set("isNameInvalid", false);
    if (widgetName && widgetName.length > 128) {
      errorMessage = Em.I18n.t("widget.create.wizard.step3.name.invalid.msg");
      this.set("isNameInvalid", true);
    }

    if (widgetName && !validator.isValidWidgetName(widgetName)) {
      errorMessage = Em.I18n.t("widget.create.wizard.step3.name.invalidCharacter.msg");
      this.set("isNameInvalid", true);
    }
    this.set('widgetNameErrorMessage', errorMessage);
  }.observes('widgetName'),

  /**
   * validates the description on 3rd step
   */
  validateDescription: function () {
    var errorMessage = '';
    var widgetDescription = this.get('widgetDescription');
    this.set("isDescriptionInvalid", false);
    if (widgetDescription && widgetDescription.length > 2048) {
      errorMessage = Em.I18n.t("widget.create.wizard.step3.description.invalid.msg");
      this.set("isDescriptionInvalid", true);
    }

    if (widgetDescription && !validator.isValidWidgetDescription(widgetDescription)) {
      errorMessage = Em.I18n.t("widget.create.wizard.step3.description.invalidCharacter.msg");
      this.set("isDescriptionInvalid", true);
    }
    this.set('descriptionErrorMessage', errorMessage);
  }.observes('widgetDescription'),

  /**
   * restore widget data set on 2nd step
   */
  initPreviewData: function initPreviewData() {
    this.set('widgetProperties', this.get('content.widgetProperties'));
    this.set('widgetValues', this.get('content.widgetValues'));
    this.set('widgetMetrics', this.get('content.widgetMetrics'));
    this.set('widgetAuthor', this.get('content.widgetAuthor'));
    this.set('widgetName', this.get('content.widgetName'));
    this.set('widgetDescription', this.get('content.widgetDescription'));
    this.set('isSharedChecked', this.get('content.widgetScope') == 'CLUSTER');
    // on editing, don't allow changing from shared scope to unshare
    var isSharedCheckboxDisabled = this.get('content.widgetScope') == 'CLUSTER' && this.get('isEditController');
    this.set('isSharedCheckboxDisabled', isSharedCheckboxDisabled);
    if (!isSharedCheckboxDisabled) {
      this.addObserver('isSharedChecked', this, this.showConfirmationOnSharing);
    }
  },

  /**
   * confirmation popup
   * @returns {App.ModalPopup|undefined}
   */
  showConfirmationOnSharing: function showConfirmationOnSharing() {
    var self = this;
    if (this.get('isSharedChecked')) {
      var bodyMessage = Em.Object.create({
        confirmMsg: Em.I18n.t('dashboard.widgets.browser.action.share.confirmation'),
        confirmButton: Em.I18n.t('dashboard.widgets.browser.action.share')
      });
      return App.showConfirmationFeedBackPopup(function (query) {
        self.set('isSharedChecked', true);
      }, bodyMessage, function (query) {
        self.set('isSharedChecked', false);
      });
    }
  },

  /**
   * collect all needed data to create new widget
   * @returns {{WidgetInfo: {cluster_name: *, widget_name: *, widget_type: *, description: *, scope: string, metrics: *, values: *, properties: *}}}
   */
  collectWidgetData: function collectWidgetData() {
    var widgetData = {
      WidgetInfo: {
        widget_name: this.get('widgetName'),
        widget_type: this.get('content.widgetType'),
        description: this.get('widgetDescription') || "",
        scope: this.get('widgetScope').toUpperCase(),
        author: this.get('widgetAuthor'),
        values: this.get('widgetValues').map(function (value) {
          delete value.computedValue;
          return value;
        }),
        properties: this.get('widgetProperties')
      }
    };

    this.get('widgetMetrics').forEach(function (metric) {
      if (metric.tag) widgetData.WidgetInfo.tag = metric.tag;
    });

    widgetData.WidgetInfo.metrics = this.get('widgetMetrics').map(function (metric) {
      delete metric.data;
      delete metric.tag;
      return metric;
    });

    return widgetData;
  },

  cancel: function cancel() {
    App.router.get(this.get('content.controllerName')).cancel();
  },

  complete: function complete() {
    App.router.send('complete', this.collectWidgetData());
    App.router.get(this.get('content.controllerName')).finishWizard();
  }
});

});

require.register("controllers/main/service/widgets/create/wizard_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetWizardController = App.WizardController.extend({

  name: 'widgetWizardController',

  totalSteps: 3,

  /**
   * Used for hiding back button in wizard
   */
  hideBackButton: true,

  content: Em.Object.create({
    controllerName: 'widgetWizardController',
    widgetService: null,
    widgetType: "",
    isMetricsLoaded: false,

    /**
     * @type {Object}
     * @default null
     */
    layout: null,

    /**
     * Example:
     * {
     *  "display_unit": "%",
     *  "warning_threshold": 70,
     *  "error_threshold": 90
     * }
     */
    widgetProperties: {},

    /**
     * Example:
     * [{
     *  widget_id: "metrics/rpc/closeRegion_num_ops",
     *  name: "rpc.rpc.closeRegion_num_ops",
     *  pointInTime: true,
     *  temporal: true,
     *  category: "default"
     *  serviceName: "HBASE"
     *  componentName: "HBASE_CLIENT"
     *  type: "GANGLIA"//or JMX
     *  level: "COMPONENT"//or HOSTCOMPONENT
     * }]
     * @type {Array}
     */
    allMetrics: [],

    /**
     * Example:
     * [{
     *  "name": "regionserver.Server.percentFilesLocal",
     *  "serviceName": "HBASE",
     *  "componentName": "HBASE_REGIONSERVER"
     * }]
     */
    widgetMetrics: [],

    /**
     * Example:
     * [{
     *  "name": "Files Local",
     *  "value": "${regionserver.Server.percentFilesLocal}"
     * }]
     */
    widgetValues: [],
    widgetName: "",
    widgetDescription: "",
    widgetAuthor: Em.computed.alias('App.router.loginName'),
    widgetScope: null
  }),

  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('widgetService');
        this.load('widgetType');
        this.load('layout', true);
      }
    }],
    '2': [{
      type: 'sync',
      callback: function callback() {
        this.load('widgetProperties', true);
        this.load('widgetValues', true);
        this.load('widgetMetrics', true);
        this.load('expressions', true);
        this.load('dataSets', true);
        this.load('templateValue', true);
      }
    }, {
      type: 'async',
      callback: function callback() {
        return this.loadAllMetrics();
      }
    }]
  },

  /**
   * set current step
   * @param {string} currentStep
   * @param {boolean} completed
   * @param {boolean} skipStateSave
   */
  setCurrentStep: function setCurrentStep(currentStep, completed, skipStateSave) {
    this._super(currentStep, completed);
    if (App.get('testMode') || skipStateSave) {
      return;
    }
    this.saveClusterStatus('WIDGET_DEPLOY');
  },

  setStepsEnable: function () {
    for (var i = 1; i <= this.get('totalSteps'); i++) {
      var step = this.get('isStepDisabled').findProperty('step', i);
      if (i <= this.get('currentStep') && App.get('router.clusterController.isLoaded')) {
        step.set('value', false);
      } else {
        step.set('value', i != this.get('currentStep'));
      }
    }
  }.observes('currentStep', 'App.router.clusterController.isLoaded'),

  /**
   * save status of the cluster.
   * @param {object} clusterStatus
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    App.clusterStatus.setClusterStatus({
      clusterState: clusterStatus,
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
  },

  /**
   * save wizard properties to controller and localStorage
   * @param {string} name
   * @param value
   */
  save: function save(name, value) {
    this.set('content.' + name, value);
    this._super(name);
  },

  /**
   * load widget metrics
   * on resolve deferred return array of widget metrics
   * @returns {$.Deferred}
   */
  loadAllMetrics: function loadAllMetrics() {
    var allMetrics = this.getDBProperty('allMetrics');
    var self = this;
    this.set("content.isMetricsLoaded", false);
    var dfd = $.Deferred();

    if (allMetrics.length === 0) {
      this.loadAllMetricsFromServer(function () {
        self.set("content.isMetricsLoaded", true);
        dfd.resolve(self.get('content.allMetrics'));
      });
    } else {
      this.set("content.isMetricsLoaded", true);
      this.set('content.allMetrics', allMetrics);
      dfd.resolve(allMetrics);
    }
    return dfd.promise();
  },

  /**
   * load metrics from server
   * @param {function} callback
   * @returns {$.ajax}
   */
  loadAllMetricsFromServer: function loadAllMetricsFromServer(callback) {
    return App.ajax.send({
      name: 'widgets.wizard.metrics.get',
      sender: this,
      data: {
        stackVersionURL: App.get('stackVersionURL'),
        serviceNames: App.Service.find().filter(function (item) {
          return App.StackService.find(item.get('id')).get('isServiceWithWidgets');
        }).mapProperty('serviceName').join(',')
      },
      callback: callback,
      success: 'loadAllMetricsFromServerCallback'
    });
  },

  /**
   *
   * @param {object} json
   */
  loadAllMetricsFromServerCallback: function loadAllMetricsFromServerCallback(json) {
    var self = this;
    var result = [];
    var metrics = {};
    var slaveComponents = App.StackServiceComponent.find().filterProperty('isSlave').mapProperty('componentName');
    if (json) {
      json.items.forEach(function (service) {
        var data = service.artifacts[0].artifact_data[service.StackServices.service_name];
        for (var componentName in data) {
          var isSlave = slaveComponents.contains(componentName);
          for (var level in data[componentName]) {
            var metricTypes = data[componentName][level]; //Ganglia or JMX
            metricTypes.forEach(function (_metricType) {
              metrics = _metricType['metrics']['default'];
              var type = _metricType["type"].toUpperCase();
              if (!(type === 'JMX' && level.toUpperCase() === 'COMPONENT' || isSlave && level.toUpperCase() === 'HOSTCOMPONENT')) {
                for (var widgetId in metrics) {
                  var _metrics = metrics[widgetId];
                  var metricObj = {
                    widget_id: widgetId,
                    point_in_time: _metrics.pointInTime,
                    temporal: _metrics.temporal,
                    name: _metrics.name,
                    level: level.toUpperCase(),
                    type: type,
                    component_name: componentName,
                    service_name: service.StackServices.service_name
                  };
                  result.push(metricObj);
                  if (metricObj.level === 'HOSTCOMPONENT') {
                    self.insertHostComponentCriteria(metricObj);
                  }
                }
              }
            }, this);
          }
        }
      }, this);
    }
    if (!!App.YARNService.find("YARN")) {
      result = this.substitueQueueMetrics(result);
    }
    this.save('allMetrics', result);
  },

  /**
   * @name substitueQueueMetrics
   * @param metrics
   * @return {Array} array of metric objects with regex substituted with actual metrics names
   */
  substitueQueueMetrics: function substitueQueueMetrics(metrics) {
    var result = [];
    var queuePaths = App.YARNService.find("YARN").get("allQueueNames");
    var queueNames = [];
    var queueMetricName;
    queuePaths.forEach(function (_queuePath) {
      var queueName = _queuePath.replace(/\//g, ".");
      queueNames.push(queueName);
    }, this);
    var regexpForAMS = new RegExp("^yarn.QueueMetrics.Queue=\\(\\.\\+\\).*$");
    var regexpForJMX = new RegExp("\\(\\.\\+\\)", "g");
    var replaceRegexForMetricName = regexpForJMX;
    var replaceRegexForMetricPath = new RegExp("\\$\\d\\..*\\)(?=\\/)", "g");
    metrics.forEach(function (_metric) {
      var isAMSQueueMetric = regexpForAMS.test(_metric.name);
      var isJMXQueueMetrics = regexpForJMX.test(_metric.name);
      if (_metric.type === 'GANGLIA' && isAMSQueueMetric || _metric.type === 'JMX' && isJMXQueueMetrics) {
        queuePaths.forEach(function (_queuePath) {
          queueMetricName = '';
          if (_metric.type === 'GANGLIA') {
            queueMetricName = _queuePath.replace(/\//g, ".");
          } else if (_metric.type === 'JMX') {
            _queuePath.split("/").forEach(function (_metricName, index) {
              queueMetricName = queueMetricName + ',q' + index + '=' + _metricName;
            }, this);
          }
          var metricName = _metric.name.replace(replaceRegexForMetricName, queueMetricName);
          var newMetricPath = _metric.widget_id.replace(replaceRegexForMetricPath, _queuePath);
          var newQueueMetric = $.extend(true, {}, _metric, { name: metricName, widget_id: newMetricPath });
          result.pushObject(newQueueMetric);
        }, this);
      } else {
        result.pushObject(_metric);
      }
    }, this);
    return result;
  },

  /**
   *
   * @param metricObj {Object}
   */
  insertHostComponentCriteria: function insertHostComponentCriteria(metricObj) {
    switch (metricObj.component_name) {
      case 'NAMENODE':
        metricObj.host_component_criteria = 'host_components/metrics/dfs/FSNamesystem/HAState=active';
        break;
      case 'RESOURCEMANAGER':
        metricObj.host_component_criteria = 'host_components/HostRoles/ha_state=ACTIVE';
        break;
      case 'HBASE_MASTER':
        metricObj.host_component_criteria = 'host_components/metrics/hbase/master/IsActiveMaster=true';
        break;
      default:
        metricObj.host_component_criteria = ' ';
    }
  },

  /**
   * post widget definition to server
   * @returns {$.ajax}
   */
  postWidgetDefinition: function postWidgetDefinition(data) {
    return App.ajax.send({
      name: 'widgets.wizard.add',
      sender: this,
      data: {
        data: data
      },
      success: 'postWidgetDefinitionSuccessCallback'
    });
  },

  /**
   * assign created widget to active layout if it present or to appropriate nameservice layout and all-nameservices layout
   * @param data
   * @param requestObject
   * @param requestData
   */
  postWidgetDefinitionSuccessCallback: function postWidgetDefinitionSuccessCallback(data, requestObject, requestData) {
    if (Em.isNone(this.get('content.layout'))) return;
    var mainServiceInfoMetricsController = App.router.get('mainServiceInfoMetricsController');
    var self = this;
    var layout = this.get('content.layout');
    var newWidgeet = data.resources[0].WidgetInfo;
    var tag = requestData.data.WidgetInfo.tag;

    if (tag) {
      this.loadLayoutByName(mainServiceInfoMetricsController.get('userLayoutName') + '_nameservice_' + tag).done(function (data) {
        data.items[0].WidgetLayoutInfo.widgets = data.items[0].WidgetLayoutInfo.widgets.map(function (w) {
          return w.WidgetInfo.id;
        });
        layout = data.items[0].WidgetLayoutInfo;
        self.addWidgetToLayout(newWidgeet, layout).done(function () {
          self.loadLayoutByName(mainServiceInfoMetricsController.get('userLayoutName') + '_nameservice_all').done(function (data) {
            data.items[0].WidgetLayoutInfo.widgets = data.items[0].WidgetLayoutInfo.widgets.map(function (w) {
              return w.WidgetInfo.id;
            });
            self.addWidgetToLayout(newWidgeet, data.items[0].WidgetLayoutInfo).done(function () {
              mainServiceInfoMetricsController.updateActiveLayout();
            });
          });
        });
      });
    } else {
      this.addWidgetToLayout(newWidgeet, layout).done(function () {
        mainServiceInfoMetricsController.updateActiveLayout();
      });
    }
  },

  addWidgetToLayout: function addWidgetToLayout(newWidget, layout) {
    var mainServiceInfoMetricsController = App.router.get('mainServiceInfoMetricsController');
    var widgets = layout.widgets.map(function (item) {
      return Em.Object.create({ id: item });
    });
    widgets.pushObject(Em.Object.create({
      id: newWidget.id
    }));
    return mainServiceInfoMetricsController.saveWidgetLayout(widgets, Em.Object.create(layout));
  },

  loadLayoutByName: function loadLayoutByName(name) {
    return App.ajax.send({
      name: 'widget.layout.name.get',
      sender: this,
      data: {
        name: name
      }
    });
  },

  /**
   * Remove all loaded data.
   * Created as copy for App.router.clearAllSteps
   */
  clearAllSteps: function clearAllSteps() {
    this.clearInstallOptions();
    // clear temporary information stored during the install
    this.set('content.cluster', this.getCluster());
  },

  clearTasksData: function clearTasksData() {
    this.saveTasksStatuses(undefined);
    this.saveRequestIds(undefined);
    this.saveTasksRequestIds(undefined);
  },

  cancel: function cancel() {
    var self = this;
    var step3Controller = App.router.get('widgetWizardStep3Controller');
    var isLastStep = parseInt(self.get('currentStep')) === self.get('totalSteps');
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      body: Em.I18n.t('dashboard.widgets.wizard.onClose.popup.body'),
      primary: isLastStep ? Em.I18n.t('common.save') : null,
      secondary: Em.I18n.t('dashboard.widgets.wizard.onClose.popup.discardAndExit'),
      third: Em.I18n.t('common.cancel'),
      disablePrimary: function () {
        return !(isLastStep && !step3Controller.get('isSubmitDisabled'));
      }.property(),
      onPrimary: function onPrimary() {
        App.router.send('complete', step3Controller.collectWidgetData());
        this.onSecondary();
      },
      onSecondary: function onSecondary() {
        this.hide();
        self.finishWizard();
      },
      onThird: function onThird() {
        this.hide();
      }
    });
  },

  /**
   * finish wizard
   */
  finishWizard: function finishWizard() {
    var service = App.Service.find(this.get('content.widgetService'));

    this.finish();
    var self = this;
    var successCallBack = function successCallBack() {
      self.get('popup').hide();
      App.router.transitionTo('main.services.service.metrics', service);
      App.get('router.updateController').updateAll();
    };

    if (App.get('testMode')) {
      successCallBack();
    } else {
      App.clusterStatus.setClusterStatus({
        clusterName: App.router.getClusterName(),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, { successCallback: successCallBack });
    }
  },

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.setCurrentStep('1', false, true);
    this.save('widgetType', '');
    this.resetDbNamespace();
  }
});

});

require.register("controllers/main/service/widgets/edit_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetEditController = App.WidgetWizardController.extend({

  name: 'widgetEditController',

  totalSteps: 2,

  content: Em.Object.create({
    controllerName: 'widgetEditController',
    widgetService: null,
    widgetType: '',

    /**
     * Example:
     * {
     *  "display_unit": "%",
     *  "warning_threshold": 70,
     *  "error_threshold": 90
     * }
     */
    widgetProperties: {},

    /**
     * Example:
     * [{
     *  widget_id: "metrics/rpc/closeRegion_num_ops",
     *  name: "rpc.rpc.closeRegion_num_ops",
     *  pointInTime: true,
     *  temporal: true,
     *  category: "default"
     *  serviceName: "HBASE"
     *  componentName: "HBASE_CLIENT"
     *  type: "GANGLIA"//or JMX
     *  level: "COMPONENT"//or HOSTCOMPONENT
     * }]
     * @type {Array}
     */
    allMetrics: [],

    /**
     * Example:
     * [{
     *  "name": "regionserver.Server.percentFilesLocal",
     *  "serviceName": "HBASE",
     *  "componentName": "HBASE_REGIONSERVER"
     * }]
     */
    widgetMetrics: [],

    /**
     * Example:
     * [{
     *  "name": "Files Local",
     *  "value": "${regionserver.Server.percentFilesLocal}"
     * }]
     */
    widgetValues: [],
    widgetName: null,
    widgetDescription: null,
    widgetScope: null,
    widgetAuthor: null,
    widgetId: null,
    isMetricsLoaded: false
  }),

  loadMap: {
    '1': [{
      type: 'sync',
      callback: function callback() {
        this.load('widgetType');
        this.load('widgetService');
        this.load('widgetProperties', true);
        this.load('widgetValues', true);
        this.load('widgetMetrics', true);
      }
    }, {
      type: 'async',
      callback: function callback() {
        return this.loadAllMetrics();
      }
    }],
    '2': [{
      type: 'sync',
      callback: function callback() {
        this.load('widgetName');
        this.load('widgetDescription');
        this.load('widgetAuthor');
      }
    }]
  },

  /**
   * post widget definition to server
   * @returns {$.ajax}
   */
  putWidgetDefinition: function putWidgetDefinition(data) {
    return App.ajax.send({
      name: 'widgets.wizard.edit',
      sender: this,
      data: {
        data: data,
        widgetId: this.get('content.widgetId')
      },
      success: 'putWidgetDefinitionSuccessCallback'
    });
  },

  putWidgetDefinitionSuccessCallback: function putWidgetDefinitionSuccessCallback() {},

  /**
   * Clear all temporary data
   */
  finish: function finish() {
    this.setCurrentStep('1', false, true);
    this.save('widgetType', '');
    this.save('widgetService', '');
    this.save('widgetProperties', null);
    this.save('widgetMetrics', []);
    this.save('widgetValues', []);
    this.save('widgetName', '');
    this.save('widgetDescription', '');
    this.save('widgetAuthor', '');
    this.save('widgetScope', '');
    this.save('allMetrics', []);
    this.save('expressions', []);
    this.save('dataSets', []);
    this.save('templateValue', '');
    this.resetDbNamespace();
  }
});

});

require.register("controllers/main/views/details_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.MainViewsDetailsController = Em.Controller.extend({
  name: 'mainViewsDetailsController'
});

});

require.register("controllers/main/views_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainViewsController = Em.Controller.extend({
  name: 'mainViewsController',

  isDataLoaded: false,

  ambariViews: [],

  visibleAmbariViews: Em.computed.filterBy('ambariViews', 'visible', true),

  dataLoading: function dataLoading() {
    var viewsController = this;
    var dfd = $.Deferred();
    if (this.get('isDataLoaded')) {
      dfd.resolve(this.get('ambariViews'));
    } else {
      var interval = setInterval(function () {
        if (viewsController.get('isDataLoaded')) {
          dfd.resolve(viewsController.get('ambariViews'));
          clearInterval(interval);
        }
      }, 50);
    }
    return dfd.promise();
  },

  loadAmbariViews: function loadAmbariViews() {
    if (App.router.get('loggedIn')) {
      return App.ajax.send({
        name: 'views.info',
        sender: this,
        success: 'loadAmbariViewsSuccess',
        error: 'loadAmbariViewsError'
      });
    }
  },

  loadAmbariViewsSuccess: function loadAmbariViewsSuccess(data, opt, params) {
    if (data.items.length) {
      App.ajax.send({
        name: 'views.instances',
        sender: this,
        success: 'loadViewInstancesSuccess',
        error: 'loadViewInstancesError'
      });
    } else {
      this.set('ambariViews', []);
      this.set('isDataLoaded', true);
    }
  },

  loadAmbariViewsError: function loadAmbariViewsError() {
    this.set('ambariViews', []);
    this.set('isDataLoaded', true);
  },

  loadViewInstancesSuccess: function loadViewInstancesSuccess(data, opt, params) {
    this.set('ambariViews', []);
    var instances = [];
    data.items.forEach(function (view) {
      view.versions.forEach(function (version) {
        version.instances.forEach(function (instance) {
          var info = instance.ViewInstanceInfo;
          var currentInstance = App.ViewInstance.create({
            iconPath: info.icon_path || '/img/ambari-view-default.png',
            label: info.label || version.ViewVersionInfo.label || info.view_name,
            visible: info.visible || false,
            version: info.version,
            description: info.description || Em.I18n.t('views.main.instance.noDescription'),
            viewName: info.view_name,
            shortUrl: info.short_url,
            instanceName: info.instance_name,
            href: info.context_path + '/'
          });
          if (currentInstance.visible) {
            instances.push(currentInstance);
          }
        }, this);
      }, this);
    }, this);
    this.get('ambariViews').pushObjects(instances);
    this.set('isDataLoaded', true);
  },

  loadViewInstancesError: function loadViewInstancesError() {
    this.set('ambariViews', []);
    this.set('isDataLoaded', true);
  },

  setView: function setView(event) {
    if (event.context) {
      window.open(event.context.get('internalAmbariUrl'));
    }
  }
});

});

require.register("controllers/wizard", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('models/host');

App.WizardController = Em.Controller.extend(App.LocalStorage, App.ThemesMappingMixin, {

  isStepDisabled: null,

  previousStep: 0,
  /**
   * map of actions which load data required by which step
   * used by <code>loadAllPriorSteps</code>
   */
  loadMap: {},

  /**
   * Wizard properties in local storage, which should be cleaned right after wizard has been finished
   */
  dbPropertiesToClean: ['service', 'hosts', 'masterComponentHosts', 'slaveComponentHosts', 'cluster', 'allHostNames', 'installOptions', 'allHostNamesPattern', 'serviceComponents', 'fileNamesToUpdate', 'componentsFromConfigs'],

  sensibleConfigs: [{ name: 'admin_principal', filename: 'krb5-conf.xml' }, { name: 'admin_password', filename: 'krb5-conf.xml' }],

  init: function init() {
    this.clusters = App.Cluster.find();
    this.setIsStepDisabled();
  },

  connectOutlet: function connectOutlet(name) {
    if (name !== 'loading') this.set('isStepDisabled.isLocked', false);
    App.router.setProperties({
      backBtnClickInProgress: false,
      nextBtnClickInProgress: false
    });
    return this._super.apply(this, arguments);
  },

  /**
   * Set <code>isStepDisabled</code> with list of available steps (basing on <code>totalSteps</code>)
   * @method setIsStepDisabled
   */
  setIsStepDisabled: function setIsStepDisabled() {
    this.set('isStepDisabled', Ember.ArrayProxy.create({
      content: [],
      isLocked: true,
      objectAtContent: function objectAtContent(idx) {
        var obj = this.get('content').objectAt(idx);
        if (obj && !obj.hasOwnProperty('isLocked')) {
          obj.reopen({
            isLocked: true,
            get: function get(key) {
              return key === 'value' && this.get('isLocked') || this._super.apply(this, arguments);
            },
            notifyValues: function () {
              this.notifyPropertyChange('value');
            }.observes('isLocked')
          });
        }
        return obj;
      },
      toggleLock: function () {
        this.setEach('isLocked', this.get('isLocked'));
      }.observes('isLocked')
    }));
    this.get('isStepDisabled').pushObject(Em.Object.create({
      step: 1,
      value: false
    }));
    for (var i = 2; i <= this.get('totalSteps'); i++) {
      this.get('isStepDisabled').pushObject(Em.Object.create({
        step: i,
        value: true
      }));
    }
  },

  slaveComponents: function () {
    return App.StackServiceComponent.find().filterProperty('isSlave', true);
  }.property('App.router.clusterController.isLoaded'),

  allHosts: function () {
    var dbHosts = this.get('content.hosts');
    var hosts = [];

    for (var hostName in dbHosts) {
      var hostComponents = [];
      dbHosts[hostName].hostComponents.forEach(function (componentName) {
        hostComponents.push(Em.Object.create({
          componentName: componentName,
          displayName: App.format.role(componentName, false)
        }));
      });

      hosts.push(Em.Object.create({
        id: hostName,
        hostName: hostName,
        hostComponents: hostComponents
      }));
    }
    return hosts;
  }.property('content.hosts'),

  setStepsEnable: function () {
    for (var i = 1; i <= this.totalSteps; i++) {
      var step = this.get('isStepDisabled').findProperty('step', i);
      if (i <= this.get('currentStep')) {
        step.set('value', false);
      } else {
        step.set('value', true);
      }
    }
  }.observes('currentStep'),

  /**
   * Enable step link in left nav menu
   * @param step - step number
   */
  enableStep: function enableStep(step) {
    this.get('isStepDisabled').findProperty('step', step).set('value', false);
  },

  setLowerStepsDisable: function setLowerStepsDisable(stepNo) {
    for (var i = 1; i < stepNo; i++) {
      var step = this.get('isStepDisabled').findProperty('step', i);
      step.set('value', true);
    }
  },

  /**
   * Set current step to new value.
   * Method moved from App.router.setInstallerCurrentStep
   * @param currentStep
   * @param completed
   */
  currentStep: function () {
    return App.get('router').getWizardCurrentStep(this.get('name').substr(0, this.get('name').length - 10));
  }.property(),

  /**
   * Set current step to new value.
   * Method moved from App.router.setInstallerCurrentStep
   * @param currentStep
   * @param completed
   */
  setCurrentStep: function setCurrentStep(currentStep, completed) {
    this.set('previousStep', this.get('currentStep'));
    App.db.setWizardCurrentStep(this.get('name').substr(0, this.get('name').length - 10), currentStep, completed);
    this.set('currentStep', currentStep);
  },

  clusters: null,

  isStep0: function () {
    return this.get('currentStep') == 0;
  }.property('currentStep'),

  isStep1: function () {
    return this.get('currentStep') == 1;
  }.property('currentStep'),

  isStep2: function () {
    return this.get('currentStep') == 2;
  }.property('currentStep'),

  isStep3: function () {
    return this.get('currentStep') == 3;
  }.property('currentStep'),

  isStep4: function () {
    return this.get('currentStep') == 4;
  }.property('currentStep'),

  isStep5: function () {
    return this.get('currentStep') == 5;
  }.property('currentStep'),

  isStep6: function () {
    return this.get('currentStep') == 6;
  }.property('currentStep'),

  isStep7: function () {
    return this.get('currentStep') == 7;
  }.property('currentStep'),

  isStep8: function () {
    return this.get('currentStep') == 8;
  }.property('currentStep'),

  isStep9: function () {
    return this.get('currentStep') == 9;
  }.property('currentStep'),

  isStep10: function () {
    return this.get('currentStep') == 10;
  }.property('currentStep'),

  /**
   * Move user to the selected step
   *
   * @param {number} step number of the step, where user is moved
   * @param {boolean} disableNaviWarning true - don't show warning about moving more than 1 step back
   * @returns {boolean}
   */
  gotoStep: function gotoStep(step, disableNaviWarning) {
    if (this.get('isStepDisabled').findProperty('step', step).get('value') !== false) {
      return false;
    }
    var currentStep = this.get('currentStep');
    var currentControllerName = this.get('content.controllerName');
    // if going back from Step 9 in Install Wizard, delete the checkpoint so that the user is not redirected
    // to Step 9
    if (currentControllerName === 'installerController' && currentStep === '9' && step < 9) {
      App.clusterStatus.setClusterStatus({
        clusterName: this.get('clusterName'),
        clusterState: 'CLUSTER_NOT_CREATED_1',
        wizardControllerName: 'installerController',
        localdb: {}
      });
    }
    var isCustomizeServicesStep = false;
    if (currentControllerName === 'installerController' && currentStep === '7' || (currentControllerName === 'addServiceController' || currentControllerName === 'addHostController') && currentStep === '4') {
      isCustomizeServicesStep = true;
    }
    var stepDiff = currentStep - step;
    if (!disableNaviWarning && (stepDiff > 1 || isCustomizeServicesStep && stepDiff > 0)) {
      App.ModalPopup.show({
        header: Em.I18n.t('installer.navigation.warning.header'),
        onPrimary: function onPrimary() {
          App.router.send('gotoStep' + step);
          this.hide();
        },
        body: "If you proceed to go back to Step " + step + ", you will lose any changes you have made beyond this step"
      });
    } else {
      App.router.send('gotoStep' + step);
    }
    return true;
  },

  gotoStep0: function gotoStep0() {
    this.gotoStep(0);
  },

  gotoStep1: function gotoStep1() {
    this.gotoStep(1);
  },

  gotoStep2: function gotoStep2() {
    this.gotoStep(2);
  },

  gotoStep3: function gotoStep3() {
    this.gotoStep(3);
  },

  gotoStep4: function gotoStep4() {
    this.gotoStep(4);
  },

  gotoStep5: function gotoStep5() {
    this.gotoStep(5);
  },

  gotoStep6: function gotoStep6() {
    this.gotoStep(6);
  },

  gotoStep7: function gotoStep7() {
    this.gotoStep(7);
  },

  gotoStep8: function gotoStep8() {
    this.gotoStep(8);
  },

  gotoStep9: function gotoStep9() {
    this.gotoStep(9);
  },

  gotoStep10: function gotoStep10() {
    this.gotoStep(10);
  },

  /**
   * Initialize host status info for step9
   */
  setInfoForStep9: function setInfoForStep9() {

    var hostInfo = this.getDBProperty('hosts');
    for (var index in hostInfo) {
      if (hostInfo.hasOwnProperty(index)) {
        hostInfo[index].status = "pending";
        hostInfo[index].message = 'Waiting';
        hostInfo[index].logTasks = [];
        hostInfo[index].tasks = [];
        hostInfo[index].progress = '0';
      }
    }
    this.setDBProperty('hosts', hostInfo);
  },

  /**
   * Remove all data for installOptions step
   */
  clearInstallOptions: function clearInstallOptions() {
    var installOptions = this.getInstallOptions();
    this.set('content.installOptions', installOptions);
    this.set('content.hosts', {});
    this.setDBProperties({
      installOptions: installOptions,
      hosts: {}
    });
  },

  toObject: function toObject(object) {
    var result = {};
    for (var i in object) {
      if (object.hasOwnProperty(i)) {
        result[i] = object[i];
      }
    }
    return result;
  },

  /**
   * Convert any object or array to pure JS instance without inherit properties
   * It is used to convert Ember.Object to pure JS Object and Ember.Array to pure JS Array
   * @param originalInstance
   * @returns {*}
   */
  toJSInstance: function toJSInstance(originalInstance) {
    var convertedInstance = originalInstance;
    if (Em.isArray(originalInstance)) {
      convertedInstance = [];
      originalInstance.forEach(function (element) {
        convertedInstance.push(this.toJSInstance(element));
      }, this);
    } else if (originalInstance && (typeof originalInstance === 'undefined' ? 'undefined' : _typeof(originalInstance)) === 'object') {
      convertedInstance = {};
      for (var property in originalInstance) {
        if (originalInstance.hasOwnProperty(property)) {
          convertedInstance[property] = this.toJSInstance(originalInstance[property]);
        }
      }
    }
    return convertedInstance;
  },

  /**
   * save status of the cluster. This is called from step8 and step9 to persist install and start requestId
   * @param clusterStatus object with status, isCompleted, requestId, isInstallError and isStartError field.
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    var oldStatus = this.toObject(this.get('content.cluster'));
    clusterStatus = jQuery.extend(oldStatus, clusterStatus);
    if (clusterStatus.requestId && clusterStatus.oldRequestsId.indexOf(clusterStatus.requestId) === -1) {
      clusterStatus.oldRequestsId.push(clusterStatus.requestId);
    }
    this.set('content.cluster', clusterStatus);
    this.setDBProperty('cluster', clusterStatus);
  },

  /**
   * Invoke installation of selected services to the server and saves the request id returned by the server.
   * @param isRetry
   */
  installServices: function installServices(isRetry, callback) {
    // clear requests since we are installing services
    // and we don't want to get tasks for previous install attempts
    this.set('content.cluster.oldRequestsId', []);
    var data;
    callback = callback || Em.K;
    if (isRetry) {
      data = {
        context: Em.I18n.t('requestInfo.installComponents'),
        HostRoles: { "state": "INSTALLED" },
        urlParams: "HostRoles/desired_state=INSTALLED&HostRoles/state!=INSTALLED"
      };
    } else {
      data = {
        context: Em.I18n.t('requestInfo.installServices'),
        ServiceInfo: { "state": "INSTALLED" },
        urlParams: "ServiceInfo/state=INIT"
      };
    }

    var clusterStatus = {
      status: 'PENDING'
    };
    this.saveClusterStatus(clusterStatus);

    App.ajax.send({
      name: isRetry ? 'common.host_components.update' : 'common.services.update',
      sender: this,
      data: data,
      success: 'installServicesSuccessCallback',
      error: 'installServicesErrorCallback'
    }).then(callback, callback);
  },

  installServicesSuccessCallback: function installServicesSuccessCallback(jsonData) {
    var installStartTime = App.dateTime();
    if (jsonData) {
      var requestId = jsonData.Requests.id;
      var clusterStatus = {
        status: 'PENDING',
        requestId: requestId,
        isInstallError: false,
        isCompleted: false,
        installStartTime: installStartTime
      };
      this.saveClusterStatus(clusterStatus);
    }
  },

  installServicesErrorCallback: function installServicesErrorCallback(request, ajaxOptions, error) {
    var clusterStatus = {
      status: 'PENDING',
      requestId: this.get('content.cluster.requestId'),
      isInstallError: true,
      isCompleted: false
    };
    this.saveClusterStatus(clusterStatus);
    App.showAlertPopup(Em.I18n.t('common.errorPopup.header'), request.responseText);
  },
  /**
   * show popup, that display status of bootstrap launching
   * @param callback
   * @return {Object}
   */
  showLaunchBootstrapPopup: function showLaunchBootstrapPopup(callback) {
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step2.bootStrap.header'),
      isError: false,
      serverError: null,
      bodyClass: Em.View.extend({
        templateName: require('templates/wizard/bootstrap_call_popup')
      }),
      showFooter: false,
      showCloseButton: false,
      secondary: null,
      /**
       * handle requestId when call is completed,
       * if it's correct call callback and hide popup
       * otherwise notify error and enable buttons to close popup
       * @param requestId
       * @param serverError
       * @param status
       * @param log
       */
      finishLoading: function finishLoading(requestId, serverError, status, log) {
        if (Em.isNone(requestId) || status === 'ERROR') {
          var stepController = App.get('router.wizardStep3Controller');
          this.setProperties({
            isError: true,
            showFooter: true,
            showCloseButton: true,
            serverError: status === 'ERROR' ? log : serverError
          });
          stepController.setProperties({
            isRegistrationInProgress: false,
            isBootstrapFailed: true
          });
          stepController.get('hosts').setEach('bootStatus', 'FAILED');
        } else {
          callback(requestId);
          this.hide();
        }
      }
    });
  },
  /**
   * Bootstrap selected hosts.
   * @param bootStrapData
   * @param callback
   * @return {Object}
   */
  launchBootstrap: function launchBootstrap(bootStrapData, callback) {
    var popup = this.showLaunchBootstrapPopup(callback);
    App.ajax.send({
      name: 'wizard.launch_bootstrap',
      sender: this,
      data: {
        bootStrapData: bootStrapData,
        popup: popup
      },
      success: 'launchBootstrapSuccessCallback',
      error: 'launchBootstrapErrorCallback'
    });
    return popup;
  },

  launchBootstrapSuccessCallback: function launchBootstrapSuccessCallback(data, opt, params) {
    params.popup.finishLoading(data.requestId, null, data.status, data.log);
  },

  launchBootstrapErrorCallback: function launchBootstrapErrorCallback(request, ajaxOptions, error, opt, params) {
    params.popup.finishLoading(null, error);
  },

  /**
   * Load <code>content.<name></code> variable from localStorage, if wasn't loaded before.
   * If you specify <code>reload</code> to true - it will reload it.
   * @param name
   * @param reload
   * @return {Boolean}
   */
  load: function load(name, reload) {
    if (this.get('content.' + name) && !reload) {
      return false;
    }
    var result = this.getDBProperty(name);
    if (!result) {
      if (this['get' + name.capitalize()]) {
        result = this['get' + name.capitalize()]();
        this.setDBProperty(name, result);
      } else {
        console.debug('get' + name.capitalize(), ' not defined in the ' + this.get('name'));
      }
    }
    this.set('content.' + name, result);
  },

  save: function save(name) {
    var convertedValue = this.toJSInstance(this.get('content.' + name));
    this.setDBProperty(name, convertedValue);
  },

  clear: function clear() {
    this.set('content', Ember.Object.create({
      'controllerName': this.get('content.controllerName')
    }));
    this.set('currentStep', 0);
    this.clearStorageData();
  },

  clusterStatusTemplate: {
    name: "",
    status: "PENDING",
    isCompleted: false,
    requestId: null,
    installStartTime: null,
    installTime: null,
    isInstallError: false,
    isStartError: false,
    oldRequestsId: []
  },

  clearStorageData: function clearStorageData() {
    var hash = {};
    this.get('dbPropertiesToClean').forEach(function (key) {
      hash[key] = undefined;
    }, this);
    this.setDBProperties(hash);
  },

  getInstallOptions: function getInstallOptions() {
    return jQuery.extend({}, App.get('isHadoopWindowsStack') ? this.get('installWindowsOptionsTemplate') : this.get('installOptionsTemplate'));
  },

  installOptionsTemplate: {
    hostNames: "", //string
    manualInstall: false, //true, false
    useSsh: true, //bool
    javaHome: App.defaultJavaHome, //string
    localRepo: false, //true, false
    sshKey: "", //string
    bootRequestId: null, //string
    sshUser: "root", //string
    sshPort: "22",
    agentUser: "root" //string
  },

  installWindowsOptionsTemplate: {
    hostNames: "", //string
    manualInstall: false, //true, false
    useSsh: true, //bool
    javaHome: App.defaultJavaHome, //string
    localRepo: false, //true, false
    sshKey: "", //string
    bootRequestId: null, //string
    sshUser: "", //string
    sshPort: "22",
    agentUser: "" //string
  },

  loadedServiceComponents: null,

  /**
   * Generate serviceComponents as pr the stack definition  and save it to localdata
   * called form stepController step4WizardController
   */
  loadServiceComponents: function loadServiceComponents() {
    return App.ajax.send({
      name: 'wizard.service_components',
      sender: this,
      data: {
        stackUrl: App.get('stackVersionURL'),
        stackVersion: App.get('currentStackVersionNumber')
      },
      success: 'loadServiceComponentsSuccessCallback',
      error: 'loadServiceComponentsErrorCallback'
    });
  },

  loadServiceComponentsSuccessCallback: function loadServiceComponentsSuccessCallback(jsonData) {
    var props = this.getDBProperties(['selectedServiceNames', 'installedServiceNames']);
    var savedSelectedServices = props.selectedServiceNames;
    var savedInstalledServices = props.installedServiceNames;
    this.set('content.selectedServiceNames', savedSelectedServices);
    this.set('content.installedServiceNames', savedInstalledServices);
    if (!savedSelectedServices) {
      jsonData.items.forEach(this.setStackServiceSelectedByDefault);
    } else {
      jsonData.items.forEach(function (service) {
        service.StackServices.is_selected = savedSelectedServices.contains(service.StackServices.service_name);
      }, this);
    }

    if (!savedInstalledServices) {
      jsonData.items.forEach(function (service) {
        service.StackServices.is_installed = false;
      }, this);
    } else {
      jsonData.items.forEach(function (service) {
        service.StackServices.is_installed = savedInstalledServices.contains(service.StackServices.service_name);
      }, this);
    }

    App.stackServiceMapper.mapStackServices(jsonData);
  },

  loadServiceComponentsErrorCallback: function loadServiceComponentsErrorCallback(request, ajaxOptions, error) {},

  /**
   * @param {object} service
   */
  setStackServiceSelectedByDefault: function setStackServiceSelectedByDefault(service) {
    service.StackServices.is_selected = !(service.StackServices.selection === "TECH_PREVIEW");
    if (service.StackServices.service_type === 'HCFS' && service.StackServices.service_name !== 'HDFS') {
      service.StackServices.is_selected = false;
    }
  },

  /**
   * load version for services to display on Choose Servoces page
   * should load from VersionDefinition endpoint
   */
  loadServiceVersionFromVersionDefinitions: function loadServiceVersionFromVersionDefinitions() {
    return App.ajax.send({
      name: 'cluster.load_current_repo_stack_services',
      sender: this,
      data: {
        clusterName: App.clusterName
      },
      success: 'loadServiceVersionFromVersionDefinitionsSuccessCallback',
      error: 'loadServiceVersionFromVersionDefinitionsErrorCallback'
    });
  },

  serviceVersionsMap: {},
  loadServiceVersionFromVersionDefinitionsSuccessCallback: function loadServiceVersionFromVersionDefinitionsSuccessCallback(jsonData) {
    var rv = Em.getWithDefault(jsonData, 'items', []).filter(function (i) {
      return Em.getWithDefault(i, 'ClusterStackVersions.stack', null) === App.get('currentStackName') && Em.getWithDefault(i, 'ClusterStackVersions.version', null) === App.get('currentStackVersionNumber');
    })[0];
    var map = this.get('serviceVersionsMap');
    var stackServices = Em.getWithDefault(rv || {}, 'repository_versions.0.RepositoryVersions.stack_services', false);
    if (stackServices) {
      stackServices.forEach(function (item) {
        map[item.name] = item.versions[0];
      });
    }
  },
  loadServiceVersionFromVersionDefinitionsErrorCallback: function loadServiceVersionFromVersionDefinitionsErrorCallback(request, ajaxOptions, error) {},

  /**
   * Load config groups from local DB
   */
  loadServiceConfigGroups: function loadServiceConfigGroups() {
    var props = this.getDBProperties(['serviceConfigGroups', 'hosts']);
    var serviceConfigGroups = props.serviceConfigGroups,
        hosts = props.hosts || {},
        host_names = Em.keys(hosts);
    if (Em.isNone(serviceConfigGroups)) {
      serviceConfigGroups = [];
    } else {
      serviceConfigGroups.forEach(function (group) {
        var hostNames = group.hosts.map(function (host_id) {
          for (var i = 0; i < host_names.length; i++) {
            if (hosts[host_names[i]].id === host_id) {
              return host_names[i];
            }
          }
          Em.assert('host is missing!!!!', false);
        });
        Em.set(group, 'hosts', hostNames);
      });
    }
    this.set('content.configGroups', serviceConfigGroups);
  },

  registerErrPopup: function registerErrPopup(header, message) {
    App.ModalPopup.show({
      header: header,
      secondary: false,
      bodyClass: Ember.View.extend({
        template: Ember.Handlebars.compile('<p>{{view.message}}</p>'),
        message: message
      })
    });
  },

  /**
   * Save hosts that the user confirmed to proceed with from step 3
   * @param stepController App.WizardStep3Controller
   */
  saveConfirmedHosts: function saveConfirmedHosts(stepController) {
    var hosts = this.get('content.hosts'),
        indx = 1;

    //add previously installed hosts
    for (var hostName in hosts) {
      if (!hosts[hostName].isInstalled) {
        delete hosts[hostName];
      }
    }

    stepController.get('confirmedHosts').forEach(function (_host) {
      if (_host.bootStatus === 'REGISTERED') {
        hosts[_host.name] = {
          name: _host.name,
          bootStatus: _host.bootStatus,
          isInstalled: false,
          id: indx++
        };
      }
    });
    this.setDBProperty('hosts', hosts);
    this.set('content.hosts', hosts);
  },

  /**
   * Save data after installation to main controller
   * @param stepController App.WizardStep9Controller
   */
  saveInstalledHosts: function saveInstalledHosts(stepController) {
    var hosts = stepController.get('hosts');
    var hostInfo = this.getDBProperty('hosts');

    for (var index in hostInfo) {
      hostInfo[index].status = "pending";
      var host = hosts.findProperty('name', hostInfo[index].name);
      if (host) {
        hostInfo[index].status = host.status;
        hostInfo[index].message = host.message;
        hostInfo[index].progress = host.progress;
      }
    }
    this.set('content.hosts', hostInfo);
    this.setDBProperty('hosts', hostInfo);
  },

  /**
   * Save slaveHostComponents to main controller
   * @param stepController
   */
  saveSlaveComponentHosts: function saveSlaveComponentHosts(stepController) {
    var hosts = stepController.get('hosts'),
        dbHosts = this.getDBProperty('hosts'),
        headers = stepController.get('headers');

    var formattedHosts = Ember.Object.create();
    headers.forEach(function (header) {
      formattedHosts.set(header.get('name'), []);
    });

    hosts.forEach(function (host) {

      var checkboxes = host.checkboxes;
      headers.forEach(function (header) {
        var cb = checkboxes.findProperty('title', header.get('label'));
        if (cb.checked) {
          formattedHosts.get(header.get('name')).push({
            group: 'Default',
            isInstalled: cb.isInstalled,
            host_id: dbHosts[host.hostName].id
          });
        }
      });
    });

    var slaveComponentHosts = [];

    headers.forEach(function (header) {
      slaveComponentHosts.push({
        componentName: header.get('name'),
        displayName: header.get('label').replace(/\s/g, ''),
        hosts: formattedHosts.get(header.get('name'))
      });
    });

    this.setDBProperty('slaveComponentHosts', slaveComponentHosts);
    this.set('content.slaveComponentHosts', slaveComponentHosts);
  },

  /**
   * Return true if cluster data is loaded and false otherwise.
   * This is used for all wizard controllers except for installer wizard.
   */
  dataLoading: function dataLoading() {
    var dfd = $.Deferred();
    this.connectOutlet('loading');
    if (App.router.get('clusterController.isLoaded')) {
      dfd.resolve();
    } else {
      var interval = setInterval(function () {
        if (App.router.get('clusterController.isLoaded')) {
          dfd.resolve();
          clearInterval(interval);
        }
      }, 50);
    }
    return dfd.promise();
  },

  /**
   * Return true if user data is loaded via App.MainServiceInfoConfigsController
   * This function is used in reassign master wizard right now.
   */

  usersLoading: function usersLoading() {
    var self = this;
    var dfd = $.Deferred();
    var miscController = App.MainAdminServiceAccountsController.create({ content: self.get('content') });
    miscController.loadUsers();
    var interval = setInterval(function () {
      if (miscController.get('dataIsLoaded')) {
        if (self.get("content.hdfsUser")) {
          self.set('content.hdfsUser', miscController.get('content.hdfsUser'));
        }
        dfd.resolve();
        clearInterval(interval);
      }
    }, 10);
    return dfd.promise();
  },

  /**
   * Save cluster status before going to deploy step
   * @param name cluster state. Unique for every wizard
   * @param callbackObj can have additional params for ajax callBacks and sender
   */
  saveClusterState: function saveClusterState(name, callbackObj) {
    App.clusterStatus.setClusterStatus({
      clusterName: this.get('content.cluster.name'),
      clusterState: name,
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    }, callbackObj);
  },

  /**
   * Load serviceConfigProperties from localStorage
   */
  loadServiceConfigProperties: function loadServiceConfigProperties() {
    var stackConfigs = App.configsCollection.getAll();
    var serviceConfigProperties = this.getDBProperty('serviceConfigProperties');
    this.set('content.serviceConfigProperties', this.applyStoredConfigs(stackConfigs, serviceConfigProperties));
  },

  /**
   *
   * @param {array} configs
   * @param {?array} storedConfigs
   * @returns {?array}
   */
  applyStoredConfigs: function applyStoredConfigs(configs, storedConfigs) {
    if (storedConfigs && storedConfigs.length) {
      var result = [];
      var configsMap = configs.toMapByProperty('id');
      storedConfigs.forEach(function (stored) {
        var config = configsMap[stored.id];
        if (config) {
          result.push(_extends({}, config, stored, { savedValue: null }));
        } else if (stored.isUserProperty) {
          result.push(_extends({}, stored));
        }
      });
      return result;
    }
    return storedConfigs;
  },

  /**
   * Save config properties
   * @param stepController Step7WizardController
   */
  saveServiceConfigProperties: function saveServiceConfigProperties(stepController) {
    var serviceConfigProperties = [];
    // properties in db should contain only mutable info to avoid localStorage overflow
    var dbConfigProperties = [];
    var fileNamesToUpdate = this.getDBProperty('fileNamesToUpdate') || [];
    var installedServiceNames = stepController.get('installedServiceNames') || [];
    var installedServiceNamesMap = installedServiceNames.toWickMap();
    stepController.get('stepConfigs').forEach(function (_content) {
      if (_content.serviceName === 'YARN') {
        _content.set('configs', App.config.textareaIntoFileConfigs(_content.get('configs'), 'capacity-scheduler.xml'));
      }
      _content.get('configs').forEach(function (_configProperties) {
        if (!Em.isNone(_configProperties.get('group'))) {
          return false;
        }
        var configProperty = App.config.createDefaultConfig(_configProperties.get('name'), _configProperties.get('filename'),
        // need to invert boolean because this argument will be inverted in method body
        !_configProperties.get('isUserProperty'), _configProperties.getProperties('value', 'isRequired', 'errorMessage', 'warnMessage', 'propertyType'));
        configProperty = App.config.mergeStaticProperties(configProperty, _configProperties, [], ['name', 'filename', 'isUserProperty', 'value']);

        var dbConfigProperty = {
          id: _configProperties.get('id'),
          value: _configProperties.get('value'),
          isFinal: _configProperties.get('isFinal')
        };
        if (_configProperties.get('isUserProperty') || _configProperties.get('filename') === 'capacity-scheduler.xml') {
          dbConfigProperty = configProperty;
        }
        if (this.isExcludedConfig(configProperty)) {
          configProperty.value = '';
          dbConfigProperty.value = '';
        }
        dbConfigProperties.push(dbConfigProperty);
        serviceConfigProperties.push(configProperty);
      }, this);
      // check for configs that need to update for installed services
      if (installedServiceNamesMap[_content.get('serviceName')]) {
        // get only modified configs
        var configs = _content.get('configs').filter(function (config) {
          if (config.get('isNotDefaultValue') || config.get('savedValue') === null) {
            return config.isRequiredByAgent !== false;
          }
          return false;
        });
        // if modified configs detected push all service's configs for update
        if (configs.length) {
          fileNamesToUpdate = fileNamesToUpdate.concat(configs.mapProperty('filename').uniq());
        }
      }
    }, this);
    this.set('content.serviceConfigProperties', serviceConfigProperties);
    this.setDBProperties({
      fileNamesToUpdate: fileNamesToUpdate,
      serviceConfigProperties: dbConfigProperties
    });
  },

  isExcludedConfig: function isExcludedConfig(configProperty) {
    return this.get('sensibleConfigs').mapProperty('name').indexOf(configProperty.name) > -1 && this.get('sensibleConfigs').mapProperty('filename').indexOf(configProperty.filename) > -1;
  },

  /**
   * save Config groups
   * @param stepController
   * @param isAddService
   */
  saveServiceConfigGroups: function saveServiceConfigGroups(stepController, isAddService) {
    var serviceConfigGroups = [],
        hosts = isAddService ? App.router.get('addServiceController').getDBProperty('hosts') : this.getDBProperty('hosts');
    stepController.get('stepConfigs').forEach(function (service) {
      // mark group of installed service
      var isForInstalledService = service.get('selected') === false;
      service.get('configGroups').forEach(function (configGroup) {
        var properties = [];
        configGroup.get('properties').forEach(function (property) {
          properties.push({
            name: property.get('name'),
            value: property.get('value'),
            isFinal: property.get('isFinal'),
            filename: property.get('filename')
          });
        });
        //configGroup copied into plain JS object to avoid Converting circular structure to JSON
        var hostIds = configGroup.get('hosts').map(function (host_name) {
          return hosts[host_name].id;
        });
        serviceConfigGroups.push({
          id: configGroup.get('id'),
          name: configGroup.get('name'),
          description: configGroup.get('description'),
          hosts: hostIds.slice(),
          properties: properties.slice(),
          is_default: configGroup.get('isDefault'),
          is_for_installed_service: isForInstalledService,
          is_for_update: configGroup.get('isForUpdate') || configGroup.get('hash') !== this.getConfigGroupHash(configGroup, configGroup.get('hosts')),
          service_name: configGroup.get('serviceName'),
          service_id: configGroup.get('serviceName'),
          desired_configs: configGroup.get('desiredConfigs'),
          child_config_groups: configGroup.get('childConfigGroups') ? configGroup.get('childConfigGroups').mapProperty('id') : [],
          parent_config_group_id: configGroup.get('parentConfigGroup.id'),
          is_temporary: configGroup.get('isTemporary')
        });
      }, this);
    }, this);
    this.setDBProperty('serviceConfigGroups', serviceConfigGroups);
    this.set('content.configGroups', serviceConfigGroups);
  },

  /**
   * generate string hash for config group
   * @param {Object} configGroup
   * @param {Array|undefined} hosts
   * @returns {String|null}
   * @method getConfigGroupHash
   */
  getConfigGroupHash: function getConfigGroupHash(configGroup, hosts) {
    if (!Em.get(configGroup, 'properties.length') && !Em.get(configGroup, 'hosts.length') && !hosts) {
      return null;
    }
    var hash = {};
    Em.get(configGroup, 'properties').forEach(function (config) {
      hash[Em.get(config, 'name')] = { value: Em.get(config, 'value'), isFinal: Em.get(config, 'isFinal') };
    });
    hash.hosts = hosts || Em.get(configGroup, 'hosts');
    return JSON.stringify(hash);
  },

  /**
   * return slaveComponents bound to hosts
   * @return {Array}
   */
  getSlaveComponentHosts: function getSlaveComponentHosts() {
    var components = this.get('slaveComponents');
    var result = [];
    var installedServices = App.Service.find().mapProperty('serviceName');
    var selectedServices = App.StackService.find().filterProperty('isSelected', true).mapProperty('serviceName');
    var installedComponentsMap = {};
    var uninstalledComponents = [];

    components.forEach(function (component) {
      if (installedServices.contains(component.get('serviceName'))) {
        installedComponentsMap[component.get('componentName')] = [];
      } else if (selectedServices.contains(component.get('serviceName'))) {
        uninstalledComponents.push(component);
      }
    }, this);
    installedComponentsMap.HDFS_CLIENT = [];

    App.HostComponent.find().forEach(function (hostComponent) {
      if (installedComponentsMap[hostComponent.get('componentName')]) {
        installedComponentsMap[hostComponent.get('componentName')].push(hostComponent.get('hostName'));
      }
    }, this);

    for (var componentName in installedComponentsMap) {
      var name = componentName === 'HDFS_CLIENT' ? 'CLIENT' : componentName;
      var component = {
        componentName: name,
        displayName: App.format.role(name, false),
        hosts: [],
        isInstalled: true
      };
      installedComponentsMap[componentName].forEach(function (hostName) {
        component.hosts.push({
          group: "Default",
          hostName: hostName,
          isInstalled: true
        });
      }, this);
      result.push(component);
    }

    uninstalledComponents.forEach(function (component) {
      var hosts = jQuery.extend(true, [], result.findProperty('componentName', 'DATANODE').hosts);
      hosts.setEach('isInstalled', false);
      result.push({
        componentName: component.get('componentName'),
        displayName: App.format.role(component.get('componentName'), false),
        hosts: hosts,
        isInstalled: false
      });
    });

    return result;
  },
  /**
   * Load master component hosts data for using in required step controllers
   */
  loadMasterComponentHosts: function loadMasterComponentHosts() {
    var masterComponentHosts = this.getDBProperty('masterComponentHosts'),
        self = this,
        observerContext = {
      setMasterComponentHosts: function setMasterComponentHosts() {
        if (App.get('router.clusterController.isServiceMetricsLoaded')) {
          var stackMasterComponents = App.get('components.masters').uniq();
          App.HostComponent.find().filter(function (component) {
            return stackMasterComponents.contains(component.get('componentName'));
          }).forEach(function (item) {
            masterComponentHosts.push({
              component: item.get('componentName'),
              hostName: item.get('hostName'),
              isInstalled: true,
              serviceId: item.get('service.id'),
              display_name: item.get('displayName')
            });
          });
          self.setDBProperty('masterComponentHosts', masterComponentHosts);
          self.set('content.masterComponentHosts', masterComponentHosts);
          self.removeObserver('App.router.clusterController.isServiceMetricsLoaded', this, 'setMasterComponentHosts');
          dfd.resolve();
        }
      }
    },
        dfd = $.Deferred();
    if (Em.isNone(masterComponentHosts)) {
      masterComponentHosts = [];
      if (App.get('router.clusterController.isServiceMetricsLoaded')) {
        observerContext.setMasterComponentHosts();
      } else {
        this.addObserver('App.router.clusterController.isServiceMetricsLoaded', observerContext, 'setMasterComponentHosts');
      }
    } else {
      this.set('content.masterComponentHosts', masterComponentHosts);
      dfd.resolve();
    }
    return dfd.promise();
  },

  /**
   * Save Master Component Hosts data to Main Controller
   * @param stepController App.WizardStep5Controller
   */
  saveMasterComponentHosts: function saveMasterComponentHosts(stepController) {
    var obj = stepController.get('selectedServicesMasters');
    var masterComponentHosts = [];
    obj.forEach(function (_component) {
      masterComponentHosts.push({
        display_name: _component.get('display_name'),
        component: _component.get('component_name'),
        hostName: _component.get('selectedHost'),
        serviceId: _component.get('serviceId'),
        isInstalled: _component.get('isInstalled')
      });
    });
    this.setDBProperty('masterComponentHosts', masterComponentHosts);
    this.set('content.masterComponentHosts', masterComponentHosts);
  },

  clearMasterComponentHosts: function clearMasterComponentHosts() {
    this.set('content.masterComponentHosts', null);
    this.setDBProperty('masterComponentHosts', null);
  },

  /**
   * Load information about hosts with clients components
   */
  loadClients: function loadClients() {
    var clients = this.getDBProperty('clientInfo');
    this.set('content.clients', clients);
  },

  /**
   * load methods assigned to each step
   * methods executed in exact order as they described in map
   * @return {object}
   */
  loadAllPriorSteps: function loadAllPriorSteps() {
    var currentStep = this.get('currentStep');
    var loadMap = this.get('loadMap');
    var operationStack = [];
    var dfd = $.Deferred();

    for (var s in loadMap) {
      if (parseInt(s, 10) <= parseInt(currentStep, 10)) {
        operationStack.pushObjects(loadMap[s]);
      }
    }

    var sequence = App.actionSequence.create({ context: this });
    sequence.setSequence(operationStack).onFinish(function () {
      dfd.resolve();
    }).start();

    return dfd.promise();
  },

  /**
   * return new object extended from clusterStatusTemplate
   * @return Object
   */
  getCluster: function getCluster() {
    return jQuery.extend({}, this.get('clusterStatusTemplate'), { name: App.router.getClusterName() });
  },

  /**
   * Load services data from server.
   */
  loadServicesFromServer: function loadServicesFromServer() {
    var services = this.getDBProperty('services');
    if (!services) {
      services = {
        selectedServices: [],
        installedServices: []
      };
      App.StackService.find().forEach(function (item) {
        var isInstalled = App.Service.find().someProperty('id', item.get('serviceName'));
        item.set('isSelected', isInstalled);
        item.set('isInstalled', isInstalled);
        if (isInstalled) {
          services.selectedServices.push(item.get('serviceName'));
          services.installedServices.push(item.get('serviceName'));
        }
      }, this);
      this.setDBProperty('services', services);
    } else {
      App.StackService.find().forEach(function (item) {
        var isSelected = services.selectedServices.contains(item.get('serviceName'));
        var isInstalled = services.installedServices.contains(item.get('serviceName'));
        item.set('isSelected', isSelected);
        item.set('isInstalled', isInstalled);
      }, this);
    }
    this.set('content.services', App.StackService.find());
  },

  /**
   * Load confirmed hosts.
   * Will be used at <code>Assign Masters(step5)</code> step
   */
  loadConfirmedHosts: function loadConfirmedHosts() {
    this.set('content.hosts', this.getDBProperty('hosts') || {});
  },

  loadHosts: function loadHosts() {
    var dfd;
    var hostsInDb = this.getDBProperty('hosts');
    if (hostsInDb) {
      this.set('content.hosts', hostsInDb);
      dfd = $.Deferred();
      dfd.resolve();
    } else {
      dfd = App.ajax.send({
        name: 'hosts.confirmed',
        sender: this,
        data: {},
        success: 'loadHostsSuccessCallback',
        error: 'loadHostsErrorCallback'
      });
    }
    return dfd.promise();
  },

  loadHostsSuccessCallback: function loadHostsSuccessCallback(response) {
    var installedHosts = {};

    response.items.forEach(function (item, indx) {
      installedHosts[item.Hosts.host_name] = {
        name: item.Hosts.host_name,
        bootStatus: "REGISTERED",
        isInstalled: true,
        hostComponents: item.host_components,
        id: indx++
      };
    });
    this.setDBProperty('hosts', installedHosts);
    this.set('content.hosts', installedHosts);
  },

  loadHostsErrorCallback: function loadHostsErrorCallback(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
  },

  /**
   * Determine if <code>Assign Slaves and Clients</code> step should be skipped
   * @method setSkipSlavesStep
   * @param services
   * @param step
   */
  setSkipSlavesStep: function setSkipSlavesStep(services, step) {
    var hasServicesWithSlave = services.someProperty('hasSlave');
    var hasServicesWithClient = services.someProperty('hasClient');
    var hasServicesWithCustomAssignedNonMasters = services.someProperty('hasNonMastersWithCustomAssignment');
    var hasDependentSlaveComponent = this.get('name') === 'addServiceController' ? this.hasDependentSlaveComponent(services) : false;
    this.set('content.skipSlavesStep', (!hasServicesWithSlave && !hasServicesWithClient || !hasServicesWithCustomAssignedNonMasters) && !hasDependentSlaveComponent);
    if (this.get('content.skipSlavesStep')) {
      this.get('isStepDisabled').findProperty('step', step).set('value', this.get('content.skipSlavesStep'));
    }
  },

  /**
   * Determine if there is some service with some component, that has dependent slave component already installed in cluster, but not on all hosts
   * @param services
   * @returns {boolean}
   */
  hasDependentSlaveComponent: function hasDependentSlaveComponent(services) {
    var result = false;
    var dependentSlaves = [];
    var hosts = this.get('content.hosts');

    if (hosts) {
      services.forEach(function (service) {
        service.get('serviceComponents').forEach(function (component) {
          component.get('dependencies').forEach(function (dependency) {
            var dependentService = App.StackService.find().findProperty('serviceName', dependency.serviceName);
            var dependentComponent = dependentService && dependentService.get('serviceComponents').findProperty('componentName', dependency.componentName);
            if (dependentComponent && dependentComponent.get('isSlave') && dependentService.get('isInstalled')) {
              dependentSlaves.push({ component: dependentComponent.get('componentName'), count: 0 });
            }
          });
        });
      });

      var hostNames = Em.keys(hosts);
      for (var i = 0; i < dependentSlaves.length; i++) {
        var maxToInstall = App.StackServiceComponent.find().findProperty('componentName', dependentSlaves[i].component).get('maxToInstall');
        maxToInstall = maxToInstall === Infinity ? hostNames.length : maxToInstall;
        hostNames.forEach(function (hostName) {
          var hostComponents = hosts[hostName].hostComponents.mapProperty('HostRoles.component_name');
          dependentSlaves[i].count += hostComponents.contains(dependentSlaves[i].component);
        });

        if (dependentSlaves[i].count < maxToInstall) {
          result = true;
          break;
        }
      }
    }

    return result;
  },

  /**
   * Load config themes for enhanced config layout.
   *
   * @method loadConfigThemes
   * @return {$.Deferred}
   */
  loadConfigThemes: function loadConfigThemes() {
    var self = this;
    var dfd = $.Deferred();
    if (!this.get('stackConfigsLoaded')) {
      var serviceNames = App.StackService.find().filter(function (s) {
        return s.get('isSelected') || s.get('isInstalled');
      }).mapProperty('serviceName');
      // Load stack configs before loading themes
      App.config.loadClusterConfigsFromStack().always(function () {
        App.config.loadConfigsFromStack(serviceNames).done(function () {
          self.loadConfigThemeForServices(serviceNames).always(function () {
            self.set('stackConfigsLoaded', true);
            dfd.resolve();
          });
        });
      });
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  /**
   * Clear all config static data
   * and theme info
   *
   * @method clearEnhancedConfigs
   */
  clearEnhancedConfigs: function clearEnhancedConfigs() {
    App.configsCollection.clearAll();
    App.Section.find().clear();
    App.SubSection.find().clear();
    App.SubSectionTab.find().clear();
    App.Tab.find().clear();
    this.set('stackConfigsLoaded', false);
  },

  /**
   * Cache "stepConfigs" to local storage in name value pairs
   * @param stepController
   */
  cacheStepConfigValues: function cacheStepConfigValues(stepController) {
    var self = this;
    var stepConfigs = [];
    stepController.get("stepConfigs").forEach(function (category) {
      var configs = category.configs.map(function (config) {
        if (self.isExcludedConfig(config)) {
          config.set('value', '');
        }
        return {
          name: config.name,
          filename: config.filename,
          value: config.value
        };
      });
      stepConfigs = stepConfigs.concat(configs);
    });
    if (stepConfigs.length > 0) {
      this.setDBProperty(stepController.name + "-sc", stepConfigs);
    }
  },

  loadCachedStepConfigValues: function loadCachedStepConfigValues(stepController) {
    return this.getDBProperty(stepController.name + "-sc");
  },

  clearCachedStepConfigValues: function clearCachedStepConfigValues(stepController) {
    this.setDBProperty(stepController.name + "-sc", null);
  },

  clearServiceConfigProperties: function clearServiceConfigProperties() {
    this.get('content.serviceConfigProperties', null);
    return this.setDBProperty('serviceConfigProperties', null);
  },

  saveTasksStatuses: function saveTasksStatuses(tasksStatuses) {
    this.set('content.tasksStatuses', tasksStatuses);
    this.setDBProperty('tasksStatuses', tasksStatuses);
  },

  loadTasksStatuses: function loadTasksStatuses() {
    var tasksStatuses = this.getDBProperty('tasksStatuses');
    this.set('content.tasksStatuses', tasksStatuses);
  },

  saveTasksRequestIds: function saveTasksRequestIds(tasksRequestIds) {
    this.set('content.tasksRequestIds', tasksRequestIds);
    this.setDBProperty('tasksRequestIds', tasksRequestIds);
  },

  loadTasksRequestIds: function loadTasksRequestIds() {
    var tasksRequestIds = this.getDBProperty('tasksRequestIds');
    this.set('content.tasksRequestIds', tasksRequestIds);
  },

  saveRequestIds: function saveRequestIds(requestIds) {
    this.set('content.requestIds', requestIds);
    this.setDBProperty('requestIds', requestIds);
  },

  loadRequestIds: function loadRequestIds() {
    var requestIds = this.getDBProperty('requestIds');
    this.set('content.requestIds', requestIds);
  },

  saveComponentsFromConfigs: function saveComponentsFromConfigs(componentsFromConfigs) {
    this.set('content.componentsFromConfigs', componentsFromConfigs);
    this.setDBProperty('componentsFromConfigs', componentsFromConfigs);
  },

  loadComponentsFromConfigs: function loadComponentsFromConfigs() {
    var componentsFromConfigs = this.getDBProperty('componentsFromConfigs');
    this.set('content.componentsFromConfigs', componentsFromConfigs);
  },

  loadRecommendations: function loadRecommendations() {
    this.set("content.recommendations", this.getDBProperty('recommendations'));
  },

  loadHdfsUserFromServer: function loadHdfsUserFromServer() {
    return App.get('router.configurationController').loadFromServer([{ 'siteName': 'hadoop-env' }]);
  },

  loadKerberosDescriptorConfigs: function loadKerberosDescriptorConfigs() {
    var kerberosDescriptorConfigs = this.getDBProperty('kerberosDescriptorConfigs');
    this.set('kerberosDescriptorConfigs', kerberosDescriptorConfigs);
  },

  saveKerberosDescriptorConfigs: function saveKerberosDescriptorConfigs(kerberosDescriptorConfigs) {
    this.setDBProperty('kerberosDescriptorConfigs', kerberosDescriptorConfigs);
    this.set('kerberosDescriptorConfigs', kerberosDescriptorConfigs);
  },
  /**
   * reset stored wizard data and reload App
   * @param {App.WizardController} controller - wizard controller
   * @param {string} route - preferable path to go after wizard finished
   */
  resetOnClose: function resetOnClose(controller, route, router) {
    App.router.get('wizardWatcherController').resetUser();
    controller.finish();
    App.clusterStatus.setClusterStatus({
      clusterName: App.get('clusterName'),
      clusterState: 'DEFAULT',
      localdb: App.db.data
    }, {
      alwaysCallback: function alwaysCallback() {
        router && router.set('nextBtnClickInProgress', false);
        controller.get('popup').hide();
        App.router.transitionTo(route);
        Em.run.next(function () {
          location.reload();
        });
      }
    });
  },

  /**
   * Replace placeholders for config properties which have dynamic names/values with actual data
   * @param value
   * @param dependencies
   * @returns {*}
     */
  replaceDependencies: function replaceDependencies(value, dependencies) {
    Em.keys(dependencies).forEach(function (key) {
      value = value.replace(new RegExp('{{' + key + '}}', 'g'), dependencies[key]);
    });
    return value;
  }
});

});

require.register("controllers/wizard/step0_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep0Controller = Em.Controller.extend({

  name: 'wizardStep0Controller',

  /**
   * Is step submitted
   * @type {bool}
   */
  hasSubmitted: false,

  /**
   * validate cluster name
   * set <code>clusterNameError</code> if validation fails
   */
  invalidClusterName: function () {
    var MAX_CLUSTER_NAME_LENGTH = 80;
    var clusterName = this.get('content.cluster.name');
    if (clusterName == '' && this.get('hasSubmitted')) {
      this.set('clusterNameError', Em.I18n.t('installer.step0.clusterName.error.required'));
      return true;
    } else if (clusterName == '') {
      return true;
    } else if (clusterName.length > MAX_CLUSTER_NAME_LENGTH) {
      this.set('clusterNameError', Em.I18n.t('installer.step0.clusterName.error.tooLong'));
      return true;
    } else if (/\s/.test(clusterName)) {
      this.set('clusterNameError', Em.I18n.t('installer.step0.clusterName.error.whitespace'));
      return true;
    } else if (/[^\w\s]/gi.test(clusterName)) {
      this.set('clusterNameError', Em.I18n.t('installer.step0.clusterName.error.specialChar'));
      return true;
    } else {
      this.set('clusterNameError', '');
      return false;
    }
  }.property('hasSubmitted', 'content.cluster.name'),

  /**
   * calculates by <code>invalidClusterName</code> property
   * todo: mix this and previous variables in one
   */
  clusterNameError: '',

  loadStep: function loadStep() {
    this.set('hasSubmitted', false);
    this.set('clusterNameError', '');
  },

  /**
   * @type {boolean}
   */
  isSubmitDisabled: Em.computed.or('invalidClusterName', 'App.router.btnClickInProgress'),

  /**
   * Onclick handler for <code>next</code> button
   * Disable 'Next' button while it is already under process. (using Router's property 'nextBtnClickInProgress')
   * @method submit
   */
  submit: function submit() {
    if (App.get('router.nextBtnClickInProgress')) {
      return;
    }
    this.set('hasSubmitted', true);
    if (!this.get('invalidClusterName')) {
      App.clusterStatus.set('clusterName', this.get('content.cluster.name'));
      this.set('content.cluster.status', 'PENDING');
      this.set('content.cluster.isCompleted', false);
      App.set('router.nextBtnClickInProgress', true);
      App.router.send('next');
    }
  }

});

});

require.register("controllers/wizard/step10_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep10Controller = Em.Controller.extend({

  /**
   * List of data about installed cluster components (hosts, services etc)
   * @type {Ember.Object[]}
   */
  clusterInfo: [],

  /**
   * is Add service wizard the ongoing wizard
   * @type {bool}
   */
  isAddServiceWizard: Em.computed.equal('content.controllerName', 'addServiceController'),

  /**
   * Clear <code>clusterInfo</code>
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.get('clusterInfo').clear();
  },

  /**
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.clearStep();
    this.loadInstalledHosts(this.loadRegisteredHosts());
    var installFlag = true;
    var startFlag = true;
    if (this.get('content.controllerName') == 'installerController') {
      installFlag = this.loadMasterComponents();
      startFlag = this.loadStartedServices();
    }
    if (installFlag && startFlag) {
      this.loadInstallTime();
    }
  },

  /**
   * Get list of clusterInfo object about registered hosts
   * @returns {{id: number, color: string, displayStatement: string, status: array}}
   * @method loadRegisteredHosts
   */
  loadRegisteredHosts: function loadRegisteredHosts() {
    var masterHosts = this.get('content.masterComponentHosts').mapProperty('hostName').uniq();
    var slaveHosts = this.get('content.slaveComponentHosts');
    var hostObj = [];
    slaveHosts.forEach(function (_hosts) {
      hostObj = hostObj.concat(_hosts.hosts);
    }, this);
    slaveHosts = hostObj.mapProperty('hostName').uniq();
    var registeredHosts = App.Host.find().mapProperty('hostName').concat(masterHosts.concat(slaveHosts)).uniq();
    var registerHostsStatement = Em.I18n.t('installer.step10.hostsSummary').format(registeredHosts.length);
    if (this.get('content.controllerName') === 'addHostController') {
      registerHostsStatement = Em.I18n.t('installer.step10.hostsSummary').format(Object.keys(this.get('content.hosts') || {}).length);
    }
    var registerHostsObj = Em.Object.create({
      id: 1,
      color: 'text-info',
      displayStatement: registerHostsStatement,
      status: []
    });
    this.get('clusterInfo').pushObject(registerHostsObj);

    return registerHostsObj;
  },

  /**
   * Push info about installed hosts to <code>clusterInfo</code>
   * @method loadInstalledHosts
   */
  loadInstalledHosts: function loadInstalledHosts() {
    var hosts = this.get('content.hosts');
    var hostsInfo = [];
    for (var index in hosts) {
      if (hosts.hasOwnProperty(index)) {
        hostsInfo.pushObject(hosts[index]);
      }
    }
    var succeededHosts = hostsInfo.filterProperty('status', 'success');
    var warnedHosts = hostsInfo.filter(function (host) {
      return ['warning', 'failed'].contains(host.status);
    });
    if (succeededHosts.length) {
      var successMessage = this.get('content.cluster.status') === 'START_SKIPPED' ? Em.I18n.t('installer.step10.installed') : Em.I18n.t('installer.step10.installedAndStarted');
      var successStatement = successMessage.format(succeededHosts.length) + (succeededHosts.length > 1 ? Em.I18n.t('installer.step8.hosts') : Em.I18n.t('installer.step8.host'));
      this.get('clusterInfo').findProperty('id', 1).get('status').pushObject(Em.Object.create({
        id: 1,
        color: 'text-success',
        displayStatement: successStatement
      }));
    }

    if (warnedHosts.length) {
      var warnStatement = warnedHosts.length + Em.I18n.t('installer.step10.warnings');
      this.get('clusterInfo').findProperty('id', 1).get('status').pushObject(Em.Object.create({
        id: 2,
        color: 'text-warning',
        displayStatement: warnStatement,
        statements: []
      }));
      warnedHosts.forEach(function (_host) {
        var clusterState = '';
        if (this.get('content.cluster.status') === 'INSTALL FAILED') {
          clusterState = Em.I18n.t('installer.step10.clusterState.installing');
        } else {
          if (this.get('content.cluster.status') === 'START FAILED') {
            clusterState = Em.I18n.t('installer.step10.clusterState.starting');
          }
        }

        var self = this;
        Em.A([{ Tst: 'FAILED', st: 'failed' }, { Tst: 'ABORTED', st: 'aborted' }, { Tst: 'TIMEDOUT', st: 'timedout' }]).forEach(function (s) {
          _host.tasks.filterProperty('Tasks.status', s.Tst).forEach(function (_task) {
            var statement = clusterState + App.format.role(_task.Tasks.role, false) + Em.I18n.t('installer.step10.taskStatus.failed') + _host.name;
            self.get('clusterInfo').findProperty('id', 1).get('status').findProperty('id', 2).get('statements').pushObject(Em.Object.create({
              status: s.st,
              color: 'text-info',
              displayStatement: statement
            }));
          });
        });
      }, this);
    }
  },

  /**
   * Push info about installed/failed master components to <code>clusterInfo</code>
   * @returns {bool}
   * @method loadMasterComponents
   */
  loadMasterComponents: function loadMasterComponents() {
    var components = this.get('content.masterComponentHosts');
    if (this.get('content.cluster.status') === 'INSTALL FAILED') {
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 2,
        displayStatement: Em.I18n.t('installer.step10.installStatus.failed'),
        color: 'text-danger',
        status: []
      }));
      return false;
    } else {
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 2,
        displayStatement: Em.I18n.t('installer.step10.installStatus.installed'),
        color: 'text-success',
        status: []
      }));
    }

    components.forEach(function (_component) {
      var component = Em.Object.create(_component);
      if (['NAMENODE', 'SECONDARY_NAMENODE', 'JOBTRACKER', 'HISTORYSERVER', 'RESOURCEMANAGER', 'HBASE_MASTER', 'HIVE_SERVER', 'OOZIE_SERVER', 'GANGLIA_SERVER'].contains(component.component)) {
        this.loadMasterComponent(component);
      }
    }, this);
    return true;
  },

  /**
   * Push component info to <code>clusterInfo</code>
   * @param {Ember.Object} component
   * @method loadMasterComponent
   */
  loadMasterComponent: function loadMasterComponent(component) {
    if (component.get('hostName')) {
      var statement = Em.I18n.t('installer.step10.master.installedOn').format(component.get('display_name'), component.get('hostName'));
      this.get('clusterInfo').findProperty('id', 2).get('status').pushObject(Em.Object.create({
        id: 1,
        color: 'text-info',
        displayStatement: statement
      }));
    }
  },

  /**
   * Push info about installed/started/failed services to <code>clusterInfo</code>
   * @returns {bool}
   * @method loadStartedServices
   */
  loadStartedServices: function loadStartedServices() {
    var status = this.get('content.cluster.status');
    if (status === 'STARTED') {
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 3,
        color: 'text-success',
        displayStatement: Em.I18n.t('installer.step10.startStatus.started'),
        status: []
      }));
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 4,
        color: 'text-success',
        displayStatement: Em.I18n.t('installer.step10.startStatus.passed'),
        status: []
      }));
      return true;
    } else if (status === 'START_SKIPPED') {
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 3,
        color: 'text-warning',
        displayStatement: Em.I18n.t('installer.step10.startStatus.skipped'),
        status: []
      }));
      return false;
    } else {
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 3,
        color: 'text-danger',
        displayStatement: Em.I18n.t('installer.step10.startStatus.failed'),
        status: []
      }));
      return false;
    }
  },

  /**
   * Push install time to <code>clusterInfo</code>
   * @method loadInstallTime
   */
  loadInstallTime: function loadInstallTime() {
    var statement, time;
    if (this.get('content.cluster.installTime')) {
      time = this.calculateInstallTime(this.get('content.cluster.installTime'));
      if (time.minutes !== 0) {
        statement = Em.I18n.t('installer.step10.installTime.minutes').format(time.minutes, time.seconds);
      } else {
        statement = Em.I18n.t('installer.step10.installTime.seconds').format(time.seconds);
      }
      this.get('clusterInfo').pushObject(Em.Object.create({
        id: 5,
        color: 'text-info',
        displayStatement: statement,
        status: []
      }));
    }
  },

  /**
   * Get used time for install process
   * @param {number} installTime
   * @returns {{minutes: *, seconds: *}}
   * @method calculateInstallTime
   */
  calculateInstallTime: function calculateInstallTime(installTime) {
    var secondsPerMinute = 60;
    var minutes = Math.floor(installTime);
    var seconds = Math.floor((installTime - minutes) * secondsPerMinute);
    return {
      minutes: minutes,
      seconds: seconds
    };
  }

});

});

require.register("controllers/wizard/step1_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var arrayUtils = require('utils/array_utils');

/**
 * @typedef {Em.Object} StackType
 * @property {string} stackName
 * @property {App.Stack[]} stacks
 * @property {boolean} isSelected
 */

/**
 * @type {Em.Object}
 */
var StackType = Em.Object.extend({
  stackName: '',
  stacks: [],
  isSelected: Em.computed.someBy('stacks', 'isSelected', true)
});

App.WizardStep1Controller = Em.Controller.extend({

  name: 'wizardStep1Controller',

  /**
   * Skip repo-validation
   *
   * @type {bool}
   */
  skipValidationChecked: false,

  /**
   * @type {App.Stack}
   */
  selectedStack: Em.computed.findBy('content.stacks', 'isSelected', true),

  /**
   * @type {App.ServiceSimple[]}
   */
  servicesForSelectedStack: Em.computed.filterBy('selectedStack.stackServices', 'isHidden', false),

  /**
   * Some network issues exist if there is no stack with <code>stackDefault</code> = false
   *
   * @type {boolean}
   */
  networkIssuesExist: function () {
    if (this.get('content.stacks') && this.get('content.stacks.length') > 1) {
      return this.get('content.stacks').everyProperty('stackDefault', true);
    }
    return false;
  }.property('content.stacks.@each.stackDefault'),

  /**
   * No stacks have repo update URL section (aka "latest") defined in repoinfo.xml
   *
   * @type {boolean}
   */
  stackRepoUpdateLinkExists: Em.computed.someBy('content.stacks', 'stackRepoUpdateLinkExists', true),

  optionsToSelect: {
    'usePublicRepo': {
      index: 0,
      isSelected: true
    },
    'useLocalRepo': {
      index: 1,
      isSelected: false,
      'uploadFile': {
        index: 0,
        name: 'uploadFile',
        file: '',
        hasError: false,
        isSelected: true
      },
      'enterUrl': {
        index: 1,
        name: 'enterUrl',
        url: '',
        placeholder: Em.I18n.t('installer.step1.useLocalRepo.enterUrl.placeholder'),
        hasError: false,
        isSelected: false
      }
    }
  },

  /**
   * Checks if user selected to input url or upload file but didn't do it
   * true  - url-radio is checked but url-field is empty
   *       - file-radio is checked but file is not selected
   * false - otherwise
   *
   * @type {boolean}
   */
  readInfoIsNotProvided: function () {
    var useLocalRepo = this.get('optionsToSelect.useLocalRepo');
    if (Em.get(useLocalRepo, 'uploadFile.isSelected')) {
      return !Em.get(useLocalRepo, 'uploadFile.file');
    }
    if (Em.get(useLocalRepo, 'enterUrl.isSelected')) {
      return !Em.get(useLocalRepo, 'enterUrl.url');
    }
    return false;
  }.property('optionsToSelect.useLocalRepo.isSelected', 'optionsToSelect.useLocalRepo.uploadFile.isSelected', 'optionsToSelect.useLocalRepo.uploadFile.file', 'optionsToSelect.useLocalRepo.enterUrl.url'),

  /**
   * List of stacks grouped by <code>stackNameVersion</code>
   *
   * @type {StackType[]}
   */
  availableStackTypes: function () {
    var stacks = this.get('content.stacks');
    return stacks ? stacks.mapProperty('stackNameVersion').uniq().sort().reverse().map(function (stackName) {
      return StackType.create({
        stackName: stackName,
        stacks: stacks.filterProperty('stackNameVersion', stackName).sort(arrayUtils.sortByIdAsVersion).reverse()
      });
    }) : [];
  }.property('content.stacks.@each.stackNameVersion'),

  /**
   * @type {StackType}
   */
  selectedStackType: Em.computed.findBy('availableStackTypes', 'isSelected', true),

  isLoadingComplete: Em.computed.equal('wizardController.loadStacksRequestsCounter', 0),

  /**
   * Load selected file to current page content
   */
  readVersionInfo: function readVersionInfo() {
    var data = {};
    var isXMLdata = false;
    if (this.get("optionsToSelect.useLocalRepo.enterUrl.isSelected")) {
      var url = this.get("optionsToSelect.useLocalRepo.enterUrl.url");
      data = {
        VersionDefinition: {
          version_url: url
        }
      };
      App.db.setLocalRepoVDFData(url);
    } else {
      if (this.get("optionsToSelect.useLocalRepo.uploadFile.isSelected")) {
        isXMLdata = true;
        // load from file browser
        data = this.get("optionsToSelect.useLocalRepo.uploadFile.file");
        App.db.setLocalRepoVDFData(data);
      }
    }
    return App.router.get('installerController').postVersionDefinitionFile(isXMLdata, data);
  },

  /**
   * On click handler for removing OS
   */
  removeOS: function removeOS(event) {
    if (this.get('selectedStack.useRedhatSatellite')) {
      return;
    }
    var osToRemove = event.context;
    Em.set(osToRemove, 'isSelected', false);
  },

  /**
   * On click handler for adding new OS
   */
  addOS: function addOS(event) {
    var osToAdd = event.context;
    Em.set(osToAdd, 'isSelected', true);
  },

  /**
   * Use Local Repo if some network issues exist
   */
  onNetworkIssuesExist: function () {
    if (this.get('networkIssuesExist')) {
      this.get('content.stacks').forEach(function (stack) {
        if (stack.get('useLocalRepo') !== true) {
          stack.setProperties({
            usePublicRepo: false,
            useLocalRepo: true
          });
          stack.cleanReposBaseUrls();
        }
      });
    }
  }.observes('networkIssuesExist'),

  /**
   * Select stack with field equal to the value
   * Example:
   * <pre>
   *   selectStackBy('id', 'HDP-2.5-2.5.0.0'); // select stack with id = 'HDP-2.5-2.5.0.0'
   *   selectStackBy('stackNameVersion', 'HDP-2.5'); // select first stack with stackNameVersion = 'HDP-2.5'
   * </pre>
   *
   * @param {string} field
   * @param {string} value
   */
  selectStackBy: function selectStackBy(field, value) {
    this.get('content.stacks').setEach('isSelected', false);
    this.get('content.stacks').findProperty(field, value).set('isSelected', true);
  },

  /**
   * Restore base urls for selected stack when user select to use public repository
   */
  usePublicRepo: function usePublicRepo() {
    var selectedStack = this.get('selectedStack');
    if (selectedStack) {
      selectedStack.setProperties({
        useRedhatSatellite: false,
        usePublicRepo: true,
        useLocalRepo: false
      });
      selectedStack.restoreReposBaseUrls();
    }
  },

  /**
   * Clean base urls for selected stack when user select to use local repository
   */
  useLocalRepo: function useLocalRepo() {
    var selectedStack = this.get('selectedStack');
    if (selectedStack) {
      selectedStack.setProperties({
        usePublicRepo: false,
        useLocalRepo: true
      });
      selectedStack.cleanReposBaseUrls();
    }
  },

  /**
   * Restores url value to be its default value.
   * @method doRestoreDefaultValue
   */
  doRestoreDefaultValue: function doRestoreDefaultValue(event) {
    var repo = event.contexts[0];
    repo.set('baseUrl', repo.get('baseUrlInit'));
  },

  /**
   * Restores url value to empty string.
   * @method doRestoreToEmpty
   */
  doRestoreToEmpty: function doRestoreToEmpty(event) {
    var repo = event.contexts[0];
    repo.set('baseUrl', '');
  },

  /**
   * Click-handler for left-tabs with stack types
   * Select first available stack with stackName equal to chosen
   *
   * @param {{context: StackType}} event
   */
  selectRepoInList: function selectRepoInList(event) {
    var id = this.get('availableStackTypes').findProperty('stackName', event.context.stackName).get('stacks.firstObject.id');
    this.selectStackBy('id', id);
  },

  /**
   * Click-handler for StackVersion-tabs
   *
   * @param {{context: App.Stack}} event
   */
  changeVersion: function changeVersion(event) {
    this.selectStackBy('id', event.context.get('id'));
  },

  /**
   * Show popup with options to upload new version
   *
   * @returns {App.ModalPopup}
   */
  uploadVdf: function uploadVdf() {
    return App.ModalPopup.show({

      controller: this,

      header: Em.I18n.t('installer.step1.addVersion.title'),

      primary: Em.I18n.t('installer.step1.useLocalRepo.readButton'),

      disablePrimary: Em.computed.alias('controller.readInfoIsNotProvided'),

      'data-qa': 'vdf-modal',

      /**
       * Try to read version info from the url or file (if provided)
       */
      onPrimary: function onPrimary() {
        var controller = this.get('controller');
        controller.readVersionInfo().done(function (response) {
          // load successfully, so make this local stack repo as selectedStack
          var newStackId = response.stackNameVersion + '-' + response.actualVersion;
          var oldStackNameVersion = controller.get('selectedStack.stackNameVersion');
          controller.selectStackBy('id', newStackId);
          if (oldStackNameVersion && oldStackNameVersion !== response.stackNameVersion) {
            App.showAlertPopup(Em.I18n.t('common.warning'), Em.I18n.t('installer.step1.addVersion.stackChanged.popup.body').format(oldStackNameVersion, response.stackNameVersion));
          }
          Ember.run.next(function () {
            App.tooltip($("[rel=skip-validation-tooltip]"), { html: true, placement: 'left' });
            $("[rel=use-redhat-tooltip]").tooltip({ placement: 'right' });
          });
        });
        this.restoreUploadOptions();
        this._super();
      },

      /**
       * Disable url/file fields on popup-close
       */
      onSecondary: function onSecondary() {
        this.restoreUploadOptions();
        this._super();
      },

      /**
       * Disable url/file fields on popup-close
       */
      onClose: function onClose() {
        this.restoreUploadOptions();
        this._super();
      },

      /**
       * Deselect file/url radio
       */
      restoreUploadOptions: function restoreUploadOptions() {
        this.set('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected', false);
        this.set('controller.optionsToSelect.useLocalRepo.enterUrl.url', '');
        this.set('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected', true);
        this.set('controller.optionsToSelect.useLocalRepo.uploadFile.file', '');
      },

      bodyClass: Em.View.extend({

        controller: this,

        templateName: require('templates/wizard/step1/vdf_upload'),

        /**
         * Wrapper for 'upload-file' elements
         *
         * @type {Em.View}
         */
        uploadFileView: Em.View.extend({

          classNames: ['clearfix'],

          /**
           * Radio button for Use local Repo > Upload VDF file
           *
           * @type {App.RadioButtonView}
           */
          uploadFileRadioButton: App.RadioButtonView.extend({
            labelTranslate: 'installer.step1.useLocalRepo.uploadFile',
            checked: Em.computed.alias('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected')
          }),

          /**
           * Is File API available
           *
           * @type {bool}
           */
          isFileApi: window.File && window.FileReader && window.FileList,

          /**
           * Upload file is disabled when some stack is selected or url-field is selected
           *
           * @type {boolean}
           */
          fileBrowserDisabled: Em.computed.alias('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected'),

          /**
           * Input to select vdf-file
           *
           * @type {Em.View}
           */
          fileInputView: Em.View.extend({
            template: Em.Handlebars.compile('<input type="file" {{bindAttr class="controller.optionsToSelect.useLocalRepo.enterUrl.isSelected:disabled"}} {{QAAttr "vdf-input"}}/>'),

            change: function change(e) {
              var self = this;
              if (e.target.files && e.target.files.length === 1) {
                var file = e.target.files[0];
                var reader = new FileReader();

                reader.onload = function () {
                  return function (event) {
                    self.set('controller.optionsToSelect.useLocalRepo.uploadFile.file', event.target.result);
                  };
                }(file);
                reader.readAsText(file);
              }
            }

          }),

          click: function click() {
            if (!this.set('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected')) {
              this.set('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected', false);
              this.set('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected', true);
              this.set('controller.optionsToSelect.useLocalRepo.enterUrl.hasError', false);
              this.set('controller.optionsToSelect.useLocalRepo.uploadFile.hasError', false);
            }
          }
        }),

        /**
         * Wrapper for 'enter-url' elements
         *
         * @type {Em.View}
         */
        enterUrlView: Em.View.extend({

          /**
           * Url-field is disable when some stack is selected or upload file is selected
           *
           * @type {boolean}
           */
          enterUrlFieldDisabled: Em.computed.alias('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected'),

          /**
           * Input for file upload
           *
           * @type {Em.TextField}
           */
          enterUrlField: Em.TextField.extend({
            classNameBindings: [':input-block-level', 'controller.optionsToSelect.useLocalRepo.uploadFile.isSelected:disabled']
          }),

          /**
           * Radio button for Use local Repo > Enter Url of VDF file
           *
           * @type {App.RadioButtonView}
           */
          enterUrlRadioButton: App.RadioButtonView.extend({
            labelTranslate: 'installer.step1.useLocalRepo.enterUrl',
            checked: Em.computed.alias('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected')
          }),

          click: function click() {
            if (!this.set('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected')) {
              this.set('controller.optionsToSelect.useLocalRepo.enterUrl.isSelected', true);
              this.set('controller.optionsToSelect.useLocalRepo.uploadFile.isSelected', false);
              this.set('controller.optionsToSelect.useLocalRepo.enterUrl.hasError', false);
              this.set('controller.optionsToSelect.useLocalRepo.uploadFile.hasError', false);
            }
          }
        })

      })
    });
  }

});

});

require.register("controllers/wizard/step2_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');
var lazyloading = require('utils/lazy_loading');

App.WizardStep2Controller = Em.Controller.extend({

  name: 'wizardStep2Controller',

  /**
   * List of not installed hostnames
   * @type {string[]}
   */
  hostNameArr: [],

  /**
   * Does pattern-expression for hostnames contains some errors
   * @type {bool}
   */
  isPattern: false,

  /**
   * Don't know if it used any more
   */
  bootRequestId: null,

  /**
   * Is step submitted
   * @type {bool}
   */
  hasSubmitted: false,

  /**
   * @type {string[]}
   */
  inputtedAgainHostNames: [],

  /**
   * Is Installer Controller used
   * @type {bool}
   */
  isInstaller: Em.computed.equal('content.controllerName', 'installerController'),

  /**
   * "Shortcut" to <code>content.installOptions.hostNames</code>
   * @type {string}
   */
  hostNames: function () {
    return this.get('content.installOptions.hostNames').toLowerCase();
  }.property('content.installOptions.hostNames'),

  /**
   * Is manual install selected
   * "Shortcut" to <code>content.installOptions.manualInstall</code>
   * @type {bool}
   */
  manualInstall: Em.computed.alias('content.installOptions.manualInstall'),

  /**
   * "Shortcut" to <code>content.installOptions.sshKey</code>
   * @type {string}
   */
  sshKey: Em.computed.alias('content.installOptions.sshKey'),

  /**
   * "Shortcut" to <code>content.installOptions.sshUser</code>
   * @type {string}
   */
  sshUser: Em.computed.alias('content.installOptions.sshUser'),

  /**
   * "Shortcut" to <code>content.installOptions.sshPort</code>
   * @type {string}
   */
  sshPort: Em.computed.alias('content.installOptions.sshPort'),

  /**
   * "Shortcut" to <code>content.installOptions.agentUser</code>
   * @type {string}
   */
  agentUser: Em.computed.alias('content.installOptions.agentUser'),

  /**
   * Installed type based on <code>manualInstall</code>
   * @type {string}
   */
  installType: Em.computed.ifThenElse('manualInstall', 'manualDriven', 'ambariDriven'),

  /**
   * List of invalid hostnames
   * @type {string[]}
   */
  invalidHostNames: [],

  /**
   * Error-message if <code>hostNames</code> is empty, null otherwise
   * @type {string|null}
   */
  hostsError: null,

  useSSH: function () {
    return !App.get('isHadoopWindowsStack');
  }.property('App.isHadoopWindowsStack'),

  /**
   * Error-message if <code>sshKey</code> is empty, null otherwise
   * @type {string|null}
   */
  sshKeyError: function () {
    if (this.get('hasSubmitted') && this.get('manualInstall') === false && this.get('useSSH') && Em.isBlank(this.get('sshKey'))) {
      return Em.I18n.t('installer.step2.sshKey.error.required');
    }
    return null;
  }.property('sshKey', 'useSSH', 'manualInstall', 'hasSubmitted'),

  /**
   * Error-message if <code>sshUser</code> is empty, null otherwise
   * @type {string|null}
   */
  sshUserError: function () {
    if (this.get('manualInstall') === false && this.get('useSSH') && Em.isBlank(this.get('sshUser'))) {
      return Em.I18n.t('installer.step2.sshUser.required');
    }
    return null;
  }.property('sshUser', 'useSSH', 'hasSubmitted', 'manualInstall'),

  /**
   * Error-message if <code>sshPort</code> is empty, null otherwise
   * @type {string|null}
   */
  sshPortError: function () {
    if (this.get('manualInstall') === false && this.get('useSSH') && Em.isBlank(this.get('sshPort'))) {
      return Em.I18n.t('installer.step2.sshPort.required');
    }
    return null;
  }.property('sshPort', 'useSSH', 'hasSubmitted', 'manualInstall'),

  /**
   * Error-message if <code>agentUser</code> is empty, null otherwise
   * @type {string|null}
   */
  agentUserError: function () {
    if (App.get('supports.customizeAgentUserAccount') && this.get('manualInstall') === false && Em.isBlank(this.get('agentUser'))) {
      return Em.I18n.t('installer.step2.sshUser.required');
    }
    return null;
  }.property('agentUser', 'hasSubmitted', 'manualInstall'),

  /**
   * is Submit button disabled
   * @type {bool}
   */
  isSubmitDisabled: Em.computed.or('hostsError', 'sshKeyError', 'sshUserError', 'sshPortError', 'agentUserError', 'App.router.btnClickInProgress'),

  installedHostNames: function () {
    var installedHostsName = [];
    var hosts = this.get('content.hosts');

    for (var hostName in hosts) {
      if (hosts[hostName].isInstalled) {
        installedHostsName.push(hostName);
      }
    }
    return installedHostsName;
  }.property('content.hosts'),

  /**
   * Set not installed hosts to the hostNameArr
   * @method updateHostNameArr
   */
  updateHostNameArr: function updateHostNameArr() {
    this.set('hostNameArr', this.get('hostNames').trim().split(new RegExp("\\s+", "g")));
    this.parseHostNamesAsPatternExpression();
    this.get('inputtedAgainHostNames').clear();
    var tempArr = [],
        hostNameArr = this.get('hostNameArr');
    for (var i = 0; i < hostNameArr.length; i++) {
      if (!this.get('installedHostNames').contains(hostNameArr[i])) {
        tempArr.push(hostNameArr[i]);
      } else {
        this.get('inputtedAgainHostNames').push(hostNameArr[i]);
      }
    }
    this.set('hostNameArr', tempArr);
  },

  /**
   * Validate host names
   * @method isAllHostNamesValid
   * @return {bool}
   */
  isAllHostNamesValid: function isAllHostNamesValid() {
    var result = true;
    this.updateHostNameArr();
    this.get('invalidHostNames').clear();
    this.get('hostNameArr').forEach(function (hostName) {
      if (!validator.isHostname(hostName)) {
        this.get('invalidHostNames').push(encodeURIComponent(hostName));
        result = false;
      }
    }, this);

    return result;
  },

  /**
   * Set hostsError if host names don't pass validation
   * @method checkHostError
   */
  checkHostError: function checkHostError() {
    if (Em.isEmpty(this.get('hostNames').trim())) {
      this.set('hostsError', Em.I18n.t('installer.step2.hostName.error.required'));
    } else {
      this.set('hostsError', null);
    }
  },

  /**
   * Check hostnames after Submit was clicked or <code>hostNames</code> were changed
   * @method checkHostAfterSubmitHandler
   */
  checkHostAfterSubmitHandler: function () {
    if (this.get('hasSubmitted')) {
      this.checkHostError();
    }
  }.observes('hasSubmitted', 'hostNames'),

  /**
   * Get host info, which will be saved in parent controller
   * @method getHostInfo
   */
  getHostInfo: function getHostInfo() {

    var hostNameArr = this.get('hostNameArr');
    var hostInfo = {};
    for (var i = 0; i < hostNameArr.length; i++) {
      hostInfo[hostNameArr[i]] = {
        name: hostNameArr[i],
        installType: this.get('installType'),
        bootStatus: 'PENDING',
        isInstalled: false
      };
    }

    return hostInfo;
  },

  /**
   * Used to set sshKey from FileUploader
   * @method setSshKey
   * @param {string} sshKey
   */
  setSshKey: function setSshKey(sshKey) {
    this.set("content.installOptions.sshKey", sshKey);
  },

  /**
   * Onclick handler for <code>next button</code>. Do all UI work except data saving.
   * This work is doing by router.
   * @method evaluateStep
   * @return {bool}
   */
  evaluateStep: function evaluateStep() {

    if (this.get('isSubmitDisabled')) {
      return false;
    }

    this.set('hasSubmitted', true);

    this.checkHostError();
    this.updateHostNameArr();

    if (!this.get('hostNameArr.length')) {
      this.set('hostsError', Em.I18n.t('installer.step2.hostName.error.already_installed'));
    }

    if (this.get('hostsError') || this.get('sshUserError') || this.get('sshPortError') || this.get('agentUserError') || this.get('sshKeyError')) {
      return false;
    }

    if (this.get('isPattern')) {
      this.hostNamePatternPopup(this.get('hostNameArr'));
      return false;
    }
    if (this.get('inputtedAgainHostNames.length')) {
      this.installedHostsPopup();
    } else {
      this.proceedNext();
    }
    return true;
  },

  /**
   * check is there a pattern expression in host name textarea
   * push hosts that match pattern in hostNamesArr
   * @method parseHostNamesAsPatternExpression
   */
  parseHostNamesAsPatternExpression: function parseHostNamesAsPatternExpression() {
    this.set('isPattern', false);
    var hostNames = [];

    this.get('hostNameArr').forEach(function (a) {
      var hn,
          allPatterns = a.match(/\[\d*\-\d*\]/g),
          patternsNumber = allPatterns ? allPatterns.length : 0;

      if (patternsNumber) {
        hn = [a];
        for (var i = 0; i < patternsNumber; i++) {
          hn = this._replacePatternInHosts(hn);
        }
        hostNames = hostNames.concat(hn);
      } else {
        hostNames.push(a);
      }
    }, this);

    this.set('hostNameArr', hostNames.uniq());
  },

  /**
   * return an array of results with pattern replacement for each host
   * replace only first pattern in each host
   * designed to be called recursively in <code>parseHostNamesAsPatternExpression</code>
   *
   * @param {Array} rawHostNames
   * @private
   * @return {Array}
   */
  _replacePatternInHosts: function _replacePatternInHosts(rawHostNames) {
    var start,
        end,
        extra,
        allHostNames = [];
    rawHostNames.forEach(function (rawHostName) {
      var hostNames = [];
      start = rawHostName.match(/\[\d*/);
      end = rawHostName.match(/\-\d*]/);
      extra = { 0: "" };

      start = start[0].substr(1);
      end = end[0].substr(1);

      if (parseInt(start) <= parseInt(end, 10) && parseInt(start, 10) >= 0) {
        this.set('isPattern', true);

        if (start[0] == "0" && start.length > 1) {
          extra = start.match(/0*/);
        }

        for (var i = parseInt(start, 10); i < parseInt(end, 10) + 1; i++) {
          hostNames.push(rawHostName.replace(/\[\d*\-\d*\]/, extra[0].substring(0, start.length - i.toString().length) + i));
        }
      } else {
        hostNames.push(rawHostName);
      }
      allHostNames = allHostNames.concat(hostNames);
    }, this);

    return allHostNames;
  },

  /**
   * launch hosts to bootstrap
   * and save already registered hosts
   * @method proceedNext
   * @return {bool}
   */
  proceedNext: function proceedNext(warningConfirmed) {
    if (this.isAllHostNamesValid() !== true && !warningConfirmed) {
      this.warningPopup();
      return false;
    }

    if (this.get('manualInstall') === true) {
      this.manualInstallPopup();
      return false;
    }

    this.saveHosts();
    App.router.send('next');
    return true;
  },

  /**
   * show warning for host names without dots or IP addresses
   * @return {App.ModalPopup}
   * @method warningPopup
   */
  warningPopup: function warningPopup() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      primary: Em.I18n.t('common.continue'),
      primaryClass: 'btn-warning',
      onPrimary: function onPrimary() {
        this.hide();
        self.proceedNext(true);
      },
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile(Em.I18n.t('installer.step2.warning.popup.body').format(self.get('invalidHostNames').join(', ')))
      })
    });
  },

  /**
   * show popup with the list of hosts that are already part of the cluster
   * @return {App.ModalPopup}
   * @method installedHostsPopup
   */
  installedHostsPopup: function installedHostsPopup() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      onPrimary: function onPrimary() {
        self.proceedNext();
        this.hide();
      },
      bodyClass: Em.View.extend({
        inputtedAgainHostNames: function () {
          return self.get('inputtedAgainHostNames').join(', ');
        }.property(),
        templateName: require('templates/wizard/step2_installed_hosts_popup')
      })
    });
  },

  /**
   * Show popup with hosts generated by pattern
   * @method hostNamePatternPopup
   * @param {string[]} hostNames
   * @return {App.ModalPopup}
   */
  hostNamePatternPopup: function hostNamePatternPopup(hostNames) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step2.hostName.pattern.header'),
      onPrimary: function onPrimary() {
        self.proceedNext();
        this.hide();
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/common/items_list_popup'),
        items: hostNames,
        insertedItems: [],
        didInsertElement: function didInsertElement() {
          lazyloading.run({
            destination: this.get('insertedItems'),
            source: this.get('items'),
            context: this,
            initSize: 100,
            chunkSize: 500,
            delay: 100
          });
        }
      })
    });
  },

  /**
   * Show notify that installation is manual
   * save hosts
   * @return {App.ModalPopup}
   * @method manualInstallPopup
   */
  manualInstallPopup: function manualInstallPopup() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step2.manualInstall.popup.header'),
      onPrimary: function onPrimary() {
        this.hide();
        self.saveHosts();
        App.router.send('next');
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/wizard/step2ManualInstallPopup')
      })
    });
  },

  /**
   * Warn to manually install ambari-agent on each host
   * @method manualInstallWarningPopup
   */
  manualInstallWarningPopup: function () {
    if (!this.get('content.installOptions.useSsh')) {
      App.ModalPopup.show({
        header: Em.I18n.t('common.warning'),
        body: Em.I18n.t('installer.step2.manualInstall.info'),
        encodeBody: false,
        secondary: null
      });
    }
    this.set('content.installOptions.manualInstall', !this.get('content.installOptions.useSsh'));
  }.observes('content.installOptions.useSsh'),

  /**
   * Load java.home value frin server
   * @method setAmbariJavaHome
   */
  setAmbariJavaHome: function setAmbariJavaHome() {
    App.ajax.send({
      name: 'ambari.service',
      sender: this,
      success: 'onGetAmbariJavaHomeSuccess',
      error: 'onGetAmbariJavaHomeError'
    });
  },

  /**
   * Set received java.home value
   * @method onGetAmbariJavaHomeSuccess
   * @param {Object} data
   */
  onGetAmbariJavaHomeSuccess: function onGetAmbariJavaHomeSuccess(data) {
    this.set('content.installOptions.javaHome', data.RootServiceComponents.properties['java.home']);
  },

  /**
   * Set default java.home value
   * @method onGetAmbariJavaHomeError
   */
  onGetAmbariJavaHomeError: function onGetAmbariJavaHomeError() {
    this.set('content.installOptions.javaHome', App.get('defaultJavaHome'));
  },

  /**
   * Save hosts info and proceed to the next step
   * @method saveHosts
   */
  saveHosts: function saveHosts() {
    var hosts = this.get('content.hosts');

    //add previously installed hosts
    for (var hostName in hosts) {
      if (!hosts[hostName].isInstalled) {
        delete hosts[hostName];
      }
    }

    this.set('content.hosts', $.extend(hosts, this.getHostInfo()));
    this.setAmbariJavaHome();
  },

  isAddHostWizard: Em.computed.equal('content.controllerName', 'addHostController')

});

});

require.register("controllers/wizard/step3_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var lazyloading = require('utils/lazy_loading');
var numberUtils = require('utils/number_utils');

App.WizardStep3Controller = Em.Controller.extend(App.ReloadPopupMixin, App.CheckHostMixin, {

  name: 'wizardStep3Controller',

  hosts: [],

  content: [],

  registeredHosts: [],

  jdkRequestIndex: null,

  registrationStartedAt: null,

  /**
   * Timeout for registration
   * Based on <code>installOptions.manualInstall</code>
   * @type {number}
   */
  registrationTimeoutSecs: Em.computed.ifThenElse('content.installOptions.manualInstall', 15, 120),

  /**
   * Bootstrap calls are stopped
   * @type {bool}
   */
  stopBootstrap: false,

  /**
   * is Submit button disabled
   * @type {bool}
   */
  isSubmitDisabled: true,

  /**
   * True if bootstrap POST request failed
   * @type {bool}
   */
  isBootstrapFailed: false,

  /**
   * is Retry button disabled
   * @type {bool}
   */
  isRetryDisabled: function () {
    return this.get('isBackDisabled') ? this.get('isBackDisabled') : !this.get('bootHosts').filterProperty('bootStatus', 'FAILED').length;
  }.property('bootHosts.@each.bootStatus', 'isBackDisabled'),

  /**
   * Is Back button disabled
   * @return {bool}
   */
  isBackDisabled: function () {
    return (this.get('isRegistrationInProgress') || !this.get('isWarningsLoaded')) && !this.get('isBootstrapFailed') || App.get('router.btnClickInProgress');
  }.property('isRegistrationInProgress', 'isWarningsLoaded', 'isBootstrapFailed'),

  /**
   * Controller is using in Add Host Wizard
   * @return {bool}
   */
  isAddHostWizard: Em.computed.equal('content.controllerName', 'addHostController'),

  /**
   * @type {bool}
   */
  isLoaded: false,

  /**
   * Polls count
   * @type {number}
   */
  numPolls: 0,

  /**
   * Is hosts registration in progress
   * @type {bool}
   */
  isRegistrationInProgress: true,

  /**
   * Are some registered hosts which are not added by user
   * @type {bool}
   */
  hasMoreRegisteredHosts: false,

  /**
   * List of installed hostnames
   * @type {string[]}
   */
  hostsInCluster: function () {
    var installedHostsName = [];
    var hosts = this.get('content.hosts');

    for (var hostName in hosts) {
      if (hosts[hostName].isInstalled) {
        installedHostsName.push(hostName);
      }
    }
    return installedHostsName;
  }.property('content.hosts'),

  /**
   * Are hosts warnings loaded
   * @type {bool}
   */
  isWarningsLoaded: Em.computed.and('isJDKWarningsLoaded', 'isHostsWarningsLoaded'),

  /**
   * Check are hosts have any warnings
   * @type {bool}
   */
  isHostHaveWarnings: Em.computed.gt('warnings.length', 0),

  /**
   * Should warnings-box be visible
   * @type {bool}
   */
  isWarningsBoxVisible: function () {
    return App.get('testMode') ? true : !this.get('isRegistrationInProgress');
  }.property('isRegistrationInProgress'),

  isNextButtonDisabled: Em.computed.or('App.router.btnClickInProgress', 'isSubmitDisabled'),

  isBackButtonDisabled: Em.computed.or('App.router.btnClickInProgress', 'isBackDisabled'),

  /**
   *
   * @method navigateStep
   */
  navigateStep: function () {
    if (this.get('isLoaded')) {
      if (!this.get('content.installOptions.manualInstall')) {
        if (!this.get('wizardController').getDBProperty('bootStatus')) {
          this.setupBootStrap();
        }
      } else {
        this.set('bootHosts', this.get('hosts'));
        if (App.get('testMode')) {
          this.startHostcheck(this.get('hosts'));
          this.get('bootHosts').setEach('cpu', '2');
          this.get('bootHosts').setEach('memory', '2000000');
          this.set('isSubmitDisabled', false);
        } else {
          this.set('registrationStartedAt', null);
          this.startRegistration();
        }
      }
    }
  }.observes('isLoaded'),

  /**
   * Clear controller data
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.set('stopBootstrap', false);
    this.set('hosts', []);
    this.get('bootHosts').clear();
    this.get('wizardController').setDBProperty('bootStatus', false);
    this.set('isHostsWarningsLoaded', false);
    this.set('isJDKWarningsLoaded', false);
    this.set('registrationStartedAt', null);
    this.set('isLoaded', false);
    this.set('isSubmitDisabled', true);
    this.set('stopChecking', false);
  },

  /**
   * setup bootstrap data and completion callback for bootstrap call
   * @method setupBootStrap
   */
  setupBootStrap: function setupBootStrap() {
    var self = this;
    var bootStrapData = JSON.stringify({
      'verbose': true,
      'sshKey': this.get('content.installOptions.sshKey'),
      'hosts': this.getBootstrapHosts(),
      'user': this.get('content.installOptions.sshUser'),
      'sshPort': this.get('content.installOptions.sshPort'),
      'userRunAs': App.get('supports.customizeAgentUserAccount') ? this.get('content.installOptions.agentUser') : 'root'
    });
    App.router.get(this.get('content.controllerName')).launchBootstrap(bootStrapData, function (requestId) {
      if (requestId == '0') {
        self.startBootstrap();
      } else if (requestId) {
        self.set('content.installOptions.bootRequestId', requestId);
        App.router.get(self.get('content.controllerName')).save('installOptions');
        self.startBootstrap();
      }
    });
  },

  getBootstrapHosts: function getBootstrapHosts() {
    var hosts = this.get('content.hosts');
    var bootstrapHosts = [];
    for (var host in hosts) {
      if (hosts.hasOwnProperty(host)) {
        if (!hosts[host].isInstalled) {
          bootstrapHosts.push(host);
        }
      }
    }

    return bootstrapHosts;
  },

  /**
   * Make basic init steps
   * @method loadStep
   */
  loadStep: function loadStep() {
    var wizardController = this.get('wizardController');
    var previousStep = wizardController && wizardController.get('previousStep');
    var currentStep = wizardController && wizardController.get('currentStep');
    var isHostsLoaded = this.get('hosts').length !== 0;
    var isPrevAndCurrStepsSetted = previousStep && currentStep;
    var isPrevStepSmallerThenCurrent = previousStep < currentStep;
    if (!isHostsLoaded || isPrevStepSmallerThenCurrent || !wizardController || !isPrevAndCurrStepsSetted) {
      this.disablePreviousSteps();
      this.clearStep();
      App.router.get('clusterController').loadAmbariProperties();
      this.loadHosts();
    }
  },

  /**
   * Loads the hostinfo from localStorage on the insertion of view. It's being called from view
   * @method loadHosts
   */
  loadHosts: function loadHosts() {
    var hostsInfo = this.get('content.hosts');
    var hosts = [];
    var bootStatus = this.get('content.installOptions.manualInstall') ? 'DONE' : 'PENDING';
    if (App.get('testMode')) {
      bootStatus = 'REGISTERED';
    }

    for (var index in hostsInfo) {
      if (hostsInfo.hasOwnProperty(index) && !hostsInfo[index].isInstalled) {
        hosts.pushObject(App.HostInfo.create({
          name: hostsInfo[index].name,
          bootStatus: bootStatus,
          isChecked: false
        }));
      }
    }
    this.set('hosts', hosts);
    this.set('isLoaded', true);
  },

  /**
   * Parses and updates the content based on bootstrap API response.
   * @return {bool} true if polling should continue (some hosts are in "RUNNING" state); false otherwise
   * @method parseHostInfo
   */
  parseHostInfo: function parseHostInfo(hostsStatusFromServer) {
    hostsStatusFromServer.forEach(function (_hostStatus) {
      var host = this.get('bootHosts').findProperty('name', _hostStatus.hostName);
      // check if hostname extracted from REST API data matches any hostname in content
      // also, make sure that bootStatus modified by isHostsRegistered call does not get overwritten
      // since these calls are being made in parallel
      if (host && !['REGISTERED', 'REGISTERING'].contains(host.get('bootStatus'))) {
        host.set('bootStatus', _hostStatus.status);
        host.set('bootLog', _hostStatus.log);
      }
    }, this);
    // if the data rendered by REST API has hosts in "RUNNING" state, polling will continue
    return this.get('bootHosts').length != 0 && this.get('bootHosts').someProperty('bootStatus', 'RUNNING');
  },

  /**
   * Remove list of hosts
   * @param {Ember.Enumerable} hosts
   * @return {App.ModalPopup}
   * @method removeHosts
   */
  removeHosts: function removeHosts(hosts) {
    var self = this;
    return App.showConfirmationPopup(function () {
      App.router.send('removeHosts', hosts);
      self.hosts.removeObjects(hosts);
      self.stopRegistration();
      if (!self.hosts.length) {
        self.set('isSubmitDisabled', true);
      }
    }, Em.I18n.t('installer.step3.hosts.remove.popup.body'));
  },

  /**
   * Removes a single element on the trash icon click. Called from View
   * @param {object} hostInfo
   * @method removeHost
   */
  removeHost: function removeHost(hostInfo) {
    if (!this.get('isBackDisabled')) this.removeHosts([hostInfo]);
  },

  /**
   * Remove selected hosts (click-handler)
   * @return App.ModalPopup
   * @method removeSelectedHosts
   */
  removeSelectedHosts: function removeSelectedHosts() {
    var selectedHosts = this.get('hosts').filterProperty('isChecked', true);
    return this.removeHosts(selectedHosts);
  },

  /**
   * Show popup with the list of hosts which are selected
   * @return App.ModalPopup
   * @method selectedHostsPopup
   */
  selectedHostsPopup: function selectedHostsPopup() {
    var selectedHosts = this.get('hosts').filterProperty('isChecked').mapProperty('name');
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step3.selectedHosts.popup.header'),
      secondary: null,
      bodyClass: Em.View.extend({
        templateName: require('templates/common/items_list_popup'),
        items: selectedHosts,
        insertedItems: [],
        didInsertElement: function didInsertElement() {
          lazyloading.run({
            destination: this.get('insertedItems'),
            source: this.get('items'),
            context: this,
            initSize: 100,
            chunkSize: 500,
            delay: 100
          });
        }
      })
    });
  },

  /**
   * Retry one host {click-handler}
   * @param {object} hostInfo
   * @method retryHost
   */
  retryHost: function retryHost(hostInfo) {
    this.retryHosts([hostInfo]);
  },

  /**
   * Retry list of hosts
   * @param {object[]} hosts
   * @method retryHosts
   */
  retryHosts: function retryHosts(hosts) {
    var self = this;
    var bootStrapData = JSON.stringify({
      'verbose': true,
      'sshKey': this.get('content.installOptions.sshKey'),
      'hosts': hosts.mapProperty('name'),
      'user': this.get('content.installOptions.sshUser'),
      'sshPort': this.get('content.installOptions.sshPort'),
      'userRunAs': App.get('supports.customizeAgentUserAccount') ? this.get('content.installOptions.agentUser') : 'root'
    });
    this.set('numPolls', 0);
    this.set('registrationStartedAt', null);
    this.set('isHostsWarningsLoaded', false);
    this.set('stopChecking', false);
    this.set('isSubmitDisabled', true);
    if (this.get('content.installOptions.manualInstall')) {
      this.startRegistration();
    } else {
      App.router.get(this.get('content.controllerName')).launchBootstrap(bootStrapData, function (requestId) {
        self.set('content.installOptions.bootRequestId', requestId);
        self.doBootstrap();
      });
    }
  },

  /**
   * Retry selected hosts (click-handler)
   * @method retrySelectedHosts
   */
  retrySelectedHosts: function retrySelectedHosts() {
    if (!this.get('isRetryDisabled')) {
      var selectedHosts = this.get('bootHosts').filterProperty('bootStatus', 'FAILED');
      selectedHosts.forEach(function (_host) {
        _host.set('bootStatus', 'DONE');
        _host.set('bootLog', 'Retrying ...');
      }, this);
      this.retryHosts(selectedHosts);
    }
  },

  /**
   * Init bootstrap settings and start it
   * @method startBootstrap
   */
  startBootstrap: function startBootstrap() {
    //this.set('isSubmitDisabled', true);    //TODO: uncomment after actual hookup
    this.set('numPolls', 0);
    this.set('registrationStartedAt', null);
    this.set('bootHosts', this.get('hosts'));
    this.doBootstrap();
  },

  /**
   * Update <code>isRegistrationInProgress</code> once
   * @method setRegistrationInProgressOnce
   */
  setRegistrationInProgressOnce: function () {
    Em.run.once(this, 'setRegistrationInProgress');
  }.observes('bootHosts.@each.bootStatus'),

  /**
   * Set <code>isRegistrationInProgress</code> value based on each host boot status
   * @method setRegistrationInProgress
   */
  setRegistrationInProgress: function setRegistrationInProgress() {
    var bootHosts = this.get('bootHosts');
    //if hosts aren't loaded yet then registration should be in progress
    var result = bootHosts.length === 0 && !this.get('isLoaded');
    for (var i = 0, l = bootHosts.length; i < l; i++) {
      if (bootHosts[i].get('bootStatus') !== 'REGISTERED' && bootHosts[i].get('bootStatus') !== 'FAILED') {
        result = true;
        break;
      }
    }
    this.set('isRegistrationInProgress', result);
  },

  /**
   * Disable wizard's previous steps (while registering)
   * @method disablePreviousSteps
   */
  disablePreviousSteps: function () {
    App.router.get('installerController.isStepDisabled').filter(function (step) {
      return step.step >= 0 && step.step <= 2;
    }).setEach('value', this.get('isBackDisabled'));
    App.router.get('addHostController.isStepDisabled').filter(function (step) {
      return step.step >= 0 && step.step <= 1;
    }).setEach('value', this.get('isBackDisabled'));
  }.observes('isBackDisabled'),

  /**
   * Close reload popup on exit from Confirm Hosts step
   * @method closeReloadPopupOnExit
   */
  closeReloadPopupOnExit: function () {
    if (this.get('stopBootstrap')) {
      this.closeReloadPopup();
    }
  }.observes('stopBootstrap'),

  /**
   * Do bootstrap calls
   * @method doBootstrap
   * @return {$.ajax|null}
   */
  doBootstrap: function doBootstrap() {
    if (this.get('stopBootstrap')) {
      return null;
    }
    this.incrementProperty('numPolls');

    return App.ajax.send({
      name: 'wizard.step3.bootstrap',
      sender: this,
      data: {
        bootRequestId: this.get('content.installOptions.bootRequestId'),
        numPolls: this.get('numPolls'),
        callback: this.doBootstrap,
        timeout: 3000,
        shouldUseDefaultHandler: true
      },
      success: 'doBootstrapSuccessCallback',
      error: 'reloadErrorCallback'
    });
  },

  /**
   * Success-callback for each boostrap request
   * @param {object} data
   * @method doBootstrapSuccessCallback
   */
  doBootstrapSuccessCallback: function doBootstrapSuccessCallback(data) {
    var self = this;
    var pollingInterval = 3000;
    this.reloadSuccessCallback();
    if (Em.isNone(data.hostsStatus)) {
      window.setTimeout(function () {
        self.doBootstrap();
      }, pollingInterval);
    } else {
      // in case of bootstrapping just one host, the server returns an object rather than an array, so
      // force into an array
      if (!(data.hostsStatus instanceof Array)) {
        data.hostsStatus = [data.hostsStatus];
      }
      var keepPolling = this.parseHostInfo(data.hostsStatus);

      // Single host : if the only hostname is invalid (data.status == 'ERROR')
      // Multiple hosts : if one or more hostnames are invalid
      // following check will mark the bootStatus as 'FAILED' for the invalid hostname
      var installedHosts = App.Host.find().mapProperty('hostName');
      var isErrorStatus = data.status == 'ERROR';
      this.set('isBootstrapFailed', isErrorStatus);
      if (isErrorStatus || data.hostsStatus.mapProperty('hostName').removeObjects(installedHosts).length != this.get('bootHosts').length) {

        var hosts = this.get('bootHosts');

        for (var i = 0; i < hosts.length; i++) {

          var isValidHost = data.hostsStatus.someProperty('hostName', hosts[i].get('name'));
          if (hosts[i].get('bootStatus') !== 'REGISTERED') {
            if (!isValidHost) {
              hosts[i].set('bootStatus', 'FAILED');
              hosts[i].set('bootLog', Em.I18n.t('installer.step3.hosts.bootLog.failed'));
            }
          }
        }
      }

      if (isErrorStatus || data.hostsStatus.someProperty('status', 'DONE') || data.hostsStatus.someProperty('status', 'FAILED')) {
        // kicking off registration polls after at least one host has succeeded
        this.startRegistration();
      }
      if (keepPolling) {
        window.setTimeout(function () {
          self.doBootstrap();
        }, pollingInterval);
      }
    }
  },

  /**
   * Start hosts registration
   * @method startRegistration
   */
  startRegistration: function startRegistration() {
    if (Em.isNone(this.get('registrationStartedAt'))) {
      this.set('registrationStartedAt', App.dateTime());
      this.isHostsRegistered();
    }
  },

  /**
   * Do requests to check if hosts are already registered
   * @return {$.ajax|null}
   * @method isHostsRegistered
   */
  isHostsRegistered: function isHostsRegistered() {
    if (this.get('stopBootstrap')) {
      return null;
    }
    return App.ajax.send({
      name: 'wizard.step3.is_hosts_registered',
      sender: this,
      success: 'isHostsRegisteredSuccessCallback',
      error: 'reloadErrorCallback',
      data: {
        callback: this.isHostsRegistered,
        timeout: 3000,
        shouldUseDefaultHandler: true
      }
    });
  },

  /**
   * Success-callback for registered hosts request
   * @param {object} data
   * @method isHostsRegisteredSuccessCallback
   */
  isHostsRegisteredSuccessCallback: function isHostsRegisteredSuccessCallback(data) {
    var hosts = this.get('bootHosts');
    var jsonData = data;
    this.reloadSuccessCallback();
    if (!jsonData) {
      return;
    }

    // keep polling until all hosts have registered/failed, or registrationTimeout seconds after the last host finished bootstrapping
    var stopPolling = true;
    hosts.forEach(function (_host, index) {
      // Change name of first host for test mode.
      if (App.get('testMode')) {
        if (index == 0) {
          _host.set('name', 'localhost.localdomain');
        }
      }
      // actions to take depending on the host's current bootStatus
      // RUNNING - bootstrap is running; leave it alone
      // DONE - bootstrap is done; transition to REGISTERING
      // REGISTERING - bootstrap is done but has not registered; transition to REGISTERED if host found in polling API result
      // REGISTERED - bootstrap and registration is done; leave it alone
      // FAILED - either bootstrap or registration failed; leave it alone
      switch (_host.get('bootStatus')) {
        case 'DONE':
          _host.set('bootStatus', 'REGISTERING');
          _host.set('bootLog', (_host.get('bootLog') != null ? _host.get('bootLog') : '') + Em.I18n.t('installer.step3.hosts.bootLog.registering'));
          // update registration timestamp so that the timeout is computed from the last host that finished bootstrapping
          this.set('registrationStartedAt', App.dateTime());
          stopPolling = false;
          break;
        case 'REGISTERING':
          if (jsonData.items.someProperty('Hosts.host_name', _host.name) && !jsonData.items.filterProperty('Hosts.host_name', _host.name).someProperty('Hosts.host_status', 'UNKNOWN')) {
            _host.set('bootStatus', 'REGISTERED');
            _host.set('bootLog', (_host.get('bootLog') != null ? _host.get('bootLog') : '') + Em.I18n.t('installer.step3.hosts.bootLog.registering'));
          } else {
            stopPolling = false;
          }
          break;
        case 'RUNNING':
          stopPolling = false;
          break;
        case 'REGISTERED':
        case 'FAILED':
        default:
          break;
      }
    }, this);

    if (stopPolling) {
      this.startHostcheck(hosts);
    } else {
      if (hosts.someProperty('bootStatus', 'RUNNING') || App.dateTime() - this.get('registrationStartedAt') < this.get('registrationTimeoutSecs') * 1000) {
        // we want to keep polling for registration status if any of the hosts are still bootstrapping (so we check for RUNNING).
        var self = this;
        window.setTimeout(function () {
          self.isHostsRegistered();
        }, 3000);
      } else {
        // registration timed out.  mark all REGISTERING hosts to FAILED
        hosts.filterProperty('bootStatus', 'REGISTERING').forEach(function (_host) {
          _host.set('bootStatus', 'FAILED');
          _host.set('bootLog', (_host.get('bootLog') != null ? _host.get('bootLog') : '') + Em.I18n.t('installer.step3.hosts.bootLog.failed'));
        });
        this.startHostcheck(hosts);
      }
    }
  },

  /**
   * Do request for all registered hosts
   * @return {$.ajax}
   * @method getAllRegisteredHosts
   */
  getAllRegisteredHosts: function () {
    return App.ajax.send({
      name: 'wizard.step3.is_hosts_registered',
      sender: this,
      success: 'getAllRegisteredHostsCallback'
    });
  }.observes('bootHosts'),

  /**
   * Success-callback for all registered hosts request
   * @param {object} hosts
   * @method getAllRegisteredHostsCallback
   */
  getAllRegisteredHostsCallback: function getAllRegisteredHostsCallback(hosts) {
    var registeredHosts = [];
    var hostsInCluster = this.get('hostsInCluster');
    var addedHosts = this.get('bootHosts').getEach('name');
    hosts.items.forEach(function (host) {
      if (!hostsInCluster.contains(host.Hosts.host_name) && !addedHosts.contains(host.Hosts.host_name)) {
        registeredHosts.push(host.Hosts.host_name);
      }
    });
    if (registeredHosts.length) {
      this.set('hasMoreRegisteredHosts', true);
      this.set('registeredHosts', registeredHosts);
    } else {
      this.set('hasMoreRegisteredHosts', false);
      this.set('registeredHosts', '');
    }
  },

  /**
   * Get JDK name from server to determine if user had setup a customized JDK path when doing 'ambari-server setup'.
   * The Ambari properties are different from default ambari-server setup, property 'jdk.name' will be missing if a customized jdk path is applied.
   * @return {$.ajax}
   * @method getJDKName
   */
  getJDKName: function getJDKName() {
    return App.ajax.send({
      name: 'ambari.service',
      sender: this,
      data: {
        fields: '?fields=RootServiceComponents/properties/jdk.name,RootServiceComponents/properties/java.home,RootServiceComponents/properties/jdk_location'
      },
      success: 'getJDKNameSuccessCallback'
    });
  },

  /**
    * Success callback for JDK name, property 'jdk.name' will be missing if a customized jdk path is applied
    * @param {object} data
    * @method getJDKNameSuccessCallback
    */
  getJDKNameSuccessCallback: function getJDKNameSuccessCallback(data) {
    this.set('needJDKCheckOnHosts', !data.RootServiceComponents.properties["jdk.name"]);
    this.set('jdkLocation', Em.get(data, "RootServiceComponents.properties.jdk_location"));
    this.set('javaHome', data.RootServiceComponents.properties["java.home"]);
  },

  doCheckJDK: function doCheckJDK() {
    var hostsNames = this.get('bootHosts').filterProperty('bootStatus', 'REGISTERED').getEach('name').join(','),
        javaHome = this.get('javaHome'),
        jdkLocation = this.get('jdkLocation');
    App.ajax.send({
      name: 'wizard.step3.jdk_check',
      sender: this,
      data: {
        host_names: hostsNames,
        java_home: javaHome,
        jdk_location: jdkLocation
      },
      success: 'doCheckJDKsuccessCallback',
      error: 'doCheckJDKerrorCallback'
    });
  },

  doCheckJDKsuccessCallback: function doCheckJDKsuccessCallback(data) {
    if (data) {
      this.set('jdkRequestIndex', data.href.split('/')[data.href.split('/').length - 1]);
    }
    if (this.get('jdkCategoryWarnings') == null) {
      // get jdk check results for all hosts
      App.ajax.send({
        name: 'wizard.step3.jdk_check.get_results',
        sender: this,
        data: {
          requestIndex: this.get('jdkRequestIndex')
        },
        success: 'parseJDKCheckResults'
      });
    } else {
      this.set('isJDKWarningsLoaded', true);
    }
  },
  doCheckJDKerrorCallback: function doCheckJDKerrorCallback() {
    this.set('isJDKWarningsLoaded', true);
  },
  parseJDKCheckResults: function parseJDKCheckResults(data) {
    var jdkWarnings = [],
        hostsJDKContext = [],
        hostsJDKNames = [];
    // check if the request ended
    if (data.Requests.end_time > 0 && data.tasks) {
      data.tasks.forEach(function (task) {
        // generate warning context
        if (Em.get(task, "Tasks.structured_out.java_home_check.exit_code") == 1) {
          var jdkContext = Em.I18n.t('installer.step3.hostWarningsPopup.jdk.context').format(task.Tasks.host_name);
          hostsJDKContext.push(jdkContext);
          hostsJDKNames.push(task.Tasks.host_name);
        }
      });
      if (hostsJDKContext.length > 0) {
        // java jdk warning exist
        var invalidJavaHome = this.get('javaHome');
        jdkWarnings.push({
          name: Em.I18n.t('installer.step3.hostWarningsPopup.jdk.name').format(invalidJavaHome),
          hosts: hostsJDKContext,
          hostsLong: hostsJDKContext,
          hostsNames: hostsJDKNames,
          category: 'jdk'
        });
      }
      this.set('jdkCategoryWarnings', jdkWarnings);
    } else {
      // still doing JDK check, data not ready to be parsed
      this.set('jdkCategoryWarnings', null);
    }
    this.doCheckJDKsuccessCallback();
  },

  /**
   * Check JDK issues on registered hosts.
   */
  checkHostJDK: function checkHostJDK() {
    this.set('isJDKWarningsLoaded', false);
    this.set('jdkCategoryWarnings', null);
    var self = this;
    this.getJDKName().done(function () {
      if (self.get('needJDKCheckOnHosts')) {
        // need to do JDK check on each host
        self.doCheckJDK();
      } else {
        // no customized JDK path, so no need to check jdk
        self.set('jdkCategoryWarnings', []);
        self.set('isJDKWarningsLoaded', true);
      }
    });
  },

  startHostcheck: function startHostcheck(hosts) {
    if (!hosts.everyProperty('bootStatus', 'FAILED')) {
      this.set('isWarningsLoaded', false);
      this.getHostNameResolution();
      this.checkHostJDK();
    } else {
      this.stopHostCheck();
    }
  },

  _submitProceed: function _submitProceed() {
    this.set('confirmedHosts', this.get('bootHosts'));
    App.get('router').send('next');
  },

  /**
   * Submit-click handler
   * Disable 'Next' button while it is already under process. (using Router's property 'nextBtnClickInProgress')
   * @return {App.ModalPopup?}
   * @method submit
   */
  submit: function submit() {
    var self = this;

    if (App.get('router.nextBtnClickInProgress')) {
      return;
    }
    if (this.get('isHostHaveWarnings')) {
      return App.showConfirmationPopup(function () {
        self._submitProceed();
      }, Em.I18n.t('installer.step3.hostWarningsPopup.hostHasWarnings'), null, Em.I18n.t('installer.step3.hostWarningsPopup.hostHasWarnings.header'), null, 'warning');
    }
    this._submitProceed();
  },

  /**
   * Show popup with host log
   * @param {object} event
   * @return {App.ModalPopup}
   */
  hostLogPopup: function hostLogPopup(event) {
    var host = event.context;

    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step3.hostLog.popup.header').format(host.get('name')),
      secondary: null,
      host: host,
      bodyClass: App.WizardStep3HostLogPopupBody
    });
  },

  /**
   * Open popup that contain hosts' warnings
   * @return {App.ModalPopup}
   * @method hostWarningsPopup
   */
  hostWarningsPopup: function hostWarningsPopup() {
    var self = this;
    return App.ModalPopup.show({

      header: Em.I18n.t('installer.step3.warnings.popup.header'),

      secondary: Em.I18n.t('installer.step3.hostWarningsPopup.rerunChecks'),

      primary: Em.I18n.t('common.close'),

      autoHeight: false,

      'data-qa': 'host-checks-modal',

      onPrimary: function onPrimary() {
        self.set('checksUpdateStatus', null);
        this.hide();
      },

      onClose: function onClose() {
        self.set('checksUpdateStatus', null);
        this.hide();
      },

      onSecondary: function onSecondary() {
        self.rerunChecks();
      },

      didInsertElement: function didInsertElement() {
        this._super();
        this.fitHeight();
      },

      footerClass: App.WizardStep3HostWarningPopupFooter.reopen({
        checkHostFinished: true
      }),

      bodyClass: App.WizardStep3HostWarningPopupBody.reopen({
        checkHostFinished: true
      })
    });
  },

  /**
   * Show popup with registered hosts
   * @return {App.ModalPopup}
   * @method registeredHostsPopup
   */
  registeredHostsPopup: function registeredHostsPopup() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step3.warning.registeredHosts').format(this.get('registeredHosts').length),
      secondary: null,
      bodyClass: Em.View.extend({
        templateName: require('templates/wizard/step3/step3_registered_hosts_popup'),
        message: Em.I18n.t('installer.step3.registeredHostsPopup'),
        registeredHosts: self.get('registeredHosts'),
        checkHostsFinished: true
      })
    });
  }

});

});

require.register("controllers/wizard/step4_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep4Controller = Em.ArrayController.extend({

  name: 'wizardStep4Controller',

  /**
   * List of Services
   * @type {Object[]}
   */
  content: [],

  /**
   * Check / Uncheck 'Select All' checkbox with one argument; Check / Uncheck all other checkboxes with more arguments
   * @type {bool}
   */
  isAllChecked: function (key, value) {
    if (arguments.length > 1) {
      this.filterProperty('isDisabled', false).filterProperty('isDFS', false).setEach('isSelected', value);
      return value;
    }
    return this.filterProperty('isInstalled', false).filterProperty('isHiddenOnSelectServicePage', false).filterProperty('isDFS', false).everyProperty('isSelected', true);
  }.property('@each.isSelected'),

  /**
   * Is Submit button disabled
   * @type {bool}
   */
  isSubmitDisabled: function () {
    return this.filterProperty('isSelected', true).filterProperty('isInstalled', false).length === 0 || App.get('router.btnClickInProgress');
  }.property('@each.isSelected', 'App.router.btnClickInProgress'),

  /**
   * List of validation errors. Look to #createError method for information
   * regarding object structure.
   *
   * @type {Object[]}
   */
  errorStack: [],

  /**
   * Services which are HDFS compatible
   */
  fileSystems: function () {
    var fileSystems = this.filterProperty('isDFS', true);;
    return fileSystems.map(function (fs) {
      return App.FileSystem.create({ content: fs, services: fileSystems });
    });
  }.property('@each.isDFS'),

  /**
   * Drop errorStack content on selected state changes.
   */
  clearErrors: function () {
    if (!this.get('errorStack').someProperty('isAccepted', false)) {
      this.set('errorStack', []);
    }
  }.observes('@each.isSelected'),

  /**
   * Check if multiple distributed file systems were selected
   * @return {bool}
   * @method multipleDFSs
   */
  multipleDFSs: function multipleDFSs() {
    return this.filterProperty('isDFS', true).filterProperty('isSelected', true).length > 1;
  },

  /**
   * Warn user if he tries to install Spark with HDP 2.2
   * @param {function} callback
   * @method sparkValidation
   */
  sparkValidation: function sparkValidation(callback) {
    var sparkService = this.findProperty('serviceName', 'SPARK');
    if (sparkService && !sparkService.get('isInstalled') && App.get('currentStackName') === 'HDP' && App.get('currentStackVersionNumber') === '2.2') {
      if (sparkService.get('isSelected')) {
        this.addValidationError({
          id: 'sparkWarning',
          type: 'WARNING',
          callback: this.sparkWarningPopup,
          callbackParams: [callback]
        });
      } else {
        //Spark is selected, remove the Spark error from errorObject array
        var sparkError = this.get('errorStack').filterProperty('id', "sparkWarning");
        if (sparkError) {
          this.get('errorStack').removeObject(sparkError[0]);
        }
      }
    }
  },

  /**
   * Onclick handler for <code>Next</code> button.
   * Disable 'Next' button while it is already under process. (using Router's property 'nextBtnClickInProgress')
   * @method submit
   */
  submit: function submit() {
    if (App.get('router.nextBtnClickInProgress')) {
      return;
    }
    if (!this.get('isSubmitDisabled')) {
      this.unSelectServices();
      this.setGroupedServices();
      if (this.validate()) {
        this.set('errorStack', []);
        App.router.send('next');
      }
    }
  },

  /**
   * Set isSelected based on property doNotShowAndInstall
   */
  unSelectServices: function unSelectServices() {
    this.filterProperty('isSelected', true).filterProperty('doNotShowAndInstall', true).setEach('isSelected', false);
  },

  /**
   * Check if validation passed:
   *  - required file system services selected
   *  - dependencies between services
   *  - monitoring services selected (not required)
   *
   * @return {Boolean}
   * @method validate
   */
  validate: function validate() {
    var result;
    var self = this;

    // callback function to reset `isAccepted` needs to be called everytime when a popup from errorStack is dismissed/proceed by user action
    var callback = function callback(id) {
      var check = self.get('errorStack').findProperty('id', id);
      if (check) {
        check.isAccepted = true;
      }
    };
    this.serviceDependencyValidation(callback);
    this.fileSystemServiceValidation(callback);
    if (this.get('wizardController.name') === 'installerController') {
      this.serviceValidation(callback, 'AMBARI_METRICS', 'ambariMetricsCheck');
      this.serviceValidation(callback, 'SMARTSENSE', 'smartSenseCheck');
      this.serviceValidation(callback, 'RANGER', 'rangerCheck');
      this.serviceValidation(callback, 'ATLAS', 'atlasCheck');
    }
    this.dependentServiceValidation('RANGER', 'AMBARI_INFRA_SOLR', 'ambariRangerInfraCheck', callback);
    this.dependentServiceValidation('ATLAS', 'AMBARI_INFRA_SOLR', 'ambariAtlasInfraCheck', callback);
    this.dependentServiceValidation('ATLAS', 'HBASE', 'ambariAtlasHbaseCheck', callback);
    this.dependentServiceValidation('LOGSEARCH', 'AMBARI_INFRA_SOLR', 'ambariLogsearchCheck', callback);
    this.sparkValidation(callback);
    if (!!this.get('errorStack').filterProperty('isShown', false).length) {
      var firstError = this.get('errorStack').findProperty('isShown', false);
      this.showError(firstError);
      result = false;
    } else {
      result = true;
    }
    return result;
  },

  /**
   * display validation warning if dependent service not selected
   * @param {string} selectedService
   * @param {string} dependentService
   * @param {string} checkId
   * @param {Function} callback
   */
  dependentServiceValidation: function dependentServiceValidation(selectedService, dependentService, checkId, callback) {
    var selected = this.findProperty('serviceName', selectedService);
    var dependent = this.findProperty('serviceName', dependentService);
    if (selected && selected.get('isSelected') && dependent && !dependent.get('isSelected')) {
      this.serviceValidation(callback, dependentService, checkId);
    } else {
      var unNeededError = this.get('errorStack').filterProperty('id', checkId);
      if (unNeededError) {
        this.get('errorStack').removeObject(unNeededError[0]);
      }
    }
  },

  /**
   * Check whether user selected service to install and go to next step
   * @param callback {Function}
   * @param serviceName {string}
   * @param id {string}
   * @method serviceValidation
   */
  serviceValidation: function serviceValidation(callback, serviceName, id) {
    var service = this.findProperty('serviceName', serviceName);
    if (service) {
      if (!service.get('isSelected')) {
        this.addValidationError({
          id: id,
          type: 'WARNING',
          callback: this.serviceCheckPopup,
          callbackParams: [callback]
        });
      } else {
        //metrics is selected, remove the metrics error from errorObject array
        var metricsError = this.get('errorStack').filterProperty('id', id);
        if (metricsError) {
          this.get('errorStack').removeObject(metricsError[0]);
        }
      }
    }
  },

  /**
   * Create error and push it to stack.
   *
   * @param {Object} errorObject - look to #createError
   * @return {Boolean}
   * @method addValidationError
   */
  addValidationError: function addValidationError(errorObject) {
    if (!this.get('errorStack').someProperty('id', errorObject.id)) {
      this.get('errorStack').push(this.createError(errorObject));
      return true;
    }
    return false;
  },

  /**
   * Show current error by passed error object.
   *
   * @param {Object} errorObject
   * @method showError
   */
  showError: function showError(errorObject) {
    return errorObject.callback.apply(errorObject.callbackContext, errorObject.callbackParams.concat(errorObject.id));
  },

  /**
   * Default primary button("Ok") callback for warning popups.
   *  Change isShown state for last shown error.
   *  Call #submit() method.
   *
   *  @param {function} callback
   *  @param {string} id
   *  @method onPrimaryPopupCallback
   */
  onPrimaryPopupCallback: function onPrimaryPopupCallback(callback, id) {
    var firstError = this.get('errorStack').findProperty('isShown', false);
    if (firstError) {
      firstError.isShown = true;
    }
    if (callback) {
      callback(id);
    }
    this.submit();
  },

  /**
   * Create error object with passed options.
   * Available options:
   *  id - {String}
   *  type - {String}
   *  isShowed - {Boolean}
   *  callback - {Function}
   *  callbackContext
   *  callbackParams - {Array}
   *
   * @param {Object} opt
   * @return {Object}
   * @method createError
   */
  createError: function createError(opt) {
    var options = {
      // {String} error identifier
      id: '',
      // {String} type of error CRITICAL|WARNING
      type: 'CRITICAL',
      // {Boolean} error was shown
      isShown: false,
      // {Boolean} error was accepted by user
      isAccepted: false,
      // {Function} callback to execute
      callback: null,
      // context which execute from
      callbackContext: this,
      // {Array} params applied to callback
      callbackParams: []
    };
    $.extend(options, opt);
    return options;
  },

  /**
   * Checks if a filesystem is present in the Stack
   *
   * @method isDFSStack
   */
  isDFSStack: function isDFSStack() {
    var bDFSStack = false;
    var dfsServices = ['HDFS', 'GLUSTERFS'];
    var availableServices = this.filterProperty('isInstalled', false);
    availableServices.forEach(function (service) {
      if (dfsServices.contains(service.get('serviceName')) || service.get('serviceType') == 'HCFS') {
        bDFSStack = true;
      }
    }, this);
    return bDFSStack;
  },

  /**
   * Checks if a filesystem is selected and only one filesystem is selected
   * @param {function} callback
   * @method isFileSystemCheckFailed
   */
  fileSystemServiceValidation: function fileSystemServiceValidation(callback) {
    if (this.isDFSStack()) {
      var primaryDFS = this.findProperty('isPrimaryDFS', true);
      if (primaryDFS) {
        var primaryDfsDisplayName = primaryDFS.get('displayNameOnSelectServicePage');
        var primaryDfsServiceName = primaryDFS.get('serviceName');
        if (this.multipleDFSs()) {
          var dfsServices = this.filterProperty('isDFS', true).filterProperty('isSelected', true).mapProperty('serviceName');
          var services = dfsServices.map(function (item) {
            return {
              serviceName: item,
              selected: item === primaryDfsServiceName
            };
          });
          this.addValidationError({
            id: 'multipleDFS',
            callback: this.needToAddServicePopup,
            callbackParams: [services, 'multipleDFS', primaryDfsDisplayName, callback]
          });
        } else {
          //if multiple DFS are not selected, remove the related error from the error array
          var fsError = this.get('errorStack').filterProperty('id', "multipleDFS");
          if (fsError) {
            this.get('errorStack').removeObject(fsError[0]);
          }
        }
      }
    }
  },

  /**
   * Checks if a dependent service is selected without selecting the main service.
   * @param {function} callback
   * @method serviceDependencyValidation
   */
  serviceDependencyValidation: function serviceDependencyValidation(callback) {
    var selectedServices = this.filterProperty('isSelected', true);
    var availableServices = this.get('content');
    var missingDependencies = [];
    selectedServices.forEach(function (service) {
      service.collectMissingDependencies(selectedServices, availableServices, missingDependencies);
    });
    this.cleanExistingServiceCheckErrors();
    this.addServiceCheckErrors(missingDependencies, callback);
  },

  cleanExistingServiceCheckErrors: function cleanExistingServiceCheckErrors() {
    var existingServiceCheckErrors = this.get('errorStack').filter(function (error) {
      return error.id.startsWith('serviceCheck_');
    });
    this.get('errorStack').removeObjects(existingServiceCheckErrors);
  },
  addServiceCheckErrors: function addServiceCheckErrors(missingDependencies, callback) {
    for (var i = 0; i < missingDependencies.length; i++) {
      this.addValidationError({
        id: 'serviceCheck_' + missingDependencies[i].get('serviceName'),
        callback: this.needToAddMissingDependency,
        callbackParams: [missingDependencies[i], 'serviceCheck', callback]
      });
    }
  },


  /**
   * Select co hosted services which not showed on UI.
   *
   * @method setGroupedServices
   */
  setGroupedServices: function setGroupedServices() {
    this.forEach(function (service) {
      var coSelectedServices = service.get('coSelectedServices');
      coSelectedServices.forEach(function (groupedServiceName) {
        var groupedService = this.findProperty('serviceName', groupedServiceName);
        if (groupedService.get('isSelected') !== service.get('isSelected')) {
          groupedService.set('isSelected', service.get('isSelected'));
        }
      }, this);
    }, this);
  },

  /**
   * Select/deselect services
   * @param {object[]|object} services array of objects
   *  <code>
   *    [
   *      {
   *        service: 'HDFS',
   *        selected: true
   *      },
   *      ....
   *    ]
   *  </code>
   * @param {string} i18nSuffix
   * @param {string} serviceName
   * @param {function} callback
   * @param {string} id
   * @return {App.ModalPopup}
   * @method needToAddServicePopup
   */

  needToAddServicePopup: function needToAddServicePopup(services, i18nSuffix, serviceName, callback, id) {
    var self = this;
    return App.ModalPopup.show({
      'data-qa': 'need-add-service-confirmation-modal',
      header: Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.header').format(serviceName),
      body: Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.body').format(serviceName, serviceName),
      onPrimary: function onPrimary() {
        Em.makeArray(services).forEach(function (service) {
          self.findProperty('serviceName', service.serviceName).set('isSelected', service.selected);
        });
        self.onPrimaryPopupCallback(callback, id);
        this.hide();
      },
      onSecondary: function onSecondary() {
        if (callback) {
          callback(id);
        }
        this._super();
      },
      onClose: function onClose() {
        if (callback) {
          callback(id);
        }
        this._super();
      }
    });
  },

  needToAddMissingDependency: function needToAddMissingDependency(missingDependency, i18nSuffix, callback, id) {
    var self = this;
    var displayName = missingDependency.get('displayName');
    if (missingDependency.get('hasMultipleOptions')) {
      return this.showDependencyPopup(id, Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.header').format(displayName), Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.body.multiOptions').format(displayName, missingDependency.get('displayOptions')), callback);
    } else {
      return this.showDependencyPopup(id, Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.header').format(displayName), Em.I18n.t('installer.step4.' + i18nSuffix + '.popup.body').format(displayName, missingDependency.get('serviceName')), callback, function () {
        missingDependency.selectFirstCompatible();
        self.onPrimaryPopupCallback(callback, id);
        this.hide();
      });
    }
  },

  showDependencyPopup: function showDependencyPopup(id, header, body, callback, primaryAction) {
    return App.ModalPopup.show({
      'data-qa': 'need-add-service-confirmation-modal',
      header: header,
      body: body,
      onPrimary: primaryAction || function () {
        this.onClose();
      },
      onSecondary: function onSecondary() {
        this.onClose();
      },
      onClose: function onClose() {
        if (callback) {
          callback(id);
        }
        this._super();
      }
    });
  },

  /**
   * Show popup with info about not selected service
   * @param {function} callback
   * @param {string} id
   * @return {App.ModalPopup}
   * @method serviceCheckPopup
   */
  serviceCheckPopup: function serviceCheckPopup(callback, id) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step4.limitedFunctionality.popup.header'),
      //body: Em.I18n.t('installer.step4.' + id + '.popup.body'),
      bodyClass: Em.View.extend({
        serviceHeader: Em.I18n.t('installer.step4.' + id + '.popup.header'),
        serviceBody: Em.I18n.t('installer.step4.' + id + '.popup.body'),
        templateName: require('templates/wizard/step4/step4_service_validation_popup')
      }),
      primary: Em.I18n.t('common.proceedAnyway'),
      primaryClass: 'btn-warning',
      onPrimary: function onPrimary() {
        self.onPrimaryPopupCallback(callback);
        this.hide();
      },
      onSecondary: function onSecondary() {
        if (callback) {
          callback(id);
        }
        this._super();
      },
      onClose: function onClose() {
        if (callback) {
          callback(id);
        }
        this._super();
      }
    });
  },

  /**
   * Show popup with Spark installation warning
   * @param {function} callback
   * @param {string} id
   * @return {App.ModalPopup}
   * @method sparkWarningPopup
   */
  sparkWarningPopup: function sparkWarningPopup(callback, id) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      body: Em.I18n.t('installer.step4.sparkWarning.popup.body'),
      primary: Em.I18n.t('common.proceed'),
      onPrimary: function onPrimary() {
        self.onPrimaryPopupCallback(callback);
        this.hide();
      },
      onSecondary: function onSecondary() {
        if (callback) {
          callback(id);
        }
        this._super();
      },
      onClose: function onClose() {
        if (callback) {
          callback(id);
        }
        this._super();
      }
    });
  }
});

});

require.register("controllers/wizard/step5_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep5Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {

  name: "wizardStep5Controller",

  _goNextStepIfValid: function _goNextStepIfValid() {
    App.set('router.nextBtnClickInProgress', false);
    if (!this.get('submitDisabled')) {
      App.router.send('next');
    }
  },

  _additionalClearSteps: function _additionalClearSteps() {
    var parentController = App.router.get(this.get('content.controllerName'));
    if (parentController && parentController.get('content.componentsFromConfigs')) {
      parentController.clearConfigActionComponents();
    }
  }

});

});

require.register("controllers/wizard/step6_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');
var validationUtils = require('utils/validator');

/**
 * By Step 6, we have the following information stored in App.db and set on this
 * controller by the router:
 *
 *   hosts: App.db.hosts (list of all hosts the user selected in Step 3)
 *   selectedServiceNames: App.db.selectedServiceNames (the services that the user selected in Step 4)
 *   masterComponentHosts: App.db.masterComponentHosts (master-components-to-hosts mapping the user selected in Step 5)
 *
 * Step 6 will set the following information in App.db:
 *   slaveComponentHosts: App.db.slaveComponentHosts (slave-components-to-hosts mapping the user selected in Step 6)
 *
 */
App.WizardStep6Controller = Em.Controller.extend(App.HostComponentValidationMixin, App.HostComponentRecommendationMixin, {

  name: 'wizardStep6Controller',

  /**
   * List of hosts
   * @type {object[]}
   */
  hosts: [],

  /**
   * List of components info about selecting/deselecting status for components.
   *
   * @type {Array}
   *  @item {Em.Object}
   *    @property name {String} - component name
   *    @property label {String} - component display name
   *    @property allChecked {bool} - all checkboxes are checked
   *    @property noChecked {bool} - no checkboxes checked
   */
  headers: [],

  /**
   * @type {bool}
   */
  isLoaded: false,

  /**
   * Indication if user has chosen hosts to install clients
   * @type {bool}
   */
  isClientsSet: false,

  /**
   * Define state for submit button
   * @type {bool}
   */
  submitDisabled: Em.computed.or('validationInProgress', 'App.router.btnClickInProgress'),

  /**
   * timer for validation request
   */
  timer: null,

  /**
   * true if request for validation is in progress
   *
   * @type {bool}
   */
  validationInProgress: false,

  /**
   * Check if <code>addHostWizard</code> used
   * @type {bool}
   */
  isAddHostWizard: Em.computed.equal('content.controllerName', 'addHostController'),

  /**
   * Check if <code>installerWizard</code> used
   * @type {bool}
   */
  isInstallerWizard: Em.computed.equal('content.controllerName', 'installerController'),

  isAllCheckboxesEmpty: function isAllCheckboxesEmpty() {
    var hosts = this.get('hosts');
    for (var i = 0; i < hosts.length; i++) {
      var checkboxes = hosts[i].checkboxes;
      for (var j = 0; j < checkboxes.length; j++) {
        if (checkboxes[j].checked) {
          return false;
        }
      }
    }
    return true;
  },

  /**
   * Check if <code>addServiceWizard</code> used
   * @type {bool}
   */
  isAddServiceWizard: Em.computed.equal('content.controllerName', 'addServiceController'),

  installedServiceNames: function () {
    return this.get('content.services').filterProperty('isInstalled').mapProperty('serviceName');
  }.property('content.services').cacheable(),

  /**
   * Validation error messages which don't related with any master
   */
  generalErrorMessages: [],

  /**
   * Validation warning messages which don't related with any master
   */
  generalWarningMessages: [],

  /**
   * true if validation has any general (which is not related with concrete host) error message
   */
  anyGeneralErrors: Em.computed.or('errorMessage', 'generalErrorMessages.length'),

  /**
   * true if validation has any general (which is not related with concrete host) warning message
   */
  anyGeneralWarnings: Em.computed.gt('generalWarningMessages.length', 0),

  /**
   * true if validation has any general (which is not related with concrete host) error or warning message
   */
  anyGeneralIssues: Em.computed.or('anyGeneralErrors', 'anyGeneralWarnings'),

  anyHostErrors: function () {
    return this.get('hosts').some(function (h) {
      return h.errorMessages ? h.errorMessages.length > 0 : false;
    });
  }.property('hosts.@each.errorMessages'),

  /**
   * true if validation has any error message (general or host specific)
   */
  anyErrors: Em.computed.or('anyGeneralErrors', 'anyHostErrors'),

  anyHostWarnings: function () {
    return this.get('hosts').some(function (h) {
      return h.warnMessages ? h.warnMessages.length > 0 : false;
    });
  }.property('hosts.@each.warnMessages'),

  /**
   * true if validation has any warning message (general or host specific)
   */
  anyWarnings: Em.computed.or('anyGeneralWarnings', 'anyHostWarnings'),

  /**
   * Verify condition that at least one checkbox of each component was checked
   * @method clearError
   */
  clearError: function clearError() {
    var self = this;
    var isError = false;
    var err = false;
    var hosts = this.get('hosts');
    var headers = this.get('headers');
    var headersMap = headers.toWickMapByProperty('name');

    hosts.forEach(function (host) {
      host.checkboxes.forEach(function (checkbox) {
        if (headersMap[checkbox.component]) {
          headersMap[checkbox.component] = !checkbox.checked;
        }
      });
    });
    for (var i in headersMap) {
      err |= headersMap[i];
    }

    if (!err) {
      this.set('errorMessage', '');
    }

    if (this.get('isAddHostWizard')) {
      hosts.forEach(function (host) {
        isError = false;
        headers.forEach(function (header) {
          isError |= host.checkboxes.findProperty('title', header.get('label')).checked;
        });
        isError = !isError;
        if (!isError) {
          self.set('errorMessage', '');
        }
      });
    }
  },

  /**
   * Clear Step6 data like <code>hosts</code>, <code>headers</code> etc
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.set('hosts', []);
    this.set('headers', []);
    this.clearError();
    this.set('isLoaded', false);
    this.set('validationInProgress', false);
  },

  /**
   * Enable some service for all hosts
   * @param {object} event
   * @method selectAllNodes
   */
  selectAllNodes: function selectAllNodes(event) {
    var name = Em.get(event, 'context.name');
    if (name) {
      this.setAllNodes(name, true);
      this.callValidation();
    }
  },

  /**
   * Disable some services for all hosts
   * @param {object} event
   * @method deselectAllNodes
   */
  deselectAllNodes: function deselectAllNodes(event) {
    var name = Em.get(event, 'context.name');
    if (name) {
      this.setAllNodes(name, false);
      this.callValidation();
    }
  },

  /**
   * Enable/disable some service for all hosts
   * @param {String} component - component name
   * @param {bool} checked - true - enable, false - disable
   * @method setAllNodes
   */
  setAllNodes: function setAllNodes(component, checked) {
    this.get('hosts').forEach(function (host) {
      host.checkboxes.filterProperty('isInstalled', false).forEach(function (checkbox) {
        if (checkbox.component === component) {
          Em.set(checkbox, 'checked', checked);
        }
      });
    });
    this.checkCallback(component);
  },

  /**
   * Checkbox check callback
   * Verify if all/none checkboxes for current component are checked
   * @param {String} component
   * @method checkCallback
   */
  checkCallback: function checkCallback(component) {
    var header = this.get('headers').findProperty('name', component);
    if (header) {
      var hosts = this.get('hosts');
      var allTrue = true;
      var allFalse = true;
      hosts.forEach(function (host) {
        host.checkboxes.forEach(function (checkbox) {
          if (checkbox.component === component && !checkbox.isInstalled) {
            allTrue = allTrue && checkbox.checked;
            allFalse = allFalse && !checkbox.checked;
          }
        });
      });
      header.set('allChecked', allTrue);
      header.set('noChecked', allFalse);
    }
    this.clearError();
  },

  /**
   * Init step6 data
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.clearStep();
    var parentController = App.router.get(this.get('content.controllerName'));
    if (parentController && parentController.get('content.componentsFromConfigs')) {
      parentController.clearConfigActionComponents();
    }

    var selectedServices = App.StackService.find().filterProperty('isSelected');
    var installedServices = App.StackService.find().filterProperty('isInstalled');
    var services;
    if (this.get('isInstallerWizard')) services = selectedServices;else if (this.get('isAddHostWizard')) services = installedServices;else if (this.get('isAddServiceWizard')) services = installedServices.concat(selectedServices);

    var headers = Em.A([]);
    services.forEach(function (stackService) {
      stackService.get('serviceComponents').forEach(function (serviceComponent) {
        if (serviceComponent.get('isShownOnInstallerSlaveClientPage')) {
          headers.pushObject(Em.Object.create({
            name: serviceComponent.get('componentName'),
            label: App.format.role(serviceComponent.get('componentName'), false),
            allChecked: false,
            isRequired: serviceComponent.get('isRequired'),
            noChecked: true,
            isDisabled: installedServices.someProperty('serviceName', stackService.get('serviceName')) && this.get('isAddServiceWizard'),
            allId: 'all-' + serviceComponent.get('componentName'),
            noneId: 'none-' + serviceComponent.get('componentName')
          }));
        }
      }, this);
    }, this);
    if (this.get('content.clients') && !!this.get('content.clients').length) {
      headers.pushObject(Em.Object.create({
        name: 'CLIENT',
        label: App.format.role('CLIENT', false),
        allChecked: false,
        noChecked: true,
        isDisabled: false,
        allId: 'all-CLIENT',
        noneId: 'none-CLIENT'
      }));
    }
    this.get('headers').pushObjects(headers);

    this.render();
    if (this.get('content.skipSlavesStep')) {
      App.router.send('next');
    } else {
      this.callValidation();
    }
  },

  /**
   * Returns list of new hosts
   *
   * @param {object[]} [allHosts=null]
   * @return {object[]}
   */
  getNewHosts: function getNewHosts(allHosts) {
    var hosts = allHosts || this.getAllHosts();
    return hosts.filterProperty('isInstalled', false);
  },

  /**
   * Returns list of registered hosts
   *
   * @return {object[{hostName, isInstalled}]}
   */
  getAllHosts: function getAllHosts() {
    var self = this;
    var hosts = self.get('content.hosts');
    return Em.keys(this.get('content.hosts')).reduce(function (res, hostName) {
      var host = hosts[hostName];
      if (Em.get(host, 'bootStatus') !== 'REGISTERED') {
        return res;
      }
      return res.concat({
        hostName: hostName,
        isInstalled: Em.getWithDefault(host, 'isInstalled', false)
      });
    }, []);
  },
  /**
   * Load all data needed for this module. Then it automatically renders in template
   * @method render
   */
  render: function render() {
    var hostsObj = [],
        masterHosts = [],
        headers = this.get('headers'),
        masterHostNames = this.get('content.masterComponentHosts').mapProperty('hostName').uniq(),
        masterHostNamesMap = masterHostNames.toWickMap(),
        hosts = this.get('isAddHostWizard') ? this.getNewHosts() : this.getAllHosts();

    var i = 1;
    hosts.mapProperty('hostName').forEach(function (_hostName) {
      var hasMaster = masterHostNamesMap[_hostName];
      var obj = {
        hostName: _hostName,
        hasMaster: hasMaster,
        checkboxes: headers.map(function (header, index) {
          return {
            component: header.name,
            title: header.label,
            checked: false,
            isInstalled: false,
            isDisabled: header.get('isDisabled'),
            uId: _hostName + '-checkbox-' + index,
            dataQaAttr: header.name === 'CLIENT' ? 'client-component' : ''
          };
        }),
        isLast: i === hosts.length ? true : false
      };

      if (hasMaster) {
        masterHosts.pushObject(obj);
      } else {
        hostsObj.pushObject(obj);
      }
      i++;
    });
    //hosts with master components should be in the beginning of list
    hostsObj.unshift.apply(hostsObj, masterHosts);
    hostsObj = this.renderSlaves(hostsObj);
    this.set('hosts', hostsObj);
    headers.forEach(function (header) {
      this.checkCallback(header.get('name'));
    }, this);
    this.set('isLoaded', true);
  },

  /**
   * Set checked values for slaves checkboxes
   * @param {Array} hostsObj
   * @return {Array}
   * @method renderSlaves
   */
  renderSlaves: function renderSlaves(hostsObj) {
    var slaveComponents = this.get('content.slaveComponentHosts');

    if (Em.isNone(slaveComponents)) {
      // we are at this page for the first time
      this.selectRecommendedComponents(hostsObj);
      this.setInstalledComponents(hostsObj);
    } else {
      this.restoreComponentsSelection(hostsObj, slaveComponents);
    }
    if (this.get('isAddServiceWizard')) {
      this.enableCheckboxesForDependentComponents(hostsObj);
    }
    this.selectClientHost(hostsObj);
    return hostsObj;
  },

  /**
   * set installed flag of host-components
   * @param {Array} hostsObj
   * @returns {boolean}
   */
  setInstalledComponents: function setInstalledComponents(hostsObj) {
    if (Em.isNone(this.get('content.installedHosts'))) return false;
    var hosts = this.get('content.installedHosts');

    hostsObj.forEach(function (host) {
      var installedHost = hosts[host.hostName];
      var installedComponents = installedHost ? installedHost.hostComponents.mapProperty('HostRoles.component_name') : [];

      host.checkboxes.forEach(function (checkbox) {
        checkbox.isInstalled = installedComponents.contains(checkbox.component);
        if (checkbox.isInstalled) {
          checkbox.checked = true;
        }
      });
    });
  },

  /**
   * Enable checkboxes for dependent components of already installed services, that can be added
   * @param hostsObj
   */
  enableCheckboxesForDependentComponents: function enableCheckboxesForDependentComponents(hostsObj) {
    var dependentSlaves = {};
    App.StackService.find().filterProperty('isSelected').forEach(function (service) {
      service.get('serviceComponents').forEach(function (component) {
        component.get('dependencies').forEach(function (dependency) {
          var dependentService = App.StackService.find().findProperty('serviceName', dependency.serviceName);
          if (!dependentService) {
            return;
          }
          var dependentComponent = dependentService.get('serviceComponents').findProperty('componentName', dependency.componentName);
          if (dependentComponent.get('isSlave') && dependentService.get('isInstalled')) {
            dependentSlaves[dependentComponent.get('componentName')] = [];
          }
        });
      });
    });

    if (!Em.keys(dependentSlaves)) return false;

    hostsObj.forEach(function (hostObj) {
      hostObj.checkboxes.forEach(function (checkbox) {
        if (dependentSlaves[checkbox.component] && !checkbox.isInstalled) {
          dependentSlaves[checkbox.component].push(checkbox);
        }
      });
    });

    for (var component in dependentSlaves) {
      if (dependentSlaves.hasOwnProperty(component)) {
        var maxToInstall = App.StackServiceComponent.find().findProperty('componentName', component).get('maxToInstall');
        maxToInstall = maxToInstall === Infinity ? hostsObj.length : maxToInstall;
        if (maxToInstall > hostsObj.length - dependentSlaves[component].length) {
          dependentSlaves[component].setEach('isDisabled', false);
        }
      }
    }
    return true;
  },

  /**
   * restore previous component selection
   * @param {Array} hostsObj
   * @param {Array} slaveComponents
   */
  restoreComponentsSelection: function restoreComponentsSelection(hostsObj, slaveComponents) {
    var slaveComponentsMap = slaveComponents.toMapByProperty('componentName');
    var hostsObjMap = hostsObj.toMapByProperty('hostName');

    this.get('headers').forEach(function (header) {
      var slaveComponent = slaveComponentsMap[header.get('name')];
      if (slaveComponent) {
        slaveComponent.hosts.forEach(function (_node) {
          var node = hostsObjMap[_node.hostName];
          if (node) {
            Em.set(node.checkboxes.findProperty('title', header.get('label')), 'checked', true);
            Em.set(node.checkboxes.findProperty('title', header.get('label')), 'isInstalled', _node.isInstalled);
          }
        });
      }
    });
  },

  /**
   * select component which should be checked according to recommendations
   * @param hostsObj
   */
  selectRecommendedComponents: function selectRecommendedComponents(hostsObj) {
    var recommendations = this.get('content.recommendations'),
        recommendedMap = {},
        clientComponentsMap = App.get('components.clients').toWickMap();

    recommendations.blueprint.host_groups.forEach(function (hostGroup) {
      var group = recommendations.blueprint_cluster_binding.host_groups.findProperty('name', hostGroup.name);
      var hosts = group.hosts || [];

      hosts.forEach(function (host) {
        recommendedMap[host.fqdn] = hostGroup.components.mapProperty('name');
      });
    });

    hostsObj.forEach(function (host) {
      var checkboxes = host.checkboxes;
      var hostComponents = recommendedMap[host.hostName] || [];
      checkboxes.forEach(function (checkbox) {
        var checked;
        if (!checkbox.isDisabled) {
          checked = hostComponents.contains(checkbox.component);

          if (checkbox.component === 'CLIENT' && !checked) {
            checked = hostComponents.some(function (componentName) {
              return clientComponentsMap[componentName];
            });
          }
          checkbox.checked = checked;
        }
      });
    });
  },

  /**
   * For clients - select first non-master host, if all has masters then last host
   * @param hostsObj
   */
  selectClientHost: function selectClientHost(hostsObj) {
    if (!this.get('isClientsSet')) {
      var nonMasterHost = hostsObj.findProperty('hasMaster', false);
      var clientHost = !!nonMasterHost ? nonMasterHost : hostsObj[hostsObj.length - 1]; // last host
      var clientCheckBox = clientHost.checkboxes.findProperty('component', 'CLIENT');
      if (clientCheckBox) {
        Em.set(clientCheckBox, 'checked', true);
      }
      this.set('isClientsSet', true);
    }
  },

  /**
   * Select checkboxes which correspond to master components
   *
   * @param {Array} hostsObj
   * @return {Array}
   * @method selectMasterComponents
   */
  selectMasterComponents: function selectMasterComponents(hostsObj) {
    var masterComponentHosts = this.get('content.masterComponentHosts');

    if (masterComponentHosts) {
      masterComponentHosts.forEach(function (item) {
        var host = hostsObj.findProperty('hostName', item.hostName);
        if (host) {
          var checkbox = host.get('checkboxes').findProperty('component', item.component);
          if (checkbox) {
            checkbox.set('checked', true);
          }
        }
      });
    }
    return hostsObj;
  },

  /**
   * Return list of master components for specified <code>hostname</code>
   * @param {string} hostName
   * @return {string[]}
   * @method getMasterComponentsForHost
   */
  getMasterComponentsForHost: function getMasterComponentsForHost(hostName) {
    return this.get('content.masterComponentHosts').filterProperty('hostName', hostName).mapProperty('component');
  },

  callValidation: function callValidation(successCallback) {
    var self = this;
    var validationCallback = function validationCallback() {
      self.set('validationInProgress', false);
      if (App.get('router.btnClickInProgress') && successCallback) {
        successCallback();
      }
    };
    clearTimeout(this.get('timer'));
    if (this.get('validationInProgress')) {
      this.set('timer', setTimeout(function () {
        self.callValidation();
      }, 700));
    } else {
      this.callServerSideValidation().then(validationCallback);
    }
  },

  /**
   * Update submit button status
   * @method callServerSideValidation
   */
  callServerSideValidation: function callServerSideValidation(successCallback) {
    this.set('validationInProgress', true);

    var selectedServices = App.StackService.find().filterProperty('isSelected').mapProperty('serviceName');
    var installedServices = App.StackService.find().filterProperty('isInstalled').mapProperty('serviceName');
    var services = installedServices.concat(selectedServices).uniq();
    var hostNames = this.getAllHosts().mapProperty('hostName');

    var bluePrintsForValidation = this.getValidationBlueprint();
    this.set('content.recommendationsHostGroups', bluePrintsForValidation);

    return this.validateSelectedHostComponents({
      services: services,
      hosts: hostNames,
      blueprint: bluePrintsForValidation
    });
  },

  /**
   * Returns blueprint passed to validation request
   * @method getValidationBlueprint
   */
  getValidationBlueprint: function getValidationBlueprint() {
    var slaveBlueprint = this.getCurrentBlueprint();
    var masterBlueprint = null;
    var invisibleInstalledMasters = [];
    var hostNames = this.getAllHosts().mapProperty('hostName');
    var invisibleSlavesAndClients = App.StackServiceComponent.find().filter(function (component) {
      return component.get("isSlave") && component.get("isShownOnInstallerSlaveClientPage") === false || component.get("isClient") && component.get("isRequiredOnAllHosts");
    }).mapProperty("componentName");
    //Existing Installed but invisible masters on `Assign Masters page` should be included in host component layout for recommnedation/validation call
    if (this.get('isAddServiceWizard')) {
      var invisibleMasters = App.StackServiceComponent.find().filterProperty("isMaster").filterProperty("isShownOnAddServiceAssignMasterPage", false);
      invisibleInstalledMasters = invisibleMasters.filter(function (item) {
        var masterComponent = App.MasterComponent.find().findProperty('componentName', item.get('componentName'));
        return masterComponent && !!masterComponent.get('totalCount');
      }).mapProperty("componentName");
    }
    if (this.get('isInstallerWizard') || this.get('isAddServiceWizard')) {
      masterBlueprint = this.getCurrentMastersBlueprint();
      var selectedClientComponents = this.get('content.clients').mapProperty('component_name');
      var alreadyInstalledClients = App.get('components.clients').reject(function (c) {
        return selectedClientComponents.contains(c);
      });
      var invisibleComponents = invisibleInstalledMasters.concat(invisibleSlavesAndClients).concat(alreadyInstalledClients);
      var invisibleBlueprint = blueprintUtils.filterByComponents(this.get('content.recommendations'), invisibleComponents);
      masterBlueprint = blueprintUtils.mergeBlueprints(masterBlueprint, invisibleBlueprint);
    } else if (this.get('isAddHostWizard')) {
      masterBlueprint = this.getCurrentMasterSlaveBlueprint();
      hostNames = hostNames.concat(App.Host.find().mapProperty("hostName")).uniq();
      slaveBlueprint = blueprintUtils.addComponentsToBlueprint(slaveBlueprint, invisibleSlavesAndClients);
    }
    return blueprintUtils.mergeBlueprints(masterBlueprint, slaveBlueprint);
  },

  /**
   * Success-callback for validations request
   * @param {object} data
   * @method updateValidationsSuccessCallback
   * @override App.HostComponentRecommendationMixin
   */
  updateValidationsSuccessCallback: function updateValidationsSuccessCallback(data) {
    var clientComponents = App.get('components.clients');

    this.set('generalErrorMessages', []);
    this.set('generalWarningMessages', []);
    this.get('hosts').setEach('warnMessages', []);
    this.get('hosts').setEach('errorMessages', []);
    this.get('hosts').setEach('anyMessage', false);
    this.get('hosts').forEach(function (host) {
      host.checkboxes.setEach('hasWarnMessage', false);
      host.checkboxes.setEach('hasErrorMessage', false);
    });
    var anyGeneralClientErrors = false; // any error/warning for any client component (under "CLIENT" alias)

    var validationData = validationUtils.filterNotInstalledComponents(data);
    validationData.filterProperty('type', 'host-component').filter(function (i) {
      return !(i['component-name'] && App.StackServiceComponent.find().findProperty('componentName', i['component-name']).get('isMaster'));
    }).forEach(function (item) {
      var checkboxWithIssue = null;
      var isGeneralClientValidationItem = clientComponents.contains(item['component-name']); // it is an error/warning for any client component (under "CLIENT" alias)
      var host = this.get('hosts').find(function (h) {
        return h.hostName === item.host && h.checkboxes.some(function (checkbox) {
          var isClientComponent = checkbox.component === "CLIENT" && isGeneralClientValidationItem;
          if (checkbox.component === item['component-name'] || isClientComponent) {
            checkboxWithIssue = checkbox;
            return true;
          }
          return false;
        });
      });
      if (host) {
        Em.set(host, 'anyMessage', true);

        if (item.level === 'ERROR') {
          host.errorMessages.pushObject(item.message);
          Em.set(checkboxWithIssue, 'hasErrorMessage', true);
        } else if (item.level === 'WARN') {
          host.warnMessages.pushObject(item.message);
          Em.set(checkboxWithIssue, 'hasWarnMessage', true);
        }
      } else {
        var componentHeader = this.get('headers').findProperty('name', item['component-name']);
        if (componentHeader && componentHeader.get('isDisabled')) {
          // skip validation messages for components which disabled for editing
          return;
        }
        var component;
        if (isGeneralClientValidationItem) {
          if (!anyGeneralClientErrors) {
            anyGeneralClientErrors = true;
            component = "Client";
          }
        } else {
          component = item['component-name'];
        }

        if (component || !item['component-name']) {
          var details = "";
          if (item.host) {
            details += " (" + item.host + ")";
          }

          if (item.level === 'ERROR') {
            this.get('generalErrorMessages').push(item.message + details);
          } else if (item.level === 'WARN') {
            this.get('generalWarningMessages').push(item.message + details);
          }
        }
      }
    }, this);
  },

  /**
   * Error-callback for validations request
   * @param {object} jqXHR
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @method updateValidationsErrorCallback
   * @override App.HostComponentRecommendationMixin
   */
  updateValidationsErrorCallback: function updateValidationsErrorCallback(jqXHR, ajaxOptions, error, opt) {},

  /**
   * Composes selected values of comboboxes into blueprint format
   */
  getCurrentBlueprint: function getCurrentBlueprint() {
    var clientComponents = this.get('content.clients').mapProperty('component_name');
    var components = this.get('hosts').reduce(function (acc, host) {
      var checked = host.checkboxes.filterProperty('checked', true).mapProperty('component');
      var componentNames = checked.indexOf('CLIENT') > -1 ? checked.without('CLIENT').concat(clientComponents) : checked;
      return acc.concat(componentNames.map(function (componentName) {
        return Em.Object.create({
          componentName: componentName,
          hostName: Em.get(host, 'hostName')
        });
      }));
    }, []);
    return this.getComponentsBlueprint(components);
  },

  /**
   * Create blueprint from assigned master components to appropriate hosts
   * @returns {Object}
   * @method getCurrentMastersBlueprint
   */
  getCurrentMastersBlueprint: function getCurrentMastersBlueprint() {
    var masters = this.get('content.masterComponentHosts');
    var mastersBlueprint = this.getComponentsBlueprint(masters.map(function (component) {
      var c = Em.Object.create(component);
      c.set('componentName', c.get('component'));
      return c;
    }), this.get('hosts').mapProperty('hostName'));
    return blueprintUtils.mergeBlueprints(mastersBlueprint, this.getCurrentSlaveBlueprint());
  },

  /**
   * In case of any validation issues shows accept dialog box for user which allow cancel and fix issues or continue anyway
   * @metohd submit
   */
  showValidationIssuesAcceptBox: function showValidationIssuesAcceptBox(callback) {
    var self = this;

    if (self.get('anyWarnings') || self.get('anyErrors')) {
      App.ModalPopup.show({
        'data-qa': 'validation-issues-modal',
        primary: Em.I18n.t('common.continueAnyway'),
        header: Em.I18n.t('installer.step6.validationIssuesAttention.header'),
        body: Em.I18n.t('installer.step6.validationIssuesAttention'),
        primaryClass: 'btn-danger',
        onPrimary: function onPrimary() {
          this.hide();
          callback();
        },
        onSecondary: function onSecondary() {
          App.set('router.nextBtnClickInProgress', false);
          this._super();
        }
      });
    } else {
      callback();
    }
  }
});

});

require.register("controllers/wizard/step7/assign_master_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var stringUtils = require('utils/string_utils');
var numberUtils = require('utils/number_utils');
var blueprintUtils = require('utils/blueprint');

App.AssignMasterOnStep7Controller = Em.Controller.extend(App.BlueprintMixin, App.AssignMasterComponents, {

  name: "assignMasterOnStep7Controller",

  useServerValidation: false,

  showInstalledMastersFirst: false,

  configWidgetContext: {},

  configActionComponent: {},

  content: function () {
    return this.get('configWidgetContext.controller.content') || {};
  }.property('configWidgetContext.controller.content'),

  popup: null,

  mastersToCreate: [],

  markSavedComponentsAsInstalled: true,

  /**
   * Array of master component names, that should be addable
   * Are used in HA wizards to add components, that are not addable for other wizards
   * @type {Array}
   * @override
   */
  mastersAddableInHA: Em.computed.alias('App.components.isMasterAddableOnlyOnHA'),

  /**
   * Marks component add/delete action to be performed ahead.
   * @param context {Object} Context of the calling function
   * @param action {String} ADD|DELETE
   * @param hostComponent {Object}
   * @public
   * @method {execute}
   */
  execute: function execute(context, action, hostComponent) {
    this.set('configWidgetContext', context);
    this.set('content', context.get('controller.content'));
    this.set('configActionComponent', hostComponent);

    switch (action) {
      case 'ADD':
        this.clearRecommendations();
        if (hostComponent.componentName === "HIVE_SERVER_INTERACTIVE") {
          this.getPendingBatchRequests(hostComponent);
        } else {
          this.showPopup(hostComponent);
        }
        break;
      case 'DELETE':
        this.set('mastersToCreate', [hostComponent.componentName]);
        this.removeMasterComponent();
        break;
    }
  },

  getPendingBatchRequests: function getPendingBatchRequests(hostComponent) {
    var self = this;
    // Send Ajax request to get status of pending batch requests
    App.ajax.send({
      name: 'request_schedule.get.pending',
      sender: self,
      error: 'pendingBatchRequestsAjaxError',
      success: 'pendingBatchRequestsAjaxSuccess',
      data: {
        hostComponent: hostComponent
      }
    });
  },

  pendingBatchRequestsAjaxError: function pendingBatchRequestsAjaxError(data) {
    var error = Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error');
    if (data && data.responseText) {
      try {
        var json = JSON.parse(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error'), error, null);
  },

  pendingBatchRequestsAjaxSuccess: function pendingBatchRequestsAjaxSuccess(data, opt, params) {
    var self = this;
    if (this.shouldShowAlertOnBatchRequest(data)) {
      App.showAlertPopup(Em.I18n.t('services.service.actions.hsi.alertPopup.header'), Em.I18n.t('services.service.actions.hsi.alertPopup.body'), function () {
        var configWidgetContext = self.get('configWidgetContext');
        var config = configWidgetContext.get('config');
        configWidgetContext.toggleProperty('controller.forceUpdateBoundaries');
        var value = config.get('initialValue');
        config.set('value', value);
        configWidgetContext.setValue(value);
        configWidgetContext.sendRequestRorDependentConfigs(config);
        this.hide();
      });
    } else {
      this.showPopup(params.hostComponent);
    }
  },

  shouldShowAlertOnBatchRequest: function shouldShowAlertOnBatchRequest(data) {
    var showAlert = false;
    if (data.hasOwnProperty('items') && data.items.length > 0) {
      data.items.forEach(function (_item) {
        if (_item && _item.RequestSchedule && _item.RequestSchedule.batch && _item.RequestSchedule.batch.batch_requests) {
          _item.RequestSchedule.batch.batch_requests.forEach(function (batchRequest) {
            // Check if a DELETE request on HIVE_SERVER_INTERACTIVE is in progress
            if (batchRequest.request_type === "DELETE" && batchRequest.request_uri.indexOf("HIVE_SERVER_INTERACTIVE") > -1) {
              showAlert = true;
            }
          });
        }
      });
    }
    return showAlert;
  },

  showPopup: function showPopup(hostComponent) {
    var missingDependentServices = this.getAllMissingDependentServices();
    var isNonWizardPage = !this.get('content.controllerName');
    if (missingDependentServices.length && isNonWizardPage) {
      this.showInstallServicesPopup(missingDependentServices);
    } else {
      this.set('mastersToCreate', [hostComponent.componentName]);
      this.showAssignComponentPopup();
    }
  },

  /**
   * Used to set showAddControl/showRemoveControl flag
   * @param componentName
   * @override
   */
  updateComponent: function updateComponent(componentName) {
    this._super(componentName);

    if (!this.get('mastersToCreate').contains(componentName)) {
      this.get("selectedServicesMasters").filterProperty("component_name", componentName).forEach(function (c) {
        c.set('showAddControl', false);
        c.set('showRemoveControl', false);
      });
    }
  },

  /**
   * Assign Master page will be displayed in the popup
   * @private
   * @method
   */
  showAssignComponentPopup: function showAssignComponentPopup() {
    var self = this;
    // Master component hosts should be loaded only when content.controller name is not defined i.e non-wizard pages
    if (!this.get('content.controllerName')) {
      this.loadMasterComponentHosts();
    }
    var mastersToCreate = this.get('mastersToCreate');
    var masterToCreateDisplayName = App.format.role(mastersToCreate[0]);
    var configWidgetContext = this.get('configWidgetContext');
    var config = this.get('configWidgetContext.config');
    var popup = App.ModalPopup.show({
      classNames: ['wizard-modal-wrapper', 'add-service-wizard-modal'],
      modalDialogClasses: ['modal-xlg'],
      header: Em.I18n.t('assign.master.popup.header').format(masterToCreateDisplayName),
      bodyClass: App.AssignMasterOnStep7View.extend({
        controller: self
      }),
      primary: Em.I18n.t('common.select'),
      onPrimary: function onPrimary() {
        self.submit();
      },
      onSecondary: function onSecondary() {
        this.showWarningPopup();
      },
      onClose: function onClose() {
        this.showWarningPopup();
      },
      showWarningPopup: function showWarningPopup() {
        var mainPopupContext = this;
        App.ModalPopup.show({
          encodeBody: false,
          header: Em.I18n.t('common.warning'),
          primaryClass: 'btn-warning',
          body: Em.I18n.t('assign.master.popup.cancel.body').format(masterToCreateDisplayName),
          onPrimary: function onPrimary() {
            configWidgetContext.toggleProperty('controller.forceUpdateBoundaries');
            var value = config.get('initialValue');
            config.set('value', value);
            configWidgetContext.setValue(value);
            configWidgetContext.sendRequestRorDependentConfigs(config);
            this.hide();
            mainPopupContext.hide();
          }
        });
      },
      didInsertElement: function didInsertElement() {
        this._super();
        this.fitHeight();
        self.set('configWidgetContext.controller.saveInProgress', false);
      }
    });
    this.set('popup', popup);
  },

  /**
   * Displays the popup to install required service dependencies for being added component with this config change
   * @param missingDependentServices {String[]}   Array of service display names
   */
  showInstallServicesPopup: function showInstallServicesPopup(missingDependentServices) {
    var displayServices = stringUtils.getFormattedStringFromArray(missingDependentServices);
    var configWidgetContext = this.get('configWidgetContext');
    var config = this.get('configWidgetContext.config');
    var configDisplayName = config.get('displayName').toLowerCase();
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step7.missing.service.header'),
      body: Em.I18n.t('installer.step7.missing.service.body').format(displayServices, configDisplayName),
      primaryClass: 'btn-danger',
      onPrimary: function onPrimary() {
        configWidgetContext.toggleProperty('controller.forceUpdateBoundaries');
        var value = config.get('initialValue');
        config.set('value', value);
        configWidgetContext.setValue(value);
        configWidgetContext.sendRequestRorDependentConfigs(config);
        this._super();
      },
      secondary: null,
      showCloseButton: false,
      didInsertElement: function didInsertElement() {
        this._super();
        configWidgetContext.set('controller.saveInProgress', false);
      }
    });
  },

  /**
   * This method is used while installing or adding a service
   * Removes the masterComponent that was previously being tracked to be added to the cluster
   * @private
   * @method {removeMasterComponent}
   */
  removeMasterComponent: function removeMasterComponent() {
    var componentsToDelete = this.get('mastersToCreate');
    var componentsFromConfigs = this.get('content.componentsFromConfigs');
    if (this.get('content.controllerName')) {
      var masterComponentHosts = this.get('content.masterComponentHosts');
      var recommendationsHostGroups = this.get('content.recommendationsHostGroups');
      componentsToDelete.forEach(function (_componentName) {
        masterComponentHosts = masterComponentHosts.rejectProperty('component', _componentName);
        recommendationsHostGroups.blueprint.host_groups.forEach(function (hostGroup) {
          hostGroup.components = hostGroup.components.rejectProperty('name', _componentName);
        }, this);
        componentsFromConfigs = componentsFromConfigs.without(_componentName);
      }, this);
      this.get('content').set('masterComponentHosts', masterComponentHosts);
      this.set('content.componentsFromConfigs', componentsFromConfigs);
      this.set('content.recommendationsHostGroups', recommendationsHostGroups);
    } else {
      this.clearComponentsToBeAdded(componentsToDelete[0]);
      var hostComponent = App.HostComponent.find().findProperty('componentName', componentsToDelete[0]);
      if (hostComponent) {
        App.set('componentToBeDeleted', Em.Object.create({
          componentName: componentsToDelete[0],
          hostName: hostComponent.get('hostName')
        }));
      }
    }
    var configActionComponent = this.get('configActionComponent');
    this.get('configWidgetContext.config').set('configActionComponent', configActionComponent);
  },

  /**
   * Success callback after loading active host list
   * @override
   * @method loadWizardHostsSuccessCallback
   */
  loadWizardHostsSuccessCallback: function loadWizardHostsSuccessCallback(data) {
    var parentController = this.get('content.controllerName');
    if (parentController) {
      this._super(data);
    } else {
      var result = [];
      data.items.forEach(function (host) {
        var hostName = host.Hosts.host_name,
            cpu = host.Hosts.cpu_count,
            memory = host.Hosts.total_mem.toFixed(2);
        result.push(Em.Object.create({
          host_name: hostName,
          cpu: cpu,
          memory: memory,
          disk_info: host.Hosts.disk_info,
          maintenance_state: host.Hosts.maintenance_state,
          host_info: Em.I18n.t('installer.step5.hostInfo').fmt(hostName, numberUtils.bytesToSize(memory, 1, 'parseFloat', 1024), cpu)
        }));
      }, this);
      this.set('hosts', result);
      this.sortHosts(this.get('hosts'));
      this.set('isHostsLoaded', true);
    }
  },

  /**
   *  This method is called on Service->config page and is responsible to load the "Assign Master popup"
   *  with the installed master component hosts.
   * @private
   * @method {loadMasterComponentHosts}
   */
  loadMasterComponentHosts: function loadMasterComponentHosts() {
    var stackMasterComponents = App.get('components.masters').uniq();
    var masterComponentHosts = [];
    App.HostComponent.find().filter(function (component) {
      return stackMasterComponents.contains(component.get('componentName'));
    }).forEach(function (item) {
      masterComponentHosts.push({
        component: item.get('componentName'),
        hostName: item.get('hostName'),
        isInstalled: true,
        serviceId: item.get('service.id'),
        display_name: item.get('displayName')
      });
    });
    this.set("masterComponentHosts", masterComponentHosts);
  },

  /**
   * Returns array of dependent services that are yet not installed in the cluster
   * @private
   * @method getAllMissingDependentServices
   * @return  missingDependentServices {Array}
   */
  getAllMissingDependentServices: function getAllMissingDependentServices() {
    var configActionComponentName = this.get('configActionComponent').componentName;
    var componentStackService = App.StackServiceComponent.find(configActionComponentName).get('stackService');
    var missing = [];
    componentStackService.collectMissingDependencies(this.installedStackServices(), App.StackService.find(), missing);
    return missing.mapProperty('displayName');
  },

  installedStackServices: function installedStackServices() {
    return App.Service.find().map(function (each) {
      return App.StackService.find(each.get('serviceName'));
    });
  },

  /**
   * This method saves masterComponent layout that is used on subsequent "Review" and "Install start and Test services" pages.
   * @private
   * @method {saveMasterComponentHosts}
   */
  saveMasterComponentHosts: function saveMasterComponentHosts() {
    var controller = App.router.get(this.get('content.controllerName'));
    var componentsFromConfigs = this.get('content.componentsFromConfigs');
    controller.saveMasterComponentHosts(this, true);
    controller.loadMasterComponentHosts(true);
    componentsFromConfigs = componentsFromConfigs.concat(this.get('mastersToCreate'));
    this.set('content.componentsFromConfigs', componentsFromConfigs);
  },

  /**
   * This method saves host group layout that is used for blueprint validation call made while transitioning to "Review" page.
   * @private
   * @method {saveRecommendationsHostGroups}
   */
  saveRecommendationsHostGroups: function saveRecommendationsHostGroups() {
    var recommendationsHostGroups = this.get('content.recommendationsHostGroups');
    var mastersToCreate = this.get('mastersToCreate');
    mastersToCreate.forEach(function (componentName) {
      var hostName = this.getSelectedHostNames(componentName)[0];
      if (hostName && recommendationsHostGroups) {
        var hostGroups = recommendationsHostGroups.blueprint_cluster_binding.host_groups;
        var isHostPresent = false;
        var i = 0;
        while (i < hostGroups.length) {
          var hosts = hostGroups[i].hosts;
          isHostPresent = hosts.someProperty('fqdn', hostName);
          if (isHostPresent) break;
          i++;
        }
        if (isHostPresent) {
          var hostGroupName = hostGroups[i].name;
          var hostGroup = recommendationsHostGroups.blueprint.host_groups.findProperty('name', hostGroupName);
          var addHostComponentInGroup = !hostGroup.components.someProperty('name', componentName);
          if (addHostComponentInGroup) {
            hostGroup.components.pushObject({ name: componentName });
          }
        }
      }
    }, this);
    this.set('content.recommendationsHostGroups', recommendationsHostGroups);
  },

  /**
   * Get the fqdn hostnames as selected by the user for the component.
   * @param componentName
   * @return {String[]}
   */
  getSelectedHostNames: function getSelectedHostNames(componentName) {
    var selectedServicesMasters = this.get('selectedServicesMasters');
    return selectedServicesMasters.filterProperty('component_name', componentName).mapProperty('selectedHost');
  },

  /**
   * set App.componentToBeAdded to use it on subsequent validation call while saving configuration
   * @param componentName {String}
   * @param hostNames {String[]}
   * @method {setGlobalComponentToBeAdded}
   */
  setGlobalComponentToBeAdded: function setGlobalComponentToBeAdded(componentName, hostNames) {
    var componentToBeAdded = Em.Object.create({
      componentName: componentName,
      hostNames: hostNames
    });
    App.set('componentToBeAdded', componentToBeAdded);
  },

  /**
   * clear 'componentToBeDeleted' object
   * @param componentName {String}
   * @public
   * @method {clearComponentsToBeDeleted}
   */
  clearComponentsToBeDeleted: function clearComponentsToBeDeleted(componentName) {
    var componentsToBeDeleted = App.get('componentToBeDeleted');
    if (!App.isEmptyObject(componentsToBeDeleted) && componentsToBeDeleted.get('componentName') === componentName) {
      App.set('componentToBeDeleted', {});
    }
  },

  /**
   * clear 'componentToBeAdded' object
   * @param componentName  {String}
   */
  clearComponentsToBeAdded: function clearComponentsToBeAdded(componentName) {
    var componentsToBeAdded = App.get('componentToBeAdded');
    if (!App.isEmptyObject(componentsToBeAdded) && componentsToBeAdded.get('componentName') === componentName) {
      App.set('componentToBeAdded', {});
    }
  },

  /**
   * Submit button click handler
   * @method submit
   */
  submit: function submit() {
    var self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      self.get('popup').hide();
      var context = self.get('configWidgetContext');
      context.toggleProperty('controller.forceUpdateBoundaries');
      var configActionComponent = self.get('configActionComponent');
      var componentHostNames = self.getSelectedHostNames(configActionComponent.componentName);
      var config = self.get('configWidgetContext.config');

      // TODO remove after stack advisor is able to handle this case
      // workaround for hadoop.proxyuser.{{hiveUser}}.hosts after adding Hive Server Interactive from Install Wizard
      var serviceConfigs = context.get('controller.stepConfigs').findProperty('serviceName', context.get('controller.selectedService.serviceName')).get('configs');
      var dependencies = context.get('config.configAction.dependencies');

      if (self.get('content.controllerName')) {
        self.saveMasterComponentHosts();
        self.saveRecommendationsHostGroups();

        // TODO remove after stack advisor is able to handle this case
        // workaround for hadoop.proxyuser.{{hiveUser}}.hosts after adding Hive Server Interactive from Install Wizard
        var miscConfigs = context.get('controller.stepConfigs').findProperty('serviceName', 'MISC').get('configs');
        serviceConfigs = serviceConfigs.concat(miscConfigs);
      } else {
        self.setGlobalComponentToBeAdded(configActionComponent.componentName, componentHostNames);
        self.clearComponentsToBeDeleted(configActionComponent.componentName);
      }

      configActionComponent.hostNames = componentHostNames;
      config.set('configActionComponent', configActionComponent);

      var oldValueKey = context.get('controller.wizardController.name') === 'installerController' ? 'initialValue' : 'savedValue';
      context.get('controller').loadConfigRecommendations([{
        type: App.config.getConfigTagFromFileName(config.get('fileName')),
        name: config.get('name'),
        old_value: config.get(oldValueKey)
      }]);
    });
  },

  /**
   *
   * @param {Em.Object} context
   * @param {object} blueprintObject
   */
  saveRecommendations: function saveRecommendations(context, blueprintObject) {
    var oldValueKey = context.get('controller.wizardController.name') === 'installerController' ? 'initialValue' : 'savedValue';
    var config = this.get('configWidgetContext.config');

    context.get('controller').loadRecommendationsSuccess({
      resources: [{
        recommendations: {
          blueprint: {
            configurations: blueprintObject
          }
        }
      }]
    }, null, {
      dataToSend: {
        changed_configurations: [{
          type: App.config.getConfigTagFromFileName(config.get('fileName')),
          name: config.get('name'),
          old_value: config.get(oldValueKey)
        }]
      }
    });
  },

  /**
   *
   * @param dependencies
   * @param serviceConfigs
   * @returns {{}}
   */
  getDependenciesForeignKeys: function getDependenciesForeignKeys(dependencies, serviceConfigs) {
    var foreignKeys = {};
    if (dependencies.foreignKeys) {
      dependencies.foreignKeys.forEach(function (dependency) {
        var matchingProperty = serviceConfigs.find(function (property) {
          return property.get('filename') === App.config.getOriginalFileName(dependency.fileName) && property.get('name') === dependency.propertyName;
        });
        if (matchingProperty) {
          foreignKeys[dependency.key] = matchingProperty.get('value');
        }
      });
    }
    return foreignKeys;
  },

  /**
   *
   * @param dependencies
   * @param context
   * @returns {Array}
   */
  getMasterComponents: function getMasterComponents(dependencies, context) {
    var masterComponents = [];
    if (this.get('content.controllerName')) {
      var savedMasterComponents = context.get('controller.content.masterComponentHosts').filter(function (componentObject) {
        return dependencies.initializer.componentNames.contains(componentObject.component);
      });
      masterComponents = savedMasterComponents.map(function (componentObject) {
        var masterComponent = Em.getProperties(componentObject, ['component', 'hostName']);
        masterComponent.isInstalled = true;
        return masterComponent;
      });
    } else {
      var hostsMap = blueprintUtils.getComponentForHosts();
      Em.keys(hostsMap).forEach(function (hostName) {
        hostsMap[hostName].forEach(function (componentName) {
          if (dependencies.initializer.componentNames.contains(componentName)) {
            masterComponents.push({
              component: componentName,
              hostName: hostName,
              isInstalled: true
            });
          }
        });
      });
    }
    return masterComponents;
  },

  getHosts: function getHosts() {
    var result,
        parentController = this.get('content.controllerName');
    if (parentController) {
      result = this._super();
    } else {
      result = this.get('hosts').mapProperty('host_name');
    }
    return result;
  },

  clearStepOnExit: Em.K
});

});

require.register("controllers/wizard/step7/pre_install_checks_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @class PreInstallChecksController
 * @type {Em.Controller}
 */
App.PreInstallChecksController = Em.Controller.extend({

  name: 'preInstallChecksController',

  /**
   * Detects when pre install checks where run
   *
   * @type {boolean}
   */
  preInstallChecksWhereRun: false,

  /**
   * Set initial settings
   *
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.set('preInstallChecksWhereRun', false);
  },

  /**
   * Show warning-popup for user that pre install checks where not run
   * User may skip run (primary) or execute them (secondary)
   *
   * @param {function} afterChecksCallback function called on primary-click
   * @returns {App.ModalPopup}
   * @method notRunChecksWarnPopup
   */
  notRunChecksWarnPopup: function notRunChecksWarnPopup(afterChecksCallback) {
    Em.assert('`afterChecksCallback` should be a function, you have provided ' + Em.typeOf(afterChecksCallback), 'function' === Em.typeOf(afterChecksCallback));
    var self = this;
    return App.ModalPopup.show({

      header: Em.I18n.t('installer.step7.preInstallChecks.notRunChecksWarnPopup.header'),
      body: Em.I18n.t('installer.step7.preInstallChecks.notRunChecksWarnPopup.body'),
      primary: Em.I18n.t('installer.step7.preInstallChecks.notRunChecksWarnPopup.primary'),
      secondary: Em.I18n.t('installer.step7.preInstallChecks.notRunChecksWarnPopup.secondary'),

      onPrimary: function onPrimary() {
        this._super();
        afterChecksCallback();
      },

      onSecondary: function onSecondary() {
        this._super();
        self.runPreInstallChecks();
      }

    });
  },

  /**
   * Run pre install checks
   *
   * @method runPreInstallChecks
   */
  runPreInstallChecks: function runPreInstallChecks() {
    this.set('preInstallChecksWhereRun', true);
    return App.ModalPopup.show({

      header: Em.I18n.t('installer.step7.preInstallChecks.checksPopup.header'),
      secondary: '',

      body: Em.View.extend({})

    });
  }

});

});

require.register("controllers/wizard/step7_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * By Step 7, we have the following information stored in App.db and set on this
 * controller by the router.
 *
 *   selectedServices: App.db.selectedServices (the services that the user selected in Step 4)
 *   masterComponentHosts: App.db.masterComponentHosts (master-components-to-hosts mapping the user selected in Step 5)
 *   slaveComponentHosts: App.db.slaveComponentHosts (slave-components-to-hosts mapping the user selected in Step 6)
 *
 */

/**
 * @typedef {object} masterComponentHost
 * @property {string} component
 * @property {string} hostName
 * @property {boolean} isInstalled is component already installed on the this host or just going to be installed
 */

/**
 * @typedef {object} topologyLocalDB
 * @property {object[]} hosts list of hosts with information of their disks usage and dirs
 * @property {masterComponentHost[]} masterComponentHosts
 * @property {?object[]} slaveComponentHosts
 */

App.WizardStep7Controller = Em.Controller.extend(App.ServerValidatorMixin, App.EnhancedConfigsMixin, App.ToggleIsRequiredMixin, App.GroupsMappingMixin, App.AddSecurityConfigs, App.KDCCredentialsControllerMixin, {

  name: 'wizardStep7Controller',

  /**
   * Contains all field properties that are viewed in this step
   * @type {object[]}
   */
  stepConfigs: [],

  hash: null,

  selectedService: null,

  addMiscTabToPage: true,

  /**
   * Is Submit-click processing now
   * @type {bool}
   */
  submitButtonClicked: false,

  isRecommendedLoaded: false,

  /**
   * Indicates if all stepConfig objects are created
   */
  stepConfigsCreated: false,

  /**
   * Define state of next button on credentials tab
   */
  credentialsTabNextEnabled: false,

  /**
   * Define state of next button on databases tab
   */
  databasesTabNextEnabled: false,

  /**
   * used in services_config.js view to mark a config with security icon
   */
  secureConfigs: require('data/configs/wizards/secure_mapping'),

  /**
   * If configChangeObserver Modal is shown
   * @type {bool}
   */
  miscModalVisible: false,

  overrideToAdd: null,

  isInstaller: true, //todo: refactor using of this property

  /**
   * Is installer controller used
   * @type {bool}
   */
  isInstallWizard: Em.computed.equal('content.controllerName', 'installerController'),

  /**
   * Is add service controller used
   * @type {bool}
   */
  isAddServiceWizard: Em.computed.equal('content.controllerName', 'addServiceController'),

  /**
   * List of config groups
   * @type {object[]}
   */
  configGroups: [],

  /**
   * List of config group to be deleted
   * @type {object[]}
   */
  groupsToDelete: [],

  preSelectedConfigGroup: null,

  /**
   * Currently selected config group
   * @type {object}
   */
  selectedConfigGroup: null,

  /**
   * Config tags of actually installed services
   * @type {array}
   */
  serviceConfigTags: [],

  /**
   * Are applied to service configs loaded
   * @type {bool}
   */
  isAppliedConfigLoaded: true,

  /**
   * Is there validation issues
   * @type {bool}
   */
  hasErrors: false,

  /**
   * Total number of recommendation and validation issues
   * @type {number}
   */
  issuesCounter: 0,

  /**
   * Number of ui-side validation issues
   * @type {number}
   */
  validationsCounter: 0,

  /**
   * Tab objects to represent each config category tab
   */
  tabs: [],

  isConfigsLoaded: Em.computed.and('wizardController.stackConfigsLoaded', 'isAppliedConfigLoaded'),

  transitionInProgress: Em.computed.alias('App.router.btnClickInProgress'),

  /**
   * PreInstall Checks allowed only for Install
   * @type {boolean}
   */
  supportsPreInstallChecks: function () {
    return App.get('supports.preInstallChecks') && this.get('isInstallWizard');
  }.property('App.supports.preInstallChecks', 'wizardController.name'),

  /**
   * Number of errors in the configs in the selected service
   * @type {number}
   */
  errorsCount: function () {
    return this.get('selectedService.configsWithErrors').filter(function (c) {
      return Em.isNone(c.get('isInDefaultTheme'));
    }).length;
  }.property('selectedService.configsWithErrors.length'),

  /**
   * Should Next-button be disabled
   * @type {bool}
   */
  isSubmitDisabled: function () {
    if (!this.get('stepConfigs.length')) return true;
    if (this.get('submitButtonClicked')) return true;
    if (App.get('router.btnClickInProgress')) return true;
    return !this.get('stepConfigs').filterProperty('showConfig', true).everyProperty('errorCount', 0) || this.get("miscModalVisible") || !!this.get('configErrorList.criticalIssues.length');
  }.property('stepConfigs.@each.errorCount', 'miscModalVisible', 'submitButtonClicked', 'App.router.btnClickInProgress', 'configErrorList.criticalIssues.length'),

  /**
   * List of selected to install service names
   * @type {string[]}
   */
  selectedServiceNames: function () {
    return this.get('content.services').filterProperty('isSelected', true).filterProperty('isInstalled', false).mapProperty('serviceName');
  }.property('content.services', 'content.services.@each.isSelected', 'content.services.@each.isInstalled', 'content.stacks.@each.isSelected').cacheable(),

  /**
   * List of installed and selected to install service names
   * @type {string[]}
   */
  allSelectedServiceNames: function () {
    return this.get('content.services').filter(function (service) {
      return service.get('isInstalled') || service.get('isSelected');
    }).mapProperty('serviceName');
  }.property('content.services', 'content.services.@each.isSelected', 'content.services.@each.isInstalled', 'content.stacks.@each.isSelected').cacheable(),

  /**
   * List of installed service names
   * @type {string[]}
   */
  installedServiceNames: function () {
    var serviceNames = this.get('content.services').filterProperty('isInstalled').mapProperty('serviceName');
    if (!this.get('isInstallWizard')) {
      serviceNames = serviceNames.filter(function (_serviceName) {
        return !App.get('services.noConfigTypes').contains(_serviceName);
      });
    }
    return serviceNames;
  }.property('content.services').cacheable(),

  installedServices: function () {
    var self = this;
    return App.StackService.find().toArray().toMapByCallback('serviceName', function (item) {
      return self.get('installedServiceNames').contains(item.get('serviceName'));
    });
  }.property('installedServiceNames.length'),

  requiredChanges: function () {
    return this.get('recommendations').filterProperty('isEditable', false);
  }.property('recommendatios.@each.isEditable'),

  /**
   * List of master components
   * @type {Ember.Enumerable}
   */
  masterComponentHosts: Em.computed.alias('content.masterComponentHosts'),

  /**
   * List of slave components
   * @type {Ember.Enumerable}
   */
  slaveComponentHosts: Em.computed.alias('content.slaveGroupProperties'),

  /**
   * Filter text will be located here
   * @type {string}
   */
  filter: '',

  /**
   * Defines if configs can be editable
   * @type {boolean}
   */
  canEdit: true,

  /**
   * list of dependencies that are user to set init value of config
   *
   * @type {Object}
   */
  configDependencies: function () {
    var dependencies = {};
    var hiveMetastore = App.configsCollection.getConfigByName('hive.metastore.uris', 'hive-site.xml');
    var clientPort = App.configsCollection.getConfigByName('clientPort', 'zoo.cfg.xml');
    var atlasTls = App.configsCollection.getConfigByName('atlas.enableTLS', 'application-properties.xml');
    var atlasHttpPort = App.configsCollection.getConfigByName('atlas.server.http.port', 'application-properties.xml');
    var atlasHttpsPort = App.configsCollection.getConfigByName('atlas.server.https.port', 'application-properties.xml');

    if (hiveMetastore) dependencies['hive.metastore.uris'] = hiveMetastore.recommendedValue;
    if (clientPort) dependencies.clientPort = clientPort.recommendedValue;
    if (atlasTls) dependencies['atlas.enableTLS'] = atlasTls.recommendedValue;
    if (atlasHttpPort) dependencies['atlas.server.http.port'] = atlasHttpPort.recommendedValue;
    if (atlasHttpsPort) dependencies['atlas.server.https.port'] = atlasHttpsPort.recommendedValue;
    return dependencies;
  }.property(),

  /**
   * List of filters for config properties to populate filter combobox
   */
  propertyFilters: [{
    attributeName: 'isOverridden',
    attributeValue: true,
    caption: 'common.combobox.dropdown.overridden'
  }, {
    attributeName: 'isFinal',
    attributeValue: true,
    caption: 'common.combobox.dropdown.final'
  }, {
    attributeName: 'hasIssues',
    attributeValue: true,
    caption: 'common.combobox.dropdown.issues'
  }],

  issuesFilterText: function () {
    return !this.get('transitionInProgress') && this.get('issuesFilterSelected') ? Em.I18n.t('installer.step7.showingPropertiesWithIssues') : '';
  }.property('isSubmitDisabled', 'submitButtonClicked', 'filterColumns.@each.selected'),

  /**
   * @type {string}
   */
  issuesFilterLinkText: function () {
    var issuesAttrSelected = this.get('issuesFilterSelected');
    if (!this.get('transitionInProgress')) {
      if (issuesAttrSelected) {
        return Em.I18n.t('installer.step7.showAllProperties');
      }
      if (this.get('hasStepConfigIssues')) {
        return Em.I18n.t('installer.step7.showPropertiesWithIssues');
      }
    }
    return '';
  }.property('isSubmitDisabled', 'submitButtonClicked', 'filterColumns.@each.selected'),

  /**
   * Dropdown menu items in filter combobox
   */
  filterColumns: function () {
    return this.get('propertyFilters').map(function (filter) {
      return Ember.Object.create({
        attributeName: filter.attributeName,
        attributeValue: filter.attributeValue,
        name: this.t(filter.caption),
        selected: false
      });
    }, this);
  }.property('propertyFilters'),

  /**
   * Clear controller's properties:
   *  <ul>
   *    <li>stepConfigs</li>
   *    <li>filter</li>
   *  </ul>
   *  and deselect all <code>filterColumns</code>
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.abortRequests();
    this.setProperties({
      configValidationGlobalMessage: [],
      submitButtonClicked: false,
      isSubmitDisabled: true,
      isRecommendedLoaded: false,
      stepConfigsCreated: false,
      initialRecommendations: []
    });
    App.ServiceConfigGroup.find().filterProperty('isDefault', false).forEach(function (record) {
      App.configGroupsMapper.deleteRecord(record);
    });
    this.get('stepConfigs').clear();
    this.set('filter', '');
    this.get('filterColumns').setEach('selected', false);
  },

  clearLastSelectedService: function clearLastSelectedService() {
    this.get('tabs').filterProperty('selectedServiceName').setEach('selectedServiceName', null);
  },

  /**
   * Generate "finger-print" for current <code>stepConfigs[0]</code>
   * Used to determine, if user has some unsaved changes (comparing with <code>hash</code>)
   * @returns {string|null}
   * @method getHash
   */
  getHash: function getHash() {
    if (!this.get('stepConfigs')[0]) {
      return null;
    }
    var hash = {};
    this.get('stepConfigs').forEach(function (stepConfig) {
      stepConfig.configs.forEach(function (config) {
        hash[config.get('name')] = { value: config.get('value'), overrides: [], isFinal: config.get('isFinal') };
        if (!config.get('overrides')) return;
        if (!config.get('overrides.length')) return;

        config.get('overrides').forEach(function (override) {
          hash[config.get('name')].overrides.push(override.get('value'));
        });
      });
    });
    return JSON.stringify(hash);
  },

  /**
  * Are some changes available
  */
  hasChanges: function hasChanges() {
    return this.get('hash') !== this.getHash();
  },

  /**
   * load all overrides for all config groups
   *
   * @returns {*|$.ajax}
   */
  loadOverrides: function loadOverrides() {
    return App.ajax.send({
      name: 'service.serviceConfigVersions.get.current.not.default',
      sender: this,
      success: 'parseOverrides'
    });
  },

  /**
   * add overriden configs to stepConfigs
   *
   * @param data
   */
  parseOverrides: function parseOverrides(data) {
    var self = this;
    data.items.forEach(function (group) {
      var stepConfig = self.get('stepConfigs').findProperty('serviceName', group.service_name),
          serviceConfigs = stepConfig.get('configs'),
          configGroup = App.ServiceConfigGroup.find().filterProperty('serviceName', group.service_name).findProperty('name', group.group_name);

      var isEditable = self.get('canEdit') && configGroup.get('name') === stepConfig.get('selectedConfigGroup.name');

      group.configurations.forEach(function (config) {
        for (var prop in config.properties) {
          var fileName = App.config.getOriginalFileName(config.type);
          var serviceConfig = serviceConfigs.filterProperty('name', prop).findProperty('filename', fileName);
          if (serviceConfig && serviceConfig.get('isOriginalSCP')) {
            var value = App.config.formatPropertyValue(serviceConfig, config.properties[prop]);
            var isFinal = !!(config.properties_attributes && config.properties_attributes.final && config.properties_attributes.final[prop]);

            App.config.createOverride(serviceConfig, {
              "value": value,
              "savedValue": value,
              "isFinal": isFinal,
              "savedIsFinal": isFinal,
              "isEditable": isEditable
            }, configGroup);
          } else {

            serviceConfigs.push(App.config.createCustomGroupConfig({
              propertyName: prop,
              filename: fileName,
              value: config.properties[prop],
              savedValue: config.properties[prop],
              isEditable: isEditable
            }, configGroup));
          }
        }
      });
    });
    this.onLoadOverrides();
  },

  onLoadOverrides: function onLoadOverrides() {
    this.get('stepConfigs').forEach(function (stepConfig) {
      stepConfig.set('configGroups', App.ServiceConfigGroup.find().filterProperty('serviceName', stepConfig.get('serviceName')));

      stepConfig.get('configGroups').filterProperty('isDefault', false).forEach(function (configGroup) {
        configGroup.set('hash', this.get('wizardController').getConfigGroupHash(configGroup));
      }, this);

      this.addOverride(stepConfig);
      // override if a property isRequired or not
      this.overrideConfigIsRequired(stepConfig);
    }, this);
    console.timeEnd('loadConfigGroups execution time: ');
  },

  /**
   * Set <code>isEditable</code>-property to <code>serviceConfigProperty</code>
   * Based on user's permissions and selected config group
   * @param {Ember.Object} serviceConfigProperty
   * @param {bool} defaultGroupSelected
   * @returns {Ember.Object} Updated config-object
   * @method _updateIsEditableFlagForConfig
   */
  _updateIsEditableFlagForConfig: function _updateIsEditableFlagForConfig(serviceConfigProperty, defaultGroupSelected) {
    if (App.isAuthorized('AMBARI.ADD_DELETE_CLUSTERS')) {
      if (App.get('isKerberosEnabled') && serviceConfigProperty.get('isConfigIdentity') && !App.StackService.find().filterProperty('isSelected').mapProperty('serviceName').contains(Em.get(serviceConfigProperty, 'serviceName'))) {
        serviceConfigProperty.set('isEditable', false);
      } else if (defaultGroupSelected && !Em.get(serviceConfigProperty, 'group')) {
        if (serviceConfigProperty.get('serviceName') === 'MISC') {
          var service = App.config.get('serviceByConfigTypeMap')[App.config.getConfigTagFromFileName(serviceConfigProperty.get('filename'))];
          serviceConfigProperty.set('isEditable', service && !this.get('installedServiceNames').contains(service.get('serviceName')));
        } else {
          serviceConfigProperty.set('isEditable', serviceConfigProperty.get('isEditable') && serviceConfigProperty.get('isReconfigurable'));
        }
      } else if (!(Em.get(serviceConfigProperty, 'group') && Em.get(serviceConfigProperty, 'group.name') === this.get('selectedConfigGroup.name'))) {
        serviceConfigProperty.set('isEditable', false);
      }
    } else {
      serviceConfigProperty.set('isEditable', false);
    }
    return serviceConfigProperty;
  },

  /**
   * Set configs with overrides, recommended defaults to stepConfig
   * @param {Ember.Object} stepConfig
   * @method loadComponentConfigs
   */
  addOverride: function addOverride(stepConfig) {
    var overrideToAdd = this.get('overrideToAdd');
    if (overrideToAdd) {
      overrideToAdd = stepConfig.get('configs').filterProperty('filename', overrideToAdd.filename).findProperty('name', overrideToAdd.name);
      if (overrideToAdd) {
        App.config.createOverride(overrideToAdd, { isEditable: true }, stepConfig.get('selectedConfigGroup'));
        this.set('overrideToAdd', null);
      }
    }
  },

  /**
   * On load function
   * @method loadStep
   */
  loadStep: function loadStep() {
    if (!this.get('isConfigsLoaded')) {
      return;
    }
    console.time('wizard loadStep: ');
    this.clearStep();

    var self = this;
    App.config.setPreDefinedServiceConfigs(this.get('addMiscTabToPage'));

    var storedConfigs = this.get('content.serviceConfigProperties');
    var configs = storedConfigs && storedConfigs.length ? storedConfigs : App.configsCollection.getAll();

    this.set('groupsToDelete', this.get('wizardController').getDBProperty('groupsToDelete') || []);
    if (this.get('wizardController.name') === 'addServiceController' && !storedConfigs) {
      App.router.get('configurationController').getConfigsByTags(this.get('serviceConfigTags')).done(function (loadedConfigs) {
        configs = self.setInstalledServiceConfigs(configs, loadedConfigs, self.get('installedServiceNames'));
        self.applyServicesConfigs(configs);
      });
    } else {
      this.applyServicesConfigs(configs, true);
    }
  },

  /**
   * Update hawq configuration depending on the state of the cluster
   * @param {Array} configs
   */
  updateHawqConfigs: function updateHawqConfigs(configs) {
    if (this.get('wizardController.name') === 'addServiceController') {
      if (App.get('isHaEnabled')) this.addHawqConfigsOnNnHa(configs);
      if (App.get('isRMHaEnabled')) this.addHawqConfigsOnRMHa(configs);
    }
    if (this.get('content.hosts') && Object.keys(this.get('content.hosts')).length === 1) this.removeHawqStandbyHostAddressConfig(configs);
    return configs;
  },

  /**
   * Remove hawq_standby_address_host config from HAWQ configs
   * @param {Array} configs
   */
  removeHawqStandbyHostAddressConfig: function removeHawqStandbyHostAddressConfig(configs) {
    var hawqStandbyAddressHostIndex = configs.indexOf(configs.findProperty('name', 'hawq_standby_address_host'));
    if (hawqStandbyAddressHostIndex > -1) configs.removeAt(hawqStandbyAddressHostIndex);
    return configs;
  },

  applyServicesConfigs: function applyServicesConfigs(configs, isRestoring) {
    var self = this;
    console.time('applyServicesConfigs execution time: ');
    if (!this.get('installedServiceNames').contains('HAWQ') && this.get('allSelectedServiceNames').contains('HAWQ')) {
      this.updateHawqConfigs(configs);
    }
    if (App.get('isKerberosEnabled') && this.get('wizardController.name') === 'addServiceController') {
      this.addKerberosDescriptorConfigs(configs, this.get('wizardController.kerberosDescriptorConfigs') || [], isRestoring);
    }
    var stepConfigs = this.createStepConfigs();
    var serviceConfigs = this.renderConfigs(stepConfigs, configs);
    this.addUidAndGidRepresentations(serviceConfigs);
    // if HA is enabled -> Make some reconfigurations
    if (this.get('wizardController.name') === 'addServiceController') {
      this.updateComponentActionConfigs(configs, serviceConfigs);
      if (App.get('isHaEnabled')) {
        serviceConfigs = this._reconfigureServicesOnNnHa(serviceConfigs);
      }
    }

    var rangerService = App.StackService.find().findProperty('serviceName', 'RANGER');
    var isRangerServicePresent = rangerService && (rangerService.get('isInstalled') || rangerService.get('isSelected'));
    var infraSolrService = App.StackService.find().findProperty('serviceName', 'AMBARI_INFRA_SOLR');
    var isInfraSolrPresent = infraSolrService && (infraSolrService.get('isInstalled') || infraSolrService.get('isSelected'));
    if (isRangerServicePresent && (this.get('wizardController.name') === 'installerController' || this.get('wizardController.name') === 'addServiceController')) {
      this.setRangerPluginsEnabled(serviceConfigs);
      if (isInfraSolrPresent) {
        this.setSolrCloudOn(serviceConfigs);
      }
    }

    this.set('stepConfigs', serviceConfigs);
    this.set('stepConfigsCreated', true);
    this.updateConfigAttributesFromThemes();
    this.checkHostOverrideInstaller();
    this.selectProperService();
    var isInstallerWizard = this.get('isInstallWizard');
    var isRangerServiceAbsent = rangerService && !rangerService.get('isInstalled') && !rangerService.get('isSelected');
    if (isRangerServiceAbsent) {
      var isExternalRangerSetup;
      if (isInstallerWizard) {
        isExternalRangerSetup = configs.filterProperty('fileName', 'cluster-env.xml').findProperty('name', 'enable_external_ranger');
        if (Em.isNone(isExternalRangerSetup) || isExternalRangerSetup.value !== "true") {
          App.config.removeRangerConfigs(this.get('stepConfigs'));
        }
        console.timeEnd('applyServicesConfigs execution time: ');
        console.time('loadConfigRecommendations execution time: ');
        self.loadConfigRecommendations(null, self.completeConfigLoading.bind(self));
      } else {
        var mainController = App.get('router.mainController');
        var clusterController = App.get('router.clusterController');
        mainController.isLoading.call(clusterController, 'clusterEnv').done(function () {
          isExternalRangerSetup = clusterController.get("clusterEnv")["properties"]["enable_external_ranger"];
          if (isExternalRangerSetup !== "true") {
            App.config.removeRangerConfigs(self.get('stepConfigs'));
          }
          console.timeEnd('applyServicesConfigs execution time: ');
          console.time('loadConfigRecommendations execution time: ');
          self.loadConfigRecommendations(null, self.completeConfigLoading.bind(self));
        });
      }
    } else {
      console.timeEnd('applyServicesConfigs execution time: ');
      console.time('loadConfigRecommendations execution time: ');
      self.loadConfigRecommendations(null, self.completeConfigLoading.bind(self));
    }
  },

  /**
  * Sets the value of ranger-<service_name>-plugin-enabled to "Yes" if ranger authorization /
  * is supported for that service.
  * @param stepConfigs Object[]
  */
  setRangerPluginsEnabled: function setRangerPluginsEnabled(stepConfigs) {
    var rangerServiceConfigs = stepConfigs.findProperty('serviceName', 'RANGER').get('configs');
    var services = this.get('selectedServiceNames').filter(function (service) {
      return service != 'RANGER';
    });

    services.forEach(function (serviceName) {
      var pluginEnabledPropertyName = 'ranger-' + serviceName.toLowerCase() + '-plugin-enabled';
      var pluginEnabledProperty = rangerServiceConfigs.findProperty('name', pluginEnabledPropertyName);
      //Kafka and Storm plugins need to be enabled only if cluster is kerberized
      if (pluginEnabledProperty && (serviceName === 'STORM' || serviceName === 'KAFKA')) {
        if (App.get('isKerberosEnabled')) {
          Em.set(pluginEnabledProperty, 'value', 'Yes');
        }
      } else if (pluginEnabledProperty) {
        Em.set(pluginEnabledProperty, 'value', 'Yes');
      }
    });
  },

  setSolrCloudOn: function setSolrCloudOn(stepConfigs) {
    var rangerServiceConfigs = stepConfigs.findProperty('serviceName', 'RANGER').get('configs');
    var solrCloudEnabledProperty = rangerServiceConfigs.findProperty('name', 'is_solrCloud_enabled');
    Em.set(solrCloudEnabledProperty, 'value', 'true');
  },

  /**
   *
   * Makes installed service's configs resulting into component actions (add/delete) non editable on Add Service Wizard
   * @param configs Object[]
   * @param  stepConfigs Object[]
   * @private
   * @method updateComponentActionConfigs
   */
  updateComponentActionConfigs: function updateComponentActionConfigs(configs, stepConfigs) {
    App.ConfigAction.find().forEach(function (item) {
      var configName = item.get('configName');
      var fileName = item.get('fileName');
      var config = configs.filterProperty('filename', fileName).findProperty('name', configName);
      if (config) {
        var isServiceInstalled = App.Service.find().findProperty('serviceName', config.serviceName);
        // service already installed or is being added in add service wizard
        if (isServiceInstalled || stepConfigs.someProperty("serviceName", config.serviceName)) {
          var serviceConfigs = stepConfigs.findProperty('serviceName', config.serviceName).get('configs');
          var serviceConfig = serviceConfigs.filterProperty('filename', fileName).findProperty('name', configName);
          var notEditableText = " " + Em.I18n.t('installer.step7.addWizard.notEditable');
          serviceConfig.set('description', serviceConfig.get('description') + notEditableText);
          serviceConfig.set('isReconfigurable', false);
          serviceConfig.set('isEditable', false);
          serviceConfig.set('disabledAsComponentAction', true);
          config.isReconfigurable = false;
          config.isEditable = false;
          config.disabledAsComponentActio = true;
        }
      }
    }, this);
  },

  completeConfigLoading: function completeConfigLoading() {
    this.get('stepConfigs').forEach(function (service) {
      App.configTheme.resolveConfigThemeConditions(service.get('configs'));
    });
    this.clearRecommendationsByServiceName(App.StackService.find().filter(function (s) {
      return s.get('isSelected') && !s.get('isInstalled');
    }).mapProperty('serviceName'));
    this.saveInitialRecommendations();
    this.set('isRecommendedLoaded', true);
    console.timeEnd('loadConfigRecommendations execution time: ');
    console.timeEnd('wizard loadStep: ');
    if (this.get('content.skipConfigStep')) {
      App.router.send('next');
    }
    this.set('hash', this.getHash());
  },

  /**
   * Update initialValues only while loading recommendations first time
   *
   * @param serviceName
   * @returns {boolean}
   * @override
   */
  updateInitialOnRecommendations: function updateInitialOnRecommendations(serviceName) {
    return this._super(serviceName) && !this.get('isRecommendedLoaded');
  },

  /**
   * Mark descriptor properties in configuration object.
   *
   * @param {Object[]} configs - config properties to change
   * @param {App.ServiceConfigProperty[]} descriptor - parsed kerberos descriptor
   * @method addKerberosDescriptorConfigs
   */
  addKerberosDescriptorConfigs: function addKerberosDescriptorConfigs(configs, descriptor, isRestoring) {
    var servicesToBeInstalled = this.get('content.services').filterProperty('isSelected').mapProperty('serviceName');
    descriptor.forEach(function (item) {
      var service = item.get('serviceName');
      if (!servicesToBeInstalled.contains(service)) return false;
      var name = item.get('name');
      var filename = Em.get(item, 'filename');
      var property = configs.filterProperty('serviceName', service).findProperty('name', name);
      var propertyObj = {
        isSecureConfig: true,
        value: Em.get(isRestoring && property ? property : item, 'value'),
        defaultValue: Em.get(item, 'value'),
        displayName: name,
        isOverridable: false,
        isConfigIdentity: Em.get(item, 'isConfigIdentity'),
        isUserProperty: !!Em.get(item, 'isUserProperty'),
        category: (Em.get(item, 'isUserProperty') ? 'Custom ' : 'Advanced ') + filename
      };
      if (property) {
        Em.setProperties(property, $.extend(propertyObj, {
          savedValue: Em.get(item, 'value'),
          description: Em.get(item, 'isConfigIdentity') ? App.config.kerberosIdentitiesDescription(Em.get(property, 'description')) : Em.get(property, 'description')
        }));
      } else {
        configs.push(App.ServiceConfigProperty.create(App.config.getDefaultConfig(name, filename, propertyObj)));
      }
    });
  },

  /**
   * Load config groups
   * and (if some services are already installed) load config groups for installed services
   * @method checkHostOverrideInstaller
   */
  checkHostOverrideInstaller: function checkHostOverrideInstaller() {
    if (this.get('wizardController.name') !== 'kerberosWizardController' && this.get('content.configGroups.length')) {
      this.restoreConfigGroups(this.get('content.configGroups'));
    } else {
      if (this.get('installedServiceNames').length > 0 && !this.get('wizardController.areInstalledConfigGroupsLoaded')) {
        console.time('loadConfigGroups execution time: ');
        this.loadConfigGroups(this.get('allSelectedServiceNames')).done(this.loadOverrides.bind(this));
      } else {
        App.store.fastCommit();
        App.configGroupsMapper.map(null, false, this.get('allSelectedServiceNames'));
        this.onLoadOverrides();
      }
    }
  },

  /**
   * Create stepConfigs array with all info except configs list
   *
   * @return {Object[]}
   * @method createStepConfigs
   */
  createStepConfigs: function createStepConfigs() {
    var stepConfigs = [];
    App.config.get('preDefinedServiceConfigs').forEach(function (service) {
      var serviceName = service.get('serviceName');
      if (['MISC'].concat(this.get('allSelectedServiceNames')).contains(serviceName)) {
        var serviceConfig = App.config.createServiceConfig(serviceName);
        serviceConfig.set('showConfig', App.StackService.find(serviceName).get('isInstallable'));
        if (this.get('wizardController.name') === 'addServiceController') {
          serviceConfig.set('selected', !this.get('installedServiceNames').concat('MISC').contains(serviceName));
          if (serviceName === 'MISC') {
            serviceConfig.set('configCategories', serviceConfig.get('configCategories').rejectProperty('name', 'Notifications'));
          }
        } else if (this.get('wizardController.name') === 'kerberosWizardController') {
          serviceConfig.set('showConfig', true);
        }
        stepConfigs.pushObject(serviceConfig);
      }
    }, this);
    return stepConfigs;
  },

  /**
   * For Namenode HA, HAWQ service requires additional config parameters in hdfs-client.xml
   * This method ensures that these additional parameters are added to hdfs-client.xml
   * @param configs existing configs on cluster
   * @returns {Object[]} existing configs + additional config parameters in hdfs-client.xml
   */
  addHawqConfigsOnNnHa: function addHawqConfigsOnNnHa(configs) {
    var nameService = configs.findProperty('id', App.config.configId('dfs.nameservices', 'hdfs-site')).value;
    var propertyNames = ['dfs.nameservices', 'dfs.ha.namenodes.' + nameService, 'dfs.namenode.rpc-address.' + nameService + '.nn1', 'dfs.namenode.rpc-address.' + nameService + '.nn2', 'dfs.namenode.http-address.' + nameService + '.nn1', 'dfs.namenode.http-address.' + nameService + '.nn2'];

    propertyNames.forEach(function (propertyName, propertyIndex) {
      var propertyFromHdfs = configs.findProperty('id', App.config.configId(propertyName, 'hdfs-site'));
      var newProperty = App.config.createDefaultConfig(propertyName, 'hdfs-client.xml', true);
      Em.setProperties(newProperty, {
        serviceName: 'HAWQ',
        description: propertyFromHdfs.description,
        displayName: propertyFromHdfs.displayName,
        displayType: 'string',
        index: propertyIndex,
        isOverridable: false,
        isReconfigurable: false,
        value: propertyFromHdfs.value,
        recommendedValue: propertyFromHdfs.recommendedValue
      });

      configs.push(App.ServiceConfigProperty.create(newProperty));
    });
    return configs;
  },

  /**
   * For ResourceManager HA, HAWQ service requires additional config parameters in yarn-client.xml
   * This method ensures that these additional parameters are added to yarn-client.xml
   * @param configs existing configs on cluster
   * @returns {Object[]} existing configs + additional config parameters in yarn-client.xml
   */
  addHawqConfigsOnRMHa: function addHawqConfigsOnRMHa(configs) {
    var rmHost1 = configs.findProperty('id', App.config.configId('yarn.resourcemanager.hostname.rm1', 'yarn-site')).value;
    var rmHost2 = configs.findProperty('id', App.config.configId('yarn.resourcemanager.hostname.rm2', 'yarn-site')).value;
    var yarnConfigToBeAdded = [{
      name: 'yarn.resourcemanager.ha',
      displayName: 'yarn.resourcemanager.ha',
      description: 'Comma separated yarn resourcemanager host addresses with port',
      port: '8032'
    }, {
      name: 'yarn.resourcemanager.scheduler.ha',
      displayName: 'yarn.resourcemanager.scheduler.ha',
      description: 'Comma separated yarn resourcemanager scheduler addresses with port',
      port: '8030'
    }];

    yarnConfigToBeAdded.forEach(function (propertyDetails) {
      var newProperty = App.config.createDefaultConfig(propertyDetails.name, 'yarn-client.xml', true);
      var value = rmHost1 + ':' + propertyDetails.port + ',' + rmHost2 + ':' + propertyDetails.port;
      Em.setProperties(newProperty, {
        serviceName: 'HAWQ',
        description: propertyDetails.description,
        displayName: propertyDetails.displayName,
        isOverridable: false,
        isReconfigurable: false,
        value: value,
        recommendedValue: value
      });

      configs.push(App.ServiceConfigProperty.create(newProperty));
    });
    return configs;
  },

  /**
   * Set the uid property for user properties. The uid is later used to help map the user and uid values in adjacent columns
   * @param {object} miscSvc
   * @param {string} svcName
   * @private
   */
  _setUID: function _setUID(miscSvc, svcName) {
    var user = miscSvc.configs.findProperty('name', svcName + '_user');
    if (user) {
      var uid = miscSvc.configs.findProperty('name', user.value + '_uid');
      if (uid) {
        user.set('ugid', uid);
      }
    }
  },

  /**
   * Set the gid property for group properties. The gid is later used to help map the group and gid values in adjacent columns
   * @param {object} miscSvc
   * @param {string} svcName
   * @private
   */
  _setGID: function _setGID(miscSvc, svcName) {
    var group = miscSvc.configs.findProperty('name', svcName + '_group');
    if (group) {
      var gid = miscSvc.configs.findProperty('name', group.value + '_gid');
      if (gid) {
        group.set('ugid', gid);
      }
    }
  },

  /**
   * render configs, distribute them by service
   * and wrap each in ServiceConfigProperty object
   * @param stepConfigs
   * @param configs
   * @return {App.ServiceConfig[]}
   */
  renderConfigs: function renderConfigs(stepConfigs, configs) {
    var localDB = {
      hosts: this.get('wizardController.content.hosts'),
      masterComponentHosts: this.get('wizardController.content.masterComponentHosts'),
      slaveComponentHosts: this.get('wizardController.content.slaveComponentHosts'),
      selectedStack: {}
    };
    var selectedRepoVersion, repoVersion;
    if (this.get('wizardController.name') === 'addServiceController') {
      repoVersion = App.RepositoryVersion.find().filter(function (i) {
        return i.get('stackVersionType') === App.get('currentStackName') && i.get('stackVersionNumber') === App.get('currentStackVersionNumber');
      })[0];
      if (repoVersion) {
        selectedRepoVersion = Em.get(repoVersion, 'repositoryVersion').split('-')[0];
      }
    } else {
      selectedRepoVersion = Em.getWithDefault(App.Stack.find().findProperty('isSelected', true) || {}, 'repositoryVersion', false);
    }
    if (selectedRepoVersion) {
      localDB.selectedStack = selectedRepoVersion;
    }
    var configsByService = {},
        dependencies = this.get('configDependencies');

    configs.forEach(function (_config) {
      if (!configsByService[_config.serviceName]) {
        configsByService[_config.serviceName] = [];
      }
      var serviceConfigProperty = App.ServiceConfigProperty.create(_config);
      this.updateHostOverrides(serviceConfigProperty, _config);
      if (this.get('wizardController.name') === 'addServiceController') {
        this._updateIsEditableFlagForConfig(serviceConfigProperty, true);
        //since the override_uid and ignore_groupusers_create changes are not saved to the database post install, they should be editable only
        //during initial cluster installation
        if (['override_uid', 'ignore_groupsusers_create'].contains(serviceConfigProperty.get('name'))) {
          serviceConfigProperty.set('isEditable', false);
        }
      }
      if (!this.get('content.serviceConfigProperties.length') && !serviceConfigProperty.get('hasInitialValue')) {
        App.ConfigInitializer.initialValue(serviceConfigProperty, localDB, dependencies);
      }
      configsByService[_config.serviceName].pushObject(serviceConfigProperty);
    }, this);

    stepConfigs.forEach(function (service) {
      if (service.get('serviceName') === 'YARN') {
        configsByService[service.get('serviceName')] = App.config.addYarnCapacityScheduler(configsByService[service.get('serviceName')]);
      }
      service.set('configs', configsByService[service.get('serviceName')] || []);
      if (['addServiceController', 'installerController'].contains(this.get('wizardController.name'))) {
        this.addHostNamesToConfigs(service, localDB.masterComponentHosts, localDB.slaveComponentHosts);
      }
    }, this);
    return stepConfigs;
  },

  addUidAndGidRepresentations: function addUidAndGidRepresentations(serviceConfigs) {
    //map the uids to the corresponding users
    var miscSvc = serviceConfigs.findProperty('serviceName', 'MISC');
    if (miscSvc) {
      //iterate through the list of users and groups and assign the uid/gid accordingly
      //user properties are servicename_user
      //uid properties are value of servicename_user + _uid
      //group properties are servicename_group
      //gid properties are value of servicename_group + _gid
      //we will map the users/uids and groups/gids based on this assumption
      this.get('selectedServiceNames').forEach(function (serviceName) {
        this._setUID(miscSvc, serviceName.toLowerCase());
        this._setGID(miscSvc, serviceName.toLowerCase());
      }, this);

      //for zookeeper, the user property name does not follow the convention that users for other services do. i.e. the user property name is not servicename_user as is the case with other services
      //the user property name is zk_user and not zookeeper_user, hence set the uid for zk_user separately
      this._setUID(miscSvc, 'zk');
      //the user property name is mapred_user and not mapreduce2_user for mapreduce2 service, hence set the uid for mapred_user separately
      this._setUID(miscSvc, 'mapred');
      //for haddop, the group property name does not follow the convention that groups for other services do. i.e. the group property name is not servicename_group as is the case with other services
      //the group property name is user_group and not zookeeper_group, hence set the gid for user_group separately
      this._setGID(miscSvc, 'user');

      // uid/gid properties are displayed in a separate column, hence prevent the properties from showing up on a separate line
      miscSvc.configs.filterProperty('displayType', 'uid_gid').setEach('isVisible', false);
    }
  },

  /**
   * Add host name properties to appropriate categories (for installer and add service)
   *
   * @param {Object} serviceConfig
   * @param {Object[]} masterComponents - info from localStorage
   * @param {Object[]} slaveComponents - info from localStorage
   */
  addHostNamesToConfigs: function addHostNamesToConfigs(serviceConfig, masterComponents, slaveComponents) {
    serviceConfig.get('configCategories').forEach(function (c) {
      if (c.showHost) {
        var componentName = c.name;
        var value = this.getComponentHostValue(componentName, masterComponents, slaveComponents);
        var stackComponent = App.StackServiceComponent.find(componentName);
        var hProperty = App.config.createHostNameProperty(serviceConfig.get('serviceName'), componentName, value, stackComponent);
        var newConfigName = Em.get(hProperty, 'name');
        if (!serviceConfig.get('configs').someProperty('name', newConfigName)) {
          serviceConfig.get('configs').push(App.ServiceConfigProperty.create(hProperty));
        }
      }
    }, this);
  },

  /**
   * Method to get host for master or slave component
   *
   * @param componentName
   * @param masterComponents
   * @param slaveComponents
   * @returns {Array}
   */
  getComponentHostValue: function getComponentHostValue(componentName, masterComponents, slaveComponents) {
    var value = [];
    var masters = masterComponents && masterComponents.filterProperty('component', componentName);
    if (masters.length) {
      value = masters.mapProperty('hostName');
    } else {
      var slaves = slaveComponents && slaveComponents.findProperty('componentName', componentName);
      if (slaves) {
        value = slaves.hosts.mapProperty('hostName');
      }
    }
    return value || [];
  },

  /**
   * create new child configs from overrides, attach them to parent config
   * override - value of config, related to particular host(s)
   * @param configProperty
   * @param storedConfigProperty
   */
  updateHostOverrides: function updateHostOverrides(configProperty, storedConfigProperty) {
    if (!Em.isEmpty(storedConfigProperty.overrides)) {
      var overrides = [];
      storedConfigProperty.overrides.forEach(function (overrideEntry) {
        // create new override with new value
        var newSCP = App.ServiceConfigProperty.create(configProperty);
        newSCP.set('value', overrideEntry.value);
        newSCP.set('isOriginalSCP', false); // indicated this is overridden value,
        newSCP.set('parentSCP', configProperty);
        overrides.pushObject(newSCP);
      });
      configProperty.set('overrides', overrides);
    }
  },

  /**
   * When NameNode HA is enabled some configs based on <code>dfs.nameservices</code> should be changed
   * This happens only if service is added AFTER NN HA is enabled
   *
   * @param {App.ServiceConfig[]} serviceConfigs
   * @method _reconfigureServiceOnNnHa
   * @private
   * @returns {App.ServiceConfig[]}
   */
  _reconfigureServicesOnNnHa: function _reconfigureServicesOnNnHa(serviceConfigs) {
    var selectedServiceNames = this.get('selectedServiceNames');
    var nameServiceId = serviceConfigs.findProperty('serviceName', 'HDFS').configs.findProperty('name', 'dfs.nameservices');
    if (nameServiceId) {
      Em.A([{
        serviceName: 'HBASE',
        configToUpdate: 'hbase.rootdir'
      }, {
        serviceName: 'ACCUMULO',
        configToUpdate: 'instance.volumes'
      }, {
        serviceName: 'HAWQ',
        configToUpdate: 'hawq_dfs_url',
        regexPattern: /(^.*:[0-9]+)(?=\/)/,
        replacementValue: nameServiceId.get('value')
      }]).forEach(function (c) {
        if (selectedServiceNames.contains(c.serviceName) && nameServiceId) {
          var cfg = serviceConfigs.findProperty('serviceName', c.serviceName).configs.findProperty('name', c.configToUpdate);
          var regexPattern = /\/\/.*:[0-9]+/i;
          var replacementValue = '//' + nameServiceId.get('value');
          if (!Em.isNone(c.regexPattern) && !Em.isNone(c.replacementValue)) {
            regexPattern = c.regexPattern;
            replacementValue = c.replacementValue;
          }
          var newValue = cfg.get('value').replace(regexPattern, replacementValue);
          cfg.setProperties({
            value: newValue,
            recommendedValue: newValue
          });
        }
      });
    }
    return serviceConfigs;
  },

  /**
   * Select previously selected service if not within the tab for the first time
   * Select first addable service for <code>addServiceWizard</code>
   * Select first service at all in other cases
   * @method selectProperService
   */
  selectProperService: function selectProperService() {
    var activeTab = this.get('tabs').findProperty('isActive', true);
    var tabSelectedServiceName = activeTab ? activeTab.get('selectedServiceName') : null;
    var lastSelectedService = tabSelectedServiceName ? this.get('stepConfigs').findProperty('serviceName', tabSelectedServiceName) : null;
    if (tabSelectedServiceName && lastSelectedService) {
      this.set('selectedService', lastSelectedService);
    } else if (this.get('wizardController.name') === 'addServiceController') {
      this.set('selectedService', this.get('stepConfigs').filterProperty('selected', true).get('firstObject'));
    } else {
      this.set('selectedService', this.get('stepConfigs').filterProperty('showConfig', true).objectAt(0));
    }
  },

  /**
   * Load config tags
   * @return {$.ajax|null}
   * @method getConfigTags
   */
  getConfigTags: function getConfigTags(resetFlag) {
    if (resetFlag) this.set('isAppliedConfigLoaded', false);
    return App.ajax.send({
      name: 'config.tags',
      sender: this,
      success: 'getConfigTagsSuccess'
    });
  },

  getServicesConfigurations: function getServicesConfigurations() {
    var dfd = $.Deferred();
    var configs, servicesConfigurations;
    configs = this.get('wizardController').getConfigsAndFilenames(this, true).serviceConfigProperties;
    servicesConfigurations = configs.reduce(function (configTags, property) {
      var fileName = App.config.getConfigTagFromFileName(property.filename),
          configType;
      if (!configTags.someProperty('type', fileName)) {
        configTags.push({
          type: fileName,
          properties: {}
        });
      }
      configType = configTags.findProperty('type', fileName);
      configType.properties[property.name] = property.value;
      return configTags;
    }, []);
    dfd.resolve(servicesConfigurations);
    return dfd.promise();
  },

  /**
   * Success callback for config tags request
   * Updates <code>serviceConfigTags</code> with tags received from server
   * @param {object} data
   * @method getConfigTagsSuccess
   */
  getConfigTagsSuccess: function getConfigTagsSuccess(data) {
    var installedServiceSites = [];
    App.StackService.find().filterProperty('isInstalled').forEach(function (service) {
      if (!service.get('configTypes')) return;
      var configTypes = Object.keys(service.get('configTypes'));
      installedServiceSites = installedServiceSites.concat(configTypes);
    }, this);
    installedServiceSites = installedServiceSites.uniq();
    var serviceConfigTags = [];
    for (var site in data.Clusters.desired_configs) {
      if (data.Clusters.desired_configs.hasOwnProperty(site)) {
        if (installedServiceSites.contains(site) || site === 'cluster-env') {
          serviceConfigTags.push({
            siteName: site,
            tagName: data.Clusters.desired_configs[site].tag,
            newTagName: null
          });
        }
      }
    }
    this.set('serviceConfigTags', serviceConfigTags);
    this.set('isAppliedConfigLoaded', true);
  },

  /**
   * set configs actual values from server
   * @param configs
   * @param configsByTags
   * @param installedServiceNames
   * @method setInstalledServiceConfigs
   */
  setInstalledServiceConfigs: function setInstalledServiceConfigs(configs, configsByTags, installedServiceNames) {
    var configsMap = {};
    var finalAttrMap = {};
    var passwordAttrMap = {};

    configsByTags.forEach(function (configSite) {
      configsMap[configSite.type] = configSite.properties || {};
      finalAttrMap[configSite.type] = configSite.properties_attributes && configSite.properties_attributes.final || {};
      passwordAttrMap[configSite.type] = configSite.properties_attributes && configSite.properties_attributes.password || {};
    });
    var allConfigs = configs.filter(function (_config) {
      // filter out alert_notification configs on add service //TODO find better place for this!
      if (_config.filename === 'alert_notification') return false;
      if (['MISC'].concat(installedServiceNames).contains(_config.serviceName)) {
        var type = _config.filename ? App.config.getConfigTagFromFileName(_config.filename) : null;
        var mappedConfigValue = type && configsMap[type] ? configsMap[type][_config.name] : null;
        if (Em.isNone(mappedConfigValue)) {
          //for now ranger plugin properties are not sending by recommendations if they are missed - it should be added
          return _config.serviceName === 'MISC' || /^ranger-/.test(_config.filename);
        }
        if (_config.savedValue != mappedConfigValue) {
          _config.savedValue = App.config.formatPropertyValue(_config, mappedConfigValue);
        }
        _config.value = App.config.formatPropertyValue(_config, mappedConfigValue);
        _config.hasInitialValue = true;
        this.updateDependencies(_config);
        delete configsMap[type][_config.name];
        return true;
      }
      return true;
    }, this);
    //add user properties
    Em.keys(configsMap).forEach(function (filename) {
      Em.keys(configsMap[filename]).forEach(function (propertyName) {
        allConfigs.push(App.config.createDefaultConfig(propertyName, App.config.getOriginalFileName(filename), false, {
          value: configsMap[filename][propertyName],
          savedValue: configsMap[filename][propertyName],
          propertyType: passwordAttrMap[filename][propertyName] === 'true' ? ['PASSWORD'] : null,
          isFinal: finalAttrMap[filename][propertyName] === 'true',
          hasInitialValue: true
        }));
      });
    });
    return allConfigs;
  },

  /**
   * update dependencies according to current config value
   *
   * @param config
   */
  updateDependencies: function updateDependencies(config) {
    if (config.filename === 'hive-site.xml') {
      if (config.name === 'hive.metastore.uris') {
        this.get('configDependencies')['hive.metastore.uris'] = config.savedValue;
      } else if (config.name === 'clientPort') {
        this.get('configDependencies').clientPort = config.savedValue;
      }
    }
    if (config.filename === 'application-properties.xml') {
      if (this.get('configDependencies').hasOwnProperty(config.name)) {
        this.get('configDependencies')[config.name] = config.savedValue;
      }
    }
  },

  /**
   * Add group ids to <code>groupsToDelete</code>
   * Also save <code>groupsToDelete</code> to local storage
   * @param {Ember.Object[]} groups
   * @method setGroupsToDelete
   */
  setGroupsToDelete: function setGroupsToDelete(groups) {
    var groupsToDelete = this.get('groupsToDelete');
    groups.forEach(function (group) {
      if (!group.get('isTemporary')) {
        groupsToDelete.push({
          id: group.get('id')
        });
      }
    });
    this.get('wizardController').setDBProperty('groupsToDelete', groupsToDelete);
  },

  /**
   * Update <code>configGroups</code> with selected service configGroups
   * Also set default group to first position
   * Update <code>selectedConfigGroup</code> with new default group
   * @method selectedServiceObserver
   */
  selectedServiceObserver: function () {
    if (this.get('selectedService') && this.get('selectedService.serviceName') !== 'MISC') {
      var serviceGroups = this.get('selectedService.configGroups');
      serviceGroups.forEach(function (item, index, array) {
        if (item.isDefault) {
          array.unshift(item);
          array.splice(index + 1, 1);
        }
      });
      this.set('configGroups', serviceGroups);
      this.set('selectedConfigGroup', serviceGroups.findProperty('isDefault'));
    }
  }.observes('selectedService.configGroups.[]'),

  /**
   * load default groups for each service in case of initial load
   * @param serviceConfigGroups
   * @method restoreConfigGroups
   */
  restoreConfigGroups: function restoreConfigGroups(serviceConfigGroups) {
    var services = this.get('stepConfigs');

    services.forEach(function (service) {
      if (service.get('serviceName') === 'MISC') return;
      var serviceRawGroups = serviceConfigGroups.filterProperty('service_name', service.serviceName);
      if (serviceRawGroups.length) {
        App.store.safeLoadMany(App.ServiceConfigGroup, serviceRawGroups);
        serviceRawGroups.forEach(function (item) {
          var modelGroup = App.ServiceConfigGroup.find(item.id);
          modelGroup.set('properties', []);
          item.properties.forEach(function (propertyData) {
            var overriddenSCP,
                parentSCP = service.configs.filterProperty('filename', propertyData.filename).filterProperty('isOriginalSCP').findProperty('name', propertyData.name);
            if (parentSCP) {
              App.config.createOverride(parentSCP, propertyData, modelGroup);
            } else {
              overriddenSCP = App.config.createCustomGroupConfig({
                propertyName: propertyData.name,
                filename: propertyData.filename,
                value: propertyData.value,
                savedValue: propertyData.value,
                isEditable: false
              }, modelGroup);
              this.get('stepConfigs').findProperty('serviceName', service.serviceName).get('configs').pushObject(overriddenSCP);
            }
          }, this);
          modelGroup.set('hash', this.get('wizardController').getConfigGroupHash(modelGroup));
        }, this);
        service.set('configGroups', App.ServiceConfigGroup.find().filterProperty('serviceName', service.get('serviceName')));
      }
    }, this);
  },

  /**
   * Click-handler on config-group to make it selected
   * @param {object} event
   * @method selectConfigGroup
   */
  selectConfigGroup: function selectConfigGroup(event) {
    this.set('selectedConfigGroup', event.context);
  },

  /**
   * Rebuild list of configs switch of config group:
   * on default - display all configs from default group and configs from non-default groups as disabled
   * on non-default - display all from default group as disabled and configs from selected non-default group
   * @method switchConfigGroupConfigs
   */
  switchConfigGroupConfigs: function () {
    var serviceConfigs = this.get('selectedService.configs'),
        selectedGroup = this.get('selectedConfigGroup'),
        overrides = [];
    if (!selectedGroup) return;

    var displayedConfigGroups = this._getDisplayedConfigGroups();
    displayedConfigGroups.forEach(function (group) {
      overrides.pushObjects(group.get('properties'));
    });
    serviceConfigs.forEach(function (config) {
      this._setEditableValue(config);
      this._setOverrides(config, overrides);
    }, this);
  }.observes('selectedConfigGroup'),

  /**
   * Get list of config groups to display
   * Returns empty array if no <code>selectedConfigGroup</code>
   * @return {Array}
   * @method _getDisplayedConfigGroups
   */
  _getDisplayedConfigGroups: function _getDisplayedConfigGroups() {
    var selectedGroup = this.get('selectedConfigGroup');
    if (!selectedGroup) return [];
    return selectedGroup.get('isDefault') ? this.get('selectedService.configGroups').filterProperty('isDefault', false) : [this.get('selectedConfigGroup')];
  },

  /**
   * Set <code>isEditable</code> property to <code>config</code>
   * @param {Ember.Object} config
   * @return {Ember.Object} updated config-object
   * @method _setEditableValue
   */
  _setEditableValue: function _setEditableValue(config) {
    var selectedGroup = this.get('selectedConfigGroup');
    if (!selectedGroup) return config;
    if (App.get('isKerberosEnabled') && config.get('isConfigIdentity') && !App.StackService.find().filterProperty('isSelected').mapProperty('serviceName').contains(Em.get(config, 'serviceName'))) {
      config.set('isEditable', false);
    } else {
      var isEditable = config.get('isEditable'),
          isServiceInstalled = this.get('installedServiceNames').contains(this.get('selectedService.serviceName'));
      if (isServiceInstalled) {
        isEditable = config.get('isReconfigurable') && selectedGroup.get('isDefault') && !config.get('disabledAsComponentAction');
      } else {
        isEditable = selectedGroup.get('isDefault') && !config.get('disabledAsComponentAction');
      }
      if (config.get('group')) {
        isEditable = config.get('group.name') === this.get('selectedConfigGroup.name');
      }
      config.set('isEditable', isEditable);
    }
    return config;
  },

  /**
   * Set <code>overrides</code> property to <code>config</code>
   * @param {Ember.Object} config
   * @param {Ember.Enumerable} overrides
   * @returns {Ember.Object}
   * @method _setOverrides
   */
  _setOverrides: function _setOverrides(config, overrides) {
    if (config.get('group')) return config;
    var selectedGroup = this.get('selectedConfigGroup'),
        overrideToAdd = this.get('overrideToAdd'),
        configOverrides = overrides.filterProperty('id', config.get('id'));
    if (!selectedGroup) return config;
    if (overrideToAdd && overrideToAdd.get('id') === config.get('id')) {
      var valueForOverride = config.get('widget') || config.get('displayType') === 'checkbox' ? config.get('value') : '';
      var group = this.get('selectedService.configGroups').findProperty('name', selectedGroup.get('name'));
      var newSCP = App.config.createOverride(config, { value: valueForOverride, recommendedValue: valueForOverride }, group);
      configOverrides.push(newSCP);
      this.set('overrideToAdd', null);
    }
    configOverrides.setEach('isEditable', !selectedGroup.get('isDefault'));
    configOverrides.setEach('parentSCP', config);
    config.set('overrides', configOverrides);
    return config;
  },

  /**
   * @param serviceName
   * @returns {boolean}
   * @override
   */
  useInitialValue: function useInitialValue(serviceName) {
    return !App.Service.find(serviceName).get('serviceName', serviceName);
  },

  /**
   *
   * @param parentProperties
   * @param name
   * @param fileName
   * @param configGroup
   * @param savedValue
   * @returns {*}
   * @override
   */
  allowUpdateProperty: function allowUpdateProperty(parentProperties, name, fileName, configGroup, savedValue) {
    if (name.contains('proxyuser')) return true;
    if (['installerController'].contains(this.get('wizardController.name')) || !!(parentProperties && parentProperties.length)) {
      return true;
    } else if (['addServiceController'].contains(this.get('wizardController.name'))) {
      var stackProperty = App.configsCollection.getConfigByName(name, fileName);
      if (!stackProperty || !this.get('installedServices')[stackProperty.serviceName]) {
        return true;
      } else if (stackProperty.propertyDependsOn.length) {
        return stackProperty.propertyDependsOn.some(function (p) {
          var service = App.config.get('serviceByConfigTypeMap')[p.type];
          return service && !this.get('installedServices')[service.get('serviceName')];
        }, this);
      } else {
        return !Em.isNone(savedValue) && stackProperty.recommendedValue === savedValue;
      }
    }
    return true;
  },

  /**
   * remove config based on recommendations
   * @param config
   * @param configsCollection
   * @param parentProperties
   * @protected
   * @override
   */
  _removeConfigByRecommendation: function _removeConfigByRecommendation(config, configsCollection, parentProperties) {
    this._super(config, configsCollection, parentProperties);
    /**
     * need to update wizard info when removing configs for installed services;
     */
    var installedServices = this.get('installedServices'),
        wizardController = this.get('wizardController'),
        fileNamesToUpdate = wizardController ? wizardController.getDBProperty('fileNamesToUpdate') || [] : [],
        fileName = Em.get(config, 'filename'),
        serviceName = Em.get(config, 'serviceName');
    var modifiedFileNames = this.get('modifiedFileNames');
    if (modifiedFileNames && !modifiedFileNames.contains(fileName)) {
      modifiedFileNames.push(fileName);
    } else if (wizardController && installedServices[serviceName]) {
      if (!fileNamesToUpdate.contains(fileName)) {
        fileNamesToUpdate.push(fileName);
      }
    }
    if (wizardController) {
      wizardController.setDBProperty('fileNamesToUpdate', fileNamesToUpdate.uniq());
    }
  },
  /**
   * @method manageConfigurationGroup
   */
  manageConfigurationGroup: function manageConfigurationGroup() {
    App.router.get('manageConfigGroupsController').manageConfigurationGroups(this);
  },

  /**
   * Check whether hive New MySQL database is on the same host as Ambari server MySQL server
   * @return {$.ajax|null}
   * @method checkMySQLHost
   */
  checkMySQLHost: function checkMySQLHost() {
    // get ambari database type and hostname
    return App.ajax.send({
      name: 'ambari.service',
      data: {
        fields: "?fields=hostComponents/RootServiceHostComponents/properties/server.jdbc.database_name,hostComponents/RootServiceHostComponents/properties/server.jdbc.url,hostComponents/RootServiceHostComponents/properties/server.jdbc.database"
      },
      sender: this,
      success: 'getAmbariDatabaseSuccess'
    });
  },

  /**
   * Success callback for ambari database, get Ambari DB type and DB server hostname, then
   * Check whether hive New MySQL database is on the same host as Ambari server MySQL server
   * @param {object} data
   * @method getAmbariDatabaseSuccess
   */
  getAmbariDatabaseSuccess: function getAmbariDatabaseSuccess(data) {
    var ambariServerDBType = Em.getWithDefault(data.hostComponents, '0.RootServiceHostComponents.properties', {})['server.jdbc.database'],
        ambariServerHostName = Em.getWithDefault(data.hostComponents, '0.RootServiceHostComponents.host_name', false),
        hiveConnectionURL = Em.getWithDefault(App.config.findConfigProperty(this.get('stepConfigs'), 'javax.jdo.option.ConnectionURL', 'hive-site.xml') || {}, 'value', '');
    if (ambariServerHostName) {
      this.set('mySQLServerConflict', ambariServerDBType.contains('mysql') && hiveConnectionURL.contains(ambariServerHostName));
    } else {
      this.set('mySQLServerConflict', false);
    }
  },

  /**
   * Check if new MySql database was chosen for Hive service
   * and it is not located on the same host as Ambari server
   * that using MySql database too.
   *
   * @method resolveHiveMysqlDatabase
   */
  resolveHiveMysqlDatabase: function resolveHiveMysqlDatabase() {
    var hiveService = this.get('content.services').findProperty('serviceName', 'HIVE');
    if (!hiveService || !hiveService.get('isSelected') || hiveService.get('isInstalled')) {
      return this.moveNext();
    }
    var hiveDBType = this.get('stepConfigs').findProperty('serviceName', 'HIVE').configs.findProperty('name', 'hive_database').value;
    if (hiveDBType === 'New MySQL Database') {
      var self = this;
      return this.checkMySQLHost().done(function () {
        self.mySQLWarningHandler();
      });
    }
    return this.moveNext();
  },

  /**
   * Show warning popup about MySQL-DB issues (on post-submit)
   *
   * @returns {*}
   * @method mySQLWarningHandler
   */
  mySQLWarningHandler: function mySQLWarningHandler() {
    var self = this;
    if (this.get('mySQLServerConflict')) {
      // error popup before you can proceed
      return App.ModalPopup.show({
        header: Em.I18n.t('installer.step7.popup.mySQLWarning.header'),
        body: Em.I18n.t('installer.step7.popup.mySQLWarning.body'),
        secondary: Em.I18n.t('installer.step7.popup.mySQLWarning.button.gotostep5'),
        primary: Em.I18n.t('installer.step7.popup.mySQLWarning.button.dismiss'),
        encodeBody: false,
        onPrimary: function onPrimary() {
          this._super();
          self.set('submitButtonClicked', false);
        },
        onSecondary: function onSecondary() {
          var parent = this;
          return App.ModalPopup.show({
            header: Em.I18n.t('installer.step7.popup.mySQLWarning.confirmation.header'),
            body: Em.I18n.t('installer.step7.popup.mySQLWarning.confirmation.body'),
            onPrimary: function onPrimary() {
              this.hide();
              parent.hide();
              // go back to step 5: assign masters and disable default navigation warning
              if (self.get('isInstallWizard')) {
                App.router.get('installerController').gotoStep(5, true);
              } else {
                if (self.get('isAddServiceWizard')) {
                  App.router.get('addServiceController').gotoStep(2, true);
                }
              }
            },
            onSecondary: function onSecondary() {
              this._super();
              self.set('submitButtonClicked', false);
            }
          });
        }
      });
    }
    return this.moveNext();
  },

  checkDatabaseConnectionTest: function checkDatabaseConnectionTest() {
    var deferred = $.Deferred();
    var configMap = [{
      serviceName: 'OOZIE',
      ignored: [Em.I18n.t('installer.step7.oozie.database.new')]
    }, {
      serviceName: 'HIVE',
      ignored: [Em.I18n.t('installer.step7.hive.database.new.mysql'), Em.I18n.t('installer.step7.hive.database.new.postgres')]
    }];
    configMap.forEach(function (config) {
      var isConnectionNotTested = false;
      var service = this.get('content.services').findProperty('serviceName', config.serviceName);
      if (service && service.get('isSelected') && !service.get('isInstalled')) {
        var serviceConfigs = this.get('stepConfigs').findProperty('serviceName', config.serviceName).configs;
        var serviceDatabase = serviceConfigs.findProperty('name', config.serviceName.toLowerCase() + '_database').get('value');
        if (!config.ignored.contains(serviceDatabase)) {
          var filledProperties = App.db.get('tmp', config.serviceName + '_connection');
          if (!filledProperties || App.isEmptyObject(filledProperties)) {
            isConnectionNotTested = true;
          } else {
            for (var key in filledProperties) {
              if (serviceConfigs.findProperty('name', key).get('value') !== filledProperties[key]) isConnectionNotTested = true;
            }
          }
        }
      }
      config.isCheckIgnored = isConnectionNotTested;
    }, this);
    var ignoredServices = configMap.filterProperty('isCheckIgnored', true);
    if (ignoredServices.length) {
      var displayedServiceNames = ignoredServices.mapProperty('serviceName').map(function (serviceName) {
        return this.get('content.services').findProperty('serviceName', serviceName).get('displayName');
      }, this);
      this.showDatabaseConnectionWarningPopup(displayedServiceNames, deferred);
    } else {
      deferred.resolve();
    }
    return deferred;
  },

  showChangesWarningPopup: function showChangesWarningPopup(goToNextStep) {
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      body: Em.I18n.t('services.service.config.exitChangesPopup.body'),
      secondary: Em.I18n.t('common.cancel'),
      primary: Em.I18n.t('yes'),
      onPrimary: function onPrimary() {
        if (goToNextStep) {
          goToNextStep();
          this.hide();
        }
      },
      onSecondary: function onSecondary() {
        this.hide();
        App.set('router.btnClickInProgress', false);
        App.set('router.backBtnClickInProgress', false);
      }
    });
  },

  showDatabaseConnectionWarningPopup: function showDatabaseConnectionWarningPopup(serviceNames, deferred) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step7.popup.database.connection.header'),
      body: Em.I18n.t('installer.step7.popup.database.connection.body').format(serviceNames.join(', ')),
      secondary: Em.I18n.t('common.cancel'),
      primary: Em.I18n.t('common.proceedAnyway'),
      onPrimary: function onPrimary() {
        deferred.resolve();
        this._super();
      },
      onClose: function onClose() {
        this.onSecondary();
      },
      onSecondary: function onSecondary() {
        this._super();
        self.set('submitButtonClicked', false);
        App.set('router.nextBtnClickInProgress', false);
        deferred.reject();
      }
    });
  },

  showOozieDerbyWarningPopup: function showOozieDerbyWarningPopup(callback) {
    var self = this;
    if (this.get('selectedServiceNames').contains('OOZIE')) {
      var databaseType = Em.getWithDefault(App.config.findConfigProperty(this.get('stepConfigs'), 'oozie_database', 'oozie-env.xml') || {}, 'value', '');
      if (databaseType === Em.I18n.t('installer.step7.oozie.database.new')) {
        return App.ModalPopup.show({
          header: Em.I18n.t('common.warning'),
          body: Em.I18n.t('installer.step7.popup.oozie.derby.warning'),
          secondary: Em.I18n.t('common.cancel'),
          primary: Em.I18n.t('common.proceedAnyway'),
          onPrimary: function onPrimary() {
            this.hide();
            if (callback) {
              callback();
            }
          },
          onSecondary: function onSecondary() {
            App.set('router.nextBtnClickInProgress', false);
            self.set('submitButtonClicked', false);
            this.hide();
          },
          onClose: function onClose() {
            this.onSecondary();
          }
        });
      }
    }
    if (callback) {
      callback();
    }
    return false;
  },

  /**
   * Proceed to the next step
   **/
  moveNext: function moveNext() {
    App.set('router.nextBtnClickInProgress', false);
    App.router.send('next');
    this.set('submitButtonClicked', false);
  },

  /**
   * Click-handler on Next button
   * Disable "Submit"-button while server-side processes are running
   * @method submit
   */
  submit: function submit() {
    if (this.get('isSubmitDisabled') || App.get('router.nextBtnClickInProgress')) {
      return false;
    }
    App.set('router.nextBtnClickInProgress', true);
    if (this.get('supportsPreInstallChecks')) {
      var preInstallChecksController = App.router.get('preInstallChecksController');
      if (preInstallChecksController.get('preInstallChecksWhereRun')) {
        return this.postSubmit();
      }
      return preInstallChecksController.notRunChecksWarnPopup(this.postSubmit.bind(this));
    }
    return this.postSubmit();
  },

  postSubmit: function postSubmit() {
    var _this = this;

    this.set('submitButtonClicked', true);
    if (this.get('isInstallWizard')) {
      this.serverSideValidationCallback();
    } else {
      this.serverSideValidation().done(function () {
        _this.serverSideValidationCallback();
      }).fail(function (value) {
        if ("invalid_configs" === value) {
          if (_this.get('isAddServiceWizard')) {
            _this.get('configErrorList.issues').clear();
            _this.get('configErrorList.criticalIssues').clear();
          }
          _this.set('submitButtonClicked', false);
          App.set('router.nextBtnClickInProgress', false);
        } else {
          // Failed due to validation mechanism failure.
          // Should proceed with other checks
          _this.serverSideValidationCallback();
        }
      });
    }
  },

  /**
   * @method serverSideValidationCallback
   */
  serverSideValidationCallback: function serverSideValidationCallback() {
    var self = this;
    this.showOozieDerbyWarningPopup(function () {
      self.checkDatabaseConnectionTest().done(function () {
        self.resolveHiveMysqlDatabase();
      });
    });
  },

  toggleIssuesFilter: function toggleIssuesFilter() {
    var errorServices = [],
        issueServices = [];

    this.get('filterColumns').findProperty('attributeName', 'hasIssues').toggleProperty('selected');

    // if currently selected service does not have issue, jump to the first service with issue.
    if (this.get('selectedService.errorCount') === 0) {
      this.get('stepConfigs').filterProperty('showConfig', true).forEach(function (service) {
        if (service.get('errorCount') > 0) errorServices.push(service);
        if (service.get('hasConfigIssues') === true) issueServices.push(service);
      });
      if (errorServices.length === 0 && issueServices.length === 0) return;
      this.switchToService(errorServices.length > 0 ? errorServices[0] : issueServices[0]);
    }
  },

  switchToService: function switchToService(service) {
    this.set('selectedService', service);
    $('a[href="#' + service.serviceName + '"]').tab('show');
  },

  issuesFilterSelected: function () {
    return this.get('filterColumns').findProperty('attributeName', 'hasIssues').get('selected');
  }.property('filterColumns.@each.selected'),

  hasStepConfigIssues: function () {
    return !this.get('stepConfigs').filterProperty('showConfig', true).everyProperty('hasConfigIssues', false);
  }.property('stepConfigs.@each.hasConfigIssues'),

  checkDescriptor: function checkDescriptor() {
    if (App.get('isKerberosEnabled')) {
      return App.ajax.send({
        sender: self,
        name: 'admin.kerberize.cluster_descriptor_artifact'
      });
    }
    return $.Deferred().resolve().promise();
  },

  /**
   * Store status of kerberos descriptor located in cluster artifacts.
   * This status needed for Add Service Wizard to select appropriate method to create
   * or update descriptor.
   *
   * @param  {Boolean} isExists <code>true</code> if cluster descriptor present
   */
  storeClusterDescriptorStatus: function storeClusterDescriptorStatus(isExists) {
    this.get('wizardController').setDBProperty('isClusterDescriptorExists', isExists);
  },

  /**
  * Get all configs, that should be stored in kerberos_descriptor
  * @returns {Array}
  */
  getDescriptorConfigs: function getDescriptorConfigs() {
    return this.get('stepConfigs').reduce(function (allConfigs, service) {
      return allConfigs.concat(service.get('configs').filterProperty('isSecureConfig'));
    }, []);
  },

  selectService: function selectService(event) {
    this.get('stepConfigs').forEach(function (service) {
      service.set('isActive', service.get('serviceName') === event.context.serviceName);
    });
    this.set('selectedService', event.context);
    var activeTabs = this.get('tabs').findProperty('isActive', true);
    if (activeTabs) {
      activeTabs.set('selectedServiceName', event.context.serviceName);
    }
  },

  /**
   * Set initial state for <code>tabs</code>
   */
  initTabs: function initTabs() {
    var storedConfigs = this.get('content.serviceConfigProperties');
    var tabs = [Em.Object.create({
      name: 'credentials',
      displayName: 'Credentials',
      icon: 'glyphicon-lock',
      isActive: false,
      isDisabled: false,
      isSkipped: false,
      validateOnSwitch: false,
      tabView: App.CredentialsTabOnStep7View
    }), Em.Object.create({
      name: 'databases',
      displayName: 'Databases',
      icon: 'glyphicon-align-justify',
      isActive: false,
      isDisabled: false,
      isSkipped: false,
      validateOnSwitch: false,
      tabView: App.DatabasesTabOnStep7View
    }), Em.Object.create({
      name: 'directories',
      displayName: 'Directories',
      icon: 'glyphicon-folder-open',
      isActive: false,
      isDisabled: false,
      isSkipped: false,
      validateOnSwitch: false,
      selectedServiceName: null,
      tabView: App.DirectoriesTabOnStep7View
    }), Em.Object.create({
      name: 'accounts',
      displayName: 'Accounts',
      icon: 'glyphicon-user',
      isActive: false,
      isDisabled: false,
      isSkipped: false,
      validateOnSwitch: false,
      tabView: App.AccountsTabOnStep7View
    }), Em.Object.create({
      name: 'all-configurations',
      displayName: 'All Configurations',
      icon: 'glyphicon-wrench',
      isActive: false,
      isDisabled: false,
      isSkipped: false,
      validateOnSwitch: true,
      selectedServiceName: null,
      tabView: App.ServicesConfigView
    })];

    this.set('tabs', tabs);

    this.setSkippedTabs();

    if (storedConfigs && storedConfigs.length) {
      tabs.findProperty('name', 'all-configurations').set('isActive', true);
    } else {
      tabs.findProperty('isDisabled', false).set('isActive', true);
      this.disableTabs();
    }
  },

  setSkippedTabs: function setSkippedTabs() {
    var servicesWithCredentials = App.Tab.find().filterProperty('themeName', 'credentials').mapProperty('serviceName');
    var servicesWithDatabase = App.Tab.find().filterProperty('themeName', 'database').mapProperty('serviceName');
    var selectedServices = this.get('content.selectedServiceNames');
    var tabs = this.get('tabs');
    var disableCredentials = true;
    var disableDatabases = true;

    for (var i = 0; i < selectedServices.length; i++) {
      var serviceName = selectedServices[i];
      disableCredentials = disableCredentials && servicesWithCredentials.indexOf(serviceName) === -1;
      disableDatabases = disableDatabases && servicesWithDatabase.indexOf(serviceName) === -1;
      if (!disableCredentials && !disableDatabases) break;
    }

    tabs.findProperty('name', 'credentials').setProperties({
      isDisabled: disableCredentials,
      isSkipped: disableCredentials
    });
    tabs.findProperty('name', 'databases').setProperties({
      isDisabled: disableDatabases,
      isSkipped: disableDatabases
    });
  },

  /**
   * Get index of current (active) tab
   */
  currentTabIndex: function () {
    return this.get('tabs').findIndex(function (tab) {
      return tab.get('isActive');
    });
  }.property('tabs.@each.isActive'),

  /**
   * Get name of current (active) tab
   */
  currentTabName: function () {
    var activeTab = this.get('tabs').findProperty('isActive');
    return activeTab ? activeTab.get('name') : '';
  }.property('tabs.@each.isActive'),

  /**
   * Make selected tab active, show tab's content
   * @param event
   * @returns {boolean}
   */
  selectTab: function selectTab(event) {
    var tab = event.context;
    if (!tab.get('isDisabled')) {
      $('a[href=#' + tab.name + ']').tab('show');
      this.get('tabs').setEach('isActive', false);
      tab.set('isActive', true);
    }
    return false;
  },

  /**
   * Get disabled state for next button
   */
  isNextDisabled: function () {
    var tabName = this.get('currentTabName');
    switch (tabName) {
      case 'credentials':
        return !this.get('credentialsTabNextEnabled');
      case 'databases':
        return !this.get('databasesTabNextEnabled');
      case 'all-configurations':
        return this.get('isSubmitDisabled');
      default:
        return false;
    }
  }.property('tabs.@each.isActive', 'isSubmitDisabled', 'credentialsTabNextEnabled', 'databasesTabNextEnabled'),

  /**
   * Set isDisabled state for tabs
   */
  disableTabs: function () {
    this.get('tabs').rejectProperty('isSkipped').setEach('isDisabled', !this.get('tabs').findProperty('name', 'credentials').get('isDisabled') && !this.get('credentialsTabNextEnabled'));
  }.observes('credentialsTabNextEnabled'),

  /**
   * Select specified service and put property name in filter input
   * @param event - jQuery event
   */
  showConfigProperty: function showConfigProperty(event) {
    var serviceName = event.context.serviceName;
    var propertyName = event.context.propertyName || event.context.name;
    var stepConfig = this.get('stepConfigs').findProperty('serviceName', serviceName) || this.get('stepConfigs').findProperty('displayName', serviceName);
    this.set('selectedService', stepConfig);
    this.get('filterColumns').setEach('selected', false);
    Em.run.next(this, function () {
      this.set('filter', propertyName);
    });

    this.get('stepConfigs').setEach('isActive', false);
    stepConfig.set('isActive', true);
  },

  /**
   * Show bell animation
   */
  ringBell: function ringBell() {
    $('#issues-bell').addClass('animated');
    $('#issues-counter').addClass('animated');
    setTimeout(function () {
      $('#issues-bell').removeClass('animated');
    }, 2000);
    setTimeout(function () {
      $('#issues-counter').removeClass('animated');
    }, 300);
  },

  /**
   * Set <code>issuesCounter</code> and run bell animation if needed
   */
  setIssues: function () {
    var recommendations = this.get('changedProperties.length');
    var validations = this.get('stepConfigs').mapProperty('configsWithErrors.length').reduce(Em.sum, 0);
    var configErrorList = this.get('configErrorList');
    this.set('hasErrors', Boolean(validations + configErrorList.get('criticalIssues.length')));
    this.set('issuesCounter', recommendations + validations + configErrorList.get('issues.length') + configErrorList.get('criticalIssues.length'));
    if (validations !== this.get('validationsCounter')) {
      this.ringBell();
    }
    this.set('validationsCounter', validations);
  }.observes('changedProperties.length', 'stepConfigs.@each.configsWithErrors.length', 'configErrorList.issues.length', 'configErrorList.criticalIssues.length'),

  /**
   * Next button action handler
   */
  next: function next(index) {
    var tabs = this.get('tabs');
    var currentTabIndex = typeof index === 'number' ? index : this.get('currentTabIndex');
    if (tabs.length - 1 > currentTabIndex) {
      tabs[currentTabIndex].set('isActive', false);
      if (tabs[currentTabIndex + 1].get('isDisabled')) {
        this.next(++currentTabIndex);
      } else {
        tabs[currentTabIndex + 1].set('isActive', true);
      }
    } else {
      this.submit();
    }
  },

  /**
   * Back button action handler
   */
  back: function back(index) {
    var tabs = this.get('tabs');
    var currentTabIndex = typeof index === 'number' ? index : this.get('currentTabIndex');
    if (currentTabIndex > 0) {
      tabs[currentTabIndex].set('isActive', false);
      if (tabs[currentTabIndex - 1].get('isDisabled')) {
        this.back(--currentTabIndex);
      } else {
        tabs[currentTabIndex - 1].set('isActive', true);
      }
    } else {
      App.router.send('back');
    }
  },

  updateConfigAttributesFromThemes: function updateConfigAttributesFromThemes() {
    var _this2 = this;

    this.get('allSelectedServiceNames').forEach(function (serviceName) {
      return _this2.updateAttributesFromTheme(serviceName);
    });
  },

  validateOnTabSwitch: function () {
    var _this3 = this;

    var activeTab = this.get('tabs')[this.get('currentTabIndex')];
    if (activeTab && activeTab.get('validateOnSwitch')) {
      if (this.get('requestTimer')) {
        clearTimeout(this.get('requestTimer'));
      }
      if (this.get('validationRequest')) {
        this.get('validationRequest').abort();
      }
      if (this.get('recommendationsInProgress')) {
        this.valueObserver();
      } else {
        this.runServerSideValidation().done(function () {
          return _this3.set('validationRequest', null);
        });
      }
    }
  }.observes('currentTabIndex')

});

});

require.register("controllers/wizard/step8_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');
var fileUtils = require('utils/file_utils');

App.WizardStep8Controller = Em.Controller.extend(App.AddSecurityConfigs, App.wizardDeployProgressControllerMixin, App.ConfigOverridable, App.ConfigsSaverMixin, {

  name: 'wizardStep8Controller',

  /**
   * @type {boolean}
   */
  isAddService: Em.computed.equal('content.controllerName', 'addServiceController'),

  /**
   * @type {boolean}
   */
  isAddHost: Em.computed.equal('content.controllerName', 'addHostController'),

  /**
   * @type {boolean}
   */
  isInstaller: Em.computed.equal('content.controllerName', 'installerController'),

  /**
   * List of raw data about cluster that should be displayed
   * @type {Array}
   */
  rawContent: [{
    config_name: 'Admin',
    display_name: 'Admin Name',
    config_value: ''
  }, {
    config_name: 'cluster',
    display_name: 'Cluster Name',
    config_value: ''
  }, {
    config_name: 'hosts',
    display_name: 'Total Hosts',
    config_value: ''
  }, {
    config_name: 'Repo',
    display_name: 'Local Repository',
    config_value: ''
  }],

  /**
   * List of data about cluster (based on formatted <code>rawContent</code>)
   * @type {Object[]}
   */
  clusterInfo: [],

  /**
   * List of services with components assigned to hosts
   * @type {Object[]}
   */
  services: [],

  /**
   * @type {Object[]}
   */
  configs: [],

  /**
   * True if Kerberos is installed on the cluster and the kdc_type on the server is set to "none"
   * @type {Boolean}
   */
  isManualKerberos: Em.computed.equal('App.router.mainAdminKerberosController.kdc_type', 'none'),

  showDownloadCsv: function () {
    return !!App.get('router.mainAdminKerberosController.kdc_type');
  }.property('App.router.mainAdminKerberosController.kdc_type'),

  /**
   * Should Submit button be disabled
   * @type {bool}
   */
  isSubmitDisabled: false,

  /**
   * Should Back button be disabled
   * @type {bool}
   */
  isBackBtnDisabled: false,

  /**
   * Is error appears while <code>ajaxQueue</code> executes
   * @type {bool}
   */
  hasErrorOccurred: false,

  /**
   * Are services installed
   * Used to hide Deploy Progress Bar
   * @type {bool}
   */
  servicesInstalled: false,

  /**
   * List of service config tags
   * @type {Object[]}
   */
  serviceConfigTags: [],

  /**
   * Selected config group
   * @type {Object}
   */
  selectedConfigGroup: null,

  /**
   * List of config groups
   * @type {Object[]}
   */
  configGroups: [],

  /**
   * List of selected but not installed services
   * @type {Object[]}
   */
  selectedServices: function () {
    return this.get('content.services').filterProperty('isSelected', true).filterProperty('isInstalled', false);
  }.property('content.services.@each.isSelected', 'content.services.@each.isInstalled').cacheable(),

  /**
   * List of installed services
   * @type {Object[]}
   */
  installedServices: Em.computed.filterBy('content.services', 'isInstalled', true),

  /**
   * Current cluster name
   * @type {string}
   */
  clusterName: Em.computed.alias('content.cluster.name'),

  /**
   * List of existing cluster names
   * @type {string[]}
   */
  clusterNames: [],

  /**
   * Number of completed cluster delete requests
   * @type {number}
   */
  clusterDeleteRequestsCompleted: 0,

  /**
   * Number of existing repo_versions
   * @type {number}
   */
  existingRepositoryVersions: 0,

  /**
   * Indicates if all cluster delete requests are completed
   * @type {boolean}
   */
  isAllClusterDeleteRequestsCompleted: Em.computed.equalProperties('clusterDeleteRequestsCompleted', 'clusterNames.length'),

  /**
   * Error popup body views for clusters that couldn't be deleted
   * @type {App.AjaxDefaultErrorPopupBodyView[]}
   */
  clusterDeleteErrorViews: [],

  /**
   * Clear current step data
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.get('services').clear();
    this.get('configs').clear();
    this.get('clusterInfo').clear();
    this.get('serviceConfigTags').clear();
    this.set('servicesInstalled', false);
    this.set('ajaxQueueLength', 0);
    this.set('ajaxRequestsQueue', App.ajaxQueue.create());
    this.set('ajaxRequestsQueue.finishedCallback', this.ajaxQueueFinished);
    this.get('clusterDeleteErrorViews').clear();
    this.set('clusterDeleteRequestsCompleted', 0);
  },

  /**
   * Load current step data
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.clearStep();
    if (this.get('content.serviceConfigProperties')) {
      this.formatProperties();
      this.loadConfigs();
    }
    this.loadClusterInfo();
    this.loadServices();
    this.set('isSubmitDisabled', false);
    this.set('isBackBtnDisabled', false);
  },

  /**
   * replace whitespace character with coma between directories
   * @method formatProperties
   */
  formatProperties: function formatProperties() {
    this.get('content.serviceConfigProperties').forEach(function (_configProperty) {
      _configProperty.value = typeof _configProperty.value === "boolean" ? _configProperty.value.toString() : App.config.trimProperty(_configProperty, false);
    });
  },

  /**
   * Load all site properties
   * @method loadConfigs
   */
  loadConfigs: function loadConfigs() {
    this.set('configs', this.get('content.serviceConfigProperties').filter(function (config) {
      return !config.group;
    }));
  },

  /**
   * Format <code>content.hosts</code> from Object to Array
   * @returns {Array}
   * @method getRegisteredHosts
   */
  getRegisteredHosts: function getRegisteredHosts() {
    var allHosts = this.get('content.hosts');
    var hosts = [];
    for (var hostName in allHosts) {
      if (allHosts.hasOwnProperty(hostName)) {
        if (allHosts[hostName].bootStatus === 'REGISTERED') {
          allHosts[hostName].hostName = allHosts[hostName].name;
          hosts.pushObject(allHosts[hostName]);
        }
      }
    }
    return hosts;
  },

  /**
   * Load all info about cluster to <code>clusterInfo</code> variable
   * @method loadClusterInfo
   */
  loadClusterInfo: function loadClusterInfo() {

    //Admin name
    var admin = this.rawContent.findProperty('config_name', 'Admin');
    admin.config_value = App.db.getLoginName();
    if (admin.config_value) {
      this.get('clusterInfo').pushObject(Ember.Object.create(admin));
    }

    // cluster name
    var cluster = this.rawContent.findProperty('config_name', 'cluster');
    cluster.config_value = this.get('content.cluster.name');
    this.get('clusterInfo').pushObject(Ember.Object.create(cluster));

    //hosts
    var newHostsCount = 0;
    var totalHostsCount = 0;
    var hosts = this.get('content.hosts');
    for (var hostName in hosts) {
      newHostsCount += ~~!hosts[hostName].isInstalled;
      totalHostsCount++;
    }

    var totalHostsObj = this.rawContent.findProperty('config_name', 'hosts');
    totalHostsObj.config_value = totalHostsCount + ' (' + newHostsCount + ' new)';
    this.get('clusterInfo').pushObject(Em.Object.create(totalHostsObj));

    //repo
    if (this.get('isAddService') || this.get('isAddHost')) {
      // For some stacks there is no info regarding stack versions to upgrade, e.g. HDP-2.1
      if (App.StackVersion.find().get('content.length')) {
        this.loadRepoInfo();
      } else {
        this.loadDefaultRepoInfo();
      }
    } else {
      // from install wizard
      var selectedStack = App.Stack.find().findProperty('isSelected', true);
      var allRepos = [];
      if (selectedStack && selectedStack.get('operatingSystems')) {
        selectedStack.get('operatingSystems').forEach(function (os) {
          if (os.get('isSelected')) {
            os.get('repositories').forEach(function (repo) {
              if (repo.get('showRepo')) {
                allRepos.push(Em.Object.create({
                  base_url: repo.get('baseUrl'),
                  os_type: repo.get('osType'),
                  repo_id: repo.get('repoId')
                }));
              }
            }, this);
          }
        }, this);
      }
      allRepos.set('display_name', Em.I18n.t("installer.step8.repoInfo.displayName"));
      this.get('clusterInfo').set('useRedhatSatellite', selectedStack.get('useRedhatSatellite'));
      this.get('clusterInfo').set('repoInfo', allRepos);
    }
  },

  /**
   * Load repo info for add Service/Host wizard review page
   * @return {$.ajax|null}
   * @method loadRepoInfo
   */
  loadRepoInfo: function loadRepoInfo() {
    var stackName = App.get('currentStackName');

    var currentRepoVersion;
    App.RepositoryVersion.find().forEach(function (repoVersion) {
      if (repoVersion.get('stackVersionType') === stackName && repoVersion.get('isCurrent') && repoVersion.get('isStandard')) {
        currentRepoVersion = repoVersion.get('repositoryVersion');
      }
    });

    if (!currentRepoVersion) {
      console.error('Error while getting current stack repository version');
    }

    return App.ajax.send({
      name: 'cluster.load_repo_version',
      sender: this,
      data: {
        stackName: stackName,
        repositoryVersion: currentRepoVersion
      },
      success: 'loadRepoInfoSuccessCallback',
      error: 'loadRepoInfoErrorCallback'
    });
  },

  /**
   * Save all repo base URL of all OS type to <code>repoInfo<code>
   * @param {object} data
   * @method loadRepoInfoSuccessCallback
   */
  loadRepoInfoSuccessCallback: function loadRepoInfoSuccessCallback(data) {
    Em.assert('Current repo-version may be only one', data.items.length === 1);
    if (data.items.length) {
      var allRepos = this.generateRepoInfo(Em.getWithDefault(data, 'items.0.repository_versions.0.operating_systems', []));
      allRepos.set('display_name', Em.I18n.t("installer.step8.repoInfo.displayName"));
      this.get('clusterInfo').set('repoInfo', allRepos);
      //if the property is missing, set as false
      this.get('clusterInfo').set('useRedhatSatellite', data.items[0].repository_versions[0].operating_systems[0].OperatingSystems.ambari_managed_repositories === false);
    } else {
      this.loadDefaultRepoInfo();
    }
  },

  /**
   * Generate list regarding info about OS versions and repositories.
   *
   * @param {Object{}} oses - OS array
   * @returns {Em.Object[]}
   */
  generateRepoInfo: function generateRepoInfo(oses) {
    return oses.map(function (os) {
      return os.repositories.map(function (repository) {
        return Em.Object.create({
          base_url: repository.Repositories.base_url,
          os_type: repository.Repositories.os_type,
          repo_id: repository.Repositories.repo_id
        });
      });
    }).reduce(function (p, c) {
      return p.concat(c);
    });
  },

  /**
   * Load repo info from stack. Used if installed stack doesn't have upgrade info.
   *
   * @returns {$.Deferred}
   * @method loadDefaultRepoInfo
   */
  loadDefaultRepoInfo: function loadDefaultRepoInfo() {
    var nameVersionCombo = App.get('currentStackVersion').split('-');

    return App.ajax.send({
      name: 'cluster.load_repositories',
      sender: this,
      data: {
        stackName: nameVersionCombo[0],
        stackVersion: nameVersionCombo[1]
      },
      success: 'loadDefaultRepoInfoSuccessCallback',
      error: 'loadRepoInfoErrorCallback'
    });
  },

  /**
   * @param {Object} data - JSON data from server
   * @method loadDefaultRepoInfoSuccessCallback
   */
  loadDefaultRepoInfoSuccessCallback: function loadDefaultRepoInfoSuccessCallback(data) {
    var allRepos = this.generateRepoInfo(Em.getWithDefault(data, 'items', []));
    allRepos.set('display_name', Em.I18n.t("installer.step8.repoInfo.displayName"));
    this.get('clusterInfo').set('repoInfo', allRepos);
    //if the property is missing, set as false
    this.get('clusterInfo').set('useRedhatSatellite', data.items[0].OperatingSystems.ambari_managed_repositories === false);
  },

  /**
   * @method loadRepoInfoErrorCallback
   */
  loadRepoInfoErrorCallback: function loadRepoInfoErrorCallback() {
    var allRepos = [];
    allRepos.set('display_name', Em.I18n.t("installer.step8.repoInfo.displayName"));
    this.get('clusterInfo').set('repoInfo', allRepos);
  },

  /**
   * Load all info about services to <code>services</code> variable
   * @method loadServices
   */
  loadServices: function loadServices() {
    this.get('selectedServices').filterProperty('isHiddenOnSelectServicePage', false).forEach(function (service) {
      var serviceObj = Em.Object.create({
        service_name: service.get('serviceName'),
        display_name: service.get('displayNameOnSelectServicePage'),
        service_components: Em.A([])
      });
      service.get('serviceComponents').forEach(function (component) {
        // show clients for services that have only clients components
        if ((component.get('isClient') || component.get('isRequiredOnAllHosts')) && !service.get('isClientOnlyService')) return;
        // no HA component
        if (component.get('isHAComponentOnly')) return;
        // skip if component is not allowed on single node cluster
        if (Object.keys(this.get('content.hosts')).length === 1 && component.get('isNotAllowedOnSingleNodeCluster')) return;
        var displayName;
        if (component.get('isClient')) {
          displayName = Em.I18n.t('common.clients');
        } else {
          // remove service name from component display name
          displayName = App.format.role(component.get('componentName'), false).replace(new RegExp('^' + service.get('serviceName') + '\\s', 'i'), '');
        }

        var componentName = component.get('componentName');
        var masterComponents = this.get('content.masterComponentHosts');
        var isMasterComponentSelected = masterComponents.someProperty('component', componentName);
        var isMaster = component.get('isMaster');

        if (!isMaster || isMasterComponentSelected) {
          serviceObj.get('service_components').pushObject(Em.Object.create({
            component_name: component.get('isClient') ? Em.I18n.t('common.client').toUpperCase() : component.get('componentName'),
            display_name: displayName,
            component_value: this.assignComponentHosts(component)
          }));
        }
      }, this);
      if (service.get('customReviewHandler')) {
        for (var displayName in service.get('customReviewHandler')) {
          serviceObj.get('service_components').pushObject(Em.Object.create({
            display_name: displayName,
            component_value: this.assignComponentHosts(Em.Object.create({
              customHandler: service.get('customReviewHandler.' + displayName)
            }))
          }));
        }
      }
      this.get('services').pushObject(serviceObj);
    }, this);
  },

  /**
   * Set <code>component_value</code> property to <code>component</code>
   * @param {Em.Object} component
   * @return {String}
   * @method assignComponentHosts
   */
  assignComponentHosts: function assignComponentHosts(component) {
    var componentValue;
    if (component.get('customHandler')) {
      componentValue = this[component.get('customHandler')].call(this, component);
    } else {
      if (component.get('isMaster')) {
        componentValue = this.getMasterComponentValue(component.get('componentName'));
      } else {
        var componentName = component.get('isClient') ? Em.I18n.t('common.client').toUpperCase() : component.get('componentName');
        var hostsLength = this.get('content.slaveComponentHosts').findProperty('componentName', componentName).hosts.length;
        componentValue = hostsLength + Em.I18n.t('installer.step8.host' + (hostsLength > 1 ? 's' : ''));
      }
    }
    return componentValue;
  },

  getMasterComponentValue: function getMasterComponentValue(componentName) {
    var masterComponents = this.get('content.masterComponentHosts');
    var hostsCount = masterComponents.filterProperty('component', componentName).length;
    return stringUtils.pluralize(hostsCount, masterComponents.findProperty('component', componentName).hostName, hostsCount + ' ' + Em.I18n.t('installer.step8.hosts'));
  },

  loadHiveDbValue: function loadHiveDbValue() {
    return this.loadDbValue('HIVE');
  },

  loadOozieDbValue: function loadOozieDbValue() {
    return this.loadDbValue('OOZIE');
  },

  /**
   * Set displayed Hive DB value based on DB type
   * @method loadHiveDbValue
   */
  loadDbValue: function loadDbValue(serviceName) {
    var serviceConfigProperties = this.get('content.serviceConfigProperties');
    var dbFull = serviceConfigProperties.findProperty('name', serviceName.toLowerCase() + '_database');
    //db = serviceConfigProperties.findProperty('name', serviceName.toLowerCase() + '_ambari_database');
    //since db.value contains the intial default value of <service>_admin_database (MySQL) and not the actual db type selected,
    //ignore the value when displaying the database name on the summary page
    return dbFull ? dbFull.value : '';
  },

  /**
   * Set displayed HBase master value
   * @param {Object} hbaseMaster
   * @method loadHbaseMasterValue
   */
  loadHbaseMasterValue: function loadHbaseMasterValue(hbaseMaster) {
    var hbaseHostName = this.get('content.masterComponentHosts').filterProperty('component', hbaseMaster.component_name);
    if (hbaseHostName.length === 1) {
      hbaseMaster.set('component_value', hbaseHostName[0].hostName);
    } else {
      hbaseMaster.set('component_value', hbaseHostName[0].hostName + " " + Em.I18n.t('installer.step8.other').format(hbaseHostName.length - 1));
    }
  },

  /**
   * Set displayed ZooKeeper Server value
   * @param {Object} serverComponent
   * @method loadZkServerValue
   */
  loadZkServerValue: function loadZkServerValue(serverComponent) {
    var zkHostNames = this.get('content.masterComponentHosts').filterProperty('component', serverComponent.component_name).length;
    var hostSuffix;
    if (zkHostNames === 1) {
      hostSuffix = Em.I18n.t('installer.step8.host');
    } else {
      hostSuffix = Em.I18n.t('installer.step8.hosts');
    }
    serverComponent.set('component_value', zkHostNames + hostSuffix);
  },

  /**
   * Onclick handler for <code>next</code> button
   * @method submit
   * @return {void}
   */
  submit: function submit() {
    var wizardController;
    if (!this.get('isSubmitDisabled')) {
      wizardController = App.router.get(this.get('content.controllerName'));
      wizardController.setLowerStepsDisable(wizardController.get('currentStep'));
      this.set('isSubmitDisabled', true);
      this.set('isBackBtnDisabled', true);
      this.showRestartWarnings().then(this.checkKDCSession.bind(this));
    }
  },

  /**
   * Warn user about services that will be restarted during installation.
   *
   * @returns {$.Deferred}
   */
  showRestartWarnings: function showRestartWarnings() {
    var self = this;
    var dfd = $.Deferred();
    var wizardController = App.router.get(this.get('content.controllerName'));
    var selectedServiceNames = this.get('selectedServices').mapProperty('serviceName');
    var installedServiceNames = this.get('installedServices').mapProperty('serviceName');

    if (this.get('isAddService') && selectedServiceNames.contains('OOZIE')) {
      var affectedServices = ['HDFS', 'YARN'].filter(function (serviceName) {
        return installedServiceNames.contains(serviceName);
      });
      if (affectedServices.length) {
        var serviceNames = affectedServices.length > 1 ? '<b>{0}</b> {1} <b>{2}</b>'.format(affectedServices[0], Em.I18n.t('and'), affectedServices[1]) : '<b>' + affectedServices[0] + '</b> ';
        App.ModalPopup.show({
          encodeBody: false,
          header: Em.I18n.t('common.warning'),
          body: Em.I18n.t('installer.step8.services.restart.required').format(serviceNames, stringUtils.pluralize(affectedServices.length, Em.I18n.t('common.service').toLowerCase())),
          secondary: Em.I18n.t('common.cancel'),
          primary: Em.I18n.t('common.proceedAnyway'),
          onPrimary: function onPrimary() {
            this.hide();
            dfd.resolve();
          },
          onClose: function onClose() {
            this.hide();
            self.set('isSubmitDisabled', false);
            self.set('isBackBtnDisabled', false);
            wizardController.setStepsEnable();
            dfd.reject();
          },
          onSecondary: function onSecondary() {
            this.onClose();
          }
        });
      } else {
        dfd.resolve();
      }
    } else {
      dfd.resolve();
    }
    return dfd.promise();
  },

  checkKDCSession: function checkKDCSession() {
    var self = this;
    var wizardController = App.router.get(this.get('content.controllerName'));
    if (!this.get('isInstaller')) {
      App.get('router.mainAdminKerberosController').getKDCSessionState(this.submitProceed.bind(this), function () {
        self.set('isSubmitDisabled', false);
        self.set('isBackBtnDisabled', false);
        wizardController.setStepsEnable();
        if (self.get('isAddService')) {
          wizardController.setSkipSlavesStep(wizardController.getDBProperty('selectedServiceNames'), 3);
        }
      });
    } else {
      this.submitProceed();
    }
  },

  /**
   * Prepare <code>ajaxQueue</code> and start to execute it
   * @method submitProceed
   */
  submitProceed: function submitProceed() {
    var self = this;
    this.set('clusterDeleteRequestsCompleted', 0);
    this.get('clusterDeleteErrorViews').clear();
    if (this.get('isAddHost')) {
      App.router.get('addHostController').setLowerStepsDisable(4);
    }

    // checkpoint the cluster status on the server so that the user can resume from where they left off
    switch (this.get('content.controllerName')) {
      case 'installerController':
        App.clusterStatus.setClusterStatus({
          clusterName: this.get('clusterName'),
          clusterState: 'CLUSTER_DEPLOY_PREP_2',
          wizardControllerName: this.get('content.controllerName'),
          localdb: App.db.data
        });
        break;
      case 'addHostController':
        App.clusterStatus.setClusterStatus({
          clusterName: this.get('clusterName'),
          clusterState: 'ADD_HOSTS_DEPLOY_PREP_2',
          wizardControllerName: this.get('content.controllerName'),
          localdb: App.db.data
        });
        break;
      case 'addServiceController':
        App.clusterStatus.setClusterStatus({
          clusterName: this.get('clusterName'),
          clusterState: 'ADD_SERVICES_DEPLOY_PREP_2',
          wizardControllerName: this.get('content.controllerName'),
          localdb: App.db.data
        });
        break;
      default:
        break;
    }
    // delete any existing clusters to start from a clean slate
    // before creating a new cluster in install wizard
    // TODO: modify for multi-cluster support
    this.getExistingClusterNames().complete(function () {
      var clusterNames = self.get('clusterNames');
      if (self.get('isInstaller') && !App.get('testMode') && clusterNames.length) {
        self.deleteClusters(clusterNames);
      } else {
        self.getExistingVersions();
      }
    });
  },

  /**
   * Get list of existing cluster names
   * @returns {object|null}
   * returns an array of existing cluster names.
   * returns an empty array if there are no existing clusters.
   * @method getExistingClusterNames
   */
  getExistingClusterNames: function getExistingClusterNames() {
    return App.ajax.send({
      name: 'wizard.step8.existing_cluster_names',
      sender: this,
      success: 'getExistingClusterNamesSuccessCallBack',
      error: 'getExistingClusterNamesErrorCallback'
    });
  },

  /**
   * Save received list to <code>clusterNames</code>
   * @param {Object} data
   * @method getExistingClusterNamesSuccessCallBack
   */
  getExistingClusterNamesSuccessCallBack: function getExistingClusterNamesSuccessCallBack(data) {
    var clusterNames = data.items.mapProperty('Clusters.cluster_name');
    this.set('clusterNames', clusterNames);
  },

  /**
   * If error appears, set <code>clusterNames</code> to <code>[]</code>
   * @method getExistingClusterNamesErrorCallback
   */
  getExistingClusterNamesErrorCallback: function getExistingClusterNamesErrorCallback() {
    this.set('clusterNames', []);
  },

  /**
   * Delete cluster by name
   * One request for one cluster!
   * @param {string[]} clusterNames
   * @method deleteClusters
   */
  deleteClusters: function deleteClusters(clusterNames) {
    this.get('clusterDeleteErrorViews').clear();
    clusterNames.forEach(function (clusterName, index) {
      App.ajax.send({
        name: 'common.delete.cluster',
        sender: this,
        data: {
          name: clusterName,
          isLast: index === clusterNames.length - 1
        },
        success: 'deleteClusterSuccessCallback',
        error: 'deleteClusterErrorCallback'
      });
    }, this);
  },

  /**
   * Method to execute after successful cluster deletion
   * @method deleteClusterSuccessCallback
   */
  deleteClusterSuccessCallback: function deleteClusterSuccessCallback() {
    this.incrementProperty('clusterDeleteRequestsCompleted');
    if (this.get('isAllClusterDeleteRequestsCompleted')) {
      if (this.get('clusterDeleteErrorViews.length')) {
        this.showDeleteClustersErrorPopup();
      } else {
        this.getExistingVersions();
      }
    }
  },

  /**
   * Method to execute after failed cluster deletion
   * @param {object} request
   * @param {string} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @method deleteClusterErrorCallback
   */
  deleteClusterErrorCallback: function deleteClusterErrorCallback(request, ajaxOptions, error, opt) {
    this.incrementProperty('clusterDeleteRequestsCompleted');
    try {
      var json = $.parseJSON(request.responseText);
      var message = json.message;
    } catch (err) {}
    this.get('clusterDeleteErrorViews').pushObject(App.AjaxDefaultErrorPopupBodyView.create({
      url: opt.url,
      type: opt.type,
      status: request.status,
      message: message
    }));
    if (this.get('isAllClusterDeleteRequestsCompleted')) {
      this.showDeleteClustersErrorPopup();
    }
  },

  /**
   * Show error popup if cluster deletion failed
   * @method showDeleteClustersErrorPopup
   */
  showDeleteClustersErrorPopup: function showDeleteClustersErrorPopup() {
    var self = this;
    this.setProperties({
      isSubmitDisabled: false,
      isBackBtnDisabled: false
    });
    App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      onPrimary: function onPrimary() {
        this.hide();
      },
      bodyClass: Em.ContainerView.extend({
        childViews: self.get('clusterDeleteErrorViews')
      })
    });
  },

  /**
   * Get existing repo_versions
   * @method getExistingVersions
   */
  getExistingVersions: function getExistingVersions() {
    return App.ajax.send({
      name: 'wizard.get_version_definitions',
      sender: this,
      success: 'getExistingVersionsSuccessCallback'
    });
  },

  /**
   * @param {Object} data
   * @method getExistingVersionsSuccessCallback
   */
  getExistingVersionsSuccessCallback: function getExistingVersionsSuccessCallback(data) {
    if (this.get('isInstaller') && !App.get('testMode') && data.items.length) {
      this.set('existingRepositoryVersions', data.items.length);
      this.deleteExistingVersions(data.items);
    } else {
      this.startDeploy();
    }
  },

  /**
   * Delete existing repo_versions
   * @param {Array} versions
   * @method deleteExistingVersions
   */
  deleteExistingVersions: function deleteExistingVersions(versions) {
    versions.forEach(function (version) {
      App.ajax.send({
        name: 'wizard.delete_repository_versions',
        sender: this,
        data: {
          id: version.VersionDefinition.id,
          stackName: version.VersionDefinition.stack_name,
          stackVersion: version.VersionDefinition.stack_version
        },
        success: 'deleteExistingVersionsSuccessCallback'
      });
    }, this);
  },

  /**
   * Method to execute after successful version deletion
   * @method deleteExistingVersionsSuccessCallback
   */
  deleteExistingVersionsSuccessCallback: function deleteExistingVersionsSuccessCallback() {
    this.decrementProperty('existingRepositoryVersions');
    if (this.get('existingRepositoryVersions') === 0) {
      this.startDeploy();
    }
  },

  /**
   * updates kerberosDescriptorConfigs
   * @method updateKerberosDescriptor
   */
  updateKerberosDescriptor: function updateKerberosDescriptor(instant) {
    var kerberosDescriptor = this.get('wizardController').getDBProperty('kerberosDescriptorConfigs');
    var descriptorExists = this.get('wizardController').getDBProperty('isClusterDescriptorExists') === true;

    var ajaxOpts = {
      name: descriptorExists ? 'admin.kerberos.cluster.artifact.update' : 'admin.kerberos.cluster.artifact.create',
      data: {
        artifactName: 'kerberos_descriptor',
        data: {
          artifact_data: this.removeIdentityReferences(kerberosDescriptor)
        }
      }
    };
    if (instant) {
      ajaxOpts.sender = this;
      App.ajax.send(ajaxOpts);
    } else {
      this.addRequestToAjaxQueue(ajaxOpts);
    }
  },

  /**
   * To Start deploy process
   * @method startDeploy
   */
  startDeploy: function startDeploy() {
    if (!this.get('isInstaller')) {
      this._startDeploy();
    } else {
      var installerController = App.router.get('installerController');
      var versionData = installerController.getSelectedRepoVersionData();
      if (versionData) {
        var self = this;
        installerController.postVersionDefinitionFileStep8(versionData.isXMLdata, versionData.data).done(function (versionInfo) {
          if (versionInfo.id && versionInfo.stackName && versionInfo.stackVersion) {
            var selectedStack = App.Stack.find().findProperty('isSelected', true);
            if (selectedStack) {
              selectedStack.set('versionInfoId', versionInfo.id);
            }
            installerController.updateRepoOSInfo(versionInfo, selectedStack).done(function () {
              self._startDeploy();
            });
          }
        });
      } else {
        this._startDeploy();
      }
    }
  },

  /**
   * Start deploy process
   * @method startDeploy
   */
  _startDeploy: function _startDeploy() {
    this.createCluster();
    this.createSelectedServices();
    if (!this.get('isAddHost')) {
      if (this.get('isAddService')) {
        // for manually enabled Kerberos descriptor was updated on transition to this step
        if (App.get('isKerberosEnabled') && !this.get('isManualKerberos')) {
          this.updateKerberosDescriptor();
        }
        var fileNamesToUpdate = this.get('wizardController').getDBProperty('fileNamesToUpdate').uniq();
        if (fileNamesToUpdate && fileNamesToUpdate.length) {
          this.applyConfigurationsToCluster(this.generateDesiredConfigsJSON(this.get('configs'), fileNamesToUpdate));
        }
      }
      this.createConfigurations();
      this.applyConfigurationsToCluster(this.get('serviceConfigTags'));
    }
    this.createComponents();
    this.registerHostsToCluster();
    this.createConfigurationGroups();
    this.createMasterHostComponents();
    this.createSlaveAndClientsHostComponents();
    if (this.get('isAddService')) {
      this.createAdditionalClientComponents();
    }
    this.createAdditionalHostComponents();

    this.set('ajaxQueueLength', this.get('ajaxRequestsQueue.queue.length'));
    this.get('ajaxRequestsQueue').start();
    this.showLoadingIndicator();
  },

  /**
   * *******************************************************************
   * The following create* functions are called upon submitting Step 8.
   * *******************************************************************
   */

  /**
   * Create cluster using selected stack version
   * Queued request
   * @method createCluster
   */
  createCluster: function createCluster() {
    if (!this.get('isInstaller')) return;
    var stackVersion = this.get('content.installOptions.localRepo') ? App.currentStackVersion.replace(/(-\d+(\.\d)*)/ig, "Local$&") : App.currentStackVersion;
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.create_cluster',
      data: {
        data: JSON.stringify({ "Clusters": { "version": stackVersion } })
      },
      success: 'createClusterSuccess'
    });
  },

  createClusterSuccess: function createClusterSuccess(data, xhr, params) {
    App.set('clusterName', params.cluster);
  },

  /**
   * Create selected to install services
   * Queued request
   * Skipped if no services where selected!
   * @method createSelectedServices
   */
  createSelectedServices: function createSelectedServices() {
    var data = this.createSelectedServicesData();
    if (!data.length) return;
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.create_selected_services',
      data: {
        data: JSON.stringify(data)
      }
    });
  },

  /**
   * Format data for <code>createSelectedServices</code> request
   * @returns {Object[]}
   * @method createSelectedServicesData
   */
  createSelectedServicesData: function createSelectedServicesData() {
    var selectedStack;
    if (this.get('isInstaller')) {
      selectedStack = App.Stack.find().findProperty('isSelected', true);
    }
    return this.get('selectedServices').map(function (service) {
      return selectedStack ? { "ServiceInfo": { "service_name": service.get('serviceName'), "desired_repository_version_id": selectedStack.get('versionInfoId') } } : { "ServiceInfo": { "service_name": service.get('serviceName') } };
    });
  },

  /**
   * Create components for selected services
   * Queued requests
   * One request for each service!
   * @method createComponents
   */
  createComponents: function createComponents() {
    var serviceComponents = App.StackServiceComponent.find().filterProperty('isInstallable');
    this.get('selectedServices').forEach(function (_service) {
      var serviceName = _service.get('serviceName');
      var componentsData = serviceComponents.filterProperty('serviceName', serviceName).map(function (_component) {
        return { "ServiceComponentInfo": { "component_name": _component.get('componentName') } };
      });

      // Service must be specified in terms of a query for creating multiple components at the same time.
      // See AMBARI-1018.
      this.addRequestToCreateComponent(componentsData, serviceName);
    }, this);

    if (this.get('isAddHost')) {
      var allServiceComponents = [];
      var services = App.Service.find().mapProperty('serviceName');
      services.forEach(function (_service) {
        var _serviceComponents = App.Service.find(_service).get('serviceComponents');
        allServiceComponents = allServiceComponents.concat(_serviceComponents);
      }, this);
      this.get('content.slaveComponentHosts').forEach(function (component) {
        if (component.componentName !== 'CLIENT' && !allServiceComponents.contains(component.componentName)) {
          this.addRequestToCreateComponent([{ "ServiceComponentInfo": { "component_name": component.componentName } }], App.StackServiceComponent.find().findProperty('componentName', component.componentName).get('serviceName'));
        }
      }, this);
      this.get('content.clients').forEach(function (component) {
        if (!allServiceComponents.contains(component.component_name)) {
          this.addRequestToCreateComponent([{ "ServiceComponentInfo": { "component_name": component.component_name } }], App.StackServiceComponent.find().findProperty('componentName', component.component_name).get('serviceName'));
        }
      }, this);
    }
  },

  /**
   * Add request to ajax queue to create service component
   * @param componentsData
   * @param serviceName
   */
  addRequestToCreateComponent: function addRequestToCreateComponent(componentsData, serviceName) {
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.create_components',
      data: {
        data: JSON.stringify({ "components": componentsData }),
        serviceName: serviceName
      }
    });
  },

  /**
   * Error callback for new service component request
   * So, if component doesn't exist we should create it
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   * @method newServiceComponentErrorCallback
   */
  newServiceComponentErrorCallback: function newServiceComponentErrorCallback(request, ajaxOptions, error, opt, params) {
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.create_components',
      data: {
        serviceName: params.serviceName,
        data: JSON.stringify({
          "components": [{
            "ServiceComponentInfo": {
              "component_name": params.componentName
            }
          }]
        })
      }
    });
  },

  /**
   * Register hosts
   * Queued request
   * @method registerHostsToCluster
   */
  registerHostsToCluster: function registerHostsToCluster() {
    var data = this.createRegisterHostData();
    if (!data.length) return;
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.register_host_to_cluster',
      data: {
        data: JSON.stringify(data)
      }
    });
  },

  /**
   * Format request-data for <code>registerHostsToCluster</code>
   * @returns {Object}
   * @method createRegisterHostData
   */
  createRegisterHostData: function createRegisterHostData() {
    return this.getRegisteredHosts().filterProperty('isInstalled', false).map(function (host) {
      return { "Hosts": { "host_name": host.hostName } };
    });
  },

  /**
   * Register new master components
   * @uses registerHostsToComponent
   * @method createMasterHostComponents
   */
  createMasterHostComponents: function createMasterHostComponents() {
    var masterOnAllHosts = [];

    this.get('content.services').filterProperty('isSelected').forEach(function (service) {
      service.get('serviceComponents').filterProperty('isRequiredOnAllHosts').forEach(function (component) {
        if (component.get('isMaster')) {
          masterOnAllHosts.push(component.get('componentName'));
        }
      }, this);
    }, this);

    // create master components for only selected services.
    var selectedMasterComponents = this.get('content.masterComponentHosts').filter(function (_component) {
      return this.get('selectedServices').mapProperty('serviceName').contains(_component.serviceId);
    }, this);
    selectedMasterComponents.mapProperty('component').uniq().forEach(function (component) {
      var hostNames = [];
      if (masterOnAllHosts.length > 0) {
        var compOnAllHosts = false;
        for (var i = 0; i < masterOnAllHosts.length; i++) {
          if (component === masterOnAllHosts[i]) {
            compOnAllHosts = true;
            break;
          }
        }
        if (!compOnAllHosts) {
          hostNames = selectedMasterComponents.filterProperty('component', component).filterProperty('isInstalled', false).mapProperty('hostName');
          this.registerHostsToComponent(hostNames, component);
        }
      } else {
        hostNames = selectedMasterComponents.filterProperty('component', component).filterProperty('isInstalled', false).mapProperty('hostName');
        this.registerHostsToComponent(hostNames, component);
      }
    }, this);
  },

  getClientsMap: function getClientsMap(flag) {
    var clients = App.StackServiceComponent.find().filterProperty('isClient'),
        clientsMap = {},
        dependedComponents = flag ? App.StackServiceComponent.find().filterProperty(flag) : App.StackServiceComponent.find();
    clients.forEach(function (client) {
      var clientName = client.get('componentName');
      clientsMap[clientName] = Em.A([]);
      dependedComponents.forEach(function (component) {
        if (component.dependsOn(client)) clientsMap[clientName].push(component.get('componentName'));
      });
      if (!clientsMap[clientName].length) delete clientsMap[clientName];
    });
    return clientsMap;
  },

  /**
   * Register slave components and clients
   * @uses registerHostsToComponent
   * @method createSlaveAndClientsHostComponents
   */
  createSlaveAndClientsHostComponents: function createSlaveAndClientsHostComponents() {
    var masterHosts = this.get('content.masterComponentHosts');
    var slaveHosts = this.get('content.slaveComponentHosts');
    var clients = this.get('content.clients').filterProperty('isInstalled', false);
    var slaveOnAllHosts = [];
    var clientOnAllHosts = [];

    this.get('content.services').filterProperty('isSelected').forEach(function (service) {
      service.get('serviceComponents').filterProperty('isRequiredOnAllHosts').forEach(function (component) {
        if (component.get('isClient')) {
          clientOnAllHosts.push(component.get('componentName'));
        } else if (component.get('isSlave')) {
          slaveOnAllHosts.push(component.get('componentName'));
        }
      }, this);
    }, this);

    /**
     * Determines on which hosts client should be installed (based on availability of master components on hosts)
     * @type {Object}
     * Format:
     * <code>
     *  {
     *    CLIENT1: Em.A([MASTER1, MASTER2, ...]),
     *    CLIENT2: Em.A([MASTER3, MASTER1, ...])
     *    ...
     *  }
     * </code>
     */
    var clientsToMasterMap = this.getClientsMap('isMaster'),
        clientsToSlaveMap = this.getClientsMap('isSlave');

    slaveHosts.forEach(function (_slave) {
      var hostNames = [];
      var compOnAllHosts;
      if (_slave.componentName !== 'CLIENT') {
        if (slaveOnAllHosts.length > 0) {
          compOnAllHosts = false;
          for (var i = 0; i < slaveOnAllHosts.length; i++) {
            if (_slave.componentName === slaveOnAllHosts[i]) {
              // component with ALL cardinality should not
              // registerHostsToComponent in createSlaveAndClientsHostComponents
              compOnAllHosts = true;
              break;
            }
          }
          if (!compOnAllHosts) {
            hostNames = _slave.hosts.filterProperty('isInstalled', false).mapProperty('hostName');
            this.registerHostsToComponent(hostNames, _slave.componentName);
          }
        } else {
          hostNames = _slave.hosts.filterProperty('isInstalled', false).mapProperty('hostName');
          this.registerHostsToComponent(hostNames, _slave.componentName);
        }
      } else {
        clients.forEach(function (_client) {
          hostNames = _slave.hosts.mapProperty('hostName');
          // The below logic to install clients to existing/New master hosts should not be applied to Add Host wizard.
          // This is with the presumption that Add Host controller does not add any new Master component to the cluster
          if (!this.get('isAddHost')) {
            if (clientsToMasterMap[_client.component_name]) {
              clientsToMasterMap[_client.component_name].forEach(function (componentName) {
                masterHosts.filterProperty('component', componentName).forEach(function (_masterHost) {
                  hostNames.pushObject(_masterHost.hostName);
                });
              });
            }
          }
          if (clientsToSlaveMap[_client.component_name]) {
            clientsToSlaveMap[_client.component_name].forEach(function (componentName) {
              slaveHosts.filterProperty('componentName', componentName).forEach(function (slaveHost) {
                hostNames = hostNames.concat(slaveHost.hosts.mapProperty('hostName')).uniq();
              });
            });
          }
          if (clientOnAllHosts.length > 0) {
            compOnAllHosts = false;
            for (var i = 0; i < clientOnAllHosts.length; i++) {
              if (_client.component_name === clientOnAllHosts[i]) {
                // component with ALL cardinality should not
                // registerHostsToComponent in createSlaveAndClientsHostComponents
                compOnAllHosts = true;
                break;
              }
            }
            if (!compOnAllHosts) {
              hostNames = hostNames.uniq();
              this.registerHostsToComponent(hostNames, _client.component_name);
            }
          } else {
            hostNames = hostNames.uniq();
            this.registerHostsToComponent(hostNames, _client.component_name);
          }
        }, this);
      }
    }, this);
  },

  /**
   * This function is specific to addServiceController
   * Newly introduced master components requires some existing client components to be hosted along with them
   */
  createAdditionalClientComponents: function createAdditionalClientComponents() {
    var masterHosts = this.get('content.masterComponentHosts');
    var clientHosts = [];
    if (this.get('content.slaveComponentHosts').someProperty('componentName', 'CLIENT')) {
      clientHosts = this.get('content.slaveComponentHosts').findProperty('componentName', 'CLIENT').hosts;
    }
    var clients = this.get('content.clients').filterProperty('isInstalled', false);
    var clientsToMasterMap = this.getClientsMap('isMaster');
    var clientsToClientMap = this.getClientsMap('isClient');
    var installedClients = [];

    // Get all the installed Client components
    this.get('content.services').filterProperty('isInstalled').forEach(function (_service) {
      var serviceClients = App.StackServiceComponent.find().filterProperty('serviceName', _service.get('serviceName')).filterProperty('isClient');
      serviceClients.forEach(function (client) {
        installedClients.push(client.get('componentName'));
      }, this);
    }, this);

    // Check if there is a dependency for being co-hosted between existing client and selected new master
    installedClients.forEach(function (_clientName) {
      if (clientsToMasterMap[_clientName] || clientsToClientMap[_clientName]) {
        var hostNames = [];
        if (clientsToMasterMap[_clientName]) {
          clientsToMasterMap[_clientName].forEach(function (componentName) {
            masterHosts.filterProperty('component', componentName).filterProperty('isInstalled', false).forEach(function (_masterHost) {
              hostNames.pushObject(_masterHost.hostName);
            }, this);
          }, this);
        }
        if (clientsToClientMap[_clientName]) {
          clientsToClientMap[_clientName].forEach(function (componentName) {
            clientHosts.forEach(function (_clientHost) {
              var host = this.get('content.hosts')[_clientHost.hostName];
              var isClientSelected = clients.someProperty('component_name', componentName);
              if (host.isInstalled && isClientSelected && !host.hostComponents.someProperty('HostRoles.component_name', componentName)) {
                hostNames.pushObject(_clientHost.hostName);
              }
            }, this);
          }, this);
        }
        hostNames = hostNames.uniq();
        if (hostNames.length > 0) {
          // If a dependency for being co-hosted is derived between existing client and selected new master but that
          // dependency is already satisfied in the cluster then disregard the derived dependency
          this.removeClientsFromList(_clientName, hostNames);
          this.registerHostsToComponent(hostNames, _clientName);
          if (hostNames.length > 0) {
            this.get('content.additionalClients').pushObject({ hostNames: hostNames, componentName: _clientName });
          }
        }
      }
    }, this);
  },

  /**
   *
   * @param clientName
   * @param hostList
   */
  removeClientsFromList: function removeClientsFromList(clientName, hostList) {
    var clientHosts = [];
    var installedHosts = this.get('content.hosts');
    for (var hostName in installedHosts) {
      if (installedHosts[hostName].isInstalled) {
        if (installedHosts[hostName].hostComponents.mapProperty('HostRoles.component_name').contains(clientName)) {
          clientHosts.push(hostName);
        }
      }
    }

    if (clientHosts.length > 0) {
      clientHosts.forEach(function (hostName) {
        if (hostList.contains(hostName)) {
          hostList.splice(hostList.indexOf(hostName), 1);
        }
      }, this);
    }
  },

  /**
   * Register additional components
   * Based on availability of some services
   * @uses registerHostsToComponent
   * @method createAdditionalHostComponents
   */
  createAdditionalHostComponents: function createAdditionalHostComponents() {
    var masterHosts = this.get('content.masterComponentHosts');

    // add all components with cardinality == ALL of selected services
    var registeredHosts = this.getRegisteredHosts();
    var notInstalledHosts = registeredHosts.filterProperty('isInstalled', false);
    this.get('content.services').filterProperty('isSelected').forEach(function (service) {
      service.get('serviceComponents').filterProperty('isRequiredOnAllHosts').forEach(function (component) {
        if (service.get('isInstalled') && notInstalledHosts.length) {
          this.registerHostsToComponent(notInstalledHosts.mapProperty('hostName'), component.get('componentName'));
        } else if (!service.get('isInstalled') && registeredHosts.length) {
          this.registerHostsToComponent(registeredHosts.mapProperty('hostName'), component.get('componentName'));
        }
      }, this);
    }, this);

    // add MySQL Server if Hive is selected
    var hiveService = this.get('content.services').filterProperty('isSelected', true).filterProperty('isInstalled', false).findProperty('serviceName', 'HIVE');
    if (hiveService) {
      var hiveDb = this.get('content.serviceConfigProperties').findProperty('name', 'hive_database');
      if (hiveDb.value === "New MySQL Database") {
        this.registerHostsToComponent(masterHosts.filterProperty('component', 'HIVE_SERVER').mapProperty('hostName'), 'MYSQL_SERVER');
      } else if (hiveDb.value === "New PostgreSQL Database") {
        this.registerHostsToComponent(masterHosts.filterProperty('component', 'HIVE_SERVER').mapProperty('hostName'), 'POSTGRESQL_SERVER');
      }
    }
  },

  /**
   * Register component to hosts
   * Queued request
   * @param {String[]} hostNames
   * @param {String} componentName
   * @method registerHostsToComponent
   */
  registerHostsToComponent: function registerHostsToComponent(hostNames, componentName) {
    if (!hostNames.length) return;

    var queryStr = '';
    hostNames.forEach(function (hostName) {
      queryStr += 'Hosts/host_name=' + hostName + '|';
    });
    //slice off last symbol '|'
    queryStr = queryStr.slice(0, -1);

    var data = {
      "RequestInfo": {
        "query": queryStr
      },
      "Body": {
        "host_components": [{
          "HostRoles": {
            "component_name": componentName
          }
        }]
      }
    };

    this.addRequestToAjaxQueue({
      name: 'wizard.step8.register_host_to_component',
      data: {
        data: JSON.stringify(data)
      }
    });
  },

  /**
   * Create config objects for cluster and services
   * @method createConfigurations
   */
  createConfigurations: function createConfigurations() {
    if (this.get('isInstaller')) {
      /** add cluster-env **/
      this.get('serviceConfigTags').pushObject(this.createDesiredConfig('cluster-env', this.get('configs').filterProperty('filename', 'cluster-env.xml')));
    }

    this.get('selectedServices').forEach(function (service) {
      Object.keys(service.get('configTypes')).forEach(function (type) {
        if (!this.get('serviceConfigTags').someProperty('type', type)) {
          var configs = this.get('configs').filterProperty('filename', App.config.getOriginalFileName(type));
          var serviceConfigNote = this.getServiceConfigNote(type, service.get('displayName'));
          this.get('serviceConfigTags').pushObject(this.createDesiredConfig(type, configs, serviceConfigNote));
        }
      }, this);
    }, this);
    this.createNotification();
  },

  /**
   * Get config version message
   *
   * @param type
   * @param serviceDisplayName
   * @returns {*}
   */
  getServiceConfigNote: function getServiceConfigNote(type, serviceDisplayName) {
    return this.get('isAddService') && type === 'core-site' ? Em.I18n.t('dashboard.configHistory.table.notes.addService') : Em.I18n.t('dashboard.configHistory.table.notes.default').format(serviceDisplayName);
  },

  /**
   * Send <code>serviceConfigTags</code> to server
   * Queued request
   * One request for each service config tag
   * @param serviceConfigTags
   * @method applyConfigurationsToCluster
   */
  applyConfigurationsToCluster: function applyConfigurationsToCluster(serviceConfigTags) {
    var allServices = this.get('installedServices').concat(this.get('selectedServices'));
    var allConfigData = [];
    allServices.forEach(function (service) {
      var serviceConfigData = [];
      Object.keys(service.get('configTypesRendered')).forEach(function (type) {
        var serviceConfigTag = serviceConfigTags.findProperty('type', type);
        if (serviceConfigTag) {
          serviceConfigData.pushObject(serviceConfigTag);
        }
      }, this);
      if (serviceConfigData.length) {
        allConfigData.pushObject(JSON.stringify({
          Clusters: {
            desired_config: serviceConfigData.map(function (item) {
              var props = {};
              Em.keys(item.properties).forEach(function (propName) {
                if (item.properties[propName] !== null) {
                  props[propName] = item.properties[propName];
                }
              });
              item.properties = props;
              return item;
            })
          }
        }));
      }
    }, this);
    var clusterConfig = serviceConfigTags.findProperty('type', 'cluster-env');
    if (clusterConfig) {
      allConfigData.pushObject(JSON.stringify({
        Clusters: {
          desired_config: [clusterConfig]
        }
      }));
    }

    this.addRequestToAjaxQueue({
      name: 'common.across.services.configurations',
      data: {
        data: '[' + allConfigData.toString() + ']'
      }
    });
  },

  /**
   * Create and update config groups
   * @method createConfigurationGroups
   */
  createConfigurationGroups: function createConfigurationGroups() {
    var configGroups = this.get('content.configGroups').filterProperty('is_default', false);
    var groupsToDelete = App.router.get(this.get('content.controllerName')).getDBProperty('groupsToDelete');
    if (groupsToDelete && groupsToDelete.length > 0) {
      this.removeInstalledServicesConfigurationGroups(groupsToDelete);
    }
    configGroups.forEach(function (configGroup) {
      if (configGroup.is_for_update || configGroup.is_temporary) {
        this.saveGroup(configGroup.properties, configGroup, this.getServiceConfigNote('', configGroup.service_id));
      }
    }, this);
    App.ServiceConfigGroup.deleteTemporaryRecords();
  },

  /**
   * add request to create config group to queue
   *
   * @param data
   * @method createConfigGroup
   */
  createConfigGroup: function createConfigGroup(data) {
    this.addRequestToAjaxQueue({
      name: 'wizard.step8.apply_configuration_groups',
      sender: this,
      data: {
        data: JSON.stringify(data)
      }
    });
  },

  /**
   * add request to update config group to queue
   *
   * @param data {Object}
   * @method updateConfigGroup
   */
  updateConfigGroup: function updateConfigGroup(data) {
    this.addRequestToAjaxQueue({
      name: 'config_groups.update_config_group',
      sender: this,
      data: {
        id: data.ConfigGroup.id,
        configGroup: data
      }
    });
  },

  /**
   * Delete selected config groups
   * @param {Object[]} groupsToDelete
   * @method removeInstalledServicesConfigurationGroups
   */
  removeInstalledServicesConfigurationGroups: function removeInstalledServicesConfigurationGroups(groupsToDelete) {
    var self = this;
    groupsToDelete.forEach(function (item) {
      self.deleteConfigurationGroup(Em.Object.create(item));
    });
  },

  /**
   * Selected and installed services
   * @override
   */
  currentServices: function () {
    return this.get('installedServices').concat(this.get('selectedServices'));
  }.property('installedServices.length', 'selectedServices.length'),

  /**
   * Add handling GLUSTREFS properties
   * @param property
   * @returns {*}
   * @override
   */
  formatValueBeforeSave: function formatValueBeforeSave(property) {
    if (this.formatGLUSTERFSProperties(Em.get(property, 'filename'))) {
      switch (property.name) {
        case "fs.default.name":
          return this.get('configs').someProperty('name', 'fs_glusterfs_default_name') ? this.get('configs').findProperty('name', 'fs_glusterfs_default_name').value : null;
        case "fs.defaultFS":
          return this.get('configs').someProperty('name', 'glusterfs_defaultFS_name') ? this.get('configs').findProperty('name', 'glusterfs_defaultFS_name').value : null;
      }
    }
    return this._super(property);
  },

  /**
   * Defines if some GLUSTERFS properties should be changed
   *
   * @param {String} type
   * @returns {boolean}
   */
  formatGLUSTERFSProperties: function formatGLUSTERFSProperties(type) {
    return App.config.getConfigTagFromFileName(type) === 'core-site' && this.get('installedServices').concat(this.get('selectedServices')).someProperty('serviceName', 'GLUSTERFS');
  },

  /**
   * Create one Alert Notification (if user select this on step7)
   * Only for Install Wizard and stack
   * @method createNotification
   */
  createNotification: function createNotification() {
    if (!this.get('isInstaller')) return;
    var miscConfigs = this.get('configs').filterProperty('serviceName', 'MISC'),
        createNotification = miscConfigs.findProperty('name', 'create_notification').value;
    if (createNotification !== 'yes') return;
    var predefinedNotificationConfigNames = require('data/configs/alert_notification').mapProperty('name'),
        configsForNotification = this.get('configs').filterProperty('filename', 'alert_notification');
    var properties = {},
        names = ['ambari.dispatch.recipients', 'mail.smtp.host', 'mail.smtp.port', 'mail.smtp.from', 'mail.smtp.starttls.enable', 'mail.smtp.startssl.enable'];
    if (miscConfigs.findProperty('name', 'smtp_use_auth').value == 'true') {
      // yes, it's not converted to boolean
      names.pushObjects(['ambari.dispatch.credential.username', 'ambari.dispatch.credential.password']);
    }

    names.forEach(function (name) {
      properties[name] = miscConfigs.findProperty('name', name).value;
    });

    properties['ambari.dispatch.recipients'] = properties['ambari.dispatch.recipients'].replace(/\s/g, '').split(',');

    configsForNotification.forEach(function (config) {
      if (predefinedNotificationConfigNames.contains(config.name)) return;
      properties[config.name] = config.value;
    });

    var apiObject = {
      AlertTarget: {
        name: 'Initial Notification',
        description: 'Notification created during cluster installing',
        global: true,
        notification_type: 'EMAIL',
        alert_states: ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN'],
        properties: properties
      }
    };
    this.addRequestToAjaxQueue({
      name: 'alerts.create_alert_notification',
      data: {
        urlParams: 'overwrite_existing=true',
        data: apiObject
      }
    });
  },

  /**
   * Should ajax-queue progress bar be displayed
   * @method showLoadingIndicator
   */
  showLoadingIndicator: function showLoadingIndicator() {
    return App.ModalPopup.show({

      header: Em.I18n.t('installer.step8.deployPopup.header'),

      showFooter: false,

      showCloseButton: false,

      bodyClass: Em.View.extend({

        templateName: require('templates/wizard/step8/step8_log_popup'),

        controllerBinding: 'App.router.wizardStep8Controller',

        /**
         * Css-property for progress-bar
         * @type {string}
         */
        barWidth: '',
        progressBarClass: 'progress log_popup',

        /**
         * Popup-message
         * @type {string}
         */
        message: '',

        /**
         * Set progress bar width and popup message when ajax-queue requests are processed
         * @method ajaxQueueChangeObs
         */
        ajaxQueueChangeObs: function () {
          var length = this.get('controller.ajaxQueueLength');
          var left = this.get('controller.ajaxRequestsQueue.queue.length');
          this.set('barWidth', 'width: ' + (length - left) / length * 100 + '%;');
          this.set('message', Em.I18n.t('installer.step8.deployPopup.message').format(length - left, length));
        }.observes('controller.ajaxQueueLength', 'controller.ajaxRequestsQueue.queue.length'),

        /**
         * Hide popup when ajax-queue is finished
         * @method autoHide
         */
        autoHide: function () {
          if (this.get('controller.servicesInstalled')) {
            this.get('parentView').hide();
          }
        }.observes('controller.servicesInstalled'),

        ajaxQueueErrorAppears: function () {
          if (this.get('controller.hasErrorOccurred')) {
            this.get('parentView').onClose();
          }
        }.observes('controller.hasErrorOccurred')

      })

    });
  },

  getComponentsForHost: function getComponentsForHost(host) {
    if (!host.hostComponents) {
      App.router.get('installerController').get('allHosts');
    }
    var componentNameDetail = [];
    host.hostComponents.mapProperty('componentName').forEach(function (componentName) {
      if (componentName === 'CLIENT') {
        this.get('content.clients').mapProperty('component_name').forEach(function (clientComponent) {
          componentNameDetail.push({ name: clientComponent });
        }, this);
        return;
      }
      componentNameDetail.push({ name: componentName });
    }, this);
    return componentNameDetail;
  },

  getPropertyAttributesForConfigType: function getPropertyAttributesForConfigType(configs) {
    //Currently only looks for final properties, if any
    var finalProperties = configs.filterProperty('isFinal', 'true');
    var propertyAttributes = {};
    finalProperties.forEach(function (finalProperty) {
      propertyAttributes[finalProperty['name']] = "true";
    });
    var finalPropertyMap = {};
    if (!App.isEmptyObject(finalProperties)) {
      finalPropertyMap = {
        "isFinal": propertyAttributes
      };
    }
    return finalPropertyMap;
  },

  getConfigurationDetailsForConfigType: function getConfigurationDetailsForConfigType(configs) {
    var configDetails = {};
    var self = this;
    configs.forEach(function (propertyObj) {
      configDetails[propertyObj['name']] = propertyObj['value'];
    }, this);
    var configurationsDetails = {
      "properties_attributes": self.getPropertyAttributesForConfigType(configs),
      "properties": configDetails
    };
    return configurationsDetails;
  },

  hostInExistingHostGroup: function hostInExistingHostGroup(newHost, host_groups) {
    var hostGroupMatched = false;
    host_groups.some(function (existingHostGroup) {
      if (!hostGroupMatched) {
        var fqdnInHostGroup = existingHostGroup.hosts[0].fqdn;
        var componentsInExistingHostGroup = this.getRegisteredHosts().filterProperty('hostName', fqdnInHostGroup)[0].hostComponents;
        if (componentsInExistingHostGroup.length !== newHost.hostComponents.length) {
          return;
        } else {
          var componentMismatch = false;
          componentsInExistingHostGroup.forEach(function (componentInExistingHostGroup, index) {
            if (!componentMismatch) {
              if (!newHost.hostComponents.mapProperty('componentName').includes(componentInExistingHostGroup.componentName)) {
                componentMismatch = true;
              }
            }
          });
          if (!componentMismatch) {
            hostGroupMatched = true;
            existingHostGroup["cardinality"] = parseInt(existingHostGroup["cardinality"]) + 1;
            existingHostGroup.hosts.push({ "fqdn": newHost.hostName });
            return true;
          }
        }
      }
    }, this);
    return hostGroupMatched;
  },

  hostInChildHostGroup: function hostInChildHostGroup(presentHostGroup, configGroupName, hostInConfigGroup) {
    return presentHostGroup['childHostGroups'].some(function (childHostGroup) {
      //Check if childHostGroup name is same as this configgroupname, if yes, update childHostGroup else, compare with other childhostgroups
      if (childHostGroup.configGroupName === configGroupName) {
        childHostGroup.hosts.push({ "fqdn": hostInConfigGroup });
        childHostGroup['cardinality'] = childHostGroup['cardinality'] + 1;
        return true;
      }
    });
  },

  /**
   * Confirmation popup before generate blueprint
   */
  generateBlueprintConfirmation: function generateBlueprintConfirmation() {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.generateBlueprint();
    }, Em.I18n.t('installer.step8.generateBlueprint.popup.msg').format(App.clusterStatus.clusterName));
  },

  generateBlueprint: function generateBlueprint() {
    console.log("Prepare blueprint for download...");
    var self = this;
    //service configurations
    var totalConf = [];
    //Add cluster-env
    var clusterEnv = this.get('configs').filterProperty('filename', 'cluster-env.xml');
    var configurations = {};
    configurations["cluster-env"] = self.getConfigurationDetailsForConfigType(clusterEnv);
    totalConf.push(configurations);
    //Add configurations for selected services
    this.get('selectedServices').forEach(function (service) {
      Object.keys(service.get('configTypes')).forEach(function (type) {
        if (!this.get('serviceConfigTags').someProperty('type', type)) {
          var configs = this.get('configs').filterProperty('filename', App.config.getOriginalFileName(type));
          var configurations = {};
          configurations[type] = self.getConfigurationDetailsForConfigType(configs);
          totalConf.push(configurations);
        }
      }, this);
    }, this);

    var host_groups = [];
    var cluster_template_host_groups = [];
    var counter = 0;

    this.getRegisteredHosts().filterProperty('isInstalled', false).map(function (host) {
      if (self.hostInExistingHostGroup(host, host_groups)) {
        return;
      }
      //Create new host_group if host is not mapped to existing host_groups
      var hostGroupId = "host_group_" + counter;
      var hostListForGroup = [];
      hostListForGroup.push({ "fqdn": host.hostName });
      var hostGroupDetail = {
        "name": hostGroupId,
        "components": self.getComponentsForHost(host),
        "hosts": hostListForGroup,
        "cardinality": 1
      };
      hostGroupDetail.toJSON = function () {
        var hostGroupDetailResult = _.omit(this, ["hosts", "childHostGroups"]);
        hostGroupDetailResult["cardinality"] = this["cardinality"].toString();
        return hostGroupDetailResult;
      };
      host_groups.push(hostGroupDetail);

      var clusterTemplateHostGroupDetail = {
        "name": hostGroupId,
        "hosts": hostListForGroup
      };

      cluster_template_host_groups.push(clusterTemplateHostGroupDetail);
      counter++;
    }, this);

    this.get('content.configGroups').filterProperty("is_default", false).forEach(function (configGroup) {
      if (configGroup.properties.length == 0) {
        return;
      }
      configGroup.hosts.forEach(function (hostInConfigGroup) {
        return host_groups.some(function (presentHostGroup) {
          return presentHostGroup.hosts.some(function (hostInPresentHostGroup, index) {
            if (hostInConfigGroup !== hostInPresentHostGroup.fqdn) {
              return;
            }
            //Check if childHostGroup already created
            if (presentHostGroup['childHostGroups']) {
              if (self.hostInChildHostGroup(presentHostGroup, configGroup.name, hostInConfigGroup)) {
                // Update to remove parentHostGroup as all the hosts are added to childHostGroup/s
                presentHostGroup.hosts.splice(index, 1);
                presentHostGroup["cardinality"] = presentHostGroup['cardinality'] - 1;
                return true;
              }
            }
            //create configuration object
            var hgConfigurations;
            if (presentHostGroup.hosts.length === 1 && presentHostGroup["configurations"]) {
              hgConfigurations = presentHostGroup["configurations"][0];
            } else if (presentHostGroup["configurations"]) {
              //Deep copy
              hgConfigurations = jQuery.extend(true, {}, presentHostGroup["configurations"][0]);
            } else {
              hgConfigurations = {};
            }
            configGroup.properties.forEach(function (hgProperties) {
              var type = App.config.getConfigTagFromFileName(hgProperties.filename);
              if (!hgConfigurations[type]) {
                hgConfigurations[type] = { properties: {} };
              }
              hgConfigurations[type]['properties'][hgProperties.name] = hgProperties.value;
            });
            var totalHgConf = [];
            totalHgConf.push(hgConfigurations);
            //If only host in presentHostGroup then merge configuration and return
            if (presentHostGroup.hosts.length === 1) {
              //If host_group already has configuration, assigned from previously processed config_group
              if (!presentHostGroup["configurations"]) {
                presentHostGroup["configurations"] = totalHgConf;
              }
              return true;
            }

            //Create new host_group from this host
            var hostGroupId = "host_group_" + counter;
            counter++;
            var hostListForGroup = [];
            hostListForGroup.push({ "fqdn": hostInPresentHostGroup.fqdn });
            var hostGroupDetail = {
              "name": hostGroupId,
              "components": presentHostGroup.components,
              "cardinality": 1,
              "hosts": hostListForGroup,
              "configurations": totalHgConf,
              "configGroupName": configGroup.name
            };
            hostGroupDetail.toJSON = function () {
              var hostGroupDetailResult = _.omit(this, ["hosts", "configGroupName", "childHostGroups"]);
              hostGroupDetailResult["cardinality"] = this["cardinality"].toString();
              return hostGroupDetailResult;
            };
            host_groups.push(hostGroupDetail);
            //Update for clustertemplate file
            var clusterTemplateHostGroupDetail = {
              "name": hostGroupId,
              "hosts": hostListForGroup
            };
            cluster_template_host_groups.push(clusterTemplateHostGroupDetail);
            //Add newly created host_group as child to existing host_group
            if (!presentHostGroup['childHostGroups']) {
              presentHostGroup['childHostGroups'] = [];
            }
            presentHostGroup['childHostGroups'].push(hostGroupDetail);
            presentHostGroup.hosts.splice(index, 1);
            presentHostGroup["cardinality"] = presentHostGroup['cardinality'] - 1;
            //return true to indicate that host has been matched
            return true;
          }, this);
        }, this);
      }, this);
    }, this);

    var selectedStack = App.Stack.find().findProperty('isSelected', true);
    var blueprint = {
      'configurations': totalConf,
      'host_groups': host_groups.filter(function (item) {
        return item.cardinality > 0;
      }),
      'Blueprints': { 'blueprint_name': App.clusterStatus.clusterName, 'stack_name': selectedStack.get('stackName'), 'stack_version': selectedStack.get('stackVersion') }
    };
    var cluster_template = {
      "blueprint": App.clusterStatus.clusterName,
      "config_recommendation_strategy": "NEVER_APPLY",
      "provision_action": "INSTALL_AND_START",
      "configurations": [],
      "host_groups": cluster_template_host_groups.filter(function (item) {
        return item.hosts.length > 0;
      }),
      "Clusters": { 'cluster_name': App.clusterStatus.clusterName }
    };
    fileUtils.downloadFilesInZip([{
      data: JSON.stringify(blueprint),
      type: 'json',
      name: 'blueprint.json'
    }, {
      data: JSON.stringify(cluster_template),
      type: 'json',
      name: 'clustertemplate.json'
    }]);
  },

  downloadCSV: function downloadCSV() {
    App.router.get('kerberosWizardStep5Controller').getCSVData(false);
  }
});

});

require.register("controllers/wizard/step9_controller", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
var stringUtils = require('utils/string_utils');

App.WizardStep9Controller = Em.Controller.extend(App.ReloadPopupMixin, {

  name: 'wizardStep9Controller',

  /**
   *  Array of host Objects that are successfully registered on "Confirm Host Options" page
   *  <code>
   *    {
   *      name: {String} host name.
   *      status: {String} Current status of the host. This field is used in the step 9 view to set the css class of the
   *             host progress bar and set the appropriate message. Possible values: info, warning, failed, heartbeat_lost and success
   *      logTasks: {Array} Tasks that are scheduled on the host for the current request.
   *      message: {String} Message to be shown in front of the host name.
   *      progress: {Int} Progress of the tasks on the host. Amount of tasks completed for all the tasks scheduled on the host.
   *      isNoTasksForInstall: {Boolean} Gets set when no task is scheduled for the host on install phase.
   *    }
   *  </code>
   *  @type {Array.<{name: string, status: string, logTasks: object[], message: string, progress: number, isNoTasksForInstall: bool}>}
   */
  hosts: [],

  /**
   * Overall progress of <Install,Start and Test> page shown as progress bar on the top of the page
   * @type {string}
   */
  progress: '0',

  /**
   * Json file for the mock data to be used in mock mode
   * @type {string}
   */
  mockDataPrefix: '/data/wizard/deploy/5_hosts',

  /**
   * Current Request data polled from the API: api/v1/clusters/{clusterName}/requests/{RequestId}?fields=tasks/Tasks/command,
   * tasks/Tasks/exit_code,tasks/Tasks/start_time,tasks/Tasks/end_time,tasks/Tasks/host_name,tasks/Tasks/id,tasks/Tasks/role,
   * tasks/Tasks/status&minimal_response=true
   * @type {Object[]}
   */
  polledData: [],

  /**
   * This flag is only used in UI mock mode as a counter for number of polls.
   * @type {number}
   */
  numPolls: 1,

  /**
   * Interval in milliseconds between API calls While polling the request status for Install and, Start and Test tasks
   * @type {number}
   */
  POLL_INTERVAL: 4000,

  /**
   * Array of objects
   * <code>
   *   {
   *     hostName: {String} Name of host that has stopped heartbeating to ambari-server
   *     componentNames: {Sting} Name of all components that are on the host
   *   }
   * </code>
   * @type {Object[]}
   */
  hostsWithHeartbeatLost: [],

  /**
   * Flag is set in the start all services error callback function
   * @type {bool}
   */
  startCallFailed: false,

  /**
   * Status of the page. Possible values: <info, warning, failed and success>.
   * This property is used in the step-9 view for displaying the appropriate color of the overall progress bar and
   * the appropriate result message at the bottom of the page
   * @type {string}
   */
  status: 'info',

  skipServiceChecks: false,

  /**
   * This computed property is used to determine if the Next button and back button on the page should be disabled. The property is
   * used in the template to grey out Next and Back buttons. Although clicking on the greyed out button do trigger the event and
   * calls submit and back function of the controller.
   * @type {bool}
   */
  isSubmitDisabled: function () {
    var validStates = ['STARTED', 'START FAILED', 'START_SKIPPED'];
    var controllerName = this.get('content.controllerName');
    if (controllerName == 'addHostController' || controllerName == 'addServiceController') {
      validStates.push('INSTALL FAILED');
    }
    return !validStates.contains(this.get('content.cluster.status')) || App.get('router.btnClickInProgress');
  }.property('content.cluster.status'),

  isNextButtonDisabled: Em.computed.or('App.router.nextBtnClickInProgress', 'isSubmitDisabled'),

  /**
   * Observer function: Enables previous steps link if install task failed in installer wizard.
   * @method togglePreviousSteps
   */
  togglePreviousSteps: function () {
    if (App.get('testMode')) {
      return;
    }
    var installerController = App.router.get('installerController');
    if ('INSTALL FAILED' === this.get('content.cluster.status') && this.get('content.controllerName') == 'installerController') {
      installerController.setStepsEnable();
    } else {
      installerController.setLowerStepsDisable(9);
    }
  }.observes('content.cluster.status', 'content.controllerName'),

  /**
   * Computed property to determine if the Retry button should be made visible on the page.
   * @type {bool}
   */
  showRetry: Em.computed.equal('content.cluster.status', 'INSTALL FAILED'),

  /**
   * Observer function: Calls {hostStatusUpdates} function once with change in a host status from any registered hosts.
   * @method hostStatusObserver
   */
  hostStatusObserver: function () {
    Em.run.once(this, 'updateStatus');
  }.observes('hosts.@each.status'),

  /**
   * A flag that gets set with installation failure.
   * @type {bool}
   */
  installFailed: false,

  /**
   * Incremental flag that triggers an event in step 9 view to change the tasks related data and icons of hosts.
   * @type {number}
   */
  logTasksChangesCounter: 0,

  /**
   * <code>taskId</code> of current open task
   * @type {number}
   */
  currentOpenTaskId: 0,

  /**
   * <code>requestId</code> of current open task
   * @type {number}
   */
  currentOpenTaskRequestId: 0,

  /**
   * True if stage transition is completed.
   * On true, polling will be stopped.
   * @type {bool}s
   */
  parseHostInfo: false,

  isPolling: true,

  changeParseHostInfo: function changeParseHostInfo(value) {
    this.set('parseHostInfo', value);
    this.parseHostInfoPolling();
  },

  parseHostInfoPolling: function parseHostInfoPolling() {
    var self = this;
    var result = this.get('parseHostInfo');
    if (!this.get('isPolling')) {
      if (this.get('content.cluster.status') === 'INSTALL FAILED') {
        this.isAllComponentsInstalled();
      }
      return;
    }
    if (result !== true) {
      window.setTimeout(function () {
        if (self.get('currentOpenTaskId')) {
          self.loadCurrentTaskLog();
        }
        if (App.router.loggedIn) {
          self.doPolling();
        }
      }, this.get('POLL_INTERVAL'));
    }
  },

  /**
   * Observer function: Updates {status} field of the controller.
   * @method updateStatus
   */
  updateStatus: function () {
    var status = 'info';
    if (this.get('hosts').someProperty('status', 'failed') || this.get('hosts').someProperty('status', 'heartbeat_lost') || this.get('startCallFailed')) {
      status = 'failed';
    } else if (this.get('hosts').someProperty('status', 'warning')) {
      if (this.isStepFailed()) {
        status = 'failed';
      } else {
        status = 'warning';
      }
    } else if (this.get('progress') == '100' && this.get('content.cluster.status') !== 'INSTALL FAILED') {
      status = 'success';
    }
    this.set('status', status);
  }.observes('progress'),

  /**
   * This function is called when a click event happens on Next button of "Install, Start and Test" page
   * @method submit
   */
  submit: function submit() {
    App.router.send('next');
  },

  /**
   * This function is called when a click event happens on back button of "Install, Start and Test" page
   * @method back
   */
  back: function back() {
    if (!this.get('isSubmitDisabled')) {
      App.router.send('back');
    }
  },

  /**
   * navigateStep is called by App.WizardStep9View's didInsertElement and "retry" from router.
   * content.cluster.status can be:
   * PENDING: set upon successful transition from step 1 to step 2
   * INSTALLED: set upon successful completion of install phase as well as successful invocation of start services API
   * STARTED: set up on successful completion of start phase
   * INSTALL FAILED: set up upon encountering a failure in install phase
   * START FAILED: set upon unsuccessful invocation of start services API and also upon encountering a failure
   * during start phase
    * content.cluster.isCompleted
   * set to false upon successful transition from step 1 to step 2
   * set to true upon successful start of services in this step
   * note: looks like this is the same thing as checking content.cluster.status == 'STARTED'
   * @method navigateStep
   */
  navigateStep: function navigateStep() {
    if (App.get('testMode')) {
      // this is for repeatedly testing out installs in test mode
      this.set('content.cluster.status', 'PENDING');
      this.set('content.cluster.isCompleted', false);
      this.set('content.cluster.requestId', 1);
    }
    var needPolling = false;
    var clusterStatus = this.get('content.cluster.status');
    if (this.get('content.cluster.isCompleted') === false) {
      if (clusterStatus !== 'INSTALL FAILED' && clusterStatus !== 'START FAILED') {
        needPolling = true;
      }
    } else {
      // handle STARTED
      // the cluster has successfully installed and started
      this.set('progress', '100');
    }
    this.loadStep();
    this.loadLogData(needPolling);
  },

  /**
   * This is called on initial page load, refreshes and retry event.
   * clears all in memory stale data for retry event.
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.get('hosts').clear();
    this.set('hostsWithHeartbeatLost', []);
    this.set('startCallFailed', false);
    this.set('status', 'info');
    this.set('progress', '0');
    this.set('numPolls', 1);
  },

  /**
   * This is called on initial page load, refreshes and retry event.
   * @method loadStep
   */
  loadStep: function loadStep() {
    this.clearStep();
    this.loadHosts();
  },

  /**
   * Reset status and message of all hosts when retry install
   * @method resetHostsForRetry
   */
  resetHostsForRetry: function resetHostsForRetry() {
    var hosts = this.get('content.hosts');
    for (var name in hosts) {
      if (hosts.hasOwnProperty(name)) {
        hosts[name].status = "pending";
        hosts[name].message = 'Waiting';
        hosts[name].isNoTasksForInstall = false;
      }
    }
    this.set('content.hosts', hosts);
  },

  /**
   * Sets the <code>hosts</code> array for the controller
   * @method loadHosts
   */
  loadHosts: function loadHosts() {
    var hosts = this.get('content.hosts');

    //flag identify whether get all hosts or only NewHosts(newly added) hosts
    var getOnlyNewHosts = this.get('content.controllerName') !== 'addServiceController';

    for (var index in hosts) {
      if (hosts[index].bootStatus === 'REGISTERED' && (!getOnlyNewHosts || !hosts[index].isInstalled)) {
        var hostInfo = App.HostInfo.create({
          name: hosts[index].name,
          status: hosts[index].status ? hosts[index].status : 'info',
          logTasks: [],
          message: hosts[index].message ? hosts[index].message : 'Waiting',
          progress: 0,
          isNoTasksForInstall: false
        });
        this.get('hosts').pushObject(hostInfo);
      }
    }
  },

  /**
   * Set new polled data
   * @param polledData sets the <code>polledData</code> object of the controller
   * @method replacePolledData
   */
  replacePolledData: function replacePolledData(polledData) {
    this.get('polledData').clear();
    this.set('polledData', polledData);
  },

  /**
   * Get the appropriate message for the host as per the running task
   * @param {object} task
   * @returns {String}
   * @method displayMessage
   */
  displayMessage: function displayMessage(task) {
    var role = App.format.role(task.role, false);
    /* istanbul ignore next */
    switch (task.command) {
      case 'INSTALL':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.install.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.install.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.install.inProgress') + role;
          case 'COMPLETED':
            return Em.I18n.t('installer.step9.serviceStatus.install.completed') + role;
          case 'FAILED':
            return Em.I18n.t('installer.step9.serviceStatus.install.failed') + role;
        }
        break;
      case 'UNINSTALL':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.uninstall.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.uninstall.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.uninstall.inProgress') + role;
          case 'COMPLETED':
            return Em.I18n.t('installer.step9.serviceStatus.uninstall.completed') + role;
          case 'FAILED':
            return Em.I18n.t('installer.step9.serviceStatus.uninstall.failed') + role;
        }
        break;
      case 'START':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.start.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.start.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.start.inProgress') + role;
          case 'COMPLETED':
            return role + Em.I18n.t('installer.step9.serviceStatus.start.completed');
          case 'FAILED':
            return role + Em.I18n.t('installer.step9.serviceStatus.start.failed');
        }
        break;
      case 'STOP':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.stop.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.stop.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.stop.inProgress') + role;
          case 'COMPLETED':
            return role + Em.I18n.t('installer.step9.serviceStatus.stop.completed');
          case 'FAILED':
            return role + Em.I18n.t('installer.step9.serviceStatus.stop.failed');
        }
        break;
      case 'CUSTOM_COMMAND':
        role = App.format.commandDetail(task.command_detail, task.request_input, task.ops_display_name);
      case 'EXECUTE':
      case 'SERVICE_CHECK':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.execute.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.execute.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.execute.inProgress') + role;
          case 'COMPLETED':
            return role + Em.I18n.t('installer.step9.serviceStatus.execute.completed');
          case 'FAILED':
            return role + Em.I18n.t('installer.step9.serviceStatus.execute.failed');
        }
        break;
      case 'ABORT':
        switch (task.status) {
          case 'PENDING':
            return Em.I18n.t('installer.step9.serviceStatus.abort.pending') + role;
          case 'QUEUED':
            return Em.I18n.t('installer.step9.serviceStatus.abort.queued') + role;
          case 'IN_PROGRESS':
            return Em.I18n.t('installer.step9.serviceStatus.abort.inProgress') + role;
          case 'COMPLETED':
            return role + Em.I18n.t('installer.step9.serviceStatus.abort.completed');
          case 'FAILED':
            return role + Em.I18n.t('installer.step9.serviceStatus.abort.failed');
        }
    }
    return '';
  },

  /**
   * Run start/check services after installation phase.
   * Does Ajax call to start all services
   * @return {$.ajax|null}
   * @method launchStartServices
   */
  launchStartServices: function launchStartServices(callback) {
    var data = {};
    var name = '';
    callback = callback || Em.K;
    switch (this.get('content.controllerName')) {
      case 'addHostController':
        name = 'common.host_components.update';
        var hostnames = [];
        var hosts = this.get('wizardController').getDBProperty('hosts');
        for (var hostname in hosts) {
          if (this.get('hosts').findProperty('name', hostname)) {
            hostnames.push(hostname);
          }
        }
        data = {
          "context": Em.I18n.t("requestInfo.startHostComponents"),
          "query": "HostRoles/component_name.in(" + App.get('components.slaves').join(',') + ")&HostRoles/state=INSTALLED&HostRoles/host_name.in(" + hostnames.join(',') + ")",
          "HostRoles": { "state": "STARTED" }
        };
        break;
      case 'addServiceController':
        var servicesList = this.get('content.services').filterProperty('isSelected').filterProperty('isInstalled', false).mapProperty('serviceName');
        if (servicesList.contains('OOZIE')) {
          servicesList = servicesList.concat(['HDFS', 'YARN', 'MAPREDUCE2']);
        }
        name = 'common.services.update';
        data = {
          "context": Em.I18n.t("requestInfo.startAddedServices"),
          "ServiceInfo": { "state": "STARTED" },
          "urlParams": "ServiceInfo/state=INSTALLED&ServiceInfo/service_name.in(" + servicesList.join(",") + ")&params/run_smoke_test=true&params/reconfigure_client=false"
        };
        break;
      default:
        name = 'common.services.update';
        data = {
          "context": Em.I18n.t("requestInfo.startServices"),
          "ServiceInfo": { "state": "STARTED" },
          "urlParams": "ServiceInfo/state=INSTALLED&params/run_smoke_test=" + !this.get('skipServiceChecks') + "&params/reconfigure_client=false"
        };
    }

    if (App.get('testMode')) {
      this.set('numPolls', 6);
    }

    return App.ajax.send({
      name: name,
      sender: this,
      data: data,
      success: 'launchStartServicesSuccessCallback',
      error: 'launchStartServicesErrorCallback'
    }).then(callback, callback);
  },

  /**
   * Success callback function for start services task.
   * @param {object} jsonData Contains Request id to poll.
   * @method launchStartServicesSuccessCallback
   */
  launchStartServicesSuccessCallback: function launchStartServicesSuccessCallback(jsonData) {
    var clusterStatus = {};
    if (jsonData) {
      var requestId = jsonData.Requests.id;
      clusterStatus = {
        status: 'INSTALLED',
        requestId: requestId,
        isStartError: false,
        isCompleted: false
      };
      this.hostHasClientsOnly(false);
      this.saveClusterStatus(clusterStatus);
    } else {
      this.hostHasClientsOnly(true);
      clusterStatus = {
        status: 'STARTED',
        isStartError: false,
        isCompleted: true
      };
      this.saveClusterStatus(clusterStatus);
      this.set('status', 'success');
      this.set('progress', '100');
    }
    // We need to do recovery if there is a browser crash
    App.clusterStatus.setClusterStatus({
      clusterState: 'SERVICE_STARTING_3',
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    });

    if (jsonData) {
      this.startPolling();
    }
  },

  /**
   * This function will be called for Add host wizard only.
   * @param {bool} jsonError Boolean is true when API to start services returns 200 ok and no json data
   * @method hostHasClientsOnly
   */
  hostHasClientsOnly: function hostHasClientsOnly(jsonError) {
    this.get('hosts').forEach(function (host) {
      var OnlyClients = true;
      var tasks = host.get('logTasks');
      tasks.forEach(function (task) {
        var component = App.StackServiceComponent.find().findProperty('componentName', task.Tasks.role);
        if (!(component && component.get('isClient'))) {
          OnlyClients = false;
        }
      });
      if (OnlyClients || jsonError) {
        host.set('status', 'success');
        host.set('progress', '100');
      }
    });
  },

  /**
   * Error callback function for start services task.
   * @param {object} jqXHR
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @method launchStartServicesErrorCallback
   */
  launchStartServicesErrorCallback: function launchStartServicesErrorCallback(jqXHR, ajaxOptions, error, opt) {
    this.set('startCallFailed', true);
    var clusterStatus = {
      status: 'INSTALL FAILED',
      isStartError: false,
      isCompleted: false
    };
    this.saveClusterStatus(clusterStatus);
    this.get('hosts').forEach(function (host) {
      host.set('progress', '100');
    });
    this.set('progress', '100');

    console.log("Error starting installed services: ", jqXHR, ajaxOptions, error, opt);
    App.ModalPopup.show({
      encodeBody: false,
      primary: Em.I18n.t('ok'),
      header: Em.I18n.t('installer.step9.service.start.header'),
      secondary: false,
      body: Em.I18n.t('installer.step9.service.start.failed'),
      'data-qa': 'start-failed-modal'
    });
  },

  /**
   * Marks a host's status as "success" if all tasks are in COMPLETED state
   * @method onSuccessPerHost
   */
  onSuccessPerHost: function onSuccessPerHost(actions, contentHost) {
    var status = this.get('content.cluster.status');
    if (actions.everyProperty('Tasks.status', 'COMPLETED') && (status === 'INSTALLED' || status === 'STARTED') || App.get('supports.skipComponentStartAfterInstall')) {
      contentHost.set('status', 'success');
    }
  },

  /**
   * marks a host's status as "warning" if at least one of the tasks is FAILED, ABORTED, or TIMEDOUT and marks host's status as "failed" if at least one master component install task is FAILED.
   * note that if the master failed to install because of ABORTED or TIMEDOUT, we don't mark it as failed, because this would mark all hosts as "failed" and makes it difficult for the user
   * to find which host FAILED occurred on, if any
   * @param actions {Array} of tasks retrieved from polled data
   * @param contentHost {Object} A host object
   * @method onErrorPerHost
   */
  onErrorPerHost: function onErrorPerHost(actions, contentHost) {
    if (!actions) return;
    if (actions.someProperty('Tasks.status', 'FAILED') || actions.someProperty('Tasks.status', 'ABORTED') || actions.someProperty('Tasks.status', 'TIMEDOUT')) {
      contentHost.set('status', 'warning');
    }
    if (this.get('content.cluster.status') === 'PENDING' && actions.someProperty('Tasks.status', 'FAILED') || this.isMasterFailed(actions)) {
      contentHost.get('status') !== 'heartbeat_lost' ? contentHost.set('status', 'failed') : '';
    }
  },

  /**
   * Check if some master component is failed
   * @param {object} polledData data polled from API.
   * @returns {bool}  true if there is at least one FAILED task of master component install
   * @method isMasterFailed
   */
  isMasterFailed: function isMasterFailed(polledData) {
    var result = false;
    polledData.filterProperty('Tasks.command', 'INSTALL').filterProperty('Tasks.status', 'FAILED').mapProperty('Tasks.role').forEach(function (role) {
      if (!App.get('components.slaves').contains(role)) {
        result = true;
      }
    });
    return result;
  },

  /**
   * Mark a host status as in_progress if the any task on the host if either in IN_PROGRESS, QUEUED or PENDONG state.
   * @param actions {Array} of tasks retrieved from polled data
   * @param contentHost {Object} A host object
   * @method onInProgressPerHost
   */
  onInProgressPerHost: function onInProgressPerHost(actions, contentHost) {
    var runningAction = actions.findProperty('Tasks.status', 'IN_PROGRESS');
    if (runningAction === undefined || runningAction === null) {
      runningAction = actions.findProperty('Tasks.status', 'QUEUED');
    }
    if (runningAction === undefined || runningAction === null) {
      runningAction = actions.findProperty('Tasks.status', 'PENDING');
    }
    if (runningAction !== null && runningAction !== undefined) {
      contentHost.set('status', 'in_progress');
      contentHost.set('message', this.displayMessage(runningAction.Tasks));
    }
  },

  /**
   * Calculate progress of tasks per host
   * @param {object} actions
   * @param {object} contentHost
   * @return {Number}
   * @method progressPerHost
   */
  progressPerHost: function progressPerHost(actions, contentHost) {
    var progress = 0;
    var actionsPerHost = actions.length;
    var completedActions = 0;
    var queuedActions = 0;
    var inProgressActions = 0;
    var installProgressFactor = App.get('supports.skipComponentStartAfterInstall') ? 100 : 33;
    actions.forEach(function (action) {
      completedActions += +['COMPLETED', 'FAILED', 'ABORTED', 'TIMEDOUT'].contains(action.Tasks.status);
      queuedActions += +(action.Tasks.status === 'QUEUED');
      inProgressActions += +(action.Tasks.status === 'IN_PROGRESS');
    }, this);
    /** for the install phase (PENDING), % completed per host goes up to 33%; floor(100 / 3)
     * for the start phase (INSTALLED), % completed starts from 34%
     * when task in queued state means it's completed on 9%
     * in progress - 35%
     * completed - 100%
     */
    switch (this.get('content.cluster.status')) {
      case 'PENDING':
        progress = actionsPerHost ? Math.ceil((queuedActions * 0.09 + inProgressActions * 0.35 + completedActions) / actionsPerHost * installProgressFactor) : installProgressFactor;
        break;
      case 'INSTALLED':
        progress = actionsPerHost ? 33 + Math.floor((queuedActions * 0.09 + inProgressActions * 0.35 + completedActions) / actionsPerHost * 67) : 100;
        break;
      default:
        progress = 100;
        break;
    }
    contentHost.set('progress', progress.toString());
    return progress;
  },

  /**
   * Check if step completed successfully
   * @param {object} polledData data retrieved from API
   * @returns {bool}
   * @method isSuccess
   */
  isSuccess: function isSuccess(polledData) {
    return polledData.everyProperty('Tasks.status', 'COMPLETED');
  },

  /**
   * Check if step is failed
   * Return true if:
   *  1. any of the master/client components failed to install
   *  OR
   *  2. at least 50% of the slave host components for the particular service component fails to install
   *  @method isStepFailed
   */
  isStepFailed: function isStepFailed() {
    var failed = false;
    var polledData = this.get('polledData');
    polledData.filterProperty('Tasks.command', 'INSTALL').mapProperty('Tasks.role').uniq().forEach(function (role) {
      if (failed) {
        return;
      }
      var actionsPerRole = polledData.filterProperty('Tasks.role', role);
      if (App.get('components.slaves').contains(role)) {
        // check slave components for success factor.
        // partial failure for slave components are allowed.
        var actionsFailed = actionsPerRole.filterProperty('Tasks.status', 'FAILED');
        var actionsAborted = actionsPerRole.filterProperty('Tasks.status', 'ABORTED');
        var actionsTimedOut = actionsPerRole.filterProperty('Tasks.status', 'TIMEDOUT');
        if ((actionsFailed.length + actionsAborted.length + actionsTimedOut.length) / actionsPerRole.length * 100 > 50) {
          failed = true;
        }
      } else if (actionsPerRole.someProperty('Tasks.status', 'FAILED') || actionsPerRole.someProperty('Tasks.status', 'ABORTED') || actionsPerRole.someProperty('Tasks.status', 'TIMEDOUT')) {
        // check non-salve components (i.e., masters and clients).  all of these must be successfully installed.
        failed = true;
      }
    }, this);
    return failed;
  },

  /**
   * polling from ui stops only when no action has 'PENDING', 'QUEUED' or 'IN_PROGRESS' status
   * Makes a state transition
   * PENDING -> INSTALLED
   * PENDING -> INSTALL FAILED
   * INSTALLED -> STARTED
   * INSTALLED -> START_FAILED
   * @param polledData json data retrieved from API
   * @method setFinishState
   */
  setFinishState: function setFinishState(polledData) {
    if (this.get('content.cluster.status') === 'INSTALLED') {
      Em.run.next(this, function () {
        this.changeParseHostInfo(this.isServicesStarted(polledData));
      });
      return;
    } else if (this.get('content.cluster.status') === 'PENDING') {
      this.setIsServicesInstalled(polledData);
      return;
    } else if (this.get('content.cluster.status') === 'INSTALL FAILED' || this.get('content.cluster.status') === 'START FAILED' || this.get('content.cluster.status') === 'STARTED') {
      this.set('progress', '100');
      this.changeParseHostInfo(true);
      return;
    }
    this.changeParseHostInfo(true);
  },

  /**
   * @param polledData JSON data retrieved from API
   * @returns {bool} Has "Start All Services" request completed successfully
   * @method isServicesStarted
   */
  isServicesStarted: function isServicesStarted(polledData) {
    var clusterStatus = {};
    if (!polledData.someProperty('Tasks.status', 'PENDING') && !polledData.someProperty('Tasks.status', 'QUEUED') && !polledData.someProperty('Tasks.status', 'IN_PROGRESS')) {
      this.set('progress', '100');
      clusterStatus = {
        status: 'INSTALLED',
        requestId: this.get('content.cluster.requestId'),
        isCompleted: true
      };
      if (this.isSuccess(polledData)) {
        clusterStatus.status = 'STARTED';
        var serviceStartTime = App.dateTime();
        clusterStatus.installTime = ((parseInt(serviceStartTime) - parseInt(this.get('content.cluster.installStartTime'))) / 60000).toFixed(2);
      } else {
        clusterStatus.status = 'START FAILED'; // 'START FAILED' implies to step10 that installation was successful but start failed
      }
      this.saveClusterStatus(clusterStatus);
      this.saveInstalledHosts(this);
      return true;
    }
    return false;
  },

  /**
   * @param polledData Json data retrieved from API
   * @method setIsServicesInstalled
   */
  setIsServicesInstalled: function setIsServicesInstalled(polledData) {
    var clusterStatus = {};
    var self = this;

    if (!polledData.someProperty('Tasks.status', 'PENDING') && !polledData.someProperty('Tasks.status', 'QUEUED') && !polledData.someProperty('Tasks.status', 'IN_PROGRESS')) {
      clusterStatus = {
        status: 'PENDING',
        requestId: this.get('content.cluster.requestId'),
        isCompleted: false
      };
      if (this.get('status') === 'failed') {
        clusterStatus.status = 'INSTALL FAILED';
        this.saveClusterStatus(clusterStatus);
        this.setProperties({
          progress: '100',
          isPolling: false
        });
        this.get('hosts').forEach(function (host) {
          host.set('progress', '100');
        });
        this.isAllComponentsInstalled().done(function () {
          self.changeParseHostInfo(false);
        });
        return;
      }
      if (App.get('supports.skipComponentStartAfterInstall')) {
        clusterStatus.status = 'START_SKIPPED';
        clusterStatus.isCompleted = true;
        this.saveClusterStatus(clusterStatus);
        this.get('hosts').forEach(function (host) {
          host.set('status', 'success');
          host.set('message', Em.I18n.t('installer.step9.host.status.success'));
          host.set('progress', '100');
        });
        this.set('progress', '100');
        self.saveInstalledHosts(self);
        this.changeParseHostInfo(true);
      } else {
        this.set('progress', '34');
        this.isAllComponentsInstalled().done(function () {
          self.saveInstalledHosts(self);
          self.changeParseHostInfo(true);
        });
        return;
      }
    }
    this.changeParseHostInfo(false);
  },

  /**
   * This is done at HostRole level.
   * @param {Ember.Enumerable} tasksPerHost
   * @param {Object} host
   * @method setLogTasksStatePerHost
   */
  setLogTasksStatePerHost: function setLogTasksStatePerHost(tasksPerHost, host) {
    tasksPerHost.forEach(function (_task) {
      var task = host.logTasks.findProperty('Tasks.id', _task.Tasks.id);
      if (task) {
        task.Tasks.status = _task.Tasks.status;
        task.Tasks.start_time = _task.Tasks.start_time;
        task.Tasks.end_time = _task.Tasks.end_time;
        task.Tasks.exit_code = _task.Tasks.exit_code;
      } else {
        host.logTasks.pushObject(_task);
      }
    }, this);
  },

  /**
   * Parses the Json data retrieved from API and sets the task on the host of {hosts} array binded to template
   * @param polledData Json data retrieved from API
   * @method setParseHostInfo
   */
  setParseHostInfo: function setParseHostInfo(polledData) {
    var self = this;
    var totalProgress = 0;
    var tasksData = polledData.tasks || [];
    var requestId = this.get('content.cluster.requestId');
    tasksData.setEach('Tasks.request_id', requestId);
    if (polledData.Requests && polledData.Requests.id && polledData.Requests.id != requestId) {
      // We don't want to use non-current requestId's tasks data to
      // determine the current install status.
      // Also, we don't want to keep polling if it is not the
      // current requestId.
      this.changeParseHostInfo(false);
      return;
    }
    this.replacePolledData(tasksData);
    var tasksHostMap = {};
    tasksData.forEach(function (task) {
      if (tasksHostMap[task.Tasks.host_name]) {
        tasksHostMap[task.Tasks.host_name].push(task);
      } else {
        tasksHostMap[task.Tasks.host_name] = [task];
      }
    });

    this.get('hosts').forEach(function (_host) {
      var actionsPerHost = tasksHostMap[_host.name] || []; // retrieved from polled Data
      if (actionsPerHost.length === 0) {
        if (this.get('content.cluster.status') === 'PENDING' || this.get('content.cluster.status') === 'INSTALL FAILED') {
          _host.set('progress', '33');
          _host.set('isNoTasksForInstall', true);
          _host.set('status', 'pending');
        }
        if (this.get('content.cluster.status') === 'INSTALLED' || this.get('content.cluster.status') === 'FAILED' || App.get('supports.skipComponentStartAfterInstall')) {
          _host.set('progress', '100');
          _host.set('status', 'success');
        }
      } else {
        _host.set('isNoTasksForInstall', false);
      }
      this.setLogTasksStatePerHost(actionsPerHost, _host);
      this.onSuccessPerHost(actionsPerHost, _host); // every action should be a success
      this.onErrorPerHost(actionsPerHost, _host); // any action should be a failure
      this.onInProgressPerHost(actionsPerHost, _host); // current running action for a host
      totalProgress += self.progressPerHost(actionsPerHost, _host);
      if (_host.get('progress') == '33' && _host.get('status') != 'failed' && _host.get('status') != 'warning') {
        _host.set('message', this.t('installer.step9.host.status.nothingToInstall'));
        _host.set('status', 'pending');
      }
    }, this);
    this.set('logTasksChangesCounter', this.get('logTasksChangesCounter') + 1);
    totalProgress = Math.floor(totalProgress / this.get('hosts.length'));
    this.set('progress', totalProgress.toString());
    this.setFinishState(tasksData);
  },

  /**
   * Starts polling to the API
   * @method startPolling
   */
  startPolling: function startPolling() {
    this.set('isSubmitDisabled', true);
    this.doPolling();
  },

  /**
   * This function calls API just once to fetch log data of all tasks.
   * @method loadLogData
   */
  loadLogData: function loadLogData(startPolling) {
    var requestsId = this.get('wizardController').getDBProperty('cluster').oldRequestsId;
    if (App.get('testMode')) {
      this.set('POLL_INTERVAL', 1);
    }
    this.getLogsByRequest(!!startPolling, requestsId[requestsId.length - 1]);
  },

  /**
   * Load form server <code>stderr, stdout</code> of current open task
   * @method loadCurrentTaskLog
   */
  loadCurrentTaskLog: function loadCurrentTaskLog() {
    var taskId = this.get('currentOpenTaskId');
    var requestId = this.get('currentOpenTaskRequestId');
    var clusterName = this.get('content.cluster.name');
    if (!taskId) {
      return;
    }
    App.ajax.send({
      name: 'background_operations.get_by_task',
      sender: this,
      data: {
        'taskId': taskId,
        'requestId': requestId,
        'clusterName': clusterName
      },
      success: 'loadCurrentTaskLogSuccessCallback',
      error: 'loadCurrentTaskLogErrorCallback'
    });
  },

  /**
   * success callback function for getting log data of the opened task
   * @param {object} data
   * @method loadCurrentTaskLogSuccessCallback
   */
  loadCurrentTaskLogSuccessCallback: function loadCurrentTaskLogSuccessCallback(data) {
    var taskId = this.get('currentOpenTaskId');
    if (taskId) {
      var host = this.get('hosts').findProperty('name', data.Tasks.host_name);
      if (host) {
        var currentTask = host.get('logTasks').findProperty('Tasks.id', data.Tasks.id);
        if (currentTask) {
          currentTask.Tasks.stderr = data.Tasks.stderr;
          currentTask.Tasks.stdout = data.Tasks.stdout;
          currentTask.Tasks.output_log = data.Tasks.output_log;
          currentTask.Tasks.error_log = data.Tasks.error_log;
        }
      }
    }
    this.set('logTasksChangesCounter', this.get('logTasksChangesCounter') + 1);
  },

  /**
   * Error callback function for getting log data of the opened task
   * @method loadCurrentTaskLogErrorCallback
   */
  loadCurrentTaskLogErrorCallback: function loadCurrentTaskLogErrorCallback() {
    this.set('currentOpenTaskId', 0);
  },

  /**
   * Function polls the API to retrieve data for the request.
   * @param {bool} polling whether to continue polling for status or not
   * @param {number} requestId
   * @method getLogsByRequest
   * @return {$.ajax|null}
   */
  getLogsByRequest: function getLogsByRequest(polling, requestId) {
    return App.ajax.send({
      name: 'wizard.step9.load_log',
      sender: this,
      data: {
        polling: polling,
        cluster: this.get('content.cluster.name'),
        requestId: requestId,
        numPolls: this.get('numPolls'),
        callback: this.getLogsByRequest,
        args: [polling, requestId],
        timeout: 3000
      },
      success: 'reloadSuccessCallback',
      error: 'reloadErrorCallback'
    });
  },

  /**
   * Success callback for get log by request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method reloadSuccessCallback
   */
  reloadSuccessCallback: function reloadSuccessCallback(data, opt, params) {
    var parsedData = jQuery.parseJSON(data);
    this._super();
    this.set('isPolling', params.polling);
    this.setParseHostInfo(parsedData);
  },

  /**
   * Error-callback for get log by request
   * @param {object} jqXHR
   * @param {string} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   * @method reloadErrorCallback
   */
  reloadErrorCallback: function reloadErrorCallback(jqXHR, ajaxOptions, error, opt, params) {
    if (jqXHR.status) {
      this.closeReloadPopup();
      this.loadLogData(true);
    } else {
      this._super(jqXHR, ajaxOptions, error, opt, params);
    }
  },

  /**
   * Delegates the function call to {getLogsByRequest} with appropriate params
   * @method doPolling
   */
  doPolling: function doPolling() {
    var requestId = this.get('content.cluster.requestId');
    if (App.get('testMode')) {
      this.incrementProperty('numPolls');
    }
    this.getLogsByRequest(true, requestId);
  },

  /**
   * Check that all components are in INSTALLED state before issuing start command
   * @method isAllComponentsInstalled
   * @return {$.ajax|null}
   */
  isAllComponentsInstalled: function isAllComponentsInstalled() {
    var dfd = $.Deferred();
    App.ajax.send({
      name: 'wizard.step9.installer.get_host_status',
      sender: this,
      data: {
        cluster: this.get('content.cluster.name')
      },
      success: 'isAllComponentsInstalledSuccessCallback',
      error: 'isAllComponentsInstalledErrorCallback'
    }).complete(function () {
      dfd.resolve();
    });
    return dfd.promise();
  },

  /**
   * Success callback function for API checking host state and host_components state.
   * @param {Object} jsonData
   * @method isAllComponentsInstalledSuccessCallback
   */
  isAllComponentsInstalledSuccessCallback: function isAllComponentsInstalledSuccessCallback(jsonData) {
    var clusterStatus = {
      status: 'INSTALL FAILED',
      isStartError: true,
      isCompleted: false
    };
    var usedHostWithHeartbeatLost = false;
    var hostsWithHeartbeatLost = [];
    var usedHosts = this.get('content.masterComponentHosts').filterProperty('isInstalled', false).mapProperty('hostName');
    this.get('content.slaveComponentHosts').forEach(function (component) {
      usedHosts = usedHosts.concat(component.hosts.filterProperty('isInstalled', false).mapProperty('hostName'));
    });
    usedHosts = usedHosts.uniq();
    jsonData.items.filterProperty('Hosts.host_state', 'HEARTBEAT_LOST').forEach(function (host) {
      var hostComponentObj = { hostName: host.Hosts.host_name };
      var componentArr = [];
      host.host_components.forEach(function (_hostComponent) {
        var componentName = App.format.role(_hostComponent.HostRoles.component_name, false);
        componentArr.pushObject(componentName);
      }, this);
      hostComponentObj.componentNames = stringUtils.getFormattedStringFromArray(componentArr);
      hostsWithHeartbeatLost.pushObject(hostComponentObj);
      if (!usedHostWithHeartbeatLost && usedHosts.contains(host.Hosts.host_name)) {
        usedHostWithHeartbeatLost = true;
      }
    }, this);
    this.set('hostsWithHeartbeatLost', hostsWithHeartbeatLost);
    if (usedHostWithHeartbeatLost) {
      this.get('hosts').forEach(function (host) {
        if (hostsWithHeartbeatLost.someProperty('hostName', host.get('name'))) {
          host.set('status', 'heartbeat_lost');
        } else if (host.get('status') !== 'failed' && host.get('status') !== 'warning') {
          host.set('message', Em.I18n.t('installer.step9.host.status.startAborted'));
        }
        host.set('progress', '100');
      });
      this.set('progress', '100');
      this.saveClusterStatus(clusterStatus);
    } else if (this.get('content.cluster.status') === 'PENDING' && this.get('isPolling')) {
      if (App.get('supports.skipComponentStartAfterInstall')) {
        this.set('progress', '100');
        this.changeParseHostInfo(true);
      } else {
        this.launchStartServices();
      }
    }
  },

  /**
   * Error callback function for API checking host state and host_components state
   * @method isAllComponentsInstalledErrorCallback
   */
  isAllComponentsInstalledErrorCallback: function isAllComponentsInstalledErrorCallback() {
    var clusterStatus = {
      status: 'INSTALL FAILED',
      isStartError: true,
      isCompleted: false
    };
    this.set('progress', '100');
    this.get('hosts').forEach(function (host) {
      if (host.get('status') !== 'failed' && host.get('status') !== 'warning') {
        host.set('message', Em.I18n.t('installer.step9.host.status.startAborted'));
        host.set('progress', '100');
      }
    });
    this.saveClusterStatus(clusterStatus);
  },

  /**
   * save cluster status in the parentController and localdb
   * @param {object} clusterStatus
   * @method saveClusterStatus
   */
  saveClusterStatus: function saveClusterStatus(clusterStatus) {
    if (!App.get('testMode')) {
      App.router.get(this.get('content.controllerName')).saveClusterStatus(clusterStatus);
    } else {
      this.set('content.cluster', clusterStatus);
    }
  },

  /**
   * save cluster status in the parentController and localdb
   * @param {object} context
   * @method saveInstalledHosts
   */
  saveInstalledHosts: function saveInstalledHosts(context) {
    if (!App.get('testMode')) {
      App.router.get(this.get('content.controllerName')).saveInstalledHosts(context);
    }
  },

  /**
   * Load ambari property to determine if we should run service checks on deploy
   */
  loadDoServiceChecksFlag: function loadDoServiceChecksFlag() {
    var def = $.Deferred();
    App.ajax.send({
      name: 'ambari.service',
      sender: this,
      data: {
        fields: '?fields=RootServiceComponents/properties/skip.service.checks'
      },
      success: 'loadDoServiceChecksFlagSuccessCallback'
    }).complete(function () {
      def.resolve();
    });
    return def.promise();
  },

  /**
   *  Callback function Load ambari property to determine if we should run service checks on deploy
   * @param {Object} data
   * @method loadDoServiceChecksFlagSuccessCallback
   */
  loadDoServiceChecksFlagSuccessCallback: function loadDoServiceChecksFlagSuccessCallback(data) {
    var properties = Em.get(data, 'RootServiceComponents.properties');
    if (properties && properties.hasOwnProperty('skip.service.checks')) {
      this.set('skipServiceChecks', properties['skip.service.checks'] === 'true');
    }
  }

});

});

require.register("data/BIGTOP/site_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports =
{
  "configProperties": [
  //***************************************** BIGTOP stack **************************************
  /**********************************************HDFS***************************************/
    {
      "name": "dfs.namenode.checkpoint.dir",
      "displayName": "SecondaryNameNode Checkpoint directories",
      "displayType": "directories",
      "isOverridable": false,
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "category": "SECONDARY_NAMENODE",
      "index": 1
    },
    {
      "name": "dfs.namenode.checkpoint.period",
      "displayName": "HDFS Maximum Checkpoint Delay",
      "displayType": "int",
      "unit": "seconds",
      "category": "General",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "index": 3
    },
    {
      "name": "dfs.namenode.name.dir",
      "displayName": "NameNode directories",
      "displayType": "directories",
      "isOverridable": false,
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "category": "NAMENODE",
      "index": 1
    },
    {
      "name": "dfs.webhdfs.enabled",
      "displayName": "WebHDFS enabled",
      "displayType": "checkbox",
      "isOverridable": false,
      "category": "General",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "index": 0
    },
    {
      "name": "dfs.datanode.failed.volumes.tolerated",
      "displayName": "DataNode volumes failure toleration",
      "displayType": "int",
      "category": "DATANODE",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "index": 3
    },
    {
      "name": "dfs.datanode.data.dir",
      "displayName": "DataNode directories",
      "displayType": "directories",
      "category": "DATANODE",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "index": 1
    },
    {
      "name": "dfs.datanode.data.dir.perm",
      "displayName": "DataNode directories permission",
      "displayType": "int",
      "category": "DATANODE",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml"
    },
    {
      "name": "dfs.replication",
      "displayName": "Block replication",
      "displayType": "int",
      "category": "General",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml"
    },
    {
      "name": "dfs.datanode.du.reserved",
      "displayName": "Reserved space for HDFS",
      "displayType": "int",
      "unit": "bytes",
      "category": "General",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml",
      "index": 2
    },
    {
      "name": "dfs.client.read.shortcircuit",
      "displayName": "HDFS Short-circuit read",
      "displayType": "checkbox",
      "category": "Advanced hdfs-site",
      "serviceName": "HDFS",
      "filename": "hdfs-site.xml"
    },
    {
      "name": "apache_artifacts_download_url",
      "displayName": "apache_artifacts_download_url",
      "description": "",
      "isRequired": false,
      "isRequiredByAgent": false,
      "isVisible": false,
      "category": "Advanced hdfs-site",
      "serviceName": "HDFS"
    },

  /**********************************************YARN***************************************/
    {
      "name": "yarn.acl.enable",
      "displayName": "yarn.acl.enable",
      "displayType": "checkbox",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "RESOURCEMANAGER"
    },
    {
      "name": "yarn.admin.acl",
      "displayName": "yarn.admin.acl",
      "isRequired": false,
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "RESOURCEMANAGER"
    },
    {
      "name": "yarn.log-aggregation-enable",
      "displayName": "yarn.log-aggregation-enable",
      "displayType": "checkbox",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "RESOURCEMANAGER"
    },
    {
      "name": "yarn.resourcemanager.scheduler.class",
      "displayName": "yarn.resourcemanager.scheduler.class",
      "serviceName": "YARN",
      "category": "CapacityScheduler",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.scheduler.minimum-allocation-mb",
      "displayName": "yarn.scheduler.minimum-allocation-mb",
      "displayType": "int",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "CapacityScheduler"
    },
    {
      "name": "yarn.scheduler.maximum-allocation-mb",
      "displayName": "yarn.scheduler.maximum-allocation-mb",
      "displayType": "int",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "CapacityScheduler"
    },
    {
      "name": "yarn.nodemanager.resource.memory-mb",
      "displayName": "yarn.nodemanager.resource.memory-mb",
      "displayType": "int",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.vmem-pmem-ratio",
      "displayName": "yarn.nodemanager.vmem-pmem-ratio",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.linux-container-executor.group",
      "displayName": "yarn.nodemanager.linux-container-executor.group",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.log-dirs",
      "displayName": "yarn.nodemanager.log-dirs",
      "displayType": "directories",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.local-dirs",
      "displayName": "yarn.nodemanager.local-dirs",
      "displayType": "directories",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.remote-app-log-dir",
      "displayName": "yarn.nodemanager.remote-app-log-dir",
      "displayType": "directory",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.remote-app-log-dir-suffix",
      "displayName": "yarn.nodemanager.remote-app-log-dir-suffix",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.aux-services",
      "displayName": "yarn.nodemanager.aux-services",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.nodemanager.log.retain-second",
      "displayName": "yarn.nodemanager.log.retain-second",
      "serviceName": "YARN",
      "filename": "yarn-site.xml",
      "category": "NODEMANAGER"
    },
    {
      "name": "yarn.log.server.url",
      "displayName": "yarn.log.server.url",
      "category": "Advanced yarn-site",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.enabled",
      "displayName": "yarn.timeline-service.enabled",
      "category": "APP_TIMELINE_SERVER",
      "displayType": "checkbox",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.leveldb-timeline-store.path",
      "displayName": "yarn.timeline-service.leveldb-timeline-store.path",
      "category": "APP_TIMELINE_SERVER",
      "displayType": "directory",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.leveldb-timeline-store.ttl-interval-ms",
      "displayName": "yarn.timeline-service.leveldb-timeline-store.ttl-interval-ms",
      "displayType": "int",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.store-class",
      "displayName": "yarn.timeline-service.store-class",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.ttl-enable",
      "displayName": "yarn.timeline-service.ttl-enable",
      "displayType": "checkbox",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.ttl-ms",
      "displayName": "yarn.timeline-service.ttl-ms",
      "displayType": "int",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.generic-application-history.store-class",
      "displayName": "yarn.timeline-service.generic-application-history.store-class",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.webapp.address",
      "displayName": "yarn.timeline-service.webapp.address",
      "displayType": "string",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.webapp.https.address",
      "displayName": "yarn.timeline-service.webapp.https.address",
      "displayType": "string",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
    {
      "name": "yarn.timeline-service.address",
      "displayName": "yarn.timeline-service.address",
      "displayType": "string",
      "category": "APP_TIMELINE_SERVER",
      "serviceName": "YARN",
      "filename": "yarn-site.xml"
    },
  /**********************************************MAPREDUCE2***************************************/
    {
      "name": "mapreduce.map.memory.mb",
      "displayName": "Default virtual memory for a job's map-task",
      "displayType": "int",
      "unit": "MB",
      "category": "General",
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-site.xml"
    },
    {
      "name": "mapreduce.reduce.memory.mb",
      "displayName": "Default virtual memory for a job's reduce-task",
      "displayType": "int",
      "unit": "MB",
      "category": "General",
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-site.xml"
    },
    {
      "name": "mapreduce.task.io.sort.mb",
      "displayName": "Map-side sort buffer memory",
      "displayType": "int",
      "unit": "MB",
      "category": "General",
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-site.xml"
    },
    {
      "name": "hadoop.security.auth_to_local",
      "displayName": "hadoop.security.auth_to_local",
      "displayType": "multiLine",
      "serviceName": "HDFS",
      "filename": "core-site.xml",
      "category": "Advanced core-site"
    },
    {
      "name": "yarn.app.mapreduce.am.resource.mb",
      "displayName": "yarn.app.mapreduce.am.resource.mb",
      "displayType": "int",
      "category": "Advanced mapred-site",
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-site.xml"
    },

  /**********************************************oozie-site***************************************/
    {
      "name": "oozie.db.schema.name",
      "displayName": "Database Name",
      "isOverridable": false,
      "displayType": "host",
      "isObserved": true,
      "category": "OOZIE_SERVER",
      "serviceName": "OOZIE",
      "filename": "oozie-site.xml",
      "index": 4
    },
    {
      "name": "oozie.service.JPAService.jdbc.username",
      "displayName": "Database Username",
      "isOverridable": false,
      "displayType": "host",
      "category": "OOZIE_SERVER",
      "serviceName": "OOZIE",
      "filename": "oozie-site.xml",
      "index": 5
    },
    {
      "name": "oozie.service.JPAService.jdbc.password",
      "displayName": "Database Password",
      "isOverridable": false,
      "displayType": "password",
      "category": "OOZIE_SERVER",
      "serviceName": "OOZIE",
      "filename": "oozie-site.xml",
      "index": 6
    },
    {
      "name": "oozie.service.JPAService.jdbc.driver", // the default value of this property is overriden in code
      "displayName": "JDBC Driver Class",
      "isOverridable": false,
      "category": "OOZIE_SERVER",
      "serviceName": "OOZIE",
      "filename": "oozie-site.xml",
      "index": 7
    },
    {
      "name": "oozie.service.JPAService.jdbc.url",
      "displayName": "Database URL",
      "isOverridable": false,
      "displayType": "advanced",
      "category": "OOZIE_SERVER",
      "serviceName": "OOZIE",
      "filename": "oozie-site.xml",
      "index": 8
    },

  /**********************************************hive-site***************************************/
    {
      "name": "javax.jdo.option.ConnectionDriverName",  // the default value is overwritten in code
      "displayName": "JDBC Driver Class",
      "isOverridable": false,
      "category": "HIVE_METASTORE",
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "index": 7
    },
    {
      "name": "hive.heapsize",
      "displayName": "Hive heap size",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "category": "General",
      "index": 9
    },
    {
      "name": "javax.jdo.option.ConnectionUserName",
      "displayName": "Database Username",
      "displayType": "host",
      "isOverridable": false,
      "category": "HIVE_METASTORE",
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "index": 5
    },
    {
      "name": "javax.jdo.option.ConnectionPassword",
      "displayName": "Database Password",
      "displayType": "password",
      "isOverridable": false,
      "category": "HIVE_METASTORE",
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "index": 6
    },
    {
      "name": "javax.jdo.option.ConnectionURL",
      "displayName": "Database URL",
      "displayType": "advanced",
      "isOverridable": false,
      "category": "HIVE_METASTORE",
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "index": 8
    },
    {
      "name": "ambari.hive.db.schema.name",
      "displayName": "Database Name",
      "displayType": "host",
      "isOverridable": false,
      "isObserved": true,
      "serviceName": "HIVE",
      "filename": "hive-site.xml",
      "category": "HIVE_METASTORE",
      "index": 4
    },
    {
      "name": "hive.server2.tez.default.queues",
      "displayName": "hive.server2.tez.default.queues",
      "isRequired": false,
      "serviceName": "HIVE",
      "category": "Advanced hive-site"
    },
    {
      "name": "hive.server2.thrift.port",
      "displayName": "Hive Server Port",
      "description": "TCP port number to listen on, default 10000.",
      "recommendedValue": "10000",
      "displayType": "int",
      "isReconfigurable": true,
      "isOverridable": false,
      "isVisible": true,
      "category": "Advanced hive-site",
      "serviceName": "HIVE",
      "filename": "hive-site.xml"
    },
    {
      "name": "hive.server2.support.dynamic.service.discovery",
      "displayName": "hive.server2.support.dynamic.service.discovery",
      "recommendedValue": true,
      "displayType": "checkbox",
      "category": "Advanced hive-site",
      "serviceName": "HIVE"
    },
    {
      "name": "hive.zookeeper.quorum",
      "displayName": "hive.zookeeper.quorum",
      "recommendedValue": "localhost:2181",
      "displayType": "multiLine",
      "isVisible": true,
      "serviceName": "HIVE",
      "category": "Advanced hive-site"
    },
  /**********************************************tez-site*****************************************/
    {
      "name": "tez.am.resource.memory.mb",
      "displayName": "tez.am.resource.memory.mb",
      "displayType": "int",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.am.java.opts",
      "displayName": "tez.am.java.opts",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.am.grouping.split-waves",
      "displayName": "tez.am.grouping.split-waves",
      "displayType": "float",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.am.grouping.min-size",
      "displayName": "tez.am.grouping.min-size",
      "displayType": "int",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.am.grouping.max-size",
      "displayName": "tez.am.grouping.max-size",
      "displayType": "int",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.am.log.level",
      "displayName": "tez.am.log.level",
      "displayType": "string",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.runtime.intermediate-input.compress.codec",
      "displayName": "tez.runtime.intermediate-input.compress.codec",
      "displayType": "string",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.runtime.intermediate-input.is-compressed",
      "displayName": "tez.runtime.intermediate-input.is-compressed",
      "displayType": "checkbox",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.runtime.intermediate-output.compress.codec",
      "displayName": "tez.runtime.intermediate-output.compress.codec",
      "displayType": "string",
      "category": "General",
      "serviceName": "TEZ"
    },
    {
      "name": "tez.runtime.intermediate-output.should-compress",
      "displayName": "tez.runtime.intermediate-output.should-compress",
      "displayType": "checkbox",
      "category": "General",
      "serviceName": "TEZ"
    },

  /**********************************************hbase-site***************************************/
    {
      "name": "hbase.tmp.dir",
      "displayName": "HBase tmp directory",
      "displayType": "directory",
      "category": "Advanced hbase-site",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.master.port",
      "displayName": "HBase Master Port",
      "isReconfigurable": true,
      "displayType": "int",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "category": "Advanced hbase-site"
    },
    {
      "name": "hbase.regionserver.global.memstore.upperLimit",
      "displayName": "hbase.regionserver.global.memstore.upperLimit",
      "displayType": "float",
      "category": "Advanced hbase-site",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.regionserver.global.memstore.lowerLimit",
      "displayName": "hbase.regionserver.global.memstore.lowerLimit",
      "displayType": "float",
      "category": "Advanced hbase-site",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.hstore.blockingStoreFiles",
      "displayName": "hstore blocking storefiles",
      "displayType": "int",
      "category": "Advanced hbase-site",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.hstore.compactionThreshold",
      "displayName": "HBase HStore compaction threshold",
      "displayType": "int",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 0
    },
    {
      "name": "hfile.block.cache.size",
      "displayName": "HFile block cache size ",
      "displayType": "float",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 1
    },
    {
      "name": "hbase.hregion.max.filesize",
      "displayName": "Maximum HStoreFile Size",
      "displayType": "int",
      "unit": "bytes",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 2
    },
    {
      "name": "hbase.regionserver.handler.count",
      "displayName": "RegionServer Handler",
      "displayType": "int",
      "category": "HBASE_REGIONSERVER",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 2
    },
    {
      "name": "hbase.hregion.majorcompaction",
      "displayName": "HBase Region Major Compaction",
      "displayType": "int",
      "unit": "ms",
      "category": "HBASE_REGIONSERVER",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 3
    },
    {
      "name": "hbase.hregion.memstore.block.multiplier",
      "displayName": "HBase Region Block Multiplier",
      "displayType": "int",
      "category": "HBASE_REGIONSERVER",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 4
    },
    {
      "name": "hbase.hregion.memstore.mslab.enabled",
      "displayName": "hbase.hregion.memstore.mslab.enabled",
      "displayType": "checkbox",
      "category": "Advanced hbase-site",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.hregion.memstore.flush.size",
      "displayName": "HBase Region Memstore Flush Size",
      "displayType": "int",
      "unit": "bytes",
      "category": "HBASE_REGIONSERVER",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 5
    },
    {
      "name": "hbase.client.scanner.caching",
      "displayName": "HBase Client Scanner Caching",
      "displayType": "int",
      "unit": "rows",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 3
    },
    {
      "name": "zookeeper.session.timeout",
      "displayName": "Zookeeper timeout for HBase Session",
      "displayType": "int",
      "unit": "ms",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 4
    },
    {
      "name": "hbase.client.keyvalue.maxsize",
      "displayName": "HBase Client Maximum key-value Size",
      "displayType": "int",
      "unit": "bytes",
      "category": "General",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "index": 5
    },
    {
      "name": "hbase.coprocessor.region.classes",
      "displayName": "hbase.coprocessor.region.classes",
      "category": "Advanced hbase-site",
      "isRequired": false,
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.coprocessor.master.classes",
      "displayName": "hbase.coprocessor.master.classes",
      "category": "Advanced hbase-site",
      "isRequired": false,
      "serviceName": "HBASE",
      "filename": "hbase-site.xml"
    },
    {
      "name": "hbase.zookeeper.quorum",
      "displayName": "hbase.zookeeper.quorum",
      "displayType": "multiLine",
      "serviceName": "HBASE",
      "filename": "hbase-site.xml",
      "category": "Advanced hbase-site"
    },

  /**********************************************storm-site***************************************/
    {
      "name": "storm.zookeeper.root",
      "displayName": "storm.zookeeper.root",
      "displayType": "directory",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.local.dir",
      "displayName": "storm.local.dir",
      "displayType": "directory",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.servers",
      "displayName": "storm.zookeeper.servers",
      "displayType": "componentHosts",
      "isOverridable": false,
      "isReconfigurable": false,
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.port",
      "displayName": "storm.zookeeper.port",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.session.timeout",
      "displayName": "storm.zookeeper.session.timeout",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.connection.timeout",
      "displayName": "storm.zookeeper.connection.timeout",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.retry.times",
      "displayName": "storm.zookeeper.retry.times",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.zookeeper.retry.interval",
      "displayName": "storm.zookeeper.retry.interval",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General",
      "unit": "ms"
    },
    {
      "name": "storm.zookeeper.retry.intervalceiling.millis",
      "displayName": "storm.zookeeper.retry.intervalceiling.millis",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "General",
      "unit": "ms"
    },
    {
      "name": "storm.cluster.mode",
      "displayName": "storm.cluster.mode",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.local.mode.zmq",
      "displayName": "storm.local.mode.zmq",
      "displayType": "checkbox",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.thrift.transport",
      "displayName": "storm.thrift.transport",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "name": "storm.messaging.transport",
      "displayName": "storm.messaging.transport",
      "serviceName": "STORM",
      "category": "General"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.buffer_size",
      "name": "storm.messaging.netty.buffer_size",
      "displayType": "int",
      "unit": "bytes"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.max_retries",
      "name": "storm.messaging.netty.max_retries",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.max_wait_ms",
      "name": "storm.messaging.netty.max_wait_ms",
      "displayType": "int",
      "unit": "ms"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.min_wait_ms",
      "name": "storm.messaging.netty.min_wait_ms",
      "displayType": "int",
      "unit": "ms"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.server_worker_threads",
      "name": "storm.messaging.netty.server_worker_threads",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "General",
      "displayName": "storm.messaging.netty.client_worker_threads",
      "name": "storm.messaging.netty.client_worker_threads",
      "displayType": "int"
    },
    {
      "name": "nimbus.host",
      "displayName": "nimbus.host",
      "displayType": "masterHost",
      "isOverridable": false,
      "isReconfigurable": false,
      "serviceName": "STORM",
      "category": "NIMBUS"
    },
    {
      "name": "nimbus.thrift.port",
      "displayName": "nimbus.thrift.port",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS"
    },
    {
      "name": "nimbus.thrift.max_buffer_size",
      "displayName": "nimbus.thrift.max_buffer_size",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "bytes"
    },
    {
      "name": "nimbus.childopts",
      "displayName": "nimbus.childopts",
      "displayType": "multiLine",
      "isOverridable": false,
      "serviceName": "STORM",
      "category": "NIMBUS",
      "filename": "storm-site.xml"
    },
    {
      "name": "nimbus.task.timeout.secs",
      "displayName": "nimbus.task.timeout.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.supervisor.timeout.secs",
      "displayName": "nimbus.supervisor.timeout.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.monitor.freq.secs",
      "displayName": "nimbus.monitor.freq.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.cleanup.inbox.freq.secs",
      "displayName": "nimbus.cleanup.inbox.freq.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.inbox.jar.expiration.secs",
      "displayName": "nimbus.inbox.jar.expiration.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.task.launch.secs",
      "displayName": "nimbus.task.launch.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.reassign",
      "displayName": "nimbus.reassign",
      "displayType": "checkbox",
      "isReconfigurable": true,
      "serviceName": "STORM",
      "category": "NIMBUS"
    },
    {
      "name": "nimbus.file.copy.expiration.secs",
      "displayName": "nimbus.file.copy.expiration.secs",
      "displayType": "int",
      "serviceName": "STORM",
      "category": "NIMBUS",
      "unit": "seconds"
    },
    {
      "name": "nimbus.topology.validator",
      "displayName": "nimbus.topology.validator",
      "serviceName": "STORM",
      "category": "NIMBUS"
    },
    {
      "name": "supervisor.slots.ports",
      "displayName": "supervisor.slots.ports",
      "displayType": "string",
      "serviceName": "STORM",
      "category": "SUPERVISOR"
    },
    {
      "isOverrideable": false,
      "serviceName": "STORM",
      "category": "SUPERVISOR",
      "displayName": "supervisor.childopts",
      "name": "supervisor.childopts",
      "displayType": "multiLine",
      "filename": "storm-site.xml"
    },
    {
      "serviceName": "STORM",
      "category": "SUPERVISOR",
      "displayName": "supervisor.worker.start.timeout.secs",
      "name": "supervisor.worker.start.timeout.secs",
      "displayType": "int",
      "unit": "seconds"
    },
    {
      "serviceName": "STORM",
      "category": "SUPERVISOR",
      "displayName": "supervisor.worker.timeout.secs",
      "name": "supervisor.worker.timeout.secs",
      "displayType": "int",
      "unit": "seconds"
    },
    {
      "serviceName": "STORM",
      "category": "SUPERVISOR",
      "displayName": "supervisor.monitor.frequency.secs",
      "name": "supervisor.monitor.frequency.secs",
      "displayType": "int",
      "unit": "seconds"
    },
    {
      "serviceName": "STORM",
      "category": "SUPERVISOR",
      "displayName": "supervisor.heartbeat.frequency.secs",
      "name": "supervisor.heartbeat.frequency.secs",
      "displayType": "int",
      "unit": "seconds"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.port",
      "name": "drpc.port",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.worker.threads",
      "name": "drpc.worker.threads",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.queue.size",
      "name": "drpc.queue.size",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.invocations.port",
      "name": "drpc.invocations.port",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.request.timeout.secs",
      "name": "drpc.request.timeout.secs",
      "displayType": "int",
      "unit": "seconds"
    },
    {
      "serviceName": "STORM",
      "category": "DRPC_SERVER",
      "displayName": "drpc.childopts",
      "name": "drpc.childopts",
      "displayType": "string"
    },
    {
      "serviceName": "STORM",
      "category": "STORM_UI_SERVER",
      "displayName": "ui.port",
      "name": "ui.port",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "STORM_UI_SERVER",
      "displayName": "ui.childopts",
      "name": "ui.childopts",
      "displayType": "string"
    },
    //@Todo: uncomment following properties when logviewer is treated as different section on storm service page
    /*
    {
      "serviceName": "STORM",
      "category": "LogviewerServer",
      "displayName": "logviewer.port",
      "name": "logviewer.port",
      "displayType": "int"
    },
    {
      "serviceName": "STORM",
      "category": "LogviewerServer",
      "displayName": "logviewer.childopts",
      "name": "logviewer.childopts",
      "displayType": "string"
    },
    {
      "serviceName": "STORM",
      "category": "LogviewerServer",
      "displayName": "logviewer.appender.name",
      "name": "logviewer.appender.name",
      "displayType": "string"
    },
    */
    {
      "serviceName": "STORM",
      "category": "Advanced storm-site",
      "displayName": "worker.childopts",
      "name": "worker.childopts",
      "displayType": "multiLine",
      "filename": "storm-site.xml"
    },
  /*********************************************oozie-site for Falcon*****************************/
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-job-submit-instances",
      "name": "oozie.service.ELService.ext.functions.coord-job-submit-instances",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-action-create-inst",
      "name": "oozie.service.ELService.ext.functions.coord-action-create-inst",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-action-create",
      "name": "oozie.service.ELService.ext.functions.coord-action-create",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-job-submit-data",
      "name": "oozie.service.ELService.ext.functions.coord-job-submit-data",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-action-start",
      "name": "oozie.service.ELService.ext.functions.coord-action-start",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-sla-submit",
      "name": "oozie.service.ELService.ext.functions.coord-sla-submit",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },
    {
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "category": "Falcon - Oozie integration",
      "displayName": "oozie.service.ELService.ext.functions.coord-sla-create",
      "name": "oozie.service.ELService.ext.functions.coord-sla-create",
      "displayType": "custom",
      "filename": "oozie-site.xml"
    },

    // Runtime properties
    {
      "name": "*.domain",
      "displayName": "*.domain",
      "category": "FalconRuntimeSite",
      "serviceName": "FALCON",
      "filename": "falcon-runtime.properties.xml"

    },
    {
      "name": "*.log.cleanup.frequency.minutes.retention",
      "displayName": "*.log.cleanup.frequency.minutes.retention",
      "category": "FalconRuntimeSite",
      "serviceName": "FALCON",
      "filename": "falcon-runtime.properties.xml"
    },
    {
      "name": "*.log.cleanup.frequency.hours.retention",
      "displayName": "*.log.cleanup.frequency.hours.retention",
      "category": "FalconRuntimeSite",
      "serviceName": "FALCON",
      "filename": "falcon-runtime.properties.xml"
    },
    {
      "name": "*.log.cleanup.frequency.days.retention",
      "displayName": "*.log.cleanup.frequency.days.retention",
      "category": "FalconRuntimeSite",
      "serviceName": "FALCON",
      "filename": "falcon-runtime.properties.xml"
    },
    {
      "name": "*.log.cleanup.frequency.months.retention",
      "displayName": "*.log.cleanup.frequency.months.retention",
      "category": "FalconRuntimeSite",
      "serviceName": "FALCON",
      "filename": "falcon-runtime.properties.xml"
    },

    //  Startup properties

    {
      "name": "*.domain",
      "displayName": "*.domain",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.workflow.engine.impl",
      "displayName": "*.workflow.engine.impl",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.oozie.process.workflow.builder",
      "displayName": "*.oozie.process.workflow.builder",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.oozie.feed.workflow.builder",
      "displayName": "*.oozie.feed.workflow.builder",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.SchedulableEntityManager.impl",
      "displayName": "*.SchedulableEntityManager.impl",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.ConfigSyncService.impl",
      "displayName": "*.ConfigSyncService.impl",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.ProcessInstanceManager.impl",
      "displayName": "*.ProcessInstanceManager.impl",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.catalog.service.impl",
      "displayName": "*.catalog.service.impl",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.application.services",
      "displayName": "*.application.services",
      "displayType": "multiLine",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.configstore.listeners",
      "displayName": "*.configstore.listeners",
      "displayType": "multiLine",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.broker.impl.class",
      "displayName": "*.broker.impl.class",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.shared.libs",
      "displayName": "*.shared.libs",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.config.store.uri",
      "displayName": "*.config.store.uri",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.system.lib.location",
      "displayName": "*.system.lib.location",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.retry.recorder.path",
      "displayName": "*.retry.recorder.path",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.cleanup.service.frequency",
      "displayName": "*.falcon.cleanup.service.frequency",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.broker.url",
      "displayName": "*.broker.url",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.broker.ttlInMins",
      "displayName": "*.broker.ttlInMins",
      "displayType": "int",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.entity.topic",
      "displayName": "*.entity.topic",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.max.retry.failure.count",
      "displayName": "*.max.retry.failure.count",
      "displayType": "int",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.internal.queue.size",
      "displayName": "*.internal.queue.size",
      "displayType": "int",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.authentication.type",
      "displayName": "*.falcon.authentication.type",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.type",
      "displayName": "*.falcon.http.authentication.type",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.token.validity",
      "displayName": "*.falcon.http.authentication.token.validity",
      "displayType": "int",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.signature.secret",
      "displayName": "*.falcon.http.authentication.signature.secret",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.simple.anonymous.allowed",
      "displayName": "*.falcon.http.authentication.simple.anonymous.allowed",
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.kerberos.name.rules",
      "displayName": "*.falcon.http.authentication.kerberos.name.rules",
      "category": "FalconStartupSite",
      "displayType": "multiLine",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },
    {
      "name": "*.falcon.http.authentication.blacklisted.users",
      "displayName": "*.falcon.http.authentication.blacklisted.users",
      "isRequired": false,
      "category": "FalconStartupSite",
      "serviceName": "FALCON",
      "filename": "falcon-startup.properties.xml"
    },

  /**********************************************webhcat-site***************************************/
    {
      "name": "templeton.hive.archive",
      "displayName": "templeton.hive.archive",
      "isRequired": false,
      "serviceName": "HIVE",
      "filename": "webhcat-site.xml",
      "category": "Advanced webhcat-site"
    },
    {
      "name": "templeton.pig.archive",
      "displayName": "templeton.pig.archive",
      "isRequired": false,
      "serviceName": "HIVE",
      "filename": "webhcat-site.xml",
      "category": "Advanced webhcat-site"
    },
    {
      "name": "templeton.zookeeper.hosts",
      "displayName": "templeton.zookeeper.hosts",
      "displayType": "multiLine",
      "serviceName": "HIVE",
      "filename": "webhcat-site.xml",
      "category": "Advanced webhcat-site"
    },
  /**********************************************pig.properties*****************************************/
    {
      "name": "content",
      "displayName": "content",
      "value": "",
      "recommendedValue": "",
      "description": "pig properties",
      "displayType": "content",
      "isRequired": false,
      "showLabel": false,
      "serviceName": "PIG",
      "filename": "pig-properties.xml",
      "category": "Advanced pig-properties"
    },
  /********************************************* flume-agent *****************************/
    {
      "name": "content",
      "displayName": "content",
      "showLabel": false,
      "isRequired": false,
      "displayType": "content",
      "serviceName": "FLUME",
      "category": "FLUME_HANDLER",
      "filename": "flume-conf.xml"
    },
    {
      "name": "flume_conf_dir",
      "displayName": "Flume Conf Dir",
      "description": "Location to save configuration files",
      "recommendedValue": "/etc/flume/conf",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "FLUME",
      "filename": "flume-env.xml",
      "category": "Advanced flume-env",
      "index": 0
    },
    {
      "name": "flume_log_dir",
      "displayName": "Flume Log Dir",
      "description": "Location to save log files",
      "recommendedValue": "/var/log/flume",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "FLUME",
      "filename": "flume-env.xml",
      "category": "Advanced flume-env",
      "index": 1
    },
  /**********************************************HDFS***************************************/
    {
      "name": "namenode_heapsize",
      "displayName": "NameNode Java heap size",
      "description": "Initial and maximum Java heap size for NameNode (Java options -Xms and -Xmx).  This also applies to the Secondary NameNode.",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "NAMENODE",
      "index": 2
    },
    {
      "name": "namenode_opt_newsize",
      "displayName": "NameNode new generation size",
      "description": "Default size of Java new generation for NameNode (Java option -XX:NewSize).  This also applies to the Secondary NameNode.",
      "recommendedValue": "200",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "NAMENODE",
      "index": 3
    },
    {
      "name": "namenode_opt_maxnewsize",
      "displayName": "NameNode maximum new generation size",
      "description": "Maximum size of Java new generation for NameNode (Java option -XX:MaxnewSize).",
      "recommendedValue": "200",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "NAMENODE",
      "index": 4
    },
    {
      "name": "namenode_opt_permsize",
      "displayName": "NameNode permanent generation size",
      "description": "Default size of Java new generation for NameNode (Java option -XX:PermSize).  This also applies to the Secondary NameNode.",
      "recommendedValue": "128",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "NAMENODE",
      "index": 5
    },
    {
      "name": "namenode_opt_maxpermsize",
      "displayName": "NameNode maximum permanent generation size",
      "description": "Maximum size of Java permanent generation for NameNode (Java option -XX:MaxPermSize).",
      "recommendedValue": "256",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "NAMENODE",
      "index": 6
    },
    {
      "name": "dtnode_heapsize",
      "displayName": "DataNode maximum Java heap size",
      "description": "Maximum Java heap size for DataNode (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "DATANODE",
      "index": 2
    },
    {
      "name": "hadoop_heapsize",
      "displayName": "Hadoop maximum Java heap size",
      "description": "Maximum Java heap size for daemons such as Balancer (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "General",
      "index": 1
    },
    {
      "name": "hdfs_log_dir_prefix",
      "displayName": "Hadoop Log Dir Prefix",
      "description": "The parent directory for Hadoop log files.  The HDFS log directory will be ${hadoop_log_dir_prefix} / ${hdfs_user} and the MapReduce log directory will be ${hadoop_log_dir_prefix} / ${mapred_user}.",
      "recommendedValue": "/var/log/hadoop",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "Advanced hadoop-env"
    },
    {
      "name": "hadoop_pid_dir_prefix",
      "displayName": "Hadoop PID Dir Prefix",
      "description": "The parent directory in which the PID files for Hadoop processes will be created.  The HDFS PID directory will be ${hadoop_pid_dir_prefix} / ${hdfs_user} and the MapReduce PID directory will be ${hadoop_pid_dir_prefix} / ${mapred_user}.",
      "recommendedValue": "/var/run/hadoop",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "Advanced hadoop-env"
    },
    {
      "name": "hadoop_root_logger",
      "displayName": "Hadoop Root Logger",
      "description": "Hadoop logging options",
      "recommendedValue": "INFO,RFA",
      "displayType": "string",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HDFS",
      "filename": "hadoop-env.xml",
      "category": "Advanced hadoop-env"
    },

  /**********************************************MAPREDUCE2***************************************/
    {
      "name": "jobhistory_heapsize",
      "displayName": "History Server heap size",
      "description": "History Server heap size",
      "recommendedValue": "900",
      "unit": "MB",
      "isOverridable": true,
      "displayType": "int",
      "isVisible": true,
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-env.xml",
      "category": "HISTORYSERVER",
      "index": 1
    },
    {
      "name": "mapred_log_dir_prefix",
      "displayName": "Mapreduce Log Dir Prefix",
      "description": "",
      "recommendedValue": "/var/log/hadoop-mapreduce",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isReconfigurable": false,
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-env.xml",
      "category": "Advanced mapred-env"
    },
    {
      "name": "mapred_pid_dir_prefix",
      "displayName": "Mapreduce PID Dir Prefix",
      "description": "",
      "recommendedValue": "/var/run/hadoop-mapreduce",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isReconfigurable": false,
      "serviceName": "MAPREDUCE2",
      "filename": "mapred-env.xml",
      "category": "Advanced mapred-env"
    },
  /**********************************************YARN***************************************/
    {
      "name": "yarn_heapsize",
      "displayName": "YARN Java heap size",
      "description": "Max heapsize for all YARN components",
      "recommendedValue": "1024",
      "isOverridable": true,
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "General",
      "index": 0
    },
    {
      "name": "resourcemanager_heapsize",
      "displayName": "ResourceManager Java heap size",
      "description": "Max heapsize for ResourceManager",
      "recommendedValue": "1024",
      "isOverridable": false,
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "RESOURCEMANAGER",
      "index": 1
    },
    {
      "name": "nodemanager_heapsize",
      "displayName": "NodeManager Java heap size",
      "description": "Max heapsize for NodeManager",
      "recommendedValue": "1024",
      "isOverridable": true,
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "NODEMANAGER",
      "index": 0
    },
    {
      "name": "yarn_log_dir_prefix",
      "displayName": "YARN Log Dir Prefix",
      "description": "",
      "recommendedValue": "/var/log/hadoop-yarn",
      "displayType": "directory",
      "isOverridable": false,
      "isReconfigurable": false,
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "Advanced yarn-env"
    },
    {
      "name": "yarn_pid_dir_prefix",
      "displayName": "YARN PID Dir Prefix",
      "description": "",
      "recommendedValue": "/var/run/hadoop-yarn",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isReconfigurable": false,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "Advanced yarn-env"
    },
    {
      "name": "min_user_id",
      "displayName": "Minimum user ID for submitting job",
      "isOverridable": true,
      "displayType": "int",
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "Advanced yarn-env"
    },
    {
      "name": "apptimelineserver_heapsize",
      "displayName": "AppTimelineServer Java heap size",
      "description": "AppTimelineServer Java heap size",
      "recommendedValue": "1024",
      "isOverridable": false,
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "YARN",
      "filename": "yarn-env.xml",
      "category": "APP_TIMELINE_SERVER",
      "index": 1
    },
  /**********************************************HBASE***************************************/
    {
      "name": "hbase_master_heapsize",
      "displayName": "HBase Master Maximum Java heap size",
      "description": "Maximum Java heap size for HBase master (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": true,
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "HBASE_MASTER",
      "index": 1
    },
    {
      "name": "hbase_regionserver_heapsize",
      "displayName": "RegionServers maximum Java heap size",
      "description": "Maximum Java heap size for RegionServers (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "HBASE_REGIONSERVER",
      "index": 1
    },
    {
      "name": "hbase_regionserver_xmn_max",
      "displayName": "RegionServers maximum value for -Xmn",
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "HBASE_REGIONSERVER",
      "index": 6
    },
    {
      "name": "hbase_regionserver_xmn_ratio",
      "displayName": "RegionServers -Xmn in -Xmx ratio",
      "displayType": "float",
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "HBASE_REGIONSERVER",
      "index": 7
    },
    {
      "name": "hbase_log_dir",
      "displayName": "HBase Log Dir",
      "description": "Directory for HBase logs",
      "recommendedValue": "/var/log/hbase",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "Advanced hbase-env"
    },
    {
      "name": "hbase_pid_dir",
      "displayName": "HBase PID Dir",
      "description": "Directory in which the pid files for HBase processes will be created",
      "recommendedValue": "/var/run/hbase",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HBASE",
      "filename": "hbase-env.xml",
      "category": "Advanced hbase-env"
    },
     //***************************************** GLUSTERFS stack********************************************
    {
      "name": "fs.glusterfs.impl",
      "displayName": "GlusterFS fs impl",
      "displayType": "string",
      "filename": "core-site.xml",
      "serviceName": "GLUSTERFS",
      "category": "General"
    },
    {
      "name": "fs.AbstractFileSystem.glusterfs.impl",
      "displayName": "GlusterFS Abstract File System Implementation",
      "displayType": "string",
      "filename": "core-site.xml",
      "serviceName": "GLUSTERFS",
      "category": "General"
    },
  /**********************************************GLUSTERFS***************************************/
    {
      "name": "fs_glusterfs_default_name",
      "displayName": "GlusterFS default fs name 1.x Hadoop",
      "description": "GlusterFS default filesystem name (glusterfs://{MasterFQDN}:9000)",
      "recommendedValue": "glusterfs:///localhost:8020",
      "displayType": "string",
      "isVisible": true,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "glusterfs_defaultFS_name",
      "displayName": "GlusterFS default fs name 2.x Hadoop",
      "description": "GlusterFS default filesystem name (glusterfs:///)",
      "recommendedValue": "glusterfs:///localhost:8020",
      "displayType": "string",
      "isVisible": true,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "hadoop_heapsize",
      "displayName": "Hadoop maximum Java heap size",
      "description": "Maximum Java heap size for daemons such as Balancer (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isVisible": true,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop",
      "index": 1
    },
    {
      "name": "hdfs_log_dir_prefix",
      "displayName": "Hadoop Log Dir Prefix",
      "description": "The parent directory for Hadoop log files.  The HDFS log directory will be ${hadoop_log_dir_prefix} / ${hdfs_user} and the MapReduce log directory will be ${hadoop_log_dir_prefix} / ${mapred_user}.",
      "recommendedValue": "/var/log/hadoop",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "hadoop_pid_dir_prefix",
      "displayName": "Hadoop PID Dir Prefix",
      "description": "The parent directory in which the PID files for Hadoop processes will be created.  The HDFS PID directory will be ${hadoop_pid_dir_prefix} / ${hdfs_user} and the MapReduce PID directory will be ${hadoop_pid_dir_prefix} / ${mapred_user}.",
      "recommendedValue": "/var/run/hadoop",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "namenode_heapsize",
      "displayName": "Name Node Heap Size",
      "description": "Name Node Heap Size, default jvm memory setting",
      "recommendedValue": "1024",
      "isReconfigurable": false,
      "displayType": "int",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "namenode_opt_newsize",
      "displayName": "NameNode new generation size",
      "description": "Default size of Java new generation for NameNode (Java option -XX:NewSize).  This also applies to the Secondary NameNode.",
      "recommendedValue": "200",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "namenode_opt_maxnewsize",
      "displayName": "NameNode maximum new generation size",
      "description": "Maximum size of Java new generation for NameNode (Java option -XX:MaxnewSize).",
      "recommendedValue": "200",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "namenode_opt_permsize",
      "displayName": "NameNode permanent generation size",
      "description": "Default size of Java permanent generation for NameNode (Java option -XX:PermSize).  This also applies to the Secondary NameNode.",
      "recommendedValue": "128",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "namenode_opt_maxpermsize",
      "displayName": "NameNode maximum permanent generation size",
      "description": "Maximum size of Java permanent generation for NameNode (Java option -XX:MaxPermSize).",
      "recommendedValue": "256",
      "displayType": "int",
      "unit": "MB",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "dtnode_heapsize",
      "displayName": "DataNode maximum Java heap size",
      "description": "Maximum Java heap size for DataNode (Java option -Xmx)",
      "recommendedValue": "1024",
      "displayType": "int",
      "unit": "MB",
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
    {
      "name": "glusterfs_user",
      "displayName": "glusterfs user",
      "description": "glusterfs user",
      "recommendedValue": "root",
      "displayType": "string",
      "isVisible": false,
      "serviceName": "GLUSTERFS",
      "filename": "hadoop-env.xml",
      "category": "General Hadoop"
    },
  /**********************************************HIVE***************************************/
    {
      "name": "hive_database",
      "displayName": "Hive Database",
      "value": "",
      "recommendedValue": "New PostgreSQL Database",
      "options": [
        {
          displayName: 'New PostgreSQL Database'
        },
        {
          displayName: 'Existing MySQL Database'
        },
        {
          displayName: 'Existing PostgreSQL Database'
        },
        {
          displayName: 'Existing Oracle Database'
        }
      ],
      "description": "PostgreSQL will be installed by Ambari",
      "displayType": "radio button",
      "isReconfigurable": true,
      "radioName": "hive-database",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "HIVE_METASTORE",
      "index": 2
    },
    {
      "name": "hive_hostname",
      "displayName": "Database Host",
      "description": "Specify the host on which the database is hosted",
      "savedValue": "",
      "recommendedValue": "",
      "isReconfigurable": true,
      "displayType": "host",
      "isOverridable": false,
      "isVisible": false,
      "isObserved": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "HIVE_METASTORE",
      "index": 3
    },
    {
      "name": "hive_metastore_port",
      "displayName": "Hive metastore port",
      "description": "",
      "recommendedValue": "9083",
      "isReconfigurable": false,
      "displayType": "int",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced hive-env"
    },
    {
      "name": "hive_lib",
      "displayName": "Hive library",
      "description": "",
      "recommendedValue": "/usr/lib/hive/lib/",
      "isReconfigurable": false,
      "displayType": "directory",
      "isVisible": false,
      "isRequiredByAgent": false, // Make this to true when we expose the property on ui by making "isVisible": true
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced hive-env"
    },
    {
      "name": "hive_dbroot",
      "displayName": "Hive db directory",
      "description": "",
      "recommendedValue": "/usr/lib/hive/lib",
      "isReconfigurable": false,
      "displayType": "directory",
      "isVisible": false,
      "isRequiredByAgent": false, // Make this to true when we expose the property on ui by making "isVisible": true
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced hive-env"
    },
    {
      "name": "hive_log_dir",
      "displayName": "Hive Log Dir",
      "description": "Directory for Hive log files",
      "recommendedValue": "/var/log/hive",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced hive-env"
    },
    {
      "name": "hive_pid_dir",
      "displayName": "Hive PID Dir",
      "description": "Directory in which the PID files for Hive processes will be created",
      "recommendedValue": "/var/run/hive",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced hive-env"
    },
  /**********************************************HIVE***************************************/
    {
      "name": "hcat_log_dir",
      "displayName": "WebHCat Log Dir",
      "description": "Directory for WebHCat log files",
      "recommendedValue": "/var/log/webhcat",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced webhcat-env"
    },
    {
      "name": "hcat_pid_dir",
      "displayName": "WebHCat PID Dir",
      "description": "Directory in which the PID files for WebHCat processes will be created",
      "recommendedValue": "/var/run/webhcat",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "HIVE",
      "filename": "hive-env.xml",
      "category": "Advanced webhcat-env"
    },
  /**********************************************OOZIE***************************************/
    {
      "name": "oozie_database",
      "displayName": "Oozie Database",
      "value": "",
      "recommendedValue": "New Derby Database",
      "options": [
        {
          displayName: 'New Derby Database'
        },
        {
          displayName: 'Existing MySQL Database'
        },
        {
          displayName: 'Existing PostgreSQL Database'
        },
        {
          displayName: 'Existing Oracle Database'
        }
      ],
      "description": "Current Derby Database will be installed by Ambari",
      "displayType": "radio button",
      "isReconfigurable": true,
      "isOverridable": false,
      "radioName": "oozie-database",
      "isVisible": true,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "OOZIE_SERVER",
      "index": 2
    },
    {
      "name": "oozie_data_dir",
      "displayName": "Oozie Data Dir",
      "description": "Data directory in which the Oozie DB exists",
      "isReconfigurable": true,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isRequired": false,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "OOZIE_SERVER",
      "index": 9
    },
    {
      "name": "oozie_hostname",
      "savedValue": "",
      "recommendedValue": "",
      "displayName": "Database Host",
      "description": "The host where the Oozie database is located",
      "isReconfigurable": true,
      "isOverridable": false,
      "displayType": "host",
      "isVisible": false,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "OOZIE_SERVER",
      "index": 3
    },
    {
      "name": "oozie_log_dir",
      "displayName": "Oozie Log Dir",
      "description": "Directory for oozie logs",
      "recommendedValue": "/var/log/oozie",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "Advanced oozie-env"
    },
    {
      "name": "oozie_pid_dir",
      "displayName": "Oozie PID Dir",
      "description": "Directory in which the pid files for oozie processes will be created",
      "recommendedValue": "/var/run/oozie",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "Advanced oozie-env"
    },
    {
      "name": "oozie_admin_port",
      "displayName": "Oozie Server Admin Port",
      "isReconfigurable": true,
      "displayType": "int",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "OOZIE",
      "filename": "oozie-env.xml",
      "category": "Advanced oozie-env"
    },
  /**********************************************ZOOKEEPER***************************************/
    {
      "name": "zk_data_dir",
      "displayName": "ZooKeeper directory",
      "description": "Data directory for ZooKeeper",
      "isReconfigurable": true,
      "displayType": "directory",
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "ZOOKEEPER_SERVER",
      "index": 1
    },
    {
      "name": "tickTime",
      "displayName": "Length of single Tick",
      "description": "The length of a single tick in milliseconds, which is the basic time unit used by ZooKeeper",
      "recommendedValue": "2000",
      "displayType": "int",
      "unit": "ms",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "ZOOKEEPER_SERVER",
      "index": 2
    },
    {
      "name": "initLimit",
      "displayName": "Ticks to allow for sync at Init",
      "description": "Amount of time, in ticks to allow followers to connect and sync to a leader",
      "recommendedValue": "10",
      "displayType": "int",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "ZOOKEEPER_SERVER",
      "index": 3
    },
    {
      "name": "syncLimit",
      "displayName": "Ticks to allow for sync at Runtime",
      "description": "Amount of time, in ticks to allow followers to connect",
      "recommendedValue": "5",
      "displayType": "int",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "ZOOKEEPER_SERVER",
      "index": 4
    },
    {
      "name": "clientPort",
      "displayName": "Port for running ZK Server",
      "description": "Port for running ZooKeeper server",
      "recommendedValue": "2181",
      "displayType": "int",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "ZOOKEEPER_SERVER",
      "index": 5
    },
    {
      "name": "zk_log_dir",
      "displayName": "ZooKeeper Log Dir",
      "description": "Directory for ZooKeeper log files",
      "recommendedValue": "/var/log/zookeeper",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "Advanced zookeeper-env",
      "index": 0
    },
    {
      "name": "zk_pid_dir",
      "displayName": "ZooKeeper PID Dir",
      "description": "Directory in which the pid files for zookeeper processes will be created",
      "recommendedValue": "/var/run/zookeeper",
      "isReconfigurable": false,
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "ZOOKEEPER",
      "filename": "zookeeper-env.xml",
      "category": "Advanced zookeeper-env",
      "index": 1
    },
  /**********************************************GANGLIA***************************************/
    {
      "name": "ganglia_conf_dir",
      "displayName": "Ganglia conf directory",
      "description": "",
      "recommendedValue": "/etc/ganglia/hdp",
      "isReconfigurable": false,
      "displayType": "directory",
      "isVisible": false,
      "isRequiredByAgent": false,
      "serviceName": "GANGLIA",
      "filename": "ganglia-env.xml",
      "category": "Advanced ganglia-env"
    },
  /**********************************************FALCON***************************************/
    {
      "name": "falcon_port",
      "displayName": "Falcon server port",
      "description": "Port the Falcon Server listens on",
      "recommendedValue": "15000",
      "isReconfigurable": true,
      "displayType": "int",
      "isVisible": true,
      "isRequiredByAgent": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "FALCON_SERVER"
    },
    {
      "name": "falcon_local_dir",
      "displayName": "Falcon data directory",
      "description": "Directory where Falcon data, such as activemq data, is stored",
      "recommendedValue": "/hadoop/falcon",
      "isReconfigurable": true,
      "displayType": "directory",
      "isVisible": true,
      "isRequiredByAgent": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "FALCON_SERVER"
    },
    {
      "name": "falcon_store_uri",
      "displayName": "Falcon store URI",
      "description": "Directory where entity definitions are stored",
      "recommendedValue": "file:///hadoop/falcon/store",
      "isReconfigurable": true,
      "displayType": "string",
      "isVisible": true,
      "isRequiredByAgent": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "FALCON_SERVER"
    },
    {
      "name": "falcon_log_dir",
      "displayName": "Falcon Log Dir",
      "description": "Directory for Falcon logs",
      "recommendedValue": "/var/log/falcon",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isRequiredByAgent": true,
      "isReconfigurable": false,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "Advanced falcon-env"
    },
    {
      "name": "falcon_pid_dir",
      "displayName": "Falcon PID Dir",
      "description": "Directory in which the pid files for Falcon processes will be created",
      "recommendedValue": "/var/run/falcon",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isRequiredByAgent": true,
      "isReconfigurable": false,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "Advanced falcon-env"
    },
    {
      "name": "falcon.embeddedmq",
      "displayName": "falcon.embeddedmq",
      "description": "Whether embeddedmq is enabled or not.",
      "recommendedValue": "true",
      "displayType": "string",
      "isOverridable": false,
      "isVisible": true,
      "isRequiredByAgent": true,
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "Advanced falcon-env"
    },
    {
      "name": "falcon.embeddedmq.data",
      "displayName": "falcon.embeddedmq.data",
      "description": "Directory in which embeddedmq data is stored.",
      "recommendedValue": "/hadoop/falcon/embeddedmq/data",
      "displayType": "directory",
      "isOverridable": false,
      "isVisible": true,
      "isRequiredByAgent": true,
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "Advanced falcon-env"
    },
    {
      "name": "falcon.emeddedmq.port",
      "displayName": "falcon.emeddedmq.port",
      "description": "Port that embeddedmq will listen on.",
      "recommendedValue": "61616",
      "displayType": "string",
      "isOverridable": false,
      "isVisible": true,
      "isRequiredByAgent": true,
      "isReconfigurable": true,
      "serviceName": "FALCON",
      "filename": "falcon-env.xml",
      "category": "Advanced falcon-env"
    },
  /**********************************************STORM***************************************/
    {
      "name": "storm_log_dir",
      "displayName": "storm_log_dir",
      "description": "Storm log directory",
      "recommendedValue": "/var/log/storm",
      "displayType": "directory",
      "isReconfigurable": true,
      "isVisible": true,
      "isRequiredByAgent": true,
      "serviceName": "STORM",
      "filename": "storm-env.xml",
      "category": "Advanced storm-env"
    },
    {
      "name": "storm_pid_dir",
      "displayName": "storm_pid_dir",
      "description": "Storm PID directory",
      "recommendedValue": "/var/run/storm",
      "displayType": "directory",
      "isReconfigurable": true,
      "isVisible": true,
      "isRequiredByAgent": true,
      "serviceName": "STORM",
      "filename": "storm-env.xml",
      "category": "Advanced storm-env"
    },
  /**********************************************MISC***************************************/
    {
      "name": "hbase_conf_dir",
      "displayName": "HBase conf dir",
      "description": "",
      "recommendedValue": "/etc/hbase",
      "isRequired": false,
      "displayType": "directory",
      "isVisible": false,
      "isRequiredByAgent": false,
      "serviceName": "MISC",
      "filename": "hbase-env.xml",
      "category": "General",
      "belongsToService": []
    },
    {
      "name": "proxyuser_group",
      "displayName": "Proxy group for Hive, WebHCat, Oozie and Falcon",
      "description": "",
      "recommendedValue": "users",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "filename": "hadoop-env.xml",
      "serviceName": "MISC",
      "category": "Users and Groups",
      "belongsToService": ["HIVE", "OOZIE", "FALCON"],
      "index": 18
    },
    {
      "name": "ganglia_runtime_dir",
      "displayName": "Ganglia runtime directory",
      "description": "",
      "recommendedValue": "/var/run/ganglia/hdp",
      "isReconfigurable": false,
      "displayType": "directory",
      "isVisible": false,
      "serviceName": "MISC",
      "filename": "ganglia-env.xml",
      "category": "General",
      "belongsToService": []
    },
    {
      "name": "hdfs_user",
      "displayName": "HDFS User",
      "description": "User to run HDFS as",
      "recommendedValue": "hdfs",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "hadoop-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["HDFS"],
      "index": 1
    },
    {
      "name": "mapred_user",
      "displayName": "MapReduce User",
      "description": "User to run MapReduce as",
      "recommendedValue": "mapred",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "mapred-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["MAPREDUCE2"],
      "index": 2
    },
    {
      "name": "yarn_user",
      "displayName": "YARN User",
      "description": "User to run YARN as",
      "recommendedValue": "yarn",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "yarn-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["YARN"],
      "index": 3
    },
    {
      "name": "hbase_user",
      "displayName": "HBase User",
      "description": "User to run HBase as",
      "recommendedValue": "hbase",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "hbase-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["HBASE"],
      "index": 4
    },
    {
      "name": "hive_user",
      "displayName": "Hive User",
      "description": "User to run Hive as",
      "recommendedValue": "hive",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "hive-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["HIVE"],
      "index": 5
    },
    {
      "name": "hcat_user",
      "displayName": "HCat User",
      "description": "User to run HCatalog as",
      "recommendedValue": "hcat",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "hive-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["HIVE"],
      "index": 6
    },
    {
      "name": "webhcat_user",
      "displayName": "WebHCat User",
      "description": "User to run WebHCat as",
      "recommendedValue": "hcat",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "hive-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["HIVE"],
      "index": 7
    },
    {
      "name": "oozie_user",
      "displayName": "Oozie User",
      "description": "User to run Oozie as",
      "recommendedValue": "oozie",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "oozie-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["OOZIE"],
      "index": 8
    },
    {
      "name": "falcon_user",
      "displayName": "Falcon User",
      "description": "User to run Falcon as",
      "recommendedValue": "falcon",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "falcon-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["FALCON"],
      "index": 10
    },
    {
      "name": "storm_user",
      "displayName": "Storm User",
      "description": "User to run Storm as",
      "recommendedValue": "storm",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "storm-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["STORM"],
      "index": 9
    },
    {
      "name": "zk_user",
      "displayName": "ZooKeeper User",
      "description": "User to run ZooKeeper as",
      "recommendedValue": "zookeeper",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "zookeeper-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["ZOOKEEPER"],
      "index": 11
    },
    {
      "name": "flume_user",
      "displayName": "Flume User",
      "description": "User to run Flume as",
      "recommendedValue": "flume",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "flume-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["FLUME"],
      "index": 12
    },
    {
      "name": "gmetad_user",
      "displayName": "Ganglia User",
      "description": "The user used to run Ganglia",
      "recommendedValue": "nobody",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "ganglia-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["GANGLIA"],
      "index": 13
    },
    {
      "name": "gmond_user",
      "displayName": "Gmond User",
      "description": "The user used to run gmond for Ganglia",
      "recommendedValue": "nobody",
      "isReconfigurable": false,
      "displayType": "advanced",
      "isOverridable": false,
      "isVisible": false,
      "serviceName": "MISC",
      "filename": "ganglia-env.xml",
      "category": "Users and Groups",
      "belongsToService": []
    },
    {
      "name": "tez_user",
      "displayName": "Tez User",
      "description": "User to run Tez as",
      "recommendedValue": "tez",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "tez-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["TEZ"],
      "index": 15
    },
    {
      "name": "sqoop_user",
      "displayName": "Sqoop User",
      "isReconfigurable": false,
      "displayType": "user",
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "MISC",
      "filename": "sqoop-env.xml",
      "category": "Users and Groups",
      "belongsToService": ["SQOOP"],
      "index": 17
    },
    {
      "name": "rrdcached_base_dir",
      "displayName": "Ganglia rrdcached base directory",
      "description": "Default directory for saving the rrd files on ganglia server",
      "recommendedValue": "/var/lib/ganglia/rrds",
      "displayType": "directory",
      "isReconfigurable": true,
      "isOverridable": false,
      "isVisible": true,
      "serviceName": "GANGLIA",
      "filename": "ganglia-env.xml",
      "category": "General",
      "belongsToService": ["GANGLIA"]
    },
    {
      "name": "ignore_groupsusers_create",
      "displayName": "Skip group modifications during install",
      "displayType": "checkbox",
      "isReconfigurable": true,
      "isOverridable": false,
      "isVisible": true,
      "filename": "cluster-env.xml",
      "category": "Users and Groups"
    }
  ]
};

});

require.register("data/configs/alert_notification", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Alert Notification Properties
 */
module.exports = [
  {
    "name": "create_notification",
    "displayName": "Create Notification",
    "isOverridable": false,
    "isVisible": false,
    "serviceName": "MISC",
    "category": "Notifications",
    "recommendedValue": "no",
    "filename": "alert_notification"
  },
  {
    "name": "mail.smtp.host",
    "displayName": "SMTP Host",
    "displayType": "host",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "mail.smtp.port",
    "displayName": "SMTP Port",
    "displayType": "int",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "mail.smtp.from",
    "displayName": "FROM Email Address",
    "displayType": "email",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "ambari.dispatch.recipients",
    "displayName": " TO Email Address",
    "displayType": "email",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "smtp_use_auth",
    "displayName": "SMTP server requires authentication",
    "displayType": "checkbox",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "recommendedValue": true,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "ambari.dispatch.credential.username",
    "displayName": "SMTP Username",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-2",
    "filename": "alert_notification"
  },
  {
    "name": "ambari.dispatch.credential.password",
    "displayName": "SMTP Password",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": true,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-2",
    "filename": "alert_notification"
  },
  {
    "name": "mail.smtp.starttls.enable",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": false,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  },
  {
    "name": "mail.smtp.startssl.enable",
    "isRequired": false,
    "isReconfigurable": true,
    "isOverridable": false,
    "isVisible": false,
    "serviceName": "MISC",
    "category": "Notifications",
    "rowStyleClass": "indent-1",
    "filename": "alert_notification"
  }
];

});

require.register("data/configs/services/accumulo_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "accumulo-env.xml",
    "name": "accumulo_instance_name",
    "serviceName": "ACCUMULO"
  },
  {
    "category": "General",
    "filename": "accumulo-env.xml",
    "name": "accumulo_root_password",
    "serviceName": "ACCUMULO"
  },
  {
    "category": "General",
    "filename": "accumulo-site.xml",
    "name": "trace.user",
    "serviceName": "ACCUMULO"
  },
  {
    "category": "General",
    "filename": "accumulo-env.xml",
    "name": "trace_password",
    "serviceName": "ACCUMULO"
  },
  {
    "category": "General",
    "filename": "accumulo-env.xml",
    "name": "instance_secret",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 0,
    "name": "instance.volumes",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 1,
    "name": "instance.zookeeper.host",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 2,
    "name": "instance.zookeeper.timeout",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 3,
    "name": "master.port.client",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 4,
    "name": "tserver.port.client",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 5,
    "name": "monitor.port.client",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 6,
    "name": "monitor.port.log4j",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 7,
    "name": "gc.port.client",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 8,
    "name": "trace.port.client",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 9,
    "name": "tserver.memory.maps.native.enabled",
    "serviceName": "ACCUMULO"
  },
  {
    "filename": "accumulo-site.xml",
    "index": 10,
    "name": "general.classpaths",
    "serviceName": "ACCUMULO"
  }
];
});

require.register("data/configs/services/ambari_infra_solr_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "Advanced infra-solr-client-log4j",
    "filename": "infra-solr-client-log4j.xml",
    "index": 1,
    "name": "infra_solr_client_log_dir",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-client-log4j",
    "filename": "infra-solr-client-log4j.xml",
    "index": 2,
    "name": "infra_client_log_maxfilesize",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-client-log4j",
    "filename": "infra-solr-client-log4j.xml",
    "index": 3,
    "name": "infra_client_log_maxbackupindex",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-client-log4j",
    "filename": "infra-solr-client-log4j.xml",
    "index": 4,
    "name": "content",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 1,
    "name": "infra_solr_log_dir",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 2,
    "name": "infra_solr_pid_dir",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 3,
    "name": "infra_solr_port",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 4,
    "name": "infra_solr_jmx_port",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 5,
    "name": "infra_solr_jmx_enabled",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 6,
    "name": "infra_solr_ssl_enabled",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 7,
    "name": "infra_solr_truststore_location",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 8,
    "name": "infra_solr_truststore_type",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 9,
    "name": "infra_solr_truststore_password",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 10,
    "name": "infra_solr_keystore_location",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 11,
    "name": "infra_solr_keystore_type",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 12,
    "name": "infra_solr_keystore_password",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 13,
    "name": "infra_solr_kerberos_keytab",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 14,
    "name": "infra_solr_kerberos_principal",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 15,
    "name": "infra_solr_web_kerberos_keytab",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 16,
    "name": "infra_solr_web_kerberos_principal",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 17,
    "name": "infra_solr_kerberos_name_rules",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-env",
    "filename": "infra-solr-env.xml",
    "index": 18,
    "name": "content",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-log4j",
    "filename": "infra-solr-log4j.xml",
    "index": 1,
    "name": "infra_log_maxfilesize",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-log4j",
    "filename": "infra-solr-log4j.xml",
    "index": 2,
    "name": "infra_log_maxbackupindex",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-log4j",
    "filename": "infra-solr-log4j.xml",
    "index": 3,
    "name": "content",
    "serviceName": "AMBARI_INFRA_SOLR"
  },
  {
    "category": "Advanced infra-solr-xml",
    "filename": "infra-solr-xml.xml",
    "index": 1,
    "name": "content",
    "serviceName": "AMBARI_INFRA_SOLR"
  }
];

});

require.register("data/configs/services/ambari_metrics_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "ams-site.xml",
    "index": 1,
    "name": "timeline.metrics.service.operation.mode",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-env.xml",
    "index": 2,
    "name": "metrics_collector_log_dir",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-env.xml",
    "index": 3,
    "name": "metrics_collector_pid_dir",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-env.xml",
    "index": 4,
    "name": "metrics_monitor_log_dir",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-env.xml",
    "index": 5,
    "name": "metrics_monitor_pid_dir",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-grafana-env.xml",
    "index": 6,
    "name": "metrics_grafana_username",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "General",
    "filename": "ams-grafana-env.xml",
    "index": 7,
    "name": "metrics_grafana_password",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 17,
    "name": "timeline.metrics.aggregator.checkpoint.dir",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 16,
    "name": "timeline.metrics.cluster.aggregator.hourly.checkpointCutOffMultiplier",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 14,
    "name": "timeline.metrics.cluster.aggregator.hourly.disabled",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 15,
    "name": "timeline.metrics.cluster.aggregator.hourly.interval",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 13,
    "name": "timeline.metrics.cluster.aggregator.minute.checkpointCutOffMultiplier",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 10,
    "name": "timeline.metrics.cluster.aggregator.minute.disabled",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 11,
    "name": "timeline.metrics.cluster.aggregator.minute.interval",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 12,
    "name": "timeline.metrics.cluster.aggregator.minute.timeslice.interval",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 9,
    "name": "timeline.metrics.host.aggregator.hourly.checkpointCutOffMultiplier",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 7,
    "name": "timeline.metrics.host.aggregator.hourly.disabled",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 8,
    "name": "timeline.metrics.host.aggregator.hourly.interval",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 6,
    "name": "timeline.metrics.host.aggregator.minute.checkpointCutOffMultiplier",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 4,
    "name": "timeline.metrics.host.aggregator.minute.disabled",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 5,
    "name": "timeline.metrics.host.aggregator.minute.interval",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 1,
    "name": "timeline.metrics.service.checkpointDelay",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 2,
    "name": "timeline.metrics.service.default.result.limit",
    "serviceName": "AMBARI_METRICS"
  },
  {
    "category": "MetricCollector",
    "filename": "ams-site.xml",
    "index": 3,
    "name": "timeline.metrics.service.resultset.fetchSize",
    "serviceName": "AMBARI_METRICS"
  }
];
});

require.register("data/configs/services/falcon_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-job-submit-instances",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-action-create-inst",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-action-create",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-job-submit-data",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-action-start",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-sla-submit",
    "serviceName": "FALCON"
  },
  {
    "category": "Falcon - Oozie integration",
    "filename": "oozie-site.xml",
    "name": "oozie.service.ELService.ext.functions.coord-sla-create",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconRuntimeSite",
    "filename": "falcon-runtime.properties.xml",
    "name": "*.domain",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconRuntimeSite",
    "filename": "falcon-runtime.properties.xml",
    "name": "*.log.cleanup.frequency.minutes.retention",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconRuntimeSite",
    "filename": "falcon-runtime.properties.xml",
    "name": "*.log.cleanup.frequency.hours.retention",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconRuntimeSite",
    "filename": "falcon-runtime.properties.xml",
    "name": "*.log.cleanup.frequency.days.retention",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconRuntimeSite",
    "filename": "falcon-runtime.properties.xml",
    "name": "*.log.cleanup.frequency.months.retention",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.domain",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.workflow.engine.impl",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.oozie.process.workflow.builder",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.oozie.feed.workflow.builder",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.SchedulableEntityManager.impl",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.ConfigSyncService.impl",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.ProcessInstanceManager.impl",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.catalog.service.impl",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.application.services",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.configstore.listeners",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.broker.impl.class",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.shared.libs",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.config.store.uri",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.system.lib.location",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.retry.recorder.path",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.cleanup.service.frequency",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.broker.url",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.broker.ttlInMins",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.entity.topic",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.max.retry.failure.count",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.internal.queue.size",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.authentication.type",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.type",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.token.validity",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.signature.secret",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.simple.anonymous.allowed",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.kerberos.name.rules",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.http.authentication.blacklisted.users",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.graph.storage.directory",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.graph.serialize.path",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.graph.preserve.history",
    "serviceName": "FALCON"
  },
  {
    "category": "FALCON_SERVER",
    "filename": "falcon-env.xml",
    "name": "falcon_port",
    "serviceName": "FALCON"
  },
  {
    "category": "FALCON_SERVER",
    "filename": "falcon-env.xml",
    "name": "falcon_local_dir",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.graph.blueprints.graph",
    "serviceName": "FALCON"
  },
  {
    "category": "FalconStartupSite",
    "filename": "falcon-startup.properties.xml",
    "name": "*.falcon.graph.storage.backend",
    "serviceName": "FALCON"
  }
];
});

require.register("data/configs/services/flume_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "FLUME_HANDLER",
    "filename": "flume-conf.xml",
    "name": "content",
    "serviceName": "FLUME"
  },
  {
    "filename": "flume-env.xml",
    "index": 0,
    "name": "flume_conf_dir",
    "serviceName": "FLUME"
  },
  {
    "filename": "flume-env.xml",
    "index": 1,
    "name": "flume_log_dir",
    "serviceName": "FLUME"
  }
];
});

require.register("data/configs/services/glusterfs_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "core-site.xml",
    "name": "fs.glusterfs.impl",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General",
    "filename": "core-site.xml",
    "name": "fs.AbstractFileSystem.glusterfs.impl",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "index": 1,
    "name": "hadoop_heapsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "hdfs_log_dir_prefix",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "hadoop_pid_dir_prefix",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "namenode_heapsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "namenode_opt_newsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "namenode_opt_maxnewsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "namenode_opt_permsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "namenode_opt_maxpermsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "dtnode_heapsize",
    "serviceName": "GLUSTERFS"
  },
  {
    "category": "General Hadoop",
    "filename": "hadoop-env.xml",
    "name": "glusterfs_user",
    "serviceName": "GLUSTERFS"
  }
];
});

require.register("data/configs/services/hawq_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 0,
    "name": "hawq_master_address_host",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 1,
    "name": "hawq_standby_address_host",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 2,
    "name": "hawq_master_address_port",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 3,
    "name": "hawq_segment_address_port",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 4,
    "name": "hawq_dfs_url",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 5,
    "name": "hawq_master_directory",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 6,
    "name": "hawq_master_temp_directory",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 7,
    "name": "hawq_segment_directory",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 8,
    "name": "hawq_segment_temp_directory",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-site.xml",
    "index": 9,
    "name": "default_segment_num",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-env.xml",
    "index": 10,
    "name": "hawq_ssh_exkeys",
    "serviceName": "HAWQ"
  },
  {
    "category": "General",
    "filename": "hawq-env.xml",
    "index": 11,
    "name": "hawq_password",
    "serviceName": "HAWQ"
  },
  {
    "category": "AdvancedHawqCheck",
    "filename": "hawq-check-env.xml",
    "name": "content",
    "serviceName": "HAWQ"
  }
];
});

require.register("data/configs/services/hbase_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 0,
    "name": "hbase.hstore.compactionThreshold",
    "serviceName": "HBASE"
  },
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 1,
    "name": "hfile.block.cache.size",
    "serviceName": "HBASE"
  },
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 2,
    "name": "hbase.hregion.max.filesize",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-site.xml",
    "index": 2,
    "name": "hbase.regionserver.handler.count",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-site.xml",
    "index": 3,
    "name": "hbase.hregion.majorcompaction",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-site.xml",
    "index": 4,
    "name": "hbase.hregion.memstore.block.multiplier",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-site.xml",
    "index": 5,
    "name": "hbase.hregion.memstore.flush.size",
    "serviceName": "HBASE"
  },
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 3,
    "name": "hbase.client.scanner.caching",
    "serviceName": "HBASE"
  },
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 4,
    "name": "zookeeper.session.timeout",
    "serviceName": "HBASE"
  },
  {
    "category": "General",
    "filename": "hbase-site.xml",
    "index": 5,
    "name": "hbase.client.keyvalue.maxsize",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_MASTER",
    "filename": "hbase-env.xml",
    "index": 1,
    "name": "hbase_master_heapsize",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-env.xml",
    "index": 1,
    "name": "hbase_regionserver_heapsize",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-env.xml",
    "index": 6,
    "name": "hbase_regionserver_xmn_max",
    "serviceName": "HBASE"
  },
  {
    "category": "HBASE_REGIONSERVER",
    "filename": "hbase-env.xml",
    "index": 7,
    "name": "hbase_regionserver_xmn_ratio",
    "serviceName": "HBASE"
  },
  {
    "filename": "ranger-hbase-plugin-properties.xml",
    "index": 1,
    "name": "ranger-hbase-plugin-enabled",
    "serviceName": "HBASE"
  }
];
});

require.register("data/configs/services/hdfs_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "SECONDARY_NAMENODE",
    "filename": "hdfs-site.xml",
    "index": 1,
    "name": "dfs.namenode.checkpoint.dir",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hdfs-site.xml",
    "index": 3,
    "name": "dfs.namenode.checkpoint.period",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hdfs-site.xml",
    "index": 1,
    "name": "dfs.namenode.name.dir",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hdfs-site.xml",
    "index": 0,
    "name": "dfs.webhdfs.enabled",
    "serviceName": "HDFS"
  },
  {
    "category": "DATANODE",
    "filename": "hdfs-site.xml",
    "index": 3,
    "name": "dfs.datanode.failed.volumes.tolerated",
    "serviceName": "HDFS"
  },
  {
    "category": "DATANODE",
    "filename": "hdfs-site.xml",
    "index": 1,
    "name": "dfs.datanode.data.dir",
    "serviceName": "HDFS"
  },
  {
    "category": "DATANODE",
    "filename": "hdfs-site.xml",
    "name": "dfs.datanode.data.dir.perm",
    "serviceName": "HDFS"
  },
  {
    "category": "NFS_GATEWAY",
    "filename": "hdfs-site.xml",
    "index": 1,
    "name": "nfs.file.dump.dir",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hdfs-site.xml",
    "index": 2,
    "name": "dfs.namenode.accesstime.precision",
    "serviceName": "HDFS"
  },
  {
    "category": "NFS_GATEWAY",
    "filename": "hdfs-site.xml",
    "index": 3,
    "name": "nfs.exports.allowed.hosts",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hdfs-site.xml",
    "name": "dfs.replication",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hdfs-site.xml",
    "index": 2,
    "name": "dfs.datanode.du.reserved",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hadoop-env.xml",
    "index": 2,
    "name": "namenode_heapsize",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hadoop-env.xml",
    "index": 3,
    "name": "namenode_opt_newsize",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hadoop-env.xml",
    "index": 5,
    "name": "namenode_opt_permsize",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hadoop-env.xml",
    "index": 6,
    "name": "namenode_opt_maxpermsize",
    "serviceName": "HDFS"
  },
  {
    "category": "NAMENODE",
    "filename": "hadoop-env.xml",
    "index": 4,
    "name": "namenode_opt_maxnewsize",
    "serviceName": "HDFS"
  },
  {
    "category": "DATANODE",
    "filename": "hadoop-env.xml",
    "index": 2,
    "name": "dtnode_heapsize",
    "serviceName": "HDFS"
  },
  {
    "category": "NFS_GATEWAY",
    "filename": "hadoop-env.xml",
    "index": 1,
    "name": "nfsgateway_heapsize",
    "serviceName": "HDFS"
  },
  {
    "category": "General",
    "filename": "hadoop-env.xml",
    "index": 1,
    "name": "hadoop_heapsize",
    "serviceName": "HDFS"
  },
  {
    "filename": "ranger-hdfs-plugin-properties.xml",
    "index": 1,
    "name": "ranger-hdfs-plugin-enabled",
    "serviceName": "HDFS"
  }
];
});

require.register("data/configs/services/hive_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-site.xml",
    "index": 7,
    "name": "javax.jdo.option.ConnectionDriverName",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-site.xml",
    "index": 5,
    "name": "javax.jdo.option.ConnectionUserName",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-site.xml",
    "index": 6,
    "name": "javax.jdo.option.ConnectionPassword",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-site.xml",
    "index": 8,
    "name": "javax.jdo.option.ConnectionURL",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-site.xml",
    "index": 4,
    "name": "ambari.hive.db.schema.name",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "displayType": "radio button",
    "filename": "hive-env.xml",
    "index": 2,
    "name": "hive_database",
    "options": [
      {
        "displayName": "New MySQL Database",
        "hidden": false
      },
      {
        "displayName": "Existing MySQL / MariaDB Database",
        "hidden": false
      },
      {
        "displayName": "Existing PostgreSQL Database",
        "hidden": false
      },
      {
        "displayName": "Existing Oracle Database",
        "hidden": false
      }
    ],
    "radioName": "hive-database",
    "serviceName": "HIVE"
  },
  {
    "category": "Advanced webhcat-env",
    "filename": "hive-env.xml",
    "name": "hcat_log_dir",
    "serviceName": "HIVE"
  },
  {
    "category": "Advanced webhcat-env",
    "filename": "hive-env.xml",
    "name": "hcat_pid_dir",
    "serviceName": "HIVE"
  },
  {
    "category": "HIVE_METASTORE",
    "filename": "hive-env.xml",
    "name": "hive_database_type",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.cbo.enable",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.optimize.reducededuplication.min.reducer",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.optimize.reducededuplication",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.orc.splits.include.file.footer",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.mapfiles",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.mapredfiles",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.tezfiles",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.smallfiles.avgsize",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.size.per.task",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.merge.orcfile.stripe.level",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.auto.convert.join",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.auto.convert.join.noconditionaltask",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.auto.convert.join.noconditionaltask.size",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.optimize.bucketmapjoin.sortedmerge",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.smb.number.waves",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.map.aggr.hash.percentmemory",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.map.aggr",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.optimize.sort.dynamic.partition",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.stats.autogather",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.stats.fetch.column.stats",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.vectorized.execution.enabled",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.vectorized.execution.reduce.enabled",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.vectorized.groupby.checkinterval",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.vectorized.groupby.flush.percent",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.limit.pushdown.memory.usage",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.optimize.index.filter",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.exec.reducers.bytes.per.reducer",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.smbjoin.cache.rows",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.exec.orc.default.stripe.size",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.fetch.task.conversion",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.fetch.task.conversion.threshold",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.fetch.task.aggr",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.compute.query.using.stats",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.auto.reducer.parallelism",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.max.partition.factor",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.min.partition.factor",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.dynamic.partition.pruning",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.dynamic.partition.pruning.max.event.size",
    "serviceName": "HIVE"
  },
  {
    "category": "Performance",
    "filename": "hive-site.xml",
    "name": "hive.tez.dynamic.partition.pruning.max.data.size",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.pre.hooks",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.post.hooks",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.failure.hooks",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.execution.engine",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.dynamic.partition",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.dynamic.partition.mode",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.max.dynamic.partitions",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.max.dynamic.partitions.pernode",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.max.created.files",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.enforce.bucketing",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "datanucleus.cache.level2.type",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.metastore.uris",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.metastore.warehouse.dir",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.exec.parallel.thread.number",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.security.authorization.enabled",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.security.authorization.manager",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.security.metastore.authenticator.manager",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.security.metastore.authorization.manager",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.authentication",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.enable.doAs",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.tez.default.queues",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.tez.initialize.default.sessions",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.tez.sessions.per.default.queue",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.thrift.http.path",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.thrift.http.port",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.thrift.max.worker.threads",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.thrift.port",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.thrift.sasl.qop",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.transport.mode",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.server2.use.SSL",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.tez.container.size",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.tez.java.opts",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.tez.log.level",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.txn.manager",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.txn.timeout",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.txn.max.open.batch",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.initiator.on",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.worker.threads",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.worker.timeout",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.check.interval",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.delta.num.threshold",
    "serviceName": "HIVE"
  },
  {
    "category": "General",
    "filename": "hive-site.xml",
    "name": "hive.compactor.delta.pct.threshold",
    "serviceName": "HIVE"
  }
];

});

require.register("data/configs/services/kafka_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "KAFKA_BROKER",
    "filename": "kafka-broker.xml",
    "index": 1,
    "name": "log.dirs",
    "serviceName": "KAFKA"
  },
  {
    "category": "KAFKA_BROKER",
    "filename": "kafka-broker.xml",
    "name": "listeners",
    "serviceName": "KAFKA"
  },
  {
    "category": "KAFKA_BROKER",
    "filename": "kafka-broker.xml",
    "index": 0,
    "name": "log.roll.hours",
    "serviceName": "KAFKA"
  },
  {
    "category": "KAFKA_BROKER",
    "filename": "kafka-broker.xml",
    "index": 0,
    "name": "log.retention.hours",
    "serviceName": "KAFKA"
  },
  {
    "category": "KAFKA_BROKER",
    "filename": "kafka-broker.xml",
    "index": 0,
    "name": "zookeeper.connect",
    "serviceName": "KAFKA"
  },
  {
    "filename": "kafka-env.xml",
    "index": 0,
    "name": "kafka_pid_dir",
    "serviceName": "KAFKA"
  },
  {
    "filename": "ranger-kafka-plugin-properties.xml",
    "index": 1,
    "name": "ranger-kafka-plugin-enabled",
    "serviceName": "KAFKA"
  }
];
});

require.register("data/configs/services/kerberos_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "KDC",
    "filename": "kerberos-env.xml",
    "index": 0,
    "name": "kdc_type",
    "serviceName": "KERBEROS"
  },
  {
    "category": "KDC",
    "displayType": "supportTextConnection",
    "filename": "kerberos-env.xml",
    "index": 1,
    "name": "kdc_hosts",
    "serviceName": "KERBEROS"
  },
  {
    "category": "KDC",
    "filename": "kerberos-env.xml",
    "index": 2,
    "name": "realm",
    "serviceName": "KERBEROS"
  },
  {
    "category": "KDC",
    "filename": "kerberos-env.xml",
    "index": 3,
    "name": "ldap_url",
    "serviceName": "KERBEROS"
  },
  {
    "category": "KDC",
    "filename": "kerberos-env.xml",
    "index": 4,
    "name": "container_dn",
    "serviceName": "KERBEROS"
  },
  {
    "category": "KDC",
    "filename": "krb5-conf.xml",
    "index": 5,
    "name": "domains",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 0,
    "name": "manage_identities",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 1,
    "name": "install_packages",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 2,
    "name": "executable_search_paths",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 3,
    "name": "encryption_types",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 4,
    "name": "password_length",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 5,
    "name": "password_min_lowercase_letters",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 6,
    "name": "password_min_uppercase_letters",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 7,
    "name": "password_min_digits",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 8,
    "name": "password_min_punctuation",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 9,
    "name": "password_min_whitespace",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 10,
    "name": "service_check_principal_name",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 11,
    "name": "ad_create_attributes_template",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 12,
    "name": "kdc_create_attributes",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 13,
    "name": "case_insensitive_username_rules",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 14,
    "name": "manage_auth_to_local",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced kerberos-env",
    "filename": "kerberos-env.xml",
    "index": 15,
    "name": "group",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Kadmin",
    "filename": "kerberos-env.xml",
    "index": 0,
    "name": "admin_server_host",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced krb5-conf",
    "dependentConfigPattern": "CATEGORY",
    "filename": "krb5-conf.xml",
    "index": 0,
    "name": "manage_krb5_conf",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced krb5-conf",
    "filename": "krb5-conf.xml",
    "index": 1,
    "name": "conf_dir",
    "serviceName": "KERBEROS"
  },
  {
    "category": "Advanced krb5-conf",
    "filename": "krb5-conf.xml",
    "index": 2,
    "name": "content",
    "serviceName": "KERBEROS"
  }
];
});

require.register("data/configs/services/knox_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "KNOX_GATEWAY",
    "filename": "knox-env.xml",
    "name": "knox_master_secret",
    "serviceName": "KNOX"
  },
  {
    "filename": "ranger-knox-plugin-properties.xml",
    "index": 1,
    "name": "ranger-knox-plugin-enabled",
    "serviceName": "KNOX"
  }
];
});

require.register("data/configs/services/logsearch_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "Advanced logfeeder-properties",
    "filename": "logfeeder-properties.xml",
    "index": 1,
    "name": "logfeeder.config.files",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-properties",
    "filename": "logfeeder-properties.xml",
    "index": 2,
    "name": "logfeeder.checkpoint.folder",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-properties",
    "filename": "logfeeder-properties.xml",
    "index": 3,
    "name": "logfeeder.metrics.collector.hosts",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-properties",
    "filename": "logfeeder-properties.xml",
    "index": 4,
    "name": "logfeeder.solr.config.interval",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-properties",
    "filename": "logfeeder-properties.xml",
    "index": 5,
    "name": "logfeeder.log.filter.enable",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 1,
    "name": "logfeeder_log_dir",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 2,
    "name": "logfeeder_pid_dir",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 3,
    "name": "logfeeder_max_mem",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 4,
    "name": "logfeeder_solr_service_logs_enable",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 5,
    "name": "logfeeder_solr_audit_logs_enable",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 6,
    "name": "logfeeder_truststore_location",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 7,
    "name": "logfeeder_truststore_type",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 8,
    "name": "logfeeder_truststore_password",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 9,
    "name": "logfeeder_keystore_location",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 10,
    "name": "logfeeder_keystore_type",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 11,
    "name": "logfeeder_keystore_password",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 12,
    "name": "logfeeder_kerberos_keytab",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 13,
    "name": "logfeeder_kerberos_principal",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-env",
    "filename": "logfeeder-env.xml",
    "index": 14,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-grok",
    "filename": "logfeeder-grok.xml",
    "index": 1,
    "name": "default_grok_patterns",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-grok",
    "filename": "logfeeder-grok.xml",
    "index": 2,
    "name": "custom_grok_patterns",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-log4j",
    "name": "logfeeder_log_maxfilesize",
    "filename": "logfeeder-log4j.xml",
    "index": 1,
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-log4j",
    "name": "logfeeder_log_maxbackupindex",
    "filename": "logfeeder-log4j.xml",
    "index": 2,
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-log4j",
    "name": "logfeeder_json_log_maxfilesize",
    "filename": "logfeeder-log4j.xml",
    "index": 3,
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-log4j",
    "name": "logfeeder_json_log_maxbackupindex",
    "filename": "logfeeder-log4j.xml",
    "index": 4,
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logfeeder-log4j",
    "name": "content",
    "filename": "logfeeder-log4j.xml",
    "index": 5,
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "LogsearchAdminJson",
    "filename": "logsearch-admin-json.xml",
    "index": 1,
    "name": "logsearch_admin_username",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "LogsearchAdminJson",
    "filename": "logsearch-admin-json.xml",
    "index": 2,
    "name": "logsearch_admin_password",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "LogsearchAdminJson",
    "filename": "logsearch-admin-json.xml",
    "index": 3,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-audit_logs-solrconfig",
    "filename": "logsearch-audit_logs-solrconfig.xml",
    "index": 1,
    "name": "logsearch_audit_logs_max_retention",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-audit_logs-solrconfig",
    "filename": "logsearch-audit_logs-solrconfig.xml",
    "index": 2,
    "name": "logsearch_audit_logs_merge_factor",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-audit_logs-solrconfig",
    "filename": "logsearch-audit_logs_logs-solrconfig.xml",
    "index": 3,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 1,
    "name": "logsearch_log_dir",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 2,
    "name": "logsearch_pid_dir",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 3,
    "name": "logsearch_app_max_memory",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 4,
    "name": "logsearch_debug_enabled",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 5,
    "name": "logsearch_debug_port",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 6,
    "name": "logsearch_truststore_location",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 7,
    "name": "logsearch_truststore_type",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 8,
    "name": "logsearch_truststore_password",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 9,
    "name": "logsearch_keystore_location",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 10,
    "name": "logsearch_keystore_type",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 11,
    "name": "logsearch_keystore_password",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 12,
    "name": "logsearch_kerberos_keytab",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 13,
    "name": "logsearch_kerberos_principal",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-env",
    "filename": "logsearch-env.xml",
    "index": 14,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 1,
    "name": "logsearch_log_maxfilesize",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 2,
    "name": "logsearch_log_maxbackupindex",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 3,
    "name": "logsearch_json_log_maxfilesize",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 4,
    "name": "logsearch_json_log_maxbackupindex",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 5,
    "name": "logsearch_audit_log_maxfilesize",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 6,
    "name": "logsearch_audit_log_maxbackupindex",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 7,
    "name": "logsearch_perf_log_maxfilesize",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 8,
    "name": "logsearch_perf_log_maxbackupindex",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-log4j",
    "filename": "logsearch-log4j.xml",
    "index": 9,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-service_logs-solrconfig",
    "filename": "logsearch-service_logs-solrconfig.xml",
    "index": 1,
    "name": "logsearch_service_logs_max_retention",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-service_logs-solrconfig",
    "filename": "logsearch-service_logs-solrconfig.xml",
    "index": 2,
    "name": "logsearch_service_logs_merge_factor",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-service_logs-solrconfig",
    "filename": "logsearch-service_logs-solrconfig.xml",
    "index": 3,
    "name": "content",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 1,
    "name": "logsearch.http.port",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 2,
    "name": "logsearch.https.port",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 3,
    "name": "logsearch.protocol",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 4,
    "name": "logsearch.solr.collection.service.logs",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 5,
    "name": "logsearch.service.logs.fields",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 6,
    "name": "logsearch.service.logs.split.interval.mins",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 7,
    "name": "logsearch.solr.collection.audit.logs",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 8,
    "name": "logsearch.audit.logs.split.interval.mins",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 9,
    "name": "logsearch.solr.audit.logs.use.ranger",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 10,
    "name": "logsearch.ranger.audit.logs.collection.name",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 11,
    "name": "logsearch.solr.metrics.collector.hosts",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 12,
    "name": "logsearch.auth.external_auth.enable",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 13,
    "name": "logsearch.auth.external_auth.host_url",
    "serviceName": "LOGSEARCH"
  },
  {
    "category": "Advanced logsearch-properties",
    "filename": "logsearch-properties.xml",
    "index": 14,
    "name": "logsearch.auth.external_auth.login_url",
    "serviceName": "LOGSEARCH"
  },
  {
    "name": "logsearch.roles.allowed",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 15
  },
  {
    "name": "logsearch.auth.jwt.audiances",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 16
  },
  {
    "name": "logsearch.auth.jwt.cookie.name",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 17
  },
  {
    "name": "logsearch.auth.jwt.query.param.original_url",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 18
  },
  {
    "name": "logsearch.spnego.kerberos.enabled",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 19
  },
  {
    "name": "logsearch.spnego.kerberos.keytab",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 20
  },
  {
    "name": "logsearch.spnego.kerberos.principal",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 21
  },
  {
    "name": "logsearch.spnego.kerberos.host",
    "serviceName": "LOGSEARCH",
    "filename": "logsearch-properties.xml",
    "category": "Advanced logsearch-properties",
    "index": 22
  }
];

});

require.register("data/configs/services/mapreduce2_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "mapred-site.xml",
    "name": "mapreduce.map.memory.mb",
    "serviceName": "MAPREDUCE2"
  },
  {
    "category": "General",
    "filename": "mapred-site.xml",
    "name": "mapreduce.reduce.memory.mb",
    "serviceName": "MAPREDUCE2"
  },
  {
    "category": "General",
    "filename": "mapred-site.xml",
    "name": "mapreduce.task.io.sort.mb",
    "serviceName": "MAPREDUCE2"
  },
  {
    "category": "HISTORYSERVER",
    "filename": "mapred-env.xml",
    "index": 1,
    "name": "jobhistory_heapsize",
    "serviceName": "MAPREDUCE2"
  }
];
});

require.register("data/configs/services/oozie_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-site.xml",
    "index": 4,
    "name": "oozie.db.schema.name",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-site.xml",
    "index": 5,
    "name": "oozie.service.JPAService.jdbc.username",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-site.xml",
    "index": 6,
    "name": "oozie.service.JPAService.jdbc.password",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-site.xml",
    "index": 7,
    "name": "oozie.service.JPAService.jdbc.driver",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-site.xml",
    "index": 8,
    "name": "oozie.service.JPAService.jdbc.url",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "displayType": "radio button",
    "filename": "oozie-env.xml",
    "index": 2,
    "name": "oozie_database",
    "options": [
      {
        "displayName": "New Derby Database",
        "hidden": false
      },
      {
        "displayName": "Existing MySQL / MariaDB Database",
        "hidden": false
      },
      {
        "displayName": "Existing PostgreSQL Database",
        "hidden": false
      },
      {
        "displayName": "Existing Oracle Database",
        "hidden": false
      }
    ],
    "radioName": "oozie-database",
    "serviceName": "OOZIE"
  },
  {
    "category": "OOZIE_SERVER",
    "filename": "oozie-env.xml",
    "index": 9,
    "name": "oozie_data_dir",
    "serviceName": "OOZIE"
  }
];

});

require.register("data/configs/services/ranger_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "RANGER_ADMIN",
    "filename": "ranger-env.xml",
    "index": 0,
    "name": "ranger_admin_username",
    "serviceName": "RANGER"
  },
  {
    "category": "RANGER_ADMIN",
    "filename": "ranger-env.xml",
    "index": 1,
    "name": "ranger_admin_password",
    "serviceName": "RANGER"
  },
  {
    "category": "RANGER_ADMIN",
    "filename": "ranger-env.xml",
    "index": 2,
    "name": "rangerusersync_user_password",
    "serviceName": "RANGER"
  },
  {
    "category": "RANGER_ADMIN",
    "filename": "ranger-env.xml",
    "index": 3,
    "name": "rangertagsync_user_password",
    "serviceName": "RANGER"
  },
  {
    "category": "RANGER_ADMIN",
    "filename": "ranger-env.xml",
    "index": 4,
    "name": "keyadmin_user_password",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "filename": "ranger-env.xml",
    "name": "ranger_user",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "filename": "ranger-env.xml",
    "name": "ranger_group",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.externalurl",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.service.http.enabled",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "displayType": "radio button",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.authentication.method",
    "options": [
      {
        "displayName": "LDAP",
        "foreignKeys": [
          "ranger.ldap.group.roleattribute",
          "ranger.ldap.url",
          "ranger.ldap.user.dnpattern",
          "ranger.ldap.base.dn",
          "ranger.ldap.bind.dn",
          "ranger.ldap.bind.password",
          "ranger.ldap.referral",
          "ranger.ldap.user.searchfilter",
          "ranger.ldap.group.searchbase",
          "ranger.ldap.group.searchfilter"
        ]
      },
      {
        "displayName": "ACTIVE_DIRECTORY",
        "foreignKeys": [
          "ranger.ldap.ad.domain",
          "ranger.ldap.ad.url",
          "ranger.ldap.ad.base.dn",
          "ranger.ldap.ad.bind.dn",
          "ranger.ldap.ad.bind.password",
          "ranger.ldap.ad.referral",
          "ranger.ldap.ad.user.searchfilter"
        ]
      },
      {
        "displayName": "UNIX",
        "foreignKeys": [
          "ranger.unixauth.service.port",
          "ranger.unixauth.service.hostname",
          "ranger.unixauth.remote.login.enabled"
        ]
      },
      {
        "displayName": "PAM"
      },
      {
        "displayName": "NONE"
      }
    ],
    "radioName": "authentication-method",
    "serviceName": "RANGER"
  },
  {
    "category": "RangerSettings",
    "filename": "admin-properties.xml",
    "name": "policymgr_external_url",
    "serviceName": "RANGER"
  },
  {
    "category": "UnixAuthenticationSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.unixauth.remote.login.enabled",
    "serviceName": "RANGER"
  },
  {
    "category": "UnixAuthenticationSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.unixauth.service.hostname",
    "serviceName": "RANGER"
  },
  {
    "category": "UnixAuthenticationSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.unixauth.service.port",
    "serviceName": "RANGER"
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.url",
    "serviceName": "RANGER",
    "index": 1
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.user.dnpattern",
    "serviceName": "RANGER"
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.group.roleattribute",
    "serviceName": "RANGER"
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.base.dn",
    "serviceName": "RANGER"
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.bind.dn",
    "serviceName": "RANGER",
    "index": 2
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.bind.password",
    "serviceName": "RANGER",
    "index": 3
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.referral",
    "serviceName": "RANGER"
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.user.searchfilter",
    "serviceName": "RANGER",
    "index": 4
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.group.searchbase",
    "serviceName": "RANGER",
    "index": 5
  },
  {
    "category": "LDAPSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.group.searchfilter",
    "serviceName": "RANGER",
    "index": 6
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "index": 2,
    "name": "ranger.sso.providerurl",
    "serviceName": "RANGER"
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "index": 3,
    "name": "ranger.sso.publicKey",
    "serviceName": "RANGER"
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.sso.cookiename",
    "serviceName": "RANGER"
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "index": 1,
    "name": "ranger.sso.enabled",
    "serviceName": "RANGER"
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.sso.query.param.originalurl",
    "serviceName": "RANGER"
  },
  {
    "category": "KnoxSSOSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.sso.browser.useragent",
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.domain",
    "index": 4,
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.url",
    "index": 1,
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.base.dn",
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.bind.dn",
    "index": 2,
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.bind.password",
    "index": 3,
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.referral",
    "serviceName": "RANGER"
  },
  {
    "category": "ADSettings",
    "filename": "ranger-admin-site.xml",
    "name": "ranger.ldap.ad.user.searchfilter",
    "serviceName": "RANGER"
  }
];

});

require.register("data/configs/services/storm_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.root",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.local.dir",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.servers",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.port",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.session.timeout",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.connection.timeout",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.retry.times",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.retry.interval",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.zookeeper.retry.intervalceiling.millis",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.cluster.mode",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.local.mode.zmq",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.transport",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.buffer_size",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.max_retries",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.max_wait_ms",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.min_wait_ms",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.server_worker_threads",
    "serviceName": "STORM"
  },
  {
    "category": "General",
    "filename": "storm-site.xml",
    "name": "storm.messaging.netty.client_worker_threads",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.thrift.port",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.thrift.max_buffer_size",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.childopts",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.task.timeout.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.supervisor.timeout.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.monitor.freq.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.cleanup.inbox.freq.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.inbox.jar.expiration.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.task.launch.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.reassign",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.file.copy.expiration.secs",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.topology.validator",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.slots.ports",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.childopts",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.worker.start.timeout.secs",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.worker.timeout.secs",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.monitor.frequency.secs",
    "serviceName": "STORM"
  },
  {
    "category": "SUPERVISOR",
    "filename": "storm-site.xml",
    "name": "supervisor.heartbeat.frequency.secs",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.port",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.worker.threads",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.queue.size",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.invocations.port",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.request.timeout.secs",
    "serviceName": "STORM"
  },
  {
    "category": "DRPC_SERVER",
    "filename": "storm-site.xml",
    "name": "drpc.childopts",
    "serviceName": "STORM"
  },
  {
    "category": "STORM_UI_SERVER",
    "filename": "storm-site.xml",
    "name": "ui.port",
    "serviceName": "STORM"
  },
  {
    "category": "STORM_UI_SERVER",
    "filename": "storm-site.xml",
    "name": "ui.childopts",
    "serviceName": "STORM"
  },
  {
    "filename": "ranger-storm-plugin-properties.xml",
    "index": 1,
    "name": "ranger-storm-plugin-enabled",
    "serviceName": "STORM"
  },
  {
    "category": "NIMBUS",
    "filename": "storm-site.xml",
    "name": "nimbus.seeds",
    "serviceName": "STORM"
  }
];
});

require.register("data/configs/services/tez_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.am.resource.memory.mb",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.am.log.level",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.am.launch.cmd-opts",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.am.launch.env",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.task.resource.memory.mb",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.task.launch.cmd-opts",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.task.launch.env",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.grouping.split-waves",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.grouping.min-size",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.grouping.max-size",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.runtime.compress",
    "serviceName": "TEZ"
  },
  {
    "category": "General",
    "filename": "tez-site.xml",
    "name": "tez.runtime.compress.codec",
    "serviceName": "TEZ"
  }
];
});

require.register("data/configs/services/yarn_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "RESOURCEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.acl.enable",
    "serviceName": "YARN"
  },
  {
    "category": "RESOURCEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.admin.acl",
    "serviceName": "YARN"
  },
  {
    "category": "RESOURCEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.log-aggregation-enable",
    "serviceName": "YARN"
  },
  {
    "category": "CapacityScheduler",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.scheduler.class",
    "serviceName": "YARN"
  },
  {
    "category": "CapacityScheduler",
    "filename": "yarn-site.xml",
    "name": "yarn.scheduler.minimum-allocation-mb",
    "serviceName": "YARN"
  },
  {
    "category": "CapacityScheduler",
    "filename": "yarn-site.xml",
    "name": "yarn.scheduler.maximum-allocation-mb",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.resource.memory-mb",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.vmem-pmem-ratio",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.cgroups.mount-path",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.group",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.log-dirs",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.local-dirs",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.remote-app-log-dir",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.remote-app-log-dir-suffix",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.aux-services",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.log.retain-second",
    "serviceName": "YARN"
  },
  {
    "category": "General",
    "filename": "yarn-env.xml",
    "index": 0,
    "name": "yarn_heapsize",
    "serviceName": "YARN"
  },
  {
    "category": "ResourceTypes",
    "filename": "resource-types.xml",
    "name": "yarn.resource-types",
    "serviceName": "YARN"
  },
  {
    "category": "ResourceTypes",
    "filename": "resource-types.xml",
    "name": "yarn.resource-types.yarn.io/gpu.maximum-allocation",
    "serviceName": "YARN"
  },
  {
    "category": "RESOURCEMANAGER",
    "filename": "yarn-env.xml",
    "index": 1,
    "name": "resourcemanager_heapsize",
    "serviceName": "YARN"
  },
  {
    "category": "NODEMANAGER",
    "filename": "yarn-env.xml",
    "index": 1,
    "name": "nodemanager_heapsize",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-env.xml",
    "index": 1,
    "name": "apptimelineserver_heapsize",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.leveldb-timeline-store.path",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.leveldb-timeline-store.ttl-interval-ms",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.store-class",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.ttl-enable",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.ttl-ms",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.generic-application-history.store-class",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.webapp.address",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.webapp.https.address",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.address",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.leveldb-state-store.path",
    "serviceName": "YARN"
  },
  {
    "category": "APP_TIMELINE_SERVER",
    "filename": "yarn-site.xml",
    "name": "yarn.timeline-service.state-store-class",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.recovery.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.recovery.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.work-preserving-recovery.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.zk-address",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.connect.retry-interval.ms",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.connect.max-wait.ms",
    "serviceName": "YARN"
  },
  {
    "category": "FaultTolerance",
    "filename": "yarn-site.xml",
    "name": "yarn.resourcemanager.ha.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.container-executor.class",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.resources-handler.class",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.cgroups.hierarchy",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.cgroups.mount",
    "serviceName": "YARN"
  },
  {
    "category": "Isolation",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.linux-container-executor.cgroups.strict-resource-usage",
    "serviceName": "YARN"
  },
  {
    "category": "CapacityScheduler",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.resource.cpu-vcores",
    "serviceName": "YARN"
  },
  {
    "category": "CapacityScheduler",
    "filename": "yarn-site.xml",
    "name": "yarn.nodemanager.resource.percentage-physical-cpu-limit",
    "serviceName": "YARN"
  },
  {
    "filename": "ranger-yarn-plugin-properties.xml",
    "index": 1,
    "name": "ranger-yarn-plugin-enabled",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_module_enabled",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_binary",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_capabilities",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_devices",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_networks",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_ro-mounts",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_rw-mounts",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_allowed_volume-drivers",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "docker_privileged-containers_enabled",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "min_user_id",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "gpu_module_enabled",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "cgroup_root",
    "serviceName": "YARN"
  },
  {
    "category": "ContainerExecutor",
    "filename": "container-executor.xml",
    "name": "yarn_hierarchy",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-site.xml",
    "name": "hadoop.registry.zk.quorum",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-env.xml",
    "name": "registry.dns.bind-port",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-site.xml",
    "name": "hadoop.registry.dns.zone-mask",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-site.xml",
    "name": "hadoop.registry.dns.zone-subnet",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-site.xml",
    "name": "hadoop.registry.dns.enabled",
    "serviceName": "YARN"
  },
  {
    "category": "Registry",
    "filename": "yarn-site.xml",
    "name": "hadoop.registry.dns.domain-name",
    "serviceName": "YARN"
  }
];
});

require.register("data/configs/services/zookeeper_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "category": "ZOOKEEPER_SERVER",
    "filename": "zoo.cfg.xml",
    "index": 1,
    "name": "dataDir",
    "serviceName": "ZOOKEEPER"
  },
  {
    "category": "ZOOKEEPER_SERVER",
    "filename": "zoo.cfg.xml",
    "index": 2,
    "name": "tickTime",
    "serviceName": "ZOOKEEPER"
  },
  {
    "category": "ZOOKEEPER_SERVER",
    "filename": "zoo.cfg.xml",
    "index": 3,
    "name": "initLimit",
    "serviceName": "ZOOKEEPER"
  },
  {
    "category": "ZOOKEEPER_SERVER",
    "filename": "zoo.cfg.xml",
    "index": 4,
    "name": "syncLimit",
    "serviceName": "ZOOKEEPER"
  },
  {
    "category": "ZOOKEEPER_SERVER",
    "filename": "zoo.cfg.xml",
    "index": 5,
    "name": "clientPort",
    "serviceName": "ZOOKEEPER"
  },
  {
    "filename": "zookeeper-env.xml",
    "index": 0,
    "name": "zk_log_dir",
    "serviceName": "ZOOKEEPER"
  },
  {
    "filename": "zookeeper-env.xml",
    "index": 1,
    "name": "zk_pid_dir",
    "serviceName": "ZOOKEEPER"
  }
];
});

require.register("data/configs/site_properties", function(exports, require, module) {
/**
* Licensed to the Apache Software Foundation (ASF) under one
* or more contributor license agreements.  See the NOTICE file
* distributed with this work for additional information
* regarding copyright ownership.  The ASF licenses this file
* to you under the Apache License, Version 2.0 (the
* "License"); you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

/**
 * Returns list of config properties
 * @param {string[]} serviceNames list of service names
 * @return {object[]}
 */
var includeSiteProperties = function() {
  var serviceNames = Array.prototype.slice.call(arguments);
  return serviceNames.reduce(function(acc, serviceName) {
    return acc.concat(require('data/configs/services/' + serviceName + '_properties'));
  }, []);
};

module.exports = {
  configProperties: includeSiteProperties(
    'accumulo',
    'ambari_infra_solr',
    'ambari_metrics',
    'falcon',
    'flume',
    'glusterfs',
    'hawq',
    'hbase',
    'hdfs',
    'hive',
    'kafka',
    'kerberos',
    'knox',
    'logsearch',
    'mapreduce2',
    'oozie',
    'ranger',
    'storm',
    'tez',
    'yarn',
    'zookeeper'
  )
};

});

require.register("data/configs/ui_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    "name": "admin_principal",
    "displayName": "Admin principal",
    "description": "Admin principal used to create principals and export key tabs (e.g. admin/admin@EXAMPLE.COM).",
    "isRequiredByAgent": false,
    "serviceName": "KERBEROS",
    "filename": "krb5-conf.xml",
    "category": "Kadmin",
    "index": 1
  },
  {
    "name": "admin_password",
    "displayName": "Admin password",
    "displayType": "password",
    "isRequiredByAgent": false,
    "serviceName": "KERBEROS",
    "filename": "krb5-conf.xml",
    "category": "Kadmin",
    "index": 2
  },
  {
    "name": "dfs.ha.fencing.methods",
    "displayName": "dfs.ha.fencing.methods",
    "serviceName": "HDFS",
    "filename": "hdfs-site.xml",
    "category": "Advanced hdfs-site",
    "displayType": "multiLine",
    "index": 1
  }
];

});

require.register("data/configs/wizards/federation_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "federationConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({ name: 'HDFS', displayName: 'HDFS'}),
      App.ServiceConfigCategory.create({ name: 'RANGER', displayName: 'Ranger'}),
      App.ServiceConfigCategory.create({ name: 'ACCUMULO', displayName: 'Accumulo'})
    ],
    sites: ['core-site'],
    configs: [
      {
        "name": "dfs.journalnode.edits.dir.{{newNameservice}}",
        "displayName": "dfs.journalnode.edits.dir.{{newNameservice}}",
        "description": "The Directory where the JournalNode will store its local state.",
        "isReconfigurable": true,
        "recommendedValue": "",
        "value": "",
        "displayType": "directory",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.journalnode.edits.dir.{{nameservice1}}",
        "displayName": "dfs.journalnode.edits.dir.{{nameservice1}}",
        "description": "The Directory where the JournalNode will store its local state.",
        "isReconfigurable": false,
        "recommendedValue": "{{journalnode_edits_dir}}",
        "value": "{{journalnode_edits_dir}}",
        "displayType": "directory",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC',
        "firstRun": true
      },
      {
        "name": "dfs.nameservices",
        "displayName": "dfs.nameservices",
        "description": "Comma-separated list of nameservices.",
        "isReconfigurable": false,
        "recommendedValue": "{{nameServicesList}},{{newNameservice}}",
        "value": "{{nameServicesList}},{{newNameservice}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.internal.nameservices",
        "displayName": "dfs.internal.nameservices",
        "description": "Comma-separated list of nameservices.",
        "isReconfigurable": false,
        "recommendedValue": "{{nameServicesList}},{{newNameservice}}",
        "value": "{{nameServicesList}},{{newNameservice}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.ha.namenodes.{{newNameservice}}",
        "displayName": "dfs.ha.namenodes.{{newNameservice}}",
        "description": "The prefix for a given nameservice, contains a comma-separated list of namenodes for a given nameservice.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode1Index}},{{newNameNode2Index}}",
        "value": "{{newNameNode1Index}},{{newNameNode2Index}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.{{newNameservice}}.{{newNameNode1Index}}",
        "displayName": "dfs.namenode.rpc-address.{{newNameservice}}.{{newNameNode1Index}}",
        "description": "RPC address that handles all clients requests for {{newNameNode1Index}}.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode1}}:{{nnRpcPort}}",
        "value": "{{newNameNode1}}:{{nnRpcPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.{{newNameservice}}.{{newNameNode2Index}}",
        "displayName": "dfs.namenode.rpc-address.{{newNameservice}}.{{newNameNode2Index}}",
        "description": "RPC address that handles all clients requests for {{newNameNode2Index}}.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode2}}:{{nnRpcPort}}",
        "value": "{{newNameNode2}}:{{nnRpcPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.{{newNameservice}}.{{newNameNode1Index}}",
        "displayName": "dfs.namenode.http-address.{{newNameservice}}.{{newNameNode1Index}}",
        "description": "The fully-qualified HTTP address for {{newNameNode1Index}} NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode1}}:{{nnHttpPort}}",
        "value": "{{newNameNode1}}:{{nnHttpPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.{{newNameservice}}.{{newNameNode2Index}}",
        "displayName": "dfs.namenode.http-address.{{newNameservice}}.{{newNameNode2Index}}",
        "description": "The fully-qualified HTTP address for {{newNameNode2Index}} NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode2}}:{{nnHttpPort}}",
        "value": "{{newNameNode2}}:{{nnHttpPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.https-address.{{newNameservice}}.{{newNameNode1Index}}",
        "displayName": "dfs.namenode.https-address.{{newNameservice}}.{{newNameNode1Index}}",
        "description": "The fully-qualified HTTPS address for {{newNameNode1Index}} NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode1}}:{{nnHttpsPort}}",
        "value": "{{newNameNode1}}:{{nnHttpsPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.https-address.{{newNameservice}}.{{newNameNode2Index}}",
        "displayName": "dfs.namenode.https-address.{{newNameservice}}.{{newNameNode2Index}}",
        "description": "The fully-qualified HTTPS address for {{newNameNode2Index}} NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode2}}:{{nnHttpsPort}}",
        "value": "{{newNameNode2}}:{{nnHttpsPort}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.client.failover.proxy.provider.{{newNameservice}}",
        "displayName": "dfs.client.failover.proxy.provider.{{newNameservice}}",
        "description": "The Java class that HDFS clients use to contact the Active NameNode.",
        "recommendedValue": "org.apache.hadoop.hdfs.server.namenode.ha.ConfiguredFailoverProxyProvider",
        "isReconfigurable": false,
        "value": "org.apache.hadoop.hdfs.server.namenode.ha.ConfiguredFailoverProxyProvider",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.shared.edits.dir.{{nameservice1}}",
        "displayName": "dfs.namenode.shared.edits.dir.{{nameservice1}}",
        "description": "The URI which identifies the group of JNs where the NameNodes will write/read edits.",
        "isReconfigurable": false,
        "recommendedValue": "qjournal://{{journalnodes}}/{{nameservice1}}",
        "value": "qjournal://{{journalnodes}}/{{nameservice1}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC',
        "firstRun": true
      },
      {
        "name": "dfs.namenode.shared.edits.dir.{{newNameservice}}",
        "displayName": "dfs.namenode.shared.edits.dir.{{newNameservice}}",
        "description": "The URI which identifies the group of JNs where the NameNodes will write/read edits.",
        "isReconfigurable": false,
        "recommendedValue": "qjournal://{{journalnodes}}/{{newNameservice}}",
        "value": "qjournal://{{journalnodes}}/{{newNameservice}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.servicerpc-address.{{nameservice1}}.nn1",
        "displayName": "dfs.namenode.servicerpc-address.{{nameservice1}}.nn1",
        "description": "RPC address for HDFS Services communication.",
        "isReconfigurable": false,
        "recommendedValue": "{{namenode1}}:8021",
        "value": "{{namenode1}}:8021",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC',
        "firstRun": true
      },
      {
        "name": "dfs.namenode.servicerpc-address.{{nameservice1}}.nn2",
        "displayName": "dfs.namenode.servicerpc-address.{{nameservice1}}.nn2",
        "description": "RPC address for HDFS Services communication.",
        "isReconfigurable": false,
        "recommendedValue": "{{namenode2}}:8021",
        "value": "{{namenode2}}:8021",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC',
        "firstRun": true
      },
      {
        "name": "dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode1Index}}",
        "displayName": "dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode1Index}}",
        "description": "RPC address for HDFS Services communication.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode1}}:8021",
        "value": "{{newNameNode1}}:8021",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode2Index}}",
        "displayName": "dfs.namenode.servicerpc-address.{{newNameservice}}.{{newNameNode2Index}}",
        "description": "RPC address for HDFS Services communication.",
        "isReconfigurable": false,
        "recommendedValue": "{{newNameNode2}}:8021",
        "value": "{{newNameNode2}}:8021",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      }
    ]
  }
};

});

require.register("data/configs/wizards/ha_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "haConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({ name: 'HDFS', displayName: 'HDFS'}),
      App.ServiceConfigCategory.create({ name: 'HBASE', displayName: 'HBase'}),
      App.ServiceConfigCategory.create({ name: 'ACCUMULO', displayName: 'Accumulo'}),
      App.ServiceConfigCategory.create({ name: 'AMBARI_METRICS', displayName: 'Ambari Metrics'}),
      App.ServiceConfigCategory.create({ name: 'HAWQ', displayName: 'HAWQ'}), 
      App.ServiceConfigCategory.create({ name: 'RANGER', displayName: 'Ranger'})
    ],
    sites: ['core-site', 'hdfs-site', 'hbase-site', 'accumulo-site', 'ams-hbase-site', 'hawq-site', 'hdfs-client', 'ranger-env', 'ranger-knox-plugin-properties', 'ranger-kms-audit', 'ranger-storm-plugin-properties', 'ranger-hbase-plugin-properties', 'ranger-hdfs-plugin-properties', 'ranger-hive-plugin-properties', 'ranger-kafka-audit', 'ranger-knox-audit', 'ranger-hdfs-audit', 'ranger-hive-audit', 'ranger-atlas-audit', 'ranger-storm-audit', 'ranger-hbase-audit', 'ranger-yarn-audit'],
    configs: [
    /**********************************************HDFS***************************************/
      {
        "name": "dfs.journalnode.edits.dir",
        "displayName": "dfs.journalnode.edits.dir",
        "description": "The Directory where the JournalNode will store its local state.",
        "isReconfigurable": true,
        "recommendedValue": "/hadoop/hdfs/journal",
        "value": "/hadoop/hdfs/journal",
        "displayType": "directory",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "fs.defaultFS",
        "displayName": "fs.defaultFS",
        "description": "The default path prefix used by the Hadoop FS client when none is given.",
        "recommendedValue": "hdfs://haCluster",
        "isReconfigurable": false,
        "value": "hdfs://haCluster",
        "category": "HDFS",
        "filename": "core-site",
        serviceName: 'MISC'
      },
      {
        "name": "ha.zookeeper.quorum",
        "displayName": "ha.zookeeper.quorum",
        "isReconfigurable": false,
        "description": "This lists the host-port pairs running the ZooKeeper service.",
        "recommendedValue": "zk1.example.com:2181,zk2.example.com:2181,zk3.example.com:2181",
        "value": "zk1.example.com:2181,zk2.example.com:2181,zk3.example.com:2181",
        "category": "HDFS",
        "filename": "core-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.nameservices",
        "displayName": "dfs.nameservices",
        "description": "Comma-separated list of nameservices.",
        "isReconfigurable": false,
        "recommendedValue": "haCluster",
        "value": "haCluster",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.internal.nameservices",
        "displayName": "dfs.internal.nameservices",
        "description": "Comma-separated list of nameservices.",
        "isReconfigurable": false,
        "recommendedValue": "haCluster",
        "value": "haCluster",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.ha.namenodes.${dfs.nameservices}",
        "displayName": "dfs.ha.namenodes.${dfs.nameservices}",
        "description": "The prefix for a given nameservice, contains a comma-separated list of namenodes for a given nameservice.",
        "isReconfigurable": false,
        "recommendedValue": "nn1,nn2",
        "value": "nn1,nn2",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.${dfs.nameservices}.nn1",
        "displayName": "dfs.namenode.rpc-address.${dfs.nameservices}.nn1",
        "description": "RPC address that handles all clients requests for nn1.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:8020",
        "value": "0.0.0.0:8020",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.${dfs.nameservices}.nn2",
        "displayName": "dfs.namenode.rpc-address.${dfs.nameservices}.nn2",
        "description": "RPC address that handles all clients requests for nn2.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:8020",
        "value": "0.0.0.0:8020",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.${dfs.nameservices}.nn1",
        "displayName": "dfs.namenode.http-address.${dfs.nameservices}.nn1",
        "description": "The fully-qualified HTTP address for nn1 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50070",
        "value": "0.0.0.0:50070",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.${dfs.nameservices}.nn2",
        "displayName": "dfs.namenode.http-address.${dfs.nameservices}.nn2",
        "description": "The fully-qualified HTTP address for nn2 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50070",
        "value": "0.0.0.0:50070",
        "category": "HDFS",
        "filename": "hdfs-site",
        serviceName: 'MISC'
      },
      {
        "name": "dfs.namenode.https-address.${dfs.nameservices}.nn1",
        "displayName": "dfs.namenode.https-address.${dfs.nameservices}.nn1",
        "description": "The fully-qualified HTTP address for nn1 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50470",
        "value": "0.0.0.0:50470",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.https-address.${dfs.nameservices}.nn2",
        "displayName": "dfs.namenode.https-address.${dfs.nameservices}.nn2",
        "description": "The fully-qualified HTTP address for nn2 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50470",
        "value": "0.0.0.0:50470",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.client.failover.proxy.provider.${dfs.nameservices}",
        "displayName": "dfs.client.failover.proxy.provider.${dfs.nameservices}",
        "description": "The Java class that HDFS clients use to contact the Active NameNode.",
        "recommendedValue": "org.apache.hadoop.hdfs.server.namenode.ha.ConfiguredFailoverProxyProvider",
        "isReconfigurable": false,
        "value": "org.apache.hadoop.hdfs.server.namenode.ha.ConfiguredFailoverProxyProvider",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.shared.edits.dir",
        "displayName": "dfs.namenode.shared.edits.dir",
        "description": " The URI which identifies the group of JNs where the NameNodes will write/read edits.",
        "isReconfigurable": false,
        "recommendedValue": "qjournal://node1.example.com:8485;node2.example.com:8485;node3.example.com:8485/mycluster",
        "value": "qjournal://node1.example.com:8485;node2.example.com:8485;node3.example.com:8485/mycluster",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.ha.fencing.methods",
        "displayName": "dfs.ha.fencing.methods",
        "description": "A list of scripts or Java classes which will be used to fence the Active NameNode during a failover.",
        "isReconfigurable": false,
        "recommendedValue": "shell(/bin/true)",
        "displayType": "multiLine",
        "value": "shell(/bin/true)",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.ha.automatic-failover.enabled",
        "displayName": "dfs.ha.automatic-failover.enabled",
        "description": "Enable Automatic failover.",
        "isReconfigurable": false,
        "recommendedValue": true,
        "value": true,
        "displayType": "checkbox",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.safemode.threshold-pct",
        "displayName": "dfs.namenode.safemode.threshold-pct",
        "description": "Specifies the percentage of blocks that should satisfy\n        the minimal replication requirement defined by dfs.namenode.replication.min.\n        Values less than or equal to 0 mean not to start in safe mode.\n        Values greater than 1 will make safe mode permanent.\n ",
        "isReconfigurable": false,
        "recommendedValue": "0.99f",
        "value": "0.99f",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC'
      },
      {
        "name": "hbase.rootdir",
        "displayName": "hbase.rootdir",
        "description": "The directory shared by region servers and into which HBase persists.",
        "isReconfigurable": false,
        "recommendedValue": "/hadoop/hdfs/journal",
        "value": "/hadoop/hdfs/journal",
        "category": "HBASE",
        "filename": "hbase-site",
        "serviceName": 'MISC'
      },
      {
        "name": "instance.volumes",
        "displayName": "instance.volumes",
        "isReconfigurable": false,
        "recommendedValue": "/hadoop/hdfs/journal",
        "value": "/hadoop/hdfs/journal",
        "category": "ACCUMULO",
        "filename": "accumulo-site",
        "serviceName": 'MISC'
      },
      {
        "name": "instance.volumes.replacements",
        "displayName": "instance.volumes.replacements",
        "isReconfigurable": false,
        "recommendedValue": "/hadoop/hdfs/journal",
        "value": "/hadoop/hdfs/journal",
        "category": "ACCUMULO",
        "filename": "accumulo-site",
        "serviceName": 'MISC'
      },
      {
        "name": "hbase.rootdir",
        "displayName": "hbase.rootdir",
        "description": "Ambari Metrics service uses HBase as default storage backend. Set the rootdir for HBase to either local filesystem path if using Ambari Metrics in embedded mode or to a HDFS dir, example: hdfs://namenode.example.org:8020/amshbase.",
        "isReconfigurable": false,
        "recommendedValue": "file:///var/lib/ambari-metrics-collector/hbase",
        "value": "file:///var/lib/ambari-metrics-collector/hbase",
        "category": "AMBARI_METRICS",
        "isVisible": false,
        "filename": "ams-hbase-site",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "RANGER",
        "filename": "ranger-env",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HDFS",
        "filename": "ranger-hdfs-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HDFS",
        "isVisible": false,
        "filename": "ranger-hdfs-plugin-properties",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "KAFKA",
        "isVisible": false,
        "filename": "ranger-kafka-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "KNOX",
        "isVisible": false,
        "filename": "ranger-knox-plugin-properties",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "RANGER_KMS",
        "isVisible": false,
        "filename": "ranger-kms-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "STORM",
        "isVisible": false,
        "filename": "ranger-storm-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "STORM",
        "isVisible": false,
        "filename": "ranger-storm-plugin-properties",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HBASE",
        "isVisible": false,
        "filename": "ranger-hbase-plugin-properties",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HIVE",
        "isVisible": false,
        "filename": "ranger-hive-plugin-properties",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HBASE",
        "isVisible": false,
        "filename": "ranger-hbase-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "KNOX",
        "isVisible": false,
        "filename": "ranger-knox-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "HIVE",
        "isVisible": false,
        "filename": "ranger-hive-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "YARN",
        "isVisible": false,
        "filename": "ranger-yarn-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "ATLAS",
        "isVisible": false,
        "filename": "ranger-atlas-audit",
        "serviceName": 'MISC'
      },
      {
        "name": "xasecure.audit.destination.hdfs.dir",
        "displayName": "Destination HDFS Directory",
        "description": "HDFS folder to write audit to, make sure all service user has required permissions. This property is overridable at service level.",
        "isReconfigurable": false,
        "recommendedValue": "hdfs://haCluster",
        "value": "hdfs://haCluster",
        "category": "RANGER_KMS",
        "isVisible": false,
        "filename": "ranger-kms-audit",
        "serviceName": 'MISC'
      },
    /**********************************************HAWQ***************************************/
      {
        "name": "hawq_dfs_url",
        "displayName": "hawq_dfs_url",
        "description": "URL for Accessing HDFS",
        "isReconfigurable": false,
        "recommendedValue": "haCluster/hawq_data",
        "value": "haCluster/hawq_data",
        "category": "HAWQ",
        "filename": "hawq-site",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.nameservices",
        "displayName": "dfs.nameservices",
        "description": "Comma-separated list of nameservices.",
        "isReconfigurable": false,
        "recommendedValue": "haCluster",
        "value": "haCluster",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.ha.namenodes.${dfs.nameservices}",
        "displayName": "dfs.ha.namenodes.${dfs.nameservices}",
        "description": "The prefix for a given nameservice, contains a comma-separated list of namenodes for a given nameservice.",
        "isReconfigurable": false,
        "recommendedValue": "nn1,nn2",
        "value": "nn1,nn2",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.${dfs.nameservices}.nn1",
        "displayName": "dfs.namenode.rpc-address.${dfs.nameservices}.nn1",
        "description": "RPC address that handles all clients requests for nn1.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:8020",
        "value": "0.0.0.0:8020",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.rpc-address.${dfs.nameservices}.nn2",
        "displayName": "dfs.namenode.rpc-address.${dfs.nameservices}.nn2",
        "description": "RPC address that handles all clients requests for nn2.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:8020",
        "value": "0.0.0.0:8020",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.${dfs.nameservices}.nn1",
        "displayName": "dfs.namenode.http-address.${dfs.nameservices}.nn1",
        "description": "The fully-qualified HTTP address for nn1 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50070",
        "value": "0.0.0.0:50070",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      },
      {
        "name": "dfs.namenode.http-address.${dfs.nameservices}.nn2",
        "displayName": "dfs.namenode.http-address.${dfs.nameservices}.nn2",
        "description": "The fully-qualified HTTP address for nn2 NameNode.",
        "isReconfigurable": false,
        "recommendedValue": "0.0.0.0:50070",
        "value": "0.0.0.0:50070",
        "category": "HAWQ",
        "filename": "hdfs-client",
        "serviceName": 'MISC'
      }
    ]
  }
};

});

require.register("data/configs/wizards/hawq_activate_standby_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "haConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({name: 'HAWQ', displayName: 'HAWQ'}),
    ],
    sites: ['hawq-site'],
    configs: [
    /**********************************************HAWQ***************************************/
      {
        "name": "hawq_master_address_host",
        "displayName": "HAWQ Master Host",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "HAWQ",
        "filename": "hawq-site",
        serviceName: 'MISC'
      }
    ]
  }
};

});

require.register("data/configs/wizards/hawq_ha_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "haConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({name: 'HAWQ', displayName: 'HAWQ'}),
    ],
    sites: ['hawq-site'],
    configs: [
    /**********************************************HAWQ***************************************/
      {
        "name": "hawq_standby_address_host",
        "displayName": "HAWQ Standby Master",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "HAWQ",
        "filename": "hawq-site",
        serviceName: 'MISC'
      },
    ]
  }
};

});

require.register("data/configs/wizards/kerberos_descriptor_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var properties = [
  {
    "name": "keytab_dir",
    "index": 3
  },
  {
    "name": "realm",
    "index": 4
  },
  {
    "name": "additional_realms",
    "index": 5,
    "placeholderText": "(" + Em.I18n.t("common.optional") + ")"
  },
  {
    "name": "principal_suffix",
    "index": 6,
    "placeholderText": "(" + Em.I18n.t("common.optional") + ")"
  }
];

module.exports = properties;

});

require.register("data/configs/wizards/kerberos_identities", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = {
  configProperties: [
    {
      "name": "smokeuser_principal_name",
      "displayName": "Smoke user principal",
      "category": "Ambari Principals",
      "filename": "cluster-env.xml",
      "index": 1
    },
    {
      "name": "smokeuser_keytab",
      "displayName": "Smoke user keytab",
      "category": "Ambari Principals",
      "filename": "cluster-env.xml",
      "index": 2
    },
    {
      "name": "ambari-server_keytab",
      "displayName": "Ambari Keytab",
      "category": "Ambari Principals",
      "filename": "cluster-env.xml",
      "index": 3
    },
    {
      "name": "hdfs_principal_name",
      "displayName": "HDFS user principal",
      "category": "Ambari Principals",
      "filename": "hadoop-env.xml",
      "index": 4
    },
    {
      "name": "hdfs_user_keytab",
      "displayName": "HDFS user keytab",
      "category": "Ambari Principals",
      "filename": "hadoop-env.xml",
      "index": 5
    },
    {
      "name": "hbase_principal_name",
      "displayName": "HBase user principal",
      "category": "Ambari Principals",
      "filename": "hbase-env.xml",
      "index": 6
    },
    {
      "name": "hbase_user_keytab",
      "displayName": "HBase user keytab",
      "category": "Ambari Principals",
      "filename": "hbase-env.xml",
      "index": 7
    },
    {
      "name": "accumulo_principal_name",
      "displayName": "Accumulo user principal",
      "category": "Ambari Principals",
      "filename": "accumulo-env.xml",
      "index": 8
    },
    {
      "name": "accumulo_user_keytab",
      "displayName": "Accumulo user keytab",
      "category": "Ambari Principals",
      "filename": "accumulo-env.xml",
      "index": 9
    },
    {
      "name": "spark.history.kerberos.principal",
      "displayName": "Spark user principal",
      "category": "Ambari Principals",
      "filename": "spark-defaults.xml",
      "index": 10
    },
    {
      "name": "spark.history.kerberos.keytab",
      "displayName": "Spark user keytab",
      "category": "Ambari Principals",
      "filename": "spark-defaults.xml",
      "index": 11
    },
    {
      "name": "spark.history.kerberos.principal",
      "displayName": "Spark2 user principal",
      "category": "Ambari Principals",
      "filename": "spark2-defaults.xml",
      "index": 12
    },
    {
      "name": "spark.history.kerberos.keytab",
      "displayName": "Spark2 user keytab",
      "category": "Ambari Principals",
      "filename": "spark2-defaults.xml",
      "index": 13
    },
    {
      "name": "storm_principal_name",
      "displayName": "Storm user principal",
      "category": "Ambari Principals",
      "filename": "storm-env.xml",
      "index": 14
    },
    {
      "name": "storm_keytab",
      "displayName": "Storm user keytab",
      "category": "Ambari Principals",
      "filename": "storm-env.xml",
      "index": 15
    },
    {
      "name": "druid.hadoop.security.kerberos.principal",
      "displayName": "Druid user principal",
      "category": "Ambari Principals",
      "filename": "druid-common.xml",
      "index": 16
    },
    {
      "name": "druid.hadoop.security.kerberos.keytab",
      "displayName": "Druid user keytab",
      "category": "Ambari Principals",
      "filename": "druid-common.xml",
      "index": 17
    },
    {
      "name": "KERBEROS_PRINCIPAL",
      "displayName": "Superset user principal",
      "category": "Ambari Principals",
      "filename": "superset.xml",
      "index": 18
    },
    {
      "name": "KERBEROS_KEYTAB",
      "displayName": "Superset user keytab",
      "category": "Ambari Principals",
      "filename": "superset.xml",
      "index": 19
    }
  ]
};

});

require.register("data/configs/wizards/move_journal_node_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "moveJNConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({ name: 'HDFS', displayName: 'HDFS'})
    ],
    sites: ['hdfs-site'],
    configs: [
      {
        "name": "dfs.namenode.shared.edits.dir",
        "displayName": "dfs.namenode.shared.edits.dir",
        "description": " The URI which identifies the group of JNs where the NameNodes will write/read edits.",
        "isReconfigurable": false,
        "recommendedValue": "qjournal://node1.example.com:8485;node2.example.com:8485;node3.example.com:8485/mycluster",
        "value": "qjournal://node1.example.com:8485;node2.example.com:8485;node3.example.com:8485/mycluster",
        "category": "HDFS",
        "filename": "hdfs-site",
        "serviceName": 'MISC',
        "presentForNonFederatedHDFS": true,
        "presentForFederatedHDFS": false,
        "dependsOnNameServiceId": false
      },
      {
        "name": "dfs.namenode.shared.edits.dir.{{namespaceId}}",
        "displayName": "dfs.namenode.shared.edits.dir.{{namespaceId}}",
        "description": "The URI which identifies the group of JNs where the NameNodes will write/read edits.",
        "isReconfigurable": false,
        "recommendedValue": "qjournal://{{journalnodes}}/{{namespaceId}}",
        "value": "qjournal://{{journalnodes}}/{{namespaceId}}",
        "category": "HDFS",
        "filename": "hdfs-site",
        "presentForNonFederatedHDFS": false,
        "presentForFederatedHDFS": true,
        "dependsOnNameServiceId": true
      }
    ]
  }
};

});

require.register("data/configs/wizards/rm_ha_properties", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports =
{
  "haConfig": {
    serviceName: 'MISC',
    displayName: 'MISC',
    configCategories: [
      App.ServiceConfigCategory.create({ name: 'YARN', displayName: 'YARN'}),
      App.ServiceConfigCategory.create({ name: 'HAWQ', displayName: 'HAWQ'}),
      App.ServiceConfigCategory.create({ name: 'HDFS', displayName: 'HDFS'})
    ],
    sites: ['yarn-site', 'hawq-site', 'core-site'],
    configs: [
    /**********************************************HDFS***************************************/
      {
        "name": "yarn.resourcemanager.ha.enabled",
        "displayName": "yarn.resourcemanager.ha.enabled",
        "isReconfigurable": false,
        "recommendedValue": true,
        "isOverridable": false,
        "value": true,
        "displayType": "checkbox",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.ha.rm-ids",
        "displayName": "yarn.resourcemanager.ha.rm-ids",
        "isReconfigurable": false,
        "recommendedValue": "rm1,rm2",
        "isOverridable": false,
        "value": "rm1,rm2",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.hostname.rm1",
        "displayName": "yarn.resourcemanager.hostname.rm1",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.resource-tracker.address.rm1",
        "displayName": "yarn.resourcemanager.resource-tracker.address.rm1",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.resource-tracker.address.rm2",
        "displayName": "yarn.resourcemanager.resource-tracker.address.rm2",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.webapp.address.rm1",
        "displayName": "yarn.resourcemanager.webapp.address.rm1",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.webapp.address.rm2",
        "displayName": "yarn.resourcemanager.webapp.address.rm2",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.webapp.https.address.rm1",
        "displayName": "yarn.resourcemanager.webapp.https.address.rm1",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.webapp.https.address.rm2",
        "displayName": "yarn.resourcemanager.webapp.https.address.rm2",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },

      {
        "name": "yarn.resourcemanager.hostname.rm2",
        "displayName": "yarn.resourcemanager.hostname.rm2",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.recovery.enabled",
        "displayName": "yarn.resourcemanager.recovery.enabled",
        "isReconfigurable": false,
        "recommendedValue": true,
        "isOverridable": false,
        "value": true,
        "displayType": "checkbox",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.store.class",
        "displayName": "yarn.resourcemanager.store.class",
        "isReconfigurable": false,
        "recommendedValue": "org.apache.hadoop.yarn.server.resourcemanager.recovery.ZKRMStateStore",
        "isOverridable": false,
        "value": "org.apache.hadoop.yarn.server.resourcemanager.recovery.ZKRMStateStore",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.zk-address",
        "displayName": "yarn.resourcemanager.zk-address",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.cluster-id",
        "displayName": "yarn.resourcemanager.cluster-id",
        "isReconfigurable": false,
        "recommendedValue": "yarn-cluster",
        "isOverridable": false,
        "value": "yarn-cluster",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.ha.automatic-failover.zk-base-path",
        "displayName": "yarn.resourcemanager.ha.automatic-failover.zk-base-path",
        "isReconfigurable": false,
        "recommendedValue": "/yarn-leader-election",
        "isOverridable": false,
        "value": "/yarn-leader-election",
        "category": "YARN",
        "filename": "yarn-site",
        serviceName: 'MISC'
      },
    /**********************************************HAWQ***************************************/
      {
        "name": "yarn.resourcemanager.ha",
        "displayName": "yarn.resourcemanager.ha",
        "description": "Comma separated yarn resourcemanager host addresses with port",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "HAWQ",
        "filename": "yarn-client",
        serviceName: 'MISC'
      },
      {
        "name": "yarn.resourcemanager.scheduler.ha",
        "displayName": "yarn.resourcemanager.scheduler.ha",
        "description": "Comma separated yarn resourcemanager scheduler addresses with port",
        "isReconfigurable": false,
        "recommendedValue": "",
        "isOverridable": false,
        "value": "",
        "category": "HAWQ",
        "filename": "yarn-client",
        serviceName: 'MISC'
      }
    ]
  }
};

});

require.register("data/configs/wizards/secure_mapping", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
// All of the "name" properties have to coincide with how they will appear in the *-site.xml file
// The "template" properties can come from the config properties in site_properties.js or secure_properties.js .
var props = [
  {
    "name": "hadoop.security.authentication",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "nonSecureValue": "simple",
    "filename": "core-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "hadoop.security.authorization",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "core-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "hadoop.rpc.protection",
    "templateName": [],
    "foreignKey": null,
    "value": "authentication,privacy",
    "filename": "core-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "hadoop.security.auth_to_local",
    "templateName": ["resourcemanager_primary_name", "kerberos_domain", "yarn_user", "nodemanager_primary_name", "namenode_primary_name", "hdfs_user", "datanode_primary_name", "hbase_master_primary_name", "hbase_user","hbase_regionserver_primary_name","oozie_primary_name","oozie_user","jobhistory_primary_name","mapred_user","journalnode_principal_name","falcon_primary_name","falcon_user"],
    "foreignKey": null,
    "value": "RULE:[2:$1@$0](<templateName[0]>@.*<templateName[1]>)s/.*/<templateName[2]>/\nRULE:[2:$1@$0](<templateName[3]>@.*<templateName[1]>)s/.*/<templateName[2]>/\nRULE:[2:$1@$0](<templateName[4]>@.*<templateName[1]>)s/.*/<templateName[5]>/\nRULE:[2:$1@$0](<templateName[6]>@.*<templateName[1]>)s/.*/<templateName[5]>/\nRULE:[2:$1@$0](<templateName[7]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nRULE:[2:$1@$0](<templateName[9]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nRULE:[2:$1@$0](<templateName[10]>@.*<templateName[1]>)s/.*/<templateName[11]>/\nRULE:[2:$1@$0](<templateName[12]>@.*<templateName[1]>)s/.*/<templateName[13]>/\nRULE:[2:$1@$0](<templateName[14]>@.*<templateName[1]>)s/.*/<templateName[5]>/\nRULE:[2:$1@$0](<templateName[15]>@.*<templateName[1]>)s/.*/<templateName[16]>/\nDEFAULT",
    "filename": "core-site.xml",
    "serviceName": "HDFS",
    "dependedServiceName": [{name: "HBASE", replace: "\nRULE:[2:$1@$0](<templateName[7]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nRULE:[2:$1@$0](<templateName[9]>@.*<templateName[1]>)s/.*/<templateName[8]>/"},{name: "OOZIE",replace: "\nRULE:[2:$1@$0](<templateName[10]>@.*<templateName[1]>)s/.*/<templateName[11]>/"},{name: "MAPREDUCE2",replace: "\nRULE:[2:$1@$0](<templateName[12]>@.*<templateName[1]>)s/.*/<templateName[13]>/"}]
  },
  {
    "name": "dfs.namenode.kerberos.principal",
    "templateName": ["namenode_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.namenode.keytab.file",
    "templateName": ["namenode_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.secondary.namenode.kerberos.principal",
    "templateName": ["snamenode_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.secondary.namenode.keytab.file",
    "templateName": ["snamenode_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.web.authentication.kerberos.principal",
    "templateName": ["hadoop_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.web.authentication.kerberos.keytab",
    "templateName": ["hadoop_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.datanode.kerberos.principal",
    "templateName": ["datanode_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.datanode.keytab.file",
    "templateName": ["datanode_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.namenode.kerberos.internal.spnego.principal",
    "templateName": [],
    "foreignKey": null,
    "value": "${dfs.web.authentication.kerberos.principal}",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.secondary.namenode.kerberos.internal.spnego.principal",
    "templateName": [],
    "foreignKey": null,
    "value": "${dfs.web.authentication.kerberos.principal}",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.journalnode.kerberos.principal",
    "templateName": ["journalnode_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.journalnode.kerberos.internal.spnego.principal",
    "templateName": ["hadoop_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.journalnode.keytab.file",
    "templateName": ["journalnode_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.datanode.address",
    "templateName": ["dfs_datanode_address"],
    "foreignKey": null,
    "value": "0.0.0.0:<templateName[0]>",
    "nonSecureValue": "0.0.0.0:50010",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.datanode.http.address",
    "templateName": ["dfs_datanode_http_address"],
    "foreignKey": null,
    "value": "0.0.0.0:<templateName[0]>",
    "nonSecureValue": "0.0.0.0:50075",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "dfs.data.transfer.protection",
    "templateName": [],
    "foreignKey": null,
    "value": "authentication,privacy",
    "filename": "hdfs-site.xml",
    "serviceName": "HDFS"
  },
  {
    "name": "mapreduce.jobhistory.principal",
    "templateName": ["jobhistory_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "mapred-site.xml",
    "serviceName": "MAPREDUCE2"
  },
  {
    "name": "mapreduce.jobhistory.keytab",
    "templateName": ["jobhistory_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "mapred-site.xml",
    "serviceName": "MAPREDUCE2"
  },
  {
    "name": "mapreduce.jobhistory.webapp.spnego-principal",
    "templateName": ["jobhistory_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "mapred-site.xml",
    "serviceName": "MAPREDUCE2"
  },
  {
    "name": "mapreduce.jobhistory.webapp.spnego-keytab-file",
    "templateName": ["jobhistory_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "mapred-site.xml",
    "serviceName": "MAPREDUCE2"
  },
  {
    "name": "yarn.timeline-service.enabled",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.acl.enable",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  // YARN Timeline Service
  // These "http-authentication" properties are supported in HDP Champlain
  {
    "name": "yarn.timeline-service.principal",
    "templateName": ["apptimelineserver_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.timeline-service.keytab",
    "templateName": ["apptimelineserver_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.timeline-service.http-authentication.type",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.timeline-service.http-authentication.kerberos.principal",
    "templateName": ["apptimelineserver_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.timeline-service.http-authentication.kerberos.keytab",
    "templateName": ["apptimelineserver_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  // YARN Resource Manager
  {
    "name": "yarn.resourcemanager.principal",
    "templateName": ["resourcemanager_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.resourcemanager.keytab",
    "templateName": ["resourcemanager_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.nodemanager.principal",
    "templateName": ["nodemanager_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.nodemanager.keytab",
    "templateName": ["nodemanager_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.resourcemanager.webapp.spnego-principal",
    "templateName": ["resourcemanager_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.resourcemanager.webapp.spnego-keytab-file",
    "templateName": ["resourcemanager_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.nodemanager.webapp.spnego-principal",
    "templateName": ["nodemanager_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.nodemanager.webapp.spnego-keytab-file",
    "templateName": ["nodemanager_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "yarn.nodemanager.container-executor.class",
    "templateName": ["yarn_nodemanager_container-executor_class"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "nonSecureValue": "org.apache.hadoop.yarn.server.nodemanager.DefaultContainerExecutor",
    "filename": "yarn-site.xml",
    "serviceName": "YARN"
  },
  {
    "name": "hbase.master.kerberos.principal",
    "templateName": ["hbase_master_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hbase.master.keytab.file",
    "templateName": ["hbase_master_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hbase.regionserver.kerberos.principal",
    "templateName": ["hbase_regionserver_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hbase.regionserver.keytab.file",
    "templateName": ["hbase_regionserver_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hive.metastore.sasl.enabled",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.security.authorization.enabled",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.server2.authentication",
    "templateName": [],
    "foreignKey": null,
    "value": "KERBEROS",
    "nonSecureValue": "NONE",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.metastore.kerberos.principal",
    "templateName": ["hive_metastore_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.metastore.kerberos.keytab.file",
    "templateName": ["hive_metastore_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.server2.authentication.kerberos.principal",
    "templateName": ["hive_metastore_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.server2.authentication.kerberos.keytab",
    "templateName": ["hive_metastore_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.server2.authentication.spnego.principal",
    "templateName": ["hive_metastore_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hive.server2.authentication.spnego.keytab",
    "templateName": ["hive_metastore_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "hive-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "oozie.service.AuthorizationService.authorization.enabled",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.service.HadoopAccessorService.kerberos.enabled",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "local.realm",
    "templateName": ["kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.service.HadoopAccessorService.keytab.file",
    "templateName": ["oozie_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.service.HadoopAccessorService.kerberos.principal",
    "templateName": ["oozie_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.authentication.type",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "nonSecureValue": "simple",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.authentication.kerberos.principal",
    "templateName": ["oozie_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.authentication.kerberos.keytab",
    "templateName": ["oozie_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "oozie.authentication.kerberos.name.rules",
    "templateName": ["resourcemanager_primary_name", "kerberos_domain", "yarn_user", "nodemanager_primary_name", "namenode_primary_name", "hdfs_user", "datanode_primary_name", "hbase_master_primary_name", "hbase_user","hbase_regionserver_primary_name"],
    "foreignKey": null,
    "value": "RULE:[2:$1@$0](<templateName[0]>@.*<templateName[1]>)s/.*/<templateName[2]>/\nRULE:[2:$1@$0](<templateName[3]>@.*<templateName[1]>)s/.*/<templateName[2]>/\nRULE:[2:$1@$0](<templateName[4]>@.*<templateName[1]>)s/.*/<templateName[5]>/\nRULE:[2:$1@$0](<templateName[6]>@.*<templateName[1]>)s/.*/<templateName[5]>/\nRULE:[2:$1@$0](<templateName[7]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nRULE:[2:$1@$0](<templateName[9]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nDEFAULT",
    "filename": "oozie-site.xml",
    "serviceName": "OOZIE",
    "dependedServiceName": [{name: "HBASE", replace: "\nRULE:[2:$1@$0](<templateName[7]>@.*<templateName[1]>)s/.*/<templateName[8]>/\nRULE:[2:$1@$0](<templateName[9]>@.*<templateName[1]>)s/.*/<templateName[8]>/"}]
  },
  {
    "name": "templeton.kerberos.principal",
    "templateName": ["webHCat_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "webhcat-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "templeton.kerberos.keytab",
    "templateName": ["webhcat_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "webhcat-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "templeton.kerberos.secret",
    "templateName": [""],
    "foreignKey": null,
    "value": "secret",
    "filename": "webhcat-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "templeton.hive.properties",
    "templateName": ["hive_metastore","hive_metastore_principal_name","kerberos_domain"],
    "foreignKey": null,
    "value": "hive.metastore.local=false,hive.metastore.uris=<templateName[0]>,hive." +
      "metastore.sasl.enabled=true,hive.metastore.execute.setugi=true,hive.metastore.warehouse.dir=/apps/hive/warehouse,hive.exec.mode.local.auto=false,hive.metastore.kerberos.principal=<templateName[1]>@<templateName[2]>",
    "filename": "webhcat-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hbase.security.authentication",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "nonSecureValue": "simple",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hbase.security.authorization",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "hbase.bulkload.staging.dir",
    "templateName": [],
    "foreignKey": null,
    "value": "/apps/hbase/staging",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },
  {
    "name": "zookeeper.znode.parent",
    "templateName": [],
    "foreignKey": null,
    "value": "/hbase-secure",
    "nonSecureValue": "/hbase-unsecure",
    "filename": "hbase-site.xml",
    "serviceName": "HBASE"
  },

  /***************************************FALCON***********************************************/
  {
    "name": "*.falcon.authentication.type",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "nonSecureValue": "simple",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.falcon.http.authentication.type",
    "templateName": [],
    "foreignKey": null,
    "value": "kerberos",
    "nonSecureValue": "simple",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.falcon.service.authentication.kerberos.principal",
    "templateName": ["falcon_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.falcon.service.authentication.kerberos.keytab",
    "templateName": ["falcon_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.falcon.http.authentication.kerberos.principal",
    "templateName": ["falcon_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.falcon.http.authentication.kerberos.keytab",
    "templateName": ["falcon_http_keytab"],
    "foreignKey": null,
    "value": "<templateName[0]>",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },
  {
    "name": "*.dfs.namenode.kerberos.principal",
    "templateName": ["namenode_principal_name_falcon", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "filename": "falcon-startup.properties.xml",
    "serviceName": "FALCON"
  },

  /***************************************KNOX***********************************************/
  {
    "name": "gateway.hadoop.kerberos.secured",
    "templateName": [],
    "foreignKey": null,
    "value": "true",
    "nonSecureValue": "false",
    "filename": "gateway-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "java.security.krb5.conf",
    "templateName": [],
    "foreignKey": null,
    "value": "/etc/krb5.conf",
    "filename": "gateway-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["knox_gateway_hosts"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "webhcat.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "webhcat-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "webhcat.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["knox_gateway_hosts"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "webhcat-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "oozie.service.ProxyUserService.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "oozie-site.xml",
    "serviceName": "KNOX"
  },
  {
    "name": "oozie.service.ProxyUserService.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["knox_gateway_hosts"],
    "foreignKey": ["knox_primary_name"],
    "value": "<templateName[0]>",
    "filename": "oozie-site.xml",
    "serviceName": "KNOX"
  },
/***************************************core-site***************************************************/
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["hive_metastore_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["hive_metastore"],
    "foreignKey": ["hive_metastore_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["oozie_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["oozieserver_host"],
    "foreignKey": ["oozie_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "OOZIE"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.groups",
    "templateName": ["proxyuser_group"],
    "foreignKey": ["webHCat_http_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "HIVE"
  },
  {
    "name": "hadoop.proxyuser.<foreignKey[0]>.hosts",
    "templateName": ["webhcat_server"],
    "foreignKey": ["webHCat_http_primary_name"],
    "value": "<templateName[0]>",
    "filename": "core-site.xml",
    "serviceName": "HIVE"
  }
];

var yarn22Mapping = [
  {
    "name": 'hadoop.http.authentication.kerberos.principal',
    "templateName": ["hadoop_http_principal_name", "kerberos_domain"],
    "foreignKey": null,
    "value": "<templateName[0]>@<templateName[1]>",
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.kerberos.keytab',
    "foreignKey": null,
    "templateName": ["hadoop_http_keytab"],
    "value": "<templateName[0]>",
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    name: 'hadoop.http.authentication.kerberos.name.rules',
    templateName: [],
    foreignKey: null,
    value: "",
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.signature.secret',
    "templateName": [],
    "foreignKey": null,
    "value": "",
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.signature.secret.file',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.signer.secret.provider',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.signer.secret.provider.object',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {

    "name": 'yarn.timeline-service.http-authentication.token.validity',
    "templateName": [],
    "foreignKey": null,
    "value": "",
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.cookie.domain',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.cookie.path',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.simple.anonymous.allowed',
    "value": "true",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.proxyuser.*.hosts',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.proxyuser.*.users',
    "value": "",
    "serviceName": "YARN",
    "templateName": [],
    "foreignKey": null,
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.proxyuser.*.groups',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'hadoop.http.filter.initializers',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.type',
    "value": "simple",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.signature.secret',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.signature.secret.file',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.signer.secret.provider',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.signer.secret.provider.object',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.token.validity',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.cookie.domain',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'hadoop.http.authentication.cookie.path',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "HDFS",
    "filename": "core-site.xml"
  },
  {
    "name": 'yarn.timeline-service.http-authentication.kerberos.name.rules',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.resourcemanager.proxyuser.*.hosts',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.resourcemanager.proxyuser.*.users',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.resourcemanager.proxyuser.*.groups',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.resourcemanager.proxy-user-privileges.enabled',
    "value": "true",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  },
  {
    "name": 'yarn.nodemanager.linux-container-executor.cgroups.mount-path',
    "value": "",
    "templateName": [],
    "foreignKey": null,
    "serviceName": "YARN",
    "filename": "yarn-site.xml"
  }
];

props.pushObjects(yarn22Mapping);

module.exports = props;

});

require.register("data/controller_route", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * This is required to reroute directly to the wizard controller. This is desired in following scenarios
 * 1. A wizard is being navigated and another browser window is simultaneously opened by the same or another user
 * 2. browser window crashed while navigating through any wizard and a new browser window is opened to complete the wizard
 * @type {Array} Array of Objects
 */
module.exports = [
  {
    wizardControllerName: App.router.get('addHostController.name'),
    route: 'main.hostAdd'
  },
  {
    wizardControllerName: App.router.get('kerberosWizardController.name'),
    route: 'main.admin.adminKerberos.adminAddKerberos'
  },
  {
    wizardControllerName: App.router.get('addServiceController.name'),
    route: 'main.serviceAdd'
  },
  {
    wizardControllerName: App.router.get('reassignMasterController.name'),
    route: 'main.reassign'
  },
  {
    wizardControllerName: App.router.get('highAvailabilityWizardController.name'),
    route: 'main.services.enableHighAvailability'
  },
  {
    wizardControllerName: App.router.get('rMHighAvailabilityWizardController.name'),
    route: 'main.services.enableRMHighAvailability'
  },
  {
    wizardControllerName: App.router.get('addHawqStandbyWizardController.name'),
    route: 'main.services.addHawqStandby'
  },
  {
    wizardControllerName: App.router.get('removeHawqStandbyWizardController.name'),
    route: 'main.services.removeHawqStandby'
  },
  {
    wizardControllerName: App.router.get('activateHawqStandbyWizardController.name'),
    route: 'main.services.activateHawqStandby'
  },
  {
    wizardControllerName: App.router.get('rAHighAvailabilityWizardController.name'),
    route: 'main.services.enableRAHighAvailability'
  },
  {
    wizardControllerName: App.router.get('rollbackHighAvailabilityWizardController.name'),
    route: 'main.services.rollbackHighAvailability'
  },
  {
    wizardControllerName: App.router.get('mainAdminStackAndUpgradeController.name'),
    route: App.db.get('MainAdminStackAndUpgrade', 'upgradeState') === 'NOT_REQUIRED' ? 'main.admin.stackAndUpgrade.index' : 'main.admin.stackAndUpgrade.versions'
  },
  {
    wizardControllerName: App.router.get('widgetWizardController.name'),
    route: 'main.createWidget'
  },
  {
    wizardControllerName: App.router.get('widgetEditController.name'),
    route: 'main.editWidget'
  },
  {
    wizardControllerName: App.router.get('nameNodeFederationWizardController.name'),
    route: 'main.services.enableNameNodeFederation'
  },
  {
    wizardControllerName: App.router.get('manageJournalNodeWizardController.name'),
    route: 'main.services.manageJournalNode'
  }
];

});

require.register("data/custom_stack_map", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
sign will be compared like: <clusterSelectedStackVersion> sign <stackVersionNumber> if true use this baseStackFolder
Example:
 {
 "stackName": "PHD",
 "stackVersionNumber": "3.0",
 "sign": "<",
 "baseStackFolder": "HDP2"
 }
 This rule will work for any PHD version number that is lower than 3.0 (2.9,2.8, e.t.c)
*/

module.exports = [
  {
    "stackName": "PHD",
    "stackVersionNumber": "3.3",
    "sign": "=",
    "baseStackFolder": "HDP2.3"
  },
  {
    "stackName": "PHD",
    "stackVersionNumber": "3.0",
    "sign": "=",
    "baseStackFolder": "HDP2.2"
  },
  {
    "stackName": "PHD",
    "stackVersionNumber": "3.0",
    "sign": "<",
    "baseStackFolder": "HDP2"
  },
  {
    "stackName": "HDF",
    "stackVersionNumber": "2.0",
    "sign": ">=",
    "baseStackFolder": "HDP2.3"
  }
];

});

require.register("data/dashboard_widgets", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = [
  {
    id: 1,
    viewName: 'NameNodeHeapPieChartView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.NameNodeHeap'),
    threshold: [80, 90],
    groupName: 'nn'
  },
  {
    id: 2,
    viewName: 'NameNodeCapacityPieChartView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.HDFSDiskUsage'),
    threshold: [85, 95]
  },
  {
    id: 3,
    viewName: 'NameNodeCpuPieChartView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.NameNodeCpu'),
    threshold: [90, 95],
    groupName: 'nn'
  },
  {
    id: 4,
    viewName: 'DataNodeUpView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.DataNodeUp'),
    threshold: [80, 90]
  },
  {
    id: 5,
    viewName: 'NameNodeRpcView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.NameNodeRpc'),
    threshold: [1000, 3000],
    groupName: 'nn'
  },
  {
    id: 6,
    viewName: 'ChartClusterMetricsMemoryWidgetView',
    sourceName: 'HOST_METRICS',
    title: Em.I18n.t('dashboard.clusterMetrics.memory'),
    threshold: []
  },
  {
    id: 7,
    viewName: 'ChartClusterMetricsNetworkWidgetView',
    sourceName: 'HOST_METRICS',
    title: Em.I18n.t('dashboard.clusterMetrics.network'),
    threshold: []
  },
  {
    id: 8,
    viewName: 'ChartClusterMetricsCPUWidgetView',
    sourceName: 'HOST_METRICS',
    title: Em.I18n.t('dashboard.clusterMetrics.cpu'),
    threshold: []
  },
  {
    id: 9,
    viewName: 'ChartClusterMetricsLoadWidgetView',
    sourceName: 'HOST_METRICS',
    title: Em.I18n.t('dashboard.clusterMetrics.load'),
    threshold: []
  },
  {
    id: 10,
    viewName: 'NameNodeUptimeView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.NameNodeUptime'),
    threshold: [],
    groupName: 'nn'
  },
  {
    id: 11,
    viewName: 'HDFSLinksView',
    sourceName: 'HDFS',
    title: Em.I18n.t('dashboard.widgets.HDFSLinks'),
    threshold: [],
    groupName: 'nn',
    isHiddenByDefault: true
  },
  {
    id: 12,
    viewName: 'HBaseLinksView',
    sourceName: 'HBASE',
    title: Em.I18n.t('dashboard.widgets.HBaseLinks'),
    threshold: [],
    isHiddenByDefault: true
  },
  {
    id: 13,
    viewName: 'HBaseMasterHeapPieChartView',
    sourceName: 'HBASE',
    title: Em.I18n.t('dashboard.widgets.HBaseMasterHeap'),
    threshold: [70, 90]
  },
  {
    id: 14,
    viewName: 'HBaseAverageLoadView',
    sourceName: 'HBASE',
    title: Em.I18n.t('dashboard.widgets.HBaseAverageLoad'),
    threshold: [150, 250]
  },
  {
    id: 15,
    viewName: 'HBaseRegionsInTransitionView',
    sourceName: 'HBASE',
    title: Em.I18n.t('dashboard.widgets.HBaseRegionsInTransition'),
    threshold: [3, 10]
  },
  {
    id: 16,
    viewName: 'HBaseMasterUptimeView',
    sourceName: 'HBASE',
    title: Em.I18n.t('dashboard.widgets.HBaseMasterUptime'),
    threshold: []
  },
  {
    id: 17,
    viewName: 'ResourceManagerHeapPieChartView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.ResourceManagerHeap'),
    threshold: [70, 90]
  },
  {
    id: 18,
    viewName: 'ResourceManagerUptimeView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.ResourceManagerUptime'),
    threshold: [],
    isHiddenByDefault: true
  },
  {
    id: 19,
    viewName: 'NodeManagersLiveView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.NodeManagersLive'),
    threshold: [50, 75]
  },
  {
    id: 20,
    viewName: 'YARNMemoryPieChartView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.YARNMemory'),
    threshold: [50, 75],
    isHiddenByDefault: true
  },
  {
    id: 21,
    viewName: 'SuperVisorUpView',
    sourceName: 'STORM',
    title: Em.I18n.t('dashboard.widgets.SuperVisorUp'),
    threshold: [85, 95]
  },
  {
    id: 22,
    viewName: 'FlumeAgentUpView',
    sourceName: 'FLUME',
    title: Em.I18n.t('dashboard.widgets.FlumeAgentUp'),
    threshold: [85, 95]
  },
  {
    id: 23,
    viewName: 'YARNLinksView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.YARNLinks'),
    threshold: [],
    isHiddenByDefault: true
  },
  {
    id: 24,
    viewName: 'HawqSegmentUpView',
    sourceName: 'HAWQ',
    title: Em.I18n.t('dashboard.widgets.HawqSegmentUp'),
    threshold: [75, 90]
  },
  {
    id: 25,
    viewName: 'PxfUpView',
    sourceName: 'PXF',
    title: Em.I18n.t('dashboard.widgets.PxfUp'),
    threshold: []
  },
  {
    id: 26,
    viewName: 'YarnContainersView',
    sourceName: 'YARN',
    title: Em.I18n.t('dashboard.widgets.YarnContainers'),
    threshold: []
  },
];
});

require.register("data/db_properties_info", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = {
  /**
   * object that shows property names for different kind of config properties
   * - radio button property (db_selector) - main config on which can depend other core properties
   * - properties used to generate connection url (host_name, db_name)
   * - properties that should be updated on radio buttons change (connection_url, driver, db_type, sql_jar_connector)
   * @see <code>setConnectionUrl<code> for details for connection_url
   * - properties that should be hidden in special case (user_name, password)
   * @see <code>handleSpecialUserPassProperties<code> method
   * @type {object}
   */
  dpPropertiesByServiceMap: {
    'HIVE': {
      db_selector: 'hive_database',
      db_name: 'ambari.hive.db.schema.name',

      connection_url: 'javax.jdo.option.ConnectionURL',
      db_type: 'hive_database_type',
      driver: 'javax.jdo.option.ConnectionDriverName',

      user_name: 'javax.jdo.option.ConnectionUserName',
      password: 'javax.jdo.option.ConnectionPassword'
    },
    'OOZIE': {
      db_selector: 'oozie_database',
      db_name: 'oozie.db.schema.name',

      connection_url: 'oozie.service.JPAService.jdbc.url',
      driver: 'oozie.service.JPAService.jdbc.driver',

      user_name: 'oozie.service.JPAService.jdbc.username',
      password: 'oozie.service.JPAService.jdbc.password'
    }
  },

  /**
   * object that shows default(recommended) values for different db type;
   * properties that should have default value ['connection_url', 'driver', 'sql_jar_connector', 'db_type']
   * some properties can be skipped but be sure they are not used.
   * @type {object}
   */
  dpPropertiesMap: {
    'MYSQL': {
      'connection_url': 'jdbc:mysql://{0}/{1}',
      /** in case property has different default value for specific service it can be overriden in such way **/
      'HIVE': {
        'connection_url': 'jdbc:mysql://{0}/{1}?createDatabaseIfNotExist=true'
      },
      'driver': 'mysql-connector-java.jar',
      'sql_jar_connector': '/usr/share/java/mysql-connector-java.jar',
      'db_type': 'mysql',
      'db_name': 'MySQL',
      'driver_download_url': 'https://dev.mysql.com/downloads/connector/j/',
      'driver_name': 'MySQL Connector/J JDBC Driver'
    },
    'POSTGRES': {
      'connection_url': 'jdbc:postgresql://{0}:5432/{1}',
      'driver': 'org.postgresql.Driver',
      'sql_jar_connector': '/usr/share/java/postgresql.jar',
      'db_type': 'postgres',
      'db_name': 'PostgreSQL',
      'driver_download_url': 'https://jdbc.postgresql.org/',
      'driver_name': 'PostgreSQL JDBC Driver'
    },
    'ORACLE': {
      'connection_url': 'jdbc:oracle:thin:@//{0}:1521/{1}',
      'driver': 'oracle.jdbc.driver.OracleDriver',
      'sql_jar_connector': '/usr/share/java/ojdbc6.jar',
      'db_type': 'oracle',
      'db_name': 'Oracle',
      'driver_download_url': 'http://www.oracle.com/technetwork/database/features/jdbc/index-091264.html',
      'driver_name': 'Oracle JDBC Driver'
    },
    'MSSQL': {
      'connection_url': 'jdbc:sqlserver://{0};databaseName={1}',
      'driver': 'com.microsoft.sqlserver.jdbc.SQLServerDriver',
      'sql_jar_connector': '/usr/share/java/sqljdbc4.jar',
      'db_type': 'mssql'
    },
    'MSSQL2': {
      'connection_url': 'jdbc:sqlserver://{0};databaseName={1};integratedSecurity=true',
      'driver': 'com.microsoft.sqlserver.jdbc.SQLServerDriver',
      'sql_jar_connector': '/usr/share/java/sqljdbc4.jar',
      'db_type': 'mssql'
    },
    /** TODO: Remove SQLA from the list of databases once Ranger DB_FLAVOR=SQLA is replaced with SQL Anywhere */
    'SQLA': {
      'connection_url': 'jdbc:sqlanywhere:host={0};database={1}',
      'driver': 'sap.jdbc4.sqlanywhere.IDriver',
      'sql_jar_connector': '/path_to_driver/sqla-client-jdbc.tar.gz',
      'db_type': 'sqlanywhere'
    },
    'ANYWHERE': {
      'connection_url': 'jdbc:sqlanywhere:host={0};database={1}',
      'driver': 'sap.jdbc4.sqlanywhere.IDriver',
      'sql_jar_connector': '/path_to_driver/sqla-client-jdbc.tar.gz',
      'db_type': 'sqlanywhere'
    },
    'DERBY': {
      'connection_url': 'jdbc:derby:${oozie.data.dir}/${oozie.db.schema.name}-db;create=true',
      'driver': 'org.apache.derby.jdbc.EmbeddedDriver',
      'db_type': 'derby'
    }
  }
};

});

require.register("data/host/categories", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
module.exports = [
  {
    value: Em.I18n.t('common.all'),
    isHealthStatus: true,
    healthStatus: '',
    healthClass: '',
    isActive: true,
    isVisible: false
  },
  {
    value: Em.I18n.t('hosts.host.healthStatusCategory.green'),
    isHealthStatus: true,
    class: App.healthIconClassGreen,
    healthStatus: 'HEALTHY',
    healthClass: 'health-status-LIVE'
  },
  {
    value: Em.I18n.t('hosts.host.healthStatusCategory.red'),
    isHealthStatus: true,
    class: App.healthIconClassRed,
    healthStatus: 'UNHEALTHY',
    healthClass: 'health-status-DEAD-RED'
  },
  {
    value: Em.I18n.t('hosts.host.healthStatusCategory.orange'),
    isHealthStatus: true,
    class: App.healthIconClassOrange,
    healthStatus: 'ALERT',
    healthClass: 'health-status-DEAD-ORANGE'
  },
  {
    value: Em.I18n.t('hosts.host.healthStatusCategory.yellow'),
    isHealthStatus: true,
    class: App.healthIconClassYellow,
    healthStatus: 'UNKNOWN',
    healthClass: 'health-status-DEAD-YELLOW'
  },
  {
    value: Em.I18n.t('hosts.host.alerts.label'),
    hostProperty: 'criticalWarningAlertsCount',
    class: 'glyphicon glyphicon-exclamation-sign',
    isHealthStatus: false,
    healthClass: 'health-status-WITH-ALERTS',
    healthStatus: 'health-status-WITH-ALERTS',
    column: 7,
    type: 'custom',
    filterValue: ['>0', '>0']
  },
  {
    value: Em.I18n.t('common.restart'),
    hostProperty: 'componentsWithStaleConfigsCount',
    class: 'glyphicon glyphicon-refresh',
    isHealthStatus: false,
    healthClass: 'health-status-RESTART',
    healthStatus: 'health-status-RESTART',
    column: 8,
    type: 'string',
    filterValue: 'true'
  },
  {
    value: Em.I18n.t('common.passive_state'),
    hostProperty: 'componentsInPassiveStateCount',
    class: 'passive-state icon-medkit',
    isHealthStatus: false,
    healthClass: 'health-status-PASSIVE_STATE',
    healthStatus: 'health-status-PASSIVE_STATE',
    column: 9,
    type: 'string',
    filterValue: ['ON','IMPLIED_FROM_HOST','IMPLIED_FROM_SERVICE','IMPLIED_FROM_SERVICE_AND_HOST']
  }
];

});

require.register("data/service_configs", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('models/configs/objects/service_config_category');

/**
 * This
 * @type {Array} Array of non-service tabs that will appears on Customizes services page
 */
module.exports = [
  Em.Object.create({
    serviceName: 'MISC',
    displayName: 'Misc',
    configCategories: [
      App.ServiceConfigCategory.create({ name: 'Users and Groups', displayName : 'Services Accounts'}),
      App.ServiceConfigCategory.create({ name: 'Notifications', displayName : 'Notifications', isCustomView: true, customView: App.NotificationsConfigsView, siteFileName: 'alert_notification'})
    ],
    configTypes: {
      "cluster-env": {supports: {final: false}}
    },
    configs: []
  })
];

});

require.register("data/service_graph_config", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 This determines the graphs to display on the service page under each service.

 This is based on the name of the object associated with it.

 The name of the object is of the format: 'App.ChartServiceMetrics<name>' where <name>
 is one of the items below.
 **/

module.exports = {
    'ambari_metrics': [
      'AMS_MasterAverageLoad',
      'AMS_RegionServerStoreFiles',
      'AMS_RegionServerRegions',
      'AMS_RegionServerRequests',
      'AMS_RegionServerBlockCacheHitPercent',
      'AMS_RegionServerCompactionQueueSize'
    ],

    'flume': [
      'Flume_ChannelSizeMMA',
      'Flume_ChannelSizeSum',
      'Flume_IncommingMMA',
      'Flume_IncommingSum',
      'Flume_OutgoingMMA',
      'Flume_OutgoingSum'
    ]
};
});

require.register("data/statusCodes", function(exports, require, module) {
/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


module.exports = {
	200: function () {
		console.log("Status code 200: Success.");
  },
  202: function () {
    console.log("Status code 202: Success for creation.");
  },
	400: function () {
		console.log("Error code 400: Bad Request.");
	},
	401: function () {
		console.log("Error code 401: Unauthorized.");
	},
	402: function () {
		console.log("Error code 402: Payment Required.");
	},
	403: function () {
		console.log("Error code 403: Forbidden.");
    App.router.logOff();
	},
	404: function () {
		console.log("Error code 404: URI not found.");
	},
	500: function () {
		console.log("Error code 500: Internal Error on server side.");
	},
	501: function () {
		console.log("Error code 501: Not implemented yet.");
	},
	502: function () {
		console.log("Error code 502: Services temporarily overloaded.");
	},
	503: function () {
		console.log("Error code 503: Gateway timeout.");
	}
};

});

require.register("ember", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = Ember;

});

require.register("initialize", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

window.App = require('app');

require('config');

var browserLanguage = '';

try {
  browserLanguage = navigator.language.slice(0, 2);
  require('locales/' + browserLanguage + '/messages');
} catch (error) {
  require('messages');
}

require('utils');
require('mixins');
require('models');
require('controllers');
require('templates');
require('views');
require('utils/handlebars_helpers');
require('router');

require('utils/ajax/ajax');
require('utils/ajax/ajax_queue');
require('utils/updater');
require('utils/action_sequence');
require('utils/load_timer');

require('mappers');

require('utils/http_client');
require('utils/host_progress_popup');

App.initialize();

console.log('after initialize');
console.log('TRACE: app.js-> localStorage:Ambari.authenticated=' + localStorage.getItem('Ambari' + 'authenticated'));
console.log('TRACE: app.js-> localStorage:currentStep=' + localStorage.getItem(App.get('router').getLoginName() + 'Installer' + 'currentStep'));
console.log('TRACE: app.js-> router.authenticated=' + App.get('router.loggedIn'));

});

require.register("mappers", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

//load all mappers
require('mappers/server_data_mapper');
require('mappers/stack_service_mapper');
require('mappers/stack_mapper');
require('mappers/stack_version_mapper');
require('mappers/configs/themes_mapper');
require('mappers/configs/stack_config_properties_mapper');
require('mappers/configs/config_groups_mapper');
require('mappers/configs/service_config_version_mapper');
require('mappers/repository_version_mapper');
require('mappers/quicklinks_mapper');
require('mappers/hosts_mapper');
require('mappers/cluster_mapper');
require('mappers/racks_mapper');
require('mappers/users_mapper');
require('mappers/service_mapper');
require('mappers/service_metrics_mapper');
require('mappers/target_cluster_mapper');
require('mappers/components_state_mapper');
require('mappers/alert_definitions_mapper');
require('mappers/alert_definition_summary_mapper');
require('mappers/alert_instances_mapper');
require('mappers/alert_groups_mapper');
require('mappers/alert_notification_mapper');
require('mappers/root_service_mapper');
require('mappers/widget_mapper');
require('mappers/widget_layout_mapper');
require('mappers/stack_upgrade_history_mapper');
require('mappers/socket/topology_mapper');
require('mappers/socket/service_state_mapper');
require('mappers/socket/host_component_status_mapper');
require('mappers/socket/alert_summary_mapper');
require('mappers/socket/host_state_mapper');
require('mappers/socket/alert_definitions_mapper_adapter');
require('mappers/socket/alert_groups_mapper_adapter');
require('mappers/socket/upgrade_state_mapper');

});

require.register("mappers/alert_definition_summary_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var dataManipulation = require('utils/data_manipulation');

App.alertDefinitionSummaryMapper = App.QuickDataMapper.create({

  config: {},

  map: function map(data) {
    console.time('App.alertDefinitionSummaryMapper execution time');

    if (!data.alerts_summary_grouped) return;
    var alertDefinitions = App.AlertDefinition.find();
    var alertDefinitionsMap = alertDefinitions.toArray().toMapByProperty('id');
    var summaryMap = {};
    data.alerts_summary_grouped.forEach(function (alertDefinitionSummary) {
      var alertDefinition = alertDefinitionsMap[alertDefinitionSummary.definition_id];
      if (alertDefinition) {
        var summary = {},
            timestamp = 0;
        Em.keys(alertDefinitionSummary.summary).forEach(function (status) {
          summary[status] = {
            count: alertDefinitionSummary.summary[status].count,
            maintenanceCount: alertDefinitionSummary.summary[status].maintenance_count
          };
          if (alertDefinitionSummary.summary[status].latest_text) {
            summary[status].latestText = alertDefinitionSummary.summary[status].latest_text;
          }
          if (alertDefinitionSummary.summary[status].original_timestamp > timestamp) {
            timestamp = alertDefinitionSummary.summary[status].original_timestamp;
          }
        });
        summaryMap[alertDefinitionSummary.definition_id] = {
          summary: summary,
          lastTriggered: App.dateTimeWithTimeZone(parseInt(timestamp, 10)),
          lastTriggeredRaw: timestamp
        };
      }
    });

    alertDefinitions.forEach(function (d) {
      var id = d.get('id');
      if (alertDefinitionsMap[id].get('stateManager.currentState.name') !== 'saved') {
        alertDefinitionsMap[id].get('stateManager').transitionTo('saved');
      }
      alertDefinitionsMap[id].setProperties(summaryMap[id]);
      if (!alertDefinitionsMap[id].get('enabled')) {
        // clear summary for disabled alert definitions
        alertDefinitionsMap[id].set('summary', {});
      }
    });
    // set alertsCount and hasCriticalAlerts for each service
    var groupedByServiceName = dataManipulation.groupPropertyValues(alertDefinitions, 'service.serviceName');
    var groupedByComponentName = dataManipulation.groupPropertyValues(alertDefinitions, 'componentName');
    var services = App.Service.find();
    var servicesMap = services.toArray().toMapByProperty('id');
    Object.keys(groupedByServiceName).forEach(function (serviceName) {
      var service = servicesMap[serviceName];
      if (service) {
        var hasCriticalAlerts = false;

        var alertsCount = groupedByServiceName[serviceName].map(function (alertDefinition) {

          var criticalCount = alertDefinition.getWithDefault('summary.CRITICAL.count', 0);
          var warningCount = alertDefinition.getWithDefault('summary.WARNING.count', 0);

          if (criticalCount) {
            hasCriticalAlerts = true;
          }
          return criticalCount + warningCount;
        }).reduce(Em.sum, 0);

        service.setProperties({
          alertsCount: alertsCount,
          hasCriticalAlerts: hasCriticalAlerts
        });

        service.get('hostComponents').filterProperty('isMaster').forEach(function (master) {

          hasCriticalAlerts = false;
          alertsCount = (groupedByComponentName[master.get('componentName')] || []).map(function (alertDefinition) {

            var criticalCount = alertDefinition.getWithDefault('summary.CRITICAL.count', 0);
            var warningCount = alertDefinition.getWithDefault('summary.WARNING.count', 0);

            if (criticalCount) {
              hasCriticalAlerts = true;
            }
            return criticalCount + warningCount;
          }).reduce(Em.sum, 0);

          master.setProperties({
            alertsCount: alertsCount,
            hasCriticalAlerts: hasCriticalAlerts
          });
        });
      }
    });
    console.timeEnd('App.alertDefinitionSummaryMapper execution time');
  }
});

});

require.register("mappers/alert_definitions_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertDefinitionsMapper = App.QuickDataMapper.create({

  model: App.AlertDefinition,
  reportModel: App.AlertReportDefinition,
  metricsSourceModel: App.AlertMetricsSourceDefinition,
  metricsUriModel: App.AlertMetricsUriDefinition,
  metricsAmsModel: App.AlertMetricsAmsDefinition,
  parameterModel: App.AlertDefinitionParameter,

  config: {
    id: 'AlertDefinition.id',
    name: 'AlertDefinition.name',
    description: 'AlertDefinition.description',
    label: 'AlertDefinition.label',
    service_id: 'AlertDefinition.service_name',
    service_name: 'AlertDefinition.service_name',
    component_name: 'AlertDefinition.component_name',
    enabled: 'AlertDefinition.enabled',
    repeat_tolerance_enabled: 'AlertDefinition.repeat_tolerance_enabled',
    repeat_tolerance: 'AlertDefinition.repeat_tolerance',
    scope: 'AlertDefinition.scope',
    interval: 'AlertDefinition.interval',
    help_url: 'AlertDefinition.help_url',
    type: 'AlertDefinition.source.type',
    reporting_key: 'reporting',
    reporting_type: 'array',
    reporting: {
      item: 'id'
    },
    parameters_key: 'parameters',
    parameters_type: 'array',
    parameters: {
      item: 'id'
    }
  },

  portConfig: {
    default_port: 'AlertDefinition.source.default_port',
    port_uri: 'AlertDefinition.source.uri'
  },

  aggregateConfig: {
    alert_name: 'AlertDefinition.source.alert_name'
  },

  scriptConfig: {
    location: 'AlertDefinition.source.path'
  },

  serverConfig: {},

  uriConfig: {
    id: 'AlertDefinition.source.uri.id',
    http: 'AlertDefinition.source.uri.http',
    https: 'AlertDefinition.source.uri.https',
    https_property: 'AlertDefinition.source.uri.https_property',
    https_property_value: 'AlertDefinition.source.uri.https_property_value',
    connection_timeout: 'AlertDefinition.source.uri.connection_timeout'
  },

  amsConfig: {
    id: 'AlertDefinition.source.ams.id',
    value: 'AlertDefinition.source.ams.value',
    minimal_value: 'AlertDefinition.source.ams.minimum_value',
    interval: 'AlertDefinition.source.ams.interval'
  },

  map: function map(json, ignoreDelete) {
    console.time('App.alertDefinitionsMapper execution time');
    if (json && json.items) {
      var self = this,
          parameters = [],
          alertDefinitions = [],
          alertReportDefinitions = [],
          alertMetricsSourceDefinitions = [],
          alertMetricsUriDefinitions = [],
          alertMetricsAmsDefinitions = [],
          alertGroupsMap = App.cache.previousAlertGroupsMap,
          existingAlertDefinitions = App.AlertDefinition.find(),
          existingAlertDefinitionsMap = existingAlertDefinitions.toArray().toMapByProperty('id'),
          alertDefinitionsToDelete = existingAlertDefinitions.mapProperty('id'),
          rawSourceData = {};

      json.items.forEach(function (item) {
        var convertedReportDefinitions = [];
        var reporting = item.AlertDefinition.source.reporting;
        for (var report in reporting) {
          if (reporting.hasOwnProperty(report)) {
            if (report === "units") {
              convertedReportDefinitions.push({
                id: item.AlertDefinition.id + report,
                type: report,
                text: reporting[report]
              });
            } else {
              convertedReportDefinitions.push({
                id: item.AlertDefinition.id + report,
                type: report,
                text: reporting[report].text,
                value: reporting[report].value
              });
            }
          }
        }

        var convertedParameters = [];
        var sourceParameters = item.AlertDefinition.source.parameters;
        if (Ember.isArray(sourceParameters)) {
          sourceParameters.forEach(function (parameter) {
            var hash = Em.getProperties(parameter, ['name', 'display_name', 'units', 'value', 'description', 'type', 'threshold', 'visibility']);
            hash.id = item.AlertDefinition.id + parameter.name;
            convertedParameters.push(hash);
          });
        }

        alertReportDefinitions = alertReportDefinitions.concat(convertedReportDefinitions);
        parameters = parameters.concat(convertedParameters);
        item.reporting = convertedReportDefinitions;
        item.parameters = convertedParameters;

        rawSourceData[item.AlertDefinition.id] = item.AlertDefinition.source;
        item.AlertDefinition.description = item.AlertDefinition.description || '';

        var alertDefinition = this.parseIt(item, this.get('config'));

        if (alertGroupsMap[alertDefinition.id]) {
          alertDefinition.groups = alertGroupsMap[alertDefinition.id];
        }

        var oldAlertDefinition = existingAlertDefinitionsMap[alertDefinition.id];
        if (oldAlertDefinition) {
          // new values will be parsed in the another mapper, so for now just use old values
          alertDefinition.summary = oldAlertDefinition.get('summary');
          alertDefinition.last_triggered = oldAlertDefinition.get('lastTriggered');
          alertDefinition.last_triggered_raw = oldAlertDefinition.get('lastTriggeredRaw');
        }

        alertDefinitionsToDelete = alertDefinitionsToDelete.without(alertDefinition.id);

        // map properties dependent on Alert Definition type
        switch (item.AlertDefinition.source.type) {
          case 'PORT':
            alertDefinitions.push($.extend(alertDefinition, this.parseIt(item, this.get('portConfig'))));
            break;
          case 'METRIC':
            // map App.AlertMetricsSourceDefinition's
            var jmxMetric = item.AlertDefinition.source.jmx;
            var gangliaMetric = item.AlertDefinition.source.ganglia;
            if (jmxMetric) {
              alertDefinition.jmx_id = item.AlertDefinition.id + 'jmx';
              alertMetricsSourceDefinitions.push({
                id: alertDefinition.jmx_id,
                value: jmxMetric.value,
                property_list: jmxMetric.property_list
              });
            }
            if (gangliaMetric) {
              alertDefinition.ganglia_id = item.AlertDefinition.id + 'ganglia';
              alertMetricsSourceDefinitions.push({
                id: alertDefinition.ganglia_id,
                value: gangliaMetric.value,
                property_list: gangliaMetric.property_list
              });
            }

            // map App.AlertMetricsUriDefinition
            alertDefinition.uri_id = item.AlertDefinition.id + 'uri';
            item.AlertDefinition.source.uri.id = alertDefinition.uri_id;
            alertMetricsUriDefinitions.push(this.parseIt(item, this.get('uriConfig')));
            alertDefinitions.push(alertDefinition);
            break;
          case 'WEB':
            // map App.AlertMetricsUriDefinition
            alertDefinition.uri_id = item.AlertDefinition.id + 'uri';
            item.AlertDefinition.source.uri.id = alertDefinition.uri_id;
            alertMetricsUriDefinitions.push(this.parseIt(item, this.get('uriConfig')));
            alertDefinitions.push(alertDefinition);
            break;
          case 'AGGREGATE':
            alertDefinitions.push($.extend(alertDefinition, this.parseIt(item, this.get('aggregateConfig'))));
            break;
          case 'SCRIPT':
            alertDefinitions.push($.extend(alertDefinition, this.parseIt(item, this.get('scriptConfig'))));
            break;
          case 'SERVER':
            alertDefinitions.push($.extend(alertDefinition, this.parseIt(item, this.get('serverConfig'))));
            break;
          case 'RECOVERY':
            alertDefinitions.push($.extend(alertDefinition, this.parseIt(item, this.get('uriConfig'))));
            break;
          case 'AMS':
            // map App.AlertMetricsUriDefinition
            alertDefinition.uri_id = item.AlertDefinition.id + 'uri';
            alertDefinition.ams_id = item.AlertDefinition.id + 'ams';
            item.AlertDefinition.source.uri.id = alertDefinition.uri_id;
            item.AlertDefinition.source.ams.id = alertDefinition.ams_id;
            alertMetricsUriDefinitions.push(this.parseIt(item, this.get('uriConfig')));
            alertMetricsAmsDefinitions.push(this.parseIt(item, this.get('amsConfig')));
            alertDefinitions.push(alertDefinition);
            break;
          default:
            console.error('Incorrect Alert Definition type:', item.AlertDefinition);
        }
      }, this);

      if (!ignoreDelete) {
        alertDefinitionsToDelete.forEach(function (definitionId) {
          self.deleteRecord(existingAlertDefinitions.findProperty('id', definitionId));
        });
      }

      // load all mapped data to model
      App.store.safeLoadMany(this.get('reportModel'), alertReportDefinitions);
      App.store.safeLoadMany(this.get('parameterModel'), parameters);
      App.store.safeLoadMany(this.get('metricsSourceModel'), alertMetricsSourceDefinitions);
      this.setMetricsSourcePropertyLists(this.get('metricsSourceModel'), alertMetricsSourceDefinitions);
      App.store.safeLoadMany(this.get('metricsUriModel'), alertMetricsUriDefinitions);
      App.store.safeLoadMany(this.get('metricsAmsModel'), alertMetricsAmsDefinitions);
      // this safeLoadMany takes too much time
      App.store.safeLoadMany(this.get('model'), alertDefinitions);
      this.setAlertDefinitionsRawSourceData(rawSourceData);
    }
    console.timeEnd('App.alertDefinitionsMapper execution time');
  },

  /**
   * set propertyList properties from <code>data</code> for records in <code>model</code>
   * @param model
   * @param data
   */
  setMetricsSourcePropertyLists: function setMetricsSourcePropertyLists(model, data) {
    var modelsMap = model.find().toArray().toMapByProperty('id');
    data.forEach(function (record) {
      var m = modelsMap[record.id];
      if (m) {
        m.set('propertyList', record.property_list);
      }
    });
  },

  /**
   * set rawSourceDate properties for <code>App.AlertDefinition</code> records
   * @param rawSourceData
   */
  setAlertDefinitionsRawSourceData: function setAlertDefinitionsRawSourceData(rawSourceData) {
    var allDefinitions = App.AlertDefinition.find();
    var allDefinitionsMap = allDefinitions.toArray().toMapByProperty('id');
    for (var alertDefinitionId in rawSourceData) {
      if (rawSourceData.hasOwnProperty(alertDefinitionId)) {
        var m = allDefinitionsMap[+alertDefinitionId];
        if (m) {
          m.set('rawSourceData', rawSourceData[alertDefinitionId]);
        }
      }
    }
  }
});

});

require.register("mappers/alert_groups_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * Mapper for <code>App.AlertGroup</code>
 * Save general information
 * Doesn't save not changed data (check it using <code>App.cache['previousAlertGroupsFullMap']</code>)
 * Use <code>App.cache['previousAlertGroupsMap']</code> to store map alertDefinitions-alertGroups. This map is used
 * in the <code>App.AlertDefinitionsMapper</code> to correctly link alertDefinitions and alertGroups
 */
App.alertGroupsMapper = App.QuickDataMapper.create({

  model: App.AlertGroup,

  config: {
    id: 'AlertGroup.id',
    name: 'AlertGroup.name',
    default: 'AlertGroup.default',
    targets_key: 'AlertGroup.targets',
    targets_type: 'array',
    targets: {
      item: 'id'
    }
  },

  map: function map(json, ignoreDelete) {
    if (Em.isNone(App.cache['previousAlertGroupsFullMap'])) {
      App.cache['previousAlertGroupsFullMap'] = {};
    }
    if (!Em.isNone(json, 'items')) {
      console.time('App.alertGroupsMapper execution time');
      var alertGroups = [],
          self = this,
          groupsMap = {},
          groupsToDelete = App.AlertGroup.find().mapProperty('id'),

      /**
       * AlertGroups-map for <code>App.AlertDefinitionsMappers</code>
       * Format:
       * <code>
       *   {
       *    alert_definition1_id: [alert_group1_id, alert_group2_id],
       *    alert_definition2_id: [alert_group3_id, alert_group1_id],
       *    ...
       *   }
       * </code>
       * @type {object}
       */
      alertDefinitionsGroupsMap = {},
          alertNotificationsGroupsMap = {};

      json.items.forEach(function (item) {
        var group = self.parseIt(item, self.get('config'));
        groupsToDelete = groupsToDelete.without(group.id);
        group.targets = [];
        group.definitions = [];
        if (item.AlertGroup.definitions) {
          item.AlertGroup.definitions.forEach(function (definition) {
            if (!group.definitions.contains(definition.id)) {
              group.definitions.push(definition.id);
            }
            if (Em.isNone(alertDefinitionsGroupsMap[definition.id])) {
              alertDefinitionsGroupsMap[definition.id] = [];
            }
            alertDefinitionsGroupsMap[definition.id].push(group.id);
          });
        }
        if (item.AlertGroup.targets) {
          item.AlertGroup.targets.forEach(function (target) {
            if (!group.targets.contains(target.id)) {
              group.targets.push(target.id);
            }
            if (Em.isNone(alertNotificationsGroupsMap[target.id])) {
              alertNotificationsGroupsMap[target.id] = [];
            }
            alertNotificationsGroupsMap[target.id].push(group.id);
          });
        }

        groupsMap[group.id] = group;
        var previousGroup = App.cache['previousAlertGroupsFullMap'][group.id] ? App.cache['previousAlertGroupsFullMap'][group.id] : {};
        var changedFields = self.getDiscrepancies(group, previousGroup, ['name', 'description', 'default', 'targets', 'definitions']);
        if (Object.keys(changedFields).length) {
          alertGroups.push(group);
        }
      }, this);

      if (!ignoreDelete) {
        groupsToDelete.forEach(function (groupId) {
          self.deleteRecord(App.AlertGroup.find(groupId));
        });
      }

      App.cache['previousAlertGroupsMap'] = alertDefinitionsGroupsMap;
      App.cache['previousAlertGroupsFullMap'] = groupsMap;
      App.cache['alertNotificationsGroupsMap'] = alertNotificationsGroupsMap;
      // initial load takes much more time than others, but it's OK (all data should be saved first time)
      App.store.safeLoadMany(this.get('model'), alertGroups);
      console.timeEnd('App.alertGroupsMapper execution time');
    }
  }
});

});

require.register("mappers/alert_instances_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertInstanceMapper = App.QuickDataMapper.create({

  model: App.AlertInstance,

  modelLocal: App.AlertInstanceLocal,

  config: {
    id: 'Alert.id',
    label: 'Alert.label',
    definition_name: 'Alert.definition_name',
    definition_id: 'Alert.definition_id',
    service_id: 'Alert.service_name',
    service_name: 'Alert.service_name',
    component_name: 'Alert.component_name',
    host_id: 'Alert.host_name',
    host_name: 'Alert.host_name',
    scope: 'Alert.scope',
    original_timestamp: 'Alert.original_timestamp',
    original_raw_timestamp: 'Alert.original_timestamp',
    latest_timestamp: 'Alert.latest_timestamp',
    maintenance_state: 'Alert.maintenance_state',
    instance: 'Alert.instance',
    state: 'Alert.state',
    text: 'Alert.text',
    repeat_tolerance: 'Alert.repeat_tolerance',
    repeat_tolerance_remaining: 'Alert.repeat_tolerance_remaining'
  },

  map: function map(json) {
    return this.parse(json, this.get('model'));
  },

  mapLocal: function mapLocal(json) {
    return this.parse(json, this.get('modelLocal'));
  },

  parse: function parse(json, model) {
    console.time('App.alertInstanceMapper execution time');
    if (json.items) {
      var alertInstances = [];
      var alertsToDelete = model.find().mapProperty('id');

      json.items.forEach(function (item) {
        var alert = this.parseIt(item, this.get('config'));
        alert.original_timestamp = App.dateTimeWithTimeZone(alert.original_timestamp);
        alert.latest_timestamp = App.dateTimeWithTimeZone(alert.latest_timestamp);
        alertInstances.push(alert);
        alertsToDelete = alertsToDelete.without(alert.id);
      }, this);

      if (alertsToDelete.length > 0) {
        model.find().clear();
      }

      App.store.safeLoadMany(model, alertInstances);
      console.timeEnd('App.alertInstanceMapper execution time');
    }
  }

});

});

require.register("mappers/alert_notification_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertNotificationMapper = App.QuickDataMapper.create({

  model: App.AlertNotification,

  config: {
    id: 'AlertTarget.id',
    name: 'AlertTarget.name',
    type: 'AlertTarget.notification_type',
    description: 'AlertTarget.description',
    global: 'AlertTarget.global',
    enabled: 'AlertTarget.enabled'
  },

  map: function map(json) {
    if (Em.isNone(App.cache['previousAlertNotificationsFullMap'])) {
      App.cache['previousAlertNotificationsFullMap'] = {};
    }
    console.time('App.alertNotificationMapper execution time');
    if (json.items) {
      var result = [];
      var notificationsProperties = {};
      var notificationsAlertStates = {};
      var groupsMap = App.cache['alertNotificationsGroupsMap'];
      var notifications = {};
      var self = this;

      json.items.forEach(function (item) {
        var notification = this.parseIt(item, this.config);
        var groups = groupsMap && groupsMap[notification.id];
        if (groups) {
          notification.groups = groups;
        }

        var previousNotification = App.cache['previousAlertNotificationsFullMap'][notification.id] ? App.cache['previousAlertNotificationsFullMap'][notification.id] : {};
        var changedFields = self.getDiscrepancies(notification, previousNotification, ['name', 'type', 'description', 'global', 'enabled', 'groups']);
        if (Object.keys(changedFields).length) {
          result.push(notification);
        }

        notifications[notification.id] = notification;
        notificationsProperties[item.AlertTarget.id] = item.AlertTarget.properties;
        notificationsAlertStates[item.AlertTarget.id] = item.AlertTarget.alert_states;
      }, this);

      App.store.safeLoadMany(this.get('model'), result);
      App.cache['previousAlertNotificationsFullMap'] = notifications;
      this._setPropertiesToEachModel('properties', notificationsProperties);
      this._setPropertiesToEachModel('alertStates', notificationsAlertStates);
    }
    console.timeEnd('App.alertNotificationMapper execution time');
  },

  /**
   * Set values from <code>propertyMap</code> for <code>propertyName</code> for each record in model
   * @param {string} propertyName
   * @param {object} propertiesMap record_id to value map
   * @method setPropertiesToEachModel
   * @private
   */
  _setPropertiesToEachModel: function _setPropertiesToEachModel(propertyName, propertiesMap) {
    var modelsMap = this.get('modelsMap');
    for (var recordId in propertiesMap) {
      if (propertiesMap.hasOwnProperty(recordId)) {
        App.AlertNotification.find(recordId).set(propertyName, propertiesMap[recordId]);
      }
    }
  }
});

});

require.register("mappers/cluster_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.clusterMapper = App.QuickDataMapper.create({
  model: App.Cluster,
  map: function map(json) {
    if (!this.get('model')) {
      return;
    }
    if (json) {
      var result = json;
      result = this.parseIt(result, this.config);
      App.store.safeLoad(this.get('model'), result);
      var cluster = App.Cluster.find(result.id);
      var clusterDesiredConfigs = [];
      // Create desired_configs_array
      if (json.Clusters.desired_configs) {
        for (var site in json.Clusters.desired_configs) {
          var tag = json.Clusters.desired_configs[site].tag;
          var configObj = App.ConfigSiteTag.create({
            site: site,
            tag: tag,
            hostOverrides: {}
          });
          if (json.Clusters.desired_configs[site].host_overrides != null) {
            var hostOverridesArray = {};
            json.Clusters.desired_configs[site].host_overrides.forEach(function (override) {
              var hostname = override.host_name;
              hostOverridesArray[hostname] = override.tag;
            });
            configObj.set('hostOverrides', hostOverridesArray);
          }
          clusterDesiredConfigs.push(configObj);
        }
      }
      cluster.set('desiredConfigs', clusterDesiredConfigs);
    }
  },
  config: {
    id: 'Clusters.cluster_id',
    cluster_name: 'Clusters.cluster_name',
    stack_name: 'Clusters.stack_name',
    version: 'Clusters.version',
    security_type: 'Clusters.security_type',
    total_hosts: 'Clusters.total_hosts',
    credential_store_properties: 'Clusters.credential_store_properties'
  }
});

});

require.register("mappers/components_state_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.componentsStateMapper = App.QuickDataMapper.create({

  model: App.Service,

  clientModel: App.ClientComponent,
  clientMap: {
    id: 'ServiceComponentInfo.component_name',
    service_id: 'ServiceComponentInfo.service_name',
    stack_info_id: 'ServiceComponentInfo.component_name',
    component_name: 'ServiceComponentInfo.component_name',
    display_name: 'ServiceComponentInfo.display_name',
    service_name: 'ServiceComponentInfo.service_name',
    installed_count: 'ServiceComponentInfo.installed_count',
    install_failed_count: 'ServiceComponentInfo.install_failed_count',
    init_count: 'ServiceComponentInfo.init_count',
    unknown_count: 'ServiceComponentInfo.unknown_count',
    started_count: 'ServiceComponentInfo.started_count',
    total_count: 'ServiceComponentInfo.total_count',
    host_names: 'host_names',
    stale_config_hosts: 'stale_config_hosts'
  },

  slaveModel: App.SlaveComponent,
  masterModel: App.MasterComponent,

  paths: {
    INSTALLED_PATH: 'ServiceComponentInfo.installed_count',
    INSTALL_FAILED_PATH: 'ServiceComponentInfo.install_failed_count',
    INIT_PATH: 'ServiceComponentInfo.init_count',
    UNKNOWN_PATH: 'ServiceComponentInfo.unknown_count',
    STARTED_PATH: 'ServiceComponentInfo.started_count',
    TOTAL_PATH: 'ServiceComponentInfo.total_count'
  },
  configMap: {
    'DATANODE': {
      data_nodes_started: 'STARTED_PATH',
      data_nodes_installed: 'INSTALLED_PATH',
      data_nodes_total: 'TOTAL_PATH'
    },
    'NFS_GATEWAY': {
      nfs_gateways_started: 'STARTED_PATH',
      nfs_gateways_installed: 'INSTALLED_PATH',
      nfs_gateways_total: 'TOTAL_PATH'
    },
    'NODEMANAGER': {
      node_managers_started: 'STARTED_PATH',
      node_managers_installed: 'INSTALLED_PATH',
      node_managers_total: 'TOTAL_PATH'
    },
    'HAWQSEGMENT': {
      hawq_segments_started: 'STARTED_PATH',
      hawq_segments_installed: 'INSTALLED_PATH',
      hawq_segments_total: 'TOTAL_PATH'
    },
    'PXF': {
      pxfs_started: 'STARTED_PATH',
      pxfs_installed: 'INSTALLED_PATH',
      pxfs_total: 'TOTAL_PATH'
    },
    'HBASE_REGIONSERVER': {
      region_servers_started: 'STARTED_PATH',
      region_servers_installed: 'INSTALLED_PATH',
      region_servers_total: 'TOTAL_PATH'
    },
    'PHOENIX_QUERY_SERVER': {
      phoenix_servers_started: 'STARTED_PATH',
      phoenix_servers_installed: 'INSTALLED_PATH',
      phoenix_servers_total: 'TOTAL_PATH'
    },
    'GANGLIA_MONITOR': {
      ganglia_monitors_started: 'STARTED_PATH',
      ganglia_monitors_installed: 'INSTALLED_PATH',
      ganglia_monitors_total: 'TOTAL_PATH'
    },
    'SUPERVISOR': {
      super_visors_started: 'STARTED_PATH',
      super_visors_installed: 'INSTALLED_PATH',
      super_visors_total: 'TOTAL_PATH'
    },
    'RANGER_TAGSYNC': {
      ranger_tagsyncs_started: 'STARTED_PATH',
      ranger_tagsyncs_installed: 'INSTALLED_PATH',
      ranger_tagsyncs_total: 'TOTAL_PATH'
    },
    'MAPREDUCE2_CLIENT': {
      map_reduce2_clients: 'INSTALLED_PATH'
    },
    'TEZ_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'HIVE_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'FALCON_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'OOZIE_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'ZOOKEEPER_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'PIG': {
      installed_clients: 'INSTALLED_PATH'
    },
    'SQOOP': {
      installed_clients: 'INSTALLED_PATH'
    },
    'YARN_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'HDFS_CLIENT': {
      installed_clients: 'INSTALLED_PATH'
    },
    'FLUME_HANDLER': {
      flume_handlers_total: 'TOTAL_PATH'
    }
  },
  /**
   * get formatted component config
   * @param componentName
   * @return {Object}
   */
  getComponentConfig: function getComponentConfig(componentName) {
    var config = {};
    var componentConfig = this.get('configMap')[componentName];
    var paths = this.get('paths');

    for (var property in componentConfig) {
      if (paths[componentConfig[property]]) {
        config[property] = paths[componentConfig[property]];
      }
    }
    return config;
  },
  /**
   * get service extended model if it has one
   * @param serviceName
   * @return {Object|null}
   */
  getExtendedModel: function getExtendedModel(serviceName) {
    if (App[App.Service.extendedModel[serviceName]]) {
      return App[App.Service.extendedModel[serviceName]].find(serviceName);
    }
    return null;
  },

  map: function map(json) {
    console.time('App.componentsStateMapper execution time');

    var clients = [];
    var slaves = [];
    var masters = [];
    var hasNewComponents = false;
    var staleConfigHostsMap = App.cache.staleConfigsComponentHosts;

    if (json.items) {
      if (!App.isEmptyObject(Em.getWithDefault(App, 'cache.services', {}))) {
        hasNewComponents = json.items.mapProperty('ServiceComponentInfo.total_count').reduce(Em.sum, 0) > App.cache.services.mapProperty('host_components.length').reduce(Em.sum, 0);
      }
      json.items.forEach(function (item) {
        var cacheService = App.cache['services'].findProperty('ServiceInfo.service_name', item.ServiceComponentInfo.service_name);

        item.stale_config_hosts = staleConfigHostsMap[item.ServiceComponentInfo.component_name] || [];
        if (item.ServiceComponentInfo.category === 'CLIENT') {
          item.host_names = item.host_components.mapProperty('HostRoles.host_name');
          clients.push(this.parseIt(item, this.clientMap));
        } else if (item.ServiceComponentInfo.category === 'SLAVE') {
          // for now map for slaves and clients are equal but it may vary in future.
          item.host_names = item.host_components.mapProperty('HostRoles.host_name');
          slaves.push(this.parseIt(item, this.clientMap));
        } else if (item.ServiceComponentInfo.category === 'MASTER') {
          item.host_names = item.host_components.mapProperty('HostRoles.host_name');
          masters.push(this.parseIt(item, this.clientMap));
        }
        if (cacheService) {
          cacheService.client_components = clients.filterProperty('service_name', cacheService.ServiceInfo.service_name).mapProperty('component_name');
          cacheService.slave_components = slaves.filterProperty('service_name', cacheService.ServiceInfo.service_name).mapProperty('component_name');
          cacheService.master_components = masters.filterProperty('service_name', cacheService.ServiceInfo.service_name).mapProperty('component_name');
        }

        this.mapExtendedModelComponents(item);
      }, this);
    }
    App.store.safeLoadMany(this.clientModel, clients);
    App.store.safeLoadMany(this.slaveModel, slaves);
    App.store.safeLoadMany(this.masterModel, masters);

    if (hasNewComponents) {
      App.get('router.clusterController').triggerQuickLinksUpdate();
    }

    console.timeEnd('App.componentsStateMapper execution time');
  },

  /**
   *
   * @param {object} item
   */
  mapExtendedModelComponents: function mapExtendedModelComponents(item) {
    var serviceName = item.ServiceComponentInfo.service_name;
    var service = App.Service.find(serviceName);
    var cacheService = App.cache['services'].findProperty('ServiceInfo.service_name', serviceName);
    var extendedModel = this.getExtendedModel(serviceName);
    var parsedItem = this.parseIt(item, this.getComponentConfig(item.ServiceComponentInfo.component_name));

    if (cacheService) {
      for (var i in parsedItem) {
        if (service.get('isLoaded')) {
          cacheService[i] = parsedItem[i];
          service.set(stringUtils.underScoreToCamelCase(i), parsedItem[i]);
          if (extendedModel && extendedModel.get('isLoaded')) {
            extendedModel.set(stringUtils.underScoreToCamelCase(i), parsedItem[i]);
          }
        }
      }
    }
  },

  /**
   *
   * @param {string} componentName
   * @param {Array} hosts
   */
  updateStaleConfigsHosts: function updateStaleConfigsHosts(componentName, hosts) {
    var staleConfigHostsMap = App.cache.staleConfigsComponentHosts;
    var model = App.ClientComponent.getModelByComponentName(componentName);

    staleConfigHostsMap[componentName] = hosts;
    if (model) {
      model.set('staleConfigHosts', hosts);
    }
  },

  /**
   * used to synchronize updates from websocket with updates from REST API
   * @param {App.ClientComponent} model
   * @returns {{ServiceComponentInfo: {component_name, service_name, installed_count, install_failed_count, init_count, unknown_count, started_count, total_count}}}
   */
  componentStateToJSON: function componentStateToJSON(model) {
    return {
      ServiceComponentInfo: {
        component_name: model.get('componentName'),
        service_name: model.get('service.serviceName'),
        installed_count: model.get('installedCount'),
        install_failed_count: model.get('installFailedCount'),
        init_count: model.get('initCount'),
        unknown_count: model.get('unknownCount'),
        started_count: model.get('startedCount'),
        total_count: model.get('totalCount')
      }
    };
  },

  /**
   * @param {object} componentState
   */
  updateComponentCountOnStateChange: function updateComponentCountOnStateChange(componentState) {
    var model = App.ClientComponent.getModelByComponentName(componentState.componentName);
    var acceptedStates = ['STARTED', 'INSTALLED', 'INIT', 'UNKNOWN', 'INSTALL_FAILED'];

    if (model && model.get('isLoaded') && componentState.currentState) {
      if (acceptedStates.contains(componentState.previousState)) {
        var previousStateProp = this.statusToProperty(componentState.previousState);
        model.set(previousStateProp, model.get(previousStateProp) - 1);
      }

      if (acceptedStates.contains(componentState.currentState)) {
        var currentStateProp = this.statusToProperty(componentState.currentState);
        model.set(currentStateProp, model.get(currentStateProp) + 1);
      }
      this.mapExtendedModelComponents(this.componentStateToJSON(model));
    }
  },

  /**
   * @param {object} componentState
   */
  updateComponentCountOnDelete: function updateComponentCountOnDelete(componentState) {
    var model = App.ClientComponent.getModelByComponentName(componentState.componentName);
    var acceptedStates = ['STARTED', 'INSTALLED', 'INIT', 'UNKNOWN', 'INSTALL_FAILED'];

    if (model) {
      if (acceptedStates.contains(componentState.lastComponentState)) {
        var currentStateProp = this.statusToProperty(componentState.lastComponentState);
        model.set(currentStateProp, model.get(currentStateProp) - 1);
      }
      model.set('totalCount', model.get('totalCount') - 1);
      this.mapExtendedModelComponents(this.componentStateToJSON(model));
    }
  },

  /**
   * @param {object} componentState
   */
  updateComponentCountOnCreate: function updateComponentCountOnCreate(componentState) {
    var model = App.ClientComponent.getModelByComponentName(componentState.componentName);

    if (model && model.get('isLoaded')) {
      model.set('initCount', model.get('initCount') + 1);
      model.set('totalCount', model.get('totalCount') + 1);
      this.mapExtendedModelComponents(this.componentStateToJSON(model));
    }
  },

  /**
   *
   * @param {string} status
   * @returns {string}
   */
  statusToProperty: function statusToProperty(status) {
    return stringUtils.underScoreToCamelCase(status.toLowerCase() + "_count");
  }
});

});

require.register("mappers/configs/config_groups_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

/**
 * THIS IS NOT USED FOR NOW
 * FOR CONFIG GROUPS WE ARE USING OLD MODELS AND LOGIC
 */

var App = require('app');

App.configGroupsMapper = App.QuickDataMapper.create({
  model: App.ServiceConfigGroup,
  config: {
    id: 'ConfigGroup.id',
    name: 'ConfigGroup.group_name',
    service_name: 'ConfigGroup.tag',
    description: 'ConfigGroup.description',
    hosts: 'hosts',
    service_id: 'ConfigGroup.tag',
    desired_configs: 'ConfigGroup.desired_configs'
  },

  /**
   * using this config when saving group from config_version api
   */
  config2: {
    id: 'group_id',
    name: 'group_name',
    service_name: 'service_name',
    hosts: 'hosts',
    service_id: 'service_name'
  },

  map: function map(json, mapFromVersions, serviceNames) {
    console.time('App.configGroupsMapper');
    if (serviceNames && serviceNames.length > 0) {
      var configGroups = [];

      /**
       * ex: { "HDFS": ["host1", "host2"], "YARN": ["host1"] }
       * this property is used to store host names for default config group.
       * While parsing data for not default groups host names will be excluded from this list.
       * In case there is no not default config groups for some service <code>hostNamesForService<code>
       * will not contain property for this service which mean all host belongs to default group
       */
      var hostNamesForService = {};
      var configGroupsForService = {};

      if (json && json.items) {
        json.items.forEach(function (configGroup) {
          if (![App.ServiceConfigGroup.defaultGroupName, App.ServiceConfigGroup.deletedGroupName].contains(configGroup.group_name)) {
            if (mapFromVersions) {
              configGroup.id = App.ServiceConfigGroup.groupId(configGroup.service_name, configGroup.group_name);
            } else {
              configGroup.id = App.ServiceConfigGroup.groupId(configGroup.ConfigGroup.tag, configGroup.ConfigGroup.group_name);
              configGroup.hosts = configGroup.ConfigGroup.hosts.mapProperty('host_name').sort();
              configGroup.service_name = configGroup.ConfigGroup.tag;
            }

            /**
             * creating (if not exists) field in <code>hostNamesForService<code> with host names for default group
             */
            if (!hostNamesForService[configGroup.service_name]) {
              hostNamesForService[configGroup.service_name] = App.get('allHostNames').slice(0);
            }

            if (!configGroupsForService[configGroup.service_name]) {
              configGroupsForService[configGroup.service_name] = [configGroup.id];
            }
            configGroupsForService[configGroup.service_name].push(configGroup.id);

            /**
             * excluding host names that belongs for current config group from default group
             */
            configGroup.hosts.forEach(function (host) {
              hostNamesForService[configGroup.service_name].splice(hostNamesForService[configGroup.service_name].indexOf(host), 1);
            });
            configGroup = this.parseIt(configGroup, mapFromVersions ? this.get('config2') : this.get('config'));
            configGroup.parent_config_group_id = App.ServiceConfigGroup.getParentConfigGroupId(configGroup.service_name);
            configGroups.push(configGroup);
          }
        }, this);
      }

      /**
       * generating default config groups
       */
      serviceNames.forEach(function (serviceName) {
        configGroups.push(this.generateDefaultGroup(serviceName, hostNamesForService[serviceName], configGroupsForService[serviceName]));
      }, this);

      configGroups.sort(function (configGroupA, configGroupB) {
        return configGroupA.is_default || configGroupA.name > configGroupB.name;
      });
      App.store.safeLoadMany(this.get('model'), configGroups);
    }
    console.timeEnd('App.configGroupsMapper');
  },

  /**
   * Get array with all hosts
   *
   * @returns {String[]}
   * @private
   */
  _getAllHosts: function _getAllHosts() {
    var hosts;
    try {
      hosts = App.get('allHostNames.length') ? App.get('allHostNames') : Object.keys(App.get('router.installerController.content.hosts'));
    } catch (e) {
      hosts = [];
    }
    return hosts;
  },

  /**
   * generate mock object for default config group
   * @param {string} serviceName
   * @param {string[]} [hostNames=null]
   * @param {Array} childConfigGroups
   * @returns {{id: string, name: string, service_name: string, description: string, host_names: [string], service_id: string, is_default: boolean}}
   */
  generateDefaultGroup: function generateDefaultGroup(serviceName, hostNames, childConfigGroups) {
    return {
      id: App.ServiceConfigGroup.getParentConfigGroupId(serviceName),
      name: 'Default',
      service_name: serviceName,
      description: 'Default cluster level ' + App.format.role(serviceName, true) + ' configuration',
      hosts: hostNames ? hostNames.slice() : this._getAllHosts().slice(),
      child_config_groups: childConfigGroups ? childConfigGroups.uniq() : [],
      service_id: serviceName,
      desired_configs: [],
      properties: [],
      is_default: true
    };
  }
});

});

require.register("mappers/configs/service_config_version_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.serviceConfigVersionsMapper = App.QuickDataMapper.create({
  model: App.ServiceConfigVersion,
  config: {
    service_name: 'service_name',
    service_id: 'service_name',
    version: "service_config_version",
    create_time: 'createtime',
    raw_create_time: 'createtime',
    group_id: 'group_id',
    group_name: 'group_name',
    hosts: 'hosts',
    author: 'user',
    notes: 'service_config_version_note',
    is_current: 'is_current',
    index: 'index',
    stack_version: 'stack_id',
    is_compatible: 'is_cluster_compatible'
  },
  map: function map(json) {
    console.time('App.serviceConfigVersionsMapper');
    var result = [];
    var itemIds = {};
    var serviceToHostMap = {};

    if (json && json.items) {
      var currentVersionsMap = this.getCurrentVersionMap();

      json.items.forEach(function (item, index) {
        var parsedItem = this.parseIt(item, this.get('config'));
        parsedItem.id = this.makeId(parsedItem.service_name, parsedItem.version);
        parsedItem.group_id = parsedItem.group_id === -1 ? parsedItem.service_name + '_default' : parsedItem.group_id;
        parsedItem.is_requested = true;
        parsedItem.create_time = App.dateTimeWithTimeZone(parsedItem.create_time);
        itemIds[parsedItem.id] = true;
        parsedItem.index = index;
        if (serviceToHostMap[item.service_name]) {
          serviceToHostMap[item.service_name] = serviceToHostMap[item.service_name].concat(item.hosts);
        } else {
          serviceToHostMap[item.service_name] = item.hosts;
        }

        // if loaded only latest versions(later than current), then current version should be reset
        if (parsedItem.is_current && currentVersionsMap[parsedItem.service_name + "_" + parsedItem.group_name]) {
          currentVersionsMap[parsedItem.service_name + "_" + parsedItem.group_name].set('isCurrent', false);
        }
        result.push(parsedItem);
      }, this);

      var itemTotal = parseInt(json.itemTotal);
      if (!isNaN(itemTotal)) {
        App.router.set('mainConfigHistoryController.filteredCount', itemTotal);
      }

      this.setHostsForDefaultCG(serviceToHostMap, result);

      // If on config history page, need to clear the model
      if (App.router.get('currentState.name') === 'configHistory') {
        this.get('model').find().clear();
      }
      App.store.safeLoadMany(this.get('model'), result);
      console.timeEnd('App.serviceConfigVersionsMapper');
    }
  },

  /**
   * sets hostNames for default config group by excluding hostNames
   * that belongs to not default groups from list of all hosts
   * @param {object} serviceToHostMap
   * @param {array} result
   */
  setHostsForDefaultCG: function setHostsForDefaultCG(serviceToHostMap, result) {
    Object.keys(serviceToHostMap).forEach(function (sName) {
      var defaultHostNames = App.get('allHostNames');
      for (var i = 0; i < serviceToHostMap[sName].length; i++) {
        defaultHostNames = defaultHostNames.without(serviceToHostMap[sName][i]);
      }
      var defVer = result.find(function (v) {
        return v.is_current && v.group_name === App.ServiceConfigGroup.defaultGroupName && v.service_name === sName;
      });
      if (defVer) {
        defVer.hosts = defaultHostNames;
      }
    });
  },

  getCurrentVersionMap: function getCurrentVersionMap() {
    var currentVersionsMap = {};
    App.ServiceConfigVersion.find().forEach(function (v) {
      if (v.get('isCurrent')) {
        currentVersionsMap[v.get('serviceName') + "_" + v.get('groupName')] = v;
      }
    });
    return currentVersionsMap;
  },

  /**
   * Conventional method to generate id for instances of <code>App.ServiceConfigVersion</code> model.
   *
   * @param {String} serviceName
   * @param {Number|String} version
   * @returns {String}
   */
  makeId: function makeId(serviceName, version) {
    return serviceName + '_' + version;
  }

});

});

require.register("mappers/configs/stack_config_properties_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.stackConfigPropertiesMapper = App.QuickDataMapper.create({
  /**
   * this is map for configs that will be stored as plan objects
   */
  configToPlain: {
    id: 'id',
    name: 'StackConfigurations.property_name',
    displayName: 'StackConfigurations.property_display_name',
    fileName: 'StackConfigurations.type',
    filename: 'StackConfigurations.type',
    description: 'StackConfigurations.property_description',
    value: 'StackConfigurations.property_value',
    recommendedValue: 'StackConfigurations.property_value',
    serviceName: 'StackConfigurations.service_name',
    stackName: 'StackConfigurations.stack_name',
    stackVersion: 'StackConfigurations.stack_version',
    isOverridable: 'StackConfigurations.property_value_attributes.overridable',
    isVisible: 'StackConfigurations.property_value_attributes.visible',
    showLabel: 'StackConfigurations.property_value_attributes.show_property_name',
    displayType: 'StackConfigurations.property_value_attributes.type',
    unit: 'StackConfigurations.property_value_attributes.unit',

    isRequired: 'is_required',
    isReconfigurable: 'is_reconfigurable',
    isEditable: 'is_editable',
    isRequiredByAgent: 'is_required_by_agent',

    isFinal: 'recommended_is_final',
    recommendedIsFinal: 'recommended_is_final',
    supportsFinal: 'supports_final',

    propertyDependedBy: 'StackConfigurations.property_depended_by',
    propertyDependsOn: 'StackConfigurations.property_depends_on',
    valueAttributes: 'StackConfigurations.property_value_attributes',

    /**** ui properties, ib future should be moved to BE ***/
    category: 'category',
    index: 'index',
    /*** temp properties until all radio buttons type will be moved to BE **/
    radioName: 'radioName',
    options: 'options',
    /*** temp property until visibility dependencies will be retrieved from stack **/
    dependentConfigPattern: 'dependentConfigPattern'
  },

  map: function map(json) {
    console.time('App.stackConfigPropertiesMapper execution time');
    if (json && json.Versions) {
      //hack for cluster versions
      json = { items: [json] };
      var clusterConfigs = true;
    }
    if (json && json.items) {
      var configs = [];
      json.items.forEach(function (stackItem) {
        var configTypeInfo = clusterConfigs ? Em.get(stackItem, 'Versions.config_types') : Em.get(stackItem, 'StackServices.config_types');

        stackItem.configurations.forEach(function (config) {
          if (clusterConfigs) {
            config.StackConfigurations = config.StackLevelConfigurations;
          }
          var configType = App.config.getConfigTagFromFileName(config.StackConfigurations.type);
          config.id = App.config.configId(config.StackConfigurations.property_name, configType);
          config.recommended_is_final = config.StackConfigurations.final === "true";
          config.supports_final = !!configTypeInfo[configType] && configTypeInfo[configType].supports.final === "true";

          var attributes = config.StackConfigurations.property_value_attributes;
          if (attributes) {
            config.is_required = this._isRequired(attributes.empty_value_valid, config.StackConfigurations.property_value);
            config.is_reconfigurable = !(attributes.editable_only_at_install || config.StackConfigurations.type === 'cluster-env.xml');
            config.is_editable = !attributes.read_only;
            config.is_required_by_agent = !attributes.ui_only_property;
          }

          if (!config.StackConfigurations.property_display_name) {
            config.StackConfigurations.property_display_name = config.StackConfigurations.property_name;
          }

          if (!config.StackConfigurations.service_name) {
            config.StackConfigurations.service_name = 'MISC';
          }

          if (!attributes || !attributes.type) {
            if (!attributes) {
              config.StackConfigurations.property_value_attributes = {};
            }
            config.StackConfigurations.property_value_attributes.type = App.config.getDefaultDisplayType(config.StackConfigurations.property_value);
          }
          // Map from /dependencies to property_depended_by
          config.StackConfigurations.property_depended_by = [];
          if (config.dependencies && config.dependencies.length > 0) {
            config.dependencies.forEach(function (dep) {
              config.StackConfigurations.property_depended_by.push({
                type: dep.StackConfigurationDependency.dependency_type,
                name: dep.StackConfigurationDependency.dependency_name
              });
              var service = App.StackService.find(config.StackConfigurations.service_name);
              var dependentService = App.config.get('serviceByConfigTypeMap')[dep.StackConfigurationDependency.dependency_type];
              if (dependentService && service && dependentService.get('serviceName') != service.get('serviceName') && !service.get('dependentServiceNames').contains(dependentService.get('serviceName'))) {
                service.set('dependentServiceNames', service.get('dependentServiceNames').concat(dependentService.get('serviceName')));
              }
            });
          }
          if (Em.get(config, 'StackConfigurations.property_depends_on.length') > 0) {
            config.StackConfigurations.property_depends_on.forEach(function (dep) {
              var service = App.StackService.find(config.StackConfigurations.service_name);
              var dependentService = App.config.get('serviceByConfigTypeMap')[dep.type];
              if (dependentService && service && dependentService.get('serviceName') != service.get('serviceName') && !service.get('dependentServiceNames').contains(dependentService.get('serviceName'))) {
                service.set('dependentServiceNames', service.get('dependentServiceNames').concat(dependentService.get('serviceName')));
              }
            });
          }
          /**
           * merging stack info with that is stored on UI
           * for now is not used; uncomment in will be needed
           * this.mergeWithUI(config);
           */
          if (this.isMiscService(config.StackConfigurations.property_type)) {
            this.handleSpecialProperties(config);
          } else {
            this.mergeWithUI(config);
          }

          var staticConfigInfo = this.parseIt(config, this.get('configToPlain'));
          var v = Em.isNone(staticConfigInfo.recommendedValue) ? staticConfigInfo.recommendedValue : staticConfigInfo.value;
          staticConfigInfo.value = staticConfigInfo.recommendedValue = App.config.formatPropertyValue(staticConfigInfo, v);
          staticConfigInfo.isSecureConfig = App.config.getIsSecure(staticConfigInfo.name);
          staticConfigInfo.description = App.config.getDescription(staticConfigInfo.description, staticConfigInfo.displayType);
          staticConfigInfo.name = JSON.parse('"' + staticConfigInfo.name + '"');
          staticConfigInfo.isUserProperty = false;
          if (Em.isNone(staticConfigInfo.index)) {
            staticConfigInfo.index = Infinity;
          }
          App.configsCollection.add(staticConfigInfo);
        }, this);
      }, this);
      this.addUIOnlyProperties(configs);
    }
    console.timeEnd('App.stackConfigPropertiesMapper execution time');
  },

  /******************* METHODS TO MERGE STACK PROPERTIES WITH STORED ON UI *********************************/

  /**
   * Config should not be required if value from stack is null
   *
   * @param allowEmpty
   * @param propertyValue
   * @returns {*|boolean}
   * @private
   */
  _isRequired: function _isRequired(allowEmpty, propertyValue) {
    return !allowEmpty && !Em.isNone(propertyValue);
  },

  /**
   * find UI config with current name and fileName
   * if there is such property - adds some info to config object
   * @param {Object} config
   * @method mergeWithUI
   */
  mergeWithUI: function mergeWithUI(config) {
    var c = config.StackConfigurations;
    var uiConfigProperty = this.getUIConfig(c.property_name, c.type);

    config.category = uiConfigProperty && uiConfigProperty.category ? uiConfigProperty.category : App.config.getDefaultCategory(true, c.type);

    if (uiConfigProperty) {
      config.index = uiConfigProperty.index !== undefined ? uiConfigProperty.index : Infinity;
      if (uiConfigProperty.displayType) {
        c.property_value_attributes.type = uiConfigProperty.displayType;
        config.radioName = uiConfigProperty.radioName;
        config.options = uiConfigProperty.options;
      }
      config.dependentConfigPattern = uiConfigProperty.dependentConfigPattern;
    }
  },

  /**
   *
   * @param config
   */
  handleSpecialProperties: function handleSpecialProperties(config) {
    var types = config.StackConfigurations.property_type;
    if (!types.contains('ADDITIONAL_USER_PROPERTY')) {
      config.index = App.StackService.displayOrder.indexOf(config.StackConfigurations.service_name) + 1 || 30;
    }
    // displayType from stack ignored, cause UID and GID should be shown along with service's user config
    if (types.contains('UID') || types.contains('GID')) {
      config.StackConfigurations.property_value_attributes.type = 'uid_gid';
    }
    config.StackConfigurations.service_name = 'MISC';
    config.category = 'Users and Groups';
  },

  /**
   * defines if property should refer to MISC tab
   * @param type
   * @returns {Boolean}
   */
  isMiscService: function isMiscService(type) {
    return type.length && (type.contains('USER') || type.contains('GROUP') || type.contains('ADDITIONAL_USER_PROPERTY') || type.contains('UID') || type.contains('GID'));
  },

  /**
   * add properties that doesn't have any info on stack definition
   * @param configs
   */
  addUIOnlyProperties: function addUIOnlyProperties(configs) {
    require('data/configs/ui_properties').concat(require('data/configs/alert_notification')).forEach(function (p) {
      if (p.name == "dfs.ha.fencing.methods" && !App.get('isHaEnabled')) return;

      configs.push({
        id: App.config.configId(p.name, p.filename),
        name: p.name,
        display_name: p.displayName,
        file_name: p.filename,
        description: p.description || '',
        is_required_by_agent: p.isRequiredByAgent !== false, // by default is_required_by_agent should be true
        service_name: p.serviceName,
        supports_final: false,
        category: p.category,
        index: p.index,
        stack_name: App.get('currentStackName'),
        stack_version: App.get('currentStackVersionNumber')
      });
      p.id = App.config.configId(p.name, p.filename);
      App.configsCollection.add(p);
    });
  },
  /**
   * returns config with such name and fileName if there is such on UI
   * otherwise returns null
   * @param propertyName
   * @param siteName
   * @returns {Object|null}
   * @method getUIConfig
   */
  getUIConfig: function getUIConfig(propertyName, siteName) {
    return App.config.get('preDefinedSitePropertiesMap')[App.config.configId(propertyName, siteName)];
  }
});

});

require.register("mappers/configs/themes_mapper", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require("app");

App.themesMapper = App.QuickDataMapper.create({
  tabModel: App.Tab,
  sectionModel: App.Section,
  subSectionModel: App.SubSection,
  subSectionTabModel: App.SubSectionTab,
  themeConditionModel: App.ThemeCondition,

  tabConfig: {
    "id": "name",
    "name": "name",
    "display_name": "display-name",
    "columns": "layout.tab-columns",
    "rows": "layout.tab-rows",
    "service_name": "service_name",
    "sections_key": "sections"
  },

  sectionConfig: {
    "name": "name",
    "display_name": "display-name",
    "row_index": "row-index",
    "column_index": "column-index",
    "row_span": "row-span",
    "column_span": "column-span",
    "section_columns": "section-columns",
    "section_rows": "section-rows",
    "tab_id": "tab_id"
  },

  subSectionConfig: {
    "name": "name",
    "display_name": "display-name",
    "border": "border",
    "row_index": "row-index",
    "column_index": "column-index",
    "column_span": "column-span",
    "row_span": "row-span",
    "configProperties": "config_properties",
    "section_id": "section_id",
    "depends_on": "depends-on",
    "left_vertical_splitter": "left-vertical-splitter"
  },

  subSectionTabConfig: {
    "id": "name",
    "name": "name",
    "display_name": "display-name",
    "depends_on": "depends-on",
    "sub_section_id": "sub_section_id"
  },

  resetModels: function resetModels() {
    this.get('tabModel').find().clear();
    this.get('sectionModel').find().clear();
    this.get('subSectionModel').find().clear();
    this.get('subSectionTabModel').find().clear();
    this.get('themeConditionModel').find().clear();
    App.clearModels([this.get('tabModel')]);
    App.resetDsStoreTypeMap(this.get('tabModel'));
  },

  /**
   * Mapper function for tabs
   *
   * @param json
   * @param [serviceNames]
   */
  map: function map(json, serviceNames) {
    console.time('App.themesMapper execution time');
    var tabs = [];
    json.items.forEach(function (item) {
      this.mapThemeLayouts(item, tabs);
      this.mapThemeConfigs(item);
      this.mapThemeWidgets(item);
    }, this);

    App.store.safeLoadMany(this.get("tabModel"), tabs);
    this.generateAdvancedTabs(serviceNames);
    console.timeEnd('App.themesMapper execution time');
  },

  /**
   * Bootstrap tab objects and link sections with subsections.
   *
   * @param {Object} json - json to parse
   * @param {Object[]} tabs - tabs list
   */
  mapThemeLayouts: function mapThemeLayouts(json, tabs) {
    var serviceName = Em.get(json, "ThemeInfo.service_name");
    var themeName = Em.get(json, "ThemeInfo.theme_data.Theme.name");
    Em.getWithDefault(json, "ThemeInfo.theme_data.Theme.configuration.layouts", []).forEach(function (layout) {
      if (layout.tabs) {
        layout.tabs.forEach(function (tab) {
          var parsedTab = this.parseIt(tab, this.get("tabConfig"));
          parsedTab.id = serviceName + "_" + tab.name;
          parsedTab.service_name = serviceName;
          parsedTab.theme_name = themeName;

          if (Em.get(tab, "layout.sections")) {
            var sections = [];
            Em.get(tab, "layout.sections").forEach(function (section) {
              var parsedSection = this.parseIt(section, this.get("sectionConfig"));
              parsedSection.id = section.name + '_' + parsedTab.id;
              parsedSection.tab_id = parsedTab.id;
              this.loadSubSections(section, parsedSection, serviceName, themeName);
              sections.push(parsedSection);
            }, this);

            App.store.safeLoadMany(this.get("sectionModel"), sections);
            parsedTab.sections = sections.mapProperty("id");
          }

          tabs.push(parsedTab);
        }, this);
      }
    }, this);
  },

  loadSubSections: function loadSubSections(section, parsedSection, serviceName, themeName) {
    if (section.subsections) {
      var subSections = [];
      var subSectionConditions = [];
      section.subsections.forEach(function (subSection) {
        var parsedSubSection = this.parseIt(subSection, this.get("subSectionConfig"));
        parsedSubSection.section_id = parsedSection.id;
        parsedSubSection.id = parsedSubSection.name + '_' + serviceName + '_' + themeName;
        parsedSubSection.theme_name = themeName;

        this.loadSubSectionTabs(subSection, parsedSubSection, themeName);
        if (parsedSubSection['depends_on']) {
          subSectionConditions.push(parsedSubSection);
        }
        subSections.push(parsedSubSection);
      }, this);
      if (subSectionConditions.length) {
        var type = 'subsection';
        this.mapThemeConditions(subSectionConditions, type, themeName);
      }
      App.store.safeLoadMany(this.get("subSectionModel"), subSections);
      parsedSection.sub_sections = subSections.mapProperty("id");
    }
  },

  loadSubSectionTabs: function loadSubSectionTabs(subSection, parsedSubSection, themeName) {
    if (subSection['subsection-tabs']) {
      var subSectionTabs = [];
      var subSectionTabConditions = [];

      subSection['subsection-tabs'].forEach(function (subSectionTab) {
        var parsedSubSectionTab = this.parseIt(subSectionTab, this.get("subSectionTabConfig"));
        parsedSubSectionTab.sub_section_id = parsedSubSection.id;
        parsedSubSectionTab.theme_name = themeName;
        if (parsedSubSectionTab['depends_on']) {
          subSectionTabConditions.push(parsedSubSectionTab);
        }
        subSectionTabs.push(parsedSubSectionTab);
      }, this);
      subSectionTabs[0].is_active = true;
      if (subSectionTabConditions.length) {
        var type = 'subsectionTab';
        this.mapThemeConditions(subSectionTabConditions, type, themeName);
      }
      App.store.safeLoadMany(this.get("subSectionTabModel"), subSectionTabs);
      parsedSubSection.sub_section_tabs = subSectionTabs.mapProperty("id");
    }
  },

  /**
   * create tie between <code>stackConfigProperty<code> and <code>subSection<code>
   *
   * @param {Object} json - json to parse
   */
  mapThemeConfigs: function mapThemeConfigs(json) {
    var serviceName = Em.get(json, "ThemeInfo.service_name");
    var themeName = Em.get(json, "ThemeInfo.theme_data.Theme.name");
    Em.getWithDefault(json, "ThemeInfo.theme_data.Theme.configuration.placement.configs", []).forEach(function (configLink) {
      var configId = this.getConfigId(configLink);
      var subSectionId = configLink["subsection-name"] ? configLink["subsection-name"] + '_' + serviceName + '_' + themeName : false;
      var subSectionTabId = configLink["subsection-tab-name"];
      var configProperty = App.configsCollection.getConfig(configId);
      var dependsOnConfigs = configLink["depends-on"] || [];

      if (subSectionTabId) {
        var subSectionTab = App.SubSectionTab.find(subSectionTabId);
      } else if (subSectionId) {
        var subSection = App.SubSection.find(subSectionId);
      }

      if (configProperty && subSection && subSection.get('isLoaded')) {
        if (!subSection.get('configProperties').contains(configProperty.id)) {
          subSection.set('configProperties', subSection.get('configProperties').concat(configProperty.id));
        }
      } else if (configProperty && subSectionTab && subSectionTab.get('isLoaded')) {
        if (!subSectionTab.get('configProperties').contains(configProperty.id)) {
          subSectionTab.set('configProperties', subSectionTab.get('configProperties').concat(configProperty.id));
        }
      } else {
        configProperty = this.getConfigByAttributes(configProperty, configLink, subSection, subSectionTab, serviceName);
      }

      // map all the configs which conditionally affect the value attributes of a config
      if (dependsOnConfigs && dependsOnConfigs.length && configProperty) {
        this.mapThemeConfigConditions(dependsOnConfigs, configProperty);
      }
    }, this);
  },

  getConfigByAttributes: function getConfigByAttributes(configProperty, configLink, subSection, subSectionTab, serviceName) {
    var valueAttributes = configLink["property_value_attributes"];
    if (valueAttributes) {
      var isUiOnlyProperty = valueAttributes["ui_only_property"];
      var isCopy = valueAttributes["copy"] || '';
      // UI only configs are mentioned in the themes for supporting widgets that is not intended for setting a value
      // And thus is affiliated with fake config property termed as ui only config property
      if (isUiOnlyProperty && subSection) {
        var split = configLink.config.split("/");
        var fileName = split[0] + '.xml',
            configName = split[1],
            id = App.config.configId(configName, fileName);
        var uiOnlyConfig = App.configsCollection.getConfig(id);
        if (!uiOnlyConfig) {
          configProperty = {
            id: id,
            isRequiredByAgent: false,
            showLabel: false,
            isOverridable: false,
            recommendedValue: true,
            name: JSON.parse('"' + configName + '"'),
            isUserProperty: false,
            filename: fileName,
            fileName: fileName,
            isNotSaved: false,
            serviceName: serviceName,
            copy: isCopy,
            stackName: App.get('currentStackName'),
            stackVersion: App.get('currentStackVersionNumber')
          };
          App.configsCollection.add(configProperty);

          if (!subSection.get('configProperties').contains(configProperty.id)) {
            subSection.set('configProperties', subSection.get('configProperties').concat(configProperty.id));
          }
        }
      }
    }
    return configProperty;
  },

  /**
   *
   * @param configConditions: Array
   * @param configProperty: DS.Model Object (App.StackConfigProperty)
   */
  mapThemeConfigConditions: function mapThemeConfigConditions(configConditions, configProperty) {
    var configConditionsCopy = [];
    configConditions.forEach(function (_configCondition, index) {
      var configCondition = $.extend({}, _configCondition);
      configCondition.id = configProperty.id + '_' + index;
      configCondition.config_name = configProperty.name;
      configCondition.file_name = configProperty.filename;
      if (_configCondition.configs && _configCondition.configs.length) {
        configCondition.configs = _configCondition.configs.map(function (item) {
          var result = {};
          result.fileName = item.split('/')[0] + '.xml';
          result.configName = item.split('/')[1];
          return result;
        });
      }

      configCondition.resource = _configCondition.resource || 'config';
      configCondition.type = _configCondition.type || 'config';

      configConditionsCopy.pushObject(configCondition);
    }, this);

    App.store.safeLoadMany(this.get("themeConditionModel"), configConditionsCopy);
  },

  /**
   *
   * @param subSections: Array
   * @param type: {String} possible values: `subsection` or `subsectionTab`
   */
  mapThemeConditions: function mapThemeConditions(subSections, type, themeName) {
    var subSectionConditionsCopy = [];
    subSections.forEach(function (_subSection) {
      var subSectionConditions = _subSection['depends_on'];
      subSectionConditions.forEach(function (_subSectionCondition, index) {
        var subSectionCondition = $.extend({}, _subSectionCondition);
        subSectionCondition.id = _subSection.id + '_' + index;
        subSectionCondition.name = _subSection.name;
        subSectionCondition.theme_name = themeName;
        if (_subSectionCondition.configs && _subSectionCondition.configs.length) {
          subSectionCondition.configs = _subSectionCondition.configs.map(function (item) {
            var result = {};
            result.fileName = item.split('/')[0] + '.xml';
            result.configName = item.split('/')[1];
            return result;
          });
        }

        subSectionCondition.resource = _subSectionCondition.resource || 'config';
        subSectionCondition.type = _subSectionCondition.type || type;
        subSectionConditionsCopy.pushObject(subSectionCondition);
      }, this);
    }, this);
    App.store.safeLoadMany(this.get("themeConditionModel"), subSectionConditionsCopy);
  },

  /**
   * add widget object to <code>stackConfigProperty<code>
   *
   * @param {Object} json - json to parse
   */
  mapThemeWidgets: function mapThemeWidgets(json) {
    var themeName = Em.get(json, 'ThemeInfo.theme_data.Theme.name');
    Em.getWithDefault(json, "ThemeInfo.theme_data.Theme.configuration.widgets", []).forEach(function (widget) {
      var configId = this.getConfigId(widget);
      var configProperty = App.configsCollection.getConfig(configId);

      if (configProperty) {
        Em.set(configProperty, 'widget', widget.widget);
        Em.set(configProperty, 'widgetType', Em.get(widget, 'widget.type'));
        if (themeName === 'default') {
          Em.set(configProperty, 'isInDefaultTheme', true);
        }
      } else {
        var split = widget.config.split("/");
        var fileName = split[0] + '.xml';
        var configName = split[1];
        var uiOnlyProperty = App.configsCollection.getConfigByName(configName, fileName);
        if (uiOnlyProperty) {
          uiOnlyProperty.set('widget', widget.widget);
          uiOnlyProperty.set('widgetType', Em.get(widget, 'widget.type'));
          if (themeName === 'default') {
            Em.set(uiOnlyProperty, 'isInDefaultTheme', true);
          }
        }
      }
    }, this);
  },

  /**
   * transform info from json to config id
   * @param {Object} json
   * @returns {string|null}
   */
  getConfigId: function getConfigId(json) {
    if (json && json.config && typeof json.config === "string") {
      // symbols until first slash is file-name and the rest is config-name
      var fileName = json.config.substr(0, json.config.indexOf('/'));
      var configName = json.config.substr(json.config.indexOf('/') + 1);
      return App.config.configId(configName, fileName);
    } else {
      return null;
    }
  },

  /**
   * generates Advanced tabs for selected or all services
   * @param {[string]} [serviceNames=null]
   */
  generateAdvancedTabs: function generateAdvancedTabs(serviceNames) {
    serviceNames = Em.isArray(serviceNames) ? serviceNames : App.StackService.find().mapProperty('serviceName');
    var advancedTabs = [];
    serviceNames.forEach(function (serviceName) {
      advancedTabs.pushObject({
        id: serviceName + '_advanced',
        name: 'advanced',
        display_name: 'Advanced',
        is_advanced: true,
        service_name: serviceName
      });
    });
    App.store.safeLoadMany(this.get("tabModel"), advancedTabs);
  }
});

});

require.register("mappers/hosts_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

var stringUtils = require('utils/string_utils');

App.hostsMapper = App.QuickDataMapper.create({

  model: App.Host,
  config: {
    id: 'Hosts.host_name',
    host_name: 'Hosts.host_name',
    public_host_name: 'Hosts.public_host_name',
    cluster_id: 'cluster_id', // Hosts.cluster_name
    rack: 'Hosts.rack_info',
    host_components_key: 'host_components',
    host_components_type: 'array',
    host_components: {
      item: 'id'
    },
    alerts_summary: 'alerts_summary',
    critical_warning_alerts_count: 'critical_warning_alerts_count',
    cpu: 'Hosts.cpu_count',
    cpu_physical: 'Hosts.ph_cpu_count',
    memory: 'Hosts.total_mem',
    has_jce_policy: "Hosts.last_agent_env.hasUnlimitedJcePolicy",
    disk_info: 'Hosts.disk_info',
    disk_total: 'metrics.disk.disk_total',
    disk_free: 'metrics.disk.disk_free',
    health_status: 'Hosts.host_status',
    state: 'Hosts.host_state',
    load_one: 'metrics.load.load_one',
    load_five: 'metrics.load.load_five',
    load_fifteen: 'metrics.load.load_fifteen',
    cpu_system: 'metrics.cpu.cpu_system',
    cpu_user: 'metrics.cpu.cpu_user',
    mem_total: 'metrics.memory.mem_total',
    mem_free: 'metrics.memory.mem_free',
    last_heart_beat_time: "Hosts.last_heartbeat_time",
    os_arch: 'Hosts.os_arch',
    os_type: 'Hosts.os_type',
    ip: 'Hosts.ip',
    passive_state: 'Hosts.maintenance_state',
    index: 'index'
  },
  hostComponentConfig: {
    component_name: 'HostRoles.component_name',
    display_name: 'HostRoles.display_name',
    service_id: 'HostRoles.service_name',
    passive_state: 'HostRoles.maintenance_state',
    work_status: 'HostRoles.state',
    stale_configs: 'HostRoles.stale_configs',
    host_name: 'host_name',
    public_host_name: 'public_host_name',
    admin_state: 'HostRoles.desired_admin_state',
    cluster_id_value: 'metrics.dfs.namenode.ClusterId'
  },
  stackVersionConfig: {
    id: 'HostStackVersions.id',
    stack: 'HostStackVersions.stack',
    repo_id: 'repository_versions[0].RepositoryVersions.id',
    repo_version: 'repository_versions[0].RepositoryVersions.repository_version',
    display_name: 'repository_versions[0].RepositoryVersions.display_name',
    version: 'HostStackVersions.version',
    status: 'HostStackVersions.state',
    host_name: 'host_name',
    host_id: 'host_name',
    is_visible: 'is_visible'
  },
  hostComponentLogsConfig: {
    name: 'logging.name',
    service_name: 'HostRoles.service_name',
    host_name: 'HostRoles.host_name',
    log_file_names_type: 'array',
    log_file_names_key: 'logging.logs',
    log_file_names: {
      item: 'name'
    }
  },
  map: function map(json, returnMapped) {
    returnMapped = !!returnMapped;
    console.time('App.hostsMapper execution time');
    if (json.items) {
      var hostsWithFullInfo = [];
      var hostIds = {};
      var components = [];
      var stackVersions = [];
      var componentsIdMap = {};
      var cacheServices = App.cache['services'];
      var currentServiceComponentsMap = this.buildServiceComponentMap(cacheServices);
      var newHostComponentsMap = {};
      var selectedHosts = App.db.getSelectedHosts();
      var clusterName = App.get('clusterName');
      var advancedHostComponents = [];
      var hostComponentLogs = [];

      // Create a map for quick access on existing hosts
      var hosts = App.Host.find().toArray();
      var hostsMap = {};
      for (var p = 0; p < hosts.length; p++) {
        hostsMap[hosts[p].get('hostName')] = hosts[p];
      }

      // Use normal for loop instead of foreach to enhance performance
      for (var index = 0; index < json.items.length; index++) {
        var item = json.items[index];
        var notStartedComponents = [];
        var componentsInPassiveState = [];
        var componentsWithStaleConfigs = [];

        item.host_components = item.host_components || [];
        for (var i = 0; i < item.host_components.length; i++) {
          var host_component = item.host_components[i];
          var id = host_component.HostRoles.component_name + "_" + item.Hosts.host_name;
          var component = this.parseIt(host_component, this.hostComponentConfig);
          var serviceName = host_component.HostRoles.service_name;

          host_component.id = id;
          component.id = id;
          component.host_id = item.Hosts.host_name;
          component.host_name = item.Hosts.host_name;
          component.public_host_name = item.Hosts.public_host_name;
          components.push(component);
          componentsIdMap[id] = component;
          if (!newHostComponentsMap[serviceName]) {
            newHostComponentsMap[serviceName] = [];
          }
          if (!currentServiceComponentsMap[serviceName]) {
            currentServiceComponentsMap[serviceName] = [];
          }
          if (!currentServiceComponentsMap[serviceName][id]) {
            newHostComponentsMap[serviceName].push(id);
          }
          if (App.serviceMetricsMapper.get('ADVANCED_COMPONENTS').contains(host_component.HostRoles.component_name)) {
            advancedHostComponents.push(id);
          }
          if (component.work_status !== App.HostComponentStatus.started) {
            notStartedComponents.push(id);
          }
          if (component.stale_configs) {
            componentsWithStaleConfigs.push(id);
          }
          if (component.passive_state !== 'OFF') {
            componentsInPassiveState.push(id);
          }
          if (host_component.hasOwnProperty('logging')) {
            var logParsed = this.parseIt(host_component, this.hostComponentLogsConfig);
            logParsed.id = logParsed.host_name + '_' + logParsed.name;
            logParsed.host_component_id = host_component.id;
            component.component_logs_id = logParsed.id;
            hostComponentLogs.push(logParsed);
          }
        }

        var currentVersion = item.stack_versions.findProperty('HostStackVersions.state', 'CURRENT');
        var currentVersionNumber = currentVersion && currentVersion.repository_versions ? Em.get(currentVersion.repository_versions[0], 'RepositoryVersions.repository_version') : '';
        for (var j = 0; j < item.stack_versions.length; j++) {
          var stackVersion = item.stack_versions[j];
          var versionNumber = Em.get(stackVersion.repository_versions[0], 'RepositoryVersions.repository_version');
          var isDifferentStack = currentVersion && stackVersion.HostStackVersions.stack !== currentVersion.HostStackVersions.stack;
          var isCompatible = App.RepositoryVersion.find(Em.get(stackVersion.repository_versions[0], 'RepositoryVersions.id')).get('isCompatible');
          stackVersion.host_name = item.Hosts.host_name;
          if (isDifferentStack && !isCompatible) {
            stackVersion.is_visible = false;
          } else {
            stackVersion.is_visible = isDifferentStack || App.get('supports.displayOlderVersions') || stringUtils.compareVersions(versionNumber, currentVersionNumber) >= 0 || !currentVersionNumber;
          }
          stackVersions.push(this.parseIt(stackVersion, this.stackVersionConfig));
        }

        item.cluster_id = clusterName;
        var existingHost = hostsMap[item.Hosts.host_name];
        // There is no need to override existing index in host detail view since old model(already have indexes) will not be cleared.
        item.index = existingHost && !json.itemTotal ? existingHost.get('index') : index;

        this.config = $.extend(this.config, {
          stack_versions_key: 'stack_versions',
          stack_versions_type: 'array',
          stack_versions: {
            item: 'HostStackVersions.id'
          }
        });
        var parsedItem = this.parseIt(item, this.config);

        parsedItem.selected = selectedHosts.contains(parsedItem.host_name);
        parsedItem.not_started_components = notStartedComponents;
        parsedItem.components_in_passive_state = componentsInPassiveState;
        parsedItem.components_with_stale_configs = componentsWithStaleConfigs;
        parsedItem.is_filtered = true;

        hostIds[item.Hosts.host_name] = parsedItem;

        hostsWithFullInfo.push(parsedItem);
      }

      if (returnMapped) {
        return hostsWithFullInfo;
      }

      for (var k = 0; k < advancedHostComponents.length; k++) {
        var key = advancedHostComponents[k];
        if (componentsIdMap[key]) {
          var existingRecord = App.HostComponent.find(key);
          componentsIdMap[key].display_name_advanced = existingRecord.get('displayNameAdvanced');
          componentsIdMap[key].ha_name_space = existingRecord.get('haNameSpace');
        }
      }

      //"itemTotal" present only for Hosts page request
      if (!Em.isNone(json.itemTotal)) {
        App.Host.find().setEach('isFiltered', false);
        App.Host.find().clear();
        //App.HostComponent.find contains master components which requested across the app hence it should not be cleared
      }
      App.store.safeLoadMany(App.HostStackVersion, stackVersions);
      App.store.safeLoadMany(App.HostComponentLog, hostComponentLogs);
      App.store.safeLoadMany(App.HostComponent, components);
      App.store.safeLoadMany(App.Host, hostsWithFullInfo);
      var itemTotal = parseInt(json.itemTotal);
      if (!isNaN(itemTotal)) {
        App.router.set('mainHostController.filteredCount', itemTotal);
      }
      //bind host-components with service records
      this.addNewHostComponents(newHostComponentsMap, cacheServices);
    }
    console.timeEnd('App.hostsMapper execution time');
  },

  /**
    * set metric fields of hosts
   * @param {object} data
   */
  setMetrics: function setMetrics(data) {
    var hosts = this.get('model').find();
    for (var i = 0; i < hosts.content.length; i++) {
      var host = hosts.objectAt(i);
      var hostMetrics = data.items.findProperty('Hosts.host_name', host.get('hostName'));
      host.setProperties({
        diskTotal: Em.get(hostMetrics, 'metrics.disk.disk_total'),
        diskFree: Em.get(hostMetrics, 'metrics.disk.disk_free'),
        loadOne: Em.get(hostMetrics, 'metrics.load.load_one')
      });
    }
  },

  /**
   * build map that include loaded host-components to avoid duplicate loading
   * @param cacheServices
   * @return {Object}
   */
  buildServiceComponentMap: function buildServiceComponentMap(cacheServices) {
    var loadedServiceComponentsMap = {};

    cacheServices.forEach(function (cacheService) {
      var componentsMap = {};

      cacheService.host_components.forEach(function (componentId) {
        componentsMap[componentId] = true;
      });
      loadedServiceComponentsMap[cacheService.ServiceInfo.service_name] = componentsMap;
    });
    return loadedServiceComponentsMap;
  },

  /**
   * add only new host-components to every service
   * to update service - host-component relations in model
   * @param {object} newHostComponentsMap
   * @param {Array} cacheServices
   * @return {boolean}
   */
  addNewHostComponents: function addNewHostComponents(newHostComponentsMap, cacheServices) {
    if (!newHostComponentsMap || !cacheServices) return false;
    cacheServices.forEach(function (service) {
      if (newHostComponentsMap[service.ServiceInfo.service_name]) {
        newHostComponentsMap[service.ServiceInfo.service_name].forEach(function (componentId) {
          service.host_components.push(componentId);
        });
      }
    }, this);
    return true;
  }
});

});

require.register("mappers/quicklinks_mapper", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require("app");

App.quicklinksMapper = App.QuickDataMapper.create({
  model: App.QuickLinksConfig,

  config: {
    "id": "QuickLinkInfo.service_name",
    "file_name": "QuickLinkInfo.file_name",
    "service_name": "QuickLinkInfo.service_name",
    "stack_name": "QuickLinkInfo.stack_name",
    "stack_version": "QuickLinkInfo.stack_version",
    "name": "QuickLinkInfo.quicklink_data.QuickLinksConfiguration.name",
    "protocol": "QuickLinkInfo.quicklink_data.QuickLinksConfiguration.configuration.protocol",
    "links": "QuickLinkInfo.quicklink_data.QuickLinksConfiguration.configuration.links"
  },

  map: function map(json) {
    console.time('App.quicklinksMapper execution time');

    var result = [];
    var linkResult = [];

    json.items.forEach(function (item) {
      var parseResult = this.parseIt(item, this.get('config'));
      console.log("parseResult", parseResult);
      result.push(parseResult);
    }, this);

    App.store.safeLoadMany(this.get('model'), result);
    console.timeEnd('App.quicklinksMapper execution time');
  }
});

});

require.register("mappers/racks_mapper", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

App.racksMapper = App.QuickDataMapper.create({
  model: App.Rack,
  config: {
    id: "Racks.id",
    name: "Racks.name"
  }
});

});

require.register("mappers/repository_version_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * 'License'); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.repoVersionMapper = App.QuickDataMapper.create({
  modelRepoVersions: App.RepositoryVersion,
  modelOperatingSystems: App.OS,
  modelRepositories: App.Repository,
  modelServices: App.ServiceSimple,

  modelRepoVersion: function modelRepoVersion(isCurrentStackOnly) {
    var repoVersionsKey = 'RepositoryVersions';
    return {
      id: repoVersionsKey + '.id',
      stack_version_id: repoVersionsKey + '.stackVersionId',
      display_name: repoVersionsKey + '.display_name',
      type: repoVersionsKey + '.type',
      hidden: repoVersionsKey + '.hidden',
      repository_version: repoVersionsKey + '.repository_version',
      upgrade_pack: repoVersionsKey + '.upgrade_pack',
      stack_version_type: repoVersionsKey + '.stack_name',
      stack_version_number: repoVersionsKey + '.stack_version',
      use_redhat_satellite: 'use_redhat_satellite',
      stack_services_key: 'stack_services',
      stack_services_type: 'array',
      stack_services: {
        item: 'id'
      },
      operating_systems_key: 'operating_systems',
      operating_systems_type: 'array',
      operating_systems: {
        item: 'id'
      }
    };
  },

  modelOS: {
    id: 'id',
    repository_version_id: 'repository_version_id',
    os_type: 'OperatingSystems.os_type',
    stack_name: 'OperatingSystems.stack_name',
    stack_version: 'OperatingSystems.stack_version',
    repositories_key: 'repositories',
    repositories_type: 'array',
    repositories: {
      item: 'id'
    }
  },

  modelRepository: {
    id: 'id',
    operating_system_id: 'Repositories.operating_system_id',
    base_url: 'Repositories.base_url',
    default_base_url: 'Repositories.default_base_url',
    latest_base_url: 'Repositories.latest_base_url',
    mirrors_list: 'Repositories.mirrors_list',
    os_type: 'Repositories.os_type',
    repo_id: 'Repositories.repo_id',
    original_repo_id: 'Repositories.repo_id',
    repo_name: 'Repositories.repo_name',
    stack_name: 'Repositories.stack_name',
    stack_version: 'Repositories.stack_version',
    tags: 'Repositories.tags',
    applicable_services: 'Repositories.applicable_services'
  },

  map: function map(json, loadAll, isCurrentStackOnly) {
    var modelRepoVersions = this.get('modelRepoVersions');
    var modelOperatingSystems = this.get('modelOperatingSystems');
    var modelRepositories = this.get('modelRepositories');
    var modelServices = this.get('modelServices');

    var resultRepoVersion = [];
    var resultOS = [];
    var resultRepo = [];
    var resultService = [];
    var repoVersionsKey = 'RepositoryVersions';

    if (!isCurrentStackOnly) {
      json = this.convertToRepoScheme(json);
    }

    if (json && json.items) {
      json.items.forEach(function (item) {
        if (loadAll || item[repoVersionsKey] && !App.StackVersion.find().someProperty('repositoryVersion.id', item[repoVersionsKey].id)) {
          var repo = item;
          var osArray = [];
          if (item.operating_systems) {
            item.operating_systems.forEach(function (os) {
              os.id = item[repoVersionsKey].repository_version + os.OperatingSystems.os_type;
              os.repository_version_id = repo.id;
              var repoArray = [];
              if (Em.get(os, 'repositories')) {
                os.repositories.forEach(function (repo) {
                  repo.id = repo.Repositories.repo_id + os.id;
                  repo.operating_system_id = os.id;
                  repoArray.pushObject(repo);
                  resultRepo.push(this.parseIt(repo, this.get('modelRepository')));
                }, this);
              }
              os.repositories = repoArray;
              osArray.pushObject(os);
              resultOS.push(this.parseIt(os, this.get('modelOS')));
            }, this);
          }
          if (item[repoVersionsKey].stack_services) {
            item[repoVersionsKey].stack_services.forEach(function (service) {
              var serviceObj = {
                id: item[repoVersionsKey].repository_version + service.name,
                name: service.name,
                display_name: service.display_name,
                latest_version: service.versions[0] ? service.versions[0] : '',
                is_available: item[repoVersionsKey].services ? item[repoVersionsKey].services.someProperty('name', service.name) : true,
                is_upgradable: json.stackServices ? json.stackServices[service.name] && json.stackServices[service.name].upgrade : true
              };
              resultService.push(serviceObj);
            }, this);
          } else if (item[repoVersionsKey].services) {
            item[repoVersionsKey].services.forEach(function (service) {
              var serviceObj = {
                id: item[repoVersionsKey].repository_version + service.name,
                name: service.name,
                display_name: service.display_name,
                latest_version: service.versions[0] ? service.versions[0].version : ''
              };
              resultService.push(serviceObj);
            }, this);
          }
          repo.use_redhat_satellite = Em.get(item, 'operating_systems.0.OperatingSystems.ambari_managed_repositories') === false;
          repo.operating_systems = osArray;
          repo.stack_services = resultService;
          resultRepoVersion.push(this.parseIt(repo, this.modelRepoVersion(isCurrentStackOnly)));
        }
      }, this);
    }
    App.store.safeLoadMany(modelRepositories, resultRepo);
    App.store.safeLoadMany(modelOperatingSystems, resultOS);
    App.store.safeLoadMany(modelServices, resultService);
    App.store.safeLoadMany(modelRepoVersions, resultRepoVersion);
  },

  /**
   *
   * @param {?object} json
   * @returns {{items: Array}}
   */
  convertToRepoScheme: function convertToRepoScheme(json) {
    var extractedJson = { items: [] };

    if (json && json.items) {
      json.items.forEach(function (stack) {
        stack.versions.forEach(function (version) {
          version.repository_versions.forEach(function (repoVersion) {
            extractedJson.items.push(repoVersion);
          }, this);
        }, this);
      }, this);
    }
    return extractedJson;
  }
});

});

require.register("mappers/root_service_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.rootServiceMapper = App.QuickDataMapper.create({
  root_service_model: App.RootService,
  root_service_component_model: App.RootServiceComponents,

  configRootService: {
    id: 'service_name',
    service_name: 'service_name',
    components_key: 'components',
    components_type: 'array',
    components: {
      item: 'id'
    }
  },

  configRootServiceComponents: {
    id: 'component_name',
    component_name: 'component_name',
    component_version: 'component_version',
    server_clock: 'server_clock',
    service_name: 'service_name',
    properties: 'properties'
  },

  map: function map(data) {
    var rootServiceModel = this.get('root_service_model');
    var rootServiceComponentModel = this.get('root_service_component_model');
    var rootService = {},
        rootServiceComponents = [];
    rootService.id = rootService.service_name = data.RootService.service_name;
    data.components.forEach(function (item) {
      item.RootServiceComponents.id = item.RootServiceComponents.component_name;
      rootServiceComponents.push(this.parseIt(item.RootServiceComponents, this.configRootServiceComponents));
    }, this);
    rootService.components = rootServiceComponents;
    App.store.safeLoadMany(rootServiceComponentModel, rootServiceComponents);
    App.store.safeLoad(rootServiceModel, this.parseIt(rootService, this.configRootService));
  }
});

});

require.register("mappers/server_data_mapper", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * initialize common cache container for mappers
 * App.cache contains shared data, used for syncronizing incoming server data among mappers
 */
App.cache = {
  'previousHostStatuses': {},
  'previousComponentStatuses': {},
  'previousComponentPassiveStates': {},
  'staleConfigsComponentHosts': {},
  'services': [],
  'currentConfigVersions': {}
};

App.cache.clear = function () {
  var clear = App.cache.clear;
  App.cache = {
    'previousHostStatuses': {},
    'previousComponentStatuses': {},
    'previousComponentPassiveStates': {},
    'services': [],
    'currentConfigVersions': {}
  };
  App.cache.clear = clear;
};

App.store.reopen({
  safeLoadMany: function safeLoadMany(model, records) {
    try {
      this.loadMany(model, records);
    } catch (e) {
      console.debug('Resolve uncommitted records before load');
      this.fastCommit();
      this.loadMany(model, records);
    }
  },

  safeLoad: function safeLoad(model, record) {
    try {
      this.load(model, record);
    } catch (e) {
      console.debug('Resolve uncommitted record before load');
      this.fastCommit();
      this.load(model, record);
    }
  },

  /**
   * App.store.commit() - creates new transaction,
   * and them move all records from old to new transactions which is expensive
   *
   * We should use only defaultTransaction,
   * and then we can stub <code>removeCleanRecords</code> method,
   * because it would remove from and add records to the same (default) transaction
   */
  fastCommit: function fastCommit() {
    console.time('store commit');
    App.store.defaultTransaction.commit();
    console.timeEnd('store commit');
  }
});

App.ServerDataMapper = Em.Object.extend({
  jsonKey: false,
  map: function map(json) {
    if (json) {
      var model = this.get('model');
      var jsonKey = this.get('jsonKey');

      if (jsonKey && json[jsonKey]) {
        // if data come as { hdfs: {...} }
        json = json[jsonKey];
      }

      $.each(json, function (field, value) {
        model.set(field, value);
      });
    }
  }
});

App.QuickDataMapper = App.ServerDataMapper.extend({
  config: {},
  model: null,
  map: function map(json) {
    if (!this.get('model')) {
      return;
    }

    if (json.items) {
      var result = [];

      json.items.forEach(function (item) {
        result.push(this.parseIt(item, this.config));
      }, this);

      App.store.safeLoadMany(this.get('model'), result);
    }
  },

  parseIt: function parseIt(data, config) {
    var result = {};
    for (var i in config) {
      if (i.substr(0, 1) === '$') {
        i = i.substr(1, i.length);
        result[i] = config['$' + i];
      } else {
        var isSpecial = false;
        if (i.substr(-5) == '_type') {
          var prefix = i.substr(0, i.length - 5);
          isSpecial = config[prefix + '_key'] != null;
        } else if (i.substr(-4) == '_key') {
          var prefix = i.substr(0, i.length - 4);
          isSpecial = config[prefix + '_type'] != null;
        }
        if (!isSpecial && typeof config[i] == 'string') {
          result[i] = this.getJsonProperty(data, config[i]);
        } else if (_typeof(config[i]) == 'object') {
          result[i] = [];
          var _data = this.getJsonProperty(data, config[i + '_key']);
          var _type = config[i + '_type'];
          var l = _data.length;
          for (var index = 0; index < l; index++) {
            if (_type == 'array') {
              result[i].push(this.getJsonProperty(_data[index], config[i].item));
            } else {
              result[i].push(this.parseIt(_data[index], config[i]));
            }
          }
          // As for 'widgets', just show the original order
          if (_type == 'array' && i != 'widgets') {
            result[i] = result[i].sort();
          }
        }
      }
    }
    return result;
  },

  getJsonProperty: function getJsonProperty(json, path) {
    var pathArr = path.split('.');
    var current = json;
    while (pathArr.length && current) {
      if (pathArr[0].substr(-1) == ']') {
        var index = parseInt(pathArr[0].substr(-2, 1));
        var attr = pathArr[0].substr(0, pathArr[0].length - 3);
        if (attr in current) {
          current = current[attr][index];
        }
      } else {
        current = current[pathArr[0]];
      }
      pathArr.splice(0, 1);
    }
    return current;
  },

  /**
   * properly delete record from model
   * @param item
   */
  deleteRecord: function deleteRecord(item) {
    if (item.get('isLoaded')) {
      item.deleteRecord();
      App.store.fastCommit();
      item.get('stateManager').transitionTo('loading');
    }
  },
  /**
   * check mutable fields whether they have been changed and if positive
   * return host object only with properties, that contains new value
   * @param current
   * @param previous
   * @param fields
   * @return {*}
   */
  getDiscrepancies: function getDiscrepancies(current, previous, fields) {
    var result = {};
    if (previous) {
      fields.forEach(function (field) {
        if (Array.isArray(current[field])) {
          if (JSON.stringify(current[field]) !== JSON.stringify(previous[field])) {
            result[field] = current[field];
            result.isLoadNeeded = true;
          }
        } else {
          if (current[field] != previous[field]) result[field] = current[field];
        }
      });
      return result;
    }
    return current;
  },

  /**
   * Binary search <code>searchElement</code> in the array (should be sorted!)
   * @param {number[]|string[]} array
   * @param {number|string} searchElement
   * @returns {number} position of the needed element or negative value, if value wasn't found
   * @method binaryIndexOf
   */
  binaryIndexOf: function binaryIndexOf(array, searchElement) {
    var minIndex = 0;
    var maxIndex = array.length - 1;
    var currentIndex;
    var currentElement;
    var resultIndex;

    if (array[0] > searchElement || array[array.length - 1] < searchElement) {
      return -1;
    }
    while (minIndex <= maxIndex) {
      resultIndex = currentIndex = (minIndex + maxIndex) / 2 | 0;
      currentElement = array[currentIndex];

      if (currentElement < searchElement) {
        minIndex = currentIndex + 1;
      } else if (currentElement > searchElement) {
        maxIndex = currentIndex - 1;
      } else {
        return currentIndex;
      }
    }

    return ~maxIndex;
  },

  /**
   * @param {Em.Object} record
   * @param {object} event
   * @param {object} config
   */
  updatePropertiesByConfig: function updatePropertiesByConfig(record, event, config) {
    if (record.get('isLoaded')) {
      for (var internalProp in config) {
        var externalProp = config[internalProp];
        if (!Em.isNone(event[externalProp])) {
          record.set(internalProp, event[externalProp]);
        }
      }
    }
  }

});

});

require.register("mappers/service_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var misc = require('utils/misc');

App.serviceMapper = App.QuickDataMapper.create({
  model: App.Service,
  config: {
    id: 'ServiceInfo.service_name',
    service_name: 'ServiceInfo.service_name',
    work_status: 'ServiceInfo.state',
    desired_repository_version_id: 'ServiceInfo.desired_repository_version_id'
  },
  initialAppLoad: false,
  passiveStateMap: {},
  map: function map(json) {
    console.time("App.serviceMapper execution time");
    var self = this;
    var passiveStateMap = this.get('passiveStateMap');
    json.items.forEach(function (service) {
      var cachedService = App.cache['services'].findProperty('ServiceInfo.service_name', service.ServiceInfo.service_name);
      if (cachedService) {
        // restore service workStatus
        App.Service.find(cachedService.ServiceInfo.service_name).set('workStatus', service.ServiceInfo.state);
        cachedService.ServiceInfo.state = service.ServiceInfo.state;
      } else {
        var serviceData = {
          ServiceInfo: {
            service_name: service.ServiceInfo.service_name,
            state: service.ServiceInfo.state,
            desired_repository_version_id: service.ServiceInfo.desired_repository_version_id
          },
          host_components: [],
          components: []
        };
        App.cache['services'].push(serviceData);
      }
      passiveStateMap[service.ServiceInfo.service_name] = service.ServiceInfo.maintenance_state;
    });

    if (!this.get('initialAppLoad')) {
      var parsedCacheServices = App.cache['services'].map(function (item) {
        App.serviceMetricsMapper.mapExtendedModel(item);
        return self.parseIt(item, self.get('config'));
      });
      parsedCacheServices = misc.sortByOrder(App.StackService.find().mapProperty('serviceName'), parsedCacheServices);
      App.store.safeLoadMany(this.get('model'), parsedCacheServices);
      this.set('initialAppLoad', true);
    }
    this.servicesLoading().done(function setMaintenanceState() {
      for (var service in passiveStateMap) {
        if (passiveStateMap.hasOwnProperty(service)) {
          App.Service.find(service).set('passiveState', passiveStateMap[service]);
        }
      }
    });

    console.timeEnd("App.serviceMapper execution time");
  },

  servicesLoading: function servicesLoading() {
    var dfd = $.Deferred();
    var isAllServicesLoaded = App.store.findAll(App.Service).everyProperty('isLoaded', true);
    if (isAllServicesLoaded) {
      dfd.resolve();
    } else {
      var interval = setInterval(function checkIfServicesLoaded() {
        var isAllServicesLoaded = App.store.findAll(App.Service).everyProperty('isLoaded', true);
        if (isAllServicesLoaded) {
          dfd.resolve();
          clearInterval(interval);
        }
      }, 5);
    }
    return dfd.promise();
  }
});

});

require.register("mappers/service_metrics_mapper", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var misc = require('utils/misc');
var stringUtils = require('utils/string_utils');
var dateUtils = require('utils/date/date');
var previousMasterComponentIds = [];

App.serviceMetricsMapper = App.QuickDataMapper.create({

  model: App.Service,
  config: {
    id: 'ServiceInfo.service_name',
    service_name: 'ServiceInfo.service_name',
    work_status: 'ServiceInfo.state',
    passive_state: 'ServiceInfo.passive_state',
    $rand: Math.random(),
    $alerts: [1, 2, 3],
    host_components: 'host_components',
    tool_tip_content: 'tool_tip_content',
    installed_clients: 'installed_clients',
    client_components: 'client_components',
    slave_components: 'slave_components',
    master_components: 'master_components',
    desired_repository_version_id: 'ServiceInfo.desired_repository_version_id'
  },
  hdfsConfig: {
    version: 'nameNodeComponent.host_components[0].metrics.dfs.namenode.Version',
    active_name_nodes: 'active_name_nodes',
    standby_name_nodes: 'standby_name_nodes',
    journal_nodes: 'journal_nodes',
    name_node_id: 'name_node_id',
    sname_node_id: 'sname_node_id',
    metrics_not_available: 'metrics_not_available',
    name_node_start_time_values: 'name_node_start_time_values',
    jvm_memory_heap_used_values: 'jvm_memory_heap_used_values',
    jvm_memory_heap_max_values: 'jvm_memory_heap_max_values',
    live_data_nodes: 'live_data_nodes',
    dead_data_nodes: 'dead_data_nodes',
    decommission_data_nodes: 'decommission_data_nodes',
    capacity_used: 'nameNodeComponent.host_components[0].metrics.dfs.FSNamesystem.CapacityUsed',
    capacity_total: 'nameNodeComponent.host_components[0].metrics.dfs.FSNamesystem.CapacityTotal',
    capacity_remaining: 'nameNodeComponent.host_components[0].metrics.dfs.FSNamesystem.CapacityRemaining',
    capacity_non_dfs_used: 'nameNodeComponent.host_components[0].metrics.dfs.FSNamesystem.CapacityNonDFSUsed',
    dfs_total_blocks_values: 'dfs_total_blocks_values',
    dfs_corrupt_blocks_values: 'dfs_corrupt_blocks_values',
    dfs_missing_blocks_values: 'dfs_missing_blocks_values',
    dfs_under_replicated_blocks_values: 'dfs_under_replicated_blocks_values',
    dfs_total_files_values: 'dfs_total_files_values',
    work_status_values: 'work_status_values',
    upgrade_status_values: 'upgrade_status_values',
    safe_mode_status_values: 'safe_mode_status_values',
    name_node_rpc_values: 'name_node_rpc_values',
    data_nodes_started: 'data_nodes_started',
    data_nodes_installed: 'data_nodes_installed',
    data_nodes_total: 'data_nodes_total',
    nfs_gateways_started: 'nfs_gateways_started',
    nfs_gateways_installed: 'nfs_gateways_installed',
    nfs_gateways_total: 'nfs_gateways_total'
  },
  activeNameNodeConfig: {
    name_node_start_time_values: 'metrics.runtime.StartTime',
    jvm_memory_heap_used_values: 'metrics.jvm.HeapMemoryUsed',
    jvm_memory_heap_max_values: 'metrics.jvm.HeapMemoryMax',
    dfs_total_blocks_values: 'metrics.dfs.FSNamesystem.BlocksTotal',
    dfs_corrupt_blocks_values: 'metrics.dfs.FSNamesystem.CorruptBlocks',
    dfs_missing_blocks_values: 'metrics.dfs.FSNamesystem.MissingBlocks',
    dfs_under_replicated_blocks_values: 'metrics.dfs.FSNamesystem.UnderReplicatedBlocks',
    dfs_total_files_values: 'metrics.dfs.FSNamesystem.FilesTotal',
    work_status_values: 'HostRoles.state',
    upgrade_status_values: 'metrics.dfs.namenode.UpgradeFinalized',
    safe_mode_status_values: 'metrics.dfs.namenode.Safemode',
    name_node_rpc_values: 'metrics.rpc.client.RpcQueueTime_avg_time'
  },
  onefsConfig: {
    metrics_not_available: 'metrics_not_available',
    name_node_start_time: 'metrics.runtime.StartTime',
    capacity_used: 'metrics.dfs.FSNamesystem.CapacityUsed',
    capacity_total: 'metrics.dfs.FSNamesystem.CapacityTotal',
    capacity_remaining: 'metrics.dfs.FSNamesystem.CapacityRemaining',
    dfs_corrupt_blocks: 'metrics.dfs.FSNamesystem.CorruptBlocks',
    dfs_under_replicated_blocks: 'metrics.dfs.FSNamesystem.UnderReplicatedBlocks',
    dfs_missing_blocks: 'metrics.dfs.FSNamesystem.MissingBlocks',
    live_data_nodes: 'live_data_nodes',
    dead_data_nodes: 'dead_data_nodes',
    decommission_data_nodes: 'decommission_data_nodes',
    dfs_total_files: 'metrics.dfs.namenode.TotalFiles',
    jvm_memory_heap_used: 'metrics.jvm.memHeapUsedM',
    jvm_memory_heap_max: 'metrics.jvm.memHeapCommittedM',
    upgrade_status: 'metrics.dfs.namenode.UpgradeFinalized'
  },
  yarnConfig: {
    resource_manager_start_time: 'resourceManagerComponent.host_components[0].metrics.runtime.StartTime',
    jvm_memory_heap_used: 'resourceManagerComponent.host_components[0].metrics.jvm.HeapMemoryUsed',
    jvm_memory_heap_max: 'resourceManagerComponent.host_components[0].metrics.jvm.HeapMemoryMax',
    containers_allocated: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AllocatedContainers',
    containers_pending: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.PendingContainers',
    containers_reserved: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.ReservedContainers',
    apps_submitted: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsSubmitted',
    apps_running: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsRunning',
    apps_pending: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsPending',
    apps_completed: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsCompleted',
    apps_killed: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsKilled',
    apps_failed: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AppsFailed',
    node_managers_count_active: 'resourceManagerComponent.host_components[0].metrics.yarn.ClusterMetrics.NumActiveNMs',
    node_managers_count_lost: 'resourceManagerComponent.host_components[0].metrics.yarn.ClusterMetrics.NumLostNMs',
    node_managers_count_unhealthy: 'resourceManagerComponent.host_components[0].metrics.yarn.ClusterMetrics.NumUnhealthyNMs',
    node_managers_count_rebooted: 'resourceManagerComponent.host_components[0].metrics.yarn.ClusterMetrics.NumRebootedNMs',
    node_managers_count_decommissioned: 'resourceManagerComponent.host_components[0].metrics.yarn.ClusterMetrics.NumDecommissionedNMs',
    allocated_memory: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AllocatedMB',
    available_memory: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.AvailableMB',
    reserved_memory: 'resourceManagerComponent.host_components[0].metrics.yarn.Queue.root.ReservedMB',
    queue: 'resourceManagerComponent.queue',
    node_managers_started: 'node_managers_started',
    node_managers_installed: 'node_managers_installed',
    node_managers_total: 'node_managers_total',
    app_timeline_server_id: 'app_timeline_server_id',
    resource_manager_id: 'resource_manager_id',
    active_resource_manager_id: 'active_resource_manager_id'
  },
  mapReduce2Config: {
    map_reduce2_clients: 'map_reduce2_clients',
    job_history_server_id: 'job_history_server_id'
  },
  hbaseConfig: {
    master_start_time: 'masterComponent.host_components[0].metrics.hbase.master.MasterStartTime',
    master_active_time: 'masterComponent.host_components[0].metrics.hbase.master.MasterActiveTime',
    master_id: 'master_id',
    average_load: 'masterComponent.host_components[0].metrics.hbase.master.AverageLoad',
    heap_memory_used: 'masterComponent.host_components[0].metrics.jvm.HeapMemoryUsed',
    heap_memory_max: 'masterComponent.host_components[0].metrics.jvm.HeapMemoryMax',
    regions_in_transition: 'regions_in_transition',
    region_servers_started: 'region_servers_started',
    region_servers_installed: 'region_servers_installed',
    region_servers_total: 'region_servers_total',
    phoenix_servers_started: 'phoenix_servers_started',
    phoenix_servers_installed: 'phoenix_servers_installed',
    phoenix_servers_total: 'phoenix_servers_total'
  },
  stormConfig: {
    total_tasks: 'restApiComponent.tasksTotal',
    total_slots: 'restApiComponent.slotsTotal',
    free_slots: 'restApiComponent.slotsFree',
    used_slots: 'restApiComponent.slotsUsed',
    topologies: 'restApiComponent.topologies',
    total_executors: 'restApiComponent.executorsTotal',
    nimbus_uptime: 'restApiComponent.nimbusUptime',
    super_visors_started: 'super_visors_started',
    super_visors_installed: 'super_visors_installed',
    super_visors_total: 'super_visors_total'
  },
  rangerConfig: {
    ranger_tagsyncs_started: 'ranger_tagsyncs_started',
    ranger_tagsyncs_installed: 'ranger_tagsyncs_installed',
    ranger_tagsyncs_total: 'ranger_tagsyncs_total'
  },
  flumeConfig: {
    flume_handlers_total: 'flume_handlers_total'
  },
  flumeAgentConfig: {
    name: 'HostComponentProcess.name',
    status: 'HostComponentProcess.status',
    host_id: 'HostComponentProcess.host_name',
    host_name: 'HostComponentProcess.host_name',
    channels_count: 'HostComponentProcess.channels_count',
    sources_count: 'HostComponentProcess.sources_count',
    sinks_count: 'HostComponentProcess.sinks_count'
  },

  model3: App.HostComponent,
  config3: {
    work_status: 'HostRoles.state',
    passive_state: 'HostRoles.maintenance_state',
    display_name: 'HostRoles.display_name',
    component_name: 'HostRoles.component_name',
    host_id: 'HostRoles.host_name',
    host_name: 'HostRoles.host_name',
    public_host_name: 'HostRoles.public_host_name',
    stale_configs: 'HostRoles.stale_configs',
    ha_status: 'HostRoles.ha_state',
    display_name_advanced: 'display_name_advanced',
    admin_state: 'HostRoles.desired_admin_state',
    cluster_id_value: 'metrics.dfs.namenode.ClusterId'
  },

  /**
   * components which have additional relations and filtered for <code>computeAdditionalRelations</code>
   * @type {Array}
   * @const
   */
  ADVANCED_COMPONENTS: ['SECONDARY_NAMENODE', 'RESOURCEMANAGER', 'NAMENODE', 'HBASE_MASTER', 'RESOURCEMANAGER', 'HIVE_SERVER_INTERACTIVE'],

  hostNameIpMap: {},

  map: function map(json) {
    console.time('App.serviceMetricsMapper execution time');
    if (json.items) {

      // Host components
      var hostComponents = [];
      var services = App.cache['services'];
      var previousComponentStatuses = App.cache['previousComponentStatuses'];
      var lastKnownStatusesLength = Em.keys(previousComponentStatuses).length;
      var previousComponentPassiveStates = App.cache['previousComponentPassiveStates'];
      var result = [];
      var advancedHostComponents = [];
      var hostComponentIdsMap = {};
      var hiveInteractiveServers = [];

      /**
       * services contains constructed service-components structure from components array
       */
      services.setEach('components', []);

      json.items.forEach(function (component) {
        var serviceName = component.ServiceComponentInfo.service_name;
        var service = services.findProperty('ServiceInfo.service_name', serviceName);
        if (service) {
          service.components.push(component);
        }
        component.host_components.forEach(function (host_component) {
          var id = host_component.HostRoles.component_name + "_" + host_component.HostRoles.host_name;
          hostComponentIdsMap[id] = true;
          previousComponentStatuses[id] = host_component.HostRoles.state;
          previousComponentPassiveStates[id] = host_component.HostRoles.maintenance_state;
          this.config3.ha_status = host_component.HostRoles.component_name == "HBASE_MASTER" ? 'metrics.hbase.master.IsActiveMaster' : 'HostRoles.ha_state';
          var comp = this.parseIt(host_component, this.config3);
          comp.id = id;
          comp.service_id = serviceName;
          if (comp.component_name === 'HIVE_SERVER_INTERACTIVE') {
            hiveInteractiveServers.push(comp);
          }
          hostComponents.push(comp);
          if (this.get('ADVANCED_COMPONENTS').contains(comp.component_name)) {
            advancedHostComponents.push(comp);
          }
        }, this);
      }, this);

      this.computeAdditionalRelations(advancedHostComponents, services);
      //load master components to model
      previousMasterComponentIds.forEach(function (id) {
        if (!hostComponentIdsMap[id]) {
          var hostComponent = App.HostComponent.find(id);
          if (hostComponent.get('isLoaded')) {
            this.deleteRecord(hostComponent);
          }
          var serviceCache = services.findProperty('ServiceInfo.service_name', hostComponent.get('service.serviceName'));
          if (serviceCache) {
            serviceCache.host_components = serviceCache.host_components.without(hostComponent.get('id'));
          }
        } else if (id.startsWith('NAMENODE')) {
          var component = App.HostComponent.find(id),
              haNameSpace = component.get('haNameSpace');
          if (component.get('isLoaded') && haNameSpace) {
            hostComponents.findProperty('id', id).ha_name_space = haNameSpace;
          }
        }
      }, this);
      previousMasterComponentIds = hostComponents.mapProperty('id');

      App.store.safeLoadMany(this.get('model3'), hostComponents);

      if (hiveInteractiveServers.length > 1) {
        this.loadInteractiveServerStatuses(hiveInteractiveServers);
      }
      //parse service metrics from components
      services.forEach(function (item) {
        hostComponents.filterProperty('service_id', item.ServiceInfo.service_name).mapProperty('id').forEach(function (hostComponent) {
          if (!item.host_components.contains(hostComponent)) {
            item.host_components.push(hostComponent);
          }
        }, this);
        item.host_components.sort();

        var extendedModelInfo = this.mapExtendedModel(item);
        if (extendedModelInfo) {
          extendedModelInfo.passive_state = App.Service.find(item.ServiceInfo.service_name).get('passiveState');
          result.push(extendedModelInfo);
        }
      }, this);

      var stackServices = App.StackService.find().mapProperty('serviceName');
      result = misc.sortByOrder(stackServices, result);

      //load services to model
      App.store.safeLoadMany(this.get('model'), result);

      // check for new components
      if (lastKnownStatusesLength > 0) {
        if (lastKnownStatusesLength < Em.keys(App.cache.previousComponentStatuses).length) {
          App.get('router.clusterController').triggerQuickLinksUpdate();
        }
      }
    }
    console.timeEnd('App.serviceMetricsMapper execution time');
  },

  loadInteractiveServerStatuses: function loadInteractiveServerStatuses(hiveInteractiveServers) {
    var self = this;
    App.router.get('configurationController').getCurrentConfigsBySites(['hive-site', 'hive-interactive-site']).done(function (configs) {
      var hiveWebUiPort = configs.findProperty('type', 'hive-interactive-site').properties['hive.server2.webui.port'];
      var hiveWebUiProtoFlag = configs.findProperty('type', 'hive-interactive-site').properties['hive.server2.webui.use.ssl'] === 'true';
      var hiveWebUiProto = hiveWebUiProtoFlag ? 'https' : 'http';
      var hostNames = hiveInteractiveServers.mapProperty('host_name');
      var useIps = configs.findProperty('type', 'hive-interactive-site').properties['hive.server2.leadertest.use.ip'];
      var notDefinedHostIp = hostNames.find(function (hostName) {
        return !self.get('hostNameIpMap')[hostName];
      });
      if (useIps && notDefinedHostIp) {
        self.getHostNameIpMap(hostNames).done(function () {
          self.getHiveServersInteractiveStatus(hiveInteractiveServers, hiveWebUiPort, hiveWebUiProto, useIps);
        });
      } else {
        self.getHiveServersInteractiveStatus(hiveInteractiveServers, hiveWebUiPort, hiveWebUiProto, useIps);
      }
    });
  },

  getHostNameIpMap: function getHostNameIpMap(hostNames) {
    var self = this;
    return App.ajax.send({
      name: 'hosts.ips',
      data: {
        hostNames: hostNames
      },
      sender: self
    }).success(function (data) {
      data.items.forEach(function (hostData) {
        var ip = hostData.Hosts.ip;
        var hostName = hostData.Hosts.host_name;
        self.get('hostNameIpMap')[hostName] = ip;
      });
    });
  },

  getHiveServersInteractiveStatus: function getHiveServersInteractiveStatus(hiveInteractiveServers, hiveWebUiPort, hiveWebUiProto, useIps) {
    var self = this;
    hiveInteractiveServers.forEach(function (hiveInteractiveServer) {
      App.ajax.send({
        name: 'hiveServerInteractive.getStatus',
        data: {
          hsiHost: useIps ? self.hostNameIpMap[hiveInteractiveServer.host_name] : hiveInteractiveServer.host_name,
          port: hiveWebUiPort,
          proto: hiveWebUiProto
        },
        sender: self
      }).success(function (isActive) {

        var advancedName = Em.I18n.t('quick.links.label.' + (isActive ? 'active' : 'standby')) + ' ' + hiveInteractiveServer.display_name;
        var hiveInteractiveServerComponent = App.HostComponent.find().find(function (component) {
          return component.get('hostName') === hiveInteractiveServer.host_name && component.get('componentName') === 'HIVE_SERVER_INTERACTIVE';
        });
        if (hiveInteractiveServerComponent) {
          hiveInteractiveServerComponent.set('displayNameAdvanced', advancedName);
        }
      });
    });
  },

  /**
   * verify that service component has host components
   * @param {object} component
   * @param {string} name
   * @returns {boolean}
   */
  isHostComponentPresent: function isHostComponentPresent(component, name) {
    return Boolean(component.ServiceComponentInfo && component.ServiceComponentInfo.component_name === name && Array.isArray(component.host_components) && component.host_components.length > 0);
  },

  /**
   * Generate service mapped object and load data to extended models.
   *
   * @method mapExtendedModel
   * @param {Object} item - json presents service information
   * @returns {Boolean|Object} - mapped info
   */
  mapExtendedModel: function mapExtendedModel(item) {
    var finalJson = false;
    if (item && item.ServiceInfo && item.ServiceInfo.service_name == "HDFS") {
      finalJson = this.hdfsMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.HDFSService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "ONEFS") {
      finalJson = this.onefsMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.ONEFSService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "HBASE") {
      finalJson = this.hbaseMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.HBaseService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "FLUME") {
      finalJson = this.flumeMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoadMany(App.FlumeAgent, finalJson.agentJsons);
      App.store.safeLoad(App.FlumeService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "YARN") {
      finalJson = this.yarnMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.YARNService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "MAPREDUCE2") {
      finalJson = this.mapreduce2Mapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.MapReduce2Service, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "STORM") {
      finalJson = this.stormMapper(item);
      finalJson.rand = Math.random();
      this.mapQuickLinks(finalJson, item);
      App.store.safeLoad(App.StormService, finalJson);
    } else if (item && item.ServiceInfo && item.ServiceInfo.service_name == "RANGER") {
      finalJson = this.rangerMapper(item);
      finalJson.rand = Math.random();
      App.store.safeLoad(App.RangerService, finalJson);
    } else {
      finalJson = this.parseIt(item, this.config);
      finalJson.rand = Math.random();
      this.mapQuickLinks(finalJson, item);
    }

    return finalJson;
  },
  /**
   * compute display name of host-components
   * compute tooltip content of services by host-components
   * @param hostComponents
   * @param services
   */
  computeAdditionalRelations: function computeAdditionalRelations(hostComponents, services) {
    var isSecondaryNamenode = hostComponents.findProperty('component_name', 'SECONDARY_NAMENODE');
    var isRMHAEnabled = hostComponents.filterProperty('component_name', 'RESOURCEMANAGER').length > 1;
    services.setEach('tool_tip_content', '');
    // set tooltip for client-only services
    var clientOnlyServiceNames = App.get('services.clientOnly');
    clientOnlyServiceNames.forEach(function (serviceName) {
      var service = services.findProperty('ServiceInfo.service_name', serviceName);
      if (service) {
        service.tool_tip_content = Em.I18n.t('services.service.summary.clientOnlyService.ToolTip');
      }
    });
    hostComponents.forEach(function (hostComponent) {
      var service = services.findProperty('ServiceInfo.service_name', hostComponent.service_id);
      if (hostComponent) {
        // set advanced nameNode display name for HA and Federation, Active NameNodes or Standby NameNodes
        // this is useful on three places: 1) HDFS health status hover tooltip, 2) HDFS service summary 3) NameNode component on host detail page
        if (hostComponent.component_name === 'NAMENODE' && !isSecondaryNamenode) {
          var hdfs = this.hdfsMapper(service);
          var hostName = hostComponent.host_id;
          var activeNNText = Em.I18n.t('services.service.summary.nameNode.active');
          var standbyNNText = Em.I18n.t('services.service.summary.nameNode.standby');
          if (hdfs) {
            // active_name_node_id format : NAMENODE_c6401.ambari.apache.org
            if (hdfs.active_name_nodes && hdfs.active_name_nodes.contains('NAMENODE_' + hostName)) {
              hostComponent.display_name_advanced = activeNNText;
            } else if (hdfs.standby_name_nodes && hdfs.standby_name_nodes.contains('NAMENODE_' + hostName)) {
              hostComponent.display_name_advanced = standbyNNText;
            } else {
              hostComponent.display_name_advanced = null;
            }
          }
        } else if (hostComponent.component_name === 'HBASE_MASTER') {
          if (hostComponent.work_status === 'STARTED') {
            hostComponent.ha_status === true || hostComponent.ha_status == 'true' ? hostComponent.display_name_advanced = this.t('dashboard.services.hbase.masterServer.active') : hostComponent.display_name_advanced = this.t('dashboard.services.hbase.masterServer.standby');
          } else {
            hostComponent.display_name_advanced = null;
          }
        } else if (hostComponent.component_name === 'RESOURCEMANAGER' && isRMHAEnabled && hostComponent.work_status === 'STARTED') {
          switch (hostComponent.ha_status) {
            case 'ACTIVE':
              hostComponent.display_name_advanced = Em.I18n.t('dashboard.services.yarn.resourceManager.active');
              break;
            case 'STANDBY':
              hostComponent.display_name_advanced = Em.I18n.t('dashboard.services.yarn.resourceManager.standby');
              break;
          }
        } else if (hostComponent.component_name === 'HIVE_SERVER_INTERACTIVE') {
          var hiveInteractiveServerComponent = App.HostComponent.find().find(function (component) {
            return component.get('hostName') === hostComponent.host_name && component.get('componentName') === 'HIVE_SERVER_INTERACTIVE';
          });
          if (hiveInteractiveServerComponent) {
            hostComponent.display_name_advanced = hiveInteractiveServerComponent.get('displayNameAdvanced');
          }
        }
        if (service) {
          if (hostComponent.display_name_advanced) {
            service.tool_tip_content += hostComponent.display_name_advanced + " " + App.HostComponentStatus.getTextStatus(hostComponent.work_status) + "<br/>";
          } else {
            service.tool_tip_content += App.format.role(hostComponent.component_name, false) + " " + App.HostComponentStatus.getTextStatus(hostComponent.work_status) + "<br/>";
          }
        }
      }
    }, this);
  },

  setNameNodeMetricsProperties: function setNameNodeMetricsProperties(item, hostComponent) {
    var activeNameNodeConfig = this.activeNameNodeConfig,
        activeNameNodeConfigKeys = Object.keys(activeNameNodeConfig);
    activeNameNodeConfigKeys.forEach(function (key) {
      item[key][Em.get(hostComponent, 'HostRoles.host_name')] = Em.get(hostComponent, activeNameNodeConfig[key]);
    });
  },

  /**
   * Map quick links to services:OOZIE,GANGLIA
   * @param finalJson
   * @param item
   */
  mapQuickLinks: function mapQuickLinks(finalJson, item) {
    if (!(item && item.ServiceInfo)) return;
    var quickLinks = {
      OOZIE: [19],
      GANGLIA: [20],
      STORM: [31],
      FALCON: [32],
      RANGER: [33],
      SPARK: [34],
      ACCUMULO: [35],
      ATLAS: [36],
      AMBARI_METRICS: [37],
      LOGSEARCH: [38]
    };
    if (quickLinks[item.ServiceInfo.service_name]) finalJson.quick_links = quickLinks[item.ServiceInfo.service_name];
  },

  onefsMapper: function onefsMapper(item) {
    var finalConfig = jQuery.extend({}, this.config);
    item.components.forEach(function (component) {
      if (this.isHostComponentPresent(component, 'ONEFS_CLIENT')) {
        item.metrics = component.metrics;
        item.decommission_data_nodes = [];
        item.dead_data_nodes = [];
        item.live_data_nodes = [];
        if (component.metrics && component.metrics.dfs && component.metrics.dfs.namenode) {
          item.metrics_not_available = false;
          for (var host in App.parseJSON(component.metrics.dfs.namenode.DecomNodes)) {
            item.decommission_data_nodes.push('DATANODE' + '_' + host);
          }
          for (var host in App.parseJSON(component.metrics.dfs.namenode.DeadNodes)) {
            item.dead_data_nodes.push('DATANODE' + '_' + host);
          }
          for (var host in App.parseJSON(component.metrics.dfs.namenode.LiveNodes)) {
            item.live_data_nodes.push('DATANODE' + '_' + host);
          }
        } else {
          item.metrics_not_available = true;
        }
        finalConfig = jQuery.extend(finalConfig, this.onefsConfig);
      }
    }, this);
    return this.parseIt(item, finalConfig);
  },

  hdfsMapper: function hdfsMapper(item) {
    var _this = this;

    var finalConfig = jQuery.extend({}, this.config);
    // Change the JSON so that it is easy to map
    var hdfsConfig = this.hdfsConfig,
        activeNameNodeConfigInitial = Object.keys(this.activeNameNodeConfig).reduce(function (obj, key) {
      return _extends({}, obj, _defineProperty({}, key, {}));
    }, {});
    _extends(item, activeNameNodeConfigInitial);
    item.components.forEach(function (component) {
      var hostComponents = component.host_components,
          firstHostComponent = hostComponents[0];
      if (_this.isHostComponentPresent(component, 'NAMENODE')) {
        //enabled HA
        if (hostComponents.length > 1) {
          var nameSpacesWithActiveNameNodes = [],
              unknownNameNodes = [];
          item.active_name_nodes = [];
          item.standby_name_nodes = [];
          hostComponents.forEach(function (hc) {
            var haState = Em.get(hc, 'metrics.dfs.FSNamesystem.HAState'),
                hostName = Em.get(hc, 'HostRoles.host_name'),
                clusterIdValue = Em.get(hc, 'metrics.dfs.namenode.ClusterId'),
                id = 'NAMENODE_' + hostName;
            switch (haState) {
              case 'active':
                nameSpacesWithActiveNameNodes.push(clusterIdValue);
                item.active_name_nodes.push(id);
                break;
              case 'standby':
                item.standby_name_nodes.push(id);
                break;
              default:
                unknownNameNodes.push({
                  hostName: hostName,
                  clusterIdValue: clusterIdValue
                });
                break;
            }
            _this.setNameNodeMetricsProperties(item, hc);
          });
          unknownNameNodes.forEach(function (nameNode) {
            if (nameSpacesWithActiveNameNodes.contains(nameNode.clusterIdValue)) {
              item.standby_name_nodes.push('NAMENODE_' + nameNode.hostName);
            }
          });
        } else {
          _this.setNameNodeMetricsProperties(item, firstHostComponent);
        }

        item.nameNodeComponent = component;
        finalConfig = jQuery.extend(finalConfig, hdfsConfig);
        // Get the live, dead & decommission nodes from string json
        if (firstHostComponent.metrics && firstHostComponent.metrics.dfs && firstHostComponent.metrics.dfs.namenode) {
          item.metrics_not_available = false;
          var decommissionNodesJson = App.parseJSON(firstHostComponent.metrics.dfs.namenode.DecomNodes);
          var deadNodesJson = App.parseJSON(firstHostComponent.metrics.dfs.namenode.DeadNodes);
          var liveNodesJson = App.parseJSON(firstHostComponent.metrics.dfs.namenode.LiveNodes);
        } else {
          item.metrics_not_available = true;
        }
        item.decommission_data_nodes = [];
        item.dead_data_nodes = [];
        item.live_data_nodes = [];
        for (var host in decommissionNodesJson) {
          item.decommission_data_nodes.push('DATANODE_' + host);
        }
        for (var _host in deadNodesJson) {
          item.dead_data_nodes.push('DATANODE_' + _host);
        }
        for (var _host2 in liveNodesJson) {
          item.live_data_nodes.push('DATANODE_' + _host2);
        }
        item.name_node_id = 'NAMENODE_' + firstHostComponent.HostRoles.host_name;
      }
      if (_this.isHostComponentPresent(component, 'JOURNALNODE')) {
        item.journal_nodes = hostComponents.map(function (hc) {
          return 'JOURNALNODE_' + hc.HostRoles.host_name;
        });
      }
      if (_this.isHostComponentPresent(component, 'SECONDARY_NAMENODE')) {
        item.sname_node_id = 'SECONDARY_NAMENODE_' + firstHostComponent.HostRoles.host_name;
      }
    });
    // Map
    var finalJson = this.parseIt(item, finalConfig);
    finalJson.quick_links = [1, 2, 3, 4];

    return finalJson;
  },

  yarnMapper: function yarnMapper(item) {
    var self = this;
    var finalConfig = jQuery.extend({}, this.config);
    // Change the JSON so that it is easy to map
    var yarnConfig = this.yarnConfig;
    item.components.forEach(function (component) {
      if (this.isHostComponentPresent(component, "RESOURCEMANAGER")) {
        item.resourceManagerComponent = component;

        // if YARN has two host components, ACTIVE one should be first in component.host_components array for proper metrics mapping
        if (component.host_components.length === 2) {
          var activeRM = component.host_components.findProperty('HostRoles.ha_state', 'ACTIVE');
          var activeHostComponentIndex = component.host_components.indexOf(activeRM);
          self.setActiveAsFirstHostComponent(component, activeHostComponentIndex);
          if (activeRM) {
            item.active_resource_manager_id = "RESOURCEMANAGER" + "_" + activeRM.HostRoles.host_name;
          }
        }

        if (component.host_components[0].metrics && component.host_components[0].metrics.yarn) {
          var root = component.host_components[0].metrics.yarn.Queue.root;
          component.queue = JSON.stringify({
            'root': self.parseObject(root)
          });
        }
        // extend config
        finalConfig = jQuery.extend(finalConfig, yarnConfig);
        item.resource_manager_id = "RESOURCEMANAGER" + "_" + component.host_components[0].HostRoles.host_name;
      }
      if (this.isHostComponentPresent(component, "APP_TIMELINE_SERVER")) {
        item.app_timeline_server_id = "APP_TIMELINE_SERVER" + "_" + component.host_components[0].HostRoles.host_name;
      }
    }, this);
    // Map
    var finalJson = this.parseIt(item, finalConfig);
    finalJson.quick_links = [23, 24, 25, 26];
    return finalJson;
  },

  parseObject: function parseObject(obj) {
    var res = {};
    for (var p in obj) {
      if (obj.hasOwnProperty(p)) {
        if (obj[p] instanceof Object) {
          res[p] = this.parseObject(obj[p]);
        }
      }
    }
    return res;
  },

  mapreduce2Mapper: function mapreduce2Mapper(item) {
    var finalConfig = jQuery.extend({}, this.config);
    // Change the JSON so that it is easy to map
    var mapReduce2Config = this.mapReduce2Config;
    item.components.forEach(function (component) {
      if (this.isHostComponentPresent(component, "HISTORYSERVER")) {
        item.jobHistoryServerComponent = component;
        finalConfig = jQuery.extend(finalConfig, mapReduce2Config);
        item.job_history_server_id = "HISTORYSERVER" + "_" + component.host_components[0].HostRoles.host_name;
      }
    }, this);
    // Map
    var finalJson = this.parseIt(item, finalConfig);
    finalJson.quick_links = [27, 28, 29, 30];

    return finalJson;
  },
  hbaseMapper: function hbaseMapper(item) {
    var self = this;
    // Change the JSON so that it is easy to map
    var finalConfig = jQuery.extend({}, this.config);
    var hbaseConfig = this.hbaseConfig;
    item.components.forEach(function (component) {
      if (this.isHostComponentPresent(component, "HBASE_MASTER")) {
        item.masterComponent = component;
        finalConfig = jQuery.extend(finalConfig, hbaseConfig);
        if (component.host_components.length) {
          var activeMaster = component.host_components.findProperty('metrics.hbase.master.IsActiveMaster', 'true');
          var activeHostComponentIndex = component.host_components.indexOf(activeMaster);
          self.setActiveAsFirstHostComponent(component, activeHostComponentIndex);
          var regionsArray = null;
          if (!Em.none(Em.get(component.host_components[0], 'metrics.master.AssignmentManager.ritCount'))) {
            regionsArray = App.parseJSON(component.host_components[0].metrics.master.AssignmentManager.ritCount);
          }
          //regions_in_transition can have various type of value: null, array or int
          if (Array.isArray(regionsArray)) {
            item.regions_in_transition = regionsArray.length;
          } else {
            item.regions_in_transition = regionsArray == null ? 0 : regionsArray;
          }
        }
        item.master_id = "HBASE_MASTER" + "_" + component.host_components[0].HostRoles.host_name;
      }
    }, this);
    // Map
    var finalJson = this.parseIt(item, finalConfig);
    finalJson.average_load = parseFloat(finalJson.average_load).toFixed(2);
    finalJson.quick_links = [13, 14, 15, 16, 17, 18];
    return finalJson;
  },

  /**
   * Sets the active host component as the first host component
   * @param component {Object}
   * @param activeHostComponentIndex {Number}
   */
  setActiveAsFirstHostComponent: function setActiveAsFirstHostComponent(component, activeHostComponentIndex) {
    // important: active component always at host_components[0];
    if (activeHostComponentIndex && activeHostComponentIndex !== -1) {
      var tmp = component.host_components[activeHostComponentIndex];
      component.host_components[activeHostComponentIndex] = component.host_components[0];
      component.host_components[0] = tmp;
    }
  },

  /**
   * Flume is different from other services, in that the important
   * data is in customizable channels. Hence we directly transfer
   * data into the JSON object.
   */
  flumeMapper: function flumeMapper(item) {
    var self = this;
    var finalConfig = jQuery.extend({}, this.config);
    var flumeConfig = this.flumeConfig;
    finalConfig = jQuery.extend(finalConfig, flumeConfig);
    var finalJson = self.parseIt(item, finalConfig);
    var flumeHandlers = item.components.findProperty('ServiceComponentInfo.component_name', "FLUME_HANDLER");
    flumeHandlers = flumeHandlers ? flumeHandlers.host_components : [];
    finalJson.agents = [];
    finalJson.agentJsons = [];
    flumeHandlers.forEach(function (flumeHandler) {
      var hostName = flumeHandler.HostRoles.host_name;
      flumeHandler.processes.forEach(function (process) {
        var agentJson = self.parseIt(process, self.flumeAgentConfig);
        var agentId = agentJson.name + "-" + hostName;
        finalJson.agents.push(agentId);
        agentJson.id = agentId;
        finalJson.agentJsons.push(agentJson);
      });
    });
    return finalJson;
  },

  /**
   * Storm mapper
   */
  stormMapper: function stormMapper(item) {
    var finalConfig = jQuery.extend({}, this.config);
    var stormConfig = this.stormConfig;
    var metricsInfoComponent = /^2.1/.test(App.get('currentStackVersionNumber')) ? 'STORM_REST_API' : 'STORM_UI_SERVER';
    var metricsPath = {
      STORM_REST_API: 'metrics.api.cluster.summary',
      STORM_UI_SERVER: 'metrics.api.v1.cluster.summary'
    }[metricsInfoComponent];
    var restApiMetrics = {};

    item.components.forEach(function (component) {
      var componentName = component.ServiceComponentInfo.component_name;
      if (component.ServiceComponentInfo && componentName == metricsInfoComponent) {
        if (Em.get(component, metricsPath)) {
          $.extend(restApiMetrics, App.keysDottedToCamelCase(Em.get(component, metricsPath)));
          if (metricsInfoComponent == 'STORM_UI_SERVER') {
            restApiMetrics.topologies = Em.get(component, 'metrics.api.v1.topology.summary.length');
            if (stringUtils.compareVersions(App.get('currentStackVersionNumber'), '2.3') > -1) {
              restApiMetrics.nimbusUptime = Em.getWithDefault(component, 'metrics.api.v1.nimbus.summary.0.nimbusUpTime', Em.I18n.t('services.service.summary.notRunning'));
            }
          } else {
            restApiMetrics.nimbusUptime = dateUtils.timingFormat(restApiMetrics.nimbusUptime * 1000);
          }
        }
        finalConfig = jQuery.extend({}, finalConfig, stormConfig);
      }
    });
    item.restApiComponent = restApiMetrics;
    return this.parseIt(item, finalConfig);
  },

  /**
   * Ranger mapper
   */
  rangerMapper: function rangerMapper(item) {
    var finalConfig = jQuery.extend({}, this.config);
    var rangerConfig = this.rangerConfig;
    finalConfig = jQuery.extend({}, finalConfig, rangerConfig);
    return this.parseIt(item, finalConfig);
  }
});

});

require.register("mappers/socket/alert_definitions_mapper_adapter", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertDefinitionsMapperAdapter = App.QuickDataMapper.create({

  /**
   * @param {object} event
   */
  map: function map(event) {
    var _this = this;

    event.clusters[App.get('clusterId')].alertDefinitions.forEach(function (alertDefinition) {
      // TODO: remove when fields will be correctly named
      alertDefinition.id = alertDefinition.definitionId;
      alertDefinition.component_name = alertDefinition.componentName;
      alertDefinition.service_name = alertDefinition.serviceName;

      if (event.eventType === 'UPDATE') {
        // UPDATE/ADD
        App.alertDefinitionsMapper.map({
          items: [{
            AlertDefinition: alertDefinition
          }]
        }, true);
      } else if (event.eventType === 'DELETE') {
        _this.deleteRecord(App.AlertDefinition.find(alertDefinition.id));
      }
    });
  }
});

});

require.register("mappers/socket/alert_groups_mapper_adapter", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertGroupsMapperAdapter = App.QuickDataMapper.create({

  /**
   * @param {object} event
   */
  map: function map(event) {
    var _this = this;

    event.groups.forEach(function (alertGroup) {
      if (event.updateType === 'UPDATE' || event.updateType === 'CREATE') {
        var definitions = alertGroup.definitions,
            targets = alertGroup.targets;

        if (definitions) {
          alertGroup.definitions = _this.convertIdsToObjects(definitions);
        }
        if (targets) {
          alertGroup.targets = _this.convertIdsToObjects(targets);
        }
        App.alertGroupsMapper.map({
          items: [{
            AlertGroup: alertGroup
          }]
        }, true);
      } else if (event.updateType === 'DELETE') {
        _this.deleteRecord(App.AlertGroup.find(alertGroup.id));
      }
    });
    App.router.get('manageAlertGroupsController').toggleProperty('changeTrigger');
  },

  convertIdsToObjects: function convertIdsToObjects(arr) {
    return arr.map(function (id) {
      return {
        id: id
      };
    });
  }
});

});

require.register("mappers/socket/alert_summary_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.alertSummaryMapper = App.QuickDataMapper.create({

  /**
   * @param {object} event
   */
  map: function map(event) {
    var data = {
      alerts_summary_grouped: []
    };
    var definitions = event.summaries[App.get('clusterId')];
    for (var name in definitions) {
      data.alerts_summary_grouped.push(definitions[name]);
    }
    App.alertDefinitionSummaryMapper.map(data);
  }
});

});

require.register("mappers/socket/host_component_status_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.hostComponentStatusMapper = App.QuickDataMapper.create({

  config: {
    workStatus: 'currentState',
    staleConfigs: 'staleConfigs',
    passiveState: 'maintenanceState'
  },

  /**
   * @param {object} event
   */
  map: function map(event) {
    var _this = this;

    event.hostComponents.forEach(function (componentState) {
      var hostComponent = App.HostComponent.find(componentState.componentName + '_' + componentState.hostName);
      _this.updatePropertiesByConfig(hostComponent, componentState, _this.config);
      _this.updateComponentsWithStaleConfigs(componentState);
      App.componentsStateMapper.updateComponentCountOnStateChange(componentState);
    });
  },

  /**
   * @param {object} componentState
   */
  updateComponentsWithStaleConfigs: function updateComponentsWithStaleConfigs(componentState) {
    var staleConfigHostsMap = App.cache.staleConfigsComponentHosts;

    if (!Em.isNone(componentState.staleConfigs)) {
      var hosts = staleConfigHostsMap[componentState.componentName] || [];
      if (componentState.staleConfigs) {
        hosts.push(componentState.hostName);
      } else {
        hosts.removeObject(componentState.hostName);
      }
      App.componentsStateMapper.updateStaleConfigsHosts(componentState.componentName, hosts);
    }
  }
});

});

require.register("mappers/socket/host_state_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.hostStateMapper = App.QuickDataMapper.create({

  config: {
    alertsSummary: 'alerts_summary',
    healthStatus: 'host_status',
    state: 'host_state',
    lastHeartBeatTime: 'last_heartbeat_time',
    passiveState: 'maintenance_state'
  },

  /**
   * @param {object} event
   */
  map: function map(event) {
    //TODO event should have properties named in CamelCase format
    this.updatePropertiesByConfig(App.Host.find(event.host_name), event, this.config);
  }
});

});

require.register("mappers/socket/service_state_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.serviceStateMapper = App.QuickDataMapper.create({

  config: {
    passiveState: 'maintenance_state',
    workStatus: 'state'
  },

  /**
   * @param {object} event
   */
  map: function map(event) {
    //TODO event should have properties named in CamelCase format
    this.updatePropertiesByConfig(App.Service.find(event.service_name), event, this.config);
    var cachedService = App.cache['services'].findProperty('ServiceInfo.service_name', event.service_name);
    if (event.state && cachedService) {
      cachedService.ServiceInfo.state = event.state;
    }
  }
});

});

require.register("mappers/socket/topology_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.topologyMapper = App.QuickDataMapper.create({

  /**
   *
   * @param {object} event
   */
  map: function map(event) {
    var hosts = event.clusters[App.get('clusterId')].hosts;

    if (event.clusters[App.get('clusterId')].components) {
      this.applyComponentTopologyChanges(event.clusters[App.get('clusterId')].components, event.eventType);
    }
    if (hosts) {
      var hostNames = hosts.filterProperty('hostName').mapProperty('hostName');
      if (event.eventType === 'DELETE') {
        App.get('allHostNames').removeObjects(hostNames);
      } else if (event.eventType === 'UPDATE') {
        App.get('allHostNames').pushObjects(hostNames);
      }
      /**
       * Since event message doesn't contain entire data of host and UI model doesn't contain all hosts of cluster,
       * App.Host model should be updated only through <code>updateHost</code> request.
       */
      App.router.get('updateController').updateHost(Em.K, null, true);
    }
  },

  /**
   *
   * @param {Array} components
   * @param {string} eventType
   */
  applyComponentTopologyChanges: function applyComponentTopologyChanges(components, eventType) {
    var _this = this;

    components.forEach(function (component) {
      component.hostNames.forEach(function (hostName, index) {
        if (eventType === 'UPDATE' && !component.commandParams.version) {
          _this.addServiceIfNew(component.serviceName);
          _this.createHostComponent(component, hostName, component.publicHostNames[index]);
          App.componentsStateMapper.updateComponentCountOnCreate(component);
        } else if (eventType === 'DELETE') {
          _this.deleteHostComponent(component, hostName);
          _this.deleteServiceIfHasNoComponents(component.serviceName);
          App.componentsStateMapper.updateComponentCountOnDelete(component);
        }
      });
    });
  },

  /**
   *
   * @param {string} serviceName
   */
  addServiceIfNew: function addServiceIfNew(serviceName) {
    if (!App.Service.find(serviceName).get('isLoaded')) {
      App.store.safeLoad(App.Service, {
        id: serviceName,
        service_name: serviceName,
        work_status: 'INIT',
        passive_state: 'OFF',
        host_components: []
      });
    }
  },

  /**
   *
   * @param {string} serviceName
   */
  deleteServiceIfHasNoComponents: function deleteServiceIfHasNoComponents(serviceName) {
    if (App.Service.find(serviceName).get('isLoaded') && App.Service.find(serviceName).get('hostComponents.length') === 0) {
      this.deleteRecord(App.Service.find(serviceName));
    }
  },

  /**
   *
   * @param {object} component
   * @param {string} hostName
   */
  deleteHostComponent: function deleteHostComponent(component, hostName) {
    var id = App.HostComponent.getId(component.componentName, hostName);
    if (!App.HostComponent.find(id).get('isLoaded')) {
      //App.HostComponent does not contain all host-components of cluster
      return;
    }
    this.deleteRecord(App.HostComponent.find(id));
    var host = App.Host.find(hostName);
    this.updateHostComponentsOfHost(host, host.get('hostComponents').rejectProperty('id', id).mapProperty('id'));

    var service = App.Service.find(component.serviceName);
    this.updateHostComponentsOfService(service, service.get('hostComponents').rejectProperty('id', id).mapProperty('id'));
  },

  /**
   *
   * @param {object} component
   * @param {string} hostName
   * @param {string} publicHostName
   */
  createHostComponent: function createHostComponent(component, hostName, publicHostName) {
    var id = App.HostComponent.getId(component.componentName, hostName);
    var host = App.Host.find(hostName);
    var service = App.Service.find(component.serviceName);

    App.store.safeLoad(App.HostComponent, {
      id: id,
      host_id: hostName,
      display_name: component.displayName,
      service_id: component.serviceName,
      host_name: hostName,
      passive_state: 'OFF',
      work_status: 'INIT',
      public_host_name: publicHostName,
      component_name: component.componentName
    });

    if (host.get('isLoaded')) {
      this.updateHostComponentsOfHost(host, host.get('hostComponents').mapProperty('id').concat(id));
    }
    this.updateHostComponentsOfService(service, service.get('hostComponents').mapProperty('id').concat(id));
  },

  /**
   *
   * @param {Em.Object} host
   * @param {Array} hostComponents
   */
  updateHostComponentsOfHost: function updateHostComponentsOfHost(host, hostComponents) {
    var updatedHost = {};
    for (var i in App.hostsMapper.config) {
      if (host.get(stringUtils.underScoreToCamelCase(i)) !== undefined) {
        updatedHost[i] = host.get(stringUtils.underScoreToCamelCase(i));
      }
    }
    updatedHost.host_components = hostComponents;
    App.store.safeLoad(App.Host, updatedHost);
  },

  /**
   *
   * @param {Em.Object} service
   * @param {Array} hostComponents
   */
  updateHostComponentsOfService: function updateHostComponentsOfService(service, hostComponents) {
    var cacheService = App.cache['services'].findProperty('ServiceInfo.service_name', service.get('serviceName'));
    if (cacheService) {
      cacheService.host_components = hostComponents;
    }
  }
});

});

require.register("mappers/socket/upgrade_state_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.upgradeStateMapper = App.QuickDataMapper.create({

  /**
   * @param {object} event
   */
  map: function map(event) {
    var controller = App.router.get('mainAdminStackAndUpgradeController');
    if (event.type === 'CREATE') {
      controller.restoreLastUpgrade({ Upgrade: event });
    }
    //TODO rename type to eventType
    if (event.type === 'UPDATE' && controller.get('upgradeId') === event.request_id) {
      if (!Em.isNone(event.request_status)) {
        App.set('upgradeState', event.request_status);
        controller.setDBProperty('upgradeState', event.request_status);
      }
      if (!Em.isNone(event.suspended)) {
        controller.set('isSuspended', event.suspended);
        controller.setDBProperty('isSuspended', event.suspended);
      }
    }
  }
});

});

require.register("mappers/stack_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.stackMapper = App.QuickDataMapper.create({
  modelStack: App.Stack,
  modelOS: App.OperatingSystem,
  modelRepo: App.Repository,
  modelServices: App.ServiceSimple,

  configStack: {
    id: 'id',
    stack_name: 'stack_name',
    stack_version: 'stack_version',
    stack_default: 'stack_default',
    stack_repo_update_link_exists: 'stack_repo_update_link_exists',
    show_available: 'show_available',
    type: 'type',
    repository_version: 'repository_version',
    active: 'active',
    parent_stack_version: 'parent_stack_version',
    min_upgrade_version: 'min_upgrade_version',
    min_jdk_version: 'min_jdk',
    max_jdk_version: 'max_jdk',
    is_selected: 'is_selected',
    config_types: 'config_types',
    use_redhat_satellite: 'use_redhat_satellite',
    stack_services_key: 'stack_services',
    stack_services_type: 'array',
    stack_services: {
      item: 'id'
    },
    operating_systems_key: 'operating_systems',
    operating_systems_type: 'array',
    operating_systems: {
      item: 'id'
    }
  },

  configOS: {
    id: 'id',
    os_type: 'os_type',
    stack_name: 'stack_name',
    stack_version: 'stack_version',
    stack_id: 'stack_id',
    is_selected: 'is_selected',
    repositories_key: 'repositories',
    repositories_type: 'array',
    repositories: {
      item: 'id'
    }
  },

  configService: {
    id: 'id',
    name: 'name',
    display_name: 'display_name',
    latest_version: 'latest_version'
  },

  configRepository: {
    id: 'id',
    base_url: 'base_url',
    base_url_init: 'base_url',
    default_base_url: 'default_base_url',
    latest_base_url: 'latest_base_url',
    mirrors_list: 'mirrors_list',
    os_type: 'os_type',
    original_repo_id: 'repo_id',
    repo_id: 'repo_id',
    repo_name: 'repo_name',
    stack_name: 'stack_name',
    stack_version: 'stack_version',
    operating_system_id: 'os_id',
    components: 'components',
    distribution: 'distribution',
    tags: 'tags',
    applicable_services: 'applicable_services'
  },

  map: function map(json) {
    var modelStack = this.get('modelStack');
    var modelOS = this.get('modelOS');
    var modelRepo = this.get('modelRepo');
    var modelServices = this.get('modelServices');
    var resultOS = [];
    var resultRepo = [];
    var resultServices = [];

    var item = json;
    var stack = item.VersionDefinition;
    if (!stack.id) {
      stack.id = stack.stack_name + '-' + stack.stack_version + '-' + stack.repository_version; //HDP-2.5-2.5.0.0
    }
    var operatingSystemsArray = [];
    var servicesArray = [];

    item.operating_systems.forEach(function (ops) {
      var operatingSystems = ops.OperatingSystems;

      var repositoriesArray = [];
      ops.repositories.forEach(function (repo) {
        repo.Repositories.id = [stack.id, repo.Repositories.os_type, repo.Repositories.repo_id].join('-');
        repo.Repositories.os_id = [stack.id, repo.Repositories.os_type].join('-');
        if (!repo.Repositories.latest_base_url) repo.Repositories.latest_base_url = repo.Repositories.base_url;
        resultRepo.push(this.parseIt(repo.Repositories, this.get('configRepository')));
        repositoriesArray.pushObject(repo.Repositories);
      }, this);

      operatingSystems.id = stack.id + "-" + operatingSystems.os_type;
      operatingSystems.stack_id = operatingSystems.stack_name + "-" + operatingSystems.stack_version;
      operatingSystems.repositories = repositoriesArray;
      operatingSystems.is_selected = ops.isSelected == true || ops.isSelected == undefined;
      resultOS.push(this.parseIt(operatingSystems, this.get('configOS')));
      operatingSystemsArray.pushObject(operatingSystems);
    }, this);

    stack.stack_services.forEach(function (service) {
      var serviceObj = {
        id: service.name + '-' + stack.id,
        name: service.name,
        display_name: service.display_name,
        latest_version: service.versions ? service.versions[0] : ''
      };
      resultServices.push(this.parseIt(serviceObj, this.get('configService')));
      servicesArray.pushObject(serviceObj);
    }, this);

    //In case ambari_managed_repositories is undefined, set use_redhat_satellite to be false
    stack.use_redhat_satellite = item.operating_systems[0].OperatingSystems.ambari_managed_repositories === false;
    stack.stack_services = servicesArray;
    stack.operating_systems = operatingSystemsArray;
    App.store.safeLoadMany(modelRepo, resultRepo);
    App.store.safeLoadMany(modelOS, resultOS);
    App.store.safeLoadMany(modelServices, resultServices);
    App.store.safeLoad(modelStack, this.parseIt(stack, this.get('configStack')));
  }
});

});

require.register("mappers/stack_service_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.stackServiceMapper = App.QuickDataMapper.create({
  model: App.StackService,
  component_model: App.StackServiceComponent,

  config: {
    id: 'service_name',
    stack_id: 'stack_id',
    service_name: 'service_name',
    service_type: 'service_type',
    display_name: 'display_name',
    config_types: 'config_types',
    comments: 'comments',
    service_version: 'service_version',
    stack_name: 'stack_name',
    stack_version: 'stack_version',
    selection: 'selection',
    is_mandatory: 'is_mandatory',
    is_selected: 'is_selected',
    is_installed: 'is_installed',
    is_installable: 'is_installable',
    is_service_with_widgets: 'is_service_with_widgets',
    required_services: 'required_services',
    service_check_supported: 'service_check_supported',
    support_delete_via_ui: 'support_delete_via_ui',
    service_components_key: 'service_components',
    service_components_type: 'array',
    service_components: {
      item: 'id'
    }
  },

  component_config: {
    id: 'component_name',
    component_name: 'component_name',
    display_name: 'display_name',
    cardinality: 'cardinality',
    custom_commands: 'custom_commands',
    reassign_allowed: 'reassign_allowed',
    decommission_allowed: 'decommission_allowed',
    has_bulk_commands_definition: 'has_bulk_commands_definition',
    bulk_commands_display_name: 'bulk_commands_display_name',
    bulk_commands_master_component_name: 'bulk_commands_master_component_name',
    service_name: 'service_name',
    component_category: 'component_category',
    rolling_restart_supported: 'rolling_restart_supported',
    is_master: 'is_master',
    is_client: 'is_client',
    component_type: 'component_type',
    stack_name: 'stack_name',
    stack_version: 'stack_version',
    stack_service_id: 'service_name',
    dependencies_key: 'dependencies',
    dependencies_type: 'array',
    dependencies: {
      item: 'Dependencies'
    }
  },

  mapStackServices: function mapStackServices(json) {
    App.set('isStackServicesLoaded', false);
    App.clearModels([this.get('model'), this.get('component_model')]);
    App.resetDsStoreTypeMap(App.StackServiceComponent);
    App.resetDsStoreTypeMap(App.StackService);
    this.map(json);
    App.set('isStackServicesLoaded', true);
  },

  map: function map(json) {
    var model = this.get('model');
    var result = [];
    var stackServiceComponents = [];
    var nonInstallableServices = ['KERBEROS'];
    var displayOrderLength = App.StackService.displayOrder.length;
    var items = json.items.map(function (item, index) {
      var displayOrderIndex = App.StackService.displayOrder.indexOf(item.StackServices.service_name);
      return $.extend(item, {
        index: displayOrderIndex == -1 ? displayOrderLength + index : displayOrderIndex
      });
    }).sortProperty('index');
    items.forEach(function (item) {
      var stackService = item.StackServices;
      var serviceComponents = [];
      item.components.forEach(function (serviceComponent) {
        var dependencies = serviceComponent.dependencies.map(function (dependecy) {
          return { Dependencies: App.keysUnderscoreToCamelCase(App.permit(dependecy.Dependencies, ['component_name', 'scope', 'service_name'])) };
        });
        serviceComponent.StackServiceComponents.id = serviceComponent.StackServiceComponents.component_name;
        serviceComponent.StackServiceComponents.dependencies = dependencies;
        serviceComponents.push(serviceComponent.StackServiceComponents);
        var parsedResult = this.parseIt(serviceComponent.StackServiceComponents, this.get('component_config'));
        if (parsedResult.id == 'MYSQL_SERVER') {
          parsedResult.custom_commands = parsedResult.custom_commands.without('CLEAN');
        }
        stackServiceComponents.push(parsedResult);
      }, this);
      stackService.stack_id = stackService.stack_name + '-' + stackService.stack_version;
      stackService.service_components = serviceComponents;
      stackService.is_service_with_widgets = item.artifacts.someProperty('Artifacts.artifact_name', 'widgets_descriptor');
      // @todo: replace with server response value after API implementation
      if (nonInstallableServices.contains(stackService.service_name)) {
        stackService.is_installable = false;
        stackService.is_selected = false;
      }
      if (stackService.selection === "MANDATORY") {
        stackService.is_mandatory = true;
      }
      result.push(this.parseIt(stackService, this.get('config')));
    }, this);
    App.store.safeLoadMany(this.get('component_model'), stackServiceComponents);
    App.store.safeLoadMany(model, result);
  }

});

});

require.register("mappers/stack_upgrade_history_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
var stringUtils = require('utils/string_utils');

App.stackUpgradeHistoryMapper = App.QuickDataMapper.create({
  model: App.StackUpgradeHistory,

  config: {
    "id": "Upgrade.request_id",
    "request_id": "Upgrade.request_id",
    "upgrade_id": "Upgrade.upgrade_id",
    "cluster_name": "Upgrade.cluster_name",
    "direction": "Upgrade.direction",
    "associated_version": "Upgrade.associated_version",
    "versions": "Upgrade.versions",
    "end_time": "Upgrade.end_time",
    "start_time": "Upgrade.start_time",
    "create_time": "Upgrade.create_time",
    "request_status": "Upgrade.request_status",
    "upgrade_type": "Upgrade.upgrade_type",
    "downgrade_allowed": "Upgrade.downgrade_allowed",
    "skip_failures": "Upgrade.skip_failures",
    "skip_service_check_failures": "Upgrade.skip_service_check_failures"
  },

  map: function map(json) {
    App.set('isStackUpgradeHistoryLoaded', false);
    var model = this.get('model');
    var result = [];
    json.items.forEach(function (item) {
      var parseResult = this.parseIt(item, this.get('config'));
      if (parseResult.direction === 'UPGRADE') {
        parseResult.to_version = parseResult.associated_version;
      } else {
        parseResult.from_version = parseResult.associated_version;
      }
      result.push(parseResult);
    }, this);

    App.store.safeLoadMany(this.get('model'), result);
    App.set('isStackUpgradeHistoryLoaded', true);
  }
});

});

require.register("mappers/stack_version_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
var stringUtils = require('utils/string_utils');

App.stackVersionMapper = App.QuickDataMapper.create({
  modelStackVerion: App.StackVersion,

  modelStack: {
    "id": "id",
    "cluster_name": "cluster_name",
    "stack": "stack",
    "version": "version",
    "repository_version_id": "repository_version_id",
    "state": "state",
    "not_installed_hosts": "host_states.NOT_INSTALLED",
    "installing_hosts": "host_states.INSTALLING",
    "installed_hosts": "host_states.INSTALLED",
    "install_failed_hosts": "host_states.INSTALL_FAILED",
    "out_of_sync_hosts": "host_states.OUT_OF_SYNC",
    "upgrading_hosts": "host_states.UPGRADING",
    "upgraded_hosts": "host_states.UPGRADED",
    "upgrade_failed_hosts": "host_states.UPGRADE_FAILED",
    "current_hosts": "host_states.CURRENT",
    "supports_revert": "supports_revert",
    "revert_upgrade_id": "revert_upgrade_id"
  },

  map: function map(json) {
    var modelStackVerion = this.get('modelStackVerion');
    var resultStack = [];

    if (json && json.items) {
      json.items.sort(function (a, b) {
        return stringUtils.compareVersions(a.repository_versions[0].RepositoryVersions.repository_version, b.repository_versions[0].RepositoryVersions.repository_version);
      });
      json.items.forEach(function (item) {
        var stack = item.ClusterStackVersions;
        stack.repository_version_id = item.ClusterStackVersions.repository_version;
        /**
         * this property contains array of hosts on which repoversion wasn't installed
         * possible states:
         * <code>INSTALLING<code>
         * <code>INSTALL_FAILED<code>
         * <code>OUT_OF_SYNC<code>
         */
        stack.host_states.NOT_INSTALLED = item.ClusterStackVersions.host_states.INSTALLING.concat(item.ClusterStackVersions.host_states.INSTALL_FAILED || []).concat(item.ClusterStackVersions.host_states.OUT_OF_SYNC || []);

        /**
         * this property contains array of hosts on which repoversion was installed
         * but state of repoveriosn for this hosts can be any postinstalled state
         * possible states:
         * <code>INSTALLED<code>
         * <code>UPGRADING<code>
         * <code>UPGRADED<code>
         * <code>UPGRADE_FAILED<code>
         */
        stack.host_states.INSTALLED = item.ClusterStackVersions.host_states.INSTALLED.concat(item.ClusterStackVersions.host_states.UPGRADING || []).concat(item.ClusterStackVersions.host_states.UPGRADED || []).concat(item.ClusterStackVersions.host_states.UPGRADE_FAILED || []);

        if (item.repository_versions && item.repository_versions[0]) {
          item.repository_versions[0].RepositoryVersions.stackVersionId = item.ClusterStackVersions.id;
          App.repoVersionMapper.map({
            "items": item.repository_versions,
            "stackServices": item.ClusterStackVersions.repository_summary && item.ClusterStackVersions.repository_summary.services
          }, true, true);
        }
        resultStack.push(this.parseIt(stack, this.get('modelStack')));
      }, this);
    }
    App.store.safeLoadMany(modelStackVerion, resultStack);
  }
});

});

require.register("mappers/target_cluster_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.targetClusterMapper = App.QuickDataMapper.create({
  model: App.TargetCluster,
  config: {
    id: 'name',
    name: 'name',
    execute: 'execute',
    workflow: 'workflow',
    write: 'write',
    readonly: 'readonly',
    staging: 'staging',
    working: 'working',
    temp: 'temp'
  },

  map: function map(json) {
    var model = this.get('model');
    if (!model) {
      return;
    }

    if (json.items) {
      var result = [];
      var clustersToDelete = model.find().mapProperty('name');
      json.items.forEach(function (item) {
        result.push(this.parseIt(item, this.config));
        clustersToDelete = clustersToDelete.without(item.name);
      }, this);
      clustersToDelete.forEach(function (name) {
        this.deleteRecord(model.find().findProperty('name', name));
      }, this);
      App.store.safeLoadMany(model, result);
    }
  }
});

});

require.register("mappers/users_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.usersMapper = App.QuickDataMapper.create({
  model: App.User,
  config: {
    id: 'Users.user_name',
    user_name: 'Users.user_name',
    admin: 'Users.admin',
    operator: 'Users.operator',
    permissions: 'permissions',
    user_type: 'Users.user_type',
    cluster_user: 'Users.cluster_user'
  },
  map: function map(json) {
    var self = this;
    json.items.forEach(function (item) {
      var result = [];
      if (!App.User.find().someProperty("userName", item.Users.user_name)) {
        item.permissions = [];
        var privileges = item.privileges || [];
        if (privileges.length) {
          item.permissions = privileges.mapProperty('PrivilegeInfo.permission_name');
        }
        item.Users.admin = self.isAdmin(item.permissions);
        item.Users.operator = self.isOperator(item.permissions);
        item.Users.cluster_user = self.isClusterUser(item.permissions);
        result.push(self.parseIt(item, self.config));
        App.store.safeLoadMany(self.get('model'), result);
      }
    });
  },

  /**
   * Check if user is admin.
   * @param {Array} permissionList
   * @return {Boolean}
   **/
  isAdmin: function isAdmin(permissionList) {
    //TODO: Separate cluster operator from admin
    return permissionList.indexOf('AMBARI.ADMINISTRATOR') > -1 || permissionList.indexOf('CLUSTER.ADMINISTRATOR') > -1;
  },

  /**
   * Check if user is operator.
   * @param {Array} permissionList
   * @return {Boolean}
   **/
  isOperator: function isOperator(permissionList) {
    return permissionList.indexOf('CLUSTER.ADMINISTRATOR') > -1 && !(permissionList.indexOf('AMBARI.ADMINISTRATOR') > -1);
  },

  /**
   * Determines that user has only one permission CLUSTER.USER.
   *
   * @param {String[]} permissionList
   * @return {Boolean}
   */
  isClusterUser: function isClusterUser(permissionList) {
    return permissionList.length === 1 && permissionList[0] === 'CLUSTER.USER';
  }
});

});

require.register("mappers/widget_layout_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.widgetLayoutMapper = App.QuickDataMapper.create({
  model: App.WidgetLayout,

  config: {
    id: 'WidgetLayoutInfo.id',
    layout_name: 'WidgetLayoutInfo.layout_name',
    section_name: 'WidgetLayoutInfo.section_name',
    scope: 'WidgetLayoutInfo.scope',
    user: 'WidgetLayoutInfo.user_name',
    display_name: 'WidgetLayoutInfo.display_name',
    widgets_key: 'WidgetLayoutInfo.widgets',
    widgets_type: 'array',
    widgets: {
      item: 'WidgetInfo.id'
    }
  }
});

});

require.register("mappers/widget_mapper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.widgetMapper = App.QuickDataMapper.create({
  model: App.Widget,
  config: {
    id: 'id',
    layout_id: 'layout_id',
    widget_name: 'widget_name',
    default_order: 'default_order',
    widget_type: 'widget_type',
    service_name: 'service_name',
    section_name: 'section_name',
    time_created: 'time_created',
    author: 'author',
    properties: 'properties',
    metrics: 'metrics',
    values: 'values',
    description: 'description',
    scope: 'scope',
    tag: 'tag'
  },
  map: function map(json) {
    if (!this.get('model')) return;

    if (json.widgets) {
      var result = [];

      json.widgets.forEach(function (item, index) {
        item.WidgetInfo.section_name = json.section_name;
        item.WidgetInfo.layout_id = json.id;
        item.WidgetInfo.metrics = JSON.parse(item.WidgetInfo.metrics);
        item.WidgetInfo.properties = JSON.parse(item.WidgetInfo.properties);
        item.WidgetInfo.values = JSON.parse(item.WidgetInfo.values);
        item.WidgetInfo.default_order = index + 1;
        result.push(this.parseIt(item.WidgetInfo, this.config));
      }, this);

      App.store.safeLoadMany(this.get('model'), result);
    }
  }
});

});

require.register("messages", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

Em.I18n.translations = {

  'app.name': 'Ambari',
  'app.name.subtitle': 'Ambari - {0}',
  'app.name.subtitle.experimental': 'Ambari Experimental',
  'app.name.subtitle.installer': 'Cluster Install Wizard',
  'app.reloadPopup.link': 'Reload Page',
  'app.reloadPopup.text': 'Trying to connect to server...',
  'app.reloadPopup.noClusterName.text': 'Failed to retrieve cluster name, trying to reload...',
  'app.reloadPopup.header': 'Reload Page',
  'app.redirectIssuePopup.header': 'Login Redirect Issue',
  'app.redirectIssuePopup.body': 'For single sign-on, make sure that Knox Gateway and Ambari Server are located on the same host or subdomain.' + '<br/>Alternatively login as an Ambari local user using the local login page.<br />' + '<a rel="noopener noreferrer" href="{0}" target="_blank">{0}</a>',

  'app.loadingPlaceholder': 'Loading...',
  'app.versionMismatchAlert.title': 'Ambari Server / Web Client Version Mismatch',
  'app.versionMismatchAlert.body': 'Ambari Server and Web Client versions do not match:<br> ' + '<br>Ambari Server: <strong>{0}</strong>' + '<br>Ambari Web Client: <strong>{1}</strong><br>' + '<br>This typically happens after upgrading Ambari due to Ambari Web Client code cached in the browser.' + '<br>Perform a hard browser cache refresh, typically \'Ctrl+Shift+R\' (\'Cmd+Shift+R\' on OS X), to see if this message disappears.' + '<br>If you keep seeing this message, clear the browser cache completely and hit this URL again.' + '<br>You must resolve this error in order to continue.',
  'app.signout': 'Sign out',
  'app.settings': 'Settings',
  'app.manageAmbari': 'Manage Ambari',
  'app.aboutAmbari': 'About',
  'app.settings.selectTimezone': 'Timezone',
  'app.settings.notshowBgOperationsPopup': 'Do not show the Background Operations dialog when starting an operation',
  'app.settings.notShowBgOperations': 'Do not show this dialog again when starting a background operation',
  'app.settings.categories.general': 'General',
  'app.settings.categories.locale': 'Locale',
  'app.settings.noData': 'Unable to retrieve settings data',
  'app.settings.no.view.privileges': 'No view privileges',
  'app.settings.no.cluster.privileges': 'No cluster privileges',
  'app.settings.admin.all.privileges': 'This user is an Ambari Admin and has all privileges.',
  'app.settings.no.privileges': 'This user does not have any privileges.',
  'app.settings.clusterRole': 'Cluster Role',
  'app.settings.viewPermissions': 'View Permissions',
  'app.goToView': 'Go To View',
  'app.debugHiveQuery': 'Debug Hive Query',

  'app.aboutAmbari.getInvolved': 'Get involved!',
  'app.aboutAmbari.version': 'Version',
  'app.aboutAmbari.licensed': 'Licensed under the Apache License, Version 2.0',

  'apply': 'apply',
  'and': 'and',
  'none': 'none',
  'all': 'all',
  'minimum': 'minimum',
  'from': 'From',
  'to': 'To',
  'ok': 'OK',
  'as': 'as',
  'on': 'on',
  'in': 'in',
  'any': 'Any',
  'more': 'more',
  'yes': 'Yes',
  'no': 'No',
  'add': 'Add',
  'op': 'op',
  'ops': 'ops',
  'or': 'or',
  'then': 'then',
  'it': 'it',

  'common.access': 'Access',
  'common.learnMore': 'Learn more',
  'common.showDetails': 'Show Details',
  'common.back': 'Back',
  'common.prev': 'Prev',
  'common.next': 'Next',
  'common.host': 'Host',
  'common.hosts': 'Hosts',
  'common.services': 'Services',
  'common.group': 'Group',
  'common.groups': 'Groups',
  'common.progress': 'Progress',
  'common.status': 'Status',
  'common.action': 'Action',
  'common.refresh': 'Refresh',
  'common.remove': 'Remove',
  'common.retry': 'Retry',
  'common.skip': 'Skip',
  'common.filter': 'Filter',
  'common.rollBack': 'Rollback',
  'common.show': 'Show',
  'common.hide': 'Hide',
  'common.cancel': 'Cancel',
  'common.apply': 'Apply',
  'common.done': 'Done',
  'common.failed': 'Failed',
  'common.service': 'Service',
  'common.version': 'Version',
  'common.downgrade': 'Downgrade',
  'common.description': 'Description',
  'common.default': 'Default',
  'common.client': 'Client',
  'common.master': 'Master',
  'common.slave': 'Slave',
  'common.zookeeper': 'ZooKeeper',
  'common.hbase': 'HBase',
  'common.regionServer': 'RegionServer',
  'common.taskTracker': 'TaskTracker',
  'common.dataNode': 'DataNode',
  'common.more': 'More...',
  'common.print': 'Print',
  'common.deploy': 'Deploy',
  'common.generate.blueprint': 'Generate Blueprint',
  'common.message': 'Message',
  'common.tasks': 'Tasks',
  'common.taskLog': 'Task Log',
  'common.open': 'Open',
  'common.copy': 'Copy',
  'common.complete': 'Complete',
  'common.completed': 'Completed',
  'common.metrics': 'Metrics',
  'common.timeRange': 'Time Range',
  'common.name': 'Name',
  'common.key': 'Key',
  'common.value': 'Value',
  'common.ipAddress': 'IP Address',
  'common.rack': 'Rack',
  'common.cpu': 'CPU',
  'common.cores': 'Cores',
  'common.cores.cpu': 'Cores (CPU)',
  'common.ram': 'RAM',
  'common.disabled': 'Disabled',
  'common.enabled': 'Enabled',
  'common.enableAll': 'Enable All',
  'common.disableAll': 'Disable All',
  'common.disk': 'Disk',
  'common.diskUsage': 'Disk Usage',
  'common.last': 'Last',
  'common.loadAvg': 'Load Avg',
  'common.components': 'Components',
  'common.component': 'Component',
  'common.quickLinks': 'Quick Links',
  'common.save': 'Save',
  'common.saveAnyway': 'Save Anyway',
  'common.servers': 'Servers',
  'common.clients': 'Clients',
  'common.all.clients': 'All Clients',
  'common.user': 'User',
  'common.users': 'Users',
  'common.issues': 'Issues',
  'common.os': 'OS',
  'common.oss': 'OSs',
  'common.memory': 'Memory',
  'common.maximum': 'Maximum',
  'common.started': 'Started',
  'common.start': 'Start',
  'common.stop': 'Stop',
  'common.pause': 'Pause',
  'common.end': 'End',
  'common.decommission': 'Decommission',
  'common.recommission': 'Recommission',
  'common.failure': 'Failure',
  'common.type': 'Type',
  'common.direction': 'Direction',
  'common.close': 'Close',
  'common.warning': 'Warning',
  'common.critical': 'Critical',
  'common.information': 'Information',
  'common.all': 'All',
  'common.success': 'Success',
  'common.fail': 'Fail',
  'common.error': 'Error',
  'common.loading': 'Loading',
  'common.search': 'Search',
  'common.confirm': 'Confirm',
  'common.upgrade': 'Upgrade',
  'common.reUpgrade': 'Retry Upgrade',
  'common.reDowngrade': 'Retry Downgrade',
  'common.security': 'Security',
  'common.kerberos': 'Kerberos',
  'common.cluster': 'Cluster',
  'common.repositories': 'Repositories',
  'common.stack.versions': 'Stack Versions',
  'common.versions': 'Versions',
  'common.upgrade.history': 'Upgrade History',
  'common.serviceAccounts': 'Service Accounts',
  'common.add': 'Add',
  'common.edit': 'Edit',
  'common.delete': 'Delete',
  'common.duplicate': 'Duplicate',
  'common.disable': 'Disable',
  'common.enable': 'Enable',
  'common.empty': 'Empty',
  'common.override': 'Override',
  'common.undo': 'Undo',
  'common.details': 'Details',
  'common.stats': 'Stats',
  'common.abort': 'Abort',
  'common.aborted': 'Aborted',
  'common.misc': 'Misc',
  'common.userSettings': 'User Settings',
  'common.aboutAmbari': 'About',
  'common.notAvailable': 'Not Available',
  'common.na': 'n/a',
  'common.operations': 'Operations',
  'common.backgroundOperations': 'Background Operations',
  'common.startTime': 'Start Time',
  'common.duration': 'Duration',
  'common.reinstall': 'Re-Install',
  'common.revert': 'Revert',
  'common.errorPopup.header': 'An error has been encountered',
  'common.use': 'Use',
  'common.stacks': 'Stacks',
  'common.stack': 'Stack',
  'common.reset': 'Reset',
  'common.reset.default': 'Reset to default',
  'common.resume': 'Resume',
  'common.path': 'Path',
  'common.patch': 'Patch',
  'common.maint': 'Maint',
  'common.package': 'Package',
  'common.proceed': 'Proceed',
  'common.proceedAnyway': 'Proceed Anyway',
  'common.exitAnyway': 'Exit Anyway',
  'common.process': 'Process',
  'common.property': 'Property',
  'common.installed': 'Installed',
  'common.persist.error': 'Error in persisting web client state at ambari server. Server respond with following error message:',
  'common.update.error': 'Error in retrieving web client state from ambari server',
  'common.tags': 'Tags',
  'common.important': 'Important',
  'common.important.strong': '<strong>Important:</strong>',
  'common.allServices': 'All Services',
  'common.move': 'Move',
  'common.change': 'Change',
  'common.overrides': 'Overrides',
  'common.properties': 'properties',
  'common.conf.group': 'Configuration Group',
  'common.ignore': 'Ignore',
  'common.restart': 'Restart',
  'common.discard': 'Discard',
  'common.actions': 'Actions',
  'common.maintenance': 'Maintenance',
  'common.passive_state': 'Maintenance Mode',
  'common.select': 'Select',
  'common.selected': 'Selected',
  'common.password': 'Password',
  'common.confirm.password': 'Confirm Password',
  'common.username': 'Username',
  'common.url': 'URL',
  'common.advanced': 'Advanced',
  'common.download': 'Download',
  'common.current': 'Current',
  'common.additional': 'Additional',
  'common.time.start': 'Start Time',
  'common.time.end': 'End Time',
  'common.hostLog.popup.logDir.path': '/var/lib/ambari-agent/data/', // TODO, this hardcoded path needs to be removed.
  'common.hostLog.popup.outputLog.value': 'output-{0}.txt',
  'common.hostLog.popup.errorLog.value': 'errors-{0}.txt',
  'common.maintenance.task': ' Toggle Maintenance Mode',
  'common.installRepo.task': ' Install Packages',
  'common.used': 'used',
  'common.free': 'free',
  'common.type.string': 'string',
  'common.type.number': 'number',
  'common.author': 'Author',
  'common.notes': 'Notes',
  'common.view': 'View',
  'common.compare': 'Compare',
  'common.latest': 'Latest',
  'common.custom': 'Custom',
  'common.continue': 'Continue',
  'common.continueAnyway': 'Continue Anyway',
  'common.property.undefined': "Undefined",
  'common.summary': "Summary",
  'common.configs': "Configs",
  'common.configuration': "Configuration",
  'common.unknown': "Unknown",
  'common.install': "Install",
  'common.alertDefinition': "Alert Definition",
  'common.prerequisites': 'Prerequisites',
  'common.finalize': "Finalize",
  'common.severity': "Severity",
  'common.dismiss': "Dismiss",
  'common.stdout': "stdout",
  'common.stderr': "stderr",
  'common.structuredOut': "structured_out",
  'common.fileName': 'File Name',
  'common.file': 'File',
  'common.days': "Days",
  'common.hours': "Hours",
  'common.minutes': "Minutes",
  'common.seconds': "Seconds",
  'common.milliseconds': "Milliseconds",
  'common.configGroup': 'Config Group',
  'common.expression': 'Expression',
  'common.dataSet': 'Data Set',
  'common.label': 'Label',
  'common.preview': 'Preview',
  'common.options': 'Options',
  'common.scope': 'Scope',
  'common.clone': 'Clone',
  'common.removed': 'Removed',
  'common.testing': 'Testing',
  'common.noData': 'No Data',
  'common.export': 'Export',
  'common.csv': 'Save as CSV',
  'common.json': 'Save as JSON',
  'common.timestamp': 'Timestamp',
  'common.timezone': 'Timezone',
  'common.loading.eclipses': 'Loading...',
  'common.optional': 'Optional',
  'common.propertyType': 'Property Type',
  'common.running': 'Running',
  'common.stopped': 'Stopped',
  'common.enter': 'Enter',
  'common.timeout.warning.popup.header': 'Automatic Logout',
  'common.timeout.warning.popup.body.before': 'You will be automatically logged out in ',
  'common.timeout.warning.popup.body.after': ' seconds due to inactivity',
  'common.timeout.warning.popup.primary': 'Remain Logged In',
  'common.timeout.warning.popup.secondary': 'Log Out Now',
  'common.openNewWindow': 'Open in New Window',
  'common.fullLogPopup.clickToCopy': 'Click to Copy',
  'common.nothingToDelete': 'Nothing to delete',
  'common.exclude': 'Exclude',
  'common.include': 'Include',
  'common.exclude.short': 'Excl',
  'common.include.short': 'Incl',
  'common.filters': 'Filters',
  'common.keywords': 'Keywods',
  'common.levels': 'Levels',
  'common.extension': 'Extension',
  'common.logs': 'Logs',
  'common.warn.message': '<div class="alert alert-warning">{0}</div>',
  'common.link': 'Link',
  'common.noLink': 'No Links',
  'common.live': 'Live',
  'common.from.version': 'From Version',
  'common.to.version': 'To Version',
  'common.start.time': 'Start Time',
  'common.end.time': 'End Time',
  'common.rolling': 'Rolling',
  'common.express': 'Express',
  'common.hostOrdered': 'Host Ordered',
  'common.repository': 'Repository',
  'common.repositoryType': 'Repository Type',
  'common.rolling.downgrade': 'Rolling Downgrade',
  'common.express.downgrade': 'Express Downgrade',
  'common.minute.ago': 'less than a minute ago',
  'common.views': 'Views',
  'common.review': 'Review',
  'common.critical.error': 'Critical',
  'common.with': 'with',
  'common.propertyName': 'Property Name',

  'models.alert_instance.tiggered.verbose': "Occurred on {0} <br> Checked on {1}",
  'models.alert_definition.triggered.verbose': "Occurred on {0}",
  'models.alert_definition.triggered.checked': "Status Changed: {0}\nLast Checked: {1}",
  'models.alert_definition.check.retry': "Alert Check {0} out of {1}",

  'passiveState.turnOn': 'Turn On Maintenance Mode',
  'passiveState.turnOff': 'Turn Off Maintenance Mode',
  'passiveState.turnOnFor': 'Turn On Maintenance Mode for {0}',
  'passiveState.turnOffFor': 'Turn Off Maintenance Mode for {0}',
  'passiveState.disabled.impliedFromHighLevel': '{0} is already in Maintenance Mode because {1} is in Maintenance Mode.',
  'passiveState.disabled.impliedFromServiceAndHost': '{0} is already in Maintenance Mode because {1} and {2} are in Maintenance Mode.',

  'requestInfo.installComponents': 'Install Components',
  'requestInfo.installKerbeorosComponents': 'Install Kerberos Components',
  'requestInfo.installServices': 'Install Services',
  'requestInfo.kerberosService': 'Install Kerberos Service',
  'requestInfo.startServices': 'Start Services',
  'requestInfo.startAddedServices': 'Start Added Services',
  'requestInfo.stopAllServices': 'Stop All Services',
  'requestInfo.startAllServices': 'Start All Services',
  'requestInfo.startHostComponent': 'Start',
  'requestInfo.startHostComponent.datanode': 'Start DataNode',
  'requestInfo.startHostComponent.nodemanager': 'Start NodeManager',
  'requestInfo.startHostComponent.hbase_regionserver': 'Start RegionServer',
  'requestInfo.startHostComponents': 'Start Components',
  'requestInfo.upgradeHostComponent': 'Upgrade',
  'requestInfo.stopHostComponent': 'Stop',
  'requestInfo.installHostComponent': 'Install',
  'requestInfo.installNewHostComponent': 'Install',
  'requestInfo.refreshComponentConfigs': 'Refresh {0} configs',
  'requestInfo.stop': 'Stop {0}',
  'requestInfo.start': 'Start {0}',
  'requestInfo.unspecified': 'Request name not specified',
  'requestInfo.kerberizeCluster': 'Kerberize Cluster',

  'hostPopup.noServicesToShow': 'No services to show',
  'hostPopup.noHostsToShow': 'No hosts to show',
  'hostPopup.noTasksToShow': 'No tasks to show',
  'hostPopup.status.category.all': 'All ({0})',
  'hostPopup.status.category.pending': 'Pending ({0})',
  'hostPopup.status.category.inProgress': 'In Progress ({0})',
  'hostPopup.status.category.failed': 'Failed ({0})',
  'hostPopup.status.category.success': 'Success ({0})',
  'hostPopup.status.category.aborted': 'Aborted ({0})',
  'hostPopup.status.category.timedout': 'Timedout ({0})',
  'hostPopup.header.postFix': ' Background Operation{0} Running',
  'hostPopup.serviceInfo.showMore': 'Show more...',
  'hostPopup.bgop.abortRequest.title': 'Abort operation',
  'hostPopup.bgop.abortRequest.confirmation.body': 'Are you sure you want to abort \'{0}\' operation?',
  'hostPopup.bgop.abortRequest.reason': 'Aborted by user',
  'hostPopup.bgop.abortRequest.modal.header': 'Abort request sent',
  'hostPopup.bgop.abortRequest.modal.body': 'The abort request for <strong>{0}</strong> has been sent to the server. Note: some tasks that are already running may have time to complete or fail before the abort request is applied.',
  'hostPopup.bgop.sourceRequestSchedule.running': 'Future operations of this batch request can be aborted',
  'hostPopup.bgop.sourceRequestSchedule.aborted': 'Future operations of this batch request have been aborted',
  'hostPopup.bgop.abort.rollingRestart': 'Abort Rolling Restart',
  'hostPopup.warning.alertsTimeOut': 'Maintenance Mode has been turned {0}. It may take a few minutes for the alerts to be {1}.',
  'hostPopup.reccomendation.beforeDecommission': '{0} Maintenance Mode is pre required for decommissioning.',
  'hostPopup.setRackId.success': 'Updating rack id to \"{0}\". It may take a few moments for it to get refreshed.',
  'hostPopup.setRackId.error': 'Updating the rack id failed.',
  'hostPopup.setRackId.invalid': 'Should start with a forward slash it may include alphanumeric chars, dots, dashes and forward slashes. Should be less than 255 symbols.',
  'hostPopup.RackId': 'Rack',
  'hostPopup.recommendation.beforeDecommission': '{0} Maintenance Mode is pre required for decommissioning.',
  'hostPopup.default.userName': 'none',

  'question.sure': 'Are you sure?',
  'question.sure.restart': 'Are you sure you want to restart {0}?',
  'question.sure.start': 'Are you sure you want to start {0}?',
  'question.sure.stop': 'Are you sure you want to stop {0}?',
  'question.sure.move': 'Are you sure you want to move {0}?',
  'question.sure.refresh': 'Are you sure you want to restart {0} on {1}?',
  'question.sure.maintenance': 'Are you sure you want to turn {0} maintenance mode for {1}?',
  'question.sure.upgrade': 'Are you sure you want to upgrade {0}?',
  'question.sure.decommission': 'Are you sure you want to decommission {0}?',
  'question.sure.recommission': 'Are you sure you want to recommission {0}?',
  'question.sure.startAll': 'Are you sure you want to start all the components?',
  'question.sure.stopAll': 'Are you sure you want to stop all the components?',
  'question.sure.restartAll': 'Are you sure you want to restart all the components?',
  'question.sure.regenerateKeytab.service': 'Are you sure you want to regenerate keytab file operations for a {0} service?',
  'question.sure.regenerateKeytab.host': 'Are you sure you want to regenerate keytab file operations for a {0} host?',

  'popup.highlight': 'click to highlight',
  'popup.confirmation.commonHeader': 'Confirmation',
  'popup.confirmation.refreshYarnQueues.body': 'It is strongly recommended to <strong>Refresh Yarn Queues</strong> after making a change to the capacity scheduler configuration. Would you like to proceed?',
  'popup.warning.refreshYarnQueues.body': '<div class="alert alert-warning"><strong>Warning: </strong>The queue configuration change may have affected the queue used by Hive LLAP. A Hive LLAP reconfiguration or restart may be required.</div>',
  'popup.confirmation.refreshYarnQueues.buttonText': 'Refresh Yarn Queues',
  'popup.confirmation.hsiRestart.body': 'It is strongly recommended to <strong>Restart HiveServer2 Interactive</strong> after making a change to the Hive LLAP sizing configuration. Would you like to proceed?',
  'popup.confirmation.hsiRestart.buttonText': 'Restart HiveServer2 Interactive',
  'popup.prompt.commonHeader': 'Prompt',
  'popup.confirmationFeedBack.sending': 'Sending...',
  'popup.confirmationFeedBack.query.fail': 'Request failed',

  'popup.clusterCheck.failedOn': 'Failed on: ',
  'popup.clusterCheck.reason': 'Reason: ',
  'popup.clusterCheck.Upgrade.header': 'Upgrade to {0}',
  'popup.clusterCheck.Upgrade.fail.title': 'Requirements',
  'popup.clusterCheck.Upgrade.fail.alert': 'You <strong>must</strong> meet these requirements before you can proceed.',
  'popup.clusterCheck.Upgrade.bypassed-failures.title': 'Errors that can be bypassed',
  'popup.clusterCheck.Upgrade.bypassed-failures.alert': 'Errors are allowed to be bypassed since config stack.upgrade.bypass.prechecks is set to true. It is strongly encouraged to look into these failures.',
  'popup.clusterCheck.Upgrade.warning.title': 'Warnings',
  'popup.clusterCheck.Upgrade.warning.alert': 'Correcting the warnings is not required but is <strong>recommended</strong>.',
  'popup.clusterCheck.Upgrade.configsMerge.title': 'Configuration Changes',
  'popup.clusterCheck.Upgrade.configsMerge.alert': 'During upgrade, the following configuration changes will be applied.',
  'popup.clusterCheck.Upgrade.configsMerge.configType': 'Config Type',
  'popup.clusterCheck.Upgrade.configsMerge.propertyName': 'Property Name',
  'popup.clusterCheck.Upgrade.configsMerge.currentValue': 'Current Value',
  'popup.clusterCheck.Upgrade.configsMerge.recommendedValue': 'Recommended Value',
  'popup.clusterCheck.Upgrade.configsMerge.resultingValue': 'Resulting Value',
  'popup.clusterCheck.Upgrade.configsMerge.deprecated': 'Property is deprecated',
  'popup.clusterCheck.Upgrade.configsMerge.willBeRemoved': 'Will be removed',
  'popup.clusterCheck.Upgrade.configsRecommend.alert': 'We\'ve detected the need to update the following properties,' + ' but cannot do so automatically since they have been customized.  Please review these properties manually,' + ' and update the properties manually where necessary.',
  'popup.clusterCheck.Upgrade.configsRecommend.title': 'Recommended Configuration Changes: Manual Review',
  'popup.clusterCheck.Security.header': 'Enable Security',
  'popup.clusterCheck.Security.title': 'Security Requirements Not Met',
  'popup.clusterCheck.Security.alert': 'You must meet the following requirements before you can enable security.',

  'popup.invalid.KDC.header': 'Admin session expiration error',
  'popup.invalid.KDC.msg': ' Please enter admin principal and password.',
  'popup.invalid.KDC.admin.principal': 'Admin principal',
  'popup.invalid.KDC.admin.password': 'Admin password',

  'popup.dependent.configs.header': 'Dependent Configurations',
  'popup.dependent.configs.title.recommendation': 'Based on your configuration changes, Ambari is recommending the following dependent configuration changes.',
  'popup.dependent.configs.title.values': 'Ambari will update all checked configuration changes to the <b>Recommended Value</b>. Uncheck any configuration to retain the <b>Current Value</b>.',
  'popup.dependent.configs.title.required': 'The following configuration changes are required and will be applied automatically.',
  'popup.dependent.configs.table.recommended': 'Recommended Changes',
  'popup.dependent.configs.table.required': 'Required Changes',
  'popup.dependent.configs.table.saveProperty': 'Save property',
  'popup.dependent.configs.table.initValue': 'Initial value',
  'popup.dependent.configs.table.currentValue': 'Current Value',
  'popup.dependent.configs.table.originalValue': 'Original Value',
  'popup.dependent.configs.table.recommendedValue': 'Recommended Value',
  'popup.dependent.configs.table.newValue': 'New Value',
  'popup.dependent.configs.table.undefined': 'Property undefined',
  'popup.dependent.configs.table.removed': 'Property removed',

  'popup.dependent.configs.select.config.group.header': 'Select Config Group',
  'popup.dependent.configs.select.config.group.no.groups': 'No config groups for current service',
  'popup.dependent.configs.select.config.group': 'Please select to which config group would you like to save dependent properties',

  'popup.dependent.configs.dependencies.config.singular': 'There is {0} configuration change ',
  'popup.dependent.configs.dependencies.config.plural': 'There are {0} configuration changes ',
  'popup.dependent.configs.dependencies.service.singular': 'in {0} service',
  'popup.dependent.configs.dependencies.service.plural': 'in {0} services',

  'popup.dependent.configs.dependencies.for.groups': 'You are changing not default group, please select config group to which you want to save dependent configs from other services',

  'popup.jdkValidation.header': 'Unsupported JDK',
  'popup.jdkValidation.body': 'The {0} Stack requires JDK {1} but Ambari is configured for JDK {2}. This could result in error or problems with running your cluster.',
  'popup.logTail.header': 'File Name',
  'popup.logTail.openInLogSearch': 'Open In Log Search',

  'login.header': 'Sign in',
  'login.message.title': 'Login Message',
  'login.username': 'Username',
  'login.loginButton': 'Sign in',
  'login.error.bad.credentials': 'Unable to sign in. Invalid username/password combination.',
  'login.error.disabled': 'Unable to sign in. Invalid username/password combination.',
  'login.error.bad.connection': 'Unable to connect to Ambari Server. Confirm Ambari Server is running and you can reach Ambari Server from this machine.',

  'titlebar.alerts.noAlerts': 'No Alerts',

  'graphs.noData.title': 'No Data',
  'graphs.noData.message': 'No Data Available',
  'graphs.noData.tooltip.title': 'No Data Available. The Ambari Metrics service may be not installed or inaccessible',
  'graphs.noDataAtTime.message': 'No available data for the time period.',
  'graphs.error.title': 'Error',
  'graphs.error.message': 'There was a problem getting data for the chart ({0}: {1})',
  'graphs.timeRange.hour': 'Last 1 hour',
  'graphs.timeRange.twoHours': 'Last 2 hours',
  'graphs.timeRange.fourHours': 'Last 4 hours',
  'graphs.timeRange.twelveHours': 'Last 12 hours',
  'graphs.timeRange.day': 'Last 24 hours',
  'graphs.timeRange.week': 'Last 1 week',
  'graphs.timeRange.month': 'Last 1 month',
  'graphs.timeRange.year': 'Last 1 year',
  'graphs.tooltip.title': 'Click to zoom',

  'users.userName.validationFail': 'Only lowercase letters and numbers are recommended; must start with a letter',
  'host.spacesValidation': 'Cannot contain whitespace',

  'services.hdfs.rebalance.title': 'HDFS Rebalance',
  'services.ganglia.description': 'Ganglia Metrics Collection system',
  'services.hdfs.description': 'Apache Hadoop Distributed File System',
  'services.glusterfs.description': 'Apache Hadoop Compatible File System (must be installed manually)',
  'services.sqoop.description': 'Tool for transferring bulk data between Apache Hadoop and structured data stores such as relational databases',
  'services.pig.description': 'Scripting platform for analyzing large datasets',
  'services.hive.description': 'Data warehouse system for ad-hoc queries & analysis of large datasets and table & storage management service',
  'services.oozie.description': 'System for workflow coordination and execution of Apache Hadoop jobs',
  'services.zookeeper.description': 'Centralized service which provides highly reliable distributed coordination',
  'services.hbase.description': 'Non-relational distributed database and centralized service for configuration management & synchronization',
  'services.hive.databaseComponent': 'Database Server',
  'services.mapreduce2.description': 'Apache Hadoop NextGen MapReduce (client libraries)',
  'services.yarn.description': 'Apache Hadoop NextGen MapReduce (YARN)',
  'services.tez.description': 'Tez is the next generation Hadoop Query Processing framework written on top of YARN',
  'services.falcon.description': 'Falcon mirroring engine',
  'services.storm.description': 'Apache Hadoop Stream processing framework',
  'services.storm.configs.range-plugin-enable.dialog.title': 'Enable Ranger for STORM',
  'services.storm.configs.range-plugin-enable.dialog.message': 'Enabling Ranger plugin for STORM is effective only on a secured cluster.',

  'services.alerts.head': 'You have {0} critical alert notification(s).',
  'services.alerts.OK.timePrefixShort': 'OK',
  'services.alerts.WARN.timePrefixShort': 'WARN',
  'services.alerts.CRIT.timePrefixShort': 'CRIT',
  'services.alerts.MAINT.timePrefixShort': 'MAINT',
  'services.alerts.UNKNOWN.timePrefixShort': 'UNKNOWN',
  'services.alerts.OK.timePrefix': 'OK for {0}',
  'services.alerts.WARN.timePrefix': 'WARN for {0}',
  'services.alerts.CRIT.timePrefix': 'CRIT for {0}',
  'services.alerts.MAINT.timePrefix': 'MAINT for {0}',
  'services.alerts.UNKNOWN.timePrefix': 'UNKNOWN for {0}',
  'services.alerts.lastCheck': 'Last Checked {0}',
  'services.alerts.brLastCheck': '<br>Last Checked {0}',
  'services.alerts.occurredOn': 'Occurred on {0}, {1}',
  'services.alerts.goToService': 'Go to Service',
  'services.alerts.goTo': 'Go to {0}',

  'services.summary.selectHostForComponent': 'Select the host to add {0} component',
  'services.summary.allHostsAlreadyRunComponent': 'All hosts already running {0} component',

  'topnav.help.href': 'https://cwiki.apache.org/confluence/display/AMBARI/Ambari',

  'installer.header': 'Cluster Install Wizard',
  'installer.navigation.warning.header': 'Navigation Warning',

  'installer.noHostsAssigned': 'No host assigned',
  'installer.slaveComponentHosts.selectHosts': 'select hosts for this group',
  'installer.slaveComponentHostsPopup.header': 'Select which hosts should belong to which {0} group',

  'installer.controls.slaveComponentGroups': ' Groups',
  'installer.controls.serviceConfigPopover.title': '{0}<br><small>{1}</small>',
  'installer.controls.checkConnection.popover': 'This action will check accessibility of {0} host and port from Ambari Server host',
  'installer.controls.serviceConfigMultipleHosts.other': '1 other',
  'installer.controls.serviceConfigMultipleHosts.others': '{0} others',
  'installer.controls.serviceConfigMasterHosts.header': '{0} Hosts',
  'installer.controls.slaveComponentChangeGroupName.error': 'group with this name already exist',

  'installer.step0.header': 'Get Started',
  'installer.step0.body.header': 'Get Started',
  'installer.step0.body': 'This wizard will walk you through the cluster installation process.  First, start by naming your new cluster.',
  'installer.step0.clusterName': 'Name your cluster',
  'installer.step0.clusterName.tooltip.title': 'Cluster Name',
  'installer.step0.clusterName.tooltip.content': 'Enter a unique cluster name.',
  'installer.step0.clusterName.error.required': 'Cluster Name is required',
  'installer.step0.clusterName.error.tooLong': 'Cluster Name is too long',
  'installer.step0.clusterName.error.whitespace': 'Cluster Name cannot contain whitespace',
  'installer.step0.clusterName.error.specialChar': 'Cluster Name cannot contain special characters',

  'installer.step1.header': 'Select Version',
  'installer.step1.body': 'Select the software version and method of delivery for your cluster.',
  'installer.step1.repo.body': 'Using a Public Repository requires Internet connectivity. Using a Local Repository requires you have configured the software in a repository available in your network.',
  'installer.step1.addVersion.title': 'Add Version',
  'installer.step1.changeVersion.defaultVersion': 'Default Version Definition',
  'installer.step1.selectUseRepoOptions.public': 'Use Public Repository',
  'installer.step1.selectUseRepoOptions.public.networkLost.button': 'Not Available',
  'installer.step1.selectUseRepoOptions.public.networkLost': 'Why is this not selected?',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.title': 'Public Repository Option Not Selected',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.msg': 'Ambari is unable to confirm access to the Public Repositories, which means you might not have Internet access and will not be able to use the Public Repository option for installing the software. Your choices:',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.msg1': 'Configure your hosts for access to the Internet.',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.msg2': 'If you are using an Internet Proxy, refer to the Ambari Documentation on how to configure Ambari to use the Internet Proxy.',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.msg3': 'Use the Local Repositoy option.',
  'installer.step1.selectUseRepoOptions.public.networkLost.popup.msg4': 'If you do not have internet access and select this option, you will fail the Repository Base URL validation.',
  'installer.step1.selectUseRepoOptions.public.select': 'Select Version',
  'installer.step1.selectUseRepoOptions.local': 'Use Local Repository',
  'installer.step1.usePublicRepo.ReposList.empty': 'No other versions',
  'installer.step1.usePublicRepo.viewRepos': 'View Repositories',
  'installer.step1.useLocalRepo.uploadFile': 'Upload Version Definition File',
  'installer.step1.useLocalRepo.uploadFile.error.title': 'Upload Version Definition File Error',
  'installer.step1.useLocalRepo.getSurpottedOs.error.title': 'Cannot get supported OS types',
  'installer.step1.useLocalRepo.addRepo.button': 'Add Repository',
  'installer.step1.addVersion': 'Add Version',
  'installer.step1.addVersion.stackChanged.popup.body': 'Stack is changed from {0} to {1}.',
  'installer.step1.useLocalRepo.enterUrl': 'Version Definition File URL',
  'installer.step1.useLocalRepo.enterUrl.placeholder': 'Enter URL to Version Definition File',
  'installer.step1.useLocalRepo.readButton': 'Read Version Info',
  'installer.step1.useLocalRepo.infoForm.details.title': 'Details',
  'installer.step1.useLocalRepo.infoForm.details.stackName': 'Stack Name',
  'installer.step1.useLocalRepo.infoForm.details.displayName': 'Display Name',
  'installer.step1.useLocalRepo.infoForm.details.version': 'Version',
  'installer.step1.useLocalRepo.infoForm.details.actualVersion': 'Actual Version',
  'installer.step1.useLocalRepo.infoForm.details.releaseNotes': 'Release Notes',
  'installer.step1.useLocalRepo.infoForm.content.title': 'Contents',
  'installer.step1.useLocalRepo.infoForm.content.empty': 'No contents to display',
  'installer.step1.useLocalRepo.infoForm.alert.baseUrl': 'Provide Base URLs for the Operating Systems you are configuring.',
  'installer.step1.useLocalRepo.infoForm.alert.warning': 'Some of the repositories failed validation. Make changes to the base url or skip validation if you are sure that urls are correct',
  'installer.step1.advancedRepo.title': 'Advanced Repository Options',
  'installer.step1.advancedRepo.message': 'Customize the repository Base URLs for downloading the Stack software packages. If your hosts do not have access to the internet, you will have to create a local mirror of the Stack repository that is accessible by all hosts and use those Base URLs here.',
  'installer.step1.advancedRepo.importantMassage': '<b>Important:</b> When using local mirror repositories, you only need to provide Base URLs for the Operating System you are installing for your Stack. Uncheck all other repositories.',
  'installer.step1.advancedRepo.localRepo.error.modifyUrl': 'Local repository URL must be modified',
  'installer.step1.advancedRepo.localRepo.error.noUrl': 'Base URL required for a local repository',
  'installer.step1.advancedRepo.localRepo.column.baseUrl': 'Base URL',
  'installer.step1.advancedRepo.localRepo.label.os': 'Operating System',
  'installer.step1.advancedRepo.localRepo.label.baseUrl': 'Repository Base URL',
  'installer.step1.advancedRepo.localRepo.label.stack': 'Stack',
  'installer.step1.advancedRepo.localRepo.placeholder': 'Enter Base URL or remove this OS',
  'installer.step1.advancedRepo.skipValidation.tooltip': 'Warning: This is for advanced users only. Use this option if you want to skip validation for Repository Base URLs.',
  'installer.step1.advancedRepo.useRedhatSatellite.tooltip': 'Disable distributed repositories and use RedHat Satellite/Spacewalk channels instead',
  'installer.step1.advancedRepo.useRedhatSatellite.disabled.tooltip': 'Use of RedHat Satellite/Spacewalk is not available when is using Public Repositories',
  'installer.step1.advancedRepo.skipValidation.message': 'Skip Repository Base URL validation (Advanced)',
  'installer.step1.advancedRepo.useRedhatSatellite.message': 'Use RedHat Satellite/Spacewalk',
  'installer.step1.advancedRepo.useRedhatSatellite.warning': "In order for Ambari to install packages from the right" + " repositories, it is recommended that you edit the names of the repo's for each operating system so they match " + "the channel names in your RedHat Satellite/Spacewalk instance.",
  'installer.step1.addOs.disabled.tooltip': 'All Operating Systems have been added',
  'installer.step1.attentionNeeded': '<b>Attention:</b> Repository Base URLs of at least one OS are REQUIRED before you can proceed. Please make sure they are in correct format with its protocol.',
  'installer.step1.invalidURLAttention': '<b>Attention:</b> Please make sure all repository URLs are valid before proceeding.',
  'installer.step1.checkAtLeastOneAttention': '<b>Attention:</b> Please check at least one repository.',
  'installer.step1.retryRepoUrls': 'Click <b>here</b> to retry.',

  'installer.step2.header': 'Install Options',
  'installer.step2.body': 'Enter the list of hosts to be included in the cluster and provide your SSH key.',
  'installer.step2.targetHosts': 'Target Hosts',
  'installer.step2.targetHosts.info': 'Enter a list of hosts using the Fully Qualified Domain Name (FQDN), one per line',
  'installer.step2.hostPattern.tooltip.title': 'Pattern Expressions',
  'installer.step2.hostPattern.tooltip.content': 'You can use pattern expressions to specify a number of target hosts. For example, to specify host01.domain thru host10.domain, enter host[01-10].domain in the target hosts textarea.',
  'installer.step2.hostName.error.required': 'You must specify at least one host name',
  'installer.step2.hostName.error.already_installed': 'All these hosts are already part of the cluster',
  'installer.step2.hostName.error.notRequired': 'Host Names will be ignored if not using SSH to automatically configure hosts',
  'installer.step2.hostName.error.invalid': 'Invalid Host Name(s)',
  'installer.step2.hostName.pattern.header': 'Host name pattern expressions',
  'installer.step2.sshKey': 'Host Registration Information',
  'installer.step2.sshKey.error.required': 'SSH Private Key is required',
  'installer.step2.passphrase.error.match': 'Passphrases do not match',
  'installer.step2.manualInstall.label': 'Do not use SSH to automatically configure hosts ',
  'installer.step2.manualInstall.info': 'By not using SSH to connect to the target hosts, you must manually install and' + ' start the Ambari Agent on each host in order for the wizard to perform the necessary configurations and' + ' software installs.',
  'installer.step2.advancedOption': 'Advanced Options',
  'installer.step2.repoConf': 'Software Repository Configuration File Path',
  'installer.step2.advancedOptions.header': 'Advanced Options',
  'installer.step2.localRepo.label_use': 'Use a',
  'installer.step2.localRepo.label_instead': 'instead of downloading software packages from the Internet',
  'installer.step2.localRepo.error.required': 'Local repository file path is required',
  'installer.step2.localRepo.tooltip.title': 'Local Software Repository',
  'installer.step2.localRepo.tooltip.content': 'The cluster install requires access to the Internet to fetch software ' + 'from a remote repository. In some cases, adequate bandwidth is not available and you want to prevent downloading ' + 'packages from the remote repository over and over again. Other times, Internet access is not available from the ' + 'hosts in your cluster. In these situations, you must set up a version of the repository that your machines can ' + 'access locally and this is called a <b>Local Software Repository</b>',
  'installer.step2.javaHome.label': 'Path to 64-bit JDK',
  'installer.step2.javaHome.tooltip.title': 'JAVA_HOME',
  'installer.step2.javaHome.tooltip.content': 'Path to 64-bit JAVA_HOME. /usr/jdk/jdk1.6.0_31 is the default used by Ambari. You can override this to a specific path that contains the JDK. <br/> Note: the path must be valid on <b>ALL</b> hosts in your cluster.',
  'installer.step2.javaHome.tooltip.placeholder': '/usr/jdk/jdk1.6.0_31',
  'installer.step2.automaticInstall.tooltip.title': 'automatic registration',
  'installer.step2.automaticInstall.tooltip.content': 'Ambari will automatically install and register the Ambari Agent on each host prior to the cluster installation.',
  'installer.step2.useSsh.provide': 'Provide your',
  'installer.step2.useSsh.provide_id_rsa': ' to automatically register hosts',
  'installer.step2.useSsh.tooltip.title': 'SSH Private Key',
  'installer.step2.useSsh.tooltip.content': 'The <b>SSH Private Key File</b> is used to connect to the target hosts in your cluster to install the Ambari Agent.',
  'installer.step2.install.perform': 'Perform',
  'installer.step2.install.perform_on_hosts': 'on hosts',
  'installer.step2.install.without_ssh': ' and do not use SSH',
  'installer.step2.manualInstall.tooltip.title': 'manual registration',
  'installer.step2.manualInstall.tooltip.content': 'Manually registering the Ambari Agent on each host eliminates the need for SSH and should be performed prior to continuing cluster installation.',
  'installer.step2.manualInstall.tooltip.content_no_ssh': 'Manually registering the Ambari Agent on each host should be performed prior to continuing cluster installation.',
  'installer.step2.manualInstall.popup.header': 'Before You Proceed',
  'installer.step2.manualInstall.popup.body': 'You must install Ambari Agents on each host you want to manage before you proceed.',
  'installer.step2.warning.popup.body': '<p>The following hostnames are not valid FQDNs:</p><p> {0} </p><p>This may cause problems during installation. Do you want to continue?</p>',
  'installer.step2.orUse': 'Or use',
  'installer.step2.registerAndConfirm': 'Register and Confirm',
  'installer.step2.evaluateStep.installedHosts': 'These hosts are already installed on the cluster and will be ignored:',
  'installer.step2.evaluateStep.continueConfirm': 'Do you want to continue?',
  'installer.step2.evaluateStep.hostRegInProgress': 'Host Registration is currently in progress.  Please try again later.',
  'installer.step2.sshUser': 'SSH User Account',
  'installer.step2.sshUser.toolTip': 'The user account used to install the Ambari Agent on the target host(s) via SSH. This user must be set up with passwordless SSH and sudo access on all the target host(s)',
  'installer.step2.sshUser.placeholder': 'Enter user name',
  'installer.step2.sshUser.required': 'User name is required',
  'installer.step2.sshPort': 'SSH Port Number',
  'installer.step2.sshPort.toolTip': 'SSH Port Number',
  'installer.step2.sshPort.required': 'SSH Port Number is required.',
  'installer.step2.agentUser': 'Ambari Agent User Account',
  'installer.step2.agentUser.toolTip': 'The user account used to run the Ambari Agent daemon on the target host(s). This user must be set up with passwordless sudo access on all the target host(s)',
  'installer.step2.bootStrap.error': 'Errors were encountered while setting up Ambari Agents on the hosts.',
  'installer.step2.bootStrap.inProgress': 'Please wait while Ambari Agents are being set up on the hosts. This can take several minutes depending on the number of hosts.',
  'installer.step2.bootStrap.header': 'Setting Up Ambari Agents',
  'installer.step2.skipHostChecks.label': 'Skip host checks',
  'installer.step2.skipHostChecks.popup.header': 'Warning',
  'installer.step2.skipHostChecks.popup.body': 'By skipping host checks, Ambari will not check and warn if any issues with the host are identified and the host will be added to the cluster as is.',

  'installer.step3.header': 'Confirm Hosts',
  'installer.step3.body': 'Registering your hosts.<br>' + 'Please confirm the host list and remove any hosts that you do not want to include in the cluster.',
  'installer.step3.hostLog.popup.header': 'Registration log for {0}',
  'installer.step3.hosts.remove.popup.header': 'Remove Hosts',
  'installer.step3.hosts.remove.popup.body': 'Are you sure you want to remove the selected host(s)?',
  'installer.step3.hostInformation.popup.header': 'Error in retrieving host Information',
  'installer.step3.hostInformation.popup.body': 'All bootstrapped hosts registered but unable to retrieve cpu and memory related information',
  'installer.step3.hostOsTypeCheck.popup.body': 'Host registered successfully, but the host operating system type NOT match the selected group in "Select Stack" step: Advanced Repository Option. You can go back to "Select Stack" step OR remove this host.' + 'The host type is {0}, but you selected group {1} in step 1.',
  'installer.step3.hostWarningsPopup.report': 'Show Report',
  'installer.step3.hostWarningsPopup.report.header': '<p style="font-family: monospace">######################################<br># Host Checks Report<br>#<br># Generated: ',
  'installer.step3.hostWarningsPopup.report.hosts': '<br>######################################<br><br>######################################<br># Hosts<br>#<br># A space delimited list of hosts which have issues.<br># Provided so that administrators can easily copy hostnames into scripts, email etc.<br>######################################<br>HOSTS<br>',
  'installer.step3.hostWarningsPopup.report.jdk': '<br><br>######################################<br># JDK Check <br>#<br># A newline delimited list of JDK issues.<br>######################################<br>JDK ISSUES<br>',
  'installer.step3.hostWarningsPopup.report.disk': '<br><br>######################################<br># Disk <br>#<br># A newline delimited list of disk issues.<br>######################################<br>DISK ISSUES<br>',
  'installer.step3.hostWarningsPopup.report.repositories': '<br><br>######################################<br># Repositories <br>#<br># A newline delimited list of repositories issues.<br>######################################<br>REPOSITORIES ISSUES<br>',
  'installer.step3.hostWarningsPopup.report.hostNameResolution': '<br><br>######################################<br># Hostname Resolution<br>#<br># A newline delimited list of hostname resolution issues.<br>######################################<br>HOSTNAME RESOLUTION ISSUES<br>',
  'installer.step3.hostWarningsPopup.report.thp': '<br><br>######################################<br># Transparent Huge Pages(THP) <br>#<br># A space delimited list of hostnames on which Transparent Huge Pages are enabled.<br>######################################<br>THP ISSUES HOSTS<br>',
  'installer.step3.hostWarningsPopup.report.firewall': '<br><br>######################################<br># Firewall<br>#<br># A newline delimited list of firewall issues.<br>######################################<br>FIREWALL<br>',
  'installer.step3.hostWarningsPopup.report.fileFolders': '<br><br>######################################<br># Files and Folders<br>#<br># A space delimited list of files and folders which should not exist.<br># Provided so that administrators can easily copy paths into scripts, email etc.<br># Example: rm -r /etc/hadoop /etc/hbase<br>######################################<br>FILES AND FOLDERS<br>',
  'installer.step3.hostWarningsPopup.report.reverseLookup': '<br><br>######################################<br># Reverse Lookup<br># <br># The hostname was not found in the reverse DNS lookup. This may result in incorrect behavior. <br># Please check the DNS setup and fix the issue.<br>######################################<br>REVERSE LOOKUP<br>',
  'installer.step3.hostWarningsPopup.report.process': '<br><br>######################################<br># Processes<br>#<br># A comma separated list of process tuples which should not be running.<br># Provided so that administrators can easily copy paths into scripts, email etc.<br>######################################<br>PROCESSES<br>',
  'installer.step3.hostWarningsPopup.report.package': '<br><br>######################################<br># Packages<br>#<br># A space delimited list of software packages which should be uninstalled.<br># Provided so that administrators can easily copy paths into scripts, email etc.<br># Example: yum remove hadoop-hdfs yarn<br>######################################<br>PACKAGES<br>',
  'installer.step3.hostWarningsPopup.report.service': '<br><br>######################################<br># Services<br>#<br># A space delimited list of services which should be up and running.<br># Provided so that administrators can easily copy paths into scripts, email etc.<br># Example: services start ntpd httpd<br>######################################<br>SERVICES<br>',
  'installer.step3.hostWarningsPopup.report.user': '<br><br>######################################<br># Users<br>#<br># A space delimited list of users who should not exist.<br># Provided so that administrators can easily copy paths into scripts, email etc.<br># Example: userdel hdfs<br>######################################<br>USERS<br>',
  'installer.step3.hostWarningsPopup.report.folder': '\\ /folder',
  'installer.step3.hostWarningsPopup.checks': 'Host Checks found',
  'installer.step3.hostWarningsPopup.notice.beginning': 'After manually resolving the issues, click <b>Rerun Checks</b>.',
  'installer.step3.hostWarningsPopup.summary': '{0} on {1}',
  'installer.step3.hostWarningsPopup.jdk': 'JDK Issues',
  'installer.step3.hostWarningsPopup.jdk.name': 'JDK not found at <i>{0}</i>',
  'installer.step3.hostWarningsPopup.jdk.context': '{0}',
  'installer.step3.hostWarningsPopup.jdk.message': 'The following registered hosts have issues related to JDK',
  'installer.step3.hostWarningsPopup.repositories': 'Repository Issues',
  'installer.step3.hostWarningsPopup.repositories.name': 'Repository for OS not available',
  'installer.step3.hostWarningsPopup.repositories.context': 'Host ({0}) is {1} OS type, but the repositories chosen in "Select Stack" step was {2}. Selected repositories do not support this host OS type.',
  'installer.step3.hostWarningsPopup.repositories.message': 'The following registered hosts have different Operating System types from the available Repositories chosen in "Select Stack" step. You can go back to "Select Stack" step to select another OS repository <b>or</b> remove the host.',
  'installer.step3.hostWarningsPopup.disk': 'Disk Issues',
  'installer.step3.hostWarningsPopup.disk.name': 'Not enough disk space ',
  'installer.step3.hostWarningsPopup.disk.context1': 'Not enough disk space on host ({0}).',
  'installer.step3.hostWarningsPopup.disk.context2': 'A minimum of {0} is required for "{1}" mount.',
  'installer.step3.hostWarningsPopup.disk.message': 'The following registered hosts have issues related to disk space',
  'installer.step3.hostWarningsPopup.thp': 'Transparent Huge Pages Issues',
  'installer.step3.hostWarningsPopup.thp.name': 'Transparent Huge Pages',
  'installer.step3.hostWarningsPopup.thp.context': '{0}',
  'installer.step3.hostWarningsPopup.thp.message': 'The following hosts have Transparent Huge Pages (THP) enabled. THP should be disabled to avoid potential Hadoop performance issues.',
  'installer.step3.hostWarningsPopup.firewall': 'Firewall Issues',
  'installer.step3.hostWarningsPopup.firewall.message': 'Firewall is running on the following hosts. Please configure the firewall to allow communications on the ports documented in the <i>Configuring Ports</i> section of  the <a target=\"_blank\" href=\"http://ambari.apache.org/current/installing-hadoop-using-ambari/content/\">Ambari documentation</a>',
  'installer.step3.hostWarningsPopup.process': 'Process Issues',
  'installer.step3.hostWarningsPopup.processes.message': 'The following processes should not be running',
  'installer.step3.hostWarningsPopup.fileAndFolder': 'File and Folder Issues',
  'installer.step3.hostWarningsPopup.fileFolders.message': 'The following files and folders should not exist',
  'installer.step3.hostWarningsPopup.package': 'Package Issues',
  'installer.step3.hostWarningsPopup.packages.message': 'The following packages should be uninstalled',
  'installer.step3.hostWarningsPopup.user': 'User Issues',
  'installer.step3.hostWarningsPopup.users.message': 'The following users should be removed',
  'installer.step3.hostWarningsPopup.service': 'Service Issues',
  'installer.step3.hostWarningsPopup.services.message': 'The following services should be up',
  'installer.step3.hostWarningsPopup.misc': 'Misc Issues',
  'installer.step3.hostWarningsPopup.misc.message': 'The following umasks should be changed',
  'installer.step3.hostWarningsPopup.misc.umask': 'Umask',
  'installer.step3.hostWarningsPopup.alternatives': 'Alternatives Issues',
  'installer.step3.hostWarningsPopup.alternatives.umask': 'Alternatives',
  'installer.step3.hostWarningsPopup.alternatives.message': 'The following alternatives should be removed',
  'installer.step3.hostWarningsPopup.alternatives.noIssues': 'There was no alternative issue following alternatives should be removed',
  'installer.step3.hostWarningsPopup.alternatives.empty': 'alternative issues',
  'installer.step3.hostWarningsPopup.issue': 'issue',
  'installer.step3.hostWarningsPopup.issues': 'issues',
  'installer.step3.hostWarningsPopup.emptyMessage': 'There were no {0}',
  'installer.step3.hostWarningsPopup.empty.filesAndFolders': 'unwanted files and folders',
  'installer.step3.hostWarningsPopup.empty.processes': 'unwanted processes',
  'installer.step3.hostWarningsPopup.empty.packages': 'unwanted packages',
  'installer.step3.hostWarningsPopup.empty.users': 'unwanted users',
  'installer.step3.hostWarningsPopup.empty.services': 'unwanted services',
  'installer.step3.hostWarningsPopup.empty.misc': 'issues',
  'installer.step3.hostWarningsPopup.empty.firewall': 'firewalls running',
  'installer.step3.hostWarningsPopup.empty.repositories': 'repositories OS type mis-match with registered hosts',
  'installer.step3.hostWarningsPopup.empty.disk': 'disk space issues',
  'installer.step3.hostWarningsPopup.empty.jdk': 'JDK issues',
  'installer.step3.hostWarningsPopup.empty.thp': 'THP issues',
  'installer.step3.hostWarningsPopup.reverseLookup.name': 'Reverse Lookup validation failed on',
  'installer.step3.hostWarningsPopup.reverseLookup': 'Reverse Lookup Issues',
  'installer.step3.hostWarningsPopup.reverseLookup.message': 'The hostname was not found in the reverse DNS lookup. This may result in incorrect behavior. Please check the DNS setup and fix the issue.',
  'installer.step3.hostWarningsPopup.reverseLookup.empty': 'reverse DNS lookup issues.',
  'installer.step3.hostWarningsPopup.resolution.validation.name': 'Hostname Resolution Issues',
  'installer.step3.hostWarningsPopup.resolution.validation.error': 'Hostname resolution',
  'installer.step3.hostWarningsPopup.resolution.validation': 'Hostname resolution validation',
  'installer.step3.hostWarningsPopup.resolution.validation.message': 'Not all hosts could resolve hostnames of other hosts. Make sure that host resolution works properly on all hosts before continuing.',
  'installer.step3.hostWarningsPopup.resolution.validation.empty': 'hostname resolution issues',
  'installer.step3.hostWarningsPopup.resolution.validation.context': '{0} could not resolve: {1}.',
  'installer.step3.hostWarningsPopup.action.exists': 'Exists on',
  'installer.step3.hostWarningsPopup.action.notRunning': 'Not running on',
  'installer.step3.hostWarningsPopup.action.installed': 'Installed on',
  'installer.step3.hostWarningsPopup.action.running': 'Running on',
  'installer.step3.hostWarningsPopup.action.invalid': 'Invalid on',
  'installer.step3.hostWarningsPopup.action.failed': 'Failed on',
  'installer.step3.hostWarningsPopup.action.enabled': 'Enabled on',
  'installer.step3.hostWarningsPopup.host': 'host',
  'installer.step3.hostWarningsPopup.hosts': 'hosts',
  'installer.step3.hostWarningsPopup.moreHosts': '{0} more hosts...<br>Click on link to view all hosts.',
  'installer.step3.hostWarningsPopup.allHosts': 'List of hosts',
  'installer.step3.hostWarningsPopup.rerunChecks': 'Rerun Checks',
  'installer.step3.hostWarningsPopup.hostHasWarnings': 'Host checks failed on some of your hosts. It is highly recommended that you fix these problems first before proceeding to prevent potentially major problems with cluster installation. Are you sure you want to ignore these warnings and proceed?',
  'installer.step3.hostWarningsPopup.hostHasWarnings.header': 'Host Check Warning',
  'installer.step3.warningsWindow.allHosts': 'Warnings across all hosts',
  'installer.step3.warningsWindow.warningsOn': 'Warnings on ',
  'installer.step3.warningsWindow.directoriesAndFiles': 'DIRECTORIES AND FILES',
  'installer.step3.warningsWindow.noWarnings': 'No warnings',
  'installer.step3.hosts.noHosts': 'No hosts to display',
  'installer.step3.warnings.popup.header': 'Host Checks',
  'installer.step3.warnings.description': 'Some warnings were encountered while performing checks against the above hosts.',
  'installer.step3.warnings.linkText': 'Click here to see the warnings.',
  'installer.step3.noWarnings.linkText': 'Click here to see the check results.',
  'installer.step3.warnings.noWarnings': 'All host checks passed on {0} registered hosts.',
  'installer.step3.warnings.fails': 'Some warnings were encountered while performing checks against the {0} registered hosts above',
  'installer.step3.warnings.someWarnings': 'All host checks passed on {0} registered hosts. Note: Host checks were skipped on {1} hosts that failed to register.',
  'installer.step3.warnings.allFailed': 'Host checks were skipped on {0} hosts that failed to register.',
  'installer.step3.warnings.updateChecks.success': 'Host Checks successfully updated',
  'installer.step3.warnings.updateChecks.failed': 'Host Checks update failed',
  'installer.step3.warnings.missingHosts': 'There is no registered host',
  'installer.step3.warning.registeredHosts': '{0} Other Registered Hosts',
  'installer.step3.warning.loading': 'Please wait while the hosts are being checked for potential problems...',
  'installer.step3.registeredHostsPopup': 'These are the hosts that have registered with the server, but do not appear in the list of hosts that you are adding.',
  'installer.step3.removeSelected': 'Remove Selected',
  'installer.step3.retryFailed': 'Retry Failed',
  'installer.step3.hosts.status.registering': 'Registering',
  'installer.step3.hosts.status.installing': 'Installing',
  'installer.step3.hosts.bootLog.failed': '\nRegistration with the server failed.',
  'installer.step3.hosts.bootLog.registering': '\nRegistering with the server...',
  'installer.step3.hostLogPopup.highlight': 'click to highlight',
  'installer.step3.hostLogPopup.copy': 'press CTRL+C',
  'installer.step3.hostsTable.selectAll': 'Select All Hosts',
  'installer.step3.selectedHosts.popup.header': 'Selected Hosts',

  'installer.step4.header': 'Choose Services',
  'installer.step4.body': 'Choose which services you want to install on your cluster.',
  'installer.step4.headerFS': 'Choose File System',
  'installer.step4.bodyFS': 'Choose which file system you want to install on your cluster.',
  'installer.step4.fsCheck.popup.header': 'File System Required',
  'installer.step4.fsCheck.popup.body': 'You did not select a File System but one is required. We will automatically add {0}. Is this OK?',
  'installer.step4.multipleDFS.popup.header': 'Multiple File Systems Selected',
  'installer.step4.multipleDFS.popup.body': 'You selected more than one file system. We will automatically select only {0}. Is this OK?',
  'installer.step4.serviceCheck.popup.header': '{0} Needed',
  'installer.step4.serviceCheck.popup.body': 'You did not select {0}, but it is needed by other services you selected. We will automatically add {1}. Is this OK?',
  'installer.step4.serviceCheck.popup.body.multiOptions': 'You did not select {0}, but it is needed by other services you selected. Select a compatible service from the following list: {1}',
  'installer.step4.hcfs.displayName': 'a Hadoop Compatible File System',
  'installer.step4.limitedFunctionality.popup.header': 'Limited Functionality Warning',
  'installer.step4.ambariMetricsCheck.popup.header': 'Ambari Metrics',
  'installer.step4.ambariMetricsCheck.popup.body': 'Ambari Metrics collects metrics from the cluster and makes them available to Ambari.  If you do not install Ambari Metrics service, metrics will not be accessible from Ambari.  Are you sure you want to proceed without Ambari Metrics?',
  'installer.step4.ambariRangerInfraCheck.popup.header': 'Infra Solr',
  'installer.step4.ambariRangerInfraCheck.popup.body': 'In order for Ranger to work, Solr is required. Either select Infra Solr to be added to the cluster, or ensure you configure your own during the Customize Services step.',
  'installer.step4.ambariAtlasInfraCheck.popup.header': 'Infra Solr',
  'installer.step4.ambariAtlasInfraCheck.popup.body': 'In order for Atlas to work, Solr is required. Either select Infra Solr to be added to the cluster, or ensure you configure your own during the Customize Services step.',
  'installer.step4.ambariAtlasHbaseCheck.popup.header': 'HBase',
  'installer.step4.ambariAtlasHbaseCheck.popup.body': 'In order for Atlas to work, HBase is required. Either select HBase to be added to the cluster, or ensure you configure your own during the Customize Services step.',
  'installer.step4.ambariLogsearchCheck.popup.header': 'Ambari Logsearch',
  'installer.step4.ambariLogsearchCheck.popup.body': 'Since Ambari Infra is not selected, you must supply your own Solr to make Log Search work. Are you sure you want to proceed?',
  'installer.step4.smartSenseCheck.popup.header': 'Apache SmartSense',
  'installer.step4.smartSenseCheck.popup.body': 'SmartSense securely collects cluster diagnostics in order to send you recommendations and automate analysis for ' + 'troubleshooting support cases. If you choose not to install SmartSense, you will not receive any recommendations, and you ' + 'will need to collect configuration and logs manually for troubleshooting support cases. ' + 'Are you sure you want to proceed without SmartSense?',
  'installer.step4.rangerCheck.popup.header': 'Apache Ranger',
  'installer.step4.rangerCheck.popup.body': 'Apache Ranger provides fine grained authorization and audit of access attempts for many Hadoop ecosystem services. If you do not install the Apache Ranger Service and enable Kerberos, the security of your cluster will be diminished. Are you sure you want to proceed without it? ',
  'installer.step4.atlasCheck.popup.header': 'Apache Atlas',
  'installer.step4.atlasCheck.popup.body': 'Apache Atlas provides strong data governance, lineage and metadata management for many hadoop ecosystem services. If you do not install Apache Atlas service, the data governance capabilities of you cluster will be diminished. Are you sure you want to proceed without it? ',

  'installer.step4.rangerRequirements.popup.header': 'Ranger Requirements',
  'installer.step4.rangerRequirements.popup.body.requirements': '<ol><li>You must have an <strong>MySQL/Oracle/Postgres/MSSQL/SQL Anywhere Server</strong> database instance running to be used by Ranger.</li>' + '<li>In Assign Masters step of this wizard, you will be prompted to specify which host for the Ranger Admin. On that host, you <strong>must have DB Client installed</strong> for Ranger to access to the database. (Note: This is applicable for only Ranger 0.4.0)</li>' + '<li>Ensure that the access for the DB Admin user is enabled in DB server from any host.</li>' + '<li>Execute the following command on the Ambari Server host. Replace <code>database-type</code> with <strong>mysql|oracle|postgres|mssql|sqlanywhere</strong> and <code>/jdbc/driver/path</code> based on the location of corresponding JDBC driver:' + '<pre>ambari-server setup --jdbc-db={database-type} --jdbc-driver={/jdbc/driver/path}</pre></li></ol>',
  'installer.step4.rangerRequirements.popup.body.confirmation': 'I have met all the requirements above.',
  'installer.step4.sparkWarning.popup.body': 'Spark requires HDP 2.2.2 or later. Attempting to install Spark to a HDP 2.2.0 cluster will fail. Confirm you are using HDP 2.2.2 or later packages. Are you sure you want to proceed?',

  'installer.step5.header': 'Assign Masters',
  'installer.step5.reassign.header': 'Select Target Host',
  'installer.step5.attention': ' hosts not running master services',
  'installer.step5.body': 'Assign master components to hosts you want to run them on.',
  'installer.step5.body.coHostedComponents': '<i class="glyphicon glyphicon-asterisks">&#10037;</i> {0} will be hosted on the same host.',
  'installer.step5.hostInfo': '%@ (%@, %@ cores)',
  'installer.step5.hiveGroup': 'HiveServer2, WebHCat Server, MySQL Server',
  'installer.step5.validationIssuesAttention.header': 'Validation Issues',
  'installer.step5.validationIssuesAttention': 'Master component assignments have issues that need attention.',
  'installer.step5.error.host.invalid': 'The host you choose could (1) have an invalid host name; (2) be conflicting with another selection; or (3) be in maintenance mode',

  'installer.step6.header': 'Assign Slaves and Clients',
  'installer.step6.body': 'Assign slave and client components to hosts you want to run them on.<br/>Hosts that are assigned master components are shown with <i class="glyphicon glyphicon-asterisks">&#10037;</i>.',
  'installer.step6.body.clientText': ' <br/>&quot;Client&quot; will install ',
  'installer.step6.error.mustSelectOne': 'You must assign at least one host to each component.',
  'installer.step6.error.mustSelectOneForHost': 'You must assign at least one slave/client component to each.',
  'installer.step6.error.mustSelectComponents': 'You must assign at least: {0}',
  'installer.step6.wizardStep6Host.title': 'master components hosted on {0}',
  'installer.step6.addHostWizard.body': 'Assign HBase master and ZooKeeper server.',
  'installer.step6.error.mustSelectOneForSlaveHost': 'You must assign at least one slave/client component to each host with no master component',
  'installer.step6.validationSlavesAndClients.click': 'Click',
  'installer.step6.validationSlavesAndClients.popup.body': 'Assignment of slave and client components has the following issues',
  'installer.step6.validationIssuesAttention.header': 'Validation Issues',
  'installer.step6.validationIssuesAttention': 'Slave and Client component assignments have issues that need attention.',

  'installer.step7.header': 'Customize Services',
  'installer.step7.body': 'We have come up with recommended configurations for the services you selected. Customize them as you see fit.',
  'installer.step7.credentialsTab.body': 'Please provide credentials for these services',
  'installer.step7.databasesTab.body': 'Please choose and configure the appropriate databases for these services',
  'installer.step7.databasesTab.radioButton.default': 'Use Ambari Database',
  'installer.step7.databasesTab.radioButton.custom': 'Use Custom Database',
  'installer.step7.credentialsTab.usersGroups': 'Users/Groups',
  'installer.step7.credentialsTab.usernames': 'Usernames',
  'installer.step7.accountsTab.body': 'Please review these settings for Service Accounts',
  'installer.step7.attentionNeeded': '<b>Attention:</b> Some configurations need your attention before you can proceed.',
  'installer.step7.noIssues': 'All configurations have been addressed.',
  'installer.step7.showPropertiesWithIssues': 'Show me properties with issues',
  'installer.step7.showingPropertiesWithIssues': 'Showing properties with issues.',
  'installer.step7.showAllProperties': 'Show all properties',
  'installer.step7.config.addProperty': 'Add Property',
  'installer.step7.ConfigErrMsg.header': 'Custom configuration error: ',
  'installer.step7.ConfigErrMsg.message': 'Error in custom configuration. Some properties entered in the box are already exposed on this page',
  'installer.step7.popup.currentValue': 'Current Value',
  'installer.step7.popup.adjustedValue': 'Adjusted Value',
  'installer.step7.popup.rddWarning.header': 'Warning: disk space low on {0}',
  'installer.step7.popup.rddWarning.body': 'A minimum of 16GB is recommended for the Ganglia Server logs but the disk mount "{0}" on {1} does not have enough space ({2} free). Go to the Misc tab and change Ganglia rrdcached base directory with more than 16GB of disk space. If you proceed without changing it, {1} will likely run out of disk space and become inoperable.',
  'installer.step7.popup.mySQLWarning.header': 'Error: New MySQL Database for Hive Conflict',
  'installer.step7.popup.mySQLWarning.body': 'You cannot install a \"New MySQL Database\" for Hive on the same host as the Ambari Server MySQL Server. Please go back to <b>Assign Masters</b> and reassign Hive to another host <b>OR</b> choose \"Existing MySQL Database\" option to specify the Database Credentials and URL for the Ambari Server MySQL Server instance. If you choose \"Existing MySQL Database\" option, you need to perform the Hive prerequisite steps to prepare the MySQL Server instance for Hive.',
  'installer.step7.popup.mySQLWarning.button.gotostep5': 'Go to Assign Masters',
  'installer.step7.popup.mySQLWarning.button.dismiss': 'Dismiss',
  'installer.step7.popup.mySQLWarning.confirmation.header': 'Confirmation: Go to Assign Masters',
  'installer.step7.popup.mySQLWarning.confirmation.body': 'You will be brought back to the \"Assign Masters\" step and will lose all your current customizations. Are you sure?',
  'installer.step7.popup.database.connection.header': 'Database Connectivity Warning',
  'installer.step7.popup.database.connection.body': 'You have not run or passed the database connection test for: {0}. It is highly recommended that you pass the connection test before proceeding to prevent failures during deployment.',
  'installer.step7.popup.validation.failed.header': 'Consistency Check Failed',
  'installer.step7.popup.validation.failed.body': 'Some services are not properly configured. You have to change the highlighted configs according to the recommended values.',
  'installer.step7.popup.validation.request.failed.body': 'The configuration changes could not be validated for consistency due to an unknown error.  Your changes have not been saved yet.  Would you like to proceed and save the changes?',
  'installer.step7.popup.validation.warning.header': 'Configurations',
  'installer.step7.popup.validation.issues.title': 'Highly Recommended Configurations',
  'installer.step7.popup.validation.issues.body': 'Please review the folowing recommended changes, and click on the property name to change its value.',
  'installer.step7.popup.validation.criticalIssues.body': 'You must correct the following critical issues before proceeding:',
  'installer.step7.popup.oozie.derby.warning': 'Derby is not recommended for production use. With Derby, Oozie Server HA and concurrent connection support will not be available.',
  'installer.step7.oozie.database.new': 'New Derby Database',
  'installer.step7.hive.database.new.mysql': 'New MySQL Database',
  'installer.step7.hive.database.new.postgres': 'New PostgreSQL Database',
  'installer.step7.addWizard.notEditable': 'This configuration cannot be edited on Add Service wizard',
  'installer.step7.misc.notification.configure': 'Configure email notifications',
  'installer.step7.misc.notification.configure.later': 'Configure email notifications later',
  'installer.step7.misc.notification.use_tls': 'Use TLS',
  'installer.step7.misc.notification.use_ssl': 'Use SSL',

  'installer.step7.preInstallChecks': 'Pre Install Checks',
  'installer.step7.preInstallChecks.notRunChecksWarnPopup.header': 'Skipping Pre Install Checks',
  'installer.step7.preInstallChecks.notRunChecksWarnPopup.body': 'Skipping Pre Install Checks is not recommended.',
  'installer.step7.preInstallChecks.notRunChecksWarnPopup.primary': 'Ignore and Proceed',
  'installer.step7.preInstallChecks.notRunChecksWarnPopup.secondary': 'Run Pre Install Checks',
  'installer.step7.preInstallChecks.checksPopup.header': 'Pre Install Checks',

  'installer.step7.assign.master.body': 'Assign <strong>{0}</strong> to {1} you want to run {2} on.',
  'installer.step7.assign.master.dependent.component.body': 'If not present {0} will also be installed on the selected host. ',
  'installer.step7.missing.service.header': 'Missing Service',
  'installer.step7.missing.service.body': '{0} service should be added to the cluster to {1}.',
  'assign.master.popup.header': 'Select {0} host',
  'assign.master.popup.cancel.body': 'Not selecting {0} host will disable interactive query.',
  'installer.step7.recommendations.popover.header': 'Review Recommendations',
  'installer.step7.recommendations.popover.message': '<p>Based on your configuration changes, Ambari is recommending the following dependent configuration changes. Ambari has updated all checked configuration changes to the <strong>Recommended Value</strong>. Uncheck any configuration to retain the <strong>Original Value</strong>.</p>',
  'installer.step7.requirements.popover.header': 'Required Configurations',
  'installer.step7.requirements.popover.message': '<p>The following properties must be set to proceed with the install.</p>',

  'installer.step8.header': 'Review',
  'installer.step8.body': 'Please review the configuration before installation',
  'installer.step8.kerberors.warning': '<strong>Because Kerberos has been manually installed on the cluster, you will have to create/distribute principals and keytabs when this operation is finished.</strong>',
  'installer.step8.generateBlueprint.popup.msg': 'You will be <b>downloading</b> blueprint.json and clustertemplate.json to your local machine. <br/><br/> You can use them to deploy a cluster via AMBARI REST API. <br/> Update clustertemplate.json with a custom cluster name if you do not want to use name <b>{0}</b>. <br/><br/>If you want to use custom repo URLs, make sure to update them via Ambari REST API before deploying the cluster.',
  'installer.step8.deployPopup.message': 'Preparing to Deploy: {0} of {1} tasks completed.',
  'installer.step8.deployPopup.header': 'Initializing Tasks',
  'installer.step8.hosts': ' hosts',
  'installer.step8.host': ' host',
  'installer.step8.other': 'and {0} other hosts',
  'installer.step8.repoInfo.osType.redhat6': 'RHEL 6/CentOS 6/Oracle Linux 6',
  'installer.step8.repoInfo.osType.redhat5': 'RHEL 5/CentOS 5/Oracle Linux 5',
  'installer.step8.repoInfo.osType.sles11': 'SLES 11/SUSE 11',
  'installer.step8.repoInfo.displayName': 'Repositories',
  'installer.step8.services.restart.required': '{0} {1} will be restarted during installation.',
  'installer.step9.header': 'Install, Start and Test',
  'installer.step9.body': 'Please wait while the selected services are installed and started.',
  'installer.step9.status.success': 'Successfully installed and started the services.',
  'installer.step9.status.skipStartSuccess': 'Successfully installed the services.',
  'installer.step9.status.warning': 'Installed and started the services with some warnings.',
  'installer.step9.status.failed': 'Failed to install/start the services.',
  'installer.step9.status.start.services.failed': 'Start all services API call failed.',
  'installer.step9.status.install.components.failed': 'Some service components are still not known to have installed successfully. Please Retry',
  'installer.step9.status.hosts.heartbeat_lost': 'Ambari agent is not running on {0} hosts.',
  'installer.step9.host.heartbeat_lost': 'Heartbeat lost for the host',
  'installer.step9.host.heartbeat_lost_popup': 'Ambari agent process is not heartbeating on the host',
  'installer.step9.host.heartbeat_lost.header': 'Hosts with heartbeat lost',
  'installer.step9.host.status.success': 'Success',
  'installer.step9.host.status.startAborted': 'Install completed. Start aborted',
  'installer.step9.host.status.warning': 'Warnings encountered',
  'installer.step9.host.status.failed': 'Failures encountered',
  'installer.step9.host.status.nothingToInstall': 'Install complete (Waiting to start)',
  'installer.step9.hosts.status.label.inProgress': 'In Progress',
  'installer.step9.hosts.status.label.warning': 'Warning',
  'installer.step9.hosts.filteredHostsInfo': '{0} of {1} hosts showing',
  'installer.step9.hostLog.popup.header': 'Tasks. executed on ',
  'installer.step9.hostLog.popup.categories.all': 'All',
  'installer.step9.hostLog.popup.categories.pending': 'Queued / Pending',
  'installer.step9.hostLog.popup.categories.in_progress': 'In Progress',
  'installer.step9.hostLog.popup.categories.failed': 'Failed',
  'installer.step9.hostLog.popup.categories.completed': 'Success',
  'installer.step9.hostLog.popup.categories.aborted': 'Aborted',
  'installer.step9.hostLog.popup.categories.timedout': 'Timed Out',
  'installer.step9.hostLog.popup.noTasksToShow': 'No tasks to show',
  'installer.step9.host.status.noTasks': 'Ambari server did not schedule any tasks on the host. Either the service component on the host is already in installed state <b>OR</b> the pre-check of host reachability failed.',
  'installer.step9.overallProgress': '{0} % overall',
  'installer.step9.serviceStatus.install.pending': 'Preparing to install ',
  'installer.step9.serviceStatus.install.queued': 'Waiting to install ',
  'installer.step9.serviceStatus.install.inProgress': 'Installing ',
  'installer.step9.serviceStatus.install.completed': 'Successfully installed ',
  'installer.step9.serviceStatus.install.failed': 'Failed to install ',
  'installer.step9.serviceStatus.uninstall.pending': 'Preparing to uninstall ',
  'installer.step9.serviceStatus.uninstall.queued': 'Waiting to uninstall ',
  'installer.step9.serviceStatus.uninstall.inProgress': 'Uninstalling ',
  'installer.step9.serviceStatus.uninstall.completed': 'Successfully uninstalled ',
  'installer.step9.serviceStatus.uninstall.failed': 'Failed to uninstall ',
  'installer.step9.serviceStatus.start.pending': 'Preparing to start ',
  'installer.step9.serviceStatus.start.queued': 'Waiting to start ',
  'installer.step9.serviceStatus.start.inProgress': 'Starting ',
  'installer.step9.serviceStatus.start.completed': ' started successfully',
  'installer.step9.serviceStatus.start.failed': ' failed to start',
  'installer.step9.serviceStatus.stop.pending': 'Preparing to stop ',
  'installer.step9.serviceStatus.stop.queued': 'Waiting to stop ',
  'installer.step9.serviceStatus.stop.inProgress': 'Stopping ',
  'installer.step9.serviceStatus.stop.completed': ' stopped successfully',
  'installer.step9.serviceStatus.stop.failed': ' failed to stop',
  'installer.step9.serviceStatus.execute.pending': 'Preparing to execute ',
  'installer.step9.serviceStatus.execute.queued': 'Waiting to execute ',
  'installer.step9.serviceStatus.execute.inProgress': 'Executing ',
  'installer.step9.serviceStatus.execute.completed': ' executed successfully',
  'installer.step9.serviceStatus.execute.failed': ' failed to execute',
  'installer.step9.serviceStatus.abort.pending': 'Preparing to abort ',
  'installer.step9.serviceStatus.abort.queued': 'Waiting to abort ',
  'installer.step9.serviceStatus.abort.inProgress': 'Aborting ',
  'installer.step9.serviceStatus.abort.completed': ' aborted successfully',
  'installer.step9.serviceStatus.abort.failed': ' failed to abort',
  'installer.step9.components.install.failed': 'Installation Failure',
  'installer.step9.service.start.failed': 'There were issues starting installed services. Please go to individual service pages to start them.',
  'installer.step9.service.start.header': 'Start Services',

  'installer.step10.header': 'Summary',
  'installer.step10.body': 'Here is the summary of the install process.',
  'installer.step10.staleServicesRestartRequired': ' After closing this wizard, please restart all services ' + 'that have the restart indicator <i class="glyphicon glyphicon-refresh"></i> next to the service name.',
  'installer.step10.hostsSummary': 'The cluster consists of {0} hosts',
  'installer.step10.installedAndStarted': 'Installed and started services successfully on {0} new ',
  'installer.step10.installed': 'Installed services successfully on {0} new ',
  'installer.step10.warnings': ' warnings',
  'installer.step10.clusterState.installing': 'Installing ',
  'installer.step10.clusterState.starting': 'Starting ',
  'installer.step10.taskStatus.failed': ' failed on ',
  'installer.step10.taskStatus.aborted': ' aborted on ',
  'installer.step10.taskStatus.timedOut': ' timed out on ',
  'installer.step10.installStatus.failed': 'Installing master services failed',
  'installer.step10.installStatus.installed': 'Master services installed',
  'installer.step10.master.installedOn': '{0} installed on {1}',
  'installer.step10.startStatus.failed': 'Starting services failed',
  'installer.step10.startStatus.skipped': 'Starting services skipped',
  'installer.step10.startStatus.passed': 'All tests passed',
  'installer.step10.startStatus.started': 'All services started',
  'installer.step10.installTime.seconds': 'Install and start completed in {0} seconds',
  'installer.step10.installTime.minutes': 'Install and start completed in {0} minutes and {1} seconds',

  'addHost.step4.header': 'Configurations',
  'addHost.step4.title': 'Select the configuration groups to which the added hosts will belong to.',

  'form.create': 'Create',
  'form.save': 'Save',
  'form.cancel': 'Cancel',
  'form.passwordRetype': 'Retype Password',
  'form.saveSuccess': 'Successfully saved.',
  'form.saveError': 'Sorry, errors occurred.',
  'form.item.placeholders.typePassword': 'Type password',

  'form.validator.invalidIp': 'Please enter valid ip address',
  'form.validator.configKey': 'Invalid Key. Only alphanumerics, hyphens, underscores, asterisks and periods are allowed.',
  'form.validator.configGroupName': 'Invalid Group Name. Only alphanumerics, hyphens, spaces and underscores are allowed.',
  'form.validator.alertGroupName': 'Invalid Alert Group Name. Only alphanumerics, hyphens, spaces and underscores are allowed.',
  'form.validator.alertNotificationName': 'Invalid Alert Notification Name. Only alphanumerics, hyphens, spaces and underscores are allowed.',
  'form.validator.configKey.specific': '"{0}" is invalid Key. Only alphanumerics, hyphens, underscores, asterisks and periods are allowed.',

  'alerts.dropdown.dialog.title': 'Notifications',
  'alerts.dropdown.dialog.filters.critical': 'Critical ({0})',
  'alerts.dropdown.dialog.filters.warning': 'Warning ({0})',
  'alerts.dropdown.dialog.filters.all': 'All ({0})',

  'alerts.add.header': 'Create Alert',
  'alerts.add.step1.header': 'Choose Alert Type',
  'alerts.add.step1.header.description': 'Select the type of alert you want to create',
  'alerts.add.step2.header': 'Define Alert',
  'alerts.add.step3.header': 'Specify Threshold',
  'alerts.add.step3.selectedType': 'Selected Type',

  'alerts.add.wizard.step1.body.choose.tooltip': 'Click to select',
  'alerts.add.wizard.step1.body.port.description': 'Check TCP connectivity to a remote endpoint',
  'alerts.add.wizard.step1.body.web.description': 'Check for TCP connectivity and verify that a proper HTTP response code was returned',
  'alerts.add.wizard.step1.body.metric.description': 'Define JMX/AMS endpoints that can be queried for values',
  'alerts.add.wizard.step1.body.script.description': 'Defer all functionality to a Python script accessible to the Ambari agents from a specified relative or absolute path',
  'alerts.add.wizard.step1.body.aggregate.description': 'Combine the results of another alert definition from different nodes',
  'alerts.add.wizard.step1.body.raw.description': 'User can upload alert parameter using a JSON script',

  'alerts.fastAccess.popup.header': '{0} Critical or Warning Alerts',
  'alerts.fastAccess.popup.body.name': 'Alert Definition Name',
  'alerts.fastAccess.popup.body.showmore': 'Go to Alerts Definitions',
  'alerts.fastAccess.popup.body.noalerts': 'No critical or warning alerts to display',

  'alerts.actions.create': 'Create Alert',
  'alerts.actions.manageGroups': 'Manage Alert Groups',
  'alerts.actions.manageNotifications': 'Manage Notifications',
  'alerts.actions.manageNotifications.info': 'You can manage notification methods and recipients.',
  'alerts.actions.editRepeatTolerance.header': 'Edit Alert Check Count',
  'alerts.actions.editRepeatTolerance.title': 'Alert Check Counts',
  'alerts.actions.editRepeatTolerance.description': 'Set the number of alert checks to perform before dispatching a notification. ' + 'If during an alert check a state change occurs, Ambari will attempt to check this number of times before dispatching a notification. ' + 'Increase this number if your environment experiences transient issues resulting in false alerts.',
  'alerts.actions.editRepeatTolerance.label': 'Check Count:',
  'alerts.actions.editRepeatTolerance.error': 'Input should be an integer between 1 and 99.',
  'alerts.actions.manageSettings': 'Manage Alert Settings',
  'alerts.actions.manageSettings.error': 'Error during remote command: ',

  'alerts.groups.successPopup.header': 'Alert Groups processing results',
  'alerts.groups.successPopup.body.created': 'New Alert Groups',
  'alerts.groups.successPopup.body.updated': 'Updated Alert Groups',
  'alerts.groups.successPopup.body.deleted': 'Removed Alert Groups',

  'alerts.table.noAlerts': 'No Alerts to display',
  'alerts.table.header.lastTriggered': 'Last Status Changed',
  'alerts.table.header.lastChecked': 'Last Checked',
  'alerts.table.header.lastTrigger': 'Last Changed',
  'alerts.table.header.check.response': 'Response',
  'alerts.table.header.definitionName': 'Alert Definition Name',
  'alerts.table.header.notification': 'Notification',
  'alerts.table.header.repeatTolerance': 'Check Count',
  'alerts.table.header.default': '(default)',
  'alerts.table.header.helpUrl': 'Alert Context',
  'alerts.table.state': 'State',
  'alerts.table.state.enabled': 'Enabled',
  'alerts.table.state.disabled': 'Disabled',
  'alerts.table.state.enabled.tooltip': 'Click to disable this alert definition',
  'alerts.table.state.disabled.tooltip': 'Click to enable this alert definition',
  'alerts.table.state.enabled.confirm.msg': 'You are about to Disable this alert definition.',
  'alerts.table.state.disabled.confirm.msg': 'You are about to Enable this alert definition.',
  'alerts.table.state.enabled.confirm.btn': 'Confirm Disable',
  'alerts.table.state.disabled.confirm.btn': 'Confirm Enable',
  'alerts.filters.filteredAlertsInfo': '{0} of {1} definitions showing',
  'alerts.definition.name': 'Alert Definition Name',
  'alerts.saveChanges': 'You have unsaved changes',

  'alert.definition.name.invalid': 'Invalid input. Only alphanumerics, underscores, hyphens, percentage, brackets and spaces are allowed. Value should be less than 255 symbols.',
  'alerts.definition.details.enable': 'Enable',
  'alerts.definition.details.disable': 'Disable',
  'alerts.definition.details.enableDisable': 'Enable / Disable',
  'alerts.definition.details.groups': 'Groups',
  'alerts.definition.details.instances': 'Instances',
  'alerts.definition.details.serviceHost': 'Service / Host',
  'alerts.definition.details.24-hour': '24-Hour',
  'alerts.definition.details.notification': 'Notification',
  'alerts.definition.details.noAlerts': 'No alert instances to display',
  'alerts.definition.details.configs.thresholdsErrorMsg': 'Critical threshold should be larger than warning threshold',
  'alerts.definition.details.infoSection': 'Alert Info',

  'alerts.notifications.error.email': 'Must be a valid email address',
  'alerts.notifications.error.integer': 'Must be an integer',
  'alerts.notifications.error.host': 'Hosts must be a valid Fully Qualified Domain Name (FQDN)',

  'alerts.notifications.error.SMTPUsername': 'Username is required',
  'alerts.notifications.error.SMTPPassword': 'Password is required',
  'alerts.notifications.error.retypePassword': 'Password confirmation must match password',

  'alerts.notifications.addCustomPropertyPopup.header': 'Add Property',
  'alerts.notifications.addCustomPropertyPopup.error.propertyExists': 'Custom Property with current name already exists',
  'alerts.notifications.addCustomPropertyPopup.error.invalidPropertyName': 'Property name can only contain letters, numbers or . -_* characters',
  'alerts.notifications.regenerateKeytab.service.error': 'Keytabs for {0} service could not be regenerated',
  'alerts.notifications.regenerateKeytab.host.error': 'Keytabs for {0} host could not be regenerated',

  'wizard.progressPage.notice.completed': 'Please proceed to the next step.',
  'wizard.progressPage.notice.failed': 'You can click on the Retry button to retry failed tasks.',
  'wizard.singleRequest.progressPage.notice.failed': 'Please click on the Retry link to retry the failed request.',

  'admin.advanced.caution': 'This section is for advanced user only.<br/>Proceed with caution.',
  'admin.advanced.button.uninstallIncludingData': 'Uninstall cluster including all data.',
  'admin.advanced.button.uninstallKeepData': 'Uninstall cluster but keep data.',

  'admin.advanced.popup.header': 'Uninstall Cluster',

  /*'admin.advanced.popup.text':'Uninstall Cluster',*/

  'admin.audit.grid.date': "Date/Time",
  'admin.audit.grid.category': "Category",
  'admin.audit.grid.operationName': "Operation",
  'admin.audit.grid.performedBy': "Performed By",
  'admin.audit.grid.service': "Category",

  'admin.authentication.form.method.database': 'Use Ambari Database to authenticate users',
  'admin.authentication.form.method.ldap': 'Use LDAP/Active Directory to authenticate',
  'admin.authentication.form.primaryServer': 'Primary Server',
  'admin.authentication.form.secondaryServer': 'Secondary Server',
  'admin.authentication.form.useSsl': 'Use SSL',
  'admin.authentication.form.bind.anonymously': "Bind Anonymously",
  'admin.authentication.form.bind.useCrenedtials': "Use Credentials To Bind",
  'admin.authentication.form.bindUserDN': 'Bind User DN',
  'admin.authentication.form.searchBaseDN': 'Search Base DN',
  'admin.authentication.form.usernameAttribute': 'Username Attribute',

  'admin.authentication.form.userDN': 'User DN',
  'admin.authentication.form.configurationTest': 'Configuration Test',
  'admin.authentication.form.testConfiguration': 'Test Configuration',

  'admin.authentication.form.test.success': 'The configuration passes the test',
  'admin.authentication.form.test.fail': 'The configuration fails the test',

  'admin.kerberos.credentials.store.hint.supported': 'When checked, Ambari will store the KDC Admin credentials so they are not required to be re-entered during future changes of services, hosts, and components.',
  'admin.kerberos.credentials.store.hint.not.supported': 'Ambari is not configured for storing credentials',
  'admin.kerberos.credentials.store.label': 'Save Admin Credentials',
  'admin.kerberos.credentials.store.menu.label': 'Manage KDC Credentials',
  'admin.kerberos.credentials.form.header.stored': 'Update or remove the stored KDC Credentials in the encrypted credential store.',
  'admin.kerberos.credentials.form.header.not.stored': 'Specify the KDC Admin Credentials to remember in the encrypted credential store.',
  'admin.kerberos.credentials.remove.confirmation.header': 'Remove KDC Credentials Confirmation',
  'admin.kerberos.credentials.remove.confirmation.body': 'You are about to remove the KDC Credentials from Ambari. Are you sure?',
  'admin.kerberos.wizard.configuration.note': 'This is the initial configuration created by Enable Kerberos wizard.',
  'admin.kerberos.wizard.header': 'Enable Kerberos Wizard',
  'admin.kerberos.button.enable': 'Enable Kerberos',
  'admin.kerberos.button.disable': 'Disable Kerberos',
  'admin.kerberos.button.regenerateKeytabs': 'Regenerate Keytabs',
  'admin.kerberos.wizard.exit.warning.msg': 'Configuring Kerberos is in progress. Do you really want to exit the Enable Kerberos Wizard?',
  'admin.kerberos.wizard.exit.critical.msg': 'Configuring Kerberos is in progress. <strong>Before dismissing, you should complete the wizard.</strong> Do you really want to exit the Enable Kerberos Wizard?',
  'admin.kerberos.wizard.step1.header': 'Get Started',
  'admin.kerberos.wizard.step2.header': 'Configure Kerberos',
  'admin.kerberos.wizard.step3.header': 'Install and Test Kerberos Client',
  'admin.kerberos.wizard.step3.checkbox.ignoreAndProceed.label': 'Ignore errors and continue to next step',
  'admin.kerberos.wizard.step4.header': 'Configure Identities',
  'admin.kerberos.wizard.step5.header': 'Confirm Configuration',
  'admin.kerberos.wizard.step6.header': 'Stop Services',
  'admin.kerberos.wizard.step7.header': 'Kerberize Cluster',
  'admin.kerberos.wizard.step8.header': 'Start and Test Services',
  'admin.kerberos.wizard.step1.info.body': 'Welcome to the Ambari Security Wizard. Use this wizard to enable kerberos security in your cluster. </br>Let\'s get started.',
  'admin.kerberos.wizard.step1.alert.body': 'Note: This process requires services to be restarted and cluster downtime. As well, depending on the options you select, might require support from your Security administrators. Please plan accordingly.',
  'admin.kerberos.wizard.step1.body.text': 'What type of KDC do you plan on using?',
  'admin.kerberos.wizard.step1.option.kdc': 'Existing MIT KDC',
  'admin.kerberos.wizard.step1.option.kdc.condition.1': 'Ambari Server and cluster hosts have network access to both the KDC and KDC admin hosts.',
  'admin.kerberos.wizard.step1.option.kdc.condition.2': 'KDC administrative credentials are on-hand.',
  'admin.kerberos.wizard.step1.option.kdc.condition.3': 'The Java Cryptography Extensions (JCE) have been setup on the Ambari Server host and all hosts in the cluster.',
  'admin.kerberos.wizard.step1.option.kdc.condition.4': 'The Isilon administrator has setup all appropriate principals in OneFS',
  'admin.kerberos.wizard.step1.option.manual': 'Manage Kerberos principals and keytabs manually',
  'admin.kerberos.wizard.step1.option.manual.condition.1': 'Cluster hosts have network access to the KDC',
  'admin.kerberos.wizard.step1.option.manual.condition.2': 'Kerberos client utilities (such as kinit) have been installed on every cluster host',
  'admin.kerberos.wizard.step1.option.manual.condition.3': 'The Java Cryptography Extensions (JCE) have been setup on the Ambari Server host and all hosts in the cluster',
  'admin.kerberos.wizard.step1.option.manual.condition.4': 'The Service and Ambari Principals will be manually created in the KDC before completing this wizard',
  'admin.kerberos.wizard.step1.option.manual.condition.5': 'The keytabs for the Service and Ambari Principals will be manually created and distributed to cluster hosts before completing this wizard',
  'admin.kerberos.wizard.step1.option.ad': 'Existing Active Directory',
  'admin.kerberos.wizard.step1.option.ad.condition.1': 'Ambari Server and cluster hosts have network access to the Domain Controllers.',
  'admin.kerberos.wizard.step1.option.ad.condition.2': 'Active Directory secure LDAP (LDAPS) connectivity has been configured.',
  'admin.kerberos.wizard.step1.option.ad.condition.3': 'Active Directory User container for principals has been created and is on-hand (e.g. OU=Hadoop,OU=People,dc=apache,dc=org)',
  'admin.kerberos.wizard.step1.option.ad.condition.4': 'Active Directory administrative credentials with delegated control of “Create, delete, and manage user accounts” on the previously mentioned User container are on-hand.',
  'admin.kerberos.wizard.step1.option.ad.condition.5': 'The Java Cryptography Extensions (JCE) have been setup on the Ambari Server host and all hosts in the cluster.',
  'admin.kerberos.wizard.step1.option.ipa': 'Existing IPA',
  'admin.kerberos.wizard.step1.option.ipa.condition.1': 'All cluster hosts are joined to the IPA domain and hosts are registered in DNS',
  'admin.kerberos.wizard.step1.option.ipa.condition.2': 'A password policy is in place that sets no expiry for created principals',
  'admin.kerberos.wizard.step1.option.ipa.condition.3': 'If you do not plan on using Ambari to manage the krb5.conf, ensure the following is set in each krb5.conf file in your cluster: default_ccache_name = /tmp/krb5cc_%{uid}',
  'admin.kerberos.wizard.step1.option.ipa.condition.4': 'The Java Cryptography Extensions (JCE) have been setup on the Ambari Server host and all hosts in the cluster.',
  'admin.kerberos.wizard.step1.prerequisites.label': 'Following prerequisites needs to be checked to progress ahead in the wizard.',
  'admin.kerberos.wizard.step2.info.body': 'Please configure kerberos related properties.',
  'admin.kerberos.wizard.step3.task0.title': 'Install Kerberos Client',
  'admin.kerberos.wizard.step3.task1.title': 'Test Kerberos Client',
  'admin.kerberos.wizard.step3.notice.inProgress': 'Please wait while the Kerberos clients are being installed and tested.',
  'admin.kerberos.wizard.step3.notice.completed': 'Kerberos service has been installed and tested successfully.',
  'admin.kerberos.wizard.progressPage.notice.inProgress': 'Please wait while cluster is being kerberized',
  'admin.kerberos.wizard.step4.info.body': 'Configure principal name and keytab location for service users and hadoop service components.',
  'admin.kerberos.wizard.step5.info.body': 'Please review the configuration before continuing the setup process',
  'admin.kerberos.wizard.step5.moreInfoNonManual.body': 'Using the <b>Download CSV button</b>, you can download a csv file which contains a list of the principals and keytabs that will automatically be created by Ambari.',
  'admin.kerberos.wizard.step5.moreInfoManual.body': 'Important: Use the <b>Download CSV</b> button to obtain a list of the <b>required</b> principals and keytabs that are needed by Ambari to enable Kerberos in the cluster. <b>Do not proceed</b> until you have manually created and distributed the principals and keytabs to the cluster hosts.',
  'admin.kerberos.wizard.step5.kdc_type.label': 'KDC Type',
  'admin.kerberos.wizard.step5.kdc_hosts.label': 'KDC Hosts',
  'admin.kerberos.wizard.step5.realm.label': 'Realm Name',
  'admin.kerberos.wizard.step5.ldap_url.label': 'LDAP URL',
  'admin.kerberos.wizard.step5.container_dn.label': 'Container DN',
  'admin.kerberos.wizard.step5.executable_search_paths.label': 'Executable path',
  'admin.kerberos.wizard.step5.exitWizard': 'Exit Wizard',
  'admin.kerberos.wizard.step5.downloadCSV': 'Download CSV',
  'admin.kerberos.wizard.step5.group.label': 'Group user principals should be member of',
  'admin.kerberos.wizard.step6.task0.title': 'Stop Services',
  'admin.kerberos.wizard.step6.task1.title': 'Delete ATS',
  'admin.kerberos.wizard.step6.notice.inProgress': 'Please wait while services are being stopped.',
  'admin.kerberos.wizard.step6.notice.completed': 'Services have been successfully stopped.',
  'admin.kerberos.wizard.step7.notice.inProgress': 'Please wait while cluster is being kerberized.',
  'admin.kerberos.wizard.step7.notice.completed': 'Kerberos has successfully been enabled on the cluster.',
  'admin.kerberos.wizard.step8.notice.inProgress': 'Please wait while services are being started and tested.',
  'admin.kerberos.wizard.step8.notice.completed': 'Services have been successfully tested with kerberos environment.',
  'admin.kerberos.wizard.step8.notice.failed': 'Some services failed to start and execute tests successfully. Click Retry to attempt again or click Complete to dismiss the wizard and fix manually.',
  'admin.kerberos.wizard.step8.task0.title': 'Start and Test Services',

  'admin.kerberos.regenerate_keytabs.popup.body': 'Regenerating keytabs for <strong>all</strong> hosts in the cluster is a disruptive operation, and requires all components to be restarted. Optionally, keytabs can be regenerated <strong>only</strong> for missing hosts and components, and this operation requires selectively restarting those affected hosts and services.',
  'admin.kerberos.regenerate_keytabs.checkbox.label': ' Only regenerate keytabs for missing hosts and components',
  'admin.kerberos.regenerate_keytabs.popup.restart.body': 'After keytab regenerate is complete, services relying on them <strong>must</strong> be restarted. This can be done automatically, or manually.',
  'admin.kerberos.regenerate_keytabs.checkbox.restart.label': 'Automatically restart components after keytab regeneration',
  'admin.kerberos.service.alert.yarn': 'YARN log and local dir will be deleted and ResourceManager state will be formatted as part of Enabling/Disabling Kerberos.',

  'admin.kerberos.disable.step1.task0.title': 'Start ZooKeeper',
  'admin.kerberos.disable.step1.task1.title': 'Stop Required Services',
  'admin.kerberos.disable.step1.task2.title': 'Unkerberize Cluster',
  'admin.kerberos.disable.step1.task3.title': 'Remove Kerberos',
  'admin.kerberos.disable.step1.task4.title': 'Start Services',

  'admin.kerberos.disable.unkerberize.header': 'Unkerberize cluster',
  'admin.kerberos.disable.unkerberize.message': 'You cannot quit wizard while cluster is being unkerberized',
  'admin.kerberos.disable.inProgress': 'Please wait while cluster is being unkerberized',
  'admin.kerberos.disable.notice.completed': 'Services have been successfully tested without kerberos environment.',
  'admin.kerberos.wizard.step1.notice.inProgress': 'Please wait while cluster is being unkerberized',

  'admin.manageJournalNode.label': 'Manage JournalNodes',
  'admin.manageJournalNode.warning': 'Manage JournalNodes Wizard requires all NameNodes be started and have Active/Standby state defined',
  'admin.manageJournalNode.wizard.header': 'Manage JournalNodes Wizard',
  'admin.manageJournalNode.wizard.step1.header': 'Assign JournalNodes',
  'admin.manageJournalNode.wizard.step2.header': 'Review',
  'admin.manageJournalNode.wizard.step3.header': 'Save Namespace',
  'admin.manageJournalNode.wizard.step4.header': 'Add/Remove JournalNodes',
  'admin.manageJournalNode.wizard.step5.header': 'Copy JournalNode directories',
  'admin.manageJournalNode.wizard.step6.header': 'Start JournalNodes',
  'admin.manageJournalNode.wizard.step7.header': 'Start All Services',

  'admin.manageJournalNode.wizard.step1.body': 'Add, or Remove JournalNodes',
  'admin.manageJournalNode.wizard.step3.confirm.hosts.body': '<b>Confirm your host selections.</b>',
  'admin.manageJournalNode.wizard.step3.confirm.config.body': '<p><b>Review Configuration Changes.</b></p>' + 'As part of this process, configuration changes are required. Please review the changes below, and note that they are for <b>review only</b>.  Future steps in this wizard will update this configuration, and restart <b>all</b> services automatically.',

  'admin.manageJournalNode.wizard.step4.task0.title': 'Stop Standby NameNode',
  'admin.manageJournalNode.wizard.step4.task1.title': 'Stop Services',
  'admin.manageJournalNode.wizard.step4.task2.title': 'Add JournalNodes',
  'admin.manageJournalNode.wizard.step4.task3.title': 'Delete JournalNodes',
  'admin.manageJournalNode.wizard.step4.task4.title': 'Reconfigure HDFS',
  'admin.manageJournalNode.wizard.step6.task0.title': 'Start JournalNodes',
  'admin.manageJournalNode.wizard.step7.task0.title': 'Start All Services',

  'admin.manageJournalNode.wizard.step5.bodyHeader': 'Manual Steps Required: Copy JournalNode directories',

  'admin.manageJournalNode.step4.save.configuration.note': 'This configuration is created by Manage JournalNode Wizard',

  'admin.manageJournalNode.wizard.progressPage.notice.inProgress': 'Please wait while JournalNodes are being deployed',
  'admin.manageJournalNode.wizard.step4.notice.inProgress': 'Please wait while JournalNodes are being deployed',
  'admin.manageJournalNode.wizard.step6.notice.inProgress': 'Please wait while JournalNodes are started',
  'admin.manageJournalNode.wizard.step7.notice.inProgress': 'Please wait while services are started',
  'admin.manageJournalNode.wizard.step7.notice.completed': 'Completed update to JournalNodes.',

  'admin.manageJournalNode.wizard.step3.error.multipleNameSpaces.nameNodes': 'Some NameNodes are in the process of being stopped. Please make sure that NameNodes are running to create checkpoints successfully.',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.safeModeText': 'Put the NameNode in Safe Mode (read-only mode)',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.safeModeText': 'Put the NameNodes in Safe Mode (read-only mode)',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.safeModeCommand': 'sudo su {0} -l -c \'hdfs dfsadmin -safemode enter\'',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.safeModeCommand': 'sudo su {0} -l -c \'hdfs dfsadmin -fs hdfs://{1} -safemode enter\'',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.checkPointText': 'Once in Safe Mode, create a Checkpoint',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.checkPointText': 'Once in Safe Mode, create Checkpoints',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.checkPointCommand': 'sudo su {0} -l -c \'hdfs dfsadmin -saveNamespace\'',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.checkPointCommand': 'sudo su {0} -l -c \'hdfs dfsadmin -fs hdfs://{1} -saveNamespace\'',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.proceed': 'You will be able to proceed once Ambari detects that the NameNode is in Safe Mode and the Checkpoint has been created successfully.',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.proceed': 'You will be able to proceed once Ambari detects that the NameNodes are in Safe Mode and the Checkpoints have been created successfully.',
  'admin.manageJournalNode.wizard.step3.body.singleNameSpace.recentCheckPoint': 'If the <b>Next</b> button is enabled before you run the <b>"Step 3: Save Namespace"</b> command, it means there is a recent Checkpoint already and you may proceed without running the <b>"Step 3: Save Namespace"</b> command.',
  'admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.recentCheckPoint': 'If the <b>Next</b> button is enabled before you run the <b>"Step 3: Save Namespace"</b> commands, it means there are recent Checkpoints already and you may proceed without running the <b>"Step 3: Save Namespace"</b> commands.',
  'admin.manageJournalNode.wizard.step3.body': '<ol>' + '<li>Login to the NameNode host <b>{0}</b>.</li>' + '<li>{1}:' + '<div class="code-snippet">{2}</div></li>' + '<li>{3}:' + '<div class="code-snippet">{4}</div></li>' + '<li>{5}</li>' + '<div class="alert alert-warn">{6}</div>' + '</ol>',
  'admin.manageJournalNode.wizard.step3.checkPointsNotCreated': 'Checkpoints not created yet',
  'admin.manageJournalNode.wizard.step3.checkPointsCreated': 'Checkpoints created',

  'admin.manageJournalNode.wizard.step5.body': '<ol>' + '<li>Login to the JournalNode host <b>{0}</b>.</li>' + '<li>Create a tarball of the Journal directories: {1}.</li>' + '<li>Copy the tarball on the new JournalNodes and untar at the respective locations as in Step 2.</li>' + '</ol>',

  'admin.highAvailability': ' High Availability',
  'admin.highAvailability.button.enable': 'Enable NameNode HA',
  'admin.highAvailability.button.disable': 'Disable NameNode HA',
  'admin.rm_highAvailability.button.enable': 'Enable ResourceManager HA',
  'admin.rm_highAvailability.button.disable': 'Disable ResourceManager HA',
  'admin.ra_highAvailability.button.enable': 'Enable Ranger Admin HA',
  'admin.highAvailability.disabled': 'NameNode HA is disabled',
  'admin.highAvailability.enabled': 'NameNode HA is enabled',
  'admin.rm_highAvailability.disabled': 'ResourceManager HA is disabled',
  'admin.rm_highAvailability.enabled': 'ResourceManager HA is enabled',
  'admin.highAvailability.confirmRollbackHeader': 'Confirm Rollback',
  'admin.highAvailability.confirmRollbackBody': 'This will rollback all operations that were done in HA wizard',
  'admin.highAvailability.confirmManualRollbackBody': 'You are in the process of enabling NameNode HA. If you exit now, you must follow manual instructions to revert back to the non-HA setup as documented in the Ambari User Guide\'s <i>Rolling Back NameNode HA</i> section.  Are you sure you want to exit the wizard?',
  'admin.highAvailability.error.hostsNum': 'You must have at least 3 hosts in your cluster to enable NameNode HA.',
  'admin.highAvailability.error.namenodeStarted': 'NameNode must be running before you enable NameNode HA.',
  'admin.highAvailability.error.maintenanceMode': 'In order to enable NameNode HA, all services, and hosts with master components need to be out of Maintenance Mode.',
  'admin.highAvailability.error.zooKeeperNum': 'You must have at least 3 ZooKeeper Servers in your cluster to enable NameNode HA.',
  'admin.rm_highAvailability.error.hostsNum': 'You must have at least 3 hosts in your cluster to enable ResourceManager HA.',
  'admin.rm_highAvailability.error.zooKeeperNum': 'You must have at least 3 ZooKeeper Servers in your cluster to enable ResourceManager HA.',
  'admin.rm_highAvailability.error.resourceManagerStarted': 'ResourceManager must be running before you enable ResourceManager HA.',
  'admin.rm_highAvailability.closePopup': 'Enable ResourceManager HA Wizard is in progress. You must allow the wizard to complete for Ambari to be in usable state. If you choose to quit, you must follow manual instructions to complete or revert enabling ResourceManager HA as documented in the Ambari User Guide. Are you sure you want to exit the wizard?',

  'admin.highAvailability.wizard.header': 'Enable NameNode HA Wizard',
  'admin.highAvailability.wizard.progressPage.notice.inProgress': 'Please wait while NameNode HA is being deployed.',
  'admin.highAvailability.wizard.progressPage.header': 'Deploy',
  'admin.highAvailability.wizard.step1.header': 'Get Started',
  'admin.highAvailability.wizard.step1.nameserviceid.tooltip.title': 'Nameservice ID',
  'admin.highAvailability.wizard.step1.nameserviceid.tooltip.content': 'This will be the ID for the NameNode HA cluster. For example, if you set Nameservice ID to <b>mycluster</b>, the logical URI for HDFS will be <b>hdfs://mycluster</b>.',
  'admin.highAvailability.wizard.step1.nameserviceid': 'Nameservice ID',
  'admin.highAvailability.wizard.step1.nameserviceid.error': 'Must consist of letters, numbers, and hyphens. Cannot begin or end with a hyphen.',
  'admin.highAvailability.wizard.step2.header': 'Select Hosts',
  'admin.highAvailability.wizard.step3.header': 'Review',
  'admin.highAvailability.wizard.step4.header': 'Create Checkpoint',
  'admin.highAvailability.wizard.step4.error.nameNode': 'NameNode is in the process of being stopped. Please make sure that NameNode is running to create checkpoint successfully.',
  'admin.highAvailability.wizard.step5.header': 'Configure Components',
  'admin.highAvailability.wizard.step6.header': 'Initialize JournalNodes',
  'admin.highAvailability.wizard.step7.header': 'Start Components',
  'admin.highAvailability.wizard.step8.header': 'Initialize Metadata',
  'admin.highAvailability.wizard.step9.header': 'Finalize HA Setup',
  'admin.highAvailability.wizard.step4.bodyHeader': 'Manual Steps Required: Create Checkpoint on NameNode',
  'admin.highAvailability.wizard.step6.bodyHeader': 'Manual Steps Required: Initialize JournalNodes',
  'admin.highAvailability.wizard.step8.bodyHeader': 'Manual Steps Required: Initialize NameNode HA Metadata',

  'admin.rollbackHighAvailability.wizard.step1.header': 'Select Hosts page',
  'admin.rollbackHighAvailability.wizard.step2.header': 'Create Checkpoint page',
  'admin.rollbackHighAvailability.wizard.step3.header': 'Progress page',

  'admin.highAvailability.wizard.step5.notice.inProgress': 'Please wait while the wizard configures the components.',
  'admin.highAvailability.wizard.step7.notice.inProgress': 'Please wait while the wizard starts the components.',
  'admin.highAvailability.wizard.step9.notice.inProgress': 'Please wait while the wizard finalizes the HA setup.',
  'admin.highAvailability.wizard.rollback.notice.inProgres': 'Reverting Back to Non-HA Setup',

  'admin.highAvailability.wizard.step5.header.title': 'Configure Components',
  'admin.highAvailability.wizard.step7.header.title': 'Start Components',
  'admin.highAvailability.wizard.step9.header.title': 'Finalize HA Setup',
  'admin.highAvailability.wizard.rollback.header.title': 'Reverting Back to Non-HA Setup.',

  'admin.highAvailability.wizard.step5.task0.title': 'Stop All Services',
  'admin.highAvailability.wizard.step5.task1.title': 'Install Additional NameNode',
  'admin.highAvailability.wizard.step5.task2.title': 'Install JournalNodes',
  'admin.highAvailability.wizard.step5.task3.title': 'Reconfigure HDFS',
  'admin.highAvailability.wizard.step5.task4.title': 'Start JournalNodes',
  'admin.highAvailability.wizard.step5.task5.title': 'Disable Secondary NameNode',

  'admin.highAvailability.wizard.step7.task0.title': 'Start ZooKeeper Servers',
  'admin.highAvailability.wizard.step7.task1.title': 'Start Ambari Infra',
  'admin.highAvailability.wizard.step7.task2.title': 'Start Mysql Server',
  'admin.highAvailability.wizard.step7.task3.title': 'Start Ranger',
  'admin.highAvailability.wizard.step7.task4.title': 'Start NameNode',

  'admin.highAvailability.wizard.step9.task0.title': 'Start Additional NameNode',
  'admin.highAvailability.wizard.step9.task1.title': 'Install Failover Controllers',
  'admin.highAvailability.wizard.step9.task2.title': 'Start Failover Controllers',
  'admin.highAvailability.wizard.step9.task3.title': 'Install PXF',
  'admin.highAvailability.wizard.step9.task4.title': 'Reconfigure Ranger',
  'admin.highAvailability.wizard.step9.task5.title': 'Reconfigure HBase',
  'admin.highAvailability.wizard.step9.task6.title': 'Reconfigure AMS',
  'admin.highAvailability.wizard.step9.task7.title': 'Reconfigure Accumulo',
  'admin.highAvailability.wizard.step9.task8.title': 'Reconfigure HAWQ',
  'admin.highAvailability.wizard.step9.task9.title': 'Delete Secondary NameNode',
  'admin.highAvailability.wizard.step9.task10.title': 'Stop HDFS',
  'admin.highAvailability.wizard.step9.task11.title': 'Start All Services',
  'admin.highAvailability.wizard.step9.notice.completed': 'NameNode HA has been enabled successfully.',

  'admin.highAvailability.wizard.step3.curNameNode': '<b>Current NameNode:</b> ',
  'admin.highAvailability.wizard.step3.addNameNode': '<b>Additional NameNode:</b> ',
  'admin.highAvailability.wizard.step3.secNameNode': '<b>Secondary NameNode:</b> ',
  'admin.highAvailability.wizard.step3.journalNode': '<b>JournalNode:</b> ',
  'admin.highAvailability.wizard.step3.toBeInstalled': 'TO BE INSTALLED',
  'admin.highAvailability.wizard.step3.toBeDeleted': 'TO BE DELETED',
  'admin.highAvailability.wizard.step4.ckNotCreated': 'Checkpoint not created yet',
  'admin.highAvailability.wizard.step4.ckCreated': 'Checkpoint created',
  'admin.highAvailability.step4.save.configuration.note': 'This configuration is created by Enable {0} HA wizard',
  'admin.highAvailability.wizard.step6.jsNoInit': 'JournalNodes not initialized yet',
  'admin.highAvailability.wizard.step6.jsInit': 'JournalNodes initialized',
  'admin.highAvailability.wizard.step6.jnStopped': 'All JournalNodes should be started before initializing',
  'admin.highAvailability.wizard.step8.metaNoInit': 'Metadata not initialized yet',
  'admin.highAvailability.wizard.step8.confirmPopup.body': 'Please confirm that you have run the manual steps before continuing.',
  'admin.highAvailability.wizard.step9.hawq.confirmPopup.header': 'Additional Steps Required for HAWQ',
  'admin.highAvailability.wizard.step9.hawq.confirmPopup.body': 'You must refer to the HAWQ documentation under section ' + '"HAWQ Filespaces and High Availability Enabled HDFS" and perform the required manual steps to update the HAWQ filespace ' + 'location. HAWQ will not use HDFS nameservice unless the steps have been completed successfully.',

  'admin.highAvailability.rollback.header': 'Disable NameNode HA Wizard',
  'admin.highAvailability.rollback.task0.title': 'Stop All Services',
  'admin.highAvailability.rollback.task1.title': 'Restore HBase Configurations',
  'admin.highAvailability.rollback.task2.title': 'Restore Accumulo Configurations',
  'admin.highAvailability.rollback.task3.title': 'Restore HAWQ Configurations',
  'admin.highAvailability.rollback.task4.title': 'Stop Failover Controllers',
  'admin.highAvailability.rollback.task5.title': 'Delete Failover Controllers',
  'admin.highAvailability.rollback.task6.title': 'Delete PXF',
  'admin.highAvailability.rollback.task7.title': 'Stop Additional NameNode',
  'admin.highAvailability.rollback.task8.title': 'Stop NameNode',
  'admin.highAvailability.rollback.task9.title': 'Restore HDFS Configurations',
  'admin.highAvailability.rollback.task10.title': 'Enable Secondary NameNode',
  'admin.highAvailability.rollback.task11.title': 'Stop JournalNodes',
  'admin.highAvailability.rollback.task12.title': 'Delete JournalNodes',
  'admin.highAvailability.rollback.task13.title': 'Delete Additional NameNode',
  'admin.highAvailability.rollback.task14.title': 'Start All Services',
  'admin.highAvailability.rollback.notice.inProgress': 'Please wait while the wizard reverts back to the non-HA setup.',

  'admin.highAvailability.rollback.step2.body': '<ol>' + '<li>Login to the NameNode host <b>{1}</b>.</li>' + '<li>Put the NameNode in Safe Mode (read-only mode):' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -safemode enter\'</div></li>' + '<li>Once in Safe Mode, create a Checkpoint:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -saveNamespace\'</div></li>' + '</ol>',

  'admin.highAvailability.wizard.step8.body': '<ol>' + '<li>Login to the NameNode host <b>{1}</b>.</li>' + '<li>Initialize the metadata for NameNode automatic failover by running:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs zkfc -formatZK\'</div></li>' + '<li>Login to the Additional NameNode host <b>{2}</b>.<br>' + '<div class="alert alert-warning"><strong>Important!</strong> Be sure to login to the Additional NameNode host.<br>This is a different host from the Steps 1 and 2 above.</div>' + '</li>' + '<li>Initialize the metadata for the Additional NameNode by running:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs namenode -bootstrapStandby\'</div></li>' + '</ol>',
  'admin.highAvailability.wizard.step8.proceed': 'Please proceed once you have completed the steps above.',
  'admin.highAvailability.wizard.step6.body': '<ol>' + '<li>Login to the NameNode host <b>{1}</b>.</li>' + '<li>Initialize the JournalNodes by running:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs namenode -initializeSharedEdits\'</div></li>' + '<li>You will be able to proceed once Ambari detects that the JournalNodes have been initialized successfully.</li>' + '</ol>',
  'admin.highAvailability.wizard.step4.body': '<ol>' + '<li>Login to the NameNode host <b>{1}</b>.</li>' + '<li>Put the NameNode in Safe Mode (read-only mode):' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -safemode enter\'</div></li>' + '<li>Once in Safe Mode, create a Checkpoint:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -saveNamespace\'</div></li>' + '<li>You will be able to proceed once Ambari detects that the NameNode is in Safe Mode and the Checkpoint has been created successfully.</li>' + '<div class="alert alert-warning">If the <b>Next</b> button is enabled before you run the <b>"Step 4: Create a Checkpoint"</b> command, it means there is a recent Checkpoint already and you may proceed without running the <b>"Step 4: Create a Checkpoint"</b> command.</div>' + '</ol>',
  'admin.highAvailability.wizard.step3.confirm.host.body': '<b>Confirm your host selections.</b>',
  'admin.highAvailability.wizard.step3.confirm.config.body': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>' + 'The following lists the configuration changes that will be made by the Wizard to enable NameNode HA. This information is for <b> review only </b> and is not editable except for the  <b>dfs.journalnode.edits.dir</b> property' + '</div>',
  'admin.highAvailability.wizard.step2.body': 'Select a host that will be running the additional NameNode.<br/> In addition,' + ' select the hosts to run JournalNodes, which store NameNode edit logs in a fault tolerant manner.',
  'admin.highAvailability.wizard.step1.body': 'This wizard will walk you through enabling NameNode HA on your cluster.' + 'Once enabled, you will be running a Standby NameNode in addition to your Active NameNode.' + 'This allows for an Active-Standby NameNode configuration that automatically performs failover.' + 'The process to enable HA involves a combination of <b>automated steps</b> (that will be handled by the wizard) and ' + '<b>manual steps</b> (that you must perform in sequence as instructed by the wizard).' + '<b>You should plan a cluster maintenance window and prepare for cluster downtime when enabling NameNode HA.</b>',
  'admin.highAvailability.wizard.step1.alert': 'If you have HBase running, please exit this wizard and stop HBase first.',
  'admin.highAvailability.wizard.step1.hawq.alert': '<br/><br/>You will need to perform additional manual ' + 'steps to update the HAWQ filespace location, once this wizard completes. Refer to the HAWQ documentation under section ' + '"HAWQ Filespaces and High Availability Enabled HDFS" for more information.',

  'admin.rm_highAvailability.wizard.header': 'Enable ResourceManager HA Wizard',
  'admin.rm_highAvailability.wizard.step1.header': 'Get Started',
  'admin.rm_highAvailability.wizard.step1.body': 'This wizard will walk you through enabling ResourceManager HA on your cluster.<br/>' + 'Once enabled, you will be running a Standby ResourceManager in addition to your Active ResourceManager.<br/>' + 'This allows for an Active-Standby ResourceManager configuration that automatically performs failover.<br/><br/>' + '<b>You should plan a cluster maintenance window and prepare for cluster downtime when enabling ResourceManager HA.</b>',
  'admin.rm_highAvailability.wizard.step2.header': 'Select Host',
  'admin.rm_highAvailability.wizard.step2.body': 'Select a host that will be running the additional ResourceManager',
  'admin.rm_highAvailability.wizard.step3.header': 'Review',
  'admin.rm_highAvailability.wizard.step3.confirm.host.body': '<b>Confirm your host selections.</b>',
  'admin.rm_highAvailability.wizard.step3.confirm.config.body': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>' + 'The following lists the configuration changes that will be made by the Wizard to enable ResourceManager HA. This information is for <b> review only </b> and is not editable.' + '</div>',
  'admin.rm_highAvailability.wizard.step3.currentRM': 'Current ResourceManager',
  'admin.rm_highAvailability.wizard.step3.additionalRM': 'Additional ResourceManager',
  'admin.rm_highAvailability.wizard.step4.header': 'Configure Components',
  'admin.rm_highAvailability.wizard.step4.task0.title': 'Stop Required Services',
  'admin.rm_highAvailability.wizard.step4.task1.title': 'Install Additional ResourceManager',
  'admin.rm_highAvailability.wizard.step4.task2.title': 'Reconfigure YARN',
  'admin.rm_highAvailability.wizard.step4.task3.title': 'Reconfigure HAWQ',
  'admin.rm_highAvailability.wizard.step4.task4.title': 'Reconfigure HDFS',
  'admin.rm_highAvailability.wizard.step4.task5.title': 'Start All Services',
  'admin.rm_highAvailability.wizard.step4.notice.inProgress': 'Please wait while ResourceManager HA is being deployed.',
  'admin.rm_highAvailability.wizard.step4.notice.completed': 'ResourceManager HA has been enabled successfully.',

  'admin.ra_highAvailability.wizard.header': 'Enable Ranger Admin HA Wizard',
  'admin.ra_highAvailability.wizard.step1.header': 'Get Started',
  'admin.ra_highAvailability.wizard.step1.body': 'This wizard will walk you through enabling Ranger Admin HA on your cluster.<br/>' + 'Once enabled, you will be running a Standby Ranger Admin in addition to your Active Ranger Admin.<br/>' + 'This allows for an Active-Standby Ranger Admin configuration that automatically performs failover.<br/><br/>' + '<b>You should plan a cluster maintenance window and prepare for cluster downtime when enabling Ranger Admin HA.</b><br/><br/>' + 'Please setup the load balancer and provide the URL to be used. Make sure that the load balancer is setup properly before proceeding.' + '<br/><br/><div class="alert">Be sure that Ranger Admin and load balancer are located on separate hosts.</div>',
  'admin.ra_highAvailability.wizard.step1.load_balancer_url': 'URL to load balancer',
  'admin.ra_highAvailability.wizard.step1.invalid_url': 'Must be valid URL',
  'admin.ra_highAvailability.wizard.step2.header': 'Select Hosts',
  'admin.ra_highAvailability.wizard.step2.body': 'Select a host or hosts that will be running the additional Ranger Admin components',
  'admin.ra_highAvailability.wizard.step2.warning': 'Be sure that load balancer located separately from Ranger Admin components.',
  'admin.ra_highAvailability.wizard.step3.header': 'Review',
  'admin.ra_highAvailability.wizard.step3.alert_message': '<b>Confirm your host selections.</b>',
  'admin.ra_highAvailability.wizard.step3.currentRA': 'Current Ranger Admin',
  'admin.ra_highAvailability.wizard.step3.additionalRA': 'Additional Ranger Admin',
  'admin.rm_highAvailability.wizard.step3.configs_changes': '<p><b>Review Configuration Changes.</b></p>' + 'The following lists the configuration changes that will be made by the Wizard to enable Ranger Admin HA. ' + 'This information is for <b> review only </b> and is not editable.',
  'admin.ra_highAvailability.wizard.step4.header': 'Install, Start and Test',
  'admin.ra_highAvailability.wizard.step4.task0.title': 'Stop All Services',
  'admin.ra_highAvailability.wizard.step4.task1.title': 'Install Additional Ranger Admin',
  'admin.ra_highAvailability.wizard.step4.task2.title': 'Reconfigure Services',
  'admin.ra_highAvailability.wizard.step4.task3.title': 'Start All Services',
  'admin.ra_highAvailability.wizard.step4.notice.inProgress': 'Please wait while Ranger Admin HA is being deployed.',
  'admin.ra_highAvailability.wizard.step4.notice.completed': 'Ranger Admin HA has been enabled successfully.',
  'admin.ra_highAvailability.closePopup': 'Enable Ranger Admin HA Wizard is in progress. You must allow the wizard to complete for Ambari to be in usable state. ' + 'If you choose to quit, you must follow manual instructions to complete or revert enabling Ranger Admin HA as documented in the Ambari User Guide. Are you sure you want to exit the wizard?',

  'admin.nameNodeFederation.button.enable': 'Add New HDFS Namespace',
  'admin.nameNodeFederation.wizard.required.zookeepers': 'All ZooKeeper Servers should be up',
  'admin.nameNodeFederation.wizard.required.journalnodes': 'All JournalNodes should be up',
  'admin.nameNodeFederation.wizard.header': 'Add New HDFS Namespace',
  'admin.nameNodeFederation.closePopup': 'Are you sure you want to quit?',
  'admin.nameNodeFederation.closePopup2': 'Add New HDFS Namespace Wizard is in progress. You must allow the wizard to complete for Ambari to be in usable state. If you choose to quit, you must follow manual instructions to complete or revert Add New HDFS Namespace as documented in the Ambari User Guide. Are you sure you want to exit the wizard?',
  'admin.nameNodeFederation.wizard.step1.header': 'Get Started',
  'admin.nameNodeFederation.wizard.step1.body': 'This wizard will walk you through the process of setting up a new Highly Available NameNode pair that will be used to create a new HDFS namespace, allowing you to use Ambari to manage multiple HDFS namespaces and take advantage of HDFS Federation.',
  'admin.nameNodeFederation.wizard.step1.alert': 'You should plan a cluster maintenance window and prepare for cluster downtime when adding a new HDFS Namespace as this Wizard will restart all services.',
  'admin.nameNodeFederation.wizard.step1.nameserviceid': 'New Nameservice ID',
  'admin.nameNodeFederation.wizard.step1.nameserviceid.existing': 'Existing Nameservice ID',
  'admin.nameNodeFederation.wizard.step1.nameserviceid.error': 'Must be unique and consist of letters, numbers, and hyphens. Cannot begin or end with a hyphen.',
  'admin.nameNodeFederation.wizard.step2.header': 'Select Hosts',
  'admin.nameNodeFederation.wizard.step2.body': 'Select hosts running the NameNodes for {0}',
  'admin.nameNodeFederation.wizard.step3.header': 'Review',
  'admin.nameNodeFederation.wizard.step3.confirm.config.body': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>' + 'The following lists the configuration changes that will be made by the Wizard to enable NameNode Federation. This information is for <b> review only </b> and is not editable except for the  <b>dfs.journalnode.edits.dir</b> properties' + '</div>',
  'admin.nameNodeFederation.wizard.step4.header': 'Configure Components',
  'admin.nameNodeFederation.wizard,step4.save.configuration.note': 'This configuration is created by Enable NameNode Federation wizard',
  'admin.nameNodeFederation.wizard.step4.notice.inProgress': 'Please wait while your new HDFS Namespace is being deployed.',
  'admin.nameNodeFederation.wizard.step4.notice.completed': 'Add New HDFS Namespace Wizard has been completed successfully.',
  'admin.nameNodeFederation.wizard.step4.task0.title': 'Stop Required Services',
  'admin.nameNodeFederation.wizard.step4.task1.title': 'Reconfigure Services',
  'admin.nameNodeFederation.wizard.step4.task2.title': 'Install Additional NameNodes',
  'admin.nameNodeFederation.wizard.step4.task3.title': 'Install Additional ZKFCs',
  'admin.nameNodeFederation.wizard.step4.task4.title': 'Start JournalNodes',
  'admin.nameNodeFederation.wizard.step4.task5.title': 'Start Infra Solr',
  'admin.nameNodeFederation.wizard.step4.task6.title': 'Start Ranger Admin',
  'admin.nameNodeFederation.wizard.step4.task7.title': 'Start Ranger Usersync',
  'admin.nameNodeFederation.wizard.step4.task8.title': 'Start NameNodes',
  'admin.nameNodeFederation.wizard.step4.task9.title': 'Start ZKFCs',
  'admin.nameNodeFederation.wizard.step4.task10.title': 'Format NameNode',
  'admin.nameNodeFederation.wizard.step4.task11.title': 'Format ZKFC',
  'admin.nameNodeFederation.wizard.step4.task12.title': 'Start ZKFC',
  'admin.nameNodeFederation.wizard.step4.task13.title': 'Start NameNode',
  'admin.nameNodeFederation.wizard.step4.task14.title': 'Bootstrap NameNode',
  'admin.nameNodeFederation.wizard.step4.task15.title': 'Start ZKFC',
  'admin.nameNodeFederation.wizard.step4.task16.title': 'Start NameNode',
  'admin.nameNodeFederation.wizard.step4.task17.title': 'Restart Required Services',

  'admin.security.title': 'Kerberos security has not been enabled',
  'admin.security.enabled': 'Kerberos security is enabled',
  'admin.security.disabled': 'Kerberos security is disabled',
  'admin.security.button.enable': 'Enable Security',
  'admin.security.button.disable': 'Disable Security',
  'admin.security.enable.popup.body': 'We will walk you through add security wizard',
  'admin.security.enable.popup.header': 'Add security',
  'admin.security.disable.popup.header': 'Remove security',
  'admin.security.disable.popup.body': 'You are about to disable Kerberos on the cluster. This will stop all the services in your cluster and remove the Kerberos configurations. <strong>Are you sure you wish to proceed with disabling Kerberos?</strong>',
  'admin.security.step1.header': 'Get Started',
  'admin.security.step2.header': 'Configure Services',
  'admin.security.step3.header': 'Create Principals and Keytabs',
  'admin.security.step4.header': 'Save and Apply Configuration',
  'admin.security.step1.body.header': 'Important: Before configuring Ambari to manage your Kerberos-enabled cluster, ' + 'you must perform the following manual steps on your cluster. Be sure to record the location of the keytab files ' + 'for each host and the principals for each Hadoop service. This information is required in order to use the wizard.',
  'admin.security.step1.body.instruction1': 'Install, configure and start your Kerberos KDC',
  'admin.security.step1.body.instruction2': 'Install and configure the Kerberos client on every host in the cluster',
  'admin.security.step1.body.instruction3': 'Create Kerberos principals for Hadoop services and hosts',
  'admin.security.step1.body.instruction4': 'Generate keytabs for each principal and place on the appropriate hosts',
  'admin.security.step1.body.instruction5': '<b>Application Timeline Server</b> component of YARN service will be <span class="text-danger"><b>deleted</b></span> as part of enabling security in this HDP stack version',
  'admin.security.step2.body.header': 'Configure Kerberos security properties',
  'admin.security.step3.notice': 'You need to create Kerberos principals and keytabs before proceeding.<br />' + 'Download the CSV file and use it to create a script to generate the principals and keytabs on specified hosts. ' + 'Once the principals and keytabs have been created, click on <i>Apply</i> to continue. If you need to make configuration changes, click <i>Back</i>.',
  'admin.security.step3.table.principal': 'Principal',
  'admin.security.step3.table.keytab': 'Keytab',
  'admin.security.step3.downloadCSV': 'Download CSV',
  'admin.security.step4.body.header': 'Applying kerberos security to the cluster',
  'admin.security.step4.body.success.header': 'Kerberos-based security has been enabled on your cluster. Please wait while services are started in secure mode.',
  'admin.security.step4.body.failure.header': 'Failed to enable Kerberos-based security on your cluster. Your cluster will keep running in non-secure mode.',
  'admin.security.step4.save.configuration.note': 'This configuration is created by Enable/Disable security wizard',
  'admin.security.apply.configuration.error': 'Failed to apply secure configurations to the cluster. Please navigate to "Configure Services" step and make sure all services are configured with appropriate values.',
  'admin.security.disable.body.header': 'Disabling kerberos security on the cluster',
  'admin.security.disable.body.success.header': 'Kerberos-based security has been disabled on your cluster. Please wait while services are started in non-secure mode.',
  'admin.security.disable.body.failure.header': 'Failed to disable Kerberos-based security on your cluster. Your cluster will keep running in secure mode.',
  'admin.security.disable.onClose': 'You are in the process of disabling security on your cluster. ' + 'Are you sure you want to quit?',
  'admin.removeSecurity.header': 'Disable Security',
  'admin.security.applying.config.header': 'Applying Configurations',
  'admin.security.applying.config.body': 'You cannot quit wizard while configurations are being applied',
  'admin.security.status.error': 'Error in retrieving cluster security status from Ambari server',
  'admin.users.ldapAuthUsed': 'LDAP Authentication is being used to authenticate users',
  'admin.users.delete.yourself.message': 'You cannot delete yourself',
  'admin.users.delete.yourself.header': 'Deleting warning',

  'admin.users.delete.header': 'Delete {0}',

  'admin.users.addButton': 'Add Local User',
  'admin.users.editButton': 'Edit Local User',
  'admin.users.delete': 'delete',
  'admin.users.edit': 'edit',
  'admin.users.privileges': 'Admin',
  'admin.users.type': 'Type',
  'admin.users.action': 'Action',
  'admin.users.passwordRetype': 'Retype Password',
  'admin.users.username': 'Username',
  'admin.users.createSuccess': 'User successfully created.',
  'admin.users.createError': 'Error occurred while user creating.',
  'admin.users.createError.passwordValidation': 'Passwords are different',
  'admin.users.createError.userNameExists': 'User with the same name is already exists',
  'admin.users.editError.requiredField': 'This is required',

  'admin.access.showJobs': 'Enable Jobs tab for non-admin users',

  'admin.confirmUninstall': 'Confirm Uninstall',
  'admin.cluster.stackVersion': 'Cluster Stack Version',
  'admin.cluster.upgradeAvailable': 'Upgrade available',
  'admin.cluster.upgradeUnavailable': 'Upgrade unavailable',
  'admin.cluster.repositories.repositories': 'Repositories',
  'admin.cluster.repositories.os': 'OS',
  'admin.cluster.repositories.baseUrl': 'Base URL',
  'admin.cluster.repositories.popup.header.success': 'Repo Base URLs Saved',
  'admin.cluster.repositories.popup.header.fail': 'Base URL Validation Failed',
  'admin.cluster.repositories.popup.body.success': 'The Repo Base URL has been saved successfully.',
  'admin.cluster.repositories.popup.body.fail': 'The Base URL failed validation and has not been saved.',
  'admin.cluster.repositories.invalidURLAttention': '<b>Attention:</b> Please make sure all repository URLs are valid before saving.',
  'admin.cluster.repositories.emptyURLattention': '<b>Attention:</b> Repository URLs are REQUIRED before you can save.',
  'admin.cluster.repositories.skipValidation.tooltip': '<b>Warning:</b> This is for advanced users only. Use this option if you want to skip validation for Repository Base URLs.',

  'admin.misc.header': 'Service Users and Groups',
  'admin.misc.nothingToShow': 'No user accounts to display',

  'admin.serviceAutoStart.title': "Service Auto Start",
  'admin.serviceAutoStart.header.text': "Ambari services can be configured to start automatically on system boot. Each service can be configured to start all components, masters and workers, or selectively.",
  'admin.serviceAutoStart.general.switcher': "Auto Start Settings",
  'admin.serviceAutoStart.tooltip.text': "{0} components enabled",
  'admin.serviceAutoStart.column.autoStart': "Auto Start",

  'admin.stackVersions.filter.notInstalled': "Not Installed ({0})",
  'admin.stackVersions.filter.all': "All ({0})",
  'admin.stackVersions.filter.upgradeReady': "Upgrade Ready ({0})",
  'admin.stackVersions.filter.installed': "Installed ({0})",
  'admin.stackVersions.filter.current': "Current ({0})",
  'admin.stackVersions.filter.upgrading': "Upgrade/Downgrade In Process ({0})",
  'admin.stackVersions.filter.upgraded': "Ready to Finalize ({0})",
  'admin.stackVersions.upgrade.start.fail.title': 'Upgrade could not be started',
  'admin.stackVersions.upgrade.installPackage.fail.title': 'Packages could not be installed',
  'admin.stackVersions.upgrade.installPackage.fail.timeout': 'Request timed out.',
  'admin.stackVersions.upgrade.installPackage.fail.noAvailableServices': "Version {0} does not contain services that are installed.",

  'admin.stackVersions.upgrade.patch.revert.confirmation': 'Are you sure you want to revert the following services:',
  'admin.stackVersions.editRepositories.info': 'Provide Base URLs for the Operating Systems you are configuring.',
  'admin.stackVersions.editRepositories.validation.warning': 'Some of the repositories failed validation. Make changes to the base url or skip validation if you are sure that urls are correct',
  'admin.stackVersions.version.install.confirm': 'You are about to install packages for version <strong>{0}</strong> on all hosts.',
  'admin.stackVersions.version.install.patch.confirm': 'You are about to install packages for version <strong>{0}</strong> on hosts which contain the following services:',
  'admin.stackVersions.version.linkTooltip': 'Click to Edit Repositories',
  'admin.stackVersions.version.hostsTooltip': 'Click to List Hosts',
  'admin.stackVersions.version.emptyHostsTooltip': 'No Hosts to List',
  'admin.stackVersions.version.notInstalled': "Not Installed",
  'admin.stackVersions.version.hostsInfoTooltip': "There are {0} hosts that do not need packages installed:<li>{1} Maintenance Mode</li><li>{2} Not Required</li>",
  'admin.stackVersions.manageVersions': "Manage Versions",
  'admin.stackVersions.manageVersions.popup.body': 'You are about to leave the <b>Cluster Management</b> interface' + ' and go to the <b>Ambari Administration</b> interface. You can return to cluster management by using the' + ' “Go to Dashboard” link in the Ambari Administration > Clusters section.',
  'admin.stackVersions.version.installNow': "Install Packages",
  'admin.stackVersions.version.reinstall': "Reinstall Packages",
  'admin.stackVersions.version.installError': "Installation Error!",
  'admin.stackVersions.version.performUpgrade': "Perform Upgrade",
  'admin.stackVersions.version.preUpgradeCheck': 'Pre-Upgrade Check',
  'admin.stackVersions.version.upgrade.pause': "Upgrade: Action Required",
  'admin.stackVersions.version.upgrade.notFinalized.warning': "The upgrade has not been finalized yet. After the cluster is verified to be functional, do not forget to finalize the upgrade as soon as possible (within a couple of days is highly recommended) as running the cluster in unfinalized state causes extra resource requirements on HDFS.",
  'admin.stackVersions.version.upgrade.running': "Upgrade: In Process",
  'admin.stackVersions.version.upgrade.running.nonWizard': "Upgrade: In Process | Initiated by {0}",
  'admin.stackVersions.version.upgrade.aborted': "Upgrade: Aborted",
  'admin.stackVersions.version.upgrade.suspended': "Upgrade: Paused",
  'admin.stackVersions.version.downgrade.pause': "Downgrade: Action Required",
  'admin.stackVersions.version.downgrade.running': "Downgrade: In Process",
  'admin.stackVersions.version.downgrade.running.nonWizard': "Downgrade: In Process | Initiated by {0}",
  'admin.stackVersions.version.downgrade.aborted': "Downgrade: Aborted",
  'admin.stackVersions.version.downgrade.suspended': "Downgrade: Paused",
  'admin.stackUpgrade.state.paused.fail.header': "Pause Upgrade failed",
  'admin.stackUpgrade.state.paused.fail.body': "Upgrade could not be paused. Try again later.",
  'admin.stackDowngrade.state.paused.fail.header': "Pause Downgrade failed",
  'admin.stackDowngrade.state.paused.fail.body': "Downgrade could not be paused. Try again later.",
  'admin.stackVersions.version.errors.outOfSync.title': 'Host component out of sync',
  'admin.stackVersions.version.errors.outOfSync.desc': 'One of more host components did not report the version that Ambari expected.' + ' Please re-install the failed host component, or remove it.',
  'admin.stackVersions.version.errors.outOfSync.reinstall.title': 'Confirm Re-Install',
  'admin.stackVersions.version.errors.outOfSync.remove.title': 'Confirm Remove',

  'admin.stackVersions.version.service.notUpgradable': "The version of this service included in this repository is already installed in the cluster.",
  'admin.stackVersions.version.service.notSupported': "This service is unsupported in the current version of the stack.",

  'admin.stackVersions.version.upgrade.upgradeOptions.header': "Upgrade Options",
  'admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.version': "You are about to perform an upgrade to <b>{0}</b>.",
  'admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.method': "Choose the upgrade method:",
  'admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.tolerance': "Select optional upgrade failure tolerance:",
  'admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option1': "Skip all Slave Component failures",
  'admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option2': "Skip all Service Check failures",
  'admin.stackVersions.version.upgrade.upgradeOptions.tolerance.tooltip': "These upgrade failure tolerance options are useful when performing an upgrade on a large cluster and you want to minimize user intervention.",
  'admin.stackVersions.version.upgrade.upgradeOptions.RU.title': "Rolling",
  'admin.stackVersions.version.upgrade.upgradeOptions.RU.description': "Services remain running while the upgrade is performed. Minimized disruption but slower upgrade.",
  'admin.stackVersions.version.upgrade.upgradeOptions.EU.title': "Express",
  'admin.stackVersions.version.upgrade.upgradeOptions.HOU.title': "Host Ordered",
  'admin.stackVersions.version.upgrade.upgradeOptions.EU.description': "Services are stopped while the upgrade is performed. Incurs downtime, but faster upgrade.",
  'admin.stackVersions.version.upgrade.upgradeOptions.errors_bypassed': "Bypassed errors,<br/>proceed at your own risk.",
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.rerun': 'Rerun Pre-Upgrade Checks',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.title': 'Checks:',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.checking': 'Checking...',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.failed.title': 'Check failed',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.failed.link': 'Rerun',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.allPassed': 'Passed',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.allPassed.msg': 'All checks passed',
  'admin.stackVersions.version.upgrade.upgradeOptions.preCheck.failed.tooltip': 'Option not available',
  'admin.stackVersions.version.upgrade.upgradeOptions.notAllowed': 'Not allowed by the current version',
  'admin.stackVersions.version.upgrade.upgradeOptions.EU.confirm.msg': 'You are about to perform an <b>Express Upgrade</b> to <b>{0}</b>. This will incur cluster downtime. Are you sure you want to proceed?',
  'admin.stackVersions.version.upgrade.upgradeOptions.RU.confirm.msg': 'You are about to perform a <b>Rolling Upgrade</b> to <b>{0}</b>. Are you sure you want to proceed?',
  'admin.stackVersions.version.upgrade.upgradeOptions.error': 'Could not proceed with upgrade:',
  'admin.stackVersions.version.upgrade.upgradeOptions.loading': 'Checking for supported upgrade types...',

  'admin.stackVersions.version.upgrade.alertsWarning': 'Cluster alerts will still be visible and recorded in Ambari but notifications (such as Email and SNMP) will be suppressed during the upgrade.',

  'admin.stackVersions.version.preUpgrade.header': 'Pre-Upgrade Check',
  'admin.stackVersions.version.preUpgrade.bodyMsg.version': 'The following checks have been run to test your ability to upgrade to <b>{0}</b>.',

  'admin.stackVersions.version.column.showDetails': "Show Details",
  'admin.stackVersions.version.column.showDetails.title': "Version Details",
  'admin.stackVersions.version.noCompatible.tooltip': 'Directly upgrading to this version is not supported.',

  'admin.stackVersions.hosts.popup.header.current': "Current",
  'admin.stackVersions.hosts.popup.header.installed': "Installed",
  'admin.stackVersions.hosts.popup.header.not_installed': "Not installed",
  'admin.stackVersions.hosts.popup.header': "Version Status: {0}",
  'admin.stackVersions.hosts.popup.title': "{0} Version is {1} on {2} hosts:",
  'admin.stackVersions.hosts.popup.current.title': "{0} is applied on {1} hosts:",
  'admin.stackVersions.hosts.popup.primary': "Go to Hosts",

  'admin.stackVersions.details.install.hosts.popup.title': "Install {0} version",
  'admin.stackVersions.upgradeHistory.upgrade': 'Upgrades',
  'admin.stackVersions.upgradeHistory.downgrade': 'Downgrades',
  'admin.stackVersions.upgradeHistory.show.details': 'Click to show more details on {0}',
  'admin.stackVersions.upgradeHistory.success': 'Sucessful {0}',
  'admin.stackVersions.upgradeHistory.aborted': 'Aborted {0}',
  'admin.stackVersions.upgradeHistory.summary': 'Summary',
  'admin.stackVersions.upgradeHistory.history': 'History',
  'admin.stackVersions.upgradeHistory.filter.all': 'All ({0})',
  'admin.stackVersions.upgradeHistory.filter.upgrade': 'Upgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.downgrade': 'Downgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.successful.upgrade': 'Successful Upgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.successful.downgrade': 'Successful Downgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.aborted.upgrade': 'Aborted Upgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.aborted.downgrade': 'Aborted Downgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.failed.upgrade': 'Failed Upgrade ({0})',
  'admin.stackVersions.upgradeHistory.filter.failed.downgrade': 'Failed Downgrade ({0})',
  'admin.stackVersions.upgradeHistory.no.history': 'No upgrade/downgrade history available',
  'admin.stackVersions.upgradeHistory.record.title.upgrade': '{0} {1} to {2}',
  'admin.stackVersions.upgradeHistory.record.title.downgrade': '{0} {1} from {2}',

  'admin.stackUpgrade.preCheck.warning.message': "{0} Warning {1}",
  'admin.stackUpgrade.preCheck.bypass.message': "{0} Error {1}",
  'admin.stackUpgrade.preCheck.fail.message': "{0} Required {1}",

  'admin.stackUpgrade.finalize.later': "Finalize Later",
  'admin.stackUpgrade.finalize.message.upgrade': "Your cluster version has been upgraded. " + "Click on <b>Finalize</b> when you are ready to finalize the upgrade and commit to the new version." + " You are strongly encouraged to run tests on your cluster to ensure it is fully operational before finalizing." + " <b>You cannot go back to the original version once the upgrade is finalized.</b>",
  'admin.stackUpgrade.finalize.message.downgrade': "Your cluster version has been downgraded. " + "Click on <b>Finalize</b> when you are ready to finalize the downgrade and commit to the new version." + " You are strongly encouraged to run tests on your cluster to ensure it is fully operational before finalizing." + " <b>You cannot go back to the original version once the downgrade is finalized.</b>",
  'admin.stackUpgrade.finalize.message.revertible': "The <b>{0}</b> upgrade to <b>{1}</b> is ready to be completed." + " After finalization, the patch can be reverted from the Stacks and Versions page if it is no longer required.",
  'admin.stackUpgrade.finalize.message.autoStart': "Please remember to re-enable Auto Start if you disabled it for upgrade.",
  'admin.stackUpgrade.finalize.message.skippedServiceChecks': "During the upgrade, checks for the following services failed and were skipped:",
  'admin.stackUpgrade.finalize.message.testServices': "You are strongly recommended to test these services before finalizing upgrade.",
  'admin.stackUpgrade.failedHosts.message': "Upgrade did not succeed on",
  'admin.stackUpgrade.failedHosts.showHosts': "{0} hosts",
  'admin.stackUpgrade.failedHosts.options': "Your options:",
  'admin.stackUpgrade.failedHosts.options.first': "<b>Pause Upgrade</b>, delete the unhealthy hosts and return to the Upgrade Wizard to Proceed.",
  'admin.stackUpgrade.failedHosts.options.second': "Perform a <b>Downgrade</b>, which will revert all hosts to the previous stack version.",
  'admin.stackUpgrade.failedHosts.options.third': "Ignore these failures and <b>Proceed</b> for now (reconcile the failures later).",
  'admin.stackUpgrade.failedHosts.isOutOfSync.options.first': '<b>Pause Upgrade</b> and restart the component',
  'admin.stackUpgrade.failedHosts.isOutOfSync.options.second': '<b>Restart</b> the Ambari Agent',
  'admin.stackUpgrade.failedHosts.header': "Failed Hosts",
  'admin.stackUpgrade.failedHosts.subHeader': "Upgrade failed on {0} hosts",
  'admin.stackUpgrade.failedHosts.details': "Open Details",
  'admin.stackUpgrade.doThisLater': "Do This Later",
  'admin.stackUpgrade.pauseUpgrade': "Pause Upgrade",
  'admin.stackUpgrade.pauseDowngrade': "Pause Downgrade",
  'admin.stackUpgrade.pauseUpgrade.warning': "You are about to Pause the {0}. You can return to continue the {0} and finalize at a later time but while the upgrade is paused, you <strong style='color:red;'>SHOULD NOT</strong> perform any cluster changes. For example:" + "<ul><li>DO NOT add / remove hosts on the cluster</li>" + "<li>DO NOT add / remove services on the cluster</li>" + "<li>DO NOT enable / disable Kerberos</li>" + "<li>DO NOT enable / disable HA</li>" + "<li>DO NOT make any drastic changes to service configurations</li></ul>" + "You <strong>MUST</strong> continue the {0} and finalize <strong>BEFORE</strong> performing <strong>ANY</strong> significant changes to the cluster.",
  'admin.stackUpgrade.downgrade.proceed': "Proceed with Downgrade",
  'admin.stackUpgrade.downgrade.body': "Are you sure you wish to abort the upgrade process and downgrade to <b>{0}</b>?",
  'admin.stackUpgrade.downgrade.retry.body': "Are you sure you wish to retry downgrade to <b>{0}</b>?",
  'admin.stackUpgrade.upgrade.confirm.body': "You are about to perform an upgrade to {0}.",
  'admin.stackUpgrade.upgrade.retry.confirm.body': "You are about to retry an upgrade to {0}.",
  'admin.stackUpgrade.title': "Stack and Versions",
  'admin.stackUpgrade.state.inProgress': "Upgrade in Progress",
  'admin.stackUpgrade.state.paused': "Upgrade Paused",
  'admin.stackUpgrade.state.aborted': "Upgrade Aborted",
  'admin.stackUpgrade.state.completed': "Upgrade Finished",
  'admin.stackUpgrade.state.inProgress.downgrade': "Downgrade in Progress",
  'admin.stackUpgrade.state.init': "Upgrade Initializing",
  'admin.stackUpgrade.state.init.downgrade': "Downgrade Initializing",
  'admin.stackUpgrade.state.paused.downgrade': "Downgrade Paused",
  'admin.stackUpgrade.state.aborted.downgrade': "Downgrade Aborted",
  'admin.stackUpgrade.state.completed.downgrade': "Downgrade Finished",
  'admin.stackUpgrade.dialog.upgrade.header': "{0} Upgrade to {1}",
  'admin.stackUpgrade.dialog.upgrade.patch.header': "{0} Patch Upgrade to {1}",
  'admin.stackUpgrade.dialog.upgrade.maint.header': "{0} Maint Upgrade to {1}",
  'admin.stackUpgrade.dialog.downgrade.header': "Downgrade from {0}",
  'admin.stackUpgrade.dialog.operationFailed': "This operation failed.",
  'admin.stackUpgrade.dialog.stop': "Stop Upgrade",
  'admin.stackUpgrade.dialog.continue': "Ignore and Proceed",
  'admin.stackUpgrade.dialog.cancel': "Cancel Upgrade",
  'admin.stackUpgrade.dialog.inProgress': "Now Running:",
  'admin.stackUpgrade.dialog.keepRunning': "Keep running Upgrade in background",
  'admin.stackUpgrade.dialog.failed': "Failed on:",
  'admin.stackUpgrade.dialog.manual.slaveComponentFailures.title': "Slave Component Failures",
  'admin.stackUpgrade.dialog.manual.serviceCheckFailures.title': "Service Check Failures",
  'admin.stackUpgrade.dialog.manual.serviceCheckFailures.msg1': "The following service checks failed but were skipped:",
  'admin.stackUpgrade.dialog.manual.serviceCheckFailures.msg2': "You have the option to Pause Upgrade and fix the above issue(s) before proceeding.",
  'admin.stackUpgrade.dialog.manual': "Manual steps required",
  'admin.stackUpgrade.dialog.manualDone': "I have performed the manual steps above.",
  'admin.stackUpgrade.dialog.closeProgress': "Upgrade is in progress. \n If you dismiss this window, Upgrade will keep running in background.",
  'admin.stackUpgrade.dialog.closePause': "Upgrade is paused. \n If you dismiss this window, you can resume Upgrade later.",
  'admin.stackUpgrade.dialog.aborted': "Upgrade is aborted. \n You can either downgrade or retry upgrade.",
  'admin.stackUpgrade.dialog.downgrade.aborted': "Downgrade is aborted. \n You can retry downgrade.",
  'admin.stackUpgrade.dialog.suspended': "Upgrade is Paused",
  'admin.stackUpgrade.dialog.suspended.downgrade': "Downgrade is Paused",
  'admin.stackUpgrade.dialog.resume': "Resume Upgrade",
  'admin.stackUpgrade.dialog.resume.downgrade': "Resume Downgrade",
  'admin.stackUpgrade.dialog.details.open': "show details",
  'admin.stackUpgrade.dialog.details.hide': "hide details",
  'admin.stackUpgrade.dialog.notActive': "Waiting to execute the next task...",
  'admin.stackUpgrade.dialog.prepareUpgrade.header': "Preparing the Upgrade...",
  'admin.stackUpgrade.dialog.skipped.failures': 'There were automatically skipped failed steps.  Please resolve each failure before continuing with the upgrade.',
  'services.service.start': 'Start',
  'services.service.stop': 'Stop',
  'services.service.allComponents': 'All Components',
  'services.service.metrics': 'Metrics',
  'services.nothingToAdd': 'Nothing to add',
  'services.service.summary.version': 'Version',
  'services.service.summary.viewHost': 'View Host',
  'services.service.summary.viewHosts': 'View Hosts',
  'services.service.summary.clientOnlyService.ToolTip': 'Client-only service',
  'services.service.summary.mapreduce2.client': 'MapReduce2 Client',
  'services.service.summary.mapreduce2.clients': 'MapReduce2 Clients',
  'services.service.summary.nodeManagersLive': 'NodeManagers Live',
  'services.service.summary.TrackersLive': 'Trackers Live',
  'services.service.summary.PhoenixServersLIVE': 'Phoenix Query servers Live',
  'services.service.summary.GangliaMonitorsLIVE': 'Ganglia Monitors Live',
  'services.service.summary.SupervisorsLIVE': 'Supervisors Live',
  'services.service.summary.nameNode': 'NameNode Web UI',
  'services.service.summary.nameNodeUptime': 'NameNode Uptime',
  'services.service.summary.nameNodeHeap': 'NameNode Heap',
  'services.service.summary.nameNode.active': 'Active NameNode',
  'services.service.summary.nameNode.standby': 'Standby NameNode',
  'services.service.summary.pendingUpgradeStatus': 'Upgrade Status',
  'services.service.summary.pendingUpgradeStatus.notFinalized': 'Upgrade not finalized',
  'services.service.summary.pendingUpgradeStatus.notPending': 'No pending upgrade',
  'services.service.summary.safeModeStatus': 'Safe Mode Status',
  'services.service.summary.safeModeStatus.inSafeMode': 'In safe mode',
  'services.service.summary.safeModeStatus.notInSafeMode': 'Not in safe mode',
  'services.service.summary.dataNodes': 'DataNodes',
  'services.service.summary.diskCapacity': 'HDFS Disk Capacity',
  'services.service.summary.blocks': 'Blocks',
  'services.service.summary.blockErrors': 'Block Errors',
  'services.service.summary.totalFiles': 'Total Files + Dirs',
  'services.service.summary.jobTracker': 'JobTracker',
  'services.service.summary.jobTrackerWebUI': 'JobTracker Web UI',
  'services.service.summary.jobTrackerUptime': 'JobTracker Uptime',
  'services.service.summary.trackersLiveTotal': 'Trackers',
  'services.service.summary.trackersBlacklistGraylist': 'Trackers',
  'services.service.summary.jobTrackerHeap': 'JobTracker Heap',
  'services.service.summary.totalSlotsCapacity': 'Total Slots Capacity',
  'services.service.summary.totalJobs': 'Total Jobs',
  'services.service.summary.currentSlotUtiliMaps': 'Map Slots',
  'services.service.summary.currentSlotUtiliReduces': 'Reduce Slots',
  'services.service.summary.tasksMaps': 'Tasks: Maps',
  'services.service.summary.tasksReduces': 'Tasks: Reduces',
  'services.service.summary.hbaseMaster': 'HBase Master Web UI',
  'services.service.summary.regionServerCount': 'RegionServer Count',
  'services.service.summary.regionInTransition': 'Region In Transition',
  'services.service.summary.masterStarted': 'Master Started',
  'services.service.summary.masterActivated': 'Master Activated',
  'services.service.summary.averageLoad': 'Average Load',
  'services.service.summary.masterHeap': 'Master Heap',
  'services.service.summary.moreStats': 'more stats here',
  'services.service.summary.clientCount': '{0} Client Hosts',
  'services.service.summary.historyServer': 'History Server Web UI',
  'services.service.summary.hiveserver2.jdbc.url.text': ' JDBC URL',
  'services.service.summary.hiveserver2.endpoint.tooltip.text': 'JDBC connection string for {0}',
  'services.service.summary.hiveserver2.endpoint.value': 'jdbc:hive2://{0}/;serviceDiscoveryMode={1};zooKeeperNamespace={2}',
  'services.service.summary.inMaintenanceMode.tooltip': 'Service is in maintenance mode',
  'services.service.actions.downloadClientConfigs': 'Download Client Configs',
  'services.service.actions.downloadClientConfigs.fail.noConfigFile': 'No configuration files defined for the component',
  'services.service.actions.downloadClientConfigs.fail.popup.header': '{0} Configs',
  'services.service.actions.downloadClientConfigs.fail.popup.body.noErrorMessage': 'Generation of {0} configurations file has failed. ',
  'services.service.actions.downloadClientConfigs.fail.popup.body.errorMessage': 'Generation of {0} configurations file has failed with {1} error: <br /><pre><span class="text-error">{2}</span></pre>',
  'services.service.actions.downloadClientConfigs.fail.popup.body.question': 'Do you want to try again?',
  'services.service.actions.deleteService': 'Delete Service',
  'services.service.actions.run.rebalancer': 'Run Rebalancer',
  'services.service.actions.run.rebalanceHdfsNodes': 'Rebalance HDFS',
  'services.service.actions.run.rebalanceHdfsNodes.title': 'HDFS Rebalance',
  'services.service.actions.run.rebalanceHdfsNodes.prompt': 'Balancer threshold (percentage of disk capacity):',
  'services.service.actions.run.rebalanceHdfsNodes.promptTooltip': 'Percentage of disk capacity. This overwrites the default threshold',
  'services.service.actions.run.rebalanceHdfsNodes.promptError': 'Value should be number between 1 and 100',
  'services.service.actions.run.rebalanceHdfsNodes.context': 'Rebalance HDFS',
  'services.service.actions.run.rebalanceHdfsNodes.error': 'Error during remote command: ',
  'services.service.actions.run.yarnRefreshQueues.title': 'Refresh Queues ResourceManager',
  'services.service.actions.run.yarnRefreshQueues.menu': 'Refresh YARN Capacity Scheduler',
  'services.service.actions.run.yarnRefreshQueues.context': 'Refresh YARN Capacity Scheduler',
  'services.service.actions.run.yarnRefreshQueues.error': 'Error during remote command: ',
  'services.service.actions.run.executeCustomCommand.menu': '{0}',
  'services.service.actions.run.executeCustomCommand.context': 'Execute {0}',
  'services.service.actions.run.executeCustomCommand.error': 'Error during remote command: ',
  'services.service.actions.run.compaction': 'Run Compaction',
  'services.service.actions.run.smoke': 'Run Service Check',
  'services.service.actions.reassign.master': 'Move {0}',
  'services.service.actions.reassign.master.hive': 'Move HiveServer2, WebHCat Server, MySQL Server',
  'services.service.actions.manage_configuration_groups': 'Manage Configuration Groups...',
  'services.service.actions.run.startLdapKnox.title': 'Start Demo LDAP Knox Gateway',
  'services.service.actions.run.startLdapKnox.context': 'Start Demo LDAP',
  'services.service.actions.run.stopLdapKnox.title': 'Stop Demo LDAP Knox Gateway',
  'services.service.actions.run.stopLdapKnox.context': 'Stop Demo LDAP',
  'services.service.actions.run.startStopLdapKnox.error': 'Error during remote command: ',
  'services.service.actions.run.updateHBaseReplication.label': 'Start/Update HBase Replication',
  'services.service.actions.run.updateHBaseReplication.context': 'Enable Cross Cluster HBase Replication',
  'services.service.actions.run.updateHBaseReplication.title': 'HBase Replication: Start/Update',
  'services.service.actions.run.updateHBaseReplication.promptError': 'ZKQuorum should be comma(,) separated ZK nodes. Port and Peer ID should be number',
  'services.service.actions.run.updateHBaseReplication.zkquorumText.prompt': 'Peer Cluster ZK Quorum: ',
  'services.service.actions.run.updateHBaseReplication.zkportText.prompt': 'Peer Cluster ZK Port: ',
  'services.service.actions.run.updateHBaseReplication.parentzkeyText.prompt': 'Parent ZKey for HBase: ',
  'services.service.actions.run.updateHBaseReplication.peerIdText.prompt': 'Peer Cluster ID: ',
  'services.service.actions.run.updateHBaseReplication.error': 'Error while setting up HBase Cross Cluster Replication ',
  'services.service.actions.run.stopHBaseReplication.label': 'Stop HBase Replication',
  'services.service.actions.run.stopHBaseReplication.context': 'Disable Cross Cluster HBase Replication',
  'services.service.actions.run.stopHBaseReplication.title': 'HBase Replication: Stop',
  'services.service.actions.run.stopHBaseReplication.promptError': 'Peer ID should be number',
  'services.service.actions.run.stopHBaseReplication.error': 'Error while disabling HBase Cross Cluster Replication ',
  'services.service.actions.run.stopHBaseReplication.prompt': 'Peer Cluster ID to remove from Replication Peer list: ',

  // Hive Server Interactive custom command to restart LLAP
  'services.service.actions.run.restartLLAP': 'Restart LLAP',
  'services.service.actions.run.immediateStopHawqService.context': 'Stop HAWQ Service (Immediate Mode)',
  'services.service.actions.run.immediateStopHawqService.label': 'Stop HAWQ Service (Immediate Mode)',
  'services.service.actions.run.immediateStopHawqSegment.label': 'Stop (Immediate Mode)',
  'services.service.actions.run.immediateStopHawqSegment.context': 'Stop HAWQ Segment (Immediate Mode)',
  'services.service.actions.run.resyncHawqStandby.context': 'Re-Sync HAWQ Standby Master',
  'services.service.actions.run.resyncHawqStandby.label': 'Re-Synchronize HAWQ Standby Master',
  'services.service.actions.run.clearHawqCache.label': 'Clear HAWQ\'s HDFS Metadata Cache',
  'services.service.actions.run.runHawqCheck.label': 'Run HAWQ Config Check',
  'services.service.actions.manage_configuration_groups.short': 'Manage Config Groups',
  'services.service.actions.hsi.alertPopup.header': 'Cannot enable Interactive Query',
  'services.service.actions.hsi.alertPopup.body': 'You cannot enable Interactive Query now because Interactive Query is in the process of being disabled. This may take a few minutes. Try again later.',

  'services.service.delete.popup.header': 'Delete Service',
  'services.service.delete.popup.kerberos': 'The Kerberos Service <i>cannot be deleted</i> because Kerberos is ' + 'currently enabled, and in use by the cluster. The Kerberos Service can only be removed by disabling Kerberos,' + ' which can be found by browsing to <b>Admin > Kerberos</b> and selecting <b>Disable Kerberos</b>.',
  'services.service.delete.popup.ranger': 'Prior to deleting <b>Ranger</b>, you must disable plugins.',
  'services.service.delete.configVersionNote': 'Update configs after {0} has been removed',
  'services.service.delete.configVersionNote.plural': 'Update configs after {0} have been removed',
  'services.service.delete.lastService.popup.body': 'The <b>{0}</b> service can\'t be deleted, at least one service must be installed.',
  'services.service.delete.popup.dependentServices': 'Prior to deleting <b>{0}</b>, you must delete the following dependent services:',
  'services.service.delete.popup.mustBeStopped': 'Prior to deleting <b>{0}</b>, you must stop the service and each slave and master component.',
  'services.service.delete.popup.mustBeStopped.dependent': ' Along with dependent service <b>{0}</b>.',
  'services.service.delete.popup.warning': 'The <b>{0} service will be removed from Ambari and all configurations' + ' and configuration history will be lost.</b>',
  'services.service.delete.popup.warning.dependent': '<b> Note: The dependent {0} service will be removed too.</b>',
  'services.service.confirmDelete.popup.header': 'Confirm Delete',
  'services.service.confirmDelete.popup.body': 'You must confirm delete of <b>{0}</b> by typing "{1}"' + ' in the confirmation box. <b>This operation is not reversible and all configuration history will be lost.</b>',
  'services.service.confirmDelete.popup.body.type': 'Type "{0}" to confirm',
  'services.service.confirmDelete.popup.body.dependent': 'You must confirm delete of <b>{0}</b> and <b>{1}</b> by typing "{2}"' + ' in the confirmation box. <b>This operation is not reversible and all configuration history will be lost.</b>',
  'services.service.delete.progressPopup.message': 'Deleting {0}...',
  'services.service.delete.service.success.confirmation': 'Service {0} was successfully deleted',
  'services.service.delete.service.success.confirmation.plural': 'Services {0} were successfully deleted',
  'services.service.delete.popup.warning.ranger_kms': '<b>Ranger KMS should not be deleted if encryption zones (EZ) and keys are being managed by the service. Otherwise data in the EZ will not be accessible.</b>',

  'services.service.summary.unknown': 'unknown',
  'services.service.summary.notRunning': 'Not Running',
  'services.service.summary.notAvailable': 'n/a',
  'services.service.summary.diskInfoBar.used': 'used',
  'services.service.summary.storm.freeslots': 'Free slots',
  'services.service.summary.storm.executors': 'Executors',
  'services.service.summary.storm.tasks': 'Tasks',
  'services.service.summary.storm.nimbus.uptime': 'Nimbus uptime',
  'services.service.summary.storm.topologies': 'Topologies',
  'services.service.summary.flume.startAgent': 'Start Agent',
  'services.service.summary.flume.stopAgent': 'Stop Agent',
  'services.service.summary.flume.stop.context': 'Stop Flume {0}',
  'services.service.summary.flume.start.context': 'Start Flume {0}',
  'services.service.summary.flume.noAgents': 'No Flume to display',

  'services.service.summary.ranger.plugin.title': 'Ranger {0} plugin',
  'services.service.summary.ranger.plugin.loadingStatus': 'Loading status...',

  'services.service.summary.alerts.noAlerts': '0',
  'services.service.summary.alerts.alertsExist': '{0} alerts',
  'services.service.summary.alerts.popup.header': 'Alerts for {0}',

  'services.service.info.metrics.ambariMetrics.master.averageLoad': 'Average load',
  'services.service.info.metrics.ambariMetrics.master.displayNames.averageLoad': 'Average load',
  'services.service.info.metrics.ambariMetrics.regionServer.storeFiles': 'Number of StoreFiles',
  'services.service.info.metrics.ambariMetrics.regionServer.displayNames.storeFilesCount': 'Number of StoreFiles',
  'services.service.info.metrics.ambariMetrics.regionServer.regions': 'Number of Regions',
  'services.service.info.metrics.ambariMetrics.regionServer.displayNames.regionsCount': 'Number of Regions',
  'services.service.info.metrics.ambariMetrics.regionServer.requests': 'Total Request Count',
  'services.service.info.metrics.ambariMetrics.regionServer.displayNames.requestCount': 'Total Request Count',
  'services.service.info.metrics.ambariMetrics.regionServer.blockCacheHitPercent': 'Block Cache Hit Percent',
  'services.service.info.metrics.ambariMetrics.regionServer.displayNames.blockCacheHitPercent': 'Block Cache Hit Percent',
  'services.service.info.metrics.ambariMetrics.regionServer.compactionQueueSize': 'Compaction Queue Size',
  'services.service.info.metrics.ambariMetrics.regionServer.displayNames.compactionQueueSize': 'Compaction Queue Size',

  'services.service.info.metrics.flume.channelFillPercent': 'Channel Fill Percentage',
  'services.service.info.metrics.flume.channelSize': 'Channel Size',
  'services.service.info.metrics.flume.sinkDrainSuccess': 'Sink Event Drain Count',
  'services.service.info.metrics.flume.sourceAccepted': 'Source Event Accepted Count',
  'services.service.info.metrics.flume.sinkConnectionFailed': 'Sink Connection Failed Count',
  'services.service.info.metrics.flume.channelSizeMMA': 'Channel Size Events',
  'services.service.info.metrics.flume.channelSizeSum': 'Channel Size Event Sum',
  'services.service.info.metrics.flume.incoming.mma': 'Incoming Event Rate',
  'services.service.info.metrics.flume.incoming.sum': 'Incoming Event Sum',
  'services.service.info.metrics.flume.outgoing.mma': 'Outgoing Event Rate',
  'services.service.info.metrics.flume.outgoing.sum': 'Outgoing Event Sum',
  'services.service.info.metrics.flume.gc': 'Garbage Collection Time',
  'services.service.info.metrics.flume.cpu.user': 'CPU (User)',
  'services.service.info.metrics.flume.jvmThreadsRunnable': 'JVM Runnable Threads',
  'services.service.info.metrics.flume.jvmHeapUsed': 'JVM Heap Memory Used',
  'services.service.info.metrics.flume.channelType': 'Channel size {0}',
  'services.service.info.metrics.flume.incoming_mma': 'Incoming {0}',
  'services.service.info.metrics.flume.outgoing_mma': 'Outgoing {0}',
  'services.service.info.metrics.flume.sourceName': 'Source {0}',
  'services.service.info.metrics.flume.hostName': 'Host: {0}',
  'services.service.info.metrics.flume.channelName': 'Channel {0}',
  'services.service.info.metrics.flume.sinkName': 'Sink {0}',

  'services.service.info.metrics.hbase.clusterRequests': 'Cluster Requests',
  'services.service.info.metrics.hbase.clusterRequests.displayNames.requestCount': 'Request Count',
  'services.service.info.metrics.hbase.hlogSplitSize': 'HLog Split Size',
  'services.service.info.metrics.hbase.hlogSplitSize.displayNames.splitSize': 'Split Size',
  'services.service.info.metrics.hbase.hlogSplitTime': 'HLog Split Time',
  'services.service.info.metrics.hbase.hlogSplitTime.displayNames.splitTime': 'Split Time',
  'services.service.info.metrics.hbase.regionServerQueueSize': 'RegionServer Queue Size',
  'services.service.info.metrics.hbase.regionServerQueueSize.displayNames.compactionQueueSize': 'Compaction Queue Size',
  'services.service.info.metrics.hbase.regionServerQueueSize.displayNames.flushQueueSize': 'Flush Queue Size',
  'services.service.info.metrics.hbase.regionServerRegions': 'RegionServer Regions',
  'services.service.info.metrics.hbase.regionServerRegions.displayNames.regions': 'Regions',
  'services.service.info.metrics.hbase.regionServerRequests': 'Cumulative Requests',
  'services.service.info.metrics.hbase.regionServerRequests.2': 'RegionServer Requests',
  'services.service.info.metrics.hbase.regionServerRequests.displayNames.writeRequests': 'Write Requests',
  'services.service.info.metrics.hbase.regionServerRequests.displayNames.readRequests': 'Read Requests',

  'services.service.info.metrics.hdfs.blockStatus': 'Block Status',
  'services.service.info.metrics.hdfs.blockStatus.displayNames.pendingReplicationBlocks': 'Pending Replication Blocks',
  'services.service.info.metrics.hdfs.blockStatus.displayNames.underReplicatedBlocks': 'Under Replicated Blocks',
  'services.service.info.metrics.hdfs.fileOperations': 'File Operations',
  'services.service.info.metrics.hdfs.fileOperations.displayNames.fileInformationOperations': 'File Information Operations',
  'services.service.info.metrics.hdfs.fileOperations.displayNames.deleteFileOperations': 'Delete File Operations',
  'services.service.info.metrics.hdfs.fileOperations.displayNames.createFileOperations': 'Create File Operations',
  'services.service.info.metrics.hdfs.gc': 'Garbage Collection',
  'services.service.info.metrics.hdfs.gc.displayNames.gcTimeMillis': 'Time',
  'services.service.info.metrics.hdfs.io': 'HDFS I/O',
  'services.service.info.metrics.hdfs.io.displayNames.bytesWritten': 'Bytes Written',
  'services.service.info.metrics.hdfs.io.displayNames.bytesRead': 'Bytes Read',
  'services.service.info.metrics.hdfs.jvmHeap': 'JVM Memory Status',
  'services.service.info.metrics.hdfs.jvmHeap.displayNames.memHeapCommittedM': 'Heap Memory Committed',
  'services.service.info.metrics.hdfs.jvmHeap.displayNames.memNonHeapUsedM': 'Non Heap Memory Used',
  'services.service.info.metrics.hdfs.jvmHeap.displayNames.memHeapUsedM': 'Heap Memory Used',
  'services.service.info.metrics.hdfs.jvmHeap.displayNames.memNonHeapCommittedM': 'Non Heap Memory Committed',
  'services.service.info.metrics.hdfs.jvmThreads': 'JVM Thread Status',
  'services.service.info.metrics.hdfs.jvmThreads.displayNames.threadsBlocked': 'Threads Blocked',
  'services.service.info.metrics.hdfs.jvmThreads.displayNames.threadsWaiting': 'Threads Waiting',
  'services.service.info.metrics.hdfs.jvmThreads.displayNames.threadsTimedWaiting': 'Threads Timed Waiting',
  'services.service.info.metrics.hdfs.jvmThreads.displayNames.threadsRunnable': 'Threads Runnable',
  'services.service.info.metrics.hdfs.rpc': 'RPC',
  'services.service.info.metrics.hdfs.rpc.displayNames.rpcQueueTimeAvgTime': 'Queue Average Wait Time',
  'services.service.info.metrics.hdfs.spaceUtilization': 'Total Space Utilization',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityRemainingGB': 'Capacity Remaining',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityUsedGB': 'Capacity Used',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityTotalGB': 'Capacity Total',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityRemaining': 'Capacity Remaining',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityUsed': 'Capacity Used',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityTotal': 'Capacity Total',
  'services.service.info.metrics.hdfs.spaceUtilization.displayNames.capacityNonDFSUsed': 'Non DFS Capacity Used',

  'services.service.info.metrics.yarn.jvmHeap': 'JVM Memory Status',
  'services.service.info.metrics.yarn.jvmHeap.displayNames.memHeapCommittedM': 'Heap Memory Committed',
  'services.service.info.metrics.yarn.jvmHeap.displayNames.memNonHeapUsedM': 'Non Heap Memory Used',
  'services.service.info.metrics.yarn.jvmHeap.displayNames.memHeapUsedM': 'Heap Memory Used',
  'services.service.info.metrics.yarn.jvmHeap.displayNames.memNonHeapCommittedM': 'Non Heap Memory Committed',
  'services.service.info.metrics.yarn.jvmThreads': 'JVM Thread Status',
  'services.service.info.metrics.yarn.jvmThreads.displayNames.threadsBlocked': 'Threads Blocked',
  'services.service.info.metrics.yarn.jvmThreads.displayNames.threadsWaiting': 'Threads Waiting',
  'services.service.info.metrics.yarn.jvmThreads.displayNames.threadsTimedWaiting': 'Threads Timed Waiting',
  'services.service.info.metrics.yarn.jvmThreads.displayNames.threadsRunnable': 'Threads Runnable',
  'services.service.info.metrics.yarn.rpc': 'RPC',
  'services.service.info.metrics.yarn.rpc.displayNames.RpcQueueTimeAvgTime': 'Queue Average Wait Time',
  'services.service.info.metrics.yarn.gc': 'Garbage Collection',
  'services.service.info.metrics.yarn.gc.displayNames.gcTimeMillis': 'Time',
  'services.service.info.metrics.yarn.allocated.memory': 'Cluster Memory',
  'services.service.info.metrics.yarn.allocated.memory.displayNames.allocated': 'Allocated',
  'services.service.info.metrics.yarn.allocated.memory.displayNames.available': 'Available',
  'services.service.info.metrics.yarn.allocated.memory.displayNames.pending': 'Pending',
  'services.service.info.metrics.yarn.allocated.container': 'Containers',
  'services.service.info.metrics.yarn.allocated.container.displayNames.allocated': 'Allocated',
  'services.service.info.metrics.yarn.allocated.container.displayNames.reserved': 'Reserved',
  'services.service.info.metrics.yarn.allocated.container.displayNames.pending': 'Pending',
  'services.service.info.metrics.yarn.nodemanager.statuses': 'NodeManagers',
  'services.service.info.metrics.yarn.nodemanager.statuses.displayNames.active': 'Active NodeManagers',
  'services.service.info.metrics.yarn.nodemanager.statuses.displayNames.decommissioned': 'Decommissioned NodeManagers',
  'services.service.info.metrics.yarn.nodemanager.statuses.displayNames.lost': 'Lost NodeManagers',
  'services.service.info.metrics.yarn.nodemanager.statuses.displayNames.rebooted': 'Rebooted NodeManagers',
  'services.service.info.metrics.yarn.nodemanager.statuses.displayNames.unhealthy': 'Unhealthy NodeManagers',
  'services.service.info.metrics.yarn.queueMemoryResource': 'Queue Memory Used',
  'services.service.info.metrics.yarn.queueMemoryResource.displayNames.allocated': 'Allocated ({0})',
  'services.service.info.metrics.yarn.queueMemoryResource.displayNames.available': 'Available ({0})',
  'services.service.info.metrics.yarn.queueMemoryResource.displayName': 'Queue: {0}',
  'services.service.info.metrics.yarn.apps.states.current.title': 'Current Applications',
  'services.service.info.metrics.yarn.apps.states.finished.title': 'Finished Applications',
  'services.service.info.metrics.yarn.apps.states.completed': 'Completed',
  'services.service.info.metrics.yarn.apps.states.failed': 'Failed',
  'services.service.info.metrics.yarn.apps.states.killed': 'Killed',
  'services.service.info.metrics.yarn.apps.states.pending': 'Pending',
  'services.service.info.metrics.yarn.apps.states.running': 'Running',
  'services.service.info.metrics.yarn.apps.states.submitted': 'Submitted',

  'services.service.info.menu.summary': 'Summary',
  'services.service.info.menu.configs': 'Configs',
  'services.service.info.menu.heatmaps': 'Heatmaps',
  'services.service.info.menu.metrics': 'Metrics',
  'services.service.info.summary.hostsRunningMonitor': '{0}/{1}',
  'services.service.info.summary.serversHostCount': '{0} more',

  'services.service.config.nothing.to.display': 'No properties to display.',
  'services.service.config.final': 'Final',
  'services.service.config.password.additionalDescription': 'For security purposes, password changes will not be shown in configuration version comparisons',
  'services.service.config.secure.additionalDescription': 'This config can be changed from Kerberos page under Admin tab by privileged users.',
  'services.service.config.saved': 'Save Configuration Changes',
  'services.service.config.notSaved': 'Unable to Save Configuration Changes',
  'services.service.config.restartService.TooltipMessage': '<b>Restart Service</b><br>Stale configuration used by {0} components on {1} hosts:{2}',
  'services.service.config.restartService.needToRestart': '<strong>Restart Required:</strong> ',
  'services.service.config.restartService.needToRestartEnd': 'should be restarted',
  'service.service.config.restartService.shouldBeRestarted': '{0} Requiring Restart',
  'services.service.config.saved.message': 'Service configuration changes saved successfully.',
  'services.service.config.saving.message': 'Configuration changes are being saved...',
  'services.service.config.msgServiceStop': 'Could not save configuration changes.  Please stop the service first. You will be able to save configuration changes after all of its components are stopped.',
  'services.service.config.failCreateConfig': 'Failure in creating service configuration',
  'services.service.config.failSaveConfig': 'Failure in saving service configuration',
  'services.service.config.failSaveConfigHostOverrides': 'Failure in saving service configuration overrides',
  'services.service.config.addPropertyWindow.error.required': 'This is required',
  'services.service.config.addPropertyWindow.error.derivedKey': 'This property is already defined',
  'services.service.config.addPropertyWindow.error.derivedKey.location': 'This property is already defined in other configuration file(s) {0}',
  'services.service.config.addPropertyWindow.error.derivedKey.infile': 'This property is already defined in this configuration file',
  'services.service.config.addPropertyWindow.error.derivedKey.specific': 'Property "{0}" is already defined in this configuration file',
  'services.service.config.addPropertyWindow.error.format': 'Key and value should be separated by "=" sign',
  'services.service.config.addPropertyWindow.error.lineNumber': 'Line {0}: ',
  'services.service.config.addPropertyWindow.filterKeyLink': 'Find property',
  'services.service.config.addPropertyWindow.properties': 'Properties',
  'services.service.config.addPropertyWindow.propertiesHelper': 'key=value (one per line)',
  'services.service.config.addPropertyWindow.propertiesPlaceholder': 'Enter key=value (one per line)',
  'services.service.config.addPropertyWindow.bulkMode': 'Bulk property add mode',
  'services.service.config.addPropertyWindow.singleMode': 'Single property add mode',
  'services.service.config.stopService.runningHostComponents': '{0} components on {1} hosts are running',
  'services.service.config.stopService.unknownHostComponents': '{0} components on {1} hosts are in unknown state.  These components might actually be running and need restarting later to make the changes effective.',
  'services.service.config.confirmDirectoryChange': 'You are about to make changes to service directories that are core to {0}. Before you proceed, be absolutely certain of the implications and that you have taken necessary manual steps, if any, for the changes. Are you sure you want to proceed?',
  'services.service.config.configOverride.head': 'Config Override',
  'services.service.config.configOverride.body': 'Cannot override a config that has not been saved yet.',
  'services.service.config.exitPopup.body': 'You have unsaved changes. Save changes or discard?',
  'services.service.config.exitChangesPopup.body': 'Going back to the previous step will result in losing all your current customizations. Are you sure?',
  'services.service.config.propertyFilterPopover.title': 'Properties filter',
  'services.service.config.propertyFilterPopover.content': 'Enter keywords to filter properties by property name, value, or description.',
  'services.service.config.hive.oozie.postgresql': 'Existing PostgreSQL Database',
  'services.service.config.database.connection.success': 'Connection OK',
  'services.service.config.database.connection.inProgress': 'Checking connectivity',
  'services.service.config.database.connection.failed': 'Connection Failed',
  'services.service.config.connection.logsPopup.header': '{0} Connection: {1}',
  'services.service.config.connection.exitPopup.msg': 'Test connection is in progress. It\'s recommended to wait until it wil be complete. Are you sure you want to exit Enable Kerberos Wizard?',
  'services.service.config.database.btn.idle': 'Test Connection',
  'services.service.config.kdc.btn.idle': 'Test KDC Connection',
  'services.service.config.database.btn.connecting': 'Connecting...',
  'services.service.config.database.msg.jdbcSetup': 'Be sure you have run:<br/>' + '<b>ambari-server setup --jdbc-db={0} --jdbc-driver=/path/to/{0}/{1}</b> ' + 'on the Ambari Server host to make the JDBC driver available and to enable testing the database connection.',
  'services.service.config.configHistory.configGroup': 'Config Group',
  'services.service.config.configHistory.rightArrow.tooltip': 'Show earlier versions',
  'services.service.config.configHistory.leftArrow.tooltip': 'Show later versions',
  'services.service.config.configHistory.dismissIcon.tooltip': 'Dismiss',
  'services.service.config.configHistory.makeCurrent.message': 'Created from service config version {0}',
  'services.service.config.configHistory.comparing': 'Comparing Changes in',
  'services.service.config.setRecommendedValue': 'Set Recommended',
  'services.service.config.database.msg.jdbcSetup.detailed': 'To use {0} with {6}, you must <a rel="noopener noreferrer" href="{3}" target="_blank">' + 'download the {4} from {0}</a>. Once downloaded to the Ambari Server host, run: <br/>' + '<b>ambari-server setup --jdbc-db={1} --jdbc-driver=/path/to/{1}/{2}</b>',

  'services.service.widgets.list-widget.nothingSelected': 'Nothing selected',

  'services.add.header': 'Add Service Wizard',
  'services.add.warning': 'Add Service Wizard is in progress. Do you really want to exit the Add Service Wizard?',
  'services.add.warningStep6': 'Closing this dialog will continue to install the selected service(s) in the background, but the installed services need to be started manually. Are you sure you want to quit?',
  'services.reassign.header': 'Move Master Wizard',
  'services.service.add': 'Add Service',
  'services.service.startAll': 'Start All',
  'services.service.stopAll': 'Stop All',
  'services.service.startAllComponents': 'Start all components for {0}',
  'services.service.stopAllComponents': 'Stop all components for {0}',
  'services.service.restartAllRequired': 'Restart All Required',
  'services.service.downloadAllClientConfigs': 'Download All Client Configs',
  'services.service.startAll.confirmMsg': 'You are about to start all services',
  'services.service.stopAll.confirmMsg': 'You are about to stop all services',
  'services.service.refreshAll.confirmMsg': '<p>You are about to restart {0}</p>' + '<div class="alert alert-warning">This will trigger alerts as the service is restarted. To suppress alerts, turn on Maintenance Mode for services listed above prior to running Restart All Required</div>',
  'services.service.start.confirmMsg': 'You are about to start {0}',
  'services.service.stop.confirmMsg': 'You are about to stop {0}',
  'services.service.stop.confirmButton': 'Confirm Stop',
  'services.service.start.confirmButton': 'Confirm Start',
  'services.service.stop.warningMsg.turnOnMM': 'This will generate alerts as the service is stopped. To suppress alerts, turn on Maintenance Mode for {0} prior to stopping.',
  'services.service.stopCertain.warningMsg.turnOnMM': 'This will generate alerts as components are stopped. To suppress alerts, turn on Maintenance Mode for {0} prior to stopping.',
  'services.service.stop.warningMsg.dependent.services': 'Stopping {0} may impair the functioning of its dependent service(s): {1}.',
  'services.service.restartAll.confirmButton': 'Confirm Restart All',
  'services.service.restartAll.confirmMsg': 'You are about to restart {0}',
  'services.service.restartAll.warningMsg.turnOnMM': 'This will trigger alerts as the service is restarted. To suppress alerts, turn on Maintenance Mode for {0} prior to running restart all',
  'services.service.restartCertain.warningMsg.turnOnMM': 'This will trigger alerts as components are restarted. To suppress alerts, turn on Maintenance Mode for {0} prior to running restart all',
  'services.service.componentsInNameSpace': 'components in {0} namespace',
  'services.service.stop.HDFS.warningMsg.checkPointNA': 'Could not determine the age of the last HDFS checkpoint. Please ensure that you have a recent checkpoint. Otherwise, the NameNode(s) can take a very long time to start up.',
  'services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions.singleHost.login': '<br><ol><li>Login to the NameNode host <b>{0}</b>.</li>',
  'services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions.multipleHosts.login': '<ol><li>Login to the NameNode hosts {0}.</li>',
  'services.service.stop.HDFS.warningMsg.checkPointTooOld.instructions': '<li>Put the NameNode in Safe Mode (read-only mode):' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -safemode enter\'</div></li>' + '<li>Once in Safe Mode, create a Checkpoint:' + '<div class="code-snippet">sudo su {0} -l -c \'hdfs dfsadmin -saveNamespace\'</div></li>' + '</ol>',
  'services.service.stop.HDFS.warningMsg.checkPointTooOld': 'The last HDFS checkpoint is older than {0} hours. ',
  'services.service.stop.HDFS.warningMsg.checkPointTooOld.makeSure': 'Make sure that you have taken a checkpoint before proceeding. Otherwise, the NameNode(s) can take a very long time to start up.',
  'services.service.stop.HDFS.warningMsg.nameSpaces.checkPointTooOld': 'The last HDFS checkpoint is older than {0} hours for the following namespaces:',
  'services.service.config_groups_popup.header': 'Manage {0} Configuration Groups',
  'services.service.config_groups_popup.notice': 'You can apply different sets of {{serviceName}} configurations to groups of hosts by managing {{serviceName}} Configuration Groups and their host membership.  Hosts belonging to a {{serviceName}} Configuration Group have the same set of configurations for {{serviceName}}. Each host belongs to one {{serviceName}} Configuration Group.',
  'services.service.config_groups_popup.rename': 'Rename',
  'services.service.config_groups_popup.duplicate': 'Duplicate',
  'services.service.config_groups_popup.group_name_lable': 'Name',
  'services.service.config_groups_popup.group_desc_lable': 'Description',
  'services.service.config_groups_popup.properties': 'Properties',
  'services.service.config_groups_popup.addButton': 'Create new Configuration Group',
  'services.service.config_groups_popup.removeButton': 'Delete Configuration Group',
  'services.service.config_groups_popup.renameButton': 'Rename Configuration Group',
  'services.service.config_groups_popup.addHost': 'Add hosts to selected Configuration Group',
  'services.service.config_groups_popup.addHostDisabled': 'There are no available hosts to add.',
  'services.service.config_groups_popup.removeHost': 'Remove hosts from selected Configuration Group',
  'services.service.config_groups_popup.duplicateButton': 'Duplicate Configuration Group',
  'services.service.config_groups.add_config_group_popup.header': 'Create New Configuration Group',
  'services.service.config_groups.duplicate_config_group_popup.header': 'Duplicate Configuration Group',
  'services.service.config_groups.rename_config_group_popup.header': 'Rename Configuration Group',
  'services.service.config_groups.switchGroupTextFull': 'Switch to \'{0}\' host config group',
  'services.service.config_groups.switchGroupTextShort': 'Switch to \'{0}\'',
  'services.reassign.closePopup': 'Move {0} wizard is in progress. You must allow the wizard to complete for Ambari to be in usable state. If you choose to quit, you must follow manual instructions to complete or revert move {0} wizard as documented in the Ambari User Guide. Are you sure you want to exit the wizard?',
  'services.reassign.error.fewHosts': 'You must have at least 2 hosts in your cluster to run Move Wizard.',

  'services.reassign.step1.header': 'Get Started',
  'services.reassign.step1.message1': 'This wizard will walk you through moving {0}.<br/>',
  'services.reassign.step1.message2': 'The process to reassign {0} involves a combination of <b>automated steps</b> (that will be handled by the wizard) and ' + '<b>manual steps</b> (that you must perform in sequence as instructed by the wizard).<br/><br/>',
  'services.reassign.step1.message3': '<br/><p class="restart-services-alert">Following services will be restarted as part of the wizard: <b>{0}</b>. You should plan a cluster maintenance window and prepare ' + 'for cluster downtime when moving {1}.</p>',

  'services.reassign.step2.header': 'Assign Master',
  'services.reassign.step2.currentHost': 'Current:',
  'services.reassign.step2.body': 'Assign {0} to new host.',
  'services.reassign.step2.body.namenodeHA': 'Move {0} to new host. You can move only one master component at a time.',
  'services.reassign.step3.header': 'Review',
  'services.reassign.step3.body': '<b>Confirm your host selections.</b>',
  'services.reassign.step3.targetHost': 'Target Host:',
  'services.reassign.step3.sourceHost': 'Source Host:',
  'services.reassign.step3.component': 'Component name:',
  'services.reassign.step3.configs': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>The Wizard will make the following configuration changes.</div>',
  'services.reassign.step4.header': 'Configure Component',

  'services.reassign.step4.tasks.stopRequiredServices.title': 'Stop Required Services',
  'services.reassign.step4.tasks.createHostComponents.title': 'Create {0}',
  'services.reassign.step4.tasks.putHostComponentsInMaintenanceMode.title': 'Disable {0}',
  'services.reassign.step4.tasks.reconfigure.title': 'Reconfigure {0}',
  'services.reassign.step4.tasks.save.configuration.note': 'This configuration is created by Move {0} wizard',
  'services.reassign.step4.tasks.installHostComponents.title': 'Install {0}',
  'services.reassign.step4.tasks.startZooKeeperServers.title': 'Start ZooKeeper Servers',
  'services.reassign.step4.tasks.startNameNode.title': 'Start NameNode',
  'services.reassign.step4.tasks.stopHostComponentsInMaintenanceMode.title': 'Stop {0}',
  'services.reassign.step4.tasks.deleteHostComponents.title': 'Delete disabled {0}',
  'services.reassign.step4.tasks.startRequiredServices.title': 'Start Required Services',
  'services.reassign.step4.tasks.cleanMySqlServer.title': 'Clean MYSQL Server',
  'services.reassign.step4.tasks.testHiveMysqlConnection.title': 'Test MYSQL Connection',
  'services.reassign.step4.tasks.configureMySqlServer.title': 'Configure MYSQL Server',
  'services.reassign.step4.tasks.startMySqlServer.title': 'Start MYSQL Server',
  'services.reassign.step4.tasks.restartMySqlServer.title': 'Restart MYSQL Server',
  'services.reassign.step4.tasks.startServices.title': 'Start Services',
  'services.reassign.step4.tasks.testDBConnection.title': 'Test DB Connection',
  'services.reassign.step4.tasks.testDBConnection.tooltip': 'Database Host: {0}\n' + 'Database Type: {1}\n' + 'Database Name: {2}\n' + 'Username: {3}\n' + 'Password: {4}\n' + 'JDBC Driver Class: {5}\n' + 'Database URL: {6}',
  'services.reassign.rollback.confirm': 'Are you sure?',

  'services.reassign.step4.task0.title': 'Stop Required Services',
  'services.reassign.step4.task1.title': 'Create {0}',
  'services.reassign.step4.task2.title': 'Disable {0}',
  'services.reassign.step4.task3.title': 'Reconfigure {0}',
  'services.reassign.step4.save.configuration.note': 'This configuration is created by Move {0} wizard',
  'services.reassign.step4.task4.title': 'Install {0}',
  'services.reassign.step4.task5.title': 'Start ZooKeeper Servers',
  'services.reassign.step4.task6.title': 'Start NameNode',
  'services.reassign.step4.task7.title': 'Delete disabled {0}',
  'services.reassign.step4.task8.title': 'Start Required Services',
  'services.reassign.step4.tasks.startNewMySqlServer.title': 'Start New MYSQL Server',
  'services.reassign.step4.status.success': 'Successfully moved <b>{0}</b> from <b>{1}</b> host to <b>{2}</b> host',
  'services.reassign.step4.status.success.withManualSteps': 'Proceed to the next step',
  'services.reassign.step4.status.failed': 'Failed to move <b>{0}</b> from <b>{1}</b> host to <b>{2}</b> host',
  'services.reassign.step4.status.info': 'Reassigning {0}. \nPlease wait for all tasks to be completed.',
  'services.reassign.step4.retry': 'You can click on the Retry or Abort button to retry failed task or abort changes',
  'services.reassign.step4.abortError': 'Error in aborting changes.',
  'services.reassign.step5.header': 'Manual commands',
  'services.reassign.step5.body.namenode': '<div class="alert alert-info">' + '<ol>' + '<li>Copy the contents of <b>{0}</b> on the source host <b>{1}</b> to <b>{0}</b> on the target host <b>{2}</b>.</li>' + '<li>Login to the target host <b>{2}</b> and change permissions for the NameNode dirs by running:' + '<div class="code-snippet">chown -R {3}:{5} {6}</div></li>' + '<li>Create marker directory by running:' + '<div class="code-snippet">mkdir -p /var/lib/hdfs/namenode/formatted</div></li>' + '</ol>' + '</div>',
  'services.reassign.step5.body.namenode_ha': '<div class="alert alert-info">' + '<ol>' + '<li>Login to the NameNode host <b>{4}</b>.</li>' + '<li>Reset automatic failover information in ZooKeeper by running:' + '<div class="code-snippet">sudo su {3} -l -c \'hdfs zkfc -formatZK\'</div></li>' + '</ol>' + '</div>' + '<div class="alert alert-info">' + '<ol start="3">' + '<li>Login to the newly installed NameNode host <b>{2}</b>.<br>' + '<div class="alert alert-warning"><strong>Important!</strong> Be sure to login to the newly installed NameNode host.<br>This is a different host from the Steps 1 and 2 above.</div>' + '</li>' + '<li>Initialize the metadata by running:' + "<div class='code-snippet'>sudo su {3} -l -c 'hdfs namenode -bootstrapStandby'</div></li>" + '</ol>' + '</div>',
  'services.reassign.step5.body.secondary_namenode': '<div class="alert alert-info">' + '<ol>' + '<li>Copy the contents of <b>{0}</b> on the source host <b>{1}</b> to <b>{0}</b> on the target host <b>{2}</b>.</li>' + '<li>Login to the target host <b>{2}</b> and change permissions for the SNameNode dirs by running:' + '<div class="code-snippet">chown -R {3}:{5} {6}</div></li>' + '</ol>' + '</div>',
  'services.reassign.step5.body.oozie_server': '<div class="alert alert-info">' + '<ol>' + '<li>On <b>{1}</b> copy the contents of' + '<div class="code-snippet"><b>/hadoop/oozie/data/oozie-db</b></div></li>' + '<li>To the target host <b>{2}</b></li>' + '<li>If the directory doesn\'t exists you can create by running' + '<div class="code-snippet">mkdir -p /hadoop/oozie/data/oozie-db</div></li>' + '<li>Update directory permissions by running' + '<div class="code-snippet">chown -R oozie:{5} /hadoop/oozie/data</div></li>' + '</ol>' + '</div>',
  'services.reassign.step5.body.mysql_server': '<div class="alert alert-info">' + '<ol>' + '<li>On <b>{1}</b> using a terminal you can export your Metastore DB (MYSQL) using:' + '<div class="code-snippet">mysqldump db_name > backup-file.sql</div></li>' + '<li>Copy the file to the target host <b>{2}</b> hosting the MySQL DB</li>' + '<li>Execute this SQL inside <b>mysql<b>' + '<div class="code-snippet">CREATE DATABASE db_name;</div></li>' + '<li>Import the database using' + '<div class="code-snippet">mysql db_name < backup-file.sql</div></li>' + '</ol>' + '</div>',
  'services.reassign.step5.body.app_timeline_server': '<div class="alert alert-info">' + '<ol>' + '<li>Copy <b>{7}/{8}</b> from the source host <b>{1}</b> to <b>{7}/{8}</b> on the target host <b>{2}</b>.</li>' + '<li>Login to the target host <b>{2}</b> and change permissions by running:' + '<div class="code-snippet">chown -R {3}:{5} {7}/{8}</div></li>' + '<div class="code-snippet">chmod -R 700 {7}/{8}</div></li>' + '</ol>' + '</div>',
  'services.reassign.step5.body.securityNotice': '<div class="alert alert-info"> <div class="alert alert-warning"> <strong>Note: </strong> Secure cluster' + ' requires generating necessary principals for reassigned component and creating keytab files with the principal on ' + 'the target host. The keytab file should be accessible to the service user.</div> {0} </div>',
  'services.reassign.step5.body.securityConfigsList': 'Create keytab file <b>{0}</b> with principal <b>{1}</b> on <b>{2}</b> host.',
  'services.reassign.step5.body.proceedMsg': 'Please proceed once you have completed the steps above',
  'services.reassign.step5.confirmPopup.body': 'Please confirm that you have run the manual steps before continuing.',
  'services.reassign.step6.header': 'Start and Test services',
  'services.reassign.step6.tasks.putHostComponentsInMaintenanceMode.title': 'Disable {0}',
  'services.reassign.step6.tasks.deleteHostComponents.title': 'Delete disabled {0}',
  'services.reassign.step6.tasks.startAllServices.title': 'Start All Services',
  'services.reassign.step6.tasks.installPxf.title': 'Install PXF on NameNode',
  'services.reassign.step6.tasks.stopMysqlService.title': 'Stop Mysql Server',
  'services.reassign.step6.tasks.stopHostComponentsInMaintenanceMode.title': 'Stop {0}',
  'services.reassign.step6.tasks.startDatanodes.title': 'Start DataNodes',
  'services.reassign.step6.status.success': 'Successfully moved <b>{0}</b> from <b>{1}</b> host to <b>{2}</b> host.',
  'services.reassign.step6.status.failed': 'Failed to move <b>{0}</b> from <b>{1}</b> host to <b>{2}</b> host.',
  'services.reassign.step6.status.info': 'Reassigning {0}. \nPlease wait for all tasks to be completed.',
  'services.reassign.step7.header': 'Rollback',
  'services.reassign.step7.info': 'Rollback in progress',
  'services.reassign.step7.failed': 'Rollback failed',
  'services.reassign.step7.success': 'Rollback successfully completed',

  /** services page constants **/

  'service.hbase.activeMaster': 'Active Master',

  'services.hive.client': 'Hive Client',
  'services.hive.clients': 'Hive Clients',

  'services.falcon.client': 'Falcon Client',
  'services.falcon.clients': 'Falcon Clients',

  'services.oozie.client': 'Oozie Client',
  'services.oozie.clients': 'Oozie Clients',
  'services.oozie.webUi': 'Oozie Web UI',

  'services.ganglia.webUi': 'Ganglia Web UI',
  'services.ganglia.monitors': 'Ganglia Monitors',

  'services.mapreduce2.webUi': 'History Server UI',

  'services.zookeeper.prefix': '{0} of',
  'services.zookeeper.title': '{0} ZooKeepers',
  'services.zookeeper.postfix': 'running',
  'services.zookeeper.clients': 'ZooKeeper Clients',
  'services.zookeeper.client': 'ZooKeeper Client',

  'services.mapreduce2.history.running': 'History server is running',
  'services.mapreduce2.history.stopped': 'History server is stopped',
  'services.mapreduce2.history.unknown': 'History server status is unknown',
  'services.mapreduce2.smokeTest.requirement': 'MapReduce2 smoke test requires YARN service be started',

  'services.tez.client': 'Tez client',
  'services.tez.clients': 'Tez clients',
  'services.pig.client': 'Pig client',
  'services.pig.clients': 'Pig clients',
  'services.glusterfs.client': 'GLUSTERFS client',
  'services.glusterfs.clients': 'GLUSTERFS clients',
  'services.sqoop.client': 'Sqoop client',
  'services.sqoop.clients': 'Sqoop clients',

  'services.hbase.master.error': 'None of the HBase masters is active',

  'alerts.actions.manage_alert_groups_popup.header': 'Manage Alert Groups',
  'alerts.actions.manage_alert_groups_popup.notice': 'You can manage alert groups for each service in this dialog. View the list of alert groups and the alert definitions configured in them. ' + 'You can also add/remove alert definitions, and pick notification for that alert group.',
  'alerts.actions.manage_alert_groups_popup.rename': 'Rename',
  'alerts.actions.manage_alert_groups_popup.duplicate': 'Duplicate',
  'alerts.actions.manage_alert_groups_popup.group_name_lable': 'Name',
  'alerts.actions.manage_alert_groups_popup.group_desc_lable': 'Description',
  'alerts.actions.manage_alert_groups_popup.notifications': 'Notifications',
  'alerts.actions.manage_alert_groups_popup.addButton': 'Create Alert Group',
  'alerts.actions.manage_alert_groups_popup.addGroup.exist': 'Alert Group with given name already exists',
  'alerts.actions.manage_alert_groups_popup.removeButton': 'Delete Alert Group',
  'alerts.actions.manage_alert_groups_popup.removeButtonDisabled': 'Cannot delete default alert group',
  'alerts.actions.manage_alert_groups_popup.renameButton': 'Rename Alert Group',
  'alerts.actions.manage_alert_groups_popup.duplicateButton': 'Duplicate Alert Group',
  'alerts.actions.manage_alert_groups_popup.addDefinition.noDefinitions': 'No alert definitions to display',
  'alerts.actions.manage_alert_groups_popup.addDefinition': 'Add alert definitions to selected Alert Group',
  'alerts.actions.manage_alert_groups_popup.addDefinitionDisabled': 'There are no available alert definitions to add',
  'alerts.actions.manage_alert_groups_popup.addDefinitionToDefault': 'Cannot modify default alert group',
  'alerts.actions.manage_alert_groups_popup.removeDefinition': 'Remove alert definitions from selected Alert Group',
  'alerts.actions.manage_alert_groups_popup.removeDefinitionDisabled': 'Cannot modify default alert group',
  'alerts.actions.manage_alert_groups_popup.selectDefsDialog.title': 'Select Alert Group Definitions',
  'alerts.actions.manage_alert_groups_popup.selectDefsDialog.message': 'Select alert definitions to be added to this "{0}" Alert Group.',
  'alerts.actions.manage_alert_groups_popup.selectDefsDialog.filter.placeHolder': 'All',
  'alerts.actions.manage_alert_groups_popup.selectDefsDialog.selectedDefsLink': '{0} out of {1} alert definitions selected',
  'alerts.actions.manage_alert_groups_popup.selectDefsDialog.message.warning': 'At least one alert definition needs to be selected.',

  'alerts.actions.manage_alert_notifications_popup.header': 'Manage Alert Notifications',
  'alerts.actions.manage_alert_notifications_popup.noAlertNotification': 'No alert notifications defined',
  'alerts.actions.manage_alert_notifications_popup.addButton': 'Create new Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.addHeader': 'Create Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.removeButton': 'Delete Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.editButton': 'Edit Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.editHeader': 'Edit Notification',
  'alerts.actions.manage_alert_notifications_popup.duplicateButton': 'Duplicate Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.disableButton': 'Disable Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.enableButton': 'Enable Alert Notification',
  'alerts.actions.manage_alert_notifications_popup.method': 'Method',
  'alerts.actions.manage_alert_notifications_popup.email': 'Email To',
  'alerts.actions.manage_alert_notifications_popup.SMTPServer': 'SMTP Server',
  'alerts.actions.manage_alert_notifications_popup.SMTPPort': 'SMTP Port',
  'alerts.actions.manage_alert_notifications_popup.SMTPUseAuthentication': 'Use authentication',
  'alerts.actions.manage_alert_notifications_popup.SMTPUsername': 'Username',
  'alerts.actions.manage_alert_notifications_popup.SMTPPassword': 'Password',
  'alerts.actions.manage_alert_notifications_popup.retypeSMTPPassword': 'Password Confirmation',
  'alerts.actions.manage_alert_notifications_popup.SMTPSTARTTLS': 'Start TLS',
  'alerts.actions.manage_alert_notifications_popup.emailFrom': 'Email From',
  'alerts.actions.manage_alert_notifications_popup.version': 'Version',
  'alerts.actions.manage_alert_notifications_popup.OIDs': 'OID',
  'alerts.actions.manage_alert_notifications_popup.community': 'Community',
  'alerts.actions.manage_alert_notifications_popup.host': 'Hosts',
  'alerts.actions.manage_alert_notifications_popup.port': 'Port',
  'alerts.actions.manage_alert_notifications_popup.global': 'Global',
  'alerts.actions.manage_alert_notifications_popup.noDescription': '<i>No description</i>',
  'alerts.actions.manage_alert_notifications_popup.noGroup': '<i>None selected</i>',
  'alerts.actions.manage_alert_notifications_popup.severityFilter': 'Severity',
  'alerts.actions.manage_alert_notifications_popup.clearAll': 'Clear All',
  'alerts.actions.manage_alert_notifications_popup.selectAll': 'Select All',
  'alerts.actions.manage_alert_notifications_popup.confirmDeleteHeader': 'Confirm Delete',
  'alerts.actions.manage_alert_notifications_popup.confirmDeleteBody': 'Are you sure you want to delete {0} notification?',
  'alerts.actions.manage_alert_notifications_popup.error.name.empty': 'Notification name is required',
  'alerts.actions.manage_alert_notifications_popup.error.name.existed': 'Notification name already exists',
  'alerts.actions.manage_alert_notifications_popup.scriptDispatchProperty': 'Script Dispatch Property',
  'alerts.actions.manage_alert_notifications_popup.scriptFileName': 'Script Filename',
  'alerts.actions.manage_alert_notifications_popup.error.scriptFileName.invalid': 'Invalid script filename',

  'hosts.host.add': 'Add New Hosts',
  'hosts.table.noHosts': 'No hosts to display',

  'hosts.filters.filteredHostsInfo': '{0} of {1} hosts showing',

  'hosts.filters.selectedHostInfo': 'host selected',
  'hosts.filters.selectedHostsInfo': 'hosts selected',

  'hosts.filters.clearSelection': 'clear selection',

  'hosts.filters.filterComponents': 'Filter by <strong>Component</strong>',

  'hosts.table.restartComponents.withNames': 'Restart {0}',
  'hosts.table.restartComponents.withoutNames': '{0} components should be restarted',

  'hosts.table.componentsInPassiveState.withNames': '{0} in Maintenance Mode',
  'hosts.table.componentsInPassiveState.withoutNames': '{0} components in Maintenance Mode',

  'hosts.table.menu.l1.selectedHosts': 'Selected Hosts',
  'hosts.table.menu.l1.filteredHosts': 'Filtered Hosts',
  'hosts.table.menu.l1.allHosts': 'All Hosts',
  'hosts.table.menu.l2.allComponents': 'All Components',
  'hosts.table.menu.l2.restartAllComponents': 'Restart All Components',
  'hosts.table.menu.l2.reinstallFailedComponents': 'Reinstall Failed Components',
  'hosts.table.menu.l2.refreshConfigsClientComponents': 'Refresh All Configs',

  'hosts.bulkOperation.confirmation.header': 'Confirm Bulk Operation',
  'hosts.bulkOperation.confirmation.hosts': 'Are you sure you want to <strong>{0}</strong> on the following {1} hosts?',
  'hosts.bulkOperation.confirmation.delete.hosts': 'Are you sure you want to delete the following {0} hosts?',
  'hosts.bulkOperation.confirmation.hostComponents': 'Are you sure you want to <strong>{0} {1}</strong> on the following {2} hosts?',
  'hosts.bulkOperation.passiveState.nothingToDo.body': 'All hosts that you selected are already in Maintenance Mode.',
  'hosts.bulkOperation.warningInfo.body': 'Components on these hosts are stopped so decommission will be skipped.',
  'hosts.bulkOperation.host_components.passiveState.nothingToDo.body': 'All host components that you selected are already in Maintenance Mode',
  'hosts.bulkOperation.confirmation.add.component': '{0} will be added to the following hosts.',
  'hosts.bulkOperation.confirmation.add.component.skip': '{0} already installed.',
  'hosts.bulkOperation.confirmation.add.component.noHeartBeat.skip': 'Host heartbeat lost',
  'hosts.bulkOperation.confirmation.add.component.nothingToDo.body': 'All the selected hosts have {0} installed already.',
  'hosts.bulkOperation.confirmation.cannot.add1': 'The following hosts will be skipped (expand for reason):',
  'hosts.bulkOperation.confirmation.cannot.add2': '{0} cannot be added to the following hosts (expand for reason):',
  'hosts.bulkOperation.deleteHosts.nonDeletableComponents': 'Deletion of the following components is not supported: {0}',
  'hosts.bulkOperation.deleteHosts.nonAddableMasterComponents': 'Host contains the following master components: {0}',
  'hosts.bulkOperation.deleteHosts.lastMasterComponents': 'Cluster does not contain any other instance of the following master components: {0}',
  'hosts.bulkOperation.deleteHosts.runningComponents': 'The following components are running and need to be stopped: {0}',
  'hosts.bulkOperation.deleteHosts.confirm.header': 'Confirm Bulk Delete Hosts',
  'hosts.bulkOperation.deleteHosts.confirm.delete': 'The following hosts will be deleted:',
  'hosts.bulkOperation.deleteHosts.cannot.delete1': 'The following hosts will be skipped (expand for reason):',
  'hosts.bulkOperation.deleteHosts.cannot.delete2': 'Selected hosts cannot be deleted (expand for reason)',
  'hosts.bulkOperation.deleteHosts.confirmation.header': 'Confirm Bulk Delete Hosts',
  'hosts.bulkOperation.deleteHosts.confirmation.body.msg1': '<b>Please note: </b>Once removed, Ambari will ignore future communications from these hosts. As part of the removal process, packages will not be removed, so please do not attempt to manually start the services on the host once they have been removed from Ambari. If you wish to re-add the hosts to the cluster, please completely clean the hosts before attempting to add them.',
  'hosts.bulkOperation.deleteHosts.confirmation.body.msg2': 'To ensure they are completely removed from Ambari database,',
  'hosts.bulkOperation.deleteHosts.confirmation.body.msg3': 'please make sure the Ambari Agent process is completely stopped on these hosts before proceeding.',
  'hosts.bulkOperation.deleteHosts.result.header': 'Delete Hosts',
  'hosts.bulkOperation.deleteHosts.result.body': 'The following hosts were successfully deleted:',
  'hosts.bulkOperation.confirmation.delete.component.cannot1': 'The following hosts will be skipped (expand for reason):',
  'hosts.bulkOperation.confirmation.delete.component.cannot2': '{0} cannot be deleted from the selected hosts:',
  'hosts.bulkOperation.confirmation.delete.component': '{0} will be deleted from the following hosts',
  'hosts.bulkOperation.confirmation.delete.component.minimum.body': 'Cannot delete. At least {0} {1} required',
  'hosts.bulkOperation.confirmation.delete.component.nothingToDo.notStopped': '{0} not Stopped on all selected hosts',
  'hosts.bulkOperation.confirmation.delete.component.nothingToDo.notInstalled': '{0} not installed in any of the selected hosts',
  'hosts.bulkOperation.confirmation.delete.component.skip': 'The following hosts will be skipped',
  'hosts.bulkOperation.delete.component.result.header': 'Delete Components',
  'hosts.bulkOperation.confirmation.delete.component.notStopped': '{0} not Stopped',
  'hosts.bulkOperation.confirmation.delete.component.notInstalled': '{0} not Installed',
  'hosts.bulkOperation.delete.component.result.body': ' was successfully removed from the following hosts:',
  'hosts.bulkOperation.delete.component.dryRun.message': '{0} could not be deleted from the following hosts(expand for reason)',
  'hosts.bulkOperation.delete.component.failed.header': 'Delete Component failed',
  'hosts.bulkOperation.delete.component.failed.body': 'Delete {0} component from {1} host failed',

  'hosts.selectHostsDialog.title': 'Select Configuration Group Hosts',
  'hosts.selectHostsDialog.message': 'Select hosts that should belong to this {0} Configuration Group. All hosts belonging to this group will have the same set of configurations.',
  'hosts.selectHostsDialog.filter.placeHolder': 'Filter...',
  'hosts.selectHostsDialog.selectedHostsLink': '{0} out of {1} hosts selected',
  'hosts.selectHostsDialog.message.warning': 'At least one host needs to be selected.',

  'hosts.host.serviceNotAvailable': 'Service not available on this host',

  'hosts.host.menu.logs': 'Logs',

  'hosts.host.menu.stackVersions': 'Versions',
  'hosts.host.stackVersions.table.allVersions': 'All Versions',
  'hosts.host.stackVersions.table.allNames': 'All Names',
  'hosts.host.stackVersions.table.noVersions': 'No versions',
  'hosts.host.stackVersions.table.filteredInfo': '{0} of {1} versions showing',
  'hosts.host.stackVersions.status.init': 'Uninstalled',
  'hosts.host.stackVersions.status.installed': 'Installed',
  'hosts.host.stackVersions.status.install_failed': 'Install Failed',
  'hosts.host.stackVersions.status.installing': 'Installing',
  'hosts.host.stackVersions.status.current': 'Current',
  'hosts.host.stackVersions.status.out_of_sync': 'Out of Sync',
  'hosts.host.stackVersions.status.upgrading': 'Upgrading',
  'hosts.host.stackVersions.status.upgrade_failed': 'Upgrade Failed',
  'hosts.host.stackVersions.outOfSync.tooltip': 'This version is Out of Sync on this host. Out of Sync can occur ' + 'the components on a host change after installing a version not in use. ' + 'Click Install to have Ambari install the packages for this version to get this host in sync.',
  'hosts.host.stackVersions.install.confirmation': 'You are about to install version <b>{0}</b> on this host.',

  'hosts.host.metrics.dataUnavailable': 'Data Unavailable',
  'hosts.host.metrics.cpu': 'CPU Usage',
  'hosts.host.metrics.cpu.displayNames.cpu_wio': 'CPU I/O Idle',
  'hosts.host.metrics.cpu.displayNames.cpu_idle': 'CPU Idle',
  'hosts.host.metrics.cpu.displayNames.cpu_nice': 'CPU Nice',
  'hosts.host.metrics.cpu.displayNames.cpu_aidle': 'CPU Boot Idle',
  'hosts.host.metrics.cpu.displayNames.cpu_system': 'CPU System',
  'hosts.host.metrics.cpu.displayNames.cpu_user': 'CPU User',
  'hosts.host.metrics.disk': 'Disk Usage',
  'hosts.host.metrics.disk.displayNames.disk_total': 'Total',
  'hosts.host.metrics.disk.displayNames.disk_free': 'Available',
  'hosts.host.metrics.load': 'Load',
  'hosts.host.metrics.load.displayNames.load_fifteen': '15 Minute Load',
  'hosts.host.metrics.load.displayNames.load_one': '1 Minute Load',
  'hosts.host.metrics.load.displayNames.load_five': '5 Minute Load',
  'hosts.host.metrics.memory': 'Memory Usage',
  'hosts.host.metrics.memory.displayNames.mem_shared': 'Shared',
  'hosts.host.metrics.memory.displayNames.swap_free': 'Swap',
  'hosts.host.metrics.memory.displayNames.mem_buffers': 'Buffers',
  'hosts.host.metrics.memory.displayNames.mem_free': 'Free',
  'hosts.host.metrics.memory.displayNames.mem_cached': 'Cached',
  'hosts.host.metrics.network': 'Network Usage',
  'hosts.host.metrics.network.displayNames.pkts_out': 'Packets Out',
  'hosts.host.metrics.network.displayNames.bytes_in': 'Bytes In',
  'hosts.host.metrics.network.displayNames.bytes_out': 'Bytes Out',
  'hosts.host.metrics.network.displayNames.pkts_in': 'Packets In',
  'hosts.host.metrics.processes': 'Processes',
  'hosts.host.metrics.processes.displayNames.proc_total': 'Total Processes',
  'hosts.host.metrics.processes.displayNames.proc_run': 'Processes Run',

  'hosts.host.summary.header': 'Summary',
  'hosts.host.summary.hostname': 'Hostname',
  'hosts.host.summary.agentHeartbeat': 'Heartbeat',
  'hosts.host.summary.agentHasJce': 'JCE Unlimited',
  'hosts.host.summary.hostMetrics': 'Host Metrics',
  'hosts.host.summary.hostLogMetrics': 'Host Log Metrics',
  'hosts.host.summary.addComponent': 'Add Component',
  'hosts.host.summary.currentVersion': 'Current Version',

  'hosts.host.details.upgradeFailed': 'Component upgrade failed',
  'hosts.host.details.hostActions': 'Host Actions',
  'hosts.host.details.needToRestart': 'Host needs {0} {1} restarted',
  'hosts.host.details.needToRestart.button': 'Restart',
  'hosts.host.details.deleteHost': 'Delete Host',
  'hosts.host.details.startAllComponents': 'Start All Components',
  'hosts.host.details.stopAllComponents': 'Stop All Components',
  'hosts.host.details.restartAllComponents': 'Restart All Components',
  'hosts.host.details.deleteHosts': 'Delete Hosts',
  'hosts.host.details.refreshConfigs': 'Refresh configs',
  'hosts.host.details.for.postfix': '{0} for host',
  'hosts.host.details.setRackId': 'Set Rack',
  'hosts.host.details.recoverHost': 'Recover Host',
  'host.host.details.installClients': 'Install clients',
  'host.host.details.reinstallClients': 'Reinstall clients',
  'host.host.details.checkHost': 'Check host',
  'host.host.details.downloadAllClients': 'All Clients On Host',

  'host.host.componentFilter.master': 'Master Components',
  'host.host.componentFilter.slave': 'Slave Components',
  'host.host.componentFilter.client': 'Client Components',
  'hosts.host.deleteComponent.popup.msg1': 'Are you sure you want to delete {0}?',
  'hosts.host.deleteComponent.popup.deleteJournalNodeMsg': 'You are about to open <i>Manage Journal Node Wizard</i>. Are you sure?',
  'hosts.host.deleteComponent.popup.warning': '<b>WARNING!</b> Delete the last <i>{0}</i> component in the cluster?</br>Deleting the last component in the cluster could result in permanent loss of service data.',
  'hosts.host.deleteComponent.popup.confirm': 'Confirm Delete',
  'hosts.host.installComponent.popup.confirm': 'Confirm Install',
  'hosts.host.installComponent.msg': 'Are you sure you want to install {0}?',
  'hosts.host.addComponent.msg': 'Are you sure you want to add {0}?',
  'hosts.host.delete.componentsRequireReconfigure': 'Deleting host with {0} may reconfigure such properties:',
  'host.host.addComponent.popup.dependedComponents.body': '{0} requires {1} to be installed along with it on the same host. Please add them first and then try adding {0}',
  'host.host.addComponent.popup.dependedComponents.header': 'Component dependencies',
  'host.host.addComponent.popup.clients.dependedComponents.body': '{0} require {1} to be installed along with them on the same host. Please add them first',
  'hosts.host.zooKeeper.configs.save.note': 'This configuration is created by ambari while installing/deleting zookeeper component on a host',
  'hosts.host.addComponent.securityNote': 'You are running your cluster in secure mode. You must set up the keytab for {0} on {1} before you proceed. Otherwise, the component will not be able to start properly.',
  'hosts.host.addComponent.popup.confirm': 'Confirm Add',
  'hosts.host.manualKerberosWarning': '<strong>Because Kerberos has been manually installed on the cluster, you will have to create/distribute principals and keytabs when this operation is finished.</strong>',
  'hosts.host.deleteComponent.popup.deleteNimbus': 'Deleting <i>Storm Nimbus</i> will reconfigure <b>nimbus.seeds</b>, <b>topology.min.replication.count</b>, <b>topology.max.replication.wait.time.sec</b> properties if they are defined.',
  'hosts.host.storm.configs.save.note': 'This configuration is created by ambari while installing/deleting storm component on a host',
  'hosts.host.datanode.decommission': 'Decommission DataNode',
  'hosts.host.datanode.recommission': 'Recommission DataNode',
  'hosts.host.nodemanager.decommission': 'Decommission NodeManager',
  'hosts.host.nodemanager.recommission': 'Recommission NodeManager',
  'hosts.host.tasktracker.decommission': 'Decommission TaskTracker',
  'hosts.host.tasktracker.recommission': 'Recommission TaskTracker',
  'hosts.host.tasktracker.restart': 'Restart TaskTracker',
  'hosts.host.regionserver.decommission.batch1': 'Decommission RegionServer - Turn drain mode on',
  'hosts.host.regionserver.decommission.batch2': 'Decommission RegionServer - Stop RegionServer',
  'hosts.host.regionserver.decommission.batch3': 'Decommission RegionServer - Turn drain mode off',
  'hosts.host.hbase_regionserver.recommission': 'Recommission RegionServer',
  'hosts.host.hbase_regionserver.decommission': 'Decommission RegionServer',
  'hosts.host.hbase_regionserver.decommission.warning': 'Last RegionServer can\'t be decommissioned',
  'hosts.host.decommissioned': 'Decommissioned',
  'hosts.host.decommissioning': 'Decommissioning',
  'hosts.host.addComponent.JOURNALNODE': 'You are about to open <i>Manage Journal Node Wizard</i>. Are you sure?',
  'hosts.host.deleteComponent.popup.deleteHiveMetastore': 'Deleting <i>Hive Metastore</i> will reconfigure such properties:<ul><li>hive.metastore.uris</li><li>templeton.hive.properties</li></ul>',
  'hosts.host.deleteComponent.popup.deleteWebHCatServer': 'You are about to delete <i>WebHCat Server</i>. Are you sure?',
  'hosts.host.configs.save.note': 'This configuration is created by ambari while installing/deleting {0} component on a host',

  'hosts.component.passive.implied.host.mode.tooltip': 'Cannot Turn Off Maintenance Mode because Host is in Maintenance Mode',
  'hosts.component.passive.implied.service.mode.tooltip': 'Cannot Turn Off Maintenance Mode because {0} is in Maintenance Mode',
  'hosts.component.passive.mode': 'Component is in Maintenance Mode',
  'hosts.component.passive.short.mode': 'Maintenance Mode',
  'hosts.host.passive.mode': 'Host is in Maintenance Mode',
  'hosts.host.alert.noAlerts': 'No alerts',
  'hosts.host.alert.noAlerts.message': 'There are no alerts for this host.',
  'hosts.host.healthStatus.heartBeatNotReceived': 'The server has not received a heartbeat from this host for more than 3 minutes.',
  'hosts.host.healthStatus.mastersDown': "The following master components are down:\n",
  'hosts.host.healthStatus.slavesDown': "The following slave components are down:\n",
  'hosts.host.healthStatus.allUp': 'All components are up',
  'hosts.host.healthStatusCategory.green': "Healthy",
  'hosts.host.healthStatusCategory.red': "Master Down",
  'hosts.host.healthStatusCategory.orange': "Slave Down",
  'hosts.host.healthStatusCategory.yellow': "Lost Heartbeat",
  'hosts.host.alerts.label': 'Alerts',
  'hosts.host.maintainance.allComponents.context': 'All Host Components',
  'hosts.host.maintainance.stopAllComponents.context': 'Stop All Host Components',
  'hosts.host.maintainance.startAllComponents.context': 'Start All Host Components',
  'hosts.host.maintainance.reinstallFailedComponents.context': 'Reinstall Failed Components',
  'hosts.host.maintainance.removeFailedComponents.context': 'Remove Failed Components',
  'hosts.host.recover.initAllComponents.context': 'Init All Host Components',
  'hosts.host.recover.installAllComponents.context': 'Install All Host Components',
  'hosts.host.recover.regenerateKeytabs.context': 'Regenerate keytabs',
  'hosts.host.alerts.st': '&nbsp;!&nbsp;',
  'hosts.decommission.popup.body': 'Are you sure?',
  'hosts.decommission.popup.header': 'Confirmation',
  'hosts.decommission.tooltip.warning': 'Cannot {0} since {1} is not running',
  'hosts.delete.popup.body': 'Are you sure you want to delete host <i>{0}</i>?',
  'hosts.delete.popup.body.msg1': 'By removing this host, Ambari will ignore future communications from this host. Software packages will not be removed from the host. The components on the host should not be restarted. If you wish to readd this host to the cluster, be sure to clean the host.',
  'hosts.delete.popup.body.msg3': 'If this host was hosting a Zookeeper Server, the Zookeeper Service should be restarted. Go to the <i>Services</i> page.',
  'hosts.delete.popup.body.msg4': '<b>WARNING!</b> Delete the last <i>{0}</i> component[s] in the cluster?</br>Deleting the last components in the cluster could result in permanent loss of service data.',
  'hosts.delete.popup.body.msg5': '<b>WARNING!</b> The agent is still heartbeating so the Host will still exist in the database.',
  'hosts.delete.popup.body.msg6': 'To completely delete the Host, first stop ambari-agent on it.',
  'hosts.delete.popup.body.msg7': '<b>WARNING!</b> {0} should be decommissioned first to prevent data loss.',
  'hosts.delete.popup.body.msg.unknownComponents': 'This host does not appear to be online and Ambari communication with the Agent has been lost.',
  'hosts.delete.popup.header': 'Confirmation',
  'hosts.delete.popup.title': 'Delete Host',
  'hosts.delete.popup.unknownComponents': 'The following components have unknown status:',
  'hosts.recover.popup.title': 'Confirmation',
  'hosts.recover.popup.body': 'This action will completely re-install all components on this host. <strong>Are you sure you want to continue?<strong>',
  'hosts.recover.error.popup.title': 'Cannot Recover Host',
  'hosts.recover.error.popup.body': 'This host cannot be recovered unless every component is Stopped. Please stop all components and try again.',
  'hosts.cant.do.popup.title': 'Unable to Delete Host',
  'hosts.cant.do.popup.masterList.body': 'This host cannot be deleted since it has the following master components:',
  'hosts.cant.do.popup.masterList.body.end': 'To delete this host, you must first move all the master components listed above.',
  'hosts.cant.do.popup.nonDeletableList.body': 'Deletion of the following {0} components is not supported. ',
  'hosts.cant.do.popup.runningList.body': 'This host cannot be deleted since the following components are running:',
  'hosts.cant.do.popup.runningList.body.end': 'To delete this host, you must first stop all the running components listed above. ' + 'If this host has a {0}, it should be decommissioned first to prevent data loss.',
  'hosts.cant.do.popup.lastMasterList.body': 'This host cannot be deleted since the following master components do not have any other instances:',
  'hosts.cant.do.popup.lastMasterList.body.end': 'To delete this host, you must first add all the components listed above to other hosts',
  'hosts.add.header': 'Add Host Wizard',
  'hosts.add.exit.header': 'Exit',
  'hosts.add.exit.body': 'Do you really want to exit Add Host Wizard?',
  'hosts.assignRack': 'Assign Rack',
  'hosts.checkHost.popup': 'Are you sure you want to <b>Check host</b> for {0}?',
  'hosts.passiveMode.popup': 'Are you sure you want to <b>Turn {0} Maintenance Mode</b> for {1}?',
  'hosts.passiveMode.popup.version.mismatch': '{0} has components from a stack which is not current. Before bringing this host out of maintenance mode, it is recommended that you upgrade its components to {1}',
  'hosts.passiveMode.popup.version.mismatch.multiple': 'Some hosts have components from a stack which is not current. Before bringing these hosts out of maintenance mode, it is recommended that you upgrade their components to {0}',
  'hosts.combo.search.placebolder': 'Filter by host and component attributes or search by keyword ...',
  'hosts.combo.search.invalidCategory': 'is not a valid category.',
  'charts.horizon.chart.showText': 'show',
  'charts.horizon.chart.hideText': 'hide',
  'charts.horizon.chart.attributes.cpu': 'CPU',
  'charts.horizon.chart.attributes.memory': 'Memory',
  'charts.horizon.chart.attributes.network': 'Network',
  'charts.horizon.chart.attributes.io': 'I/O',

  'charts.heatmap.selectMetric': 'Select Metric',

  'charts.heatmap.category.host': 'Host',
  'charts.heatmap.item.host.memory': 'Memory Used',
  'charts.heatmap.item.host.disk': 'Disk Space Used',
  'charts.heatmap.item.host.process': 'Total Running Processes',
  'charts.heatmap.category.hdfs': 'HDFS',
  'charts.heatmap.category.yarn': 'YARN',
  'charts.heatmap.category.hbase': 'HBase',
  'charts.heatmap.unknown': 'Unknown',
  'charts.heatmap.label.notApplicable': 'Not Applicable',
  'charts.heatmap.label.invalidData': 'Invalid data',
  'charts.heatmap.label.notAvailable': 'Data Not Available',
  'metric.notFound': 'no items found',
  'metric.default': 'combined',
  'metric.cpu': 'cpu',
  'metric.memory': 'disk used',
  'metric.network': 'network',
  'metric.io': 'io',
  'metric.more': 'more',
  'metric.more.cpu': 'CPU',
  'metric.more.disk': 'Disk',
  'metric.more.load': 'Load',
  'metric.more.memory': 'Memory',
  'metric.more.network': 'Network',
  'metric.more.process': 'Process',

  'dashboard.clusterMetrics': 'Cluster Metrics',

  'dashboard.clusterMetrics.cpu': 'CPU Usage',
  'dashboard.clusterMetrics.cpu.displayNames.idle': 'Idle',
  'dashboard.clusterMetrics.load': 'Cluster Load',
  'dashboard.clusterMetrics.memory': 'Memory Usage',
  'dashboard.clusterMetrics.network': 'Network Usage',

  'dashboard.widgets.title': 'Metrics',
  'dashboard.heatmaps.title': 'Heatmaps',
  'dashboard.button.switch': 'Switch to classic dashboard',
  'dashboard.button.switchShort': 'Switch',
  'dashboard.button.reset': 'Reset all widgets to default ',
  'dashboard.widgets.actions.title': 'Metric Actions',
  'dashboard.widgets.create': 'Create Widget',
  'dashboard.widgets.actions.browse': 'Browse Widgets',
  'dashboard.widgets.layout.import': 'Import a layout',
  'dashboard.widgets.layout.save': 'Save a layout',
  'dashboard.widgets.addButton.tooltip': 'Add Widget',
  'dashboard.widgets.browser.header': 'Widget Browser',
  'dashboard.widgets.browser.menu.shared': 'Shared',
  'dashboard.widgets.browser.menu.mine': 'Mine',
  'dashboard.widgets.browser.noWidgets': 'No widgets to display',
  'dashboard.widgets.browser.footer.checkbox': 'Show only my widgets',
  'dashboard.widgets.browser.action.add': 'Add',
  'dashboard.widgets.browser.action.added': 'Added',
  'dashboard.widgets.browser.action.delete': 'Delete',
  'dashboard.widgets.browser.action.unshare': 'Unshare',
  'dashboard.widgets.browser.action.share': 'Share',
  'dashboard.widgets.browser.action.share.confirmation': 'You are about to make this a shared widget. All cluster operators will be able to modify or delete this widget. Are you sure you want to share this widget?',
  'dashboard.widgets.browser.shareIcon.tooltip': 'Shared',
  'dashboard.widgets.browser.action.delete.shared.bodyMsg': 'You are about to permanently delete the <b>{0}</b> widget. ' + 'This widget is a shared widget and this operation will delete the widget from the shared widget library and remove the ' + 'widget from all users.<br /> <br /> <b>Are you sure you wish to permanently delete this shared widget?</b>',
  'dashboard.widgets.browser.action.delete.mine.bodyMsg': 'You are about to permanently delete the <b>{0}</b> widget. ' + 'This operation will delete the widget from your widget library.<br /><br /><b>Are you sure you wish to permanently ' + 'delete this widget?</b>',
  'dashboard.widgets.browser.action.delete.btnMsg': 'Confirm Delete',

  'dashboard.widgets.NameNodeHeap': 'NameNode Heap',
  'dashboard.widgets.NameNodeCpu': 'NameNode CPU WIO',
  'dashboard.widgets.HDFSDiskUsage': 'HDFS Disk Usage',
  'dashboard.widgets.HDFSDiskUsage.DFSused': 'DFS used',
  'dashboard.widgets.HDFSDiskUsage.nonDFSused': 'non DFS used',
  'dashboard.widgets.HDFSDiskUsage.remaining': 'remaining',
  'dashboard.widgets.HDFSDiskUsage.info': '{0} ({1}%)',
  'dashboard.widgets.DataNodeUp': 'DataNodes Live',
  'dashboard.widgets.SuperVisorUp': 'Supervisors Live',
  'dashboard.widgets.FlumeAgentUp': 'Flume Live',
  'dashboard.widgets.NameNodeRpc': 'NameNode RPC',
  'dashboard.widgets.nothing': 'No Widget to Add',
  'dashboard.widgets.NameNodeUptime': 'NameNode Uptime',
  'dashboard.widgets.HDFSLinks': 'HDFS Links',
  'dashboard.widgets.HDFSLinks.activeNameNode': 'Active NameNode',
  'dashboard.widgets.HDFSLinks.standbyNameNode': 'Standby NameNode',
  'dashboard.widgets.HDFSLinks.standbyNameNodes': '2 Standby NameNodes',
  'dashboard.widgets.HBaseLinks': 'HBase Links',
  'dashboard.widgets.HBaseAverageLoad': 'HBase Ave Load',
  'dashboard.widgets.HBaseMasterHeap': 'HBase Master Heap',
  'dashboard.widgets.HBaseRegionsInTransition': 'Region In Transition',
  'dashboard.widgets.HBaseMasterUptime': 'HBase Master Uptime',
  'dashboard.widgets.ResourceManagerHeap': 'ResourceManager Heap',
  'dashboard.widgets.ResourceManagerUptime': 'ResourceManager Uptime',
  'dashboard.widgets.NodeManagersLive': 'NodeManagers Live',
  'dashboard.widgets.YARNMemory': 'YARN Memory',
  'dashboard.widgets.YARNLinks': 'YARN Links',
  'dashboard.widgets.YarnContainers': 'YARN Containers',
  'dashboard.widgets.error.invalid': 'Invalid! Enter a number between {0} - {1}',
  'dashboard.widgets.error.smaller': 'Threshold 1 should be smaller than threshold 2!',
  'dashboard.widgets.HawqSegmentUp': 'HAWQ Segments Live',
  'dashboard.widgets.PxfUp': 'PXF Agents Live',
  'dashboard.widgets.PXFAgents': 'PXF Agents',
  'dashboard.widgets.nameSpace': 'Namespace',

  'dashboard': {
    'widgets': {
      'popupHeader': 'Customize Widget',
      'hintInfo': {
        'common': 'Edit the percentage thresholds to change the color of current pie chart. <br />Enter two numbers between 0 to {0}',
        'hint1': 'Edit the percentage of thresholds to change the color of current widget. <br />Assume all components UP is 100, and all DOWN is 0. <br /> So enter two numbers between 0 to {0}',
        'hint2': 'Edit the thresholds to change the color of current widget.<br /><br />So enter two numbers larger than 0.',
        'hint3': 'Edit the thresholds to change the color of current widget.<br />The unit is milli-second. <br />So enter two numbers larger than 0. ',
        'hint4': 'Edit the threshold value representing the number of {0} that are down to change the color of the widget. <br />If the number of {0} that are down is greater than the threshold, the widget will show warning in Red. <br />Choose a number between 0 and {1}. '
      }
    }
  },

  'dashboard.services': 'Services',
  'dashboard.services.hosts': 'Hosts',
  'dashboard.services.uptime': '{0}',
  'dashboard.services.summary.slaves.live': 'Live',
  'dashboard.services.hdfs.summary': '{0} of {1} nodes live, {2}% capacity used',
  'dashboard.services.hdfs.summary.components': 'Components',
  'dashboard.services.hdfs.summary.service-metrics': 'Service Metrics',
  'dashboard.services.hdfs.nanmenode': 'NameNode',
  'dashboard.services.hdfs.snanmenode': 'Secondary NameNode',
  'dashboard.services.hdfs.journalnodes': 'JournalNodes',
  'dashboard.services.hdfs.capacity': 'HDFS Disk Usage',
  'dashboard.services.hdfs.capacity.dfsUsed': 'Disk Usage (DFS Used)',
  'dashboard.services.hdfs.capacity.nonDfsUsed': 'Disk Usage (Non DFS Used)',
  'dashboard.services.hdfs.capacity.remaining': 'Disk Remaining',
  'dashboard.services.hdfs.capacityUsed': '{0} / {1}',
  'dashboard.services.hdfs.capacityUsedPercent': '{0}%',
  'dashboard.services.hdfs.totalFilesAndDirs': 'Total Files + Directories',
  'dashboard.services.hdfs.datanodes': 'DataNodes',
  'dashboard.services.hdfs.datanodecounts': 'DataNodes Status',
  'dashboard.services.hdfs.nfsgateways': 'NFSGateways',
  'dashboard.services.hdfs.version': 'Version',
  'dashboard.services.hdfs.nameNodeWebUI': 'NameNode Web UI',
  'dashboard.services.hdfs.nodes.live': 'live',
  'dashboard.services.hdfs.nodes.dead': 'dead',
  'dashboard.services.hdfs.nodes.decom': 'decommissioning',
  'dashboard.services.hdfs.nodes.uptime': 'NameNode Uptime',
  'dashboard.services.hdfs.nodes.heap': 'NameNode Heap',
  'dashboard.services.hdfs.nodes.heapUsedPercent': '{0}%',
  'dashboard.services.hdfs.nodes.heapUsed': '{0} / {1}',

  'dashboard.services.hdfs.chart.label': 'Capacity (Used/Total)',
  'dashboard.services.hdfs.blockErrors.total': 'total',
  'dashboard.services.hdfs.blockErrors.corrupt': 'corrupt replica',
  'dashboard.services.hdfs.blockErrors.replicated': 'under replicated',
  'dashboard.services.hdfs.blockErrors.missing': 'missing',
  'dashboard.services.hdfs.datanode.status.tooltip.live': 'This is the number of DataNodes that are live as reported from ' + 'the NameNode. Even if a DataNode process is up, NameNode might see the status as dead ' + 'if the DataNode is not communicating with the NameNode as expected. This can be due situations ' + 'such as a network issue or a hanging DataNode process due to excessive garbage collection.',
  'dashboard.services.hdfs.datanode.status.tooltip.dead': 'This is the number of DataNodes that are dead as reported from ' + 'the NameNode. Even if a DataNode process is up, NameNode might see the status as dead ' + 'if the DataNode is not communicating with the NameNode as expected. This can be due situations ' + 'such as a network issue or a hanging DataNode process due to excessive garbage collection.',
  'dashboard.services.hdfs.datanode.status.tooltip.decommission': 'This is the number of DataNodes that are currently ' + 'Decommissioning as reported from the NameNode. If there are not enough other DataNodes ' + 'in the cluster to create the configured number of block replicas based on the dfs.replication ' + 'property (typically 3), a DataNode can get stuck in decommissioning state until ' + 'more DataNodes become available to the NameNode.',

  'dashboard.services.yarn.summary': '{0} of {1} nodes live',
  'dashboard.services.yarn.resourceManager': 'ResourceManager',
  'dashboard.services.yarn.nodeManagers': 'NodeManagers',
  'dashboard.services.yarn.nodeManager': 'NodeManager',
  'dashboard.services.yarn.nodes.heapUsed': '{0} / {1}',
  'dashboard.services.yarn.nodes.heapUsedPercent': '{0}%',
  'dashboard.services.yarn.clients': 'YARN Clients',
  'dashboard.services.yarn.client': 'YARN Client',
  'dashboard.services.yarn.resourceManager.uptime': 'ResourceManager Uptime',
  'dashboard.services.yarn.resourceManager.active': 'Active ResourceManager',
  'dashboard.services.yarn.resourceManager.standby': 'Standby ResourceManager',
  'dashboard.services.resourceManager.nodes.heap': 'ResourceManager Heap',
  'dashboard.services.yarn.nodeManagers.status': 'NodeManagers Status',
  'dashboard.services.yarn.nodeManagers.status.active': 'active',
  'dashboard.services.yarn.nodeManagers.status.lost': 'lost',
  'dashboard.services.yarn.nodeManagers.status.unhealthy': 'unhealthy',
  'dashboard.services.yarn.nodeManagers.status.rebooted': 'rebooted',
  'dashboard.services.yarn.nodeManagers.status.decommissioned': 'decommissioned',
  'dashboard.services.yarn.nodeManagers.status.tooltip.active': 'This is the number of NodeManagers that are active as reported from the ResourceManager',
  'dashboard.services.yarn.nodeManagers.status.tooltip.lost': 'This is the number of NodeManagers that are lost as reported from the ResourceManager. ' + 'Even if a NodeManager process is up, the ResourceManager might see the status as lost if the NodeManager is not communicating with the ' + 'ResourceManager as expected. This can be due to situations such as a network issue or a hanging NodeManager process due to excessive garbage collection.',
  'dashboard.services.yarn.nodeManagers.status.tooltip.unhealthy': 'This is the number of NodeManagers that are unhealthy as reported from the ResourceManager.',
  'dashboard.services.yarn.nodeManagers.status.tooltip.rebooted': 'This is the number of NodeManagers that are in the rebooted state as reported from the ResourceManager.',
  'dashboard.services.yarn.nodeManagers.status.tooltip.decommissioned': 'This is the number of NodeManagers that are decommissioned as reported from the ResourceManager.',
  'dashboard.services.yarn.containers': 'Containers',
  'dashboard.services.yarn.containers.allocated': 'allocated',
  'dashboard.services.yarn.containers.pending': 'pending',
  'dashboard.services.yarn.containers.reserved': 'reserved',
  'dashboard.services.yarn.apps': 'Applications',
  'dashboard.services.yarn.apps.submitted': 'submitted',
  'dashboard.services.yarn.apps.running': 'running',
  'dashboard.services.yarn.apps.pending': 'pending',
  'dashboard.services.yarn.apps.completed': 'completed',
  'dashboard.services.yarn.apps.killed': 'killed',
  'dashboard.services.yarn.apps.failed': 'failed',

  'dashboard.services.yarn.memory': 'Cluster Memory',
  'dashboard.services.yarn.memory.used': 'used',
  'dashboard.services.yarn.memory.reserved': 'reserved',
  'dashboard.services.yarn.memory.available': 'available',
  'dashboard.services.yarn.queues': 'Queues',
  'dashboard.services.yarn.queues.msg': '{0} Queues',
  'dashboard.services.hawq.hawqSegments': 'HAWQ Segments',
  'dashboard.services.pxf.pxfHosts': 'PXF',

  'dashboard.services.flume.summary.title': 'Flume installed on {0} host{1} ({2} agent{3})',
  'dashboard.services.flume.summary.configure': 'Configure Agents',
  'dashboard.services.flume.agentsLabel': 'Flume',
  'dashboard.services.flume.agentLabel': 'Flume Component',
  'dashboard.services.flume.channels': 'Channels',
  'dashboard.services.flume.sources': 'Sources',
  'dashboard.services.flume.sinks': 'Sinks',
  'dashboard.services.flume.agent': 'Agent',

  'dashboard.services.hbase.summary': '{0} region servers with {1} average load',
  'dashboard.services.hbase.masterServer': 'HBase Master',
  'dashboard.services.hbase.masterServer.active': 'Active HBase Master',
  'dashboard.services.hbase.masterServer.standby': 'Standby HBase Master',
  'dashboard.services.hbase.noMasterServer': 'No Active Master',
  'dashboard.services.hbase.masterServerHeap': 'Master Heap',
  'dashboard.services.hbase.masterServerHeap.summary': '{0} / {1}',
  'dashboard.services.hbase.masterServerHeap.percent': '{0}%',
  'dashboard.services.hbase.masterServerUptime': 'Master Server Uptime',
  'dashboard.services.hbase.averageLoad': 'Average Load',
  'dashboard.services.hbase.averageLoadPerServer': '{0} regions per RegionServer',
  'dashboard.services.hbase.regionServers': 'RegionServers',
  'dashboard.services.hbase.regionServersSummary': '{0} live / {1} total',
  'dashboard.services.hbase.phoenixServers': 'Phoenix Query Servers',
  'dashboard.services.hbase.phoenixServersSummary': '{0} live / {1} total',
  'dashboard.services.hbase.chart.label': 'Request Count',
  'dashboard.services.hbase.masterWebUI': 'Master Web UI',
  'dashboard.services.hbase.regions.transition': 'Regions In Transition',
  'dashboard.services.hbase.masterStarted': 'Master Started',
  'dashboard.services.hbase.masterActivated': 'Master Activated',

  'dashboard.services.ranger.rangerTagsyncs': 'Ranger Tagsyncs',

  'dashboard.services.components.started': 'started',
  'dashboard.services.components.stopped': 'stopped',
  'dashboard.services.components.total': 'in total',

  'dashboard.services.hive.clients': 'Hive Clients',
  'dashboard.services.hive.client': 'Hive Client',
  'dashboard.services.hive.metastore': 'Hive Metastore',
  'dashboard.services.hive.server2': 'HiveServer2',
  'dashboard.services.hive.server2interactive': 'HiveServer2 Interactive',

  'dashboard.services.oozie.clients': 'Oozie Clients',
  'dashboard.services.oozie.client': 'Oozie Client',
  'dashboard.services.storm.supervisor': 'Supervisor',
  'dashboard.services.storm.supervisors': 'Supervisors',

  'dashboard.services.configs.popup.stopService.header': 'Stop service',
  'dashboard.services.configs.popup.stopService.body': 'Service needs to be stopped for reconfiguration',
  'dashboard.services.configs.popup.restartService.header': 'Restart service',
  'dashboard.services.configs.popup.restartService.body': 'Service needs to be restarted for reconfiguration',

  'dashboard.services.zookeeper.server': 'ZooKeeper Server',

  'dashboard.configHistory.title': 'Config History',
  'dashboard.configHistory.table.version.title': 'Service',
  'dashboard.configHistory.table.configGroup.title': 'Config Group',
  'dashboard.configHistory.table.created.title': 'Created',
  'dashboard.configHistory.table.configGroup.default': 'default',
  'dashboard.configHistory.table.empty': 'No history to display',
  'dashboard.configHistory.table.notes.default': 'Initial configurations for {0}',
  'dashboard.configHistory.table.notes.addService': 'Configuration updated while adding service',
  'dashboard.configHistory.table.notes.no': '',
  'dashboard.configHistory.table.version.versionText': 'V{0}',
  'dashboard.configHistory.table.version.prefix': 'V',
  'dashboard.configHistory.table.current.tooltip': 'Current config for {0}:{1}',
  'dashboard.configHistory.table.restart.tooltip': 'Restart required',
  'dashboard.configHistory.table.filteredHostsInfo': '{0} of {1} versions showing',
  'dashboard.configHistory.info-bar.authoredOn': 'authored on',
  'dashboard.configHistory.info-bar.changesToHandle': 'Changes to handle',
  'dashboard.configHistory.info-bar.showMore': 'Show more',
  'dashboard.configHistory.info-bar.save.popup.title': 'Save Configuration',
  'dashboard.configHistory.info-bar.makeCurrent.popup.title': 'Make Current Confirmation',
  'dashboard.configHistory.info-bar.save.popup.placeholder': 'What did you change?',
  'dashboard.configHistory.info-bar.save.popup.warningForPasswordChange': 'This configuration change includes a password change. The password change will be saved but for security purposes, password changes will not be shown in configuration version comparisons.',
  'dashboard.configHistory.info-bar.save.popup.notesForPasswordChange': 'Password change',
  'dashboard.configHistory.info-bar.revert.button': 'Make Current',
  'dashboard.configHistory.info-bar.revert.versionButton': 'Make {0} Current',
  'dashboard.configHistory.info-bar.view.button.disabled': 'You are currently viewing this version.',
  'dashboard.configHistory.info-bar.compare.button.disabled': 'You cannot compare against the same version.',
  'dashboard.configHistory.info-bar.revert.button.disabled': 'This is the current version.',
  'dashboard.configHistory.info-bar.compare.tooltip': 'Compare this version with current',

  'timeRange.presets.1hour': '1h',
  'timeRange.presets.12hour': '12h',
  'timeRange.presets.1day': '1d',
  'timeRange.presets.1week': '1wk',
  'timeRange.presets.1month': '1mo',
  'timeRange.presets.1year': '1yr',

  'tableView.filters.all': 'All',
  'tableView.filters.filtered': 'Filtered',
  'tableView.filters.clearFilters': 'Clear filters',
  'tableView.filters.itemsPerPage': 'Items per page:',
  'tableView.filters.paginationInfo': '{0} - {1} of {2}',
  'tableView.filters.clearAllFilters': 'clear filters',
  'tableView.filters.showAll': 'Show All',
  'tableView.filters.filteredConfigVersionInfo': '{0} of {1} versions showing',
  'tableView.filters.filteredAlertInstancesInfo': '{0} of {1} instances showing',
  'tableView.filters.filteredLogsInfo': '{0} of {1} logs showing',

  'rollingrestart.dialog.title': 'Restart {0}',
  'rollingrestart.dialog.primary': 'Trigger Rolling Restart',
  'rollingrestart.notsupported.hostComponent': 'Rolling restart not supported for {0} components',
  'rollingrestart.dialog.msg.restart': 'This will restart a specified number of {0} at a time.',
  'rollingrestart.dialog.msg.noRestartHosts': 'There are no {0} to do rolling restarts',
  'rollingrestart.dialog.msg.maintainance': 'Note: {0} {1} in Maintenance Mode will not be restarted',
  'rollingrestart.dialog.msg.componentsAtATime': '{0} at a time',
  'rollingrestart.dialog.msg.timegap.prefix': 'Wait ',
  'rollingrestart.dialog.msg.timegap.suffix': 'seconds between batches ',
  'rollingrestart.dialog.msg.toleration.prefix': 'Tolerate up to ',
  'rollingrestart.dialog.msg.toleration.suffix': 'restart failures',
  'rollingrestart.dialog.err.invalid.batchsize': 'Invalid restart batch size: {0}',
  'rollingrestart.dialog.err.invalid.waitTime': 'Invalid wait time between batches: {0}',
  'rollingrestart.dialog.err.invalid.toleratesize': 'Invalid failure toleration count: {0}',
  'rollingrestart.dialog.warn.datanode.batch.size': 'Restarting more than one DataNode at a time is not recommended. Doing so can lead to data unavailability and/or possible loss of data being actively written to HDFS.',
  'rollingrestart.dialog.msg.serviceNotInMM': 'Note: This will trigger alerts. To suppress alerts, turn on Maintenance Mode for {0} prior to triggering a rolling restart',
  'rollingrestart.dialog.msg.staleConfigsOnly': 'Only restart {0} with stale configs',
  'rollingrestart.rest.context': 'Rolling Restart of {0}s - batch {1} of {2}',
  'rollingrestart.context.allOnSelectedHosts': 'Restart all components on the selected hosts',
  'rollingrestart.context.configs.allOnSelectedHosts': 'Refresh all configs on the selected hosts',
  'rollingrestart.context.allForSelectedService': 'Restart all components for {0}',
  'rollingrestart.context.allWithStaleConfigsForSelectedService': 'Restart all components with Stale Configs for {0}',
  'rollingrestart.context.ClientOnSelectedHost': 'Restart {0} on {1}',
  'rollingrestart.context.allWithStaleConfigsOnSelectedHost': 'Restart components with Stale Configs on {0}',
  'rollingrestart.context.allOnSelectedHost': 'Restart all components on {0}',
  'rollingrestart.context.selectedComponentOnSelectedHost': 'Restart {0}',
  'rollingrestart.context.default': 'Restart components',

  'rolling.command.context': 'Rolling set {0} to state "{1}" - batch {2} of {3}',
  'rolling.nothingToDo.header': 'Nothing to do',
  'rolling.nothingToDo.body': '{0} on selected hosts are already in selected state or in Maintenance Mode.',

  'widget.type.gauge.description': 'A view to display metrics that can be expressed in percentage.',
  'widget.type.number.description': 'A view to display metrics that can be expressed as a single number with optional unit field.',
  'widget.type.graph.description': 'A view to display metrics that can be expressed in line graph or area graph over a time range.',
  'widget.type.template.description': 'A view to display metric value along with a templated text.',
  'widget.create.wizard.header': 'Create Widget',
  'widget.create.wizard.step1.header': 'Select Type',
  'widget.create.wizard.step1.body.text': 'What type of widget do you want to create?',
  'widget.create.wizard.step1.body.choose.tooltip': 'Click to select',
  'widget.create.wizard.step2.header': 'Metrics and Expression',
  'widget.create.wizard.step2.template.header': 'Template',
  'widget.create.wizard.step2.body.text': 'Define the expression with any metrics and valid operators. </br>Use parentheses when necessary.',
  'widget.create.wizard.step2.body.template': 'Define the template with any number of expressions and any string. An expression can be referenced from a template by enclosing its name with double curly braces.',
  'widget.create.wizard.step2.body.warning': 'Note: Valid operators are +, -, *, /',
  'widget.create.wizard.step2.body.invalid.label': 'Invalid name. Only alphanumerics, underscores, hyphens, percentage and spaces are allowed.',
  'widget.create.wizard.step2.body.invalid.expression': 'Expression is not complete or is invalid!',
  'widget.create.wizard.step2.body.invalid.expression2': 'Invalid expression!',
  'widget.create.wizard.step2.body.invalid.msg2': 'Only numbers or operators are allowed in this field.',
  'widget.create.wizard.step2.body.template.invalid.msg': 'Invalid expression name existed. Should use name "Expression#" with double curly braces.',
  'widget.create.wizard.step2.addExpression': 'Add Expression',
  'widget.create.wizard.step2.addDataset': 'Add data set',
  'widget.create.wizard.step2.body.gauge.overflow.warning': 'Overflowed! Gauge can only display number between 0 and 1.',
  'widget.create.wizard.step2.allComponents': 'All {0}',
  'widget.create.wizard.step2.activeComponents': 'Active {0}',
  'widget.create.wizard.step2.noMetricFound': 'No metric found',
  'widget.create.wizard.step3.widgetName': 'Name',
  'widget.create.wizard.step3.sharing': 'Sharing',
  'widget.create.wizard.step3.sharing.msg': 'Share this widget in the widget library',
  'widget.create.wizard.step3.header': 'Name and Description',
  'widget.create.wizard.step3.name.invalid.msg': 'Widget name is too long. Please enter a widget name less than 129 characters.',
  'widget.create.wizard.step3.name.invalidCharacter.msg': 'Invalid widget name. Only alphanumerics, underscores, hyphens, percentage and spaces are allowed.',
  'widget.create.wizard.step3.description.invalid.msg': 'Description is too long. Please enter a description less than 2049 characters.',
  'widget.create.wizard.step3.description.invalidCharacter.msg': 'Invalid input. Only alphanumerics, underscores, hyphens, percentage and spaces are allowed.',

  'widget.edit.wizard.header': 'Edit Widget',

  'widget.clone.body': 'Are you sure you want to clone current widget <b>{0}</b>?',
  'widget.edit.body': 'This is a shared widget and the edits are going to change this widget for all users. Would you prefer to Clone the widget instead?',
  'widget.edit.button.primary': 'Edit Shared',
  'widget.edit.button.secondary': 'Clone and Edit',

  'dashboard.widgets.wizard.step2.dataSeries': 'Data Series {0}',
  'dashboard.widgets.wizard.step2.addMetrics': 'Add Metrics and operators here...',
  'dashboard.widgets.wizard.step2.newMetric': '+ Add Metric',
  'dashboard.widgets.wizard.step2.newOperator': '+ Add Operator',
  'dashboard.widgets.wizard.step2.newNumber': '+ Add Number',
  'dashboard.widgets.wizard.step2.Component': 'Component',
  'dashboard.widgets.wizard.step2.Metric': 'Metric',
  'dashboard.widgets.wizard.step2.selectComponent': 'Select a Component',
  'dashboard.widgets.wizard.step2.selectMetric': 'Select a Metric',
  'dashboard.widgets.wizard.step2.addMetric': 'Add Metric',
  'dashboard.widgets.wizard.step2.aggregateFunction': 'Aggregator Function',
  'dashboard.widgets.wizard.step2.aggregateFunction.scanOps': 'Select Aggregation',
  'dashboard.widgets.wizard.step2.aggregateFunction.notFound': 'No aggregator function found',
  'dashboard.widgets.wizard.step2.aggregateTooltip': 'This mathematical function will be applied to compute single value for selected metric across all host components',
  'dashboard.widgets.wizard.step2.threshold.ok.tooltip': 'This is the threshold value at which the widget color changes from green (OK) to orange (Warning)',
  'dashboard.widgets.wizard.step2.threshold.warning.tooltip': 'This is the threshold value at which the widget color changes from orange (Warning) to red (Critical)',
  'dashboard.widgets.wizard.onClose.popup.body': 'You have unsaved changes. Your changes will not be saved if you exit the wizard at this step.',
  'dashboard.widgets.wizard.onClose.popup.discardAndExit': 'Discard and Exit',
  'dashboard.widgets.wizard.step2.nameSpaceDropDownItem': '{0} (Active)',

  'restart.service.all': 'Restart All',
  'restart.service.all.affected': 'Restart All Affected',
  'restart.service.rest.context': 'Restart {0}s',

  'menu.item.dashboard': 'Dashboard',
  'menu.item.services': 'Services',
  'menu.item.hosts': 'Hosts',
  'menu.item.admin': 'Cluster Admin',
  'menu.item.alerts': 'Alerts',
  'menu.item.views': '<i class="glyphicon glyphicon-th"></i>',
  'menu.item.views.noViews': 'No Views',

  'jobs.nothingToShow': 'No jobs to display',
  'jobs.error.ats.down': 'Jobs data cannot be shown since YARN App Timeline Server is not running.',
  'jobs.error.400': 'Unable to load data.',
  'jobs.table.custom.date.am': 'AM',
  'jobs.table.custom.date.pm': 'PM',
  'jobs.table.custom.date.header': 'Select Time Range',
  'jobs.table.job.fail': 'Job failed to run',
  'jobs.customDateFilter.error.required': 'This field is required',
  'jobs.customDateFilter.error.incorrect': 'Date is incorrect',
  'jobs.customDateFilter.error.laterThanNow': 'The specified time is in the future',
  'jobs.customDateFilter.error.date.order': 'End Date must be after Start Date',
  'jobs.customDateFilter.startTime': 'Start Time',
  'jobs.customDateFilter.endTime': 'End Time',
  'jobs.customDateFilter.duration.15min': '15 minutes',
  'jobs.customDateFilter.duration.30min': '30 minutes',
  'jobs.customDateFilter.duration.1hr': '1 hour',
  'jobs.customDateFilter.duration.2hr': '2 hours',
  'jobs.customDateFilter.duration.4hr': '4 hours',
  'jobs.customDateFilter.duration.12hr': '12 hours',
  'jobs.customDateFilter.duration.24hr': '24 hours',
  'jobs.customDateFilter.duration.1w': '1 week',
  'jobs.customDateFilter.duration.1m': '1 month',
  'jobs.customDateFilter.duration.1yr': '1 year',

  'views.main.yourViews': 'Your Views',
  'views.main.noViews': 'No views',
  'views.main.instance.noDescription': 'No description',

  'number.validate.empty': 'cannot be empty',
  'number.validate.notValidNumber': 'not a valid number',
  'number.validate.lessThanMinimum': 'value less than {0}',
  'number.validate.moreThanMaximum': 'value greater than {0}',

  'common.combobox.placeholder': 'Filter...',
  'common.combobox.dropdown.overridden': 'Overridden properties',
  'common.combobox.dropdown.final': 'Final properties',
  'common.combobox.dropdown.changed': 'Changed properties',
  'common.combobox.dropdown.issues': 'Show property issues',
  'common.combobox.dropdown.warnings': 'Show property warnings',

  'quick.links.error.quicklinks.unavailable.label': 'Quick Links are not available',
  'quick.links.error.nohosts.label': 'Failed to obtain host information for {0}',
  'quick.links.error.oozie.label': 'Quick Links are not available. Make sure Oozie server is running.',
  'quick.links.publicHostName': '{0} ({1})',
  'quick.links.label.active': 'Active',
  'quick.links.label.standby': 'Standby',

  'contact.administrator': 'Contact System Administrator for more information!',

  'config.group.selection.dialog.title': '{0} Configuration Group',
  'config.group.selection.dialog.subtitle': 'Select or create a {0} Configuration Group where the configuration value will be overridden.',
  'config.group.selection.dialog.option.select': 'Select an existing {0} Configuration Group',
  'config.group.selection.dialog.option.select.msg': 'Overridden property will be changed for hosts belonging to the selected group.',
  'config.group.selection.dialog.option.create': 'Create a new {0} Configuration Group',
  'config.group.selection.dialog.option.create.msg': 'A new {0} Configuration Group will be created with the given name. Initially there will be no hosts in the group, with only the selected property overridden.',
  'config.group.selection.dialog.err.name.exists': 'Configuration Group with given name already exists',
  'config.group.selection.dialog.err.create': 'Error creating new Configuration Group [{0}]',
  'config.group.selection.dialog.no.groups': 'There are no existing {0} Configuration Groups.',
  'config.group.host.switch.dialog.title': 'Change Group',

  'config.group.save.confirmation.header': 'Save Configuration Group',
  'config.group.save.confirmation.msg': 'Click <em>Manage Hosts</em> to manage host membership to the configuration group',
  'config.group.save.confirmation.configGroup': 'Configuration Group',
  'config.group.save.confirmation.saved': 'has been successfully saved',
  'config.group.save.confirmation.manage.button': 'Manage Hosts',
  'config.group.description.default': 'New configuration group created on {0}',

  'config.infoMessage.wrong.value.for.widget': 'Configuration value cannot be converted into UI control value',
  'config.infoMessage.wrong.value.for.combobox.widget': '"{0}" is not available in the list of valid values',
  'config.warnMessage.outOfBoundaries.greater': 'Values greater than {0} are not recommended',
  'config.warnMessage.outOfBoundaries.less': 'Values smaller than {0} are not recommended',

  'errorMessage.config.required': 'This is required',
  'errorMessage.config.number.integer': 'Must contain digits only',
  'errorMessage.config.number.float': 'Must be a valid number',
  'errorMessage.config.mail': 'Must be a valid email address',
  'errorMessage.config.user': 'Value is not valid',
  'errorMessage.config.ldapUrl': 'Must be a valid LDAP url',
  'errorMessage.config.password': 'Passwords do not match',
  'errorMessage.config.password.length': 'Password should contain at least {0} symbols',
  'errorMessage.config.directory.heterogeneous': 'dir format is wrong, can be "[{storage type}]/{dir name}"',
  'errorMessage.config.directory.default': 'Must be a slash or drive at the start, and must not contain white spaces',
  'errorMessage.config.directory.allowed': 'Can\'t start with "home(s)"',
  'errorMessage.config.spaces.trailing': 'Cannot contain trailing whitespace',
  'errorMessage.config.spaces.trim': 'Cannot contain leading or trailing whitespace',

  'utils.ajax.errorMessage': 'Error message',
  'utils.ajax.defaultErrorPopupBody.message': 'received on {0} method for API: {1}',
  'utils.ajax.defaultErrorPopupBody.statusCode': '{0} status code',

  'wizard.inProgress': '{0} in Progress | Initiated by {1}',

  'alerts.instance.fullLogPopup.header': 'Instance Response',
  'admin.addHawqStandby.button.enable': 'Add HAWQ Standby Master',
  'admin.addHawqStandby.closePopup': 'Add HAWQ Standby Master Wizard is in progress. You must allow the wizard to' + ' complete for Ambari to be in usable state. If you choose to quit, you must follow documented manual' + ' instructions to complete or reverting adding HAWQ Standby Master. Are you sure you want to exit the wizard?',
  'admin.addHawqStandby.wizard.header': 'Add HAWQ Standby Master Wizard',
  'admin.addHawqStandby.wizard.step1.header': 'Get Started',
  'admin.addHawqStandby.wizard.step1.body': 'This wizard walks you through the process of adding the HAWQ Standby ' + 'Master as a backup of the current HAWQ Master host. After you add the HAWQ Standby Master, it serves as a <i>warm standby</i> ' + 'which may be activated in the event of the primary HAWQ Master host becoming non-operational.<br/><br/>' + '<b>This procedure restarts the HAWQ service. Perform this procedure during a scheduled cluster maintenance window.</b>',
  'admin.addHawqStandby.wizard.step2.header': 'Select Host',
  'admin.addHawqStandby.wizard.step2.body': 'Select a host that will be running the HAWQ Standby Master',
  'admin.addHawqStandby.wizard.step3.header': 'Review',
  'admin.addHawqStandby.wizard.step3.configs_changes': 'Review Configuration Changes.',
  'admin.addHawqStandby.wizard.step3.confirm.host.body': '<b>Confirm your host selections.</b>',
  'admin.addHawqStandby.wizard.step3.confirm.config.body': '<p class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>' + 'The following lists the configuration changes that will be made by the Wizard to add HAWQ Standby Master. ' + 'This information is for <b> review only </b> and is not editable.</div>',
  'admin.addHawqStandby.wizard.step3.hawqMaster': 'Current HAWQ Master',
  'admin.addHawqStandby.wizard.step3.newHawqStandby': 'New HAWQ Standby Master',
  'admin.addHawqStandby.wizard.step3.confirm.dataDir.title': 'HAWQ Standby Master Directory Confirmation',
  'admin.addHawqStandby.wizard.step3.confirm.dataDir.body': 'Before you complete this procedure, ensure that you ' + 'rename the directory <b>{0}</b> on the HAWQ Standby Master host <b>{1}</b> if it exists (for example, ' + 'change it to {0}_old).<br/><br/><b>If {0} exists on the HAWQ Standby Master host, then the new HAWQ ' + 'Standby Master may be started with stale data, leaving the cluster in an inconsistent state.</b><br/><br/>' + 'Click Confirm to indicate that you have renamed any existing <b>{0}</b> directory on the HAWQ Standby Master host <b>{1}</b>.',
  'admin.addHawqStandby.step4.save.configuration.note': 'This configuration is created by Add HAWQ Standby wizard',
  'admin.addHawqStandby.wizard.step4.header': 'Configure Components',
  'admin.addHawqStandby.wizard.step4.task0.title': 'Stop HAWQ Service',
  'admin.addHawqStandby.wizard.step4.task1.title': 'Install HAWQ Standby Master',
  'admin.addHawqStandby.wizard.step4.task2.title': 'Reconfigure HAWQ',
  'admin.addHawqStandby.wizard.step4.task3.title': 'Start HAWQ Service',
  'admin.addHawqStandby.wizard.step4.notice.inProgress': 'Please wait while HAWQ Standby Master is being deployed.',
  'admin.addHawqStandby.wizard.step4.notice.completed': 'HAWQ Standby Master has been added successfully.',
  'admin.removeHawqStandby.button.enable': 'Remove HAWQ Standby',
  'admin.removeHawqStandby.wizard.header': 'Remove HAWQ Standby Wizard',
  'admin.removeHawqStandby.wizard.step1.header': 'Get Started',
  'admin.removeHawqStandby.wizard.step1.body': 'This wizard walks you through the process of removing the HAWQ Standby Master ' + 'in the event of HAWQ Standby Master host failure or maintenance. After you remove the HAWQ Standby Master, the existing HAWQ Master will run in standalone mode without a backup.' + '<br/><br/><b>This procedure stops and restarts the HAWQ service. Perform this procedure during a scheduled cluster ' + 'maintenance window, only when HAWQ Master is functioning properly.</b>' + '<br/><br/>After you complete this wizard, the HAWQ cluster will no longer have a Standby Master. ' + 'As a best practice, use the “Add HAWQ Standby Master” service action to configure a new HAWQ Standby Master for the cluster.',
  'admin.removeHawqStandby.wizard.step2.header': 'Review',
  'admin.removeHawqStandby.wizard.step2.hawqStandby': '<b>Current HAWQ Standby:</b>',
  'admin.removeHawqStandby.wizard.step2.confirm.config.body': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>After removing the HAWQ Standby Master, the Wizard removes the ' + 'hawq_standby_address_host property from hawq-site.xml. As a best practice, you should configure a new HAWQ Standby Master host after the Wizard completes.</div>',
  'admin.removeHawqStandby.wizard.step2.confirm.host.body': '<b>Review HAWQ Standby Master role changes.</b>',
  'admin.removeHawqStandby.wizard.step2.confirmPopup.body': 'Do you wish to continue with removing HAWQ Standby Master? Please confirm, before proceeding as you will not be able to rollback from Ambari.',
  'admin.removeHawqStandby.wizard.step3.header': 'Finalize Setup',
  'admin.removeHawqStandby.wizard.step3.task0.title': 'Remove HAWQ Standby',
  'admin.removeHawqStandby.wizard.step3.task1.title': 'Stop HAWQ Service',
  'admin.removeHawqStandby.wizard.step3.task2.title': 'Reconfigure HAWQ',
  'admin.removeHawqStandby.wizard.step3.task3.title': 'Delete Role: HAWQ Standby',
  'admin.removeHawqStandby.wizard.step3.task4.title': 'Start HAWQ Service',
  'admin.removeHawqStandby.closePopup': 'Remove HAWQ Standby Wizard is in progress. You must allow the wizard to' + ' complete for Ambari to be in usable state. If you choose to quit, you must follow manual instructions to' + ' get back to a stable state. Are you sure you want to exit the wizard?',
  'admin.removeHawqStandby.wizard.step3.notice.inProgress': 'Please wait while HAWQ Standby is being removed',
  'admin.removeHawqStandby.wizard.step3.notice.completed': 'HAWQ Standby has been removed successfully.',
  'admin.removeHawqStandby.wizard.step3.removeHawqStandbyCommand.context': 'Execute HAWQ Standby remove command',
  'admin.removeHawqStandby.wizard.step3.save.configuration.note': 'This configuration was created by Remove HAWQ Standby wizard',
  'admin.activateHawqStandby.button.enable': 'Activate HAWQ Standby Master',
  'admin.activateHawqStandby.wizard.header': 'Activate HAWQ Standby Master Wizard',
  'admin.activateHawqStandby.wizard.step1.header': 'Get Started',
  'admin.activateHawqStandby.wizard.step1.body': 'This wizard walks you through the process of activating the HAWQ Standby Master ' + 'in the event of HAWQ Master host failure. After you activate the HAWQ Standby Master, ' + 'it is promoted as the new HAWQ Master, and the previous HAWQ Master configuration ' + 'is removed from the cluster.<br/><br/><b>This procedure restarts the HAWQ service. ' + 'Perform this procedure during a scheduled cluster maintenance window, unless the current ' + 'HAWQ Master is not functioning.</b><br/><br/> After you complete this wizard, the HAWQ cluster ' + 'will no longer have a Standby Master. As a best practice, use the “Add HAWQ Standby Master” ' + 'service action to configure a new HAWQ Standby Master for the cluster.',
  'admin.activateHawqStandby.wizard.step2.header': 'Review',
  'admin.highAvailability.wizard.step2.toBeDeleted': 'TO BE DELETED',
  'admin.activateHawqStandby.wizard.step2.hawqMaster': '<b>Current HAWQ Master:</b>',
  'admin.activateHawqStandby.wizard.step2.hawqStandby': '<b>Current HAWQ Standby Master:</b>',
  'admin.activateHawqStandby.wizard.step2.toBeActivated': 'TO BE ACTIVATED AS NEW HAWQ MASTER',
  'admin.activateHawqStandby.step4.save.configuration.note': 'This configuration is created by Activate HAWQ Standby wizard',
  'admin.activateHawqStandby.wizard.step2.confirm.config.body': '<div class="alert alert-info">' + '<p><b>Review Configuration Changes.</b></p>The Wizard will make the following configuration changes. ' + 'This information is for review only, and cannot be edited.<br/><br/><b>After activating the HAWQ Standby ' + 'Master, the wizard removes the hawq_standby_address_host property from hawq-site.xml.</b> ' + 'As a best practice, you should configure a new HAWQ Standby Master host after the wizard completes.</div>',
  'admin.activateHawqStandby.wizard.step2.confirm.host.body': '<b>Review HAWQ Master & Standby Master role changes.</b>',
  'admin.activateHawqStandby.wizard.step2.confirmPopup.body': 'Do you wish to continue with activating HAWQ Standy Master? ' + 'Please confirm, before proceeding as you will not be able to rollback from Ambari.',
  'admin.activateHawqStandby.wizard.step3.header': 'Finalize Setup',
  'admin.activateHawqStandby.wizard.step3.task0.title': 'Activate HAWQ Standby Master',
  'admin.activateHawqStandby.wizard.step3.task1.title': 'Stop HAWQ Service',
  'admin.activateHawqStandby.wizard.step3.task2.title': 'Reconfigure HAWQ',
  'admin.activateHawqStandby.wizard.step3.task3.title': 'Install Role: New HAWQ Master',
  'admin.activateHawqStandby.wizard.step3.task4.title': 'Delete Role: Previous HAWQ Master',
  'admin.activateHawqStandby.wizard.step3.task5.title': 'Delete Role: Previous HAWQ Standby',
  'admin.activateHawqStandby.wizard.step3.task6.title': 'Start HAWQ Service',
  'admin.activateHawqStandby.closePopup': 'Activate HAWQ Standby Wizard is in progress. You must allow the wizard to' + ' complete for Ambari to be in usable state. If you choose to quit, you must follow manual instructions to' + ' get back to a stable state. Are you sure you want to exit the wizard?',
  'admin.activateHawqStandby.wizard.step3.notice.inProgress': 'Please wait while HAWQ Standby Master is being activated',
  'admin.activateHawqStandby.wizard.step3.notice.completed': 'HAWQ Standby Master has been activated successfully.',
  'admin.activateHawqStandby.wizard.step3.activateHawqStandbyCommand.context': "Execute HAWQ Standby Master activate command",
  'admin.serviceAutoStart.save.popup.title': 'Save Auto-Start Configuration',
  'admin.serviceAutoStart.save.popup.body': 'You are changing the auto-start configuration.' + 'Click <b>Save</b> to commit the change or <b>Discard</b> to revert your changes',
  'admin.serviceAutoStart.save.popup.transition.title': 'Warning',
  'admin.serviceAutoStart.save.popup.transition.body': 'You have unsaved changes',

  'reset.ui.states': 'Reset UI State',
  'reset.ui.states.body': 'You should proceed only if the UI is misbehaving (such as incorrect navigation upon login, UI is stuck and won&apos;t load, etc.).  Are you sure you want to reset the UI state?',

  'app.logger.ajax': 'Ajax loading time: {0} [{1}]',
  'app.logger.wizard': 'wizard step {0} loading time'

};

});

require.register("mixins", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load all mixins here

require('mixins/common/blueprint');
require('mixins/common/kdc_credentials_controller_mixin');
require('mixins/common/localStorage');
require('mixins/common/infinite_scroll_mixin');
require('mixins/common/persist');
require('mixins/common/reload_popup');
require('mixins/common/serverValidator');
require('mixins/common/table_server_view_mixin');
require('mixins/common/table_server_mixin');
require('mixins/common/track_request_mixin');
require('mixins/common/loading_overlay_support');
require('mixins/main/dashboard/widgets/editable');
require('mixins/main/dashboard/widgets/editable_with_limit');
require('mixins/main/dashboard/widgets/namenode_widget');
require('mixins/main/dashboard/widgets/single_numeric_threshold');
require('mixins/main/host/details/host_components/decommissionable');
require('mixins/main/host/details/host_components/install_component');
require('mixins/main/host/details/actions/install_new_version');
require('mixins/main/host/details/actions/check_host');
require('mixins/main/host/details/support_client_configs_download');
require('mixins/main/service/groups_mapping');
require('mixins/main/service/themes_mapping');
require('mixins/main/service/configs/config_overridable');
require('mixins/main/service/configs/widget_popover_support');
require('mixins/main/service/configs/component_actions_by_configs');
require('mixins/main/service/summary/hdfs_summary_widgets');
require('mixins/routers/redirections');
require('mixins/wizard/wizardProgressPageController');
require('mixins/wizard/wizardDeployProgressController');
require('mixins/wizard/wizardProgressPageView');
require('mixins/wizard/wizardEnableDone');
require('mixins/wizard/selectHost');
require('mixins/wizard/addSecurityConfigs');
require('mixins/wizard/wizard_menu_view');
require('mixins/wizard/wizard_misc_property_checker');
require('mixins/wizard/assign_master_components');
require('mixins/wizard/wizardHostsLoading');
require('mixins/common/configs/widgets/unique/num_llap_nodes');
require('mixins/common/configs/config_recommendations');
require('mixins/common/configs/config_recommendation_parser');
require('mixins/common/configs/config_with_override_recommendation_parser');
require('mixins/common/configs/enhanced_configs');
require('mixins/common/configs/configs_saver');
require('mixins/common/configs/configs_loader');
require('mixins/common/configs/configs_comparator');
require('mixins/common/configs/toggle_isrequired');
require('mixins/common/hosts/host_component_recommendation_mixin');
require('mixins/common/hosts/host_component_validation_mixin');
require('mixins/common/widgets/export_metrics_mixin');
require('mixins/common/widgets/time_range_mixin');
require('mixins/common/widgets/widget_mixin');
require('mixins/common/widgets/widget_section');
require('mixins/unit_convert/base_unit_convert_mixin');
require('mixins/unit_convert/convert_unit_widget_view_mixin');
require('mixins/main/service/configs/hive_interactive_check');

});

require.register("mixins/common/blueprint", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');
var dataManipulation = require('utils/data_manipulation');

App.BlueprintMixin = Em.Mixin.create({

  /**
   * returns blueprint for selected components
   * @method getComponentsBlueprint
   * @param  {Em.Object[]} components components objects or model @see App.HostComponent
   *  required properties are <code>hostName, componentName</code>
   * @param {string[]} [allHostNames=[]] host names, provide when host groups without components should be created
   * @return {Object} blueprint object
   */
  getComponentsBlueprint: function getComponentsBlueprint(components, allHostNames) {
    var hosts = components.mapProperty('hostName').concat(allHostNames || []).uniq();
    var mappedComponents = dataManipulation.groupPropertyValues(components, 'hostName');
    var res = {
      blueprint: { host_groups: [] },
      blueprint_cluster_binding: { host_groups: [] }
    };

    hosts.forEach(function (host, i) {
      var group_name = 'host-group-' + (i + 1);

      res.blueprint.host_groups.push({
        name: group_name,
        components: mappedComponents[host] ? mappedComponents[host].map(function (c) {
          return { name: Em.get(c, 'componentName') };
        }) : []
      });

      res.blueprint_cluster_binding.host_groups.push({
        name: group_name,
        hosts: [{ fqdn: host }]
      });
    });
    return res;
  },

  /**
   * returns blueprint for all currently installed master, slave and client components
   * @method getCurrentMasterSlaveBlueprint
   */
  getCurrentMasterSlaveBlueprint: function getCurrentMasterSlaveBlueprint() {
    return this.getComponentsBlueprint(App.HostComponent.find());
  },

  /**
   * Returns blueprint for all currently installed slave and client components
   */
  getCurrentSlaveBlueprint: function getCurrentSlaveBlueprint() {
    var self = this;
    var fullBlueprint = self.getCurrentMasterSlaveBlueprint();

    var components = App.StackServiceComponent.find().filter(function (c) {
      return c.get('isSlave') || c.get('isClient');
    }).mapProperty("componentName");

    return blueprintUtils.filterByComponents(fullBlueprint, components);
  }
});

});

require.register("mixins/common/configs/config_recommendation_parser", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigRecommendationParser = Em.Mixin.create(App.ConfigRecommendations, {

  stepConfigs: [],

  modifiedFileNames: [],

  /**
   * Adds new modified file name if it's not in the list yet
   *
   * @param filename
   */
  addModifiedFileName: function addModifiedFileName(filename) {
    App.assertExists(filename);
    if (!this.get('modifiedFileNames').contains(filename)) this.get('modifiedFileNames').push(filename);
  },

  /**
   * Method that goes through all configs
   * and apply recommendations using callbacks
   *
   * @param recommendationObject
   * @param configs
   * @param parentProperties
   * @param configGroup
   * @param updateCallback
   * @param removeCallback
   * @param updateBoundariesCallback
   */
  parseRecommendations: function parseRecommendations(recommendationObject, configs, parentProperties, configGroup, updateCallback, removeCallback, updateBoundariesCallback) {

    App.assertObject(recommendationObject);
    App.assertArray(configs);
    App.assertFunction(updateCallback);
    App.assertFunction(removeCallback);
    App.assertFunction(updateBoundariesCallback);
    var propertiesToDelete = [];
    configs.forEach(function (config) {
      var name = Em.get(config, 'name'),
          fileName = Em.get(config, 'filename'),
          value = Em.get(config, 'value'),
          recommendations = recommendationObject[App.config.getConfigTagFromFileName(fileName)];

      if (recommendations) {

        if (recommendations.properties) {
          var recommendedValue = App.config.formatValue(recommendations.properties[name]);
          if (!Em.isNone(recommendedValue)) {
            /** update config **/
            updateCallback(config, recommendedValue, parentProperties, configGroup);

            delete recommendations.properties[name];
          }
        }

        if (recommendations.property_attributes) {
          var propertyAttributes = recommendations.property_attributes[name];
          if (propertyAttributes) {
            var stackProperty = App.configsCollection.getConfigByName(name, fileName);
            for (var attr in propertyAttributes) {
              if (attr === 'delete' && this.allowUpdateProperty(parentProperties, name, fileName, null, value)) {
                propertiesToDelete.push(config);
              } else if (attr === 'visible' || stackProperty) {
                /** update config boundaries **/
                updateBoundariesCallback(stackProperty, attr, propertyAttributes[attr], name, fileName, configGroup);
              }
            }
            Em.tryInvoke(config, 'validate');
          }
        }
      }
    }, this);

    if (propertiesToDelete.length) {
      propertiesToDelete.forEach(function (property) {
        /** remove config **/
        removeCallback(property, configs, parentProperties, configGroup);
      }, this);
    }
  },

  /**
   * Method that goes through all configs
   * and apply recommendations to configs when it's needed
   *
   * @param {Object} recommendationObject
   * @param {Object[]} configs
   * @param {Object[]} parentProperties
   */
  updateConfigsByRecommendations: function updateConfigsByRecommendations(recommendationObject, configs, parentProperties) {
    this.parseRecommendations(recommendationObject, configs, parentProperties, null, this._updateConfigByRecommendation.bind(this), this._removeConfigByRecommendation.bind(this), this._updateBoundaries.bind(this));
  },

  /**
   * This method goes through all config recommendations
   * and trying to add new properties
   *
   * @param {Object} recommendationObject
   * @param {Object[]} parentProperties
   */
  addByRecommendations: function addByRecommendations(recommendationObject, parentProperties) {
    App.assertObject(recommendationObject);

    for (var site in recommendationObject) {
      var properties = recommendationObject[site].properties;
      if (properties && Object.keys(properties).length) {
        var stepConfig = App.config.getStepConfigForProperty(this.get('stepConfigs'), site),
            configs = [];
        if (stepConfig) {
          for (var propertyName in properties) {
            if (this.allowUpdateProperty(parentProperties, propertyName, site)) {
              configs.pushObject(this._createNewProperty(propertyName, site, stepConfig.get('serviceName'), properties[propertyName], parentProperties));
            }
          }
          if (configs.length) {
            var mergedConfigs = configs.concat(stepConfig.get('configs'));
            stepConfig.set('configs', mergedConfigs);
            stepConfig.propertyDidChange('redrawConfigs');
          }
        }
      }
    }
  },

  /**
   * Update config based on recommendations
   *
   * @param {recommendation} config
   * @param {String} recommendedValue
   * @param {String[]} [parentProperties]
   * @returns {recommendation}
   * @protected
   */
  _updateConfigByRecommendation: function _updateConfigByRecommendation(config, recommendedValue, parentProperties) {
    App.assertObject(config);
    var name = Em.get(config, 'name'),
        fileName = Em.get(config, 'filename'),
        group = Em.get(config, 'group.name'),
        value = Em.get(config, 'value'),
        isUserOverriden = Em.get(config, 'didUserOverrideValue');;
    Em.set(config, 'recommendedValue', recommendedValue);
    if (this.allowUpdateProperty(parentProperties, name, fileName, group, value)) {
      var allowConfigUpdate = true;
      // workaround for capacity-scheduler
      if (this.get('currentlyChangedConfig')) {
        var cId = App.config.configId(this.get('currentlyChangedConfig.name'), this.get('currentlyChangedConfig.fileName'));
        if (App.config.configId(name, fileName) === cId) {
          allowConfigUpdate = false;
        }
      }

      if (isUserOverriden && name !== "capacity-scheduler") {
        allowConfigUpdate = false;
      }

      if (allowConfigUpdate) {
        Em.setProperties(config, {
          value: recommendedValue,
          errorMessage: '',
          warnMessage: ''
        });
      }
      if (!Em.isNone(recommendedValue) && !Em.get(config, 'hiddenBySection')) {
        Em.set(config, 'isVisible', true);
      }
      var recommendationExists = this.getRecommendation(name, fileName, group);
      if (recommendationExists && isUserOverriden) {
        this.removeRecommendation(name, fileName, group);
      } else if (!isUserOverriden) {
        this.applyRecommendation(name, fileName, group, recommendedValue, this._getInitialValue(config), parentProperties, Em.get(config, 'isEditable'));
      }
    }
    if (this.updateInitialOnRecommendations(Em.get(config, 'serviceName'))) {
      Em.set(config, 'initialValue', recommendedValue);
    }
    Em.tryInvoke(config, 'validate');
    return config;
  },

  /**
   * Add config based on recommendations
   *
   * @param name
   * @param fileName
   * @param serviceName
   * @param recommendedValue
   * @param parentProperties
   * @protected
   */
  _createNewProperty: function _createNewProperty(name, fileName, serviceName, recommendedValue, parentProperties) {
    App.assertExists(name, 'name');
    App.assertExists(fileName, 'fileName');
    App.assertExists(serviceName, 'serviceName');

    var coreObject = this._getCoreProperties(serviceName, recommendedValue, this._getInitialFromRecommendations(name, fileName)),
        newConfig = App.config.getDefaultConfig(name, fileName, coreObject),
        addedPropertyObject = App.ServiceConfigProperty.create(newConfig);

    this.applyRecommendation(name, fileName, "Default", recommendedValue, null, parentProperties, true);

    return addedPropertyObject;
  },

  /**
   *
   * @param serviceName
   * @param recommendedValue
   * @param initialValue
   * @returns {{value: *, recommendedValue: *, initialValue: *, savedValue: *}}
   * @private
   */
  _getCoreProperties: function _getCoreProperties(serviceName, recommendedValue, initialValue) {
    return {
      "value": recommendedValue,
      "recommendedValue": recommendedValue,
      "initialValue": this.updateInitialOnRecommendations(serviceName) ? recommendedValue : initialValue,
      "savedValue": !this.useInitialValue(serviceName) ? initialValue : null,
      "isNotSaved": Em.isNone(initialValue)
    };
  },

  /**
   * Remove config based on recommendations
   *
   * @param config
   * @param configsCollection
   * @param parentProperties
   * @protected
   */
  _removeConfigByRecommendation: function _removeConfigByRecommendation(config, configsCollection, parentProperties) {
    App.assertObject(config);
    App.assertArray(configsCollection);

    if (this._configHasInitialValue(config)) {
      this.addModifiedFileName(Em.get(config, 'filename'));
    }

    configsCollection.removeObject(config);

    this.applyRecommendation(Em.get(config, 'name'), Em.get(config, 'filename'), Em.get(config, 'group.name'), null, this._getInitialValue(config), parentProperties, Em.get(config, 'isEditable'));
  },

  /**
   * Defines if property was defined on initial load or was saved.
   *
   * @param config
   * @returns {boolean}
   * @private
   */
  _configHasInitialValue: function _configHasInitialValue(config) {
    App.assertObject(config);
    return !Em.isNone(Em.get(config, 'savedValue')) && !Em.isNone(Em.get(config, 'initialValue'));
  },

  /**
   * Update config valueAttributes by recommendations
   *
   * @param {Object} stackProperty
   * @param {string} attr
   * @param {Number|String|Boolean} value
   * @param {String} name
   * @param {String} fileName
   * @protected
   */
  _updateBoundaries: function _updateBoundaries(stackProperty, attr, value, name, fileName) {
    if (attr === 'visible') {
      var p = App.config.findConfigProperty(this.get('stepConfigs'), name, App.config.getOriginalFileName(fileName));
      if (p) {
        p.set('isVisible', value);
      }
    }
    if (stackProperty) {
      if (!Em.get(stackProperty, 'valueAttributes')) {
        stackProperty.valueAttributes = {};
      }
      Em.set(stackProperty.valueAttributes, attr, value);
    }
    return stackProperty || null;
  },

  /**
   * Get initial config value that was before recommendations was applied
   *
   * @param name
   * @param fileName
   * @returns {*}
   * @protected
   */
  _getInitialFromRecommendations: function _getInitialFromRecommendations(name, fileName) {
    try {
      return this.getRecommendation(name, fileName).initialValue;
    } catch (e) {
      return null;
    }
  },

  /**
   * Get default config value
   * <code>savedValue<code> for installed services
   * <code>initialValue<code> for new services
   *
   * @param configProperty
   * @returns {*}
   * @protected
   */
  _getInitialValue: function _getInitialValue(configProperty) {
    if (!configProperty) return null;
    return this.useInitialValue(Em.get(configProperty, 'serviceName')) ? Em.get(configProperty, 'initialValue') : Em.get(configProperty, 'savedValue');
  },

  /**
   * Update initial only when <code>initialValue<code> is used
   *
   * @param {string} serviceName
   * @returns {boolean}
   */
  updateInitialOnRecommendations: function updateInitialOnRecommendations(serviceName) {
    return this.useInitialValue(serviceName);
  },

  /**
   * Defines if initialValue of config can be used on current controller
   * if not savedValue is used instead
   *
   * @param {String} serviceName
   * @return {boolean}
   */
  useInitialValue: function useInitialValue(serviceName) {
    return false;
  },

  /**
   * Defines if recommendation allowed to be applied
   *
   * @param {Array} parentProperties
   * @param {string} name
   * @param {string} fileName
   * @param {string} configGroup
   * @param {*} savedValue
   * @returns {boolean}
   */
  allowUpdateProperty: function allowUpdateProperty(parentProperties, name, fileName, configGroup, savedValue) {
    try {
      return Em.get(this.getRecommendation(name, fileName, configGroup), 'saveRecommended');
    } catch (e) {
      return true;
    }
  }
});

});

require.register("mixins/common/configs/config_recommendations", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @typedef {object} recommendation
 * @property {boolean} saveRecommended - by default is true (checkbox binding)
 * @property {boolean} saveRecommendedDefault - used for cancel operation to restore previous state (saved checkbox value)
 * @property {boolean} isDeleted - true if property was deleted
 * @property {boolean} notDefined - true if property was added
 * @property {string} propertyName
 * @property {string} propertyFileName - file name without '.xml'
 * @property {string} configGroup - name of config group, by default "Default"
 * @property {string} serviceName
 * @property {string} serviceDisplayName
 * @property {string} initialValue
 * @property {string} recommendedValue
 * @property {boolean} allowChangeGroup - flag that allows to change config group for config from dependent not default group
 * @property {string[]} parentConfigs - list of properties based on which current recommendation was performed
 */

App.ConfigRecommendations = Em.Mixin.create({

  /**
   * List of recommendations that was applied to configs
   *
   * @type {recommendation[]}
   */
  recommendations: [],

  /**
   * Update recommendation property if exists
   * otherwise add new
   *
   * @param {string} name
   * @param {string} fileName
   * @param {string} configGroupName
   * @param {string} recommendedValue
   * @param {string} initialValue
   * @param {Object[]} parentProperties
   * @param {boolean} isEditable
   * @returns {recommendation}
   */
  applyRecommendation: function applyRecommendation(name, fileName, configGroupName, recommendedValue, initialValue, parentProperties, isEditable) {
    try {
      var parentPropertyIds = this.formatParentProperties(parentProperties);
      var recommendation = this.getRecommendation(name, fileName, configGroupName);
      if (recommendation) {
        return this.updateRecommendation(recommendation, recommendedValue, parentPropertyIds);
      }
      return this.addRecommendation(name, fileName, configGroupName, recommendedValue, initialValue, parentPropertyIds, isEditable);
    } catch (e) {
      console.error(e.message);
    }
  },

  /**
   * Format objects like {name: {String}, type: {String}} to config Id
   *
   * @param parentProperties
   * @returns {*}
   */
  formatParentProperties: function formatParentProperties(parentProperties) {
    return Em.isArray(parentProperties) ? parentProperties.map(function (p) {
      return App.config.configId(p.name, p.type);
    }) : [];
  },

  /**
   * Add new recommendation
   *
   * @param {string} name
   * @param {string} fileName
   * @param {string} configGroupName
   * @param {string} recommendedValue
   * @param {string} initialValue
   * @param {string[]} parentPropertyIds
   * @param {boolean} isEditable
   * @returns {recommendation}
   */
  addRecommendation: function addRecommendation(name, fileName, configGroupName, recommendedValue, initialValue, parentPropertyIds, isEditable) {
    Em.assert('name and fileName should be defined', name && fileName);
    var site = App.config.getConfigTagFromFileName(fileName);
    var service = App.config.get('serviceByConfigTypeMap')[site];
    var configObject = App.configsCollection.getConfigByName(name, fileName);
    var displayName = configObject && configObject.displayName;

    var recommendation = {
      saveRecommended: true,
      saveRecommendedDefault: true,
      propertyFileName: site,
      propertyName: name,
      propertyTitle: configObject && Em.I18n.t('installer.controls.serviceConfigPopover.title').format(displayName, displayName === name ? '' : name),
      propertyDescription: configObject && configObject.description,

      isDeleted: Em.isNone(recommendedValue),
      notDefined: Em.isNone(initialValue),

      configGroup: configGroupName || "Default",
      initialValue: initialValue,
      parentConfigs: parentPropertyIds || [],
      serviceName: service.get('serviceName'),
      allowChangeGroup: false, //TODO groupName!= "Default" && (service.get('serviceName') != this.get('selectedService.serviceName'))
      //TODO&& (App.ServiceConfigGroup.find().filterProperty('serviceName', service.get('serviceName')).length > 1), //TODO
      serviceDisplayName: service.get('displayName'),
      recommendedValue: recommendedValue,
      isEditable: isEditable !== false
    };
    this.get('recommendations').pushObject(recommendation);
    return recommendation;
  },

  /**
   * Remove recommendation
   * based on unique identifiers
   *
   * @param {string} name
   * @param {string} fileName
   * @param {string} configGroupName
   */
  removeRecommendation: function removeRecommendation(name, fileName, configGroupName) {
    this.removeRecommendationObject(this.getRecommendation(name, fileName, configGroupName));
  },

  /**
   * Remove recommended Object
   *
   * @param {recommendation} recommendation
   */
  removeRecommendationObject: function removeRecommendationObject(recommendation) {
    Em.assert('recommendation should be defined object', recommendation && (typeof recommendation === 'undefined' ? 'undefined' : _typeof(recommendation)) === 'object');
    this.get('recommendations').removeObject(recommendation);
  },

  /**
   * Update recommended object
   *
   * @param recommendation
   * @param recommendedValue
   * @param parentPropertyIds
   * @returns {*|recommendation|null}
   */
  updateRecommendation: function updateRecommendation(recommendation, recommendedValue, parentPropertyIds) {
    Em.assert('recommendation should be defined object', recommendation && (typeof recommendation === 'undefined' ? 'undefined' : _typeof(recommendation)) === 'object');
    Em.set(recommendation, 'recommendedValue', recommendedValue);
    if (parentPropertyIds && parentPropertyIds.length) {
      var mergedProperties = parentPropertyIds.concat(Em.get(recommendation, 'parentConfigs') || []).uniq();
      Em.set(recommendation, 'parentConfigs', mergedProperties);
    }
    return recommendation;
  },

  /**
   *
   * @param recommendation
   * @param saveRecommendation
   */
  saveRecommendation: function saveRecommendation(recommendation, _saveRecommendation) {
    Em.assert('recommendation should be defined object', recommendation && (typeof recommendation === 'undefined' ? 'undefined' : _typeof(recommendation)) === 'object');
    if (recommendation.saveRecommended !== _saveRecommendation) {
      Em.setProperties(recommendation, {
        'saveRecommended': !!_saveRecommendation,
        'saveRecommendedDefault': !!_saveRecommendation
      });
      return true;
    }
    return false;
  },

  /**
   * Get single recommendation
   *
   * @param name
   * @param fileName
   * @param configGroupName
   * @returns {recommendation|null}
   */
  getRecommendation: function getRecommendation(name, fileName, configGroupName) {
    Em.assert('name and fileName should be defined', name && fileName);
    return this.get('recommendations').find(function (dcv) {
      return dcv.propertyName === name && dcv.propertyFileName === App.config.getConfigTagFromFileName(fileName) && dcv.configGroup === (configGroupName || "Default");
    }) || null;
  },

  /**
   * Clear recommendations that are
   * same as initial value
   *
   * @method cleanUpRecommendations
   */
  cleanUpRecommendations: function cleanUpRecommendations() {
    var cleanDependentList = this.get('recommendations').filter(function (d) {
      return !(Em.isNone(d.initialValue) && Em.isNone(d.recommendedValue) || d.initialValue == d.recommendedValue);
    }, this);
    this.set('recommendations', cleanDependentList);
  },

  /**
   * Remove all recommendations
   *
   * @method clearAllRecommendations
   */
  clearAllRecommendations: function clearAllRecommendations() {
    this.set('recommendations', []);
  },

  /**
   * Clear values for dependent configs for given services
   *
   * @method clearRecommendationsByServiceName
   */
  clearRecommendationsByServiceName: function clearRecommendationsByServiceName(serviceNames) {
    var filteredRecommendations = this.get('recommendations').reject(function (c) {
      return serviceNames.contains(c.serviceName);
    }, this);
    this.set('recommendations', filteredRecommendations);
  }

});

});

require.register("mixins/common/configs/config_with_override_recommendation_parser", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigWithOverrideRecommendationParser = Em.Mixin.create(App.ConfigRecommendationParser, {

  /**
   * Method that goes through all configs
   * and apply recommendations to overrides when it's needed
   *
   * @param {Object} recommendationObject
   * @param {Object[]} configs
   * @param {Object[]} parentProperties
   * @param {App.ServiceConfigGroup} configGroup
   */
  updateOverridesByRecommendations: function updateOverridesByRecommendations(recommendationObject, configs, parentProperties, configGroup) {
    Em.assert('Config groups should be defined and not default', configGroup && configGroup.get('name') && !configGroup.get('isDefault'));
    this.parseRecommendations(recommendationObject, configs, parentProperties, configGroup, this._updateOverride.bind(this), this._removeOverride.bind(this), this._updateOverrideBoundaries.bind(this));
  },

  /**
   * Update override by recommendations
   * includes add/update actions
   *
   * @param config
   * @param recommendedValue
   * @param parentProperties
   * @param configGroup
   * @protected
   */
  _updateOverride: function _updateOverride(config, recommendedValue, parentProperties, configGroup) {
    var name = Em.get(config, 'name'),
        fileName = Em.get(config, 'filename'),
        group = Em.get(config, 'group.name'),
        value = Em.get(config, 'value');

    var updateValue = this.allowUpdateProperty(parentProperties, name, fileName, group, value);
    var override = config.getOverride(configGroup.get('name'));
    if (override) {
      this._updateConfigByRecommendation(override, recommendedValue, parentProperties);
    } else if (updateValue) {
      this._addConfigOverrideRecommendation(config, recommendedValue, parentProperties, configGroup);
    }
  },

  /**
   * Remove override by recommendations
   *
   * @param property
   * @param configs
   * @param parentProperties
   * @param configGroup
   * @protected
   */
  _removeOverride: function _removeOverride(property, configs, parentProperties, configGroup) {
    var config = property.getOverride(configGroup.get('name'));
    config = config ? config : property;
    this._removeConfigByRecommendation(config, property.get('overrides') || [], parentProperties);
  },

  /**
   * Add override by recommendations
   *
   * @param config
   * @param recommendedValue
   * @param configGroup
   * @param parentProperties
   * @protected
   */
  _addConfigOverrideRecommendation: function _addConfigOverrideRecommendation(config, recommendedValue, parentProperties, configGroup) {
    var popupProperty = this.getRecommendation(Em.get(config, 'name'), Em.get(config, 'filename'), configGroup.get('name')),
        initialValue = popupProperty ? popupProperty.value : null;
    var coreObject = {
      "value": recommendedValue,
      "recommendedValue": recommendedValue,
      "initialValue": initialValue,
      "savedValue": !this.useInitialValue(Em.get(config, 'serviceName')) && !Em.isNone(initialValue) ? initialValue : null,
      "isEditable": true,
      "errorMessage": '',
      "warnMessage": ''
    };
    var override = App.config.createOverride(config, coreObject, configGroup);

    this.applyRecommendation(Em.get(config, 'name'), Em.get(config, 'filename'), configGroup.get('name'), recommendedValue, this._getInitialValue(override), parentProperties, Em.get(config, 'isEditable'));
  },

  /**
   * Update override valueAttributes by recommendations
   *
   * @param {Object} stackProperty
   * @param {string} attr
   * @param {Number|String|Boolean} value
   * @param {String} name
   * @param {String} fileName
   * @param {App.ServiceConfigGroup} configGroup
   * @protected
   */
  _updateOverrideBoundaries: function _updateOverrideBoundaries(stackProperty, attr, value, name, fileName, configGroup) {
    if (!stackProperty.valueAttributes[configGroup.get('name')]) {
      stackProperty.valueAttributes[configGroup.get('name')] = {};
    }
    Em.set(stackProperty.valueAttributes[configGroup.get('name')], attr, value);
  }
});

});

require.register("mixins/common/configs/configs_comparator", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var objectUtils = require('utils/object_utils');

App.ConfigsComparator = Em.Mixin.create({

  isCompareMode: false,

  compareServiceVersion: null,

  selectedVersion: null,

  isVersionDefault: Em.K,
  /**
   * load version configs for comparison
   * @param allConfigs
   * @return {object}
   * @public
   * @method loadCompareVersionConfigs
   */
  loadCompareVersionConfigs: function loadCompareVersionConfigs(allConfigs) {
    var dfd = $.Deferred();
    var self = this;
    var compareServiceVersions = [];

    if (this.get('compareServiceVersion')) {
      if (!this.isVersionDefault(this.get('compareServiceVersion').get('version'))) {
        compareServiceVersions = [this.get('compareServiceVersion').get('version'), this.get('selectedVersion')];
      } else {
        compareServiceVersions = [this.get('compareServiceVersion').get('version')];
      }
      this.set('isCompareMode', true);
      this.getCompareVersionConfigs(compareServiceVersions).done(function (json) {
        allConfigs.setEach('isEditable', false);
        self.initCompareConfig(allConfigs, json);
        dfd.resolve(true);
      }).fail(function () {
        self.set('compareServiceVersion', null);
        dfd.resolve(true);
      });
    } else {
      self.set('isCompareMode', false);
      allConfigs.setEach('isComparison', false);
      dfd.resolve(false);
    }
    return dfd.promise();
  },

  /**
   * attach analogical config to each property for comparison
   * @param allConfigs
   * @param json
   * @private
   * @method initCompareConfig
   */
  initCompareConfig: function initCompareConfig(allConfigs, json) {
    var serviceVersionMap = {};
    var configNamesMap = allConfigs.toWickMapByProperty('name');
    var serviceName = this.get('content.serviceName');
    var compareVersionNumber = this.get('compareServiceVersion.version');
    //indicate whether compared versions are from non-default group
    var compareNonDefaultVersions = json.items.length > 1;

    serviceVersionMap[compareVersionNumber] = {};
    if (compareNonDefaultVersions) {
      serviceVersionMap[this.get('selectedVersion')] = {};
    }

    json.items.forEach(function (item) {
      item.configurations.forEach(function (configuration) {
        if (serviceName === 'YARN' && configuration.type === 'capacity-scheduler') {
          this.addCompareCSConfigs(configuration, serviceVersionMap, item);
        } else {
          for (var prop in configuration.properties) {
            var name = JSON.parse('"' + prop + '"');
            serviceVersionMap[item.service_config_version][name + '-' + configuration.type] = {
              name: name,
              value: configuration.properties[prop],
              type: configuration.type,
              tag: configuration.tag,
              version: configuration.version,
              service_config_version: item.service_config_version,
              filename: App.config.getOriginalFileName(configuration.type)
            };
            if (Em.isNone(configNamesMap[name])) {
              allConfigs.push(this.getMockConfig(name, serviceName, App.config.getOriginalFileName(configuration.type)));
            }
          }
        }
        if (configuration.properties_attributes && configuration.properties_attributes.final) {
          for (var final in configuration.properties_attributes.final) {
            var config = serviceVersionMap[item.service_config_version][final + '-' + configuration.type];
            if (config) {
              config.isFinal = configuration.properties_attributes.final[final] === 'true';
            }
          }
        }
      }, this);
    }, this);

    this.addCompareConfigs(compareNonDefaultVersions, allConfigs, serviceVersionMap);
  },

  /**
   *
   * @param {boolean} compareNonDefaultVersions
   * @param {Array} allConfigs
   * @param {object} serviceVersionMap
   */
  addCompareConfigs: function addCompareConfigs(compareNonDefaultVersions, allConfigs, serviceVersionMap) {
    var _this = this;

    var compareVersionNumber = this.get('compareServiceVersion.version');
    var serviceName = this.get('content.serviceName');
    if (compareNonDefaultVersions) {
      allConfigs.forEach(function (serviceConfig) {
        if (Em.get(serviceConfig, 'isRequiredByAgent') !== false) {
          this.setCompareConfigs(serviceConfig, serviceVersionMap, compareVersionNumber, this.get('selectedVersion'));
        }
      }, this);
    } else {
      var serviceCfgVersionMap = serviceVersionMap[this.get('compareServiceVersion.version')] || {};
      var allConfigsMap = {};
      allConfigs.forEach(function (serviceConfig) {
        var id = serviceConfig.name + '-' + App.config.getConfigTagFromFileName(serviceConfig.filename);
        allConfigsMap[id] = serviceConfig;
        if (Em.get(serviceConfig, 'isRequiredByAgent') !== false) {
          var compareConfig = serviceCfgVersionMap[id];
          this.setCompareDefaultGroupConfig(serviceConfig, compareConfig);
        }
      }, this);
      if (allConfigs.length !== Object.keys(serviceCfgVersionMap).length) {
        Object.keys(serviceCfgVersionMap).forEach(function (id) {
          if (!allConfigsMap[id]) {
            var mockConfig = _this.getMockConfig(serviceCfgVersionMap[id].name, serviceName, serviceCfgVersionMap[id].filename);
            _this.setCompareDefaultGroupConfig(mockConfig, serviceCfgVersionMap[id]);
            allConfigs.push(mockConfig);
          }
        });
      }
    }
  },

  /**
   * init compare configs for Capacity-scheduler
   * @param {object} configuration
   * @param {object} serviceVersionMap
   * @param {object} item
   */
  addCompareCSConfigs: function addCompareCSConfigs(configuration, serviceVersionMap, item) {
    var configsToSkip = App.config.getPropertiesFromTheme('YARN');
    // put all properties in a single textarea for capacity-scheduler
    var value = '';
    for (var prop in configuration.properties) {
      if (configsToSkip.contains(App.config.configId(prop, configuration.type))) {
        serviceVersionMap[item.service_config_version][prop + '-' + configuration.type] = {
          name: prop,
          value: configuration.properties[prop],
          type: configuration.type,
          tag: configuration.tag,
          version: configuration.version,
          service_config_version: item.service_config_version
        };
      } else {
        value += prop + '=' + configuration.properties[prop] + '\n';
      }
    }
    serviceVersionMap[item.service_config_version][configuration.type + '-' + configuration.type] = {
      name: configuration.type,
      value: value,
      type: configuration.type,
      tag: configuration.tag,
      version: configuration.version,
      service_config_version: item.service_config_version
    };
  },

  /**
   * set compare properties to service config of non-default group
   * @param serviceConfig
   * @param serviceVersionMap
   * @param compareVersion
   * @param selectedVersion
   * @private
   * @method setCompareConfigs
   */
  setCompareConfigs: function setCompareConfigs(serviceConfig, serviceVersionMap, compareVersion, selectedVersion) {
    var tag = App.config.getConfigTagFromFileName(Em.get(serviceConfig, 'filename'));
    var compareConfig = serviceVersionMap[compareVersion][Em.get(serviceConfig, 'name') + '-' + tag];
    var selectedConfig = serviceVersionMap[selectedVersion][Em.get(serviceConfig, 'name') + '-' + tag];
    var compareConfigs = [];

    Em.set(serviceConfig, 'isComparison', true);

    if (!Em.get(serviceConfig, 'isCustomGroupConfig')) {
      Em.set(serviceConfig, 'hasCompareDiffs', false);
    }

    if (compareConfig && selectedConfig) {
      compareConfigs.push(this.getComparisonConfig(serviceConfig, compareConfig));
      compareConfigs.push(this.getComparisonConfig(serviceConfig, selectedConfig));
      Em.set(serviceConfig, 'hasCompareDiffs', this.hasCompareDiffs(compareConfigs[0], compareConfigs[1]));
    } else if (compareConfig && !selectedConfig) {
      compareConfigs.push(this.getComparisonConfig(serviceConfig, compareConfig));
      compareConfigs.push(this.getMockComparisonConfig(selectedConfig, selectedVersion));
      Em.set(serviceConfig, 'hasCompareDiffs', true);
    } else if (!compareConfig && selectedConfig) {
      compareConfigs.push(this.getMockComparisonConfig(selectedConfig, compareVersion));
      compareConfigs.push(this.getComparisonConfig(serviceConfig, selectedConfig));
      Em.set(serviceConfig, 'hasCompareDiffs', true);
    }

    Em.set(serviceConfig, 'compareConfigs', compareConfigs);
  },

  /**
   * init attributes and wrap mock compare config into App.ServiceConfigProperty
   * @param serviceConfig
   * @param compareServiceVersion
   * @return {object}
   * @private
   * @method getMockComparisonConfig
   */
  getMockComparisonConfig: function getMockComparisonConfig(serviceConfig, compareServiceVersion) {
    var compareObject = $.extend(true, { isComparison: false }, serviceConfig);
    Em.setProperties(compareObject, {
      isEditable: false,
      serviceVersion: App.ServiceConfigVersion.find(this.get('content.serviceName') + "_" + compareServiceVersion),
      isMock: true,
      displayType: 'label'
    });
    compareObject = App.ServiceConfigProperty.create(compareObject);
    compareObject.set('value', Em.I18n.t('common.property.undefined'));
    return compareObject;
  },

  /**
   * init attributes and wrap compare config into App.ServiceConfigProperty
   * @param serviceConfig
   * @param compareConfig
   * @return {object}
   * @private
   * @method getComparisonConfig
   */
  getComparisonConfig: function getComparisonConfig(serviceConfig, compareConfig) {
    var compareObject = $.extend(true, { isComparison: false, isOriginalSCP: false }, serviceConfig);
    Em.set(compareObject, 'isEditable', false);

    if (compareConfig) {
      if (Em.get(serviceConfig, 'isMock')) {
        Em.set(compareObject, 'displayType', 'string');
        Em.set(compareObject, 'isMock', false);
      }
      Em.set(compareObject, 'serviceVersion', App.ServiceConfigVersion.find(this.get('content.serviceName') + "_" + compareConfig.service_config_version));
      compareObject = App.ServiceConfigProperty.create(compareObject);
      compareObject.setProperties({
        isFinal: Boolean(compareConfig.isFinal),
        value: App.config.formatPropertyValue(serviceConfig, compareConfig.value),
        compareConfigs: null,
        isOriginalSCP: false
      });
    }
    return compareObject;
  },

  /**
   * set compare properties to service config of default group
   * @param serviceConfig
   * @param compareConfig
   * @private
   * @method setCompareDefaultGroupConfig
   */
  setCompareDefaultGroupConfig: function setCompareDefaultGroupConfig(serviceConfig, compareConfig) {
    var compareObject = {};
    var isEmptyProp = App.isEmptyObject(serviceConfig);

    Em.set(serviceConfig, 'compareConfigs', []);
    Em.set(serviceConfig, 'isComparison', true);
    //if config isn't reconfigurable then it can't have changed value to compare
    if (compareConfig) {
      compareObject = this.getComparisonConfig(serviceConfig, compareConfig);
      Em.set(serviceConfig, 'hasCompareDiffs', Em.get(serviceConfig, 'isMock') || this.hasCompareDiffs(serviceConfig, compareObject));
      Em.get(serviceConfig, 'compareConfigs').push(compareObject);
      // user custom property or property that was added during upgrade
    } else {
      if (Em.get(serviceConfig, 'value') !== 'Undefined') {
        var addToComparison = !Em.get(serviceConfig, 'isCustomGroupConfig') && (Em.get(serviceConfig, 'isUserProperty') || !isEmptyProp && !compareConfig && Em.get(serviceConfig, 'isRequiredByAgent') !== false);
        if (addToComparison) {
          Em.get(serviceConfig, 'compareConfigs').push(this.getMockComparisonConfig(serviceConfig, this.get('compareServiceVersion.version')));
          Em.set(serviceConfig, 'hasCompareDiffs', true);
        }
      }
    }

    return serviceConfig;
  },

  /**
   * check value and final attribute of original and compare config for differences
   * if config is password, it won't be shown in the comparison
   *
   * @param originalConfig
   * @param compareConfig
   * @return {Boolean}
   * @private
   * @method hasCompareDiffs
   */
  hasCompareDiffs: function hasCompareDiffs(originalConfig, compareConfig) {
    var displayType = Em.get(originalConfig, 'displayType');
    var notShownTypes = ['password'];
    if (notShownTypes.contains(displayType)) {
      return false;
    }
    var originalValue = App.config.trimProperty({ value: Em.get(originalConfig, 'value'), displayType: 'string' });
    var compareValue = App.config.trimProperty({ value: Em.get(compareConfig, 'value'), displayType: 'string' });

    if (originalValue.toArray) {
      originalValue = originalValue.toArray();
    }
    if (compareValue.toArray) {
      compareValue = compareValue.toArray();
    }

    if (originalValue instanceof Array) {
      originalValue.sort();
    }
    if (compareValue instanceof Array) {
      compareValue.sort();
    }

    return !objectUtils.deepEqual(originalValue, compareValue) || !!Em.get(originalConfig, 'isFinal') !== !!Em.get(compareConfig, 'isFinal');
  },

  /**
   * generate mock config object
   * @param name
   * @param serviceName
   * @param filename
   * @return {Object}
   * @private
   * @method getMockConfig
   */
  getMockConfig: function getMockConfig(name, serviceName, filename) {
    var undefinedConfig = App.configsCollection.getConfigByName(name, filename) || {
      description: name,
      displayName: name,
      isOverridable: false,
      isReconfigurable: false,
      isRequired: false,
      isRequiredByAgent: true,
      isSecureConfig: false,
      isUserProperty: true,
      isVisible: true,
      isOriginalSCP: false,
      name: name,
      filename: filename,
      serviceName: serviceName,
      category: App.config.getDefaultCategory(false, filename),
      value: Em.I18n.t('common.property.undefined'),
      isMock: true,
      displayType: 'label'
    };
    return App.ServiceConfigProperty.create(undefinedConfig);
  },

  /**
   * get configs of chosen version from server to compare
   * @param compareServiceVersions
   * @return {$.ajax}
   * @private
   * @method getCompareVersionConfigs
   */
  getCompareVersionConfigs: function getCompareVersionConfigs(compareServiceVersions) {
    this.set('versionLoaded', false);

    return App.ajax.send({
      name: 'service.serviceConfigVersions.get.multiple',
      sender: this,
      data: {
        serviceName: this.get('content.serviceName'),
        serviceConfigVersions: compareServiceVersions
      }
    });
  }

});

});

require.register("mixins/common/configs/configs_loader", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigsLoader = Em.Mixin.create(App.GroupsMappingMixin, {

  /**
   * version of default config group, configs of which currently applied
   */
  currentDefaultVersion: null,

  /**
   * defines if service config versions are loaded to model
   * @type {boolean}
   */
  allVersionsLoaded: false,

  /**
   * this method should be used in clear step method
   * @method clearLoadInfo
   */
  clearLoadInfo: function clearLoadInfo() {
    this.set('allVersionsLoaded', false);
  },

  /**
   * loads all versions that is not saved on UI for current moment
   * @returns {$.ajax}
   */
  loadServiceConfigVersions: function loadServiceConfigVersions() {
    this.set('allVersionsLoaded', false);
    return App.ajax.send({
      name: 'service.serviceConfigVersions.get',
      data: {
        serviceName: this.get('content.serviceName')
      },
      sender: this,
      success: 'loadServiceConfigVersionsSuccess'
    });
  },

  /**
   * success handler for <code>loadServiceConfigVersions<code>
   * @param data
   */
  loadServiceConfigVersionsSuccess: function loadServiceConfigVersionsSuccess(data) {
    if (Em.get(data, 'items.length')) {
      App.serviceConfigVersionsMapper.map(data);
      var currentDefault = data.items.filterProperty('group_name', App.ServiceConfigGroup.defaultGroupName).findProperty('is_current');
      if (currentDefault) {
        this.set('currentDefaultVersion', currentDefault.service_config_version);
      }
    }
    this.set('allVersionsLoaded', true);
    if (!this.get('preSelectedConfigVersion')) {
      this.set('selectedVersion', this.get('currentDefaultVersion'));
    }
    this.set('preSelectedConfigVersion', null);
  },

  /**
   * @method loadPreSelectedConfigVersion
   */
  loadPreSelectedConfigVersion: function loadPreSelectedConfigVersion() {
    var preSelectedVersion = this.get('preSelectedConfigVersion');

    this.set('selectedVersion', this.get('preSelectedConfigVersion.version'));
    /** handling redirecting from config history page **/
    var self = this;
    this.loadConfigGroups(this.get('servicesToLoad')).done(function () {
      var selectedGroup = App.ServiceConfigGroup.find().find(function (g) {
        return g.get('serviceName') === preSelectedVersion.get('serviceName') && (g.get('name') === preSelectedVersion.get('groupName') || preSelectedVersion.get('groupName') === App.ServiceConfigGroup.defaultGroupName && g.get('isDefault'));
      });
      self.set('selectedConfigGroup', selectedGroup);
      self.loadSelectedVersion(preSelectedVersion.get('version'), selectedGroup);
      preSelectedVersion = null;
    });
  },

  /**
   * loads current versions of current and dependent services
   * and all current version for config groups
   * @method loadCurrentVersions
   */
  loadCurrentVersions: function loadCurrentVersions() {
    this.set('isCompareMode', false);
    this.set('compareServiceVersion', null);
    this.set('versionLoaded', false);
    this.set('selectedVersion', this.get('currentDefaultVersion'));
    this.set('preSelectedConfigVersion', null);
    this.trackRequestChain(App.ajax.send({
      name: 'service.serviceConfigVersions.get.current',
      sender: this,
      data: {
        serviceNames: this.get('servicesToLoad').join(',')
      },
      success: 'loadCurrentVersionsSuccess'
    }));
  },

  /**
   * success handler for <code>loadCurrentVersions<code>
   * @param data
   * @param opt
   * @param params
   */
  loadCurrentVersionsSuccess: function loadCurrentVersionsSuccess(data, opt, params) {
    var self = this;
    var serviceGroups;
    App.configGroupsMapper.map(data, true, params.serviceNames.split(','));
    this.loadConfigGroups(params.serviceNames.split(',')).done(function () {
      serviceGroups = App.ServiceConfigGroup.find().filterProperty('serviceName', self.get('content.serviceName'));
      if (self.get('isHostsConfigsPage')) {
        self.set('selectedConfigGroup', serviceGroups.find(function (cg) {
          return !cg.get('isDefault') && cg.get('hosts').contains(self.get('host.hostName'));
        }) || serviceGroups.findProperty('isDefault'));
      } else {
        self.set('selectedConfigGroup', serviceGroups.findProperty('isDefault'));
      }
      self.parseConfigData(data);
    });
  },

  /**
   * loads selected versions of current service
   * @method loadSelectedVersion
   */
  loadSelectedVersion: function loadSelectedVersion(version, switchToGroup, stayInCompare) {
    if (!stayInCompare) {
      this.set('isCompareMode', false);
      this.set('compareServiceVersion', null);
    }
    this.set('versionLoaded', false);
    version = version || this.get('currentDefaultVersion');
    this.clearRecommendationsInfo();
    if (version === this.get('currentDefaultVersion') && (!switchToGroup || switchToGroup.get('isDefault'))) {
      // current version with default group
      this.set('selectedVersion', this.get('currentDefaultVersion'));
      this.loadCurrentVersions();
    } else {
      // - version with DEFAULT config group but not CURRENT
      // - ANY version with NON_DEFAULT config group
      this.loadDefaultGroupVersion(version, switchToGroup);
    }
  },

  /**
   * version with NON_DEFAULT group require properties from CURRENT version of DEFAULT group to correctly display page
   * @param {string} version
   * @param {?Em.Object} switchToGroup
   */
  loadDefaultGroupVersion: function loadDefaultGroupVersion(version, switchToGroup) {
    var versions = this.isVersionDefault(version) ? [version] : [this.get('currentDefaultVersion'), version];
    var selectedVersion = versions.length > 1 ? versions[1] : versions[0];

    this.setSelectedConfigGroup(version, switchToGroup);
    this.set('selectedVersion', selectedVersion);
    this.trackRequest(App.ajax.send({
      name: 'service.serviceConfigVersions.get.multiple',
      sender: this,
      data: {
        serviceName: this.get('content.serviceName'),
        serviceConfigVersions: versions,
        additionalParams: this.get('dependentServiceNames.length') ? '|(service_name.in(' + this.get('dependentServiceNames') + ')&is_current=true)' : ''
      },
      success: 'loadSelectedVersionsSuccess'
    }));
  },

  /**
   *
   * @param {string} version
   * @param {?Em.Object} switchToGroup
   */
  setSelectedConfigGroup: function setSelectedConfigGroup(version, switchToGroup) {
    switchToGroup = this.isVersionDefault(version) && !switchToGroup ? this.get('configGroups').findProperty('isDefault') : switchToGroup;

    if (this.get('dataIsLoaded') && switchToGroup) {
      this.set('selectedConfigGroup', switchToGroup);
    }
  },

  /**
   * success handler for <code>loadSelectedVersionsSuccess<code>
   * @param data
   */
  loadSelectedVersionsSuccess: function loadSelectedVersionsSuccess(data) {
    this.parseConfigData(data);
  }
});

});

require.register("mixins/common/configs/configs_saver", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var lazyLoading = require('utils/lazy_loading');
var stringUtils = require('utils/string_utils');

/**
 * Mixin for saving configs
 * Contains methods for
 * - generation JSON for saving configs and groups
 * - format properties if needed
 * - requests to save configs and groups
 */
App.ConfigsSaverMixin = Em.Mixin.create({

  /**
   * @type {boolean}
   */
  saveConfigsFlag: true,

  /**
   * file names of changed configs
   * @type {string[]}
   */
  modifiedFileNames: [],

  /**
   * List of heapsize properties not to be parsed
   * @type {string[]}
   */
  heapsizeException: ['hadoop_heapsize', 'yarn_heapsize', 'nodemanager_heapsize', 'resourcemanager_heapsize', 'apptimelineserver_heapsize', 'jobhistory_heapsize', 'nfsgateway_heapsize', 'accumulo_master_heapsize', 'accumulo_tserver_heapsize', 'accumulo_monitor_heapsize', 'accumulo_gc_heapsize', 'accumulo_other_heapsize', 'hbase_master_heapsize', 'hbase_regionserver_heapsize', 'metrics_collector_heapsize', 'hive_heapsize'],

  /**
   * Regular expression for heapsize properties detection
   * @type {regexp}
   */
  heapsizeRegExp: /_heapsize|_newsize|_maxnewsize|_permsize|_maxpermsize$/,

  /**
   * If some of services has such type core-site can be saved
   *
   * @type {string}
   */
  coreSiteServiceType: 'HCFS',

  /**
   * Core site can be used by multiple services
   * If some of services listed below is selected/installed than core site can be saved
   *
   * @type {string[]}
   */
  coreSiteServiceNames: ['HDFS', 'GLUSTERFS', 'RANGER_KMS'],

  /**
   * List of services which configs should be saved
   *
   * @type {App.StackService[]}
   */
  currentServices: function () {
    return [App.StackService.find(this.get('content.serviceName'))];
  }.property('content.serviceName'),

  /**
   * clear info to default
   * @method clearSaveInfo
   */
  clearSaveInfo: function clearSaveInfo() {
    this.set('modifiedFileNames', []);
  },

  /**
   * method to run saving configs
   * @method saveStepConfigs
   */
  saveStepConfigs: function saveStepConfigs() {
    if (!this.get("isSubmitDisabled")) {
      this.startSave();
      this.showWarningPopupsBeforeSave();
    }
  },

  /**
   * get config group object for current service
   * @param serviceName
   * @returns {App.ConfigGroup}
   */
  getGroupFromModel: function getGroupFromModel(serviceName) {
    if (this.get('selectedService.serviceName') === serviceName) {
      return this.get('selectedConfigGroup');
    } else {
      var groups = App.ServiceConfigGroup.find().filterProperty('serviceName', serviceName);
      if (this.get('selectedConfigGroup.isDefault')) {
        return groups.length ? groups.findProperty('isDefault', true) : null;
      } else {
        return groups.length ? groups.findProperty('name', this.get('selectedConfigGroup.dependentConfigGroups')[serviceName]) : null;
      }
    }
  },

  /**
   * Save changed configs and config groups
   * @method saveConfigs
   */
  saveConfigs: function saveConfigs() {
    if (this.get('selectedConfigGroup.isDefault')) {
      this.saveConfigsForDefaultGroup();
    } else {
      this.saveConfigsForNonDefaultGroup();
    }
  },

  saveConfigsForNonDefaultGroup: function saveConfigsForNonDefaultGroup() {
    this.get('stepConfigs').forEach(function (stepConfig) {
      var serviceName = stepConfig.get('serviceName');
      var configs = stepConfig.get('configs');
      var configGroup = this.getGroupFromModel(serviceName);
      if (configGroup && !configGroup.get('isDefault')) {
        var overriddenConfigs = this.getConfigsForGroup(configs, configGroup.get('name'));

        if (Em.isArray(overriddenConfigs) && this.isOverriddenConfigsModified(overriddenConfigs, configGroup)) {
          var successCallback = this.get('content.serviceName') === serviceName ? 'putConfigGroupChangesSuccess' : null;
          this.saveGroup(overriddenConfigs, configGroup, this.get('serviceConfigVersionNote'), successCallback);
        }
      }
    }, this);
  },

  /**
   * @param {Array} overriddenConfigs
   * @returns {boolean}
   */
  isOverriddenConfigsModified: function isOverriddenConfigsModified(overriddenConfigs, group) {
    var hasChangedConfigs = overriddenConfigs.some(function (config) {
      return config.get('savedValue') !== config.get('value') || config.get('savedIsFinal') !== config.get('isFinal');
    });
    var overriddenConfigsNames = overriddenConfigs.mapProperty('name');
    return hasChangedConfigs || group.get('properties').some(function (property) {
      return !overriddenConfigsNames.contains(Em.get(property, 'name'));
    });
  },

  saveConfigsForDefaultGroup: function saveConfigsForDefaultGroup() {
    var data = [];
    this.get('stepConfigs').forEach(function (stepConfig) {
      var serviceConfig = this.getServiceConfigToSave(stepConfig.get('serviceName'), stepConfig.get('configs'));

      if (serviceConfig) {
        data.push(serviceConfig);
      }
    }, this);

    if (data.length) {
      this.putChangedConfigurations(data, 'doPUTClusterConfigurationSiteSuccessCallback');
    } else {
      this.onDoPUTClusterConfigurations();
    }
  },

  /*********************************** 0. HELPERS ********************************************/

  /**
   * tells controller in saving configs was started
   * for now just changes flag <code>saveInProgress<code> to true
   * @private
   * @method startSave
   */
  startSave: function startSave() {
    this.set("saveInProgress", true);
  },

  /**
   * tells controller that save has been finished
   * for now just changes flag <code>saveInProgress<code> to true
   * @private
   * @method completeSave
   */
  completeSave: function completeSave() {
    this.set("saveInProgress", false);
  },

  /**
   * Are some unsaved changes available
   * @returns {boolean}
   * @method hasUnsavedChanges
   */
  hasUnsavedChanges: function hasUnsavedChanges() {
    return !Em.isNone(this.get('hash')) && this.get('hash') !== this.getHash();
  },

  /*********************************** 1. PRE SAVE CHECKS ************************************/

  /**
   * show some warning popups before user save configs
   * @private
   * @method showWarningPopupsBeforeSave
   */
  showWarningPopupsBeforeSave: function showWarningPopupsBeforeSave() {
    var self = this;
    if (this.isDirChanged()) {
      App.showConfirmationPopup(function () {
        self.showChangedDependentConfigs(null, function () {
          self.restartServicePopup();
        });
      }, Em.I18n.t('services.service.config.confirmDirectoryChange').format(self.get('content.displayName')), this.completeSave.bind(this));
    } else {
      self.showChangedDependentConfigs(null, function () {
        self.restartServicePopup();
      }, this.completeSave.bind(this));
    }
  },

  /**
   * Runs config validation before save
   * @private
   * @method restartServicePopup
   */
  restartServicePopup: function restartServicePopup() {
    this.serverSideValidation().done(this.saveConfigs.bind(this)).fail(this.completeSave.bind(this));
  },

  /**
   * Define if user has changed some dir properties
   * @return {Boolean}
   * @private
   * @method isDirChanged
   */
  isDirChanged: function isDirChanged() {
    var dirChanged = false;
    var serviceName = this.get('content.serviceName');

    if (serviceName === 'HDFS') {
      var hdfsConfigs = this.get('stepConfigs').findProperty('serviceName', 'HDFS').get('configs');
      if (hdfsConfigs.findProperty('name', 'dfs.namenode.name.dir') && hdfsConfigs.findProperty('name', 'dfs.namenode.name.dir').get('isNotDefaultValue') || hdfsConfigs.findProperty('name', 'dfs.namenode.checkpoint.dir') && hdfsConfigs.findProperty('name', 'dfs.namenode.checkpoint.dir').get('isNotDefaultValue') || hdfsConfigs.findProperty('name', 'dfs.datanode.data.dir') && hdfsConfigs.findProperty('name', 'dfs.datanode.data.dir').get('isNotDefaultValue')) {
        dirChanged = true;
      }
    }
    return dirChanged;
  },

  /*********************************** 2. GENERATING DATA TO SAVE ****************************/

  /**
   * get config properties for that fileNames that was changed
   * @param stepConfigs
   * @private
   * @returns {Array}
   */
  getModifiedConfigs: function getModifiedConfigs(stepConfigs) {
    var modifiedConfigs = stepConfigs
    // get only modified and created configs
    .filter(function (config) {
      return config.get('isNotDefaultValue') || config.get('isNotSaved');
    })
    // get file names and add file names that was modified, for example after property removing
    .mapProperty('filename').concat(this.get('modifiedFileNames')).uniq()
    // get configs by filename
    .map(function (fileName) {
      return stepConfigs.filterProperty('filename', fileName);
    });

    if (!!modifiedConfigs.length) {
      // concatenate results
      modifiedConfigs = modifiedConfigs.reduce(function (current, prev) {
        return current.concat(prev);
      });
    }
    return modifiedConfigs;
  },

  /**
   * get configs that belongs to config group
   * @param stepConfigs
   * @private
   * @param configGroupName
   */
  getConfigsForGroup: function getConfigsForGroup(stepConfigs, configGroupName) {
    var overridenConfigs = [];

    stepConfigs.filterProperty('overrides').forEach(function (config) {
      overridenConfigs = overridenConfigs.concat(config.get('overrides'));
    });
    // find custom original properties that assigned to selected config group
    return overridenConfigs.concat(stepConfigs.filterProperty('group').filter(function (config) {
      return config.get('group.name') == configGroupName;
    }));
  },

  /**
   *
   * @param serviceName
   * @param configs
   * @private
   * @returns {*}
   */
  getServiceConfigToSave: function getServiceConfigToSave(serviceName, configs) {

    if (serviceName === 'YARN') {
      configs = App.config.textareaIntoFileConfigs(configs, 'capacity-scheduler.xml');
    }

    //generates list of properties that was changed
    var modifiedConfigs = this.getModifiedConfigs(configs);
    var serviceFileNames = Object.keys(App.StackService.find(serviceName).get('configTypes')).map(function (type) {
      return App.config.getOriginalFileName(type);
    });

    // save modified original configs that have no group and are not Undefined label
    modifiedConfigs = this.saveSiteConfigs(modifiedConfigs.filter(function (config) {
      return !config.get('group') && !config.get('isUndefinedLabel');
    }));

    if (!Em.isArray(modifiedConfigs) || modifiedConfigs.length == 0) return null;

    var fileNamesToSave = modifiedConfigs.mapProperty('filename').concat(this.get('modifiedFileNames')).filter(function (filename) {
      return serviceFileNames.contains(filename);
    }).uniq();

    var configsToSave = this.generateDesiredConfigsJSON(modifiedConfigs, fileNamesToSave, this.get('serviceConfigVersionNote'));

    if (configsToSave.length > 0) {
      return JSON.stringify({
        Clusters: {
          desired_config: configsToSave
        }
      });
    } else {
      return null;
    }
  },

  /**
   * save site configs
   * @param configs
   * @private
   * @method saveSiteConfigs
   */
  saveSiteConfigs: function saveSiteConfigs(configs) {
    this.formatConfigValues(configs);
    return configs;
  },

  /**
   * Represent boolean value as string (true => 'true', false => 'false') and trim other values
   * @param serviceConfigProperties
   * @private
   * @method formatConfigValues
   */
  formatConfigValues: function formatConfigValues(serviceConfigProperties) {
    serviceConfigProperties.forEach(function (_config) {
      if (typeof _config.get('value') === "boolean") _config.set('value', _config.value.toString());
      _config.set('value', App.config.trimProperty(_config, true));
    });
  },

  /*********************************** 3. GENERATING JSON TO SAVE *****************************/

  /**
   * Map that contains last used timestamp.
   * There is a case when two config groups can update same filename almost simultaneously
   * so they have equal timestamp and this causes collision. So to prevent this we need to check
   * if specific filename with specific timestamp is not saved yet.
   *
   * @type {Object}
   */
  _timeStamps: {},

  /**
   * generating common JSON object for desired configs
   * @param configsToSave
   * @param fileNamesToSave
   * @param {string} [serviceConfigNote='']
   * @param {boolean} [ignoreVersionNote=false]
   * @returns {Array}
   */
  generateDesiredConfigsJSON: function generateDesiredConfigsJSON(configsToSave, fileNamesToSave, serviceConfigNote, ignoreVersionNote) {
    var desired_config = [];
    if (Em.isArray(configsToSave) && Em.isArray(fileNamesToSave) && fileNamesToSave.length && configsToSave.length) {
      serviceConfigNote = serviceConfigNote || "";

      fileNamesToSave.forEach(function (fName) {

        if (this.allowSaveSite(fName)) {
          var properties = configsToSave.filterProperty('filename', fName);
          var type = App.config.getConfigTagFromFileName(fName);
          desired_config.push(this.createDesiredConfig(type, properties, serviceConfigNote, ignoreVersionNote));
        }
      }, this);
    }
    return desired_config;
  },

  /**
   * For some file names we have a restriction
   * and can't save them, in this case method will return false
   *
   * @param fName
   * @returns {boolean}
   */
  allowSaveSite: function allowSaveSite(fName) {
    switch (App.config.getConfigTagFromFileName(fName)) {
      case 'mapred-queue-acls':
        return false;
      case 'core-site':
        return this.allowSaveCoreSite();
      default:
        return true;
    }
  },

  /**
   * Defines conditions in which core-site can be saved
   *
   * @returns {boolean}
   */
  allowSaveCoreSite: function allowSaveCoreSite() {
    return this.get('currentServices').some(function (service) {
      return this.get('coreSiteServiceNames').contains(service.get('serviceName')) || this.get('coreSiteServiceType') === service.get('serviceType');
    }, this);
  },

  /**
   * generating common JSON object for desired config
   * @param {string} type - file name without '.xml'
   * @param {App.ConfigProperty[]} properties - array of properties from model
   * @param {string} [serviceConfigNote='']
   * @param {boolean} [ignoreVersionNote=false]
   * @returns {{type: string, tag: string, properties: {}, properties_attributes: {}|undefined, service_config_version_note: string|undefined}}
   */
  createDesiredConfig: function createDesiredConfig(type, properties, serviceConfigNote, ignoreVersionNote) {
    Em.assert('type should be defined', type);
    var desired_config = {
      "type": type,
      "properties": {}
    };
    if (!ignoreVersionNote) {
      desired_config.service_config_version_note = serviceConfigNote || "";
    }
    var attributes = { final: {}, password: {}, user: {}, group: {}, text: {}, additional_user_property: {}, not_managed_hdfs_path: {}, value_from_property_file: {} };
    if (Em.isArray(properties)) {
      properties.forEach(function (property) {

        if (Em.get(property, 'isRequiredByAgent') !== false) {
          var name = stringUtils.unicodeEscape(Em.get(property, 'name'), /[\/]/g);
          desired_config.properties[name] = this.formatValueBeforeSave(property);
          /**
           * add is final value
           */
          if (Em.get(property, 'isFinal')) {
            attributes.final[name] = "true";
          }
          if (Em.get(property, 'propertyType') != null) {
            Em.get(property, 'propertyType').map(function (propType) {
              attributes[propType.toLowerCase()][name] = "true";
            });
          }
        }
      }, this);
    }

    if (Object.keys(attributes.final).length || Object.keys(attributes.password).length) {
      desired_config.properties_attributes = attributes;
    }
    return desired_config;
  },

  /**
   * format value before save performs some changing of values
   * according to the rules that includes heapsizeException trimming and some custom rules
   * @param {App.ConfigProperty} property
   * @returns {string}
   */
  formatValueBeforeSave: function formatValueBeforeSave(property) {
    var name = Em.get(property, 'name');
    var value = Em.get(property, 'value');
    var kdcTypesMap = App.router.get('mainAdminKerberosController.kdcTypesValues');

    if (this.addM(name, value)) {
      return value += "m";
    }
    if (typeof value === "boolean") {
      return value.toString();
    }
    switch (name) {
      case 'kdc_type':
        return Em.keys(kdcTypesMap).filter(function (key) {
          return kdcTypesMap[key] === value;
        })[0];
      case 'storm.zookeeper.servers':
      case 'nimbus.seeds':
        if (Em.isArray(value)) {
          return JSON.stringify(value).replace(/"/g, "'");
        } else {
          return value;
        }
        break;
      default:
        return App.config.trimProperty(property);
    }
  },

  /**
   * Site object name follow the format *permsize/*heapsize and the value NOT ends with "m"
   *
   * @param name
   * @param value
   * @returns {*|boolean}
   */
  addM: function addM(name, value) {
    return this.get('heapsizeRegExp').test(name) && !this.get('heapsizeException').contains(name) && !value.endsWith("m");
  },

  /*********************************** 4. AJAX REQUESTS **************************************/

  /**
   * save config group
   * @param overriddenConfigs
   * @param selectedConfigGroup
   * @param configVersionNote
   * @param successCallback
   */
  saveGroup: function saveGroup(overriddenConfigs, selectedConfigGroup, configVersionNote, successCallback) {
    var fileNamesToSave = overriddenConfigs.mapProperty('filename').uniq();
    var group = Ember.typeOf(selectedConfigGroup) === "instance" ? selectedConfigGroup.toJSON() : selectedConfigGroup;
    var groupHosts = group.hosts.map(function (hostName) {
      return { "host_name": hostName };
    });

    var groupData = {
      ConfigGroup: {
        "cluster_name": App.get('clusterName') || this.get('clusterName'),
        "group_name": group.name,
        "tag": group.service_id,
        "service_name": group.service_id,
        "description": group.description,
        "hosts": groupHosts,
        "service_config_version_note": configVersionNote || "",
        "desired_configs": this.generateDesiredConfigsJSON(overriddenConfigs, fileNamesToSave, null, true)
      }
    };

    if (group.is_temporary) {
      this.createConfigGroup(groupData, successCallback);
    } else {
      groupData.ConfigGroup.id = group.id;
      this.updateConfigGroup(groupData, successCallback);
    }
  },

  /**
   *
   * @param data
   * @param successCallback
   * @returns {*|$.ajax}
   */
  createConfigGroup: function createConfigGroup(data, successCallback) {
    var ajaxOptions = {
      name: 'wizard.step8.apply_configuration_groups',
      sender: this,
      data: {
        data: JSON.stringify(data)
      }
    };
    if (successCallback) {
      ajaxOptions.success = successCallback;
    }
    return App.ajax.send(ajaxOptions);
  },

  /**
   * persist properties of config groups to server
   * show result popup if <code>showPopup</code> is true
   * @param data {Object}
   * @param [successCallback] {String}
   * @method putConfigGroupChanges
   */
  updateConfigGroup: function updateConfigGroup(data, successCallback) {
    var ajaxOptions = {
      name: 'config_groups.update_config_group',
      sender: this,
      data: {
        id: data.ConfigGroup.id,
        configGroup: data
      }
    };
    if (successCallback) {
      ajaxOptions.success = successCallback;
    }
    return App.ajax.send(ajaxOptions);
  },

  /**
   * Saves configuration of set of sites. The provided data
   * contains the site name and tag to be used.
   * @param {Object[]} services
   * @param {String} [successCallback]
   * @param {Function} [alwaysCallback]
   * @return {$.ajax}
   * @method putChangedConfigurations
   */
  putChangedConfigurations: function putChangedConfigurations(services, successCallback, alwaysCallback) {
    var ajaxData = {
      name: 'common.across.services.configurations',
      sender: this,
      data: {
        data: '[' + services.toString() + ']'
      },
      error: 'doPUTClusterConfigurationSiteErrorCallback'
    };
    if (successCallback) {
      ajaxData.success = successCallback;
    }
    if (alwaysCallback) {
      ajaxData.callback = alwaysCallback;
    }
    return App.ajax.send(ajaxData);
  },

  /*********************************** 5. AFTER SAVE INFO ************************************/

  /**
   * @private
   * @method putConfigGroupChangesSuccess
   */
  putConfigGroupChangesSuccess: function putConfigGroupChangesSuccess() {
    this.set('saveConfigsFlag', true);
    this.onDoPUTClusterConfigurations();
  },

  /**
   * @private
   * @method doPUTClusterConfigurationSiteSuccessCallback
   */
  doPUTClusterConfigurationSiteSuccessCallback: function doPUTClusterConfigurationSiteSuccessCallback() {
    var doConfigActions = true;
    this.onDoPUTClusterConfigurations(doConfigActions);
  },

  /**
   * @private
   * @method doPUTClusterConfigurationSiteErrorCallback
   */
  doPUTClusterConfigurationSiteErrorCallback: function doPUTClusterConfigurationSiteErrorCallback() {
    this.set('saveConfigsFlag', false);
    this.onDoPUTClusterConfigurations();
  },

  /**
   * On save configs handler. Open save configs popup with appropriate message
   * and clear config dependencies list.
   * @param  {Boolean} doConfigActions
   * @private
   * @method onDoPUTClusterConfigurations
   */
  onDoPUTClusterConfigurations: function onDoPUTClusterConfigurations(doConfigActions) {
    var status = 'unknown',
        result = {
      flag: this.get('saveConfigsFlag'),
      message: null,
      value: null
    },
        extendedModel = App.Service.extendedModel[this.get('content.serviceName')],
        currentService = extendedModel ? App[extendedModel].find(this.get('content.serviceName')) : App.Service.find(this.get('content.serviceName'));

    if (!result.flag) {
      result.message = Em.I18n.t('services.service.config.failSaveConfig');
    }

    App.router.get('clusterController').updateClusterData();
    var popupOptions = this.getSaveConfigsPopupOptions(result);
    if (currentService) {
      App.router.get('clusterController').triggerQuickLinksUpdate();
    }

    //  update configs for service actions
    App.router.get('mainServiceItemController').loadConfigs();

    this.showSaveConfigsPopup(popupOptions.header, result.flag, popupOptions.message, popupOptions.messageClass, popupOptions.value, status, popupOptions.urlParams, doConfigActions);
    this.clearAllRecommendations();
  },

  /**
   *
   * @param {object} result
   * @returns {object}
   */
  getSaveConfigsPopupOptions: function getSaveConfigsPopupOptions(result) {
    var options;
    if (result.flag === true) {
      options = {
        header: Em.I18n.t('services.service.config.saved'),
        message: Em.I18n.t('services.service.config.saved.message'),
        messageClass: 'alert alert-success',
        urlParams: ',ServiceComponentInfo/installed_count,ServiceComponentInfo/total_count'
      };

      if (this.get('content.serviceName') === 'HDFS') {
        options.urlParams += '&ServiceComponentInfo/service_name.in(HDFS)';
      }
    } else {
      options = {
        urlParams: '',
        header: Em.I18n.t('common.failure'),
        message: result.message,
        messageClass: 'alert alert-error',
        value: result.value
      };
    }
    return options;
  },

  /**
   * Show save configs popup
   * @return {App.ModalPopup}
   * @private
   * @method showSaveConfigsPopup
   */
  showSaveConfigsPopup: function showSaveConfigsPopup(header, flag, message, messageClass, value, status, urlParams, doConfigActions) {
    var self = this;
    return App.ModalPopup.show({
      header: header,
      primary: Em.I18n.t('ok'),
      secondary: null,
      isBackgroundPopupToBeShown: false,

      onPrimary: function onPrimary() {
        this.hide();
        if (!flag) {
          self.completeSave();
        }
        this.showBackgroundPopup();
      },
      onClose: function onClose() {
        this.hide();
        self.completeSave();
        this.showBackgroundPopup();
      },
      showBackgroundPopup: function showBackgroundPopup() {
        if (this.get('isBackgroundPopupToBeShown')) {
          App.router.get('backgroundOperationsController').showPopup();
        }
      },
      disablePrimary: true,
      bodyClass: Ember.View.extend({
        flag: flag,
        message: function () {
          return this.get('isLoaded') ? message : Em.I18n.t('services.service.config.saving.message');
        }.property('isLoaded'),
        messageClass: function () {
          return this.get('isLoaded') ? messageClass : 'alert alert-info';
        }.property('isLoaded'),
        setDisablePrimary: function () {
          this.get('parentView').set('disablePrimary', !this.get('isLoaded'));
        }.observes('isLoaded'),
        runningHosts: [],
        runningComponentCount: 0,
        unknownHosts: [],
        unknownComponentCount: 0,
        siteProperties: value,
        isLoaded: false,
        componentsFilterSuccessCallback: function componentsFilterSuccessCallback(response) {
          var count = 0,
              view = this,
              lazyLoadHosts = function lazyLoadHosts(dest) {
            lazyLoading.run({
              initSize: 20,
              chunkSize: 50,
              delay: 50,
              destination: dest,
              source: hosts,
              context: view
            });
          },

          /**
           * Map components for their hosts
           * Return format:
           * <code>
           *   {
           *    host1: [component1, component2, ...],
           *    host2: [component3, component4, ...]
           *   }
           * </code>
           * @return {object}
           */
          setComponents = function setComponents(item, components) {
            item.host_components.forEach(function (c) {
              var name = c.HostRoles.host_name;
              if (!components[name]) {
                components[name] = [];
              }
              components[name].push(App.format.role(item.ServiceComponentInfo.component_name, false));
            });
            return components;
          },

          /**
           * Map result of <code>setComponents</code> to array
           * @return {{name: string, components: string}[]}
           */
          setHosts = function setHosts(components) {
            var hosts = [];
            Em.keys(components).forEach(function (key) {
              hosts.push({
                name: key,
                components: components[key].join(', ')
              });
            });
            return hosts;
          },
              components = {},
              hosts = [];
          switch (status) {
            case 'unknown':
              response.items.filter(function (item) {
                return item.ServiceComponentInfo.total_count > item.ServiceComponentInfo.started_count + item.ServiceComponentInfo.installed_count;
              }).forEach(function (item) {
                var total = item.ServiceComponentInfo.total_count,
                    started = item.ServiceComponentInfo.started_count,
                    installed = item.ServiceComponentInfo.installed_count,
                    unknown = total - (started + installed);
                components = setComponents(item, components);
                count += unknown;
              });
              hosts = setHosts(components);
              this.set('unknownComponentCount', count);
              lazyLoadHosts(this.get('unknownHosts'));
              break;
            case 'started':
              response.items.filterProperty('ServiceComponentInfo.started_count').forEach(function (item) {
                var started = item.ServiceComponentInfo.started_count;
                components = setComponents(item, components);
                count += started;
                hosts = setHosts(components);
              });
              this.set('runningComponentCount', count);
              lazyLoadHosts(this.get('runningHosts'));
              break;
          }
        },
        componentsFilterErrorCallback: function componentsFilterErrorCallback() {
          this.set('isLoaded', true);
        },
        didInsertElement: function didInsertElement() {
          var context = this;
          var dfd = App.ajax.send({
            name: 'components.filter_by_status',
            sender: this,
            data: {
              clusterName: App.get('clusterName'),
              urlParams: urlParams
            },
            success: 'componentsFilterSuccessCallback',
            error: 'componentsFilterErrorCallback'
          });

          dfd.done(function () {
            if (doConfigActions && self.doConfigActions) {
              self.doConfigActions.bind(self)();
              var isBackgroundPopupToBeShown = self.isComponentActionsPresent.bind(self)();
              context.set('parentView.isBackgroundPopupToBeShown', isBackgroundPopupToBeShown);
            }
            if (flag) {
              self.loadStep();
            }
          });
        },
        getDisplayMessage: function () {
          var displayMsg = [];
          var siteProperties = this.get('siteProperties');
          if (siteProperties) {
            siteProperties.forEach(function (_siteProperty) {
              var displayProperty = _siteProperty.siteProperty;
              var displayNames = _siteProperty.displayNames;

              if (displayNames && displayNames.length) {
                if (displayNames.length === 1) {
                  displayMsg.push(displayProperty + Em.I18n.t('as') + displayNames[0]);
                } else {
                  var name;
                  displayNames.forEach(function (_name, index) {
                    if (index === 0) {
                      name = _name;
                    } else if (index === siteProperties.length - 1) {
                      name = name + Em.I18n.t('and') + _name;
                    } else {
                      name = name + ', ' + _name;
                    }
                  }, this);
                  displayMsg.push(displayProperty + Em.I18n.t('as') + name);
                }
              } else {
                displayMsg.push(displayProperty);
              }
            }, this);
          }
          return displayMsg;
        }.property('siteProperties'),

        runningHostsMessage: Em.computed.i18nFormat('services.service.config.stopService.runningHostComponents', 'runningComponentCount', 'runningHosts.length'),

        unknownHostsMessage: Em.computed.i18nFormat('services.service.config.stopService.unknownHostComponents', 'unknownComponentCount', 'unknownHosts.length'),

        templateName: require('templates/main/service/info/configs_save_popup')
      })
    });
  },

  /*********************************** 6. ADDITIONAL *******************************************/

  /**
   * If some configs are changed and user navigates away or select another config-group, show this popup with propose to save changes
   * @param {String} path
   * @param {object} callback - callback with action to change configs view(change group or version)
   * @return {App.ModalPopup}
   * @method showSavePopup
   */
  showSavePopup: function showSavePopup(transitionCallback, callback) {
    var self = this;
    var passwordWasChanged = this.get('passwordConfigsAreChanged');
    return App.ModalPopup.show({
      header: Em.I18n.t('common.warning'),
      bodyClass: Em.View.extend({
        templateName: require('templates/common/configs/save_configuration'),
        classNames: ['col-md-12'],
        showSaveWarning: true,
        showPasswordChangeWarning: passwordWasChanged,
        notesArea: Em.TextArea.extend({
          value: passwordWasChanged ? Em.I18n.t('dashboard.configHistory.info-bar.save.popup.notesForPasswordChange') : '',
          classNames: ['full-width'],
          placeholder: Em.I18n.t('dashboard.configHistory.info-bar.save.popup.placeholder'),
          didInsertElement: function didInsertElement() {
            if (this.get('value')) {
              this.onChangeValue();
            }
          },
          onChangeValue: function () {
            this.get('parentView.parentView').set('serviceConfigNote', this.get('value'));
          }.observes('value')
        })
      }),
      footerClass: Em.View.extend({
        templateName: require('templates/main/service/info/save_popup_footer'),
        isSaveDisabled: function () {
          return self.get('isSubmitDisabled');
        }.property()
      }),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.cancel'),
      onSave: function onSave() {
        self.set('serviceConfigVersionNote', this.get('serviceConfigNote'));
        self.saveStepConfigs();
        this.hide();
      },
      onDiscard: function onDiscard() {
        self.set('preSelectedConfigVersion', null);
        if (transitionCallback) {
          transitionCallback();
        } else if (callback) {
          self.doCancel();
          // Prevent multiple popups
          self.set('hash', self.getHash());
          callback();
        }
        this.hide();
      },
      onCancel: function onCancel() {
        this.hide();
      }
    });
  }
});

});

require.register("mixins/common/configs/enhanced_configs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');

App.EnhancedConfigsMixin = Em.Mixin.create(App.ConfigWithOverrideRecommendationParser, {

  /**
   * this value is used for observing
   * whether recommendations for dependent properties was received from server
   * @type {number}
   */
  recommendationTimeStamp: null,

  /**
   * this property is used to force update min/max values
   * for not default config groups
   * @type {boolean}
   */
  forceUpdateBoundaries: false,

  /**
   * object with loaded recommendations
   *
   * @type {?Object}
   */
  recommendationsConfigs: null,

  /**
   * list of recommendations that should be applied without any config changes
   *
   * @type {Object[]}
   */
  initialRecommendations: [],

  /**
   * flag is true when Ambari changes some of the dependent properties
   * @type {boolean}
   */
  hasChangedDependencies: Em.computed.and('isControllerSupportsEnhancedConfigs', 'changedProperties.length'),

  /**
   * defines is block with changed dependent configs should be shown
   * rely on controller
   * @type {boolean}
   */
  isControllerSupportsEnhancedConfigs: Em.computed.existsIn('name', ['wizardStep7Controller', 'mainServiceInfoConfigsController']),

  /**
   * Stores name and file name of changed config
   * This used only for capacity-scheduler
   *
   * @property {object}
   */
  currentlyChangedConfig: null,

  dependenciesGroupMessage: Em.I18n.t('popup.dependent.configs.dependencies.for.groups'),

  /**
   * ConfigType-Widget map
   * key - widget type
   * value - widget view
   * @type {object}
   */
  widgetTypeMap: {
    checkbox: 'CheckboxConfigWidgetView',
    combo: 'ComboConfigWidgetView',
    directory: 'TextFieldConfigWidgetView',
    directories: 'DirectoryConfigWidgetView',
    list: 'ListConfigWidgetView',
    password: 'PasswordConfigWidgetView',
    'radio-buttons': 'RadioButtonConfigWidgetView',
    slider: 'SliderConfigWidgetView',
    'text-field': 'TextFieldConfigWidgetView',
    'time-interval-spinner': 'TimeIntervalSpinnerView',
    toggle: 'ToggleConfigWidgetView',
    'text-area': 'StringConfigWidgetView',
    'label': 'LabelView',
    'test-db-connection': 'TestDbConnectionWidgetView'
  },

  configNameWidgetMixinMap: {
    num_llap_nodes: App.NumLlapNodesWidgetMixin
  },

  /**
   * message for alert box for dependent configs
   * @type {string}
   */
  dependenciesMessage: function () {
    var changedProperties = this.get('changedProperties').filterProperty('saveRecommended');
    var changedServices = changedProperties.mapProperty('serviceName').uniq();
    var cfgLenSuffix = changedProperties.length === 1 ? 'singular' : 'plural';
    var sLenSuffix = changedServices.length === 1 ? 'singular' : 'plural';
    return Em.I18n.t('popup.dependent.configs.dependencies.config.' + cfgLenSuffix).format(changedProperties.length) + Em.I18n.t('popup.dependent.configs.dependencies.service.' + sLenSuffix).format(changedServices.length);
  }.property('changedProperties.length'),

  /**
   * dependent properties that was changed by Ambari
   * @type {Object[]}
   */
  changedProperties: function () {
    return this.get('recommendations').filter(function (dp) {
      return this.get('selectedConfigGroup.isDefault') && Em.get(dp, 'configGroup').contains('Default') || [this.get('selectedConfigGroup.name'), this.get('selectedConfigGroup.dependentConfigGroups') && this.get('selectedConfigGroup.dependentConfigGroups')[Em.get(dp, 'serviceName')]].contains(Em.get(dp, 'configGroup'));
    }, this);
  }.property('recommendations.@each.saveRecommended', 'recommendations.@each.recommendedValue', 'selectedConfigGroup'),

  /**
   * defines if change dependent group message should be shown
   * @type {boolean}
   */
  showSelectGroupsPopup: Em.computed.and('!selectedConfigGroup.isDefault', 'selectedService.dependentServiceNames.length'),

  /**
    * set default values for dependentGroups
   * @method setDependentGroups
   */
  setDependentGroups: function () {
    if (this.shouldSetDependentConfigs()) {
      this.get('selectedService.dependentServiceNames').forEach(function (serviceName) {
        if (!this.get('selectedConfigGroup.dependentConfigGroups')[serviceName]) {
          var stepConfig = this.get('stepConfigs').findProperty('serviceName', serviceName);
          if (stepConfig) {
            this.setDependentConfigGroups(stepConfig, serviceName);
          }
        }
      }, this);
    }
  }.observes('selectedConfigGroup'),

  /**
   *
   * @param {Em.Object} stepConfig
   * @param {string} serviceName
   */
  setDependentConfigGroups: function setDependentConfigGroups(stepConfig, serviceName) {
    stepConfig.get('configGroups').filterProperty('isDefault', false).forEach(function (configGroup) {
      this.get('selectedService.configGroups').filterProperty('isDefault', false).forEach(function (currentServiceGroup) {
        if (currentServiceGroup.get('dependentConfigGroups')[serviceName] !== configGroup.get('name')) {
          var dependentGroups = $.extend({}, this.get('selectedConfigGroup.dependentConfigGroups'));
          dependentGroups[serviceName] = configGroup.get('name');
          this.set('selectedConfigGroup.dependentConfigGroups', dependentGroups);
        }
      }, this);
    }, this);
  },

  /**
   *
   * @returns {boolean}
   */
  shouldSetDependentConfigs: function shouldSetDependentConfigs() {
    return Boolean(this.get('selectedConfigGroup') && this.get('isControllerSupportsEnhancedConfigs') && !this.get('selectedConfigGroup.isDefault') && this.get('selectedService.dependentServiceNames.length'));
  },

  /******************************METHODS THAT WORKS WITH DEPENDENT CONFIGS *************************************/

  /**
   * get config group object for current service
   * @param serviceName
   * @returns {App.ConfigGroup|null}
   */
  getGroupForService: function getGroupForService(serviceName) {
    if (!this.get('stepConfigs') || this.get('stepConfigs.length') === 0) {
      return null;
    }
    if (this.get('selectedService.serviceName') === serviceName) {
      return this.get('selectedConfigGroup');
    } else {
      var stepConfig = this.get('stepConfigs').findProperty('serviceName', serviceName);
      if (stepConfig) {
        var groups = stepConfig.get('configGroups');
        if (this.get('selectedConfigGroup.isDefault')) {
          return groups.length ? groups.findProperty('isDefault') : null;
        } else {
          return groups.length ? groups.findProperty('name', this.get('selectedConfigGroup.dependentConfigGroups')[serviceName]) : null;
        }
      } else {
        return null;
      }
    }
  },

  clearRecommendationsInfo: function clearRecommendationsInfo() {
    this.set('recommendationsConfigs', null);
  },

  clearRecommendations: function clearRecommendations() {
    this.clearRecommendationsInfo();
    this.clearAllRecommendations();
  },

  /**
   * sends request to get values for dependent configs
   * @param {{type: string, name: string}[]} changedConfigs - list of changed configs to track recommendations
   * @param {Function} [onComplete]
   * @returns {$.ajax|null}
   */
  loadConfigRecommendations: function loadConfigRecommendations(changedConfigs, onComplete) {
    var updateDependencies = Em.isArray(changedConfigs) && changedConfigs.length > 0;
    var stepConfigs = this.get('stepConfigs');
    var requiredTags = [];
    var isAutoComplete = !updateDependencies && this.get('isRecommendationsAutoComplete') !== undefined;
    if (this.get('isRecommendationsAutoComplete') !== undefined) {
      this.set('isRecommendationsAutoComplete', isAutoComplete);
    }

    if (updateDependencies || Em.isNone(this.get('recommendationsConfigs'))) {
      var recommendations = isAutoComplete ? {} : this.get('hostGroups');
      var dataToSend = this.getConfigRecommendationsParams(updateDependencies, changedConfigs, isAutoComplete);
      this.modifyRecommendationConfigGroups(recommendations, isAutoComplete);

      if (!stepConfigs.someProperty('serviceName', 'MISC')) {
        requiredTags.push({ site: 'cluster-env', serviceName: 'MISC' });
      }
      if (Em.isArray(changedConfigs) && changedConfigs.mapProperty('type').uniq()[0] === 'capacity-scheduler') {
        this.set('currentlyChangedConfig', {
          name: 'capacity-scheduler',
          fileName: 'capacity-scheduler.xml'
        });
      } else {
        this.set('currentlyChangedConfig', null);
      }

      if (App.Service.find().someProperty('serviceName', 'HDFS') && !stepConfigs.someProperty('serviceName', 'HDFS')) {
        requiredTags.push({ site: 'core-site', serviceName: 'HDFS' });
      }

      this.setUserContext(dataToSend);
      if (requiredTags.length) {
        this.loadAdditionalSites(requiredTags, stepConfigs, recommendations, dataToSend, onComplete);
      } else {
        this.addRecommendationRequestParams(recommendations, dataToSend, stepConfigs);
        return this.getRecommendationsRequest(dataToSend, onComplete);
      }
    } else {
      if (onComplete) {
        onComplete();
      }
    }
    return $.Deferred().resolve().promise();
  },

  /**
   *
   * @param recommendations
   * @param dataToSend
   * @param stepConfigs
   */
  addRecommendationRequestParams: function addRecommendationRequestParams(recommendations, dataToSend, stepConfigs) {
    var isAutoComplete = Boolean(this.get('isRecommendationsAutoComplete'));
    if (!isAutoComplete) {
      recommendations.blueprint.configurations = blueprintUtils.buildConfigsJSON(stepConfigs);
    }
    dataToSend.recommendations = recommendations;
    dataToSend.serviceName = this.get('content.serviceName');
    dataToSend.clusterId = App.get('clusterId');
    dataToSend.autoComplete = String(isAutoComplete);
    // configsResponse - tells server to return only configurations in recommendations call
    dataToSend.configsResponse = String(isAutoComplete);
  },

  /**
   *
   * @param {Array} stepConfigs
   * @param {object} recommendations
   * @param {object} dataToSend
   * @param {Function} onComplete
   */
  loadAdditionalSites: function loadAdditionalSites(sites, stepConfigs, recommendations, dataToSend, onComplete) {
    var self = this;
    App.router.get('configurationController').getCurrentConfigsBySites(sites).done(function (configs) {
      stepConfigs = stepConfigs.concat(configs);

      self.addRecommendationRequestParams(recommendations, dataToSend, stepConfigs);
      self.getRecommendationsRequest(dataToSend, onComplete);
    });
  },

  /**
   *
   * @param {object} recommendations
   * @param {boolean} isAutoComplete
   */
  modifyRecommendationConfigGroups: function modifyRecommendationConfigGroups(recommendations, isAutoComplete) {
    var configGroup = this.get('selectedConfigGroup');

    if (configGroup && !configGroup.get('isDefault') && configGroup.get('hosts.length') > 0) {
      recommendations.config_groups = [this.buildConfigGroupJSON(this.get('selectedService.configs'), configGroup, isAutoComplete)];
    } else {
      delete recommendations.config_groups;
    }
  },

  /**
   *
   * @param {boolean} updateDependencies
   * @param {Array} changedConfigs
   * @param {boolean} isAutoComplete
   * @returns {{recommend: string, hosts: *, services: *, changed_configurations: *}}
   */
  getConfigRecommendationsParams: function getConfigRecommendationsParams(updateDependencies, changedConfigs, isAutoComplete) {
    return {
      recommend: updateDependencies ? 'configuration-dependencies' : 'configurations',
      hosts: isAutoComplete ? undefined : this.get('hostNames'),
      services: isAutoComplete ? undefined : this.get('serviceNames'),
      changed_configurations: updateDependencies ? changedConfigs : undefined
    };
  },

  getRecommendationsRequest: function getRecommendationsRequest(dataToSend, _callback) {
    var self = this;
    this.set('recommendationsInProgress', true);
    return App.ajax.send({
      name: 'config.recommendations',
      sender: self,
      data: {
        stackVersionUrl: App.get('stackVersionURL'),
        dataToSend: dataToSend
      },
      success: 'loadRecommendationsSuccess',
      error: 'loadRecommendationsError',
      callback: function callback() {
        self.set('recommendationsInProgress', false);
        self.set('recommendationTimeStamp', new Date().getTime());
        if (_callback) {
          _callback();
        }
      }
    });
  },

  setUserContext: function setUserContext(dataToSend) {
    var controllerName = this.get('content.controllerName');
    var changes = dataToSend.changed_configurations;
    if (changes) {
      dataToSend['user_context'] = { "operation": "EditConfig" };
    } else {
      if (!controllerName) {
        dataToSend['user_context'] = { "operation": "RecommendAttribute" };
      } else if (controllerName == 'addServiceController') {
        dataToSend['user_context'] = {
          "operation": "AddService",
          "operation_details": (this.get('content.services') || []).filterProperty('isSelected').filterProperty('isInstalled', false).mapProperty('serviceName').join(',')
        };
      } else if (controllerName == 'installerController') {
        dataToSend['user_context'] = { "operation": "ClusterCreate" };
      }
    }
  },

  /**
   * Defines if there is any changes made by user.
   * Check all properties except recommended properties from popup
   *
   * @returns {boolean}
   */
  isConfigHasInitialState: function isConfigHasInitialState() {
    return Boolean(this.get('selectedConfigGroup.isDefault') && !Em.isNone(this.get('recommendationsConfigs')) && !this.get('stepConfigs').filter(function (stepConfig) {
      return stepConfig.get('changedConfigProperties').filter(function (c) {
        var changedConfigIds = this.get('changedProperties').map(function (changed) {
          return App.config.configId(changed.propertyName, changed.propertyFileName);
        });
        return !changedConfigIds.contains(App.config.configId(c.get('name'), c.get('filename')));
      }, this).length;
    }, this).length);
  },

  /**
   * generates JSON with config group info to send it for recommendations
   * @param configs
   * @param configGroup
   * @param {boolean} isAutoComplete
   * @returns {{configurations: Object[], hosts: string[]}}
   */
  buildConfigGroupJSON: function buildConfigGroupJSON(configs, configGroup, isAutoComplete) {
    Em.assert('configGroup can\'t be null', configGroup);
    if (isAutoComplete) {
      return {
        group_id: Number(configGroup.get('id'))
      };
    }
    var hosts = configGroup.get('hosts');
    var configurations = {};
    var overrides = configs.forEach(function (cp) {
      var override = cp.get('overrides') && cp.get('overrides').findProperty('group.name', configGroup.get('name'));
      if (override) {
        var tag = App.config.getConfigTagFromFileName(cp.get('filename'));
        if (!configurations[tag]) {
          configurations[tag] = { properties: {} };
        }
        configurations[tag].properties[cp.get('name')] = override.get('value');
      }
    });
    return {
      configurations: [configurations],
      hosts: hosts,
      group_id: Number(configGroup.get('id'))
    };
  },

  /**
   * shows popup with results for recommended value
   * if case properties that was changes belongs to not default group
   * user should pick to what config group from dependent service dependent properties will be saved
   * @param data
   * @param opt
   * @param params
   * @method dependenciesSuccess
   */
  loadRecommendationsSuccess: function loadRecommendationsSuccess(data, opt, params) {
    this._saveRecommendedValues(data, params.dataToSend.changed_configurations);
    if (this.isConfigHasInitialState()) {
      /** clearing all recommendations info **/
      this.undoRedoRecommended(this.get('recommendations'), false);
      this.clearAllRecommendations();

      /**
       * resetting recommendations to initial state
       * this case can be present when installed services depends on new added service on ADW
       **/
      this.undoRedoRecommended(this.get('initialRecommendations'), true);
      this.get('initialRecommendations').forEach(function (r) {
        this.get('recommendations').pushObject(r);
      }, this);
    }
    this.set("recommendationsConfigs", Em.get(data, "resources.0.recommendations.blueprint.configurations"));
  },

  loadRecommendationsError: Em.K,

  changedDependentGroup: function changedDependentGroup() {
    var dependentServices = this.get('selectedService.dependentServiceNames');
    var isInstallWizard = this.get('content.controllerName') === 'installerController';
    var installedServices = App.Service.find().mapProperty('serviceName');
    var services = this.get('stepConfigs').filter(function (stepConfig) {
      if (dependentServices.contains(stepConfig.get('serviceName'))) {
        return isInstallWizard ? true : installedServices.contains(stepConfig.get('serviceName'));
      }
      return false;
    }, this);
    App.showSelectGroupsPopup(this.get('selectedService.serviceName'), this.get('selectedService.configGroups').findProperty('name', this.get('selectedConfigGroup.name')), services, this.get('recommendations'));
  },

  /**
   * saves values from response for dependent config properties to <code>recommendations<code>
   * @param data
   * @param [changedConfigs=null]
   * @method saveRecommendedValues
   * @private
   */
  _saveRecommendedValues: function _saveRecommendedValues(data, changedConfigs) {
    Em.assert('invalid data - `data.resources[0].recommendations.blueprint.configurations` not defined ', data && data.resources[0] && Em.get(data.resources[0], 'recommendations.blueprint.configurations'));
    var recommendations = data.resources[0].recommendations;
    if (recommendations['config-groups'] && this.get('selectedConfigGroup') && !this.get('selectedConfigGroup.isDefault')) {
      this.saveConfigGroupsRecommendations(recommendations, changedConfigs);
    } else {
      var configObject = recommendations.blueprint.configurations;
      this.get('stepConfigs').forEach(function (stepConfig) {
        this.updateConfigsByRecommendations(configObject, stepConfig.get('configs'), changedConfigs);
      }, this);
      this.addByRecommendations(configObject, changedConfigs);
    }
    this.cleanUpRecommendations();
  },

  /**
   *
   * @param {object} recommendations
   * @param {?Array} changedConfigs
   */
  saveConfigGroupsRecommendations: function saveConfigGroupsRecommendations(recommendations, changedConfigs) {
    var configForGroup = recommendations['config-groups'][0];
    this.get('stepConfigs').forEach(function (stepConfig) {
      var configGroup = this.getGroupForService(stepConfig.get('serviceName'));
      if (configGroup && this.isConfigGroupAffected(configForGroup.hosts, configGroup.get('hosts'))) {
        this.updateOverridesByRecommendations(configForGroup.configurations, stepConfig.get('configs'), changedConfigs, configGroup);
        this.updateOverridesByRecommendations(configForGroup.dependent_configurations, stepConfig.get('configs'), changedConfigs, configGroup);
        this.toggleProperty('forceUpdateBoundaries');
      }
    }, this);
  },

  /**
   * determine whether hosts of group affected by config modifications
   * @param {Array} affectedHosts
   * @param {Array} groupHosts
   * @returns {boolean}
   */
  isConfigGroupAffected: function isConfigGroupAffected(affectedHosts, groupHosts) {
    return _.intersection(affectedHosts, groupHosts).length > 0;
  },

  /**
   * method to show popup with dependent configs
   * @method showChangedDependentConfigs
   */
  showChangedDependentConfigs: function showChangedDependentConfigs(event, callback, secondary) {
    var self = this;
    var recommendations = event ? this.get('changedProperties') : this.get('recommendations'),
        recommendedChanges = recommendations.filterProperty('isEditable'),
        requiredChanges = this.filterRequiredChanges(recommendations);
    if (recommendedChanges.length > 0 || requiredChanges.length > 0) {
      App.showDependentConfigsPopup(recommendedChanges, requiredChanges, function () {
        self.onSaveRecommendedPopup(recommendedChanges.concat(requiredChanges));
        if (callback) callback();
      }, secondary, this);
    } else {
      if (callback) callback();
    }
  },

  /**
   *
   * @param {Array} recommendations
   * @returns {Array}
   */
  filterRequiredChanges: function filterRequiredChanges(recommendations) {
    return recommendations.filter(function (recommendation) {
      if (recommendation.isEditable === false) {
        if (!this.get('selectedConfigGroup.isDefault')) {
          return App.ServiceConfigGroup.defaultGroupName !== recommendation.configGroup;
        } else {
          return true;
        }
      }
    }, this);
  },

  /**
   * update configs when toggle checkbox on dependent configs popup
   */
  onSaveRecommendedPopup: function onSaveRecommendedPopup(recommendations) {
    var propertiesToUpdate = recommendations.filter(function (c) {
      return Em.get(c, 'saveRecommendedDefault') !== Em.get(c, 'saveRecommended');
    }),
        propertiesToUndo = propertiesToUpdate.filterProperty('saveRecommended', false),
        propertiesToRedo = propertiesToUpdate.filterProperty('saveRecommended', true);

    this.undoRedoRecommended(propertiesToUndo, false);
    this.undoRedoRecommended(propertiesToRedo, true);
    this.set('recommendationTimeStamp', new Date().getTime());
  },

  /**
   * run through config properties list (form dependent popup)
   * and set value to default (undo) or recommended (redo)
   * this happens when toggle checkbox in popup
   * @param {Object[]} propertiesToUpdate
   * @param {boolean} redo
   */
  undoRedoRecommended: function undoRedoRecommended(propertiesToUpdate, redo) {
    propertiesToUpdate.forEach(function (p) {
      var initial = redo ? Em.get(p, 'initialValue') : Em.get(p, 'recommendedValue');
      var recommended = redo ? Em.get(p, 'recommendedValue') : Em.get(p, 'initialValue');
      var stepConfig = this.get('stepConfigs').findProperty('serviceName', Em.get(p, 'serviceName'));
      var config = stepConfig.get('configs').find(function (scp) {
        return scp.get('name') === Em.get(p, 'propertyName') && scp.get('filename') === App.config.getOriginalFileName(Em.get(p, 'propertyFileName'));
      });
      var selectedGroup = App.ServiceConfigGroup.find().filterProperty('serviceName', Em.get(p, 'serviceName')).findProperty('name', Em.get(p, 'configGroup'));

      if (selectedGroup.get('isDefault')) {
        this.setRecommendedForDefaultGroup(recommended, stepConfig, p, initial, config);
      } else {
        this.setRecommendedForGroup(recommended, selectedGroup, config, initial);
      }
    }, this);
  },

  /**
   *
   * @param {string} recommended
   * @param {Em.Object} selectedGroup
   * @param {object} config
   * @param {string}initial
   */
  setRecommendedForGroup: function setRecommendedForGroup(recommended, selectedGroup, config, initial) {
    if (Em.isNone(recommended)) {
      config.get('overrides').removeObject(config.getOverride(selectedGroup.get('name')));
    } else if (Em.isNone(initial)) {
      this._addConfigOverrideRecommendation(config, recommended, null, selectedGroup);
    } else {
      var override = config.getOverride(selectedGroup.get('name'));
      if (override) {
        override.set('value', recommended);
      }
    }
  },

  /**
   *
   * @param {string} recommended
   * @param {Em.Object} stepConfig
   * @param {object|Em.Object} prop
   * @param {string} initial
   * @param {object} config
   */
  setRecommendedForDefaultGroup: function setRecommendedForDefaultGroup(recommended, stepConfig, prop, initial, config) {
    var name = Em.get(prop, 'propertyName'),
        filename = Em.get(prop, 'propertyFileName');

    if (Em.isNone(recommended)) {
      stepConfig.get('configs').removeObject(config);
    } else if (Em.isNone(initial)) {
      var stackConfigProperty = App.configsCollection.getConfigByName(name, filename);
      stepConfig.get('configs').pushObject(this._createNewProperty(name, filename, Em.get(prop, 'serviceName'), recommended, stackConfigProperty ? stackConfigProperty.propertyDependsOn : []));
    } else {
      Em.set(config, 'value', recommended);
    }
  },

  saveInitialRecommendations: function saveInitialRecommendations() {
    this.get('recommendations').forEach(function (r) {
      this.get('initialRecommendations').pushObject(r);
    }, this);
  },

  /**
   * disable saving recommended value for current config. Remove recommendation for useroverriden values
   * @param config
   * @param {boolean} saveRecommended
   * @method removeCurrentFromDependentList
   */
  removeCurrentFromDependentList: function removeCurrentFromDependentList(config, saveRecommended) {
    var name = config.get('name'),
        fileName = config.get('filename'),
        group = config.get('group.name');
    var recommendation = this.getRecommendation(name, fileName, group);
    if (recommendation) {
      if (config.get('didUserOverrideValue')) {
        this.removeRecommendation(name, fileName, group);
      } else {
        this.saveRecommendation(recommendation, saveRecommended);
      }
    }
  },

  updateAttributesFromTheme: function updateAttributesFromTheme(serviceName) {
    this.prepareSectionsConfigProperties(serviceName);
    var service = this.get('stepConfigs').findProperty('serviceName', serviceName);
    if (service) {
      var serviceConfigs = service.get('configs'),
          configConditions = App.ThemeCondition.find().filter(function (condition) {
        var dependentConfigName = condition.get('configName'),
            dependentConfigFileName = condition.get('fileName'),
            configsToDependOn = condition.getWithDefault('configs', []);
        return serviceConfigs.some(function (serviceConfig) {
          var serviceConfigName = Em.get(serviceConfig, 'name'),
              serviceConfigFileName = Em.get(serviceConfig, 'filename');
          return serviceConfigName === dependentConfigName && serviceConfigFileName === dependentConfigFileName || configsToDependOn.some(function (config) {
            var configName = config.configName,
                fileName = config.fileName;

            return serviceConfigName === configName && serviceConfigFileName === fileName;
          });
        });
      });
      this.updateAttributesFromConditions(configConditions, serviceConfigs, serviceName);
    }
  },

  prepareSectionsConfigProperties: function prepareSectionsConfigProperties(serviceName) {
    var _this = this;

    var tabs = App.Tab.find().filterProperty('serviceName', serviceName);
    tabs.forEach(function (tab) {
      _this.processTab(tab);
      tab.get('sectionRows').forEach(function (row) {
        row.forEach(function (section) {
          section.get('subsectionRows').forEach(function (subRow) {
            subRow.forEach(function (subsection) {
              _this.setConfigsToContainer(subsection);
              subsection.get('subSectionTabs').forEach(function (subSectionTab) {
                _this.setConfigsToContainer(subSectionTab);
              });
            });
          });
        });
      });
    });
  },

  /**
   * set {code} configs {code} array of subsection or subsection tab.
   * Also correct widget should be used for each config (it's selected according to <code>widget.type</code> and
   * <code>widgetTypeMap</code>). It may throw an error if needed widget can't be found in the <code>widgetTypeMap</code>
   * @param containerObject
   */
  setConfigsToContainer: function setConfigsToContainer(containerObject) {
    containerObject.set('configs', []);

    containerObject.get('configProperties').forEach(function (configId) {
      var config = App.configsCollection.getConfig(configId);
      if (Em.get(config, 'widgetType')) {
        var stepConfig = this.get('stepConfigs').findProperty('serviceName', Em.get(config, 'serviceName'));
        if (!stepConfig) return;

        var configProperty = stepConfig.get('configs').findProperty('id', Em.get(config, 'id'));
        if (!configProperty) return;

        containerObject.get('configs').pushObject(configProperty);

        var widget = this.getWidgetView(config);
        Em.assert('Unknown config widget view for config ' + configProperty.get('id') + ' with type ' + Em.get(config, 'widgetType'), widget);

        var additionalProperties = {
          widget: widget,
          stackConfigProperty: config
        };

        var configConditions = App.ThemeCondition.find().filter(function (_configCondition) {
          // Filter config condition depending on the value of another config
          var conditionalConfigs = _configCondition.getWithDefault('configs', []).filterProperty('fileName', Em.get(config, 'filename')).filterProperty('configName', Em.get(config, 'name'));
          // Filter config condition depending on the service existence or service state
          var serviceConfigConditionFlag = _configCondition.get('configName') === Em.get(config, 'name') && _configCondition.get('fileName') === Em.get(config, 'filename') && _configCondition.get('resource') === 'service';
          var conditions = void 0;

          if (serviceConfigConditionFlag) {
            var configCondition = {
              configName: _configCondition.get('configName'),
              fileName: _configCondition.get('fileName')
            };
            conditions = conditionalConfigs.concat(configCondition);
          } else {
            conditions = conditionalConfigs;
          }
          return conditions && conditions.length;
        });

        if (configConditions && configConditions.length) {
          additionalProperties.configConditions = configConditions;
        }

        var configAction = App.ConfigAction.find().filterProperty('fileName', Em.get(config, 'filename')).findProperty('configName', Em.get(config, 'name'));

        if (configAction) {
          additionalProperties.configAction = configAction;
        }

        configProperty.setProperties(additionalProperties);

        if (configProperty.get('overrides')) {
          configProperty.get('overrides').setEach('stackConfigProperty', config);
        }
        if (configProperty.get('compareConfigs')) {
          configProperty.get('compareConfigs').invoke('setProperties', {
            isComparison: false,
            stackConfigProperty: config
          });
        }
      }
    }, this);
  },

  /**
   *
   * @param {object} config
   * @returns {Em.View}
   */
  getWidgetView: function getWidgetView(config) {
    var configWidgetType = Em.get(config, 'widgetType'),
        name = Em.get(config, 'name'),
        mixin = this.get('configNameWidgetMixinMap')[name],
        viewClass = App[this.get('widgetTypeMap')[configWidgetType]];
    return Em.isNone(mixin) ? viewClass : viewClass.extend(mixin);
  },

  updateAttributesFromConditions: function updateAttributesFromConditions(configConditions, serviceConfigs, serviceName) {
    var _this2 = this;

    var isConditionTrue = void 0;
    configConditions.forEach(function (configCondition) {
      var ifStatement = configCondition.get('if');
      if (configCondition.get('resource') === 'config') {
        isConditionTrue = App.configTheme.calculateConfigCondition(ifStatement, serviceConfigs);
        if (configCondition.get('type') === 'subsection' || configCondition.get('type') === 'subsectionTab') {
          _this2.changeSubsectionAttribute(configCondition, isConditionTrue);
        } else {
          _this2.changeConfigAttribute(configCondition, isConditionTrue, serviceName);
        }
      } else if (configCondition.get('resource') === 'service') {
        var service = App.Service.find().findProperty('serviceName', ifStatement);
        if (service) {
          isConditionTrue = true;
        } else if (!service && _this2.get('allSelectedServiceNames') && _this2.get('allSelectedServiceNames').length) {
          isConditionTrue = _this2.get('allSelectedServiceNames').contains(ifStatement);
        } else {
          isConditionTrue = false;
        }
        _this2.changeConfigAttribute(configCondition, isConditionTrue, serviceName);
      }
    });
  },

  /**
   *
   * @param configCondition {App.ThemeCondition}
   * @param isConditionTrue {boolean}
   */
  changeConfigAttribute: function changeConfigAttribute(configCondition, isConditionTrue, serviceName) {
    var conditionalConfigName = configCondition.get('configName'),
        conditionalConfigFileName = configCondition.get('fileName'),
        serviceConfigs = this.get('stepConfigs').findProperty('serviceName', serviceName).get('configs'),
        action = isConditionTrue ? configCondition.get('then') : configCondition.get('else'),
        valueAttributes = action.property_value_attributes;
    this.set('isChangingConfigAttributes', true);
    for (var key in valueAttributes) {
      if (valueAttributes.hasOwnProperty(key)) {
        var valueAttribute = App.StackConfigValAttributesMap[key] || key,
            conditionalConfig = serviceConfigs.filterProperty('filename', conditionalConfigFileName).findProperty('name', conditionalConfigName);
        if (conditionalConfig) {
          if (key === 'visible') {
            conditionalConfig.set('hiddenBySection', !valueAttributes[key]);
          }
          conditionalConfig.set(valueAttribute, valueAttributes[key]);
        }
      }
    }
    this.set('isChangingConfigAttributes', false);
  },
  /**
   *
   * @param subsectionCondition {App.ThemeCondition}
   * @param isConditionTrue {boolean}
   */
  changeSubsectionAttribute: function changeSubsectionAttribute(subsectionCondition, isConditionTrue) {
    var subsectionConditionName = subsectionCondition.get('name');
    var action = isConditionTrue ? subsectionCondition.get('then') : subsectionCondition.get('else');
    if (subsectionCondition.get('id')) {
      var valueAttributes = action.property_value_attributes;
      if (valueAttributes && !Em.none(valueAttributes.visible)) {
        var themeResource = void 0;
        if (subsectionCondition.get('type') === 'subsection') {
          themeResource = App.SubSection.find().find(function (subsection) {
            return subsection.get('name') === subsectionConditionName && subsectionCondition.get('themeName') === subsection.get('themeName');
          });
        } else if (subsectionCondition.get('type') === 'subsectionTab') {
          themeResource = App.SubSectionTab.find().find(function (subsectionTab) {
            return subsectionTab.get('name') === subsectionConditionName && subsectionCondition.get('themeName') === subsectionTab.get('themeName');
          });
        }
        themeResource.set('isHiddenByConfig', !valueAttributes.visible);
        themeResource.get('configs').setEach('hiddenBySection', !valueAttributes.visible);
        themeResource.get('configs').setEach('hiddenBySubSection', !valueAttributes.visible);
      }
    }
  },

  /**
   * Data reordering before tabs rendering.
   * Reorder all sections/subsections into rows based on their rowIndex
   * @param tab
   */
  processTab: function processTab(tab) {
    // process sections
    var sectionRows = [];
    var sections = tab.get('sections');
    for (var j = 0; j < sections.get('length'); j++) {
      var section = sections.objectAt(j);
      var sectionRow = sectionRows[section.get('rowIndex')];
      if (!sectionRow) {
        sectionRow = sectionRows[section.get('rowIndex')] = [];
      }
      sectionRow.push(section);

      //process subsections
      var subsections = section.get('subSections');
      var subsectionRows = [];
      for (var k = 0; k < subsections.get('length'); k++) {
        var subsection = subsections.objectAt(k);
        var subsectionRow = subsectionRows[subsection.get('rowIndex')];
        if (!subsectionRow) {
          subsectionRow = subsectionRows[subsection.get('rowIndex')] = [];
        }
        subsectionRow.push(subsection);
        // leave a title gap if one of the subsection on the same row within the same section has title
        if (subsection.get('displayName')) {
          subsectionRow.hasTitleGap = true;
        }
      }
      section.set('subsectionRows', subsectionRows);
    }
    tab.set('sectionRows', sectionRows);
  }
});

});

require.register("mixins/common/configs/toggle_isrequired", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ToggleIsRequiredMixin = Em.Mixin.create({
  /**
   * Override isRequired property of the configurations in given situation
   * @param serviceConfigs - make changes only for properties from this service
   */
  overrideConfigIsRequired: function overrideConfigIsRequired(serviceConfigs) {
    var excludeProperties = [{
      name: 'KERBEROS', // affected service
      exclude: ['kdc_hosts', 'admin_server_host', 'admin_principal', 'admin_password'] // affected properties
    }, {
      name: 'KERBEROS_GENERAL', // affected service
      exclude: ['kdc_hosts', 'admin_server_host', 'admin_principal', 'admin_password'] // affected properties
    }];
    var serviceName = serviceConfigs.get('serviceName'),
        service = excludeProperties.findProperty('name', serviceName),
        configs = serviceConfigs.get('configs');

    if (service && !Em.isEmpty(configs)) {
      service.exclude.forEach(function (property) {
        var serviceProperty = configs.findProperty('name', property);
        if (serviceProperty) {
          var value = serviceProperty.get('isRequired');
          Em.set(serviceProperty, "isRequired", !value);
          if (value && serviceProperty.get('value') === '') {
            // clear validation errors because validation does not clear isRequired validations
            Em.set(serviceProperty, "error", false);
            Em.set(serviceProperty, "errorMessage", '');
          }
          // validate property
          serviceProperty.validate();
        }
      });
    }
  }

});

});

require.register("mixins/common/configs/widgets/unique/num_llap_nodes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Slider for `num_llap_nodes` should have unique widget
 *
 * @type {Em.Mixin}
 */
App.NumLlapNodesWidgetMixin = Em.Mixin.create({

  /**
   * @type {boolean}
   */
  readOnly: Em.computed.alias('config.stackConfigProperty.valueAttributes.read_only'),

  /**
   * @type {boolean}
   */
  doNotShowWidget: function () {
    var self = this;
    Em.run.next(function () {
      self.handleReadOnlyAttribute();
    });
    if (this.get('readOnly')) {
      return false;
    }
    return this.get('isPropertyUndefined') || this.get('config.showAsTextBox');
  }.property('isPropertyUndefined', 'config.showAsTextBox', 'readOnly'),

  handleReadOnlyAttribute: function handleReadOnlyAttribute() {
    var readOnly = this.get('readOnly') || false;
    this.set('disabled', readOnly);
    this.set('supportSwitchToTextBox', !readOnly);
    var action = readOnly ? 'disable' : 'enable';
    this.toggleSlider(action);
  },

  toggleWidgetView: function toggleWidgetView() {
    this._super();
    var action = !this.get('config.showAsTextBox') && this.get('readOnly') ? 'disable' : 'enable';
    this.toggleSlider(action);
  },

  toggleWidgetState: function () {
    this.set('disabled', !this.get('config.isEditable'));
  }.observes('config.isEditable'),

  toggleSlider: function toggleSlider(action) {
    var self = this;
    Em.run.next(function () {
      Em.tryInvoke(self.get('slider'), action);
    });
  }

});

});

require.register("mixins/common/hosts/host_component_recommendation_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('mixins/common/blueprint');

var App = require('app');

/**
 * @typedef {object} RecommendComponentObject
 * @property {string} componentName name of the component
 * @property {number} [size=0] desired components size
 * @property {string[]} [hosts=[]] hosts assigned to component
 */

/**
 * @typedef {object} HostComponentRecommendationOptions
 * @property {string[]} hosts list of host names, in most cases all available host names
 * @property {RecommendComponentObject[]} components list of components
 * @property {string[]} services list of service names
 * @property {object} [blueprint=null] when null blueprint will be created by <code>HostComponentRecommendationOptions.components</code> attribute
 */

/**
 * @typedef {object} HostRecommendationRequestData
 * @property {string} stackVersionUrl stack version url
 * @property {string[]} hosts host names
 * @property {string[]} services service names
 * @property {string} recommend recommendation type e.g. 'host_groups'
 * @property {object} recommendations blueprint object
 */

/**
 * Contains methods to get server recommendation and validation for host components
 * @type {Em.Mixin}
 */
App.HostComponentRecommendationMixin = Em.Mixin.create(App.BlueprintMixin, {

  /**
   * Get recommendations for selected components
   * @method getRecommendedHosts
   * @param {HostComponentRecommendationOptions} recommend list of the components
   * @return {$.Deferred}
   */
  getRecommendedHosts: function getRecommendedHosts(options) {
    var opts = $.extend({
      services: [],
      hosts: [],
      components: [],
      blueprint: null
    }, options || {});

    opts.components = this.formatRecommendComponents(opts.components);
    return this.loadComponentsRecommendationsFromServer(this.getRecommendationRequestData(opts));
  },

  /**
   * @method formatRecommendComponents
   * @param {RecommendComponentsObject[]} components
   * @returns {Em.Object[]}
   */
  formatRecommendComponents: function formatRecommendComponents(components) {
    var res = [];
    if (!components) return [];
    components.forEach(function (component) {
      var componentName = Em.get(component, 'componentName');
      if (Em.get(component, 'hosts.length')) {
        Em.get(component, 'hosts').forEach(function (hostName) {
          res.push(Em.Object.create({
            componentName: componentName,
            hostName: hostName
          }));
        });
      }
    });
    return res;
  },

  /**
   * Returns request data for recommendation request
   * @param {HostComponentRecommendationOptions} options
   * @return {HostRecommendationRequestData}
   * @method getRecommendationRequestData
   */
  getRecommendationRequestData: function getRecommendationRequestData(options) {
    return {
      recommend: 'host_groups',
      stackVersionUrl: App.get('stackVersionURL'),
      hosts: options.hosts,
      services: options.services,
      recommendations: options.blueprint || this.getComponentsBlueprint(options.components)
    };
  },

  /**
   * Get recommendations info from API
   * @method loadComponentsRecommendationsFromServer
   * @param {object} recommendationData
   * @returns {$.Deferred}
   */
  loadComponentsRecommendationsFromServer: function loadComponentsRecommendationsFromServer(recommendationData) {
    return App.ajax.send({
      name: 'wizard.loadrecommendations',
      sender: this,
      data: recommendationData,
      success: 'loadRecommendationsSuccessCallback',
      error: 'loadRecommendationsErrorCallback'
    });
  },

  loadRecommendationsSuccessCallback: function loadRecommendationsSuccessCallback() {},
  loadRecommendationsErrorCallback: function loadRecommendationsErrorCallback() {}
});

});

require.register("mixins/common/hosts/host_component_validation_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('mixins/common/blueprint');

var App = require('app');

/**
 * @typedef {object} HostValidationRequestData
 * @property {string} stackVersionUrl stack version url
 * @property {string[]} hosts host names
 * @property {string[]} services service names
 * @property {string} validate validation type e.g. 'host_groups'
 * @property {object} recommendations blueprint object
 */

App.HostComponentValidationMixin = Em.Mixin.create(App.BlueprintMixin, {
  /**
   * Validate host components
   * @method validateSelectedHostComponents
   * @param {HostComponentRecommendationOptions} options
   * @return {$.Deferred}
   */
  validateSelectedHostComponents: function validateSelectedHostComponents(options) {
    var opts = $.extend({
      services: [],
      blueprint: null,
      hosts: [],
      components: []
    }, options || {});

    opts.components = this.formatValidateComponents(opts.components);
    return this.getHostComponentValidationRequest(this.getHostComponentValidationParams(opts));
  },

  /**
   * @method formatValidateComponents
   * @param {RecommendComponentObject[]} components
   * @returns {Em.Object[]}
   */
  formatValidateComponents: function formatValidateComponents(components) {
    var res = [];
    if (!components) return [];
    components.forEach(function (component) {
      var componentName = Em.get(component, 'componentName');
      if (Em.get(component, 'hosts.length')) {
        Em.get(component, 'hosts').forEach(function (hostName) {
          res.push(Em.Object.create({
            componentName: componentName,
            hostName: hostName
          }));
        });
      }
    });
    return res;
  },

  /**
   * Returns request data for validation request
   * @method getHostComponentValidationParams
   * @return {HostValidationRequestData}
   */
  getHostComponentValidationParams: function getHostComponentValidationParams(options) {
    return {
      stackVersionUrl: App.get('stackVersionURL'),
      hosts: options.hosts,
      services: options.services,
      validate: 'host_groups',
      recommendations: options.blueprint || this.getComponentsBlueprint(options.components)
    };
  },

  /**
   * Performs request to validate components location
   * @method getHostComponentValidationRequest
   * @param {object} validationData
   * @returns {$.Deferred}
   */
  getHostComponentValidationRequest: function getHostComponentValidationRequest(validationData) {
    return App.ajax.send({
      name: 'config.validations',
      sender: this,
      data: validationData,
      success: 'updateValidationsSuccessCallback',
      error: 'updateValidationsErrorCallback'
    });
  },

  updateValidationsSuccessCallback: function updateValidationsSuccessCallback() {},
  updateValidationsErrorCallback: function updateValidationsErrorCallback() {}
});

});

require.register("mixins/common/infinite_scroll_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
* @typedef {Object} InfiniteScrollMixinOptions
* @property {String} appendHtml html to append when scroll ends and callback executed. It's common
*   that root node has unique <code>id</code> or <code>class</code> attributes.
* @property {Function} callback function to execute when scroll ends. This function should return
*   <code>$.Deferred().promise()</code> instance
* @property {Function} onReject function to execute when <code>callback</code> rejected.
*/

/**
 * @mixin App.InfiniteScrollMixin
 * This mixin provides methods to attach infinite scroll to specific scrollable element.
 *
 * Usage:
 * <code>
 *  // mix it
 *  var myView = Em.View.extend(App.InfiniteScrollMixin, {
 *    didInsertElement: function() {
 *    	// call method infiniteScrollInit
 *    	this.infiniteScrollInit($('.some-scrollable'), {
 *    		callback: this.someCallbacOnEndReached
 *    	});
 *    }
 *  });
 * </code>
 *
 */
App.InfiniteScrollMixin = Ember.Mixin.create({

  /**
   * Stores callback execution progress.
   *
   * @type {Boolean}
   */
  _infiniteScrollCallbackInProgress: false,

  /**
   * Stores HTMLElement infinite scroll initiated on.
   *
   * @type {HTMLElement}
   */
  _infiniteScrollEl: null,

  /**
   * Default options for infinite scroll.
   *
   * @type {InfiniteScrollMixinOptions}
   */
  _infiniteScrollDefaults: {
    appendHtml: '<div id="infinite-scroll-append"><i class="icon-spinner icon-spin"></i></div>',
    callback: function callback() {
      return $.Deferred().resolve().promise();
    },
    onReject: function onReject() {},
    onResolve: function onResolve() {}
  },

  /**
   * Determines that there is no data to load on next callback call.
   *
   */
  _infiniteScrollMoreData: true,

  /**
   * Initialize infinite scroll on specified HTMLElement.
   *
   * @param  {HTMLElement} el DOM element to attach infinite scroll.
   * @param  {InfiniteScrollMixinOptions} opts
   */
  infiniteScrollInit: function infiniteScrollInit(el, opts) {
    var options = $.extend({}, this.get('_infiniteScrollDefaults'), opts || {});
    this.set('_infiniteScrollEl', el);
    this.get('_infiniteScrollEl').on('scroll', this._infiniteScrollHandler.bind(this));
    this.get('_infiniteScrollEl').on('infinite-scroll-end', this._infiniteScrollEndHandler(options).bind(this));
  },

  /**
   * Handler executed on scrolling.
   * @param  {jQuery.Event} e
   */
  _infiniteScrollHandler: function _infiniteScrollHandler(e) {
    var el = $(e.target);
    var height = el.get(0).clientHeight;
    var scrollHeight = el.prop('scrollHeight');
    var endPoint = scrollHeight - height;
    if (endPoint === el.scrollTop() && !this.get('_infiniteScrollCallbackInProgress')) {
      el.trigger('infinite-scroll-end');
    }
  },

  /**
   * Handler called when scroll ends.
   *
   * @param  {InfiniteScrollMixinOptions} options
   * @return {Function}
   */
  _infiniteScrollEndHandler: function _infiniteScrollEndHandler(options) {
    return function (e) {
      var self = this;
      if (this.get('_infiniteScrollCallbackInProgress') || !this.get('_infiniteScrollMoreData')) return;
      this._infiniteScrollAppendHtml(options.appendHtml);
      // always scroll to bottom
      this.get('_infiniteScrollEl').scrollTop(this.get('_infiniteScrollEl').get(0).scrollHeight);
      this.set('_infiniteScrollCallbackInProgress', true);
      options.callback().then(function () {
        options.onResolve();
      }, function () {
        options.onReject();
      }).always(function () {
        self.set('_infiniteScrollCallbackInProgress', false);
        self._infiniteScrollRemoveHtml(options.appendHtml);
      });
    }.bind(this);
  },

  /**
   * Helper function to append String as html node to.
   * @param  {String} htmlString string to append
   */
  _infiniteScrollAppendHtml: function _infiniteScrollAppendHtml(htmlString) {
    this.get('_infiniteScrollEl').append(htmlString);
  },

  /**
   * Remove HTMLElement by specified string that can be converted to html. HTMLElement root node
   * should have unique <code>id</code> or <code>class</code> attribute to avoid removing additional
   * elements.
   *
   * @param  {String} htmlString string to remove
   */
  _infiniteScrollRemoveHtml: function _infiniteScrollRemoveHtml(htmlString) {
    this.get('_infiniteScrollEl').find(this._infiniteScrollGetSelector(htmlString)).remove();
  },

  /**
   * Get root node selector.
   * <code>id</code> attribute has higher priority and will return if found.
   * <code>class</code> if no <code>id</code> attribute found <code>class</code> attribute
   * will be used.
   *
   * @param  {String} htmlString string processed as HTML
   * @return {[type]}            [description]
   */
  _infiniteScrollGetSelector: function _infiniteScrollGetSelector(htmlString) {
    var html = $(htmlString);
    var elId = html.attr('id');
    var elClass = (html.attr('class') || '').split(' ').join('.');
    html = null;
    return !!elId ? '#' + elId : '.' + elClass;
  },

  /**
   * Remove infinite scroll.
   * Unbind all listeners.
   */
  infiniteScrollDestroy: function infiniteScrollDestroy() {
    this.get('_infiniteScrollEl').off('scroll', this._infiniteScrollHandler);
    this.get('_infiniteScrollEl').off('infinite-scroll-end', this._infiniteScrollHandler);
    this.set('_infiniteScrollEl', null);
  },

  /**
   * Set if there is more data to load on next scroll end event.
   * @param {boolean} isAvailable <code>true</code> when there are more data to fetch
   */
  infiniteScrollSetDataAvailable: function infiniteScrollSetDataAvailable(isAvailable) {
    this.set('_infiniteScrollMoreData', isAvailable);
  }
});

});

require.register("mixins/common/kdc_credentials_controller_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var credentialsUtils = require('utils/credentials');

App.KDCCredentialsControllerMixin = Em.Mixin.create({

  /**
   * Alias name used to store KDC credentials
   *
   * @type {string}
   */
  credentialAlias: credentialsUtils.ALIAS.KDC_CREDENTIALS,

  /**
   * Returns <code>true</code> if persisted secure storage available.
   *
   * @type {boolean}
   */
  isStorePersisted: Em.computed.alias('App.isCredentialStorePersistent'),

  /**
   * List of required UI-only properties needed for storing KDC credentials
   *
   * @type {object[]}
   */
  credentialsStoreConfigs: [{
    name: 'persist_credentials',
    displayType: 'checkbox',
    value: 'false',
    recommendedValue: 'false',
    supportsFinal: false,
    recommendedIsFinal: false,
    displayName: Em.I18n.t('admin.kerberos.credentials.store.label'),
    category: 'Kadmin',
    isRequired: false,
    isRequiredByAgent: false,
    hintMessage: false,
    rightSideLabel: true,
    isEditable: true,
    index: 3
  }],

  /**
   * @param {object} resource resource info to set e.g.
   * <code>
   * {
   *   principal: "USERNAME",
   *   key: "SecretKey",
   *   type: "persisted"
   * }
   * </code>
   *
   * Where:
   * <ul>
   *   <li>principal: the principal (or username) part of the credential to store</li>
   *   <li>key: the secret key part of the credential to store</li>
   *   <li>type: declares the storage facility type: "persisted" or "temporary"</li>
   * </ul>
   * @returns {$.Deferred} promise object
   */
  createKDCCredentials: function createKDCCredentials(configs) {
    var resource = credentialsUtils.createCredentialResource(Em.getWithDefault(configs.findProperty('name', 'admin_principal') || {}, 'value', ''), Em.getWithDefault(configs.findProperty('name', 'admin_password') || {}, 'value', ''), this._getStorageTypeValue(configs));
    return credentialsUtils.createOrUpdateCredentials(App.get('clusterName'), this.get('credentialAlias'), resource);
  },

  /**
   * Remove KDC credentials
   *
   * @returns {$.Deferred} promise object
   */
  removeKDCCredentials: function removeKDCCredentials() {
    return credentialsUtils.removeCredentials(App.get('clusterName'), this.get('credentialAlias'));
  },

  /**
   * @see createKDCCredentials
   * @param {object} resource
   * @returns {$.Deferred} promise object
   */
  updateKDCCredentials: function updateKDCCredentials(resource) {
    return credentialsUtils.updateCredentials(App.get('clusterName'), this.get('credentialAlias'), resource);
  },

  /**
   * initialize additional properties regarding KDC credential storage
   * @method initializeKDCStoreProperties
   * @param {App.ServiceConfigProperty[]} configs list of configs
   */
  initializeKDCStoreProperties: function initializeKDCStoreProperties(configs) {
    this.generateKDCStoreProperties().forEach(function (configObject) {
      var configProperty = configs.findProperty('name', configObject.name);
      if (!Em.isNone(configProperty)) {
        Em.setProperties(configProperty, configObject);
      } else {
        configs.pushObject(configObject);
      }
    });
  },

  /**
   * Generate additional properties regarding KDC credential storage
   * @method updateKDCStoreProperties
   * @param {App.ServiceConfigProperty[]} configs list of configs
   */
  updateKDCStoreProperties: function updateKDCStoreProperties(configs) {
    this.generateKDCStoreProperties().forEach(function (configObject) {
      var configProperty = configs.findProperty('name', configObject.name);
      if (!Em.isNone(configProperty)) {
        Em.setProperties(configProperty, configObject);
      }
    });
  },

  /**
   * generate additional properties regarding KDC credential storage
   * @method generateKDCStoreProperties
   * @returns {Array} properties
   */
  generateKDCStoreProperties: function generateKDCStoreProperties() {
    var properties = [];

    this.get('credentialsStoreConfigs').forEach(function (item) {
      var configObject = App.config.createDefaultConfig(item.name, 'krb5-conf.xml', false);
      $.extend(configObject, item);
      if (item.name === 'persist_credentials') {
        if (this.get('isStorePersisted')) {
          configObject.hintMessage = Em.I18n.t('admin.kerberos.credentials.store.hint.supported');
        } else {
          configObject.hintMessage = Em.I18n.t('admin.kerberos.credentials.store.hint.not.supported');
          configObject.isEditable = false;
        }
      }
      properties.push(configObject);
    }, this);
    return properties;
  },

  /**
   * Return storage type e.g. <b>temporary</b>, <b>persisted</b>
   *
   * @param {App.ServiceConfigProperty[]} configs configs array from step configs
   * @returns {string} storage type value
   */
  _getStorageTypeValue: function _getStorageTypeValue(configs) {
    if (this.get('isStorePersisted')) {
      return Em.getWithDefault(configs.findProperty('name', 'persist_credentials') || {}, 'value', '') === "true" ? credentialsUtils.STORE_TYPES.PERSISTENT : credentialsUtils.STORE_TYPES.TEMPORARY;
    }
    return credentialsUtils.STORE_TYPES.TEMPORARY;
  }

});

});

require.register("mixins/common/loading_overlay_support", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * This Mixin is used for views that should "wait" while some external action is executed (e.g., ajax-request).
 * You should set <code>fieldToObserve</code> to fieldName which value determines begin and end of such action
 * When <code>fieldToObserve</code>-value becomes truly, overlay is shown
 * Overlay size is taken from current view size
 *
 * Usage:
 *
 * <pre>
 *   Em.View.extend(App.LoadingOverlaySupport, {
 *    fieldToObserve: 'someFieldName'
 *   });
 * </pre>
 *
 * Don't forget to add <code><div class="loading-overlay"></div></code> to your view's template
 *
 * @type {Em.Mixin}
 */
App.LoadingOverlaySupport = Em.Mixin.create({

  /**
   * @type {string}
   */
  fieldToObserve: '',

  /**
   * Determines if <code>handleFieldChanges</code> should be call on <code>didInsertElement</code>
   *
   * @type {boolean}
   * @default {true}
   */
  handleFieldChangesOnDidInsert: true,

  init: function init() {
    var fieldToObserve = this.get('fieldToObserve');
    Em.assert('`fieldToObserve` should be defined', fieldToObserve);
    this.addObserver(fieldToObserve, this, 'handleFieldChanges');
    return this._super.apply(this, arguments);
  },


  /**
   * Don't allow change `fieldToObserve` after view is initialized. This may cause a lot of errors
   */
  doNotChangeFieldToObserve: function () {
    Em.assert('Do not change `fieldToObserve` after view is initialized', false);
  }.observes('fieldToObserve'),

  /**
   * Value in the <code>fieldToObserve</code> may be <code>truly</code> on view init, so
   * <code>handleFieldChanges</code> should be called to check this
   *
   * Example use case:
   *  Switching tabs on the configs page for some service. Every tab should be disabled when switching on it
   *  if needed property is set to <code>true</code>
   *
   * @returns {*}
   */
  didInsertElement: function didInsertElement() {
    this._super.apply(this, arguments);
    if (this.get('handleFieldChangesOnDidInsert')) {
      this.handleFieldChanges();
    }
  },


  /**
   * Remove observer `handleFieldChanges`
   */
  willDestroyElement: function willDestroyElement() {
    var fieldToObserve = this.get('fieldToObserve');
    this.removeObserver(fieldToObserve, this, 'handleFieldChanges');
    this._cleanUp();
  },


  /**
   * Show overlay when `fieldToObserve`-value is set to some truly
   * Remove overlay otherwise
   */
  handleFieldChanges: function handleFieldChanges() {
    var _this = this;

    var fieldToObserve = this.get('fieldToObserve');
    var fieldValue = this.get(fieldToObserve);
    var overlay = this.$('.loading-overlay');
    var polling = this.get('_polling');
    if (fieldValue) {
      if (!polling) {
        polling = setInterval(function () {
          if (fieldValue && overlay) {
            overlay.addClass('overlay-visible').css({
              width: _this.$().width(),
              height: _this.$().height()
            });
          } else {
            _this._cleanUp();
          }
        }, 50);
        this.set('_polling', polling);
      }
    } else {
      this._cleanUp();
    }
  },


  /**
   * @private
   */
  _cleanUp: function _cleanUp() {
    clearInterval(this.get('_polling'));
    this.set('_polling', null);
    var overlay = this.$('.loading-overlay');
    if (overlay) {
      overlay.removeClass('overlay-visible');
    }
  }
});

});

require.register("mixins/common/localStorage", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Fast save/load value to Local Storage
 * Object that implements it should have property <code>name</code> defined
 * @type {Ember.Mixin}
 */
App.LocalStorage = Em.Mixin.create({

  /**
   * <code>App.db.data</code> key
   * @returns {String}
   */
  dbNamespace: function () {
    var name = this.get('name');
    if (Em.isNone(name)) {
      name = this.get('controller.name');
    }
    return name.capitalize().replace('Controller', '');
  }.property('name'),

  /**
   * get property from local storage
   * @param {String} key
   * @return {*}
   */
  getDBProperty: function getDBProperty(key) {
    return App.db.get(this.get('dbNamespace'), key);
  },

  /**
   * get properties from local storage
   * @param {String[]} listOfProperties
   * @return {*}
   */
  getDBProperties: function getDBProperties(listOfProperties) {
    return App.db.getProperties(this.get('dbNamespace'), listOfProperties);
  },

  /**
   * set property to local storage
   * @param {String} key
   * @param {*} value
   */
  setDBProperty: function setDBProperty(key, value) {
    App.db.set(this.get('dbNamespace'), key, value);
  },

  /**
   * set properties to local storage
   * @param {object} hash
   */
  setDBProperties: function setDBProperties(hash) {
    App.db.setProperties(this.get('dbNamespace'), hash);
  },

  /**
   * Delete the dbNamespace from the localStorage
   * Usually this function is called on quiting/completing the wizard
   */
  resetDbNamespace: function resetDbNamespace() {
    App.db.data[this.get('dbNamespace')] = {};
    localStorage.setObject('ambari', App.db.data);
  }

});

});

require.register("mixins/common/persist", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var LZString = require('utils/lz-string');

/**
 * Small mixin for processing user preferences
 * Provide methods to save/load some values in <code>persist</code> storage
 * Save available only for admin users!
 * When using this mixin you should redeclare methods:
 * <ul>
 *   <li>getUserPrefSuccessCallback</li>
 *   <li>getUserPrefErrorCallback</li>
 *   <li>postUserPrefSuccessCallback</li>
 *   <li>postUserPrefErrorCallback</li>
 * </ul>
 * @type {Em.Mixin}
 */
App.Persist = Em.Mixin.create({

  /**
   * Additional to request data
   * @type {object}
   */
  additionalData: {},

  /**
   * Get persist value from server with persistKey
   * @param {String} key
   */
  getUserPref: function getUserPref(key) {
    return App.ajax.send({
      name: 'persist.get',
      sender: this,
      data: {
        key: key,
        data: this.get('additionalData')
      },
      success: 'getUserPrefSuccessCallback',
      error: 'getUserPrefErrorCallback'
    });
  },

  /**
   *
   * @param {string} key
   * @returns {$.Deferred}
   */
  getDecompressedData: function getDecompressedData(key) {
    var dfd = $.Deferred();
    App.ajax.send({
      name: 'persist.get.text',
      sender: this,
      data: {
        key: key
      }
    }).always(function (data, textStatus, error) {
      if (data && typeof data === 'string') {
        dfd.resolve(JSON.parse(LZString.decompressFromBase64(data)));
      } else {
        dfd.reject({ error: error });
      }
    });
    return dfd.promise();
  },

  /**
   * Should be redefined in objects that use this mixin
   * @param {*} response
   * @param {Object} request
   * @param {Object} data
   * @returns {*}
   */
  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response, request, data) {},

  /**
   * Should be redefined in objects that use this mixin
   * @param {Object} request
   * @param {Object} ajaxOptions
   * @param {String} error
   */
  getUserPrefErrorCallback: function getUserPrefErrorCallback(request, ajaxOptions, error) {},

  /**
   * Post persist key/value to server, value is object
   * Only for admin users!
   * @param {String} key
   * @param {Object} value
   */
  postUserPref: function postUserPref(key, value) {
    if (!App.isAuthorized('CLUSTER.MANAGE_USER_PERSISTED_DATA')) {
      return $.Deferred().reject().promise();
    }
    var keyValuePair = {};
    keyValuePair[key] = JSON.stringify(value);
    return this.post(keyValuePair);
  },

  /**
   *
   * @param {string} key
   * @param {Object} value
   * @returns {$.ajax}
   */
  postCompressedData: function postCompressedData(key, value) {
    var keyValuePair = {};
    keyValuePair[key] = !Em.isEmpty(value) ? LZString.compressToBase64(JSON.stringify(value)) : '';
    return this.post(keyValuePair);
  },

  post: function post(keyValuePair) {
    return App.ajax.send({
      'name': 'persist.post',
      'sender': this,
      'beforeSend': 'postUserPrefBeforeSend',
      'data': {
        'keyValuePair': keyValuePair
      },
      'success': 'postUserPrefSuccessCallback',
      'error': 'postUserPrefErrorCallback'
    });
  },

  /**
   * Should be redefined in objects that use this mixin
   * @param {*} response
   * @param {Object} request
   * @param {Object} data
   * @returns {*}
   */
  postUserPrefSuccessCallback: function postUserPrefSuccessCallback(response, request, data) {},

  /**
   * Should be redefined in objects that use this mixin
   * @param {Object} request
   * @param {Object} ajaxOptions
   * @param {String} error
   */
  postUserPrefErrorCallback: function postUserPrefErrorCallback(request, ajaxOptions, error) {},

  /**
   * Little log before post request
   * @param {Object} request
   * @param {Object} ajaxOptions
   * @param {Object} data
   */
  postUserPrefBeforeSend: function postUserPrefBeforeSend(request, ajaxOptions, data) {}

});

});

require.register("mixins/common/reload_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReloadPopupMixin = Em.Mixin.create({

  reloadPopup: null,

  reloadSuccessCallback: function reloadSuccessCallback() {
    this.closeReloadPopup();
  },

  reloadErrorCallback: function reloadErrorCallback(jqXHR, ajaxOptions, error, opt, params) {
    if (jqXHR.status) {
      this.closeReloadPopup();
      if (params.shouldUseDefaultHandler) {
        App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
      }
    } else {
      var timeout = Em.isNone(params.timeout) ? App.get('timeout') : params.timeout;
      this.showReloadPopup(params.reloadPopupText);
      if (params.callback) {
        var self = this;
        window.setTimeout(function () {
          params.callback.apply(self, params.args || []);
        }, timeout);
      }
    }
  },

  popupText: function popupText(text) {
    return text || Em.I18n.t('app.reloadPopup.text');
  },

  showReloadPopup: function showReloadPopup(text) {
    var self = this,
        bodyText = this.popupText(text);
    if (!this.get('reloadPopup')) {
      this.set('reloadPopup', App.ModalPopup.show({
        primary: null,
        secondary: null,
        showFooter: false,
        header: this.t('app.reloadPopup.header'),
        bodyClass: Ember.View.extend({
          template: Ember.Handlebars.compile("<div id='reload_popup' class='alert alert-info'>" + "{{view App.SpinnerView}}" + "<div><span>" + bodyText + "</span><a href='javascript:void(null)' onclick='location.reload();'>" + this.t('app.reloadPopup.link') + "</a></div>")
        }),
        onClose: function onClose() {
          self.set('reloadPopup', null);
          this._super();
        }
      }));
    }
  },

  closeReloadPopup: function closeReloadPopup() {
    var reloadPopup = this.get('reloadPopup');
    if (reloadPopup) {
      reloadPopup.onClose();
    }
  }

});

});

require.register("mixins/common/serverValidator", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');

/**
 * @typedef {object} ConfigsValidationRequestData
 * @property {string} stackVersionUrl stack version url
 * @property {string[]} hosts host names
 * @property {string[]} services service names
 * @property {string} validate validation type e.g. 'configurations'
 * @property {object} recommendations blueprint object
 */

/**
 * @typedef {object} ConfigsValidationOptions
 * @property {string[]} hosts host names
 * @property {string[]} services service names
 * @property {object} blueprint service configurations blueprint
 */
App.ServerValidatorMixin = Em.Mixin.create({

  /**
   * defines if we use validation and recommendation on wizards
   * depend on this flag some properties will be taken from different places
   * @type {boolean}
   */
  isWizard: function () {
    return this.get('wizardController') && ['addServiceController', 'installerController'].contains(this.get('wizardController.name'));
  }.property('wizardController.name'),

  /**
   * recommendation configs loaded from server
   * (used only during install)
   * @type {Object}
   */
  recommendationsConfigs: null,

  /**
   * Collection of all config validation errors
   *
   * @type {Object[]}
   */
  configErrorList: Em.Object.create({
    issues: [],
    criticalIssues: []
  }),

  /**
   * Ajax request object to validation API
   * @type {$.ajax}
   */
  validationRequest: null,

  /**
   * Timer id returned by setTimeout for calling function to send validation request
   */
  requestTime: 0,

  /**
   * Map with allowed error types
   *
   * @type {Object}
   */
  errorTypes: {
    CRITICAL_ERROR: 'NOT_APPLICABLE',
    ERROR: 'ERROR',
    WARN: 'WARN',
    GENERAL: 'GENERAL'
  },

  /**
   * by default loads data from model otherwise must be overridden as computed property
   * refer to \assets\data\stacks\HDP-2.1\recommendations_configs.json to learn structure
   * (shouldn't contain configurations filed)
   * @type {Object}
   */
  hostNames: function () {
    return this.get('isWizard') ? Object.keys(this.get('content.hosts')) : App.get('allHostNames');
  }.property('isWizard', 'content.hosts', 'App.allHostNames'),

  /**
   * by default loads data from model otherwise must be overridden as computed property
   * @type {Array} - of strings (serviceNames)
   */
  serviceNames: function () {
    // When editing a service we validate only that service's configs.
    // However, we should pass the IDs of services installed, or else,
    // default value calculations will alter.
    return this.get('isWizard') ? this.get('allSelectedServiceNames') : App.Service.find().mapProperty('serviceName');
  }.property('isWizard', 'allSelectedServiceNames'),

  /**
   * by default loads data from model otherwise must be overridden as computed property
   * can be used for service|host configs pages
   * @type {Array} of strings (hostNames)
   */
  hostGroups: function () {
    return this.get('content.recommendationsHostGroups') || blueprintUtils.generateHostGroups(App.get('allHostNames'));
  }.property('content.recommendationsHostGroups', 'App.allHostNames', 'App.componentToBeAdded', 'App.componentToBeDeleted'),

  /**
   * controller that is child of this mixin has to contain stepConfigs
   * @type {Array}
   */
  stepConfigs: null,

  serverSideValidation: function serverSideValidation() {
    var deferred = $.Deferred(),
        self = this,
        primary = function primary() {
      deferred.resolve();
    },
        secondary = function secondary() {
      deferred.reject('invalid_configs');
    };
    this.set('configErrorList', Em.Object.create({
      issues: [],
      criticalIssues: []
    }));

    this.runServerSideValidation().done(function () {
      if (self.get('configErrorList.issues.length') || self.get('configErrorList.criticalIssues.length')) {
        App.showConfigValidationPopup(self.get('configErrorList'), primary, secondary, self);
      } else {
        deferred.resolve();
      }
    }).fail(function () {
      App.showConfigValidationFailedPopup(primary, secondary);
    });

    return deferred.promise();
  },

  /**
   * @method serverSideValidation
   * send request to validate configs
   * @returns {*}
   */
  runServerSideValidation: function runServerSideValidation() {
    var self = this;
    var recommendations = this.get('hostGroups');
    var stepConfigs = this.get('stepConfigs');
    var dfd = $.Deferred();

    this.getBlueprintConfigurations(this.get('stepConfigs')).done(function (blueprintConfigurations) {
      recommendations.blueprint.configurations = blueprintConfigurations;
      var request = self.validateSelectedConfigs({
        hosts: self.get('hostNames'),
        services: self.get('serviceNames'),
        blueprint: recommendations
      });
      self.set('validationRequest', request);
      request.done(dfd.resolve).fail(dfd.reject);
    });
    return dfd.promise();
  },

  /**
   * Perform service config validation
   * @param validateSelectedConfigs
   * @param {ConfigsValidationOptions} options
   * @returns {$.Deferred}
   */
  validateSelectedConfigs: function validateSelectedConfigs(options) {
    var opts = $.extend({
      services: [],
      hosts: [],
      blueprint: null
    }, options || {});

    return this.getServiceConfigsValidationRequest(this.getServiceConfigsValidationParams(opts));
  },

  /**
   * @method getServiceConfigsValidationRequest
   * @param {ConfigsValidationRequestData} validationData
   * @returns {$.Deferred}
   */
  getServiceConfigsValidationRequest: function getServiceConfigsValidationRequest(validationData) {
    return App.ajax.send({
      name: 'config.validations',
      sender: this,
      data: validationData,
      success: 'validationSuccess'
    });
  },

  /**
   * @method getServiceConfigsValidationParams
   * @param {ConfigsValidationOptions} options
   * @returns {ConfigsValidationRequestData}
   */
  getServiceConfigsValidationParams: function getServiceConfigsValidationParams(options) {
    return {
      stackVersionUrl: App.get('stackVersionURL'),
      hosts: options.hosts,
      services: options.services,
      validate: 'configurations',
      recommendations: options.blueprint
    };
  },

  /**
   * Return JSON for blueprint configurations
   * @param {App.ServiceConfigs[]} serviceConfigs
   * @returns {*}
   */
  getBlueprintConfigurations: function getBlueprintConfigurations(serviceConfigs) {
    var dfd = $.Deferred();
    // check if we have configs from 'cluster-env', if not, then load them, as they are mandatory for validation request
    if (!serviceConfigs.findProperty('serviceName', 'MISC')) {
      App.config.getConfigsByTypes([{ site: 'cluster-env', serviceName: 'MISC' }]).done(function (configs) {
        dfd.resolve(blueprintUtils.buildConfigsJSON(serviceConfigs.concat(configs)));
      });
    } else {
      dfd.resolve(blueprintUtils.buildConfigsJSON(serviceConfigs));
    }
    return dfd.promise();
  },

  /**
   * Creates config validation error object
   *
   * @param type - error type, see <code>errorTypes<code>
   * @param property - config property object
   * @param messages - array of messages
   * @returns {{type: String, isError: boolean, isWarn: boolean, isGeneral: boolean, messages: Array}}
   */
  createErrorMessage: function createErrorMessage(type, property, messages) {
    var errorTypes = this.get('errorTypes');
    var error = {
      type: type,
      isCriticalError: type === errorTypes.CRITICAL_ERROR,
      isError: type === errorTypes.ERROR,
      isWarn: type === errorTypes.WARN,
      isGeneral: type === errorTypes.GENERAL,
      cssClass: this.get('isInstallWizard') ? '' : type === errorTypes.ERROR ? 'error' : 'warning',
      messages: Em.makeArray(messages)
    };

    Em.assert('Unknown config error type ' + type, error.isError || error.isWarn || error.isGeneral || error.isCriticalError);
    if (property) {
      var value = Em.get(property, 'value');
      error.id = Em.get(property, 'id');
      error.serviceName = Em.get(property, 'serviceDisplayName') || App.StackService.find(Em.get(property, 'serviceName')).get('displayName');
      error.propertyName = Em.get(property, 'name');
      error.filename = Em.get(property, 'filename');
      error.value = value && Em.get(property, 'displayType') === 'password' ? new Array(value.length + 1).join('*') : value;
      error.description = Em.get(property, 'description');
    }
    return error;
  },

  /**
   * Parse data from server to
   *  <code>configErrorsMap<code> and
   *  <code>generalErrors<code>
   *
   * @param data
   * @returns {{configErrorsMap: {}, generalErrors: Array}}
   */
  parseValidation: function parseValidation(data) {
    var configErrorsMap = {},
        generalErrors = [];

    data.resources.forEach(function (r) {
      r.items.forEach(function (item) {
        if (item.type == "configuration") {
          var configId = item['config-name'] && item['config-type'] && App.config.configId(item['config-name'], item['config-type']);
          if (configId) {
            if (configErrorsMap[configId]) {
              configErrorsMap[configId].messages.push(item.message);
            } else {
              configErrorsMap[configId] = {
                type: item.level,
                messages: [item.message],
                name: item['config-name'],
                filename: item['config-type']
              };
            }
          } else {
            generalErrors.push({
              type: this.get('errorTypes').GENERAL,
              messages: [item.message]
            });
          }
        }
      }, this);
    }, this);

    return {
      configErrorsMap: configErrorsMap,
      generalErrors: generalErrors
    };
  },

  /**
   * Generates list of all config errors that should be displayed in popup
   *
   * @param configErrorsMap
   * @param generalErrors
   * @returns {Array}
   */
  collectAllIssues: function collectAllIssues(configErrorsMap, generalErrors) {
    var errorTypes = this.get('errorTypes');
    var configErrorList = {};
    configErrorList[errorTypes.WARN] = [];
    configErrorList[errorTypes.ERROR] = [];
    configErrorList[errorTypes.CRITICAL_ERROR] = [];
    configErrorList[errorTypes.GENERAL] = [];

    this.get('stepConfigs').forEach(function (service) {
      service.get('configs').forEach(function (property) {
        if (property.get('isVisible') && !property.get('hiddenBySection')) {
          var serverIssue = configErrorsMap[property.get('id')];
          if (serverIssue) {
            configErrorList[serverIssue.type].push(this.createErrorMessage(serverIssue.type, property, serverIssue.messages));
          } else if (property.get('warnMessage')) {
            configErrorList[errorTypes.WARN].push(this.createErrorMessage(errorTypes.WARN, property, [property.get('warnMessage')]));
          }
        }
      }, this);
    }, this);

    generalErrors.forEach(function (serverIssue) {
      configErrorList[errorTypes.GENERAL].push(this.createErrorMessage(errorTypes.GENERAL, null, serverIssue.messages));
    }, this);

    Em.keys(configErrorsMap).forEach(function (id) {
      var serverIssue = configErrorsMap[id];
      if (!configErrorList[serverIssue.type].someProperty('id', id)) {
        var filename = Em.get(serverIssue, 'filename'),
            service = App.config.get('serviceByConfigTypeMap')[filename],
            property = {
          id: id,
          name: Em.get(serverIssue, 'name'),
          filename: App.config.getOriginalFileName(filename),
          serviceDisplayName: service && Em.get(service, 'displayName')
        };
        configErrorList[serverIssue.type].push(this.createErrorMessage(serverIssue.type, property, serverIssue.messages));
      }
    }, this);

    return Em.Object.create({
      criticalIssues: configErrorList[errorTypes.CRITICAL_ERROR],
      issues: configErrorList[errorTypes.ERROR].concat(configErrorList[errorTypes.WARN], configErrorList[errorTypes.GENERAL])
    });
  },

  /**
   * @method validationSuccess
   * success callback after getting response from server
   * go through the step configs and set warn and error messages
   * @param data
   */
  validationSuccess: function validationSuccess(data) {
    var parsed = this.parseValidation(data);
    this.set('configErrorList', this.collectAllIssues(parsed.configErrorsMap, parsed.generalErrors));
  },

  valueObserver: function () {
    var self = this;
    if (this.get('isInstallWizard') && this.get('currentTabName') === 'all-configurations') {
      if (this.get('requestTimer')) clearTimeout(this.get('requestTimer'));
      self.set('requestTimer', setTimeout(function () {
        if (self.get('validationRequest')) {
          self.get('validationRequest').abort();
        }
        if (self.get('recommendationsInProgress')) {
          self.valueObserver();
        } else {
          self.runServerSideValidation().done(function () {
            self.set('validationRequest', null);
            self.set('requestTimer', 0);
          });
        }
      }, 500));
    }
  }.observes('selectedService.configs.@each.value')

});

});

require.register("mixins/common/table_server_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.TableServerMixin = Em.Mixin.create({
  queryParams: [],
  resetStartIndex: false,

  /**
   * filterProps support follow types of filter:
   * MATCH - match of RegExp
   * EQUAL - equality "="
   * LESS - "<"
   * MORE - ">"
   * MULTIPLE - multiple values to compare
   * CUSTOM - substitute values with keys "{#}" in alias
   */
  filterProps: [],

  /**
   * include "from" and "page_size"
   */
  paginationProps: [{
    name: 'displayLength',
    key: 'page_size',
    value: '25',
    type: 'EQUAL'
  }, {
    name: 'startIndex',
    key: 'from',
    value: 0,
    type: 'EQUAL'
  }],

  sortProps: [],

  /**
   * update values of pagination properties from local db and return them
   * @return {array}
   */
  getPaginationProps: function getPaginationProps() {
    var displayLength = App.db.getDisplayLength(this.get('name'));
    if (displayLength) {
      this.get('paginationProps').findProperty('name', 'displayLength').value = displayLength;
    }

    var startIndex = App.db.getStartIndex(this.get('name'));
    if (!Em.isNone(startIndex)) {
      startIndex = startIndex > 0 ? startIndex - 1 : startIndex;
      this.get('paginationProps').findProperty('name', 'startIndex').value = startIndex;
    }
    return this.get('paginationProps').slice(0);
  },

  /**
   * get sort properties from local db
   * @return {Array}
   */
  getSortProps: function getSortProps() {
    var savedSortConditions = App.db.getSortingStatuses(this.get('name')) || [],
        sortProperties = this.get('sortProps'),
        sortParams = [];
    savedSortConditions.forEach(function (sort) {
      var property = sortProperties.findProperty('name', sort.name);
      if (property && (sort.status === 'sorting_asc' || sort.status === 'sorting_desc')) {
        property.value = sort.status.replace('sorting_', '');
        property.type = 'SORT';
        sortParams.push(property);
      }
    });
    return sortParams;
  },
  /**
   * get filter properties from local db
   * @return {Array}
   */
  getFilterProps: function getFilterProps() {
    var savedFilterConditions = App.db.getFilterConditions(this.get('name')) || [],
        filterProperties = this.get('filterProps'),
        filterParams = [],
        colPropAssoc = this.get('colPropAssoc');

    savedFilterConditions.forEach(function (filter) {
      var property = filterProperties.findProperty('name', colPropAssoc[filter.iColumn]);

      if (property && filter.value && filter.value.length > 0 && !filter.skipFilter) {
        property.isFilter = true;

        if (filter.type === 'range') {
          //range value should contain array of two element with start and end values accordingly
          filter.value.forEach(function (val, index) {
            if (val) {
              property.type = index === 0 ? "MORE" : "LESS";
              property.value = val;
              filterParams.push(property);
            }
          });
        } else if (filter.type === 'string') {
          property.value = this.getRegExp(filter.value);
          filterParams.push(property);
        } else {
          property.value = filter.value;
          filterParams.push(property);
        }
      }
    }, this);
    return filterParams;
  },

  getQueryParameters: function getQueryParameters() {
    var queryParams = [];

    queryParams.pushObjects(this.getPaginationProps());
    queryParams.pushObjects(this.getSortProps());
    queryParams.pushObjects(this.getFilterProps());
    this.set('queryParams', queryParams);
    return queryParams;
  },

  getRegExp: function getRegExp(value) {
    value = validator.isValidMatchesRegexp(value) ? value.replace(/(\.+\*?|(\.\*)+)$/, '') + '.*' : '^$';
    value = /^\.\*/.test(value) || value == '^$' ? value : '.*' + value;
    return value;
  }

});

});

require.register("mixins/common/table_server_view_mixin", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin should be used for Em.View of table that uses server to filter, sort, paginate content
 */
App.TableServerViewMixin = Em.Mixin.create({
  filteringComplete: true,
  timeOut: null,
  /**
   * filter delay time, used to combine filters change into one content update
   */
  filterWaitingTime: 500,

  /**
   * count of filtered items
   */
  filteredCount: Em.computed.alias('controller.filteredCount'),
  /**
   * total count of items
   */
  totalCount: Em.computed.alias('controller.totalCount'),

  /**
   * data requested from server
   */
  content: Em.computed.alias('controller.content'),

  /**
   * content already filtered on server-side
   */
  filteredContent: Em.computed.alias('content'),
  /**
   * sort and slice recieved content by pagination parameters
   */
  pageContent: function () {
    var content = this.get('filteredContent').toArray();
    if (content.length > this.get('endIndex') - this.get('startIndex') + 1) {
      content = content.slice(0, this.get('endIndex') - this.get('startIndex') + 1);
    }
    return content.sort(function (a, b) {
      return a.get('index') - b.get('index');
    });
  }.property('filteredContent.length'),

  /**
   * compute applied filter and run content update from server
   * @param iColumn
   * @param value
   * @param type
   */
  updateFilter: function updateFilter(iColumn, value, type) {
    // Do not even trigger update flow if it's a blank update
    if (this.isBlankFilterUpdate(iColumn, value, type)) {
      return;
    }

    var self = this;
    this.set('controller.resetStartIndex', false);
    if (!this.get('filteringComplete')) {
      clearTimeout(this.get('timeOut'));
      this.set('timeOut', setTimeout(function () {
        self.updateFilter(iColumn, value, type);
      }, this.get('filterWaitingTime')));
    } else {
      clearTimeout(this.get('timeOut'));
      this.set('controller.resetStartIndex', true);
      //save filter only when it's applied
      this.saveFilterConditions(iColumn, value, type, false);
      this.refresh();
    }
    return true;
  },

  updateComboFilter: function updateComboFilter(filterConditions) {
    clearTimeout(this.get('timeOut'));
    this.set('controller.resetStartIndex', true);
    this.set('filterConditions', filterConditions);
    this.saveAllFilterConditions();
    this.refresh();
  },

  /**
   * 1) Has previous saved filter
   * 2) Value to update is empty
   * 3) Value to update is same as before
   * Returns true if (!1&&2 || 1&&3)
   * @param iColumn
   * @param value
   * @param type
   * @returns {boolean|*}
   */
  isBlankFilterUpdate: function isBlankFilterUpdate(iColumn, value, type) {
    var result = false;
    var filterConfitions = this.get('filterConditions');
    var filterCondition = filterConfitions ? filterConfitions.findProperty('iColumn', iColumn) : null;
    if (!filterCondition && Em.isEmpty(value) || filterCondition && filterCondition.value == value || filterCondition && _typeof(filterCondition.value) == 'object' && JSON.stringify(filterCondition.value) == JSON.stringify(value)) {
      result = true;
    }
    return result;
  },

  /**
   * success callback for updater request
   */
  updaterSuccessCb: function updaterSuccessCb() {
    this.set('filteringComplete', true);
    this.propertyDidChange('pageContent');
    App.loadTimer.finish('Hosts Page');
  },

  /**
   * error callback for updater request
   */
  updaterErrorCb: function updaterErrorCb() {
    this.set('requestError', arguments);
  },

  /**
   * synchronize properties of view with controller to generate query parameters
   */
  updatePagination: function updatePagination(key) {
    if (!Em.isNone(this.get('displayLength'))) {
      App.db.setDisplayLength(this.get('controller.name'), this.get('displayLength'));
    }
    if (!Em.isNone(this.get('startIndex'))) {
      App.db.setStartIndex(this.get('controller.name'), this.get('startIndex'));
    }

    if (key !== 'SKIP_REFRESH') {
      this.refresh();
    }
  },

  /**
   * Placeholder for `saveStartIndex`
   */
  saveStartIndex: Em.K,

  /**
   * when new filter applied page index should be reset to first page
   */
  resetStartIndex: function () {
    if (this.get('controller.resetStartIndex') && this.get('filteredCount') > 0) {
      this.set('startIndex', 1);
      this.saveStartIndex();
      this.updatePagination('SKIP_REFRESH');
    }
  }.observes('controller.resetStartIndex')
});

});

require.register("mixins/common/track_request_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TrackRequestMixin = Em.Mixin.create({

  requestsInProgress: [],

  init: function init() {
    this.set('requestsInProgress', []);
    this._super([].slice.call(arguments));
  },

  /**
   * register request to view to track his progress
   * @param {$.ajax} request
   * @method trackRequest
   */
  trackRequest: function trackRequest(request) {
    var requestId = this.get('requestsInProgress').length;
    var self = this;
    this.get('requestsInProgress').pushObject({
      request: request,
      id: requestId,
      status: Em.tryInvoke(request, 'state'),
      completed: ['resolved', 'rejected'].contains(Em.tryInvoke(request, 'state'))
    });
    request.always(function () {
      var requestInProgress = self.get('requestsInProgress').findProperty('id', requestId) || {};
      Em.setProperties(requestInProgress, {
        completed: true,
        status: Em.tryInvoke(request, 'state')
      });
    });
  },

  /**
   * This method used to put promise to requests queue which is waiting for another request to be put in tracking queue
   * after tracking request promise will be completed. Basically it used for places where trackRequest called after
   * tracked promise gets resolved.
   *
   * @param {$.ajax} request
   */
  trackRequestChain: function trackRequestChain(request) {
    var dfd = $.Deferred();
    request.always(function () {
      dfd.resolve();
    });
    this.trackRequest(request);
    this.trackRequest(dfd);
  },

  /**
   * abort all tracked requests
   * @method abortRequest
   */
  abortRequests: function abortRequests() {
    this.get('requestsInProgress').forEach(function (r) {
      var request = r.request;
      if (request && request.readyState !== undefined && request.readyState !== 4) {
        request.abort();
      }
    });
    this.get('requestsInProgress').clear();
  }

});

});

require.register("mixins/common/widgets/export_metrics_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/common/export_metrics_menu_view');
var stringUtils = require('utils/string_utils');
var fileUtils = require('utils/file_utils');

App.ExportMetricsMixin = Em.Mixin.create({

  /**
   * Used as argument passed from template to indicate that resulting format is CSV instead of JSON
   */
  exportToCSVArgument: true,

  isExportMenuHidden: true,

  isExportButtonHidden: false,

  exportMetricsMenuView: App.ExportMetricsMenuView.extend(),

  targetView: function () {
    return this.get('exportTargetView') || this;
  }.property(),

  hideMenuForNoData: function () {
    if (this.get('isExportButtonHidden')) {
      this.set('isExportMenuHidden', true);
    }
  }.observes('isExportButtonHidden'),

  toggleFormatsList: function toggleFormatsList() {
    this.toggleProperty('isExportMenuHidden');
  },

  exportGraphData: function exportGraphData(event) {
    this.set('isExportMenuHidden', true);
    var ajaxIndex = this.get('targetView.ajaxIndex');
    App.ajax.send({
      name: ajaxIndex,
      data: $.extend(this.get('targetView').getDataForAjaxRequest(), {
        isCSV: !!event.context
      }),
      sender: this,
      success: 'exportGraphDataSuccessCallback',
      error: 'exportGraphDataErrorCallback'
    });
  },

  getCustomFileName: function getCustomFileName() {
    return this.get('targetView.title').replace(/\s+/g, '_').toLowerCase();
  },

  exportGraphDataSuccessCallback: function exportGraphDataSuccessCallback(response, request, params) {
    var seriesData = this.get('targetView').getData(response);
    if (!seriesData.length) {
      App.showAlertPopup(Em.I18n.t('graphs.noData.title'), Em.I18n.t('graphs.noData.tooltip.title'));
    } else {
      var fileType = params.isCSV ? 'csv' : 'json',
          fileName = (Em.isEmpty(this.get('targetView.title')) ? 'data' : this.getCustomFileName()) + '.' + fileType,
          data = params.isCSV ? this.prepareCSV(seriesData) : JSON.stringify(seriesData, this.jsonReplacer(), 4);

      fileUtils.downloadTextFile(data, fileType, fileName);
    }
  },

  exportGraphDataErrorCallback: function exportGraphDataErrorCallback(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
  },

  prepareCSV: function prepareCSV(data) {
    var displayUnit = this.get('targetView.displayUnit'),
        titles,
        ticksNumber,
        metricsNumber,
        metricsArray;
    this.checkGraphDataForValidity(data);
    titles = data.map(function (item) {
      return displayUnit ? item.name + ' (' + displayUnit + ')' : item.name;
    }, this);
    titles.unshift(Em.I18n.t('common.timestamp'));
    ticksNumber = data[0].data.length;
    metricsNumber = data.length;
    metricsArray = [titles];
    for (var i = 0; i < ticksNumber; i++) {
      metricsArray.push([data[0].data[i][1]]);
      for (var j = 0; j < metricsNumber; j++) {
        metricsArray[i + 1].push(data[j].data[i][0]);
      };
    };
    return stringUtils.arrayToCSV(metricsArray);
  },

  checkGraphDataForValidity: function checkGraphDataForValidity(data) {
    data.sort(function (a, b) {
      return b.data.length - a.data.length;
    });

    var maxLength = data[0].data.length;

    for (var i = 1; i < data.length; i++) {
      if (data[i].data.length !== maxLength) this.fillGraphDataArrayWithMockedData(data[i], maxLength);
    }
  },

  fillGraphDataArrayWithMockedData: function fillGraphDataArrayWithMockedData(dataArray, neededLength) {
    var startIndex = dataArray.data.length,
        timestampInterval = dataArray.data[2][1] - dataArray.data[1][1];

    for (var i = startIndex; i < neededLength; i++) {
      var previousTimestamp = dataArray.data[i - 1][1];

      dataArray.data.push([null, previousTimestamp + timestampInterval]);
    }
  },

  jsonReplacer: function jsonReplacer() {
    var displayUnit = this.get('targetView.displayUnit');
    return function (key, value) {
      if (['name', 'data'].contains(key) || !isNaN(key)) {
        return key == 'name' && displayUnit ? value + ' (' + displayUnit + ')' : value;
      }
    };
  }

});

});

require.register("mixins/common/widgets/time_range_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/common/time_range_list');
var timeRangePopup = require('views/common/custom_date_popup');

App.TimeRangeMixin = Em.Mixin.create({

  timeRangeClassName: 'pull-right',

  /**
   * time range options for service metrics, a dropdown will list all options
   * value set in hours
   */
  timeRangeOptions: [{ index: 0, name: Em.I18n.t('graphs.timeRange.hour'), value: '1' }, { index: 1, name: Em.I18n.t('graphs.timeRange.twoHours'), value: '2' }, { index: 2, name: Em.I18n.t('graphs.timeRange.fourHours'), value: '4' }, { index: 3, name: Em.I18n.t('graphs.timeRange.twelveHours'), value: '12' }, { index: 4, name: Em.I18n.t('graphs.timeRange.day'), value: '24' }, { index: 5, name: Em.I18n.t('graphs.timeRange.week'), value: '168' }, { index: 6, name: Em.I18n.t('graphs.timeRange.month'), value: '720' }, { index: 7, name: Em.I18n.t('graphs.timeRange.year'), value: '8760' }, { index: 8, name: Em.I18n.t('common.custom'), value: '0' }],

  currentTimeRangeIndex: 0,

  currentTimeRange: function () {
    return this.get('timeRangeOptions').objectAt(this.get('currentTimeRangeIndex'));
  }.property('currentTimeRangeIndex'),

  isCustomTimeRange: function () {
    return !(Em.isNone(this.get('customStartTime')) || Em.isNone(this.get('customEndTime')));
  }.property('customStartTime', 'customEndTime'),

  customStartTime: null,

  customEndTime: null,

  customDurationFormatted: null,

  customStartTimeFormatted: function () {
    var time = this.get('customStartTime');
    return Em.isNone(time) ? '' : App.formatDateTimeWithTimeZone(time, 'M/D/YYYY hh:mmA');
  }.property('customStartTime'),

  customEndTimeFormatted: function () {
    var time = this.get('customEndTime');
    return Em.isNone(time) ? '' : App.formatDateTimeWithTimeZone(time, 'M/D/YYYY hh:mmA');
  }.property('customEndTime'),

  tooltipMessage: function () {
    var message;
    if (this.get('isCustomTimeRange')) {
      message = Em.I18n.t('common.start') + ': ' + this.get('customStartTimeFormatted') + '<br>' + Em.I18n.t('common.duration') + ': ' + this.get('customDurationFormatted') + '<br>' + Em.I18n.t('common.end') + ': ' + this.get('customEndTimeFormatted');
    } else {
      message = '';
    }
    return message;
  }.property('customStartTimeFormatted', 'customEndTimeFormatted', 'customDurationFormatted'),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$(), {
      selector: '.dropdown-toggle[rel="tooltip"]',
      html: true,
      placement: 'left'
    });
  },

  /**
   * onclick handler for a time range option
   * @param {object} event
   * @param {function} callback
   * @param {object} context
   */
  setTimeRange: function setTimeRange(event, callback, context) {
    var prevCustomTimeRange = this.getProperties(['currentTimeRangeIndex', 'customStartTime', 'customEndTime', 'customDurationFormatted']),
        index = event.context.index,
        primary = function primary() {
      var timeRange = {
        customEndTime: timeRangePopup.endTime,
        customStartTime: timeRangePopup.startTime,
        customDurationFormatted: timeRangePopup.customDuration
      };
      if (callback) {
        callback();
      }
      this.set('currentTimeRangeIndex', index);
      this.setProperties(timeRange);
      if (context) {
        context.setProperties(timeRange);
      }
    },
        secondary = this.setProperties.bind(this, prevCustomTimeRange);

    if (index === 8) {
      // Custom start and end time is specified by user
      var defaultStartTime, defaultEndTime, duration;
      if (prevCustomTimeRange.currentTimeRangeIndex === 8) {
        // Custom time range is active
        defaultStartTime = new Date(this.get('customStartTime')).getTime();
        defaultEndTime = new Date(this.get('customEndTime')).getTime();
        duration = this.get('customDurationFormatted') || 0;
      } else {
        // Preset time range is active
        var minutes;
        duration = this.get('currentTimeRange.value') * 3600000;
        defaultEndTime = new Date();
        minutes = 5 * Math.ceil(defaultEndTime.getMinutes() / 5);
        defaultEndTime.setMinutes(minutes);
        defaultStartTime = defaultEndTime.getTime() - duration;
      }
      timeRangePopup.showCustomDatePopup(primary.bind(this), secondary, {
        startDate: App.formatDateTimeWithTimeZone(defaultStartTime, 'MM/DD/YYYY'),
        hoursForStart: App.formatDateTimeWithTimeZone(defaultStartTime, 'hh'),
        minutesForStart: App.formatDateTimeWithTimeZone(defaultStartTime, 'mm'),
        middayPeriodForStart: App.formatDateTimeWithTimeZone(defaultStartTime, 'A'),
        duration: duration,
        endDate: App.formatDateTimeWithTimeZone(defaultEndTime, 'MM/DD/YYYY'),
        hoursForEnd: App.formatDateTimeWithTimeZone(defaultEndTime, 'hh'),
        minutesForEnd: App.formatDateTimeWithTimeZone(defaultEndTime, 'mm'),
        middayPeriodForEnd: App.formatDateTimeWithTimeZone(defaultEndTime, 'A')
      });
    } else {
      // Preset time range is specified by user
      this.setProperties({
        currentTimeRangeIndex: index,
        customStartTime: null,
        customEndTime: null,
        customDurationFormatted: null
      });
    }
  },

  timeRangeListView: App.TimeRangeListView.extend()

});

});

require.register("mixins/common/widgets/widget_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.WidgetMixin = Ember.Mixin.create({

  /**
   *  type of metric query from which the widget is comprised
   */

  metricType: 'POINT_IN_TIME',
  /**
   * @type {RegExp}
   * @const
   */
  EXPRESSION_REGEX: /\$\{([\w\s\.\,\+\-\*\/\(\)\:\=\[\]]*)\}/g,

  /**
   * @type {RegExp}
   * @const
   */
  MATH_EXPRESSION_REGEX: /^[\d\s\+\-\*\/\(\)\.]+$/,

  /**
   * @type {RegExp}
   * @const
   */
  VALUE_NAME_REGEX: /[^(\s+\-\*\/\\+\s+)+](\w+\s+\w+)?[\w\.\,\-\:\=\[\]]*/g,

  /**
   * @type {string}
   * @const
   */
  CLONE_SUFFIX: '(Copy)',

  /**
   * @type {number|null}
   */
  timeoutId: null,

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: [],

  /**
   *
   */
  aggregatorFunc: ['._sum', '._avg', '._min', '._max', '._rate'],

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * @type {App.Widget}
   * @default null
   */
  content: null,

  /**
   * color of content calculated by thresholds
   * @type {string}
   */
  contentColor: Em.computed.ifThenElse('value', 'green', 'grey'),

  beforeRender: function beforeRender() {
    this.loadMetrics();
  },

  /**
   * load metrics
   */
  loadMetrics: function () {
    var requestData = this.getRequestData(this.get('content.metrics')),
        request,
        requestCounter = 0,
        self = this;

    this.set('metrics', []);

    for (var i in requestData) {
      request = requestData[i];
      requestCounter++;
      if (this.get('content.widgetType') === 'HEATMAP') {
        if (request.service_name === 'STACK') {
          this.getHostsMetrics(request).complete(function () {
            requestCounter--;
            if (requestCounter === 0) self.onMetricsLoaded();
          });
        } else {
          this.getHostComponentsMetrics(request).complete(function () {
            requestCounter--;
            if (requestCounter === 0) self.onMetricsLoaded();
          });
        }
      } else if (request.host_component_criteria) {
        App.WidgetLoadAggregator.add({
          data: request,
          context: this,
          startCallName: 'getHostComponentMetrics',
          successCallback: this.getHostComponentMetricsSuccessCallback,
          errorCallback: this.getMetricsErrorCallback,
          completeCallback: function completeCallback(xhr) {
            requestCounter--;
            if (requestCounter === 0) this.onMetricsLoaded();
            if (this.get('graphView')) {
              var graph = this.get('childViews') && this.get('childViews').findProperty('runningRequests');
              if (graph) {
                var requestsArrayName = graph.get('isPopup') ? 'runningPopupRequests' : 'runningRequests';
                graph.set(requestsArrayName, graph.get(requestsArrayName).reject(function (item) {
                  return item === xhr;
                }));
              }
            }
          }
        });
      } else {
        App.WidgetLoadAggregator.add({
          data: request,
          context: this,
          startCallName: 'getServiceComponentMetrics',
          successCallback: this.getMetricsSuccessCallback,
          errorCallback: this.getMetricsErrorCallback,
          completeCallback: function completeCallback(xhr) {
            requestCounter--;
            if (requestCounter === 0) this.onMetricsLoaded();
            if (this.get('graphView')) {
              var graph = this.get('childViews') && this.get('childViews').findProperty('runningRequests');
              if (graph) {
                var requestsArrayName = graph.get('isPopup') ? 'runningPopupRequests' : 'runningRequests';
                graph.set(requestsArrayName, graph.get(requestsArrayName).reject(function (item) {
                  return item === xhr;
                }));
              }
            }
          }
        });
      }
    }
  }.observes('customTimeRange', 'content.properties.time_range'),

  /**
   * get data formatted for request
   * @param {Array} metrics
   */
  getRequestData: function getRequestData(metrics) {
    var requestsData = {};
    if (metrics) {
      metrics.forEach(function (metric) {
        var key;
        if (metric.host_component_criteria) {
          key = metric.service_name + '_' + metric.component_name + '_' + metric.host_component_criteria;
        } else {
          key = metric.service_name + '_' + metric.component_name;
        }
        var requestMetric = $.extend({}, metric);

        if (requestsData[key]) {
          requestsData[key]["metric_paths"].push({
            metric_path: requestMetric["metric_path"],
            metric_type: this.get('metricType'),
            id: requestMetric["metric_path"] + "_" + this.get('metricType'),
            context: this
          });
        } else {
          requestMetric["metric_paths"] = [{
            metric_path: requestMetric["metric_path"],
            metric_type: this.get('metricType'),
            id: requestMetric["metric_path"] + "_" + this.get('metricType'),
            context: this }];
          delete requestMetric["metric_path"];
          requestMetric.tag = this.get('content.tag');
          requestsData[key] = requestMetric;
        }
      }, this);
    }
    return requestsData;
  },

  /**
   * Tweak host component criteria  from the actual host component criteria
   * @param {object} request
   */
  computeHostComponentCriteria: function computeHostComponentCriteria(request) {
    return request.host_component_criteria.replace('host_components/', '&').trim();
  },

  /**
   * make GET call to server in order to fetch service-component metrics
   * @param {object} request
   * @returns {$.ajax}
   */
  getServiceComponentMetrics: function getServiceComponentMetrics(request) {
    var xhr = App.ajax.send({
      name: 'widgets.serviceComponent.metrics.get',
      sender: this,
      data: {
        serviceName: request.service_name,
        componentName: request.component_name,
        metricPaths: this.prepareMetricPaths(request.metric_paths)
      }
    });
    if (this.get('graphView')) {
      var graph = this.get('childViews') && this.get('childViews').findProperty('runningRequests');
      if (graph) {
        var requestsArrayName = graph.get('isPopup') ? 'runningPopupRequests' : 'runningRequests';
        graph.get(requestsArrayName).push(xhr);
      }
    }
    return xhr;
  },

  /**
   * aggregate all metric names in the query. Add time range and step to temporal queries
   * @param {Array} metricPaths
   * @returns {string}
   */
  prepareMetricPaths: function prepareMetricPaths(metricPaths) {
    var temporalMetrics = metricPaths.filterProperty('metric_type', 'TEMPORAL');
    var pointInTimeMetrics = metricPaths.filterProperty('metric_type', 'POINT_IN_TIME');
    var result = temporalMetrics.length ? temporalMetrics[0].context.addTimeProperties(temporalMetrics.mapProperty('metric_path')) : [];

    if (pointInTimeMetrics.length) {
      result = result.concat(pointInTimeMetrics.mapProperty('metric_path'));
    }

    return result.join(',');
  },

  /**
   * make GET call to server in order to fetch specific host-component metrics
   * @param {object} request
   * @returns {$.ajax}
   */
  getHostComponentMetrics: function getHostComponentMetrics(request) {
    var metricPaths = this.prepareMetricPaths(request.metric_paths);
    var data = {
      componentName: request.component_name,
      metricPaths: this.prepareMetricPaths(request.metric_paths),
      hostComponentCriteria: this.computeHostComponentCriteria(request)
    };

    if (request.tag) {
      data.selectedHostsParam = '&HostRoles/host_name.in(' + App.HDFSService.find().objectAt(0).get('masterComponentGroups').findProperty('name', request.tag).hosts.join(',') + ')';
    }

    if (metricPaths.length) {
      var xhr = App.ajax.send({
        name: 'widgets.hostComponent.metrics.get',
        sender: this,
        data: data
      }),
          graph = this.get('graphView') && this.get('childViews') && this.get('childViews').findProperty('runningRequests');
      if (graph) {
        var requestsArrayName = graph.get('isPopup') ? 'runningPopupRequests' : 'runningRequests';
        graph.get(requestsArrayName).push(xhr);
      }
      return xhr;
    }
    return jQuery.Deferred().reject().promise();
  },

  getHostComponentMetricsSuccessCallback: function getHostComponentMetricsSuccessCallback(data) {
    if (data.items[0]) {
      this.getMetricsSuccessCallback(data.items[0]);
    }
  },

  /**
   * callback on getting aggregated metrics and host component metrics
   * @param data
   */
  getMetricsSuccessCallback: function getMetricsSuccessCallback(data) {
    var metrics = [];
    var atLeastOneMetricPresent = false;

    if (this.get('content.metrics')) {
      this.get('content.metrics').forEach(function (_metric) {
        var metric_path = _metric.metric_path;
        var isAggregatorFunc = false;
        var metric_data = Em.get(data, metric_path.replace(/\//g, '.'));
        if (Em.isNone(metric_data)) {
          this.aggregatorFunc.forEach(function (_item) {
            if (metric_path.endsWith(_item) && !isAggregatorFunc) {
              isAggregatorFunc = true;
              var metricBeanProperty = metric_path.split("/").pop();
              var metricBean;
              metric_path = metric_path.substring(0, metric_path.indexOf(metricBeanProperty));
              if (metric_path.endsWith("/")) {
                metric_path = metric_path.slice(0, -1);
              }
              metricBean = Em.get(data, metric_path.replace(/\//g, '.'));
              if (!Em.isNone(metricBean)) {
                metric_data = metricBean[metricBeanProperty];
              }
            }
          }, this);
        }
        if (!Em.isNone(metric_data)) {
          atLeastOneMetricPresent = true;
          _metric.data = metric_data;
          this.get('metrics').pushObject(_metric);
        }
      }, this);

      if (!atLeastOneMetricPresent) {
        this.disableGraph();
      }
    }
  },

  /**
   * if no metrics were received from server then disable graph
   */
  disableGraph: function disableGraph() {
    if (this.get('graphView')) {
      var graph = this.get('childViews') && this.get('childViews').findProperty('_showMessage');
      if (graph) {
        graph.set('hasData', false);
        this.set('isExportButtonHidden', true);
        graph._showMessage('info', this.t('graphs.noData.title'), this.t('graphs.noDataAtTime.message'));
      }
    }
    this.set('metrics', this.get('metrics').reject(function (item) {
      return this.get('content.metrics').someProperty('name', item.name);
    }, this));
  },

  /**
   * error callback on getting aggregated metrics and host component metrics
   * @param {object} xhr
   * @param {string} textStatus
   * @param {string} errorThrown
   */
  getMetricsErrorCallback: function getMetricsErrorCallback(xhr, textStatus, errorThrown) {
    if (this.get('graphView') && !xhr.isForcedAbort) {
      var graph = this.get('childViews') && this.get('childViews').findProperty('_showMessage');
      if (graph) {
        if (xhr.readyState == 4 && xhr.status) {
          textStatus = xhr.status + " " + textStatus;
        }
        graph.set('hasData', false);
        this.set('isExportButtonHidden', true);
        graph._showMessage('warn', this.t('graphs.error.title'), this.t('graphs.error.message').format(textStatus, errorThrown));
        this.set('metrics', this.get('metrics').reject(function (item) {
          return this.get('content.metrics').someProperty('name', item.name);
        }, this));
      }
    }
  },

  /**
   * make GET call to get metrics value for all host components
   * @param {object} request
   * @return {$.ajax}
   */
  getHostComponentsMetrics: function getHostComponentsMetrics(request) {
    request.metric_paths.forEach(function (_metric, index) {
      request.metric_paths[index] = "host_components/" + _metric.metric_path;
    });
    return App.ajax.send({
      name: 'widgets.serviceComponent.metrics.get',
      sender: this,
      data: {
        serviceName: request.service_name,
        componentName: request.component_name,
        metricPaths: request.metric_paths.join(',')
      },
      success: 'getHostComponentsMetricsSuccessCallback'
    });
  },

  getHostComponentsMetricsSuccessCallback: function getHostComponentsMetricsSuccessCallback(data) {
    var metrics = this.get('content.metrics');
    data.host_components.forEach(function (item) {
      metrics.forEach(function (_metric) {
        var metric = $.extend({}, _metric, true);
        metric.hostName = item.HostRoles.host_name;
        if (!Em.isNone(Em.get(item, _metric.metric_path.replace(/\//g, '.')))) {
          metric.data = Em.get(item, _metric.metric_path.replace(/\//g, '.'));
        }
        this.get('metrics').pushObject(metric);
      }, this);
    }, this);
  },

  getHostsMetrics: function getHostsMetrics(request) {
    var metricPaths = request.metric_paths.mapProperty('metric_path');
    return App.ajax.send({
      name: 'widgets.hosts.metrics.get',
      sender: this,
      data: {
        metricPaths: metricPaths.join(',')
      },
      success: 'getHostsMetricsSuccessCallback'
    });
  },

  getHostsMetricsSuccessCallback: function getHostsMetricsSuccessCallback(data) {
    var metrics = this.get('content.metrics');
    data.items.forEach(function (item) {
      metrics.forEach(function (_metric, index) {
        var metric = $.extend({}, _metric, true);
        metric.hostName = item.Hosts.host_name;
        if (!Em.isNone(Em.get(item, _metric.metric_path.replace(/\//g, '.')))) {
          metric.data = Em.get(item, _metric.metric_path.replace(/\//g, '.'));
        }
        this.get('metrics').pushObject(metric);
      }, this);
    }, this);
  },

  /**
   * callback on metrics loaded
   */
  onMetricsLoaded: function onMetricsLoaded() {
    var self = this;
    if (!this.get('isLoaded')) this.set('isLoaded', true);
    this.drawWidget();
    clearTimeout(this.get('timeoutId'));
    this.set('timeoutId', setTimeout(function () {
      self.loadMetrics();
    }, App.contentUpdateInterval));
  },

  /**
   * draw widget
   */
  drawWidget: function drawWidget() {
    if (this.get('isLoaded')) {
      this.calculateValues();
      this.set('value', this.get('content.values')[0] ? this.get('content.values')[0].computedValue : '');
    }
  },

  /**
   * initialize tooltips
   */
  initTooltip: function () {
    var self = this;

    if (this.get('isLoaded')) {
      Em.run.next(function () {
        if (self.get('state') === 'inDOM') {
          App.tooltip(self.$(".corner-icon > .glyphicon-copy"), { title: Em.I18n.t('common.clone') });
          App.tooltip(self.$(".corner-icon > .glyphicon-edit"), { title: Em.I18n.t('common.edit') });
          App.tooltip(self.$(".corner-icon > .glyphicon-save"), { title: Em.I18n.t('common.export') });
        }
      });
    }
  }.observes('isLoaded'),

  willDestroyElement: function willDestroyElement() {
    this.$(".corner-icon > .glyphicon-copy").tooltip('destroy');
    this.$(".corner-icon > .glyphicon-edit").tooltip('destroy');
    this.$(".corner-icon > .glyphicon-save").tooltip('destroy');
  },

  /**
   * calculate series datasets for graph widgets
   */
  calculateValues: function calculateValues() {
    this.get('content.values').forEach(function (value) {
      var computeExpression = this.computeExpression(this.extractExpressions(value), this.get('metrics'));
      value.computedValue = value.value.replace(this.get('EXPRESSION_REGEX'), function (match) {
        var float = parseFloat(computeExpression[match]);
        if (isNaN(float)) {
          return computeExpression[match] || "<span class=\"grey\">n/a</span>";
        } else {
          return String(float % 1 !== 0 ? float.toFixed(2) : float);
        }
      });
    }, this);
  },

  /**
   * extract expressions
   * Example:
   *  input: "${a/b} equal ${b+a}"
   *  expressions: ['a/b', 'b+a']
   *
   * @param {object} input
   * @returns {Array}
   */
  extractExpressions: function extractExpressions(input) {
    var pattern = this.get('EXPRESSION_REGEX'),
        expressions = [],
        match;

    if (Em.isNone(input)) return expressions;

    while (match = pattern.exec(input.value)) {
      expressions.push(match[1]);
    }
    return expressions;
  },

  /**
   * compute expression
   * @param expressions
   * @param metrics
   * @returns {object}
   */
  computeExpression: function computeExpression(expressions, metrics) {
    var result = {};

    expressions.forEach(function (_expression) {
      var validExpression = true;
      var value = "";

      //replace values with metrics data
      var beforeCompute = _expression.replace(this.get('VALUE_NAME_REGEX'), function (match) {
        if (window.isNaN(match)) {
          if (metrics.someProperty('name', match)) {
            return metrics.findProperty('name', match).data;
          } else {
            validExpression = false;
          }
        } else {
          return match;
        }
      });

      //check for correct math expression
      if (!(validExpression && this.get('MATH_EXPRESSION_REGEX').test(beforeCompute))) {
        validExpression = false;
      }

      result['${' + _expression + '}'] = validExpression ? Number(window.eval(beforeCompute)).toString() : value;
    }, this);
    return result;
  },

  /*
   * make call when clicking on "remove icon" on widget
   */
  hideWidget: function hideWidget(event) {
    this.get('controller').hideWidget({
      context: Em.Object.create({
        id: event.contexts[0],
        nsLayout: event.contexts[1]
      })
    });
  },

  /*
   * make call when clicking on "clone icon" on widget
   */
  cloneWidget: function cloneWidget(event) {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.postWidgetDefinition(true);
    }, Em.I18n.t('widget.clone.body').format(stringUtils.htmlEntities(self.get('content.widgetName'))), null, null, Em.I18n.t('common.clone'));
  },

  /**
   * collect all needed data to create new widget
   * @returns {{WidgetInfo: {widget_name: *, widget_type: *, description: *, scope: *, metrics: *, values: *, properties: *}}}
   */
  collectWidgetData: function collectWidgetData() {
    var widgetData = {
      WidgetInfo: {
        widget_name: this.get('content.widgetName'),
        widget_type: this.get('content.widgetType'),
        description: this.get('content.widgetDescription'),
        scope: this.get('content.scope'),
        "metrics": this.get('content.metrics').map(function (metric) {
          return {
            "name": metric.name,
            "service_name": metric.service_name,
            "component_name": metric.component_name,
            "host_component_criteria": metric.host_component_criteria,
            "metric_path": metric.metric_path
          };
        }),
        values: this.get('content.values'),
        properties: this.get('content.properties')
      }
    };

    this.get('content.metrics').forEach(function (metric) {
      if (metric.tag) widgetData.WidgetInfo.tag = metric.tag;
    });

    return widgetData;
  },

  /**
   * post widget definition to server
   * @param {boolean} isClone
   *  * @param {boolean} isEditClonedWidget
   * @returns {$.ajax}
   */
  postWidgetDefinition: function postWidgetDefinition(isClone, isEditClonedWidget) {
    var data = this.collectWidgetData();
    if (isClone) {
      data.WidgetInfo.widget_name += this.get('CLONE_SUFFIX');
      data.WidgetInfo.scope = 'USER';
    }
    var successCallback = isEditClonedWidget ? 'editNewClonedWidget' : 'postWidgetDefinitionSuccessCallback';
    return App.ajax.send({
      name: 'widgets.wizard.add',
      sender: this,
      data: {
        data: data
      },
      success: successCallback
    });
  },

  postWidgetDefinitionSuccessCallback: function postWidgetDefinitionSuccessCallback(data) {
    var widgets = this.get('content.layout.widgets').toArray();
    widgets.pushObject(Em.Object.create({
      id: data.resources[0].WidgetInfo.id
    }));
    var mainServiceInfoMetricsController = App.router.get('mainServiceInfoMetricsController');
    mainServiceInfoMetricsController.saveWidgetLayout(widgets).done(function () {
      mainServiceInfoMetricsController.updateActiveLayout();
    });
  },

  /*
   * enter edit wizard of the newly cloned widget
   */
  editNewClonedWidget: function editNewClonedWidget(data) {
    var controller = this.get('controller');
    var widgets = this.get('content.layout.widgets').toArray();
    var id = data.resources[0].WidgetInfo.id;
    widgets.pushObject(Em.Object.create({
      id: id
    }));
    var mainServiceInfoMetricsController = App.router.get('mainServiceInfoMetricsController');
    mainServiceInfoMetricsController.saveWidgetLayout(widgets).done(function () {
      mainServiceInfoMetricsController.getActiveWidgetLayout().done(function () {
        var newWidget = App.Widget.find().findProperty('id', id);
        controller.editWidget(newWidget);
      });
    });
  },

  /*
   * make call when clicking on "edit icon" on widget
   */
  editWidget: function editWidget(event) {
    var self = this;
    var isShared = this.get('content.scope') == 'CLUSTER';
    if (!isShared) {
      self.get('controller').editWidget(self.get('content'));
    } else {
      return App.ModalPopup.show({
        header: Em.I18n.t('common.warning'),
        bodyClass: Em.View.extend({
          template: Ember.Handlebars.compile('{{t widget.edit.body}}')
        }),
        primary: Em.I18n.t('widget.edit.button.primary'),
        secondary: Em.I18n.t('widget.edit.button.secondary'),
        third: Em.I18n.t('common.cancel'),
        onPrimary: function onPrimary() {
          this.hide();
          self.get('controller').editWidget(self.get('content'));
        },
        onSecondary: function onSecondary() {
          this.hide();
          self.postWidgetDefinition(true, true);
        },
        onThird: function onThird() {
          this.hide();
        }
      });
    }
  }

});
App.WidgetPreviewMixin = Ember.Mixin.create({
  beforeRender: Em.K,
  isLoaded: true,
  isPreview: true,
  metrics: [],
  content: Em.Object.create({
    id: 1,
    values: []
  }),
  loadMetrics: function () {
    this.get('content').setProperties({
      'values': this.get('controller.widgetValues'),
      'properties': this.get('controller.widgetProperties'),
      'widgetName': this.get('controller.widgetName'),
      'metrics': this.get('controller.widgetMetrics')
    });
    this._super();
  }.observes('controller.widgetProperties', 'controller.widgetValues', 'controller.widgetMetrics', 'controller.widgetName'),
  onMetricsLoaded: function onMetricsLoaded() {
    this.drawWidget();
  }
});

/**
 * aggregate requests to load metrics by component name
 * requests can be added via add method
 * input example:
 * {
 *   data: request,
 *   context: this,
 *   startCallName: this.getServiceComponentMetrics,
 *   successCallback: this.getMetricsSuccessCallback,
 *   completeCallback: function () {
 *     requestCounter--;
 *     if (requestCounter === 0) this.onMetricsLoaded();
 *   }
 * }
 * @type {Em.Object}
 */
App.WidgetLoadAggregator = Em.Object.create({
  /**
   * @type {Array}
   */
  requests: [],

  /**
   * @type {number|null}
   */
  timeoutId: null,

  /**
   * @type {number}
   * @const
   */
  BULK_INTERVAL: 1000,

  arrayUtils: require('utils/array_utils'),

  /**
   * add request
   * every {{BULK_INTERVAL}} requests get collected, aggregated and sent to server
   *
   * @param {object} request
   */
  add: function add(request) {
    var self = this;

    this.get('requests').push(request);
    if (Em.isNone(this.get('timeoutId'))) {
      this.set('timeoutId', window.setTimeout(function () {
        //clear requests that are belongs to destroyed views
        self.set('requests', self.get('requests').filterProperty('context.state', 'inDOM'));

        self.runRequests(self.get('requests'));
        self.get('requests').clear();
        clearTimeout(self.get('timeoutId'));
        self.set('timeoutId', null);
      }, this.get('BULK_INTERVAL')));
    }
  },

  /**
   * return requests which grouped into bulks
   * @param {Array} requests
   * @returns {object} bulks
   */
  groupRequests: function groupRequests(requests) {
    var bulks = {};

    requests.forEach(function (request) {
      //poll metrics for graph widgets separately
      var graphSuffix = request.context.get('content.widgetType') === "GRAPH" ? "_graph" : '';
      var tagSuffix = request.context.get('content.tag') ? '_' + request.context.get('content.tag') : '';
      var id = request.startCallName + "_" + request.data.component_name + graphSuffix + tagSuffix;

      if (Em.isNone(bulks[id])) {
        bulks[id] = {
          data: request.data,
          context: request.context,
          startCallName: request.startCallName
        };
        bulks[id].subRequests = [{
          context: request.context,
          successCallback: request.successCallback,
          errorCallback: request.errorCallback,
          completeCallback: request.completeCallback
        }];
      } else {
        bulks[id].data.metric_paths.pushObjects(request.data.metric_paths);
        bulks[id].subRequests.push({
          context: request.context,
          successCallback: request.successCallback,
          errorCallback: request.errorCallback,
          completeCallback: request.completeCallback
        });
      }
    }, this);
    return bulks;
  },

  /**
   * run aggregated requests
   * @param {Array} requests
   */
  runRequests: function runRequests(requests) {
    var bulks = this.groupRequests(requests);
    var self = this;
    for (var id in bulks) {
      (function (_request) {
        _request.data.metric_paths = self.arrayUtils.uniqObjectsbyId(_request.data.metric_paths, "id");
        _request.context[_request.startCallName].call(_request.context, _request.data).done(function (response) {
          _request.subRequests.forEach(function (subRequest) {
            subRequest.successCallback.call(subRequest.context, response);
          }, this);
        }).fail(function (xhr, textStatus, errorThrown) {
          _request.subRequests.forEach(function (subRequest) {
            if (subRequest.errorCallback) {
              subRequest.errorCallback.call(subRequest.context, xhr, textStatus, errorThrown);
            }
          }, this);
        }).always(function (xhr) {
          _request.subRequests.forEach(function (subRequest) {
            subRequest.completeCallback.call(subRequest.context, xhr);
          }, this);
        });
      })(bulks[id]);
    }
  }
});

});

require.register("mixins/common/widgets/widget_section", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetSectionMixin = Ember.Mixin.create({
  /**
   * UI default layout name
   */
  defaultLayoutName: function () {
    var heatmapType;
    if (this.get('content.serviceName')) {
      heatmapType = this.get('content.serviceName').toLowerCase();
    } else {
      heatmapType = "system";
    }
    return "default_" + heatmapType + this.layoutNameSuffix;
  }.property('content.serviceName'),

  /**
   * UI user default layout name
   */
  userLayoutName: function () {
    var heatmapType;
    var loginName = App.router.get('loginName');
    if (this.get('content.serviceName')) {
      heatmapType = this.get('content.serviceName').toLowerCase();
    } else {
      heatmapType = "system";
    }
    return loginName + "_" + heatmapType + this.layoutNameSuffix;
  }.property('content.serviceName'),

  /**
   * UI section name
   */
  sectionName: function () {
    if (this.get('content.serviceName')) {
      return this.get('content.serviceName') + this.sectionNameSuffix;
    } else {
      return "SYSTEM" + this.sectionNameSuffix;
    }
  }.property('content.serviceName'),

  /**
   * @type {Em.A}
   */
  widgetLayouts: function () {
    return App.WidgetLayout.find();
  }.property('isWidgetLayoutsLoaded'),

  /**
   * Does Service has widget descriptor defined in the stack
   * @type {boolean}
   */
  isServiceWithEnhancedWidgets: function () {
    var isServiceWithWidgetdescriptor;
    var serviceName = this.get('content.serviceName');
    if (serviceName) {
      isServiceWithWidgetdescriptor = App.StackService.find().findProperty('serviceName', serviceName).get('isServiceWithWidgets');
    } else if (this.get('sectionName') === 'SYSTEM_HEATMAPS') {
      isServiceWithWidgetdescriptor = true;
    }
    return isServiceWithWidgetdescriptor;
  }.property('content.serviceName'),

  isHDFSFederatedSummary: function () {
    return this.get('content.serviceName') === 'HDFS' && this.get('sectionNameSuffix') === '_SUMMARY' && App.get('hasNameNodeFederation');
  }.property('content.serviceName', 'sectionNameSuffix', 'App.hasNameNodeFederation'),

  /**
   *  @Type {App.WidgetLayout}
   */
  activeWidgetLayout: {},

  /**
   * @type {Em.A}
   */
  widgets: function () {
    if (this.get('isWidgetsLoaded') && this.get('activeWidgetLayout.widgets')) {
      return this.get('activeWidgetLayout.widgets').toArray();
    }
    return [];
  }.property('isWidgetsLoaded', 'activeWidgetLayout.widgets'),

  activeNSWidgetLayouts: [],

  selectedNSWidgetLayout: {},

  isAmbariMetricsInstalled: function () {
    return App.Service.find().someProperty('serviceName', 'AMBARI_METRICS');
  }.property('App.router.mainServiceController.content.length'),

  switchNameServiceLayout: function switchNameServiceLayout(data) {
    this.set('selectedNSWidgetLayout', data.context);
  },

  /**
   * load widgets defined by user
   * @returns {$.ajax}
   */
  getActiveWidgetLayout: function getActiveWidgetLayout() {
    var sectionName = this.get('sectionName');
    var urlParams = 'WidgetLayoutInfo/section_name=' + sectionName;
    this.set('activeWidgetLayout', {});
    this.set('activeNSWidgetLayouts', []);
    this.set('isWidgetsLoaded', false);
    if (this.get('isServiceWithEnhancedWidgets')) {
      return App.ajax.send({
        name: 'widgets.layouts.active.get',
        sender: this,
        data: {
          userName: App.router.get('loginName'),
          sectionName: sectionName,
          urlParams: urlParams
        },
        success: 'getActiveWidgetLayoutSuccessCallback'
      });
    } else {
      this.set('isWidgetsLoaded', true);
    }
  },

  /**
   * success callback of <code>getActiveWidgetLayout()</code>
   * @param {object|null} data
   */
  getActiveWidgetLayoutSuccessCallback: function getActiveWidgetLayoutSuccessCallback(data) {
    var self = this;
    if (this.get('isHDFSFederatedSummary')) {
      this.getNameNodeWidgets().done(function (widgets) {
        widgets = widgets.items;
        var widgetsNamespaces = widgets.mapProperty('WidgetInfo.tag').uniq().without(null);
        var namespaces = App.HDFSService.find().objectAt(0).get('masterComponentGroups').mapProperty('name');
        var defaultNNWidgets = widgets.filterProperty('WidgetInfo.tag', null);
        var widgetsToCreateCount;
        if (namespaces.length > widgetsNamespaces.length) {
          widgetsToCreateCount = (namespaces.length - widgetsNamespaces.length) * defaultNNWidgets.length;
          namespaces.forEach(function (namespace) {
            if (!widgetsNamespaces.contains(namespace)) {
              defaultNNWidgets.forEach(function (widget) {
                if (widget.href) {
                  delete widget.href;
                  delete widget.WidgetInfo.id;
                  delete widget.WidgetInfo.cluster_name;
                  delete widget.WidgetInfo.author;
                  widget.WidgetInfo.metrics = JSON.parse(widget.WidgetInfo.metrics);
                  widget.WidgetInfo.values = JSON.parse(widget.WidgetInfo.values);
                }
                widget.WidgetInfo.tag = namespace;
                self.postWidget(widget).done(function () {
                  if (! --widgetsToCreateCount) {
                    self.createLayouts(data);
                  }
                });
              });
            }
          });
        } else {
          self.createLayouts(data);
        }
      });
    } else {
      this.createLayouts(data);
    }
  },

  getNameNodeWidgets: function getNameNodeWidgets() {
    return App.ajax.send({
      name: 'widgets.get',
      sender: this,
      data: {
        urlParams: 'WidgetInfo/widget_type.in(GRAPH,NUMBER,GAUGE)&WidgetInfo/scope=CLUSTER&WidgetInfo/metrics.matches(.*\"component_name\":\"NAMENODE\".*)&fields=*'
      }
    });
  },

  postWidget: function postWidget(data) {
    return App.ajax.send({
      name: 'widgets.wizard.add',
      sender: this,
      data: {
        data: data
      }
    });
  },

  createLayouts: function createLayouts(data) {
    var self = this;
    if (data.items[0]) {
      var hdfs = App.HDFSService.find().objectAt(0);
      if (hdfs && this.get('isHDFSFederatedSummary') && hdfs.get('masterComponentGroups').length + 2 !== data.items.length) {
        this.createFederationWidgetLayouts(data);
      } else {
        self.getWidgetLayoutSuccessCallback(data);
      }
    } else {
      self.getAllActiveWidgetLayouts().done(function (activeWidgetLayoutsData) {
        self.getDefaultWidgetLayoutByName(self.get('defaultLayoutName')).done(function (defaultWidgetLayoutData) {
          defaultWidgetLayoutData = defaultWidgetLayoutData.items[0].WidgetLayoutInfo;
          defaultWidgetLayoutData.layout_name = self.get('userLayoutName');
          self.createUserWidgetLayout(defaultWidgetLayoutData).done(function (userLayoutIdData) {
            var activeWidgetLayouts;
            var widgetLayouts = [];
            if (!!activeWidgetLayoutsData.items.length) {
              widgetLayouts = activeWidgetLayoutsData.items.map(function (item) {
                return {
                  "id": item.WidgetLayoutInfo.id
                };
              });
            }
            widgetLayouts.push({ id: userLayoutIdData.resources[0].WidgetLayoutInfo.id });
            activeWidgetLayouts = {
              "WidgetLayouts": widgetLayouts
            };
            self.saveActiveWidgetLayouts(activeWidgetLayouts).done(function () {
              self.getActiveWidgetLayout();
            });
          });
        });
      });
    }
  },

  getAllActiveWidgetLayouts: function getAllActiveWidgetLayouts() {
    return App.ajax.send({
      name: 'widgets.layouts.all.active.get',
      sender: this,
      data: {
        userName: App.router.get('loginName')
      }
    });
  },

  /**
   * success callback of <code>getWidgetLayout()</code>
   * @param {object|null} data
   */
  getWidgetLayoutSuccessCallback: function getWidgetLayoutSuccessCallback(data) {
    if (data) {
      data.items.forEach(function (item) {
        App.widgetMapper.map(item.WidgetLayoutInfo);
      });
      App.widgetLayoutMapper.map(data);
      this.set('activeWidgetLayout', App.WidgetLayout.find().findProperty('layoutName', this.get('userLayoutName')));
      if (this.get('isHDFSFederatedSummary')) {
        this.set('activeNSWidgetLayouts', App.WidgetLayout.find().filterProperty('sectionName', this.get('sectionName')).rejectProperty('layoutName', this.get('userLayoutName')));
        this.set('selectedNSWidgetLayout', this.get('activeNSWidgetLayouts')[0]);
      }
      this.set('isWidgetsLoaded', true);
    }
  },

  getDefaultWidgetLayoutByName: function getDefaultWidgetLayoutByName(layoutName) {
    var urlParams = 'WidgetLayoutInfo/layout_name=' + layoutName;
    return App.ajax.send({
      name: 'widget.layout.get',
      sender: this,
      data: {
        urlParams: urlParams
      }
    });
  },

  createUserWidgetLayout: function createUserWidgetLayout(widgetLayoutData) {
    var layout = widgetLayoutData;
    var data = {
      "WidgetLayoutInfo": {
        "display_name": layout.display_name,
        "layout_name": layout.layout_name,
        "scope": "USER",
        "section_name": layout.section_name,
        "user_name": App.router.get('loginName'),
        "widgets": layout.widgets.map(function (widget) {
          return {
            "id": widget.WidgetInfo.id
          };
        })
      }
    };
    return App.ajax.send({
      name: 'widget.layout.create',
      sender: this,
      data: {
        data: data
      }
    });
  },

  updateUserWidgetLayout: function updateUserWidgetLayout(widgetLayoutData) {
    var layout = widgetLayoutData;
    var data = {
      "WidgetLayoutInfo": {
        "display_name": layout.display_name,
        "layout_name": layout.layout_name,
        "id": layout.id,
        "scope": "USER",
        "section_name": layout.section_name,
        "user_name": App.router.get('loginName'),
        "widgets": layout.widgets.map(function (widget) {
          return {
            "id": widget.WidgetInfo.id
          };
        })
      }
    };
    return App.ajax.send({
      name: 'widget.layout.edit',
      sender: this,
      data: {
        layoutId: layout.id,
        data: data
      }
    });
  },

  saveActiveWidgetLayouts: function saveActiveWidgetLayouts(activeWidgetLayouts) {
    return App.ajax.send({
      name: 'widget.activelayouts.edit',
      sender: this,
      data: {
        data: activeWidgetLayouts,
        userName: App.router.get('loginName')
      }
    });
  },

  /**
   * save layout after re-order widgets
   * @param {Array} widgets
   * @param {Object} widgetLayout:  Optional. by default active widget layout is honored.
   * return {$.ajax}
   */
  saveWidgetLayout: function saveWidgetLayout(widgets, widgetLayout) {
    var activeLayout = widgetLayout || this.get('activeWidgetLayout');
    var data = {
      "WidgetLayoutInfo": {
        "display_name": activeLayout.get("displayName"),
        "id": activeLayout.get("id"),
        "layout_name": activeLayout.get("layoutName"),
        "scope": activeLayout.get("scope"),
        "section_name": activeLayout.get("sectionName"),
        "widgets": widgets.map(function (widget) {
          return {
            "id": widget.get('id')
          };
        })
      }
    };
    return App.ajax.send({
      name: 'widget.layout.edit',
      sender: this,
      data: {
        layoutId: activeLayout.get("id"),
        data: data
      }
    });
  },

  /**
   * After closing widget section, layout should be reset
   */
  clearActiveWidgetLayout: function clearActiveWidgetLayout() {
    this.set('activeWidgetLayout', {});
    this.set('activeNSWidgetLayouts', []);
  },

  createFederationWidgetLayouts: function createFederationWidgetLayouts(data) {
    var self = this;
    var currentLayouts = data.items;
    self.getDefaultWidgetLayoutByName(self.get('defaultLayoutName')).done(function (defaultWidgetLayoutData) {
      self.getNameNodeWidgets().done(function (widgets) {
        var newNameServices = [];
        var newWidgets = [];
        var currentLayoutsNames = [];
        var newLayout = defaultWidgetLayoutData.items[0].WidgetLayoutInfo;
        var newLayoutsIds = [];
        var nameServiceToWidgetMap = { all: [] };
        var namespaces = App.HDFSService.find().objectAt(0).get('masterComponentGroups').mapProperty('name');
        var nonNameServiceSpecific = newLayout.widgets.slice();
        var userLayoutName = self.get('userLayoutName');
        widgets.items.forEach(function (widget) {
          var tag = widget.WidgetInfo.tag;
          nonNameServiceSpecific = nonNameServiceSpecific.without(nonNameServiceSpecific.findProperty('WidgetInfo.id', widget.WidgetInfo.id));
          if (tag) {
            if (!nameServiceToWidgetMap[tag]) {
              nameServiceToWidgetMap[tag] = [];
            }
            nameServiceToWidgetMap[tag].push(widget);
            nameServiceToWidgetMap.all.push(widget);
          }
        });
        if (currentLayouts.length === 1) {
          self.removeWidgetLayout(currentLayouts[0].WidgetLayoutInfo.id).done(function () {
            newLayout.layout_name = userLayoutName;
            newLayout.widgets = nonNameServiceSpecific;
            self.createUserWidgetLayout(newLayout).done(function (data) {
              newLayoutsIds.push(data.resources[0].WidgetLayoutInfo.id);
              Em.keys(nameServiceToWidgetMap).forEach(function (nameService) {
                newLayout.layout_name = userLayoutName + '_nameservice_' + nameService;
                newLayout.display_name = nameService === 'all' ? 'All' : nameService;
                newLayout.widgets = nameServiceToWidgetMap[nameService];
                self.createUserWidgetLayout(newLayout).done(function (data) {
                  newLayoutsIds.push(data.resources[0].WidgetLayoutInfo.id);
                  if (newLayoutsIds.length >= Em.keys(nameServiceToWidgetMap).length + 1) {
                    self.saveActiveWidgetLayouts({
                      "WidgetLayouts": newLayoutsIds.map(function (layout) {
                        return { id: layout };
                      })
                    }).done(function () {
                      self.getActiveWidgetLayout();
                    });
                  }
                });
              });
            });
          });
        } else {
          currentLayoutsNames = currentLayouts.map(function (l) {
            return l.WidgetLayoutInfo.layout_name.split('_nameservice_')[1];
          }).without(undefined).without('all');
          namespaces.forEach(function (n) {
            if (!currentLayoutsNames.contains(n)) {
              newNameServices.push(n);
            }
          });
          newNameServices.forEach(function (nameService) {
            newLayout.layout_name = userLayoutName + '_nameservice_' + nameService;
            newLayout.display_name = nameService;
            newLayout.widgets = nameServiceToWidgetMap[nameService];
            newWidgets = newWidgets.concat(nameServiceToWidgetMap[nameService]);
            self.createUserWidgetLayout(newLayout).done(function (data) {
              newLayoutsIds.push(data.resources[0].WidgetLayoutInfo.id);
              if (newLayoutsIds.length >= newNameServices.length) {
                self.saveActiveWidgetLayouts({
                  "WidgetLayouts": newLayoutsIds.concat(currentLayouts.mapProperty('WidgetLayoutInfo.id')).map(function (layout) {
                    return { id: layout };
                  })
                }).done(function () {
                  var allNSLayout = currentLayouts.findProperty('WidgetLayoutInfo.display_name', 'All').WidgetLayoutInfo;
                  allNSLayout.widgets = allNSLayout.widgets.concat(newWidgets);
                  self.updateUserWidgetLayout(allNSLayout).done(function () {
                    self.getActiveWidgetLayout();
                  });
                });
              }
            });
          });
        }
      });
    });
  },

  removeWidgetLayout: function removeWidgetLayout(id) {
    return App.ajax.send({
      name: 'widget.layout.delete',
      sender: this,
      data: {
        layoutId: id
      }
    });
  }
});

});

require.register("mixins/main/dashboard/widgets/editable", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.EditableWidgetMixin = Em.Mixin.create({

  editWidget: function editWidget() {
    return App.EditDashboardWidgetPopup.show({

      widgetView: this,

      sliderHandlersManager: App.EditDashboardWidgetPopup.DoubleHandlers.create({
        maxValue: 'infinity',
        thresholdMin: Number(this.get('thresholdMin')),
        thresholdMax: Number(this.get('thresholdMax'))
      }),

      sliderDisabled: true,
      sliderHandlers: [33, 66],
      sliderMaxValue: 100,
      sliderColors: [App.healthStatusGreen, App.healthStatusOrange, App.healthStatusRed]

    });
  }

});

});

require.register("mixins/main/dashboard/widgets/editable_with_limit", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @type {Em.Mixin}
 */
App.EditableWithLimitWidgetMixin = Em.Mixin.create({

  editWidget: function editWidget() {

    return App.EditDashboardWidgetPopup.show({

      widgetView: this,

      sliderHandlersManager: App.EditDashboardWidgetPopup.DoubleHandlers.create({
        maxValue: parseFloat(this.get('maxValue')),
        thresholdMin: Number(this.get('thresholdMin')),
        thresholdMax: Number(this.get('thresholdMax'))
      })

    });
  }

});

});

require.register("mixins/main/dashboard/widgets/namenode_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeWidgetMixin = Em.Mixin.create({

  groupId: 'nn',

  subGroupId: 'default',

  isAllItemsSubGroup: false,

  componentGroup: Em.computed.findByKey('model.masterComponentGroups', 'name', 'subGroupId'),

  clusterId: Em.computed.alias('componentGroup.clusterId'),

  hostName: function () {
    var allHostNames = this.get('componentGroup.hosts') || [],
        hostComponents = App.HostComponent.find().filter(function (component) {
      return component.get('componentName') === 'NAMENODE' && allHostNames.contains(component.get('hostName'));
    }),
        resultingComponent = hostComponents.findProperty('haStatus', 'active') || hostComponents.get('firstObject');
    return resultingComponent && resultingComponent.get('hostName');
  }.property('clusterId')

});

});

require.register("mixins/main/dashboard/widgets/single_numeric_threshold", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @type {Em.Mixin}
 */
App.SingleNumericThresholdMixin = Em.Mixin.create({

  editWidget: function editWidget() {
    return App.EditDashboardWidgetPopup.show({

      widgetView: this,

      sliderHandlersManager: App.EditDashboardWidgetPopup.SingleHandler.create({
        thresholdMin: Number(this.get('thresholdMin')),
        maxValue: parseFloat(this.get('maxValue'))
      }),

      bodyClass: App.EditDashboardWidgetPopup.EditDashboardWidgetPopupBody.extend({
        templateName: require('templates/main/dashboard/edit_widget_popup_single_threshold')
      }),

      sliderIsRange: false,

      sliderColors: [App.healthStatusGreen, App.healthStatusRed]

    });
  }

});

});

require.register("mixins/main/host/details/actions/check_host", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.CheckHostMixin = Em.Mixin.create({

  bootHosts: [],

  hostCheckResult: null,

  requestId: 0,

  stopChecking: false,

  /**
   * Progress value for "update hosts status" process
   * @type {number}
   */
  checksUpdateProgress: 0,

  /**
   *
   * @type {object}
   */
  checksUpdateStatus: null,

  /**
   * Contain data about installed packages on hosts
   * @type {Array}
   */
  hostsPackagesData: [],

  /**
   * @typedef {{
   *  name: string,
   *  hosts: string[],
   *  hostsLong: string[],
   *  hostsNames: string[],
   * }} checkWarning
   */

  /**
   * @type {checkWarning[]}
   */
  jdkCategoryWarnings: null,

  /**
   * @type {checkWarning[]}
   */
  hostCheckWarnings: [],

  /**
   * @type {checkWarning[]}
   */
  repoCategoryWarnings: [],

  /**
   * @type {checkWarning[]}
   */
  diskCategoryWarnings: [],

  /**
   * @type {checkWarning[]}
   */
  thpCategoryWarnings: [],

  /**
   * All hosts warnings
   * @type {object[]}
   */
  warnings: [],

  /**
   * Warnings grouped by host
   * @type {Ember.Enumerable}
   */
  warningsByHost: [],

  /**
   * This property should be overridden if being used for Add Host Wizard
   * @return {bool}
   */
  isAddHostWizard: false,

  /**
   * True if user triggered host checks rerun
   * @type {boolean}
   */
  isRerun: false,

  /**
   * Timeout for "warning"-requests
   * @type {number}
   */
  warningsTimeInterval: 60000,

  finishStates: ["FAILED", "COMPLETED", "TIMEDOUT", "ABORTED"],

  /**
   * disables host check on Add host wizard as per the experimental flag
   */
  disableHostCheck: function () {
    return this.get('content.installOptions.skipHostChecks') && this.get('isAddHostWizard');
  }.property('content.installOptions.skipHostChecks', 'isAddHostWizard'),

  /**
   * send request to create tasks for performing hosts checks
   * @params {object} data
   *    {
   *       RequestInfo: {
   *           "action": {string},
   *           "context": {string},
   *           "parameters": {
   *             "check_execute_list": {string},
   *             "jdk_location" : {string},
   *             "threshold": {string}
   *             "hosts": {string|undefined}
   *       },
   *       resource_filters: {
   *         "hosts": {string}
   *       }
   *    }
   * @returns {$.ajax}
   * @method requestToPerformHostCheck
   */
  requestToPerformHostCheck: function requestToPerformHostCheck(data) {
    return App.ajax.send({
      name: 'preinstalled.checks',
      sender: this,
      data: {
        RequestInfo: data.RequestInfo,
        resource_filters: data.resource_filters
      },
      success: "getHostCheckSuccess",
      error: "getHostCheckError"
    });
  },

  /**
   * set all fields from which depends running host check to true value
   * which force finish checking;
   */
  stopHostCheck: function stopHostCheck() {
    this.set('stopChecking', true);
    this.set('isJDKWarningsLoaded', true);
    this.set('isHostsWarningsLoaded', true);
  },

  getHostCheckSuccess: function getHostCheckSuccess(response) {
    if (!App.get('testMode')) {
      this.set("requestId", response.Requests.id);
    }
    this.getHostCheckTasks();
  },

  getHostCheckError: function getHostCheckError() {
    this.getHostInfo();
  },

  /**
   * send ajax request to get all tasks
   * @method getHostCheckTasks
   */
  getHostCheckTasks: function getHostCheckTasks() {
    var self = this;
    var requestId = this.get("requestId");
    var checker = setTimeout(function () {
      if (self.get('stopChecking') == true) {
        clearTimeout(checker);
      } else {
        App.ajax.send({
          name: 'preinstalled.checks.tasks',
          sender: self,
          data: {
            requestId: requestId
          },
          success: 'getHostCheckTasksSuccess',
          error: 'getHostCheckTasksError'
        });
      }
    }, 1000);
  },

  /**
   * add warnings to host warning popup if needed
   * @param data {Object} - json
   * @method getHostCheckTasksSuccess
   */
  getHostCheckTasksSuccess: function getHostCheckTasksSuccess(data) {
    if (!data) {
      //if resolution host check has corrupted data then skip it
      return this.getGeneralHostCheck();
    }
    if (this.get('finishStates').contains(data.Requests.request_status)) {
      if (data.Requests.inputs.indexOf("last_agent_env_check") != -1) {
        this.set('stopChecking', true);
        this.set('hostsPackagesData', data.tasks.map(function (task) {
          var installed_packages = Em.get(task, 'Tasks.structured_out.installed_packages');
          return {
            hostName: Em.get(task, 'Tasks.host_name'),
            transparentHugePage: Em.get(task, 'Tasks.structured_out.transparentHugePage.message'),
            installedPackages: installed_packages && Array.isArray(installed_packages) ? installed_packages : []
          };
        }));

        this.set("hostCheckResult", data); //store the data so that it can be used later on in the getHostInfo handling logic.
        /**
         * Still need to get host info for checks that the host check does not perform currently
         * Such as the OS type check and the disk space check
         * */
        this.getHostInfo();
      } else if (data.Requests.inputs.indexOf("host_resolution_check") != -1) {
        this.parseHostNameResolution(data);
        this.getGeneralHostCheck();
      }
    } else {
      this.getHostCheckTasks();
    }
  },

  /**
   * @method getHostCheckTasksError
   */
  getHostCheckTasksError: function getHostCheckTasksError() {
    this.set('stopChecking', true);
  },

  /**
   * Get disk info and cpu count of booted hosts from server
   * @return {$.ajax}
   * @method getHostInfo
   */
  getHostInfo: function getHostInfo() {
    this.set('isHostsWarningsLoaded', false);
    // begin JDK check for each host
    return App.ajax.send({
      name: 'wizard.step3.host_info',
      sender: this,
      success: 'getHostInfoSuccessCallback',
      error: 'getHostInfoErrorCallback'
    });
  },

  /**
   * Success-callback for hosts info request
   * @param {object} jsonData
   * @method getHostInfoSuccessCallback
   */
  getHostInfoSuccessCallback: function getHostInfoSuccessCallback(jsonData) {
    var hosts = this.get('bootHosts'),
        self = this,
        repoWarnings = [],
        hostsRepoNames = [],
        hostsContext = [],
        diskWarnings = [],
        hostsDiskContext = [],
        hostsDiskNames = [],
        thpWarnings = [],
        thpContext = [],
        thpHostsNames = [];

    // parse host checks warning
    var hostCheckResult = this.get("hostCheckResult");
    if (hostCheckResult) {
      this.parseHostCheckWarnings(hostCheckResult);
      this.set("hostCheckResult", null);
    } else {
      this.parseWarnings(jsonData);
    }
    this.set('isHostsWarningsLoaded', true);
    hosts.forEach(function (_host) {

      var host = App.get('testMode') ? jsonData.items[0] : jsonData.items.findProperty('Hosts.host_name', _host.name);
      if (App.get('skipBootstrap')) {
        self._setHostDataWithSkipBootstrap(_host);
      } else {
        if (host) {
          self._setHostDataFromLoadedHostInfo(_host, host);
          var host_name = Em.get(host, 'Hosts.host_name');

          var context = self.checkHostOSType(host.Hosts.os_family, host_name);
          if (context) {
            hostsContext.push(context);
            hostsRepoNames.push(host_name);
          }
          var diskContext = self.checkHostDiskSpace(host_name, host.Hosts.disk_info);
          if (diskContext) {
            hostsDiskContext.push(diskContext);
            hostsDiskNames.push(host_name);
          }
          // "Transparent Huge Pages" check
          var _hostPackagesData = self.get('hostsPackagesData').findProperty('hostName', host.Hosts.host_name);
          if (_hostPackagesData) {
            var transparentHugePage = _hostPackagesData.transparentHugePage;
            context = self.checkTHP(host_name, transparentHugePage);
          } else {
            context = self.checkTHP(host_name, Em.get(host, 'Hosts.last_agent_env.transparentHugePage'));
          }
          if (context) {
            thpContext.push(context);
            thpHostsNames.push(host_name);
          }
        }
      }
    });
    if (hostsContext.length > 0) {
      // repository warning exist
      repoWarnings.push({
        name: Em.I18n.t('installer.step3.hostWarningsPopup.repositories.name'),
        hosts: hostsContext,
        hostsLong: hostsContext,
        hostsNames: hostsRepoNames,
        category: 'repositories'
      });
    }
    if (hostsDiskContext.length > 0) {
      // disk space warning exist
      diskWarnings.push({
        name: Em.I18n.t('installer.step3.hostWarningsPopup.disk.name'),
        hosts: hostsDiskContext,
        hostsLong: hostsDiskContext,
        hostsNames: hostsDiskNames,
        category: 'disk'
      });
    }
    if (thpContext.length > 0) {
      // THP warning existed
      thpWarnings.push({
        name: Em.I18n.t('installer.step3.hostWarningsPopup.thp.name'),
        hosts: thpContext,
        hostsLong: thpContext,
        hostsNames: thpHostsNames,
        category: 'thp'
      });
    }

    this.set('repoCategoryWarnings', repoWarnings);
    this.set('diskCategoryWarnings', diskWarnings);
    this.set('thpCategoryWarnings', thpWarnings);
    this.stopRegistration();
    this.get('name') === 'mainHostDetailsController' && !this.get('isRerun') && this.set('checkHostFinished', true);
  },

  /**
   * Error-callback for hosts info request
   * @method getHostInfoErrorCallback
   */
  getHostInfoErrorCallback: function getHostInfoErrorCallback() {
    this.set('isHostsWarningsLoaded', true);
    this.registerErrPopup(Em.I18n.t('installer.step3.hostInformation.popup.header'), Em.I18n.t('installer.step3.hostInformation.popup.body'));
  },

  /**
   * parse warnings for host names resolution only
   * @param {object} data
   * @method parseHostNameResolution
   */
  parseHostNameResolution: function parseHostNameResolution(data) {
    if (!data) {
      return;
    }
    data.tasks.forEach(function (task) {
      var name = Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.error');
      var hostInfo = this.get("hostCheckWarnings").findProperty('name', name);
      if (this.get('finishStates').contains(task.Tasks.status)) {
        if (task.Tasks.status === "COMPLETED" && !!Em.get(task, "Tasks.structured_out.host_resolution_check.failed_count")) {
          var targetHostName = Em.get(task, "Tasks.host_name");
          var relatedHostNames = Em.get(task, "Tasks.structured_out.host_resolution_check.hosts_with_failures") || [];
          var contextMessage = Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.context').format(targetHostName, relatedHostNames.length + ' ' + Em.I18n.t('installer.step3.hostWarningsPopup.host' + (relatedHostNames.length == 1 ? '' : 's')));
          var contextMessageLong = Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.context').format(targetHostName, relatedHostNames.join(', '));
          if (!hostInfo) {
            hostInfo = {
              name: name,
              hosts: [contextMessage],
              hostsLong: [contextMessageLong],
              hostsNames: [targetHostName]
            };
            this.get("hostCheckWarnings").push(hostInfo);
          } else {
            if (!hostInfo.hostsNames.contains(targetHostName)) {
              hostInfo.hosts.push(contextMessage);
              hostInfo.hostsLong.push(contextMessageLong);
              hostInfo.hostsNames.push(targetHostName);
            }
          }
        }
      }
    }, this);
  },

  getGeneralHostCheck: function getGeneralHostCheck(dataForHostCheck) {
    if (App.get('testMode')) {
      this.getHostInfo();
    } else {
      var data = dataForHostCheck || this.getDataForCheckRequest("last_agent_env_check,installed_packages,existing_repos,transparentHugePage", false);
      data ? this.requestToPerformHostCheck(data) : this.stopHostCheck();
    }
  },

  getHostNameResolution: function getHostNameResolution(dataForHostCheck) {
    if (App.get('testMode')) {
      this.getHostCheckSuccess();
    } else {
      var data = dataForHostCheck || this.getDataForCheckRequest("host_resolution_check", true);
      if (data && !this.get('disableHostCheck')) {
        this.requestToPerformHostCheck(data);
      } else {
        this.stopHostCheck();
        this.stopRegistration();
      }
    }
  },

  /**
   * Check warnings from server and put it in parsing
   * @method rerunChecks
   */
  rerunChecks: function rerunChecks() {
    var self = this;
    var currentProgress = 0;
    this.get('name') === 'wizardStep3Controller' ? this.getHostNameResolution() : this.getHostNameResolution(this.getDataForHostCheck());
    this.setProperties({
      stopChecking: false,
      isRerun: true
    });
    this.get('name') === 'wizardStep3Controller' ? this.getGeneralHostCheck() : this.getGeneralHostCheck(this.getDataForHostCheck());
    this.get('name') === 'wizardStep3Controller' && this.checkHostJDK();
    var interval = setInterval(function () {
      currentProgress += 100000 / self.get('warningsTimeInterval');
      if (currentProgress < 100) {
        self.set('checksUpdateProgress', currentProgress);
      } else {
        clearInterval(interval);
        App.ajax.send({
          name: 'wizard.step3.rerun_checks',
          sender: self,
          success: 'rerunChecksSuccessCallback',
          error: 'rerunChecksErrorCallback',
          callback: function callback() {
            self.set('isRerun', false);
            self.get('name') === 'mainHostDetailsController' && self.set('checkHostFinished', true);
          }
        });
      }
    }, 1000);
  },

  /**
   * Success-callback for rerun request
   * @param {object} data
   * @method rerunChecksSuccessCallback
   */
  rerunChecksSuccessCallback: function rerunChecksSuccessCallback(data) {
    this.set('checksUpdateProgress', 100);
    this.set('checksUpdateStatus', 'SUCCESS');
    this.parseWarnings(data);
  },

  /**
   * Error-callback for rerun request
   * @method rerunChecksErrorCallback
   */
  rerunChecksErrorCallback: function rerunChecksErrorCallback() {
    this.set('checksUpdateProgress', 100);
    this.set('checksUpdateStatus', 'FAILED');
  },

  /**
   * generates data for reuest to perform check
   * @param {string} checkExecuteList - for now supported:
   *  <code>"last_agent_env_check"<code>
   *  <code>"host_resolution_check"<code>
   * @param {boolean} addHostsParameter - define whether add hosts parameter to RequestInfo
   * @return {object|null}
   * @method getDataForCheckRequest
   */
  getDataForCheckRequest: function getDataForCheckRequest(checkExecuteList, addHostsParameter) {
    var newHosts = this.get('bootHosts').filterProperty('bootStatus', 'REGISTERED').getEach('name');
    var hosts = this.get('isAddHostWizard') ? [].concat.apply([], App.MasterComponent.find().mapProperty('hostNames')).concat(newHosts).uniq() : newHosts;
    hosts = hosts.join(',');
    if (hosts.length == 0) return null;
    var jdk_location = App.router.get('clusterController.ambariProperties.jdk_location');
    var RequestInfo = {
      "action": "check_host",
      "context": "Check host",
      "parameters": {
        "check_execute_list": checkExecuteList,
        "jdk_location": jdk_location,
        "threshold": "20"
      }
    };
    if (addHostsParameter) {
      RequestInfo.parameters.hosts = hosts;
    }
    var resource_filters = {
      "hosts": hosts
    };
    return {
      RequestInfo: RequestInfo,
      resource_filters: resource_filters
    };
  },

  /**
   * Show popup with regitration error-message
   * @param {string} header
   * @param {string} message
   * @return {App.ModalPopup}
   * @method registerErrPopup
   */
  registerErrPopup: function registerErrPopup(header, message) {
    return App.ModalPopup.show({
      header: header,
      secondary: false,
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile('<p>{{view.message}}</p>'),
        message: message
      })
    });
  },

  parseHostCheckWarnings: function parseHostCheckWarnings(data) {
    data = App.get('testMode') ? data : this.filterHostsData(data);
    var warnings = [];
    var warning;
    var hosts = [];
    var warningCategories = {
      fileFoldersWarnings: {},
      packagesWarnings: {},
      processesWarnings: {},
      servicesWarnings: {},
      usersWarnings: {},
      alternativeWarnings: {}
    };

    var hostsPackagesData = this.get('hostsPackagesData');
    data.tasks.sortPropertyLight('Tasks.host_name').forEach(function (_task) {
      var hostName = _task.Tasks.host_name;
      var host = {
        name: hostName,
        warnings: []
      };

      if (!_task.Tasks.structured_out || !_task.Tasks.structured_out.last_agent_env_check) {
        return;
      }

      var lastAgentEnvCheck = _task.Tasks.structured_out.last_agent_env_check;

      //parse all directories and files warnings for host
      var stackFoldersAndFiles = lastAgentEnvCheck.stackFoldersAndFiles || [];
      stackFoldersAndFiles.forEach(function (path) {
        warning = warningCategories.fileFoldersWarnings[path.name];
        if (warning) {
          warning.hosts.push(hostName);
          warning.hostsLong.push(hostName);
        } else {
          warningCategories.fileFoldersWarnings[path.name] = warning = {
            name: path.name,
            hosts: [hostName],
            hostsLong: [hostName],
            category: 'fileFolders'
          };
        }
        host.warnings.push(warning);
      }, this);

      //parse all package warnings for host
      var _hostPackagesData = hostsPackagesData.findProperty('hostName', hostName);

      if (_hostPackagesData) {
        _hostPackagesData.installedPackages.forEach(function (_package) {
          warning = warningCategories.packagesWarnings[_package.name];
          if (warning) {
            warning.hosts.push(hostName);
            warning.hostsLong.push(hostName);
            warning.version = _package.version;
          } else {
            warningCategories.packagesWarnings[_package.name] = warning = {
              name: _package.name,
              version: _package.version,
              hosts: [hostName],
              hostsLong: [hostName],
              category: 'packages'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse all process warnings for host
      var hostHealth = lastAgentEnvCheck.hostHealth;

      var liveServices = null;
      var javaProcs = null;

      if (hostHealth) {
        if (hostHealth.activeJavaProcs) javaProcs = hostHealth.activeJavaProcs;
        if (hostHealth.liveServices) liveServices = hostHealth.liveServices;
      }

      if (javaProcs) {
        javaProcs.forEach(function (process) {
          warning = warningCategories.processesWarnings[process.pid];
          if (warning) {
            warning.hosts.push(hostName);
            warning.hostsLong.push(hostName);
          } else {
            var command = 'pid=' + process.pid + ', user=' + process.user;
            warningCategories.processesWarnings[process.pid] = warning = {
              name: command.length > 36 ? command.substr(0, 35) + '...' : command,
              hosts: [hostName],
              hostsLong: [hostName],
              category: 'processes',
              user: process.user,
              pid: process.pid,
              command: command
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse all service warnings for host
      if (liveServices) {
        liveServices.forEach(function (service) {
          if (service.status === 'Unhealthy') {
            warning = warningCategories.servicesWarnings[service.name];
            if (warning) {
              warning.hosts.push(hostName);
              warning.hostsLong.push(hostName);
            } else {
              warningCategories.servicesWarnings[service.name] = warning = {
                name: service.name,
                hosts: [hostName],
                hostsLong: [hostName],
                category: 'services'
              };
            }
            host.warnings.push(warning);
          }
        }, this);
      }
      //parse all user warnings for host
      var existingUsers = lastAgentEnvCheck.existingUsers;
      if (existingUsers) {
        existingUsers.forEach(function (user) {
          warning = warningCategories.usersWarnings[user.name];
          if (warning) {
            warning.hosts.push(hostName);
            warning.hostsLong.push(hostName);
          } else {
            warningCategories.usersWarnings[user.name] = warning = {
              name: user.name,
              hosts: [hostName],
              hostsLong: [hostName],
              category: 'users'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse misc warnings for host
      var umask = lastAgentEnvCheck.umask;
      if (umask && umask > 23) {
        warning = warnings.filterProperty('category', 'misc').findProperty('name', umask);
        if (warning) {
          warning.hosts.push(hostName);
          warning.hostsLong.push(hostName);
        } else {
          warning = {
            name: umask,
            hosts: [hostName],
            hostsLong: [hostName],
            category: 'misc'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }

      var firewallRunning = lastAgentEnvCheck.firewallRunning;
      if (firewallRunning !== null && firewallRunning) {
        var name = lastAgentEnvCheck.firewallName + " Running";
        warning = warnings.filterProperty('category', 'firewall').findProperty('name', name);
        if (warning) {
          warning.hosts.push(hostName);
          warning.hostsLong.push(hostName);
        } else {
          warning = {
            name: name,
            hosts: [hostName],
            hostsLong: [hostName],
            category: 'firewall'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }

      if (lastAgentEnvCheck.alternatives) {
        lastAgentEnvCheck.alternatives.forEach(function (alternative) {
          warning = warningCategories.alternativeWarnings[alternative.name];
          if (warning) {
            warning.hosts.push(hostName);
            warning.hostsLong.push(hostName);
          } else {
            warningCategories.alternativeWarnings[alternative.name] = warning = {
              name: alternative.name,
              target: alternative.target,
              hosts: [hostName],
              hostsLong: [hostName],
              category: 'alternatives'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      if (lastAgentEnvCheck.reverseLookup === false) {
        var name = Em.I18n.t('installer.step3.hostWarningsPopup.reverseLookup.name');
        warning = warnings.filterProperty('category', 'reverseLookup').findProperty('name', name);
        if (warning) {
          warning.hosts.push(hostName);
          warning.hostsLong.push(hostName);
        } else {
          warning = {
            name: name,
            hosts: [hostName],
            hostsLong: [hostName],
            category: 'reverseLookup'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }
      hosts.push(host);
    }, this);

    for (var categoryId in warningCategories) {
      var category = warningCategories[categoryId];
      for (var warningId in category) {
        warnings.push(category[warningId]);
      }
    }

    hosts.unshift({
      name: 'All Hosts',
      warnings: warnings
    });
    this.set('warnings', warnings);
    this.set('warningsByHost', hosts);
  },

  /**
   * Parse warnings data for each host and total
   * @param {object} data
   * @method parseWarnings
   */
  parseWarnings: function parseWarnings(data) {
    data = App.get('testMode') ? data : this.filterBootHosts(data);
    var warnings = [];
    var warning;
    var hosts = [];
    var warningCategories = {
      fileFoldersWarnings: {},
      packagesWarnings: {},
      processesWarnings: {},
      servicesWarnings: {},
      usersWarnings: {},
      alternativeWarnings: {}
    };
    var hostsPackagesData = this.get('hostsPackagesData');

    data.items.sortPropertyLight('Hosts.host_name').forEach(function (_host) {
      var host = {
        name: _host.Hosts.host_name,
        warnings: []
      };
      if (!_host.Hosts.last_agent_env) {
        // in some unusual circumstances when last_agent_env is not available from the _host,
        // skip the _host and proceed to process the rest of the hosts.
        return;
      }

      //parse all directories and files warnings for host

      //todo: to be removed after check in new API
      var stackFoldersAndFiles = _host.Hosts.last_agent_env.stackFoldersAndFiles || [];
      stackFoldersAndFiles.forEach(function (path) {
        warning = warningCategories.fileFoldersWarnings[path.name];
        if (warning) {
          warning.hosts.push(_host.Hosts.host_name);
          warning.hostsLong.push(_host.Hosts.host_name);
        } else {
          warningCategories.fileFoldersWarnings[path.name] = warning = {
            name: path.name,
            hosts: [_host.Hosts.host_name],
            hostsLong: [_host.Hosts.host_name],
            category: 'fileFolders'
          };
        }
        host.warnings.push(warning);
      }, this);

      //parse all package warnings for host
      var _hostPackagesData = hostsPackagesData.findProperty('hostName', _host.Hosts.host_name);

      if (_hostPackagesData) {
        _hostPackagesData.installedPackages.forEach(function (_package) {
          warning = warningCategories.packagesWarnings[_package.name];
          if (warning) {
            warning.hosts.push(_host.Hosts.host_name);
            warning.hostsLong.push(_host.Hosts.host_name);
            warning.version = _package.version;
          } else {
            warningCategories.packagesWarnings[_package.name] = warning = {
              name: _package.name,
              version: _package.version,
              hosts: [_host.Hosts.host_name],
              hostsLong: [_host.Hosts.host_name],
              category: 'packages'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse all process warnings for host

      //todo: to be removed after check in new API
      var javaProcs = _host.Hosts.last_agent_env.hostHealth ? _host.Hosts.last_agent_env.hostHealth.activeJavaProcs : _host.Hosts.last_agent_env.javaProcs;
      if (javaProcs) {
        javaProcs.forEach(function (process) {
          warning = warningCategories.processesWarnings[process.pid];
          if (warning) {
            warning.hosts.push(_host.Hosts.host_name);
            warning.hostsLong.push(_host.Hosts.host_name);
          } else {
            var command = 'pid=' + process.pid + ', user=' + process.user;
            warningCategories.processesWarnings[process.pid] = warning = {
              name: command.length > 36 ? command.substr(0, 35) + '...' : command,
              hosts: [_host.Hosts.host_name],
              hostsLong: [_host.Hosts.host_name],
              category: 'processes',
              user: process.user,
              pid: process.pid,
              command: command
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse all service warnings for host

      //todo: to be removed after check in new API
      if (_host.Hosts.last_agent_env.hostHealth && _host.Hosts.last_agent_env.hostHealth.liveServices) {
        _host.Hosts.last_agent_env.hostHealth.liveServices.forEach(function (service) {
          if (service.status === 'Unhealthy') {
            warning = warningCategories.servicesWarnings[service.name];
            if (warning) {
              warning.hosts.push(_host.Hosts.host_name);
              warning.hostsLong.push(_host.Hosts.host_name);
            } else {
              warningCategories.servicesWarnings[service.name] = warning = {
                name: service.name,
                hosts: [_host.Hosts.host_name],
                hostsLong: [_host.Hosts.host_name],
                category: 'services'
              };
            }
            host.warnings.push(warning);
          }
        }, this);
      }
      //parse all user warnings for host

      //todo: to be removed after check in new API
      if (_host.Hosts.last_agent_env.existingUsers) {
        _host.Hosts.last_agent_env.existingUsers.forEach(function (user) {
          warning = warningCategories.usersWarnings[user.name];
          if (warning) {
            warning.hosts.push(_host.Hosts.host_name);
            warning.hostsLong.push(_host.Hosts.host_name);
          } else {
            warningCategories.usersWarnings[user.name] = warning = {
              name: user.name,
              hosts: [_host.Hosts.host_name],
              hostsLong: [_host.Hosts.host_name],
              category: 'users'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      //parse misc warnings for host
      var umask = _host.Hosts.last_agent_env.umask;
      if (umask && umask > 23) {
        warning = warnings.filterProperty('category', 'misc').findProperty('name', umask);
        if (warning) {
          warning.hosts.push(_host.Hosts.host_name);
          warning.hostsLong.push(_host.Hosts.host_name);
        } else {
          warning = {
            name: umask,
            hosts: [_host.Hosts.host_name],
            hostsLong: [_host.Hosts.host_name],
            category: 'misc'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }

      var firewallRunning = _host.Hosts.last_agent_env.firewallRunning;
      if (firewallRunning !== null && firewallRunning) {
        var name = _host.Hosts.last_agent_env.firewallName + " Running";
        warning = warnings.filterProperty('category', 'firewall').findProperty('name', name);
        if (warning) {
          warning.hosts.push(_host.Hosts.host_name);
          warning.hostsLong.push(_host.Hosts.host_name);
        } else {
          warning = {
            name: name,
            hosts: [_host.Hosts.host_name],
            hostsLong: [_host.Hosts.host_name],
            category: 'firewall'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }

      if (_host.Hosts.last_agent_env.alternatives) {
        _host.Hosts.last_agent_env.alternatives.forEach(function (alternative) {
          warning = warningCategories.alternativeWarnings[alternative.name];
          if (warning) {
            warning.hosts.push(_host.Hosts.host_name);
            warning.hostsLong.push(_host.Hosts.host_name);
          } else {
            warningCategories.alternativeWarnings[alternative.name] = warning = {
              name: alternative.name,
              target: alternative.target,
              hosts: [_host.Hosts.host_name],
              hostsLong: [_host.Hosts.host_name],
              category: 'alternatives'
            };
          }
          host.warnings.push(warning);
        }, this);
      }

      if (_host.Hosts.last_agent_env.reverseLookup === false) {
        var name = Em.I18n.t('installer.step3.hostWarningsPopup.reverseLookup.name');
        warning = warnings.filterProperty('category', 'reverseLookup').findProperty('name', name);
        if (warning) {
          warning.hosts.push(_host.Hosts.host_name);
          warning.hostsLong.push(_host.Hosts.host_name);
        } else {
          warning = {
            name: name,
            hosts: [_host.Hosts.host_name],
            hostsLong: [_host.Hosts.host_name],
            category: 'reverseLookup'
          };
          warnings.push(warning);
        }
        host.warnings.push(warning);
      }
      hosts.push(host);
    }, this);

    for (var categoryId in warningCategories) {
      var category = warningCategories[categoryId];
      for (var warningId in category) {
        warnings.push(category[warningId]);
      }
    }

    hosts.unshift({
      name: 'All Hosts',
      warnings: warnings
    });
    this.set('warnings', warnings);
    this.set('warningsByHost', hosts);
  },

  /**
   * Filter data for warnings parse
   * is data from host in bootStrap
   * @param {object} data
   * @return {Object}
   * @method filterBootHosts
   */
  filterBootHosts: function filterBootHosts(data) {
    var bootHostNames = {};
    this.get('bootHosts').forEach(function (bootHost) {
      bootHostNames[bootHost.get('name')] = true;
    });
    var filteredData = {
      href: data.href,
      items: []
    };
    data.items.forEach(function (host) {
      if (bootHostNames[host.Hosts.host_name]) {
        filteredData.items.push(host);
      }
    });
    return filteredData;
  },

  /**
   * Set metrics to host object
   * Used when <code>App.skipBootstrap</code> is true
   * @param {Ember.Object} host
   * @returns {object}
   * @private
   * @methos _setHostDataWithSkipBootstrap
   */
  _setHostDataWithSkipBootstrap: function _setHostDataWithSkipBootstrap(host) {
    host.set('cpu', 2);
    host.set('memory', parseInt(2000000).toFixed(2));
    host.set('disk_info', [{ "mountpoint": "/", "type": "ext4" }, { "mountpoint": "/grid/0", "type": "ext4" }, { "mountpoint": "/grid/1", "type": "ext4" }, { "mountpoint": "/grid/2", "type": "ext4" }]);
    return host;
  },

  /**
   * Set loaded metrics to host object
   * @param {object} host
   * @param {object} hostInfo
   * @returns {object}
   * @method _setHostDataFromLoadedHostInfo
   * @private
   */
  _setHostDataFromLoadedHostInfo: function _setHostDataFromLoadedHostInfo(host, hostInfo) {
    host.set('cpu', Em.get(hostInfo, 'Hosts.cpu_count'));
    host.set('memory', parseInt(Em.get(hostInfo, 'Hosts.total_mem')).toFixed(2));
    host.set('disk_info', Em.get(hostInfo, 'Hosts.disk_info').filter(function (h) {
      return h.mountpoint != "/boot";
    }));
    host.set('os_type', Em.get(hostInfo, 'Hosts.os_type'));
    host.set('os_family', Em.get(hostInfo, 'Hosts.os_family'));
    host.set('os_arch', Em.get(hostInfo, 'Hosts.os_arch'));
    host.set('ip', Em.get(hostInfo, 'Hosts.ip'));
    return host;
  },

  /**
   * Check if the customized os group contains the registered host os type. If not the repo on that host is invalid.
   * @param {string} osType
   * @param {string} hostName
   * @return {string} error-message or empty string
   * @method checkHostOSType
   */
  checkHostOSType: function checkHostOSType(osFamily, hostName) {
    if (this.get('content.stacks')) {
      var selectedStack = this.get('content.stacks').findProperty('isSelected', true);
      var selectedOS = [];
      var isValid = false;
      if (selectedStack && selectedStack.get('operatingSystems')) {
        selectedStack.get('operatingSystems').filterProperty('isSelected', true).forEach(function (os) {
          selectedOS.pushObject(os.get('osType'));
          if (os.get('osType') === osFamily) {
            isValid = true;
          }
        });
      }
      if (isValid) {
        return '';
      } else {
        return Em.I18n.t('installer.step3.hostWarningsPopup.repositories.context').format(hostName, osFamily, selectedOS.uniq());
      }
    } else {
      return '';
    }
  },

  /**
   * Check if current host has enough free disk usage.
   * @param {string} hostName
   * @param {object} diskInfo
   * @return {string} error-message or empty string
   * @method checkHostDiskSpace
   */
  checkHostDiskSpace: function checkHostDiskSpace(hostName, diskInfo) {
    var minFreeRootSpace = App.minDiskSpace * 1024 * 1024; //in kilobyte
    var minFreeUsrLibSpace = App.minDiskSpaceUsrLib * 1024 * 1024; //in kilobyte
    var warningString = '';

    diskInfo.forEach(function (info) {
      switch (info.mountpoint) {
        case '/':
          warningString = info.available < minFreeRootSpace ? Em.I18n.t('installer.step3.hostWarningsPopup.disk.context2').format(App.minDiskSpace + 'GB', info.mountpoint) + ' ' + warningString : warningString;
          break;
        case '/usr':
        case '/usr/lib':
          warningString = info.available < minFreeUsrLibSpace ? Em.I18n.t('installer.step3.hostWarningsPopup.disk.context2').format(App.minDiskSpaceUsrLib + 'GB', info.mountpoint) + ' ' + warningString : warningString;
          break;
        default:
          break;
      }
    });
    if (warningString) {
      return Em.I18n.t('installer.step3.hostWarningsPopup.disk.context1').format(hostName) + ' ' + warningString;
    } else {
      return '';
    }
  },

  /**
   * Check if the 'Transparent Huge Pages' enabled.
   * @param {string} transparentHugePage
   * @param {string} hostName
   * @return {string} error-message or empty string
   * @method checkTHP
   */
  checkTHP: function checkTHP(hostName, transparentHugePage) {
    if (transparentHugePage == "always") {
      return Em.I18n.t('installer.step3.hostWarningsPopup.thp.context').format(hostName);
    } else {
      return '';
    }
  },

  /**
   * Enable or disable submit/retry buttons according to hosts boot statuses
   * @method stopRegistration
   */
  stopRegistration: function stopRegistration() {
    this.set('isSubmitDisabled', !this.get('bootHosts').someProperty('bootStatus', 'REGISTERED'));
  },

  /**
   * Filter data for warnings parse
   * is data from host in bootStrap
   * @param {object} data
   * @return {Object}
   * @method filterBootHosts
   */
  filterHostsData: function filterHostsData(data) {
    var bootHostNames = {};
    this.get('bootHosts').forEach(function (bootHost) {
      bootHostNames[bootHost.get('name')] = true;
    });
    var filteredData = {
      href: data.href,
      tasks: []
    };
    data.tasks.forEach(function (_task) {
      if (bootHostNames[_task.Tasks.host_name]) {
        filteredData.tasks.push(_task);
      }
    });
    return filteredData;
  }
});

});

require.register("mixins/main/host/details/actions/install_new_version", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.InstallNewVersion = Em.Mixin.create({

  /**
   * show popup confirmation of version installation
   * @param event
   */
  installVersionConfirmation: function installVersionConfirmation(event) {
    var self = this;

    return App.showConfirmationPopup(function () {
      self.installVersion(event);
    }, Em.I18n.t('hosts.host.stackVersions.install.confirmation').format(event.context.get('displayName')));
  },

  /**
   * install HostStackVersion on host
   * @param {object} event
   */
  installVersion: function installVersion(event) {
    App.ajax.send({
      name: 'host.stack_versions.install',
      sender: this,
      data: {
        hostName: this.get('content.hostName'),
        version: event.context
      },
      success: 'installVersionSuccessCallback'
    });
  },

  /**
   * success callback of <code>installVersion</code>
   * on success set version status to INSTALLING
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   */
  installVersionSuccessCallback: function installVersionSuccessCallback(data, opt, params) {
    App.HostStackVersion.find(params.version.get('id')).set('status', 'INSTALLING');
    App.db.set('repoVersionInstall', 'id', [data.Requests.id]);
    App.clusterStatus.setClusterStatus({
      wizardControllerName: this.get('name'),
      localdb: App.db.data
    });
  },

  /**
   * Call callback after loading service metrics
   * @param callback
   */
  isServiceMetricsLoaded: function isServiceMetricsLoaded(callback) {
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(callback);
  }

});

});

require.register("mixins/main/host/details/host_components/decommissionable", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var uiEffects = require('utils/ui_effects');

/**
 * Mixin for <code>App.HostComponentView</code>
 * Contains code for processing components with allowed decommission
 * @type {Em.Mixin}
 */
App.Decommissionable = Em.Mixin.create({

  /**
   * Should be redeclared in views that use this mixin
   * @type {String}
   */
  componentForCheckDecommission: '',

  /**
   * Is component in decommission process right know
   * @type {boolean}
   */
  isComponentDecommissioning: false,

  /**
   * May component be decommissioned
   * @type {boolean}
   */
  isComponentDecommissionAvailable: false,

  /**
   * May component be recommissioned
   * @type {boolean}
   */
  isComponentRecommissionAvailable: false,

  /**
   * Timer id for decommission status polling
   * @type {number}
   */
  decommissionStatusPollingTimer: null,

  /**
   * Interval for decommission status polling
   * @type {number}
   */
  POLLING_INTERVAL: 6000,

  /**
   * Component with stopped masters can't be docommissioned
   * @type {bool}
   */
  isComponentDecommissionDisable: function () {
    var masterComponent = this.get('content.service.hostComponents').findProperty('componentName', this.get('componentForCheckDecommission'));
    if (masterComponent && masterComponent.get('workStatus') != App.HostComponentStatus.started) return true;
    return this.get('content.service.workStatus') != App.HostComponentStatus.started;
  }.property('content.service.workStatus', 'content.service.hostComponents.@each.workStatus'),

  /**
   * @override App.HostComponentView.isRestartableComponent
   */
  isRestartableComponent: function () {
    return this.get('isComponentDecommissionAvailable') && App.get('components.restartable').contains(this.get('content.componentName'));
  }.property('isComponentDecommissionAvailable'),

  /**
   * Tooltip message shows if decommission/recommission is disabled
   * when masters for current component is down
   * @type {string}
   */
  decommissionTooltipMessage: function () {
    if (this.get('isComponentDecommissionDisable') && (this.get('isComponentRecommissionAvailable') || this.get('isComponentDecommissionAvailable'))) {
      var decom = this.get('isComponentRecommissionAvailable') ? Em.I18n.t('common.recommission') : Em.I18n.t('common.decommission');
      return Em.I18n.t('hosts.decommission.tooltip.warning').format(decom, App.format.role(this.get('componentForCheckDecommission'), false));
    }
    return '';
  }.property('isComponentDecommissionDisable', 'isComponentRecommissionAvailable', 'isComponentDecommissionAvailable', 'componentForCheckDecommission'),
  /**
   * Recalculated component status based on decommission
   * @type {string}
   */
  statusClass: function () {

    //Class when install failed
    if (this.get('workStatus') === App.HostComponentStatus.install_failed) {
      return 'health-status-color-red glyphicon glyphicon-cog';
    }

    //Class when installing
    if (this.get('workStatus') === App.HostComponentStatus.installing) {
      return 'health-status-color-blue glyphicon glyphicon-cog';
    }

    if (this.get('isComponentRecommissionAvailable') && (this.get('isStart') || this.get('workStatus') == 'INSTALLED')) {
      return 'health-status-DEAD-ORANGE';
    }

    //For all other cases
    return 'health-status-' + App.HostComponentStatus.getKeyName(this.get('workStatus'));
  }.property('workStatus', 'isComponentRecommissionAvailable', 'isComponentDecommissioning'),

  /**
   * Return host component text status
   * @type {String}
   */
  componentTextStatus: function () {
    var componentTextStatus = this.get('content.componentTextStatus');
    if (this.get('isComponentRecommissionAvailable')) {
      if (this.get('isComponentDecommissioning')) {
        componentTextStatus = Em.I18n.t('hosts.host.decommissioning');
      } else {
        componentTextStatus = Em.I18n.t('hosts.host.decommissioned');
      }
    }
    return componentTextStatus;
  }.property('workStatus', 'isComponentRecommissionAvailable', 'isComponentDecommissioning'),

  /**
   * For Stopping or Starting states, also for decommissioning
   * @type {bool}
   */
  isInProgress: function () {
    return this.get('workStatus') === App.HostComponentStatus.stopping || this.get('workStatus') === App.HostComponentStatus.starting || this.get('isDecommissioning');
  }.property('workStatus', 'isDecommissioning'),

  /**
   * load Recommission/Decommission status of component
   */
  loadComponentDecommissionStatus: function loadComponentDecommissionStatus() {
    return this.getDesiredAdminState();
  },

  /**
   * Get desired_admin_state status from server
   */
  getDesiredAdminState: function getDesiredAdminState() {
    if (Em.isNone(this.get('content'))) return null;
    return App.ajax.send({
      name: 'host.host_component.slave_desired_admin_state',
      sender: this,
      data: {
        hostName: this.get('content.hostName'),
        componentName: this.get('content.componentName')
      },
      success: 'getDesiredAdminStateSuccessCallback',
      error: 'getDesiredAdminStateErrorCallback'
    });
  },

  /**
   * pass received value or null to <code>setDesiredAdminState</code>
   * @param {Object} response
   * @returns {String|null}
   */
  getDesiredAdminStateSuccessCallback: function getDesiredAdminStateSuccessCallback(response) {
    var status = response.HostRoles.desired_admin_state;
    if (!Em.isNone(status)) {
      this.setDesiredAdminState(status);
      return status;
    }
    return null;
  },

  /**
   * error callback of <code>getDesiredAdminState</code>
   */
  getDesiredAdminStateErrorCallback: Em.K,

  /**
   * compute decommission state by desiredAdminState
   * @param {Object} status
   */
  setDesiredAdminState: Em.K,

  /**
   * Get component decommission status from server
   * @returns {$.ajax}
   */
  getDecommissionStatus: function getDecommissionStatus() {
    return App.ajax.send({
      name: 'host.host_component.decommission_status',
      sender: this,
      data: {
        hostName: this.get('content.hostName'),
        componentName: this.get('componentForCheckDecommission'),
        serviceName: this.get('content.service.serviceName')
      },
      success: 'getDecommissionStatusSuccessCallback',
      error: 'getDecommissionStatusErrorCallback'
    });
  },

  /**
   * pass received value or null to <code>setDecommissionStatus</code>
   * @param {Object} response
   * @returns {Object|null}
   */
  getDecommissionStatusSuccessCallback: function getDecommissionStatusSuccessCallback(response) {
    var statusObject = response.ServiceComponentInfo;
    if (!Em.isNone(statusObject)) {
      statusObject.component_state = response.host_components[0].HostRoles.state;
      this.setDecommissionStatus(statusObject);
      return statusObject;
    }
    return null;
  },

  /**
   * Set null to <code>decommissionedStatusObject</code> if server returns error
   * @returns {null}
   */
  getDecommissionStatusErrorCallback: Em.K,

  /**
   * compute decommission state by component info
   * @param {Object} status
   */
  setDecommissionStatus: Em.K,

  /**
   * set decommission and recommission flags according to status
   * @param status
   */
  setStatusAs: function setStatusAs(status) {
    switch (status) {
      case "INSERVICE":
        this.set('isComponentRecommissionAvailable', false);
        this.set('isComponentDecommissioning', false);
        this.set('isComponentDecommissionAvailable', this.get('isStart'));
        break;
      case "DECOMMISSIONING":
        this.set('isComponentRecommissionAvailable', true);
        this.set('isComponentDecommissioning', true);
        this.set('isComponentDecommissionAvailable', false);
        break;
      case "DECOMMISSIONED":
        this.set('isComponentRecommissionAvailable', true);
        this.set('isComponentDecommissioning', false);
        this.set('isComponentDecommissionAvailable', false);
        break;
      case "RS_DECOMMISSIONED":
        this.set('isComponentRecommissionAvailable', true);
        this.set('isComponentDecommissioning', this.get('isStart'));
        this.set('isComponentDecommissionAvailable', false);
        break;
    }
  },

  /**
   * Do blinking for 1 minute
   */
  doBlinking: function doBlinking() {
    var workStatus = this.get('workStatus');
    var pulsate = [App.HostComponentStatus.starting, App.HostComponentStatus.stopping, App.HostComponentStatus.installing].contains(workStatus);

    if (!pulsate) {
      var component = this.get('content');
      if (component && workStatus !== "INSTALLED") {
        pulsate = this.get('isDecommissioning');
      }
    }
    if (pulsate && !this.get('isBlinking')) {
      this.set('isBlinking', true);
      this.pulsate();
    }
  },

  /**
   * perform the pulsation
   */
  pulsate: function pulsate() {
    var self = this;

    uiEffects.pulsate(self.$('.components-health'), 1000, function () {
      self.set('isBlinking', false);
      self.doBlinking();
    });
  },

  /**
   * Start blinking when host component is starting/stopping/decommissioning
   */
  startBlinking: function () {
    this.$('.components-health').stop(true, true);
    this.$('.components-health').css({ opacity: 1.0 });
    this.doBlinking();
  }.observes('workStatus', 'isComponentRecommissionAvailable', 'isDecommissioning'),

  didInsertElement: function didInsertElement() {
    var self = this;
    this._super();
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
      self.loadComponentDecommissionStatus();
    });
  },

  willDestroyElement: function willDestroyElement() {
    this._super();
    var timer = this.get('decommissionStatusPollingTimer');
    if (timer) {
      clearInterval(timer);
      this.set('decommissionStatusPollingTimer', null);
    }
  },

  /**
   * Update Decommission status only one time when component was changed
   */
  updateDecommissionStatus: function () {
    var self = this;
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
      Em.run.once(self, 'loadComponentDecommissionStatus');
    });
  }.observes('content.workStatus', 'content.passiveState'),

  /**
   * Update Decommission status periodically
   */
  decommissionStatusPolling: function decommissionStatusPolling() {
    var self = this;
    this.set('decommissionStatusPollingTimer', setTimeout(function () {
      self.loadComponentDecommissionStatus().done(function () {
        self.decommissionStatusPolling();
      });
    }, this.POLLING_INTERVAL));
  },

  /**
   * Start Decommission status polling if it is not started yet
   */
  startDecommissionStatusPolling: function startDecommissionStatusPolling() {
    if (!this.get('decommissionStatusPollingTimer')) {
      this.decommissionStatusPolling();
    }
  },

  decommissionView: Em.View.extend({
    classNameBindings: ['parentView.noActionAvailable'],
    tagName: 'li',
    templateName: require('templates/main/host/decommission'),

    text: Em.computed.ifThenElse('parentView.isComponentDecommissionAvailable', Em.I18n.t('common.decommission'), Em.I18n.t('common.recommission')),

    didInsertElement: function didInsertElement() {
      this._super();
      App.tooltip($("[rel='decommissionTooltip']"));
    },

    click: function click() {
      if (!this.get('parentView.isComponentDecommissionDisable')) {
        if (this.get('parentView.isComponentDecommissionAvailable')) {
          this.get('controller').decommission(this.get('parentView.content'), this.get('parentView.startDecommissionStatusPolling').bind(this.get('parentView')));
        } else {
          this.get('controller').recommission(this.get('parentView.content'), this.get('parentView.startDecommissionStatusPolling').bind(this.get('parentView')));
        }
      }
    }
  })
});

});

require.register("mixins/main/host/details/host_components/install_component", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.InstallComponent = Em.Mixin.create({

  installHostComponentCall: function installHostComponentCall(hostName, component) {
    var self = this,
        dfd = $.Deferred(),
        componentName = component.get('componentName'),
        displayName = component.get('displayName');
    this.updateAndCreateServiceComponent(componentName).done(function () {
      return App.ajax.send({
        name: 'host.host_component.add_new_component',
        sender: self,
        data: {
          hostName: hostName,
          component: component,
          data: JSON.stringify({
            RequestInfo: {
              "context": Em.I18n.t('requestInfo.installHostComponent') + " " + displayName
            },
            Body: {
              host_components: [{
                HostRoles: {
                  component_name: componentName
                }
              }]
            }
          })
        },
        success: 'addNewComponentSuccessCallback',
        error: 'ajaxErrorCallback'
      }).then(dfd.resolve, dfd.reject);
    });
    return dfd.promise();
  },

  /**
   * Success callback for add host component request
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @return {$.ajax}
   * @method addNewComponentSuccessCallback
   */
  addNewComponentSuccessCallback: function addNewComponentSuccessCallback(data, opt, params) {
    return App.ajax.send({
      name: 'common.host.host_component.update',
      sender: App.router.get('mainHostDetailsController'),
      data: {
        hostName: params.hostName,
        componentName: params.component.get('componentName'),
        serviceName: params.component.get('serviceName'),
        component: params.component,
        "context": Em.I18n.t('requestInfo.installNewHostComponent') + " " + params.component.get('displayName'),
        HostRoles: {
          state: 'INSTALLED'
        },
        urlParams: "HostRoles/state=INIT"
      },
      success: 'installNewComponentSuccessCallback',
      error: 'ajaxErrorCallback'
    });
  },

  /**
   * Default error-callback for ajax-requests in current page
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   * @method ajaxErrorCallback
   */
  ajaxErrorCallback: function ajaxErrorCallback(request, ajaxOptions, error, opt, params) {
    App.ajax.defaultErrorHandler(request, opt.url, opt.type);
  },

  /**
   *
   * @param componentName
   * @returns {*}
   */
  updateAndCreateServiceComponent: function updateAndCreateServiceComponent(componentName) {
    var self = this;
    var dfd = $.Deferred();
    var updater = App.router.get('updateController');
    updater.updateComponentsState(function () {
      updater.updateServiceMetric(function () {
        self.createServiceComponent(componentName, dfd);
      });
    });
    return dfd.promise();
  },

  /**
   *
   * @param {string} componentName
   * @param {$.Deferred} dfd
   * @returns {$.ajax|null}
   */
  createServiceComponent: function createServiceComponent(componentName, dfd) {
    var allServiceComponents = [];
    var services = App.Service.find().mapProperty('serviceName');
    services.forEach(function (_service) {
      var _serviceComponents = App.Service.find(_service).get('serviceComponents');
      allServiceComponents = allServiceComponents.concat(_serviceComponents);
    }, this);
    if (allServiceComponents.contains(componentName)) {
      dfd.resolve();
      return null;
    } else {
      return App.ajax.send({
        name: 'common.create_component',
        sender: this,
        data: {
          componentName: componentName,
          serviceName: App.StackServiceComponent.find().findProperty('componentName', componentName).get('serviceName')
        }
      }).complete(function () {
        dfd.resolve();
      });
    }
  }

});

});

require.register("mixins/main/host/details/support_client_configs_download", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.SupportClientConfigsDownload = Em.Mixin.create({

  /**
   * This object is supposed to be used as an enum  for resource types supported by ambari
   */
  resourceTypeEnum: Object.freeze({
    CLUSTER: "ClusterResource",
    HOST: "HostResource",
    SERVICE: "ServiceResource",
    SERVICE_COMPONENT: "ServiceComponentResource",
    HOST_COMPONENT: "HostComponentResource"
  }),

  /**
   *
   * @param {{hostName: string, componentName: string, displayName: string, serviceName: string, resourceType: resourceTypeEnum}} data
   */
  downloadClientConfigsCall: function downloadClientConfigsCall(data) {
    var url = this._getUrl(data.hostName, data.serviceName, data.componentName, data.resourceType);
    var newWindow = window.open(url);
    newWindow.focus();
  },

  /**
   *
   * @param {string|null} hostName
   * @param {string} serviceName
   * @param {string} componentName
   * @param {string} resourceType
   * @returns {string}
   * @private
   */
  _getUrl: function _getUrl(hostName, serviceName, componentName, resourceType) {
    var result;
    var prefix = App.get('apiPrefix') + '/clusters/' + App.router.getClusterName() + '/';

    switch (resourceType) {
      case this.resourceTypeEnum.SERVICE_COMPONENT:
        result = prefix + 'services/' + serviceName + '/components/' + componentName;
        break;
      case this.resourceTypeEnum.HOST_COMPONENT:
        result = prefix + 'hosts/' + hostName + '/host_components/' + componentName;
        break;
      case this.resourceTypeEnum.HOST:
        result = prefix + 'hosts/' + hostName + '/host_components';
        break;
      case this.resourceTypeEnum.SERVICE:
        result = prefix + 'services/' + serviceName + '/components';
        break;
      case this.resourceTypeEnum.CLUSTER:
      default:
        result = prefix + 'components';
    }

    result += '?format=client_config_tar';
    return result;
  }

});

});

require.register("mixins/main/service/configs/component_actions_by_configs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var stringUtils = require('utils/string_utils');

/**
 * Mixin with methods for component actions that needs to be done when a config with specific value is saved
 * Used in the service config controller
 * @type {Em.Mixin}
 */
App.ComponentActionsByConfigs = Em.Mixin.create({

  configAction: null,

  /**
   * Flag to check if Yarn Queues have been refreshed already
   */
  isYarnQueueRefreshed: false,

  /**
   * Do component add/delete actions as inferred from value of service configs
   * @public
   * @method doConfigActions
   */
  doConfigActions: function doConfigActions() {
    var serviceConfigs = this.get('stepConfigs').findProperty('serviceName', this.get('content.serviceName')).get('configs');
    var configActionComponents = serviceConfigs.filterProperty('configActionComponent');
    this.isYarnQueueRefreshed = false;
    this.doComponentDeleteActions(configActionComponents);
    this.doComponentAddActions(configActionComponents);
    this.showPopup();
  },

  /**
   * Method to show confirmation popup before sending an ajax request
   */
  showPopup: function showPopup() {
    var config_actions = App.ConfigAction.find().filterProperty('actionType', 'showPopup');
    var self = this;
    if (config_actions.length) {
      config_actions.forEach(function (config_action) {
        var configs = self.get('allConfigs').filterProperty('filename', config_action.get('fileName')).filter(function (item) {
          return item.get('value') !== item.get('initialValue');
        });

        if (configs.length) {
          var hostComponents = App.HostComponent.find();
          if (config_action.get('fileName') === 'capacity-scheduler.xml' && !self.isYarnQueueRefreshed) {
            var isRMRunning = hostComponents.some(function (component) {
              return component.get('componentName') === 'RESOURCEMANAGER' && component.get('isRunning');
            });
            if (isRMRunning) {
              var hsiInstance = hostComponents.filterProperty('componentName', 'HIVE_SERVER_INTERACTIVE');
              if (self.get('content.serviceName') === 'HIVE') {
                // Auto refresh yarn capacity scheduler if capacity-scheduler configs are changed from Hive configs page
                self.popupPrimaryButtonCallback(config_action);
                // Show a popup to restart HSI if HSI is enabled
                if (hsiInstance.length > 0) {
                  self.showHsiRestartPopup(hsiInstance);
                }
              } else {
                self.configAction = config_action;
                var body = config_action.get('popupProperties').body;
                if (config_action.get('popupProperties').hasOwnProperty('conditionalWarning') && config_action.get('popupProperties').conditionalWarning === true) {
                  // Check if Hive Server 2 Interactive is enabled and show a warning message if it is enabled
                  if (hsiInstance.length > 0) {
                    body += "<br/><br/>" + config_action.get('popupProperties').warningMessage;
                  }
                }
                App.showConfirmationPopup(function () {
                  self.popupPrimaryButtonCallback(config_action);
                }, body, null, Em.I18n.t('popup.confirmation.commonHeader'), config_action.get('popupProperties').primaryButton.label, 'success', 'refresh_yarn_queues');
              }
            }
          }
        }
      });
    }
  },

  showHsiRestartPopup: function showHsiRestartPopup(components) {
    var self = this;
    App.showConfirmationPopup(function () {
      self.hsiRestartPopupPrimaryButtonCallback(components);
    }, Em.I18n.t('popup.confirmation.hsiRestart.body'), null, Em.I18n.t('popup.confirmation.commonHeader'), Em.I18n.t('popup.confirmation.hsiRestart.buttonText'), 'success', 'restart_hsi');
  },

  hsiRestartPopupPrimaryButtonCallback: function hsiRestartPopupPrimaryButtonCallback(components) {
    batchUtils.restartHostComponents(components, Em.I18n.t('rollingrestart.context.selectedComponentOnSelectedHost').format(components[0].get('displayName')), "HOST_COMPONENT");
  },

  popupPrimaryButtonCallback: function popupPrimaryButtonCallback(config_action) {
    var hosts = App.Service.find(config_action.get('serviceName')).get('hostComponents').filterProperty('componentName', config_action.get('componentName')).mapProperty('hostName');
    var self = this;
    App.ajax.send({
      name: config_action.get('popupProperties').primaryButton.metaData.name,
      sender: self,
      data: {
        command: config_action.get('popupProperties').primaryButton.metaData.command,
        context: config_action.get('popupProperties').primaryButton.metaData.context,
        hosts: hosts.join(','),
        serviceName: config_action.get('serviceName'),
        componentName: config_action.get('componentName'),
        forceRefreshConfigTags: config_action.get('configName')
      },
      error: 'popupPrimaryButtonCallbackError'
    });
  },

  popupPrimaryButtonCallbackError: function popupPrimaryButtonCallbackError(data) {
    var error = this.configAction.get('popupProperties').errorMessage;
    if (data && data.responseText) {
      try {
        var json = $.parseJSON(data.responseText);
        error += json.message;
      } catch (err) {}
    }
    App.showAlertPopup(this.configAction.get('popupProperties').errorMessage, error, null);
  },

  /**
   * Method informs if any component will be added/deleted on saving configurations
   * @return {boolean}
   * @public
   * @method isComponentActionsPresent
   */
  isComponentActionsPresent: function isComponentActionsPresent() {
    var serviceConfigs = this.get('stepConfigs').findProperty('serviceName', this.get('content.serviceName')).get('configs');
    var configActionComponents = serviceConfigs.filterProperty('configActionComponent');
    return !!(this.getComponentsToDelete(configActionComponents).length + this.getComponentsToAdd(configActionComponents).length);
  },

  /**
   * Get Component that will be deleted on saving configurations
   * @param configActionComponents {Object}
   * @return {Array}
   * @private
   * @method getComponentsToDelete
   */
  getComponentsToDelete: function getComponentsToDelete(configActionComponents) {
    var hostComponents = App.HostComponent.find();
    var componentsToDelete = [];

    configActionComponents.filterProperty('configActionComponent.action', 'delete').map(function (item) {
      return item.configActionComponent;
    }).forEach(function (_componentToDelete) {
      var installedHosts = hostComponents.filterProperty('componentName', _componentToDelete.componentName).mapProperty('hostName');
      _componentToDelete.hostNames.forEach(function (hostsToDelete) {
        if (installedHosts.contains(hostsToDelete)) {
          componentsToDelete.push({
            componentName: _componentToDelete.componentName,
            hostName: hostsToDelete,
            isClient: _componentToDelete.isClient
          });
        }
      });
    }, this);
    return componentsToDelete;
  },

  /**
   * Get Component that will be added on saving configurations
   * @param configActionComponents {Object}
   * @return {array}
   * @private
   * @method getComponentsToDelete
   */
  getComponentsToAdd: function getComponentsToAdd(configActionComponents) {
    var componentsToAdd = [];

    configActionComponents.filterProperty('configActionComponent.action', 'add').map(function (item) {
      return item.configActionComponent;
    }).forEach(function (_componentToAdd) {
      var serviceNameForComponent = App.StackServiceComponent.find(_componentToAdd.componentName).get('serviceName');
      var hostsToInstall = _componentToAdd.hostNames;

      App.Service.find(serviceNameForComponent).get('hostComponents').filterProperty('componentName', _componentToAdd.componentName).mapProperty('hostName').forEach(function (installedHost) {
        // List of host components to be added should not include ones that are already present in the cluster.
        hostsToInstall = _componentToAdd.hostNames.without(installedHost);
      });
      hostsToInstall.forEach(function (hostToInstall) {
        componentsToAdd.push({
          componentName: _componentToAdd.componentName,
          hostName: hostToInstall,
          isClient: _componentToAdd.isClient
        });
      });
    }, this);
    return componentsToAdd;
  },

  /**
   * Do component Delete actions as inferred from value of service config
   * @param configActionComponents Object[]
   * @private
   * @method {configActionComponents}
   */
  doComponentDeleteActions: function doComponentDeleteActions(configActionComponents) {
    var _this = this;

    var componentsToDelete = this.getComponentsToDelete(configActionComponents);
    if (componentsToDelete.length) {
      componentsToDelete.forEach(function (componentToDelete) {
        var componentName = componentToDelete.componentName;
        var hostName = componentToDelete.hostName;
        var displayName = App.StackServiceComponent.find(componentName).get('displayName');
        var context = Em.I18n.t('requestInfo.stop').format(displayName);
        var batches = [];

        _this.setRefreshYarnQueueRequest(batches);
        batches.push(_this.getInstallHostComponentsRequest(hostName, componentName, context));
        batches.push(_this.getDeleteHostComponentRequest(hostName, componentName));
        _this.setOrderIdForBatches(batches);
        App.ajax.send({
          name: 'common.batch.request_schedules',
          sender: {
            checkIfComponentWasDeleted: _this.checkIfComponentWasDeleted
          },
          data: {
            intervalTimeSeconds: 60,
            tolerateSize: 0,
            batches: batches,
            displayName: displayName,
            hostName: hostName
          },
          success: 'checkIfComponentWasDeleted'
        });
      });
    }
  },

  checkIfComponentWasDeleted: function checkIfComponentWasDeleted(resp, req, data) {
    var scheduleId = resp.resources[0].RequestSchedule.id;
    var self = this;
    setTimeout(function () {
      batchUtils.getRequestSchedule(scheduleId, function (resp) {
        var lastStatus = resp.RequestSchedule.last_execution_status;
        var status = resp.RequestSchedule.status;
        if (lastStatus === 'FAILED' || status === 'FAILED') {
          App.showAlertPopup(Em.I18n.t('hosts.bulkOperation.delete.component.failed.header'), Em.I18n.t('hosts.bulkOperation.delete.component.failed.body').format(data.displayName, data.hostName));
        }
      }, function () {});
    }, data.batches.length * 60000);
  },

  /**
   * Do component Add actions as inferred from value of service config
   * @param configActionComponents Object[]
   * @private
   * @method {doComponentAddActions}
   */
  doComponentAddActions: function doComponentAddActions(configActionComponents) {
    var componentsToAdd = this.getComponentsToAdd(configActionComponents);

    if (componentsToAdd.length) {
      var allComponentsToAdd = componentsToAdd.concat(this.getDependentComponents(componentsToAdd));
      var allComponentsToAddHosts = allComponentsToAdd.mapProperty('hostName').uniq();

      allComponentsToAddHosts.forEach(function (_hostName) {
        var hostComponents = allComponentsToAdd.filterProperty('hostName', _hostName).mapProperty('componentName').uniq();
        var masterHostComponents = allComponentsToAdd.filterProperty('hostName', _hostName).filterProperty('isClient', false).mapProperty('componentName').uniq();
        var displayNames = masterHostComponents.map(function (item) {
          return App.StackServiceComponent.find().findProperty('componentName', item).get('displayName');
        });

        var displayStr = stringUtils.getFormattedStringFromArray(displayNames);
        var context = Em.I18n.t('requestInfo.start').format(displayStr);
        var batches = [];
        this.setCreateComponentRequest(batches, hostComponents);
        batches.push(this.getCreateHostComponentsRequest(_hostName, hostComponents));
        batches.push(this.getInstallHostComponentsRequest(_hostName, hostComponents));
        this.setRefreshYarnQueueRequest(batches);
        batches.push(this.getStartHostComponentsRequest(_hostName, masterHostComponents, context));
        this.setOrderIdForBatches(batches);

        App.ajax.send({
          name: 'common.batch.request_schedules',
          sender: this,
          data: {
            intervalTimeSeconds: 1,
            tolerateSize: 0,
            batches: batches
          }
        });
      }, this);
    }
  },

  /**
   * @method getDependentComponents
   * @param {Array} componentsToAdd
   * @returns {Array}
   */
  getDependentComponents: function getDependentComponents(componentsToAdd) {
    var dependentComponents = [];

    componentsToAdd.forEach(function (_component) {
      var componentToAdd = App.StackServiceComponent.find(_component.componentName);
      var installedComponents = App.HostComponent.find().filterProperty('hostName', _component.hostName).mapProperty('componentName').uniq();
      var dependencies = componentToAdd.missingDependencies(installedComponents, { 'scope': 'host' }).map(function (_dependency) {
        return {
          componentName: _dependency.chooseCompatible(),
          hostName: _component.hostName,
          isClient: App.StackServiceComponent.find(_dependency.componentName).get('isClient')
        };
      }, this);
      var dependenciesToInstall = dependencies.filter(function (_dependencyToAdd) {
        var isInstalled = App.HostComponent.find().filterProperty('componentName', _dependencyToAdd.componentName).someProperty('hostName', _dependencyToAdd.hostName);
        var isAddedToInstall = dependentComponents.filterProperty('componentName', _dependencyToAdd.componentName).someProperty('hostName', _dependencyToAdd.hostName);
        return !(isInstalled || isAddedToInstall);
      }, this);
      dependentComponents = dependentComponents.concat(dependenciesToInstall);
    }, this);
    return dependentComponents;
  },

  /**
   * Sets order_id for each batch request in the `batches` array
   * @param batches {Array}
   * @private
   * @method {setOrderIdForBatches}
   */
  setOrderIdForBatches: function setOrderIdForBatches(batches) {
    batches.forEach(function (_batch, index) {
      _batch.order_id = index + 1;
    }, this);
  },

  /**
   * Gets the API request to create multiple components on a host
   * @param hostName {String}
   * @param components {String[]}|{String}
   * @return {Object} Deferred promise
   */
  getCreateHostComponentsRequest: function getCreateHostComponentsRequest(hostName, components) {
    var query = "Hosts/host_name.in(" + hostName + ")";
    components = Array.isArray(components) ? components : [components];
    var hostComponent = components.map(function (_componentName) {
      return {
        "HostRoles": {
          "component_name": _componentName
        }
      };
    }, this);

    return {
      "type": 'POST',
      "uri": "/clusters/" + App.get('clusterName') + "/hosts",
      "RequestBodyInfo": {
        "RequestInfo": {
          "query": query
        },
        "Body": {
          "host_components": hostComponent
        }
      }
    };
  },

  /**
   * Gets the API request to install multiple components on a host
   * @param hostName {String}
   * @param components {String[]}
   * @param context {String} Optional
   * @return {Object}
   */
  getInstallHostComponentsRequest: function getInstallHostComponentsRequest(hostName, components, context) {
    context = context || Em.I18n.t('requestInfo.installComponents');
    return this.getUpdateHostComponentsRequest(hostName, components, App.HostComponentStatus.stopped, context);
  },

  /**
   * Gets the API request to start multiple components on a host
   * @param hostName {String}
   * @param components {String[]}
   * @param context {String} Optional
   * @return {Object}
   */
  getStartHostComponentsRequest: function getStartHostComponentsRequest(hostName, components, context) {
    context = context || Em.I18n.t('requestInfo.startHostComponents');
    return this.getUpdateHostComponentsRequest(hostName, components, App.HostComponentStatus.started, context);
  },

  /**
   * Gets the API request to start/stop multiple components on a host
   * @param hostName {String}
   * @param components
   * @param desiredState
   * @param context {String}
   * @private
   * @method {getUpdateHostComponentsRequest}
   * @return {Object}
   */
  getUpdateHostComponentsRequest: function getUpdateHostComponentsRequest(hostName, components, desiredState, context) {
    components = Array.isArray(components) ? components : [components];
    var query = "HostRoles/component_name.in(" + components.join(',') + ")";

    return {
      "type": 'PUT',
      "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hostName + "/host_components",
      "RequestBodyInfo": {
        "RequestInfo": {
          "context": context,
          "operation_level": {
            "level": "HOST",
            "cluster_name": App.get('clusterName'),
            "host_names": hostName
          },
          "query": query
        },
        "Body": {
          "HostRoles": {
            "state": desiredState
          }
        }
      }
    };
  },

  /**
   * Gets the API request to delete component on a host
   * @param hostName {String}
   * @param component {String}
   * @private
   * @method {getDeleteHostComponentRequest}
   * @return {Object}
   */
  getDeleteHostComponentRequest: function getDeleteHostComponentRequest(hostName, component) {
    return {
      "type": 'DELETE',
      "uri": "/clusters/" + App.get('clusterName') + "/hosts/" + hostName + "/host_components/" + component
    };
  },

  /**
   * Add `Create component` as a request in the batched API call
   * @param batches  {Array}
   * @param hostComponents {Array}
   * @private
   * @method {setCreateComponentRequest}
   */
  setCreateComponentRequest: function setCreateComponentRequest(batches, hostComponents) {
    var stackServices = App.StackServiceComponent.find(),
        services = App.Service.find();

    hostComponents.forEach(function (_componentName) {
      var serviceName = stackServices.findProperty('componentName', _componentName).get('serviceName');
      var serviceComponents = services.findProperty('serviceName', serviceName).get('serviceComponents');
      if (!serviceComponents.contains(_componentName)) {
        batches.push({
          "type": 'POST',
          "uri": "/clusters/" + App.get('clusterName') + "/services/" + serviceName + "/components/" + _componentName
        });
      }
    });
  },

  /**
   * Add `Refresh YARN Queue` as a request in the batched API call
   * @param batches  {Array}
   * @private
   * @method {setRefreshYarnQueueRequest}
   */
  setRefreshYarnQueueRequest: function setRefreshYarnQueueRequest(batches) {
    var capacitySchedulerConfigs = this.get('allConfigs').filterProperty('filename', 'capacity-scheduler.xml').filter(function (item) {
      return item.get('value') !== item.get('initialValue');
    });

    if (capacitySchedulerConfigs.length) {
      var serviceName = 'YARN';
      var componentName = 'RESOURCEMANAGER';
      var hostNames = App.Service.find(serviceName).get('hostComponents').filterProperty('componentName', componentName).mapProperty('hostName');
      // Set the flag to true
      this.isYarnQueueRefreshed = true;
      batches.push({
        "type": 'POST',
        "uri": "/clusters/" + App.get('clusterName') + "/requests",
        "RequestBodyInfo": {
          "RequestInfo": {
            "context": Em.I18n.t('services.service.actions.run.yarnRefreshQueues.context'),
            "command": "REFRESHQUEUES",
            "parameters/forceRefreshConfigTags": "capacity-scheduler"
          },
          "Requests/resource_filters": [{
            service_name: serviceName,
            component_name: componentName,
            hosts: hostNames.join(',')
          }]
        }
      });
    }
  }
});

});

require.register("mixins/main/service/configs/config_overridable", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

/**
 * Mixin with methods for config groups and overrides processing
 * Used in the installer step7, service configs page and others
 * @type {Em.Mixin}
 */
App.ConfigOverridable = Em.Mixin.create({

  /**
   *
   * @method createOverrideProperty
   */
  createOverrideProperty: function createOverrideProperty(event) {
    var serviceConfigProperty = event.contexts[0];
    var serviceConfigController = this.get('isView') ? this.get('controller') : this;
    var selectedConfigGroup = serviceConfigController.get('selectedConfigGroup');
    var isInstaller = this.get('controller.name') === 'wizardStep7Controller';
    var configGroups = isInstaller ? serviceConfigController.get('selectedService.configGroups') : serviceConfigController.get('configGroups');

    //user property is added, and it has not been saved, not allow override
    if (serviceConfigProperty.get('isUserProperty') && serviceConfigProperty.get('isNotSaved') && !isInstaller) {
      App.ModalPopup.show({
        header: Em.I18n.t('services.service.config.configOverride.head'),
        body: Em.I18n.t('services.service.config.configOverride.body'),
        secondary: false
      });
      return;
    }
    if (selectedConfigGroup.get('isDefault')) {
      // Launch dialog to pick/create Config-group
      this.launchConfigGroupSelectionCreationDialog(this.get('service.serviceName'), configGroups, serviceConfigProperty, function (selectedGroupInPopup) {
        if (selectedGroupInPopup) {
          serviceConfigController.set('overrideToAdd', serviceConfigProperty);
          serviceConfigController.set('selectedConfigGroup', selectedGroupInPopup);
        }
      }, isInstaller);
    } else {
      var valueForOverride = serviceConfigProperty.get('widget') || serviceConfigProperty.get('displayType') == 'checkbox' ? serviceConfigProperty.get('value') : '';
      App.config.createOverride(serviceConfigProperty, {
        "value": valueForOverride,
        "isEditable": true
      }, selectedConfigGroup);
    }
    Em.$('body>.tooltip').remove();
  },

  /**
   * Open popup with list of config groups
   * User may select existing group or create a new one
   * @param {string} serviceId service name like 'HDFS', 'HBASE' etc
   * @param {App.ConfigGroup[]} configGroups
   * @param {App.ConfigProperty} configProperty
   * @param {Function} callback function called after config group is selected (or new one is created)
   * @param {Boolean} isInstaller determines if user is currently on the installer
   * @return {App.ModalPopup}
   * @method launchConfigGroupSelectionCreationDialog
   */
  launchConfigGroupSelectionCreationDialog: function launchConfigGroupSelectionCreationDialog(serviceId, configGroups, configProperty, callback, isInstaller) {
    var self = this;
    var availableConfigGroups = configGroups.slice();
    // delete Config Groups, that already have selected property overridden
    var alreadyOverriddenGroups = [];
    if (configProperty.get('overrides')) {
      alreadyOverriddenGroups = configProperty.get('overrides').mapProperty('group.name');
    }
    var result = [];
    availableConfigGroups.forEach(function (group) {
      if (!group.get('isDefault') && (!alreadyOverriddenGroups.length || !alreadyOverriddenGroups.contains(Em.get(group, 'name')))) {
        result.push(group);
      }
    }, this);
    availableConfigGroups = result;
    var selectedConfigGroup = availableConfigGroups && availableConfigGroups.length > 0 ? availableConfigGroups[0] : null;
    var serviceName = App.format.role(serviceId, true);

    return App.ModalPopup.show({
      classNames: ['common-modal-wrapper'],
      modalDialogClasses: ['modal-lg'],
      header: Em.I18n.t('config.group.selection.dialog.title').format(serviceName),
      subTitle: Em.I18n.t('config.group.selection.dialog.subtitle').format(serviceName),
      selectExistingGroupLabel: Em.I18n.t('config.group.selection.dialog.option.select').format(serviceName),
      noGroups: Em.I18n.t('config.group.selection.dialog.no.groups').format(serviceName),
      createNewGroupLabel: Em.I18n.t('config.group.selection.dialog.option.create').format(serviceName),
      createNewGroupDescription: Em.I18n.t('config.group.selection.dialog.option.create.msg').format(serviceName),
      warningMessage: '&nbsp;',
      isWarning: false,
      optionSelectConfigGroup: true,
      optionCreateConfigGroup: Em.computed.not('optionSelectConfigGroup'),
      hasExistedGroups: Em.computed.bool('availableConfigGroups.length'),
      availableConfigGroups: availableConfigGroups,
      selectedConfigGroup: selectedConfigGroup,
      newConfigGroupName: '',
      disablePrimary: function () {
        return !(this.get('optionSelectConfigGroup') || this.get('newConfigGroupName').trim().length > 0 && !this.get('isWarning'));
      }.property('newConfigGroupName', 'optionSelectConfigGroup', 'warningMessage'),
      onPrimary: function onPrimary() {
        var popup = this;
        if (this.get('optionSelectConfigGroup')) {
          var selectedConfigGroup = this.get('selectedConfigGroup');
          popup.hide();
          callback(selectedConfigGroup);
          if (!isInstaller) {
            App.get('router.mainServiceInfoConfigsController').doSelectConfigGroup({ context: selectedConfigGroup });
          }
        } else {
          var newConfigGroupName = this.get('newConfigGroupName').trim();
          var newConfigGroup = {
            id: new Date().getTime(),
            name: newConfigGroupName,
            is_default: false,
            parent_config_group_id: App.ServiceConfigGroup.getParentConfigGroupId(serviceId),
            description: Em.I18n.t('config.group.description.default').format(new Date().toDateString()),
            service_id: serviceId,
            service_name: serviceId,
            hosts: [],
            desired_configs: [],
            properties: []
          };
          if (!isInstaller) {
            self.postNewConfigurationGroup(newConfigGroup, function () {
              newConfigGroup = App.ServiceConfigGroup.find().filterProperty('serviceName', serviceId).findProperty('name', newConfigGroupName);
              self.saveGroupConfirmationPopup(newConfigGroupName);
              callback(newConfigGroup);
              popup.hide();
            });
          } else {
            newConfigGroup.is_temporary = true;
            App.store.safeLoad(App.ServiceConfigGroup, newConfigGroup);
            App.store.fastCommit();
            newConfigGroup = App.ServiceConfigGroup.find(newConfigGroup.id);
            configGroups.pushObject(newConfigGroup);
            self.persistConfigGroups();
            callback(newConfigGroup);
            popup.hide();
          }
        }
      },
      onSecondary: function onSecondary() {
        this.hide();
        callback(null);
      },
      doSelectConfigGroup: function doSelectConfigGroup(event) {
        var configGroup = event.context;
        this.set('selectedConfigGroup', configGroup);
      },
      validate: function () {
        var msg = '&nbsp;';
        var isWarning = false;
        var optionSelect = this.get('optionSelectConfigGroup');
        if (!optionSelect) {
          var nn = this.get('newConfigGroupName').trim();
          if (nn) {
            if (!validator.isValidConfigGroupName(nn)) {
              msg = Em.I18n.t("form.validator.configGroupName");
              isWarning = true;
            } else if (configGroups.mapProperty('name').contains(nn)) {
              msg = Em.I18n.t("config.group.selection.dialog.err.name.exists");
              isWarning = true;
            }
          }
        }
        this.set('warningMessage', msg);
        this.set('isWarning', isWarning);
      }.observes('newConfigGroupName', 'optionSelectConfigGroup'),
      bodyClass: Em.View.extend({
        templateName: require('templates/common/configs/selectCreateConfigGroup'),
        controllerBinding: 'App.router.mainServiceInfoConfigsController',
        selectConfigGroupRadioButton: App.RadioButtonView.extend({
          label: Em.computed.alias('parentView.parentView.selectExistingGroupLabel'),
          checked: Em.computed.alias('parentView.parentView.optionSelectConfigGroup'),
          disabled: Em.computed.not('parentView.parentView.hasExistedGroups'),
          click: function click() {
            if (this.get('disabled')) {
              return;
            }
            this.set('parentView.parentView.optionSelectConfigGroup', true);
          },
          didInsertElement: function didInsertElement() {
            if (!this.get('parentView.parentView.hasExistedGroups')) {
              this.set('parentView.parentView.optionSelectConfigGroup', false);
            }
          }
        }),
        createConfigGroupRadioButton: App.RadioButtonView.extend({
          label: Em.computed.alias('parentView.parentView.createNewGroupLabel'),
          checked: Em.computed.not('parentView.parentView.optionSelectConfigGroup'),
          click: function click() {
            this.set('parentView.parentView.optionSelectConfigGroup', false);
          }
        })
      })
    });
  },

  /**
   * Create a new config-group for a service.
   *
   * @param {object} newConfigGroupData config group to post to server
   * @param {Function} [callback] Callback function for Success or Error handling
   * @return {$.ajax}
   * @method postNewConfigurationGroup
   */
  postNewConfigurationGroup: function postNewConfigurationGroup(newConfigGroupData, callback) {
    var typeToPropertiesMap = {};
    newConfigGroupData.properties.forEach(function (property) {
      if (!typeToPropertiesMap[property.get('type')]) {
        typeToPropertiesMap[property.get('type')] = {};
      }
      typeToPropertiesMap[property.get('type')][property.get('name')] = property.get('value');
    });
    var newGroupData = {
      "ConfigGroup": {
        "group_name": newConfigGroupData.name,
        "tag": newConfigGroupData.service_id,
        "description": newConfigGroupData.description,
        "service_name": newConfigGroupData.service_id,
        "desired_configs": newConfigGroupData.desired_configs.map(function (cst) {
          var type = Em.get(cst, 'site') || Em.get(cst, 'type');
          return {
            type: type,
            properties: typeToPropertiesMap[type]
          };
        }),
        "hosts": newConfigGroupData.hosts.map(function (h) {
          return {
            host_name: h
          };
        })
      }
    };
    return App.ajax.send({
      name: 'config_groups.create',
      sender: this,
      data: {
        data: [newGroupData],
        modelData: newConfigGroupData
      },
      success: 'postNewConfigurationGroupSuccess',
      error: 'postNewConfigurationGroupError'
    }).always(function (xhr, text, errorThrown) {
      if (callback) {
        callback(xhr, text, errorThrown);
      }
    });
  },

  /**
   *
   * @param {string} response
   * @param {object} opt
   * @param {object} params
   */
  postNewConfigurationGroupSuccess: function postNewConfigurationGroupSuccess(response, opt, params) {
    var modelData = params.modelData;
    modelData.id = response.resources[0].ConfigGroup.id;
    App.store.safeLoad(App.ServiceConfigGroup, modelData);
    App.store.fastCommit();
    App.ServiceConfigGroup.deleteTemporaryRecords();
  },

  /**
   *
   * @param {object} xhr
   * @param {string} text
   * @param {Error} errorThrown
   */
  postNewConfigurationGroupError: Em.K,

  /**
   * PUTs the new configuration-group on the server.
   * Changes possible here are the name, description and
   * host memberships of the configuration-group.
   *
   * @param {App.ConfigGroup} configGroup Configuration group to update
   * @param {Function} successCallback
   * @param {Function} errorCallback
   * @return {$.ajax}
   * @method updateConfigurationGroup
   */
  updateConfigurationGroup: function updateConfigurationGroup(configGroup, successCallback, errorCallback) {
    var sendData = {
      name: 'config_groups.update',
      data: {
        id: configGroup.get('id'),
        data: this.getConfigGroupData(configGroup)
      },
      success: 'successFunction',
      error: 'errorFunction',
      successFunction: function successFunction() {
        if (successCallback) {
          successCallback();
        }
      },
      errorFunction: function errorFunction(xhr, text, errorThrown) {
        if (errorCallback) {
          errorCallback(xhr, text, errorThrown);
        }
      }
    };
    sendData.sender = sendData;
    return App.ajax.send(sendData);
  },

  /**
   *
   * @param {Em.Object} configGroup
   * @returns {{ConfigGroup: {group_name: *, description: *, tag: *, hosts: *, desired_configs: (Array|*)}}}
   */
  getConfigGroupData: function getConfigGroupData(configGroup) {
    var desiredConfigs = configGroup.get('desiredConfigs') || [];

    return {
      ConfigGroup: {
        group_name: configGroup.get('name'),
        description: configGroup.get('description'),
        tag: configGroup.get('service.id'),
        service_name: configGroup.get('service.id'),
        hosts: configGroup.get('hosts').map(function (h) {
          return {
            host_name: h
          };
        }),
        desired_configs: desiredConfigs.map(function (cst) {
          return {
            type: Em.get(cst, 'site') || Em.get(cst, 'type'),
            tag: Em.get(cst, 'tag')
          };
        })
      }
    };
  },

  /**
   * launch dialog where can be assigned another group to host
   * @param {App.ConfigGroup} selectedGroup
   * @param {App.ConfigGroup[]} configGroups
   * @param {String} hostName
   * @param {Function} callback
   * @return {App.ModalPopup}
   * @method launchSwitchConfigGroupOfHostDialog
   */
  launchSwitchConfigGroupOfHostDialog: function launchSwitchConfigGroupOfHostDialog(selectedGroup, configGroups, hostName, callback) {
    var self = this;
    return App.ModalPopup.show({
      classNames: ['change-config-group-modal'],
      header: Em.I18n.t('config.group.host.switch.dialog.title'),
      configGroups: configGroups,
      selectedConfigGroup: selectedGroup,
      disablePrimary: function () {
        return this.get('selectedConfigGroup.name') === selectedGroup.get('name');
      }.property('selectedConfigGroup.name'),
      onPrimary: function onPrimary() {
        var newGroup = this.get('selectedConfigGroup');
        if (selectedGroup.get('isDefault')) {
          selectedGroup.set('hosts.length', selectedGroup.get('hosts.length') - 1);
        } else {
          selectedGroup.get('hosts').removeObject(hostName);
        }
        if (!selectedGroup.get('isDefault')) {
          self.updateConfigurationGroup(selectedGroup, Em.K, Em.K);
        }

        if (newGroup.get('isDefault')) {
          newGroup.set('hosts.length', newGroup.get('hosts.length') + 1);
        } else {
          newGroup.get('hosts').pushObject(hostName);
        }
        callback(newGroup);
        if (!newGroup.get('isDefault')) {
          self.updateConfigurationGroup(newGroup, Em.K, Em.K);
        }
        this.hide();
      },
      bodyClass: Em.View.extend({
        templateName: require('templates/utils/config_launch_switch_config_group_of_host')
      })
    });
  },

  /**
   * Do request to delete config group
   * @param {App.ConfigGroup} configGroup
   * @param {Function} [successCallback]
   * @param {Function} [errorCallback]
   * @return {$.ajax}
   * @method deleteConfigurationGroup
   */
  deleteConfigurationGroup: function deleteConfigurationGroup(configGroup, successCallback, errorCallback) {
    var self = this;
    var sendData = {
      name: 'common.delete.config_group',
      sender: this,
      data: {
        id: configGroup.get('id')
      },
      success: 'successFunction',
      error: 'errorFunction',
      successFunction: function successFunction(data, xhr, params) {
        self.deleteConfigurationGroupSuccess(data, xhr, params);
        if (successCallback) {
          successCallback();
        }
      },
      errorFunction: function errorFunction(xhr, text, errorThrown) {
        if (errorCallback) {
          errorCallback(xhr, text, errorThrown);
        }
      }
    };
    sendData.sender = sendData;
    return App.ajax.send(sendData);
  },

  /**
   *
   * @param {object} data
   * @param {object} xhr
   * @param {object} params
   */
  deleteConfigurationGroupSuccess: function deleteConfigurationGroupSuccess(data, xhr, params) {
    var groupFromModel = App.ServiceConfigGroup.find().findProperty('id', params.id);
    if (groupFromModel) {
      if (groupFromModel.get('stateManager.currentState.name') !== 'saved') {
        groupFromModel.get('stateManager').transitionTo('loaded');
      }
      App.configGroupsMapper.deleteRecord(groupFromModel);
    }
  },

  /**
   * Launches a dialog where an existing config-group can be selected, or a new
   * one can be created. This is different than the config-group management
   * dialog where host membership can be managed.
   *
   * The callback will be passed the created/selected config-group in the form
   * of {id:2, name:'New hardware group'}. In the case of dialog being cancelled,
   * the callback is provided <code>null</code>
   *
   * @param {String} groupName
   *  is closed, cancelled or OK is pressed.
   * @return {App.ModalPopup}
   * @method saveGroupConfirmationPopup
   */
  saveGroupConfirmationPopup: function saveGroupConfirmationPopup(groupName) {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('config.group.save.confirmation.header'),
      secondary: Em.I18n.t('config.group.save.confirmation.manage.button'),
      groupName: groupName,
      bodyClass: Em.View.extend({
        templateName: require('templates/common/configs/saveConfigGroup')
      }),
      onPrimary: function onPrimary() {
        if (self.get('controller.name') === 'mainServiceInfoConfigsController') {
          self.get('controller').loadConfigGroups([self.get('controller.content.serviceName')]).done(function () {
            var group = App.ServiceConfigGroup.find().find(function (g) {
              return g.get('serviceName') === self.get('controller.content.serviceName') && g.get('name') === groupName;
            });
            self.get('controller').doSelectConfigGroup({ context: group });
          });
        }
        this._super();
      },
      onSecondary: function onSecondary() {
        App.router.get('manageConfigGroupsController').manageConfigurationGroups(null, self.get('controller.content'));
        this.hide();
      }
    });
  },

  /**
   * Persist config groups created in step7 wizard controller
   * @method persistConfigGroups
   */
  persistConfigGroups: function persistConfigGroups() {
    var installerController = App.router.get('installerController');
    var step7Controller = App.router.get('wizardStep7Controller');
    installerController.saveServiceConfigGroups(step7Controller, step7Controller.get('content.controllerName') === 'addServiceController');
    App.clusterStatus.setClusterStatus({
      localdb: App.db.data
    });
  }

});

});

require.register("mixins/main/service/configs/hive_interactive_check", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HiveInteractiveCheck = Em.Mixin.create({
  enableHiveInteractive: false,

  loadHiveConfigs: function loadHiveConfigs() {
    App.ajax.send({
      name: 'service.serviceConfigVersions.get.current',
      sender: this,
      data: {
        serviceNames: ['HIVE']
      },
      success: 'onLoadHiveConfigs'
    });
  },

  onLoadHiveConfigs: function onLoadHiveConfigs(data) {
    var enableHiveInteractive = false;
    data.items.forEach(function (item) {
      var hiveInteractive = item.configurations.findProperty('type', 'hive-interactive-env');
      if (hiveInteractive) {
        enableHiveInteractive = hiveInteractive.properties['enable_hive_interactive'] === 'true';
      }
    });
    this.set('enableHiveInteractive', enableHiveInteractive);
  }
});

});

require.register("mixins/main/service/configs/widget_popover_support", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Popover for configs widgets
 * Usage:
 * <code>
 *  didInsertElement: function () {
 *    this._super();
 *    this.initPopover();
 *  }
 *  </code>
 * @type {Em.Mixin}
 */
App.WidgetPopoverSupport = Em.Mixin.create({

  /**
   * Should popover be on the page
   * @type {boolean}
   */
  isPopoverEnabled: true,

  /**
   * Where popover should be displayed - top|left|right|bottom
   * popover to left if config is located at the right most sub-section of the right most section.
   *
   * @type {string}
   */
  popoverPlacement: function () {
    return this.get('section.isLastColumn') && this.get('subSection.isLastColumn') ? 'left' : 'right';
  }.property('section.isLastColumn', 'subSection.isLastColumn'),

  initPopover: function initPopover() {
    if (this.get('isPopoverEnabled') !== false) {
      this.destroyPopover();
      var leftPopoverTemplate = '<div class="popover config-widget-left-popover"><div class="arrow"></div><div class="popover-inner"><h3 class="popover-title"></h3><div class="popover-content"><p></p></div></div></div>',
          isWidget = !Em.isEmpty(this.$('.original-widget')),
          popoverSelector = isWidget ? this.$('.original-widget') : this.$('.input-group');

      App.popover(popoverSelector, {
        template: this.get('popoverPlacement') === 'left' ? leftPopoverTemplate : undefined,
        title: Em.I18n.t('installer.controls.serviceConfigPopover.title').format(this.get('configLabel'), this.get('configLabel') === this.get('config.name') ? '' : this.get('config.name')),
        content: this.get('config.description'),
        placement: this.get('popoverPlacement'),
        trigger: 'hover',
        html: true,
        delay: {
          show: 1000,
          hide: 0
        }
      });
      this.on('willDestroyElement', this, this.destroyPopover);
    }
  },

  /**
   * Destroy popover after config becomes hidden
   */
  destroyPopover: function destroyPopover() {
    this.movePopover('destroy');
  },

  /**
   * Hide popover on config state changing (from widget-view to raw-mode and from raw-mode to widget-view)
   */
  hidePopover: function () {
    this.movePopover('hide');
  }.observes('config.showAsTextBox'),

  movePopover: function movePopover(action) {
    var popoverSelector = Em.isEmpty(this.$('.original-widget')) ? this.$('.input-group') : this.$('.original-widget');
    if (popoverSelector) {
      this.$(popoverSelector).popover(action);
    }
  }

});

});

require.register("mixins/main/service/groups_mapping", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Provide methods for config-groups loading from server and saving them into models
 *
 * @type {Em.Mixin}
 */
App.GroupsMappingMixin = Em.Mixin.create(App.TrackRequestMixin, {

  /**
   * Load config groups
   * @param {String[]} serviceNames
   * @returns {$.Deferred}
   * @method loadConfigGroups
   */
  loadConfigGroups: function loadConfigGroups(serviceNames) {
    var dfd = $.Deferred();
    if (!serviceNames || serviceNames.length === 0) {
      this.set('configGroupsAreLoaded', true);
      dfd.resolve();
    } else {
      this.trackRequest(App.ajax.send({
        name: 'configs.config_groups.load.services',
        sender: this,
        data: {
          serviceNames: serviceNames.join(','),
          dfd: dfd
        },
        success: 'saveConfigGroupsToModel'
      }));
    }
    return dfd.promise();
  },

  /**
   * Runs <code>configGroupsMapper<code>
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   * @method saveConfigGroupsToModel
   */
  saveConfigGroupsToModel: function saveConfigGroupsToModel(data, opt, params) {
    App.configGroupsMapper.map(data, false, params.serviceNames.split(','));
    this.set('configGroupsAreLoaded', true);
    params.dfd.resolve();
  }

});

});

require.register("mixins/main/service/summary/hdfs_summary_widgets", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.HDFSSummaryWidgetsMixin = Em.Mixin.create({

  model: function () {
    return App.HDFSService.find().objectAt(0);
  }.property('controller.content.serviceName'),

  isFederatedHDFS: Em.computed.gt('model.masterComponentGroups.length', 1),

  isNonFederatedHDFS: Em.computed.not('isFederatedHDFS', 1)

});

});

require.register("mixins/main/service/themes_mapping", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Provide methods for config-themes loading from server and saving them into models
 *
 * @type {Em.Mixin}
 */
App.ThemesMappingMixin = Em.Mixin.create({

  /**
   * Load config themes from server and save them into models
   * @param {string} serviceName
   * @returns {$.ajax}
   * @method loadConfigTheme
   */
  loadConfigTheme: function loadConfigTheme(serviceName) {
    var dfd = $.Deferred();
    if (App.Tab.find().mapProperty('serviceName').contains(serviceName)) {
      dfd.resolve();
    } else {
      App.ajax.send({
        name: 'configs.theme',
        sender: this,
        data: {
          serviceName: serviceName,
          stackVersionUrl: App.get('stackVersionURL')
        },
        success: '_saveThemeToModel'
      }).complete(dfd.resolve);
    }
    return dfd.promise();
  },

  /**
   * Success-callback for <code>loadConfigTheme</code>
   * runs <code>themeMapper<code>
   * @param {object} data
   * @param opt
   * @param params
   * @private
   * @method saveThemeToModel
   */
  _saveThemeToModel: function _saveThemeToModel(data, opt, params) {
    App.themesMapper.map(data, [params.serviceName]);
  },

  /**
   * Load themes for specified services by one API call
   *
   * @method loadConfigThemeForServices
   * @param {String|String[]} serviceNames
   * @returns {$.ajax}
   */
  loadConfigThemeForServices: function loadConfigThemeForServices(serviceNames) {
    return App.ajax.send({
      name: 'configs.theme.services',
      sender: this,
      data: {
        serviceNames: Em.makeArray(serviceNames).join(','),
        stackVersionUrl: App.get('stackVersionURL')
      },
      success: '_loadConfigThemeForServicesSuccess',
      error: '_loadConfigThemeForServicesError'
    });
  },

  /**
   * Success-callback for <code>loadConfigThemeForServices</code>
   * @param {object} data
   * @param opt
   * @param params
   * @private
   * @method _loadConfigThemeForServicesSuccess
   */
  _loadConfigThemeForServicesSuccess: function _loadConfigThemeForServicesSuccess(data, opt, params) {
    if (!data.items.length) return;
    App.themesMapper.map({
      items: data.items.mapProperty('themes').reduce(function (p, c) {
        return p.concat(c);
      })
    }, params.serviceNames.split(','));
  },

  /**
   * Error-callback for <code>loadConfigThemeForServices</code>
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   * @private
   * @method _loadConfigThemeForServicesError
   */
  _loadConfigThemeForServicesError: function _loadConfigThemeForServicesError(request, ajaxOptions, error, opt, params) {}

});

});

require.register("mixins/routers/redirections", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for routers which may redirect user to the some cluster installer steps
 * Based on cluster state
 * @type {Em.Mixin}
 */
App.RouterRedirections = Em.Mixin.create({

  /**
   * List of cluster statuses when it's not installed
   * @type {string[]}
   */
  installerStatuses: ['CLUSTER_NOT_CREATED_1', 'CLUSTER_DEPLOY_PREP_2', 'CLUSTER_INSTALLING_3', 'SERVICE_STARTING_3', 'CLUSTER_INSTALLED_4'],

  /**
   * Redirect user to the proper installer step if needed
   * Based on cluster state:
   *  1. CLUSTER_NOT_CREATED_1 - to current step
   *  2. CLUSTER_DEPLOY_PREP_2 - to step 8
   *  3. CLUSTER_INSTALLING_3/SERVICE_STARTING_3 - to step 9
   *  4. CLUSTER_INSTALLED_4 - to step 10
   * @param {Em.Router} router
   * @param {Object} currentClusterStatus
   * @param {Boolean} isOnInstaller true - user is on the cluster installer, false - something else
   * @method redirectToInstaller
   */
  redirectToInstaller: function redirectToInstaller(router, currentClusterStatus, isOnInstaller) {

    var installerController = router.get('installerController');
    var path = isOnInstaller ? '' : 'installer.';
    switch (currentClusterStatus.clusterState) {
      case 'CLUSTER_NOT_CREATED_1':
        App.get('router').setAuthenticated(true);
        router.transitionTo(path + 'step' + installerController.get('currentStep'));
        break;
      case 'CLUSTER_DEPLOY_PREP_2':
        installerController.setCurrentStep('8');
        App.db.data = currentClusterStatus.localdb;
        App.get('router').setAuthenticated(true);
        router.transitionTo(path + 'step' + installerController.get('currentStep'));
        break;
      case 'CLUSTER_INSTALLING_3':
      case 'SERVICE_STARTING_3':
        if (!installerController.get('isStep9')) {
          installerController.setCurrentStep('9');
          installerController.setLowerStepsDisable(9);
        }
        router.transitionTo(path + 'step' + installerController.get('currentStep'));
        break;
      case 'CLUSTER_INSTALLED_4':
        if (!installerController.get('isStep10')) {
          installerController.setCurrentStep('10');
          installerController.setLowerStepsDisable(10);
        }
        App.db.data = currentClusterStatus.localdb;
        App.get('router').setAuthenticated(true);
        router.transitionTo(path + 'step' + installerController.get('currentStep'));
        break;
    }
  }

});

});

require.register("mixins/unit_convert/base_unit_convert_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.BaseUnitConvertMixin = Em.Mixin.create({
  /**
   * Which type of units should be used e.g. size, time.
   * Conversion dimensions will be used according this type.
   * For more info regarding dimension table @see convertMapTable property.
   *
   * This property can be used in mixed object to specify which dimension table map will
   * be used for conversion. If type is not specified we try to find correct table by input unit type.
   *
   * Note: You should specify dimension for `percentage` conversion with specified property unit type.
   *
   * @property currentDimensionType
   * @type {String}
   */
  currentDimensionType: null,

  units: ['b', 'kb', 'mb', 'gb', 'tb', 'pb'],

  /**
   * Labels related to units. Specify desired display names here that not much actual unit name.
   */
  unitLabelMap: {
    percent: '%',
    int: '',
    float: ''
  },

  convertMapTable: Em.Object.create({
    size: {
      b: 1024,
      kb: 1024,
      mb: 1024,
      gb: 1024,
      tb: 1024,
      pb: 1024
    },
    time: {
      milliseconds: 1,
      seconds: 1000,
      minutes: 60,
      hours: 60,
      days: 24
    },
    /**
     * Percent dimension type should be specified directly through `currentDimensionType` property.
     * For example:
     * 'percent.percent_int' if widget `unit-name` is "percent" and config property `type` is "int"
     * 'percent.percent_float' if widget `unit-name` is "percent" and config property `type` is "float"
     */
    percent: {
      percent_int: {
        int: 1,
        percent: 1
      },
      percent_float: {
        float: 1,
        percent: 0.01
      }
    }
  }),

  /**
   * Convert value between different unit types. Conversion works for following directions:
   *  single unit -> single unit e.g. from "s" -> "s".
   *  single unit -> multi unit e.g. from "ms" -> "days,hours"
   *  multi unit -> single unit e.g. from "days,hours" -> "ms"
   * For single -> single unit conversion returned value will be as is or an array of objects if
   * isObjectOutput flag passed.
   * If any passed unit is multi dimensions like "days,hours" array of objects will return always.
   * For example:
   *  <code>
   *     convertValue(1024, 'kb', 'kb') // returns 1024
   *     convertValue(1024, 'kb', 'kb', true) // returns [{ type: 'kb', value: 1024 }]
   *     convertValue(60000, 'ms', 'hours,minutes') // returns [{type: 'hours', value: 0 }, {type: 'minutes', value: 1}]
   *  </code>
   * For more examples see unit tests.
   *
   * @method convertValue
   * @param {String|Number|Object[]} value - value to convert
   * @param {String|String[]} fromUnit - specified value unit type(s) e.g. "mb"
   *    Form multi dimensional format pass units separated with "," or list of unit types
   *    e.g. "gb,mb", ['gb', 'mb']
   * @param {Boolean} [isObjectOutput=false] - 'returned' value should be object this option useful for widgets where its
   *    value should be an Object
   * @param {String|String[]} toUnit - desired unit(s) to convert specified value. Same format as for `fromUnit`
   * @returns {Number|Object[]} returns single value or array of objects according to multi unit format
   */
  convertValue: function convertValue(value, fromUnit, toUnit, isObjectOutput) {
    var self = this;
    var valueIsArray = Em.isArray(value);
    if (!valueIsArray) {
      value = +value;
    }
    // if desired unit not passed or fromUnit and toUnit are the same
    // just return the value
    if ((fromUnit == toUnit || !toUnit) && !isObjectOutput) {
      return value;
    }
    if (isNaN(value) && !valueIsArray) {
      return null;
    }
    // convert passed toUnit string to array of units by ','
    // for example "mb,kb" to ['mb','kb']
    if (!Em.isArray(toUnit)) {
      toUnit = toUnit.split(',');
    }
    // process multi unit format
    if (toUnit.length > 1 || isObjectOutput) {
      // returned array has following structure
      //  .value - value according to unit
      //  .type - unit name
      return toUnit.map(function (unit) {
        var convertedValue = Math.floor(self._convertToSingleValue(value, fromUnit, unit));
        value -= self._convertToSingleValue(convertedValue, unit, fromUnit);
        return {
          value: convertedValue,
          type: unit
        };
      });
    }
    // for single unit format just return single value
    else {
        if (!valueIsArray) {
          return this._convertToSingleValue(value, fromUnit, toUnit[0]);
        } else {
          return value.map(function (item) {
            return self._convertToSingleValue(item.value, item.type, toUnit[0]);
          }).reduce(Em.sum, 0);
        }
      }
  },

  /**
   * Get dimension table map. `currentDimensionType` will be used if specified or
   * detected by property unit type.
   *
   * @method _converterGetUnitTable
   * @private
   * @param {String|String[]} unit - unit type
   * @return {Object}
   */
  _converterGetUnitTable: function _converterGetUnitTable(unit) {
    var unitType;
    if (this.get('currentDimensionType')) {
      unitType = this.get('currentDimensionType');
    } else {
      unitType = Em.keys(this.get('convertMapTable')).filter(function (item) {
        return Em.keys(this.get('convertMapTable.' + item)).contains(Em.isArray(unit) ? unit[0].toLowerCase() : unit.toLowerCase());
      }, this)[0];
    }
    return this.get('convertMapTable.' + unitType);
  },

  /**
   * @method _convertToSingleValue
   * @private
   * @param {Number} value - value to convert
   * @param {String} fromUnit - unit type for current value
   * @param {String} toUnit - desired unit type
   * @return {Number}
   */
  _convertToSingleValue: function _convertToSingleValue(value, fromUnit, toUnit) {
    var convertTable = this._converterGetUnitTable(fromUnit);
    var units = Em.keys(convertTable);
    var fromUnitIndex = units.indexOf(fromUnit.toLowerCase());
    var toUnitIndex = units.indexOf(toUnit.toLowerCase());
    var isInt = function isInt(val) {
      return parseInt(val) === val;
    };
    Em.assert("Invalid value unit type " + fromUnit, fromUnitIndex > -1);
    Em.assert("Invalid desired unit type " + toUnit, toUnitIndex > -1);
    if (fromUnitIndex == toUnitIndex) {
      return value;
    }
    var range = [fromUnitIndex + 1, toUnitIndex + 1];
    var processedUnits = Array.prototype.slice.apply(units, range[0] < range[1] ? range : range.slice().reverse());
    var factor = processedUnits.map(function (unit) {
      return Em.get(convertTable, unit);
    }, this).reduce(function (p, c) {
      return p * c;
    });
    if (range[0] < range[1]) {
      value /= factor;
    } else {
      value *= factor;
    }
    return isInt(value) ? value : parseFloat(value.toFixed(3));
  }

});

});

require.register("mixins/unit_convert/convert_unit_widget_view_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
/**
 * Unit conversion mixin for widget views.
 * @type {Em.Mixin}
 */
App.ConvertUnitWidgetViewMixin = Em.Mixin.create(App.BaseUnitConvertMixin, {
  /**
   * Get converted value according to widget value format from specified config property value.
   *
   * @method widgetValueByConfigAttributes
   * @param {String|Number} value - config property value to convert
   * @param {Boolean} [returnObject=false] - returned value should be an array of objects
   * @returns {Number|Object[]}
   */
  widgetValueByConfigAttributes: function widgetValueByConfigAttributes(value, returnObject) {
    return this.convertValue(value, this._converterGetPropertyAttributes(), this._converterGetWidgetUnits(), returnObject);
  },

  /**
   * Get converted value according to config property unit format from specified widget value.
   *
   * @param {String|Number|Object[]} value - widget value to convert
   * @returns {String}
   * @method configValueByWidget
   */
  configValueByWidget: function configValueByWidget(value) {
    var cfgValue = this.convertValue(value, this._converterGetWidgetUnits(), this._converterGetPropertyAttributes());
    return '' + this.get('config.stackConfigProperty.valueAttributes.type') === 'int' ? Math.round(cfgValue) : cfgValue;
  },

  /**
   * @private
   * @method _converterGetWidgetUnits
   * @throw Error
   * @returns {String|String[]}
   */
  _converterGetWidgetUnits: function _converterGetWidgetUnits() {
    var widgetAttributes = this.get('config.stackConfigProperty.widget');
    var widgetUnits = Em.getWithDefault(widgetAttributes, 'units.0.unit-name', false) || Em.getWithDefault(widgetAttributes, 'units.0.unit', false);
    Em.assert('Invalid widget units for ' + this.get('config.name') + ' widget object: ' + JSON.stringify(widgetAttributes), widgetUnits);
    return widgetUnits;
  },

  /**
   * @private
   * @method _converterGetPropertyAttributes
   * @throw Error
   * @returns {String|String[]}
   */
  _converterGetPropertyAttributes: function _converterGetPropertyAttributes() {
    var propertyAttributes = this.get('config.stackConfigProperty.valueAttributes');
    var propertyUnits = Em.getWithDefault(propertyAttributes, 'unit', false) || Em.getWithDefault(propertyAttributes, 'type', false);
    Em.assert('Invalid property unit type for ' + this.get('config.name') + ' valueAttributes: ' + JSON.stringify(propertyAttributes), propertyUnits);
    return propertyUnits;
  }
});

});

require.register("mixins/wizard/addSecurityConfigs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var blueprintUtils = require('utils/blueprint');

/**
 * Mixin for loading and setting secure configs
 *
 * @type {Ember.Mixin}
 */
App.AddSecurityConfigs = Em.Mixin.create({

  kerberosDescriptor: {},

  kerberosDescriptorProperties: require('data/configs/wizards/kerberos_descriptor_properties'),

  /**
   * security configs, which values should be modified after APPLY CONFIGURATIONS stage
   */
  secureConfigs: [{
    name: 'zookeeper_principal_name',
    serviceName: 'ZOOKEEPER'
  }, {
    name: 'knox_principal_name',
    serviceName: 'KNOX'
  }, {
    name: 'storm_principal_name',
    serviceName: 'STORM'
  }, {
    name: 'nimbus_principal_name',
    serviceName: 'STORM'
  }],

  /**
   *
   * @param {object[]} items - stack descriptor json response
   * @returns {App.ServiceConfigProperty[]}
   */
  createServicesStackDescriptorConfigs: function createServicesStackDescriptorConfigs(items) {
    var self = this;
    var configs = [];
    var clusterConfigs = [];
    var kerberosDescriptor = Em.get(items, 'KerberosDescriptor.kerberos_descriptor');
    this.set('kerberosDescriptor', kerberosDescriptor);
    // generate configs for root level properties object, currently realm, keytab_dir
    clusterConfigs = clusterConfigs.concat(this.expandKerberosStackDescriptorProps(kerberosDescriptor.properties, 'Cluster', 'stackConfigs'));
    // generate configs for root level identities object, currently spnego property
    clusterConfigs = clusterConfigs.concat(this.createConfigsByIdentities(kerberosDescriptor.identities, 'Cluster'));
    kerberosDescriptor.services.forEach(function (service) {
      var serviceName = service.name;
      // generate configs for service level identity objects
      configs = configs.concat(self.createResourceConfigs(service, serviceName));
      // generate configs for service component level identity  object
      Em.getWithDefault(service, 'components', []).forEach(function (component) {
        configs = configs.concat(self.createResourceConfigs(component, serviceName));
      });
    });
    // unite cluster, service and component configs
    configs = configs.concat(clusterConfigs);
    self.processConfigReferences(kerberosDescriptor, configs);
    return configs;
  },

  /**
   *
   * @param {Object} resource
   * @param {String} serviceName
   * @return {Array}
   */
  createResourceConfigs: function createResourceConfigs(resource, serviceName) {
    var identityConfigs = [];
    var resourceConfigs = [];
    if (resource.identities) {
      identityConfigs = this.createConfigsByIdentities(resource.identities, serviceName);
    }
    if (resource.configurations) {
      resource.configurations.forEach(function (_configuration) {
        for (var key in _configuration) {
          resourceConfigs = resourceConfigs.concat(this.expandKerberosStackDescriptorProps(_configuration[key], serviceName, key));
        }
      }, this);
    }
    return identityConfigs.concat(resourceConfigs);
  },

  /**
   * Create service properties based on component identity
   *
   * @param {object[]} identities
   * @param {string} serviceName
   * @returns {App.ServiceConfigProperty[]}
   */
  createConfigsByIdentities: function createConfigsByIdentities(identities, serviceName) {
    var self = this;
    var configs = [];
    identities.forEach(function (identity) {
      var defaultObject = {
        isConfigIdentity: true,
        isOverridable: false,
        // the user should not be changing the principal name and keytab file values for identity items
        // that reference some other identity item,
        // if an identity entry has a "reference" attribute or the name attribute starts with a "/"
        isVisible: !Boolean(identity.reference || identity.name.startsWith('/')),
        isSecureConfig: true,
        serviceName: serviceName,
        name: identity.name,
        identityType: identity.principal && identity.principal.type
      };
      self.parseIdentityObject(identity).forEach(function (item) {
        configs.push(App.ServiceConfigProperty.create($.extend({}, defaultObject, item)));
      });
    });

    return configs;
  },

  /**
   * Bootstrap base object according to identity info. Generate objects will be converted to
   * App.ServiceConfigProperty model class instances.
   *
   * @param {object} identity
   * @returns {object[]}
   */
  parseIdentityObject: function parseIdentityObject(identity) {
    var result = [];
    var name = identity.name;
    var self = this;
    Em.keys(identity).without('name').forEach(function (item) {
      var configObject = {};
      var prop = identity[item];
      var itemValue = prop[{ keytab: 'file', principal: 'value' }[item]];
      var predefinedProperty;
      // skip inherited property without `configuration` and `keytab` or `file` values
      if (!prop.configuration && !itemValue) return;
      // inherited property with value should not observe value from reference
      if (name.startsWith('/') && !itemValue) {
        configObject.referenceProperty = name.substring(1) + ':' + item;
        configObject.isEditable = false;
      }
      Em.setProperties(configObject, {
        recommendedValue: itemValue,
        initialValue: itemValue,
        defaultValue: itemValue,
        value: itemValue
      });
      configObject.filename = prop.configuration ? prop.configuration.split('/')[0] : 'cluster-env';
      configObject.name = prop.configuration ? prop.configuration.split('/')[1] : name + '_' + item;
      predefinedProperty = self.get('kerberosDescriptorProperties').findProperty('name', configObject.name);
      configObject.displayName = self._getDisplayNameForConfig(configObject.name, configObject.filename);
      configObject.index = predefinedProperty && !Em.isNone(predefinedProperty.index) ? predefinedProperty.index : Infinity;
      result.push(configObject);
    });
    return result;
  },

  /**
   * Get new config display name basing on its name and filename
   * If config <code>fileName</code> is `cluster-env`, normalizing for its <code>name</code> is used (@see App.format.normalizeName)
   * If config is predefined in the <code>secureProperties</code> (and it's displayName isn't empty there), predefined displayName is used
   * Otherwise - config <code>name</code> is returned
   *
   * @param {string} name config name
   * @param {string} fileName config filename
   * @returns {String} new config display name
   * @method _getDisplayNameForConfig
   * @private
   */
  _getDisplayNameForConfig: function _getDisplayNameForConfig(name, fileName) {
    var predefinedConfig = App.config.get('kerberosIdentitiesMap')[App.config.configId(name, fileName)];
    return predefinedConfig && predefinedConfig.displayName ? predefinedConfig.displayName : fileName == 'cluster-env' ? App.format.normalizeName(name) : name;
  },

  /**
   * Wrap kerberos properties to App.ServiceConfigProperty model class instances.
   *
   * @param {object} kerberosProperties
   * @param {string} serviceName
   * @param {string} filename
   * @returns {App.ServiceConfigProperty[]}
   */
  expandKerberosStackDescriptorProps: function expandKerberosStackDescriptorProps(kerberosProperties, serviceName, filename) {
    var configs = [];

    for (var propertyName in kerberosProperties) {
      var predefinedProperty = this.get('kerberosDescriptorProperties').findProperty('name', propertyName);
      var value = kerberosProperties[propertyName];
      var isRequired = ['additional_realms', 'principal_suffix'].contains(propertyName) ? false : value !== "";
      var propertyObject = {
        name: propertyName,
        value: value,
        defaultValue: value,
        recommendedValue: value,
        initialValue: value,
        serviceName: serviceName,
        filename: filename,
        displayName: serviceName == "Cluster" ? App.format.normalizeName(propertyName) : propertyName,
        isOverridable: false,
        isEditable: propertyName != 'realm',
        isRequired: isRequired,
        isSecureConfig: true,
        placeholderText: predefinedProperty && !Em.isNone(predefinedProperty.index) ? predefinedProperty.placeholderText : '',
        index: predefinedProperty && !Em.isNone(predefinedProperty.index) ? predefinedProperty.index : Infinity
      };
      configs.push(App.ServiceConfigProperty.create(propertyObject));
    }

    return configs;
  },

  /**
   * Take care about configs that should observe value from referenced configs.
   * Reference is set with `referenceProperty` key.
   *
   * @param {object[]} kerberosDescriptor
   * @param {App.ServiceConfigProperty[]} configs
   */
  processConfigReferences: function processConfigReferences(kerberosDescriptor, configs) {
    var identities = kerberosDescriptor.identities;

    /**
     * Returns indentity object with additional attribute `referencePath`.
     * Reference path depends on how deep identity is. Each level separated by `/` sign.
     *
     * @param {object} identity
     * @param {string} [prefix=false] prefix to append e.g. 'SERVICE_NAME'
     * @returns {object} identity object
     */
    var setReferencePath = function setReferencePath(identity, prefix) {
      var name = Em.getWithDefault(identity, 'name', false);
      if (name) {
        if (prefix) {
          name = prefix + '/' + name;
        }
        identity.referencePath = name;
      }
      return identity;
    };

    // map all identities and add attribute `referencePath`
    // `referencePath` is a path to identity it can be 1-3 levels
    // 1 for "/global" identity e.g. `/spnego`
    // 2 for "/SERVICE/identity"
    // 3 for "/SERVICE/COMPONENT/identity"
    identities = identities.map(function (i) {
      return setReferencePath(i);
    }).concat(kerberosDescriptor.services.map(function (service) {
      var serviceName = Em.getWithDefault(service, 'name', false);
      var serviceIdentities = Em.getWithDefault(service, 'identities', []).map(function (i) {
        return setReferencePath(i, serviceName);
      });
      var componentIdentities = Em.getWithDefault(service || {}, 'components', []).map(function (i) {
        var componentName = Em.getWithDefault(i, 'name', false);
        return Em.getWithDefault(i, 'identities', []).map(function (componentIdentity) {
          return setReferencePath(componentIdentity, serviceName + '/' + componentName);
        });
      }).reduce(function (p, c) {
        return p.concat(c);
      }, []);
      serviceIdentities.pushObjects(componentIdentities);
      return serviceIdentities;
    }).reduce(function (p, c) {
      return p.concat(c);
    }, []));
    // clean up array
    identities = identities.compact().without(undefined);
    configs.forEach(function (item) {
      var reference = item.get('referenceProperty');
      if (!!reference) {
        // first find identity by `name`
        // if not found try to find by `referencePath`
        var identity = Em.getWithDefault(identities.findProperty('name', reference.split(':')[0]) || {}, reference.split(':')[1], false) || Em.getWithDefault(identities.findProperty('referencePath', reference.split(':')[0]) || {}, reference.split(':')[1], false);
        if (identity && !!identity.configuration) {
          item.set('observesValueFrom', identity.configuration.split('/')[1]);
        } else {
          item.set('observesValueFrom', reference.replace(':', '_'));
        }
      }
    });
  },

  /**
   * update the kerberos descriptor to be put on cluster resource with user customizations
   * @param kerberosDescriptor {Object}
   * @param configs {Object}
   */
  updateKerberosDescriptor: function updateKerberosDescriptor(kerberosDescriptor, configs) {
    configs.forEach(function (_config) {
      var isConfigUpdated;
      var isStackResource = true;
      isConfigUpdated = this.updateResourceIdentityConfigs(kerberosDescriptor, _config, isStackResource);
      if (!isConfigUpdated) {
        kerberosDescriptor.services.forEach(function (_service) {
          isConfigUpdated = this.updateResourceIdentityConfigs(_service, _config);
          if (!isConfigUpdated) {
            Em.getWithDefault(_service, 'components', []).forEach(function (_component) {
              isConfigUpdated = this.updateResourceIdentityConfigs(_component, _config);
            }, this);
          }
        }, this);
      }
    }, this);
  },

  /**
   * Updates the identity configs or configurations at a resource. A resource could be
   * 1) Stack
   * 2) Service
   * 3) Component
   * @param resource
   * @param config
   * @param isStackResource
   * @return boolean
   */
  updateResourceIdentityConfigs: function updateResourceIdentityConfigs(resource, config, isStackResource) {
    var isConfigUpdated;
    var identities = resource.identities;
    var properties = !!isStackResource ? resource.properties : resource.configurations;
    isConfigUpdated = this.updateDescriptorConfigs(properties, config);
    if (!isConfigUpdated) {
      if (identities) {
        isConfigUpdated = this.updateDescriptorIdentityConfig(identities, config);
      }
    }
    return isConfigUpdated;
  },

  /**
   * This function updates stack/service/component level configurations of the kerberos descriptor
   * with the values entered by the user on the rendered ui
   * @param configurations
   * @param config
   * @return boolean
   */
  updateDescriptorConfigs: function updateDescriptorConfigs(configurations, config) {
    var isConfigUpdated;
    if (!!configurations) {
      if (Array.isArray(configurations)) {
        configurations.forEach(function (_configuration) {
          for (var key in _configuration) {
            if (Object.keys(_configuration[key]).contains(config.name) && App.config.getConfigTagFromFileName(config.filename) === key) {
              _configuration[key][config.name] = config.value;
              isConfigUpdated = true;
            }
          }
        }, this);
      } else if (Object.keys(configurations).contains(config.name) && App.config.getConfigTagFromFileName(config.filename) === 'stackConfigs') {
        configurations[config.name] = config.value;
        isConfigUpdated = true;
      }
    }
    return isConfigUpdated;
  },

  /**
   * This function updates stack/service/component level kerberos descriptor identities (principal and keytab)
   * with the values entered by the user on the rendered ui
   * @param identities
   * @param config
   * @return boolean
   */
  updateDescriptorIdentityConfig: function updateDescriptorIdentityConfig(identities, config) {
    var isConfigUpdated = false;
    identities.forEach(function (identity) {
      var keys = Em.keys(identity).without('name');
      keys.forEach(function (item) {
        var prop = identity[item];

        // compare ui rendered config against identity with `configuration attribute` (Most of the identities have `configuration attribute`)
        var isIdentityWithConfig = prop.configuration && prop.configuration.split('/')[0] === App.config.getConfigTagFromFileName(config.filename) && prop.configuration.split('/')[1] === config.name;

        // compare ui rendered config against identity without `configuration attribute` (For example spnego principal and keytab)
        var isIdentityWithoutConfig = !prop.configuration && identity.name === config.name.split('_')[0] && item === config.name.split('_')[1];

        if (isIdentityWithConfig || isIdentityWithoutConfig) {
          prop[{ keytab: 'file', principal: 'value' }[item]] = config.value;
          isConfigUpdated = true;
        }
      });
    }, this);
    return isConfigUpdated;
  },

  /**
   * Make request for cluster descriptor configs.
   *
   * @param {string[]|null} [services=null] services to be added
   * @returns {$.ajax}
   * @method loadClusterDescriptorConfigs
   */
  loadClusterDescriptorConfigs: function loadClusterDescriptorConfigs(services) {
    var servicesParam = services ? 'additional_services=' + services.join(',') : null,
        queryParams = ['evaluate_when=true'].concat(servicesParam).compact().join('&');

    return App.ajax.send({
      sender: this,
      name: 'admin.kerberize.cluster_descriptor',
      data: {
        queryParams: '?' + queryParams
      }
    });
  },

  loadClusterDescriptorStackConfigs: function loadClusterDescriptorStackConfigs() {
    return App.ajax.send({
      sender: this,
      name: 'admin.kerberize.cluster_descriptor.stack'
    });
  },

  mergeDescriptorStackWithConfigs: function mergeDescriptorStackWithConfigs(descriptorStack, descriptorConfigs) {
    var result = [];
    var stackConfigs = this.createServicesStackDescriptorConfigs(descriptorStack);
    var currentConfigs = this.createServicesStackDescriptorConfigs(descriptorConfigs);

    stackConfigs.forEach(function (stackConfig) {
      var currentConfig = currentConfigs.filterProperty('name', stackConfig.get('name')).findProperty('filename', stackConfig.get('filename'));
      if (currentConfig) {
        currentConfigs = currentConfigs.without(currentConfig);
        result.push(currentConfig);
      } else {
        result.push(stackConfig);
      }
    });

    // add all properties from descriptor/COMPOSITE, that are absent in descriptor/STACK as custom properties
    currentConfigs.setEach('isUserProperty', true);
    result = result.concat(currentConfigs);

    return result;
  },

  /**
   * Prepare step configs using stack descriptor properties.
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @param {App.ServiceConfigProperty[]} stackConfigs converted kerberos descriptor
   */
  setStepConfigs: function setStepConfigs(configs, stackConfigs, showAdminProperties) {
    var configProperties = this.prepareConfigProperties(configs, showAdminProperties),
        stackConfigProperties = stackConfigs ? this.prepareConfigProperties(stackConfigs, showAdminProperties) : [],
        alterProperties = ['value', 'initialValue', 'defaultValue'];
    if (this.get('wizardController.name') === 'addServiceController') {
      // config properties for installed services should be disabled on Add Service Wizard
      configProperties.forEach(function (item) {
        if (this.get('installedServiceNames').contains(item.get('serviceName')) || item.get('serviceName') == 'Cluster') {
          item.set('isEditable', false);
        } else if (stackConfigs) {
          var stackConfigProperty = stackConfigProperties.filterProperty('filename', item.get('filename')).findProperty('name', item.get('name'));
          if (stackConfigProperty) {
            alterProperties.forEach(function (alterProperty) {
              item.set(alterProperty, stackConfigProperty.get(alterProperty));
            });
          }
        }
      }, this);
      // Concat properties that are present in the stack's kerberos  descriptor but not in the cluster kerberos descriptor
      stackConfigProperties.forEach(function (_stackConfigProperty) {
        var isPropertyInClusterDescriptor = configProperties.filterProperty('filename', _stackConfigProperty.get('filename')).someProperty('name', _stackConfigProperty.get('name'));
        if (!isPropertyInClusterDescriptor) {
          if (this.get('installedServiceNames').contains(_stackConfigProperty.get('serviceName')) || _stackConfigProperty.get('serviceName') === 'Cluster') {
            _stackConfigProperty.set('isEditable', false);
          }
          configProperties.pushObject(_stackConfigProperty);
        }
      }, this);
    }
    configProperties = App.config.sortConfigs(configProperties);
    var stepConfigs = this.createServiceConfig(configProperties);
    this.get('stepConfigs').pushObjects(stepConfigs);
    return stepConfigs;
  },

  /**
   * Filter configs by installed services for Kerberos Wizard or by installed + selected services
   * for Add Service Wizard.
   * Set property value observer.
   * Set realm property with value from previous configuration step.
   * Set appropriate category for all configs.
   * Hide KDC related credentials properties if kerberos was manually enabled.
   *
   * @param {App.ServiceConfigProperty[]} configs
   * @returns {App.ServiceConfigProperty[]}
   */
  prepareConfigProperties: function prepareConfigProperties(configs, showAdminProperties) {
    var self = this;
    // stored configs from previous steps (Configure Kerberos or Customize Services for ASW)
    var storedServiceConfigs = this.get('wizardController.content.serviceConfigProperties');
    var installedServiceNames = ['Cluster', 'AMBARI'].concat(App.Service.find().mapProperty('serviceName'));
    var configProperties = configs.slice(0);
    var siteProperties = App.configsCollection.getAll();
    var realmValue;
    // override stored values
    App.config.mergeStoredValue(configProperties, this.get('wizardController').loadCachedStepConfigValues(this));

    // show admin properties in add service wizard
    if (showAdminProperties) {
      installedServiceNames = installedServiceNames.concat(this.get('selectedServiceNames'));
    }
    configProperties = configProperties.filter(function (item) {
      return installedServiceNames.contains(item.get('serviceName'));
    });
    if (this.get('wizardController.name') !== 'addServiceController') {
      realmValue = storedServiceConfigs.findProperty('name', 'realm').value;
      configProperties.findProperty('name', 'realm').set('value', realmValue);
      configProperties.findProperty('name', 'realm').set('savedValue', realmValue);
      configProperties.findProperty('name', 'realm').set('recommendedValue', realmValue);
    }

    configProperties.setEach('isSecureConfig', false);
    configProperties.forEach(function (property, item, allConfigs) {
      if (['spnego_keytab', 'spnego_principal'].contains(property.get('name'))) {
        property.addObserver('value', self, 'spnegoPropertiesObserver');
      }
      if (property.get('observesValueFrom') && allConfigs.someProperty('name', property.get('observesValueFrom'))) {
        var observedValue = Em.get(allConfigs.findProperty('name', property.get('observesValueFrom')), 'value');
        property.set('value', observedValue);
        property.set('recommendedValue', observedValue);
        property.set('isVisible', true);
      }
      if (property.get('serviceName') === 'Cluster') {
        property.set('category', 'Global');
      } else {
        property.set('category', property.get('serviceName'));
      }
      // All user identity except storm should be grouped under "Ambari Principals" category
      if (property.get('identityType') == 'user') property.set('category', 'Ambari Principals');
      var siteProperty = siteProperties.findProperty('name', property.get('name'));
      if (siteProperty) {
        if (siteProperty.category === property.get('category')) {
          property.set('displayName', siteProperty.displayName);
          if (siteProperty.index) {
            property.set('index', siteProperty.index);
          }
        }
        if (siteProperty.displayType) {
          property.set('displayType', siteProperty.displayType);
        }
      }
      this.tweakConfigProperty(property);
    }, this);

    return configProperties;
  },

  /**
   * Function to override kerberos descriptor's property values
   */
  tweakConfigProperty: function tweakConfigProperty(config) {
    var defaultHiveMsPort = "9083",
        hiveMSHosts,
        port,
        hiveMSHostNames,
        configValue;
    if (config.name === 'templeton.hive.properties') {
      hiveMSHosts = App.HostComponent.find().filterProperty('componentName', 'HIVE_METASTORE');
      if (hiveMSHosts.length > 1) {
        hiveMSHostNames = hiveMSHosts.mapProperty('hostName');
        port = config.value.match(/:[0-9]{2,4}/);
        port = port ? port[0].slice(1) : defaultHiveMsPort;
        for (var i = 0; i < hiveMSHostNames.length; i++) {
          hiveMSHostNames[i] = "thrift://" + hiveMSHostNames[i] + ":" + port;
        }
        configValue = config.value.replace(/thrift.+[0-9]{2,},/i, hiveMSHostNames.join('\\,') + ",");
        config.set('value', configValue);
        config.set('recommendedValue', configValue);
      }
    }
  },

  /**
   * Sync up values between inherited property and its reference.
   *
   * @param {App.ServiceConfigProperty} configProperty
   */
  spnegoPropertiesObserver: function spnegoPropertiesObserver(configProperty) {
    var stepConfig = this.get('stepConfigs').findProperty('name', 'KERBEROS') || this.get('stepConfigs').findProperty('name', 'ADVANCED');

    stepConfig.get('configs').forEach(function (config) {
      if (config.get('observesValueFrom') === configProperty.get('name')) {
        Em.run.once(this, function () {
          config.set('value', configProperty.get('value'));
          config.set('recommendedValue', configProperty.get('value'));
        });
      }
    }, this);
  },
  /**
  * Prepare all necessary data for recommendations payload.
  *
  * #mutates initialConfigValues
  * @returns {$.Deferred.promise()}
  */
  bootstrapRecommendationPayload: function bootstrapRecommendationPayload(kerberosDescriptor) {
    var dfd = $.Deferred();
    var self = this;

    this.getServicesConfigurations().then(function (configurations) {
      var recommendations = self.getBlueprintPayloadObject(configurations, kerberosDescriptor);
      self.set('servicesConfigurations', configurations);
      self.set('initialConfigValues', recommendations.blueprint.configurations);
      dfd.resolve(recommendations);
    });
    return dfd.promise();
  },

  getServicesConfigurations: function getServicesConfigurations() {
    var dfd = $.Deferred();
    var self = this;
    this.getConfigTags().then(function () {
      App.router.get('configurationController').getConfigsByTags(self.get('serviceConfigTags')).done(function (configurations) {
        dfd.resolve(configurations);
      });
    });

    return dfd.promise();
  },

  /**
   * Returns payload for recommendations request.
   * Takes services' configurations and merge them with kerberos descriptor properties.
   *
   * @param {object[]} configurations services' configurations fetched from API
   * @param {App.ServiceConfigProperty[]} kerberosDescriptor descriptor configs
   * @returns {object} payload for recommendations request
   */
  getBlueprintPayloadObject: function getBlueprintPayloadObject(configurations, kerberosDescriptor) {
    var recommendations = blueprintUtils.generateHostGroups(App.get('allHostNames'));
    var mergedConfigurations = this.mergeDescriptorToConfigurations(configurations, this.createServicesStackDescriptorConfigs(kerberosDescriptor));
    recommendations.blueprint.configurations = mergedConfigurations.reduce(function (p, c) {
      p[c.type] = {};
      p[c.type].properties = c.properties;
      return p;
    }, {});

    if (this.get('isWithinAddService')) {
      this.get('content.masterComponentHosts').filterProperty('isInstalled', false).forEach(function (item) {
        var hostGroupName = blueprintUtils.getHostGroupByFqdn(recommendations, item.hostName);
        blueprintUtils.addComponentToHostGroup(recommendations, item.component, hostGroupName);
      }, this);
    }

    return recommendations;
  },

  /**
   * Returns map with appropriate action and properties to process with.
   * Key is an action e.g. `add`, `update`, `delete` and value is  an object `fileName` -> `propertyName`: `propertyValue`.
   *
   * @param {object} recommendedConfigurations
   * @param {object[]} servicesConfigurations services' configurations fetched from API
   * @param {App.ServiceConfigProperty[]} allConfigs all current configurations stored in controller, basically kerberos descriptor
   * @returns {object}
   */
  groupRecommendationProperties: function groupRecommendationProperties(recommendedConfigurations, servicesConfigurations, allConfigs) {
    var resultMap = {
      update: {},
      add: {},
      delete: {}
    };

    /**
     * Adds property to associated group `add`,`delete`,`update`.
     *
     * @param {object} propertyMap <code>resultMap</code> object
     * @param {string} name property name
     * @param {string} propertyValue property value
     * @param {string} fileName property file name
     * @return {object} <code>resultMap</code>
     * @param {string} group, `add`,`delete`,`update`
     */
    var addProperty = function addProperty(propertyMap, name, propertyValue, fileName, group) {
      var ret = $.extend(true, {}, propertyMap);
      if (ret.hasOwnProperty(group)) {
        if (!ret[group].hasOwnProperty(fileName)) {
          ret[group][fileName] = {};
        }
        ret[group][fileName][name] = propertyValue;
      }
      return ret;
    };

    return Em.keys(recommendedConfigurations || {}).reduce(function (acc, fileName) {
      var propertyMap = acc;
      var recommendedProperties = Em.getWithDefault(recommendedConfigurations, fileName + '.properties', {});
      var recommendedAttributes = Em.getWithDefault(recommendedConfigurations, fileName + '.property_attributes', {});
      // check for properties that should be delted
      Em.keys(recommendedAttributes).forEach(function (propertyName) {
        var attribute = recommendedAttributes[propertyName];
        // delete properties which are present in kerberos descriptor
        if (attribute.hasOwnProperty('delete') && allConfigs.filterProperty('filename', fileName).someProperty('name', propertyName)) {
          propertyMap = addProperty(propertyMap, propertyName, '', fileName, 'delete');
        }
      });

      return Em.keys(recommendedProperties).reduce(function (a, propertyName) {
        var propertyValue = recommendedProperties[propertyName];
        // check if property exist in saved configurations on server
        var isExist = Em.getWithDefault(servicesConfigurations.findProperty('type', fileName) || {}, 'properties', {}).hasOwnProperty(propertyName);
        if (!isExist) {
          return addProperty(a, propertyName, propertyValue, fileName, 'add');
        }
        // when property exist check that it present in current step configs (kerberos descriptor)
        // and add it as property to `update`
        if (allConfigs.filterProperty('filename', fileName).someProperty('name', propertyName)) {
          return addProperty(a, propertyName, propertyValue, fileName, 'update');
        }
        return a;
      }, propertyMap);
    }, resultMap);
  },

  /**
   *
   * @method getServiceByFilename
   * @param {string}fileName
   * @returns {string}
   */
  getServiceByFilename: function getServiceByFilename(fileName) {
    // core-site properties goes to HDFS
    if (fileName === 'core-site' && App.Service.find().someProperty('serviceName', 'HDFS')) {
      return 'HDFS';
    }
    var associatedService = App.StackService.find().filter(function (service) {
      return Em.keys(service.get('configTypes')).contains(fileName);
    })[0];
    return associatedService ? associatedService.get('serviceName') : '';
  },

  loadServerSideConfigsRecommendations: function loadServerSideConfigsRecommendations(recommendations) {
    return App.ajax.send({
      'name': 'config.recommendations',
      'sender': this,
      'data': {
        stackVersionUrl: App.get('stackVersionURL'),
        dataToSend: {
          recommend: 'configurations',
          hosts: App.get('allHostNames'),
          services: this.get('serviceNames'),
          recommendations: recommendations
        }
      },
      'success': 'loadRecommendationsSuccess',
      'error': 'loadRecommendationsError'
    });
  },

  loadRecommendationsError: function loadRecommendationsError(req, ajaxOpts, error, opt) {
    var resp;
    try {
      resp = $.parseJSON(req.responseText);
    } catch (e) {}
    return App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      bodyClass: App.AjaxDefaultErrorPopupBodyView.extend({
        type: opt.type || 'GET',
        url: opt.url,
        status: req.status,
        message: resp && resp.message || req.responseText
      })
    });
  },

  /**
   * Callback executed when all configs specified by tags are loaded.
   * Here we handle configurations for instlled services and Kerberos.
   * Gather needed info for recommendation request such as configurations object.
   *
   * @override
   */
  getConfigTagsSuccess: function getConfigTagsSuccess(data) {
    // here we get all installed services including KERBEROS
    var serviceNames = App.Service.find().mapProperty('serviceName').concat(['KERBEROS']).uniq();
    // collect all config types for selected services
    var installedServiceSites = Array.prototype.concat.apply([], App.config.get('preDefinedServiceConfigs').filter(function (serviceConfig) {
      return serviceNames.contains(Em.get(serviceConfig, 'serviceName'));
    }).map(function (service) {
      // when service have no configs return <code>null</code> instead return config types
      if (!service.get('configTypes')) return null;
      return Object.keys(service.get('configTypes'));
    }, this).compact()).uniq(); // cleanup <code>null</code>

    // take all configs for selected services by config types recieved from API response
    var serviceConfigTags = Em.keys(data.Clusters.desired_configs).reduce(function (tags, site) {
      if (data.Clusters.desired_configs.hasOwnProperty(site)) {
        // push cluster-env.xml also since it not associated with any service but need to further processing
        if (installedServiceSites.contains(site) || site === 'cluster-env') {
          tags.push({
            siteName: site,
            tagName: data.Clusters.desired_configs[site].tag,
            newTagName: null
          });
        }
      }
      return tags;
    }, []);
    // store configurations
    this.set('serviceConfigTags', serviceConfigTags);
    this.set('isAppliedConfigLoaded', true);
  },

  /**
   * Add/update property in `properties` object for each config type with
   * associated kerberos descriptor config value.
   *
   * @private
   * @param {object[]} configurations
   * @param {App.ServiceConfigProperty[]} kerberosDescriptor
   * @returns {object[]}
   */
  mergeDescriptorToConfigurations: function mergeDescriptorToConfigurations(configurations, kerberosDescriptor) {
    return configurations.map(function (configType) {
      var properties = $.extend({}, configType.properties);
      var filteredDescriptor = kerberosDescriptor.filterProperty('filename', configType.type);
      if (filteredDescriptor.length) {
        filteredDescriptor.forEach(function (descriptorConfig) {
          var configName = Em.get(descriptorConfig, 'name');
          properties[configName] = Em.get(descriptorConfig, 'value');
        });
      }
      return {
        type: configType.type,
        version: configType.version,
        tag: configType.tag,
        properties: properties
      };
    });
  },

  postKerberosDescriptor: function postKerberosDescriptor(kerberosDescriptor) {
    return App.ajax.send({
      name: 'admin.kerberos.cluster.artifact.create',
      sender: this,
      data: {
        artifactName: 'kerberos_descriptor',
        data: {
          artifact_data: this.removeIdentityReferences(kerberosDescriptor)
        }
      }
    });
  },

  /**
   * Send request to update kerberos descriptor
   * @param kerberosDescriptor
   * @returns {$.ajax|*}
   */
  putKerberosDescriptor: function putKerberosDescriptor(kerberosDescriptor) {
    return App.ajax.send({
      name: 'admin.kerberos.cluster.artifact.update',
      sender: this,
      data: {
        artifactName: 'kerberos_descriptor',
        data: {
          artifact_data: this.removeIdentityReferences(kerberosDescriptor)
        }
      },
      success: 'unkerberizeCluster',
      error: 'unkerberizeCluster'
    });
  },

  /**
   * The UI should ignore Kerberos identity references
   * when setting the user-supplied Kerberos descriptor
   * @param {object} kerberosDescriptor
   * @returns {object}
   */
  removeIdentityReferences: function removeIdentityReferences(kerberosDescriptor) {
    var notReference = function notReference(identity) {
      return Em.isNone(identity.reference) && !identity.name.startsWith('/');
    };
    kerberosDescriptor.services.forEach(function (service) {
      if (service.identities) {
        service.identities = service.identities.filter(notReference);
      }
      if (service.components) {
        service.components.forEach(function (component) {
          if (component.identities) {
            component.identities = component.identities.filter(notReference);
          }
        });
      }
    });
    return kerberosDescriptor;
  }

});

});

require.register("mixins/wizard/assign_master_components", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('mixins/common/hosts/host_component_recommendation_mixin');
require('mixins/common/hosts/host_component_validation_mixin');

var App = require('app');
var blueprintUtils = require('utils/blueprint');
var numberUtils = require('utils/number_utils');
var validationUtils = require('utils/validator');

/**
 * Mixin for assign master-to-host step in wizards
 * Implements basic logic of assign masters page
 * Should be used with controller linked with App.AssignMasterComponentsView
 * @type {Ember.Mixin}
 */
App.AssignMasterComponents = Em.Mixin.create(App.HostComponentValidationMixin, App.HostComponentRecommendationMixin, {

  /**
   * Array of master component names to show on the page
   * By default is empty, this means that masters of all selected services should be shown
   * @type {Array}
   */
  mastersToShow: [],

  /**
   * Array of master component names to show on the service config assign master page
   * @type {Array}
   */
  mastersToCreate: [],

  /**
   * Array of master component names to add for install
   * @type {Array}
   */
  mastersToAdd: [],

  /**
   * Array of master component names, that are already installed, but should have ability to change host
   * @type {Array}
   */
  mastersToMove: [],

  /**
   * Array of master component names, that should be addable
   * Are used in HA wizards to add components, that are not addable for other wizards
   * @type {Array}
   */
  mastersAddableInHA: [],

  /**
   * Array of master component names to show 'Current' prefix in label before component name
   * Prefix will be shown only for installed instances
   * @type {Array}
   */
  showCurrentPrefix: [],

  /**
   * Array of master component names to show 'Additional' prefix in label before component name
   * Prefix will be shown only for not installed instances
   * @type {Array}
   */
  showAdditionalPrefix: [],

  /**
   * Array of objects with label and host keys to show specific hosts on the page
   * @type {Array}
   * format:
   * [
   *   {
   *     label: 'Current',
   *     host: 'c6401.ambari.apache.org'
   *   },
   *   {
   *     label: 'Additional',
   *     host: function () {
   *       return 'c6402.ambari.apache.org';
   *     }.property()
   *   }
   * ]
   */
  additionalHostsList: [],

  /**
   * Define whether show already installed masters first
   * @type {Boolean}
   */
  showInstalledMastersFirst: false,

  /**
   * Map of component name to list of hostnames for that component
   * format:
   * {
   *   NAMENODE: [
   *     'c6401.ambari.apache.org'
   *   ],
   *   DATANODE: [
   *     'c6402.ambari.apache.org',
   *     'c6403.ambari.apache.org',
   *   ]
   * }
   * @type {Object}
   */
  recommendedHostsForComponents: {},

  recommendations: null,

  markSavedComponentsAsInstalled: false,

  /**
   * @type {boolean}
   * @default false
   */
  validationInProgress: false,

  /**
   * run validation call which was skipped
   * validation should be always ran after last change
   * @type {boolean}
   * @default false
   */
  runQueuedValidation: false,

  /**
   * Array of <code>servicesMasters</code> objects, that will be shown on the page
   * Are filtered using <code>mastersToShow</code>
   * @type {Array}
   */
  servicesMastersToShow: function () {
    var mastersToShow = this.get('mastersToShow');
    var servicesMasters = this.get('servicesMasters');
    var result = [];
    if (!mastersToShow.length) {
      result = servicesMasters;
    } else {
      mastersToShow.forEach(function (master) {
        result = result.concat(servicesMasters.filterProperty('component_name', master));
      });
    }

    if (this.get('showInstalledMastersFirst')) {
      result = this.sortMasterComponents(result);
    }

    return result;
  }.property('servicesMasters.length', 'mastersToShow.length', 'showInstalledMastersFirst'),

  /**
   * Sort masters, installed masters will be first.
   * @param masters
   * @returns {Array}
   */
  sortMasterComponents: function sortMasterComponents(masters) {
    return [].concat(masters.filterProperty('isInstalled'), masters.filterProperty('isInstalled', false));
  },

  /**
   * Check if <code>installerWizard</code> used
   * @type {bool}
   */
  isInstallerWizard: Em.computed.equal('content.controllerName', 'installerController'),

  /**
   * Master components which could be assigned to multiple hosts
   * @type {string[]}
   */
  multipleComponents: Em.computed.alias('App.components.multipleMasters'),

  /**
   * Master components which could be assigned to multiple hosts
   * @type {string[]}
   */
  addableComponents: function () {
    return App.get('components.addableMasterInstallerWizard').concat(this.get('mastersAddableInHA')).uniq();
  }.property('App.components.addableMasterInstallerWizard', 'mastersAddableInHA'),

  /**
   * Define state for submit button
   * @type {bool}
   */
  submitDisabled: false,

  /**
   * Is Submit-click processing now
   * @type {bool}
   */
  submitButtonClicked: false,

  /**
   * Either use or not use server validation in this controller
   * @type {bool}
   */
  useServerValidation: true,

  /**
   * Trigger for executing host names check for components
   * Should de "triggered" when host changed for some component and when new multiple component is added/removed
   * @type {bool}
   */
  hostNameCheckTrigger: false,

  /**
   * List of hosts
   * @type {Array}
   */
  hosts: [],

  /**
   * Name of multiple component which host name was changed last
   * @type {Object|null}
   */
  componentToRebalance: null,

  /**
   * Name of component which host was changed last
   * @type {string}
   */
  lastChangedComponent: null,

  /**
   * Flag for rebalance multiple components
   * @type {number}
   */
  rebalanceComponentHostsCounter: 0,

  /**
   * @type {Ember.Enumerable}
   */
  servicesMasters: [],

  /**
   * @type {Ember.Enumerable}
   */
  selectedServicesMasters: [],

  /**
   * Is hosts data loaded
   * @type {bool}
   */
  isHostsLoaded: false,

  /**
   * Are recommendations loaded
   * @type {bool}
   */
  isRecommendationsLoaded: false,

  /**
   * Is data for current step loaded
   * @type {bool}
   */
  isLoaded: Em.computed.and('isHostsLoaded', 'isRecommendationsLoaded'),

  /**
   * Is back from the next step
   * @type {bool}
   */
  backFromNextStep: false,

  /**
   * Validation error messages which don't related with any master
   */
  generalErrorMessages: [],

  /**
   * Validation warning messages which don't related with any master
   */
  generalWarningMessages: [],

  /**
   * Is masters-hosts layout initial one
   * @type {bool}
   */
  isInitialLayout: true,

  /**
   * true if any error exists
   */
  anyError: function () {
    return this.get('servicesMasters').some(function (m) {
      return m.get('errorMessage');
    }) || this.get('generalErrorMessages').some(function (m) {
      return m;
    });
  }.property('servicesMasters.@each.errorMessage', 'generalErrorMessages'),

  /**
   * true if any warning exists
   */
  anyWarning: function () {
    return this.get('servicesMasters').some(function (m) {
      return m.get('warnMessage');
    }) || this.get('generalWarningMessages').some(function (m) {
      return m;
    });
  }.property('servicesMasters.@each.warnMessage', 'generalWarningMessages'),

  /**
   * Clear loaded recommendations
   */
  clearRecommendations: function clearRecommendations() {
    if (this.get('content.recommendations')) {
      this.set('content.recommendations', null);
    }
    if (this.get('recommendations')) {
      this.set('recommendations', null);
    }
  },

  /**
   * List of host with assigned masters
   * Format:
   * <code>
   *   [
   *     {
   *       host_name: '',
   *       hostInfo: {},
   *       masterServices: [],
   *       masterServicesToDisplay: [] // used only in template
   *    },
   *    ....
   *   ]
   * </code>
   * @type {Ember.Enumerable}
   */
  masterHostMapping: function () {
    var mapping = [],
        mappingObject,
        mappedHosts,
        hostObj;
    //get the unique assigned hosts and find the master services assigned to them
    mappedHosts = this.get("selectedServicesMasters").mapProperty("selectedHost").uniq();
    mappedHosts.forEach(function (item) {
      hostObj = this.get("hosts").findProperty("host_name", item);
      // User may input invalid host name (this is handled in hostname checker). Here we just skip it
      if (!hostObj) return;
      var masterServices = this.get("selectedServicesMasters").filterProperty("selectedHost", item),
          masterServicesToDisplay = [];
      masterServices.mapProperty('display_name').uniq().forEach(function (n) {
        masterServicesToDisplay.pushObject(masterServices.findProperty('display_name', n));
      });
      mappingObject = Em.Object.create({
        host_name: item,
        hostInfo: hostObj.host_info,
        masterServices: masterServices,
        masterServicesToDisplay: masterServicesToDisplay
      });

      mapping.pushObject(mappingObject);
    }, this);

    return mapping.sortProperty('host_name');
  }.property('selectedServicesMasters.@each.selectedHost', 'selectedServicesMasters.@each.isHostNameValid', 'isLoaded'),

  /**
   * Count of hosts without masters
   * @type {number}
   */
  remainingHosts: function () {
    if (this.get('content.controllerName') === 'installerController') {
      return 0;
    } else {
      return this.get("hosts.length") - this.get("masterHostMapping.length");
    }
  }.property('masterHostMapping.length', 'selectedServicesMasters.@each.selectedHost'),

  /**
   * Update submit button status
   * @method updateIsSubmitDisabled
   */
  updateIsSubmitDisabled: function () {

    if (this.thereIsNoMasters()) {
      return false;
    }

    var isSubmitDisabled = this.get('servicesMasters').someProperty('isHostNameValid', false);

    if (this.get('useServerValidation')) {
      this.set('submitDisabled', true);

      if (this.get('servicesMasters').length === 0) {
        return;
      }

      if (!isSubmitDisabled) {
        if (!this.get('isInitialLayout')) {
          this.clearRecommendations(); // reset previous recommendations
        } else {
          this.set('isInitialLayout', false);
        }
        this.recommendAndValidate();
      }
    } else {
      isSubmitDisabled = isSubmitDisabled || !this.customClientSideValidation();
      this.set('submitDisabled', isSubmitDisabled);
      return isSubmitDisabled;
    }
  }.observes('servicesMasters.@each.selectedHost'),

  /**
   * Function to validate master-to-host assignments
   * Should be defined in controller
   * @returns {boolean}
   */
  customClientSideValidation: function customClientSideValidation() {
    return true;
  },

  /**
   * Success-callback for validations request
   * @param {object} data
   * @method updateValidationsSuccessCallback
   */
  updateValidationsSuccessCallback: function updateValidationsSuccessCallback(data) {
    var self = this;

    var generalErrorMessages = [];
    var generalWarningMessages = [];
    this.get('servicesMasters').setEach('warnMessage', null);
    this.get('servicesMasters').setEach('errorMessage', null);
    var anyErrors = false;

    var validationData = validationUtils.filterNotInstalledComponents(data);
    validationData.filterProperty('type', 'host-component').forEach(function (item) {
      var master = self.get('servicesMasters').find(function (m) {
        return m.component_name === item['component-name'] && m.selectedHost === item.host;
      });
      if (master) {
        if (item.level === 'ERROR') {
          anyErrors = true;
          master.set('errorMessage', item.message);
        } else if (item.level === 'WARN') {
          master.set('warnMessage', item.message);
        }
      }
    });

    this.set('generalErrorMessages', generalErrorMessages);
    this.set('generalWarningMessages', generalWarningMessages);

    // use this.set('submitDisabled', anyErrors); is validation results should block next button
    // It's because showValidationIssuesAcceptBox allow use accept validation issues and continue
    this.set('submitDisabled', false); //this.set('submitDisabled', anyErrors);
  },

  /**
   * Error-callback for validations request
   * @param {object} jqXHR
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @method updateValidationsErrorCallback
   */
  updateValidationsErrorCallback: function updateValidationsErrorCallback(jqXHR, ajaxOptions, error, opt) {},

  /**
   * @override App.HostComponentRecommendationMixin
   */
  getRecommendationRequestData: function getRecommendationRequestData(options) {
    var res = this._super(options);
    res.services = this.getCurrentServiceNames();
    if (!this.get('isInstallerWizard')) {
      res.recommendations = this.getCurrentMasterSlaveBlueprint();
    }
    return res;
  },

  /**
   * @override App.HostComponentRecommendationMixin
   */
  getHostComponentValidationParams: function getHostComponentValidationParams(options) {
    var res = this._super(options);
    res.services = this.getCurrentServiceNames();
    return res;
  },

  /**
   * Returns selected and installed service names
   * @return {string[]}
   */
  getCurrentServiceNames: function getCurrentServiceNames() {
    return App.StackService.find().filter(function (i) {
      return i.get('isSelected') || i.get('isInstalled');
    }).mapProperty('serviceName').uniq();
  },

  /**
   * Clear controller data (hosts, masters etc)
   * @method clearStep
   */
  clearStep: function clearStep() {
    this.setProperties({
      hosts: [],
      isHostsLoaded: false,
      isRecommendationsLoaded: false,
      backFromNextStep: false,
      selectedServicesMasters: [],
      servicesMasters: []
    });
    App.StackServiceComponent.find().forEach(function (stackComponent) {
      stackComponent.set('serviceComponentId', 1);
    }, this);
  },

  clearStepOnExit: function clearStepOnExit() {
    this.clearStep();
  },

  /**
   * Load controller data (hosts, host components etc)
   * @method loadStep
   */
  loadStep: function loadStep() {
    var self = this;
    this.clearStep();
    if (this._additionalClearSteps) {
      this._additionalClearSteps();
    }
    this.renderHostInfo().done(function () {
      //when returning from step Assign Slaves and Clients, recommendations are already available
      //set the flag so that recommendations AJAX call is not made unnecessarily
      if (self.get('recommendations')) {
        self.set('backFromNextStep', true);
      }
      self.getRecommendedHosts({
        hosts: self.getHosts()
      }).then(function () {
        self.loadStepCallback(self.createComponentInstallationObjects(), self);
      });
    });
  },

  /**
   * Callback after load controller data (hosts, host components etc)
   * @method loadStepCallback
   */
  loadStepCallback: function loadStepCallback(components, self) {
    self.renderComponents(components);

    self.get('addableComponents').forEach(function (componentName) {
      self.updateComponent(componentName);
    }, self);
    self.set('isRecommendationsLoaded', true);
    if (self.thereIsNoMasters() && !self.get('mastersToCreate').length) {
      App.router.send('next');
    }
  },

  /**
   * Returns true if there is no new master components which need assigment to host
   */
  thereIsNoMasters: function thereIsNoMasters() {
    return !this.get("selectedServicesMasters").filterProperty('isInstalled', false).length;
  },

  /**
   * Used to set showAddControl flag for installer wizard
   * @method updateComponent
   */
  updateComponent: function updateComponent(componentName) {
    var component = this.last(componentName);
    if (!component) {
      return;
    }

    var showControl = !App.StackServiceComponent.find().findProperty('componentName', componentName).get('stackService').get('isInstalled') || this.get('mastersAddableInHA').contains(componentName);

    if (showControl) {
      var mastersLength = this.get("selectedServicesMasters").filterProperty("component_name", componentName).length;
      if (mastersLength < this.getMaxNumberOfMasters(componentName)) {
        component.set('showAddControl', true);
      } else {
        component.set('showRemoveControl', mastersLength != 1);
      }
    }
  },

  /**
   * Count max number of instances for masters <code>componentName</code>, according to their cardinality and number of hosts
   * @param componentName
   * @returns {Number}
   */
  getMaxNumberOfMasters: function getMaxNumberOfMasters(componentName) {
    var maxByCardinality = App.StackServiceComponent.find().findProperty('componentName', componentName).get('maxToInstall');
    var hostsNumber = this.get("hosts.length");
    return Math.min(maxByCardinality, hostsNumber);
  },

  /**
   * Load active host list to <code>hosts</code> variable
   * @method renderHostInfo
   */
  renderHostInfo: function renderHostInfo() {
    var self = this;
    var isInstaller = this.get('wizardController.name') === 'installerController' || this.get('content.controllerName') === 'installerController';
    return App.ajax.send({
      name: isInstaller ? 'hosts.info.install' : 'hosts.high_availability.wizard',
      sender: this,
      data: {
        hostNames: isInstaller ? this.getHosts().join() : null
      }
    }).success(function (data) {
      self.loadWizardHostsSuccessCallback(data);
    });
  },

  loadWizardHostsSuccessCallback: function loadWizardHostsSuccessCallback(data) {
    var hostInfo = this.get('content.hosts'),
        result = [];
    data.items.forEach(function (host) {
      var hostName = host.Hosts.host_name,
          _host = hostInfo[hostName],
          cpu = host.Hosts.cpu_count,
          memory = host.Hosts.total_mem.toFixed(2);
      if (_host.bootStatus === 'REGISTERED') {
        result.push(Em.Object.create({
          host_name: hostName,
          cpu: cpu,
          memory: memory,
          disk_info: host.Hosts.disk_info,
          maintenance_state: host.Hosts.maintenance_state,
          isInstalled: _host.isInstalled,
          host_info: Em.I18n.t('installer.step5.hostInfo').fmt(hostName, numberUtils.bytesToSize(memory, 1, 'parseFloat', 1024), cpu)
        }));
      }
    }, this);
    this.set('hosts', result);
    this.sortHosts(this.get('hosts'));
    this.set('isHostsLoaded', true);
  },

  /**
   * Sort list of host-objects by properties (memory - desc, cpu - desc, hostname - asc)
   * @param {object[]} hosts
   */
  sortHosts: function sortHosts(hosts) {
    hosts.sort(function (a, b) {
      if (a.get('memory') == b.get('memory')) {
        if (a.get('cpu') == b.get('cpu')) {
          return a.get('host_name').localeCompare(b.get('host_name')); // hostname asc
        }
        return b.get('cpu') - a.get('cpu'); // cores desc
      }
      return b.get('memory') - a.get('memory'); // ram desc
    });
  },

  /**
   * Get recommendations info from API
   * @param {object} recommendationBlueprint
   * @method loadComponentsRecommendationsFromServer
   * @override App.HostComponentRecommendationMixin
   */
  loadComponentsRecommendationsFromServer: function loadComponentsRecommendationsFromServer(recommendationBlueprint) {
    var self = this;

    //when returning from step Assign Slaves and Clients, backFromNextStep will be true
    if (this.get('recommendations') && this.get('backFromNextStep')) {
      // Don't do AJAX call if recommendations has been already received
      // But if user returns to previous step (selecting services), stored recommendations will be cleared in routers' next handler and AJAX call will be made again
      return $.Deferred().resolve().promise();
    } else {
      return this._super(recommendationBlueprint);
    }
  },

  /**
   * Create components for displaying component-host comboboxes in UI assign dialog
   * expects content.recommendations will be filled with recommendations API call result
   * @return {Object[]}
   */
  createComponentInstallationObjects: function createComponentInstallationObjects() {
    var stackMasterComponentsMap = {},
        masterHosts = this.get('content.masterComponentHosts') || this.get('masterComponentHosts'),
        //saved to local storage info
    servicesToAdd = (this.get('content.services') || []).filterProperty('isSelected').filterProperty('isInstalled', false).mapProperty('serviceName'),
        recommendations = this.get('recommendations'),
        resultComponents = [],
        multipleComponentHasBeenAdded = {},
        hostGroupsMap = {};

    App.StackServiceComponent.find().forEach(function (component) {
      var isMasterCreateOnConfig = this.get('mastersToCreate').contains(component.get('componentName'));
      if (this.get('isInstallerWizard') && (component.get('isShownOnInstallerAssignMasterPage') || isMasterCreateOnConfig)) {
        stackMasterComponentsMap[component.get('componentName')] = component;
      } else if (component.get('isShownOnAddServiceAssignMasterPage') || this.get('mastersToShow').contains(component.get('componentName')) || isMasterCreateOnConfig) {
        stackMasterComponentsMap[component.get('componentName')] = component;
      }
    }, this);

    recommendations.blueprint_cluster_binding.host_groups.forEach(function (group) {
      hostGroupsMap[group.name] = group;
    });

    recommendations.blueprint.host_groups.forEach(function (host_group) {
      var hosts = hostGroupsMap[host_group.name] ? hostGroupsMap[host_group.name].hosts : [];

      hosts.forEach(function (host) {
        host_group.components.forEach(function (component) {
          var willBeDisplayed = true;
          var stackMasterComponent = stackMasterComponentsMap[component.name];
          if (stackMasterComponent) {
            var isMasterCreateOnConfig = this.get('mastersToCreate').contains(component.name);
            // If service is already installed and not being added as a new service then render on UI only those master components
            // that have already installed hostComponents.
            // NOTE: On upgrade there might be a prior installed service with non-installed newly introduced serviceComponent
            if (!servicesToAdd.contains(stackMasterComponent.get('serviceName')) && !isMasterCreateOnConfig) {
              willBeDisplayed = masterHosts.someProperty('component', component.name);
            }

            if (willBeDisplayed) {
              var savedComponents = masterHosts.filterProperty('component', component.name);

              if (this.get('multipleComponents').contains(component.name) && savedComponents.length > 0) {
                if (!multipleComponentHasBeenAdded[component.name]) {
                  multipleComponentHasBeenAdded[component.name] = true;

                  savedComponents.forEach(function (saved) {
                    resultComponents.push(this.createComponentInstallationObject(stackMasterComponent, host.fqdn.toLowerCase(), saved));
                  }, this);
                }
              } else {
                var savedComponent = masterHosts.findProperty('component', component.name);
                resultComponents.push(this.createComponentInstallationObject(stackMasterComponent, host.fqdn.toLowerCase(), savedComponent));
              }
            }
          }
        }, this);
      }, this);
    }, this);
    return resultComponents;
  },

  /**
   * Create component for displaying component-host comboboxes in UI assign dialog
   * @param fullComponent - full component description
   * @param hostName - host fqdn where component will be installed
   * @param savedComponent - the same object which function returns but created before
   * @return {Object}
   */
  createComponentInstallationObject: function createComponentInstallationObject(fullComponent, hostName, savedComponent) {
    var componentName = fullComponent.get('componentName'),
        resultingHostName = savedComponent ? savedComponent.hostName : hostName;
    var componentObj = {};
    componentObj.component_name = componentName;
    componentObj.display_name = App.format.role(fullComponent.get('componentName'), false);
    componentObj.serviceId = fullComponent.get('serviceName');
    componentObj.isServiceCoHost = App.StackServiceComponent.find().findProperty('componentName', componentName).get('isCoHostedComponent') && !this.get('mastersToMove').contains(componentName);
    componentObj.selectedHost = resultingHostName;
    componentObj.isInstalled = savedComponent ? savedComponent.isInstalled || this.get('markSavedComponentsAsInstalled') && !this.get('mastersToCreate').contains(fullComponent.get('componentName')) : false;
    if (this.get('content.controllerName') === 'reassignMasterController' && componentName === 'NAMENODE' && App.get('hasNameNodeFederation')) {
      componentObj.nameSpace = App.HostComponent.find(componentName + '_' + resultingHostName).get('haNameSpace');
    }
    return componentObj;
  },

  /**
   * Success-callback for recommendations request
   * @param {object} data
   * @method loadRecommendationsSuccessCallback
   */
  loadRecommendationsSuccessCallback: function loadRecommendationsSuccessCallback(data) {
    var recommendations = data.resources[0].recommendations;
    this.set('recommendations', recommendations);
    if (this.get('content.controllerName')) {
      this.set('content.recommendations', recommendations);
    }

    var recommendedHostsForComponent = {};
    var hostsForHostGroup = {};

    recommendations.blueprint_cluster_binding.host_groups.forEach(function (hostGroup) {
      hostsForHostGroup[hostGroup.name] = hostGroup.hosts.mapProperty('fqdn');
    });

    recommendations.blueprint.host_groups.forEach(function (hostGroup) {
      var components = hostGroup.components.mapProperty('name');
      components.forEach(function (componentName) {
        var hostList = recommendedHostsForComponent[componentName] || [];
        var hostNames = hostsForHostGroup[hostGroup.name] || [];
        hostList.pushObjects(hostNames);
        recommendedHostsForComponent[componentName] = hostList;
      });
    });

    this.set('recommendedHostsForComponents', recommendedHostsForComponent);
    if (this.get('content.controllerName')) {
      this.set('content.recommendedHostsForComponents', recommendedHostsForComponent);
    }
  },

  /**
   * Error-callback for recommendations request
   * @param {object} jqXHR
   * @param {object} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @method loadRecommendationsErrorCallback
   */
  loadRecommendationsErrorCallback: function loadRecommendationsErrorCallback(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
  },

  /**
   * Put master components to <code>selectedServicesMasters</code>, which will be automatically rendered in template
   * @param {Ember.Enumerable} masterComponents
   * @method renderComponents
   */
  renderComponents: function renderComponents(masterComponents) {
    var installedServices = App.StackService.find().filterProperty('isSelected').filterProperty('isInstalled', false).mapProperty('serviceName'); //list of shown services
    var result = [];
    var serviceComponentId, previousComponentName;

    this.addNewMasters(masterComponents);

    masterComponents.forEach(function (item) {
      var masterComponent = App.StackServiceComponent.find().findProperty('componentName', item.component_name);
      var componentObj = Em.Object.create(item, item.nameSpace ? {
        allMasters: result,
        /**
         * Namespace of NameNode for enabled HDFS federation.
         * If a new host is assigned to component, looking for other NameNodes
         * to find which namespace has a 'free' host after this assignment.
         * NameNodes with new host assigned are excluded from this process
         * since moving more than one component at once is not allowed.
         */
        nameSpace: function () {
          var _this = this;

          var hostComponent = App.HostComponent.find(this.get('component_name') + '_' + this.get('selectedHost'));
          if (hostComponent.get('isLoaded')) {
            return hostComponent.get('haNameSpace');
          } else {
            var nameSpacesCounts = {};
            var allNameSpaces = this.get('allMasters').filter(function (masterComponent) {
              return masterComponent.get('serviceComponentId') !== _this.get('serviceComponentId') && App.HostComponent.find(masterComponent.get('component_name') + '_' + masterComponent.get('selectedHost')).get('isLoaded') && masterComponent.get('nameSpace');
            }).mapProperty('nameSpace');
            allNameSpaces.forEach(function (nameSpace) {
              var currentCount = nameSpacesCounts[nameSpace];
              nameSpacesCounts[nameSpace] = currentCount ? currentCount + 1 : 1;
            });
            var nameSpacesWithMissingHost = Object.keys(nameSpacesCounts).filter(function (key) {
              return nameSpacesCounts[key] === 1;
            });
            if (nameSpacesWithMissingHost.length === 1) {
              return nameSpacesWithMissingHost[0];
            }
          }
        }.property('allMasters.@each.selectedHost')
      } : {});
      var showRemoveControl;
      if (masterComponent.get('isMasterWithMultipleInstances')) {
        showRemoveControl = installedServices.contains(masterComponent.get('stackService.serviceName')) && masterComponents.filterProperty('component_name', item.component_name).length > 1;
        previousComponentName = item.component_name;
        componentObj.set('serviceComponentId', result.filterProperty('component_name', item.component_name).length + 1);
        componentObj.set("showRemoveControl", showRemoveControl);
      }
      componentObj.set('isHostNameValid', true);
      componentObj.set('showCurrentPrefix', this.get('showCurrentPrefix').contains(item.component_name) && item.isInstalled);
      componentObj.set('showAdditionalPrefix', this.get('showAdditionalPrefix').contains(item.component_name) && !item.isInstalled);
      if (this.get('mastersToMove').contains(item.component_name)) {
        componentObj.set('isInstalled', false);
      }

      result.push(componentObj);
    }, this);
    result = this.sortComponentsByServiceName(result);
    this.set("selectedServicesMasters", result);
    this.set('servicesMasters', result);
  },

  /**
   * Add new master components from <code>mastersToAdd</code> list
   * @param masterComponents
   * @returns {masterComponents[]}
   */
  addNewMasters: function addNewMasters(masterComponents) {
    this.get('mastersToAdd').forEach(function (masterName, index, mastersToAdd) {
      var toBeAddedNumber = mastersToAdd.filter(function (name) {
        return name === masterName;
      }).length,
          alreadyAddedNumber = masterComponents.filterProperty('component_name', masterName).rejectProperty('isInstalled').length;
      if (toBeAddedNumber > alreadyAddedNumber) {
        var hostName = this.getHostForMaster(masterName, masterComponents),
            serviceName = this.getServiceByMaster(masterName);
        masterComponents.push(this.createComponentInstallationObject(Em.Object.create({
          componentName: masterName,
          serviceName: serviceName
        }), hostName));
      }
    }, this);
    return masterComponents;
  },

  /**
   * Find available host for master and return it
   * If there is no available hosts returns false
   * @param master
   * @param allMasters
   * @returns {*}
   */
  getHostForMaster: function getHostForMaster(master, allMasters) {
    var masterHostList = [];

    allMasters.forEach(function (component) {
      if (component.component_name === master) {
        masterHostList.push(component.selectedHost);
      }
    });

    var recommendedHostsForMaster = this.get('recommendedHostsForComponents')[master] || [];
    for (var k = 0; k < recommendedHostsForMaster.length; k++) {
      if (!masterHostList.contains(recommendedHostsForMaster[k])) {
        return recommendedHostsForMaster[k];
      }
    }

    var usedHosts = allMasters.filterProperty('component_name', master).mapProperty('selectedHost');
    var allHosts = this.get('hosts');
    for (var i = 0; i < allHosts.length; i++) {
      if (!usedHosts.contains(allHosts[i].get('host_name'))) {
        return allHosts[i].get('host_name');
      }
    }

    return false;
  },

  /**
   * Find serviceName for master by it's componentName
   * @param master
   * @returns {*}
   */
  getServiceByMaster: function getServiceByMaster(master) {
    return App.StackServiceComponent.find().findProperty('componentName', master).get('serviceName');
  },

  /**
   * Sort components by their service (using <code>App.StackService.displayOrder</code>)
   * Services not in App.StackService.displayOrder are moved to the end of the list
   *
   * @param components
   * @returns {*}
   */
  sortComponentsByServiceName: function sortComponentsByServiceName(components) {
    var displayOrder = App.StackService.displayOrder;
    var componentsOrderForService = App.StackService.componentsOrderForService;
    var indexForUnordered = Math.max(displayOrder.length, components.length);
    return components.sort(function (a, b) {
      if (a.serviceId === b.serviceId && a.serviceId in componentsOrderForService) return componentsOrderForService[a.serviceId].indexOf(a.component_name) - componentsOrderForService[b.serviceId].indexOf(b.component_name);
      var aValue = displayOrder.indexOf(a.serviceId) != -1 ? displayOrder.indexOf(a.serviceId) : indexForUnordered;
      var bValue = displayOrder.indexOf(b.serviceId) != -1 ? displayOrder.indexOf(b.serviceId) : indexForUnordered;
      return aValue - bValue;
    });
  },
  /**
   * Update dependent co-hosted components according to the change in the component host
   * @method updateCoHosts
   */
  updateCoHosts: function () {
    var components = App.StackServiceComponent.find().filterProperty('isOtherComponentCoHosted');
    var selectedServicesMasters = this.get('selectedServicesMasters');
    components.forEach(function (component) {
      var componentName = component.get('componentName');
      var hostComponent = selectedServicesMasters.findProperty('component_name', componentName);
      var dependentCoHosts = component.get('coHostedComponents');
      dependentCoHosts.forEach(function (coHostedComponent) {
        var dependentHostComponent = selectedServicesMasters.findProperty('component_name', coHostedComponent);
        if (!this.get('mastersToMove').contains(coHostedComponent) && hostComponent && dependentHostComponent) dependentHostComponent.set('selectedHost', hostComponent.get('selectedHost'));
      }, this);
    }, this);
  }.observes('selectedServicesMasters.@each.selectedHost'),

  /**
   * On change callback for inputs
   * @param {string} componentName
   * @param {string} selectedHost
   * @param {number} serviceComponentId
   * @method assignHostToMaster
   */
  assignHostToMaster: function assignHostToMaster(componentName, selectedHost, serviceComponentId) {
    var flag = this.isHostNameValid(componentName, selectedHost);
    var component;
    this.updateIsHostNameValidFlag(componentName, serviceComponentId, flag);
    if (serviceComponentId) {
      component = this.get('selectedServicesMasters').filterProperty('component_name', componentName).findProperty("serviceComponentId", serviceComponentId);
      if (component) component.set("selectedHost", selectedHost);
    } else {
      this.get('selectedServicesMasters').findProperty("component_name", componentName).set("selectedHost", selectedHost);
    }
  },

  /**
   * Determines if hostName is valid for component:
   * <ul>
   *  <li>host name shouldn't be empty</li>
   *  <li>host should exist</li>
   *  <li>if host installed maintenance state should be 'OFF'</li>
   *  <li>host should have only one component with <code>componentName</code></li>
   * </ul>
   * @param {string} componentName
   * @param {string} selectedHost
   * @returns {boolean} true - valid, false - invalid
   * @method isHostNameValid
   */
  isHostNameValid: function isHostNameValid(componentName, selectedHost) {
    return selectedHost.trim() !== '' && this.get('hosts').filter(function (host) {
      return host.host_name === selectedHost && (!host.isInstalled || host.maintenance_state === 'OFF');
    }).length > 0 && this.get('selectedServicesMasters').filterProperty('component_name', componentName).mapProperty('selectedHost').filter(function (h) {
      return h === selectedHost;
    }).length <= 1;
  },

  /**
   * Update <code>isHostNameValid</code> property with <code>flag</code> value
   * for component with name <code>componentName</code> and
   * <code>serviceComponentId</code>-property equal to <code>serviceComponentId</code>-parameter value
   * @param {string} componentName
   * @param {number} serviceComponentId
   * @param {bool} flag
   * @method updateIsHostNameValidFlag
   */
  updateIsHostNameValidFlag: function updateIsHostNameValidFlag(componentName, serviceComponentId, flag) {
    var component;
    if (componentName) {
      if (serviceComponentId) {
        component = this.get('selectedServicesMasters').filterProperty('component_name', componentName).findProperty("serviceComponentId", serviceComponentId);
      } else {
        component = this.get('selectedServicesMasters').findProperty("component_name", componentName);
      }
      if (component) {
        component.set("isHostNameValid", flag);
        component.set("errorMessage", flag ? null : Em.I18n.t('installer.step5.error.host.invalid'));
      }
    }
  },

  /**
   * Returns last component of selected type
   * @param {string} componentName
   * @return {Em.Object|null}
   * @method last
   */
  last: function last(componentName) {
    return this.get("selectedServicesMasters").filterProperty("component_name", componentName).get("lastObject");
  },

  /**
   * Add new component to ZooKeeper Server and Hbase master
   * @param {string} componentName
   * @return {bool} true - added, false - not added
   * @method addComponent
   */
  addComponent: function addComponent(componentName) {
    /*
     * Logic: If ZooKeeper or Hbase service is selected then there can be
     * minimum 1 ZooKeeper or Hbase master in total, and
     * maximum 1 ZooKeeper or Hbase on every host
     */

    var maxNumMasters = this.getMaxNumberOfMasters(componentName),
        currentMasters = this.get("selectedServicesMasters").filterProperty("component_name", componentName).sortProperty('serviceComponentId'),
        newMaster = null,
        masterHosts = null,
        suggestedHost = null,
        i = 0,
        lastMaster = null;

    if (!currentMasters.length) {
      return false;
    }

    if (currentMasters.get("length") < maxNumMasters) {

      currentMasters.set("lastObject.showAddControl", false);
      currentMasters.set("lastObject.showRemoveControl", true);

      //create a new master component host based on an existing one
      newMaster = Em.Object.create({});
      lastMaster = currentMasters.get("lastObject");
      newMaster.set("display_name", lastMaster.get("display_name"));
      newMaster.set("component_name", lastMaster.get("component_name"));
      newMaster.set("selectedHost", lastMaster.get("selectedHost"));
      newMaster.set("serviceId", lastMaster.get("serviceId"));
      newMaster.set("isInstalled", false);
      newMaster.set('showAdditionalPrefix', this.get('showAdditionalPrefix').contains(lastMaster.get("component_name")));

      if (currentMasters.get("length") === maxNumMasters - 1) {
        newMaster.set("showAddControl", false);
      } else {
        newMaster.set("showAddControl", true);
      }
      newMaster.set("showRemoveControl", true);

      //get recommended host for the new Zookeeper server
      masterHosts = currentMasters.mapProperty("selectedHost").uniq();

      for (i = 0; i < this.get("hosts.length"); i++) {
        if (!masterHosts.contains(this.get("hosts")[i].get("host_name"))) {
          suggestedHost = this.get("hosts")[i].get("host_name");
          break;
        }
      }

      newMaster.set("selectedHost", suggestedHost);
      newMaster.set("serviceComponentId", currentMasters.get("lastObject.serviceComponentId") + 1);

      this.get("selectedServicesMasters").insertAt(this.get("selectedServicesMasters").indexOf(lastMaster) + 1, newMaster);

      this.setProperties({
        componentToRebalance: componentName,
        lastChangedComponent: componentName
      });
      this.incrementProperty('rebalanceComponentHostsCounter');
      this.toggleProperty('hostNameCheckTrigger');
      return true;
    }
    return false; //if no more zookeepers can be added
  },

  /**
   * Remove component from ZooKeeper server or Hbase Master
   * @param {string} componentName
   * @param {number} serviceComponentId
   * @return {bool} true - removed, false - no
   * @method removeComponent
   */
  removeComponent: function removeComponent(componentName, serviceComponentId) {
    var currentMasters = this.get("selectedServicesMasters").filterProperty("component_name", componentName);

    //work only if the multiple master service is selected in previous step
    if (currentMasters.length <= 1) {
      return false;
    }

    this.get("selectedServicesMasters").removeAt(this.get("selectedServicesMasters").indexOf(currentMasters.findProperty("serviceComponentId", serviceComponentId)));

    currentMasters = this.get("selectedServicesMasters").filterProperty("component_name", componentName);
    if (currentMasters.get("length") < this.getMaxNumberOfMasters(componentName)) {
      currentMasters.set("lastObject.showAddControl", true);
    }

    if (currentMasters.filterProperty('isInstalled', false).get("length") === 1) {
      currentMasters.set("lastObject.showRemoveControl", false);
    }

    this.setProperties({
      componentToRebalance: componentName,
      lastChangedComponent: componentName
    });
    this.incrementProperty('rebalanceComponentHostsCounter');
    this.toggleProperty('hostNameCheckTrigger');
    return true;
  },

  recommendAndValidate: function recommendAndValidate(callback) {
    var self = this,
        hostNames = this.getHosts();

    if (this.get('validationInProgress')) {
      this.set('runQueuedValidation', true);
      return;
    }

    this.set('validationInProgress', true);

    // load recommendations with partial request
    this.getRecommendedHosts({
      hosts: hostNames,
      components: this.getCurrentComponentHostMap()
    }).done(function () {
      self.validateSelectedHostComponents({
        hosts: hostNames,
        blueprint: self.get('recommendations')
      }).always(function () {
        if (callback) {
          callback();
        }
        self.set('validationInProgress', false);
        if (self.get('runQueuedValidation')) {
          self.set('runQueuedValidation', false);
          self.recommendAndValidate(callback);
        }
      });
    });
  },

  getCurrentComponentHostMap: function getCurrentComponentHostMap() {
    return this.get('masterHostMapping').reduce(function (acc, i) {
      var components = Em.getWithDefault(i, 'masterServices', []);
      var hostName = Em.get(i, 'host_name');
      components.forEach(function (i) {
        var componentName = Em.get(i, 'component_name');
        var component = acc.findProperty('componentName', componentName);
        if (component) {
          Em.set(component, 'hosts', Em.getWithDefault(component, 'hosts', []).concat(hostName).uniq());
          Em.set(component, 'size', Em.getWithDefault(component, 'hosts.length', 0));
        } else {
          acc.push({
            componentName: componentName,
            hosts: [hostName],
            size: 1
          });
        }
      });
      return acc;
    }, []);
  },

  _goNextStepIfValid: function _goNextStepIfValid() {
    if (!this.get('submitDisabled')) {
      App.router.send('next');
    } else {
      App.set('router.nextBtnClickInProgress', false);
    }
  },

  nextButtonDisabled: Em.computed.or('App.router.btnClickInProgress', 'submitDisabled', 'validationInProgress', '!isLoaded'),

  /**
   * Submit button click handler
   * Disable 'Next' button while it is already under process. (using Router's property 'nextBtnClickInProgress')
   * @method submit
   */
  submit: function submit() {
    var self = this;
    if (this.get('submitDisabled')) {
      return;
    }
    if (!this.get('submitButtonClicked') && !App.get('router.nextBtnClickInProgress')) {
      this.set('submitButtonClicked', true);
      App.router.set('nextBtnClickInProgress', true);

      if (this.get('useServerValidation')) {
        self.recommendAndValidate(function () {
          self.showValidationIssuesAcceptBox(self._goNextStepIfValid.bind(self));
        });
      } else {
        this.updateIsSubmitDisabled();
        this._goNextStepIfValid();
        this.set('submitButtonClicked', false);
      }
    }
  },

  /**
   * In case of any validation issues shows accept dialog box for user which allow cancel and fix issues or continue anyway
   * @method showValidationIssuesAcceptBox
   */
  showValidationIssuesAcceptBox: function showValidationIssuesAcceptBox(callback) {
    var self = this;

    // If there are no warnings and no errors, return
    if (!self.get('anyWarning') && !self.get('anyError')) {
      callback();
      self.set('submitButtonClicked', false);
      return;
    }

    App.ModalPopup.show({
      'data-qa': 'validation-issues-modal',
      primary: Em.I18n.t('common.continueAnyway'),
      header: Em.I18n.t('installer.step5.validationIssuesAttention.header'),
      body: Em.I18n.t('installer.step5.validationIssuesAttention'),
      onPrimary: function onPrimary() {
        this._super();
        callback();
        self.set('submitButtonClicked', false);
      },
      onSecondary: function onSecondary() {
        this._super();
        App.router.set('nextBtnClickInProgress', false);
        self.set('submitButtonClicked', false);
      },
      onClose: function onClose() {
        this._super();
        self.set('submitButtonClicked', false);
      }
    });
  },

  getHosts: function getHosts() {
    return Em.keys(this.get('content.hosts'));
  }
});

});

require.register("mixins/wizard/selectHost", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for host select view on step 5 install wizard
 * Implements base login for select, input types
 * Each view which use this mixin should redefine method <code>changeHandler</code> (with
 * observing <code>controller.hostNameCheckTrigger</code>)
 *
 * @type {Ember.Mixin}
 */
App.SelectHost = Em.Mixin.create({

  /**
   * Element of <code>controller.servicesMasters</code>
   * Binded from template
   * @type {object}
   */
  component: null,

  /**
   * List of avaiable host names
   * @type {string[]}
   */
  content: [],

  /**
   * Host component name
   * @type {string}
   */
  componentName: null,

  /**
   * Handler for selected value change
   * Triggers <code>changeHandler</code> execution
   * @method change
   */
  change: function change() {
    if ('destroyed' === this.get('state')) return;
    this.set('controller.lastChangedComponent', this.get('component.component_name'));
    this.get('controller').toggleProperty('hostNameCheckTrigger');
  },

  /**
   * Add or remove <code>error</code> class from parent div-element
   * @param {bool} flag true - add class, false - remove
   * @method updateErrorStatus
   */
  updateErrorStatus: function updateErrorStatus(flag) {
    var parentBlock = this.$().parent('div');
    /* istanbul ignore next */
    if (flag) {
      parentBlock.removeClass('error');
    } else {
      parentBlock.addClass('error');
    }
  },

  /**
   * If component is multiple (defined in <code>controller.multipleComponents</code>),
   * should update <code>controller.componentToRebalance</code> and trigger rebalancing
   * @method tryTriggerRebalanceForMultipleComponents
   */
  tryTriggerRebalanceForMultipleComponents: function tryTriggerRebalanceForMultipleComponents() {
    var componentIsMultiple = this.get('controller.multipleComponents').contains(this.get("component.component_name"));
    if (componentIsMultiple) {
      this.get('controller').set('componentToRebalance', this.get("component.component_name"));
      this.get('controller').incrementProperty('rebalanceComponentHostsCounter');
    }
  },

  /**
   * Handler for value changing
   * Should be redeclared in child views
   * Each redeclarion should contains code:
   * <code>
   *   if (!this.shouldChangeHandlerBeCalled()) return;
   * </code>
   * @method changeHandler
   */
  changeHandler: function () {}.observes('controller.hostNameCheckTrigger'),

  /**
   * If view is destroyed or not current component's host name was changed we should do nothing
   * This method should be called from <code>changeHandler</code>:
   * <code>
   *   changeHandler: function() {
   *     if (!this.shouldChangeHandlerBeCalled()) return;
   *     // your code
   *   }.observes(...)
   * </code>
   * @returns {boolean}
   * @method shouldChangeHandlerBeCalled
   */
  shouldChangeHandlerBeCalled: function shouldChangeHandlerBeCalled() {
    return !('destroyed' === this.get('state') || this.get('controller.lastChangedComponent') !== this.get("component.component_name"));
  },

  /**
   * If <code>component.isHostNameValid</code> was changed,
   * error status should be updated according to new value
   * @method isHostNameValidObs
   */
  isHostNameValidObs: function () {
    this.updateErrorStatus(this.get('component.isHostNameValid'));
  }.observes('component.isHostNameValid'),

  /**
   * Recalculate available hosts
   * This should be done only once per Ember loop
   * @method rebalanceComponentHosts
   */
  rebalanceComponentHosts: function () {
    Em.run.once(this, 'rebalanceComponentHostsOnce');
  }.observes('controller.rebalanceComponentHostsCounter'),

  /**
   * Recalculate available hosts
   * @method rebalanceComponentHostsOnce
   */
  rebalanceComponentHostsOnce: function rebalanceComponentHostsOnce() {
    if (this.get('component.component_name') === this.get('controller.componentToRebalance')) {
      this.initContent();
    }
  },

  /**
   * Get available hosts
   * multipleComponents component can be assigned to multiple hosts,
   * shared hosts among the same component should be filtered out
   * @return {string[]}
   * @method getAvailableHosts
   */
  getAvailableHosts: function getAvailableHosts() {
    var hosts = this.get('controller.hosts').slice(),
        componentName = this.get('component.component_name'),
        multipleComponents = this.get('controller.multipleComponents'),
        occupiedHosts = this.get('controller.selectedServicesMasters').filterProperty('component_name', componentName).mapProperty('selectedHost').without(this.get('component.selectedHost'));

    if (multipleComponents.contains(componentName)) {
      hosts = hosts.filter(function (host) {
        return !occupiedHosts.contains(host.get('host_name'));
      }, this);
    }
    return hosts.sortProperty('host_name');
  },

  /**
   * Extract hosts from controller,
   * filter out available to selection and
   * push them into form element content
   * @method initContent
   */
  initContent: function initContent() {
    this.set("content", this.getAvailableHosts());
  }

});

});

require.register("mixins/wizard/wizardDeployProgressController", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for wizard controller for showing command progress on wizard pages
 * This should
 * @type {Ember.Mixin}
 */
App.wizardDeployProgressControllerMixin = Em.Mixin.create({

  /**
   * Ajax-requests count
   * @type {number}
   */
  ajaxQueueLength: 0,

  /**
   * Ajax-requests queue
   * @type {App.ajaxQueue}
   */
  ajaxRequestsQueue: null,

  /**
   * This flag when turned to true launches deploy progress bar
   */
  isDeployStarted: '',

  /**
   * We need to do a lot of ajax calls async in special order. To do this,
   * generate array of ajax objects and then send requests step by step. All
   * ajax objects are stored in <code>ajaxRequestsQueue</code>
   *
   * @param {Object} params object with ajax-request parameters like url, type, data etc
   * @method addRequestToAjaxQueue
   */
  addRequestToAjaxQueue: function addRequestToAjaxQueue(params) {
    if (App.get('testMode')) return;

    params = jQuery.extend({
      sender: this,
      error: 'ajaxQueueRequestErrorCallback'
    }, params);
    params.data.cluster = this.get('clusterName');

    this.get('ajaxRequestsQueue').addRequest(params);
  },

  /**
   * Navigate to next step after all requests are sent
   * @method ajaxQueueFinished
   */
  ajaxQueueFinished: function ajaxQueueFinished() {
    App.router.send('next');
  },

  /**
   * Error callback for each queued ajax-request
   * @param {object} xhr
   * @param {string} status
   * @param {string} error
   * @method ajaxQueueRequestErrorCallback
   */
  ajaxQueueRequestErrorCallback: function ajaxQueueRequestErrorCallback(xhr, status, error) {
    var responseText;
    try {
      responseText = JSON.parse(xhr.responseText);
    } catch (e) {
      responseText = { message: xhr.responseText };
    }
    var controller = App.router.get(App.clusterStatus.wizardControllerName);
    controller.registerErrPopup(Em.I18n.t('common.error'), responseText.message);
    this.set('hasErrorOccurred', true);
    // an error will break the ajax call chain and allow submission again
    this.set('isSubmitDisabled', false);
    this.set('isBackBtnDisabled', false);
    App.router.get(this.get('content.controllerName')).setStepsEnable();
  }
});

});

require.register("mixins/wizard/wizardEnableDone", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for wizard controller for enabling done/complete buttons if last task fails
 * This should
 * @type {Ember.Mixin}
 */
App.WizardEnableDone = Em.Mixin.create({
  statusChangeCallback: function statusChangeCallback(data) {
    var wizardController = App.router.get(this.get('content.controllerName'));
    this._super(data);
    if (wizardController.get('totalSteps') == wizardController.get('currentStep') && this.get('tasks.lastObject.status') === 'FAILED') {
      this.set('isSubmitDisabled', false);
    }
  }
});

});

require.register("mixins/wizard/wizardHostsLoading", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for loading hosts info in wizards
 * @type {Ember.Mixin}
 */
App.WizardHostsLoading = Em.Mixin.create({

  /**
   * @type {boolean}
   * @default false
   */
  isLoaded: false,

  willInsertElement: function willInsertElement() {
    this.set('isLoaded', false);
    this.loadHosts();
  },

  /**
   * load hosts from server
   */
  loadHosts: function loadHosts() {
    return App.ajax.send({
      name: 'hosts.high_availability.wizard',
      data: {},
      sender: this,
      success: 'loadHostsSuccessCallback',
      error: 'loadHostsErrorCallback'
    });
  },

  loadHostsSuccessCallback: function loadHostsSuccessCallback(data, opt, params) {
    var hosts = {};

    data.items.forEach(function (item) {
      hosts[item.Hosts.host_name] = {
        name: item.Hosts.host_name,
        bootStatus: "REGISTERED",
        isInstalled: true
      };
    });
    this.get('controller').setDBProperty('hosts', hosts);
    this.set('controller.content.hosts', hosts);
    this.set('isLoaded', true);
  },

  loadHostsErrorCallback: function loadHostsErrorCallback() {
    this.set('isLoaded', true);
  }

});

});

require.register("mixins/wizard/wizardProgressPageController", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for wizard controller for showing command progress on wizard pages
 * This should
 * @type {Ember.Mixin}
 */
App.wizardProgressPageControllerMixin = Em.Mixin.create(App.InstallComponent, {
  controllerName: '',
  clusterDeployState: 'WIZARD_DEPLOY',
  status: 'IN_PROGRESS',
  tasks: [],
  commands: [],
  currentRequestIds: [], //todo: replace with using requestIds from tasks
  logs: [],
  currentTaskId: null,
  POLL_INTERVAL: 4000,
  isSubmitDisabled: true,
  isBackButtonDisabled: true,
  stages: [],
  /**
   * List of statuses that inform about the end of request progress.
   * @type {String[]}
   */
  completedStatuses: ['COMPLETED', 'FAILED', 'TIMEDOUT', 'ABORTED'],
  currentPageRequestId: null,
  isSingleRequestPage: false,
  isCommandLevelRetry: Em.computed.not('isSingleRequestPage'),
  showRetry: false,
  /**
   * Show whether tasks data was loaded
   * @type {Boolean}
   */
  isLoading: false,

  k: Em.K,
  /**
   *  tasksMessagesPrefix should be overloaded by any controller including the mixin
   */
  tasksMessagesPrefix: '',

  loadStep: function loadStep() {
    this.clearStep();
    var self = this;
    if (!self.isSingleRequestPage) {
      this.initStep();
    } else {
      var requestIds = this.get('content.tasksRequestIds');
      var currentRequestId = requestIds && requestIds[0][0];
      if (!currentRequestId) {
        this.set('isLoaded', false);
        this.submitRequest();
      } else {
        self.set('currentPageRequestId', currentRequestId);
        self.doPollingForPageRequest();
      }
    }
  },

  initStep: function initStep() {
    this.initializeTasks();
    if (!this.isSingleRequestPage) {
      this.loadTasks();
    }
    this.addObserver('tasks.@each.status', this, 'onTaskStatusChange');
    if (this.isSingleRequestPage) {
      var dfd = $.Deferred();
      var dfdObject = {
        deferred: dfd,
        isJqueryPromise: true
      };
      this.onTaskStatusChange(dfdObject);
      return dfd.promise();
    } else {
      this.onTaskStatusChange();
    }
  },

  clearStep: function clearStep() {
    this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
    this.set('isSubmitDisabled', true);
    this.set('isBackButtonDisabled', true);
    this.set('tasks', []);
    this.set('currentRequestIds', []);
    this.set('isLoaded', false);
  },

  /**
   * Clear stages info for single page request.
   */
  clearStage: function clearStage() {
    this.setDBProperties({
      tasksRequestIds: null,
      tasksStatuses: null
    });
    this.set('showRetry', false);
    this.set('content.tasksRequestIds', null);
    this.set('content.tasksStatuses', null);
    this.set('content.currentTaskId', null);
    this.get('stages').clear();
  },

  retry: function retry() {
    this.set('showRetry', false);
    this.get('tasks').setEach('status', 'PENDING');
    this.loadStep();
  },

  submitRequest: function submitRequest() {
    var self = this;
    return App.ajax.send({
      name: this.get('request.ajaxName'),
      data: this.get('request.ajaxData'),
      sender: this,
      error: 'onSingleRequestError',
      success: 'submitRequestSuccess',
      kdcCancelHandler: function kdcCancelHandler() {
        self.set('status', 'FAILED');
        self.set('isLoaded', true);
        self.set('showRetry', true);
      }
    });
  },

  submitRequestSuccess: function submitRequestSuccess(data, result, request) {
    if (data) {
      this.set('currentPageRequestId', data.Requests.id);
      this.doPollingForPageRequest();
    } else {
      //Step has been successfully completed
      if (request.status === 200) {
        this.set('status', 'COMPLETED');
        this.set('isSubmitDisabled', false);
        this.set('isLoaded', true);
      }
    }
  },

  doPollingForPageRequest: function doPollingForPageRequest() {
    App.ajax.send({
      name: 'admin.poll.kerberize.cluster.request',
      sender: this,
      data: {
        requestId: this.get('currentPageRequestId')
      },
      success: 'initializeStages'
    });
  },

  initializeStages: function initializeStages(data) {
    var self = this;
    var stages = [];
    this.set('logs', []);
    data.stages.forEach(function (_stage) {
      stages.pushObject(Em.Object.create(_stage.Stage));
    }, this);
    if (!this.get('stages').length) {
      this.get('stages').pushObjects(stages);
      this.initStep().done(function () {
        self.updatePageWithPolledData(data);
      });
    } else {
      this.updatePageWithPolledData(data);
    }
  },

  updatePageWithPolledData: function updatePageWithPolledData(data) {
    // If all tasks completed no need to update each task status.
    // Preferable to skip polling of data for completed tasks after page refresh.
    if (this.get('status') === 'COMPLETED') return;
    var self = this;
    var tasks = [];
    var currentPageRequestId = this.get('currentPageRequestId');
    var currentTaskId = this.get('currentTaskId');
    var currentTask = this.get('tasks').findProperty('id', currentTaskId);
    var currentStage = data.stages.findProperty('Stage.stage_id', currentTask.get('stageId'));
    var tasksInCurrentStage = currentStage.tasks;
    this.set('logs', tasksInCurrentStage);

    this.setRequestIds(this.get('currentTaskId'), [this.get('currentPageRequestId')]);

    if (!tasksInCurrentStage.someProperty('Tasks.status', 'PENDING') && !tasksInCurrentStage.someProperty('Tasks.status', 'QUEUED') && !tasksInCurrentStage.someProperty('Tasks.status', 'IN_PROGRESS')) {
      this.set('currentRequestIds', []);
      if (tasksInCurrentStage.someProperty('Tasks.status', 'FAILED') || tasksInCurrentStage.someProperty('Tasks.status', 'TIMEDOUT') || tasksInCurrentStage.someProperty('Tasks.status', 'ABORTED')) {
        this.setTaskStatus(currentTaskId, 'FAILED');
      } else {
        this.setTaskStatus(currentTaskId, 'COMPLETED');
      }
    } else {
      var completedActions = tasksInCurrentStage.filterProperty('Tasks.status', 'COMPLETED').length + tasksInCurrentStage.filterProperty('Tasks.status', 'FAILED').length + tasksInCurrentStage.filterProperty('Tasks.status', 'ABORTED').length + tasksInCurrentStage.filterProperty('Tasks.status', 'TIMEDOUT').length;
      var queuedActions = tasksInCurrentStage.filterProperty('Tasks.status', 'QUEUED').length;
      var inProgressActions = tasksInCurrentStage.filterProperty('Tasks.status', 'IN_PROGRESS').length;
      var progress = Math.floor((queuedActions * 0.09 + inProgressActions * 0.35 + completedActions) / tasksInCurrentStage.length * 100);
      this.get('tasks').findProperty('id', currentTaskId).set('progress', progress);
    }

    // start polling if current request not completed
    if (!this.get('completedStatuses').contains(this.get('status'))) {
      window.setTimeout(function () {
        self.doPollingForPageRequest();
      }, self.POLL_INTERVAL);
    }
  },

  initializeTasks: function initializeTasks() {
    var self = this;
    var commands = this.isSingleRequestPage ? this.get('stages') : this.get('commands');
    var currentStep = App.router.get(this.get('content.controllerName') + '.currentStep');
    var tasksMessagesPrefix = this.get('tasksMessagesPrefix');
    // check that all stages have been completed for single request type
    var allStagesCompleted = commands.everyProperty('status', 'COMPLETED');
    for (var i = 0; i < commands.length; i++) {
      this.get('tasks').pushObject(Ember.Object.create({
        title: self.isSingleRequestPage ? commands[i].get('context') : Em.I18n.t(tasksMessagesPrefix + currentStep + '.task' + i + '.title'),
        // set COMPLETED status for task if all stages completed successfully
        status: allStagesCompleted ? 'COMPLETED' : 'PENDING',
        id: i,
        stageId: self.isSingleRequestPage ? commands[i].get('stage_id') : null,
        command: self.isSingleRequestPage ? 'k' : commands[i],
        showRetry: false,
        showRollback: false,
        name: self.isSingleRequestPage ? commands[i].get('context') : Em.I18n.t(tasksMessagesPrefix + currentStep + '.task' + i + '.title'),
        displayName: self.isSingleRequestPage ? commands[i].get('context') : Em.I18n.t(tasksMessagesPrefix + currentStep + '.task' + i + '.title'),
        progress: 0,
        isRunning: false,
        requestIds: self.isSingleRequestPage ? [this.get('stages')[0].request_id] : []
      }));
    }
    this.set('isLoaded', true);
  },

  loadTasks: function loadTasks() {
    var self = this;
    var loadedStatuses = this.get('content.tasksStatuses');
    var loadedRequestIds = this.get('content.tasksRequestIds');
    if (loadedStatuses && loadedStatuses.length === this.get('tasks').length) {
      this.get('tasks').forEach(function (task, i) {
        self.setTaskStatus(task.get('id'), loadedStatuses[i]);
        self.setRequestIds(task.get('id'), loadedRequestIds[i]);
      });
      if (loadedStatuses.contains('IN_PROGRESS')) {
        var curTaskId = this.get('tasks')[loadedStatuses.indexOf('IN_PROGRESS')].get('id');
        this.set('currentRequestIds', this.get('content.requestIds'));
        this.set('currentTaskId', curTaskId);
        this.doPolling();
      } else if (loadedStatuses.contains('QUEUED')) {
        var curTaskId = this.get('tasks')[loadedStatuses.indexOf('QUEUED')].get('id');
        this.set('currentTaskId', curTaskId);
        this.runTask(curTaskId);
      }
    }
  },

  /**
   * remove tasks by command name
   */
  removeTasks: function removeTasks(commands) {
    var tasks = this.get('tasks');

    commands.forEach(function (command) {
      var cmd = tasks.filterProperty('command', command);
      var index = null;

      if (cmd.length === 0) {
        return false;
      } else {
        index = tasks.indexOf(cmd[0]);
      }

      tasks.splice(index, 1);
    });
  },

  setTaskStatus: function setTaskStatus(taskId, status) {
    this.get('tasks').findProperty('id', taskId).set('status', status);
  },

  setTaskCanSkip: function setTaskCanSkip(taskId, canSkip) {
    this.get('tasks').findProperty('id', taskId).set('canSkip', true);
  },

  setRequestIds: function setRequestIds(taskId, requestIds) {
    this.get('tasks').findProperty('id', taskId).set('requestIds', requestIds);
  },

  retryTask: function retryTask() {
    var task = this.get('tasks').findProperty('status', 'FAILED');
    task.set('showRetry', false);
    task.set('showRollback', false);
    this.set('isSubmitDisabled', true);
    this.set('isBackButtonDisabled', true);
    task.set('status', 'PENDING');
  },

  onTaskStatusChange: function onTaskStatusChange(dfdObject) {
    var statuses = this.get('tasks').mapProperty('status');
    var tasksRequestIds = this.get('tasks').mapProperty('requestIds');
    var requestIds = this.get('currentRequestIds');
    // save task info
    App.router.get(this.get('content.controllerName')).saveTasksStatuses(statuses);
    App.router.get(this.get('content.controllerName')).saveTasksRequestIds(tasksRequestIds);
    App.router.get(this.get('content.controllerName')).saveRequestIds(requestIds);
    // call saving of cluster status asynchronous
    // synchronous executing cause problems in Firefox
    var successCallbackData;
    if (dfdObject && dfdObject.isJqueryPromise) {
      successCallbackData = { deferred: dfdObject.deferred };
    }
    App.clusterStatus.setClusterStatus({
      clusterName: App.router.getClusterName(),
      clusterState: this.get('clusterDeployState'),
      wizardControllerName: this.get('content.controllerName'),
      localdb: App.db.data
    }, { successCallback: this.statusChangeCallback, sender: this, successCallbackData: successCallbackData });
  },

  /**
   * Method that called after saving persist data to server.
   * Switch task according its status.
   */
  statusChangeCallback: function statusChangeCallback(data) {
    if (!this.get('tasks').someProperty('status', 'IN_PROGRESS') && !this.get('tasks').someProperty('status', 'QUEUED') && !this.get('tasks').someProperty('status', 'FAILED')) {
      var nextTask = this.get('tasks').findProperty('status', 'PENDING');
      if (nextTask) {
        this.set('status', 'IN_PROGRESS');
        var taskStatus = this.isSingleRequestPage ? 'IN_PROGRESS' : 'QUEUED';
        this.setTaskStatus(nextTask.get('id'), taskStatus);
        this.set('currentTaskId', nextTask.get('id'));
        this.runTask(nextTask.get('id'));
      } else {
        this.set('status', 'COMPLETED');
        this.set('isSubmitDisabled', false);
        this.set('isBackButtonDisabled', false);
      }
    } else if (this.get('tasks').someProperty('status', 'FAILED')) {
      this.set('status', 'FAILED');
      this.set('isBackButtonDisabled', false);
      if (this.get('isCommandLevelRetry')) {
        this.get('tasks').findProperty('status', 'FAILED').set('showRetry', true);
      } else {
        this.set('showRetry', true);
      }

      if (this.get('tasks').someProperty('canSkip', true)) {
        this.get('tasks').findProperty('canSkip', true).set('showSkip', true);
      }

      if (App.supports.autoRollbackHA) {
        this.get('tasks').findProperty('status', 'FAILED').set('showRollback', true);
      }
    }
    this.get('tasks').filterProperty('status', 'COMPLETED').setEach('showRetry', false);
    this.get('tasks').filterProperty('status', 'COMPLETED').setEach('showRollback', false);
    this.get('tasks').filterProperty('status', 'COMPLETED').setEach('showSkip', false);
    this.get('tasks').filterProperty('status', 'IN_PROGRESS').setEach('showSkip', false);

    if (data && data.deferred) {
      data.deferred.resolve();
    }
  },

  /**
   * Run command of appropriate task
   */
  runTask: function runTask(taskId) {
    this[this.get('tasks').findProperty('id', taskId).get('command')]();
  },

  onTaskError: function onTaskError() {
    this.setTaskStatus(this.get('currentTaskId'), 'FAILED');
  },

  onTaskErrorWithSkip: function onTaskErrorWithSkip() {
    this.onTaskError();
    this.setTaskCanSkip(this.get('currentTaskId'), true);
  },

  onSingleRequestError: function onSingleRequestError(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
    this.set('status', 'FAILED');
    this.set('isLoaded', true);
    this.set('showRetry', true);
  },

  onTaskCompleted: function onTaskCompleted() {
    this.setTaskStatus(this.get('currentTaskId'), 'COMPLETED');
  },

  /**
   * check whether component installed on specified hosts
   * @param {string} componentName
   * @param {string[]} hostNames
   * @return {$.ajax}
   */
  checkInstalledComponents: function checkInstalledComponents(componentName, hostNames) {
    return App.ajax.send({
      name: 'host_component.installed.on_hosts',
      sender: this,
      data: {
        componentName: componentName,
        hostNames: hostNames.join(',')
      }
    });
  },

  /**
   * make server call to stop services
   * if stopListedServicesFlag == false; stop all services excluding the services passed as parameters
   * if stopListedServicesFlag == true; stop only services passed as parameters
   * if namenode or secondary namenode then stop all services
   * @param services, stopListedServicesFlag, stopAllServices
   * @returns {$.ajax}
   */
  stopServices: function stopServices(services, stopListedServicesFlag, stopAllServices) {
    var stopAllServices = stopAllServices || false;
    var stopListedServicesFlag = stopListedServicesFlag || false;
    var data = {
      'ServiceInfo': {
        'state': 'INSTALLED'
      }
    };
    if (stopAllServices) {
      data.context = "Stop all services";
    } else {
      if (!services || !services.length) {
        services = App.Service.find().mapProperty('serviceName').filter(function (service) {
          return service != 'HDFS';
        });
      }
      var servicesList;
      if (stopListedServicesFlag) {
        servicesList = services.join(',');
      } else {
        servicesList = App.Service.find().mapProperty("serviceName").filter(function (s) {
          return !services.contains(s);
        }).join(',');
      }
      data.context = "Stop required services";
      data.urlParams = "ServiceInfo/service_name.in(" + servicesList + ")";
    }
    return App.ajax.send({
      name: 'common.services.update',
      sender: this,
      data: data,
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  /*tep4
   * make server call to start services
   * if startListedServicesFlag == false; start all services excluding the services passed as parameters
   * if startListedServicesFlag == true; start only services passed as parameters
   * if no parameters are passed; start all services
   * and run smoke tests if runSmokeTest is true
   * @param runSmokeTest
   * @param services
   * @param startListedServicesFlag
   * @returns {$.ajax}
   */
  startServices: function startServices(runSmokeTest, services, startListedServicesFlag) {
    var startListedServicesFlag = startListedServicesFlag || false;
    var skipServiceCheck = App.router.get('clusterController.ambariProperties')['skip.service.checks'] === "true";
    var data = {
      'ServiceInfo': {
        'state': 'STARTED'
      }
    };
    var servicesList;
    if (services && services.length) {
      if (startListedServicesFlag) {
        servicesList = services.join(',');
      } else {
        servicesList = App.Service.find().mapProperty("serviceName").filter(function (s) {
          return !services.contains(s);
        }).join(',');
      }
      data.context = "Start required services";
      data.urlParams = "ServiceInfo/service_name.in(" + servicesList + ")";
    } else {
      data.context = "Start all services";
    }

    if (runSmokeTest) {
      data.urlParams = data.urlParams ? data.urlParams + '&' : '';
      data.urlParams += 'params/run_smoke_test=' + !skipServiceCheck;
    }

    return App.ajax.send({
      name: 'common.services.update',
      sender: this,
      data: data,
      success: 'startPolling',
      error: 'startServicesErrorCallback'
    });
  },

  startServicesErrorCallback: function startServicesErrorCallback(jqXHR, ajaxOptions, error, opt) {
    App.ajax.defaultErrorHandler(jqXHR, opt.url, opt.type, jqXHR.status);
    this.onTaskError(jqXHR, ajaxOptions, error, opt);
  },

  /**
   * Create component on single or multiple hosts.
   *
   * @method createComponent
   * @param {string} componentName - name of the component
   * @param {(string|string[])} hostName - host/hosts where components should be installed
   * @param {string} serviceName - name of the services
   */
  createComponent: function createComponent(componentName, hostName, serviceName) {
    var hostNames = Array.isArray(hostName) ? hostName : [hostName];
    var self = this;

    this.set('showRetry', false);

    this.checkInstalledComponents(componentName, hostNames).then(function (data) {
      var hostsWithComponents = data.items.mapProperty('HostRoles.host_name');
      var result = hostNames.map(function (item) {
        return {
          componentName: componentName,
          hostName: item,
          hasComponent: hostsWithComponents.contains(item)
        };
      });
      var hostsWithoutComponents = result.filterProperty('hasComponent', false).mapProperty('hostName');
      var taskNum = 1;
      var requestData = {
        "RequestInfo": {
          "query": hostsWithoutComponents.map(function (item) {
            return 'Hosts/host_name=' + item;
          }).join('|')
        },
        "Body": {
          "host_components": [{
            "HostRoles": {
              "component_name": componentName
            }
          }]
        }
      };
      if (!!hostsWithoutComponents.length) {
        self.updateAndCreateServiceComponent(componentName).done(function () {
          App.ajax.send({
            name: 'wizard.step8.register_host_to_component',
            sender: self,
            data: {
              data: JSON.stringify(requestData),
              hostName: result.mapProperty('hostName'),
              componentName: componentName,
              serviceName: serviceName,
              taskNum: taskNum,
              cluster: App.get('clusterName')
            },
            success: 'onCreateComponent',
            error: 'onCreateComponent'
          });
        });
      } else {
        self.onCreateComponent(null, null, {
          hostName: result.mapProperty('hostName'),
          componentName: componentName,
          serviceName: serviceName,
          taskNum: taskNum
        }, self);
      }
    });
  },

  onCreateComponent: function onCreateComponent() {
    var hostName = arguments[2].hostName;
    var componentName = arguments[2].componentName;
    var taskNum = arguments[2].taskNum;
    var serviceName = arguments[2].serviceName;
    this.updateComponent(componentName, hostName, serviceName, "Install", taskNum);
  },

  onCreateComponentError: function onCreateComponentError(error) {
    if (error.responseText.indexOf('org.apache.ambari.server.controller.spi.ResourceAlreadyExistsException') !== -1) {
      this.onCreateComponent();
    } else {
      this.onTaskError();
    }
  },

  /**
   * Update component status on selected hosts.
   *
   * @param {string} componentName
   * @param {(string|string[]|null)} hostName - use null to update components on all hosts
   * @param {string} serviceName
   * @param {string} context
   * @param {number} taskNum
   * @returns {$.ajax}
   */
  updateComponent: function updateComponent(componentName, hostName, serviceName, context, taskNum) {
    if (hostName && !(hostName instanceof Array)) {
      hostName = [hostName];
    }
    var state = context.toLowerCase() == "start" ? "STARTED" : "INSTALLED";
    return App.ajax.send({
      name: 'common.host_components.update',
      sender: this,
      data: {
        HostRoles: {
          state: state
        },
        query: 'HostRoles/component_name=' + componentName + (hostName ? '&HostRoles/host_name.in(' + hostName.join(',') + ')' : '') + '&HostRoles/maintenance_state=OFF',
        context: context + " " + App.format.role(componentName, false),
        hostName: hostName,
        taskNum: taskNum || 1,
        componentName: componentName,
        serviceName: serviceName
      },
      success: 'startPolling',
      error: 'onTaskError'
    });
  },

  /**
   * Update state for array of components of different services and on different hosts
   *
   * @param {Array} components - array of components object with fields serviceName, hostName and componentName
   * @param {String} state - new state to update
   */
  updateComponentsState: function updateComponentsState(components, state) {
    components.forEach(function (component) {
      App.ajax.send({
        name: 'common.host.host_component.update',
        sender: this,
        data: {
          hostName: component.hostName,
          serviceName: component.serviceName,
          componentName: component.componentName,
          HostRoles: {
            state: state
          },
          taskNum: components.length
        },
        success: 'startPolling',
        error: 'onTaskError'
      });
    }, this);
  },

  startPolling: function startPolling(data) {
    if (data) {
      this.get('currentRequestIds').push(data.Requests.id);
      var tasksCount = arguments[2].taskNum || 1;
      if (tasksCount === this.get('currentRequestIds').length) {
        this.setRequestIds(this.get('currentTaskId'), this.get('currentRequestIds'));
        this.doPolling();
      }
    } else {
      this.onTaskCompleted();
    }
  },

  doPolling: function doPolling() {
    this.setTaskStatus(this.get('currentTaskId'), 'IN_PROGRESS');
    var requestIds = this.get('currentRequestIds');
    this.set('logs', []);
    for (var i = 0; i < requestIds.length; i++) {
      App.ajax.send({
        name: 'background_operations.get_by_request',
        sender: this,
        data: {
          requestId: requestIds[i]
        },
        success: 'parseLogs',
        error: 'onTaskError'
      });
    }
  },

  parseLogs: function parseLogs(logs) {
    this.get('logs').pushObject(logs.tasks);
    if (this.get('currentRequestIds').length === this.get('logs').length) {
      var tasks = [];
      this.get('logs').forEach(function (logs) {
        tasks.pushObjects(logs);
      }, this);
      var self = this;
      var currentTaskId = this.get('currentTaskId');
      if (!tasks.someProperty('Tasks.status', 'PENDING') && !tasks.someProperty('Tasks.status', 'QUEUED') && !tasks.someProperty('Tasks.status', 'IN_PROGRESS')) {
        this.set('currentRequestIds', []);
        if (tasks.someProperty('Tasks.status', 'FAILED') || tasks.someProperty('Tasks.status', 'TIMEDOUT') || tasks.someProperty('Tasks.status', 'ABORTED')) {
          this.setTaskStatus(currentTaskId, 'FAILED');
        } else {
          this.setTaskStatus(currentTaskId, 'COMPLETED');
        }
      } else {
        var actionsPerHost = tasks.length;
        var completedActions = tasks.filterProperty('Tasks.status', 'COMPLETED').length + tasks.filterProperty('Tasks.status', 'FAILED').length + tasks.filterProperty('Tasks.status', 'ABORTED').length + tasks.filterProperty('Tasks.status', 'TIMEDOUT').length;
        var queuedActions = tasks.filterProperty('Tasks.status', 'QUEUED').length;
        var inProgressActions = tasks.filterProperty('Tasks.status', 'IN_PROGRESS').length;
        var progress = Math.floor((queuedActions * 0.09 + inProgressActions * 0.35 + completedActions) / actionsPerHost * 100);
        this.get('tasks').findProperty('id', currentTaskId).set('progress', progress);
        window.setTimeout(function () {
          self.doPolling();
        }, self.POLL_INTERVAL);
      }
    }
  },

  showHostProgressPopup: function showHostProgressPopup(event) {
    if (!['IN_PROGRESS', 'FAILED', 'COMPLETED'].contains(Em.get(event.context, 'status')) || !event.contexts[0].requestIds.length) {
      return;
    }
    var popupTitle = event.contexts[0].title,
        requestIds = event.contexts[0].requestIds,
        stageId = event.contexts[0].stageId,
        hostProgressPopupController = App.router.get('highAvailabilityProgressPopupController');
    hostProgressPopupController.initPopup(popupTitle, requestIds, this, true, stageId);
  },

  done: function done() {
    if (!this.get('isSubmitDisabled')) {
      this.set('isSubmitDisabled', true);
      this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
      App.router.send('next');
    }
  },

  back: function back() {
    if (!this.get('isBackButtonDisabled')) {
      this.set('isBackButtonDisabled', true);
      this.removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
      App.router.send('back');
    }
  },

  /**
   * Delete component on single hosts.
   *
   * @method deleteComponent
   * @param {string} componentName - name of the component
   * @param {string} hostName - host from where components should be deleted
   */
  deleteComponent: function deleteComponent(componentName, hostName) {
    App.ajax.send({
      name: 'common.delete.host_component',
      sender: this,
      data: {
        componentName: componentName,
        hostName: hostName
      },
      success: 'onTaskCompleted',
      error: 'onDeleteHostComponentsError'
    });
  },

  onDeleteHostComponentsError: function onDeleteHostComponentsError(error) {
    // If the component does not exist on the host, NoSuchResourceException is thrown.
    // If NoSuchResourceException is thrown, there is no action required and operation should continue.
    if (error.responseText.indexOf('org.apache.ambari.server.controller.spi.NoSuchResourceException') !== -1) {
      this.onTaskCompleted();
    } else {
      this.onTaskError();
    }
  },

  /**
   * Same as <code>createComponent</code> but with kdc session check and status changes
   * when KDC auth dialog dissmised.
   *
   * @see createComponent
   */
  createInstallComponentTask: function createInstallComponentTask(componentName, hostName, serviceName, options) {
    var self = this;
    App.get('router.mainAdminKerberosController').getKDCSessionState(function () {
      self.createComponent(componentName, hostName, serviceName);
    }, function () {
      self.onTaskError();
    });
  }
});

});

require.register("mixins/wizard/wizardProgressPageView", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Mixin for wizard view for showing command progress on wizard pages
 * This should
 * @type {Ember.Mixin}
 */
App.wizardProgressPageViewMixin = Em.Mixin.create({

  /**
   * Following computed property needs to be overridden by the view implementing this mixin
   */

  currentStep: '',

  /**
   * Following computed property needs to be overridden by the view implementing this mixin
   */

  headerTitle: function () {}.property(),

  submitButtonText: Em.I18n.t('common.next'),

  noticeCompleted: Em.I18n.t('wizard.progressPage.notice.completed'),

  noticeFailed: Em.computed.ifThenElse('controller.isSingleRequestPage', Em.I18n.t('wizard.singleRequest.progressPage.notice.failed'), Em.I18n.t('wizard.progressPage.notice.failed')),

  /**
   * @noticeInProgress: Following computed property needs to be overridden to show the label text while the commands
   * on the page are progressing
   */
  noticeInProgress: function () {}.property(),

  /**
   * @showBackButton: Override this property to show back button on the wizard progress page
   */
  showBackButton: false,

  /**
   * Following computed property needs to be overridden by the view implementing this mixin
   */
  notice: '',

  noticeClass: 'alert alert-info',

  /**
   * Class to define task label width
   * @type {String}
   */
  labelWidth: 'col-md-4',

  onStatusChange: function () {
    var status = this.get('controller.status');
    if (status === 'COMPLETED') {
      this.set('notice', this.get('noticeCompleted'));
      this.set('noticeClass', 'alert alert-success');
    } else if (status === 'FAILED') {
      this.set('notice', this.get('noticeFailed'));
      this.set('noticeClass', 'alert alert-error');
    } else {
      this.set('notice', this.get('noticeInProgress'));
      this.set('noticeClass', 'alert alert-info');
    }
  }.observes('controller.status'),

  taskView: Em.View.extend({
    icon: '',
    iconColor: '',
    linkClass: '',

    didInsertElement: function didInsertElement() {
      this.onStatus();
    },

    barWidth: Em.computed.format('width: {0}%;', 'content.progress'),

    onStatus: function () {
      var linkClass = !!this.get('content.requestIds.length') ? 'active-link' : 'active-text';
      this.set('linkClass', linkClass);
      if (this.get('content.status') === 'IN_PROGRESS') {
        this.set('icon', 'glyphicon glyphicon-cog');
        this.set('iconColor', 'text-info');
      } else if (this.get('content.status') === 'FAILED') {
        this.set('icon', 'glyphicon glyphicon-exclamation-sign');
        this.set('iconColor', 'text-danger');
      } else if (this.get('content.status') === 'COMPLETED') {
        this.set('icon', 'glyphicon glyphicon-ok');
        this.set('iconColor', 'text-success');
      } else {
        this.set('icon', 'glyphicon glyphicon-cog');
        this.set('iconColor', '');
        this.set('linkClass', 'not-active-link');
      }
    }.observes('content.status', 'content.hosts.length', 'content.requestIds'),

    showProgressBar: Em.computed.equal('content.status', 'IN_PROGRESS')
  })

});

});

require.register("mixins/wizard/wizard_menu_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function isStepDisabled(index) {
  return Em.computed('controller.isStepDisabled.@each.{step,value}', function () {
    return this.isStepDisabled(index);
  }).cacheable();
}

function isStepCompleted(index) {
  return Em.computed('controller.{currentStep,isStepDisabled.@each.value}', function () {
    return this.isStepCompleted(index);
  }).cacheable();
}

App.WizardMenuMixin = Em.Mixin.create({

  isStepDisabled: function isStepDisabled(index) {
    return this.get('controller.isStepDisabled').findProperty('step', index).get('value');
  },

  isStepCompleted: function isStepCompleted(index) {
    return this.get('controller.currentStep') > index;
  },


  isStep0Disabled: isStepDisabled(0),
  isStep1Disabled: isStepDisabled(1),
  isStep2Disabled: isStepDisabled(2),
  isStep3Disabled: isStepDisabled(3),
  isStep4Disabled: isStepDisabled(4),
  isStep5Disabled: isStepDisabled(5),
  isStep6Disabled: isStepDisabled(6),
  isStep7Disabled: isStepDisabled(7),
  isStep8Disabled: isStepDisabled(8),
  isStep9Disabled: isStepDisabled(9),
  isStep10Disabled: isStepDisabled(10),

  isStep0Completed: isStepCompleted(0),
  isStep1Completed: isStepCompleted(1),
  isStep2Completed: isStepCompleted(2),
  isStep3Completed: isStepCompleted(3),
  isStep4Completed: isStepCompleted(4),
  isStep5Completed: isStepCompleted(5),
  isStep6Completed: isStepCompleted(6),
  isStep7Completed: isStepCompleted(7),
  isStep8Completed: isStepCompleted(8),
  isStep9Completed: isStepCompleted(9),
  isStep10Completed: isStepCompleted(10)

});

});

require.register("mixins/wizard/wizard_misc_property_checker", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardMiscPropertyChecker = Em.Mixin.create({
  showConfirmationDialogIfShouldChangeProps: function showConfirmationDialogIfShouldChangeProps(changedProperty, stepConfigs, serviceId) {
    var serviceConfigModificationHandler = null;
    this.affectedProperties = [];
    try {
      serviceConfigModificationHandler = require('utils/configs/modification_handlers/' + serviceId.toLowerCase());
    } catch (e) {
      console.log("Unable to load modification handler for ", serviceId);
    }
    if (serviceConfigModificationHandler != null) {
      var securityEnabled = App.router.get('mainAdminKerberosController.securityEnabled');
      this.affectedProperties = serviceConfigModificationHandler.getDependentConfigChanges(changedProperty, this.get("controller.selectedServiceNames"), stepConfigs, securityEnabled);
    }
    changedProperty.set("editDone", false); // Turn off flag

    if (this.affectedProperties.length > 0 && !this.get("controller.miscModalVisible")) {
      this.newAffectedProperties = this.affectedProperties;
      var self = this;
      return App.ModalPopup.show({
        classNames: ['modal-690px-width'],
        modalDialogClasses: ['modal-lg'],
        showCloseButton: false,
        primary: Em.I18n.t('common.apply'),
        secondary: serviceId == 'MISC' ? Em.I18n.t('common.ignore') : null,
        third: Em.I18n.t('common.cancel'),
        secondaryClass: 'btn-warning',
        header: "Warning: you must also change these Service properties",
        onPrimary: function onPrimary() {
          self.get("newAffectedProperties").forEach(function (item) {
            if (item.isNewProperty) {
              self.createProperty({
                name: item.propertyName,
                displayName: item.propertyDisplayName,
                value: item.newValue,
                categoryName: item.categoryName,
                serviceName: item.serviceName,
                filename: item.filename
              });
            } else {
              self.get("controller.stepConfigs").findProperty("serviceName", item.serviceName).get("configs").find(function (config) {
                return item.propertyName == config.get('name') && (item.filename == null || item.filename == config.get('filename'));
              }).set("value", item.newValue);
            }
          });
          self.get("controller").set("miscModalVisible", false);
          this.hide();
        },
        onSecondary: function onSecondary() {
          self.get("controller").set("miscModalVisible", false);
          this.hide();
        },
        onThird: function onThird() {
          var affected = self.get("newAffectedProperties").objectAt(0),
              changedProperty = self.get("controller.stepConfigs").findProperty("serviceName", affected.sourceServiceName).get("configs").findProperty("name", affected.changedPropertyName);
          changedProperty.set('value', changedProperty.get('savedValue') || changedProperty.get('initialValue'));
          self.get("controller").set("miscModalVisible", false);
          this.hide();
        },
        bodyClass: Em.View.extend({
          templateName: require('templates/common/configs/propertyDependence'),
          controller: this,
          propertyChange: self.get("newAffectedProperties"),
          didInsertElement: function didInsertElement() {
            self.get("controller").set("miscModalVisible", true);
          }
        })
      });
    }
  }
});

});

require.register("models", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load all models here

require('models/form'); // should be the 1st
require('models/authentication');
require('models/cluster');
require('models/cluster_states');
require('models/hosts');
require('models/stack');
require('models/stack_version/version');
require('models/stack_version/repository_version');
require('models/stack_version/os');
require('models/stack_version/service_simple');
require('models/stack_version/stack_upgrade_history');
require('models/operating_system');
require('models/repository');
require('models/stack_service');
require('models/stack_service_component');
require('models/quick_links');
require('models/quicklinks/quick_links_config');
require('models/service');
require('models/service_audit');
require('models/service/hdfs');
require('models/service/onefs');
require('models/service/yarn');
require('models/service/mapreduce2');
require('models/service/hbase');
require('models/service/flume');
require('models/service/storm');
require('models/service/ranger');
require('models/alerts/alert_definition');
require('models/alerts/alert_instance');
require('models/alerts/alert_instance_local');
require('models/alerts/alert_notification');
require('models/alerts/alert_config');
require('models/alerts/alert_group');
require('models/user');
require('models/host');
require('models/rack');
require('models/background_operation');
require('models/client_component');
require('models/host_component');
require('models/host_component_log');
require('models/target_cluster');
require('models/slave_component');
require('models/master_component');
require('models/host_stack_version');
require('models/root_service');
require('models/upgrade_entity');
require('models/finished_upgrade_entity');
require('models/configs/theme/theme_condition');
require('models/configs/service_config_version');
require('models/configs/stack_config_property');
require('models/configs/config_group');
require('models/configs/theme/tab');
require('models/configs/theme/section');
require('models/configs/theme/sub_section');
require('models/configs/theme/sub_section_tab');
require('models/configs/theme/config_action');
require('models/configs/objects/service_config');
require('models/configs/objects/service_config_category');
require('models/configs/objects/service_config_property');
require('models/view_instance');
require('models/widget');
require('models/widget_property');
require('models/widget_layout');

});

require.register("models/alerts/alert_config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');
var numericUtils = require('utils/number_utils');

App.AlertConfigProperty = Ember.Object.extend({

  /**
   * label to be shown for config property
   * @type {String}
   */
  label: '',

  /**
   * PORT|METRIC|AGGREGATE
   * @type {String}
   */
  type: '',

  /**
   * config property value
   * @type {*}
   */
  value: null,

  /**
   * property value cache to realise undo function
   * @type {*}
   */
  previousValue: null,

  /**
   * define either input is disabled or enabled
   * @type {Boolean}
   */
  isDisabled: false,

  /**
   * options that Select list will have
   * @type {Array}
   */
  options: [],

  /**
   * input displayType
   * one of 'textFields', 'textArea', 'select' or 'threshold'
   * @type {String}
   */
  displayType: '',

  /**
   * unit to be shown with value
   * @type {String}
   */
  unit: null,

  /**
   * space separated list of css class names to use
   * @type {String}
   */
  classNames: '',

  /**
   * define whether row with property should be shifted right
   * @type {Boolean}
   */
  isShifted: false,

  /**
   * name or names of properties related to config
   * may be either string for one property or array of strings for multiple properties
   * if this property is array, then <code>apiFormattedValue</code> should also be an array
   * example: <code>apiProperty[0]</code> relates to <code>apiFormattedValue[0]</code>
   * @type {String|Array}
   */
  apiProperty: '',

  /**
   * for some metrics properties may be set true or false
   * depending on what property is related to (JMX or Ganglia)
   */
  isJMXMetric: null,

  /**
   * define place to show label
   * if true - label is shown before input
   * if false - label is shown after input
   * @type {Boolean}
   */
  isPreLabeled: Em.computed.notExistsIn('displayType', ['radioButton']),

  /**
   * value converted to appropriate format for sending to server
   * should be computed property
   * should be defined in child class
   * @type {*}
   */
  apiFormattedValue: Em.computed.alias('value'),

  /**
   * define if property was changed by user
   * @type {Boolean}
   */
  wasChanged: function () {
    return this.get('previousValue') !== null && this.get('value') !== this.get('previousValue');
  }.property('value', 'previousValue'),

  /**
   * view class according to <code>displayType</code>
   * @type {Em.View}
   */
  viewClass: function () {
    var displayType = this.get('displayType');
    switch (displayType) {
      case 'textField':
        return App.AlertConfigTextFieldView;
      case 'textArea':
        return App.AlertConfigTextAreaView;
      case 'select':
        return App.AlertConfigSelectView;
      case 'threshold':
        return App.AlertConfigThresholdView;
      case 'radioButton':
        return App.AlertConfigRadioButtonView;
      case 'parameter':
        return App.AlertConfigParameterView;
      default:
    }
  }.property('displayType'),

  /**
   * Define whether property is valid
   * Computed property
   * Should be defined in child class
   * @type {Boolean}
   */
  isValid: function () {
    return true;
  }.property()

});

App.AlertConfigProperties = {

  AlertName: App.AlertConfigProperty.extend({
    name: 'alert_name',
    label: 'Alert Name',
    displayType: 'textField',
    apiProperty: 'name'
  }),

  AlertNameSelected: App.AlertConfigProperty.extend({
    name: 'alert_name',
    label: 'Alert Name',
    displayType: 'select',
    apiProperty: 'name'
  }),

  Service: App.AlertConfigProperty.extend({
    name: 'service',
    label: 'Service',
    displayType: 'select',
    apiProperty: 'service_name',
    apiFormattedValue: function () {
      return this.get('value') === 'CUSTOM' ? this.get('value') : App.StackService.find().findProperty('displayName', this.get('value')).get('serviceName');
    }.property('value'),
    change: function change() {
      this.set('property.value', true);
      this.get('parentView.controller').changeService(this.get('property.name'));
    }
  }),

  Component: App.AlertConfigProperty.extend({
    name: 'component',
    label: 'Component',
    displayType: 'select',
    apiProperty: 'component_name',
    apiFormattedValue: function () {
      return this.get('value') === 'No component' ? this.get('value') : App.StackServiceComponent.find().findProperty('displayName', this.get('value')).get('componentName');
    }.property('value')
  }),

  Scope: App.AlertConfigProperty.extend({
    name: 'scope',
    label: 'Scope',
    displayType: 'select',
    apiProperty: 'scope',
    apiFormattedValue: function () {
      return this.get('value').toUpperCase();
    }.property('value')
  }),

  Description: App.AlertConfigProperty.extend({
    name: 'description',
    label: 'Description',
    displayType: 'textArea',
    // todo: check value after API will be provided
    apiProperty: 'description'
  }),

  Interval: App.AlertConfigProperty.extend({
    name: 'interval',
    label: 'Check Interval',
    displayType: 'textField',
    unit: 'Minute',
    colWidth: 'col-md-4',
    apiProperty: 'interval',
    isValid: function () {
      var value = this.get('value');
      if (!value) return false;
      return String(value) === String(parseInt(value, 10)) && value >= 1;
    }.property('value')
  }),

  /**
   * Implements threshold
   * Main difference from other alertConfigProperties:
   * it has two editable parts - <code>value</code> and <code>text</code>
   * User may configure it to edit only one of them (use flags <code>showInputForValue</code> and <code>showInputForText</code>)
   * This flags also determines update value and text in the API-request or not (see <code>App.AlertConfigProperties.Thresholds</code> for more examples)
   *
   * @type {App.AlertConfigProperty.Threshold}
   */
  Threshold: App.AlertConfigProperty.extend({

    name: 'threshold',

    /**
     * Property text cache to realise undo function
     * @type {*}
     */
    previousText: null,

    label: '',

    /**
     * OK|WARNING|CRITICAL
     * @type {string}
     */
    badge: '',

    /**
     * threshold-value
     * @type {string}
     */
    value: '',

    /**
     * Type of value. This will be a fixed set of types (like %).
     */
    valueMetric: null,

    /**
     * Value actually displayed to the user. This value is transformed
     * based on the limited types of 'valueMetric's. Mappings from
     * 'value' to 'displayValue' is handled by observers.
     */
    displayValue: '',

    /**
     * threshold-text
     * @type {string}
     */
    text: '',

    displayType: 'threshold',

    apiProperty: [],

    init: function init() {
      this.set('displayValue', this.getNewValue());
      this._super();
    },

    /**
     * @type {string[]}
     */
    apiFormattedValue: function () {
      var ret = [];
      if (this.get('showInputForValue')) {
        ret.push(this.get('value'));
      }
      if (this.get('showInputForText')) {
        ret.push(this.get('text'));
      }
      return ret;
    }.property('value', 'text', 'showInputForValue', 'showInputForText'),

    /**
     * Determines if <code>value</code> should be visible and editable (if not - won't update API-value)
     * @type {bool}
     */
    showInputForValue: true,

    /**
     * Determines if <code>text</code> should be visible and editable (if not - won't update API-text)
     * @type {bool}
     */
    showInputForText: true,

    /**
     * Custom css-class for different badges
     * type {string}
     */
    badgeCssClass: Em.computed.format('alert-state-{0}', 'badge'),

    /**
     * Determines if <code>value</code> or <code>text</code> were changed
     * @type {bool}
     */
    wasChanged: function () {
      return this.get('previousValue') !== null && this.get('value') !== this.get('previousValue') || this.get('previousText') !== null && this.get('text') !== this.get('previousText');
    }.property('value', 'text', 'previousValue', 'previousText'),

    /**
     * May be redefined in child-models, mixins etc
     * @method getValue
     * @returns {string}
     */
    getNewValue: function getNewValue() {
      return this.get('value');
    },

    valueWasChanged: function () {
      var displayValue = this.get('displayValue');
      var newDisplayValue = this.getNewValue();
      if (newDisplayValue !== displayValue && !(isNaN(newDisplayValue) || isNaN(displayValue))) {
        this.set('displayValue', newDisplayValue);
      }
    }.observes('value'),

    /**
     * May be redefined in child-models, mixins etc
     * @method getDisplayValue
     * @returns {string}
     */
    getNewDisplayValue: function getNewDisplayValue() {
      return this.get('displayValue');
    },

    displayValueWasChanged: function () {
      var value = this.get('value');
      var newValue = this.getNewDisplayValue();
      if (newValue !== value && !(isNaN(newValue) || isNaN(value))) {
        this.set('value', newValue);
      }
    }.observes('displayValue'),

    /**
     * Check if <code>displayValue</code> is valid float number
     * If this value isn't shown (see <code>showInputForValue</code>), result is always true
     * @return {boolean}
     */
    isValid: function () {
      if (!this.get('showInputForValue')) {
        return true;
      }

      var value = this.get('displayValue');

      if (Em.isNone(value)) {
        return false;
      }

      value = ('' + value).trim();

      //only allow 1/10th of a second
      if (numericUtils.getFloatDecimals(value) > 1) {
        return false;
      }

      return validator.isValidFloat(value);
    }.property('displayValue', 'showInputForValue')

  }),

  URI: App.AlertConfigProperty.extend({
    name: 'uri',
    label: 'Host',
    displayType: 'textField',
    apiProperty: 'source.uri'
  }),

  URIExtended: App.AlertConfigProperty.extend({
    name: 'uri',
    label: 'URI',
    displayType: 'textArea',
    apiProperty: 'source.uri',
    apiFormattedValue: function () {
      var result = {};
      try {
        result = JSON.parse(this.get('value'));
      } catch (e) {
        console.error('Wrong format of URI');
      }
      return result;
    }.property('value')
  }),

  DefaultPort: App.AlertConfigProperty.extend({
    name: 'default_port',
    label: 'Default Port',
    displayType: 'textField',
    classNames: 'alert-port-input',
    apiProperty: 'source.default_port',
    isValid: function () {
      var value = this.get('value');
      if (!value) return false;
      return String(value) === String(parseInt(value, 10)) && value >= 1;
    }.property('value')
  }),

  Path: App.AlertConfigProperty.extend({
    name: 'path',
    label: 'Path',
    displayType: 'textField',
    apiProperty: 'source.path'
  }),

  Metrics: App.AlertConfigProperty.extend({
    name: 'metrics',
    label: 'JMX/Ganglia Metrics',
    displayType: 'textArea',
    apiProperty: Em.computed.ifThenElse('isJMXMetric', 'source.jmx.property_list', 'source.ganglia.property_list'),
    apiFormattedValue: function () {
      return this.get('value').split(',\n');
    }.property('value')
  }),

  FormatString: App.AlertConfigProperty.extend({
    name: 'metrics_string',
    label: 'Format String',
    displayType: 'textArea',
    apiProperty: Em.computed.ifThenElse('isJMXMetric', 'source.jmx.value', 'source.ganglia.value')
  }),

  Parameter: App.AlertConfigProperty.extend({

    name: 'parameter',

    displayType: 'parameter',

    badge: Em.computed.alias('threshold'),

    thresholdExists: Em.computed.bool('threshold'),

    thresholdNotExists: Em.computed.empty('threshold'),

    /**
     * Custom css-class for different badges
     * type {string}
     */
    badgeCssClass: Em.computed.format('alert-state-{0}', 'badge')

  })

};
App.AlertConfigProperties.Parameters = {

  StringMixin: Em.Mixin.create({
    isValid: function () {
      var value = this.get('value');
      return String(value).trim() !== '';
    }.property('value')
  }),
  NumericMixin: Em.Mixin.create({
    isValid: function () {
      var value = this.get('value');
      if (!value) {
        return false;
      }
      value = ('' + value).trim();
      if (!numericUtils.isPositiveNumber(value)) {
        return false;
      }
      value = parseFloat(value);
      return !isNaN(value);
    }.property('value')
  }),
  PercentageMixin: Em.Mixin.create({
    isValid: function () {
      var value = this.get('value');
      if (!value) {
        return false;
      }
      if (!validator.isValidFloat(value) || !numericUtils.isPositiveNumber(value)) {
        return false;
      }
      value = String(value).trim();
      value = parseFloat(value);

      return !isNaN(value) && value > 0;
    }.property('value')
  })

};
App.AlertConfigProperties.Thresholds = {

  OkThreshold: App.AlertConfigProperties.Threshold.extend({

    badge: 'OK',

    name: 'ok_threshold',

    apiProperty: function () {
      var ret = [];
      if (this.get('showInputForValue')) {
        ret.push('source.reporting.ok.value');
      }
      if (this.get('showInputForText')) {
        ret.push('source.reporting.ok.text');
      }
      return ret;
    }.property('showInputForValue', 'showInputForText')

  }),

  WarningThreshold: App.AlertConfigProperties.Threshold.extend({

    badge: 'WARNING',

    name: 'warning_threshold',

    apiProperty: function () {
      var ret = [];
      if (this.get('showInputForValue')) {
        ret.push('source.reporting.warning.value');
      }
      if (this.get('showInputForText')) {
        ret.push('source.reporting.warning.text');
      }
      return ret;
    }.property('showInputForValue', 'showInputForText')

  }),

  CriticalThreshold: App.AlertConfigProperties.Threshold.extend({

    badge: 'CRITICAL',

    name: 'critical_threshold',

    apiProperty: function () {
      var ret = [];
      if (this.get('showInputForValue')) {
        ret.push('source.reporting.critical.value');
      }
      if (this.get('showInputForText')) {
        ret.push('source.reporting.critical.text');
      }
      return ret;
    }.property('showInputForValue', 'showInputForText')

  }),

  /**
   * Mixin for <code>App.AlertConfigProperties.Threshold</code>
   * Used to validate values in percentage range (0..1]
   * @type {Em.Mixin}
   */
  PercentageMixin: Em.Mixin.create({

    isValid: function () {
      var value = this.get('displayValue');

      if (!value) {
        return false;
      }

      value = ('' + value).trim();
      if (numericUtils.getFloatDecimals(value)) {
        return false;
      }

      value = parseFloat(value);

      //do not allow float values
      if (parseInt(value, 10) !== value) {
        return false;
      }

      return this.get('showInputForValue') ? !isNaN(value) && value > 0 : true;
    }.property('displayValue', 'showInputForValue')

  }),

  /**
   * Mixin for <code>App.AlertConfigProperties.Threshold</code>
   * Used to validate values that should be greater than 0
   * @type {Em.Mixin}
   */
  PositiveMixin: Em.Mixin.create({

    isValid: function () {
      if (!this.get('showInputForValue')) {
        return true;
      }
      var value = this.get('displayValue');

      if (!value) {
        return false;
      }

      //only allow 1/10th of a second
      if (numericUtils.getFloatDecimals(value) > 1) {
        return false;
      }

      value = ('' + value).trim();
      value = parseFloat(value);

      return !isNaN(value) && value > 0;
    }.property('displayValue', 'showInputForValue')

  })

};

});

require.register("models/alerts/alert_definition", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dateUtils = require('utils/date/date');

function computedOnSummaryState(state) {
  return Em.computed('summary.' + state, 'summary.' + state + '.count', 'summary.' + state + '.maintenanceCount', function () {
    var summary = Em.get(this, 'summary');
    return !!summary[state] && !!(summary[state].count || summary[state].maintenanceCount);
  });
}

App.AlertDefinition = DS.Model.extend({

  name: DS.attr('string'),
  label: DS.attr('string'),
  description: DS.attr('string'),
  service: DS.belongsTo('App.Service'),
  serviceName: DS.attr('string'),
  componentName: DS.attr('string'),
  enabled: DS.attr('boolean'),
  repeat_tolerance_enabled: DS.attr('boolean'),
  repeat_tolerance: DS.attr('number'),
  scope: DS.attr('string'),
  interval: DS.attr('number'),
  type: DS.attr('string'),
  helpUrl: DS.attr('string'),
  groups: DS.hasMany('App.AlertGroup'),
  reporting: DS.hasMany('App.AlertReportDefinition'),
  parameters: DS.hasMany('App.AlertDefinitionParameter'),
  lastTriggered: 0,
  lastTriggeredRaw: 0,

  //relates only to SCRIPT-type alert definition
  location: DS.attr('string'),
  //relates only to AGGREGATE-type alert definition
  alertName: DS.attr('string'),
  //relates only to WEB and METRIC types of alert definition
  uri: DS.belongsTo('App.AlertMetricsUriDefinition'),
  //relates only METRIC-type alert definition
  jmx: DS.belongsTo('App.AlertMetricsSourceDefinition'),
  ganglia: DS.belongsTo('App.AlertMetricsSourceDefinition'),
  ams: DS.belongsTo('App.AlertMetricsAmsDefinition'),
  //relates only PORT-type alert definition
  defaultPort: DS.attr('number'),
  portUri: DS.attr('string'),

  /**
   * Raw data from AlertDefinition/source
   * used to format request content for updating alert definition
   * @type {Object}
   */
  rawSourceData: {},

  /**
   * Counts of alert grouped by their status
   * Format:
   * <code>
   *   {
   *    "CRITICAL": {
   *      count: 1,
   *      maintenanceCount: 0
   *    },
   *    "OK": {
   *      count: 0,
   *      maintenanceCount: 1
   *    },
   *    "UNKNOWN": {
   *      count: 0,
   *      maintenanceCount: 0
   *    },
   *    "WARNING": {
   *      count: 1,
   *      maintenanceCount: 1
   *    }
   *   }
   * </code>
   * @type {object}
   */
  summary: {},

  /**
   * Formatted timestamp for latest alert triggering for current alertDefinition
   * @type {string}
   */
  lastTriggeredFormatted: function () {
    var lastTriggered = this.get('lastTriggered');
    return lastTriggered ? dateUtils.dateFormat(lastTriggered) : '';
  }.property('lastTriggered'),

  /**
   * Formatted timestamp with <code>$.timeago</code>
   * @type {string}
   */
  lastTriggeredAgoFormatted: function () {
    var lastTriggered = this.get('lastTriggeredRaw');
    return lastTriggered ? $.timeago(new Date(lastTriggered)) : '';
  }.property('lastTriggeredRaw'),

  lastTriggeredVerboseDisplay: function () {
    var lastTriggered = this.get('lastTriggered');
    return Em.I18n.t('models.alert_definition.triggered.verbose').format(dateUtils.dateFormat(lastTriggered));
  }.property('lastTriggered'),

  /**
   * Formatted timestamp in format: for 4 days
   * @type {string}
   */
  lastTriggeredForFormatted: function () {
    var lastTriggered = this.get('lastTriggeredRaw');
    var previousSuffixAgo = $.timeago.settings.strings.suffixAgo;
    var previousPrefixAgo = $.timeago.settings.strings.prefixAgo;
    $.timeago.settings.strings.suffixAgo = null;
    $.timeago.settings.strings.prefixAgo = 'for';
    var triggeredFor = lastTriggered ? $.timeago(new Date(lastTriggered)) : '';
    $.timeago.settings.strings.suffixAgo = previousSuffixAgo;
    $.timeago.settings.strings.prefixAgo = previousPrefixAgo;
    return triggeredFor;
  }.property('lastTriggeredRaw'),

  /**
   * Formatted displayName for <code>componentName</code>
   * @type {String}
   */
  componentNameFormatted: Em.computed.formatRole('componentName', false),

  hostCnt: function () {
    var order = this.get('order'),
        summary = this.get('summary'),
        hostCnt = 0;
    order.forEach(function (state) {
      hostCnt += summary[state] ? summary[state].count + summary[state].maintenanceCount : 0;
    });
    return hostCnt;
  }.property('order', 'summary'),

  latestText: function () {
    var order = this.get('order'),
        summary = this.get('summary'),
        text = '';
    order.forEach(function (state) {
      var cnt = summary[state] ? summary[state].count + summary[state].maintenanceCount : 0;
      if (cnt > 0) {
        text = Em.getWithDefault(summary[state], 'latestText', '');
      }
    });
    return text;
  }.property('summary'),

  latestTextSummary: function () {
    var latestText = this.get('latestText');
    var ellipsis = '...';
    var summaryLength = 400;
    if (latestText.length > summaryLength) {
      latestText = latestText.substring(0, summaryLength - ellipsis.length) + ellipsis;
    }
    return latestText;
  }.property('latestText'),

  isAmbariService: Em.computed.equal('service._id', 'AMBARI'),

  isAmbariAgentComponent: Em.computed.equal('componentName', 'AMBARI_AGENT'),

  isHostAlertDefinition: Em.computed.and('isAmbariService', 'isAmbariAgentComponent'),

  typeIconClass: Em.computed.getByKey('typeIcons', 'type'),

  isTypeAggregate: Em.computed.equal('type', 'AGGREGATE'),

  /**
   * if this definition is in state: CRITICAL / WARNING, if true, will show up in alerts fast access popup
   * instances with maintenance mode ON are ignored
   * @type {boolean}
   */
  isCriticalOrWarning: Em.computed.or('summary.CRITICAL.count', 'summary.WARNING.count'),

  /**
   * if this definition is in state: CRITICAL
   * @type {boolean}
   */
  isCritical: computedOnSummaryState('CRITICAL'),

  /**
   * if this definition is in state: WARNING
   * @type {boolean}
   */
  isWarning: computedOnSummaryState('WARNING'),

  /**
   * if this definition is in state: OK
   * @type {boolean}
   */
  isOK: computedOnSummaryState('OK'),

  /**
   * if this definition is in state: UNKNOWN
   * @type {boolean}
   */
  isUnknown: computedOnSummaryState('UNKNOWN'),

  /**
   * For alerts we will have processes which are not typical
   * cluster services - like Ambari-Server. This method unifies
   * cluster services and other services into a common display-name.
   * @see App.AlertInstance#serviceDisplayName()
   */
  serviceDisplayName: function () {
    var serviceName = this.get('service.displayName');
    if (!serviceName) {
      serviceName = this.get('serviceName');
      if (serviceName) {
        serviceName = serviceName.toCapital();
      }
    }
    return serviceName;
  }.property('serviceName', 'service.displayName'),

  /**
   * Determines if alert definition has help url
   * @type {boolean}
   */
  hasHelpUrl: Em.computed.bool('helpUrl'),

  /**
   * List of css-classes for alert types
   * @type {object}
   */
  typeIcons: {
    'METRIC': 'glyphicon glyphicon-flash',
    'SCRIPT': 'glyphicon glyphicon-file',
    'WEB': 'glyphicon glyphicon-globe',
    'PORT': 'glyphicon glyphicon-log-in',
    'AGGREGATE': 'glyphicon glyphicon-plus',
    'SERVER': 'glyphicon glyphicon-oil',
    'RECOVERY': 'glyphicon glyphicon-oil',
    'AMS': 'glyphicon glyphicon-stats'
  },

  /**
   * Sort on load definitions by this severity order
   */
  severityOrder: ['CRITICAL', 'WARNING', 'OK', 'UNKNOWN', 'PENDING'],
  order: ['OK', 'WARNING', 'CRITICAL', 'UNKNOWN']
});

App.AlertDefinition.shortState = {
  'CRITICAL': 'CRIT',
  'WARNING': 'WARN',
  'OK': 'OK',
  'UNKNOWN': 'UNKWN',
  'PENDING': 'NONE'
};

App.AlertDefinition.reopenClass({

  /**
   * Return function to sort list of AlertDefinitions by their status
   * It sorts according to <code>severityOrder</code>
   * @param {boolean} order true - DESC, false - ASC
   * @returns {Function}
   * @method getSortDefinitionsByStatus
   */
  getSortDefinitionsByStatus: function getSortDefinitionsByStatus(order) {
    return function (a, b) {
      var aSummary = a.get('summary'),
          bSummary = b.get('summary'),
          stOrder = a.get('severityOrder'),
          ret = 0;
      for (var i = 0; i < stOrder.length; i++) {
        var aV = Em.isNone(aSummary[stOrder[i]]) ? 0 : aSummary[stOrder[i]].count + aSummary[stOrder[i]].maintenanceCount,
            bV = Em.isNone(bSummary[stOrder[i]]) ? 0 : bSummary[stOrder[i]].count + bSummary[stOrder[i]].maintenanceCount;
        ret = bV - aV;
        if (ret !== 0) {
          break;
        }
      }
      return order ? ret : -ret;
    };
  }

});

App.AlertDefinitionParameter = DS.Model.extend({
  name: DS.attr('string'),
  displayName: DS.attr('string'),
  units: DS.attr('string'),
  value: DS.attr('string'),
  description: DS.attr('string'),
  type: DS.attr('string'),
  threshold: DS.attr('string'),
  visibility: DS.attr('string')
});

App.AlertReportDefinition = DS.Model.extend({
  type: DS.attr('string'),
  text: DS.attr('string'),
  value: DS.attr('number')
});

App.AlertMetricsSourceDefinition = DS.Model.extend({
  propertyList: [],
  value: DS.attr('string')
});

App.AlertMetricsUriDefinition = DS.Model.extend({
  http: DS.attr('string'),
  https: DS.attr('string'),
  httpsProperty: DS.attr('string'),
  httpsPropertyValue: DS.attr('string'),
  connectionTimeout: DS.attr('number')
});

App.AlertMetricsAmsDefinition = DS.Model.extend({
  value: DS.attr('string'),
  minimalValue: DS.attr('number'),
  interval: DS.attr('number')
});

App.AlertDefinition.FIXTURES = [];
App.AlertReportDefinition.FIXTURES = [];
App.AlertMetricsSourceDefinition.FIXTURES = [];
App.AlertMetricsUriDefinition.FIXTURES = [];
App.AlertMetricsAmsDefinition.FIXTURES = [];
App.AlertDefinitionParameter.FIXTURES = [];

App.AlertType = DS.Model.extend({
  name: DS.attr('string'),
  displayName: DS.attr('string'),
  iconPath: DS.attr('string'),
  description: DS.attr('string'),
  properties: DS.attr('array')
});

App.AlertType.FIXTURES = [{
  id: 'PORT',
  name: 'PORT',
  icon_path: 'glyphicon glyphicon-log-in',
  display_name: 'Port',
  description: Em.I18n.t('alerts.add.wizard.step1.body.port.description')
}, {
  id: 'WEB',
  name: 'WEB',
  icon_path: 'glyphicon glyphicon-globe',
  display_name: 'Web',
  description: Em.I18n.t('alerts.add.wizard.step1.body.web.description')
}, {
  id: 'METRIC',
  name: 'METRIC',
  display_name: 'Metric',
  icon_path: 'glyphicon glyphicon-flash',
  description: Em.I18n.t('alerts.add.wizard.step1.body.metric.description')
}, {
  id: 'SCRIPT',
  name: 'SCRIPT',
  icon_path: 'glyphicon glyphicon-file',
  display_name: 'Script',
  description: Em.I18n.t('alerts.add.wizard.step1.body.script.description')
}, {
  id: 'AGGREGATE',
  name: 'AGGREGATE',
  icon_path: 'glyphicon glyphicon-plus',
  display_name: 'Aggregate',
  description: Em.I18n.t('alerts.add.wizard.step1.body.aggregate.description')
}, {
  id: 'RAW',
  name: 'RAW',
  icon_path: 'glyphicon glyphicon-align-justify',
  display_name: 'Raw',
  description: Em.I18n.t('alerts.add.wizard.step1.body.raw.description')
}];

});

require.register("models/alerts/alert_group", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Represents an alert-group on the cluster.
 * A alert group is a collection of alert definitions
 *
 * Alert group hierarchy is at 2 levels. For
 * each service there is a 'Default' alert group
 * containing all definitions , this group is read-only
 *
 * User can create new alert group containing alert definitions from
 * any service.
 */
App.AlertGroup = DS.Model.extend({

  name: DS.attr('string'),

  description: DS.attr('string'),

  /**
   * Is this group default for some service
   * @type {boolean}
   */
  default: DS.attr('boolean'),

  /**
   * @type {App.AlertDefinition[]}
   */
  definitions: DS.hasMany('App.AlertDefinition'),

  /**
   * @type {App.AlertNotification[]}
   */
  targets: DS.hasMany('App.AlertNotification'),

  /**
   * @type {string}
   */
  displayName: function () {
    var name = App.config.truncateGroupName(App.format.role(this.get('name'), true));
    return this.get('default') ? name + ' Default' : name;
  }.property('name', 'default'),

  /**
   * @type {string}
   */
  displayNameDefinitions: Em.computed.format('{0} ({1})', 'displayName', 'definitions.length'),

  isAddDefinitionsDisabled: Em.computed.alias('default')

});
App.AlertGroup.FIXTURES = [];

});

require.register("models/alerts/alert_instance", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dateUtils = require('utils/date/date');

App.AlertInstance = DS.Model.extend({
  id: DS.attr('number'),
  label: DS.attr('string'),
  definitionName: DS.attr('string'),
  definitionId: DS.attr('number'),
  service: DS.belongsTo('App.Service'),
  serviceName: DS.attr('string'),
  componentName: DS.attr('string'),
  host: DS.belongsTo('App.Host'),
  hostName: DS.attr('string'),
  scope: DS.attr('string'),
  originalTimestamp: DS.attr('number'),
  originalRawTimestamp: DS.attr('number'),
  latestTimestamp: DS.attr('number'),
  maintenanceState: DS.attr('string'),
  instance: DS.attr('string'),
  state: DS.attr('string'),
  text: DS.attr('string'),
  repeatTolerance: DS.attr('number'),
  repeatToleranceRemaining: DS.attr('number'),
  notification: DS.hasMany('App.AlertNotification'),

  /**
   * @type {boolean}
   */
  isMaintenanceStateOn: Em.computed.equal('maintenanceState', 'ON'),

  /**
   * @type {string}
   */
  shortStateMsg: Em.computed.getByKey('shortState', 'state'),

  /**
   * @type {string}
   */
  stateClass: function () {
    return 'alert-state-' + (this.get('isMaintenanceStateOn') ? 'PENDING' : this.get('state'));
  }.property('isMaintenanceStateOn'),

  /**
   * For alerts we will have processes which are not typical
   * cluster services - like Ambari-Server. This method unifies
   * cluster services and other services into a common display-name.
   * @see App.AlertDefinition#serviceDisplayName()
   */
  serviceDisplayName: function () {
    var serviceName = this.get('service.displayName');
    if (!serviceName) {
      serviceName = this.get('serviceName');
      if (serviceName) {
        serviceName = serviceName.toCapital();
      }
    }
    return serviceName;
  }.property('serviceName', 'service.displayName'),

  /**
   * Formatted timestamp for latest instance triggering
   * @type {string}
   */
  lastCheckedFormatted: function () {
    return dateUtils.dateFormat(this.get('latestTimestamp'));
  }.property('latestTimestamp'),

  /**
   * Formatted timestamp for latest instance triggering
   * @type {string}
   */
  lastTriggeredFormatted: function () {
    return dateUtils.dateFormat(this.get('originalTimestamp'));
  }.property('originalTimestamp'),

  /**
   * Formatted timestamp with <code>$.timeago</code>
   * @type {string}
   */
  lastTriggeredAgoFormatted: function () {
    var lastTriggered = this.get('originalRawTimestamp');
    return lastTriggered ? $.timeago(new Date(lastTriggered)) : '';
  }.property('originalTimestamp'),

  lastTriggeredVerboseDisplay: function () {
    var originalTimestamp = this.get('originalTimestamp');
    var latestTimestamp = this.get('latestTimestamp');
    return Em.I18n.t('models.alert_instance.tiggered.verbose').format(dateUtils.dateFormat(originalTimestamp), dateUtils.dateFormat(latestTimestamp));
  }.property('originalTimestamp', 'latestTimestamp'),

  /**
   * Formatted timestamp with <code>$.timeago</code>
   * @type {string}
   */
  lastTriggeredForFormatted: function () {
    var lastTriggered = this.get('originalRawTimestamp');
    var previousSuffixAgo = $.timeago.settings.strings.suffixAgo;
    var previousPrefixAgo = $.timeago.settings.strings.prefixAgo;
    $.timeago.settings.strings.suffixAgo = null;
    $.timeago.settings.strings.prefixAgo = 'for';
    var triggeredFor = lastTriggered ? $.timeago(new Date(lastTriggered)) : '';
    $.timeago.settings.strings.suffixAgo = previousSuffixAgo;
    $.timeago.settings.strings.prefixAgo = previousPrefixAgo;
    return triggeredFor;
  }.property('originalTimestamp'),

  /**
  * escaped '<' and '>' special characters.
  * @type {string}
  */
  escapeSpecialCharactersFromTooltip: function () {
    var displayedText = this.get('text');
    return displayedText.replace(/[<>]/g, '');
  }.property('text'),

  /**
   * Formatted lastChecked and lastTriggered timestamp
   * @returns {string}
   */
  statusChangedAndLastCheckedFormatted: Em.computed.i18nFormat('models.alert_definition.triggered.checked', 'lastTriggeredFormatted', 'lastCheckedFormatted'),

  /**
   * List of css-classes for alert instance status
   * @type {object}
   */
  typeIcons: {
    'DISABLED': 'glyphicon glyphicon-off'
  },

  repeatToleranceReceived: function () {
    return this.get('repeatTolerance') - this.get('repeatToleranceRemaining');
  }.property('repeatToleranceRemaining', 'repeatTolerance'),

  retryText: function () {
    return this.get('state') === 'OK' ? '' : Em.I18n.t('models.alert_definition.check.retry').format(this.get('repeatToleranceReceived'), this.get('repeatTolerance'));
  }.property('state', 'repeatToleranceRemaining', 'repeatTolerance'),

  /**
   * Define if definition serviceName is Ambari
   * Used in some logic in templates to distinguish definitions with Ambari serviceName
   * @returns {boolean}
   */
  isAmbariServiceName: Em.computed.equal('serviceName', 'AMBARI'),

  shortState: {
    'CRITICAL': 'CRIT',
    'WARNING': 'WARN',
    'OK': 'OK',
    'UNKNOWN': 'UNKWN',
    'PENDING': 'NONE'
  }
});

App.AlertInstance.FIXTURES = [];

});

require.register("models/alerts/alert_instance_local", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Just a copy of <code>App.AlertInstance</code>, used to save separately alert instances for host and definition
 * Sometimes alert instances can't be stored in the one model. Example: user is in the definition details page and open
 * alerts popup. Different lists of instances should be shown and loaded. Also it's pretty hard to determine, which
 * models should be removed from store, when user navigates in the popup or just leave the page
 * @type {DS.Model}
 */
App.AlertInstanceLocal = App.AlertInstance.extend({});

App.AlertInstanceLocal.FIXTURES = [];

});

require.register("models/alerts/alert_notification", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AlertNotification = DS.Model.extend({
  id: DS.attr('number'),
  name: DS.attr('string'),
  type: DS.attr('string'),
  description: DS.attr('string'),
  groups: DS.hasMany('App.AlertGroup'),
  global: DS.attr('boolean'),
  enabled: DS.attr('boolean'),
  displayName: Ember.computed('enabled', 'name', function () {
    return this.get('name') + (this.get('enabled') === true ? '' : ' (Disabled)');
  }),

  properties: {},
  alertStates: []
});

App.AlertNotification.FIXTURES = [];

});

require.register("models/authentication", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.Authentication = DS.Model.extend({
  method: DS.attr('boolean'), // use LDAP
  primaryServer: DS.attr('string'),
  secondaryServer: DS.attr('string'),
  useSsl: DS.attr('boolean'),
  bindMethod: DS.attr('boolean'), // use credentials
  bindUser: DS.attr('string'),
  password: DS.attr('string'),
  passwordRetype: DS.attr('string'),
  searchBaseDn: DS.attr('string'),
  usernameAttribute: DS.attr('string')
});

App.Authentication.FIXTURES = [{
  id: 1,
  method: 0,
  primary_server: "",
  secondary_server: "",
  use_ssl: false,
  bind_method: 0,
  bind_user: "",
  password: "",
  password_retype: "",
  search_base_dn: "",
  username_attribute: ""
}];

App.AuthenticationForm = App.Form.extend({
  testResult: false,
  fieldsOptions: [{ name: "method", displayName: "", isRequired: false, displayType: "select",
    values: [{ value: 0, label: Em.I18n.t("admin.authentication.form.method.database") }, { value: 1, label: Em.I18n.t("admin.authentication.form.method.ldap") }]
  }, { name: "primaryServer", displayName: Em.I18n.t("admin.authentication.form.primaryServer"), /*validator:'ipaddress',*/
    isRequired: function () {
      return this.get('form.field.method.value');
    }.property('form.field.method.value')
  }, { name: "secondaryServer", displayName: Em.I18n.t("admin.authentication.form.secondaryServer"), /*validator:'ipaddress',*/isRequired: false }, { name: "useSsl", displayName: Em.I18n.t("admin.authentication.form.useSsl"), displayType: "checkbox", isRequired: false }, { name: "bindMethod", displayName: '', displayType: "select", isRequired: false,
    values: [{ value: 0, label: Em.I18n.t("admin.authentication.form.bind.anonymously") }, { value: 1, label: Em.I18n.t("admin.authentication.form.bind.useCrenedtials") }] }, { name: "bindUser", displayName: Em.I18n.t('admin.authentication.form.bindUserDN'), isRequired: function () {
      return this.get('form.field.bindMethod.value');
    }.property('form.field.bindMethod.value') }, { name: "password", displayName: Em.I18n.t('common.password'), displayType: "password",
    isRequired: function () {
      return this.get('form.field.bindMethod.value');
    }.property('form.field.bindMethod.value') }, { name: "passwordRetype", displayName: Em.I18n.t('form.passwordRetype'), displayType: "password",
    validator: "passwordRetype",
    isRequired: function () {
      return this.get('form.field.bindMethod.value');
    }.property('form.field.bindMethod.value') }, { name: "searchBaseDn", displayName: Em.I18n.t('admin.authentication.form.searchBaseDN'),
    isRequired: function () {
      return this.get('form.field.method.value');
    }.property('form.field.method.value')
  }, { name: "usernameAttribute", displayName: Em.I18n.t('admin.authentication.form.usernameAttribute'),
    isRequired: function () {
      return this.get('form.field.method.value');
    }.property('form.field.method.value')
  }, { name: "userDN", displayName: Em.I18n.t('admin.authentication.form.userDN') }, { name: "userPassword", displayName: Em.I18n.t('common.password'), displayType: 'password' }],
  fields: [],
  testConfiguration: function testConfiguration() {
    this.set('testResult', parseInt(Math.random() * 2));
    return true;
  },
  testConfigurationMessage: Em.computed.ifThenElse('testResult', Em.I18n.t('admin.authentication.form.test.success'), Em.I18n.t('admin.authentication.form.test.fail')),

  testConfigurationClass: Em.computed.ifThenElse('testResult', 'text-success', 'text-danger')

});

});

require.register("models/background_operation", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.BackgroundOperation = DS.Model.extend({
  operationName: DS.attr('string'),
  events: DS.hasMany('App.BackgroundOperationEvent'),
  cluster: DS.belongsTo('App.Cluster'),
  host: DS.belongsTo('App.Host'),
  operationLog: DS.attr('string')
});

App.BackgroundOperation.FIXTURES = [{
  id: 1,
  operation_name: 'Decommissioning host1',
  operation_log: 'Decommissioning log',
  events: [1, 2],
  cluster_id: 1,
  host_id: 1
}, {
  id: 2,
  operation_name: 'Starting DataNode on host4',
  operation_log: 'Starting DataNode log',
  events: [3],
  cluster_id: 1,
  host_id: 1
}];

App.BackgroundOperationEvent = DS.Model.extend({
  eventName: DS.attr('string'),
  operation: DS.belongsTo('App.BackgroundOperation'),
  eventDate: DS.attr('string')
});

App.BackgroundOperationEvent.FIXTURES = [{
  id: 1,
  event_name: 'Some intermediate operation',
  operation_id: 1,
  event_date: '4 min ago'
}, {
  id: 2,
  event_name: 'Operation started',
  operation_id: 1,
  event_date: '5 min ago'
}, {
  id: 3,
  event_name: 'Operation started',
  operation_id: 2,
  event_date: '5 min ago'
}];

});

require.register("models/client_component", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.ClientComponent = DS.Model.extend({
  service: DS.belongsTo('App.Service'),
  componentName: DS.attr('string'),
  displayName: DS.attr('string'),
  installedCount: DS.attr('number', { defaultValue: 0 }),
  installFailedCount: DS.attr('number', { defaultValue: 0 }),
  initCount: DS.attr('number', { defaultValue: 0 }),
  unknownCount: DS.attr('number', { defaultValue: 0 }),
  startedCount: DS.attr('number', { defaultValue: 0 }),
  totalCount: DS.attr('number', { defaultValue: 0 }),
  stackInfo: DS.belongsTo('App.StackServiceComponent'),
  hostNames: DS.attr('array'),
  staleConfigHosts: DS.attr('array'),

  /**
   * Determines if component may be deleted
   *
   * @type {boolean}
   */
  allowToDelete: function () {
    return this.get('totalCount') === this.get('installedCount') + this.get('installFailedCount') + this.get('initCount') + this.get('unknownCount');
  }.property('totalCount', 'installedCount', 'installFailedCount', 'initCount', 'unknownCount'),

  summaryLabelClassName: function () {
    return 'label_for_' + this.get('componentName').toLowerCase();
  }.property('componentName'),

  summaryValueClassName: function () {
    return 'value_for_' + this.get('componentName').toLowerCase();
  }.property('componentName'),

  displayNamePluralized: function () {
    return stringUtils.pluralize(this.get('installedCount'), this.get('displayName'));
  }.property('installedCount')
});

App.ClientComponent.getModelByComponentName = function (componentName) {
  if (App.HostComponent.isMaster(componentName)) {
    return App.MasterComponent.find(componentName);
  } else if (App.HostComponent.isSlave(componentName)) {
    return App.SlaveComponent.find(componentName);
  } else if (App.HostComponent.isClient(componentName)) {
    return App.ClientComponent.find(componentName);
  }
  return null;
};

App.ClientComponent.FIXTURES = [];

});

require.register("models/cluster", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var credentialUtils = require('utils/credentials');

App.Cluster = DS.Model.extend({
  clusterName: DS.attr('string'),
  stackName: DS.attr('string'),
  version: DS.attr('string'),
  totalHosts: DS.attr('number'),
  securityType: DS.attr('string'),
  credentialStoreProperties: DS.attr('object', { defaultValue: {} }),
  /**
   * Array containing desired configs. New array
   * should be set by instances of class.
   */
  desiredConfigs: null,

  isKerberosEnabled: Em.computed.equal('securityType', 'KERBEROS'),

  isCredentialStorePersistent: function () {
    return this.get('credentialStoreProperties')[credentialUtils.STORE_TYPES.PERSISTENT_PATH] === "true";
  }.property('credentialStoreProperties')
});

App.Cluster.FIXTURES = [];

});

require.register("models/cluster_states", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
var LZString = require('utils/lz-string');
App.clusterStatus = Em.Object.create(App.Persist, {

  /**
   * Cluster name
   * @type {string}
   */
  clusterName: '',

  /**
   * List valid cluster states
   * @type {string[]}
   */
  validStates: ['DEFAULT', 'CLUSTER_NOT_CREATED_1', 'CLUSTER_DEPLOY_PREP_2', 'CLUSTER_INSTALLING_3', 'SERVICE_STARTING_3', 'CLUSTER_INSTALLED_4', 'ADD_HOSTS_DEPLOY_PREP_2', 'ADD_HOSTS_INSTALLING_3', 'ADD_HOSTS_INSTALLED_4', 'ADD_SERVICES_DEPLOY_PREP_2', 'ADD_SERVICES_INSTALLING_3', 'ADD_SERVICES_INSTALLED_4', 'STOPPING_SERVICES', 'STACK_UPGRADING', 'STACK_UPGRADE_FAILED', 'STACK_UPGRADED', 'ADD_SECURITY_STEP_1', 'ADD_SECURITY_STEP_2', 'ADD_SECURITY_STEP_3', 'ADD_SECURITY_STEP_4', 'DISABLE_SECURITY', 'HIGH_AVAILABILITY_DEPLOY', 'ROLLBACK_HIGH_AVAILABILITY'],

  /**
   * Default cluster state
   * @type {string}
   */
  clusterState: 'CLUSTER_NOT_CREATED_1',

  /**
   * Current used wizard <code>controller.name</code>
   * @type {string|null}
   */
  wizardControllerName: null,

  /**
   * Local DB
   * @type {object|null}
   */
  localdb: null,

  /**
   * Persist key
   * @type {string}
   */
  key: 'CLUSTER_CURRENT_STATUS',

  /**
   * Is cluster installed
   * @type {bool}
   */
  isInstalled: Em.computed.notExistsIn('clusterState', ['CLUSTER_NOT_CREATED_1', 'CLUSTER_DEPLOY_PREP_2', 'CLUSTER_INSTALLING_3', 'SERVICE_STARTING_3']),

  /**
   * Stores instance of <code>App.ModalPopup</code> created by <code>postUserPrefErrorCallback</code>
   * @property {App.ModalPopup|null}
   */
  persistErrorModal: null,

  /**
   * General info about cluster
   * @type {{clusterName: string, clusterState: string, wizardControllerName: string, localdb: object}}
   */
  value: function () {
    return {
      clusterName: this.get('clusterName'),
      clusterState: this.get('clusterState'),
      wizardControllerName: this.get('wizardControllerName'),
      localdb: this.get('localdb')
    };
  }.property('clusterName', 'clusterState', 'localdb', 'wizardControllerName'),

  /**
   * get cluster data from server and update cluster status
   * @param {bool} overrideLocaldb
   * @return promise object for the get call
   * @method updateFromServer
   */
  updateFromServer: function updateFromServer(overrideLocaldb) {
    this.set('additionalData', {
      user: App.db.getUser(),
      login: App.db.getLoginName(),
      auth: App.db.getAuth(),
      overrideLocaldb: !overrideLocaldb
    });
    return this.getUserPref(this.get('key'));
  },

  /**
   * Success callback for get-persist request
   * @param {object} response
   * @param {object} opt
   * @param {object} params
   * @method getUserPrefSuccessCallback
   */
  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response, opt, params) {
    if (response) {
      // decompress response
      if ((typeof response === 'undefined' ? 'undefined' : _typeof(response)) != 'object') {
        response = JSON.parse(LZString.decompressFromBase64(response));
      }
      if (response.clusterState) {
        this.set('clusterState', response.clusterState);
      }
      if (response.clusterName) {
        this.set('clusterName', response.clusterName);
      }
      if (response.wizardControllerName) {
        this.set('wizardControllerName', response.wizardControllerName);
      }
      if (response.localdb && !$.isEmptyObject(response.localdb)) {
        this.set('localdb', response.localdb);
        // restore HAWizard data if process was started
        var isHAWizardStarted = App.isAuthorized('SERVICE.ENABLE_HA') && !App.isEmptyObject(response.localdb.HighAvailabilityWizard);
        // restore Kerberos Wizard is started
        var isKerberosWizardStarted = App.isAuthorized('CLUSTER.TOGGLE_KERBEROS') && !App.isEmptyObject(response.localdb.KerberosWizard);
        if (params.data.overrideLocaldb || isHAWizardStarted || isKerberosWizardStarted) {
          var localdbTables = App.db.data.app && App.db.data.app.tables ? App.db.data.app.tables : {};
          var authenticated = Em.get(App, 'db.data.app.authenticated') || false;
          App.db.data = response.localdb;
          App.db.setLocalStorage();
          App.db.setUser(params.data.user);
          App.db.setLoginName(params.data.login);
          App.db.setAuth(params.data.auth);
          App.db.setAuthenticated(authenticated);
          App.db.data.app.tables = localdbTables;
        }
      }
    }
    // this is to ensure that the local storage namespaces are initialized with all expected namespaces.
    // after upgrading ambari, loading local storage data from the "persist" data saved via an older version of
    // Ambari can result in missing namespaces that are defined in the new version of Ambari.
    App.db.mergeStorage();
  },

  /**
   * Error callback for get-persist request
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @method getUserPrefErrorCallback
   */
  getUserPrefErrorCallback: function getUserPrefErrorCallback(request, ajaxOptions, error) {
    if (request.status == 404) {
      // default status already set
      return;
    }
    App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile('<p>{{t common.update.error}}</p>')
      })
    });
  },

  /**
   * update cluster status and post it on server.
   * This function should always be called by admin user
   * @param {object} newValue
   * @param {object} opt - Can have additional params for ajax callBacks and sender
   *                 opt.successCallback
   *                 opt.successCallbackData
   *                 opt.errorCallback
   *                 opt.errorCallbackData
   *                 opt.alwaysCallback
   *                 opt.alwaysCallbackData
   *                 opt.sender
   * @method setClusterStatus
   * @return {*}
   */
  setClusterStatus: function setClusterStatus(newValue, opt) {
    if (App.get('testMode')) return false;
    var user = App.db.getUser();
    var auth = App.db.getAuth();
    var login = App.db.getLoginName();
    var val = { clusterName: this.get('clusterName') };
    if (newValue) {
      App.db.cleanTmp();
      //setter
      if (newValue.clusterName) {
        this.set('clusterName', newValue.clusterName);
        val.clusterName = newValue.clusterName;
      }

      if (newValue.clusterState) {
        this.set('clusterState', newValue.clusterState);
        val.clusterState = newValue.clusterState;
      }
      if (newValue.wizardControllerName) {
        this.set('wizardControllerName', newValue.wizardControllerName);
        val.wizardControllerName = newValue.wizardControllerName;
      }

      if (newValue.localdb) {
        if (newValue.localdb.app && newValue.localdb.app.user) delete newValue.localdb.app.user;
        if (newValue.localdb.app && newValue.localdb.app.auth) delete newValue.localdb.app.auth;
        if (newValue.localdb.app && newValue.localdb.app.loginName) delete newValue.localdb.app.loginName;
        if (newValue.localdb.app && newValue.localdb.app.tables) delete newValue.localdb.app.tables;
        if (newValue.localdb.app && newValue.localdb.app.authenticated) delete newValue.localdb.app.authenticated;
        this.set('localdb', newValue.localdb);
        val.localdb = newValue.localdb;
      } else {
        delete App.db.data.app.user;
        delete App.db.data.app.auth;
        delete App.db.data.app.loginName;
        delete App.db.data.app.tables;
        delete App.db.data.app.authenticated;
        val.localdb = App.db.data;
        App.db.setUser(user);
        App.db.setAuth(auth);
        App.db.setLoginName(login);
      }
      if (!$.mocho) {
        // compress val
        val = LZString.compressToBase64(JSON.stringify(val));
        this.postUserPref(this.get('key'), val).done(function () {
          !!opt && Em.typeOf(opt.successCallback) === 'function' && opt.successCallback.call(opt.sender || this, opt.successCallbackData);
        }).fail(function () {
          !!opt && Em.typeOf(opt.errorCallback) === 'function' && opt.errorCallback.call(opt.sender || this, opt.errorCallbackData);
        }).always(function () {
          !!opt && Em.typeOf(opt.alwaysCallback) === 'function' && opt.alwaysCallback.call(opt.sender || this, opt.alwaysCallbackData);
        });
      }
      return newValue;
    }
  },

  /**
   * Error callback for post-persist request
   * @param {object} request
   * @param {object} ajaxOptions
   * @param {string} error
   * @method postUserPrefErrorCallback
   */
  postUserPrefErrorCallback: function postUserPrefErrorCallback(request, ajaxOptions, error) {
    var msg = '',
        doc;
    try {
      msg = 'Error ' + request.status + ' ';
      doc = $.parseXML(request.responseText);
      msg += $(doc).find("body p").text();
    } catch (e) {
      msg += JSON.parse(request.responseText).message;
    }

    if (this.get('persistErrorModal')) {
      if (this.get('persistErrorModal').get('state') === 'destroyed') {
        this.set('persistErrorModal', null);
      } else {
        this.get('persistErrorModal').onPrimary();
        this.set('persistErrorModal', null);
      }
    }

    var modal = App.ModalPopup.show({
      header: Em.I18n.t('common.error'),
      secondary: false,
      response: msg,
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile('<p>{{t common.persist.error}} {{response}}</p>')
      })
    });
    this.set('persistErrorModal', modal);
  }

});

});

require.register("models/configs/config_group", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigGroup = DS.Model.extend({
  /**
   * @property {string}
   */
  id: DS.attr('string'),

  name: DS.attr('string'),
  serviceName: DS.attr('string'),
  description: DS.attr('string'),
  hosts: DS.attr('array'),
  configVersions: DS.hasMany('App.ConfigVersion'),
  service: DS.belongsTo('App.Service'),
  desiredConfigs: DS.attr('array', { defaultValue: [] }),

  /**
   * define if group is persisted on server or is just UI representation
   * temporary groups are deleted from store after persisting on server by <code>App.ServiceConfigGroup.deleteTemporaryRecords</code> method
   * @property {boolean}
   */
  isTemporary: DS.attr('boolean', { defaultValue: false }),

  /**
   * define if group is default
   * @type {boolean}
   */
  isDefault: DS.attr('boolean', { defaultValue: false }),

  /**
   * this flag is used for installed services' config groups
   * if user make changes to them - mark this flag to true
   * @default [false]
   */
  isForUpdate: DS.attr('boolean', { defaultValue: false }),

  /**
   * mark config groups for installed services
   * @default [false]
   */
  isForInstalledService: DS.attr('boolean', { defaultValue: false }),

  /**
   * all hosts that belong to cluster
   */
  clusterHostsBinding: 'App.router.manageConfigGroupsController.clusterHosts',

  /**
   * list of group names that shows which config
   * groups should be updated as dependent when current is changed
   * @type App.ServiceConfigGroup[]
   */
  dependentConfigGroups: DS.attr('object', { defaultValue: {} }),

  /**
   * Parent configuration group for this group.
   * When {@link #isDefault} is true, this value is <code>null</code>
   * When {@link #isDefault} is false, this represents the configuration
   * deltas that are applied on the default.
   */
  parentConfigGroup: DS.belongsTo('App.ServiceConfigGroup'),

  /**
   * Children configuration groups for this group.
   * When {@link #isDefault} is false, this value is <code>null</code>
   * When {@link #isDefault} is true, this represents the various
   * configuration groups that override the default.
   */
  childConfigGroups: DS.hasMany('App.ServiceConfigGroup'),

  hash: DS.attr('string'),

  /**
   * Provides a display friendly name. This includes trimming
   * names to a certain length.
   */
  displayName: function () {
    return App.config.truncateGroupName(this.get('name'));
  }.property('name'),

  /**
   *
   */
  displayNameHosts: Em.computed.format('{0} ({1})', 'displayName', 'hosts.length'),

  switchGroupTextShort: function () {
    return Em.I18n.t('services.service.config_groups.switchGroupTextShort').format(this.get('displayName'));
  }.property('displayName'),

  switchGroupTextFull: function () {
    return Em.I18n.t('services.service.config_groups.switchGroupTextFull').format(this.get('displayName'));
  }.property('displayName'),

  /**
   * Provides hosts which are available for inclusion in
   * non-default configuration groups.
   * @type {Array}
   */
  availableHosts: function () {
    if (this.get('isDefault')) return [];
    var unusedHostsMap = this.get('parentConfigGroup.hosts').toWickMap();
    var availableHosts = [];
    var sharedHosts = this.get('clusterHosts');
    // parentConfigGroup.hosts(hosts from default group) - are available hosts, which don't belong to any group
    sharedHosts.forEach(function (host) {
      if (unusedHostsMap[host.get('id')]) {
        availableHosts.pushObject(Ember.Object.create({
          selected: false,
          host: host,
          hostComponentNames: host.get('hostComponents').mapProperty('componentName')
        }));
      }
    });
    return availableHosts;
  }.property('isDefault', 'parentConfigGroup', 'childConfigGroups', 'parentConfigGroup.hosts.@each', 'clusterHosts'),

  /**
   * @type {boolean}
   */
  isAddHostsDisabled: Em.computed.or('isDefault', '!availableHosts.length'),

  /**
   * @type {Array}
   */
  properties: DS.attr('array', { defaultValue: [] }),

  /**
   * @type {string}
   */
  propertiesList: function () {
    var result = '';

    if (Array.isArray(this.get('properties'))) {
      this.get('properties').forEach(function (item) {
        result += _.escape(item.name) + " : " + _.escape(item.value) + '<br/>';
      }, this);
    }
    return result;
  }.property('properties.length')
});

App.ServiceConfigGroup.FIXTURES = [];

App.ServiceConfigGroup.getParentConfigGroupId = function (serviceName) {
  return App.ServiceConfigGroup.groupId(serviceName, App.ServiceConfigGroup.defaultGroupName);
};

App.ServiceConfigGroup.groupId = function (serviceName, groupName) {
  return serviceName + "_" + groupName;
};

App.ServiceConfigGroup.defaultGroupName = 'Default';

App.ServiceConfigGroup.deletedGroupName = 'Deleted';

/**
 * Delete all records with isTemporary:true
 * @method
 */
App.ServiceConfigGroup.deleteTemporaryRecords = function () {
  App.ServiceConfigGroup.find().filterProperty('isTemporary').forEach(function (record) {
    App.configGroupsMapper.deleteRecord(record);
  }, this);
};

});

require.register("models/configs/objects/service_config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfig = Ember.Object.extend({
  serviceName: '',
  configCategories: [],
  configCategoriesMap: function () {
    var categoriesMap = {};
    this.get('configCategories').forEach(function (c) {
      if (!categoriesMap[c.get('name')]) categoriesMap[c.get('name')] = c;
    });
    return categoriesMap;
  }.property('configCategories.[]'),
  configs: [],
  restartRequired: false,
  restartRequiredMessage: '',
  restartRequiredHostsAndComponents: {},
  configGroups: [],
  dependentServiceNames: [],
  initConfigsLength: 0, // configs length after initialization in order to watch changes

  errorCount: Em.computed.alias('configsWithErrors.length'),

  /**
   * Properties for which some aggregations should be calculated
   * like <code>configsWithErrors<code>, <code>changedConfigProperties<code> etc.
   *
   * @type {Object[]}
   */
  activeProperties: [],

  configsWithErrors: [],

  /**
   * Collection of properties that were changed:
   * for saved properties use - <code>isNotDefaultValue<code>
   * for not saved properties (on wizards, for new services) use
   *    - <code>isNotInitialValue<code>
   * for added properties use - <code>isNotSaved<code>
   * @type {Object[]}
   */
  changedConfigProperties: [],

  setActiveProperties: function () {
    Em.run.once(this, 'setActivePropertiesOnce');
  }.observes('configs.@each.isActive', 'configs.@each.isRequiredByAgent', 'configs.@each.value', 'configs.@each.isUndefinedLabel'),

  setActivePropertiesOnce: function setActivePropertiesOnce() {
    if (this.get('isDestroyed')) return false;
    var activeProperties = this.get('configs').filter(function (c) {
      return (c.get('isActive') || c.get('isUndefinedLabel')) && (c.get('isRequiredByAgent') || c.get('isRequired'));
    });
    this.set('activeProperties', activeProperties);
  },

  setChangedConfigProperties: function () {
    Em.run.once(this, 'setChangedConfigPropertiesOnce');
  }.observes('activeProperties.@each.isNotDefaultValue', 'activeProperties.@each.isNotSaved', 'activeProperties.@each.isNotInitialValue'),

  setChangedConfigPropertiesOnce: function setChangedConfigPropertiesOnce() {
    var changedConfigProperties = this.get('activeProperties').filter(function (c) {
      return c.get('isNotDefaultValue') || c.get('isNotSaved') || c.get('isNotInitialValue');
    }, this);
    this.set('changedConfigProperties', changedConfigProperties);
  },

  setConfigsWithErrors: function () {
    Em.run.once(this, 'setConfigsWithErrorsOnce');
  }.observes('activeProperties.@each.isValid', 'activeProperties.@each.isValidOverride', 'activeProperties.length'),

  setConfigsWithErrorsOnce: function setConfigsWithErrorsOnce() {
    var configsWithErrors = this.get('activeProperties').filter(function (c) {
      return !c.get('isValid') || !c.get('isValidOverride');
    });
    this.set('configsWithErrors', configsWithErrors);
  },

  observeErrors: function () {
    this.get('configCategories').setEach('errorCount', 0);
    this.get('configsWithErrors').forEach(function (c) {
      //configurations with widget shouldn't affect advanced category error counter
      if (this.get('configCategoriesMap')[c.get('category')] && !c.get('isInDefaultTheme')) {
        this.get('configCategoriesMap')[c.get('category')].incrementProperty('errorCount');
      }
    }, this);
  }.observes('configsWithErrors'),

  configTypes: function () {
    return App.StackService.find(this.get('serviceName')).get('configTypeList') || [];
  }.property('serviceName'),

  radioConfigs: Em.computed.filterBy('configs', 'displayType', 'radio button'),

  observeForeignKeys: function () {
    //TODO refactor or move this logic to other place
    Em.run.once(this, 'updateVisibilityByForeignKeys');
  }.observes('radioConfigs.@each.value'),

  updateVisibilityByForeignKeys: function updateVisibilityByForeignKeys() {
    var configs = this.get('configs');
    configs.forEach(function (item) {
      if (item.get('isVisible')) {
        var options = item.get('options');
        if (options && options.someProperty('foreignKeys')) {
          var options = options.filterProperty('foreignKeys');
          options.forEach(function (opt) {
            opt.foreignKeys.forEach(function (key) {
              var config = configs.findProperty('name', key);
              if (config) {
                config.set('isVisible', item.get('value') === opt.displayName);
              }
            });
          });
        }
      }
    });
  },

  /**
   * Config with overrides that has values that differs from saved
   *
   * @type {Object[]}
   */
  configsWithChangedOverrides: Em.computed.filterBy('activeProperties', 'isOverrideChanged', true),

  /**
   * Defines if some configs were added/removed
   * @type {boolean}
   */
  configsLengthWasChanged: Em.computed.notEqualProperties('configs.length', 'initConfigsLength'),

  /**
   * @type {boolean}
   */
  isPropertiesChanged: Em.computed.or('configsLengthWasChanged', 'changedConfigProperties.length', 'configsWithChangedOverrides.length'),

  init: function init() {
    this._super();
    this.set('dependentServiceNames', App.StackService.find(this.get('serviceName')).get('dependentServiceNames') || []);
    this.observeForeignKeys();
    this.setActiveProperties();
  },

  hasConfigIssues: Em.computed.someBy('activeProperties', 'hasIssues', true)
});

App.ConfigSiteTag = Ember.Object.extend({
  site: DS.attr('string'),
  tag: DS.attr('string'),
  /**
   * Object map of hostname->override-tag for overrides.
   * <b>Creators should set new object here.<b>
   */
  hostOverrides: null
});

});

require.register("models/configs/objects/service_config_category", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigCategory = Ember.Object.extend({
  name: null,
  /**
   *  We cant have spaces in the name as this is being used as HTML element id while rendering. Hence we introduced 'displayName' where we can have spaces like 'Secondary Name Node' etc.
   */
  displayName: null,
  /**
   * check whether to show custom view in category instead of default
   */
  isCustomView: false,
  customView: null,
  /**
   * Each category might have a site-name associated (hdfs-site, core-site, etc.)
   * and this will be used when determining which category a particular property
   * ends up in, based on its site.
   */
  siteFileName: null,
  /**
   * Can this category add new properties. Used for custom configurations.
   */
  canAddProperty: false,

  errorCount: 0,

  isAdvanced: function () {
    var name = this.get('name');
    return name.indexOf("Advanced") !== -1;
  }.property('name')
});

});

require.register("models/configs/objects/service_config_property", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @class ServiceConfigProperty
 */
App.ServiceConfigProperty = Em.Object.extend({

  name: '',
  displayName: '',

  /**
   * value that is shown on IU
   * and is changing by user
   * @type {String|null}
   */
  value: '',

  /**
   * value that is saved on cluster configs
   * and stored in /api/v1/clusters/{name}/configurations
   * @type {String|null}
   */
  savedValue: null,

  /**
   * value that is returned from server as recommended
   * or stored on stack
   * @type {String|null}
   */
  recommendedValue: null,

  /**
   * initial value of config. if value is saved it will be initial
   * otherwise first recommendedValue will be initial
   * @type {String|null}
   */
  initialValue: null,

  /**
   * value that is shown on IU
   * and is changing by user
   * @type {boolean}
   */
  isFinal: false,

  /**
   * value that is saved on cluster configs api
   * @type {boolean}
   */
  savedIsFinal: null,

  /**
   * value that is returned from server as recommended
   * or stored on stack
   * @type {boolean}
   */
  recommendedIsFinal: null,

  /**
   * @type {boolean}
   */
  supportsFinal: false,

  /**
   * Hint message to display in tooltip. Tooltip will be wrapped on question mark icon.
   * If value is <code>false</code> no tooltip and question mark icon.
   *
   * @type {boolean|string}
   */
  hintMessage: false,

  /**
   * Display label on the right side from input. In general used for checkbox only.
   *
   * @type {boolean}
   */
  rightSideLabel: false,

  /**
   * type of property
   * @type {String[]}
   * @default empty array
   */
  propertyType: [],

  /**
   * Text to be shown as placeholder
   * By default savedValue is shown as placeholder
   * @type {String}
   */
  placeholderText: '',

  /**
   * type of widget View
   * @type {string}
   * @default null
   */
  widgetType: null,

  /**
   * Placeholder used for configs with input type text
   */
  placeholder: function () {
    if (this.isEditable) {
      return this.get('placeholderText') || this.get('savedValue');
    }
    return null;
  }.property('isEditable', 'placeholderText', 'savedValue'),

  retypedPassword: '',
  description: '',
  displayType: 'string', // string, digits, number, directories, custom
  unit: '',
  category: 'General',
  isRequired: true, // by default a config property is required
  isReconfigurable: true, // by default a config property is reconfigurable
  isEditable: true, // by default a config property is editable
  disabledAsComponentAction: false, // is true for component action configs
  isNotEditable: Em.computed.not('isEditable'),
  hideFinalIcon: Em.computed.and('!isFinal', 'isNotEditable'),
  isVisible: true,
  isMock: false, // mock config created created only to displaying
  isRequiredByAgent: true, // Setting it to true implies property will be stored in configuration
  isSecureConfig: false,
  errorMessage: '',
  warnMessage: '',
  validationErrors: [], // stores messages from validation response marked as ERRROR
  validationWarnings: [], // stores message from validation response marked as WARN
  serviceConfig: null, // points to the parent App.ServiceConfig object
  filename: '',
  isOriginalSCP: true, // if true, then this is original SCP instance and its value is not overridden value.
  parentSCP: null, // This is the main SCP which is overridden by this. Set only when isOriginalSCP is false.
  overrides: null,
  overrideValues: [],
  group: null, // Contain group related to this property. Set only when isOriginalSCP is false.
  isUserProperty: null, // This property was added by user. Hence they get removal actions etc.
  isOverridable: true,
  compareConfigs: [],
  isComparison: false,
  hasCompareDiffs: false,
  showLabel: true,
  isConfigIdentity: false,
  copy: '',

  /**
   * Determines if config exists in the non-default config group but is loaded for default config group
   *
   * @type {boolean}
   */
  isCustomGroupConfig: false,

  /**
   * Determines if config is Undefined label, used for overrides, that do not have original property in default group
   * @type {boolean}
   */
  isUndefinedLabel: function () {
    return this.get('displayType') === 'label' && this.get('value') === 'Undefined';
  }.property('displayType', 'value'),

  error: Em.computed.bool('errorMessage.length'),
  warn: Em.computed.bool('warnMessage.length'),
  hasValidationErrors: Em.computed.bool('validationErrors.length'),
  hasValidationWarnings: Em.computed.bool('validationWarnings.length'),
  isValid: Em.computed.equal('errorMessage', ''),

  previousValue: null, // cached value before changing config <code>value</code>

  /**
   * List of <code>isFinal</code>-values for overrides
   * Set in the controller
   * Should be empty array by default!
   * @type {boolean[]}
   */
  overrideIsFinalValues: [],

  /**
   * true if property has warning or error
   * @type {boolean}
   */
  hasIssues: Em.computed.or('error', 'warn', 'overridesWithIssues.length'),

  overridesWithIssues: Em.computed.filterBy('overrides', 'hasIssues', true),

  index: null, //sequence number in category
  editDone: false, //Text field: on focusOut: true, on focusIn: false
  isNotSaved: false, // user property was added but not saved
  hasInitialValue: false, //if true then property value is defined and saved to server
  isHiddenByFilter: false, //if true then hide this property (filtered out)
  rowStyleClass: null, // CSS-Class to be applied on the row showing this config
  showAsTextBox: false,

  /**
   * config is invisible since wrapper section is hidden
   * @type {boolean}
   */
  hiddenBySection: false,

  /**
   * Determines config visibility on subsection level when wrapped.
   * @type {boolean}
   */
  hiddenBySubSection: false,

  /**
   * Determines visibility state including section/subsection state.
   * When <code>true</code> means that property is shown and may affect validation process.
   * When <code>false</code> means that property won't affect validation.
   */
  isActive: Em.computed.and('isVisible', '!hiddenBySubSection', '!hiddenBySection'),

  /**
   * @type {boolean}
   */
  recommendedValueExists: function () {
    return !Em.isNone(this.get('recommendedValue')) && this.get('recommendedValue') != "" && this.get('isRequiredByAgent') && !this.get('cantBeUndone');
  }.property('recommendedValue'),

  /**
   * Usage example see on <code>App.ServiceConfigRadioButtons.handleDBConnectionProperty()</code>
   *
   * @property {Ember.View} additionalView - custom view related to property
   **/
  additionalView: null,

  /**
   * If config is saved we should compare config <code>value<code> with <code>savedValue<code> to
   * find out if it was changed, but if config in not saved there is no <code>savedValue<code>, so
   * we should use <code>initialValue<code> instead.
   */
  isNotInitialValue: function () {
    if (Em.isNone(this.get('savedValue')) && !Em.isNone(this.get('initialValue'))) {
      var value = this.get('value'),
          initialValue = this.get('initialValue');
      if (this.get('stackConfigProperty.valueAttributes.type') == 'float') {
        initialValue = !Em.isNone(initialValue) ? '' + parseFloat(initialValue) : null;
        value = '' + parseFloat(value);
      }
      return initialValue !== value;
    }
    return false;
  }.property('initialValue', 'savedValue', 'value', 'stackConfigProperty.valueAttributes.type'),

  /**
   * Is property has active override with error
   */
  isValidOverride: function () {
    return this.get('overrides.length') ? !this.get('overrides').find(function (o) {
      return Em.get(o, 'isEditable') && Em.get(o, 'errorMessage');
    }) : true;
  }.property("overrides.@each.errorMessage"),
  /**
   * No override capabilities for fields which are not edtiable
   * and fields which represent master hosts.
   */
  isPropertyOverridable: function () {
    var overrideable = this.get('isOverridable');
    var editable = this.get('isEditable');
    var overrides = this.get('overrides');
    var dt = this.get('displayType');
    return overrideable && (editable || !overrides || !overrides.length) && !["componentHost", "password"].contains(dt);
  }.property('isEditable', 'displayType', 'isOverridable', 'overrides.length'),

  isOverridden: Em.computed.or('overrides.length', '!isOriginalSCP'),

  isOverrideChanged: function () {
    if (Em.isNone(this.get('overrides')) && this.get('overrideValues.length') === 0) return false;
    return JSON.stringify(this.get('overrides').mapProperty('isFinal')) !== JSON.stringify(this.get('overrideIsFinalValues')) || JSON.stringify(this.get('overrides').mapProperty('value')) !== JSON.stringify(this.get('overrideValues'));
  }.property('overrides.@each.isNotDefaultValue', 'overrides.@each.overrideIsFinalValues', 'overrideValues.length'),

  isRemovable: function () {
    return this.get('isEditable') && this.get('isRequiredByAgent') && !(this.get('overrides.length') > 0) && (this.get('isUserProperty') || !this.get('isOriginalSCP'));
  }.property('isUserProperty', 'isOriginalSCP', 'overrides.length', 'isRequiredByAgent'),

  init: function init() {
    this.setInitialValues();
    this.set('viewClass', App.config.getViewClass(this.get("displayType"), this.get('dependentConfigPattern'), this.get('unit')));
    this.set('validateErrors', App.config.getErrorValidator(this.get("displayType")));
    this.set('validateWarnings', App.config.getWarningValidator(this.get("displayType")));
    this.validate();
  },

  setInitialValues: function setInitialValues() {
    if (Em.isNone(this.get('value'))) {
      if (!Em.isNone(this.get('savedValue'))) {
        this.set('value', this.get('savedValue'));
      } else if (!Em.isNone(this.get('recommendedValue'))) {
        this.set('value', this.get('recommendedValue'));
      }
    }
    this.set('previousValue', this.get('value'));
    if (this.get('value') === null) {
      this.set('isVisible', false);
    }
    if (this.get("displayType") === "password") {
      this.set('retypedPassword', this.get('value'));
      this.set('recommendedValue', '');
    }
    this.set('initialValue', this.get('value'));
  },

  /**
   * updates configs list that belongs to config group
   */
  updateGroupConfigs: function () {
    if (this.get('group')) {
      var o = this.get('group.properties').find(function (c) {
        return Em.get(c, 'name') === this.get('name') && Em.get(c, 'filename') === this.get('filename');
      }, this);

      if (o) {
        Em.set(o, 'value', this.get('value'));
      }
    }
  }.observes('value'),

  /**
   * Indicates when value is not the default value.
   * Returns false when there is no default value.
   *
   * @type {boolean}
   */
  isNotDefaultValue: function () {
    var value = this.get('value'),
        savedValue = this.get('savedValue'),
        supportsFinal = this.get('supportsFinal'),
        isFinal = this.get('isFinal'),
        savedIsFinal = this.get('savedIsFinal');

    // ignore precision difference for configs with type of `float` which value may ends with 0
    // e.g. between 0.4 and 0.40
    if (this.get('stackConfigProperty') && this.get('stackConfigProperty.valueAttributes.type') == 'float') {
      savedValue = !Em.isNone(savedValue) ? '' + parseFloat(savedValue) : null;
      value = '' + parseFloat(value);
    }
    return savedValue != null && value !== savedValue || supportsFinal && !Em.isNone(savedIsFinal) && isFinal !== savedIsFinal;
  }.property('value', 'savedValue', 'isEditable', 'isFinal', 'savedIsFinal'),

  /**
   * Don't show "Undo" for hosts on Installer Step7
   */
  cantBeUndone: Em.computed.existsIn('displayType', ["componentHost", "componentHosts", "radio button"]),

  validate: function () {
    if (_typeof(this.get('value')) != 'object' && (this.get('value') + '').length === 0) {
      var widgetType = this.get('widgetType');
      this.set('errorMessage', this.get('isRequired') && !['test-db-connection', 'label'].contains(widgetType) ? Em.I18n.t('errorMessage.config.required') : '');
    } else {
      this.set('errorMessage', this.validateErrors(this.get('value'), this.get('name'), this.get('retypedPassword')));
    }
    if (!this.get('widgetType') || 'text-field' === this.get('widgetType')) {
      //temp conditions, since other warnings are calculated directly in widget view
      this.set('warnMessage', this.validateWarnings(this.get('value'), this.get('name'), this.get('filename'), this.get('stackConfigProperty'), this.get('unit')));
    }
  }.observes('value', 'retypedPassword', 'isEditable'),

  viewClass: App.ServiceConfigTextField,

  validateErrors: function validateErrors() {
    return '';
  },

  validateWarnings: function validateWarnings() {
    return '';
  },

  /**
   * Get override for selected group
   *
   * @param {String} groupName
   * @returns {App.ServiceConfigProperty|null}
   */
  getOverride: function getOverride(groupName) {
    Em.assert('Group name should be defined string', typeof groupName === 'string' && groupName);
    if (this.get('overrides.length')) {
      return this.get('overrides').findProperty('group.name', groupName);
    }
    return null;
  }

});

});

require.register("models/configs/service_config_version", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dateUtil = require('utils/date/date');

App.ServiceConfigVersion = DS.Model.extend({
  MAX_NOTES_LENGTH: 80,
  serviceName: DS.attr('string'),
  displayName: Em.computed.formatRole('serviceName', true),
  groupName: DS.attr('string'),
  groupId: DS.attr('string'),
  version: DS.attr('number'),
  createTime: DS.attr('number'),
  rawCreateTime: DS.attr('number'),
  author: DS.attr('string'),
  notes: DS.attr('string'),
  service: DS.belongsTo('App.Service'),
  hosts: DS.attr('array'),
  index: DS.attr('number'),
  isCurrent: DS.attr('boolean'),
  isDisplayed: DS.attr('boolean'),
  stackVersion: DS.attr('string'),
  isCompatible: DS.attr('boolean'),
  canBeMadeCurrent: Em.computed.and('isCompatible', '!isCurrent'),
  isDefault: function () {
    return this.get('groupName') === App.ServiceConfigGroup.defaultGroupName;
  }.property('groupName'),
  currentTooltip: Em.computed.i18nFormat('dashboard.configHistory.table.current.tooltip', 'displayName', 'configGroupName'),

  /**
   * @type {string}
   */
  configGroupName: function () {
    return this.get('isDefault') ? App.ServiceConfigGroup.defaultGroupName : this.get('groupName');
  }.property('groupName', 'isDefault'),

  /**
   * @type {Boolean}
   */
  isConfigGroupDeleted: function () {
    return this.get('groupName') === App.ServiceConfigGroup.deletedGroupName;
  }.property('groupName'),

  /**
   * @type {string}
   */
  authorFormatted: Em.computed.truncate('author', 15, 15),

  /**
   * @type {string}
   */
  fullNotes: function () {
    return typeof this.get('notes') === 'string' ? this.get('notes') || Em.I18n.t('dashboard.configHistory.table.notes.no') : Em.I18n.t('dashboard.configHistory.table.notes.no');
  }.property('notes'),

  /**
   * @type {string}
   */
  briefNotes: Em.computed.truncate('fullNotes', 81, 81, ''),

  /**
   * @type {boolean}
   */
  moreNotesExists: Em.computed.notEqualProperties('fullNotes', 'briefNotes'),

  /**
   * @type {string}
   */
  versionText: Em.computed.i18nFormat('dashboard.configHistory.table.version.versionText', 'version'),

  /**
   * @type {string}
   */
  makeCurrentButtonText: Em.computed.i18nFormat('dashboard.configHistory.info-bar.revert.versionButton', 'versionText'),

  /**
   * @type {string}
   */
  createdDate: function () {
    return dateUtil.dateFormat(this.get('createTime'));
  }.property('createTime'),

  /**
   * @type {string}
   */
  timeSinceCreated: function () {
    return $.timeago(this.get('rawCreateTime'));
  }.property('rawCreateTime'),

  /**
   * determine whether ServiceConfigVersion is requested from server
   */
  isRequested: DS.attr('boolean'),

  /**
   * @type {boolean}
   */
  isRestartRequired: function () {
    if (this.get('service.isRestartRequired') && this.get('isCurrent')) {
      var hostNames = this.get('hosts');
      if (!hostNames.length) return false;
      for (var i = 0; i < hostNames.length; i++) {
        if (Object.keys(this.get('service.restartRequiredHostsAndComponents')).contains(hostNames[i])) {
          return true;
        }
      }
    }
    return false;
  }.property('service.isRestartRequired', 'isDefault', 'isCurrent', 'hosts', 'service.restartRequiredHostsAndComponents', 'router.mainServiceInfoConfigsController.configGroups'),

  /**
   * {{view: string, compare: string, revert: string}} disabledActionMessages
   */
  disabledActionMessages: function () {
    return {
      view: this.get('isDisplayed') ? Em.I18n.t('dashboard.configHistory.info-bar.view.button.disabled') : '',
      compare: this.get('isDisplayed') ? Em.I18n.t('dashboard.configHistory.info-bar.compare.button.disabled') : '',
      revert: this.get('isCurrent') ? Em.I18n.t('dashboard.configHistory.info-bar.revert.button.disabled') : ''
    };
  }.property('isDisplayed', 'isCurrent'),

  /**
   * {{view: (string|boolean), compare: (string|boolean), revert: (string|boolean)}} disabledActionAttr
   */
  disabledActionAttr: function () {
    var isNonWizardUser = App.router.get('wizardWatcherController.isNonWizardUser');
    return {
      view: this.get('isDisplayed') || isNonWizardUser ? 'disabled' : false,
      compare: this.get('isDisabled') || isNonWizardUser || this.get('isDisplayed') ? 'disabled' : false,
      revert: this.get('isDisabled') || isNonWizardUser || this.get('isCurrent') ? 'disabled' : false
    };
  }.property('isDisplayed', 'isCurrent', 'isDisabled', 'App.router.wizardWatcherController.isNonWizardUser')
});

App.ServiceConfigVersion.FIXTURES = [];

});

require.register("models/configs/stack_config_property", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
//TODO delete this file soon
App.StackConfigProperty = DS.Model.extend({
  /**
   * id is consist of property <code>name<code>+<code>fileName<code>
   */
  id: DS.attr('string'),
  /**
   * name of property that is taken from stack
   * @property {string}
   */
  name: DS.attr('string'),

  /**
   * display name of property that is taken from ui or
   * if UI has no info about property this can be formatted or plain name of property
   * @property {string}
   */
  displayName: DS.attr('string'),

  /**
   * filename of property that is taken from stack
   * ex: "hdfs-site.xml"
   * @property {string}
   */
  fileName: DS.attr('string'),

  /**
   * same as fileName
   * @property {string}
   */
  filename: Em.computed.alias('fileName'),

  /**
   * description of config property meaning
   * @property {string}
   */
  description: DS.attr('string'),

  /**
   * defaultValue value of property is taken from stack before cluster is created
   * after cluster created is taken from cluster properties value
   * @property {string}
   */
  recommendedValue: DS.attr('string'),

  /**
   * defines if property support usage <code>isFinal<code> flag
   * @property {boolean}
   */
  supportsFinal: DS.attr('boolean', { defaultValue: true }),

  /**
   * defines the recommended value of <code>isFinal<code> value
   * @property {boolean}
   */
  recommendedIsFinal: DS.attr('boolean', { defaultValue: false }),

  /**
   * type of property
   * @property {string[]}
   */
  type: DS.attr('array', { defaultValue: [] }),

  /**
   * service name
   * @property {string}
   */
  serviceName: DS.attr('string', { defaultValue: 'MISC' }),

  /**
   * stack name
   * @property {string}
   */
  stackName: DS.attr('string'),

  /**
   * stack version
   * @property {string}
   */
  stackVersion: DS.attr('string'),

  /**
   * describe widget details
   * @property {object}
   */
  widget: DS.attr('object', { defaultValue: null }),

  /**
   * widget type
   * @property {string}
   */
  widgetType: DS.attr('string', { defaultValue: null }),

  /**
   * this property contains array of properties which value
   * is dependent from current property
   * @property {array}
   */
  propertyDependedBy: DS.attr('array', { defaultValue: [] }),

  /**
   * this property contains list of file type and name of properties
   * which values depends on current property
   * @property {object[]}
   */
  propertyDependsOn: DS.attr('array', { defaultValue: [] }),

  /**
   * info for displaying property
   * example:
   * {
   *    "type": "value-list",
   *    "entries": ["true", "false"],
   *    "entry_labels": ["Active", "Inactive"],
   *    "entries_editable": "false",
   *    "selection_cardinality": "1"
   * },
   * OR
   * {
   *    "type": "int",
   *    "minimum": "512",
   *    "maximum": "10240",
   *    "unit": "MB"
   * },
   * OR
   * {
   *    "type": "value-list",
   *    "entries": ["New_MySQL_Database", "Existing_MySQL_Database", "Existing_PostgreSQL_Database", "Existing_Oracle_Database"],
   *    "entry_labels": ["New MySQL Database", "Existing MySQL Database", "Existing PostgreSQL Database", "Existing Oracle Database"],
   *    "entry_descriptions": ["d1", "d2", "d3", "d4"],
   *    "entries_editable": "false",
   *    "selection_cardinality": "1" // 0+, 1+, etc.
   * }
   * @property {object}
   */
  valueAttributes: DS.attr('object', { defaultValue: null }),

  /**
   * sub section to which belongs this property
   * @property {App.SubSection}
   */
  subSection: DS.belongsTo('App.SubSection'),

  /**
   * sub section tab to which belongs this property
   * @property {App.SubSectionTab}
   */
  subSectionTab: DS.belongsTo('App.SubSectionTab'),

  /******************************* UI properties ****************************************/

  /**
   * if property should be saved to server
   * @property {boolean}
   */
  isRequiredByAgent: DS.attr('boolean', { defaultValue: true }),

  /**
   * defines category name of property
   * used for advanced tab
   * @property {string}
   */
  category: DS.attr('string'),

  /**
   * config property value same as default
   * @property {string}
   */
  value: DS.attr('string'),

  /**
   * config property isFinal value same as recommendedIsFinal
   * @property {boolean}
   */
  isFinal: DS.attr('boolean', { defaultValue: false }),

  /**
   * @type {boolean}
   */
  index: DS.attr('number', { defaultValue: null }),

  /**
   * defines if property is listed on some default config theme
   * @type {boolean}
   */
  isInDefaultTheme: DS.attr('boolean', { defaultValue: false }),

  /**
   * defines what kind of value this property contains
   * ex: string, digits, number, directories, custom
   * @property {string}
   */
  displayType: function () {
    return this.getAttribute('type', 'string');
  }.property('valueAttributes.type'),

  /**
   * defines if the property can be overriden in the host config group
   * @type {boolean}
   */
  isOverridable: function () {
    return this.getAttribute('overridable', true);
  }.property('valueAttributes.overridable'),

  /**
   * defines if the property is visible or hidden
   * @type {boolean}
   */
  isVisible: function () {
    return this.getAttribute('visible', true);
  }.property('valueAttributes.visible'),

  /**
   * defines if the value of property can be left empty or not
   * @type {boolean}
   */
  isRequired: function () {
    return !this.getAttribute('empty_value_valid', false);
  }.property('valueAttributes.empty_value_valid'),

  /**
   * defines if the value of property can be reconfigured post-install
   * @type {boolean}
   */
  isReconfigurable: function () {
    return !this.getAttribute('editable_only_at_install', false);
  }.property('valueAttributes.editable_only_at_install'),

  /**
   * defines if the name of the property is visible or hidden
   * @type {boolean}
   */
  showLabel: function () {
    return this.getAttribute('show_property_name', true);
  }.property('valueAttributes.show_property_name'),

  /**
   * defines if the property is editable or not
   * @type {boolean}
   */
  isEditable: function () {
    return this.getAttribute('read_only', true);
  }.property('valueAttributes.read_only'),

  /**
   * defines if the property is editable or not
   * @type {boolean}
   */
  unit: function () {
    return this.getAttribute('unit', '');
  }.property('valueAttributes.unit'),

  /**
   *
   * @param propertyName
   * @param defaultValue
   * @returns {string}
   */
  getAttribute: function getAttribute(propertyName, defaultValue) {
    var result = defaultValue;
    if (!Em.empty(this.get('valueAttributes.' + propertyName))) {
      result = this.get('valueAttributes.' + propertyName);
    }
    return result;
  }
});

App.StackConfigProperty.FIXTURES = [];

App.StackConfigValAttributesMap = {
  'overridable': 'isOverridable',
  'visible': 'isVisible',
  'empty_value_valid': 'isRequired',
  'editable_only_at_install': 'isReconfigurable',
  'show_property_name': 'showLabel',
  'read_only': 'isEditable',
  'ui_only_property': 'isRequiredByAgent'
};

});

require.register("models/configs/theme/config_action", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * THIS IS NOT USED FOR NOW
 * FOR CONFIG GROUPS WE ARE USING OLD MODELS AND LOGIC
 */

var App = require('app');

App.ConfigAction = DS.Model.extend({

  componentName: DS.attr('string'),

  /**
   * Name of the config that is being affected with the condition
   */
  configName: DS.attr('string'),

  /**
   * File name to which the config getting affected belongs
   */
  fileName: DS.attr('string'),

  /**
   * conditional String which can be evaluated to boolean result.
   * If evaluated result of this staring is true then use the statement provided by `then` attribute.
   * Otherwise use the attribute provided by `else` attributes
   */
  if: DS.attr('string'),
  then: DS.attr('string'),
  else: DS.attr('string'),
  hostComponentConfig: DS.attr('object', {
    defaultValue: function defaultValue() {
      return {};
    }
  }),
  actionType: DS.attr('string'),
  popupProperties: DS.attr('object', {
    defaultValue: function defaultValue() {
      return {};
    }
  }),
  serviceName: DS.attr('string'),
  // TODO remove after stack advisor is able to handle this case
  // dependencies is used as workaround for hadoop.proxyuser.{{hiveUser}}.hosts after adding Hive Server Interactive from Install Wizard
  dependencies: DS.attr('object', {
    defaultValue: function defaultValue() {
      return {};
    }
  })
});

App.ConfigAction.FIXTURES = [{
  id: 1,
  component_name: 'HIVE_SERVER_INTERACTIVE',
  config_name: "enable_hive_interactive",
  file_name: "hive-interactive-env.xml",
  if: '${hive-interactive-env/enable_hive_interactive}',
  then: 'add',
  else: 'delete',
  // TODO remove after stack advisor is able to handle this case
  // dependencies is used as workaround for hadoop.proxyuser.{{hiveUser}}.hosts after adding Hive Server Interactive from Install Wizard
  dependencies: {
    initializer: {
      name: 'AddHiveServerInteractiveInitializer',
      setupKeys: ['hiveUser'],
      componentNames: ['HIVE_SERVER', 'WEBHCAT_SERVER', 'HIVE_METASTORE', 'HIVE_SERVER_INTERACTIVE']
    },
    properties: [{
      fileName: 'core-site',
      nameTemplate: 'hadoop.proxyuser.{{hiveUser}}.hosts',
      isHostsList: true,
      isHostsArray: false
    }],
    foreignKeys: [{
      key: 'hiveUser',
      fileName: 'hive-env.xml',
      propertyName: 'hive_user'
    }]
  }
}, {
  id: 2,
  service_name: 'YARN',
  component_name: 'RESOURCEMANAGER',
  config_name: "capacity-scheduler",
  file_name: "capacity-scheduler.xml",
  action_type: "showPopup",
  popup_properties: {
    primaryButton: {
      label: Em.I18n.t('popup.confirmation.refreshYarnQueues.buttonText'),
      metaData: {
        name: 'service.item.refreshQueueYarnRequest',
        command: "REFRESHQUEUES",
        context: Em.I18n.t('services.service.actions.run.yarnRefreshQueues.context')
      }
    },
    body: Em.I18n.t('popup.confirmation.refreshYarnQueues.body'),
    conditionalWarning: true,
    warningMessage: Em.I18n.t('popup.warning.refreshYarnQueues.body'),
    errorMessage: Em.I18n.t('services.service.actions.run.yarnRefreshQueues.error')
  }
}];

});

require.register("models/configs/theme/section", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.Section = DS.Model.extend({

  id: DS.attr('string'),

  /**
   * @type {string}
   */
  name: DS.attr('string'),

  /**
   * @type {string}
   */
  displayName: DS.attr('string'),

  /**
   * @type {number}
   */
  rowIndex: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  rowSpan: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  columnIndex: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  columnSpan: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  sectionColumns: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  sectionRows: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {App.SubSection[]}
   */
  subSections: DS.hasMany('App.SubSection'),

  /**
   * @type {App.Tab}
   */
  tab: DS.belongsTo('App.Tab'),

  /**
   * Number of the errors in all subsections in the current section
   * @type {number}
   */
  errorsCount: function () {
    var errors = this.get('subSections').filterProperty('isSectionVisible').mapProperty('errorsCount');
    return errors.length ? errors.reduce(Em.sum, 0) : 0;
  }.property('subSections.@each.errorsCount', 'subSections.@each.isSectionVisible'),

  /**
   * @type {boolean}
   */
  isFirstRow: Em.computed.equal('rowIndex', 0),

  /**
   * @type {boolean}
   */
  isMiddleRow: function () {
    return this.get('rowIndex') != 0 && this.get('rowIndex') + this.get('rowSpan') < this.get('tab.rows');
  }.property('rowIndex', 'rowSpan', 'tab.rows'),

  /**
   * @type {boolean}
   */
  isLastRow: function () {
    return this.get('rowIndex') + this.get('rowSpan') == this.get('tab.rows');
  }.property('rowIndex', 'rowSpan', 'tab.rows'),

  /**
   * @type {boolean}
   */
  isFirstColumn: Em.computed.equal('columnIndex', 0),

  /**
   * @type {boolean}
   */
  isMiddleColumn: function () {
    return this.get('columnIndex') != 0 && this.get('columnIndex') + this.get('columnSpan') < this.get('tab.columns');
  }.property('columnIndex', 'columnSpan', 'tab.columns'),

  /**
   * @type {boolean}
   */
  isLastColumn: function () {
    return this.get('columnIndex') + this.get('columnSpan') == this.get('tab.columns');
  }.property('columnIndex', 'columnSpan', 'tab.columns'),

  /**
   * Determines if section is filtered out (all it's subsections should be hidden)
   * @type {boolean}
   */
  isHiddenByFilter: Em.computed.everyBy('subSections', 'isSectionVisible', false)

});

App.Section.FIXTURES = [];

});

require.register("models/configs/theme/sub_section", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SubSection = DS.Model.extend({

  id: DS.attr('string'),

  /**
   * @type {string}
   */
  name: DS.attr('string'),

  /**
   * theme from which this is coming from , eg: default, database, credentials, etc.
   */
  themeName: DS.attr('string'),

  /**
   * @type {string}
   */
  displayName: DS.attr('string'),

  /**
   * @type {boolean}
   */
  border: DS.attr('boolean', { defaultValue: false }),

  /**
   * @type {number}
   */
  rowIndex: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  columnIndex: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  rowSpan: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {number}
   */
  columnSpan: DS.attr('number', { defaultValue: 1 }),

  /**
   * @type {App.Section}
   */
  section: DS.belongsTo('App.Section'),

  /**
   * @type {String[]}
   */
  configProperties: DS.attr('array', { defaultValue: [] }),

  /**
   * @type {App.SubSectionTab[]}
   */
  subSectionTabs: DS.hasMany('App.SubSectionTab'),

  dependsOn: DS.attr('array', { defaultValue: [] }),

  /**
   * @type {boolean}
   */
  leftVerticalSplitter: DS.attr('boolean', { defaultValue: true }),

  /**
   * @type {App.ServiceConfigProperty[]}
   */
  configs: [],

  /**
   * @type {boolean}
   */
  hasTabs: Em.computed.bool('subSectionTabs.length'),

  someSubSectionTabIsVisible: Em.computed.someBy('subSectionTabs', 'isVisible', true),

  showTabs: Em.computed.and('hasTabs', 'someSubSectionTabIsVisible'),

  visibleProperties: function () {
    return this.get('configs').filter(function (c) {
      return c.get('isVisible') && !c.get('hiddenBySection');
    });
  }.property('configs.@each.isVisible', 'configs.@each.hiddenBySection'),

  visibleTabs: Em.computed.filterBy('subSectionTabs', 'isVisible', true),

  /**
   * Number of the errors in all configs
   * @type {number}
   */
  errorsCount: function () {
    var propertiesWithErrors = this.get('visibleProperties').filter(function (c) {
      return !c.get('isValid') || !c.get('isValidOverride');
    }).length;
    var tabsWithErrors = this.get('visibleTabs').mapProperty('errorsCount').reduce(Em.sum, 0);
    return propertiesWithErrors + tabsWithErrors;
  }.property('visibleProperties.@each.isValid', 'visibleProperties.@each.isValidOverride', 'visibleTabs.@each.errorsCount'),

  /**
   * @type {boolean}
   */
  addLeftVerticalSplitter: Em.computed.and('!isFirstColumn', 'leftVerticalSplitter'),

  /**
   * @type {boolean}
   */
  addRightVerticalSplitter: Em.computed.not('isLastColumn'),

  /**
   * @type {boolean}
   */
  showTopSplitter: Em.computed.and('!isFirstRow', '!border'),

  /**
   * @type {boolean}
   */
  isFirstRow: Em.computed.equal('rowIndex', 0),

  /**
   * @type {boolean}
   */
  isMiddleRow: function () {
    return this.get('rowIndex') != 0 && this.get('rowIndex') + this.get('rowSpan') < this.get('section.sectionRows');
  }.property('rowIndex', 'rowSpan', 'section.sectionRows'),

  /**
   * @type {boolean}
   */
  isLastRow: function () {
    return this.get('rowIndex') + this.get('rowSpan') == this.get('section.sectionRows');
  }.property('rowIndex', 'rowSpan', 'section.sectionRows'),

  /**
   * @type {boolean}
   */
  isFirstColumn: Em.computed.equal('columnIndex', 0),

  /**
   * @type {boolean}
   */
  isMiddleColumn: function () {
    return this.get('columnIndex') != 0 && this.get('columnIndex') + this.get('columnSpan') < this.get('section.sectionColumns');
  }.property('columnIndex', 'columnSpan', 'section.sectionColumns'),

  /**
   * @type {boolean}
   */
  isLastColumn: function () {
    return this.get('columnIndex') + this.get('columnSpan') == this.get('section.sectionColumns');
  }.property('columnIndex', 'columnSpan', 'section.sectionColumns'),

  /**
   * If the visibility of subsection is dependent on a value of some config
   */
  isHiddenByConfig: false,

  /**
   * Determines if subsection is filtered by checking it own configs
   * If there is no configs, subsection can't be hidden
   * @type {boolean}
   */
  isHiddenByFilter: function () {
    var configs = this.get('configs').filter(function (c) {
      return !c.get('hiddenBySection') && c.get('isVisible');
    });
    return configs.length ? configs.everyProperty('isHiddenByFilter', true) && !this.get('someSubSectionTabIsVisible') : false;
  }.property('configs.@each.isHiddenByFilter'),

  /**
   * @type {boolean}
   */
  someConfigIsVisible: Em.computed.someBy('configs', 'isVisible', true),

  /**
   * Determines if subsection is visible
   * @type {boolean}
   */
  isSectionVisible: Em.computed.and('!isHiddenByFilter', '!isHiddenByConfig', 'someConfigIsVisible')

});

App.SubSection.FIXTURES = [];

});

require.register("models/configs/theme/sub_section_tab", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SubSectionTab = DS.Model.extend({

  id: DS.attr('string'),

  /**
   * @type {string}
   */
  name: DS.attr('string'),

  /**
   * theme from which this is coming from , eg: default, database, credentials, etc.
   */
  themeName: DS.attr('string'),

  /**
   * @type {string}
   */
  displayName: DS.attr('string'),

  /**
   * @type {App.Section}
   */
  subSection: DS.belongsTo('App.SubSection'),

  /**
   * @type {String[]}
   */
  configProperties: DS.attr('array', { defaultValue: [] }),

  /**
   * @type {App.ServiceConfigProperty[]}
   */
  configs: [],

  dependsOn: DS.attr('array', { defaultValue: [] }),

  /**
   * @type {boolean}
   */
  isActive: DS.attr('boolean', { defaultValue: false }),

  visibleProperties: function () {
    return this.get('configs').filter(function (c) {
      return c.get('isVisible') && !c.get('hiddenBySection');
    });
  }.property('configs.@each.isVisible', 'configs.@each.hiddenBySection'),

  /**
   * Number of the errors in all configs
   * @type {number}
   */
  errorsCount: function () {
    return this.get('visibleProperties').filter(function (config) {
      return !config.get('isValid') || !config.get('isValidOverride');
    }).length;
  }.property('visibleProperties.@each.isValid', 'visibleProperties.@each.isValidOverride'),

  /**
   * If the visibility of subsection is dependent on a value of some config
   */
  isHiddenByConfig: false,

  /**
   * Determines if subsection is filtered by checking it own configs
   * If there is no configs, subsection can't be hidden
   * @type {boolean}
   */
  isHiddenByFilter: function () {
    var configs = this.get('visibleProperties');
    return configs.length ? configs.everyProperty('isHiddenByFilter', true) : false;
  }.property('configs.@each.isHiddenByFilter').volatile(),

  /**
   * @type {boolean}
   */
  someConfigIsVisible: Em.computed.gt('visibleProperties.length', 0),

  /**
   * Determines if subsection is visible
   * @type {boolean}
   */
  isVisible: function () {
    return !this.get('isHiddenByFilter') && !this.get('isHiddenByConfig') && this.get('someConfigIsVisible');
  }.property('isHiddenByFilter', 'isHiddenByConfig', 'someConfigIsVisible').volatile()

});

App.SubSectionTab.FIXTURES = [];

});

require.register("models/configs/theme/tab", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.Tab = DS.Model.extend({
  id: DS.attr('string'),
  name: DS.attr('string'),
  displayName: DS.attr('string'),
  columns: DS.attr('number', { defaultValue: 1 }),
  rows: DS.attr('number', { defaultValue: 1 }),
  isAdvanced: DS.attr('boolean', { defaultValue: false }),
  serviceName: DS.attr('string'),
  sections: DS.hasMany('App.Section'),
  isAdvancedHidden: DS.attr('boolean', { defaultValue: false }),
  isRendered: DS.attr('boolean', { defaultValue: false }),
  themeName: DS.attr('string'),

  /**
   * @type {boolean}
   * @default false
   */
  isActive: false,

  isHidden: false,

  /**
   * Determines if all <code>configs</code> were attached to tab.
   */
  isConfigsPrepared: DS.attr('boolean', { defaultValue: false }),

  /**
   * Number of the errors in all sections in the current tab
   * @type {number}
   */
  errorsCount: Em.computed.sumBy('sections', 'errorsCount'),

  /**
   * Class name used for tab switching
   *
   * @type {String}
   * @property headingClass
   */
  headingClass: Em.computed.format('.{0}', 'id'),

  /**
   * tooltip message.
   * for now used when tab is disabled
   * @type {String}
   */
  tooltipMsg: Em.computed.ifThenElse('isHiddenByFilter', Em.I18n.t('services.service.config.nothing.to.display'), ''),

  /**
   * @type {boolean}
   */
  allSectionsAreHiddenByFilter: Em.computed.everyBy('sections', 'isHiddenByFilter', true),

  /**
   * Determines if tab is filtered out (all it's sections should be hidden)
   * If it's an Advanced Tab it can't be hidden
   * @type {boolean}
   */
  isHiddenByFilter: Em.computed.ifThenElseByKeys('isAdvanced', 'isAdvancedHidden', 'allSectionsAreHiddenByFilter'),

  /**
   * define whether tab is related to specific category
   * @type {boolean}
   */
  isCategorized: function () {
    return !this.get('isAdvanced') && this.get('themeName') !== 'default';
  }.property('isAdvanced', 'themeName')

});

App.Tab.FIXTURES = [];

});

require.register("models/configs/theme/theme_condition", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * THIS IS NOT USED FOR NOW
 * FOR CONFIG GROUPS WE ARE USING OLD MODELS AND LOGIC
 */

var App = require('app');

App.ThemeCondition = DS.Model.extend({
  /**
   * unique id generated as <code>config_name<code><code>filename<code>
   * @property {string}
   */
  id: DS.attr('string'),

  /**
   *  type can be `config`, `service`
   */
  type: DS.attr('string'),

  /**
   * This is specific to subsection and subsectionTab type
   */

  name: DS.attr('string'),

  /**
   * theme from which this is coming from , eg: default, database, credentials, etc.
   */
  themeName: DS.attr('string'),

  /**
   * Name of the config that is being affected with the condition
   */
  configName: DS.attr('string'),

  /**
   * File name to which the config getting affected belongs
   */
  fileName: DS.attr('string'),

  /**
   * List of configs whose values affect the config
   * Each Object in an array consists of configName and fileName
   */
  configs: DS.attr('array', { defaultValue: [] }),

  /**
   * resource can be `config`, `service`
   */
  resource: DS.attr('string', { defaultValue: 'config' }),

  /**
   * conditional String which can be evaluated to boolean result.
   * If evaluated result of this staring is true then use the statement provided by `then` attribute.
   * Otherwise use the attribute provided by `else` attributes
   */
  if: DS.attr('string'),
  then: DS.attr('object', { defaultValue: null }),
  else: DS.attr('object', { defaultValue: null })

});

App.ThemeCondition.FIXTURES = [];

});

require.register("models/finished_upgrade_entity", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @type {Ember.Object}
 * @class
 */
App.finishedUpgradeEntity = Em.Object.extend({

  /**
   * type of entity "GROUP", "ITEM", "TASK"
   * @type {string}
   */
  type: null,

  /**
   * @type {boolean}
   */
  isExpanded: false,

  /**
   * @type {boolean}
   */
  hasExpandableItems: false,

  /**
   * @type {boolean}
   */
  isVisible: Em.computed.notEqual('status', 'PENDING'),

  /**
   * status of tasks/items/groups which should be grayed out and disabled
   * @type {Array}
   */
  nonActiveStates: ['PENDING'],

  /**
   * @type {boolean}
   */
  isRunning: Em.computed.existsIn('status', ['IN_PROGRESS']),

  /**
   * @type {number}
   */
  progress: function () {
    return Math.floor(this.get('progress_percent'));
  }.property('progress_percent'),

  /**
   * indicate whether entity has active link
   * @type {boolean}
   */
  isActive: function () {
    return !this.get('nonActiveStates').contains(this.get('status'));
  }.property('status'),

  /**
   * indicate whether upgrade group should be expanded
   * @type {boolean}
   */
  isExpandableGroup: function () {
    return this.get('type') === 'GROUP' && (this.get('isActive') || this.get('hasExpandableItems'));
  }.property('isActive', 'hasExpandableItems'),

  upgradeItemStatus: Em.computed.firstNotBlank('display_status', 'status'),

  /**
   * @type {string}
   */
  upgradeGroupStatus: function () {
    if (this.get('type') === 'GROUP' && !this.get('isActive') && this.get('hasExpandableItems')) {
      return 'SUBITEM_FAILED';
    }
    return this.get('display_status') || this.get('status');
  }.property('isExpandableGroup', 'display_status', 'status')
});

});

require.register("models/form", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

// move this to models cause some errors
App.Form = Em.View.extend({
  /**
   * generating fields from fieldsOptions
   */
  classNames: ["form-horizontal"],
  attributeBindings: ['autocomplete'],
  autocomplete: 'off',
  i18nprefix: 'form.',
  fields: [],
  field: {},
  messages: [],
  object: false,
  result: 0, // save result var (-1 - error; 0 - init; 1 - success)
  templateName: require('templates/common/form'),
  tagName: 'form',

  init: function init() {
    var thisForm = this;
    if (!this.fields.length) {
      this.fieldsOptions.forEach(function (options) {
        var field = App.FormField.create(options);
        field.set('form', thisForm);
        thisForm.fields.push(field);
        thisForm.set("field." + field.get('name'), field);
      });
    }
    this._super();
  },

  /**
   * get field of form by name
   * @param name
   * @return {Object}
   */
  getField: function getField(name) {
    return this.get('fields').findProperty('name', name);
  },

  isValid: function isValid() {
    var isValid = true;
    $.each(this.fields, function () {
      this.validate();
      if (!this.get('isValid')) {
        isValid = false;
      }
    });

    return isValid;
  },

  updateValues: function () {
    var object = this.get('object');
    if (object instanceof Em.Object) {
      $.each(this.fields, function () {
        this.set('value', this.get('displayType') == 'password' ? '' : object.get(this.get('name')));
      });
    } else {
      this.clearValues();
    }
  }.observes("object"),

  /**
   * reset values to default of every field in the form
   */
  clearValues: function clearValues() {
    this.get('fields').forEach(function (field) {
      var value = field.get('defaultValue') === undefined ? '' : field.get('defaultValue');
      field.set('value', value);
    }, this);
  },

  visibleFields: Em.computed.filterBy('fields', 'isHiddenField', false),

  resultText: function () {
    var text = "";
    switch (this.get('result')) {
      case -1:
        text = this.t("form.saveError");
        break;
      case 1:
        text = this.t("form.saveSuccess");
        break;
    }

    return text;
  }.property('result')
});

App.FormField = Em.Object.extend({ // try to realize this as view
  name: '',
  displayName: '',
  //  defaultValue:'', NOT REALIZED YET
  description: '',
  disabled: false,
  displayType: 'string', // string, digits, number, directories, textarea, checkbox
  disableRequiredOnPresent: false,
  errorMessage: '',
  warnMessage: '',
  form: false,
  isRequired: true, // by default a config property is required
  unit: '',
  value: '',

  isValid: Em.computed.equal('errorMessage', ''),

  viewClass: function () {
    var options = {};
    var element = Em.TextField;
    switch (this.get('displayType')) {
      case 'checkbox':
        element = App.CheckboxView;
        options.checkedBinding = "value";
        break;
      case 'select':
        element = Em.Select;
        options.content = this.get('values');
        options.valueBinding = "value";
        options.optionValuePath = "content.value";
        options.optionLabelPath = "content.label";
        break;
      case 'password':
        options['type'] = 'password';
        break;
      case 'textarea':
        element = Em.TextArea;
        break;
      case 'hidden':
        options.type = "hidden";
        break;
    }

    return element.extend(options);
  }.property('displayType'),

  validate: function validate() {
    var value = this.get('value');
    var isError = false;
    this.set('errorMessage', '');

    if (this.get('isRequired') && typeof value === 'string' && value.trim().length === 0) {
      this.set('errorMessage', 'This is required');
      isError = true;
    }

    if (typeof value === 'string' && value.trim().length === 0) {
      // this is not to validate empty field.
      isError = true;
    }

    if (!isError) {
      this.set('errorMessage', '');
    }
  },

  isHiddenField: Em.computed.equal('displayType', 'hidden')
});

});

require.register("models/host", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var misc = require('utils/misc');

App.Host = DS.Model.extend({
  hostName: DS.attr('string'),
  publicHostName: DS.attr('string'),
  cluster: DS.belongsTo('App.Cluster'),
  hostComponents: DS.hasMany('App.HostComponent'),
  notStartedComponents: DS.hasMany('App.HostComponent'),
  componentsWithStaleConfigs: DS.hasMany('App.HostComponent'),
  componentsInPassiveState: DS.hasMany('App.HostComponent'),
  cpu: DS.attr('string'),
  cpuPhysical: DS.attr('string'),
  memory: DS.attr('string'),
  diskTotal: DS.attr('number'),
  diskFree: DS.attr('number'),
  osArch: DS.attr('string'),
  ip: DS.attr('string'),
  rack: DS.attr('string'),
  healthStatus: DS.attr('string'),
  state: DS.attr('string'),
  lastHeartBeatTime: DS.attr('number'),
  hasJcePolicy: DS.attr('string'),
  osType: DS.attr("string"),
  diskInfo: DS.attr('object'),
  loadOne: DS.attr('number'),
  loadFive: DS.attr('number'),
  loadFifteen: DS.attr('number'),
  memTotal: DS.attr('number'),
  memFree: DS.attr('number'),
  cpuSystem: DS.attr('number'),
  cpuUser: DS.attr('number'),
  alertsSummary: DS.attr('object'),
  passiveState: DS.attr('string'),
  index: DS.attr('number'),
  stackVersions: DS.hasMany('App.HostStackVersion'),

  isFiltered: DS.attr('boolean'),

  /**
   * Is host checked at the main Hosts page
   */
  selected: DS.attr('boolean'),

  isActive: Em.computed.equal('passiveState', 'OFF'),

  criticalWarningAlertsCount: function () {
    var alertsSummary = this.get('alertsSummary');
    return alertsSummary ? (alertsSummary.CRITICAL || 0) + (alertsSummary.WARNING || 0) : 0;
  }.property('alertsSummary.CRITICAL', 'alertsSummary.WARNING'),

  currentVersion: function () {
    var current = this.get('stackVersions').findProperty('isCurrent');
    return current ? current.get('repoVersion') : null;
  }.property('stackVersions.@each.isCurrent'),

  /**
   * Overall CPU usage (system and user)
   * @returns {Number}
   */
  cpuUsage: function () {
    if (this.get('cpuSystem') && this.get('cpuUser')) {
      return this.get('cpuSystem') + this.get('cpuUser');
    }
    return 0;
  }.property('cpuSystem', 'cpuUser'),

  /**
   * Percent value of used memory
   * @returns {Number}
   */
  memoryUsage: function () {
    if (this.get('memFree') && this.get('memTotal')) {
      var memUsed = this.get('memTotal') - this.get('memFree');
      return 100 * memUsed / this.get('memTotal');
    }
    return 0;
  }.property('memTotal', 'memFree'),

  /**
   * @type {number}
   */
  componentsInPassiveStateCount: Em.computed.alias('componentsInPassiveState.length'),

  /**
   * Get count of host components with stale configs
   * @returns {Number}
   */
  componentsWithStaleConfigsCount: Em.computed.alias('componentsWithStaleConfigs.length'),

  /**
   * Count of mounted on host disks
   * @returns {Number}
   */
  disksMounted: Em.computed.alias('diskInfo.length'),

  coresFormatted: Em.computed.format('{0} ({1})', 'cpu', 'cpuPhysical'),

  /**
   * API return diskTotal and diskFree. Need to save their different
   * @returns {Number}
   */
  diskUsed: function () {
    return this.get('diskTotal') - this.get('diskFree');
  }.property('diskFree', 'diskTotal'),

  /**
   * Format diskUsed value to float with 2 digits (also convert to GB)
   * @returns {String} Format: '*** GB'
   */
  diskUsedFormatted: function () {
    return Math.round(this.get('diskUsed') * Math.pow(10, 2)) / Math.pow(10, 2) + 'GB';
  }.property('diskUsed'),

  /**
   * Format diskTotal value to float with 2 digits (also convert to GB)
   * @returns {String} Format: '*** GB'
   */
  diskTotalFormatted: function () {
    return Math.round(this.get('diskTotal') * Math.pow(10, 2)) / Math.pow(10, 2) + 'GB';
  }.property('diskTotal'),

  /**
   * Percent value of used disk space
   * @returns {Number}
   */
  diskUsage: Em.computed.percents('diskUsed', 'diskTotal', 2),

  /**
   * Format diskUsage to float with 2 digits
   * @returns {String} Format: '**.** %'
   */
  diskUsageFormatted: function () {
    if (isNaN(this.get('diskUsage')) || this.get('diskUsage') < 0) {
      return Em.I18n.t('hosts.host.metrics.dataUnavailable');
    }
    var s = Math.round(this.get('diskUsage') * Math.pow(10, 2)) / Math.pow(10, 2);
    if (isNaN(s)) {
      s = 0;
    }
    return s + '%';
  }.property('diskUsage'),

  /**
   * Formatted string with data about disk usage
   * @returns {String}
   */
  diskInfoBar: function () {
    if (isNaN(this.get('diskUsage')) || this.get('diskUsage') < 0) {
      return this.get('diskUsageFormatted');
    }
    return this.get('diskUsedFormatted') + '/' + this.get('diskTotalFormatted') + ' (' + this.get('diskUsageFormatted') + ' ' + Em.I18n.t('services.service.summary.diskInfoBar.used') + ')';
  }.property('diskUsedFormatted', 'diskTotalFormatted'),

  /**
   * Formatted bytes to appropriate value
   * @returns {String}
   */
  memoryFormatted: function () {
    return misc.formatBandwidth(this.get('memory') * 1024);
  }.property('memory'),

  /**
   * Return true if the host <code>healthStatus</code> is UNKNOWN
   * @returns {bool}
   */
  isNotHeartBeating: function () {
    return this.get('state') === "HEARTBEAT_LOST";
  }.property('state'),

  /**
   * Average load
   * @returns {Number}
   */
  loadAvg: function () {
    if (this.get('loadOne') != null) return this.get('loadOne').toFixed(2);
    if (this.get('loadFive') != null) return this.get('loadFive').toFixed(2);
    if (this.get('loadFifteen') != null) return this.get('loadFifteen').toFixed(2);
    return null;
  }.property('loadOne', 'loadFive', 'loadFifteen'),

  /**
   * Host health indicator
   * Based on  <code>healthStatus</code>
   * @returns {String}
   */
  healthClass: function () {
    if (this.get('passiveState') != 'OFF') {
      return 'icon-medkit';
    }
    var statusMap = {
      'UNKNOWN': 'health-status-DEAD-YELLOW',
      'HEALTHY': 'health-status-LIVE',
      'UNHEALTHY': 'health-status-DEAD-RED',
      'ALERT': 'health-status-DEAD-ORANGE'
    };
    return statusMap[this.get('healthStatus')] || 'health-status-DEAD-YELLOW';
  }.property('healthStatus', 'passiveState'),

  healthIconClass: Em.computed.getByKey('healthIconClassMap', 'healthClass', ''),

  healthIconClassMap: {
    'health-status-LIVE': App.healthIconClassGreen,
    'health-status-DEAD-RED': App.healthIconClassRed,
    'health-status-DEAD-YELLOW': App.healthIconClassYellow,
    'health-status-DEAD-ORANGE': App.healthIconClassOrange
  },

  /**
   * Tooltip for host indicator
   * Contains affected host components names (based on <code>healthClass</code>)
   * @returns {String}
   */
  healthToolTip: function () {
    var hostComponents = this.get('notStartedComponents');
    var output = '';
    if (this.get('passiveState') != 'OFF') {
      return Em.I18n.t('hosts.host.passive.mode');
    }
    switch (this.get('healthClass')) {
      case 'health-status-DEAD-RED':
        hostComponents = hostComponents.filterProperty('isMaster');
        output = Em.I18n.t('hosts.host.healthStatus.mastersDown');
        hostComponents.forEach(function (hc, index) {
          output += index == hostComponents.length - 1 ? hc.get('displayName') : hc.get('displayName') + ", ";
        }, this);
        break;
      case 'health-status-DEAD-YELLOW':
        output = Em.I18n.t('hosts.host.healthStatus.heartBeatNotReceived');
        break;
      case 'health-status-DEAD-ORANGE':
        hostComponents = hostComponents.filterProperty('isSlave');
        output = Em.I18n.t('hosts.host.healthStatus.slavesDown');
        hostComponents.forEach(function (hc, index) {
          output += index == hostComponents.length - 1 ? hc.get('displayName') : hc.get('displayName') + ", ";
        }, this);
        break;
      case 'health-status-LIVE':
        output = Em.I18n.t('hosts.host.healthStatus.allUp');
        break;
    }
    return output;
  }.property('passiveState', 'healthClass', 'notStartedComponents.@each')
});

App.Host.FIXTURES = [];

});

require.register("models/host_component", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HostComponent = DS.Model.extend({
  workStatus: DS.attr('string'),
  passiveState: DS.attr('string'),
  componentName: DS.attr('string'),
  displayName: DS.attr('string'),
  haStatus: DS.attr('string'),
  displayNameAdvanced: DS.attr('string'),
  staleConfigs: DS.attr('boolean'),
  host: DS.belongsTo('App.Host'),
  componentLogs: DS.belongsTo('App.HostComponentLog'),
  hostName: DS.attr('string'),
  publicHostName: DS.attr('string'),
  service: DS.belongsTo('App.Service'),
  adminState: DS.attr('string'),
  haNameSpace: DS.attr('string'),
  clusterIdValue: DS.attr('string'),

  serviceDisplayName: Em.computed.truncate('service.displayName', 14, 11),

  getDisplayName: Em.computed.truncate('displayName', 30, 25),

  getDisplayNameAdvanced: Em.computed.truncate('displayNameAdvanced', 30, 25),

  summaryLabelClassName: function () {
    return 'label_for_' + this.get('componentName').toLowerCase();
  }.property('componentName'),

  summaryValueClassName: function () {
    return 'value_for_' + this.get('componentName').toLowerCase();
  }.property('componentName'),
  /**
   * Determine if component is client
   * @returns {bool}
   */
  isClient: function () {
    return App.HostComponent.isClient(this.get('componentName'));
  }.property('componentName'),
  /**
   * Determine if component is running now
   * Based on <code>workStatus</code>
   * @returns {bool}
   */
  isRunning: Em.computed.existsIn('workStatus', ['STARTED', 'STARTING']),

  /**
   * Determines if component is not installed
   * Based on <code>workStatus</code>
   *
   * @type {boolean}
   */
  isNotInstalled: Em.computed.existsIn('workStatus', ['INIT', 'INSTALL_FAILED']),

  /**
   * Determine if component is master
   * @returns {bool}
   */
  isMaster: function () {
    return App.HostComponent.isMaster(this.get('componentName'));
  }.property('componentName', 'App.components.masters'),

  /**
   * Determine if component is slave
   * @returns {bool}
   */
  isSlave: function () {
    return App.HostComponent.isSlave(this.get('componentName'));
  }.property('componentName'),
  /**
   * Only certain components can be deleted.
   * They include some from master components,
   * some from slave components, and rest from
   * client components.
   * @returns {bool}
   */
  isDeletable: function () {
    return App.get('components.deletable').contains(this.get('componentName'));
  }.property('componentName'),
  /**
   * A host-component is decommissioning when it is in HDFS service's list of
   * decomNodes.
   * @returns {bool}
   */
  isDecommissioning: function () {
    var decommissioning = false;
    var hostName = this.get('hostName');
    var componentName = this.get('componentName');
    var hdfsSvc = App.HDFSService.find().objectAt(0);
    if (componentName === 'DATANODE' && hdfsSvc) {
      var decomNodes = hdfsSvc.get('decommissionDataNodes');
      var decomNode = decomNodes != null ? decomNodes.findProperty("hostName", hostName) : null;
      decommissioning = decomNode != null;
    }
    return decommissioning;
  }.property('componentName', 'hostName', 'App.router.clusterController.isLoaded', 'App.router.updateController.isUpdated'),
  /**
   * User friendly host component status
   * @returns {String}
   */
  isActive: function () {
    var passiveState = this.get('passiveState');
    if (passiveState === 'IMPLIED_FROM_HOST') {
      passiveState = this.get('host.passiveState');
    } else if (passiveState === 'IMPLIED_FROM_SERVICE') {
      passiveState = this.get('service.passiveState');
    } else if (passiveState === 'IMPLIED_FROM_SERVICE_AND_HOST') {
      return this.get('service.passiveState') === 'OFF' && this.get('host.passiveState') === 'OFF';
    }
    return passiveState === 'OFF';
  }.property('passiveState', 'host.passiveState', 'service.passiveState'),

  /**
   * Determine if passiveState is implied from host or/and service
   * @returns {Boolean}
   */
  isImpliedState: Em.computed.existsIn('passiveState', ['IMPLIED_FROM_SERVICE_AND_HOST', 'IMPLIED_FROM_HOST', 'IMPLIED_FROM_SERVICE']),

  passiveTooltip: Em.computed.ifThenElse('isActive', '', Em.I18n.t('hosts.component.passive.mode')),
  /**
   * Determine if component is a HDP component
   * @returns {bool}
   */
  isHDPComponent: function () {
    return !App.get('components.nonHDP').contains(this.get('componentName'));
  }.property('componentName', 'App.components.nonHDP'),

  /**
   * Does component have Critical Alerts
   * @type {boolean}
   */
  hasCriticalAlerts: false,

  /**
   * Number of the Critical and Warning alerts for current component
   * @type {number}
   */
  alertsCount: 0,

  statusClass: function () {
    return this.get('isActive') ? this.get('workStatus') : 'icon-medkit';
  }.property('workStatus', 'isActive'),

  statusIconClass: Em.computed.getByKey('statusIconClassMap', 'statusClass', ''),

  statusIconClassMap: {
    STARTED: App.healthIconClassGreen,
    STARTING: App.healthIconClassGreen,
    INSTALLED: App.healthIconClassRed,
    STOPPING: App.healthIconClassRed,
    UNKNOWN: App.healthIconClassYellow
  },

  componentTextStatus: function () {
    if (this.get('isClient') && this.get("workStatus") === 'INSTALLED') {
      return Em.I18n.t('common.installed');
    }
    return App.HostComponentStatus.getTextStatus(this.get("workStatus"));
  }.property('workStatus', 'isDecommissioning')
});

App.HostComponent.FIXTURES = [];

App.HostComponent.isClient = function (componentName) {
  return App.get('components.clients').contains(componentName);
};

App.HostComponent.isMaster = function (componentName) {
  return App.get('components.masters').contains(componentName);
};

App.HostComponent.isSlave = function (componentName) {
  return App.get('components.slaves').contains(componentName);
};

/**
 * get particular counter of host-component by name
 * @param {string} componentName
 * @param {string} type (installedCount|startedCount|totalCount)
 * @returns {number}
 */
App.HostComponent.getCount = function (componentName, type) {
  switch (App.StackServiceComponent.find(componentName).get('componentCategory')) {
    case 'MASTER':
      return Number(App.MasterComponent.find(componentName).get(type));
    case 'SLAVE':
      return Number(App.SlaveComponent.find(componentName).get(type));
    case 'CLIENT':
      return Number(App.ClientComponent.find(componentName).get(type));
    default:
      return 0;
  }
};

/**
 * @param {string} componentName
 * @param {string} hostName
 * @returns {string}
 */
App.HostComponent.getId = function (componentName, hostName) {
  return componentName + '_' + hostName;
};

App.HostComponentStatus = {
  started: "STARTED",
  starting: "STARTING",
  stopped: "INSTALLED",
  stopping: "STOPPING",
  install_failed: "INSTALL_FAILED",
  installing: "INSTALLING",
  upgrade_failed: "UPGRADE_FAILED",
  unknown: "UNKNOWN",
  disabled: "DISABLED",
  init: "INIT",

  /**
   * Get host component status in "machine" format
   * @param {String} value
   * @returns {String}
   */
  getKeyName: function getKeyName(value) {
    switch (value) {
      case this.started:
        return 'started';
      case this.starting:
        return 'starting';
      case this.stopped:
        return 'installed';
      case this.stopping:
        return 'stopping';
      case this.install_failed:
        return 'install_failed';
      case this.installing:
        return 'installing';
      case this.upgrade_failed:
        return 'upgrade_failed';
      case this.disabled:
      case this.unknown:
        return 'unknown';
    }
    return 'unknown';
  },

  /**
   * Get user-friendly host component status
   * @param {String} value
   * @returns {String}
   */
  getTextStatus: function getTextStatus(value) {
    switch (value) {
      case this.installing:
        return 'Installing...';
      case this.install_failed:
        return 'Install Failed';
      case this.stopped:
        return 'Stopped';
      case this.started:
        return 'Started';
      case this.starting:
        return 'Starting...';
      case this.stopping:
        return 'Stopping...';
      case this.unknown:
        return 'Heartbeat Lost';
      case this.upgrade_failed:
        return 'Upgrade Failed';
      case this.disabled:
        return 'Disabled';
      case this.init:
        return 'Install Pending...';
    }
    return 'Unknown';
  },

  /**
   * Get list of possible <code>App.HostComponent</code> statuses
   * @returns {String[]}
   */
  getStatusesList: function getStatusesList() {
    var ret = [];
    for (var st in this) {
      if (this.hasOwnProperty(st) && Em.typeOf(this[st]) == 'string') {
        ret.push(this[st]);
      }
    }
    return ret;
  }
};

App.HostComponentActionMap = {
  getMap: function getMap(ctx) {
    var NN = ctx.get('controller.content.hostComponents').findProperty('componentName', 'NAMENODE'),
        RM = ctx.get('controller.content.hostComponents').findProperty('componentName', 'RESOURCEMANAGER'),
        RA = ctx.get('controller.content.hostComponents').findProperty('componentName', 'RANGER_ADMIN'),
        HM = ctx.get('controller.content.hostComponents').findProperty('componentName', 'HAWQMASTER'),
        HS = ctx.get('controller.content.hostComponents').findProperty('componentName', 'HAWQSTANDBY'),
        HMComponent = App.MasterComponent.find('HAWQMASTER'),
        HSComponent = App.MasterComponent.find('HAWQSTANDBY'),
        hasMultipleMasterComponentGroups = ctx.get('service.hasMultipleMasterComponentGroups'),
        isClientsOnlyService = ctx.get('controller.isClientsOnlyService'),
        isStartDisabled = ctx.get('controller.isStartDisabled'),
        isStopDisabled = ctx.get('controller.isStopDisabled');

    return {
      START_ALL: {
        action: hasMultipleMasterComponentGroups ? '' : 'startService',
        label: Em.I18n.t('services.service.start'),
        cssClass: 'glyphicon glyphicon-play ' + (isStartDisabled ? 'disabled' : 'enabled'),
        isHidden: isClientsOnlyService,
        disabled: isStartDisabled,
        hasSubmenu: !isStartDisabled && hasMultipleMasterComponentGroups,
        submenuOptions: !isStartDisabled && hasMultipleMasterComponentGroups ? this.getMastersSubmenu(ctx, 'startCertainHostComponents') : []
      },
      STOP_ALL: {
        action: hasMultipleMasterComponentGroups ? '' : 'stopService',
        label: Em.I18n.t('services.service.stop'),
        cssClass: 'glyphicon glyphicon-stop ' + (isStopDisabled ? 'disabled' : 'enabled'),
        isHidden: isClientsOnlyService,
        disabled: isStopDisabled,
        hasSubmenu: !isStopDisabled && hasMultipleMasterComponentGroups,
        submenuOptions: !isStopDisabled && hasMultipleMasterComponentGroups ? this.getMastersSubmenu(ctx, 'stopCertainHostComponents') : []
      },
      RESTART_ALL: {
        action: hasMultipleMasterComponentGroups ? '' : 'restartAllHostComponents',
        context: ctx.get('serviceName'),
        label: hasMultipleMasterComponentGroups ? Em.I18n.t('common.restart') : Em.I18n.t('restart.service.all'),
        cssClass: 'glyphicon glyphicon-time',
        disabled: false,
        hasSubmenu: hasMultipleMasterComponentGroups,
        submenuOptions: hasMultipleMasterComponentGroups ? this.getMastersSubmenu(ctx, 'restartCertainHostComponents') : []
      },
      RESTART_NAMENODES: {
        action: '',
        label: Em.I18n.t('rollingrestart.dialog.title').format(pluralize(App.format.role('NAMENODE', false))),
        cssClass: 'glyphicon glyphicon-time',
        isHidden: !hasMultipleMasterComponentGroups,
        disabled: false,
        hasSubmenu: true,
        submenuOptions: this.getMastersSubmenu(ctx, 'restartCertainHostComponents', ['NAMENODE'])
      },
      RUN_SMOKE_TEST: {
        action: 'runSmokeTest',
        label: Em.I18n.t('services.service.actions.run.smoke'),
        cssClass: 'glyphicon glyphicon-thumbs-up',
        disabled: ctx.get('controller.isSmokeTestDisabled')
      },
      REFRESH_CONFIGS: {
        action: 'refreshConfigs',
        label: Em.I18n.t('hosts.host.details.refreshConfigs'),
        cssClass: 'glyphicon glyphicon-refresh',
        disabled: false
      },
      REGENERATE_KEYTAB_FILE_OPERATIONS: {
        action: 'regenerateKeytabFileOperations',
        label: Em.I18n.t('admin.kerberos.button.regenerateKeytabs'),
        cssClass: 'glyphicon glyphicon-repeat',
        isHidden: !App.get('isKerberosEnabled') || App.get('router.mainAdminKerberosController.isManualKerberos')
      },
      REFRESHQUEUES: {
        action: 'refreshYarnQueues',
        customCommand: 'REFRESHQUEUES',
        context: Em.I18n.t('services.service.actions.run.yarnRefreshQueues.context'),
        label: Em.I18n.t('services.service.actions.run.yarnRefreshQueues.menu'),
        cssClass: 'glyphicon glyphicon-refresh',
        disabled: false
      },
      ROLLING_RESTART: {
        action: 'rollingRestart',
        context: ctx.get('rollingRestartComponent'),
        label: Em.I18n.t('rollingrestart.dialog.title'),
        cssClass: 'glyphicon glyphicon-time',
        disabled: false
      },
      TOGGLE_PASSIVE: {
        action: 'turnOnOffPassive',
        context: ctx.get('isPassive') ? Em.I18n.t('passiveState.turnOffFor').format(ctx.get('displayName')) : Em.I18n.t('passiveState.turnOnFor').format(ctx.get('displayName')),
        label: ctx.get('isPassive') ? Em.I18n.t('passiveState.turnOff') : Em.I18n.t('passiveState.turnOn'),
        cssClass: 'icon-medkit',
        disabled: false
      },
      MANAGE_JN: {
        action: 'manageJournalNode',
        label: Em.I18n.t('admin.manageJournalNode.label'),
        cssClass: 'icon-cog',
        isHidden: !App.get('supports.manageJournalNode') || !App.get('isHaEnabled') || App.router.get('mainHostController.totalCount') == 3 && App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').get('length') == 3
      },
      TOGGLE_NN_HA: {
        action: App.get('isHaEnabled') ? 'disableHighAvailability' : 'enableHighAvailability',
        label: App.get('isHaEnabled') ? Em.I18n.t('admin.highAvailability.button.disable') : Em.I18n.t('admin.highAvailability.button.enable'),
        cssClass: App.get('isHaEnabled') ? 'glyphicon glyphicon-arrow-down' : 'glyphicon glyphicon-arrow-up',
        isHidden: App.get('isHaEnabled'),
        disabled: App.get('isSingleNode') || !NN || NN.get('isNotInstalled')
      },
      TOGGLE_RM_HA: {
        action: 'enableRMHighAvailability',
        label: App.get('isRMHaEnabled') ? Em.I18n.t('admin.rm_highAvailability.button.disable') : Em.I18n.t('admin.rm_highAvailability.button.enable'),
        cssClass: App.get('isRMHaEnabled') ? 'glyphicon glyphicon-arrow-down' : 'glyphicon glyphicon-arrow-up',
        isHidden: App.get('isRMHaEnabled'),
        disabled: App.get('isSingleNode') || !RM || RM.get('isNotInstalled')
      },
      TOGGLE_RA_HA: {
        action: 'enableRAHighAvailability',
        label: Em.I18n.t('admin.ra_highAvailability.button.enable'),
        cssClass: 'glyphicon glyphicon-arrow-up',
        isHidden: App.get('isRAHaEnabled'),
        disabled: App.get('isSingleNode') || !RA || RA.get('isNotInstalled')
      },
      MOVE_COMPONENT: {
        action: 'reassignMaster',
        context: '',
        isHidden: !App.isAuthorized('SERVICE.MOVE'),
        label: Em.I18n.t('services.service.actions.reassign.master'),
        cssClass: 'glyphicon glyphicon-share-alt'
      },
      STARTDEMOLDAP: {
        action: 'startLdapKnox',
        customCommand: 'STARTDEMOLDAP',
        context: Em.I18n.t('services.service.actions.run.startLdapKnox.context'),
        label: Em.I18n.t('services.service.actions.run.startLdapKnox.context'),
        cssClass: 'icon icon-play-sign',
        disabled: false
      },
      STOPDEMOLDAP: {
        action: 'stopLdapKnox',
        customCommand: 'STOPDEMOLDAP',
        context: Em.I18n.t('services.service.actions.run.stopLdapKnox.context'),
        label: Em.I18n.t('services.service.actions.run.stopLdapKnox.context'),
        cssClass: 'glyphicon glyphicon-stop',
        disabled: false
      },
      RESTART_LLAP: {
        action: 'restartLLAP',
        customCommand: 'RESTART_LLAP',
        context: Em.I18n.t('services.service.actions.run.restartLLAP'),
        label: Em.I18n.t('services.service.actions.run.restartLLAP') + ' ∞',
        cssClass: 'glyphicon glyphicon-refresh'
      },
      REBALANCEHDFS: {
        action: 'rebalanceHdfsNodes',
        customCommand: 'REBALANCEHDFS',
        context: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.context'),
        label: Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes'),
        cssClass: 'glyphicon glyphicon-refresh',
        disabled: false
      },
      DOWNLOAD_CLIENT_CONFIGS: {
        action: ctx.get('controller.isSeveralClients') ? '' : 'downloadClientConfigs',
        label: Em.I18n.t('services.service.actions.downloadClientConfigs'),
        cssClass: 'glyphicon glyphicon-download-alt',
        isHidden: !!ctx.get('controller.content.clientComponents') ? ctx.get('controller.content.clientComponents').rejectProperty('totalCount', 0).length == 0 : false,
        disabled: false,
        hasSubmenu: ctx.get('controller.isSeveralClients'),
        submenuOptions: ctx.get('controller.clientComponents')
      },
      DELETE_SERVICE: {
        action: 'deleteService',
        context: ctx.get('serviceName'),
        label: Em.I18n.t('services.service.actions.deleteService'),
        cssClass: 'glyphicon glyphicon-remove',
        isHidden: !App.get('services.supportsDeleteViaUI').contains(ctx.get('serviceName')) //hide the menu item when the service has a custom behavior setting in its metainfo.xml to disallow Delete Services via UI
      },
      IMMEDIATE_STOP_HAWQ_SERVICE: {
        action: 'executeHawqCustomCommand',
        customCommand: 'IMMEDIATE_STOP_HAWQ_SERVICE',
        context: Em.I18n.t('services.service.actions.run.immediateStopHawqService.context'),
        label: Em.I18n.t('services.service.actions.run.immediateStopHawqService.label'),
        cssClass: 'glyphicon glyphicon-stop',
        disabled: !HM || HM.get('workStatus') != App.HostComponentStatus.started
      },
      IMMEDIATE_STOP_HAWQ_SEGMENT: {
        customCommand: 'IMMEDIATE_STOP_HAWQ_SEGMENT',
        context: Em.I18n.t('services.service.actions.run.immediateStopHawqSegment.context'),
        label: Em.I18n.t('services.service.actions.run.immediateStopHawqSegment.label'),
        cssClass: 'glyphicon glyphicon-stop'
      },
      RESYNC_HAWQ_STANDBY: {
        action: 'executeHawqCustomCommand',
        customCommand: 'RESYNC_HAWQ_STANDBY',
        context: Em.I18n.t('services.service.actions.run.resyncHawqStandby.context'),
        label: Em.I18n.t('services.service.actions.run.resyncHawqStandby.label'),
        cssClass: 'glyphicon glyphicon-refresh',
        isHidden: App.get('isSingleNode') || !HS,
        disabled: !(!!HMComponent && HMComponent.get('startedCount') === 1 && !!HSComponent && HSComponent.get('startedCount') === 1)
      },
      TOGGLE_ADD_HAWQ_STANDBY: {
        action: 'addHawqStandby',
        label: Em.I18n.t('admin.addHawqStandby.button.enable'),
        cssClass: 'glyphicon glyphicon-plus',
        isHidden: App.get('isSingleNode') || HS,
        disabled: false
      },
      REMOVE_HAWQ_STANDBY: {
        action: 'removeHawqStandby',
        context: Em.I18n.t('admin.removeHawqStandby.button.enable'),
        label: Em.I18n.t('admin.removeHawqStandby.button.enable'),
        cssClass: 'glyphicon glyphicon-minus',
        isHidden: App.get('isSingleNode') || !HS,
        disabled: !HM || HM.get('workStatus') != App.HostComponentStatus.started,
        hideFromComponentView: true
      },
      ACTIVATE_HAWQ_STANDBY: {
        action: 'activateHawqStandby',
        label: Em.I18n.t('admin.activateHawqStandby.button.enable'),
        context: Em.I18n.t('admin.activateHawqStandby.button.enable'),
        cssClass: 'glyphicon glyphicon-arrow-up',
        isHidden: App.get('isSingleNode') || !HS,
        disabled: false,
        hideFromComponentView: true
      },
      HAWQ_CLEAR_CACHE: {
        action: 'executeHawqCustomCommand',
        customCommand: 'HAWQ_CLEAR_CACHE',
        context: Em.I18n.t('services.service.actions.run.clearHawqCache.label'),
        label: Em.I18n.t('services.service.actions.run.clearHawqCache.label'),
        cssClass: 'glyphicon glyphicon-refresh',
        isHidden: false,
        disabled: !HM || HM.get('workStatus') != App.HostComponentStatus.started
      },
      RUN_HAWQ_CHECK: {
        action: 'executeHawqCustomCommand',
        customCommand: 'RUN_HAWQ_CHECK',
        context: Em.I18n.t('services.service.actions.run.runHawqCheck.label'),
        label: Em.I18n.t('services.service.actions.run.runHawqCheck.label'),
        cssClass: 'glyphicon glyphicon-thumbs-up',
        isHidden: false,
        disabled: false
      },
      MASTER_CUSTOM_COMMAND: {
        action: 'executeCustomCommand',
        cssClass: 'glyphicon glyphicon-play-circle',
        isHidden: false,
        disabled: false
      },
      TOGGLE_NN_FEDERATION: {
        action: 'openNameNodeFederationWizard',
        label: Em.I18n.t('admin.nameNodeFederation.button.enable'),
        cssClass: 'icon icon-sitemap',
        disabled: !App.get('isHaEnabled')
      },
      UPDATE_REPLICATION: {
        action: 'updateHBaseReplication',
        customCommand: 'UPDATE_REPLICATION',
        context: Em.I18n.t('services.service.actions.run.updateHBaseReplication.context'),
        label: Em.I18n.t('services.service.actions.run.updateHBaseReplication.label'),
        cssClass: 'glyphicon glyphicon-refresh',
        disabled: false
      },
      STOP_REPLICATION: {
        action: 'stopHBaseReplication',
        customCommand: 'STOP_REPLICATION',
        context: Em.I18n.t('services.service.actions.run.stopHBaseReplication.context'),
        label: Em.I18n.t('services.service.actions.run.stopHBaseReplication.label'),
        cssClass: 'glyphicon glyphicon-refresh',
        disabled: false
      }
    };
  },

  getMastersSubmenu: function getMastersSubmenu(context, action, components) {
    var serviceName = context.get('service.serviceName'),
        groups = context.get('service.masterComponentGroups') || [],
        allItem = {
      action: action,
      context: _extends({
        label: Em.I18n.t('services.service.allComponents')
      }, components ? {
        components: components,
        hosts: groups.mapProperty('hosts').reduce(function (acc, groupHosts) {
          return [].concat(_toConsumableArray(acc), _toConsumableArray(groupHosts));
        }, []).uniq()
      } : {
        serviceName: serviceName
      }),
      disabled: false
    },
        groupItems = groups.map(function (group) {
      return {
        action: action,
        context: {
          label: group.title,
          hosts: group.hosts,
          components: components || group.components,
          serviceName: serviceName
        },
        disabled: false,
        tooltip: group.title
      };
    });
    return [allItem].concat(_toConsumableArray(groupItems));
  }
};

});

require.register("models/host_component_log", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HostComponentLog = DS.Model.extend({
  name: DS.attr('string'),
  hostName: DS.attr('string'),
  serviceName: DS.attr('string'),
  hostComponent: DS.belongsTo('App.HostComponent'),
  logFileNames: DS.attr('array')
});

App.HostComponentLog.FIXTURES = [];

});

require.register("models/host_stack_version", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var stringUtils = require('utils/string_utils');

App.HostStackVersion = DS.Model.extend({
  stack: DS.attr('string'),
  version: DS.attr('string'),
  repo: DS.belongsTo('App.Repository'),
  repoVersion: DS.attr('string'),
  displayName: DS.attr('string'),
  isVisible: DS.attr('boolean', { defaultValue: true }),
  /**
   * possible property value defined at App.HostStackVersion.statusDefinition
   * @type {string}
   */
  status: DS.attr('string'),
  host: DS.belongsTo('App.Host'),
  hostName: DS.attr('string'),

  /**
   * @type {boolean}
   */
  isCurrent: Em.computed.equal('status', 'CURRENT'),

  /**
   * @type {boolean}
   */
  isInstalling: Em.computed.equal('status', 'INSTALLING'),

  /**
   * @type {boolean}
   */
  isOutOfSync: Em.computed.equal('status', 'OUT_OF_SYNC'),
  /**
   * @type {string}
   */
  displayStatus: function () {
    return App.HostStackVersion.formatStatus(this.get('status'));
  }.property('status'),

  /**
   * @type {boolean}
   */
  installEnabled: Em.computed.existsIn('status', ['OUT_OF_SYNC', 'INSTALL_FAILED']),

  installDisabled: function () {
    return !this.get('installEnabled') || App.router.get('wizardWatcherController.isNonWizardUser');
  }.property('installEnabled', 'App.routerwizardWatcherController.isNonWizardUser')
});

App.HostStackVersion.FIXTURES = [];

/**
 * definition of possible statuses of Stack Version
 * @type {Array}
 */
App.HostStackVersion.statusDefinition = ["INSTALLED", "INSTALLING", "INSTALL_FAILED", "OUT_OF_SYNC", "CURRENT", "UPGRADING", "UPGRADE_FAILED"];

/**
 * translate status to label
 * @param status
 * @return {string}
 */
App.HostStackVersion.formatStatus = function (status) {
  return App.HostStackVersion.statusDefinition.contains(status) ? Em.I18n.t('hosts.host.stackVersions.status.' + status.toLowerCase()) : stringUtils.upperUnderscoreToText(status);
};

});

require.register("models/hosts", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HostInfo = Ember.Object.extend({
  elementId: 'host',
  name: '',
  cpu: null,
  memory: null,
  message: 'Information',
  barColor: 'progress-bar-info',
  isChecked: true,
  bootLog: null,
  bootStatus: 'PENDING',

  bootStatusForDisplay: Em.computed.getByKey('bootStatusForDisplayMap', 'bootStatus', 'Registering'),

  bootStatusForDisplayMap: {
    PENDING: 'Preparing',
    REGISTERED: 'Success',
    FAILED: 'Failed',
    RUNNING: 'Installing',
    DONE: 'Registering',
    REGISTERING: 'Registering'
  },

  bootBarColor: Em.computed.getByKey('bootBarColorMap', 'bootStatus', 'progress-bar-info'),

  bootBarColorMap: {
    REGISTERED: 'progress-bar-success',
    FAILED: 'progress-bar-danger',
    PENDING: 'progress-bar-info',
    RUNNING: 'progress-bar-info',
    DONE: 'progress-bar-info',
    REGISTERING: 'progress-bar-info'
  },

  bootStatusColor: Em.computed.getByKey('bootStatusColorMap', 'bootStatus', 'text-info'),

  bootStatusColorMap: {
    REGISTERED: 'text-success',
    FAILED: 'text-danger',
    PENDING: 'text-info',
    RUNNING: 'text-info',
    DONE: 'text-info',
    REGISTERING: 'text-info'
  },

  isBootDone: Em.computed.existsIn('bootStatus', ['REGISTERED', 'FAILED'])

});

});

require.register("models/master_component", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.MasterComponent = App.ClientComponent.extend({
  displayNamePluralized: function () {
    return stringUtils.pluralize(this.get('startedCount'), this.get('displayName'));
  }.property('startedCount')
});

App.MasterComponent.FIXTURES = [];

});

require.register("models/operating_system", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.OperatingSystem = DS.Model.extend({
  id: DS.attr('string'), // This is ${stackName}-${stackVersion}-${osType}.
  osType: DS.attr('string'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  repositories: DS.hasMany('App.Repository'),
  stack: DS.belongsTo('App.Stack'),
  isSelected: DS.attr('boolean', { defaultValue: true }),
  isDeselected: Em.computed.not('isSelected'),
  isNotFilled: Em.computed.someBy('repositories', 'isEmpty', true),
  isEmpty: Em.computed.everyBy('repositories', 'isEmpty', true)
});

App.OperatingSystem.FIXTURES = [];

});

require.register("models/quick_links", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var portRegex = '\\w*:(\\d+)';

App.QuickLinks = DS.Model.extend({
  label: DS.attr('string'),
  url: DS.attr('string'),
  serviceName: DS.attr('string'),
  template: DS.attr('string'),
  http_config: DS.attr('string'),
  https_config: DS.attr('string'),
  site: DS.attr('string'),
  regex: DS.attr('string'),
  default_http_port: DS.attr('number'),
  default_https_port: DS.attr('number')
});

App.QuickLinks.FIXTURES = [{
  id: 1,
  label: 'NameNode UI',
  url: '%@://%@:%@',
  service_name: 'HDFS',
  template: '%@://%@:%@',
  http_config: 'dfs.namenode.http-address',
  https_config: 'dfs.namenode.https-address',
  site: 'hdfs-site',
  regex: portRegex,
  default_http_port: 50070,
  default_https_port: 50470
}, {
  id: 2,
  label: 'NameNode logs',
  url: '%@://%@:%@/logs',
  service_name: 'HDFS',
  template: '%@://%@:%@/logs',
  http_config: 'dfs.namenode.http-address',
  https_config: 'dfs.namenode.https-address',
  site: 'hdfs-site',
  regex: portRegex,
  default_http_port: 50070,
  default_https_port: 50470
}, {
  id: 3,
  label: 'NameNode JMX',
  url: '%@://%@:%@/jmx',
  service_name: 'HDFS',
  template: '%@://%@:%@/jmx',
  http_config: 'dfs.namenode.http-address',
  https_config: 'dfs.namenode.https-address',
  site: 'hdfs-site',
  regex: portRegex,
  default_http_port: 50070,
  default_https_port: 50470
}, {
  id: 4,
  label: 'Thread Stacks',
  url: '%@://%@:%@/stacks',
  service_name: 'HDFS',
  template: '%@://%@:%@/stacks',
  http_config: 'dfs.namenode.http-address',
  https_config: 'dfs.namenode.https-address',
  site: 'hdfs-site',
  regex: portRegex,
  default_http_port: 50070,
  default_https_port: 50470
}, {
  id: 13,
  label: 'HBase Master UI',
  url: '%@://%@:%@/master-status',
  service_name: 'HBASE',
  template: '%@://%@:%@/master-status',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 14,
  label: 'HBase Logs',
  url: '%@://%@:60010/logs',
  service_name: 'HBASE',
  template: '%@://%@:%@/logs',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 15,
  label: 'Zookeeper Info',
  url: '%@://%@:60010/zk.jsp',
  service_name: 'HBASE',
  template: '%@://%@:%@/zk.jsp',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 16,
  label: 'HBase Master JMX',
  url: '%@://%@:60010/jmx',
  service_name: 'HBASE',
  template: '%@://%@:%@/jmx',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 17,
  label: 'Debug Dump',
  url: '%@://%@:%@/dump',
  service_name: 'HBASE',
  template: '%@://%@:%@/dump',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 18,
  label: 'Thread Stacks',
  url: '%@://%@:%@/stacks',
  service_name: 'HBASE',
  template: '%@://%@:%@/stacks',
  http_config: 'hbase.master.info.port',
  site: 'hbase-site',
  regex: '^(\\d+)$',
  default_http_port: 60010
}, {
  id: 19,
  label: 'Oozie Web UI',
  url: '%@://%@:%@/oozie?user.name=%@',
  service_name: 'OOZIE',
  template: '%@://%@:%@/oozie?user.name=%@',
  http_config: 'oozie.base.url',
  site: 'oozie-site',
  regex: portRegex,
  default_http_port: 11000
}, {
  id: 20,
  label: 'Ganglia Web UI',
  url: '%@://%@/ganglia',
  service_name: 'GANGLIA',
  template: '%@://%@/ganglia'

}, {
  id: 23,
  label: 'ResourceManager UI',
  url: '%@://%@:%@',
  service_name: 'YARN',
  template: '%@://%@:%@',
  http_config: 'yarn.resourcemanager.webapp.address',
  https_config: 'yarn.resourcemanager.webapp.https.address',
  site: 'yarn-site',
  regex: portRegex,
  default_http_port: 8088,
  default_https_port: 8090

}, {
  id: 24,
  label: 'ResourceManager logs',
  url: '%@://%@:%@/logs',
  service_name: 'YARN',
  template: '%@://%@:%@/logs',
  http_config: 'yarn.resourcemanager.webapp.address',
  https_config: 'yarn.resourcemanager.webapp.https.address',
  site: 'yarn-site',
  regex: portRegex,
  default_http_port: 8088,
  default_https_port: 8090
}, {
  id: 25,
  label: 'ResourceManager JMX',
  url: '%@://%@:%@/jmx',
  service_name: 'YARN',
  template: '%@://%@:%@/jmx',
  http_config: 'yarn.resourcemanager.webapp.address',
  https_config: 'yarn.resourcemanager.webapp.https.address',
  site: 'yarn-site',
  regex: portRegex,
  default_http_port: 8088,
  default_https_port: 8090
}, {
  id: 26,
  label: 'Thread Stacks',
  url: '%@://%@:%@/stacks',
  service_name: 'YARN',
  template: '%@://%@:%@/stacks',
  http_config: 'yarn.resourcemanager.webapp.address',
  https_config: 'yarn.resourcemanager.webapp.https.address',
  site: 'yarn-site',
  regex: portRegex,
  default_http_port: 8088,
  default_https_port: 8090
}, {
  id: 27,
  label: 'JobHistory UI',
  url: '%@://%@:%@',
  service_name: 'MAPREDUCE2',
  template: '%@://%@',
  http_config: 'mapreduce.jobhistory.webapp.address',
  https_config: 'mapreduce.jobhistory.webapp.https.address',
  site: 'mapred-site',
  regex: portRegex,
  default_http_port: 19888
}, {
  id: 28,
  label: 'JobHistory logs',
  url: '%@://%@:%@/logs',
  service_name: 'MAPREDUCE2',
  template: '%@://%@/logs',
  http_config: 'mapreduce.jobhistory.webapp.address',
  https_config: 'mapreduce.jobhistory.webapp.https.address',
  site: 'mapred-site',
  regex: portRegex,
  default_http_port: 19888
}, {
  id: 29,
  label: 'JobHistory JMX',
  url: '%@://%@:%@/jmx',
  service_name: 'MAPREDUCE2',
  template: '%@://%@/jmx',
  http_config: 'mapreduce.jobhistory.webapp.address',
  https_config: 'mapreduce.jobhistory.webapp.https.address',
  site: 'mapred-site',
  regex: portRegex,
  default_http_port: 19888
}, {
  id: 30,
  label: 'Thread Stacks',
  url: '%@://%@:%@/stacks',
  service_name: 'MAPREDUCE2',
  template: '%@://%@/stacks',
  http_config: 'mapreduce.jobhistory.webapp.address',
  https_config: 'mapreduce.jobhistory.webapp.https.address',
  site: 'mapred-site',
  regex: portRegex,
  default_http_port: 19888
}, {
  id: 31,
  label: 'Storm UI',
  url: '%@://%@:%@/',
  service_name: 'STORM',
  template: '%@://%@:%@/',
  http_config: 'ui.port',
  site: 'storm-site',
  regex: '^(\\d+)$',
  default_http_port: 8744
}, {
  id: 32,
  label: 'Falcon Web UI',
  url: '%@://%@:%@/index.html?user.name=%@',
  service_name: 'FALCON',
  template: '%@://%@:%@/index.html?user.name=%@',
  http_config: 'falcon_port',
  site: 'falcon-env',
  regex: '^(\\d+)$',
  default_http_port: 15000
}, {
  id: 33,
  label: 'Ranger Admin UI',
  url: '%@://%@:%@/',
  service_name: 'RANGER',
  template: '%@://%@:%@/',
  http_config: 'http.service.port',
  https_config: 'https.service.port',
  regex: '(\\d*)+',
  site: 'ranger-site',
  default_http_port: 6080,
  default_https_port: 6182
}, {
  id: 34,
  label: 'Spark History Server UI',
  url: '%@://%@:%@/',
  service_name: 'SPARK',
  template: '%@://%@:%@/',
  http_config: 'spark.history.ui.port',
  site: 'spark-defaults',
  regex: '^(\\d+)$',
  default_http_port: 18080
}, {
  id: 35,
  label: 'Accumulo Monitor UI',
  url: '%@://%@:%@/',
  service_name: 'ACCUMULO',
  template: '%@://%@:%@/',
  http_config: 'monitor.port.client',
  https_config: 'monitor.port.client',
  site: 'accumulo-site',
  regex: '^(\\d+)$',
  default_http_port: 50095,
  default_https_port: 50095
}, {
  id: 36,
  label: 'Atlas Dashboard',
  url: '%@://%@:%@/#!/search?user.name=%@',
  service_name: 'ATLAS',
  template: '%@://%@:%@/#!/search?user.name=%@',
  http_config: 'atlas.server.http.port',
  https_config: 'atlas.server.https.port',
  site: 'application-properties',
  regex: '^(\\d+)$',
  default_http_port: 21000,
  default_https_port: 21443
}, {
  id: 37,
  label: 'Grafana',
  url: '%@://%@:%@',
  service_id: 'AMBARI_METRICS',
  template: '%@://%@:%@',
  http_config: 'port',
  site: 'ams-grafana-ini',
  regex: '^(\\d+)$',
  default_http_port: 3000,
  default_https_port: 3000
}, {
  id: 38,
  label: 'Log Search UI',
  url: '%@://%@:%@',
  service_id: 'LOGSEARCH',
  template: '%@://%@:%@',
  http_config: 'logsearch_ui_port',
  https_config: 'logsearch_ui_port',
  site: 'logsearch-env',
  regex: '^(\\d+)$',
  default_http_port: 61888,
  default_https_port: 61888
}];

});

require.register("models/quicklinks/quick_links_config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.QuickLinksConfig = DS.Model.extend({
  //general information
  name: DS.attr('string'),
  description: DS.attr('string'),
  fileName: DS.attr('string'),
  serviceName: DS.attr('string'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  //protocol configurations
  protocol: DS.attr('object'),
  //links configurations
  links: DS.attr('array')
});

App.QuickLinksConfig.FIXTURES = [];

});

require.register("models/rack", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.Rack = DS.Model.extend({
  name: DS.attr('string'),
  status: DS.attr('string'),
  criticalHostsCount: DS.attr('number'),
  deadHostsCount: DS.attr('number'),
  liveHostsCount: DS.attr('number'),
  hosts: []
});

App.Rack.FIXTURES = [
  //here example of data
  /*{
    id: 1,
    name: 'Rack-0',
    hosts: ['host01', 'host06', 'host05'],
    status: 'LIVE',
    live_hosts_count: 5,
    critical_hosts_count: 0,
    dead_hosts_count: 2
  }*/
];

});

require.register("models/repository", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.Repository = DS.Model.extend({
  id: DS.attr('string'), // This is ${osType}-${repoId}.
  repoId: DS.attr('string'),
  originalRepoId: DS.attr('string'),
  osType: DS.attr('string'),
  baseUrl: DS.attr('string'),
  baseUrlInit: DS.attr('string'),
  defaultBaseUrl: DS.attr('string'),
  latestBaseUrl: DS.attr('string'),
  repoName: DS.attr('string'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  operatingSystem: DS.belongsTo('App.OperatingSystem'),
  components: DS.attr('string'),
  distribution: DS.attr('string'),
  tags: DS.attr('array'),
  applicable_services: DS.attr('array'),

  validation: DS.attr('string', { defaultValue: '' }),
  validationClassName: Em.computed.getByKey('validationClassNameMap', 'validation', ''),
  validationClassNameMap: {
    INVALID: 'glyphicon glyphicon-exclamation-sign',
    OK: 'glyphicon glyphicon-ok',
    INPROGRESS: 'glyphicon glyphicon-repeat'
  },
  errorContent: DS.attr('string', { defaultValue: '' }),
  errorTitle: DS.attr('string', { defaultValue: '' }),

  isSelected: Em.computed.alias('operatingSystem.isSelected'),

  invalidFormatError: function () {
    return !validator.isValidBaseUrl(this.get('baseUrl'));
  }.property('baseUrl'),

  isEmpty: function () {
    return this.get('showRepo') && this.get('baseUrl') === '';
  }.property('baseUrl'),

  invalidError: function () {
    return this.get('validation') === 'INVALID';
  }.property('validation'),

  /**
   * @type {boolean}
   */
  isUtils: function () {
    return this.get('repoName').contains('UTILS');
  }.property('repoName'),

  /**
   * @type {boolean}
   */
  isGPL: function () {
    var tags = this.get('tags');
    return tags && tags.contains('GPL');
  }.property('tags'),

  /**
   * Determines whether a repo needs to be displayed in the UI or not
   * @type {boolean}
   */
  showRepo: function () {
    var isGPLAccepted = App.router.get('clusterController.ambariProperties')['gpl.license.accepted'] === 'true';
    return isGPLAccepted || !this.get('isGPL');
  }.property('isGPL'),

  undo: Em.computed.notEqualProperties('baseUrl', 'baseUrlInit'),

  notEmpty: Em.computed.notEqual('baseUrl', ''),

  clearAll: Em.computed.alias('baseUrl'),

  /**
   * @type {string}
   */
  placeholder: Em.computed.ifThenElse('isUtils', '', Em.I18n.t('installer.step1.advancedRepo.localRepo.placeholder'))

});

App.Repository.FIXTURES = [];

});

require.register("models/root_service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * This model loads the rootService (Ambari)
 * The model maps to the  http://localhost:8080/api/v1/services/AMBARI?fields=components/RootServiceComponents
 * @type {*}
 */
App.RootService = DS.Model.extend({
  serviceName: DS.attr('string'),
  displayName: Em.computed.formatRole('serviceName', true),
  components: DS.hasMany('App.RootServiceComponents')
});

App.RootService.FIXTURES = [];

App.RootServiceComponents = DS.Model.extend({
  componentName: DS.attr('string'),
  displayName: Em.computed.formatRole('componentName', false),
  componentVersion: DS.attr('string'),
  serverClock: DS.attr('number'),
  serviceName: DS.attr('string'),
  properties: DS.attr('object')
});

App.RootServiceComponents.FIXTURES = [];

});

require.register("models/service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/config');

App.Service = DS.Model.extend({
  serviceName: DS.attr('string', { defaultValue: '' }),
  displayName: Em.computed.formatRole('serviceName', true),
  passiveState: DS.attr('string', { defaultValue: "OFF" }),
  workStatus: DS.attr('string'),
  rand: DS.attr('string'),
  toolTipContent: DS.attr('string'),
  quickLinks: DS.hasMany('App.QuickLinks'), // mapped in app/mappers/service_metrics_mapper.js method - mapQuickLinks
  hostComponents: DS.hasMany('App.HostComponent'),
  serviceConfigsTemplate: App.config.get('preDefinedServiceConfigs'),
  desiredRepositoryVersionId: DS.attr('number'),
  /**
   * used by services("OOZIE", "ZOOKEEPER", "HIVE", "MAPREDUCE2", "TEZ", "SQOOP", "PIG","FALCON")
   * that have only client components
   */
  installedClients: DS.attr('number'),

  clientComponents: DS.hasMany('App.ClientComponent'),
  slaveComponents: DS.hasMany('App.SlaveComponent'),
  masterComponents: DS.hasMany('App.MasterComponent'),

  masterComponentGroups: DS.attr('array', {
    defaultValue: []
  }),

  hasMultipleMasterComponentGroups: Em.computed.gt('masterComponentGroups.length', 1),

  /**
   * Check master/slave component state of service
   * and general services state to define if it can be removed
   *
   * @type {boolean}
   */
  allowToDelete: function () {
    var workStatus = this.get('workStatus');
    return App.Service.allowUninstallStates.contains(workStatus) && this.get('slaveComponents').everyProperty('allowToDelete') && this.get('masterComponents').everyProperty('allowToDelete');
  }.property('slaveComponents.@each.allowToDelete', 'masterComponents.@each.allowToDelete', 'workStatus'),

  /**
   * @type {bool}
   */
  isInPassive: Em.computed.equal('passiveState', 'ON'),

  serviceComponents: function () {
    var clientComponents = this.get('clientComponents').mapProperty('componentName');
    var slaveComponents = this.get('slaveComponents').mapProperty('componentName');
    var masterComponents = this.get('masterComponents').mapProperty('componentName');
    return clientComponents.concat(slaveComponents).concat(masterComponents);
  }.property('clientComponents.@each', 'slaveComponents.@each', 'masterComponents.@each'),

  healthStatus: Em.computed.getByKey('healthStatusMap', 'workStatus', 'yellow'),

  healthStatusMap: {
    STARTED: 'green',
    STARTING: 'green-blinking',
    INSTALLED: 'red',
    STOPPING: 'red-blinking',
    UNKNOWN: 'yellow'
  },

  isStopped: Em.computed.equal('workStatus', 'INSTALLED'),

  isStarted: Em.computed.equal('workStatus', 'STARTED'),

  /**
   * Indicates when service deleting is in progress
   * Used to stop update service's data and topology
   *
   * @type {boolean}
   * @default false
   */
  deleteInProgress: false,

  /**
   * Service Tagging by their type.
   * @type {String[]}
   **/
  serviceTypes: function () {
    var typeServiceMap = {
      GANGLIA: ['MONITORING'],
      HDFS: ['HA_MODE', 'FEDERATION'],
      YARN: ['HA_MODE'],
      RANGER: ['HA_MODE'],
      HAWQ: ['HA_MODE']
    };
    return typeServiceMap[this.get('serviceName')] || [];
  }.property('serviceName'),

  /**
   * For each host-component, if the desired_configs dont match the
   * actual_configs, then a restart is required.
   */
  isRestartRequired: function () {
    var serviceComponents = this.get('clientComponents').toArray().concat(this.get('slaveComponents').toArray()).concat(this.get('masterComponents').toArray());
    var hc = {};

    serviceComponents.forEach(function (component) {
      var displayName = component.get('displayName');
      component.get('staleConfigHosts').forEach(function (hostName) {
        if (!hc[hostName]) {
          hc[hostName] = [];
        }
        hc[hostName].push(displayName);
      });
    });

    this.set('restartRequiredHostsAndComponents', hc);
    return serviceComponents.filterProperty('staleConfigHosts.length').length > 0;
  }.property('serviceName'),

  /**
   * Contains a map of which hosts and host_components
   * need a restart. This is populated when calculating
   * #isRestartRequired()
   * Example:
   * {
   *  'publicHostName1': ['TaskTracker'],
   *  'publicHostName2': ['JobTracker', 'TaskTracker']
   * }
   */
  restartRequiredHostsAndComponents: {},

  /**
   * Based on the information in #restartRequiredHostsAndComponents
   */
  restartRequiredMessage: function () {
    var restartHC = this.get('restartRequiredHostsAndComponents');
    var hostCount = 0;
    var hcCount = 0;
    var hostsMsg = "<ul>";
    for (var host in restartHC) {
      hostCount++;
      hostsMsg += "<li>" + host + "</li><ul>";
      restartHC[host].forEach(function (c) {
        hcCount++;
        hostsMsg += "<li>" + c + "</li>";
      });
      hostsMsg += "</ul>";
    }
    hostsMsg += "</ul>";
    return this.t('services.service.config.restartService.TooltipMessage').format(hcCount, hostCount, hostsMsg);
  }.property('restartRequiredHostsAndComponents'),

  /**
   * Does service have Critical Alerts
   * @type {boolean}
   */
  hasCriticalAlerts: false,

  /**
   * Number of the Critical and Warning alerts for current service
   * @type {number}
   */
  alertsCount: 0

});

/**
 * Map of all service states
 *
 * @type {Object}
 */
App.Service.statesMap = {
  init: 'INIT',
  installing: 'INSTALLING',
  install_failed: 'INSTALL_FAILED',
  stopped: 'INSTALLED',
  starting: 'STARTING',
  started: 'STARTED',
  stopping: 'STOPPING',
  uninstalling: 'UNINSTALLING',
  uninstalled: 'UNINSTALLED',
  wiping_out: 'WIPING_OUT',
  upgrading: 'UPGRADING',
  maintenance: 'MAINTENANCE',
  unknown: 'UNKNOWN'
};

/**
 * @type {String[]}
 */
App.Service.inProgressStates = [App.Service.statesMap.installing, App.Service.statesMap.starting, App.Service.statesMap.stopping, App.Service.statesMap.uninstalling, App.Service.statesMap.upgrading, App.Service.statesMap.wiping_out];

/**
 * @type {String[]}
 */
App.Service.allowUninstallStates = [App.Service.statesMap.init, App.Service.statesMap.install_failed, App.Service.statesMap.stopped, App.Service.statesMap.unknown];

App.Service.Health = {
  live: "LIVE",
  dead: "DEAD-RED",
  starting: "STARTING",
  stopping: "STOPPING",
  unknown: "DEAD-YELLOW",

  getKeyName: function getKeyName(value) {
    switch (value) {
      case this.live:
        return 'live';
      case this.dead:
        return 'dead';
      case this.starting:
        return 'starting';
      case this.stopping:
        return 'stopping';
      case this.unknown:
        return 'unknown';
    }
    return 'none';
  }
};

/**
 * association between service and extended model name
 * @type {Object}
 */
App.Service.extendedModel = {
  'HDFS': 'HDFSService',
  'ONEFS': 'ONEFSService',
  'HBASE': 'HBaseService',
  'YARN': 'YARNService',
  'MAPREDUCE2': 'MapReduce2Service',
  'STORM': 'StormService',
  'RANGER': 'RangerService',
  'FLUME': 'FlumeService'
};

App.Service.FIXTURES = [];

});

require.register("models/service/flume", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.FlumeService = App.Service.extend({
  version: DS.attr('string'),
  agents: DS.hasMany('App.FlumeAgent'),
  flumeHandlersTotal: DS.attr('number')
});

App.FlumeAgent = DS.Model.extend({
  /**
   * ID of a flume agent will be of the format
   * '<agent-name>-<host-name>'
   */
  id: DS.attr('string'),
  name: DS.attr('string'),
  /**
   * Status of agent. One of 'RUNNING', 'NOT_RUNNING', 'UNKNOWN'.
   */
  status: DS.attr('string'),
  host: DS.belongsTo('App.Host'),
  hostName: DS.attr('string'),

  channelsCount: DS.attr('number'),
  sourcesCount: DS.attr('number'),
  sinksCount: DS.attr('number'),

  healthClass: Em.computed.getByKey('healthClassMap', 'status', App.healthIconClassYellow),

  healthClassMap: {
    RUNNING: App.healthIconClassGreen,
    NOT_RUNNING: App.healthIconClassRed,
    UNKNOWN: App.healthIconClassYellow
  },

  healthIconClass: Em.computed.getByKey('healthIconClassMap', 'status', ''),

  healthIconClassMap: {
    RUNNING: 'health-status-LIVE',
    NOT_RUNNING: 'health-status-DEAD-RED',
    UNKNOWN: 'health-status-DEAD-YELLOW'
  },

  displayStatus: Em.computed.getByKey('displayStatusMap', 'status', Em.I18n.t('common.unknown')),

  displayStatusMap: {
    RUNNING: Em.I18n.t('common.running'),
    NOT_RUNNING: Em.I18n.t('common.stopped'),
    UNKNOWN: Em.I18n.t('common.unknown')
  }

});

App.FlumeAgent.FIXTURES = [];
App.FlumeService.FIXTURES = [];

});

require.register("models/service/hbase", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.HBaseService = App.Service.extend({
  master: DS.belongsTo('App.HostComponent'),
  regionServersStarted: DS.attr('number'),
  regionServersInstalled: DS.attr('number'),
  regionServersTotal: DS.attr('number'),
  phoenixServersStarted: DS.attr('number'),
  phoenixServersInstalled: DS.attr('number'),
  phoenixServersTotal: DS.attr('number'),
  masterStartTime: DS.attr('number'),
  masterActiveTime: DS.attr('number'),
  averageLoad: DS.attr('number'),
  regionsInTransition: DS.attr('number'),
  heapMemoryUsed: DS.attr('number'),
  heapMemoryMax: DS.attr('number')
});

App.HBaseService.FIXTURES = [];

});

require.register("models/service/hdfs", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.HDFSService = App.Service.extend({
  version: DS.attr('string'),
  nameNode: DS.belongsTo('App.HostComponent'),
  snameNode: DS.belongsTo('App.HostComponent'),

  activeNameNodes: DS.hasMany('App.HostComponent', {
    defaultValue: []
  }),
  standbyNameNodes: DS.hasMany('App.HostComponent', {
    defaultValue: []
  }),
  isNnHaEnabled: function () {
    return !this.get('snameNode') && this.get('hostComponents').filterProperty('componentName', 'NAMENODE').length > 1;
  }.property('snameNode', 'hostComponents'),
  dataNodesStarted: DS.attr('number'),
  dataNodesInstalled: DS.attr('number'),
  dataNodesTotal: DS.attr('number'),
  nfsGatewaysStarted: DS.attr('number', { defaultValue: 0 }),
  nfsGatewaysInstalled: DS.attr('number', { defaultValue: 0 }),
  nfsGatewaysTotal: DS.attr('number', { defaultValue: 0 }),
  journalNodes: DS.hasMany('App.HostComponent'),
  nameNodeStartTimeValues: DS.attr('object', {
    defaultValue: {}
  }),
  jvmMemoryHeapUsedValues: DS.attr('object', {
    defaultValue: {}
  }),
  jvmMemoryHeapMaxValues: DS.attr('object', {
    defaultValue: {}
  }),
  decommissionDataNodes: DS.hasMany('App.HostComponent'),
  liveDataNodes: DS.hasMany('App.HostComponent'),
  deadDataNodes: DS.hasMany('App.HostComponent'),
  capacityUsed: DS.attr('number'),
  capacityTotal: DS.attr('number'),
  capacityRemaining: DS.attr('number'),
  capacityNonDfsUsed: DS.attr('number'),
  dfsTotalBlocksValues: DS.attr('object', {
    defaultValue: {}
  }),
  dfsCorruptBlocksValues: DS.attr('object', {
    defaultValue: {}
  }),
  dfsMissingBlocksValues: DS.attr('object', {
    defaultValue: {}
  }),
  dfsUnderReplicatedBlocksValues: DS.attr('object', {
    defaultValue: {}
  }),
  dfsTotalFilesValues: DS.attr('object', {
    defaultValue: {}
  }),
  workStatusValues: DS.attr('object', {
    defaultValue: {}
  }),
  healthStatusValues: function () {
    var _this = this;

    var workStatusValues = Object.keys(this.get('workStatusValues'));
    return workStatusValues.reduce(function (acc, key) {
      return _extends({}, acc, _defineProperty({}, key, _this.get('healthStatusMap')[workStatusValues[key]] || 'yellow'));
    }, {});
  }.property('workStatusValues'),
  upgradeStatusValues: DS.attr('object', {
    defaultValue: {}
  }),
  safeModeStatusValues: DS.attr('object', {
    defaultValue: {}
  }),
  nameNodeRpcValues: DS.attr('object', {
    defaultValue: {}
  }),
  metricsNotAvailable: DS.attr('boolean'),
  masterComponentGroups: function () {
    var result = [];
    this.get('hostComponents').forEach(function (component) {
      if (component.get('componentName') === 'NAMENODE') {
        var nameSpace = component.get('haNameSpace') || 'default',
            hostName = component.get('hostName'),
            clusterId = component.get('clusterIdValue') || 'default',
            existingNameSpace = result.findProperty('name', nameSpace),
            currentNameSpace = existingNameSpace || {
          name: nameSpace,
          title: nameSpace,
          hosts: [],
          components: ['NAMENODE', 'ZKFC'],
          clusterId: clusterId
        };
        if (!existingNameSpace) {
          result.push(currentNameSpace);
        }
        if (!currentNameSpace.hosts.contains(hostName)) {
          currentNameSpace.hosts.push(hostName);
        }
      }
    });
    return result.sortProperty('name');
  }.property('hostComponents.length', 'App.router.clusterController.isHDFSNameSpacesLoaded')
});

App.HDFSService.FIXTURES = [];

});

require.register("models/service/mapreduce2", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MapReduce2Service = App.Service.extend({
  jobHistoryServer: DS.belongsTo('App.HostComponent'),
  mapReduce2Clients: DS.attr('number')
});

App.MapReduce2Service.FIXTURES = [];

});

require.register("models/service/onefs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.ONEFSService = App.Service.extend({
  nameNodeStartTime: DS.attr('number'),
  jvmMemoryHeapUsed: DS.attr('number'),
  jvmMemoryHeapMax: DS.attr('number'),
  capacityUsed: DS.attr('number'),
  capacityTotal: DS.attr('number'),
  capacityRemaining: DS.attr('number'),
  capacityNonDfsUsed: DS.attr('number'),
  dfsTotalBlocks: DS.attr('number'),
  dfsCorruptBlocks: DS.attr('number'),
  dfsMissingBlocks: DS.attr('number'),
  dfsUnderReplicatedBlocks: DS.attr('number'),
  dfsTotalFiles: DS.attr('number'),
  metricsNotAvailable: DS.attr('boolean'),
  decommissionDataNodes: DS.hasMany('App.HostComponent'),
  liveDataNodes: DS.hasMany('App.HostComponent'),
  deadDataNodes: DS.hasMany('App.HostComponent'),
  upgradeStatus: DS.attr('string'),
  nfsGatewaysStarted: DS.attr('number', { defaultValue: 0 }),
  nfsGatewaysInstalled: DS.attr('number', { defaultValue: 0 }),
  nfsGatewaysTotal: DS.attr('number', { defaultValue: 0 })
});

App.ONEFSService.FIXTURES = [];

});

require.register("models/service/ranger", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.RangerService = App.Service.extend({
  rangerTagsyncsStarted: DS.attr('number', { defaultValue: 0 }),
  rangerTagsyncsInstalled: DS.attr('number', { defaultValue: 0 }),
  rangerTagsyncsTotal: DS.attr('number', { defaultValue: 0 })
});

App.RangerService.FIXTURES = [];

});

require.register("models/service/storm", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.StormService = App.Service.extend({
  version: DS.attr('string'),
  superVisorsStarted: DS.attr('number'),
  superVisorsInstalled: DS.attr('number'),
  superVisorsTotal: DS.attr('number'),
  totalTasks: DS.attr('number'),
  totalSlots: DS.attr('number'),
  usedSlots: DS.attr('number'),
  freeSlots: DS.attr('number'),
  totalExecutors: DS.attr('number'),
  topologies: DS.attr('number'),
  nimbusUptime: DS.attr('string')
});

App.StormService.FIXTURES = [];

});

require.register("models/service/yarn", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var objectUtils = require('utils/object_utils');

App.YARNService = App.Service.extend({
  resourceManager: DS.belongsTo('App.HostComponent'),
  isRMHaEnabled: function () {
    return this.get('hostComponents').filterProperty('componentName', 'RESOURCEMANAGER').length > 1;
  }.property('hostComponents'),
  activeResourceManager: DS.belongsTo('App.HostComponent'),
  appTimelineServer: DS.belongsTo('App.HostComponent'),
  nodeManagersStarted: DS.attr('number'),
  nodeManagersInstalled: DS.attr('number'),
  nodeManagersTotal: DS.attr('number'),
  nodeManagersCountActive: DS.attr('number'),
  nodeManagersCountLost: DS.attr('number'),
  nodeManagersCountUnhealthy: DS.attr('number'),
  nodeManagersCountRebooted: DS.attr('number'),
  nodeManagersCountDecommissioned: DS.attr('number'),
  containersAllocated: DS.attr('number'),
  containersPending: DS.attr('number'),
  containersReserved: DS.attr('number'),
  appsSubmitted: DS.attr('number'),
  appsRunning: DS.attr('number'),
  appsPending: DS.attr('number'),
  appsCompleted: DS.attr('number'),
  appsKilled: DS.attr('number'),
  appsFailed: DS.attr('number'),
  ahsWebPort: function () {
    var yarnConf = App.db.getConfigs().findProperty('type', 'yarn-site');
    if (yarnConf) {
      return yarnConf.properties['yarn.timeline-service.webapp.address'].match(/:(\d+)/)[1];
    }
    return "8188";
  }.property(),
  resourceManagerStartTime: DS.attr('number'),
  jvmMemoryHeapUsed: DS.attr('number'),
  jvmMemoryHeapMax: DS.attr('number'),
  allocatedMemory: DS.attr('number'),
  reservedMemory: DS.attr('number'),
  availableMemory: DS.attr('number'),
  queue: DS.attr('string'),
  queueFormatted: function () {
    var queue = JSON.parse(this.get('queue'));
    return objectUtils.recursiveTree(queue);
  }.property('queue'),
  queuesCount: function () {
    var queue = JSON.parse(this.get('queue'));
    return objectUtils.recursiveKeysCount(queue);
  }.property('queue'),
  allQueueNames: [],
  childQueueNames: [],

  maxMemory: Em.computed.sumProperties('allocatedMemory', 'availableMemory'),

  /**
   * Provides a flat array of queue names.
   * Example: root, root/default
   */
  queueNames: function () {
    var queueString = this.get('queue');
    var allQueueNames = [];
    var childQueueNames = [];
    if (queueString != null) {
      var queues = JSON.parse(queueString);
      var addQueues = function addQueues(queuesObj, path) {
        var names = [];
        for (var subQueue in queuesObj) {
          if (queuesObj[subQueue] instanceof Object) {
            var qFN = path == '' ? subQueue : path + '/' + subQueue;
            names.push(qFN);
            var subNames = addQueues(queuesObj[subQueue], qFN);
            names = names.concat(subNames);
            if (!subNames || subNames.length < 1) {
              childQueueNames.push(qFN);
            }
          }
        }
        return names;
      };
      allQueueNames = addQueues(queues, '');
    }
    this.set('allQueueNames', allQueueNames);
    this.set('childQueueNames', childQueueNames);
  }.observes('queue')
  /**
   * ResourceManager's lost count is not accurate once RM is rebooted. Since
   * Ambari knows the total number of nodes and the counts of nodes in other
   * states, we calculate the lost count.
   *
  nodeManagersCountLost: function () {
    var totalCount = this.get('nodeManagersTotal');
    var activeCount = this.get('nodeManagersCountActive');
    var rebootedCount = this.get('nodeManagersCountRebooted');
    var unhealthyCount = this.get('nodeManagersCountUnhealthy');
    var decomCount = this.get('nodeManagersCountDecommissioned');
    var nonLostHostsCount = activeCount + rebootedCount + decomCount + unhealthyCount;
    return totalCount >= nonLostHostsCount ? totalCount - nonLostHostsCount : 0;
  }.property('nodeManagersTotal', 'nodeManagersCountActive', 'nodeManagersCountRebooted', 'nodeManagersCountUnhealthy', 'nodeManagersCountDecommissioned')*/
});

App.YARNService.FIXTURES = [];

});

require.register("models/service_audit", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceAudit = DS.Model.extend({
  date: DS.attr('date'),
  service: DS.belongsTo('App.Service'),
  operationName: DS.attr('string'),
  user: DS.belongsTo('App.User')
});

App.ServiceAudit.FIXTURES = [{
  id: 1,
  date: 'September 12, 2012 17:00',
  operation_name: 'Reconfigure',
  user_id: 2,
  service_id: 1
}, {
  id: 2,
  date: 'September 13, 2012 17:00',
  operation_name: 'Start',
  user_id: 1,
  service_id: 1
}, {
  id: 3,
  date: 'September 14, 2012 17:00',
  operation_name: 'Install',
  user_id: 1,
  service_id: 1
}, {
  id: 4,
  date: 'September 12, 2012 17:00',
  operation_name: 'Reconfigure',
  user_id: 2,
  service_id: 2
}, {
  id: 5,
  date: 'September 13, 2012 17:00',
  operation_name: 'Start',
  user_id: 1,
  service_id: 2
}, {
  id: 6,
  date: 'September 14, 2012 17:00',
  operation_name: 'Install',
  user_id: 1,
  service_id: 2
}];

});

require.register("models/slave_component", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.SlaveComponent = App.ClientComponent.extend({
  displayNamePluralized: function () {
    return stringUtils.pluralize(this.get('startedCount'), this.get('displayName'));
  }.property('startedCount')
});

App.SlaveComponent.FIXTURES = [];

});

require.register("models/stack", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.Stack = DS.Model.extend({
  id: DS.attr('string'), //  ${stackName}-${stackVersion}-${repositoryVersion}.
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  stackDefault: DS.attr('boolean'),
  stackRepoUpdateLinkExists: DS.attr('boolean'),
  minJdkVersion: DS.attr('string'),
  maxJdkVersion: DS.attr('string'),
  repositoryVersion: DS.attr('string'),
  showAvailable: DS.attr('boolean'), // All of the instances should have this value to true. We should map only those stacks that has this flag set to true
  type: DS.attr('string'), // ["PATCH", "STANDARD"]
  useRedhatSatellite: DS.attr('boolean'),
  stackServices: DS.hasMany('App.ServiceSimple'),
  operatingSystems: DS.hasMany('App.OperatingSystem'),
  isSelected: DS.attr('boolean', { defaultValue: false }),

  versionInfoId: null,

  stackNameVersion: Em.computed.concat('-', 'stackName', 'stackVersion'),

  isPatch: Em.computed.equal('type', 'PATCH'),

  displayName: Em.computed.concat('-', 'stackName', 'repositoryVersion'),

  /**
   * @type {boolean}
   */
  usePublicRepo: true,

  /**
   * @type {boolean}
   */
  useLocalRepo: false,

  /**
   * @return: {Array} returns supported repositories for all OperatingSystem's supported by a stack instance
   */
  repositories: function () {
    var operatingSystems = this.get('operatingSystems');
    var repositories = [];
    operatingSystems.filterProperty('isSelected', true).forEach(function (os) {
      os.get('repositories').forEach(function (repository) {
        repositories.pushObject(repository);
      }, this);
    }, this);
    return repositories;
  }.property('operatingSystems.@each.isSelected'),

  cleanReposBaseUrls: function cleanReposBaseUrls() {
    this.get('operatingSystems').forEach(function (os) {
      os.get('repositories').setEach('baseUrl', '');
    });
  },

  restoreReposBaseUrls: function restoreReposBaseUrls() {
    this.get('operatingSystems').forEach(function (os) {
      os.get('repositories').forEach(function (repo) {
        repo.set('baseUrl', repo.get('baseUrlInit'));
      });
    });
  }

});

App.Stack.FIXTURES = [];

});

require.register("models/stack_service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');
require('utils/helper');
require('models/configs/objects/service_config_category');

var Dependency = Ember.Object.extend({
  name: Ember.computed('service', function () {
    return this.get('service.serviceName');
  }),

  displayName: Ember.computed('service', function () {
    return this.get('service.displayNameOnSelectServicePage');
  }),

  compatibleServices: function compatibleServices(services) {
    return services.filterProperty('serviceName', this.get('name'));
  },

  isMissing: function isMissing(selectedServices) {
    return Em.isEmpty(this.compatibleServices(selectedServices));
  }
});

var HcfsDependency = Dependency.extend({
  displayName: Ember.computed(function () {
    return Em.I18n.t('installer.step4.hcfs.displayName');
  }),

  compatibleServices: function compatibleServices(services) {
    return services.filterProperty("isDFS", true);
  }
});

Dependency.reopenClass({
  fromService: function fromService(service) {
    return service.get('isDFS') ? HcfsDependency.create({ service: service }) : Dependency.create({ service: service });
  }
});

var MissingDependency = Ember.Object.extend({
  hasMultipleOptions: Ember.computed('compatibleServices', function () {
    return this.get('compatibleServices').length > 1;
  }),

  selectFirstCompatible: function selectFirstCompatible() {
    this.get('compatibleServices')[0].set('isSelected', true);
  },

  displayOptions: Ember.computed('compatibleServices', function () {
    return this.get('compatibleServices').mapProperty('serviceName').join(', ');
  })
});

App.FileSystem = Ember.ObjectProxy.extend({
  content: null,
  services: [],

  isSelected: function (key, aBoolean) {
    if (arguments.length > 1) {
      this.clearAllSelection();
      this.get('content').set('isSelected', aBoolean);
    }
    return this.get('content.isSelected');
  }.property('content.isSelected', 'services.@each.isSelected'),

  clearAllSelection: function clearAllSelection() {
    this.get('services').setEach('isSelected', false);
  }
});

/**
 * This model loads all services supported by the stack
 * The model maps to the  http://hostname:8080/api/v1/stacks/HDP/versions/${versionNumber}/services?fields=StackServices/*,serviceComponents/*
 * @type {*}
 */
App.StackService = DS.Model.extend({
  serviceName: DS.attr('string'),
  serviceType: DS.attr('string'),
  displayName: DS.attr('string'),
  comments: DS.attr('string'),
  configTypes: DS.attr('object'),
  serviceVersion: DS.attr('string'),
  serviceCheckSupported: DS.attr('boolean'),
  supportDeleteViaUi: DS.attr('boolean'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  selection: DS.attr('string'),
  isMandatory: DS.attr('boolean', { defaultValue: false }),
  isSelected: DS.attr('boolean', { defaultValue: true }),
  isInstalled: DS.attr('boolean', { defaultValue: false }),
  isInstallable: DS.attr('boolean', { defaultValue: true }),
  isServiceWithWidgets: DS.attr('boolean', { defaultValue: false }),
  stack: DS.belongsTo('App.Stack'),
  serviceComponents: DS.hasMany('App.StackServiceComponent'),
  configs: DS.attr('array'),
  requiredServices: DS.attr('array', { defaultValue: [] }),

  isDisabled: function () {
    return this.get('isInstalled') || this.get('isMandatory') && !App.get('router.clusterInstallCompleted');
  }.property('isMandatory', 'isInstalled', 'App.router.clusterInstallCompleted'),

  /**
   * @type {String[]}
   */
  configTypeList: function () {
    var configTypes = Object.keys(this.get('configTypes') || {});
    //Falcon has dependency on oozie-site but oozie-site advanced/custom section should not be shown on Falcon page
    if (this.get('serviceName') === 'FALCON') {
      configTypes = configTypes.without('oozie-site');
    }
    return configTypes;
  }.property('configTypes'),

  /**
   * contains array of serviceNames that have configs that
   * depends on configs from current service
   * @type {String[]}
   */
  dependentServiceNames: DS.attr('array', { defaultValue: [] }),

  dependencies: function dependencies(availableServices) {
    var result = [];
    this.get('requiredServices').forEach(function (serviceName) {
      var service = availableServices.findProperty('serviceName', serviceName);
      if (service) {
        result.push(Dependency.fromService(service));
      }
    });
    return result;
  },

  /**
   * Add dependencies which are not already included in selectedServices to the given missingDependencies collection
  */
  collectMissingDependencies: function collectMissingDependencies(selectedServices, availableServices, missingDependencies) {
    this._missingDependencies(selectedServices, availableServices).forEach(function (dependency) {
      this._addMissingDependency(dependency, availableServices, missingDependencies);
    }.bind(this));
  },

  _missingDependencies: function _missingDependencies(selectedServices, availableServices) {
    return this.dependencies(availableServices).filter(function (dependency) {
      return dependency.isMissing(selectedServices);
    });
  },

  _addMissingDependency: function _addMissingDependency(dependency, availableServices, missingDependencies) {
    if (!missingDependencies.someProperty('serviceName', dependency.get('name'))) {
      missingDependencies.push(MissingDependency.create({
        serviceName: dependency.get('name'),
        displayName: dependency.get('displayName'),
        compatibleServices: dependency.compatibleServices(availableServices)
      }));
    }
  },

  // Is the service a distributed filesystem
  isDFS: function () {
    return this.get('serviceType') === 'HCFS' || ['HDFS', 'GLUSTERFS'].contains(this.get('serviceName'));
  }.property('serviceName', 'serviceType'),

  // Primary DFS. used if there is more than one DFS in a stack.
  // Only one service in the stack should be tagged as primary DFS.
  isPrimaryDFS: Em.computed.equal('serviceName', 'HDFS'),

  configTypesRendered: function () {
    var configTypes = this.get('configTypes');
    var renderedConfigTypes = $.extend(true, {}, configTypes);
    if (this.get('serviceName') == 'FALCON') {
      delete renderedConfigTypes['oozie-site'];
    }
    return renderedConfigTypes;
  }.property('serviceName', 'configTypes'),

  displayNameOnSelectServicePage: function () {
    var displayName = this.get('displayName');
    var services = this.get('coSelectedServices').slice();
    var serviceDisplayNames = services.map(function (item) {
      return App.format.role(item, true);
    }, this);
    if (!!serviceDisplayNames.length) {
      serviceDisplayNames.unshift(displayName);
      displayName = serviceDisplayNames.join(" + ");
    }
    return displayName;
  }.property('coSelectedServices', 'serviceName'),

  isHiddenOnSelectServicePage: function () {
    var hiddenServices = ['MAPREDUCE2'];
    return hiddenServices.contains(this.get('serviceName')) || !this.get('isInstallable') || this.get('doNotShowAndInstall');
  }.property('serviceName', 'isInstallable'),

  doNotShowAndInstall: function () {
    var skipServices = [];
    if (!App.supports.installGanglia) {
      skipServices.push('GANGLIA');
    }
    return skipServices.contains(this.get('serviceName'));
  }.property('serviceName'),

  // Is the service required for monitoring of other hadoop ecosystem services
  isMonitoringService: Em.computed.existsIn('serviceName', ['GANGLIA']),

  // Is the service required for reporting host metrics
  isHostMetricsService: Em.computed.existsIn('serviceName', ['GANGLIA', 'AMBARI_METRICS']),

  // Is the service required for reporting hadoop service metrics
  isServiceMetricsService: Em.computed.existsIn('serviceName', ['GANGLIA']),

  coSelectedServices: function () {
    var coSelectedServices = App.StackService.coSelected[this.get('serviceName')];
    if (!!coSelectedServices) {
      return coSelectedServices;
    } else {
      return [];
    }
  }.property('serviceName'),

  hasClient: Em.computed.someBy('serviceComponents', 'isClient', true),

  hasMaster: Em.computed.someBy('serviceComponents', 'isMaster', true),

  hasSlave: Em.computed.someBy('serviceComponents', 'isSlave', true),

  hasNonMastersWithCustomAssignment: function () {
    var serviceComponents = this.get('serviceComponents');
    return serviceComponents.rejectProperty('isMaster').rejectProperty('cardinality', 'ALL').length > 0;
  }.property('serviceName'),

  isClientOnlyService: Em.computed.everyBy('serviceComponents', 'isClient', true),

  isNoConfigTypes: function () {
    var configTypes = this.get('configTypes');
    return !(configTypes && !!Object.keys(configTypes).length);
  }.property('configTypes'),

  customReviewHandler: function () {
    return App.StackService.reviewPageHandlers[this.get('serviceName')];
  }.property('serviceName'),

  hasHeatmapSection: Em.computed.existsIn('serviceName', ['HDFS', 'YARN', 'HBASE']),

  /**
   * configCategories are fetched from  App.StackService.configCategories.
   * Also configCategories that does not match any serviceComponent of a service and not included in the permissible default pattern are omitted
   */
  configCategories: function () {
    var configCategories = [];
    var configTypes = this.get('configTypes');
    var serviceComponents = this.get('serviceComponents');
    if (configTypes && Object.keys(configTypes).length) {
      var pattern = ["General", "ResourceType", "CapacityScheduler", "ContainerExecutor", "Registry", "FaultTolerance", "Isolation", "Performance", "HIVE_SERVER2", "KDC", "Kadmin", "^Advanced", "Env$", "^Custom", "Falcon - Oozie integration", "FalconStartupSite", "FalconRuntimeSite", "MetricCollector", "Settings$", "AdvancedHawqCheck", "LogsearchAdminJson"];
      configCategories = App.StackService.configCategories.call(this).filter(function (_configCategory) {
        var serviceComponentName = _configCategory.get('name');
        var isServiceComponent = serviceComponents.someProperty('componentName', serviceComponentName);
        if (isServiceComponent) return isServiceComponent;
        var result = false;
        pattern.forEach(function (_pattern) {
          var regex = new RegExp(_pattern);
          if (regex.test(serviceComponentName)) result = true;
        });
        return result;
      });
    }
    return configCategories;
  }.property('serviceName', 'configTypes', 'serviceComponents'),

  /**
   * Compare specified version with current service version
   * @param  {string} version [description]
   * @return {number} 0 - equal, -1 - less, +1 - more @see stringUtils#compareVersions
   */
  compareCurrentVersion: function compareCurrentVersion(version) {
    var toMinor = this.get('serviceVersion').split('.').slice(0, 2).join('.');
    return stringUtils.compareVersions(toMinor, version);
  }
});

App.StackService.FIXTURES = [];

App.StackService.displayOrder = ['HDFS', 'GLUSTERFS', 'YARN', 'MAPREDUCE2', 'TEZ', 'GANGLIA', 'HIVE', 'HAWQ', 'PXF', 'HBASE', 'PIG', 'SQOOP', 'OOZIE', 'ZOOKEEPER', 'FALCON', 'STORM', 'FLUME', 'ACCUMULO', 'AMBARI_INFRA_SOLR', 'AMBARI_METRICS', 'ATLAS', 'KAFKA', 'KNOX', 'LOGSEARCH', 'RANGER', 'RANGER_KMS', 'SMARTSENSE', 'SPARK', 'SPARK2', 'ZEPPELIN'];

App.StackService.componentsOrderForService = {
  'HAWQ': ['HAWQMASTER', 'HAWQSTANDBY']
};

//@TODO: Write unit test for no two keys in the object should have any intersecting elements in their values
App.StackService.coSelected = {
  'YARN': ['MAPREDUCE2']
};

App.StackService.reviewPageHandlers = {
  'HIVE': {
    'Database': 'loadHiveDbValue'
  },
  'OOZIE': {
    'Database': 'loadOozieDbValue'
  }
};

App.StackService.configCategories = function () {
  var serviceConfigCategories = [];
  switch (this.get('serviceName')) {
    case 'HDFS':
      serviceConfigCategories.pushObject(App.ServiceConfigCategory.create({ name: 'NAMENODE', displayName: 'NameNode', showHost: true }));
      if (!App.get('isHaEnabled')) {
        serviceConfigCategories.pushObject(App.ServiceConfigCategory.create({ name: 'SECONDARY_NAMENODE', displayName: 'Secondary NameNode', showHost: true }));
      }
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'DATANODE', displayName: 'DataNode', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' }), App.ServiceConfigCategory.create({ name: 'NFS_GATEWAY', displayName: 'NFS Gateway', showHost: true })]);
      break;
    case 'GLUSTERFS':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'YARN':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'RESOURCEMANAGER', displayName: 'Resource Manager', showHost: true }), App.ServiceConfigCategory.create({ name: 'NODEMANAGER', displayName: 'Node Manager', showHost: true }), App.ServiceConfigCategory.create({ name: 'APP_TIMELINE_SERVER', displayName: 'Application Timeline Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' }), App.ServiceConfigCategory.create({ name: 'ResourceTypes', displayName: 'Resource Types' }), App.ServiceConfigCategory.create({ name: 'FaultTolerance', displayName: 'Fault Tolerance' }), App.ServiceConfigCategory.create({ name: 'Isolation', displayName: 'Isolation' }), App.ServiceConfigCategory.create({ name: 'CapacityScheduler', displayName: 'Scheduler', siteFileName: 'capacity-scheduler.xml' }), App.ServiceConfigCategory.create({ name: 'ContainerExecutor', displayName: 'Container Executor', siteFileName: 'container-executor.xml' }), App.ServiceConfigCategory.create({ name: 'Registry', displayName: 'Registry' })]);
      break;
    case 'MAPREDUCE2':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'HISTORYSERVER', displayName: 'History Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'HIVE':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'HIVE_METASTORE', displayName: 'Hive Metastore', showHost: true }), App.ServiceConfigCategory.create({ name: 'WEBHCAT_SERVER', displayName: 'WebHCat Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' }), App.ServiceConfigCategory.create({ name: 'Performance', displayName: 'Performance' }), App.ServiceConfigCategory.create({ name: 'HIVE_SERVER2', displayName: 'Hive Server2' }), App.ServiceConfigCategory.create({ name: 'HIVE_CLIENT', displayName: 'Hive Client' })]);
      break;
    case 'HBASE':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'HBASE_MASTER', displayName: 'HBase Master', showHost: true }), App.ServiceConfigCategory.create({ name: 'HBASE_REGIONSERVER', displayName: 'RegionServer', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'ZOOKEEPER':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'ZOOKEEPER_SERVER', displayName: 'ZooKeeper Server', showHost: true })]);
      break;
    case 'OOZIE':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'OOZIE_SERVER', displayName: 'Oozie Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'Falcon - Oozie integration', displayName: 'Falcon - Oozie integration' })]);
      break;
    case 'FALCON':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'FALCON_SERVER', displayName: 'Falcon Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'Falcon - Oozie integration', displayName: 'Falcon - Oozie integration' }), App.ServiceConfigCategory.create({ name: 'FalconStartupSite', displayName: 'Falcon startup.properties' }), App.ServiceConfigCategory.create({ name: 'FalconRuntimeSite', displayName: 'Falcon runtime.properties' }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'STORM':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'NIMBUS', displayName: 'Nimbus', showHost: true }), App.ServiceConfigCategory.create({ name: 'SUPERVISOR', displayName: 'Supervisor', showHost: true }), App.ServiceConfigCategory.create({ name: 'STORM_UI_SERVER', displayName: 'Storm UI Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'STORM_REST_API', displayName: 'Storm REST API Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'DRPC_SERVER', displayName: 'DRPC Server', showHost: true }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'TEZ':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'FLUME':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'FLUME_HANDLER', displayName: 'flume.conf', siteFileName: 'flume-conf', canAddProperty: false })]);
      break;
    case 'KNOX':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'KNOX_GATEWAY', displayName: 'Knox Gateway', showHost: true })]);
      break;
    case 'KAFKA':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'KAFKA_BROKER', displayName: 'Kafka Broker', showHost: true })]);
      break;
    case 'KERBEROS':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'KDC', displayName: 'KDC' }), App.ServiceConfigCategory.create({ name: 'Kadmin', displayName: 'Kadmin' }), App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'AMBARI_METRICS':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' }), App.ServiceConfigCategory.create({ name: 'MetricCollector', displayName: 'Metric Collector' })]);
      break;
    case 'RANGER':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'RANGER_ADMIN', displayName: 'Admin Settings', showHost: true }), App.ServiceConfigCategory.create({ name: 'DBSettings', displayName: 'DB Settings' }), App.ServiceConfigCategory.create({ name: 'RangerSettings', displayName: 'Ranger Settings' }), App.ServiceConfigCategory.create({ name: 'UnixAuthenticationSettings', displayName: 'Unix Authentication Settings' }), App.ServiceConfigCategory.create({ name: 'ADSettings', displayName: 'AD Settings' }), App.ServiceConfigCategory.create({ name: 'LDAPSettings', displayName: 'LDAP Settings' }), App.ServiceConfigCategory.create({ name: 'KnoxSSOSettings', displayName: 'Knox SSO Settings' })]);
      break;
    case 'ACCUMULO':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
      break;
    case 'PIG':
      break;
    case 'SQOOP':
      break;
    case 'HAWQ':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' }), App.ServiceConfigCategory.create({ name: 'AdvancedHawqCheck', displayName: 'Advanced HAWQ Check' })]);
      break;
    case 'LOGSEARCH':
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'LogsearchAdminJson', displayName: 'Advanced logsearch-admin-json' })]);
      break;
    default:
      serviceConfigCategories.pushObjects([App.ServiceConfigCategory.create({ name: 'General', displayName: 'General' })]);
  }
  serviceConfigCategories.pushObject(App.ServiceConfigCategory.create({ name: 'Advanced', displayName: 'Advanced' }));

  var configTypes = this.get('configTypeList');

  // Add Advanced section for every configType to all the services
  configTypes.forEach(function (type) {
    serviceConfigCategories.pushObject(App.ServiceConfigCategory.create({
      name: 'Advanced ' + type,
      displayName: Em.I18n.t('common.advanced') + " " + type,
      canAddProperty: false
    }));
  }, this);

  // Add custom section for every configType to all the services
  configTypes.forEach(function (type) {
    if (Em.getWithDefault(this.get('configTypes')[type] || {}, 'supports.adding_forbidden', 'true') === 'false') {
      serviceConfigCategories.pushObject(App.ServiceConfigCategory.create({
        name: 'Custom ' + type,
        displayName: Em.I18n.t('common.custom') + " " + type,
        siteFileName: type + '.xml',
        canAddProperty: true
      }));
    }
  }, this);
  return serviceConfigCategories;
};

});

require.register("models/stack_service_component", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var numberUtils = require('utils/number_utils');

var ComponentDependency = Ember.Object.extend({
  componentName: null,
  compatibleComponents: [],

  /**
   * Find the first compatible component which belongs to a service that is installed
   */
  chooseCompatible: function chooseCompatible() {
    var compatibleComponent = this.get('compatibleComponents').find(function (component) {
      return App.Service.find().someProperty('serviceName', component.get('serviceName'));
    });
    return (compatibleComponent ? compatibleComponent : this.get('compatibleComponents')[0]).get('componentName');
  }
});

/**
 * This model loads all serviceComponents supported by the stack
 * @type {*}
 */
App.StackServiceComponent = DS.Model.extend({
  componentName: DS.attr('string'),
  displayName: DS.attr('string'),
  cardinality: DS.attr('string'),
  customCommands: DS.attr('array'),
  reassignAllowed: DS.attr('boolean'),
  decommissionAllowed: DS.attr('boolean'),
  hasBulkCommandsDefinition: DS.attr('boolean'),
  bulkCommandsDisplayName: DS.attr('string'),
  bulkCommandsMasterComponentName: DS.attr('string'),
  dependencies: DS.attr('array'),
  serviceName: DS.attr('string'),
  componentCategory: DS.attr('string'),
  rollingRestartSupported: DS.attr('boolean'),
  isMaster: DS.attr('boolean'),
  isClient: DS.attr('boolean'),
  componentType: DS.attr('string'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  stackService: DS.belongsTo('App.StackService'),
  serviceComponentId: DS.attr('number', { defaultValue: 1 }), // this is used on Assign Master page for multiple masters

  /**
   * Minimum required count for installation.
   *
   * @property {Number} minToInstall
   **/
  minToInstall: function () {
    return numberUtils.getCardinalityValue(this.get('cardinality'), false);
  }.property('cardinality'),

  /**
   * Maximum required count for installation.
   *
   * @property {Number} maxToInstall
   **/
  maxToInstall: function () {
    return numberUtils.getCardinalityValue(this.get('cardinality'), true);
  }.property('cardinality'),

  /**
   * Check if the given component is compatible with this one. Having the same name or componentType indicates compatibility.
   **/
  dependsOn: function dependsOn(aStackServiceComponent, opt) {
    return this.get('dependencies').some(function (each) {
      return aStackServiceComponent.compatibleWith(App.StackServiceComponent.find(each.componentName));
    });
  },

  compatibleWith: function compatibleWith(aStackServiceComponent) {
    return this.get('componentName') === aStackServiceComponent.get('componentName') || this.get('componentType') && this.get('componentType') === aStackServiceComponent.get('componentType');
  },

  /**
   * Collect dependencies which are required by this component but not installed.
   * A compatible installed component (e.g.: componentType=HCFS_CLIENT) is not considered as a missing dependency.
   **/
  missingDependencies: function missingDependencies(installedComponents, opt) {
    opt = opt || {};
    opt.scope = opt.scope || '*';
    var dependencies = this.get('dependencies');
    dependencies = opt.scope === '*' ? dependencies : dependencies.filterProperty('scope', opt.scope);
    if (dependencies.length === 0) return [];
    installedComponents = installedComponents.map(function (each) {
      return App.StackServiceComponent.find(each);
    });
    var missingComponents = dependencies.filter(function (dependency) {
      return !installedComponents.some(function (each) {
        return each.compatibleWith(App.StackServiceComponent.find(dependency.componentName));
      });
    }).mapProperty('componentName');
    return missingComponents.map(function (missingComponentName) {
      return ComponentDependency.create({
        'componentName': missingComponentName,
        'compatibleComponents': App.StackServiceComponent.find().filter(function (each) {
          return each.compatibleWith(App.StackServiceComponent.find(missingComponentName));
        })
      });
    });
  },

  /** @property {Boolean} isRequired - component required to install **/
  isRequired: Em.computed.gt('minToInstall', 0),

  /** @property {Boolean} isMultipleAllowed - component can be assigned for more than one host **/
  isMultipleAllowed: Em.computed.gt('maxToInstall', 1),

  /** @property {Boolean} isSlave **/
  isSlave: Em.computed.equal('componentCategory', 'SLAVE'),

  /** @property {Boolean} isRestartable - component supports restart action **/
  isRestartable: Em.computed.not('isClient'),

  /** @property {Boolean} isReassignable - component supports reassign action **/
  isReassignable: function () {
    return this.get('reassignAllowed');
  }.property('reassignAllowed'),

  /** @property {Boolean} isNonHDPComponent - component not belongs to HDP services **/
  isNonHDPComponent: function () {
    return ['METRICS_COLLECTOR', 'METRICS_MONITOR'].contains(this.get('componentName'));
  }.property('componentName'),

  /** @property {Boolean} isRollinRestartAllowed - component supports rolling restart action **/
  isRollinRestartAllowed: function () {
    return this.get('isSlave') || this.get('rollingRestartSupported');
  }.property('componentName'),

  /** @property {Boolean} isDecommissionAllowed - component supports decommission action **/
  isDecommissionAllowed: function () {
    return this.get('decommissionAllowed');
  }.property('decommissionAllowed'),

  /** @property {Boolean} isRefreshConfigsAllowed - component supports refresh configs action **/
  isRefreshConfigsAllowed: Em.computed.existsIn('componentName', ['FLUME_HANDLER']),

  /** @property {Boolean} isAddableToHost - component can be added on host details page **/
  isAddableToHost: function () {
    return this.get('isMasterAddableInstallerWizard') || (this.get('isNotAddableOnlyInInstall') || this.get('isSlave') || this.get('isClient')) && (!this.get('isHAComponentOnly') || App.get('isHaEnabled') && this.get('componentName') === 'JOURNALNODE');
  }.property('componentName'),

  /** @property {Boolean} isDeletable - component supports delete action **/
  isDeletable: function () {
    var ignored = [];
    return this.get('isAddableToHost') && !ignored.contains(this.get('componentName')) || this.get('componentName') === 'MYSQL_SERVER';
  }.property('componentName'),

  /**
   * @type {boolean}
   */
  isInstallable: function () {
    var notInstallable = App.get('currentStackName') === 'HDF' ? ['ACTIVITY_ANALYZER', 'ACTIVITY_EXPLORER'] : [];
    return !notInstallable.contains(this.get('componentName'));
  }.property('componentName'),

  /** @property {Boolean} isShownOnInstallerAssignMasterPage - component visible on "Assign Masters" step of Install Wizard **/
  // Note: Components that are not visible on Assign Master Page are not saved as part of host component recommendation/validation layout
  isShownOnInstallerAssignMasterPage: function () {
    var component = this.get('componentName');
    var mastersNotShown = ['MYSQL_SERVER', 'POSTGRESQL_SERVER', 'HIVE_SERVER_INTERACTIVE'];
    return this.get('isMaster') && this.get('isInstallable') && !mastersNotShown.contains(component);
  }.property('isMaster', 'componentName', 'isInstallable'),

  /** @property {Boolean} isShownOnInstallerSlaveClientPage - component visible on "Assign Slaves and Clients" step of Install Wizard**/
  // Note: Components that are not visible on Assign Slaves and Clients Page are saved as part of host component recommendation/validation layout
  isShownOnInstallerSlaveClientPage: function () {
    var component = this.get('componentName');
    var slavesNotShown = ['JOURNALNODE', 'ZKFC', 'APP_TIMELINE_SERVER'];
    return this.get('isSlave') && !this.get('isRequiredOnAllHosts') && !slavesNotShown.contains(component);
  }.property('isSlave', 'componentName', 'isRequiredOnAllHosts'),

  /** @property {Boolean} isShownOnAddServiceAssignMasterPage - component visible on "Assign Masters" step of Add Service Wizard **/
  // Note: Components that are not visible on Assign Master Page are not saved as part of host component recommendation/validation layout
  isShownOnAddServiceAssignMasterPage: function () {
    var isVisible = this.get('isShownOnInstallerAssignMasterPage');
    if (App.get('isHaEnabled')) {
      isVisible = isVisible && this.get('componentName') !== 'SECONDARY_NAMENODE';
    }
    return isVisible;
  }.property('isShownOnInstallerAssignMasterPage', 'App.isHaEnabled'),

  /** @property {Boolean} isMasterWithMultipleInstances **/
  isMasterWithMultipleInstances: function () {
    // @todo: safe removing JOURNALNODE from masters list
    return this.get('isMaster') && this.get('isMultipleAllowed') || this.get('componentName') == 'JOURNALNODE';
  }.property('componentName'),

  /**
   * Master component list that could be assigned for more than 1 host.
   * Some components like NameNode and ResourceManager have range cardinality value, so they are excluded using isMasterAddableOnlyOnHA property
   *
   * @property {Boolean} isMasterAddableInstallerWizard
   **/
  isMasterAddableInstallerWizard: Em.computed.and('isMaster', 'isMultipleAllowed', '!isMasterAddableOnlyOnHA', '!isNotAddableOnlyInInstall'),

  /**
   * Master components with cardinality more than 1 (n+ or n-n) that could not be added in wizards
   * New instances of these components are added in appropriate HA wizards
   * @property {Boolean} isMasterAddableOnlyOnHA
   */
  isMasterAddableOnlyOnHA: Em.computed.existsIn('componentName', ['NAMENODE', 'RESOURCEMANAGER', 'RANGER_ADMIN']),

  /** @property {Boolean} isHAComponentOnly - Components that can be installed only if HA enabled **/
  isHAComponentOnly: Em.computed.existsIn('componentName', ['ZKFC', 'JOURNALNODE']),

  /** @property {Boolean} isRequiredOnAllHosts - Is It require to install the components on all hosts. used in step-6 wizard controller **/
  isRequiredOnAllHosts: Em.computed.equal('minToInstall', Infinity),

  /** @property {Number} defaultNoOfMasterHosts - default number of master hosts on Assign Master page: **/
  defaultNoOfMasterHosts: function () {
    if (this.get('isMasterAddableInstallerWizard')) {
      return this.get('componentName') === 'ZOOKEEPER_SERVER' ? 3 : this.get('minToInstall');
    }
  }.property('componentName'),

  /** @property {Boolean} coHostedComponents - components that are co-hosted with this component **/
  coHostedComponents: function () {
    var componentName = this.get('componentName');
    var key,
        coHostedComponents = [];
    for (key in App.StackServiceComponent.coHost) {
      if (App.StackServiceComponent.coHost[key] === componentName) {
        coHostedComponents.push(key);
      }
    }
    return coHostedComponents;
  }.property('componentName'),

  /** @property {Boolean} isOtherComponentCoHosted - Is any other component co-hosted with this component **/
  isOtherComponentCoHosted: Em.computed.bool('coHostedComponents.length'),

  /** @property {Boolean} isCoHostedComponent - Is this component co-hosted with other component **/
  isCoHostedComponent: function () {
    var componentName = this.get('componentName');
    return !!App.StackServiceComponent.coHost[componentName];
  }.property('componentName'),

  /** @property {Boolean} isNotAddableOnlyInInstall - is this component addable, except Install and Add Service Wizards  **/
  isNotAddableOnlyInInstall: Em.computed.existsIn('componentName', ['HIVE_METASTORE', 'HIVE_SERVER', 'RANGER_KMS_SERVER', 'OOZIE_SERVER', 'TIMELINE_READER', 'YARN_REGISTRY_DNS']),

  /** @property {Boolean} isNotAllowedOnSingleNodeCluster - is this component allowed on single node  **/
  isNotAllowedOnSingleNodeCluster: Em.computed.existsIn('componentName', ['HAWQSTANDBY'])

});

App.StackServiceComponent.FIXTURES = [];

App.StackServiceComponent.coHost = {};

});

require.register("models/stack_version/os", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.OS = DS.Model.extend({
  id: DS.attr('string'),
  osType: DS.attr('string'),
  stackName: DS.attr('string'),
  stackVersion: DS.attr('string'),
  repositoryVersion: DS.belongsTo('App.RepositoryVersion'),
  repositories: DS.hasMany('App.Repository')
});

App.OS.FIXTURES = [];

});

require.register("models/stack_version/repository_version", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RepositoryVersion = DS.Model.extend({
  displayName: DS.attr('string'),
  type: DS.attr('string'), // "PATCH" /* STANDARD, (future: SERVICE) */
  repositoryVersion: DS.attr('string'),
  upgradePack: DS.attr('string'),
  stackVersionType: DS.attr('string'),
  stackVersionNumber: DS.attr('string'),
  useRedhatSatellite: DS.attr('boolean'),
  operatingSystems: DS.hasMany('App.OS'),
  stackServices: DS.hasMany('App.ServiceSimple'),
  stackVersion: DS.belongsTo('App.StackVersion'),
  stack: Em.computed.concat(' ', 'stackVersionType', 'stackVersionNumber'),
  hidden: DS.attr('boolean'),
  displayNameSimple: function () {
    return this.get('stackVersionType') + '-' + this.get('repositoryVersion').split('-')[0];
  }.property('stackVersionType', 'repositoryVersion'),

  displayNameFull: function () {
    return this.get('stackVersionType') + '-' + this.get('repositoryVersion');
  }.property('stackVersionType', 'repositoryVersion'),

  isPatch: Em.computed.equal('type', 'PATCH'),

  isMaint: Em.computed.equal('type', 'MAINT'),

  isService: Em.computed.equal('type', 'SERVICE'),

  isStandard: Em.computed.equal('type', 'STANDARD'),

  /**
   * status used until corresponding stack version get created
   * @type {string}
   */
  defaultStatus: 'NOT_REQUIRED',

  /**
   * @type {boolean}
   * @default false
   */
  isCompatible: false,

  /**
   * @type {string}
   */
  status: Em.computed.firstNotBlank('stackVersion.state', 'defaultStatus'),

  /**
   * @type {Array}
   */
  notInstalledHosts: function () {
    return this.get('stackVersion.notInstalledHosts').length || this.get('stackVersion.installedHosts').length || this.get('stackVersion.currentHosts').length ? this.get('stackVersion.notInstalledHosts') : App.get('allHostNames');
  }.property('stackVersion.notInstalledHosts'),

  /**
   * @type {Array}
   */
  installedHosts: function () {
    return this.get('stackVersion.installedHosts') || [];
  }.property('stackVersion.installedHosts'),

  /**
   * @type {Array}
   */
  currentHosts: function () {
    return this.get('stackVersion.currentHosts') || [];
  }.property('stackVersion.currentHosts'),

  /**
   * @type {boolean}
   */
  noInstalledHosts: function () {
    return this.get('stackVersion') ? this.get('stackVersion.noInstalledHosts') : true;
  }.property('stackVersion.noInstalledHosts'),

  /**
   * @type {boolean}
   */
  noCurrentHosts: function () {
    return this.get('stackVersion') ? this.get('stackVersion.noCurrentHosts') : true;
  }.property('stackVersion.noCurrentHosts'),

  /**
   * @type {boolean}
   */
  noInitHosts: function () {
    return this.get('stackVersion') ? this.get('stackVersion.noInitHosts') : false;
  }.property('stackVersion.noInitHosts'),

  /**
   * @type {string}
   */
  noInitHostsTooltip: Em.computed.ifThenElse('noInitHosts', Em.I18n.t('admin.stackVersions.version.emptyHostsTooltip'), Em.I18n.t('admin.stackVersions.version.hostsTooltip')),

  /**
   * @type {string}
   */
  noCurrentHostsTooltip: Em.computed.ifThenElse('noCurrentHosts', Em.I18n.t('admin.stackVersions.version.emptyHostsTooltip'), Em.I18n.t('admin.stackVersions.version.hostsTooltip')),

  /**
   * @type {string}
   */
  noInstalledHostsTooltip: Em.computed.ifThenElse('noInstalledHosts', Em.I18n.t('admin.stackVersions.version.emptyHostsTooltip'), Em.I18n.t('admin.stackVersions.version.hostsTooltip')),

  /**
   * @type {boolean}
   */
  isVisible: true,

  /**
   * @type {boolean}
   */
  isCurrent: Em.computed.equal('status', 'CURRENT')
});

App.RepositoryVersion.FIXTURES = [];

});

require.register("models/stack_version/service_simple", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceSimple = DS.Model.extend({
  id: DS.attr('string'),
  name: DS.attr('string'),
  displayName: DS.attr('string'),
  latestVersion: DS.attr('string'),
  isAvailable: DS.attr('boolean'),
  isUpgradable: DS.attr('boolean'),
  isHidden: function () {
    var hiddenServices = ['MAPREDUCE2'];
    return hiddenServices.contains(this.get('name')) || this.get('doNotShowAndInstall');
  }.property('name'),

  doNotShowAndInstall: function () {
    var skipServices = ['KERBEROS'];
    if (!App.get('supports.installGanglia')) {
      skipServices.push('GANGLIA');
    }
    return skipServices.contains(this.get('name'));
  }.property('name')
});

App.ServiceSimple.FIXTURES = [];

});

require.register("models/stack_version/stack_upgrade_history", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var stringUtils = require('utils/string_utils');

App.StackUpgradeHistory = DS.Model.extend({
  requestId: DS.attr('number'),
  upgradeId: DS.attr('number'),
  clusterName: DS.attr('string'),
  direction: DS.attr('string'),
  associatedVersion: DS.attr('string'),
  requestStatus: DS.attr('string'),
  upgradeType: DS.attr('string'),
  downgradeAllowed: DS.attr('boolean'),
  skipFailures: DS.attr('boolean'),
  skipServiceCheckFailures: DS.attr('boolean'),
  endTime: DS.attr('number'),
  startTime: DS.attr('number'),
  createTime: DS.attr('number'),
  versions: DS.attr('object'),
  displayStatus: function () {
    return stringUtils.upperUnderscoreToText(this.get('requestStatus'));
  }.property('requestStatus')
});

App.StackUpgradeHistory.FIXTURES = [];

});

require.register("models/stack_version/version", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.StackVersion = DS.Model.extend({
  clusterName: DS.attr('string'),
  stack: DS.attr('string'),
  version: DS.attr('string'),
  name: Em.computed.concat(' ', 'stack', 'version'),
  state: DS.attr('string'),
  repositoryVersion: DS.belongsTo('App.RepositoryVersion'),
  notInstalledHosts: DS.attr('array'),
  installingHosts: DS.attr('array'),
  installedHosts: DS.attr('array'),
  installFailedHosts: DS.attr('array'),
  outOfSyncHosts: DS.attr('array'),
  upgradingHosts: DS.attr('array'),
  upgradedHosts: DS.attr('array'),
  upgradeFailedHosts: DS.attr('array'),
  currentHosts: DS.attr('array'),
  supportsRevert: DS.attr('boolean'),
  revertUpgradeId: DS.attr('number'),

  noInstalledHosts: Em.computed.empty('installedHosts'),

  noCurrentHosts: Em.computed.empty('currentHosts'),

  noInitHosts: Em.computed.empty('notInstalledHosts'),

  isCurrent: Em.computed.equal('state', 'CURRENT'),

  isOutOfSync: Em.computed.equal('state', 'OUT_OF_SYNC')
});

App.StackVersion.FIXTURES = [];

});

require.register("models/target_cluster", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TargetCluster = DS.Model.extend({
  name: DS.attr('string'),
  execute: DS.attr('string'),
  workflow: DS.attr('string'),
  write: DS.attr('string'),
  readonly: DS.attr('string'),
  staging: DS.attr('string'),
  working: DS.attr('string'),
  temp: DS.attr('string')
});

App.TargetCluster.FIXTURES = [];

});

require.register("models/upgrade_entity", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @type {Ember.Object}
 * @class
 */
App.upgradeEntity = Em.Object.extend({

  /**
   * type of entity "GROUP", "ITEM", "TASK"
   * @type {string}
   */
  type: null,

  /**
   * @type {boolean}
   */
  isExpanded: false,

  /**
   * @type {boolean}
   */
  hasExpandableItems: false,

  /**
   * @type {boolean}
   */
  isVisible: Em.computed.notEqual('status', 'PENDING'),

  /**
   * status of tasks/items/groups which should be grayed out and disabled
   * @type {Array}
   */
  nonActiveStates: ['PENDING', 'ABORTED'],

  /**
   * @type {boolean}
   */
  isRunning: Em.computed.existsIn('status', ['IN_PROGRESS']),

  /**
   * @type {boolean}
   */
  isCompleted: Em.computed.existsIn('status', ['COMPLETED', 'FAILED', 'TIMEDOUT', 'ABORTED']),

  /**
   * @type {number}
   */
  progress: function () {
    return Math.floor(this.get('progress_percent'));
  }.property('progress_percent'),

  /**
   * indicate whether entity has active link
   * @type {boolean}
   */
  isActive: function () {
    return !this.get('nonActiveStates').contains(this.get('status'));
  }.property('status'),

  /**
   * indicate whether upgrade group should be expanded
   * @type {boolean}
   */
  isExpandableGroup: function () {
    return this.get('type') === 'GROUP' && (this.get('isActive') || this.get('hasExpandableItems'));
  }.property('isActive', 'hasExpandableItems'),

  upgradeItemStatus: Em.computed.firstNotBlank('display_status', 'status'),

  /**
   * @type {string}
   */
  upgradeGroupStatus: function () {
    if (App.get('upgradeSuspended') && this.get('status') === 'ABORTED') {
      return 'SUSPENDED';
    }
    if (this.get('type') === 'GROUP' && !this.get('isActive') && this.get('hasExpandableItems')) {
      return 'SUBITEM_FAILED';
    }
    return this.get('display_status') || this.get('status');
  }.property('isExpandableGroup', 'display_status', 'status', 'App.upgradeSuspended')
});

});

require.register("models/user", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.User = DS.Model.extend({
  userName: DS.attr('string'),
  id: Em.computed.alias('userName'),
  userType: DS.attr('string'),
  auditItems: DS.hasMany('App.ServiceAudit'),
  admin: DS.attr('boolean'),
  operator: DS.attr('boolean'),
  clusterUser: DS.attr('boolean'),

  /**
   * List of permissions assigned to user
   *  Available permissions:
   *    AMBARI.ADMINISTRATOR
   *    CLUSTER.USER
   *    CLUSTER.ADMINISTRATOR
   *    VIEW.USER
   * @property {Array} permissions
   **/
  permissions: DS.attr('array'),

  /**
   * @type {Boolean}
   */
  isLdap: Em.computed.equal('userType', 'LDAP')
});

App.CreateUserForm = App.Form.extend({
  className: App.User,
  object: Em.computed.alias('App.router.mainAdminUserCreateController.content'),

  fieldsOptions: [{ name: "userName", displayName: "Username", toLowerCase: function () {
      var v = this.get('value');this.set('value', v.toLowerCase());
    }.observes('value') }, { name: "password", displayName: "Password", displayType: "password", isRequired: true }, { name: "passwordRetype", displayName: "Retype Password", displayType: "password", validator: "passwordRetype", isRequired: true }, { name: "admin", displayName: "Admin", displayType: "checkbox", isRequired: false, defaultValue: true }],
  fields: [],

  isValid: function isValid() {
    var isValid = this._super();

    var passField = this.get('field.password');
    var passRetype = this.get('field.passwordRetype');

    if (!validator.empty(passField.get('value'))) {
      if (passField.get('value') != passRetype.get('value')) {
        passRetype.set('errorMessage', this.t('admin.users.createError.passwordValidation'));
        isValid = false;
      }
    }

    if (isValid) {
      var users = App.User.find();
      var userNameField = this.getField('userName');
      var userName = userNameField.get('value');

      if (users.mapProperty('userName').contains(userName)) {
        userNameField.set('errorMessage', this.t('admin.users.createError.userNameExists'));
        return isValid = false;
      }
    }

    return isValid;
  },

  isWarn: function isWarn() {
    var isWarn = false;
    var userNameField = this.getField('userName');
    userNameField.set('warnMessage', '');
    var userName = userNameField.get('value');

    if (this.isValid() && !validator.isValidUserName(userName)) {
      userNameField.set('warnMessage', this.t('users.userName.validationFail'));
      isWarn = true;
    }
    return isWarn;
  },

  save: function save() {

    var object = this.get('object');
    var formValues = {};
    $.each(this.get('fields'), function () {
      formValues[Ember.String.decamelize(this.get('name'))] = this.get('value');
    });

    if (this.get('className')) {
      App.store.safeLoad(this.get('className'), App.dateTime(), formValues);
    }

    this.set('result', 1);

    return true;
  }
});
App.User.FIXTURES = [];

});

require.register("models/view_instance", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ViewInstance = Em.Object.extend({

  /**
   * @type {string}
   */
  iconPath: '',

  /**
   * @type {string}
   */
  label: '',

  /**
   * @type {boolean}
   */
  visible: false,

  /**
   * @type {string}
   */
  version: '',

  /**
   * @type {string}
   */
  description: '',

  /**
   * @type {string}
   */
  viewName: '',

  /**
   * @type {string}
   */
  shortUrl: '',

  /**
   * @type {string}
   */
  instanceName: '',

  /**
   * @type {string}
   */
  href: '',

  /**
   * @type {string}
   */
  internalAmbariUrl: function () {
    var shortUrl = this.get('shortUrl');
    var viewName = this.get('viewName');
    var version = this.get('version');
    var instanceName = this.get('instanceName');
    if (shortUrl) {
      return '#/main/view/' + viewName + '/' + shortUrl;
    }
    return '#/main/views/' + viewName + '/' + version + '/' + instanceName;
  }.property('shortUrl', 'viewName', 'version', 'instanceName')
});

});

require.register("models/widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.Widget = DS.Model.extend({
  widgetName: DS.attr('string'),

  /**
   * types:
   *  - GAUGE (shown as a percentage dial)
   *  - HEATMAP
   *  - GRAPH (Line graph and stack graph)
   *  - NUMBER (e.g., “1 ms” for RPC latency)
   *  - LINKS
   *  - TEMPLATE
   */
  widgetType: DS.attr('string'),
  layout: DS.belongsTo('App.WidgetLayout'),
  description: DS.attr('string'),
  serviceName: DS.attr('string'),
  timeCreated: DS.attr('number'),
  sectionName: DS.attr('string'),
  author: DS.attr('string'),
  scope: DS.attr('string'),
  properties: DS.attr('object'),
  expression: DS.attr('array'),
  metrics: DS.attr('array'),
  values: DS.attr('array'),
  tag: DS.attr('string'),
  isVisible: DS.attr('boolean', { defaultValue: true }),
  /**
   * @type {number}
   * @default 0
   */
  defaultOrder: 0, // This field is not derived from API but needs to be filled in the mapper on the client side

  /**
   * @type Em.View
   * @class
   */
  viewClass: function () {
    switch (this.get('widgetType')) {
      case 'GRAPH':
        return App.GraphWidgetView;
      case 'TEMPLATE':
        return App.TemplateWidgetView;
      case 'NUMBER':
        return App.NumberWidgetView;
      case 'GAUGE':
        return App.GaugeWidgetView;
      case 'HEATMAP':
        return App.HeatmapWidgetView;
      default:
        return Em.View;
    }
  }.property('widgetType')
});

App.Widget.FIXTURES = [];

App.WidgetType = DS.Model.extend({
  name: DS.attr('string'),
  displayName: DS.attr('string'),
  iconPath: DS.attr('string'),
  description: DS.attr('string'),
  properties: DS.attr('array')
});

App.WidgetType.FIXTURES = [{
  id: 'GAUGE',
  name: 'GAUGE',
  icon_path: '/img/widget-gauge.png',
  display_name: 'Gauge',
  description: Em.I18n.t('widget.type.gauge.description'),
  properties: [{
    name: 'threshold',
    isRequired: false,
    smallValue: '0.7',
    bigValue: '0.9'
  }]
}, {
  id: 'NUMBER',
  name: 'NUMBER',
  icon_path: '/img/widget-number.png',
  display_name: 'Number',
  description: Em.I18n.t('widget.type.number.description'),
  properties: [{
    name: 'threshold',
    isRequired: false,
    smallValue: '10',
    bigValue: '20',
    MAX_VALUE: Infinity
  }, {
    name: 'display_unit',
    isRequired: false,
    value: '',
    placeholder: 'Optional: MB, ms, etc.'
  }]
}, {
  id: 'GRAPH',
  name: 'GRAPH',
  display_name: 'Graph',
  icon_path: '/img/widget-graph.png',
  description: Em.I18n.t('widget.type.graph.description'),
  properties: [{
    name: 'graph_type',
    isRequired: true,
    value: 'LINE'
  }, {
    name: 'display_unit',
    isRequired: false,
    value: '',
    placeholder: 'Optional: MB, ms, etc.'
  }]
}, {
  id: 'TEMPLATE',
  name: 'TEMPLATE',
  icon_path: '/img/widget-template.png',
  display_name: 'Template',
  description: Em.I18n.t('widget.type.template.description'),
  properties: []
}];

});

require.register("models/widget_layout", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetLayout = DS.Model.extend({
  layoutName: DS.attr('string'),
  displayName: DS.attr('string'),
  sectionName: DS.attr('string'),
  widgets: DS.hasMany('App.Widget'),
  scope: DS.attr('string'),
  user: DS.attr('string'),

  nameServiceId: function () {
    return this.get('layoutName').split('_nameservice_')[1] || '';
  }.property('layoutName')
});

App.WidgetLayout.FIXTURES = [];

});

require.register("models/widget_property", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.WidgetProperty = Ember.Object.extend({

  /**
   * label to be shown for property
   * @type {String}
   */
  label: '',

  /**
   * PORT|METRIC|AGGREGATE
   * @type {String}
   */
  type: '',

  /**
   * config property value
   * @type {*}
   */
  value: null,

  /**
   * input displayType
   * one of 'textFields', 'textArea', 'select' or 'threshold'
   * @type {String}
   */
  displayType: '',

  /**
   * space separated list of css class names to use
   * @type {String}
   */
  classNames: '',

  /**
   * view class according to <code>displayType</code>
   * @type {Em.View}
   */
  viewClass: function () {
    var displayType = this.get('displayType');
    switch (displayType) {
      case 'textField':
        return App.WidgetPropertyTextFieldView;
      case 'threshold':
        return App.WidgetPropertyThresholdView;
      case 'select':
        return App.WidgetPropertySelectView;
      default:
    }
  }.property('displayType'),

  /**
   * Define whether property is valid
   * Computed property should be defined in child class
   * @type {Boolean}
   */
  isValid: true,

  /**
   * Define whether property is required by user
   * @type {Boolean}
   */
  isRequired: true
});

App.WidgetPropertyTypes = [{
  name: 'display_unit',
  label: 'Unit',
  displayType: 'textField',
  classNames: 'widget-property-unit',
  isValid: function () {
    return this.get('isRequired') ? Boolean(this.get('value')) : true;
  }.property('value'),
  valueMap: {
    "value": "display_unit"
  }
}, {
  name: 'graph_type',
  label: 'Graph Type',
  displayType: 'select',
  options: [{
    label: "LINE",
    value: "LINE"
  }, {
    label: "STACK",
    value: "STACK"
  }],
  valueMap: {
    "value": "graph_type"
  }
}, {
  name: 'time_range',
  label: 'Time Range',
  displayType: 'select',
  options: [{
    label: "Last 1 hour",
    value: "1"
  }, {
    label: "Last 2 hours",
    value: "2"
  }, {
    label: "Last 4 hour",
    value: "4"
  }, {
    label: "Last 12 hour",
    value: "12"
  }, {
    label: "Last 24 hour",
    value: "24"
  }, {
    label: "Last 1 week",
    value: "168"
  }, {
    label: "Last 1 month",
    value: "720"
  }, {
    label: "Last 1 year",
    value: "8760"
  }],
  valueMap: {
    "value": "time_range"
  }
}, {
  name: 'threshold',
  label: 'Thresholds',

  MIN_VALUE: 0,

  MAX_VALUE: 1,

  valueMap: {
    "smallValue": "warning_threshold",
    "bigValue": "error_threshold"
  },

  /**
   * threshold-value
   * @type {string}
   */
  smallValue: '',
  bigValue: '',
  badgeOK: 'OK',
  badgeWarning: 'WARNING',
  badgeCritical: 'CRITICAL',

  displayType: 'threshold',

  classNames: 'widget-property-threshold',

  /**
   * Check if <code>smallValue</code> is valid float number
   * @return {boolean}
   */
  isSmallValueValid: function () {
    return this.validate(this.get('smallValue'));
  }.property('smallValue', 'bigValue'),

  /**
   * Check if <code>bigValue</code> is valid float number
   * @return {boolean}
   */
  isBigValueValid: function () {
    return this.validate(this.get('bigValue'));
  }.property('bigValue', 'smallValue'),

  /**
   * validate
   * @param {string} value
   * @returns {boolean}
   */
  validate: function validate(value) {
    value = Number(('' + value).trim());
    if (!value && !isNaN(value)) {
      return true;
    }
    return validator.isValidFloat(value) && value > this.get('MIN_VALUE') && value <= this.get('MAX_VALUE');
  },

  isValid: Em.computed.and('isSmallValueValid', 'isBigValueValid')
}];

});

require.register("router", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var misc = require('utils/misc');
var App = require('app');

App.WizardRoute = Em.Route.extend({

  gotoStep0: Em.Router.transitionTo('step0'),

  gotoStep1: Em.Router.transitionTo('step1'),

  gotoStep2: Em.Router.transitionTo('step2'),

  gotoStep3: Em.Router.transitionTo('step3'),

  gotoStep4: Em.Router.transitionTo('step4'),

  gotoStep5: Em.Router.transitionTo('step5'),

  gotoStep6: Em.Router.transitionTo('step6'),

  gotoStep7: Em.Router.transitionTo('step7'),

  gotoStep8: Em.Router.transitionTo('step8'),

  gotoStep9: Em.Router.transitionTo('step9'),

  gotoStep10: Em.Router.transitionTo('step10'),

  isRoutable: function () {
    return typeof this.get('route') === 'string' && App.router.get('loggedIn');
  }.property('App.router.loggedIn')

});

/**
 * This route executes "back" and "next" handler on the next run-loop
 * Reason: It's done like this, because in general transitions have highest priority in the Ember run-loop
 * So, CP's and observers for <code>App.router.backBtnClickInProgress</code> and <code>App.router.nextBtnClickInProgress</code>
 * will be triggered after "back" or "next" are complete and not before them.
 * It's more important for "back", because usually it doesn't do any requests that may cause a little delay for run loops
 * <code>Em.run.next</code> is used to avoid this
 *
 * Example:
 * <pre>
 *   App.Step2Route = App.StepRoute.extend({
 *
 *    route: '/step2',
 *
 *    connectOutlets: function (router, context) {
 *      // some code
 *    },
 *
 *    nextTransition: function (router) {
 *      router.transitionTo('step3');
 *    },
 *
 *    backTransition: function (router) {
 *      router.transitionTo('step1');
 *    }
 *
 *   });
 * </pre>
 * In this case both <code>transitionTo</code> will be executed in the next run loop after loop where "next" or "back" were called
 * <b>IMPORTANT!</b> Flags <code>App.router.backBtnClickInProgress</code> and <code>App.router.nextBtnClickInProgress</code> are set to <code>true</code>
 * in the "back" and "next". Be sure to set them <code>false</code> when needed
 *
 * @type {Em.Route}
 */
App.StepRoute = Em.Route.extend({

  /**
   * @type {Function}
   */
  backTransition: Em.K,

  /**
   * @type {Function}
   */
  nextTransition: Em.K,

  /**
   * Default "Back"-action
   * Execute <code>backTransition</code> once
   *
   * @param {Em.Router} router
   */
  back: function back(router) {
    if (App.get('router.btnClickInProgress')) {
      return;
    }
    App.set('router.backBtnClickInProgress', true);
    var self = this;
    Em.run.next(function () {
      Em.tryInvoke(self, 'backTransition', [router]);
    });
  },

  /**
   * Default "Next"-action
   * Execute <code>nextTransition</code> once
   *
   * @param {Em.Router} router
   */
  next: function next(router) {
    if (App.get('router.btnClickInProgress')) {
      return;
    }
    App.set('router.nextBtnClickInProgress', true);
    var self = this;
    Em.run.next(function () {
      Em.tryInvoke(self, 'nextTransition', [router]);
    });
  }

});

App.Router = Em.Router.extend({

  enableLogging: true,
  isFwdNavigation: true,
  backBtnForHigherStep: false,

  /**
   * Checks if Back button is clicked
   * Set to default value on the <code>App.WizardController.connectOutlet</code>
   *
   * @type {boolean}
   * @default false
   */
  backBtnClickInProgress: false,

  /**
   * Checks if Next button is clicked
   * Set to default value on the <code>App.WizardController.connectOutlet</code>
   *
   * @type {boolean}
   * @default false
   */
  nextBtnClickInProgress: false,

  /**
   * Checks if Next or Back button is clicked
   * Used in the <code>App.StepRoute</code>-instances to avoid "next"/"back" double-clicks
   *
   * @type {boolean}
   * @default false
   */
  btnClickInProgress: Em.computed.or('backBtnClickInProgress', 'nextBtnClickInProgress'),

  /**
   * Path for local login page. This page will be always accessible without
   * redirect to auth server different from ambari-server. Used in some types of
   * authorizations like knox sso.
   *
   * @type {string}
   */
  localUserAuthUrl: '/login/local',

  /**
   * LocalStorage property <code>redirectsCount</code> from <code>tmp</code> namespace
   * will be incremented by each redirect action performed by UI and reset on success login.
   * <code>redirectsLimitCount</code> determines maximum redirect tries. When redirects count overflow
   * then something goes wrong and we have to inform user about the problem.
   *
   * @type {number}
   */
  redirectsLimitCount: 0,

  /**
   * Is true, if cluster.provisioning_state is equal to 'INSTALLED'
   * @type {Boolean}
   */
  clusterInstallCompleted: false,
  /**
   * user prefered path to route
   */
  preferedPath: null,

  setNavigationFlow: function setNavigationFlow(step) {
    var matches = step.match(/\d+$/);
    var newStep;
    if (matches) {
      newStep = parseInt(matches[0], 10);
    }
    var previousStep = parseInt(this.getInstallerCurrentStep(), 10);
    this.set('isFwdNavigation', newStep >= previousStep);
  },

  clearAllSteps: function clearAllSteps() {
    this.get('installerController').clear();
    this.get('addHostController').clear();
    this.get('addServiceController').clear();
    this.get('backgroundOperationsController').clear();
    for (var i = 1; i < 11; i++) {
      this.set('wizardStep' + i + 'Controller.hasSubmitted', false);
      this.set('wizardStep' + i + 'Controller.isDisabled', true);
    }
  },

  /**
   * Temporary fix for getting cluster name
   * @return {*}
   */

  getClusterName: function getClusterName() {
    return App.router.get('clusterController').get('clusterName');
  },

  /**
   * Get current step of Installer wizard
   * @return {*}
   */
  getInstallerCurrentStep: function getInstallerCurrentStep() {
    return this.getWizardCurrentStep('installer');
  },

  /**
   * Get current step for <code>wizardType</code> wizard
   * @param wizardType one of <code>installer</code>, <code>addHost</code>, <code>addServices</code>
   */
  getWizardCurrentStep: function getWizardCurrentStep(wizardType) {
    var currentStep = App.db.getWizardCurrentStep(wizardType);
    if (!currentStep) {
      currentStep = wizardType === 'installer' ? '0' : '1';
    }
    return currentStep;
  },

  /**
   * @type {boolean}
   */
  loggedIn: App.db.getAuthenticated(),

  loginName: function () {
    return this.getLoginName();
  }.property('loggedIn'),

  displayLoginName: Em.computed.truncate('loginName', 10, 10),

  /**
   * @type {$.ajax|null}
   */
  clusterDataRequest: null,

  /**
   * If request was already sent on login then use saved clusterDataRequest and don't make second call
   * @returns {$.ajax}
   */
  getClusterDataRequest: function getClusterDataRequest() {
    var clusterDataRequest = this.get('clusterDataRequest');
    if (clusterDataRequest) {
      this.set('clusterDataRequest', null);
      return clusterDataRequest;
    } else {
      return App.ajax.send({
        name: 'router.login.clusters',
        sender: this,
        success: 'onAuthenticationSuccess',
        error: 'onAuthenticationError'
      });
    }
  },

  getAuthenticated: function getAuthenticated() {
    var dfd = $.Deferred();
    var self = this;
    var auth = App.db.getAuthenticated();
    this.getClusterDataRequest().complete(function (xhr) {
      if (xhr.state() === 'resolved') {
        // if server knows the user and user authenticated by UI
        if (auth) {
          dfd.resolve(self.get('loggedIn'));
          // if server knows the user but UI don't, check the response header
          // and try to authorize
        } else if (xhr.getResponseHeader('User')) {
          var user = xhr.getResponseHeader('User');
          App.ajax.send({
            name: 'router.afterLogin',
            sender: self,
            data: {
              usr: user,
              loginName: encodeURIComponent(user)
            },
            success: 'afterLoginSuccessCallback',
            error: 'loginErrorCallback'
          }).then(function () {
            dfd.resolve(true);
          });
        } else {
          self.setAuthenticated(false);
          dfd.resolve(false);
        }
      } else {
        //if provisioning state unreachable then consider user as unauthenticated
        self.setAuthenticated(false);
        dfd.resolve(false);
      }
    });
    return dfd.promise();
  },

  /**
   * Response for <code>/clusters?fields=Clusters/provisioning_state</code>
   * @type {null|object}
   */
  clusterData: null,

  onAuthenticationSuccess: function onAuthenticationSuccess(data) {
    if (App.db.getAuthenticated() === true) {
      this.set('clusterData', data);
      this.setAuthenticated(true);
      if (data.items.length) {
        this.setClusterInstalled(data);
      }
    }
  },

  /**
    * If authentication failed, need to check for jwt auth url
   * and redirect user if current location is not <code>localUserAuthUrl</code>
   *
   * @param {?object} data
   */
  onAuthenticationError: function onAuthenticationError(data) {
    if (data.status === 403 || data.status === 401) {
      try {
        var responseJson = JSON.parse(data.responseText);
        if (responseJson.jwtProviderUrl && this.get('location.lastSetURL') !== this.get('localUserAuthUrl')) {
          this.redirectByURL(responseJson.jwtProviderUrl + encodeURIComponent(this.getCurrentLocationUrl()));
        }
      } catch (e) {} finally {
        this.setAuthenticated(false);
      }
    } else if (data.status >= 500) {
      this.setAuthenticated(false);
      this.loginErrorCallback(data);
    }
  },

  setAuthenticated: function setAuthenticated(authenticated) {
    App.db.setAuthenticated(authenticated);
    this.set('loggedIn', authenticated);
  },

  getLoginName: function getLoginName() {
    return App.db.getLoginName();
  },

  setLoginName: function setLoginName(loginName) {
    App.db.setLoginName(loginName);
  },

  /**
   * Set user model to local storage
   * @param user
   */
  setUser: function setUser(user) {
    App.db.setUser(user);
  },

  /**
   * Get user model from local storage
   * @return {*}
   */
  getUser: function getUser() {
    return App.db.getUser();
  },

  setUserLoggedIn: function setUserLoggedIn(userName) {
    this.setAuthenticated(true);
    this.setLoginName(userName);
    this.setUser(App.User.find().findProperty('id', userName));
    App.db.set('tmp', 'redirectsCount', 0);
  },

  /**
   * Set `clusterInstallCompleted` property based on cluster info response.
   *
   * @param {Object} clusterObject
   **/
  setClusterInstalled: function setClusterInstalled(clusterObject) {
    this.set('clusterInstallCompleted', clusterObject.items[0].Clusters.provisioning_state === 'INSTALLED');
  },

  login: function login() {
    var controller = this.get('loginController');
    var loginName = controller.get('loginName');
    controller.set('loginName', loginName);
    var hash = misc.utf8ToB64(loginName + ":" + controller.get('password'));
    var usr = '';

    if (App.get('testMode')) {
      if (loginName === "admin" && controller.get('password') === 'admin') {
        usr = 'admin';
      } else if (loginName === 'user' && controller.get('password') === 'user') {
        usr = 'user';
      }
    }

    App.ajax.send({
      name: 'router.login',
      sender: this,
      data: {
        auth: "Basic " + hash,
        usr: usr,
        loginName: encodeURIComponent(loginName)
      },
      beforeSend: 'authBeforeSend',
      success: 'loginSuccessCallback',
      error: 'loginErrorCallback'
    });
  },

  authBeforeSend: function authBeforeSend(opt, xhr, data) {
    xhr.setRequestHeader("Authorization", data.auth);
  },

  loginSuccessCallback: function loginSuccessCallback(data, opt, params) {
    App.ajax.send({
      name: 'router.afterLogin',
      sender: this,
      data: {
        usr: params.usr,
        loginName: params.loginName
      },
      success: 'afterLoginSuccessCallback',
      error: 'loginErrorCallback'
    });
  },

  afterLoginSuccessCallback: function afterLoginSuccessCallback(data, opt, params) {
    var self = this;
    App.usersMapper.map({ "items": [data] });
    this.setUserLoggedIn(data.Users.user_name);
    var requestData = {
      loginName: data.Users.user_name,
      loginData: data
    };
    App.router.get('clusterController').loadAuthorizations().complete(function () {
      App.ajax.send({
        name: 'router.login.message',
        sender: self,
        data: requestData,
        success: 'showLoginMessageSuccessCallback',
        error: 'showLoginMessageErrorCallback'
      });
    });
  },

  loginErrorCallback: function loginErrorCallback(request) {
    var controller = this.get('loginController');
    this.setAuthenticated(false);
    if (request.status > 400) {
      var responseMessage = request.responseText;
      try {
        responseMessage = JSON.parse(request.responseText).message;
      } catch (e) {}
    }
    if (request.status == 403) {
      controller.postLogin(true, false, responseMessage);
    } else if (request.status == 500) {
      controller.postLogin(false, false, responseMessage);
    } else {
      controller.postLogin(false, false, null);
    }
  },

  /**
   * success callback of router.login.message
   * @param {object} data
   * @param {object} opt
   * @param {object} params
   */
  showLoginMessageSuccessCallback: function showLoginMessageSuccessCallback(data, opt, params) {
    try {
      var response = JSON.parse(data.Settings.content.replace(/\n/g, "\\n"));
    } catch (e) {
      this.setClusterData(data, opt, params);
      return false;
    }

    var text = response.text ? response.text.replace(/(\r\n|\n|\r)/gm, '<br>') : "",
        buttonText = response.button ? response.button : Em.I18n.t('ok'),
        status = response.status && response.status == "true" ? true : false,
        self = this;

    if (text && status) {
      return App.ModalPopup.show({
        classNames: ['common-modal-wrapper'],
        modalDialogClasses: ['modal-lg'],
        header: Em.I18n.t('login.message.title'),
        bodyClass: Ember.View.extend({
          template: Ember.Handlebars.compile(text)
        }),
        primary: null,
        secondary: null,
        footerClass: Ember.View.extend({
          template: Ember.Handlebars.compile('<div class="modal-footer">' + '<button class="btn btn-success" {{action onPrimary target="view"}}>' + buttonText + '</button>' + '</div>'),
          onPrimary: function onPrimary() {
            this.get('parentView').onPrimary();
          }
        }),

        onPrimary: function onPrimary() {
          self.setClusterData(data, opt, params);
          this.hide();
        },
        onClose: function onClose() {
          self.setClusterData(data, opt, params);
          this.hide();
        }
      });
    }
    this.setClusterData(data, opt, params);
    return false;
  },

  /**
   * error callback of router.login.message
   * @param {object} request
   * @param {string} ajaxOptions
   * @param {string} error
   * @param {object} opt
   * @param {object} params
   */
  showLoginMessageErrorCallback: function showLoginMessageErrorCallback(request, ajaxOptions, error, opt, params) {
    this.showLoginMessageSuccessCallback(null, opt, params);
  },

  setClusterData: function setClusterData(data, opt, params) {
    var self = this,
        requestData = {
      loginName: params.loginName,
      loginData: params.loginData
    };
    // no need to load cluster data if it's already loaded
    if (this.get('clusterData')) {
      this.loginGetClustersSuccessCallback(self.get('clusterData'), {}, requestData);
    } else {
      this.set('clusterDataRequest', App.ajax.send({
        name: 'router.login.clusters',
        sender: self,
        data: requestData,
        success: 'loginGetClustersSuccessCallback'
      }));
    }
  },

  /**
   * success callback of login request
   * @param {object} clustersData
   * @param {object} opt
   * @param {object} params
   */
  loginGetClustersSuccessCallback: function loginGetClustersSuccessCallback(clustersData, opt, params) {
    var privileges = params.loginData.privileges || [];
    var router = this;
    var isAdmin = privileges.mapProperty('PrivilegeInfo.permission_name').contains('AMBARI.ADMINISTRATOR');

    App.set('isAdmin', isAdmin);

    if (clustersData.items.length) {
      var clusterPermissions = privileges.filterProperty('PrivilegeInfo.cluster_name', clustersData.items[0].Clusters.cluster_name).mapProperty('PrivilegeInfo.permission_name');

      //cluster installed
      router.setClusterInstalled(clustersData);
      if (clusterPermissions.contains('CLUSTER.ADMINISTRATOR')) {
        App.setProperties({
          isAdmin: true,
          isOperator: true,
          isClusterUser: false
        });
      }
      if (App.get('isOnlyViewUser')) {
        router.transitionToViews();
      } else {
        router.transitionToApp();
      }
    } else {
      if (App.get('isOnlyViewUser')) {
        router.transitionToViews();
      } else {
        router.transitionToAdminView();
      }
    }
    // set cluster name and security type
    App.router.get('clusterController').reloadSuccessCallback(clustersData);
    App.set('isPermissionDataLoaded', true);
    App.router.get('loginController').postLogin(true, true);
  },

  /**
   * redirect user to Admin View
   * @returns {$.ajax}
   */
  transitionToAdminView: function transitionToAdminView() {
    return App.ajax.send({
      name: 'ambari.service.load_server_version',
      sender: this,
      success: 'adminViewInfoSuccessCallback',
      error: 'adminViewInfoErrorCallback'
    });
  },

  /**
   * redirect user to application Dashboard
   */
  transitionToApp: function transitionToApp() {
    var router = this;
    if (!router.restorePreferedPath()) {
      router.getSection(function (route) {
        router.transitionTo(route);
      });
    }
  },

  /**
   * redirect user to application Views
   */
  transitionToViews: function transitionToViews() {
    App.router.get('mainViewsController').loadAmbariViews();
    this.transitionTo('main.views.index');
  },

  adminViewInfoSuccessCallback: function adminViewInfoSuccessCallback(data) {
    var components = Em.get(data, 'components');
    if (Em.isArray(components)) {
      var mappedVersions = components.map(function (component) {
        if (Em.get(component, 'RootServiceComponents.component_version')) {
          return Em.get(component, 'RootServiceComponents.component_version');
        }
      }),
          sortedMappedVersions = mappedVersions.sort(),
          latestVersion = sortedMappedVersions[sortedMappedVersions.length - 1].replace(/[^\d.-]/g, '');
      window.location.replace(App.appURLRoot + 'views/ADMIN_VIEW/' + latestVersion + '/INSTANCE/#/');
    }
  },

  adminViewInfoErrorCallback: function adminViewInfoErrorCallback() {
    this.transitionToViews();
  },

  getSection: function getSection(callback) {
    if (App.get('testMode')) {
      if (App.alwaysGoToInstaller) {
        callback('installer');
      } else {
        callback('main.dashboard.index');
      }
    } else {
      if (this.get('clusterInstallCompleted')) {
        App.router.get('wizardWatcherController').getUser().complete(function () {
          App.clusterStatus.updateFromServer(false).complete(function () {
            var route = 'main.dashboard.index';
            var clusterStatusOnServer = App.clusterStatus.get('value');
            if (clusterStatusOnServer) {
              var wizardControllerRoutes = require('data/controller_route');
              var wizardControllerRoute = wizardControllerRoutes.findProperty('wizardControllerName', clusterStatusOnServer.wizardControllerName);
              if (wizardControllerRoute && !App.router.get('wizardWatcherController').get('isNonWizardUser')) {
                route = wizardControllerRoute.route;
              }
            }
            if (wizardControllerRoute && wizardControllerRoute.wizardControllerName === 'mainAdminStackAndUpgradeController') {
              var clusterController = App.router.get('clusterController');
              clusterController.loadClusterName().done(function () {
                clusterController.restoreUpgradeState().done(function () {
                  callback(route);
                });
              });
            } else {
              callback(route);
            }
          });
        });
      } else {
        callback('installer');
      }
    }
  },

  logOff: function logOff(context) {
    var self = this;

    $('title').text(Em.I18n.t('app.name'));
    App.router.get('mainController').stopPolling();
    // App.db.cleanUp() must be called before router.clearAllSteps().
    // otherwise, this.set('installerController.currentStep, 0) would have no effect
    // since it's a computed property but we are not setting it as a dependent of App.db.
    App.db.cleanUp();
    App.setProperties({
      isAdmin: false,
      auth: null,
      isOperator: false,
      isClusterUser: false,
      isPermissionDataLoaded: false
    });
    this.set('loggedIn', false);
    this.clearAllSteps();
    this.set('loginController.loginName', '');
    this.set('loginController.password', '');
    // When logOff is called by Sign Out button, context contains event object. As it is only case we should send logoff request, we are checking context below.
    if (!App.get('testMode') && context) {
      App.ajax.send({
        name: 'router.logoff',
        sender: this,
        success: 'logOffSuccessCallback',
        error: 'logOffErrorCallback',
        beforeSend: 'logOffBeforeSend'
      }).complete(function () {
        self.logoffRedirect(context);
      });
    } else {
      this.logoffRedirect();
    }
  },

  logOffSuccessCallback: function logOffSuccessCallback() {
    var applicationController = App.router.get('applicationController');
    applicationController.set('isPollerRunning', false);
  },

  logOffErrorCallback: function logOffErrorCallback() {},

  logOffBeforeSend: function logOffBeforeSend(opt, xhr) {
    xhr.setRequestHeader('Authorization', '');
  },

  /**
   * Redirect function on sign off request.
   *
   * @param {$.Event} [context=undefined] - triggered event context
   */
  logoffRedirect: function logoffRedirect(context) {
    this.transitionTo('login', context);
    if (App.router.get('clusterController.isLoaded')) {
      Em.run.next(function () {
        window.location.reload();
      });
    }
  },

  /**
   * save prefered path
   * @param {string} path
   * @param {string} key
   */
  savePreferedPath: function savePreferedPath(path, key) {
    if (key) {
      if (path.contains(key)) {
        this.set('preferedPath', path.slice(path.indexOf(key) + key.length));
      }
    } else {
      this.set('preferedPath', path);
    }
  },

  /**
   * If path exist route to it, otherwise return false
   * @returns {boolean}
   */
  restorePreferedPath: function restorePreferedPath() {
    var preferredPath = this.get('preferedPath');
    var isRestored = false;

    if (preferredPath) {
      // If the preferred path is relative, allow a redirect to it.
      // If the path is not relative, silently ignore it - if the path is an absolute URL, the user
      // may be routed to a different server where the possibility exists for a phishing attack.
      if ((preferredPath.startsWith('/') || preferredPath.startsWith('#')) && !preferredPath.contains('#/login')) {
        window.location = preferredPath;
        isRestored = true;
      }
      // Unset preferedPath
      this.set('preferedPath', null);
    }

    return isRestored;
  },

  /**
   * initialize isAdmin if user is administrator
   */
  initAdmin: function initAdmin() {
    if (App.db) {
      var user = App.db.getUser();
      if (user) {
        if (user.admin) {
          App.set('isAdmin', true);
        }
        if (user.operator) {
          App.set('isOperator', true);
        }
        if (user.cluster_user) {
          App.set('isClusterUser', true);
        }
        App.set('isPermissionDataLoaded', true);
      }
    }
  },

  /**
   * initialize Auth for user
   */
  initAuth: function initAuth() {
    if (App.db) {
      var auth = App.db.getAuth();
      if (auth) {
        App.set('auth', auth);
      }
    }
  },

  /**
   * Increment redirect count if <code>redirected</code> parameter passed.
   */
  handleUIRedirect: function handleUIRedirect() {
    if (/(\?|&)redirected=/.test(location.hash)) {
      var redirectsCount = App.db.get('tmp', 'redirectsCount') || 0;
      App.db.set('tmp', 'redirectsCount', ++redirectsCount);
    }
  },

  /**
   * <code>window.location</code> setter. Will add query param which determines that we redirect user
   * @param {string} url - url to navigate
   */
  redirectByURL: function redirectByURL(url) {
    var suffix = "?redirected=true";
    var redirectsCount = App.db.get('tmp', 'redirectsCount') || 0;
    if (redirectsCount > this.get('redirectsLimitCount')) {
      this.showRedirectIssue();
      return;
    }
    // skip adding redirected parameter if added
    if (/(\?|&)redirected=/.test(location.hash)) {
      this.setLocationUrl(url);
      return;
    }
    // detect if query params were assigned and replace "?" with "&" for suffix param
    if (/\?\w+=/.test(location.hash)) {
      suffix = suffix.replace('?', '&');
    }
    this.setLocationUrl(url + suffix);
  },

  /**
   * Convenient method to set <code>window.location</code>.
   * Useful for faking url manipulation in tests.
   *
   * @param {string} url
   */
  setLocationUrl: function setLocationUrl(url) {
    window.location = url;
  },

  /**
   * Convenient method to get current <code>window.location</code>.
   * Useful for faking url manipulation in tests.
   */
  getCurrentLocationUrl: function getCurrentLocationUrl() {
    return window.location.href;
  },

  /**
   * Inform user about redirect issue in modal popup.
   *
   * @returns {App.ModalPopup}
   */
  showRedirectIssue: function showRedirectIssue() {
    var bodyMessage = Em.I18n.t('app.redirectIssuePopup.body').format(location.origin + '/#' + this.get('localUserAuthUrl'));
    var popupHeader = Em.I18n.t('app.redirectIssuePopup.header');
    var popup = App.showAlertPopup(popupHeader, bodyMessage);
    popup.set('encodeBody', false);
    return popup;
  },

  root: Em.Route.extend({
    index: Em.Route.extend({
      route: '/',
      redirectsTo: 'login'
    }),

    enter: function enter(router) {
      router.initAdmin();
      router.initAuth();
      router.handleUIRedirect();
    },

    login: Em.Route.extend({
      route: '/login:suffix',

      /**
       *  If the user is already logged in, redirect to where the user was previously
       */
      enter: function enter(router, context) {
        if ($.mocho) {
          return;
        }
        var location = router.location.location.hash;
        router.getAuthenticated().done(function (loggedIn) {
          if (loggedIn) {
            Ember.run.next(function () {
              router.getSection(function (route) {
                router.transitionTo(route, context);
              });
            });
          } else {
            //key to parse URI for prefered path to route
            router.savePreferedPath(location, '?targetURI=');
          }
        });
      },

      connectOutlets: function connectOutlets(router, context) {
        $('title').text(Em.I18n.t('app.name'));
        router.get('applicationController').connectOutlet('login');
      },

      serialize: function serialize(router, context) {
        // check for login/local hash
        var location = router.get('location.location.hash');
        return {
          suffix: location === '#' + router.get('localUserAuthUrl') ? '/local' : ''
        };
      }
    }),

    installer: require('routes/installer'),

    main: require('routes/main'),

    adminView: Em.Route.extend({
      route: '/adminView',
      enter: function enter(router) {
        if (!router.get('loggedIn') || !App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK')) {
          Em.run.next(function () {
            router.transitionTo('login');
          });
        } else {
          App.ajax.send({
            name: 'ambari.service.load_server_version',
            sender: router,
            success: 'adminViewInfoSuccessCallback'
          });
        }
      }
    }),

    experimental: Em.Route.extend({
      route: '/experimental',
      enter: function enter(router, context) {
        if (!App.isAuthorized('AMBARI.MANAGE_SETTINGS')) {
          if (App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK')) {
            Em.run.next(function () {
              if (router.get('clusterInstallCompleted')) {
                router.transitionTo("main.dashboard.widgets");
              } else {
                router.transitionTo("main.views.index");
              }
            });
          } else {
            Em.run.next(function () {
              router.transitionTo("main.views.index");
            });
          }
        }
      },
      connectOutlets: function connectOutlets(router, context) {
        if (App.isAuthorized('AMBARI.MANAGE_SETTINGS')) {
          App.router.get('experimentalController').loadSupports().complete(function () {
            $('title').text(Em.I18n.t('app.name.subtitle.experimental'));
            router.get('applicationController').connectOutlet('experimental');
          });
        }
      }
    }),

    logoff: function logoff(router, context) {
      router.logOff(context);
    }

  })
});

});

require.register("routes/activate_hawq_standby_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/Hawq/activate',

  breadcrumbs: {
    label: Em.I18n.t('admin.activateHawqStandby.wizard.header')
  },

  enter: function enter(router, transition) {
    var activateHawqStandbyWizardController = router.get('activateHawqStandbyWizardController');
    activateHawqStandbyWizardController.dataLoading().done(function () {
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'HAWQ'));
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.activateHawqStandby.wizard.header'),
        bodyClass: App.ActivateHawqStandbyWizardView.extend({
          controller: activateHawqStandbyWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,
        updateClusterStatus: function updateClusterStatus() {
          var controller = router.get('activateHawqStandbyWizardController');
          router.get('updateController').set('isWorking', true);
          activateHawqStandbyWizardController.finish();
          App.clusterStatus.setClusterStatus({
            clusterName: App.router.getClusterName(),
            clusterState: 'DEFAULT',
            localdb: App.db.data
          }, {
            alwaysCallback: function alwaysCallback() {
              controller.get('popup').hide();
              router.transitionTo('main.services.index');
              Em.run.next(function () {
                location.reload();
              });
            }
          });
        },

        onClose: function onClose() {
          var activateHawqStandbyWizardController = router.get('activateHawqStandbyWizardController'),
              currStep = activateHawqStandbyWizardController.get('currentStep');
          if (parseInt(currStep) === 3) {
            App.showConfirmationPopup(function () {
              popup.updateClusterStatus();
            }, Em.I18n.t('admin.activateHawqStandby.closePopup'));
          } else {
            popup.updateClusterStatus();
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      activateHawqStandbyWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'ACTIVATE_HAWQ_STANDBY':
            activateHawqStandbyWizardController.setCurrentStep(currentClusterStatus.localdb.ActivateHawqStandbyWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('activateHawqStandbyWizardController.currentStep');
            activateHawqStandbyWizardController.setCurrentStep(currStep);
            break;
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(activateHawqStandbyWizardController.get('name'));
        router.transitionTo('step' + activateHawqStandbyWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('activateHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.connectOutlet('activateHawqStandbyWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('activateHawqStandbyWizardController');
      var hawqMaster = App.HostComponent.find().findProperty('componentName', 'HAWQMASTER').get('hostName');
      var hawqStandby = App.HostComponent.find().findProperty('componentName', 'HAWQSTANDBY').get('hostName');
      var hawqHosts = {
        hawqMaster: hawqMaster,
        hawqStandby: hawqStandby
      };
      controller.saveHawqHosts(hawqHosts);
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('activateHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('activateHawqStandbyWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      App.showConfirmationPopup(function () {
        var wizardController = router.get('activateHawqStandbyWizardController');
        var stepController = router.get('activateHawqStandbyWizardStep2Controller');
        var configs = stepController.get('selectedService.configs');
        wizardController.saveConfigs(configs);
        router.transitionTo('step3');
      }, Em.I18n.t('admin.activateHawqStandby.wizard.step2.confirmPopup.body'));
    },
    back: Em.Router.transitionTo('step1')
  }),
  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('activateHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.setLowerStepsDisable(3);
        controller.loadAllPriorSteps();
        controller.connectOutlet('activateHawqStandbyWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath(router, path) {
      // allow user to leave route if wizard has finished
      if (router.get('activateHawqStandbyWizardController').get('isFinished')) {
        this._super(router, path);
      } else {
        return false;
      }
    },
    next: function next(router) {
      var controller = router.get('activateHawqStandbyWizardController');
      controller.finish();
      App.clusterStatus.setClusterStatus({
        clusterName: controller.get('content.cluster.name'),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, {
        alwaysCallback: function alwaysCallback() {
          controller.get('popup').hide();
          router.transitionTo('main.services.index');
          Em.run.next(function () {
            location.reload();
          });
        }
      });
    }
  })

});

});

require.register("routes/add_alert_definition_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({

  route: '/alerts/add',

  enter: function enter(router) {
    if (App.isAuthorized('SERVICE.TOGGLE_ALERTS')) {
      Em.run.next(function () {
        var addAlertDefinitionController = router.get('addAlertDefinitionController');
        App.router.get('updateController').set('isWorking', false);
        var popup = App.ModalPopup.show({
          classNames: ['wizard-modal-wrapper'],
          modalDialogClasses: ['modal-xlg'],
          header: Em.I18n.t('alerts.add.header'),
          bodyClass: App.AddAlertDefinitionView.extend({
            controllerBinding: 'App.router.addAlertDefinitionController'
          }),
          primary: Em.I18n.t('form.cancel'),
          showFooter: false,
          secondary: null,

          onPrimary: function onPrimary() {
            this.hide();
            App.router.transitionTo('main.alerts.index');
          },
          onClose: function onClose() {
            this.set('showCloseButton', false); // prevent user to click "Close" many times
            App.router.get('updateController').set('isWorking', true);
            router.transitionTo('main.alerts.index');
            this.hide();
          },
          didInsertElement: function didInsertElement() {
            this._super();
            this.fitHeight();
          }
        });
        addAlertDefinitionController.set('popup', popup);
        router.transitionTo('step' + addAlertDefinitionController.get('currentStep'));
      });
    } else {
      Em.run.next(function () {
        App.router.transitionTo('main.alerts');
      });
    }
  },

  step1: Em.Route.extend({

    route: '/step1',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.setCurrentStep('1');
      controller.set('content', controller.getDBProperty('content'));
      controller.connectOutlet('addAlertDefinitionStep1', controller.get('content'));
    },

    next: function next(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.setDBProperty('content', controller.get('content'));
      router.transitionTo('step2');
    }

  }),

  step2: Em.Route.extend({

    route: '/step2',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.setCurrentStep('2');
      controller.set('content', controller.getDBProperty('content'));
      controller.connectOutlet('addAlertDefinitionStep2', controller.get('content'));
    },

    back: Em.Router.transitionTo('step1'),

    next: function next(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.set('content.configs', App.router.get('mainAlertDefinitionConfigsController.configs'));
      var newDefinitionData = App.router.get('mainAlertDefinitionConfigsController').getPropertiesToUpdate(false);
      newDefinitionData['AlertDefinition/source'].type = controller.get('content.selectedType');
      newDefinitionData['AlertDefinition/label'] = newDefinitionData['AlertDefinition/name'];
      newDefinitionData['AlertDefinition/name'] = newDefinitionData['AlertDefinition/name'].toLowerCase().replace(/\s+/g, '_');
      controller.set('content.formattedToRequestConfigs', newDefinitionData);
      controller.setDBProperty('content', controller.get('content'));
      router.transitionTo('step3');
    }

  }),

  step3: Em.Route.extend({

    route: '/step3',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.setCurrentStep('3');
      controller.set('content', controller.getDBProperty('content'));
      controller.connectOutlet('addAlertDefinitionStep3', controller.get('content'));
    },

    back: Em.Router.transitionTo('step2'),

    done: function done(router) {
      var controller = router.get('addAlertDefinitionController');
      controller.createNewAlertDefinition(Em.get(controller.getDBProperty('content'), 'formattedToRequestConfigs')).done(function () {
        controller.get('popup').hide();
        controller.setDBProperty('content', {});
        controller.finish();
        App.clusterStatus.setClusterStatus({
          clusterName: controller.get('content.cluster.name'),
          clusterState: 'DEFAULT',
          localdb: App.db.data
        }, {
          alwaysCallback: function alwaysCallback() {
            controller.get('popup').hide();
            router.transitionTo('main.alerts');
            Em.run.next(function () {
              location.reload();
            });
          }
        });
      });
    }
  })

});

});

require.register("routes/add_hawq_standby_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/Hawq/add',

  breadcrumbs: {
    label: Em.I18n.t('admin.addHawqStandby.wizard.header')
  },

  enter: function enter(router, transition) {
    var addHawqStandbyWizardController = router.get('addHawqStandbyWizardController');
    addHawqStandbyWizardController.dataLoading().done(function () {
      //Set HAWQ as current service
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'HAWQ'));
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.addHawqStandby.wizard.header'),
        bodyClass: App.AddHawqStandbyWizardView.extend({
          controller: addHawqStandbyWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,
        updateClusterStatus: function updateClusterStatus() {
          var controller = router.get('addHawqStandbyWizardController');
          router.get('updateController').set('isWorking', true);
          addHawqStandbyWizardController.finish();
          App.clusterStatus.setClusterStatus({
            clusterName: App.router.getClusterName(),
            clusterState: 'DEFAULT',
            localdb: App.db.data
          }, {
            alwaysCallback: function alwaysCallback() {
              controller.get('popup').hide();
              router.transitionTo('main.services.index');
              Em.run.next(function () {
                location.reload();
              });
            }
          });
        },

        onClose: function onClose() {
          var addHawqStandbyWizardController = router.get('addHawqStandbyWizardController'),
              currStep = addHawqStandbyWizardController.get('currentStep'),
              self = this;

          if (parseInt(currStep) === 4) {
            App.showConfirmationPopup(function () {
              popup.updateClusterStatus();
            }, Em.I18n.t('admin.addHawqStandby.closePopup'));
          } else {
            popup.updateClusterStatus();
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      addHawqStandbyWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      switch (currentClusterStatus.clusterState) {
        case 'ADD_HAWQ_STANDBY':
          addHawqStandbyWizardController.setCurrentStep(currentClusterStatus.localdb.AddHawqStandbyWizard.currentStep);
          break;
        default:
          var currStep = App.router.get('addHawqStandbyWizardController.currentStep');
          addHawqStandbyWizardController.setCurrentStep(currStep);
          break;
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(addHawqStandbyWizardController.get('name'));
        router.transitionTo('step' + addHawqStandbyWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.connectOutlet('addHawqStandbyWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.setDBProperty('hawqHosts', undefined);
      controller.clearMasterComponentHosts();
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('addHawqStandbyWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('addHawqStandbyWizardController');
      var stepController = router.get('addHawqStandbyWizardStep2Controller');
      var hawqHost = {
        hawqMaster: stepController.get('servicesMasters').filterProperty('component_name', 'HAWQMASTER').findProperty('isInstalled', true).get('selectedHost'),
        newHawqStandby: stepController.get('servicesMasters').filterProperty('component_name', 'HAWQSTANDBY').findProperty('isInstalled', false).get('selectedHost')
      };
      controller.saveHawqHosts(hawqHost);
      controller.saveMasterComponentHosts(stepController);
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.loadAllPriorSteps();
        controller.connectOutlet('addHawqStandbyWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('addHawqStandbyWizardController');
      var stepController = router.get('addHawqStandbyWizardStep3Controller');
      var configs = stepController.get('selectedService.configs');
      controller.saveConfigs(configs);
      router.transitionTo('step4');
    },
    back: Em.Router.transitionTo('step2')
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps();
        controller.connectOutlet('addHawqStandbyWizardStep4', controller.get('content'));
      });
    },
    unroutePath: function unroutePath(router, path) {
      // allow user to leave route if wizard has finished
      if (router.get('addHawqStandbyWizardController').get('isFinished')) {
        this._super(router, path);
      } else {
        return false;
      }
    },
    next: function next(router) {
      var controller = router.get('addHawqStandbyWizardController');
      controller.finish();
      App.clusterStatus.setClusterStatus({
        clusterName: controller.get('content.cluster.name'),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, {
        alwaysCallback: function alwaysCallback() {
          controller.get('popup').hide();
          router.transitionTo('main.services.index');
          Em.run.next(function () {
            location.reload();
          });
        }
      });
    }
  })

});

});

require.register("routes/add_host_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/host/add',

  leaveWizard: function leaveWizard(router, context) {
    var addHostController = router.get('addHostController');
    addHostController.resetOnClose(addHostController, 'hosts.index', router);
  },

  enter: function enter(router) {
    var self = this;
    router.get('mainController').dataLoading().done(function () {
      Ember.run.next(function () {
        var addHostController = router.get('addHostController');
        App.router.get('updateController').set('isWorking', false);
        var popup = App.ModalPopup.show({
          classNames: ['wizard-modal-wrapper'],
          modalDialogClasses: ['modal-xlg'],
          header: Em.I18n.t('hosts.add.header'),
          bodyClass: App.AddHostView.extend({
            controllerBinding: 'App.router.addHostController'
          }),
          primary: Em.I18n.t('form.cancel'),
          secondary: null,
          showFooter: false,

          onPrimary: function onPrimary() {
            this.hide();
            App.router.get('updateController').set('isWorking', true);
            router.transitionTo('hosts.index');
          },
          onClose: function onClose() {
            var popupContext = this;
            router.set('nextBtnClickInProgress', true);
            if (addHostController.get('currentStep') == '6') {
              App.ModalPopup.show({
                header: Em.I18n.t('hosts.add.exit.header'),
                body: Em.I18n.t('hosts.add.exit.body'),
                onPrimary: function onPrimary() {
                  self.leaveWizard(router, popupContext);
                }
              });
            } else {
              self.leaveWizard(router, this);
            }
          },
          didInsertElement: function didInsertElement() {
            this._super();
            this.fitHeight();
          }
        });
        var currentClusterStatus = App.clusterStatus.get('value');
        if (currentClusterStatus) {
          switch (currentClusterStatus.clusterState) {
            case 'ADD_HOSTS_DEPLOY_PREP_2':
              addHostController.setCurrentStep('4');
              break;
            case 'ADD_HOSTS_INSTALLING_3':
            case 'SERVICE_STARTING_3':
              addHostController.setCurrentStep('5');
              break;
            case 'ADD_HOSTS_INSTALLED_4':
              addHostController.setCurrentStep('6');
              break;
            default:
              break;
          }
        }

        addHostController.set('popup', popup);
        App.router.get('wizardWatcherController').setUser(addHostController.get('name'));
        router.transitionTo('step' + addHostController.get('currentStep'));
      });
    });
  },

  step1: App.StepRoute.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('1');
      controller.set('hideBackButton', true);
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        var wizardStep2Controller = router.get('wizardStep2Controller');
        wizardStep2Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep2', controller.get('content'));
      });
    },

    nextTransition: function nextTransition(router) {
      var controller = router.get('addHostController');
      controller.save('installOptions');
      //hosts was saved to content.hosts inside wizardStep2Controller
      controller.save('hosts');
      router.transitionTo('step2');
      controller.setDBProperty('bootStatus', false);
    },
    evaluateStep: function evaluateStep(router) {
      var addHostController = router.get('addHostController');
      var wizardStep2Controller = router.get('wizardStep2Controller');

      wizardStep2Controller.set('hasSubmitted', true);

      if (!wizardStep2Controller.get('isSubmitDisabled')) {
        wizardStep2Controller.evaluateStep();
      }
    }
  }),

  step2: App.StepRoute.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('2');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        var wizardStep3Controller = router.get('wizardStep3Controller');
        wizardStep3Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep3', controller.get('content'));
      });
    },
    backTransition: function backTransition(router) {
      router.transitionTo('step1');
    },
    exit: function exit(router) {
      router.get('wizardStep3Controller').set('stopBootstrap', true);
    },
    nextTransition: function nextTransition(router, context) {
      var addHostController = router.get('addHostController');
      var wizardStep3Controller = router.get('wizardStep3Controller');
      var wizardStep6Controller = router.get('wizardStep6Controller');
      addHostController.saveConfirmedHosts(wizardStep3Controller);
      addHostController.saveClients();

      addHostController.setDBProperties({
        bootStatus: true,
        slaveComponentHosts: undefined
      });
      wizardStep6Controller.set('isClientsSet', false);
      router.transitionTo('step3');
    },
    /**
     * Wrapper for remove host action.
     * Since saving data stored in addHostController, we should call this from router
     * @param router
     * @param context Array of hosts to delete
     */
    removeHosts: function removeHosts(router, context) {
      var controller = router.get('addHostController');
      controller.removeHosts(context);
    }
  }),

  step3: App.StepRoute.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('3');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        var wizardStep6Controller = router.get('wizardStep6Controller');
        wizardStep6Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep6', controller.get('content'));
      });
    },
    backTransition: function backTransition(router) {
      router.transitionTo('step2');
    },
    next: function next(router) {
      App.set('router.nextBtnClickInProgress', true);
      var addHostController = router.get('addHostController');
      var wizardStep6Controller = router.get('wizardStep6Controller');

      wizardStep6Controller.callValidation(function () {
        wizardStep6Controller.showValidationIssuesAcceptBox(function () {
          addHostController.saveSlaveComponentHosts(wizardStep6Controller);
          if (wizardStep6Controller.isAllCheckboxesEmpty()) {
            App.set('router.nextBtnClickInProgress', false);
            router.transitionTo('step5');
            addHostController.set('content.configGroups', []);
            addHostController.saveServiceConfigGroups();
          } else {
            App.set('router.nextBtnClickInProgress', false);
            router.transitionTo('step4');
          }
        });
      });
    }
  }),

  step4: App.StepRoute.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addHostController');
      var addHostStep4Controller = router.get('addHostStep4Controller');
      controller.setCurrentStep('4');
      controller.dataLoading().done(function () {
        addHostStep4Controller.loadConfigGroups();
        addHostStep4Controller.set('isConfigGroupLoaded', false);
        addHostStep4Controller.configGroupsLoading().done(function () {
          controller.loadAllPriorSteps();
          controller.loadServiceConfigGroups();
          addHostStep4Controller.set('wizardController', controller);
          controller.connectOutlet('addHostStep4', controller.get('content'));
        });
      });
    },
    backTransition: function backTransition(router) {
      var goToPreviousStep = function goToPreviousStep() {
        router.transitionTo('step3');
      };
      var wizardStep7Controller = router.get('wizardStep7Controller');
      if (wizardStep7Controller.hasChanges()) {
        wizardStep7Controller.showChangesWarningPopup(goToPreviousStep);
      } else {
        goToPreviousStep();
      }
    },
    nextTransition: function nextTransition(router) {
      var addHostController = router.get('addHostController');
      addHostController.saveServiceConfigGroups();
      router.transitionTo('step5');
    }
  }),

  step5: Em.Route.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('5');
      controller.dataLoading().done(function () {
        router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
          controller.loadAllPriorSteps();
          controller.getServiceConfigGroups();
          var wizardStep8Controller = router.get('wizardStep8Controller');
          wizardStep8Controller.set('wizardController', controller);
          controller.connectOutlet('wizardStep8', controller.get('content'));
        });
      });
    },
    back: function back(router) {
      var addHostController = router.get('addHostController');

      if (addHostController.isConfigGroupsEmpty() && !router.get('wizardStep8Controller.isBackBtnDisabled')) {
        router.transitionTo('step3');
      } else if (!router.get('wizardStep8Controller.isBackBtnDisabled')) {
        router.transitionTo('step4');
      }
    },
    next: function next(router) {
      router.get('mainAdminKerberosController').getKDCSessionState(function () {
        var addHostController = router.get('addHostController');
        var wizardStep8Controller = router.get('wizardStep8Controller');
        addHostController.applyConfigGroup();
        addHostController.installServices(false, function () {
          addHostController.setInfoForStep9();
          // We need to do recovery based on whether we are in Add Host or Installer wizard
          router.set('nextBtnClickInProgress', true);
          addHostController.saveClusterState('ADD_HOSTS_INSTALLING_3', {
            alwaysCallback: function alwaysCallback() {
              wizardStep8Controller.set('servicesInstalled', true);
              router.set('nextBtnClickInProgress', false);
              router.transitionTo('step6');
            }
          });
        });
      });
    }
  }),

  step6: Em.Route.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('6');
      controller.dataLoading().done(function () {
        var wizardStep9Controller = router.get('wizardStep9Controller');
        wizardStep9Controller.set('wizardController', controller);
        controller.loadAllPriorSteps();
        if (!App.get('testMode')) {
          //if test mode is ON don't disable prior steps link.
          controller.setLowerStepsDisable(6);
        }
        controller.connectOutlet('wizardStep9', controller.get('content'));
      });
    },
    back: Em.Router.transitionTo('step5'),
    retry: function retry(router, context) {
      var addHostController = router.get('addHostController');
      var wizardStep9Controller = router.get('wizardStep9Controller');
      wizardStep9Controller.set('wizardController', addHostController);
      if (wizardStep9Controller.get('showRetry')) {
        if (wizardStep9Controller.get('content.cluster.status') === 'INSTALL FAILED') {
          var isRetry = true;
          addHostController.installServices(isRetry, function () {
            addHostController.setInfoForStep9();
            wizardStep9Controller.resetHostsForRetry();
            // We need to do recovery based on whether we are in Add Host or Installer wizard
            addHostController.saveClusterState('ADD_HOSTS_INSTALLING_3', {
              alwaysCallback: function alwaysCallback() {
                wizardStep9Controller.navigateStep();
              }
            });
          });
        } else {
          wizardStep9Controller.navigateStep();
        }
      }
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var addHostController = router.get('addHostController');
      var wizardStep9Controller = router.get('wizardStep9Controller');
      wizardStep9Controller.set('wizardController', addHostController);
      addHostController.saveInstalledHosts(wizardStep9Controller);
      router.set('nextBtnClickInProgress', true);

      // We need to do recovery based on whether we are in Add Host or Installer wizard
      addHostController.saveClusterState('ADD_HOSTS_INSTALLED_4', {
        alwaysCallback: function alwaysCallback() {
          router.set('nextBtnClickInProgress', false);
          router.transitionTo('step7');
        }
      });
    }
  }),

  step7: Em.Route.extend({
    route: '/step7',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addHostController');
      controller.setCurrentStep('7');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        var wizardStep10Controller = router.get('wizardStep10Controller');
        wizardStep10Controller.set('wizardController', controller);
        if (!App.get('testMode')) {
          //if test mode is ON don't disable prior steps link.
          controller.setLowerStepsDisable(7);
        }
        controller.connectOutlet('wizardStep10', controller.get('content'));
        router.get('updateController').set('isWorking', true);
      });
    },
    back: Em.Router.transitionTo('step6'),
    complete: function complete(router, context) {
      $(context.currentTarget).parents("#modal").find(".close").trigger('click');
    }
  }),

  backToHostsList: function backToHostsList(router, event) {
    App.router.get('updateController').set('isWorking', true);
    router.transitionTo('hosts.index');
  }

});

});

require.register("routes/add_kerberos_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/enable',
  breadcrumbs: null,
  enter: function enter(router) {
    router.get('mainController').dataLoading().done(function () {
      return App.clusterStatus.updateFromServer();
    }).done(function () {
      var kerberosWizardController = router.get('kerberosWizardController');
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.kerberos.wizard.header'),
        bodyClass: App.KerberosWizardView.extend({
          controller: kerberosWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          var self = this;
          switch (kerberosWizardController.get('currentStep')) {
            case "2":
              var step2Controller = router.get('kerberosWizardStep2Controller');
              if (step2Controller.get('testConnectionInProgress')) {
                step2Controller.showConnectionInProgressPopup(function () {
                  kerberosWizardController.warnBeforeExitPopup(function () {
                    self.exitWizard();
                  }, false);
                });
              } else {
                kerberosWizardController.warnBeforeExitPopup(function () {
                  self.exitWizard();
                }, false);
              }
              break;
            case "6":
            case "7":
              kerberosWizardController.warnBeforeExitPopup(function () {
                self.exitWizard();
              }, true);
              break;
            case "8":
              kerberosWizardController.warnBeforeExitPopup(function () {
                self.exitWizard(true);
              }, false);
              break;
            default:
              kerberosWizardController.warnBeforeExitPopup(function () {
                self.exitWizard();
              }, false);
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        },

        exitWizard: function exitWizard(skipDiscardChanges) {
          var controller = App.router.get('kerberosWizardController');
          var exitPath = controller.getDBProperty('onClosePath') || 'adminKerberos.index';
          controller.clearTasksData();
          if (skipDiscardChanges) {
            controller.resetOnClose(controller, exitPath);
          } else {
            controller.discardChanges().then(function () {
              controller.resetOnClose(controller, exitPath);
            });
          }
        }
      });
      kerberosWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        if (App.get('testMode')) {
          kerberosWizardController.setCurrentStep(App.db.data.KerberosWizard.currentStep);
        } else {
          switch (currentClusterStatus.clusterState) {
            case 'KERBEROS_DEPLOY':
              kerberosWizardController.setCurrentStep(currentClusterStatus.localdb.KerberosWizard.currentStep);
              break;
            default:
              var currStep = App.get('router.kerberosWizardController.currentStep');
              kerberosWizardController.setCurrentStep(currStep);
              break;
          }
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(kerberosWizardController.get('name'));
        router.transitionTo('step' + kerberosWizardController.get('currentStep'));
      });
    });
  },

  step1: App.StepRoute.extend({
    route: '/step1',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('1');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('kerberosWizardStep1', controller.get('content'));
        });
      });
    },

    unroutePath: function unroutePath() {
      return false;
    },

    nextTransition: function nextTransition(router) {
      var kerberosWizardController = router.get('kerberosWizardController');
      var kerberosStep1controller = router.get('kerberosWizardStep1Controller');

      kerberosWizardController.saveKerberosOption(kerberosStep1controller);
      kerberosWizardController.clearServiceConfigProperties();
      router.transitionTo('step2');
    }
  }),

  step2: App.StepRoute.extend({
    route: '/step2',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('2');
        controller.loadAllPriorSteps().done(function () {
          var kerberosWizardStep2Controller = router.get('kerberosWizardStep2Controller');
          kerberosWizardStep2Controller.set('wizardController', controller);
          controller.connectOutlet('kerberosWizardStep2', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    backTransition: function backTransition(router) {
      var controller = router.get('kerberosWizardStep2Controller');
      var kerberosWizardController = router.get('kerberosWizardController');
      kerberosWizardController.overrideVisibility(controller.get('configs'), true, []);
      router.transitionTo('step1');
    },

    nextTransition: function nextTransition(router) {
      var kerberosWizardController = router.get('kerberosWizardController');
      var kerberosWizardStep2Controller = router.get('kerberosWizardStep2Controller');

      if (kerberosWizardController.get('skipClientInstall')) {
        kerberosWizardStep2Controller.get('stepConfigs')[0].get('configs').findProperty('name', 'manage_identities').set('value', 'false');
        kerberosWizardStep2Controller.get('stepConfigs')[0].get('configs').findProperty('name', 'install_packages').set('value', 'false');
        kerberosWizardStep2Controller.get('stepConfigs')[0].get('configs').findProperty('name', 'manage_krb5_conf').set('value', 'false');
      }

      kerberosWizardController.saveServiceConfigProperties(kerberosWizardStep2Controller, true);
      kerberosWizardController.clearTasksData();
      if (kerberosWizardController.get('skipClientInstall')) {
        kerberosWizardController.setDBProperty('kerberosDescriptorConfigs', null);
        router.transitionTo('step4');
      } else {
        router.transitionTo('step3');
      }
    }
  }),

  step3: App.StepRoute.extend({
    route: '/step3',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('3');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('kerberosWizardStep3', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    back: Em.Router.transitionTo('step2'),
    nextTransition: function nextTransition(router) {
      var kerberosWizardController = router.get('kerberosWizardController');
      kerberosWizardController.setDBProperty('kerberosDescriptorConfigs', null);
      kerberosWizardController.clearCachedStepConfigValues(router.get('kerberosWizardStep4Controller'));
      router.transitionTo('step4');
    }
  }),

  step4: App.StepRoute.extend({
    route: '/step4',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      var step4Controller = router.get('kerberosWizardStep4Controller');
      controller.setCurrentStep(4);
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('4');
        controller.loadAllPriorSteps().done(function () {
          controller.setLowerStepsDisable(4);
          step4Controller.set('wizardController', controller);
          controller.connectOutlet('kerberosWizardStep4', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    backTransition: function backTransition(router) {
      if (router.get('kerberosWizardController.skipClientInstall')) {
        router.transitionTo('step2');
      } else {
        router.transitionTo('step3');
      }
    },
    nextTransition: function nextTransition(router) {
      var wizardCtrl = router.get('kerberosWizardController');
      var step5Controller = router.get('kerberosWizardStep5Controller');
      var kerberosDescriptor = wizardCtrl.get('kerberosDescriptorConfigs');
      wizardCtrl.cacheStepConfigValues(router.get('kerberosWizardStep4Controller'));
      step5Controller.postKerberosDescriptor(kerberosDescriptor).always(function (data, result) {
        if (result === 'error' && data.status === 409) {
          step5Controller.putKerberosDescriptor(kerberosDescriptor);
        } else {
          step5Controller.unkerberizeCluster();
        }
      });
    }
  }),

  step5: App.StepRoute.extend({
    route: '/step5',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('5');
        controller.setLowerStepsDisable(5);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('kerberosWizardStep5', controller.get('content'));
        });
      });
    },

    unroutePath: function unroutePath() {
      return false;
    },

    exitWizard: function exitWizard(router) {
      var popup = router.get('kerberosWizardController.popup');
      popup.onClose();
    },

    downloadCSV: function downloadCSV(router) {
      var kerberosWizardStep5Controller = router.get('kerberosWizardStep5Controller');
      kerberosWizardStep5Controller.getCSVData();
    },

    backTransition: Em.Router.transitionTo('step4'),

    nextTransition: function nextTransition(router) {
      var kerberosWizardController = router.get('kerberosWizardController');
      kerberosWizardController.setDBProperties({
        tasksStatuses: null,
        tasksRequestIds: null
      });
      router.transitionTo('step6');
    }
  }),

  step6: App.StepRoute.extend({
    route: '/step6',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('6');
        controller.setLowerStepsDisable(6);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('kerberosWizardStep6', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    backTransition: Em.Router.transitionTo('step4'),
    nextTransition: function nextTransition(router) {
      var kerberosWizardController = router.get('kerberosWizardController');
      kerberosWizardController.setDBProperties({
        tasksStatuses: null,
        tasksRequestIds: null
      });
      router.transitionTo('step7');
    }
  }),

  step7: App.StepRoute.extend({
    route: '/step7',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      var step7Controller = router.get('kerberosWizardStep7Controller');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('7');
        controller.setLowerStepsDisable(7);
        controller.loadAllPriorSteps().done(function () {
          step7Controller.setRequest();
          controller.connectOutlet('kerberosWizardStep7', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    backTransition: Em.Router.transitionTo('step4'),
    nextTransition: function nextTransition(router) {
      router.transitionTo('step8');
    }
  }),

  step8: App.StepRoute.extend({
    route: '/step8',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('kerberosWizardController');
      controller.dataLoading().done(function () {
        router.get('kerberosWizardController').setCurrentStep('8');
        controller.setLowerStepsDisable(8);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('kerberosWizardStep8', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    backTransition: Em.Router.transitionTo('step7'),
    nextTransition: function nextTransition(router) {
      var controller = router.get('kerberosWizardController');
      controller.resetOnClose(controller, 'adminKerberos.index');
    }
  })
});

});

require.register("routes/add_service_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
// @todo: remove App.supports.kerberosAutomated after Kerberos Automation Wizard support
var consoleMsg = Em.I18n.t('app.logger.wizard');

module.exports = App.WizardRoute.extend({
  route: '/service/add',

  enter: function enter(router) {
    if (App.isAuthorized('SERVICE.ADD_DELETE_SERVICES') && App.supports.enableAddDeleteServices) {
      // `getSecurityStatus` call is required to retrieve information related to kerberos type: Manual or automated kerberos
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isClusterNameLoaded').done(function () {
        App.router.get('mainAdminKerberosController').getSecurityStatus().always(function () {
          Em.run.next(function () {
            var addServiceController = router.get('addServiceController');
            App.router.get('updateController').set('isWorking', false);
            var popup = App.ModalPopup.show({
              classNames: ['wizard-modal-wrapper', 'add-service-wizard-modal'],
              header: Em.I18n.t('services.add.header'),
              modalDialogClasses: ['modal-xlg'],
              bodyClass: App.AddServiceView.extend({
                controllerBinding: 'App.router.addServiceController'
              }),
              primary: Em.I18n.t('form.cancel'),
              showFooter: false,
              secondary: null,

              onPrimary: function onPrimary() {
                this.hide();
                App.router.transitionTo('main.services.index');
              },
              onClose: function onClose() {
                this.showWarningPopup();
              },
              afterWarning: function afterWarning() {
                this.set('showCloseButton', false); // prevent user to click "Close" many times
                App.router.get('updateController').set('isWorking', true);
                App.router.get('updateController').updateServices(function () {
                  App.router.get('updateController').updateServiceMetric();
                });
                var exitPath = addServiceController.getDBProperty('onClosePath') || 'main.services.index';
                addServiceController.resetOnClose(addServiceController, exitPath);
              },
              showWarningPopup: function showWarningPopup() {
                var mainPopupContext = this;
                var currentStep = addServiceController.get('currentStep');
                var DEPLOY_STEP = '6';
                var LAST_STEP = '7';
                if (currentStep === LAST_STEP) {
                  mainPopupContext.afterWarning();
                } else {
                  App.ModalPopup.show({
                    encodeBody: false,
                    header: currentStep === DEPLOY_STEP ? Em.I18n.t('common.warning') : Em.I18n.t('popup.confirmation.commonHeader'),
                    primaryClass: currentStep === DEPLOY_STEP ? 'btn-warning' : 'btn-success',
                    secondary: Em.I18n.t('form.cancel'),
                    body: currentStep === DEPLOY_STEP ? Em.I18n.t('services.add.warningStep6') : Em.I18n.t('services.add.warning'),
                    onPrimary: function onPrimary() {
                      this.hide();
                      mainPopupContext.afterWarning();
                    }
                  });
                }
              },
              didInsertElement: function didInsertElement() {
                this._super();
                this.fitHeight();
              }
            });
            addServiceController.set('popup', popup);
            var currentClusterStatus = App.clusterStatus.get('value');
            if (currentClusterStatus) {
              switch (currentClusterStatus.clusterState) {
                case 'ADD_SERVICES_DEPLOY_PREP_2':
                  addServiceController.setCurrentStep('5');
                  break;
                case 'ADD_SERVICES_INSTALLING_3':
                case 'SERVICE_STARTING_3':
                case 'ADD_SERVICES_INSTALLED_4':
                  addServiceController.setCurrentStep('7');
                  break;
                default:
                  if (App.db.data.AddService.currentStep && App.db.data.AddService.currentStep !== addServiceController.get('currentStep')) {
                    addServiceController.setCurrentStep(App.db.data.AddService.currentStep);
                  }
                  break;
              }
            }

            App.router.get('wizardWatcherController').setUser(addServiceController.get('name'));
            router.transitionTo('step' + addServiceController.get('currentStep'));
          });
        });
      });
    } else {
      Em.run.next(function () {
        App.router.transitionTo('main.services');
      });
    }
  },

  step1: App.StepRoute.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      App.logger.setTimer(consoleMsg.format(1));
      var controller = router.get('addServiceController');
      controller.setCurrentStep('1');
      controller.set('hideBackButton', true);
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps().done(function () {
          var wizardStep4Controller = router.get('wizardStep4Controller');
          wizardStep4Controller.set('wizardController', controller);
          controller.loadServiceVersionFromVersionDefinitions().complete(function () {
            controller.set('content.services', App.StackService.find().forEach(function (item) {
              // user the service version from VersionDefinition
              Ember.set(item, 'serviceVersionDisplay', controller.get('serviceVersionsMap')[item.get('serviceName')]);
              //item.set('serviceVersionDisplay', controller.get('serviceVersionsMap')[item.get('serviceName')]);
            }));
            App.logger.logTimerIfMoreThan(consoleMsg.format(1));
            controller.connectOutlet('wizardStep4', controller.get('content.services').filterProperty('isInstallable', true));
          });
        });
      });
    },
    nextTransition: function nextTransition(router) {
      var addServiceController = router.get('addServiceController');
      var wizardStep4Controller = router.get('wizardStep4Controller');
      addServiceController.saveServices(wizardStep4Controller);
      addServiceController.saveClients(wizardStep4Controller);
      addServiceController.setDBProperty('masterComponentHosts', undefined);

      var wizardStep5Controller = router.get('wizardStep5Controller');
      wizardStep5Controller.clearRecommendations(); // Force reload recommendation between steps 1 and 2
      addServiceController.setDBProperty('recommendations', undefined);
      addServiceController.clearEnhancedConfigs();
      router.transitionTo('step2');
    }
  }),

  step2: App.StepRoute.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      App.logger.setTimer(consoleMsg.format(2));
      var controller = router.get('addServiceController');
      var wizardStep2Controller = router.get('wizardStep5Controller');
      controller.setCurrentStep('2');
      controller.set('hideBackButton', false);
      wizardStep2Controller.set('isInitialLayout', true);
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps().done(function () {
          App.logger.logTimerIfMoreThan(consoleMsg.format(2));
          wizardStep2Controller.set('wizardController', controller);
          controller.connectOutlet('wizardStep5', controller.get('content'));
        });
      });
    },

    backTransition: function backTransition(router) {
      router.transitionTo('step1');
    },

    next: function next(router) {
      var addServiceController = router.get('addServiceController');
      var wizardStep5Controller = router.get('wizardStep5Controller');
      var wizardStep6Controller = router.get('wizardStep6Controller');
      addServiceController.saveMasterComponentHosts(wizardStep5Controller);
      addServiceController.setDBProperty('slaveComponentHosts', undefined);
      addServiceController.setDBProperty('recommendations', wizardStep5Controller.get('content.recommendations'));
      wizardStep6Controller.set('isClientsSet', false);
      router.transitionTo('step3');
    }
  }),

  step3: App.StepRoute.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      App.logger.setTimer(consoleMsg.format(3));
      var controller = router.get('addServiceController');
      controller.setCurrentStep('3');
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
        controller.loadAllPriorSteps().done(function () {
          var wizardStep6Controller = router.get('wizardStep6Controller');
          wizardStep6Controller.set('wizardController', controller);
          App.logger.logTimerIfMoreThan(consoleMsg.format(3));
          controller.connectOutlet('wizardStep6', controller.get('content'));
        });
      });
    },

    backTransition: function backTransition(router) {
      var controller = router.get('addServiceController');
      if (!controller.get('content.skipMasterStep')) {
        return router.transitionTo('step2');
      }
      return router.transitionTo('step1');
    },

    next: function next(router) {
      App.set('router.nextBtnClickInProgress', true);
      var addServiceController = router.get('addServiceController');
      var wizardStep6Controller = router.get('wizardStep6Controller');

      wizardStep6Controller.callValidation(function () {
        wizardStep6Controller.showValidationIssuesAcceptBox(function () {
          addServiceController.saveSlaveComponentHosts(wizardStep6Controller);
          addServiceController.get('content').set('serviceConfigProperties', null);
          addServiceController.setDBProperties({
            groupsToDelete: null,
            recommendationsHostGroups: wizardStep6Controller.get('content.recommendationsHostGroups'),
            recommendationsConfigs: null
          });
          router.get('wizardStep7Controller').set('recommendationsConfigs', null);
          router.get('wizardStep7Controller').clearAllRecommendations();
          addServiceController.setDBProperty('serviceConfigGroups', undefined);
          App.ServiceConfigGroup.find().clear();
          addServiceController.clearServiceConfigProperties();
          if (App.get('isKerberosEnabled')) {
            addServiceController.setDBProperty('kerberosDescriptorConfigs', null);
          }
          router.transitionTo('step4');
        });
      });
    }
  }),

  step4: App.StepRoute.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      App.logger.setTimer(consoleMsg.format(4));
      var controller = router.get('addServiceController');
      controller.setCurrentStep('4');
      controller.dataLoading().done(function () {
        var wizardStep7Controller = router.get('wizardStep7Controller');
        controller.loadAllPriorSteps().done(function () {
          wizardStep7Controller.getConfigTags(true);
          wizardStep7Controller.set('wizardController', controller);
          controller.usersLoading().done(function () {
            router.get('mainController').isLoading.call(router.get('clusterController'), 'isClusterNameLoaded').done(function () {
              router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
                App.logger.logTimerIfMoreThan(consoleMsg.format(4));
                controller.connectOutlet('wizardStep7', controller.get('content'));
              });
            });
          });
        });
      });
    },

    backTransition: function backTransition(router) {
      var controller = router.get('addServiceController');
      var wizardStep7Controller = router.get('wizardStep7Controller');
      var step = 'step1';
      if (!controller.get('content.skipSlavesStep')) {
        step = 'step3';
      } else if (!controller.get('content.skipMasterStep')) {
        step = 'step2';
      }
      var goToPreviousStep = function goToPreviousStep() {
        router.transitionTo(step);
      };
      if (wizardStep7Controller.hasChanges()) {
        wizardStep7Controller.showChangesWarningPopup(goToPreviousStep);
      } else {
        goToPreviousStep();
      }
    },
    next: function next(router) {
      var addServiceController = router.get('addServiceController');
      var wizardStep7Controller = router.get('wizardStep7Controller');
      var kerberosDescriptor = addServiceController.get('kerberosDescriptor');
      wizardStep7Controller.checkDescriptor().always(function (data, status) {
        wizardStep7Controller.storeClusterDescriptorStatus(status === 'success');
        if (App.get('isKerberosEnabled')) {
          wizardStep7Controller.updateKerberosDescriptor(kerberosDescriptor, wizardStep7Controller.getDescriptorConfigs());
          addServiceController.saveKerberosDescriptorConfigs(kerberosDescriptor);
          if (router.get('mainAdminKerberosController.isManualKerberos')) {
            router.get('wizardStep8Controller').set('wizardController', router.get('addServiceController'));
            router.get('wizardStep8Controller').updateKerberosDescriptor(true);
          }
        }
        addServiceController.saveServiceConfigGroups(wizardStep7Controller, true);
        addServiceController.saveServiceConfigProperties(wizardStep7Controller);
        router.transitionTo('step5');
      });
    }
  }),

  step5: App.StepRoute.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router) {
      App.logger.setTimer(consoleMsg.format(5));
      var controller = router.get('addServiceController');
      controller.setCurrentStep('5');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps().done(function () {
          var wizardStep8Controller = router.get('wizardStep8Controller');
          wizardStep8Controller.set('wizardController', controller);
          App.logger.logTimerIfMoreThan(consoleMsg.format(5));
          controller.connectOutlet('wizardStep8', controller.get('content'));
        });
      });
      if (!!App.get('router.mainAdminKerberosController.kdc_type')) {
        router.get('kerberosWizardStep5Controller').getCSVData(true);
      }
    },
    backTransition: function backTransition(router) {
      var controller = router.get('addServiceController');
      if (!controller.get('content.skipConfigStep')) {
        return router.transitionTo('step4');
      }
      if (!controller.get('content.skipSlavesStep')) {
        return router.transitionTo('step3');
      }
      if (!controller.get('content.skipMasterStep')) {
        return router.transitionTo('step2');
      }
      return router.transitionTo('step1');
    },
    next: function next(router) {
      var addServiceController = router.get('addServiceController');
      addServiceController.installServices(function () {
        router.get('wizardStep8Controller').set('servicesInstalled', true);
        addServiceController.setInfoForStep9();
        addServiceController.saveClusterState('ADD_SERVICES_INSTALLING_3');
        App.router.transitionTo('step6');
      });
    }
  }),

  step6: Em.Route.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router, context) {
      App.logger.setTimer(consoleMsg.format(6));
      var controller = router.get('addServiceController');
      controller.setCurrentStep('6');
      if (!App.get('testMode')) {
        //if test mode is ON don't disable prior steps link.
        controller.setLowerStepsDisable(6);
      }
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps().done(function () {
          var wizardStep9Controller = router.get('wizardStep9Controller');
          wizardStep9Controller.set('wizardController', controller);
          App.logger.setTimer(consoleMsg.format(6));
          controller.connectOutlet('wizardStep9', controller.get('content'));
        });
      });
    },
    back: Em.Router.transitionTo('step5'),
    retry: function retry(router, context) {
      var addServiceController = router.get('addServiceController');
      var wizardStep9Controller = router.get('wizardStep9Controller');
      if (wizardStep9Controller.get('showRetry')) {
        if (wizardStep9Controller.get('content.cluster.status') === 'INSTALL FAILED') {
          addServiceController.installServices(function () {
            addServiceController.setInfoForStep9();
            wizardStep9Controller.resetHostsForRetry();
            // We need to do recovery based on whether we are in Add Host or Installer wizard
            addServiceController.saveClusterState('ADD_SERVICES_INSTALLING_3');
            wizardStep9Controller.navigateStep();
          });
        } else {
          wizardStep9Controller.navigateStep();
        }
      }
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var addServiceController = router.get('addServiceController');
      var wizardStep9Controller = router.get('wizardStep9Controller');
      addServiceController.saveInstalledHosts(wizardStep9Controller);

      // We need to do recovery based on whether we are in Add Host or Installer wizard
      addServiceController.saveClusterState('ADD_SERVICES_INSTALLED_4');
      router.transitionTo('step7');
    }
  }),

  step7: Em.Route.extend({
    route: '/step7',
    connectOutlets: function connectOutlets(router, context) {
      var controller = router.get('addServiceController');
      controller.setCurrentStep('7');
      controller.setLowerStepsDisable(7);
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('wizardStep10', controller.get('content'));
        });
      });
    },
    back: Em.Router.transitionTo('step6'),
    complete: function complete(router, context) {
      var addServiceController = router.get('addServiceController');
      addServiceController.get('popup').onClose();
    }
  }),

  backToServices: function backToServices(router) {
    App.router.get('updateController').set('isWorking', true);
    router.transitionTo('services');
  }

});

});

require.register("routes/create_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/widget/create',
  enter: function enter(router, context) {
    router.get('mainController').dataLoading().done(function () {
      var widgetWizardController = router.get('widgetWizardController');
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('widget.create.wizard.header'),
        bodyClass: App.WidgetWizardView.extend({
          controller: widgetWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          widgetWizardController.cancel();
        },

        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }

      });
      widgetWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        if (App.get('testMode')) {
          widgetWizardController.setCurrentStep(App.db.data.WidgetWizard.currentStep);
        } else {
          var currStep = App.get('router.widgetWizardController.currentStep');
          widgetWizardController.setCurrentStep(currStep);
        }
      }
      Em.run.next(function () {
        router.transitionTo('step' + widgetWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('widgetWizardController');
      controller.dataLoading().done(function () {
        router.get('widgetWizardController').setCurrentStep('1');
        controller.loadAllPriorSteps();
        controller.connectOutlet('widgetWizardStep1', controller.get('content'));
      });
    },

    unroutePath: function unroutePath() {
      return false;
    },

    next: function next(router) {
      var widgetWizardController = router.get('widgetWizardController');
      var widgetStep1controller = router.get('widgetWizardStep1Controller');
      widgetWizardController.save('widgetType', widgetStep1controller.get('widgetType'));
      widgetWizardController.setDBProperties({
        widgetProperties: {},
        widgetMetrics: [],
        allMetrics: [],
        widgetValues: [],
        expressions: [],
        dataSets: [],
        templateValue: ''
      });
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('widgetWizardController');
      controller.dataLoading().done(function () {
        router.get('widgetWizardController').setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('widgetWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    back: Em.Router.transitionTo('step1'),

    next: function next(router) {
      var widgetWizardController = router.get('widgetWizardController');
      var widgetStep2controller = router.get('widgetWizardStep2Controller');
      widgetWizardController.save('widgetProperties', widgetStep2controller.get('widgetProperties'));
      widgetWizardController.save('widgetMetrics', widgetStep2controller.get('widgetMetrics'));
      widgetWizardController.save('widgetValues', widgetStep2controller.get('widgetValues'));
      widgetWizardController.save('templateValue', widgetStep2controller.get('templateValue'));
      widgetWizardController.save('widgetName', "");
      widgetWizardController.save('widgetDescription', "");
      widgetWizardController.save('widgetScope', null);
      router.transitionTo('step3');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('widgetWizardController');
      controller.dataLoading().done(function () {
        router.get('widgetWizardController').setCurrentStep('3');
        controller.loadAllPriorSteps();
        controller.connectOutlet('widgetWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    back: Em.Router.transitionTo('step2'),
    complete: function complete(router, context) {
      router.get('widgetWizardController').postWidgetDefinition(context);
    }
  })
});

});

require.register("routes/edit_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/widget/edit',
  enter: function enter(router, context) {
    router.get('mainController').dataLoading().done(function () {
      var widgetEditController = router.get('widgetEditController');
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('widget.edit.wizard.header'),
        bodyClass: App.WidgetEditView.extend({
          controller: widgetEditController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          widgetEditController.cancel();
        },

        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }

      });
      widgetEditController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        if (App.get('testMode')) {
          widgetEditController.setCurrentStep(App.db.data.WidgetWizard.currentStep);
        } else {
          var currStep = App.get('router.widgetEditController.currentStep');
          widgetEditController.setCurrentStep(currStep);
        }
      }
      Em.run.next(function () {
        router.transitionTo('step' + widgetEditController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('widgetEditController');
      controller.dataLoading().done(function () {
        router.get('widgetEditController').setCurrentStep('1');
        controller.loadAllPriorSteps();
        controller.connectOutlet('widgetWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },

    next: function next(router) {
      var widgetEditController = router.get('widgetEditController');
      var widgetStep2controller = router.get('widgetWizardStep2Controller');
      widgetEditController.save('widgetProperties', widgetStep2controller.get('widgetProperties'));
      widgetEditController.save('widgetMetrics', widgetStep2controller.get('widgetMetrics'));
      widgetEditController.save('widgetValues', widgetStep2controller.get('widgetValues'));
      widgetEditController.save('expressions', widgetStep2controller.get('expressions'));
      widgetEditController.save('dataSets', widgetStep2controller.get('dataSets'));
      widgetEditController.save('templateValue', widgetStep2controller.get('templateValue'));
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',

    connectOutlets: function connectOutlets(router) {
      var controller = router.get('widgetEditController');
      controller.dataLoading().done(function () {
        router.get('widgetEditController').setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('widgetWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    back: Em.Router.transitionTo('step1'),
    complete: function complete(router, context) {
      router.get('widgetEditController').putWidgetDefinition(context);
    }
  })
});

});

require.register("routes/high_availability_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/NameNode/enable',

  breadcrumbs: {
    label: Em.I18n.t('admin.highAvailability.wizard.header')
  },

  enter: function enter(router) {
    var highAvailabilityWizardController = router.get('highAvailabilityWizardController');
    highAvailabilityWizardController.dataLoading().done(function () {
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.highAvailability.wizard.header'),
        bodyClass: App.HighAvailabilityWizardView.extend({
          controller: highAvailabilityWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,
        hideCloseButton: function () {
          var currStep = App.router.get('highAvailabilityWizardController.currentStep');
          switch (currStep) {
            case "5":
            case "7":
            case "9":
              if (App.supports.autoRollbackHA) {
                this.set('showCloseButton', false);
              } else {
                this.set('showCloseButton', true);
              }
              break;
            default:
              this.set('showCloseButton', true);
          }
        }.observes('App.router.highAvailabilityWizardController.currentStep'),

        onClose: function onClose() {
          var currStep = App.router.get('highAvailabilityWizardController.currentStep');
          var highAvailabilityProgressPageController = App.router.get('highAvailabilityProgressPageController');
          if (parseInt(currStep, 10) > 4) {
            if (!App.supports.autoRollbackHA) {
              highAvailabilityProgressPageController.manualRollback();
            } else {
              this.hide();
              App.router.get('highAvailabilityWizardController').setCurrentStep('1');
              App.router.transitionTo('rollbackHighAvailability');
            }
          } else {
            var controller = App.router.get('highAvailabilityWizardController');
            controller.clearTasksData();
            controller.resetOnClose(controller, 'main.services.index');
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      highAvailabilityWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'HIGH_AVAILABILITY_DEPLOY':
            highAvailabilityWizardController.setCurrentStep(currentClusterStatus.localdb.HighAvailabilityWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('highAvailabilityWizardController.currentStep');
            highAvailabilityWizardController.setCurrentStep(currStep);
            break;
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(highAvailabilityWizardController.get('name'));
        router.transitionTo('step' + highAvailabilityWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController'),
          highAvailabilityWizardStep1Controller = router.get('highAvailabilityWizardStep1Controller');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        highAvailabilityWizardStep1Controller.setHawqInstalled();
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep1', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.saveNameServiceId(router.get('highAvailabilityWizardStep1Controller.content.nameServiceId'));
      controller.clearMasterComponentHosts();
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep2', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('highAvailabilityWizardController');
      var highAvailabilityWizardStep2Controller = router.get('highAvailabilityWizardStep2Controller');
      var addNN = highAvailabilityWizardStep2Controller.get('selectedServicesMasters').filterProperty('component_name', 'NAMENODE').findProperty('isInstalled', false).get('selectedHost');
      var sNN = highAvailabilityWizardStep2Controller.get('selectedServicesMasters').findProperty('component_name', 'SECONDARY_NAMENODE').get('selectedHost');
      if (addNN) {
        App.db.setRollBackHighAvailabilityWizardAddNNHost(addNN);
      }
      if (sNN) {
        App.db.setRollBackHighAvailabilityWizardSNNHost(sNN);
      }

      controller.saveMasterComponentHosts(highAvailabilityWizardStep2Controller);
      controller.get('content').set('serviceConfigProperties', null);
      controller.setDBProperty('serviceConfigProperties', null);
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep3', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      App.set('router.nextBtnClickInProgress', true);
      var controller = router.get('highAvailabilityWizardController');
      var stepController = router.get('highAvailabilityWizardStep3Controller');
      controller.saveServiceConfigProperties(stepController);
      controller.saveConfigTag(stepController.get("hdfsSiteTag"));
      controller.saveConfigTag(stepController.get("coreSiteTag"));
      if (App.Service.find().someProperty('serviceName', 'HBASE')) {
        controller.saveConfigTag(stepController.get("hbaseSiteTag"));
      }
      if (App.Service.find().someProperty('serviceName', 'RANGER')) {
        controller.saveConfigTag(stepController.get("rangerEnvTag"));
      }
      App.set('router.nextBtnClickInProgress', false);
      router.transitionTo('step4');
    },
    back: Em.Router.transitionTo('step2')
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep4', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.clearTasksData();
      router.transitionTo('step5');
    }
  }),

  step5: Em.Route.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('5');
        controller.setLowerStepsDisable(5);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep5', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.clearTasksData();
      router.transitionTo('step6');
    }
  }),

  step6: Em.Route.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('6');
        controller.setLowerStepsDisable(6);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep6', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      router.transitionTo('step7');
    }
  }),

  step7: Em.Route.extend({
    route: '/step7',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('7');
        controller.setLowerStepsDisable(7);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep7', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.clearTasksData();
      router.transitionTo('step8');
    }
  }),

  step8: Em.Route.extend({
    route: '/step8',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('8');
        controller.setLowerStepsDisable(8);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep8', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      App.showConfirmationPopup(function () {
        router.transitionTo('step9');
      }, Em.I18n.t('admin.highAvailability.wizard.step8.confirmPopup.body'));
    }
  }),

  step9: Em.Route.extend({
    route: '/step9',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('highAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('9');
        controller.setLowerStepsDisable(9);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('highAvailabilityWizardStep9', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var proceed = function proceed() {
        var controller = router.get('highAvailabilityWizardController');
        controller.clearTasksData();
        controller.resetOnClose(controller, 'main.services.index');
      };
      if (App.Service.find().someProperty('serviceName', 'HAWQ')) {
        App.showAlertPopup(Em.I18n.t('admin.highAvailability.wizard.step9.hawq.confirmPopup.header'), Em.I18n.t('admin.highAvailability.wizard.step9.hawq.confirmPopup.body'), proceed);
      } else {
        proceed();
      }
    }
  })

});

});

require.register("routes/installer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = Em.Route.extend(App.RouterRedirections, {
  route: '/installer',
  App: require('app'),

  enter: function enter(router) {
    var self = this;

    App.clusterStatus.set('wizardControllerName', App.router.get('installerController.name'));

    router.getAuthenticated().done(function (loggedIn) {
      if (loggedIn) {
        var applicationController = router.get('applicationController');
        App.router.get('experimentalController').loadSupports().complete(function () {
          applicationController.startKeepAlivePoller();
          // check server/web client versions match
          App.router.get('installerController').checkServerClientVersion().done(function () {

            $('title').text(Em.I18n.t('app.name.subtitle').format(Em.I18n.t('app.name.subtitle.installer')));
            $('#main').addClass('install-wizard-content');

            App.router.get('mainViewsController').loadAmbariViews();
            if (App.isAuthorized('AMBARI.ADD_DELETE_CLUSTERS')) {
              router.get('mainController').stopPolling();
              Em.run.next(function () {
                App.clusterStatus.updateFromServer().complete(function () {
                  var currentClusterStatus = App.clusterStatus.get('value');
                  //@TODO: Clean up  following states. Navigation should be done solely via currentStep stored in the localDb and API persist endpoint.
                  //       Actual currentStep value for the installer controller should always remain in sync with localdb and at persist store in the server.
                  if (currentClusterStatus) {
                    if (self.get('installerStatuses').contains(currentClusterStatus.clusterState)) {
                      self.redirectToInstaller(router, currentClusterStatus, true);
                    } else {
                      router.transitionTo('main.dashboard.index');
                    }
                  }
                });
              });
            } else {
              Em.run.next(function () {
                App.router.transitionTo('main.views.index');
              });
            }
          });
        });
      } else {
        Ember.run.next(function () {
          router.transitionTo('login');
        });
      }
    });
  },

  routePath: function routePath(router, event) {
    router.setNavigationFlow(event);
    if (!router.isFwdNavigation) {
      this._super(router, event);
    } else {
      router.set('backBtnForHigherStep', true);

      var installerController = router.get('installerController');
      router.transitionTo('step' + installerController.get('currentStep'));
    }
  },

  connectOutlets: function connectOutlets(router, context) {
    router.get('applicationController').connectOutlet('installer');
  },

  step0: Em.Route.extend({
    route: '/step0',
    connectOutlets: function connectOutlets(router) {
      console.time('step0 connectOutlets');
      var self = this;
      var wizardStep0Controller = router.get('wizardStep0Controller');
      var controller = router.get('installerController');
      controller.setCurrentStep('0');
      controller.loadAllPriorSteps().done(function () {
        wizardStep0Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep0', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step0 connectOutlets');
      });
    },

    next: function next(router) {
      console.time('step0 next');
      var installerController = router.get('installerController');
      var wizardStep1Controller = router.get('wizardStep1Controller');
      installerController.save('cluster');
      App.db.setStacks(undefined);
      App.db.setRepos(undefined);
      App.db.setLocalRepoVDFData(undefined);
      App.Stack.find().clear();

      installerController.set('content.stacks', undefined);
      wizardStep1Controller.set('skipValidationChecked', false);
      wizardStep1Controller.set('optionsToSelect', {
        'usePublicRepo': {
          index: 0,
          isSelected: true
        },
        'useLocalRepo': {
          index: 1,
          isSelected: false,
          'uploadFile': {
            index: 0,
            name: 'uploadFile',
            file: '',
            hasError: false,
            isSelected: true
          },
          'enterUrl': {
            index: 1,
            name: 'enterUrl',
            url: '',
            placeholder: Em.I18n.t('installer.step1.useLocalRepo.enterUrl.placeholder'),
            hasError: false,
            isSelected: false
          }
        }
      });
      router.transitionTo('step1');
      console.timeEnd('step0 next');
    }
  }),

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      console.time('step1 connectOutlets');
      var self = this;
      var controller = router.get('installerController'),
          wizardStep1Controller = router.get('wizardStep1Controller');
      controller.setCurrentStep('1');
      controller.loadAllPriorSteps().done(function () {
        wizardStep1Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep1', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step1 connectOutlets');
      });
    },
    back: Em.Router.transitionTo('step0'),
    next: function next(router) {
      console.time('step1 next');
      if (router.get('btnClickInProgress')) {
        return;
      }
      var wizardStep1Controller = router.get('wizardStep1Controller');
      var installerController = router.get('installerController');
      installerController.validateJDKVersion(function () {
        installerController.checkRepoURL(wizardStep1Controller).done(function () {
          App.set('router.nextBtnClickInProgress', true);
          installerController.setDBProperty('service', undefined);
          installerController.setStacks();
          installerController.clearInstallOptions();
          router.transitionTo('step2');
          console.timeEnd('step1 next');
        });
      }, function () {});
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step2 connectOutlets');
      var self = this;
      router.setNavigationFlow('step2');

      var controller = router.get('installerController');
      var wizardStep2Controller = router.get('wizardStep2Controller');
      controller.setCurrentStep('2');
      controller.loadAllPriorSteps().done(function () {
        wizardStep2Controller.set('wizardController', controller);
        self.scrollTop();
        controller.connectOutlet('wizardStep2', controller.get('content'));
        console.timeEnd('step2 connectOutlets');
      });
    },
    back: Em.Router.transitionTo('step1'),
    next: function next(router) {
      console.time('step2 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var controller = router.get('installerController');
        controller.save('installOptions');
        //hosts was saved to content.hosts inside wizardStep2Controller
        controller.save('hosts');
        router.transitionTo('step3');
      }
      console.timeEnd('step2 next');
    }
  }),

  step3: App.StepRoute.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      console.time('step3 connectOutlets');
      var self = this;
      var controller = router.get('installerController');
      controller.setCurrentStep('3');
      controller.loadAllPriorSteps().done(function () {
        var wizardStep3Controller = router.get('wizardStep3Controller');
        wizardStep3Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep3', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step3 connectOutlets');
      });
    },

    backTransition: function backTransition(router) {
      router.transitionTo('step2');
    },

    next: function next(router, context) {
      console.time('step3 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var installerController = router.get('installerController');
        var wizardStep3Controller = router.get('wizardStep3Controller');
        installerController.saveConfirmedHosts(wizardStep3Controller);
        installerController.setDBProperties({
          bootStatus: true,
          selectedServiceNames: undefined,
          installedServiceNames: undefined
        });
        router.transitionTo('step4');
        console.timeEnd('step3 next');
      }
    },
    exit: function exit(router) {
      router.get('wizardStep3Controller').set('stopBootstrap', true);
    },
    /**
     * Wrapper for remove host action.
     * Since saving data stored in installerController, we should call this from router
     * @param router
     * @param context Array of hosts to delete
     */
    removeHosts: function removeHosts(router, context) {
      var controller = router.get('installerController');
      controller.removeHosts(context);
    }
  }),

  step4: App.StepRoute.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step4 connectOutlets');
      var self = this;
      router.setNavigationFlow('step4');
      var controller = router.get('installerController');
      controller.setCurrentStep('4');
      controller.loadAllPriorSteps().done(function () {
        var wizardStep4Controller = router.get('wizardStep4Controller');
        wizardStep4Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep4', App.StackService.find().filterProperty('isInstallable', true));
        self.scrollTop();
        console.timeEnd('step4 connectOutlets');
      });
    },

    backTransition: function backTransition(router) {
      router.transitionTo('step3');
    },

    next: function next(router) {
      console.time('step4 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var controller = router.get('installerController');
        var wizardStep4Controller = router.get('wizardStep4Controller');
        controller.saveServices(wizardStep4Controller);
        controller.saveClients(wizardStep4Controller);
        router.get('wizardStep5Controller').clearRecommendations(); // Force reload recommendation between steps 4 and 5
        controller.setDBProperties({
          recommendations: undefined,
          masterComponentHosts: undefined
        });
        controller.clearEnhancedConfigs();
        router.transitionTo('step5');
      }
      console.timeEnd('step4 next');
    }
  }),

  step5: App.StepRoute.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step5 connectOutlets');
      var self = this;
      router.setNavigationFlow('step5');

      var controller = router.get('installerController');
      var wizardStep5Controller = router.get('wizardStep5Controller');
      wizardStep5Controller.setProperties({
        servicesMasters: [],
        isInitialLayout: true
      });
      controller.setCurrentStep('5');
      controller.loadAllPriorSteps().done(function () {
        wizardStep5Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep5', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step5 connectOutlets');
      });
    },
    backTransition: function backTransition(router) {
      router.transitionTo('step4');
    },
    next: function next(router) {
      console.time('step5 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var controller = router.get('installerController');
        var wizardStep5Controller = router.get('wizardStep5Controller');
        var wizardStep6Controller = router.get('wizardStep6Controller');
        controller.saveMasterComponentHosts(wizardStep5Controller);
        controller.setDBProperties({
          slaveComponentHosts: undefined,
          recommendations: wizardStep5Controller.get('content.recommendations')
        });
        wizardStep6Controller.set('isClientsSet', false);
        router.transitionTo('step6');
      }
      console.timeEnd('step5 next');
    }
  }),

  step6: App.StepRoute.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step6 connectOutlets');
      var self = this;
      router.setNavigationFlow('step6');

      var controller = router.get('installerController');
      var wizardStep6Controller = router.get('wizardStep6Controller');
      router.get('wizardStep6Controller').set('hosts', []);
      controller.setCurrentStep('6');
      controller.loadAllPriorSteps().done(function () {
        wizardStep6Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep6', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step6 connectOutlets');
      });
    },
    backTransition: function backTransition(router) {
      router.transitionTo('step5');
    },

    next: function next(router) {
      console.time('step6 next');
      var controller = router.get('installerController');
      var wizardStep6Controller = router.get('wizardStep6Controller');
      if (!wizardStep6Controller.get('submitDisabled')) {
        wizardStep6Controller.showValidationIssuesAcceptBox(function () {
          if (!router.get('btnClickInProgress')) {
            App.set('router.nextBtnClickInProgress', true);
            controller.saveSlaveComponentHosts(wizardStep6Controller);
            controller.get('content').set('serviceConfigProperties', null);
            controller.get('content').set('componentsFromConfigs', []);
            controller.setDBProperties({
              serviceConfigGroups: null,
              recommendationsHostGroups: wizardStep6Controller.get('content.recommendationsHostGroups'),
              recommendationsConfigs: null,
              componentsFromConfigs: []
            });
            controller.clearServiceConfigProperties();
            router.transitionTo('step7');
            console.timeEnd('step6 next');
          }
        });
      }
    }
  }),

  step7: App.StepRoute.extend({
    route: '/step7',

    enter: function enter(router) {
      console.time('step7 enter');
      var controller = router.get('installerController');
      controller.setCurrentStep('7');
      console.timeEnd('step7 enter');
    },

    connectOutlets: function connectOutlets(router, context) {
      console.time('step7 connectOutlets');
      var self = this;
      var controller = router.get('installerController');
      router.get('preInstallChecksController').loadStep();
      var wizardStep7Controller = router.get('wizardStep7Controller');
      controller.loadAllPriorSteps().done(function () {
        wizardStep7Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep7', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step7 connectOutlets');
      });
    },

    backTransition: function backTransition(router) {
      console.time('step7 back');
      var step = router.get('installerController.content.skipSlavesStep') ? 'step5' : 'step6';
      var wizardStep7Controller = router.get('wizardStep7Controller');

      var goToPreviousStep = function goToPreviousStep() {
        router.transitionTo(step);
      };

      if (wizardStep7Controller.hasChanges()) {
        wizardStep7Controller.showChangesWarningPopup(goToPreviousStep);
      } else {
        goToPreviousStep();
      }
      console.timeEnd('step7 back');
    },
    next: function next(router) {
      console.time('step7 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var controller = router.get('installerController');
        var wizardStep7Controller = router.get('wizardStep7Controller');
        controller.saveServiceConfigProperties(wizardStep7Controller);
        controller.saveServiceConfigGroups(wizardStep7Controller);
        controller.setDBProperty('recommendationsConfigs', wizardStep7Controller.get('recommendationsConfigs'));
        controller.saveComponentsFromConfigs(controller.get('content.componentsFromConfigs'));
        controller.setDBProperty('recommendationsHostGroup', wizardStep7Controller.get('content.recommendationsHostGroup'));
        controller.setDBProperty('masterComponentHosts', wizardStep7Controller.get('content.masterComponentHosts'));
        App.clusterStatus.setClusterStatus({
          localdb: App.db.data
        });
        router.transitionTo('step8');
        console.timeEnd('step7 next');
      }
    }
  }),

  step8: App.StepRoute.extend({
    route: '/step8',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step8 connectOutlets');
      var controller = router.get('installerController');
      var self = this;
      controller.setCurrentStep('8');
      controller.loadAllPriorSteps().done(function () {
        var wizardStep8Controller = router.get('wizardStep8Controller');
        wizardStep8Controller.set('wizardController', controller);
        controller.connectOutlet('wizardStep8', controller.get('content'));
        self.scrollTop();
        console.timeEnd('step8 connectOutlets');
      });
    },
    backTransition: function backTransition(router) {
      if (router.get('wizardStep8Controller.isBackBtnDisabled') == false) {
        router.transitionTo('step7');
      }
    },
    next: function next(router) {
      console.time('step8 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var installerController = router.get('installerController');
        var wizardStep8Controller = router.get('wizardStep8Controller');
        // invoke API call to install selected services
        installerController.installServices(false, function () {
          installerController.setInfoForStep9();
          // We need to do recovery based on whether we are in Add Host or Installer wizard
          installerController.saveClusterState('CLUSTER_INSTALLING_3');
          wizardStep8Controller.set('servicesInstalled', true);
          router.transitionTo('step9');
          console.timeEnd('step8 next');
        });
      }
    }
  }),

  step9: Em.Route.extend({
    route: '/step9',
    connectOutlets: function connectOutlets(router, context) {
      console.time('step9 connectOutlets');
      var self = this;
      var controller = router.get('installerController'),
          wizardStep9Controller = router.get('wizardStep9Controller');
      controller.loadAllPriorSteps().done(function () {
        wizardStep9Controller.loadDoServiceChecksFlag().done(function () {
          controller.setCurrentStep('9');
          if (!App.get('testMode')) {
            controller.setLowerStepsDisable(9);
          }
          wizardStep9Controller.set('wizardController', controller);
          controller.connectOutlet('wizardStep9', controller.get('content'));
          self.scrollTop();
          console.timeEnd('step9 connectOutlets');
        });
      });
    },
    back: Em.Router.transitionTo('step8'),
    retry: function retry(router) {
      console.time('step9 retry');
      var installerController = router.get('installerController');
      var wizardStep9Controller = router.get('wizardStep9Controller');
      if (wizardStep9Controller.get('showRetry')) {
        if (wizardStep9Controller.get('content.cluster.status') === 'INSTALL FAILED') {
          var isRetry = true;
          installerController.installServices(isRetry, function () {
            installerController.setInfoForStep9();
            wizardStep9Controller.resetHostsForRetry();
            // We need to do recovery based on whether we are in Add Host or Installer wizard
            installerController.saveClusterState('CLUSTER_INSTALLING_3');
            wizardStep9Controller.navigateStep();
          });
        } else {
          wizardStep9Controller.navigateStep();
        }
        console.timeEnd('step9 retry');
      }
    },
    unroutePath: function unroutePath(router, context) {
      // exclusion for transition to Admin view or Views view
      if (context === '/adminView' || context === '/main/views.index' || context === '/main/view.index') {
        this._super(router, context);
      } else {
        return false;
      }
    },
    next: function next(router) {
      console.time('step9 next');
      if (!router.get('btnClickInProgress')) {
        App.set('router.nextBtnClickInProgress', true);
        var installerController = router.get('installerController');
        var wizardStep9Controller = router.get('wizardStep9Controller');
        installerController.saveInstalledHosts(wizardStep9Controller);
        installerController.saveClusterState('CLUSTER_INSTALLED_4');
        router.transitionTo('step10');
        console.timeEnd('step9 next');
      }
    }
  }),

  step10: Em.Route.extend({
    route: '/step10',
    connectOutlets: function connectOutlets(router, context) {
      var self = this;
      var controller = router.get('installerController');
      controller.loadAllPriorSteps().done(function () {
        if (!App.get('testMode')) {
          controller.setCurrentStep('10');
          controller.setLowerStepsDisable(10);
        }
        controller.connectOutlet('wizardStep10', controller.get('content'));
        self.scrollTop();
      });
    },
    back: Em.Router.transitionTo('step9'),
    complete: function complete(router, context) {
      var controller = router.get('installerController');
      controller.finish();
      controller.setClusterProvisioningState('INSTALLED').complete(function () {
        // We need to do recovery based on whether we are in Add Host or Installer wizard
        controller.saveClusterState('DEFAULT');
        App.router.set('clusterController.isLoaded', false);
        router.set('clusterInstallCompleted', true);
        router.transitionTo('main.dashboard.index');
      });
    }
  }),

  gotoStep0: Em.Router.transitionTo('step0'),

  gotoStep1: Em.Router.transitionTo('step1'),

  gotoStep2: Em.Router.transitionTo('step2'),

  gotoStep3: Em.Router.transitionTo('step3'),

  gotoStep4: Em.Router.transitionTo('step4'),

  gotoStep5: Em.Router.transitionTo('step5'),

  gotoStep6: Em.Router.transitionTo('step6'),

  gotoStep7: Em.Router.transitionTo('step7'),

  gotoStep8: Em.Router.transitionTo('step8'),

  gotoStep9: Em.Router.transitionTo('step9'),

  gotoStep10: Em.Router.transitionTo('step10')

});

});

require.register("routes/main", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = Em.Route.extend(App.RouterRedirections, {

  breadcrumbs: {
    label: '<span class="glyphicon glyphicon-home"></span>',
    route: 'dashboard'
  },

  route: '/main',
  enter: function enter(router) {
    App.db.updateStorage();
    var self = this;
    var location = router.location.location.hash;
    var clusterController = App.router.get('clusterController');

    router.getAuthenticated().done(function (loggedIn) {
      if (loggedIn) {
        var applicationController = App.router.get('applicationController');
        App.router.get('experimentalController').loadSupports().complete(function () {
          applicationController.startKeepAlivePoller();
          clusterController.loadAmbariProperties().complete(function () {
            App.router.get('mainViewsController').loadAmbariViews();
            clusterController.loadClusterName(false).done(function () {
              $('#main').removeClass('install-wizard-content');
              if (App.get('testMode')) {
                router.get('mainController').initialize();
              } else {
                if (router.get('clusterInstallCompleted')) {
                  if (!App.get('isOnlyViewUser')) {
                    clusterController.checkDetailedRepoVersion().done(function () {
                      router.get('mainController').initialize();
                    });
                  } else {
                    // Don't transit to Views when user already on View page
                    if (App.router.currentState.name !== 'viewDetails' && App.router.currentState.name !== 'shortViewDetails') {
                      App.router.transitionTo('main.views.index');
                    }
                    clusterController.set('isLoaded', true); // hide loading bar
                  }
                } else {
                  Em.run.next(function () {
                    App.clusterStatus.updateFromServer().complete(function () {
                      var currentClusterStatus = App.clusterStatus.get('value');
                      if (router.get('currentState.parentState.name') !== 'views' && router.get('currentState.parentState.name') !== 'view' && currentClusterStatus && self.get('installerStatuses').contains(currentClusterStatus.clusterState)) {
                        if (App.isAuthorized('AMBARI.ADD_DELETE_CLUSTERS')) {
                          self.redirectToInstaller(router, currentClusterStatus, false);
                        } else {
                          clusterController.set('isLoaded', true);
                          Em.run.next(function () {
                            App.router.transitionTo('main.views.index');
                          });
                        }
                      } else {
                        clusterController.set('isLoaded', true);
                      }
                    });
                  });
                }
              }
            });
          });
          // TODO: redirect to last known state
        });
      } else {
        router.savePreferedPath(location);
        Em.run.next(function () {
          router.transitionTo('login');
        });
      }
    });
  },
  /*
   routePath: function(router,event) {
   if (router.getAuthenticated()) {
   App.router.get('clusterController').loadClusterName(false);
   router.get('mainController').initialize();
   // TODO: redirect to last known state
   } else {
   Ember.run.next(function () {
   router.transitionTo('login');
   });
   }
   }, */

  index: Ember.Route.extend({
    route: '/',
    redirectsTo: 'dashboard.index'
  }),

  connectOutlets: function connectOutlets(router, context) {
    router.get('applicationController').connectOutlet('main');
  },

  test: Em.Route.extend({
    route: '/test',
    connectOutlets: function connectOutlets(router, context) {
      router.get('mainController').connectOutlet('mainTest');
    }
  }),

  dashboard: Em.Route.extend({

    breadcrumbs: {
      label: Em.I18n.t('menu.item.dashboard'),
      route: 'dashboard'
    },

    route: '/dashboard',
    connectOutlets: function connectOutlets(router, context) {
      router.get('mainController').connectOutlet('mainDashboard');
    },
    index: Em.Route.extend({
      route: '/',
      enter: function enter(router) {
        Em.run.next(function () {
          router.transitionTo('main.dashboard.widgets');
        });
      }
    }),
    goToDashboardView: function goToDashboardView(router, event) {
      router.transitionTo(event.context);
    },
    widgets: Em.Route.extend({
      route: '/metrics',
      breadcrumbs: {
        label: Em.I18n.t('common.metrics')
      },
      connectOutlets: function connectOutlets(router, context) {
        App.loadTimer.start('Dashboard Metrics Page');
        router.set('mainDashboardController.selectedCategory', 'widgets');
        router.get('mainDashboardController').connectOutlet('mainDashboardWidgets');
      }
    }),
    charts: Em.Route.extend({
      route: '/charts',
      breadcrumbs: null,
      connectOutlets: function connectOutlets(router, context) {
        App.loadTimer.start('Heatmaps Page');
        router.set('mainDashboardController.selectedCategory', 'charts');
        router.get('mainDashboardController').connectOutlet('mainCharts');
      },
      index: Ember.Route.extend({
        route: '/',
        enter: function enter(router) {
          Em.run.next(function () {
            router.transitionTo('heatmap');
          });
        }
      }),
      heatmap: Em.Route.extend({
        route: '/heatmap',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainController').dataLoading().done(function () {
            router.get('mainChartsController').connectOutlet('mainChartsHeatmap');
          });
        }
      }),
      horizon_chart: Em.Route.extend({
        route: '/horizon_chart',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainChartsController').connectOutlet('mainChartsHorizon');
        }
      }),
      showChart: function showChart(router, event) {
        var parent = event.view._parentView;
        parent.deactivateChildViews();
        event.view.set('active', "active");
        router.transitionTo(event.context);
      }
    }),
    configHistory: Em.Route.extend({
      route: '/config_history',
      connectOutlets: function connectOutlets(router, context) {
        App.loadTimer.start('Config History Page');
        router.set('mainDashboardController.selectedCategory', 'configHistory');
        router.get('mainDashboardController').connectOutlet('mainConfigHistory');
      }
    }),
    goToServiceConfigs: function goToServiceConfigs(router, event) {
      router.get('mainServiceItemController').set('routeToConfigs', true);
      router.get('mainServiceInfoConfigsController').set('preSelectedConfigVersion', event.context);
      router.transitionTo('main.services.service.configs', App.Service.find(event.context.get('serviceName')));
      router.get('mainServiceItemController').set('routeToConfigs', false);
    }
  }),

  views: require('routes/views'),
  view: require('routes/view'),

  hosts: Em.Route.extend({

    breadcrumbs: {
      label: Em.I18n.t('menu.item.hosts'),
      route: 'hosts',
      beforeTransition: function beforeTransition() {
        App.router.set('mainHostController.showFilterConditionsFirstLoad', true);
        App.router.set('mainHostController.saveSelection', true);
      }
    },

    route: '/hosts',
    index: Ember.Route.extend({
      route: '/',
      connectOutlets: function connectOutlets(router, context) {
        App.loadTimer.start('Hosts Page');
        router.get('mainController').connectOutlet('mainHost');
      }
    }),

    hostDetails: Em.Route.extend({

      breadcrumbs: {
        itemView: Em.View.extend({
          tagName: "a",
          contentBinding: 'App.router.mainHostDetailsController.content',
          isActive: Em.computed.equal('content.passiveState', 'OFF'),
          click: function click() {
            App.router.transitionTo('hosts.hostDetails.summary', this.get('content'));
          },
          template: Em.Handlebars.compile('<span class="host-breadcrumb">{{view.content.hostName}}</span>' + '<span rel="HealthTooltip" {{bindAttr class="view.content.healthClass view.content.healthIconClass :icon"}} ' + 'data-placement="bottom" {{bindAttr data-original-title="view.content.healthToolTip" }}></span>')
        })
      },

      route: '/:host_id',
      connectOutlets: function connectOutlets(router, host) {
        router.get('mainHostController').set('showFilterConditionsFirstLoad', true);
        router.get('mainController').connectOutlet('mainHostDetails', host);
      },

      index: Ember.Route.extend({
        route: '/',
        redirectsTo: 'summary'
      }),

      summary: Em.Route.extend({
        route: '/summary',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainController').dataLoading().done(function () {
            var controller = router.get('mainHostDetailsController');
            var tags = ['hive-env'];
            if (App.Service.find().mapProperty('serviceName').contains('OOZIE')) {
              controller.loadConfigs('loadOozieConfigs');
              controller.isOozieConfigLoaded.always(function () {
                if (App.Service.find().mapProperty('serviceName').contains('HIVE')) {
                  App.router.get('configurationController').getCurrentConfigsBySites(tags).always(function () {
                    controller.connectOutlet('mainHostSummary');
                  });
                } else controller.connectOutlet('mainHostSummary');
              });
            } else if (App.Service.find().mapProperty('serviceName').contains('HIVE')) {
              App.router.get('configurationController').getCurrentConfigsBySites(tags).always(function () {
                controller.connectOutlet('mainHostSummary');
              });
            } else {
              controller.connectOutlet('mainHostSummary');
            }
          });
        }
      }),

      configs: Em.Route.extend({
        route: '/configs',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainController').isLoading.call(router.get('clusterController'), 'isConfigsPropertiesLoaded').done(function () {
            router.get('mainHostDetailsController').connectOutlet('mainHostConfigs');
          });
        },
        exitRoute: function exitRoute(router, context, callback) {
          callback();
        }
      }),

      alerts: Em.Route.extend({
        route: '/alerts',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainHostDetailsController').connectOutlet('mainHostAlerts');
        },
        exit: function exit(router) {
          router.set('mainAlertInstancesController.isUpdating', false);
        }
      }),

      metrics: Em.Route.extend({
        route: '/metrics',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainHostDetailsController').connectOutlet('mainHostMetrics');
        }
      }),

      stackVersions: Em.Route.extend({
        breadcrumbs: {
          label: Em.I18n.t('common.versions')
        },
        route: '/stackVersions',
        connectOutlets: function connectOutlets(router, context) {
          if (App.get('stackVersionsAvailable')) {
            router.get('mainHostDetailsController').connectOutlet('mainHostStackVersions');
          } else {
            router.transitionTo('summary');
          }
        }
      }),

      logs: Em.Route.extend({
        route: '/logs:query',
        connectOutlets: function connectOutlets(router, context) {
          if (App.get('supports.logSearch')) {
            router.get('mainHostDetailsController').connectOutlet('mainHostLogs');
          } else {
            router.transitionTo('summary');
          }
        },
        serialize: function serialize(router, params) {
          return this.serializeQueryParams(router, params, 'mainHostDetailsController');
        }
      }),

      hostNavigate: function hostNavigate(router, event) {
        var parent = event.view._parentView;
        parent.deactivateChildViews();
        event.view.set('active', "active");
        router.transitionTo(event.context);
      }
    }),

    back: function back(router, event) {
      var referer = router.get('mainHostDetailsController.referer');
      if (referer) {
        router.route(referer);
      } else {
        window.history.back();
      }
    },

    addHost: function addHost(router) {
      router.transitionTo('hostAdd');
    },

    exit: function exit(router) {
      router.set('mainHostController.saveSelection', false);
    }

  }),

  hostAdd: require('routes/add_host_routes'),

  alerts: Em.Route.extend({

    breadcrumbs: {
      label: Em.I18n.t('menu.item.alerts'),
      route: 'alerts',
      beforeTransition: function beforeTransition() {
        App.router.set('mainAlertDefinitionsController.showFilterConditionsFirstLoad', false);
      }
    },

    route: '/alerts',
    index: Em.Route.extend({
      route: '/',
      connectOutlets: function connectOutlets(router, context) {
        router.get('mainController').connectOutlet('mainAlertDefinitions');
      }
    }),

    alertDetails: Em.Route.extend({

      breadcrumbs: {
        labelBindingPath: 'App.router.mainAlertDefinitionDetailsController.content.label',
        disabled: true
      },

      route: '/:alert_definition_id',

      connectOutlets: function connectOutlets(router, alertDefinition) {
        App.router.set('mainAlertDefinitionsController.showFilterConditionsFirstLoad', true);
        router.get('mainController').connectOutlet('mainAlertDefinitionDetails', alertDefinition);
      },

      exit: function exit(router) {
        router.set('mainAlertInstancesController.isUpdating', false);
      },

      exitRoute: function exitRoute(router, context, callback) {
        var controller = router.get('mainAlertDefinitionDetailsController');
        if (App.router.get('clusterController.isLoaded') && controller.get('isEditing')) {
          controller.showSavePopup(callback);
        } else {
          callback();
        }
      }
    }),

    back: function back(router, event) {
      window.history.back();
    }
  }),

  alertAdd: require('routes/add_alert_definition_routes'),

  admin: Em.Route.extend({
    route: '/admin',
    breadcrumbs: {
      disabled: true
    },
    enter: function enter(router, transition) {
      if (router.get('loggedIn') && !App.isAuthorized('CLUSTER.TOGGLE_KERBEROS, SERVICE.SET_SERVICE_USERS_GROUPS, CLUSTER.UPGRADE_DOWNGRADE_STACK, CLUSTER.VIEW_STACK_DETAILS') && !(App.get('upgradeInProgress') || App.get('upgradeHolding'))) {
        Em.run.next(function () {
          router.transitionTo('main.dashboard.index');
        });
      }
    },

    routePath: function routePath(router, event) {
      if (!App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK') && !(App.get('upgradeInProgress') || App.get('upgradeHolding'))) {
        Em.run.next(function () {
          App.router.transitionTo('main.dashboard.index');
        });
      } else {
        this._super(router, event);
      }
    },
    connectOutlets: function connectOutlets(router, context) {
      router.get('mainController').connectOutlet('mainAdmin');
    },

    index: Em.Route.extend({
      route: '/',
      redirectsTo: 'stackAndUpgrade.index'
    }),

    adminAuthentication: Em.Route.extend({
      route: '/authentication',
      connectOutlets: function connectOutlets(router, context) {
        router.set('mainAdminController.category', "authentication");
        router.get('mainAdminController').connectOutlet('mainAdminAuthentication');
      }
    }),

    adminKerberos: Em.Route.extend({

      breadcrumbs: {
        label: Em.I18n.t('common.kerberos')
      },

      route: '/kerberos',
      enter: function enter(router, transition) {
        if (router.get('loggedIn') && (!App.isAuthorized('CLUSTER.TOGGLE_KERBEROS') || !App.supports.enableToggleKerberos)) {
          router.transitionTo('main.dashboard.index');
        }
      },
      index: Em.Route.extend({
        route: '/',
        connectOutlets: function connectOutlets(router, context) {
          router.set('mainAdminController.category', "kerberos");
          router.set('mainAdminController.categoryLabel', Em.I18n.t('common.kerberos'));
          router.get('mainAdminController').connectOutlet('mainAdminKerberos');
        }
      }),
      adminAddKerberos: require('routes/add_kerberos_routes'),

      disableSecurity: Em.Route.extend({
        route: '/disableSecurity',
        enter: function enter(router) {
          App.router.get('updateController').set('isWorking', false);
          router.get('mainController').dataLoading().done(function () {
            App.ModalPopup.show({
              classNames: ['wizard-modal-wrapper', 'disable-security-modal'],
              modalDialogClasses: ['modal-xlg'],
              header: Em.I18n.t('admin.removeSecurity.header'),
              bodyClass: App.KerberosDisableView.extend({
                controllerBinding: 'App.router.kerberosDisableController'
              }),
              primary: Em.I18n.t('common.complete'),
              secondary: null,
              disablePrimary: Em.computed.alias('App.router.kerberosDisableController.isSubmitDisabled'),

              onPrimary: function onPrimary() {
                this.onClose();
              },


              onClose: function onClose() {
                var self = this;
                var controller = router.get('kerberosDisableController');
                if (!controller.get('isSubmitDisabled')) {
                  self.proceedOnClose();
                  return;
                }
                var unkerberizeCommand = controller.get('tasks').findProperty('command', 'unkerberize') || Em.Object.create();
                var isUnkerberizeInProgress = unkerberizeCommand.get('status') === 'IN_PROGRESS';
                if (controller.get('tasks').everyProperty('status', 'COMPLETED')) {
                  self.proceedOnClose();
                  return;
                }
                // user cannot exit wizard during removing kerberos
                if (isUnkerberizeInProgress) {
                  App.showAlertPopup(Em.I18n.t('admin.kerberos.disable.unkerberize.header'), Em.I18n.t('admin.kerberos.disable.unkerberize.message'));
                  return;
                }
                App.showConfirmationPopup(function () {
                  self.proceedOnClose();
                }, Em.I18n.t('admin.security.disable.onClose'));
              },
              proceedOnClose: function proceedOnClose() {
                var self = this;
                var disableController = router.get('kerberosDisableController');
                disableController.clearStep();
                disableController.resetDbNamespace();
                App.db.setSecurityDeployCommands(undefined);
                App.router.get('updateController').set('isWorking', true);
                router.get('mainAdminKerberosController').setDisableSecurityStatus(undefined);
                router.get('addServiceController').finish();
                App.clusterStatus.setClusterStatus({
                  clusterName: router.get('content.cluster.name'),
                  clusterState: 'DEFAULT',
                  localdb: App.db.data
                }, {
                  alwaysCallback: function alwaysCallback() {
                    self.hide();
                    router.transitionTo('adminKerberos.index');
                    Em.run.next(function () {
                      location.reload();
                    });
                  }
                });
              },
              didInsertElement: function didInsertElement() {
                this._super();
                this.fitHeight();
              }
            });
          });
        },

        unroutePath: function unroutePath() {
          return false;
        },
        next: function next(router, context) {
          $("#modal").find(".close").trigger('click');
        },
        done: function done(router, context) {
          var controller = router.get('kerberosDisableController');
          if (!controller.get('isSubmitDisabled')) {
            $(context.currentTarget).parents("#modal").find(".close").trigger('click');
          }
        }
      })
    }),

    stackAndUpgrade: Em.Route.extend({
      route: '/stack',
      breadcrumbs: null,
      connectOutlets: function connectOutlets(router) {
        router.set('mainAdminController.category', "stackAndUpgrade");
        router.set('mainAdminController.categoryLabel', Em.I18n.t('admin.stackUpgrade.title'));
        router.get('mainAdminController').connectOutlet('mainAdminStackAndUpgrade');
      },

      index: Em.Route.extend({
        route: '/',
        redirectsTo: 'services'
      }),

      services: Em.Route.extend({

        breadcrumbs: {
          label: Em.I18n.t('common.stack')
        },

        route: '/services',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainAdminStackAndUpgradeController').connectOutlet('mainAdminStackServices');
        }
      }),

      versions: Em.Route.extend({
        breadcrumbs: {
          label: Em.I18n.t('common.versions')
        },
        route: '/versions',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainAdminStackAndUpgradeController').connectOutlet('MainAdminStackVersions');
        }
      }),

      upgradeHistory: Em.Route.extend({

        breadcrumbs: {
          label: Em.I18n.t('common.upgrade.history')
        },

        route: '/history',
        connectOutlets: function connectOutlets(router, context) {
          router.get('mainAdminStackAndUpgradeController').connectOutlet('mainAdminStackUpgradeHistory');
        }
      }),

      stackNavigate: function stackNavigate(router, event) {
        var parent = event.view._parentView;
        parent.deactivateChildViews();
        event.view.set('active', "active");
        router.transitionTo(event.context);
      }
    }),
    stackUpgrade: require('routes/stack_upgrade_routes'),

    adminAdvanced: Em.Route.extend({
      route: '/advanced',
      connectOutlets: function connectOutlets(router) {
        router.set('mainAdminController.category', "advanced");
        router.get('mainAdminController').connectOutlet('mainAdminAdvanced');
      }
    }),
    adminServiceAccounts: Em.Route.extend({

      breadcrumbs: {
        label: Em.I18n.t('common.serviceAccounts')
      },

      route: '/serviceAccounts',
      enter: function enter(router, transition) {
        if (router.get('loggedIn') && !App.isAuthorized('SERVICE.SET_SERVICE_USERS_GROUPS')) {
          router.transitionTo('main.dashboard.index');
        }
      },
      connectOutlets: function connectOutlets(router) {
        router.set('mainAdminController.category', "adminServiceAccounts");
        router.set('mainAdminController.categoryLabel', Em.I18n.t('common.serviceAccounts'));
        router.get('mainAdminController').connectOutlet('mainAdminServiceAccounts');
      }
    }),

    adminServiceAutoStart: Em.Route.extend({

      breadcrumbs: {
        label: Em.I18n.t('admin.serviceAutoStart.title')
      },

      route: '/serviceAutoStart',
      enter: function enter(router, transition) {
        if (router.get('loggedIn') && !App.isAuthorized('CLUSTER.MANAGE_AUTO_START') && !App.isAuthorized('SERVICE.MANAGE_AUTO_START')) {
          router.transitionTo('main.dashboard.index');
        }
      },
      connectOutlets: function connectOutlets(router) {
        router.set('mainAdminController.category', "serviceAutoStart");
        router.set('mainAdminController.categoryLabel', Em.I18n.t('admin.serviceAutoStart.title'));
        router.get('mainAdminController').connectOutlet('mainAdminServiceAutoStart');
      },
      exitRoute: function exitRoute(router, context, callback) {
        var controller = router.get('mainAdminServiceAutoStartController');
        if (controller.get('isModified')) {
          controller.showSavePopup(callback);
        } else {
          callback();
        }
      }
    }),

    adminAudit: Em.Route.extend({
      route: '/audit',
      connectOutlets: function connectOutlets(router) {
        router.set('mainAdminController.category', "audit");
        router.get('mainAdminController').connectOutlet('mainAdminAudit');
      }
    }),
    upgradeStack: function upgradeStack(router, event) {
      if (!$(event.currentTarget).hasClass('inactive')) {
        router.transitionTo('stackUpgrade');
      }
    },

    adminNavigate: function adminNavigate(router, object) {
      router.transitionTo('admin' + object.context.capitalize());
    },

    //events
    goToAdmin: function goToAdmin(router, event) {
      var isDisabled = !!event.context.disabled;
      if (!isDisabled) {
        router.transitionTo(event.context.url);
      }
    }

  }),

  createServiceWidget: function createServiceWidget(router, context) {
    if (context) {
      var widgetController = router.get('widgetWizardController');
      widgetController.save('widgetService', context.get('serviceName'));
      var layout = JSON.parse(JSON.stringify(context.get('layout')));
      layout.widgets = context.get('layout.widgets').mapProperty('id');
      widgetController.save('layout', layout);
    }
    router.transitionTo('createWidget');
  },

  createWidget: require('routes/create_widget'),

  editServiceWidget: function editServiceWidget(router, context) {
    if (context) {
      var widgetController = router.get('widgetEditController');
      widgetController.save('widgetService', context.get('serviceName'));
      widgetController.save('widgetType', context.get('widgetType'));
      widgetController.save('widgetProperties', context.get('properties'));
      widgetController.save('widgetMetrics', context.get('metrics'));
      widgetController.save('widgetValues', context.get('values'));
      widgetController.save('widgetName', context.get('widgetName'));
      widgetController.save('widgetDescription', context.get('description'));
      widgetController.save('widgetScope', context.get('scope'));
      widgetController.save('widgetAuthor', context.get('author'));
      widgetController.save('widgetId', context.get('id'));
      widgetController.save('allMetrics', []);
    }
    router.transitionTo('editWidget');
  },

  editWidget: require('routes/edit_widget'),

  services: Em.Route.extend({

    breadcrumbs: {
      disabled: true
    },

    route: '/services',
    index: Em.Route.extend({
      route: '/',
      enter: function enter(router) {
        Em.run.next(function () {
          var controller = router.get('mainController');
          controller.dataLoading().done(function () {
            if (router.currentState.parentState.name === 'services' && router.currentState.name === 'index') {
              var service = router.get('mainServiceItemController.content');
              if (!service || !service.get('isLoaded')) {
                service = App.Service.find().objectAt(0); // getting the first service to display
              }
              if (router.get('mainServiceItemController').get('routeToConfigs')) {
                router.transitionTo('service.configs', service);
              } else if (router.get('mainServiceItemController.routeToHeatmaps')) {
                router.transitionTo('service.heatmaps', service);
              } else {
                router.transitionTo('service.summary', service);
              }
            }
          });
        });
      }
    }),
    connectOutlets: function connectOutlets(router, context) {
      router.get('mainController').connectOutlet('mainService');
    },
    service: Em.Route.extend({
      route: '/:service_id',
      breadcrumbs: {
        labelBindingPath: 'App.router.mainServiceItemController.content.displayName',
        disabled: true
      },
      connectOutlets: function connectOutlets(router, service) {
        router.get('mainServiceController').connectOutlet('mainServiceItem', service);
        if (service.get('isLoaded')) {
          if (router.get('mainServiceItemController').get('routeToConfigs')) {
            router.transitionTo('configs');
          } else if (router.get('mainServiceItemController.routeToHeatmaps')) {
            router.transitionTo('heatmaps');
          } else {
            router.transitionTo('summary');
          }
        } else {
          router.transitionTo('index');
        }
      },
      index: Ember.Route.extend({
        route: '/'
      }),
      summary: Em.Route.extend({
        route: '/summary',
        connectOutlets: function connectOutlets(router, context) {
          App.loadTimer.start('Service Summary Page');
          var item = router.get('mainServiceItemController.content');
          if (router.get('clusterController.isServiceMetricsLoaded')) router.get('updateController').updateServiceMetric(Em.K);
          //if service is not existed then route to default service
          if (item.get('isLoaded')) {
            router.get('mainServiceItemController').connectOutlet('mainServiceInfoSummary', item);
          } else {
            router.transitionTo('services.index');
          }
        }
      }),
      metrics: Em.Route.extend({
        route: '/metrics',
        connectOutlets: function connectOutlets(router, context) {
          var item = router.get('mainServiceItemController.content');
          router.get('mainServiceItemController').connectOutlet('mainServiceInfoMetrics', item);
        }
      }),
      configs: Em.Route.extend({
        route: '/configs',
        connectOutlets: function connectOutlets(router, context) {
          App.loadTimer.start('Service Configs Page');
          router.get('mainController').dataLoading().done(function () {
            var item = router.get('mainServiceItemController.content');
            //if service is not existed then route to default service
            if (item.get('isLoaded')) {
              if (router.get('mainServiceItemController.isConfigurable')) {
                router.get('mainServiceItemController').connectOutlet('mainServiceInfoConfigs', item);
              } else {
                // if service doesn't have configs redirect to summary
                router.transitionTo('summary');
              }
            } else {
              item.set('routeToConfigs', true);
              router.transitionTo('services.index');
            }
          });
        },
        exitRoute: function exitRoute(router, nextRoute, callback) {
          var controller = router.get('mainServiceInfoConfigsController');
          // If another user is running some wizard, current user can't save configs
          if (controller.hasUnsavedChanges() && !router.get('wizardWatcherController.isWizardRunning')) {
            controller.showSavePopup(callback);
          } else {
            callback();
          }
        }
      }),
      heatmaps: Em.Route.extend({
        route: '/heatmaps',
        connectOutlets: function connectOutlets(router, context) {
          App.loadTimer.start('Service Heatmaps Page');
          router.get('mainController').dataLoading().done(function () {
            var item = router.get('mainServiceItemController.content');
            if (item.get('isLoaded')) {
              router.get('mainServiceItemController').connectOutlet('mainServiceInfoHeatmap', item);
            } else {
              item.set('routeToHeatmaps', true);
              router.transitionTo('services.index');
            }
          });
        }
      }),
      audit: Em.Route.extend({
        route: '/audit',
        connectOutlets: function connectOutlets(router, context) {
          var item = router.get('mainServiceItemController.content');
          router.get('mainServiceItemController').connectOutlet('mainServiceInfoAudit', item);
        }
      }),
      showInfo: function showInfo(router, event) {
        router.transitionTo(event.context);
      }
    }),
    showService: Em.Router.transitionTo('service'),
    addService: Em.Router.transitionTo('serviceAdd'),
    reassign: Em.Router.transitionTo('reassign'),

    enableHighAvailability: require('routes/high_availability_routes'),

    manageJournalNode: require('routes/manage_journalnode_routes'),

    enableRMHighAvailability: require('routes/rm_high_availability_routes'),

    enableRAHighAvailability: require('routes/ra_high_availability_routes'),

    enableNameNodeFederation: require('routes/namenode_federation_routes'),

    addHawqStandby: require('routes/add_hawq_standby_routes'),

    removeHawqStandby: require('routes/remove_hawq_standby_routes'),

    activateHawqStandby: require('routes/activate_hawq_standby_routes'),

    rollbackHighAvailability: require('routes/rollbackHA_routes')
  }),

  reassign: require('routes/reassign_master_routes'),

  serviceAdd: require('routes/add_service_routes'),

  selectService: Em.Route.transitionTo('services.service.summary'),
  selectHost: function selectHost(router, event) {
    router.get('mainHostDetailsController').set('isFromHosts', false);
    router.transitionTo('hosts.hostDetails.index', event.context);
  },
  filterHosts: function filterHosts(router, component) {
    if (!component.context) return;
    router.get('mainHostController').filterByComponent(component.context);
    router.get('mainHostController').set('showFilterConditionsFirstLoad', true);
    router.get('mainHostController').set('filterChangeHappened', true);
    router.transitionTo('hosts.index');
  },
  showDetails: function showDetails(router, event) {
    router.get('mainHostDetailsController').set('referer', router.location.lastSetURL);
    router.get('mainHostDetailsController').set('isFromHosts', true);
    router.transitionTo('hosts.hostDetails.summary', event.context);
  },
  gotoAlertDetails: function gotoAlertDetails(router, event) {
    router.transitionTo('alerts.alertDetails', event.context);
  },

  /**
   * Open summary page of the selected service
   * @param {object} event
   * @method routeToService
   */
  routeToService: function routeToService(router, event) {
    var service = event.context;
    router.transitionTo('main.services.service.summary', service);
  }
});

});

require.register("routes/manage_journalnode_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/JournalNode/manage',

  breadcrumbs: {
    label: Em.I18n.t('admin.manageJournalNode.wizard.header')
  },

  enter: function enter(router) {
    var manageJournalNodeWizardController = router.get('manageJournalNodeWizardController');
    manageJournalNodeWizardController.dataLoading().done(function () {
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.manageJournalNode.wizard.header'),
        bodyClass: App.ManageJournalNodeWizardView.extend({
          controller: manageJournalNodeWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,
        hideCloseButton: function () {
          var currStep = App.router.get('manageJournalNodeWizardController.currentStep');
          switch (currStep) {
            default:
              this.set('showCloseButton', true);
          }
        }.observes('App.router.manageJournalNodeWizardController.currentStep'),

        onClose: function onClose() {
          var controller = App.router.get('manageJournalNodeWizardController');
          controller.clearTasksData();
          controller.resetOnClose(controller, 'main.services.index');
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      manageJournalNodeWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'JOURNALNODE_MANAGEMENT':
            manageJournalNodeWizardController.setCurrentStep(currentClusterStatus.localdb.ManageJournalNodeWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('manageJournalNodeWizardController.currentStep');
            manageJournalNodeWizardController.setCurrentStep(currStep);
            break;
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(manageJournalNodeWizardController.get('name'));
        router.transitionTo('step' + manageJournalNodeWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep1', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('manageJournalNodeWizardController');
      var step1Controller = router.get('manageJournalNodeWizardStep1Controller');
      controller.saveMasterComponentHosts(step1Controller);
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.setLowerStepsDisable(2);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep2', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      App.set('router.nextBtnClickInProgress', true);
      var controller = router.get('manageJournalNodeWizardController');
      var stepController = router.get('manageJournalNodeWizardStep2Controller');
      controller.saveServiceConfigProperties(stepController);
      controller.saveConfigTag(stepController.get("hdfsSiteTag"));
      controller.saveNameServiceIds(stepController.get('content.nameServiceIds'));
      App.set('router.nextBtnClickInProgress', false);
      if (controller.get('isDeleteOnly')) {
        router.transitionTo('step4');
      } else {
        router.transitionTo('step3');
      }
    },
    back: Em.Router.transitionTo('step1')
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.setLowerStepsDisable(3);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep3', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.clearTasksData();
      router.transitionTo('step4');
    }
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep4', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.clearTasksData();
      if (controller.get('isDeleteOnly')) {
        router.transitionTo('step6');
      } else {
        router.transitionTo('step5');
      }
    }
  }),

  step5: Em.Route.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('5');
        controller.setLowerStepsDisable(5);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep5', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      router.transitionTo('step6');
    }
  }),

  step6: Em.Route.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('6');
        controller.setLowerStepsDisable(6);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep6', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.clearTasksData();
      router.transitionTo('step7');
    }
  }),

  step7: Em.Route.extend({
    route: '/step7',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('7');
        controller.setLowerStepsDisable(7);
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('manageJournalNodeWizardStep7', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('manageJournalNodeWizardController');
      controller.clearTasksData();
      controller.resetOnClose(controller, 'main.services.index');
    }
  })
});

});

require.register("routes/namenode_federation_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/NameNode/federation',

  breadcrumbs: {
    label: Em.I18n.t('admin.nameNodeFederation.wizard.header')
  },

  enter: function enter(router, transition) {
    var nameNodeFederationWizardController = router.get('nameNodeFederationWizardController');
    nameNodeFederationWizardController.dataLoading().done(function () {
      //Set HDFS as current service
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'HDFS'));
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.nameNodeFederation.wizard.header'),
        bodyClass: App.NameNodeFederationWizardView.extend({
          controller: nameNodeFederationWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          var nameNodeFederationWizardController = router.get('nameNodeFederationWizardController'),
              currStep = nameNodeFederationWizardController.get('currentStep');
          App.showConfirmationPopup(function () {
            nameNodeFederationWizardController.resetOnClose(nameNodeFederationWizardController, 'main.services.index');
          }, Em.I18n.t(parseInt(currStep) === 4 ? 'admin.nameNodeFederation.closePopup2' : 'admin.nameNodeFederation.closePopup'));
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      nameNodeFederationWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'NN_FEDERATION_DEPLOY':
            nameNodeFederationWizardController.setCurrentStep(currentClusterStatus.localdb.NameNodeFederationWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('nameNodeFederationWizardController.currentStep');
            nameNodeFederationWizardController.setCurrentStep(currStep);
            break;
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(nameNodeFederationWizardController.get('name'));
        router.transitionTo('step' + nameNodeFederationWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.connectOutlet('nameNodeFederationWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.saveNameServiceId(router.get('nameNodeFederationWizardStep1Controller.content.nameServiceId'));
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('nameNodeFederationWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var wizardController = router.get('nameNodeFederationWizardController');
      var stepController = router.get('nameNodeFederationWizardStep2Controller');
      wizardController.saveMasterComponentHosts(stepController);
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.loadAllPriorSteps();
        controller.connectOutlet('nameNodeFederationWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('nameNodeFederationWizardController');
      var stepController = router.get('nameNodeFederationWizardStep3Controller');
      controller.saveServiceConfigProperties(stepController);
      router.transitionTo('step4');
    },
    back: Em.Router.transitionTo('step2')
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps();
        controller.connectOutlet('nameNodeFederationWizardStep4', controller.get('content'));
      });
    },
    unroutePath: function unroutePath(router, path) {
      // allow user to leave route if wizard has finished
      if (router.get('nameNodeFederationWizardController').get('isFinished')) {
        this._super(router, path);
      } else {
        return false;
      }
    },
    next: function next(router) {
      var controller = router.get('nameNodeFederationWizardController');
      controller.resetOnClose(controller, 'main.services.index');
    }
  })

});

});

require.register("routes/ra_high_availability_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/RangerAdmin/enable',

  breadcrumbs: {
    label: Em.I18n.t('admin.ra_highAvailability.wizard.header')
  },

  enter: function enter(router) {
    var rAHighAvailabilityWizardController = router.get('rAHighAvailabilityWizardController');
    rAHighAvailabilityWizardController.dataLoading().done(function () {
      //Set RANGER as current service
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'RANGER'));
    });
    Em.run.next(function () {
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.ra_highAvailability.wizard.header'),
        bodyClass: App.RAHighAvailabilityWizardView.extend({
          controller: rAHighAvailabilityWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          var rAHighAvailabilityWizardController = router.get('rAHighAvailabilityWizardController'),
              currStep = rAHighAvailabilityWizardController.get('currentStep');
          if (parseInt(currStep) === 4) {
            App.showConfirmationPopup(function () {
              rAHighAvailabilityWizardController.resetOnClose(rAHighAvailabilityWizardController, 'main.services.index');
            }, Em.I18n.t('admin.ra_highAvailability.closePopup'));
          } else {
            rAHighAvailabilityWizardController.resetOnClose(rAHighAvailabilityWizardController, 'main.services.index');
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      rAHighAvailabilityWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'RA_HIGH_AVAILABILITY_DEPLOY':
            rAHighAvailabilityWizardController.setCurrentStep(currentClusterStatus.localdb.RAHighAvailabilityWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('rAHighAvailabilityWizardController.currentStep');
            rAHighAvailabilityWizardController.setCurrentStep(currStep);
            break;
        }
      }
      App.router.get('wizardWatcherController').setUser(rAHighAvailabilityWizardController.get('name'));
      router.transitionTo('step' + rAHighAvailabilityWizardController.get('currentStep'));
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rAHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('rAHighAvailabilityWizardStep1', controller.get('content'));
        });
      });
    },
    next: function next(router) {
      var controller = router.get('rAHighAvailabilityWizardController');
      controller.save('loadBalancerURL');
      controller.clearMasterComponentHosts();
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rAHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps().done(function () {
          controller.connectOutlet('rAHighAvailabilityWizardStep2', controller.get('content'));
        });
      });
    },
    next: function next(router) {
      var wizardController = router.get('rAHighAvailabilityWizardController');
      var stepController = router.get('rAHighAvailabilityWizardStep2Controller');
      var currentRA = stepController.get('servicesMasters').filterProperty('component_name', 'RANGER_ADMIN').findProperty('isInstalled', true);
      var additionalRAs = stepController.get('servicesMasters').filterProperty('component_name', 'RANGER_ADMIN').filterProperty('isInstalled', false);
      var raHosts = {
        currentRA: currentRA.get('selectedHost'),
        additionalRA: additionalRAs.mapProperty('selectedHost')
      };
      wizardController.set('content.raHosts', raHosts);
      wizardController.save('raHosts');
      wizardController.saveMasterComponentHosts(stepController);
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rAHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.loadAllPriorSteps().done(function () {
          var stepController = router.get('rAHighAvailabilityWizardStep3Controller');
          stepController.set('wizardController', controller);
          controller.connectOutlet('rAHighAvailabilityWizardStep3', controller.get('content'));
        });
      });
    },
    next: function next(router) {
      router.transitionTo('step4');
    },
    back: function back(router) {
      router.transitionTo('step2');
    }
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rAHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps().done(function () {
          var stepController = router.get('rAHighAvailabilityWizardStep4Controller');
          stepController.set('wizardController', controller);
          controller.connectOutlet('rAHighAvailabilityWizardStep4', controller.get('content'));
        });
      });
    },
    next: function next(router) {
      router.get('updateController').set('isWorking', true);
      var rAHighAvailabilityWizardController = router.get('rAHighAvailabilityWizardController');
      rAHighAvailabilityWizardController.finish();
      App.clusterStatus.setClusterStatus({
        clusterName: App.router.getClusterName(),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, {
        alwaysCallback: function alwaysCallback() {
          rAHighAvailabilityWizardController.get('popup').hide();
          router.transitionTo('main.services.index');
          Em.run.next(function () {
            location.reload();
          });
        }
      });
    }
  })
});

});

require.register("routes/reassign_master_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/service/reassign',

  leaveWizard: function leaveWizard(router, context) {
    var reassignMasterController = router.get('reassignMasterController');
    reassignMasterController.resetOnClose(reassignMasterController, 'main.index');
  },

  breadcrumbs: {
    labelBindingPath: 'App.router.reassignMasterController.content.reassign.display_name',
    labelPostFormat: function labelPostFormat(label) {
      var msg = Em.I18n.t('services.reassign.header');
      return label ? msg + ' (' + label + ')' : msg;
    }
  },

  enter: function enter(router) {
    var context = this;
    var reassignMasterController = router.get('reassignMasterController');

    reassignMasterController.dataLoading().done(function () {
      if (App.get('allHostNames.length') > 1) {
        Em.run.next(function () {
          App.router.get('updateController').set('isWorking', false);
          var popup = App.ModalPopup.show({
            classNames: ['wizard-modal-wrapper'],
            modalDialogClasses: ['modal-xlg'],
            header: Em.I18n.t('services.reassign.header'),
            bodyClass: App.ReassignMasterView.extend({
              controller: reassignMasterController
            }),
            primary: Em.I18n.t('form.cancel'),
            showFooter: false,
            secondary: null,

            onPrimary: function onPrimary() {
              this.hide();
              App.router.get('updateController').set('isWorking', true);
              App.router.transitionTo('main.services.index');
            },
            onClose: function onClose() {
              var currStep = reassignMasterController.get('currentStep');
              if (parseInt(currStep) > 3) {
                var self = this;

                var step4Controller = router.get('reassignMasterWizardStep4Controller');
                var testDBTaskId = step4Controller.get('tasks').filterProperty('command', 'testDBConnection').get('firstObject.id');

                if (currStep !== "7" && testDBTaskId && reassignMasterController.get('content.tasksStatuses').get(testDBTaskId) === "FAILED") {
                  App.showConfirmationPopup(function () {
                    App.router.transitionTo('step7');
                  }, Em.I18n.t('services.reassign.rollback.confirm'));
                } else {
                  App.showConfirmationPopup(function () {
                    router.get('reassignMasterWizardStep' + currStep + 'Controller').removeObserver('tasks.@each.status', this, 'onTaskStatusChange');
                    context.leaveWizard(router, self);
                  }, Em.I18n.t('services.reassign.closePopup').format(reassignMasterController.get('content.reassign.display_name')));
                }
              } else {
                context.leaveWizard(router, this);
              }
            },
            didInsertElement: function didInsertElement() {
              this._super();
              this.fitHeight();
            }
          });
          reassignMasterController.set('popup', popup);
          reassignMasterController.loadComponentToReassign();
          var currStep = reassignMasterController.get('currentStep');
          var currentClusterStatus = App.clusterStatus.get('value');
          if (currentClusterStatus) {
            switch (currentClusterStatus.clusterState) {
              case 'REASSIGN_MASTER_INSTALLING':
                if (currentClusterStatus.localdb.ReassignMaster.currentStep !== currStep) {
                  reassignMasterController.setCurrentStep(currentClusterStatus.localdb.ReassignMaster.currentStep);
                }
                break;
            }
          }
          App.router.get('wizardWatcherController').setUser(reassignMasterController.get('name'));
          router.transitionTo('step' + currStep);
        });
      } else {
        App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('services.reassign.error.fewHosts'), function () {
          router.transitionTo('main.services.index');
        });
      }
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController');
      var step1Controller = router.get('reassignMasterWizardStep1Controller');
      controller.setCurrentStep('1');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        controller.connectOutlet('reassignMasterWizardStep1', controller.get('content'));
        step1Controller.loadConfigsTags();
      });
    },
    next: function next(router) {
      var controller = router.get('reassignMasterController');
      App.db.setMasterComponentHosts(undefined);
      controller.clearMasterComponentHosts();
      router.transitionTo('step2');
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController');
      controller.setCurrentStep('2');
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
        controller.loadAllPriorSteps();
        controller.connectOutlet('reassignMasterWizardStep2', controller.get('content'));
      });
    },
    back: Em.Router.transitionTo('step1'),
    next: function next(router) {
      var controller = router.get('reassignMasterController');
      var reassignMasterWizardStep2 = router.get('reassignMasterWizardStep2Controller');
      controller.saveMasterComponentHosts(reassignMasterWizardStep2);
      var reassignHosts = {};
      var componentName = reassignMasterWizardStep2.get('content.reassign.component_name');
      var masterAssignmentsHosts = reassignMasterWizardStep2.get('selectedServicesMasters').filterProperty('component_name', componentName).mapProperty('selectedHost');
      var currentMasterHosts = App.HostComponent.find().filterProperty('componentName', componentName).mapProperty('hostName');
      masterAssignmentsHosts.forEach(function (host) {
        if (!currentMasterHosts.contains(host)) {
          reassignHosts.target = host;
        }
      }, this);
      currentMasterHosts.forEach(function (host) {
        if (!masterAssignmentsHosts.contains(host)) {
          reassignHosts.source = host;
        }
      }, this);
      controller.saveReassignHosts(reassignHosts);
      router.transitionTo('step3');
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController'),
          stepController = router.get('reassignMasterWizardStep3Controller');
      controller.setCurrentStep('3');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        stepController.set('wizardController', controller);
        controller.connectOutlet('reassignMasterWizardStep3', controller.get('content'));
      });
    },
    back: Em.Router.transitionTo('step2'),
    next: function next(router) {
      var controller = router.get('reassignMasterController'),
          stepController = router.get('reassignMasterWizardStep3Controller'),
          configs = stepController.get('configs'),
          attributes = stepController.get('configsAttributes'),
          secureConfigs = stepController.get('secureConfigs');
      App.db.setReassignTasksStatuses(undefined);
      App.db.setReassignTasksRequestIds(undefined);
      App.clusterStatus.setClusterStatus({
        clusterName: router.get('reassignMasterController.content.cluster.name'),
        clusterState: 'REASSIGN_MASTER_INSTALLING',
        wizardControllerName: 'reassignMasterController',
        localdb: App.db.data
      });
      controller.saveReassignComponentsInMM(controller.getReassignComponentsInMM());
      stepController.updateServiceConfigs();
      controller.saveConfigs(configs, attributes);
      controller.saveSecureConfigs(secureConfigs);
      router.transitionTo('step4');
    },

    exit: function exit(router) {
      router.get('reassignMasterWizardStep3Controller').clearStep();
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController'),
          stepController = router.get('reassignMasterWizardStep4Controller');
      controller.setCurrentStep('4');
      controller.setLowerStepsDisable(4);
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
        controller.loadAllPriorSteps();
        stepController.set('wizardController', controller);
        controller.connectOutlet('reassignMasterWizardStep4', controller.get('content'));
      });
    },
    next: function next(router) {
      router.get('reassignMasterController').setCurrentStep('5');

      App.clusterStatus.setClusterStatus({
        clusterName: router.get('reassignMasterController.content.cluster.name'),
        clusterState: 'REASSIGN_MASTER_INSTALLING',
        wizardControllerName: 'reassignMasterController',
        localdb: App.db.data
      });

      router.transitionTo('step5');
    },

    complete: function complete(router) {
      var controller = router.get('reassignMasterController');
      var reassignMasterWizardStep4 = router.get('reassignMasterWizardStep4Controller');
      if (!reassignMasterWizardStep4.get('isSubmitDisabled')) {
        controller.resetOnClose(controller, 'main.index');
      }
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step5: Em.Route.extend({
    route: '/step5',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController');
      controller.setCurrentStep('5');
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
        controller.loadAllPriorSteps();
        controller.setLowerStepsDisable(5);
        if (controller.get('content.reassign.component_name') === 'NAMENODE' || controller.get('content.reassign.component_name') === 'SECONDARY_NAMENODE') {
          controller.usersLoading().done(function () {
            controller.connectOutlet('reassignMasterWizardStep5', controller.get('content'));
          });
        } else {
          controller.connectOutlet('reassignMasterWizardStep5', controller.get('content'));
        }
      });
    },
    next: function next(router) {
      App.showConfirmationPopup(function () {
        var controller = router.get('reassignMasterController');
        controller.saveReassignComponentsInMM(controller.getReassignComponentsInMM());
        router.transitionTo('step6');
      }, Em.I18n.t('services.reassign.step5.confirmPopup.body'));
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step6: Em.Route.extend({
    route: '/step6',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController');
      controller.setCurrentStep('6');
      controller.setLowerStepsDisable(6);
      router.get('mainController').isLoading.call(router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
        controller.loadAllPriorSteps();
        controller.connectOutlet('reassignMasterWizardStep6', controller.get('content'));
      });
    },

    next: function next(router) {
      var controller = router.get('reassignMasterController');
      controller.finish();
      App.clusterStatus.setClusterStatus({
        clusterName: router.get('reassignMasterController.content.cluster.name'),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, {
        alwaysCallback: function alwaysCallback() {
          controller.get('popup').hide();
          router.transitionTo('main.index');
          Em.run.next(function () {
            location.reload();
          });
        }
      });
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  step7: Em.Route.extend({
    route: '/step7',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('reassignMasterController'),
          stepController = router.get('reassignMasterWizardStep7Controller');
      controller.setCurrentStep('7');
      controller.setLowerStepsDisable(7);
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        stepController.set('wizardController', controller);
        controller.connectOutlet('reassignMasterWizardStep7', controller.get('content'));
      });
    },

    next: function next(router) {
      var controller = router.get('reassignMasterController');
      controller.resetOnClose(controller, 'main.index');
    },

    complete: function complete(router) {
      var controller = router.get('reassignMasterController');
      controller.resetOnClose(controller, 'main.index');
    },

    unroutePath: function unroutePath() {
      return false;
    }
  }),

  gotoStep7: Em.Router.transitionTo('step7'),

  backToServices: function backToServices(router) {
    App.router.get('updateController').set('isWorking', true);
    router.transitionTo('services');
  }

});

});

require.register("routes/remove_hawq_standby_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/Hawq/remove',

  breadcrumbs: {
    label: Em.I18n.t('admin.removeHawqStandby.wizard.header')
  },

  enter: function enter(router, transition) {
    var removeHawqStandbyWizardController = router.get('removeHawqStandbyWizardController');
    removeHawqStandbyWizardController.dataLoading().done(function () {
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'HAWQ'));
    });
    Em.run.next(function () {
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.removeHawqStandby.wizard.header'),
        bodyClass: App.RemoveHawqStandbyWizardView.extend({
          controller: removeHawqStandbyWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          var removeHawqStandbyWizardController = router.get('removeHawqStandbyWizardController'),
              currStep = removeHawqStandbyWizardController.get('currentStep');
          if (parseInt(currStep) === 3) {
            App.showConfirmationPopup(function () {
              var controller = router.get('removeHawqStandbyWizardController');
              router.get('updateController').set('isWorking', true);
              removeHawqStandbyWizardController.finish();
              App.clusterStatus.setClusterStatus({
                clusterName: App.router.getClusterName(),
                clusterState: 'DEFAULT',
                localdb: App.db.data
              }, {
                alwaysCallback: function alwaysCallback() {
                  controller.get('popup').hide();
                  router.transitionTo('main.services.index');
                  Em.run.next(function () {
                    location.reload();
                  });
                }
              });
            }, Em.I18n.t('admin.removeHawqStandby.closePopup'));
          } else {
            var controller = router.get('removeHawqStandbyWizardController');
            router.get('updateController').set('isWorking', true);
            removeHawqStandbyWizardController.finish();
            App.clusterStatus.setClusterStatus({
              clusterName: App.router.getClusterName(),
              clusterState: 'DEFAULT',
              localdb: App.db.data
            }, {
              alwaysCallback: function alwaysCallback() {
                controller.get('popup').hide();
                router.transitionTo('main.services.index');
                Em.run.next(function () {
                  location.reload();
                });
              }
            });
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      removeHawqStandbyWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'REMOVE_HAWQ_STANDBY':
            removeHawqStandbyWizardController.setCurrentStep(currentClusterStatus.localdb.RemoveHawqStandbyWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('removeHawqStandbyWizardController.currentStep');
            removeHawqStandbyWizardController.setCurrentStep(currStep);
            break;
        }
      }
      router.transitionTo('step' + removeHawqStandbyWizardController.get('currentStep'));
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('removeHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.connectOutlet('removeHawqStandbyWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('removeHawqStandbyWizardController');
      var hawqMaster = App.HostComponent.find().findProperty('componentName', 'HAWQMASTER').get('hostName');
      var hawqStandby = App.HostComponent.find().findProperty('componentName', 'HAWQSTANDBY').get('hostName');
      var hawqHosts = {
        hawqMaster: hawqMaster,
        hawqStandby: hawqStandby
      };
      controller.saveHawqHosts(hawqHosts);
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('removeHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('removeHawqStandbyWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      App.showConfirmationPopup(function () {
        var wizardController = router.get('removeHawqStandbyWizardController');
        var stepController = router.get('removeHawqStandbyWizardStep2Controller');
        var configs = stepController.get('selectedService.configs');
        wizardController.saveConfigs(configs);
        router.transitionTo('step3');
      }, Em.I18n.t('admin.removeHawqStandby.wizard.step2.confirmPopup.body'));
    },
    back: Em.Router.transitionTo('step1')
  }),
  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('removeHawqStandbyWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.setLowerStepsDisable(3);
        controller.loadAllPriorSteps();
        controller.connectOutlet('removeHawqStandbyWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath(router, path) {
      // allow user to leave route if wizard has finished
      if (router.get('removeHawqStandbyWizardController').get('isFinished')) {
        this._super(router, path);
      } else {
        return false;
      }
    },
    next: function next(router) {
      var controller = router.get('removeHawqStandbyWizardController');
      controller.finish();
      App.clusterStatus.setClusterStatus({
        clusterName: controller.get('content.cluster.name'),
        clusterState: 'DEFAULT',
        localdb: App.db.data
      }, {
        alwaysCallback: function alwaysCallback() {
          controller.get('popup').hide();
          router.transitionTo('main.services.index');
          Em.run.next(function () {
            location.reload();
          });
        }
      });
    }
  })

});

});

require.register("routes/rm_high_availability_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/ResourceManager/enable',

  breadcrumbs: {
    label: Em.I18n.t('admin.rm_highAvailability.wizard.header')
  },

  enter: function enter(router, transition) {
    var rMHighAvailabilityWizardController = router.get('rMHighAvailabilityWizardController');
    rMHighAvailabilityWizardController.dataLoading().done(function () {
      //Set YARN as current service
      App.router.set('mainServiceItemController.content', App.Service.find().findProperty('serviceName', 'YARN'));
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.rm_highAvailability.wizard.header'),
        bodyClass: App.RMHighAvailabilityWizardView.extend({
          controller: rMHighAvailabilityWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,

        onClose: function onClose() {
          var rMHighAvailabilityWizardController = router.get('rMHighAvailabilityWizardController'),
              currStep = rMHighAvailabilityWizardController.get('currentStep');
          if (parseInt(currStep) === 4) {
            App.showConfirmationPopup(function () {
              rMHighAvailabilityWizardController.resetOnClose(rMHighAvailabilityWizardController, 'main.services.index');
            }, Em.I18n.t('admin.rm_highAvailability.closePopup'));
          } else {
            rMHighAvailabilityWizardController.resetOnClose(rMHighAvailabilityWizardController, 'main.services.index');
          }
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      rMHighAvailabilityWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'RM_HIGH_AVAILABILITY_DEPLOY':
            rMHighAvailabilityWizardController.setCurrentStep(currentClusterStatus.localdb.RMHighAvailabilityWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('rMHighAvailabilityWizardController.currentStep');
            rMHighAvailabilityWizardController.setCurrentStep(currStep);
            break;
        }
      }
      Em.run.next(function () {
        App.router.get('wizardWatcherController').setUser(rMHighAvailabilityWizardController.get('name'));
        router.transitionTo('step' + rMHighAvailabilityWizardController.get('currentStep'));
      });
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('1');
        controller.connectOutlet('rMHighAvailabilityWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.setDBProperty('rmHosts', undefined);
      controller.clearMasterComponentHosts();
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('2');
        controller.loadAllPriorSteps();
        controller.connectOutlet('rMHighAvailabilityWizardStep2', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var wizardController = router.get('rMHighAvailabilityWizardController');
      var stepController = router.get('rMHighAvailabilityWizardStep2Controller');
      var currentRM = stepController.get('servicesMasters').filterProperty('component_name', 'RESOURCEMANAGER').findProperty('isInstalled', true);
      var additionalRM = stepController.get('servicesMasters').filterProperty('component_name', 'RESOURCEMANAGER').findProperty('isInstalled', false);
      var rmHost = {
        currentRM: currentRM.get('selectedHost'),
        additionalRM: additionalRM.get('selectedHost')
      };
      wizardController.saveRmHosts(rmHost);
      wizardController.saveMasterComponentHosts(stepController);
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('3');
        controller.loadAllPriorSteps();
        controller.connectOutlet('rMHighAvailabilityWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var wizardController = router.get('rMHighAvailabilityWizardController');
      var stepController = router.get('rMHighAvailabilityWizardStep3Controller');
      var configs = stepController.get('selectedService.configs');
      wizardController.saveConfigs(configs);
      router.transitionTo('step4');
    },
    back: Em.Router.transitionTo('step2')
  }),

  step4: Em.Route.extend({
    route: '/step4',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.dataLoading().done(function () {
        controller.setCurrentStep('4');
        controller.setLowerStepsDisable(4);
        controller.loadAllPriorSteps();
        controller.connectOutlet('rMHighAvailabilityWizardStep4', controller.get('content'));
      });
    },
    unroutePath: function unroutePath(router, path) {
      // allow user to leave route if wizard has finished
      if (router.get('rMHighAvailabilityWizardController').get('isFinished')) {
        this._super(router, path);
      } else {
        return false;
      }
    },
    next: function next(router) {
      var controller = router.get('rMHighAvailabilityWizardController');
      controller.resetOnClose(controller, 'main.services.index');
    }
  })

});

});

require.register("routes/rollbackHA_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: '/highAvailability/NameNode/rollbackHA',

  enter: function enter(router) {
    Em.run.next(function () {
      var rollbackHighAvailabilityWizardController = router.get('rollbackHighAvailabilityWizardController');
      App.router.get('updateController').set('isWorking', false);
      var popup = App.ModalPopup.show({
        classNames: ['wizard-modal-wrapper'],
        modalDialogClasses: ['modal-xlg'],
        header: Em.I18n.t('admin.highAvailability.wizard.rollback.header.title'),
        bodyClass: App.RollbackHighAvailabilityWizardView.extend({
          controller: rollbackHighAvailabilityWizardController
        }),
        primary: Em.I18n.t('form.cancel'),
        showFooter: false,
        secondary: null,
        hideCloseButton: function () {
          var currStep = App.router.get('rollbackHighAvailabilityWizardController.currentStep');
          if (currStep == 3) {
            this.set('showCloseButton', false);
          } else {
            this.set('showCloseButton', true);
          }
        }.observes('App.router.rollbackHighAvailabilityWizardController.currentStep'),

        onClose: function onClose() {
          this.hide();
          App.router.get('rollbackHighAvailabilityWizardController').setCurrentStep('1');
          App.router.get('updateController').set('isWorking', true);
          App.router.transitionTo('main.services');
        },
        didInsertElement: function didInsertElement() {
          this._super();
          this.fitHeight();
        }
      });
      rollbackHighAvailabilityWizardController.set('popup', popup);
      var currentClusterStatus = App.clusterStatus.get('value');
      if (currentClusterStatus) {
        switch (currentClusterStatus.clusterState) {
          case 'ROLLBACK_HIGH_AVAILABILITY':
            rollbackHighAvailabilityWizardController.setCurrentStep(currentClusterStatus.localdb.RollbackHighAvailabilityWizard.currentStep);
            break;
          default:
            var currStep = App.router.get('rollbackHighAvailabilityWizardController.currentStep');
            rollbackHighAvailabilityWizardController.setCurrentStep(currStep);
            break;
        }
      }
      router.transitionTo('step' + rollbackHighAvailabilityWizardController.get('currentStep'));
    });
  },

  step1: Em.Route.extend({
    route: '/step1',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rollbackHighAvailabilityWizardController');
      controller.setCurrentStep('1');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        controller.connectOutlet('rollbackHighAvailabilityWizardStep1', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('rollbackHighAvailabilityWizardController');
      controller.saveSelectedAddNN(controller.get('content.selectedAddNNHost'));
      controller.saveSelectedSNN(controller.get('content.selectedSNNHost'));
      router.transitionTo('step2');
    }
  }),

  step2: Em.Route.extend({
    route: '/step2',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rollbackHighAvailabilityWizardController');
      controller.setCurrentStep('2');
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        controller.usersLoading().done(function () {
          controller.connectOutlet('rollbackHighAvailabilityWizardStep2', controller.get('content'));
        });
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      router.transitionTo('step3');
    },
    back: function back(router) {
      router.transitionTo('step1');
    }
  }),

  step3: Em.Route.extend({
    route: '/step3',
    connectOutlets: function connectOutlets(router) {
      var controller = router.get('rollbackHighAvailabilityWizardController');
      controller.setCurrentStep('3');
      controller.setLowerStepsDisable(3);
      App.clusterStatus.setClusterStatus({
        clusterName: router.get('content.cluster.name'),
        clusterState: 'ROLLBACK_HIGH_AVAILABILITY',
        wizardControllerName: 'rollbackHighAvailabilityWizardController',
        localdb: App.db.data
      });
      controller.dataLoading().done(function () {
        controller.loadAllPriorSteps();
        controller.connectOutlet('rollbackHighAvailabilityWizardStep3', controller.get('content'));
      });
    },
    unroutePath: function unroutePath() {
      return false;
    },
    next: function next(router) {
      var controller = router.get('rollbackHighAvailabilityWizardController');
      controller.clearTasksData();
      controller.clearStorageData();
      controller.finish();
      controller.get('popup').hide();
      App.router.get('updateController').set('isWorking', true);
      App.clusterStatus.setClusterStatus({
        clusterName: router.get('content.cluster.name'),
        clusterState: 'DEFAULT',
        wizardControllerName: 'rollbackHighAvailabilityWizardController',
        localdb: App.db.data
      });
      router.transitionTo('main.index');
      Em.run.next(function () {
        location.reload();
      });
    }
  })

});

});

require.register("routes/stack_upgrade_routes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = App.WizardRoute.extend({
  route: 'stack/upgrade',

  breadcrumbs: {
    labelBindingPath: 'App.router.mainAdminStackAndUpgradeController.wizardModalTitle'
  },

  enter: function enter(router) {
    if (App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK')) {
      router.get('mainController').dataLoading().done(function () {
        Ember.run.next(function () {
          //if upgrade id is absent then upgrade is completed
          if (Em.isNone(App.db.get('MainAdminStackAndUpgrade', 'upgradeId'))) {
            router.transitionTo('main.admin.stackAndUpgrade.versions');
            return null;
          }

          App.router.get('updateController').set('isWorking', false);

          return App.ModalPopup.show({
            classNames: ['upgrade-wizard-modal'],
            modalDialogClasses: ['modal-xlg'],
            headerClass: Em.View.extend({
              header: Em.computed.alias('controller.wizardModalTitle'),
              controllerBinding: 'App.router.mainAdminStackAndUpgradeController',
              template: Ember.Handlebars.compile('{{view.header}}' + '<div {{bindAttr class=":upgrade-options-link controller.isDowngrade:disabled" disabled="controller.isDowngrade"}} {{action openUpgradeOptions target="controller"}}>' + '<i class="icon-cogs"></i><a>{{t admin.stackVersions.version.upgrade.upgradeOptions.header}}</a>' + '</div>')
            }),
            bodyClass: App.upgradeWizardView,
            primary: Em.I18n.t('common.dismiss'),
            secondary: null,
            didInsertElement: function didInsertElement() {
              this._super();
              this.fitHeight();
              this.fitInnerHeight();
            },

            /**
             * fit height of scrollable block inside of modal body
             */
            fitInnerHeight: function fitInnerHeight() {
              var block = this.$().find('.modal .modal-body');
              var scrollable = this.$().find('.modal .scrollable-block');

              scrollable.css('max-height', Number(block.css('max-height').slice(0, -2)) - block.height());
              block.css('max-height', 'none');
            },
            onPrimary: function onPrimary() {
              this.closeWizard();
            },
            onClose: function onClose() {
              this.closeWizard();
            },
            closeWizard: function closeWizard() {
              App.router.get('updateController').set('isWorking', true);
              App.router.transitionTo('main.admin.stackAndUpgrade.versions');
              this.hide();
              if (['NOT_REQUIRED', 'COMPLETED'].contains(App.get('upgradeState'))) {
                location.reload();
                App.router.get('wizardWatcherController').resetUser();
              }
            }
          });
        });
      });
    } else {
      Em.run.next(function () {
        App.router.transitionTo('main.views');
      });
    }
  }
});

});

require.register("routes/view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = Em.Route.extend({

  breadcrumbs: {
    labelBindingPath: 'App.router.mainViewsDetailsController.content.label'
  },

  route: '/view',
  enter: function enter(router) {
    Em.$('body').addClass('contribview');
    router.get('mainViewsController').loadAmbariViews();
  },

  exit: function exit(router) {
    this._super();
    Em.$('body').removeClass('contribview');
  },

  index: Em.Route.extend({
    route: '/',
    connectOutlets: function connectOutlets(router) {
      router.get('mainViewsController').dataLoading().done(function () {
        router.get('mainController').connectOutlet('mainViews');
      });
    }
  }),

  shortViewDetails: Em.Route.extend({
    route: '/:viewName/:shortName',
    breadcrumbs: null,
    connectOutlets: function connectOutlets(router, params) {
      var viewPath = this.parseViewPath(window.location.href.slice(window.location.href.indexOf('?')));
      var slicedShortName = params.shortName;
      if (viewPath) {
        slicedShortName = this._getSlicedShortName(params.shortName);
        if (slicedShortName === params.shortName) {
          viewPath = '';
        }

        if (viewPath.charAt(0) === '/') viewPath = viewPath.slice(1);
      }

      router.get('mainViewsController').dataLoading().done(function () {
        var content = App.router.get('mainViewsController.ambariViews').filterProperty('viewName', params.viewName).findProperty('shortUrl', slicedShortName);
        if (content) content.set('viewPath', viewPath);
        router.get('mainController').connectOutlet('mainViewsDetails', content);
      });
    },

    /**
     * parse the short name and slice if needed
     *
     * @param {string}
     * @returns {string}
     * @private
     */
    _getSlicedShortName: function _getSlicedShortName(instanceName) {
      if (instanceName.lastIndexOf('?') > -1) {
        return instanceName.slice(0, instanceName.lastIndexOf('?'));
      }

      return instanceName;
    },

    /**
     * parse internal view path
     * "viewPath" - used as a key of additional path
     * Example:
     *   origin URL: viewName?viewPath=%2Fuser%2Fadmin%2Faddress&foo=bar&count=1
     * should be translated to
     *   view path: /user/admin/address?foo=bar&count=1
     *
     * @param {string} instanceName
     * @returns {string}
     */
    parseViewPath: function parseViewPath(instanceName) {
      var path = '';
      if (instanceName.contains('?')) {
        path = instanceName.slice(instanceName.indexOf('?'));
        if (path.contains('viewPath')) {
          path = decodeURIComponent(path.slice(path.lastIndexOf('?viewPath=') + 10)).replace('&', '?');
        }
      }
      return path;
    }

  })
});

});

require.register("routes/views", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = Em.Route.extend({

  breadcrumbs: null,

  route: '/views',
  enter: function enter(router) {
    router.get('mainViewsController').loadAmbariViews();
  },
  index: Em.Route.extend({
    route: '/',
    connectOutlets: function connectOutlets(router) {
      router.get('mainViewsController').dataLoading().done(function () {
        router.get('mainController').connectOutlet('mainViews');
      });
    }
  }),

  viewDetails: Em.Route.extend({

    route: '/:viewName/:version/:instanceName',

    enter: function enter(router) {
      Em.$('body').addClass('contribview');
    },

    breadcrumbs: {
      labelBindingPath: 'App.router.mainViewsDetailsController.content.label'
    },

    exit: function exit(router) {
      this._super();
      Em.$('body').removeClass('contribview');
    },

    connectOutlets: function connectOutlets(router, params) {
      // find and set content for `mainViewsDetails` and associated controller
      var href = ['/views', params.viewName, params.version, params.instanceName + "/"].join('/');
      var viewPath = this.parseViewPath(window.location.href.slice(window.location.href.indexOf('?')));
      if (viewPath) {
        var slicedInstanceName = this._getSlicedInstanceName(params.instanceName);
        if (slicedInstanceName === params.instanceName) {
          viewPath = '';
        }
        href = ['/views', params.viewName, params.version, slicedInstanceName + "/"].join('/');
        //remove slash from viewPath since href already contains it at the end
        if (viewPath.charAt(0) === '/') viewPath = viewPath.slice(1);
      }

      router.get('mainViewsController').dataLoading().done(function () {
        var content = App.router.get('mainViewsController.ambariViews').filter(function (i) {
          return Em.get(i, 'href').endsWith(href);
        })[0];
        if (content) content.set('viewPath', viewPath);
        router.get('mainController').connectOutlet('mainViewsDetails', content);
      });
    },

    /**
     * parse the instance name and slice if needed
     *
     * @param {string}
     * @returns {string}
     * @private
     */
    _getSlicedInstanceName: function _getSlicedInstanceName(instanceName) {
      if (instanceName.lastIndexOf('?') > -1) {
        return instanceName.slice(0, instanceName.lastIndexOf('?'));
      }

      return instanceName;
    },

    /**
     * parse internal view path
     * "viewPath" - used as a key of additional path
     * Example:
     *   origin URL: viewName?viewPath=%2Fuser%2Fadmin%2Faddress&foo=bar&count=1
     * should be translated to
     *   view path: /user/admin/address?foo=bar&count=1
     *
     * @param {string} instanceName
     * @returns {string}
     */
    parseViewPath: function parseViewPath(instanceName) {
      var path = '';
      if (instanceName.contains('?')) {
        path = instanceName.slice(instanceName.indexOf('?'));
        if (path.contains('viewPath')) {
          path = decodeURIComponent(path.slice(path.lastIndexOf('?viewPath=') + 10)).replace('&', '?');
        }
      }
      return path;
    }
  })
});

});

require.register("templates", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load templates here

require('templates/main/service/info/summary/base');
require('templates/common/progress');
require('templates/main/alerts/alert_instance/status');
require("templates/main/service/widgets/create/step2_number");
require("templates/main/service/widgets/create/step2_template");
require("templates/main/service/widgets/create/step2_graph");
require('templates/common/configs/widgets/controls');
require('templates/common/configs/widgets/controls/create_override');
require('templates/common/configs/widgets/controls/remove_override');
require('templates/common/configs/widgets/controls/final');
require('templates/common/configs/widgets/controls/set_recommended');
require('templates/common/configs/widgets/controls/undo');
require('templates/common/configs/widgets/controls/edit');
require('templates/common/configs/widgets/config_label');
require('templates/common/configs/configs_comparison_row');
require('templates/common/configs/configs_comparison_cell');
require('templates/wizard/installer_cancel_button');

});

require.register("templates/application", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.ClockView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div id=\"side-nav\" class=\"navigation-bar\">\n    <div class=\"navigation-bar-container\">\n    <ul class=\"side-nav-header nav nav-pills nav-stacked\">\n      <li class=\"navigation-header\">\n        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "goToDashboard";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"ambari-logo\">\n          <img src=\"/img/ambari-logo.png\" alt=\"Apache Ambari\" title=\"Apache Ambari\" ");
  stack1 = depth0;
  stack2 = "ambari-logo";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        </a>\n        <div class=\"btn-group\" ");
  stack1 = depth0;
  stack2 = "goToDashboard";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <span class=\"ambari-header\" title=\"Apache Ambari\" ");
  stack1 = depth0;
  stack2 = "ambari-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n        </div>\n      </li>\n    </ul>\n    ");
  stack1 = depth0;
  stack2 = "App.MainSideMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <ul class=\"side-nav-footer nav nav-pills nav-stacked\">\n      <li class=\"navigation-footer\">\n        <a href=\"#\" data-toggle=\"collapse-side-nav\">\n          <span class=\"navigation-icon icon-double-angle-left\"></span>\n        </a>\n      </li>\n    </ul>\n    </div>\n  </div>\n");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <nav class=\"navbar navbar-default navbar-static-top\">\n        <div class=\"container main-container\">\n          <div class=\"navbar-header navbar-nav\">\n            ");
  stack1 = depth0;
  stack2 = "App.BreadcrumbsView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n\n          ");
  data.buffer.push("\n          <ul class=\"nav navbar-nav navbar-right\"></ul>\n\n          ");
  data.buffer.push("\n          <div class=\"top-nav-user navbar-right btn-group\">\n            <button ");
  stack1 = depth0;
  stack2 = "user-dropdown";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"dropdown-toggle navbar-btn btn btn-default\" data-toggle=\"dropdown\" role=\"button\" aria-haspopup=\"true\" aria-expanded=\"false\">\n              <i class=\"glyphicon glyphicon-user\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "App.router.displayLoginName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;<span class=\"caret\"></span>\n            </button>\n            <ul class=\"dropdown-menu\">\n\n              ");
  data.buffer.push("\n              <li><a href=\"#\" id=\"about\" ");
  stack1 = depth0;
  stack2 = "showAboutPopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.aboutAmbari";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n              ");
  data.buffer.push("\n\n              ");
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "showManageAmbari";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  data.buffer.push("\n\n              ");
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "isExistingClusterDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  data.buffer.push("\n\n              ");
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "showExitLink";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  data.buffer.push("\n            </ul>\n          </div>\n          ");
  data.buffer.push("\n\n          ");
  data.buffer.push("\n          <div class=\"navbar-nav navbar-right ambari-views notifications-group\">\n            ");
  stack1 = depth0;
  stack2 = "enableLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          ");
  data.buffer.push("\n\n          <div class=\"navbar-nav navbar-right cluster-notifications\">\n            ");
  stack1 = depth0;
  stack2 = "enableLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n\n          <div class=\"navbar-nav navbar-text navbar-right brand-wrapper\">\n            ");
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "clusterName";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"cluster-name\">\n              ");
  stack1 = depth0;
  stack2 = "App.isClusterUser";
  stack3 = helpers.unless;
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            ");
  data.buffer.push("\n          </div>\n        </div>\n      </nav>\n    ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "AMBARI.ADD_DELETE_CLUSTERS, AMBARI.ASSIGN_ROLES, AMBARI.EDIT_STACK_REPOS, AMBARI.MANAGE_GROUPS, AMBARI.MANAGE_STACK_VERSIONS, AMBARI.MANAGE_USERS, AMBARI.MANAGE_VIEWS, AMBARI.RENAME_CLUSTER, AMBARI.MANAGE_USERS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <li>\n                    <a href=\"#\" id=\"manage-ambari\" ");
  stack1 = depth0;
  stack2 = "goToAdminView";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "app.manageAmbari";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </li>\n                ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "AMBARI.MANAGE_SETTINGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <li>\n                    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "showSettingsPopup";
  stack3 = {};
  stack4 = "App.router.userSettingsController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "app.settings";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </li>\n                ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <li role=\"separator\" class=\"divider\"></li>\n                <li><a href=\"\" id=\"sign-out\" ");
  stack1 = depth0;
  stack2 = "sign-out";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "logoff";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.signout";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n              ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <i class=\"icon-th dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\" aria-expanded=\"false\"></i>\n              <ul class=\"notifications-dropdown row dropdown-menu\">\n                <div class=\"popup-arrow-up\"></div>\n                <div class=\"notifications-header col-sm-12\">\n                  <div class=\"notifications-title\">");
  stack1 = depth0;
  stack2 = "common.views";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                </div>\n                <div class=\"notifications-body col-sm-12\">\n                  ");
  stack1 = depth0;
  stack2 = "view.views.length";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(18, program18, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </ul>\n            ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.views";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                      <li>\n                        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "setView";
  stack5 = {};
  stack6 = "App.router.mainViewsController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      </li>\n                    ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <li class=\"disabled\"><a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "menu.item.views.noViews";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                  ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  data.buffer.push("\n            <a href=\"#\" class=\"bg-label\" ");
  stack1 = depth0;
  stack2 = "showPopup";
  stack3 = {};
  stack4 = "App.router.backgroundOperationsController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "App.router.backgroundOperationsController";
  stack3 = helpers['with'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </a>\n            ");
  data.buffer.push("\n\n              ");
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isAlertsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  data.buffer.push("\n            ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"glyphicon glyphicon-cog\"></span>\n                <span id=\"span-bg-operation-count\" ");
  stack1 = {};
  stack2 = "runningOperationsCount:operations-count :numberCircle";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "runningOperationsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"notifications-group\">\n                <span class=\"alerts-label dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\n                      aria-expanded=\"false\">\n                  <span class=\"glyphicon glyphicon-bell\"></span>\n                  ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertDefinitionsController.unhealthyAlertInstancesCount";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(26, program26, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </span>\n                <div id=\"notifications-dropdown\" class=\"dropdown-menu row\">\n                  ");
  stack1 = depth0;
  stack2 = "App.AlertInstancesPopupView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              </span>\n              ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  data.buffer.push("\n                    <span ");
  stack1 = {};
  stack2 = ":numberCircle App.router.mainAlertDefinitionsController.isCriticalAlerts:alert-crit-count:alert-warn-count";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertDefinitionsController.unhealthyAlertInstancesCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </span>\n                  ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  data.buffer.push("\n                    <span ");
  stack1 = {};
  stack2 = "titlebar.alerts.noAlerts";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"numberCircle alerts-none-count\">\n                      ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertDefinitionsController.unhealthyAlertInstancesCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </span>\n                  ");
  return buffer;}

function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span>");
  stack1 = depth0;
  stack2 = "clusterDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <nav class=\"navigation-bar-container login-header\">\n        <ul class=\"side-nav-header nav nav-pills nav-stacked\">\n          <li class=\"navigation-header\">\n            <a class=\"ambari-logo\">\n              <img src=\"/img/ambari-logo.png\" alt=\"Apache Ambari\" title=\"Apache Ambari\" ");
  stack1 = depth0;
  stack2 = "ambari-logo";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n            </a>\n            <div class=\"btn-group\">\n              <span class=\"ambari-header\" title=\"Apache Ambari\" ");
  stack1 = depth0;
  stack2 = "ambari-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </div>\n          </li>\n        </ul>\n      </nav>\n    ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"ru-badge row\">\n          ");
  stack1 = depth0;
  stack2 = "isExistingClusterDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div class=\"row\">\n          ");
  stack1 = depth0;
  stack2 = "App.router.mainAdminStackAndUpgradeController.isFinalizeItem";
  stack3 = helpers['if'];
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <p class=\"col-md-4 col-md-offset-4\">\n              ");
  stack1 = depth0;
  stack2 = "showUpgradeLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(36, program36, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </p>\n          ");
  return buffer;}
function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "openUpgradeDialog";
  stack3 = {};
  stack4 = "App.router.mainAdminStackAndUpgradeController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <span ");
  stack1 = {};
  stack2 = "upgradeMap.cls :label";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <i ");
  stack1 = {};
  stack2 = "upgradeMap.icon :glyphicon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>&nbsp;\n                    ");
  stack1 = depth0;
  stack2 = "upgradeMap.msg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </span>\n                </a>\n              ");
  return buffer;}

function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack3 = helpers['if'];
  tmp1 = self.program(37, program37, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program37(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span class=\"label upgrade-in-progress\">\n                  <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "App.router.wizardWatcherController.wizardDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n                ");
  return buffer;}

function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <p class=\"col-md-10 col-md-offset-1 alert alert-danger\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.notFinalized.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n          ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "App.enableDigitalClock";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "App.router.mainController.isClusterDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div id=\"main\">\n  <div id=\"top-nav\">\n    ");
  stack1 = depth0;
  stack2 = "App.router.loggedIn";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(30, program30, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n\n  <div class=\"container main-container\">\n    <div id=\"content\">\n      ");
  stack1 = depth0;
  stack2 = "App.wizardIsNotFinished";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/about", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"about row\">\n  <div class=\"logo col-md-2\">\n    <img src=\"/img/logo.png\" alt=\"Apache Ambari\" title=\"Apache Ambari\" />\n  </div>\n  <div class=\"content col-md-10\">\n    <div class=\"project\">Apache Ambari</div>\n      <br />\n      ");
  stack1 = depth0;
  stack2 = "app.aboutAmbari.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.ambariVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <br />\n      <br />\n      <a href=\"http://ambari.apache.org/\" target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "app.aboutAmbari.getInvolved";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      <br />\n      <a href=\"http://www.apache.org/licenses/LICENSE-2.0\" target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "app.aboutAmbari.licensed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/assign_master_components", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"alert alert-danger\">");
  stack1 = depth0;
  stack2 = "msg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "msg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"assign-masters row\">\n          <div class=\"select-hosts col-md-8\">\n            <div class=\"row\">\n              ");
  stack1 = depth0;
  stack2 = "showCurrentHost";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <div class=\"clearfix\"></div>\n              <div class=\"row\">\n                <div class=\"col-md-12 control-group\">\n                    <!-- View for array controller -->\n                    ");
  stack1 = depth0;
  stack2 = "controller.additionalHostsList";
  stack3 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "servicesMastersToShow";
  stack3 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </div>\n            </div>\n          </div>\n\n          <div class=\"host-assignments col-md-4\">\n            ");
  stack1 = depth0;
  stack2 = "masterHostMapping";
  stack3 = helpers.each;
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            ");
  stack1 = depth0;
  stack2 = "remainingHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(35, program35, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"clearfix\"></div>\n        </div>\n      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"col-md-12 control-group mlc\">\n                  <div class=\"row\">\n                    <div class=\"col-md-4\"><span class=\"pull-right control-label\">");
  stack1 = depth0;
  stack2 = "services.reassign.step2.currentHost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    </div>\n                    <div class=\"col-md-8\"><span>");
  stack1 = depth0;
  stack2 = "currentHostId";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></div>\n                  </div>\n                </div>\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <div class=\"row row-no-pad additional-hosts-list\">\n                        <div class=\"col-md-4\">\n                          <label class=\"pts pull-right\" ");
  stack1 = depth0;
  stack2 = "component-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                            ");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          </label>\n                        </div>\n                        <div class=\"col-md-8 host-cell\">\n                          ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        </div>\n                      </div>\n                    ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <div ");
  stack1 = depth0;
  stack2 = "component-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"row row-no-pad form-group\">\n                        <div class=\"col-md-4\">\n                          <div class=\"control-group\">\n                            <label class=\"pts pull-right\" ");
  stack1 = depth0;
  stack2 = "component-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                              ");
  stack1 = depth0;
  stack2 = "showCurrentPrefix";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "showAdditionalPrefix";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "nameSpace";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  stack1 = depth0;
  stack2 = "display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ":\n                            </label>\n                          </div>\n                        </div>\n                        <div class=\"col-md-6 hosts-dropdown\">\n                          ");
  stack1 = depth0;
  stack2 = "isServiceCoHost";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        </div>\n                        <div class=\"col-md-2 pts assign-masters-controls\">\n                          ");
  stack1 = depth0;
  stack2 = "showAddControl";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "showRemoveControl";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        </div>\n                      </div>\n                    ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "common.additional";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            <div class=\"hostName\" ");
  stack1 = depth0;
  stack2 = "service-co-host";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                              ");
  stack1 = depth0;
  stack2 = "selectedHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<i class=\"glyphicon glyphicon-asterisks\">&#10037;</i>\n                            </div>\n                          ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            <div ");
  stack1 = {};
  stack2 = "errorMessage:error: warnMessage:warning:";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                              ");
  stack1 = depth0;
  stack2 = "view.shouldUseInputs";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(22, program22, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              <span rel=\"popover\" title=\"Warning\" ");
  stack1 = {};
  stack2 = "warnMessage";
  stack1['data-content'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                ");
  stack1 = depth0;
  stack2 = "warnMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              </span>\n                              <span rel=\"popover\" title=\"Error\" ");
  stack1 = {};
  stack2 = "errorMessage";
  stack1['data-content'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              </span>\n                            </div>\n                          ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "App.InputHostView";
  stack3 = {};
  stack4 = "this";
  stack3['componentBinding'] = stack4;
  stack4 = "isInstalled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "App.SelectHostView";
  stack3 = {};
  stack4 = "this";
  stack3['componentBinding'] = stack4;
  stack4 = "isInstalled";
  stack3['disabledBinding'] = stack4;
  stack4 = "content";
  stack3['contentBinding'] = stack4;
  stack4 = "host_name";
  stack3['optionValuePath'] = stack4;
  stack4 = "host_info";
  stack3['optionLabelPath'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  return buffer;}

function program24(depth0,data) {
  
  
  data.buffer.push("\n                                  <i class=\"glyphicon glyphicon-warning-sign\"></i>\n                                ");}

function program26(depth0,data) {
  
  
  data.buffer.push("\n                                  <i class=\"glyphicon glyphicon-exclamation-sign\"></i>\n                                ");}

function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                            <div>\n                              ");
  stack1 = depth0;
  stack2 = "App.AddControlView";
  stack3 = {};
  stack4 = "component_name";
  stack3['componentNameBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            </div>\n                          ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                            <div>\n                              ");
  stack1 = depth0;
  stack2 = "App.RemoveControlView";
  stack3 = {};
  stack4 = "component_name";
  stack3['componentNameBinding'] = stack4;
  stack4 = "serviceComponentId";
  stack3['serviceComponentIdBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            </div>\n                          ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"well\">\n                <div>\n                  <div class=\"hostString\"><span>");
  stack1 = depth0;
  stack2 = "hostInfo";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></div>\n                  ");
  stack1 = depth0;
  stack2 = "masterServicesToDisplay";
  stack3 = helpers.each;
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </div>\n            ");
  return buffer;}
function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <span ");
  stack1 = depth0;
  stack2 = "component-on-host";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isInstalled:assigned-service:new-service :service-component :label";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  ");
  return buffer;}

function program35(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"remaining-hosts well side-menu-well\">\n                <div class=\"row\">\n                  <div class=\"col-md-10 col-md-offset-1\">\n                    <strong>");
  stack1 = depth0;
  stack2 = "remainingHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong> ");
  stack1 = depth0;
  stack2 = "installer.step5.attention";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>\n                </div>\n              </div>\n            ");
  return buffer;}

function program37(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "view.isBackButtonVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(40, program40, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <button ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " type=\"button\" class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "nextButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(43, program43, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n      </button>\n    </div>\n  </div>\n");
  return buffer;}
function program40(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <button ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " type=\"button\" class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(41, program41, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </button>\n      ");
  return buffer;}
function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program43(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n<div id=\"assign-masters\" class=\"wizard-content col-md-9\">\n");
  stack1 = depth0;
  stack2 = "view.showTitle";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "view.alertMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "controller.generalErrorMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "msg";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "controller.generalWarningMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "msg";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(37, program37, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n\n");
  stack1 = depth0;
  stack2 = "view.isWizardStep";
  stack3 = helpers['if'];
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/breadcrumbs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "item.itemView";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "item.isLast";
  stack3 = helpers.unless;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "item.itemView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n  <a ");
  stack1 = {};
  stack2 = "item.disabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "moveTo";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "item.formattedLabel";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </a>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  
  data.buffer.push("\n    <p class=\"breadcrumbs-forward-slash\">/</p>\n  ");}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.items";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/button_progress", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "yield";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "yield";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<button ");
  stack1 = {};
  stack2 = "view.buttonId";
  stack1['id'] = stack2;
  stack2 = ":btn view.buttonClassNames";
  stack1['class'] = stack2;
  stack2 = "view.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "handleClick";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "view.doSpinRight";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.doSpinRight";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</button>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/chart/linear_time", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.leftArrowVisible:visibleArrow :arrow-left";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "switchTimeBack";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.rightArrowVisible:visibleArrow :arrow-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "forward";
  stack3 = depth0;
  stack4 = "switchTimeForward";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  ");
  return buffer;}

  data.buffer.push("\n<div class=\"chart-wrapper\">\n  <div ");
  stack1 = {};
  stack2 = "view.isReady:hide:show :screensaver :no-borders :chart-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div ");
  stack1 = {};
  stack2 = "view.isReady::hidden :actions-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.timeRangeListView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <a ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden :corner-icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleFormatsList";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.export";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <i class=\"glyphicon glyphicon-save\"></i>\n    </a>\n    ");
  stack1 = depth0;
  stack2 = "view.exportMetricsMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isTimePagingEnable";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = "view.containerId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isReady:shown:hidden view.containerClass :chart-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div ");
  stack1 = {};
  stack2 = "view.yAxisId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.yAxisClass :chart-y-axis";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    <div ");
  stack1 = {};
  stack2 = "view.xAxisId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.xAxisClass :chart-x-axis";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    <div ");
  stack1 = {};
  stack2 = "view.legendId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.legendClass :chart-legend";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    <div ");
  stack1 = {};
  stack2 = "view.chartId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.chartClass :chart";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    <div ");
  stack1 = {};
  stack2 = "view.titleId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.titleClass view.isExportButtonHidden::has-data :chart-title";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isTimePagingEnable";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"timezone mtl15 mll\">\n    <strong>");
  stack1 = depth0;
  stack2 = "common.timezone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong>: ");
  stack1 = depth0;
  stack2 = "App.router.userSettingsController.userSettings.timezone.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/checkbox", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.checkboxView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "view.labelView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isCustomTemplate";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "yield";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/chosen_plugin", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <option ");
  stack1 = {};
  stack2 = "this";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</option>\n");
  return buffer;}

  data.buffer.push("\n\n<option value=\"\" disabled\n        selected>");
  stack1 = depth0;
  stack2 = "view.selectionObj.placeholder_text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</option>\n");
  stack1 = depth0;
  stack2 = "view.options";
  stack3 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/addPropertyWindow", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.serviceConfigObj.bulkConfigError:has-error :each-row :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <label class=\"control-label col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "services.service.config.addPropertyWindow.properties";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <br>\n        <small>");
  stack1 = depth0;
  stack2 = "services.service.config.addPropertyWindow.propertiesHelper";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</small>\n      </label>\n\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "view.serviceConfigObj.bulkConfigValue";
  stack3['valueBinding'] = stack4;
  stack4 = "4";
  stack3['rows'] = stack4;
  stack4 = "input-block-level form-control";
  stack3['classNames'] = stack4;
  stack4 = "services.service.config.addPropertyWindow.propertiesPlaceholder";
  stack3['placeholderTranslation'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.bulkConfigErrorMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span>\n      </div>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.serviceConfigObj.isKeyError:has-error :each-row :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "common.key";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.serviceConfigObj.name";
  stack3['valueBinding'] = stack4;
  stack4 = "input-block-level form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n        ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.isKeyError";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n        ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.showFilterLink";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n    <div class=\"each-row form-group\">\n      <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "common.value";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "view.serviceConfigObj.value";
  stack3['valueBinding'] = stack4;
  stack4 = "4";
  stack3['rows'] = stack4;
  stack4 = "input-block-level form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    <div class=\"each-row form-group\">\n      <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "common.propertyType";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "input-block-level form-control";
  stack3['classNames'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "view.serviceConfigObj.content";
  stack3['contentBinding'] = stack4;
  stack4 = "view.serviceConfigObj.propertyType";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"help-block validation-block\">\n            ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </span>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.isKeyWarning";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"has-warning\">\n              <span class=\"help-block validation-block\">\n                ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n            </span>\n          ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.notMisc";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <a href=\"#\" class=\"btn-padding\" ");
  stack1 = depth0;
  stack2 = "filterByKey";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "services.service.config.addPropertyWindow.filterKeyLink";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n            ");
  return buffer;}

  data.buffer.push("\n<form autocomplete=\"off\" class=\"form-horizontal add-property-window\">\n  <div class=\"each-row form-group\">\n    <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n    <div class=\"col-md-5\">\n      <input class=\"form-control\" type=\"text\" disabled ");
  stack1 = {};
  stack2 = "view.fileName";
  stack1['value'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/>\n    </div>\n    <div class=\"col-md-5\">\n      <div class=\"btn-group add-mode pull-right\" ");
  stack1 = depth0;
  stack2 = "toggleBulkMode";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <a href=\"#\"\n          ");
  stack1 = {};
  stack2 = ":btn :btn-default view.serviceConfigObj.isBulkMode::active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = {};
  stack2 = "services.service.config.addPropertyWindow.singleMode";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n           data-toggle=\"tooltip\"><i class=\"glyphicon glyphicon-tag\"></i></a>\n        <a href=\"#\"\n          ");
  stack1 = {};
  stack2 = ":btn :btn-default view.serviceConfigObj.isBulkMode:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = {};
  stack2 = "services.service.config.addPropertyWindow.bulkMode";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n           data-toggle=\"tooltip\"><i class=\"glyphicon glyphicon-tags\"></i></a>\n      </div>\n    </div>\n    <div class=\"clearfix\"></div>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.serviceConfigObj.isBulkMode";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/config_versions_control", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"compare-bar\">\n      <i class=\"icon-exchange\" aria-hidden=\"true\"></i>\n      <span class=\"compare-label\">");
  stack1 = depth0;
  stack2 = "services.service.config.configHistory.comparing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</span>\n      ");
  stack1 = depth0;
  stack2 = "App.ConfigVersionsDropdownView";
  stack3 = {};
  stack4 = "view.primaryServiceVersionsInCompare";
  stack3['serviceVersionsBinding'] = stack4;
  stack4 = "true";
  stack3['isCompareMode'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <span class=\"grey-text\">");
  stack1 = depth0;
  stack2 = "common.with";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  stack1 = depth0;
  stack2 = "App.ConfigVersionsDropdownView";
  stack3 = {};
  stack4 = "view.secondaryServiceVersionsInCompare";
  stack3['serviceVersionsBinding'] = stack4;
  stack4 = "true";
  stack3['isSecondary'] = stack4;
  stack4 = "true";
  stack3['isCompareMode'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <a ");
  stack1 = depth0;
  stack2 = "removeCompareVersionBar";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right close form-text\">&times;</a>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.ConfigVersionsDropdownView";
  stack3 = {};
  stack4 = "view.serviceVersions";
  stack3['serviceVersionsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.displayedServiceVersion.isCurrent";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <button class=\"btn btn-secondary make-current\" ");
  stack1 = depth0;
  stack2 = "makeCurrent";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.info-bar.revert.button";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"config-versions-control\">\n  ");
  stack1 = depth0;
  stack2 = "view.isCompareMode";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/config_versions_dropdown", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <li>\n        <div ");
  stack1 = {};
  stack2 = "item.isDisplayed:selected view.isCompareMode:full-width :version-info";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "mainClickAction";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div class=\"row\">\n            <div class=\"col-md-4\">\n              <span class=\"h2\">");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "item.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;</span>\n              <span class=\"description grey-text\">");
  stack1 = depth0;
  stack2 = "item.stackVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </div>\n            <div class=\"col-md-8 description grey-text\">\n                <span class=\"pull-right\">\n                  <strong>");
  stack1 = depth0;
  stack2 = "item.authorFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>\n                  &nbsp;");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.info-bar.authoredOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;\n                  <strong>");
  stack1 = depth0;
  stack2 = "item.createdDate";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>\n                </span>\n            </div>\n          </div>\n          <div class=\"row\">\n            <div class=\"col-md-10 description notes-color\">");
  stack1 = depth0;
  stack2 = "item.fullNotes";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            <div class=\"col-md-2\">\n              ");
  stack1 = depth0;
  stack2 = "item.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "view.isCompareMode";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </li>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <strong class=\"pull-right current-color\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong>\n              ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <button data-toggle=\"tooltip\" data-placement=\"right\"\n            ");
  stack1 = {};
  stack2 = "item.isDisplayed:hide :compare-button :btn :btn-default";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "compare";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = {};
  stack2 = "dashboard.configHistory.info-bar.compare.tooltip";
  stack1['title'] = stack2;
  stack2 = "dashboard.configHistory.info-bar.compare.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-exchange\" aria-hidden=\"true\"></i>\n          </button>\n        ");
  return buffer;}

  data.buffer.push("\n\n<button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\n        aria-expanded=\"false\">\n  ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n  <span><strong>");
  stack1 = depth0;
  stack2 = "view.displayedServiceVersion.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong><span class=\"caret\"></span></span>\n</button>\n<div class=\"dropdown-menu bottom-shadow\">\n  <li class=\"input-group search-input\">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.filterValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.searchLabel";
  stack3['placeholderBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <span class=\"input-group-btn\">\n            <button class=\"btn btn-default\" type=\"button\">\n              <span class=\"glyphicon glyphicon-search\" aria-hidden=\"true\"></span>\n            </button>\n          </span>\n  </li>\n  <ul class=\"versions-list\">\n    ");
  stack1 = depth0;
  stack2 = "view.filteredServiceVersions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/configs_comparison_cell", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "compareConfig.supportsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-lock compareConfig.isFinal::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n  ");
  return buffer;}

  data.buffer.push("\n\n<span ");
  stack1 = {};
  stack2 = "compareConfig.isMock:undefined :compare-config-cell";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "compareConfig.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "compareConfig.unit";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n");
  stack1 = depth0;
  stack2 = "compareConfig.isMock";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/configs_comparison_row", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <span class=\"control-label\">\n          ");
  stack1 = depth0;
  stack2 = "configData.displayName";
  foundHelper = helpers.formatWordBreak;
  stack3 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "configData.isSecureConfig";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </span>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  
  data.buffer.push("\n            <a href=\"javascript:void(null);\">\n              <i class=\"glyphicon glyphicon-lock\" rel=\"tooltip\" data-toggle=\"tooltip\" title=\"security knob\"></i>\n            </a>\n          ");}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  data.buffer.push("\n      <span ");
  stack1 = {};
  stack2 = "configData.isMock:undefined :compare-config-cell";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "configData.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "configData.unit";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  stack1 = depth0;
  stack2 = "configData.isMock";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.supportsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n          <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-lock configData.isFinal::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "configData.hasCompareDiffs";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.compareConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "compareConfig";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "compareConfig.isComparison";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_cell";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.overrides";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "compareConfig";
  stack7 = helpers.each;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_cell";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "configData.compareConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "compareConfig";
  stack7 = helpers.each;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_cell";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "configData.hasCompareDiffs";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(24, program24, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.compareConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "compareConfig";
  stack7 = helpers.each;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "compareConfig.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_cell";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.overrides";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "compareConfig";
  stack7 = helpers.each;
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_cell";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = ":row :comparison-row :bottom-border configData.isHiddenByFilter:hide configData.hasCompareDiffs:has-compare-diffs";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "property-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"col-md-3 property-name-column\">\n    ");
  stack1 = depth0;
  stack2 = "configData.showLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"col-md-4 property-value-column\">\n    ");
  stack1 = depth0;
  stack2 = "controller.selectedConfigGroup.isDefault";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"col-md-4 col-md-offset-1 property-value-column\">\n    ");
  stack1 = depth0;
  stack2 = "controller.selectedConfigGroup.isDefault";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/controls", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SwitchToGroupView";
  stack3 = {};
  stack4 = "controller";
  stack3['controllerBinding'] = stack4;
  stack4 = "view.serviceConfigProperty.group";
  stack3['groupBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showIsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showOverride";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showUndo";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showRemove";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showSetRecommended";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a href=\"#\" data-toggle=\"tooltip\"\n      ");
  stack1 = {};
  stack2 = ":btn-sm :display-inline-block :btn-final view.serviceConfigProperty.isFinal:active view.serviceConfigProperty.hideFinalIcon:hidden";
  stack1['class'] = stack2;
  stack2 = "view.serviceConfigProperty.isNotEditable";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty";
  stack3 = depth0;
  stack4 = "toggleFinalFlag";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "services.service.config.final";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"glyphicon glyphicon-lock\"></i>\n    </a>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a href=\"#\" data-toggle=\"tooltip\" class=\"btn-sm\"\n      ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty";
  stack3 = depth0;
  stack4 = "createOverrideProperty";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "common.override";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"glyphicon glyphicon-plus-sign\"></i>\n    </a>\n  ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a href=\"#\" data-toggle=\"tooltip\" class=\"btn-sm\"\n      ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty";
  stack3 = depth0;
  stack4 = "doRestoreDefaultValue";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "common.undo";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"icon-undo\"></i>\n    </a>\n  ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a href=\"#\" data-toggle=\"tooltip\" class=\"btn-sm remove-config-property-button\"\n      ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty";
  stack3 = depth0;
  stack4 = "removeProperty";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "common.remove";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"glyphicon glyphicon-minus-sign\"></i>\n    </a>\n  ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a href=\"#\" data-toggle=\"tooltip\"\n      ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty";
  stack3 = depth0;
  stack4 = "setRecommendedValue";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "services.service.config.setRecommendedValue";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"icon-repeat\"></i>\n    </a>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.showSwitchToGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/notifications_configs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = ":entry-row :row config.rowStyleClass config.isUserProperty:indent-1";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <span ");
  stack1 = {};
  stack2 = "config.errorMessage:has-error :form-group :control-label-span :col-md-3";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <label class=\"control-label\">\n                  ");
  stack1 = depth0;
  stack2 = "config.displayName";
  foundHelper = helpers.formatWordBreak;
  stack3 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </label>\n              </span>\n\n              <div class=\"col-md-9\">\n                <div ");
  stack1 = {};
  stack2 = "config.errorMessage:has-error config.warnMessage:warning: :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <div class=\"row\">\n                    <div class=\"col-md-9\">\n                        ");
  stack1 = depth0;
  stack2 = "config.viewClass";
  stack3 = {};
  stack4 = "config";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "view.categoryConfigsAll";
  stack3['categoryConfigsAllBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </div>\n                    <div class=\"col-md-3\">\n                        ");
  stack1 = depth0;
  stack2 = "config.isRemovable";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "config.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "config.warnMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                  </div>\n                </div>\n              </div>\n            </div>\n          ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "config.isComparison";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                                  <a class=\"btn-sm\" href=\"#\" data-toggle=\"tooltip\"\n                                      ");
  stack1 = depth0;
  stack2 = "config";
  stack3 = depth0;
  stack4 = "removeProperty";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                      ");
  stack1 = {};
  stack2 = "common.remove";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                                    <i class=\"icon-minus-sign\"></i>\n                                  </a>\n                                ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "config.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                      ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "config.warnMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                      ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.canEdit";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.configsAreDisabled";
  stack3 = helpers.unless;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <div class=\"entry-row indent-1\">\n                      <a\n                        href=\"#\" ");
  stack1 = depth0;
  stack2 = "";
  stack3 = depth0;
  stack4 = "showAddPropertyWindow";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >");
  stack1 = depth0;
  stack2 = "installer.step7.config.addProperty";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        ...</a>\n                    </div>\n                  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"panel panel-default\">\n  <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "onToggleBlock";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <h3 class=\"panel-title\">\n      <i ");
  stack1 = {};
  stack2 = ":pull-left :panel-toggle view.category.isCollapsed:icon-caret-right:icon-caret-down";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n      <a class=\"panel-toggle category-header\"><span class=\"category-name\">");
  stack1 = depth0;
  stack2 = "view.category.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></a>\n    </h3>\n  </div>\n\n  <div class=\"panel-body collapse in\" ");
  stack1 = {};
  stack2 = "view.isCategoryBodyVisible";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"service-config-section\">\n      <form autocomplete=\"off\" class=\"form-horizontal\">\n\n        <div class=\"entry-row row\">\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "createNotification";
  stack3['name'] = stack4;
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "installer.step7.misc.notification.configure.later";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.createNotification";
  stack3['selectionBinding'] = stack4;
  stack4 = "no";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        <div class=\"entry-row row\">\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "createNotification";
  stack3['name'] = stack4;
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "installer.step7.misc.notification.configure";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.createNotification";
  stack3['selectionBinding'] = stack4;
  stack4 = "yes";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n          ");
  stack1 = depth0;
  stack2 = "view.categoryConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "config";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n        <div class=\"entry-row row indent-1\">\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "installer.step7.misc.notification.use_tls";
  stack3['labelTranslate'] = stack4;
  stack4 = "tlsOrSsl";
  stack3['name'] = stack4;
  stack4 = "view.configsAreDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "view.tlsOrSsl";
  stack3['selectionBinding'] = stack4;
  stack4 = "tls";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        <div class=\"entry-row row indent-1\">\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "installer.step7.misc.notification.use_ssl";
  stack3['labelTranslate'] = stack4;
  stack4 = "tlsOrSsl";
  stack3['name'] = stack4;
  stack4 = "view.configsAreDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "view.tlsOrSsl";
  stack3['selectionBinding'] = stack4;
  stack4 = "ssl";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n          ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </form>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/overriddenProperty", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <a href=\"#\" data-toggle=\"tooltip\"\n          ");
  stack1 = {};
  stack2 = ":btn-sm :display-inline-block :btn-final view.overriddenSCP.isFinal:active view.overriddenSCP.hideFinalIcon:hidden";
  stack1['class'] = stack2;
  stack2 = "view.overriddenSCP.isNotEditable";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP";
  stack3 = depth0;
  stack4 = "toggleFinalFlag";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = {};
  stack2 = "services.service.config.final";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"glyphicon glyphicon-lock\"></i>\n        </a>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.group";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SwitchToGroupView";
  stack3 = {};
  stack4 = "controller";
  stack3['controllerBinding'] = stack4;
  stack4 = "view.overriddenSCP.group";
  stack3['groupBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.isEditable";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "isNotDefaultValue";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <a class=\"btn-sm\" href=\"#\" data-toggle=\"tooltip\"\n              ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP";
  stack3 = depth0;
  stack4 = "doRestoreDefaultValue";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = {};
  stack2 = "common.undo";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-undo\"></i>\n            </a>\n          ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <a class=\"btn-sm\" href=\"#\" data-toggle=\"tooltip\"\n              ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP";
  stack3 = depth0;
  stack4 = "removeOverride";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = {};
  stack2 = "common.remove";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-minus-sign\"></i>\n            </a>\n          ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"help-block validation-block pull-right\" ");
  stack1 = depth0;
  stack2 = "validation-block";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "controller.isHostsConfigsPage";
  stack3 = helpers.unless;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <a class=\"action\">");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.group.switchGroupTextShort";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  return buffer;}

  data.buffer.push("\n");
  data.buffer.push("\n<div ");
  stack1 = {};
  stack2 = "view.overriddenSCP.errorMessage:has-error: :form-group :overrideField";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "override-field";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"row\">\n    <div class=\"col-md-9\">\n      ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.viewClass";
  stack3 = {};
  stack4 = "view.overriddenSCP";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "view.parentView.categoryConfigs";
  stack3['categoryConfigsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"col-md-3 config-controls\">\n      ");
  stack1 = depth0;
  stack2 = "view.overriddenSCP.supportsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.isDefaultGroupSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/overriddenPropertyRow", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty.overrides";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "overriddenSCP";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView.SCPOverriddenPropertyView";
  stack3 = {};
  stack4 = "overriddenSCP";
  stack3['overriddenSCPBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView.SCPOverriddenPropertyView";
  stack3 = {};
  stack4 = "view.serviceConfigProperty";
  stack3['overriddenSCPBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/overrideWindow", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"text-warning\">\n            ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  
  data.buffer.push("\n              <i class='glyphicon glyphicon-ok-sign'></i>\n            ");}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                      <li>\n                        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "column";
  stack3 = depth0;
  stack4 = "selectFilterColumn";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                          ");
  stack1 = depth0;
  stack2 = "column.selected";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "column.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        </a>\n                      </li>\n                    ");
  return buffer;}
function program6(depth0,data) {
  
  
  data.buffer.push("\n                            <i class='glyphicon glyphicon-ok-sign'></i>\n                          ");}

function program8(depth0,data) {
  
  
  data.buffer.push("\n                            <i class='glyphicon glyphicon-placeholder'></i>\n                          ");}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <li>\n                      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "component";
  stack3 = depth0;
  stack4 = "selectFilterComponent";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        ");
  stack1 = depth0;
  stack2 = "component.selected";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "component.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </a>\n                    </li>\n                  ");
  return buffer;}
function program11(depth0,data) {
  
  
  data.buffer.push("\n                          <i class='glyphicon glyphicon-ok-sign'></i>\n                        ");}

function program13(depth0,data) {
  
  
  data.buffer.push("\n                          <i class='glyphicon glyphicon-placeholder'></i>\n                        ");}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <tr ");
  stack1 = {};
  stack2 = "entry.filtered::hidden entry.selected:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <td>\n                    ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "entry.selected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                  <td>\n                    ");
  stack1 = depth0;
  stack2 = "entry.host.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                  <td>\n                    ");
  stack1 = depth0;
  stack2 = "entry.filterColumnValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                </tr>\n              ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <table class=\"table table-hover\">\n            <tfoot>\n              <tr>\n                <td>\n                  ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n              </tr>\n            </tfoot>\n          </table>\n        ");
  return buffer;}

  data.buffer.push("\n\n");
  data.buffer.push("\n  <form autocomplete=\"off\" class=\"form-horizontal mbm\">\n      <div class=\"override-controls\">\n        <div class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "view.parentView.dialogMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n        <div class=\"row\">\n          <div class=\"col-md-6 selected-hosts-link\">\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleShowSelectedHosts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.hostSelectMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  stack1 = depth0;
  stack2 = "view.showOnlySelectedHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"col-md-6\">\n            <div class=\"col-md-7\" id=\"filter-dropdown-div\">\n              <!-- Filter text box with drop-down -->\n              <div class=\"input-group\">\n                ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.filterText";
  stack3['valueBinding'] = stack4;
  stack4 = "appendedDropdownButton";
  stack3['id'] = stack4;
  stack4 = "view.filterTextPlaceholder";
  stack3['placeholderBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                <div class=\"input-group-btn\">\n                  <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" ");
  stack1 = {};
  stack2 = "view.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <span class=\"caret\"></span>\n                  </button>\n                  <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "view.filterColumns";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "column";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </div>\n              </div>\n            </div>\n            <div class=\"col-md-5\" id=\"component-dropdown-div\">\n              <!-- Host-components drop-down -->\n              <div>\n                <button class=\"btn btn-block btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\" ");
  stack1 = {};
  stack2 = "view.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  <span class=\"caret\"></span>\n                </button>\n                <ul class=\"dropdown-menu\">\n                  <!-- dropdown menu links -->\n                  ");
  stack1 = depth0;
  stack2 = "view.filterComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n              </div>\n            </div>\n          </div>\n        </div>\n\n        <table class=\"table table-hover\">\n          <thead>\n            <tr>\n              <th>\n                ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.allHostsSelected";
  stack3['checkedBinding'] = stack4;
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </th>\n              <th>");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n              <th>");
  stack1 = depth0;
  stack2 = "view.filterColumn.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n            </tr>\n          </thead>\n         </table>\n        <div class=\"hosts-table-container\">\n          <table class=\"table table-hover\">\n            <tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "entry";
  stack7 = helpers.each;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tbody>\n         </table>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n  </form>\n\n");
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/propertyDependence", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "property.serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "property.propertyDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "property.curValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "property.newValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n    </tr>\n  ");
  return buffer;}

  data.buffer.push("\n\n<table class=\"table table-hover\">\n  <thead>\n  <tr>\n    <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th>");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th>");
  stack1 = depth0;
  stack2 = "installer.step7.popup.currentValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th>");
  stack1 = depth0;
  stack2 = "installer.step7.popup.adjustedValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n  </tr>\n  </thead>\n  <tbody>\n  ");
  stack1 = depth0;
  stack2 = "view.propertyChange";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "property";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/queuePopup_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div class=\"rw-fluid control-group\">\n        <div ");
  stack1 = {};
  stack2 = "errorMessage:error :control-label-span :col-md-4";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <label>");
  stack1 = depth0;
  stack2 = "displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n        </div>\n        <div ");
  stack1 = {};
  stack2 = "errorMessage:error :form-group :col-md-8";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "viewClass";
  stack3 = {};
  stack4 = "this";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "view.categoryConfigs";
  stack3['categoryConfigsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

  data.buffer.push("\n\n<form autocomplete=\"off\" class=\"form-horizontal pre-scrollable\">\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.content.configs";
  stack3 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/queuePopup_footer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"pull-left\">\n    <button class=\"btn btn-danger\" ");
  stack1 = depth0;
  stack2 = "onDelete";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.delete";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <button type=\"button\" class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <button type=\"button\" ");
  stack1 = {};
  stack2 = "view.parentView.isError";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "onSecondary";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.secondary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.parentView.delete";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<p class=\"pull-right\">\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.secondary";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</p>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/saveConfigGroup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"config-group-confirm-save\">\n  <div class=\"alert alert-success\">");
  stack1 = depth0;
  stack2 = "config.group.save.confirmation.configGroup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <em>");
  stack1 = depth0;
  stack2 = "groupName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</em> ");
  stack1 = depth0;
  stack2 = "config.group.save.confirmation.saved";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n  <div class=\"message\">");
  stack1 = depth0;
  stack2 = "config.group.save.confirmation.msg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/save_configuration", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"row\">\n        <div class=\"col-md-12 alert alert-warning\">");
  stack1 = depth0;
  stack2 = "services.service.config.exitPopup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <p class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.info-bar.save.popup.warningForPasswordChange";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.showSaveWarning";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.showPasswordChangeWarning";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<div class=\"row\">\n    <div class=\"col-md-1\">");
  stack1 = depth0;
  stack2 = "common.notes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-10\">");
  stack1 = depth0;
  stack2 = "view.notesArea";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/selectCreateConfigGroup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div class=\"select-create-config-group-div\">\n        <div class=\"btn-group\">\n          <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "view.parentView.optionCreateConfigGroup";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.parentView.selectedConfigGroup.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</button>\n          <button class=\"btn btn-default dropdown-toggle btn-icon\" data-toggle=\"dropdown\" ");
  stack1 = {};
  stack2 = "view.parentView.optionCreateConfigGroup";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <span class=\"caret\"></span>\n          </button>\n          <ul class=\"dropdown-menu\">\n            <!-- available config group menu links -->\n            ");
  stack1 = depth0;
  stack2 = "view.parentView.availableConfigGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configGroup";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </div>\n        <div>");
  stack1 = depth0;
  stack2 = "config.group.selection.dialog.option.select.msg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n               <li>\n                 <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "configGroup";
  stack3 = depth0;
  stack4 = "doSelectConfigGroup";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                   ");
  stack1 = depth0;
  stack2 = "configGroup.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                 </a>\n               </li>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"select-create-config-group-div\">\n        <div class=\"alert alert-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.noGroups";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    ");
  return buffer;}

  data.buffer.push("\n<div id=\"config-group-select-create-dialog\">\n  <div class=\"alert alert-info\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.subTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div class=\"text-warning\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <form>\n    <div>\n      ");
  stack1 = depth0;
  stack2 = "view.selectConfigGroupRadioButton";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.availableConfigGroups.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div>\n      ");
  stack1 = depth0;
  stack2 = "view.createConfigGroupRadioButton";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"select-create-config-group-div\">\n      <div class=\"row\">\n        <div class=\"col-md-5\">\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.parentView.newConfigGroupName";
  stack3['valueBinding'] = stack4;
  stack4 = "view.parentView.optionSelectConfigGroup";
  stack3['disabledBinding'] = stack4;
  stack4 = "255";
  stack3['maxlength'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"clearfix\"></div>\n      <div>");
  stack1 = depth0;
  stack2 = "view.parentView.createNewGroupDescription";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </form>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_config", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "selectedService.restartRequired";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"alert\">\n      <div class=\"clearfix like_pointer collapsable\" ");
  stack1 = depth0;
  stack2 = "toggleRestartMessageView";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.isRestartMessageCollapsed";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <i class=\"glyphicon glyphicon-refresh\"></i>\n        <strong>");
  stack1 = depth0;
  stack2 = "dashboard.services.configs.popup.restartService.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong> ");
  stack1 = depth0;
  stack2 = "selectedService.restartRequiredMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"service-body\">\n        <ul>\n          ");
  stack1 = depth0;
  stack2 = "selectedService.restartRequiredHostsAndComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </div>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  
  data.buffer.push("\n          <i class=\"glyphicon glyphicon-caret-down pull-left\"></i>\n        ");}

function program5(depth0,data) {
  
  
  data.buffer.push("\n          <i class=\"iglyphicon glyphicon-caret-right pull-left\"></i>\n        ");}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "host.hostData";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.hostData.publicHostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "host.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </li>\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <li>");
  stack1 = depth0;
  stack2 = "component.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "allVersionsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.ConfigVersionsControlView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "controller.isCompareMode:pull-bottom :pull-top controller.isHostsConfigsPage:pull-bottom :config-manage-nav :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "controller.isHostsConfigsPage";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.configGroup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;\n      ");
  stack1 = depth0;
  stack2 = "controller.configGroupsAreLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(24, program24, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.FilterComboCleanableView";
  stack3 = {};
  stack4 = "col-lg-4";
  stack3['classNames'] = stack4;
  stack4 = "view.filter";
  stack3['filterBinding'] = stack4;
  stack4 = "view.columns";
  stack3['columnsBinding'] = stack4;
  stack4 = "view.propertyFilterPopover";
  stack3['popoverDescriptionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n    </div>\n  ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "SERVICE.MANAGE_CONFIG_GROUPS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          &nbsp;<a href=\"#\" ");
  stack1 = depth0;
  stack2 = "switchHostGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.change";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div class=\"btn-group config-groups-dropdown\">\n          <button ");
  stack1 = {};
  stack2 = "controller.isHostsConfigsPage";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            class=\"btn btn-default dropdown-toggle\"\n            data-toggle=\"dropdown\">\n            <span class=\"caret pull-caret-right\"></span>\n            ");
  stack1 = depth0;
  stack2 = "selectedConfigGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n          <ul class=\"dropdown-menu config-groups-dropdown-menu\">\n            ");
  stack1 = depth0;
  stack2 = "SERVICE.MANAGE_CONFIG_GROUPS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <!-- available config group menu links -->\n            ");
  stack1 = depth0;
  stack2 = "configGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configGroup";
  stack7 = helpers.each;
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </div>\n      ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <li>\n                <a href=\"#\" class=\"link-left-pad\" ");
  stack1 = depth0;
  stack2 = "manageConfigurationGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "services.service.actions.manage_configuration_groups.short";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </li>\n              <li role=\"separator\" class=\"divider\"></li>\n            ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "configGroup";
  stack3 = depth0;
  stack4 = "selectConfigGroup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "configGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "pull-left";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "hideDependenciesInfoBar";
  stack3 = helpers.unless;
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.supportsConfigLayout";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(53, program53, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isAllConfigsHidden";
  stack3 = helpers['if'];
  tmp1 = self.program(59, program59, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "hasChangedDependencies:show:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"dependencies-info-bar-wrapper\">\n        <div class=\"alert alert-warning\">\n          <span>");
  stack1 = depth0;
  stack2 = "dependenciesMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <a\n            href=\"#\" ");
  stack1 = depth0;
  stack2 = "showChangedDependentConfigs";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "showSelectGroupsPopup";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  ");
  return buffer;}
function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"alert alert-warning\">\n            <span>");
  stack1 = depth0;
  stack2 = "dependenciesGroupMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "changedDependentGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </div>\n        ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <ul class=\"nav nav-tabs mbm background-text\">\n      ");
  stack1 = depth0;
  stack2 = "view.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n    <div class=\"tab-content service-config-tab-content\">\n      ");
  stack1 = depth0;
  stack2 = "view.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(38, program38, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <li rel='tooltip' ");
  stack1 = {};
  stack2 = "tab.isActive:active tab.isHiddenByFilter:disabled";
  stack1['class'] = stack2;
  stack2 = "tab.tooltipMsg";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "tab";
  stack3 = depth0;
  stack4 = "setActiveTab";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "tab.headingClass";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"tab\">\n            ");
  stack1 = depth0;
  stack2 = "tab.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "tab.isAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(35, program35, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </a>\n        </li>\n      ");
  return buffer;}
function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "controller.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "controller.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program35(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(36, program36, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program38(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":tab-pane tab.isActive:active tab.id";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "tab.isAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(48, program48, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "tab.isRendered";
  stack3 = helpers['if'];
  tmp1 = self.program(40, program40, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(46, program46, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program40(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "isCompareMode";
  stack3 = helpers['if'];
  tmp1 = self.program(41, program41, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.ConfigCategoryContainerView";
  stack3 = {};
  stack4 = "selectedService.configCategories";
  stack3['categoriesBinding'] = stack4;
  stack4 = "view.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = "selectedService";
  stack3['serviceBinding'] = stack4;
  stack4 = "selectedService.configs";
  stack3['serviceConfigsBinding'] = stack4;
  stack4 = "view.supportsHostOverrides";
  stack3['supportsHostOverridesBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}
function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"row bottom-border comparison-table-header\">\n                  <div class=\"col-md-3 property-name-column\">");
  stack1 = depth0;
  stack2 = "common.propertyName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                  <div class=\"col-md-4 property-value-column\">\n                    ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "selectedVersionRecord.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "selectedVersionRecord.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(42, program42, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                  <div class=\"col-md-4 col-md-offset-1 property-value-column\">\n                    ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "compareServiceVersion.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "compareServiceVersion.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                </div>\n              ");
  return buffer;}
function program42(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    ");
  return buffer;}

function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    ");
  return buffer;}

function program46(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program48(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "isCompareMode";
  stack3 = helpers['if'];
  tmp1 = self.program(49, program49, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(51, program51, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program49(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigLayoutTabCompareView";
  stack3 = {};
  stack4 = "tab";
  stack3['contentBinding'] = stack4;
  stack4 = "selectedVersionRecord";
  stack3['primaryCompareVersionBinding'] = stack4;
  stack4 = "compareServiceVersion";
  stack3['secondaryCompareVersionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program51(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigLayoutTabView";
  stack3 = {};
  stack4 = "tab";
  stack3['contentBinding'] = stack4;
  stack4 = "view.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program53(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"tab-content service-config-tab-content\">\n      ");
  stack1 = depth0;
  stack2 = "isCompareMode";
  stack3 = helpers['if'];
  tmp1 = self.program(54, program54, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.ConfigCategoryContainerView";
  stack3 = {};
  stack4 = "selectedService.configCategories";
  stack3['categoriesBinding'] = stack4;
  stack4 = "view.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = "selectedService";
  stack3['serviceBinding'] = stack4;
  stack4 = "selectedService.configs";
  stack3['serviceConfigsBinding'] = stack4;
  stack4 = "view.supportsHostOverrides";
  stack3['supportsHostOverridesBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}
function program54(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"row bottom-border comparison-table-header accordion-without-tabs\">\n          <div class=\"col-md-3 property-name-column\">");
  stack1 = depth0;
  stack2 = "common.propertyName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"col-md-4 property-value-column\">\n            ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "selectedVersionRecord.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "selectedVersionRecord.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(55, program55, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"col-md-4 col-md-offset-1 property-value-column\">\n            ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "compareServiceVersion.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "compareServiceVersion.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(57, program57, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      ");
  return buffer;}
function program55(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n            ");
  return buffer;}

function program57(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n            ");
  return buffer;}

function program59(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"alert alert-info\">\n      ");
  stack1 = depth0;
  stack2 = "services.service.config.nothing.to.display";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

function program61(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.supportsHostOverrides";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n<div class=\"config-layout clearfix\">\n  <div class=\"mbm\">\n    ");
  stack1 = depth0;
  stack2 = "view.isOnTheServicePage";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.supportsHostOverrides";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n");
  stack1 = depth0;
  stack2 = "versionLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(61, program61, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.showSavePanel::hidden :configs-save-panel";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"pull-right\">\n    <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "doCancel";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "view.isDiscardDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.discard";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </button> &nbsp;\n    <button class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "save";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "view.isSaveDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  </div>\n  <div class=\"clearfix\"></div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_config_category", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "view.category.errorCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "configData.isInDefaultTheme";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push(" ");
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "configData.isComparison";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_row";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "controller.isCompareMode";
  stack3 = helpers.unless;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div ");
  stack1 = {};
  stack2 = ":entry-row :form-group configData.isHiddenByFilter:hide configData.isOverridden:overridden-property configData.hasCompareDiffs:overridden-property";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "property-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "configData.showLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "configData.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                <div class=\"col-md-9\">\n                  ");
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "configData.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "configData.isOverridden";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </div>\n              ");
  stack1 = depth0;
  stack2 = "configData.additionalView";
  stack3 = helpers['if'];
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "configData.rightSideLabel";
  stack3 = helpers.unless;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <span ");
  stack1 = {};
  stack2 = "configData.errorMessage:has-error: :form-group :control-label-span :col-md-3";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <label class=\"control-label\" ");
  stack1 = depth0;
  stack2 = "control-label";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "configData.displayName";
  foundHelper = helpers.formatWordBreak;
  stack3 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "configData.isSecureConfig";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </label>\n                </span>\n                  ");
  return buffer;}
function program11(depth0,data) {
  
  
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">\n                        <i class=\"glyphicon glyphicon-lock\" rel=\"tooltip\" data-toggle=\"tooltip\" title=\"security knob\"></i>\n                      </a>\n                    ");}

function program13(depth0,data) {
  
  
  data.buffer.push("\n                    <span class=\"control-group control-label-span col-md-3\"> </span>\n                  ");}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "configData.rightSideLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <span ");
  stack1 = {};
  stack2 = "configData.isEditable::muted :col-md-3 :clear-left :form-group :control-label-span";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "configData.displayName";
  foundHelper = helpers.formatWordBreak;
  stack3 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    <div ");
  stack1 = {};
  stack2 = "configData.errorMessage:has-error: configData.warnMessage:has-warning: :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "property-form-group";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "configData.viewClass";
  stack3 = {};
  stack4 = "configData";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "view.categoryConfigsAll";
  stack3['categoryConfigsAllBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "configData.hintMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "App.ControlsView";
  stack3 = {};
  stack4 = "configData";
  stack3['serviceConfigPropertyBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "configData.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "configData.warnMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                  ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n                        <a class=\"icon-question-sign glyphicon glyphicon-blue\" href=\"#\"\n                           data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "configData.hintMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n                      ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <span class=\"help-block validation-block\" ");
  stack1 = depth0;
  stack2 = "validation-block";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "configData.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                      ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <span class=\"has-warning\">\n                      <span class=\"help-block validation-block\" ");
  stack1 = depth0;
  stack2 = "validation-block";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "configData.warnMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    </span>\n                      ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView.SCPOverriddenRowsView";
  stack3 = {};
  stack4 = "configData";
  stack3['serviceConfigPropertyBinding'] = stack4;
  stack4 = "controller.selectedConfigGroup.isDefault";
  stack3['isDefaultGroupSelectedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "configData.additionalView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.canEdit";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.category.customCanAddProperty";
  stack3 = helpers['if'];
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <div>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "";
  stack3 = depth0;
  stack4 = "showAddPropertyWindow";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >");
  stack1 = depth0;
  stack2 = "installer.step7.config.addProperty";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ...</a>\n            </div>\n          ");
  return buffer;}

  data.buffer.push("\n<div class=\"panel panel-default\">\n  <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "onToggleBlock";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <h3 class=\"panel-title\">\n      <i ");
  stack1 = {};
  stack2 = ":pull-right :panel-toggle view.category.isCollapsed:icon-angle-down:icon-angle-up";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n      <a class=\"panel-toggle category-header\" ");
  stack1 = depth0;
  stack2 = "category-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <span class=\"category-name\" ");
  stack1 = depth0;
  stack2 = "category-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.category.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        ");
  stack1 = depth0;
  stack2 = "view.category.errorCount";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </a>\n    </h3>\n  </div>\n\n  <div class=\"panel-body collapse in\" ");
  stack1 = {};
  stack2 = "view.isCategoryBodyVisible";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "panel-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"service-config-section\">\n    <form autocomplete=\"off\" class=\"form-horizontal\">\n      ");
  stack1 = depth0;
  stack2 = "view.categoryConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configData";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </form>\n  </div>\n</div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_config_layout_tab", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"section-layout-container\">\n    <div class=\"loading-overlay\"></div>\n    <table class=\"config-section-table\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.sectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </table>\n  </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <tr>\n          ");
  stack1 = depth0;
  stack2 = "row";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "section";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tr>\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <td ");
  stack1 = {};
  stack2 = "section.isHiddenByFilter:invisible :config-section";
  stack1['class'] = stack2;
  stack2 = "section.columnSpan";
  stack1['colspan'] = stack2;
  stack2 = "section.rowSpan";
  stack1['rowspan'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <h4>");
  stack1 = depth0;
  stack2 = "section.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n              <table class=\"config-subsection-table\">\n                ");
  stack1 = depth0;
  stack2 = "section.subsectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subRow";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </table>\n            </td>\n          ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <tr>\n                    ");
  stack1 = depth0;
  stack2 = "subRow";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subsection";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </tr>\n                ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      <td ");
  stack1 = {};
  stack2 = "subsection.isSectionVisible::invisible subsection.showTopSplitter:top-horizontal-splitter:no-horizontal-splitter :config-subsection";
  stack1['class'] = stack2;
  stack2 = "subsection.columnSpan";
  stack1['colspan'] = stack2;
  stack2 = "subsection.rowSpan";
  stack1['rowspan'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        <div ");
  stack1 = {};
  stack2 = "subsection.addLeftVerticalSplitter:vertical-splitter-l";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                          <div ");
  stack1 = {};
  stack2 = "subsection.border:with-border";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                            <h5 class=\"subsection-display-name\">\n                              ");
  stack1 = depth0;
  stack2 = "subsection.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  stack1 = depth0;
  stack2 = "subRow.hasTitleGap";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </h5>\n                            ");
  stack1 = depth0;
  stack2 = "subsection.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "config";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "subsection.showTabs";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          </div>\n                        </div>\n                      </td>\n                    ");
  return buffer;}
function program7(depth0,data) {
  
  
  data.buffer.push("\n                                &nbsp;\n                              ");}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "config.widget";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "config.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                  ");
  stack1 = depth0;
  stack2 = "config.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                    ");
  stack1 = depth0;
  stack2 = "config.widget";
  stack3 = {};
  stack4 = "config";
  stack3['configBinding'] = stack4;
  stack4 = "view.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = "section";
  stack3['sectionBinding'] = stack4;
  stack4 = "subsection";
  stack3['subSectionBinding'] = stack4;
  stack4 = "view.content";
  stack3['tabBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                    ");
  stack1 = depth0;
  stack2 = "config.additionalView";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                  ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                      ");
  stack1 = depth0;
  stack2 = "config.additionalView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                    ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                              <ul class=\"nav nav-tabs mbm\">\n                                ");
  stack1 = depth0;
  stack2 = "subsection.subSectionTabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subSectionTab";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              </ul>\n                              <div class=\"tab-content service-config-tab-content\">\n                              ");
  stack1 = depth0;
  stack2 = "subsection.subSectionTabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subSectionTab";
  stack7 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              </div>\n                            ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                                  <li rel='tooltip' ");
  stack1 = {};
  stack2 = "subSectionTab.isActive:active subSectionTab.isVisible::disabled";
  stack1['class'] = stack2;
  stack2 = "view.content.tooltipMsg";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "subSectionTab";
  stack3 = depth0;
  stack4 = "setActiveSubTab";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  stack1 = {};
  stack2 = "subSectionTab.id";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"tab\">\n                                      ");
  stack1 = depth0;
  stack2 = "subSectionTab.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                      ");
  stack1 = depth0;
  stack2 = "subSectionTab.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                    </a>\n                                  </li>\n                                ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                        <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "subSectionTab.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                                      ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "subSectionTab.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "config";
  stack7 = helpers.each;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                  <div ");
  stack1 = {};
  stack2 = ":tab-pane subSectionTab.isActive:active subSectionTab.id";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                    ");
  stack1 = depth0;
  stack2 = "config.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                  </div>\n                                ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                      ");
  stack1 = depth0;
  stack2 = "config.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                    ");
  return buffer;}
function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                        ");
  stack1 = depth0;
  stack2 = "config.widget";
  stack3 = {};
  stack4 = "config";
  stack3['configBinding'] = stack4;
  stack4 = "view.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = "section";
  stack3['sectionBinding'] = stack4;
  stack4 = "subsection";
  stack3['subSectionBinding'] = stack4;
  stack4 = "view.content";
  stack3['tabBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                        ");
  stack1 = depth0;
  stack2 = "config.additionalView";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                      ");
  return buffer;}
function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                          ");
  stack1 = depth0;
  stack2 = "config.additionalView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                        ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.dataIsReady";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_config_layout_tab_compare", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"row bottom-border comparison-table-header\">\n      <div class=\"col-md-3 property-name-column\">");
  stack1 = depth0;
  stack2 = "common.propertyName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"col-md-4 property-value-column\">\n        ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "view.primaryCompareVersion.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.primaryCompareVersion.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-4 col-md-offset-1 property-value-column\">\n        ");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "view.secondaryCompareVersion.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.secondaryCompareVersion.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n    <div class=\"accordion\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.sectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "row";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "section";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div class=\"panel-group\">\n            <div class=\"panel panel-default\">\n              <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "section";
  stack3 = depth0;
  stack4 = "onToggleBlock";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <h3 class=\"panel-title\">\n                  <i ");
  stack1 = {};
  stack2 = ":pull-right :panel-toggle section.isCollapsed:icon-angle-down:icon-angle-up";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                  <a class=\"panel-toggle category-header\" ");
  stack1 = depth0;
  stack2 = "category-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <span class=\"category-name\" ");
  stack1 = depth0;
  stack2 = "category-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "section.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  </a>\n                </h3>\n              </div>\n              <div class=\"panel-body collapse in\" ");
  stack1 = {};
  stack2 = "view.isCategoryBodyVisible";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "panel-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <div class=\"service-config-section\">\n                  ");
  stack1 = depth0;
  stack2 = "section.subsectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subRow";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </div>\n            </div>\n          </div>\n        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "subRow";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subsection";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "subsection.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configData";
  stack7 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "subsection.showTabs";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "configData.widget";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "configData.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_row";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "subsection.subSectionTabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subSectionTab";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "subSectionTab.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configData";
  stack7 = helpers.each;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "configData.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "templates/common/configs/configs_comparison_row";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.dataIsReady";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_config_wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"alert alert-info config-manage-nav\">\n      ");
  stack1 = depth0;
  stack2 = "common.group";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;\n      <div class=\"btn-group config-groups-dropdown\">\n        <button ");
  stack1 = {};
  stack2 = "controller.isHostsConfigsPage";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default dropdown-toggle first\" data-toggle=\"dropdown\">\n          <span class=\"caret pull-caret-right\"></span>\n          ");
  stack1 = depth0;
  stack2 = "selectedConfigGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </button>\n        <ul class=\"dropdown-menu config-groups-dropdown-menu\">\n          <!-- available config group menu links -->\n          ");
  stack1 = depth0;
  stack2 = "configGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configGroup";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "SERVICE.MANAGE_CONFIG_GROUPS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </div>\n      <div class=\"pull-right\">\n        ");
  stack1 = depth0;
  stack2 = "App.FilterComboCleanableView";
  stack3 = {};
  stack4 = "view.filter";
  stack3['filterBinding'] = stack4;
  stack4 = "view.columns";
  stack3['columnsBinding'] = stack4;
  stack4 = "view.propertyFilterPopover";
  stack3['popoverDescriptionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"clearfix\"></div>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "configGroup";
  stack3 = depth0;
  stack4 = "selectConfigGroup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "configGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <li class=\"divider\"></li>\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "manageConfigurationGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "services.service.actions.manage_configuration_groups.short";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = ":config-actions view.actionsStacked:pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"pull-right\">\n        ");
  stack1 = depth0;
  stack2 = "App.FilterComboCleanableView";
  stack3 = {};
  stack4 = "view.filter";
  stack3['filterBinding'] = stack4;
  stack4 = "view.columns";
  stack3['columnsBinding'] = stack4;
  stack4 = "view.propertyFilterPopover";
  stack3['popoverDescriptionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"pull-right\">\n        <span class=\"btn-group config-groups-dropdown\">\n          <button ");
  stack1 = {};
  stack2 = "controller.isHostsConfigsPage";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default dropdown-toggle first\" data-toggle=\"dropdown\">");
  stack1 = depth0;
  stack2 = "selectedConfigGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</button>\n          <button ");
  stack1 = {};
  stack2 = "controller.isHostsConfigsPage";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n            <span class=\"caret\"></span>\n          </button>\n          <ul class=\"dropdown-menu config-groups-dropdown-menu\">\n            <!-- available config group menu links -->\n            ");
  stack1 = depth0;
  stack2 = "configGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "configGroup";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </span>\n      </div>\n    </div>\n  ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "configGroup";
  stack3 = depth0;
  stack4 = "selectConfigGroup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "configGroup.displayNameHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "SERVICE.MANAGE_CONFIG_GROUPS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <li class=\"divider\"></li>\n                <li>\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "manageConfigurationGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "services.service.actions.manage_configuration_groups.short";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.hideTabHeaders";
  stack3 = helpers.unless;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = ":tab-content :service-config-tab-content";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <ul ");
  stack1 = {};
  stack2 = ":nav :nav-tabs :mbm view.supportsHostOverrides:with-overrides controller.isInstallWizard:clear";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "tab.isHidden";
  stack3 = helpers.unless;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li rel='tooltip' ");
  stack1 = {};
  stack2 = "tab.isActive:active tab.isHiddenByFilter:disabled";
  stack1['class'] = stack2;
  stack2 = "tab.tooltipMsg";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "tab";
  stack3 = depth0;
  stack4 = "setActiveTab";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "tab.headingClass";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n               data-toggle=\"tab\">\n              ");
  stack1 = depth0;
  stack2 = "tab.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "tab.isAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(20, program20, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </a>\n          </li>\n        ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "controller.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "controller.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":tab-pane tab.isActive:active tab.id";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "tab.isAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(27, program27, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.serviceConfigsByCategoryView.childViews";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "catView";
  stack7 = helpers.each;
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "catView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigLayoutTabView";
  stack3 = {};
  stack4 = "tab";
  stack3['contentBinding'] = stack4;
  stack4 = "view.themeTemplate";
  stack3['customTemplateBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.serviceConfigsByCategoryView.childViews";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "catView";
  stack7 = helpers.each;
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "catView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-info col-sm-12\">\n    ");
  stack1 = depth0;
  stack2 = "services.service.config.nothing.to.display";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.supportsHostOverrides";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.supportsConfigLayout";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(29, program29, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.isAllConfigsHidden";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/service_version_box", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n        <div class=\"current-label\">\n          <span class=\"label label-success label-current glyphicon glyphicon-ok\" data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "common.current";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-refresh :restart-required-service view.serviceVersion.isRestartRequired::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n          </span>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n          <button ");
  stack1 = {};
  stack2 = "view.disabledActionAttr.compare";
  stack1['disabled'] = stack2;
  stack2 = ":btn :btn-default view.serviceVersion.isDisplayed:not-allowed-cursor";
  stack1['class'] = stack2;
  stack2 = "view.disabledActionMessages.compare";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.actionTypes.COMPARE";
  stack3 = depth0;
  stack4 = "view.serviceVersion";
  stack5 = depth0;
  stack6 = "doAction";
  stack7 = {};
  stack8 = "view.parentView";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"glyphicon glyphicon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.compare";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n          <button ");
  stack1 = {};
  stack2 = "view.disabledActionAttr.revert";
  stack1['disabled'] = stack2;
  stack2 = ":btn :btn-default view.serviceVersion.isCurrent:not-allowed-cursor view.serviceVersion.isCompatible::hidden";
  stack1['class'] = stack2;
  stack2 = "view.disabledActionMessages.revert";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.actionTypes.REVERT";
  stack3 = depth0;
  stack4 = "view.serviceVersion";
  stack5 = depth0;
  stack6 = "doAction";
  stack7 = {};
  stack8 = "view.parentView";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.info-bar.revert.button";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n        ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = ":flow-element :pull-left view.serviceVersion.first:first";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"arrow-box pull-left\"></div>\n  <div class=\"version-box\">\n    <div ");
  stack1 = {};
  stack2 = ":version-info :box :pull-right view.serviceVersion.isDisplayed:displayed view.serviceVersion.isDisabled:grayedOut";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.actionTypes.SWITCH";
  stack3 = depth0;
  stack4 = "view.serviceVersion";
  stack5 = depth0;
  stack6 = "doAction";
  stack7 = {};
  stack8 = "view.parentView";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"top-label\">\n        <span class=\"label label-info\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.versionText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <span class=\"author pull-right\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.authorFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      </div>\n      <div class=\"content\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.timeSinceCreated";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"stack-label\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.stackVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  stack1 = depth0;
  stack2 = "view.serviceVersion.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n\n    <div class=\"version-popover\">\n      <div class=\"content\">\n        <span class=\"label label-info\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.versionText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <span class=\"pull-right\"><strong>");
  stack1 = depth0;
  stack2 = "view.serviceVersion.configGroupName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></span>\n        <span class=\"stack\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.stackVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <div class=\"date\"><strong data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "view.serviceVersion.author";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.authorFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>&nbsp;");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.info-bar.authoredOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;<strong>");
  stack1 = depth0;
  stack2 = "view.serviceVersion.createdDate";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></div>\n        <div class=\"notes\">");
  stack1 = depth0;
  stack2 = "view.serviceVersion.fullNotes";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"version-operations-buttons\">\n        <button ");
  stack1 = {};
  stack2 = "view.disabledActionAttr.view";
  stack1['disabled'] = stack2;
  stack2 = ":btn :btn-default view.serviceVersion.isDisplayed:not-allowed-cursor";
  stack1['class'] = stack2;
  stack2 = "view.disabledActionMessages.view";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.actionTypes.SWITCH";
  stack3 = depth0;
  stack4 = "view.serviceVersion";
  stack5 = depth0;
  stack6 = "doAction";
  stack7 = {};
  stack8 = "view.parentView";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"glyphicon glyphicon-search\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.view";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </button>\n        ");
  stack1 = depth0;
  stack2 = "SERVICE.COMPARE_CONFIGS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/services_config", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"row one-row-tabs-container\">\n    <div ");
  stack1 = {};
  stack2 = ":left-arrow view.isNavArrowsHidden:not-visible";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <a href=\"#\"");
  stack1 = {};
  stack2 = "view.isLeftArrowDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "scrollTabsLeft";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "> <i class=\"icon-angle-left\"></i>\n      </a>\n    </div>\n    <div class=\"tabs-container\">\n      <ul class=\"nav nav-tabs mbm\">\n        ");
  stack1 = depth0;
  stack2 = "controller.stepConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n    <div ");
  stack1 = {};
  stack2 = ":right-arrow view.isNavArrowsHidden:not-visible";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <a href=\"#\" ");
  stack1 = {};
  stack2 = "view.isRightArrowDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "scrollTabsRight";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"icon-angle-right\"></i>\n      </a>\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li rel='tooltip' ");
  stack1 = {};
  stack2 = "tab.isActive:active";
  stack1['class'] = stack2;
  stack2 = "tab.tooltipMsg";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "tab";
  stack3 = depth0;
  stack4 = "selectService";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "tab.headingClass";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n               data-toggle=\"tab\">\n              ");
  stack1 = depth0;
  stack2 = "tab.serviceName";
  foundHelper = helpers.formatRole;
  stack3 = foundHelper || depth0.formatRole;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatRole", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </a>\n          </li>\n        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.isRecommendedLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <ul class=\"nav nav-tabs\" ");
  stack1 = depth0;
  stack2 = "config-tabs";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "controller.stepConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "service.showConfig";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigTab";
  stack3 = helpers.view;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <a href=\"#");
  stack1 = depth0;
  stack2 = "service.serviceName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" ");
  stack1 = {};
  stack2 = ":active service.selected:new";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n               data-toggle=\"tab\" ");
  stack1 = depth0;
  stack2 = "service";
  stack3 = depth0;
  stack4 = "selectService";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "service.errorCount";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</a>\n          ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("<span\n                class=\"badge badge-important\" ");
  stack1 = depth0;
  stack2 = "error-badge";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.errorCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "issuesCounter";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"notifications-group align-right installer-all-configs-notifications\">\n      <span class=\"alerts-label dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\" aria-expanded=\"false\">\n        <span id=\"issues-bell\" ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-bell controller.hasErrors:important:warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n        <span id=\"issues-counter\" ");
  stack1 = {};
  stack2 = ":badge controller.hasErrors:badge-important:badge-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "issuesCounter";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n      </span>\n      <div id=\"notifications-dropdown\" class=\"dropdown-menu row\">\n        <div class=\"popover-content\">\n          <div class=\"popup-arrow-up\"></div>\n          ");
  stack1 = depth0;
  stack2 = "App.DependentConfigsListView";
  stack3 = {};
  stack4 = "controller.recommendations";
  stack3['recommendationsBinding'] = stack4;
  stack4 = "controller.requiredChanges";
  stack3['requiredChangesBinding'] = stack4;
  stack4 = true;
  stack3['isRecommendationsClickable'] = stack4;
  stack4 = false;
  stack3['showRecommendationsPopovers'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n    </div>\n  ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers.unless;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"alert alert-warning\">\n      <span>");
  stack1 = depth0;
  stack2 = "dependenciesMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <a\n        href=\"#\" ");
  stack1 = depth0;
  stack2 = "showChangedDependentConfigs";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </div>\n  ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"alert alert-warning\">\n    <span>");
  stack1 = depth0;
  stack2 = "dependenciesGroupMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <a\n      href=\"#\" ");
  stack1 = depth0;
  stack2 = "changedDependentGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <p class=\"loading align-center\"></p>\n  ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers.unless;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.transitionInProgress";
  stack3 = helpers.unless;
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "controller.hasStepConfigIssues";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(27, program27, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"alert alert-warning\" ");
  stack1 = depth0;
  stack2 = "warning-alert";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          <span class=\"glyphicon glyphicon-warning-sign\"></span> ");
  stack1 = depth0;
  stack2 = "installer.step7.attentionNeeded";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<br/>\n          ");
  stack1 = depth0;
  stack2 = "issuesFilterText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <a\n            href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleIssuesFilter";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "issuesFilterLinkText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"alert alert-success\">\n          <span class=\"glyphicon glyphicon-check\"></span> ");
  stack1 = depth0;
  stack2 = "installer.step7.noIssues";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<br/>\n          ");
  stack1 = depth0;
  stack2 = "controller.issuesFilterSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleIssuesFilter";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "issuesFilterLinkText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "hasChangedDependencies";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "showSelectGroupsPopup";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "App.ServiceConfigContainerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  stack1 = depth0;
  stack2 = "controller.isRecommendedLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/checkbox_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <label ");
  stack1 = {};
  stack2 = ":widget-config-label view.configLabelClass";
  stack1['class'] = stack2;
  stack2 = "view.configViewId";
  stack1['for'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.issueView";
  stack3 = {};
  stack4 = "view.config";
  stack3['configBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.configLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </label>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"widget-config-label text-success\">");
  stack1 = depth0;
  stack2 = "common.override";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

  data.buffer.push("\n<div ");
  stack1 = {};
  stack2 = ":pull-left view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"checkbox\">\n    ");
  stack1 = depth0;
  stack2 = "view.configView";
  stack3 = {};
  stack4 = "pull-left";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"clearfix\"></div>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/combo_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li>\n            <a href=\"#\" ");
  stack1 = {};
  stack2 = "item.configValue :btn-dropdown-menu-combo-config";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "item.configValue";
  stack3 = depth0;
  stack4 = "setConfigValue";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.widgetValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </li>\n        ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<div ");
  stack1 = {};
  stack2 = ":pull-left view.isOriginalSCP:original-widget view.disabled:disabled view.doNotShowWidget:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"input-group\">\n    <div class=\"dropdown btn-group\">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.content.value";
  stack3['valueBinding'] = stack4;
  stack4 = "disabled";
  stack3['disabled'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <button ");
  stack1 = {};
  stack2 = "view.disabled:disabled :btn :btn-default :dropdown-toggle :btn-icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"dropdown\">\n        <span class=\"caret\"></span>\n      </button>\n      <ul ");
  stack1 = {};
  stack2 = ":pull-left :dropdown-menu";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.content.valuesList";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/config_label", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"clearfix\"></div>\n  <p ");
  stack1 = {};
  stack2 = ":widget-config-label view.configLabelClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n   ");
  stack1 = depth0;
  stack2 = "view.issueView";
  stack3 = {};
  stack4 = "view.config";
  stack3['configBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n   ");
  stack1 = depth0;
  stack2 = "view.configLabel";
  stack3 = {};
  stack4 = "view.issueMessage";
  stack3['data-original-titleBinding'] = stack4;
  foundHelper = helpers.formatWordBreak;
  stack4 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isPropertyUndefined";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.config.showAsTextBox::hide :widget-config-raw";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.PlainConfigTextField";
  stack3 = {};
  stack4 = "view.config.showAsTextBox";
  stack3['isVisibleBinding'] = stack4;
  stack4 = "view.section";
  stack3['sectionBinding'] = stack4;
  stack4 = "view.subSection";
  stack3['subSectionBinding'] = stack4;
  stack4 = "view.config";
  stack3['configBinding'] = stack4;
  stack4 = "view.config";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "view.canNotEdit";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.readOnly";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.inlineControls";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"widget-config-controls\">\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/undo";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/edit";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/remove_override";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/create_override";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/set_recommended";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/final";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":widget-config-controls view.referToSelectedGroup::clear-min-height";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div ");
  stack1 = {};
  stack2 = ":widget-col :left view.config.isNotDefaultValue::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/undo";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"widget-col left\">\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/edit";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/set_recommended";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"widget-col right\">\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/final";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/remove_override";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls/create_override";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n    ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.configGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SwitchToGroupView";
  stack3 = {};
  stack4 = "controller";
  stack3['controllerBinding'] = stack4;
  stack4 = "view.configGroup";
  stack3['groupBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"widget-config-controls version-labels\">\n    ");
  stack1 = depth0;
  stack2 = "controller.selectedConfigGroup.isDefault";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"label label-info\">");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.table.version.prefix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "controller.selectedVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  stack1 = depth0;
  stack2 = "controller.isCurrentSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.config.supportsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n        <span class=\"label label-success glyphicon glyphicon-ok label-current\" data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "common.current";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></span>\n      ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n        <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-lock view.config.isFinal::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n      ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"label label-info\">");
  stack1 = depth0;
  stack2 = "common.default";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        &nbsp;");
  stack1 = depth0;
  stack2 = "services.service.config.configHistory.configGroup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.config.hasCompareDiffs";
  stack3 = helpers.unless;
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"clearfix\"></div>\n    ");
  stack1 = depth0;
  stack2 = "App.ConfigWidgetOverrideView";
  stack3 = {};
  stack4 = "view.config";
  stack3['serviceConfigPropertyBinding'] = stack4;
  stack4 = "controller.selectedConfigGroup.isDefault";
  stack3['isDefaultGroupSelectedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.isPropertyUndefined::hide :left :widget-config-raw";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <i>");
  stack1 = depth0;
  stack2 = "common.property.undefined";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</i>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.supportSwitchToTextBox";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.canEdit";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "controller.selectedConfigGroup.isDefault";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isComparison";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n");
  stack1 = depth0;
  stack2 = "view.isOriginalSCP";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/create_override", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = ":widget-action :widget-action-override";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <a class=\"widget-action widget-action-override\" href=\"#\" data-toggle=\"tooltip\"\n        ");
  stack1 = depth0;
  stack2 = "view.config";
  stack3 = depth0;
  stack4 = "createOverrideProperty";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "common.override";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-plus-sign\"></i>\n      </a>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.overrideAllowed";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/edit", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isWidgetViewAllowed";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <a href=\"#\"\n       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "common.edit";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-pencil :widget-action :widget-action-switch-view view.config.showAsTextBox::active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "toggleWidgetView";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    </a>\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <i class=\"icon-pencil widget-action widget-action-switch-view text-danger\"\n       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "view.issueMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.showPencil";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/final", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n  <a href=\"#\" data-toggle=\"tooltip\"\n    ");
  stack1 = {};
  stack2 = ":widget-action :widget-action-final view.config.isFinal:active view.showFinalConfig:show:hide";
  stack1['class'] = stack2;
  stack2 = "view.config.isNotEditable";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.config";
  stack3 = depth0;
  stack4 = "toggleFinalFlag";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = {};
  stack2 = "services.service.config.final";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    <i class=\"glyphicon glyphicon-lock\"></i>\n  </a>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.config.supportsFinal";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/remove_override", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.disabled";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <a ");
  stack1 = {};
  stack2 = ":widget-action :widget-action-remove";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" data-toggle=\"tooltip\"\n        ");
  stack1 = depth0;
  stack2 = "view.config";
  stack3 = depth0;
  stack4 = "removeOverride";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "common.remove";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-minus-sign\"></i>\n      </a>\n    ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isOriginalSCP";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/set_recommended", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.disabled";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <a href=\"#\"\n      ");
  stack1 = {};
  stack2 = ":icon-repeat :widget-action-set-recommended :widget-action";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "setRecommendedValue";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "services.service.config.setRecommendedValue";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    </a>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.config.recommendedValueExists";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/controls/undo", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<a href=\"#\" ");
  stack1 = {};
  stack2 = ":widget-action :widget-action-undo view.undoAllowed::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n   data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "common.undo";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "restoreValue";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <i class=\"icon-undo\"></i>\n</a>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/directory_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n   <div  ");
  stack1 = {};
  stack2 = ":directory-textarea-wrapper view.doNotShowWidget:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n     ");
  stack1 = depth0;
  stack2 = "view.configView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n   </div>\n   <div class=\"right\">\n     ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n   </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/label_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div>\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div ");
  stack1 = {};
  stack2 = ":text-field-wrapper view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.configView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <br/>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/list_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <li>\n          <a rel=\"tooltip\"\n             href=\"javascript:void(0);\" ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "toggleOption";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "option.description";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.checkBoxWithoutAction";
  stack3 = {};
  stack4 = "option.value";
  stack3['valueBinding'] = stack4;
  stack4 = "option.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "option.isSelected";
  stack3['checkedBinding'] = stack4;
  stack4 = "option.label";
  stack3['labelBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.config.isHiddenByFilter:hide view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div class=\"pull-left btn-group\">\n    <a ");
  stack1 = {};
  stack2 = ":btn :btn-default :dropdown-toggle view.config.isEditable::disabled view.doNotShowWidget:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"dropdown\">");
  stack1 = depth0;
  stack2 = "view.displayVal";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span></a>\n    <ul class=\"dropdown-menu\">\n      ");
  stack1 = depth0;
  stack2 = "view.options";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/overrides/config_widget_override", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n");
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = "overriddenSCP.errorMessage:error: :form-group :overrideField";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty.widget";
  stack3 = {};
  stack4 = "overriddenSCP";
  stack3['configBinding'] = stack4;
  stack4 = "false";
  stack3['isPopoverEnabled'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div class=\"clearfix\"></div>\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.serviceConfigProperty.overrides";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "overriddenSCP";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/plain_config_text_field", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"input-group\">\n  ");
  stack1 = depth0;
  stack2 = "App.ConfigTextField";
  stack3 = {};
  stack4 = "view.value";
  stack3['valueBinding'] = stack4;
  stack4 = "col-md-9 form-control";
  stack3['class'] = stack4;
  stack4 = "view.placeholder";
  stack3['placeholderBinding'] = stack4;
  stack4 = "view.disabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div ");
  stack1 = {};
  stack2 = ":input-group-addon view.unit::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.displayUnit";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/radio_button_config", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "view.elementId";
  stack3['nameBinding'] = stack4;
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "item.label";
  stack3['labelBinding'] = stack4;
  stack4 = "item.value";
  stack3['valueBinding'] = stack4;
  stack4 = "view.config.value";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<div class=\"pull-left\">\n  ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/service_config_password_field", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-6\">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.data-qa";
  stack3['data-qaBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.readOnly";
  stack3['readOnlyBinding'] = stack4;
  stack4 = "view.serviceConfig.value";
  stack3['valueBinding'] = stack4;
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "view.placeholder";
  stack3['placeholderBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div class=\"col-md-6\">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.data-qa";
  stack3['data-qaBinding'] = stack4;
  stack4 = "form-control retyped-password";
  stack3['class'] = stack4;
  stack4 = "view.readOnly";
  stack3['readOnlyBinding'] = stack4;
  stack4 = "view.serviceConfig.retypedPassword";
  stack3['valueBinding'] = stack4;
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "view.placeholderRetype";
  stack3['placeholderBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/slider_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"widget-config slider-widget\">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div ");
  stack1 = {};
  stack2 = "view.isOriginalSCP:original-widget view.doNotShowWidget:hide :ui-slider-wrapper";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div ");
  stack1 = {};
  stack2 = ":pull-left :ui-slider-wrapper-inner";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.config.value";
  stack3['valueBinding'] = stack4;
  stack4 = "input-mini slider-input form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/test_db_connection_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"entry-row db-connection form-group\">\n    <div class=\"control-group\">\n      <div class=\"row\">\n        <div class=\"col-md-10\">\n          <span ");
  stack1 = {};
  stack2 = ":pull-left :btn :btn-primary view.isBtnDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "connectToDatabase";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.btnCaption";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n\n          <div class=\"pull-left connection-result mll\">\n            <a ");
  stack1 = {};
  stack2 = "view.isConnectionSuccess:mute:action";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "showLogsPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.responseCaption";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "view.isConnecting";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          <i ");
  stack1 = {};
  stack2 = ":pull-right view.isConnectionSuccess:glyphicon-ok-sign:glyphicon-warning-sign :glyphicon view.isRequestResolved::hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        </div>\n      </div>\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "mll pull-left";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isNewSelected";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/textfield_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div>\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div ");
  stack1 = {};
  stack2 = ":text-field-wrapper view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.configView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/time_interval_spinner", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerInputView";
  stack3 = {};
  stack4 = "spinnerContent";
  stack3['contentBinding'] = stack4;
  stack4 = "view.disabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<div ");
  stack1 = {};
  stack2 = ":pull-left view.isOriginalSCP:original-widget view.doNotShowWidget:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "spinnerContent";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"clearfix\"></div>\n</div>\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/configs/widgets/toggle_config_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/config_label";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<div ");
  stack1 = {};
  stack2 = ":pull-left view.doNotShowWidget:hide view.isOriginalSCP:original-widget";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.switcherValue";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  stack1 = depth0;
  stack2 = "templates/common/configs/widgets/controls";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/custom_date_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"jobs-custom-dates\">\n  <div ");
  stack1 = {};
  stack2 = ":form-group view.errors.isStartDateError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <label>");
  stack1 = depth0;
  stack2 = "jobs.customDateFilter.startTime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    <div class=\"form-inline\">\n      ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.customDateFormFields.startDate";
  stack3['valueBinding'] = stack4;
  stack4 = "input-sm datepicker no-autofocus form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.hourOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.hoursForStart";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.minuteOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.minutesForStart";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.middayPeriodOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.middayPeriodForStart";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.errorMessages.startDate";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n  <div class=\"form-group\">\n    <label>");
  stack1 = depth0;
  stack2 = "common.duration";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    <div class=\"form-inline\">\n      ");
  stack1 = depth0;
  stack2 = "view.durationSelect";
  stack3 = {};
  stack4 = "view.customDateFormFields.duration";
  stack3['selectionBinding'] = stack4;
  stack4 = "form-control input-sm";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n  <div ");
  stack1 = {};
  stack2 = ":form-group :form-inline view.errors.isEndDateError:has-error view.isCustomEndDate::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <label>");
  stack1 = depth0;
  stack2 = "jobs.customDateFilter.endTime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    <div class=\"form-inline\">\n      ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.customDateFormFields.endDate";
  stack3['valueBinding'] = stack4;
  stack4 = "input-sm datepicker no-autofocus form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.hourOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.hoursForEnd";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.minuteOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.minutesForEnd";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.middayPeriodOptions";
  stack3['contentBinding'] = stack4;
  stack4 = "view.customDateFormFields.middayPeriodForEnd";
  stack3['selectionBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.errorMessages.endDate";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n  <div class=\"timezone\">\n    <strong>");
  stack1 = depth0;
  stack2 = "common.timezone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong>: ");
  stack1 = depth0;
  stack2 = "App.router.userSettingsController.userSettings.timezone.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/editable_list", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <li ");
  stack1 = {};
  stack2 = ":item item.global:global:deletable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <span>\n            <a href>");
  stack1 = depth0;
  stack2 = "item.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  stack1 = depth0;
  stack2 = "item.global";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </span>\n        </li>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <button ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "removeFromItems";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " type=\"button\" class=\"close\">\n                <span>&times;</span>\n              </button>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <li class=\"item add-item-input\">\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.input";
  stack3['valueBinding'] = stack4;
  stack4 = "New";
  stack3['placeholder'] = stack4;
  stack4 = "form-control";
  stack3['classBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.typeahead";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </li>\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div class=\"typeahead-box\">\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "view.typeahead";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "addItem";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"\">");
  stack1 = depth0;
  stack2 = "item.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.items";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program9(depth0,data) {
  
  
  data.buffer.push("\n        ");}

function program11(depth0,data) {
  
  
  data.buffer.push("\n           <li class=\"item add-item\">Add</li>\n        ");}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"actions-panel\">\n      <button class=\"btn btn-default btn-xs\" ");
  stack1 = depth0;
  stack2 = "onSecondary";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <span class=\"glyphicon glyphicon-remove\"></span>\n      </button>\n      <button class=\"btn btn-primary btn-xs\" ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <span class=\"glyphicon glyphicon-ok\"></span>\n      </button>\n    </div>\n  ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"pencil-box\" ");
  stack1 = depth0;
  stack2 = "enableEditMode";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span class=\"glyphicon glyphicon-pencil\"></span>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = ":editable-list-container :well view.editMode";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"items-box\" ");
  stack1 = depth0;
  stack2 = "enableEditMode";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <ul class=\"items-list\">\n      ");
  stack1 = depth0;
  stack2 = "view.items";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.editMode";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n\n  ");
  stack1 = depth0;
  stack2 = "view.editMode";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/error_log_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <h5>");
  stack1 = depth0;
  stack2 = "common.structuredOut";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp;</h5>\n        <pre class=\"stdout\">");
  stack1 = depth0;
  stack2 = "view.openedTask.structuredOut";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"task-detail-log-info\">\n  <div class=\"content-area\">\n    <div class=\"task-detail-log-clipboard-wrap\"></div>\n    <div class=\"task-detail-log-maintext\">\n      <h5>");
  stack1 = depth0;
  stack2 = "common.stderr";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp; <span class=\"muted\">");
  stack1 = depth0;
  stack2 = "view.openedTask.errorLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></h5>\n      <pre class=\"stderr\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stderr";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      <h5>");
  stack1 = depth0;
  stack2 = "common.stdout";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp; <span class=\"muted\"> ");
  stack1 = depth0;
  stack2 = "view.openedTask.outputLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></h5>\n      <pre class=\"stdout\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stdout";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      ");
  stack1 = depth0;
  stack2 = "view.openedTask.structuredOut";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/export_metrics_menu", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<ul class=\"export-graph-list pull-right dropdown-menu\">\n  <li><a ");
  stack1 = depth0;
  stack2 = "view.parentView.exportToCSVArgument";
  stack3 = depth0;
  stack4 = "exportGraphData";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.csv";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n  <li><a ");
  stack1 = depth0;
  stack2 = "exportGraphData";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.json";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n</ul>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/filter_combo_cleanable", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li ");
  stack1 = {};
  stack2 = "column.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "column";
  stack3 = depth0;
  stack4 = "selectFilterColumn";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = {};
  stack2 = "column.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i ");
  stack1 = {};
  stack2 = "column.isDisabled:disabled column.selected:glyphicon-ok-sign:glyphicon-placeholder :glyphicon :text-success";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                ");
  stack1 = depth0;
  stack2 = "column.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n          </li>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <li class=\"divider\"></li>\n        <li>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "clearFilterColumn";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "tableView.filters.clearFilters";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"btn-group\">\n  <div class=\"input-group\">\n\n  ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.filter";
  stack3['valueBinding'] = stack4;
  stack4 = "view.placeHolder";
  stack3['placeholderBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div class=\"input-group-btn\">\n    <div id=\"cleanerContainer\" class=\"pull-left\">\n      <a href=\"#\" id=\"filterCleaner\" ");
  stack1 = depth0;
  stack2 = "clearFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "view.filterNotEmpty::hidden :ui-icon :ui-icon-circle-close";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n    </div>\n    <button class=\"btn btn-default dropdown-toggle btn-icon\" data-toggle=\"dropdown\">\n        <span class=\"caret\"></span>\n    </button>\n    <ul class=\"dropdown-menu\">\n      ");
  stack1 = depth0;
  stack2 = "view.columns";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "column";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.showClearFilter";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/filter_combobox", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <li>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "column";
  stack3 = depth0;
  stack4 = "selectFilterColumn";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <i ");
  stack1 = {};
  stack2 = "column.selected:glyphicon-ok-sign:glyphicon-placeholder :glyphicon :text-success";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                  ");
  stack1 = depth0;
  stack2 = "column.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n            </li>\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"btn-group\">\n  <div class=\"input-group\">\n    ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.filter";
  stack3['valueBinding'] = stack4;
  stack4 = "view.placeHolder";
  stack3['placeholderBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"input-group-btn\">\n      <button class=\"btn btn-default dropdown-toggle btn-icon\" data-toggle=\"dropdown\">\n          <span class=\"caret\"></span>\n      </button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "view.columns";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "column";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.FormFieldTemplate";
  stack3 = {};
  stack4 = "field";
  stack3['fieldBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.fields";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "field";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/check_db_connection", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "mll display-inline-block";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"entry-row db-connection\">\n  <div class=\"col-md-3 form-group control-label\"></div>\n  <div class=\"col-md-9 form-group\">\n    <div class=\"row\">\n      <div class=\"col-md-4 btn-wrapper\">\n        <button ");
  stack1 = {};
  stack2 = ":pull-left :btn :btn-primary view.isBtnDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "connectToDatabase";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.btnCaption";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</button>\n      </div>\n      <div class=\"connection-result col-md-6\">\n        <a ");
  stack1 = {};
  stack2 = "view.isConnectionSuccess:mute:action :btn :btn-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "showLogsPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.responseCaption";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        <i ");
  stack1 = {};
  stack2 = "view.isConnectionSuccess:glyphicon-ok-sign:glyphicon-warning-sign :glyphicon view.isRequestResolved::hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        ");
  stack1 = depth0;
  stack2 = "view.isConnecting";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/checkbox", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div>\n  <div class=\"checkbox\">\n    <label for=\"input");
  stack1 = depth0;
  stack2 = "view.field.name";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\">\n      ");
  stack1 = depth0;
  stack2 = "view.field.viewClass";
  stack3 = {};
  stack4 = "view.field.value";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.field.displayName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </label>\n  </div>\n  <span class=\"help-block\">");
  stack1 = depth0;
  stack2 = "view.field.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/dropdown", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "selectOption";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "App.DropdownOptionView";
  stack3 = {};
  stack4 = "option";
  stack3['optionBinding'] = stack4;
  stack4 = "view.optionLabelPath";
  stack3['optionLabelPathBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"dropdown dropdown-select\" ");
  stack1 = depth0;
  stack2 = "{view.qaAttr}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <button ");
  stack1 = {};
  stack2 = ":btn :btn-default :dropdown-toggle view.disabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " type=\"button\" data-toggle=\"dropdown\">\n    <span class=\"caret pull-caret-right\"></span>\n    ");
  stack1 = depth0;
  stack2 = "App.DropdownOptionView";
  stack3 = {};
  stack4 = "view.selection";
  stack3['optionBinding'] = stack4;
  stack4 = "view.optionLabelPath";
  stack3['optionLabelPathBinding'] = stack4;
  stack4 = "selected-item";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </button>\n  <ul class=\"dropdown-menu\">\n    ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/field", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.field.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}

  data.buffer.push("\n\n<label class=\"control-label\"\n       for=\"input");
  stack1 = depth0;
  stack2 = "view.field.name";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\">");
  stack1 = depth0;
  stack2 = "view.field.displayName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n<div>\n  ");
  stack1 = depth0;
  stack2 = "view.field.viewClass";
  stack3 = {};
  stack4 = "view.field.value";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "view.field.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/manage_credentilas_form", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p class=\"mtm text-danger\">\n          ");
  stack1 = depth0;
  stack2 = "view.principalError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </p>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p class=\"mtm text-danger\">\n          ");
  stack1 = depth0;
  stack2 = "view.passwordError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </p>\n      ");
  return buffer;}

  data.buffer.push("\n\n<form autocomplete=\"off\" class=\"form-horizontal\">\n  <p class=\"alert alert-info\">\n    ");
  stack1 = depth0;
  stack2 = "view.formHeader";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  <div class=\"control-group\">\n    <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "popup.invalid.KDC.admin.principal";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    <div>\n      ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.principal";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.principalError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n  <div class=\"control-group\">\n    <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "popup.invalid.KDC.admin.password";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    <div>\n      ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "view.password";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.passwordError";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/form/spinner_input", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <span class=\"spinner-input-label\">");
  stack1 = depth0;
  stack2 = "view.content.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n");
  return buffer;}

  data.buffer.push("\n\n<div class=\"input-group\">\n  <span class=\"input-group-btn\">\n    <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "view.computedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "decrementValue";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span class=\"icon-caret-left\"></span>\n    </button>\n  </span>\n  ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.content.value";
  stack3['valueBinding'] = stack4;
  stack4 = "view.computedDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <span class=\"input-group-btn\">\n    <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "view.computedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "incrementValue";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span class=\"icon-caret-right\"></span>\n    </button>\n  </span>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.content.label";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/grid/filter", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n<li>\n  <div class=\"checkbox\">\n    ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "filter.checked";
  stack3['checkedBinding'] = stack4;
  stack4 = "filter.label";
  stack3['labelBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</li>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.filters";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "filter";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/grid/header", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.filter";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <a ");
  stack1 = depth0;
  stack2 = "applyFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "apply";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<i ");
  stack1 = depth0;
  stack2 = "toggleFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"glyphicon glyphicon-filter\"></i>\n");
  stack1 = depth0;
  stack2 = "view.showFilter";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/grid/pager", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <li ");
  stack1 = {};
  stack2 = "page.activeClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ><a ");
  stack1 = depth0;
  stack2 = "page";
  stack3 = depth0;
  stack4 = "activatePage";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "page.number";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n  ");
  return buffer;}

  data.buffer.push("\n\n<ul>\n    <li ");
  stack1 = {};
  stack2 = "view.prevPageDisabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a ");
  stack1 = depth0;
  stack2 = "activatePrevPage";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "common.prev";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n  ");
  stack1 = depth0;
  stack2 = "view.pages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "page";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <li ");
  stack1 = {};
  stack2 = "view.nextPageDisabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a ");
  stack1 = depth0;
  stack2 = "activateNextPage";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n</ul>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/host_progress_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  ");
  data.buffer.push("\n\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isServiceListHidden:hidden :task-list-main-wrap :table-body-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"top-wrap\">\n        <div class=\"row\">\n          <h2 class=\"table-title col-sm-6\">");
  stack1 = depth0;
  stack2 = "view.parentView.titleClass";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h2>\n          <div class=\"table-controls col-sm-6\">\n            <div class=\"btn-group pull-right status-dropdown\">\n              <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\n                      aria-expanded=\"false\">\n                ");
  stack1 = depth0;
  stack2 = "view.serviceCategory.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n        </div>\n      </div>\n      <div id=\"service-info\">\n      <table class=\"table table-hover\">\n        <thead>\n        <tr>\n          <th class=\"col-sm-5\">");
  stack1 = depth0;
  stack2 = "common.operations";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-3\">");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-1\">");
  stack1 = depth0;
  stack2 = "common.user";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-2\">");
  stack1 = depth0;
  stack2 = "common.startTime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-2\" colspan=\"2\">");
  stack1 = depth0;
  stack2 = "common.duration";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n        </thead>\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.isServiceEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </table>\n        ");
  stack1 = depth0;
  stack2 = "view.isShowMore";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n\n    ");
  data.buffer.push("\n\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isHostListHidden:hidden :task-list-main-wrap :table-body-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"top-wrap\">\n        <div class=\"row\">\n          <h2 class=\"table-title col-sm-2\">");
  stack1 = depth0;
  stack2 = "common.hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n          <div class=\"table-controls col-sm-10\">\n            <div class=\"btn-group pull-right status-dropdown\">\n              <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\n                      aria-expanded=\"false\">\n                ");
  stack1 = depth0;
  stack2 = "view.hostCategory.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isRequestSchedule";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isLevelLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(29, program29, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n\n\n    ");
  data.buffer.push("\n\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isTaskListHidden:hidden :task-list-main-wrap :table-body-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"top-wrap\">\n        <div class=\"row\">\n          <h2 class=\"table-title col-sm-2\">");
  stack1 = depth0;
  stack2 = "common.tasks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n          <div class=\"table-controls col-sm-10\">\n            <div class=\"btn-group pull-right status-dropdown\">\n              <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\n                      aria-expanded=\"false\">\n                ");
  stack1 = depth0;
  stack2 = "view.taskCategory.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n        </div>\n      </div>\n      <div id=\"host-log\">\n        ");
  stack1 = depth0;
  stack2 = "view.hostInfoLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(41, program41, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n\n    <!-- TASK DETAILS --->\n\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isLogWrapHidden:hidden :task-detail-info view.hostComponentLogsExists:task-detail-info-tabbed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"task-top-wrap top-wrap\">\n        <div ");
  stack1 = {};
  stack2 = "view.hostComponentLogsExists:task-detail-log-nav-actions :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <h2 class=\"table-title col-sm-5\">");
  stack1 = depth0;
  stack2 = "common.taskLog";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n          <div class=\"table-controls col-sm-7 pull-right\">\n            ");
  stack1 = depth0;
  stack2 = "App.supports.logSearch";
  stack3 = helpers['if'];
  tmp1 = self.program(43, program43, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <button type=\"button\" class=\"btn btn-link pull-right\"\n              ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "openTaskLogInDialog";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon icon-external-link\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n            <button type=\"button\" class=\"btn btn-link pull-right copy-clipboard\"\n              ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "taskInfo";
  stack3 = depth0;
  stack4 = "textTrigger";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"glyphicon glyphicon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n          </div>\n        </div>\n        <ul ");
  stack1 = {};
  stack2 = "view.hostComponentLogsExists::hide :nav :nav-tabs :task-detail-nav";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <li ");
  stack1 = {};
  stack2 = "view.isLevelLoaded:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" data-target=\"#task-log-tab\"\n               data-toggle=\"tab\" ");
  stack1 = depth0;
  stack2 = "setActiveTaskLogTab";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " stdout/stderr</a>\n          </li>\n          ");
  stack1 = depth0;
  stack2 = "view.hostComponentLogs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "hostLog";
  stack7 = helpers.each;
  tmp1 = self.program(47, program47, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLevelLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(49, program49, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(58, program58, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectServiceCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td colspan=\"5\">\n            <div class=\"log-list-wrap\">");
  stack1 = depth0;
  stack2 = "hostPopup.noServicesToShow";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          </td>\n        </tr>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "servicesInfo";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <tr ");
  stack1 = depth0;
  stack2 = "servicesInfo";
  stack3 = depth0;
  stack4 = "onOpClick";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "servicesInfo.isVisible::hidden :pointer";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <td class=\"col-sm-5\">\n              ");
  stack1 = depth0;
  stack2 = "statusIcon";
  stack3 = {};
  stack4 = "servicesInfo";
  stack3['servicesInfoBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              <a href=\"#\">\n                ");
  stack1 = depth0;
  stack2 = "servicesInfo.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </td>\n            <td class=\"col-sm-3\">\n              <div class=\"progress-wrapper col-sm-8\">\n                <div class=\"progress\">\n                  <div ");
  stack1 = {};
  stack2 = "servicesInfo.isInProgress:active servicesInfo.isInProgress:progress-bar-striped servicesInfo.barColor :progress-bar";
  stack1['class'] = stack2;
  stack2 = "servicesInfo.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                </div>\n              </div>\n              <div class=\"progress-percentage col-sm-2\">");
  stack1 = depth0;
  stack2 = "servicesInfo.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</div>\n            </td>\n            ");
  data.buffer.push("\n            <td class=\"col-sm-1\">");
  stack1 = depth0;
  stack2 = "servicesInfo.userName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"col-sm-2\">");
  stack1 = depth0;
  stack2 = "servicesInfo.startTime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"col-sm-2\">");
  stack1 = depth0;
  stack2 = "servicesInfo.duration";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"col-sm-1 text-right\">\n              ");
  stack1 = depth0;
  stack2 = "servicesInfo.isAbortable";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <a class=\"action\" href=\"#\">\n                <span class=\"icon icon-caret-right\"></span>\n              </a>\n            </td>\n          </tr>\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "abortIcon";
  stack3 = {};
  stack4 = "servicesInfo";
  stack3['servicesInfoBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tfoot>\n            <tr>\n              <td colspan=\"5\">\n                ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </td>\n            </tr>\n          </tfoot>\n        ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"show-more\" ");
  stack1 = depth0;
  stack2 = "requestMoreOperations";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"#\">");
  stack1 = depth0;
  stack2 = "hostPopup.serviceInfo.showMore";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </div>\n        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectHostCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.sourceRequestScheduleRunning";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.sourceRequestScheduleAborted";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div class=\"alert alert-info request-schedule-abort\">\n            ");
  stack1 = depth0;
  stack2 = "hostPopup.bgop.sourceRequestSchedule.running";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <button type=\"button\" class=\"btn btn-warning pull-right\"\n              ");
  stack1 = depth0;
  stack2 = "view.sourceRequestScheduleId";
  stack3 = depth0;
  stack4 = "doAbortRequestSchedule";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "view.requestScheduleAbortLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n          </div>\n        ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"alert alert-info request-schedule-abort\">\n            ");
  stack1 = depth0;
  stack2 = "hostPopup.bgop.sourceRequestSchedule.aborted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div id=\"host-info\" ");
  stack1 = {};
  stack2 = "view.isRequestSchedule:scheduled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "view.isHostEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(24, program24, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"log-list-wrap\">");
  stack1 = depth0;
  stack2 = "hostPopup.noHostsToShow";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <table class=\"table table-hover\">\n              <tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "hostInfo";
  stack7 = helpers.each;
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </table>\n          ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <tr ");
  stack1 = depth0;
  stack2 = "hostInfo";
  stack3 = depth0;
  stack4 = "onHostClick";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "hostInfo.isVisible::hidden :pointer";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <td class=\"col-sm-6 text-nowrap\">\n                    ");
  stack1 = depth0;
  stack2 = "statusIcon";
  stack3 = {};
  stack4 = "hostInfo";
  stack3['servicesInfoBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    <a href=\"#\">\n                      ");
  stack1 = depth0;
  stack2 = "hostInfo.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </td>\n                  <td class=\"col-sm-5\">\n                    <div class=\"progress-wrapper col-md-6\">\n                      <div class=\"progress\">\n                        <div ");
  stack1 = {};
  stack2 = "hostInfo.isInProgress:progress-bar-striped :active hostInfo.barColor :progress-bar";
  stack1['class'] = stack2;
  stack2 = "hostInfo.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      </div>\n                    </div>\n                    <div class=\"progress-percentage col-sm-2\">");
  stack1 = depth0;
  stack2 = "hostInfo.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</div>\n                  </td>\n                  <td class=\"col-sm-1 text-right\">\n                    <a class=\"action\" href=\"#\">\n                      <span class=\"icon icon-caret-right\"></span>\n                    </a>\n                  </td>\n                </tr>\n              ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <tfoot>\n                <tr>\n                  <td colspan=\"3\">\n                    ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                </tr>\n                </tfoot>\n              ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectTaskCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                ");
  return buffer;}

function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.isTasksEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(36, program36, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"log-list-wrap\">");
  stack1 = depth0;
  stack2 = "hostPopup.noTasksToShow";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          ");
  return buffer;}

function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <table class=\"table table-hover\">\n              <tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.tasks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "taskInfo";
  stack7 = helpers.each;
  tmp1 = self.program(37, program37, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </table>\n          ");
  return buffer;}
function program37(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <tr ");
  stack1 = depth0;
  stack2 = "taskInfo";
  stack3 = depth0;
  stack4 = "onTaskClick";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "taskInfo.isVisible::hidden :pointer";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <td class=\"col-sm-3\">\n                    ");
  stack1 = depth0;
  stack2 = "statusIcon";
  stack3 = {};
  stack4 = "taskInfo";
  stack3['servicesInfoBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    <a href=\"#\">\n                      ");
  stack1 = depth0;
  stack2 = "taskInfo.commandDetail";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </td>\n                  <td class=\"col-sm-9 text-right\">\n                    <a class=\"action\" href=\"#\">\n                      <span class=\"icon icon-caret-right\"></span>\n                    </a>\n                  </td>\n                </tr>\n              ");
  return buffer;}

function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <tfoot>\n                <tr>\n                  <td colspan=\"2\">\n                    ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                </tr>\n                </tfoot>\n              ");
  return buffer;}

function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program43(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isLogSearchInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "SERVICE.VIEW_OPERATIONAL_LOGS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(45, program45, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program45(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <button type=\"button\" class=\"btn btn-link pull-right\" ");
  stack1 = depth0;
  stack2 = "navigateToHostLogs";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = {};
  stack2 = "view.isLogsLinkVisible::hidden";
  stack1['class'] = stack2;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <i class=\"glyphicon glyphicon-file\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "common.logs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </button>\n                ");
  return buffer;}

function program47(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "hostLog";
  stack3 = depth0;
  stack4 = "setActiveLogTab";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "hostLog.tabClassNameSelector";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                 data-toggle=\"tab\">");
  stack1 = depth0;
  stack2 = "hostLog.displayedFileName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n          ");
  return buffer;}

function program49(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div class=\"task-detail-log-info\">\n          <div class=\"content-area\">\n            <div class=\"tab-content\">\n              <div class=\"task-detail-log-clipboard-wrap\">\n                ");
  stack1 = depth0;
  stack2 = "view.isClipBoardActive";
  stack3 = helpers['if'];
  tmp1 = self.program(50, program50, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n              <div id=\"task-log-tab\" class=\"tab-pane active\">\n                <div ");
  stack1 = {};
  stack2 = ":task-detail-log-maintext view.isClipBoardActive:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.openedTask.isRebalanceHDFSTask";
  stack3 = helpers['if'];
  tmp1 = self.program(52, program52, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  <p>");
  stack1 = depth0;
  stack2 = "common.stderr";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp; <span class=\"muted\">");
  stack1 = depth0;
  stack2 = "view.openedTask.errorLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></p>\n                  <pre class=\"stderr\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stderr";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n                  <p>");
  stack1 = depth0;
  stack2 = "common.stdout";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp; <span class=\"muted\"> ");
  stack1 = depth0;
  stack2 = "view.openedTask.outputLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></p>\n                  <pre class=\"stdout\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stdout";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n                </div>\n              </div>\n              ");
  stack1 = depth0;
  stack2 = "view.hostComponentLogs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "hostLog";
  stack7 = helpers.each;
  tmp1 = self.program(55, program55, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        </div>\n      ");
  return buffer;}
function program50(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "App.NotScrollableTextArea";
  stack3 = {};
  stack4 = "task-detail-log-clipboard";
  stack3['class'] = stack4;
  stack4 = "view.textAreaValue";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program52(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <h5>");
  stack1 = depth0;
  stack2 = "services.hdfs.rebalance.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h5>\n\n                    <div class=\"progresspopup-rebalancehdfs\">\n                      <div class=\"progress-bar\">\n                        <div ");
  stack1 = {};
  stack2 = ":progress view.openedTask.isInProgress:progress-bar-striped view.openedTask.barColor :active";
  stack1['class'] = stack2;
  stack2 = "view.openedTask.completionProgressStyle";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      </div>\n                    </div>\n                    <div class=\"clearfix\">\n                      <div class=\"pull-left\">\n                        ");
  stack1 = depth0;
  stack2 = "view.openedTask.dataMoved";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " moved /\n                        ");
  stack1 = depth0;
  stack2 = "view.openedTask.dataLeft";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " left /\n                        ");
  stack1 = depth0;
  stack2 = "view.openedTask.dataBeingMoved";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " being processed\n                      </div>\n                      ");
  stack1 = depth0;
  stack2 = "view.openedTask.isNotComplete";
  stack3 = helpers['if'];
  tmp1 = self.program(53, program53, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                    <hr>\n                  ");
  return buffer;}
function program53(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <button class=\"btn btn-danger pull-right\" ");
  stack1 = depth0;
  stack2 = "stopRebalanceHDFS";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                      ");
  return buffer;}

function program55(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <div ");
  stack1 = {};
  stack2 = ":tab-pane :log-component-tab hostLog.tabClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <p ");
  stack1 = {};
  stack2 = "view.isClipBoardActive:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <strong>");
  stack1 = depth0;
  stack2 = "common.file";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp; </strong>\n                    <strong class=\"muted\">");
  stack1 = depth0;
  stack2 = "hostLog.fileName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>\n                    ");
  stack1 = depth0;
  stack2 = "App.LogSearchUILinkView";
  stack3 = {};
  stack4 = "hostLog.linkTail";
  stack3['linkQueryParamsBinding'] = stack4;
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(56, program56, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </p>\n                  <div ");
  stack1 = {};
  stack2 = "view.isClipBoardActive:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "view.logTailView";
  stack3 = {};
  stack4 = "hostLog";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>\n                </div>\n              ");
  return buffer;}
function program56(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a ");
  stack1 = {};
  stack2 = "view.formatedLink";
  stack1['href'] = stack2;
  stack2 = ":pull-right view.isLodaded::disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              target=\"_blank\" rel=\"noopener noreferrer\">\n                        <i class=\"icon-external-link\"></i>\n                        ");
  stack1 = depth0;
  stack2 = "popup.logTail.openInLogSearch";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program58(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program60(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"host-component-popup-wrap\">\n\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(60, program60, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/host_progress_popup_footer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"checkbox pull-left checkbox-row\">\n    ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "footer-checkbox";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.isNotShowBgChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "app.settings.notShowBgOperations";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <button ");
  stack1 = {};
  stack2 = "view.parentView.disablePrimary";
  stack1['disabled'] = stack2;
  stack2 = ":btn view.parentView.primaryClass :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </button>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "AMBARI.MANAGE_SETTINGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/items_list_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <p>");
  stack1 = depth0;
  stack2 = "item";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.insertedItems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/log_file_search", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"display-inline-block level-checkbox\">\n          <label>\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "level.checked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "level.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " (");
  stack1 = depth0;
  stack2 = "level.counter";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")\n          </label>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = ":keyword-item :btn keyword.isIncluded:btn-success:btn-danger";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "keyword.isIncluded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <span> '");
  stack1 = depth0;
  stack2 = "keyword.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "'</span>\n              <a ");
  stack1 = depth0;
  stack2 = "keyword";
  stack3 = depth0;
  stack4 = "removeKeyword";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"close\" href=\"#\">x</a>\n            </div>\n          ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.include.short";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n              ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.exclude.short";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n              ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"row log-data-item\">\n      <div class=\"col-md-1 log-data-date\">\n        ");
  stack1 = depth0;
  stack2 = "logData.date";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <br />\n        <span class=\"log-data-level\">\n          ");
  stack1 = depth0;
  stack2 = "logData.level";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n      </div>\n      <div class=\"col-md-11 log-data-message\">");
  stack1 = depth0;
  stack2 = "logData.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"clearfix\"></div>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"log-file-search-toolbar\">\n  <div class=\"filter-block pull-left\">\n    <div class=\"toolbar-row\">\n      <div class=\"display-inline-block\">\n        <div class=\"display-inline-block\">");
  stack1 = depth0;
  stack2 = "common.keywords";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n        ");
  stack1 = depth0;
  stack2 = "view.keywordsFilterView";
  stack3 = {};
  stack4 = "display-inline-block";
  stack3['classNames'] = stack4;
  stack4 = "view.keywordsFilterValue";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <button ");
  stack1 = depth0;
  stack2 = "submitKeywordsValue";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary display-inline-block\">Search</button>\n      </div>\n      <div class=\"display-inline-block date-filter\">\n        <div class=\"display-inline-block date-filter-item\">\n          ");
  stack1 = depth0;
  stack2 = "from";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n          ");
  stack1 = depth0;
  stack2 = "App.DatepickerFieldView";
  stack3 = {};
  stack4 = "view.dateFromValue";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"display-inline-block date-filter-item\">\n          ");
  stack1 = depth0;
  stack2 = "to";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n          ");
  stack1 = depth0;
  stack2 = "App.DatepickerFieldView";
  stack3 = {};
  stack4 = "view.dateToValue";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"pull-right\">\n        <a ");
  stack1 = depth0;
  stack2 = "resetKeywordsDateFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" class=\"reset-link\">");
  stack1 = depth0;
  stack2 = "common.reset";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </div>\n    </div>\n    <div class=\"clearfix\"></div>\n    <div class=\"toolbar-row levels-filter-block\">\n      <div class=\"display-inline-block\">");
  stack1 = depth0;
  stack2 = "common.levels";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n      ");
  stack1 = depth0;
  stack2 = "view.levelsContext";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "level";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <div class=\"pull-right\">\n        <a ");
  stack1 = depth0;
  stack2 = "resetLevelsFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" class=\"reset-link\">");
  stack1 = depth0;
  stack2 = "common.reset";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </div>\n    </div>\n    <div class=\"toolbar-row include-exclude-filter-block\">\n      <div class=\"display-inline-block\">\n        ");
  stack1 = depth0;
  stack2 = "common.filters";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n        <div class=\"keywords-list\">\n          ");
  stack1 = depth0;
  stack2 = "view.selectedKeywords";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "keyword";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n      <div class=\"pull-right\">\n        <a ");
  stack1 = depth0;
  stack2 = "resetKeywordsFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" class=\"reset-link\">");
  stack1 = depth0;
  stack2 = "common.reset";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </div>\n    </div>\n  </div>\n  <div class=\"context-menu pull-left\">\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "moveTableTop";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"move-to-top glyphicon glyphicon-caret-up\"></a>\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "moveTableBottom";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"move-to-bottom glyphicon glyphicon-caret-down\"></a>\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "navigateToLogUI";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"icon-external-link\"></a>\n  </div>\n  <div class=\"clearfix\"></div>\n</div>\n<div class=\"log-file-search-content container\">\n  ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "logData";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n<div id=\"log-file-search-item-context-menu\">\n  <ul class=\"dropdown-menu\" role=\"menu\">\n    <li>\n      <a ");
  stack1 = depth0;
  stack2 = "includeSelected";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "common.include";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </li>\n    <li>\n      <a ");
  stack1 = depth0;
  stack2 = "excludeSelected";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "common.exclude";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/log_tail", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.oldLogsIsFetching";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.logRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"log-tail-spinner-container text-center\">\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div>");
  stack1 = depth0;
  stack2 = "row.logtimeFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "row.level";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "row.logMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"log-tail-spinner-container text-center\">");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"log-tail-wrapper\">\n  <div class=\"log-tail-content pre-styled\">\n    ");
  stack1 = depth0;
  stack2 = "view.isDataReady";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/metric", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.itemView";
  stack3 = {};
  stack4 = "metric";
  stack3['metricBinding'] = stack4;
  stack4 = "view";
  stack3['widgetBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"panel panel-default\" id=\"metricAccordion\">\n      ");
  stack1 = depth0;
  stack2 = "view.moreMetrics";
  stack3 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"panel-group\">\n        <div class=\"panel-heading\">\n          <a class=\"panel-toggle\" data-toggle=\"collapse\" data-parent=\"#metricAccordion\"\n             href=\"#");
  stack1 = depth0;
  stack2 = "code";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "Collapse\">\n            <i class=\"glyphicon glyphicon-play\"></i>");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </div>\n        <div id=\"");
  stack1 = depth0;
  stack2 = "code";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "Collapse\" class=\"panel-body collapse\">\n          ");
  stack1 = depth0;
  stack2 = "items.length";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <ul class=\"items\">\n            ");
  stack1 = depth0;
  stack2 = "items";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "metric";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.moreItemView";
  stack3 = {};
  stack4 = "metric";
  stack3['metricBinding'] = stack4;
  stack4 = "view";
  stack3['widgetBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "metric.notFound";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"span title\">\n  ");
  stack1 = depth0;
  stack2 = "common.metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <i class=\"glyphicon glyphicon-question-sign\"></i>\n</div>\n<ul class=\"display-inline-block nav nav-pills\">\n  ");
  stack1 = depth0;
  stack2 = "view.metrics";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "metric";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  <li class=\"dropdown\">\n    <a ");
  stack1 = depth0;
  stack2 = "toggleMore";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n      ");
  stack1 = depth0;
  stack2 = "metric.more";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      <b class=\"caret\"></b>\n    </a>\n\n    ");
  stack1 = depth0;
  stack2 = "view.showMore";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </li>\n</ul>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <a ");
  stack1 = depth0;
  stack2 = "close-modal";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"close\" ");
  stack1 = depth0;
  stack2 = "onClose";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&times;</a>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.headerClass";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.header";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.bodyClass";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.encodeBody";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.body";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.body";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.footerClass";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(17, program17, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.footerClass";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"modal-footer\">\n            ");
  stack1 = depth0;
  stack2 = "view.third";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  stack1 = depth0;
  stack2 = "view.secondary";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  stack1 = depth0;
  stack2 = "view.primary";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <button ");
  stack1 = depth0;
  stack2 = "modal-tertiary";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = {};
  stack2 = "view.disableThird";
  stack1['disabled'] = stack2;
  stack2 = ":btn view.thirdClass";
  stack1['class'] = stack2;
  stack2 = "view.thirdId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "onThird";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.third";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </button>");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("<button ");
  stack1 = depth0;
  stack2 = "modal-secondary";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = {};
  stack2 = "view.disableSecondary";
  stack1['disabled'] = stack2;
  stack2 = ":btn view.secondaryClass";
  stack1['class'] = stack2;
  stack2 = "view.secondaryId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "onSecondary";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.secondary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </button>");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("<button ");
  stack1 = depth0;
  stack2 = "modal-primary";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = {};
  stack2 = "view.disablePrimary";
  stack1['disabled'] = stack2;
  stack2 = ":btn view.primaryClass";
  stack1['class'] = stack2;
  stack2 = "view.primaryId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.primary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n            ");
  return buffer;}

  data.buffer.push("\n\n\n<div class=\"modal-backdrop in\"></div>\n<div ");
  stack1 = {};
  stack2 = ":modal view.showFooter:with-footer:no-footer";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " id=\"modal\" tabindex=\"-1\" role=\"dialog\" aria-labelledby=\"modal-label\" aria-hidden=\"true\" ");
  stack1 = depth0;
  stack2 = "modal";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <div ");
  stack1 = {};
  stack2 = "view.modalDialogClassesStr :modal-dialog";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"modal-content\">\n      ");
  data.buffer.push("\n      <div class=\"modal-header\" ");
  stack1 = depth0;
  stack2 = "modal-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.showCloseButton";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <h4 ");
  stack1 = depth0;
  stack2 = "modal-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " id=\"modal-label\">\n          ");
  stack1 = depth0;
  stack2 = "view.headerClass";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </h4>\n      </div>\n\n      ");
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":modal-body view.isHideBodyScroll:hidden-scroll";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "modal-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.bodyClass";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.showFooter";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/alerts_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <table class=\"table table-hover\">\n      <thead>\n        <tr>\n          <th class=\"col-sm-4\">");
  stack1 = depth0;
  stack2 = "alerts.definition.details.serviceHost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-4\">");
  stack1 = depth0;
  stack2 = "alerts.fastAccess.popup.body.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th class=\"col-sm-4\">");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n      </thead>\n    </table>\n    <div id=\"alert-info\">\n      ");
  stack1 = depth0;
  stack2 = "view.isAlertEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    <div ");
  stack1 = {};
  stack2 = "view.filteringComplete:hidden :table-overlay";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"alert-list-wrap\">");
  stack1 = depth0;
  stack2 = "alerts.fastAccess.popup.body.noalerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <table class=\"table table-hover\">\n          <tbody>\n            ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "instance";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n        </table>\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <tr>\n                <td class=\"col-sm-4\">\n                  ");
  stack1 = depth0;
  stack2 = "App.AlertInstanceServiceHostView";
  stack3 = {};
  stack4 = "instance";
  stack3['instanceBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n                <td class=\"col-sm-4\">\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "instance";
  stack3 = depth0;
  stack4 = "gotoAlertDetails";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "instance.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  <div class=\"instance-text\">\n                    ");
  stack1 = depth0;
  stack2 = "instance.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>task-detail-log-info\n                </td>\n                <td class=\"col-sm-4\" ");
  stack1 = {};
  stack2 = "instance.lastTriggered";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "App.AlertInstanceStateView";
  stack3 = {};
  stack4 = "instance";
  stack3['instanceBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n              </tr>\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <table class=\"table table-hover\">\n        <tfoot>\n          <tr>\n            <td colspan=\"3\">\n              ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </td>\n          </tr>\n        </tfoot>\n      </table>\n    ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"alerts-popup-wrap table-body-wrap\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/cluster_check_dialog", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <i class=\"glyphicon glyphicon-ok\"></i>&nbsp;<span>");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.preCheck.allPassed.msg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.failTitle";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.failAlert";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"limited-height-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.fails";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <h4>");
  stack1 = depth0;
  stack2 = "view.failTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-warning\">\n        ");
  stack1 = depth0;
  stack2 = "view.failAlert";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <i class=\"glyphicon glyphicon-remove\"></i>&nbsp;<span>");
  stack1 = depth0;
  stack2 = "item.check";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <pre>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.reason";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.reason";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br/>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.failedOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.failed_on";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.failTitle";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.failAlert";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"limited-height-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.bypass";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <h4>");
  stack1 = depth0;
  stack2 = "view.failTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-warning\">\n        ");
  stack1 = depth0;
  stack2 = "view.failAlert";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <i class=\"glyphicon glyphicon-remove\"></i>&nbsp;<span>");
  stack1 = depth0;
  stack2 = "item.check";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <pre>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.reason";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.reason";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br/>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.failedOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.failed_on";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.warningTitle";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.warningAlert";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"limited-height-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.warnings";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <h4>");
  stack1 = depth0;
  stack2 = "view.warningTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-warning\">\n        ");
  stack1 = depth0;
  stack2 = "view.warningAlert";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <i class=\"glyphicon glyphicon-warning-sign\"></i>&nbsp;<span>");
  stack1 = depth0;
  stack2 = "item.check";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <pre>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.reason";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.reason";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br/>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.failedOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "item.failed_on";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.configsMergeTable";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.configsRecommendTable";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n<div id=\"pre-upgrade-check\">\n  ");
  stack1 = depth0;
  stack2 = "view.isAllPassed";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.fails.length";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  <!-- Errors that can by bypassed. -->\n  ");
  stack1 = depth0;
  stack2 = "view.bypass.length";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  ");
  stack1 = depth0;
  stack2 = "view.warnings.length";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.hasConfigsMergeConflicts";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.hasConfigsRecommendations";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/config_recommendation_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <p>\n    ");
  stack1 = depth0;
  stack2 = "installer.step7.popup.validation.criticalIssues.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "view.configErrors.criticalIssues.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </p>\n  <div ");
  stack1 = {};
  stack2 = "controller.isInstallWizard::limited-height-2 :config-validation-warnings";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <table class=\"table table-hover\">\n      <thead>\n      <tr>\n        <th class=\"issue-type-cell\">");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"service-name-cell\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-name-cell\">");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-value-cell\">");
  stack1 = depth0;
  stack2 = "common.value";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-description-cell\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      </tr>\n      </thead>\n      <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.configErrors.criticalIssues";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "error";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <tr ");
  stack1 = {};
  stack2 = "error.cssClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <td class=\"issue-type-cell\">\n            ");
  stack1 = depth0;
  stack2 = "common.critical";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n          <td class=\"service-name-cell\">");
  stack1 = depth0;
  stack2 = "error.serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td class=\"property-name-cell\">\n            ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </td>\n          <td class=\"property-value-cell\">");
  stack1 = depth0;
  stack2 = "error.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td class=\"property-description-cell\">\n            ");
  stack1 = depth0;
  stack2 = "error.messages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "message";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <div class=\"property-description\">");
  stack1 = depth0;
  stack2 = "error.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          </td>\n        </tr>\n      ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "error";
  stack3 = depth0;
  stack4 = "showConfigProperty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "error.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "error.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"property-message\">");
  stack1 = depth0;
  stack2 = "message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <p>\n    ");
  stack1 = depth0;
  stack2 = "installer.step7.popup.validation.issues.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    <span class=\"badge badge-warning\">");
  stack1 = depth0;
  stack2 = "view.configErrors.issues.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </p>\n  <p class=\"recommendations-message\">\n    ");
  stack1 = depth0;
  stack2 = "installer.step7.popup.validation.issues.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  <div ");
  stack1 = {};
  stack2 = "controller.isInstallWizard::limited-height-2 :config-validation-warnings";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <table class=\"table table-hover\">\n      <thead>\n      <tr>\n        <th class=\"issue-type-cell\">");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"service-name-cell\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-name-cell\">");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-value-cell\">");
  stack1 = depth0;
  stack2 = "installer.step7.popup.currentValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"property-description-cell\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      </tr>\n      </thead>\n      <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.configErrors.issues";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "error";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  </div>\n");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr ");
  stack1 = {};
  stack2 = "error.cssClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <td class=\"issue-type-cell\">\n            ");
  stack1 = depth0;
  stack2 = "error.isError";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </td>\n\n          ");
  stack1 = depth0;
  stack2 = "error.isGeneral";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(18, program18, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tr>\n      ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "common.error";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "common.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "error.messages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "message";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <td colspan=\"4\">");
  stack1 = depth0;
  stack2 = "error.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <td class=\"service-name-cell\">");
  stack1 = depth0;
  stack2 = "error.serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"property-name-cell\">\n              ");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n            <td class=\"property-value-cell\">");
  stack1 = depth0;
  stack2 = "error.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"property-description-cell\">\n              ");
  stack1 = depth0;
  stack2 = "error.messages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "message";
  stack7 = helpers.each;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <div class=\"property-description\">");
  stack1 = depth0;
  stack2 = "error.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </td>\n          ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "error";
  stack3 = depth0;
  stack4 = "showConfigProperty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "error.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "error.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"property-message\">");
  stack1 = depth0;
  stack2 = "message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n              ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.configErrors.criticalIssues.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.configErrors.issues.length";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/confirmation_feedback", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <br />\n  <div class='alert alert-warning'>\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.additionalWarningMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.turnOnMmMsg";
  stack3['labelBinding'] = stack4;
  stack4 = "view.parentView.runMmOperation";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n<div>\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.statusMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  stack1 = depth0;
  stack2 = "view.parentView.additionalWarningMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.parentView.putInMaintenance";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/dependent_configs_list", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.allConfigsWithErrors.length";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.ValidationsView";
  stack3 = {};
  stack4 = "controller.configErrorList";
  stack3['configErrorsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "controller.hasChangedDependencies";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <h5>\n      ");
  stack1 = depth0;
  stack2 = "installer.step7.requirements.popover.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "view.allConfigsWithErrors.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    </h5>\n    <div class=\"recommendations-message\">\n      ");
  stack1 = depth0;
  stack2 = "installer.step7.requirements.popover.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n\n    <table class=\"table table-hover required-table\">\n      <thead>\n      <tr>\n        <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      </tr>\n      </thead>\n      <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.allConfigsWithErrors";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "config";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <tr>\n          <td>");
  stack1 = depth0;
  stack2 = "config.serviceName";
  foundHelper = helpers.formatRole;
  stack3 = foundHelper || depth0.formatRole;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatRole", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "config";
  stack3 = depth0;
  stack4 = "showConfigProperty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "config.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </td>\n          <td>\n            <button ");
  stack1 = depth0;
  stack2 = "config";
  stack3 = depth0;
  stack4 = "showConfigProperty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </td>\n        </tr>\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <h5>\n      ");
  stack1 = depth0;
  stack2 = "installer.step7.recommendations.popover.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"badge badge-warning\">");
  stack1 = depth0;
  stack2 = "view.recommendations.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    </h5>\n    <div class=\"recommendations-message\">\n      ");
  stack1 = depth0;
  stack2 = "installer.step7.recommendations.popover.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.DependentConfigsTableView";
  stack3 = {};
  stack4 = "view.recommendations";
  stack3['recommendationsBinding'] = stack4;
  stack4 = "view.isRecommendationsClickable";
  stack3['isClickableBinding'] = stack4;
  stack4 = "view.showRecommendationsPopovers";
  stack3['showPopoversBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.DependentConfigsTableView";
  stack3 = {};
  stack4 = "view.requiredChanges";
  stack3['recommendationsBinding'] = stack4;
  stack4 = false;
  stack3['isEditable'] = stack4;
  stack4 = "view.isRecommendationsClickable";
  stack3['isClickableBinding'] = stack4;
  stack4 = "view.showRecommendationsPopovers";
  stack3['showPopoversBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "controller.isInstallWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<span id=\"config-dependencies\" class=\"limited-height-2\">\n  ");
  stack1 = depth0;
  stack2 = "view.recommendations.length";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.requiredChanges.length";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</span>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/dependent_configs_table", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <h4>");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <th class=\"check-box-col\">");
  stack1 = depth0;
  stack2 = "view.parentView.toggleAll";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<label ");
  stack1 = {};
  stack2 = "view.parentView.toggleAllId";
  stack1['for'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></label></th>\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "popup.dependent.configs.table.recommendedValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "popup.dependent.configs.table.newValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <tr ");
  stack1 = {};
  stack2 = "recommendation.saveRecommended:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.isEditable";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <td class=\"config-dependency-name\" ");
  stack1 = {};
  stack2 = "recommendation.propertyTitle";
  stack1['data-original-title'] = stack2;
  stack2 = "recommendation.propertyDescription";
  stack1['data-content'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.isClickable";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </td>\n      <td class=\"config-dependency-service\">");
  stack1 = depth0;
  stack2 = "recommendation.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td class=\"config-dependency-group\">\n          <span ");
  stack1 = {};
  stack2 = "recommendation.allowChangeGroup::not-active-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n            <a href=\"javascript:void(null);\" class=\"black\"\n              ");
  stack1 = depth0;
  stack2 = "recommendation.serviceName";
  stack3 = depth0;
  stack4 = "showSelectGroupPopup";
  stack5 = {};
  stack6 = "App.router.mainServiceInfoConfigsController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "recommendation.configGroup";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </span>\n      </td>\n      <td class=\"config-dependency-filename\">");
  stack1 = depth0;
  stack2 = "recommendation.propertyFileName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>\n        <div>\n          ");
  stack1 = depth0;
  stack2 = "App.ConfigDiffView";
  stack3 = {};
  stack4 = "recommendation";
  stack3['configBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </td>\n    </tr>\n  ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <td class=\"check-box-col\">");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "recommendation.saveRecommended";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "recommendation";
  stack3 = depth0;
  stack4 = "showConfigProperty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "recommendation.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "recommendation.propertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.hideMessage";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<table class=\"table table-hover\">\n  <thead>\n    <tr>\n      ");
  stack1 = depth0;
  stack2 = "view.isEditable";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <th>");
  stack1 = depth0;
  stack2 = "common.property";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.configGroup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.fileName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>\n        <div class=\"config-dependencies-headings-wrapper\">\n          <div class=\"config-dependencies-heading\">\n            ");
  stack1 = depth0;
  stack2 = "popup.dependent.configs.table.originalValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n          <div class=\"config-dependencies-heading\">\n            ");
  stack1 = depth0;
  stack2 = "view.isEditable";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </th>\n    </tr>\n  </thead>\n  <tbody>\n  ");
  stack1 = depth0;
  stack2 = "view.recommendations";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "recommendation";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/hosts_table_list_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <table class=\"table table-hover\">\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.items";
  stack3 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tr>\n            <td>");
  stack1 = depth0;
  stack2 = "name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td>");
  stack1 = depth0;
  stack2 = "status";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </tr>\n        ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <ul class=\"list-unstyled\">\n      ");
  stack1 = depth0;
  stack2 = "view.items";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <li>");
  stack1 = depth0;
  stack2 = "item";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<div class=\"host-elements\">\n  ");
  stack1 = depth0;
  stack2 = "view.isObjectsList";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/install_repo_confirmation", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"available-services-list\">\n      ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <p class=\"service-version-info\">\n            <span class=\"col-md-1\">\n              ");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n            <span class=\"label available\">\n              ");
  stack1 = depth0;
  stack2 = "service.latestVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n          </p>\n      ");
  return buffer;}

  data.buffer.push("\n<p>");
  stack1 = depth0;
  stack2 = "popupBody";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n");
  stack1 = depth0;
  stack2 = "view.showAvailableServices";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/invalid_KDC_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"alert alert-warning\">\n  ");
  stack1 = depth0;
  stack2 = "view.warningMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<form autocomplete=\"off\" class=\"form-horizontal\">\n  <div class=\"control-group\">\n    <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "popup.invalid.KDC.admin.principal";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.parentView.principal";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div class=\"control-group\">\n    <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "popup.invalid.KDC.admin.password";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "view.parentView.password";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div class=\"control-group\">\n    <span class=\"control-label\"></span>\n    <div>\n      <label>\n        ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.parentView.storeCredentials";
  stack3['checkedBinding'] = stack4;
  stack4 = "view.parentView.checkboxDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "pull-left";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <span ");
  stack1 = {};
  stack2 = ":mls view.parentView.checkboxDisabled:muted";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "admin.kerberos.credentials.store.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          <a class=\"glyphicon-question-sign glyphicon glyphicon-blue\" rel=\"tooltip\" href=\"javascript:void(null);\" data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "view.parentView.hintMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n        </span>\n      </label>\n    </div>\n  </div>\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/log_tail_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"top-wrap top-wrap-header\">\n  <div class=\"log-tail-popup-info\">\n    <strong>");
  stack1 = depth0;
  stack2 = "common.file";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": </strong>\n    <span class=\"muted\">");
  stack1 = depth0;
  stack2 = "view.content.filePath";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    <div class=\"pull-right action-bar\">\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleCopy";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-copy\"></i>\n        ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "openInNewTab";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"icon-external-link\"></i>\n        ");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n      <a class=\"open-in-log-search\" ");
  stack1 = {};
  stack2 = "view.logSearchUrl";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">\n        <i class=\"icon-external-link\"></i>\n        ");
  stack1 = depth0;
  stack2 = "popup.logTail.openInLogSearch";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </div>\n  </div>\n  <div class=\"clearfix\"></div>\n</div>\n<div>\n  <div ");
  stack1 = {};
  stack2 = ":task-detail-info view.isCopyActive::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"clipboard-wrap task-detail-log-info\">\n      ");
  stack1 = depth0;
  stack2 = "App.NotScrollableTextArea";
  stack3 = {};
  stack4 = "task-detail-log-clipboard copy-textarea";
  stack3['class'] = stack4;
  stack4 = "view.copyContent";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n  <div ");
  stack1 = {};
  stack2 = "view.isCopyActive:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.logTailContentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/logs_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div id=\"logs-popup\">\n  <div class=\"controls-block pull-right\">\n    <a ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "taskInfo";
  stack3 = depth0;
  stack4 = "textTrigger";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-copy\"><i\n      class=\"glyphicon glyphicon-copy\"></i> ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "openTaskLogInDialog";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-open-dialog\"><i\n      class=\"icon-external-link\"></i> ");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n  <div class=\"clearfix\"></div>\n\n  <div class=\"task-detail-log-info\">\n    <div class=\"content-area\">\n      <div class=\"task-detail-log-clipboard-wrap\"></div>\n      <div class=\"task-detail-log-maintext\">\n        <pre>");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/prompt_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "view.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <a href=\"#\" data-toggle=\"tooltip\" class=\"btn-sm\"\n            ");
  stack1 = depth0;
  stack2 = "";
  stack3 = depth0;
  stack4 = "doRestoreDefaultValue";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = {};
  stack2 = "common.reset.default";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-undo\"></i>\n          </a>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  return buffer;}

  data.buffer.push("\n<div class=\"prompt-popup\">\n  <form>\n    <div ");
  stack1 = {};
  stack2 = ":form-group view.parentView.isInvalid:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.text";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <h4>");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n      <p>");
  stack1 = depth0;
  stack2 = "view.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n      <div>\n        <strong>");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;&nbsp;</strong>\n        ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "prompt-input form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.inputValue";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.isChanged";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.isInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </form>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/revert_patch_upgrade_confirmation", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td>");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td class=\"service-version-info\">\n            <span class=\"label available\">\n              ");
  stack1 = depth0;
  stack2 = "service.toVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n          </td>\n          <td><i class=\"icon-arrow-left\"></i></td>\n          <td class=\"service-version-info\">\n            <span class=\"label available\">\n              ");
  stack1 = depth0;
  stack2 = "service.fromVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n          </td>\n        </tr>\n      ");
  return buffer;}

  data.buffer.push("\n<p>");
  stack1 = depth0;
  stack2 = "popupBody";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n<div>\n  <table>\n    <thead>\n      <tr>\n        <th></th>\n        <th><strong>");
  stack1 = depth0;
  stack2 = "view.stackToVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></th>\n        <th></th>\n        <th><strong>");
  stack1 = depth0;
  stack2 = "view.stackFromVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></th>\n      </tr>\n    </thead>\n    <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.servicesToBeReverted";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/select_groups_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "view.groups";
  stack3['contentBinding'] = stack4;
  stack4 = "view.selectedGroup";
  stack3['valueBinding'] = stack4;
  stack4 = "view.disabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <span>");
  stack1 = depth0;
  stack2 = "popup.dependent.configs.select.config.group.no.groups";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  return buffer;}

  data.buffer.push("\n\n<form autocomplete=\"off\" class=\"form-horizontal\">\n  <div class=\"form-group form-text\">\n    <div class=\"col-sm-3 align-right\"><strong>");
  stack1 = depth0;
  stack2 = "view.serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;</strong>&nbsp;</div>\n    <div class=\"col-sm-7\">\n      ");
  stack1 = depth0;
  stack2 = "view.hasGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</form>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/update_replication_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <a href=\"#\" data-toggle=\"tooltip\" class=\"btn-sm\"\n                        ");
  stack1 = depth0;
  stack2 = "";
  stack3 = depth0;
  stack4 = "doRestoreDefaultValue";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  stack1 = {};
  stack2 = "common.reset.default";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                        <i class=\"icon-undo\"></i>\n                    </a>\n                ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            ");
  return buffer;}

  data.buffer.push("\n<div class=\"prompt-popup\">\n    <form>\n        <div ");
  stack1 = {};
  stack2 = ":form-group view.parentView.isInvalid:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <h4>");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n            <p>");
  stack1 = depth0;
  stack2 = "view.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n            <br/>\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "view.zkquorumText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n            <div>\n                &nbsp;&nbsp;\n                ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "large-prompt-input form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.zkquorum";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <br/>\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "view.zkportText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n            <div>\n                &nbsp;&nbsp;\n                ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "prompt-input form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.zkport";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <br/>\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "view.parentzkeyText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n            <div>\n                &nbsp;&nbsp;\n                ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "prompt-input form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.parentzkey";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <br/>\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "view.peerIdText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n            <div>\n                &nbsp;&nbsp;\n                ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "prompt-input form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.peerId";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <div>\n                ");
  stack1 = depth0;
  stack2 = "view.parentView.isChanged";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            ");
  stack1 = depth0;
  stack2 = "view.parentView.isInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n    </form>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/widget_browser_footer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", blockHelperMissing=helpers.blockHelperMissing, helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"checkbox pull-left\">\n      ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "footer-checkbox";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.isShowMineOnly";
  stack3['checkedBinding'] = stack4;
  stack4 = "dashboard.widgets.browser.footer.checkbox";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"modal-footer\">\n  ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <button class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.close";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/modal_popups/widget_browser_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"widget-browser-content\">\n\n      <!--Filters bar: service name filter, status filter here-->\n      <ul id=\"services-filter-bar\" class=\"nav nav-tabs\">\n        ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n\n      <!--Widgets table two column-->\n      <div id=\"widgets-info\">\n        ");
  stack1 = depth0;
  stack2 = "view.isWidgetEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n    <div class=\"table-overlay\">\n       ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li ");
  stack1 = {};
  stack2 = "service.isActive:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a ");
  stack1 = depth0;
  stack2 = "service.value";
  stack3 = depth0;
  stack4 = "filterByService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n        ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li class=\"pull-right\">\n            <button type=\"button\" class=\"btn btn-primary\" ");
  stack1 = {};
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "createWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n              <i class=\"glyphicon glyphicon-plus\"></i> &nbsp; ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.create";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n          </li>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n           <div class=\"no-widgets-text\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.browser.noWidgets";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div class=\"widgets-info-container row\">\n            ");
  stack1 = depth0;
  stack2 = "view.filteredContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"col-md-6 widget-info-section\">\n                <div class=\"row\">\n                  <div class=\"col-md-4\">\n                    <div class=\"icon\">\n                      <a class=\"widget-icon-image\" target=\"_blank\"><img ");
  stack1 = {};
  stack2 = "widget.iconPath";
  stack1['src'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n                    </div>\n                  </div>\n                  <div class=\"col-md-8\">\n                    <div class=\"label-description\">\n                      <p class=\"label-text\">\n                        ");
  stack1 = depth0;
  stack2 = "widget.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  stack1 = depth0;
  stack2 = "widget.tag";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </p>\n                      <p class=\"is-shared-icon pull-right\">\n                        ");
  stack1 = depth0;
  stack2 = "widget.isShared";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </p>\n                      <p class=\"description-text\">");
  stack1 = depth0;
  stack2 = "widget.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n                    </div>\n                    ");
  data.buffer.push("\n                    <div class=\"widget-section-actions align-right\">\n                      ");
  stack1 = depth0;
  stack2 = "widget.added";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(16, program16, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                    ");
  data.buffer.push("\n                  </div>\n                </div>\n              </div>\n            ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push(" - ");
  stack1 = depth0;
  stack2 = "widget.tag";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n                          <i class=\"glyphicon glyphicon-group\" rel=\"shared-icon-tooltip\"\n                            ");
  stack1 = {};
  stack2 = "dashboard.widgets.browser.shareIcon.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n                        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                        <button type=\"button\" class=\"btn added-btn\" ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = depth0;
  stack4 = "hideWidgetBrowser";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n                          <i class=\"glyphicon glyphicon-ok\"></i> &nbsp; ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.browser.action.added";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        </button>\n                      ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                        <div class=\"btn-group\">\n                          <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                            ");
  stack1 = depth0;
  stack2 = "common.more";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<span class=\"caret\"></span>\n                          </button>\n                          <ul class=\"dropdown-menu\">\n                            <li>\n                              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = depth0;
  stack4 = "deleteWidget";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.browser.action.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                              </a>\n                            </li>\n                            <li>\n                              ");
  stack1 = depth0;
  stack2 = "widget.isShared";
  stack3 = helpers.unless;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </li>\n                          </ul>\n                        </div>\n                        <button type=\"button\" class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = depth0;
  stack4 = "addWidget";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n                          ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.browser.action.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        </button>\n                      ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = depth0;
  stack4 = "shareWidget";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                  ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.browser.action.share";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                                </a>\n                              ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"widget-browser-popup\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/pagination", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <a ");
  stack1 = depth0;
  stack2 = "firstPage";
  stack3 = {};
  stack4 = "view.dataView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isPreviousDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><span class=\"icon icon-step-backward\"></span></a>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <a ");
  stack1 = depth0;
  stack2 = "lastPage";
  stack3 = {};
  stack4 = "view.dataView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isNextDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><span class=\"icon icon-step-forward\"></span></a>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"pagination-block-item\">\n  ");
  stack1 = depth0;
  stack2 = "tableView.filters.itemsPerPage";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "view.rowsPerPageSelectView";
  stack3 = {};
  stack4 = "view.dataView.displayLength";
  stack3['selectionBinding'] = stack4;
  stack4 = "pagination-select";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<div class=\"pagination-block-item\">");
  stack1 = depth0;
  stack2 = "view.dataView.paginationInfo";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n<div class=\"pagination-block-item\">\n  ");
  stack1 = depth0;
  stack2 = "hasFirstAndLastPageLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <a ");
  stack1 = depth0;
  stack2 = "previousPage";
  stack3 = {};
  stack4 = "view.dataView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isPreviousDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><span class=\"icon icon-chevron-left\"></span></a>\n  <a ");
  stack1 = depth0;
  stack2 = "nextPage";
  stack3 = {};
  stack4 = "view.dataView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isNextDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><span class=\"icon icon-chevron-right\"></span></a>\n  ");
  stack1 = depth0;
  stack2 = "hasFirstAndLastPageLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/progress", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  
  data.buffer.push("\n<div id=\"common-progress-page\" class=\"wizard-content col-md-12\">\n");}

function program3(depth0,data) {
  
  
  data.buffer.push("\n<div id=\"common-progress-page\" class=\"wizard-content col-md-9\">\n");}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <button class=\"btn btn-primary retry\" ");
  stack1 = depth0;
  stack2 = "retry";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>&nbsp;\n        ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n    ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "controller.tasks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "task";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.taskView";
  stack3 = {};
  stack4 = "task";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <div class=\"item row\">\n              <div ");
  stack1 = {};
  stack2 = "view.linkClass view.parentView.labelWidth";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i ");
  stack1 = {};
  stack2 = "view.icon view.iconColor";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                <a ");
  stack1 = depth0;
  stack2 = "task";
  stack3 = depth0;
  stack4 = "showHostProgressPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >");
  stack1 = depth0;
  stack2 = "task.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </div>\n              <div ");
  stack1 = {};
  stack2 = "view.showProgressBar::hide :col-md-5";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <div ");
  stack1 = {};
  stack2 = ":progress-wrapper controller.isRollback::col-md-8 controller.isRollback:col-md-3";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <div class=\"progress\">\n                    <div class=\"progress-bar-striped active progress-bar-info progress-bar\" ");
  stack1 = {};
  stack2 = "view.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                  </div>\n                </div>\n                <div ");
  stack1 = {};
  stack2 = ":col-md-1 view.hidePercent:noDisplay";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "task.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&#37;</div>\n              </div>\n              <div>\n                ");
  stack1 = depth0;
  stack2 = "task.showRetry";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "task.showRollback";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "task.showSkip";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            </div>\n          ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <a ");
  stack1 = depth0;
  stack2 = "retryTask";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\"\n                                                              rel=\"tooltip\"\n                                                              data-trigger=\"click\" ");
  stack1 = {};
  stack2 = "view.showDBTooltip:testDBRetryTooltip";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>&nbsp;\n                    ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <a ");
  stack1 = depth0;
  stack2 = "rollback";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\">\n                    <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>\n                    ");
  stack1 = depth0;
  stack2 = "common.rollBack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <a ");
  stack1 = depth0;
  stack2 = "skipTask";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\">\n                    <i class=\"glyphicon-step-forward glyphicon glyphicon-white\"></i>\n                    ");
  stack1 = depth0;
  stack2 = "common.skip";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "view.showBackButton";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "done";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.WizardBackButton";
  stack3 = {};
  stack4 = "back";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-back";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.submitButtonText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.disableKerberos";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "view.headerTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n\n  <div ");
  stack1 = {};
  stack2 = "view.noticeClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.notice";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.showRetry";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(16, program16, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.isSimpleModal";
  stack3 = helpers.unless;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/rolling_restart_view", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <p>\n        ");
  stack1 = depth0;
  stack2 = "view.maintainanceMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </p>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <p>\n        ");
  stack1 = depth0;
  stack2 = "view.suggestTurnOnMaintenanceMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </p>\n    ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <table>\n      <tr>\n        <td>");
  stack1 = depth0;
  stack2 = "common.restart";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.batchSize";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control input-sm";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td><span>");
  stack1 = depth0;
  stack2 = "view.batchSizeMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n      </tr>\n      <tr>\n        <td>");
  stack1 = depth0;
  stack2 = "rollingrestart.dialog.msg.timegap.prefix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.interBatchWaitTimeSeconds";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control input-sm";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "rollingrestart.dialog.msg.timegap.suffix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      </tr>\n      <tr>\n        <td>");
  stack1 = depth0;
  stack2 = "rollingrestart.dialog.msg.toleration.prefix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.tolerateSize";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control input-sm";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "rollingrestart.dialog.msg.toleration.suffix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      </tr>\n      <tr>\n        <td>&nbsp;</td>\n        <td colspan=\"2\">\n          ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.staleConfigsOnlyMessage";
  stack3['labelBinding'] = stack4;
  stack4 = "view.staleConfigsOnly";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </td>\n      </tr>\n      <tr>\n        <td>&nbsp;</td>\n        <td colspan=\"2\">\n          ");
  stack1 = depth0;
  stack2 = "view.isServiceInMM";
  stack3 = helpers.unless;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n      </tr>\n\n    </table>\n  ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.turnOnMmMsg";
  stack3['labelBinding'] = stack4;
  stack4 = "view.turnOnMm";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"alert alert-danger\">\n      <ul>\n      ");
  stack1 = depth0;
  stack2 = "view.errors";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "error";
  stack7 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <li>");
  stack1 = depth0;
  stack2 = "error";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n      ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"alert alert-warning\">\n      <ul>\n      ");
  stack1 = depth0;
  stack2 = "view.warnings";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "warning";
  stack7 = helpers.each;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <li>");
  stack1 = depth0;
  stack2 = "warning";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"rolling-restart-view\">\n  <div class=\"alert alert-info\">\n    <p>\n      ");
  stack1 = depth0;
  stack2 = "view.restartMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </p>\n    ");
  stack1 = depth0;
  stack2 = "view.maintainanceMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.suggestTurnOnMaintenanceMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.errors";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.warnings";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/selectable_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div>\n  <div class=\"task-top-wrap\">\n    <div class=\"task-detail-ico-wrap\">\n      <a href=\"#\" ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "textTrigger";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-copy\"><i class=\"glyphicon glyphicon-copy\"></i> ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </div>\n  </div>\n  <div class=\"task-detail-log-info\">\n    <div class=\"content-area\" >\n      <div ");
  stack1 = {};
  stack2 = "view.textareaVisible::hidden :task-detail-log-clipboard-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <textarea class=\"task-detail-log-clipboard form-control\"></textarea>\n      </div>\n      <div ");
  stack1 = {};
  stack2 = "view.textareaVisible:hidden :task-detail-log-maintext";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "content";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/settings", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <table class=\"table table-hover\">\n        <thead>\n        <tr>\n          <th>");
  stack1 = depth0;
  stack2 = "common.cluster";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  stack1 = depth0;
  stack2 = "app.settings.clusterRole";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.privileges.clusters";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "cluster";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "noClusterPriv";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n        <thead class=\"view-permission-header\">\n        <tr>\n          <th>");
  stack1 = depth0;
  stack2 = "common.view";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  stack1 = depth0;
  stack2 = "app.settings.viewPermissions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.privileges.views";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "v";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "noViewPriv";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <tr>\n            <td>\n              <span class=\"glyphicon glyphicon-cloud\"></span>\n              <span>");
  stack1 = depth0;
  stack2 = "cluster.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </td>\n            <td>\n              ");
  stack1 = depth0;
  stack2 = "cluster.privileges";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "privilege";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n          </tr>\n        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span tooltip=\"");
  stack1 = depth0;
  stack2 = "privilege";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\">");
  stack1 = depth0;
  stack2 = "privilege";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;</span>\n              ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tr>\n              <td>");
  stack1 = depth0;
  stack2 = "app.settings.no.cluster.privileges";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </tr>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <tr>\n            <td>\n              <span class=\"glyphicon glyphicon-th\"></span>\n              <span>");
  stack1 = depth0;
  stack2 = "v.instance_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </td>\n            <td>\n              ");
  stack1 = depth0;
  stack2 = "v.privileges";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "privilege";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n          </tr>\n        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span tooltip=\"");
  stack1 = depth0;
  stack2 = "privilege";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" >");
  stack1 = depth0;
  stack2 = "privilege";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;</span>\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "app.settings.no.view.privileges";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n            </tr>\n        ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "app.settings.admin.all.privileges";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.privileges";
  stack3 = helpers.unless;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "app.settings.no.privileges";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"admin-user-settings\">\n  <div class=\"setting-wrapper\">\n    <h4>");
  stack1 = depth0;
  stack2 = "app.settings.categories.general";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <hr />\n    <div class=\"checkbox\">\n      ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isNotShowBgChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "app.settings.notshowBgOperationsPopup";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n  <div class=\"setting-wrapper\">\n    <h4>");
  stack1 = depth0;
  stack2 = "app.settings.categories.locale";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <hr />\n    <label>\n      ");
  stack1 = depth0;
  stack2 = "app.settings.selectTimezone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "view.timezonesList";
  stack3['contentBinding'] = stack4;
  stack4 = "content.label";
  stack3['optionLabelPath'] = stack4;
  stack4 = "content.value";
  stack3['optionValuePath'] = stack4;
  stack4 = "view.parentView.selectedTimezone";
  stack3['selectionBinding'] = stack4;
  stack4 = "group-select select-group-box form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </label>\n  </div>\n  <div>\n    ");
  stack1 = depth0;
  stack2 = "view.hidePrivileges";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isAdmin";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/time_range", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.presetView";
  stack3 = {};
  stack4 = "preset";
  stack3['presetBinding'] = stack4;
  stack4 = "view";
  stack3['widgetBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"span title\">\n  ");
  stack1 = depth0;
  stack2 = "common.timeRange";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <i class=\"glyphicon glyphicon-question-sign\"></i>\n</div>\n<ul class=\"display-inline-block nav nav-pills\">\n  ");
  stack1 = depth0;
  stack2 = "view.presets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "preset";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "from";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.dateFromView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "to";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.dateToView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</ul>\n<div id=\"slider\">\n  <span class=\"period-marker\">");
  stack1 = depth0;
  stack2 = "view.rangeLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  <span class=\"now-marker\">");
  stack1 = depth0;
  stack2 = "view.nowLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/time_range_list", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push(": ");
  stack1 = depth0;
  stack2 = "view.parentView.customStartTimeFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "setTimeRange";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "option.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n  ");
  return buffer;}

  data.buffer.push("\n\n<button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" rel=\"tooltip\" ");
  stack1 = {};
  stack2 = "view.parentView.tooltipMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.currentTimeRange.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "view.parentView.isCustomTimeRange";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push(" &nbsp;<span class=\"caret\"></span>\n</button>\n<ul class=\"dropdown-menu\">\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.timeRangeOptions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</ul>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/widget/gauge_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n    <div class=\"caption title col-md-11\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isMoving:hidden :corner-icon :widget-icons view.isPreview:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <li>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nsLayout";
  stack3 = depth0;
  stack4 = "view.id";
  stack5 = depth0;
  stack6 = "hideWidget";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      </ul>\n    </div>\n    <div class=\"content\">\n      ");
  stack1 = depth0;
  stack2 = "view.isUnavailable";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        - ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "cloneWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.clone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isOverflowed";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"alert alert-warning overflow-warning\">\n            ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.gauge.overflow.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"grey unavailable\">\n            ");
  stack1 = depth0;
  stack2 = "common.na";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.chartView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"hidden-description\">\n        ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"gauge-widget img-thumbnail\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/widget/graph_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n    <div class=\"caption title col-md-11\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isMoving:hidden :corner-icon :widget-icons view.isPreview:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n      <ul class=\"dropdown-menu\">\n        <li>\n          <a href=\"#\" ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "view.exportToCSVArgument";
  stack3 = depth0;
  stack4 = "exportGraphData";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-file-text\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.csv";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </li>\n        <li>\n          <a href=\"#\" ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "exportGraphData";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"icon-file-text-alt\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.json";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </li>\n        ");
  stack1 = depth0;
  stack2 = "controller.isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <li>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nsLayout";
  stack3 = depth0;
  stack4 = "view.id";
  stack5 = depth0;
  stack6 = "hideWidget";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      </ul>\n    </div>\n    <div class=\"content\"> ");
  stack1 = depth0;
  stack2 = "view.graphView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        - ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li>\n            <a href=\"#\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "cloneWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.clone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"graph-widget img-thumbnail\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/widget/heatmap_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = "controller.rackClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "App.MainChartsHeatmapRackView";
  stack3 = {};
  stack4 = "rack";
  stack3['rackBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"heatmap-widget\">\n  <div class=\"heatmap-content\">\n    <h4 id=\"heatmap-metric-loading\">\n	        <span class=\"background-text\" id=\"heatmap-metric-title\">\n						");
  stack1 = depth0;
  stack2 = "view.content.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " &nbsp;\n            ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n					</span>\n    </h4>\n\n    <div class=\"row\">\n      ");
  stack1 = depth0;
  stack2 = "view.racks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "rack";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "App.MainChartsHeatmapHostDetailView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/widget/number_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n    <div class=\"caption title col-md-11\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isMoving:hidden :corner-icon :widget-icons view.isPreview:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <li>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nsLayout";
  stack3 = depth0;
  stack4 = "view.id";
  stack5 = depth0;
  stack6 = "hideWidget";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      </ul>\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.contentColor :content";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.displayValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        - ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "cloneWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.clone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"hidden-description\">\n        ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"number-widget img-thumbnail\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/common/widget/template_widget", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n    <div class=\"caption title col-md-11\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.widgetName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.parentView.isMoving:hidden :corner-icon :widget-icons view.isPreview:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <li>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nsLayout";
  stack3 = depth0;
  stack4 = "view.id";
  stack5 = depth0;
  stack6 = "hideWidget";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      </ul>\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.contentColor :content";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.displayValue";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n    ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        - ");
  stack1 = depth0;
  stack2 = "view.content.tag";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n          <li>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "cloneWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.clone";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"hidden-description\">\n        ");
  stack1 = depth0;
  stack2 = "view.content.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"template-widget img-thumbnail\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/experimental", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.supports";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <table class=\"table table-hover\">\n      <thead>\n        <tr>\n          <th style=\"width: 20%;\">Experimental Functionality</th>\n          <th style=\"width: 60%;\">Description</th>\n          <th style=\"width: 20%;\">Enabled?</th>\n        </tr>\n      </thead>\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "controller.supports";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "support";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n    </table>\n    <div class=\"control-group\" style=\"margin-bottom: 70px;\">\n        <div class=\"pull-right\">\n        <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "doCancel";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "form.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n        <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "doSave";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      </div>\n  </div>\n    <hr>\n    <div class=\"control-group\" style=\"margin-bottom: 100px;\">\n      ");
  stack1 = depth0;
  stack2 = "CLUSTER.MANAGE_USER_PERSISTED_DATA";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <tr ");
  stack1 = {};
  stack2 = "support.selected:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <td>");
  stack1 = depth0;
  stack2 = "support.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td></td>\n            <td>");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "support.selected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </tr>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"pull-left\" style=\"margin-bottom: 100px;\">\n          <button class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "doResetUIStates";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "reset.ui.states";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          <p class=\"pull-right\" style=\"margin-left: 10px; padding-top: 5px;\">\n              Reset UI state locally and on the server\n          </p>\n        </div>\n      ");
  return buffer;}

function program7(depth0,data) {
  
  
  data.buffer.push("\n    <div class=\"alert alert-danger\">\n      No application supports found. Please login to Ambari and revisit this page.\n    </div>\n  ");}

function program9(depth0,data) {
  
  
  data.buffer.push("\n  <div class=\"alert alert-danger\">\n    No application found. Please login to Ambari and revisit this page.\n  </div>\n");}

  data.buffer.push("\n\n<div class=\"alert alert-danger\">\n  <h2>Ambari's Experimental Functionality</h2>\n  <p>\n    <span class=\"label label-important\">Important</span> <strong>Changes made here are neither tested nor supported by Ambari</strong>\n  </p>\n  <p>\n    Ambari's experimental functionality is provided <strong>as-is</strong>.<br>\n    Any changes will effect application behavior, including data corruption. Users making changes do so at their own risk.<br>\n    Experimental functionality can be modified or removed at any time without notice.\n  </p>\n</div>\n\n");
  stack1 = depth0;
  stack2 = "App";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/installer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <h3 id=\"version-mismatch-title\">");
  stack1 = depth0;
  stack2 = "app.versionMismatchAlert.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h3>\n  <div class=\"alert alert-danger\">\n    ");
  stack1 = depth0;
  stack2 = "versionConflictAlertBody";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"wizard\">\n    <div class=\"wizard-body row\">\n      <div class=\"wizard-nav\">\n        <ul class=\"nav nav-pills nav-stacked\" ");
  stack1 = depth0;
  stack2 = "wizard-nav";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep0:active view.isStep0Disabled:disabled view.isStep0Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep0";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">0</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step0.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">7</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep8:active view.isStep8Disabled:disabled view.isStep8Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep8";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">8</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep9:active view.isStep9Disabled:disabled view.isStep9Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep9";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">9</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step9.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = depth0;
  stack2 = "wizard-nav-step";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isStep10:active view.isStep10Disabled:disabled view.isStep10Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\" ");
  stack1 = depth0;
  stack2 = "gotoStep10";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">10</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step10.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        </ul>\n      </div>\n        ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "isServerClientVersionMismatch";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/login", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n<div class=\"well login col-md-4 col-md-offset-4 col-sm-offset-4\">\n  <h2>");
  stack1 = depth0;
  stack2 = "login.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n  ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <form>\n    <div class=\"form-group\">\n      <label ");
  stack1 = depth0;
  stack2 = "username-label";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "login.username";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      ");
  stack1 = depth0;
  stack2 = "view.loginTextField";
  stack3 = {};
  stack4 = "loginName";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "isSubmitDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "username-input";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"form-group\">\n      <label ");
  stack1 = depth0;
  stack2 = "password-label";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.password";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      ");
  stack1 = depth0;
  stack2 = "view.passTextField";
  stack3 = {};
  stack4 = "password";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "isSubmitDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "password-input";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <button class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "login-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "login.loginButton";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n  </form>\n</div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-danger\" ");
  stack1 = depth0;
  stack2 = "login-error";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "isServerClientVersionMismatch";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <h3 id=\"version-mismatch-title\">");
  stack1 = depth0;
  stack2 = "app.versionMismatchAlert.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h3>\n  <div class=\"alert alert-danger\">\n    ");
  stack1 = depth0;
  stack2 = "versionConflictAlertBody";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "isClusterDataLoaded";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "isClusterDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <h2>");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n    <div class=\"progress\">\n      <div class=\"progress-bar progress-bar-striped active\" ");
  stack1 = depth0;
  stack2 = "main-progress-bar";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.clusterDataLoadedPercent";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    </div>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "isServerClientVersionMismatch";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-12\">\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/federation/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"step-description\">\n    <p>");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    <div class=\"alert alert-danger\">\n      ");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.alert";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.isNameServiceIdError:has-error :form-horizontal";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"row\">\n          <div class=\"col-md-2\">\n            <label>");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.nameserviceid.existing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</label>\n          </div>\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "controller.existingNameServicesString";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"col-md-2\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.nameserviceid";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</label>\n          </div>\n          <div class=\"col-md-3\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "content.nameServiceId";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n          <div class=\"col-md-7\">\n            <span ");
  stack1 = {};
  stack2 = ":help-block :validation-block controller.isNameServiceIdError::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.nameserviceid.error";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button ");
  stack1 = {};
  stack2 = "controller.isNameServiceIdError";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/federation/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.curNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td></td>\n              </tr>\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.addNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n              </tr>\n            ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step3.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = {};
  stack4 = "controller.isNotEditable";
  stack3['isNotEditableBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "pull-right";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"ha-step3-content\" class=\"well pre-scrollable\">\n        <div id=\"step8-info\">\n          <table id=\"ha-step3-review-table\">\n            ");
  stack1 = depth0;
  stack2 = "view.curNameNodes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.addNameNodes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </table>\n        </div>\n      </div>\n      <div id=\"serviceConfig\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "controller.isNextDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n    ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/federation/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/federation/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"rm-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.nameNodeFederation.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/activateStandby/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n</div>\n</div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/activateStandby/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div id=\"serviceConfig\">\n            ");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"activate-standby-step2-content\" class=\"well pre-scrollable\">\n        <div id=\"activate-standby-step2-info\">\n          <table id=\"ha-step3-review-table\">\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.hawqMaster";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "controller.content.hawqHosts.hawqMaster";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-disabled-red\"><i class=\"glyphicon glyphicon-minus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeDeleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.hawqStandby";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "controller.content.hawqHosts.hawqStandby";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-arrow-up\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.toBeActivated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n          </table>\n        </div>\n      </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n        <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n    </div>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/activateStandby/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/activateStandby/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"hawq-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.activateHawqStandby.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/addStandby/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/addStandby/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div id=\"serviceConfig\">\n        ");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n    <div id=\"ha-step3-content\" class=\"well pre-scrollable\">\n    <div id=\"step8-info\">\n      <table id=\"ha-step3-review-table\">\n        <tr>\n            <td>\n         <b>");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.hawqMaster";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n            </td>\n            <td>");
  stack1 = depth0;
  stack2 = "controller.content.hawqHosts.hawqMaster";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td></td>\n        </tr>\n        <tr>\n            <td>\n              <b>");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.newHawqStandby";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n            </td>\n            <td>");
  stack1 = depth0;
  stack2 = "controller.content.hawqHosts.newHawqStandby";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n        </tr>\n      </table>\n    </div>\n    </div>\n\n    ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/addStandby/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/addStandby/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"hawq-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.addHawqStandby.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/removeStandby/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n</div>\n</div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/removeStandby/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step2.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"remove-standby-step2-content\" class=\"well pre-scrollable\">\n        <div id=\"remove-standby-step2-info\">\n            <table id=\"ha-step3-review-table\">\n                <tr>\n                    <td>");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step2.hawqStandby";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                    <td>");
  stack1 = depth0;
  stack2 = "controller.content.hawqHosts.hawqStandby";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                    <td><span class=\"to-be-disabled-red\"><i class=\"glyphicon glyphicon-minus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeDeleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n                </tr>\n            </table>\n        </div>\n      </div>\n      <div id=\"serviceConfig\">\n        ");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step2.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/removeStandby/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/hawq/removeStandby/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"hawq-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.removeHawqStandby.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.journalNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>\n                  <ul>\n                    ");
  stack1 = depth0;
  stack2 = "view.journalNodesToAdd";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </td>\n                <td>\n                  <ul>\n                    ");
  stack1 = depth0;
  stack2 = "view.journalNodesToAdd";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </td>\n              </tr>\n            ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li>");
  stack1 = depth0;
  stack2 = "item";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li><span class=\"to-be-installed-green\"><i class=\"icon-plus\"></i>\n                                &nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></li>\n                    ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.journalNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>\n                  <ul>\n                    ");
  stack1 = depth0;
  stack2 = "view.journalNodesToDelete";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </td>\n                <td>\n                  <ul>\n                    ");
  stack1 = depth0;
  stack2 = "view.journalNodesToDelete";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </td>\n              </tr>\n            ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li>");
  stack1 = depth0;
  stack2 = "item";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li><span class=\"to-be-disabled-red\"><i class=\"icon-minus\"></i>\n                                &nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeDeleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></li>\n                    ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"alert alert-info\">\n            ");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step3.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = {};
  stack4 = "controller.isNotEditable";
  stack3['isNotEditableBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "pull-right";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step3.confirm.hosts.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"manage-journal-node-step2-content\" class=\"well pre-scrollable\">\n        <div id=\"step8-info\">\n          <table id=\"manage-journal-node-step2-review-table\">\n            ");
  stack1 = depth0;
  stack2 = "view.journalNodesToAdd.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            ");
  stack1 = depth0;
  stack2 = "view.journalNodesToDelete.length";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </table>\n        </div>\n      </div>\n      <div id=\"serviceConfig\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "controller.isNextDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n    ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <p class=\"step-description\">\n      ");
  stack1 = depth0;
  stack2 = "view.errorText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </p>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.step3BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n<div id=\"manage-journal-node-step3\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  ");
  stack1 = depth0;
  stack2 = "view.errorText";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "controller.isHDFSNameSpacesLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a ");
  stack1 = {};
  stack2 = "controller.isDataLoadedAndNextEnabled::disabled :btn :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n    <span class=\"pull-right btn-extra-info\">");
  stack1 = depth0;
  stack2 = "view.nnCheckPointText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step5", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.step5BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n<div id=\"manage-journal-node-step5\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step5.bodyHeader";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "controller.isHDFSNameSpacesLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a ");
  stack1 = {};
  stack2 = "controller.isNextEnabled::disabled :btn :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step6", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/step7", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/journalNode/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  
  data.buffer.push("3");}

function program3(depth0,data) {
  
  
  data.buffer.push("4");}

function program5(depth0,data) {
  
  
  data.buffer.push("4");}

function program7(depth0,data) {
  
  
  data.buffer.push("6");}

function program9(depth0,data) {
  
  
  data.buffer.push("5");}

function program11(depth0,data) {
  
  
  data.buffer.push("7");}

  data.buffer.push("\n\n<div class=\"wizard\">\n    <div class=\"wizard-body row\">\n        <div class=\"wizard-nav col-md-3\">\n            <ul class=\"nav nav-pills nav-stacked\">\n                <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed isDeleteOnly:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">");
  stack1 = depth0;
  stack2 = "isDeleteOnly";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed isDeleteOnly:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">");
  stack1 = depth0;
  stack2 = "isDeleteOnly";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n                <li ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">");
  stack1 = depth0;
  stack2 = "isDeleteOnly";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.manageJournalNode.wizard.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n            </ul>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/rollback", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard\">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/rollbackHA/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"wizard-content col-md-9\">\n      <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n\n      <div class=\"alert alert-info\">\n          <p>\n              Select Additional NameNode host:\n              ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.addNNHosts";
  stack3['contentBinding'] = stack4;
  stack4 = "view.selectedAddNNHost";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </p>\n\n          <div class=\"code-snippet\">Tip: Suggested host <b>");
  stack1 = depth0;
  stack2 = "view.tipAddNNHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b></div>\n      </div>\n      <div class=\"alert alert-info\">\n          <p>\n              Select Secondary NameNode host:\n              ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.sNNHosts";
  stack3['contentBinding'] = stack4;
  stack4 = "view.selectedSNNHost";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </p>\n\n          <div class=\"code-snippet\">Tip: Suggested host <b>");
  stack1 = depth0;
  stack2 = "view.tipSNNHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b></div>\n      </div>\n    </div>\n    <div class=\"wizard-footer col-md-12\">\n      <div class=\"btn-area\">\n          <a ");
  stack1 = {};
  stack2 = ":btn controller.isNameServiceIdValid::disabled :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n      </div>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n<div>\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/rollbackHA/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"alert alert-info\">\n      ");
  stack1 = depth0;
  stack2 = "view.step2BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n   <div class=\"btn-area\">\n      <a  ");
  stack1 = {};
  stack2 = ":btn :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n      <span class=\"pull-right btn-extra-info\">");
  stack1 = depth0;
  stack2 = "view.nnCheckPointText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/rollbackHA/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.rollbackHighAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a ");
  stack1 = {};
  stack2 = ":btn controller.isNameServiceIdValid::disabled :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.hawq.alert";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}

  data.buffer.push("\n<div id=\"ha-step1\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"step-description\">\n    <p>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    <div class=\"alert alert-warning\">\n      ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.alert";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "controller.isHawqInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div ");
  stack1 = {};
  stack2 = "view.showInputError:has-error :form-horizontal";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"row\">\n          <div class=\"col-md-2\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.nameserviceid";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</label>\n          </div>\n          <div class=\"col-md-3\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "content.nameServiceId";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n          <div class=\"col-md-7\">\n            <span ");
  stack1 = {};
  stack2 = ":help-block :validation-block view.showInputError::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.nameserviceid.error";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button ");
  stack1 = {};
  stack2 = ":btn controller.isNameServiceIdValid::disabled :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            <li>");
  stack1 = depth0;
  stack2 = "item.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            <li><span class=\"to-be-installed-green\"><i\n                                    class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></li>\n                        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = {};
  stack4 = "controller.isNotEditable";
  stack3['isNotEditableBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "pull-right";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n    <div id=\"ha-step3-content\" class=\"well pre-scrollable\">\n      <div id=\"step8-info\">\n        <table id=\"ha-step3-review-table\">\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.curNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "view.curNameNode";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td></td>\n            </tr>\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.secNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "view.secondaryNameNode";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-disabled-red\"><i class=\"glyphicon glyphicon-minus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeDeleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.addNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>");
  stack1 = depth0;
  stack2 = "view.addNameNode";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n            <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.journalNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td>\n                    <ul class=\"list-unstyled\">\n                        ");
  stack1 = depth0;
  stack2 = "view.journalNodes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                </td>\n                <td>\n                    <ul class=\"list-unstyled\">\n                        ");
  stack1 = depth0;
  stack2 = "view.journalNodes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                </td>\n            </tr>\n        </table>\n      </div>\n    </div>\n    <div id=\"serviceConfig\">\n        ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "controller.isNextDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <p class=\"step-description\">\n      ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step4.error.nameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </p>\n  ");
  return buffer;}

  data.buffer.push("\n<div id=\"ha-step4\" class=\"wizard-content col-md-9\">\n <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step4.bodyHeader";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  ");
  stack1 = depth0;
  stack2 = "controller.isNameNodeStarted";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "view.step4BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button  ");
  stack1 = {};
  stack2 = "controller.isNextEnabled::disabled :btn :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n    <span class=\"pull-right btn-extra-info\">");
  stack1 = depth0;
  stack2 = "view.nnCheckPointText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step5", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step6", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"ha-step6\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step6.bodyHeader";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "view.step6BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button ");
  stack1 = {};
  stack2 = "controller.isNextEnabled::disabled :btn :btn-success :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n    <span class=\"pull-right btn-extra-info\">");
  stack1 = depth0;
  stack2 = "view.jnCheckPointText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step7", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step8", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"ha-step8\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step8.bodyHeader";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "view.step8BodyText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n  <div>\n    ");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step8.proceed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n    <span class=\"pull-right btn-extra-info\">");
  data.buffer.push("</span>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/step9", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/nameNode/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">7</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep8:active view.isStep8Disabled:disabled view.isStep8Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep8";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">8</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep9:active view.isStep9Disabled:disabled view.isStep9Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep9";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">9</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step9.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/rangerAdmin/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"help-block validation-block col-sm-9 help-inline\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step1.invalid_url";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n            ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <form autocomplete=\"off\" class=\"form-horizontal\">\n        <div ");
  stack1 = {};
  stack2 = ":form-group showloadBalancerURLError:error showloadBalancerURLError:has-error :col-md-12";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step1.load_balancer_url";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</label>\n\n          <div class=\"row\">\n            <div class=\"col-md-3\">\n              ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "content.loadBalancerURL";
  stack3['valueBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            ");
  stack1 = depth0;
  stack2 = "showloadBalancerURLError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </form>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/rangerAdmin/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <tr>\n                  <td>\n                    <b>");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step3.additionalRA";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n                  </td>\n                  <td>");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                  <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n                </tr>\n              ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div id=\"serviceConfig\">\n            ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\"> ");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step3.alert_message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n        <div id=\"ha-step3-content\" class=\"well pre-scrollable\">\n          <div id=\"step8-info\">\n            <table id=\"ha-step3-review-table\">\n              <tr>\n                <td>\n                  <b>");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step3.currentRA";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n                </td>\n                <td>");
  stack1 = depth0;
  stack2 = "controller.content.raHosts.currentRA";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td></td>\n              </tr>\n              ");
  stack1 = depth0;
  stack2 = "controller.content.raHosts.additionalRA";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </table>\n          </div>\n        </div>\n        <div class=\"alert alert-info\">\n          ");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.configs_changes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/rangerAdmin/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/rangerAdmin/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div id=\"ra-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li class=\"nav-header\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.ra_highAvailability.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/resourceManager/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</a>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/resourceManager/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div id=\"serviceConfig\">\n          ");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.confirm.config.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.confirm.host.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"ha-step3-content\" class=\"well pre-scrollable\">\n        <div id=\"step8-info\">\n          <table id=\"ha-step3-review-table\">\n            <tr>\n                <td>\n                    <b>");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.currentRM";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n                </td>\n                <td>");
  stack1 = depth0;
  stack2 = "controller.content.rmHosts.currentRM";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td></td>\n            </tr>\n            <tr>\n                <td>\n                  <b>");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.additionalRM";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b>:\n                </td>\n                <td>");
  stack1 = depth0;
  stack2 = "controller.content.rmHosts.additionalRM";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n          </table>\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/resourceManager/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/highAvailability/resourceManager/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"rm-ha-wizard\" class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.rm_highAvailability.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "securityEnabled";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div>\n      <span class=\"text-success\">");
  stack1 = depth0;
  stack2 = "admin.security.enabled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  stack1 = depth0;
  stack2 = "CLUSTER.TOGGLE_KERBEROS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "showDownloadCsv";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <br/>\n      ");
  stack1 = depth0;
  stack2 = "CLUSTER.TOGGLE_KERBEROS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div id=\"serviceConfig\">\n      ");
  stack1 = depth0;
  stack2 = "App.ServicesConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div ");
  stack1 = {};
  stack2 = ":btn-area :pull-right controller.isEditMode::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <button id=\"cancel-kerberos-reconfigure-identities\" ");
  stack1 = depth0;
  stack2 = "makeConfigsUneditable";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              class=\"btn btn-default\">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button id=\"save-kerberos-reconfigure-identities\"\n              class=\"btn btn-success\" ");
  stack1 = {};
  stack2 = "isSaveButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.supports.enableToggleKerberos";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <button class=\"btn btn-padding btn-warning admin-disable-security-btn\" ");
  stack1 = {};
  stack2 = "controller.isKerberosButtonsDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "notifySecurityOffPopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.kerberos.button.disable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </button>\n          ");
  stack1 = depth0;
  stack2 = "controller.isManualKerberos";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <button class=\"btn btn-success\"\n                    id=\"regenerate-keytabs\" ");
  stack1 = {};
  stack2 = "controller.isKerberosButtonsDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "regenerateKeytabs";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"glyphicon glyphicon-repeat\"></i>&nbsp; ");
  stack1 = depth0;
  stack2 = "admin.kerberos.button.regenerateKeytabs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            ");
  stack1 = depth0;
  stack2 = "App.isCredentialStorePersistent";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "showManageKDCCredentialsPopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.kerberos.credentials.store.menu.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <button class=\"btn btn-success\"}} ");
  stack1 = depth0;
  stack2 = "downloadCSV";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"glyphicon glyphicon-download-alt\"></i>&nbsp; ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.downloadCSV";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </button>\n      ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.supports.enableToggleKerberos";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <button href=\"#\" ");
  stack1 = depth0;
  stack2 = "makeConfigsEditable";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = {};
  stack2 = "controller.isEditMode:hidden :btn :btn-default :pull-right";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n        ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div>\n      <p class=\"muted background-text\">");
  stack1 = depth0;
  stack2 = "admin.security.disabled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "CLUSTER.TOGGLE_KERBEROS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </p>\n    </div>\n  ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.supports.enableToggleKerberos";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <a class=\"btn btn-padding btn-success admin-enable-security-btn\" ");
  stack1 = depth0;
  stack2 = "checkAndStartKerberosWizard";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.kerberos.button.enable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </a>\n            <br/>\n          ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "dataIsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(17, program17, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/disable", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"row col-sm-12\">\n    ");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/notify_security_off_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"alert alert-warning\">\n  ");
  stack1 = depth0;
  stack2 = "admin.security.disable.popup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/regenerate_keytabs_popup_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<p class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.regenerate_keytabs.popup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n<p>\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.regenerateKeytabsOnlyForMissing";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.kerberos.regenerate_keytabs.checkbox.label";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</p>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/restart_services_after_regenerate_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<p class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.regenerate_keytabs.popup.restart.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n<p>\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.restartComponents";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.kerberos.regenerate_keytabs.checkbox.restart.label";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</p>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "option.displayName";
  stack3['name'] = stack4;
  stack4 = "kerberos-step1-option-radiobutton";
  stack3['labelClassNames'] = stack4;
  stack4 = "option.displayName";
  stack3['labelBinding'] = stack4;
  stack4 = "selectedItem";
  stack3['selectionBinding'] = stack4;
  stack4 = "option.value";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "condition.hidden";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "condition.checked";
  stack3['checkedBinding'] = stack4;
  stack4 = "kerberos-step1-prerequiste-checkboxes";
  stack3['labelClassNames'] = stack4;
  stack4 = "condition.displayText";
  stack3['labelBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"kerberos-wizard-step1\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.info.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  <div class=\"alert alert-warning\">\n    ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.alert.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"kerberos-step1-options-body\">\n        ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.body.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "options";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <br/>\n      </div>\n      <div class=\"alert alert-info\" id=\"kerberos-step1-prerequisites-body\">\n        <h5>");
  stack1 = depth0;
  stack2 = "selectedOption.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n        <b>");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.prerequisites.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b> <br/> <br/>\n        ");
  stack1 = depth0;
  stack2 = "selectedOption.preConditions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "condition";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "submit-kerberos-step1";
  stack3['buttonId'] = stack4;
  stack4 = "submit";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.ServicesConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"kerberos-wizard-step2\">\n  <div id=\"serviceConfig\" class=\"wizard-content col-md-9\">\n    <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <p class=\"step-description\">\n      ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step2.info.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </p>\n    <div class=\"panel panel-default\">\n      <div class=\"panel-body\">\n        ");
  stack1 = depth0;
  stack2 = "isConfigsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "App.WizardBackButton";
  stack3 = {};
  stack4 = "back-kerberos-step2";
  stack3['buttonId'] = stack4;
  stack4 = "back";
  stack3['action'] = stack4;
  stack4 = "controller.isBackBtnDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "wizard-back";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "submit-kerberos-step2";
  stack3['buttonId'] = stack4;
  stack4 = "submit";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "retry";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </a>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div class=\"panel panel-default\">\n        <div class=\"panel-body\">\n          <div class=\"clearfix\">\n            ");
  stack1 = depth0;
  stack2 = "controller.tasks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "task";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "showIgnore";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.isHostHeartbeatLost";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.taskView";
  stack3 = {};
  stack4 = "task";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <div class=\"item row\">\n                  <div ");
  stack1 = {};
  stack2 = "view.linkClass view.parentView.labelWidth";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <i ");
  stack1 = {};
  stack2 = "view.icon view.iconColor";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                    <a ");
  stack1 = depth0;
  stack2 = "task";
  stack3 = depth0;
  stack4 = "showHostProgressPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >");
  stack1 = depth0;
  stack2 = "task.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </div>\n                  <div ");
  stack1 = {};
  stack2 = "view.showProgressBar::hide :col-md-5";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <div ");
  stack1 = {};
  stack2 = ":progress-wrapper controller.isRollback::col-md-8 controller.isRollback:col-md-3";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      <div class=\"progress\">\n                        <div class=\"progress-bar-striped active progress-bar-info progress-bar\" ");
  stack1 = {};
  stack2 = "view.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      </div>\n                    </div>\n                    <div ");
  stack1 = {};
  stack2 = ":col-md-1 view.hidePercent:noDisplay";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "task.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&#37;</div>\n                  </div>\n                  <div>\n                    ");
  stack1 = depth0;
  stack2 = "task.showRetry";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "task.showRollback";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "task.showSkip";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                </div>\n              ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <a ");
  stack1 = depth0;
  stack2 = "retryTask";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\"\n                                                                  rel=\"tooltip\"\n                                                                  data-trigger=\"click\" ");
  stack1 = {};
  stack2 = "view.showDBTooltip:testDBRetryTooltip";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>&nbsp;\n                        ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      </a>\n\n                    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <a ");
  stack1 = depth0;
  stack2 = "rollback";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\">\n                        <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>\n                        ");
  stack1 = depth0;
  stack2 = "common.rollBack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      </a>\n                    ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <a ");
  stack1 = depth0;
  stack2 = "skipTask";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-primary retry\">\n                        <i class=\"glyphicon-step-forward glyphicon glyphicon-white\"></i>\n                        ");
  stack1 = depth0;
  stack2 = "common.skip";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      </a>\n                    ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div class=\"alert alert-warning\">\n              <label>");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "ignore";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.kerberos.wizard.step3.checkbox.ignoreAndProceed.label";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n            </div>\n          ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <p class=\"alert alert-danger\">");
  stack1 = depth0;
  stack2 = "view.resultMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              <a href=\"javascript:void(null)\"\n                 data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "showHostsWithLostHeartBeat";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </p>\n          ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.WizardBackButton";
  stack3 = {};
  stack4 = "back";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-back";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n<div id=\"kerberos-wizard-step3\">\n  <div id=\"common-progress-page\" class=\"wizard-content col-md-9\">\n    <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "view.headerTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <div ");
  stack1 = {};
  stack2 = "view.noticeClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.notice";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "controller.showRetry";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(16, program16, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "view.showBackButton";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "submit-kerberos-step3";
  stack3['buttonId'] = stack4;
  stack4 = "done";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"kerberos-wizard-step4\">\n  <div id=\"serviceConfig\" class=\"wizard-content col-md-9\">\n    <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step4.info.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    <div class=\"panel panel-default\">\n      <div class=\"panel-body\">\n        ");
  stack1 = depth0;
  stack2 = "App.ServicesConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  </div>\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "App.WizardBackButton";
  stack3 = {};
  stack4 = "back-kerberos-step4";
  stack3['buttonId'] = stack4;
  stack4 = "back";
  stack3['action'] = stack4;
  stack4 = "wizard-back";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "submit-kerberos-step4";
  stack3['buttonId'] = stack4;
  stack4 = "submit";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step5", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p class=\"step-description\">\n          ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.moreInfoNonManual.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </p>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p class=\"step-description\">\n          ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.moreInfoManual.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </p>\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "value";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <p><strong>");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>: ");
  stack1 = depth0;
  stack2 = "value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n            ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.downloadCSV";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

  data.buffer.push("\n<div id=\"kerberos-wizard-step5\">\n  <div id=\"confirm-configuration\" class=\"wizard-content col-md-9\">\n    <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <div>\n      <p class=\"step-description\">\n        ");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.info.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </p>\n      ");
  stack1 = depth0;
  stack2 = "App.router.kerberosWizardController.skipClientInstall";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.router.kerberosWizardController.skipClientInstall";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n\n    <div class=\"panel panel-default\">\n      <div class=\"panel-body\">\n        <div class=\"well pre-scrollable\">\n          ");
  stack1 = depth0;
  stack2 = "confirmProperties";
  stack3 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n\n        <div class=\"additional btn-area clearfix\">\n          ");
  stack1 = depth0;
  stack2 = "App.ButtonProgressView";
  stack3 = {};
  stack4 = "pull-right mls";
  stack3['classNames'] = stack4;
  stack4 = "btn-secondary";
  stack3['buttonClassNames'] = stack4;
  stack4 = "downloadCSV";
  stack3['action'] = stack4;
  stack4 = "controller.isCSVRequestInProgress";
  stack3['isInProgressBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          <button class=\"btn btn-default pull-right\" ");
  stack1 = depth0;
  stack2 = "exitWizard";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.exitWizard";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      ");
  stack1 = depth0;
  stack2 = "App.WizardBackButton";
  stack3 = {};
  stack4 = "back-kerberos-step5";
  stack3['buttonId'] = stack4;
  stack4 = "back";
  stack3['action'] = stack4;
  stack4 = "wizard-back";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.WizardNextButton";
  stack3 = {};
  stack4 = "submit-kerberos-step5";
  stack3['buttonId'] = stack4;
  stack4 = "submit";
  stack3['action'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "wizard-next";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step6", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"kerberos-wizard-step6\">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step7", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"kerberos-wizard-step7\">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/step8", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div id=\"kerberos-wizard-step8\">\n  ");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/kerberos/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">7</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep8:active view.isStep8Disabled:disabled view.isStep8Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep8";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">8</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/serviceAccounts", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.users";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div class=\"row\">\n        <div class=\"col-md-6\">\n          <table class=\"table table-hover\">\n            <thead>\n            <tr>\n              <th>");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n              <th>");
  stack1 = depth0;
  stack2 = "common.value";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            </tr>\n            </thead>\n            <tbody>\n            ");
  stack1 = depth0;
  stack2 = "controller.users";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "user";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tbody>\n          </table>\n        </div>\n      </div>\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "user.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <tr>\n                  <td>");
  stack1 = depth0;
  stack2 = "user.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                  <td>");
  stack1 = depth0;
  stack2 = "user.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                </tr>\n              ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <p>");
  stack1 = depth0;
  stack2 = "admin.misc.nothingToShow";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n<div class=\"admin-misc container-wrap-table\">\n  <div class=\"row\">\n    <h2 class=\"table-title col-sm-4\">");
  stack1 = depth0;
  stack2 = "admin.misc.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "controller.dataIsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/service_auto_start", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"cluster-level-switcher\">\n      <span class=\"h2\">");
  stack1 = depth0;
  stack2 = "admin.serviceAutoStart.general.switcher";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.initSwitcher";
  stack3['didInsertElementBinding'] = stack4;
  stack4 = "general-auto-start circle-toggle-switch";
  stack3['classNames'] = stack4;
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.isGeneralRecoveryEnabled";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"form-horizontal\">\n      <table class=\"table\">\n        <thead>\n          <tr>\n            <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th>");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th>\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "all-checkbox";
  stack3['classNames'] = stack4;
  stack4 = "admin.serviceAutoStart.column.autoStart";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.allComponentsChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </th>\n            <th></th>\n          </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "componentsConfigsGrouped";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n    </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <tr>\n            <td ");
  stack1 = {};
  stack2 = "component.isFirst::skip-border";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "component.isFirst";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n            <td>");
  stack1 = depth0;
  stack2 = "component.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td>");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "component.recoveryEnabled";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td></td>\n          </tr>\n        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "component.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"admin-auto-start\">\n  <p class=\"description\">");
  stack1 = depth0;
  stack2 = "admin.serviceAutoStart.header.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  ");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = "view.isDisablad:hidden :configs-save-panel";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"pull-right\">\n      <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "restoreCachedValues";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isSaveDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "showSavePopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isSaveDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    </div>\n    <div class=\"clearfix\"></div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_and_upgrade", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div id=\"stack-upgrade-page\">\n  ");
  stack1 = depth0;
  stack2 = "App.MainAdminStackMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/edit_repositories", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <tr ");
  stack1 = {};
  stack2 = "os.osType";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <td class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "os.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td class=\"col-md-10\" colspan=\"2\">\n          <table class=\"table table-condensed no-borders inner-table\">\n            <tbody>\n              ");
  stack1 = depth0;
  stack2 = "os.repositories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "repository";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tbody>\n          </table>\n        </td>\n      </tr>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "repository.showRepo";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <tr ");
  stack1 = {};
  stack2 = "repository.repoName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <td class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "repository.repoName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                    <td class=\"col-md-8\">\n                      <div ");
  stack1 = {};
  stack2 = "repository.hasError:error :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "App.BaseUrlTextField";
  stack3 = {};
  stack4 = "repository";
  stack3['repositoryBinding'] = stack4;
  stack4 = "view.content.useRedhatSatellite";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    </td>\n                  </tr>\n                ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <i class=\"glyphicon glyphicon-question-sign\" rel=\"skip-validation-tooltip\"\n       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step1.advancedRepo.skipValidation.tooltip";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n     </i>\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <i class=\"glyphicon glyphicon-question-sign\" rel=\"use-redhat-tooltip\"\n       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step1.advancedRepo.useRedhatSatellite.tooltip";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    </i>\n   ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"alert alert-info\">\n  ");
  stack1 = depth0;
  stack2 = "admin.stackVersions.editRepositories.info";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<div ");
  stack1 = {};
  stack2 = "view.parentView.serverValidationFailed::hidden :alert :alert-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "admin.stackVersions.editRepositories.validation.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<table class=\"table table-hover\">\n  <thead>\n    <tr>\n      <th class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "common.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th class=\"col-md-8\">");
  stack1 = depth0;
  stack2 = "admin.cluster.repositories.baseUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n  </thead>\n  <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.content.operatingSystems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "os";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>\n\n<div id=\"skip-validation\">\n  ");
  stack1 = depth0;
  stack2 = "view.skipCheckBox";
  stack3 = {};
  stack4 = "installer.step1.advancedRepo.skipValidation.message";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.parentView.skipValidation";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n<div id=\"use-redhat\">\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "align-checkbox";
  stack3['checkboxClassNames'] = stack4;
  stack4 = "installer.step1.advancedRepo.useRedhatSatellite.message";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.content.useRedhatSatellite";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/failed_hosts_modal", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div class=\"panel panel-default\">\n        <div class=\"panel-group\">\n          <div class=\"panel-heading\">\n            <i class=\"pull-left panel-toggle icon-caret-right\"></i>\n            <a class=\"panel-toggle\" data-toggle=\"collapse\" data-parent=\"#accordion2\" href=\"");
  stack1 = depth0;
  stack2 = "host.collapseHref";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\">\n              <span rel=\"UsageTooltip\" data-original-title=\"");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\">");
  stack1 = depth0;
  stack2 = "host.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              <span class=\"pull-right label label-info\">");
  stack1 = depth0;
  stack2 = "host.hostComponents.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </a>\n          </div>\n          <div id=\"");
  stack1 = depth0;
  stack2 = "host.collapseId";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" class=\"panel-body collapse\">\n            ");
  stack1 = depth0;
  stack2 = "host.hostComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "hostComponent";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"row\">\n                <div class=\"col-md-4\">");
  stack1 = depth0;
  stack2 = "hostComponent.serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                <div class=\"col-md-4\">");
  stack1 = depth0;
  stack2 = "hostComponent.componentName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n              </div>\n            ");
  return buffer;}

  data.buffer.push("\n\n\n<div id=\"upgrade-failed-hosts\">\n  <div class=\"row col-md-12\">\n    <div class=\"col-md-6\">");
  stack1 = depth0;
  stack2 = "view.subHeader";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"pull-right\">\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "openDetails";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.details";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </div>\n  </div>\n  <div class=\"col-md-12 host-list-container\">\n    ");
  stack1 = depth0;
  stack2 = "view.hosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  </div>\n\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/services", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <tr>\n        <td class=\"service-display-name\">");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td class=\"service-stack-version\">");
  stack1 = depth0;
  stack2 = "service.serviceVersionDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td class=\"stack-version-state\">\n          ");
  stack1 = depth0;
  stack2 = "service.isInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n        <td class=\"service-description\">");
  stack1 = depth0;
  stack2 = "service.comments";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</td>\n      </tr>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.installed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <a ");
  stack1 = {};
  stack2 = ":path-link view.isAddServiceAvailable::not-active-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "service.serviceName";
  stack3 = depth0;
  stack4 = "goToAddService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "services.service.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n\n    <div class=\"header\">\n      <strong>");
  stack1 = depth0;
  stack2 = "admin.cluster.repositories.repositories";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong>\n    </div>\n    <ul class=\"nav nav-tabs\">\n      <li class=\"active\">\n        <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.allRepositoriesGroups.stackVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </li>\n    </ul>\n\n    <div class=\"admin-cluster\">\n      <div class=\"repositories-table\">\n        <div class=\"thead\">\n          <div class=\"th os-th\">");
  stack1 = depth0;
  stack2 = "common.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"th name-th\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"th url-th\">");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.localRepo.column.baseUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"tbody\">\n          ");
  stack1 = depth0;
  stack2 = "view.allRepositoriesGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "repoGroup";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n  ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div class=\"trow\">\n              <div class=\"os-td\">\n                <label>\n                  <span class=\"os\">");
  stack1 = depth0;
  stack2 = "repoGroup.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                </label>\n              </div>\n              <div style=\"width:89%\">\n                ");
  stack1 = depth0;
  stack2 = "repoGroup.repositories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "repository";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            </div>\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div class=\"sub-trow\">\n                    <div class=\"name-td\">");
  stack1 = depth0;
  stack2 = "repository.repoId";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    <!--edit mode for current url-->\n                    ");
  stack1 = depth0;
  stack2 = "repository.onEdit";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(16, program16, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                      <div ");
  stack1 = {};
  stack2 = ":url-td repository.empty-error:textfield-error repository.invalid-error:textfield-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "repository.baseUrl";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </div>\n                      <div class=\"clear-td\">\n                        ");
  stack1 = depth0;
  stack2 = "repository.clearAll";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </div>\n                      <div class=\"edit-buttons-td\">\n                        <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "doCancel";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      </div>\n                      <div class=\"edit-buttons-td\">\n                        ");
  stack1 = depth0;
  stack2 = "repository.empty-error";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </div>\n                      <!--non-edit mode for current url-->\n                    ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                          <a ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "clearGroupLocalRepository";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                            <i class=\"glyphicon glyphicon-remove-sign\"></i>\n                          </a>\n                        ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          <a class=\"btn btn-primary\" disabled=\"disabled\">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                          <a class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "saveRepoUrls";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                        ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                      <div class=\"url-text-td\">\n                        ");
  stack1 = depth0;
  stack2 = "repository.baseUrl";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </div>\n                      <div class=\"edit-td\">\n                        <a ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "onEditClick";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                          <i class=\"glyphicon glyphicon-edit\"></i> ");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        </a>\n                      </div>\n                    ");
  return buffer;}

  data.buffer.push("\n\n\n<div class=\"container-wrap-table\">\n  <div class=\"row\">\n    <h2 class=\"table-title col-sm-1\">");
  stack1 = depth0;
  stack2 = "common.stack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n  </div>\n  <table class=\"table table-hover\">\n    <thead>\n    <tr>\n      <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n    </thead>\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n\n  ");
  stack1 = depth0;
  stack2 = "App.stackVersionsAvailable";
  stack3 = helpers.unless;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/stack_upgrade_wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "showPauseButton";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "isDowngrade";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <button\n                    class=\"btn btn-default pull-right\" ");
  stack1 = depth0;
  stack2 = "confirmPauseDowngrade";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.pauseDowngrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <button\n                    class=\"btn btn-default pull-right\" ");
  stack1 = depth0;
  stack2 = "confirmPauseUpgrade";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.pauseUpgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div class=\"panel-group\">\n            ");
  stack1 = depth0;
  stack2 = "view.runningItem";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.failedItem";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.plainManualItem";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isFinalizeItem";
  stack3 = helpers['if'];
  tmp1 = self.program(40, program40, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isSlaveComponentFailuresItem";
  stack3 = helpers['if'];
  tmp1 = self.program(56, program56, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isServiceCheckFailuresItem";
  stack3 = helpers['if'];
  tmp1 = self.program(73, program73, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.noActiveItem";
  stack3 = helpers['if'];
  tmp1 = self.program(85, program85, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.upgradeSuspended";
  stack3 = helpers['if'];
  tmp1 = self.program(87, program87, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.upgradeGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(96, program96, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"panel panel-default details-box row\">\n                <div class=\"panel-heading row\">\n                  <div class=\"pull-left col-md-6\">\n                      ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.inProgress";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "view.runningItem.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>\n                    ");
  stack1 = depth0;
  stack2 = "view.isDetailsOpened";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n                <div id=\"collapseBox\" class=\"panel-collapse collapse\">\n                  <div class=\"panel-body\">\n                      ");
  stack1 = depth0;
  stack2 = "view.runningItem.isTasksLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                </div>\n              </div>\n            ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <div class=\"pull-right col-md-6\">\n                        <a href=\"#collapseBox\" class=\"pull-right\" ");
  stack1 = depth0;
  stack2 = "toggleDetails";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                           data-toggle=\"collapse\">\n                            ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.details.hide";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        </a>\n                      </div>\n                    ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <div class=\"pull-right col-md-6\">\n                        <a href=\"#collapseBox\" class=\"pull-right\" ");
  stack1 = depth0;
  stack2 = "toggleDetails";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                           data-toggle=\"collapse\">\n                            ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.details.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        </a>\n                      </div>\n                    ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                        <div class=\"clear col-md-12\">\n                            ");
  stack1 = depth0;
  stack2 = "App.upgradeTaskView";
  stack3 = {};
  stack4 = "view.taskDetails";
  stack3['contentBinding'] = stack4;
  stack4 = "view";
  stack3['outsideViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        </div>\n                      ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "clear";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.isSlaveComponentFailuresItem";
  stack3 = helpers.unless;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div class=\"panel panel-default details-box row\">\n                    <div class=\"panel-heading row\">\n                      <div class=\"pull-left col-md-6\">\n                          ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.failed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&nbsp;");
  stack1 = depth0;
  stack2 = "view.failedItem.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </div>\n                        ");
  stack1 = depth0;
  stack2 = "view.isDetailsOpened";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                    <div id=\"collapseBox\" class=\"panel-collapse collapse\">\n                      <div class=\"panel-body\">\n                          ");
  stack1 = depth0;
  stack2 = "view.failedItem.isTasksLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(25, program25, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </div>\n                    </div>\n                      ");
  stack1 = depth0;
  stack2 = "view.isHoldingState";
  stack3 = helpers['if'];
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                          <div class=\"pull-right col-md-6\">\n                            <a href=\"#collapseBox\" class=\"pull-right\" ");
  stack1 = depth0;
  stack2 = "toggleDetails";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                               data-toggle=\"collapse\">\n                                ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.details.hide";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                            </a>\n                          </div>\n                        ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                          <div class=\"pull-right col-md-6\">\n                            <a href=\"#collapseBox\" class=\"pull-right\" ");
  stack1 = depth0;
  stack2 = "toggleDetails";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                               data-toggle=\"collapse\">\n                                ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.details.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                            </a>\n                          </div>\n                        ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                            <div class=\"clear col-md-12\">\n                                ");
  stack1 = depth0;
  stack2 = "App.upgradeTaskView";
  stack3 = {};
  stack4 = "view.taskDetails";
  stack3['contentBinding'] = stack4;
  stack4 = "view";
  stack3['outsideViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            </div>\n                          ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "clear";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                        <div class=\"button-row\">\n                            ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "view.canSkipFailedItem";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          <button\n                            class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.failedItem";
  stack3 = depth0;
  stack4 = "retry";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                        </div>\n                      ");
  return buffer;}
function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                              <button\n                                class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.failedItem";
  stack3 = depth0;
  stack4 = "confirmDowngrade";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                            ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                              <button\n                                class=\"btn btn-warning\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.failedItem";
  stack3 = depth0;
  stack4 = "continue";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.continue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                            ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <p class=\"manual-steps-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n            ");
  stack1 = depth0;
  stack2 = "view.manualItem.messages.length";
  stack3 = helpers['if'];
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(36, program36, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <label class=\"message\">\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isManualDone";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.stackUpgrade.dialog.manualDone";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </label>\n            <div class=\"button-row\">\n              ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(38, program38, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <button class=\"btn btn-success\" ");
  stack1 = {};
  stack2 = "view.isManualProceedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "complete";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "common.proceed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n            </div>\n          </div>\n        ");
  return buffer;}
function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.manualItem.messages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "message";
  stack7 = helpers.each;
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <p class=\"manual-steps-content\">");
  stack1 = depth0;
  stack2 = "message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n              ");
  return buffer;}

function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <p class=\"manual-steps-content\">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.skipped.failures";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n            ");
  return buffer;}

function program38(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <button class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "confirmDowngrade";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n              ");
  return buffer;}

function program40(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <p class=\"manual-steps-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n            ");
  stack1 = depth0;
  stack2 = "isDowngrade";
  stack3 = helpers['if'];
  tmp1 = self.program(41, program41, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(43, program43, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <p class=\"manual-steps-content\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.message.autoStart";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n\n            ");
  stack1 = depth0;
  stack2 = "areSkippedServiceChecksLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(48, program48, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(52, program52, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            <label class=\"message\">\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isManualDone";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.stackUpgrade.dialog.manualDone";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </label>\n            <div class=\"button-row\">\n              ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(54, program54, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "confirmPauseUpgrade";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.later";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n                <button class=\"btn btn-success\" ");
  stack1 = {};
  stack2 = "view.isManualProceedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "complete";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "common.finalize";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </button>\n            </div>\n          </div>\n        ");
  return buffer;}
function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <p class=\"manual-steps-content\">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.message.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n            ");
  return buffer;}

function program43(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.isRevertibleUpgrade";
  stack3 = helpers['if'];
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(46, program46, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <p class=\"manual-steps-content\">");
  stack1 = depth0;
  stack2 = "view.revertibleFinalizeMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n                ");
  return buffer;}

function program46(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <p class=\"manual-steps-content\">");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.message.upgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n                ");
  return buffer;}

function program48(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "skippedServiceChecks.length";
  stack3 = helpers['if'];
  tmp1 = self.program(49, program49, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program49(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <div>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.message.skippedServiceChecks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                <ul>\n                  ");
  stack1 = depth0;
  stack2 = "skippedServiceChecks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "serviceName";
  stack7 = helpers.each;
  tmp1 = self.program(50, program50, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n                <div>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.finalize.message.testServices";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n              ");
  return buffer;}
function program50(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <li>");
  stack1 = depth0;
  stack2 = "serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                  ");
  return buffer;}

function program52(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program54(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <button class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "confirmDowngrade";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n              ");
  return buffer;}

function program56(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <p class=\"manual-steps-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n            <div class=\"slave-failures-info\">\n            <p>\n              ");
  stack1 = depth0;
  stack2 = "areSlaveComponentFailuresHostsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(57, program57, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(59, program59, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </p>\n                <p>\n                  ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                <ul>\n                  ");
  stack1 = depth0;
  stack2 = "view.isOutOfSync";
  stack3 = helpers['if'];
  tmp1 = self.program(61, program61, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options.first";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                  ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(63, program63, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n                </p>\n            </div>\n            ");
  stack1 = depth0;
  stack2 = "view.isHoldingState";
  stack3 = helpers.unless;
  tmp1 = self.program(65, program65, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <div class=\"button-row\">\n              ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(67, program67, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isHoldingState";
  stack3 = helpers['if'];
  tmp1 = self.program(69, program69, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(71, program71, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        ");
  return buffer;}
function program57(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "showFailedHosts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.failedHostsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              ");
  return buffer;}

function program59(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program61(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.isOutOfSync.options.first";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.isOutOfSync.options.second";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                  ");
  return buffer;}

function program63(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options.second";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                  ");
  return buffer;}

function program65(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <label class=\"message\">\n                ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isManualDone";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.stackUpgrade.dialog.manualDone";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </label>\n            ");
  return buffer;}

function program67(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <button class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "confirmDowngrade";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n              ");
  return buffer;}

function program69(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.failedItem";
  stack3 = depth0;
  stack4 = "retry";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n              ");
  return buffer;}

function program71(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <button class=\"btn btn-success\" ");
  stack1 = {};
  stack2 = "view.isManualProceedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "complete";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "common.proceed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n              ");
  return buffer;}

function program73(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <p class=\"manual-steps-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n\n            ");
  stack1 = depth0;
  stack2 = "areServiceCheckFailuresServicenamesLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(74, program74, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(81, program81, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            <label class=\"message\">\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.isManualDone";
  stack3['checkedBinding'] = stack4;
  stack4 = "admin.stackUpgrade.dialog.manualDone";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </label>\n            <div class=\"button-row\">\n              ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(83, program83, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <button class=\"btn btn-success\" ");
  stack1 = {};
  stack2 = "view.isManualProceedDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "complete";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "common.proceed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n            </div>\n          </div>\n        ");
  return buffer;}
function program74(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "serviceCheckFailuresServicenames.length";
  stack3 = helpers['if'];
  tmp1 = self.program(75, program75, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "slaveComponentStructuredInfo.hosts.length";
  stack3 = helpers['if'];
  tmp1 = self.program(78, program78, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program75(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <div class=\"service-check-failures-info\">\n                  <p class=\"service-check-failures-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual.serviceCheckFailures.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n                  <div>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual.serviceCheckFailures.msg1";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                  <ul class=\"failed-info-list\">\n                    ");
  stack1 = depth0;
  stack2 = "serviceCheckFailuresServicenames";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "serviceName";
  stack7 = helpers.each;
  tmp1 = self.program(76, program76, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                  <div>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual.serviceCheckFailures.msg2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                </div>\n              ");
  return buffer;}
function program76(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li>");
  stack1 = depth0;
  stack2 = "serviceName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    ");
  return buffer;}

function program78(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <div class=\"slave-failures-info\">\n                  <p class=\"slave-failures-title\"><strong>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.manual.slaveComponentFailures.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n                  <p>\n                    ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "showFailedHosts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.failedHostsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </p>\n                  <p>\n                    ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  <ul>\n                    <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options.first";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options.third";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    ");
  stack1 = depth0;
  stack2 = "view.isDowngradeAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(79, program79, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                  </p>\n                </div>\n              ");
  return buffer;}
function program79(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li>");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.failedHosts.options.second";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n                    ");
  return buffer;}

function program81(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program83(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <button class=\"btn btn-danger\" ");
  stack1 = {};
  stack2 = "controller.requestInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.manualItem";
  stack3 = depth0;
  stack4 = "confirmDowngrade";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n              ");
  return buffer;}

function program85(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <div>\n              ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.notActive";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          </div>\n        ");
  return buffer;}

function program87(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"panel panel-default details-box\">\n            <label class=\"message\">\n            ");
  stack1 = depth0;
  stack2 = "isDowngrade";
  stack3 = helpers['if'];
  tmp1 = self.program(88, program88, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(90, program90, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </label>\n            <div class=\"button-row\">\n              ");
  stack1 = depth0;
  stack2 = "isDowngrade";
  stack3 = helpers['if'];
  tmp1 = self.program(92, program92, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(94, program94, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        ");
  return buffer;}
function program88(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.suspended.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program90(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.suspended";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program92(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "resumeUpgrade";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.resume.downgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </button>\n              ");
  return buffer;}

function program94(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "resumeUpgrade";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "admin.stackUpgrade.dialog.resume";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </button>\n              ");
  return buffer;}

function program96(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "group.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(97, program97, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program97(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.upgradeGroupView";
  stack3 = {};
  stack4 = "group";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program99(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"stack-upgrade-dialog\">\n\n  <div ");
  stack1 = {};
  stack2 = "view.isLoaded::hidden :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"col-md-2 task-list-main-wrap\">");
  stack1 = depth0;
  stack2 = "controller.requestStatus";
  foundHelper = helpers.statusIcon;
  stack3 = foundHelper || depth0.statusIcon;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "statusIcon", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      &nbsp;");
  stack1 = depth0;
  stack2 = "view.upgradeStatusLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-7\">\n      ");
  stack1 = depth0;
  stack2 = "App.ProgressBarView";
  stack3 = {};
  stack4 = "view.overallProgress";
  stack3['progressBinding'] = stack4;
  stack4 = "controller.requestStatus";
  stack3['statusBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"col-md-1\">\n      ");
  stack1 = depth0;
  stack2 = "view.overallProgress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%\n    </div>\n    <div class=\"col-md-2 pause-button pull-right\">\n        ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n\n  <div class=\"task-list scrollable-block task-list-main-wrap upgrade-groups\">\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers.unless;
  tmp1 = self.program(99, program99, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_configs_merge_table", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td>");
  stack1 = depth0;
  stack2 = "type";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "currentValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td ");
  stack1 = {};
  stack2 = "willBeRemoved:no-value";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "resultingValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      </tr>\n    ");
  return buffer;}

  data.buffer.push("\n\n<h4 class=\"configurations-changes-header\">");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n<a ");
  stack1 = depth0;
  stack2 = "view.configs";
  stack3 = depth0;
  stack4 = "openConfigsInNewWindow";
  stack5 = {};
  stack6 = "App.router.mainAdminStackAndUpgradeController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right open-in-new-window\" href=\"#\">\n  <i class=\"icon-external-link\"></i> <span>");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n</a>\n<div class=\"alert alert-warning\">\n  ");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.alert";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<table class=\"configs-table table table-hover\">\n  <thead>\n    <tr>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.configType";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.propertyName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.currentValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.resultingValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n  </thead>\n</table>\n<div class=\"limited-height-2\">\n  <table class=\"configs-table table table-hover\">\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.configs";
  stack3 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_configs_recommend_table", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td>");
  stack1 = depth0;
  stack2 = "type";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>");
  stack1 = depth0;
  stack2 = "currentValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td ");
  stack1 = {};
  stack2 = "willBeRemoved:no-value";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "recommendedValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      </tr>\n    ");
  return buffer;}

  data.buffer.push("\n\n<h4 class=\"configurations-changes-header\">");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsRecommend.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n<a ");
  stack1 = depth0;
  stack2 = "view.configs";
  stack3 = depth0;
  stack4 = "openConfigsInNewWindow";
  stack5 = {};
  stack6 = "App.router.mainAdminStackAndUpgradeController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right open-in-new-window\" href=\"#\">\n  <i class=\"icon-external-link\"></i> <span id=\"i18n-34\">");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n</a>\n<div class=\"alert alert-warning\">\n  ");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsRecommend.alert";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<div class=\"configs-table-header\">\n  <table class=\"configs-table table table-striped\">\n    <thead>\n    <tr>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.configType";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.propertyName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.currentValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th>");
  stack1 = depth0;
  stack2 = "popup.clusterCheck.Upgrade.configsMerge.recommendedValue";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n    </thead>\n  </table>\n</div>\n<div class=\"limited-height-2\">\n  <table class=\"configs-table table table-striped\">\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.configs";
  stack3 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_group", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"col-md-4 pull-right\">\n      ");
  stack1 = depth0;
  stack2 = "view.progressBarView";
  stack3 = {};
  stack4 = "view.content.progress";
  stack3['progressBinding'] = stack4;
  stack4 = "view.content.status";
  stack3['statusBinding'] = stack4;
  stack4 = "view.content.total_task_count";
  stack3['totalTasksBinding'] = stack4;
  stack4 = "view.content.completed_task_count";
  stack3['completedTasksBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"percent pull-right\">\n        <span>");
  stack1 = depth0;
  stack2 = "view.content.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</span>\n      </div>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "item.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n      <div class=\"row\">\n        <div ");
  stack1 = {};
  stack2 = "item.isActive::not-active-link :col-md-8 :upgrade-item-text";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "item.upgradeItemStatus";
  foundHelper = helpers.statusIcon;
  stack3 = foundHelper || depth0.statusIcon;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "statusIcon", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.content.upgradeItems";
  stack3 = depth0;
  stack4 = "item";
  stack5 = depth0;
  stack6 = "toggleExpanded";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.context";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "item.isRunning";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      ");
  stack1 = depth0;
  stack2 = "item.isExpanded";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"col-md-4 pull-right\">\n            ");
  stack1 = depth0;
  stack2 = "App.ProgressBarView";
  stack3 = {};
  stack4 = "item.progress";
  stack3['progressBinding'] = stack4;
  stack4 = "item.status";
  stack3['statusBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            <div class=\"percent pull-right\">\n                <span>");
  stack1 = depth0;
  stack2 = "item.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</span>\n            </div>\n          </div>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "item.isTasksLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  data.buffer.push("\n          <div class=\"task-list\">\n            ");
  stack1 = depth0;
  stack2 = "item.tasks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "task";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.upgradeTaskView";
  stack3 = {};
  stack4 = "task";
  stack3['contentBinding'] = stack4;
  stack4 = "item.tasks";
  stack3['tasksBinding'] = stack4;
  stack4 = "row";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n\n<div class=\"row\">\n  <div ");
  stack1 = {};
  stack2 = "view.content.isExpandableGroup::not-active-link :col-md-8 :upgrade-group-text";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.content.upgradeGroupStatus";
  foundHelper = helpers.statusIcon;
  stack3 = foundHelper || depth0.statusIcon;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "statusIcon", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.upgradeData.upgradeGroups";
  stack3 = depth0;
  stack4 = "view.content";
  stack5 = depth0;
  stack6 = "toggleExpanded";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.content.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.content.isRunning";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div ");
  stack1 = {};
  stack2 = "view.content.isExpanded::hidden :task-list";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n\n  ");
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.upgradeItems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_history", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li>\n                  <a ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                    ");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <tr>\n              <td class=\"name accordion-group\">\n                <span class=\"trim_hostname accordion-heading\">\n                  <i class=\"icon-caret-right accordion-toggle\"\n                     data-toggle=\"collapse\"\n                     data-parent=\"#versionsAccordion\"\n                     ");
  stack1 = {};
  stack2 = "item.idHref";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                  <a href=\"#\" class=\"black\"\n                     ");
  stack1 = depth0;
  stack2 = "item.stackUpgradeHistoryItem";
  stack3 = depth0;
  stack4 = "showUpgradeHistoryRecord";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "item.directionLabel";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </span>\n                <div class=\"accordion-body collapse\" ");
  stack1 = {};
  stack2 = "item.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <div class=\"accordion-inner\">\n                      ");
  stack1 = depth0;
  stack2 = "item.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                </div>\n              </td>\n              <td>\n                <span>");
  stack1 = depth0;
  stack2 = "item.upgradeTypeLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td class=\"widest-column\">\n                <span ");
  stack1 = {};
  stack2 = "item.repositoryName";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.repositoryName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td class=\"wide-column\">\n                <span>");
  stack1 = depth0;
  stack2 = "item.repositoryType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td>\n                <span>");
  stack1 = depth0;
  stack2 = "item.startTimeLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td>\n                <span>");
  stack1 = depth0;
  stack2 = "item.duration";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td>\n                <span>");
  stack1 = depth0;
  stack2 = "item.endTimeLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n              <td>\n                <span>");
  stack1 = depth0;
  stack2 = "item.displayStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </td>\n            </tr>\n          ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <div class=\"row service-version-info\">\n                          <div class=\"col-xs-3 service-name\">\n                            ");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          </div>\n                          <div class=\"col-xs-3\">\n                            <span class=\"label label-from available\">\n                              ");
  stack1 = depth0;
  stack2 = "service.fromVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            </span>\n                          </div>\n                          <div class=\"col-xs-3 align-center\">\n                            <i class=\"icon-arrow-right\"></i>\n                          </div>\n                          <div class=\"col-xs-3\">\n                            <span class=\"label label-to available\">\n                              ");
  stack1 = depth0;
  stack2 = "service.toVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            </span>\n                          </div>\n                        </div>\n                      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"upgrade-history-section\">\n  <div ");
  stack1 = {};
  stack2 = "view.isReady:hide:show :screensaver :no-borders ";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <div ");
  stack1 = {};
  stack2 = "view.isReady::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"row\">\n      <h2 class=\"table-title col-sm-3\">");
  stack1 = depth0;
  stack2 = "common.upgrade.history";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n      <div class=\"table-controls row col-sm-9 pull-right\">\n        <div class=\"col-sm-12\">\n          <div class=\"btn-group pull-right\">\n            <a class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n              <span class=\"filters-label\">");
  stack1 = depth0;
  stack2 = "common.filter";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": </span>\n              <span>\n                ");
  stack1 = depth0;
  stack2 = "view.selectedCategory.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                <span class=\"caret\"></span>\n              </span>\n            </a>\n            <ul class=\"dropdown-menu\">\n              ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        </div>\n      </div>\n    </div>\n    <table class=\"table advanced-header-table table-hover\" id=\"upgrade-summary-table\">\n      <thead>\n      <tr>\n        <th>");
  stack1 = depth0;
  stack2 = "common.direction";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"widest-column\">");
  stack1 = depth0;
  stack2 = "common.repository";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th class=\"wide-column\">");
  stack1 = depth0;
  stack2 = "common.repositoryType";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.start.time";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.duration";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.end.time";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      </tr>\n      </thead>\n      <tbody class=\"accordion\" id=\"versionsAccordion\">\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n      <tfoot>\n        <tr>\n          <td colspan=\"8\">\n            ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n        </tfoot>\n    </table>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_history_details", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "controller.upgradeData.upgradeGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "group.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.upgradeGroupView";
  stack3 = {};
  stack4 = "group";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"stack-upgrade-record-dialog\">\n  <div ");
  stack1 = {};
  stack2 = "view.isLoaded::hidden :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"col-md-3 task-list-main-wrap\">");
  stack1 = depth0;
  stack2 = "controller.requestStatus";
  foundHelper = helpers.statusIcon;
  stack3 = foundHelper || depth0.statusIcon;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "statusIcon", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      &nbsp;");
  stack1 = depth0;
  stack2 = "view.upgradeStatusLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-8\">\n      ");
  stack1 = depth0;
  stack2 = "App.ProgressBarView";
  stack3 = {};
  stack4 = "view.overallProgress";
  stack3['progressBinding'] = stack4;
  stack4 = "controller.requestStatus";
  stack3['statusBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"col-md-1\">\n      ");
  stack1 = depth0;
  stack2 = "view.overallProgress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%\n    </div>\n  </div>\n\n  <div class=\"task-list scrollable-block task-list-main-wrap\">\n    ");
  stack1 = depth0;
  stack2 = "view.isReady";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isReady";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_options", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.controller.getSupportedUpgradeError";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <i class=\"glyphicon glyphicon-remove\"/>");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.error";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <pre>");
  stack1 = depth0;
  stack2 = "view.parentView.controller.getSupportedUpgradeError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n    ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div id=\"upgrade-options-popup-content\">\n        <div class=\"text\">");
  stack1 = depth0;
  stack2 = "view.versionText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n        ");
  stack1 = depth0;
  stack2 = "view.upgradeShow";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":row :method-options view.isInUpgradeWizard:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.upgradeMethods";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "method";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "view.upgradeShow";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.alertsWarning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"text method-text\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.method";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "method.cantBeStarted";
  stack3 = helpers.unless;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <div class=\"method-option col-md-6\">\n                    <div ");
  stack1 = {};
  stack2 = "method.allowed::not-allowed method.allowed::not-allowed-by-version\n                            method.isPrecheckFailed:not-allowed method.isPrecheckFailed:check-failed\n                            method.selected:selected method.type :img-thumbnail\n                            view.upgradeShow::default-cursor";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  stack1 = depth0;
  stack2 = "method";
  stack3 = depth0;
  stack4 = "selectMethod";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      <div ");
  stack1 = {};
  stack2 = "method.icon :method-icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      <div class=\"method-name\">");
  stack1 = depth0;
  stack2 = "method.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                      <div class=\"method-description\">");
  stack1 = depth0;
  stack2 = "method.description";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n                        ");
  stack1 = depth0;
  stack2 = "view.showPreUpgradeChecks";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                  </div>\n                ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "method.isCheckRequestInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                              <div class=\"method-precheck-message checking\">\n                                  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "admin.stackVersions.version.upgrade.upgradeOptions.preCheck.msg.checking";
  stack3['message'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              </div>\n                            ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                              <div ");
  stack1 = {};
  stack2 = ":method-precheck-message method.precheckResultsMessageClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                <i ");
  stack1 = {};
  stack2 = "method.precheckResultsMessageIconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                                <b>");
  stack1 = depth0;
  stack2 = "method.precheckResultsTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b>&nbsp;\n                                <a ");
  stack1 = depth0;
  stack2 = "method";
  stack3 = depth0;
  stack4 = "runAction";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                    ");
  stack1 = depth0;
  stack2 = "method.precheckResultsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                </a>\n                                  ");
  stack1 = depth0;
  stack2 = "method.bypassedFailures";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                              </div>\n                            ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                    <div\n                                      class=\"alert-danger\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.errors_bypassed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                                  ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"text tolerance-text\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.tolerance";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <i class=\"glyphicon glyphicon-question-sign failure-tolerance-tooltip\" data-toggle=\"tooltip\"></i>\n            <div>\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "tolerance-option";
  stack3['labelClassNames'] = stack4;
  stack4 = "admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option2";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.parentView.skipSCFailures";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <div>\n              ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "tolerance-option";
  stack3['labelClassNames'] = stack4;
  stack4 = "admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option1";
  stack3['labelTranslate'] = stack4;
  stack4 = "view.parentView.skipComponentFailures";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          </div>\n        ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"row\">\n    <div class=\"col-md-1 col-md-offset-3\">");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-8 spinner-text\">\n      ");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.loading";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n      <div class=\"text tolerance-text\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.bodyMsg.tolerance";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <i class=\"icon-question-sign failure-tolerance-tooltip\" data-toggle=\"tooltip\"></i>\n        <label class=\"tolerance-option\">");
  stack1 = depth0;
  stack2 = "Ember.Checkbox";
  stack3 = {};
  stack4 = "view.parentView.skipSCFailures";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n        <label class=\"tolerance-option\">");
  stack1 = depth0;
  stack2 = "Ember.Checkbox";
  stack3 = {};
  stack4 = "view.parentView.skipComponentFailures";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n           ");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.upgradeOptions.tolerance.option1";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n    </div>\n    <div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.upgrade.alertsWarning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.parentView.controller.isUpgradeTypesLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(17, program17, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_task", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = "view.content.isActive::not-active-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.content.status";
  foundHelper = helpers.statusIcon;
  stack3 = foundHelper || depth0.statusIcon;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "statusIcon", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleExpanded";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.content.command_detail";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </a>\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isContentLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div class=\"task-details task-detail-info\">\n      <ul class=\"nav nav-tabs\">\n        <li class=\"active\"><a data-toggle=\"tab\" ");
  stack1 = {};
  stack2 = "view.logTabIdLink";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.stdout";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n        <li><a ");
  stack1 = {};
  stack2 = "view.errorTabIdLInk";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"tab\">");
  stack1 = depth0;
  stack2 = "common.stderr";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n      </ul>\n      <div class=\"tab-content\">\n        <div class=\"tab-pane active\" ");
  stack1 = {};
  stack2 = "view.logTabId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <p>");
  stack1 = depth0;
  stack2 = "view.content.host_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n          <div class=\"row\">\n            <p class=\"pull-left col-sm-6\">");
  stack1 = depth0;
  stack2 = "view.content.output_log";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n            <div class=\"manage-controls pull-right col-sm-6\">\n              <a ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "openOutLog";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-open-dialog pull-right\">\n                <i class=\"icon-external-link\"></i> ");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n              <a ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "copyOutLog";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-copy pull-right\">\n                <i class=\"glyphicon glyphicon-copy\"></i> ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </div>\n          </div>\n          <pre ");
  stack1 = {};
  stack2 = "view.outputLogOpened:hidden :stdout";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.content.stdout";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "view.content.stdout";
  stack3['valueBinding'] = stack4;
  stack4 = "view.outputLogOpened::hidden";
  stack3['classBinding'] = stack4;
  stack4 = "readonly";
  stack3['readonly'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"tab-pane\" ");
  stack1 = {};
  stack2 = "view.errorTabId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <p>");
  stack1 = depth0;
  stack2 = "view.content.host_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n          <div class=\"row\">\n            <p class=\"pull-left col-sm-6\">");
  stack1 = depth0;
  stack2 = "view.content.error_log";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n            <div class=\"manage-controls pull-right\">\n              <a ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "openErrorLog";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-open-dialog pull-right\">\n                <i class=\"icon-external-link\"></i> ");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n              <a ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "copyErrLog";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-copy pull-right\">\n                <i class=\"glyphicon glyphicon-copy\"></i> ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </div>\n          </div>\n          <pre ");
  stack1 = {};
  stack2 = "view.errorLogOpened:hidden :stderr";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.content.stderr";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "view.content.stderr";
  stack3['valueBinding'] = stack4;
  stack4 = "view.errorLogOpened::hidden";
  stack3['classBinding'] = stack4;
  stack4 = "readonly";
  stack3['readonly'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n    </div>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.outsideView";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.isExpanded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_version_box", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  
  data.buffer.push("\n  <i class=\"glyphicon glyphicon-exclamation-sign out-of-sync-badge\"></i>\n");}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <i class=\"icon-wrench\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.maint";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <i class=\"icon-bug\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.patch";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <i class=\"icon-umbrella\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <button class=\"btn btn-primary\"\n      ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"btn-group\">\n      <button class=\"btn btn-primary\"\n        ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n      <button class=\"btn btn-primary dropdown-toggle\" data-toggle=\"dropdown\" ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <span class=\"caret\"></span>\n      </button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "view.stateElement.buttons";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "button";
  stack7 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li ");
  stack1 = {};
  stack2 = "button.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a ");
  stack1 = depth0;
  stack2 = "button.action";
  stack3 = depth0;
  stack4 = "runAction";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "button.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.iconClass";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <span ");
  stack1 = {};
  stack2 = "view.stateElement.class";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n      <i ");
  stack1 = {};
  stack2 = "view.stateElement.iconClass";
  stack1['class'] = stack2;
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n    ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.iconClass";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.isInstalling";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n      <i ");
  stack1 = {};
  stack2 = "view.stateElement.iconClass";
  stack1['class'] = stack2;
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n    ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.ProgressBarView";
  stack3 = {};
  stack4 = "view.installProgress";
  stack3['progressBinding'] = stack4;
  stack4 = "view.PROGRESS_STATUS";
  stack3['statusBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isOutOfSync";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<p ");
  stack1 = {};
  stack2 = ":version view.isRepoUrlsEditDisabled:not-active-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <span>");
  stack1 = depth0;
  stack2 = "view.content.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  <a class=\"pull-right not-active link-tooltip\" ");
  stack1 = depth0;
  stack2 = "editRepositories";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <i class=\"glyphicon glyphicon-edit\"></i>\n  </a>\n</p>\n<p class=\"repository-name\">(");
  stack1 = depth0;
  stack2 = "view.content.repositoryVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</p>\n\n<p class=\"patch-icon\">\n  ");
  stack1 = depth0;
  stack2 = "view.content.isMaint";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isPatch";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isService";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</p>\n\n\n<div ");
  stack1 = {};
  stack2 = "view.stateElement.isInstalling:installing :align-center :state :repo-version-tooltip";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isButton";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isButtonGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isLink";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isSpinner";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div class=\"hosts-section\">\n  <div class=\"hosts-bar\">");
  stack1 = depth0;
  stack2 = "common.hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n  <div class=\"row host-link\">\n    <div class=\"col-md-4 align-center not-installed-hosts\">\n      <div><a href=\"#\" class=\"hosts-tooltip not-active\" ");
  stack1 = {};
  stack2 = "view.content.noInitHostsTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.versionStateMap.not_installed";
  stack3 = depth0;
  stack4 = "showHosts";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.notInstalledHosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></div>\n      <div>");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.notInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    <div class=\"col-md-4 align-center installed-hosts\">\n      <div><a href=\"#\" class=\"hosts-tooltip not-active\" ");
  stack1 = {};
  stack2 = "view.content.noInstalledHostsTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.versionStateMap.installed";
  stack3 = depth0;
  stack4 = "showHosts";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.installedHosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></div>\n      <div>");
  stack1 = depth0;
  stack2 = "common.installed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    <div class=\"col-md-4 align-center current-hosts\">\n      <div><a href=\"#\" class=\"hosts-tooltip not-active\" ");
  stack1 = {};
  stack2 = "view.content.noCurrentHostsTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.versionStateMap.current";
  stack3 = depth0;
  stack4 = "showHosts";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.currentHosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></div>\n      <div>");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/upgrade_version_column", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  
  data.buffer.push("\n  <i class=\"glyphicon glyphicon-exclamation-sign out-of-sync-badge\"></i>\n");}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <span>");
  stack1 = depth0;
  stack2 = "view.content.displayNameSimple";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <span>");
  stack1 = depth0;
  stack2 = "view.content.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <p class=\"version-column-display-type\">\n    ");
  stack1 = depth0;
  stack2 = "view.content.isMaint";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.content.isPatch";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.content.isService";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </p>\n");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <i class=\"icon-wrench\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.maint";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <i class=\"icon-bug\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.patch";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <i class=\"icon-umbrella\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <button class=\"btn btn-primary\"\n      ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"btn-group\">\n      <button class=\"btn btn-primary\"\n        ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n      <button class=\"btn btn-primary dropdown-toggle\" data-toggle=\"dropdown\" ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <span class=\"caret\"></span>\n      </button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "view.stateElement.buttons";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "button";
  stack7 = helpers.each;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li ");
  stack1 = {};
  stack2 = "button.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a ");
  stack1 = depth0;
  stack2 = "button.action";
  stack3 = depth0;
  stack4 = "runAction";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "button.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </li>\n        ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.iconClass";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <span ");
  stack1 = {};
  stack2 = "view.stateElement.class";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n      <i ");
  stack1 = {};
  stack2 = "view.stateElement.iconClass";
  stack1['class'] = stack2;
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n    ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.iconClass";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "runAction";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = {};
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.stateElement.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  stack1 = depth0;
  stack2 = "view.stateElement.isInstalling";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program23(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n      <i ");
  stack1 = {};
  stack2 = "view.stateElement.iconClass";
  stack1['class'] = stack2;
  stack2 = "view.stateElement.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n    ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.ProgressBarView";
  stack3 = {};
  stack4 = "view.installProgress";
  stack3['progressBinding'] = stack4;
  stack4 = "view.PROGRESS_STATUS";
  stack3['statusBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <p class=\"service-version-info\">\n       <span ");
  stack1 = {};
  stack2 = "service.isVersionInvisible:invisible service.isAvailable:available :label";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n           ");
  stack1 = depth0;
  stack2 = "service.latestVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n       </span>\n        ");
  stack1 = depth0;
  stack2 = "service.notUpgradable";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </p>\n  ");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n          <i class=\"icon-info-sign not-upgradable\" ");
  stack1 = {};
  stack2 = "service.notUpgradableTitle";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isOutOfSync";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<p class=\"version-column-display-name\">\n  ");
  stack1 = depth0;
  stack2 = "view.content.isStandard";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</p>\n\n<p class=\"version-column-repository-name\">(");
  stack1 = depth0;
  stack2 = "view.content.repositoryVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</p>\n\n");
  stack1 = depth0;
  stack2 = "view.displayVersionTypeBlock";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<p class=\"version-column-show-details\">\n  <a ");
  stack1 = depth0;
  stack2 = "openVersionBoxPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.stackVersions.version.column.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n</p>\n<div ");
  stack1 = {};
  stack2 = "view.stateElement.isInstalling:installing :align-center :state :repo-version-tooltip";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isButton";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isButtonGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isLink";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.stateElement.isSpinner";
  stack3 = helpers['if'];
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div class=\"services-section\">\n  ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/admin/stack_upgrade/versions", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-warning\" role=\"alert\">\n    <div class=\"stack-error-row\">\n      <div class=\"image-container\">\n        <i class=\"glyphicon glyphicon-warning-sign\"></i>\n      </div>\n      <div class=\"message-container\">\n        <h4 class=\"display-inline\">");
  stack1 = depth0;
  stack2 = "view.stackVersionError.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n        <span class=\"stack-name\">");
  stack1 = depth0;
  stack2 = "view.stackVersionError.stack";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <div>\n          ");
  stack1 = depth0;
  stack2 = "view.stackVersionError.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "AMBARI.MANAGE_STACK_VERSIONS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <div class=\"btn-container\">\n          <button class=\"btn btn-warning\" ");
  stack1 = depth0;
  stack2 = "view.stackVersionError";
  stack3 = depth0;
  stack4 = "reinstallOutOfSyncComponents";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "common.reinstall";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n          <button class=\"btn btn-warning\" ");
  stack1 = depth0;
  stack2 = "view.stackVersionError";
  stack3 = depth0;
  stack4 = "removeOutOfSyncComponents";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "common.remove";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </button>\n        </div>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "goToVersions";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "  ");
  stack1 = {};
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " id=\"manage-versions-link\">\n        <i class=\"icon-external-link\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.stackVersions.manageVersions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </button>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <li>\n          <a ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n            ");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tr>\n            <td class=\"service-display-name\">");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </tr>\n        ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <div class=\"versions-slides-bar\">\n        ");
  stack1 = depth0;
  stack2 = "view.repoVersions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "version";
  stack7 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "version.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.UpgradeVersionColumnView";
  stack3 = {};
  stack4 = "version";
  stack3['contentBinding'] = stack4;
  stack4 = "view.hasSpecialTypeRepoVersion";
  stack3['displayVersionTypeBlockBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.stackVersionError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div id=\"versions-filter-section\" class=\"btn-toolbar\">\n  ");
  stack1 = depth0;
  stack2 = "AMBARI.MANAGE_STACK_VERSIONS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"btn-group display-inline-block\">\n    <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n      <span class=\"filters-label\">");
  stack1 = depth0;
  stack2 = "common.filter";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": </span>\n        <span>\n          ");
  stack1 = depth0;
  stack2 = "view.selectedCategory.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          <span class=\"caret\"></span>\n        </span>\n    </button>\n    <ul class=\"dropdown-menu\">\n      ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n</div>\n<div id=\"versions-section\" class=\"row col-md-12\">\n  <div ");
  stack1 = {};
  stack2 = ":col-md-3 :left-menu-table view.hasSpecialTypeRepoVersion:has-special-type-repo-version";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <table class=\"table border-extended-table\">\n      <thead>\n      </thead>\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  </div>\n  <div class=\"col-md-9 versions-slides\">\n    ");
  stack1 = depth0;
  stack2 = "isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.MainAlertDefinitionActionsView";
  stack3 = {};
  stack4 = "App.router.mainAlertDefinitionActionsController";
  stack3['controllerBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.statusSort";
  stack3 = {};
  stack4 = "first status-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.nameSort";
  stack3 = {};
  stack4 = "name-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.serviceSort";
  stack3 = {};
  stack4 = "service-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.lastTriggeredSort";
  stack3 = {};
  stack4 = "last-triggred-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.enabledSort";
  stack3 = {};
  stack4 = "enabled-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "alertDefinition";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <tr>\n            <td class=\"first alert-status\">\n              ");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionSummary";
  stack3 = {};
  stack4 = "alertDefinition";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </td>\n            <td class=\"alert-name\">\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "alertDefinition";
  stack3 = depth0;
  stack4 = "gotoAlertDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alertDefinition.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </td>\n            <td class=\"alert-service\">");
  stack1 = depth0;
  stack2 = "alertDefinition.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td class=\"alert-time\">\n              <time class=\"timeago\" ");
  stack1 = {};
  stack2 = "alertDefinition.lastTriggeredFormatted";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alertDefinition.lastTriggeredAgoFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</time>\n            </td>\n            <td class=\"last toggle-state-button alert-state\">\n              ");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionState";
  stack3 = {};
  stack4 = "alertDefinition";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </td>\n          </tr>\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td class=\"first\"></td>\n          <td colspan=\"4\">\n            ");
  stack1 = depth0;
  stack2 = "alerts.table.noAlerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n      ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td class=\"first\"></td>\n        <td colspan=\"2\">\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </td>\n        <td colspan=\"2\"></td>\n      </tr>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"alerts\" class=\"container-wrap-table\">\n  <div class=\"row\">\n    <h2 class=\"table-title col-sm-1\">");
  stack1 = depth0;
  stack2 = "menu.item.alerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n    <div class=\"table-controls row col-sm-11 pull-right\">\n      <div class=\"col-sm-12\">\n        ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <div class=\"VS-open-box pull-right\">\n          <button class=\"btn btn-default\">\n            <i class=\"icon-filter\"></i>\n          </button>\n          <div class=\"popup-arrow-up hide\"></div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"row search-box-row hide\">\n    ");
  stack1 = depth0;
  stack2 = "App.MainAlertDefinitionSearchBoxView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <table class=\"table advanced-header-table table-hover alerts-table\" id=\"alert-definitions-table\">\n    <thead>\n    ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "label-row";
  stack3['classNames'] = stack4;
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </thead>\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isAlertsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n    <tfoot>\n      <tr>\n        <td colspan=\"5\">\n          ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </td>\n      </tr>\n    </tfoot>\n  </table>\n\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_alert_definition/add_alert_definition", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li class=\"nav-header\">");
  stack1 = depth0;
  stack2 = "alerts.add.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "alerts.add.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "alerts.add.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "alerts.add.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_alert_definition/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <div class=\"col-md-4 row alert-type\" rel=\"selectable-tooltip\"\n          ");
  stack1 = {};
  stack2 = "alerts.add.wizard.step1.body.choose.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "type";
  stack3 = depth0;
  stack4 = "selectType";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div ");
  stack1 = {};
  stack2 = "type.name :icon :col-md-3";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <span ");
  stack1 = {};
  stack2 = "type.icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n          </div>\n          <div class=\"label-description col-md-9\">\n            <p class=\"label-text\">");
  stack1 = depth0;
  stack2 = "type.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n            <p class=\"description-text\">");
  stack1 = depth0;
  stack2 = "type.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n          </div>\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"create-alert-wizard-step1\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "alerts.add.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "alerts.add.step1.header.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    <div class=\"alert-types-container row\">\n      ");
  stack1 = depth0;
  stack2 = "controller.alertDefinitionsTypes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "type";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_alert_definition/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "alerts.add.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n    ");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionConfigsView";
  stack3 = {};
  stack4 = true;
  stack3['canEdit'] = stack4;
  stack4 = true;
  stack3['isWizard'] = stack4;
  stack4 = "controller.content.selectedType";
  stack3['alertDefinitionTypeBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "isBackBtnDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_alert_definition/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n    <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "alerts.add.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n    <div class=\"panel panel-default\">\n      <div class=\"panel-body\">\n        <div>\n          ");
  stack1 = depth0;
  stack2 = "alerts.add.step3.selectedType";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": <strong>");
  stack1 = depth0;
  stack2 = "controller.content.selectedType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong>\n        </div>\n        <div class=\"control-group\">\n            <pre class=\"alert-definition-review\">");
  stack1 = depth0;
  stack2 = "view.alertDefinitionToDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n        </div>\n      </div>\n    </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "isBackBtnDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "done";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.done";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </button>\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_custom_config_to_alert_notification_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"alert alert-danger\">\n      ");
  stack1 = depth0;
  stack2 = "controller.errorMessageForNewCustomPropertyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<form autocomplete=\"off\" class=\"form-horizontal alert-configs\">\n  <div ");
  stack1 = {};
  stack2 = "controller.isErrorWithNewCustomPropertyName:error :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <label class=\"control-label col-md-3\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </label>\n\n    <div class=\"col-md-7\">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.newCustomProperty.name";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n\n  <div class=\"form-group\">\n    <label class=\"control-label col-md-3\">");
  stack1 = depth0;
  stack2 = "common.value";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </label>\n\n    <div class=\"col-md-7\">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.newCustomProperty.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n\n  ");
  stack1 = depth0;
  stack2 = "controller.isErrorWithNewCustomPropertyName";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/add_definition_to_group_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <form autocomplete=\"off\" class=\"form-horizontal mbm\">\n    <div class=\"override-controls\">\n      <div class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "view.parentView.dialogMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <div class=\"row\">\n        <div class=\"col-md-8\">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleShowSelectedDefs";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.defSelectMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          ");
  stack1 = depth0;
  stack2 = "view.showOnlySelectedDefs";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div class=\"col-md-2\" id=\"component-dropdown-div\">\n          <!-- definition-components drop-down -->\n          <div class=\"btn-group pull-right\">\n            <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\"\n                    href=\"#\" ");
  stack1 = {};
  stack2 = "view.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "common.component";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              <span class=\"caret\"></span>\n            </button>\n            <ul class=\"dropdown-menu alert-definition-filter-list\">\n              <!-- dropdown menu links -->\n              ");
  stack1 = depth0;
  stack2 = "view.filterComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        </div>\n        <div class=\"col-md-2\" id=\"filter-dropdown-div\">\n          <!-- services drop-down -->\n          <div class=\"btn-group pull-right\">\n            <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\"\n                    href=\"#\" ");
  stack1 = {};
  stack2 = "view.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              <span class=\"caret\"></span>\n            </button>\n            <ul class=\"dropdown-menu alert-definition-filter-list\">\n              <!-- dropdown menu links -->\n              ");
  stack1 = depth0;
  stack2 = "view.filterServices";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        </div>\n      </div>\n      <table class=\"table table-hover\">\n        <thead>\n        <tr>\n          <th width=\"35%\">");
  stack1 = depth0;
  stack2 = "common.alertDefinition";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th width=\"25%\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th width=\"30%\">");
  stack1 = depth0;
  stack2 = "common.component";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th width=\"10%\">\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.allDefsSelected";
  stack3['checkedBinding'] = stack4;
  stack4 = "view.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </th>\n        </tr>\n        </thead>\n      </table>\n      <div class=\"hosts-table-container\">\n        <table class=\"table table-hover\">\n          <tbody>\n            ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n        </table>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isPaginate";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </form>\n\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"text-warning\">\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  
  data.buffer.push("\n            <i class='glyphicon glyphicon-ok-sign'></i>\n          ");}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li>\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "component";
  stack3 = depth0;
  stack4 = "selectFilterComponent";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "component.selected";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "component.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}
function program7(depth0,data) {
  
  
  data.buffer.push("\n                      <i class='glyphicon glyphicon-ok-sign'></i>\n                    ");}

function program9(depth0,data) {
  
  
  data.buffer.push("\n                      <i class='glyphicon glyphicon-placeholder'></i>\n                    ");}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li>\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "service";
  stack3 = depth0;
  stack4 = "selectFilterService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "service.selected";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}
function program12(depth0,data) {
  
  
  data.buffer.push("\n                      <i class='glyphicon glyphicon-ok-sign'></i>\n                    ");}

function program14(depth0,data) {
  
  
  data.buffer.push("\n                      <i class='glyphicon glyphicon-placeholder'></i>\n                    ");}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "entry";
  stack7 = helpers.each;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <tr ");
  stack1 = {};
  stack2 = "entry.filtered::hidden entry.selected:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <td width=\"35%\">\n                    ");
  stack1 = depth0;
  stack2 = "entry.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                  <td width=\"25%\">\n                    ");
  stack1 = depth0;
  stack2 = "entry.serviceNameDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                  <td width=\"30%\">\n                    ");
  stack1 = depth0;
  stack2 = "entry.componentNameDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                  <td width=\"10%\">\n                    ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "entry.selected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                </tr>\n              ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td width=\"30%\"></td>\n                <td>");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_groups_popup.addDefinition.noDefinitions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n              </tr>\n            ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <table class=\"table table-hover\">\n          <tfoot>\n            <tr>\n              <td>\n                ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.parentView.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_definition/alert_definition_state", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "toggleState";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = {};
  stack2 = "view.content.enabled:alert-definition-enable:alert-definition-disable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span class=\"enable-disable-button\" ");
  stack1 = {};
  stack2 = "view.tooltipText";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.labelText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </span>\n  </a>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <span ");
  stack1 = {};
  stack2 = "view.content.enabled:alert-definition-enable:alert-definition-disable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "view.labelText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </span>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "CLUSTER.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_definition/alert_definition_summary", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.definitionState";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "st";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "st.count";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "st.maintenanceCount";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span ");
  stack1 = {};
  stack2 = ":alert-state-single-host :label st.state";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "st.count";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"alert-state-single-host label alert-state-PENDING\"><span\n        class=\"icon-medkit\"></span> ");
  stack1 = depth0;
  stack2 = "st.maintenanceCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <span class=\"alert-state-single-host label alert-state-PENDING\">");
  stack1 = depth0;
  stack2 = "App.AlertDefinition.shortState.PENDING";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isActiveDefinitionState";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_definitions_actions", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <li>\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "action";
  stack3 = depth0;
  stack4 = "actionHandler";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <span ");
  stack1 = {};
  stack2 = "action.icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n        ");
  stack1 = depth0;
  stack2 = "action.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </li>\n    ");
  stack1 = depth0;
  stack2 = "action.showDivider";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  
  data.buffer.push("\n      <li class=\"divider\"></li>\n    ");}

  data.buffer.push("\n\n<button class=\"btn btn-success dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">");
  stack1 = depth0;
  stack2 = "common.actions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span></button>\n<ul class=\"dropdown-menu pull-left\">\n  ");
  stack1 = depth0;
  stack2 = "controller";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "action";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</ul>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_groups/success_popup_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "alerts.groups.successPopup.body.created";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " - <strong>");
  stack1 = depth0;
  stack2 = "view.dataForSuccessPopup.created";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n<p>");
  stack1 = depth0;
  stack2 = "alerts.groups.successPopup.body.updated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " - <strong>");
  stack1 = depth0;
  stack2 = "view.dataForSuccessPopup.updated";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></p>\n<p>");
  stack1 = depth0;
  stack2 = "alerts.groups.successPopup.body.deleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " - <strong>");
  stack1 = depth0;
  stack2 = "view.dataForSuccessPopup.deleted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></p>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_instance/status", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  
  data.buffer.push("\n    <span class=\"icon-medkit\"></span>\n  ");}

  data.buffer.push("\n\n");
  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = ":label :alert-state-single-host instance.stateClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "instance.isMaintenanceStateOn";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <span rel=\"StateTooltip\" ");
  stack1 = {};
  stack2 = "instance.retryText";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "instance.shortStateMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </span>\n</div>\n<time class=\"timeago\"\n      rel=\"tooltip\" ");
  stack1 = {};
  stack2 = "instance.statusChangedAndLastCheckedFormatted";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "instance.lastTriggeredForFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</time>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/alert_notifications_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"state-filter pull-right\">\n      <span>");
  stack1 = depth0;
  stack2 = "common.show";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":&nbsp;</span>\n      <div class=\"btn-group\">\n        ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.categories";
  stack3['contentBinding'] = stack4;
  stack4 = "content.value";
  stack3['optionValuePath'] = stack4;
  stack4 = "content.label";
  stack3['optionLabelPath'] = stack4;
  stack4 = "view.selectedCategory";
  stack3['selectionBinding'] = stack4;
  stack4 = "filter-select form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <table class=\"alerts-table table table-hover\">\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.isAlertEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tr class=\"no-alert-tr\">\n            <td class=\"no-alert-text\">");
  stack1 = depth0;
  stack2 = "alerts.fastAccess.popup.body.noalerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </tr>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "instance";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <tr ");
  stack1 = depth0;
  stack2 = "instance";
  stack3 = depth0;
  stack4 = "gotoAlertDetails";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "instance.isVisible::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <td class=\"status\">\n                <span ");
  stack1 = {};
  stack2 = "instance.stateClass :icon-circle";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n              </td>\n              <td class=\"content\">\n                <div class=\"name\">");
  stack1 = depth0;
  stack2 = "instance.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                <div class=\"description\">");
  stack1 = depth0;
  stack2 = "instance.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                <div class=\"timestamp\">");
  stack1 = depth0;
  stack2 = "instance.lastTriggeredAgoFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n              </td>\n            </tr>\n          ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"popup-arrow-up\"></div>\n<div class=\"notifications-header col-sm-12\">\n  <div class=\"notifications-title pull-left\">");
  stack1 = depth0;
  stack2 = "alerts.dropdown.dialog.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " (");
  stack1 = depth0;
  stack2 = "view.alertsNumber";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</div>\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div class=\"notifications-body col-sm-12\">\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div class=\"notifications-footer col-sm-12\">\n  <button type=\"button\" class=\"btn btn-primary pull-right\" ");
  stack1 = depth0;
  stack2 = "gotoAllAlerts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">View All</button>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/configs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "property.hidden";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":form-group property.name property.isValid::has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "property.isPreLabeled";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <label ");
  stack1 = {};
  stack2 = ":control-label :col-md-2";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "property.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n          <div ");
  stack1 = {};
  stack2 = ":col-md-10";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "property.viewClass";
  stack3 = {};
  stack4 = "property";
  stack3['propertyBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div>\n            <label class=\"radio\">\n              ");
  stack1 = depth0;
  stack2 = "property.viewClass";
  stack3 = {};
  stack4 = "property";
  stack3['propertyBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "property.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </label>\n          </div>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"error-message\">\n      ");
  stack1 = depth0;
  stack2 = "view.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n<form autocomplete=\"off\" class=\"form-horizontal alert-configs\">\n  ");
  stack1 = depth0;
  stack2 = "controller.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "property";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.hasThresholdsError";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/configs/alert_config_parameter", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"col-md-4\">\n    <span ");
  stack1 = {};
  stack2 = "view.property.badgeCssClass :label :alert-label :alert-state-single-host";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.property.badge";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </span>\n  </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <span class=\"input-group-addon\">");
  stack1 = depth0;
  stack2 = "view.property.units";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div>\n  ");
  stack1 = depth0;
  stack2 = "view.property.thresholdExists";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div rel=\"parameter-tooltip\" ");
  stack1 = {};
  stack2 = "view.property.description";
  stack1['data-original-title'] = stack2;
  stack2 = "view.bigInput:col-md-12:col-md-4";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div ");
  stack1 = {};
  stack2 = "view.property.units:input-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.property.value";
  stack3['valueBinding'] = stack4;
  stack4 = "view.property.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.property.units";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/configs/alert_config_text_area", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"col-md-12\">\n  ");
  stack1 = depth0;
  stack2 = "Em.TextArea";
  stack3 = {};
  stack4 = "4";
  stack3['rows'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.property.value";
  stack3['valueBinding'] = stack4;
  stack4 = "view.property.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/configs/alert_config_text_field", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"input-group-addon\">");
  stack1 = depth0;
  stack2 = "view.property.unit";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

  data.buffer.push("\n<div ");
  stack1 = {};
  stack2 = "view.colWidth";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div ");
  stack1 = {};
  stack2 = "view.property.unit:input-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.property.value";
  stack3['valueBinding'] = stack4;
  stack4 = "view.property.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.property.unit";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/configs/alert_config_threshold", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"col-md-4\">\n    <div ");
  stack1 = {};
  stack2 = "view.property.valueMetric:input-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.property.displayValue";
  stack3['valueBinding'] = stack4;
  stack4 = "view.property.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.property.valueMetric";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <span class=\"input-group-addon\">");
  stack1 = depth0;
  stack2 = "view.property.valueMetric";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = "view.property.showInputForValue:col-md-4:col-md-8";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.property.text";
  stack3['valueBinding'] = stack4;
  stack4 = "view.property.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

  data.buffer.push("\n\n<div class=\"col-md-4\">\n  <span ");
  stack1 = {};
  stack2 = "view.property.badgeCssClass :alert-label :label :alert-state-single-host";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.property.badge";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.property.showInputForValue";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.property.showInputForText";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/create_alert_notification", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n          ");
  stack1 = depth0;
  stack2 = "controller.inputFields.name.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.email.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\" for=\"inputEmail\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.email.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.email.value";
  stack3['valueBinding'] = stack4;
  stack4 = "inputEmail";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.email.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPServer.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.SMTPServer.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.SMTPPort.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPort.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.SMTPPort.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPort.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.emailFrom.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.emailFrom.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.emailFrom.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.emailFrom.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "form-group";
  stack3['classNames'] = stack4;
  stack4 = "true";
  stack3['isCustomTemplate'] = stack4;
  stack4 = "controller.inputFields.SMTPUseAuthentication.value";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.SMTPUsername.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPUsername.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.SMTPUseAuthentication.invertedValue";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.inputFields.SMTPUsername.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPUsername.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.SMTPPassword.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPassword.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "controller.inputFields.SMTPUseAuthentication.invertedValue";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.inputFields.SMTPPassword.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPassword.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.retypeSMTPPassword.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.retypeSMTPPassword.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "password";
  stack3['type'] = stack4;
  stack4 = "controller.inputFields.SMTPUseAuthentication.invertedValue";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.inputFields.retypeSMTPPassword.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.retypeSMTPPassword.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\" for=\"inputSMTPSTARTTLS\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPSTARTTLS.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "controller.inputFields.SMTPUseAuthentication.invertedValue";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.inputFields.SMTPSTARTTLS.value";
  stack3['checkedBinding'] = stack4;
  stack4 = "inputSMTPSTARTTLS";
  stack3['id'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.email.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPort.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.emailFrom.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <label class=\"control-label col-md-2 inputUseAuthentication\" ");
  stack1 = {};
  stack2 = "view.checkboxId";
  stack1['for'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPUseAuthentication.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "view.checkboxView";
  stack3 = {};
  stack4 = "inputUseAuthentication";
  stack3['customCheckboxId'] = stack4;
  stack4 = "left";
  stack3['labelAlign'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.labelView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPUsername.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.SMTPPassword.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.retypeSMTPPassword.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.version.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "controller.inputFields.version.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "controller.SNMPVersions";
  stack3['contentBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.community.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.community.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.host.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.host.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.port.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.port.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.version.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "controller.inputFields.version.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "controller.SNMPVersions";
  stack3['contentBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.OIDs.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.OIDs.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.community.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.community.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.host.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.host.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n\n      <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.port.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.port.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.host.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n            ");
  stack1 = depth0;
  stack2 = "controller.inputFields.port.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.scriptFileName.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n       <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.scriptFileName.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n       <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.scriptFileName.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n       </div>\n\n       ");
  stack1 = depth0;
  stack2 = "controller.inputFields.scriptFileName.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n\n    <div class=\"form-group\">\n      <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.scriptDispatchProperty.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n         ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.scriptDispatchProperty.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    ");
  return buffer;}
function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"col-md-10 col-md-offset-2 help-block validation-block error-msg\">\n           ");
  stack1 = depth0;
  stack2 = "controller.inputFields.scriptFileName.errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n       ");
  return buffer;}

function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <div class=\"form-group\">\n        <label class=\"control-label col-md-2\" for=\"inputGroups\">");
  stack1 = depth0;
  stack2 = "customProperty.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n        <div class=\"col-md-9\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "customProperty.value";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"col-md-1\">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "customProperty";
  stack3 = depth0;
  stack4 = "removeCustomPropertyHandler";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"btn-sm\"><span class=\"icon-minus-sign\"></span></a>\n        </div>\n      </div>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"create-edit-alert-notification row\">\n\n  <form autocomplete=\"off\" class=\"form-horizontal\">\n\n    ");
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = ":form-group controller.inputFields.name.errorMsg:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <label class=\"control-label col-md-2\" for=\"inputName\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.name.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.inputFields.name.value";
  stack3['valueBinding'] = stack4;
  stack4 = "inputName";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "controller.inputFields.name.errorMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"form-group\">\n      <label class=\"control-label col-md-2\" for=\"inputGroups\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.groups.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n      <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "allGroups";
  stack3['name'] = stack4;
  stack4 = "controller.inputFields.allGroups.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "common.all";
  stack3['labelTranslate'] = stack4;
  stack4 = "all";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "App.RadioButtonView";
  stack3 = {};
  stack4 = "allGroups";
  stack3['name'] = stack4;
  stack4 = "controller.inputFields.allGroups.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "common.custom";
  stack3['labelTranslate'] = stack4;
  stack4 = "custom";
  stack3['value'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <div>\n          ");
  stack1 = depth0;
  stack2 = "view.groupsSelectView";
  stack3 = {};
  stack4 = "true";
  stack3['multiple'] = stack4;
  stack4 = "inputGroups";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "controller.inputFields.allGroups.isAll";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.inputFields.groups.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "controller.allAlertGroups";
  stack3['contentBinding'] = stack4;
  stack4 = "content.displayName";
  stack3['optionLabelPath'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "selectAllGroups";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.inputFields.allGroups.isAll:disabled view.allGroupsSelected:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.selectAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a> |\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "clearAllGroups";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.inputFields.allGroups.isAll:disabled view.noneGroupsSelected:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.clearAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"form-group\">\n      <label class=\"control-label col-md-2\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.severityFilter.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n        <div>\n          ");
  stack1 = depth0;
  stack2 = "view.severitySelectView";
  stack3 = {};
  stack4 = "true";
  stack3['multiple'] = stack4;
  stack4 = "inputSeverity";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "controller.inputFields.severityFilter.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "controller.severities";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div>\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "selectAllSeverity";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.allSeveritySelected:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.selectAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a> |\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "clearAllSeverity";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.noneSeveritySelected:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.clearAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"form-group\">\n      <label class=\"control-label col-md-2\" for=\"inputDescription\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.description.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Em.TextArea";
  stack3 = {};
  stack4 = "controller.inputFields.description.value";
  stack3['valueBinding'] = stack4;
  stack4 = "inputDescription";
  stack3['id'] = stack4;
  stack4 = "4";
  stack3['rows'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"form-group\">\n      <label class=\"control-label col-md-2\" for=\"inputMethod\">");
  stack1 = depth0;
  stack2 = "controller.inputFields.method.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "controller.methods";
  stack3['contentBinding'] = stack4;
  stack4 = "controller.inputFields.method.value";
  stack3['selectionBinding'] = stack4;
  stack4 = "inputMethod";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isEmailMethodSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isSNMPMethodSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isCustomSNMPMethodSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isAlertScriptMethodSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.inputFields.customProperties";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "customProperty";
  stack7 = helpers.each;
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    <a href=\"#\" class=\"add-custom-property\" ");
  stack1 = depth0;
  stack2 = "addCustomPropertyHandler";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step7.config.addProperty";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ...</a>\n\n  </form>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/create_new_alert_group", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  
  data.buffer.push("\n        &nbsp;\n    ");}

  data.buffer.push("\n\n<div class=\"new-config-group-div\" id=\"create-new-alert-group\">\n  <div class=\"text-warning\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <form autocomplete=\"off\" class=\"form-horizontal\">\n    <div class=\"form-group\">\n     <label class=\"control-label col-md-3\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": &nbsp;</label>\n     <div class=\"col-md-7\">\n       ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "255";
  stack3['maxlength'] = stack4;
  stack4 = "alertGroupName";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n     </div>\n    </div>\n  </form>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/definition_details", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n\n  <div id=\"alert-definition-details\">\n\n  <div class=\"alert-definition-details-header row\">\n    <div class=\"col-md-8\">\n      ");
  data.buffer.push("\n      <div class=\"definition-name row\">\n        ");
  stack1 = depth0;
  stack2 = "controller.editing.label.isEditing";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"error-msg\"> ");
  stack1 = depth0;
  stack2 = "controller.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n      <div>\n        <a href=\"javascript:void(null)\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-arrow-left\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </div>\n    </div>\n\n    <div class=\"status align-right col-md-4\">\n      ");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionSummary";
  stack3 = {};
  stack4 = "view.controller.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n\n  <div class=\"row\">\n\n    ");
  data.buffer.push("\n    ");
  data.buffer.push("\n    <div class=\"col-md-8\">\n    <div class=\"panel panel-default\">\n      <div class=\"panel-heading\">\n        <div class=\"row no-vertical-padding\">\n          <div class=\"col-md-7\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "common.configuration";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n          <div class=\"col-md-5\">\n            ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n      <div class=\"panel-body\">\n        ");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionConfigsView";
  stack3 = {};
  stack4 = "view.controller.content";
  stack3['contentBinding'] = stack4;
  stack4 = "view.controller.content.type";
  stack3['alertDefinitionTypeBinding'] = stack4;
  stack4 = false;
  stack3['canEdit'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertDefinitionConfigsController.canEdit";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n    </div>\n    ");
  data.buffer.push("\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"col-md-4 right-column\">\n      <div class=\"panel panel-default\">\n        <div class=\"panel-heading\">\n          <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "alerts.definition.details.infoSection";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n        </div>\n        <div class=\"properties-list background-text panel-body\">\n          <div class=\"row\">\n            <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "alerts.table.state";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n            <div class=\"col-md-7\">\n              ");
  stack1 = depth0;
  stack2 = "controller.content.enabled";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(17, program17, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "controller.content.serviceDisplayName";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "controller.content.componentNameFormatted";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          <div class=\"row\">\n            <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n            <div class=\"col-md-7\">\n              <span ");
  stack1 = {};
  stack2 = ":type-icon  controller.content.typeIconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span> ");
  stack1 = depth0;
  stack2 = "controller.content.type";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "controller.groupsList.length";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          <div class=\"row\">\n            <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "alerts.table.header.lastTrigger";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n            <div class=\"col-md-7\">");
  stack1 = depth0;
  stack2 = "controller.content.lastTriggeredFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "controller.content.isTypeAggregate";
  stack3 = helpers.unless;
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "controller.content.hasHelpUrl";
  stack3 = helpers['if'];
  tmp1 = self.program(36, program36, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n  </div>\n  ");
  data.buffer.push("\n\n  ");
  data.buffer.push("\n    <div class=\"row\">\n      <div class=\"col-md-12\">\n        <h2 class=\"table-title\">");
  stack1 = depth0;
  stack2 = "alerts.definition.details.instances";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"col-md-12\">\n    ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertInstancesController.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(38, program38, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(53, program53, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n  ");
  data.buffer.push("\n\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = "controller.editing.label.isError:error :col-md-8 :name-text-field";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "controller.editing.label.value";
  stack3['valueBinding'] = stack4;
  stack4 = "255";
  stack3['maxlength'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n          <div class=\"edit-buttons align-right col-md-4\">\n            <button ");
  stack1 = depth0;
  stack2 = "controller.editing.label";
  stack3 = depth0;
  stack4 = "cancelEdit";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                class=\"btn btn-default cancel-button\">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            <button ");
  stack1 = {};
  stack2 = "controller.editing.label.isError";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "controller.editing.label";
  stack3 = depth0;
  stack4 = "saveEdit";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                class=\"btn btn-primary save-button\">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n          </div>\n        ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"col-md-12\">\n            <span>\n              ");
  stack1 = depth0;
  stack2 = "controller.content.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </span>\n            ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.editing.label";
  stack3 = depth0;
  stack4 = "edit";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"edit-description-button\">\n                <i class=\"glyphicon glyphicon-pencil\"></i>\n              </a>\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertDefinitionConfigsController.canEdit";
  stack3 = helpers.unless;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "editConfigs";
  stack3 = {};
  stack4 = "App.router.mainAlertDefinitionConfigsController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right btn btn-link\">\n                  ");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"edit-buttons align-right\">\n            <button ");
  stack1 = depth0;
  stack2 = "cancelEditConfigs";
  stack3 = {};
  stack4 = "App.router.mainAlertDefinitionConfigsController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                class=\"btn btn-default\">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            <button ");
  stack1 = {};
  stack2 = "App.router.mainAlertDefinitionConfigsController.hasErrors";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "saveConfigs";
  stack3 = {};
  stack4 = "App.router.mainAlertDefinitionConfigsController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                class=\"btn btn-primary\">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </button>\n          </div>\n        ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "toggleState";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.content.enabled:alert-definition-enable:alert-definition-disable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <span\n                    class=\"enable-disable-button\" ");
  stack1 = {};
  stack2 = "alerts.table.state.enabled.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "view.enabledDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </span>\n                  </a>\n                ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.enabledDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(20, program20, data);
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "toggleState";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "controller.content.enabled:alert-definition-enable:alert-definition-disable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <span\n                    class=\"enable-disable-button\" ");
  stack1 = {};
  stack2 = "alerts.table.state.disabled.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "view.disabledDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </span>\n                  </a>\n                ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.disabledDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"row\">\n              <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-7\"><span>");
  stack1 = depth0;
  stack2 = "controller.content.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></div>\n            </div>\n          ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"row\">\n              <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "common.component";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-7\">");
  stack1 = depth0;
  stack2 = "controller.content.componentNameFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n          ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div class=\"row\">\n              <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "alerts.definition.details.groups";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-7\">\n                <ul class=\"list-unstyled\">\n                  ");
  stack1 = depth0;
  stack2 = "controller.groupsList";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "groupName";
  stack7 = helpers.each;
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n              </div>\n            </div>\n          ");
  return buffer;}
function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <li>");
  stack1 = depth0;
  stack2 = "groupName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                  ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"row\">\n              <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "alerts.table.header.repeatTolerance";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-7\">\n                ");
  stack1 = depth0;
  stack2 = "controller.content.repeat_tolerance_enabled";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(32, program32, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "SERVICE.TOGGLE_ALERTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            </div>\n          ");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span>\n                    ");
  stack1 = depth0;
  stack2 = "controller.content.repeat_tolerance";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </span>\n                ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span>\n                  ");
  stack1 = depth0;
  stack2 = "controller.globalAlertsRepeatTolerance";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "alerts.table.header.default";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n                ");
  return buffer;}

function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "editRepeatTolerance";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"edit-description-button\">\n                    <i class=\"glyphicon glyphicon-pencil\"></i>\n                  </a>\n                ");
  return buffer;}

function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div class=\"row\">\n              <div class=\"col-md-5 property-name\">");
  stack1 = depth0;
  stack2 = "alerts.table.header.helpUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-7\"><label for=\"\"><a ");
  stack1 = {};
  stack2 = "controller.content.helpUrl";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "common.link";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></label></div>\n            </div>\n          ");
  return buffer;}

function program38(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n\n      <table class=\"table advanced-header-table table-hover alerts-table\" id=\"alert-instances-table\">\n        <thead>\n        ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(39, program39, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <tr class=\"filter-row\">\n          <th class=\"first\">");
  stack1 = depth0;
  stack2 = "view.serviceFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  stack1 = depth0;
  stack2 = "view.hostNameFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  stack1 = depth0;
  stack2 = "view.stateFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  data.buffer.push("</th>\n          <th>");
  data.buffer.push("</th>\n        </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(41, program41, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(51, program51, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n        <tfoot>\n        <tr>\n          <td colspan=\"5\">\n            ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n        </tfoot>\n      </table>\n\n    ");
  return buffer;}
function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.serviceSort";
  stack3 = {};
  stack4 = "first service-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.hostNameSort";
  stack3 = {};
  stack4 = "host-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.stateSort";
  stack3 = {};
  stack4 = "state-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          <th>");
  stack1 = depth0;
  stack2 = "alerts.definition.details.24-hour";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th>");
  stack1 = depth0;
  stack2 = "alerts.table.header.check.response";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        ");
  return buffer;}

function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "instance";
  stack7 = helpers.each;
  tmp1 = self.program(42, program42, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program42(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.instanceTableRow";
  stack3 = helpers.view;
  tmp1 = self.program(43, program43, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program43(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <td class=\"first\">\n                ");
  stack1 = depth0;
  stack2 = "instance.serviceDisplayName";
  stack3 = helpers['if'];
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </td>\n              <td>\n                ");
  stack1 = depth0;
  stack2 = "instance.hostName";
  stack3 = helpers['if'];
  tmp1 = self.program(49, program49, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </td>\n              <td>\n                ");
  stack1 = depth0;
  stack2 = "App.AlertInstanceStateView";
  stack3 = {};
  stack4 = "instance";
  stack3['instanceBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </td>\n              <td>");
  stack1 = depth0;
  stack2 = "view.parentView.lastDayCount";
  stack3 = {};
  stack4 = "instance.hostName";
  stack3['hostNameBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td>\n                <a href=\"#\" class=\"alert-text\" ");
  stack1 = depth0;
  stack2 = "instance";
  stack3 = depth0;
  stack4 = "openFullResponse";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "instance.text";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "instance.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </td>\n            ");
  return buffer;}
function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "instance.service.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(45, program45, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(47, program47, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program45(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <a ");
  stack1 = depth0;
  stack2 = "instance.service";
  stack3 = depth0;
  stack4 = "goToService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      href=\"#\">");
  stack1 = depth0;
  stack2 = "instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program47(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program49(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <a ");
  stack1 = depth0;
  stack2 = "instance.host";
  stack3 = depth0;
  stack4 = "goToHostAlerts";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                    ");
  stack1 = depth0;
  stack2 = "instance.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                ");
  return buffer;}

function program51(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <tr>\n            <td class=\"first\" colspan=\"5\">\n              ");
  stack1 = depth0;
  stack2 = "alerts.definition.details.noAlerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </td>\n          </tr>\n        ");
  return buffer;}

function program53(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program55(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(55, program55, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/instance_service_host", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.serviceIsLink";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <a ");
  stack1 = depth0;
  stack2 = "view.instance.service";
  stack3 = depth0;
  stack4 = "goToService";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "view.instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program6(depth0,data) {
  
  
  data.buffer.push("\n    &nbsp;/&nbsp;\n  ");}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a rel=\"UsageTooltip\" ");
  stack1 = depth0;
  stack2 = "view.instance.host";
  stack3 = depth0;
  stack4 = "goToHostAlerts";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-original-title=\"");
  stack1 = depth0;
  stack2 = "view.instance.hostName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" href=\"#\">\n      ");
  stack1 = depth0;
  stack2 = "view.instance.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </a>\n  ");
  return buffer;}

  data.buffer.push("\n<span class=\"trim_hostname\">\n  ");
  stack1 = depth0;
  stack2 = "view.instance.serviceDisplayName";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.showSeparator";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.instance.hostName";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</span>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/manage_alert_groups_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"row\">\n    <div class=\"col-md-12 col-lg-12\">\n      <p class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_groups_popup.notice";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n    </div>\n  </div>\n  <div class=\"row manage-configuration-group-content\">\n    ");
  data.buffer.push("\n    <div class=\"col-md-4 col-lg-4\">\n      <span>&nbsp;</span>\n      ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "alertGroups";
  stack3['contentBinding'] = stack4;
  stack4 = "content.label";
  stack3['optionLabelPath'] = stack4;
  stack4 = "view.selectedAlertGroup";
  stack3['selectionBinding'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "group-select select-group-box form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"btn-toolbar pull-right\">\n        <button rel=\"button-info\" class=\"btn btn-default add-group-button\" ");
  stack1 = {};
  stack2 = "view.addButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "addAlertGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-plus\"></i></button>\n        <button rel=\"button-info\" class=\"btn btn-default remove-group-button\" ");
  stack1 = {};
  stack2 = "view.removeButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = "controller.isRemoveButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "confirmDelete";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-minus\"></i></button>\n        <div class=\"btn-group\">\n          <button class=\"btn btn-default dropdown-toggle actions-group-button\" data-toggle=\"dropdown\">\n            <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;<span class=\"caret\"></span>\n          </button>\n          <ul class=\"dropdown-menu\">\n            <li ");
  stack1 = {};
  stack2 = "controller.isRenameButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"#\" rel=\"button-info-dropdown\" ");
  stack1 = {};
  stack2 = "view.renameButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "renameAlertGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_groups_popup.rename";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n            <li ");
  stack1 = {};
  stack2 = "controller.isDuplicateButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"#\" rel=\"button-info-dropdown\" ");
  stack1 = {};
  stack2 = "view.duplicateButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "duplicateAlertGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_groups_popup.duplicate";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n          </ul>\n        </div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"col-md-8 col-lg-8\">\n      <span>&nbsp;</span>\n      ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "selectedAlertGroup.definitions";
  stack3['contentBinding'] = stack4;
  stack4 = "content.label";
  stack3['optionLabelPath'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "group-select pull-right select-definiton-box form-control";
  stack3['class'] = stack4;
  stack4 = "selectedDefinitions";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"button-toolbar\">\n        <div class=\"pull-right\">\n          <a rel=\"button-info\" ");
  stack1 = {};
  stack2 = "view.addDefinitionTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = ":btn :btn-default selectedAlertGroup.isAddDefinitionsDisabled:disabled :add-definition";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "addDefinitions";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ><i\n            class=\"glyphicon glyphicon-plus\"></i></a>\n          <a rel=\"button-info\" ");
  stack1 = {};
  stack2 = "view.removeDefinitionTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = ":btn :btn-default isDeleteDefinitionsDisabled:disabled :remove-definition";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "deleteDefinitions";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ><i\n            class=\"glyphicon glyphicon-minus\"></i></a>\n        </div>\n        <div class=\"clearfix\"></div>\n      </div>\n      <div class=\"notification-editable-list\">\n        <span class=\"icon-envelope-alt\"></span>&nbsp;");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_groups_popup.notifications";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "App.EditableList";
  stack3 = {};
  stack4 = "selectedAlertGroup.notifications";
  stack3['itemsBinding'] = stack4;
  stack4 = "controller.alertNotifications";
  stack3['resourcesBinding'] = stack4;
  stack4 = "selectedAlertGroup.displayName";
  stack3['nameBinding'] = stack4;
  stack4 = false;
  stack3['isCaseSensitive'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    ");
  data.buffer.push("\n  </div>\n\n  ");
  data.buffer.push("\n  <div class=\"row\">\n    <div class=\"col-md-12 col-lg-12 text-danger\" id=\"manage-config-group-error-div\">\n      ");
  stack1 = depth0;
  stack2 = "controller.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "controller.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  
  data.buffer.push("\n        &nbsp;\n      ");}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/alerts/manage_alert_notifications_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"row manage-configuration-group-content\" id=\"manage-alert-notification-content\">\n    <div class=\"col-md-12\">\n      <div class=\"row\">\n        <div class=\"col-md-4 notification-list\">\n          <span>&nbsp;</span>\n          ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "alertNotifications";
  stack3['contentBinding'] = stack4;
  stack4 = "content.displayName";
  stack3['optionLabelPath'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "group-select form-control";
  stack3['class'] = stack4;
  stack4 = "view.selectedAlertNotification";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          <div class=\"btn-toolbar pull-right\">\n            <button rel=\"button-info\" class=\"btn btn-default add-notification-button\"\n              ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.addButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "addAlertNotification";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-plus\"></i></button>\n            <button rel=\"button-info\" class=\"btn btn-default remove-notification-button\"\n              ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.removeButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = {};
  stack2 = "view.isRemoveButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "deleteAlertNotification";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-minus\"></i></button>\n            <div class=\"btn-group notification-actions-button dropup\">\n              <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;<span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                  <li ");
  stack1 = {};
  stack2 = "view.isEditButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"\" rel=\"button-info-dropdown\"\n                      ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.editButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "editAlertNotification";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                  <li ");
  stack1 = {};
  stack2 = "view.isDuplicateButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"\" rel=\"button-info-dropdown\"\n                      ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.duplicateButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "duplicateAlertNotification";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.duplicate";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                ");
  stack1 = depth0;
  stack2 = "selectedAlertNotification.enabled";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n        </div>\n        <div class=\"col-md-8 notification-info\">\n          <span>&nbsp;</span>\n\n          <div class=\"row\">\n            <div class=\"col-md-12 pull-right\">\n              ");
  stack1 = depth0;
  stack2 = "alertNotifications.length";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(23, program23, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        </div>\n        <div class=\"clearfix\"></div>\n        <div class=\"col-md-12 row\">\n          <div class=\"col-md-12 text-danger\" id=\"manage-alert-notifications-error\">\n            ");
  stack1 = depth0;
  stack2 = "controller.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(27, program27, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = {};
  stack2 = "view.isEnableOrDisableButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"\" rel=\"button-info-dropdown\"\n                      ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.disableButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "enableOrDisable.disable";
  stack3 = depth0;
  stack4 = "enableOrDisableAlertNotification";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.disable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = {};
  stack2 = "view.isEnableOrDisableButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"\" rel=\"button-info-dropdown\"\n                      ");
  stack1 = {};
  stack2 = "alerts.actions.manage_alert_notifications_popup.enableButton";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "enableOrDisable.enable";
  stack3 = depth0;
  stack4 = "enableOrDisableAlertNotification";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.enable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "selectedAlertNotification";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div class=\"row notification-name\">\n                    <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "selectedAlertNotification.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                  </div>\n                  <div class=\"row notification-groups\">\n                    <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "common.groups";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    <div class=\"col-md-9 input-value\">\n                      ");
  stack1 = depth0;
  stack2 = "selectedAlertNotification.global";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </div>\n                  </div>\n                  <div class=\"row notification-severity\">\n                    <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "common.severity";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "view.severities";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                  </div>\n                  <div class=\"row notification-method\">\n                    <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.method";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "view.selectedAlertNotificationTypeText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                  </div>\n                  ");
  stack1 = depth0;
  stack2 = "view.showEmailDetails";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.showSNMPDetails";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  <div class=\"row notification-description\">\n                    <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    ");
  stack1 = depth0;
  stack2 = "selectedAlertNotification.description";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "common.all";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "view.selectedAlertNotificationGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "view.selectedAlertNotificationGroups";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.noGroup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <div class=\"row notification-email\">\n                      <div class=\"col-md-3 input-label\">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.email";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                      <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "view.email";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    </div>\n                  ");
  return buffer;}

function program17(depth0,data) {
  
  
  data.buffer.push("\n                  ");}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "selectedAlertNotification.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <div class=\"col-md-9 input-value\">");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.noDescription";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"alert alert-info\">\n                  ");
  stack1 = depth0;
  stack2 = "alerts.actions.manage_alert_notifications_popup.noAlertNotification";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "controller.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program27(depth0,data) {
  
  
  data.buffer.push("\n              &nbsp;\n            ");}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n<div class=\"alert alert-info margin-bottom-5\">");
  stack1 = depth0;
  stack2 = "alerts.actions.manageNotifications.info";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(29, program29, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/heatmap", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"row\">\n        <div class=\"col-md-3 legend-column background-text\">\n          <div class=\"panel panel-default\">\n            <div class=\"panel-body\">\n              ");
  stack1 = depth0;
  stack2 = "view.dropdownView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n              ");
  stack1 = depth0;
  stack2 = "controller.selectedMetric";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "controller.activeWidget";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n\n                <table class=\"legend\">\n                  ");
  stack1 = depth0;
  stack2 = "controller.selectedMetric.slotDefinitions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "slot";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </table>\n                ");
  stack1 = depth0;
  stack2 = "common.maximum";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":\n                <div id=\"inputMaximum\" class=\"control-group input-group\">\n                  ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "text";
  stack3['type'] = stack4;
  stack4 = "8";
  stack3['maxlength'] = stack4;
  stack4 = "controller.inputMaximum";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  <span class=\"input-group-addon\">");
  stack1 = depth0;
  stack2 = "controller.selectedMetric.units";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                </div>\n              ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <tr>\n                      <td>\n                        <div class=\"tile\" ");
  stack1 = {};
  stack2 = "slot.cssStyle";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      </td>\n                      <td>");
  stack1 = depth0;
  stack2 = "slot.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                    </tr>\n                  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"active-widget col-md-9\" ");
  stack1 = {};
  stack2 = "activeWidget.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "activeWidget.viewClass";
  stack3 = {};
  stack4 = "activeWidget";
  stack3['contentBinding'] = stack4;
  stack4 = "racks";
  stack3['racksBinding'] = stack4;
  stack4 = "activeWidget.id";
  stack3['idBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"heatmap\">\n  ");
  stack1 = depth0;
  stack2 = "isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/heatmap/heatmap_host", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.hostClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.hostTemperatureStyle";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <a href=\"#\" class=\"heatmap-host\" ");
  stack1 = depth0;
  stack2 = "view.hostModelLink";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/heatmap/heatmap_host_detail", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.metricName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.metricValue";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n    </tr>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "common.disk";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.diskUsage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</td>\n    </tr>\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "common.cpu";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.cpuUsage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</td>\n    </tr>\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "common.memory";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.memoryUsage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</td>\n    </tr>\n  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "view.details.hostComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n    </tr>\n  ");
  return buffer;}

  data.buffer.push("\n\n<table class=\"table\">\n  <thead>\n  <tr>\n    <td colspan=\"2\">");
  stack1 = depth0;
  stack2 = "view.details.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n  </tr>\n  </thead>\n  <tbody>\n  ");
  stack1 = depth0;
  stack2 = "view.details.metricName";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <tr>\n    <td>");
  stack1 = depth0;
  stack2 = "common.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n    <td>");
  stack1 = depth0;
  stack2 = "view.details.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n  </tr>\n  <tr>\n    <td>");
  stack1 = depth0;
  stack2 = "common.ipAddress";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n    <td>");
  stack1 = depth0;
  stack2 = "view.details.ip";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n  </tr>\n  <tr>\n    <td>");
  stack1 = depth0;
  stack2 = "common.rack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n    <td>");
  stack1 = depth0;
  stack2 = "view.details.rack";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n  </tr>\n  ");
  stack1 = depth0;
  stack2 = "view.details.diskUsage";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.details.cpuUsage";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.details.memoryUsage";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.details.hostComponents";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/heatmap/heatmap_rack", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = "view.hostCssStyle";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "App.MainChartsHeatmapHostView";
  stack3 = {};
  stack4 = "this";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"rackHeader panel-heading\">\n  <div class=\"statusName\">\n    <h3 class=\"panel-title rackName\">");
  stack1 = depth0;
  stack2 = "rack.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h3>\n  </div>\n</div>\n\n<div class=\"isActive hosts clearfix panel-body\">\n  <div ");
  stack1 = {};
  stack2 = "view.rack.isLoaded::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"row row-no-pad\">\n      ");
  stack1 = depth0;
  stack2 = "view.hosts";
  stack3 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "view.rack.isLoaded";
  stack3 = helpers.unless;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/heatmap_dropdown", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <li class=\"dropdown-submenu\">\n        <a tabindex=\"-1\">");
  stack1 = depth0;
  stack2 = "category.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        <ul class=\"dropdown-menu\">\n          ");
  stack1 = depth0;
  stack2 = "category.heatmaps";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "heatmap";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </li>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <li>\n              <a tabindex=\"-1\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "heatmap";
  stack3 = depth0;
  stack4 = "showHeatMapMetric";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "heatmap.widget_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n          ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"dropdown\">\n  <button class=\"btn btn-default btn-block dropdown-toggle heatmap-toggle-metrics-btn btn-icon\" data-toggle=\"dropdown\">\n    ");
  stack1 = depth0;
  stack2 = "charts.heatmap.selectMetric";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span>\n  </button>\n  <ul class=\"dropdown-menu\">\n    ");
  stack1 = depth0;
  stack2 = "controller.heatmapCategories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/charts/linear_time", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-title\" ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden::has-data :chart-title";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "view.isReady";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <a ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden :corner-icon :col-md-1";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" ");
  stack1 = depth0;
  stack2 = "toggleFormatsList";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-save\"></i>\n      </a>\n      <div class=\"export-graph-list-top\"></div>\n      ");
  stack1 = depth0;
  stack2 = "view.exportMetricsMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"screensaver chart-container\" ");
  stack1 = {};
  stack2 = "view.isReady:hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-title\" class=\"chart-title\">");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n<div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-container\" class=\"chart-container hide\" ");
  stack1 = {};
  stack2 = "view.isReady:show";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"ZoomInTooltip\">\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-yaxis\" class=\"chart-y-axis\" ");
  stack1 = depth0;
  stack2 = "showGraphInPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-xaxis\" class=\"chart-x-axis\" ");
  stack1 = depth0;
  stack2 = "showGraphInPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-legend\" class=\"chart-legend\"></div>\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-chart\" class=\"chart\"  ");
  stack1 = depth0;
  stack2 = "showGraphInPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  <div id=\"");
  stack1 = depth0;
  stack2 = "view.id";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "-timeline\" class=\"timeline\" ");
  stack1 = depth0;
  stack2 = "showGraphInPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n  ");
  stack1 = depth0;
  stack2 = "view.noTitleUnderGraph";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.NavItemView";
  stack3 = {};
  stack4 = "category.name";
  stack3['itemBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "category.url";
  stack3 = depth0;
  stack4 = "goToDashboardView";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"summary-width col-md-12\" id=\"dashboard-widgets-container\">\n    <ul class=\"nav nav-tabs background-text\">\n      ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n\n    <!--show widgets, heatmaps or configs in the content-->\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/config_history", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.versionSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.configGroupSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.modifiedSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.authorSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.notesSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(20, program20, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.ConfigVersionView";
  stack3 = {};
  stack4 = "item";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <td class=\"first\">\n              <span class=\"label label-info\">");
  stack1 = depth0;
  stack2 = "item.versionText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  stack1 = depth0;
  stack2 = "view.isServiceLinkDisabled";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-refresh :restart-required-service item.isRestartRequired::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                rel=\"Tooltip\" ");
  stack1 = {};
  stack2 = "dashboard.configHistory.table.restart.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n            </td>\n            <td>");
  stack1 = depth0;
  stack2 = "item.configGroupName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "item.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n            <td>");
  stack1 = depth0;
  stack2 = "item.createdDate";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            <td><span rel=\"Tooltip\" ");
  stack1 = {};
  stack2 = "item.author";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.authorFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            <td class=\"notes\">\n              ");
  stack1 = depth0;
  stack2 = "item.moreNotesExists";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(18, program18, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n          ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "item.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <a ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "goToServiceConfigs";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"service-name\">\n                  ");
  stack1 = depth0;
  stack2 = "item.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <span class=\"label label-success\" rel=\"Tooltip\"\n                ");
  stack1 = {};
  stack2 = "item.currentTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.showLessNotes";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(16, program16, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "item.briefNotes";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  <a ");
  stack1 = depth0;
  stack2 = "toggleShowLessStatus";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"show-more-button\">>> More</a>\n                ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "item.fullNotes";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  <a ");
  stack1 = depth0;
  stack2 = "toggleShowLessStatus";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"show-more-button\"><< Less</a>\n                ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "item.briefNotes";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td class=\"first\" colspan=\"5\">\n            ");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.table.empty";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n      ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr><td colspan=\"5\">");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td></tr>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"config_history\" class=\"container-wrap-table\">\n  <div class=\"row\">\n    <h2 class=\"table-title col-md-6\">");
  stack1 = depth0;
  stack2 = "dashboard.configHistory.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n    <div class=\"table-controls row col-sm-6 pull-right\">\n      <div class=\"col-sm-12\">\n        <div class=\"VS-open-box pull-right\">\n          <button class=\"btn btn-default\">\n            <i class=\"icon-filter\"></i>\n          </button>\n          <div class=\"popup-arrow-up hide\"></div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"row search-box-row hide\">\n    ");
  stack1 = depth0;
  stack2 = "App.MainConfigHistorySearchBoxView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <table class=\"table advanced-header-table table-hover\">\n    <thead>\n    ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "label-row config-history-sort-row";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </thead>\n    <tbody class=\"services-menu\">\n    ");
  stack1 = depth0;
  stack2 = "view.filteringComplete";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(22, program22, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n    <tfoot>\n    <tr>\n      <td colspan=\"5\">\n        ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = {};
  stack4 = "view";
  stack3['dataViewBinding'] = stack4;
  stack4 = "view.filteringComplete";
  stack3['isDataLoadedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </td>\n    </tr>\n    </tfoot>\n  </table>\n\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/edit_widget_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinErrorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMaxErrorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

  data.buffer.push("\n<form autocomplete=\"off\" class=\"form-horizontal\">\n    <div class=\"each-row\">\n        <div class=\"alert alert-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.widgetView.hintInfo";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n    </div>\n\n    <div class=\"row\" id= \"min-height-limit\">\n      <div class=\"col-md-12\">\n        <div id=\"slider-range\"></div>\n      </div>\n    </div>\n\n    <div class=\"row\">\n        <div id=\"slider-value1\" class=\"value-on-slider col-md-2\">\n          <input type=\"text\" value=\"0\" disabled=\"disabled\" class=\"form-control\" />\n        </div>\n        <div id=\"slider-value2\" ");
  stack1 = {};
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinError:slider-error :value-on-slider :col-md-4 view.parentView.sliderHandlersManager.thresholdMinError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.sliderHandlersManager.thresholdMin";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div id=\"slider-value3\" ");
  stack1 = {};
  stack2 = "view.parentView.sliderHandlersManager.thresholdMaxError:slider-error :value-on-slider :col-md-4 view.parentView.sliderHandlersManager.thresholdMaxError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "view.parentView.sliderHandlersManager.thresholdMax";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMaxError";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div id=\"slider-value4\" class=\"value-on-slider col-md-2\">\n          ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.parentView.sliderHandlersManager.maxValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "disabled";
  stack3['disabled'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n    </div>\n\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/edit_widget_popup_single_threshold", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinErrorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n        ");
  return buffer;}

  data.buffer.push("\n\n<form autocomplete=\"off\" class=\"form-horizontal\">\n  <div class=\"each-row\">\n    <div class=\"alert alert-info\">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.widgetView.hintInfo";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n\n  <div class=\"row\" id= \"min-height-limit\">\n    <div class=\"col-md-12\">\n      <div id=\"slider-range\"></div>\n    </div>\n  </div>\n\n  <div class=\"row\">\n    <div id=\"slider-value1\" class=\"value-on-slider col-md-2\">\n      <input type=\"text\" value=\"0\" disabled=\"disabled\" class=\"form-control\" />\n    </div>\n      <div id=\"slider-value2\" ");
  stack1 = {};
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinError:slider-error :value-on-slider :col-md-4 :col-md-offset-2 :col-sm-offset-2 view.parentView.sliderHandlersManager.thresholdMinError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.parentView.sliderHandlersManager.thresholdMin";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.sliderHandlersManager.thresholdMinErrorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    <div id=\"slider-value3\" class=\"value-on-slider col-md-2 col-md-offset-2 col-sm-offset-2\">\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.parentView.sliderHandlersManager.maxValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "disabled";
  stack3['disabled'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n\n</form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/plus_button_filter", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <li class=\"add-widgets-dropdown-list\">\n      <ul>\n        ");
  stack1 = depth0;
  stack2 = "view.hiddenWidgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </li>\n    <li class=\"add-widgets-apply-button\">\n      <button class=\"btn btn-primary pull-right\" ");
  stack1 = depth0;
  stack2 = "applyFilter";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    </li>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li>\n            ");
  stack1 = depth0;
  stack2 = "view.widgetCheckbox";
  stack3 = {};
  stack4 = "checkbox";
  stack3['class'] = stack4;
  stack4 = "widget.checked";
  stack3['checkedBinding'] = stack4;
  stack4 = "widget.title";
  stack3['labelBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </li>\n        ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <li class=\"nothing-to-add\">\n      <span>");
  stack1 = depth0;
  stack2 = "dashboard.widgets.nothing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n    </li>\n  ");
  return buffer;}

  data.buffer.push("\n\n  ");
  stack1 = depth0;
  stack2 = "view.hiddenWidgets.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n\n  <div class=\"btn-toolbar\" role=\"toolbar\">\n    <div id=\"widgets-options-menu\" class=\"widgets-options-menu dropdown btn-group pull-left\">\n      <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n        <span>");
  stack1 = depth0;
  stack2 = "dashboard.widgets.actions.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n        <span class=\"caret\"></span>\n      </button>\n      <ul class=\"dropdown-menu\">\n        <li class=\"add-widgets-text dropdown-submenu pull-left\">\n          <a href=\"javascript:void(null);\"><i class=\"glyphicon glyphicon-plus\"></i> &nbsp; ");
  stack1 = depth0;
  stack2 = "common.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          ");
  stack1 = depth0;
  stack2 = "view.plusButtonFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </li>\n        <li class=\"reset-all-widgets dropdown-submenu pull-left\">\n          <a href=\"javascript:void(null);\"><i class=\"glyphicon glyphicon-cog\"></i> &nbsp; ");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          <ul class=\"dropdown-menu\">\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "resetAllWidgets";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-refresh\"></i> &nbsp; ");
  stack1 = depth0;
  stack2 = "dashboard.button.reset";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n          </ul>\n        </li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "view.timeRangeListView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"clearfix\"></div>\n  </div>\n\n  <div class=\"dashboard-widgets-box\">\n    <div id=\"dashboard-widgets\" class=\"widgets-container dashboard-widgets\">\n      <div class=\"thumbnails\" id=\"sortable\">\n        ");
  stack1 = depth0;
  stack2 = "view.allWidgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.displayedWidgetGroups.length";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "widget.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "widget.viewClass";
  stack3 = {};
  stack4 = "widget";
  stack3['widgetBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.displayedWidgetGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div class=\"dashboard-widget-groups-container\">\n            <div class=\"col-md-12 row\">\n              <div class=\"col-md-6\">\n                <h5 class=\"widgets-group-title\">");
  stack1 = depth0;
  stack2 = "group.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h5>\n                ");
  stack1 = depth0;
  stack2 = "group.subGroups.length";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n              <div class=\"widgets-options-menu col-md-6\">\n                <div class=\"add-widgets-text dropdown btn-group pull-right\">\n                  <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n                    <span>");
  stack1 = depth0;
  stack2 = "common.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    <span class=\"caret\"></span>\n                  </button>\n                  <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "group.allWidgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "groupLayout";
  stack7 = helpers.each;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </div>\n              </div>\n            </div>\n            <div class=\"thumbnails sortable\" ");
  stack1 = {};
  stack2 = "group.name";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "group.allWidgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "groupLayout";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <div class=\"btn-group dropdown\">\n                    <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                      <span class=\"selected-item\">\n                        ");
  stack1 = depth0;
  stack2 = "group.activeSubGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </span>\n                      <span class=\"caret\"></span>\n                    </button>\n                    <ul class=\"dropdown-menu\">\n                      ");
  stack1 = depth0;
  stack2 = "group.subGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                  </div>\n                ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "group.activeSubGroup.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "common.all";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                        ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n                        <li>\n                          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "item.name";
  stack3 = depth0;
  stack4 = "group.subGroups";
  stack5 = depth0;
  stack6 = "setActiveSubGroup";
  stack7 = {};
  stack8 = "view";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                            ");
  stack1 = depth0;
  stack2 = "item.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          </a>\n                        </li>\n                      ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <li class=\"dropdown-submenu pull-left\">\n                        <a href=\"#\">");
  stack1 = depth0;
  stack2 = "groupLayout.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                        ");
  stack1 = depth0;
  stack2 = "view.groupWidgetsFilterView";
  stack3 = {};
  stack4 = "groupLayout.hiddenWidgets";
  stack3['hiddenWidgetsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </li>\n                    ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "groupLayout.isActive";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "groupLayout.widgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "widget.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "widget.viewClass";
  stack3 = {};
  stack4 = "widget";
  stack3['widgetBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(22, program22, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/cluster_metrics", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.childViews.lastObject.hasData";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program2(depth0,data) {
  
  
  data.buffer.push("\n        ");}

  data.buffer.push("\n\n<div class=\"cluster-metrics\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n      <div class=\"caption col-md-10\">");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"corner-icon col-md-1\">\n        <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n        <ul class=\"dropdown-menu\">\n          <li>\n            <button type=\"button\" href=\"#\" ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden :button-border";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "view.exportToCSVArgument";
  stack3 = depth0;
  stack4 = "exportGraphData";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-file-text\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.csv";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </li>\n          <li>\n            <button type=\"button\" href=\"#\" ");
  stack1 = {};
  stack2 = "view.isExportButtonHidden:hidden :button-border";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "exportGraphData";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"icon-file-text-alt\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.json";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </li>\n          <li><button type=\"button\" href=\"#\" class=\"button-border\"\n            ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </li>\n        </ul>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      <div class=\"widget-content\" >\n        ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/hbase_links", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n        <div class=\"widget-content\">\n          <table>\n            <!--hbase master server-->\n            <tr class=\"hbase-master-link\">\n              <td>\n                ");
  stack1 = depth0;
  stack2 = "view.activeMaster";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </td>\n            </tr>\n            <!--region servers-->\n            <tr class=\"hbase-regionserver-link\">\n              <td>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.model.regionServersTotal";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.regionServers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </td>\n            </tr>\n            <!--hbase master Web UI-->\n            <tr class=\"hbase-web-ui-link\">\n              <td>\n                ");
  stack1 = depth0;
  stack2 = "view.activeMaster";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </td>\n            </tr>\n          </table>\n        </div>\n\n        <div class=\"link-button\">\n          ");
  stack1 = depth0;
  stack2 = "view.model.quickLinks.length";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.activeMaster.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.masterServer";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.noMasterServer";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <a ");
  stack1 = {};
  stack2 = "view.hbaseMasterWebUrl";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.masterWebUI";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.QuickLinksView";
  stack3 = {};
  stack4 = "view.model";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"btn-group\">\n                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n                  ");
  stack1 = depth0;
  stack2 = "common.more";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  <span class=\"caret\"></span>\n                </a>\n                <ul class=\"dropdown-menu\">\n                  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n              </div>\n            ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(18, program18, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      <!--there are multiple masters eg, HBase multiple masters or HDFS HA enabled-->\n                      ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "group.links";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "quickLinks";
  stack7 = helpers.each;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          <li class=\"dropdown-submenu\">\n                            <a href=\"javascript:void(null)\">");
  stack1 = depth0;
  stack2 = "quickLinks.publicHostNameLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " &nbsp;</a>\n                            <ul class=\"dropdown-menu\">\n                              ");
  stack1 = depth0;
  stack2 = "quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </ul>\n                          </li>\n                        ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                              ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "view.quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                      ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"links\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n\n      <div class=\"caption col-md-10\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"corner-icon col-md-1\">\n        <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n        <ul class=\"dropdown-menu\">\n          <li><button type=\"button\" href=\"#\" class=\"button-border\"\n            ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </li>\n        </ul>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(23, program23, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/hdfs_links", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n        <table>\n          <!--Active NameNode-->\n          <tr class=\"active-namenode-link\">\n            ");
  stack1 = depth0;
  stack2 = "view.isActiveNNValid";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tr>\n          <!--Standby NameNodes-->\n          <tr class=\"standby-namenode-link\">\n            ");
  stack1 = depth0;
  stack2 = "view.isStandbyNNValid";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tr>\n          <!--Data Nodes-->\n          <tr class=\"datanode-link\">\n            <td>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.model.dataNodesTotal";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </td>\n          </tr>\n        </table>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n              <td><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.activeNameNode.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.HDFSLinks.activeNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <td><a class=\"disabled-hdfs-link\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.HDFSLinks.activeNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n            ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isTwoStandbyNN";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <!--Two Standby NameNodes-->\n                <td><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.twoStandbyComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.HDFSLinks.standbyNameNodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <!--One Standby NameNode-->\n                <td><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.standbyNameNodes.firstObject.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.HDFSLinks.standbyNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n              ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <td><a class=\"disabled-hdfs-link\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.HDFSLinks.standbyNameNode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n            ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n        <table>\n          <!--NameNode-->\n          <tr class=\"namenode-link\">\n            <td><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.model.nameNode.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nanmenode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n          </tr>\n          <!--SecondaryNameNode-->\n          <tr class=\"secondary-namenode-link\">\n            <td><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.model.snameNode.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.snanmenode";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></td>\n          </tr>\n          <!--Data Nodes-->\n          <tr class=\"datanode-link\">\n            <td>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.model.dataNodesTotal";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </td>\n          </tr>\n        </table>\n      ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.QuickLinksView";
  stack3 = {};
  stack4 = "view.model";
  stack3['contentBinding'] = stack4;
  stack4 = "view.masterGroupsArray";
  stack3['masterGroupsBinding'] = stack4;
  stack4 = true;
  stack3['shouldSetGroupedLinks'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"btn-group\">\n            <a class=\"dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n              ");
  stack1 = depth0;
  stack2 = "common.more";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              <span class=\"caret\"></span>\n            </a>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(26, program26, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n          </div>\n        ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(23, program23, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                    <!--there are multiple masters eg, HBase multiple masters or HDFS HA enabled-->\n                    ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "group.links";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "quickLinks";
  stack7 = helpers.each;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <li class=\"dropdown-submenu\">\n                          <a href=\"javascript:void(null)\">");
  stack1 = depth0;
  stack2 = "quickLinks.publicHostNameLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " &nbsp;</a>\n                          <ul class=\"dropdown-menu\">\n                            ");
  stack1 = depth0;
  stack2 = "quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          </ul>\n                        </li>\n                      ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                              <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                            ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                    ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"links\">\n  <ul class=\"list-unstyled\">\n  <li class=\"img-thumbnail row\">\n    <div class=\"caption col-md-10\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"corner-icon col-md-1\">\n      <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n      <ul class=\"dropdown-menu\">\n        <li><button type=\"button\" href=\"#\" class=\"button-border\"\n          ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n        </li>\n      </ul>\n    </div>\n    <div class=\"widget-content\" >\n      ");
  stack1 = depth0;
  stack2 = "view.isHAEnabled";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    </div>\n\n    <div class=\"link-button\">\n      ");
  stack1 = depth0;
  stack2 = "view.model.quickLinks.length";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </li>\n    </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/pie_chart", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"corner-icon col-md-1\">\n          <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n          <ul class=\"dropdown-menu\">\n            <li><button type=\"button\" href=\"#\" class=\"button-border\"\n              ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            </li>\n            <li><button type=\"button\" href=\"#\" class=\"button-border\"\n              ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            </li>\n          </ul>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":hidden-info-general view.hiddenInfoClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <table align=\"center\">\n            ");
  stack1 = depth0;
  stack2 = "view.hiddenInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "line";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </table>\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "view.isPieExist";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "line";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              </tr>\n            ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"widget-content\">\n            ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = {};
  stack4 = "view.model";
  stack3['modelBinding'] = stack4;
  stack4 = "view.thresholdMin";
  stack3['thresholdMinBinding'] = stack4;
  stack4 = "view.thresholdMax";
  stack3['thresholdMaxBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"widget-content-isNA\">");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"has-hidden-info\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n\n      <div class=\"caption col-md-10\">");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  stack1 = depth0;
  stack2 = "view.showActions";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/simple_text", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"corner-icon col-md-1\">\n          <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n          <ul class=\"dropdown-menu\">\n            ");
  stack1 = depth0;
  stack2 = "view.isEditable";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            <li><button type=\"button\" href=\"#\" class=\"button-border\"\n              ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            </li>\n          </ul>\n        </div>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <li>\n                <button type=\"button\" href=\"#\" class=\"button-border\"\n                  ");
  stack1 = depth0;
  stack2 = "editWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <i class=\"icon-pencil\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.edit";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </button>\n              </li>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":hidden-info-general view.hiddenInfoClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <table align=\"center\">\n            ");
  stack1 = depth0;
  stack2 = "view.hiddenInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "line";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </table>\n        </div>\n        <div class=\"widget-content\">");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td>");
  stack1 = depth0;
  stack2 = "line";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              </tr>\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"has-hidden-info\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n\n      <div class=\"caption col-md-10\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  stack1 = depth0;
  stack2 = "view.showActions";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/uptime", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"corner-icon col-md-1\">\n          <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n          <ul class=\"dropdown-menu\">\n            <li><button type=\"button\" href=\"#\" class=\"button-border\"\n              ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n            </li>\n          </ul>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":hidden-info-general view.hiddenInfoClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <table align=\"center\">\n            ");
  stack1 = depth0;
  stack2 = "view.hiddenInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "line";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </table>\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "view.isNA";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr>\n                <td>\n                  ");
  stack1 = depth0;
  stack2 = "line";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n              </tr>\n            ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"widget-content-isNA uptime-content-isNA\">");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"widget-content uptime-content\">\n            ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"has-hidden-info\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n      <div class=\"caption col-md-10\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n      ");
  stack1 = depth0;
  stack2 = "view.showActions";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/dashboard/widgets/yarn_links", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n        <div class=\"widget-content\">\n          <table>\n            <!-- Resource Manager-->\n            <tr class=\"resourcemanager-link\">\n              <td>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.model.resourceManager.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "> ");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.resourceManager";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </td>\n            </tr>\n\n            <!-- Node Manages-->\n            <tr class=\"nodemanager-link\">\n              <td>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "> ");
  stack1 = depth0;
  stack2 = "view.model.nodeManagersTotal";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </td>\n            </tr>\n          </table>\n        </div>\n\n        <div class=\"link-button\">\n          ");
  stack1 = depth0;
  stack2 = "view.model.quickLinks.length";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.QuickLinksView";
  stack3 = {};
  stack4 = "view.model";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"btn-group\">\n                <a class=\"dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n                  ");
  stack1 = depth0;
  stack2 = "common.more";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  <span class=\"caret\"></span>\n                </a>\n                <ul class=\"dropdown-menu\">\n                  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </ul>\n              </div>\n            ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      <!--there are multiple masters eg, HBase multiple masters or HDFS HA enabled-->\n                      ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "group.links";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "quickLinks";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          <li class=\"dropdown-submenu\">\n                            <a href=\"javascript:void(null)\">");
  stack1 = depth0;
  stack2 = "quickLinks.publicHostNameLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " &nbsp;</a>\n                            <ul class=\"dropdown-menu\">\n                              ");
  stack1 = depth0;
  stack2 = "quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </ul>\n                          </li>\n                        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "view.quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <li><a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n                      ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"links\">\n  <ul class=\"list-unstyled\">\n    <li class=\"img-thumbnail row\">\n      <div class=\"caption col-md-10\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"corner-icon col-md-1\">\n        <button class=\"dropdown-toggle ellipsis-menu button-border\" data-toggle=\"dropdown\" href=\"#\"></button>\n        <ul class=\"dropdown-menu\">\n          <li><button type=\"button\" href=\"#\" class=\"button-border\"\n            ");
  stack1 = depth0;
  stack2 = "deleteWidget";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <i class=\"icon-trash\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          </li>\n        </ul>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isDataLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </li>\n  </ul>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.HostTableMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <th>\n          <div class=\"ember-view view-wrapper\">\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "host-select-all";
  stack3['classNames'] = stack4;
  stack4 = "view.parentView.selectAllHosts";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </th>\n        <th> </th>\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.nameSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <th> </th>\n        <th> </th>\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.ipSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.rackSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.cpuSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.memorySort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.diskUsageSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.loadAvgSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <th ");
  stack1 = {};
  stack2 = "App.stackVersionsAvailable::hidden :sort-view-11";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "hosts.host.menu.stackVersions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </th>\n        <th class=\"sort-view-6\">");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.HostView";
  stack3 = {};
  stack4 = "host";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <td>");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "host.selected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n\n          <td class=\"health\">\n            <span rel=\"HealthTooltip\" ");
  stack1 = {};
  stack2 = "host.healthClass host.healthIconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "host.healthToolTip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n          </td>\n\n          <td class=\"name\">\n            <span class=\"trim_hostname\">\n              <a  rel=\"UsageTooltip\" data-original-title=\"");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </span>\n            ");
  stack1 = depth0;
  stack2 = "host.criticalWarningAlertsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </td>\n          <td class=\"restart\">\n            ");
  stack1 = depth0;
  stack2 = "host.componentsWithStaleConfigsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </td>\n          <td class=\"passive-state\">\n              <span rel=\"ComponentsTooltip\" ");
  stack1 = {};
  stack2 = "view.componentsInPassiveStateMessage";
  stack1['data-original-title'] = stack2;
  stack2 = "host.componentsInPassiveStateCount:icon-medkit";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n          </td>\n          <td class=\"host-ip\">");
  stack1 = depth0;
  stack2 = "host.ip";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td class=\"rack-id\">\n            <span rel=\"UsageTooltip\" ");
  stack1 = {};
  stack2 = "host.rack";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.rack";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n          </td>\n          <td class=\"cores-formatted\">");
  stack1 = depth0;
  stack2 = "host.coresFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td class=\"memory-formatted\">");
  stack1 = depth0;
  stack2 = "host.memoryFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n\n          <td class=\"disk-info\">\n            <div class=\"progress\" ");
  stack1 = {};
  stack2 = "host.diskInfoBar";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"UsageTooltip\">\n              <div class=\"progress-bar progress-bar-info\" ");
  stack1 = {};
  stack2 = "view.usageStyle";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n            </div>\n          </td>\n\n          <td class=\"load-avg\">");
  stack1 = depth0;
  stack2 = "host.loadAvg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td ");
  stack1 = {};
  stack2 = "App.stackVersionsAvailable::hidden view.hasSingleVersion:not-active-link :host-table-versions";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" class=\"black\" ");
  stack1 = depth0;
  stack2 = "displayVersions";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "view.currentVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </td>\n          <td ");
  stack1 = {};
  stack2 = "view.hasNoComponents:not-active-link :host-table-components";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" class=\"black\" ");
  stack1 = depth0;
  stack2 = "displayComponents";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "view.content.hostComponents.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.content.hostComponents.length";
  stack3 = {};
  stack4 = "t:common.component";
  stack3['singular'] = stack4;
  stack4 = "t:common.components";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </td>\n        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "host.alertsSummary.CRITICAL";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <span class=\"label alerts-crit-count\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "goToHostAlerts";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.criticalWarningAlertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <span class=\"label alerts-warn-count\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "goToHostAlerts";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.criticalWarningAlertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n              <span class=\"muted glyphicon glyphicon-refresh\" rel=\"ComponentsTooltip\" ");
  stack1 = {};
  stack2 = "view.restartRequiredComponentsMessage";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n            ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td class=\"first\"></td>\n        <td colspan=\"12\">\n          ");
  stack1 = depth0;
  stack2 = "hosts.table.noHosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </td>\n      </tr>\n    ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div class=\"display-inline-block\">\n              <a ");
  stack1 = depth0;
  stack2 = "filterSelected";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "selected-hosts-count";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                ");
  stack1 = depth0;
  stack2 = "view.selectedHosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "view.selectedHostsCount";
  stack3 = {};
  stack4 = "t:hosts.filters.selectedHostInfo";
  stack3['singular'] = stack4;
  stack4 = "t:hosts.filters.selectedHostsInfo";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n              - <a ");
  stack1 = depth0;
  stack2 = "clearSelection";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "clear-hosts-selection";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "  href=\"#\">");
  stack1 = depth0;
  stack2 = "hosts.filters.clearSelection";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n          ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"hosts\" class=\"container-wrap-table\">\n\n  <div class=\"row\">\n    <h2 class=\"table-title col-sm-1\">");
  stack1 = depth0;
  stack2 = "common.hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n    <div class=\"table-controls row col-sm-10 pull-right\">\n      <div class=\"col-sm-12\">\n        ");
  stack1 = depth0;
  stack2 = "HOST.ADD_DELETE_COMPONENTS, HOST.TOGGLE_MAINTENANCE, HOST.ADD_DELETE_HOSTS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <div class=\"VS-open-box pull-right\">\n          <button class=\"btn btn-default\">\n            <i class=\"icon-filter\"></i>\n          </button>\n          <div class=\"popup-arrow-up hide\"></div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"row search-box-row hide\">\n    ");
  stack1 = depth0;
  stack2 = "App.MainHostComboSearchBoxView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n  <table class=\"table advanced-header-table table-hover\" id=\"hosts-table\">\n    <thead>\n      ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "label-row";
  stack3['classNames'] = stack4;
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </thead>\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n    <tfoot>\n      <tr>\n        <td colspan=\"13\">\n          ");
  stack1 = depth0;
  stack2 = "view.showSelectedFilter";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = {};
  stack4 = "view.filteringComplete";
  stack3['isDataLoadedBinding'] = stack4;
  stack4 = "view.rowsPerPageSelectView";
  stack3['rowsPerPageSelectViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </td>\n      </tr>\n    </tfoot>\n  </table>\n\n  <div ");
  stack1 = {};
  stack2 = "view.filteringComplete:hidden :table-overlay";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/add", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "addHost.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step9.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">7</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step10.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/addHost/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <tr ");
  stack1 = {};
  stack2 = "service.serviceId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <td>");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td>\n                <div class=\"col-md-6\">\n                  ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "service.configGroupsNames";
  stack3['contentBinding'] = stack4;
  stack4 = "service.selectedConfigGroup";
  stack3['selectionBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              </td>\n            </tr>\n          ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "addHost.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "addHost.step4.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div class=\"pre-scrollable\">\n        <table class=\"table table-hover\" id=\"host-configurations-table\">\n          <thead>\n          <tr>\n            <th>");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th>");
  stack1 = depth0;
  stack2 = "common.conf.group";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          </tr>\n          </thead>\n          <tbody>\n          ");
  stack1 = depth0;
  stack2 = "controller.content.configGroups";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n        </table>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12 btn-area\">\n  <button class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </button>\n  <button\n    class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n  </button>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/bulk_add_delete_confirm_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <p>");
  stack1 = depth0;
  stack2 = "view.modifyMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n  <div class=\"bulk-host-display\">\n    <pre>");
  stack1 = depth0;
  stack2 = "view.hostsToModify";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <p>");
  stack1 = depth0;
  stack2 = "view.skipMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n  <div class=\"skipped-hosts-panel panel panel-default\" >\n  ");
  stack1 = depth0;
  stack2 = "view.skippedHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div class=\"bulk-host-skipped\">\n      <div class=\"skipped-hosts\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "onToggleHost";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i ");
  stack1 = {};
  stack2 = ":pull-left :icon host.isCollapsed:icon-caret-right:icon-caret-down";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        <a><p>");
  stack1 = depth0;
  stack2 = "host.error.key";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p></a>\n      </div>\n      <div class=\"skipped-hosts-text collapse in\" ");
  stack1 = {};
  stack2 = "host.isBodyVisible";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <p><i>");
  stack1 = depth0;
  stack2 = "host.error.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</i></p>\n      </div>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.hostsToModify";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.skippedHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/bulk_operation_confirm_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "showAll";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "tableView.filters.showAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"alert alert-warning\">\n      ");
  stack1 = depth0;
  stack2 = "view.warningInfo";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br />\n      <pre class=\"limited-height alert alert-warning\"><strong>");
  stack1 = depth0;
  stack2 = "hostNamesSkippedVisible";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></pre>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n<div class=\"task-top-wrap\">\n  <div class=\"task-detail-ico-wrap\">\n    <a href=\"#\" ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "textTrigger";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"task-detail-copy\"><i class=\"glyphicon glyphicon-copy\"></i> ");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n</div>\n<div class=\"task-detail-log-info\">\n  <div class=\"content-area\">\n    <div ");
  stack1 = {};
  stack2 = "view.textareaVisible::hidden :task-detail-log-clipboard-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <textarea class=\"task-detail-log-clipboard\"></textarea>\n    </div>\n    <div ");
  stack1 = {};
  stack2 = "view.textareaVisible:hidden :task-detail-log-maintext";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <pre class=\"limited-height\">");
  stack1 = depth0;
  stack2 = "visibleHosts";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "expanded";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  ");
  stack1 = depth0;
  stack2 = "hostNamesSkipped";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/bulk_operation_menu", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n<ul class=\"dropdown-menu\">\n  ");
  stack1 = depth0;
  stack2 = "HOST.ADD_DELETE_HOSTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <li class=\"dropdown-submenu submenu-left\">\n    <a ");
  stack1 = {};
  stack2 = "view.parentView.showSelectedFilter::disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " tabindex=\"-1\" href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.menuItems.s.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      (");
  stack1 = depth0;
  stack2 = "view.parentView.selectedHosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</a>\n    <div class=\"dropdown-menu-wrap\">\n      <ul ");
  stack1 = {};
  stack2 = "view.parentView.showSelectedFilter::hidden :dropdown-menu";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.hostItemView";
  stack3 = helpers.view;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  </li>\n  <li class=\"dropdown-submenu submenu-left\">\n    <a ");
  stack1 = {};
  stack2 = "view.parentView.hasFilteredItems::disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " tabindex=\"-1\" href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.menuItems.f.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      (");
  stack1 = depth0;
  stack2 = "view.parentView.filteredCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</a>\n    <div class=\"dropdown-menu-wrap\">\n      <ul ");
  stack1 = {};
  stack2 = "view.parentView.hasFilteredItems::hidden :dropdown-menu";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.hostItemView";
  stack3 = helpers.view;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  </li>\n  <li class=\"dropdown-submenu submenu-left\">\n    <a tabindex=\"-1\" href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.menuItems.a.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      (");
  stack1 = depth0;
  stack2 = "view.parentView.totalCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</a>\n    <div class=\"dropdown-menu-wrap\">\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "view.hostItemView";
  stack3 = helpers.view;
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(37, program37, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  </li>\n</ul>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "addHost";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-plus glyphicon-white\"></i> ");
  stack1 = depth0;
  stack2 = "hosts.host.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n    <li class=\"divider\"></li>\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          <div class=\"dropdown-menu-wrap\">\n            <ul class=\"dropdown-menu\">\n              ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "operation.label.length";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.operationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "s";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.slaveItemView";
  stack3 = {};
  stack4 = "component";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "component.componentNameFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            <div class=\"dropdown-menu-wrap\">\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "operation.decommission";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.advancedOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "s";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(13, program13, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.commonOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "s";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          <div class=\"dropdown-menu-wrap\">\n            <ul class=\"dropdown-menu\">\n              ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "operation.label.length";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.operationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "f";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.slaveItemView";
  stack3 = {};
  stack4 = "component";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(24, program24, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "component.componentNameFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            <div class=\"dropdown-menu-wrap\">\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "operation.decommission";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(29, program29, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.advancedOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "f";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(27, program27, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.commonOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "f";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(30, program30, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          <div class=\"dropdown-menu-wrap\">\n            <ul class=\"dropdown-menu\">\n              ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(33, program33, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n        ");
  return buffer;}
function program33(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "operation.label.length";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.operationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "a";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(35, program35, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program35(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program37(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.slaveItemView";
  stack3 = {};
  stack4 = "component";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(38, program38, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program38(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "component.componentNameFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            <div class=\"dropdown-menu-wrap\">\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "view.operationsInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operation";
  stack7 = helpers.each;
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "operation.decommission";
  stack3 = helpers['if'];
  tmp1 = self.program(40, program40, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(43, program43, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program40(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.advancedOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "a";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(41, program41, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                    ");
  return buffer;}

function program43(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "operation.delete";
  stack3 = helpers.unless;
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "view.commonOperationView";
  stack3 = {};
  stack4 = "operation.operationData";
  stack3['contentBinding'] = stack4;
  stack4 = "a";
  stack3['selection'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(45, program45, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program45(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "operation.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      ");
  return buffer;}

  data.buffer.push("\n\n<button class=\"btn btn-success dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\" ");
  stack1 = {};
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.actions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span></button>\n");
  stack1 = depth0;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/combo_search_box", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-danger\">\n    ");
  stack1 = depth0;
  stack2 = "view.errMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

  data.buffer.push("\n\n<div id=\"combo_search_box\"></div>\n");
  stack1 = depth0;
  stack2 = "view.errMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/configs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"row host-configs\">\n        <div class=\"col-md-2\">\n          <div class=\"well side-menu-well\">\n            ");
  stack1 = depth0;
  stack2 = "App.MainHostServiceMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n        <div class=\"col-md-10\">\n          ");
  stack1 = depth0;
  stack2 = "service_config_outlet";
  foundHelper = helpers.outlet;
  stack3 = foundHelper || depth0.outlet;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "outlet", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n    </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"side-menu-well\">\n        <i class=\"message alert alert-info\">");
  stack1 = depth0;
  stack2 = "hosts.host.serviceNotAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</i>\n    </div>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isConfigAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/configs_service", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"clearfix\"></div>\n    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = {};
  stack4 = "view.isConfigsEditable";
  stack3['canEditBinding'] = stack4;
  stack4 = "controller.filter";
  stack3['filterBinding'] = stack4;
  stack4 = "controller.filterColumns";
  stack3['columnsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"serviceConfig non-wizard\">\n  ");
  stack1 = depth0;
  stack2 = "dataIsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/decommission", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n\n<a href=\"javascript:void(null)\" rel=\"decommissionTooltip\" ");
  stack1 = {};
  stack2 = "view.parentView.decommissionTooltipMessage";
  stack1['data-original-title'] = stack2;
  stack2 = "view.parentView.noActionAvailable view.parentView.isComponentDecommissionDisable:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"modal\">\n  ");
  stack1 = depth0;
  stack2 = "view.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</a>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/delete_hosts_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div>");
  stack1 = depth0;
  stack2 = "hosts.bulkOperation.deleteHosts.confirmation.body.msg1";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n<br />\n<div>\n  ");
  stack1 = depth0;
  stack2 = "hosts.bulkOperation.deleteHosts.confirmation.body.msg2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <span class=\"text-danger\">");
  stack1 = depth0;
  stack2 = "hosts.bulkOperation.deleteHosts.confirmation.body.msg3";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n</div>\n\n<div class=\"form-inline align-center\"><br />\n  <label><strong>");
  stack1 = depth0;
  stack2 = "view.typeMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></label>&nbsp;\n  ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.parentView.confirmInput";
  stack3['valueBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br />\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/delete_hosts_result_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.deleteComponents";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <p>");
  stack1 = depth0;
  stack2 = "view.componentName";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  stack1 = depth0;
  stack2 = "hosts.bulkOperation.delete.component.result.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <p>");
  stack1 = depth0;
  stack2 = "hosts.bulkOperation.deleteHosts.result.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div><i>");
  stack1 = depth0;
  stack2 = "deletedHost";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</i></div>\n");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <p>");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n  ");
  stack1 = depth0;
  stack2 = "view.undeletableHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "undeletableHost";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div class=\"panel panel-default\">\n      <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "undeletableHost";
  stack3 = depth0;
  stack4 = "onToggleHost";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i ");
  stack1 = {};
  stack2 = ":pull-left :panel-toggle undeletableHost.isCollapsed:icon-caret-right:icon-caret-down";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n        <a class=\"panel-toggle\">\n          <p>");
  stack1 = depth0;
  stack2 = "undeletableHost.error.key";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n        </a>\n      </div>\n\n      <div class=\"panel-body collapse in\" ");
  stack1 = {};
  stack2 = "undeletableHost.isBodyVisible";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <p>");
  stack1 = depth0;
  stack2 = "undeletableHost.error.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n      </div>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.deletedHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.deletedHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "deletedHost";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<br />\n");
  stack1 = depth0;
  stack2 = "view.undeletableHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div id=\"host-details\">\n       <div class=\"content\">\n      ");
  stack1 = depth0;
  stack2 = "App.MainHostMenuView";
  stack3 = {};
  stack4 = "view.content";
  stack3['hostBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <div class=\"service-button\">\n          <div class=\"btn-group display-inline-block\">\n            <button href=\"javascript:void(null)\" ");
  stack1 = {};
  stack2 = ":btn :btn-default :dropdown-toggle";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " id=\"host-details-action-btn\" data-toggle=\"dropdown\">\n              ");
  stack1 = depth0;
  stack2 = "hosts.host.details.hostActions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              <span class=\"caret\"></span>\n            </button>\n            <ul class=\"dropdown-menu pull-right\">\n              <!-- dropdown menu links -->\n              ");
  stack1 = depth0;
  stack2 = "view.maintenance";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.clients";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <li>\n                <a ");
  stack1 = depth0;
  stack2 = "confirmRecoverHost";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"icon-time\"></i> ");
  stack1 = depth0;
  stack2 = "hosts.host.details.recoverHost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </a>\n              </li>\n            </ul>\n          </div>\n        </div>\n      ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = "controller.isStopDisabled:disabled option.liClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <a ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "doAction";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i ");
  stack1 = {};
  stack2 = "option.cssClass option.liClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i> ");
  stack1 = depth0;
  stack2 = "option.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <li class=\"dropdown-submenu submenu-left\">\n                  <a><i class=\"glyphicon glyphicon-download-alt\"></i> ");
  stack1 = depth0;
  stack2 = "services.service.actions.downloadClientConfigs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n\n                  <div class=\"dropdown-menu-wrap\">\n                    <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "view.hasManyClientsWithConfigs";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "view.clientsWithConfigs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "client";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                  </div>\n                </li>\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      <li>\n                         <a id=\"all-clients-on-host-link\" ");
  stack1 = depth0;
  stack2 = "downloadAllClientConfigs";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = true;
  stack3['href'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.host.details.downloadAllClients";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      </li>\n                    ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                        <li>\n                          <a ");
  stack1 = depth0;
  stack2 = "client";
  stack3 = depth0;
  stack4 = "downloadClientConfigs";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "client.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                        </li>\n                      ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/addDeleteComponentPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"alert alert-danger\">\n    <div class=\"row row-no-pad\">\n      <div class=\"col-md-1 checkbox-colored-background\">");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.parentView.isChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"col-md-11\">");
  stack1 = depth0;
  stack2 = "view.lastComponentError";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n    </div>\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.fromServiceSummary";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.isReconfigureRequired";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div>");
  stack1 = depth0;
  stack2 = "view.selectHostMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"row\">\n      <div class=\"col-md-12\">\n        ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "view.hostsWithoutComponent";
  stack3['contentBinding'] = stack4;
  stack4 = "view.selectedHost";
  stack3['selectionBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.isConfigsLoadingInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.commonMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "controller.hasPropertiesToChange";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.manualKerberosWarning";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.DependentConfigsListView";
  stack3 = {};
  stack4 = false;
  stack3['isAfterRecommendation'] = stack4;
  stack4 = "controller.recommendedPropertiesToChange";
  stack3['recommendationsBinding'] = stack4;
  stack4 = "controller.requiredPropertiesToChange";
  stack3['requiredChangesBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div>");
  stack1 = depth0;
  stack2 = "view.commonMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div>");
  stack1 = depth0;
  stack2 = "view.manualKerberosWarning";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</div>\n  ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.thereIsNoHostsMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.lastComponent";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.anyHostsWithoutComponent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/doDeleteHostPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"row\">\n    <div class=\"col-md-12\">\n      <p>");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.unknownComponents";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<br /><i>");
  stack1 = depth0;
  stack2 = "unknownComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</i></p>\n    </div>\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"alert alert-danger\">\n    <div class=\"row row-no-pad\">\n      <div class=\"col-md-1\">");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "isChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"col-md-10\">");
  stack1 = depth0;
  stack2 = "lastComponentError";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </div>\n");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-warning\">\n    ");
  stack1 = depth0;
  stack2 = "decommissionWarning";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.body.msg.unknownComponents";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-warning\">\n    <!-- Agent is still online, so host record will be re-inserted on next heartbeat. -->\n    ");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.body.msg5";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    <span class=\"text-danger\">");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.body.msg6";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n  </div>\n");
  return buffer;}

  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "unknownComponents";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<p><i class=\"glyphicon glyphicon-warning-sign\"></i> ");
  stack1 = depth0;
  stack2 = "deletePopupBody";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n");
  stack1 = depth0;
  stack2 = "lastComponent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "toDecommissionComponents.length";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "common.important.strong";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "unknownComponents";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.body.msg1";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n\n");
  stack1 = depth0;
  stack2 = "unknownComponents";
  stack3 = helpers.unless;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "common.important.strong";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push(" ");
  stack1 = depth0;
  stack2 = "hosts.delete.popup.body.msg3";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/host_component", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", blockHelperMissing=helpers.blockHelperMissing, helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "HOST.ADD_DELETE_COMPONENTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <a href=\"#\" rel='componentHealthTooltip'\n          ");
  stack1 = {};
  stack2 = "view.upgradeFailedMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "upgradeComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"glyphicon glyphicon-arrow-up\"></i>\n        </a>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n      <i rel='componentHealthTooltip'\n        ");
  stack1 = {};
  stack2 = "view.statusClass view.statusIconClass :components-health";
  stack1['class'] = stack2;
  stack2 = "view.componentTextStatus";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n    ");
  return buffer;}

function program6(depth0,data) {
  
  
  data.buffer.push("\n      <i class=\"text-warning glyphicon glyphicon-refresh\"></i>\n    ");}

function program8(depth0,data) {
  
  
  data.buffer.push("\n      <i class=\"icon-medkit\"></i>\n    ");}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span rel=\"componentNameTooltip\" ");
  stack1 = {};
  stack2 = "component.displayNameAdvanced";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "component.getDisplayNameAdvanced";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;/</span>\n    ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span rel=\"componentNameTooltip\" ");
  stack1 = {};
  stack2 = "component.displayName";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "component.getDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;/</span>\n    ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      - <span></span>");
  stack1 = depth0;
  stack2 = "component.haNameSpace";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"dropdown\">\n      <a ");
  stack1 = {};
  stack2 = ":dropdown-toggle view.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"dropdown\" href=\"javascript:void(null)\">\n        <i class=\"glyphicon glyphicon-option-horizontal action-icon\"></i>\n      </a>\n      ");
  stack1 = depth0;
  stack2 = "isDisabled";
  stack3 = helpers.unless;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n\n        ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.content.isClient";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(23, program23, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <ul class=\"dropdown-menu\">\n            <li>\n              <a href=\"javascript:void(null)\" data-toggle=\"modal\"\n                ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "refreshConfigs";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "hosts.host.details.refreshConfigs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n            <li>\n              <a href=\"javascript:void(null)\" ");
  stack1 = {};
  stack2 = "view.isInit::disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                 data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "installClient";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "common.install";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n            ");
  stack1 = depth0;
  stack2 = "view.isInstallFailed";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.clientCustomCommands";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "command";
  stack7 = helpers.each;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <li>\n                <a href=\"javascript:void(null)\" ");
  stack1 = depth0;
  stack2 = "installClient";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "common.reinstall";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"javascript:void(null)\"\n                  ");
  stack1 = depth0;
  stack2 = "command";
  stack3 = depth0;
  stack4 = "executeCustomCommand";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "command.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n\n          ");
  data.buffer.push("\n          <ul class=\"dropdown-menu\">\n            ");
  stack1 = depth0;
  stack2 = "view.isComponentDecommissionAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isComponentRecommissionAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isRestartableComponent";
  stack3 = helpers['if'];
  tmp1 = self.program(28, program28, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isInstalling";
  stack3 = helpers.unless;
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isInit";
  stack3 = helpers['if'];
  tmp1 = self.program(49, program49, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isDeletableComponent";
  stack3 = helpers['if'];
  tmp1 = self.program(51, program51, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isRefreshConfigsAllowed";
  stack3 = helpers['if'];
  tmp1 = self.program(54, program54, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            ");
  stack1 = depth0;
  stack2 = "view.customCommands";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "command";
  stack7 = helpers.each;
  tmp1 = self.program(56, program56, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.decommissionView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.decommissionView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li ");
  stack1 = {};
  stack2 = "view.isRestartComponentDisabled:hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "restartComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "common.restart";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "SERVICE.START_STOP";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isUpgradeFailed";
  stack3 = helpers['if'];
  tmp1 = self.program(37, program37, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isInstallFailed";
  stack3 = helpers['if'];
  tmp1 = self.program(39, program39, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.isReassignable";
  stack3 = helpers['if'];
  tmp1 = self.program(41, program41, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "HOST.TOGGLE_MAINTENANCE";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(44, program44, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.isStart";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.isStart";
  stack3 = helpers.unless;
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = {};
  stack2 = "view.isComponentDecommissioning:hidden view.noActionAvailable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "stopComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "common.stop";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </li>\n                ");
  return buffer;}

function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.isInit";
  stack3 = helpers.unless;
  tmp1 = self.program(35, program35, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program35(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <li ");
  stack1 = {};
  stack2 = "view.isUpgradeFailed:hidden view.isInstallFailed:hidden view.isDecommissioning:hidden view.noActionAvailable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "startComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        ");
  stack1 = depth0;
  stack2 = "common.start";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                      </a>\n                    </li>\n                  ");
  return buffer;}

function program37(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = "view.noActionAvailable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "upgradeComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "common.reUpgrade";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program39(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = "view.noActionAvailable";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "installComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "common.reinstall";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program41(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "SERVICE.MOVE";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(42, program42, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program42(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = {};
  stack2 = "view.noActionAvailable view.isMoveComponentDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "moveComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "common.move";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </li>\n                ");
  return buffer;}

function program44(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = "view.noActionAvailable view.content.isImpliedState:disabled :allow-tooltip";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = {};
  stack2 = "view.maintenanceTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"passiveTooltip\">\n                  <a href=\"javascript:void(null)\"\n                     data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "toggleMaintenanceMode";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "view.isActive";
  stack3 = helpers['if'];
  tmp1 = self.program(45, program45, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(47, program47, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </a>\n                </li>\n              ");
  return buffer;}
function program45(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "passiveState.turnOn";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  return buffer;}

function program47(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "passiveState.turnOff";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  return buffer;}

function program49(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "installComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "common.reinstall";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program51(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "HOST.ADD_DELETE_COMPONENTS";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(52, program52, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program52(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = "view.isDeleteComponentDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "deleteComponent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "common.delete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n              ");
  return buffer;}

function program54(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "refreshComponentConfigs";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "hosts.host.details.refreshConfigs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </li>\n            ");
  return buffer;}

function program56(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li ");
  stack1 = {};
  stack2 = "command.disabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <a href=\"javascript:void(null)\" ");
  stack1 = depth0;
  stack2 = "command";
  stack3 = depth0;
  stack4 = "executeCustomCommand";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "command.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </li>\n            ");
  return buffer;}

  data.buffer.push("\n\n<td class=\"status-icons\">\n  <span class=\"health-icon-block\">\n    ");
  stack1 = depth0;
  stack2 = "view.isUpgradeFailed";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </span>\n  <span class=\"refresh-icon-block\">\n    ");
  stack1 = depth0;
  stack2 = "component.staleConfigs";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </span>\n  <span class=\"medkit-icon-block\">\n    ");
  stack1 = depth0;
  stack2 = "component.isActive";
  stack3 = helpers.unless;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </span>\n</td>\n\n<td class=\"component-name-block\">\n  <span>\n    ");
  stack1 = depth0;
  stack2 = "component.displayNameAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <span class=\"hidden-lg\"><br/></span>\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "component.service";
  stack3 = depth0;
  stack4 = "routeToService";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "component.service.displayName";
  stack1['title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "component.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </a>\n    ");
  stack1 = depth0;
  stack2 = "component.haNameSpace";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </span>\n</td>\n\n<td class=\"component-type-block\">\n  <span>");
  stack1 = depth0;
  stack2 = "view.typeDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n</td>\n\n<td class=\"align-center\">\n  ");
  stack1 = depth0;
  stack2 = "SERVICE.DECOMMISSION_RECOMMISSION";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</td>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/installComponentPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "installComponentMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br /><br />");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/raiseDeleteComponentErrorPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"warning-details\">\n          ");
  stack1 = depth0;
  stack2 = "componentsBodyEnd";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"dialog-delete-component\">\n  <div class=\"warning row row-no-pad\">\n    <div class=\"col-md-1\">\n      <i class=\"glyphicon glyphicon-warning-sign\"></i>\n    </div>\n    <div class=\"col-md-11 warning-list\">\n      <p><strong>");
  stack1 = depth0;
  stack2 = "componentsBody";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong><br /><em>");
  stack1 = depth0;
  stack2 = "componentsStr";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</em></p>\n      ");
  stack1 = depth0;
  stack2 = "showBodyEnd";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/recoverHostErrorPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"dialog-delete-component\">\n  <div class=\"warning\">\n    <i class=\"icon-warning-sign\"></i> ");
  stack1 = depth0;
  stack2 = "recoverErrorPopupBody";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/details/recoverHostPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n\n<div class=\"alert\">\n  ");
  stack1 = depth0;
  stack2 = "hosts.recover.popup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/host_alerts", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n\n    <table class=\"table advanced-header-table table-hover alerts-table\" id=\"host-alerts-table\">\n      <thead>\n      ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <tr class=\"filter-row\">\n        <th class=\"first\">");
  stack1 = depth0;
  stack2 = "view.serviceFilterView";
  stack3 = {};
  stack4 = "service-filter";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "view.nameFilterView";
  stack3 = {};
  stack4 = "name-filter";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "view.stateFilterView";
  stack3 = {};
  stack4 = "status-filter";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>");
  stack1 = depth0;
  stack2 = "view.textView";
  stack3 = {};
  stack4 = "text-filter";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n      </tr>\n      </thead>\n      <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n      <tfoot>\n        <tr>\n          <td colspan=\"4\">\n            ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n      </tfoot>\n    </table>\n\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.serviceSort";
  stack3 = {};
  stack4 = "first service-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.nameSort";
  stack3 = {};
  stack4 = "name-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.statusSort";
  stack3 = {};
  stack4 = "status-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.textSort";
  stack3 = {};
  stack4 = "text-sorting";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "instance";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <tr>\n            <td class=\"first\">\n              ");
  stack1 = depth0;
  stack2 = "instance.isAmbariServiceName";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </td>\n            <td>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "instance.definitionId";
  stack3 = depth0;
  stack4 = "routeToAlertDefinition";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "instance.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </td>\n            <td>\n              ");
  stack1 = depth0;
  stack2 = "templates/main/alerts/alert_instance/status";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </td>\n            <td>\n              <span class=\"alert-text\" ");
  stack1 = {};
  stack2 = "instance.escapeSpecialCharactersFromTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "instance.text";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n            </td>\n          </tr>\n        ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "instance.service";
  stack3 = depth0;
  stack4 = "routeToService";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "instance.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td class=\"first\">\n            ");
  stack1 = depth0;
  stack2 = "alerts.table.noAlerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n          <td colspan=\"3\"></td>\n        </tr>\n      ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"host-alerts\">\n  <div class=\"row\">\n    <div class=\"col-md-12\">\n    <h2 class=\"table-title\">");
  stack1 = depth0;
  stack2 = "hosts.host.alerts.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n    </div>\n    <div class=\"col-md-12\">\n  ");
  stack1 = depth0;
  stack2 = "App.router.mainAlertInstancesController.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/log_metrics", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div class=\"col-md-6 text-center mtl\">\n      ");
  stack1 = depth0;
  stack2 = "view.chartView";
  stack3 = {};
  stack4 = "item";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "transitionByService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row log-metrics-charts mtl\">\n  ");
  stack1 = depth0;
  stack2 = "view.logsData";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/logs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.serviceNameSort";
  stack3 = {};
  stack4 = "first";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.componentNameSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.fileExtensionsSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.logFileRowView";
  stack3 = {};
  stack4 = "row";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <td class=\"first\">");
  stack1 = depth0;
  stack2 = "row.serviceDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td>");
  stack1 = depth0;
  stack2 = "row.componentDisplayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          <td>\n            ");
  stack1 = depth0;
  stack2 = "row.fileNamesObject";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "file";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </td>\n        ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8;
  data.buffer.push("\n              <div>\n                <a ");
  stack1 = depth0;
  stack2 = "file.filePath";
  stack3 = depth0;
  stack4 = "row";
  stack5 = depth0;
  stack6 = "openLogFile";
  stack7 = {};
  stack8 = "view.parentView";
  stack7['target'] = stack8;
  stack8 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack7;
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack8.call(depth0, stack6, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" rel=\"log-file-name-tooltip\" ");
  stack1 = {};
  stack2 = "file.filePath";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "file.fileName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                ");
  stack1 = depth0;
  stack2 = "App.LogSearchUILinkView";
  stack3 = {};
  stack4 = "file.linkTail";
  stack3['linkQueryParamsBinding'] = stack4;
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n              ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <a ");
  stack1 = {};
  stack2 = "view.formatedLink";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"log-file-name-tooltip noopener noreferrer\" ");
  stack1 = {};
  stack2 = "popup.logTail.openInLogSearch";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right external-link\">\n                    <i class=\"icon-external-link\"></i>\n                    ");
  stack1 = depth0;
  stack2 = "popup.logTail.openInLogSearch";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  <h2 class=\"table-title col-sm-1\">");
  stack1 = depth0;
  stack2 = "hosts.host.menu.logs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n</div>\n<table class=\"table advanced-header-table table-hover\">\n  <thead>\n    ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <tr class=\"filter-row\">\n      <th class=\"first service-name-filter\">");
  stack1 = depth0;
  stack2 = "view.serviceNameFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th class=\"component-name-filter\">");
  stack1 = depth0;
  stack2 = "view.componentNameFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th class=\"file-extension-filter\">");
  stack1 = depth0;
  stack2 = "view.fileExtensionsFilter";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n  </thead>\n  <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n  <tfoot>\n  <tr>\n    <td colspan=\"3\">\n      ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </td>\n  </tr>\n  </tfoot>\n</table>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/metrics", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"host-metrics\">\n  <div class=\"graphs\">\n    <div class=\"row\">\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsCPU";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsDisk";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsLoad";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsMemory";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n    <div class=\"row\">\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsNetwork";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"col-md-6\">\n        ");
  stack1 = depth0;
  stack2 = "App.ChartHostMetricsProcesses";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/rack_id_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <span class=\"help-block validation-block\">");
  stack1 = depth0;
  stack2 = "view.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"host-rack-id-popup\">\n  <div ");
  stack1 = {};
  stack2 = ":form-group :list-inline view.isValid::has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <span>");
  stack1 = depth0;
  stack2 = "hostPopup.RackId";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":&nbsp;</span>\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.parentView.rackId";
  stack3['valueBinding'] = stack4;
  stack4 = "input-xlarge form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/stack_versions", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.stackSort";
  stack3 = {};
  stack4 = "first";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.repoVersionSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.statusSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <th></th>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "version";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td class=\"first stack-name\">");
  stack1 = depth0;
  stack2 = "version.stack";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td class=\"repo-version\">");
  stack1 = depth0;
  stack2 = "version.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td class=\"host-stack-version-status\">\n          ");
  stack1 = depth0;
  stack2 = "version.isCurrent";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n        <td class=\"install-repo-version align-center\">\n          ");
  stack1 = depth0;
  stack2 = "AMBARI.MANAGE_STACK_VERSIONS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n      </tr>\n    ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <span class=\"label label-success\">");
  stack1 = depth0;
  stack2 = "common.current";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "version.isInstalling";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "version";
  stack3 = depth0;
  stack4 = "showInstallProgress";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-cog in_progress\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "version.displayStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "version.displayStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "version.isOutOfSync";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.outOfSyncInfo";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "version";
  stack3 = depth0;
  stack4 = "installVersionConfirmation";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "version.installDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-off\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.install";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n          ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td colspan=\"4\" class=\"first\">\n        ");
  stack1 = depth0;
  stack2 = "hosts.host.stackVersions.table.noVersions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </td>\n    </tr>\n  ");
  return buffer;}

  data.buffer.push("\n<div class=\"row\">\n  <h2 class=\"table-title col-sm-1\">");
  stack1 = depth0;
  stack2 = "hosts.host.menu.stackVersions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n</div>\n<table class=\"table advanced-header-table table-hover\">\n  <thead>\n  ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "label-row";
  stack3['classNames'] = stack4;
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <tr class=\"filter-row\">\n    <th class=\"first stack-name-filter\">");
  stack1 = depth0;
  stack2 = "view.stackFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th class=\"repo-version-filter\">");
  stack1 = depth0;
  stack2 = "view.repoVersionFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th class=\"status-filter\">");
  stack1 = depth0;
  stack2 = "view.statusFilterView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</th>\n    <th> </th>\n  </tr>\n  </thead>\n  <tbody>\n  ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n  <tfoot>\n  <tr>\n    <td colspan=\"4\">\n      ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </td>\n  </tr>\n  </tfoot>\n</table>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/host/summary", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.componentsWithStaleConfigsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"alert alert-warning clearfix restart-required\">\n      <i class=\"glyphicon glyphicon-refresh\"></i> ");
  stack1 = depth0;
  stack2 = "view.needToRestartMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <button ");
  stack1 = {};
  stack2 = ":btn :restart-components :pull-right :btn-warning";
  stack1['class'] = stack2;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "restartAllStaleConfigComponents";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "hosts.host.details.needToRestart.button";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n    </div>\n  ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"dropdown pull-right\">\n                <button id=\"add_component\" data-toggle=\"dropdown\" ");
  stack1 = {};
  stack2 = "view.addComponentDisabled:disabled :btn :btn-default :dropdown-toggle";
  stack1['class'] = stack2;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <span class=\"glyphicon glyphicon-plus\"></span>&nbsp;");
  stack1 = depth0;
  stack2 = "add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </button>\n                ");
  stack1 = depth0;
  stack2 = "App.router.wizardWatcherController.isNonWizardUser";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "view.addableComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                      <li>\n                        <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "component";
  stack3 = depth0;
  stack4 = "addComponentWithCheck";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                          ");
  stack1 = depth0;
  stack2 = "component.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        </a>\n                      </li>\n                    ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.sortedComponents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "component.viewClass";
  stack3 = {};
  stack4 = "component";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <a href=\"#\" class=\"set-rack-id\" ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "setRackId";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i\n                    class=\"glyphicon glyphicon-pencil\"></i></a>\n                ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"panel panel-default\">\n          <div class=\"panel-heading\">\n            <div class=\"row\">\n              <div class=\"col-md-6\"><h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.hostMetrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4></div>\n              <div class=\"col-md-6\">");
  stack1 = depth0;
  stack2 = "view.timeRangeListView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n          </div>\n          <div class=\"panel-body\">\n            ");
  stack1 = depth0;
  stack2 = "view.hasHostMetricsService";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.hasNameNode";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.MainHostMetricsView";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              <div class=\"namenode-metrics dashboard-widgets\">\n                <h5>\n                  ");
  stack1 = depth0;
  stack2 = "App.isHaEnabled";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </h5>\n                <div class=\"thumbnails\">\n                  ");
  stack1 = depth0;
  stack2 = "view.nameNodeWidgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              </div>\n            ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <span class=\"text-uppercase\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.nameSpace";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>");
  stack1 = depth0;
  stack2 = "view.nameNodeComponent.haNameSpace";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push(": ");
  stack1 = depth0;
  stack2 = "view.nameNodeComponent.haNameSpace";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "widget";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"panel panel-default\">\n          <div class=\"panel-heading\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.hostLogMetrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n          <div class=\"panel-body\">\n              ");
  stack1 = depth0;
  stack2 = "App.MainHostLogMetrics";
  stack3 = {};
  stack4 = "view.content";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "SERVICE.MODIFY_CONFIGS, SERVICE.START_STOP, HOST.ADD_DELETE_COMPONENTS, HOST.TOGGLE_MAINTENANCE";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div class=\"row host-block\">\n  <div class=\"col-md-6\">\n      ");
  data.buffer.push("\n    <div class=\"panel panel-default\">\n      <div class=\"panel-heading\">\n        <div class=\"row\">\n          <div class=\"col-md-6\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n          <div class=\"col-md-6\">\n            ");
  stack1 = depth0;
  stack2 = "HOST.ADD_DELETE_COMPONENTS";
  foundHelper = helpers.havePermissions;
  stack3 = foundHelper || depth0.havePermissions;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n      <div class=\"host-components panel-body\">\n        <table class=\"table\">\n          <thead>\n            <th>");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th>");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th>");
  stack1 = depth0;
  stack2 = "common.type";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            <th class=\"align-center\">");
  stack1 = depth0;
  stack2 = "common.action";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          </thead>\n          <tbody>\n          ");
  stack1 = depth0;
  stack2 = "view.sortedComponents.length";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n        </table>\n      </div>\n    </div>\n    <div class=\"host-configuration\">\n      <div class=\"panel panel-default\">\n        <div class=\"panel-heading\">\n          <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n        </div>\n        <div class=\"host-summary-content panel-body\">\n          <div class=\"host-details-summary-list\">\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-hostname-label\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.hostname";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-hostname-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-ipaddress-label\">");
  stack1 = depth0;
  stack2 = "common.ipAddress";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-ipaddress-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.ip";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-rack-label\">");
  stack1 = depth0;
  stack2 = "common.rack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-rack-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.rack";
  foundHelper = helpers.formatWordBreak;
  stack3 = foundHelper || depth0.formatWordBreak;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatWordBreak", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "HOST.TOGGLE_MAINTENANCE";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-os-label\">");
  stack1 = depth0;
  stack2 = "common.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-os-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "&nbsp;(");
  stack1 = depth0;
  stack2 = "view.content.osArch";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-cores-label\">");
  stack1 = depth0;
  stack2 = "common.cores.cpu";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-cores-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.coresFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-disk-label\">");
  stack1 = depth0;
  stack2 = "common.disk";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-disk-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.diskInfoBar";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-memory-label\">");
  stack1 = depth0;
  stack2 = "common.memory";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-memory-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.memoryFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-load-avg-label\">");
  stack1 = depth0;
  stack2 = "common.loadAvg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-load-avg-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.loadAvg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-agent-heart-label\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.agentHeartbeat";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-agent-heart-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.timeSinceHeartBeat";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label summary-cur-ver-label\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.currentVersion";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-cur-ver-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.currentVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n            <div class=\"row row-no-pad\">\n              <div class=\"col-md-4 summary-label\">");
  stack1 = depth0;
  stack2 = "hosts.host.summary.agentHasJce";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</div>\n              <div class=\"col-md-8 summary-agent-jce-value\">&nbsp;");
  stack1 = depth0;
  stack2 = "view.content.hasJcePolicy";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            </div>\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n  <div class=\"col-md-6\">\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.showHostMetricsBlock";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.supports.logCountVizualization";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-12 summary-width\">\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/add", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep5Disabled:disabled view.isStep5Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep5";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep7:active view.isStep6Disabled:disabled view.isStep6Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep6";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step9.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep8:active view.isStep7Disabled:disabled view.isStep7Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep7";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">7</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "installer.step10.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/all_services_actions", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\" aria-expanded=\"false\">\n    <span class=\"glyphicon glyphicon-option-horizontal\"></span>\n  </div>\n  <ul class=\"dropdown-menu\">\n    ");
  stack1 = depth0;
  stack2 = "SERVICE.ADD_DELETE_SERVICES";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "SERVICE.START_STOP";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <li>\n        <a href=\"#\" id=\"all-clients-of-cluster-link\"\n          ");
  stack1 = depth0;
  stack2 = "downloadAllClientConfigs";
  stack3 = {};
  stack4 = "view.serviceController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-download-alt";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n          ");
  stack1 = depth0;
  stack2 = "services.service.downloadAllClientConfigs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </a>\n      </li>\n  </ul>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.supports.enableAddDeleteServices";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <li ");
  stack1 = {};
  stack2 = "view.serviceController.isAllServicesInstalled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <a href=\"#\"\n          ");
  stack1 = {};
  stack2 = "view.serviceController.isAllServicesInstalled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "gotoAddService";
  stack3 = {};
  stack4 = "view.serviceController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i class=\"glyphicon glyphicon-plus\"></i> ");
  stack1 = depth0;
  stack2 = "services.service.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </li>\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <li class=\"divider\"></li>\n      <li ");
  stack1 = {};
  stack2 = "view.serviceController.isStartAllDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <a href=\"#\" data-toggle=\"modal\"\n          ");
  stack1 = {};
  stack2 = "view.serviceController.isStartAllDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "startAllService";
  stack3 = {};
  stack4 = "view.serviceController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-play view.serviceController.isStartAllDisabled:disabled:enabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n          ");
  stack1 = depth0;
  stack2 = "services.service.startAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </a>\n      </li>\n      <li ");
  stack1 = {};
  stack2 = "view.serviceController.isStopAllDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <a href=\"#\" data-toggle=\"modal\"\n          ");
  stack1 = {};
  stack2 = "view.serviceController.isStopAllDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "stopAllService";
  stack3 = {};
  stack4 = "view.serviceController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-stop view.serviceController.isStopAllDisabled:disabled:enabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n          ");
  stack1 = depth0;
  stack2 = "services.service.stopAll";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </a>\n      </li>\n      <li ");
  stack1 = {};
  stack2 = "view.serviceController.isRestartAllRequiredDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <a href=\"#\" data-toggle=\"modal\"\n          ");
  stack1 = {};
  stack2 = "view.serviceController.isRestartAllRequiredDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "restartAllRequired";
  stack3 = {};
  stack4 = "view.serviceController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <i ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-repeat view.serviceController.isRestartAllRequiredDisabled:disabled:enabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n          ");
  stack1 = depth0;
  stack2 = "services.service.restartAllRequired";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </a>\n      </li>\n    ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "SERVICE.START_STOP, SERVICE.ADD_DELETE_SERVICES";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/configs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "controller.content.isRestartRequired";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"clearfix\"></div>\n    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = {};
  stack4 = "controller.filter";
  stack3['filterBinding'] = stack4;
  stack4 = "controller.filterColumns";
  stack3['columnsBinding'] = stack4;
  stack4 = "controller.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "SERVICE.START_STOP";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <div>\n          <div class=\"alert alert-warning clearfix restart-required\">\n            <i class=\"glyphicon glyphicon-refresh\"></i> ");
  stack1 = depth0;
  stack2 = "services.service.config.restartService.needToRestart";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "  <a\n                  href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "showComponentsShouldBeRestarted";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.componentsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.componentsCount";
  stack3 = {};
  stack4 = "t:common.component";
  stack3['singular'] = stack4;
  stack4 = "t:common.components";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "</a> ");
  stack1 = depth0;
  stack2 = "on";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "showHostsShouldBeRestarted";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.hostsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.hostsCount";
  stack3 = {};
  stack4 = "t:common.host";
  stack3['singular'] = stack4;
  stack4 = "t:common.hosts";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            <span class=\"restart-components pull-right\">&nbsp</span>\n\n            <div class=\"btn-group pull-right\">\n              <button type=\"button\" class=\"btn dropdown-toggle btn-warning\" data-toggle=\"dropdown\">\n                ");
  stack1 = depth0;
  stack2 = "hosts.host.details.needToRestart.button";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                <span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                <li class=\"restart-all-components\">\n                  <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "restartAllStaleConfigComponents";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "restart.service.all.affected";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                </li>\n                ");
  stack1 = depth0;
  stack2 = "view.rollingRestartSlaveComponentName";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n        </div>\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li class=\"restart-slave-components\">\n                    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.rollingRestartSlaveComponentName";
  stack3 = depth0;
  stack4 = "rollingRestartStaleConfigSlaveComponents";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.rollingRestartActionName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"serviceConfig\" class=\"non-wizard\">\n  ");
  stack1 = depth0;
  stack2 = "dataIsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/configs_save_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"pre-scrollable\" style=\"max-height: 250px;\">\n    <ul>\n    ");
  stack1 = depth0;
  stack2 = "view.getDisplayMessage";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "val";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <li>\n        ");
  stack1 = depth0;
  stack2 = "val";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </li>\n    ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.runningHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.unknownHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <i class=\"glyphicon glyphicon-warning-sign\"></i>  ");
  stack1 = depth0;
  stack2 = "view.runningHostsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <table class=\"table table-hover\">\n        <thead>\n          <tr><th>");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th><th>");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th></tr>\n        </thead>\n        <tbody>\n          ");
  stack1 = depth0;
  stack2 = "view.runningHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n  ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <tr><td>");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td><td>");
  stack1 = depth0;
  stack2 = "host.components";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td></tr>\n          ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <i class=\"glyphicon glyphicon-warning-sign\"></i>  ");
  stack1 = depth0;
  stack2 = "view.unknownHostsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <table class=\"table table-hover\">\n        <thead>\n          <tr><th>");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th><th>");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th></tr>\n        </thead>\n        <tbody>\n          ");
  stack1 = depth0;
  stack2 = "view.unknownHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n  ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <tr><td>");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td><td>");
  stack1 = depth0;
  stack2 = "host.components";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td></tr>\n          ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = "view.messageClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n");
  stack1 = depth0;
  stack2 = "view.flag";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/confirm_delete_service", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.parentView.message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<div class=\"form-inline align-center\"><br />\n  <label><strong>");
  stack1 = depth0;
  stack2 = "view.typeMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></label>&nbsp;\n  ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.parentView.confirmInput";
  stack3['valueBinding'] = stack4;
  stack4 = "input-sm form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "<br />\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/delete_service_warning_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "controller.changedProperties.length";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.DependentConfigsListView";
  stack3 = {};
  stack4 = "controller.changedProperties";
  stack3['recommendationsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  stack1 = depth0;
  stack2 = "controller.isRecommendationInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/dependent_services_warning", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <li>");
  stack1 = depth0;
  stack2 = "serviceName";
  foundHelper = helpers.formatRole;
  stack3 = foundHelper || depth0.formatRole;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatRole", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n  ");
  return buffer;}

  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "view.parentView.dependentMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n<ul>\n  ");
  stack1 = depth0;
  stack2 = "view.parentView.dependentServices";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "serviceName";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</ul>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/heatmap_dropdown", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <li class=\"heatmap-metrics-dropdown-links\">\n        <a tabindex=\"-1\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "heatmap";
  stack3 = depth0;
  stack4 = "showHeatMapMetric";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "heatmap.widget_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n      </li>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"dropdown\" id=\"select-metric-btn-group\">\n  <button class=\"btn btn-default btn-block dropdown-toggle heatmap-toggle-metrics-btn\" data-toggle=\"dropdown\">\n    ");
  stack1 = depth0;
  stack2 = "charts.heatmap.selectMetric";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <span class=\"caret\"></span>\n  </button>\n\n  <ul class=\"dropdown-menu\">\n    ");
  stack1 = depth0;
  stack2 = "controller.allHeatmaps";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "heatmap";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/metric_graphs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <tr>\n      ");
  stack1 = depth0;
  stack2 = "graphs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "graph";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tr>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <td>\n          <div class=\"\">\n            ");
  stack1 = depth0;
  stack2 = "graph";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </td>\n      ");
  return buffer;}

  data.buffer.push("\n<table class=\"graphs\">\n  ");
  stack1 = depth0;
  stack2 = "view.serviceMetricGraphs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "graphs";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</table>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/metrics", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"service-metrics-block\">\n    <div class=\"panel panel-default\">\n      <div class=\"panel-heading\">\n        <div class=\"row\">\n          <div class=\"col-md-7 col-lg-7\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "services.service.metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n          <div class=\"col-md-5 col-lg-5\">\n            ");
  stack1 = depth0;
  stack2 = "showTimeRangeControl";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "isServiceWithEnhancedWidgets";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n      <div class=\"panel-body service-widgets-box\">\n        ");
  stack1 = depth0;
  stack2 = "isServiceWithEnhancedWidgets";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        <table class=\"graphs\">\n          ");
  stack1 = depth0;
  stack2 = "view.serviceMetricGraphs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "graphs";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </table>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "controller.isHDFSFederatedSummary";
  stack3 = helpers['if'];
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.timeRangeListView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <div class=\"btn-group pull-right actions\">\n                  <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                    ");
  stack1 = depth0;
  stack2 = "common.actions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &nbsp;<span class=\"caret\"></span>\n                  </button>\n                  <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "view.widgetActions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </div>\n              ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <li ");
  stack1 = {};
  stack2 = "option.layouts:dropdown-submenu";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        ");
  stack1 = depth0;
  stack2 = "option.isAction";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      </li>\n                    ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "option.action";
  stack3 = depth0;
  stack4 = "doWidgetAction";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                            <i ");
  stack1 = {};
  stack2 = "option.class";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                            ");
  stack1 = depth0;
  stack2 = "option.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          </a>\n                          ");
  stack1 = depth0;
  stack2 = "option.layouts";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                            <ul class=\"dropdown-menu\">\n                              ");
  stack1 = depth0;
  stack2 = "option.layouts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "layout";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </ul>\n                          ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                                <li>\n                                  <a href=\"javascript:void(0);\">\n                                    ");
  stack1 = depth0;
  stack2 = "layout.layoutName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                  </a>\n                                </li>\n                              ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div id=\"widget_layout\" class=\"thumbnails\">\n            ");
  stack1 = depth0;
  stack2 = "controller.widgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "isAmbariMetricsInstalled";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <div class=\"widget span2p4\" ");
  stack1 = {};
  stack2 = "widget.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "widget.viewClass";
  stack3 = {};
  stack4 = "widget";
  stack3['contentBinding'] = stack4;
  stack4 = "widget.id";
  stack3['idBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n            ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <div class=\"span2p4\">\n                <button id=\"add-widget-action-box\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "goToWidgetsBrowser";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        rel=\"add-widget-tooltip\" ");
  stack1 = {};
  stack2 = "dashboard.widgets.addButton.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  <i class=\"glyphicon glyphicon-plus\"></i></button>\n              </div>\n            ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <tr>\n              ");
  stack1 = depth0;
  stack2 = "graphs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "graph";
  stack7 = helpers.each;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tr>\n          ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <td>\n                  <div>\n                    ");
  stack1 = depth0;
  stack2 = "graph";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>\n                </td>\n              ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "controller.selectedNSWidgetLayout";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div class=\"widgets-group-select-wrapper\">\n            <h5 class=\"widgets-group-title\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.nameSpace";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h5>\n            <div class=\"btn-group dropdown\">\n              <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                <span class=\"selected-item\">");
  stack1 = depth0;
  stack2 = "controller.selectedNSWidgetLayout.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                <span class=\"caret\"></span>\n              </button>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "controller.activeNSWidgetLayouts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n          <div class=\"panel-body service-widgets-box\">\n            <div id=\"ns_widget_layout\" class=\"thumbnails\">\n              ");
  stack1 = depth0;
  stack2 = "controller.selectedNSWidgetLayout.widgets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "widget";
  stack7 = helpers.each;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        ");
  return buffer;}
function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li>\n                    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "switchNameServiceLayout";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "option.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </li>\n                ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <div class=\"widget span2p4\" ");
  stack1 = {};
  stack2 = "widget.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "widget.viewClass";
  stack3 = {};
  stack4 = "controller.selectedNSWidgetLayout";
  stack3['nsLayoutBinding'] = stack4;
  stack4 = "widget";
  stack3['contentBinding'] = stack4;
  stack4 = "widget.id";
  stack3['idBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.serviceHasMetrics";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/metrics/flume/flume_agent_metrics_section", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.metricView";
  stack3 = {};
  stack4 = "view.metricViewData";
  stack3['viewDataBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div ");
  stack1 = {};
  stack2 = ":panel view.toggleIndex";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"panel-group box\">\n    <div class=\"panel-heading box-header\">\n      <div class=\"pull-left panel-toggle metrics-header\"\n            data-toggle=\"collapse\" ");
  stack1 = {};
  stack2 = "view.toggleIndex";
  stack1['dataParent'] = stack2;
  stack2 = "view.toggleIndex";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <h4>\n          <i class=\"pull-left glyphicon\"></i>\n          ");
  stack1 = depth0;
  stack2 = "view.header";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </h4>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.timeRangeListView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"clearfix\"></div>\n    </div>\n    <div class=\"panel-body collapse in\" ");
  stack1 = {};
  stack2 = "view.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.metricView";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/metrics/hive/hive_quick_links", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n    <div class=\"caption title col-md-11\"> ");
  stack1 = depth0;
  stack2 = "view.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n\n    <div class=\"content\">\n      <table>\n        <tr class=\"grafana-link\">\n          <td>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.model.nameNode.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.service.hive.grafana";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </td>\n        </tr>\n        <tr class=\"hive-interactive-server-link\">\n          <td>\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.model.snameNode.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.service.hive.interactive.server";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </td>\n        </tr>\n      </table>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"widget span2p4\">\n  <div class=\"img-thumbnail quick-link-widget\">\n\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/save_popup_footer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"modal-footer\">\n  <button type=\"button\" class=\"btn btn-default\"\n    ");
  stack1 = depth0;
  stack2 = "onCancel";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </button><button type=\"button\" class=\"btn btn-default\"\n    ");
  stack1 = depth0;
  stack2 = "onDiscard";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.discard";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </button><button type=\"button\" class=\"btn btn-success\"\n    ");
  stack1 = {};
  stack2 = "view.isSaveDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "onSave";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/service_alert_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <ul id='summary-alerts-popup' class=\"alerts list-unstyled\">\n    ");
  stack1 = depth0;
  stack2 = "view.alerts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "alert";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <li class=\"alert-list-wrap\">\n        <div class=\"row\">\n          <div class=\"col-md-8 name-text\">\n            <div>\n              <a class=\"accordion-toggle\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "alert";
  stack3 = depth0;
  stack4 = "gotoAlertDetails";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "alert.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n            <div class=\"definition-latest-text\" ");
  stack1 = {};
  stack2 = "alert.latestText";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "alert.latestTextSummary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          </div>\n          <div class=\"col-md-4 status-col\" rel=\"alert-status-tooltip\" >\n            <div class=\"display-inline-block\">");
  stack1 = depth0;
  stack2 = "App.AlertDefinitionSummary";
  stack3 = {};
  stack4 = "alert";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            <span class=\"timeago\" ");
  stack1 = {};
  stack2 = "alert.lastTriggeredVerboseDisplay";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <span class=\"status-icon\">");
  stack1 = depth0;
  stack2 = "alert.status";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span>\n              <time>");
  stack1 = depth0;
  stack2 = "alert.lastTriggeredForFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</time>\n            </span>\n          </div>\n        </div>\n      </li>\n    ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "services.service.summary.alerts.noAlerts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.alerts.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "SERVICE.START_STOP";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <div id=\"summary-restart-bar\">\n      <div class=\"alert alert-warning clearfix restart-required\">\n        <i class=\"glyphicon glyphicon-refresh\"></i> ");
  stack1 = depth0;
  stack2 = "view.needToRestartMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push(" ");
  stack1 = depth0;
  stack2 = "services.service.config.restartService.needToRestart";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "showComponentsShouldBeRestarted";
  stack5 = {};
  stack6 = "App.router.mainServiceInfoConfigsController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.componentsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.componentsCount";
  stack3 = {};
  stack4 = "t:common.component";
  stack3['singular'] = stack4;
  stack4 = "t:common.components";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "</a> ");
  stack1 = depth0;
  stack2 = "on";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "showHostsShouldBeRestarted";
  stack5 = {};
  stack6 = "App.router.mainServiceInfoConfigsController";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.hostsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.hostsCount";
  stack3 = {};
  stack4 = "t:common.host";
  stack3['singular'] = stack4;
  stack4 = "t:common.hosts";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        <span class=\"restart-components pull-right\">&nbsp</span>\n\n        <div class=\"btn-group pull-right\">\n          <button type=\"button\" class=\"btn dropdown-toggle btn-warning\" data-toggle=\"dropdown\">\n            ");
  stack1 = depth0;
  stack2 = "hosts.host.details.needToRestart.button";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <span class=\"caret\"></span>\n          </button>\n          <ul class=\"dropdown-menu\">\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "restartAllStaleConfigComponents";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "restart.service.all.affected";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n            ");
  stack1 = depth0;
  stack2 = "view.rollingRestartSlaveComponentName";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </div>\n      </div>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li>\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.rollingRestartSlaveComponentName";
  stack3 = depth0;
  stack4 = "rollingRestartStaleConfigSlaveComponents";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.rollingRestartActionName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </li>\n            ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <span ");
  stack1 = depth0;
  stack2 = "controller.content";
  stack3 = depth0;
  stack4 = "showServiceAlertsPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"pull-right\">\n                <i class=\"glyphicon glyphicon-bell\"></i>\n                <span ");
  stack1 = {};
  stack2 = ":label view.hasCriticalAlerts:alerts-crit-count view.hasNonCriticalAlertsOnly:alerts-warn-count view.hasNoAlerts:alerts-none-count";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.alertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              </span>\n            ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "view.serviceSummaryView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <div class=\"panel-heading\">\n          <div class=\"row col-md-8 col-lg-12\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "common.quickLinks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n        </div>\n        <div class=\"panel-body\">\n          ");
  stack1 = depth0;
  stack2 = "view.showQuickLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(29, program29, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(27, program27, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <!--there are multiple masters eg, HBase multiple masters or HDFS HA enabled-->\n                ");
  stack1 = depth0;
  stack2 = "view.quickLinksArray";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <div ");
  stack1 = depth0;
  stack2 = "{group.title}-quick-links";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "group.title";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "group.links";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "quickLinks";
  stack7 = helpers.each;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <h5 class=\"ellipsis-overflow-nowrap\"\n                        ");
  stack1 = {};
  stack2 = "group.title";
  stack1['data-title'] = stack2;
  stack2 = "view.tooltipAttribute";
  stack1['rel'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        ");
  stack1 = depth0;
  stack2 = "group.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </h5>\n                    ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <h6>");
  stack1 = depth0;
  stack2 = "quickLinks.publicHostNameLabel";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h6>\n                      ");
  stack1 = depth0;
  stack2 = "quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.quickLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(25, program25, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.quickLinks";
  stack3 = helpers.each;
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <a ");
  stack1 = {};
  stack2 = "url";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " target=\"_blank\" rel=\"noopener noreferrer\">");
  stack1 = depth0;
  stack2 = "label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div class=\"alert alert-danger\">\n                    <small>");
  stack1 = depth0;
  stack2 = "quick.links.error.quicklinks.unavailable.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</small>\n                  </div>\n                ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "view.showNoLinks";
  stack3 = helpers['if'];
  tmp1 = self.program(30, program30, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(32, program32, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program30(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "common.noLink";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"row\">\n    <div class=\"col-md-12 metrics-collapsed-graphs\">\n      ");
  stack1 = depth0;
  stack2 = "view.collapsedSections";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "collapsedSection";
  stack7 = helpers.each;
  tmp1 = self.program(35, program35, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program35(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "collapsedSection";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.service.isRestartRequired";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div class=\"service-block\">\n  <div class=\"summary-links-block\">\n    <div class=\"panel panel-default summary-block\">\n      <div class=\"panel-heading summary-box-header\">\n        <div class=\"row\">\n          <div class=\"col-md-8 col-lg-8\">\n            <h4 class=\"panel-title\">");
  stack1 = depth0;
  stack2 = "controller.content.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "services.service.info.menu.summary";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n          </div>\n          <div class=\"col-md-4 col-lg-4 service-alerts\">\n            ");
  stack1 = depth0;
  stack2 = "view.hasAlertDefinitions";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n      <div class=\"panel-body\">\n        ");
  stack1 = depth0;
  stack2 = "view.serviceSummaryView";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n    <div class=\"panel panel-default quick-links-block\">\n      ");
  stack1 = depth0;
  stack2 = "App.QuickLinksView";
  stack3 = {};
  stack4 = "view.svc";
  stack3['contentBinding'] = stack4;
  stack4 = "view.mastersObj";
  stack3['masterGroupsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  data.buffer.push("\n      <!--div class=\"panel-heading\">\n         <div class=\"row col-md-8 col-lg-12\">\n           <h4 class=\"panel-title\">");
  data.buffer.push("</h4>\n         </div>\n      </div>\n      <div class=\"panel-body\">\n       ");
  data.buffer.push("\n         ");
  data.buffer.push("\n           <a href=\"#\" ");
  data.buffer.push(">");
  data.buffer.push("</a>\n         ");
  data.buffer.push("\n       ");
  data.buffer.push("\n         <span>");
  data.buffer.push("</span>\n       ");
  data.buffer.push("\n      </div-->\n    </div>\n  </div>\n</div>\n\n");
  stack1 = depth0;
  stack2 = "view.collapsedSections";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/base", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row component-summary\">\n  ");
  stack1 = depth0;
  stack2 = "App.SummaryMasterComponentsView";
  stack3 = {};
  stack4 = "view.parentView.mastersObj";
  stack3['mastersCompBinding'] = stack4;
  stack4 = "view.parentView.componentCommonWidgetsView";
  stack3['componentCommonWidgetsViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div class=\"col-md-12\">\n    <div class=\"col-md-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.showComponentsTitleForNonMasters";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div class=\"col-md-10\">\n      ");
  stack1 = depth0;
  stack2 = "App.SummarySlaveComponentsView";
  stack3 = {};
  stack4 = "view.parentView.slavesObj";
  stack3['slavesObjBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.SummaryClientComponentsView";
  stack3 = {};
  stack4 = "view.parentView.clientObj";
  stack3['clientsObjBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/client_components", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n  <div class=\"row col-md-3\">\n    <div ");
  stack1 = {};
  stack2 = ":summary-value :main-info clientComponent.summaryValueClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span class=\"green-live\">");
  stack1 = depth0;
  stack2 = "clientComponent.installedCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>  ");
  stack1 = depth0;
  stack2 = "common.installed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div ");
  stack1 = {};
  stack2 = ":summary-label clientComponent.summaryLabelClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <a ");
  stack1 = depth0;
  stack2 = "clientComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"javascript:void(null)\" >\n        ");
  stack1 = depth0;
  stack2 = "clientComponent.displayNamePluralized";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </div>\n  </div>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.clientsObj";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "clientComponent";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/hdfs/common_widgets", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  
  data.buffer.push("\n      <div class=\"row\"></div>\n    ");}

  data.buffer.push("\n\n");
  data.buffer.push("\n<div class=\"col-md-12 metrics-summary not-separate-section\">\n  <div class=\"col-md-2\">\n    ");
  stack1 = depth0;
  stack2 = "view.isFederatedHDFS";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"col-md-10\">\n    ");
  stack1 = depth0;
  stack2 = "view.isNonFederatedHDFS";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n    <div class=\"row dfs-usage col-md-3\">\n      <div class=\"summary-value\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsUsedDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.dfsUsedDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.dfsUsed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    <div class=\"row non-dfs-used col-md-3\">\n      <div class=\"summary-value main-info\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.nonDfsUsedDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.nonDfsUsedDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.nonDfsUsed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    <div class=\"row capacity-remaining col-md-3\">\n      <div class=\"summary-value\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.remainingDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.remainingDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.remaining";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/hdfs/slaves", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.dataNodeComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div class=\"summary-label\">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.dataNodeComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <span>\n              ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.dataNodesStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.dataNodesTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </span>\n            ");
  stack1 = depth0;
  stack2 = "common.started";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.journalNodeComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isComponentsStateLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div class=\"summary-label\">\n          <a\n            href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.journalNodeComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.journalnodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <span>\n              ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.journalNodesLive";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.journalNodesTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </span>\n            <span> ");
  stack1 = depth0;
  stack2 = "common.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </span>\n          ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n      <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.nfsGatewayComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n        <div class=\"summary-label\">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nfsGatewayComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nfsgateways";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    ");
  return buffer;}
function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <span>\n              ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.nfsGatewaysStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.nfsGatewaysTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(17, program17, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </span>\n            ");
  stack1 = depth0;
  stack2 = "common.started";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}
function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"main-info\">\n            ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"main-info\" rel=\"tooltip\"\n            ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.live";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.service.liveDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"main-info\">\n            ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"main-info\" rel=\"tooltip\"\n            ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.dead";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.service.deadDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.dead";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"main-info\">\n            ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <span class=\"main-info\" rel=\"tooltip\"\n            ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.decommission";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.service.decommissionDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.decom";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"col-md-12\">\n  <div class=\"col-md-2\">\n    ");
  stack1 = depth0;
  stack2 = "view.showTitle";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"col-md-10\">\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isDataNodeCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isJournalNodeCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.isNfsInStack";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  data.buffer.push("\n    <div class=\"row\"></div>\n    <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodecounts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  data.buffer.push("\n    <div class=\"row datanode-count-live col-md-3\">\n      <div class=\"summary-value\">\n        ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(23, program23, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row datanode-count-dead col-md-3\">\n      <div class=\"summary-value\">\n        ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(27, program27, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row datanode-count-decommission col-md-3\">\n      <div class=\"summary-value\">\n        ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(31, program31, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/hdfs/widgets", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.slaveComponentsView";
  stack3 = {};
  stack4 = "view.parentView.summaryView";
  stack3['summaryViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"col-md-12\">\n  <div class=\"col-md-10 col-md-offset-2\">\n    ");
  data.buffer.push("\n    <div class=\"row namenode-uptime col-md-3\">\n      <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.nodeUptime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.uptime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row namenode-heap col-md-3\">\n      <div class=\"summary-value\">\n        <div class=\"main-info\"> ");
  stack1 = depth0;
  stack2 = "view.nodeHeapPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n        <div class=\"info-desc\"> ");
  stack1 = depth0;
  stack2 = "view.nodeHeap";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.heap";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </div>\n</div>\n\n");
  stack1 = depth0;
  stack2 = "view.showSlaveComponents";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n");
  data.buffer.push("\n<div ");
  stack1 = {};
  stack2 = ":col-md-12 :metrics-summary view.isNonFederatedHDFS::not-separate-section";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <div class=\"col-md-2\">\n    ");
  stack1 = depth0;
  stack2 = "view.isNonFederatedHDFS";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"col-md-10\">\n    <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "services.service.summary.blocks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  data.buffer.push("\n    <div class=\"row blocks-total col-md-3\">\n      <div class=\"summary-value main-info\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsTotalBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.total";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row block-errors-corrupt col-md-3\">\n      <div class=\"summary-value main-info\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsCorruptBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.corrupt";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row block-errors-missing col-md-3\">\n      <div class=\"summary-value main-info\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsMissingBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.missing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row block-errors-replicated col-md-3\">\n      <div class=\"summary-value main-info\">\n        <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsUnderReplicatedBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.replicated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row\"></div>\n    ");
  data.buffer.push("\n    <div class=\"row total-files-dirs col-md-3\">\n      <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsTotalFiles";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.totalFilesAndDirs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row upgrade-status col-md-3\">\n      <div class=\"summary-value main-info\">\n        <span ");
  stack1 = {};
  stack2 = "view.isUpgradeStatusWarning:upgrade-status-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.upgradeStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "services.service.summary.pendingUpgradeStatus";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n    ");
  data.buffer.push("\n    <div class=\"row safe-mode-status col-md-3\">\n      <div class=\"summary-value main-info\">\n        ");
  stack1 = depth0;
  stack2 = "view.safeModeStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n      <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "services.service.summary.safeModeStatus";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/master_components", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "group.components.length";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"col-md-12\">\n      <div class=\"col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "group.title";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "group.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "comp";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "group.componentWidgetsView";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"master-group-title\" rel=\"MasterGroupTitleTooltip\" ");
  stack1 = {};
  stack2 = "group.title";
  stack1['data-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "group.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = "comp.isSubComponent:component-small :component :col-md-3 comp.componentName :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <div ");
  stack1 = {};
  stack2 = ":summary-value comp.summaryValueClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <span rel='SummaryComponentHealthTooltip'\n                ");
  stack1 = {};
  stack2 = "comp.statusClass comp.statusIconClass";
  stack1['class'] = stack2;
  stack2 = "comp.passiveTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n              <span class=\"main-info\">");
  stack1 = depth0;
  stack2 = "comp.componentTextStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              ");
  stack1 = depth0;
  stack2 = "comp.alertsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div ");
  stack1 = {};
  stack2 = ":summary-label comp.summaryLabelClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "comp.host";
  stack3 = depth0;
  stack4 = "showDetails";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-original-title=\"");
  stack1 = depth0;
  stack2 = "comp.publicHostName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\"\n                 rel=\"UsageTooltip\">\n                ");
  stack1 = depth0;
  stack2 = "comp.displayNameAdvanced";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <span ");
  stack1 = depth0;
  stack2 = "comp";
  stack3 = depth0;
  stack4 = "showServiceAlertsPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = {};
  stack2 = ":label comp.hasCriticalAlerts:alerts-crit-count:alerts-warn-count";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "comp.alertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "comp.displayNameAdvanced";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "comp.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "group.componentWidgetsView";
  stack3 = {};
  stack4 = "view.slaveComponentsView";
  stack3['slaveComponentsViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.componentCommonWidgetsView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.mastersComp";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.componentCommonWidgetsView";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/info/summary/slave_components", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n  <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 slaveComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div ");
  stack1 = {};
  stack2 = ":summary-value :main-info slaveComponent.summaryValueClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "slaveComponent.startedCount";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "slaveComponent.totalCount";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "dashboard.services.summary.slaves.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div ");
  stack1 = {};
  stack2 = ":summary-label slaveComponent.summaryLabelClassName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "slaveComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "slaveComponent.displayNamePluralized";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.slavesObj";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "slaveComponent";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/item", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, blockHelperMissing=helpers.blockHelperMissing;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n<div class=\"service-button\">\n  ");
  stack1 = depth0;
  stack2 = "view.isMaintenanceActive";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"btn-group display-inline-block\">\n      <button class=\"btn btn-success dropdown-toggle\" id=\"service-actions-dropdown-btn\" data-toggle=\"dropdown\" href=\"#\">\n        ");
  stack1 = depth0;
  stack2 = "common.actions";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <span class=\"caret\"></span>\n      </button>\n      <ul class=\"pull-right dropdown-menu\">\n        <!-- dropdown menu links -->\n\n        ");
  stack1 = depth0;
  stack2 = "view.maintenance.length";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <!-- Service actions -->\n          ");
  stack1 = depth0;
  stack2 = "view.maintenance";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "option.isHidden";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li ");
  stack1 = {};
  stack2 = "option.disabled option.hasSubmenu:dropdown-submenu option.hasSubmenu:submenu-left";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <a ");
  stack1 = depth0;
  stack2 = "option";
  stack3 = depth0;
  stack4 = "doAction";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = {};
  stack2 = "option.tooltip";
  stack1['data-title'] = stack2;
  stack2 = "view.tooltipAttribute";
  stack1['rel'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <i ");
  stack1 = {};
  stack2 = "option.cssClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                  ");
  stack1 = depth0;
  stack2 = "option.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n                ");
  stack1 = depth0;
  stack2 = "option.hasSubmenu";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </li>\n            ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <div class=\"dropdown-menu-wrap\">\n                    <ul class=\"dropdown-menu\">\n                      ");
  stack1 = depth0;
  stack2 = "option.submenuOptions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                  </div>\n                ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                        <li>\n                          <a class=\"ellipsis-overflow\" ");
  stack1 = depth0;
  stack2 = "item";
  stack3 = depth0;
  stack4 = "doAction";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            ");
  stack1 = {};
  stack2 = "item.tooltip";
  stack1['data-title'] = stack2;
  stack2 = "view.tooltipAttribute";
  stack1['rel'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                            ");
  stack1 = depth0;
  stack2 = "item.context.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          </a>\n                        </li>\n                      ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "App.MainServiceInfoMenuView";
  stack3 = {};
  stack4 = "view.hasConfigTab";
  stack3['configTabBinding'] = stack4;
  stack4 = "view.hasHeatmapTab";
  stack3['heatmapTabBinding'] = stack4;
  stack4 = "view.hasMetricTab";
  stack3['metricTabBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n");
  stack1 = depth0;
  stack2 = "SERVICE.RUN_CUSTOM_COMMAND, SERVICE.RUN_SERVICE_CHECK, SERVICE.START_STOP, SERVICE.TOGGLE_MAINTENANCE, SERVICE.ENABLE_HA";
  foundHelper = helpers.isAuthorized;
  stack3 = foundHelper || depth0.isAuthorized;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  if(foundHelper && typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else { stack1 = blockHelperMissing.call(depth0, stack3, stack2, tmp1); }
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/manage_configuration_groups_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <div class=\"row manage-configuration-group-content\">\n    <div class=\"col-md-4 col-lg-4 group-list\">\n      <span>&nbsp;</span>\n      ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "configGroups";
  stack3['contentBinding'] = stack4;
  stack4 = "content.displayNameHosts";
  stack3['optionLabelPath'] = stack4;
  stack4 = "view.selectedConfigGroup";
  stack3['selectionBinding'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "group-select form-control form-group";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"btn-toolbar pull-right\">\n        <button rel=\"button-info\" class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "view.addButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "addConfigGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-plus\"></i></button>\n        <button rel=\"button-info\" class=\"btn btn-default\" ");
  stack1 = {};
  stack2 = "view.removeButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = "view.isRemoveButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "confirmDelete";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-minus\"></i></button>\n        <div class=\"btn-group\">\n          <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n            <i class=\"glyphicon glyphicon-cog\"></i>&nbsp;<span class=\"caret\"></span>\n          </button>\n          <ul class=\"dropdown-menu\">\n            <li ");
  stack1 = {};
  stack2 = "view.isRenameButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"\" rel=\"button-info-dropdown\" ");
  stack1 = {};
  stack2 = "view.renameButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "renameConfigGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "services.service.config_groups_popup.rename";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n            <li ");
  stack1 = {};
  stack2 = "view.isDuplicateButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"\" rel=\"button-info-dropdown\" ");
  stack1 = {};
  stack2 = "view.duplicateButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "duplicateConfigGroup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "services.service.config_groups_popup.duplicate";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </li>\n          </ul>\n        </div>\n      </div>\n    </div>\n    <div class=\"col-md-8 col-lg-8 host-list\">\n      <span>&nbsp;</span>\n      ");
  stack1 = depth0;
  stack2 = "Em.Select";
  stack3 = {};
  stack4 = "selectedConfigGroup.hosts";
  stack3['contentBinding'] = stack4;
  stack4 = "multiple";
  stack3['multiple'] = stack4;
  stack4 = "group-select form-control form-group";
  stack3['class'] = stack4;
  stack4 = "selectedHosts";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <div class=\"clearfix\">\n        <div class=\"pull-right btn-toolbar\">\n          <a rel=\"button-info\" ");
  stack1 = {};
  stack2 = "view.addHostTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = ":btn :btn-default selectedConfigGroup.isAddHostsDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "addHosts";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ><i class=\"glyphicon glyphicon-plus\"></i></a>\n          <a rel=\"button-info\" ");
  stack1 = {};
  stack2 = "view.removeHostTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = ":btn :btn-default isDeleteHostsDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "deleteHosts";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ><i class=\"glyphicon glyphicon-minus\"></i></a>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"col-md-3 col-lg-3 align-right\">");
  stack1 = depth0;
  stack2 = "common.overrides";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"col-md-9 col-lg-9\">\n          <a href=\"#\" class=\"properties-link\" ");
  stack1 = depth0;
  stack2 = "showProperties";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = {};
  stack2 = "tooltipText";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "selectedConfigGroup.properties.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "common.properties";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n      <div class=\"row\">\n        <div class=\"col-md-3 col-lg-3 align-right\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"col-md-9 col-lg-9 group-description\">");
  stack1 = depth0;
  stack2 = "selectedConfigGroup.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n  </div>\n  <div class=\"row\">\n    <div class=\"col-md-12 col-lg-12 text-danger\" id=\"manage-config-group-error-div\">\n      ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  
  data.buffer.push("\n        &nbsp;\n      ");}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-12 col-lg-12\">\n    <p class=\"alert alert-info\">");
  stack1 = depth0;
  stack2 = "services.service.config_groups_popup.notice";
  stack3 = {};
  stack4 = "view.displayName";
  stack3['serviceNameBinding'] = stack4;
  foundHelper = helpers['t'];
  stack4 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "controller.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/menu_item", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <i ");
  stack1 = {};
  stack2 = "view.content.isStarted:started:stopped :icon-circle :status-icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n  ");
  return buffer;}

  data.buffer.push("\n\n<a class=\"services-menu-blocks\" ");
  stack1 = {};
  stack2 = "view.link";
  stack1['href'] = stack2;
  stack2 = "view.dataHref";
  stack1['data-href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n  <span class=\"pull-right alerts-block\">\n    <span ");
  stack1 = {};
  stack2 = ":label view.noAlerts:hidden view.hasCriticalAlerts:alerts-crit-count:alerts-warn-count";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.alertsCountDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </span>\n  </span>\n  <span class=\"pull-right\">\n    <i rel=\"tooltip\" ");
  stack1 = depth0;
  stack2 = "goToConfigs";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = ":glyphicon :glyphicon-refresh :restart-required-service view.content.isRestartRequired::hidden";
  stack1['class'] = stack2;
  stack2 = "view.restartRequiredMessage";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n  </span>\n  <span class=\"pull-right\">\n    <i rel=\"tooltip\" ");
  stack1 = {};
  stack2 = ":icon-medkit :passive-state-service view.content.isInPassive::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "services.service.summary.inMaintenanceMode.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n  </span>\n  ");
  stack1 = depth0;
  stack2 = "view.isClientOnlyService";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <span ");
  stack1 = {};
  stack2 = "view.isClientOnlyService:client-only-service :menu-item-name view.content.workStatus view.isMasterDown:master-down";
  stack1['class'] = stack2;
  stack2 = "view.content.toolTipContent";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"serviceHealthTooltip\">\n      ");
  stack1 = depth0;
  stack2 = "view.content.displayName";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </span>\n</a>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/new_config_group", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  
  data.buffer.push("\n     &nbsp;\n    ");}

  data.buffer.push("\n\n<div class=\"new-config-group-div\">\n  <div class=\"text-warning\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.warningMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"row\">\n    <div class=\"col-md-12\">\n      <form autocomplete=\"off\" class=\"form-horizontal\">\n        <div class=\"form-group\">\n          <label class=\"col-md-2 control-label\">");
  stack1 = depth0;
  stack2 = "services.service.config_groups_popup.group_name_lable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":&nbsp;</label>\n          <div class=\"col-md-6\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "255";
  stack3['maxlength'] = stack4;
  stack4 = "configGroupName";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n          </div>\n        </div>\n        <div class=\"form-group\">\n          <label class=\"col-md-2 control-label\">");
  stack1 = depth0;
  stack2 = "services.service.config_groups_popup.group_desc_lable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":&nbsp;</label>\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "col-md-12 form-control";
  stack3['classNames'] = stack4;
  stack4 = "configGroupDesc";
  stack3['valueBinding'] = stack4;
  stack4 = "4";
  stack3['rows'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      </form>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <li ");
  stack1 = {};
  stack2 = "isStep5:active view.isStep5Disabled:disabled view.isStep5Completed:completed :step5-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">5</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n          <li ");
  stack1 = {};
  stack2 = "isStep6:active view.isStep6Disabled:disabled view.isStep6Completed:completed :step6-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">6</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed :step1-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed :step2-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed :step3-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep4:active view.isStep4Disabled:disabled view.isStep4Completed:completed :step4-link";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep4";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">4</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "services.reassign.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        ");
  stack1 = depth0;
  stack2 = "controller.content.hasManualSteps";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n  ");
  stack1 = depth0;
  stack2 = "view.isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "services.reassign.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "message";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </p>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"alert alert-warning\">\n      ");
  stack1 = depth0;
  stack2 = "view.jdbcSetupMessage";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "isLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "stepConfigs.length";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div id=\"serviceConfig\">\n              ");
  stack1 = depth0;
  stack2 = "services.reassign.step3.configs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "services.reassign.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "services.reassign.step3.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  ");
  stack1 = depth0;
  stack2 = "view.jdbcSetupMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"step8-content\" class=\"well pre-scrollable\">\n        <div id=\"step8-info\">\n          <table id=\"reassign-review-table\">\n            <tr>\n              <td><b>");
  stack1 = depth0;
  stack2 = "services.reassign.step3.component";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b></td>\n              <td colspan=\"2\">");
  stack1 = depth0;
  stack2 = "controller.content.reassign.display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n            </tr>\n            <tr>\n              <td><b>");
  stack1 = depth0;
  stack2 = "services.reassign.step3.sourceHost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b></td>\n              <td>");
  stack1 = depth0;
  stack2 = "view.sourceHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td><span class=\"to-be-disabled-red\"><i class=\"glyphicon glyphicon-minus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeDeleted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n            <tr>\n              <td><b>");
  stack1 = depth0;
  stack2 = "services.reassign.step3.targetHost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</b></td>\n              <td>");
  stack1 = depth0;
  stack2 = "view.targetHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td><span class=\"to-be-installed-green\"><i class=\"glyphicon glyphicon-plus\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "admin.highAvailability.wizard.step3.toBeInstalled";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span></td>\n            </tr>\n          </table>\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "wizardController.isComponentWithReconfiguration";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n      <button class=\"btn pull-left\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = {};
  stack4 = "true";
  stack3['href'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button class=\"btn btn-success pull-right\"\n              id=\"spinner\" ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.deploy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step5", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<div class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "services.reassign.step5.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "view.manualCommands";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  data.buffer.push("\n      ");
  data.buffer.push("\n      ");
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step6", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reassign/step7", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "templates/common/progress";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/reconfigure", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "command.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <tr>\n        <td>\n          ");
  stack1 = depth0;
  stack2 = "App.StageLabelView";
  stack3 = {};
  stack4 = "command";
  stack3['commandBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n        <td>\n          ");
  stack1 = depth0;
  stack2 = "command.isCompleted";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n        <td>\n          ");
  stack1 = depth0;
  stack2 = "command.isError";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n      </tr>\n    ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <p ");
  stack1 = {};
  stack2 = "view.isStarted::faintText";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "command.index";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ".&nbsp");
  stack1 = depth0;
  stack2 = "command.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "command.isSuccess";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.StageSuccessView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "command.isError";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.StageFailureView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "command.isStarted";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "command.isPolling";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <div class=\"progress-wrapper pull-left\">\n                  ");
  stack1 = depth0;
  stack2 = "App.StageInProgressView";
  stack3 = {};
  foundHelper = helpers.command;
  stack4 = foundHelper || depth0.command;
  stack3['commandBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n                <div class=\"progress-percentage pull-left\">");
  stack1 = depth0;
  stack2 = "command.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</div>\n              ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <a class=\"btn btn-primary\"\n               href=\"javascript:void(null)\" ");
  stack1 = depth0;
  stack2 = "retry";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i\n                    class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>\n              ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          ");
  return buffer;}

  data.buffer.push("\n\n<table id=\"security-stages\" class=\"table no-borders\">\n  <tbody>\n  ");
  stack1 = depth0;
  stack2 = "controller.commands";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "command";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/service", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<div class=\"service-content\">\n  <div class=\"summary-info\">\n    ");
  stack1 = depth0;
  stack2 = "yield";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/flume", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.hostSort";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <th>");
  stack1 = depth0;
  stack2 = "dashboard.services.flume.agent";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        <th>\n          <i class=\"icon icon-signin\" ");
  stack1 = {};
  stack2 = "dashboard.services.flume.sources";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i> ");
  stack1 = depth0;
  stack2 = "dashboard.services.flume.sources";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </th>\n        <th>\n          <i class=\"icon icon-random\" ");
  stack1 = {};
  stack2 = "dashboard.services.flume.channels";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i> ");
  stack1 = depth0;
  stack2 = "dashboard.services.flume.channels";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </th>\n        <th>\n          <i class=\"icon icon-signout\" ");
  stack1 = {};
  stack2 = "dashboard.services.flume.sinks";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i> ");
  stack1 = depth0;
  stack2 = "dashboard.services.flume.sinks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </th>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <tr ");
  stack1 = {};
  stack2 = "host.isActive:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <td class=\"agent-host-name\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "selectHost";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <a href=\"javascript:void(null)\">\n                ");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </td>\n            <td colspan=\"4\">\n              <table class=\"table table-hover table-striped\">\n                <tbody>\n                  ");
  stack1 = depth0;
  stack2 = "host.agents";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "agent";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </tbody>\n              </table>\n            </td>\n          </tr>\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <tr ");
  stack1 = {};
  stack2 = "host.isActive:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                      <td class=\"agent-status\">\n                        <div class=\"pull-left\">\n                          <span ");
  stack1 = {};
  stack2 = "agent.healthClass agent.healthIconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span> ");
  stack1 = depth0;
  stack2 = "agent.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        </div>\n                        <div class=\"flume-agents-actions pull-right\">\n                          <div class=\"btn-group display-inline-block\">\n                            <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\">\n                              ");
  stack1 = depth0;
  stack2 = "agent.displayStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              <span class=\"caret\"></span>\n                            </button>\n                            <ul class=\"pull-left dropdown-menu\">\n                              <li ");
  stack1 = {};
  stack2 = "agent.isStartAgentDisabled:disabled :start-agent";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                <a href=\"javascript:void(null)\"\n                                  ");
  stack1 = {};
  stack2 = "agent.isStartAgentDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                  ");
  stack1 = depth0;
  stack2 = "agent";
  stack3 = depth0;
  stack4 = "startFlumeAgent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                  ");
  stack1 = depth0;
  stack2 = "services.service.summary.flume.startAgent";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                              </li>\n                              <li ");
  stack1 = {};
  stack2 = "agent.isStopAgentDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                <a href=\"javascript:void(null)\"\n                                  ");
  stack1 = {};
  stack2 = "agent.isStopAgentDisabled:disabled :stop-agent";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                  ");
  stack1 = depth0;
  stack2 = "agent";
  stack3 = depth0;
  stack4 = "stopFlumeAgent";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                  ");
  stack1 = depth0;
  stack2 = "services.service.summary.flume.stopAgent";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                              </li>\n                            </ul>\n                          </div>\n                        </div>\n                      </td>\n                      <td class=\"flume-agent-sources-count\">\n                        ");
  stack1 = depth0;
  stack2 = "agent.sourcesCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </td>\n                      <td class=\"flume-agent-channels-count\">\n                        ");
  stack1 = depth0;
  stack2 = "agent.channelsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </td>\n                      <td class=\"flume-agent-sink-count\">\n                        ");
  stack1 = depth0;
  stack2 = "agent.sinksCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      </td>\n                    </tr>\n                  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <tr>\n          <td colspan=\"5\" class=\"empty-label\">\n            ");
  stack1 = depth0;
  stack2 = "services.service.summary.flume.noAgents";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          </td>\n        </tr>\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  data.buffer.push("\n<div id=\"flume-summary\" class=\"row\">\n  <div class=\"col-md-12\">\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.flumeHandlerComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.summaryHeader";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    <a href=\"#\" class=\"pull-right\" ");
  stack1 = depth0;
  stack2 = "gotoConfigs";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.flume.summary.configure";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n  </div>\n  <div id=\"flume-agent-table-wrap\" class=\"scrollable-container col-md-12\">\n    <table class=\"table advanced-header-table\" id=\"flume-agents-table\">\n      <thead>\n      ");
  stack1 = depth0;
  stack2 = "view.sortView";
  stack3 = {};
  stack4 = "view.filteredContent";
  stack3['contentBinding'] = stack4;
  stack4 = "label-row";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </thead>\n      <tbody>\n      ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/hbase", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.regionServerComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <div class=\"summary-value main-info\">\n              ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"summary-label\">\n              <a\n                href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.regionServerComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.regionServers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <span>\n                  ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.regionServersStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.regionServersTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </span>\n                ");
  stack1 = depth0;
  stack2 = "common.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.phoenixServerComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <div class=\"summary-value main-info\">\n            <span>\n              ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.phoenixServersStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.phoenixServersTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </span>\n              ");
  stack1 = depth0;
  stack2 = "common.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            <div class=\"summary-label\">\n              <a\n                href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.phoenixServerComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.phoenixServers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"component-summary\">\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"col-md-12\">\n      <div class=\"col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-10\">\n        ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isRegionServerCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isPhoenixQueryServerCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 regions-in-trnasition\">\n          <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.service.regionsInTransition";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.regions.transition";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n    </div>\n  </div>\n\n  <div class=\"col-md-12 metrics-summary\">\n    <div class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-10\">\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 started-time\">\n        <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.masterStartedTime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.masterStarted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 activated-time\">\n        <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.masterStartedTime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.masterActivated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 average-load\">\n        <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.averageLoad";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.averageLoad";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 server-heap\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.masterServerHeapSummaryPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.masterServerHeapSummary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hbase.masterServerHeap";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/hdfs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.slaveComponentsView";
  stack3 = {};
  stack4 = "view.slaveComponentsView";
  stack3['slaveComponentsViewBinding'] = stack4;
  stack4 = "view";
  stack3['summaryViewBinding'] = stack4;
  stack4 = true;
  stack3['showTitle'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  ");
  data.buffer.push("\n  <div class=\"component-summary\">\n    ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = {};
  stack4 = "view.slaveComponentsView";
  stack3['slaveComponentsViewBinding'] = stack4;
  stack4 = "view";
  stack3['summaryViewBinding'] = stack4;
  stack4 = "view.componentCommonWidgetsView";
  stack3['componentCommonWidgetsViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/hive", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "endpoint.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = ":row :component endpoint.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"col-md-2 col-lg-2 summary-label\" style=\"margin-top: 0px;\">");
  stack1 = depth0;
  stack2 = "endpoint.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      <div class=\"col-md-10 col-lg-10 summary-value\">\n        <span class=\"endpoint-value\">\n          ");
  stack1 = depth0;
  stack2 = "endpoint.value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.clipBoardView";
  stack3 = {};
  stack4 = "copy";
  stack3['data-clipboard-action'] = stack4;
  stack4 = "endpoint.value";
  stack3['data-clipboard-textBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </span>\n      </div>\n    </div>\n  ");
  return buffer;}
function program5(depth0,data) {
  
  
  data.buffer.push("\n            <img src=\"/img/clippy.svg\" alt=\"Copy\" rel=\"clipboard-tooltip\" data-toggle=\"tooltip\" data-placement=\"bottom\" title=\"Copied!\"/>\n          ");}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <div class=\"row\">\n        <div class=\"col-md-6 summary-label\" style=\"margin-top: 0px;\">");
  stack1 = depth0;
  stack2 = "link.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"col-md-6 summary-value\">\n          <a href=\"#\" target=\"_blank\" rel=\"noopener noreferrer\" ");
  stack1 = depth0;
  stack2 = "link.viewInstance";
  stack3 = depth0;
  stack4 = "goToView";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "app.goToView";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </div>\n      </div>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row component-summary\">\n  ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div class=\"col-md-12\">\n    <div class=\"col-md-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div class=\"col-md-10\">\n      ");
  stack1 = depth0;
  stack2 = "App.SummaryClientComponentsView";
  stack3 = {};
  stack4 = "view.parentView.clientObj";
  stack3['clientsObjBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "hiveServerEndPoints";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "endpoint";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-6\">\n    ");
  stack1 = depth0;
  stack2 = "view.viewLinks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "link";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/mapreduce2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <a href=\"#\" class=\"label label-important\" ");
  stack1 = depth0;
  stack2 = "view.service";
  stack3 = depth0;
  stack4 = "showAlertsPopup";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.alertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    ");
  return buffer;}

  data.buffer.push("\n\n\n<div class=\"clearfix\">\n  <div class=\"name col-md-2\">\n    <i class=\"pull-left glyphicon glyphicon-empty\"></i>\n    ");
  stack1 = depth0;
  stack2 = "App.MainDashboardServiceHealthView";
  stack3 = {};
  stack4 = "view.service";
  stack3['serviceBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <a ");
  stack1 = depth0;
  stack2 = "view.service";
  stack3 = depth0;
  stack4 = "selectService";
  stack5 = {};
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  stack1 = depth0;
  stack2 = "view.alertsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"summary span\">\n    ");
  stack1 = depth0;
  stack2 = "view.titleInfo";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/onefs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.dataNodeComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <div class=\"summary-value main-info\">\n              ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"summary-label\">\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.dataNodeComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodes";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                <span>\n                  ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.dataNodesStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.dataNodesTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </span>\n                ");
  stack1 = depth0;
  stack2 = "common.started";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"main-info\">\n                ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n            ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"main-info\" rel=\"tooltip\"\n                ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.live";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "view.service.liveDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n              <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n            ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"main-info\">\n                ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n            ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"main-info\" rel=\"tooltip\"\n                ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.dead";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "view.service.deadDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n              <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.dead";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n            ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"main-info\">\n                ");
  stack1 = depth0;
  stack2 = "services.service.summary.notAvailable";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n            ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"main-info\" rel=\"tooltip\"\n                ");
  stack1 = {};
  stack2 = "dashboard.services.hdfs.datanode.status.tooltip.decommission";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "view.service.decommissionDataNodes.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n              <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.decom";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n            ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  ");
  data.buffer.push("\n  <div class=\"component-summary\">\n    ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"col-md-12\">\n      <div class=\"col-md-10 col-md-offset-2\">\n        ");
  data.buffer.push("\n        <div class=\"row namenode-uptime col-md-3\">\n          <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.nodeUptime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.uptime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row namenode-heap col-md-3\">\n          <div class=\"summary-value\">\n            <div class=\"main-info\"> ");
  stack1 = depth0;
  stack2 = "view.nodeHeapPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n            <div class=\"info-desc\"> ");
  stack1 = depth0;
  stack2 = "view.nodeHeap";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </div>\n          </div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.nodes.heap";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n    </div>\n    <div class=\"col-md-12\">\n      <div class=\"col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-10\">\n        ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isDataNodeCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  data.buffer.push("\n        <div class=\"row\"></div>\n        ");
  data.buffer.push("\n        <div class=\"row datanode-count-live col-md-3\">\n          <div class=\"summary-value\">\n            ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodecounts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row datanode-count-dead col-md-3\">\n          <div class=\"summary-value\">\n            ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(15, program15, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodecounts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row datanode-count-decommission col-md-3\">\n          <div class=\"summary-value\">\n            ");
  stack1 = depth0;
  stack2 = "view.service.metricsNotAvailable";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.datanodecounts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n    </div>\n  </div>\n  ");
  data.buffer.push("\n\n  ");
  data.buffer.push("\n  <div class=\"col-md-12 metrics-summary\">\n    <div class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-10\">\n      ");
  data.buffer.push("\n      <div class=\"row dfs-usage col-md-3\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsUsedDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.dfsUsedDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.dfsUsed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"row non-dfs-used col-md-3\">\n        <div class=\"summary-value main-info\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.nonDfsUsedDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.nonDfsUsedDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.nonDfsUsed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      <div class=\"row capacity-remaining col-md-3\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.remainingDiskPercent";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "view.remainingDisk";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.capacity.remaining";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row\"></div>\n      ");
  data.buffer.push("\n      <div class=\"row block-errors-corrupt col-md-3\">\n        <div class=\"summary-value main-info\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsCorruptBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.corrupt";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "services.service.summary.blockErrors";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row block-errors-missing col-md-3\">\n        <div class=\"summary-value main-info\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsMissingBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.missing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "services.service.summary.blockErrors";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row block-errors-replicated col-md-3\">\n        <div class=\"summary-value main-info\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsUnderReplicatedBlocks";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.blockErrors.replicated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "services.service.summary.blockErrors";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row\"></div>\n      ");
  data.buffer.push("\n      <div class=\"row total-files-dirs col-md-3\">\n        <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.dfsTotalFiles";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.totalFilesAndDirs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n  </div>\n  ");
  data.buffer.push("\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/ranger", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.rangerTagsyncComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div class=\"summary-value main-info\">\n            ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <div class=\"summary-label\">\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.rangerTagsyncComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.ranger.rangerTagsyncs";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </div>\n        </div>\n      ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <span>\n                ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.rangerTagsyncsStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.rangerTagsyncsTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </span>");
  stack1 = depth0;
  stack2 = "common.started";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "item.isDisplayed";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"row ranger-plugin col-md-3\">\n            <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "item.status";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n            <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "item.pluginTitle";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          </div>\n        ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row component-summary\">\n  ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <div class=\"col-md-12\">\n    <div class=\"col-md-2\">\n      ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    <div class=\"col-md-10\">\n      ");
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isRangerTagsyncCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      ");
  stack1 = depth0;
  stack2 = "controller.rangerPlugins";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/storm", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div class=\"row col-md-3 component SUPERVISOR\">\n            <div class=\"summary-value main-info\">\n              ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"summary-label\">\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.filterComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "dashboard.services.storm.supervisors";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.superVisorsLive";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.superVisorsTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.live";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-12 component-summary\">\n    ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"col-md-12\">\n      <div class=\"col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-10\">\n        ");
  stack1 = depth0;
  stack2 = "view.isSupervisorCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n  <div class=\"col-md-12 metrics-summary\">\n    <div class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-10\">\n      <div class=\"row col-md-3 storm-slots\">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.service.freeSlots";
  foundHelper = helpers.formatNull;
  stack3 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " / ");
  stack1 = depth0;
  stack2 = "view.service.totalSlots";
  foundHelper = helpers.formatNull;
  stack3 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          (");
  stack1 = depth0;
  stack2 = "view.freeSlotsPercentage";
  stack3 = {};
  stack4 = "0";
  stack3['empty'] = stack4;
  foundHelper = helpers.formatNull;
  stack4 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "% ");
  stack1 = depth0;
  stack2 = "common.free";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ")\n        </div>\n        <div class=\"summary-label\">\n          ");
  stack1 = depth0;
  stack2 = "services.service.summary.storm.freeslots";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"row col-md-3 storm-tasks\">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.service.totalTasks";
  foundHelper = helpers.formatNull;
  stack3 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"summary-label\">\n          ");
  stack1 = depth0;
  stack2 = "services.service.summary.storm.tasks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"row col-md-3 storm-executors\">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.service.totalExecutors";
  foundHelper = helpers.formatNull;
  stack3 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"summary-label\">\n          ");
  stack1 = depth0;
  stack2 = "services.service.summary.storm.executors";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"row col-md-3 storm-topologies\">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.service.topologies";
  foundHelper = helpers.formatNull;
  stack3 = foundHelper || depth0.formatNull;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatNull", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"summary-label\">\n          ");
  stack1 = depth0;
  stack2 = "services.service.summary.storm.topologies";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n      <div class=\"row col-md-3 nimbus-uptime\">\n        <div class=\"summary-value main-info\">\n          ");
  stack1 = depth0;
  stack2 = "view.nimbusUptimeFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        <div class=\"summary-label\">\n          ");
  stack1 = depth0;
  stack2 = "services.service.summary.storm.nimbus.uptime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n      </div>\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/yarn", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.nodeManagerComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            <div class=\"summary-value main-info\">\n              ");
  stack1 = depth0;
  stack2 = "App.router.clusterController.isServiceContentFullyLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"summary-label\">\n              <a\n                href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.nodeManagerComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n            </div>\n          </div>\n        ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.ComponentLiveTextView";
  stack3 = {};
  stack4 = "view.service.nodeManagersStarted";
  stack3['liveComponentsBinding'] = stack4;
  stack4 = "view.service.nodeManagersTotal";
  stack3['totalComponentsBinding'] = stack4;
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.started";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.liveComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "/");
  stack1 = depth0;
  stack2 = "view.totalComponents";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "common.loading.eclipses";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"row\">\n  ");
  data.buffer.push("\n  <div class=\"component-summary\">\n    ");
  stack1 = depth0;
  stack2 = "view.dashboardMasterComponentView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"col-md-12\">\n      <div class=\"col-md-2\">\n        ");
  stack1 = depth0;
  stack2 = "view.hasMultipleMasterGroups";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      <div class=\"col-md-10\">\n        ");
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.isNodeManagerCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  data.buffer.push("\n        <div ");
  stack1 = {};
  stack2 = ":row :component :col-md-3 view.yarnClientComponent.componentName";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div class=\"summary-value main-info\">\n            <span>");
  stack1 = depth0;
  stack2 = "view.service.installedClients";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "common.installed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " </span>\n          </div>\n          <div class=\"summary-label\">\n            <a ");
  stack1 = depth0;
  stack2 = "view.yarnClientComponent";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"javascript:void(null)\">\n              ");
  stack1 = depth0;
  stack2 = "view.service.installedClients";
  stack3 = {};
  stack4 = "t:dashboard.services.yarn.client";
  stack3['singular'] = stack4;
  stack4 = "t:dashboard.services.yarn.clients";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n            </a>\n          </div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row\"></div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 resourcemanager-uptime\">\n          <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.nodeUptime";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.resourceManager.uptime";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row\"></div>\n        <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 nodemanager-active\">\n        <span class=\"summary-value main-info\"\n          ");
  stack1 = {};
  stack2 = "dashboard.services.yarn.nodeManagers.status.tooltip.active";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              rel=\"tooltip\">\n          ");
  stack1 = depth0;
  stack2 = "view._nmActive";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status.active";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 nodemanager-lost\">\n        <span class=\"summary-value main-info\"\n          ");
  stack1 = {};
  stack2 = "dashboard.services.yarn.nodeManagers.status.tooltip.lost";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              rel=\"tooltip\">\n          ");
  stack1 = depth0;
  stack2 = "view._nmLost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status.lost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 nodemanager-unhealthy\">\n        <span class=\"summary-value main-info\"\n          ");
  stack1 = {};
  stack2 = "dashboard.services.yarn.nodeManagers.status.tooltip.unhealthy";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              rel=\"tooltip\">\n          ");
  stack1 = depth0;
  stack2 = "view._nmUnhealthy";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status.unhealthy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 nodemanager-rebooted\">\n        <span class=\"summary-value main-info\"\n          ");
  stack1 = {};
  stack2 = "dashboard.services.yarn.nodeManagers.status.tooltip.rebooted";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              rel=\"tooltip\">\n          ");
  stack1 = depth0;
  stack2 = "view._nmRebooted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status.rebooted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 nodemanager-decommissioned\">\n        <span class=\"summary-value main-info\"\n          ");
  stack1 = {};
  stack2 = "dashboard.services.yarn.nodeManagers.status.tooltip.decommissioned";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              rel=\"tooltip\">\n          ");
  stack1 = depth0;
  stack2 = "view._nmDecom";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        </span>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.nodeManagers.status.decommissioned";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n        ");
  data.buffer.push("\n        <div class=\"row\"></div>\n        ");
  data.buffer.push("\n        <div class=\"row col-md-3 resourcemanager-heap\">\n          <div class=\"summary-value main-info\">");
  stack1 = depth0;
  stack2 = "view.nodeHeap";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.resourceManager.nodes.heap";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n    </div>\n  </div>\n\n  ");
  data.buffer.push("\n  <div class=\"col-md-12 metrics-summary\">\n    <div class=\"col-md-2\">");
  stack1 = depth0;
  stack2 = "dashboard.services.hdfs.summary.service-metrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    <div class=\"col-md-10\">\n      <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.containers";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-containers-allocated\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._allocated";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.containers.allocated";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-containers-pending\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._pending";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.containers.pending";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-containers-reserved\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._reserved";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.containers.reserved";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row\"></div>\n      <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-submitted\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsSubmitted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.submitted";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-running\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsRunning";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.running";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-pending\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsPending";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.pending";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-completed\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsCompleted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.completed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-killed\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsKilled";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.killed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-applications-failed\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view._appsFailed";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.apps.failed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row\"></div>\n      <div class=\"summary-label col-md-12\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.memory";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-memory-allocated\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.allocatedMemoryFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.memory.used";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-memory-reserved\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.reservedMemoryFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.memory.reserved";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-memory-available\">\n        <div class=\"summary-value\">\n          <div class=\"main-info\">");
  stack1 = depth0;
  stack2 = "view.availableMemoryFormatted";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n          <div class=\"info-desc\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.memory.available";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        </div>\n      </div>\n      ");
  data.buffer.push("\n      <div class=\"row\"></div>\n      ");
  data.buffer.push("\n      <div class=\"row col-md-3 yarn-queues\">\n        <div class=\"summary-value main-info\">\n          <span rel=\"queue-tooltip\"\n                class=\"text-tooltip\" ");
  stack1 = {};
  stack2 = "view.service.queueFormatted";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.queues";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </span>\n        </div>\n        <div class=\"summary-label\">");
  stack1 = depth0;
  stack2 = "dashboard.services.yarn.queues";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n    </div>\n  </div>\n  ");
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/services/zookeeper", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <a href=\"#\" class=\"label label-important\" ");
  stack1 = depth0;
  stack2 = "view.service";
  stack3 = depth0;
  stack4 = "showAlertsPopup";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.alertsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"clearfix\">\n  <div class=\"name col-md-2\">\n    <i class=\"pull-left glyphicon glyphicon-empty\"></i>\n    ");
  stack1 = depth0;
  stack2 = "App.MainDashboardServiceHealthView";
  stack3 = {};
  stack4 = "view.service";
  stack3['serviceBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <a ");
  stack1 = depth0;
  stack2 = "view.service";
  stack3 = depth0;
  stack4 = "selectService";
  stack5 = {};
  stack6 = true;
  stack5['href'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  stack1 = depth0;
  stack2 = "view.alertsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <div class=\"summary span\">\n    ");
  stack1 = depth0;
  stack2 = "view.titleInfo.pre";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "view.titleInfo.component";
  stack3 = depth0;
  stack4 = "filterHosts";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.titleInfo.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n    ");
  stack1 = depth0;
  stack2 = "services.zookeeper.postfix";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/expression", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <li><a href=\"#\" ");
  stack1 = depth0;
  stack2 = "operator";
  stack3 = depth0;
  stack4 = "addOperator";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "operator";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div class=\"alert alert-danger\">\n    ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.expression2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.isInvalidTextfield";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.msg2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <a ");
  stack1 = depth0;
  stack2 = "view.expression";
  stack3 = depth0;
  stack4 = "removeExpression";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"remove-link\"><i class=\"glyphicon glyphicon-trash\"></i></a>\n  ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.expression.data";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "element";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <div class=\"add-item-input\">\n      ");
  stack1 = depth0;
  stack2 = "App.InputCursorTextfieldView";
  stack3 = {};
  stack4 = "view";
  stack3['parentViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n  ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n      <div class=\"metric-instance\" ");
  stack1 = {};
  stack2 = "element.id";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "element.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "element";
  stack3 = depth0;
  stack4 = "removeElement";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-remove\"></i></a>\n      </div>\n    ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"add-item-input\">\n      ");
  stack1 = depth0;
  stack2 = "App.InputCursorTextfieldView";
  stack3 = {};
  stack4 = "view";
  stack3['parentViewBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"placeholder\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.wizard.step2.addMetrics";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"actions-field row\">\n  <div class=\"col-md-12\">\n    ");
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.AddMetricExpressionView";
  stack3 = {};
  stack4 = "add-metric-menu pull-left";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"btn-group add-operator pull-left mlm\">\n      <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n        ");
  stack1 = depth0;
  stack2 = "dashboard.widgets.wizard.step2.newOperator";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        <span class=\"caret\"></span>\n      </button>\n      <ul class=\"dropdown-menu\">\n        ");
  stack1 = depth0;
  stack2 = "view.OPERATORS";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operator";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </ul>\n    </div>\n    ");
  data.buffer.push("\n\n    ");
  data.buffer.push("\n    <div class=\"add-number pull-left mlm\">\n      <div class=\"input-group\">\n        ");
  stack1 = depth0;
  stack2 = "App.AddNumberExpressionView";
  stack3 = {};
  stack4 = "view.numberValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        <div class=\"input-group-btn\">\n          <button class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "addNumber";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.isNumberValueInvalid";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.wizard.step2.newNumber";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n        </div>\n      </div>\n    </div>\n    ");
  data.buffer.push("\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.isInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n<div class=\"col-md-12 metric-field\">\n  ");
  stack1 = depth0;
  stack2 = "view.expression.isRemovable";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.expression.data.length";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(11, program11, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <div class=\"col-md-6 widget-info-section pts\" rel=\"selectable-tooltip\"\n              ");
  stack1 = {};
  stack2 = "widget.create.wizard.step1.body.choose.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "option.name";
  stack3 = depth0;
  stack4 = "chooseOption";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"row\">\n                <div class=\"col-md-4\">\n                  <div class=\"icon\">\n                    <a class=\"widget-icon-image\" target=\"_blank\" rel=\"noopener noreferrer\"><img ");
  stack1 = {};
  stack2 = "option.iconPath";
  stack1['src'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n                  </div>\n                </div>\n                <div class=\"label-description col-md-8\">\n                  <p class=\"label-text\">");
  stack1 = depth0;
  stack2 = "option.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n                  <p class=\"description-text\">");
  stack1 = depth0;
  stack2 = "option.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n                </div>\n              </div>\n            </div>\n          ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"add-widget-step1\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"add-widget-step1-options-body\">\n        <h4>");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step1.body.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n        <div class=\"widgets-info-container row\">\n          ");
  stack1 = depth0;
  stack2 = "options";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "templates/main/service/widgets/create/step2_number";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "templates/main/service/widgets/create/step2_graph";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "templates/main/service/widgets/create/step2_template";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n        <form autocomplete=\"off\" class=\"form-horizontal\">\n          ");
  stack1 = depth0;
  stack2 = "widgetPropertiesViews";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "property";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </form>\n    </div>\n  </div>\n  ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "property.name property.isValid::error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"form-group row\">\n                <div class=\"col-md-2\">\n                  <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "property.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "property.isRequired";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </label>\n                </div>\n                <div class=\"col-md-3\">\n                  ");
  stack1 = depth0;
  stack2 = "property.viewClass";
  stack3 = {};
  stack4 = "property";
  stack3['propertyBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              </div>\n            </div>\n          ");
  return buffer;}
function program9(depth0,data) {
  
  
  data.buffer.push("\n                      <i class=\"glyphicon glyphicon-asterisk\"></i>\n                    ");}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <button id=\"add-widget-step2-back\" class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"step2 wizard-content col-md-9\">\n\n  ");
  stack1 = depth0;
  stack2 = "view.templateType.isNumber";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  ");
  stack1 = depth0;
  stack2 = "view.templateType.isGraph";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n  ");
  stack1 = depth0;
  stack2 = "view.templateType.isTemplate";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "widgetPropertiesViews.length";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    ");
  stack1 = depth0;
  stack2 = "isEditWidget";
  stack3 = helpers.unless;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    <button id=\"add-widget-step2-next\" class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2_add_metric", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n      <!--service level-->\n      ");
  stack1 = depth0;
  stack2 = "view.componentMap";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        <li class=\"dropdown-submenu\">\n          <a class=\"\" tabindex=\"-1\" href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n          <ul class=\"dropdown-menu\">\n            ");
  stack1 = depth0;
  stack2 = "service.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "comp";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </li>\n      ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "comp.isGroup";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <li class=\"dropdown-submenu\">\n                  <a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "comp.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  <ul class=\"dropdown-menu\">\n                    ");
  stack1 = depth0;
  stack2 = "comp.components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "group";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </ul>\n                </li>\n              ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "App.WidgetWizardComponentDropdownView";
  stack3 = {};
  stack4 = "group.component";
  stack3['componentBinding'] = stack4;
  stack4 = "group.displayName";
  stack3['displayNameBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.WidgetWizardComponentDropdownView";
  stack3 = {};
  stack4 = "comp";
  stack3['componentBinding'] = stack4;
  stack4 = "comp.displayName";
  stack3['displayNameBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"btn-group dropdown\">\n  <button class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">");
  stack1 = depth0;
  stack2 = "dashboard.widgets.wizard.step2.newMetric";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    &nbsp;<span class=\"caret\"></span></button>\n  <ul class=\"dropdown-menu service-level-dropdown\">\n    ");
  stack1 = depth0;
  stack2 = "controller.content.isMetricsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </ul>\n</div>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2_component_dropdown", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n<a href=\"javascript:void(null);\">");
  stack1 = depth0;
  stack2 = "view.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n<ul class=\"dropdown-menu select-options-dropdown\">\n  <!--metrics level-->\n  <li class=\"keep-open metric-select\" ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "selectComponents";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.JqueryChosenView";
  stack3 = {};
  stack4 = "view.component.metrics";
  stack3['optionsBinding'] = stack4;
  stack4 = "view.component.id";
  stack3['elementIdBinding'] = stack4;
  stack4 = "view.parentView.metricsSelectionObj";
  stack3['selectionObjBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </li>\n  <!--select aggregator function -->\n  <li ");
  stack1 = {};
  stack2 = ":keep-open :aggregator-select view.component.showAggregateSelect::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "selectComponents";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = {};
  stack2 = "dashboard.widgets.wizard.step2.aggregateTooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.JqueryChosenView";
  stack3 = {};
  stack4 = "view.parentView.parentView.AGGREGATE_FUNCTIONS";
  stack3['optionsBinding'] = stack4;
  stack4 = "view.component.aggregatorId";
  stack3['elementIdBinding'] = stack4;
  stack4 = "view.parentView.aggregateFnSelectionObj";
  stack3['selectionObjBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </li>\n  <li class=\"actions-buttons\">\n    <div>\n      <button class=\"btn btn-default\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "cancel";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n      <button ");
  stack1 = {};
  stack2 = ":btn :btn-primary view.component.isAddEnabled::disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "view.component";
  stack3 = depth0;
  stack4 = "addMetric";
  stack5 = {};
  stack6 = "view.parentView";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    </div>\n  </li>\n</ul>\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2_graph", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-danger\">\n        ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.expression";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <fieldset>\n        <h5 ");
  stack1 = {};
  stack2 = "dataSet.isInvalidLabel:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "dataSet.label";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "dataSet.isInvalidLabel";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </h5>\n\n        <h5>");
  stack1 = depth0;
  stack2 = "common.expression";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n        ");
  stack1 = depth0;
  stack2 = "App.WidgetWizardExpressionView";
  stack3 = {};
  stack4 = "dataSet.expression";
  stack3['expressionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "dataSet.isRemovable";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </fieldset>\n    ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div>\n              <span class=\"help-block validation-block\">\n                ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </span>\n            </div>\n          ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <a ");
  stack1 = depth0;
  stack2 = "dataSet";
  stack3 = depth0;
  stack4 = "removeDataSet";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"remove-link\"><i class=\"glyphicon glyphicon-trash\"></i></a>\n        ");
  return buffer;}

  data.buffer.push("\n\n<h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n<p class=\"step-description\">\n  ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</p>\n<div class=\"panel panel-default\">\n  <div class=\"panel-body\">\n    ");
  stack1 = depth0;
  stack2 = "isAnyExpressionInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "dataSets";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "dataSet";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    <div class=\"align-right\">\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "addDataSet";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-plus\"></i>\n        ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.addDataset";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2_number", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-danger\">\n        ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.expression";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.WidgetWizardExpressionView";
  stack3 = {};
  stack4 = "expression";
  stack3['expressionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n<p class=\"step-description\">\n  ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</p>\n<div class=\"panel panel-default\">\n  <div class=\"panel-body\">\n    ");
  stack1 = depth0;
  stack2 = "isAnyExpressionInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "expressions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "expression";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step2_template", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div>");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.template.invalid.msg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <div class=\"alert alert-danger\">\n        ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.invalid.expression";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <h5>");
  stack1 = depth0;
  stack2 = "EXPRESSION_PREFIX";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "expression.id";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h5>\n      ");
  stack1 = depth0;
  stack2 = "App.WidgetWizardExpressionView";
  stack3 = {};
  stack4 = "expression";
  stack3['expressionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n\n<h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n\n<p class=\"step-description\">\n  ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.template";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n</p>\n\n<div class=\"panel panel-default\">\n  <div class=\"panel-body\">\n    <div ");
  stack1 = {};
  stack2 = ":template isTemplateInvalid:error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <h5>");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.template.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h5>\n      ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "templateValue";
  stack3['valueBinding'] = stack4;
  stack4 = "view.templatePlaceholder";
  stack3['placeholderBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "isTemplateInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n\n    <div class=\"alert alert-info\">\n      ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.body.text";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "isAnyExpressionInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "expressions";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "expression";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    <div class=\"align-right\">\n      <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "addExpression";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <i class=\"glyphicon glyphicon-plus\"></i>\n        ");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.addExpression";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </a>\n    </div>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div>\n                <span class=\"help-block validation-block\">\n                  ");
  stack1 = depth0;
  stack2 = "widgetNameErrorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              </div>\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div>\n                <span class=\"help-block validation-block\">\n                  ");
  stack1 = depth0;
  stack2 = "descriptionErrorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </span>\n              </div>\n            ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"step3 wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <form autocomplete=\"off\" class=\"form-horizontal\">\n        <div ");
  stack1 = {};
  stack2 = "isNameInvalid:has-error :row :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div class=\"col-md-2 title\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step3.widgetName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<i class=\"glyphicon glyphicon-asterisk\"></i></label>\n          </div>\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "widgetName";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "isNameInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n        <div class=\"row\">\n          <div class=\"col-md-2 title\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "common.author";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n          </div>\n          <div class=\"col-md-10\"><span class=\"form-text\">");
  stack1 = depth0;
  stack2 = "widgetAuthor";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></div>\n        </div>\n        <div class=\"row\">\n          <div class=\"col-md-2 title\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step3.sharing";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n          </div>\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "checkbox";
  stack3['classNames'] = stack4;
  stack4 = "controller.isSharedCheckboxDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "controller.isSharedChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "widget.create.wizard.step3.sharing.msg";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n\n        <div ");
  stack1 = {};
  stack2 = ":row :form-group isDescriptionInvalid:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <div class=\"col-md-2 title\">\n            <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n          </div>\n          <div class=\"col-md-10\">\n            ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "widgetDescription";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control description-text-area";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "isDescriptionInvalid";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n\n      </form>\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button id=\"add-widget-step3-back\" class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">&larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    <button id=\"add-widget-step3-save\" class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "complete";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.save";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    <button id=\"add-widget-step3-cancel\" class=\"btn btn-default pull-right\" ");
  stack1 = depth0;
  stack2 = "cancel";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/widget_property_threshold", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;


  data.buffer.push("\n\n<div class=\"row\">\n  <div class=\"col-md-4 badge-container\">\n    <label><span ");
  stack1 = {};
  stack2 = ":label view.property.badgeWarning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.property.badgeWarning";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></label>\n  </div>\n  <div ");
  stack1 = {};
  stack2 = ":col-md-8 :threshold-input property.isSmallValueValid::error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"threshold-tooltip\"\n    ");
  stack1 = {};
  stack2 = "dashboard.widgets.wizard.step2.threshold.ok.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.property.smallValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n<div class=\"row\">\n  <div class=\"col-md-4 badge-container\">\n    <label><span ");
  stack1 = {};
  stack2 = ":label view.property.badgeCritical";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.property.badgeCritical";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></label>\n  </div>\n  <div ");
  stack1 = {};
  stack2 = ":col-md-8 :threshold-input property.isBigValueValid::error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " rel=\"threshold-tooltip\"\n    ");
  stack1 = {};
  stack2 = "dashboard.widgets.wizard.step2.threshold.warning.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.property.bigValue";
  stack3['valueBinding'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/create/wizard", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"preview\" id=\"widget-preview\">\n          <h5>");
  stack1 = depth0;
  stack2 = "common.preview";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n          <div class=\"widget\">\n            ");
  stack1 = depth0;
  stack2 = "view.previewWidgetClass";
  stack3 = {};
  stack4 = "App.router.widgetWizardStep2Controller";
  stack3['controllerBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"preview\" id=\"widget-preview\">\n          <h5>");
  stack1 = depth0;
  stack2 = "common.preview";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n          <div class=\"widget\">\n            ");
  stack1 = depth0;
  stack2 = "view.previewWidgetClass";
  stack3 = {};
  stack4 = "App.router.widgetWizardStep3Controller";
  stack3['controllerBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard\" id=\"add-widget-wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep3:active view.isStep3Disabled:disabled view.isStep3Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep3";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">3</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n      ");
  stack1 = depth0;
  stack2 = "view.isStep2";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isStep3";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/service/widgets/edit", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"preview\" id=\"widget-preview\">\n          <h5>");
  stack1 = depth0;
  stack2 = "common.preview";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n          <div class=\"widget\">\n            ");
  stack1 = depth0;
  stack2 = "view.previewWidgetClass";
  stack3 = {};
  stack4 = "App.router.widgetWizardStep2Controller";
  stack3['controllerBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <div class=\"preview\" id=\"widget-preview\">\n          <h5>");
  stack1 = depth0;
  stack2 = "common.preview";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</h5>\n          <div class=\"widget\">\n            ");
  stack1 = depth0;
  stack2 = "view.previewWidgetClass";
  stack3 = {};
  stack4 = "App.router.widgetWizardStep3Controller";
  stack3['controllerBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </div>\n      ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"wizard\" id=\"edit-widget-wizard\">\n  <div class=\"wizard-body row\">\n    <div class=\"wizard-nav col-md-3\">\n      <ul class=\"nav nav-pills nav-stacked\">\n        <li ");
  stack1 = {};
  stack2 = "isStep1:active view.isStep1Disabled:disabled view.isStep1Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep1";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">1</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n        <li ");
  stack1 = {};
  stack2 = "isStep2:active view.isStep2Disabled:disabled view.isStep2Completed:completed";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><a href=\"javascript:void(null);\"  ");
  stack1 = depth0;
  stack2 = "gotoStep2";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"step-marker\"><span class=\"step-index\">2</span></i><p class=\"step-name\">");
  stack1 = depth0;
  stack2 = "widget.create.wizard.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></a></li>\n      </ul>\n      ");
  stack1 = depth0;
  stack2 = "view.isStep1";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "view.isStep2";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "outlet";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/side-menu-item", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <a ");
  stack1 = {};
  stack2 = "view.content.label";
  stack1['title'] = stack2;
  stack2 = "view.content.href";
  stack1['href'] = stack2;
  stack2 = "view.content.isView:views-button-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"collapse-sub-menu\">\n    <span class=\"toggle-icon glyphicon glyphicon-menu-down pull-right\"></span>\n    <span ");
  stack1 = {};
  stack2 = ":navigation-icon view.content.iconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n    <span class=\"navigation-menu-item\">");
  stack1 = depth0;
  stack2 = "view.content.label";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span>\n  </a>\n  ");
  stack1 = depth0;
  stack2 = "App.AllServicesActionView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  stack1 = depth0;
  stack2 = "App.SideNavServiceMenuView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isAdminItem";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(8, program8, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <a ");
  stack1 = {};
  stack2 = "view.content.label";
  stack1['data-original-title'] = stack2;
  stack2 = "view.content.href";
  stack1['href'] = stack2;
  stack2 = "view.content.isView:views-button-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " data-toggle=\"collapse-sub-menu\">\n      <span class=\"toggle-icon glyphicon glyphicon-menu-down pull-right\"></span>\n      <span ");
  stack1 = {};
  stack2 = ":navigation-icon view.content.iconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n      <span class=\"navigation-menu-item\">");
  stack1 = depth0;
  stack2 = "view.content.label";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span>\n    </a>\n    <ul class=\"admin-submenu sub-menu nav nav-pills nav-stacked\">\n      ");
  stack1 = depth0;
  stack2 = "view.dropdownCategories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.AdminDropdownItemView";
  stack3 = {};
  stack4 = "category.name";
  stack3['itemBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <a ");
  stack1 = {};
  stack2 = "category.href";
  stack1['href'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "category.url";
  stack3 = depth0;
  stack4 = "goToCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n        ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n    <a ");
  stack1 = depth0;
  stack2 = "view.content.routing";
  stack3 = depth0;
  stack4 = "goToSection";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.content.label";
  stack1['data-original-title'] = stack2;
  stack2 = "view.content.href";
  stack1['href'] = stack2;
  stack2 = "view.content.isView:views-button-container";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <span ");
  stack1 = {};
  stack2 = ":navigation-icon view.content.iconClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></span>\n      <span class=\"navigation-menu-item\">");
  stack1 = depth0;
  stack2 = "view.content.label";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</span>\n    </a>\n  ");
  return buffer;}

  data.buffer.push("\n\n<!--dropdown menu for the items had dropdowns-->\n");
  stack1 = depth0;
  stack2 = "view.isServicesItem";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n\n\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/main/views", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <table class=\"table table-hover views-table\">\n      <tbody>\n        ");
  stack1 = depth0;
  stack2 = "view.views";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "instance";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </tbody>\n    </table>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "instance.visible";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <tr ");
  stack1 = depth0;
  stack2 = "instance";
  stack3 = depth0;
  stack4 = "setView";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <td class=\"icon\">\n                <a class=\"view-icon-image\"><img ");
  stack1 = {};
  stack2 = "instance.iconPath";
  stack1['src'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></a>\n              </td>\n              <td class=\"label-version-description\">\n                <h4 class=\"view-label-text\">");
  stack1 = depth0;
  stack2 = "instance.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " <small class=\"view-version-text\"> &nbsp;&nbsp;(");
  stack1 = depth0;
  stack2 = "instance.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")</small></h4>\n                <p class=\"view-description-text\">");
  stack1 = depth0;
  stack2 = "instance.description";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n              </td>\n            </tr>\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div class=\"muted no-views-text background-text\">");
  stack1 = depth0;
  stack2 = "views.main.noViews";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"views\">\n  <h2 class=\"table-title\">");
  stack1 = depth0;
  stack2 = "views.main.yourViews";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n  ");
  stack1 = depth0;
  stack2 = "view.views.length";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/utils/ajax", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <br />\n  <br />\n  <pre><strong>");
  stack1 = depth0;
  stack2 = "utils.ajax.errorMessage";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ": </strong><span class=\"text-danger\">");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></pre>\n");
  return buffer;}

  data.buffer.push("\n\n<span class=\"text-danger\">");
  stack1 = depth0;
  stack2 = "view.statusCode";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span> <span>");
  stack1 = depth0;
  stack2 = "view.api";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n");
  stack1 = depth0;
  stack2 = "view.showMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/utils/config_launch_switch_config_group_of_host", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<span>");
  stack1 = depth0;
  stack2 = "installer.controls.slaveComponentGroups";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "&#58;&nbsp;</span>\n");
  stack1 = depth0;
  stack2 = "App.DropdownView";
  stack3 = {};
  stack4 = "view.parentView.configGroups";
  stack3['contentBinding'] = stack4;
  stack4 = "displayName";
  stack3['optionLabelPath'] = stack4;
  stack4 = "view.parentView.selectedConfigGroup";
  stack3['selectionBinding'] = stack4;
  stack4 = "display-inline-block";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/bootstrap_call_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div>\n        <div class=\"alert alert-error\">\n          ");
  stack1 = depth0;
  stack2 = "installer.step2.bootStrap.error";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        </div>\n        ");
  stack1 = depth0;
  stack2 = "view.parentView.serverError";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <pre>");
  stack1 = depth0;
  stack2 = "view.parentView.serverError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n        ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <div>\n      ");
  stack1 = depth0;
  stack2 = "installer.step2.bootStrap.inProgress";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n\n");
  stack1 = depth0;
  stack2 = "view.parentView.isError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/component_hosts", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "installer.noHostsAssigned";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.hasMultipleHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "showHosts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.firstHost";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "and";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "view.otherLength";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.value.0";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.hasNoHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/component_hosts_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <li>");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n  ");
  return buffer;}

  data.buffer.push("\n\n<ul>\n  ");
  stack1 = depth0;
  stack2 = "view.serviceConfig.value";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</ul>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/controls_service_config_radio_buttons", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "option.hidden";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"radio\">\n      ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigRadioButton";
  stack3 = {};
  stack4 = "view.name";
  stack3['nameBinding'] = stack4;
  stack4 = "option.displayName";
  stack3['valueBinding'] = stack4;
  stack4 = "option.radioId";
  stack3['radioIdBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      <label ");
  stack1 = {};
  stack2 = "option.radioId";
  stack1['for'] = stack2;
  stack2 = "option.className";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "option.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " &nbsp;\n      </label>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.options";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "option";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/controls_service_config_textfield_with_unit", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "App.ServiceConfigTextField";
  stack3 = {};
  stack4 = "view.serviceConfig";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "false";
  stack3['isPopoverEnabled'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n<span class=\"input-group-addon\">");
  stack1 = depth0;
  stack2 = "view.serviceConfig.unit";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/controls_service_config_usergroup_with_id", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigTextField";
  stack3 = {};
  stack4 = "view.serviceConfig";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "serviceConfigUGID";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <label class=\"serviceConfigUGIDLbl control-label\" ");
  stack1 = {};
  stack2 = "view.serviceConfig.ugid.name";
  stack1['for'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.serviceConfig.ugid.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </label>\n    ");
  stack1 = depth0;
  stack2 = "Ember.TextField";
  stack3 = {};
  stack4 = "view.serviceConfig.ugid.value";
  stack3['valueBinding'] = stack4;
  stack4 = "serviceConfigUGID";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.ServiceConfigTextField";
  stack3 = {};
  stack4 = "view.serviceConfig";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "serviceConfigNoUGID";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isUIDGIDVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/installer_cancel_button", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <button type=\"button\" class=\"btn btn-default pull-left\"\n    ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "cancelInstall";
  stack3 = {};
  stack4 = "App.router.installerController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "wizard-cancel";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "common.cancel";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </button>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "controller.wizardController.isInstallerWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step0", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <p class=\"help-block validation-block col-sm-9\"  ");
  stack1 = depth0;
  stack2 = "cluster-name-validation";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "clusterNameError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n          ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n<div id=\"get-started\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step0.body.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step0.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div ");
  stack1 = {};
  stack2 = "view.onError:has-error :form-group";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        <label class=\"control-label\" for=\"cluster-name\" ");
  stack1 = depth0;
  stack2 = "cluster-name-inscription";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step0.clusterName";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n          <a href=\"javascript:void(null)\"\n           rel=\"popover\"\n            ");
  stack1 = depth0;
  stack2 = "cluster-name-tooltip";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = {};
  stack2 = "installer.step0.clusterName.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step0.clusterName.tooltip.content";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.learnMore";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n        </label>\n\n        <div>\n          <div class=\"col-sm-3 cluster-name-input\">\n            ");
  stack1 = depth0;
  stack2 = "App.WizardStep0ViewClusterNameInput";
  stack3 = {};
  stack4 = "cluster-name";
  stack3['id'] = stack4;
  stack4 = "content.cluster.name";
  stack3['valueBinding'] = stack4;
  stack4 = "CLUSTER NAME";
  stack3['placeholder'] = stack4;
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "cluster-name-input";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n          ");
  stack1 = depth0;
  stack2 = "clusterNameError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12 btn-area\">\n  ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <button type=\"button\" class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n  </button>\n</div>\n\n\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step1", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n        ");
  data.buffer.push("\n        <div class=\"tabs-menu\">\n          <ul class=\"nav nav-tabs\" ");
  stack1 = depth0;
  stack2 = "stack-tabs";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "availableStackTypes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "stack";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </ul>\n        </div>\n        <div class=\"stacks-options\">\n          <div class=\"details-panel\">\n            <div class=\"btn-group\" ");
  stack1 = depth0;
  stack2 = "select-version";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <button type=\"button\" data-toggle=\"dropdown\"\n                      class=\"btn dropdown-toggle btn-default\" ");
  stack1 = depth0;
  stack2 = "select-version-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "controller.selectedStack.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                <span class=\"caret\"></span></button>\n              <ul class=\"dropdown-menu\">\n                ");
  stack1 = depth0;
  stack2 = "selectedStackType.stacks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "stack";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                <li><a\n                  href=\"#\" ");
  stack1 = depth0;
  stack2 = "uploadVdf";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "add-new-version";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step1.addVersion";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  ...</a></li>\n              </ul>\n            </div>\n            <div class=\"version-contents-section\">\n              ");
  stack1 = depth0;
  stack2 = "servicesForSelectedStack";
  stack3 = helpers.unless;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              <table class=\"table table-hover\">\n                <tbody>\n                ");
  stack1 = depth0;
  stack2 = "servicesForSelectedStack";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </tbody>\n              </table>\n            </div>\n          </div>\n        </div>\n        ");
  data.buffer.push("\n\n        ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.invalidUrlExist";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <li ");
  stack1 = {};
  stack2 = "stack.isSelected:active";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "stack-tab";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <a ");
  stack1 = depth0;
  stack2 = "stack-link-{stack.stackName}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "stack";
  stack3 = depth0;
  stack4 = "selectRepoInList";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  href=\"#\">");
  stack1 = depth0;
  stack2 = "stack.stackName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a></li>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li>\n                    ");
  data.buffer.push("\n                    <a\n                      href=\"#\" ");
  stack1 = depth0;
  stack2 = "stack";
  stack3 = depth0;
  stack4 = "changeVersion";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "{stack.displayName}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "stack.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "stack.stackDefault";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </a>\n                  </li>\n                ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        (");
  stack1 = depth0;
  stack2 = "installer.step1.changeVersion.defaultVersion";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ")\n                      ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"alert alert-info\"\n                     role=\"alert\">");
  stack1 = depth0;
  stack2 = "installer.step1.useLocalRepo.infoForm.content.empty";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <tr>\n                    <td class=\"col-md-4\">");
  stack1 = depth0;
  stack2 = "service.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                    <td class=\"col-md-8\">");
  stack1 = depth0;
  stack2 = "service.latestVersion";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                  </tr>\n                ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <form autocomplete=\"off\" id=\"repoVersionInfoForm\" class=\"form-horizontal\" role=\"form\" name=\"localVersionInfoForm\" novalidate>\n\n            <div class=\"panel panel-default repos-panel\">\n              <div class=\"step-title\">\n                <p>");
  stack1 = depth0;
  stack2 = "common.repositories";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n              </div>\n              <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step1.repo.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n              <div class=\"panel-body version-contents-body\">\n                <div class=\"row radio-group\">\n                  ");
  data.buffer.push("\n                  <div ");
  stack1 = {};
  stack2 = ":col-sm-4 :radio :big-radio :public-radio :wizard-plain-text";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "view.usePublicRepoRadioButton";
  stack3 = {};
  stack4 = "display-inline-block";
  stack3['classNames'] = stack4;
  stack4 = "use-public-repo";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  stack1 = depth0;
  stack2 = "stackRepoUpdateLinkExists";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                  ");
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "view.useLocalRepoRadioButton";
  stack3 = {};
  stack4 = "radio big-radio col-sm-4";
  stack3['classNames'] = stack4;
  stack4 = "use-local-repo";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n                <div class=\"alert alert-info\"\n                     role=\"alert\">");
  stack1 = depth0;
  stack2 = "installer.step1.useLocalRepo.infoForm.alert.baseUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                ");
  stack1 = depth0;
  stack2 = "view.showWarning";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n                ");
  data.buffer.push("\n                <div class=\"row\">\n                  <div class=\"table-controls row col-sm-12 pull-right\">\n                    <div class=\"col-sm-12\">\n                      ");
  data.buffer.push("\n                      <div class=\"btn-group pull-right\">\n                        <button\n                          type=\"button\" ");
  stack1 = {};
  stack2 = "view.addOsButtonTooltip";
  stack1['data-original-title'] = stack2;
  stack2 = ":btn :btn-default :dropdown-toggle :add-os-button view.isAddOsButtonDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          ");
  stack1 = depth0;
  stack2 = "add-os-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                          data-toggle=\"dropdown\">\n                          <i class=\"glyphicon glyphicon-plus\"></i> ");
  stack1 = depth0;
  stack2 = "common.add";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &nbsp;<span class=\"caret\"></span>\n                        </button>\n                        <ul class=\"dropdown-menu\">\n                          ");
  stack1 = depth0;
  stack2 = "selectedStack.operatingSystems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operatingSystem";
  stack7 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        </ul>\n                      </div>\n                      ");
  data.buffer.push("\n                    </div>\n                  </div>\n                </div>\n                <table class=\"table table-hover\">\n                  <thead>\n                  <tr>\n                    <th class=\"col-sm-2\">");
  stack1 = depth0;
  stack2 = "common.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                    <th class=\"col-sm-3\">");
  stack1 = depth0;
  stack2 = "common.name";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                    <th class=\"col-sm-6\">");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.localRepo.column.baseUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                    <th class=\"col-sm-1\"></th>\n                  </tr>\n                  </thead>\n                  <tbody>\n                  ");
  stack1 = depth0;
  stack2 = "selectedStack.operatingSystems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "operatingSystem";
  stack7 = helpers.each;
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </tbody>\n                </table>\n                ");
  data.buffer.push("\n\n                ");
  data.buffer.push("\n                <div id=\"skip-validation\" ");
  stack1 = {};
  stack2 = "controller.selectedStack.useRedhatSatellite:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "installer.step1.advancedRepo.skipValidation.message";
  stack3['labelTranslate'] = stack4;
  stack4 = "controller.selectedStack.useRedhatSatellite:disabled";
  stack3['labelClassNamesBinding'] = stack4;
  stack4 = "skipValidationChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "controller.selectedStack.useRedhatSatellite";
  stack3['disabledBinding'] = stack4;
  stack4 = "skip-validation";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(30, program30, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  <div class=\"clearfix\"></div>\n                </div>\n                ");
  data.buffer.push("\n\n                ");
  data.buffer.push("\n                <div id=\"use-redhat\">\n                  ");
  stack1 = depth0;
  stack2 = "view.redhatCheckBoxView";
  stack3 = {};
  stack4 = "use-redhat-checkbox";
  stack3['classNames'] = stack4;
  stack4 = "installer.step1.advancedRepo.useRedhatSatellite.message";
  stack3['labelTranslate'] = stack4;
  stack4 = "controller.selectedStack.usePublicRepo:disabled";
  stack3['labelClassNamesBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(32, program32, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n                ");
  data.buffer.push("\n              </div>\n            </div>\n          </form>\n        ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "networkIssuesExist";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                        <a id=\"public-disabled-link\"\n                           class=\"display-inline-block\" ");
  stack1 = depth0;
  stack2 = "openPublicOptionDisabledWindow";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                      ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div\n                    class=\"alert alert-warning\" ");
  stack1 = depth0;
  stack2 = "step1-warning";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step1.attentionNeeded";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n                ");
  return buffer;}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "operatingSystem.isSelected";
  stack3 = helpers.unless;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                              <li>\n                                <a ");
  stack1 = depth0;
  stack2 = "operatingSystem";
  stack3 = depth0;
  stack4 = "addOS";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "add-os-{operatingSystem.osType}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "operatingSystem.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                              </li>\n                            ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "operatingSystem.isSelected";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  return buffer;}
function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      <tr ");
  stack1 = depth0;
  stack2 = "os-wrapper";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                        <td class=\"col-sm-2\" ");
  stack1 = depth0;
  stack2 = "os-type-label";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "operatingSystem.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                        <td class=\"col-sm-9\" colspan=\"2\">\n                          <table\n                            class=\"table table-condensed no-borders inner-table\" ");
  stack1 = depth0;
  stack2 = "{operatingSystem.osType}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                            <tbody>\n                            ");
  stack1 = depth0;
  stack2 = "operatingSystem.repositories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "repository";
  stack7 = helpers.each;
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            </tbody>\n                          </table>\n                        </td>\n                        <td class=\"col-sm-1\">\n                          <a ");
  stack1 = {};
  stack2 = "controller.selectedStack.useRedhatSatellite:disabled :action :remove-icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                             href=\"#\" ");
  stack1 = depth0;
  stack2 = "operatingSystem";
  stack3 = depth0;
  stack4 = "removeOS";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "remove-icon";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                            <span class=\"icon icon-minus\"></span>\n                            ");
  stack1 = depth0;
  stack2 = "common.remove";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                          </a>\n                        </td>\n                      </tr>\n                    ");
  return buffer;}
function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "repository.showRepo";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  return buffer;}
function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                <tr ");
  stack1 = depth0;
  stack2 = "{repository.repoName}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                                  <td class=\"col-sm-3\">\n                                    ");
  stack1 = depth0;
  stack2 = "view.editableRepoView";
  stack3 = {};
  stack4 = "repository";
  stack3['repositoryBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                  </td>\n                                  <td class=\"col-sm-1\">\n                                    ");
  stack1 = depth0;
  stack2 = "view.popoverView";
  stack3 = {};
  stack4 = "repository";
  stack3['repositoryBinding'] = stack4;
  stack4 = "repository.validation";
  stack3['data-qaBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                  </td>\n                                  <td class=\"col-sm-8\">\n                                    <div ");
  stack1 = {};
  stack2 = ":repo-url repository.invalidFormatError:has-error repository.invalidError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                                      ");
  stack1 = depth0;
  stack2 = "view.repositoryTextField";
  stack3 = {};
  stack4 = "repository";
  stack3['repositoryBinding'] = stack4;
  stack4 = "controller.selectedStack.useRedhatSatellite";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                      ");
  stack1 = depth0;
  stack2 = "controller.selectedStack.usePublicRepo";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(28, program28, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                                    </div>\n                                  </td>\n                                </tr>\n                              ");
  return buffer;}
function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                                        <i ");
  stack1 = {};
  stack2 = "repository.undo::invisible :icon-undo";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "doRestoreDefaultValue";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = {};
  stack2 = "common.undo";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = depth0;
  stack2 = "undo-icon";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                                        </i>\n                                      ");
  return buffer;}

function program28(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                                        <i ");
  stack1 = {};
  stack2 = "repository.notEmpty::invisible :icon-undo";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = depth0;
  stack2 = "repository";
  stack3 = depth0;
  stack4 = "doRestoreToEmpty";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = {};
  stack2 = "common.undo";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                                          ");
  stack1 = depth0;
  stack2 = "undo-icon";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                                        </i>\n                                      ");
  return buffer;}

function program30(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n                    <i class=\"glyphicon glyphicon-question-sign\" rel=\"skip-validation-tooltip\"\n                       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step1.advancedRepo.skipValidation.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    </i>\n                  ");
  return buffer;}

function program32(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n                    <i class=\"glyphicon glyphicon-question-sign\" rel=\"use-redhat-tooltip\"\n                       data-toggle=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step1.advancedRepo.useRedhatSatellite.tooltip";
  stack1['data-original-title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    </i>\n                  ");
  return buffer;}

function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"alert alert-warning\">\n            ");
  stack1 = depth0;
  stack2 = "installer.step1.invalidURLAttention";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <a\n              href=\"javascript:void(null)\" ");
  stack1 = depth0;
  stack2 = "retryRepoUrls";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step1.retryRepoUrls";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          </div>\n        ");
  return buffer;}

function program36(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program38(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program40(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

  data.buffer.push("\n<div id=\"select-stack\" class=\"wizard-content col-md-9\" ");
  stack1 = depth0;
  stack2 = "select-stack-page";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step1.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step1.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "isLoadingComplete";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(36, program36, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <button type=\"button\" class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n    &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(38, program38, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </button>\n  ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  <button type=\"button\" class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "view.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n    ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(40, program40, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n  </button>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step1/editable_repo", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.repository.repoId";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.repository.repoId";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <i class=\"icon-pencil\" ");
  stack1 = depth0;
  stack2 = "editRepoId";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <i class=\"icon-undo\" ");
  stack1 = depth0;
  stack2 = "revertToOriginal";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isEditing";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.showEditIcon";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  stack1 = depth0;
  stack2 = "view.showRevertIcon";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step1/public_option_disabled_window_body", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n\n<p class=\"public-disabled-message\">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost.popup.msg";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n<ul>\n  <li class=\"public-disabled-option\">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost.popup.msg1";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n  <li class=\"public-disabled-option\">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost.popup.msg2";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n  <li class=\"public-disabled-option\">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost.popup.msg3";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</li>\n</ul>\n<p class=\"public-disabled-message\">");
  stack1 = depth0;
  stack2 = "installer.step1.selectUseRepoOptions.public.networkLost.popup.msg4";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step1/vdf_upload", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"col-md-5 option-radio-button\">\n      ");
  stack1 = depth0;
  stack2 = "view.uploadFileRadioButton";
  stack3 = {};
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "local-option-label";
  stack3['labelClassNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"col-md-12\">\n      ");
  stack1 = depth0;
  stack2 = "view.isFileApi";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "view.fileInputView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div class=\"option-radio-button col-md-12\">\n      ");
  stack1 = depth0;
  stack2 = "view.enterUrlRadioButton";
  stack3 = {};
  stack4 = "radio";
  stack3['classNames'] = stack4;
  stack4 = "local-option-label";
  stack3['labelClassNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"col-md-12 vdf-url\">\n      <div ");
  stack1 = {};
  stack2 = "optionsToSelect.useLocalRepo.enterUrl.name optionsToSelect.useLocalRepo.enterUrl.url.hasError:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "view.enterUrlField";
  stack3 = {};
  stack4 = "input-block-level";
  stack3['class'] = stack4;
  stack4 = "optionsToSelect.useLocalRepo.enterUrl.url";
  stack3['valueBinding'] = stack4;
  stack4 = "optionsToSelect.useLocalRepo.enterUrl.placeholder";
  stack3['placeholderBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    </div>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"clearfix register-version-options row\">\n  ");
  stack1 = depth0;
  stack2 = "view.uploadFileView";
  stack3 = {};
  stack4 = "col-md-12";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.enterUrlView";
  stack3 = {};
  stack4 = "col-md-12";
  stack3['class'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step10", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    <div id=\"step10-alert-message\" class=\"alert alert-danger\">\n      ");
  stack1 = depth0;
  stack2 = "view.serviceRestartText";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <li>\n              <span ");
  stack1 = {};
  stack2 = "item.color";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "item.displayStatement";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "item.status";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "status";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </li>\n          ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <li>\n                    <span ");
  stack1 = {};
  stack2 = "status.color";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "status.displayStatement";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    <ul>\n                      ");
  stack1 = depth0;
  stack2 = "status.statements";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "statement";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                  </li>\n                ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <li>\n                          <span ");
  stack1 = {};
  stack2 = "status.color";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "statement.displayStatement";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                        </li>\n                      ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n<div id=\"deploy-summary\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step10.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  ");
  stack1 = depth0;
  stack2 = "view.serviceRestartText";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "installer.step10.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"step10-content\" class=\"well pre-scrollable\" ");
  stack1 = depth0;
  stack2 = "summary";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <ul>\n          ");
  stack1 = depth0;
  stack2 = "clusterInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "complete";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.complete";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step1_addLocalRepository", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <li>\n              <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "os";
  stack3 = depth0;
  stack4 = "selectOS";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "os.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </a>\n            </li>\n          ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n	    <div class=\"alert alert-warning\">\n			  <strong>");
  stack1 = depth0;
  stack2 = "common.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</strong>  ");
  stack1 = depth0;
  stack2 = "view.errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n			</div>\n	  ");
  return buffer;}

function program5(depth0,data) {
  
  
  data.buffer.push("\n	    <div class=\"alert alert-info\">\n        Provide location of the local repository for the selected OS\n      </div>\n		");}

  data.buffer.push("\n  <form autocomplete=\"off\" class=\"form-horizontal\" id=\"addLocalRepositoryPopup\">\n    <div class=\"each-row\">\n      <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.localRepo.label.os";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      <div class=\"btn-group\">\n        <a class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" href=\"#\">\n          ");
  stack1 = depth0;
  stack2 = "view.selectedOS.osType";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          <span class=\"caret\"></span>\n        </a>\n        <ul class=\"dropdown-menu\">\n          ");
  stack1 = depth0;
  stack2 = "view.oses";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "os";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </ul>\n      </div>\n    </div>\n    <div class=\"each-row\">\n      <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.localRepo.label.baseUrl";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      ");
  stack1 = depth0;
  stack2 = "Em.TextField";
  stack3 = {};
  stack4 = "view.enteredUrl";
  stack3['valueBinding'] = stack4;
  stack4 = "col-md-6 form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n    <div class=\"each-row\">\n      <label class=\"control-label\">");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.localRepo.label.stack";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</label>\n      ");
  stack1 = depth0;
  stack2 = "view.stackName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    </div>\n\n    ");
  stack1 = depth0;
  stack2 = "view.errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(5, program5, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </form>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step2", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <div class=\"col-md-4\">\n                <p class=\"help-block validation-block\" ");
  stack1 = depth0;
  stack2 = "host-names-validation";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "hostsError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n              </div>\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '';
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "installer.step2.useSsh.provide";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                <a href=\"javascript:void(null)\"\n                   rel=\"popover\"\n                  ");
  stack1 = {};
  stack2 = "installer.step2.useSsh.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step2.useSsh.tooltip.content";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = depth0;
  stack2 = "ssh-tooltip-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.useSsh.tooltip.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                ");
  stack1 = depth0;
  stack2 = "installer.step2.useSsh.provide_id_rsa";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "installer.step2.install.perform";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                <a href=\"javascript:void(null)\"\n                   rel=\"popover\"\n                  ");
  stack1 = {};
  stack2 = "installer.step2.automaticInstall.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step2.automaticInstall.tooltip.content";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = depth0;
  stack2 = "ssh-tooltip-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.automaticInstall.tooltip.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                ");
  stack1 = depth0;
  stack2 = "installer.step2.install.perform_on_hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '';
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <a href=\"javascript:void(null)\"\n                   rel=\"popover\"\n                  ");
  stack1 = {};
  stack2 = "installer.step2.manualInstall.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step2.manualInstall.tooltip.content";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = depth0;
  stack2 = "ssh-tooltip-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.manualInstall.tooltip.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <a href=\"javascript:void(null)\"\n                   rel=\"popover\"\n                  ");
  stack1 = {};
  stack2 = "installer.step2.manualInstall.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step2.manualInstall.tooltip.content_no_ssh";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = depth0;
  stack2 = "ssh-tooltip-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.manualInstall.tooltip.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "installer.step2.install.without_ssh";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program17(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div class=\"ssh-key-input\">\n            ");
  stack1 = depth0;
  stack2 = "view.isFileApi";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            ");
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "sshKeyError:has-error :form-group :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"col-md-8\">\n                ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "3";
  stack3['rows'] = stack4;
  stack4 = "sshKey";
  stack3['id'] = stack4;
  stack4 = "ssh private key";
  stack3['placeholder'] = stack4;
  stack4 = "view.sshKeyState";
  stack3['disabledBinding'] = stack4;
  stack4 = "content.installOptions.sshKey";
  stack3['valueBinding'] = stack4;
  stack4 = "ssh-key-textarea";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n              ");
  stack1 = depth0;
  stack2 = "sshKeyError";
  stack3 = helpers['if'];
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            ");
  data.buffer.push("\n\n            ");
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "sshUserError:has-error :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"col-md-4\">\n                <label rel=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step2.sshUser.toolTip";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"ssh-user wizard-plain-text\">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.sshUser";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </label>\n              </div>\n              <div ");
  stack1 = {};
  stack2 = "sshUserError:has-error :form-group :col-md-4";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "view.textFieldView";
  stack3 = {};
  stack4 = "content.installOptions.sshUser";
  stack3['valueBinding'] = stack4;
  stack4 = "content.installOptions.useSsh";
  stack3['isEnabledBinding'] = stack4;
  stack4 = "form-control pull-right";
  stack3['classNames'] = stack4;
  stack4 = "ssh-user-input";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n              ");
  stack1 = depth0;
  stack2 = "sshUserError";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            ");
  data.buffer.push("\n\n            ");
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "sshPortError:has-error :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"col-md-4\">\n                <label rel=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step2.sshPort.toolTip";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"ssh-port wizard-plain-text\">\n                  ");
  stack1 = depth0;
  stack2 = "installer.step2.sshPort";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </label>\n              </div>\n              <div class=\"form-group col-md-4\">\n                ");
  stack1 = depth0;
  stack2 = "view.textFieldView";
  stack3 = {};
  stack4 = "content.installOptions.sshPort";
  stack3['valueBinding'] = stack4;
  stack4 = "content.installOptions.useSsh";
  stack3['isEnabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n              ");
  stack1 = depth0;
  stack2 = "sshPortError";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            ");
  data.buffer.push("\n\n            ");
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "App.supports.customizeAgentUserAccount";
  stack3 = helpers['if'];
  tmp1 = self.program(26, program26, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <div>\n                ");
  stack1 = depth0;
  stack2 = "App.SshKeyFileUploader";
  stack3 = {};
  stack4 = "view.sshKeyState";
  stack3['disabledBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </div>\n            ");
  return buffer;}

function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"col-md-4\">\n                  <span class=\"help-block validation-block\" ");
  stack1 = depth0;
  stack2 = "ssh-key-validation";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "sshKeyError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                </div>\n              ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"col-md-4\">\n                  <span class=\"help-block validation-block help-inline\">");
  stack1 = depth0;
  stack2 = "sshUserError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                </div>\n              ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div class=\"col-md-4\">\n                  <span class=\"help-block validation-block help-inline\">");
  stack1 = depth0;
  stack2 = "sshPortError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                </div>\n              ");
  return buffer;}

function program26(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <div ");
  stack1 = {};
  stack2 = "agentUserError:has-error :row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <div class=\"col-md-4\">\n                  <label rel=\"tooltip\" ");
  stack1 = {};
  stack2 = "installer.step2.agentUser.toolTip";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " class=\"ssh-user wizard-plain-text\">\n                    ");
  stack1 = depth0;
  stack2 = "installer.step2.agentUser";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </label>\n                </div>\n                <div ");
  stack1 = {};
  stack2 = "agentUserError:has-error :form-group :col-md-4";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "view.textFieldView";
  stack3 = {};
  stack4 = "content.installOptions.agentUser";
  stack3['valueBinding'] = stack4;
  stack4 = "content.installOptions.useSsh";
  stack3['isEnabledBinding'] = stack4;
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n                ");
  stack1 = depth0;
  stack2 = "agentUserError";
  stack3 = helpers['if'];
  tmp1 = self.program(27, program27, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            ");
  return buffer;}
function program27(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <div class=\"col-md-4\">\n                    <span class=\"help-block validation-block help-inline\">");
  stack1 = depth0;
  stack2 = "agentUserError";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  </div>\n                ");
  return buffer;}

function program29(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <br>\n        ");
  stack1 = depth0;
  stack2 = "view.skipHostsCheckBox";
  stack3 = {};
  stack4 = "content.installOptions.skipHostChecks";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "installer.step2.skipHostChecks.label";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program31(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <button class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(32, program32, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </button>\n    ");
  return buffer;}
function program32(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program34(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"installOptions\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step2.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step2.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n\n      <div id=\"targetHosts\" ");
  stack1 = depth0;
  stack2 = "target-hosts";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"step-title\" ");
  stack1 = depth0;
  stack2 = "target-hosts-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step2.targetHosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div ");
  stack1 = {};
  stack2 = "hostsError:has-error :form-group :target-hosts-input";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <p class=\"wizard-plain-text\" ");
  stack1 = depth0;
  stack2 = "target-hosts-description";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step2.targetHosts.info";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ". ");
  stack1 = depth0;
  stack2 = "installer.step2.orUse";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            <a href=\"javascript:void(null)\"\n               rel=\"popover\"\n              ");
  stack1 = {};
  stack2 = "installer.step2.hostPattern.tooltip.title";
  stack1['title'] = stack2;
  stack2 = "installer.step2.hostPattern.tooltip.content";
  stack1['data-content'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "pattern-expressions-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <span ");
  stack1 = depth0;
  stack2 = "pattern-expressions-link-text";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step2.hostPattern.tooltip.title";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n            </a>\n          </p>\n\n          <div class=\"row\">\n            <div class=\"col-md-8\">\n              ");
  stack1 = depth0;
  stack2 = "Ember.TextArea";
  stack3 = {};
  stack4 = "host-names";
  stack3['id'] = stack4;
  stack4 = "form-control";
  stack3['class'] = stack4;
  stack4 = "content.installOptions.hostNames";
  stack3['valueBinding'] = stack4;
  stack4 = "5";
  stack3['rows'] = stack4;
  stack4 = "host names";
  stack3['placeholder'] = stack4;
  stack4 = "target-hosts-textarea";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            ");
  stack1 = depth0;
  stack2 = "hostsError";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n\n      <div ");
  stack1 = depth0;
  stack2 = "host-registration-info";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"step-title\" ");
  stack1 = depth0;
  stack2 = "host-registration-info-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step2.sshKey";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        <div class=\"row radio-button-options\">\n          <div class=\"col-sm-5 row\">\n            <div class=\"col-sm-1\">\n              ");
  stack1 = depth0;
  stack2 = "view.providingSSHKeyRadioButton";
  stack3 = {};
  stack4 = "ssh-key-radio-button";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"col-sm-10 wizard-plain-text\">\n              ");
  stack1 = depth0;
  stack2 = "useSSH";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n          <div class=\"col-sm-5 row\">\n            <div class=\"col-sm-1\">\n              ");
  stack1 = depth0;
  stack2 = "view.manualRegistrationRadioButton";
  stack3 = {};
  stack4 = "manual-registration-radio-button";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n            <div class=\"col-sm-10 wizard-plain-text\">\n              ");
  stack1 = depth0;
  stack2 = "installer.step2.install.perform";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "useSSH";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "installer.step2.install.perform_on_hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  stack1 = depth0;
  stack2 = "useSSH";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </div>\n          </div>\n        </div>\n\n        ");
  stack1 = depth0;
  stack2 = "useSSH";
  stack3 = helpers['if'];
  tmp1 = self.program(17, program17, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "isAddHostWizard";
  stack3 = helpers['if'];
  tmp1 = self.program(29, program29, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.controller.hideBackButton";
  stack3 = helpers.unless;
  tmp1 = self.program(31, program31, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <button class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "evaluateStep";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(34, program34, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "installer.step2.registerAndConfirm";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step2ManualInstallPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "installer.step2.manualInstall.popup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step2_installed_hosts_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "installer.step2.evaluateStep.installedHosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n<p>");
  stack1 = depth0;
  stack2 = "view.inputtedAgainHostNames";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n<p>");
  stack1 = depth0;
  stack2 = "installer.step2.evaluateStep.continueConfirm";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step3", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "removeSelectedHosts";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "remove-selected-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-trash glyphicon-white\"></i>\n                <span class=\"hidden-sm\">");
  stack1 = depth0;
  stack2 = "installer.step3.removeSelected";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </button>\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <button class=\"btn btn-primary\" ");
  stack1 = depth0;
  stack2 = "retrySelectedHosts";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "retry-failed-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>&nbsp;\n                <span class=\"hidden-sm\">");
  stack1 = depth0;
  stack2 = "installer.step3.retryFailed";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n              </button>\n            ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <li ");
  stack1 = {};
  stack2 = ":filter-status category.itemClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "hosts-filter-{category.hostsBootStatus}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                      ");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                  </li>\n                  ");
  stack1 = depth0;
  stack2 = "category.last";
  stack3 = helpers.unless;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program6(depth0,data) {
  
  
  data.buffer.push("\n                    <li class=\"divider\">|</li>\n                  ");}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "App.WizardHostView";
  stack3 = {};
  stack4 = "controller.category";
  stack3['categoryBinding'] = stack4;
  stack4 = "host";
  stack3['hostInfoBinding'] = stack4;
  stack4 = "confirm-hosts-table-body-row";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                    <td class=\"step3-table-checkbox\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "host.isChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "select-host-checkbox";
  stack3['labelIdentifier'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    </td>\n                    <td class=\"host\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      <span title=\"");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" class=\"trim_hostname\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-host-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    </td>\n                    <td class=\"step3-table-progress\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      <div class=\"progress\">\n                        <div ");
  stack1 = {};
  stack2 = "host.bootBarColor host.isBootDone::progress-bar-striped host.isBootDone::active :progress-bar";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          ");
  stack1 = depth0;
  stack2 = "{host.bootBarColor}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                          style=\"width:100%\">\n                        </div>\n                      </div>\n                    </td>\n                    <td class=\"step3-table-status\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      <a href=\"javascript:void(null)\" data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "host";
  stack3 = depth0;
  stack4 = "hostLogPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-status-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                        <span ");
  stack1 = {};
  stack2 = "host.bootStatusColor";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-status-{host.bootStatus}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.bootStatusForDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                      </a>\n                    </td>\n                    <td class=\"step3-table-action\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      <a class=\"action\" id=\"display-action\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "remove";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isBackDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "remove-host-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                        <i class=\"glyphicon glyphicon-trash\" ");
  stack1 = {};
  stack2 = "common.remove";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n                      </a>\n                    </td>\n                    <!-- empty td to contain extra space of table -->\n                    <td></td>\n                  ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <tr ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  <td class=\"step3-table-no-hosts\" colspan=\"5\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    ");
  stack1 = depth0;
  stack2 = "hosts.table.noHosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                  </td>\n                </tr>\n              ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                    <a ");
  stack1 = depth0;
  stack2 = "selectedHostsPopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                      ");
  stack1 = depth0;
  stack2 = "view.selectedHostsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                      ");
  stack1 = depth0;
  stack2 = "view.selectedHostsCount";
  stack3 = {};
  stack4 = "t:hosts.filters.selectedHostInfo";
  stack3['singular'] = stack4;
  stack4 = "t:hosts.filters.selectedHostsInfo";
  stack3['plural'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n                    </a>\n                    -\n                    <a ");
  stack1 = depth0;
  stack2 = "unSelectAll";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">");
  stack1 = depth0;
  stack2 = "hosts.filters.clearSelection";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":alert :alert-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "alert-warning";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "registeredHostsPopup";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "alert-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "><span ");
  stack1 = depth0;
  stack2 = "other-hosts-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.registeredHostsMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span></a>\n          </div>\n        ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n          <div ");
  stack1 = {};
  stack2 = ":alert :alert-warning view.status isWarningsBoxVisible::hidden";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "{view.status}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "warnings";
  stack3 = depth0;
  stack4 = "hostWarningsPopup";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "alert-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.linkText";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n            ");
  stack1 = depth0;
  stack2 = "isBootstrapFailed";
  stack3 = helpers.unless;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "isWarningsLoaded";
  stack3 = helpers.unless;
  tmp1 = self.program(20, program20, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program20(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "spinner";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              ");
  return buffer;}

function program22(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program24(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"confirm-hosts\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step3.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\" ");
  stack1 = depth0;
  stack2 = "step3-description";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step3.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n\n      <div class=\"panel panel-default panel-internal\">\n        <div class=\"panel-heading compact-panel-heading\">\n          <div class=\"button-section\">\n            ");
  stack1 = depth0;
  stack2 = "view.noNotRunningHostsSelected";
  stack3 = helpers.unless;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "isRetryDisabled";
  stack3 = helpers.unless;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n            <div id=\"host-filter\" class=\"pull-right\">\n              <ul class=\"clearfix\">\n                <li class=\"first\">");
  stack1 = depth0;
  stack2 = "common.show";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</li>\n                ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          </div>\n          <div class=\"clearfix\"></div>\n        </div>\n        <div class=\"panel-body hosts-panel\">\n          <div class=\"pre-scrollable col-sm-12\" style=\"max-height: 440px;\">\n            <table style=\"table-layout: fixed\" id=\"confirm-hosts-table\" class=\"table table-hover\" ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <thead>\n              <tr>\n                <th ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-heading-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "view.pageChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </th>\n                <th ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-heading-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                <!-- retrieved from local storage initially -->\n                <th ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-heading-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.progress";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                <th ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-heading-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                <!-- given by the parsing function that parses data from bootstrap call, dynamically assign the color -->\n                <th ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-heading-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.action";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n                <!-- empty th to contain extra space of table -->\n                <th></th>\n                <!-- trash icon -->\n                <!-- retry icon -->\n              </tr>\n              </thead>\n              <tbody ");
  stack1 = depth0;
  stack2 = "confirm-hosts-table-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(12, program12, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tbody>\n            <tfoot>\n              <tr>\n                <td colspan=\"5\">\n                  ");
  stack1 = depth0;
  stack2 = "view.selectedHostsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n              </tr>\n            </tfoot>\n          </table>\n        </div>\n      </div>\n      <div id=\"warningsSection\">\n        ");
  stack1 = depth0;
  stack2 = "hasMoreRegisteredHosts";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "disableHostCheck";
  stack3 = helpers.unless;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  </div>\n</div>\n</div>\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = {};
  stack2 = "isBackButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n      &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(22, program22, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </button>\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <button type=\"button\" class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isNextButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(24, program24, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n  </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step3/step3_host_log_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div>\n        ");
  stack1 = depth0;
  stack2 = "view.textArea";
  stack3 = {};
  stack4 = "view.bootLog";
  stack3['contentBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <pre class=\"bootLog\">");
  stack1 = depth0;
  stack2 = "view.bootLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n    ");
  return buffer;}

  data.buffer.push("\n\n\n<div id=\"host-log\">\n  <div class=\"content-area\">\n    <div class=\"textTrigger\">");
  stack1 = depth0;
  stack2 = "popup.highlight";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n    ");
  stack1 = depth0;
  stack2 = "view.isTextArea";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step3/step3_host_warning_popup_footer", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2;
  data.buffer.push("\n    <div class=\"progress\">\n      <div class=\"progress-bar-info active progress-bar\" ");
  stack1 = {};
  stack2 = "view.progressWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></div>\n    </div>\n  ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <span ");
  stack1 = {};
  stack2 = "view.updateStatusClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-checks-result-{view.footerController.checksUpdateStatus}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.updateStatus";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n  ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n  <button type=\"button\" class=\"btn btn-warning\"\n    ");
  stack1 = {};
  stack2 = "view.isUpdateInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "onSecondary";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "modal-secondary";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"glyphicon glyphicon-repeat\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "view.parentView.secondary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </button>");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("<button type=\"button\" class=\"btn btn-default\"\n    ");
  stack1 = depth0;
  stack2 = "onPrimary";
  stack3 = {};
  stack4 = "view.parentView";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  </button>\n");
  return buffer;}

  data.buffer.push("\n\n<div class=\"update-progress pull-left\">\n  ");
  stack1 = depth0;
  stack2 = "view.isUpdateInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  stack1 = depth0;
  stack2 = "view.parentView.secondary";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  stack1 = depth0;
  stack2 = "view.parentView.primary";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step3/step3_host_warnings_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div id=\"host-warnings\">\n      <div class=\"notice\">\n        <span>");
  stack1 = depth0;
  stack2 = "installer.step3.hostWarningsPopup.checks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " <b>");
  stack1 = depth0;
  stack2 = "view.warningsNotice";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b>.<br>\n          ");
  stack1 = depth0;
  stack2 = "installer.step3.hostWarningsPopup.notice.beginning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</span>\n      </div>\n      <div class=\"row\">\n        <form autocomplete=\"off\" class=\"form-horizontal\">\n          <div class=\"from-group\">\n            <div class=\"col-md-2\">\n              <label class=\"control-label\">\n                ");
  stack1 = depth0;
  stack2 = "common.hosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </label>\n            </div>\n            <div class=\"col-md-5\">\n              ");
  stack1 = depth0;
  stack2 = "view.hostSelectView";
  stack3 = {};
  stack4 = "form-control input-sm";
  stack3['classNames'] = stack4;
  stack4 = "host-select";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n            ");
  stack1 = depth0;
  stack2 = "view.totalWarningsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </form>\n      </div>\n      <div class=\"accordion\">\n        <div class=\"panel-group warnings-list\" id=\"accordion2\" ");
  stack1 = depth0;
  stack2 = "host-checks-list";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          ");
  stack1 = depth0;
  stack2 = "view.content";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n              <div class=\"col-md-3 col-md-offset-2\">\n                <a href=\"javascript:void(null)\" title=\"Show Details\" ");
  stack1 = depth0;
  stack2 = "openWarningsInDialog";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                   class=\"control-label display-inline-block\">\n                   <i class=\"icon-external-link\"></i>&nbsp;\n                   ");
  stack1 = depth0;
  stack2 = "installer.step3.hostWarningsPopup.report";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n                </a>\n              </div>\n            ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n            <div class=\"panel panel-default\" ");
  stack1 = depth0;
  stack2 = "host-checks-category";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n              <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "onToggleBlock";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-checks-category-heading";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "category.warnings.length";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                <i ");
  stack1 = {};
  stack2 = ":pull-right :panel-toggle category.isCollapsed:icon-angle-down:icon-angle-up";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                <a class=\"panel-toggle\" ");
  stack1 = depth0;
  stack2 = "{category.category}-issues";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "category.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " (");
  stack1 = depth0;
  stack2 = "category.warnings.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ")\n                </a>\n              </div>\n              <div id=\"");
  stack1 = depth0;
  stack2 = "category.category";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" class=\"panel-body collapse in\"\n                   style=\"display: none\" ");
  stack1 = depth0;
  stack2 = "host-checks-category-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "category.warnings.length";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(19, program19, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </div>\n            </div>\n          ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <i class=\"pull-left icon-warning-sign\" ");
  stack1 = depth0;
  stack2 = "host-checks-category-warning";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <i class=\"pull-left glyphicon glyphicon-ok\"");
  stack1 = depth0;
  stack2 = "host-checks-category-success";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n                ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <table class=\"table\">\n                    <thead ");
  stack1 = depth0;
  stack2 = "host-checks-category-description";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <tr>\n                      <th colspan=\"4\">");
  stack1 = depth0;
  stack2 = "category.message";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</th>\n                    </tr>\n                    <tr>\n                      <th colspan=\"4\"><b>");
  stack1 = depth0;
  stack2 = "category.type";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b></th>\n                    </tr>\n                    </thead>\n                    <tbody>\n                    ");
  stack1 = depth0;
  stack2 = "category.warnings";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "warning";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </tbody>\n                  </table>\n                ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n                      <tr>\n                        <td class=\"warning-name\" ");
  stack1 = {};
  stack2 = "warning.command";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-warning-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "warning.name";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</td>\n                        ");
  stack1 = depth0;
  stack2 = "warning.version";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(13, program13, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "warning.target";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(17, program17, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        <td>");
  stack1 = depth0;
  stack2 = "category.action";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                          <a href=\"javascript:void(null);\"\n                             rel='HostsListTooltip' ");
  stack1 = {};
  stack2 = "warning.hostsList";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "warning.hostsLong";
  stack3 = depth0;
  stack4 = "showHostsPopup";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-with-warning-link";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                            ");
  stack1 = depth0;
  stack2 = "warning.hosts.length";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                            ");
  stack1 = depth0;
  stack2 = "warning.hosts.length";
  stack3 = {};
  stack4 = "t:installer.step3.hostWarningsPopup.host";
  stack3['singular'] = stack4;
  stack4 = "t:installer.step3.hostWarningsPopup.hosts";
  stack3['plular'] = stack4;
  foundHelper = helpers.pluralize;
  stack4 = foundHelper || depth0.pluralize;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack4 === functionType) { stack1 = stack4.call(depth0, stack2, tmp1); }
  else if(stack4=== undef) { stack1 = helperMissing.call(depth0, "pluralize", stack2, tmp1); }
  else { stack1 = stack4; }
  data.buffer.push(escapeExpression(stack1) + "\n                          </a>\n                        </td>\n                      </tr>\n                    ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          <td class=\"package-version\">");
  stack1 = depth0;
  stack2 = "warning.version";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                        ");
  return buffer;}

function program13(depth0,data) {
  
  
  data.buffer.push("\n                          <td></td>\n                        ");}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          <td class=\"package-version\">");
  stack1 = depth0;
  stack2 = "warning.target";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                        ");
  return buffer;}

function program17(depth0,data) {
  
  
  data.buffer.push("\n                          <td></td>\n                        ");}

function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "category.emptyName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.checkHostFinished";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(21, program21, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step3/step3_registered_hosts_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <li ");
  stack1 = depth0;
  stack2 = "unused-host";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n  ");
  return buffer;}

  data.buffer.push("\n\n<p>");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n<ul>\n  ");
  stack1 = depth0;
  stack2 = "view.registeredHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</ul>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step4", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "service.isHiddenOnSelectServicePage";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n            <tr ");
  stack1 = depth0;
  stack2 = "service-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "service.isSelected:active service.isSelected:service-selected";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "service";
  stack3 = depth0;
  stack4 = "toggleCheckBox";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " >\n              <td class=\"checkbox-cell\">\n                <div class=\"checkbox\">\n                  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "service.serviceName";
  stack3['checkboxClassNamesBinding'] = stack4;
  stack4 = "toggle-service";
  stack3['data-qa'] = stack4;
  stack4 = "service.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "service.isSelected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </div>\n              </td>\n              <td ");
  stack1 = depth0;
  stack2 = "service-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.displayNameOnSelectServicePage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td ");
  stack1 = depth0;
  stack2 = "service-version";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.serviceVersionDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n              <td ");
  stack1 = depth0;
  stack2 = "service-description";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.comments";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</td>\n            </tr>\n          ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "service.isHiddenOnSelectServicePage";
  stack3 = helpers.unless;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "service.isDFS";
  stack3 = helpers.unless;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <tr ");
  stack1 = depth0;
  stack2 = "service-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "service.isSelected:active service.isSelected:service-selected";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "service";
  stack3 = depth0;
  stack4 = "toggleCheckBox";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <td class=\"checkbox-cell\">\n                  <div class=\"checkbox\">\n                    ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "service.serviceName";
  stack3['checkboxClassNamesBinding'] = stack4;
  stack4 = "toggle-service";
  stack3['data-qa'] = stack4;
  stack4 = "service.isDisabled";
  stack3['disabledBinding'] = stack4;
  stack4 = "service.isSelected";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </div>\n                </td>\n                <td ");
  stack1 = depth0;
  stack2 = "service-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.displayNameOnSelectServicePage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td ");
  stack1 = depth0;
  stack2 = "service-version";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.serviceVersionDisplay";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n                <td ");
  stack1 = depth0;
  stack2 = "service-description";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "service.comments";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</td>\n              </tr>\n            ");
  return buffer;}

function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </button>\n    ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"step4\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step4.headerFS";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step4.bodyFS";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <table class=\"table table-hover\">\n        <thead>\n        <tr>\n          <th class=\"checkbox-row\"></th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-name-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"col-md-3\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-version-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " id=\"service-version\">");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-description-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " id=\"service-description\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "controller.fileSystems";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n    </div>\n  </div>\n\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step4.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step4.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <table class=\"table table-hover\">\n        <thead>\n        <tr>\n          <th>\n            ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "toggle-all";
  stack3['data-qa'] = stack4;
  stack4 = "isInstalled";
  stack3['disabledBinding'] = stack4;
  stack4 = "isAllChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-name-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"col-md-3\">");
  stack1 = depth0;
  stack2 = "common.service";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-version-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " id=\"service-version\">");
  stack1 = depth0;
  stack2 = "common.version";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n          <th ");
  stack1 = depth0;
  stack2 = "service-description-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " id=\"service-description\">");
  stack1 = depth0;
  stack2 = "common.description";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n        </tr>\n        </thead>\n        <tbody>\n        ");
  stack1 = depth0;
  stack2 = "controller";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "service";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </tbody>\n      </table>\n    </div>\n  </div>\n</div>\n\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    ");
  stack1 = depth0;
  stack2 = "view.parentView.controller.hideBackButton";
  stack3 = helpers.unless;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step4/step4_ranger_requirements_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;


  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "installer.step4.rangerRequirements.popup.body.requirements";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n<div class=\"checkbox\">\n  ");
  stack1 = depth0;
  stack2 = "App.CheckboxView";
  stack3 = {};
  stack4 = "ranger-requirements-confirmation";
  stack3['data-qa'] = stack4;
  stack4 = "isChecked";
  stack3['checkedBinding'] = stack4;
  stack4 = "installer.step4.rangerRequirements.popup.body.confirmation";
  stack3['labelTranslate'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step4/step4_service_validation_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<h5><strong>");
  stack1 = depth0;
  stack2 = "view.serviceHeader";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</strong></h5>\n");
  stack1 = depth0;
  stack2 = "view.serviceBody";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1));
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step6", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p>");
  stack1 = depth0;
  stack2 = "installer.step6.validationSlavesAndClients.popup.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n        <div class=\"limited-height-2\">\n          ");
  stack1 = depth0;
  stack2 = "anyGeneralErrors";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n          ");
  stack1 = depth0;
  stack2 = "anyGeneralWarnings";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div class=\"alert alert-danger\">\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "controller.generalErrorMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "msg";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <li>");
  stack1 = depth0;
  stack2 = "errorMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <li>");
  stack1 = depth0;
  stack2 = "msg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div class=\"alert alert-warning\">\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "controller.generalWarningMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "msg";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <li>");
  stack1 = depth0;
  stack2 = "msg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</li>\n                ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n              <th ");
  stack1 = {};
  stack2 = "header.name";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "select-all";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "header.allChecked:selected:deselected header.isDisabled:remove-link";
  stack1['class'] = stack2;
  stack2 = "header.allId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  stack1 = depth0;
  stack2 = "header";
  stack3 = depth0;
  stack4 = "selectAllNodes";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "all";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a> &nbsp;|&nbsp; <a\n                ");
  stack1 = depth0;
  stack2 = "deselect-all";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " href=\"#\" ");
  stack1 = {};
  stack2 = "header.noChecked:selected:deselected header.isDisabled:remove-link";
  stack1['class'] = stack2;
  stack2 = "header.noneId";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                ");
  stack1 = depth0;
  stack2 = "header";
  stack3 = depth0;
  stack4 = "deselectAllNodes";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "none";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n              </th>\n            ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n              <tr ");
  stack1 = depth0;
  stack2 = "host-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "host.isLast:last-row";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "App.WizardStep6HostView";
  stack3 = {};
  stack4 = "host";
  stack3['hostBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(13, program13, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "host.checkboxes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "checkbox";
  stack7 = helpers.each;
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </tr>\n              ");
  stack1 = depth0;
  stack2 = "host.anyMessage";
  stack3 = helpers['if'];
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <span class=\"trim_hostname\">");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  ");
  stack1 = depth0;
  stack2 = "host.hasMaster";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    <i ");
  stack1 = depth0;
  stack2 = "has-masters";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"glyphicon glyphicon-asterisks\">&#10037;</i>\n                  ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                  <td ");
  stack1 = depth0;
  stack2 = "toggle-component";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "checkbox.hasErrorMessage:error checkbox.hasWarnMessage:warning checkbox.component";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                    <div class=\"checkbox\" ");
  stack1 = {};
  stack2 = "checkbox.dataQaAttr";
  stack1['data-qa'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                        <input ");
  stack1 = {};
  stack2 = "checkbox.uId";
  stack1['id'] = stack2;
  stack2 = "checkbox.checked";
  stack1['checked'] = stack2;
  stack2 = "checkbox.isDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "checkbox";
  stack3 = depth0;
  stack4 = "checkboxClick";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                                type=\"checkbox\"/>\n                      <label class=\"host-component-checkbox\" ");
  stack1 = {};
  stack2 = "checkbox.uId";
  stack1['for'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "checkbox.title";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</label>\n                    </div>\n                  </td>\n                ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <tr>\n                  <td ");
  stack1 = {};
  stack2 = "view.columnCount";
  stack1['colspan'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " class=\"no-borders\">\n                    ");
  stack1 = depth0;
  stack2 = "host.errorMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "errorMsg";
  stack7 = helpers.each;
  tmp1 = self.program(19, program19, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "host.warnMessages";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "warnMsg";
  stack7 = helpers.each;
  tmp1 = self.program(21, program21, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </td>\n                </tr>\n              ");
  return buffer;}
function program19(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <div class=\"alert alert-danger\">");
  stack1 = depth0;
  stack2 = "errorMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    ");
  return buffer;}

function program21(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                      <div class=\"alert alert-warning\">");
  stack1 = depth0;
  stack2 = "warnMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n                    ");
  return buffer;}

function program23(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program25(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"step6\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\" ");
  stack1 = depth0;
  stack2 = "step-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step6.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "view.label";
  stack3 = {};
  stack4 = "true";
  stack3['unescaped'] = stack4;
  stack4 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "anyErrors";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n      <div class=\"pre-scrollable col-sm-12\">\n        <table class=\"table table-hover\" id=\"component_assign_table\" ");
  stack1 = depth0;
  stack2 = "slave-clients-table";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          <thead>\n          <tr>\n            <th class=\"host-column freeze\">");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n            ");
  stack1 = depth0;
  stack2 = "controller.headers";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "header";
  stack7 = helpers.each;
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tr>\n          </thead>\n          <tbody>\n            ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n          <tfoot>\n            <tr>\n              <td colspan=\"100\">\n                ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = {};
  stack4 = "static-pagination";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n              </td>\n            </tr>\n          </tfoot>\n        </table>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default installer-back-btn pull-left\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(23, program23, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </button>\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success pull-right\" ");
  stack1 = {};
  stack2 = "submitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(25, program25, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.ServicesConfigView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "runPreInstallChecks";
  stack3 = {};
  stack4 = "App.router.preInstallChecksController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-exclamation\"></i> ");
  stack1 = depth0;
  stack2 = "installer.step7.preInstallChecks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"serviceConfig\" class=\"wizard-content col-md-9\" ");
  stack1 = depth0;
  stack2 = "service-configs";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  <h4 class=\"step-title\" ");
  stack1 = depth0;
  stack2 = "step-title";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "installer.step7.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1));
  stack1 = depth0;
  stack2 = "controller.finnalyLoaded";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step7.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      ");
  stack1 = depth0;
  stack2 = "isConfigsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default installer-back-btn\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </button>\n    <div class=\"pull-right\">\n      ");
  stack1 = depth0;
  stack2 = "App.supports.preInstallChecks";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <button type=\"button\" class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n      </button>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7/accounts_tab", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <div>\n        ");
  stack1 = depth0;
  stack2 = "checkbox.viewClass";
  stack3 = {};
  stack4 = "checkbox";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = "checkbox.displayName";
  stack3['labelBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </div>\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <tr>\n        <td class=\"account-title\">");
  stack1 = depth0;
  stack2 = "property.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>\n          <div ");
  stack1 = {};
  stack2 = "property.errorMessage:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n            ");
  stack1 = depth0;
  stack2 = "property.viewClass";
  stack3 = {};
  stack4 = "property";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = true;
  stack3['fullWidth'] = stack4;
  stack4 = "false";
  stack3['isPopoverEnabled'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </div>\n        </td>\n      </tr>\n    ");
  return buffer;}

  data.buffer.push("\n\n<div class=\"col-md-7\">\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step7.accountsTab.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n  <div class=\"use-ambari-checkboxes\">\n    ");
  stack1 = depth0;
  stack2 = "view.checkboxes";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "checkbox";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n  <table class=\"table accounts-table\">\n    <thead>\n    <tr>\n      <td class=\"username-col\">");
  stack1 = depth0;
  stack2 = "installer.step7.credentialsTab.usersGroups";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n      <td>");
  stack1 = depth0;
  stack2 = "installer.step7.credentialsTab.usernames";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</td>\n    </tr>\n    </thead>\n    <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.properties";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "property";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7/credentials_tab", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <tr>\n        <td class=\"account-title\">");
  stack1 = depth0;
  stack2 = "row.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</td>\n        <td>\n          ");
  stack1 = depth0;
  stack2 = "row.usernameProperty";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(4, program4, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n        <td>\n          ");
  stack1 = depth0;
  stack2 = "row.passwordProperty";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </td>\n      </tr>\n    ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "row.usernameProperty.errorMessage:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "row.usernameProperty.viewClass";
  stack3 = {};
  stack4 = "row.usernameProperty";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          ");
  return buffer;}

function program4(depth0,data) {
  
  
  data.buffer.push("\n            N/A\n          ");}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <div ");
  stack1 = {};
  stack2 = "row.passwordProperty.errorMessage:has-error";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n              ");
  stack1 = depth0;
  stack2 = "row.passwordProperty.viewClass";
  stack3 = {};
  stack4 = "row.passwordProperty";
  stack3['serviceConfigBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          ");
  return buffer;}

  data.buffer.push("\n\n<p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step7.credentialsTab.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n<table class=\"table credentials-table\">\n  <thead>\n  <tr>\n    <td></td>\n    <td>");
  stack1 = depth0;
  stack2 = "common.username";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<span class=\"red\">*</span></td>\n    <td>\n      <div class=\"row\">\n        <div class=\"col-md-6\">\n          ");
  stack1 = depth0;
  stack2 = "common.password";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<span class=\"red\">*</span>\n        </div>\n        <div class=\"col-md-6\">\n          ");
  stack1 = depth0;
  stack2 = "common.confirm.password";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "<span class=\"red\">*</span>\n        </div>\n      </div>\n    </td>\n  </tr>\n  </thead>\n  <tbody>\n    ");
  stack1 = depth0;
  stack2 = "view.rows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </tbody>\n</table>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7/databases_tab", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "view.configsView";
  stack3 = {};
  stack4 = "view.tabs";
  stack3['tabModelsBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

  data.buffer.push("\n\n<p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step7.databasesTab.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n<div style=\"margin-top: 20px\">\n  ");
  stack1 = depth0;
  stack2 = "view.configsView";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7/directories_tab", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <li rel='tooltip' ");
  stack1 = {};
  stack2 = "tab.isActive:active";
  stack1['class'] = stack2;
  stack2 = "tab.tooltipMsg";
  stack1['data-original-title'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <a href=\"#\" ");
  stack1 = depth0;
  stack2 = "tab";
  stack3 = depth0;
  stack4 = "selectService";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "tab.headingClass";
  stack1['data-target'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n             data-toggle=\"tab\">\n            ");
  stack1 = depth0;
  stack2 = "tab.serviceName";
  foundHelper = helpers.formatRole;
  stack3 = foundHelper || depth0.formatRole;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "formatRole", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n            ");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers['if'];
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </a>\n        </li>\n      ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <span class=\"badge badge-important\">");
  stack1 = depth0;
  stack2 = "tab.errorsCount";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n            ");
  return buffer;}

  data.buffer.push("\n<div class=\"row one-row-tabs-container\">\n  <div ");
  stack1 = {};
  stack2 = ":left-arrow view.isNavArrowsHidden:not-visible";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <a href=\"#\"");
  stack1 = {};
  stack2 = "view.isLeftArrowDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "scrollTabsLeft";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "> <i class=\"icon-angle-left\"></i>\n    </a>\n  </div>\n  <div class=\"tabs-container\">\n    <ul class=\"nav nav-tabs mbm\">\n      ");
  stack1 = depth0;
  stack2 = "view.services";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n  </div>\n  <div ");
  stack1 = {};
  stack2 = ":right-arrow view.isNavArrowsHidden:not-visible";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <a href=\"#\" ");
  stack1 = {};
  stack2 = "view.isRightArrowDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "scrollTabsRight";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <i class=\"icon-angle-right\"></i>\n    </a>\n  </div>\n</div>\n");
  stack1 = depth0;
  stack2 = "view.configsView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7/directories_theme_layout", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.content.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div class=\"accordion\">\n      <div class=\"panel-group\">\n        <div class=\"panel panel-default\">\n          ");
  stack1 = depth0;
  stack2 = "view.content.sectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "row";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </div>\n      </div>\n    </div>\n  ");
  return buffer;}
function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            ");
  stack1 = depth0;
  stack2 = "row";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "section";
  stack7 = helpers.each;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          ");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "App.DirectoriesLayoutCategoryView";
  stack3 = helpers.view;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                <div class=\"panel-heading\" ");
  stack1 = depth0;
  stack2 = "toggleCollapsed";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <h3 class=\"panel-title\">\n                    <i ");
  stack1 = {};
  stack2 = ":pull-right :panel-toggle view.isCollapsed:icon-angle-down:icon-angle-up";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n                    <a class=\"panel-toggle category-header\" ");
  stack1 = depth0;
  stack2 = "category-header";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                      <span class=\"category-name\" ");
  stack1 = depth0;
  stack2 = "category-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "section.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                    </a>\n                  </h3>\n                </div>\n                <div class=\"panel-body collapse in\" ");
  stack1 = depth0;
  stack2 = "panel-body";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                  <div class=\"service-config-section\">\n                    ");
  stack1 = depth0;
  stack2 = "section.subsectionRows";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subRow";
  stack7 = helpers.each;
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                </div>\n              ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "subRow";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "subsection";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                        ");
  stack1 = depth0;
  stack2 = "subsection.configs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "config";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                      ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                          ");
  stack1 = depth0;
  stack2 = "config.widget";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                        ");
  return buffer;}
function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                            ");
  stack1 = depth0;
  stack2 = "config.isVisible";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                          ");
  return buffer;}
function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                              ");
  stack1 = depth0;
  stack2 = "config.isHiddenByFilter";
  stack3 = helpers.unless;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                            ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                                ");
  stack1 = depth0;
  stack2 = "config.widget";
  stack3 = {};
  stack4 = "config";
  stack3['configBinding'] = stack4;
  stack4 = "view.parentView.canEdit";
  stack3['canEditBinding'] = stack4;
  stack4 = "section";
  stack3['sectionBinding'] = stack4;
  stack4 = "subsection";
  stack3['subSectionBinding'] = stack4;
  stack4 = "view.parentView.content";
  stack3['tabBinding'] = stack4;
  stack4 = true;
  stack3['inlineControls'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                              ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.dataIsReady";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step7_with_category_tabs", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <ul class=\"nav nav-tabs config-categories-tabs\">\n      ");
  stack1 = depth0;
  stack2 = "controller.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(2, program2, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </ul>\n    <div class=\"panel panel-default\">\n      <div class=\"panel-body\">\n        ");
  stack1 = depth0;
  stack2 = "isConfigsLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(9, program9, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n  ");
  return buffer;}
function program2(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n        <li ");
  stack1 = {};
  stack2 = "tab.isActive:active tab.isDisabled:disabled";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n          <a href=\"#");
  stack1 = depth0;
  stack2 = "tab.name";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" ");
  stack1 = depth0;
  stack2 = "tab";
  stack3 = depth0;
  stack4 = "selectTab";
  stack5 = {};
  stack6 = "controller";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i ");
  stack1 = {};
  stack2 = ":glyphicon tab.icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i> ");
  stack1 = depth0;
  stack2 = "tab.displayName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </a>\n        </li>\n      ");
  return buffer;}

function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n          <div class=\"tab-content\">\n            ");
  stack1 = depth0;
  stack2 = "controller.tabs";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "tab";
  stack7 = helpers.each;
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        ");
  return buffer;}
function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              ");
  stack1 = depth0;
  stack2 = "tab.isActive";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            ");
  return buffer;}
function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                <div ");
  stack1 = {};
  stack2 = ":tab-pane tab.isActive:active";
  stack1['class'] = stack2;
  stack2 = "tab.name";
  stack1['id'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  ");
  stack1 = depth0;
  stack2 = "tab.tabView";
  stack3 = helpers['if'];
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                </div>\n              ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                    ");
  stack1 = depth0;
  stack2 = "tab.tabView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        <a class=\"btn btn-default\" ");
  stack1 = depth0;
  stack2 = "runPreInstallChecks";
  stack3 = {};
  stack4 = "App.router.preInstallChecksController";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-exclamation\"></i> ");
  stack1 = depth0;
  stack2 = "installer.step7.preInstallChecks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n      ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"serviceConfig\" class=\"wizard-content col-md-9\" ");
  stack1 = depth0;
  stack2 = "service-configs";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n  ");
  stack1 = depth0;
  stack2 = "stepConfigsCreated";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default installer-back-btn pull-left\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </button>\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <div class=\"pull-right\">\n      ");
  stack1 = depth0;
  stack2 = "App.supports.preInstallChecks";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      <button type=\"button\" class=\"btn btn-success\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isNextDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "next";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n      </button>\n    </div>\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step8", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n      <br/><br/>\n      ");
  stack1 = depth0;
  stack2 = "installer.step8.kerberors.warning";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <p>\n              <b>");
  stack1 = depth0;
  stack2 = "item.display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b> : ");
  stack1 = depth0;
  stack2 = "item.config_value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </p>\n          ");
  return buffer;}

function program5(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n            <div>\n              <ul>\n                <li>\n                  <p>");
  stack1 = depth0;
  stack2 = "installer.step1.advancedRepo.useRedhatSatellite.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n                </li>\n              </ul>\n            </div>\n          ");
  return buffer;}

function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n            <div>\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "view.repoInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            </div>\n          ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                  <li>\n                    <p><span class=\"text text-info\">");
  stack1 = depth0;
  stack2 = "item.os_type";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " (");
  stack1 = depth0;
  stack2 = "item.repo_id";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "): <br/></span>");
  stack1 = depth0;
  stack2 = "item.base_url";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n                  </li>\n                ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <p><b>");
  stack1 = depth0;
  stack2 = "menu.item.services";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</b></p>\n              <ul>\n                ");
  stack1 = depth0;
  stack2 = "controller.services";
  stack3 = helpers.each;
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              </ul>\n            ");
  return buffer;}
function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                  <li ");
  stack1 = depth0;
  stack2 = "service-block";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <em><b ");
  stack1 = depth0;
  stack2 = "service-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"service-name\">");
  stack1 = depth0;
  stack2 = "display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b></em>\n                    <ul>\n                      ");
  stack1 = depth0;
  stack2 = "service_components";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "component";
  stack7 = helpers.each;
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                    </ul>\n                  </li>\n                ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n                        <li>\n                          <span class=\"text text-info\" ");
  stack1 = depth0;
  stack2 = "component-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "component.display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " : </span> <span ");
  stack1 = depth0;
  stack2 = "host-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "component.component_value";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                        </li>\n                      ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program16(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

function program18(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n      <button type=\"button\" class=\"btn btn-secondary pull-right\" ");
  stack1 = depth0;
  stack2 = "generateBlueprintConfirmation";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "><i class=\"glyphicon glyphicon-download-alt\"></i>&nbsp;\n        ");
  stack1 = depth0;
  stack2 = "common.generate.blueprint";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      </button>\n    ");
  return buffer;}

  data.buffer.push("\n<div id=\"deploy-review\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step8.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">\n    ");
  stack1 = depth0;
  stack2 = "installer.step8.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n    ");
  stack1 = depth0;
  stack2 = "controller.isManualKerberos";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div id=\"step8-content\" class=\"well pre-scrollable\" ");
  stack1 = depth0;
  stack2 = "review-content";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <div id=\"step8-info\">\n          ");
  stack1 = depth0;
  stack2 = "controller.clusterInfo";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "item";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n          <p><b>");
  stack1 = depth0;
  stack2 = "controller.clusterInfo.repoInfo.display_name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</b>:</p>\n\n          ");
  stack1 = depth0;
  stack2 = "controller.clusterInfo.useRedhatSatellite";
  stack3 = helpers['if'];
  tmp1 = self.program(5, program5, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(7, program7, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n          <div>\n            ");
  stack1 = depth0;
  stack2 = "controller.services.length";
  stack3 = helpers['if'];
  tmp1 = self.program(10, program10, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n\n<div class=\"wizard-footer col-md-12\">\n  <div class=\"btn-area\">\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-back";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-default pull-left installer-back-btn\" ");
  stack1 = {};
  stack2 = "App.router.btnClickInProgress";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "back";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      &larr; ");
  stack1 = depth0;
  stack2 = "common.back";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  stack1 = depth0;
  stack2 = "App.router.backBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(14, program14, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </button>\n    ");
  stack1 = depth0;
  stack2 = "templates/wizard/installer_cancel_button";
  stack3 = helpers.template;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n    <button type=\"button\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"btn btn-success pull-right\"\n       id=\"spinner\" ");
  stack1 = {};
  stack2 = "controller.isSubmitDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(16, program16, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      ");
  stack1 = depth0;
  stack2 = "common.deploy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n    </button>\n    <button type=\"button\" class=\"btn btn-secondary pull-right\" ");
  stack1 = depth0;
  stack2 = "printReview";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.print";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    <button type=\"button\" ");
  stack1 = {};
  stack2 = ":btn :btn-primary :pull-right :mrm controller.showDownloadCsv::hide";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "downloadCSV";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "admin.kerberos.wizard.step5.downloadCSV";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</button>\n    ");
  stack1 = depth0;
  stack2 = "App.router.clusterInstallCompleted";
  stack3 = helpers.unless;
  tmp1 = self.program(18, program18, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n  </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step8/step8_log_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression;


  data.buffer.push("\n<p>");
  stack1 = depth0;
  stack2 = "view.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</p>\n<div ");
  stack1 = {};
  stack2 = "view.progressBarClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    <div class=\"progress-bar progress-bar-striped active\" ");
  stack1 = {};
  stack2 = "view.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n    </div>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step9", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <button class=\"btn btn-default\" href=\"#\" ");
  stack1 = depth0;
  stack2 = "retry";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "retry-button";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-repeat glyphicon-white\"></i>&nbsp;\n                ");
  stack1 = depth0;
  stack2 = "common.retry";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n            ");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6;
  data.buffer.push("\n                <li ");
  stack1 = {};
  stack2 = ":filter-status category.itemClass";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                  <a ");
  stack1 = depth0;
  stack2 = "category";
  stack3 = depth0;
  stack4 = "selectCategory";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " href=\"#\">\n                    ");
  stack1 = depth0;
  stack2 = "category.label";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </li>\n                ");
  stack1 = depth0;
  stack2 = "category.last";
  stack3 = helpers.unless;
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program4(depth0,data) {
  
  
  data.buffer.push("\n                  <li class=\"divider\">|</li>\n                ");}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n                ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n              ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  ");
  stack1 = depth0;
  stack2 = "App.HostStatusView";
  stack3 = {};
  stack4 = "host";
  stack3['objBinding'] = stack4;
  stack4 = "controller";
  stack3['controllerBinding'] = stack4;
  stack4 = "hosts-table-body-row";
  stack3['data-qa'] = stack4;
  stack4 = helpers.view;
  tmp1 = self.program(8, program8, data);
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                ");
  return buffer;}
function program8(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                  <td class=\"host\" ");
  stack1 = depth0;
  stack2 = "hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <span title=\"");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers.unbound;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\" class=\"trim_hostname\" ");
  stack1 = depth0;
  stack2 = "host-name";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.name";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</span>\n                  </td>\n                  <td ");
  stack1 = depth0;
  stack2 = "hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <div class=\"progress-wrapper pull-left\">\n                      <div class=\"progress\">\n                        <div ");
  stack1 = {};
  stack2 = "view.isHostCompleted::active view.isHostCompleted::progress-bar-striped view.barColor :progress-bar";
  stack1['class'] = stack2;
  stack2 = "view.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "progress-bar";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "></div>\n                      </div>\n                    </div>\n                    <div class=\"progress-percentage pull-left\" ");
  stack1 = depth0;
  stack2 = "host-progress-{host.progress}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.progress";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "%</div>\n                  </td>\n                  <td class=\"host-message\" ");
  stack1 = depth0;
  stack2 = "hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                    <a ");
  stack1 = {};
  stack2 = "view.isFailed:text-danger view.isSuccess:text-success view.isWarning:text-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                        href=\"javascript:void(null)\"\n                        data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "hostLogPopup";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "host-status-message";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "host.message";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</a>\n                  </td>\n                  ");
  return buffer;}

function program10(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n              <tr ");
  stack1 = depth0;
  stack2 = "hosts-table-body-row";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                <td colspan=\"3\" ");
  stack1 = depth0;
  stack2 = "hosts-table-body-cell";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "><p>");
  stack1 = depth0;
  stack2 = "installer.step3.hosts.noHosts";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p></td>\n              </tr>\n              ");
  return buffer;}

function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        <p ");
  stack1 = {};
  stack2 = "view.resultMsgColor :alert :alert-warning";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "deploy-result-{status}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "view.resultMsg";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          ");
  stack1 = depth0;
  stack2 = "view.isHostHeartbeatLost";
  stack3 = helpers['if'];
  tmp1 = self.program(13, program13, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        </p>\n      ");
  return buffer;}
function program13(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n            <a href=\"javascript:void(null)\"\n               data-toggle=\"modal\" ");
  stack1 = depth0;
  stack2 = "hostWithInstallFailed";
  stack3 = {};
  stack4 = "view";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.showDetails";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</a>\n          ");
  return buffer;}

function program15(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n          ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = {};
  stack4 = "span";
  stack3['tagName'] = stack4;
  stack4 = "service-button-spinner";
  stack3['classNames'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n        ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"deploy\" class=\"wizard-content col-md-9\">\n  <h4 class=\"step-title\">");
  stack1 = depth0;
  stack2 = "installer.step9.header";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h4>\n  <p class=\"step-description\">");
  stack1 = depth0;
  stack2 = "installer.step9.body";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</p>\n\n  <div class=\"panel panel-default\">\n    <div class=\"panel-body\">\n      <div class=\"overall-progress\" ");
  stack1 = depth0;
  stack2 = "overall-progress";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        <div class=\"progress\" ");
  stack1 = depth0;
  stack2 = "progress-{status}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          <div ");
  stack1 = {};
  stack2 = "view.isStepCompleted::progress-bar-striped view.isStepCompleted::active view.barColor :progress-bar";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "view.barWidth";
  stack1['style'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "progress-bar-{progress}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n          </div>\n        </div>\n        <div ");
  stack1 = depth0;
  stack2 = "progress-message";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " class=\"progress-message\">");
  stack1 = depth0;
  stack2 = "view.progressMessage";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</div>\n      </div>\n\n      <div class=\"panel panel-default panel-internal install-retry-panel\">\n        <div class=\"panel-heading\">\n          <div class=\"pull-right\">\n            ");
  stack1 = depth0;
  stack2 = "controller.showRetry";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </div>\n          <!-- filter by host level -->\n          <div id=\"host-filter\" class=\"pull-left\">\n            <ul class=\"clearfix\">\n              <li class=\"first\">");
  stack1 = depth0;
  stack2 = "common.show";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ":</li>\n              ");
  stack1 = depth0;
  stack2 = "view.categories";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "category";
  stack7 = helpers.each;
  tmp1 = self.program(3, program3, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </ul>\n          </div>\n          <div class=\"clearfix\"></div>\n        </div>\n        <div class=\"panel-body pre-scrollable\">\n          <table id=\"deploy-status-by-host\" class=\"table table-hover\" ");
  stack1 = depth0;
  stack2 = "hosts-table";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n            <thead>\n            <tr>\n              <th class=\"host\" ");
  stack1 = depth0;
  stack2 = "hosts-table-heading";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n                ");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </th>\n              <th class=\"status\" ");
  stack1 = depth0;
  stack2 = "hosts-table-heading";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.status";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n              <!--  given by the parsing function that parses data from bootstrap call -->\n              <th class=\"message\" ");
  stack1 = depth0;
  stack2 = "hosts-table-heading";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">");
  stack1 = depth0;
  stack2 = "common.message";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n              <!-- retrieved from local storage initially -->\n            </tr>\n            </thead>\n            <tbody>\n              ");
  stack1 = depth0;
  stack2 = "view.pageContent";
  stack3 = helpers['if'];
  tmp1 = self.program(6, program6, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(10, program10, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n            </tbody>\n            <tfoot>\n              <tr>\n                <td colspan=\"3\">\n                  ");
  stack1 = depth0;
  stack2 = "App.PaginationView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                </td>\n              </tr>\n            </tfoot>\n          </table>\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.resultMsg";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n\n    </div>\n\n  </div>\n\n</div>\n<div class=\"wizard-footer col-md-12\">\n    <div class=\"btn-area\">\n      <button class=\"btn btn-success pull-right\" ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "isNextButtonDisabled";
  stack1['disabled'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "submit";
  stack3 = {};
  stack4 = "controller";
  stack3['target'] = stack4;
  stack4 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "wizard-next";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + ">\n        ");
  stack1 = depth0;
  stack2 = "App.router.nextBtnClickInProgress";
  stack3 = helpers['if'];
  tmp1 = self.program(15, program15, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "common.next";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + " &rarr;\n      </button>\n    </div>\n</div>\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step9/step9HostTasksLogPopup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, foundHelper, tmp1, self=this, functionType="function", helperMissing=helpers.helperMissing, undef=void 0, escapeExpression=this.escapeExpression;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  <div>\n    ");
  stack1 = depth0;
  stack2 = "installer.step9.host.heartbeat_lost_popup";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  </div>\n");
  return buffer;}

function program3(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n  ");
  stack1 = depth0;
  stack2 = "view.isNoTasksScheduled";
  stack3 = helpers['if'];
  tmp1 = self.program(4, program4, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(6, program6, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;}
function program4(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    ");
  stack1 = depth0;
  stack2 = "installer.step9.host.status.noTasks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n  ");
  return buffer;}

function program6(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7;
  data.buffer.push("\n    <div ");
  stack1 = {};
  stack2 = "view.isLogWrapHidden::hidden :task-list-main-wrap :table-body-wrap";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"row\">\n        <h2 class=\"table-title col-sm-2\">");
  stack1 = depth0;
  stack2 = "common.tasks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</h2>\n        <div class=\"table-controls row col-sm-10 pull-right\">\n          <div class=\"col-sm-12\">\n            <div class=\"btn-group pull-right\">\n              ");
  stack1 = depth0;
  stack2 = "Ember.Select";
  stack3 = {};
  stack4 = "form-control";
  stack3['classNames'] = stack4;
  stack4 = "view.categories";
  stack3['contentBinding'] = stack4;
  stack4 = "content.value";
  stack3['optionValuePath'] = stack4;
  stack4 = "content.label";
  stack3['optionLabelPath'] = stack4;
  stack4 = "view.category";
  stack3['selectionBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n            </div>\n          </div>\n        </div>\n      </div>\n      <div id=\"host-log\">\n        <table class=\"table table-hover\">\n          <tbody>\n            ");
  stack1 = depth0;
  stack2 = "view.tasks";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "taskInfo";
  stack7 = helpers.each;
  tmp1 = self.program(7, program7, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n          </tbody>\n        </table>\n        ");
  stack1 = depth0;
  stack2 = "view.isEmptyList";
  stack3 = helpers['if'];
  tmp1 = self.program(9, program9, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n      </div>\n    </div>\n\n    <div ");
  stack1 = {};
  stack2 = "view.isLogWrapHidden:hidden :task-detail-info";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n      <div class=\"task-top-wrap\">\n        <div class=\"row\">\n          <h2 class=\"table-title col-sm-5\">\n            <i ");
  stack1 = {};
  stack2 = "view.openedTask.status :task-detail-status-ico view.openedTask.icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + "></i>\n            ");
  stack1 = depth0;
  stack2 = "view.openedTask.commandDetail";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n          </h2>\n          <div class=\"table-controls row col-sm-7 pull-right\">\n            <div class=\"col-sm-12\">\n              <button type=\"button\" class=\"btn btn-link pull-right\" ");
  stack1 = {};
  stack2 = "common.openNewWindow";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "openTaskLogInDialog";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"icon icon-external-link\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.open";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n              <button type=\"button\" class=\"btn btn-link pull-right\" ");
  stack1 = {};
  stack2 = "common.fullLogPopup.clickToCopy";
  stack1['title'] = stack2;
  foundHelper = helpers.translateAttr;
  stack2 = foundHelper || depth0.translateAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  if(typeof stack2 === functionType) { stack1 = stack2.call(depth0, tmp1); }
  else if(stack2=== undef) { stack1 = helperMissing.call(depth0, "translateAttr", tmp1); }
  else { stack1 = stack2; }
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "taskInfo";
  stack3 = depth0;
  stack4 = "textTrigger";
  stack5 = {};
  stack6 = "view";
  stack5['target'] = stack6;
  stack6 = helpers.action;
  tmp1 = {};
  tmp1.hash = stack5;
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack6.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-copy\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.copy";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n              <button type=\"button\" class=\"btn btn-link pull-right\" ");
  stack1 = depth0;
  stack2 = "backToTaskList";
  stack3 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <i class=\"glyphicon glyphicon-arrow-left\"></i>&nbsp;");
  stack1 = depth0;
  stack2 = "common.tasks";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "\n              </button>\n            </div>\n          </div>\n        </div>\n      </div>\n      ");
  stack1 = depth0;
  stack2 = "view.isTaskLoaded";
  stack3 = helpers['if'];
  tmp1 = self.program(11, program11, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(14, program14, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </div>\n  ");
  return buffer;}
function program7(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4, stack5;
  data.buffer.push("\n              <tr ");
  stack1 = depth0;
  stack2 = "taskInfo";
  stack3 = depth0;
  stack4 = "toggleTaskLog";
  stack5 = helpers.action;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack5.call(depth0, stack4, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = {};
  stack2 = "taskInfo.isVisible::hidden :pointer";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + ">\n                <td class=\"col-sm-3\">\n                  <i ");
  stack1 = {};
  stack2 = "taskInfo.status taskInfo.icon";
  stack1['class'] = stack2;
  stack2 = helpers.bindAttr;
  tmp1 = {};
  tmp1.hash = stack1;
  tmp1.contexts = [];
  tmp1.data = data;
  stack1 = stack2.call(depth0, tmp1);
  data.buffer.push(escapeExpression(stack1) + " ");
  stack1 = depth0;
  stack2 = "task-status-{taskInfo.status}";
  foundHelper = helpers.QAAttr;
  stack3 = foundHelper || depth0.QAAttr;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "QAAttr", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "></i>\n                  <a href=\"#\">\n                    ");
  stack1 = depth0;
  stack2 = "taskInfo.commandDetail";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                  </a>\n                </td>\n                <td class=\"col-sm-9 text-right\">\n                  <a class=\"action\" href=\"#\">\n                    <span class=\"icon icon-caret-right\"></span>\n                  </a>\n                </td>\n              </tr>\n            ");
  return buffer;}

function program9(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"log-list-wrap\">");
  stack1 = depth0;
  stack2 = "installer.step9.hostLog.popup.noTasksToShow";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</div>\n        ");
  return buffer;}

function program11(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n          <div class=\"task-detail-log-info\">\n              <div class=\"content-area\">\n                  <div class=\"task-detail-log-clipboard-wrap\">\n                    ");
  stack1 = depth0;
  stack2 = "view.showClipBoard";
  stack3 = helpers['if'];
  tmp1 = self.program(12, program12, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n                  </div>\n                  <div class=\"task-detail-log-maintext\">\n                      <h5>stderr: &nbsp; <span class=\"muted\">");
  stack1 = depth0;
  stack2 = "view.openedTask.errorLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></h5>\n                      <pre class=\"stderr\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stderr";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n                      <h5>stdout: &nbsp; <span class=\"muted\"> ");
  stack1 = depth0;
  stack2 = "view.openedTask.outputLog";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + " </span></h5>\n                      <pre class=\"stdout\">");
  stack1 = depth0;
  stack2 = "view.openedTask.stdout";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "</pre>\n                  </div>\n              </div>\n          </div>\n      ");
  return buffer;}
function program12(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3, stack4;
  data.buffer.push("\n                      ");
  stack1 = depth0;
  stack2 = "App.NotScrollableTextArea";
  stack3 = {};
  stack4 = "task-detail-log-clipboard";
  stack3['class'] = stack4;
  stack4 = "view.formattedLogsForOpenedTask";
  stack3['valueBinding'] = stack4;
  stack4 = helpers.view;
  tmp1 = {};
  tmp1.hash = stack3;
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack4.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n                    ");
  return buffer;}

function program14(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n        ");
  stack1 = depth0;
  stack2 = "App.SpinnerView";
  stack3 = helpers.view;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      ");
  return buffer;}

  data.buffer.push("\n\n");
  stack1 = depth0;
  stack2 = "view.isHeartbeatLost";
  stack3 = helpers['if'];
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.program(3, program3, data);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n");
  return buffer;
});
module.exports = module.id;
});

require.register("templates/wizard/step9/step9_install_host_popup", function(exports, require, module) {
Ember.TEMPLATES[module.id] = Ember.Handlebars.template(function anonymous(Handlebars,depth0,helpers,partials,data
/**/) {
helpers = helpers || Ember.Handlebars.helpers;
  var buffer = '', stack1, stack2, stack3, stack4, stack5, stack6, stack7, foundHelper, tmp1, self=this, escapeExpression=this.escapeExpression, functionType="function", helperMissing=helpers.helperMissing, undef=void 0;

function program1(depth0,data) {
  
  var buffer = '', stack1, stack2, stack3;
  data.buffer.push("\n    <tr>\n      <td>\n        ");
  stack1 = depth0;
  stack2 = "host.hostName";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </td>\n      <td>\n        ");
  stack1 = depth0;
  stack2 = "host.componentNames";
  stack3 = helpers._triageMustache;
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  stack1 = stack3.call(depth0, stack2, tmp1);
  data.buffer.push(escapeExpression(stack1) + "\n      </td>\n    </tr>\n  ");
  return buffer;}

  data.buffer.push("\n\n<div id=\"install-fail-popup\" class=\"pre-scrollable\">\n  <table class=\"table table-hover\">\n    <thead>\n    <tr>\n      <th class=\"col-md-3\">");
  stack1 = depth0;
  stack2 = "common.host";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n      <th class=\"col-md-3\">");
  stack1 = depth0;
  stack2 = "common.components";
  foundHelper = helpers['t'];
  stack3 = foundHelper || depth0['t'];
  tmp1 = {};
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack1);
  tmp1.data = data;
  if(typeof stack3 === functionType) { stack1 = stack3.call(depth0, stack2, tmp1); }
  else if(stack3=== undef) { stack1 = helperMissing.call(depth0, "t", stack2, tmp1); }
  else { stack1 = stack3; }
  data.buffer.push(escapeExpression(stack1) + "</th>\n    </tr>\n    </thead>\n    <tbody>\n  ");
  stack1 = depth0;
  stack2 = "view.failedHosts";
  stack3 = depth0;
  stack4 = "in";
  stack5 = depth0;
  stack6 = "host";
  stack7 = helpers.each;
  tmp1 = self.program(1, program1, data);
  tmp1.hash = {};
  tmp1.contexts = [];
  tmp1.contexts.push(stack5);
  tmp1.contexts.push(stack3);
  tmp1.contexts.push(stack1);
  tmp1.fn = tmp1;
  tmp1.inverse = self.noop;
  tmp1.data = data;
  stack1 = stack7.call(depth0, stack6, stack4, stack2, tmp1);
  if(stack1 || stack1 === 0) { data.buffer.push(stack1); }
  data.buffer.push("\n    </tbody>\n  </table>\n</div>");
  return buffer;
});
module.exports = module.id;
});

require.register("utils", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load needed utils here

require('utils/errors/assertions');
require('utils/base64');
require('utils/db');
require('utils/helper');
require('utils/config');
require('utils/configs/theme/theme');
require('utils/configs/config_initializer');
require('utils/configs/nn_ha_config_initializer');
require('utils/configs/rm_ha_config_initializer');
require('utils/configs/move_namenode_config_initializer');
require('utils/configs/move_rm_config_initializer');
require('utils/configs/move_os_config_initializer');
require('utils/configs/move_hm_config_initializer');
require('utils/configs/move_hs_config_initializer');
require('utils/configs/move_ws_config_initializer');

});

require.register("utils/action_sequence", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Utility to define sequence of actions (sync or async either) and execute them in exact order
 * methods executed in order that described in sequence
 * each action obtain result of previous action <code>previousResponse</code>
 * For example:
 * loadMap: {
 *  '0': {
 *   type: 'async',
 *   callback: function (previousResponse) {
 *     //method1
 *   }
 *  },
 *  '1': {
 *    type: 'sync',
 *    callback: function (previousResponse) {
 *      //method2
 *    }
 *  }
 * }
 * However method1 is async, method2 will be executed only after method1 is completed
 * @type {*}
 */
App.actionSequence = Em.Object.extend({
  /**
   * List of actions which will be executed
   * format:
   * [
   *   {
   *     type: 'async',
   *     callback: function(previousResponse) {
   *       //async method should return Deferred object to continue sequence
   *     }
   *   },
   *   {
   *     type: 'sync',
   *     callback: function(previousResponse) {
   *       //code
   *     }
   *   }
   * ]
   * @type {Object[]}
   */
  sequence: [],
  /**
   * specific context for methods executed in actions
   */
  context: undefined,
  /**
   * Function called after sequence is completed
   * @type {callback}
   */
  finishedCallback: Em.K,
  /**
   * counter of actions
   */
  actionCounter: 0,
  /**
   * set sequence
   * @param sequence
   * @return {object}
   */
  setSequence: function setSequence(sequence) {
    if (Array.isArray(sequence)) {
      this.set('sequence', sequence);
    }
    return this;
  },
  /**
   * start executing sequence of actions
   */
  start: function start() {
    this.set('actionCounter', this.get('sequence.length'));
    this.runNextAction(0, null);
  },
  /**
   * set <code>finishedCallback</code>
   * @param callback
   * @return {object}
   */
  onFinish: function onFinish(callback) {
    if (typeof callback === 'function') {
      this.set('finishedCallback', callback);
    }
    return this;
  },
  /**
   * run next action in sequence
   * @param index
   * @param previousResponse
   * @return {Boolean}
   */
  runNextAction: function runNextAction(index, previousResponse) {
    var action = this.get('sequence')[index];
    var context = this.get('context');
    var self = this;

    index++;
    this.decrementProperty('actionCounter');

    if (this.get('actionCounter') >= 0 && !Em.isNone(action)) {
      if (action.type === 'sync') {
        this.runNextAction(index, action.callback.call(context, previousResponse));
        return true;
      } else if (action.type === 'async') {
        action.callback.call(context, previousResponse).done(function (response) {
          self.runNextAction(index, response);
        });
        return true;
      }
    } else {
      //if no more actions left then finish sequence
      this.finishedCallback();
    }
    return false;
  }
});

});

require.register("utils/ajax/ajax", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

/**
 * Config for each ajax-request
 *
 * Fields example:
 *  mock - testMode url
 *  real - real url (without API prefix)
 *  type - request type (also may be defined in the format method)
 *  format - function for processing ajax params after default formatRequest. May be called with one or two parameters (data, opt). Return ajax-params object
 *  testInProduction - can this request be executed on production tests (used only in tests)
 *
 * @type {Object}
 */
var urls = {

  'common.cluster.update': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },

  'common.services.update': {
    'real': '/clusters/{clusterName}/services?{urlParams}',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context,
            "operation_level": {
              "level": "CLUSTER",
              "cluster_name": data.clusterName
            }
          },
          Body: {
            ServiceInfo: data.ServiceInfo
          }
        })
      };
    }
  },

  'common.service.update': {
    'real': '/clusters/{clusterName}/services/{serviceName}',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context,
            "operation_level": {
              "level": "SERVICE",
              "cluster_name": data.clusterName,
              "service_name": data.serviceName
            }
          },
          Body: {
            ServiceInfo: data.ServiceInfo
          }
        })
      };
    }
  },

  'common.service_component.info': {
    'real': '/clusters/{clusterName}/services/{serviceName}/components/{componentName}?{urlParams}',
    'mock': '/data/wizard/deploy/poll_1.json'
  },

  'common.service.hdfs.getNnCheckPointTime': {
    'real': '/clusters/{clusterName}/services/HDFS/components/NAMENODE?fields=host_components/metrics/dfs/FSNamesystem/HAState,host_components/metrics/dfs/FSNamesystem/LastCheckpointTime,host_components/metrics/dfs/namenode/ClusterId',
    'mock': ''
  },

  'common.host_component.getNnCheckPointTime': {
    'real': '/clusters/{clusterName}/hosts/{host}/host_components/NAMENODE?fields=metrics/dfs/FSNamesystem/HAState,metrics/dfs/FSNamesystem/LastCheckpointTime',
    'mock': ''
  },

  'common.host_component.update': {
    'real': '/clusters/{clusterName}/host_components',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            context: data.context,
            query: data.query
          },
          Body: {
            "HostRoles": data.HostRoles
          }
        })
      };
    }
  },

  'common.host.host_components.create': {
    'real': '/clusters/{clusterName}/hosts',
    'mock': '',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            "query": data.query
          },
          Body: {
            "host_components": data.host_components
          }
        })
      };
    }
  },

  'common.host.host_components.update': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components?{urlParams}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context,
            "operation_level": {
              level: "HOST",
              cluster_name: data.clusterName,
              host_names: data.hostName
            },
            query: data.query
          },
          Body: {
            "HostRoles": data.HostRoles
          }
        })
      };
    }
  },

  'common.host.host_component.update': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/{componentName}?{urlParams}',
    'mock': '/data/wizard/deploy/2_hosts/poll_9.json',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context,
            "operation_level": {
              level: "HOST_COMPONENT",
              cluster_name: data.clusterName,
              host_name: data.hostName,
              service_name: data.serviceName || null
            }
          },
          Body: {
            "HostRoles": data.HostRoles
          }
        })
      };
    }
  },

  'common.hosts.all': {
    'real': '/clusters/{clusterName}/host_components?{urlParams}&minimal_response=true',
    'mock': ''
  },

  'common.service.configurations': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          Clusters: {
            desired_config: data.desired_config
          }
        })
      };
    }
  },

  'common.service.multiConfigurations': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.configs)
      };
    }
  },

  'common.across.services.configurations': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}',
    'mock': '/data/services/ambari.json',
    'format': function format(data) {
      return {
        dataType: 'text',
        data: data.data
      };
    }
  },

  'common.request.polling': {
    'real': '/clusters/{clusterName}/requests/{requestId}?fields=tasks/Tasks/request_id,tasks/Tasks/command,tasks/Tasks/command_detail,tasks/Tasks/ops_display_name,tasks/Tasks/start_time,tasks/Tasks/end_time,tasks/Tasks/exit_code,tasks/Tasks/host_name,tasks/Tasks/id,tasks/Tasks/role,tasks/Tasks/status,tasks/Tasks/structured_out,Requests/*&tasks/Tasks/stage_id={stageId}',
    'mock': '/data/background_operations/host_upgrade_tasks.json'
  },

  'service.ambari': {
    'real': '/services/AMBARI?fields=components/RootServiceComponents',
    'mock': '/data/services/ambari.json'
  },

  'ambari.service.load_server_version': {
    'real': '/services/AMBARI?fields=components/RootServiceComponents/component_version&components/RootServiceComponents/component_name=AMBARI_SERVER&minimal_response=true',
    'mock': '/data/services/ambari.json'
  },

  'service.flume.agent.command': {
    'real': '/clusters/{clusterName}/hosts/{host}/host_components/FLUME_HANDLER',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          "RequestInfo": {
            "context": data.context,
            "flume_handler": data.agentName,
            "operation_level": {
              level: "HOST_COMPONENT",
              cluster_name: data.clusterName,
              service_name: "FLUME",
              host_name: data.host
            }
          },
          "Body": {
            "HostRoles": {
              "state": data.state
            }
          }
        })
      };
    }
  },

  'common.host_components.update': {
    'real': '/clusters/{clusterName}/host_components?{urlParams}',
    'mock': '/data/wizard/deploy/poll_1.json',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context,
            "operation_level": {
              level: data.level || "CLUSTER",
              cluster_name: data.clusterName
            },
            query: data.query
          },
          Body: {
            "HostRoles": data.HostRoles
          }
        })
      };
    }
  },

  'common.hosts.delete': {
    'real': '/clusters/{clusterName}/hosts{urlParams}',
    'type': 'DELETE',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            query: data.query
          }
        })
      };
    }
  },

  'common.service.passive': {
    'real': '/clusters/{clusterName}/services/{serviceName}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            "context": data.requestInfo
          },
          Body: {
            ServiceInfo: {
              maintenance_state: data.passive_state
            }
          }
        })
      };
    }
  },

  'common.service.host_component.update': {
    'real': '/clusters/{clusterName}/host_components',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            context: data.context,
            operation_level: {
              level: 'SERVICE',
              cluster_name: data.clusterName,
              service_name: data.serviceName
            },
            query: data.query
          },
          Body: {
            HostRoles: {
              state: data.state
            }
          }
        })
      };
    }
  },

  'common.host.host_component.passive': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/{componentName}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            "context": data.context
          },
          Body: {
            HostRoles: {
              maintenance_state: data.passive_state
            }
          }
        })
      };
    }
  },

  'common.host.with_host_component': {
    'real': '/clusters/{clusterName}/hosts?host_components/HostRoles/component_name={componentName}&minimal_response=true',
    'mock': ''
  },

  'common.batch.request_schedules': {
    'real': '/clusters/{clusterName}/request_schedules',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify([{
          "RequestSchedule": {
            "batch": [{
              "requests": data.batches
            }, {
              "batch_settings": {
                "batch_separation_in_seconds": data.intervalTimeSeconds,
                "task_failure_tolerance": data.tolerateSize
              }
            }]
          }
        }])
      };
    }
  },

  'common.delete.host': {
    'real': '/clusters/{clusterName}/hosts/{hostName}',
    'type': 'DELETE'
  },
  'common.delete.host_component': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/{componentName}',
    'type': 'DELETE'
  },
  'common.delete.user': {
    'real': '/users/{user}',
    'type': 'DELETE'
  },
  'common.delete.config_group': {
    'real': '/clusters/{clusterName}/config_groups/{id}',
    'type': 'DELETE'
  },
  'common.delete.cluster': {
    'real': '/clusters/{name}',
    'type': 'DELETE'
  },
  'common.delete.service': {
    'real': '/clusters/{clusterName}/services/{serviceName}',
    'mock': '/data/services/ambari.json',
    'type': 'DELETE'
  },
  'common.delete.request_schedule': {
    'real': '/clusters/{clusterName}/request_schedules/{request_schedule_id}',
    'type': 'DELETE'
  },
  'common.get.request.status': {
    'real': '/clusters/{clusterName}/requests/{requestId}?fields=Requests/request_status',
    'type': 'GET'
  },
  'alerts.load_alert_groups': {
    'real': '/clusters/{clusterName}/alert_groups?fields=*',
    'mock': 'data/alerts/alertGroups.json'
  },
  'alerts.load_an_alert_group': {
    'real': '/clusters/{clusterName}/alert_groups/{group_id}',
    'mock': 'data/alerts/alertGroup.json'
  },
  'alert_groups.create': {
    'real': '/clusters/{clusterName}/alert_groups',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "AlertGroup": {
            "name": data.name,
            "definitions": data.definitions,
            "targets": data.targets
          }
        })
      };
    }
  },
  'alert_groups.update': {
    'real': '/clusters/{clusterName}/alert_groups/{group_id}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          "AlertGroup": {
            "name": data.name,
            "definitions": data.definitions,
            "targets": data.targets
          }
        })
      };
    }
  },
  'alert_groups.delete': {
    'real': '/clusters/{clusterName}/alert_groups/{group_id}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'DELETE'
      };
    }
  },
  'alerts.load_all_alert_definitions': {
    'real': '/clusters/{clusterName}/alert_definitions?fields=*',
    'mock': 'data/alerts/alertDefinitions.json'
  },
  'alerts.notifications': {
    'real': '/alert_targets?fields=*',
    'mock': '/data/alerts/alertNotifications.json'
  },
  'alerts.instances': {
    'real': '/clusters/{clusterName}/alerts?fields=*',
    'mock': '/data/alerts/alert_instances.json'
  },
  'alerts.instances.unhealthy': {
    'real': '/clusters/{clusterName}/alerts?fields=*&Alert/state.in(CRITICAL,WARNING)&{paginationInfo}',
    'mock': '/data/alerts/alert_instances.json'
  },
  'alerts.instances.by_definition': {
    'real': '/clusters/{clusterName}/alerts?fields=*&Alert/definition_id={definitionId}',
    'mock': '/data/alerts/alert_instances.json'
  },
  'alerts.instances.by_host': {
    'real': '/clusters/{clusterName}/alerts?fields=*&Alert/host_name={hostName}',
    'mock': '/data/alerts/alert_instances.json'
  },
  'alerts.update_alert_definition': {
    'real': '/clusters/{clusterName}/alert_definitions/{id}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },
  'alerts.create_alert_definition': {
    'real': '/clusters/{clusterName}/alert_definitions/',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },
  'alerts.delete_alert_definition': {
    'real': '/clusters/{clusterName}/alert_definitions/{id}',
    'mock': '',
    'type': 'DELETE'
  },
  'alerts.create_alert_notification': {
    'real': '/alert_targets?{urlParams}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },
  'alerts.update_alert_notification': {
    'real': '/alert_targets/{id}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },
  'alerts.delete_alert_notification': {
    'real': '/alert_targets/{id}',
    'mock': '',
    'type': 'DELETE'
  },
  'alerts.get_instances_history': {
    'real': '/clusters/{clusterName}/alert_history?(AlertHistory/definition_name={definitionName})&(AlertHistory/timestamp>={timestamp})',
    'mock': '/data/alerts/alert_instances_history.json'
  },
  'background_operations.get_most_recent': {
    'real': '/clusters/{clusterName}/requests?to=end&page_size={operationsCount}&fields=' + 'Requests/end_time,Requests/id,Requests/progress_percent,Requests/request_context,' + 'Requests/request_status,Requests/start_time,Requests/cluster_name,Requests/user_name&minimal_response=true',
    'mock': '/data/background_operations/list_on_start.json',
    'testInProduction': true
  },
  'background_operations.get_by_request': {
    'real': '/clusters/{clusterName}/requests/{requestId}?fields=*,tasks/Tasks/request_id,tasks/Tasks/command,tasks/Tasks/command_detail,tasks/Tasks/ops_display_name,tasks/Tasks/host_name,tasks/Tasks/id,tasks/Tasks/role,tasks/Tasks/status&minimal_response=true',
    'mock': '/data/background_operations/task_by_request{requestId}.json',
    'testInProduction': true
  },
  'background_operations.get_by_task': {
    'real': '/clusters/{clusterName}/requests/{requestId}/tasks/{taskId}',
    'mock': '/data/background_operations/list_on_start.json',
    'testInProduction': true
  },
  'background_operations.abort_request': {
    'real': '/clusters/{clusterName}/requests/{requestId}',
    'mock': '',
    'format': function format() {
      return {
        type: 'PUT',
        data: JSON.stringify({
          "Requests": {
            "request_status": "ABORTED",
            "abort_reason": Em.I18n.t('hostPopup.bgop.abortRequest.reason')
          }
        })
      };
    }
  },
  'service.item.smoke': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      var requestData = {
        "RequestInfo": {
          "context": data.displayName + " Service Check",
          "command": data.actionName
        },
        "Requests/resource_filters": [{ "service_name": data.serviceName }]
      };
      if (data.operationLevel) {
        requestData.RequestInfo.operation_level = data.operationLevel;
      }
      return {
        'type': 'POST',
        data: JSON.stringify(requestData)
      };
    }
  },
  'service.item.rebalanceHdfsNodes': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.context'),
            'command': 'REBALANCEHDFS',
            'namenode': JSON.stringify({ threshold: data.threshold })
          },
          "Requests/resource_filters": [{
            'service_name': 'HDFS',
            'component_name': 'NAMENODE',
            'hosts': data.hosts
          }]
        })
      };
    }
  },

  'cancel.background.operation': {
    'real': '/clusters/{clusterName}/requests/{requestId}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            'context': 'Cancel operation',
            "parameters": {
              "cancel_policy": "SIGKILL"
            }
          },
          "Requests/request_status": 'ABORTED',
          "Requests/abort_reason": "Cancel background operation"
        })
      };
    }
  },

  'service.item.refreshQueueYarnRequest': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': data.context,
            'command': data.command,
            'parameters/forceRefreshConfigTags': data.forceRefreshConfigTags
          },
          "Requests/resource_filters": [{
            "service_name": data.serviceName,
            "component_name": data.componentName,
            'hosts': data.hosts
          }]
        })
      };
    }
  },

  'service.item.startStopLdapKnox': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': data.context,
            'command': data.command
          },
          "Requests/resource_filters": [{
            "service_name": data.serviceName,
            "component_name": data.componentName,
            'hosts': data.host
          }]
        })
      };
    }
  },

  'service.item.updateHBaseReplication': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': Em.I18n.t('services.service.actions.run.updateHBaseReplication.context'),
            'command': 'UPDATE_REPLICATION',
            "parameters": {
              "replication_cluster_keys": data.replication_cluster_keys,
              "replication_peers": data.replication_peers
            }
          },
          "Requests/resource_filters": [{
            'service_name': 'HBASE',
            'component_name': 'HBASE_MASTER',
            'hosts': data.hosts
          }]
        })
      };
    }
  },

  'service.item.stopHBaseReplication': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': Em.I18n.t('services.service.actions.run.stopHBaseReplication.context'),
            'command': 'STOP_REPLICATION',
            "parameters": {
              "replication_peers": data.replication_peers
            }
          },
          "Requests/resource_filters": [{
            'service_name': 'HBASE',
            'component_name': 'HBASE_MASTER',
            'hosts': data.hosts
          }]
        })
      };
    }
  },

  'service.item.executeCustomCommand': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': data.context,
            'command': data.command
          },
          "Requests/resource_filters": [{
            "service_name": data.serviceName,
            "component_name": data.componentName,
            'hosts': data.hosts
          }]
        })
      };
    }
  },
  /*************************CONFIG THEME****************************************/

  'configs.theme': {
    'real': '{stackVersionUrl}/services/{serviceName}/themes?ThemeInfo/default=true&fields=*',
    'mock': '/data/configurations/theme.json'
  },

  'configs.theme.services': {
    'real': '{stackVersionUrl}/services?StackServices/service_name.in({serviceNames})&themes/ThemeInfo/default=true&fields=themes/*',
    'mock': '/data/configurations/theme_services.json'
  },

  /*************************CONFIG QUICKLINKS****************************************/

  'configs.quicklinksconfig': {
    'real': '{stackVersionUrl}/services/{serviceName}/quicklinks?QuickLinkInfo/default=true&fields=*',
    'mock': '/data/configurations/quicklinks.json'
  },

  'configs.quicklinksconfig.services': {
    'real': '{stackVersionUrl}/services?StackServices/service_name.in({serviceNames})&quicklinks/QuickLinkInfo/default=true&fields=quicklinks/*',
    'mock': '/data/configurations/quicklinks_services.json'
  },

  /*************************CONFIG GROUPS***************************************/

  'configs.config_groups.load.all': {
    'real': '/clusters/{clusterName}/config_groups?fields=*',
    'mock': '/data/configurations/config_groups.json'
  },

  'configs.config_groups.load.services': {
    'real': '/clusters/{clusterName}/config_groups?ConfigGroup/tag.in({serviceNames})&fields=*',
    'mock': '/data/configurations/config_groups.json'
  },

  /*************************STACK CONFIGS**************************************/

  'configs.stack_configs.load.cluster_configs': {
    'real': '{stackVersionUrl}?fields=configurations/*,Versions/config_types/*',
    'mock': '/data/stacks/HDP-2.2/configurations.json'
  },

  'configs.stack_configs.load.all': {
    'real': '{stackVersionUrl}/services?fields=configurations/*,StackServices/config_types/*',
    'mock': '/data/stacks/HDP-2.2/configurations.json'
  },

  'configs.stack_configs.load.services': {
    'real': '{stackVersionUrl}/services?StackServices/service_name.in({serviceList})&fields=configurations/*,configurations/dependencies/*,StackServices/config_types/*',
    'mock': '/data/stacks/HDP-2.2/configurations.json'
  },

  'configs.stack_configs.load.service': {
    'real': '{stackVersionUrl}/services/{serviceName}?fields=configurations/*,StackServices/config_types/*',
    'mock': '/data/stacks/HDP-2.2/configurations.json'
  },

  /*************************CONFIG VERSIONS*************************************/

  'configs.config_versions.load': {
    'real': '/clusters/{clusterName}/configurations/service_config_versions?service_name={serviceName}&service_config_version={configVersion}&fields=*',
    'mock': '/data/configurations/config_versions.json'
  },

  'configs.config_versions.load.group': {
    'real': '/clusters/{clusterName}/configurations/service_config_versions?service_name={serviceName}&group_id={id}&fields=*',
    'mock': '/data/configurations/config_versions.json'
  },

  'configs.config_versions.load.current_versions': {
    'real': '/clusters/{clusterName}/configurations/service_config_versions?service_name.in({serviceNames})&is_current=true&fields=*',
    'mock': '/data/configurations/config_versions.json'
  },

  'service.load_config_groups': {
    'real': '/clusters/{clusterName}/config_groups?ConfigGroup/tag={serviceName}&fields=*',
    'mock': '/data/configurations/config_group.json'
  },
  'reassign.load_configs': {
    'real': '/clusters/{clusterName}/configurations?{urlParams}',
    'mock': ''
  },

  'reassign.save_configs': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Clusters: {
            desired_config: {
              "type": data.siteName,
              "properties": data.properties,
              "service_config_version_note": data.service_config_version_note

            }
          }
        })
      };
    }
  },
  'config.cluster': {
    'real': '{stackVersionUrl}/configurations?fields=*',
    'mock': ''
  },
  'config.advanced': {
    'real': '{stackVersionUrl}/services/{serviceName}/configurations?fields=*',
    'mock': '/data/wizard/stack/hdp/version{stackVersion}/{serviceName}.json'
  },
  'config.advanced.multiple.services': {
    'real': '{stackVersionUrl}/services?StackServices/service_name.in({serviceNames})&fields=configurations/*',
    'mock': '/data/wizard/stack/hdp/version{stackVersion}/{serviceName}.json'
  },
  'config.advanced.partial': {
    'real': '{stackVersionUrl}/services/?StackServices/service_name.in({serviceList})&fields=configurations/*{queryFilter}',
    'mock': ''
  },
  'config.config_types': {
    'real': '{stackVersionUrl}/services/{serviceName}?fields=StackServices/config_types',
    'mock': ''
  },
  'config.tags': {
    'real': '/clusters/{clusterName}?fields=Clusters/desired_configs',
    'mock': '/data/clusters/cluster.json'
  },
  'config.tags.site': {
    'real': '/clusters/{clusterName}?fields=Clusters/desired_configs/{site}',
    'mock': ''
  },
  'config.tags_and_groups': {
    'real': '/clusters/{clusterName}?fields=Clusters/desired_configs,config_groups/*{urlParams}',
    'mock': '/data/clusters/tags_and_groups.json'
  },
  'config_groups.all_fields': {
    'real': '/clusters/{clusterName}/config_groups?fields=*',
    'mock': ''
  },
  'config_groups.get_config_group_by_id': {
    'real': '/clusters/{clusterName}/config_groups/{id}',
    'mock': ''
  },
  'config_groups.update_config_group': {
    'real': '/clusters/{clusterName}/config_groups/{id}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify([data.configGroup])
      };
    }
  },
  'config.on_site': {
    'real': '/clusters/{clusterName}/configurations?{params}',
    'mock': '/data/configurations/cluster_level_configs.json?{params}'
  },
  'config.host_overrides': {
    'real': '/clusters/{clusterName}/configurations?{params}',
    'mock': '/data/configurations/host_level_overrides_configs.json?{params}'
  },

  'config.tags.selected': {
    'real': '/clusters/{clusterName}/configurations?type.in({tags})',
    'mock': '/data/configuration/cluster_env_site.json'
  },

  'credentials.store.info': {
    'real': '/clusters/{clusterName}?fields=Clusters/credential_store_properties',
    'mock': ''
  },

  'credentials.list': {
    'real': '/clusters/{clusterName}/credentials?fields=Credential/*',
    'mock': ''
  },

  'credentials.get': {
    'real': '/clusters/{clusterName}/credentials/{alias}',
    'mock': ''
  },

  'credentials.create': {
    'real': '/clusters/{clusterName}/credentials/{alias}',
    'mock': '',
    type: 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Credential: data.resource
        })
      };
    }
  },

  'credentials.update': {
    'real': '/clusters/{clusterName}/credentials/{alias}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Credential: data.resource
        })
      };
    }
  },

  'credentials.delete': {
    'real': '/clusters/{clusterName}/credentials/{alias}',
    'mock': '',
    'type': 'DELETE'
  },

  'host.host_component.add_new_component': {
    'real': '/clusters/{clusterName}/hosts?Hosts/host_name={hostName}',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      return {
        type: 'POST',
        data: data.data
      };
    }
  },

  'host.host_component.add_new_components': {
    'real': '/clusters/{clusterName}/hosts',
    'mock': '/data/wizard/deploy/poll_1.json',
    'format': function format(data) {
      return {
        type: 'POST',
        data: data.data
      };
    }
  },

  'host.host_component.delete_components': {
    'real': '/clusters/{clusterName}/host_components',
    'format': function format(data) {
      return {
        type: 'DELETE',
        data: data.data
      };
    }
  },

  'host.host_component.slave_desired_admin_state': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/{componentName}/?fields=HostRoles/desired_admin_state',
    'mock': '/data/hosts/HDP2/decommission_state.json'
  },
  'host.host_component.decommission_status': {
    'real': '/clusters/{clusterName}/services/{serviceName}/components/{componentName}/?fields=ServiceComponentInfo,host_components/HostRoles/state',
    'mock': ''
  },
  'host_components.hbase_regionserver.active': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name=HBASE_REGIONSERVER&HostRoles/maintenance_state=OFF&HostRoles/desired_admin_state=INSERVICE&HostRoles/host_name.in({hostNames})',
    'mock': ''
  },
  'host.host_component.decommission_status_datanode': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name=NAMENODE&HostRoles/host_name.in({hostNames})&fields=metrics/dfs/namenode',
    'mock': '/data/hosts/HDP2/decommission_state.json'
  },
  'host.host_component.decommission_status_regionserver': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name=HBASE_MASTER&HostRoles/host_name={hostName}&fields=metrics/hbase/master/liveRegionServersHosts,metrics/hbase/master/deadRegionServersHosts&minimal_response=true'
  },
  'host.region_servers.in_inservice': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name=HBASE_REGIONSERVER&HostRoles/desired_admin_state=INSERVICE&fields=HostRoles/host_name&minimal_response=true',
    'mock': ''
  },
  'host.host_component.decommission_slave': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          RequestInfo: {
            'context': data.context,
            'command': data.command,
            'parameters': {
              'slave_type': data.slaveType,
              'excluded_hosts': data.hostName
            },
            'operation_level': {
              level: "HOST_COMPONENT",
              cluster_name: data.clusterName,
              host_name: data.hostName,
              service_name: data.serviceName
            }
          },
          "Requests/resource_filters": [{ "service_name": data.serviceName, "component_name": data.componentName }]
        })
      };
    }
  },

  'host.host_component.refresh_configs': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "CONFIGURE",
            "context": data.context
          },
          "Requests/resource_filters": data.resource_filters
        })
      };
    }
  },

  'hosts.metrics': {
    'real': '/clusters/{clusterName}/hosts?fields={metricName}',
    'mock': '/data/cluster_metrics/cpu_1hr.json'
  },
  'hosts.metrics.host_component': {
    'real': '/clusters/{clusterName}/services/{serviceName}/components/{componentName}?fields=host_components/{metricName}',
    'mock': '/data/cluster_metrics/cpu_1hr.json'
  },
  'service.metrics.flume.channel_fill_percent': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/flume/flume/CHANNEL/ChannelFillPercentage[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/channelFillPct.json',
    'testInProduction': true
  },
  'service.metrics.flume.channel_size': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/flume/flume/CHANNEL/ChannelSize[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/channelSize.json',
    'testInProduction': true
  },
  'service.metrics.flume.sink_drain_success': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/flume/flume/SINK/EventDrainSuccessCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/sinkDrainSuccessCount.json',
    'testInProduction': true
  },
  'service.metrics.flume.sink_connection_failed': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/flume/flume/SINK/ConnectionFailedCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/sinkConnectionFailedCount.json',
    'testInProduction': true
  },
  'service.metrics.flume.source_accepted': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/flume/flume/SOURCE/EventAcceptedCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/sourceEventAccepted.json',
    'testInProduction': true
  },
  'service.metrics.flume.channel_size_for_all': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/ChannelSize/rate[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.channel_size_for_all.mma': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/ChannelSize/rate/avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/ChannelSize/rate/max[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/ChannelSize/rate/min[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.channel_size_for_all.sum': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/ChannelSize/rate/sum[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.gc': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/jvm/gcTimeMillis[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/jvmGcTime.json',
    'testInProduction': true
  },
  'service.metrics.flume.jvm_heap_used': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/jvm/memHeapUsedM[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/jvmMemHeapUsedM.json',
    'testInProduction': true
  },
  'service.metrics.flume.jvm_threads_runnable': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/jvm/threadsRunnable[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/flume/jvmThreadsRunnable.json',
    'testInProduction': true
  },
  'service.metrics.flume.cpu_user': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=host_components/metrics/cpu/cpu_user[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.flume.incoming_event_put_successCount': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventPutSuccessCount/rate[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.incoming_event_put_successCount.mma': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventPutSuccessCount/rate/avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/EventPutSuccessCount/rate/max[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/EventPutSuccessCount/rate/min[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.incoming_event_put_successCount.sum': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventPutSuccessCount/rate/sum[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.outgoing_event_take_success_count': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventTakeSuccessCount/rate[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.outgoing_event_take_success_count.mma': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventTakeSuccessCount/rate/avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/EventTakeSuccessCount/rate/max[{fromSeconds},{toSeconds},{stepSeconds}],metrics/flume/flume/CHANNEL/EventTakeSuccessCount/rate/min[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.flume.outgoing_event_take_success_count.sum': {
    'real': '/clusters/{clusterName}/services/FLUME/components/FLUME_HANDLER?fields=metrics/flume/flume/CHANNEL/EventTakeSuccessCount/rate/sum[{fromSeconds},{toSeconds},{stepSeconds}]'
  },
  'service.metrics.hbase.cluster_requests': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_MASTER?fields=metrics/hbase/master/cluster_requests[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/cluster_requests.json',
    'testInProduction': true
  },
  'service.metrics.hbase.hlog_split_size': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_MASTER?fields=metrics/hbase/master/splitSize_avg_time[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/hlog_split_size.json',
    'testInProduction': true
  },
  'service.metrics.hbase.hlog_split_time': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_MASTER?fields=metrics/hbase/master/splitTime_avg_time[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/hlog_split_time.json',
    'testInProduction': true
  },
  'service.metrics.hbase.regionserver_queuesize': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_REGIONSERVER?fields=metrics/hbase/regionserver/flushQueueSize[{fromSeconds},{toSeconds},{stepSeconds}],metrics/hbase/regionserver/compactionQueueSize[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/regionserver_queuesize.json',
    'testInProduction': true
  },
  'service.metrics.hbase.regionserver_regions': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_REGIONSERVER?fields=metrics/hbase/regionserver/regions[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/regionserver_regions.json',
    'testInProduction': true
  },
  'service.metrics.hbase.regionserver_rw_requests': {
    'real': '/clusters/{clusterName}/services/HBASE/components/HBASE_REGIONSERVER?fields=metrics/hbase/regionserver/readRequestsCount[{fromSeconds},{toSeconds},{stepSeconds}],metrics/hbase/regionserver/writeRequestsCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hbase/regionserver_rw_requests.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.master.average_load': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/master/AverageLoad[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/master_average_load.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.region_server.store_files': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/regionserver/storefiles[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/regionserver_store_files.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.region_server.regions': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/regionserver/regions[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/regionserver_regions.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.region_server.request': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/regionserver/requests._rate[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/regionserver_requests.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.region_server.block_cache_hit_percent': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/regionserver/blockCacheHitPercent[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/regionserver_blockcache_hitpercent.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.region_server.compaction_queue_size': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields=metrics/hbase/regionserver/compactionQueueSize[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/ambari_metrics/regionserver_compaction_queue_size.json',
    'testInProduction': true
  },
  'service.metrics.ambari_metrics.aggregated': {
    'real': '/clusters/{clusterName}/services/AMBARI_METRICS/components/METRICS_COLLECTOR?fields={fields}',
    'mock': '/data/services/metrics/ambari_metrics/master_average_load.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.block_status': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/dfs/FSNamesystem/PendingReplicationBlocks[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/FSNamesystem/UnderReplicatedBlocks[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/block_status.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.file_operations': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/dfs/namenode/FileInfoOps[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/namenode/CreateFileOps[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/file_operations.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.gc': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/jvm/gcTimeMillis[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/gc.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.io': {
    'real': '/clusters/{clusterName}/services/HDFS/components/DATANODE?fields=metrics/dfs/datanode/bytes_written[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/datanode/bytes_read[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/io.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.jvm_heap': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/jvm/memNonHeapUsedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memNonHeapCommittedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memHeapUsedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memHeapCommittedM[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/jvm_heap.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.jvm_threads': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/jvm/threadsRunnable[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsBlocked[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsWaiting[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsTimedWaiting[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/jvm_threads.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.rpc': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/rpc/client/RpcQueueTime_avg_time[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/rpc.json',
    'testInProduction': true
  },
  'service.metrics.hdfs.space_utilization': {
    'real': '/clusters/{clusterName}/hosts/{nameNodeName}/host_components/NAMENODE?fields=metrics/dfs/FSNamesystem/CapacityRemaining[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/FSNamesystem/CapacityUsed[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/FSNamesystem/CapacityTotal[{fromSeconds},{toSeconds},{stepSeconds}],metrics/dfs/FSNamesystem/CapacityNonDFSUsed[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/hdfs/space_utilization.json',
    'testInProduction': true
  },
  'service.metrics.yarn.gc': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/jvm/gcTimeMillis[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/yarn/gc.json',
    'testInProduction': true
  },
  'service.metrics.yarn.jobs_threads': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/jvm/threadsRunnable[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsBlocked[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsWaiting[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/threadsTimedWaiting[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/yarn/jvm_threads.json',
    'testInProduction': true
  },
  'service.metrics.yarn.rpc': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/rpc/RpcQueueTime_avg_time[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/yarn/rpc.json',
    'testInProduction': true
  },
  'service.metrics.yarn.jobs_heap': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/jvm/memNonHeapUsedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memNonHeapCommittedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memHeapUsedM[{fromSeconds},{toSeconds},{stepSeconds}],metrics/jvm/memHeapCommittedM[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/services/metrics/yarn/jvm_heap.json',
    'testInProduction': true
  },
  'service.metrics.yarn.queue.allocated': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/yarn/Queue/root/AvailableMB[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/PendingMB[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/AllocatedMB[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.yarn.queue.allocated.container': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/yarn/Queue/root/AllocatedContainers[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/ReservedContainers[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/PendingContainers[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.yarn.node.manager.statuses': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/yarn/ClusterMetrics/NumActiveNMs[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/ClusterMetrics/NumDecommissionedNMs[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/ClusterMetrics/NumLostNMs[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/ClusterMetrics/NumRebootedNMs[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/ClusterMetrics/NumUnhealthyNMs[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.yarn.queue.memory.resource': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=',
    'mock': '',
    'format': function format(data, opt) {
      var field1 = 'metrics/yarn/Queue/{queueName}/AllocatedMB[{fromSeconds},{toSeconds},{stepSeconds}]';
      var field2 = 'metrics/yarn/Queue/{queueName}/AvailableMB[{fromSeconds},{toSeconds},{stepSeconds}]';
      if (opt.url != null && data.queueNames != null && data.queueNames.length > 0) {
        data.queueNames.forEach(function (q) {
          data.queueName = q;
          opt.url += formatUrl(field1, data) + ",";
          opt.url += formatUrl(field2, data) + ",";
        });
      } else {
        opt.url += formatUrl(field1, data) + ",";
        opt.url += formatUrl(field2, data) + ",";
      }
    },
    'testInProduction': true
  },
  'service.metrics.yarn.queue.apps.states.current': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/yarn/Queue/root/AppsPending[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/AppsRunning[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.yarn.queue.apps.states.finished': {
    'real': '/clusters/{clusterName}/hosts/{resourceManager}/host_components/RESOURCEMANAGER?fields=metrics/yarn/Queue/root/AppsKilled[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/AppsFailed[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/AppsSubmitted[{fromSeconds},{toSeconds},{stepSeconds}],metrics/yarn/Queue/root/AppsCompleted[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '',
    'testInProduction': true
  },
  'service.metrics.kafka.broker.topic': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/server/BrokerTopicMetrics/AllTopicsBytesInPerSec/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}],metrics/kafka/server/BrokerTopicMetrics/AllTopicsBytesOutPerSec/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}],metrics/kafka/server/BrokerTopicMetrics/AllTopicsMessagesInPerSec/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.kafka.controller.KafkaController': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/controller/KafkaController/ActiveControllerCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.kafka.controller.ControllerStats': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/controller/ControllerStats/LeaderElectionRateAndTimeMs/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}],metrics/kafka/controller/ControllerStats/UncleanLeaderElectionsPerSec/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.kafka.log.LogFlushStats': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/log/LogFlushStats/LogFlushRateAndTimeMs/1MinuteRate[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.kafka.server.ReplicaManager': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/server/ReplicaManager/PartitionCount[{fromSeconds},{toSeconds},{stepSeconds}],metrics/kafka/server/ReplicaManager/UnderReplicatedPartitions[{fromSeconds},{toSeconds},{stepSeconds}],metrics/kafka/server/BrokerTopicMetrics/ReplicaManager/LeaderCount[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.kafka.server.ReplicaFetcherManager': {
    'real': '/clusters/{clusterName}/services/KAFKA/components/KAFKA_BROKER?fields=metrics/kafka/server/ReplicaFetcherManager/Replica-MaxLag[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': ''
  },
  'service.metrics.storm.nimbus': {
    'real': '/clusters/{clusterName}/services/STORM/components/NIMBUS?fields={metricsTemplate}',
    'mock': ''
  },
  'dashboard.cluster_metrics.cpu': {
    'real': '/clusters/{clusterName}?fields=metrics/cpu/Nice._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/System._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/User._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/Idle._avg[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/cluster_metrics/cpu_1hr.json',
    'testInProduction': true
  },
  'dashboard.cluster_metrics.load': {
    'real': '/clusters/{clusterName}/?fields=metrics/load/1-min._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/load/CPUs._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/load/Nodes._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/load/Procs._avg[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/cluster_metrics/load_1hr.json',
    'testInProduction': true
  },
  'dashboard.cluster_metrics.memory': {
    'real': '/clusters/{clusterName}/?fields=metrics/memory/Buffer._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/Cache._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/Share._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/Swap._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/Total._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/Use._avg[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/cluster_metrics/memory_1hr.json',
    'testInProduction': true
  },
  'dashboard.cluster_metrics.network': {
    'real': '/clusters/{clusterName}/?fields=metrics/network/In._avg[{fromSeconds},{toSeconds},{stepSeconds}],metrics/network/Out._avg[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/cluster_metrics/network_1hr.json',
    'testInProduction': true
  },
  'host.metrics.aggregated': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields={fields}',
    'mock': '/data/hosts/metrics/cpu.json'
  },
  'host.metrics.cpu': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/cpu/cpu_user[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/cpu_wio[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/cpu_nice[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/cpu_aidle[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/cpu_system[{fromSeconds},{toSeconds},{stepSeconds}],metrics/cpu/cpu_idle[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/cpu.json',
    'testInProduction': true
  },
  'host.metrics.disk': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/disk/disk_total[{fromSeconds},{toSeconds},{stepSeconds}],metrics/disk/disk_free[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/disk.json',
    'testInProduction': true
  },
  'host.metrics.load': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/load/load_fifteen[{fromSeconds},{toSeconds},{stepSeconds}],metrics/load/load_one[{fromSeconds},{toSeconds},{stepSeconds}],metrics/load/load_five[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/load.json',
    'testInProduction': true
  },
  'host.metrics.memory': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/memory/swap_free[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/mem_shared[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/mem_free[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/mem_cached[{fromSeconds},{toSeconds},{stepSeconds}],metrics/memory/mem_buffers[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/memory.json',
    'testInProduction': true
  },
  'host.metrics.network': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/network/bytes_in[{fromSeconds},{toSeconds},{stepSeconds}],metrics/network/bytes_out[{fromSeconds},{toSeconds},{stepSeconds}],metrics/network/pkts_in[{fromSeconds},{toSeconds},{stepSeconds}],metrics/network/pkts_out[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/network.json',
    'testInProduction': true
  },
  'host.metrics.processes': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=metrics/process/proc_total[{fromSeconds},{toSeconds},{stepSeconds}],metrics/process/proc_run[{fromSeconds},{toSeconds},{stepSeconds}]',
    'mock': '/data/hosts/metrics/processes.json',
    'testInProduction': true
  },
  'admin.security_status': {
    'real': '/clusters/{clusterName}?fields=Clusters/security_type',
    'mock': '',
    'format': function format() {
      return {
        timeout: 10000
      };
    }
  },
  'cluster.load_cluster_name': {
    'real': '/clusters?fields=Clusters/security_type,Clusters/version,Clusters/cluster_id',
    'mock': '/data/clusters/info.json'
  },
  'cluster.load_last_upgrade': {
    'real': '/clusters/{clusterName}/upgrades?fields=Upgrade/*',
    'mock': '/data/stack_versions/upgrades.json'
  },
  'cluster.update_upgrade_version': {
    'real': '/stacks/{stackName}/versions?fields=services/StackServices,Versions',
    'mock': '/data/wizard/stack/stacks.json',
    'format': function format(data) {
      return {
        data: data.data
      };
    }
  },
  'cluster.load_repositories': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/operating_systems?fields=repositories/*,OperatingSystems/*',
    'mock': '/data/stacks/HDP-2.1/operating_systems.json',
    'format': function format(data) {
      return {
        data: data.data
      };
    }
  },
  'cluster.load_repo_version': {
    'real': '/stacks/{stackName}/versions?fields=repository_versions/operating_systems/repositories/*,repository_versions/operating_systems/OperatingSystems/*,repository_versions/RepositoryVersions/display_name&repository_versions/RepositoryVersions/repository_version={repositoryVersion}',
    'mock': ''
  },
  'cluster.load_detailed_repo_version': {
    'real': '/clusters/{clusterName}/stack_versions?fields=repository_versions/RepositoryVersions/repository_version,ClusterStackVersions/stack,ClusterStackVersions/version&minimal_response=true',
    'mock': '/data/stack_versions/stack_version_all.json'
  },
  'cluster.load_current_repo_stack_services': {
    'real': '/clusters/{clusterName}/stack_versions?fields=repository_versions/RepositoryVersions/stack_services,ClusterStackVersions/stack,ClusterStackVersions/version',
    'mock': '/data/stack_versions/stack_version_all.json'
  },
  'cluster.save_provisioning_state': {
    'real': '/clusters/{clusterName}',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Clusters": {
            "provisioning_state": data.state
          }
        })
      };
    }
  },

  'cluster.logging.searchEngine': {
    real: '/clusters/{clusterName}/logging/searchEngine?{query}',
    mock: ''
  },
  'admin.high_availability.polling': {
    'real': '/clusters/{clusterName}/requests/{requestId}?fields=tasks/*,Requests/*',
    'mock': '/data/background_operations/host_upgrade_tasks.json'
  },
  'admin.high_availability.getNnCheckPointStatus': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/NAMENODE',
    'mock': ''
  },
  'admin.high_availability.getNnCheckPointsStatuses': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name=NAMENODE&HostRoles/host_name.in({hostNames})&fields=HostRoles/desired_state,metrics/dfs/namenode&minimal_response=true',
    'mock': ''
  },
  'admin.high_availability.getJnCheckPointStatus': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/JOURNALNODE?fields=metrics',
    'mock': ''
  },
  'admin.high_availability.getHostComponent': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/{componentName}',
    'mock': ''
  },
  'common.create_component': {
    'real': '/clusters/{clusterName}/services?ServiceInfo/service_name={serviceName}',
    'mock': '',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "components": [{
            "ServiceComponentInfo": {
              "component_name": data.componentName
            }
          }]
        })
      };
    }
  },
  'admin.high_availability.load_configs': {
    'real': '/clusters/{clusterName}/configurations?(type=core-site&tag={coreSiteTag})|(type=hdfs-site&tag={hdfsSiteTag})',
    'mock': ''
  },
  'admin.save_configs': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Clusters: {
            desired_config: {
              "type": data.siteName,
              "properties": data.properties
            }
          }
        })
      };
    }
  },
  'admin.high_availability.load_hbase_configs': {
    'real': '/clusters/{clusterName}/configurations?type=hbase-site&tag={hbaseSiteTag}',
    'mock': ''
  },
  'admin.security.cluster_configs': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'format': function format() {
      return {
        timeout: 10000
      };
    }
  },
  'admin.security.cluster_configs.kerberos': {
    'real': '/clusters/{clusterName}/configurations/service_config_versions?service_name=KERBEROS&is_current=true',
    'mock': '',
    'format': function format() {
      return {
        timeout: 10000
      };
    }
  },
  'admin.get.all_configurations': {
    'real': '/clusters/{clusterName}/configurations?{urlParams}',
    'mock': '',
    'format': function format() {
      return {
        timeout: 10000
      };
    }
  },

  'kerberos.session.state': {
    'real': '/clusters/{clusterName}/services/KERBEROS?fields=Services/attributes/kdc_validation_result,Services/attributes/kdc_validation_failure_details',
    'mock': ''
  },

  'admin.kerberize.cluster': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}',
    'mock': '/data/wizard/kerberos/kerberize_cluster.json',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },
  'admin.unkerberize.cluster': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Clusters: {
            security_type: "NONE"
          }
        })
      };
    }
  },

  'admin.kerberize.cluster.force': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}?force_toggle_kerberos=true',
    'mock': '/data/wizard/kerberos/kerberize_cluster.json',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Clusters: {
            security_type: "KERBEROS"
          }
        })
      };
    }
  },

  'admin.unkerberize.cluster.skip': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}?manage_kerberos_identities=false',
    'mock': '',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          Clusters: {
            security_type: "NONE"
          }
        })
      };
    }
  },

  'get.cluster.artifact': {
    'real': '/clusters/{clusterName}/artifacts/{artifactName}?fields=artifact_data',
    'mock': '/data/wizard/kerberos/stack_descriptors.json'
  },
  'admin.kerberize.stack_descriptor': {
    'real': '/clusters/{clusterName}/kerberos_descriptors/STACK',
    'mock': '/data/wizard/kerberos/stack_descriptors.json'
  },
  'admin.kerberize.cluster_descriptor_artifact': {
    'real': '/clusters/{clusterName}/artifacts/kerberos_descriptor?fields=artifact_data',
    'mock': '/data/wizard/kerberos/stack_descriptors.json'
  },
  'admin.kerberize.cluster_descriptor': {
    'real': '/clusters/{clusterName}/kerberos_descriptors/COMPOSITE{queryParams}',
    'mock': '/data/wizard/kerberos/stack_descriptors.json'
  },
  'admin.kerberize.cluster_descriptor.stack': {
    'real': '/clusters/{clusterName}/kerberos_descriptors/STACK',
    'mock': '/data/wizard/kerberos/stack_descriptors.json'
  },
  'admin.kerberos.cluster.artifact.create': {
    'type': 'POST',
    'real': '/clusters/{clusterName}/artifacts/{artifactName}',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },
  'admin.kerberos.cluster.artifact.update': {
    'type': 'PUT',
    'real': '/clusters/{clusterName}/artifacts/{artifactName}',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },
  'admin.kerberos.cluster.csv': {
    'real': '/clusters/{clusterName}/kerberos_identities?fields=*&format=csv',
    'mock': '',
    'format': function format(data) {
      return {
        dataType: 'text',
        data: data.data
      };
    }
  },
  'admin.poll.kerberize.cluster.request': {
    'real': '/clusters/{clusterName}/requests/{requestId}?fields=stages/Stage/context,stages/Stage/status,stages/Stage/progress_percent,stages/tasks/*,Requests/*',
    'mock': '/data/wizard/kerberos/kerberize_cluster.json'
  },
  'admin.stack_upgrade.run_upgrade': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: data.data
      };
    }
  },
  'admin.user.create': {
    'real': '/users/{user}',
    'mock': '/data/users/users.json',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },

  'admin.user.edit': {
    'real': '/users/{user}',
    'mock': '/data/users/users.json',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: data.data
      };
    }
  },

  'admin.stack_upgrade.do_poll': {
    'real': '/clusters/{cluster}/requests/{requestId}?fields=tasks/*',
    'mock': '/data/wizard/{mock}'
  },
  'admin.upgrade.data': {
    'real': '/clusters/{clusterName}/upgrades/{id}?upgrade_groups/UpgradeGroup/status!=PENDING&fields=' + 'Upgrade/progress_percent,Upgrade/request_context,Upgrade/request_status,Upgrade/direction,Upgrade/downgrade_allowed,' + 'upgrade_groups/UpgradeGroup,' + 'Upgrade/*,' + 'upgrade_groups/upgrade_items/UpgradeItem/status,' + 'upgrade_groups/upgrade_items/UpgradeItem/display_status,' + 'upgrade_groups/upgrade_items/UpgradeItem/context,' + 'upgrade_groups/upgrade_items/UpgradeItem/group_id,' + 'upgrade_groups/upgrade_items/UpgradeItem/progress_percent,' + 'upgrade_groups/upgrade_items/UpgradeItem/request_id,' + 'upgrade_groups/upgrade_items/UpgradeItem/skippable,' + 'upgrade_groups/upgrade_items/UpgradeItem/stage_id,' + 'upgrade_groups/upgrade_items/UpgradeItem/text&' + 'minimal_response=true',
    'mock': '/data/stack_versions/upgrade.json'
  },
  'admin.upgrade.state': {
    'real': '/clusters/{clusterName}/upgrades/{id}?fields=Upgrade/*',
    'mock': '/data/stack_versions/upgrade.json'
  },
  'admin.upgrade.finalizeContext': {
    'real': '/clusters/{clusterName}/upgrades/{id}?upgrade_groups/upgrade_items/UpgradeItem/status=HOLDING&fields=upgrade_groups/upgrade_items/UpgradeItem/context',
    'mock': '/data/stack_versions/upgrade.json'
  },
  'admin.upgrade.upgrade_item': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}/upgrade_groups/{groupId}/upgrade_items/{stageId}?fields=' + 'UpgradeItem/group_id,' + 'UpgradeItem/stage_id,' + 'tasks/Tasks/command_detail,' + 'tasks/Tasks/host_name,' + 'tasks/Tasks/role,' + 'tasks/Tasks/request_id,' + 'tasks/Tasks/stage_id,' + 'tasks/Tasks/status,' + 'tasks/Tasks/structured_out&' + 'minimal_response=true',
    'mock': '/data/stack_versions/upgrade_item.json'
  },
  'admin.upgrade.upgrade_task': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}/upgrade_groups/{groupId}/upgrade_items/{stageId}/tasks/{taskId}',
    'mock': ''
  },
  'admin.upgrade.service_checks': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}/upgrade_groups?upgrade_items/UpgradeItem/status=COMPLETED&upgrade_items/tasks/Tasks/status.in(FAILED,ABORTED,TIMEDOUT)&upgrade_items/tasks/Tasks/command=SERVICE_CHECK&fields=upgrade_items/tasks/Tasks/command_detail,tasks/Tasks/ops_display_name,upgrade_items/tasks/Tasks/status&minimal_response=true'
  },
  'admin.upgrade.update.options': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}',
    'mock': '/data/stack_versions/start_upgrade.json',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Upgrade": {
            "skip_failures": data.skipComponentFailures,
            "skip_service_check_failures": data.skipSCFailures
          }
        })
      };
    }
  },
  'admin.upgrade.start': {
    'real': '/clusters/{clusterName}/upgrades',
    'mock': '/data/stack_versions/start_upgrade.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        timeout: 600000,
        data: JSON.stringify({
          "Upgrade": {
            "repository_version_id": data.id,
            "upgrade_type": data.type,
            "skip_failures": data.skipComponentFailures,
            "skip_service_check_failures": data.skipSCFailures,
            "direction": "UPGRADE"
          }
        })
      };
    }
  },
  'admin.downgrade.start': {
    'real': '/clusters/{clusterName}/upgrades',
    'mock': '/data/stack_versions/start_upgrade.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Upgrade": {
            "upgrade_type": data.upgradeType,
            "direction": "DOWNGRADE"
          }
        })
      };
    }
  },
  'admin.upgrade.abort': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Upgrade": {
            "request_status": "ABORTED",
            "suspended": "false"
          }
        })
      };
    }
  },
  'admin.upgrade.suspend': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Upgrade": {
            "request_status": "ABORTED",
            "suspended": "true"
          }
        })
      };
    }
  },
  'admin.upgrade.retry': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Upgrade": {
            "request_status": "PENDING"
          }
        })
      };
    }
  },
  'admin.upgrade.revert': {
    'real': '/clusters/{clusterName}/upgrades',
    'mock': '/data/stack_versions/start_upgrade.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        timeout: 600000,
        data: JSON.stringify({
          "Upgrade": {
            "revert_upgrade_id": data.upgradeId
          }
        })
      };
    }
  },
  'admin.upgrade.upgradeItem.setState': {
    'real': '/clusters/{clusterName}/upgrades/{upgradeId}/upgrade_groups/{groupId}/upgrade_items/{itemId}',
    'mock': '',
    type: 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "UpgradeItem": {
            "status": data.status
          }
        })
      };
    }
  },
  'admin.stack_versions.all': {
    'real': '/clusters/{clusterName}/stack_versions?fields=ClusterStackVersions/*,repository_versions/RepositoryVersions/*&minimal_response=true',
    'mock': '/data/stack_versions/stack_version_all.json'
  },
  'admin.stack_version.install.repo_version': {
    'real': '/clusters/{clusterName}/stack_versions',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          ClusterStackVersions: data.ClusterStackVersions
        })
      };
    },
    'mock': ''
  },

  'admin.stack_versions.edit.repo': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/repository_versions/{repoVersionId}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.repoVersion)
      };
    }
  },
  'admin.stack_versions.validate.repo': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/operating_systems/{osType}/repositories/{repoId}?validate_only=true',
    'mock': '',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Repositories": {
            "base_url": data.baseUrl,
            "repo_name": data.repoName
          }
        })
      };
    }
  },

  'admin.stack_versions.discard': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/repository_versions/{id}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "RepositoryVersions": {
            "hidden": "true"
          }
        })
      };
    }
  },

  'admin.upgrade.pre_upgrade_check': {
    'real': '/clusters/{clusterName}/rolling_upgrades_check?fields=*&UpgradeChecks/repository_version_id={id}&UpgradeChecks/upgrade_type={type}',
    'mock': '/data/stack_versions/pre_upgrade_check.json'
  },

  'admin.upgrade.get_supported_upgradeTypes': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/compatible_repository_versions?CompatibleRepositoryVersions/repository_version={toVersion}',
    'mock': '/data/stack_versions/supported_upgrade_types.json'
  },

  'admin.upgrade.get_compatible_versions': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/compatible_repository_versions?fields=CompatibleRepositoryVersions/repository_version&minimal_response=true',
    'mock': '/data/stack_versions/supported_upgrade_types.json'
  },

  'admin.kerberos_security.checks': {
    //TODO when api will be known
    'real': '',
    'mock': '/data/stack_versions/pre_upgrade_check.json'
  },

  'admin.kerberos_security.test_connection': {
    'real': '/kdc_check/{kdcHostname}',
    'mock': '',
    'format': function format() {
      return {
        dataType: 'text'
      };
    }
  },

  'admin.kerberos_security.regenerate_keytabs': {
    'real': '/clusters/{clusterName}?regenerate_keytabs={type}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Clusters": {
            "security_type": "KERBEROS"
          }
        })
      };
    }
  },

  'admin.kerberos_security.regenerate_keytabs.service': {
    'real': '/clusters/{clusterName}?regenerate_keytabs=all&regenerate_components={serviceName}&config_update_policy=none',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Clusters": {
            "security_type": "KERBEROS"
          }
        })
      };
    }
  },

  'admin.kerberos_security.regenerate_keytabs.host': {
    'real': '/clusters/{clusterName}?regenerate_keytabs=all&regenerate_hosts={hostName}&config_update_policy=none',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "Clusters": {
            "security_type": "KERBEROS"
          }
        })
      };
    }
  },

  'wizard.step1.post_version_definition_file.xml': {
    'real': '/version_definitions?dry_run=true',
    'mock': '',
    'format': function format(data) {
      return {
        headers: {
          'X-Requested-By': 'ambari',
          'Content-Type': 'text/xml'
        },
        type: 'POST',
        data: data.data
      };
    }
  },
  'wizard.step1.post_version_definition_file.url': {
    'real': '/version_definitions?dry_run=true',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },
  'wizard.step8.post_version_definition_file.xml': {
    'real': '/version_definitions',
    'mock': '',
    'format': function format(data) {
      return {
        headers: {
          'X-Requested-By': 'ambari',
          'Content-Type': 'text/xml'
        },
        type: 'POST',
        data: data.data
      };
    }
  },
  'wizard.step8.post_version_definition_file': {
    'real': '/version_definitions',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },
  'wizard.step1.get_repo_version_by_id': {
    'real': '/stacks/{stackName}/versions?fields=repository_versions/operating_systems/repositories/*' + ',repository_versions/RepositoryVersions/*' + '&repository_versions/RepositoryVersions/id={repoId}&Versions/stack_version={stackVersion}',
    'mock': ''
  },

  'wizard.step1.get_supported_os_types': {
    'real': '/stacks/{stackName}/versions/{stackVersion}?fields=operating_systems/repositories/Repositories',
    'mock': ''
  },

  'wizard.advanced_repositories.valid_url': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/operating_systems/{osType}/repositories/{repoId}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },
  'wizard.get_version_definitions': {
    'real': '/version_definitions'
  },
  'wizard.delete_repository_versions': {
    'real': '/stacks/{stackName}/versions/{stackVersion}/repository_versions/{id}',
    'type': 'DELETE'
  },
  'wizard.service_components': {
    'real': '{stackUrl}/services?fields=StackServices/*,components/*,components/dependencies/Dependencies/scope,components/dependencies/Dependencies/service_name,artifacts/Artifacts/artifact_name',
    'mock': '/data/stacks/HDP-2.1/service_components.json'
  },
  'wizard.step9.installer.get_host_status': {
    'real': '/clusters/{cluster}/hosts?fields=Hosts/host_state,host_components/HostRoles/state',
    'mock': '/data/wizard/deploy/5_hosts/get_host_state.json'
  },
  'wizard.step9.load_log': {
    'real': '/clusters/{cluster}/requests/{requestId}?fields=tasks/Tasks/command,tasks/Tasks/command_detail,tasks/Tasks/ops_display_name,tasks/Tasks/exit_code,tasks/Tasks/start_time,tasks/Tasks/end_time,tasks/Tasks/host_name,tasks/Tasks/id,tasks/Tasks/role,tasks/Tasks/status&minimal_response=true',
    'mock': '/data/wizard/deploy/5_hosts/poll_{numPolls}.json',
    'format': function format() {
      return {
        dataType: 'text'
      };
    }
  },

  'wizard.step8.existing_cluster_names': {
    'real': '/clusters',
    'mock': ''
  },

  'wizard.step8.create_cluster': {
    'real': '/clusters/{cluster}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.create_selected_services': {
    'type': 'POST',
    'real': '/clusters/{cluster}/services',
    'mock': '/data/stacks/HDP-2.1/recommendations.json',
    'format': function format(data) {
      return {
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.create_components': {
    'real': '/clusters/{cluster}/services?ServiceInfo/service_name={serviceName}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.register_host_to_cluster': {
    'real': '/clusters/{cluster}/hosts',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.register_host_to_component': {
    'real': '/clusters/{cluster}/hosts',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.apply_configuration_groups': {
    'real': '/clusters/{cluster}/config_groups',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        dataType: 'text',
        data: data.data
      };
    }
  },

  'wizard.step8.set_local_repos': {
    'real': '{stackVersionURL}/operating_systems/{osType}/repositories/{repoId}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        dataType: 'text',
        data: data.data
      };
    }
  },
  'wizard.step3.jdk_check': {
    'real': '/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "context": "Check hosts",
            "action": "check_host",
            "parameters": {
              "threshold": "60",
              "java_home": data.java_home,
              "jdk_location": data.jdk_location,
              "check_execute_list": "java_home_check"
            }
          },
          "Requests/resource_filters": [{
            "hosts": data.host_names
          }]
        })
      };
    }
  },
  'wizard.step3.jdk_check.get_results': {
    'real': '/requests/{requestIndex}?fields=*,tasks/Tasks/host_name,tasks/Tasks/status,tasks/Tasks/structured_out',
    'mock': '/data/requests/host_check/jdk_check_results.json'
  },
  'wizard.step3.host_info': {
    'real': '/hosts?fields=Hosts/total_mem,Hosts/cpu_count,Hosts/disk_info,Hosts/last_agent_env,Hosts/host_name,Hosts/os_type,Hosts/os_arch,Hosts/os_family,Hosts/ip',
    'mock': '/data/wizard/bootstrap/two_hosts_information.json',
    'format': function format() {
      return {
        contentType: 'application/json'
      };
    }
  },

  'wizard.loadrecommendations': {
    'real': '{stackVersionUrl}/recommendations',
    'mock': '/data/stacks/HDP-2.1/recommendations.json',
    'type': 'POST',
    'format': function format(data) {
      var q = {
        hosts: data.hosts,
        services: data.services,
        recommend: data.recommend
      };

      if (data.recommendations) {
        q.recommendations = data.recommendations;
      }

      return {
        data: JSON.stringify(q)
      };
    }
  },

  // TODO: merge with wizard.loadrecommendations query
  'config.recommendations': {
    'real': '{stackVersionUrl}/recommendations',
    'mock': '/data/configurations/recommendations/configuration_dependencies.json',
    //'mock': '/data/stacks/HDP-2.1/recommendations_configs.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.dataToSend)
      };
    }
  },

  'config.validations': {
    'real': '{stackVersionUrl}/validations',
    'mock': '/data/stacks/HDP-2.1/validations.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          hosts: data.hosts,
          services: data.services,
          validate: data.validate,
          recommendations: data.recommendations
        })
      };
    }
  },

  'preinstalled.checks': {
    'real': '/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": data.RequestInfo,
          "Requests/resource_filters": [data.resource_filters]
        })
      };
    }
  },

  'preinstalled.checks.tasks': {
    'real': '/requests/{requestId}?fields=Requests/inputs,Requests/request_status,tasks/Tasks/host_name,' + 'tasks/Tasks/structured_out/host_resolution_check/hosts_with_failures,' + 'tasks/Tasks/structured_out/host_resolution_check/failed_count,' + 'tasks/Tasks/structured_out/installed_packages,' + 'tasks/Tasks/structured_out/last_agent_env_check,' + 'tasks/Tasks/structured_out/transparentHugePage,' + 'tasks/Tasks/stdout,' + 'tasks/Tasks/stderr,' + 'tasks/Tasks/error_log,' + 'tasks/Tasks/command_detail,' + 'tasks/Tasks/status' + '&minimal_response=true',
    'mock': '/data/requests/host_check/1.json'
  },

  'persist.get.text': {
    'real': '/persist/{key}',
    'mock': '',
    'type': 'GET',
    'format': function format() {
      return {
        dataType: 'text'
      };
    }
  },

  'persist.get': {
    'real': '/persist/{key}',
    'mock': '',
    'type': 'GET'
  },
  'persist.post': {
    'real': '/persist',
    'mock': '',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.keyValuePair)
      };
    }
  },

  'wizard.step3.rerun_checks': {
    'real': '/hosts?fields=Hosts/last_agent_env',
    'mock': '/data/wizard/bootstrap/two_hosts_information.json',
    'format': function format() {
      return {
        contentType: 'application/json'
      };
    }
  },
  'wizard.step3.bootstrap': {
    'real': '/bootstrap/{bootRequestId}',
    'mock': '/data/wizard/bootstrap/poll_{numPolls}.json'
  },
  'wizard.step3.is_hosts_registered': {
    'real': '/hosts?fields=Hosts/host_status',
    'mock': '/data/wizard/bootstrap/single_host_registration.json'
  },
  'wizard.stacks': {
    'real': '/stacks',
    'mock': '/data/wizard/stack/stacks2.json'
  },
  'wizard.stacks_versions': {
    'real': '/stacks/{stackName}/versions?fields=Versions,operating_systems/repositories/Repositories',
    'mock': '/data/wizard/stack/{stackName}_versions.json'
  },

  'wizard.stacks_versions_definitions': {
    'real': '/version_definitions?fields=VersionDefinition/stack_default,VersionDefinition/stack_repo_update_link_exists,VersionDefinition/max_jdk,VersionDefinition/min_jdk,operating_systems/repositories/Repositories/*,operating_systems/OperatingSystems/*,VersionDefinition/stack_services,VersionDefinition/repository_version' + '&VersionDefinition/show_available=true&VersionDefinition/stack_name={stackName}',
    'mock': '/data/wizard/stack/{stackName}_version_definitions.json'
  },
  'wizard.launch_bootstrap': {
    'real': '/bootstrap',
    'mock': '/data/wizard/bootstrap/bootstrap.json',
    'type': 'POST',
    'format': function format(data) {
      return {
        contentType: 'application/json',
        data: data.bootStrapData,
        popup: data.popup
      };
    }
  },
  'router.login': {
    'real': '/auth',
    'mock': '/data/users/user_{usr}.json',
    'type': 'POST',
    'format': function format(data) {
      var statusCode = jQuery.extend({}, require('data/statusCodes'));
      statusCode['403'] = function () {
        console.log("Error code 403: Forbidden.");
      };
      return {
        statusCode: statusCode
      };
    }
  },
  'router.afterLogin': {
    'real': '/users/{loginName}?fields=*,privileges/PrivilegeInfo/cluster_name,privileges/PrivilegeInfo/permission_name',
    'mock': '/data/users/user_{usr}.json',
    'format': function format(data) {
      var statusCode = jQuery.extend({}, require('data/statusCodes'));
      statusCode['403'] = function () {
        console.log("Error code 403: Forbidden.");
      };
      return {
        statusCode: statusCode
      };
    }
  },
  'users.all': {
    real: '/users/?fields=*',
    mock: '/data/users/users.json'
  },
  'users.privileges': {
    real: '/privileges?fields=*',
    mock: '/data/users/privileges.json'
  },
  'router.user.privileges': {
    real: '/users/{userName}/privileges?fields=*',
    mock: '/data/users/privileges_{userName}.json'
  },
  'router.user.authorizations': {
    real: '/users/{userName}/authorizations?fields=*',
    mock: '/data/users/privileges_{userName}.json'
  },
  'router.login.clusters': {
    'real': '/clusters?fields=Clusters/provisioning_state,Clusters/security_type,Clusters/version,Clusters/cluster_id',
    'mock': '/data/clusters/info.json'
  },
  'router.login.message': {
    'real': '/settings/motd',
    'mock': '/data/settings/motd.json'
  },
  'router.logoff': {
    'real': '/logout',
    'mock': '',
    format: function format() {
      // Workaround for sign off within Basic Authorization
      return {
        username: Date.now(),
        password: Date.now()
      };
    }
  },
  'ambari.service': {
    'real': '/services/AMBARI/components/AMBARI_SERVER{fields}',
    'mock': '/data/services/ambari_server.json'
  },

  'config_groups.create': {
    'real': '/clusters/{clusterName}/config_groups',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },
  'config_groups.update': {
    'real': '/clusters/{clusterName}/config_groups/{id}',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },
  'request_schedule.get': {
    'real': '/clusters/{clusterName}/request_schedules/{request_schedule_id}',
    'mock': ''
  },
  'request_schedule.get.pending': {
    'real': '/clusters/{clusterName}/request_schedules?fields=*&(RequestSchedule/status.in(SCHEDULED,IN_PROGRESS))',
    'mock': ''
  },
  'restart.hostComponents': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "RESTART",
            "context": data.context,
            "operation_level": data.operation_level
          },
          "Requests/resource_filters": data.resource_filters
        })
      };
    }
  },

  'restart.allServices': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "RESTART",
            "context": 'Restart all services',
            "operation_level": 'host_component'
          },
          "Requests/resource_filters": [{
            "hosts_predicate": "HostRoles/cluster_name=" + data.clusterName
          }]
        })
      };
    }
  },

  'restart.staleConfigs': {
    'real': "/clusters/{clusterName}/requests",
    'mock': "",
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "RESTART",
            "context": "Restart all required services",
            "operation_level": "host_component"
          },
          "Requests/resource_filters": [{
            "hosts_predicate": "HostRoles/stale_configs=true&HostRoles/cluster_name=" + data.clusterName
          }]
        })
      };
    }
  },

  'restart.custom.filter': {
    'real': "/clusters/{clusterName}/requests",
    'mock': "",
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "RESTART",
            "context": data.context,
            "operation_level": "host_component"
          },
          "Requests/resource_filters": [{
            "hosts_predicate": data.filter
          }]
        })
      };
    }
  },

  'bulk_request.decommission': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          'RequestInfo': {
            'context': data.context,
            'command': 'DECOMMISSION',
            'parameters': data.parameters,
            'operation_level': {
              'level': "CLUSTER",
              'cluster_name': data.clusterName
            }
          },
          "Requests/resource_filters": [{ "service_name": data.serviceName, "component_name": data.componentName }]
        })
      };
    }
  },

  'bulk_request.hosts.passive_state': {
    'real': '/clusters/{clusterName}/hosts',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            context: data.requestInfo,
            query: 'Hosts/host_name.in(' + data.hostNames + ')'
          },
          Body: {
            Hosts: {
              maintenance_state: data.passive_state
            }
          }
        })
      };
    }
  },

  'bulk_request.hosts.update_rack_id': {
    'real': '/clusters/{clusterName}/hosts',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            context: data.requestInfo,
            query: 'Hosts/host_name.in(' + data.hostNames + ')'
          },
          Body: {
            Hosts: {
              rack_info: data.rackId
            }
          }
        })
      };
    }
  },

  'bulk_request.hosts.all_components.passive_state': {
    'real': '/clusters/{clusterName}/host_components',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify({
          RequestInfo: {
            context: data.requestInfo,
            query: data.query
          },
          Body: {
            HostRoles: {
              maintenance_state: data.passive_state
            }
          }
        })
      };
    }
  },
  'views.info': {
    'real': '/views',
    'mock': '/data/views/views.json'
  },
  /**
   * Get all instances of all views across versions
   */
  'views.instances': {
    'real': '/views?fields=versions/instances/ViewInstanceInfo,versions/ViewVersionInfo/label&versions/ViewVersionInfo/system=false',
    'mock': '/data/views/instances.json'
  },
  'host.host_component.flume.metrics': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/host_components/FLUME_HANDLER?fields=metrics/flume/flume/{flumeComponent}/*',
    'mock': ''
  },
  'host.host_component.flume.metrics.timeseries': {
    'real': '',
    'mock': '',
    format: function format(data) {
      return {
        url: data.url
      };
    }
  },
  'host.host_components.filtered': {
    'real': '/clusters/{clusterName}/hosts?{fields}',
    'mock': '',
    format: function format(data) {
      return {
        headers: {
          'X-Http-Method-Override': 'GET'
        },
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": { "query": data.parameters }
        })
      };
    }
  },
  'host.stack_versions.install': {
    'real': '/clusters/{clusterName}/hosts/{hostName}/stack_versions',
    'mock': '',
    'type': 'POST',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          "HostStackVersions": {
            "stack": data.version.get('stack'),
            "version": data.version.get('version'),
            "repository_version": data.version.get('repoVersion')
          }
        })
      };
    }
  },

  'host.logging': {
    'real': '/clusters/{clusterName}/hosts/{hostName}?fields=host_components/logging,host_components/HostRoles/service_name{fields}{query}&minimal_response=true',
    'mock': ''
  },
  'components.filter_by_status': {
    'real': '/clusters/{clusterName}/components?fields=host_components/HostRoles/host_name,ServiceComponentInfo/component_name,ServiceComponentInfo/started_count{urlParams}&minimal_response=true',
    'mock': ''
  },
  'components.get_category': {
    'real': '/clusters/{clusterName}/components?fields=ServiceComponentInfo/component_name,ServiceComponentInfo/service_name,ServiceComponentInfo/category,ServiceComponentInfo/recovery_enabled,ServiceComponentInfo/total_count&minimal_response=true',
    'mock': ''
  },
  'components.get.staleConfigs': {
    'real': '/clusters/{clusterName}/components?host_components/HostRoles/stale_configs=true' + '&fields=host_components/HostRoles/host_name&minimal_response=true',
    'mock': ''
  },
  'components.update': {
    'real': '/clusters/{clusterName}/components?{urlParams}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify({
          RequestInfo: {
            query: data.query
          },
          ServiceComponentInfo: data.ServiceComponentInfo
        })
      };
    }
  },
  'hosts.all.install': {
    'real': '/hosts?minimal_response=true',
    'mock': ''
  },
  'hosts.all': {
    'real': '/clusters/{clusterName}/hosts?minimal_response=true',
    'mock': '/data/hosts/HDP2/hosts.json'
  },
  'hosts.with_public_host_names': {
    'real': '/clusters/{clusterName}/hosts?fields=Hosts/public_host_name&minimal_response=true',
    'mock': ''
  },
  'hosts.for_quick_links': {
    'real': '/clusters/{clusterName}/hosts?Hosts/host_name.in({hosts})&fields=Hosts/public_host_name{urlParams}&minimal_response=true',
    'mock': '/data/hosts/quick_links.json'
  },
  'hosts.confirmed.install': {
    'real': '/hosts?fields=Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/maintenance_state&minimal_response=true',
    'mock': ''
  },
  'hosts.confirmed': {
    'real': '/clusters/{clusterName}/hosts?fields=host_components/HostRoles/state&minimal_response=true',
    'mock': '/data/hosts/HDP2/hosts.json'
  },
  'hosts.with_searchTerm': {
    'real': '/clusters/{clusterName}/hosts?fields=Hosts/{facet}&minimal_response=true&page_size={page_size}',
    'mock': '',
    format: function format(data) {
      return {
        headers: {
          'X-Http-Method-Override': 'GET'
        },
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": { "query": data.searchTerm ? "Hosts/" + data.facet + ".matches(.*" + data.searchTerm + ".*)" : "" }
        })
      };
    }
  },
  'hosts.confirmed.minimal': {
    'real': '/clusters/{clusterName}/hosts?fields=host_components/HostRoles/state&minimal_response=true',
    'mock': '/data/hosts/HDP2/hosts.json'
  },
  'hosts.heartbeat_lost': {
    'real': '/clusters/{clusterName}/hosts?Hosts/host_state=HEARTBEAT_LOST',
    'mock': ''
  },
  'host_components.all': {
    'real': '/clusters/{clusterName}/host_components?fields=HostRoles/host_name&minimal_response=true',
    'mock': ''
  },
  'host_components.with_services_names': {
    'real': '/clusters/{clusterName}/host_components?fields=component/ServiceComponentInfo/service_name,HostRoles/host_name&minimal_response=true',
    'mock': ''
  },
  'components.get_installed': {
    'real': '/clusters/{clusterName}/components',
    'mock': ''
  },
  'hosts.heatmaps': {
    'real': '/clusters/{clusterName}/hosts?fields=Hosts/rack_info,Hosts/host_name,Hosts/public_host_name,Hosts/os_type,Hosts/ip,host_components,metrics/disk,metrics/cpu/cpu_system,metrics/cpu/cpu_user,metrics/memory/mem_total,metrics/memory/mem_free&minimal_response=true',
    'mock': '/data/hosts/HDP2/hosts.json'
  },
  'namenode.cpu_wio': {
    'real': '/clusters/{clusterName}/hosts/{nnHost}?fields=metrics/cpu',
    'mock': '/data/cluster_metrics/cpu.json'
  },

  'custom_action.create': {
    'real': '/requests',
    'mock': '',
    'format': function format(data) {
      var requestInfo = {
        context: 'Check host',
        action: 'check_host',
        parameters: {}
      };
      $.extend(true, requestInfo, data.requestInfo);
      return {
        type: 'POST',
        data: JSON.stringify({
          'RequestInfo': requestInfo,
          'Requests/resource_filters': [{
            hosts: data.filteredHosts.join(',')
          }]
        })
      };
    }
  },

  'cluster.custom_action.create': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      var requestInfo = {
        context: 'Check host',
        action: 'check_host',
        parameters: {}
      };
      $.extend(true, requestInfo, data.requestInfo);
      return {
        type: 'POST',
        data: JSON.stringify({
          'RequestInfo': requestInfo,
          'Requests/resource_filters': [{
            hosts: data.filteredHosts.join(',')
          }]
        })
      };
    }
  },

  'custom_action.request': {
    'real': '/requests/{requestId}/tasks/{taskId}',
    'mock': '/data/requests/1.json',
    'format': function format(data) {
      return {
        requestId: data.requestId,
        taskId: data.taskId || ''
      };
    }
  },
  'hosts.high_availability.wizard': {
    'real': '/clusters/{clusterName}/hosts?fields=Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/maintenance_state&minimal_response=true',
    'mock': ''
  },
  'hosts.security.wizard': {
    'real': '/clusters/{clusterName}/hosts?fields=host_components/HostRoles/service_name&minimal_response=true',
    'mock': ''
  },
  'host_component.installed.on_hosts': {
    'real': '/clusters/{clusterName}/host_components?HostRoles/component_name={componentName}&HostRoles/host_name.in({hostNames})&fields=HostRoles/host_name&minimal_response=true',
    'mock': ''
  },
  'hosts.by_component.one': {
    'real': '/clusters/{clusterName}/hosts?host_components/HostRoles/component_name.in({componentNames})&fields=host_components,Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/ip,Hosts/os_type,Hosts/os_arch,Hosts/public_host_name&page_size=1&minimal_response=true',
    'mock': ''
  },
  'hosts.by_component.all': {
    'real': '/clusters/{clusterName}/hosts?host_components/HostRoles/component_name.in({componentNames})&fields=host_components,Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/ip,Hosts/os_type,Hosts/os_arch,Hosts/public_host_name&minimal_response=true',
    'mock': ''
  },
  'hosts.config_groups': {
    'real': '/clusters/{clusterName}/hosts?fields=Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/ip,Hosts/os_type,Hosts/os_arch,Hosts/public_host_name,host_components&minimal_response=true',
    'mock': ''
  },
  'hosts.info.install': {
    'real': '/hosts?Hosts/host_name.in({hostNames})&fields=Hosts/cpu_count,Hosts/disk_info,Hosts/total_mem,Hosts/ip,Hosts/os_type,Hosts/os_arch,Hosts/public_host_name&minimal_response=true',
    'mock': ''
  },
  'hosts.ips': {
    'real': '/hosts?Hosts/host_name.in({hostNames})&fields=Hosts/ip',
    'mock': ''
  },
  'hosts.host_components.pre_load': {
    real: '',
    mock: '/data/hosts/HDP2/hosts.json',
    format: function format(data) {
      return {
        url: data.url
      };
    }
  },
  'hosts.metrics.lazy_load': {
    real: '',
    mock: '/data/hosts/HDP2/hosts.json',
    format: function format(data) {
      return {
        url: data.url,
        headers: {
          'X-Http-Method-Override': 'GET'
        },
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": { "query": data.parameters }
        })
      };
    }
  },
  'hosts.bulk.operations': {
    real: '/clusters/{clusterName}/hosts?fields=Hosts/host_name,Hosts/host_state,Hosts/maintenance_state,' + 'host_components/HostRoles/state,host_components/HostRoles/maintenance_state,' + 'Hosts/total_mem,stack_versions/HostStackVersions,stack_versions/repository_versions/RepositoryVersions/repository_version,' + 'stack_versions/repository_versions/RepositoryVersions/id,' + 'host_components/HostRoles/stale_configs' + 'host_components/HostRoles/service_name&minimal_response=true',
    mock: '',
    format: function format(data) {
      return {
        headers: {
          'X-Http-Method-Override': 'GET'
        },
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": { "query": data.parameters }
        })
      };
    }
  },

  'logtail.get': {
    'real': '/clusters/{clusterName}/logging/searchEngine?component_name={logComponentName}&host_name={hostName}&pageSize={pageSize}&startIndex={startIndex}',
    'mock': ''
  },

  'service.serviceConfigVersions.get': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?service_name={serviceName}&fields=service_config_version,user,hosts,group_id,group_name,is_current,createtime,service_name,service_config_version_note,stack_id,is_cluster_compatible&sortBy=service_config_version.desc&minimal_response=true',
    mock: '/data/configurations/service_versions.json'
  },
  'service.serviceConfigVersions.get.current': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?service_name.in({serviceNames})&is_current=true&fields=*',
    mock: '/data/configurations/service_versions.json'
  },
  'service.serviceConfigVersions.get.current.not.default': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?is_current=true&group_id>0&fields=*',
    mock: '/data/configurations/service_versions.json'
  },
  'service.serviceConfigVersions.get.total': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?page_size=1&minimal_response=true',
    mock: '/data/configurations/service_versions_total.json'
  },
  'service.serviceConfigVersion.get': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?service_name={serviceName}&service_config_version={serviceConfigVersion}',
    mock: '/data/configurations/service_version.json'
  },
  'service.serviceConfigVersions.get.multiple': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?(service_name={serviceName}&service_config_version.in({serviceConfigVersions})){additionalParams}',
    mock: '/data/configurations/service_version.json',
    format: function format(data) {
      return {
        serviceConfigVersions: data.serviceConfigVersions.join(','),
        additionalParams: data.additionalParams || ''
      };
    }
  },
  'service.serviceConfigVersions.get.suggestions': {
    real: '/clusters/{clusterName}/configurations/service_config_versions?fields={key}&minimal_response=true',
    mock: ''
  },
  'service.serviceConfigVersion.revert': {
    'real': '/clusters/{clusterName}',
    'mock': '',
    'type': 'PUT',
    'format': function format(data) {
      return {
        data: JSON.stringify(data.data)
      };
    }
  },

  'service.mysql.clean': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "CLEAN", "context": "Clean MYSQL Server"
          },
          "Requests/resource_filters": [{
            "service_name": "HIVE",
            "component_name": "MYSQL_SERVER",
            "hosts": data.host
          }]
        })
      };
    }
  },

  'service.mysql.configure': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "CONFIGURE", "context": "Configure MYSQL Server"
          },
          "Requests/resource_filters": [{
            "service_name": "HIVE",
            "component_name": "MYSQL_SERVER",
            "hosts": data.host
          }]
        })
      };
    }
  },

  'service.mysql.testHiveConnection': {
    'real': '/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "context": "Check host",
            "action": "check_host",
            "parameters": {
              "db_name": data.db_name,
              "user_name": data.db_user,
              "user_passwd": data.db_pass,
              "db_connection_url": data.db_connection_url,
              "jdk_location": data.jdk_location,
              "threshold": "60",
              "ambari_server_host": "c6403.ambari.apache.org",
              "check_execute_list": "db_connection_check",
              "java_home": data.java_home,
              "jdk_name": data.jdk_name
            }
          },
          "Requests/resource_filters": [{ "hosts": data.hosts }] })
      };
    }
  },

  'widgets.get': {
    real: '/clusters/{clusterName}/widgets?{urlParams}',
    mock: '/data/widget_layouts/{sectionName}_WIDGETS.json'
  },

  'widgets.all.shared.get': {
    real: '/clusters/{clusterName}/widgets?WidgetInfo/scope=CLUSTER&fields=*',
    mock: '/data/widget_layouts/all_shared_widgets.json'
  },

  'widgets.all.mine.get': {
    real: '/clusters/{clusterName}/widgets?WidgetInfo/scope=USER&WidgetInfo/author={loginName}&fields=*',
    mock: '/data/widget_layouts/all_mine_widgets.json'
  },

  'widgets.layout.stackDefined.get': {
    real: '{stackVersionURL}/services/{serviceName}/artifacts/widget_descriptor',
    mock: '/data/widget_layouts/HBASE/stack_layout.json'
  },

  'widget.layout.id.get': {
    real: '/clusters/{clusterName}/widget_layouts/{layoutId}',
    mock: '/data/widget_layouts/{serviceName}/default_dashboard.json'
  },

  'widget.layout.name.get': {
    real: '/clusters/{clusterName}/widget_layouts?WidgetLayoutInfo/layout_name={name}',
    mock: '/data/widget_layouts/{serviceName}/default_dashboard.json'
  },

  'widget.layout.delete': {
    real: '/clusters/{clusterName}/widget_layouts/{layoutId}',
    type: 'DELETE'
  },

  'widget.layout.get': {
    real: '/clusters/{clusterName}/widget_layouts?{urlParams}',
    mock: '/data/widget_layouts/{serviceName}/default_dashboard.json'
  },

  'widget.layout.edit': {
    real: '/clusters/{clusterName}/widget_layouts/{layoutId}',
    mock: '',
    format: function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },

  'widget.layout.create': {
    real: '/clusters/{clusterName}/widget_layouts',
    mock: '',
    format: function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },

  'widgets.layout.userDefined.get': {
    real: '/users/{loginName}/widget_layouts?section_name={sectionName}',
    mock: '/data/widget_layouts/HBASE/empty_user_layout.json'
  },

  'widgets.layouts.get': {
    real: '/users?widget_layouts/section_name={sectionName}&widget_layouts/scope=CLUSTER',
    mock: '/data/widget_layouts/HBASE/layouts.json'
  },

  'widgets.layouts.active.get': {
    real: '/users/{userName}/activeWidgetLayouts?{urlParams}',
    mock: '/data/widget_layouts/{sectionName}.json'
  },

  'widgets.layouts.all.active.get': {
    real: '/users/{userName}/activeWidgetLayouts',
    mock: ''
  },

  'widget.activelayouts.edit': {
    real: '/users/{userName}/activeWidgetLayouts/',
    mock: '',
    format: function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },

  'widget.action.delete': {
    real: '/clusters/{clusterName}/widgets/{id}',
    mock: '',
    format: function format(data) {
      return {
        type: 'DELETE'
      };
    }
  },

  'widgets.serviceComponent.metrics.get': {
    real: '/clusters/{clusterName}/services/{serviceName}/components/{componentName}?fields={metricPaths}&format=null_padding',
    mock: '/data/metrics/{serviceName}/Append_num_ops_&_Delete_num_ops.json'
  },

  'widgets.hostComponent.metrics.get': {
    real: '/clusters/{clusterName}/host_components?HostRoles/component_name={componentName}{hostComponentCriteria}&fields={metricPaths}&format=null_padding{selectedHostsParam}',
    mock: '/data/metrics/{serviceName}/Append_num_ops.json'
  },

  'widgets.hosts.metrics.get': {
    real: '/clusters/{clusterName}/hosts?fields={metricPaths}',
    mock: '/data/metrics/{serviceName}/Append_num_ops.json'
  },

  'widgets.wizard.metrics.get': {
    real: '{stackVersionURL}/services?artifacts/Artifacts/artifact_name=metrics_descriptor&StackServices/service_name.in({serviceNames})&fields=artifacts/*',
    mock: '/data/metrics/HBASE/definition.json'
  },

  'widgets.wizard.add': {
    real: '/clusters/{clusterName}/widgets/',
    mock: '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify(data.data)
      };
    }
  },

  'widgets.wizard.edit': {
    real: '/clusters/{clusterName}/widgets/{widgetId}',
    mock: '',
    'format': function format(data) {
      return {
        type: 'PUT',
        data: JSON.stringify(data.data)
      };
    }
  },

  'service.components.load': {
    real: '/clusters/{clusterName}/services?fields=components&minimal_response=true',
    mock: '/data/services/components.json'
  },

  'nameNode.federation.formatNameNode': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "FORMAT", "context": "Format NameNode"
          },
          "Requests/resource_filters": [{
            "service_name": "HDFS",
            "component_name": "NAMENODE",
            "hosts": data.host
          }]
        })
      };
    }
  },

  'nameNode.federation.formatZKFC': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "FORMAT", "context": "Format ZKFC"
          },
          "Requests/resource_filters": [{
            "service_name": "HDFS",
            "component_name": "ZKFC",
            "hosts": data.host
          }]
        })
      };
    }
  },
  'nameNode.federation.bootstrapNameNode': {
    'real': '/clusters/{clusterName}/requests',
    'mock': '',
    'format': function format(data) {
      return {
        type: 'POST',
        data: JSON.stringify({
          "RequestInfo": {
            "command": "BOOTSTRAP_STANDBY", "context": "Bootstrap NameNode"
          },
          "Requests/resource_filters": [{
            "service_name": "HDFS",
            "component_name": "NAMENODE",
            "hosts": data.host
          }]
        })
      };
    }
  },
  'hiveServerInteractive.getStatus': {
    real: '',
    mock: '',
    format: function format(data) {
      return {
        url: data.proto + '://' + data.hsiHost + ':' + data.port + '/leader'
      };
    }
  }
};
/**
 * Replace data-placeholders to its values
 *
 * @param {String} url
 * @param {Object} data
 * @return {String}
 */
var formatUrl = function formatUrl(url, data) {
  if (!url) return null;
  var keys = url.match(/\{\w+\}/g);
  keys = keys === null ? [] : keys;
  if (keys) {
    keys.forEach(function (key) {
      var raw_key = key.substr(1, key.length - 2);
      var replace;
      if (!data || !data[raw_key]) {
        replace = '';
      } else {
        replace = data[raw_key];
      }
      url = url.replace(new RegExp(key, 'g'), replace);
    });
  }
  return url;
};

/**
 * this = object from config
 * @return {Object}
 */
var formatRequest = function formatRequest(data) {
  var opt = {
    type: this.type || 'GET',
    timeout: App.timeout,
    dataType: 'json',
    headers: {}
  };
  if (App.get('testMode')) {
    opt.url = formatUrl(this.mock ? this.mock : '', data);
    opt.type = 'GET';
  } else {
    var prefix = this.apiPrefix != null ? this.apiPrefix : App.apiPrefix;
    opt.url = prefix + formatUrl(this.real, data);
  }

  if (this.format) {
    jQuery.extend(opt, this.format(data, opt));
  }
  if (!('headers' in opt && 'Content-Type' in opt.headers) && opt.contentType === undefined) {
    // With the default www-url-form-encoded Content-Type KNOX would corrupt the json content.
    opt.headers['Content-Type'] = 'text/plain';
  }
  var statusCode = jQuery.extend({}, require('data/statusCodes'));
  statusCode['404'] = function () {
    console.log("Error code 404: URI not found. -> " + opt.url);
  };
  opt.statusCode = statusCode;
  return opt;
};

/**
 * transform GET to POST call
 * @param {object} opt
 * @returns {object} opt
 */
var doGetAsPost = function doGetAsPost(opt) {
  var delimiterPos = opt.url.indexOf('?');
  var fieldsIndex = opt.url.indexOf('&fields');

  opt.type = "POST";
  opt.headers["X-Http-Method-Override"] = "GET";
  if (delimiterPos !== -1) {
    var query = fieldsIndex !== -1 ? opt.url.substring(delimiterPos + 1, fieldsIndex) : opt.url.substr(delimiterPos + 1);
    opt.data = JSON.stringify({
      "RequestInfo": { "query": query }
    });
    if (fieldsIndex !== -1) {
      opt.url = opt.url.substr(0, delimiterPos) + '?' + opt.url.substr(fieldsIndex + 1) + '&_=' + App.dateTime();
    } else {
      opt.url = opt.url.substr(0, delimiterPos) + '?_=' + App.dateTime();
    }
  } else {
    opt.url += '?_=' + App.dateTime();
  }
  return opt;
};

/**
 * Wrapper for all ajax requests
 *
 * @type {Object}
 */
var ajax = Em.Object.extend({
  /**
   * max number of symbols in URL of GET call
   * @const
   * @type {number}
   */
  MAX_GET_URL_LENGTH: 2048,

  consoleMsg: function consoleMsg(name, url) {
    return Em.I18n.t('app.logger.ajax').format(name, url ? url.substr(7, 100) : '');
  },

  /**
   * Send ajax request
   *
   * @param {Object} config
   * @return {$.ajax} jquery ajax object
   *
   * config fields:
   *  name - url-key in the urls-object *required*
   *  sender - object that send request (need for proper callback initialization) *required*
   *  data - object with data for url-format
   *  beforeSend - method-name for ajax beforeSend response callback
   *  success - method-name for ajax success response callback
   *  error - method-name for ajax error response callback
   *  callback - callback from <code>App.updater.run</code> library
   */
  send: function send(config) {

    if (!config.sender) {
      console.warn('Ajax sender should be defined!');
      return null;
    }

    var loadingPopup = null;
    var loadingPopupTimeout = null;
    if (config.hasOwnProperty("showLoadingPopup") && config.showLoadingPopup === true) {
      loadingPopupTimeout = setTimeout(function () {
        loadingPopup = App.ModalPopup.show({
          header: Em.I18n.t('common.loading.eclipses'),
          backdrop: false,
          primary: false,
          secondary: false,
          bodyClass: Em.View.extend({
            template: Em.Handlebars.compile('{{view App.SpinnerView}}')
          })
        });
      }, 500);
    }

    // default parameters
    var params = {
      clusterName: App.get('clusterName') || App.clusterStatus.get('clusterName')
    };

    // extend default parameters with provided
    if (config.hasOwnProperty("data") && config.data) {
      jQuery.extend(params, config.data);
    }

    var opt = {};
    if (!urls[config.name]) {
      console.warn('Invalid name provided `' + config.name + '`!');
      return null;
    }
    opt = formatRequest.call(urls[config.name], params);

    var consoleMsg = this.consoleMsg(config.name, opt.url);

    App.logger.setTimer(consoleMsg);

    if (opt.url && opt.url.length > this.get('MAX_GET_URL_LENGTH')) {
      opt = doGetAsPost(opt);
    }

    opt.context = this;

    // object sender should be provided for processing beforeSend, success and error responses
    opt.beforeSend = function (xhr) {
      if (config.beforeSend) {
        config.sender[config.beforeSend](opt, xhr, params);
      }
    };
    opt.success = function (data, textStatus, request) {
      if (config.success) {
        config.sender[config.success](data, opt, params, request);
      }
    };
    opt.error = function (request, ajaxOptions, error) {
      var KDCErrorMsg = this.getKDCErrorMgs(request);
      if (!Em.isNone(KDCErrorMsg)) {
        this.defaultErrorKDCHandler(opt, KDCErrorMsg);
      } else if (config.error) {
        config.sender[config.error](request, ajaxOptions, error, opt, params);
      } else {
        this.defaultErrorHandler(request, opt.url, opt.type);
      }
    };
    opt.complete = function () {
      if (loadingPopupTimeout) {
        clearTimeout(loadingPopupTimeout);
      }
      if (loadingPopup) {
        Em.tryInvoke(loadingPopup, 'hide');
      }
      App.logger.logTimerIfMoreThan(consoleMsg, 1000);
      if (config.callback) {
        config.callback();
      }
    };

    /**
     * run this handler when click cancle on KDC error popup
     */
    if (config.kdcCancelHandler) {
      opt.kdcCancelHandler = config.kdcCancelHandler;
    }

    if ($.mocho) {
      opt.url = 'http://' + $.hostName + opt.url;
    }
    return $.ajax(opt);
  },

  // A single instance of App.ModalPopup view
  modalPopup: null,

  /**
   * Upon error with one of these statuses modal should be displayed
   * @type {Array}
   */
  statuses: [500, 401, 407, 413],

  /**
   * defaultErrorHandler function is referred from App.ajax.send function and App.HttpClient.defaultErrorHandler function
   * @jqXHR {jqXHR Object}
   * @url {string}
   * @method {String} Http method
   * @showStatus {number} HTTP response code which should be shown. Default is 500.
   */
  defaultErrorHandler: function defaultErrorHandler(jqXHR, url, method, showStatus) {
    method = method || 'GET';
    var self = this;
    showStatus = Em.isNone(showStatus) ? this.get('statuses') : [showStatus];
    try {
      var json = $.parseJSON(jqXHR.responseText);
      var message = json.message;
    } catch (err) {}

    if (showStatus.contains(jqXHR.status) && !this.get('modalPopup')) {
      this.set('modalPopup', App.ModalPopup.show({
        elementId: 'default-error-modal',
        header: Em.I18n.t('common.error'),
        secondary: false,
        onPrimary: function onPrimary() {
          this.hide();
          self.set('modalPopup', null);
        },
        bodyClass: App.AjaxDefaultErrorPopupBodyView.extend({
          type: method,
          url: url,
          status: jqXHR.status,
          message: message
        })
      }));
    }
  },

  /**
   * defines if it's admin session expiration error
   * and if so returns short error message
   * @param jqXHR
   * @returns {string|null}
   */
  getKDCErrorMgs: function getKDCErrorMgs(jqXHR) {
    try {
      var message = $.parseJSON(jqXHR.responseText).message;
    } catch (err) {}
    if (jqXHR.status === 400 && message) {
      return App.format.kdcErrorMsg(message, true);
    }
  },

  /**
   * default handler for admin session expiration error
   * @param {object} opt
   * @param {string} msg
   * @returns {*}
   */
  defaultErrorKDCHandler: function defaultErrorKDCHandler(opt, msg) {
    return App.showInvalidKDCPopup(opt, msg);
  },

  /**
   * Abort all requests stored in the certain array
   * @param requestsArray
   */
  abortRequests: function abortRequests(requestsArray) {
    requestsArray.forEach(function (xhr) {
      xhr.isForcedAbort = true;
      Em.tryInvoke(xhr, 'abort');
    });
    requestsArray.clear();
  }

});

/**
 * Add few access-methods for test purposes
 */
if ($.mocho) {
  ajax.reopen({
    /**
     * Don't use it anywhere except tests!
     * @returns {Array}
     */
    fakeGetUrlNames: function fakeGetUrlNames() {
      return Em.keys(urls);
    },

    /**
     * Don't use it anywhere except tests!
     * @param name
     * @returns {*}
     */
    fakeGetUrl: function fakeGetUrl(name) {
      return urls[name];
    },

    /**
     * Don't use it anywhere except tests!
     * @param url
     * @param data
     * @returns {String}
     */
    fakeFormatUrl: function fakeFormatUrl(url, data) {
      return formatUrl(url, data);
    },

    /**
     * Don't use it anywhere except tests!
     * @param urlObj
     * @param data
     * @returns {Object}
     */
    fakeFormatRequest: function fakeFormatRequest(urlObj, data) {
      return formatRequest.call(urlObj, data);
    },

    /**
     * Don't use it anywhere except tests!
     * @param urlObj
     * @param data
     * @returns {Object}
     */
    fakeDoGetAsPost: function fakeDoGetAsPost(urlObj, data) {
      return doGetAsPost.call(urlObj, data);
    }
  });
}

App.ajax = ajax.create({});

});

require.register("utils/ajax/ajax_queue", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Simple util for executing ajax-requests queue
 *
 * @uses App.ajax
 * @type {Em.Object}
 *
 * Don't add requests directly to <code>queue</code>!
 * Bad example:
 * <code>
 *  var q = App.ajax.Queue.create();
 *  q.get('queue').pushObject({...}); // Don't do this!!!!
 * </code>
 *
 * Don't call <code>runNextRequest</code> directly!
 * Bad example:
 * <code>
 *  var q = App.ajax.Queue.create();
 *  q.runNextRequest(); // Don't do this!!!!
 * </code>
 *
 * Usage example 1:
 * <code>
 *  var q = App.ajax.Queue.create();
 *  q.addRequest({
 *    name: 'some_request',
 *    sender: this,
 *    success: 'someFunc'
 *  }).addRequest({
 *    name: 'some_request2',
 *    sender: this,
 *    success: 'someFunc2'
 *  }).start();
 * </code>
 *
 * Usage example 2:
 * <code>
 *  var q = App.ajax.Queue.create();
 *  q.addRequest({
 *    name: 'some_request',
 *    sender: this,
 *    success: 'someFunc'
 *  });
 *  q.addRequest({
 *    name: 'some_request2',
 *    sender: this,
 *    success: 'someFunc2'
 *  });
 *  q.start();
 * </code>
 *
 * Usage example 3:
 * <code>
 *  var q = App.ajax.Queue.create();
 *  q.addRequests(Em.A([{
 *    name: 'some_request',
 *    sender: this,
 *    success: 'someFunc'
 *  },
 *  {
 *    name: 'some_request2',
 *    sender: this,
 *    success: 'someFunc2'
 *  }]));
 *  q.start();
 * </code>
 */
App.ajaxQueue = Em.Object.extend({

  /**
   * List of requests
   * @type {Object[]}
   */
  queue: Em.A([]),

  /**
   * About query executing if some request failed
   * @type {bool}
   */
  abortOnError: true,

  /**
   * Function called after queue is complete
   * @type {callback}
   */
  finishedCallback: Em.K,

  /**
   * Add request to the <code>queue</code>
   * @param {Object} request object that uses in <code>App.ajax.send</code>
   * @return {App.ajaxQueue}
   * @method addRequest
   */
  addRequest: function addRequest(request) {
    Em.assert('Each ajax-request should has non-blank `name`', !Em.isBlank(Em.get(request, 'name')));
    Em.assert('Each ajax-request should has object `sender`', Em.typeOf(Em.get(request, 'sender')) !== 'object');
    this.get('queue').pushObject(request);
    return this;
  },

  /**
   * Add requests to the <code>queue</code>
   * @param {Object[]} requests list of objects that uses in <code>App.ajax.send</code>
   * @return {App.ajaxQueue}
   * @method addRequests
   */
  addRequests: function addRequests(requests) {
    requests.map(function (request) {
      this.addRequest(request);
    }, this);
    return this;
  },

  /**
   * Enter point to start requests executing
   * @method start
   */
  start: function start() {
    this.runNextRequest();
  },

  /**
   * Execute first request from the <code>queue</code>
   * @method runNextRequest
   */
  runNextRequest: function runNextRequest() {
    var queue = this.get('queue');
    if (queue.length === 0) {
      this.finishedCallback();
      return;
    }
    var r = App.ajax.send(queue.shift());
    this.propertyDidChange('queue');
    if (r) {
      r.complete(this._complete.bind(this));
    } else {
      if (this.get('abortOnError')) {
        this.clear();
      } else {
        this.runNextRequest();
      }
    }
  },

  _complete: function _complete(xhr) {
    if (xhr.status >= 200 && xhr.status <= 299) {
      this.runNextRequest();
    } else {
      if (this.get('abortOnError')) {
        this.clear();
      } else {
        this.runNextRequest();
      }
    }
  },

  /**
   * Remove all requests from <code>queue</code>
   * @method clear
   */
  clear: function clear() {
    this.get('queue').clear();
  }

});

});

require.register("utils/array_utils", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

function _parseId(id) {
  return id.replace(/[^\d|\.]/g, '').split('.').map(function (i) {
    return parseInt(i, 10);
  });
}

var flatten = function flatten(list) {
  return list.reduce(function (a, b) {
    return a.concat(Array.isArray(b) ? flatten(b) : b);
  }, []);
};

module.exports = {
  /**
   *
   * @param arr {Array}
   * @param id
   * @return result {Array}
   */
  uniqObjectsbyId: function uniqObjectsbyId(arr, id) {
    var result = [];
    arr.forEach(function (item) {
      var isIdPresent = result.someProperty(id, item[id]);
      if (!isIdPresent) {
        result.pushObject(item);
      }
    });
    return result;
  },

  /**
   * intersect two arrays and return the common values
   *
   * @param  {Array} arr1 - first array
   * @param  {Array} arr2 - second array
   * @return {Array} intersection - the intersection of arr1 and arr2
   */
  intersect: function intersect(arr1, arr2) {
    var intersection = [];
    var shortest = arr1.length >= arr2.length ? arr2 : arr1;
    var longest = arr1.length >= arr2.length ? arr1 : arr2;

    shortest.forEach(function (entry) {
      longest.indexOf(entry) > -1 ? intersection.push(entry) : null;
    });

    return intersection;
  },

  /**
   * Callback for sorting models with `id`-property equal to something like version number: 'HDP-1.2.3', '4.2.52' etc
   *
   * @param {{id: string}} obj1
   * @param {{id: string}} obj2
   * @returns {number}
   */
  sortByIdAsVersion: function sortByIdAsVersion(obj1, obj2) {
    var id1 = _parseId(Em.get(obj1, 'id'));
    var id2 = _parseId(Em.get(obj2, 'id'));
    var lId1 = id1.length;
    var lId2 = id2.length;
    var limit = lId1 > lId2 ? lId2 : lId1;
    for (var i = 0; i < limit; i++) {
      if (id1[i] > id2[i]) {
        return 1;
      }
      if (id1[i] < id2[i]) {
        return -1;
      }
    }
    if (lId1 === lId2) {
      return 0;
    }
    return lId1 > lId2 ? 1 : -1;
  },

  flatten: flatten

};

});

require.register("utils/base64", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function () {

  var object = typeof window != 'undefined' ? window : exports,
      chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=',
      INVALID_CHARACTER_ERR = function () {
    // fabricate a suitable error object
    try {
      document.createElement('$');
    } catch (error) {
      return error;
    }
  }();

  // encoder
  // [https://gist.github.com/999166] by [https://github.com/nignag]
  object.btoa || (object.btoa = function (input) {
    for (
    // initialize result and counter
    var block, charCode, idx = 0, map = chars, output = '';
    // if the next input index does not exist:
    // change the mapping table to "="
    // check if d has no fractional digits
    input.charAt(idx | 0) || (map = '=', idx % 1);
    // "8 - idx % 1 * 8" generates the sequence 2, 4, 6, 8
    output += map.charAt(63 & block >> 8 - idx % 1 * 8)) {
      charCode = input.charCodeAt(idx += 3 / 4);
      if (charCode > 0xFF) throw INVALID_CHARACTER_ERR;
      block = block << 8 | charCode;
    }
    return output;
  });

  // decoder
  // [https://gist.github.com/1020396] by [https://github.com/atk]
  object.atob || (object.atob = function (input) {
    input = input.replace(/=+$/, '');
    if (input.length % 4 == 1) throw INVALID_CHARACTER_ERR;
    for (
    // initialize result and counters
    var bc = 0, bs, buffer, idx = 0, output = '';
    // get next character
    buffer = input.charAt(idx++);
    // character found in table? initialize bit storage and add its ascii value;
    ~buffer && (bs = bc % 4 ? bs * 64 + buffer : buffer,
    // and if not first of each 4 characters,
    // convert the first 8 bits to one ascii character
    bc++ % 4) ? output += String.fromCharCode(255 & bs >> (-2 * bc & 6)) : 0) {
      // try to find character in table (0-63, not found => -1)
      buffer = chars.indexOf(buffer);
    }
    return output;
  });
})();

});

require.register("utils/batch_scheduled_requests", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * Default success callback for ajax-requests in this module
 * @type {Function}
 */
var defaultSuccessCallback = function defaultSuccessCallback(data, ajaxOptions, params) {
  App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
    params.query && params.query.set('status', 'SUCCESS');
    if (initValue) {
      App.router.get('backgroundOperationsController').showPopup();
    }
  });
};
/**
 * Default error callback for ajax-requests in this module
 * @param {Object} xhr
 * @param {String} textStatus
 * @param {String} error
 * @param {Object} opt
 * @param {Object} params
 * @type {Function}
 */
var defaultErrorCallback = function defaultErrorCallback(xhr, textStatus, error, opt, params) {
  params.query && params.query.set('status', 'FAIL');
  App.ajax.defaultErrorHandler(xhr, opt.url, 'POST', xhr.status);
};

/**
 * Contains helpful utilities for handling batch and scheduled requests.
 */
module.exports = {

  /**
   * Some services have components which have a need for rolling restarts. This
   * method returns the name of the host-component which supports rolling
   * restarts for a service.
   * @param {String} serviceName
   */
  getRollingRestartComponentName: function getRollingRestartComponentName(serviceName) {
    var rollingRestartComponents = {
      HDFS: 'DATANODE',
      YARN: 'NODEMANAGER',
      HBASE: 'HBASE_REGIONSERVER',
      STORM: 'SUPERVISOR'
    };
    return rollingRestartComponents[serviceName] ? rollingRestartComponents[serviceName] : null;
  },

  /**
   * Facade-function for restarting host components of specific service
   * @param {String} serviceDisplayName for which service hostComponents should be restarted
   * @param {String} serviceName for which service hostComponents should be restarted
   * @param {bool} staleConfigsOnly restart only hostComponents with <code>staleConfig</code> true
   * @param {Object} query
   * @param {bool} runMmOperation
   */
  restartAllServiceHostComponents: function restartAllServiceHostComponents(serviceDisplayName, serviceName, staleConfigsOnly, query, runMmOperation) {
    var _this = this;

    var context = staleConfigsOnly ? Em.I18n.t('rollingrestart.context.allWithStaleConfigsForSelectedService').format(serviceDisplayName) : Em.I18n.t('rollingrestart.context.allForSelectedService').format(serviceDisplayName);

    if (runMmOperation) {
      this.turnOnOffPassiveRequest('ON', Em.I18n.t('passiveState.turnOnFor').format(serviceName), serviceName);
    }
    this.getComponentsFromServer({
      services: [serviceName],
      staleConfigs: staleConfigsOnly ? staleConfigsOnly : null,
      passiveState: 'OFF',
      displayParams: ['host_components/HostRoles/component_name']
    }, function (data) {
      var hostComponents = _this.getRestartComponentsArray(data);
      _this.restartHostComponents(hostComponents, context, 'SERVICE', query);
    });
  },

  restartCertainServiceHostComponents: function restartCertainServiceHostComponents(serviceName, components, hosts, label, query, runMmOperation) {
    var _this2 = this;

    var context = Em.I18n.t('rollingrestart.context.allForSelectedService').format(label);
    if (runMmOperation) {
      this.turnOnOffPassiveRequest('ON', Em.I18n.t('passiveState.turnOnFor').format(serviceName), serviceName);
    }
    this.getComponentsFromServer({
      services: serviceName && [serviceName],
      hosts: hosts,
      components: components,
      passiveState: 'OFF',
      displayParams: ['host_components/HostRoles/component_name']
    }, function (data) {
      var hostComponents = _this2.getRestartComponentsArray(data);
      _this2.restartHostComponents(hostComponents, context, 'SERVICE', query);
    });
  },

  getRestartComponentsArray: function getRestartComponentsArray(data) {
    var hostComponents = [];
    data.items.forEach(function (host) {
      host.host_components.forEach(function (hostComponent) {
        hostComponents.push(Em.Object.create({
          componentName: hostComponent.HostRoles.component_name,
          hostName: host.Hosts.host_name
        }));
      });
    });
    return hostComponents;
  },

  /**
   * construct URL from parameters for request in <code>getComponentsFromServer()</code>
   * @param {object} options
   * @return {{fields: string, params: string}}
   */
  constructComponentsCallUrl: function constructComponentsCallUrl(options) {
    var multipleValueParams = {
      'services': 'host_components/HostRoles/service_name.in(<entity-names>)',
      'hosts': 'Hosts/host_name.in(<entity-names>)',
      'components': 'host_components/HostRoles/component_name.in(<entity-names>)'
    },
        singleValueParams = {
      staleConfigs: 'host_components/HostRoles/stale_configs=',
      passiveState: 'Hosts/maintenance_state=',
      workStatus: 'host_components/HostRoles/state='
    },
        displayParams = options.displayParams || [],
        urlParams = '',
        addAmpersand = false;

    for (var i in multipleValueParams) {
      var arrayParams = options[i];
      if (Array.isArray(arrayParams) && arrayParams.length > 0) {
        if (addAmpersand) {
          urlParams += '&';
          addAmpersand = false;
        }
        urlParams += multipleValueParams[i].replace('<entity-names>', arrayParams.join(','));
        addAmpersand = true;
      }
    }

    for (var j in singleValueParams) {
      var param = options[j];
      if (!Em.isNone(param)) {
        urlParams += addAmpersand ? '&' : '';
        urlParams += singleValueParams[j] + param.toString();
        addAmpersand = true;
      }
    }
    var params = urlParams,
        fields = '';
    displayParams.forEach(function (displayParam, index, array) {
      if (index === 0) {
        fields += addAmpersand ? '&' : '';
        fields += 'fields=';
      }
      fields += displayParam;
      fields += array.length === index + 1 ? '' : ",";
    });
    fields += '&minimal_response=true';

    return { fields: fields.substring(1, fields.length), params: params };
  },

  /**
   * make GET call to server in order to obtain host-components
   * which correspond to filter params from <code>options</code>
   * @param options
   * @param callback
   */
  getComponentsFromServer: function getComponentsFromServer(options, callback) {
    var request_parameters = this.constructComponentsCallUrl(options);
    return App.ajax.send({
      name: 'host.host_components.filtered',
      sender: this,
      data: {
        parameters: request_parameters.params,
        fields: request_parameters.fields,
        callback: callback
      },
      success: 'getComponentsFromServerSuccessCallback'
    });
  },

  /**
   * pass request outcome to <code>callback()<code>
   * @param data
   * @param opt
   * @param params
   */
  getComponentsFromServerSuccessCallback: function getComponentsFromServerSuccessCallback(data, opt, params) {
    params.callback(data);
  },

  /**
   * Restart list of host components
   * @param {Ember.Enumerable} hostComponentsList list of host components should be restarted
   * @param {String} context message to show in BG popup
   * @param {String} level - operation level, can be ("CLUSTER", "SERVICE", "HOST", "HOSTCOMPONENT")
   * @param {String} query
   */
  restartHostComponents: function restartHostComponents(hostComponentsList, context, level, query) {
    context = context || Em.I18n.t('rollingrestart.context.default');
    /**
     * Format: {
     *  'DATANODE': ['host1', 'host2'],
     *  'NAMENODE': ['host1', 'host3']
     *  ...
     * }
     */
    var componentToHostsMap = {};
    var hosts = [];
    hostComponentsList.forEach(function (hc) {
      var hostName = hc.get('hostName');
      var componentName = hc.get('componentName');
      if (!componentToHostsMap[componentName]) {
        componentToHostsMap[componentName] = [];
      }
      componentToHostsMap[componentName].push(hostName);
      hosts.push(hostName);
    });
    var resource_filters = [];
    for (var componentName in componentToHostsMap) {
      if (componentToHostsMap.hasOwnProperty(componentName)) {
        resource_filters.push({
          service_name: App.StackServiceComponent.find(componentName).get('serviceName'),
          component_name: componentName,
          hosts: componentToHostsMap[componentName].join(",")
        });
      }
    }
    if (hostComponentsList.length > 0) {
      var serviceComponentName = hostComponentsList[0].get("componentName");
      var serviceName = App.StackServiceComponent.find(serviceComponentName).get('serviceName');
      var operation_level = this.getOperationLevelObject(level, serviceName, serviceComponentName);
    }

    if (resource_filters.length) {
      App.ajax.send({
        name: 'restart.hostComponents',
        sender: {
          successCallback: defaultSuccessCallback,
          errorCallback: defaultErrorCallback
        },
        data: {
          context: context,
          resource_filters: resource_filters,
          query: query,
          operation_level: operation_level
        },
        success: 'successCallback',
        error: 'errorCallback',
        showLoadingPopup: true
      });
    }
  },

  /**
   * @param {String} level - operation level name, can be ("CLUSTER", "SERVICE", "HOST", "HOSTCOMPONENT")
   * @param {String} serviceName
   * @param {String} componentName
   * @returns {Object} {{level: *, cluster_name: *}} - operation level object
   * @method getOperationLevelObject - create operation level object to be included into ajax query
   */
  getOperationLevelObject: function getOperationLevelObject(level, serviceName, componentName) {
    var operationLevel = {
      "level": level,
      "cluster_name": App.get("clusterName")
    };
    if (level === "SERVICE") {
      operationLevel["service_name"] = serviceName;
    } else {
      if (level !== "HOST") {
        operationLevel["service_name"] = serviceName;
        operationLevel["hostcomponent_name"] = componentName;
      }
    }
    return operationLevel;
  },

  turnOnOffPassiveRequest: function turnOnOffPassiveRequest(state, message, serviceName, callback) {
    return App.ajax.send({
      'name': 'common.service.passive',
      'sender': {
        'successCallback': callback || defaultSuccessCallback,
        'errorCallback': defaultErrorCallback
      },
      'data': {
        'requestInfo': message,
        'serviceName': serviceName,
        'passive_state': state
      },
      'success': 'successCallback'
    });
  },

  /**
   * Makes a REST call to the server requesting the rolling restart of the
   * provided host components.
   * @param {Array} restartHostComponents list of host components should be restarted
   * @param {Number} batchSize size of each batch
   * @param {Number} intervalTimeSeconds delay between two batches
   * @param {Number} tolerateSize task failure tolerance
   * @param {callback} [successCallback]
   * @param {callback} [errorCallback]
   */
  _doPostBatchRollingRestartRequest: function _doPostBatchRollingRestartRequest(restartHostComponents, batchSize, intervalTimeSeconds, tolerateSize, successCallback, errorCallback) {
    successCallback = successCallback || defaultSuccessCallback;
    errorCallback = errorCallback || defaultErrorCallback;

    if (!restartHostComponents.length) {
      return;
    }
    return App.ajax.send({
      name: 'common.batch.request_schedules',
      sender: {
        successCallback: successCallback,
        errorCallback: errorCallback
      },
      data: {
        intervalTimeSeconds: intervalTimeSeconds,
        tolerateSize: tolerateSize,
        batches: this.getBatchesForRollingRestartRequest(restartHostComponents, batchSize)
      },
      success: 'successCallback',
      error: 'errorCallback',
      showLoadingPopup: true
    });
  },

  /**
   * Create list of batches for rolling restart request
   * @param {Array} restartHostComponents list host components should be restarted
   * @param {Number} batchSize size of each batch
   * @returns {Array} list of batches
   */
  getBatchesForRollingRestartRequest: function getBatchesForRollingRestartRequest(restartHostComponents, batchSize) {
    var hostIndex = 0,
        batches = [],
        batchCount = Math.ceil(restartHostComponents.length / batchSize),
        sampleHostComponent = restartHostComponents.objectAt(0),
        componentName = sampleHostComponent.get('componentName'),
        serviceName = sampleHostComponent.get('serviceName');

    for (var count = 0; count < batchCount; count++) {
      var hostNames = [];
      for (var hc = 0; hc < batchSize && hostIndex < restartHostComponents.length; hc++) {
        hostNames.push(restartHostComponents.objectAt(hostIndex++).get('hostName'));
      }
      if (hostNames.length) {
        batches.push({
          "order_id": count + 1,
          "type": "POST",
          "uri": "/clusters/" + App.get('clusterName') + "/requests",
          "RequestBodyInfo": {
            "RequestInfo": {
              "context": "_PARSE_.ROLLING-RESTART." + componentName + "." + (count + 1) + "." + batchCount,
              "command": "RESTART"
            },
            "Requests/resource_filters": [{
              "service_name": serviceName,
              "component_name": componentName,
              "hosts": hostNames.join(",")
            }]
          }
        });
      }
    }
    return batches;
  },

  /**
   * Launches dialog to handle rolling restarts of host components.
   *
   * Rolling restart is supported only for components listed in <code>getRollingRestartComponentName</code>
   * @see getRollingRestartComponentName
   * @param {String} hostComponentName
   *           Type of host-component to restart across cluster
   *          (ex: DATANODE)
   * @param {bool} staleConfigsOnly
   *           Pre-select host-components which have stale
   *          configurations
   */
  launchHostComponentRollingRestart: function launchHostComponentRollingRestart(hostComponentName, serviceName, isMaintenanceModeOn, staleConfigsOnly, skipMaintenance) {
    if (App.get('components.rollinRestartAllowed').contains(hostComponentName)) {
      return this.showRollingRestartPopup(hostComponentName, serviceName, isMaintenanceModeOn, staleConfigsOnly, null, skipMaintenance);
    }
    return this.showWarningRollingRestartPopup(hostComponentName);
  },

  /**
   * Show popup with rolling restart dialog
   * @param {String} hostComponentName name of the host components that should be restarted
   * @param {String} serviceName
   * @param {boolean} isMaintenanceModeOn
   * @param {bool} staleConfigsOnly restart only components with <code>staleConfigs</code> = true
   * @param {App.hostComponent[]} [hostComponents] list of hostComponents that should be restarted
   * @param {boolean} [skipMaintenance]
   * Using this parameter will reset hostComponentName
   */
  showRollingRestartPopup: function showRollingRestartPopup(hostComponentName, serviceName, isMaintenanceModeOn, staleConfigsOnly, hostComponents, skipMaintenance) {
    hostComponents = hostComponents || [];
    var componentDisplayName = App.format.role(hostComponentName, false);
    var self = this;
    if (!componentDisplayName) {
      componentDisplayName = hostComponentName;
    }
    var title = Em.I18n.t('rollingrestart.dialog.title').format(componentDisplayName);
    var viewExtend = {
      turnOnMmMsg: Em.I18n.t('passiveState.turnOnFor').format(serviceName),
      turnOnMm: false,
      staleConfigsOnly: staleConfigsOnly,
      hostComponentName: hostComponentName,
      skipMaintenance: skipMaintenance,
      serviceName: serviceName,
      isServiceInMM: isMaintenanceModeOn,
      didInsertElement: function didInsertElement() {
        var view = this;

        this.set('parentView.innerView', this);
        if (hostComponents.length) {
          view.initialize();
          return;
        }

        self.getComponentsFromServer({
          components: [hostComponentName],
          displayParams: ['host_components/HostRoles/stale_configs', 'Hosts/maintenance_state', 'host_components/HostRoles/maintenance_state'],
          staleConfigs: staleConfigsOnly ? staleConfigsOnly : null
        }, function (data) {
          var wrappedHostComponents = [];
          data.items.forEach(function (host) {
            host.host_components.forEach(function (hostComponent) {
              wrappedHostComponents.push(Em.Object.create({
                componentName: hostComponent.HostRoles.component_name,
                serviceName: App.StackServiceComponent.find(hostComponent.HostRoles.component_name).get('serviceName'),
                hostName: host.Hosts.host_name,
                staleConfigs: hostComponent.HostRoles.stale_configs,
                hostPassiveState: host.Hosts.maintenance_state,
                passiveState: hostComponent.HostRoles.maintenance_state
              }));
            });
          });
          view.set('allHostComponents', wrappedHostComponents);
          view.initialize();
        });
      }
    };
    if (hostComponents.length) {
      viewExtend.allHostComponents = hostComponents;
    }

    App.ModalPopup.show({
      header: title,
      hostComponentName: hostComponentName,
      serviceName: serviceName,
      isServiceInMM: isMaintenanceModeOn,
      staleConfigsOnly: staleConfigsOnly,
      skipMaintenance: skipMaintenance,
      innerView: null,
      bodyClass: App.RollingRestartView.extend(viewExtend),
      classNames: ['rolling-restart-popup'],
      primary: Em.I18n.t('rollingrestart.dialog.primary'),
      onPrimary: function onPrimary() {
        var dialog = this;
        var restartComponents = this.get('innerView.restartHostComponents');
        var batchSize = this.get('innerView.batchSize');
        var waitTime = this.get('innerView.interBatchWaitTimeSeconds');
        var tolerateSize = this.get('innerView.tolerateSize');
        if (this.get('innerView.turnOnMm')) {
          self.turnOnOffPassiveRequest('ON', Em.I18n.t('passiveState.turnOnFor').format(serviceName), serviceName.toUpperCase());
        }
        self._doPostBatchRollingRestartRequest(restartComponents, batchSize, waitTime, tolerateSize, function (data, ajaxOptions, params) {
          dialog.hide();
          defaultSuccessCallback(data, ajaxOptions, params);
        });
      },
      updateButtons: function () {
        var errors = this.get('innerView.errors');
        this.set('disablePrimary', errors != null && errors.length > 0);
      }.observes('innerView.errors')
    });
  },

  /**
   * Show warning popup about not supported host components
   * @param {String} hostComponentName
   */
  showWarningRollingRestartPopup: function showWarningRollingRestartPopup(hostComponentName) {
    var componentDisplayName = App.format.role(hostComponentName, false) || hostComponentName;
    var title = Em.I18n.t('rollingrestart.dialog.title').format(componentDisplayName);
    var msg = Em.I18n.t('rollingrestart.notsupported.hostComponent').format(componentDisplayName);

    return App.ModalPopup.show({
      header: title,
      secondary: false,
      msg: msg,
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile('<div class="alert alert-warning">{{msg}}</div>')
      })
    });
  },

  /**
   * Warn user that alerts will be updated in few minutes
   * @param {String} passiveState
   */
  infoPassiveState: function infoPassiveState(passiveState) {
    var enabled = passiveState == 'OFF' ? 'enabled' : 'suppressed';
    App.ModalPopup.show({
      header: Em.I18n.t('common.information'),
      secondary: null,
      bodyClass: Em.View.extend({
        template: Em.Handlebars.compile('<p>{{view.message}}</p>'),
        message: Em.I18n.t('hostPopup.warning.alertsTimeOut').format(passiveState.toLowerCase(), enabled)
      })
    });
  },

  /**
   * Retrieves the latest information about a specific request schedule
   * identified by 'requestScheduleId'
   *
   * @param {Number} requestScheduleId ID of the request schedule to get
   * @param {Function} successCallback Called with request_schedule data from server. An
   *          empty object returned for invalid ID.
   * @param {Function} errorCallback Optional error callback. Default behavior is to
   *          popup default error dialog.
   */
  getRequestSchedule: function getRequestSchedule(requestScheduleId, successCallback, errorCallback) {
    if (requestScheduleId != null && !isNaN(requestScheduleId) && requestScheduleId > -1) {
      errorCallback = errorCallback || defaultErrorCallback;
      App.ajax.send({
        name: 'request_schedule.get',
        sender: {
          successCallbackFunction: successCallback,
          errorCallbackFunction: errorCallback
        },
        data: {
          request_schedule_id: requestScheduleId
        },
        success: 'successCallbackFunction',
        error: 'errorCallbackFunction'
      });
    } else {
      successCallback({});
    }
  },

  /**
   * Attempts to abort a specific request schedule identified by 'requestScheduleId'
   *
   * @param {Number} requestScheduleId ID of the request schedule to get
   * @param {Function} successCallback Called when request schedule successfully aborted
   * @param {Function} errorCallback Optional error callback. Default behavior is to
   *          popup default error dialog.
   */
  doAbortRequestSchedule: function doAbortRequestSchedule(requestScheduleId, successCallback, errorCallback) {
    if (requestScheduleId != null && !isNaN(requestScheduleId) && requestScheduleId > -1) {
      errorCallback = errorCallback || defaultErrorCallback;
      App.ajax.send({
        name: 'common.delete.request_schedule',
        sender: {
          successCallbackFunction: successCallback,
          errorCallbackFunction: errorCallback
        },
        data: {
          request_schedule_id: requestScheduleId
        },
        success: 'successCallbackFunction',
        error: 'errorCallbackFunction'
      });
    } else {
      successCallback({});
    }
  }
};

});

require.register("utils/blueprint", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @typedef {object} BlueprintObject
 * @property {BlueprintMappings} blueprint
 * @property {BlueprintClusterBindings} blueprint_cluster_bindings
 */
/**
 * @typedef {object} BlueprintMappings
 * @property {BlueprintMappingsHostGroup[]} host_groups
 */
/**
 * @typedef {object[]} BlueprintMappingsHostGroup
 * @property {BlueprintHostGroupComponent[]} components
 * @property {string} name host group name
 */
/**
 * @typedef {object} BlueprintHostGroupComponent
 * @property {string} name component name
 */
/**
 * @typedef {object} BlueprintClusterBindings
 * @property {BlueprintClusterBindingsHostGroup[]} host_groups
 */
/**
 * @typedef {object} BlueprintClusterBindingsHostGroup
 * @property {BlueprintClusterBindingsHostGroupHosts[]} hosts
 * @property {string} name host group name
 */
/**
 * @typedef {object} BlueprintClusterBindingsHostGroupHosts
 * @property {string} fqdn host fqdn
 */
module.exports = {

  /**
   * @param {BlueprintObject} masterBlueprint
   * @param {BlueprintObject} slaveBlueprint
   * @return {BlueprintObject}
   */
  mergeBlueprints: function mergeBlueprints(masterBlueprint, slaveBlueprint) {
    console.time('mergeBlueprints');
    var self = this;

    // Check edge cases
    if (!slaveBlueprint && !masterBlueprint) {
      throw 'slaveBlueprint or masterBlueprint should not be empty';
    } else if (slaveBlueprint && !masterBlueprint) {
      return slaveBlueprint;
    } else if (!slaveBlueprint && masterBlueprint) {
      return masterBlueprint;
    }

    // Work with main case (both blueprint are presented)
    var matches = self.matchGroups(masterBlueprint, slaveBlueprint);

    var res = {
      blueprint: { host_groups: [] },
      blueprint_cluster_binding: { host_groups: [] }
    };

    var tmpObj = { hosts: [], components: [] };
    var masterBluePrintHostGroupsCluster = this.blueprintToObject(masterBlueprint, 'blueprint_cluster_binding.host_groups');
    var slaveBluePrintHostGroupsCluster = this.blueprintToObject(slaveBlueprint, 'blueprint_cluster_binding.host_groups');
    var masterBluePrintHostGroupsBlueprint = this.blueprintToObject(masterBlueprint, 'blueprint.host_groups');
    var slaveBluePrintHostGroupsBlueprint = this.blueprintToObject(slaveBlueprint, 'blueprint.host_groups');

    matches.forEach(function (match, i) {
      var group_name = 'host-group-' + (i + 1);

      var masterComponents = match.g1 ? Em.getWithDefault(masterBluePrintHostGroupsBlueprint, match.g1, tmpObj).components : [];
      var slaveComponents = match.g2 ? Em.getWithDefault(slaveBluePrintHostGroupsBlueprint, match.g2, tmpObj).components : [];

      res.blueprint.host_groups.push({
        name: group_name,
        components: masterComponents.concat(slaveComponents)
      });

      var hosts = match.g1 ? Em.getWithDefault(masterBluePrintHostGroupsCluster, match.g1, tmpObj).hosts : Em.getWithDefault(slaveBluePrintHostGroupsCluster, match.g2, tmpObj).hosts;

      res.blueprint_cluster_binding.host_groups.push({
        name: group_name,
        hosts: hosts
      });
    });
    console.timeEnd('mergeBlueprints');
    return res;
  },

  /**
   * Convert <code>blueprint</code>-object to the array with keys equal to the host-groups names
   * Used to improve performance when user try to search value in the blueprint using host-group name as search-field
   * Example:
   *  Before:
   *  <pre>
   *    // blueprint
   *    {
   *      blueprint: {
   *        host_groups: [
   *          {
   *            components: [{}, {}, ...],
   *            name: 'n1'
   *          },
   *          {
   *            components: [{}, {}, ...],
   *            name: 'n2'
   *          }
   *        ]
   *      },
   *      blueprint_cluster_binding: {
   *        host_groups: [
   *          {
   *            hosts: [{}, {}, ...],
   *            name: 'n1'
   *          },
   *          {
   *            hosts: [{}, {}, ...],
   *            name: 'n2'
   *          }
   *        ]
   *      }
   *    }
   *  </pre>
   *  Return:
   *  <pre>
   *    // field = 'blueprint_cluster_binding.host_groups'
   *    {
   *      n1: {
   *        hosts: [{}, {}, ...],
   *        name: 'n1'
   *      },
   *      n2: {
   *        hosts: [{}, {}, ...],
   *        name: 'n2'
   *      }
   *    }
   *
   *    // field = 'blueprint.host_groups'
   *    {
   *      n1: {
   *        components: [{}, {}, ...],
   *        name: 'n1'
   *      },
   *      n2: {
   *        components: [{}, {}, ...],
   *        name: 'n2'
   *      }
   *    }
   *  </pre>
   * @param {object} blueprint
   * @param {string} field
   * @returns {object}
   */
  blueprintToObject: function blueprintToObject(blueprint, field) {
    var valueToMap = Em.get(blueprint, field);
    if (!Array.isArray(valueToMap)) {
      return {};
    }
    return valueToMap.toMapByProperty('name');
  },

  matchGroups: function matchGroups(masterBlueprint, slaveBlueprint) {
    var self = this;
    var res = [];

    var groups1 = masterBlueprint.blueprint_cluster_binding.host_groups;
    var groups2 = slaveBlueprint.blueprint_cluster_binding.host_groups;

    var groups1_used = groups1.map(function () {
      return false;
    });
    var groups2_used = groups2.map(function () {
      return false;
    });

    self.matchGroupsWithLeft(groups1, groups2, groups1_used, groups2_used, res, false);
    self.matchGroupsWithLeft(groups2, groups1, groups2_used, groups1_used, res, true);

    return res;
  },

  matchGroupsWithLeft: function matchGroupsWithLeft(groups1, groups2, groups1_used, groups2_used, res, inverse) {
    var gs2 = this.groupsToObject(groups2);
    for (var i = 0; i < groups1.length; i++) {
      if (groups1_used[i]) {
        continue;
      }

      var group1 = groups1[i];
      groups1_used[i] = true;

      var group2 = gs2[group1.hosts.mapProperty('fqdn').join(',')];
      if (group2) {
        groups2_used[group2.index] = true;
      }

      var item = {};

      if (inverse) {
        item.g2 = group1.name;
        if (group2) {
          item.g1 = group2.name;
        }
      } else {
        item.g1 = group1.name;
        if (group2) {
          item.g2 = group2.name;
        }
      }
      res.push(item);
    }

    // remove unneeded property
    groups2.forEach(function (group) {
      delete group.index;
    });
  },

  /**
   * Convert array of objects to the one object to improve performance with searching objects in the provided array
   * Example:
   *  Before:
   *  <pre>
   *    // groups
   *    [
   *      {
   *        hosts: [
   *          {fqdn: 'h1'}, {fqdn: 'h2'}
   *        ],
   *        name: 'n1'
   *      },
   *      {
   *        hosts: [
   *          {fqdn: 'h3'}, {fqdn: 'h4'}
   *        ]
   *      }
   *    ]
   *  </pre>
   *  Return:
   *  <pre>
   *    {
   *      'h1,h2': {
   *        hosts: [
   *          {fqdn: 'h1'}, {fqdn: 'h2'}
   *        ],
   *        name: 'n1',
   *        index: 0
   *      },
   *      'h3,h4': {
   *        hosts: [
   *          {fqdn: 'h3'}, {fqdn: 'h4'}
   *        ],
   *        name: 'n2',
   *        index: 1
   *      }
   *    }
   *  </pre>
   * @param {{hosts: object[], name: string}[]} groups
   * @returns {object}
   */
  groupsToObject: function groupsToObject(groups) {
    var ret = {};
    groups.forEach(function (group, index) {
      var key = group.hosts.mapProperty('fqdn').join(',');
      ret[key] = group;
      ret[key].index = index;
    });
    return ret;
  },

  /**
   * Remove from blueprint all components expect given components
   * @param {object} blueprint
   * @param {string[]} components
   */
  filterByComponents: function filterByComponents(blueprint, components) {
    if (!blueprint) {
      return null;
    }

    var res = JSON.parse(JSON.stringify(blueprint));
    var emptyGroups = [];

    for (var i = 0; i < res.blueprint.host_groups.length; i++) {
      res.blueprint.host_groups[i].components = res.blueprint.host_groups[i].components.filter(function (c) {
        return components.contains(c.name);
      });

      if (res.blueprint.host_groups[i].components.length == 0) {
        emptyGroups.push(res.blueprint.host_groups[i].name);
      }
    }

    res.blueprint.host_groups = res.blueprint.host_groups.filter(function (g) {
      return !emptyGroups.contains(g.name);
    });

    res.blueprint_cluster_binding.host_groups = res.blueprint_cluster_binding.host_groups.filter(function (g) {
      return !emptyGroups.contains(g.name);
    });

    return res;
  },

  addComponentsToBlueprint: function addComponentsToBlueprint(blueprint, components) {
    var res = JSON.parse(JSON.stringify(blueprint));

    res.blueprint.host_groups.forEach(function (group) {
      components.forEach(function (component) {
        group.components.push({ name: component });
      });
    });

    return res;
  },

  /**
   * @method buildConfigsJSON - generates JSON according to blueprint format
   * @param {Em.Array} stepConfigs - array of Ember Objects
   * @returns {Object}
   * Example:
   * {
   *   "yarn-env": {
   *     "properties": {
   *       "content": "some value",
   *       "yarn_heapsize": "1024",
   *       "resourcemanager_heapsize": "1024",
   *     }
   *   },
   *   "yarn-log4j": {
   *     "properties": {
   *       "content": "some other value"
   *     }
   *   }
   * }
   */
  buildConfigsJSON: function buildConfigsJSON(stepConfigs) {
    var configurations = {};
    stepConfigs.forEach(function (stepConfig) {
      var configs = stepConfig.get('configs');
      if (!configs) return false;
      configs.forEach(function (config) {
        var type = App.config.getConfigTagFromFileName(config.get('filename'));
        if (!configurations[type]) {
          configurations[type] = { properties: {} };
        }
        configurations[type]['properties'][config.get('name')] = config.get('value');
      });
    });
    return configurations;
  },

  /**
   * @method generateHostGroups
   * @param {Array} hostNames - list of all hostNames
   * @returns {{blueprint: {host_groups: Array}, blueprint_cluster_binding: {host_groups: Array}}}
   */
  generateHostGroups: function generateHostGroups(hostNames) {
    var recommendations = {
      blueprint: {
        host_groups: []
      },
      blueprint_cluster_binding: {
        host_groups: []
      }
    };
    var hostsMap = this.getComponentForHosts();

    for (var i = 0; i <= hostNames.length; i++) {
      var host_group = {
        name: "host-group-" + (i + 1),
        components: []
      };
      var hcFiltered = hostsMap[hostNames[i]];
      if (Em.isNone(hcFiltered)) continue;
      for (var j = 0; j < hcFiltered.length; j++) {
        host_group.components.push({
          name: hcFiltered[j]
        });
      }
      recommendations.blueprint.host_groups.push(host_group);
      recommendations.blueprint_cluster_binding.host_groups.push({
        name: "host-group-" + (i + 1),
        hosts: [{
          fqdn: hostNames[i]
        }]
      });
    }
    return recommendations;
  },

  /**
   * Small helper method to update hostMap
   * it perform update of object only
   * if unique component per host is added
   *
   * @param {Object} hostMapObject
   * @param {string[]} hostNames
   * @param {string} componentName
   * @returns {Object}
   * @private
   */
  _generateHostMap: function _generateHostMap(hostMapObject, hostNames, componentName) {
    Em.assert('hostMapObject, hostNames, componentName should be defined', !!hostMapObject && !!hostNames && !!componentName);
    if (!hostNames.length) return hostMapObject;
    hostNames.forEach(function (hostName) {
      if (!hostMapObject[hostName]) hostMapObject[hostName] = [];

      if (!hostMapObject[hostName].contains(componentName)) hostMapObject[hostName].push(componentName);
    });
    return hostMapObject;
  },

  /**
   * Clean up host groups from components that should be removed
   *
   * @param hostGroups
   * @param serviceNames
   */
  removeDeletedComponents: function removeDeletedComponents(hostGroups, serviceNames) {
    var components = [];
    App.StackService.find().forEach(function (s) {
      if (serviceNames.contains(s.get('serviceName'))) {
        components = components.concat(s.get('serviceComponents').mapProperty('componentName'));
      }
    });

    hostGroups.blueprint.host_groups.forEach(function (hg) {
      hg.components = hg.components.filter(function (c) {
        return !components.contains(c.name);
      });
    });
    return hostGroups;
  },

  /**
   * collect all component names that are present on hosts
   * @returns {object}
   */
  getComponentForHosts: function getComponentForHosts() {
    var hostsMap = {};
    var componentModels = [App.ClientComponent, App.SlaveComponent, App.MasterComponent];
    componentModels.forEach(function (_model) {
      _model.find().forEach(function (c) {
        hostsMap = this._generateHostMap(hostsMap, c.get('hostNames'), c.get('componentName'));
      }, this);
    }, this);

    this.changeHostsMapForConfigActions(hostsMap);
    return hostsMap;
  },

  /**
   * Adds or removes the component name entry as saved in App.componentToBeAdded and App.componentToBeDeleted from the 'hostsMap'
   * @param hostsMap {object}
   * @private
   * @method {changeHostsMapForConfigActions}
   */
  changeHostsMapForConfigActions: function changeHostsMapForConfigActions(hostsMap) {
    var componentToBeAdded = App.get('componentToBeAdded');
    var componentToBeDeleted = App.get('componentToBeDeleted');
    if (!App.isEmptyObject(componentToBeAdded)) {
      hostsMap = this._generateHostMap(hostsMap, componentToBeAdded.get('hostNames'), componentToBeAdded.get('componentName'));
    } else if (!App.isEmptyObject(componentToBeDeleted) && hostsMap[componentToBeDeleted.hostName]) {
      var index = hostsMap[componentToBeDeleted.hostName].indexOf(componentToBeDeleted.componentName);
      if (index > -1) {
        hostsMap[componentToBeDeleted.hostName].splice(index, 1);
      }
    }
  },
  /**
   * Returns host-group name by fqdn.
   * Returns <code>null</code> when not found.
   *
   * @param  {object} blueprint
   * @param  {string} fqdn
   * @returns {string|null}
   */
  getHostGroupByFqdn: function getHostGroupByFqdn(blueprint, fqdn) {
    return Em.getWithDefault(blueprint || {}, 'blueprint_cluster_binding.host_groups', []).filter(function (i) {
      return Em.getWithDefault(i, 'hosts', []).mapProperty('fqdn').contains(fqdn);
    }).mapProperty('name')[0] || null;
  },

  /**
   * Add component to specified host group.
   *
   * @param  {object} blueprint
   * @param  {string} componentName
   * @param  {string} hostGroupName
   * @return {object}
   */
  addComponentToHostGroup: function addComponentToHostGroup(blueprint, componentName, hostGroupName) {
    var hostGroup = blueprint.blueprint.host_groups.findProperty('name', hostGroupName);
    if (hostGroup) {
      if (!hostGroup.hasOwnProperty('components')) {
        hostGroup.components = [];
      }
      if (!hostGroup.components.someProperty('name', componentName)) {
        hostGroup.components.pushObject({ name: componentName });
      }
    }
    return blueprint;
  }
};

});

require.register("utils/bootstrap_reopen", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * This file contains patched methods for twitter bootstrap.js
 */

/**
 * Fixes error when <code>destroy</code> method called more than one time.
 * For more info check https://github.com/twbs/bootstrap/issues/20511
 */
$.fn.tooltip.Constructor.prototype.destroy = function () {
  var that = this;
  clearTimeout(this.timeout);
  this.hide(function () {
    if (that.$element !== null) {
      that.$element.off('.' + that.type).removeData('bs.' + that.type);
    }
    if (that.$tip) {
      that.$tip.detach();
    }
    that.$tip = null;
    that.$arrow = null;
    that.$viewport = null;
    that.$element = null;
  });
};

// function required for clearMenus
var getParent = function getParent($this) {
  var selector = $this.attr('data-target');

  if (!selector) {
    selector = $this.attr('href');
    selector = selector && /#[A-Za-z]/.test(selector) && selector.replace(/.*(?=#[^\s]*$)/, ''); // strip for ie7;
  }

  var $parent = selector && $(selector);

  return $parent && $parent.length ? $parent : $this.parent();
};

// new exclusion added, clicking on elements with class checkbox-label should be ignored to close dropdown
var clearMenus = function clearMenus(e) {
  if (e && e.which === 3) return;
  $('.dropdown-backdrop').remove();
  $('[data-toggle="dropdown"]').each(function () {
    var $this = $(this);
    var $parent = getParent($this);
    var relatedTarget = { relatedTarget: this };

    if (!$parent.hasClass('open')) return;

    if (e && e.type == 'click' && (/input|textarea/i.test(e.target.tagName) || e.target.className.contains('checkbox-label')) && $.contains($parent[0], e.target)) return;

    $parent.trigger(e = $.Event('hide.bs.dropdown', relatedTarget));

    if (e.isDefaultPrevented()) return;

    $this.attr('aria-expanded', 'false');
    $parent.removeClass('open').trigger($.Event('hidden.bs.dropdown', relatedTarget));
  });
};

$(document).off('click.bs.dropdown.data-api');
$(document).on('click.bs.dropdown.data-api', clearMenus);
$(document).on('click.bs.dropdown.data-api', '.dropdown form', function (e) {
  e.stopPropagation();
});
$(document).on('click.bs.dropdown.data-api', '[data-toggle="dropdown"]', $.fn.dropdown.Constructor.prototype.toggle);

});

require.register("utils/config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs_collection');
var stringUtils = require('utils/string_utils');
var validator = require('utils/validator');

var configTagFromFileNameMap = {};
var PASSWORD = "PASSWORD";

App.config = Em.Object.create({

  CONFIG_GROUP_NAME_MAX_LENGTH: 18,

  /**
   * filename exceptions used to support substandard sitenames which don't have "xml" extension
   * @type {string[]}
   */
  filenameExceptions: ['alert_notification'],

  preDefinedServiceConfigs: [],

  /**
   * Map for methods used to parse hosts lists from certain config properties
   */
  uniqueHostsListParsers: [{
    propertyName: 'templeton.hive.properties',
    type: 'webhcat-site',
    method: 'getTempletonHiveHosts'
  }],

  /**
   *
   * Returns file name version that stored on server.
   *
   * Example:
   *   App.config.getOriginalFileName('core-site') // returns core-site.xml
   *   App.config.getOriginalFileName('zoo.cfg') // returns zoo.cfg
   *
   * @param {String} fileName
   * @method getOriginalFileName
   **/
  getOriginalFileName: function getOriginalFileName(fileName) {
    if (/\.xml$/.test(fileName)) return fileName;
    return this.get('filenameExceptions').contains(fileName) ? fileName : fileName + '.xml';
  },

  /**
   * truncate Config Group name to <CONFIG_GROUP_NAME_MAX_LENGTH> length and paste "..." in the middle
   */
  truncateGroupName: function truncateGroupName(name) {
    if (name && name.length > App.config.CONFIG_GROUP_NAME_MAX_LENGTH) {
      var middle = Math.floor(App.config.CONFIG_GROUP_NAME_MAX_LENGTH / 2);
      name = name.substring(0, middle) + "..." + name.substring(name.length - middle);
    }
    return name;
  },

  /**
   * Check if Hive installation with new MySQL database created via Ambari is allowed
   * @param osFamily
   * @returns {boolean}
   */
  isManagedMySQLForHiveAllowed: function isManagedMySQLForHiveAllowed(osFamily) {
    var osList = ['redhat5', 'suse11'];
    return !osList.contains(osFamily);
  },

  /**
   *
   * Returns the configuration tagName from supplied filename
   *
   * Example:
   *   App.config.getConfigTagFromFileName('core-site.xml') // returns core-site
   *   App.config.getConfigTagFromFileName('zoo.cfg') // returns zoo.cfg
   *
   * @param {String} fileName
   * @method getConfigTagFromFileName
   **/
  getConfigTagFromFileName: function getConfigTagFromFileName(fileName) {
    if (configTagFromFileNameMap[fileName]) {
      return configTagFromFileNameMap[fileName];
    }
    var ret = fileName.endsWith('.xml') ? fileName.slice(0, -4) : fileName;
    configTagFromFileNameMap[fileName] = ret;
    return ret;
  },

  /**
   *
   * @param name
   * @param fileName
   * @returns {string}
   */
  configId: function configId(name, fileName) {
    return name + "__" + App.config.getConfigTagFromFileName(fileName);
  },

  setPreDefinedServiceConfigs: function setPreDefinedServiceConfigs(isMiscTabToBeAdded) {
    var stackServices = App.StackService.find().filterProperty('id');
    // Only include services that has configTypes related to them for service configuration page
    var servicesWithConfigTypes = stackServices.filter(function (service) {
      var configtypes = service.get('configTypes');
      return configtypes && !!Object.keys(configtypes).length;
    }, this);

    var allTabs;
    if (isMiscTabToBeAdded) {
      var nonServiceTab = require('data/service_configs');
      var miscService = nonServiceTab.findProperty('serviceName', 'MISC');
      var tagTypes = {};
      servicesWithConfigTypes.mapProperty('configTypes').forEach(function (configTypes) {
        for (var fileName in configTypes) {
          if (fileName.endsWith('-env') && !miscService.get('configTypes')[fileName]) {
            tagTypes[fileName] = configTypes[fileName];
          }
        }
      });
      miscService.set('configTypes', $.extend(miscService.get('configTypes'), tagTypes));
      allTabs = servicesWithConfigTypes.concat(nonServiceTab);
    } else {
      allTabs = servicesWithConfigTypes;
    }
    this.set('preDefinedServiceConfigs', allTabs);
  },

  secureConfigs: require('data/configs/wizards/secure_mapping'),

  secureConfigsMap: function () {
    var ret = {};
    this.get('secureConfigs').forEach(function (sc) {
      ret[sc.name] = true;
    });
    return ret;
  }.property('secureConfigs.[]'),

  kerberosIdentities: require('data/configs/wizards/kerberos_identities').configProperties,

  kerberosIdentitiesMap: function () {
    var map = {};

    this.get('kerberosIdentities').forEach(function (c) {
      map[this.configId(c.name, c.filename)] = c;
    }, this);
    return map;
  }.property('kerberosIdentities'),

  customStackMapping: require('data/custom_stack_map'),

  mapCustomStack: function mapCustomStack() {
    var baseStackFolder = App.get('currentStackName'),
        singMap = {
      "1": [">", ">="],
      "-1": ["<", "<="],
      "0": ["=", ">=", "<="]
    };

    this.get('customStackMapping').every(function (stack) {
      if (stack.stackName == App.get('currentStackName')) {
        var versionCompare = Em.compare(App.get('currentStackVersionNumber'), stack.stackVersionNumber);
        if (singMap[versionCompare + ""].contains(stack.sign)) {
          baseStackFolder = stack.baseStackFolder;
          return false;
        }
      }
      return true;
    });

    return baseStackFolder;
  },

  allPreDefinedSiteProperties: require('data/configs/site_properties').configProperties,

  preDefinedSiteProperties: function () {
    var serviceNames = App.StackService.find().mapProperty('serviceName').concat('MISC');
    return this.get('allPreDefinedSiteProperties').filter(function (p) {
      return serviceNames.contains(p.serviceName);
    });
  }.property().volatile(),

  /**
   * map of <code>preDefinedSiteProperties</code> provide search by index
   * @type {object}
   */
  preDefinedSitePropertiesMap: function () {
    var map = {};

    this.get('preDefinedSiteProperties').forEach(function (c) {
      map[this.configId(c.name, c.filename)] = c;
    }, this);
    return map;
  }.property('preDefinedSiteProperties'),

  serviceByConfigTypeMap: function () {
    var ret = {};
    App.StackService.find().forEach(function (s) {
      s.get('configTypeList').forEach(function (ct) {
        ret[ct] = s;
      });
    });
    return ret;
  }.property(),

  /**
   * Generate configs collection with Ember or plain config objects
   * from config JSON
   *
   * @param configJSON
   * @param useEmberObject
   * @returns {Array}
   */
  getConfigsFromJSON: function getConfigsFromJSON(configJSON, useEmberObject) {
    var configs = [],
        filename = App.config.getOriginalFileName(configJSON.type),
        properties = configJSON.properties,
        attributes = [];
    ['FINAL', 'PASSWORD', 'USER', 'GROUP', 'TEXT', 'ADDITIONAL_USER_PROPERTY', 'NOT_MANAGED_HDFS_PATH', 'VALUE_FROM_PROPERTY_FILE'].forEach(function (attribute) {
      var json = {};
      json[attribute] = Em.get(configJSON, 'properties_attributes.' + attribute.toLowerCase()) || {};
      attributes.push(json);
    });

    for (var index in properties) {
      var serviceConfigObj = this.getDefaultConfig(index, filename);

      if (serviceConfigObj.isRequiredByAgent !== false) {
        serviceConfigObj.value = serviceConfigObj.savedValue = this.formatPropertyValue(serviceConfigObj, properties[index]);
        serviceConfigObj.isFinal = serviceConfigObj.savedIsFinal = attributes[0]['FINAL'][index] === "true";

        var propertyType = [];
        // iterate through all the attributes, except for FINAL
        for (var i = 1; i < attributes.length; i++) {
          for (var type in attributes[i]) {
            if (attributes[i][type][index] === "true") {
              propertyType.push(type);

              if (type === PASSWORD) {
                serviceConfigObj.displayType = "password";
              }
            }
          }
        }
        serviceConfigObj.propertyType = propertyType;
        serviceConfigObj.isEditable = serviceConfigObj.isReconfigurable;
      }

      if (useEmberObject) {
        configs.push(App.ServiceConfigProperty.create(serviceConfigObj));
      } else {
        configs.push(serviceConfigObj);
      }
    }
    return configs;
  },

  /**
   * Get config from configsCollections or
   * generate new default config in collection does not contain
   * such config
   *
   * @param name
   * @param fileName
   * @param coreObject
   * @returns {*|Object}
   */
  getDefaultConfig: function getDefaultConfig(name, fileName, coreObject) {
    name = JSON.parse('"' + name + '"');
    var cfg = App.configsCollection.getConfigByName(name, fileName) || App.config.createDefaultConfig(name, fileName, false);
    if (Em.typeOf(coreObject) === 'object') {
      Em.setProperties(cfg, coreObject);
    }
    return cfg;
  },

  /**
   * This method sets default values for config property
   * These property values has the lowest priority and can be overridden be stack/UI
   * config property but is used when such properties are absent in stack/UI configs
   * @param {string} name
   * @param {string} fileName
   * @param {boolean} definedInStack
   * @param {Object} [coreObject]
   * @returns {Object}
   */
  createDefaultConfig: function createDefaultConfig(name, fileName, definedInStack, coreObject) {
    var service = this.get('serviceByConfigTypeMap')[App.config.getConfigTagFromFileName(fileName)];
    var serviceName = service ? service.get('serviceName') : 'MISC';
    var tpl = {
      /** core properties **/
      id: this.configId(name, fileName),
      name: name,
      filename: this.getOriginalFileName(fileName),
      value: '',
      savedValue: null,
      isFinal: false,
      savedIsFinal: null,
      /** UI and Stack properties **/
      recommendedValue: null,
      recommendedIsFinal: null,
      supportsFinal: this.shouldSupportFinal(serviceName, fileName),
      supportsAddingForbidden: this.shouldSupportAddingForbidden(serviceName, fileName),
      serviceName: serviceName,
      displayName: name,
      displayType: coreObject && coreObject.propertyType && coreObject.propertyType.contains(PASSWORD) ? 'password' : this.getDefaultDisplayType(coreObject ? coreObject.value : ''),
      description: '',
      category: this.getDefaultCategory(definedInStack, fileName),
      isSecureConfig: this.getIsSecure(name),
      showLabel: true,
      isVisible: true,
      isUserProperty: !definedInStack,
      isRequired: definedInStack,
      group: null,
      isRequiredByAgent: true,
      isReconfigurable: true,
      unit: null,
      hasInitialValue: false,
      isOverridable: true,
      index: Infinity,
      dependentConfigPattern: null,
      options: null,
      radioName: null,
      widgetType: null,
      errorMessage: '',
      warnMessage: ''
    };
    return Object.keys(coreObject || {}).length ? $.extend(tpl, coreObject) : tpl;
  },

  /**
   * This method creates host name properties
   * @param serviceName
   * @param componentName
   * @param value
   * @param stackComponent
   * @returns Object
   */
  createHostNameProperty: function createHostNameProperty(serviceName, componentName, value, stackComponent) {
    var hostOrHosts = stackComponent.get('isMultipleAllowed') ? 'hosts' : 'host';
    var name = componentName.toLowerCase() + '_' + hostOrHosts;
    var filename = serviceName.toLowerCase() + "-site.xml";
    return {
      "id": App.config.configId(name, filename),
      "name": name,
      "displayName": stackComponent.get('displayName') + ' ' + (value.length > 1 ? 'hosts' : 'host'),
      "value": value,
      "recommendedValue": value,
      "description": "The " + hostOrHosts + " that has been assigned to run " + stackComponent.get('displayName'),
      "displayType": "component" + hostOrHosts.capitalize(),
      "isOverridable": false,
      "isRequiredByAgent": false,
      "serviceName": serviceName,
      "filename": filename,
      "category": componentName,
      "index": 0
    };
  },

  /**
   * Get component name from config name string
   *
   * @param configName
   * @returns {string}
   */
  getComponentName: function getComponentName(configName) {
    var match = configName.match(/^(.*)_host[s]?$/) || [],
        component = match[1];
    return component ? component.toUpperCase() : "";
  },

  /**
   * This method merge properties form <code>stackConfigProperty<code> which are taken from stack
   * with <code>UIConfigProperty<code> which are hardcoded on UI
   * @param coreObject
   * @param stackProperty
   * @param preDefined
   * @param [propertiesToSkip]
   */
  mergeStaticProperties: function mergeStaticProperties(coreObject, stackProperty, preDefined, propertiesToSkip) {
    propertiesToSkip = propertiesToSkip || ['name', 'filename', 'value', 'savedValue', 'isFinal', 'savedIsFinal'];
    for (var k in coreObject) {
      if (coreObject.hasOwnProperty(k)) {
        if (!propertiesToSkip.contains(k)) {
          coreObject[k] = this.getPropertyIfExists(k, coreObject[k], stackProperty, preDefined);
        }
      }
    }
    return coreObject;
  },

  /**
   * This method using for merging some properties from two objects
   * if property exists in <code>firstPriority<code> result will be it's property
   * else if property exists in <code>secondPriority<code> result will be it's property
   * otherwise <code>defaultValue<code> will be returned
   * @param {String} propertyName
   * @param {*} defaultValue=null
   * @param {Em.Object|Object} firstPriority
   * @param {Em.Object|Object} [secondPriority=null]
   * @returns {*}
   */
  getPropertyIfExists: function getPropertyIfExists(propertyName, defaultValue, firstPriority, secondPriority) {
    firstPriority = firstPriority || {};
    secondPriority = secondPriority || {};
    var fp = Em.get(firstPriority, propertyName);
    if (firstPriority && !Em.isNone(fp)) {
      return fp;
    } else {
      var sp = Em.get(secondPriority, propertyName);
      if (secondPriority && !Em.isNone(sp)) {
        return sp;
      } else {
        return defaultValue;
      }
    }
  },

  /**
   * Get displayType for properties that has not defined value
   * @param value
   * @returns {string}
   */
  getDefaultDisplayType: function getDefaultDisplayType(value) {
    return value && !stringUtils.isSingleLine(value) ? 'multiLine' : 'string';
  },

  /**
   * Get category for properties that has not defined value
   * @param stackConfigProperty
   * @param fileName
   * @returns {string}
   */
  getDefaultCategory: function getDefaultCategory(stackConfigProperty, fileName) {
    return (stackConfigProperty ? 'Advanced ' : 'Custom ') + this.getConfigTagFromFileName(fileName);
  },

  /**
   * Get isSecureConfig for properties that has not defined value
   * @param propertyName
   * @returns {boolean}
   */
  getIsSecure: function getIsSecure(propertyName) {
    return !!this.get('secureConfigsMap')[propertyName];
  },

  /**
   * Returns description, formatted if needed
   *
   * @param {String} description
   * @param {String} displayType
   * @returns {String}
   */
  getDescription: function getDescription(description, displayType) {
    var additionalDescription = Em.I18n.t('services.service.config.password.additionalDescription');
    if ('password' === displayType) {
      if (description && !description.contains(additionalDescription)) {
        return description + '\n' + additionalDescription;
      } else {
        return additionalDescription;
      }
    }
    return description;
  },

  /**
   * parse Kerberos descriptor
   *
   * @param kerberosDescriptor
   * @returns {{}}
   */
  parseDescriptor: function parseDescriptor(kerberosDescriptor) {
    var identitiesMap = {};
    Em.get(kerberosDescriptor, 'KerberosDescriptor.kerberos_descriptor.services').forEach(function (service) {
      this.parseIdentities(service, identitiesMap);
      if (Array.isArray(service.components)) {
        service.components.forEach(function (component) {
          this.parseIdentities(component, identitiesMap);
        }, this);
      }
    }, this);
    return identitiesMap;
  },

  /**
   * Looking for configs identities and add them to <code>identitiesMap<code>
   *
   * @param item
   * @param identitiesMap
   */
  parseIdentities: function parseIdentities(item, identitiesMap) {
    if (item.identities) {
      item.identities.forEach(function (identity) {

        Em.keys(identity).without('name').forEach(function (item) {
          if (identity[item].configuration) {
            var cfg = identity[item].configuration.split('/'),
                name = cfg[1],
                fileName = cfg[0];
            identitiesMap[App.config.configId(name, fileName)] = true;
          }
        });
      });
    }
    return identitiesMap;
  },

  /**
   * Update description for disabled kerberos configs which are identities
   *
   * @param description
   * @returns {*}
   */
  kerberosIdentitiesDescription: function kerberosIdentitiesDescription(description) {
    if (!description) return Em.I18n.t('services.service.config.secure.additionalDescription');
    description = description.trim();
    return (description.endsWith('.') ? description + ' ' : description + '. ') + Em.I18n.t('services.service.config.secure.additionalDescription');
  },

  /**
   * Get view class based on display type of config
   *
   * @param displayType
   * @param dependentConfigPattern
   * @param unit
   * @returns {*}
   */
  getViewClass: function getViewClass(displayType, dependentConfigPattern, unit) {
    switch (displayType) {
      case 'user':
      case 'group':
        return App.ServiceConfigTextFieldUserGroupWithID;
      case 'checkbox':
      case 'boolean':
        return dependentConfigPattern ? App.ServiceConfigCheckboxWithDependencies : App.ServiceConfigCheckbox;
      case 'boolean-inverted':
        return App.ServiceConfigCheckbox;
      case 'password':
        return App.ServiceConfigPasswordField;
      case 'combobox':
        return App.ServiceConfigComboBox;
      case 'radio button':
        return App.ServiceConfigRadioButtons;
      case 'directories':
        return App.ServiceConfigTextArea;
      case 'directory':
        return App.ServiceConfigTextField;
      case 'content':
        return App.ServiceConfigTextAreaContent;
      case 'multiLine':
        return App.ServiceConfigTextArea;
      case 'custom':
        return App.ServiceConfigBigTextArea;
      case 'componentHost':
        return App.ServiceConfigMasterHostView;
      case 'label':
        return App.ServiceConfigLabelView;
      case 'componentHosts':
        return App.ServiceConfigComponentHostsView;
      case 'supportTextConnection':
        return App.checkConnectionView;
      case 'capacityScheduler':
        return App.CapacitySceduler;
      default:
        return unit ? App.ServiceConfigTextFieldWithUnit : App.ServiceConfigTextField;
    }
  },

  /**
   * Returns error validator function based on config type
   *
   * @param displayType
   * @returns {Function}
   */
  getErrorValidator: function getErrorValidator(displayType) {
    switch (displayType) {
      case 'checkbox':
      case 'custom':
        return function () {
          return '';
        };
      case 'int':
        return function (value) {
          return !validator.isValidInt(value) && !validator.isConfigValueLink(value) ? Em.I18n.t('errorMessage.config.number.integer') : '';
        };
      case 'float':
        return function (value) {
          return !validator.isValidFloat(value) && !validator.isConfigValueLink(value) ? Em.I18n.t('errorMessage.config.number.float') : '';
        };
      case 'directories':
      case 'directory':
        return function (value, name) {
          if (App.config.isDirHeterogeneous(name)) {
            if (!validator.isValidDataNodeDir(value)) return Em.I18n.t('errorMessage.config.directory.heterogeneous');
          } else {
            if (!validator.isValidDir(value)) return Em.I18n.t('errorMessage.config.directory.default');
          }
          if (!validator.isAllowedDir(value)) {
            return Em.I18n.t('errorMessage.config.directory.allowed');
          }
          return validator.isNotTrimmedRight(value) ? Em.I18n.t('errorMessage.config.spaces.trailing') : '';
        };
      case 'email':
        return function (value) {
          return !validator.isValidEmail(value) ? Em.I18n.t('errorMessage.config.mail') : '';
        };
      case 'supportTextConnection':
      case 'host':
        return function (value) {
          return validator.isNotTrimmed(value) ? Em.I18n.t('errorMessage.config.spaces.trim') : '';
        };
      case 'password':
        return function (value, name, retypedPassword) {
          if (name === 'ranger_admin_password') {
            if (String(value).length < 9) {
              return Em.I18n.t('errorMessage.config.password.length').format(9);
            }
          }
          return value !== retypedPassword ? Em.I18n.t('errorMessage.config.password') : '';
        };
      case 'user':
      case 'database':
      case 'db_user':
        return function (value) {
          return !validator.isValidDbName(value) ? Em.I18n.t('errorMessage.config.user') : '';
        };
      case 'ldap_url':
        return function (value) {
          return !validator.isValidLdapsURL(value) ? Em.I18n.t('errorMessage.config.ldapUrl') : '';
        };
      default:
        return function (value, name) {
          if (['javax.jdo.option.ConnectionURL', 'oozie.service.JPAService.jdbc.url'].contains(name) && !validator.isConfigValueLink(value) && validator.isConfigValueLink(value)) {
            return Em.I18n.t('errorMessage.config.spaces.trim');
          }
          return validator.isNotTrimmedRight(value) ? Em.I18n.t('errorMessage.config.spaces.trailing') : '';
        };
    }
  },

  /**
   * Returns warning validator function based on config type
   *
   * @param displayType
   * @returns {Function}
   */
  getWarningValidator: function getWarningValidator(displayType) {
    switch (displayType) {
      case 'int':
      case 'float':
        return function (value, name, filename, stackConfigProperty, unitLabel) {
          stackConfigProperty = stackConfigProperty || App.configsCollection.getConfigByName(name, filename);
          var maximum = Em.get(stackConfigProperty || {}, 'valueAttributes.maximum'),
              minimum = Em.get(stackConfigProperty || {}, 'valueAttributes.minimum'),
              min = validator.isValidFloat(minimum) ? parseFloat(minimum) : NaN,
              max = validator.isValidFloat(maximum) ? parseFloat(maximum) : NaN,
              val = validator.isValidFloat(value) ? parseFloat(value) : NaN;

          if (!isNaN(val) && !isNaN(max) && val > max) {
            return Em.I18n.t('config.warnMessage.outOfBoundaries.greater').format(max + unitLabel);
          }
          if (!isNaN(val) && !isNaN(min) && val < min) {
            return Em.I18n.t('config.warnMessage.outOfBoundaries.less').format(min + unitLabel);
          }
          return '';
        };
      default:
        return function () {
          return '';
        };
    }
  },

  /**
   * Defines if config support heterogeneous devices
   *
   * @param {string} name
   * @returns {boolean}
   */
  isDirHeterogeneous: function isDirHeterogeneous(name) {
    return ['dfs.datanode.data.dir'].contains(name);
  },

  /**
   * format property value depending on displayType
   * and one exception for 'kdc_type'
   * @param serviceConfigProperty
   * @param [originalValue]
   * @returns {*}
   */
  formatPropertyValue: function formatPropertyValue(serviceConfigProperty, originalValue) {
    var value = Em.isNone(originalValue) ? Em.get(serviceConfigProperty, 'value') : originalValue,
        displayType = Em.get(serviceConfigProperty, 'displayType') || Em.get(serviceConfigProperty, 'valueAttributes.type');
    if (Em.get(serviceConfigProperty, 'name') === 'kdc_type') {
      return App.router.get('mainAdminKerberosController.kdcTypesValues')[value];
    }
    if (/^\s+$/.test("" + value)) {
      return " ";
    }
    switch (displayType) {
      case 'int':
        if (/\d+m$/.test(value)) {
          return value.slice(0, value.length - 1);
        } else {
          var int = parseInt(value);
          return isNaN(int) ? "" : int.toString();
        }
      case 'float':
        var float = parseFloat(value);
        return isNaN(float) ? "" : float.toString();
      case 'componentHosts':
        if (typeof value == 'string') {
          return value.replace(/\[|]|'|&apos;/g, "").split(',');
        }
        return value;
      case 'content':
      case 'string':
      case 'multiLine':
      case 'directories':
      case 'directory':
        return this.trimProperty({ displayType: displayType, value: value });
      default:
        return value;
    }
  },

  /**
   * Format float value
   *
   * @param {*} value
   * @returns {string|*}
   */
  formatValue: function formatValue(value) {
    return validator.isValidFloat(value) ? parseFloat(value).toString() : value;
  },

  /**
   * Get step config by file name
   *
   * @param stepConfigs
   * @param fileName
   * @returns {Object|null}
   */
  getStepConfigForProperty: function getStepConfigForProperty(stepConfigs, fileName) {
    return stepConfigs.find(function (s) {
      return s.get('configTypes').contains(App.config.getConfigTagFromFileName(fileName));
    });
  },

  /**
   *
   * @param configs
   * @returns {Object[]}
   */
  sortConfigs: function sortConfigs(configs) {
    return configs.sort(function (a, b) {
      if (Em.get(a, 'index') > Em.get(b, 'index')) return 1;
      if (Em.get(a, 'index') < Em.get(b, 'index')) return -1;
      if (Em.get(a, 'name') > Em.get(b, 'name')) return 1;
      if (Em.get(a, 'name') < Em.get(b, 'name')) return -1;
      return 0;
    });
  },

  /**
   * create new ServiceConfig object by service name
   * @param {string} serviceName
   * @param {App.ServiceConfigGroup[]} [configGroups]
   * @param {App.ServiceConfigProperty[]} [configs]
   * @param {Number} [initConfigsLength]
   * @return {App.ServiceConfig}
   * @method createServiceConfig
   */
  createServiceConfig: function createServiceConfig(serviceName, configGroups, configs, initConfigsLength) {
    var preDefinedServiceConfig = App.config.get('preDefinedServiceConfigs').findProperty('serviceName', serviceName);
    return App.ServiceConfig.create({
      serviceName: preDefinedServiceConfig.get('serviceName'),
      displayName: preDefinedServiceConfig.get('displayName'),
      configCategories: preDefinedServiceConfig.get('configCategories'),
      configs: configs || [],
      configGroups: configGroups || [],
      initConfigsLength: initConfigsLength || 0
    });
  },

  /**
   * GETs all cluster level sites in one call.
   *
   * @return {$.ajax}
   */
  loadConfigsByTags: function loadConfigsByTags(tags) {
    var urlParams = [];
    tags.forEach(function (_tag) {
      urlParams.push('(type=' + _tag.siteName + '&tag=' + _tag.tagName + ')');
    });
    var params = urlParams.join('|');
    return App.ajax.send({
      name: 'config.on_site',
      sender: this,
      data: {
        params: params
      }
    });
  },

  configTypesInfoMap: {},

  /**
   * Get config types and config type attributes from stack service
   *
   * @param service
   * @return {object}
   */
  getConfigTypesInfoFromService: function getConfigTypesInfoFromService(service) {
    var configTypesInfoMap = this.get('configTypesInfoMap');
    if (configTypesInfoMap[service]) {
      // don't recalculate
      return configTypesInfoMap[service];
    }
    var configTypes = service.get('configTypes');
    var configTypesInfo = {
      items: [],
      supportsFinal: [],
      supportsAddingForbidden: []
    };
    if (configTypes) {
      for (var key in configTypes) {
        if (configTypes.hasOwnProperty(key)) {
          configTypesInfo.items.push(key);
          if (configTypes[key].supports && configTypes[key].supports.final === "true") {
            configTypesInfo.supportsFinal.push(key);
          }
          if (configTypes[key].supports && configTypes[key].supports.adding_forbidden === "true") {
            configTypesInfo.supportsAddingForbidden.push(key);
          }
        }
      }
    }
    configTypesInfoMap[service] = configTypesInfo;
    this.set('configTypesInfoMap', configTypesInfoMap);
    return configTypesInfo;
  },

  /**
   *
   * @param configs
   */
  addYarnCapacityScheduler: function addYarnCapacityScheduler(configs) {
    var value = '',
        savedValue = '',
        recommendedValue = '',
        excludedConfigs = App.config.getPropertiesFromTheme('YARN');

    var connectedConfigs = configs.filter(function (config) {
      return !excludedConfigs.contains(App.config.configId(config.get('name'), config.get('filename'))) && config.get('filename') === 'capacity-scheduler.xml';
    });
    var names = connectedConfigs.mapProperty('name');

    connectedConfigs.forEach(function (config) {
      value += config.get('name') + '=' + config.get('value') + '\n';
      if (!Em.isNone(config.get('savedValue'))) {
        savedValue += config.get('name') + '=' + config.get('savedValue') + '\n';
      }
      if (!Em.isNone(config.get('recommendedValue'))) {
        recommendedValue += config.get('name') + '=' + config.get('recommendedValue') + '\n';
      }
    }, this);

    var isFinal = connectedConfigs.someProperty('isFinal', true);
    var savedIsFinal = connectedConfigs.someProperty('savedIsFinal', true);
    var recommendedIsFinal = connectedConfigs.someProperty('recommendedIsFinal', true);

    var cs = App.config.createDefaultConfig('capacity-scheduler', 'capacity-scheduler.xml', true, {
      'value': value,
      'serviceName': 'YARN',
      'savedValue': savedValue || null,
      'recommendedValue': recommendedValue || null,
      'isFinal': isFinal,
      'savedIsFinal': savedIsFinal,
      'recommendedIsFinal': recommendedIsFinal,
      'category': 'CapacityScheduler',
      'displayName': 'Capacity Scheduler',
      'description': 'Capacity Scheduler properties',
      'displayType': 'capacityScheduler'
    });

    configs = configs.filter(function (c) {
      return !(names.contains(c.get('name')) && c.get('filename') === 'capacity-scheduler.xml');
    });
    configs.push(App.ServiceConfigProperty.create(cs));
    return configs;
  },

  /**
   *
   * @param serviceName
   * @returns {Array}
   */
  getPropertiesFromTheme: function getPropertiesFromTheme(serviceName) {
    var properties = [];
    App.Tab.find().rejectProperty('isCategorized').forEach(function (t) {
      if (!t.get('isAdvanced') && t.get('serviceName') === serviceName) {
        t.get('sections').forEach(function (s) {
          s.get('subSections').forEach(function (ss) {
            properties = properties.concat(ss.get('configProperties'));
          });
        });
      }
    }, this);
    return properties;
  },

  /**
   * transform one config with textarea content
   * into set of configs of file
   * @param configs
   * @param filename
   * @return {*}
   */
  textareaIntoFileConfigs: function textareaIntoFileConfigs(configs, filename) {
    var configsTextarea = configs.findProperty('name', 'capacity-scheduler');
    var stackConfigs = App.configsCollection.getAll();
    if (configsTextarea && !App.get('testMode')) {
      var properties = configsTextarea.get('value').split('\n');

      properties.forEach(function (_property) {
        var name, value, isUserProperty;
        if (_property) {
          _property = _property.split(/=(.+)/);
          name = _property[0];
          value = _property[1] ? _property[1] : "";
          isUserProperty = !stackConfigs.filterProperty('filename', 'capacity-scheduler.xml').findProperty('name', name);

          configs.push(Em.Object.create({
            name: name,
            value: value,
            savedValue: value,
            serviceName: configsTextarea.get('serviceName'),
            filename: filename,
            isFinal: configsTextarea.get('isFinal'),
            isNotDefaultValue: configsTextarea.get('isNotDefaultValue'),
            isRequiredByAgent: configsTextarea.get('isRequiredByAgent'),
            isUserProperty: isUserProperty,
            group: null
          }));
        }
      });
      return configs.without(configsTextarea);
    }
    return configs;
  },

  /**
   * trim trailing spaces for all properties.
   * trim both trailing and leading spaces for host displayType and hive/oozie datebases url.
   * for directory or directories displayType format string for further using.
   * for password and values with spaces only do nothing.
   * @param {Object} property
   * @returns {*}
   */
  trimProperty: function trimProperty(property) {
    var displayType = Em.get(property, 'displayType');
    var value = Em.get(property, 'value');
    var name = Em.get(property, 'name');
    var rez;
    switch (displayType) {
      case 'directories':
      case 'directory':
        rez = value.replace(/,/g, ' ').trim().split(/\s+/g).join(',');
        break;
      case 'host':
        rez = value.trim();
        break;
      case 'password':
        break;
      default:
        if (name == 'javax.jdo.option.ConnectionURL' || name == 'oozie.service.JPAService.jdbc.url') {
          rez = value.trim();
        }
        rez = typeof value == 'string' ? value.replace(/(\s+$)/g, '') : value;
    }
    return rez == '' || rez == undefined ? value : rez;
  },

  /**
   * Generate minimal config property object used in *_properties.js files.
   * Example:
   * <code>
   *   var someProperties = App.config.generateConfigPropertiesByName([
   *    'property_1', 'property_2', 'property_3'], { category: 'General', filename: 'myFileName'});
   *   // someProperties contains Object[]
   *   [
   *    {
   *      name: 'property_1',
   *      displayName: 'property_1',
   *      isVisible: true,
   *      isReconfigurable: true,
   *      category: 'General',
   *      filename: 'myFileName'
   *    },
   *    .......
   *   ]
   * </code>
   * @param {string[]} names
   * @param {Object} properties - additional properties which will merge with base object definition
   * @returns {object[]}
   * @method generateConfigPropertiesByName
   */
  generateConfigPropertiesByName: function generateConfigPropertiesByName(names, properties) {
    return names.map(function (item) {
      var baseObj = {
        name: item
      };
      if (properties) return $.extend(baseObj, properties);else return baseObj;
    });
  },

  /**
   * load cluster stack configs from server and run mapper
   * @returns {$.ajax}
   * @method loadConfigsFromStack
   */
  loadClusterConfigsFromStack: function loadClusterConfigsFromStack() {
    return App.ajax.send({
      name: 'configs.stack_configs.load.cluster_configs',
      sender: this,
      data: {
        stackVersionUrl: App.get('stackVersionURL')
      },
      success: 'saveConfigsToModel'
    });
  },

  /**
   * load stack configs from server and run mapper
   * @param {String[]} [serviceNames=null]
   * @returns {$.ajax}
   * @method loadConfigsFromStack
   */
  loadConfigsFromStack: function loadConfigsFromStack(serviceNames) {
    serviceNames = serviceNames || [];
    var name = serviceNames.length > 0 ? 'configs.stack_configs.load.services' : 'configs.stack_configs.load.all';
    return App.ajax.send({
      name: name,
      sender: this,
      data: {
        stackVersionUrl: App.get('stackVersionURL'),
        serviceList: serviceNames.join(',')
      },
      success: 'saveConfigsToModel'
    });
  },

  /**
   * Runs <code>stackConfigPropertiesMapper<code>
   * @param {object} data
   * @method saveConfigsToModel
   */
  saveConfigsToModel: function saveConfigsToModel(data) {
    App.stackConfigPropertiesMapper.map(data);
  },

  /**
   * Check if config filename supports final attribute
   * @param serviceName
   * @param filename
   * @returns {boolean}
   */
  shouldSupportFinal: function shouldSupportFinal(serviceName, filename) {
    var unsupportedServiceNames = ['MISC', 'Cluster'];
    if (!serviceName || unsupportedServiceNames.contains(serviceName) || !filename) {
      return false;
    } else {
      var stackService = App.StackService.find(serviceName);
      if (!stackService) {
        return false;
      }
      return !!this.getConfigTypesInfoFromService(stackService).supportsFinal.find(function (configType) {
        return filename.startsWith(configType);
      });
    }
  },

  shouldSupportAddingForbidden: function shouldSupportAddingForbidden(serviceName, filename) {
    var unsupportedServiceNames = ['MISC', 'Cluster'];
    if (!serviceName || unsupportedServiceNames.contains(serviceName) || !filename) {
      return false;
    } else {
      var stackServiceName = App.StackService.find().findProperty('serviceName', serviceName);
      if (!stackServiceName) {
        return false;
      }
      var stackService = App.StackService.find(serviceName);
      return !!this.getConfigTypesInfoFromService(stackService).supportsAddingForbidden.find(function (configType) {
        return filename.startsWith(configType);
      });
    }
  },

  /**
   * Remove all ranger-related configs, that should be available only if Ranger is installed
   * @param configs - stepConfigs object
   */
  removeRangerConfigs: function removeRangerConfigs(configs) {
    configs.forEach(function (service) {
      var filteredConfigs = [];
      service.get('configs').forEach(function (config) {
        if (!/^ranger-/.test(config.get('filename'))) {
          filteredConfigs.push(config);
        }
      });
      service.set('configs', filteredConfigs);
      var filteredCategories = [];
      service.get('configCategories').forEach(function (category) {
        if (!/ranger-/.test(category.get('name'))) {
          filteredCategories.push(category);
        }
      });
      service.set('configCategories', filteredCategories);
    });
  },

  /**
   * Create config with non default config group. Some custom config properties
   * can be created and assigned to non-default config group.
   *
   * @param {Em.Object} override - config value
   * @param {Em.Object} configGroup - config group to set
   * @return {Object}
   **/
  createCustomGroupConfig: function createCustomGroupConfig(override, configGroup) {
    App.assertObject(override);
    App.assertEmberObject(configGroup);

    var newOverride = App.ServiceConfigProperty.create(this.createDefaultConfig(override.propertyName, override.filename, false, override));

    newOverride.setProperties({
      'isOriginalSCP': false,
      'overrides': null,
      'group': configGroup,
      'parentSCP': null,
      isCustomGroupConfig: true
    });

    if (!configGroup.get('properties.length')) {
      configGroup.set('properties', Em.A([]));
    }
    configGroup.set('properties', configGroup.get('properties').concat(newOverride));

    return newOverride;
  },

  /**
   * @param {App.ServiceConfigProperty} serviceConfigProperty
   * @param {Object} override - plain object with properties that is different from parent SCP
   * @param {App.ServiceConfigGroup} configGroup
   * @returns {App.ServiceConfigProperty}
   */
  createOverride: function createOverride(serviceConfigProperty, override, configGroup) {
    App.assertObject(serviceConfigProperty);
    App.assertEmberObject(configGroup);

    if (Em.isNone(serviceConfigProperty.get('overrides'))) serviceConfigProperty.set('overrides', []);

    var newOverride = App.ServiceConfigProperty.create(serviceConfigProperty);

    newOverride.setProperties({ 'savedValue': null, 'savedIsFinal': null });

    if (!Em.isNone(override)) {
      for (var key in override) {
        newOverride.set(key, override[key]);
      }
    }

    newOverride.setProperties({
      'isOriginalSCP': false,
      'overrides': null,
      'group': configGroup,
      'parentSCP': serviceConfigProperty
    });

    if (!configGroup.get('properties.length')) {
      configGroup.set('properties', Em.A([]));
    }
    configGroup.set('properties', configGroup.get('properties').concat(newOverride));

    serviceConfigProperty.get('overrides').pushObject(newOverride);

    var savedOverrides = serviceConfigProperty.get('overrides').filter(function (override) {
      return !Em.isNone(Em.get(override, 'savedValue'));
    });
    serviceConfigProperty.set('overrideValues', savedOverrides.mapProperty('savedValue'));
    serviceConfigProperty.set('overrideIsFinalValues', savedOverrides.mapProperty('savedIsFinal'));

    return newOverride;
  },

  /**
   * Merge values in "stored" to "base" if name matches, it's a value only merge.
   * @param base {Array} Em.Object
   * @param stored {Array} Object
   * @returns {Object[]|Em.Object[]} base
   */
  mergeStoredValue: function mergeStoredValue(base, stored) {
    if (stored) {
      base.forEach(function (p) {
        var sp = stored.filterProperty('filename', p.filename).findProperty('name', p.name);
        if (sp) {
          Em.set(p, 'value', Em.get(sp, 'value'));
        }
      });
    }
    return base;
  },

  /**
   * Helper method to get property from the <code>stepConfigs</code>
   *
   * @param {String} name - config property name
   * @param {String} fileName - config property filename
   * @param {Object[]} stepConfigs
   * @return {App.ServiceConfigProperty|Boolean} - App.ServiceConfigProperty instance or <code>false</code> when property not found
   */
  findConfigProperty: function findConfigProperty(stepConfigs, name, fileName) {
    if (!name || !fileName) return false;
    if (stepConfigs && stepConfigs.length) {
      return stepConfigs.mapProperty('configs').filter(function (item) {
        return item.length;
      }).reduce(function (p, c) {
        if (p) {
          return p.concat(c);
        }
      }).filterProperty('filename', fileName).findProperty('name', name);
    }
    return false;
  },

  getTempletonHiveHosts: function getTempletonHiveHosts(value) {
    var pattern = /thrift:\/\/.+:\d+/,
        patternMatch = value.match(pattern);
    return patternMatch ? patternMatch[0].split('\\,') : [];
  },

  /**
   * Update config property value based on its current value and list of zookeeper server hosts.
   * Used to prevent sort order issues.
   * <code>siteConfigs</code> object formatted according server's persist format e.g.
   *
   * <code>
   *   {
   *     'yarn-site': {
   *       'property_name1': 'property_value1'
   *       'property_name2': 'property_value2'
   *       .....
   *     }
   *   }
   * </code>
   *
   * @method updateHostsListValue
   * @param {Object} siteConfigs - prepared site config object to store
   * @param {String} propertyType - type of the property to update
   * @param {String} propertyName - name of the property to update
   * @param {String} hostsList - list of ZooKeeper Server names to set as config property value
   * @param {Boolean} isArray - determines whether value string is formatted as array
   * @return {String} - result value
   */
  updateHostsListValue: function updateHostsListValue(siteConfigs, propertyType, propertyName, hostsList, isArray) {
    var value = hostsList,
        propertyHosts = siteConfigs[propertyName] || '',
        hostsToSet = hostsList,
        parser = this.get('uniqueHostsListParsers').find(function (property) {
      return property.type === propertyType && property.propertyName === propertyName;
    });
    if (parser) {
      propertyHosts = this.get(parser.method)(propertyHosts);
      hostsToSet = this.get(parser.method)(hostsToSet);
    } else {
      if (isArray) {
        var pattern = /(^\[|]$)/g;
        propertyHosts = propertyHosts.replace(pattern, '');
        hostsToSet = hostsToSet.replace(pattern, '');
      }
      propertyHosts = propertyHosts.split(',');
      hostsToSet = hostsToSet.split(',');
    }

    if (!Em.isEmpty(siteConfigs[propertyName])) {
      var diffLength = propertyHosts.filter(function (hostName) {
        return !hostsToSet.contains(hostName);
      }).length;
      if (diffLength == 0 && propertyHosts.length == hostsToSet.length) {
        value = siteConfigs[propertyName];
      }
    }
    siteConfigs[propertyName] = value;
    return value;
  },

  /**
   * Load cluster-env configs mapped to array
   * @return {*|{then}}
   */
  getConfigsByTypes: function getConfigsByTypes(sites) {
    var _this = this;

    var dfd = $.Deferred();
    App.router.get('configurationController').getCurrentConfigsBySites(sites.mapProperty('site')).done(function (configs) {
      dfd.resolve(_this.getMappedConfigs(configs, sites));
    });
    return dfd.promise();
  },

  /**
   *
   * @param configs
   * @param sites
   */
  getMappedConfigs: function getMappedConfigs(configs, sites) {
    var result = [];
    configs.forEach(function (config) {
      var configsArray = [];
      var configsObject = config.properties;
      for (var property in configsObject) {
        if (configsObject.hasOwnProperty(property)) {
          configsArray.push(Em.Object.create({
            name: property,
            value: configsObject[property],
            filename: App.config.getOriginalFileName(config.type)
          }));
        }
      }
      result.push(Em.Object.create({
        serviceName: sites.findProperty('site', config.type).serviceName,
        configs: configsArray
      }));
    });

    return result;
  }
});

});

require.register("utils/configs/add_component_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');
require('utils/configs/ha_config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');
require('utils/configs/control_flow_initializer_mixin');

var _slice = Array.prototype.slice;

/**
 * Main class responsible for properties computation.
 * This class contains all required info to manipulate properties regarding value updates
 * during removing/adding components.
 * To determine when component removed or added you just need to setup properly localDB object
 * and set `isInstalled` flag to `true` where selected component(s) will be located after adding/removing.
 * By default all initializer handlers filtering localDB by `isInstalled` `true`.
 *
 * @mixes App.ControlFlowInitializerMixin
 * @mixes App.HostsBasedInitializerMixin
 * @type {AddComponentConfigInitializer}
 * @augments {HaConfigInitializerClass}
 */
App.AddComponentConfigInitializer = App.HaConfigInitializerClass.extend(App.HostsBasedInitializerMixin, App.ControlFlowInitializerMixin, {
  /**
   * All initializer properties definition.
   * Object format is the same as for App.ConfigInitializerClass.initializers
   * @see App.ConfigInitializerClass.initializers
   *
   * @return {object} property name - initializer map
   */
  __defaultInitializers: function __defaultInitializers() {
    return {
      'zookeeper.connect': this.getHDPStackOnlyHostsPortConfig('2.2', 'ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'ha.zookeeper.quorum': this.getNameNodeHAOnlyHostsPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'hbase.zookeeper.quorum': this.getHostsListComponentConfig('ZOOKEEPER_SERVER', true),
      'instance.zookeeper.host': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'templeton.zookeeper.hosts': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'hive.cluster.delegation.token.store.zookeeper.connectString': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'storm.zookeeper.servers': this.getHostsListComponentJSONStringifiedConfig('ZOOKEEPER_SERVER', true),
      'hive.zookeeper.quorum': this.getHDPStackOnlyHostsPortConfig('2.2', 'ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'hadoop.registry.zk.quorum': this.getHDPStackOnlyHostsPortConfig('2.2', 'ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'nimbus.seeds': this.getHostsListComponentJSONStringifiedConfig('NIMBUS', true),
      'hadoop.proxyuser.{{webhcatUser}}.hosts': this.getComponentsHostsConfig(['HIVE_SERVER', 'WEBHCAT_SERVER', 'HIVE_METASTORE'], false, true),
      'hadoop.proxyuser.{{hiveUser}}.hosts': this.getComponentsHostsConfig(['HIVE_SERVER', 'WEBHCAT_SERVER', 'HIVE_METASTORE', 'HIVE_SERVER_INTERACTIVE'], false, true),
      'hive.metastore.uris': this.getHostsWithPortConfig(['HIVE_METASTORE'], 'thrift://', '', ',thrift://', 'hiveMetastorePort', true),
      'atlas.audit.hbase.zookeeper.quorum': this.getHostsListComponentConfig('ZOOKEEPER_SERVER', true),
      'atlas.graph.storage.hostname': this.getHostsListComponentConfig('ZOOKEEPER_SERVER', true),
      'atlas.kafka.zookeeper.connect': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true)
    };
  },

  /**
   * All unique initializer definition.
   * Object format is the same as for App.ConfigInitializerClass.uniqueInitializers
   * @see App.ConfigInitializerClass.uniqueInitializers
   *
   * @type {Object}
   */
  __defaultUniqueInitializers: {
    'yarn.resourcemanager.zk-address': '_initYarnRMZkAdress',
    'templeton.hive.properties': '_initTempletonHiveProperties',
    'atlas.graph.index.search.solr.zookeeper-url': '_initAtlasGraphIndexSearchSolrZkUrl'
  },

  /**
   * Property names to initialize. This attribute should be overrided in class instance.
   * `initializers` property will set up according this list from `__defaultUniqueInitializers` and
   * `__defaultInitializers`
   *
   * @type {string[]}
   */
  initializeForProperties: null,

  initializers: function () {
    return {};
  }.property(),

  uniqueInitializers: {},

  init: function init() {
    this._super();
    this._bootstrapInitializers(this.get('initializeForProperties'));
  },

  initializerTypes: [{
    name: 'json_stringified_value',
    method: '_initAsJSONStrigifiedValueConfig'
  }],

  /**
   * @override
   * @param {object} settings
   */
  setup: function setup(settings) {
    this._updateInitializers(settings);
  },

  /**
   * @override
   */
  cleanup: function cleanup() {
    this._restoreInitializers();
  },

  getJSONStringifiedValueConfig: function getJSONStringifiedValueConfig() {
    return {
      type: 'json_stringified_value'
    };
  },

  _initAsJSONStrigifiedValueConfig: function _initAsJSONStrigifiedValueConfig(configProperty, localDB, dependencies, initializer) {
    var hostsValue = Em.get(configProperty, 'value').split(Em.getWithDefault(initializer, 'modifier.delimiter', ','));
    var propertyValue = JSON.stringify(hostsValue).replace(/"/g, "'");
    Em.setProperties(configProperty, {
      value: propertyValue,
      recommendedValue: propertyValue
    });
    return configProperty;
  },

  /**
   * Perform value update according to hosts. Mutate <code>siteConfigs</code>
   *
   * @param {object} siteConfigs
   * @param {configProperty} configProperty
   * @returns {boolean}
   */
  updateSiteObj: function updateSiteObj(siteConfigs, configProperty) {
    if (!siteConfigs || !configProperty) return false;
    var initializer = this.get('initializers')[configProperty.name],
        isArray = !!(initializer && (initializer.type === 'json_stringified_value' || Em.isArray(initializer) && initializer.someProperty('type', 'json_stringified_value')));
    App.config.updateHostsListValue(siteConfigs, configProperty.fileName, configProperty.name, configProperty.value, isArray);
    return true;
  },

  /**
   * @see App.ControlFlowInitializerMixin.getNameNodeHAControl
   * @see App.HostsBasedInitializerMixin.getComponentsHostsConfig
   */
  getNameNodeHAOnlyHostsConfig: function getNameNodeHAOnlyHostsConfig(components, asArray) {
    return [this.getNameNodeHAControl(), this.getComponentsHostsConfig.apply(this, _slice.call(arguments))];
  },

  /**
   * @override
   **/
  getHostsWithPortConfig: function getHostsWithPortConfig(component, prefix, suffix, delimiter, port, portFromDependencies) {
    var ret = this._super.apply(this, _slice.call(arguments));
    ret.componentExists = true;
    return ret;
  },

  /**
   * @see App.ControlFlowInitializerMixin.getNameNodeHAControl
   * @see App.HostsBasedInitializerMixin.getHostsWithPortConfig
   */
  getNameNodeHAOnlyHostsPortConfig: function getNameNodeHAOnlyHostsPortConfig(component, prefix, suffix, delimiter, port, portFromDependencies) {
    return [this.getNameNodeHAControl(), this.getHostsWithPortConfig.apply(this, _slice.call(arguments))];
  },

  /**
   * @see App.ControlFlowInitializerMixin.getResourceManagerHAControl
   * @see App.HostsBasedInitializerMixin.getHostsWithPortConfig
   */
  getResourceManagerHAOnlyHostsPortConfig: function getResourceManagerHAOnlyHostsPortConfig(component, prefix, suffix, delimiter, port, portFromDependencies) {
    return [this.getResourceManagerHAControl(), this.getHostsWithPortConfig.apply(this, _slice.call(arguments))];
  },

  /**
   * @see App.HostsBasedInitializerMixin.getHostsListComponentConfig
   * @see getJSONStringifiedValueConfig
   */
  getHostsListComponentJSONStringifiedConfig: function getHostsListComponentJSONStringifiedConfig(component, componentExists, delimiter) {
    return [this.getHostsListComponentConfig.apply(this, _slice.call(arguments)), this.getJSONStringifiedValueConfig()];
  },

  /**
   * @see App.ControlFlowInitializerMixin.getHDPStackVersionControl
   * @see App.HostsBasedInitializerMixin.getHostsWithPortConfig
   */
  getHDPStackOnlyHostsPortConfig: function getHDPStackOnlyHostsPortConfig(minStackVersion, component, prefix, suffix, delimiter, port, portFromDependencies) {
    return [this.getHDPStackVersionControl(minStackVersion), this.getHostsWithPortConfig.apply(this, _slice.call(arguments, 1))];
  },

  _initYarnRMZkAdress: function _initYarnRMZkAdress(configProperty, localDB, dependencies) {
    return this._initAsHostsWithPort(configProperty, localDB, dependencies, {
      component: 'ZOOKEEPER_SERVER',
      componentExists: true,
      modifier: {
        prefix: '',
        suffix: '',
        delimiter: ','
      },
      portKey: 'zkClientPort'
    });
  },

  _initTempletonHiveProperties: function _initTempletonHiveProperties(configProperty, localDB, dependecies, initializer) {
    var hostNames = localDB.masterComponentHosts.filter(function (masterComponent) {
      return ['HIVE_METASTORE'].contains(masterComponent.component) && masterComponent.isInstalled === true;
    }).mapProperty('hostName').uniq().sort();
    var hiveMSHosts = hostNames.map(function (hostName) {
      return "thrift://" + hostName + ":" + dependecies.hiveMetastorePort;
    }).join('\\,');
    var value = configProperty.value.replace(/thrift.+[0-9]{2,},/i, hiveMSHosts + ",");
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  _initAtlasGraphIndexSearchSolrZkUrl: function _initAtlasGraphIndexSearchSolrZkUrl(configProperty, localDB, dependencies, initializer) {
    var solr = dependencies.infraSolrZnode;
    return this._initAsHostsWithPort(configProperty, localDB, dependencies, {
      component: 'ZOOKEEPER_SERVER',
      componentExists: true,
      modifier: {
        prefix: '',
        suffix: solr,
        delimiter: solr + ','
      },
      portKey: 'zkClientPort'
    });
  },

  /**
   * Set up `this.initializers` and `this.uniqueInitializers` properties according
   * to property list names.
   *
   * @param  {string[]} properties list of property names
   */
  _bootstrapInitializers: function _bootstrapInitializers(properties) {
    var initializers = {},
        uniqueInitializers = {},
        defaultInitializers = this.__defaultInitializers(),
        defaultUniqueInitializers = this.get('__defaultUniqueInitializers');

    if (Em.isNone(properties)) {
      initializers = this.__defaultInitializers();
      uniqueInitializer = this.get('__defaultUniqueInitializers');
    } else {
      properties.forEach(function (propertyName) {
        if (defaultInitializers[propertyName]) {
          initializers[propertyName] = defaultInitializers[propertyName];
        } else if (defaultUniqueInitializers[propertyName]) {
          uniqueInitializers[propertyName] = defaultUniqueInitializers[propertyName];
        }
      });
    }
    this._setForComputed('initializers', initializers);
    this.set('uniqueInitializers', uniqueInitializers);
  }
});

/**
 * ZooKeeper service add/remove components initializer.
 * @instance App.AddComponentConfigInitializer
 */
App.AddZooKeeperComponentsInitializer = App.AddComponentConfigInitializer.create({
  initializeForProperties: ['zookeeper.connect', 'ha.zookeeper.quorum', 'hbase.zookeeper.quorum', 'instance.zookeeper.host', 'templeton.zookeeper.hosts', 'hive.cluster.delegation.token.store.zookeeper.connectString', 'yarn.resourcemanager.zk-address', 'hive.zookeeper.quorum', 'storm.zookeeper.servers', 'hadoop.registry.zk.quorum', 'atlas.audit.hbase.zookeeper.quorum', 'atlas.graph.index.search.solr.zookeeper-url', 'atlas.graph.storage.hostname', 'atlas.kafka.zookeeper.connect']
});

/**
 * Hive service add/remove components initializer.
 * @instance App.AddComponentConfigInitializer
 */
App.AddHiveComponentsInitializer = App.AddComponentConfigInitializer.create({
  initializeForProperties: ['hive.metastore.uris', 'templeton.hive.properties', 'hadoop.proxyuser.{{hiveUser}}.hosts']
});

/**
 * WebHCat service add/remove components initializer.
 * @instance App.AddComponentConfigInitializer
 */
App.AddWebHCatComponentsInitializer = App.AddComponentConfigInitializer.create({
  initializeForProperties: ['hadoop.proxyuser.{{webhcatUser}}.hosts']
});

/**
 * Hive Server Interactive component add initializer.
 * @instance App.AddHiveServerInteractiveInitializer
 */
App.AddHiveServerInteractiveInitializer = App.AddComponentConfigInitializer.create({
  initializeForProperties: ['hadoop.proxyuser.{{hiveUser}}.hosts']
});

});

require.register("utils/configs/config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

require('utils/configs/config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');

/**
 * Zookeeper-based configs don't have any customization settings
 *
 * @see _initAsZookeeperServersList
 * @returns {{type: string}}
 */
function getZKBasedConfig() {
  return {
    type: 'zookeeper_based'
  };
}

/**
 * Initializer for configs
 * Used on the cluster install
 *
 * Usage:
 * <pre>
 *   var configProperty = Object.create({});
 *   var localDB = {
 *    hosts: [],
 *    masterComponentHosts: [],
 *    slaveComponentHosts: []
 *   };
 *   var dependencies = {};
 *   App.ConfigInitializer.initialValue(configProperty, localDB, dependencies);
 * </pre>
 *
 * @instance ConfigInitializer
 */
App.ConfigInitializer = App.ConfigInitializerClass.create(App.HostsBasedInitializerMixin, {

  initializers: function () {
    return {
      'dfs.namenode.rpc-address': this.getSimpleComponentConfig('NAMENODE'),
      'dfs.http.address': this.getSimpleComponentConfig('NAMENODE'),
      'dfs.namenode.http-address': this.getSimpleComponentConfig('NAMENODE'),
      'dfs.https.address': this.getSimpleComponentConfig('NAMENODE'),
      'dfs.namenode.https-address': this.getSimpleComponentConfig('NAMENODE'),
      'dfs.secondary.http.address': this.getSimpleComponentConfig('SECONDARY_NAMENODE'),
      'dfs.namenode.secondary.http-address': this.getSimpleComponentConfig('SECONDARY_NAMENODE'),
      'yarn.resourcemanager.hostname': this.getSimpleComponentConfig('RESOURCEMANAGER', false),
      'yarn.resourcemanager.resource-tracker.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.resourcemanager.webapp.https.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.resourcemanager.webapp.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.resourcemanager.scheduler.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.resourcemanager.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.resourcemanager.admin.address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'yarn.timeline-service.webapp.address': this.getSimpleComponentConfig('APP_TIMELINE_SERVER'),
      'yarn.timeline-service.webapp.https.address': this.getSimpleComponentConfig('APP_TIMELINE_SERVER'),
      'yarn.timeline-service.address': this.getSimpleComponentConfig('APP_TIMELINE_SERVER'),
      'mapred.job.tracker': this.getSimpleComponentConfig('JOBTRACKER'),
      'mapred.job.tracker.http.address': this.getSimpleComponentConfig('JOBTRACKER'),
      'mapreduce.history.server.http.address': this.getSimpleComponentConfig('HISTORYSERVER'),
      'oozie.base.url': this.getComponentConfigWithAffixes('OOZIE_SERVER', '://'),
      'hawq_dfs_url': this.getSimpleComponentConfig('NAMENODE'),
      'hawq_rm_yarn_address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'hawq_rm_yarn_scheduler_address': this.getSimpleComponentConfig('RESOURCEMANAGER'),
      'fs.default.name': this.getComponentConfigWithAffixes('NAMENODE', '://'),
      'fs.defaultFS': this.getComponentConfigWithAffixes('NAMENODE', '://'),
      'hbase.rootdir': this.getComponentConfigWithAffixes('NAMENODE', '://'),
      'instance.volumes': this.getComponentConfigWithAffixes('NAMENODE', '://'),
      'yarn.log.server.url': this.getComponentConfigWithAffixes('HISTORYSERVER', '://'),
      'mapreduce.jobhistory.webapp.address': this.getSimpleComponentConfig('HISTORYSERVER'),
      'mapreduce.jobhistory.address': this.getSimpleComponentConfig('HISTORYSERVER'),
      'kafka.ganglia.metrics.host': this.getSimpleComponentConfig('GANGLIA_SERVER', false),
      'hive_master_hosts': this.getComponentsHostsConfig(['HIVE_METASTORE', 'HIVE_SERVER']),
      'hadoop_host': this.getSimpleComponentConfig('NAMENODE', false),
      'nimbus.host': this.getSimpleComponentConfig('NIMBUS', false),
      'nimbus.seeds': this.getComponentsHostsConfig('NIMBUS', true),
      'storm.zookeeper.servers': this.getComponentsHostsConfig('ZOOKEEPER_SERVER', true),
      'hawq_master_address_host': this.getSimpleComponentConfig('HAWQMASTER', false),
      'hawq_standby_address_host': this.getSimpleComponentConfig('HAWQSTANDBY', false),

      '*.broker.url': {
        type: 'host_with_component',
        component: 'FALCON_SERVER',
        modifier: {
          type: 'regexp',
          regex: 'localhost'
        }
      },

      'zookeeper.connect': getZKBasedConfig(),
      'hive.zookeeper.quorum': getZKBasedConfig(),
      'templeton.zookeeper.hosts': getZKBasedConfig(),
      'hadoop.registry.zk.quorum': getZKBasedConfig(),
      'hive.cluster.delegation.token.store.zookeeper.connectString': getZKBasedConfig(),
      'instance.zookeeper.host': getZKBasedConfig()
    };
  }.property(''),

  uniqueInitializers: {
    'ranger_admin_password': '_setRangerAdminPassword',
    'hive_database': '_initHiveDatabaseValue',
    'templeton.hive.properties': '_initTempletonHiveProperties',
    'hbase.zookeeper.quorum': '_initHBaseZookeeperQuorum',
    'yarn.resourcemanager.zk-address': '_initYarnRMzkAddress',
    'RANGER_HOST': '_initRangerHost',
    'hive.metastore.uris': '_initHiveMetastoreUris',
    'atlas.rest.address': '_initAtlasRestAddress'
  },

  initializerTypes: [{ name: 'zookeeper_based', method: '_initAsZookeeperServersList' }],

  /**
   * Some strange method that should define <code>ranger_admin_password</code>
   * TODO DELETE as soon as <code>ranger_admin_password</code> will be fetched from stack adviser!
   *
   * @param {configProperty} configProperty
   * @private
   */
  _setRangerAdminPassword: function _setRangerAdminPassword(configProperty) {
    var value = 'P1!q' + stringUtils.getRandomString(12);
    Em.setProperties(configProperty, { 'value': value, 'recommendedValue': value, 'retypedPassword': value });
    return configProperty;
  },

  /**
   * Unique initializer for <code>hive_database</code>-config
   *
   * @param {configProperty} configProperty
   * @returns {Object}
   * @private
   */
  _initHiveDatabaseValue: function _initHiveDatabaseValue(configProperty) {
    var newMySQLDBOption = Em.get(configProperty, 'options').findProperty('displayName', 'New MySQL Database');
    if (newMySQLDBOption) {
      var isNewMySQLDBOptionHidden = !App.get('supports.alwaysEnableManagedMySQLForHive') && App.get('router.currentState.name') !== 'configs' && !App.get('isManagedMySQLForHiveEnabled');
      if (isNewMySQLDBOptionHidden && Em.get(configProperty, 'value') === 'New MySQL Database') {
        Em.set(configProperty, 'value', 'Existing MySQL Database');
      }
      Em.set(newMySQLDBOption, 'hidden', isNewMySQLDBOptionHidden);
    }
    return configProperty;
  },

  /**
   * Initializer for configs with value equal to hostNames-list where ZOOKEEPER_SERVER is installed
   * Value example: 'host1:2020,host2:2020,host3:2020'
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @returns {Object}
   * @private
   */
  _initAsZookeeperServersList: function _initAsZookeeperServersList(configProperty, localDB) {
    var zkHosts = localDB.masterComponentHosts.filterProperty('component', 'ZOOKEEPER_SERVER').mapProperty('hostName');
    var zkHostPort = zkHosts;
    var regex = '\\w*:(\\d+)'; //regex to fetch the port
    var portValue = Em.get(configProperty, 'recommendedValue') && Em.get(configProperty, 'recommendedValue').match(new RegExp(regex));
    if (!portValue) {
      return configProperty;
    }
    if (portValue[1]) {
      for (var i = 0; i < zkHosts.length; i++) {
        zkHostPort[i] = zkHosts[i] + ':' + portValue[1];
      }
    }
    this.setRecommendedValue(configProperty, '(.*)', zkHostPort);
    return configProperty;
  },

  /**
   * Unique initializer for <code>templeton.hive.properties</code>
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @returns {Object}
   * @private
   */
  _initTempletonHiveProperties: function _initTempletonHiveProperties(configProperty, localDB, dependencies) {
    var hiveMSUris = this.getHiveMetastoreUris(localDB.masterComponentHosts, dependencies['hive.metastore.uris']).replace(',', '\\,');
    if (/\/\/localhost:/g.test(Em.get(configProperty, 'value'))) {
      Em.set(configProperty, 'recommendedValue', Em.get(configProperty, 'value') + ',hive.metastore.execute.setugi=true');
    }
    this.setRecommendedValue(configProperty, "(hive\\.metastore\\.uris=)([^\\,]+)", "$1" + hiveMSUris);
    return configProperty;
  },

  /**
   * Unique initializer for <code>hbase.zookeeper.quorum</code>
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @returns {Object}
   * @private
   */
  _initHBaseZookeeperQuorum: function _initHBaseZookeeperQuorum(configProperty, localDB) {
    if ('hbase-site.xml' === Em.get(configProperty, 'filename')) {
      var zkHosts = localDB.masterComponentHosts.filterProperty('component', 'ZOOKEEPER_SERVER').mapProperty('hostName');
      this.setRecommendedValue(configProperty, "(.*)", zkHosts);
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>RANGER_HOST</code>
   * If RANGER_ADMIN-component isn't installed, this config becomes unneeded (isVisible - false, isRequired - false)
   * Value example: 'hostName'
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @returns {Object}
   * @private
   */
  _initRangerHost: function _initRangerHost(configProperty, localDB) {
    var rangerAdminHost = localDB.masterComponentHosts.findProperty('component', 'RANGER_ADMIN');
    if (rangerAdminHost) {
      Em.setProperties(configProperty, {
        value: rangerAdminHost.hostName,
        recommendedValue: rangerAdminHost.hostName
      });
    } else {
      Em.setProperties(configProperty, {
        isVisible: 'false',
        isRequired: 'false'
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>yarn.resourcemanager.zk-address</code>
   * List of hosts where ZOOKEEPER_SERVER is installed
   * Port is taken from <code>dependencies.clientPort</code>
   * Value example: 'host1:111,host2:111,host3:111'
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @returns {Object}
   * @private
   */
  _initYarnRMzkAddress: function _initYarnRMzkAddress(configProperty, localDB, dependencies) {
    var value = localDB.masterComponentHosts.filterProperty('component', 'ZOOKEEPER_SERVER').map(function (component) {
      return component.hostName + ':' + dependencies.clientPort;
    }).join(',');
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  /**
   * Unique initializer for <code>hive.metastore.uris</code>
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @returns {Object}
   * @private
   */
  _initHiveMetastoreUris: function _initHiveMetastoreUris(configProperty, localDB, dependencies) {
    if (App.config.getConfigTagFromFileName(Em.get(configProperty, 'filename')) === 'hive-site') {
      var hiveMSUris = this.getHiveMetastoreUris(localDB.masterComponentHosts, dependencies['hive.metastore.uris']);
      if (hiveMSUris) {
        this.setRecommendedValue(configProperty, "(.*)", hiveMSUris);
      }
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>atlas.rest.address</code>
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @return {Object}
   * @private
   */
  _initAtlasRestAddress: function _initAtlasRestAddress(configProperty, localDB, dependencies) {
    var atlasTls = dependencies['atlas.enableTLS'];
    var httpPort = dependencies['atlas.server.http.port'];
    var httpsPort = dependencies['atlas.server.https.port'];
    var protocol = atlasTls ? 'https' : 'http';
    var port = atlasTls ? httpsPort : httpPort;
    var value = localDB.masterComponentHosts.filterProperty('component', 'ZOOKEEPER_SERVER').map(function (component) {
      return protocol + '://' + component.hostName + ':' + port;
    }).join(',');
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  /**
   * Get hive.metastore.uris initial value
   *
   * @param {object[]} hosts
   * @param {string} recommendedValue
   * @returns {string}
   */
  getHiveMetastoreUris: function getHiveMetastoreUris(hosts, recommendedValue) {
    var hiveMSHosts = hosts.filterProperty('component', 'HIVE_METASTORE').mapProperty('hostName'),
        hiveMSUris = hiveMSHosts,
        regex = "\\w*:(\\d+)",
        portValue = recommendedValue && recommendedValue.match(new RegExp(regex));

    if (!portValue) {
      return '';
    }
    if (portValue[1]) {
      for (var i = 0; i < hiveMSHosts.length; i++) {
        hiveMSUris[i] = "thrift://" + hiveMSHosts[i] + ":" + portValue[1];
      }
    }
    return hiveMSUris.join(',');
  },

  /**
   * Set <code>value</code> and <code>recommendedValue</code> for <code>configProperty</code>
   * basing on <code>recommendedValue</code> with replacing <code>regex</code> for <code>replaceWith</code>
   *
   * @param {configProperty} configProperty
   * @param {string} regex
   * @param {string} replaceWith
   * @return {Object}
   */
  setRecommendedValue: function setRecommendedValue(configProperty, regex, replaceWith) {
    var recommendedValue = Em.get(configProperty, 'recommendedValue');
    recommendedValue = Em.isNone(recommendedValue) ? '' : recommendedValue;
    var re = new RegExp(regex);
    recommendedValue = recommendedValue.replace(re, replaceWith);
    Em.set(configProperty, 'recommendedValue', recommendedValue);
    var value = Em.isNone(Em.get(configProperty, 'recommendedValue')) ? '' : recommendedValue;
    Em.set(configProperty, 'value', value);
    Em.set(configProperty, 'initialValue', value);
    return configProperty;
  }
});

});

require.register("utils/configs/config_initializer_class", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @typedef {object} initializer
 * @property {string} type initializer type name
 * @property {boolean} [isChecker] determines control flow callback
 */

/**
 * @typedef {object} initializerType
 * @property {string} name key
 * @property {string} method function's name (prefer to start method-name with '_init' or '_initAs'). Each method here is called with arguments equal to <code>initialValue</code>-call args. Initializer-settings are added as last argument
 */

/**
 * Minimal fields-list that should be in the config-object
 * There is no way to set value without any of them
 *
 * @typedef {object} configProperty
 * @property {string} name config's name
 * @property {number|string} value current value
 * @property {string} filename file name where this config is
 * @property {number|string} [recommendedValue] value which is recommended
 */

/**
 * Basic class for config-initializers
 * Each child should fill <code>initializers</code> or <code>uniqueInitializers</code> and <code>initializerTypes</code>
 * Usage:
 * <pre>
 * var myCoolInitializer = App.ConfigInitializerClass.create({
 *  initializers: {
 *    'my-cool-config': {
 *      type: 'some_type'
 *    }
 *  },
 *
 *  initializerTypes: {
 *    some_type: {
 *      method: '_initAsCool'
 *    }
 *  },
 *
 *  _initAsCool: function (configProperty, localDB, dependencies, initializer) {
 *    // some magic
 *    return configProperty;
 *  }
 * });
 *
 * var myConfig = { name: 'my-cool-config' };
 * var localDB = getLocalDB();
 * var dependencies = {};
 * myCoolInitializer.initialValue(myConfig, localDB, dependencies);
 * </pre>
 * <code>dependencies</code> - it's an object with almost any information that might be needed to set config's value. It
 * shouldn't contain App.*-flags like 'isHaEnabled' or 'isHadoop2Stack'. They might be accessed directly from App. But something
 * bigger data should be pulled to the <code>dependencies</code>.
 * Information about cluster's hosts, topology of master components should be in the <code>localDB</code>
 *
 * @type {ConfigInitializerClass}
 * @augments {Em.Object}
 */
App.ConfigInitializerClass = Em.Object.extend({

  _initializerFlowCode: {
    next: 0,
    skipNext: 1,
    skipAll: 2
  },

  concatenatedProperties: ['initializerTypes'],

  /**
   * Map with configurations for config initializers
   * It's used only for initializers which are common for some configs (if not - use <code>uniqueInitializers</code>-map)
   * Key {string} configProperty-name
   * Value {initializer|initializer[]} settings for initializer
   *
   * @type {object}
   */
  initializers: {},

  /**
   * Map with initializers types
   * It's not overridden in the child-classes (@see Ember's concatenatedProperties)
   *
   * @type {initializerType[]}
   */
  initializerTypes: [],

  /**
   * Map with initializers that are used only for one config (are unique)
   * Key: configProperty-name
   * Value: method-name
   * Every method from this map is called with same arguments as <code>initialValue</code> is (prefer to start method-name with '_init' or '_initAs')
   *
   * @type {object}
   */
  uniqueInitializers: {},

  /**
   * Wrapper for common initializers
   * Execute initializer if it is a function or throw an error otherwise
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @returns {Object}
   * @private
   */
  _defaultInitializer: function _defaultInitializer(configProperty, localDB, dependencies) {
    var args = [].slice.call(arguments);
    var self = this;
    var initializers = this.get('initializers');
    var initializerTypes = this.get('initializerTypes');
    var initializer = initializers[Em.get(configProperty, 'name')];
    if (initializer) {
      initializer = Em.makeArray(initializer);
      var i = 0;
      while (i < initializer.length) {
        var init = initializer[i];
        var _args = [].slice.call(args);
        var type = initializerTypes.findProperty('name', init.type);
        // add initializer-settings
        _args.push(init);
        var methodName = type.method;
        Em.assert('method-initializer is not a function ' + methodName, 'function' === Em.typeOf(self[methodName]));
        if (init.isChecker) {
          var result = self[methodName].apply(self, _args);
          if (result === this.flowSkipNext()) {
            i++; // skip next
          } else {
            if (result === this.flowSkipAll()) {
              break;
            }
          }
        } else {
          configProperty = self[methodName].apply(self, _args);
        }
        i++;
      }
    }
    return configProperty;
  },

  /**
   * Entry-point for any config's value initializing
   * Before calling it, be sure that <code>initializers</code> or <code>uniqueInitializers</code>
   * contains record about needed configs
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @returns {Object}
   * @method initialValue
   */
  initialValue: function initialValue(configProperty, localDB, dependencies) {
    var configName = Em.get(configProperty, 'name');
    var initializers = this.get('initializers');
    var initializer = initializers[configName];
    if (initializer) {
      return this._defaultInitializer(configProperty, localDB, dependencies);
    }

    var uniqueInitializers = this.get('uniqueInitializers');
    var uniqueInitializer = uniqueInitializers[configName];
    if (uniqueInitializer) {
      var args = [].slice.call(arguments);
      return this[uniqueInitializer].apply(this, args);
    }
    Em.set(configProperty, 'initialValue', Em.get(configProperty, 'value'));
    return configProperty;
  },

  /**
   * Should do some preparing for initializing-process
   * Shouldn't be redefined without needs
   * Should be used before any <code>initialValue</code>-calls (if needed)
   *
   * @method setup
   */
  setup: Em.K,

  /**
   * Should restore Initializer's to the initial state
   * Basically, should revert changes done in the <code>setup</code>
   *
   * @method cleanup
   */
  cleanup: Em.K,

  /**
   * Setup <code>initializers</code>
   * There are some configs with names based on other config's values.
   * Example: config with name <code>hadoop.proxyuser.{{oozieUser}}.hosts</code>
   * Here <code>{{oozieUser}}</code> is value of the config <code>['oozie-env']['oozie_user']</code>.
   * So, when <code>initializers</code> are manually set up in the Initializer-instance, there is no way to populate initializer
   * for such configs. Reason: each initializer's key is a config-name which is compared strictly to the provided config-name.
   * Solution: use config-names with placeholders. Example: <code>hadoop.proxyuser.${oozieUser}.hosts</code>
   * In this case, before calling <code>initialValue</code> this key should be updated with real value. And it should be done
   * in the common way for the all such configs.
   * Code example:
   * <pre>
   *   var mySettings = {
   *    oozieUser: 'someDude'
   *   };
   *
   *   var myCoolInitializer = App.MoveComponentConfigInitializerClass.create({
   *    initializers: {
   *      'hadoop.proxyuser.{{oozieUser}}.hosts': {
   *        // some setting are here
   *      }
   *    },
   *
   *    setup: function (settings) {
   *      this._updateInitializers(settings);
   *    },
   *
   *    cleanup: function () {
   *      this._restoreInitializers();
   *    }
   *
   *   });
   *
   *   myCoolInitializer.setup(mySettings); // after this call `myCoolInitializer.initializers` will contain two keys
   *   console.log(myCoolInitializer.initializers); // {'hadoop.proxyuser.{{oozieUser}}.hosts': {}, 'hadoop.proxyuser.someDude.hosts': {}}
   *   // it's possible to call `initialValue` now
   *   myCoolInitializer.initialValue({name: 'hadoop.proxyuser.someDude.hosts', value: ''}, {}, {});
   *   myCoolInitializer.cleanup(); // after updating values for the all configs
   * </pre>
   * Additional key in the <code>initializers</code> won't cause any issues.
   * Keep in mind replacing-rules:
   * <ul>
   *  <li>Each field in the <code>settings</code> should be a string or number</li>
   *  <li>Substring that will be replaced should be wrapped with '{{', '}}'</li>
   *  <li>Each field is searched in the each initializers-key, so try to avoid names-collision</li>
   *  <li>Value for 'new' key is the same as for 'old' ('hadoop.proxyuser.{{oozieUser}}.hosts' and 'hadoop.proxyuser.someDude.hosts' will have same initializer)</li>
   * </ul>
   * <b>Important! Be sure, that you call <code>_restoreInitializers</code> before calling <code>_updateInitializers</code> second time</b>
   *
   * @param {object} settings
   * @method _updateInitializers
   */
  _updateInitializers: function _updateInitializers(settings) {
    settings = settings || {};
    var originalInitializers = this.get('initializers');
    var copyInitializers = Em.copy(originalInitializers, true);
    this.set('__copyInitializers', copyInitializers);
    var initializers = this._updateNames('initializers', settings);
    this._setForComputed('initializers', initializers);

    var originalUniqueInitializers = this.get('uniqueInitializers');
    var copyUniqueInitializers = Em.copy(originalUniqueInitializers, true);
    this.set('__copyUniqueInitializers', copyUniqueInitializers);
    var uniqueInitializers = this._updateNames('uniqueInitializers', settings);
    this._setForComputed('uniqueInitializers', uniqueInitializers);
  },

  /**
   * Revert names changes done in the <code>_updateInitializers</code>
   *
   * @method _restoreInitializers
   */
  _restoreInitializers: function _restoreInitializers() {
    var copyInitializers = this.get('__copyInitializers');
    var copyUniqueInitializers = this.get('__copyUniqueInitializers');
    if ('object' === Em.typeOf(copyInitializers)) {
      this._setForComputed('initializers', Em.copy(copyInitializers, true));
    }
    if ('object' === Em.typeOf(copyUniqueInitializers)) {
      this._setForComputed('uniqueInitializers', Em.copy(copyUniqueInitializers, true));
    }
  },

  /**
   * Replace list of <code>settings</code> in the sourceKey
   * Example:
   * <pre>
   *   var sourceKey = '{{A}},{{B}},{{C}}';
   *   var settings = {A: 'a', B: 'b', C: 'c'};
   *   sourceKey = _updateNames(sourceKey, settings);
   *   console.log(sourceKey); // 'a,b,c'
   * </pre>
   *
   * @param {string} sourceKey
   * @param {object} settings
   * @returns {object}
   * @private
   * @method _updateNames
   */
  _updateNames: function _updateNames(sourceKey, settings) {
    settings = settings || {};
    var source = this.get(sourceKey);
    Object.keys(source).forEach(function (configName) {
      var initializer = source[configName];
      Object.keys(settings).forEach(function (key) {
        var replaceWith = settings[key];
        var toReplace = '{{' + key + '}}';
        configName = configName.replace(toReplace, replaceWith);
      });
      source[configName] = initializer;
    });
    return source;
  },

  flowNext: function flowNext() {
    return this.get('_initializerFlowCode.next');
  },

  flowSkipNext: function flowSkipNext() {
    return this.get('_initializerFlowCode.skipNext');
  },

  flowSkipAll: function flowSkipAll() {
    return this.get('_initializerFlowCode.skipAll');
  },

  /**
   * Set value for computed property using `reopen`. Currently used to update 'initializers'
   * and 'uniqueInitializers'.
   * Used to set value for props like:
   * <code>cp: function() { }.property()</code>
   * <code>
   * var obj = App.ConfigInitializerClass.create({
   *   cp: function() {
   *   		return {
   *     		key: "value"
   *   		}
   *   }.property(),
   *   setProp: function() {
   *   		this.set('cp', {newKey: "new_value"}); // will not change `cp` value
   *   },
   *   updateProp: function() {
   *   		this._setForComputed('cp', { newKey: "new_value"}); // will update
   *   }
   * });
   *
   * obj.get('cp'); // {key: "value"}
   * obj.setProp();
   * obj.get('cp'); // {key: "value"}
   * obj.updateProp();
   * obj.get('cp'); // {newKey: "new_value"}
   * </code>
   * @private
   * @param  {string} key
   * @param  {*} value
   */
  _setForComputed: function _setForComputed(key, value) {
    var obj = {};
    obj[key] = function () {
      return value;
    }.property();
    this.reopen(obj);
  }
});

});

require.register("utils/configs/control_flow_initializer_mixin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

/**
 * Mixin with preconfigured initializers that helps to build conditional execution
 * based on exit code from App.ConfigInitializerClass._initializerFlowCode.
 * Each control flow initializer should has attribute <b>isChecker: true</b>
 * Each handler should return exit code value based on App.ConfigInitializerClass._initializerFlowCode.
 *
 * There are few methods:
 * @see App.ConfigInitializerClass.flowNext
 * @see App.ConfigInitializerClass.flowSkipNext
 * @see App.ConfigInitializerClass.flowSkipAll
 *
 * For details and examples @see App.AddComponentConfigInitializer
 *
 * @mixin App.ControlFlowInitializerMixin
 */
App.ControlFlowInitializerMixin = Em.Mixin.create({

  initializerTypes: [{
    name: 'namenode_ha_enabled',
    method: '_initNameNodeHACheck'
  }, {
    name: 'resourcemanager_ha_enabled',
    method: '_initResourceManagerHACheck'
  }, {
    name: 'hdp_stack_version_checker',
    method: '_initHDPStackVersionCheck'
  }],

  /**
   * Control flow initializer based on minimal stack version.
   *
   * @param  {string} minStackVersionNumber
   * @return {object}
   */
  getHDPStackVersionControl: function getHDPStackVersionControl(minStackVersionNumber) {
    return { type: 'hdp_stack_version_checker', isChecker: true, stackVersion: minStackVersionNumber };
  },

  /**
   * getHDPStackVersionControl handler.
   * When stack version satisfies passed minStackVersionNumber computation process will continue.
   * If not all next computation will be skipped.
   *
   * @param  {configProperty} configProperty
   * @param  {topologyLocalDB} localDB
   * @param  {dependencies} dependencies
   * @param  {initializer} initializer
   * @return {number} _initializerFlowCode exit code
   */
  _initHDPStackVersionCheck: function _initHDPStackVersionCheck(configProperty, localDB, dependencies, initializer) {
    return stringUtils.compareVersions(App.get('currentStackVersionNumber'), initializer.stackVersion) > -1 ? this.flowNext() : this.flowSkipAll();
  },

  /**
   * Control flow initializer based on NameNode HA Status.
   *
   * @return {initializer}
   */
  getNameNodeHAControl: function getNameNodeHAControl() {
    return { type: 'namenode_ha_enabled', isChecker: true };
  },

  /**
   * getNameNodeHAControl handler.
   * When NameNode HA enabled next computation will be performed, either next will be skipped.
   *
   * @param  {configProperty} configProperty
   * @param  {topologyLocalDB} localDB
   * @param  {dependencies} dependencies
   * @param  {initializer} initializer
   * @return {number} _initializerFlowCode exit code
   */
  _initNameNodeHACheck: function _initNameNodeHACheck(configProperty, localDB, dependencies) {
    return App.get('isHaEnabled') ? this.flowNext() : this.flowSkipNext();
  },

  /**
   * Control flow initializer based on ResourceManager HA Status.
   *
   * @return {initializer}
   */
  getResourceManagerHAControl: function getResourceManagerHAControl(trueBranch, falseBranch) {
    return { type: 'resourcemanager_ha_enabled', isChecker: true };
  },

  /**
   * getResourceManagerHAControl handler.
   * When ResourceManager HA enabled next computation will be performed, either next will be skipped.
   *
   * @param  {configProperty} configProperty
   * @param  {topologyLocalDB} localDB
   * @param  {dependencies} dependencies
   * @param  {initializer} initializer
   * @return {number} _initializerFlowCode exit code
   */
  _initResourceManagerHACheck: function _initResourceManagerHACheck(configProperty, localDB, dependencies) {
    return App.get('isRMHaEnabled') ? this.flowNext() : this.flowSkipNext();
  }

});

});

require.register("utils/configs/database", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var validators = require('utils/validator');
var stringUtils = require('utils/string_utils');

/**
 * @typedef {parsedJDBCUrl}
 * @type {object}
 * @property {string} dbType alias name by type of database @see utils/configs/database.DB_UI_TYPE_ALIAS
 * @property {string} location parsed host name
 */

/**
 * Helper methods to process database values and properties
 * @module utils/configs/database
 */
module.exports = {
  /**
   * Database type properties with <code>options</code> attribute usually displayed as radio-button.
   * <code>options</code> attribute contains mapping of database type to special properties name such as
   * database host and database name.
   * @type {object}
   */
  dbServicePropertyMap: {
    HIVE: {
      dbType: 'hive_database',
      databaseName: 'ambari.hive.db.schema.name',
      connectionUrl: 'javax.jdo.option.ConnectionURL'
    },
    OOZIE: {
      dbType: 'oozie_database',
      connectionUrl: 'oozie.service.JPAService.jdbc.url',
      databaseName: 'oozie.db.schema.name'
    },
    RANGER: {
      dbType: 'DB_FLAVOR',
      connectionUrl: 'ranger.jpa.jdbc.url',
      databaseName: 'db_name',
      fallbackHostName: 'db_host'
    }
  },

  /**
   * Map of jdbc url patterns related to supported database types.
   *
   * @type {object}
   */
  DB_JDBC_PATTERNS: {
    mysql: 'jdbc:mysql://{0}/{1}',
    mssql: 'jdbc:sqlserver://{0};databaseName={1}',
    postgres: 'jdbc:postgresql://{0}:5432/{1}',
    derby: 'jdbc:derby:{0}/{1}',
    oracle: 'jdbc:oracle:thin:@(?:\/?\/?){0}:1521(\:|\/){1}',
    sqla: 'jdbc:sqlanywhere:host={0};database={1}'
  },

  DB_UI_TYPE_ALIAS: {
    mysql: 'mysql',
    sqlserver: 'mssql',
    postgresql: 'postgres',
    derby: 'derby',
    oracle: 'oracle',
    sqlanywhere: 'sqla'
  },

  /**
   * Setup database related properties.
   *
   * @method bootstrapDatabaseProperties
   * @param {App.ServiceConfigProperty[]} serviceConfigs
   * @param {string} [serviceName=false]
   */
  bootstrapDatabaseProperties: function bootstrapDatabaseProperties(serviceConfigs, serviceName) {
    var self = this;
    var supportedServices = Em.keys(this.dbServicePropertyMap);
    if (serviceName && !supportedServices.contains(serviceName)) return;
    var serviceNames = serviceName ? [serviceName] : serviceConfigs.mapProperty('serviceName').uniq();
    serviceNames.forEach(function (serviceName) {
      if (!supportedServices.contains(serviceName)) return;
      var configs = serviceConfigs.filterProperty('serviceName', serviceName) || [];
      var connectionConfigs = self.dbServicePropertyMap[serviceName];
      var databaseTypeProperty = configs.findProperty('name', connectionConfigs.dbType);
      if (!databaseTypeProperty) return;
      var databaseTypePropertyIndex = configs.indexOf(databaseTypeProperty);
      var generatedProperties = self.getPropsByOptions(databaseTypeProperty, configs);
      var jdbcObject = self.parseJdbcUrl(Em.get(configs.findProperty('name', connectionConfigs.connectionUrl), 'value'));
      generatedProperties.forEach(function (property) {
        if (Em.get(property, 'name').endsWith('_host')) {
          // set UI host names for each database type with value parsed from jdbc connection url
          // if value is not ip or hostname (in case of New Derby Database) for Oozie set <code>fallbackUrl</code>
          // from <code>dbServicePropertyMap</code>
          Em.setProperties(property, {
            value: jdbcObject.location || ''
          });
          self.addPropertyToServiceConfigs(property, serviceConfigs, databaseTypePropertyIndex);
        }
      });
    });
  },

  /**
   * Add UI specific property to serviceConfigObject if it does not exist, and update value for existed property.
   * This code affects properties related to `_host` and `_database` which are hardcoded on UI.
   *
   * @param {object} property - property to append/update
   * @param {App.ServiceConfigProperty[]} configs - loaded and processed service configs
   * @param {integer} index of first occurrence of database type in config
   */
  addPropertyToServiceConfigs: function addPropertyToServiceConfigs(property, configs, index) {
    var configProperty = configs.findProperty('name', Em.get(property, 'name'));
    if (configProperty) {
      Em.set(configProperty, 'value', Em.get(property, 'value'));
    } else {
      if (index) {
        configs.insertAt(index, App.ServiceConfigProperty.create(property));
      } else {
        configs.pushObject(App.ServiceConfigProperty.create(property));
      }
    }
  },

  /**
   * Get hardcoded properties from site_properties.js which UI use to display host name and database name.
   *
   * @method getPropsByOptions
   * @param {object} databaseTypeProperty - hardcoded property from site_properties.js usualy used as radiobutton
   * @param {App.ServiceConfigProperty[]} configs - loaded and processed configs
   * @returns {object[]} - hardcoded properties from site_properties.js related to database name and location
   */
  getPropsByOptions: function getPropsByOptions(databaseTypeProperty) {
    Em.assert('Property related to database type should contains `options` attribute', databaseTypeProperty.get('options'));
    return databaseTypeProperty.options.mapProperty('foreignKeys').reduce(function (p, c) {
      return p.concat(c);
    }).uniq().map(function (name) {
      return App.config.get('preDefinedSiteProperties').findProperty('name', name) || null;
    }).compact();
  },

  /**
   * Get database location from jdbc url value
   *
   * @method getDBLocationFromJDBC
   * @param {string} jdbcUrl - url to parse
   * @returns {string|null}
   */
  getDBLocationFromJDBC: function getDBLocationFromJDBC(jdbcUrl) {
    var dbProvider = this.getJDBCProviderName(jdbcUrl),
        protocol = this._makeProtocol(dbProvider),
        pattern = /^\/\//,
        url;
    if (!this.isSupportedProvider(dbProvider)) {
      return '';
    }
    if (dbProvider === 'derby') {
      return this.getDerbyPath(jdbcUrl);
    }
    url = "http://" + jdbcUrl.replace(protocol, '').replace(pattern, '');
    if (dbProvider === 'sqlserver') {
      url = url.split(';')[0];
    }
    if (dbProvider === 'oracle') {
      var matches = jdbcUrl.replace(protocol, '').match(/@(?:\/?\/?)(.+)/);
      if (matches.length) {
        var result = Em.getWithDefault(matches, '1', '').split(':')[0];
        return result === '{0}' ? '' : result;
      }
      return '';
    }
    if (dbProvider === 'sqlanywhere') {
      url = url.split(';').map(function (i) {
        return (/host=/.test(i) ? i.replace('host=', '').replace('http://', '') : null
        );
      }).compact()[0];
      return this.getHostNameByUrl('http://' + url);
    }
    url = url.split(':').slice(0, 2).join(':');
    return this.getHostNameByUrl(url);
  },

  /**
   * Return derby database path by jdbcUrl
   *
   * @param {string} jdbcUrl
   * @return {string} database path
   */
  getDerbyPath: function getDerbyPath(jdbcUrl) {
    var matches = jdbcUrl.match(new RegExp(this.DB_JDBC_PATTERNS['derby'].format('(.*)', '(.*)')));
    if (matches.length) {
      var dbLocation = Em.getWithDefault(matches, '1', '');
      if (dbLocation.startsWith('${')) {
        return Em.getWithDefault(matches, '0', '').match(/\${[^}]+}/)[0];
      }
      return dbLocation != '{0}' ? dbLocation : null;
    } else {
      return null;
    }
  },

  /**
   * Returns host name by url input
   *
   * @param {string} url
   * @returns {string} host name
   */
  getHostNameByUrl: function getHostNameByUrl(url) {
    var link = document.createElement('a');
    link.href = url;
    var hostName = link.hostname;
    link = null;
    return hostName;
  },

  _makeProtocol: function _makeProtocol(dbProvider) {
    var protocol = 'jdbc:' + dbProvider + ':';
    if (dbProvider === 'oracle') {
      return protocol + 'thin:';
    }
    return protocol;
  },

  /**
   * Returns provider name from jdbcUrl
   *
   * @param {string} jdbcUrl
   * @returns {string} provider name e.g. `jdbc:some_provider:another-opt//additional` -> `some_provider`
   */
  getJDBCProviderName: function getJDBCProviderName(jdbcUrl) {
    return jdbcUrl.split(':')[1];
  },

  /**
   * Returns true when provider supported by UI.
   *
   * @returns {boolean}
   */
  isSupportedProvider: function isSupportedProvider(dbProvider) {
    return !!this.DB_UI_TYPE_ALIAS[dbProvider];
  },

  /**
   * Determines alias value (DB_UI_TYPE_ALIAS attribute value) by provider name
   *
   * @param {string} dbProvider provider name parsed from jdbcUrl e,g. from `jdbc:mysql://` -> `mysql`
   * @returns {string} alias value used on UI.
   */
  getJDBCAlias: function getJDBCAlias(dbProvider) {
    return this.DB_UI_TYPE_ALIAS[dbProvider];
  },

  /**
   * Returns parsed info from jdbcUrl connection string
   *
   * @param {string} jdbcUrl
   * @returns {parsedJDBCUrl}
   */
  parseJdbcUrl: function parseJdbcUrl(jdbcUrl) {
    var result = {
      dbType: null,
      location: null
    };
    if (jdbcUrl === '') {
      return result;
    }
    result.dbType = this.getJDBCAlias(this.getJDBCProviderName(jdbcUrl)) || null;
    result.location = this.getDBLocationFromJDBC(jdbcUrl);
    return result;
  },

  /**
   * Convert db config value to db type string
   * @param displayName
   * @returns {String|null}
   */
  getDBType: function getDBType(displayName) {
    var type = displayName.toUpperCase().match(/DERBY|POSTGRES|ORACLE|MYSQL|MSSQL|ANYWHERE/);
    type = type && type[0];
    return type;
  }

};

});

require.register("utils/configs/ha_config_initializer_class", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');

/**
 * @type {HaConfigInitializerClass}
 * @augments {ConfigInitializerClass}
 */
App.HaConfigInitializerClass = App.ConfigInitializerClass.extend({

  initializerTypes: [{ name: 'host_with_port', method: '_initAsHostWithPort' }, { name: 'hosts_with_port', method: '_initAsHostsWithPort' }]

});

});

require.register("utils/configs/hawq_activate_standby_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');

/**
 * Initializer for configs that are updated when Hawq Standby is added
 *
 * @class {HawqActivateStandbyConfigInitializer}
 */
App.HawqActivateStandbyConfigInitializer = App.HaConfigInitializerClass.create(App.HostsBasedInitializerMixin, {

  initializers: function () {
    return {
      'hawq_master_address_host': this.getHostWithPortConfig('HAWQSTANDBY', true, '', '', '')
    };
  }.property(),

  /**
   * @override
   * @param {object} settings
   */
  setup: function setup(settings) {
    this._updateInitializers(settings);
  },

  /**
   * @override
   */
  cleanup: function cleanup() {
    this._restoreInitializers();
  }

});

});

require.register("utils/configs/hawq_ha_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');

/**
 * Initializer for configs that are updated when Hawq Standby is added
 *
 * @class {HawqHaConfigInitializer}
 */
App.HawqHaConfigInitializer = App.HaConfigInitializerClass.create(App.HostsBasedInitializerMixin, {

  initializers: function () {
    return {
      'hawq_standby_address_host': this.getHostWithPortConfig('HAWQSTANDBY', false, '', '', '')
    };
  }.property(),

  /**
   * @override
   * @param {object} settings
   */
  setup: function setup(settings) {
    this._updateInitializers(settings);
  },

  /**
   * @override
   */
  cleanup: function cleanup() {
    this._restoreInitializers();
  }

});

});

require.register("utils/configs/hosts_based_initializer_mixin", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Regexp for host with port ('hostName:1234')
 *
 * @type {string}
 */
var hostWithPort = "([\\w|\\.]*)(?=:)";

/**
 * Regexp for host with port and protocol ('://hostName:1234')
 *
 * @type {string}
 */
var hostWithPrefix = ":\/\/" + hostWithPort;

/**
 * Mixin describes host name computations initializers and handlers.
 *
 * @mixin App.HostsBasedInitializerMixin
 */
App.HostsBasedInitializerMixin = Em.Mixin.create({

  initializerTypes: [{ name: 'host_with_component', method: '_initAsHostWithComponent' }, { name: 'hosts_with_components', method: '_initAsHostsWithComponents' }, { name: 'hosts_list_with_component', method: '_initAsHostsListWithComponent' }],

  /**
   * Initializer for configs with value equal to hostName with needed component
   * Value example: 'hostName'
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @param {object} initializer
   * @returns {Object}
   * @private
   */
  _initAsHostWithComponent: function _initAsHostWithComponent(configProperty, localDB, dependencies, initializer) {
    var component = localDB.masterComponentHosts.findProperty('component', initializer.component);
    if (!component) {
      return configProperty;
    }
    if (initializer.modifier) {
      var replaceWith = Em.getWithDefault(initializer.modifier, 'prefix', '') + component.hostName + Em.getWithDefault(initializer.modifier, 'suffix', '');
      this.setRecommendedValue(configProperty, initializer.modifier.regex, replaceWith);
    } else {
      Em.setProperties(configProperty, {
        recommendedValue: component.hostName,
        value: component.hostName
      });
    }

    return configProperty;
  },

  /**
   * Settings for <code>hosts_with_components</code>-initializer
   * Used for configs with value equal to the hosts list
   * May set value as array (if <code>asArray</code> is true) or as comma-sepratated string (if <code>asArray</code> is false)
   *
   * @see _initAsHostsWithComponents
   * @param {string|string[]} components
   * @param {boolean} [asArray=false]
   * @param {boolean|undefined} [isInstalled=undefined]
   * @returns {{type: string, components: string[], asArray: boolean}}
   */
  getComponentsHostsConfig: function getComponentsHostsConfig(components, asArray, isInstalled) {
    if (1 === arguments.length) {
      asArray = false;
    }
    return {
      type: 'hosts_with_components',
      components: Em.makeArray(components),
      asArray: asArray,
      isInstalled: isInstalled
    };
  },

  /**
   * Initializer for configs with value equal to hostNames with needed components
   * May be array or comma-separated list
   * Depends on <code>initializer.asArray</code> (true - array, false - string)
   * Value example: 'hostName1,hostName2,hostName3' or ['hostName1', 'hostName2', 'hostName3']
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @param {object} initializer
   * @return {Object}
   * @private
   */
  _initAsHostsWithComponents: function _initAsHostsWithComponents(configProperty, localDB, dependencies, initializer) {
    var hostNames = localDB.masterComponentHosts.filter(function (masterComponent) {
      var hasFound = initializer.components.contains(masterComponent.component);
      if (Em.isNone(initializer.isInstalled)) {
        return hasFound;
      }
      return hasFound && masterComponent.isInstalled === initializer.isInstalled;
    }).sortProperty('hostName').mapProperty('hostName');
    if (!initializer.asArray) {
      hostNames = hostNames.uniq().join(',');
    }
    Em.setProperties(configProperty, {
      value: hostNames,
      recommendedValue: hostNames
    });
    return configProperty;
  },

  /**
   * Settings for <code>host_with_component</code>-initializer
   * Used for configs with value equal to hostName that has <code>component</code>
   * Value may be modified with if <code>withModifier</code> is true (it is by default)
   * <code>hostWithPort</code>-regexp will be used in this case
   *
   * @see _initAsHostWithComponent
   * @param {string} component
   * @param {boolean} [withModifier=true]
   * @return {object}
   */
  getSimpleComponentConfig: function getSimpleComponentConfig(component, withModifier) {
    if (arguments.length === 1) {
      withModifier = true;
    }
    var config = {
      type: 'host_with_component',
      component: component
    };
    if (withModifier) {
      config.modifier = {
        type: 'regexp',
        regex: hostWithPort
      };
    }
    return config;
  },

  /**
   * Almost the same to <code>getSimpleComponentConfig</code>, but with possibility to modify <code>replaceWith</code>-value
   * <code>prefix</code> is added before it
   * <code>suffix</code> is added after it
   * <code>hostWithPrefix</code>-regexp is used
   *
   * @see _initAsHostWithComponent
   * @param {string} component
   * @param {string} [prefix]
   * @param {string} [suffix]
   * @returns {object}
   */
  getComponentConfigWithAffixes: function getComponentConfigWithAffixes(component, prefix, suffix) {
    prefix = prefix || '';
    suffix = suffix || '';
    return {
      type: 'host_with_component',
      component: component,
      modifier: {
        type: 'regexp',
        regex: hostWithPrefix,
        prefix: prefix,
        suffix: suffix
      }
    };
  },

  /**
   * Settings for <code>host_with_port</code>-initializer
   * Used for configs with value equal to hostName where some component exists concatenated with port-value
   * Port-value is calculated according to <code>port</code> and <code>portFromDependencies</code> values
   * If <code>portFromDependencies</code> is <code>true</code>, <code>port</code>-value is used as key of the <code>dependencies</code> (where real port-value is)
   * Otherwise - <code>port</code>-value used as is
   * If calculated port-value is empty, it will be skipped (and ':' too)
   * Value also may be customized with prefix and suffix
   *
   * @param {string} component needed component
   * @param {boolean} componentExists component already exists or just going to be installed
   * @param {string} prefix=''
   * @param {string} suffix=''
   * @param {string} port
   * @param {boolean} [portFromDependencies=false]
   * @returns {{type: string, component: string, componentExists: boolean, modifier: {prefix: (string), suffix: (string)}}}
   * @method getHostWithPortConfig
   * @static
   */
  getHostWithPortConfig: function getHostWithPortConfig(component, componentExists, prefix, suffix, port, portFromDependencies) {
    if (arguments.length < 6) {
      portFromDependencies = false;
    }
    prefix = prefix || '';
    suffix = suffix || '';
    var ret = {
      type: 'host_with_port',
      component: component,
      componentExists: componentExists,
      modifier: {
        prefix: prefix,
        suffix: suffix
      }
    };
    if (portFromDependencies) {
      ret.portKey = port;
    } else {
      ret.port = port;
    }
    return ret;
  },

  /**
   * Initializer for configs with value equal to the hostName where some component exists
   * Value may be customized with prefix and suffix (see <code>initializer.modifier</code>)
   * Port-value is calculated according to <code>initializer.portKey</code> or <code>initializer.port</code> values
   * If calculated port-value is empty, it will be skipped (and ':' too)
   * Value-examples: 'SOME_COOL_PREFIXhost1:port1SOME_COOL_SUFFIX', 'host1:port2'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {object} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsHostWithPort
   */
  _initAsHostWithPort: function _initAsHostWithPort(configProperty, localDB, dependencies, initializer) {
    var hostName = localDB.masterComponentHosts.filterProperty('component', initializer.component).findProperty('isInstalled', initializer.componentExists).hostName;
    var port = this.__getPort(dependencies, initializer);
    var value = initializer.modifier.prefix + hostName + (port ? ':' + port : '') + initializer.modifier.suffix;
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  /**
   * Settings for <code>hosts_with_port</code>-initializer
   * Used for configs with value equal to the list of hostNames with port
   * Value also may be customized with prefix, suffix and delimiter between host:port elements
   * Port-value is calculated according to <code>port</code> and <code>portFromDependencies</code> values
   * If <code>portFromDependencies</code> is <code>true</code>, <code>port</code>-value is used as key of the <code>dependencies</code> (where real port-value is)
   * Otherwise - <code>port</code>-value used as is
   * If calculated port-value is empty, it will be skipped (and ':' too)
   *
   * @param {string|string[]} component hosts where this component(s) exists are used as config-value
   * @param {string} prefix='' substring added before hosts-list
   * @param {string} suffix='' substring added after hosts-list
   * @param {string} delimiter=',' delimiter between hosts in the value
   * @param {string} port if <code>portFromDependencies</code> is <code>false</code> this value is used as port for hosts
   * if <code>portFromDependencies</code> is <code>true</code> `port` is used as key in the <code>dependencies</code> to get real port-value
   * @param {boolean} portFromDependencies=false true - use <code>port</code> as key for <code>dependencies</code> to get real port-value,
   * false - use <code>port</code> as port-value
   * @returns {{type: string, component: string, modifier: {prefix: (string), suffix: (string), delimiter: (string)}}}
   * @method getHostsWithPortConfig
   * @static
   */
  getHostsWithPortConfig: function getHostsWithPortConfig(component, prefix, suffix, delimiter, port, portFromDependencies) {
    if (arguments.length < 6) {
      portFromDependencies = false;
    }
    prefix = prefix || '';
    suffix = suffix || '';
    delimiter = delimiter || ',';
    var ret = {
      type: 'hosts_with_port',
      component: component,
      modifier: {
        prefix: prefix,
        suffix: suffix,
        delimiter: delimiter
      }
    };
    if (portFromDependencies) {
      ret.portKey = port;
    } else {
      ret.port = port;
    }
    return ret;
  },

  /**
   * Initializer for configs with value equal to the list of hosts where some component exists
   * Value may be customized with prefix and suffix (see <code>initializer.modifier</code>)
   * Delimiter between hostNames also may be customized in the <code>initializer.modifier</code>
   * Port-value is calculated according to <code>initializer.portKey</code> or <code>initializer.port</code> values
   * If calculated port-value is empty, it will be skipped (and ':' too)
   * Value examples: 'SOME_COOL_PREFIXhost1:port,host2:port,host2:portSOME_COOL_SUFFIX', 'host1:port|||host2:port|||host2:port'
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsHostsWithPort
   */
  _initAsHostsWithPort: function _initAsHostsWithPort(configProperty, localDB, dependencies, initializer) {
    var hostNames, hosts;
    if (Em.isArray(initializer.component)) {
      hosts = localDB.masterComponentHosts.filter(function (masterComponent) {
        return initializer.component.contains(masterComponent.component);
      }).sortProperty('hostName');
    } else {
      hosts = localDB.masterComponentHosts.filterProperty('component', initializer.component);
    }
    if (Em.isNone(initializer.componentExists)) {
      hostNames = hosts.mapProperty('hostName');
    } else {
      hostNames = hosts.filterProperty('isInstalled', initializer.componentExists).mapProperty('hostName');
    }
    var port = this.__getPort(dependencies, initializer);
    var value = initializer.modifier.prefix + hostNames.uniq().map(function (hostName) {
      return hostName + (port ? ':' + port : '');
    }).join(initializer.modifier.delimiter) + initializer.modifier.suffix;
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  /**
   * Returns port-value from <code>dependencies</code> accorfing to <code>initializer.portKey</code> or <code>initializer.port</code> values
   *
   * @param {object} dependencies
   * @param {object} initializer
   * @returns {string|number}
   * @private
   * @method __getPort
   */
  __getPort: function __getPort(dependencies, initializer) {
    var portKey = initializer.portKey;
    if (portKey) {
      return dependencies[portKey];
    }
    return initializer.port;
  },

  /**
   *
   * @param {string} component component name
   * @param {boolean} componentExists
   * @returns {object}
   */
  getHostsListComponentConfig: function getHostsListComponentConfig(component, componentExists, delimiter) {
    return {
      type: 'hosts_list_with_component',
      component: component,
      componentExists: componentExists,
      modifier: {
        delimiter: Em.isNone(delimiter) ? ',' : delimiter
      }
    };
  },

  /**
   *
   * @param {configProperty} configProperty
   * @param {topologyLocalDB} localDB
   * @param {object} dependencies
   * @param {object} initializer
   * @returns {configProperty}
   */
  _initAsHostsListWithComponent: function _initAsHostsListWithComponent(configProperty, localDB, dependencies, initializer) {
    var hostNames = localDB.masterComponentHosts.filterProperty('component', initializer.component).filterProperty('isInstalled', initializer.componentExists).mapProperty('hostName').join(initializer.modifier.delimiter);

    Em.setProperties(configProperty, {
      value: hostNames,
      recommendedValue: hostNames
    });
    return configProperty;
  }
});

});

require.register("utils/configs/modification_handlers/misc", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
require('utils/configs/modification_handlers/modification_handler');

module.exports = App.ServiceConfigModificationHandler.create({
  serviceId: 'MISC',
  getDependentConfigChanges: function getDependentConfigChanges(changedConfig, selectedServices, allConfigs, securityEnabled) {
    var affectedProperties = [];
    var newValue = changedConfig.get("value");
    var curConfigs = "";
    var affectedPropertyName;
    var affectedPropery;
    if (changedConfig.get("name") == "hdfs_user") {
      curConfigs = allConfigs.findProperty("serviceName", "HDFS").get("configs");
      affectedPropertyName = "dfs.permissions.superusergroup";
      affectedPropery = curConfigs.findProperty("name", affectedPropertyName);
      if (affectedPropery && newValue != affectedPropery.get('value')) {
        affectedProperties.push({
          serviceName: "HDFS",
          sourceServiceName: "MISC",
          propertyName: affectedPropertyName,
          propertyDisplayName: affectedPropertyName,
          newValue: newValue,
          curValue: affectedPropery.get("value"),
          changedPropertyName: "hdfs_user",
          remove: false,
          filename: 'hdfs-site.xml'
        });
      }

      affectedPropertyName = "dfs.cluster.administrators";
      affectedPropery = curConfigs.findProperty("name", affectedPropertyName);
      if (affectedPropery && $.trim(newValue) != $.trim(affectedPropery.get("value"))) {
        affectedProperties.push({
          serviceName: "HDFS",
          sourceServiceName: "MISC",
          propertyName: affectedPropertyName,
          propertyDisplayName: affectedPropertyName,
          newValue: " " + $.trim(newValue),
          curValue: affectedPropery.get("value"),
          changedPropertyName: "hdfs_user",
          remove: false,
          filename: 'hdfs-site.xml'
        });
      }
    } else if (changedConfig.get("name") == "yarn_user") {
      curConfigs = allConfigs.findProperty('serviceName', 'YARN').get('configs');
      var currentUsers, currentGroups;
      var initialUser = changedConfig.get('initialValue');
      var newUser = newValue;
      var currentAclValue = curConfigs.findProperty("name", "yarn.admin.acl").get("value");
      var currentAclValueSplits = $.trim(currentAclValue).split(/\s+/).filter(function (i) {
        return !Em.isEmpty(i);
      });
      if (currentAclValueSplits.length == 2) {
        currentUsers = currentAclValueSplits[0];
        currentGroups = currentAclValueSplits[1];
      } else {
        currentUsers = currentAclValueSplits.length > 0 ? currentAclValueSplits.shift() : '';
        currentGroups = currentAclValueSplits.join(" ");
      }
      var currentUserList = currentUsers.split(',').filter(function (i) {
        return !Em.isEmpty(i);
      });
      if (!currentUserList.contains(newUser)) {
        var currentUserIndex = currentUserList.indexOf(initialUser);
        if (currentUserIndex > -1) {
          currentUserList.splice(currentUserIndex, 1);
        }
        if (!currentUserList.contains(newUser)) {
          currentUserList.push(newUser);
        }
        var newAclValue = $.trim(currentUserList.join(',') + ' ' + currentGroups);
        if (currentAclValue != newAclValue) {
          affectedProperties.push({
            serviceName: "YARN",
            sourceServiceName: "MISC",
            propertyName: "yarn.admin.acl",
            propertyDisplayName: "yarn.admin.acl",
            newValue: newAclValue,
            curValue: currentAclValue,
            changedPropertyName: "yarn_user",
            remove: false,
            filename: 'yarn-site.xml'
          });
        }
      }
    } else if (changedConfig.get("name") == "user_group") {
      if (!selectedServices.contains('YARN')) {
        return [];
      }
      if (selectedServices.contains("MAPREDUCE2")) {
        curConfigs = allConfigs.findProperty("serviceName", "MAPREDUCE2").get("configs");
        if ($.trim(newValue) != $.trim(curConfigs.findProperty("name", "mapreduce.cluster.administrators").get("value"))) {
          affectedProperties.push({
            serviceName: "MAPREDUCE2",
            sourceServiceName: "MISC",
            propertyName: "mapreduce.cluster.administrators",
            propertyDisplayName: "mapreduce.cluster.administrators",
            newValue: " " + $.trim(newValue),
            curValue: curConfigs.findProperty("name", "mapreduce.cluster.administrators").get("value"),
            changedPropertyName: "user_group",
            filename: 'mapred-site.xml'
          });
        }
      }
      if (selectedServices.contains("YARN")) {
        curConfigs = allConfigs.findProperty("serviceName", "YARN").get("configs");
        if (newValue != curConfigs.findProperty("name", "yarn.nodemanager.linux-container-executor.group").get("value")) {
          affectedProperties.push({
            serviceName: "YARN",
            sourceServiceName: "MISC",
            propertyName: "yarn.nodemanager.linux-container-executor.group",
            propertyDisplayName: "yarn.nodemanager.linux-container-executor.group",
            newValue: newValue,
            curValue: curConfigs.findProperty("name", "yarn.nodemanager.linux-container-executor.group").get("value"),
            changedPropertyName: "user_group",
            filename: 'yarn-site.xml'
          });
        }
      }
    }
    return affectedProperties;
  }
});

});

require.register("utils/configs/modification_handlers/modification_handler", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * Config modification handler for a given service.
 * 
 * This handler should take care of scenarios that arise from modification of a
 * config. Scenarios like updating dependency configs, or doing any special
 * processing.
 */
App.ServiceConfigModificationHandler = Em.Object.extend({

  /**
   * Service whose configs this handler will react to.
   */
  serviceId: DS.attr('string'),

  /**
   * Given a changed config, provide an array of changes to dependent configs
   * across all services. Dependent configs can be created, updated and deleted.
   * 
   *  Example: Here, setting 'hdfs_user' updates 'dfs.permissions.superusergroup'. 
   *  [
   *   {
   *     serviceName: "HDFS",
   *     sourceServiceName: "MISC",  // Cross service configs
   *     propertyName: "dfs.permissions.superusergroup",
   *     propertyDisplayName: "dfs.permissions.superusergroup",
   *     filename: 'hdfs-site',
   *     remove: false,
   *     curValue: "hdfs",
   *     newValue: "my_hdfs",
   *     changedPropertyName: "hdfs_user"
   *   }
   *  ]
   *  
   * @param changedConfig
   * @param selectedServices
   * @param allConfigs
   * @param securityEnabled
   * @return Array of dependent config updates
   */
  getDependentConfigChanges: function getDependentConfigChanges(changedConfig, selectedServices, allConfigs, securityEnabled) {
    return [];
  },

  getConfig: function getConfig(allConfigs, configName, configFilename, configServiceName) {
    return allConfigs.findProperty('serviceName', configServiceName).get('configs').find(function (config) {
      return configName == config.get('name') && (configFilename == null || configFilename == config.get('filename'));
    });
  }

});

});

require.register("utils/configs/move_component_config_initializer_class", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');

/**
 * Basic class for all Initializers used for configs which are affected by some component's moving from
 * one host to the another
 *
 * @type {ConfigInitializerClass}
 * @augments {ConfigInitializerClass}
 */
App.MoveComponentConfigInitializerClass = App.ConfigInitializerClass.extend({

  initializerTypes: [{ name: 'hosts_with_component', method: '_initAsHostsWithComponentConsideringMoving' }, { name: 'target_host', method: '_initAsTargetHost' }],

  /**
   * @override
   * @param {object} settings
   */
  setup: function setup(settings) {
    this._updateInitializers(settings);
  },

  /**
   * @override
   */
  cleanup: function cleanup() {
    this._restoreInitializers();
  },

  /**
   * Initializer for configs with vlaue equal to the hostName where component will be moved (with port)
   *
   * @param {object} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsTargetHost
   */
  _initAsTargetHost: function _initAsTargetHost(configProperty, localDB, dependencies, initializer) {
    var hostName = dependencies.targetHostName;
    var port = initializer.port;
    Em.set(configProperty, 'value', hostName + ':' + port);
    return configProperty;
  },

  /**
   * Initializer for configs with value equal to the hosts list where needed component exists
   * Value considers component-moving, so <code>targetHostName</code> will be added to the list
   * and <code>sourceHostName</code> will be removed
   * Hosts are sorted by name and linked with ','
   * Value examples: 'host1', 'host1,host2,host3'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsHostsWithComponentConsideringMoving
   */
  _initAsHostsWithComponentConsideringMoving: function _initAsHostsWithComponentConsideringMoving(configProperty, localDB, dependencies, initializer) {
    var hosts = localDB.masterComponentHosts.filterProperty('component', initializer.component).mapProperty('hostName');
    hosts = hosts.without(dependencies.sourceHostName);
    hosts.pushObject(dependencies.targetHostName);
    Em.set(configProperty, 'value', hosts.uniq().sort().join(','));
    return configProperty;
  }

});

App.MoveComponentConfigInitializerClass.reopenClass({

  /**
   * Settings for <code>target_host</code>-initializer
   * Used for configs with value equal to the new host where component is moved
   * <code>port</code> is added to the end
   *
   * @param {number|string} port
   * @returns {{type: string, port: (number|string)}}
   */
  getTargetHostConfig: function getTargetHostConfig(port) {
    port = port || '';
    return {
      type: 'target_host',
      port: port
    };
  },

  /**
   * Settings for <code>hosts_with_component</code>-initializer
   * Used for configs with value equal to the hosts where needed component exists
   * List of hosts considers moving component (source-host with be excluded, and target-host will be added)
   *
   * @param {string} component
   * @returns {{type: string, component: string}}
   */
  getHostsWithComponentConfig: function getHostsWithComponentConfig(component) {
    return {
      type: 'hosts_with_component',
      component: component
    };
  }

});

});

require.register("utils/configs/move_hive_component_config_initializer_class", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_component_config_initializer_class');

/**
 * Common class for Initializers which are used when some HIVE's component is moved from one host to another
 *
 * @type {MoveComponentConfigInitializerClass}
 * @augments {MoveComponentConfigInitializerClass}
 */
App.MoveHiveComponentConfigInitializerClass = App.MoveComponentConfigInitializerClass.extend({

  initializerTypes: [{ name: 'hive_hosts_with_components', method: '_initAsHostsWithComponentsConsideringMovedComponent' }],

  /**
   * Initializer for configs with value equal to the list of hosts where some components exist
   * This list is affected by component's moving.
   * Example: movedComponent is <code>'Component1'</code>, list of needed components is <code>['Component1', 'Component2']</code>
   * So, when hosts for each needed component will be mapped, host where 'Component1' was before moving ('host1') will be skipped
   * and host where 'Component1' is moved will be added ('host2')
   * But, if 'Component2' exists of the 'host1', 'host1' will be added to the hosts list
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsHostsWithComponentsConsideringMovedComponent
   */
  _initAsHostsWithComponentsConsideringMovedComponent: function _initAsHostsWithComponentsConsideringMovedComponent(configProperty, localDB, dependencies, initializer) {
    var allHosts = [];
    initializer.components.forEach(function (component) {
      var hosts = localDB.masterComponentHosts.filterProperty('component', component).mapProperty('hostName');
      if (component === initializer.movedComponent) {
        hosts = hosts.without(dependencies.sourceHostName);
        hosts.pushObject(dependencies.targetHostName);
      }
      allHosts = allHosts.concat(hosts);
    });
    Em.set(configProperty, 'value', allHosts.uniq().sort().join(','));
    return configProperty;
  }

});

App.MoveHiveComponentConfigInitializerClass.reopenClass({

  /**
   * Settings for <code>hive_hosts_with_components</code>-initializer
   * Used for configs with value equal to the hosts list where needed components exist.
   * This list is affected by component's moving
   *
   * @param {string|string[]} neededComponents
   * @param {string} movedComponent
   * @returns {{type: string, components: string[], movedComponent: string}}
   */
  getHostsWithComponentsConfig: function getHostsWithComponentsConfig(neededComponents, movedComponent) {
    return {
      type: 'hive_hosts_with_components',
      components: Em.makeArray(neededComponents),
      movedComponent: movedComponent
    };
  }

});

});

require.register("utils/configs/move_hm_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_hive_component_config_initializer_class');

/**
 * Initializer for configs which should be affected when Hive Metastore is moved from one host to another
 *
 * @type {MoveHiveComponentConfigInitializerClass}
 */
App.MoveHmConfigInitializer = App.MoveHiveComponentConfigInitializerClass.create({

  initializers: {
    'hadoop.proxyuser.{{hiveUser}}.hosts': App.MoveHiveComponentConfigInitializerClass.getHostsWithComponentsConfig(['HIVE_SERVER', 'HIVE_METASTORE', 'HIVE_SERVER_INTERACTIVE'], 'HIVE_METASTORE')
  },

  uniqueInitializers: {
    'hive.metastore.uris': '_initHiveMetastoreUris',
    'templeton.hive.properties': '_initTempletonHiveProperties'
  },

  /**
   * Unique initializer for <code>hive.metastore.uris</code>-config
   * Value example: 'thrift://host1:1234,thrift://host2:1234,thrift://host3:1234'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initHiveMetastoreUris
   */
  _initHiveMetastoreUris: function _initHiveMetastoreUris(configProperty, localDB, dependencies) {
    var hiveMSHosts = this.__getHmHostsConsideringMoved(localDB, dependencies);

    var value = Em.get(configProperty, 'value');

    var port = value.match(/:[0-9]{2,4}/);
    port = port ? port[0].slice(1) : '9083';

    value = hiveMSHosts.uniq().map(function (hiveMSHost) {
      return 'thrift://' + hiveMSHost + ':' + port;
    }).join(',');

    Em.set(configProperty, 'value', value);
    return configProperty;
  },

  /**
   * Unique initializer for <code>templeton.hive.properties</code>-config
   * Replace existing hosts with new
   * Value example: 'hive.metastore.local=false,hive.metastore.uris=thrift://host1:9083,hive.metastore.sasl.enabled=false,hive.metastore.execute.setugi=true'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initTempletonHiveProperties
   */
  _initTempletonHiveProperties: function _initTempletonHiveProperties(configProperty, localDB, dependencies) {
    var hiveMSHosts = this.__getHmHostsConsideringMoved(localDB, dependencies);
    var value = Em.get(configProperty, 'value');
    value = value.replace(/thrift.+[0-9]{2,},/i, hiveMSHosts.join('\\,') + ',');

    Em.set(configProperty, 'value', value);
    return configProperty;
  },

  /**
   * Get list of hosts where HIVE_METASTORE exists considering component's moving (host where it was is removed and host
   * where it will be is added)
   *
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {string[]}
   * @private
   * @method __getHmHostsConsideringMoved
   */
  __getHmHostsConsideringMoved: function __getHmHostsConsideringMoved(localDB, dependencies) {
    var hiveMSHosts = localDB.masterComponentHosts.filterProperty('component', 'HIVE_METASTORE').mapProperty('hostName');
    hiveMSHosts = hiveMSHosts.removeObject(dependencies.sourceHostName).addObject(dependencies.targetHostName);
    return hiveMSHosts.uniq();
  }

});

});

require.register("utils/configs/move_hs_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_hive_component_config_initializer_class');

/**
 * Initializer for configs which should be affected when Hive Server is moved from one host to another
 *
 * @type {MoveHiveComponentConfigInitializerClass}
 */
App.MoveHsConfigInitializer = App.MoveHiveComponentConfigInitializerClass.create({

  initializers: {
    'hadoop.proxyuser.{{hiveUser}}.hosts': App.MoveHiveComponentConfigInitializerClass.getHostsWithComponentsConfig(['HIVE_SERVER', 'HIVE_METASTORE', 'HIVE_SERVER_INTERACTIVE'], 'HIVE_SERVER')
  }

});

});

require.register("utils/configs/move_namenode_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_component_config_initializer_class');

/**
 * Initializer for configs which should be affected when NameNode is moved from one host to another
 * If NameNode HA-mode is already activated, several configs are also updated
 *
 * @instance MoveComponentConfigInitializerClass
 */
App.MoveNameNodeConfigInitializer = App.MoveComponentConfigInitializerClass.create({

  initializers: {
    'dfs.namenode.http-address.{{namespaceId}}.{{suffix}}': App.MoveComponentConfigInitializerClass.getTargetHostConfig(50070),
    'dfs.namenode.https-address.{{namespaceId}}.{{suffix}}': App.MoveComponentConfigInitializerClass.getTargetHostConfig(50470),
    'dfs.namenode.rpc-address.{{namespaceId}}.{{suffix}}': App.MoveComponentConfigInitializerClass.getTargetHostConfig(8020),
    'dfs.namenode.servicerpc-address.{{namespaceId}}.{{suffix}}': App.MoveComponentConfigInitializerClass.getTargetHostConfig(8021),
    'dfs.namenode.lifeline.rpc-address.{{namespaceId}}.{{suffix}}': App.MoveComponentConfigInitializerClass.getTargetHostConfig(8050)
  },

  uniqueInitializers: {
    'instance.volumes': '_initInstanceVolumes',
    'instance.volumes.replacements': '_initInstanceVolumesReplacements',
    'hbase.rootdir': '_initHbaseRootDir',
    'hawq_dfs_url': '_initHawqDfsUrl'
  },

  /**
   * Unique initializer for <code>instance.volumes</code>-config
   * Value example: 'hdfs://host1:8020/apps/accumulo/data'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initInstanceVolumes
   */
  _initInstanceVolumes: function _initInstanceVolumes(configProperty, localDB, dependencies) {
    if (!App.get('isHaEnabled') && localDB.installedServices.contains('ACCUMULO')) {
      var value = Em.get(configProperty, 'value');
      value = value.replace(/\/\/[^\/]*/, '//' + dependencies.targetHostName + ':8020');
      Em.set(configProperty, 'value', value);
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>instance.volumes.replacements</code>-config
   * Value example: 'hdfs://host1:8020/apps/accumulo/data hdfs://host2:8020/apps/accumulo/data'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initInstanceVolumesReplacements
   */
  _initInstanceVolumesReplacements: function _initInstanceVolumesReplacements(configProperty, localDB, dependencies) {
    if (!App.get('isHaEnabled') && localDB.installedServices.contains('ACCUMULO')) {
      var target = 'hdfs://' + dependencies.targetHostName + ':8020' + '/apps/accumulo/data';
      var source = 'hdfs://' + dependencies.sourceHostName + ':8020' + '/apps/accumulo/data';
      var value = source + ' ' + target;
      Em.set(configProperty, 'value', value);
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>hbase.rootdir</code>-config (for HIVE service)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initHbaseRootDir
   */
  _initHbaseRootDir: function _initHbaseRootDir(configProperty, localDB, dependencies) {
    if (!App.get('isHaEnabled') && localDB.installedServices.contains('HBASE') && 'hbase-site' === configProperty.filename) {
      var value = Em.get(configProperty, 'value');
      value = value.replace(/\/\/[^\/]*/, '//' + dependencies.targetHostName + ':8020');
      Em.set(configProperty, 'value', value);
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>hawq_dfs_url</code>-config (for HAWQ service)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @returns {object}
   * @private
   * @method _initHawqDfsUrl
   */
  _initHawqDfsUrl: function _initHawqDfsUrl(configProperty, localDB, dependencies) {
    if (!App.get('isHaEnabled') && localDB.installedServices.contains('HAWQ') && 'hawq-site' === configProperty.filename) {
      var value = Em.get(configProperty, 'value');
      value = value.replace(/(.*):/, dependencies.targetHostName + ':');
      Em.set(configProperty, 'value', value);
    }
    return configProperty;
  }

});

});

require.register("utils/configs/move_os_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_component_config_initializer_class');

/**
 * Initializer for configs which should be affected when Oozie Server is moved from one host to another
 *
 * @instance MoveComponentConfigInitializerClass
 */
App.MoveOSConfigInitializer = App.MoveComponentConfigInitializerClass.create({

  initializers: {
    'hadoop.proxyuser.{{oozieUser}}.hosts': App.MoveComponentConfigInitializerClass.getHostsWithComponentConfig('OOZIE_SERVER')
  }

});

});

require.register("utils/configs/move_rm_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_component_config_initializer_class');

/**
 * Settings for <code>rm_ha_depended</code>-initializer
 * Used for configs with value equal to the host name (host where component is moved)
 *
 * @param {boolean} rmHaShouldBeEnabled
 * @returns {{type: string, rmHaShouldBeEnabled: boolean}}
 */
function getRmHaDependedConfig(rmHaShouldBeEnabled) {
  return {
    type: 'rm_ha_depended',
    rmHaShouldBeEnabled: Boolean(rmHaShouldBeEnabled)
  };
}

function getRmHaHawqConfig(rmHaShouldBeEnabled) {
  return {
    type: 'rm_ha_hawq',
    rmHaShouldBeEnabled: Boolean(rmHaShouldBeEnabled)
  };
}

/**
 * Initializer for configs which should be affected when Resource Manager is moved from one host to another
 * If Resource Manager HA-mode is already activated, several configs are also updated
 *
 * @instance MoveComponentConfigInitializerClass
 */
App.MoveRmConfigInitializer = App.MoveComponentConfigInitializerClass.create({

  initializerTypes: [{ name: 'rm_ha_depended', method: '_initAsRmHaDepended' }, { name: 'rm_ha_hawq', method: '_initAsRmHaHawq' }],

  initializers: {
    'yarn.resourcemanager.hostname.{{suffix}}': getRmHaDependedConfig(true),
    'yarn.resourcemanager.webapp.address.{{suffix}}': getRmHaDependedConfig(true),
    'yarn.resourcemanager.webapp.https.address.{{suffix}}': getRmHaDependedConfig(true),
    'yarn.resourcemanager.resource-tracker.address.{{suffix}}': getRmHaDependedConfig(true),
    'yarn.resourcemanager.address.{{suffix}}': getRmHaDependedConfig(true),
    'yarn.resourcemanager.ha': getRmHaHawqConfig(true),
    'yarn.resourcemanager.scheduler.ha': getRmHaHawqConfig(true)
  },

  /**
   * Initializer for configs with value equal to the target hostName
   * and based on <code>App.isRMHaEnabled</code>
   * Value example: 'host1:port1'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {reassignComponentDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsRmHaDepended
   */
  _initAsRmHaDepended: function _initAsRmHaDepended(configProperty, localDB, dependencies, initializer) {
    if (App.get('isRMHaEnabled') === initializer.rmHaShouldBeEnabled) {
      var value = Em.get(configProperty, 'value');
      var parts = value.split(':');
      parts[0] = dependencies.targetHostName;
      Em.set(configProperty, 'value', parts.join(':'));
    }
    return configProperty;
  },

  /**
   * This function to generate "rm1_host:port,rm2_host:port" for HAWQ configuration (RM HA specific)
   * configProperty = {
   *   filename: "yarn-client",
   *   name: "yarn.resourcemanager.ha",
   *   value: "ip-10-32-36-162.ore1.vpc.pivotal.io:8050"
   * }
   *
   * dependencies = {
   *   rm1: "ip-10-32-38-225.ore1.vpc.pivotal.io",
   *   rm2: "ip-10-32-38-162.ore1.vpc.pivotal.io",
   *   sourceHostName: "ip-10-32-36-162.ore1.vpc.pivotal.io",
   *   targetHostName: "ip-10-32-38-34.ore1.vpc.pivotal.io"
   * }
   *
   * dependencies is defined by ReassignMasterWizardStep4Controller._getRmAdditionalDependencies()
   * Move RM Wizard has 2 combobox in HA case, but it seems to gurantee users can change only one of them.
   * (Thus, there is only one pair of sourceHostName and targetHostName)
   *
   */
  _initAsRmHaHawq: function _initAsRmHaHawq(configProperty, localDB, dependencies, initializer) {
    if (App.get('isRMHaEnabled') === initializer.rmHaShouldBeEnabled) {

      // which of rm1 and rm2 is changed?
      var rm1, rm2;
      if (dependencies.rm1 === dependencies.sourceHostName) {
        rm1 = dependencies.targetHostName;
        rm2 = dependencies.rm2;
      } else {
        rm1 = dependencies.rm1;
        rm2 = dependencies.targetHostName;
      }

      // https://github.com/apache/hadoop/blob/release-2.7.1/hadoop-yarn-project/hadoop-yarn/hadoop-yarn-api/src/main/java/org/apache/hadoop/yarn/conf/YarnConfiguration.java#L133-L163
      var DEFAULT_RM_PORT = 8032;
      var DEFAULT_RM_SCHEDULER_PORT = 8030;

      if (configProperty.name === "yarn.resourcemanager.ha") {
        var newValue = rm1 + ":" + DEFAULT_RM_PORT + "," + rm2 + ":" + DEFAULT_RM_PORT;
      } else {
        var newValue = rm1 + ":" + DEFAULT_RM_SCHEDULER_PORT + "," + rm2 + ":" + DEFAULT_RM_SCHEDULER_PORT;
      }

      Em.set(configProperty, 'value', newValue);
    }
    return configProperty;
  }

});

});

require.register("utils/configs/move_ws_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/move_hive_component_config_initializer_class');

/**
 * Initializer for configs which should be affected when Hive Server is moved from one host to another
 *
 * @type {MoveHiveComponentConfigInitializerClass}
 */
App.MoveWsConfigInitializer = App.MoveHiveComponentConfigInitializerClass.create({

  initializers: {
    'hadoop.proxyuser.{{webhcatUser}}.hosts': App.MoveHiveComponentConfigInitializerClass.getHostsWithComponentsConfig(['WEBHCAT_SERVER'], 'WEBHCAT_SERVER')
  }

});

});

require.register("utils/configs/nn_ha_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/ha_config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');

/**
 * @typedef {topologyLocalDB} extendedTopologyLocalDB
 * @property {string[]} installedServices list of installed service names
 */

/**
 * Setting for <code>rename</code>-initializer
 * Used for configs which should be renamed
 * Replace some part if their names with <code>namespaceId</code> (provided by user on the wizard's 1st step)
 *
 * @param {string} toReplace
 * @returns {{type: string, toReplace: string}}
 */
function getRenameWithNamespaceConfig(toReplace) {
  return {
    type: 'rename',
    toReplace: toReplace
  };
}

/**
 * Settings for <code>namespace</code>-initializer
 * Used for configs with value equal to the <code>namespaceId</code> (provided by user on the wizard's 1st step)
 * Value may be customized with prefix and suffix
 *
 * @param {string} [prefix=''] substring added before namespace in the replace
 * @param {string} [suffix=''] substring added after namespace in the replace
 * @returns {{type: string, modifier: {prefix: (string), suffix: (string)}}}
 */
function getNamespaceConfig(prefix, suffix) {
  prefix = prefix || '';
  suffix = suffix || '';
  return {
    type: 'namespace',
    modifier: {
      prefix: prefix,
      suffix: suffix
    }
  };
}

/**
 * Settings for <code>replace_namespace</code>
 * Used for configs with values that have to be modified with replacing some part of them
 * to the <code>namespaceId</code> (provided by user on the wizard's 1st step)
 *
 * @param {string} toReplace
 * @returns {{type: string, toReplace: *}}
 */
function getReplaceNamespaceConfig(toReplace) {
  return {
    type: 'replace_namespace',
    toReplace: toReplace
  };
}

/**
 * Initializer for configs that are updated when NameNode HA-mode is activated
 *
 * @class {NnHaConfigInitializer}
 */
App.NnHaConfigInitializer = App.HaConfigInitializerClass.create(App.HostsBasedInitializerMixin, {

  initializers: function () {

    return {
      'dfs.ha.namenodes.${dfs.nameservices}': getRenameWithNamespaceConfig('${dfs.nameservices}'),
      'dfs.namenode.rpc-address.${dfs.nameservices}.nn1': [this.getHostWithPortConfig('NAMENODE', true, '', '', 'nnRpcPort', true), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.namenode.rpc-address.${dfs.nameservices}.nn2': [this.getHostWithPortConfig('NAMENODE', false, '', '', '8020', false), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.namenode.http-address.${dfs.nameservices}.nn1': [this.getHostWithPortConfig('NAMENODE', true, '', '', 'nnHttpPort', true), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.namenode.http-address.${dfs.nameservices}.nn2': [this.getHostWithPortConfig('NAMENODE', false, '', '', '50070', false), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.namenode.https-address.${dfs.nameservices}.nn1': [this.getHostWithPortConfig('NAMENODE', true, '', '', 'nnHttpsPort', true), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.namenode.https-address.${dfs.nameservices}.nn2': [this.getHostWithPortConfig('NAMENODE', false, '', '', '50470', false), getRenameWithNamespaceConfig('${dfs.nameservices}')],
      'dfs.client.failover.proxy.provider.${dfs.nameservices}': getRenameWithNamespaceConfig('${dfs.nameservices}'),
      'dfs.nameservices': getNamespaceConfig(),
      'dfs.internal.nameservices': getNamespaceConfig(),
      'fs.defaultFS': getNamespaceConfig('hdfs://'),
      'dfs.namenode.shared.edits.dir': [this.getHostsWithPortConfig('JOURNALNODE', 'qjournal://', '/${dfs.nameservices}', ';', '8485', false), getReplaceNamespaceConfig('${dfs.nameservices}')],
      'ha.zookeeper.quorum': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true)
    };
  }.property(),

  uniqueInitializers: {
    'hbase.rootdir': '_initHbaseRootDir',
    'hawq_dfs_url': '_initHawqDfsUrl',
    'instance.volumes': '_initInstanceVolumes',
    'instance.volumes.replacements': '_initInstanceVolumesReplacements',
    'dfs.journalnode.edits.dir': '_initDfsJnEditsDir',
    'xasecure.audit.destination.hdfs.dir': '_initXasecureAuditDestinationHdfsDir'
  },

  initializerTypes: [{ name: 'rename', method: '_initWithRename' }, { name: 'namespace', method: '_initAsNamespace' }, { name: 'replace_namespace', method: '_initWithNamespace' }],

  /**
   * Initializer for configs that should be renamed
   * Some part of their names should be replaced with <code>namespaceId</code> (user input this value on the wizard's 1st step)
   * Affects both - name and displayName
   * <b>Important! It's not the same as <code>_updateInitializers</code>!</b>
   * Main diff - this initializer used for configs
   * with names that come with some "predicates" in their names. <code>_updateInitializers</code> is used to determine needed
   * config name that depends on other config values or someting else
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initWithRename
   */
  _initWithRename: function _initWithRename(configProperty, localDB, dependencies, initializer) {
    var replaceWith = dependencies.namespaceId;
    var toReplace = initializer.toReplace;
    Em.assert('`dependencies.namespaceId` should be not empty string', !!replaceWith);
    var name = Em.getWithDefault(configProperty, 'name', '');
    var displayName = Em.getWithDefault(configProperty, 'displayName', '');
    name = name.replace(toReplace, replaceWith);
    displayName = displayName.replace(toReplace, replaceWith);
    Em.setProperties(configProperty, {
      name: name,
      displayName: displayName
    });
    return configProperty;
  },

  /**
   * Initializer for configs wih value equal to the <code>namespaceId</code> (user input this value on the wizard's 1st step)
   * Value may be customized with prefix and suffix (see <code>initializer.modifier</code>)
   * Value-examples: 'SOME_COOL_PREFIXmy_namespaceSOME_COOL_SUFFIX', 'my_namespace'
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initAsNamespace
   */
  _initAsNamespace: function _initAsNamespace(configProperty, localDB, dependencies, initializer) {
    var value = dependencies.namespaceId;
    Em.assert('`dependencies.namespaceId` should be not empty string', !!value);
    value = initializer.modifier.prefix + value + initializer.modifier.suffix;
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: value
    });
    return configProperty;
  },

  /**
   * Initializer for configs with value that should be modified with replacing some substring
   * to the <code>namespaceId</code> (user input this value on the wizard's 1st step)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initWithNamespace
   */
  _initWithNamespace: function _initWithNamespace(configProperty, localDB, dependencies, initializer) {
    var replaceWith = dependencies.namespaceId;
    var toReplace = initializer.toReplace;
    Em.assert('`dependencies.namespaceId` should be not empty string', !!replaceWith);
    var value = Em.get(configProperty, 'value').replace(toReplace, replaceWith);
    var recommendedValue = Em.get(configProperty, 'recommendedValue').replace(toReplace, replaceWith);
    Em.setProperties(configProperty, {
      value: value,
      recommendedValue: recommendedValue
    });
    return configProperty;
  },

  /**
   * Unique initializer for <code>hbase.rootdir</code>
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initHbaseRootDir
   * @return {object}
   * @private
   */
  _initHbaseRootDir: function _initHbaseRootDir(configProperty, localDB, dependencies, initializer) {
    var fileName = Em.get(configProperty, 'filename');
    var args = [].slice.call(arguments);
    if ('hbase-site' === fileName) {
      return this._initHbaseRootDirForHbase.apply(this, args);
    }
    if ('ams-hbase-site' === fileName) {
      return this._initHbaseRootDirForAMS.apply(this, args);
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>hbase.rootdir</code> (HBASE-service)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initHbaseRootDirForHbase
   * @return {object}
   * @private
   */
  _initHbaseRootDirForHbase: function _initHbaseRootDirForHbase(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('HBASE')) {
      var value = dependencies.serverConfigs.findProperty('type', 'hbase-site').properties['hbase.rootdir'].replace(/\/\/[^\/]*/, '//' + dependencies.namespaceId);
      Em.setProperties(configProperty, {
        value: value,
        recommendedValue: value
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>hbase.rootdir</code> (Ambari Metrics-service)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initHbaseRootDirForAMS
   * @return {object}
   * @private
   */
  _initHbaseRootDirForAMS: function _initHbaseRootDirForAMS(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('AMBARI_METRICS')) {
      var value = dependencies.serverConfigs.findProperty('type', 'ams-hbase-site').properties['hbase.rootdir'];
      var currentNameNodeHost = localDB.masterComponentHosts.filterProperty('component', 'NAMENODE').findProperty('isInstalled', true).hostName;
      if (value.contains("hdfs://" + currentNameNodeHost)) {
        value = value.replace(/\/\/[^\/]*/, '//' + dependencies.namespaceId);
        configProperty.isVisible = true;
      }
      Em.setProperties(configProperty, {
        value: value,
        recommendedValue: value
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>hawq_dfs_url</code> (Hawq service)
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initHawqDfsUrl
   * @return {object}
   * @private
   */
  _initHawqDfsUrl: function _initHawqDfsUrl(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('HAWQ')) {
      var value = dependencies.serverConfigs.findProperty('type', 'hawq-site').properties['hawq_dfs_url'].replace(/(^.*:[0-9]+)(?=\/)/, dependencies.namespaceId);
      Em.setProperties(configProperty, {
        value: value,
        recommendedValue: value
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>instance.volumes</code>
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initInstanceVolumes
   * @return {object}
   * @private
   */
  _initInstanceVolumes: function _initInstanceVolumes(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('ACCUMULO')) {
      var oldValue = dependencies.serverConfigs.findProperty('type', 'accumulo-site').properties['instance.volumes'];
      var value = oldValue.replace(/\/\/[^\/]*/, '//' + dependencies.namespaceId);
      Em.setProperties(configProperty, {
        value: value,
        recommendedValue: value
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>instance.volumes.replacements</code>
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initInstanceVolumesReplacements
   * @return {object}
   * @private
   */
  _initInstanceVolumesReplacements: function _initInstanceVolumesReplacements(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('ACCUMULO')) {
      var oldValue = dependencies.serverConfigs.findProperty('type', 'accumulo-site').properties['instance.volumes'];
      var value = oldValue.replace(/\/\/[^\/]*/, '//' + dependencies.namespaceId);
      var replacements = oldValue + " " + value;
      Em.setProperties(configProperty, {
        value: replacements,
        recommendedValue: replacements
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>dfs.journalnode.edits.dir</code>
   * Used only for Windows Stacks
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initDfsJnEditsDir
   * @return {object}
   * @private
   */
  _initDfsJnEditsDir: function _initDfsJnEditsDir(configProperty, localDB, dependencies, initializer) {
    if (App.get('isHadoopWindowsStack') && localDB.installedServices.contains('HDFS')) {
      var value = dependencies.serverConfigs.findProperty('type', 'hdfs-site').properties['dfs.journalnode.edits.dir'];
      Em.setProperties(configProperty, {
        value: value,
        recommendedValue: value
      });
    }
    return configProperty;
  },

  /**
   * Unique initializer for <code>xasecure.audit.destination.hdfs.dir</code>
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {nnHaConfigDependencies} dependencies
   * @param {object} initializer
   * @method _initXasecureAuditDestinationHdfsDir
   * @return {object}
   * @private
   */
  _initXasecureAuditDestinationHdfsDir: function _initXasecureAuditDestinationHdfsDir(configProperty, localDB, dependencies, initializer) {
    if (localDB.installedServices.contains('RANGER')) {
      var oldValue = dependencies.serverConfigs.findProperty('type', 'ranger-env').properties['xasecure.audit.destination.hdfs.dir'];
      // Example of value - hdfs://c6401.ambari.apache.org:8020/ranger/audit
      // Replace hostname and port with Namespace
      var valueArray = oldValue.split("/");
      valueArray[2] = dependencies.namespaceId;
      var newValue = valueArray.join("/");
      Em.setProperties(configProperty, {
        value: newValue,
        recommendedValue: newValue
      });
    }
    return configProperty;
  }

});

});

require.register("utils/configs/rm_ha_config_initializer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/configs/config_initializer_class');
require('utils/configs/hosts_based_initializer_mixin');

/**
 * Settings for <code>rm_hosts_with_port</code> initializer
 * Used for configs that have to be updated with Yarn Resourcemanager HA hosts
 * addresses and a port to the <code>rmhost1:port,rmhost2:port</code>
 *
 * @param {int} port
 * @returns {{type: string, port: *}}
 */
function getRmHaHostsWithPort(port) {
  return {
    type: 'rm_hosts_with_port',
    port: port
  };
}

/**
 * Initializer for configs that are updated when Resource Manager HA-mode is activated
 *
 * @class {RmHaConfigInitializer}
 */
App.RmHaConfigInitializer = App.HaConfigInitializerClass.create(App.HostsBasedInitializerMixin, {

  initializers: function () {
    return {
      'yarn.resourcemanager.hostname.rm1': this.getHostWithPortConfig('RESOURCEMANAGER', true, '', '', ''),
      'yarn.resourcemanager.hostname.rm2': this.getHostWithPortConfig('RESOURCEMANAGER', false, '', '', ''),
      'yarn.resourcemanager.zk-address': this.getHostsWithPortConfig('ZOOKEEPER_SERVER', '', '', ',', 'zkClientPort', true),
      'yarn.resourcemanager.resource-tracker.address.rm1': this.getHostWithPortConfig('RESOURCEMANAGER', true, '', '', 'trackerAddressPort', true),
      'yarn.resourcemanager.resource-tracker.address.rm2': this.getHostWithPortConfig('RESOURCEMANAGER', false, '', '', 'trackerAddressPort', true),
      'yarn.resourcemanager.webapp.address.rm1': this.getHostWithPortConfig('RESOURCEMANAGER', true, '', '', 'webAddressPort', true),
      'yarn.resourcemanager.webapp.address.rm2': this.getHostWithPortConfig('RESOURCEMANAGER', false, '', '', 'webAddressPort', true),
      'yarn.resourcemanager.webapp.https.address.rm1': this.getHostWithPortConfig('RESOURCEMANAGER', true, '', '', 'httpsWebAddressPort', true),
      'yarn.resourcemanager.webapp.https.address.rm2': this.getHostWithPortConfig('RESOURCEMANAGER', false, '', '', 'httpsWebAddressPort', true),
      'yarn.resourcemanager.ha': getRmHaHostsWithPort(8032),
      'yarn.resourcemanager.scheduler.ha': getRmHaHostsWithPort(8030),
      'hadoop.proxyuser.{{yarnUser}}.hosts': this.getComponentsHostsConfig(['RESOURCEMANAGER'])
    };
  }.property(),

  initializerTypes: [{ name: 'rm_hosts_with_port', method: '_initRmHaHostsWithPort' }],

  /**
   * @override
   * @param {object} settings
   */
  setup: function setup(settings) {
    this._updateInitializers(settings);
  },

  /**
   * @override
   */
  cleanup: function cleanup() {
    this._restoreInitializers();
  },

  /**
   * Initializer for configs that should be updated with yarn resourcemanager ha host addresses with port
   *
   * @param {configProperty} configProperty
   * @param {extendedTopologyLocalDB} localDB
   * @param {HaConfigDependencies} dependencies
   * @param {object} initializer
   * @returns {object}
   * @private
   * @method _initRmHaHostsWithPort
   */
  _initRmHaHostsWithPort: function _initRmHaHostsWithPort(configProperty, localDB, dependencies, initializer) {
    var rmHosts = localDB.masterComponentHosts.filterProperty('component', 'RESOURCEMANAGER').getEach('hostName');
    for (var rmHost in rmHosts) {
      rmHosts[rmHost] = rmHosts[rmHost] + ":" + initializer.port;
    }
    var value = rmHosts.join(',');
    Em.setProperties(configProperty, {
      'value': value,
      'recommendedValue': value
    });
    return configProperty;
  }

});

});

require.register("utils/configs/theme/theme", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.configTheme = Em.Object.create({

  /**
   * Resolve config theme conditions
   * in order to correctly calculate config errors number of service
   * @param {Array} configs
   */
  resolveConfigThemeConditions: function resolveConfigThemeConditions(configs) {
    App.ThemeCondition.find().forEach(function (configCondition) {
      var _configs = Em.A(configCondition.get('configs'));
      if (configCondition.get("resource") === 'config' && _configs.length > 0) {
        var isConditionTrue = this.calculateConfigCondition(configCondition.get("if"), configs);
        var action = isConditionTrue ? configCondition.get("then") : configCondition.get("else");
        if (configCondition.get('id')) {
          var valueAttributes = action.property_value_attributes;
          if (valueAttributes && !Em.none(valueAttributes.visible)) {
            var themeResource = this.getThemeResource(configCondition);
            if (themeResource) {
              themeResource.get('configProperties').forEach(function (_configId) {
                configs.forEach(function (item) {
                  if (App.config.configId(item.get('name'), item.get('filename')) === _configId) {
                    // if config has already been hidden by condition with "subsection" or "subsectionTab" type
                    // then ignore condition of "config" type
                    if (configCondition.get('type') === 'config' && item.get('hiddenBySection')) return false;
                    item.set('hiddenBySection', !valueAttributes.visible);
                  }
                }, this);
              }, this);
            }
          }
        }
      }
    }, this);
  },

  /**
   *
   * @param {App.ThemeCondition} configCondition
   * @returns {?Em.Object}
   */
  getThemeResource: function getThemeResource(configCondition) {
    var themeResource = null;

    if (configCondition.get('type') === 'subsection') {
      themeResource = App.SubSection.find().findProperty('name', configCondition.get('name'));
    } else if (configCondition.get('type') === 'subsectionTab') {
      themeResource = App.SubSectionTab.find().findProperty('name', configCondition.get('name'));
    } else if (configCondition.get('type') === 'config') {
      //simulate section wrapper for condition type "config"
      themeResource = Em.Object.create({
        configProperties: [App.config.configId(configCondition.get('configName'), configCondition.get('fileName'))]
      });
    }
    return themeResource;
  },

  /**
   *
   * @param {Array} configs
   * @param {?Array} storedConfigs
   * @returns {{add: Array, delete: Array}}
   */
  getConfigThemeActions: function getConfigThemeActions(configs, storedConfigs) {
    //config actions for changed configs should be only effective
    var configActions = this.getConfigActions(configs, storedConfigs);

    var componentsToAdd = [];
    var componentsToDelete = [];
    configActions.forEach(function (_action) {
      var isConditionTrue = this.calculateConfigCondition(_action.get('if'), configs);
      var action = isConditionTrue ? _action.get("then") : _action.get("else");
      switch (action) {
        case 'add':
          componentsToAdd.push(_action.get('hostComponent'));
          break;
        case 'delete':
          componentsToDelete.push(_action.get('hostComponent'));
          break;
      }
    }, this);

    return {
      add: componentsToAdd,
      delete: componentsToDelete
    };
  },

  /**
   *
   * @param {Array} configs
   * @param {Array} storedConfigs
   * @returns {Array}
   */
  getConfigActions: function getConfigActions(configs, storedConfigs) {
    return App.ConfigAction.find().filter(function (item) {
      var isAnyConfigAbsent = false;
      var configChanged = false;
      item.get("configs").forEach(function (_config) {
        var config = configs.filterProperty('filename', _config.fileName).findProperty('name', _config.configName);
        if (!config) {
          isAnyConfigAbsent = true;
        } else {
          configChanged = configChanged || config.get('value') !== (config.get('savedValue') || config.get('recommendedValue'));
          var storedConfig = storedConfigs.filterProperty('filename', _config.fileName).findProperty('name', _config.configName);
          if (storedConfig) {
            configChanged = configChanged || config.get('value') !== storedConfig.value;
          }
        }
      }, this);
      return !isAnyConfigAbsent && configChanged;
    }, this);
  },

  /**
   * ifStatement format: ${site-name/config-name}
   * @param {string} ifStatement
   * @param {Array} serviceConfigs
   * @returns {boolean}
   */
  calculateConfigCondition: function calculateConfigCondition(ifStatement, serviceConfigs) {
    // Split `if` statement if it has logical operators
    var ifStatementRegex = /(&&|\|\|)/;
    var IfConditions = ifStatement.split(ifStatementRegex);
    var allConditionResult = [];
    IfConditions.forEach(function (_condition) {
      var condition = _condition.trim();
      if (condition === '&&' || condition === '||') {
        allConditionResult.push(_condition);
      } else {
        var splitIfCondition = condition.split('===');
        var ifCondition = splitIfCondition[0];
        var result = splitIfCondition[1] || "true";
        var parseIfConditionVal = ifCondition;
        var regex = /\$\{.*?\}/g;
        var configStrings = ifCondition.match(regex);
        configStrings.forEach(function (_configString) {
          var configObject = _configString.substring(2, _configString.length - 1).split("/");
          var config = serviceConfigs.filterProperty('filename', configObject[0] + '.xml').findProperty('name', configObject[1]);
          if (config) {
            var configValue = Em.get(config, 'value');
            parseIfConditionVal = parseIfConditionVal.replace(_configString, configValue);
          }
        }, this);
        var conditionResult = window.eval(JSON.stringify(parseIfConditionVal.trim())) === result.trim();
        allConditionResult.push(conditionResult);
      }
    }, this);
    return Boolean(window.eval(allConditionResult.join('')));
  }

});

});

require.register("utils/configs_collection", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * This is util for storing static config info
 * Such way of managing config is much faster rather then Ember data
 * since it's using plain objects
 * This util has simple method to manipulate config objects
 */

/**
 * these variables are used to store configs in array and in object to get config quickly
 * these properties shouldn't be managed in other way rather then method of configCollection
 * @type {Array}
 */
var configsCollection = [],

/**
 * this should be object with key - config id and value - config object
 * @type {Object}
 */
configsCollectionMap = {};

App.configsCollection = Em.Object.create({

  /**
   * adds config property to configs array and map
   * should assert error if config has no id
   * @param c
   * @method add
   */
  add: function add(c) {
    Em.assert('id should be defined', c && c.id);
    if (!configsCollectionMap[c.id]) {
      configsCollection.push(c);
    }
    configsCollectionMap[c.id] = c;
  },

  /**
   * get config by id from map
   * @param id
   * @returns {Object}
   * @method getConfig
   */
  getConfig: function getConfig(id) {
    Em.assert('id should be defined', id);
    return configsCollectionMap[id];
  },

  /**
   * get config from map by name and fileName
   * @param name
   * @param fileName
   * @returns {*|Object}
   * @method getConfigByName
   */
  getConfigByName: function getConfigByName(name, fileName) {
    Em.assert('name and filename should be defined', name && fileName);
    return this.getConfig(App.config.configId(name, fileName));
  },

  /**
   * get all configs
   * @returns {Array}
   * @method getAll
   */
  getAll: function getAll() {
    return configsCollection;
  },

  /**
   * clear all configs
   * @method clearAll
   */
  clearAll: function clearAll() {
    configsCollection = [];
    configsCollectionMap = {};
  }

});

});

require.register("utils/credentials", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/** @module utils.credentials **/

/**
 * Credential Resource format.
 * @typedef {object} credentialResourceObject
 * @property {string} principal user principal name
 * @property {string} key user password
 * @property {string} type type of credential store e.g. <b>persistent</b> or <b>temporary</b>
 */
module.exports = {

  STORE_TYPES: {
    TEMPORARY: 'temporary',
    PERSISTENT: 'persisted',
    PERSISTENT_KEY: 'persistent',
    TEMPORARY_KEY: 'temporary',
    PERSISTENT_PATH: 'storage.persistent',
    TEMPORARY_PATH: 'storage.temporary'
  },

  ALIAS: {
    KDC_CREDENTIALS: 'kdc.admin.credential'
  },

  /**
   * Store credentials to server
   *
   * @member utils.credentials
   * @param {string} clusterName cluster name
   * @param {string} alias credential alias name e.g. "kdc.admin.credentials"
   * @param {credentialResourceObject} resource resource info to set e.g.
   * <code>
   * {
   *   principal: "USERNAME",
   *   key: "SecretKey",
   *   type: "persisted"
   * }
   * </code>
   *
   * Where:
   * <ul>
   *   <li>principal: the principal (or username) part of the credential to store</li>
   *   <li>key: the secret key part of the credential to store</li>
   *   <li>type: declares the storage facility type: "persisted" or "temporary"</li>
   * </ul>
   * @returns {$.Deferred} promise object
   */
  createCredentials: function createCredentials(clusterName, alias, resource) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.create',
      data: {
        clusterName: clusterName,
        resource: resource,
        alias: alias
      },
      error: 'createCredentialsErrorCallback'
    });
  },

  credentialsSuccessCallback: function credentialsSuccessCallback(data, opt, params) {
    params.callback(data.items.length ? data.items.mapProperty('Credential') : []);
  },

  createCredentialsErrorCallback: function createCredentialsErrorCallback(req, ajaxOpts, error) {},

  /**
   * @see createCredentials
   * @member utils.credentials
   * @param {string} clusterName
   * @param {string} alias
   * @param {credentialResourceObject} resource
   * @returns {$.Deferred} promise object
   */
  createOrUpdateCredentials: function createOrUpdateCredentials(clusterName, alias, resource) {
    var self = this;
    var dfd = $.Deferred();
    this.getCredential(clusterName, alias).then(function () {
      // update previously stored credentials
      self.updateCredentials(clusterName, alias, resource).always(function () {
        var status = arguments[1];
        var result = arguments[2];
        dfd.resolve(status === "success", result);
      });
    }, function () {
      // create credentials if they not exist
      self.createCredentials(clusterName, alias, resource).always(function () {
        var status = arguments[1];
        var result = arguments[2];
        dfd.resolve(status === "success", result);
      });
    });
    return dfd.promise();
  },

  /**
   * Retrieve single credential from cluster by specified alias name
   *
   * @member utils.credentials
   * @param {string} clusterName cluster name
   * @param {string} alias credential alias name e.g. "kdc.admin.credentials"
   * @param {function} [callback] success callback to invoke, credential will be passed to first argument
   * @returns {$.Deferred} promise object
   */
  getCredential: function getCredential(clusterName, alias, callback) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.get',
      data: {
        clusterName: clusterName,
        alias: alias,
        callback: callback
      },
      success: 'getCredentialSuccessCallback',
      error: 'getCredentialErrorCallback'
    });
  },

  getCredentialSuccessCallback: function getCredentialSuccessCallback(data, opt, params) {
    if (params.callback) {
      params.callback(Em.getWithDefault(data, 'Credential', null));
    }
  },

  getCredentialErrorCallback: function getCredentialErrorCallback() {},

  /**
   * Update credential by alias and cluster name
   *
   * @see createCredentials
   * @param {string} clusterName
   * @param {string} alias
   * @param {object} resource
   * @returns {$.Deferred} promise object
   */
  updateCredentials: function updateCredentials(clusterName, alias, resource) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.update',
      data: {
        clusterName: clusterName,
        alias: alias,
        resource: resource
      }
    });
  },

  /**
   * Get credenial list from server by specified cluster name
   *
   * @param {string} clusterName cluster name
   * @param {function} callback
   * @returns {$.Deferred} promise object
   */
  credentials: function credentials(clusterName, callback) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.list',
      data: {
        clusterName: clusterName,
        callback: callback
      },
      success: 'credentialsSuccessCallback'
    });
  },

  /**
   * Remove credential from server by specified cluster name and alias
   *
   * @param {string} clusterName cluster name
   * @param {string} alias credential alias name e.g. "kdc.admin.credentials"
   */
  removeCredentials: function removeCredentials(clusterName, alias) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.delete',
      data: {
        clusterName: clusterName,
        alias: alias
      }
    });
  },

  /**
   * Get info regarding credential storage type like <code>persistent</code> and <code>temporary</code>
   *
   * @param {string} clusterName cluster name
   * @param {function} callback
   * @returns {$.Deferred} promise object
   */
  storageInfo: function storageInfo(clusterName, callback) {
    return App.ajax.send({
      sender: this,
      name: 'credentials.store.info',
      data: {
        clusterName: clusterName,
        callback: callback
      },
      success: 'storageInfoSuccessCallback'
    });
  },

  storageInfoSuccessCallback: function storageInfoSuccessCallback(json, opt, params, request) {
    if (json.Clusters) {
      var storage = Em.getWithDefault(json, 'Clusters.credential_store_properties', {});
      var storeTypesObject = {};

      storeTypesObject[this.STORE_TYPES.PERSISTENT_KEY] = storage[this.STORE_TYPES.PERSISTENT_PATH] === "true";
      storeTypesObject[this.STORE_TYPES.TEMPORARY_KEY] = storage[this.STORE_TYPES.TEMPORARY_PATH] === "true";
      params.callback(storeTypesObject);
    } else {
      params.callback(null);
    }
  },

  /**
   * Resolves promise with <code>true</code> value if secure store is persistent
   *
   * @param {string} clusterName
   * @returns {$.Deferred} promise object
   */
  isStorePersisted: function isStorePersisted(clusterName) {
    return this.storeTypeStatus(clusterName, this.STORE_TYPES.PERSISTENT_KEY);
  },

  /**
   * Resolves promise with <code>true</code> value if secure store is temporary
   *
   * @param {string} clusterName
   * @returns {$.Deferred} promise object
   */
  isStoreTemporary: function isStoreTemporary(clusterName) {
    return this.storeTypeStatus(clusterName, this.STORE_TYPES.TEMPORARY_KEY);
  },

  /**
   * Get store type value for specified cluster and store type e.g. <b>persistent</b> or <b>temporary</b>
   *
   * @member utils.credentials
   * @param {string} clusterName
   * @param {string} type store type e.g. <b>persistent</b> or <b>temporary</b>
   * @returns {$.Deferred} promise object
   */
  storeTypeStatus: function storeTypeStatus(clusterName, type) {
    var dfd = $.Deferred();
    this.storageInfo(clusterName, function (storage) {
      dfd.resolve(Em.get(storage, type));
    }).fail(function (error) {
      dfd.reject(error);
    });
    return dfd.promise();
  },

  /**
   * Generate payload for storing credential.
   *
   * @member utils.credentials
   * @param {string} principal principal name
   * @param {string} key secret key
   * @param {string} type storage type e.g. <b>persisted</b>, <b>temporary</b>
   * @returns {credentialResourceObject} resource template
   */
  createCredentialResource: function createCredentialResource(principal, key, type) {
    return {
      principal: principal,
      key: key,
      type: type
    };
  },

  /**
   * Check that KDC credentials stored as <b>persisted</b> and not <b>temporary</b> from specified credentials list.
   *
   * @member utils.credentials
   * @param {object[]} credentials credentials list retrieved from API @see credentials
   * @returns {boolean} <code>true</code> if credentials are persisted
   */
  isKDCCredentialsPersisted: function isKDCCredentialsPersisted(credentials) {
    var kdcCredentials = credentials.findProperty('alias', this.ALIAS.KDC_CREDENTIALS);
    if (kdcCredentials) {
      return Em.getWithDefault(kdcCredentials, 'type', this.STORE_TYPES.TEMPORARY) === this.STORE_TYPES.PERSISTENT;
    }
    return false;
  }
};

});

require.register("utils/data_manipulation", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {

  /**
   * Exclude objects from <code>collection</code> if its <code>key</code> exist in <code>valuesToReject</code>
   * Example:
   * <code>
   *   var collection = [{n: 'v1'}, {n: 'v2'}, {n: 'v3'}, {n: 'v4'}],
   *   key = 'n',
   *   valuesToReject = ['v2', 'v3'];
   *   var result = rejectPropertyValues(collection, key, valuesToReject); // [{n: 'v1'}, {n: 'v4'}]
   * </code>
   * @param {Object[]} collection array of objects
   * @param {String} key property name of each object to be checked
   * @param {String[]} valuesToReject list of <code>key</code> values. Objects with <code>key</code>,
   *                                    equal to one of them, will be excluded
   * @returns {Object[]} Rejected array
   */
  rejectPropertyValues: function rejectPropertyValues(collection, key, valuesToReject) {
    return collection.filter(function (item) {
      var propertyValue = Em.get(item, key);
      return !valuesToReject.contains(propertyValue);
    });
  },

  /**
   * Wrapper to <code>filterProperty</code>-method that allows using list of values to filter
   * Example:
   * <code>
   *   var collection = [{n: 'v1'}, {n: 'v2'}, {n: 'v3'}, {n: 'v4'}],
   *   key = 'n',
   *   valuesToFilter = ['v2', 'v3'];
   *   var result = filterPropertyValues(collection, key, valuesToFilter); // [{n: 'v2'}, {n: 'v3'}]
   * </code>
   * @param {Object[]} collection array of objects
   * @param {String} key property name of each object to be filtered
   * @param {String|String[]} valuesToFilter array of values (or one value-string) to filter
   * @returns {Object[]} Filtered array
   */
  filterPropertyValues: function filterPropertyValues(collection, key, valuesToFilter) {
    var type = Em.typeOf(valuesToFilter);
    if ('string' === type) return collection.filterProperty(key, valuesToFilter);

    var ret = [];
    if ('array' === type) {
      valuesToFilter.forEach(function (value) {
        ret = ret.concat(collection.filterProperty(key, value));
      });
    }
    return ret;
  },

  /**
   * Group <code>collection</code> items by <code>key</code> value
   * Example:
   * <code>
   *   var collection = [{n: 'v1'}, {n: 'v2'}, {n: 'v2'}, {n: 'v4'}],
   *   key = 'n';
   *   var result = groupPropertyValues(collection, key); // {v1: [{n: 'v1'}], v2: [{n: 'v2'}, {n: 'v2'}], v4: [{n: 'v4'}]}
   * </code>
   * @param {Object[]} collection array of objects
   * @param {String} key property name of each object to be grouped
   * @returns {object}
   */
  groupPropertyValues: function groupPropertyValues(collection, key) {
    var group = {};
    collection.forEach(function (item) {
      var value = Em.get(item, key);
      if (Em.isNone(group[value])) {
        group[value] = [item];
      } else {
        group[value].pushObject(item);
      }
    });
    return group;
  }

};

});

require.register("utils/date/date", function(exports, require, module) {
'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var validator = require('utils/validator');
var App = require('app');

module.exports = {

  /**
   * List of monthes short names
   * @type {string[]}
   */
  dateMonths: ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'],

  /**
   * List of days short names
   * @type {string[]}
   */
  dateDays: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],

  /**
   * Add leading zero
   *
   * @param {string} time
   * @returns {string}
   * @method dateFormatZeroFirst
   */
  dateFormatZeroFirst: function dateFormatZeroFirst(time) {
    return (time < 10 ? '0' : '') + time;
  },

  /**
   * Convert timestamp to date-string
   * default format - 'DAY_OF_THE_WEEK, MONTH DAY, YEAR HOURS:MINUTES'
   *
   * @param {number} timestamp
   * @param {bool} format
   * @return {*} date
   * @method dateFormat
   */
  dateFormat: function dateFormat(timestamp, format) {
    if (!validator.isValidInt(timestamp)) {
      return timestamp;
    }
    format = format || 'ddd, MMM DD, YYYY HH:mm';

    return moment(new Date(timestamp)).format(format);
  },

  /**
   * Convert timestamp to date-string 'DAY_OF_THE_WEEK MONTH DAY YEAR'
   *
   * @param {string} timestamp
   * @return {string}
   * @method dateFormatShort
   */
  dateFormatShort: function dateFormatShort(timestamp) {
    if (!validator.isValidInt(timestamp)) {
      return timestamp;
    }
    var format = 'ddd MMM DD YYYY';
    var date = moment(new Date(timestamp)).format(format);
    var today = moment(new Date()).format(format);
    if (date === today) {
      return 'Today ' + new Date(timestamp).toLocaleTimeString();
    }
    return date;
  },

  startTime: function startTime(startTimestamp) {
    return this._time(startTimestamp, 'Not started');
  },

  endTime: function endTime(endTimestamp) {
    return this._time(endTimestamp, 'Not finished');
  },

  /**
   * Convert timestamp to 'DAY_OF_THE_WEEK, MONTH DAY, YEAR HOURS:MINUTES', except for the case: year equals 1969
   *
   * @param {string} timestamp
   * @param {string} msg
   * @return {string} TimeSummary
   */
  _time: function _time(timestamp, msg) {
    if (!validator.isValidInt(timestamp)) {
      return '';
    }
    var startDate = new Date(timestamp);
    var months = this.dateMonths;
    var days = this.dateDays;
    if (startDate.getFullYear() === 1969 || timestamp < 1) {
      return msg;
    }
    var startTimeSummary = '';
    if (new Date(timestamp).setHours(0, 0, 0, 0) === new Date().setHours(0, 0, 0, 0)) {
      //today
      startTimeSummary = 'Today ' + this.dateFormatZeroFirst(startDate.getHours()) + ':' + this.dateFormatZeroFirst(startDate.getMinutes());
    } else {
      startTimeSummary = days[startDate.getDay()] + ' ' + months[startDate.getMonth()] + ' ' + this.dateFormatZeroFirst(startDate.getDate()) + ' ' + startDate.getFullYear() + ' ' + this.dateFormatZeroFirst(startDate.getHours()) + ':' + this.dateFormatZeroFirst(startDate.getMinutes());
    }
    return startTimeSummary;
  },

  /**
   * Provides the duration between the given start and end timestamp. If start time
   * not valid, duration will be ''. If end time is not valid, duration will
   * be till now, showing 'Lasted for xxx secs'.
   *
   * @param {string} startTimestamp
   * @param {string} endTimestamp
   * @return {string} durationSummary
   * @method durationSummary
   */
  durationSummary: function durationSummary(startTimestamp, endTimestamp) {
    // generate duration
    var durationSummary = '';
    var startDate = new Date(startTimestamp);
    var endDate = new Date(endTimestamp);
    if (startDate.getFullYear() == 1969 || startTimestamp < 1) {
      // not started
      return Em.I18n.t('common.na');
    }
    if (endDate.getFullYear() != 1969 && endTimestamp > 0) {
      durationSummary = '' + this.timingFormat(endTimestamp - startTimestamp);
      return durationSummary.contains('-') ? Em.I18n.t('common.na') : durationSummary; //lasted for xx secs
    } else {
      // still running, duration till now
      var time = App.dateTimeWithTimeZone() - startTimestamp < 0 ? 0 : App.dateTimeWithTimeZone() - startTimestamp;
      durationSummary = '' + this.timingFormat(time);
    }
    return durationSummary.contains('-') ? Em.I18n.t('common.na') : durationSummary;
  },

  /**
   * Format: "{#days}d {#hours}h {#minutes}m {#seconds}s"
   * Example: "1d 3h 46m"
   *
   * Display optimization rules:
   *   if time more than a day then hide time lower than minute
   *   if time more than a minute and less than an hour then hide time lower than second
   * @param {number|string} time
   * @return {string|null} formatted date
   * @method timingFormat
   */
  timingFormat: function timingFormat(time) {
    if (Em.isNone(time)) {
      return null;
    }

    time = parseInt(time);
    var fullTime = time;
    var duration = '';

    if (time === 0) {
      return '0s';
    }

    var oneSecMs = 1000;
    var oneMinMs = 60000;
    var oneHourMs = 3600000;
    var oneDayMs = 86400000;
    var days = void 0,
        hours = void 0,
        minutes = void 0,
        seconds = void 0;

    var _extractTimeUnit = this.extractTimeUnit(time, oneDayMs, 'd');

    var _extractTimeUnit2 = _slicedToArray(_extractTimeUnit, 2);

    days = _extractTimeUnit2[0];
    time = _extractTimeUnit2[1];

    var _extractTimeUnit3 = this.extractTimeUnit(time, oneHourMs, 'h');

    var _extractTimeUnit4 = _slicedToArray(_extractTimeUnit3, 2);

    hours = _extractTimeUnit4[0];
    time = _extractTimeUnit4[1];

    var _extractTimeUnit5 = this.extractTimeUnit(time, oneMinMs, 'm');

    var _extractTimeUnit6 = _slicedToArray(_extractTimeUnit5, 2);

    minutes = _extractTimeUnit6[0];
    time = _extractTimeUnit6[1];

    duration += days + hours + minutes;
    if (fullTime < oneDayMs) {
      var _extractTimeUnit7 = this.extractTimeUnit(time, oneSecMs, 's');

      var _extractTimeUnit8 = _slicedToArray(_extractTimeUnit7, 2);

      seconds = _extractTimeUnit8[0];
      time = _extractTimeUnit8[1];

      duration += seconds;
      if (fullTime < oneSecMs) {
        duration += '1s';
      }
    }

    return duration.trim();
  },

  extractTimeUnit: function extractTimeUnit(time, unitValue, unitSuffix) {
    var result = '';
    if (time >= unitValue) {
      result = Math.floor(time / unitValue) + (unitSuffix + ' ');
      time -= Math.floor(time / unitValue) * unitValue;
    }
    return [result, time];
  },

  /**
   * Provides the duration between the given start and end time. If start time
   * is not given, duration will be 0. If end time is not given, duration will
   * be till now.
   *
   * @param {Number} startTime Start time from epoch
   * @param {Number} endTime End time from epoch
   * @return {Number} duration
   * @method duration
   */
  duration: function duration(startTime, endTime) {
    var duration = 0;
    if (startTime && startTime > 0) {
      if (!endTime || endTime < 1) {
        endTime = App.dateTime();
      }
      duration = endTime - startTime;
    }
    return duration;
  }

};

});

require.register("utils/date/timezone", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var dataUtils = require('utils/data_manipulation');

/**
 * Transitional list of timezones (used to create list of shownTimezone @see shownTimezone)
 *
 * <code>utcOffset</code> - offset-value (0, 180, 240 etc)
 * <code>formattedOffset</code> - formatted offset-value ('+00:00', '-02:00' etc)
 * <code>value</code> - timezone's name (like 'Europe/Athens')
 * <code>region</code> - timezone's region (for 'Europe/Athens' it will be 'Europe')
 * <code>city</code> - timezone's city (for 'Europe/Athens' it will be 'Athens')
 *
 * @typedef {{utcOffset: number, formattedOffset: string, value: string, region: string, city: string}} formattedTimezone
 */

/**
 * List of timezones used in the user's settings popup
 *
 * <code>utcOffset</code> - offset-value (0, 180, 240 etc)
 * <code>value</code> - string like '120180|Europe'
 * <code>label</code> - string like '(UTC+02:00) Europe / Athens, Kiev, Minsk'
 * <code>zones</code> - list of zone-objects from <code>moment.tz</code> included to the <code>value</code>
 *
 * @typedef {{utcOffset: number, label: string, value: string, label: string, zones: object[]}} shownTimezone
 */

module.exports = Em.Object.create({

  /**
   * @type {shownTimezone[]}
   * @readOnly
   */
  timezones: [],

  /**
   * Map of <code>timezones</code>
   * Key - timezone value (like '(UTC+01:00) Region / City1, City2')
   * Value - zone-object
   *
   * @type {object}
   * @readOnly
   */
  timezonesMappedByValue: function () {
    var ret = {};
    this.get('timezones').forEach(function (tz) {
      ret[tz.value] = tz;
    });
    return ret;
  }.property('timezones.[]'),

  init: function init() {
    this.set('timezones', this._parseTimezones());
    return this._super();
  },

  /**
   * Load list of timezones from moment.tz
   * Zones "Etc/*" and abbreviations are excluded
   *
   * @returns {string[]}
   */
  getAllTimezoneNames: function getAllTimezoneNames() {
    return moment.tz.names().filter(function (timeZoneName) {
      return timeZoneName.indexOf('Etc/') !== 0 && timeZoneName !== timeZoneName.toUpperCase();
    });
  },

  /**
   * Try detect user's timezone using timezoneOffset and moment.tz
   * Checking current year January and July offsets
   * If <code>region</code> is provided, timezone for it is returned and not first valid
   *
   * @param {string} [region] preferred region (may be 'Europe', 'America', 'Africa', 'Asia' etc)
   * @returns {string}
   */
  detectUserTimezone: function detectUserTimezone(region) {
    region = (region || '').toLowerCase();
    var currentYear = new Date().getFullYear();
    var jan = new Date(currentYear, 0, 1);
    var jul = new Date(currentYear, 6, 1);
    var janOffset = jan.getTimezoneOffset();
    var julOffset = jul.getTimezoneOffset();
    var timezones = this.get('timezones');

    var validZones = [];

    for (var i = 0; i < timezones.length; i++) {
      var zones = timezones[i].zones;
      for (var j = 0; j < zones.length; j++) {
        var zone = moment.tz.zone(zones[j].value);
        if (zone.offset(jan) === janOffset && zone.offset(jul) === julOffset) {
          validZones.pushObject(timezones[i].value);
        }
      }
    }
    if (validZones.length) {
      if (region) {
        for (i = 0; i < validZones.length; i++) {
          if (validZones[i].toLowerCase().indexOf(region) !== -1) {
            return validZones[i];
          }
        }
        // Timezone for `region` wasn't found
        return validZones[0];
      }
      // `region` isn't provided, so return first valid timezone
      return validZones[0];
    }
    // are you from Venus?
    return '';
  },

  /**
   * Reformat timezones list and sort it by utcOffset and timeZoneName
   *
   * @private
   * @method _parseTimezones
   * @returns {shownTimezone[]}
   */
  _parseTimezones: function _parseTimezones() {
    var currentYear = new Date().getFullYear();
    var jan = new Date(currentYear, 0, 1);
    var jul = new Date(currentYear, 6, 1);
    var zones = this.getAllTimezoneNames().map(function (timeZoneName) {
      var zone = moment(new Date()).tz(timeZoneName);
      var z = moment.tz.zone(timeZoneName);
      var offset = zone.format('Z');
      var regionCity = timeZoneName.split('/');
      var region = regionCity[0];
      var city = regionCity.length === 2 ? regionCity[1] : '';
      return {
        groupByKey: z.offset(jan) + '' + z.offset(jul),
        utcOffset: zone.utcOffset(),
        formattedOffset: offset,
        value: timeZoneName,
        region: region,
        city: city.replace(/_/g, ' ')
      };
    }).sort(function (zoneA, zoneB) {
      if (zoneA.utcOffset === zoneB.utcOffset) {
        if (zoneA.value === zoneB.value) {
          return 0;
        }
        return zoneA.value < zoneB.value ? -1 : 1;
      } else {
        if (zoneA.utcOffset === zoneB.utcOffset) {
          return 0;
        }
        return zoneA.utcOffset < zoneB.utcOffset ? -1 : 1;
      }
    });

    return this._groupTimezones(zones);
  },

  /**
   * Group timezones by <code>groupByKey</code>
   * Group timezones in the each group by <code>region</code>
   * <code>city</code> for each regions are joined into string 'city1, city2, city3' (empty cities and abbreviations are ignored)
   * Example:
   * <pre>
   *   var zones = [
   *    {groupByKey: '1', formattedOffset: '+01:00', value: 'a/Aa', region: 'a', city: 'Aa'},
   *    {groupByKey: '1', formattedOffset: '+01:00', value: 'a/Bb', region: 'a', city: 'Bb'},
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'a/Cc', region: 'a', city: 'Cc'},
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'a/Dd', region: 'a', city: 'Dd'},
   *    {groupByKey: '1', formattedOffset: '+01:00', value: 'b/Ee', region: 'b', city: 'Ee'},
   *    {groupByKey: '1', formattedOffset: '+01:00', value: 'b/Ff', region: 'b', city: 'Ff'},
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'b/Gg', region: 'b', city: 'Gg'},
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'b/Hh', region: 'b', city: 'Hh'},
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'b/II', region: 'b', city: 'II'}, // will be ignored, because city is abbreviation
   *    {groupByKey: '2', formattedOffset: '+02:00', value: 'b',    region: 'b', city: ''  }  // will be ignored, because city is empty
   *   ];
   *   var groupedZones = _groupTimezones(zones);
   *   // groupedZones is:
   *   [
   *    {utcOffset: 1, label: '(UTC+01:00) a / Aa, Bb', value: '1|a'},
   *    {utcOffset: 1, label: '(UTC+01:00) b / Ee, Ff', value: '1|b'},
   *    {utcOffset: 2, label: '(UTC+02:00) a / Cc, Dd', value: '2|a'},
   *    {utcOffset: 2, label: '(UTC+02:00) b / Gg, Hh', value: '2|b'}
   *   ]
   * </pre>
   *
   * @param {formattedTimezone[]} zones
   * @returns {shownTimezone[]}
   * @method _groupTimezones
   * @private
   */
  _groupTimezones: function _groupTimezones(zones) {
    var z = dataUtils.groupPropertyValues(zones, 'groupByKey');
    var newZones = [];
    Object.keys(z).forEach(function (offset) {
      var groupedByRegionZones = dataUtils.groupPropertyValues(z[offset], 'region');
      Object.keys(groupedByRegionZones).forEach(function (region) {
        var cities = groupedByRegionZones[region].mapProperty('city').filter(function (city) {
          return city !== '' && city !== city.toUpperCase();
        }).uniq().join(', ');
        var formattedOffset = Em.get(groupedByRegionZones[region], 'firstObject.formattedOffset');
        var utcOffset = Em.get(groupedByRegionZones[region], 'firstObject.utcOffset');
        var value = Em.get(groupedByRegionZones[region], 'firstObject.groupByKey') + '|' + region;
        var abbr = moment.tz(Em.get(groupedByRegionZones[region], 'firstObject.value')).format('z');
        newZones.pushObject({
          utcOffset: utcOffset,
          label: '(UTC' + formattedOffset + ' ' + abbr + ') ' + region + (cities ? ' / ' + cities : ''),
          value: value,
          zones: groupedByRegionZones[region]
        });
      });
    });
    return newZones.sortProperty('utcOffset');
  }

});

});

require.register("utils/db", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
App.db = {};
var InitialData = {
  'app': {
    'loginName': '',
    'authenticated': false,
    'configs': [],
    'tags': [],
    'tables': {
      'filterConditions': {},
      'displayLength': {},
      'startIndex': {},
      'sortingConditions': {},
      'selectedItems': {}
    }
  },

  'Installer': {},
  'AddHost': {},
  'AddService': {},
  'WidgetWizard': {},
  'KerberosWizard': {},
  'ReassignMaster': {},
  'AddSecurity': {},
  'AddAlertDefinition': {
    content: {}
  },
  'HighAvailabilityWizard': {},
  'RMHighAvailabilityWizard': {},
  'AddHawqStandbyWizard': {},
  'RemoveHawqStandbyWizard': {},
  'ActivateHawqStandbyWizard': {},
  'RAHighAvailabilityWizard': {},
  'NameNodeFederationWizard': {},
  'RollbackHighAvailabilityWizard': {},
  'MainAdminStackAndUpgrade': {},
  'KerberosDisable': {},
  'tmp': {}

};

function checkNamespace(namespace) {
  if (!namespace) {
    return false;
  }
  if (Em.isNone(Em.get(App.db.data, namespace))) {
    Em.setFullPath(App.db.data, namespace, {});
  }
  return true;
}

if (typeof Storage === 'undefined') {
  // stub for unit testing purposes
  window.localStorage = {};
  localStorage.setItem = function (key, val) {
    this[key] = val;
  };
  localStorage.getItem = function (key) {
    return this[key];
  };
  window.localStorage.setObject = function (key, value) {
    this[key] = value;
  };
  window.localStorage.getObject = function (key) {
    return this[key];
  };
} else {
  Storage.prototype.setObject = function (key, value) {
    this.setItem(key, JSON.stringify(value));
  };

  Storage.prototype.getObject = function (key) {
    var value = this.getItem(key);
    return value && JSON.parse(value);
  };
}

App.db.cleanUp = function () {
  App.db.data = InitialData;
  localStorage.setObject('ambari', App.db.data);
};

App.db.cleanTmp = function () {
  App.db.data.tmp = {};
  localStorage.setObject('ambari', App.db.data);
};

App.db.updateStorage = function () {
  App.db.data = localStorage.getObject('ambari');
  if (Em.get(App, 'db.data.app.tables') && Em.get(App, 'db.data.app.configs')) {
    return true;
  }
  App.db.cleanUp();
  return false;
};

/*
 Initialize wizard namespaces if they are not initialized on login.
 This will be required during upgrade.
 */
App.db.mergeStorage = function () {
  if (localStorage.getObject('ambari') == null) {
    App.db.cleanUp();
  } else {
    localStorage.setObject('ambari', $.extend(true, {}, InitialData, App.db.data));
  }
};

// called whenever user logs in
if (localStorage.getObject('ambari') == null) {
  App.db.cleanUp();
}

/**
 *
 * @param {string} namespace
 * @param {string} key
 * @returns {*}
 */
App.db.get = function (namespace, key) {
  App.db.data = localStorage.getObject('ambari');
  Em.assert('`namespace` should be defined', !!namespace);
  checkNamespace(namespace);
  if (key.contains('user-pref')) {
    // username may contain "." which is the part of "user-pref-*" key so Em.set should be avoided
    return Em.get(App.db.data, namespace)[key];
  }
  return Em.get(Em.get(App.db.data, namespace), key);
};

/**
 *
 * @param {string} namespace
 * @param {string[]} listOfProperties
 * @returns {object}
 */
App.db.getProperties = function (namespace, listOfProperties) {
  App.db.data = localStorage.getObject('ambari');
  Em.assert('`namespace` should be defined', !!namespace);
  checkNamespace(namespace);
  return Em.getProperties(Em.get(App.db.data, namespace), listOfProperties);
};

/**
 *
 * @param {string} namespace
 * @param {string} key
 * @param {*} value
 */
App.db.set = function (namespace, key, value) {
  App.db.data = localStorage.getObject('ambari');
  Em.assert('`namespace` should be defined', !!namespace);
  checkNamespace(namespace);
  if (key.contains('user-pref')) {
    // username may contain "." which is the part of "user-pref-*" key so Em.set should be avoided
    Em.get(App.db.data, namespace)[key] = value;
  } else {
    Em.set(Em.get(App.db.data, namespace), key, value);
  }
  localStorage.setObject('ambari', App.db.data);
};

/**
 *
 * @param {string} namespace
 * @param {{key: value}} hash
 */
App.db.setProperties = function (namespace, hash) {
  App.db.data = localStorage.getObject('ambari');
  Em.assert('`namespace` should be defined', !!namespace);
  checkNamespace(namespace);
  Em.setProperties(Em.get(App.db.data, namespace), hash);
  localStorage.setObject('ambari', App.db.data);
};

App.db.setLoginName = function (name) {
  App.db.set('app', 'loginName', name);
};

/**
 * Set user model to db
 * @param user
 */
App.db.setUser = function (user) {
  App.db.set('app', 'user', user);
};

App.db.setAuth = function (auth) {
  App.db.set('app', 'auth', auth);
};

App.db.setAuthenticated = function (authenticated) {
  App.db.set('app', 'authenticated', authenticated);
  App.db.data = localStorage.getObject('ambari');
};

App.db.setFilterConditions = function (name, filterConditions) {
  App.db.set('app.tables.filterConditions', name, filterConditions);
};

App.db.setComboSearchQuery = function (name, query) {
  App.db.set('app.tables.comboSearchQuery', name, query);
};

App.db.setDisplayLength = function (name, displayLength) {
  App.db.set('app.tables.displayLength', name, displayLength);
};

App.db.setStartIndex = function (name, startIndex) {
  App.db.set('app.tables.startIndex', name, startIndex);
};

App.db.setSortingStatuses = function (name, sortingConditions) {
  App.db.set('app.tables.sortingConditions', name, sortingConditions);
};

App.db.setSelectedHosts = function (selectedHosts) {
  App.db.set('app.tables.selectedItems', 'mainHostController', selectedHosts);
};

App.db.unselectHosts = function () {
  var hostsToUnselect = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  var selectedHosts = App.db.getSelectedHosts();
  selectedHosts = selectedHosts.filter(function (host) {
    return hostsToUnselect.indexOf(host) === -1;
  });
  App.db.setSelectedHosts(selectedHosts);
};

App.db.setMasterComponentHosts = function (masterComponentHosts) {
  App.db.set('Installer', 'masterComponentHosts', masterComponentHosts);
};

App.db.setMasterToReassign = function (masterComponent) {
  App.db.set('ReassignMaster', 'masterComponent', masterComponent);
};

App.db.setReassignTasksStatuses = function (tasksStatuses) {
  App.db.set('ReassignMaster', 'tasksStatuses', tasksStatuses);
};

App.db.setReassignTasksRequestIds = function (requestIds) {
  App.db.set('ReassignMaster', 'tasksRequestIds', requestIds);
};

App.db.setStacks = function (stacks) {
  App.db.set('Installer', 'stacksVersions', stacks);
};

App.db.setOses = function (oses) {
  App.db.set('Installer', 'operatingSystems', oses);
};

App.db.setRepos = function (repos) {
  App.db.set('Installer', 'repositories', repos);
};

App.db.setLocalRepoVDFData = function (data) {
  App.db.set('Installer', 'localRepoVDFData', data);
};

App.db.setConfigs = function (configs) {
  App.db.set('app', 'configs', configs);
};

App.db.setTags = function (tags) {
  App.db.set('app', 'tags', tags);
};

/**
 * Set current step value for specified Wizard Type
 * @param wizardType
 * @param currentStep
 */
App.db.setWizardCurrentStep = function (wizardType, currentStep) {
  App.db.set(wizardType.capitalize(), 'currentStep', currentStep);
};

/**
 * Set localStorage with data from server
 */
App.db.setLocalStorage = function () {
  localStorage.setObject('ambari', App.db.data);
};

App.db.setSecurityWizardStatus = function (status) {
  App.db.set('AddSecurity', 'status', status);
};

App.db.setDisableSecurityStatus = function (status) {
  App.db.set('AddSecurity', 'disableSecurityStatus', status);
};

App.db.setSecurityDeployCommands = function (commands) {
  App.db.set('AddSecurity', 'securityDeployCommands', commands);
};

App.db.setHighAvailabilityWizardConfigTag = function (tag) {
  App.db.set('HighAvailabilityWizard', tag.name, tag.value);
};

App.db.setHighAvailabilityWizardHdfsClientHosts = function (hostNames) {
  App.db.set('HighAvailabilityWizard', 'hdfsClientHostNames', hostNames);
};

App.db.setHighAvailabilityWizardTasksStatuses = function (tasksStatuses) {
  App.db.set('HighAvailabilityWizard', 'tasksStatuses', tasksStatuses);
};

App.db.setHighAvailabilityWizardTasksRequestIds = function (requestIds) {
  App.db.set('HighAvailabilityWizard', 'tasksRequestIds', requestIds);
};

App.db.setHighAvailabilityWizardHdfsUser = function (hdfsUser) {
  App.db.set('HighAvailabilityWizard', 'hdfsUser', hdfsUser);
};

App.db.setHighAvailabilityWizardRequestIds = function (requestIds) {
  App.db.set('HighAvailabilityWizard', 'requestIds', requestIds);
};

App.db.setHighAvailabilityWizardNameServiceId = function (nameServiceId) {
  App.db.set('HighAvailabilityWizard', 'nameServiceId', nameServiceId);
};

App.db.setRollBackHighAvailabilityWizardAddNNHost = function (host) {
  App.db.set('RollbackHighAvailabilityWizard', 'addNNHost', host);
};

App.db.setRollBackHighAvailabilityWizardSNNHost = function (host) {
  App.db.set('RollbackHighAvailabilityWizard', 'sNNHost', host);
};

App.db.setRollBackHighAvailabilityWizardSelectedAddNN = function (host) {
  App.db.set('RollbackHighAvailabilityWizard', 'selectedAddNN', host);
};

App.db.setRollBackHighAvailabilityWizardSelectedSNN = function (host) {
  App.db.set('RollbackHighAvailabilityWizard', 'selectedSNNH', host);
};

App.db.setRollbackHighAvailabilityWizardTasksStatuses = function (tasksStatuses) {
  App.db.set('RollbackHighAvailabilityWizard', 'tasksStatuses', tasksStatuses);
};

App.db.setRollbackHighAvailabilityWizardRequestIds = function (requestIds) {
  App.db.set('RollbackHighAvailabilityWizard', 'requestIds', requestIds);
};

App.db.setReassignMasterWizardRequestIds = function (requestIds) {
  App.db.set('ReassignMaster', 'requestIds', requestIds);
};

App.db.setReassignMasterWizardComponentDir = function (componentDir) {
  App.db.set('ReassignMaster', 'componentDir', componentDir);
};

App.db.setReassignMasterWizardReassignHosts = function (reassignHosts) {
  App.db.set('ReassignMaster', 'reassignHosts', reassignHosts);
};

App.db.setKerberosWizardConfigTag = function (tag) {
  App.db.set('KerberosWizard', tag.name, tag.value);
};

App.db.setManageJournalNodeWizardConfigTag = function (tag) {
  App.db.set('ManageJournalNodeWizard', tag.name, tag.value);
};

/**
 * Get user model from db
 * @return {*}
 */
App.db.getUser = function () {
  return App.db.get('app', 'user');
};

App.db.getAuth = function () {
  return App.db.get('app', 'auth');
};

App.db.getLoginName = function () {
  return App.db.get('app', 'loginName');
};

App.db.getAuthenticated = function () {
  return Boolean(App.db.get('app', 'authenticated'));
};

App.db.getFilterConditions = function (name) {
  return name ? App.db.get('app.tables.filterConditions', name) : null;
};

App.db.getComboSearchQuery = function (name) {
  return name ? App.db.get('app.tables.comboSearchQuery', name) : null;
};

App.db.getDisplayLength = function (name) {
  return name ? App.db.get('app.tables.displayLength', name) : null;
};

App.db.getStartIndex = function (name) {
  return name ? App.db.get('app.tables.startIndex', name) : null;
};

App.db.getSortingStatuses = function (name) {
  return name ? App.db.get('app.tables.sortingConditions', name) : null;
};

App.db.getSelectedHosts = function () {
  return App.db.get('app.tables.selectedItems', 'mainHostController') || [];
};

/**
 * Return current step for specified Wizard Type
 * @param wizardType
 * @return {*}
 */
App.db.getWizardCurrentStep = function (wizardType) {
  return App.db.get(wizardType.capitalize(), 'currentStep') || 0;
};

App.db.getAllHostNames = function () {
  return App.db.get('Installer', 'hostNames');
};

App.db.getMasterToReassign = function () {
  return App.db.get('ReassignMaster', 'masterComponent');
};

App.db.getReassignTasksStatuses = function () {
  return App.db.get('ReassignMaster', 'tasksStatuses');
};

App.db.getReassignTasksRequestIds = function () {
  return App.db.get('ReassignMaster', 'tasksRequestIds');
};

App.db.getSecurityWizardStatus = function () {
  return App.db.get('AddSecurity', 'status');
};

App.db.getDisableSecurityStatus = function () {
  return App.db.get('AddSecurity', 'disableSecurityStatus');
};

App.db.getStacks = function () {
  return App.db.get('Installer', 'stacksVersions');
};

App.db.getOses = function () {
  return App.db.get('Installer', 'operatingSystems');
};

App.db.getRepos = function () {
  return App.db.get('Installer', 'repositories');
};

App.db.getLocalRepoVDFData = function () {
  return App.db.get('Installer', 'localRepoVDFData');
};

App.db.getHighAvailabilityWizardHdfsUser = function () {
  return App.db.get('HighAvailabilityWizard', 'hdfsUser');
};

App.db.getHighAvailabilityWizardTasksStatuses = function () {
  return App.db.get('HighAvailabilityWizard', 'tasksStatuses');
};

App.db.getHighAvailabilityWizardTasksRequestIds = function () {
  return App.db.get('HighAvailabilityWizard', 'tasksRequestIds');
};

App.db.getHighAvailabilityWizardFailedTask = function () {
  return App.db.get('HighAvailabilityWizard', 'failedTask');
};

App.db.getHighAvailabilityWizardHdfsClientHosts = function () {
  return App.db.get('HighAvailabilityWizard', 'hdfsClientHostNames');
};

App.db.getHighAvailabilityWizardConfigTag = function (tag) {
  return App.db.get('HighAvailabilityWizard', tag);
};

App.db.getHighAvailabilityWizardRequestIds = function () {
  return App.db.get('HighAvailabilityWizard', 'requestIds');
};

App.db.getHighAvailabilityWizardNameServiceId = function () {
  return App.db.get('HighAvailabilityWizard', 'nameServiceId');
};

App.db.getRollbackHighAvailabilityWizardTasksStatuses = function () {
  return App.db.get('RollbackHighAvailabilityWizard', 'tasksStatuses');
};

App.db.getRollbackHighAvailabilityWizardRequestIds = function () {
  return App.db.get('RollbackHighAvailabilityWizard', 'requestIds');
};

App.db.getRollBackHighAvailabilityWizardAddNNHost = function () {
  return App.db.get('RollbackHighAvailabilityWizard', 'addNNHost');
};

App.db.getRollBackHighAvailabilityWizardSNNHost = function () {
  return App.db.get('RollbackHighAvailabilityWizard', 'sNNHost');
};

App.db.getReassignMasterWizardRequestIds = function () {
  return App.db.get('ReassignMaster', 'requestIds');
};

App.db.getReassignMasterWizardComponentDir = function () {
  return App.db.get('ReassignMaster', 'componentDir');
};

App.db.getManageJournalNodeWizardConfigTag = function (tag) {
  return App.db.get('ManageJournalNodeWizard', tag);
};

App.db.getConfigs = function () {
  return App.db.get('app', 'configs');
};

App.db.getTags = function () {
  return App.db.get('app', 'tags');
};

App.db.getReassignMasterWizardReassignHosts = function () {
  return App.db.get('ReassignMaster', 'reassignHosts');
};

module.exports = App.db;

});

require.register("utils/ember_computed", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var computed = Em.computed;
var get = Em.get;
var makeArray = Em.makeArray;

var slice = [].slice;

var dataUtils = require('utils/data_manipulation');

/**
 * Returns hash with values of name properties from the context
 * If <code>propertyName</code> starts with 'App.', <code>App</code> is used as context, <code>self</code> used otherwise
 * If some <code>propertyName</code> starts with '!' its value will be inverted
 *
 * @param {object} self current context
 * @param {string[]} propertyNames needed properties
 * @returns {object} hash with needed values
 */
function getProperties(self, propertyNames) {
  var ret = {};
  for (var i = 0; i < propertyNames.length; i++) {
    var propertyName = propertyNames[i];
    var shouldBeInverted = propertyName.startsWith('!');
    propertyName = shouldBeInverted ? propertyName.substr(1) : propertyName;
    var isApp = propertyName.startsWith('App.');
    var name = isApp ? propertyName.replace('App.', '') : propertyName;
    var value = isApp ? App.get(name) : self.get(name);
    value = shouldBeInverted ? !value : value;
    ret[propertyName] = value;
  }
  return ret;
}

/**
 * Returns value of named property from the context
 * If <code>propertyName</code> starts with 'App.', <code>App</code> is used as context, <code>self</code> used otherwise
 *
 * @param {object} self current context
 * @param {string} propertyName needed property
 * @returns {*} needed value
 */
function smartGet(self, propertyName) {
  var isApp = propertyName.startsWith('App.');
  var name = isApp ? propertyName.replace('App.', '') : propertyName;
  return isApp ? App.get(name) : self.get(name);
}

/**
 * Returns list with values of name properties from the context
 * If <code>propertyName</code> starts with 'App.', <code>App</code> is used as context, <code>self</code> used otherwise
 *
 * @param {object} self current context
 * @param {string[]} propertyNames needed properties
 * @returns {array} list of needed values
 */
function getValues(self, propertyNames) {
  return propertyNames.map(function (propertyName) {
    return smartGet(self, propertyName);
  });
}

function generateComputedWithKey(macro) {
  return function () {
    var properties = slice.call(arguments, 1);
    var key = arguments[0];
    var computedFunc = computed(function () {
      var values = getValues(this, properties);
      return macro.call(this, key, values);
    });

    return computedFunc.property.apply(computedFunc, properties);
  };
}

function generateComputedWithProperties(macro) {
  return function () {
    var properties = slice.call(arguments);
    var computedFunc = computed(function () {
      return macro.apply(this, [getProperties(this, properties)]);
    });

    var realProperties = properties.slice().invoke('replace', '!', '');
    return computedFunc.property.apply(computedFunc, realProperties);
  };
}

function generateComputedWithValues(macro) {
  return function () {
    var properties = slice.call(arguments);
    var computedFunc = computed(function () {
      return macro.apply(this, [getValues(this, properties)]);
    });

    return computedFunc.property.apply(computedFunc, properties);
  };
}

/**
 *
 * A computed property that returns true if the provided dependent property
 * is equal to the given value.
 * App.*-keys are supported
 * Example*
 * ```javascript
 * var Hamster = Ember.Object.extend({
 *    napTime: Ember.computed.equal('state', 'sleepy')
 *  });
 * var hamster = Hamster.create();
 * hamster.get('napTime'); // false
 * hamster.set('state', 'sleepy');
 * hamster.get('napTime'); // true
 * hamster.set('state', 'hungry');
 * hamster.get('napTime'); // false
 * ```
 * @method equal
 * @param {String} dependentKey
 * @param {String|Number|Object} value
 * @return {Ember.ComputedProperty} computed property which returns true if
 * the original value for property is equal to the given value.
 * @public
 */
computed.equal = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) === value;
  }).cacheable();
};

/**
 * A computed property that returns true if the provided dependent property is not equal to the given value
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'a',
 *  p2: Em.computed.notEqual('p1', 'a')
 * });
 * console.log(o.get('p2')); // false
 * o.set('p1', 'b');
 * console.log(o.get('p2')); // true
 * </pre>
 *
 * @method notEqual
 * @param {string} dependentKey
 * @param {*} value
 * @returns {Ember.ComputedProperty}
 */
computed.notEqual = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) !== value;
  });
};

/**
 * A computed property that returns true if provided dependent properties are equal to the each other
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'a',
 *  p2: 'b',
 *  p3: Em.computed.equalProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // false
 * o.set('p1', 'b');
 * console.log(o.get('p3')); // true
 * </pre>
 *
 * @method equalProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.equalProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) === smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns true if provided dependent properties are not equal to the each other
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'a',
 *  p2: 'b',
 *  p3: Em.computed.notEqualProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // true
 * o.set('p1', 'b');
 * console.log(o.get('p3')); // false
 * </pre>
 *
 * @method notEqualProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.notEqualProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) !== smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns grouped collection's items by propertyName-value
 *
 * @method groupBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @returns {Ember.ComputedProperty}
 */
computed.groupBy = function (collectionKey, propertyName) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = get(this, collectionKey);
    return dataUtils.groupPropertyValues(collection, propertyName);
  });
};

/**
 * A computed property that returns filtered collection by propertyName values-list
 * Wrapper to filterProperty-method that allows using list of values to filter
 *
 * @method filterByMany
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {array} valuesToFilter
 * @returns {Ember.ComputedProperty}
 */
computed.filterByMany = function (collectionKey, propertyName, valuesToFilter) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = get(this, collectionKey);
    return dataUtils.filterPropertyValues(collection, propertyName, makeArray(valuesToFilter));
  });
};

/**
 * A computed property that returns collection without elements with value that is in <code>valuesToReject</code>
 * Exclude objects from <code>collection</code> if its <code>key</code> exist in <code>valuesToReject</code>
 *
 * @method rejectMany
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {array} valuesToReject
 * @returns {Ember.ComputedProperty}
 */
computed.rejectMany = function (collectionKey, propertyName, valuesToReject) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = get(this, collectionKey);
    return dataUtils.rejectPropertyValues(collection, propertyName, makeArray(valuesToReject));
  });
};

/**
 * A computed property that returns trueValue if dependent value is true and falseValue otherwise
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: true,
 *  p2: Em.computed.ifThenElse('p1', 'abc', 'cba')
 * });
 * console.log(o.get('p2')); // 'abc'
 * o.set('p1', false);
 * console.log(o.get('p2')); // 'cba'
 * </pre>
 *
 * @method ifThenElse
 * @param {string} dependentKey
 * @param {*} trueValue
 * @param {*} falseValue
 * @returns {Ember.ComputedProperty}
 */
computed.ifThenElse = function (dependentKey, trueValue, falseValue) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) ? trueValue : falseValue;
  });
};

/**
 * A computed property that returns value for trueKey property if dependent value is true and falseKey-value otherwise
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: true,
 *  p2: 1,
 *  p3: 2,
 *  p4: Em.computed.ifThenElseByKeys('p1', 'p2', 'p3')
 * });
 * console.log(o.get('p4')); // 1
 * o.set('p1', false);
 * console.log(o.get('p4')); // 2
 *
 * o.set('p3', 3);
 * console.log(o.get('p4')); // 3
 * </pre>
 *
 * @method ifThenElseByKeys
 * @param {string} dependentKey
 * @param {string} trueKey
 * @param {string} falseKey
 * @returns {Ember.ComputedProperty}
 */
computed.ifThenElseByKeys = function (dependentKey, trueKey, falseKey) {
  return computed(dependentKey, trueKey, falseKey, function () {
    return smartGet(this, dependentKey) ? smartGet(this, trueKey) : smartGet(this, falseKey);
  });
};

/**
 * A computed property that is equal to the logical 'and'
 * Takes any number of arguments
 * Returns true if all of them are truly, false - otherwise
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: true,
 *  p2: true,
 *  p3: true,
 *  p4: Em.computed.and('p1', 'p2', 'p3')
 * });
 * console.log(o.get('p4')); // true
 * o.set('p1', false);
 * console.log(o.get('p4')); // false
 * </pre>
 *
 * @method and
 * @param {...string} dependentKeys
 * @returns {Ember.ComputedProperty}
 */
computed.and = generateComputedWithProperties(function (properties) {
  var value;
  for (var key in properties) {
    value = !!properties[key];
    if (properties.hasOwnProperty(key) && !value) {
      return false;
    }
  }
  return value;
});

/**
 * A computed property that is equal to the logical 'or'
 * Takes any number of arguments
 * Returns true if at least one of them is truly, false - otherwise
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: false,
 *  p2: false,
 *  p3: false,
 *  p4: Em.computed.or('p1', 'p2', 'p3')
 * });
 * console.log(o.get('p4')); // false
 * o.set('p1', true);
 * console.log(o.get('p4')); // true
 * </pre>
 *
 * @method or
 * @param {...string} dependentKeys
 * @returns {Ember.ComputedProperty}
 */
computed.or = generateComputedWithProperties(function (properties) {
  var value;
  for (var key in properties) {
    value = !!properties[key];
    if (properties.hasOwnProperty(key) && value) {
      return value;
    }
  }
  return value;
});

/**
 * A computed property that returns sum on the dependent properties values
 * Takes any number of arguments
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 1,
 *  p2: 2,
 *  p3: 3,
 *  p4: Em.computed.sumProperties('p1', 'p2', 'p3')
 * });
 * console.log(o.get('p4')); // 6
 * o.set('p1', 2);
 * console.log(o.get('p4')); // 7
 * </pre>
 *
 * @method sumProperties
 * @param {...string} dependentKeys
 * @returns {Ember.ComputedProperty}
 */
computed.sumProperties = generateComputedWithProperties(function (properties) {
  var sum = 0;
  for (var key in properties) {
    if (properties.hasOwnProperty(key)) {
      sum += Number(properties[key]);
    }
  }
  return sum;
});

/**
 * A computed property that returns true if dependent value is greater or equal to the needed value
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: Em.computed.gte('p1', 1)
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1', 4);
 * console.log(o.get('p2')); // true
 * o.set('p1', 5);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method gte
 * @param {string} dependentKey
 * @param {*} value
 * @returns {Ember.ComputedProperty}
 */
computed.gte = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) >= value;
  });
};

/**
 * A computed property that returns true if first dependent property is greater or equal to the second dependent property
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: 1,
 *  p3: Em.computed.gteProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // true
 * o.set('p2', 4);
 * console.log(o.get('p3')); // true
 * o.set('p2', 5);
 * console.log(o.get('p3')); // false
 * </pre>
 *
 * @method gteProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.gteProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) >= smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns true if dependent property is less or equal to the needed value
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: Em.computed.lte('p1', 1)
 * });
 * console.log(o.get('p2')); // false
 * o.set('p1', 4);
 * console.log(o.get('p2')); // true
 * o.set('p1', 5);
 * console.log(o.get('p2')); // true
 * </pre>
 *
 * @method lte
 * @param {string} dependentKey
 * @param {*} value
 * @returns {Ember.ComputedProperty}
 */
computed.lte = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) <= value;
  });
};

/**
 * A computed property that returns true if first dependent property is less or equal to the second dependent property
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: 1,
 *  p3: Em.computed.lteProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // false
 * o.set('p2', 4);
 * console.log(o.get('p3')); // true
 * o.set('p2', 5);
 * console.log(o.get('p3')); // true
 * </pre>
 *
 * @method lteProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.lteProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) <= smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns true if dependent value is greater than the needed value
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: Em.computed.gt('p1', 1)
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1', 4);
 * console.log(o.get('p2')); // false
 * o.set('p1', 5);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method gt
 * @param {string} dependentKey
 * @param {*} value
 * @returns {Ember.ComputedProperty}
 */
computed.gt = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) > value;
  });
};

/**
 * A computed property that returns true if first dependent property is greater than the second dependent property
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: 1,
 *  p3: Em.computed.gteProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // true
 * o.set('p2', 4);
 * console.log(o.get('p3')); // false
 * o.set('p2', 5);
 * console.log(o.get('p3')); // false
 * </pre>
 *
 * @method gtProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.gtProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) > smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns true if dependent value is less than the needed value
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: Em.computed.lt('p1', 1)
 * });
 * console.log(o.get('p2')); // false
 * o.set('p1', 4);
 * console.log(o.get('p2')); // false
 * o.set('p1', 5);
 * console.log(o.get('p2')); // true
 * </pre>
 *
 * @method lt
 * @param {string} dependentKey
 * @param {*} value
 * @returns {Ember.ComputedProperty}
 */
computed.lt = function (dependentKey, value) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey) < value;
  });
};

/**
 * A computed property that returns true if first dependent property is less than the second dependent property
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 4,
 *  p2: 1,
 *  p3: Em.computed.ltProperties('p1', 'p2')
 * });
 * console.log(o.get('p3')); // false
 * o.set('p2', 4);
 * console.log(o.get('p3')); // false
 * o.set('p2', 5);
 * console.log(o.get('p3')); // true
 * </pre>
 *
 * @method gtProperties
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @returns {Ember.ComputedProperty}
 */
computed.ltProperties = function (dependentKey1, dependentKey2) {
  return computed(dependentKey1, dependentKey2, function () {
    return smartGet(this, dependentKey1) < smartGet(this, dependentKey2);
  });
};

/**
 * A computed property that returns true if dependent property is match to the needed regular expression
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'abc',
 *  p2: Em.computed.match('p1', /^a/)
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1', 'bc');
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method match
 * @param {string} dependentKey
 * @param {RegExp} regexp
 * @returns {Ember.ComputedProperty}
 */
computed.match = function (dependentKey, regexp) {
  return computed(dependentKey, function () {
    var value = get(this, dependentKey);
    if (!regexp) {
      return false;
    }
    return regexp.test(value);
  });
};

/**
 * A computed property that returns true of some collection's item has property with needed value
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.someBy('p1', 'a', 1)
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method someBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {*} neededValue
 * @returns {Ember.ComputedProperty}
 */
computed.someBy = function (collectionKey, propertyName, neededValue) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return false;
    }
    return collection.someProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns true of some collection's item has property with needed value
 * Needed value is stored in the another property
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.someByKey('p1', 'a', 'v1'),
 *  v1: 1
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method someByKey
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {string} neededValueKey
 * @returns {Ember.ComputedProperty}
 */
computed.someByKey = function (collectionKey, propertyName, neededValueKey) {
  return computed(collectionKey + '.@each.' + propertyName, neededValueKey, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return false;
    }
    var neededValue = smartGet(this, neededValueKey);
    return collection.someProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns true of all collection's items have property with needed value
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 1}, {a: 1}],
 *  p2: Em.computed.everyBy('p1', 'a', 1)
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method everyBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {*} neededValue
 * @returns {Ember.ComputedProperty}
 */
computed.everyBy = function (collectionKey, propertyName, neededValue) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return false;
    }
    return collection.everyProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns true of all collection's items have property with needed value
 * Needed value is stored in the another property
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 1}, {a: 1}],
 *  p2: Em.computed.everyByKey('p1', 'a', 'v1'),
 *  v1: 1
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method everyByKey
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {string} neededValueKey
 * @returns {Ember.ComputedProperty}
 */
computed.everyByKey = function (collectionKey, propertyName, neededValueKey) {
  return computed(collectionKey + '.@each.' + propertyName, neededValueKey, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return false;
    }
    var neededValue = smartGet(this, neededValueKey);
    return collection.everyProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns array with values of named property on all items in the collection
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.everyBy('p1', 'a')
 * });
 * console.log(o.get('p2')); // [1, 2, 3]
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // [2, 2, 3]
 * </pre>
 *
 * @method mapBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @returns {Ember.ComputedProperty}
 */
computed.mapBy = function (collectionKey, propertyName) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return [];
    }
    return collection.mapProperty(propertyName);
  });
};

/**
 * A computed property that returns array with collection's items that have needed property value
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.filterBy('p1', 'a', 2)
 * });
 * console.log(o.get('p2')); // [{a: 2}]
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // [{a: 2}, {a: 2}]
 * </pre>
 *
 * @method filterBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {*} neededValue
 * @returns {Ember.ComputedProperty}
 */
computed.filterBy = function (collectionKey, propertyName, neededValue) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return [];
    }
    return collection.filterProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns array with collection's items that have needed property value
 * Needed value is stored in the another property
 *
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.filterByKey('p1', 'a', 'v1'),
 *  v1: 2
 * });
 * console.log(o.get('p2')); // [{a: 2}]
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // [{a: 2}, {a: 2}]
 * </pre>
 *
 * @method filterByKey
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {string} neededValueKey
 * @returns {Ember.ComputedProperty}
 */
computed.filterByKey = function (collectionKey, propertyName, neededValueKey) {
  return computed(collectionKey + '.@each.' + propertyName, neededValueKey, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return [];
    }
    var neededValue = smartGet(this, neededValueKey);
    return collection.filterProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns first collection's item that has needed property value
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1, b: 1}, {a: 2, b: 2}, {a: 3, b: 3}],
 *  p2: Em.computed.findBy('p1', 'a', 2)
 * });
 * console.log(o.get('p2')); // [{a: 2, b: 2}]
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // [{a: 2, b: 1}]
 * </pre>
 *
 * @method findBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {*} neededValue
 * @returns {Ember.ComputedProperty}
 */
computed.findBy = function (collectionKey, propertyName, neededValue) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return null;
    }
    return collection.findProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns first collection's item that has needed property value
 * Needed value is stored in the another property
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1, b: 1}, {a: 2, b: 2}, {a: 3, b: 3}],
 *  p2: Em.computed.findByKey('p1', 'a', 'v1'),
 *  v1: 2
 * });
 * console.log(o.get('p2')); // [{a: 2, b: 2}]
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // [{a: 2, b: 1}]
 * </pre>
 *
 * @method findByKey
 * @param {string} collectionKey
 * @param {string} propertyName
 * @param {string} neededValueKey
 * @returns {Ember.ComputedProperty}
 */
computed.findByKey = function (collectionKey, propertyName, neededValueKey) {
  return computed(collectionKey + '.@each.' + propertyName, neededValueKey, function () {
    var collection = smartGet(this, collectionKey);
    if (!collection) {
      return null;
    }
    var neededValue = smartGet(this, neededValueKey);
    return collection.findProperty(propertyName, neededValue);
  });
};

/**
 * A computed property that returns value equal to the dependent
 * Should be used as 'short-name' for deeply-nested values
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: {a: {b: {c: 2}}},
 *  p2: Em.computed.alias('p1.a.b.c')
 * });
 * console.log(o.get('p2')); // 2
 * o.set('p1.a.b.c', 4);
 * console.log(o.get('p2')); // 4
 * </pre>
 *
 * @method alias
 * @param {string} dependentKey
 * @returns {Ember.ComputedProperty}
 */
computed.alias = function (dependentKey) {
  return computed(dependentKey, function () {
    return smartGet(this, dependentKey);
  });
};

/**
 * A computed property that returns true if dependent property exists in the needed values
 * <pre>
 * var o = Em.Object.create({
 *  p1: 2,
 *  p2: Em.computed.existsIn('p1', [1, 2, 3])
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1', 4);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method existsIn
 * @param {string} dependentKey
 * @param {array} neededValues
 * @returns {Ember.ComputedProperty}
 */
computed.existsIn = function (dependentKey, neededValues) {
  return computed(dependentKey, function () {
    var value = smartGet(this, dependentKey);
    return makeArray(neededValues).contains(value);
  });
};

/**
 * A computed property that returns true if dependent property exists in the property with needed values
 * <pre>
 * var o = Em.Object.create({
 *  p1: 2,
 *  p2: Em.computed.existsInByKey('p1', 'p3'),
 *  p3: [1, 2, 3]
 * });
 * console.log(o.get('p2')); // true
 * o.set('p1', 4);
 * console.log(o.get('p2')); // false
 * </pre>
 *
 * @method existsIn
 * @param {string} dependentKey
 * @param {string} neededValuesKey
 * @returns {Ember.ComputedProperty}
 */
computed.existsInByKey = function (dependentKey, neededValuesKey) {
  return computed(dependentKey, neededValuesKey + '.[]', function () {
    var value = smartGet(this, dependentKey);
    var neededValues = smartGet(this, neededValuesKey);
    return makeArray(neededValues).contains(value);
  });
};

/**
 * A computed property that returns true if dependent property doesn't exist in the needed values
 * <pre>
 * var o = Em.Object.create({
 *  p1: 2,
 *  p2: Em.computed.notExistsIn('p1', [1, 2, 3])
 * });
 * console.log(o.get('p2')); // false
 * o.set('p1', 4);
 * console.log(o.get('p2')); // true
 * </pre>
 *
 * @method notExistsIn
 * @param {string} dependentKey
 * @param {array} neededValues
 * @returns {Ember.ComputedProperty}
 */
computed.notExistsIn = function (dependentKey, neededValues) {
  return computed(dependentKey, function () {
    var value = smartGet(this, dependentKey);
    return !makeArray(neededValues).contains(value);
  });
};

/**
 * A computed property that returns true if dependent property doesn't exist in the property with needed values
 * <pre>
 * var o = Em.Object.create({
 *  p1: 2,
 *  p2: Em.computed.notExistsInByKey('p1', 'p3'),
 *  p3: [1, 2, 3]
 * });
 * console.log(o.get('p2')); // false
 * o.set('p1', 4);
 * console.log(o.get('p2')); // true
 * </pre>
 *
 * @method notExistsInByKey
 * @param {string} dependentKey
 * @param {string} neededValuesKey
 * @returns {Ember.ComputedProperty}
 */
computed.notExistsInByKey = function (dependentKey, neededValuesKey) {
  return computed(dependentKey, neededValuesKey + '.[]', function () {
    var value = smartGet(this, dependentKey);
    var neededValues = smartGet(this, neededValuesKey);
    return !makeArray(neededValues).contains(value);
  });
};

/**
 * A computed property that returns result of calculation <code>(dependentProperty1/dependentProperty2 * 100)</code>
 * If accuracy is 0 (by default), result is rounded to integer
 * Otherwise - result is float with provided accuracy
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 2,
 *  p2: 4,
 *  p3: Em.computed.percents('p1', 'p2')
 * });
 * console.log(o.get('p3')); // 50
 * o.set('p2', 5);
 * console.log(o.get('p3')); // 40
 * </pre>
 *
 * @method percents
 * @param {string} dependentKey1
 * @param {string} dependentKey2
 * @param {number} [accuracy=0]
 * @returns {Ember.ComputedProperty}
 */
computed.percents = function (dependentKey1, dependentKey2, accuracy) {
  if (arguments.length < 3) {
    accuracy = 0;
  }
  return computed(dependentKey1, dependentKey2, function () {
    var v1 = Number(smartGet(this, dependentKey1));
    var v2 = Number(smartGet(this, dependentKey2));
    var result = v1 / v2 * 100;
    if (0 === accuracy) {
      return Math.round(result);
    }
    return parseFloat(result.toFixed(accuracy));
  });
};

/**
 * A computed property that returns result of <code>App.format.role</code> for dependent value
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'SECONDARY_NAMENODE',
 *  p3: Em.computed.formatRole('p1', false)
 * });
 * console.log(o.get('p2')); // 'SNameNode'
 * o.set('p1', 'FLUME_HANDLER);
 * console.log(o.get('p2')); // 'Flume'
 * </pre>
 *
 * @method formatRole
 * @param {string} dependentKey
 * @param {boolean} isServiceRole
 * @returns {Ember.ComputedProperty}
 */
computed.formatRole = function (dependentKey, isServiceRole) {
  return computed(dependentKey, function () {
    var value = get(this, dependentKey);
    return App.format.role(value, isServiceRole);
  });
};

/**
 * A computed property that returns sum of the named property in the each collection's item
 * <pre>
 * var o = Em.Object.create({
 *  p1: [{a: 1}, {a: 2}, {a: 3}],
 *  p2: Em.computed.sumBy('p1', 'a')
 * });
 * console.log(o.get('p2')); // 6
 * o.set('p1.0.a', 2);
 * console.log(o.get('p2')); // 7
 * </pre>
 *
 * @method sumBy
 * @param {string} collectionKey
 * @param {string} propertyName
 * @returns {Ember.ComputedProperty}
 */
computed.sumBy = function (collectionKey, propertyName) {
  return computed(collectionKey + '.@each.' + propertyName, function () {
    var collection = smartGet(this, collectionKey);
    if (Em.isEmpty(collection)) {
      return 0;
    }
    var sum = 0;
    collection.forEach(function (item) {
      sum += Number(get(item, propertyName));
    });
    return sum;
  });
};

/**
 * A computed property that returns I18n-string formatted with dependent properties
 * Takes at least one argument
 * App.*-keys are supported
 *
 * @param {string} key key in the I18n-messages
 * @param {...string} dependentKeys
 * @method i18nFormat
 * @returns {Ember.ComputedProperty}
 */
computed.i18nFormat = generateComputedWithKey(function (key, dependentValues) {
  var str = Em.I18n.t(key);
  if (!str) {
    return '';
  }
  return str.format.apply(str, dependentValues);
});

/**
 * A computed property that returns string formatted with dependent properties
 * Takes at least one argument
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'abc',
 *  p2: 'cba',
 *  p3: Em.computed.format('{0} => {1}', 'p1', 'p2')
 * });
 * console.log(o.get('p3')); // 'abc => cba'
 * o.set('p1', 'aaa');
 * console.log(o.get('p3')); // 'aaa => cba'
 * </pre>
 *
 * @param {string} str string to format
 * @param {...string} dependentKeys
 * @method format
 * @returns {Ember.ComputedProperty}
 */
computed.format = generateComputedWithKey(function (str, dependentValues) {
  if (!str) {
    return '';
  }
  return str.format.apply(str, dependentValues);
});

/**
 * A computed property that returns dependent values joined with separator
 * Takes at least one argument
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 'abc',
 *  p2: 'cba',
 *  p3: Em.computed.concat('|', 'p1', 'p2')
 * });
 * console.log(o.get('p3')); // 'abc|cba'
 * o.set('p1', 'aaa');
 * console.log(o.get('p3')); // 'aaa|cba'
 * </pre>
 *
 * @param {string} separator
 * @param {...string} dependentKeys
 * @method concat
 * @return {Ember.ComputedProperty}
 */
computed.concat = generateComputedWithKey(function (separator, dependentValues) {
  return dependentValues.join(separator);
});

/**
 * A computed property that returns first not blank value from dependent values
 * Based on <code>Ember.isBlank</code>
 * Takes at least 1 argument
 * Dependent values order affects the result
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: null,
 *  p2: '',
 *  p3: 'abc'
 *  p4: Em.computed.firstNotBlank('p1', 'p2', 'p3')
 * });
 * console.log(o.get('p4')); // 'abc'
 * o.set('p1', 'aaa');
 * console.log(o.get('p4')); // 'aaa'
 * </pre>
 *
 * @param {...string} dependentKeys
 * @method firstNotBlank
 * @return {Ember.ComputedProperty}
 */
computed.firstNotBlank = generateComputedWithValues(function (values) {
  for (var i = 0; i < values.length; i++) {
    if (!Em.isBlank(values[i])) {
      return values[i];
    }
  }
  return null;
});

/**
 * A computed property that returns dependent value if it is truly or ('0'|0)
 * Returns <code>'n/a'</code> otherwise
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 0,
 *  p2: Em.computed.formatUnavailable('p1')
 * });
 * console.log(o.get('p2')); // 0
 * o.set('p1', 12);
 * console.log(o.get('p2')); // 12
 * o.set('p1', 'some string');
 * console.log(o.get('p2')); // 'n/a'
 * </pre>
 *
 * @param {string} dependentKey
 * @method formatUnavailable
 * @returns {Ember.ComputedProperty}
 */
computed.formatUnavailable = function (dependentKey) {
  return computed(dependentKey, function () {
    var value = smartGet(this, dependentKey);
    return value || value == 0 ? value : Em.I18n.t('services.service.summary.notAvailable');
  });
};

/**
 * A computed property that returns one of provided values basing on dependent value
 * If dependent value is 0, <code>zeroMsg</code> is returned
 * If dependent value is 1, <code>oneMsg</code> is returned
 * If dependent value is greater than 1, <code>manyMsg</code> is returned
 * App.*-keys are supported
 * <pre>
 * var o = Em.Object.create({
 *  p1: 0,
 *  p2: Em.computed.formatUnavailable('p1', '0msg', '1msg', '2+msg')
 * });
 * console.log(o.get('p2')); // '0msg'
 * o.set('p1', 1);
 * console.log(o.get('p2')); // '1msg'
 * o.set('p1', 100500);
 * console.log(o.get('p2')); // '2+msg'
 * </pre>
 *
 * @param {string} dependentKey
 * @param {string} zeroMsg
 * @param {string} oneMsg
 * @param {string} manyMsg
 * @returns {Ember.ComputedProperty}
 * @method countBasedMessage
 */
computed.countBasedMessage = function (dependentKey, zeroMsg, oneMsg, manyMsg) {
  return computed(dependentKey, function () {
    var value = Number(smartGet(this, dependentKey));
    if (value === 0) {
      return zeroMsg;
    }
    if (value > 1) {
      return manyMsg;
    }
    return oneMsg;
  });
};

/**
 * A computed property that returns property value according to the property key and object key
 * App.*-keys are supported
 * <pre>
 *   var o = Em.Object.create({
 *    p1: {a: 1, b: 2, c: 3},
 *    p2: 'a',
 *    p3: Em.computed.getByKey('p1', 'p2')
 *   });
 *   console.log(o.get('p3')); // 1
 *   o.set('p2', 'b');
 *   console.log(o.get('p3')); // 2
 *   o.set('p2', 'c');
 *   console.log(o.get('p3')); // 3
 * </pre>
 *
 * With `defaultValue`
 * <pre>
 *   var o = Em.Object.create({
 *    p1: {a: 1, b: 2, c: 3},
 *    p2: 'd',
 *    p3: Em.computed.getByKey('p1', 'p2', 100500)
 *   });
 *   console.log(o.get('p3')); // 100500 - default value is returned, because there is no key `d` in the `p1`
 * </pre>
 * <b>IMPORTANT!</b> This CP <b>SHOULD NOT</b> be used with for object with values equal to the views (like <code>{a: App.MyViewA, b: App.MyViewB}</code>)
 * This restriction exists because views may be undefined on the moment when this CP is calculated (files are not `required` yet)
 *
 * @param {string} objectKey
 * @param {string} propertyKey
 * @param {*} [defaultValue]
 * @returns {Ember.ComputedProperty}
 */
computed.getByKey = function (objectKey, propertyKey, defaultValue) {
  return computed(objectKey, propertyKey, function () {
    var object = smartGet(this, objectKey);
    var property = smartGet(this, propertyKey);
    if (!object) {
      return null;
    }
    return object.hasOwnProperty(property) ? object[property] : defaultValue;
  });
};

/**
 * A computed property that returns dependent value truncated to the `reduceTo`-size if its length is greater than `maxLength`
 * Truncated part may be replaced with `replacer` if it's provided ('...' by default)
 * <pre>
 *   var o = Em.Object.create({
 *     p1: Em.computed.truncate('p2', 8, 5, '###'),
 *     p2: 'some string',
 *     p3: Em.computed.truncate('p2', 8, 5)
 *   });
 *   console.log(o.get('p1')); // 'some ###'
 *   console.log(o.get('p3')); // 'some ...'
 *   o.set('p2', '123456789');
 *   console.log(o.get('p1')); // '12345###'
 *   console.log(o.get('p3')); // '12345...'
 * </pre>
 *
 * @param {string} dependentKey
 * @param {number} maxLength
 * @param {number} reduceTo
 * @param {string} [replacer] default - '...'
 * @returns {Ember.ComputedProperty}
 */
computed.truncate = function (dependentKey, maxLength, reduceTo, replacer) {
  Em.assert('`reduceTo` should be <=`maxLength`', reduceTo <= maxLength);
  var _replacer = arguments.length > 3 ? replacer : '...';
  return computed(dependentKey, function () {
    var value = smartGet(this, dependentKey) || '';
    if (value.length > maxLength) {
      return value.substr(0, reduceTo) + _replacer;
    }
    return value;
  });
};

});

require.register("utils/ember_reopen", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var _Ember = Ember,
    assert = _Ember.assert,
    warn = _Ember.warn,
    EXTEND_PROTOTYPES = _Ember.EXTEND_PROTOTYPES;


var END_WITH_EACH_REGEX = /\.@each$/;
var DEEP_EACH_REGEX = /\.@each\.[^.]+\./;

/**
 Merge the contents of two objects together into the first object.

 ```javascript
 Ember.merge({first: 'Tom'}, {last: 'Dale'}); // {first: 'Tom', last: 'Dale'}
 var a = {first: 'Yehuda'}, b = {last: 'Katz'};
 Ember.merge(a, b); // a == {first: 'Yehuda', last: 'Katz'}, b == {last: 'Katz'}
 ```

 @method merge
 @for Ember
 @param {Object} original The object to merge into
 @param {Object} updates The object to copy properties from
 @return {Object}
 */
Ember.merge = function (original, updates) {
  for (var prop in updates) {
    if (!updates.hasOwnProperty(prop)) {
      continue;
    }
    original[prop] = updates[prop];
  }
  return original;
};

/**
 Returns true if the passed value is null or undefined. This avoids errors
 from JSLint complaining about use of ==, which can be technically
 confusing.

 ```javascript
 Ember.isNone();              // true
 Ember.isNone(null);          // true
 Ember.isNone(undefined);     // true
 Ember.isNone('');            // false
 Ember.isNone([]);            // false
 Ember.isNone(function() {});  // false
 ```

 @method isNone
 @for Ember
 @param {Object} obj Value to test
 @return {Boolean}
 */
Ember.isNone = function (obj) {
  return obj === null || obj === undefined;
};

/**
 Verifies that a value is `null` or an empty string, empty array,
 or empty function.

 Constrains the rules on `Ember.isNone` by returning false for empty
 string and empty arrays.

 ```javascript
 Ember.isEmpty();                // true
 Ember.isEmpty(null);            // true
 Ember.isEmpty(undefined);       // true
 Ember.isEmpty('');              // true
 Ember.isEmpty([]);              // true
 Ember.isEmpty('Adam Hawkins');  // false
 Ember.isEmpty([0,1,2]);         // false
 ```

 @method isEmpty
 @for Ember
 @param {Object} obj Value to test
 @return {Boolean}
 */
Ember.isEmpty = function (obj) {
  return Ember.isNone(obj) || obj.length === 0 && typeof obj !== 'function' || (typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' && Ember.get(obj, 'length') === 0;
};

/**
 A value is blank if it is empty or a whitespace string.

 ```javascript
 Ember.isBlank();                // true
 Ember.isBlank(null);            // true
 Ember.isBlank(undefined);       // true
 Ember.isBlank('');              // true
 Ember.isBlank([]);              // true
 Ember.isBlank('\n\t');          // true
 Ember.isBlank('  ');            // true
 Ember.isBlank({});              // false
 Ember.isBlank('\n\t Hello');    // false
 Ember.isBlank('Hello world');   // false
 Ember.isBlank([1,2,3]);         // false
 ```

 @method isBlank
 @for Ember
 @param {Object} obj Value to test
 @return {Boolean}
 */
Ember.isBlank = function (obj) {
  return Ember.isEmpty(obj) || typeof obj === 'string' && obj.match(/\S/) === null;
};

/**
 * Calculates sum of two numbers
 * Use this function as a callback on <code>invoke</code> etc
 *
 * @method sum
 * @param {Number} a
 * @param {Number} b
 * @returns {Number}
 */
Ember.sum = function (a, b) {
  return a + b;
};

/**
 * Execute passed callback
 *
 * @param {Function} callback
 * @returns {*}
 */
Ember.clb = function (callback) {
  return callback();
};

/**
 *
 */
Ember.RadioButton = Ember.Checkbox.extend({
  tagName: "input",
  type: "radio",
  attributeBindings: ["type", "name", "value", "checked", "style", "disabled"],
  click: function click() {
    this.set("selection", this.$().val());
  },
  checked: function () {
    return this.get("value") === this.get("selection");
  }.property('value', 'selection')
});

/**
 * Set value to obj by path
 * Create nested objects if needed
 * Example:
 * <code>
 *   var a = {b: {}};
 *   var path = 'b.c.d';
 *   var value = 1;
 *   Em.setFullPath(a, path, value); // a = {b: {c: {d: 1}}}
 * </code>
 *
 * @param {object} obj
 * @param {string} path
 * @param {*} value
 */
Ember.setFullPath = function (obj, path, value) {
  var parts = path.split('.'),
      sub_path = '';
  parts.forEach(function (_path, _index) {
    Em.assert('path parts can\'t be empty', _path.length);
    sub_path += '.' + _path;
    if (_index === parts.length - 1) {
      Em.set(obj, sub_path, value);
      return;
    }
    if (Em.isNone(Em.get(obj, sub_path))) {
      Em.set(obj, sub_path, {});
    }
  });
};

/**
 *
 * @param {object} target
 * @param {string[]} propertyNames
 * @returns {{}}
 */
Ember.getProperties = function (target, propertyNames) {
  var ret = {};
  for (var i = 0; i < propertyNames.length; i++) {
    ret[propertyNames[i]] = Em.get(target, propertyNames[i]);
  }
  return ret;
};

Em.View.reopen({
  /**
   * overwritten set method of Ember.View to avoid uncaught errors
   * when trying to set property of destroyed view
   */
  set: function set(attr, value) {
    if (!this.get('isDestroyed') && !this.get('isDestroying')) {
      this._super(attr, value);
    } else {
      console.debug('Calling set on destroyed view');
    }
  },

  /**
   * overwritten setProperties method of Ember.View to avoid uncaught errors
   * when trying to set multiple properties of destroyed view
   */
  setProperties: function setProperties(hash) {
    if (!this.get('isDestroyed') && !this.get('isDestroying')) {
      this._super(hash);
    } else {
      console.debug('Calling setProperties on destroyed view');
    }
  },

  attributeBindings: ['data-qa']
});

Ember._HandlebarsBoundView.reopen({
  /**
   * overwritten set method of Ember._HandlebarsBoundView to avoid uncaught errors
   * when trying to set property of destroyed view
   */
  render: function render(buffer) {
    if (!this.get('isDestroyed') && !this.get('isDestroying')) {
      this._super(buffer);
    } else {
      console.debug('Calling render on destroyed view');
    }
  }
});

Ember.TextField.reopen({
  attributeBindings: ['readOnly']
});

Ember.TextArea.reopen({
  attributeBindings: ['readonly']
});

/**
 * Simply converts query string to object.
 *
 * @param  {string} queryString query string e.g. '?param1=value1&param2=value2'
 * @return {object} converted object
 */
function parseQueryParams(queryString) {
  if (!queryString) {
    return {};
  }
  return queryString.replace(/^\?/, '').split('&').map(decodeURIComponent).reduce(function (p, c) {
    var keyVal = c.split('=');
    p[keyVal[0]] = keyVal[1];
    return p;
  }, {});
};

Ember.Route.reopen({
  /**
   *  When you move to a new route by pressing the back or forward button, change url manually, click on link with url defined in href,
   *  call Router.transitionTo or Router.route this method is called.
   *  This method unites unroutePath, navigateAway and exit events to handle Route leaving in one place.
   *  Also unlike the exit event it is possible to stop transition inside this handler.
   *  To proceed transition just call callback..
   *
   * @param {Router}  router
   * @param {Object|String} context context from transition or path from route
   * @param {callback} callback should be called to proceed transition
   */
  exitRoute: function exitRoute(router, context, callback) {
    callback();
  },

  /**
   * Query Params serializer. This method should be used inside <code>serialize</code> method.
   * You need to specify `:query` dynamic sygment in your route's <code>route</code> attribute
   * e.g. Em.Route.extend({ route: '/login:query'}) and return result of this method.
   * This method will set <code>serializedQuery</code> property to specified controller by name.
   * For concrete example see `app/routes/main.js`.
   *
   * @example
   *  queryParams: Em.Route.extend({
   *   route: '/queryDemo:query',
   *   serialize: function(route, params) {
   *     return this.serializeQueryParams(route, params, 'controllerNameToSetQueryObject');
   *   }
   *  });
   *  // now when navigated to http://example.com/#/queryDemo?param1=value1&param2=value2
   *  // App.router.get('controllerNameToSetQueryObject').get('serializedQuery')
   *  // will return { param1: 'value1', param2: 'value2' }
   *
   * @param  {Em.Router} router router instance passed to <code>serialize</code> method
   * @param  {object} params dynamic segment passed to <code>seriazlie</code>
   * @param  {string} controllerName name of the controller to set `serializedQuery` as result
   * @return {object}
   */
  serializeQueryParams: function serializeQueryParams(router, params, controllerName) {
    var controller = router.get(controllerName);
    controller.set('serializedQuery', parseQueryParams(params ? params.query : ''));
    return params || { query: '' };
  },

  scrollTop: function scrollTop() {
    window.scrollTo(0, 0);
  }
});

Ember.Router.reopen({

  // reopen original transitionTo and route methods to add calling of exitRoute

  transitionTo: function transitionTo(router, context) {
    var self = this;
    var args = arguments;
    var transitionTo = self._super;
    var callback = function callback() {
      transitionTo.apply(self, args);
    };
    if (!this.get('currentState.exitRoute')) {
      callback();
    } else {
      this.get('currentState').exitRoute(this, context, callback);
    }
  },

  route: function route(path) {
    var self = this;
    var args = arguments;
    var transitionTo = self._super;
    var callback = function callback() {
      self.get('location').setURL(path);
      transitionTo.apply(self, args);
    };
    var realPath;
    if (!this.get('currentState.exitRoute')) {
      callback();
    } else {
      realPath = this.get('currentState').absoluteRoute(this);
      this.get('location').setURL(realPath);
      this.get('currentState').exitRoute(this, path, callback);
    }
  }
});

function expandProperties(pattern, callback) {
  assert('A computed property key must be a string', typeof pattern === 'string');
  assert('Brace expanded properties cannot contain spaces, e.g. "user.{firstName, lastName}" should be "user.{firstName,lastName}"', pattern.indexOf(' ') === -1);

  var unbalancedNestedError = 'Brace expanded properties have to be balanced and cannot be nested, pattern: ' + pattern;
  var properties = [pattern];

  // Iterating backward over the pattern makes dealing with indices easier.
  var bookmark = void 0;
  var inside = false;
  for (var i = pattern.length; i > 0; --i) {
    var current = pattern[i - 1];

    switch (current) {
      // Closing curly brace will be the first character of the brace expansion we encounter.
      // Bookmark its index so long as we're not already inside a brace expansion.
      case '}':
        if (!inside) {
          bookmark = i - 1;
          inside = true;
        } else {
          assert(unbalancedNestedError, false);
        }
        break;
      // Opening curly brace will be the last character of the brace expansion we encounter.
      // Apply the brace expansion so long as we've already seen a closing curly brace.
      case '{':
        if (inside) {
          var expansion = pattern.slice(i, bookmark).split(',');
          // Iterating backward allows us to push new properties w/out affecting our "cursor".
          for (var j = properties.length; j > 0; --j) {
            // Extract the unexpanded property from the array.
            var property = properties.splice(j - 1, 1)[0];
            // Iterate over the expansion, pushing the newly formed properties onto the array.
            for (var k = 0; k < expansion.length; ++k) {
              properties.push(property.slice(0, i - 1) + expansion[k] + property.slice(bookmark + 1));
            }
          }
          inside = false;
        } else {
          assert(unbalancedNestedError, false);
        }
        break;
    }
  }
  if (inside) {
    assert(unbalancedNestedError, false);
  }

  for (var _i = 0; _i < properties.length; _i++) {
    callback(properties[_i].replace(END_WITH_EACH_REGEX, '.[]'));
  }
}

Ember.ComputedProperty.prototype.property = function () {
  var args = [];

  function addArg(property) {
    warn('Dependent keys containing @each only work one level deep. ' + ('You used the key "' + property + '" which is invalid. ') + 'Please create an intermediary computed property.', DEEP_EACH_REGEX.test(property) === false);
    args.push(property);
  }

  for (var i = 0; i < arguments.length; i++) {
    expandProperties(arguments[i], addArg);
  }

  this._dependentKeys = args.uniq();
  return this;
};

Ember.observer = function (func) {
  var args = [];

  for (var _len = arguments.length, paths = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    paths[_key - 1] = arguments[_key];
  }

  for (var i = 0; i < paths.length; i++) {
    expandProperties(paths[i], function (property) {
      return args.push(property);
    });
  }
  func.__ember_observes__ = args.uniq();
  return func;
};

if (EXTEND_PROTOTYPES) {
  Function.prototype.observes = function () {
    var args = [];
    for (var i = 0; i < arguments.length; i++) {
      expandProperties(arguments[i], function (property) {
        return args.push(property);
      });
    }
    this.__ember_observes__ = args.uniq();
    return this;
  };
}

});

;require.register("utils/errors/assertions", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('utils/errors/definitions');

/**
 * Method to check custom statement and
 * throws custom error if statement is false
 *
 * @param desc
 * @param test
 * @param ErrorObject
 */
App.assert = function (desc, test, ErrorObject) {
  if (!test) {
    ErrorObject = ErrorObject || Error;
    throw new ErrorObject(desc ? ' Info:  ' + desc : '');
  }
};

/**
 * Check if passed variable should be not null
 *
 * @param object
 * @param desc
 */
App.assertExists = function (object, desc) {
  App.assert(desc, !Em.isNone(object), App.NotNullTypeError);
};

/**
 * Check if passed variable is object
 *
 * @param object
 * @param desc
 */
App.assertObject = function (object, desc) {
  App.assert(desc, (typeof object === 'undefined' ? 'undefined' : _typeof(object)) === 'object' && object, App.ObjectTypeError);
};

/**
 * Check if variable is instance of ember object
 *
 * @param object
 * @param desc
 */
App.assertEmberObject = function (object, desc) {
  App.assert(desc, Em.typeOf(object) === 'instance', App.EmberObjectTypeError);
};

/**
 * Check if variable is array
 *
 * @param object
 * @param desc
 */
App.assertArray = function (object, desc) {
  App.assert(desc, Em.isArray(object), App.ArrayTypeError);
};

/**
 * Check if variable is function
 *
 * @param object
 * @param desc
 */
App.assertFunction = function (object, desc) {
  App.assert(desc, Em.typeOf(object) === 'function', App.FunctionTypeError);
};

});

require.register("utils/errors/definitions", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Error that should be used when
 * not null type required
 *
 * @param message
 * @constructor
 */
App.NotNullTypeError = function (message) {
  this.name = "NotNullTypeError";
  this.message = "Not null expected. " + (message || "");
};

App.NotNullTypeError.prototype = new TypeError();

/**
 * Error that should be used when
 * object required
 *
 * @param message
 * @constructor
 */
App.ObjectTypeError = function (message) {
  this.name = "ObjectTypeError";
  this.message = "Object expected. " + (message || "");
};

App.ObjectTypeError.prototype = new App.NotNullTypeError();

/**
 * Error that should be used when
 * array required
 *
 * @param message
 * @constructor
 */
App.ArrayTypeError = function (message) {
  this.name = "ArrayTypeError";
  this.message = "Array expected. " + (message || "");
};

App.ArrayTypeError.prototype = new App.NotNullTypeError();
/**
 * Error that should be used when
 * function required
 *
 * @param message
 * @constructor
 */
App.FunctionTypeError = function (message) {
  this.name = "FunctionTypeError";
  this.message = "Function expected. " + (message || "");
};

App.FunctionTypeError.prototype = new App.NotNullTypeError();
/**
 * Error that should be used when
 * ember object required
 *
 * @param message
 * @constructor
 */
App.EmberObjectTypeError = function (message) {
  this.name = "EmberObjectTypeError";
  this.message = "Ember object expected. " + (message || "");
};

App.EmberObjectTypeError.prototype = new App.ObjectTypeError();

});

require.register("utils/file_utils", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {

  fileTypeMap: {
    csv: 'text/csv',
    json: 'application/json'
  },

  /**
   * download text file
   * @param data {String}
   * @param fileType {String}
   * @param fileName {String}
   */
  downloadTextFile: function downloadTextFile(data, fileType, fileName) {
    if ($.browser.msie && $.browser.version < 10) {
      this.openInfoInNewTab(data);
    } else if (typeof safari !== 'undefined') {
      this.safariDownload(data, fileType, fileName);
    } else {
      try {
        var blob = new Blob([data], {
          type: (this.fileTypeMap[fileType] || 'text/' + fileType) + ';charset=utf-8;'
        });
        saveAs(blob, fileName);
      } catch (e) {
        this.openInfoInNewTab(data);
      }
    }
  },

  /**
   * download multiple files archived in ZIP
   * @param {object[]} files
   */
  downloadFilesInZip: function downloadFilesInZip(files) {
    var zip = new JSZip();
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = files[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var file = _step.value;

        var blob = new Blob([file.data], {
          type: (this.fileTypeMap[file.type] || 'text/' + file.type) + ';charset=utf-8;'
        });
        zip.file(file.name, blob);
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    zip.generateAsync({ type: "blob" }).then(function (content) {
      saveAs(content, "blueprint.zip");
    });
  },

  /**
   * open content of text file in new window
   * @param data {String}
   */
  openInfoInNewTab: function openInfoInNewTab(data) {
    var newWindow = window.open('');
    var newDocument = newWindow.document;
    newDocument.write(data);
    newWindow.focus();
  },

  /**
   * Hack to dowload text data in Safari
   * @param data {String}
   * @param fileType {String}
   * @param fileName {String}
   */
  safariDownload: function safariDownload(data, fileType, fileName) {
    var file = 'data:attachment/' + fileType + ';charset=utf-8,' + encodeURI(data);
    var linkEl = document.createElement("a");
    linkEl.href = file;
    linkEl.download = fileName;

    document.body.appendChild(linkEl);
    linkEl.click();
    document.body.removeChild(linkEl);
  },

  fileNameFromPath: function fileNameFromPath(path) {
    return path.split('/').slice(-1);
  }

};

});

require.register("utils/handlebars_helpers", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Helper function for bound property helper registration
 * @memberof App
 * @method registerBoundHelper
 * @param name {String} name of helper
 * @param view {Em.View} view
 */
App.registerBoundHelper = function (name, view) {
  Em.Handlebars.registerHelper(name, function (property, options) {
    options.hash.contentBinding = property;
    return Em.Handlebars.helpers.view.call(this, view, options);
  });
};

/*
 * Return singular or plural word based on Em.I18n, view|controller context property key.
 *
 *  Example: {{pluralize hostsCount singular="t:host" plural="t:hosts"}}
 *           {{pluralize hostsCount singular="@view.hostName"}}
 */
App.registerBoundHelper('pluralize', App.PluralizeView);
/**
 * Return defined string instead of empty if value is null/undefined
 * by default is `n/a`.
 *
 * @param empty {String} - value instead of empty string (not required)
 *  can be used with Em.I18n pass value started with't:'
 *
 * Examples:
 *
 * default value will be returned
 * {{formatNull service.someValue}}
 *
 * <code>empty<code> will be returned
 * {{formatNull service.someValue empty="I'm empty"}}
 *
 * Em.I18n translation will be returned
 * {{formatNull service.someValue empty="t:my.key.to.translate"
 */
App.registerBoundHelper('formatNull', App.FormatNullView);

/**
 * Return formatted string with inserted <code>wbr</code>-tag after each dot or each '_'
 *
 * @param {String} content
 *
 * Examples:
 *
 * returns 'apple'
 * {{formatWordBreak 'apple'}}
 *
 * returns 'apple.<wbr />banana'
 * {{formatWordBreak 'apple.banana'}}
 *
 * returns 'apple.<wbr />banana.<wbr />uranium'
 * {{formatWordBreak 'apple.banana.uranium'}}
 *
 * returns 'apple_<wbr />banana_<wbr />uranium'
 * {{formatWordBreak 'apple_banana_uranium'}}
 *
 * returns 'Very<wbr />Long<wbr />String<wbr />With<wbr />Uppercase'
 * {{formatWordBreak 'VeryLongStringWithUppercase'}}
 */
App.registerBoundHelper('formatWordBreak', App.FormatWordBreakView);

/**
 * Return <i></i> with class that correspond to status
 *
 * @param {string} content - status
 *
 * Examples:
 *
 * {{statusIcon view.status}}
 * returns 'glyphicon glyphicon-cog'
 *
 */
App.registerBoundHelper('statusIcon', App.StatusIconView);

/**
 * Return `span` with formatted service name
 * @param {string} content - serviceName
 */
App.registerBoundHelper('formatRole', App.FormatRoleView);

});

require.register("utils/helper", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var stringUtils = require('utils/string_utils');
var timezoneUtils = require('utils/date/timezone');

/**
 * Remove spaces at beginning and ending of line.
 * @example
 *  var str = "  I'm a string  "
 *  str.trim() // return "I'm a string"
 * @method trim
 * @return {string}
 */
String.prototype.trim = function () {
  return this.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
};
/**
 * Determines whether string end within another string.
 *
 * @method endsWith
 * @param suffix {string}  substring for search
 * @return {boolean}
 */
String.prototype.endsWith = function (suffix) {
  return this.indexOf(suffix, this.length - suffix.length) !== -1;
};
/**
 * Determines whether string start within another string.
 *
 * @method startsWith
 * @param prefix {string} substring for search
 * @return {boolean}
 */
String.prototype.startsWith = function (prefix) {
  return this.indexOf(prefix) == 0;
};
/**
 * Determines whether string founded within another string.
 *
 * @method contains
 * @param substring {string} substring for search
 * @return {boolean}
 */
String.prototype.contains = function (substring) {
  return this.indexOf(substring) != -1;
};
/**
 * Capitalize the first letter of string.
 * @method capitalize
 * @return {string}
 */
String.prototype.capitalize = function () {
  return this.charAt(0).toUpperCase() + this.slice(1);
};

/**
 * Capitalize the first letter of string.
 * And set to lowercase other part of string
 * @method toCapital
 * @return {string}
 */
String.prototype.toCapital = function () {
  return this.charAt(0).toUpperCase() + this.slice(1).toLowerCase();
};

/**
 * Finds the value in an object where this string is a key.
 * Optionally, the index of the key can be provided where the
 * value of the nth key in the hierarchy is returned.
 *
 * Example:
 *  var tofind = 'smart';
 *  var person = {'name': 'Bob Bob', 'smart': 'no', 'age': '28', 'personality': {'smart': 'yes', 'funny': 'yes', 'emotion': 'happy'} };
 *  tofind.findIn(person); // 'no'
 *  tofind.findIn(person, 0); // 'no'
 *  tofind.findIn(person, 1); // 'yes'
 *  tofind.findIn(person, 2); // null
 *
 *  @method findIn
 *  @param multi {object}
 *  @param index {number} Occurrence count of this key
 *  @return {*} Value of key at given index
 */
String.prototype.findIn = function (multi, index, _foundValues) {
  if (!index) {
    index = 0;
  }
  if (!_foundValues) {
    _foundValues = [];
  }
  multi = multi || '';
  var value = null;
  var str = this.valueOf();
  if ((typeof multi === 'undefined' ? 'undefined' : _typeof(multi)) == 'object') {
    for (var key in multi) {
      if (value != null) {
        break;
      }
      if (key == str) {
        _foundValues.push(multi[key]);
      }
      if (_foundValues.length - 1 == index) {
        // Found the value
        return _foundValues[index];
      }
      if (_typeof(multi[key]) == 'object') {
        value = value || this.findIn(multi[key], index, _foundValues);
      }
    }
  }
  return value;
};
/**
 * Replace {i} with argument. where i is number of argument to replace with.
 * @example
 *  var str = "{0} world{1}";
 *  str.format("Hello", "!") // return "Hello world!"
 *
 * @method format
 * @return {string}
 */
String.prototype.format = function () {
  var args = arguments;
  return this.replace(/{(\d+)}/g, function (match, number) {
    return typeof args[number] != 'undefined' ? args[number] : match;
  });
};
/**
 * Wrap words in string within template.
 *
 * @method highlight
 * @param {string[]} words - words to wrap
 * @param {string} [highlightTemplate="<b>{0}</b>"] - template for wrapping
 * @return {string}
 */
String.prototype.highlight = function (words, highlightTemplate) {
  var self = this;
  highlightTemplate = highlightTemplate ? highlightTemplate : "<b>{0}</b>";

  words.forEach(function (word) {
    var searchRegExp = new RegExp("\\b" + word + "\\b", "gi");
    self = self.replace(searchRegExp, function (found) {
      return highlightTemplate.format(found);
    });
  });

  return self;
};
/**
 * Convert time in milliseconds to object contained days, hours and minutes.
 * @typedef ConvertedTime
 *  @type {Object}
 *  @property {number} d - days
 *  @property {number} h - hours
 *  @property {string} m - minutes
 * @example
 *  var time = 1000000000;
 *  time.toDaysHoursMinutes() // {d: 11, h: 13, m: "46.67"}
 *
 * @method toDaysHoursMinutes
 * @return {object}
 */
Number.prototype.toDaysHoursMinutes = function () {
  var formatted = {},
      dateDiff = this,
      secK = 1000,
      //ms
  minK = 60 * secK,
      // sec
  hourK = 60 * minK,
      // sec
  dayK = 24 * hourK;

  dateDiff = parseInt(dateDiff);
  formatted.d = Math.floor(dateDiff / dayK);
  dateDiff -= formatted.d * dayK;
  formatted.h = Math.floor(dateDiff / hourK);
  dateDiff -= formatted.h * hourK;
  formatted.m = (dateDiff / minK).toFixed(2);

  return formatted;
};

/**
 *
 * @param bound1 {Number}
 * @param bound2 {Number}
 * @return {boolean}
 */
Number.prototype.isInRange = function (bound1, bound2) {
  var upperBound, lowerBound;
  upperBound = bound1 > bound2 ? bound1 : bound2;
  lowerBound = bound1 < bound2 ? bound1 : bound2;
  return this > lowerBound && this < upperBound;
};

/**
 Sort an array by the key specified in the argument.
 Handle only native js objects as element of array, not the Ember's object.

 Can be used as alternative to sortProperty method of Ember library
 in order to speed up executing on large data volumes

 @method sortBy
 @param {String} path name(s) to sort on
 @return {Array} The sorted array.
 */
Array.prototype.sortPropertyLight = function (path) {
  var realPath = typeof path === "string" ? path.split('.') : [];
  this.sort(function (a, b) {
    var aProperty = a;
    var bProperty = b;
    realPath.forEach(function (key) {
      aProperty = aProperty[key];
      bProperty = bProperty[key];
    });
    if (aProperty > bProperty) return 1;
    if (aProperty < bProperty) return -1;
    return 0;
  });
  return this;
};

/**
 * Create map from array with executing provided callback for each array's item
 * Example:
 * <pre>
 *   var array = [{a: 1, b: 3}, {a: 2, b: 2}, {a: 3, b: 1}];
 *   var map = array.toMapByCallback('a', function (item) {
 *    return Em.get(item, 'b');
 *   });
 *   console.log(map); // {1: 3, 2: 2, 3: 1}
 * </pre>
 * <code>map[1]</code> is much more faster than <code>array.findProperty('a', 1).get('b')</code>
 *
 * @param {string} property
 * @param {Function} callback
 * @returns {object}
 * @method toMapByCallback
 */
Array.prototype.toMapByCallback = function (property, callback) {
  var ret = {};
  Em.assert('`property` can\'t be empty string', property.length);
  Em.assert('`callback` should be a function', 'function' === Em.typeOf(callback));
  this.forEach(function (item) {
    var key = Em.get(item, property);
    ret[key] = callback(item, property);
  });
  return ret;
};

/**
 * Create map from array
 * Example:
 * <pre>
 *   var array = [{a: 1}, {a: 2}, {a: 3}];
 *   var map = array.toMapByProperty('a'); // {1: {a: 1}, 2: {a: 2}, 3: {a: 3}}
 * </pre>
 * <code>map[1]</code> is much more faster than <code>array.findProperty('a', 1)</code>
 *
 * @param {string} property
 * @return {object}
 * @method toMapByProperty
 * @see toMapByCallback
 */
Array.prototype.toMapByProperty = function (property) {
  return this.toMapByCallback(property, function (item) {
    return item;
  });
};

/**
 * Create wick map from array
 * Example:
 * <pre>
 *   var array = [{a: 1}, {a: 2}, {a: 3}];
 *   var map = array.toWickMapByProperty('a'); // {1: true, 2: true, 3: true}
 * </pre>
 * <code>map[1]</code> works faster than <code>array.someProperty('a', 1)</code>
 *
 * @param {string} property
 * @return {object}
 * @method toWickMapByProperty
 * @see toMapByCallback
 */
Array.prototype.toWickMapByProperty = function (property) {
  return this.toMapByCallback(property, function () {
    return true;
  });
};

/**
 * Create wick map from array of primitives
 * Example:
 * <pre>
 *   var array = [1, 2, 3];
 *   var map = array.toWickMap(); // {1: true, 2: true, 3: true}
 * </pre>
 * <code>map[1]</code> works faster than <code>array.contains(1)</code>
 *
 * @returns {object}
 * @method toWickMap
 */
Array.prototype.toWickMap = function () {
  var ret = {};
  this.forEach(function (item) {
    ret[item] = true;
  });
  return ret;
};

if (!Array.prototype.findIndex) {
  Array.prototype.findIndex = function (predicate) {
    if (this == null) {
      throw new TypeError('"this" is null or not defined');
    }
    var o = Object(this),
        len = o.length >>> 0;
    if (typeof predicate !== 'function') {
      throw new TypeError('predicate must be a function');
    }
    var thisArg = arguments[1],
        k = 0;
    while (k < len) {
      var kValue = o[k];
      if (predicate.call(thisArg, kValue, k, o)) {
        return k;
      }
      k++;
    }
    return -1;
  };
}

/** @namespace Em **/
Em.CoreObject.reopen({
  t: function t(key, attrs) {
    return Em.I18n.t(key, attrs);
  }
});

Em.TextArea.reopen(Em.I18n.TranslateableAttributes);

/** @namespace Em.Handlebars **/
Em.Handlebars.registerHelper('log', function (variable) {
  console.log(variable);
});

Em.Handlebars.registerHelper('warn', function (variable) {
  console.warn(variable);
});

Em.Handlebars.registerHelper('highlight', function (property, words, fn) {
  var context = fn.contexts && fn.contexts[0] || this;
  property = Em.Handlebars.getPath(context, property, fn);

  words = words.split(";");

  //  if (highlightTemplate == undefined) {
  var highlightTemplate = "<b>{0}</b>";
  //  }

  words.forEach(function (word) {
    var searchRegExp = new RegExp("\\b" + word + "\\b", "gi");
    property = property.replace(searchRegExp, function (found) {
      return highlightTemplate.format(found);
    });
  });

  return new Em.Handlebars.SafeString(property);
});

/**
 * Usage:
 *
 * <div {{QAAttr "someText"}}></div>
 * <div {{QAAttr "{someProperty}"}}></div>
 * <div {{QAAttr "someText-and-{someProperty}"}}></div>
 * <div {{QAAttr "{someProperty:some-text}"}}></div>
 * <div {{QAAttr "someText-and-{someProperty:some-text}"}}></div>
 * <div {{QAAttr "{someProperty:some-text:another-text}"}}></div>
 * <div {{QAAttr "someText-and-{someProperty:some-text:another-text}"}}></div>
 * <div {{QAAttr "{someProperty::another-text}"}}></div>
 * <div {{QAAttr "someText-and-{someProperty::another-text}"}}></div>
 *
 */
Em.Handlebars.registerHelper('QAAttr', function (text, options) {
  var _this = this;

  var textToReplace = text.match(/\{(.*?)\}/g);
  var attributes = void 0;
  if (textToReplace) {
    var id = ++Em.$.uuid,
        expressions = textToReplace.map(function (str) {
      var parsed = Em.View._parsePropertyPath(str.slice(1, str.length - 1)),
          normalized = Ember.Handlebars.normalizePath(_this, parsed.path, options.data),
          classNames = parsed.classNames,
          className = parsed.className,
          falsyClassName = parsed.falsyClassName,
          root = normalized.root,
          path = normalized.path;

      return { src: str, classNames: classNames, className: className, falsyClassName: falsyClassName, root: root, path: path };
    }),
        observer = function observer() {
      var dataQA = text;
      for (var i = expressions.length; i--;) {
        var el = Em.tryInvoke(options.data.view, '$', ['[' + attributes + ']']);
        var e = expressions[i];
        if (!el || el.length === 0) {
          Em.removeObserver(e.root, e.path, invoker);
          break;
        }
        var value = void 0,
            sourceValue = Em.Handlebars.getPath(e.root, e.path, options.data);
        if (e.classNames) {
          value = sourceValue ? e.className : e.falsyClassName;
        } else {
          value = sourceValue;
        }
        if (Em.isNone(value)) {
          value = '';
        }
        dataQA = dataQA.replace(e.src, value);
        el.attr('data-qa', dataQA);
      }
    },
        invoker = function invoker() {
      return Em.run.once(observer);
    };
    attributes = 'data-qa-bind-id="' + id + '"';
    expressions.forEach(function (e) {
      Em.addObserver(e.root, e.path, invoker);
    });
    Em.run.next(observer);
  } else {
    attributes = 'data-qa="' + text + '"';
  }
  return new Em.Handlebars.SafeString(attributes);
});

/**
 * Usage:
 *
 * <pre>
 *   {{#isAuthorized "SERVICE.TOGGLE_ALERTS"}}
 *     {{! some truly code }}
 *   {{else}}
 *     {{! some falsy code }}
 *   {{/isAuthorized}}
 * </pre>
 */
Em.Handlebars.registerHelper('isAuthorized', function (property, options) {
  var permission = Ember.Object.create({
    isAuthorized: function () {
      return App.isAuthorized(property);
    }.property('App.router.wizardWatcherController.isWizardRunning')
  });

  // wipe out contexts so boundIf uses `this` (the permission) as the context
  options.contexts = null;
  return Ember.Handlebars.helpers.boundIf.call(permission, "isAuthorized", options);
});

/**
 * Usage:
 *
 * <pre>
 *   {{#havePermissions "SERVICE.TOGGLE_ALERTS"}}
 *     {{! some truly code }}
 *   {{else}}
 *     {{! some falsy code }}
 *   {{/havePermissions}}
 * </pre>
 */
Em.Handlebars.registerHelper('havePermissions', function (property, options) {
  var permission = Ember.Object.create({
    havePermissions: function () {
      return App.havePermissions(property);
    }.property()
  });

  // wipe out contexts so boundIf uses `this` (the permission) as the context
  options.contexts = null;
  return Ember.Handlebars.helpers.boundIf.call(permission, "havePermissions", options);
});

/**
 * @namespace App
 */
App = require('app');

/**
 * Certain variables can have JSON in string
 * format, or in JSON format itself.
 *
 * @memberof App
 * @function parseJson
 * @param {string|object}
 * @return {object}
 */
App.parseJSON = function (value) {
  if (typeof value == "string") {
    return jQuery.parseJSON(value);
  }
  return value;
};
/**
 * Check for empty <code>Object</code>, built in Em.isEmpty()
 * doesn't support <code>Object</code> type
 *
 * @memberof App
 * @method isEmptyObject
 * @param obj {Object}
 * @return {Boolean}
 */
App.isEmptyObject = function (obj) {
  var empty = true;
  for (var prop in obj) {
    if (obj.hasOwnProperty(prop)) {
      empty = false;break;
    }
  }
  return empty;
};

/**
 * Convert object under_score keys to camelCase
 *
 * @param {Object} object
 * @return {Object}
 **/
App.keysUnderscoreToCamelCase = function (object) {
  var tmp = {};
  for (var key in object) {
    tmp[stringUtils.underScoreToCamelCase(key)] = object[key];
  }
  return tmp;
};

/**
 * Convert dotted keys to camelcase
 *
 * @param {Object} object
 * @return {Object}
 **/
App.keysDottedToCamelCase = function (object) {
  var tmp = {};
  for (var key in object) {
    tmp[key.split('.').reduce(function (p, c) {
      return p + c.capitalize();
    })] = object[key];
  }
  return tmp;
};
/**
 * Returns object with defined keys only.
 *
 * @memberof App
 * @method permit
 * @param {Object} obj - input object
 * @param {String|Array} keys - allowed keys
 * @return {Object}
 */
App.permit = function (obj, keys) {
  var result = {};
  if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) !== 'object' || App.isEmptyObject(obj)) return result;
  if (typeof keys == 'string') keys = Array(keys);
  keys.forEach(function (key) {
    if (obj.hasOwnProperty(key)) result[key] = obj[key];
  });
  return result;
};
/**
 *
 * @namespace App
 * @namespace App.format
 */
App.format = {
  /**
   * @memberof App.format
   * @type {object}
   * @property components
   */
  components: {
    'API': 'API',
    'DECOMMISSION_DATANODE': 'Update Exclude File',
    'DRPC': 'DRPC',
    'FLUME_HANDLER': 'Flume',
    'GLUSTERFS': 'GLUSTERFS',
    'HBASE': 'HBase',
    'HBASE_REGIONSERVER': 'RegionServer',
    'HCAT': 'HCat Client',
    'HDFS': 'HDFS',
    'HISTORYSERVER': 'History Server',
    'HIVE_SERVER': 'HiveServer2',
    'JCE': 'JCE',
    'MAPREDUCE2': 'MapReduce2',
    'MYSQL': 'MySQL',
    'REST': 'REST',
    'SECONDARY_NAMENODE': 'SNameNode',
    'STORM_REST_API': 'Storm REST API Server',
    'WEBHCAT': 'WebHCat',
    'YARN': 'YARN',
    'UI': 'UI',
    'ZKFC': 'ZKFailoverController',
    'ZOOKEEPER': 'ZooKeeper',
    'ZOOKEEPER_QUORUM_SERVICE_CHECK': 'ZK Quorum Service Check',
    'HAWQ': 'HAWQ',
    'PXF': 'PXF'
  },

  /**
   * @memberof App.format
   * @property command
   * @type {object}
   */
  command: {
    'INSTALL': 'Install',
    'UNINSTALL': 'Uninstall',
    'START': 'Start',
    'STOP': 'Stop',
    'EXECUTE': 'Execute',
    'ABORT': 'Abort',
    'UPGRADE': 'Upgrade',
    'RESTART': 'Restart',
    'SERVICE_CHECK': 'Check',
    'SET_KEYTAB': 'Set Keytab:',
    'Excluded:': 'Decommission:',
    'Included:': 'Recommission:'
  },

  /**
   * cached map of service names
   * @type {object}
   */
  stackServiceRolesMap: {},

  /**
   * cached map of component names
   * @type {object}
   */
  stackComponentRolesMap: {},

  /**
   * convert role to readable string
   *
   * @memberof App.format
   * @method role
   * @param {string} role
   * @param {boolean} isServiceRole
   * return {string}
   */
  role: function role(_role, isServiceRole) {

    if (isServiceRole) {
      var model = App.StackService;
      var map = this.stackServiceRolesMap;
    } else {
      var model = App.StackServiceComponent;
      var map = this.stackComponentRolesMap;
    }

    this.initializeStackRolesMap(map, model);

    if (map[_role]) {
      return map[_role];
    }
    return this.normalizeName(_role);
  },

  initializeStackRolesMap: function initializeStackRolesMap(map, model) {
    if (App.isEmptyObject(map)) {
      model.find().forEach(function (item) {
        map[item.get('id')] = item.get('displayName');
      });
    }
  },

  /**
   * Try to format non predefined names to readable format.
   *
   * @method normalizeNameBySeparator
   * @param name {String} - name to format
   * @param separators {String} - token use to split the string
   * @return {String}
   */
  normalizeNameBySeparators: function normalizeNameBySeparators(name, separators) {
    if (!name || typeof name != 'string') return '';
    name = name.toLowerCase();
    if (!separators || separators.length == 0) {
      console.debug("No separators specified. Use default separator '_' instead");
      separators = ["_"];
    }

    for (var i = 0; i < separators.length; i++) {
      var separator = separators[i];
      if (new RegExp(separator, 'g').test(name)) {
        name = name.split(separator).map(function (singleName) {
          return this.normalizeName(singleName.toUpperCase());
        }, this).join(' ');
        break;
      }
    }
    return name.capitalize();
  },

  /**
   * Try to format non predefined names to readable format.
   *
   * @method normalizeName
   * @param name {String} - name to format
   * @return {String}
   */
  normalizeName: function normalizeName(name) {
    if (!name || typeof name != 'string') return '';
    if (this.components[name]) return this.components[name];
    name = name.toLowerCase();
    var suffixNoSpaces = ['node', 'tracker', 'manager'];
    var suffixRegExp = new RegExp('(\\w+)(' + suffixNoSpaces.join('|') + ')', 'gi');
    if (/_/g.test(name)) {
      name = name.split('_').map(function (singleName) {
        return this.normalizeName(singleName.toUpperCase());
      }, this).join(' ');
    } else if (suffixRegExp.test(name)) {
      suffixRegExp.lastIndex = 0;
      var matches = suffixRegExp.exec(name);
      name = matches[1].capitalize() + matches[2].capitalize();
    }
    return name.capitalize();
  },

  /**
   * convert command_detail to readable string, show the string for all tasks name
   *
   * @memberof App.format
   * @method commandDetail
   * @param {string} command_detail
   * @param {string} request_inputs
   * @return {string}
   */
  commandDetail: function commandDetail(command_detail, request_inputs, ops_display_name) {
    var detailArr = command_detail.split(' ');
    var self = this;
    var result = '';
    var isIncludeExcludeFiles = false;
    //if an optional operation display name has been specified in the service metainfo.xml
    if (ops_display_name != null && ops_display_name.length > 0) {
      result = result + ' ' + ops_display_name;
    } else {
      detailArr.forEach(function (item) {
        // if the item has the pattern SERVICE/COMPONENT, drop the SERVICE part
        if (item.contains('/') && !isIncludeExcludeFiles) {
          item = item.split('/')[1];
        }
        if (item === 'DECOMMISSION,') {
          // ignore text 'DECOMMISSION,'( command came from 'excluded/included'), here get the component name from request_inputs
          var parsedInputs = jQuery.parseJSON(request_inputs);
          item = parsedInputs ? parsedInputs.slave_type || '' : '';
          isIncludeExcludeFiles = parsedInputs ? parsedInputs.is_add_or_delete_slave_request === 'true' : false;
        }
        if (self.components[item]) {
          result = result + ' ' + self.components[item];
        } else if (self.command[item]) {
          result = result + ' ' + self.command[item];
        } else if (isIncludeExcludeFiles) {
          result = result + ' ' + item;
        } else {
          result = result + ' ' + self.role(item, false);
        }
      });
    }

    if (result.indexOf('Decommission:') > -1 || result.indexOf('Recommission:') > -1) {
      // for Decommission command, make sure the hostname is in lower case
      result = result.split(':')[0] + ': ' + result.split(':')[1].toLowerCase();
    }
    //TODO check if UI use this
    if (result === ' Nagios Update Ignore Actionexecute') {
      result = Em.I18n.t('common.maintenance.task');
    }
    if (result.indexOf('Install Packages Actionexecute') != -1) {
      result = Em.I18n.t('common.installRepo.task');
    }
    if (result === ' Rebalancehdfs NameNode') {
      result = Em.I18n.t('services.service.actions.run.rebalanceHdfsNodes.title');
    }
    if (result === " Startdemoldap Knox Gateway") {
      result = Em.I18n.t('services.service.actions.run.startLdapKnox.title');
    }
    if (result === " Stopdemoldap Knox Gateway") {
      result = Em.I18n.t('services.service.actions.run.stopLdapKnox.title');
    }
    if (result === ' Refreshqueues ResourceManager') {
      result = Em.I18n.t('services.service.actions.run.yarnRefreshQueues.title');
    }
    // HAWQ custom commands on back Ops page.
    if (result === ' Resync Hawq Standby HAWQ Standby Master') {
      result = Em.I18n.t('services.service.actions.run.resyncHawqStandby.label');
    }
    if (result === ' Immediate Stop Hawq Service HAWQ Master') {
      result = Em.I18n.t('services.service.actions.run.immediateStopHawqService.label');
    }
    if (result === ' Immediate Stop Hawq Segment HAWQ Segment') {
      result = Em.I18n.t('services.service.actions.run.immediateStopHawqSegment.label');
    }
    if (result === ' Activate Hawq Standby HAWQ Standby Master') {
      result = Em.I18n.t('admin.activateHawqStandby.button.enable');
    }
    if (result === ' Hawq Clear Cache HAWQ Master') {
      result = Em.I18n.t('services.service.actions.run.clearHawqCache.label');
    }
    if (result === ' Run Hawq Check HAWQ Master') {
      result = Em.I18n.t('services.service.actions.run.runHawqCheck.label');
    }
    //<---End HAWQ custom commands--->
    return result;
  },

  /**
   * Convert uppercase status name to lowercase.
   * <br>
   * <br>PENDING - Not queued yet for a host
   * <br>QUEUED - Queued for a host
   * <br>IN_PROGRESS - Host reported it is working
   * <br>COMPLETED - Host reported success
   * <br>FAILED - Failed
   * <br>TIMEDOUT - Host did not respond in time
   * <br>ABORTED - Operation was abandoned
   *
   * @memberof App.format
   * @method taskStatus
   * @param {string} _taskStatus
   * @return {string}
   *
   */
  taskStatus: function taskStatus(_taskStatus) {
    return _taskStatus.toLowerCase();
  },

  /**
   * simplify kdc error msg
   * @param {string} message
   * @param {boolean} strict if this flag is true ignore not defined msgs return null
   *  else return input msg as is;
   * @returns {*}
   */
  kdcErrorMsg: function kdcErrorMsg(message, strict) {
    /**
     * Error messages for KDC administrator credentials error
     * is used for checking if error message is caused by bad KDC credentials
     * @type {{missingKDC: string, invalidKDC: string}}
     */
    var specialMsg = {
      "missingKDC": "Missing KDC administrator credentials.",
      "invalidKDC": "Invalid KDC administrator credentials.",
      "missingRDCForRealm": "Failed to find a KDC for the specified realm - kadmin"
    };

    for (var m in specialMsg) {
      if (specialMsg.hasOwnProperty(m) && message.contains(specialMsg[m])) return specialMsg[m];
    }
    return strict ? null : message;
  }
};

/**
 * wrapper to bootstrap popover
 * fix issue when popover stuck on view routing
 *
 * @memberof App
 * @method popover
 * @param {DOMElement} self
 * @param {object} options
 */
App.popover = function (self, options) {
  var opts = $.extend(true, {
    container: 'body',
    html: true
  }, options || {});
  if (!self) return;
  self.popover(opts);
  self.on("remove", function () {
    $(this).trigger('mouseleave').off().removeData('bs.popover');
  });
  self = null;
};

/**
 * wrapper to bootstrap tooltip
 * fix issue when tooltip stuck on view routing
 * @memberof App
 * @method tooltip
 * @param {DOMElement} self
 * @param {object} options
 */
App.tooltip = function (self, options) {
  var opts = $.extend(true, {
    container: 'body'
  }, options || {});
  if (!self || !self.tooltip) return;
  self.tooltip(opts);
  /* istanbul ignore next */
  self.on("remove", function () {
    $(this).trigger('mouseleave').off().removeData('bs.tooltip');
  });
  self = null;
};

/**
 * wrapper to Date().getTime()
 * fix issue when client clock and server clock not sync
 *
 * @memberof App
 * @method dateTime
 * @return {Number} timeStamp of current server clock
 */
App.dateTime = function () {
  return new Date().getTime() + App.clockDistance;
};

/**
 *
 * @param {number} [x] timestamp
 * @returns {number}
 */
App.dateTimeWithTimeZone = function (x) {
  var timezone = App.router.get('userSettingsController.userSettings.timezone');
  if (timezone) {
    var tz = Em.getWithDefault(timezone, 'zones.0.value', '');
    return moment(moment.tz(x ? new Date(x) : new Date(), tz).toArray()).toDate().getTime();
  }
  return x || new Date().getTime();
};

App.formatDateTimeWithTimeZone = function (timeStamp, format) {
  var timezone = App.router.get('userSettingsController.userSettings.timezone'),
      time;
  if (timezone) {
    var tz = Em.getWithDefault(timezone, 'zones.0.value', '');
    time = moment.tz(timeStamp, tz);
  } else {
    time = moment(timeStamp);
  }
  return moment(time).format(format);
};

App.getTimeStampFromLocalTime = function (time) {
  var timezone = App.router.get('userSettingsController.userSettings.timezone'),
      offsetString = '',
      date = moment(time).format('YYYY-MM-DD HH:mm:ss');
  if (timezone) {
    var offset = timezone.utcOffset;
    offsetString = moment().utcOffset(offset).format('Z');
  }
  return moment(date + offsetString).toDate().getTime();
};

/**
 * Ambari overrides the default date transformer.
 * This is done because of the non-standard data
 * sent. For example Nagios sends date as "12345678".
 * The problem is that it is a String and is represented
 * only in seconds whereas Javascript's Date needs
 * milliseconds representation.
 */
DS.attr.transforms.date = {
  from: function from(serialized) {
    var type = typeof serialized === 'undefined' ? 'undefined' : _typeof(serialized);
    if (type === Em.I18n.t('common.type.string')) {
      serialized = parseInt(serialized);
      type = typeof serialized === 'undefined' ? 'undefined' : _typeof(serialized);
    }
    if (type === Em.I18n.t('common.type.number')) {
      if (!serialized) {
        //serialized timestamp = 0;
        return 0;
      }
      // The number could be seconds or milliseconds.
      // If seconds, then the length is 10
      // If milliseconds, the length is 13
      if (serialized.toString().length < 13) {
        serialized = serialized * 1000;
      }
      return new Date(serialized);
    } else if (serialized === null || serialized === undefined) {
      // if the value is not present in the data,
      // return undefined, not null.
      return serialized;
    } else {
      return null;
    }
  },
  to: function to(deserialized) {
    if (deserialized instanceof Date) {
      return deserialized.getTime();
    } else if (deserialized === undefined) {
      return undefined;
    } else {
      return null;
    }
  }
};

DS.attr.transforms.object = {
  from: function from(serialized) {
    return Ember.none(serialized) ? null : Object(serialized);
  },

  to: function to(deserialized) {
    return Ember.none(deserialized) ? null : Object(deserialized);
  }
};

/**
 * Allows EmberData models to have array properties.
 *
 * Declare the property as <code>
 *  operations: DS.attr('array'),
 * </code> and
 * during load provide a JSON array for value.
 *
 * This transform simply assigns the same array in both directions.
 */
DS.attr.transforms.array = {
  from: function from(serialized) {
    return serialized;
  },
  to: function to(deserialized) {
    return deserialized;
  }
};

/**
 *  Utility method to delete all existing records of a DS.Model type from the model's associated map and
 *  store's persistence layer (recordCache)
 * @param type DS.Model Class
 */
App.resetDsStoreTypeMap = function (type) {
  var allRecords = App.get('store.recordCache'); //This fetches all records in the ember-data persistence layer
  var typeMaps = App.get('store.typeMaps');
  var guidForType = Em.guidFor(type);
  var typeMap = typeMaps[guidForType];
  if (typeMap) {
    var idToClientIdMap = typeMap.idToCid;
    for (var id in idToClientIdMap) {
      if (idToClientIdMap.hasOwnProperty(id) && idToClientIdMap[id] && allRecords[idToClientIdMap[id]] !== undefined) {
        delete allRecords[idToClientIdMap[id]]; // deletes the cached copy of the record from the store
      }
    }
    typeMaps[guidForType] = {
      idToCid: {},
      clientIds: [],
      cidToHash: {},
      recordArrays: []
    };
  }
};

/**
 * Clean store from already loaded data.
 * @param {DS.Model[]} models to clear
 **/
App.clearModels = function (models) {
  models.forEach(function (model) {
    var records = App.get('store').findAll(model).filterProperty('id');
    records.forEach(function (rec) {
      Ember.run(this, function () {
        rec.deleteRecord();
        App.store.fastCommit();
      });
    }, this);
  }, this);
};

App.logger = function () {

  var timers = {};

  return {

    maxAllowedLoadingTime: 1000,

    setTimer: function setTimer(name) {
      if (!App.get('enableLogger')) return;
      timers[name] = window.performance.now();
    },

    logTimerIfMoreThan: function logTimerIfMoreThan(name, loadingTime) {
      if (!App.get('enableLogger')) return;
      this.maxAllowedLoadingTime = loadingTime || this.maxAllowedLoadingTime;
      if (timers[name]) {
        var diff = window.performance.now() - timers[name];
        if (diff > this.maxAllowedLoadingTime) {
          console.debug(name + ': ' + diff.toFixed(3) + 'ms');
        }
        delete timers[name];
      }
    }
  };
}();

});

require.register("utils/host_progress_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');
var dataUtils = require('utils/data_manipulation');

/**
 * Host information shown in the operations popup
 * @typedef {Em.Object} wrappedHost
 * @property {string} name
 * @property {string} publicName
 * @property {string} displayName
 * @property {number} progress
 * @property {boolean} isInProgress
 * @property {string} serviceName
 * @property {string} status
 * @property {number} isVisible
 * @property {string} icon
 * @property {string} barColor
 * @property {string} barWidth
 */

/**
 * Task information shown in the operations popup
 * @typedef {Em.Object} wrappedTask
 * @property {string} id
 * @property {string} hostName
 * @property {string} command
 * @property {string} commandDetail
 * @property {string} status
 * @property {string} role
 * @property {string} stderr
 * @property {string} stdout
 * @property {number} request_id
 * @property {boolean} isVisible
 * @property {string} startTime
 * @property {string} duration
 * @property {string} icon
 */

/**
 * Service information shown in the operations popup
 * @typedef {Em.Object} wrappedService
 * @property {string} id
 * @property {string} displayName
 * @property {string} progress
 * @property {string} status
 * @property {string} userName
 * @property {boolean} isRunning
 * @property {string} name
 * @property {boolean} isVisible
 * @property {string} startTime
 * @property {string} duration
 * @property {string} icon
 * @property {string} barColor
 * @property {boolean} isInProgress
 * @property {string} barWidth
 * @property {number} sourceRequestScheduleId
 * @property {string} contextCommand
 */

/**
 * App.HostPopup is for the popup that shows up upon clicking already-performed or currently-in-progress operations
 * Allows to abort executing operations
 *
 * @type {Em.Object}
 * @class {HostPopup}
 */
App.HostPopup = Em.Object.create({

  name: 'hostPopup',

  /**
   * @type {object[]}
   */
  servicesInfo: [],

  /**
   * @type {wrappedHost[]}
   */
  hosts: [],

  /**
   * @type {?object[]}
   */
  inputData: null,

  /**
   * @type {string}
   */
  serviceName: '',

  /**
   * @type {?Number}
   */
  currentServiceId: null,

  /**
   * @type {?Number}
   */
  previousServiceId: null,

  /**
   * @type {string}
   */
  popupHeaderName: '',

  //This is what the breadcrumbs will be reset to every time the modal is opened.
  rootBreadcrumb: null,

  /**
   * @type {object[]}
   */
  breadcrumbs: [],

  operationInfo: null,

  /**
   * @type {?App.Controller}
   */
  dataSourceController: null,

  /**
   * @type {bool}
   */
  isBackgroundOperations: false,

  /**
   * @type {?string}
   */
  currentHostName: null,

  /**
   * @type {?App.ModalPopup}
   */
  isPopup: null,

  /**
   * @type {object}
   */
  detailedProperties: {
    stdout: 'stdout',
    stderr: 'stderr',
    outputLog: 'output_log',
    errorLog: 'error_log'
  },

  /**
   * @type {object}
   */
  barColorMap: {
    'FAILED': 'progress-bar-danger',
    'ABORTED': 'progress-bar-warning',
    'TIMEDOUT': 'progress-bar-warning',
    'IN_PROGRESS': 'progress-bar-info',
    'COMPLETED': 'progress-bar-success'
  },

  /**
   * map to get css class with styles by service status
   *
   * @type {object}
   */
  statusesStyleMap: {
    'FAILED': ['FAILED', 'icon-exclamation-sign', 'progress-bar-danger', false],
    'ABORTED': ['ABORTED', 'glyphicon glyphicon-minus', 'progress-bar-warning', false],
    'TIMEDOUT': ['TIMEDOUT', 'glyphicon glyphicon-time', 'progress-bar-warning', false],
    'IN_PROGRESS': ['IN_PROGRESS', 'icon-cogs', 'progress-bar-info', true],
    'COMPLETED': ['SUCCESS', 'glyphicon glyphicon-ok', 'progress-bar-success', false]
  },

  /**
   * View with "Abort Request"-button
   *
   * @type {Em.View}
   */
  abortIcon: Em.View.extend({
    tagName: 'a',
    classNames: ['abort-icon'],
    template: Em.Handlebars.compile('<span class="icon icon-remove-circle"></span>'),
    click: function click() {
      this.get('controller').abortRequest(this.get('servicesInfo'));
      return false;
    },
    didInsertElement: function didInsertElement() {
      App.tooltip($(this.get('element')), {
        placement: "top",
        title: Em.I18n.t('hostPopup.bgop.abortRequest.title')
      });
    },
    willDestroyElement: function willDestroyElement() {
      $(this.get('element')).tooltip('destroy');
    }
  }),

  /**
   * View with status icon (and tooltip on it)
   *
   * @type {Em.View}
   */
  statusIcon: Em.View.extend({
    tagName: 'i',
    classNames: ["service-status"],
    classNameBindings: ['servicesInfo.status', 'servicesInfo.icon', 'additionalClass'],
    attributeBindings: ['data-original-title'],
    'data-original-title': function () {
      return this.get('servicesInfo.status');
    }.property('servicesInfo.status'),
    didInsertElement: function didInsertElement() {
      App.tooltip($(this.get('element')));
    },
    willDestroyElement: function willDestroyElement() {
      $(this.get('element')).tooltip('destroy');
    }
  }),

  /**
   * Determines if background operation can be aborted depending on its status
   *
   * @param status
   * @returns {boolean}
   */
  isAbortableByStatus: function isAbortableByStatus(status) {
    var statuses = this.get('statusesStyleMap');
    return !Em.keys(statuses).contains(status) || status === 'IN_PROGRESS';
  },

  /**
   * Send request to abort operation
   *
   * @method abortRequest
   */
  abortRequest: function abortRequest(serviceInfo) {
    var requestName = serviceInfo.get('name');
    var self = this;
    App.showConfirmationPopup(function () {
      serviceInfo.set('isAbortable', false);
      return App.ajax.send({
        name: 'background_operations.abort_request',
        sender: self,
        data: {
          requestId: serviceInfo.get('id'),
          requestName: requestName,
          serviceInfo: serviceInfo
        },
        success: 'abortRequestSuccessCallback',
        error: 'abortRequestErrorCallback'
      });
    }, Em.I18n.t('hostPopup.bgop.abortRequest.confirmation.body').format(requestName));
    return false;
  },

  /**
   * Method called on successful sending request to abort operation
   *
   * @return {App.ModalPopup}
   * @method abortRequestSuccessCallback
   */
  abortRequestSuccessCallback: function abortRequestSuccessCallback(response, request, data) {
    return App.ModalPopup.show({
      header: Em.I18n.t('hostPopup.bgop.abortRequest.modal.header'),
      encodeBody: false,
      body: Em.I18n.t('hostPopup.bgop.abortRequest.modal.body').format(data.requestName),
      secondary: null
    });
  },

  /**
   * Method called on unsuccessful sending request to abort operation
   *
   * @method abortRequestErrorCallback
   */
  abortRequestErrorCallback: function abortRequestErrorCallback(xhr, textStatus, error, opt, data) {
    data.serviceInfo.set('isAbortable', this.isAbortableByStatus(data.serviceInfo.status));
    App.ajax.defaultErrorHandler(xhr, opt.url, 'PUT', xhr.status);
  },

  /**
   * Entering point of this component
   *
   * @param {String} serviceName
   * @param {Object} controller
   * @param {Boolean} isBackgroundOperations
   * @param {Integer} requestId
   * @method initPopup
   */
  initPopup: function initPopup(serviceName, controller, isBackgroundOperations, requestId) {
    if (App.get('isClusterUser')) return;

    if (!isBackgroundOperations) {
      this.clearHostPopup();
      this.set("rootBreadcrumb", { label: serviceName });
    } else {
      this.set("rootBreadcrumb", { label: Em.I18n.t("common.backgroundOperations") });
    }

    this.setProperties({
      breadcrumbs: [this.get("rootBreadcrumb")],
      currentServiceId: requestId,
      serviceName: serviceName,
      dataSourceController: controller,
      isBackgroundOperations: isBackgroundOperations,
      inputData: controller.get("services")
    });

    if (isBackgroundOperations) {
      this.onServiceUpdate();
    } else {
      this.onHostUpdate();
    }

    return this.createPopup();
  },

  /**
   * clear info popup data
   *
   * @method clearHostPopup
   */
  clearHostPopup: function clearHostPopup() {
    this.setProperties({
      servicesInfo: [],
      host: null,
      inputData: null,
      serviceName: '',
      currentServiceId: null,
      previousServiceId: null,
      popupHeaderName: '',
      dataSourceController: null,
      currentHostName: null
    });
    if (this.get('isPopup')) {
      this.get('isPopup').remove();
    }
  },

  /**
   * Depending on tasks status
   *
   * @param {object[]} tasks
   * @return {*[]} [Status, Icon type, Progressbar color, is IN_PROGRESS]
   * @method getStatus
   */
  getStatus: function getStatus(tasks) {
    var isCompleted = true;
    var tasksLength = tasks.length;
    for (var i = 0; i < tasksLength; i++) {
      var taskStatus = tasks[i].Tasks.status;
      if (taskStatus !== 'COMPLETED') {
        isCompleted = false;
      }
      if (taskStatus === 'FAILED') {
        return ['FAILED', 'icon-exclamation-sign', 'progress-bar-danger', false];
      }
      if (taskStatus === 'ABORTED') {
        return ['ABORTED', 'glyphicon glyphicon-minus', 'progress-bar-warning', false];
      }
      if (taskStatus === 'TIMEDOUT') {
        return ['TIMEDOUT', 'glyphicon glyphicon-time', 'progress-bar-warning', false];
      }
      if (taskStatus === 'IN_PROGRESS') {
        return ['IN_PROGRESS', 'icon-cogs', 'progress-bar-info', true];
      }
    }
    if (isCompleted) {
      return ['SUCCESS', 'glyphicon glyphicon-ok', 'progress-bar-success', false];
    }
    return ['PENDING', 'glyphicon glyphicon-cog', 'progress-bar-info', true];
  },

  /**
   * Progress of host or service depending on tasks status
   * If no tasks, progress is 0
   *
   * @param {?object[]} tasks
   * @return {Number} percent of completion
   * @method getProgress
   */
  getProgress: function getProgress(tasks) {
    if (!tasks || !tasks.length) {
      return 0;
    }

    var groupedByStatus = dataUtils.groupPropertyValues(tasks, 'Tasks.status');

    var completedActions = Em.getWithDefault(groupedByStatus, 'COMPLETED.length', 0) + Em.getWithDefault(groupedByStatus, 'FAILED.length', 0) + Em.getWithDefault(groupedByStatus, 'ABORTED.length', 0) + Em.getWithDefault(groupedByStatus, 'TIMEDOUT.length', 0);
    var queuedActions = Em.getWithDefault(groupedByStatus, 'QUEUED.length', 0);
    var inProgressActions = Em.getWithDefault(groupedByStatus, 'IN_PROGRESS.length', 0);

    return Math.ceil((queuedActions * 0.09 + inProgressActions * 0.35 + completedActions) / tasks.length * 100);
  },

  /**
   * Count number of operations for select box options
   *
   * @param {?Object[]} obj
   * @param {progressPopupCategoryObject[]} categories
   * @method setSelectCount
   */
  setSelectCount: function setSelectCount(obj, categories) {
    if (!obj) {
      return;
    }
    var groupedByStatus = dataUtils.groupPropertyValues(obj, 'status');

    categories.findProperty("value", 'all').set("count", obj.length);
    categories.findProperty("value", 'pending').set("count", Em.getWithDefault(groupedByStatus, 'pending.length', 0) + Em.getWithDefault(groupedByStatus, 'queued.length', 0));
    categories.findProperty("value", 'in_progress').set("count", Em.getWithDefault(groupedByStatus, 'in_progress.length', 0));
    categories.findProperty("value", 'failed').set("count", Em.getWithDefault(groupedByStatus, 'failed.length', 0));
    categories.findProperty("value", 'completed').set("count", Em.getWithDefault(groupedByStatus, 'success.length', 0) + Em.getWithDefault(groupedByStatus, 'completed.length', 0));
    categories.findProperty("value", 'aborted').set("count", Em.getWithDefault(groupedByStatus, 'aborted.length', 0));
    categories.findProperty("value", 'timedout').set("count", Em.getWithDefault(groupedByStatus, 'timedout.length', 0));
  },

  /**
   * For Background operation popup calculate number of running Operations, and set popup header
   *
   * @param {bool} isServiceListHidden
   * @method setBackgroundOperationHeader
   */
  setBackgroundOperationHeader: function setBackgroundOperationHeader(isServiceListHidden) {
    if (this.get('isBackgroundOperations') && !isServiceListHidden) {
      var numRunning = App.router.get('backgroundOperationsController.runningOperationsCount');
      this.set("popupHeaderName", numRunning + Em.I18n.t('hostPopup.header.postFix').format(numRunning === 1 ? "" : "s"));
    }
  },

  /**
   * Create services obj data structure for popup
   * Set data for services
   *
   * @param {bool} isServiceListHidden
   * @method onServiceUpdate
   */
  onServiceUpdate: function onServiceUpdate(isServiceListHidden) {
    var servicesInfo = this.get("servicesInfo");
    var currentServices = [];

    var inputData = this.get("inputData");
    if (inputData) {
      inputData.forEach(function (service, index) {
        var updatedService;
        var id = service.id;
        currentServices.push(id);
        var existedService = servicesInfo.findProperty('id', id);
        updatedService = existedService;
        if (existedService) {
          updatedService = this.updateService(existedService, service);
        } else {
          updatedService = this.createService(service);
          servicesInfo.insertAt(index, updatedService);
        }
        updatedService.set('isAbortable', App.isAuthorized('SERVICE.START_STOP') && this.isAbortableByStatus(service.status));
      }, this);
    }

    this.removeOldServices(servicesInfo, currentServices);
    this.setBackgroundOperationHeader(isServiceListHidden);
  },

  /**
   * Create service object from transmitted data
   *
   * @param {object} service
   * @return {wrappedService}
   * @method createService
   */
  createService: function createService(service) {
    var statuses = this.get('statusesStyleMap');
    var pendingStatus = ['PENDING', 'glyphicon glyphicon-cog', 'progress-bar-info', true];
    var status = statuses[service.status] || pendingStatus;
    return Em.Object.create({
      id: service.id,
      displayName: service.displayName,
      progress: service.progress,
      status: App.format.taskStatus(status[0]),
      userName: service.userName,
      isRunning: service.isRunning,
      name: service.name,
      isVisible: true,
      startTime: date.startTime(service.startTime),
      duration: date.durationSummary(service.startTime, service.endTime),
      icon: status[1],
      barColor: status[2],
      isInProgress: status[3],
      barWidth: "width:" + service.progress + "%;",
      sourceRequestScheduleId: service.sourceRequestScheduleId,
      contextCommand: service.contextCommand
    });
  },

  /**
   * Update properties of existed service with new data
   *
   * @param {wrappedService} service
   * @param {object} newData
   * @returns {wrappedService}
   * @method updateService
   */
  updateService: function updateService(service, newData) {
    var statuses = this.get('statusesStyleMap');
    var pendingStatus = ['PENDING', 'glyphicon glyphicon-cog', 'progress-bar-info', true];
    var status = statuses[newData.status] || pendingStatus;
    return service.setProperties({
      progress: newData.progress,
      status: App.format.taskStatus(status[0]),
      userName: newData.userName,
      isRunning: newData.isRunning,
      startTime: date.startTime(newData.startTime),
      duration: date.durationSummary(newData.startTime, newData.endTime),
      icon: status[1],
      barColor: status[2],
      isInProgress: status[3],
      barWidth: "width:" + newData.progress + "%;",
      sourceRequestScheduleId: newData.get && newData.get('sourceRequestScheduleId'),
      contextCommand: newData.get && newData.get('contextCommand')
    });
  },

  /**
   * remove old requests
   * as API returns 10, or  20 , or 30 ...etc latest request, the requests that absent in response should be removed
   *
   * @param {wrappedService[]} services
   * @param {number[]} currentServicesIds
   * @method removeOldServices
   */
  removeOldServices: function removeOldServices(services, currentServicesIds) {
    for (var i = 0, l = services.length; i < l; i++) {
      if (!currentServicesIds.contains(services[i].id)) {
        services.splice(i, 1);
        i--;
        l--;
      }
    }
  },

  /**
   * Wrap task as Ember-object
   *
   * @param {Object} _task
   * @return {wrappedTask}
   * @method createTask
   */
  createTask: function createTask(_task) {
    return Em.Object.create({
      id: _task.Tasks.id,
      hostName: _task.Tasks.host_name,
      command: _task.Tasks.command.toLowerCase() === 'service_check' ? '' : _task.Tasks.command.toLowerCase(),
      commandDetail: App.format.commandDetail(_task.Tasks.command_detail, _task.Tasks.request_inputs, _task.Tasks.ops_display_name),
      status: App.format.taskStatus(_task.Tasks.status),
      role: App.format.role(_task.Tasks.role, false),
      stderr: _task.Tasks.stderr,
      stdout: _task.Tasks.stdout,
      request_id: _task.Tasks.request_id,
      isVisible: true,
      startTime: date.startTime(_task.Tasks.start_time),
      duration: date.durationSummary(_task.Tasks.start_time, _task.Tasks.end_time),
      icon: function () {
        var statusIconMap = {
          'pending': 'glyphicon glyphicon-cog',
          'queued': 'glyphicon glyphicon-cog',
          'in_progress': 'icon-cogs',
          'completed': 'glyphicon glyphicon-ok',
          'failed': 'icon-exclamation-sign',
          'aborted': 'glyphicon glyphicon-minus',
          'timedout': 'glyphicon glyphicon-time'
        };
        return statusIconMap[this.get('status')] || 'glyphicon glyphicon-cog';
      }.property('status')
    });
  },

  /**
   * Create hosts and tasks data structure for popup
   * Set data for hosts and tasks
   *
   * @method onHostUpdate
   */
  onHostUpdate: function onHostUpdate() {
    var self = this;
    if (this.get("inputData")) {
      var hostsMap = this._getHostsMap();
      var existedHosts = self.get('hosts');

      if (existedHosts && existedHosts.length && this.get('currentServiceId') === this.get('previousServiceId')) {
        this._processingExistingHostsWithSameService(hostsMap);
      } else {
        var hostsArr = this._hostMapProcessing(hostsMap);
        hostsArr = hostsArr.sortProperty('name');
        hostsArr.setEach("serviceName", this.get("serviceName"));
        self.set("hosts", hostsArr);
        self.set('previousServiceId', this.get('currentServiceId'));
      }
    }
    var operation = this.get('servicesInfo').findProperty('id', this.get('currentServiceId'));
    this.set('operationInfo', !operation || operation && operation.get('progress') === 100 ? null : operation);
  },

  /**
   * Generate hosts map for further processing <code>inputData</code>
   *
   * @returns {object}
   * @private
   * @method _getHostsMap
   */
  _getHostsMap: function _getHostsMap() {
    var hostsData;
    var hostsMap = {};
    var inputData = this.get('inputData');

    if (this.get('isBackgroundOperations') && this.get("currentServiceId")) {
      //hosts popup for Background Operations
      hostsData = inputData.findProperty("id", this.get("currentServiceId"));
    } else {
      if (this.get("serviceName")) {
        //hosts popup for Wizards
        hostsData = inputData.findProperty("name", this.get("serviceName"));
      }
    }

    if (hostsData) {
      if (hostsData.hostsMap) {
        //hosts data come from Background Operations as object map
        hostsMap = hostsData.hostsMap;
      } else {
        if (hostsData.hosts) {
          //hosts data come from Wizard as array
          hostsMap = hostsData.hosts.toMapByProperty('name');
        }
      }
    }

    return hostsMap;
  },

  /**
   *
   * @param {object} hostsMap
   * @returns {wrappedHost[]}
   * @private
   * @method _hostMapProcessing
   */
  _hostMapProcessing: function _hostMapProcessing(hostsMap) {
    var self = this;
    var hostsArr = [];
    for (var hostName in hostsMap) {
      if (!hostsMap.hasOwnProperty(hostName)) {
        continue;
      }
      var _host = hostsMap[hostName];
      var tasks = _host.logTasks;
      var hostInfo = Em.Object.create({
        name: hostName,
        publicName: _host.publicName,
        displayName: Em.computed.truncate('name', 43, 40),
        progress: 0,
        status: App.format.taskStatus("PENDING"),
        serviceName: _host.serviceName,
        isVisible: true,
        icon: "glyphicon glyphicon-cog",
        barColor: "progress-bar-info",
        barWidth: "width:0%;"
      });

      if (tasks.length) {
        tasks = tasks.sortProperty('Tasks.id');
        var hostStatus = self.getStatus(tasks);
        var hostProgress = self.getProgress(tasks);
        hostInfo.setProperties({
          status: App.format.taskStatus(hostStatus[0]),
          icon: hostStatus[1],
          barColor: hostStatus[2],
          isInProgress: hostStatus[3],
          progress: hostProgress,
          barWidth: "width:" + hostProgress + "%;"
        });
      }
      hostInfo.set('logTasks', tasks);
      hostsArr.push(hostInfo);
    }
    return hostsArr;
  },

  /**
   *
   * @param {object} hostsMap
   * @private
   * @method _processingExistingHostsWithSameService
   */
  _processingExistingHostsWithSameService: function _processingExistingHostsWithSameService(hostsMap) {
    var self = this;
    var existedHosts = self.get('hosts');
    var detailedProperties = this.get('detailedProperties');
    var detailedPropertiesKeys = Em.keys(detailedProperties);

    existedHosts.forEach(function (host) {
      var newHostInfo = hostsMap[host.get('name')];

      //update only hosts with changed tasks or currently opened tasks of host
      if (newHostInfo && (!this.get('isBackgroundOperations') || newHostInfo.isModified || this.get('currentHostName') === host.get('name'))) {
        var hostStatus = self.getStatus(newHostInfo.logTasks);
        var hostProgress = self.getProgress(newHostInfo.logTasks);

        host.setProperties({
          status: App.format.taskStatus(hostStatus[0]),
          icon: hostStatus[1],
          barColor: hostStatus[2],
          isInProgress: hostStatus[3],
          progress: hostProgress,
          barWidth: "width:" + hostProgress + "%;",
          logTasks: newHostInfo.logTasks
        });

        var existTasks = host.get('tasks');

        if (existTasks) {
          newHostInfo.logTasks.forEach(function (_task) {
            var existTask = existTasks.findProperty('id', _task.Tasks.id);

            if (existTask) {
              var status = _task.Tasks.status;

              detailedPropertiesKeys.forEach(function (key) {
                var name = detailedProperties[key];
                var value = _task.Tasks[name];

                if (!Em.isNone(value)) {
                  existTask.set(key, value);
                }
              }, this);

              existTask.setProperties({
                status: App.format.taskStatus(status),
                startTime: date.startTime(_task.Tasks.start_time),
                duration: date.durationSummary(_task.Tasks.start_time, _task.Tasks.end_time)
              });

              existTask = self._handleRebalanceHDFS(_task, existTask);
            } else {
              existTasks.pushObject(this.createTask(_task));
            }
          }, this);
        }
      }
    }, this);
  },

  /**
   * Custom processing for "Rebalance HDFS"-task
   *
   * @param {object} task
   * @param {object} existTask
   * @returns {object}
   * @private
   * @method _handleRebalanceHDFS
   */
  _handleRebalanceHDFS: function _handleRebalanceHDFS(task, existTask) {
    var barColorMap = this.get('barColorMap');
    var isRebalanceHDFSTask = task.Tasks.command === 'CUSTOM_COMMAND' && task.Tasks.custom_command_name === 'REBALANCEHDFS';
    existTask.set('isRebalanceHDFSTask', isRebalanceHDFSTask);
    if (isRebalanceHDFSTask) {
      var structuredOut = task.Tasks.structured_out || {};
      var status = task.Tasks.status;
      existTask.setProperties({
        dataMoved: structuredOut.dataMoved || '0',
        dataLeft: structuredOut.dataLeft || '0',
        dataBeingMoved: structuredOut.dataBeingMoved || '0',
        barColor: barColorMap[status],
        isInProgress: status === 'IN_PROGRESS',
        isNotComplete: ['QUEUED', 'IN_PROGRESS'].contains(status),
        completionProgressStyle: 'width:' + (structuredOut.completePercent || 0) * 100 + '%;',
        command: task.Tasks.command,
        custom_command_name: task.Tasks.custom_command_name
      });
    }
    return existTask;
  },

  /**
   * Show popup
   *
   * @return {App.ModalPopup} PopupObject For testing purposes
   */
  createPopup: function createPopup() {
    var self = this;
    var isBackgroundOperations = this.get('isBackgroundOperations');

    this.set('isPopup', App.ModalPopup.show({

      /**
       * Controls visiblity of Task Details view.
       * @type {boolean}
       */
      isLogWrapHidden: true,

      /**
       * Controls visiblity of Tasks view.
       * @type {boolean}
       */
      isTaskListHidden: true,

      /**
       * Controls visiblity of Hosts view.
       * @type {boolean}
       */
      isHostListHidden: true,

      /**
       * Controls visiblity of Background Operations view.
       * @type {boolean}
       */
      isServiceListHidden: false,

      /**
       * Single function to handle changing the currently displayed view in the modal.
       * Use this rather than setting the booleans above directly.
       */
      switchView: function switchView(to) {
        switch (to) {
          case "OPS_LIST":
            this.set("isLogWrapHidden", true);
            this.set("isTaskListHidden", true);
            this.set("isHostListHidden", true);
            this.set("isServiceListHidden", false);
            break;
          case "HOSTS_LIST":
            this.set("isLogWrapHidden", true);
            this.set("isTaskListHidden", true);
            this.set("isHostListHidden", false);
            this.set("isServiceListHidden", true);
            break;
          case "TASKS_LIST":
            this.set("isLogWrapHidden", true);
            this.set("isTaskListHidden", false);
            this.set("isHostListHidden", true);
            this.set("isServiceListHidden", true);
            break;
          case "TASK_DETAILS":
            this.set("isLogWrapHidden", false);
            this.set("isTaskListHidden", true);
            this.set("isHostListHidden", true);
            this.set("isServiceListHidden", true);
            break;
        }
      },

      /**
       * @type {boolean}
       */
      isHideBodyScroll: true,

      /**
       * no need to track is it loaded when popup contain only list of hosts
       * @type {bool}
       */
      isLoaded: !isBackgroundOperations,

      /**
       * is BG-popup opened
       * @type {bool}
       */
      isOpen: false,

      /**
       * @type {object}
       */
      detailedProperties: self.get('detailedProperties'),

      isVisible: function () {
        return !(App.get('isClusterUser') && isBackgroundOperations);
      }.property('App.isClusterUser'),

      didInsertElement: function didInsertElement() {
        this._super();
        this.set('isOpen', true);
      },

      /**
       * @type {Em.View}
       */
      headerClass: App.BreadcrumbsView.extend({
        controller: this,
        items: function () {
          var items = this.get('controller.breadcrumbs');
          items = items.map(function (item) {
            return App.BreadcrumbItem.extend(item).create();
          });
          if (items.length) {
            items.get('lastObject').setProperties({
              disabled: true,
              isLast: true
            });
          }
          return items;
        }.property('controller.breadcrumbs')
      }),

      /**
       * @type {Em.View}
       */
      titleClass: Em.View.extend({
        controller: this,
        template: Em.Handlebars.compile('{{popupHeaderName}} ' + '{{#unless view.parentView.isHostListHidden}}{{#if controller.operationInfo.isAbortable}}' + '{{view controller.abortIcon servicesInfoBinding="controller.operationInfo"}}' + '{{/if}}{{/unless}}')
      }),

      /**
       * @type {Em.View}
       */
      footerClass: Em.View.extend({
        classNames: ["modal-footer"],
        templateName: require('templates/common/host_progress_popup_footer')
      }),

      /**
       * @type {String[]}
       */
      classNames: ['common-modal-wrapper', 'host-progress-popup', 'full-height-modal'],
      modalDialogClasses: ['modal-lg'],

      /**
       * Auto-display BG-popup
       *
       * @type {bool}
       */
      isNotShowBgChecked: null,

      /**
       * Save user pref about auto-display BG-popup
       *
       * @method updateNotShowBgChecked
       */
      updateNotShowBgChecked: function () {
        var curVal = !this.get('isNotShowBgChecked');
        if (!App.get('testMode')) {
          App.router.get('userSettingsController').postUserPref('show_bg', curVal);
        }
      }.observes('isNotShowBgChecked'),

      autoHeight: false,

      /**
       * @method closeModelPopup
       */
      closeModelPopup: function closeModelPopup() {
        this.set('isOpen', false);
        if (isBackgroundOperations) {
          $(this.get('element')).detach();
          App.router.get('backgroundOperationsController').set('levelInfo.name', 'OPS_LIST');
        } else {
          this.hide();
          self.set('isPopup', null);
        }
      },

      onPrimary: function onPrimary() {
        this.closeModelPopup();
      },

      onClose: function onClose() {
        this.closeModelPopup();
      },

      secondary: null,

      bodyClass: App.HostProgressPopupBodyView.extend({
        controller: self
      })

    }));

    return this.get('isPopup');
  }
});

});

require.register("utils/hosts", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

require('views/common/table_view');

var App = require('app');
var validator = require('utils/validator');

module.exports = {

  /**
   * Launches a dialog to select hosts from the provided available hosts.
   *
   * Once the user clicks OK or Cancel, the callback is called with the
   * array of hosts (App.Host[]) selected. If the dialog was cancelled
   * or closed, <code>null</code> is provided to the callback. Else
   * an array (maybe empty) will be provided to the callback.
   *
   * @param initialHosts  {App.Host[]} List of hosts to pick from
   * @param selectedHosts {App.Host[]} List of hosts already selected from the available hosts
   * @param selectAtleastOneHost  {boolean} If true atleast one host has to be selected
   * @param validComponents {App.HostComponent[]} List of host-component types to pick from.
   * @param callback  Callback function which is invoked when dialog
   * @param popupDescription {Object} Consist header and message for popup
   *   Example: {header: 'header', dialogMessage: 'message'}
   *  is closed, cancelled or OK is pressed.
   */
  launchHostsSelectionDialog: function launchHostsSelectionDialog(initialHosts, selectedHosts, selectAtleastOneHost, validComponents, callback, popupDescription) {
    // set default popup description
    var defaultPopupDescription = {
      header: Em.I18n.t('hosts.selectHostsDialog.title'),
      dialogMessage: Em.I18n.t('hosts.selectHostsDialog.message')
    };
    if (popupDescription !== null) {
      popupDescription = $.extend(true, defaultPopupDescription, popupDescription);
    }
    App.ModalPopup.show({

      classNames: ['common-modal-wrapper', 'full-height-modal'],
      modalDialogClasses: ['modal-lg'],

      elementId: 'host-selection-dialog',

      header: popupDescription.header,

      dialogMessage: popupDescription.dialogMessage,

      warningMessage: null,

      availableHosts: [],

      onPrimary: function onPrimary() {
        this.set('warningMessage', null);
        var arrayOfSelectedHosts = this.get('availableHosts').filterProperty('selected', true).mapProperty('host.id');
        if (selectAtleastOneHost && arrayOfSelectedHosts.length < 1) {
          this.set('warningMessage', Em.I18n.t('hosts.selectHostsDialog.message.warning'));
          return;
        }
        callback(arrayOfSelectedHosts);
        this.hide();
      },

      disablePrimary: Em.computed.not('isLoaded'),

      onSecondary: function onSecondary() {
        callback(null);
        this.hide();
      },

      bodyClass: App.TableView.extend({

        templateName: require('templates/common/configs/overrideWindow'),

        controllerBinding: 'App.router.mainServiceInfoConfigsController',

        isPaginate: true,

        filteredContent: [],

        filteredContentObs: function () {
          Em.run.once(this, this.filteredContentObsOnce);
        }.observes('parentView.availableHosts.@each.filtered'),

        filteredContentObsOnce: function filteredContentObsOnce() {
          var filteredContent = this.get('parentView.availableHosts').filterProperty('filtered') || [];
          this.set('filteredContent', filteredContent);
        },

        filterText: '',

        filterTextPlaceholder: Em.I18n.t('hosts.selectHostsDialog.filter.placeHolder'),

        filterColumn: null,

        filterColumns: Ember.A([Ember.Object.create({ id: 'ip', name: 'IP Address', selected: true }), Ember.Object.create({ id: 'cpu', name: 'CPU', selected: false }), Ember.Object.create({ id: 'memory', name: 'RAM', selected: false }), Ember.Object.create({ id: 'osArch', name: 'OS Architecture', selected: false }), Ember.Object.create({ id: 'osType', name: 'OS Type', selected: false }), Ember.Object.create({ id: 'diskTotal', name: 'Total Disks Capacity', selected: false }), Ember.Object.create({ id: 'disksMounted', name: '# of Disk Mounts', selected: false })]),

        showOnlySelectedHosts: false,

        filterComponents: validComponents,

        filterComponent: null,

        isDisabled: Em.computed.not('parentView.isLoaded'),

        didInsertElement: function didInsertElement() {
          var defaultFilterColumn = this.get('filterColumns').findProperty('selected');
          this.get('filterComponents').setEach('selected', false);
          this.set('filterColumn', defaultFilterColumn);
          initialHosts.setEach('filtered', true);
          this.set('parentView.availableHosts', initialHosts);
          this.set('parentView.isLoaded', true);
          this.filteredContentObsOnce();
        },

        /**
         * Default filter-method isn't needed
         */
        filter: Em.K,

        filterHosts: function () {
          var filterText = this.get('filterText');
          var showOnlySelectedHosts = this.get('showOnlySelectedHosts');
          var filterComponent = this.get('filterComponent');
          var filterColumn = this.get('filterColumn');

          this.get('parentView.availableHosts').forEach(function (host) {
            var skip = showOnlySelectedHosts && !host.get('selected');
            var value = host.get('host').get(filterColumn.id);
            var hostComponentNames = host.get('hostComponentNames');

            host.set('filterColumnValue', value);

            if (!skip && filterText && (value == null || !value.toString().match(filterText)) && !host.get('host.publicHostName').match(filterText)) {
              skip = true;
            }
            if (!skip && filterComponent && hostComponentNames.length > 0) {
              skip = !hostComponentNames.contains(filterComponent.get('componentName'));
            }
            host.set('filtered', !skip);
          }, this);

          this.set('startIndex', this.get('parentView.availableHosts').someProperty('filtered') ? 1 : 0);
        }.observes('parentView.availableHosts', 'filterColumn', 'filterText', 'filterComponent', 'filterComponent.componentName', 'showOnlySelectedHosts'),

        hostSelectMessage: function () {
          var hosts = this.get('parentView.availableHosts');
          var selectedHosts = hosts.filterProperty('selected', true);
          return this.t('hosts.selectHostsDialog.selectedHostsLink').format(selectedHosts.get('length'), hosts.get('length'));
        }.property('parentView.availableHosts.@each.selected'),

        selectFilterColumn: function selectFilterColumn(event) {
          if (event != null && event.context != null && event.context.id != null) {
            var filterColumn = this.get('filterColumn');
            if (filterColumn != null) {
              filterColumn.set('selected', false);
            }
            event.context.set('selected', true);
            this.set('filterColumn', event.context);
          }
        },

        selectFilterComponent: function selectFilterComponent(event) {
          if (event != null && event.context != null && event.context.componentName != null) {
            var currentFilter = this.get('filterComponent');
            if (currentFilter != null) {
              currentFilter.set('selected', false);
            }
            if (currentFilter != null && currentFilter.componentName === event.context.componentName) {
              // selecting the same filter deselects it.
              this.set('filterComponent', null);
            } else {
              this.set('filterComponent', event.context);
              event.context.set('selected', true);
            }
          }
        },

        allHostsSelected: false,

        toggleSelectAllHosts: function (event) {
          this.get('parentView.availableHosts').filterProperty('filtered').setEach('selected', this.get('allHostsSelected'));
        }.observes('allHostsSelected'),

        toggleShowSelectedHosts: function toggleShowSelectedHosts() {
          var currentFilter = this.get('filterComponent');
          if (currentFilter != null) {
            currentFilter.set('selected', false);
          }
          this.setProperties({
            filterComponent: null,
            filterText: null
          });
          this.toggleProperty('showOnlySelectedHosts');
        }
      })
    });
  },

  /**
  * Bulk setting of for rack id
  * @param {Object} operationData - data about bulk operation (action, hostComponents etc)
  * @param {Ember.Enumerable} hosts - list of affected hosts
  */
  setRackInfo: function setRackInfo(operationData, hosts, rackId) {
    var self = this;
    var hostNames = hosts.mapProperty('hostName');
    return App.ModalPopup.show({
      header: Em.I18n.t('hosts.host.details.setRackId'),
      disablePrimary: true,
      rackId: rackId ? rackId : "",
      bodyClass: Em.View.extend({
        templateName: require('templates/main/host/rack_id_popup'),
        errorMessage: null,
        isValid: true,
        validation: function () {
          this.set('isValid', validator.isValidRackId(this.get('parentView.rackId')));
          this.set('errorMessage', this.get('isValid') ? '' : Em.I18n.t('hostPopup.setRackId.invalid'));
          this.set('parentView.disablePrimary', !this.get('isValid'));
        }.observes('parentView.rackId')
      }),
      onPrimary: function onPrimary() {
        var rackId = this.get('rackId');
        if (hostNames.length) {
          App.ajax.send({
            name: 'bulk_request.hosts.update_rack_id',
            sender: self,
            data: {
              hostNames: hostNames.join(','),
              requestInfo: operationData.message,
              rackId: rackId,
              hostNamesArray: hostNames
            },
            success: 'successRackId',
            error: 'errorRackId'
          });
        }
        this.hide();
      }
    });
  },

  /**
   * Success callback for set rack id request
   */
  successRackId: function successRackId(response, request, params) {
    App.Host.find().forEach(function (host) {
      if (params.hostNamesArray.contains(host.get('hostName'))) {
        host.set('rack', params.rackId);
      }
    });
  },

  /**
   * Warn user that the rack id will not be updated
   */
  errorRackId: function errorRackId() {
    App.showAlertPopup(Em.I18n.t('common.error'), Em.I18n.t('hostPopup.setRackId.error'));
  }
};

});

require.register("utils/http_client", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * App.HttpClient perform an ajax request
 */
App.HttpClient = Em.Object.create({

  /**
   *
   * @param jqXHR
   * @param textStatus
   * @param errorThrown
   * @param url api that invoked this callback function
   */
  defaultErrorHandler: function defaultErrorHandler(jqXHR, textStatus, errorThrown, url) {
    try {
      var json = $.parseJSON(jqXHR.responseText);
    } catch (err) {}
    App.ajax.defaultErrorHandler(jqXHR, url);
    if (json) {
      Em.assert("HttpClient:", json);
    } else {
      if (!$.mocho) {
        // don't use this assert on tests
        Em.assert("HttpClient:", errorThrown);
      }
    }
  },

  /**
   * @param {string} url
   * @param {Object} ajaxOptions
   * @param {App.ServerDataMapper} mapper - json processor
   * @param {boolean} isGetAsPost - if true, do POST-request equal to GET-request but with some params put to body
   * @param {callback} errorHandler
   */
  request: function request(url, ajaxOptions, mapper, errorHandler, isGetAsPost) {

    if (!errorHandler) {
      errorHandler = this.defaultErrorHandler;
    }

    var xhr = new XMLHttpRequest(),
        curTime = App.dateTime(),
        method = isGetAsPost ? 'POST' : 'GET',
        params = isGetAsPost ? JSON.stringify({
      "RequestInfo": { "query": ajaxOptions.params }
    }) : null;

    xhr.open(method, url + (url.indexOf('?') >= 0 ? '&_=' : '?_=') + curTime, true);
    if (isGetAsPost) {
      xhr.setRequestHeader("X-Http-Method-Override", "GET");
      xhr.setRequestHeader("Content-type", "text/plain");
    }
    xhr.send(params);

    this.onReady(xhr, "", ajaxOptions, mapper, errorHandler, url);
  },

  /**
   This function checks if we get response from server
   Not using onreadystatechange cuz of possible closure
   */
  onReady: function onReady(xhr, tm, tmp_val, mapper, errorHandler, url) {
    var self = this;
    clearTimeout(tm);
    var timeout = setTimeout(function () {
      if (xhr.readyState == 4) {
        if (xhr.status == 200) {
          var response = $.parseJSON(xhr.responseText);
          if (tmp_val.beforeMap) {
            tmp_val.beforeMap.call(self, response);
          }
          mapper.map(response);
          tmp_val.complete.call(self);
          xhr.abort();
        } else {
          errorHandler(xhr, "error", xhr.statusText, url);
        }

        tmp_val = null;
        xhr = null;
        clearTimeout(timeout);
        timeout = null;
      } else {
        self.onReady(xhr, timeout, tmp_val, mapper, errorHandler, url);
      }
    }, 10);
  },

  /**
   * @param {string} url
   * @param {App.ServerDataMapper} mapper - json processor
   * @param {Object} data - ajax data property
   * @param {callback} errorHandler
   * @param {number} interval - frequency request
   */
  get: function get(url, mapper, data, errorHandler, interval) {
    if (!errorHandler && data.error) {
      errorHandler = data.error;
    }
    var client = this,
        request = function request() {
      var isGetAsPost = Boolean(data.doGetAsPost && !App.get('testMode'));
      client.request(url, data, mapper, errorHandler, isGetAsPost);
      url = null;
      data = null;
      mapper = null;
      errorHandler = null;
    };

    interval = "" + interval;
    if (interval.match(/\d+/)) {
      $.periodic({ period: interval }, request);
    } else {
      request();
    }
  },

  /**
   * @param {string} url
   * @param {Object} data - ajax data property
   * @param {App.ServerDataMapper} mapper - json processor
   * @param {callback} errorHandler
   * @param {number} interval - frequency request
   */
  post: function post(url, data, mapper, errorHandler, interval) {
    this.get.apply(this, arguments);
  }
});

});

require.register("utils/lazy_loading", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {
  /**
   * Divide source array in chunks, then push each chunk into destination array
   * with delay time one by one. So then destination array gets more and more items
   * till all items are loaded.
   * @param options
   * options.initSize - number of items which will be pushed in array immediately
   * options.chunkSize - number of items which will be pushed after defined delay
   * options.delay - interval between each chunk push
   * options.destination - array where items will be pushed
   * options.source - source of items
   * options.context - the object that should know when data is completely loaded,
   * lazy loading will define "isLoaded" property for context object and update it
   * @return {Object} - instance of lazy loading run
   */
  run: function run(options) {
    var initSize = options.initSize || 25,
        chunkSize = options.chunkSize || 50,
        delay = options.delay || 300,
        destination = options.destination,
        source = options.source,
        self = this,
        instance = {
      context: options.context,
      timeoutRef: null
    },
        chunks;
    if (Array.isArray(destination) && Array.isArray(source)) {
      destination.pushObjects(source.slice(0, initSize));
      if (source.length > initSize) {
        chunks = this.divideIntoChunks(source.slice(initSize, source.length), chunkSize);
        this.pushChunk(chunks, 0, delay, destination, instance);
      } else {
        instance.context.set('isLoaded', true);
      }
    }
    return instance;
  },

  /**
   * push chunks into destination array in delay time
   * @param chunks
   * @param index
   * @param delay
   * @param destination
   * @param instance
   */
  pushChunk: function pushChunk(chunks, index, delay, destination, instance) {
    var self = this;
    instance.timeoutRef = setTimeout(function () {
      destination.pushObjects(chunks[index]);
      if (chunks.length === ++index) {
        instance.context.set('isLoaded', true);
      } else {
        self.pushChunk(chunks, index, delay, destination, instance);
      }
    }, delay);
  },

  /**
   * divide source array into chunks
   * @param source
   * @param chunkSize
   * @return {Array}
   */
  divideIntoChunks: function divideIntoChunks(source, chunkSize) {
    var chunk = [];
    var chunks = [];
    var counter = 0;
    source.forEach(function (item) {
      counter++;
      chunk.push(item);
      if (counter === chunkSize) {
        chunks.push(chunk);
        chunk = [];
        counter = 0;
      }
    });
    if (chunk.length > 0) {
      chunks.push(chunk);
    }
    return chunks;
  },
  /**
   * terminate lazy loading run
   * @param instance - lazy loading instance
   */
  terminate: function terminate(instance) {
    clearTimeout(instance.timeoutRef);
  }
};

});

require.register("utils/load_timer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.loadTimer = Em.Object.create({

  timeStampCache: {},

  /**
   * save start timestamp
   * @param {string} label
   */
  start: function start(label) {
    $('.alert.attach-to-bottom-right').remove();
    if (App.get('supports.showPageLoadTime')) {
      this.get('timeStampCache')[label] = window.performance.now();
    }
  },

  /**
   * calculate time difference
   * @param {string} label
   * @returns {undefined|string}
   */
  finish: function finish(label) {
    var result;

    if (typeof this.get('timeStampCache')[label] === "number") {
      result = Number(window.performance.now() - this.get('timeStampCache')[label]).toFixed(2);
      console.debug(label + " loaded in: " + result + "ms");
      this.display(label + " loaded in: " + result + "ms");
      delete this.get('timeStampCache')[label];
    }
    return result;
  },

  /**
   * display time results on the screen
   * @param {string} result
   */
  display: function display(result) {
    var alert = $("<div class='alert alert-warning attach-to-bottom-right'>" + result + "</div>");
    var closeButton = $("<i class='glyphicon glyphicon-remove-circle'></i>").click(function () {
      $(this).remove();
      $(alert).remove();
    });
    alert.append("&nbsp;", closeButton);
    $('body').append(alert);
  }
});

});

require.register("utils/lz-string", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var LZString = function () {

  // private property
  var f = String.fromCharCode;
  var keyStrBase64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
  var keyStrUriSafe = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+-$";
  var baseReverseDic = {};

  function getBaseValue(alphabet, character) {
    if (!baseReverseDic[alphabet]) {
      baseReverseDic[alphabet] = {};
      for (var i = 0; i < alphabet.length; i++) {
        baseReverseDic[alphabet][alphabet.charAt(i)] = i;
      }
    }
    return baseReverseDic[alphabet][character];
  }

  var LZString = {
    compressToBase64: function compressToBase64(input) {
      if (input == null) return "";
      var res = LZString._compress(input, 6, function (a) {
        return keyStrBase64.charAt(a);
      });
      switch (res.length % 4) {// To produce valid Base64
        default: // When could this happen ?
        case 0:
          return res;
        case 1:
          return res + "===";
        case 2:
          return res + "==";
        case 3:
          return res + "=";
      }
    },

    decompressFromBase64: function decompressFromBase64(input) {
      if (input == null) return "";
      if (input == "") return null;
      return LZString._decompress(input.length, 32, function (index) {
        return getBaseValue(keyStrBase64, input.charAt(index));
      });
    },

    compressToUTF16: function compressToUTF16(input) {
      if (input == null) return "";
      return LZString._compress(input, 15, function (a) {
        return f(a + 32);
      }) + " ";
    },

    decompressFromUTF16: function decompressFromUTF16(compressed) {
      if (compressed == null) return "";
      if (compressed == "") return null;
      return LZString._decompress(compressed.length, 16384, function (index) {
        return compressed.charCodeAt(index) - 32;
      });
    },

    //compress into uint8array (UCS-2 big endian format)
    compressToUint8Array: function compressToUint8Array(uncompressed) {
      var compressed = LZString.compress(uncompressed);
      var buf = new Uint8Array(compressed.length * 2); // 2 bytes per character

      for (var i = 0, TotalLen = compressed.length; i < TotalLen; i++) {
        var current_value = compressed.charCodeAt(i);
        buf[i * 2] = current_value >>> 8;
        buf[i * 2 + 1] = current_value % 256;
      }
      return buf;
    },

    //decompress from uint8array (UCS-2 big endian format)
    decompressFromUint8Array: function decompressFromUint8Array(compressed) {
      if (compressed === null || compressed === undefined) {
        return LZString.decompress(compressed);
      } else {
        var buf = new Array(compressed.length / 2); // 2 bytes per character
        for (var i = 0, TotalLen = buf.length; i < TotalLen; i++) {
          buf[i] = compressed[i * 2] * 256 + compressed[i * 2 + 1];
        }

        var result = [];
        buf.forEach(function (c) {
          result.push(f(c));
        });
        return LZString.decompress(result.join(''));
      }
    },

    //compress into a string that is already URI encoded
    compressToEncodedURIComponent: function compressToEncodedURIComponent(input) {
      if (input == null) return "";
      return LZString._compress(input, 6, function (a) {
        return keyStrUriSafe.charAt(a);
      });
    },

    //decompress from an output of compressToEncodedURIComponent
    decompressFromEncodedURIComponent: function decompressFromEncodedURIComponent(input) {
      if (input == null) return "";
      if (input == "") return null;
      input = input.replace(/ /g, "+");
      return LZString._decompress(input.length, 32, function (index) {
        return getBaseValue(keyStrUriSafe, input.charAt(index));
      });
    },

    compress: function compress(uncompressed) {
      return LZString._compress(uncompressed, 16, function (a) {
        return f(a);
      });
    },
    _compress: function _compress(uncompressed, bitsPerChar, getCharFromInt) {
      if (uncompressed == null) return "";
      var i,
          value,
          context_dictionary = {},
          context_dictionaryToCreate = {},
          context_c = "",
          context_wc = "",
          context_w = "",
          context_enlargeIn = 2,
          // Compensate for the first entry which should not count
      context_dictSize = 3,
          context_numBits = 2,
          context_data = [],
          context_data_val = 0,
          context_data_position = 0,
          ii;

      for (ii = 0; ii < uncompressed.length; ii += 1) {
        context_c = uncompressed.charAt(ii);
        if (!Object.prototype.hasOwnProperty.call(context_dictionary, context_c)) {
          context_dictionary[context_c] = context_dictSize++;
          context_dictionaryToCreate[context_c] = true;
        }

        context_wc = context_w + context_c;
        if (Object.prototype.hasOwnProperty.call(context_dictionary, context_wc)) {
          context_w = context_wc;
        } else {
          if (Object.prototype.hasOwnProperty.call(context_dictionaryToCreate, context_w)) {
            if (context_w.charCodeAt(0) < 256) {
              for (i = 0; i < context_numBits; i++) {
                context_data_val = context_data_val << 1;
                if (context_data_position == bitsPerChar - 1) {
                  context_data_position = 0;
                  context_data.push(getCharFromInt(context_data_val));
                  context_data_val = 0;
                } else {
                  context_data_position++;
                }
              }
              value = context_w.charCodeAt(0);
              for (i = 0; i < 8; i++) {
                context_data_val = context_data_val << 1 | value & 1;
                if (context_data_position == bitsPerChar - 1) {
                  context_data_position = 0;
                  context_data.push(getCharFromInt(context_data_val));
                  context_data_val = 0;
                } else {
                  context_data_position++;
                }
                value = value >> 1;
              }
            } else {
              value = 1;
              for (i = 0; i < context_numBits; i++) {
                context_data_val = context_data_val << 1 | value;
                if (context_data_position == bitsPerChar - 1) {
                  context_data_position = 0;
                  context_data.push(getCharFromInt(context_data_val));
                  context_data_val = 0;
                } else {
                  context_data_position++;
                }
                value = 0;
              }
              value = context_w.charCodeAt(0);
              for (i = 0; i < 16; i++) {
                context_data_val = context_data_val << 1 | value & 1;
                if (context_data_position == bitsPerChar - 1) {
                  context_data_position = 0;
                  context_data.push(getCharFromInt(context_data_val));
                  context_data_val = 0;
                } else {
                  context_data_position++;
                }
                value = value >> 1;
              }
            }
            context_enlargeIn--;
            if (context_enlargeIn == 0) {
              context_enlargeIn = Math.pow(2, context_numBits);
              context_numBits++;
            }
            delete context_dictionaryToCreate[context_w];
          } else {
            value = context_dictionary[context_w];
            for (i = 0; i < context_numBits; i++) {
              context_data_val = context_data_val << 1 | value & 1;
              if (context_data_position == bitsPerChar - 1) {
                context_data_position = 0;
                context_data.push(getCharFromInt(context_data_val));
                context_data_val = 0;
              } else {
                context_data_position++;
              }
              value = value >> 1;
            }
          }
          context_enlargeIn--;
          if (context_enlargeIn == 0) {
            context_enlargeIn = Math.pow(2, context_numBits);
            context_numBits++;
          }
          // Add wc to the dictionary.
          context_dictionary[context_wc] = context_dictSize++;
          context_w = String(context_c);
        }
      }

      // Output the code for w.
      if (context_w !== "") {
        if (Object.prototype.hasOwnProperty.call(context_dictionaryToCreate, context_w)) {
          if (context_w.charCodeAt(0) < 256) {
            for (i = 0; i < context_numBits; i++) {
              context_data_val = context_data_val << 1;
              if (context_data_position == bitsPerChar - 1) {
                context_data_position = 0;
                context_data.push(getCharFromInt(context_data_val));
                context_data_val = 0;
              } else {
                context_data_position++;
              }
            }
            value = context_w.charCodeAt(0);
            for (i = 0; i < 8; i++) {
              context_data_val = context_data_val << 1 | value & 1;
              if (context_data_position == bitsPerChar - 1) {
                context_data_position = 0;
                context_data.push(getCharFromInt(context_data_val));
                context_data_val = 0;
              } else {
                context_data_position++;
              }
              value = value >> 1;
            }
          } else {
            value = 1;
            for (i = 0; i < context_numBits; i++) {
              context_data_val = context_data_val << 1 | value;
              if (context_data_position == bitsPerChar - 1) {
                context_data_position = 0;
                context_data.push(getCharFromInt(context_data_val));
                context_data_val = 0;
              } else {
                context_data_position++;
              }
              value = 0;
            }
            value = context_w.charCodeAt(0);
            for (i = 0; i < 16; i++) {
              context_data_val = context_data_val << 1 | value & 1;
              if (context_data_position == bitsPerChar - 1) {
                context_data_position = 0;
                context_data.push(getCharFromInt(context_data_val));
                context_data_val = 0;
              } else {
                context_data_position++;
              }
              value = value >> 1;
            }
          }
          context_enlargeIn--;
          if (context_enlargeIn == 0) {
            context_enlargeIn = Math.pow(2, context_numBits);
            context_numBits++;
          }
          delete context_dictionaryToCreate[context_w];
        } else {
          value = context_dictionary[context_w];
          for (i = 0; i < context_numBits; i++) {
            context_data_val = context_data_val << 1 | value & 1;
            if (context_data_position == bitsPerChar - 1) {
              context_data_position = 0;
              context_data.push(getCharFromInt(context_data_val));
              context_data_val = 0;
            } else {
              context_data_position++;
            }
            value = value >> 1;
          }
        }
        context_enlargeIn--;
        if (context_enlargeIn == 0) {
          context_enlargeIn = Math.pow(2, context_numBits);
          context_numBits++;
        }
      }

      // Mark the end of the stream
      value = 2;
      for (i = 0; i < context_numBits; i++) {
        context_data_val = context_data_val << 1 | value & 1;
        if (context_data_position == bitsPerChar - 1) {
          context_data_position = 0;
          context_data.push(getCharFromInt(context_data_val));
          context_data_val = 0;
        } else {
          context_data_position++;
        }
        value = value >> 1;
      }

      // Flush the last char
      while (true) {
        context_data_val = context_data_val << 1;
        if (context_data_position == bitsPerChar - 1) {
          context_data.push(getCharFromInt(context_data_val));
          break;
        } else context_data_position++;
      }
      return context_data.join('');
    },

    decompress: function decompress(compressed) {
      if (compressed == null) return "";
      if (compressed == "") return null;
      return LZString._decompress(compressed.length, 32768, function (index) {
        return compressed.charCodeAt(index);
      });
    },

    _decompress: function _decompress(length, resetValue, getNextValue) {
      var dictionary = [],
          next,
          enlargeIn = 4,
          dictSize = 4,
          numBits = 3,
          entry = "",
          result = [],
          i,
          w,
          bits,
          resb,
          maxpower,
          power,
          c,
          data = { val: getNextValue(0), position: resetValue, index: 1 };

      for (i = 0; i < 3; i += 1) {
        dictionary[i] = i;
      }

      bits = 0;
      maxpower = Math.pow(2, 2);
      power = 1;
      while (power != maxpower) {
        resb = data.val & data.position;
        data.position >>= 1;
        if (data.position == 0) {
          data.position = resetValue;
          data.val = getNextValue(data.index++);
        }
        bits |= (resb > 0 ? 1 : 0) * power;
        power <<= 1;
      }

      switch (next = bits) {
        case 0:
          bits = 0;
          maxpower = Math.pow(2, 8);
          power = 1;
          while (power != maxpower) {
            resb = data.val & data.position;
            data.position >>= 1;
            if (data.position == 0) {
              data.position = resetValue;
              data.val = getNextValue(data.index++);
            }
            bits |= (resb > 0 ? 1 : 0) * power;
            power <<= 1;
          }
          c = f(bits);
          break;
        case 1:
          bits = 0;
          maxpower = Math.pow(2, 16);
          power = 1;
          while (power != maxpower) {
            resb = data.val & data.position;
            data.position >>= 1;
            if (data.position == 0) {
              data.position = resetValue;
              data.val = getNextValue(data.index++);
            }
            bits |= (resb > 0 ? 1 : 0) * power;
            power <<= 1;
          }
          c = f(bits);
          break;
        case 2:
          return "";
      }
      dictionary[3] = c;
      w = c;
      result.push(c);
      while (true) {
        if (data.index > length) {
          return "";
        }

        bits = 0;
        maxpower = Math.pow(2, numBits);
        power = 1;
        while (power != maxpower) {
          resb = data.val & data.position;
          data.position >>= 1;
          if (data.position == 0) {
            data.position = resetValue;
            data.val = getNextValue(data.index++);
          }
          bits |= (resb > 0 ? 1 : 0) * power;
          power <<= 1;
        }

        switch (c = bits) {
          case 0:
            bits = 0;
            maxpower = Math.pow(2, 8);
            power = 1;
            while (power != maxpower) {
              resb = data.val & data.position;
              data.position >>= 1;
              if (data.position == 0) {
                data.position = resetValue;
                data.val = getNextValue(data.index++);
              }
              bits |= (resb > 0 ? 1 : 0) * power;
              power <<= 1;
            }

            dictionary[dictSize++] = f(bits);
            c = dictSize - 1;
            enlargeIn--;
            break;
          case 1:
            bits = 0;
            maxpower = Math.pow(2, 16);
            power = 1;
            while (power != maxpower) {
              resb = data.val & data.position;
              data.position >>= 1;
              if (data.position == 0) {
                data.position = resetValue;
                data.val = getNextValue(data.index++);
              }
              bits |= (resb > 0 ? 1 : 0) * power;
              power <<= 1;
            }
            dictionary[dictSize++] = f(bits);
            c = dictSize - 1;
            enlargeIn--;
            break;
          case 2:
            return result.join('');
        }

        if (enlargeIn == 0) {
          enlargeIn = Math.pow(2, numBits);
          numBits++;
        }

        if (dictionary[c]) {
          entry = dictionary[c];
        } else {
          if (c === dictSize) {
            entry = w + w.charAt(0);
          } else {
            return null;
          }
        }
        result.push(entry);

        // Add w+entry[0] to the dictionary.
        dictionary[dictSize++] = w + entry.charAt(0);
        enlargeIn--;

        w = entry;

        if (enlargeIn == 0) {
          enlargeIn = Math.pow(2, numBits);
          numBits++;
        }
      }
    }
  };
  return LZString;
}();

if (typeof define === 'function' && define.amd) {
  define(function () {
    return LZString;
  });
} else if (typeof module !== 'undefined' && module != null) {
  module.exports = LZString;
}

});

;require.register("utils/misc", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
module.exports = {
  /**
   * Convert value from bytes to appropriate measure
   */
  formatBandwidth: function formatBandwidth(value) {
    if (value) {
      if (value < 1024) {
        value = '<1KB';
      } else {
        if (value < 1048576) {
          value = (value / 1024).toFixed(1) + 'KB';
        } else if (value >= 1048576 && value < 1073741824) {
          value = (value / 1048576).toFixed(1) + 'MB';
        } else {
          value = (value / 1073741824).toFixed(2) + 'GB';
        }
      }
    }
    return value;
  },
  /**
   * Convert ip address to integer
   * @param ip
   * @return integer
   */
  ipToInt: function ipToInt(ip) {
    // *     example 1: ipToInt('192.0.34.166');
    // *     returns 1: 3221234342
    // *     example 2: ipToInt('255.255.255.256');
    // *     returns 2: false
    // Verify IP format.
    if (!/^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/.test(ip)) {
      return false; // Invalid format.
    }
    // Reuse ip variable for component counter.
    var d = ip.split('.');
    return ((+d[0] * 256 + +d[1]) * 256 + +d[2]) * 256 + +d[3];
  },

  sortByOrder: function sortByOrder(sortOrder, array) {
    var sorted = [];
    for (var i = 0; i < sortOrder.length; i++) {
      for (var j = 0; j < array.length; j++) {
        if (sortOrder[i] == Em.get(array[j], 'id')) {
          sorted.push(array[j]);
        }
      }
    }return sorted;
  },

  utf8ToB64: function utf8ToB64(stringToEncode) {
    return window.btoa(unescape(encodeURIComponent(stringToEncode)));
  },

  b64ToUtf8: function b64ToUtf8(stringTodecode) {
    return decodeURIComponent(escape(window.atob(stringTodecode)));
  }

};

});

require.register("utils/number_utils", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
module.exports = {

  /**
   * @const
   */
  BYTES_IN_MB: 1024 * 1024,

  /**
   * Convert byte size to other metrics.
   *
   * @param {Number} bytes to convert to string
   * @param {Number} precision Number to adjust precision of return value. Default is 0.
   * @param {String} parseType
   *           JS method name for parse string to number. Default is "parseInt".
   * @param {Number} multiplyBy bytes by this number if given. This is needed
   *          as <code>null * 1024 = 0</null>
   * @remarks The parseType argument can be "parseInt" or "parseFloat".
   * @return {String} Returns converted value with abbreviation.
   */
  bytesToSize: function bytesToSize(bytes, precision, parseType, multiplyBy) {
    if (bytes === null || bytes === undefined) {
      return 'n/a';
    } else {
      if (arguments[2] === undefined) {
        parseType = 'parseInt';
      }
      if (arguments[3] === undefined) {
        multiplyBy = 1;
      }
      var value = bytes * multiplyBy;
      var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];
      var posttxt = 0;
      while (value >= 1024) {
        posttxt++;
        value = value / 1024;
      }
      if (value === 0) {
        precision = 0;
      }
      var parsedValue = window[parseType](value);
      return parsedValue.toFixed(precision) + " " + sizes[posttxt];
    }
  },

  /**
   * Validates if the given string or number is an integer between the
   * values of min and max (inclusive). The minimum and maximum
   * checks are ignored if their valid is NaN.
   *
   * @method validateInteger
   * @param {string|number} str - input string
   * @param {string|number} [min]
   * @param {string|number} [max]
   */
  validateInteger: function validateInteger(str, min, max) {
    if (str == null || str == undefined || (str + "").trim().length < 1) {
      return Em.I18n.t('number.validate.empty');
    } else {
      str = (str + "").trim();
      var number = parseInt(str);
      if (isNaN(number)) {
        return Em.I18n.t('number.validate.notValidNumber');
      } else {
        if (str.length != (number + "").length) {
          // parseInt("1abc") returns 1 as integer
          return Em.I18n.t('number.validate.notValidNumber');
        }
        if (!isNaN(min) && number < min) {
          return Em.I18n.t('number.validate.lessThanMinimum').format(min);
        }
        if (!isNaN(max) && number > max) {
          return Em.I18n.t('number.validate.moreThanMaximum').format(max);
        }
      }
    }
    return null;
  },
  /**
   * @param {String|Number} cardinality - value to parse
   * @param {Boolean} isMax - return maximum count
   * @return {Number}
   **/
  getCardinalityValue: function getCardinalityValue(cardinality, isMax) {
    if (cardinality) {
      var isOptional = cardinality.toString().split('-').length > 1;
      if (isOptional) {
        return parseInt(cardinality.split('-')[isMax ? 1 : 0]);
      } else {
        if (isMax) return (/^\d+\+/.test(cardinality) || cardinality == 'ALL' ? Infinity : parseInt(cardinality)
        );
        return cardinality == 'ALL' ? Infinity : parseInt(cardinality.toString().replace('+', ''));
      }
    } else {
      return 0;
    }
  },

  getFloatDecimals: function getFloatDecimals(number, separator) {
    separator = separator || '.';

    var value = '' + number;
    var decimals = value.split(separator);

    decimals = decimals[1] ? decimals[1].length : 0;

    return decimals;
  },

  /**
   * @param n
   * @return {boolean}
   */
  isPositiveNumber: function isPositiveNumber(n) {
    var number = Number(n);
    return !isNaN(number) && isFinite(number) && number > 0;
  }
};

});

require.register("utils/object_utils", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var stringUtils = require('utils/string_utils');

var types = {
  'get': function get(prop) {
    return Object.prototype.toString.call(prop);
  },
  'object': '[object Object]',
  'array': '[object Array]',
  'string': '[object String]',
  'boolean': '[object Boolean]',
  'number': '[object Number]'
};

module.exports = {

  isChild: function isChild(obj) {
    for (var k in obj) {
      if (obj.hasOwnProperty(k)) {
        if (obj[k] instanceof Object) {
          return false;
        }
      }
    }
    return true;
  },

  recursiveKeysCount: function recursiveKeysCount(obj) {
    if (!(obj instanceof Object)) {
      return null;
    }
    var self = this;
    function r(obj) {
      var count = 0;
      for (var k in obj) {
        if (self.isChild(obj[k])) {
          count++;
        } else {
          count += r(obj[k]);
        }
      }
      return count;
    }
    return r(obj);
  },

  deepEqual: function deepEqual() {
    var i, l, leftChain, rightChain;
    var values = arguments;
    function compare2Objects(x, y) {
      var p;
      if (isNaN(x) && isNaN(y) && typeof x === 'number' && typeof y === 'number') {
        return true;
      }

      if (x === y) {
        return true;
      }

      if (typeof x === 'function' && typeof y === 'function' || x instanceof Date && y instanceof Date || x instanceof RegExp && y instanceof RegExp || x instanceof String && y instanceof String || x instanceof Number && y instanceof Number) {
        return x.toString() === y.toString();
      }

      if (!(x instanceof Object && y instanceof Object)) {
        return false;
      }

      if (x.isPrototypeOf(y) || y.isPrototypeOf(x)) {
        return false;
      }

      if (x.constructor !== y.constructor) {
        return false;
      }

      if (x.prototype !== y.prototype) {
        return false;
      }

      if (leftChain.indexOf(x) > -1 || rightChain.indexOf(y) > -1) {
        return false;
      }

      for (p in y) {
        if (y.hasOwnProperty(p) !== x.hasOwnProperty(p)) {
          return false;
        } else if (_typeof(y[p]) !== _typeof(x[p])) {
          return false;
        }
      }

      for (p in x) {
        if (y.hasOwnProperty(p) !== x.hasOwnProperty(p)) {
          return false;
        } else if (_typeof(y[p]) !== _typeof(x[p])) {
          return false;
        }
        switch (_typeof(x[p])) {
          case 'object':
          case 'function':
            leftChain.push(x);
            rightChain.push(y);
            if (!compare2Objects(x[p], y[p])) {
              return false;
            }
            leftChain.pop();
            rightChain.pop();
            break;
          default:
            if (x[p] !== y[p]) {
              return false;
            }
            break;
        }
      }

      return true;
    }

    if (arguments.length < 1) {
      return true;
    }

    for (i = 1, l = arguments.length; i < l; i++) {
      leftChain = [];
      rightChain = [];
      if (!compare2Objects(arguments[0], arguments[i])) {
        return false;
      }
    }

    return true;
  },

  recursiveTree: function recursiveTree(obj) {
    if (!(obj instanceof Object)) {
      return null;
    }
    var self = this;
    function r(obj, parent) {
      var leaf = '';
      for (var k in obj) {
        if (self.isChild(obj[k])) {
          leaf += k + ' (' + parent + ')' + '<br/>';
        } else {
          leaf += r(obj[k], parent + '/' + k);
        }
      }
      return leaf;
    }
    return r(obj, '');
  },

  /**
   *
   * @param {object|array|object[]} target
   * @param {object|array|object[]} source
   * @param {function} handler
   * @returns {object|array|object[]}
   */
  deepMerge: function deepMerge(target, source, handler) {
    if ((typeof target === 'undefined' ? 'undefined' : _typeof(target)) !== 'object' || (typeof source === 'undefined' ? 'undefined' : _typeof(source)) !== 'object') return target;
    var handlerOpts = Array.prototype.slice.call(arguments, 3);
    var isArray = Em.isArray(source);
    var ret = handler && typeof handler.apply(this, [target, source].concat(handlerOpts)) !== 'undefined' ? handler(target, source) : isArray ? [] : {};
    var self = this;

    // handle array
    if (isArray) {
      target = target || [];
      ret = ret.concat(target);

      if (types.object === types.get(target[0])) {
        ret = self.smartArrayObjectMerge(target, source);
      } else {
        for (var i = 0; i < source.length; i++) {
          if (typeof ret[i] === 'undefined') {
            ret[i] = source[i];
          } else if (_typeof(source[i]) === 'object') {
            ret[i] = this.deepMerge(target[i], source[i], handler, target, source);
          } else {
            if (target.indexOf(source[i]) === -1) {
              ret.push(source[i]);
            }
          }
        }
      }
    } else {
      if (target && (typeof target === 'undefined' ? 'undefined' : _typeof(target)) === 'object') {
        Em.keys(target).forEach(function (key) {
          ret[key] = target[key];
        });
      }
      Em.keys(source).forEach(function (key) {
        // handle value which is not Array or Object
        if (_typeof(source[key]) !== 'object' || !source[key]) {
          ret[key] = source[key];
        } else {
          if (!target[key]) {
            ret[key] = source[key];
          } else {
            ret[key] = self.deepMerge(target[key], source[key], handler, target, source);
          }
        }
      });
    }

    return ret;
  },

  /**
   * Find objects by index key (@see detectIndexedKey) and merge them.
   *
   * @param {object[]} target
   * @param {object[]} source
   * @returns {object[]}
   */
  smartArrayObjectMerge: function smartArrayObjectMerge(target, source) {
    // keep the first object and take all keys that contains primitive value
    var id = this.detectIndexedKey(target);
    var self = this;
    // when uniq key not found let's merge items by the key itself
    if (!id) {
      source.forEach(function (obj) {
        Em.keys(obj).forEach(function (objKey) {
          var ret = self.objectByRoot(objKey, target);
          if (!Em.isNone(ret)) {
            if ([types.object, types.array].contains(types.get(ret))) {
              target[objKey] = self.deepMerge(obj[objKey], ret);
            } else {
              target[objKey] = ret;
            }
          } else {
            var _obj = {};
            _obj[objKey] = obj[objKey];
            target.push(_obj);
          }
        });
      });
      return target;
    }

    return target.mapProperty(id).concat(source.mapProperty(id)).uniq().map(function (value) {
      if (!target.someProperty(id, value)) {
        return source.findProperty(id, value);
      } else if (!source.someProperty(id, value)) {
        return target.findProperty(id, value);
      }
      return self.deepMerge(target.findProperty(id, value), source.findProperty(id, value));
    });
  },

  /**
   * Determines key with uniq value. This key will be used to find correct objects in target and source to merge.
   *
   * @param {object} target
   * @returns {string|undefined}
   */
  detectIndexedKey: function detectIndexedKey(target) {
    var keys = Em.keys(target[0]).map(function (key) {
      if ([types.object, types.array].contains(types.get(target[0][key]))) {
        return null;
      }
      return key;
    }).compact();
    return keys.filter(function (key) {
      var values = target.mapProperty(key);
      return values.length === values.uniq().length;
    })[0];
  },

  /**
   *
   * @param {string} rootKey
   * @param {object[]} target
   */
  objectByRoot: function objectByRoot(rootKey, target) {
    return target.map(function (item) {
      return item[rootKey] || null;
    }).compact()[0];
  }
};

});

require.register("utils/polling", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
App.Poll = Em.Object.extend(App.ReloadPopupMixin, {
  name: '',
  stage: '',
  label: '',
  isVisible: true,
  isStarted: false,
  isPolling: true,
  clusterName: null,
  requestId: null,
  temp: false,
  progress: 0,
  url: null,
  testUrl: null,
  data: null,
  isError: false,
  isSuccess: false,
  POLL_INTERVAL: 4000,
  polledData: [],
  numPolls: 0,
  mockDataPrefix: '/data/wizard/deploy/5_hosts',
  currentTaskId: null,

  barWidth: Em.computed.format('width: {0}%;', 'progress'),

  isCompleted: Em.computed.or('isError', 'isSuccess'),

  showLink: Em.computed.or('isPolling', 'isStarted'),

  start: function start() {
    if (Em.isNone(this.get('requestId'))) {
      this.setRequestId();
    } else {
      this.startPolling();
    }
  },

  setRequestId: function setRequestId() {
    if (App.get('testMode')) {
      this.set('requestId', '1');
      this.doPolling();
      return;
    }
    var self = this;
    var url = this.get('url');
    var method = 'PUT';
    var data = this.get('data');

    $.ajax({
      type: method,
      url: url,
      data: data,
      dataType: 'text',
      timeout: App.timeout,
      success: function success(data) {
        var jsonData = jQuery.parseJSON(data);
        if (Em.isNone(jsonData)) {
          self.set('isSuccess', true);
          self.set('isError', false);
        } else {
          var requestId = Em.get(jsonData, 'Requests.id');
          self.set('requestId', requestId);
          self.doPolling();
        }
      },

      error: function error() {
        self.set('isError', true);
        self.set('isSuccess', false);
      },

      statusCode: require('data/statusCodes')
    });
  },

  /**
   * set current task id and send request
   * @param taskId
   */
  updateTaskLog: function updateTaskLog(taskId) {
    this.set('currentTaskId', taskId);
    this.pollTaskLog();
  },

  doPolling: function doPolling() {
    if (!Em.isNone(this.get('requestId'))) {
      this.startPolling();
    }
  },

  /**
   * server call to obtain task logs
   */
  pollTaskLog: function pollTaskLog() {
    if (!Em.isNone(this.get('currentTaskId'))) {
      App.ajax.send({
        name: 'background_operations.get_by_task',
        sender: this,
        data: {
          requestId: this.get('requestId'),
          taskId: this.get('currentTaskId')
        },
        success: 'pollTaskLogSuccessCallback'
      });
    }
  },

  /**
   * update logs of current task
   * @param data
   */
  pollTaskLogSuccessCallback: function pollTaskLogSuccessCallback(data) {
    var currentTask = this.get('polledData').findProperty('Tasks.id', data.Tasks.id);
    currentTask.Tasks.stdout = data.Tasks.stdout;
    currentTask.Tasks.stderr = data.Tasks.stderr;
    Em.propertyDidChange(this, 'polledData');
  },

  /**
   * start polling operation data
   * @return {Boolean}
   */
  startPolling: function startPolling() {
    if (Em.isNone(this.get('requestId'))) return false;

    this.pollTaskLog();
    App.ajax.send({
      name: 'background_operations.get_by_request',
      sender: this,
      data: {
        requestId: this.get('requestId'),
        callback: this.startPolling
      },
      success: 'reloadSuccessCallback',
      error: 'reloadErrorCallback'
    });
    return true;
  },

  reloadSuccessCallback: function reloadSuccessCallback(data) {
    var self = this;
    var result = this.parseInfo(data);
    this._super();
    if (!result) {
      window.setTimeout(function () {
        self.startPolling();
      }, this.POLL_INTERVAL);
    }
  },

  reloadErrorCallback: function reloadErrorCallback(request, ajaxOptions, error, opt, params) {
    this._super(request, ajaxOptions, error, opt, params);
    if (request.status && !this.get('isSuccess')) {
      this.set('isError', true);
    }
  },

  stopPolling: function stopPolling() {
    //this.set('isSuccess', true);
  },

  replacePolledData: function replacePolledData(polledData) {
    var currentTaskId = this.get('currentTaskId');
    if (!Em.isNone(currentTaskId)) {
      var task = this.get('polledData').findProperty('Tasks.id', currentTaskId);
      var currentTask = polledData.findProperty('Tasks.id', currentTaskId);
      if (task && currentTask) {
        currentTask.Tasks.stdout = task.Tasks.stdout;
        currentTask.Tasks.stderr = task.Tasks.stderr;
      }
    }
    this.set('polledData', polledData);
  },

  calculateProgressByTasks: function calculateProgressByTasks(tasksData) {
    var queuedTasks = tasksData.filterProperty('Tasks.status', 'QUEUED').length;
    var completedTasks = tasksData.filter(function (task) {
      return ['COMPLETED', 'FAILED', 'ABORTED', 'TIMEDOUT'].contains(task.Tasks.status);
    }).length;
    var inProgressTasks = tasksData.filterProperty('Tasks.status', 'IN_PROGRESS').length;
    return Math.ceil((queuedTasks * 0.09 + inProgressTasks * 0.35 + completedTasks) / tasksData.length * 100);
  },

  isPollingFinished: function isPollingFinished(polledData) {
    var runningTasks;
    runningTasks = polledData.filterProperty('Tasks.status', 'QUEUED').length;
    runningTasks += polledData.filterProperty('Tasks.status', 'IN_PROGRESS').length;
    runningTasks += polledData.filterProperty('Tasks.status', 'PENDING').length;
    if (runningTasks === 0) {
      if (polledData.everyProperty('Tasks.status', 'COMPLETED')) {
        this.set('isSuccess', true);
        this.set('isError', false);
      } else if (polledData.someProperty('Tasks.status', 'FAILED') || polledData.someProperty('Tasks.status', 'TIMEDOUT') || polledData.someProperty('Tasks.status', 'ABORTED')) {
        this.set('isSuccess', false);
        this.set('isError', true);
      }
      return true;
    } else {
      return false;
    }
  },

  parseInfo: function parseInfo(polledData) {
    var tasksData = polledData.tasks;
    var requestId = this.get('requestId');
    if (polledData.Requests && !Em.isNone(polledData.Requests.id) && polledData.Requests.id != requestId) {
      // We don't want to use non-current requestId's tasks data to
      // determine the current install status.
      // Also, we don't want to keep polling if it is not the
      // current requestId.
      return false;
    }
    this.replacePolledData(tasksData);
    var totalProgress = this.calculateProgressByTasks(tasksData);
    this.set('progress', totalProgress.toString());
    return this.isPollingFinished(tasksData);
  }

});

});

require.register("utils/stomp_client", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Example:
 *
 * stompClient.connect();
 * stompClient.subscribe('topic1', handlerFunc1);
 * stompClient.addHandler('topic1', 'handler2-name', handlerFunc2);
 * stompClient.removeHandler('topic1', 'handler2-name');
 * stompClient.unsubscribe('topic1');
 * stompClient.disconnect();
 *
 */

module.exports = Em.Object.extend({
  /**
   * @type {Stomp}
   */
  client: null,

  /**
   * @type {string}
   */
  webSocketUrl: '{protocol}://{hostname}{port}/api/stomp/v1/websocket',

  /**
   * @type {string}
   */
  sockJsUrl: '{protocol}://{hostname}{port}/api/stomp/v1',

  /**
   * sockJs should use only alternative options as transport in case when websocket supported but connection fails
   * @const
   * @type {Array}
   */
  sockJsTransports: ['eventsource', 'xhr-polling', 'iframe-xhr-polling', 'jsonp-polling'],

  /**
   * @type {boolean}
   */
  isConnected: false,

  /**
   * @type {boolean}
   */
  isWebSocketSupported: true,

  /**
   * @type {number}
   * @const
   */
  RECONNECT_TIMEOUT: 6000,

  /**
   * @type {object}
   */
  subscriptions: {},

  /**
   * default headers
   * @type {object}
   */
  headers: {},

  /**
   *
   * @param {boolean} useSockJS
   * @returns {$.Deferred}
   */
  connect: function connect(useSockJS) {
    var _this = this;

    var dfd = $.Deferred();
    var socket = this.getSocket(useSockJS);
    var client = Stomp.over(socket);
    var headers = this.get('headers');

    client.connect(headers, function () {
      _this.onConnectionSuccess();
      dfd.resolve();
    }, function () {
      dfd.reject(_this.onConnectionError(useSockJS));
    });
    client.debug = Em.K;
    this.set('client', client);
    return dfd.promise();
  },

  /**
   *
   * @param {boolean} useSockJS
   * @returns {SockJS|WebSocket}
   */
  getSocket: function getSocket(useSockJS) {
    if (!WebSocket || useSockJS) {
      this.set('isWebSocketSupported', false);
      var sockJsUrl = this.getSocketUrl(this.get('sockJsUrl'), false);
      return new SockJS(sockJsUrl, null, { transports: this.get('sockJsTransports') });
    } else {
      return new WebSocket(this.getSocketUrl(this.get('webSocketUrl'), true));
    }
  },

  /**
   *
   * @param {string} template
   * @param {boolean} isWebsocket
   * @returns {string}
   */
  getSocketUrl: function getSocketUrl(template, isWebsocket) {
    var hostname = this.getHostName();
    var isSecure = this.isSecure();
    var protocol = isWebsocket ? isSecure ? 'wss' : 'ws' : isSecure ? 'https' : 'http';
    var port = this.getPort();
    return template.replace('{hostname}', hostname).replace('{protocol}', protocol).replace('{port}', port);
  },

  getHostName: function getHostName() {
    return window.location.hostname;
  },

  isSecure: function isSecure() {
    return window.location.protocol === 'https:';
  },

  getPort: function getPort() {
    return window.location.port ? ':' + window.location.port : '';
  },

  onConnectionSuccess: function onConnectionSuccess() {
    this.set('isConnected', true);
  },

  onConnectionError: function onConnectionError(useSockJS) {
    if (this.get('isConnected')) {
      this.reconnect(useSockJS);
    } else if (!useSockJS) {
      //if SockJs connection failed too the stop trying to connect
      //if webSocket failed on initial connect then switch to SockJS
      return this.connect(true);
    }
  },

  reconnect: function reconnect(useSockJS) {
    var _this2 = this;

    var subscriptions = _extends({}, this.get('subscriptions'));
    setTimeout(function () {
      console.debug('Reconnecting to WebSocket...');
      _this2.connect(useSockJS).done(function () {
        _this2.set('subscriptions', {});
        for (var i in subscriptions) {
          _this2.subscribe(subscriptions[i].destination, subscriptions[i].handlers['default']);
          for (var key in subscriptions[i].handlers) {
            key !== 'default' && _this2.addHandler(subscriptions[i].destination, key, subscriptions[i].handlers[key]);
          }
        }
      });
    }, this.RECONNECT_TIMEOUT);
  },

  disconnect: function disconnect() {
    this.get('client').disconnect();
  },

  /**
   *
   * @param {string} destination
   * @param {string} body
   * @param {object} headers
   */
  send: function send(destination, body) {
    var headers = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    if (this.get('client.connected')) {
      this.get('client').send(destination, headers, body);
      return true;
    }
    return false;
  },

  /**
   *
   * @param destination
   * @param {function} handler
   * @returns {*}
   */
  subscribe: function subscribe(destination) {
    var handler = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Em.K;

    var handlers = {
      default: handler
    };
    if (!this.get('client.connected')) {
      return null;
    }
    if (this.get('subscriptions')[destination]) {
      console.error('Subscription with default handler for ' + destination + ' already exists');
      return this.get('subscriptions')[destination];
    } else {
      var subscription = this.get('client').subscribe(destination, function (message) {
        for (var i in handlers) {
          handlers[i](JSON.parse(message.body));
        }
      });
      subscription.destination = destination;
      subscription.handlers = handlers;
      this.get('subscriptions')[destination] = subscription;
      return subscription;
    }
  },

  /**
   * If trying to add handler to not existing subscription then it will be created and handler added as default
   * @param {string} destination
   * @param {string} key
   * @param {function} handler
   */
  addHandler: function addHandler(destination, key, handler) {
    var subscription = this.get('subscriptions')[destination];
    if (!subscription) {
      this.subscribe(destination);
      return this.addHandler(destination, key, handler);
    }
    if (subscription.handlers[key]) {
      console.error('You can\'t override subscription handler');
      return;
    }
    subscription.handlers[key] = handler;
  },

  /**
   * If removed handler is last and subscription have zero handlers then topic will be unsubscribed
   * @param {string} destination
   * @param {string} key
   */
  removeHandler: function removeHandler(destination, key) {
    var subscription = this.get('subscriptions')[destination];
    delete subscription.handlers[key];
    if (Em.keys(subscription.handlers).length === 0) {
      this.unsubscribe(destination);
    }
  },

  /**
   *
   * @param {string} destination
   * @returns {boolean}
   */
  unsubscribe: function unsubscribe(destination) {
    if (this.get('subscriptions')[destination]) {
      this.get('subscriptions')[destination].unsubscribe();
      delete this.get('subscriptions')[destination];
      return true;
    }
    return false;
  }
});

});

require.register("utils/string_utils", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {

  pad: function pad(str, len, _pad, dir) {

    var STR_PAD_LEFT = 1;
    var STR_PAD_RIGHT = 2;
    var STR_PAD_BOTH = 3;

    if (typeof len == "undefined") {
      len = 0;
    }
    if (typeof _pad == "undefined") {
      _pad = ' ';
    }
    if (typeof dir == "undefined") {
      dir = STR_PAD_RIGHT;
    }

    if (len + 1 >= str.length) {

      switch (dir) {

        case STR_PAD_LEFT:
          str = Array(len + 1 - str.length).join(_pad) + str;
          break;

        case STR_PAD_BOTH:
          var padlen = len - str.length;
          var right = Math.ceil(padlen / 2);
          var left = padlen - right;
          str = Array(left + 1).join(_pad) + str + Array(right + 1).join(_pad);
          break;

        default:
          str = str + Array(len + 1 - str.length).join(_pad);
          break;

      } // switch
    }
    return str;
  },
  underScoreToCamelCase: function underScoreToCamelCase(name) {
    function replacer(str, p1, p2, offset, s) {
      return str[1].toUpperCase();
    }
    return name.replace(/_\w/g, replacer);
  },

  /**
   * Forces given string into upper camel-case representation. The first
   * character of each word will be capitalized with the rest in lower case.
   */
  getCamelCase: function getCamelCase(name) {
    if (name != null) {
      return name.toLowerCase().replace(/(\b\w)/g, function (f) {
        return f.toUpperCase();
      });
    }
    return name;
  },

  /**
   * Compare two versions by following rules:
   * first higher than second then return 1
   * first lower than second then return -1
   * first equal to second then return 0
   * @param first {string}
   * @param second {string}
   * @return {number}
   */
  compareVersions: function compareVersions(first, second) {
    if (!(typeof first === 'string' && typeof second === 'string')) {
      return -1;
    }
    if (first === '' || second === '') {
      return -1;
    }
    var firstNumbers = first.split(/[\.-]/);
    var secondNumbers = second.split(/[\.-]/);
    var length = 0;
    var i = 0;
    var result = false;
    if (firstNumbers.length === secondNumbers.length) {
      length = firstNumbers.length;
    } else if (firstNumbers.length < secondNumbers.length) {
      length = secondNumbers.length;
    } else {
      length = firstNumbers.length;
    }

    while (i < length && !result) {
      firstNumbers[i] = firstNumbers[i] === undefined ? 0 : window.parseInt(firstNumbers[i]);
      secondNumbers[i] = secondNumbers[i] === undefined ? 0 : window.parseInt(secondNumbers[i]);
      if (firstNumbers[i] > secondNumbers[i]) {
        result = 1;
        break;
      } else if (firstNumbers[i] === secondNumbers[i]) {
        result = 0;
      } else if (firstNumbers[i] < secondNumbers[i]) {
        result = -1;
        break;
      }
      i++;
    }
    return result;
  },

  isSingleLine: function isSingleLine(string) {
    return String(string).trim().indexOf("\n") == -1;
  },
  /**
   * transform array of objects into CSV format content
   * @param array
   * @return {Array}
   */
  arrayToCSV: function arrayToCSV(array) {
    var content = "";
    array.forEach(function (item) {
      var row = [];
      for (var i in item) {
        if (item.hasOwnProperty(i)) {
          row.push(item[i]);
        }
      }
      content += row.join(',') + '\n';
    });
    return content;
  },

  /**
   * Extracts filename from linux/unix path
   * @param path
   * @return {string}: filename
   */
  getFileFromPath: function getFileFromPath(path) {
    if (!path || typeof path !== 'string') {
      return '';
    }
    return path.replace(/^.*[\/]/, '');
  },

  getPath: function getPath(path) {
    if (!path || typeof path !== 'string' || path[0] != '/') {
      return '';
    }
    var last_slash = path.lastIndexOf('/');
    return last_slash != 0 ? path.substr(0, last_slash) : '/';
  },

  /**
   * @method getFormattedStringFromArray Get formatted string of elements to display on the UI
   * Example:
   * var arr = [ambari, bigdata, hadoop]
   * getFormattedStringFromArray(arr);  // ambari, bigdata and hadoop
   * @param array {Array}  Array of elements
   * @param [endSeparator=Em.I18n.t('and')] {String}
   * @returns {String}
   */
  getFormattedStringFromArray: function getFormattedStringFromArray(array, endSeparator) {
    var label = '';
    endSeparator = endSeparator || Em.I18n.t('and');
    array.forEach(function (_arrElement) {
      if (array.length === 1) {
        label = _arrElement;
      } else {
        if (_arrElement !== array[array.length - 1]) {
          // [clients.length - 1]
          label = label + ' ' + _arrElement;
          if (_arrElement !== array[array.length - 2]) {
            label = label + ',';
          }
        } else {
          label = label + ' ' + endSeparator + ' ' + _arrElement;
        }
      }
    }, this);
    return label.trim();
  },
  /**
   * Get plural|singular value of string by related count.
   *
   * @param {Number} count
   * @param {String} singular
   * @param {String} [plural]
   * @return {String}
   * @method pluralize
   */
  pluralize: function (_pluralize) {
    function pluralize(_x, _x2, _x3) {
      return _pluralize.apply(this, arguments);
    }

    pluralize.toString = function () {
      return _pluralize.toString();
    };

    return pluralize;
  }(function (count, singular, plural) {
    var _plural = plural || pluralize(singular);
    return count > 1 ? _plural : singular;
  }),

  /**
   * decode html entities
   * @param {string} string
   * @returns {string}
   */
  htmlEntities: function htmlEntities(string) {
    if (typeof string !== 'string') return "";
    return $("<div/>").text(string).html();
  },

  /**
   * Escaping user input to be treated as a literal string within a regular expression
   * get from https://developer.mozilla.org/en/docs/Web/JavaScript/Guide/Regular_Expressions
   * @param {string} str
   * @returns {*}
   */
  escapeRegExp: function escapeRegExp(str) {
    return str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
  },

  /**
   * Generates random string using upper and lower letters and digits
   *
   * @param {number} len
   * @param {String} [allowed]
   * @returns {String}
   * @method getRandomString
   */
  getRandomString: function getRandomString(len, allowed) {
    Em.assert('len should be defined and more than 0', len > 0);
    var text = '';
    allowed = typeof allowed === 'string' ? allowed : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    for (var i = 0; i < len; i++) {
      text += allowed.charAt(Math.floor(Math.random() * allowed.length));
    }
    return text;
  },

  /**
   * @param {string} string
   * @returns {string}
   * @method upperUnderscoreToText
   */
  upperUnderscoreToText: function upperUnderscoreToText(string) {
    if (typeof string !== 'string') {
      return '';
    }
    return string.split('_').map(function (word) {
      return word.toLowerCase().capitalize();
    }).join(' ');
  },

  /**
   *
   * @param {string} string
   * @param {RegExp} regexp
   */
  unicodeEscape: function unicodeEscape(string) {
    var regexp = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : /[\s\S]/g;

    return string.replace(regexp, function (escape) {
      return "\\u" + ('0000' + escape.charCodeAt().toString(16)).slice(-4);
    });
  }
};

});

require.register("utils/ui_effects", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {
  /**
   *
   * @param node {DOMElement} - DOM element which blinking
   * @param delay {number} - overall time of blinking
   * @param callback {function}
   * @param interval {number} - change frequence of blinking
   */
  pulsate: function pulsate(node, delay, callback, interval) {
    var self = this;
    /**
     * execute single blink
     * @param interval {number} - time of single blink
     * @param callback {function}
     * @param opacity {string|number}
     * @param iteration {number} - current iteration(default amount of iterations: 10)
     * @param isReverse {boolean} - flag, that mean opacity increase or decrease
     */
    var blink = function blink(interval, callback, opacity, iteration, isReverse) {
      var iterations = 10;
      opacity = opacity || 1;
      iteration = iteration !== undefined ? iteration : 10;
      if (iteration > 0) {
        iteration--;
        setTimeout(function () {
          isReverse = isReverse || opacity <= 1 / (iterations / 2);
          opacity = isReverse ? opacity + 1 / (iterations / 2) : opacity - 1 / (iterations / 2);
          node.css('opacity', opacity);
          blink(interval, callback, opacity, iteration, isReverse);
        }, interval / iterations);
      } else {
        node.css('opacity', 1);
        callback();
      }
    };
    interval = interval || 200;
    if (delay > 0) {
      delay -= interval;
      setTimeout(function () {
        blink(interval, function () {
          self.pulsate(node, delay, callback, interval);
        });
      }, interval);
    } else {
      callback();
    }
  }
};

});

require.register("utils/updater", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

var states = {};

function update(obj, name, isWorking, interval, urlPattern) {
  if (typeof isWorking == 'string' && !obj.get(isWorking)) {
    return false;
  }

  var state = states[name];

  if (!state) {
    var callback = function callback() {
      update(obj, name, isWorking, interval, urlPattern);
    };
    states[name] = state = {
      timeout: null,
      func: function func() {
        var urlRegExp = new RegExp(urlPattern);
        if (typeof isWorking == 'string' && !obj.get(isWorking)) {
          return false;
        }

        if (urlRegExp.test(App.router.get('location.lastSetURL'))) {
          obj[name](callback);
        } else {
          callback();
        }
        return true;
      },
      interval: interval,
      callback: callback
    };
  }

  clearTimeout(state.timeout);

  state.timeout = setTimeout(state.func, state.interval);
  return true;
}

function rerun(name) {
  var state = states[name];
  if (state) {
    clearTimeout(state.timeout);
    state.func();
  }
}

App.updater = {

  /**
   * Run function periodically with <code>App.contentUpdateInterval</code> delay.
   * Example 1(wrong way, will not be working):
   *    var obj = {
   *      method: function(callback){
   *        //do something
   *      }
   *    };
   *    App.updater.run(obj, 'method');
   *
   * Will be called only once, because <code>callback</code> will never execute. Below is right way:
   *
   * Example 2:
   *    var obj = {
   *      method: function(callback){
   *        //do something
   *        callback();
   *      }
   *    };
   *    App.updater.run(obj, 'method');
   *
   * Method will always be called.
   *
   * Example 3:
   *    var obj = {
   *      method: function(callback){
   *          //do something
   *          callback();
   *      },
   *      isWorking: true
   *    };
   *    App.updater.run(obj, 'method', 'isWorking');
   *
   * <code>obj.method</code> will be called automatically.
   * Warning: You should call <code>callback</code> parameter when function finished iteration.
   * Otherwise nothing happened next time.
   * If <code>isWorking</code> provided, library will check <code>obj.isWorking</code> before iteration and
   * stop working when it equals to false. Otherwise method will always be called.
   *
   *
   *
   * @param obj Object
   * @param name Method name
   * @param isWorking Property, which will be checked as a rule for working
   * @param interval Interval between calls
   * @param urlPattern Pattern to match allowed urls
   * @param [interval] Interval between calls
   * @return {*}
   */
  run: function run(obj, name, isWorking, interval, urlPattern) {
    interval = interval || App.contentUpdateInterval;
    return update(obj, name, isWorking, interval, urlPattern);
  },

  /**
   * Immediate run function, which is periodically running using <code>run</code> method
   * Example:
   *    App.updater.run(obj, 'clickIt');
   *    App.updater.immediateRun('clickIt');
   *
   * <code>clickIt</code> will be executed immediately and proceed executing periodically
   *
   * @param name Method name, which was used as a parameter in <code>run</code> method
   * @return {*}
   */
  immediateRun: function immediateRun(name) {
    return rerun(name);
  },
  /**
   * Update interval for state by its name.
   *
   * @param name - state name
   * @param value - new interval value
   **/
  updateInterval: function updateInterval(name, value) {
    if (!states[name]) return;
    states[name].interval = value;
  }

};

});

require.register("utils/validator", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

module.exports = {

  isValidEmail: function isValidEmail(value) {
    var emailRegex = /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))$/i;
    return emailRegex.test(value);
  },

  isValidInt: function isValidInt(value) {
    var intRegex = /^-?\d+$/;
    return intRegex.test(value);
  },

  isValidUNIXUser: function isValidUNIXUser(value) {
    var regex = /^[a-z_][a-z0-9_-]{0,31}$/;
    return regex.test(value);
  },

  isValidFloat: function isValidFloat(value) {
    if (typeof value === 'string' && value.trim() === '') {
      return false;
    }
    var floatRegex = /^-?(?:\d+|\d{1,3}(?:,\d{3})+)?(?:\.\d+)?$/;
    return floatRegex.test(value);
  },
  /**
   * validate directory with slash or drive at the start
   * @param value
   * @return {Boolean}
   */
  isValidDir: function isValidDir(value) {
    var floatRegex = /^\/[0-9a-z]*/;
    var winRegex = /^[a-z]:\\[0-9a-zA-Z]*/;
    var winUrlRegex = /^file:\/\/\/[a-zA-Z]:\/[0-9a-zA-Z]*/;
    var dirs = value.split(',');
    if (dirs.some(function (i) {
      return i.startsWith(' ');
    })) {
      return false;
    }
    for (var i = 0; i < dirs.length; i++) {
      if (!floatRegex.test(dirs[i]) && !winRegex.test(dirs[i]) && !winUrlRegex.test(dirs[i])) {
        return false;
      }
    }
    return true;
  },

  /**
   * validate filename
   */
  isValidFileName: function isValidFileName(value) {
    var filenameRegex = /^[0-9a-zA-Z_-]+\.[a-zA-Z]+$/;
    return filenameRegex.test(value);
  },

  /**
   * defines if config value looks like link to other config
   * @param value
   * @returns {boolean}
   */
  isConfigValueLink: function isConfigValueLink(value) {
    return (/^\${.+}$/.test(value)
    );
  },

  /**
   * validate directory with slash at the start
   * @param value
   * @returns {boolean}
   */
  isValidDataNodeDir: function isValidDataNodeDir(value) {
    var dirRegex = /^(\[[0-9a-zA-Z]+_?[0-9a-zA-Z]+\])?(file:\/\/)?(\/[0-9a-z]*)/;
    var winRegex = /^(\[[0-9a-zA-Z]+_?[0-9a-zA-Z]+\])?[a-zA-Z]:\\[0-9a-zA-Z]*/;
    var winUrlRegex = /^(\[[0-9a-zA-Z]+_?[0-9a-zA-Z]+\])?file:\/\/\/[a-zA-Z]:\/[0-9a-zA-Z]*/;
    var dirs = value.split(',');
    if (dirs.some(function (i) {
      return i.startsWith(' ');
    })) {
      return false;
    }
    for (var i = 0; i < dirs.length; i++) {
      if (!dirRegex.test(dirs[i]) && !winRegex.test(dirs[i]) && !winUrlRegex.test(dirs[i])) {
        return false;
      }
    }
    return true;
  },

  /**
   * validate directory doesn't start "home" or "homes"
   * @param value
   * @returns {boolean}
   */
  isAllowedDir: function isAllowedDir(value) {
    var dirs = value.replace(/,/g, ' ').trim().split(new RegExp("\\s+", "g"));
    for (var i = 0; i < dirs.length; i++) {
      if (dirs[i].startsWith('/home') || dirs[i].startsWith('/homes')) {
        return false;
      }
    }
    return true;
  },

  /**
   * validate ip address with port
   * @param value
   * @return {Boolean}
   */
  isIpAddress: function isIpAddress(value) {
    var ipRegex = /^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)($|\:[0-9]{1,5})$/;
    return ipRegex.test(value);
  },

  /**
   * validate hostname
   * @param value
   * @return {Boolean}
   */
  isHostname: function isHostname(value) {
    var regex = /(?=^.{3,254}$)(^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])(\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]))*(\.[a-zA-Z]{1,62})$)/;
    return value === 'localhost' || regex.test(value);
  },

  hasSpaces: function hasSpaces(value) {
    var regex = /(\s+)/;
    return regex.test(value);
  },

  isNotTrimmed: function isNotTrimmed(value) {
    var regex = /(^\s+|\s+$)/;
    return regex.test(value);
  },

  /**
   * Check if string ends with spaces.
   * For multiline content only last line will be checked.
   *
   * @method isNotTrimmedLeft
   * @param {String} value
   * @returns {Boolean} - <code>true</code> if ends with spaces
   */
  isNotTrimmedRight: function isNotTrimmedRight(value) {
    return value !== ' ' && /\s+$/.test(("" + value).split(/\n/).slice(-1)[0]);
  },

  /**
   * validate domain name with port
   * @param value
   * @return {Boolean}
   */
  isDomainName: function isDomainName(value) {
    var domainRegex = /^([a-zA-Z0-9]([a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,6}$/;
    return domainRegex.test(value);
  },

  /**
   * validate username
   * @param value
   * @return {Boolean}
   */
  isValidUserName: function isValidUserName(value) {
    var usernameRegex = /^[a-z]([-a-z0-9]{0,30})$/;
    return usernameRegex.test(value);
  },

  /**
   * validate db name
   * @param value
   * @returns {boolean}
   */
  isValidDbName: function isValidDbName(value) {
    var dbPattern = /^\S+$/;
    return dbPattern.test(value);
  },

  /**
   * validate key of configurations
   * allow spaces as prefix and suffix
   *
   * @param value
   * @return {Boolean}
   */
  isValidConfigKey: function isValidConfigKey(value) {
    var configKeyRegex = /^\s*[0-9a-z_\-\.\/\*]+\s*$/i;
    return configKeyRegex.test(value);
  },

  /**
   * validate configuration group name
   * @param value
   * @return {Boolean}
   */
  isValidConfigGroupName: function isValidConfigGroupName(value) {
    var configKeyRegex = /^[\s0-9a-z_\-]+$/i;
    return configKeyRegex.test(value);
  },

  /**
   * validate alert notification name
   * @param value
   * @return {Boolean}
   */
  isValidAlertNotificationName: function isValidAlertNotificationName(value) {
    var configKeyRegex = /^[\s0-9a-z_\-]+$/i;
    return configKeyRegex.test(value);
  },

  /**
   * validate alert group name
   * @param value
   * @return {Boolean}
   */
  isValidAlertGroupName: function isValidAlertGroupName(value) {
    var configKeyRegex = /^[\s0-9a-z_\-]+$/i;
    return configKeyRegex.test(value);
  },

  empty: function empty(e) {
    switch (e) {
      case "":
      case 0:
      case "0":
      case null:
      case false:
      case undefined:
      case typeof this == "undefined":
        return true;
      default:
        return false;
    }
  },
  /**
   * Validate string that will pass as parameter to .matches() url param.
   * Try to prevent invalid regexp.
   * For example: /api/v1/clusters/c1/hosts?Hosts/host_name.matches(.*localhost.)
   *
   * @param {String} value - string to validate
   * @return {Boolean}
   * @method isValidMatchesRegexp
   */
  isValidMatchesRegexp: function isValidMatchesRegexp(value) {
    var checkPair = function checkPair(chars) {
      chars = chars.map(function (c) {
        return '\\' + c;
      });
      var charsReg = new RegExp(chars.join('|'), 'g');
      if (charsReg.test(value)) {
        var pairContentReg = new RegExp(chars.join('.*'), 'g');
        if (!pairContentReg.test(value)) return false;
        var pairCounts = chars.map(function (c) {
          return value.match(new RegExp(c, 'g')).length;
        });
        if (pairCounts[0] != pairCounts[1]) return false;
      }
      return true;
    };
    if (/^[\?\|\*\!,]/.test(value)) return false;
    return (/^((\.\*?)?([\w\s\[\]\/\?\-_,\|\*\!\{\}\(\)]*)?)+(\.\*?)?$/g.test(value) && checkPair(['[', ']']) && checkPair(['{', '}'])
    );
  },

  /**
  * Remove validation messages for components which are already installed
  */
  filterNotInstalledComponents: function filterNotInstalledComponents(validationData) {
    var hostComponents = App.HostComponent.find();
    return validationData.resources[0].items.filter(function (item) {
      // true is there is no host with this component
      return hostComponents.filterProperty("componentName", item["component-name"]).filterProperty("hostName", item.host).length === 0;
    });
  },

  isValidRackId: function isValidRackId(path) {
    var _path = '' + path;
    // See app/message.js:hostPopup.setRackId.invalid
    return (/^\/[/.\w-]+$/.test(_path) && _path.length < 255
    );
  },

  /**
   * Validate url
   * @param value
   * @return {Boolean}
   */
  isValidURL: function isValidURL(value) {
    var urlRegex = /^(https?|ftp):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i;
    return urlRegex.test(value);
  },

  /**
   * Validate base URL
   * @param {string} value
   * @returns {boolean}
   */
  isValidBaseUrl: function isValidBaseUrl(value) {
    var remotePattern = /^$|^(?:(?:https?|ftp):\/{2})(?:\S+(?::\S*)?@)?(?:(?:(?:[\w\-.]))*)(?::[0-9]+)?(?:\/\S*)?$/,
        localPattern = /^$|^file:\/{2,3}([a-zA-Z][:|]\/){0,1}[\w~!*'();@&=\/\\\-+$,?%#.\[\]]+$/;
    return remotePattern.test(value) || localPattern.test(value);
  },

  /**
   * Validate widget name
   * @param {string} value
   * @returns {boolean}
   */
  isValidWidgetName: function isValidWidgetName(value) {
    var widgetNameRegex = /^[\s0-9a-z_\-%]+$/i;
    return widgetNameRegex.test(value);
  },

  /**
   * Validate widget description
   * @param {string} value
   * @returns {boolean}
   */
  isValidWidgetDescription: function isValidWidgetDescription(value) {
    var widgetDescriptionRegex = /^[\s0-9a-z_\-%]+$/i;
    return widgetDescriptionRegex.test(value);
  },

  isValidChartWidgetDatasetLabel: function isValidChartWidgetDatasetLabel(value) {
    var widgetDescriptionRegex = /^[\s0-9a-z_\-%]+$/i;
    return widgetDescriptionRegex.test(value);
  },

  /**
   * Validate alert name
   * @param {string} value
   * @returns {boolean}
   */
  isValidAlertName: function isValidAlertName(value) {
    var alertNameRegex = /^[\s0-9a-z_\-%\(\)]+$/i;
    return alertNameRegex.test(value);
  },

  /**
   * Validate ldaps URL
   * @param {string} value
   * @returns {boolean}
   */
  isValidLdapsURL: function isValidLdapsURL(value) {
    var ldapsUrlRegex = /^(ldaps):\/\/(((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:)*@)?(((\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5])\.(\d|[1-9]\d|1\d\d|2[0-4]\d|25[0-5]))|((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?)(:\d*)?)(\/((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)+(\/(([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)*)*)?)?(\?((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|[\uE000-\uF8FF]|\/|\?)*)?(\#((([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(%[\da-f]{2})|[!\$&'\(\)\*\+,;=]|:|@)|\/|\?)*)?$/i;
    return ldapsUrlRegex.test(value);
  },

  isValidNameServiceId: function isValidNameServiceId(value) {
    var nameSarviceIdRegex = /^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])$/;
    return nameSarviceIdRegex.test(value);
  }

};

});

require.register("views", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// load all views here

require('views/application');
require('views/common/breadcrumbs_view');
require('views/common/clock_view');
require('views/common/checkbox_view');
require('views/common/log_search_ui_link_view');
require('views/common/log_file_search_view');
require('views/common/log_tail_view');
require('views/common/global/spinner');
require('views/common/ajax_default_error_popup_body');
require('views/common/buttons/button_progress_view');
require('views/common/buttons/wizard_buttons');
require('views/common/chart');
require('views/common/chart/pie');
require('views/common/chart/linear');
require('views/common/chart/linear_time');
require('views/common/modal_popup');
require('views/common/search_box_view');
require('views/common/modal_popups/alert_popup');
require('views/common/modal_popups/edit_dashboard_widget_popup');
require('views/common/modal_popups/manage_kdc_credentials_popup');
require('views/common/modal_popups/confirmation_feedback_popup');
require('views/common/modal_popups/confirmation_popup');
require('views/common/modal_popups/hosts_table_list_popup');
require('views/common/modal_popups/cluster_check_popup');
require('views/common/modal_popups/invalid_KDC_popup');
require('views/common/modal_popups/dependent_configs_list_popup');
require('views/common/modal_popups/select_groups_popup');
require('views/common/modal_popups/logs_popup');
require('views/common/modal_popups/log_file_search_popup');
require('views/common/modal_popups/log_tail_popup');
require('views/common/modal_popups/config_validation/config_validation_failed_popup');
require('views/common/modal_popups/config_validation/config_validation_popup');
require('views/common/editable_list');
require('views/common/not-scrollable-textarea');
require('views/common/host_progress_popup_body_view');
require('views/common/radio_button_view');
require('views/common/rolling_restart_view');
require('views/common/select_custom_date_view');
require('views/common/metric');
require('views/common/time_range');
require('views/common/time_range_list');
require('views/common/form/field');
require('views/common/form/datepicker_view');
require('views/common/form/spinner_input_view');
require('views/common/form/manage_credentials_form_view');
require('views/common/form/dropdown');
require('views/common/quick_view_link_view');
require('views/common/configs/services_config');
require('views/common/configs/service_config_container_view');
require('views/common/configs/service_configs_by_category_view');
require('views/common/configs/service_config_view');
require('views/common/configs/service_config_tab_view');
require('views/common/configs/config_category_container_view');
require('views/common/configs/overriddenPropertyRow_view');
require('views/common/configs/overriddenProperty_view');
require('views/common/configs/config_versions_control_view');
require('views/common/configs/config_versions_dropdown_view');
require('views/common/configs/selectable_popup_body_view');
require('views/common/configs/custom_category_views/notification_configs_view');
require('views/common/configs/config_diff_view');
require('views/common/configs/widgets/config_text_field');
require('views/common/configs/widgets/plain_config_text_field');
require('views/common/configs/widgets/config_widget_view');
require('views/common/configs/widgets/checkbox_config_widget_view');
require('views/common/configs/widgets/directory_config_widget_view');
require('views/common/configs/widgets/combo_config_widget_view');
require('views/common/configs/widgets/number_config_widget_view');
require('views/common/configs/widgets/password_config_widget_view');
require('views/common/configs/widgets/list_config_widget_view');
require('views/common/configs/widgets/radio_button_config_widget_view');
require('views/common/configs/widgets/slider_config_widget_view');
require('views/common/configs/widgets/string_config_widget_view');
require('views/common/configs/widgets/textfield_config_widget_view');
require('views/common/configs/widgets/time_interval_spinner_view');
require('views/common/configs/widgets/toggle_config_widget_view');
require('views/common/configs/widgets/label_config_widget_view');
require('views/common/configs/widgets/test_db_connection_widget_view');
require('views/common/configs/widgets/overrides/config_widget_override_view');
require('views/common/configs/service_config_layout_tab_view');
require('views/common/configs/service_config_layout_tab_compare_view');
require('views/common/filter_combobox');
require('views/common/filter_combo_cleanable');
require('views/common/pagination_view');
require('views/common/table_view');
require('views/common/progress_bar_view');
require('views/common/controls_view');
require('views/common/configs/controls_view');
require('views/common/widget/graph_widget_view');
require('views/common/widget/template_widget_view');
require('views/common/widget/gauge_widget_view');
require('views/common/widget/number_widget_view');
require('views/common/widget/heatmap_widget_view');
require('views/common/assign_master_components_view');
require('views/common/helpers/format_word_break_view');
require('views/common/helpers/format_null_view');
require('views/common/helpers/format_role_view');
require('views/common/helpers/pluralize_view');
require('views/common/helpers/status_icon_view');
require('views/common/chosen_plugin');
require('views/common/export_metrics_menu_view');
require('views/login');
require('views/main');
require('views/main/menu');
require('views/main/alert_definitions_view');
require('views/main/alerts/alert_definition/alert_definition_summary');
require('views/main/alerts/alert_definition/alert_definition_state');
require('views/main/alerts/definition_details_view');
require('views/main/alerts/alert_search_box');
require('views/main/alerts/alert_definitions_actions_view');
require('views/main/alerts/definition_configs_view');
require('views/main/alerts/manage_alert_groups/select_definitions_popup_body_view');
require('views/main/alerts/add_alert_definition/add_alert_definition_view');
require('views/main/alerts/add_alert_definition/step1_view');
require('views/main/alerts/add_alert_definition/step2_view');
require('views/main/alerts/add_alert_definition/step3_view');
require('views/main/alerts');
require('views/main/alerts/manage_alert_groups_view');
require('views/main/alerts/alert_instances_popup_view');
require('views/main/alerts/manage_alert_notifications_view');
require('views/main/charts');
require('views/main/views/details');
require('views/main/host');
require('views/main/host/combo_search_box');
require('views/main/host/hosts_table_menu_view');
require('views/main/host/details');
require('views/main/host/details/host_component_view');
require('views/main/host/details/host_component_views/datanode_view');
require('views/main/host/details/host_component_views/nodemanager_view');
require('views/main/host/details/host_component_views/regionserver_view');
require('views/main/host/details/host_component_views/tasktracker_view');
require('views/main/host/logs_view');
require('views/main/host/menu');
require('views/main/host/summary');
require('views/main/host/configs');
require('views/main/host/configs_service');
require('views/main/host/configs_service_menu');
require('views/main/host/log_metrics');
require('views/main/host/metrics');
require('views/main/host/stack_versions_view');
require('views/main/host/add_view');
require('views/main/host/host_alerts_view');
require('views/main/host/metrics/cpu');
require('views/main/host/metrics/disk');
require('views/main/host/metrics/load');
require('views/main/host/metrics/memory');
require('views/main/host/metrics/network');
require('views/main/host/metrics/processes');
require('views/main/host/addHost/step4_view');
require('views/main/admin');
require('views/main/admin/service_auto_start');
require('views/main/admin/highAvailability/nameNode/wizard_view');
require('views/main/admin/highAvailability/progress_view');
require('views/main/admin/highAvailability/nameNode/rollback_view');
require('views/main/admin/highAvailability/nameNode/step1_view');
require('views/main/admin/highAvailability/nameNode/step2_view');
require('views/main/admin/highAvailability/nameNode/step3_view');
require('views/main/admin/highAvailability/nameNode/step4_view');
require('views/main/admin/highAvailability/nameNode/step5_view');
require('views/main/admin/highAvailability/nameNode/step6_view');
require('views/main/admin/highAvailability/nameNode/step7_view');
require('views/main/admin/highAvailability/nameNode/step8_view');
require('views/main/admin/highAvailability/nameNode/step9_view');
require('views/main/admin/highAvailability/nameNode/rollbackHA/step1_view');
require('views/main/admin/highAvailability/nameNode/rollbackHA/step2_view');
require('views/main/admin/highAvailability/nameNode/rollbackHA/step3_view');
require('views/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard_view');
require('views/main/admin/highAvailability/journalNode/wizard_view');
require('views/main/admin/highAvailability/journalNode/progress_view');
require('views/main/admin/highAvailability/journalNode/step1_view');
require('views/main/admin/highAvailability/journalNode/step2_view');
require('views/main/admin/highAvailability/journalNode/step3_view');
require('views/main/admin/highAvailability/journalNode/step4_view');
require('views/main/admin/highAvailability/journalNode/step5_view');
require('views/main/admin/highAvailability/journalNode/step6_view');
require('views/main/admin/highAvailability/journalNode/step7_view');
require('views/main/admin/highAvailability/resourceManager/wizard_view');
require('views/main/admin/highAvailability/resourceManager/step1_view');
require('views/main/admin/highAvailability/resourceManager/step2_view');
require('views/main/admin/highAvailability/resourceManager/step3_view');
require('views/main/admin/highAvailability/resourceManager/step4_view');
require('views/main/admin/highAvailability/hawq/addStandby/wizard_view');
require('views/main/admin/highAvailability/hawq/addStandby/step1_view');
require('views/main/admin/highAvailability/hawq/addStandby/step2_view');
require('views/main/admin/highAvailability/hawq/addStandby/step3_view');
require('views/main/admin/highAvailability/hawq/addStandby/step4_view');
require('views/main/admin/highAvailability/hawq/removeStandby/wizard_view');
require('views/main/admin/highAvailability/hawq/removeStandby/step1_view');
require('views/main/admin/highAvailability/hawq/removeStandby/step2_view');
require('views/main/admin/highAvailability/hawq/removeStandby/step3_view');
require('views/main/admin/highAvailability/hawq/activateStandby/wizard_view');
require('views/main/admin/highAvailability/hawq/activateStandby/step1_view');
require('views/main/admin/highAvailability/hawq/activateStandby/step2_view');
require('views/main/admin/highAvailability/hawq/activateStandby/step3_view');
require('views/main/admin/highAvailability/rangerAdmin/wizard_view');
require('views/main/admin/highAvailability/rangerAdmin/step1_view');
require('views/main/admin/highAvailability/rangerAdmin/step2_view');
require('views/main/admin/highAvailability/rangerAdmin/step3_view');
require('views/main/admin/highAvailability/rangerAdmin/step4_view');
require('views/main/admin/federation/wizard_view');
require('views/main/admin/federation/step1_view');
require('views/main/admin/federation/step2_view');
require('views/main/admin/federation/step3_view');
require('views/main/admin/federation/step4_view');
require('views/main/admin/serviceAccounts_view');
require('views/main/admin/stack_upgrade/upgrade_wizard_view');
require('views/main/admin/stack_upgrade/upgrade_version_box_view');
require('views/main/admin/stack_upgrade/upgrade_version_column_view');
require('views/main/admin/stack_upgrade/upgrade_group_view');
require('views/main/admin/stack_upgrade/upgrade_task_view');
require('views/main/admin/stack_upgrade/upgrade_history_view');
require('views/main/admin/stack_upgrade/upgrade_history_details_view');
require('views/main/admin/stack_upgrade/services_view');
require('views/main/admin/stack_upgrade/versions_view');
require('views/main/admin/stack_upgrade/menu_view');
require('views/main/admin/stack_upgrade/failed_hosts_modal_view');
require('views/main/admin/stack_and_upgrade_view');

require('views/main/admin/kerberos');
require('views/main/admin/kerberos/disable_view');
require('views/main/admin/kerberos/wizard_view');
require('views/main/admin/kerberos/progress_view');
require('views/main/admin/kerberos/step1_view');
require('views/main/admin/kerberos/step2_view');
require('views/main/admin/kerberos/step3_view');
require('views/main/admin/kerberos/step4_view');
require('views/main/admin/kerberos/step5_view');
require('views/main/admin/kerberos/step6_view');
require('views/main/admin/kerberos/step7_view');
require('views/main/admin/kerberos/step8_view');

require('views/main/dashboard');
require('views/main/dashboard/cluster_metrics/cpu');
require('views/main/dashboard/cluster_metrics/load');
require('views/main/dashboard/cluster_metrics/memory');
require('views/main/dashboard/cluster_metrics/network');

require('views/main/dashboard/widget');
require('views/main/dashboard/widgets');
require('views/main/dashboard/widgets/text_widget');
require('views/main/dashboard/widgets/text_widget_single_threshold');
require('views/main/dashboard/widgets/uptime_text_widget');
require('views/main/dashboard/widgets/links_widget');
require('views/main/dashboard/widgets/pie_chart_widget');
require('views/main/dashboard/widgets/cluster_metrics_widget');
require('views/main/dashboard/widgets/namenode_heap');
require('views/main/dashboard/widgets/namenode_cpu');
require('views/main/dashboard/widgets/hdfs_capacity');
require('views/main/dashboard/widgets/datanode_live');
require('views/main/dashboard/widgets/hawqsegment_live');
require('views/main/dashboard/widgets/pxf_live');
require('views/main/dashboard/widgets/namenode_rpc');
require('views/main/dashboard/widgets/metrics_memory');
require('views/main/dashboard/widgets/metrics_network');
require('views/main/dashboard/widgets/metrics_cpu');
require('views/main/dashboard/widgets/metrics_load');
require('views/main/dashboard/widgets/namenode_uptime');
require('views/main/dashboard/widgets/hdfs_links');
require('views/main/dashboard/widgets/yarn_links');
require('views/main/dashboard/widgets/hbase_links');
require('views/main/dashboard/widgets/hbase_master_heap');
require('views/main/dashboard/widgets/hbase_average_load');
require('views/main/dashboard/widgets/hbase_regions_in_transition');
require('views/main/dashboard/widgets/hbase_master_uptime');
require('views/main/dashboard/widgets/resource_manager_heap');
require('views/main/dashboard/widgets/resource_manager_uptime');
require('views/main/dashboard/widgets/node_managers_live');
require('views/main/dashboard/widgets/yarn_memory');
require('views/main/dashboard/widgets/yarn_containers');
require('views/main/dashboard/widgets/supervisor_live');
require('views/main/dashboard/widgets/flume_agent_live');
require('views/main/dashboard/config_history_view');
require('views/main/dashboard/config_history_search_box');

require('views/main/service');
require('views/main/service/service');
require('views/main/service/services/hdfs');
require('views/main/service/services/onefs');
require('views/main/service/services/yarn');
require('views/main/service/services/mapreduce2');
require('views/main/service/services/hbase');
require('views/main/service/services/hive');
require('views/main/service/services/zookeeper');
require('views/main/service/services/flume');
require('views/main/service/services/storm');
require('views/main/service/services/ranger');
require('views/main/service/all_services_actions');
require('views/main/service/menu');
require('views/main/service/item');
require('views/main/service/reconfigure');
require('views/main/service/info/components_list_view');
require('views/main/service/info/menu');
require('views/main/service/info/summary');
require('views/main/service/info/summary/hdfs/common_widgets');
require('views/main/service/info/summary/hdfs/slaves');
require('views/main/service/info/summary/hdfs/widgets');
require('views/main/service/info/configs');
require('views/main/service/info/metric_graphs_view');
require('views/main/service/info/metrics/ambari_metrics/master_average_load');
require('views/main/service/info/metrics/ambari_metrics/regionserver_base');
require('views/main/service/info/metrics/ambari_metrics/regionserver_store_files');
require('views/main/service/info/metrics/ambari_metrics/regionserver_regions');
require('views/main/service/info/metrics/ambari_metrics/regionserver_requests');
require('views/main/service/info/metrics/ambari_metrics/regionserver_block_cache_hit_percent');
require('views/main/service/info/metrics/ambari_metrics/regionserver_compaction_queue_size');
require('views/main/service/info/metrics/flume/channel_sum');
require('views/main/service/info/metrics/flume/channel_size_mma');
require('views/main/service/info/metrics/flume/flume_agent_metrics_section');
require('views/main/service/info/metrics/flume/flume_incoming_sum');
require('views/main/service/info/metrics/flume/flume_incoming_mma');
require('views/main/service/info/metrics/flume/flume_outgoing_sum');
require('views/main/service/info/metrics/flume/flume_outgoing_mma');
require('views/main/service/info/metrics/flume/channel_fill_pct');
require('views/main/service/info/metrics/flume/channel_size');
require('views/main/service/info/metrics/flume/sink_connection_failed');
require('views/main/service/info/metrics/flume/sink_drain_success');
require('views/main/service/info/metrics/flume/source_accepted');
require('views/main/service/info/metrics/flume/gc');
require('views/main/service/info/metrics/flume/jvm_heap');
require('views/main/service/info/metrics/flume/jvm_threads_runnable');
require('views/main/service/info/metrics/flume/cpu_user');
require('views/main/service/info/metrics/flume/flume_metric_graph');
require('views/main/service/info/metrics/flume/flume_metric_graphs');

require('views/main/service/add_view');
require('views/main/service/reassign_view');
require('views/main/service/reassign/step1_view');
require('views/main/service/reassign/step2_view');
require('views/main/service/reassign/step3_view');
require('views/main/service/reassign/step4_view');
require('views/main/service/reassign/step5_view');
require('views/main/service/reassign/step6_view');
require('views/main/service/manage_config_groups_view');
require('views/main/charts/menu');
require('views/main/charts/heatmap');
require('views/main/charts/heatmap/heatmap_rack');
require('views/main/charts/heatmap/heatmap_host');
require('views/main/charts/heatmap/heatmap_host_detail');
require('views/main/service/info/heatmap_view');
require('views/main/service/info/metrics_view');

require('views/main/service/widgets/create/wizard_view');
require('views/main/service/widgets/create/step1_view');
require('views/main/service/widgets/create/step2_view');
require('views/main/service/widgets/create/step3_view');
require('views/main/service/widgets/edit_view');
require('views/main/service/widgets/create/expression_view');

require('views/main/views_view');

require('views/installer');
require('views/wizard/step0_view');
require('views/wizard/step1_view');
require('views/wizard/step2_view');
require('views/wizard/step3_view');
require('views/wizard/step3/hostLogPopupBody_view');
require('views/wizard/step3/hostWarningPopupBody_view');
require('views/wizard/step3/hostWarningPopupFooter_view');
require('views/wizard/step4_view');
require('views/wizard/step5_view');
require('views/wizard/step6_view');
require('views/wizard/step7_view');
require('views/wizard/step7/assign_master_view');
require('views/wizard/step7/credentials_tab_view');
require('views/wizard/step7/databases_tab_view');
require('views/wizard/step7/accounts_tab_view');
require('views/wizard/step7/directories_tab_view');
require('views/main/service/reassign/step7_view');
require('views/wizard/step8_view');
require('views/wizard/step9_view');
require('views/wizard/step9/hostLogPopupBody_view');
require('views/wizard/step10_view');
require('views/loading');

require('views/experimental');

});

require.register("views/application", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ApplicationView = Em.View.extend({
  templateName: require('templates/application'),

  views: function () {
    return App.router.get('loggedIn') ? App.router.get('mainViewsController.visibleAmbariViews') : [];
  }.property('App.router.mainViewsController.visibleAmbariViews.[]', 'App.router.loggedIn'),

  didInsertElement: function didInsertElement() {
    // on 'Enter' pressed, trigger modal window primary button if primary button is enabled(green)
    // on 'Esc' pressed, close the modal
    $(document).keydown(function (event) {
      if (event.which === 13 || event.keyCode === 13) {
        $('.modal:last').trigger('enter-key-pressed');
      }
      return true;
    });
    $(document).keyup(function (event) {
      if (event.which === 27 || event.keyCode === 27) {
        $('.modal:last').trigger('escape-key-pressed');
      }
      return true;
    });
  },

  /**
   * Navigation Bar should be initialized after cluster data is loaded
   */
  initNavigationBar: function () {
    if (App.get('router.mainController.isClusterDataLoaded')) {
      $('body').on('DOMNodeInserted', '.navigation-bar', function () {
        $('.navigation-bar').navigationBar({
          fitHeight: true,
          collapseNavBarClass: 'icon-double-angle-left',
          expandNavBarClass: 'icon-double-angle-right'
        });
        $('body').off('DOMNodeInserted', '.navigation-bar');
      });
    }
  }.observes('App.router.mainController.isClusterDataLoaded')

});

});

require.register("views/common/ajax_default_error_popup_body", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AjaxDefaultErrorPopupBodyView = Em.View.extend({

  classNames: ['api-error'],
  templateName: require('templates/utils/ajax'),

  /**
   * HTTP request URL
   * @type {string}
   */
  url: '',

  /**
   * HTTP request type
   * @type {string}
   */
  type: '',

  /**
   * HTTP response status code
   * @type {number}
   */
  status: 0,

  /**
   * Received error message
   * @type {string}
   */
  message: '',

  /**
   * Status code string
   * @type {string}
   */
  statusCode: Em.computed.i18nFormat('utils.ajax.defaultErrorPopupBody.statusCode', 'status'),

  /**
   * Indicates if error message should be displayed
   * @type {boolean}
   */
  showMessage: Em.computed.bool('message'),

  /**
   * HTTP response error description
   * @type {string}
   */
  api: Em.computed.i18nFormat('utils.ajax.defaultErrorPopupBody.message', 'type', 'url')

});

});

require.register("views/common/assign_master_components_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AssignMasterComponentsView = Em.View.extend({

  templateName: require('templates/common/assign_master_components'),

  /**
   * Title to be shown on the page
   * @type {String}
   */
  title: '',

  showTitle: true,

  /**
   * Alert message to be shown on the page
   * @type {String}
   */
  alertMessage: '',

  /**
   * If install more than 25 hosts, should use App.InputHostView for hosts selection
   * Otherwise - App.SelectHostView
   * @type {bool}
   */
  shouldUseInputs: Em.computed.gt('controller.hosts.length', 25),

  isWizardStep: true,

  isBackButtonVisible: true,

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  willDestroyElement: function willDestroyElement() {
    this.get('controller').clearStepOnExit();
  }
});

App.InputHostView = Em.TextField.extend(App.SelectHost, {

  attributeBindings: ['disabled'],
  /**
   * Saved typeahead component
   * @type {$}
   */
  typeahead: null,

  classNames: ['form-control'],

  /**
   * When <code>value</code> (host_info) is changed this method is triggered
   * If new hostname is valid, this host is assigned to master component
   * @method changeHandler
   */
  changeHandler: function () {
    if (!this.shouldChangeHandlerBeCalled()) return;
    var host = this.get('controller.hosts').findProperty('host_name', this.get('value'));
    if (Em.isNone(host)) {
      this.get('controller').updateIsHostNameValidFlag(this.get("component.component_name"), this.get("component.serviceComponentId"), false);
      this.get('controller').updateIsSubmitDisabled();
      return;
    }
    this.get('controller').assignHostToMaster(this.get("component.component_name"), host.get('host_name'), this.get("component.serviceComponentId"));
    this.tryTriggerRebalanceForMultipleComponents();
    this.get('controller').updateIsSubmitDisabled();
  }.observes('controller.hostNameCheckTrigger'),

  didInsertElement: function didInsertElement() {
    this.initContent();
    var value = this.get('content').findProperty('host_name', this.get('component.selectedHost')).get('host_name');
    this.set("value", value);
    var content = this.get('content').mapProperty('host_info'),
        self = this,
        updater = function updater(item) {
      return self.get('content').findProperty('host_info', item).get('host_name');
    },
        typeahead = this.$().typeahead({ items: 10, source: content, updater: updater, minLength: 0 });
    typeahead.on('blur', function () {
      self.change();
    }).on('keyup', function (e) {
      self.set('value', $(e.currentTarget).val());
      self.change();
    });
    this.set('typeahead', typeahead);
    App.popover($("[rel=popover]"), { 'placement': 'right', 'trigger': 'hover' });
  },

  /**
   * Extract hosts from controller,
   * filter out available to selection and
   * push them into Em.Select content
   * @method initContent
   */
  initContent: function initContent() {
    this._super();
    this.updateTypeaheadData(this.get('content').mapProperty('host_info'));
  },

  /**
   * Update <code>source</code> property of <code>typeahead</code> with a new list of hosts
   * @param {string[]} hosts
   * @method updateTypeaheadData
   */
  updateTypeaheadData: function updateTypeaheadData(hosts) {
    if (this.get('typeahead')) {
      this.get('typeahead').data('typeahead').source = hosts;
    }
  }

});

App.SelectHostView = App.DropdownView.extend(App.SelectHost, {

  qaAttr: 'select-host-for-component',

  didInsertElement: function didInsertElement() {
    this.initContent();
    this.set("value", this.get("component.selectedHost"));
    App.popover($("[rel=popover]"), { 'placement': 'right', 'trigger': 'hover' });
  },

  /**
   * Handler for selected value change
   * @method change
   */
  changeHandler: function () {
    if (!this.shouldChangeHandlerBeCalled()) return;
    this.get('controller').assignHostToMaster(this.get("component.component_name"), this.get("value"), this.get("component.serviceComponentId"));
    this.tryTriggerRebalanceForMultipleComponents();
  }.observes('controller.hostNameCheckTrigger'),

  /**
   * On change DOM event handler
   * @method change
   */
  change: function change() {
    this._super();
    this.initContent();
  }

});

App.AddControlView = Em.View.extend({

  /**
   * DOM node class attribute
   * @type {string}
   */
  uniqueId: Em.computed.format('{0}-add', 'componentName'),

  /**
   * Current component name
   * @type {string}
   */
  componentName: null,

  tagName: "div",

  classNames: ["label", 'extra-component'],

  classNameBindings: ['uniqueId'],

  'data-qa': 'add-master',

  template: Em.Handlebars.compile('+'),

  /**
   * Onclick handler
   * Add selected component
   * @method click
   */
  click: function click() {
    this.get('controller').addComponent(this.get('componentName'));
  }
});

App.RemoveControlView = Em.View.extend({
  /**
   * DOM node class attribute
   * @type {string}
   */
  uniqueId: Em.computed.format('{0}-{1}-remove', 'componentName', 'serviceComponentId'),

  classNameBindings: ['uniqueId'],

  /**
   * Index for multiple component
   * @type {number}
   */
  serviceComponentId: null,

  /**
   * Current component name
   * @type {string}
   */
  componentName: null,

  tagName: "div",

  'data-qa': 'remove-master',

  classNames: ["label", 'extra-component'],

  template: Em.Handlebars.compile('-'),

  /**
   * Onclick handler
   * Remove current component
   * @method click
   */
  click: function click() {
    this.get('controller').removeComponent(this.get('componentName'), this.get("serviceComponentId"));
  }
});

});

require.register("views/common/breadcrumbs_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @param {string} labelBindingPath
 * @returns {string}
 * @private
 */
function _getLabelPathWithoutApp(labelBindingPath) {
  return labelBindingPath.startsWith('App.') ? labelBindingPath.replace('App.', '') : labelBindingPath;
}

/**
 * @param {string} stateName
 * @returns {string}
 * @private
 */
function _formatLabel(stateName) {
  return stateName.capitalize().replace(/([a-z])([A-Z])/g, '$1 $2');
}

/**
 * Don't create instances directly
 * Only <code>App.BreadcrumbsView</code>-instance will create them
 *
 * @type {Em.Object}
 */
App.BreadcrumbItem = Em.Object.extend({

  /**
   * String shown as breadcrumb
   *
   * @type {string}
   */
  label: '',

  /**
   * Path to variable that will be used as breadcrumb
   * If <code>labelBindingPath</code> is <code>'App.router.somePath'</code>, its value will be used
   *
   * @type {string}
   */
  labelBindingPath: '',

  /**
   * View shown as breadcrumb.
   * If provied, <code>itemView</code> supersedes <code>label</code> and <code>labelBindingPath</code>.
   *
   * @type {object}
   */
  itemView: null,

  /**
   * Determines if breadcrumb is disabled
   *
   * @type {boolean}
   */
  disabled: false,

  /**
   * Check if current breadcrumb is last
   *
   * @type {boolean}
   */
  isLast: false,

  /**
   * Invoke this action when click on breadcrumb item
   * If provided, <code>action</code> supersedes <code>route</code>.
   *
   * @type {Function}
   */
  action: null,

  /**
   * Move user to this route when click on breadcrumb item (don't add prefix <code>main</code>)
   * This is used if an action is not defined.
   *
   * @type {string}
   */
  route: '',

  /**
   * Hook executed before transition
   * may be overridden when needed
   *
   * @type {Function}
   */
  beforeTransition: Em.K,

  /**
   * Hook executed after transition
   * may be overridden when needed
   *
   * @type {Function}
   */
  afterTransition: Em.K,

  /**
   * Label shown on the page
   * Result of <code>createLabel</code> execution
   *
   * @type {string}
   */
  formattedLabel: '',

  /**
   * Hook for label formatting
   * It's executed after <code>label</code> or <code>labelBindingPath</code> is processed
   *
   * @param {string} label
   * @returns {string}
   */
  labelPostFormat: function labelPostFormat(label) {
    return label;
  },

  transition: function transition() {
    var action = this.get('action');
    if (action) {
      return action();
    } else {
      return App.router.route('main/' + this.get('route'));
    }
  },

  /**
   * Generate <code>formattedLabel</code> shown on the page
   *
   * @method createLabel
   */
  createLabel: function createLabel() {
    var label = this.get('label');
    var labelBindingPath = this.get('labelBindingPath');
    var formattedLabel = void 0;

    if (labelBindingPath) {
      formattedLabel = Ember.Handlebars.Utils.escapeExpression(App.get(_getLabelPathWithoutApp(labelBindingPath)));
    } else {
      formattedLabel = label;
    }

    this.set('formattedLabel', this.labelPostFormat(formattedLabel));
  },


  /**
   * If <code>labelBindingPath</code> is provided, <code>createLabel</code> should observe value in path <code>${labelBindingPath}</code>
   *
   * @returns {*}
   */
  init: function init() {
    var labelBindingPath = this.get('labelBindingPath');
    if (labelBindingPath) {
      labelBindingPath = 'App.' + _getLabelPathWithoutApp(labelBindingPath);
      this.addObserver(labelBindingPath, this, 'createLabel');
    }
    this.createLabel();
    return this._super.apply(this, arguments);
  }
});

/**
 * Usage:
 * <code>{{view App.BreadcrumbsView}}</code>
 *
 * @type {Em.View}
 */
App.BreadcrumbsView = Em.View.extend({

  templateName: require('templates/common/breadcrumbs'),

  /**
   * List of the breadcrumbs
   * It's updated if <code>App.router.currentState</code> is changed. This happens when user is moved from one page to another
   *
   * @type {BreadcrumbItem[]}
   */
  items: function () {
    var currentState = App.get('router.currentState');
    var items = [];
    var wizardStepRegex = /^step[0-9]?/;
    while (currentState) {
      if (currentState.breadcrumbs !== undefined) {
        // breadcrumbs should be defined and be not null or any other falsie-value
        if (currentState.breadcrumbs) {
          var _currentState$breadcr = currentState.breadcrumbs,
              label = _currentState$breadcr.label,
              labelBindingPath = _currentState$breadcr.labelBindingPath,
              route = _currentState$breadcr.route,
              disabled = _currentState$breadcr.disabled;
          // generate label if it isn't provided

          if (!label && !labelBindingPath) {
            currentState.breadcrumbs.label = _formatLabel(currentState.name);
          }
          // generate route if it isn't provided and breadcrumb is not disabled
          if (!route && !disabled) {
            currentState.breadcrumbs.route = currentState.absoluteRoute(App.router).replace('/main/', '');
          }
          items.pushObject(currentState.breadcrumbs);
        }
      } else {
        // generate breadcrumb if it is not defined
        // breadcrumbs of wizard step such as "Step #" should be ignored
        if (currentState.name && !['root', 'index'].contains(currentState.name) && !wizardStepRegex.test(currentState.name)) {
          items.pushObject({ label: _formatLabel(currentState.name) });
        }
      }
      currentState = currentState.get('parentState');
    }
    items.reverse();
    items.slice(1).forEach(function (item) {
      return item.label = Ember.Handlebars.Utils.escapeExpression(item.label);
    });
    items = items.map(function (item) {
      return App.BreadcrumbItem.extend(item).create();
    });
    if (items.length) {
      items.get('lastObject').setProperties({
        disabled: true,
        isLast: true
      });
    }
    return items;
  }.property('App.router.currentState'),

  /**
   * Move user to the route described in the breadcrumb item
   * <code>beforeTransition</code> hook is executed
   * <code>afterTransition</code> hook is executed
   *
   * @param {{context: App.BreadcrumbItem}} event
   * @returns {*}
   */
  moveTo: function moveTo(event) {
    var item = event.context;
    if (!item || item.get('disabled')) {
      return;
    }
    Em.tryInvoke(item, 'beforeTransition');
    Em.tryInvoke(item, 'transition');
    Em.tryInvoke(item, 'afterTransition');
  }
});

});

require.register("views/common/buttons/button_progress_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ButtonProgressView = Em.View.extend(Em.TargetActionSupport, {
  layoutName: require('templates/common/button_progress'),
  isDisabled: Em.computed.or('disabled', 'isInProgress'),
  /**
   * Target to perform `onClick` function default to App.router
   * @type {Em.Object|Em.View|Em.Controller}
   */
  target: null,
  /**
   * Property determines progress state
   * @type {Boolean}
   */
  isInProgress: null,
  /**
   * on click handler
   * @type {Function}
   */
  action: null,
  /**
   * When true spinner appears to right side, when false - to left
   * @type {Boolean}
   */
  doSpinRight: true,

  targetObject: function () {
    var target = this.get('target'),
        splitted;
    if (!target) {
      return this.get('controller.target');
    } else if (typeof target === 'string') {
      splitted = target.split('.');
      if (splitted[0] === 'view') {
        splitted = ['parentView'].concat(splitted.slice(1));
      }
      return Em.get(this, splitted.join('.'));
    } else {
      return target;
    }
  }.property('target'),

  handleClick: function handleClick() {
    if (this.get('isDisabled')) {
      return;
    }
    var target = this.get('targetObject');
    var targetMethod = this.get('action');
    if (target.isState && typeof target.send === 'function') {
      target.send(targetMethod);
    } else if (targetMethod && typeof targetMethod === 'function') {
      targetMethod.apply(target);
    } else if (typeof targetMethod === 'string' && typeof Em.get(target, targetMethod) === 'function') {
      Em.get(target, targetMethod).call(target);
    } else {
      Ember.Logger.error('Cannot invoke action %s on target %s', targetMethod, target);
    }
  }
});

});

require.register("views/common/buttons/wizard_buttons", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardNextButton = App.ButtonProgressView.extend({
  classNames: ['pull-right'],
  isInProgressBinding: 'App.router.nextBtnClickInProgress',
  doSpinRight: false,
  buttonClassNames: 'btn-success pull-right',
  template: Em.Handlebars.compile('{{t common.next}} &rarr;'),
  disabledBinding: 'controller.isSubmitDisabled'
});

App.WizardBackButton = App.ButtonProgressView.extend({
  classNames: ['pull-left'],
  isInProgressBinding: 'App.router.backBtnClickInProgress',
  template: Em.Handlebars.compile('&larr; {{t common.back}}'),
  buttonClassNames: 'btn-default pull-left',
  disabledBinding: 'controller.isBackButtonDisabled'
});

});

require.register("views/common/chart", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartView = Em.View.extend({
  dateFormat: 'dd/mm/yy',
  timeFormat: 'h:m',
  w: 900,
  p: 30, // axis padding
  shift: 30,
  ticksCount: 10,
  pointsLimit: 300,
  areaHeight: 30, // px
  axis: false,
  x: false,
  y: false,

  init: function init() {
    this._super();
    var renderer = this;

    this.x = d3.time.scale().domain([renderer.getMinDate({}), renderer.getMaxDate({})]).range([0, this.get('w')]);
    this.y = d3.scale.linear().domain([0, 50]).range([this.get('h'), 0]);
    this.axis = d3.svg.axis().orient("top").scale(this.x).ticks(this.get('ticksCount'));
  },

  h: function () {
    return this.get('p') + this.get('nodeAttributes').length * this.get('areaHeight'); //default: 160
  }.property('nodeAttributes', 'p'),

  activeH: function () {
    return this.get('nodeAttributes').length * this.get('areaHeight'); // default 160;
  }.property('h'),

  ruleHeight: function () {
    return this.get('nodeAttributes').length * this.get('areaHeight');
  }.property('nodeAttributes'),

  updateY: function () {
    this.y = d3.scale.linear().domain([0, 50]).range([this.get('h'), 0]);
  }.observes('h'),

  getMinDate: function getMinDate(data) {
    if (data.length) return new Date(Date.parse(data[0]['date']));

    return new Date();
  },

  getMaxDate: function getMaxDate(data) {
    if (data.length) return new Date(Date.parse(data[data.length - 1]['date']));

    return new Date();
  },

  area: function area() {
    var renderer = this;
    var area = d3.svg.area().x(function (d) {
      return renderer.x(renderer.getDate(d));
    });

    area.y1(function (d) {
      return renderer.get('h') - (renderer.get('h') - renderer.y(d[$(this).attr("getter")])) / renderer.get('koef');
    });

    area.y0(function (d) {
      return renderer.get('h');
    });

    return area;
  },

  line: function line() {
    var renderer = this;
    var area = d3.svg.line().x(function (d) {
      return renderer.x(renderer.getDate(d));
    }).interpolate("basis");

    area.y(function (d) {
      return renderer.get('h');
    });

    return area;
  },

  /**
   * @todo: calculate this
   * coefficient of compression
   * @param shift
   * @return {Number}
   */

  koef: function () {
    // max value divide on area height;
    return 2 * (this.get('nodeAttributes').length + 1);
  }.property('h'),

  getDate: function getDate(d) {
    return new Date(Date.parse(d.date));
  },

  dateTimeToDateObject: function dateTimeToDateObject(string) {
    var ren = this;
    return new Date($.datepicker.parseDateTime(ren.dateFormat, ren.timeFormat, string));
  },

  getDefaultShift: function getDefaultShift() {
    return -1 * this.get('areaHeight') * (this.get('nodeAttributes').length - 1);
  },

  percentScaleXDefaultTranslate: function percentScaleXDefaultTranslate() {
    return this.w + 3;
  },

  clearPlot: function clearPlot() {
    d3.select(this.get('chartContainerSelector')).selectAll("*").remove();
  },

  drawPlot: function drawPlot() {
    this.clearPlot();

    var renderer = this;
    this.x.domain([renderer.getMinDate({}), renderer.getMaxDate({})]);

    var rule = $('<div></div>').addClass("rule").css('height', renderer.get('ruleHeight')).mouseenter(function () {
      $(this).hide();
    });
    $(this.get('chartContainerSelector')).prepend(rule);

    var vis = d3.select(this.get('chartContainerSelector')).append("svg:svg").attr("width", renderer.get('w') + 5).attr("height", renderer.get('h')).attr("rendererId", this.get('elementId')).on("mousemove", function () {

      var area = d3.select(this).select("path.line");
      var d = area.data()[0];
      var x = d3.mouse(this)[0];

      var renderer = Em.View.views[d3.select(this).attr('rendererId')];
      var container = $(this).parent();
      var scale = renderer.x;

      // first move rule
      var rule = $(container).children("div.rule");
      rule.css("left", 168 + x + "px"); // 168 - left container margin
      rule.show();

      x = x + 5; // some correction
      var selectedDate = scale.invert(x);

      // search date between this coordinates
      var prevVal = false;
      var nextVal = d[0];

      $.each(d, function (i, point) {
        if (renderer.getDate(point).getTime() <= selectedDate.getTime()) {
          prevVal = nextVal;
          nextVal = point;
        }
      });

      var len1 = Math.abs(x - scale(renderer.getDate(prevVal)));
      var len2 = Math.abs(x - scale(renderer.getDate(nextVal)));

      var clearing = 5;
      var pointToShow = false;
      // if the first point if closer
      if (len1 < len2 && len1 <= clearing) {
        pointToShow = prevVal;
      } else if (len2 <= clearing) {
        // the second point is closer
        pointToShow = nextVal;
      }

      $.each(renderer.get('nodeAttributes'), function (i, v) {
        var value = !pointToShow ? "" : pointToShow[v] + "%";
        $(rule).children("div." + v).html(value);
      });
    });

    vis.append("svg:g").attr("class", "axis").attr("transform", "translate(0," + this.get('p') + ")").call(renderer.axis);

    $.each(this.get('nodeAttributes'), function (i, v) {
      var element = $('<div></div>').addClass(v).addClass("stateValue").html("");
      rule.append(element);
    });

    var shift = this.getDefaultShift();
    vis.append("svg:path").attr("class", "horizontal-line").data([{}]).attr("transform", "translate(0," + (shift - this.get('areaHeight')) + ")").attr("d", renderer.line()).style("stroke", "#000");

    $.each(this.get('nodeAttributes'), function (i, v) {
      vis.append("svg:path").data([{}]).attr("class", "line").attr("getter", v).attr("transform", "translate(0, " + shift + ")").attr("d", renderer.area()).style("fill", function () {
        return "#31a354";
      });

      vis.append("svg:path").attr("class", "horizontal-line").data([{}]).attr("transform", "translate(0," + shift + ")").attr("d", renderer.line()).style("stroke", "#000");

      shift += renderer.get('areaHeight');
    });
  },

  getData: function getData(containerId) {
    return d3.select(containerId + " path.line").data()[0];
  },

  drawChart: function drawChart() {
    var containerSel = this.get('chartContainerSelector');
    var data = this.get('data');

    while (data.length > this.get('pointsLimit')) {
      data.shift();
    }

    var renderer = this;
    var minDate = this.getMinDate(data);
    var maxDate = this.getMaxDate(data);

    this.x.domain([minDate, maxDate]);

    var ticks = data.length > 10 ? 10 : data.length;
    this.axis.scale(renderer.x).ticks(ticks);

    //        remove dots axis
    $(containerSel + " svg g.axis g").remove();
    d3.select(containerSel + " svg g.axis").call(this.axis);

    $.each(this.get('nodeAttributes'), function (i, v) {
      d3.select(containerSel + " path.line[getter='" + v + "']").data([data]).transition().attr("d", renderer.area());
    });

    // lines between charts
    $(containerSel + " path.horizontal-line").each(function (i, path) {
      d3.select(path).data([[{ date: minDate }, { date: maxDate }]]).attr("d", renderer.line());
    });
  }
});

});

require.register("views/common/chart/linear", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartLinearView = Em.View.extend({
  w: 90,
  h: 90,
  margin: 2,

  classNames: ['linear'],

  init: function init() {
    this._super();

    this.x = d3.scale.linear().domain([0, this.get('data').length]).range([0 + this.get('margin'), this.get('w') - this.get('margin')]);
    this.y = d3.scale.linear().domain([0, d3.max(this.get('data'))]).range([0 + this.get('margin'), this.get('h') - this.get('margin')]);
    this.line = d3.svg.line().x(function (d, i) {
      return this.x(i);
    }).y(function (d) {
      return -1 * this.y(d);
    });
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.appendSvg();
  },

  selector: Em.computed.format('#{0}', 'elementId'),

  appendSvg: function appendSvg() {
    var thisChart = this;

    this.set('svg', d3.select(this.get('selector')).append("svg:svg").attr("width", thisChart.get('w')).attr("height", thisChart.get('h')));

    this.set('g', thisChart.get('svg').append("svg:g").attr("transform", "translate(0, " + thisChart.get('h') + ")"));
    this.get('g').append("svg:path").attr("d", thisChart.line(thisChart.get('data')));

    // axis
    this.get('g').append("svg:line").attr("x1", thisChart.x(0)).attr("y1", -1 * thisChart.y(0)).attr("x2", thisChart.x(this.get('data').length)).attr("y2", -1 * thisChart.y(0));

    this.get('g').append("svg:line").attr("x1", thisChart.x(0)).attr("y1", -1 * thisChart.y(0)).attr("x2", thisChart.x(0)).attr("y2", -1 * thisChart.y(d3.max(thisChart.get('data'))));
  }
});

});

require.register("views/common/chart/linear_time", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var string_utils = require('utils/string_utils');
var dateUtils = require('utils/date/date');

/**
 * @class
 *
 * This is a view which GETs data from a URL and shows it as a time based line
 * graph. Time is shown on the X axis with data series shown on Y axis. It
 * optionally also has the ability to auto refresh itself over a given time
 * interval.
 *
 * This is an abstract class which is meant to be extended.
 *
 * Extending classes should override the following:
 * <ul>
 * <li>url - from where the data can be retrieved
 * <li>title - Title to be displayed when showing the chart
 * <li>id - which uniquely identifies this chart in any page
 * <li>seriesTemplate - template used by getData method to process server data
 * </ul>
 *
 * Extending classes could optionally override the following:
 * <ul>
 * <li>#getData(jsonData) - function to map server data into series format
 * ready for export to graph and JSON formats
 * <li>#colorForSeries(series) - function to get custom colors per series
 * </ul>
 *
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartLinearTimeView = Ember.View.extend(App.ExportMetricsMixin, {
  templateName: require('templates/main/charts/linear_time'),

  /**
   * The URL from which data can be retrieved.
   *
   * This property must be provided for the graph to show properly.
   *
   * @type String
   * @default null
   */
  url: null,

  /**
   * A unique ID for this chart.
   *
   * @type String
   * @default null
   */
  id: null,

  /**
   * Title to be shown under the chart.
   *
   * @type String
   * @default null
   */
  title: null,

  /**
   * @private
   *
   * @type Rickshaw.Graph
   * @default null
   */
  _graph: null,

  /**
   * Array of classnames for each series (in widget)
   * @type Rickshaw.Graph
   */
  _popupGraph: null,

  /**
   * Array of classnames for each series
   * @type Array
   */
  _seriesProperties: null,

  /**
   * Array of classnames for each series (in widget)
   * @type Array
   */
  _seriesPropertiesWidget: null,

  /**
   * Renderer type
   * See <code>Rickshaw.Graph.Renderer</code> for more info
   * @type String
   */
  renderer: 'area',

  /**
   * Suffix used in DOM-elements selectors
   * @type String
   */
  popupSuffix: '-popup',

  /**
   * Is popup for current graph open
   * @type Boolean
   */
  isPopup: false,

  /**
   * Is graph ready
   * @type Boolean
   */
  isReady: false,

  /**
   * Is popup-graph ready
   * @type Boolean
   */
  isPopupReady: false,

  /**
   * Is data for graph available
   * @type Boolean
   */
  hasData: true,

  /**
   * chart height
   * @type {number}
   * @default 150
   */
  height: 150,

  /**
   * @type {string}
   * @default null
   */
  displayUnit: null,

  /**
   * Object containing information to get metrics data from API response
   * Supported properties:
   * <ul>
   * <li>path - exact path to metrics data in response JSON (required)
   * <li>displayName(name, hostName) - returns display name for metrics
   * depending on property name in response JSON and host name for Flume Agents
   * <li>factor - number that metrics values should be multiplied by
   * <li>flumePropertyName - property name to access certain Flume metrics
   * </ul>
   * @type {Object}
   */
  seriesTemplate: null,

  /**
   * Incomplete metrics requests
   * @type {array}
   * @default []
   */
  runningRequests: [],

  /**
   * Incomplete metrics requests for detailed view
   * @type {array}
   * @default []
   */
  runningPopupRequests: [],

  _containerSelector: Em.computed.format('#{0}-container', 'id'),

  _popupSelector: Em.computed.concat('', '_containerSelector', 'popupSuffix'),

  /**
   * @type {boolean}
   */
  isRequestRunning: function () {
    var requestsArrayName = this.get('isPopup') ? 'runningPopupRequests' : 'runningRequests';
    return this.get(requestsArrayName).mapProperty('ajaxIndex').contains(this.get('ajaxIndex'));
  }.property('runningPopupRequests', 'runningRequests'),

  didInsertElement: function didInsertElement() {
    var self = this;
    this.setYAxisFormatter();
    this.loadData();
    this.registerGraph();
    this.$().parent().on('mouseleave', function () {
      self.set('isExportMenuHidden', true);
    });
    App.tooltip(this.$("[rel='ZoomInTooltip']"), {
      placement: 'left',
      template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner graph-tooltip"></div></div>'
    });
  },

  setCurrentTimeIndex: function () {
    this.set('currentTimeIndex', this.get('parentView.currentTimeRangeIndex'));
  }.observes('parentView.currentTimeRangeIndex'),

  /**
   * Maps server data into series format ready for export to graph and JSON formats
   * @param jsonData
   * @returns {Array}
   */
  getData: function getData(jsonData) {
    var dataArray = [],
        template = this.get('seriesTemplate'),
        data = Em.get(jsonData, template.path);
    if (data) {
      for (var name in data) {
        var currentData = data[name];
        if (currentData) {
          if (Array.isArray(currentData)) {
            currentData = currentData.slice(0);
          }
          var factor = template.factor,
              displayName = template.displayName ? template.displayName(name) : name;
          if (!Em.isNone(factor)) {
            var dataLength = currentData.length;
            for (var i = dataLength; i--;) {
              currentData[i] = [currentData[i][0] * factor, currentData[i][1]];
            }
          }
          dataArray.push({
            name: displayName,
            data: currentData
          });
        }
      }
    }
    return dataArray;
  },

  /**
   * Maps server data for certain Flume metrics into series format ready for export
   * to graph and JSON formats
   * @param jsonData
   * @returns {Array}
   */
  getFlumeData: function getFlumeData(jsonData) {
    var dataArray = [];
    if (jsonData && jsonData.host_components) {
      jsonData.host_components.forEach(function (hc) {
        var hostName = hc.HostRoles.host_name,
            host = App.Host.find(hostName),
            template = this.get('template'),
            data = Em.get(hc, template.path);
        if (host && host.get('publicHostName')) {
          hostName = host.get('publicHostName');
        }
        if (data) {
          for (var cname in data) {
            var seriesName = template.displayName ? template.displayName(cname, hostName) : cname,
                seriesData = template.flumePropertyName ? data[cname][template.flumePropertyName] : data[cname];
            if (seriesData) {
              var factor = template.factor;
              if (!Em.isNone(factor)) {
                var dataLength = seriesData.length;
                for (var i = dataLength; i--;) {
                  seriesData[i][0] *= factor;
                }
              }
              dataArray.push({
                name: seriesName,
                data: seriesData
              });
            }
          }
        }
      }, this);
    }
    return dataArray;
  },

  /**
   * Function to map data into graph series
   * @param jsonData
   * @returns {Array}
   */
  transformToSeries: function transformToSeries(jsonData) {
    var seriesArray = [],
        seriesData = this.getData(jsonData),
        dataLength = seriesData.length;
    for (var i = 0; i < dataLength; i++) {
      seriesArray.push(this.transformData(seriesData[i].data, seriesData[i].name));
    }
    return seriesArray;
  },

  setExportTooltip: function () {
    if (this.get('isReady')) {
      Em.run.next(this, function () {
        var icon = this.$('.corner-icon');
        if (icon) {
          icon.on('mouseover', function () {
            $(this).closest("[rel='ZoomInTooltip']").trigger('mouseleave');
          });
          App.tooltip(icon.children('.glyphicon-save'), {
            title: Em.I18n.t('common.export')
          });
        }
      });
    }
  }.observes('isReady'),

  willDestroyElement: function willDestroyElement() {
    this.$("[rel='ZoomInTooltip']").tooltip('destroy');
    $(this.get('_containerSelector') + ' li.line').off();
    $(this.get('_popupSelector') + ' li.line').off();
    App.ajax.abortRequests(this.get('runningRequests'));
  },

  registerGraph: function registerGraph() {
    var graph = {
      name: this.get('title'),
      id: this.get('elementId'),
      popupId: this.get('id')
    };
    App.router.get('updateController.graphs').push(graph);
  },

  loadData: function loadData() {
    var self = this,
        isPopup = this.get('isPopup');
    if (this.get('loadGroup') && !isPopup) {
      return App.ChartLinearTimeView.LoadAggregator.add(this, this.get('loadGroup'));
    } else {
      var requestsArrayName = isPopup ? 'runningPopupRequests' : 'runningRequests',
          request = App.ajax.send({
        name: this.get('ajaxIndex'),
        sender: this,
        data: this.getDataForAjaxRequest(),
        success: 'loadDataSuccessCallback',
        error: 'loadDataErrorCallback',
        callback: function callback() {
          self.set(requestsArrayName, self.get(requestsArrayName).reject(function (item) {
            return item === request;
          }));
        }
      });
      if (request) request.ajaxIndex = this.get('ajaxIndex');
      this.get(requestsArrayName).push(request);
      return request;
    }
  },

  getDataForAjaxRequest: function getDataForAjaxRequest() {
    var fromSeconds,
        toSeconds,
        hostName = this.get('content') ? this.get('content.hostName') : "",
        YARNService = App.YARNService.find().objectAt(0),
        resourceManager = YARNService ? YARNService.get('resourceManager.hostName') : "";
    if (this.get('currentTimeIndex') === 8 && !Em.isNone(this.get('customStartTime')) && !Em.isNone(this.get('customEndTime'))) {
      // Custom start and end time is specified by user
      toSeconds = this.get('customEndTime') / 1000;
      fromSeconds = this.get('customStartTime') / 1000;
    } else {
      // Preset time range is specified by user
      var timeUnit = this.get('timeUnitSeconds');
      toSeconds = Math.round(App.dateTime() / 1000);
      fromSeconds = toSeconds - timeUnit;
    }
    return {
      toSeconds: toSeconds,
      fromSeconds: fromSeconds,
      stepSeconds: 15,
      hostName: hostName,
      resourceManager: resourceManager
    };
  },

  loadDataSuccessCallback: function loadDataSuccessCallback(response) {
    this._refreshGraph(response);
  },

  loadDataErrorCallback: function loadDataErrorCallback(xhr, textStatus, errorThrown) {
    if (!xhr.isForcedAbort) {
      this.set('isReady', true);
      if (xhr.readyState == 4 && xhr.status) {
        textStatus = xhr.status + " " + textStatus;
      }
      this._showMessage('warn', this.t('graphs.error.title'), this.t('graphs.error.message').format(textStatus, errorThrown));
      this.setProperties({
        hasData: false,
        isExportButtonHidden: true
      });
    }
  },

  /**
   * Shows a yellow warning message in place of the chart.
   *
   * @param type  Can be any of 'warn', 'error', 'info', 'success'
   * @param title Bolded title for the message
   * @param message String representing the message
   * @param tooltip Tooltip content
   * @type: Function
   */
  _showMessage: function _showMessage(type, title, message, tooltip) {
    var popupSuffix = this.get('isPopup') ? this.get('popupSuffix') : '';
    var chartOverlay = '#' + this.get('id');
    var chartOverlayId = chartOverlay + '-chart' + popupSuffix;
    var chartOverlayY = chartOverlay + '-yaxis' + popupSuffix;
    var chartOverlayX = chartOverlay + '-xaxis' + popupSuffix;
    var chartOverlayLegend = chartOverlay + '-legend' + popupSuffix;
    var chartOverlayTimeline = chartOverlay + '-timeline' + popupSuffix;
    var tooltipTitle = tooltip ? tooltip : Em.I18n.t('graphs.tooltip.title');
    var chartContent = '';

    var typeClass;
    switch (type) {
      case 'error':
        typeClass = 'alert-danger';
        break;
      case 'success':
        typeClass = 'alert-success';
        break;
      case 'info':
        typeClass = 'alert-info';
        break;
      default:
        typeClass = 'alert-warning';
        break;
    }
    $(chartOverlayId + ', ' + chartOverlayY + ', ' + chartOverlayX + ', ' + chartOverlayLegend + ', ' + chartOverlayTimeline).html('');
    chartContent += '<div class=\"alert ' + typeClass + '\">';
    if (title) {
      chartContent += '<strong>' + title + '</strong> ';
    }
    chartContent += message + '</div>';
    $(chartOverlayId).append(chartContent);
    $(chartOverlayId).parent().attr('data-original-title', tooltipTitle);
  },

  /**
   * Transforms the JSON data retrieved from the server into the series
   * format that Rickshaw.Graph understands.
   *
   * The series object is generally in the following format: [ { name :
   * "Series 1", data : [ { x : 0, y : 0 }, { x : 1, y : 1 } ] } ]
   *
   * Extending classes should override this method.
   *
   * @param seriesData
   *          Data retrieved from the server
   * @param displayName
   *          Graph title
   * @type: Function
   *
   */
  transformData: function transformData(seriesData, displayName) {
    if (!Em.isNone(seriesData)) {
      // Is it a string?
      if ("string" == typeof seriesData) {
        seriesData = JSON.parse(seriesData);
      }
      // Is it a number?
      if ("number" == typeof seriesData) {
        // Same number applies to all time.
        var number = seriesData;
        seriesData = [];
        seriesData.push([number, App.dateTime() - 60 * 60]);
        seriesData.push([number, App.dateTime()]);
      }
      // We have valid data
      var series = {};
      series.name = displayName;
      series.data = [];
      var timeDiff = App.dateTimeWithTimeZone(seriesData[0][1] * 1000) / 1000 - seriesData[0][1];
      for (var index = 0; index < seriesData.length; index++) {
        series.data.push({
          x: seriesData[index][1] + timeDiff,
          y: seriesData[index][0]
        });
      }
      return series;
    }
    return null;
  },

  /**
   * Provides the formatter to use in displaying Y axis.
   *
   * By default, uses the App.ChartLinearTimeView.DefaultFormatter which shows 10K,
   * 300M etc.
   *
   * @type Function
   */
  yAxisFormatter: function yAxisFormatter(y) {
    return App.ChartLinearTimeView.DefaultFormatter(y);
  },

  /**
   * Sets the formatter to use in displaying Y axis depending on graph unit.
   *
   * @type Function
   */
  setYAxisFormatter: function setYAxisFormatter() {
    var method,
        formatterMap = {
      '%': 'PercentageFormatter',
      '/s': 'CreateRateFormatter',
      'B': 'BytesFormatter',
      'ms': 'TimeElapsedFormatter'
    },
        methodName = formatterMap[this.get('displayUnit')];
    if (methodName) {
      method = methodName == 'CreateRateFormatter' ? App.ChartLinearTimeView.CreateRateFormatter('', App.ChartLinearTimeView.DefaultFormatter) : App.ChartLinearTimeView[methodName];
      this.set('yAxisFormatter', method);
    }
  },

  /**
   * Provides the color (in any HTML color format) to use for a particular
   * series.
   * May be redefined in child views
   *
   * @param series
   *          Series for which color is being requested
   * @return color String. Returning null allows this chart to pick a color
   *         from palette.
   * @default null
   * @type Function
   */
  colorForSeries: function colorForSeries(series) {
    return null;
  },

  /**
   * Check whether seriesData is correct data for chart drawing
   * @param {Array} seriesData
   * @return {Boolean}
   */
  checkSeries: function checkSeries(seriesData) {
    if (!seriesData || !seriesData.length) {
      return false;
    }
    var result = true;
    seriesData.forEach(function (item) {
      if (Em.isNone(Em.get(item, 'data.0.x'))) {
        result = false;
      }
    });
    return result;
  },

  /**
   * @private
   *
   * Refreshes the graph with the latest JSON data.
   *
   * @type Function
   */
  _refreshGraph: function _refreshGraph(jsonData, graphView) {
    if (this.get('isDestroyed')) {
      return;
    }
    console.time('_refreshGraph');
    var seriesData = this.transformToSeries(jsonData);

    //if graph opened as modal popup
    var popup_path = $(this.get('_popupSelector'));
    var graph_container = $(this.get('_containerSelector'));
    var seconds = this.get('parentView.graphSeconds');
    var container;
    if (!Em.isNone(seconds)) {
      this.set('timeUnitSeconds', seconds);
      this.set('parentView.graphSeconds', null);
    }
    if (popup_path.length) {
      popup_path.children().each(function () {
        $(this).children().remove();
      });
      this.set('isPopup', true);
    } else {
      graph_container.children().each(function () {
        if (!$(this).is('.export-graph-list-container, .corner-icon')) {
          $(this).children().remove();
        }
      });
    }
    var hasData = this.checkSeries(seriesData);
    var view = graphView || this;
    view.set('isExportButtonHidden', !hasData);
    if (hasData) {
      // Check container exists (may be not, if we go to another page and wait while graphs loading)
      if (graph_container.length) {
        container = $(this.get('_containerSelector'));
        this.draw(seriesData);
        this.set('hasData', true);
        //move yAxis value lower to make them fully visible
        container.find('.y_axis text').attr('y', 8);
        container.attr('data-original-title', Em.I18n.t('graphs.tooltip.title'));
      }
    } else {
      this.set('isPopupReady', true);
      this.set('isReady', true);
      //if Axis X time interval is default(60 minutes)
      if (this.get('timeUnitSeconds') === 3600) {
        this._showMessage('info', null, this.t('graphs.noData.message'), this.t('graphs.noData.tooltip.title'));
        this.set('hasData', false);
      } else {
        this._showMessage('info', this.t('graphs.noData.title'), this.t('graphs.noDataAtTime.message'));
      }
    }
    graph_container = null;
    container = null;
    popup_path = null;
    console.timeEnd('_refreshGraph');
  },

  /**
   * Returns a custom time unit, that depends on X axis interval length, for the graph's X axis.
   * This is needed as Rickshaw's default time X axis uses UTC time, which can be confusing
   * for users expecting locale specific time.
   *
   * If <code>null</code> is returned, Rickshaw's default time unit is used.
   *
   * @type Function
   * @return Rickshaw.Fixtures.Time
   */
  localeTimeUnit: function localeTimeUnit(timeUnitSeconds) {
    var timeUnit = new Rickshaw.Fixtures.Time(),
        unitName;
    if (timeUnitSeconds < 172800) {
      timeUnit = {
        name: timeUnitSeconds / 240 + ' minute',
        seconds: timeUnitSeconds / 4,
        formatter: function formatter(d) {
          // format locale specific time
          var minutes = dateUtils.dateFormatZeroFirst(d.getMinutes());
          var hours = dateUtils.dateFormatZeroFirst(d.getHours());
          return hours + ":" + minutes;
        }
      };
    } else if (timeUnitSeconds < 1209600) {
      timeUnit = timeUnit.unit('day');
    } else if (timeUnitSeconds < 5184000) {
      timeUnit = timeUnit.unit('week');
    } else if (timeUnitSeconds < 62208000) {
      timeUnit = timeUnit.unit('month');
    } else {
      timeUnit = timeUnit.unit('year');
    }
    return timeUnit;
  },

  /**
   * calculate statistic data for popup legend and set proper colors for series
   * @param {Array} data
   */
  dataPreProcess: function dataPreProcess(data) {
    var self = this;
    var palette = new Rickshaw.Color.Palette({ scheme: 'munin' });
    // Format series for display
    var series_min_length = 100000000;
    data.forEach(function (series) {
      var displayUnit = self.get('displayUnit');
      var seriesColor = self.colorForSeries(series);
      if (Em.isNone(seriesColor)) {
        seriesColor = palette.color();
      }
      series.color = seriesColor;
      series.stroke = 'rgba(0,0,0,0.3)';
      // calculate statistic data for popup legend
      var avg = 0;
      var min = Number.MAX_VALUE;
      var max = Number.MIN_VALUE;
      var numberOfNotNullValues = 0;
      series.isZero = true;
      for (var i = 0; i < series.data.length; i++) {
        avg += series.data[i]['y'];
        if (series.data[i]['y'] !== null) {
          numberOfNotNullValues++;
        }
        if (!Em.isNone(series.data[i]['y'])) {
          if (series.data[i]['y'] < min) {
            min = series.data[i]['y'];
          }
        }
        if (series.data[i]['y'] > max) {
          max = series.data[i]['y'];
        }
        if (series.data[i]['y'] > 1) {
          series.isZero = false;
        }
      }
      if (self.get('isPopup')) {
        series.name = string_utils.pad(series.name.length > 36 ? series.name.substr(0, 36) + '...' : series.name, 40, '&nbsp;', 2) + '|&nbsp;' + string_utils.pad('min', 5, '&nbsp;', 3) + string_utils.pad(self.get('yAxisFormatter')(min), 12, '&nbsp;', 3) + string_utils.pad('avg', 5, '&nbsp;', 3) + string_utils.pad(self.get('yAxisFormatter')(avg / numberOfNotNullValues), 12, '&nbsp;', 3) + string_utils.pad('max', 12, '&nbsp;', 3) + string_utils.pad(self.get('yAxisFormatter')(max), 5, '&nbsp;', 3);
      }
      if (series.isZero) {
        series.stroke = series.color;
      }
      if (series.data.length < series_min_length) {
        series_min_length = series.data.length;
      }
    });

    // All series should have equal length
    data.forEach(function (series) {
      if (series.data.length > series_min_length) {
        series.data.length = series_min_length;
      }
    });
  },

  draw: function draw(seriesData) {
    var self = this;
    var isPopup = this.get('isPopup');
    var p = isPopup ? this.get('popupSuffix') : '';

    this.dataPreProcess(seriesData);

    var chartElement = document.querySelector("#" + this.get('id') + "-chart" + p);
    var overlayElement = document.querySelector(+this.get('id') + "-container" + p);
    var xaxisElement = document.querySelector("#" + this.get('id') + "-xaxis" + p);
    var yaxisElement = document.querySelector("#" + this.get('id') + "-yaxis" + p);
    var legendElement = document.querySelector("#" + this.get('id') + "-legend" + p);

    var graphSize = this._calculateGraphSize();

    var _graph = new Rickshaw.GraphReopened({
      height: graphSize.height,
      width: graphSize.width,
      element: chartElement,
      series: seriesData,
      interpolation: 'step-after',
      stroke: true,
      renderer: this.get('renderer'),
      strokeWidth: 'area' === this.get('renderer') ? 1 : 2,
      max: seriesData.everyProperty('isZero') ? 1 : null
    });

    if ('area' === this.get('renderer')) {
      _graph.renderer.unstack = false;
    }

    new Rickshaw.Graph.Axis.Time({
      graph: _graph,
      timeUnit: this.localeTimeUnit(this.get('timeUnitSeconds'))
    });

    new Rickshaw.Graph.Axis.Y({
      tickFormat: this.yAxisFormatter,
      pixelsPerTick: isPopup ? 75 : 40,
      element: yaxisElement,
      orientation: isPopup ? 'left' : 'right',
      graph: _graph
    });

    var legend = new Rickshaw.Graph.Legend({
      graph: _graph,
      element: legendElement,
      description: self.get('description')
    });

    new Rickshaw.Graph.Behavior.Series.Toggle({
      graph: _graph,
      legend: legend
    });

    new Rickshaw.Graph.Behavior.Series.Order({
      graph: _graph,
      legend: legend
    });

    if (!isPopup) {
      $(overlayElement).on('mousemove', function () {
        $(xaxisElement).removeClass('hide');
        $(legendElement).removeClass('hide');
        $(chartElement).children("div").removeClass('hide');
      });
      $(overlayElement).on('mouseout', function () {
        $(legendElement).addClass('hide');
      });
      _graph.onUpdate(function () {
        $(legendElement).addClass('hide');
      });
    }

    this.$().on('remove', function () {
      $(overlayElement).off();
    });

    //show the graph when it's loaded
    _graph.onUpdate(function () {
      self.set('isReady', true);
    });
    _graph.render();

    if (isPopup) {
      new Rickshaw.Graph.HoverDetail({
        graph: _graph,
        yFormatter: function yFormatter(y) {
          return self.yAxisFormatter(y);
        },
        xFormatter: function xFormatter(x) {
          return new Date(x * 1000).toLocaleTimeString();
        },
        formatter: function formatter(series, x, y, formattedX, formattedY) {
          return formattedY + '<br />' + formattedX;
        }
      });
    }

    _graph = this.updateSeriesInGraph(_graph);
    if (isPopup) {
      //show the graph when it's loaded
      _graph.onUpdate(function () {
        self.set('isPopupReady', true);
      });
      _graph.update();

      var popupSelector = this.get('_popupSelector');
      $(popupSelector + ' li.line').click(function () {
        var series = [];
        $(popupSelector + ' a.action').each(function (index, v) {
          series[index] = v.parentNode.classList;
        });
        self.set('_seriesProperties', series);
      });

      this.set('_popupGraph', _graph);
    } else {
      _graph.update();
      var containerSelector = this.get('_containerSelector');
      $(containerSelector + ' li.line').click(function () {
        var series = [];
        $(containerSelector + ' a.action').each(function (index, v) {
          series[index] = v.parentNode.classList;
        });
        self.set('_seriesPropertiesWidget', series);
      });

      this.set('_graph', _graph);
    }
  },

  /**
   * Calculate graph size
   * @returns {{width: number, height: number}}
   * @private
   */
  _calculateGraphSize: function _calculateGraphSize() {
    var isPopup = this.get('isPopup');
    var height = this.get('height');
    var width = 400;
    var diff = 32;

    if (this.get('inWidget')) {
      height = 105; // for widgets view
      diff = 22;
    }
    if (isPopup) {
      height = 180;
      width = 670;
    } else {
      // If not in popup, the width could vary.
      // We determine width based on div's size.
      var thisElement = this.get('element');
      if (!Em.isNone(thisElement)) {
        var calculatedWidth = $(thisElement).width();
        if (calculatedWidth > diff) {
          width = calculatedWidth - diff;
        }
      }
    }
    return {
      width: width,
      height: height
    };
  },

  /**
   *
   * @param {Rickshaw.Graph} graph
   * @returns {Rickshaw.Graph}
   */
  updateSeriesInGraph: function updateSeriesInGraph(graph) {
    var id = this.get('id');
    var isPopup = this.get('isPopup');
    var popupSuffix = this.get('popupSuffix');
    var _series = isPopup ? this.get('_seriesProperties') : this.get('_seriesPropertiesWidget');
    graph.series.forEach(function (series, index) {
      if (_series && !Em.isNone(_series[index])) {
        if (_series[_series.length - index - 1].length > 1) {
          var s = '#' + id + '-container' + (isPopup ? popupSuffix : '') + ' a.action:eq(' + (_series.length - index - 1) + ')';
          $(s).parent('li').addClass('disabled');
          series.disable();
        }
      }
    });
    return graph;
  },

  showGraphInPopup: function showGraphInPopup() {
    if (!this.get('hasData') || this.get('isPreview')) {
      return;
    }

    this.set('isPopup', true);
    if (this.get('inWidget') && !this.get('parentView.isClusterMetricsWidget')) {
      this.setProperties({
        currentTimeIndex: this.get('parentView.timeIndex'),
        customStartTime: this.get('parentView.startTime'),
        customEndTime: this.get('parentView.endTime'),
        customDurationFormatted: this.get('parentView.durationFormatted')
      });
    }
    var self = this;

    App.ModalPopup.show({
      bodyClass: Em.View.extend(App.ExportMetricsMixin, App.TimeRangeMixin, {

        containerId: null,
        containerClass: null,
        yAxisId: null,
        yAxisClass: null,
        xAxisId: null,
        xAxisClass: null,
        legendId: null,
        legendClass: null,
        chartId: null,
        chartClass: null,
        titleId: null,
        titleClass: null,

        timeRangeClassName: 'graph-details-time-range',

        isReady: Em.computed.alias('parentView.graph.isPopupReady'),

        currentTimeRangeIndex: self.get('currentTimeIndex'),
        customStartTime: self.get('currentTimeIndex') === 8 ? self.get('customStartTime') : null,
        customEndTime: self.get('currentTimeIndex') === 8 ? self.get('customEndTime') : null,
        customDurationFormatted: self.get('currentTimeIndex') === 8 ? self.get('customDurationFormatted') : null,

        didInsertElement: function didInsertElement() {
          var popupBody = this;
          this._super();
          App.tooltip(this.$('.corner-icon > .glyphicon-save'), {
            title: Em.I18n.t('common.export')
          });
          this.$().closest('.modal').on('click', function (event) {
            if (!$(event.target).is('.corner-icon, .corner-icon span, .glyphicon-save, .export-graph-list-container, .export-graph-list-container *')) {
              popupBody.set('isExportMenuHidden', true);
            }
          });
          $('#modal').addClass('modal-graph-line');
          var popupSuffix = this.get('parentView.graph.popupSuffix');
          var id = this.get('parentView.graph.id');
          var idTemplate = id + '-{element}' + popupSuffix;

          this.set('containerId', idTemplate.replace('{element}', 'container'));
          this.set('containerClass', 'chart-container' + popupSuffix);
          this.set('yAxisId', idTemplate.replace('{element}', 'yaxis'));
          this.set('yAxisClass', this.get('yAxisId').replace(popupSuffix, ''));
          this.set('xAxisId', idTemplate.replace('{element}', 'xaxis'));
          this.set('xAxisClass', this.get('xAxisId').replace(popupSuffix, ''));
          this.set('legendId', idTemplate.replace('{element}', 'legend'));
          this.set('legendClass', this.get('legendId').replace(popupSuffix, ''));
          this.set('chartId', idTemplate.replace('{element}', 'chart'));
          this.set('chartClass', this.get('chartId').replace(popupSuffix, ''));
          this.set('titleId', idTemplate.replace('{element}', 'title'));
          this.set('titleClass', this.get('titleId').replace(popupSuffix, ''));
        },

        templateName: require('templates/common/chart/linear_time'),
        /**
         * check is time paging feature is enable for graph
         */
        isTimePagingEnable: function () {
          return !self.get('isTimePagingDisable');
        }.property(),

        rightArrowVisible: function () {
          // Time range is neither custom nor the least possible preset
          return this.get('isReady') && this.get('parentView.currentTimeIndex') != 0 && this.get('parentView.currentTimeIndex') != 8;
        }.property('isReady', 'parentView.currentTimeIndex'),

        leftArrowVisible: function () {
          // Time range is neither custom nor the largest possible preset
          return this.get('isReady') && this.get('parentView.currentTimeIndex') < 7;
        }.property('isReady', 'parentView.currentTimeIndex'),

        exportGraphData: function exportGraphData(event) {
          this.set('isExportMenuHidden', true);
          var ajaxIndex = this.get('parentView.graph.ajaxIndex'),
              targetView = ajaxIndex ? this.get('parentView.graph') : self.get('parentView');
          targetView.exportGraphData({
            context: event.context
          });
        },

        setTimeRange: function setTimeRange(event) {
          var index = event.context.index,
              callback = this.get('parentView').reloadGraphByTime.bind(this.get('parentView'), index);
          this._super(event, callback, self);

          // Preset time range is specified by user
          if (index !== 8) {
            callback();
          }
        }
      }),
      header: this.get('title'),
      /**
       * App.ChartLinearTimeView
       */
      graph: self,

      secondary: null,

      onPrimary: function onPrimary() {
        var targetView = Em.isNone(self.get('parentView.currentTimeRangeIndex')) ? self.get('parentView.parentView') : self.get('parentView');
        self.setProperties({
          currentTimeIndex: targetView.get('currentTimeRangeIndex'),
          customStartTime: targetView.get('customStartTime'),
          customEndTime: targetView.get('customEndTime'),
          customDurationFormatted: targetView.get('customDurationFormatted'),
          isPopup: false
        });
        App.ajax.abortRequests(this.get('graph.runningPopupRequests'));
        this._super();
      },

      onClose: function onClose() {
        this.onPrimary();
      },
      /**
       * move graph back by time
       * @param event
       */
      switchTimeBack: function switchTimeBack(event) {
        var index = this.get('currentTimeIndex');
        // 7 - number of last preset time state
        if (index < 7) {
          this.reloadGraphByTime(++index);
        }
      },
      /**
       * move graph forward by time
       * @param event
       */
      switchTimeForward: function switchTimeForward(event) {
        var index = this.get('currentTimeIndex');
        if (index) {
          this.reloadGraphByTime(--index);
        }
      },
      /**
       * reload graph depending on the time
       * @param index
       */
      reloadGraphByTime: function reloadGraphByTime(index) {
        this.set('childViews.firstObject.currentTimeRangeIndex', index);
        this.set('currentTimeIndex', index);
        self.setProperties({
          currentTimeIndex: index,
          isPopupReady: false
        });
        App.ajax.abortRequests(this.get('graph.runningPopupRequests'));
      },
      currentTimeIndex: self.get('currentTimeIndex'),

      currentTimeState: function () {
        return self.get('timeStates').objectAt(this.get('currentTimeIndex'));
      }.property('currentTimeIndex')

    });
    Em.run.next(function () {
      self.loadData();
      self.set('isPopupReady', false);
    });
  },
  reloadGraphByTime: function () {
    Em.run.once(this, function () {
      this.loadData();
    });
  }.observes('timeUnitSeconds', 'customStartTime', 'customEndTime'),

  timeStates: [{ name: Em.I18n.t('graphs.timeRange.hour'), seconds: 3600 }, { name: Em.I18n.t('graphs.timeRange.twoHours'), seconds: 7200 }, { name: Em.I18n.t('graphs.timeRange.fourHours'), seconds: 14400 }, { name: Em.I18n.t('graphs.timeRange.twelveHours'), seconds: 43200 }, { name: Em.I18n.t('graphs.timeRange.day'), seconds: 86400 }, { name: Em.I18n.t('graphs.timeRange.week'), seconds: 604800 }, { name: Em.I18n.t('graphs.timeRange.month'), seconds: 2592000 }, { name: Em.I18n.t('graphs.timeRange.year'), seconds: 31104000 }, { name: Em.I18n.t('common.custom'), seconds: 0 }],
  // should be set by time range control dropdown list when create current graph
  currentTimeIndex: 0,
  customStartTime: null,
  customEndTime: null,
  customDurationFormatted: null,
  setCurrentTimeIndexFromParent: function () {
    // 8 index corresponds to custom start and end time selection
    var targetView = !Em.isNone(this.get('parentView.currentTimeRangeIndex')) ? this.get('parentView') : this.get('parentView.parentView'),
        index = targetView.get('currentTimeRangeIndex'),
        customStartTime = index === 8 ? targetView.get('customStartTime') : null,
        customEndTime = index === 8 ? targetView.get('customEndTime') : null,
        customDurationFormatted = index === 8 ? targetView.get('customDurationFormatted') : null;
    this.setProperties({
      currentTimeIndex: index,
      customStartTime: customStartTime,
      customEndTime: customEndTime,
      customDurationFormatted: customDurationFormatted
    });
    if (index !== 8 || targetView.get('customStartTime') && targetView.get('customEndTime')) {
      App.ajax.abortRequests(this.get('runningRequests'));
      if (this.get('inWidget') && !this.get('parentView.isClusterMetricsWidget')) {
        this.set('parentView.isLoaded', false);
      } else {
        this.set('isReady', false);
      }
    }
  }.observes('parentView.parentView.currentTimeRangeIndex', 'parentView.currentTimeRangeIndex', 'parentView.parentView.customStartTime', 'parentView.customStartTime', 'parentView.parentView.customEndTime', 'parentView.customEndTime'),
  timeUnitSeconds: 3600,
  timeUnitSecondsSetter: function () {
    var index = this.get('currentTimeIndex'),
        startTime = this.get('customStartTime'),
        endTime = this.get('customEndTime'),
        durationFormatted = this.get('customDurationFormatted'),
        seconds;
    if (index !== 8) {
      // Preset time range is specified by user
      seconds = this.get('timeStates').objectAt(this.get('currentTimeIndex')).seconds;
    } else if (!Em.isNone(startTime) && !Em.isNone(endTime)) {
      // Custom start and end time is specified by user
      seconds = (endTime - startTime) / 1000;
    }
    if (!Em.isNone(seconds)) {
      this.set('timeUnitSeconds', seconds);
      if (this.get('inWidget') && !this.get('parentView.isClusterMetricsWidget')) {
        this.get('parentView').setProperties({
          graphSeconds: seconds,
          timeIndex: index,
          startTime: startTime,
          endTime: endTime,
          durationFormatted: durationFormatted
        });
      }
    }
  }.observes('currentTimeIndex', 'customStartTime', 'customEndTime')

});

/**
 * A formatter which will turn a number into computer storage sizes of the
 * format '23 GB' etc.
 *
 * @type {Function}
 * @return {string}
 */
App.ChartLinearTimeView.BytesFormatter = function (y) {
  if (0 == y) {
    return '0 B';
  }
  var value = Rickshaw.Fixtures.Number.formatBase1024KMGTP(y);
  if (!y) {
    return '0 B';
  }
  if ("number" == typeof value) {
    value = String(value);
  }
  if ("string" == typeof value) {
    value = value.replace(/\.\d(\d+)/, function ($0, $1) {
      // Remove only 1-digit after decimal part
      return $0.replace($1, '');
    });
    // Either it ends with digit or ends with character
    value = value.replace(/(\d$)/, '$1 '); // Ends with digit like '120'
    value = value.replace(/([a-zA-Z]$)/, ' $1'); // Ends with character like
    // '120M'
    value = value + 'B'; // Append B to make B, MB, GB etc.
  }
  return value;
};

/**
 * A formatter which will turn a number into percentage display like '42%'
 *
 * @type {Function}
 * @param {number} percentage
 * @return {string}
 */
App.ChartLinearTimeView.PercentageFormatter = function (percentage) {
  return percentage ? percentage.toFixed(3).replace(/0+$/, '').replace(/\.$/, '') + '%' : '0 %';
};

/**
 * A formatter which will turn a number into percentage display like '42%'
 *
 * @type {Function}
 * @param {number} value
 * @param {string} displayUnit
 * @return {string}
 */
App.ChartLinearTimeView.DisplayUnitFormatter = function (value, displayUnit) {
  return value ? value.toFixed(3).replace(/0+$/, '').replace(/\.$/, '') + " " + displayUnit : '0 ' + displayUnit;
};

/**
 * A formatter which will turn elapsed time into display time like '50 ms',
 * '5s', '10 m', '3 hr' etc. Time is expected to be provided in milliseconds.
 *
 * @type {Function}
 * @return {string}
 */
App.ChartLinearTimeView.TimeElapsedFormatter = function (millis) {
  if (!millis) {
    return '0 ms';
  }
  if ('number' == Em.typeOf(millis)) {
    var seconds = millis > 1000 ? Math.round(millis / 1000) : 0;
    var minutes = seconds > 60 ? Math.round(seconds / 60) : 0;
    var hours = minutes > 60 ? Math.round(minutes / 60) : 0;
    var days = hours > 24 ? Math.round(hours / 24) : 0;
    if (days > 0) {
      return days + ' d';
    }
    if (hours > 0) {
      return hours + ' hr';
    }
    if (minutes > 0) {
      return minutes + ' m';
    }
    if (seconds > 0) {
      return seconds + ' s';
    }
    return millis.toFixed(3).replace(/0+$/, '').replace(/\.$/, '') + ' ms';
  }
  return millis;
};

/**
 * The default formatter which uses Rickshaw.Fixtures.Number.formatKMBT
 * which shows 10K, 300M etc.
 *
 * @type {Function}
 * @return {string|number}
 */
App.ChartLinearTimeView.DefaultFormatter = function (y) {
  if (!y) {
    return 0;
  }
  var value = Rickshaw.Fixtures.Number.formatKMBT(y);
  if ('' == value) return '0';
  value = String(value);
  var c = value[value.length - 1];
  return isNaN(parseInt(c)) ? parseFloat(value.substr(0, value.length - 1)).toFixed(3).replace(/0+$/, '').replace(/\.$/, '') + c : parseFloat(value).toFixed(3).replace(/0+$/, '').replace(/\.$/, '');
};

/**
 * Creates and returns a formatter that can convert a 'value'
 * to 'value units/s'.
 *
 * @param unitsPrefix Prefix which will be used in 'unitsPrefix/s'
 * @param valueFormatter  Value itself will need further processing
 *        via provided formatter. Ex: '10M requests/s'. Generally
 *        should be App.ChartLinearTimeView.DefaultFormatter.
 * @return {Function}
 */
App.ChartLinearTimeView.CreateRateFormatter = function (unitsPrefix, valueFormatter) {
  var suffix = " " + unitsPrefix + "/s";
  return function (value) {
    value = valueFormatter(value) + suffix;
    return value;
  };
};

Rickshaw.GraphReopened = function (a) {
  Rickshaw.Graph.call(this, a);
};

//reopened in order to exclude "null" value from validation
Rickshaw.GraphReopened.prototype = Object.create(Rickshaw.Graph.prototype, {
  validateSeries: {
    value: function value(series) {

      if (!(series instanceof Array) && !(series instanceof Rickshaw.Series)) {
        var seriesSignature = Object.prototype.toString.apply(series);
        throw "series is not an array: " + seriesSignature;
      }

      var pointsCount;

      series.forEach(function (s) {

        if (!(s instanceof Object)) {
          throw "series element is not an object: " + s;
        }
        if (!s.data) {
          throw "series has no data: " + JSON.stringify(s);
        }
        if (!(s.data instanceof Array)) {
          throw "series data is not an array: " + JSON.stringify(s.data);
        }

        pointsCount = pointsCount || s.data.length;

        if (pointsCount && s.data.length != pointsCount) {
          throw "series cannot have differing numbers of points: " + pointsCount + " vs " + s.data.length + "; see Rickshaw.Series.zeroFill()";
        }
      });
    },
    enumerable: true,
    configurable: false,
    writable: false
  }
});

//show no line if point is "null"
Rickshaw.Graph.Renderer.Line.prototype.seriesPathFactory = function () {

  var graph = this.graph;

  return d3.svg.line().x(function (d) {
    return graph.x(d.x);
  }).y(function (d) {
    return graph.y(d.y);
  }).defined(function (d) {
    return d.y != null;
  }).interpolate(this.graph.interpolation).tension(this.tension);
};

//show no area if point is null
Rickshaw.Graph.Renderer.Stack.prototype.seriesPathFactory = function () {

  var graph = this.graph;

  return d3.svg.area().x(function (d) {
    return graph.x(d.x);
  }).y0(function (d) {
    return graph.y(d.y0);
  }).y1(function (d) {
    return graph.y(d.y + d.y0);
  }).defined(function (d) {
    return d.y != null;
  }).interpolate(this.graph.interpolation).tension(this.tension);
};

/**
 * aggregate requests to load metrics by component name
 * requests can be added via add method
 * input example:
 * {
 *   data: request,
 *   context: this,
 *   startCallName: this.getServiceComponentMetrics,
 *   successCallback: this.getMetricsSuccessCallback,
 *   completeCallback: function () {
 *     requestCounter--;
 *     if (requestCounter === 0) this.onMetricsLoaded();
 *   }
 * }
 * @type {Em.Object}
 */
App.ChartLinearTimeView.LoadAggregator = Em.Object.create({
  /**
   * @type {Array}
   */
  requests: [],

  /**
   * @type {number|null}
   */
  timeoutId: null,

  /**
   * time interval within which calls get collected
   * @type {number}
   * @const
   */
  BULK_INTERVAL: 1000,

  /**
   * add request
   * every {{BULK_INTERVAL}} requests get collected, aggregated and sent to server
   *
   * @param {object} context
   * @param {object} requestData
   */
  add: function add(context, requestData) {
    var self = this;

    requestData.context = context;
    this.get('requests').push(requestData);
    if (Em.isNone(this.get('timeoutId'))) {
      this.set('timeoutId', window.setTimeout(function () {
        self.runRequests(self.get('requests'));
        self.get('requests').clear();
        clearTimeout(self.get('timeoutId'));
        self.set('timeoutId', null);
      }, this.get('BULK_INTERVAL')));
    }
  },

  /**
   * return requests which grouped into bulks
   * @param {Array} requests
   * @returns {object} bulks
   */
  groupRequests: function groupRequests(requests) {
    var bulks = {};

    requests.forEach(function (request) {
      var id = request.name;

      if (Em.isNone(bulks[id])) {
        bulks[id] = {
          name: request.name,
          fields: request.fields.slice(),
          context: request.context
        };
        bulks[id].subRequests = [{
          context: request.context
        }];
      } else {
        bulks[id].fields.pushObjects(request.fields);
        bulks[id].subRequests.push({
          context: request.context
        });
      }
    }, this);
    return bulks;
  },

  /**
   * run aggregated requests
   * @param {Array} requests
   */
  runRequests: function runRequests(requests) {
    var bulks = this.groupRequests(requests);
    var self = this;

    for (var id in bulks) {
      (function (_request) {
        var fields = self.formatRequestData(_request);
        var hostName = _request.context.get('content') ? _request.context.get('content.hostName') : "";
        var xhr = App.ajax.send({
          name: _request.name,
          sender: _request.context,
          data: {
            fields: fields,
            hostName: hostName
          }
        });

        xhr.done(function (response) {
          console.time('==== runRequestsDone');
          _request.subRequests.forEach(function (subRequest) {
            subRequest.context._refreshGraph.call(subRequest.context, response);
          }, this);
          console.timeEnd('==== runRequestsDone');
        }).fail(function (jqXHR, textStatus, errorThrown) {
          if (!jqXHR.isForcedAbort) {
            _request.subRequests.forEach(function (subRequest) {
              subRequest.context.loadDataErrorCallback.call(subRequest.context, jqXHR, textStatus, errorThrown);
            }, this);
          }
        }).always(function () {
          _request.context.set('runningRequests', Em.tryInvoke(_request.context.get('runningRequests'), 'without', [xhr]));
        });
        Em.tryInvoke(_request.context.get('runningRequests'), 'push', [xhr]);
      })(bulks[id]);
    }
  },

  /**
   *
   * @param {object} request
   * @returns {number[]}
   */
  formatRequestData: function formatRequestData(request) {
    var fromSeconds, toSeconds;
    if (request.context.get('currentTimeIndex') === 8 && !Em.isNone(request.context.get('customStartTime')) && !Em.isNone(request.context.get('customEndTime'))) {
      // Custom start and end time is specified by user
      toSeconds = request.context.get('customEndTime') / 1000;
      fromSeconds = request.context.get('customStartTime') / 1000;
    } else {
      var timeUnit = request.context.get('timeUnitSeconds');
      toSeconds = Math.round(App.dateTime() / 1000);
      fromSeconds = toSeconds - timeUnit;
    }
    var fields = request.fields.uniq().map(function (field) {
      return field + "[" + fromSeconds + "," + toSeconds + "," + 15 + "]";
    });

    return fields.join(",");
  }
});

});

require.register("views/common/chart/pie", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartPieView = Em.View.extend({
  w: 100,
  h: 100,
  data: [300, 500],
  id: null,
  palette: new Rickshaw.Color.Palette({ scheme: 'munin' }),
  stroke: 'black',
  strokeWidth: 1,
  donut: d3.layout.pie().sort(null),
  existCenterText: false,
  centerTextColor: 'black',

  r: function () {
    return Math.min(this.get('w'), this.get('h')) / 2 - this.get('strokeWidth');
  }.property('w', 'h'),

  outerR: Em.computed.alias('r'),

  innerR: function () {
    return 0; // this.get('r') - 20;
  }.property('r'),

  arc: function () {
    return d3.svg.arc().innerRadius(this.get('innerR')).outerRadius(this.get('outerR'));
  }.property(),

  didInsertElement: function didInsertElement() {
    this._super();
    this.appendSvg();
  },

  selector: Em.computed.format('#{0}', 'elementId'),

  appendSvg: function appendSvg() {

    var thisChart = this;
    var svg = d3.select(thisChart.get('selector')).append("svg:svg").attr("id", thisChart.get('id')).attr("width", thisChart.get('w')).attr("height", thisChart.get('h')).attr("stroke", thisChart.get('stroke')).attr("stroke-width", thisChart.get('strokeWidth'));

    // set percentage data in center if there exist a center text
    if (thisChart.get('existCenterText')) {
      this.set('svg', svg.append("svg:g").attr("render-order", 1).append("svg:text").style('fill', thisChart.get('centerTextColor')).attr("stroke", thisChart.get('centerTextColor')).attr("font-size", 24).attr("transform", "translate(" + thisChart.get('w') / 2 + "," + (thisChart.get('h') / 2 + 8) + ")").attr("text-anchor", "middle").text(function (d) {
        return thisChart.get('data')[0] + '%';
      }));
    }

    this.set('svg', svg.append("svg:g").attr("transform", "translate(" + thisChart.get('w') / 2 + "," + thisChart.get('h') / 2 + ")"));

    this.set('arcs', thisChart.get('svg').selectAll(".arc").data(thisChart.donut(thisChart.get('data'))).enter().append("svg:g").attr('class', 'arc').append('svg:path').attr("fill", function (d, i) {
      return thisChart.palette.color(i);
    }).attr("d", thisChart.get('arc')));
  }
});

});

require.register("views/common/checkbox_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Create custom styled checkbox with appropriate bindings and HTML template.
 * You can specify <code><label></code> text with <code>label</code> property, or pass I18n translation key
 * to <code>labelTranslate</code> property. Checkbox specific attributes can be bound in the same way as
 * for Ember.Checkbox <code>checked</code>, <code>disabled</code>.
 * Additional class names can be added with <code>labelClassNames</code> and <code>checkboxClassNames</code>
 * properties.
 *
 * @example
 * Usage:
 * {{view App.CheckboxView checkedBinding="someBinding" label="someLabel"}}
 * {{view App.CheckboxView checkedBinding="someBinding" labelTranslate="i18n.translate"}}
 * {{view App.CheckboxView checkedBinding="someBinding"
 *        labelClassNames="class-1 class-2"
 *        checkboxClassNames="class-3 class-4"}}
 *
 * <!-- will append additional markup after <code><label></code> tag
 * {{#view App.CheckboxView}}
 *   <span>Additional content</span>
 * {{/view}}
 *
 * <!-- custom template -->
 * {{#view App.CheckboxView isCustomTemplate="true"}}
 *   <!-- any markup here -->
 *   <label {{bindAttr for="view.checkboxId"}}>Left aligned label</label>
 *   <!-- minimal template to display checkbox -->
 *   {{view view.checkboxView}}
 *   {{view view.labelView}}
 * {{/view}}
 * @type {Ember.View}
 */
App.CheckboxView = Em.View.extend({
  layoutName: require('templates/common/checkbox'),
  concatenatedProperties: ['labelClassNames', 'checkboxClassNames'],
  classNames: ['checkbox'],

  /**
   * Checkbox id generated by Ember. Used for <code>for</code> label attribute
   * @private
   * @type {String}
   */
  checkboxId: '',
  /**
   * When <code>true</code> custom html markup will be used.
   * @type {Boolean}
   */
  isCustomTemplate: false,

  /**
   * Stores checkbox's <code>checked</code> attribute value
   * @type {Boolean}
   */
  checked: false,

  /**
   * Stores checkbox's <code>disabled</code> attribute value
   * @type {Boolean}
   */
  disabled: false,

  /**
   * Label's text
   * @type {String}
   */
  label: '',

  /**
   * Label's i18n code
   * @type {[type]}
   */
  labelTranslate: null,

  /**
   * Checkbox's <code>name</code> attribute value
   * @type {String}
   */
  name: '',

  /**
   * Default class for label element
   * @type {String[]}
   */
  labelClassNames: ['checkbox-label'],

  /**
   * Label's text to display. <code>label</code> property value will be used if no
   * <code>labelTranslate</code> passed
   * @return {String}
   */
  labelText: function () {
    return this.get('labelTranslate') ? Em.I18n.t(this.get('labelTranslate')) : this.get('label');
  }.property('label', 'labelTranslate'),

  /**
   * Ember checkbox view
   * @type {Em.View}
   */
  checkboxView: Ember.Checkbox.extend({
    attributeBindings: ['name'],
    checkedBinding: 'parentView.checked',
    disabledBinding: 'parentView.disabled',
    nameBinding: 'parentView.name',
    classNameBindings: 'parentView.checkboxClassNames',

    didInsertElement: function didInsertElement() {
      this.set('parentView.checkboxId', this.get('elementId'));
      this._super();
    },
    change: function change() {
      if (typeof this.get('parentView.change') === 'function') {
        this.get('parentView.change').apply(this.get('parentView'), arguments);
      }
    }
  }),

  /**
   * Label view
   * @type {[type]}
   */
  labelView: Em.View.extend({
    labelTextBinding: 'parentView.labelText',
    classNameBindings: 'parentView.labelClassNames',
    tagName: 'label',
    attributeBindings: ['for', 'parentView.checked:checked'],
    'data-qaBinding': 'parentView.labelIdentifier',
    template: Em.Handlebars.compile('{{view.labelText}}'),
    for: Em.computed.alias('parentView.checkboxId')
  })
});

});

require.register("views/common/chosen_plugin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.JqueryChosenView = Em.View.extend({
  templateName: require('templates/common/chosen_plugin'),
  tagName: 'select',
  classNames: ['form-control'],
  // This needs to be bound from template
  elementId: '',
  title: '',
  options: [],
  /**
   * @name: selectionObj {Object}
   *  Object =   {
   *    placeholder_text: {String}
   *    no_results_text: {String}
   *    nChangeCallback: {Function}
   *  }
   */
  selectionObj: {},

  didInsertElement: function didInsertElement() {
    var self = this;
    var elementId = "#" + self.get("elementId");
    $(elementId).chosen({
      search_contains: true,
      placeholder_text: self.get('selectionObj.placeholder_text'),
      no_results_text: self.get('selectionObj.no_results_text')
    }).change(self.get('selectionObj.onChangeCallback'));
  }
});

});

require.register("views/common/clock_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dateUtils = require('utils/date/date');

App.ClockView = Em.View.extend({
  classNames: ['clock-view'],
  template: Em.Handlebars.compile('<span>{{view.currentTime}}'),
  dateFormat: 'HH:mm:ss',
  updateInterval: 1000,
  /**
   * Interval function id
   * @type {Number}
   */
  intervalId: null,

  didInsertElement: function didInsertElement() {
    var self = this;
    this.set('intervalId', setInterval(function () {
      self.set('currentTime', dateUtils.dateFormat(new Date().getTime() + App.get('clockDistance'), self.get('dateFormat')));
    }, this.get('updateInterval')));
  },

  willDestroyElement: function willDestroyElement() {
    if (this.get('intervalId')) {
      clearInterval(this.get('intervalId'));
      this.set('intervalId', null);
    }
  }
});

});

require.register("views/common/configs/config_category_container_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var lazyLoading = require('utils/lazy_loading');

App.ConfigCategoryContainerView = Em.ContainerView.extend(App.LoadingOverlaySupport, {

  lazyLoading: null,

  categories: [],

  classNames: ['accordion'],

  fieldToObserve: 'controller.recommendationsInProgress',

  handleFieldChangesOnDidInsert: false,

  didInsertElement: function didInsertElement() {
    this.pushViews();
    this._super();
  },

  willDestroyElement: function willDestroyElement() {
    if (this.get('lazyLoading')) {
      lazyLoading.terminate(this.get('lazyLoading'));
    }
    this._super();
  },

  /**
   * extra calls for <code>handleFieldChanges</code>:
   * <ul>
   *  <li>child views are pushed to this (lazy loading is used)</li>
   *  <li>tab is switched</li>
   * </ul>
   */
  checkOverlay: function () {
    this.handleFieldChanges();
  }.observes('isLoaded', 'controller.activeTab.id', 'controller.activeTab.isRendered'),

  pushViews: function pushViews() {
    var self = this;
    // Create view with loading-overlay. For some unknown reasons overlay can't be added to the container view
    var categoriesViews = [Em.View.extend({
      template: Em.Handlebars.compile('<div class="loading-overlay"></div>')
    }).create()];
    var categories = this.get('categories');
    if (!categories) return;
    categories.forEach(function (category) {
      var viewClass = category.isCustomView ? category.customView : App.ServiceConfigsByCategoryView;
      categoriesViews.push(viewClass.create({
        category: category,
        canEdit: self.get("canEdit"),
        service: self.get("service"),
        serviceConfigs: self.get("serviceConfigs"),
        supportsHostOverrides: self.get("supportsHostOverrides")
      }));
    });
    this.set('lazyLoading', lazyLoading.run({
      destination: self.get('childViews'),
      source: categoriesViews,
      initSize: 3,
      chunkSize: 3,
      delay: 200,
      context: this
    }));
  }

});

});

require.register("views/common/configs/config_diff_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.ConfigDiffView = Em.View.extend({
  template: Em.Handlebars.compile('{{view.diff}}'),
  diff: function () {
    var trimAndSort = function trimAndSort(value) {
      if (value == null) {
        return [];
      }
      var values = value.split("\n").filter(function (item) {
        return item != "";
      }).sort().join("\n");
      return difflib.stringAsLines(values);
    };
    var initialValues = trimAndSort(this.get('config.initialValue')),
        recommendedValues = trimAndSort(this.get('config.recommendedValue')),
        opcodes = new difflib.SequenceMatcher(initialValues, recommendedValues).get_opcodes();
    if (initialValues.length === 1 && recommendedValues.length === 1) {
      // changes in properties with single-line values shouldn't be highlighted
      opcodes[0][0] = 'equal';
    }
    if (!initialValues.length) {
      if (recommendedValues.length > 1) {
        // initial and recommended values should have the same number of rows
        initialValues = Array(recommendedValues.length - 1).join('.').split('.');
      }
      initialValues.unshift(Em.I18n.t('popup.dependent.configs.table.undefined'));
      opcodes[0][0] = 'not-defined'; // class name for cell corresponding to undefined property
      opcodes[0][2] = recommendedValues.length; // specifying rows number explicitly to avoid omitting of 'Property undefined' message
    }
    if (!recommendedValues.length) {
      if (initialValues.length > 1) {
        // initial and recommended values should have the same number of rows
        recommendedValues = Array(initialValues.length - 1).join('.').split('.');
      }
      recommendedValues.unshift(Em.I18n.t('popup.dependent.configs.table.removed'));
      opcodes[0][0] = 'is-removed'; // class name for cell corresponding to removed property
      opcodes[0][4] = initialValues.length; // specifying rows number explicitly to avoid omitting of 'Property removed' message
    }
    return new Handlebars.SafeString(diffview.buildView({
      baseTextLines: initialValues,
      newTextLines: recommendedValues,
      opcodes: opcodes
    }).outerHTML);
  }.property('config.initialValues', 'config.recommendedValues')
});

});

require.register("views/common/configs/config_versions_control_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigVersionsControlView = Em.View.extend({
  templateName: require('templates/common/configs/config_versions_control'),

  serviceName: Em.computed.alias('controller.content.serviceName'),

  displayedServiceVersion: Em.computed.findBy('serviceVersions', 'isDisplayed', true),

  isCompareMode: Em.computed.notEqual('controller.compareServiceVersion', null),

  allServiceVersions: function () {
    return App.ServiceConfigVersion.find().filterProperty('serviceName', this.get('serviceName'));
  }.property('serviceName'),

  serviceVersions: function () {
    var _this = this;

    var isDefaultGroupSelected = this.get('controller.selectedConfigGroup.isDefault');
    var groupId = this.get('controller.selectedConfigGroup.id');

    var serviceVersions = this.get('allServiceVersions').filter(function (s) {
      return s.get('groupId') === groupId || isDefaultGroupSelected && s.get('groupName') === App.ServiceConfigGroup.defaultGroupName;
    });

    if (!serviceVersions.findProperty('isDisplayed')) {
      //recompute serviceVersions if displayed version absent
      Em.run.next(function () {
        return _this.propertyDidChange('controller.selectedConfigGroup.name');
      });
    }

    return serviceVersions.sort(function (a, b) {
      return Em.get(b, 'createTime') - Em.get(a, 'createTime');
    });
  }.property('controller.selectedConfigGroup.name'),

  primaryServiceVersionsInCompare: function () {
    var _this2 = this;

    return this.get('serviceVersions').filter(function (sv) {
      return sv.get('version') !== _this2.get('controller.compareServiceVersion.version');
    });
  }.property('serviceVersions', 'controller.compareServiceVersion'),

  secondaryServiceVersionsInCompare: function () {
    var _this3 = this;

    if (this.get('controller.compareServiceVersion')) {
      return this.get('serviceVersions').filter(function (serviceVersion) {
        return !serviceVersion.get('isDisplayed');
      }).map(function (serviceVersion) {
        var copy = Em.Object.create({
          version: serviceVersion.get('version'),
          stackVersion: serviceVersion.get('stackVersion'),
          authorFormatted: serviceVersion.get('authorFormatted'),
          createdDate: serviceVersion.get('createdDate'),
          fullNotes: serviceVersion.get('fullNotes'),
          isCurrent: serviceVersion.get('isCurrent')
        });
        copy.set('isDisplayed', serviceVersion.get('version') === _this3.get('controller.compareServiceVersion.version'));
        return copy;
      });
    } else {
      return [];
    }
  }.property('serviceVersions', 'controller.compareServiceVersion'),

  willInsertElement: function willInsertElement() {
    this.setDisplayVersion();
  },

  setDisplayVersion: function setDisplayVersion() {
    var serviceVersions = this.get('serviceVersions');
    var selectedVersion = this.get('controller.selectedVersion');
    serviceVersions.forEach(function (serviceVersion) {
      serviceVersion.set('isDisplayed', selectedVersion === serviceVersion.get('version'));
    });
  },

  onChangeConfigGroup: function () {
    var serviceVersions = this.get('serviceVersions');
    var selectedGroupName = this.get('controller.selectedConfigGroup.name');
    var preselectedVersion = this.get('controller.selectedVersion');

    serviceVersions.forEach(function (serviceVersion) {
      var isSelected = serviceVersion.get('version') === preselectedVersion && serviceVersion.get('groupName') === selectedGroupName;
      serviceVersion.set('isDisplayed', isSelected);
    });

    if (!serviceVersions.someProperty('isDisplayed')) {
      serviceVersions.forEach(function (serviceVersion) {
        if (serviceVersion.get('isCurrent') && serviceVersion.get('groupName') === selectedGroupName) {
          serviceVersion.set('isDisplayed', true);
        }
      });
    }
  }.observes('controller.selectedConfigGroup'),

  /**
   * switch configs view version to chosen
   */
  switchVersion: function switchVersion(event, stayInCompare) {
    var version = event.contexts[0];
    if (this.get('serviceVersions').filterProperty('isDisplayed').someProperty('version', version)) {
      return;
    }

    this.get('serviceVersions').forEach(function (serviceVersion) {
      serviceVersion.set('isDisplayed', serviceVersion.get('version') === version);
    });
    this.get('controller').loadSelectedVersion(version, null, true);
  },

  switchPrimaryInCompare: function switchPrimaryInCompare(event) {
    this.switchVersion({ contexts: [event.contexts[0].get('version')] }, true);
  },

  /**
   * add config values of chosen version to view for comparison
   * add a second version-info-bar for the chosen version
   */
  compare: function compare(event) {
    var serviceConfigVersion = event.contexts[0];
    this.set('controller.compareServiceVersion', serviceConfigVersion);

    var controller = this.get('controller');
    controller.get('stepConfigs').clear();
    controller.loadCompareVersionConfigs(controller.get('allConfigs')).done(function () {
      controller.onLoadOverrides(controller.get('allConfigs'));
    });
    if ($(event.currentTarget).hasClass('compare-button')) {
      // after entering Compare mode compare button should be destroyed before "focusOut" event, otherwise JS error will be thrown
      event.view.destroy();
    }
  },

  removeCompareVersionBar: function removeCompareVersionBar() {
    var displayedVersion = this.get('displayedServiceVersion.version');

    this.get('serviceVersions').forEach(function (serviceVersion) {
      serviceVersion.set('isDisplayed', serviceVersion.get('version') === displayedVersion);
    });
    this.get('controller').loadSelectedVersion(displayedVersion);
  },

  /**
   * revert config values to chosen version and apply reverted configs to server
   */
  makeCurrent: function makeCurrent() {
    var self = this;
    var serviceConfigVersion = this.get('displayedServiceVersion');
    var versionText = serviceConfigVersion.get('versionText');
    return App.ModalPopup.show({
      header: Em.I18n.t('dashboard.configHistory.info-bar.makeCurrent.popup.title'),
      serviceConfigNote: Em.I18n.t('services.service.config.configHistory.makeCurrent.message').format(versionText),
      bodyClass: Em.View.extend({
        templateName: require('templates/common/configs/save_configuration'),
        classNames: ['col-md-12'],
        notesArea: Em.TextArea.extend({
          classNames: ['full-width'],
          value: Em.I18n.t('services.service.config.configHistory.makeCurrent.message').format(versionText),
          onChangeValue: function () {
            this.get('parentView.parentView').set('serviceConfigNote', this.get('value'));
          }.observes('value')
        })
      }),
      primary: Em.I18n.t('dashboard.configHistory.info-bar.revert.button'),
      secondary: Em.I18n.t('common.discard'),
      third: Em.I18n.t('common.cancel'),
      onPrimary: function onPrimary() {
        serviceConfigVersion.set('serviceConfigNote', this.get('serviceConfigNote'));
        self.sendRevertCall(serviceConfigVersion);
        this.hide();
      },
      onSecondary: function onSecondary() {
        // force <code>serviceVersions</code> recalculating
        self.propertyDidChange('controller.selectedConfigGroup.name');
        this._super();
      },
      onThird: function onThird() {
        this.onSecondary();
      }
    });
  },

  /**
   * send PUT call to revert config to selected version
   * @param serviceConfigVersion
   */
  sendRevertCall: function sendRevertCall(serviceConfigVersion) {
    App.ajax.send({
      name: 'service.serviceConfigVersion.revert',
      sender: this,
      data: {
        data: {
          "Clusters": {
            "desired_service_config_versions": {
              "service_config_version": serviceConfigVersion.get('version'),
              "service_name": serviceConfigVersion.get('serviceName'),
              "service_config_version_note": serviceConfigVersion.get('serviceConfigNote')
            }
          }
        }
      },
      success: 'sendRevertCallSuccess'
    });
  },

  sendRevertCallSuccess: function sendRevertCallSuccess(data, opt, params) {
    // revert to an old version would generate a new version with latest version number,
    // so, need to loadStep to update
    this.get('controller').loadStep();
  }
});

});

require.register("views/common/configs/config_versions_dropdown_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigVersionsDropdownView = Em.View.extend({
  templateName: require('templates/common/configs/config_versions_dropdown'),
  classNames: ['btn-group'],

  searchLabel: Em.I18n.t('common.search'),

  /**
   * if true then it's secondary dropdown in Compare Mode
   * @type {boolean}
   */
  isSecondary: false,
  serviceVersions: [],
  filterValue: '',
  isCompareMode: false,
  displayedServiceVersion: Em.computed.findBy('serviceVersions', 'isDisplayed', true),

  didInsertElement: function didInsertElement() {
    var _this = this;

    this.$().on("shown.bs.dropdown", function () {
      var versionsBlock = $(_this).find('.versions-list');
      if (versionsBlock.height() < versionsBlock.prop('scrollHeight')) {
        versionsBlock.addClass('bottom-shadow');
      } else {
        versionsBlock.removeClass('bottom-shadow');
      }
      App.tooltip(_this.$('[data-toggle="tooltip"]'));
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.$('[data-toggle="tooltip"]').tooltip('destroy');
  },

  mainClickAction: function mainClickAction(event) {
    if (this.get('isSecondary')) {
      this.get('parentView').compare(event);
    } else {
      this.get('parentView').switchPrimaryInCompare(event);
    }
  },

  filteredServiceVersions: function () {
    var _this2 = this;

    return this.get('serviceVersions').filter(function (serviceVersion) {
      if (!_this2.get('filterValue').trim()) return true;
      var searchString = Em.I18n.t('common.version') + ' ' + serviceVersion.get('version') + ' ' + serviceVersion.get('notes');
      return searchString.toLowerCase().indexOf(_this2.get('filterValue').trim().toLowerCase()) !== -1;
    });
  }.property('serviceVersions.length', 'filterValue')
});

});

require.register("views/common/configs/controls_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.ControlsView = Ember.View.extend({

  classNames: ['display-inline-block', 'config-controls'],

  templateName: require('templates/common/configs/controls'),

  serviceConfigProperty: null,

  showActions: function () {
    return App.isAuthorized('SERVICE.MODIFY_CONFIGS') && this.get('serviceConfigProperty.isRequiredByAgent') && !this.get('serviceConfigProperty.isComparison');
  }.property('serviceConfigProperty.isEditable', 'serviceConfigProperty.isRequiredByAgent', 'serviceConfigProperty.isComparison'),

  showSwitchToGroup: Em.computed.and('!serviceConfigProperty.isEditable', 'serviceConfigProperty.group'),

  showIsFinal: Em.computed.and('serviceConfigProperty.supportsFinal', '!serviceConfigProperty.isUndefinedLabel'),

  showRemove: Em.computed.and('showActions', 'serviceConfigProperty.isEditable', 'serviceConfigProperty.isRemovable'),

  showOverride: Em.computed.and('showActions', 'serviceConfigProperty.isPropertyOverridable', 'controller.canEdit'),

  showUndo: Em.computed.and('showActions', 'serviceConfigProperty.isEditable', '!serviceConfigProperty.cantBeUndone', 'serviceConfigProperty.isNotDefaultValue'),

  showSetRecommended: Em.computed.and('showActions', 'serviceConfigProperty.isEditable', 'serviceConfigProperty.recommendedValueExists')

});

});

require.register("views/common/configs/custom_category_views/notification_configs_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Custom view for Notification section for MISC tab
 * Used only in the Install Wizard
 * Show configs for creating Init Notification (@see 'data/site_properties' for list of the configs used here)
 *
 * @type {App.NotificationsConfigsView}
 */
App.NotificationsConfigsView = App.ServiceConfigsByCategoryView.extend({

  templateName: require('templates/common/configs/notifications_configs'),

  /**
   * @type {string} "yes|no"
   */
  createNotification: 'no',

  categoryConfigsAll: Em.computed.filterBy('serviceConfigs', 'filename', 'alert_notification'),

  /**
   * @type {string} "tls|ssl"
   */
  tlsOrSsl: 'tls',

  /**
   * Determines if notification configs should be disabled
   * @type {boolean}
   */
  configsAreDisabled: Em.computed.equal('createNotification', 'no'),

  /**
   * Config with flag for user auth in the notification
   * @type {App.ServiceConfigProperty}
   */
  useAuthConfig: Em.computed.findBy('categoryConfigs', 'name', 'smtp_use_auth'),

  /**
   * Empty categoryConfigsAll means that user isn't at Installer, so manage notification view shouldn't be processed
   * @method didInsertElement
   */
  didInsertElement: function didInsertElement() {
    this._super();
    if (!this.get('categoryConfigsAll.length')) return;
    this.set('createNotification', this.get('categoryConfigsAll').findProperty('name', 'create_notification').get('value'));
    this.set('tlsOrSsl', this.get('categoryConfigsAll').findProperty('name', 'mail.smtp.starttls.enable').get('value') ? 'tls' : 'ssl');
    var smtpUseAuth = this.get('categoryConfigsAll').findProperty('name', 'smtp_use_auth');
    smtpUseAuth.set('value', Boolean(smtpUseAuth.get('value') === 'true'));
    this.updateCategoryConfigs();
  },

  /**
   * If <code>tlsOrSsl</code> was changed two configs should be updated
   * TLS and SSL can't be enabled in the same time
   * @method onTlsOrSslChanged
   */
  onTlsOrSslChanged: function () {
    var tlsOrSsl = this.get('tlsOrSsl');
    this.get('categoryConfigsAll').findProperty('name', 'mail.smtp.starttls.enable').set('value', tlsOrSsl === 'tls');
    this.get('categoryConfigsAll').findProperty('name', 'mail.smtp.startssl.enable').set('value', tlsOrSsl === 'ssl');
  }.observes('tlsOrSsl'),

  /**
   * Update username and password fields state according to <code>useAuthConfig.value</code>
   * @method onUseAuthConfigChange
   */
  onUseAuthConfigChange: function () {
    var configsToUpdate = ['ambari.dispatch.credential.username', 'ambari.dispatch.credential.password'],
        useAuthConfigValue = this.get('useAuthConfig.value'),
        useAuthConfigIsEditable = this.get('useAuthConfig.isEditable');

    this.getWithDefault('categoryConfigs', []).forEach(function (config) {
      if (configsToUpdate.contains(config.get('name'))) {
        var flag = useAuthConfigIsEditable ? useAuthConfigValue : false;
        this.updateConfig(config, flag);
      }
    }, this);
  }.observes('useAuthConfig.value'),

  /**
   * Update all notification fields state according to <code>createNotification</code>
   * If user select to create notification, all fields become editable and required (also run validation for each)
   * If user select don't create notification, all fields become disabled, not required and valid
   * @method updateCategoryConfigs
   */
  updateCategoryConfigs: function () {
    var createNotification = this.get('createNotification');

    this.getWithDefault('categoryConfigs', []).forEach(function (config) {
      this.updateConfig(config, Boolean(createNotification === 'yes'));
    }, this);
    this.onUseAuthConfigChange();
    this.get('categoryConfigsAll').findProperty('name', 'create_notification').set('value', createNotification);
  }.observes('createNotification'),

  /**
   * According to <code>flag</code>, enable/disable <code>config</code>
   * If <code>flag</code> is true - validate <code>config</code> value
   * If <code>flag</code> is false - mark <code>config</code> as valid
   * @param {App.ServiceConfigProperty} config
   * @param {boolean} flag
   * @method updateConfig
   */
  updateConfig: function updateConfig(config, flag) {
    config.set('isRequired', flag);
    config.set('isEditable', flag);
  },

  /**
   * No sense to store config to <code>serviceConfigs</code> and <code>categoryConfigsAll</code> because
   * <code>categoryConfigsAll</code> is a subset of <code>serviceConfigs</code>
   *
   * @override
   */
  _appendConfigToCollection: function _appendConfigToCollection(serviceConfigProperty) {
    this.get('serviceConfigs').pushObject(serviceConfigProperty);
  }

});

});

require.register("views/common/configs/overriddenPropertyRow_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
// SCP means ServiceConfigProperty

var App = require('app');

App.ServiceConfigView.SCPOverriddenRowsView = Ember.View.extend({
  templateName: require('templates/common/configs/overriddenPropertyRow'),
  classNames: ['overriden-value'],
  serviceConfigProperty: null, // is passed dynamically at runtime where ever
  // we are declaring this from configs.hbs ( we are initializing this from UI )
  categoryConfigs: null, // just declared as viewClass need it

  toggleFinalFlag: function toggleFinalFlag(event) {
    var override = event.contexts[0];
    if (override.get('isNotEditable')) {
      return;
    }
    override.set('isFinal', !override.get('isFinal'));
  },

  removeOverride: function removeOverride(event) {
    // arg 1 SCP means ServiceConfigProperty
    var scpToBeRemoved = event.contexts[0];
    var scp = this.get('serviceConfigProperty');
    var overrides = scp.get('overrides');
    var controller = this.get('controller');
    var group = controller.get('selectedService.configGroups').findProperty('name', controller.get('selectedConfigGroup.name'));
    // remove override property from selectedService on installer 7-th step
    if (this.get('controller.name') === 'wizardStep7Controller') {
      group.get('properties').removeObject(scpToBeRemoved);
    }
    if (overrides) {
      overrides = overrides.without(scpToBeRemoved);
      this.set('serviceConfigProperty.overrides', overrides);
    } else {
      if (scp.get('isUserProperty')) {
        this.get('parentView.serviceConfigs').removeObject(scpToBeRemoved);
        this.get('parentView.categoryConfigsAll').removeObject(scpToBeRemoved);
      }
    }
    if (scp.get('isUndefinedLabel')) {
      scp.set('isVisible', false);
    }
  }
});

});

require.register("views/common/configs/overriddenProperty_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
// SCP means ServiceConfigProperty

var App = require('app');

App.ServiceConfigView.SCPOverriddenPropertyView = Ember.View.extend({
  templateName: require('templates/common/configs/overriddenProperty'),

  willDestroyElement: function willDestroyElement() {
    this.$('[data-toggle=tooltip]').tooltip('destroy');
  }

});

});

require.register("views/common/configs/selectable_popup_body_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Body-view for popups where shown content may be switched to textarea-view
 * @type {Ember.View}
 */
App.SelectablePopupBodyView = Em.View.extend({

  templateName: require('templates/common/selectable_popup'),

  /**
   * True - if editable textarea is visible, false - otherwise
   * @type {boolean}
   */
  textareaVisible: false,

  /**
   * Switch <code>textareaVisible</code> value
   * @method textTrigger
   */
  textTrigger: function textTrigger() {
    this.toggleProperty('textareaVisible');
  },

  /**
   * Set data to the visible textarea
   * @method putContentToTextarea
   */
  putContentToTextarea: function () {
    var content = this.get('parentView.content');
    if (this.get('textareaVisible')) {
      var wrapper = $(".task-detail-log-maintext");
      $('.task-detail-log-clipboard').html(content).css('width', wrapper.css('width')).height(wrapper.height());
      Em.run.next(function () {
        $('.task-detail-log-clipboard').select();
      });
    }
  }.observes('textareaVisible')

});

});

require.register("views/common/configs/service_config_container_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var lazyLoading = require('utils/lazy_loading');

App.ServiceConfigContainerView = Em.ContainerView.extend({

  view: null,

  lazyLoading: null,

  pushViewTimeout: null,

  didInsertElement: function didInsertElement() {
    if (this.get('controller.isInstallWizard')) {
      this.selectedServiceObserver();
    }
  },

  pushView: function pushView() {
    if (this.get('controller.selectedService')) {
      var self = this;
      var controllerRoute = 'App.router.' + this.get('controller.name');
      if (!this.get('view')) {
        this.get('childViews').pushObject(App.ServiceConfigView.create({
          templateName: require('templates/common/configs/service_config_wizard'),
          controllerBinding: controllerRoute,
          isNotEditableBinding: controllerRoute + '.isNotEditable',
          filterBinding: controllerRoute + '.filter',
          columnsBinding: controllerRoute + '.filterColumns',
          selectedServiceBinding: controllerRoute + '.selectedService',
          actionsStacked: function () {
            return this.get('controller.isInstallWizard') && this.get('supportsConfigLayout');
          }.property('controller.isInstallWizard', 'supportsConfigLayout'),
          serviceConfigsByCategoryView: Em.ContainerView.create(),
          willDestroyElement: function willDestroyElement() {
            $('.loading').append(Em.I18n.t('common.loading.eclipses'));
          },
          didInsertElement: function didInsertElement() {
            $('.loading').empty();
            this._super();
          },
          setActiveTab: function setActiveTab(event) {
            if (event.context.get('isHiddenByFilter')) return false;
            this.get('tabs').forEach(function (tab) {
              tab.set('isActive', false);
            });
            var currentTab = event.context;
            currentTab.set('isActive', true);
          }
        }));
      } else {
        this.get('childViews').pushObject(this.get('view'));
      }
      var categoriesToPush = [];
      this.get('controller.selectedService.configCategories').forEach(function (item) {

        var categoryView = item.get('isCustomView') ? item.get('customView') : App.ServiceConfigsByCategoryView;
        if (categoryView !== null) {
          categoriesToPush.pushObject(categoryView.extend({
            category: item,
            controllerBinding: controllerRoute,
            canEditBinding: 'parentView.canEdit',
            serviceBinding: controllerRoute + '.selectedService',
            serviceConfigsBinding: controllerRoute + '.selectedService.configs',
            supportsHostOverridesBinding: 'parentView.supportsHostOverrides'
          }));
        }
      });
      this.set('lazyLoading', lazyLoading.run({
        destination: self.get('childViews.lastObject.serviceConfigsByCategoryView.childViews'),
        source: categoriesToPush,
        initSize: 3,
        chunkSize: 3,
        delay: 200,
        context: this
      }));
    }
  },

  selectedServiceObserver: function () {
    if (this.get('childViews.length')) {
      var view = this.get('childViews.firstObject');
      if (view.get('serviceConfigsByCategoryView.childViews.length')) {
        view.get('serviceConfigsByCategoryView.childViews').clear();
      }
      view.removeFromParent();
      this.set('view', view);
    }
    //terminate lazy loading when switch service
    if (this.get('lazyLoading')) lazyLoading.terminate(this.get('lazyLoading'));
    this.pushViewAfterRecommendation();
  }.observes('controller.selectedService', 'controller.selectedService.redrawConfigs'),

  pushViewAfterRecommendation: function pushViewAfterRecommendation() {
    var _this = this;

    if (this.get('controller.isRecommendedLoaded')) {
      this.pushView();
    } else {
      clearTimeout(this.get('pushViewTimeout'));
      this.set('pushViewTimeout', setTimeout(function () {
        return _this.pushViewAfterRecommendation();
      }, 300));
    }
  }

});

});

require.register("views/common/configs/service_config_layout_tab_compare_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigLayoutTabCompareView = App.ServiceConfigLayoutTabView.extend({
  templateName: require('templates/common/configs/service_config_layout_tab_compare'),

  canEdit: false,
  primaryCompareVersion: null,
  secondaryCompareVersion: null,

  onToggleBlock: function onToggleBlock(event) {
    event.context.toggleProperty('isCollapsed');
    $(event.currentTarget).siblings('.panel-body').slideToggle(500);
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.get('content.sectionRows').forEach(function (row) {
      row.setEach('isCollapsed', false);
      row.setEach('isCategoryBodyVisible', Em.computed.ifThenElse('isCollapsed', 'display: none;', 'display: block;'));
    });
  }

});

});

require.register("views/common/configs/service_config_layout_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigLayoutTabView = Em.View.extend(App.ConfigOverridable, App.LoadingOverlaySupport, {

  /**
   * Determines if view is editable
   * It true - show all control-elements (undo, override, finalize etc) for each widget
   * If false - no widgets control-elements will be shown
   * Bound from template
   * @type {boolean}
   */
  canEdit: true,

  /**
   * view need some time to prepare data to display it correct
   * before that it's better not to show anything
   * @type {boolean}
   */
  dataIsReady: false,

  /**
   * @type {App.Service}
   */
  service: Em.computed.alias('controller.selectedService'),

  templateName: function () {
    var customTemplate = this.get('customTemplate');
    return customTemplate ? customTemplate : require('templates/common/configs/service_config_layout_tab');
  }.property('customTemplate'),

  customTemplate: null,

  fieldToObserve: 'controller.recommendationsInProgress',

  classNames: ['enhanced-config-tab-content'],

  checkOverlay: function () {
    this.handleFieldChanges();
  }.observes('controller.activeTab.id', 'controller.activeTab.isRendered'),

  /**
   * Prepare configs for render
   * <code>subsection.configs</code> is an array of App.StackConfigProperty, but not App.ConfigProperty,
   * so proper config-properties should be linked to the subsections.
   * @method prepareConfigProperties
   */
  prepareConfigProperties: function prepareConfigProperties() {
    var self = this;
    this.get('content.sectionRows').forEach(function (row) {
      row.forEach(function (section) {
        section.get('subsectionRows').forEach(function (subRow) {
          subRow.forEach(function (subsection) {
            self.setConfigsToContainer(subsection);
            subsection.get('subSectionTabs').forEach(function (subSectionTab) {
              self.setConfigsToContainer(subSectionTab);
            });
          });
        });
      });
    });
  },

  /**
   * changes active subsection tab
   * @param event
   */
  setActiveSubTab: function setActiveSubTab(event) {
    if (!event.context || !event.context.get('isVisible')) {
      return false;
    }
    try {
      event.context.get('subSection.subSectionTabs').setEach('isActive', false);
      event.context.set('isActive', true);
    } catch (e) {
      console.error('Can\'t update active subsection tab');
    }
  },

  didInsertElement: function didInsertElement() {
    this.set('dataIsReady', false);
    this.set('content.isConfigsPrepared', false);
    this._super();
    if (this.get('controller.isCompareMode')) {
      this.get('parentView').filterEnhancedConfigs();
    }
    this.set('content.isConfigsPrepared', true);
    this.set('dataIsReady', true);
    this._super.apply(this, arguments);
  }

});

});

require.register("views/common/configs/service_config_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigTab = Em.View.extend({

  tagName: 'li',

  didInsertElement: function didInsertElement() {
    var serviceName = this.get('controller.selectedService.serviceName');
    this.$('a[href="#' + serviceName + '"]').tab('show');
  }
});

});

require.register("views/common/configs/service_config_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServiceConfigView = Em.View.extend({

  templateName: require('templates/common/configs/service_config'),

  isRestartMessageCollapsed: false,

  isDiscardDisabled: Em.computed.or('!controller.versionLoaded', '!controller.isPropertiesChanged'),

  isSaveDisabled: Em.computed.or('controller.isSubmitDisabled', '!controller.versionLoaded', '!controller.isPropertiesChanged'),

  /**
   * Bound from parent view in the template
   * @type {string}
   */
  filter: '',

  /**
   * Determines that active tab is set during view initialize.
   * @type {boolean}
   */
  initialActiveTabIsSet: false,

  /**
   * Bound from parent view in the template
   * @type {object[]}
   */
  columns: [],

  propertyFilterPopover: [Em.I18n.t('services.service.config.propertyFilterPopover.title'), Em.I18n.t('services.service.config.propertyFilterPopover.content')],

  canEdit: true, // View is editable or read-only?

  supportsHostOverrides: function () {
    switch (this.get('controller.name')) {
      case 'wizardStep7Controller':
        return this.get('controller.selectedService.serviceName') !== 'MISC';
      case 'mainServiceInfoConfigsController':
      case 'mainHostServiceConfigsController':
        return true;
      default:
        return false;
    }
  }.property('controller.name', 'controller.selectedService'),

  showSavePanel: function () {
    return this.get('isOnTheServicePage') && !this.get('controller.isCompareMode') && this.get('controller.selectedVersionRecord.isCurrent') && !this.get('controller.isHostsConfigsPage');
  }.property('isOnTheServicePage', 'controller.isCompareMode', 'controller.selectedVersionRecord.isCurrent', 'controller.isHostsConfigsPage'),

  /**
   * Determines if user is on the service configs page
   * @type {boolean}
   */
  isOnTheServicePage: Em.computed.equal('controller.name', 'mainServiceInfoConfigsController'),

  classNameBindings: ['isOnTheServicePage:serviceConfigs', 'controller.isCompareMode:settings-compare-layout'],

  /**
   * flag defines if any config match filter
   * true if all configs should be hidden
   * @type {boolean}
   */
  isAllConfigsHidden: false,

  /**
   * method that runs <code>updateFilterCounters<code> to
   * update filter counters for advanced tab
   * @method showHideAdvancedByFilter
   */
  showHideAdvancedByFilter: function () {
    Em.run.once(this, 'updateFilterCounters');
  }.observes('controller.selectedService.configs.@each.isHiddenByFilter'),

  /**
   * save configuration
   * @return {object}
   */
  save: function save() {
    var controller = this.get('controller');
    var passwordWasChanged = this.get('controller.passwordConfigsAreChanged');
    return App.ModalPopup.show({
      header: Em.I18n.t('dashboard.configHistory.info-bar.save.popup.title'),
      serviceConfigNote: '',
      bodyClass: Em.View.extend({
        templateName: require('templates/common/configs/save_configuration'),
        classNames: ['col-md-12'],
        showPasswordChangeWarning: passwordWasChanged,
        notesArea: Em.TextArea.extend({
          classNames: ['full-width'],
          value: passwordWasChanged ? Em.I18n.t('dashboard.configHistory.info-bar.save.popup.notesForPasswordChange') : '',
          placeholder: Em.I18n.t('dashboard.configHistory.info-bar.save.popup.placeholder'),
          didInsertElement: function didInsertElement() {
            if (this.get('value')) {
              this.onChangeValue();
            }
          },
          onChangeValue: function () {
            this.get('parentView.parentView').set('serviceConfigNote', this.get('value'));
          }.observes('value')
        })
      }),
      footerClass: Em.View.extend({
        templateName: require('templates/main/service/info/save_popup_footer')
      }),
      primary: Em.I18n.t('common.save'),
      secondary: Em.I18n.t('common.cancel'),
      onSave: function onSave() {
        var newVersionToBeCreated = Math.max.apply(null, App.ServiceConfigVersion.find().mapProperty('version')) + 1;
        controller.setProperties({
          saveConfigsFlag: true,
          serviceConfigVersionNote: this.get('serviceConfigNote'),
          preSelectedConfigVersion: Em.Object.create({
            version: newVersionToBeCreated,
            serviceName: controller.get('content.serviceName'),
            groupName: controller.get('selectedConfigGroup.name')
          })
        });
        controller.saveStepConfigs();
        this.hide();
      },
      onDiscard: function onDiscard() {
        this.hide();
        controller.set('preSelectedConfigVersion', null);
        controller.loadStep();
      },
      onCancel: function onCancel() {
        this.hide();
      }
    });
  },

  /**
   * updates filter counters for advanced tab
   * @method updateFilterCounters
   */
  updateFilterCounters: function updateFilterCounters() {
    if (this.get('controller.selectedService.configs') && this.get('state') !== 'destroyed') {
      var categories = this.get('controller.selectedService.configCategories').mapProperty('name');
      var configsToShow = this.get('controller.selectedService.configs').filter(function (config) {
        return config.get('isHiddenByFilter') == false && categories.contains(config.get('category')) && config.get('isVisible');
      });
      var isAllConfigsHidden = configsToShow.get('length') == 0;
      var isAdvancedHidden = isAllConfigsHidden || configsToShow.filter(function (config) {
        return Em.isNone(config.get('isInDefaultTheme'));
      }).get('length') == 0;
      this.set('isAllConfigsHidden', isAllConfigsHidden);
      var advancedTab = App.Tab.find().filterProperty('serviceName', this.get('controller.selectedService.serviceName')).findProperty('isAdvanced');
      advancedTab && advancedTab.set('isAdvancedHidden', isAdvancedHidden);
    }
  },

  /**
   * Check for layout config supports.
   * @returns {Boolean}
   */
  supportsConfigLayout: function () {
    var supportedControllers = ['wizardStep7Controller', 'mainServiceInfoConfigsController', 'mainHostServiceConfigsController'];
    if (App.Tab.find().rejectProperty('isCategorized').someProperty('serviceName', this.get('controller.selectedService.serviceName')) && supportedControllers.contains(this.get('controller.name'))) {
      return !Em.isEmpty(App.Tab.find().rejectProperty('isCategorized').filterProperty('serviceName', this.get('controller.selectedService.serviceName')).filterProperty('isAdvanced', false));
    } else {
      return false;
    }
  }.property('controller.name', 'controller.selectedService'),

  toggleRestartMessageView: function toggleRestartMessageView() {
    this.$('.service-body').toggle('blind', 200);
    this.set('isRestartMessageCollapsed', !this.get('isRestartMessageCollapsed'));
  },

  didInsertElement: function didInsertElement() {
    if (this.get('isNotEditable') === true) {
      this.set('canEdit', false);
    }
    if (this.$('.service-body')) {
      this.$('.service-body').hide();
    }
    App.tooltip($(".restart-required-property"), { html: true });
    App.tooltip($(".glyphicon .glyphicon-lock"), { placement: 'right' });
    App.tooltip($("[rel=tooltip]"));
    this.checkCanEdit();
    this.set('filter', '');
  },

  willDestroyElement: function willDestroyElement() {
    //Force configs remove in order to speed up rendering
    this.$().detach().remove();
    this._super();
  },

  /**
   * Check if we should show Custom Property category
   * @method checkCanEdit
   */
  checkCanEdit: function () {
    var controller = this.get('controller');
    if (!controller.get('selectedService.configCategories')) {
      return;
    }

    if (controller.get('selectedConfigGroup')) {
      controller.get('selectedService.configCategories').filterProperty('siteFileName').forEach(function (config) {
        var supportsAddingForbidden = App.config.shouldSupportAddingForbidden(controller.get('selectedService').serviceName, config.siteFileName); //true if the UI should not display the Custom ... section
        config.set('customCanAddProperty', !supportsAddingForbidden);
      });
    }
  }.observes('controller.selectedConfigGroup.name'),

  setActiveTab: function setActiveTab(event) {
    if (event.context.get('isHiddenByFilter')) return false;
    this.set('initialActiveTabIsSet', true);
    this.get('tabs').forEach(function (tab) {
      tab.set('isActive', false);
    });
    var currentTab = event.context;
    currentTab.set('isActive', true);
    currentTab.set('isRendered', true);
  },

  /**
   * Object that used for Twitter Bootstrap tabs markup.
   *
   * @returns {Ember.A}
   */
  tabs: function () {
    var tabs = App.Tab.find().rejectProperty('isCategorized').filterProperty('serviceName', this.get('controller.selectedService.serviceName'));
    var advancedTab = tabs.findProperty('isAdvanced', true);
    if (advancedTab) {
      advancedTab.set('isRendered', advancedTab.get('isActive'));
    }
    return tabs;
  }.property('controller.selectedService.serviceName'),

  /**
   * Set active tab when view attached and configs are linked to tabs.
   */
  initialActiveTabObserver: function () {
    var tabs = this.get('tabs').filterProperty('isAdvanced', false);
    if (tabs.everyProperty('isConfigsPrepared', true) && !this.get('initialActiveTabIsSet')) {
      this.pickActiveTab(this.get('tabs'));
      this.set('initialActiveTabIsSet', true);
    }
  }.observes('tabs.@each.isConfigsPrepared'),

  /**
   * Pick the first non hidden tab and make it active when there is no active tab
   * @method pickActiveTab
   */
  pickActiveTab: function pickActiveTab(tabs) {
    if (!tabs) return;
    var activeTab = tabs.findProperty('isActive', true);
    if (activeTab) {
      if (activeTab.get('isHiddenByFilter')) {
        activeTab.set('isActive', false);
        this.pickActiveTab(tabs);
      }
    } else {
      var firstHotHiddenTab = tabs.filterProperty('isHiddenByFilter', false).get('firstObject');
      if (firstHotHiddenTab) {
        firstHotHiddenTab.set('isActive', true);
        if (firstHotHiddenTab.get('isAdvanced') && !firstHotHiddenTab.get('isRendered')) {
          firstHotHiddenTab.set('isRendered', true);
        }
      }
    }
  },

  /**
   * Mark isHiddenByFilter flag for configs, sub-sections, and tab
   * @method filterEnhancedConfigs
   */
  filterEnhancedConfigs: function () {
    if (!this.get('controller.selectedService') || this.get('state') === 'destroyed') return true;
    var self = this;

    var serviceConfigs = this.get('controller.selectedService.configs').filterProperty('isVisible', true);
    var filter = (this.get('filter') || '').toLowerCase();
    var selectedFilters = (this.get('columns') || []).filterProperty('selected');

    if (selectedFilters.length > 0 || filter.length > 0) {
      serviceConfigs.forEach(function (config) {
        var passesFilters = true;

        selectedFilters.forEach(function (filter) {
          if (config.get(filter.attributeName) !== filter.attributeValue && !(config.get('overrides') && config.get('overrides').someProperty(filter.attributeName, filter.attributeValue))) {
            passesFilters = false;
          }
        });

        if (!passesFilters) {
          config.set('isHiddenByFilter', true);
          return false;
        }

        var searchString = config.get('savedValue') + config.get('description') + config.get('displayName') + config.get('name') + config.get('value') + config.getWithDefault('stackConfigProperty.displayName', '');

        if (config.get('overrides')) {
          config.get('overrides').forEach(function (overriddenConf) {
            searchString += overriddenConf.get('value') + overriddenConf.get('group.name');
          });
        }

        if (filter != null && typeof searchString === "string") {
          config.set('isHiddenByFilter', !(searchString.toLowerCase().indexOf(filter) > -1));
        } else {
          config.set('isHiddenByFilter', false);
        }
      });
    } else {
      serviceConfigs.setEach('isHiddenByFilter', false);
    }

    Em.run.next(function () {
      self.pickActiveTab(self.get('tabs'));
    });
  }.observes('filter', 'columns.@each.selected', 'tabs.@each.isHiddenByFilter')
});

});

require.register("views/common/configs/service_configs_by_category_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var validator = require('utils/validator');
require('utils/configs/modification_handlers/modification_handler');

App.ServiceConfigsByCategoryView = Em.View.extend(App.Persist, App.ConfigOverridable, App.WizardMiscPropertyChecker, {

  templateName: require('templates/common/configs/service_config_category'),

  classNames: ['panel-group', 'common-config-category', 'clear'],

  classNameBindings: ['isShowBlock::hidden'],

  'data-qa': function () {
    return this.get('category.name') + ' ' + 'panel-group';
  }.property('category.name'),

  content: null,

  category: null,

  service: null,

  /**
   * View is editable or read-only?
   * @type {boolean}
   */
  canEdit: true,

  /**
   * All configs for current <code>service</code>
   * @type {App.ServiceConfigProperty[]}
   */
  serviceConfigs: null,
  isUIDGIDVisible: true,

  /**
   * This is array of all the properties which apply
   * to this category, irrespective of visibility. This
   * is helpful in Oozie/Hive database configuration, where
   * MySQL etc. database options don't show up, because
   * they were not visible initially.
   * @type {App.ServiceConfigProperty[]}
   */
  categoryConfigsAll: [],

  /**
   * Configs for current category filtered by <code>isVisible</code>
   * and sorted by <code>displayType</code> and <code>index</code>
   * @type {App.ServiceConfigProperty[]}
   */
  categoryConfigs: [],

  /**
   * Link to main configs view
   * @type {Ember.View}
   */
  mainView: function () {
    //todo: Get rid of this logic. Get data from controller instead.
    return ['mainHostServiceConfigsController', 'mainServiceInfoConfigsController'].contains(this.get('controller.name')) ? this.get('parentView.parentView') : this.get('parentView');
  }.property('controller.name'),

  didInsertElement: function didInsertElement() {
    var self = this;
    // If `this.categoryConfigsAll` is a computed property then don't set it.
    // some extended class like `App.NotificationsConfigsView` overrides `categoryConfigsAll` as computed property
    Em.run.next(function () {
      if (self.get('state') !== 'inDOM') {
        return;
      }
      if ($.isArray(self.categoryConfigsAll)) {
        self.setCategoryConfigsAll();
      }
      self.setVisibleCategoryConfigs();
      var isCollapsed = self.calcIsCollapsed();
      self.set('category.isCollapsed', isCollapsed);
      if (isCollapsed) {
        self.$('.accordion-body').hide();
      } else {
        self.$('.accordion-body').show();
      }
      $('#serviceConfig').tooltip({
        selector: '[data-toggle=tooltip]',
        placement: 'top'
      });
      self.filteredCategoryConfigs();
      self.updateReadOnlyFlags();
    });
  },

  willDestroyElement: function willDestroyElement() {
    $('[data-toggle=tooltip]').tooltip('destroy');
  },

  setVisibleCategoryConfigsOnce: function () {
    if (this.get('controller.isChangingConfigAttributes')) {
      this.setVisibleCategoryConfigs();
    } else {
      Em.run.once(this, 'addConfigToCategoryConfigs');
    }
  }.observes('categoryConfigsAll.@each.isVisible'),

  setCategoryConfigsAll: function setCategoryConfigsAll() {
    // reset `categoryConfigsAll` to empty array
    this.set('categoryConfigsAll', []);
    var categoryConfigsAll = this.get('serviceConfigs').filterProperty('category', this.get('category.name'));
    if (categoryConfigsAll && categoryConfigsAll.length) {
      this.set('categoryConfigsAll', categoryConfigsAll);
    }
  },

  setVisibleCategoryConfigs: function setVisibleCategoryConfigs() {
    var orderedCategoryConfigs = this.getOrderedCategoryConfigs();
    this.set('categoryConfigs', orderedCategoryConfigs);
  },

  /**
   * This method is invoked when any change in visibility of items of `categoryConfigsAll` array happens and accordingly sets `categoryConfigs` array
   * Instead of completely resetting `categoryConfigs` array whenever this function is invoked , items are added/removed in `categoryConfigs` (which is binded in template)
   * Doing so is observed to avoid memory leak
   */
  addConfigToCategoryConfigs: function addConfigToCategoryConfigs() {
    var orderedCategoryConfigs = this.getOrderedCategoryConfigs();
    var categoryConfigs = this.get('categoryConfigs');
    var configsToAdd = [];
    var configsToRemove = [];
    // If property is added or made visible then add it to visible categoryconfigs
    orderedCategoryConfigs.forEach(function (item) {
      var isPresent = categoryConfigs.filterProperty('name', item.get('name')).findProperty('filename', item.get('filename'));
      if (!isPresent) {
        configsToAdd.pushObject(item);
      }
    }, this);

    // If property is removed or made invisible, then remove it from visible categoryconfigs
    categoryConfigs.forEach(function (item) {
      var orderedCategoryConfig = orderedCategoryConfigs.filterProperty('name', item.get('name')).findProperty('filename', item.get('filename'));
      if (!orderedCategoryConfig) {
        configsToRemove.pushObject(item);
      }
    }, this);

    categoryConfigs.pushObjects(configsToAdd);
    categoryConfigs.removeObjects(configsToRemove);
  },

  getOrderedCategoryConfigs: function getOrderedCategoryConfigs() {
    var categoryConfigsAll = this.get('categoryConfigsAll');
    var orderedCategoryConfigs = [];
    if (categoryConfigsAll) {
      var contentOrderedArray = this.orderContentAtLast(categoryConfigsAll.filterProperty('displayType', 'content')),
          contentFreeConfigs = categoryConfigsAll.filter(function (config) {
        return config.get('displayType') !== 'content';
      }),
          indexOrdered = this.sortByIndex(contentFreeConfigs);
      orderedCategoryConfigs = indexOrdered.concat(contentOrderedArray).filterProperty('isVisible', true);
    }
    return orderedCategoryConfigs;
  },

  /**
   * If added/removed a serverConfigObject, this property got updated.
   * Without this property, all serviceConfigs Objects will show up even if some was collapsed before.
   * @type {boolean}
   */
  isCategoryBodyVisible: Em.computed.ifThenElse('category.isCollapsed', 'display: none;', 'display: block;'),

  /**
   * Should we show config group or not
   * @type {boolean}
   */
  isShowBlock: function () {
    var isFilterEmpty = this.get('controller.filter') === '';
    var isFilterActive = this.get('mainView.columns') && this.get('mainView.columns').someProperty('selected');
    var isCustomPropertiesCategory = this.get('category.customCanAddProperty');
    var isCompareMode = this.get('controller.isCompareMode');
    var hasFilteredAdvancedConfigs = this.get('categoryConfigs').filter(function (config) {
      return config.get('isHiddenByFilter') === false && Em.isNone(config.get('isInDefaultTheme'));
    }, this).length > 0;
    return isCustomPropertiesCategory && !isCompareMode && isFilterEmpty && !isFilterActive || hasFilteredAdvancedConfigs;
  }.property('category.customCanAddProperty', 'categoryConfigs.@each.isHiddenByFilter', 'categoryConfigs.@each.isInDefaultTheme', 'controller.filter', 'controller.isCompareMode', 'mainView.columns.@each.selected'),

  /**
   * Re-order the configs to list content displayType properties at last in the category
   * @param {App.ServiceConfigProperty[]} categoryConfigs
   * @method orderContentAtLast
   */
  orderContentAtLast: function orderContentAtLast(categoryConfigs) {
    var contentProperties = categoryConfigs.filterProperty('displayType', 'content');
    if (!contentProperties.length) {
      return categoryConfigs;
    } else {
      return categoryConfigs.sort(function (a, b) {
        var aContent = contentProperties.someProperty('name', a.get('name'));
        var bContent = contentProperties.someProperty('name', b.get('name'));
        if (aContent && bContent) {
          return 0;
        }
        return aContent ? 1 : -1;
      });
    }
  },

  /**
   * Warn/prompt user to adjust Service props when changing user/groups in Misc
   * Is triggered when user ended editing text field
   */
  configChangeObserver: function (manuallyChangedProperty) {
    var changedProperty;
    if (manuallyChangedProperty.get("id")) {
      changedProperty = [manuallyChangedProperty];
    } else {
      changedProperty = this.get("serviceConfigs").filterProperty("editDone", true);
    }

    if (changedProperty.length > 0) {
      changedProperty = changedProperty.objectAt(0);
    } else {
      return;
    }

    var stepConfigs = this.get("controller.stepConfigs");
    var serviceId = this.get('controller.selectedService.serviceName');
    return this.showConfirmationDialogIfShouldChangeProps(changedProperty, stepConfigs, serviceId);
  }.observes('categoryConfigs.@each.editDone'),

  /**
   * When the view is in read-only mode, it marks
   * the properties as read-only.
   */
  updateReadOnlyFlags: function updateReadOnlyFlags() {
    var configs = this.get('serviceConfigs');
    var canEdit = this.get('canEdit');
    if (!canEdit && configs) {
      configs.forEach(function (c) {
        c.set('isEditable', false);
        var overrides = c.get('overrides');
        if (overrides != null) {
          overrides.setEach('isEditable', false);
        }
      });
    }
  },

  /**
   * Filtered <code>categoryConfigs</code> array. Used to show filtered result
   * @method filteredCategoryConfigs
   */
  filteredCategoryConfigs: function () {
    Em.run.once(this, 'collapseCategory');
  }.observes('serviceConfigs.@each.isHiddenByFilter'),

  collapseCategory: function collapseCategory() {
    if (this.get('state') === 'destroyed') return;
    $('.popover').remove();
    var filter = this.get('mainView.filter');
    filter = filter ? filter.toLowerCase() : filter; // filter can be undefined in some wizard
    var filteredResult = this.get('categoryConfigs');
    var isInitialRendering = !arguments.length || arguments[1] != 'categoryConfigs';

    filteredResult = filteredResult.filterProperty('isHiddenByFilter', false);
    filteredResult = this.sortByIndex(filteredResult);

    if (filter) {
      if (filteredResult.length && typeof this.get('category.collapsedByDefault') === 'undefined') {
        // Save state
        this.set('category.collapsedByDefault', this.get('category.isCollapsed'));
      }
      this.set('category.isCollapsed', !filteredResult.length);
    } else if (typeof this.get('category.collapsedByDefault') !== 'undefined') {
      // If user clear filter -- restore defaults
      this.set('category.isCollapsed', this.get('category.collapsedByDefault'));
      this.set('category.collapsedByDefault', undefined);
    } else if (isInitialRendering && !filteredResult.length) {
      this.set('category.isCollapsed', true);
    }
    var classNames = this.get('category.name').split(' ');
    // Escape the dots in category names
    classNames = classNames.map(function (className) {
      if (className.indexOf(".")) {
        className = className.split(".").join("\\.");
      }
      return className;
    });
    var categoryBlock = $('.' + classNames.join('.') + '>.panel-body');
    this.get('category.isCollapsed') ? categoryBlock.hide() : categoryBlock.show();
  },

  /**
   * sort configs in current category by index
   * @param configs
   * @return {Array}
   * @method sortByIndex
   */
  sortByIndex: function sortByIndex(configs) {
    var sortedConfigs = [];
    var unSorted = [];
    if (!configs.someProperty('index')) {
      return configs;
    }
    configs.forEach(function (config) {
      var index = config.get('index');
      if (index !== null && isFinite(index)) {
        sortedConfigs[index] ? sortedConfigs.splice(index, 0, config) : sortedConfigs[index] = config;
      } else {
        unSorted.push(config);
      }
    });
    // remove undefined elements from array
    sortedConfigs = sortedConfigs.filter(function (config) {
      return config !== undefined;
    });
    return sortedConfigs.concat(unSorted);
  },

  /**
   * Onclick handler for Config Group Header. Used to show/hide block
   * @method onToggleBlock
   */
  onToggleBlock: function onToggleBlock() {
    this.$('.panel-body').toggle('blind', 500);
    this.toggleProperty('category.isCollapsed');
  },

  /**
   * Determines should panel be collapsed by default
   * @returns {boolean}
   * @method calcIsCollapsed
   */
  calcIsCollapsed: function calcIsCollapsed() {
    return Em.isNone(this.get('category.isCollapsed')) ? this.get('category.name').indexOf('Advanced') != -1 || this.get('category.name').indexOf('CapacityScheduler') != -1 || this.get('category.name').indexOf('Custom') != -1 : this.get('category.isCollapsed');
  },

  /**
   * @returns {string}
   */
  persistKey: function persistKey() {
    return 'admin-bulk-add-properties-' + App.router.get('loginName');
  },

  isSecureConfig: function isSecureConfig(configName, filename) {
    var secureConfigs = App.config.get('secureConfigs').filterProperty('filename', filename);
    return !!secureConfigs.findProperty('name', configName);
  },

  createProperty: function createProperty(propertyObj) {
    var config;
    var selectedConfigGroup = this.get('controller.selectedConfigGroup');
    var categoryConfigsAll = this.get('categoryConfigsAll');
    if (selectedConfigGroup.get('isDefault')) {
      config = App.config.createDefaultConfig(propertyObj.name, propertyObj.filename, false, {
        value: propertyObj.value,
        propertyType: propertyObj.propertyType,
        category: propertyObj.categoryName,
        isNotSaved: true
      });
    } else {
      config = App.config.createCustomGroupConfig({
        propertyName: propertyObj.name,
        filename: propertyObj.filename,
        value: propertyObj.value,
        propertyType: propertyObj.propertyType,
        category: propertyObj.categoryName,
        isNotSaved: true
      }, selectedConfigGroup);
    }
    var serviceConfigProperty = App.ServiceConfigProperty.create(config);
    var duplicatedProperty = categoryConfigsAll.findProperty('name', config.name);
    if (duplicatedProperty && duplicatedProperty.get('isUndefinedLabel')) {
      serviceConfigProperty.set('overrides', duplicatedProperty.get('overrides'));
      categoryConfigsAll.removeAt(categoryConfigsAll.indexOf(duplicatedProperty));
    }
    this._appendConfigToCollection(serviceConfigProperty);
  },

  /**
   * @param {App.ServiceConfigProperty} serviceConfigProperty
   * @private
   */
  _appendConfigToCollection: function _appendConfigToCollection(serviceConfigProperty) {
    this.get('serviceConfigs').pushObject(serviceConfigProperty);
    this.get('categoryConfigsAll').pushObject(serviceConfigProperty);
    this.setVisibleCategoryConfigs();
  },

  /**
   * Find duplications within the same confType
   * Result in error, as no duplicated property keys are allowed in the same configType
   * */
  isDuplicatedConfigKey: function isDuplicatedConfigKey(name) {
    var category = this.get('category');
    var siteFileName = category.get('siteFileName');

    var service = this.get('service');
    var serviceName = service.get('serviceName');

    var configsOfFile = service.get('configs').filterProperty('filename', siteFileName);
    var duplicatedProperty = configsOfFile.findProperty('name', name);
    return duplicatedProperty && !duplicatedProperty.get('isUndefinedLabel');
  },

  /**
   * find duplications in all confTypes of the service
   * Result in warning, to remind user the existence of a same-named property
   * */
  isDuplicatedConfigKeyinConfigs: function isDuplicatedConfigKeyinConfigs(name) {
    var files = [];
    var service = this.get('service');
    var configFiles = service.get('configs').mapProperty('filename').uniq();
    configFiles.forEach(function (configFile) {
      var configsOfFile = service.get('configs').filterProperty('filename', configFile);
      var duplicatedProperty = configsOfFile.findProperty('name', name);
      if (duplicatedProperty && !duplicatedProperty.get('isUndefinedLabel')) {
        files.push(configFile);
      }
    }, this);
    return files;
  },

  processAddPropertyWindow: function processAddPropertyWindow(isBulkMode, modePersistKey) {
    var self = this;
    var category = this.get('category');
    var siteFileName = category.get('siteFileName');

    var service = this.get('service');
    var serviceName = service.get('serviceName');

    var serviceConfigObj = Em.Object.create({
      isBulkMode: isBulkMode,
      bulkConfigValue: '',
      bulkConfigError: false,
      bulkConfigErrorMessage: '',

      name: '',
      value: '',
      propertyType: [],
      content: ['PASSWORD', 'USER', 'GROUP', 'TEXT', 'ADDITIONAL_USER_PROPERTY', 'NOT_MANAGED_HDFS_PATH', 'VALUE_FROM_PROPERTY_FILE'],
      isKeyError: false,
      showFilterLink: false,
      errorMessage: '',
      observeAddPropertyValue: function observeAddPropertyValue() {
        var name = this.get('name').trim();
        if (name !== '') {
          if (validator.isValidConfigKey(name)) {
            if (!self.isDuplicatedConfigKey(name)) {
              //no duplication within the same confType
              var files = self.isDuplicatedConfigKeyinConfigs(name);
              if (files.length > 0) {
                //still check for a warning, if there are duplications across confTypes
                this.set('showFilterLink', true);
                this.set('isKeyWarning', true);
                this.set('isKeyError', false);
                this.set('warningMessage', Em.I18n.t('services.service.config.addPropertyWindow.error.derivedKey.location').format(files.join(" ")));
              } else {
                this.set('showFilterLink', false);
                this.set('isKeyError', false);
                this.set('isKeyWarning', false);
                this.set('errorMessage', '');
              }
            } else {
              this.set('showFilterLink', true);
              this.set('isKeyError', true);
              this.set('isKeyWarning', false);
              this.set('errorMessage', Em.I18n.t('services.service.config.addPropertyWindow.error.derivedKey.infile'));
            }
          } else {
            this.set('showFilterLink', false);
            this.set('isKeyError', true);
            this.set('isKeyWarning', false);
            this.set('errorMessage', Em.I18n.t('form.validator.configKey'));
          }
        } else {
          this.set('showFilterLink', false);
          this.set('isKeyError', true);
          this.set('isKeyWarning', false);
          this.set('errorMessage', Em.I18n.t('services.service.config.addPropertyWindow.error.required'));
        }
      }
    });

    App.ModalPopup.show({
      classNames: ['common-modal-wrapper'],
      modalDialogClasses: ['modal-lg'],
      header: Em.I18n.t('installer.step7.config.addProperty'),
      primary: Em.I18n.t('add'),
      secondary: Em.I18n.t('common.cancel'),
      onPrimary: function onPrimary() {
        var propertyObj = {
          filename: siteFileName,
          serviceName: serviceName,
          categoryName: category.get('name')
        };
        if (serviceConfigObj.isBulkMode) {
          var popup = this;
          this.processConfig(serviceConfigObj.bulkConfigValue, function (error, parsedConfig) {
            if (error) {
              serviceConfigObj.set('bulkConfigError', true);
              serviceConfigObj.set('bulkConfigErrorMessage', error);
            } else {
              for (var key in parsedConfig) {
                if (parsedConfig.hasOwnProperty(key)) {
                  propertyObj.name = key;
                  propertyObj.value = parsedConfig[key];
                  self.createProperty(propertyObj);
                }
              }
              popup.hide();
            }
          });
        } else {
          serviceConfigObj.observeAddPropertyValue();
          /**
           * For the first entrance use this if (serviceConfigObj.name.trim() != '')
           */
          if (!serviceConfigObj.isKeyError) {
            propertyObj.name = serviceConfigObj.get('name').trim();
            propertyObj.value = serviceConfigObj.get('value');
            propertyObj.propertyType = serviceConfigObj.get('propertyType');
            self.createProperty(propertyObj);
            this.hide();
          }
        }
      },

      lineNumber: function lineNumber(index) {
        return Em.I18n.t('services.service.config.addPropertyWindow.error.lineNumber').format(index + 1);
      },

      processConfig: function processConfig(config, callback) {
        var lines = config.split('\n');
        var errorMessages = [];
        var parsedConfig = {};
        var propertyCount = 0;
        lines.forEach(function (line, index) {
          if (line.trim() === '') {
            return;
          }
          var delimiter = '=';
          var delimiterPosition = line.indexOf(delimiter);
          if (delimiterPosition === -1) {
            errorMessages.push(this.lineNumber(index) + Em.I18n.t('services.service.config.addPropertyWindow.error.format'));
            return;
          }
          var key = Em.Handlebars.Utils.escapeExpression(line.slice(0, delimiterPosition).trim());
          var value = line.slice(delimiterPosition + 1);
          if (validator.isValidConfigKey(key)) {
            if (!self.isDuplicatedConfigKey(key) && !(key in parsedConfig)) {
              parsedConfig[key] = value;
              propertyCount++;
            } else {
              errorMessages.push(this.lineNumber(index) + Em.I18n.t('services.service.config.addPropertyWindow.error.derivedKey.specific').format(key));
            }
          } else {
            errorMessages.push(this.lineNumber(index) + Em.I18n.t('form.validator.configKey.specific').format(key));
          }
        }, this);

        if (errorMessages.length > 0) {
          callback(errorMessages.join('<br>'), parsedConfig);
        } else if (propertyCount === 0) {
          callback(Em.I18n.t('services.service.config.addPropertyWindow.propertiesPlaceholder', parsedConfig));
        } else {
          callback(null, parsedConfig);
        }
      },
      willDestroyElement: function willDestroyElement() {
        serviceConfigObj.destroy();
        this._super();
      },
      bodyClass: Em.View.extend({
        fileName: siteFileName,
        notMisc: serviceName !== 'MISC',
        templateName: require('templates/common/configs/addPropertyWindow'),
        serviceConfigObj: serviceConfigObj,
        didInsertElement: function didInsertElement() {
          this._super();
          serviceConfigObj.addObserver('name', serviceConfigObj, 'observeAddPropertyValue');
          App.tooltip(this.$("[data-toggle=tooltip]"), {
            placement: "top"
          });
        },
        willDestroyElement: function willDestroyElement() {
          this.$().popover('destroy');
          serviceConfigObj.removeObserver('name', serviceConfigObj, 'observeAddPropertyValue');
          this.set('serviceConfigObj', null);
          this._super();
        },
        toggleBulkMode: function toggleBulkMode() {
          this.toggleProperty('serviceConfigObj.isBulkMode');
          self.postUserPref(modePersistKey, this.get('serviceConfigObj.isBulkMode'));
        },
        filterByKey: function filterByKey(event) {
          var controller = App.router.get('currentState.name') == 'configs' ? App.router.get('mainServiceInfoConfigsController') : App.router.get('wizardStep7Controller');
          controller.set('filter', this.get('serviceConfigObj.name'));
          this.get('parentView').onClose();
        }
      })
    });
  },

  /**
   * Show popup for adding new config-properties
   * @method showAddPropertyWindow
   */
  showAddPropertyWindow: function showAddPropertyWindow() {
    var persistController = this;
    var modePersistKey = this.persistKey();
    var selectedConfigGroup = this.get('controller.selectedConfigGroup');

    persistController.getUserPref(modePersistKey).then(function (data) {
      return !!data;
    }, function () {
      return false;
    }).always(function (isBulkMode) {
      persistController.processAddPropertyWindow(isBulkMode, modePersistKey);
    });
  },

  /**
   * Toggle <code>isFinal</code> for selected config-property if <code>isNotEditable</code> is false
   * @param {object} event
   * @method toggleFinalFlag
   */
  toggleFinalFlag: function toggleFinalFlag(event) {
    var serviceConfigProperty = event.contexts[0];
    if (serviceConfigProperty.get('isNotEditable')) {
      return;
    }
    serviceConfigProperty.toggleProperty('isFinal');
    serviceConfigProperty = null;
  },

  /**
   * Removes the top-level property from list of properties.
   * Should be only called on user properties.
   * @method removeProperty
   */
  removeProperty: function removeProperty(event) {
    var serviceConfigProperty = event.contexts[0];
    // push config's file name if this config was stored on server
    if (!serviceConfigProperty.get('isNotSaved')) {
      var modifiedFileNames = this.get('controller.modifiedFileNames'),
          wizardController = this.get('controller.wizardController'),
          filename = serviceConfigProperty.get('filename');
      if (modifiedFileNames && !modifiedFileNames.contains(filename)) {
        modifiedFileNames.push(serviceConfigProperty.get('filename'));
      } else if (wizardController) {
        var fileNamesToUpdate = wizardController.getDBProperty('fileNamesToUpdate') || [];
        if (!fileNamesToUpdate.contains(filename)) {
          fileNamesToUpdate.push(filename);
          wizardController.setDBProperty('fileNamesToUpdate', fileNamesToUpdate);
        }
      }
    }
    this.get('serviceConfigs').removeObject(serviceConfigProperty);
    this.get('categoryConfigsAll').removeObject(serviceConfigProperty);
    serviceConfigProperty = null;
    Em.$('body>.tooltip').remove(); //some tooltips get frozen when their owner's DOM element is removed
  },

  /**
   * Set config's value to recommended
   * @param event
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue(event) {
    var serviceConfigProperty = event.contexts[0];
    serviceConfigProperty.set('value', serviceConfigProperty.get('recommendedValue'));

    //in case of USER/GROUP fields, if they have uid/gid set, then these need to be reset to the recommended value as well
    if (serviceConfigProperty.get('ugid')) {
      serviceConfigProperty.set('ugid.value', serviceConfigProperty.get('ugid.recommendedValue'));
    }
    serviceConfigProperty = null;
  },

  /**
   * Restores given property's value to be its default value.
   * Does not update if there is no default value.
   * @method doRestoreDefaultValue
   */
  doRestoreDefaultValue: function doRestoreDefaultValue(event) {
    var serviceConfigProperty = event.contexts[0];
    var savedValue = serviceConfigProperty.get('savedValue');
    var supportsFinal = serviceConfigProperty.get('supportsFinal');
    var savedIsFinal = serviceConfigProperty.get('savedIsFinal');

    if (savedValue != null) {
      if (serviceConfigProperty.get('displayType') === 'password') {
        serviceConfigProperty.set('retypedPassword', savedValue);
      }
      serviceConfigProperty.set('changedViaUndoValue', true);
      serviceConfigProperty.set('value', savedValue);
      serviceConfigProperty.set('changedViaUndoValue', false);
    }
    if (supportsFinal) {
      serviceConfigProperty.set('isFinal', savedIsFinal);
    }
    this.configChangeObserver(serviceConfigProperty);
    serviceConfigProperty = null;
    Em.$('body>.tooltip').remove(); //some tooltips get frozen when their owner's DOM element is removed
  }

});

});

require.register("views/common/configs/services_config", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ServicesConfigView = Em.View.extend({

  templateName: require('templates/common/configs/services_config'),

  classNames: ['accordion'],

  didInsertElement: function didInsertElement() {
    if (!this.get('controller.isInstallWizard')) {
      this.get('controller').loadStep();
    } else {
      this.get('controller').selectProperService();
      this.set('controller.selectedService.isActive', true);
      this.get('controller').selectedServiceObserver();
      Em.run.next(this, function () {
        this.enableRightArrow();
      });
    }
  },

  isLeftArrowDisabled: true,

  isRightArrowDisabled: true,

  isNavArrowsHidden: Em.computed.and('isLeftArrowDisabled', 'isRightArrowDisabled'),

  enableRightArrow: function enableRightArrow() {
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    this.set('isRightArrowDisabled', container.width() >= content.width());
  },

  getScrollInterval: function getScrollInterval() {
    var INTERVAL = 300;
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    var gap = content.width() - container.width();
    var gapLeft = gap % INTERVAL;
    var totalScrollsNamber = Math.floor(gap / INTERVAL) || 1;
    return INTERVAL + Math.round(gapLeft / totalScrollsNamber) + 1;
  },

  scrollTabsLeft: function scrollTabsLeft() {
    if (!this.get('isLeftArrowDisabled')) this.scrollTabs('left');
  },

  scrollTabsRight: function scrollTabsRight() {
    if (!this.get('isRightArrowDisabled')) this.scrollTabs('right');
  },

  scrollTabs: function scrollTabs(dir) {
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    var interval = this.getScrollInterval();
    this.set('isLeftArrowDisabled', dir === 'left' && interval >= container.scrollLeft());
    this.set('isRightArrowDisabled', dir === 'right' && content.width() - container.width() <= container.scrollLeft() + interval);
    container.animate({
      scrollLeft: (dir === 'left' ? '-' : '+') + '=' + interval + 'px'
    });
  }

});

});

require.register("views/common/configs/widgets/checkbox_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');
/**
 * Checkbox config widget for enhanced configs.
 * @type {Em.View}
 */
App.CheckboxConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/checkbox_config_widget'),
  classNames: ['widget-config', 'checkbox-widget'],

  configViewId: null,
  didInsertElement: function didInsertElement() {
    var self = this;
    this.initPopover();
    this._super(arguments);
  },

  configView: App.ServiceConfigCheckbox.extend({
    containerClassName: '',
    serviceConfigBinding: 'parentView.config',
    checkboxIdBinding: 'parentView.configViewId',
    // @TODO maybe find use case of this method for widget
    focusIn: Em.K,
    toggleValue: function () {
      this._super();
      this.sendRequestRorDependentConfigs(this.get('serviceConfig'));
    }.observes('checked')
  })
});

});

require.register("views/common/configs/widgets/combo_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dbInfo = require('data/db_properties_info') || {};
var dbUtils = require('utils/configs/database');

/**
 * Combo box widget view for config property.
 * @type {Em.View}
 */
App.ComboConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/combo_config_widget'),
  classNames: ['widget-config', 'combo-widget'],
  supportSwitchToTextBox: true,
  /**
   * Object with following structure:
   * {String} .value - value in widget format
   * {Object[]} .valuesList - map of entries and entry_labels
   *   {String} .configValue - value in config format
   *   {String} .widgetValue - value in widget format
   *
   * @property content
   * @type {Em.Object}
   */
  content: null,

  didInsertElement: function didInsertElement() {
    this.initWidget();
    this._super();
    this.toggleWidgetState();
    this.initPopover();
    this.disableSwitchToTextBox();
    this.addObserver('config.stackConfigProperty.valueAttributes.entries.[]', this, this.updateValuesList);
    this.addObserver('controller.forceUpdateBoundaries', this, this.updateValuesList);
    this.addObserver('config.value', this, this.isValueCompatibleWithWidget);
    this.addCustomMessage();
  },

  customMessageServiceMapping: [{
    serviceName: 'HIVE', dbConfigName: 'hive_database'
  }, {
    serviceName: 'RANGER', dbConfigName: 'DB_FLAVOR'
  }, {
    serviceName: 'RANGER_KMS', dbConfigName: 'DB_FLAVOR'
  }],

  addCustomMessage: function addCustomMessage() {
    // show warning alert about downloading db connector for services with databases
    if (this.isCustomMessageRequired()) {
      this.set('config.additionalView', Em.View.extend({
        template: Em.Handlebars.compile('<div class="alert alert-warning enhanced-configs">{{{view.message}}}</div>'),
        message: function () {
          var selectedDb = dbUtils.getDBType(this.get('config.value'));
          var dbData = dbInfo.dpPropertiesMap[selectedDb];
          return Em.I18n.t('services.service.config.database.msg.jdbcSetup.detailed').format(dbData.db_name, dbData.db_type, dbData.driver, dbData.driver_download_url, dbData.driver_download_url, dbData.driver_name, this.get('config.serviceName').toCapital());
        }.property('config.value'),
        config: this.get('config')
      }));
    }
  },

  isCustomMessageRequired: function isCustomMessageRequired() {
    var self = this;
    return this.get('customMessageServiceMapping').find(function (configMap) {
      return configMap['serviceName'].toLowerCase() === self.get('config.serviceName').toLowerCase() && configMap['dbConfigName'].toLowerCase() === self.get('config.name').toLowerCase();
    });
  },

  disableSwitchToTextBox: function disableSwitchToTextBox() {
    var valueAttributes = this.get('config.valueAttributes');
    if (valueAttributes && valueAttributes.hasOwnProperty('entriesEditable') && !valueAttributes.entriesEditable) {
      this.set('supportSwitchToTextBox', false);
    }
  },

  /**
   * Update options list by recommendations
   * @method updateValuesList
   */
  updateValuesList: function updateValuesList() {
    if (!this.get('content')) {
      this.set('content', Em.Object.create({}));
    }
    this.set('content.valuesList', this.convertToWidgetUnits(this.get('config.stackConfigProperty.valueAttributes')));
  },

  /**
   * Generate content for view. Set values map and current value.
   *
   * @method generateContent
   */
  initWidget: function initWidget() {
    this.set('content', Em.Object.create({}));
    this.set('content.valuesList', this.convertToWidgetUnits(this.get('config.stackConfigProperty.valueAttributes')));
    this.set('content.value', this.generateWidgetValue(this.get('config.value')));
  },

  /**
   * Generate values map according to widget/value format.
   *
   * @method convertToWidgetUnits
   * @param {Object} valueAttributes
   * @returns {Object[]} - values list map @see content.valuesList
   */
  convertToWidgetUnits: function convertToWidgetUnits(valueAttributes) {
    return Em.get(valueAttributes, 'entries').map(function (item) {
      return Em.Object.create({
        configValue: item.value,
        widgetValue: item.label || item.value
      });
    });
  },

  /**
   * Get widget value by specified config value.
   *
   * @method generateWidgetValue
   * @param {String} value - value in config property format
   * @returns {String}
   */
  generateWidgetValue: function generateWidgetValue(value) {
    if (this.isValueCompatibleWithWidget()) {
      return this.get('content.valuesList').findProperty('configValue', value).get('widgetValue');
    }
    return value;
  },

  /**
   * Get config value by specified widget value.
   *
   * @method generateConfigValue
   * @param {String} value - value in widget property format
   * @returns {String}
   */
  generateConfigValue: function generateConfigValue(value) {
    return this.get('content.valuesList').findProperty('widgetValue', value).get('configValue');
  },

  /**
   * Action to set config value.
   *
   * @method setConfigValue
   * @param {Object} e
   */
  setConfigValue: function setConfigValue(e) {
    this.set('config.value', e.context);
    this.set('content.value', this.generateWidgetValue(e.context));
    if (this.get('config.previousValue') != this.get('config.value')) {
      this.sendRequestRorDependentConfigs(this.get('config'));
    }
    this.set('config.previousValue', this.get('config.value'));
  },

  /**
   * @override App.ConfigWidgetView.restoreValue
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this.setConfigValue({ context: this.get('config.savedValue') });
    if (this.get('config.supportsFinal')) {
      this.get('config').set('isFinal', this.get('config.savedIsFinal'));
    }
  },

  /**
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue() {
    this.setConfigValue({ context: this.get('config.recommendedValue') });
    if (this.get('config.supportsFinal')) {
      this.get('config').set('isFinal', this.get('config.recommendedIsFinal'));
    }
  },

  /**
   * Delegate event from text input in combo widget to trigger dropdown
   */
  click: function click(event) {
    if (!this.get('disabled') && event.target.className.contains('ember-text-field')) {
      $(event.target).closest('.dropdown').toggleClass('open');
      return false;
    }
  },

  /**
   * This method is called when the value of the widget is changed by recommendation received from stack advisor api
   * @override
   * @method setValue
   */
  setValue: function setValue() {
    this.setConfigValue({ context: this.get('config.value') });
  },

  /**
   * switch display of config to widget
   * @override
   * @method textBoxToWidget
   */
  textBoxToWidget: function textBoxToWidget() {
    this.setValue(this.get('config.value'));
    this.set("config.showAsTextBox", false);
  },

  /**
   * Returns <code>true</code> if raw value can be used by widget or widget view is activated.
   * @override
   * @returns {Boolean}
   */
  isWidgetViewAllowed: true,

  /**
   * Initialize widget with incompatible value as textbox
   * @override
   */
  initIncompatibleWidgetAsTextBox: function initIncompatibleWidgetAsTextBox() {
    this.isValueCompatibleWithWidget();
  },

  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    var res = this._super() && this.get('content.valuesList').someProperty('configValue', this.get('config.value'));
    if (!res) {
      this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.infoMessage.wrong.value.for.combobox.widget').format(this.get('config.value')));
      return false;
    }
    this.updateWarningsForCompatibilityWithWidget('');
    return true;
  }

});

});

require.register("views/common/configs/widgets/config_text_field", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigTextField = Ember.TextField.extend({
  insertNewline: function insertNewline() {
    this.get('parentView').trigger('insertNewline');
  }
});

});

require.register("views/common/configs/widgets/config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('views/common/controls_view');
/**
 * Common view for config widgets
 * @type {Em.View}
 */
App.ConfigWidgetView = Em.View.extend(App.SupportsDependentConfigs, App.WidgetPopoverSupport, App.ConvertUnitWidgetViewMixin, App.ServiceConfigCalculateId, {

  /**
   * @type {App.ConfigProperty}
   */
  config: null,

  /**
   * @type {App.ConfigProperty}
   */
  copyFromConfig: null,

  /**
   * Determines if user hover on widget-view
   * @type {boolean}
   */
  isHover: false,

  /**
   * Determines if widget controls should be disabled
   * @type {boolean}
   */
  disabled: false,

  /**
   * Determines if widget is editable
   * It true - show all control-elements (undo, override, finalize etc) for widget
   * If false - no widget control-elements will be shown
   * Bound from template
   * @type {boolean}
   */
  canEdit: true,

  canNotEdit: Em.computed.not('canEdit'),

  /**
   * Config label class attribute. Displays validation status of config.
   * @type {string}
   */
  configLabelClass: '',

  /**
   * defines if widget should be shown
   * if not, text-field with config value or label "Undefined" should be shown
   * @type {boolean}
   */
  doNotShowWidget: Em.computed.or('isPropertyUndefined', 'config.showAsTextBox'),

  /**
   * defines if property in not defined in selected version
   * in this case "Undefined" should be shown instead of widget
   * @type {boolean}
   */
  isPropertyUndefined: Em.computed.equal('config.value', 'Undefined'),

  /**
   * Tab where current widget placed
   * Bound in the template
   * @type {App.Tab}
   */
  tab: null,

  /**
   * Section where current widget placed
   * Bound in the template
   * @type {App.Section}
   */
  section: null,

  /**
   * Subsection where current widget placed
   * Bound in the template
   * @type {App.SubSection}
   */
  subSection: null,

  /**
   * Determines if user can switch custom widget-view to the input-field
   * @type {boolean}
   */
  supportSwitchToTextBox: false,

  /**
   * @type {boolean}
   */
  showPencil: Em.computed.and('supportSwitchToTextBox', '!disabled'),

  /**
   * Alias to <code>config.isOriginalSCP</code>
   * Should be used in the templates
   * Don't use original <code>config.isOriginalSCP</code> in the widget-templates!!!
   * @type {boolean}
   */
  isOriginalSCPBinding: 'config.isOriginalSCP',

  /**
   * Check if property validation failed for overridden property in case when its value is equal to parent
   * config property.
   * @type {boolean}
   */
  isOverrideEqualityError: function () {
    return this.get('config.parentSCP') && this.get('config.parentSCP.value') == this.get('config.value');
  }.property('config.isValid'),

  /**
   * Alias to <code>config.isComparison</code>
   * Should be used in the templates
   * Don't use original <code>config.isComparison</code> in the widget-templates!!!
   * @type {boolean}
   */
  isComparisonBinding: 'config.isComparison',

  classNameBindings: ['isComparison:compare-mode', 'config.overrides.length:overridden-property'],

  issueMessage: '',

  issueView: Em.View.extend({

    tagName: 'i',

    classNames: ['glyphicon glyphicon-warning-sign'],

    classNameBindings: ['issueIconClass'],

    attributeBindings: ['issueMessage:data-original-title'],

    /**
     * @type {App.ServiceConfigProperty}
     */
    config: null,

    /**
     * @type {string}
     */
    issueIconClass: '',

    /**
     * @type {string}
     */
    issueMessage: '',

    didInsertElement: function didInsertElement() {
      App.tooltip($(this.get('element')));
      this.errorLevelObserver();
      this.addObserver('issuedConfig.warnMessage', this, this.errorLevelObserver);
      this.addObserver('issuedConfig.errorMessage', this, this.errorLevelObserver);
      this.addObserver('parentView.isPropertyUndefined', this, this.errorLevelObserver);
    },

    willDestroyElement: function willDestroyElement() {
      $(this.get('element')).tooltip('destroy');
      this.removeObserver('issuedConfig.warnMessage', this, this.errorLevelObserver);
      this.removeObserver('issuedConfig.errorMessage', this, this.errorLevelObserver);
      this.removeObserver('parentView.isPropertyUndefined', this, this.errorLevelObserver);
    },

    /**
     *
     * @method errorLevelObserver
     */
    errorLevelObserver: function errorLevelObserver() {
      var messageLevel = this.get('issuedConfig.errorMessage') ? 'ERROR' : this.get('issuedConfig.warnMessage') ? 'WARN' : 'NONE';
      if (this.get('parentView.isPropertyUndefined')) {
        messageLevel = 'NONE';
      }
      var issue = {
        ERROR: {
          iconClass: '',
          message: this.get('issuedConfig.errorMessage'),
          configLabelClass: 'text-danger'
        },
        WARN: {
          iconClass: 'warning',
          message: this.get('issuedConfig.warnMessage'),
          configLabelClass: 'text-warning'
        },
        NONE: {
          iconClass: 'hide',
          message: false,
          configLabelClass: ''
        }
      }[messageLevel];
      this.set('parentView.configLabelClass', issue.configLabelClass);
      this.set('issueIconClass', issue.iconClass);
      this.set('issueMessage', issue.message);
      this.set('parentView.issueMessage', issue.message);
    },

    /**
     * @type {App.ServiceConfigProperty}
     */
    issuedConfig: function () {
      var config = this.get('config');
      // check editable override
      if (!config.get('isEditable') && config.get('isOriginalSCP') && config.get('overrides.length') && config.get('overrides').someProperty('isEditable', true)) {
        config = config.get('overrides').findProperty('isEditable', true);
      } else if (config.get('isOriginalSCP') && config.get('isEditable')) {
        // use original config if it is not valid
        if (!config.get('isValid')) {
          return config;
          // scan overrides for non valid values and use it
        } else if (config.get('overrides.length') && config.get('overrides').someProperty('isValid', false)) {
          return config.get('overrides').findProperty('isValid', false);
        }
      }
      return config;
    }.property('config.isEditable', 'config.overrides.length')

  }),

  /**
   * Config name to display.
   * @type {String}
   */
  configLabel: Em.computed.firstNotBlank('config.stackConfigProperty.displayName', 'config.displayName', 'config.name'),

  /**
   * Error message computed in config property model
   * @type {String}
   */
  configErrorMessageBinding: 'config.errorMessage',

  /**
   * Determines if config-value was changed
   * @type {boolean}
   */
  valueIsChanged: function () {
    return !Em.isNone(this.get('config.savedValue')) && this.get('config.value') != this.get('config.savedValue');
  }.property('config.value', 'config.savedValue'),

  /**
   * Enable/disable widget state
   * @method toggleWidgetState
   */
  toggleWidgetState: function () {
    this.set('disabled', !this.get('config.isEditable'));
  }.observes('config.isEditable'),

  /**
   * Reset config-value to its default
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    var self = this;
    this.set('config.value', this.get('config.savedValue'));
    this.sendRequestRorDependentConfigs(this.get('config')).done(function () {
      self.restoreDependentConfigs(self.get('config'));
    });

    if (this.get('config.supportsFinal')) {
      this.get('config').set('isFinal', this.get('config.savedIsFinal'));
    }
    Em.$('body > .tooltip').remove();
  },

  /**
   * set <code>recommendedValue<code> to config
   * and send request to change dependent configs
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue() {
    var self = this;
    this.set('config.value', this.get('config.recommendedValue'));
    this.sendRequestRorDependentConfigs(this.get('config')).done(function () {
      if (self.get('config.value') === self.get('config.savedValue')) {
        self.restoreDependentConfigs(self.get('config'));
      }
    });

    if (this.get('config.supportsFinal')) {
      this.get('config').set('isFinal', this.get('config.recommendedIsFinal'));
    }
    Em.$('body > .tooltip').remove();
  },

  /**
   * Determines if override is allowed for <code>config</code>
   * @type {boolean}
   */
  overrideAllowed: function () {
    var config = this.get('config');
    if (!config) return false;
    return config.get('isOriginalSCP') && config.get('isPropertyOverridable') && !this.get('config.isComparison');
  }.property('config.isOriginalSCP', 'config.isPropertyOverridable', 'config.isComparison'),

  /**
   * Determines if undo is allowed for <code>config</code>
   * @type {boolean}
   */
  undoAllowed: function () {
    var config = this.get('config');
    if (!config) return false;
    if (!this.get('isOriginalSCP') || this.get('disabled')) return false;
    return !config.get('cantBeUndone') && config.get('isNotDefaultValue');
  }.property('config.cantBeUndone', 'config.isNotDefaultValue', 'isOriginalSCP', 'disabled'),

  /**
   * Determines if "final"-button should be shown
   * @type {boolean}
   */
  showFinalConfig: function () {
    var config = this.get('config');
    return config.get('isFinal') || !config.get('isNotEditable');
  }.property('config.isFinal', 'config.isNotEditable'),

  /**
   *
   * @param {{context: App.ServiceConfigProperty}} event
   * @method toggleFinalFlag
   */
  toggleFinalFlag: function toggleFinalFlag(event) {
    var configProperty = event.context;
    if (configProperty.get('isNotEditable')) {
      return;
    }
    configProperty.toggleProperty('isFinal');
  },

  /**
   * sync widget value with config value when dependent properties
   * have been loaded or changed
   * @method syncValueWithConfig
   */
  syncValueWithConfig: function () {
    this.setValue(this.get('config.value'));
  }.observes('controller.recommendationTimeStamp'),

  /**
   * defines if config has same config group as selected
   * @type {boolean}
   */
  referToSelectedGroup: function () {
    return this.get('controller.selectedConfigGroup.isDefault') && this.get('config.group') === null || this.get('controller.selectedConfigGroup.name') === this.get('config.group.name');
  }.property('controller.selectedConfigGroup.name', 'controller.selectedConfigGroup.isDefault'),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$('[data-toggle=tooltip]'), { placement: 'top' });
    App.tooltip($(this.get('element')).find('span'));
    var self = this;
    var element = this.$();
    if (element) {
      element.hover(function () {
        self.set('isHover', true);
      }, function () {
        self.set('isHover', false);
      });
    }
    this.initIncompatibleWidgetAsTextBox();
  },

  willInsertElement: function willInsertElement() {
    var configConditions = this.get('config.configConditions');
    var configAction = this.get('config.configAction');
    var isCopy = this.get('config.copy');
    if (!Em.empty(isCopy)) {
      this.bindConfigValue();
    }

    if (configConditions && configConditions.length) {
      //Add Observer to configCondition that depends on another config value
      var isConditionConfigDependent = configConditions.filterProperty('resource', 'config').length;
      if (isConditionConfigDependent) {
        this.addObserver('config.value', this, this.configValueObserverForAttributes);
      }

      if (configAction) {
        this.addObserver('config.value', this, this.configValueObserverForAction);
      }
    }
  },

  willDestroyElement: function willDestroyElement() {
    this.$('[data-toggle=tooltip]').tooltip('destroy');
    $(this.get('element')).find('span').tooltip('destroy');
    if (this.get('config.configConditions')) {
      this.removeObserver('config.value', this, this.configValueObserverForAttributes);
    }
    if (this.get('config.configAction')) {
      this.removeObserver('config.value', this, this.configValueObserverForAction);
    }
    if (this.get('copyFromConfig')) {
      this.removeObserver('copyFromConfig.value', this, this.configValueObserverForCopy);
    }
  },

  configValueObserverForAttributes: function configValueObserverForAttributes() {
    var configConditions = this.get('config.configConditions'),
        serviceName = this.get('config.serviceName'),
        serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');
    this.get('controller').updateAttributesFromConditions(configConditions, serviceConfigs, serviceName);
  },

  /**
   * This is an observer that is fired when a value of a config that is suppose to add/delete a component is changed
   * @private
   * @method {configValueObserverForAction}
   */
  configValueObserverForAction: function configValueObserverForAction() {
    var assignMasterOnStep7Controller = App.router.get('assignMasterOnStep7Controller');
    var configAction = this.get('config.configAction');
    var serviceName = this.get('config.serviceName');
    var serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');

    this.set('config.configActionComponent', null);
    var hostComponent = {
      componentName: configAction.get('componentName'),
      isClient: '',
      hostNames: [],
      action: ''
    };

    var hostComponentRecords = App.HostComponent.find().filterProperty('componentName', hostComponent.componentName);
    var stackComponentObj = App.StackServiceComponent.find(configAction.get('componentName'));

    if (stackComponentObj) {
      hostComponent.isClient = stackComponentObj.get('isClient');
    }

    if (hostComponentRecords.length) {
      hostComponent.hostNames = hostComponentRecords.mapProperty('hostName');
    }

    var isConditionTrue = App.configTheme.calculateConfigCondition(configAction.get("if"), serviceConfigs);
    var action = isConditionTrue ? configAction.get("then") : configAction.get("else");
    hostComponent.action = action;
    switch (action) {
      case 'add':
        // Disable save button until a host is selected
        var isComponentToBeInstalled = !App.HostComponent.find().someProperty('componentName', hostComponent.componentName);
        if (isComponentToBeInstalled) {
          this.set('controller.saveInProgress', true);
          assignMasterOnStep7Controller.execute(this, 'ADD', hostComponent);
        } else {
          if (hostComponent.componentName === "HIVE_SERVER_INTERACTIVE") {
            assignMasterOnStep7Controller.execute(this, 'ADD', hostComponent);
          }
          assignMasterOnStep7Controller.clearComponentsToBeDeleted(hostComponent.componentName);
        }
        break;
      case 'delete':
        assignMasterOnStep7Controller.execute(this, 'DELETE', hostComponent);
        break;
    }
  },

  bindConfigValue: function bindConfigValue() {
    var serviceName = this.get('config.serviceName');
    var fileName = this.get('config.copy').split('/')[0] + '.xml';
    var configName = this.get('config.copy').split('/')[1];

    var serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');
    var config = serviceConfigs.filterProperty('filename', fileName).findProperty('name', configName);
    this.addObserver('copyFromConfig.value', this, this.configValueObserverForCopy);
    this.set('copyFromConfig', config);
  },

  configValueObserverForCopy: function configValueObserverForCopy() {
    var copyFromConfig = this.get('copyFromConfig');
    var displayName = copyFromConfig.get('displayName');
    var value = copyFromConfig.get('value');
    var description = copyFromConfig.get('description');
    var config = this.get('config');
    config.setProperties({
      value: value,
      initialValue: value,
      displayName: displayName,
      description: description
    });
  },

  /**
   * set widget value same as config value
   * useful for widgets that work with intermediate config value, not original
   * for now used in slider widget
   * @abstract
   */
  setValue: Em.K,

  /**
   * Config group bound property. Needed for correct render process in template.
   *
   * @returns {App.ConfigGroup|Boolean}
   */
  configGroup: function () {
    return !this.get('config.group') || this.get('config.group.isDefault') ? false : this.get('config.group');
  }.property('config.group.name'),

  /**
   * switcher to display config as widget or text field
   * @method toggleWidgetView
   */
  toggleWidgetView: function toggleWidgetView() {
    if (!this.get('isWidgetViewAllowed')) {
      return false;
    }
    if (this.get('config.showAsTextBox')) {
      this.textBoxToWidget();
    } else {
      this.widgetToTextBox();
    }
  },

  /**
   * switch display of config to text field
   * @method widgetToTextBox
   */
  widgetToTextBox: function widgetToTextBox() {
    this.set("config.showAsTextBox", true);
  },

  /**
   * switch display of config to widget
   * @method textBoxToWidget
   */
  textBoxToWidget: function textBoxToWidget() {
    if (this.isValueCompatibleWithWidget()) {
      this.setValue(this.get('config.value'));
      this.set("config.showAsTextBox", false);
    }
  },

  /**
   * check if config value can be converted to config widget value
   * IMPORTANT! Each config-widget that override this method should use <code>updateWarningsForCompatibilityWithWidget</code>
   * @returns {boolean}
   */
  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    return this.get('isOverrideEqualityError') && !this.get('config.isValid') || this.get('config.isValid') || !this.get('supportSwitchToTextBox');
  },

  /**
   * Initialize widget with incompatible value as textbox
   */
  initIncompatibleWidgetAsTextBox: function initIncompatibleWidgetAsTextBox() {
    this.get('config').set('showAsTextBox', !this.isValueCompatibleWithWidget());
  },

  /**
   * Returns <code>true</code> if raw value can be used by widget or widget view is activated.
   * @returns {Boolean}
   */
  isWidgetViewAllowed: function () {
    if (!this.get('config.showAsTextBox')) {
      return true;
    }
    return this.isValueCompatibleWithWidget();
  }.property('config.value', 'config.isFinal', 'config.showAsTextBox'),

  /**
   * Used in <code>isValueCompatibleWithWidget</code>
   * Updates issue-parameters if config is in the raw-mode
   * @param {string} message empty string if value compatible with widget, error-message if value isn't compatible with widget
   * @method updateWarningsForCompatibilityWithWidget
   */
  updateWarningsForCompatibilityWithWidget: function updateWarningsForCompatibilityWithWidget(message) {
    this.setProperties({
      warnMessage: message,
      'config.warnMessage': message,
      issueMessage: message,
      configLabelClass: message ? 'text-warning' : ''
    });
  }

});

});

require.register("views/common/configs/widgets/directory_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.DirectoryConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/directory_config_widget'),
  classNames: ['widget-config', 'directory-widget'],

  disabled: Em.computed.not('config.isEditable'),

  /**
   * Control to edit value.
   *
   * @type {App.ServiceConfigTextArea}
   * @property configView
   */
  configView: App.ServiceConfigTextArea.extend({
    isPopoverEnabled: 'false',
    widthClass: 'col-md-12',
    serviceConfigBinding: 'parentView.config',
    popoverPlacement: 'top'
  }),

  didInsertElement: function didInsertElement() {
    this.initPopover();
    this._super();
    this.set('config.displayType', this.get('config.stackConfigProperty.widget.type'));
  }

});

});

require.register("views/common/configs/widgets/label_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.LabelView = App.ConfigWidgetView.extend({
  classNames: ['widget-config'],
  templateName: require('templates/common/configs/widgets/label_config_widget'),

  /**
   *
   * @type {App.ServiceConfigLabelView}
   * @property configView
   */
  configView: App.ServiceConfigLabelView.extend({
    isPopoverEnabled: 'false',
    textFieldClassName: 'col-md-12',
    serviceConfigBinding: 'parentView.config'
  })
});

});

require.register("views/common/configs/widgets/list_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var numberUtils = require('utils/number_utils');

/**
 * Template for list-option
 * @type {Em.object}
 */
var configOption = Em.Object.extend({
  label: '',
  value: '',
  description: '',
  isSelected: false,
  isDisabled: false,
  order: 0
});

/**
 * Config Widget for List
 * Usage:
 *  <code>{{view App.ListConfigWidgetView configBinding="someObject"}}</code>
 * @type {App.ConfigWidgetView}
 */
App.ListConfigWidgetView = App.ConfigWidgetView.extend({
  classNames: ['widget-config', 'list-widget'],
  supportSwitchToTextBox: true,

  /**
   * Counter used to determine order of options selection (<code>order<code>-field in the <code>configOption</code>)
   * Greater number - later selection
   * @type {number}
   */
  orderCounter: 1,

  /**
   * Maximum length of the <code>displayVal</code>
   * If its length is greater, it will cut to current value and ' ...' will be added to the end
   * @type {number}
   */
  maxDisplayValLength: 30,

  /**
   * <code>options</code> where <code>isSelected</code> is true
   * @type {configOption[]}
   */
  val: [],

  /**
   * List of options for <code>config.value</code>
   * @type {configOption[]}
   */
  options: [],

  /**
   * String with selected options labels separated with ', '
   * If result string is too long (@see maxDisplayValLength) it's cut and ' ...' is added to the end
   * If nothing is selected, default placeholder is used
   * @type {string}
   */
  displayVal: function () {
    var v = this.get('val').sortProperty('order').mapProperty('label');
    if (v.length > 0) {
      var output = v.join(', '),
          maxDisplayValLength = this.get('maxDisplayValLength');
      if (output.length > maxDisplayValLength - 3) {
        return output.substring(0, maxDisplayValLength - 3) + ' ...';
      }
      return output;
    }
    return Em.I18n.t('services.service.widgets.list-widget.nothingSelected');
  }.property('val.[]'),

  /**
   * Config-object bound on the template
   * @type {App.StackConfigProperty}
   */
  config: null,

  /**
   * Maximum number of options allowed to select (based on <code>config.valueAttributes.selection_cardinality</code>)
   * @type {number}
   */
  allowedToSelect: 1,

  /**
   * Minimum number of options needed to select (based on <code>config.valueAttributes.selection_cardinality</code>)
   * @type {number}
   */
  neededToSelect: 0,

  templateName: require('templates/common/configs/widgets/list_config_widget'),

  willInsertElement: function willInsertElement() {
    this._super();
    this.parseCardinality();
    this.calculateOptions();
    this.calculateInitVal();
  },

  didInsertElement: function didInsertElement() {
    this.initPopover();
    this._super();
    this.toggleWidgetState();
    this.addObserver('options.@each.isSelected', this, this.calculateVal);
    this.addObserver('options.@each.isSelected', this, this.checkSelectedItemsCount);
    if (this.isValueCompatibleWithWidget()) {
      this.calculateVal();
    }
    this.checkSelectedItemsCount();
    Em.run.next(function () {
      App.tooltip(this.$('[rel="tooltip"]'));
    });
  },

  /**
   * Get list of <code>options</code> basing on <code>config.valueAttributes</code>
   * <code>configOption</code> is used
   * @method calculateOptions
   */
  calculateOptions: function calculateOptions() {
    var valueAttributes = this.get('config.stackConfigProperty.valueAttributes'),
        options = [];
    valueAttributes.entries.forEach(function (entryValue) {
      options.pushObject(configOption.create({
        value: entryValue.value,
        label: entryValue.label || entryValue.value,
        description: entryValue.description || ''
      }));
    });
    this.set('options', options);
  },

  /**
   * Update options list by recommendations
   * @method updateList
   */
  updateList: function () {
    this.removeObserver('options.@each.isSelected', this, this.calculateVal);
    this.removeObserver('options.@each.isSelected', this, this.checkSelectedItemsCount);
    /**
     * This method should update options only. Observes should be removed
     * until new options will be applies, to avoid changing of config value.
     */
    this.calculateOptions();

    this.addObserver('options.@each.isSelected', this, this.calculateVal);
    this.addObserver('options.@each.isSelected', this, this.checkSelectedItemsCount);
    this.set('config.showAsTextBox', !this.isValueCompatibleWithWidget());
  }.observes('config.stackConfigProperty.valueAttributes.entries.[]', 'controller.forceUpdateBoundaries'),

  /**
   * Get initial value for <code>val</code> using calculated earlier <code>options</code>
   * Used on <code>willInsertElement</code> and when user click on "Undo"-button (to restore default value)
   * @method calculateInitVal
   */
  calculateInitVal: function calculateInitVal(configValue) {
    var config = this.get('config'),
        options = this.get('options'),
        value = configValue || config.get('value'),
        self = this,
        val = [];
    if (value !== '' && this.isOptionExist(value)) {
      if ('string' === Em.typeOf(value)) {
        value = value.split(',');
      }
      options.invoke('setProperties', { isSelected: false, isDisabled: false });
      val = value.map(function (v) {
        var option = options.findProperty('value', v.trim());
        option.setProperties({
          order: self.get('orderCounter'),
          isSelected: true
        });
        self.incrementProperty('orderCounter');
        return option;
      });
    }
    this.set('val', val);
  },

  /**
   * Get config-value basing on selected <code>options</code> sorted by <code>order</code>-field
   * Triggers on each option select/deselect
   * @method calculateVal
   */
  calculateVal: function calculateVal() {
    var val = this.get('options').filterProperty('isSelected').sortProperty('order');
    this.set('val', val);
    this.set('config.value', val.mapProperty('value').join(','));
  },

  /**
   * If user already selected maximum of allowed options, disable other options
   * If user selected less than minimum of needed options, mark config.value as invalid
   * If user deselect some option, all disabled options become enabled
   * Triggers on each option select/deselect
   * @method checkSelectedItemsCount
   */
  checkSelectedItemsCount: function checkSelectedItemsCount() {
    var allowedToSelect = this.get('allowedToSelect'),
        neededToSelect = this.get('neededToSelect'),
        currentlySelected = this.get('options').filterProperty('isSelected').length,
        selectionDisabled = allowedToSelect <= currentlySelected;
    this.get('options').filterProperty('isSelected', false).setEach('isDisabled', selectionDisabled);
    if (currentlySelected < neededToSelect) {
      this.set('config.errorMessage', 'You should select at least ' + neededToSelect + ' item(s)');
    } else {
      this.get('config').validate();
    }
  },

  /**
   * Get maximum number of options allowed to select and needed to select basing on config cardinality value
   * @method parseCardinality
   */
  parseCardinality: function parseCardinality() {
    var cardinality = this.get('config.stackConfigProperty.valueAttributes.selection_cardinality');
    this.set('allowedToSelect', numberUtils.getCardinalityValue(cardinality, true) || 1);
    this.set('neededToSelect', numberUtils.getCardinalityValue(cardinality, false) || 1);
  },

  /**
   * Option click-handler
   * toggle selection for current option and increment <code>orderCounter</code> for proper options selection order
   * @param {{context: Object}} e
   * @returns {boolean} always returns false to avoid list hiding
   */
  toggleOption: function toggleOption(e) {
    if (e.context.get('isDisabled')) return false;
    var orderCounter = this.get('orderCounter'),
        option = this.get('options').findProperty('value', e.context.get('value'));
    option.set('order', orderCounter);
    option.toggleProperty('isSelected');
    this.incrementProperty('orderCounter');
    this.sendRequestRorDependentConfigs(this.get('config'));
    return false;
  },

  /**
   * Restore config value
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this._super();
    this.setValue(this.get('config.value'));
  },

  /**
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue() {
    this._super();
    this.setValue(this.get('config.value'));
  },

  /**
   * Just a small checkbox-wrapper with modified click-handler
   * Should call <code>parentView.toggleOption</code>
   * User may click on the checkbox or on the link which wraps it, but action in both cases should be the same (<code>toggleOption</code>)
   * @type {App.CheckboxView}
   */
  checkBoxWithoutAction: App.CheckboxView.extend({
    init: function init() {
      this._super();
      this.off('change', this, this._updateElementValue);
    },
    _updateElementValue: function _updateElementValue() {
      var option = this.get('parentView.options').findProperty('value', this.get('value'));
      this.get('parentView').toggleOption({ context: option });
    }
  }),

  setValue: function setValue(value) {
    if (value && this.isOptionExist(value)) {
      this.calculateInitVal(value);
    } else {
      this.calculateInitVal();
    }
    this.set('config.showAsTextBox', !this.isValueCompatibleWithWidget());
  },

  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    var res = this._super() && this.isOptionExist(this.get('config.value'));
    if (!res) {
      this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.infoMessage.wrong.value.for.widget'));
      return false;
    }
    this.updateWarningsForCompatibilityWithWidget('');
    return true;
  },

  isOptionExist: function isOptionExist(value) {
    var isExist = true;
    if (Em.isNone(value)) {
      return !isExist;
    } else {
      value = Em.typeOf(value) == 'string' ? value.split(',') : value;
      value.forEach(function (item) {
        isExist = isExist && this.get('options').mapProperty('value').contains(item);
      }, this);
      return isExist;
    }
  }
});

});

require.register("views/common/configs/widgets/number_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.NumberConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/textfield_config_widget'),
  classNames: ['widget', 'number-widget'],

  configView: App.ServiceConfigTextField.extend({
    popoverPlacement: 'top',
    serviceConfigBinding: 'parentView.config',
    focusIn: function focusIn() {}
  }),

  didInsertElement: function didInsertElement() {
    this.set('config.displayType', this.get('config.stackConfigProperty.widget.type'));
  }

});

});

require.register("views/common/configs/widgets/overrides/config_widget_override_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ConfigWidgetOverrideView = App.ServiceConfigView.SCPOverriddenRowsView.extend({

  templateName: require('templates/common/configs/widgets/overrides/config_widget_override')

});

});

require.register("views/common/configs/widgets/password_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.PasswordConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/textfield_config_widget'),
  classNames: ['widget-config', 'password-widget'],

  configView: App.ServiceConfigPasswordField.extend({
    serviceConfigBinding: 'parentView.config',
    popoverPlacement: 'top'
  }),

  didInsertElement: function didInsertElement() {
    this._super();
    this.set('config.displayType', this.get('config.stackConfigProperty.widget.type'));
  }
});

});

require.register("views/common/configs/widgets/plain_config_text_field", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Default input control
 * @type {*}
 */

var App = require('app');
require('views/common/controls_view');

//TODO should use only "serviceConfig" binding instead of "config"
App.PlainConfigTextField = Ember.View.extend(App.SupportsDependentConfigs, App.WidgetPopoverSupport, App.WidgetValueObserver, {
  templateName: require('templates/common/configs/widgets/plain_config_text_field'),
  valueBinding: 'config.value',
  classNames: ['widget-config-plain-text-field'],
  placeholderBinding: 'config.placeholder',

  disabled: Em.computed.not('config.isEditable'),

  configLabel: Em.computed.firstNotBlank('config.stackConfigProperty.displayName', 'config.displayName', 'config.name'),

  /**
   * @type {string|boolean}
   */
  unit: function () {
    return Em.getWithDefault(this, 'config.stackConfigProperty.valueAttributes.unit', false);
  }.property('config.stackConfigProperty.valueAttributes.unit'),

  /**
   * @type {string}
   */
  displayUnit: function () {
    var unit = this.get('unit');
    if ('milliseconds' == unit) {
      unit = 'ms';
    }
    return unit;
  }.property('unit'),

  insertNewline: function insertNewline() {
    this.get('parentView').trigger('toggleWidgetView');
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.initPopover();
    this.set('config.displayType', Em.getWithDefault(this, 'config.stackConfigProperty.valueAttributes.type', 'string'));
  }

});

});

require.register("views/common/configs/widgets/radio_button_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RadioButtonConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/radio_button_config'),
  classNames: ['widget-config', 'radio-button-widget'],

  /**
   * Content for radio buttons has following structure:
   *    .value {String} - radio value attribute
   *    .label {String} - label to display along with radio button
   *    .description {String} - description for specified value
   * @type {Em.Object[]}
   * @property content
   */

  didInsertElement: function didInsertElement() {
    this.generateContent();
  },

  generateContent: function generateContent() {
    this.set('content', this.get('config.stackConfigProperty.valueAttributes.entries').map(function (item, index, collection) {
      return Em.Object.create({
        value: item,
        descripton: this.get('config.stackConfigProperty.valueAttributes.entry_descriptions.' + index),
        label: this.get('config.stackConfigProperty.valueAttributes.entry_labels.' + index)
      });
    }, this));
  }

});

});

require.register("views/common/configs/widgets/slider_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

/**
 * Slider-view for configs
 * Used to numeric values
 * Config value attributes should contain minimum and maximum limits for value
 * @type {App.ConfigWidgetView}
 */
App.SliderConfigWidgetView = App.ConfigWidgetView.extend({

  classNames: ['widget-config'],

  templateName: require('templates/common/configs/widgets/slider_config_widget'),

  supportSwitchToTextBox: true,

  /**
   * Slider-object created on the <code>initSlider</code>
   * @type {Object}
   */
  slider: null,

  /**
   * Mirror of the config-value shown in the input on the left of the slider
   * @type {number}
   */
  mirrorValue: 0,

  /**
   * Previous mirrorValue
   * @type {number}
   */
  prevMirrorValue: 0,

  /**
   * Determines if used-input <code>mirrorValue</code> is valid
   * Calculated on the <code>mirrorValueObs</code>
   * @type {boolean}
   */
  isMirrorValueValid: true,

  /**
   * Unit label to display.
   * @type {String}
   */
  unitLabel: '',

  /**
   * List of widget's properties which <code>changeBoundaries</code>-method should observe
   * @type {string[]}
   */
  changeBoundariesProperties: ['maxMirrorValue', 'widgetRecommendedValue', 'minMirrorValue', 'mirrorStep'],

  /**
   * Flag to check if value should be changed to recommended or saved.
   * @type {boolean}
   */
  isRestoring: false,

  /**
   * max allowed value transformed form config unit to widget unit
   * @type {Number}
   */
  maxMirrorValue: function () {
    var parseFunction = this.get('mirrorValueParseFunction');
    var maximum = this.getValueAttributeByGroup('maximum');
    var max = this.widgetValueByConfigAttributes(maximum);
    return parseFunction(max);
  }.property('config.stackConfigProperty.valueAttributes.maximum', 'controller.forceUpdateBoundaries'),

  /**
   * min allowed value transformed form config unit to widget unit
   * @type {Number}
   */
  minMirrorValue: function () {
    var parseFunction = this.get('mirrorValueParseFunction');
    var minimum = this.getValueAttributeByGroup('minimum');
    var min = this.widgetValueByConfigAttributes(minimum);
    return parseFunction(min);
  }.property('config.stackConfigProperty.valueAttributes.minimum', 'controller.forceUpdateBoundaries'),

  /**
   *
   * if group is default look into (ex. for maximum)
   * <code>config.stackConfigProperty.valueAttributes.maximum<code>
   * if group is not default look into
   * <code>config.stackConfigProperty.valueAttributes.{group.name}.maximum<code>
   * @param {String} attribute - name of attribute, for current moment
   * can be ["maximum","minimum","increment_step"] but allows to use other it there will be available
   * @returns {string}
   */
  getValueAttributeByGroup: function getValueAttributeByGroup(attribute) {
    var parseFunction = this.get('parseFunction');
    var configValue = this.get('config.value');
    var defaultGroupAttr = this.get('config.stackConfigProperty.valueAttributes');
    var groupAttr = this.get('configGroup') && defaultGroupAttr[this.get('configGroup.name')];
    var usedGroupAttr = groupAttr && !Em.isNone(groupAttr[attribute]) ? groupAttr : defaultGroupAttr;
    var boundary = usedGroupAttr[attribute];

    if (!this.get('referToSelectedGroup')) {
      if (attribute === 'minimum') {
        if (parseFunction(configValue) < parseFunction(boundary)) {
          return configValue;
        }
      } else if (attribute === 'maximum') {
        if (parseFunction(configValue) > parseFunction(boundary)) {
          return configValue;
        }
      }
    }
    if (isNaN(boundary) && !isNaN(configValue)) {
      if (attribute === 'minimum') {
        return isNaN(usedGroupAttr['maximum']) ? configValue : Math.min(usedGroupAttr['maximum'], configValue).toString();
      }
      if (attribute === 'maximum') {
        return isNaN(usedGroupAttr['minimum']) ? configValue : Math.max(usedGroupAttr['minimum'], configValue).toString();
      }
    }
    return boundary;
  },
  /**
   * step transformed form config units to widget units
   * @type {Number}
   */
  mirrorStep: function () {
    var parseFunction = this.get('mirrorValueParseFunction');
    var step = this.widgetValueByConfigAttributes(this.get('config.stackConfigProperty.valueAttributes.increment_step'));
    return step ? parseFunction(step) : this.get('unitType') === 'int' ? 1 : 0.1;
  }.property('config.stackConfigProperty.valueAttributes.increment_step'),

  /**
   * Default value of config property transformed according widget format
   * @returns {Number}
   */
  widgetDefaultValue: function () {
    var parseFunction = this.get('mirrorValueParseFunction');
    return parseFunction(this.widgetValueByConfigAttributes(this.get('config.savedValue')));
  }.property('config.savedValue'),

  /**
   * Default value of config property transformed according widget format
   * @returns {Number}
   */
  widgetRecommendedValue: function () {
    var parseFunction = this.get('mirrorValueParseFunction');
    return parseFunction(this.widgetValueByConfigAttributes(this.get('config.recommendedValue')));
  }.property('config.recommendedValue'),

  /**
   * unit type of widget
   * @type {String}
   */
  unitType: function () {
    var widgetUnit = this.get('config.stackConfigProperty.widget.units.length') && this.get('config.stackConfigProperty.widget.units')[0]['unit-name'].toLowerCase();
    var configUnit = this.get('config.stackConfigProperty.valueAttributes.type').toLowerCase();
    if (widgetUnit) {
      return this.get('units').indexOf(widgetUnit) > this.get('units').indexOf(configUnit) ? 'float' : this.get('config.stackConfigProperty.valueAttributes.type');
    } else {
      return 'float';
    }
  }.property('config.stackConfigProperty.widget.units.@each.unit-name'),
  /**
   * Function used to parse widget mirror value
   * For integer - parseInt, for float - parseFloat
   * @type {Function}
   */
  mirrorValueParseFunction: function () {
    return this.get('unitType') === 'int' ? parseInt : parseFloat;
  }.property('unitType'),

  /**
   * Function used to validate widget mirror value
   * For integer - validator.isValidInt, for float - validator.isValidFloat
   * @type {Function}
   */
  mirrorValueValidateFunction: function () {
    return this.get('unitType') === 'int' ? validator.isValidInt : validator.isValidFloat;
  }.property('unitType'),

  /**
   * Function used to parse config value (based on <code>config.stackConfigProperty.valueAttributes.type</code>)
   * For integer - parseInt, for float - parseFloat
   * @type {Function}
   */
  parseFunction: function () {
    return this.get('config.stackConfigProperty.valueAttributes.type') === 'int' ? parseInt : parseFloat;
  }.property('config.stackConfigProperty.valueAttributes.type'),

  /**
   * Function used to validate config value (based on <code>config.stackConfigProperty.valueAttributes.type</code>)
   * For integer - validator.isValidInt, for float - validator.isValidFloat
   * @type {Function}
   */
  validateFunction: function () {
    return this.get('config.stackConfigProperty.valueAttributes.type') === 'int' ? validator.isValidInt : validator.isValidFloat;
  }.property('config.stackConfigProperty.valueAttributes.type'),

  /**
   * Enable/disable slider state
   * @method toggleWidgetState
   */
  toggleWidgetState: function () {
    var slider = this.get('slider');
    this.get('config.isEditable') ? slider.enable() : slider.disable();
    this._super();
  }.observes('config.isEditable'),

  willInsertElement: function willInsertElement() {
    this._super();
    this.prepareValueConverter();
    this.addObserver('mirrorValue', this, this.mirrorValueObs);
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.setValue();
    this.initSlider();
    this.toggleWidgetState();
    this.initPopover();
    var self = this;
    this.get('changeBoundariesProperties').forEach(function (property) {
      self.addObserver(property, self, self.changeBoundaries);
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.$('[data-toggle=tooltip]').tooltip('destroy');
    var self = this;
    this.get('changeBoundariesProperties').forEach(function (property) {
      self.removeObserver(property, self, self.changeBoundaries);
    });
    this.removeObserver('mirrorValue', this, this.mirrorValueObs);
    if (this.get('slider')) {
      try {
        if (self.get('slider')) {
          self.get('slider').destroy();
        }
      } catch (e) {
        console.error('error while clearing slider for config: ' + self.get('config.name'));
      }
    }
  },

  mirrorValueObs: function mirrorValueObs() {
    Em.run.once(this, 'mirrorValueObsOnce');
  },

  /**
   * Check if <code>mirrorValue</code> was updated by user
   * Validate it. If value is correct, set it to slider and config.value
   * @method mirrorValueObs
   */
  mirrorValueObsOnce: function mirrorValueObsOnce() {
    var mirrorValue = this.get('mirrorValue'),
        slider = this.get('slider'),
        min = this.get('minMirrorValue'),
        max = this.get('maxMirrorValue'),
        validationFunction = this.get('mirrorValueValidateFunction'),
        parseFunction = this.get('mirrorValueParseFunction');
    if (validationFunction(mirrorValue)) {
      var parsed = parseFunction(mirrorValue);
      if (parsed > max) {
        this.set('isMirrorValueValid', false);
        this.get('config').setProperties({
          warnMessage: Em.I18n.t('config.warnMessage.outOfBoundaries.greater').format(max + this.get('unitLabel')),
          warn: true
        });
      } else if (parsed < min) {
        this.set('isMirrorValueValid', false);
        this.get('config').setProperties({
          warnMessage: Em.I18n.t('config.warnMessage.outOfBoundaries.less').format(min + this.get('unitLabel')),
          warn: true
        });
      } else {
        this.set('isMirrorValueValid', !this.get('config.error'));
        this.set('config.value', '' + this.configValueByWidget(parsed));
        if (slider) {
          slider.setValue(parsed);
        }
      }
      // avoid precision during restore value
      if (!Em.isNone(this.get('config.savedValue')) && parsed == parseFunction(this.widgetValueByConfigAttributes(this.get('config.savedValue')))) {
        this.set('config.value', this.get('config.savedValue'));
      }
      // ignore precision during set recommended value
      if (!Em.isNone(this.get('config.recommendedValue')) && parsed == parseFunction(this.widgetValueByConfigAttributes(this.get('config.recommendedValue')))) {
        this.set('config.value', this.get('config.recommendedValue'));
      }
    } else {
      this.set('isMirrorValueValid', false);
      this.set('config.errorMessage', 'Invalid value');
    }
  },

  /**
   * set widget value same as config value
   * @override
   * @method setValue
   */
  setValue: function setValue(value) {
    var parseFunction = this.get('parseFunction');
    value = value || parseFunction(this.get('config.value'));
    this.set('mirrorValue', this.widgetValueByConfigAttributes(value));
    this.set('prevMirrorValue', this.get('mirrorValue'));
  },

  /**
   * Setup convert table according to widget unit-name and property type.
   * Set label for unit to display.
   * @method prepareValueConverter
   */
  prepareValueConverter: function prepareValueConverter() {
    var widgetUnit = this._converterGetWidgetUnits();
    if (['int', 'float'].contains(this._converterGetPropertyAttributes()) && widgetUnit == 'percent') {
      this.set('currentDimensionType', 'percent.percent_' + this._converterGetPropertyAttributes());
    }
    this.set('unitLabel', Em.getWithDefault(this.get('unitLabelMap'), widgetUnit, widgetUnit));
  },

  /**
   * Draw slider for current config
   * @method initSlider
   */
  initSlider: function initSlider() {
    var self = this,
        config = this.get('config'),
        valueAttributes = config.get('stackConfigProperty.valueAttributes'),
        parseFunction = this.get('parseFunction'),
        ticks = [this.valueForTick(this.get('minMirrorValue'))],
        ticksLabels = [],
        maxMirrorValue = this.get('maxMirrorValue'),
        minMirrorValue = this.get('minMirrorValue'),
        mirrorStep = this.get('mirrorStep'),
        recommendedValue = this.valueForTick(+this.get('widgetRecommendedValue')),
        range = Math.floor((maxMirrorValue - minMirrorValue) / mirrorStep) * mirrorStep,
        isOriginalSCP = config.get('isOriginalSCP'),

    // for little odd numbers in range 4..23 and widget type 'int' use always 4 ticks
    isSmallInt = this.get('unitType') == 'int' && range > 4 && range < 23 && range % 2 == 1,
        recommendedValueMirroredId,
        recommendedValueId;

    // ticks and labels
    for (var i = 1; i <= 3; i++) {
      var val = minMirrorValue + this.valueForTickProportionalToStep(range * (i / (isSmallInt ? 3 : 4)));
      // if value's type is float, ticks may be float too
      ticks.push(this._extraRound(val));
    }

    ticks.push(this.valueForTick(maxMirrorValue));
    ticks = ticks.uniq();
    ticks.forEach(function (tick, index, items) {
      var label = '';
      if (items.length < 5 || index % 2 === 0 || items.length - 1 == index) {
        label = this.formatTickLabel(tick, ' ');
      }
      ticksLabels.push(label);
    }, this);

    ticks = ticks.uniq();

    if (!(this.get('controller.isCompareMode') && !isOriginalSCP)) {
      // default marker should be added only if recommendedValue is in range [min, max]
      if (recommendedValue <= maxMirrorValue && recommendedValue >= minMirrorValue && recommendedValue != '') {
        // process additional tick for default value if it not defined in previous computation
        if (!ticks.contains(recommendedValue)) {
          // push default value
          ticks.push(recommendedValue);
          // and resort array
          ticks = ticks.sort(function (a, b) {
            return a - b;
          });
          recommendedValueId = ticks.indexOf(recommendedValue);
          // to save nice tick labels layout we should add new tick value which is mirrored by index to default value
          recommendedValueMirroredId = ticks.length - recommendedValueId;
          // push mirrored default value behind default
          if (recommendedValueId == recommendedValueMirroredId) {
            recommendedValueMirroredId--;
          }
          // push empty label for default value tick
          ticksLabels.insertAt(recommendedValueId, '');
          // push empty to mirrored position
          ticksLabels.insertAt(recommendedValueMirroredId, '');
          // for saving correct sliding need to add value to mirrored position which is average between previous
          // and next value
          ticks.insertAt(recommendedValueMirroredId, this.valueForTick((ticks[recommendedValueMirroredId] + ticks[recommendedValueMirroredId - 1]) / 2));
          // get new index for default value
          recommendedValueId = ticks.indexOf(recommendedValue);
        } else {
          recommendedValueId = ticks.indexOf(recommendedValue);
        }
      }
    }

    /**
     * Slider some times change config value while being created,
     * this may happens when slider recreating couple times during small period.
     * To cover this situation need to reset config value after slider initializing
     * @type {String}
     */
    var correctConfigValue = this.get('config.value');

    var slider = new Slider(this.$('input.slider-input')[0], {
      value: this.get('mirrorValue'),
      ticks: ticks,
      tooltip: 'always',
      ticks_labels: ticksLabels,
      step: mirrorStep,
      formatter: function formatter(val) {
        var labelValue = Em.isArray(val) ? val[0] : val;
        return self.formatTickLabel(labelValue);
      }
    });

    /**
     * Resetting config value, look for <code>correctConfigValue<code>
     * for more info
     */
    this.set('config.value', correctConfigValue);

    slider.on('change', this.onSliderChange.bind(this)).on('slideStop', function () {
      /**
       * action to run sendRequestRorDependentConfigs when
       * we have changed config value within slider
       */
      if (self.get('prevMirrorValue') != self.get('mirrorValue')) {
        self.sendRequestRorDependentConfigs(self.get('config'));
      }
    });
    this.set('slider', slider);
    var sliderTicks = this.$('.ui-slider-wrapper:eq(0) .slider-tick');

    if (recommendedValueId) {
      sliderTicks.eq(recommendedValueId).addClass('slider-tick-default').on('mousedown', function (e) {
        if (self.get('disabled')) return false;
        self.setRecommendedValue();
        e.stopPropagation();
        return false;
      });
      // create label for default value and align it
      // defaultSliderTick.append('<span>{0}</span>'.format(recommendedValue + this.get('unitLabel')));
      // defaultSliderTick.find('span').css('marginLeft', -defaultSliderTick.find('span').width()/2 + 'px');
      // if mirrored value was added need to hide the tick for it
      if (recommendedValueMirroredId) {
        sliderTicks.eq(recommendedValueMirroredId).hide();
      }
    }
    // mark last tick to fix it style
    sliderTicks.last().addClass('last');
  },

  /**
   * Callback function triggered on slider change event.
   * Set config property and widget value with new one, or ignore changes in case value restoration executed by
   * <code>restoreValue</code>, <code>setRecommendedValue</code>.
   *
   * @param {Object} e - object that contains <code>oldValue</code> and <code>newValue</code> attributes.
   * @method onSliderChange
   */
  onSliderChange: function onSliderChange(e) {
    if (!this.get('isRestoring')) {
      var val = this.get('mirrorValueParseFunction')(e.newValue);
      this.set('config.value', '' + this.configValueByWidget(val));
      this.set('mirrorValue', val);
    } else {
      this.set('isRestoring', false);
    }
  },
  /**
   * Convert value according to property attribute unit.
   *
   * @method valueForTick
   * @param {Number} val
   * @private
   * @returns {Number}
   */
  valueForTick: function valueForTick(val) {
    return this.get('unitType') === 'int' ? Math.round(val) : this._extraRound(val);
  },

  /**
   * Convert value according to property attribute unit
   * Also returned value is proportional to the <code>mirrorStep</code>
   *
   * @param {Number} val
   * @private
   * @returns {Number}
   */
  valueForTickProportionalToStep: function valueForTickProportionalToStep(val) {
    if (this.get('unitType') === 'int') {
      return Math.round(val);
    }
    var mirrorStep = this.get('mirrorStep');
    var r = Math.round(val / mirrorStep);
    return this._extraRound(r * mirrorStep);
  },

  /**
   * Round number to 3 digits after "."
   * Used for all slider's ticks
   * @param {Number} v
   * @returns {Number} number with 3 digits after "."
   * @private
   * @method _extraRound
   */
  _extraRound: function _extraRound(v) {
    return parseFloat(v.toFixed(3));
  },

  /**
   * Restore <code>savedValue</code> for config
   * Restore <code>mirrorValue</code> too
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this._super();
    this.set('isRestoring', true);
    this.get('slider').setValue(this.get('widgetDefaultValue'));
    if (this.get('config.value') === this.get('config.savedValue')) {
      this.set('isRestoring', false);
    }
  },

  /**
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue() {
    this._super();
    this.set('isRestoring', true);
    this.get('slider').setValue(this.get('widgetRecommendedValue'));
    if (this.get('config.value') === this.get('config.recommendedValue')) {
      this.set('isRestoring', false);
    }
  },

  /**
   * Determines if config-value was changed
   * @type {boolean}
   */
  valueIsChanged: function () {
    return !Em.isNone(this.get('config.savedValue')) && this.get('parseFunction')(this.get('config.value')) != this.get('parseFunction')(this.get('config.savedValue'));
  }.property('config.value', 'config.savedValue'),

  /**
   * Run changeBoundariesOnce only once
   * @method changeBoundaries
   */
  changeBoundaries: function changeBoundaries() {
    if (this.get('config.stackConfigProperty.widget')) {
      Em.run.once(this, 'changeBoundariesOnce');
    }
  },

  /**
   * Recreate widget in case max or min values were changed
   *
   * @method changeBoundariesOnce
   */
  changeBoundariesOnce: function changeBoundariesOnce() {
    if ($.mocho) {
      //temp fix as it can broke test that doesn't have any connection with this method
      return;
    }
    if (this.get('config')) {
      try {
        if (this.get('slider')) {
          this.get('slider').destroy();
        }
        this.initIncompatibleWidgetAsTextBox();
        this.initSlider();
        this.toggleWidgetState();
        // arguments exists - means method is called as observer and not directly from other method
        // so, no need to call <code>refreshSliderObserver</code> and this prevent recursive calls
        // like changeBoundariesOnce -> refreshSliderObserver -> changeBoundariesOnce -> ...
        if (arguments.length) {
          this.refreshSliderObserver();
        }
      } catch (e) {
        console.error('error while rebuilding slider for config: ' + this.get('config.name'));
      }
    }
  },

  /**
   * Method used for initializing sliders in the next Ember run-loop
   * It's useful for overrides, because they are redrawn a little bit later than origin config
   *
   * If this method is called without arguments, it will call itself recursively using <code>changeBoundariesOnce</code>
   * and <code>refreshSliderObserver</code>
   * If not - it will just call <code>changeBoundariesOnce</code> in the next run-loop
   * @method changeBoundariesOnceLater
   */
  _changeBoundariesOnceLater: function _changeBoundariesOnceLater() {
    var self = this;
    Em.run.later('sync', function () {
      self.changeBoundariesOnce();
    }, 10);
  },

  /**
   * Workaround for bootstrap-slider widget that was initiated inside hidden container.
   * @method refreshSliderObserver
   */
  refreshSliderObserver: function () {
    var self = this;
    var sliderTickLabel = this.$('.ui-slider-wrapper:eq(0) .slider-tick-label:first');
    if (sliderTickLabel.width() == 0 && this.isValueCompatibleWithWidget()) {
      Em.run.next(function () {
        self._changeBoundariesOnceLater();
      });
    }
  }.observes('parentView.content.isActive', 'parentView.parentView.tab.isActive'),

  /**
   * Check if value provided by user in the textbox may be used in the slider
   * @returns {boolean}
   * @method isValueCompatibleWithWidget
   */
  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    if (this._super()) {
      if (!this.get('validateFunction')(this.get('config.value'))) {
        return false;
      }
      var configValue = this.get('parseFunction')(this.get('config.value'));
      if (this.get('config.stackConfigProperty.valueAttributes.minimum')) {
        var min = this.get('parseFunction')(this.getValueAttributeByGroup('minimum'));
        if (configValue < min) {
          min = this.widgetValueByConfigAttributes(min);
          this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.warnMessage.outOfBoundaries.less').format(min + this.get('unitLabel')));
          return false;
        }
      }
      if (this.get('config.stackConfigProperty.valueAttributes.maximum')) {
        var max = this.get('parseFunction')(this.getValueAttributeByGroup('maximum'));
        if (configValue > max) {
          max = this.widgetValueByConfigAttributes(max);
          this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.warnMessage.outOfBoundaries.greater').format(max + this.get('unitLabel')));
          return false;
        }
      }

      this.updateWarningsForCompatibilityWithWidget('');
      return true;
    }
    return false;
  },

  /**
   * Returns formatted value of slider label
   * @param tick - starting value
   * @param separator - will be inserted between value and unit
   * @returns {string}
   */
  formatTickLabel: function formatTickLabel(tick, separator) {
    var label,
        separator = separator || '',
        valueLabel = tick,
        units = ['B', 'KB', 'MB', 'GB', 'TB'],
        unitLabel = this.get('unitLabel'),
        unitLabelIndex = units.indexOf(unitLabel);
    if (unitLabelIndex > -1) {
      while (tick > 9999 && unitLabelIndex < units.length - 1) {
        tick /= 1024;
        unitLabelIndex++;
      }
      unitLabel = units[unitLabelIndex];
      valueLabel = this._extraRound(tick);
    }
    label = valueLabel + separator + unitLabel;
    return label;
  }

});

});

require.register("views/common/configs/widgets/string_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.StringConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/textfield_config_widget'),
  classNames: ['widget', 'string-widget'],

  /**
   * Control to edit value.
   *
   * @type {App.ServiceConfigTextArea}
   * @property configView
   */
  configView: App.ServiceConfigTextArea.extend({
    widthClass: 'col-md-12',
    serviceConfigBinding: 'parentView.config',
    popoverPlacement: 'top'
  }),

  didInsertElement: function didInsertElement() {
    this.set('config.displayType', this.get('config.stackConfigProperty.widget.type'));
  }
});

});

require.register("views/common/configs/widgets/test_db_connection_widget_view", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');
var dbUtils = require('utils/configs/database');

//@Todo merge with CheckDBConnectionView
App.TestDbConnectionWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/test_db_connection_widget'),
  classNames: ['widget'],
  dbInfo: require('data/db_properties_info'),

  /** @property {string} btnCaption - text for button **/
  btnCaption: Em.computed.alias('config.stackConfigProperty.widget.display-name'),
  /** @property {string} responseCaption - text for status link **/
  responseCaption: null,
  /** @property {boolean} isConnecting - is request to server activated **/
  isConnecting: false,
  /** @property {boolean} isValidationPassed - check validation for required fields **/
  isValidationPassed: null,
  /** @property {string} db_type- name of current database **/
  db_type: null,
  /** @property {string} db_type_label - label of current database **/
  db_type_label: null,
  /** @property {boolean} isRequestResolved - check for finished request to server **/
  isRequestResolved: false,
  /** @property {boolean} isConnectionSuccess - check for successful connection to database **/
  isConnectionSuccess: null,
  /** @property {string} responseFromServer - message from server response **/
  responseFromServer: null,
  /** @property {Object} ambariRequiredProperties - properties that need for custom action request **/
  ambariRequiredProperties: null,
  /** @property {Number} currentRequestId - current custom action request id **/
  currentRequestId: null,
  /** @property {Number} currentTaskId - current custom action task id **/
  currentTaskId: null,
  /** @property {jQuery.Deferred} request - current $.ajax request **/
  request: null,
  /** @property {Number} pollInterval - timeout interval for ajax polling **/
  pollInterval: 3000,
  /** @property {Object} logsPopup - popup with DB connection check info **/
  logsPopup: null,
  /** @property {Array} or {String} masterHostName: The name of hosts from which the db connection will happen**/
  masterHostName: null,
  /** @property {String} db_connection_url: The jdbc urlfor performing db connection**/
  db_connection_url: null,
  /** @property {String} user_name: The user name to be used for performing db connection**/
  user_name: null,
  /** @property {String} user_passwd: password for the  user name to be used for performing db connection**/
  user_passwd: null,

  someRequiredPropertyIsInvalid: Em.computed.someBy('requiredProperties', 'isValid', false),
  /** @property {boolean} isBtnDisabled - disable button on failed validation or active request **/
  isBtnDisabled: Em.computed.or('someRequiredPropertyIsInvalid', 'isConnecting'),
  /** @property {object} requiredProperties - properties that necessary for database connection **/
  requiredProperties: [],

  // define if label of selected database contains "new"
  isNewSelected: function () {
    return (/new/i.test(this.get('db_type_label.value'))
    );
  }.property('db_type_label.value'),

  /** Check validation and load ambari properties **/
  didInsertElement: function didInsertElement() {
    var requiredProperties = this.get('config.stackConfigProperty.widget.required-properties');
    var serviceName = this.get('config.serviceName');
    var serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');
    var requiredServiceConfigs = Object.keys(requiredProperties).map(function (key) {
      var split = requiredProperties[key].split('/');
      var fileName = split[0] + '.xml';
      var configName = split[1];
      var requiredConfig = serviceConfigs.filterProperty('filename', fileName).findProperty('name', configName);
      if (!requiredConfig) {
        var componentName = App.config.getComponentName(configName);
        var stackComponent = App.StackServiceComponent.find(componentName);
        if (stackComponent && stackComponent.get('componentName')) {
          var value = this.get('controller').getComponentHostValue(componentName, this.get('controller.wizardController.content.masterComponentHosts'), this.get('controller.wizardController.content.slaveComponentHosts'));
          var hProperty = App.config.createHostNameProperty(serviceName, componentName, value, stackComponent);
          return App.ServiceConfigProperty.create(hProperty);
        }
      } else {
        return requiredConfig;
      }
    }, this);

    this.set('requiredProperties', requiredServiceConfigs);
    this.setDbProperties(requiredProperties);
    this.getAmbariProperties();
  },

  /** On view destroy **/
  willDestroyElement: function willDestroyElement() {
    this.set('isConnecting', false);
    this._super();
  },

  /**
   *  This function is used to set Database name and master host name
   * @param requiredProperties: `config.stackConfigProperty.widget.required-properties` as stated in the theme
   */
  setDbProperties: function setDbProperties(requiredProperties) {
    var dbProperties = {
      'db.connection.source.host': 'masterHostName',
      'db.type': 'db_type',
      'db.connection.user': 'user_name',
      'db.connection.password': 'user_passwd',
      'jdbc.driver.url': 'db_connection_url',
      'db.type.label': 'db_type_label'
    };

    for (var key in dbProperties) {
      var masterHostNameProperty = requiredProperties[key];
      if (masterHostNameProperty) {
        var split = masterHostNameProperty.split('/');
        var fileName = split[0] + '.xml';
        var configName = split[1];
        var dbConfig = this.get('requiredProperties').filterProperty('filename', fileName).findProperty('name', configName);
        this.set(dbProperties[key], dbConfig);
      }
    }
  },

  /**
   * Set up ambari properties required for custom action request
   *
   * @method getAmbariProperties
   **/
  getAmbariProperties: function getAmbariProperties() {
    var clusterController = App.router.get('clusterController');
    var _this = this;
    if (!App.isEmptyObject(App.db.get('tmp', 'ambariProperties')) && !this.get('ambariProperties')) {
      this.set('ambariProperties', App.db.get('tmp', 'ambariProperties'));
      return;
    }
    if (App.isEmptyObject(clusterController.get('ambariProperties'))) {
      clusterController.loadAmbariProperties().done(function (data) {
        _this.formatAmbariProperties(data.RootServiceComponents.properties);
      });
    } else {
      this.formatAmbariProperties(clusterController.get('ambariProperties'));
    }
  },

  formatAmbariProperties: function formatAmbariProperties(properties) {
    var defaults = {
      threshold: "60",
      ambari_server_host: location.hostname,
      check_execute_list: "db_connection_check"
    };
    var properties = App.permit(properties, ['jdk.name', 'jdk_location', 'java.home']);
    var renameKey = function renameKey(oldKey, newKey) {
      if (properties[oldKey]) {
        defaults[newKey] = properties[oldKey];
        delete properties[oldKey];
      }
    };
    renameKey('java.home', 'java_home');
    renameKey('jdk.name', 'jdk_name');
    $.extend(properties, defaults);
    App.db.set('tmp', 'ambariProperties', properties);
    this.set('ambariProperties', properties);
  },
  /**
   * `Action` method for starting connect to current database.
   *
   * @method connectToDatabase
   **/
  connectToDatabase: function connectToDatabase() {
    if (this.get('isBtnDisabled')) return;
    this.set('isRequestResolved', false);
    App.db.set('tmp', this.get('parentView.service.serviceName') + '_connection', {});
    this.setConnectingStatus(true);
    if (App.get('testMode')) {
      this.startPolling();
    } else {
      this.runCheckConnection();
    }
  },

  /**
   * runs check connections methods depending on service
   * @return {void}
   * @method runCheckConnection
   */
  runCheckConnection: function runCheckConnection() {
    this.createCustomAction();
  },

  /**
   * Run custom action for database connection.
   *
   * @method createCustomAction
   **/
  createCustomAction: function createCustomAction() {
    var connectionProperties = this.getProperties('db_connection_url', 'user_name', 'user_passwd');
    var db_name = this.dbInfo.dpPropertiesMap[dbUtils.getDBType(this.get('db_type').value)].db_type;
    var isServiceInstalled = App.Service.find(this.get('config.serviceName')).get('isLoaded');
    for (var key in connectionProperties) {
      if (connectionProperties.hasOwnProperty(key)) {
        connectionProperties[key] = connectionProperties[key].value;
      }
    }
    var params = $.extend(true, {}, { db_name: db_name }, connectionProperties, this.get('ambariProperties'));
    var filteredHosts = Array.isArray(this.get('masterHostName.value')) ? this.get('masterHostName.value') : [this.get('masterHostName.value')];
    App.ajax.send({
      name: isServiceInstalled ? 'cluster.custom_action.create' : 'custom_action.create',
      sender: this,
      data: {
        requestInfo: {
          parameters: params
        },
        filteredHosts: filteredHosts
      },
      success: 'onCreateActionSuccess',
      error: 'onCreateActionError'
    });
  },
  /**
   * Run updater if task is created successfully.
   *
   * @method onConnectActionS
   **/
  onCreateActionSuccess: function onCreateActionSuccess(data) {
    this.set('currentRequestId', data.Requests.id);
    App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('currentRequestId')
      },
      success: 'setCurrentTaskId'
    });
  },

  setCurrentTaskId: function setCurrentTaskId(data) {
    this.set('currentTaskId', data.items[0].Tasks.id);
    this.startPolling();
  },

  startPolling: function startPolling() {
    if (this.get('isConnecting')) this.getTaskInfo();
  },

  getTaskInfo: function getTaskInfo() {
    var request = App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('currentRequestId'),
        taskId: this.get('currentTaskId')
      },
      success: 'getTaskInfoSuccess'
    });
    this.set('request', request);
  },

  preparedDBProperties: function () {
    var propObj = {};
    var serviceName = this.get('config.serviceName');
    var serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');
    for (var key in this.get('propertiesPattern')) {
      var propName = this.getConnectionProperty(this.get('propertiesPattern')[key], true);
      propObj[propName] = serviceConfigs.findProperty('name', propName).get('value');
    }
    return propObj;
  }.property(),

  requiredProps: function () {
    var ranger = App.StackService.find().findProperty('serviceName', 'RANGER');
    var propertiesMap = {
      OOZIE: ['oozie.db.schema.name', 'oozie.service.JPAService.jdbc.username', 'oozie.service.JPAService.jdbc.password', 'oozie.service.JPAService.jdbc.driver', 'oozie.service.JPAService.jdbc.url'],
      HIVE: ['ambari.hive.db.schema.name', 'javax.jdo.option.ConnectionUserName', 'javax.jdo.option.ConnectionPassword', 'javax.jdo.option.ConnectionDriverName', 'javax.jdo.option.ConnectionURL'],
      KERBEROS: ['kdc_hosts'],
      RANGER: ranger && ranger.compareCurrentVersion('0.5') > -1 ? ['db_user', 'db_password', 'db_name', 'ranger.jpa.jdbc.url', 'ranger.jpa.jdbc.driver'] : ['db_user', 'db_password', 'db_name', 'ranger_jdbc_connection_url', 'ranger_jdbc_driver'],
      RANGER_KMS: ['db_user', 'db_password', 'ranger.ks.jpa.jdbc.url', 'ranger.ks.jpa.jdbc.driver']
    };
    return propertiesMap[this.get('parentView.content.serviceName')];
  }.property(),

  getConnectionProperty: function getConnectionProperty(regexp, isGetName) {
    var serviceName = this.get('config.serviceName');
    var serviceConfigs = this.get('controller.stepConfigs').findProperty('serviceName', serviceName).get('configs');
    var propertyName = this.get('requiredProps').filter(function (item) {
      return regexp.test(item);
    })[0];
    return isGetName ? propertyName : serviceConfigs.findProperty('name', propertyName).get('value');
  },

  propertiesPattern: function () {
    var patterns = {
      db_connection_url: /jdbc\.url|connection_url|connectionurl|kdc_hosts/ig
    };
    if (this.get('parentView.service.serviceName') != "KERBEROS") {
      patterns.user_name = /(username|dblogin|db_user)$/ig;
      patterns.user_passwd = /(dbpassword|password|db_password)$/ig;
    }
    return patterns;
  }.property('parentView.service.serviceName'),

  getTaskInfoSuccess: function getTaskInfoSuccess(data) {
    var task = data.Tasks;
    this.set('responseFromServer', {
      stderr: task.stderr,
      stdout: task.stdout
    });
    if (task.status === 'COMPLETED') {
      var structuredOut = task.structured_out.db_connection_check;
      if (structuredOut.exit_code != 0) {
        this.set('responseFromServer', {
          stderr: task.stderr,
          stdout: task.stdout,
          structuredOut: structuredOut.message
        });
        this.setResponseStatus('failed');
      } else {
        App.db.set('tmp', this.get('parentView.service.serviceName') + '_connection', this.get('preparedDBProperties'));
        this.setResponseStatus('success');
      }
    }
    if (task.status === 'FAILED') {
      this.setResponseStatus('failed');
    }
    if (/PENDING|QUEUED|IN_PROGRESS/.test(task.status)) {
      Em.run.later(this, function () {
        this.startPolling();
      }, this.get('pollInterval'));
    }
  },

  onCreateActionError: function onCreateActionError(jqXhr, status, errorMessage) {
    this.setResponseStatus('failed');
    this.set('responseFromServer', errorMessage);
  },

  setResponseStatus: function setResponseStatus(isSuccess) {
    var db_type = this.dbInfo.dpPropertiesMap[dbUtils.getDBType(this.get('db_type').value)].db_type.toUpperCase();
    var isSuccess = isSuccess == 'success';
    this.setConnectingStatus(false);
    this.set('responseCaption', isSuccess ? Em.I18n.t('services.service.config.database.connection.success') : Em.I18n.t('services.service.config.database.connection.failed'));
    this.set('isConnectionSuccess', isSuccess);
    this.set('isRequestResolved', true);
    if (this.get('logsPopup')) {
      var statusString = isSuccess ? 'common.success' : 'common.error';
      this.set('logsPopup.header', Em.I18n.t('services.service.config.connection.logsPopup.header').format(db_type, Em.I18n.t(statusString)));
    }
  },
  /**
   * Switch captions and statuses for active/non-active request.
   *
   * @method setConnectionStatus
   * @param {Boolean} [active]
   */
  setConnectingStatus: function setConnectingStatus(active) {
    if (active) {
      this.set('responseCaption', Em.I18n.t('services.service.config.database.connection.inProgress'));
    }
    this.set('controller.testConnectionInProgress', !!active);
    this.set('btnCaption', !!active ? Em.I18n.t('services.service.config.database.btn.connecting') : Em.I18n.t('services.service.config.database.btn.idle'));
    this.set('isConnecting', !!active);
  },
  /**
   * Set view to init status.
   *
   * @method restore
   **/
  restore: function restore() {
    if (this.get('request')) {
      this.get('request').abort();
      this.set('request', null);
    }
    this.set('responseCaption', null);
    this.set('responseFromServer', null);
    this.setConnectingStatus(false);
    this.set('isRequestResolved', false);
  },
  /**
   * `Action` method for showing response from server in popup.
   *
   * @method showLogsPopup
   **/
  showLogsPopup: function showLogsPopup() {
    if (this.get('isConnectionSuccess')) return;
    var _this = this;
    var db_type = this.dbInfo.dpPropertiesMap[dbUtils.getDBType(this.get('db_type').value)].db_type.toUpperCase();
    var statusString = this.get('isRequestResolved') ? 'common.error' : 'common.testing';
    var popup = App.showAlertPopup(Em.I18n.t('services.service.config.connection.logsPopup.header').format(db_type, Em.I18n.t(statusString)), null, function () {
      _this.set('logsPopup', null);
    });
    popup.reopen({
      onClose: function onClose() {
        this._super();
        _this.set('logsPopup', null);
      }
    });
    if (_typeof(this.get('responseFromServer')) == 'object') {
      popup.set('bodyClass', Em.View.extend({
        checkDBConnectionView: _this,
        templateName: require('templates/common/error_log_body'),
        openedTask: function () {
          return this.get('checkDBConnectionView.responseFromServer');
        }.property('checkDBConnectionView.responseFromServer.stderr', 'checkDBConnectionView.responseFromServer.stdout', 'checkDBConnectionView.responseFromServer.structuredOut')
      }));
    } else {
      popup.set('body', this.get('responseFromServer'));
    }
    this.set('logsPopup', popup);
    return popup;
  }
});

});

require.register("views/common/configs/widgets/textfield_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

require('views/common/controls_view');

var App = require('app');

App.TextFieldConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/textfield_config_widget'),
  classNames: ['widget-config', 'text-field-widget'],

  /**
   * Control to edit value.
   *
   * @type {App.ServiceConfigTextField|App.ServiceConfigTextFieldWithUnit}
   * @property configView
   */
  configView: function () {
    var fieldView = this.get('config.unit') ? App.ServiceConfigTextFieldWithUnit : App.ServiceConfigTextField;
    return fieldView.extend({
      isPopoverEnabled: 'false',
      textFieldClassName: 'col-md-12',
      serviceConfigBinding: 'parentView.config'
    });
  }.property('config.unit'),

  didInsertElement: function didInsertElement() {
    this._super();
    this.initPopover();
    this.get('config').set('displayType', this.get('config.stackConfigProperty.valueAttributes.type') || this.get('config.stackConfigProperty.widget.units.0.unit-name') || 'string');
  }

});

});

require.register("views/common/configs/widgets/time_interval_spinner_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TimeIntervalSpinnerView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/time_interval_spinner'),
  classNames: ['widget-config', 'spinner-input-widget'],

  supportSwitchToTextBox: true,

  /**
   * @property isValid
   * @type {Boolean}
   */
  isValid: true,

  /**
   * @property disabled
   * @type {Boolean}
   */
  disabled: false,

  /**
   * @property valueIsChanged
   * @type {Boolean}
   */
  valueIsChanged: false,

  /**
   * Default property value in widget format.
   *
   * @property savedValue
   * @type {Object[]}
   */
  savedValue: null,

  /**
   * Maximum property value in widget format.
   *
   * @property maxValue
   * @type {Object[]}
   */
  maxValue: function () {
    return this.generateWidgetValue(this.get('config.stackConfigProperty.valueAttributes.maximum'));
  }.property('config.stackConfigProperty.valueAttributes.maximum'),

  /**
   * Minimum property value in widget format.
   *
   * @property minValue
   * @type {Object[]}
   */
  minValue: function () {
    return this.generateWidgetValue(this.get('config.stackConfigProperty.valueAttributes.minimum'));
  }.property('config.stackConfigProperty.valueAttributes.minimum'),

  propertyUnitBinding: 'config.stackConfigProperty.valueAttributes.unit',

  /**
   * @TODO move it to unit conversion view mixin?
   * Map with maximum value per unit.
   *
   * @property timeMaxValueOverflow
   * @type {Object}
   */
  timeMaxValueOverflow: {
    milliseconds: 999,
    seconds: 59,
    minutes: 59,
    hours: 23,
    days: 365
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.prepareContent();
    this.toggleWidgetState();
    this.initPopover();
  },

  /**
   * Content setter.
   * Affects to view attributes:
   *  @see savedValue
   *       content
   * @method prepareContent
   */
  prepareContent: function prepareContent() {
    var property = this.get('config');
    this.setValue(!isNaN(parseInt(property.get('value'))) ? property.get('value') : 0);
    this.parseIncrement();
  },

  /**
   * If <code>config.stackConfigProperty.valueAttributes.increment_step</code> exists, it should be used as increment
   * for controls. Also if it's value greater than maximum value for last unit, it (unit) should be disabled
   * @method parseIncrement
   */
  parseIncrement: function parseIncrement() {
    var property = this.get('config');
    var step = property.get('stackConfigProperty.valueAttributes.increment_step');
    if (step) {
      var type = this.get('content.lastObject.type');
      step = this.convertValue(step, property.get('stackConfigProperty.valueAttributes.unit'), type);
      this.set('content.lastObject.incrementStep', step);
      if (step > Em.get(this, 'timeMaxValueOverflow.' + type)) {
        this.set('content.lastObject.enabled', false);
      }
    }
  },

  /**
   * @TODO move it to unit conversion view mixin?
   * Generate formatted value for widget.
   *
   * @param {String|Number} value
   * @returns {Object[]}
   * @method generateWidgetValue
   */
  generateWidgetValue: function generateWidgetValue(value) {
    return this.widgetValueByConfigAttributes(value, true).map(function (item) {
      item.label = Em.I18n.t('common.' + item.type);
      item.minValue = 0;
      item.maxValue = Em.get(this, 'timeMaxValueOverflow.' + item.type);
      item.incrementStep = 1;
      item.enabled = true;
      item.invertOnOverflow = true;
      return item;
    }, this);
  },

  /**
   * Handle changes for widget value.
   *  * Capture validation errors.
   *  * Set original config value converted from widget format
   *  * Send recommendations.
   */
  widgetValueObserver: function () {
    this.checkModified();
    this.setConfigValue();
    this.checkErrors();
    this.sendRequestRorDependentConfigs(this.get('config'));
  }.observes('content.@each.value'),

  /**
   * Handle config value changes in raw mode.
   * Check specific validation errors regarding widget format value.
   */
  configValueObserverForAttributes: function () {
    if (this.get('config.showAsTextBox')) {
      this.checkErrors();
    }
  }.observes('config.value', 'config.showAsTextBox'),

  /**
   * Check for property modification.
   * @method checkModified
   */
  checkModified: function checkModified() {
    this.set('valueIsChanged', this.configValueByWidget(this.get('content')) != parseInt(this.get('config.savedValue')));
  },

  /**
   * Check for validation errors like minimum or maximum required value
   * @param {*} [val]
   * @method checkErrors
   */
  checkErrors: function checkErrors(val) {
    val = val || this.get('content');
    // in raw mode we should use config value instead of converted from widget
    var convertedValue = this.get('config.showAsTextBox') ? parseInt(this.get('config.value'), 10) : this.configValueByWidget(val);
    var warnMessage = '';
    var warn = false;
    // if not a number ignore check since it validation failed in config model as well.
    if (isNaN(convertedValue)) {
      return;
    }
    if (convertedValue < parseInt(this.get('config.stackConfigProperty.valueAttributes.minimum'))) {
      warnMessage = Em.I18n.t('config.warnMessage.outOfBoundaries.less').format(this.dateToText(this.get('minValue')));
      warn = true;
    } else if (convertedValue > parseInt(this.get('config.stackConfigProperty.valueAttributes.maximum'))) {
      warnMessage = Em.I18n.t('config.warnMessage.outOfBoundaries.greater').format(this.dateToText(this.get('maxValue')));
      warn = true;
    }
    this.setProperties({
      isValid: !warnMessage,
      warnMessage: warnMessage
    });
    this.get('config').setProperties({
      warnMessage: warnMessage,
      warn: warn
    });
  },

  /**
   * set appropriate attribute for configProperty model
   * @method setConfigValue
   */
  setConfigValue: function setConfigValue(value) {
    this.set('config.value', '' + (value || this.configValueByWidget(this.get('content'))));
  },

  setValue: function setValue(value) {
    if (value && !isNaN(value) || !isNaN(this.get('config.value'))) {
      this.set('content', this.generateWidgetValue(value || this.get('config.value')));
    }
  },

  setRecommendedValue: function setRecommendedValue() {
    this._super();
    this.setValue();
    this.parseIncrement();
  },

  /**
   * Convert value to readable format using widget value.
   *
   * @param {Object[]} widgetFormatValue - value formatted for widget @see convertToWidgetUnits
   * @return {String}
   * @method dateToText
   */
  dateToText: function dateToText(widgetFormatValue) {
    return widgetFormatValue.map(function (item) {
      if (Em.get(item, 'value') > 0) {
        return Em.get(item, 'value') + ' ' + Em.get(item, 'label');
      } else {
        return null;
      }
    }).compact().join(' ');
  },

  /**
   * Restore value to default.
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this._super();
    this.setValue(this.get('config.savedValue'));
    this.parseIncrement();
  },

  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    if (this._super()) {
      var configValue = parseInt(this.get('config.value'));
      if (isNaN(configValue)) return false;
      if (this.get('config.stackConfigProperty.valueAttributes.minimum')) {
        var min = parseInt(this.get('config.stackConfigProperty.valueAttributes.minimum'));
        if (configValue < min) {
          this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.warnMessage.outOfBoundaries.less').format(this.dateToText(this.get('minValue'))));
          return false;
        }
      }
      if (this.get('config.stackConfigProperty.valueAttributes.maximum')) {
        var max = parseInt(this.get('config.stackConfigProperty.valueAttributes.maximum'));
        if (configValue > max) {
          this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.warnMessage.outOfBoundaries.greater').format(this.dateToText(this.get('maxValue'))));
          return false;
        }
      }
      if (this.get('config.stackConfigProperty.valueAttributes.increment_step')) {
        if (configValue % this.get('config.stackConfigProperty.valueAttributes.increment_step') != 0) return false;
      }
      this.updateWarningsForCompatibilityWithWidget('');
      return true;
    }
    return false;
  }

});

});

require.register("views/common/configs/widgets/toggle_config_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Toggle widget view for config property.
 * @type {Em.View}
 */
App.ToggleConfigWidgetView = App.ConfigWidgetView.extend({
  templateName: require('templates/common/configs/widgets/toggle_config_widget'),
  classNames: ['widget-config', 'toggle-widget'],

  /**
   * Saved switcher for current config.
   *
   * @property switcher
   */
  switcher: null,

  /**
   * Minimum required handle width for toggle widget in pixels.
   * @type {Number}
   */
  minHandleWidth: 30,

  /**
   /**
   * Value used in the checkbox.
   * <code>config.value</code> can't be used because it's string.
   *
   * @property switcherValue
   * @type {boolean}
   */
  switcherValue: false,

  /**
   * when value is changed by recommendations we don't need to send
   * another request for recommendations.
   *
   * @type {boolean}
   */
  skipRequestForDependencies: false,

  supportSwitchToTextBox: true,

  /**
   * Update config value using <code>switcherValue</code>.
   * switcherValue is boolean, but config value should be a string.
   *
   * @method updateConfigValue
   */
  updateConfigValue: function updateConfigValue() {
    this.set('config.value', this.get('config.stackConfigProperty.valueAttributes.entries')[this.get('switcherValue') ? 0 : 1].value);
  },

  /**
   * Get value for <code>switcherValue</code> (boolean) using <code>config.value</code> (string).
   *
   * @param configValue
   * @returns {boolean}
   * @method getNewSwitcherValue
   */
  getNewSwitcherValue: function getNewSwitcherValue(configValue) {
    return this.get('config.stackConfigProperty.valueAttributes.entries')[0].value === '' + configValue;
  },

  didInsertElement: function didInsertElement() {
    this.set('switcherValue', this.getNewSwitcherValue(this.get('config.value')));
    // plugin should be initiated after applying binding for switcherValue
    Em.run.later('sync', function () {
      this.initSwitcher();
      this.toggleWidgetState();
      this.initPopover();
    }.bind(this), 10);
    this.addObserver('switcherValue', this, this.updateConfigValue);
    this._super();
  },

  /**
   * Value may be changed after recommendations are received
   * So, switcher should be updated too
   *
   * @method setValue
   */
  setValue: function setValue(configValue) {
    var value = this.getNewSwitcherValue(configValue);
    if (this.get('switcherValue') !== value) {
      this.set('skipRequestForDependencies', true);
      this.get('switcher').bootstrapSwitch('toggleState', value);
      this.set('skipRequestForDependencies', false);
      this.set('switcherValue', value);
    }
  },

  /**
   * Init switcher plugin.
   *
   * @method initSwitcher
   */
  initSwitcher: function initSwitcher() {
    var labels = this.get('config.stackConfigProperty.valueAttributes.entries'),
        self = this;
    Em.assert('toggle for `' + this.get('config.name') + '` should contain two entries', labels.length === 2);
    if (this.$()) {
      var switcher = this.$("input:eq(0)").bootstrapSwitch({
        onText: labels[0].label,
        offText: labels[1].label,
        offColor: 'default',
        onColor: 'success',
        handleWidth: self.getHandleWidth(labels.mapProperty('label.length')),
        onSwitchChange: function onSwitchChange(event, state) {
          self.set('switcherValue', state);
          if (!self.get('skipRequestForDependencies')) {
            self.sendRequestRorDependentConfigs(self.get('config'));
          }
        }
      });
      this.set('switcher', switcher);
    }
  },

  /**
   * Calculate handle width by longest label.
   *
   * @param {String[]} labels
   * @returns {Number}
   */
  getHandleWidth: function getHandleWidth(labels) {
    var labelWidth = Math.max.apply(null, labels) * 8;
    return labelWidth < this.get('minHandleWidth') ? this.get('minHandleWidth') : labelWidth;
  },

  /**
   * Restore default config value and toggle switcher.
   *
   * @override App.ConfigWidgetView.restoreValue
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this._super();
    this.setValue(this.get('config.value'));
  },

  /**
   * @method setRecommendedValue
   */
  setRecommendedValue: function setRecommendedValue() {
    this._super();
    this.setValue(this.get('config.recommendedValue'));
  },

  /**
   * Enable/disable switcher basing on config isEditable value
   * @method toggleWidgetState
   */
  toggleWidgetState: function () {
    if (this.get('switcher')) {
      this.get('switcher').bootstrapSwitch('disabled', !this.get('config.isEditable'));
    }
    this._super();
  }.observes('config.isEditable'),

  /**
   * Check if value provided by user in the textbox may be used in the toggle
   * @returns {boolean}
   * @method isValueCompatibleWithWidget
   */
  isValueCompatibleWithWidget: function isValueCompatibleWithWidget() {
    if (this._super()) {
      var res = this.get('config.stackConfigProperty.valueAttributes.entries').mapProperty('value').contains(this.get('config.value'));
      if (!res) {
        this.updateWarningsForCompatibilityWithWidget(Em.I18n.t('config.infoMessage.wrong.value.for.widget'));
        return false;
      }
      this.updateWarningsForCompatibilityWithWidget('');
      return true;
    }
    return false;
  }

});

});

require.register("views/common/controls_view", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

var dbInfo = require('data/db_properties_info') || {};

var delay = function () {
  var timer = 0;
  return function (callback, ms) {
    clearTimeout(timer);
    timer = setTimeout(callback, ms);
  };
}();

/**
 * Abstract view for config fields.
 * Add popover support to control
 */
App.ServiceConfigPopoverSupport = Ember.Mixin.create({

  /**
   * Config object. It will instance of App.ServiceConfigProperty
   */
  serviceConfig: null,
  attributeBindings: ['readOnly'],
  isPopoverEnabled: true,
  popoverPlacement: 'auto right',

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$('[data-toggle=tooltip]'), { placement: 'top' });
    // if description for this serviceConfig not exist, then no need to show popover
    if (this.get('isPopoverEnabled') !== 'false' && this.get('serviceConfig.description')) {
      this.addPopover();
    }
  },

  /**
   * Element where popover is "appended"
   */
  elementForPopover: function () {
    return this.$();
  }.property(),

  /**
   * Add Popover for config-boxes
   *
   * @method addPopover
   */
  addPopover: function addPopover() {
    App.popover(this.get('elementForPopover'), {
      title: Em.I18n.t('installer.controls.serviceConfigPopover.title').format(this.get('serviceConfig.displayName'), this.get('serviceConfig.displayName') === this.get('serviceConfig.name') ? '' : this.get('serviceConfig.name')),
      content: this.get('serviceConfig.description'),
      placement: this.get('popoverPlacement'),
      trigger: 'hover',
      html: true
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.$().popover('destroy');
  },

  readOnly: Em.computed.not('serviceConfig.isEditable')
});

App.SupportsDependentConfigs = Ember.Mixin.create({

  /**
   * method send request to check if some of dependent configs was changes
   * and in case there was changes shows popup with info about changed configs
   *
   * @param {App.ServiceConfigProperty} config
   * @param [controller]
   * @returns {$.Deferred}
   */
  sendRequestRorDependentConfigs: function sendRequestRorDependentConfigs(config, controller) {
    if (!config || !config.get('isValid') && config.get('isNotDefaultValue')) return $.Deferred().resolve().promise();
    controller = controller || this.get('controller');
    if (controller && ['mainServiceInfoConfigsController', 'wizardStep7Controller'].contains(controller.get('name'))) {
      var name = config.get('name');
      var saveRecommended = config.get('value') === config.get('recommendedValue');
      var type = App.config.getConfigTagFromFileName(config.get('filename'));
      var p = App.configsCollection.getConfig(App.config.configId(name, type));
      controller.removeCurrentFromDependentList(config, saveRecommended);
      if (p && Em.get(p, 'propertyDependedBy.length') > 0 || config.get('displayType') === 'user' && config.get('oldValue') !== config.get('value')) {
        var old = config.get('oldValue');
        config.set('oldValue', config.get('value'));
        return controller.loadConfigRecommendations([{
          "type": type,
          "name": name,
          "old_value": Em.isNone(old) ? config.get('initialValue') : old
        }]);
      }
    }
    return $.Deferred().resolve().promise();
  },

  /**
   * Restore values for dependent configs by parent config info.
   * NOTE: If dependent config inherited from multiply configs its
   * value will be restored only when all parent configs are being restored.
   *
   * @param {App.ServiceConfigProperty} parentConfig
   */
  restoreDependentConfigs: function restoreDependentConfigs(parentConfig) {
    var controller = this.get('controller');
    var dependentConfigs = controller.get('recommendations');
    try {
      controller.set('recommendations', dependentConfigs.reject(function (item) {
        if (item.parentConfigs.contains(parentConfig.get('name'))) {
          if (item.parentConfigs.length > 1) {
            item.parentConfigs.removeObject(parentConfig.get('name'));
          } else {
            // reset property value
            var property = App.config.findConfigProperty(controller.get('stepConfigs'), item.propertyName, App.config.getOriginalFileName(item.fileName));
            if (property) {
              property.set('value', property.get('savedValue') || property.get('initialValue'));
            }
            return true;
          }
        }
        return false;
      }));
    } catch (e) {
      console.warn('Dependent properties popup was not cleared');
    }
  }

});

/**
 * mixin is used to send request for recommendations
 * when config value is updated by user
 */
App.ValueObserver = Em.Mixin.create(App.SupportsDependentConfigs, {

  selected: false,

  focusOut: function focusOut() {
    this.set('selected', false);
  },

  focusIn: function focusIn() {
    this.set('selected', true);
  },

  onValueUpdate: function () {
    if (!this.get('isVisible')) return;
    if (this.get('selected') || this.get('serviceConfig.changedViaUndoValue')) {
      var self = this,
          config = this.get('serviceConfig'),
          controller = this.get('controller');
      Em.set(config, 'didUserOverrideValue', true);
      delay(function () {
        self.sendRequestRorDependentConfigs(config, controller);
      }, 500);
    }
  }.observes('serviceConfig.value')
});

/**
 * mixin is used to send request for recommendations
 * when config value is updated by user
 */
App.WidgetValueObserver = Em.Mixin.create(App.ValueObserver, {
  onValueUpdate: function () {
    if (this.get('selected')) {
      var self = this,
          config = this.get('config'),
          controller = this.get('controller');
      Em.set(config, 'didUserOverrideValue', true);
      delay(function () {
        self.sendRequestRorDependentConfigs(config, controller);
      }, 500);
    }
  }.observes('config.value')
});

/**
 * mixin set class that serve as unique element identifier,
 * id not used in order to avoid collision with ember ids
 */
App.ServiceConfigCalculateId = Ember.Mixin.create({
  'data-qa': Ember.computed(function () {
    var config = this.get('config') && this.get('config.widget') ? this.get('config') : this.get('serviceConfig') || {};
    var configName = Em.get(config, 'name') || '';
    if (configName === 'content') {
      return Em.get(config, 'id') || '';
    }
    return configName;
  })
});

/**
 * Default input control
 * @type {*}
 */
App.ServiceConfigTextField = Ember.TextField.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, App.ValueObserver, {
  classNames: ['form-control'],
  valueBinding: 'serviceConfig.value',
  classNameBindings: 'textFieldClassName',
  placeholderBinding: 'serviceConfig.placeholder',

  //Set editDone true for last edited config text field parameter
  focusOut: function focusOut() {
    this._super();
    this.get('serviceConfig').set("editDone", true);
  },
  //Set editDone false for all current category config text field parameter
  focusIn: function focusIn() {
    this._super();
    if (!this.get('serviceConfig.isOverridden') && !this.get('serviceConfig.isComparison')) {
      if (this.get('parentView.categoryConfigsAll')) {
        this.get("parentView.categoryConfigsAll").setEach("editDone", false);
      }
    }
  },

  textFieldClassName: function () {
    if (this.get('serviceConfig.unit')) {
      return ['input-sm'];
    } else if (this.get('serviceConfig.displayType') === 'principal') {
      return ['col-md-12'];
    } else {
      return ['col-md-9 long-input'];
    }
  }.property('serviceConfig.displayType', 'serviceConfig.unit')

});

/**
 * Customized input control for user/group configs with corresponding uid/gid specified
 * @type {Em.View}
 */
App.ServiceConfigTextFieldUserGroupWithID = Ember.View.extend(App.ServiceConfigPopoverSupport, {
  valueBinding: 'serviceConfig.value',
  placeholderBinding: 'serviceConfig.savedValue',
  classNamesBindings: 'view.fullWidth::display-inline-block',
  isPopoverEnabled: 'false',

  fullWidth: false,

  templateName: require('templates/wizard/controls_service_config_usergroup_with_id'),

  isUIDGIDVisible: function () {
    var serviceConfigs = this.get('parentView').serviceConfigs;
    var overrideUid = serviceConfigs && serviceConfigs.findProperty('name', 'override_uid');
    //don't display the ugid field if there is no uid/gid for this property or override_uid is unchecked
    if (Em.isNone(this.get('serviceConfig.ugid')) || overrideUid && overrideUid.value === 'false') {
      return false;
    }

    var serviceName = this.get('serviceConfig').name.substr(0, this.get('serviceConfig').name.indexOf('_')).toUpperCase();
    if (serviceName === 'ZK') {
      serviceName = 'ZOOKEEPER';
    }
    if (serviceName === 'MAPRED') {
      serviceName = 'YARN';
    }
    //addServiceController and service already installed or Hadoop user group
    if (App.Service.find(serviceName).get('isLoaded') || serviceName === 'USER') {
      return false;
    }

    return this.get('parentView.isUIDGIDVisible');
  }.property('parentView.isUIDGIDVisible')
});

/**
 * Customized input control with Units type specified
 * @type {Em.View}
 */
App.ServiceConfigTextFieldWithUnit = Ember.View.extend(App.ServiceConfigPopoverSupport, App.ValueObserver, {
  valueBinding: 'serviceConfig.value',
  classNames: ['input-group', 'with-unit', 'col-md-4'],
  placeholderBinding: 'serviceConfig.savedValue',

  templateName: require('templates/wizard/controls_service_config_textfield_with_unit')
});

/**
 * Password control
 * @type {*}
 */
App.ServiceConfigPasswordField = Ember.View.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, {

  serviceConfig: null,

  placeholder: Em.I18n.t('form.item.placeholders.typePassword'),

  templateName: require('templates/common/configs/widgets/service_config_password_field'),

  classNames: ['password-field-wrapper'],

  readOnly: Em.computed.not('serviceConfig.isEditable'),

  keyPress: function keyPress(event) {
    if (event.keyCode == 13) {
      return false;
    }
  }

});

/**
 * Textarea control
 * @type {*}
 */
App.ServiceConfigTextArea = Ember.TextArea.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, App.ValueObserver, {

  valueBinding: 'serviceConfig.value',
  rows: 4,
  classNames: ['directories', 'form-control'],
  classNameBindings: ['widthClass'],
  widthClass: 'col-md-9'
});

/**
 * Special config type for Capacity Scheduler
 */
App.CapacitySceduler = App.ServiceConfigTextArea.extend({

  rows: 16,

  /**
   * specific property handling for cs
   *
   * @param {App.ServiceConfigProperty} config
   * @param [controller]
   * @returns {$.Deferred}
   * @override
   */
  sendRequestRorDependentConfigs: function sendRequestRorDependentConfigs(config, controller) {
    if (!config.get('isValid') && config.get('isNotDefaultValue')) return $.Deferred().resolve().promise();
    controller = controller || this.get('controller');
    if (controller && ['mainServiceInfoConfigsController', 'wizardStep7Controller'].contains(controller.get('name'))) {
      var schedulerConfigs = config.get('value').split('\n').map(function (_property) {
        return {
          "type": 'capacity-scheduler',
          "name": _property.split('=')[0]
        };
      });
      if (!schedulerConfigs.someProperty('name', 'capacity-scheduler')) {
        schedulerConfigs.push({
          "type": 'capacity-scheduler',
          "name": 'capacity-scheduler'
        });
      }
      return controller.loadConfigRecommendations(schedulerConfigs);
    }

    return $.Deferred().resolve().promise();
  }

});

/**
 * Textarea control for content type
 * @type {*}
 */
App.ServiceConfigTextAreaContent = Ember.TextArea.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, App.ValueObserver, {

  valueBinding: 'serviceConfig.value',
  rows: 20,
  classNames: ['col-md-10', 'form-control']
});

/**
 * Textarea control with bigger height
 * @type {*}
 */
App.ServiceConfigBigTextArea = App.ServiceConfigTextArea.extend(App.ServiceConfigCalculateId, {
  rows: 10
});

var checkboxConfigView = Ember.Checkbox.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, App.SupportsDependentConfigs, {
  allowedPairs: {
    'trueFalse': ["true", "false"],
    'YesNo': ["Yes", "No"],
    'YESNO': ["YES", "NO"],
    'yesNo': ["yes", "no"]
  },

  trueValue: true,
  falseValue: false,

  checked: false,

  elementForPopover: function () {
    return this.$().next();
  }.property(),

  /**
   * set appropriate config values pair
   * to define which value is positive (checked) property
   * and what value is negative (unchecked) property
   */
  didInsertElement: function didInsertElement() {
    var self = this;
    if (this.get('serviceConfig').isDestroyed) return;
    this._super();
    this.addObserver('serviceConfig.value', this, 'toggleChecker');
    var isInverted = this.get('serviceConfig.displayType') === 'boolean-inverted';
    Object.keys(this.get('allowedPairs')).forEach(function (key) {
      if (this.get('allowedPairs')[key].contains(this.get('serviceConfig.value'))) {
        this.set('trueValue', isInverted ? this.get('allowedPairs')[key][1] : this.get('allowedPairs')[key][0]);
        this.set('falseValue', isInverted ? this.get('allowedPairs')[key][0] : this.get('allowedPairs')[key][1]);
      }
    }, this);
    this.set('checked', this.get('serviceConfig.value') === this.get('trueValue'));
    this.propertyDidChange('checked');
    Em.run.next(function () {
      if (self.$()) {
        self.propertyDidChange('elementForPopover');
        self.addPopover();
      }
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.removeObserver('serviceConfig.value', this, 'checkedBinding');
  },

  /***
   * defines if checkbox value appropriate to the config value
   * @returns {boolean}
   */
  isNotAppropriateValue: function isNotAppropriateValue() {
    return this.get('serviceConfig.value') !== this.get(this.get('checked') + 'Value');
  },

  /**
   * change service config value if click on checkbox
   */
  toggleValue: function () {
    if (this.isNotAppropriateValue()) {
      this.set('serviceConfig.value', this.get(this.get('checked') + 'Value'));
      this.get('serviceConfig').set("editDone", true);
      this.sendRequestRorDependentConfigs(this.get('serviceConfig'));

      //if the checkbox being toggled is the 'Have Ambari manage UIDs' in Misc Tab, show/hide uid/gid column accordingly
      if (this.get('serviceConfig.name') === 'override_uid') {
        this.set('parentView.isUIDGIDVisible', this.get('checked'));
      }
    }
  }.observes('checked'),

  /**
   * change checkbox value if click on undo
   */
  toggleChecker: function toggleChecker() {
    if (this.isNotAppropriateValue()) {
      this.set('checked', !this.get('checked'));
    }
  },

  disabled: Em.computed.not('serviceConfig.isEditable'),

  disabledDidChange: function () {
    if (!this.get('checkboxInstance')) return;
  }.observes('disabled'),

  //Set editDone false for all current category config text field parameter
  focusIn: function focusIn(event) {
    if (!this.get('serviceConfig.isOverridden') && !this.get('serviceConfig.isComparison')) {
      this.get("parentView.categoryConfigsAll").setEach("editDone", false);
    }
  }
});

/**
 * Checkbox control
 * @type {*}
 */
App.ServiceConfigCheckbox = App.CheckboxView.extend({
  classNames: ['display-inline-block'],
  classNameBindings: ['containerClassName'],
  containerClassName: 'checkbox',
  categoryConfigsAllBinding: 'parentView.categoryConfigsAll',
  checkboxView: checkboxConfigView.extend({
    serviceConfigBinding: 'parentView.serviceConfig',
    didInsertElement: function didInsertElement() {
      this.set('parentView.checkboxId', this.get('elementId'));
      this._super();
    }
  })
});

/**
 * Checkbox control which can hide or show dependent  properties
 * @type {*|void}
 */
App.ServiceConfigCheckboxWithDependencies = App.ServiceConfigCheckbox.extend({

  toggleDependentConfigs: function () {
    if (this.get('serviceConfig.dependentConfigPattern')) {
      if (this.get('serviceConfig.dependentConfigPattern') === "CATEGORY") {
        this.disableEnableCategoryConfigs();
      } else {
        this.showHideDependentConfigs();
      }
    }
  }.observes('checked'),

  disableEnableCategoryConfigs: function disableEnableCategoryConfigs() {
    this.get('categoryConfigsAll').setEach('isEditable', this.get('checked'));
    this.set('serviceConfig.isEditable', true);
  },

  showHideDependentConfigs: function showHideDependentConfigs() {
    this.get('categoryConfigsAll').forEach(function (c) {
      if (c.get('name').match(this.get('serviceConfig.dependentConfigPattern')) && c.get('name') != this.get('serviceConfig.name')) c.set('isVisible', this.get('checked'));
    }, this);
  }
});

App.ServiceConfigRadioButtons = Ember.View.extend(App.ServiceConfigCalculateId, App.SupportsDependentConfigs, {
  templateName: require('templates/wizard/controls_service_config_radio_buttons'),

  didInsertElement: function didInsertElement() {
    // on page render, automatically populate JDBC URLs only for default database settings
    // so as to not lose the user's customizations on these fields
    if (this.get('hostNameProperty')) {
      this.get('hostNameProperty').set('isEditable', !this.get('isNewDb'));
    }
    if (['addServiceController', 'installerController'].contains(this.get('controller.wizardController.name')) && !App.StackService.find(this.get('serviceConfig.serviceName')).get('isInstalled')) {
      if (this.get('isNewDb') || this.get('dontUseHandleDbConnection').contains(this.get('serviceConfig.name'))) {
        this.onOptionsChange();
      } else {
        if (App.get('isHadoopWindowsStack') && this.get('inMSSQLWithIA') || this.get('serviceConfig.name') === 'DB_FLAVOR') {
          this.onOptionsChange();
        }
      }
      this.handleDBConnectionProperty();
    }
  },

  /**
   * Radio buttons that are not DB options and should not trigger any observer or change any other property's value
   * Ranger service -> "Authentication method" property is an example for non DB related radio button
   */
  nonDBRadioButtons: function () {
    return this.get('dontUseHandleDbConnection').without('DB_FLAVOR');
  }.property('dontUseHandleDbConnection'),

  /**
   * properties with these names don'use handleDBConnectionProperty method
   */
  dontUseHandleDbConnection: function () {
    // functionality added in Ranger 0.5
    // remove DB_FLAVOR so it can handle DB Connection checks
    var rangerService = App.StackService.find().findProperty('serviceName', 'RANGER');
    var rangerVersion = rangerService ? rangerService.get('serviceVersion') : '';
    if (rangerVersion && rangerVersion.split('.')[0] < 1 && rangerVersion.split('.')[1] < 5) {
      return ['DB_FLAVOR', 'authentication_method'];
    }
    return ['ranger.authentication.method'];
  }.property('App.currentStackName'),

  serviceConfig: null,
  categoryConfigsAll: null,

  /**
   * defines if new db is selected;
   * @type {boolean}
   */
  isNewDb: function () {
    return (/New /g.test(this.get('serviceConfig.value'))
    );
  }.property('serviceConfig.serviceName', 'serviceConfig.value'),

  /**
   * defines if 'Existing MSSQL Server database with integrated authentication' is selected
   * in this case some properties can have different behaviour
   * @type {boolean}
   */
  inMSSQLWithIA: Em.computed.equal('serviceConfig.value', 'Existing MSSQL Server database with integrated authentication'),

  /**
   * Radio button has very uncomfortable values for managing it's state
   * so it's better to use code values that easier to manipulate. Ex:
   * "Existing MySQL Database" transforms to "MYSQL"
   * @type {string}
   */
  getDbTypeFromRadioValue: function () {
    var currentValue = this.get('serviceConfig.value');
    /** TODO: Remove SQLA from the list of databases once Ranger DB_FLAVOR=SQLA is replaced with SQL Anywhere */
    var databases = /MySQL|Postgres|Oracle|Derby|MSSQL|SQLA|Anywhere/gi;
    if (this.get('inMSSQLWithIA')) {
      return 'MSSQL2';
    } else {
      var matches = currentValue.match(databases);
      if (matches) {
        return currentValue.match(databases)[0].toUpperCase();
      } else {
        return "MYSQL";
      }
    }
  }.property('serviceConfig.serviceName', 'serviceConfig.value'),

  onOptionsChange: function () {
    if (!this.get('nonDBRadioButtons').contains(this.get('serviceConfig.name'))) {
      /** if new db is selected host name must be same as master of selected service (and can't be changed)**/
      this.handleSpecialUserPassProperties();
    }
  }.observes('databaseProperty.value', 'hostNameProperty.value', 'serviceConfig.value'),

  name: function () {
    var name = this.get('serviceConfig.radioName');
    if (!this.get('serviceConfig.isOriginalSCP')) {
      if (this.get('serviceConfig.isComparison')) {
        var version = this.get('serviceConfig.compareConfigs') ? this.get('controller.selectedVersion') : this.get('version');
        name += '-v' + version;
      } else {
        var group = this.get('serviceConfig.group.name');
        name += '-' + group;
      }
    }
    return name;
  }.property('serviceConfig.radioName'),

  /**
   * Just property object for database name
   * @type {App.ServiceConfigProperty}
   */
  databaseProperty: function () {
    return this.getPropertyByType('db_name');
  }.property('serviceConfig.serviceName'),

  /**
   * Just property object for host name
   * @type {App.ServiceConfigProperty}
   */
  hostNameProperty: function () {
    var host = this.getPropertyByType('host_name');
    if (host && !host.get('value')) {
      if (host.get('savedValue')) {
        host.set('value', host.get('savedValue'));
      } else if (host.get('recommendedValue')) {
        host.set('value', host.get('recommendedValue'));
      }
    }
    return host;
  }.property('serviceConfig.serviceName', 'serviceConfig.value'),

  /**
   * Just property object for database name
   * @type {App.ServiceConfigProperty}
   */
  userProperty: function () {
    return this.getPropertyByType('user_name');
  }.property('serviceConfig.serviceName'),

  /**
   * Just property object for database name
   * @type {App.ServiceConfigProperty}
   */
  passwordProperty: function () {
    return this.getPropertyByType('password');
  }.property('serviceConfig.serviceName'),

  /**
   *
   * @param propertyType
   * @returns {*}
   */
  getDefaultPropertyValue: function getDefaultPropertyValue(propertyType) {
    var dbProperties = dbInfo.dpPropertiesMap[this.get('getDbTypeFromRadioValue')],
        serviceName = this.get('serviceConfig.serviceName');
    return dbProperties[serviceName] && dbProperties[serviceName][propertyType] ? dbProperties[serviceName][propertyType] : dbProperties[propertyType];
  },

  /**
   *
   * @param propertyType
   * @returns {*|Object}
   */
  getPropertyByType: function getPropertyByType(propertyType) {
    if (dbInfo.dpPropertiesByServiceMap[this.get('serviceConfig.serviceName')]) {
      //@TODO: dbInfo.dpPropertiesByServiceMap has corresponding property name but does not have filenames with it. this can cause issue when there are multiple db properties with same name belonging to different files
      /** check if selected service has db properties**/
      return this.get('controller.selectedService.configs').findProperty('name', dbInfo.dpPropertiesByServiceMap[this.get('serviceConfig.serviceName')][propertyType]);
    }
    return null;
  },

  /**
   * This method hides properties <code>user_name<code> and <code>password<code> in case selected db is
   * "Existing MSSQL Server database with integrated authentication" or similar
   * @method handleSpecialUserPassProperties
   */
  handleSpecialUserPassProperties: function handleSpecialUserPassProperties() {
    ['user_name', 'password'].forEach(function (pType) {
      var property = this.getPropertyByType(pType);
      if (property) {
        property.setProperties({
          'isVisible': !this.get('inMSSQLWithIA'),
          'isRequired': !this.get('inMSSQLWithIA')
        });
      }
    }, this);
  },

  /**
   * `Observer` that add <code>additionalView</code> to <code>App.ServiceConfigProperty</code>
   * that responsible for (if existing db selected)
   * 1. checking database connection
   * 2. showing jdbc driver setup warning msg.
   *
   * @method handleDBConnectionProperty
   **/
  handleDBConnectionProperty: function () {
    if (this.get('dontUseHandleDbConnection').contains(this.get('serviceConfig.name'))) {
      return;
    }
    var handledProperties = ['oozie_database', 'hive_database', 'DB_FLAVOR'];
    var currentValue = this.get('serviceConfig.value');
    /** TODO: Remove SQLA from the list of databases once Ranger DB_FLAVOR=SQLA is replaced with SQL Anywhere */
    var databases = /MySQL|PostgreSQL|Postgres|Oracle|Derby|MSSQL|SQLA|Anywhere/gi;
    var currentDB = currentValue.match(databases)[0];
    /** TODO: Remove SQLA from the list of databases once Ranger DB_FLAVOR=SQLA is replaced with SQL Anywhere */
    var checkDatabase = /existing/gi.test(currentValue);
    // db connection check button show up if existed db selected
    var propertyAppendTo1 = this.get('categoryConfigsAll').findProperty('displayName', 'Database URL');
    // warning msg under database type radio buttons, to warn the user to setup jdbc driver if existed db selected
    var propertyHive = this.get('categoryConfigsAll').findProperty('displayName', 'Hive Database');
    var propertyOozie = this.get('categoryConfigsAll').findProperty('displayName', 'Oozie Database');
    var propertyAppendTo2 = propertyHive ? propertyHive : propertyOozie;
    // RANGER specific
    if (this.get('serviceConfig.serviceName') === 'RANGER') {
      propertyAppendTo1 = this.get('categoryConfigsAll').findProperty('name', 'ranger.jpa.jdbc.url');
      propertyAppendTo2 = this.get('categoryConfigsAll').findProperty('name', 'DB_FLAVOR');
      // check for all db types when installing Ranger - not only for existing ones
      checkDatabase = true;
    }
    // Hive specific
    if (this.get('serviceConfig.serviceName') === 'HIVE') {
      // check for all db types when installing Hive - not only for existing ones
      checkDatabase = true;
    }
    if (propertyAppendTo1) {
      propertyAppendTo1.set('additionalView', null);
    }
    if (propertyAppendTo2) {
      propertyAppendTo2.set('additionalView', null);
    }
    var shouldAdditionalViewsBeSet = currentDB && checkDatabase && handledProperties.contains(this.get('serviceConfig.name')),
        driver = this.getDefaultPropertyValue('sql_jar_connector') ? this.getDefaultPropertyValue('sql_jar_connector').split("/").pop() : 'driver.jar',
        dbType = this.getDefaultPropertyValue('db_type'),
        dbName = this.getDefaultPropertyValue('db_name'),
        driverName = this.getDefaultPropertyValue('driver_name'),
        driverDownloadUrl = this.getDefaultPropertyValue('driver_download_url'),
        additionalView1 = shouldAdditionalViewsBeSet && !this.get('isNewDb') ? App.CheckDBConnectionView.extend({ databaseName: dbType }) : null,
        additionalView2 = shouldAdditionalViewsBeSet ? Ember.View.extend({
      template: Ember.Handlebars.compile('<div class="alert alert-warning">{{{view.message}}}</div>'),
      message: function () {
        return Em.I18n.t('services.service.config.database.msg.jdbcSetup.detailed').format(dbName, dbType, driver, driverDownloadUrl, driverName);
      }.property()
    }) : null;
    if (propertyAppendTo1) {
      Em.run.next(function () {
        propertyAppendTo1.set('additionalView', additionalView1);
      });
    }
    if (propertyAppendTo2) {
      Em.run.next(function () {
        propertyAppendTo2.set('additionalView', additionalView2);
      });
    }
  }.observes('serviceConfig.value'),

  options: function () {
    return this.get('serviceConfig.options').map(function (option) {
      var dbTypePattern = /mysql|postgres|oracle|derby|mssql|sql\s?a/i,
          className = '',
          displayName = Em.get(option, 'displayName'),
          dbTypeMatch = displayName.match(dbTypePattern);
      if (dbTypeMatch) {
        var dbSourcePattern = /new/i,
            newDbMatch = displayName.match(dbSourcePattern);
        if (newDbMatch) {
          className += 'new-';
        }
        className += dbTypeMatch[0].replace(' ', '').toLowerCase();
      }
      return className ? Em.Object.create(option, {
        className: className,
        radioId: ''
      }) : option;
    });
  }.property('serviceConfig.options')
});

App.ServiceConfigRadioButton = Ember.Checkbox.extend(App.SupportsDependentConfigs, {
  tagName: 'input',
  attributeBindings: ['type', 'name', 'value', 'checked', 'disabled'],
  checked: false,
  clicked: false,
  type: 'radio',
  name: null,
  value: null,
  /**
   * Element id passed to label's <code>for</code> attribute.
   * @type {String}
   */
  radioId: '',

  didInsertElement: function didInsertElement() {
    this.set('radioId', this.get('elementId'));
    this.set('checked', this.get('parentView.serviceConfig.value') === this.get('value'));
  },

  click: function click() {
    this.set('clicked', true);
    this.set('checked', true);
    this.onChecked();
  },

  onChecked: function () {
    // Wrapping the call with Ember.run.next prevents a problem where setting isVisible on component
    // causes JS error due to re-rendering.  For example, this occurs when switching the Config Group
    // in Service Config page
    if (this.get('clicked')) {
      Em.run.next(this, function () {
        this.set('parentView.serviceConfig.value', this.get('value'));
        this.sendRequestRorDependentConfigs(this.get('parentView.serviceConfig'));
        this.set('clicked', false);
        this.updateForeignKeys();
      });
    }
  }.observes('checked'),

  updateCheck: function () {
    if (!this.get('clicked')) {
      this.set('checked', this.get('parentView.serviceConfig.value') === this.get('value'));
      this.updateForeignKeys();
    }
  }.observes('parentView.serviceConfig.value'),

  updateForeignKeys: function updateForeignKeys() {
    var components = this.get('parentView.serviceConfig.options');
    if (components && components.someProperty('foreignKeys')) {
      this.get('controller.stepConfigs').findProperty('serviceName', this.get('parentView.serviceConfig.serviceName')).propertyDidChange('errorCount');
    }
  },

  disabled: function () {
    return !this.get('parentView.serviceConfig.isEditable') || !['addServiceController', 'installerController'].contains(this.get('controller.wizardController.name')) && /^New\s\w+\sDatabase$/.test(this.get('value'));
  }.property('parentView.serviceConfig.isEditable')
});

App.ServiceConfigComboBox = Ember.Select.extend(App.ServiceConfigPopoverSupport, App.ServiceConfigCalculateId, App.SupportsDependentConfigs, {
  contentBinding: 'serviceConfig.options',
  selectionBinding: 'serviceConfig.value',
  placeholderBinding: 'serviceConfig.savedValue',
  classNames: ['col-md-3']
});

/**
 * Base component for host config with popover support
 */
App.ServiceConfigHostPopoverSupport = Ember.Mixin.create({

  /**
   * Config object. It will instance of App.ServiceConfigProperty
   */
  serviceConfig: null,

  didInsertElement: function didInsertElement() {
    App.popover(this.$(), {
      title: this.get('serviceConfig.displayName'),
      content: this.get('serviceConfig.description'),
      placement: 'right',
      trigger: 'hover'
    });
  }
});

/**
 * Master host component.
 * Show hostname without ability to edit it
 * @type {*}
 */
App.ServiceConfigMasterHostView = Ember.View.extend(App.ServiceConfigHostPopoverSupport, App.ServiceConfigCalculateId, {

  classNames: ['master-host', 'col-md-6'],
  valueBinding: 'serviceConfig.value',

  template: Ember.Handlebars.compile('{{view.value}}')

});

/**
 * text field property view that enables possibility
 * for check connection
 * @type {*}
 */
App.checkConnectionView = App.ServiceConfigTextField.extend({
  didInsertElement: function didInsertElement() {
    this._super();
    if (this.get('controller.isHostsConfigsPage')) {
      return;
    }
    var kdc = this.get('categoryConfigsAll').findProperty('name', 'kdc_type');
    var propertyAppendTo = this.get('categoryConfigsAll').findProperty('name', 'domains');
    if (propertyAppendTo) {
      try {
        propertyAppendTo.set('additionalView', App.CheckDBConnectionView.extend({ databaseName: kdc && kdc.get('value') }));
      } catch (e) {
        console.error('error while adding "Test connection button"');
      }
    }
  }
});

/**
 * Show value as plain label in italics
 * @type {*}
 */
App.ServiceConfigLabelView = Ember.View.extend(App.ServiceConfigHostPopoverSupport, App.ServiceConfigCalculateId, {

  classNames: ['master-host', 'col-md-6'],
  valueBinding: 'serviceConfig.value',
  unitBinding: 'serviceConfig.unit',

  fullValue: function () {
    var value = this.get('value');
    var unit = this.get('unit');
    return unit ? value + ' ' + unit : value;
  }.property('value', 'unit'),

  template: Ember.Handlebars.compile('<i>{{formatWordBreak view.fullValue}}</i>')
});

/**
 * Base component to display Multiple hosts
 * @type {*}
 */
App.ServiceConfigMultipleHostsDisplay = Ember.Mixin.create(App.ServiceConfigHostPopoverSupport, App.ServiceConfigCalculateId, {

  hasNoHosts: function () {
    if (!this.get('value')) {
      return true;
    }
    return !this.get('value.length');
  }.property('value'),

  formatValue: Em.computed.ifThenElseByKeys('hasOneHost', 'value.firstObject', 'value'),

  hasOneHost: Em.computed.equal('value.length', 1),

  hasMultipleHosts: Em.computed.gt('value.length', 1),

  otherLength: function () {
    var len = this.get('value.length');
    if (len > 2) {
      return Em.I18n.t('installer.controls.serviceConfigMultipleHosts.others').format(len - 1);
    }
    return Em.I18n.t('installer.controls.serviceConfigMultipleHosts.other');
  }.property('value')

});

/**
 * Multiple Slave Hosts component
 * @type {*}
 */
App.ServiceConfigComponentHostsView = Ember.View.extend(App.ServiceConfigMultipleHostsDisplay, App.ServiceConfigCalculateId, {

  viewName: 'serviceConfigSlaveHostsView',

  classNames: ['component-hosts', 'form-text'],

  valueBinding: 'serviceConfig.value',

  templateName: require('templates/wizard/component_hosts'),

  firstHost: Em.computed.firstNotBlank('value.firstObject', 'serviceConfig.value.firstObject'),

  /**
   * Onclick handler for link
   */
  showHosts: function showHosts() {
    var serviceConfig = this.get('serviceConfig');
    App.ModalPopup.show({
      header: Em.I18n.t('installer.controls.serviceConfigMasterHosts.header').format(serviceConfig.category),
      bodyClass: Ember.View.extend({
        serviceConfig: serviceConfig,
        templateName: require('templates/wizard/component_hosts_popup')
      }),
      secondary: null
    });
  }

});

/**
 * DropDown component for <code>select hosts for groups</code> popup
 * @type {*}
 */
App.SlaveComponentDropDownGroupView = Ember.View.extend(App.ServiceConfigCalculateId, {

  viewName: "slaveComponentDropDownGroupView",

  /**
   * On change handler for <code>select hosts for groups</code> popup
   * @param event
   */
  changeGroup: function changeGroup(event) {
    var host = this.get('content');
    var groupName = $('#' + this.get('elementId') + ' select').val();
    this.get('controller').changeHostGroup(host, groupName);
  },

  optionTag: Ember.View.extend({

    /**
     * Whether current value(OptionTag value) equals to host value(assigned to SlaveComponentDropDownGroupView.content)
     */
    selected: function () {
      return this.get('parentView.content.group') === this.get('content');
    }.property('content')
  })
});

/**
 * View for testing connection to database.
 **/
App.CheckDBConnectionView = Ember.View.extend({
  templateName: require('templates/common/form/check_db_connection'),
  /** @property {string} btnCaption - text for button **/
  btnCaption: function () {
    return this.get('parentView.service.serviceName') === 'KERBEROS' ? Em.I18n.t('services.service.config.kdc.btn.idle') : Em.I18n.t('services.service.config.database.btn.idle');
  }.property('parentView.service.serviceName'),
  /** @property {string} responseCaption - text for status link **/
  responseCaption: null,
  /** @property {boolean} isConnecting - is request to server activated **/
  isConnecting: false,
  /** @property {boolean} isValidationPassed - check validation for required fields **/
  isValidationPassed: null,
  /** @property {string} databaseName- name of current database **/
  databaseName: null,
  /** @property {boolean} isRequestResolved - check for finished request to server **/
  isRequestResolved: false,
  /** @property {boolean} isConnectionSuccess - check for successful connection to database **/
  isConnectionSuccess: null,
  /** @property {string} responseFromServer - message from server response **/
  responseFromServer: null,
  /** @property {Object} ambariRequiredProperties - properties that need for custom action request **/
  ambariRequiredProperties: null,
  /** @property {Number} currentRequestId - current custom action request id **/
  currentRequestId: null,
  /** @property {Number} currentTaskId - current custom action task id **/
  currentTaskId: null,
  /** @property {jQuery.Deferred} request - current $.ajax request **/
  request: null,
  /** @property {Number} pollInterval - timeout interval for ajax polling **/
  pollInterval: 3000,
  /** @property {Object} logsPopup - popup with DB connection check info **/
  logsPopup: null,
  /** @property {string} hostNameProperty - host name property based on service and database names **/
  hostNameProperty: function () {
    if (!/wizard/i.test(this.get('controller.name')) && this.get('parentView.service.serviceName') === 'HIVE') {
      return this.get('parentView.service.serviceName').toLowerCase() + '_hostname';
    } else if (this.get('parentView.service.serviceName') === 'KERBEROS') {
      return 'kdc_hosts';
    } else if (this.get('parentView.service.serviceName') === 'RANGER') {
      return '{0}_{1}_host'.format(this.get('parentView.service.serviceName').toLowerCase(), this.get('databaseName').toLowerCase());
    }
    return '{0}_existing_{1}_host'.format(this.get('parentView.service.serviceName').toLowerCase(), this.get('databaseName').toLowerCase());
  }.property('databaseName'),
  /** @property {boolean} isBtnDisabled - disable button on failed validation or active request **/
  isBtnDisabled: Em.computed.or('!isValidationPassed', 'isConnecting'),
  /** @property {object} requiredProperties - properties that necessary for database connection **/
  requiredProperties: function () {
    var ranger = App.StackService.find().findProperty('serviceName', 'RANGER');
    var propertiesMap = {
      OOZIE: ['oozie.db.schema.name', 'oozie.service.JPAService.jdbc.username', 'oozie.service.JPAService.jdbc.password', 'oozie.service.JPAService.jdbc.driver', 'oozie.service.JPAService.jdbc.url'],
      HIVE: ['ambari.hive.db.schema.name', 'javax.jdo.option.ConnectionUserName', 'javax.jdo.option.ConnectionPassword', 'javax.jdo.option.ConnectionDriverName', 'javax.jdo.option.ConnectionURL'],
      KERBEROS: ['kdc_hosts'],
      RANGER: ranger && App.StackService.find().findProperty('serviceName', 'RANGER').compareCurrentVersion('0.5') > -1 ? ['db_user', 'db_password', 'db_name', 'ranger.jpa.jdbc.url', 'ranger.jpa.jdbc.driver'] : ['db_user', 'db_password', 'db_name', 'ranger_jdbc_connection_url', 'ranger_jdbc_driver']
    };
    return propertiesMap[this.get('parentView.service.serviceName')];
  }.property(),
  /** @property {Object} propertiesPattern - check pattern according to type of connection properties **/
  propertiesPattern: function () {
    var patterns = {
      db_connection_url: /jdbc\.url|connection_url|connectionurl|kdc_hosts/ig
    };
    if (this.get('parentView.service.serviceName') != "KERBEROS") {
      patterns.user_name = /(username|dblogin|db_user)$/ig;
      patterns.user_passwd = /(dbpassword|password|db_password)$/ig;
    }
    return patterns;
  }.property('parentView.service.serviceName'),
  /** @property {String} masterHostName - host name location of Master Component related to Service **/
  masterHostName: function () {
    var serviceMasterMap = {
      'OOZIE': 'oozie_server_hosts',
      'HDFS': 'hadoop_host',
      'HIVE': 'hive_metastore_hosts',
      'KERBEROS': 'kdc_hosts',
      'RANGER': 'ranger_server_hosts'
    };
    return this.get('parentView.categoryConfigsAll').findProperty('name', serviceMasterMap[this.get('parentView.service.serviceName')]).get('value');
  }.property('parentView.service.serviceName', 'parentView.categoryConfigsAll.@each.value'),
  /** @property {Object} connectionProperties - service specific config values mapped for custom action request **/
  connectionProperties: function () {
    var propObj = {};
    for (var key in this.get('propertiesPattern')) {
      propObj[key] = this.getConnectionProperty(this.get('propertiesPattern')[key]);
    }

    if (this.get('parentView.service.serviceName') === 'RANGER') {
      var dbFlavor = this.get('parentView.categoryConfigsAll').findProperty('name', 'DB_FLAVOR').get('value'),

      /** TODO: Remove SQLA from the list of databases once Ranger DB_FLAVOR=SQLA is replaced with SQL Anywhere */
      databasesTypes = /MYSQL|POSTGRES|ORACLE|MSSQL|SQLA|Anywhere/gi,
          dbType = dbFlavor.match(databasesTypes) ? dbFlavor.match(databasesTypes)[0].toLowerCase() : '';

      if (dbType === 'oracle') {
        // fixes oracle SYSDBA issue
        propObj['user_name'] = "\'%@ as sysdba\'".fmt(propObj['user_name']);
      }
    }
    return propObj;
  }.property('parentView.categoryConfigsAll.@each.value'),
  /**
   * Properties that stores in local storage used for handling
   * last success connection.
   *
   * @property {Object} preparedDBProperties
   **/
  preparedDBProperties: function () {
    var propObj = {};
    for (var key in this.get('propertiesPattern')) {
      var propName = this.getConnectionProperty(this.get('propertiesPattern')[key], true);
      propObj[propName] = this.get('parentView.categoryConfigsAll').findProperty('name', propName).get('value');
    }
    return propObj;
  }.property(),
  /** Check validation and load ambari properties **/
  didInsertElement: function didInsertElement() {
    var kdc = this.get('parentView.categoryConfigsAll').findProperty('name', 'kdc_type');
    if (kdc) {
      var name = kdc.get('value');
      if (name == 'Existing MIT KDC') {
        name = 'KDC';
      } else if (name == 'Existing IPA') {
        name = 'IPA';
      } else {
        name = 'AD';
      }
      App.popover(this.$('.connection-result'), {
        title: Em.I18n.t('services.service.config.database.btn.idle'),
        content: Em.I18n.t('installer.controls.checkConnection.popover').format(name),
        placement: 'right',
        trigger: 'hover'
      });
    }
    this.handlePropertiesValidation();
    this.getAmbariProperties();
  },
  /** On view destroy **/
  willDestroyElement: function willDestroyElement() {
    this.set('isConnecting', false);
    this._super();
  },
  /**
   * Observer that take care about enabling/disabling button based on required properties validation.
   *
   * @method handlePropertiesValidation
   **/
  handlePropertiesValidation: function () {
    this.restore();
    var isValid = true;
    var properties = [].concat(this.get('requiredProperties'));
    properties.push(this.get('hostNameProperty'));
    properties.forEach(function (propertyName) {
      var property = this.get('parentView.categoryConfigsAll').findProperty('name', propertyName);
      if (property && !property.get('isValid')) isValid = false;
    }, this);
    this.set('isValidationPassed', isValid);
  }.observes('parentView.categoryConfigsAll.@each.isValid', 'parentView.categoryConfigsAll.@each.value', 'databaseName'),

  getConnectionProperty: function getConnectionProperty(regexp, isGetName) {
    var _this = this;
    var propertyName = _this.get('requiredProperties').filter(function (item) {
      return regexp.test(item);
    })[0];
    return isGetName ? propertyName : _this.get('parentView.categoryConfigsAll').findProperty('name', propertyName).get('value');
  },
  /**
   * Set up ambari properties required for custom action request
   *
   * @method getAmbariProperties
   **/
  getAmbariProperties: function getAmbariProperties() {
    var clusterController = App.router.get('clusterController');
    var _this = this;
    if (!App.isEmptyObject(App.db.get('tmp', 'ambariProperties')) && !this.get('ambariProperties')) {
      this.set('ambariProperties', App.db.get('tmp', 'ambariProperties'));
      return;
    }
    if (App.isEmptyObject(clusterController.get('ambariProperties'))) {
      clusterController.loadAmbariProperties().done(function (data) {
        _this.formatAmbariProperties(data.RootServiceComponents.properties);
      });
    } else {
      this.formatAmbariProperties(clusterController.get('ambariProperties'));
    }
  },

  formatAmbariProperties: function formatAmbariProperties(properties) {
    var defaults = {
      threshold: "60",
      ambari_server_host: location.hostname,
      check_execute_list: "db_connection_check"
    };
    var properties = App.permit(properties, ['jdk.name', 'jdk_location', 'java.home']);
    var renameKey = function renameKey(oldKey, newKey) {
      if (properties[oldKey]) {
        defaults[newKey] = properties[oldKey];
        delete properties[oldKey];
      }
    };
    renameKey('java.home', 'java_home');
    renameKey('jdk.name', 'jdk_name');
    $.extend(properties, defaults);
    App.db.set('tmp', 'ambariProperties', properties);
    this.set('ambariProperties', properties);
  },
  /**
   * `Action` method for starting connect to current database.
   *
   * @method connectToDatabase
   **/
  connectToDatabase: function connectToDatabase() {
    if (this.get('isBtnDisabled')) return;
    this.set('isRequestResolved', false);
    App.db.set('tmp', this.get('parentView.service.serviceName') + '_connection', {});
    this.setConnectingStatus(true);
    if (App.get('testMode')) {
      this.startPolling();
    } else {
      this.runCheckConnection();
    }
  },

  /**
   * runs check connections methods depending on service
   * @return {void}
   * @method runCheckConnection
   */
  runCheckConnection: function runCheckConnection() {
    if (this.get('parentView.service.serviceName') === 'KERBEROS') {
      this.runKDCCheck();
    } else {
      this.createCustomAction();
    }
  },

  /**
   * send ajax request to perforn kdc host check
   * @return {App.ajax}
   * @method runKDCCheck
   */
  runKDCCheck: function runKDCCheck() {
    return App.ajax.send({
      name: 'admin.kerberos_security.test_connection',
      sender: this,
      data: {
        kdcHostname: this.get('masterHostName')
      },
      success: 'onRunKDCCheckSuccess',
      error: 'onCreateActionError'
    });
  },

  /**
   *
   * @param data
   */
  onRunKDCCheckSuccess: function onRunKDCCheckSuccess(data) {
    var statusCode = {
      success: 'REACHABLE',
      failed: 'UNREACHABLE'
    };
    if (data == statusCode['success']) {
      this.setResponseStatus('success');
    } else {
      this.setResponseStatus('failed');
    }
    this.set('responseFromServer', data);
  },

  /**
   * Run custom action for database connection.
   *
   * @method createCustomAction
   **/
  createCustomAction: function createCustomAction() {
    var isServiceInstalled = App.Service.find(this.get('parentView.service.serviceName')).get('isLoaded');
    var params = $.extend(true, {}, { db_name: this.get('databaseName').toLowerCase() }, this.get('connectionProperties'), this.get('ambariProperties'));
    App.ajax.send({
      name: isServiceInstalled ? 'cluster.custom_action.create' : 'custom_action.create',
      sender: this,
      data: {
        requestInfo: {
          parameters: params
        },
        filteredHosts: [this.get('masterHostName')]
      },
      success: 'onCreateActionSuccess',
      error: 'onCreateActionError'
    });
  },
  /**
   * Run updater if task is created successfully.
   *
   * @method onConnectActionS
   **/
  onCreateActionSuccess: function onCreateActionSuccess(data) {
    this.set('currentRequestId', data.Requests.id);
    App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('currentRequestId')
      },
      success: 'setCurrentTaskId'
    });
  },

  setCurrentTaskId: function setCurrentTaskId(data) {
    this.set('currentTaskId', data.items[0].Tasks.id);
    this.startPolling();
  },

  startPolling: function startPolling() {
    if (this.get('isConnecting')) this.getTaskInfo();
  },

  getTaskInfo: function getTaskInfo() {
    var request = App.ajax.send({
      name: 'custom_action.request',
      sender: this,
      data: {
        requestId: this.get('currentRequestId'),
        taskId: this.get('currentTaskId')
      },
      success: 'getTaskInfoSuccess'
    });
    this.set('request', request);
  },

  getTaskInfoSuccess: function getTaskInfoSuccess(data) {
    var task = data.Tasks;
    this.set('responseFromServer', {
      stderr: task.stderr,
      stdout: task.stdout
    });
    if (task.status === 'COMPLETED') {
      var structuredOut = task.structured_out.db_connection_check;
      if (structuredOut.exit_code != 0) {
        this.set('responseFromServer', {
          stderr: task.stderr,
          stdout: task.stdout,
          structuredOut: structuredOut.message
        });
        this.setResponseStatus('failed');
      } else {
        App.db.set('tmp', this.get('parentView.service.serviceName') + '_connection', this.get('preparedDBProperties'));
        this.setResponseStatus('success');
      }
    }
    if (task.status === 'FAILED') {
      this.setResponseStatus('failed');
    }
    if (/PENDING|QUEUED|IN_PROGRESS/.test(task.status)) {
      Em.run.later(this, function () {
        this.startPolling();
      }, this.get('pollInterval'));
    }
  },

  onCreateActionError: function onCreateActionError(jqXhr, status, errorMessage) {
    this.setResponseStatus('failed');
    this.set('responseFromServer', errorMessage);
  },

  setResponseStatus: function setResponseStatus(isSuccess) {
    var isSuccess = isSuccess == 'success';
    this.setConnectingStatus(false);
    this.set('responseCaption', isSuccess ? Em.I18n.t('services.service.config.database.connection.success') : Em.I18n.t('services.service.config.database.connection.failed'));
    this.set('isConnectionSuccess', isSuccess);
    this.set('isRequestResolved', true);
    if (this.get('logsPopup')) {
      var statusString = isSuccess ? 'common.success' : 'common.error';
      this.set('logsPopup.header', Em.I18n.t('services.service.config.connection.logsPopup.header').format(this.get('databaseName'), Em.I18n.t(statusString)));
    }
  },
  /**
   * Switch captions and statuses for active/non-active request.
   *
   * @method setConnectionStatus
   * @param {Boolean} [active]
   */
  setConnectingStatus: function setConnectingStatus(active) {
    if (active) {
      this.set('responseCaption', Em.I18n.t('services.service.config.database.connection.inProgress'));
    }
    this.set('controller.testConnectionInProgress', !!active);
    this.set('btnCaption', !!active ? Em.I18n.t('services.service.config.database.btn.connecting') : Em.I18n.t('services.service.config.database.btn.idle'));
    this.set('isConnecting', !!active);
  },
  /**
   * Set view to init status.
   *
   * @method restore
   **/
  restore: function restore() {
    if (this.get('request')) {
      this.get('request').abort();
      this.set('request', null);
    }
    this.set('responseCaption', null);
    this.set('responseFromServer', null);
    this.setConnectingStatus(false);
    this.set('isRequestResolved', false);
  },
  /**
   * `Action` method for showing response from server in popup.
   *
   * @method showLogsPopup
   **/
  showLogsPopup: function showLogsPopup() {
    if (this.get('isConnectionSuccess')) return;
    var _this = this;
    var statusString = this.get('isRequestResolved') ? 'common.error' : 'common.testing';
    var popup = App.showAlertPopup(Em.I18n.t('services.service.config.connection.logsPopup.header').format(this.get('databaseName'), Em.I18n.t(statusString)), null, function () {
      _this.set('logsPopup', null);
    });
    popup.reopen({
      onClose: function onClose() {
        this._super();
        _this.set('logsPopup', null);
      }
    });
    if (_typeof(this.get('responseFromServer')) == 'object') {
      popup.set('bodyClass', Em.View.extend({
        checkDBConnectionView: _this,
        templateName: require('templates/common/error_log_body'),
        openedTask: function () {
          return this.get('checkDBConnectionView.responseFromServer');
        }.property('checkDBConnectionView.responseFromServer.stderr', 'checkDBConnectionView.responseFromServer.stdout', 'checkDBConnectionView.responseFromServer.structuredOut')
      }));
    } else {
      popup.set('body', this.get('responseFromServer'));
    }
    this.set('logsPopup', popup);
    return popup;
  }
});

/**
 * View for switch group text
 *
 * @type {Em.View}
 */
App.SwitchToGroupView = Em.View.extend({

  group: null,

  tagName: 'a',

  classNames: ['action'],

  template: Ember.Handlebars.compile('{{ view.group.switchGroupTextShort }}'),

  didInsertElement: function didInsertElement() {
    var self = this;
    App.tooltip($(self.get('element')), {
      placement: 'top',
      title: self.get('group.switchGroupTextFull')
    });
  },

  willDestroyElement: function willDestroyElement() {
    $(this.get('element')).tooltip('destroy');
  },

  click: function click() {
    this.get('controller').selectConfigGroup({ context: this.get('group') });
  }
});

/**
 * View with input field used to repo-version URLs
 * @type {*}
 */
App.BaseUrlTextField = Ember.TextField.extend({

  layout: Ember.Handlebars.compile('<div class="pull-left">{{yield}}</div> {{#if view.valueWasChanged}}<div class="pull-right"><a class="btn-sm" {{action "restoreValue" target="view"}}><i class="icon-undo"></i></a></div>{{/if}}'),

  /**
   * Binding in the template
   * @type {App.RepositoryVersion}
   */
  repository: null,

  /**
   * @type {string}
   */
  valueBinding: 'repository.baseUrl',

  /**
   * @type {string}
   */
  defaultValue: '',

  /**
   *  validate base URL
   */
  validate: function () {
    if (this.get('repository.skipValidation')) {
      this.set('repository.hasError', false);
    } else {
      this.set('repository.hasError', !validator.isValidBaseUrl(this.get('value')));
    }
    this.get('parentView').uiValidation();
  }.observes('value', 'repository.skipValidation'),

  /**
   * Determines if user have put some new value
   * @type {boolean}
   */
  valueWasChanged: function () {
    return this.get('value') !== this.get('defaultValue');
  }.property('value', 'defaultValue'),

  didInsertElement: function didInsertElement() {
    this.set('defaultValue', this.get('value'));
  },

  /**
   * Restore value and unset error-flag
   * @method restoreValue
   */
  restoreValue: function restoreValue() {
    this.set('value', this.get('defaultValue'));
  }
});

});

require.register("views/common/custom_date_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

module.exports = Em.Object.create({

  startTime: null,

  endTime: null,

  customDuration: null,

  showCustomDatePopup: function showCustomDatePopup(primary, secondary, defaults) {
    var self = this;
    defaults = defaults || {
      startDate: null,
      hoursForStart: null,
      minutesForStart: null,
      middayPeriodForStart: null,
      duration: null,
      endDate: null,
      hoursForEnd: null,
      minutesForEnd: null,
      middayPeriodForEnd: null
    };

    return App.ModalPopup.show({
      header: Em.I18n.t('jobs.table.custom.date.header'),
      onPrimary: function onPrimary() {
        if (primary) {
          primary();
        }
        this._super();
      },
      onSecondary: function onSecondary() {
        if (secondary) {
          secondary();
        }
        this._super();
      },
      onClose: function onClose() {
        if (secondary) {
          secondary();
        }
        this._super();
      },
      disablePrimary: false,
      bodyClass: App.JobsCustomDatesSelectView.extend({
        controller: self,
        customDateFormFields: Em.Object.create(defaults)
      })
    });
  }

});

});

require.register("views/common/editable_list", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.EditableList = Ember.View.extend({

  templateName: require('templates/common/editable_list'),

  items: [], // items show on list
  resources: [], // original resources including all items
  itemsOriginal: [], //backup of items

  editMode: false,
  input: '',
  typeahead: [],
  selectedTypeahed: 0,

  isCaseSensitive: true,

  init: function init() {
    this._super();
    this.updateItemsOriginal();
    this.set('input', '');
    this.set('editMode', false);
  },

  updateItemsOriginal: function () {
    this.set('itemsOriginal', Em.copy(this.get('items')));
  }.observes('items'),

  onPrimary: function onPrimary(event) {
    this.set('editMode', false);
    this.set('input', '');
    if (event) {
      event.stopPropagation();
    }
  },

  onSecondary: function onSecondary() {
    // restore all items
    this.set('items', this.get('itemsOriginal'));
    this.set('input', '');
    this.set('editMode', false);
  },

  enableEditMode: function enableEditMode() {
    this.set('input', '');
    this.set('editMode', true);
  },

  removeFromItems: function removeFromItems(event) {
    var items = this.get('items');
    items.removeObject(event.context);
    this.set('input', '');
  },

  /**
   * available items to add, will show up typing ahead
   */
  availableItemsToAdd: function () {
    var allItems = Em.copy(this.get('resources'));
    var isCaseSensitive = this.get('isCaseSensitive');
    var input = isCaseSensitive ? this.get('input') : this.get('input').toLowerCase();
    var toRemove = [];
    var existed = this.get('items');
    allItems.forEach(function (item) {
      var nameToCompare = isCaseSensitive ? item.name : item.name.toLowerCase();
      if (nameToCompare.indexOf(input) < 0 || existed.findProperty('name', item.name)) {
        toRemove.push(item);
      }
    });
    toRemove.forEach(function (item) {
      allItems.removeObject(item);
    });
    return allItems;
  }.property('items', 'input'),

  addItem: function addItem(event) {
    var items = this.get('items');
    var toAdd;
    if (event.context) {
      toAdd = event.context;
    } else if (this.get('typeahead.length') > 0) {
      toAdd = this.get('typeahead')[this.get('selectedTypeahead')];
    }
    items.pushObject(event.context);
    this.set('input', '');
  },

  updateTypeahead: function () {
    var newValue = this.get('input');
    var self = this;
    if (newValue) {
      var newValue = newValue.split(',');
      if (newValue.length > 1) {
        // If coma separated string, then just add all items to list
        newValue.forEach(function (item) {
          self.addItem(item);
        });
        self.clearInput();
      } else {
        // Load typeahed items based on current input
        var items = self.get('availableItemsToAdd');
        self.set('typeahead', items);
        self.set('selectedTypeahed', 0);
      }
    } else {
      self.set('typeahead', []);
      self.set('selectedTypeahed', 0);
    }
  }.observes('input')
});

});

require.register("views/common/export_metrics_menu_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ExportMetricsMenuView = Em.View.extend({

  classNameBindings: ['parentView.isExportMenuHidden::open', ':export-graph-list-container'],

  templateName: require('templates/common/export_metrics_menu')

});

});

require.register("views/common/filter_combo_cleanable", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Filter component with custom dropdown and cross-circled cleaner. Build on Twitter Bootstrap styles
 * @type {*}
 */
App.FilterComboCleanableView = Ember.View.extend({
  templateName: require('templates/common/filter_combo_cleanable'),

  classNames: ['filter-combobox', 'input-group'],

  didInsertElement: function didInsertElement() {
    App.popover(this.$("input[type=text]"), {
      title: this.get('popoverDescription')[0],
      content: this.get('popoverDescription')[1],
      placement: 'bottom',
      trigger: 'hover'
    });
    this.clearFilter();
  },

  /**
   * @type {string}
   */
  placeHolder: Em.I18n.t('common.combobox.placeholder'),

  /**
   * Onclick handler for dropdown menu
   * @param event
   */
  selectFilterColumn: function selectFilterColumn(event) {
    var column = event.context;
    if (!column.get('isDisabled')) {
      column.set('selected', !column.get('selected'));
    }
  },

  filterNotEmpty: Em.computed.gt('filter.length', 0),

  /**
   * true if any of filter columns is selected
   * in this case clear filter row should be shown
   * @type {boolean}
   */
  showClearFilter: Em.computed.someBy('columns', 'selected', true),

  /**
   * clears all filter columns.
   * @method clearFilterColumn
   */
  clearFilterColumn: function clearFilterColumn() {
    this.get('columns').filter(function (column) {
      return !column.get('isDisabled');
    }).setEach('selected', false);
  },

  /**
   * clear Filter textfield
   */
  clearFilter: function clearFilter() {
    this.set('filter', '');
  }
});

});

require.register("views/common/filter_combobox", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Filter component with custom dropdown. Build on Twitter Bootstrap styles
 * @type {*}
 */
App.FilterComboboxView = Ember.View.extend({
  templateName: require('templates/common/filter_combobox'),

  classNames: ['filter-combobox', 'input-group'],

  //filter: '', binding outside
  //columns: [], binding outside

  placeHolder: Em.I18n.t('common.combobox.placeholder'),

  /**
   * Onclick handler for dropdown menu
   * @param event
   */
  selectFilterColumn: function selectFilterColumn(event) {
    var column = event.context;
    column.set('selected', !column.get('selected'));
  }

});

});

require.register("views/common/filter_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Wrapper View for all filter components. Layout template and common actions are located inside of it.
 * Logic specific for data component(input, select, or custom multi select, which fire any changes on interface) are
 * located in inner view - <code>filterView</code>.
 *
 * If we want to have input filter, put <code>textFieldView</code> to it.
 * All inner views implemented below this view.
 * @type {*}
 */

var App = require('app');

var validator = require('utils/validator');

var wrapperView = Ember.View.extend({
  classNames: ['view-wrapper'],
  layout: Ember.Handlebars.compile('{{yield}} <a href="#" {{action "clearFilter" target="view"}} class="ui-icon ui-icon-circle-close"></a>'),
  template: Ember.Handlebars.compile('{{#if view.fieldId}}<input type="hidden" id="{{unbound view.fieldId}}" value="" />{{/if}}' + '{{view view.filterView}}' + '{{#if view.showApply}}<button {{action "setValueOnApply" target="view"}} class="apply-btn btn btn-default"><span>{{t common.apply}}</span></button>{{/if}} '),

  value: null,

  /**
   * Column index
   */
  column: null,

  /**
   * If this field is exists we dynamically create hidden input element and set value there.
   * Used for some cases, where this values will be used outside of component
   */
  fieldId: null,

  /**
   * This option is useful for Em.Select, if you want get events when same option selected more than one time a raw.
   * This property is Object[] (for example see App.MainConfigHistoryView.modifiedFilterView),
   * that have followed structure:
   *
   * <code>
   * [
   *  {
   *    values: ['DefinedValue1', 'DefinedValue2'], // this properties should be defined in `content` property
   *    displayAs: 'Choose me' // this value will be displayed
   *  }
   * ]
   * </code>
   * @type {Array}
   *
   **/
  //TODO delete if will not be used
  triggeredOnSameValue: null,

  clearFilter: function clearFilter() {
    this.set('value', this.get('emptyValue'));
    if (this.get('setPropertyOnApply')) {
      this.setValueOnApply();
    }
    return false;
  },

  setValueOnApply: function setValueOnApply() {
    if (this.get('value') == null) {
      this.set('value', '');
    }
    this.set(this.get('setPropertyOnApply'), this.get('value'));
    return false;
  },

  /**
   * Use to determine whether filter is clear or not. Also when we want to set empty value
   */
  emptyValue: '',

  /**
   * empty value that actually applied to filtering on server,
   * used with Select filter where displayed and actual filter value are different
   */
  appliedEmptyValue: "",

  /**
   * Whether our <code>value</code> is empty or not
   * @return {Boolean}
   */
  isEmpty: function isEmpty() {
    if (Em.isNone(this.get('value'))) {
      return true;
    }
    return this.get('value').toString() === this.get('emptyValue').toString();
  },

  setValue: function setValue(value) {
    this.set('value', value);
  },

  /**
   * Show/Hide <code>Clear filter</code> button.
   * Also this method updates computed field related to <code>fieldId</code> if it exists.
   * Call <code>onChangeValue</code> callback when everything is done.
   */
  showClearFilter: function () {
    if (!this.get('parentNode')) {
      return;
    }
    // get the sort view element in the same column to current filter view to highlight them together
    var relatedSort = $(this.get('element')).parents('thead').find('.sort-view-' + this.get('column'));
    if (this.isEmpty()) {
      this.get('parentNode').removeClass('active-filter');
      this.get('parentNode').addClass('notActive');
      relatedSort.removeClass('active-sort');
    } else {
      this.get('parentNode').removeClass('notActive');
      this.get('parentNode').addClass('active-filter');
      relatedSort.addClass('active-sort');
    }

    if (this.get('fieldId')) {
      this.$('> input').eq(0).val(this.get('value'));
    }

    this.onChangeValue();
  }.observes('value'),

  /**
   * Callback for value changes
   */
  onChangeValue: Em.K,

  /**
   * Filter components is located here. Should be redefined
   */
  filterView: Em.View,

  /**
   * Update class of parentNode(hide clear filter button) on page load
   */
  didInsertElement: function didInsertElement() {
    var parent = this.$().parent();
    this.set('parentNode', parent);
    parent.addClass('notActive');
    //TODO delete if will not be used
    //this.checkSelectSpecialOptions();
  },

  /**
   * Check for Em.Select that should use dispatching event when option with same value selected more than one time.
   **/
  //TODO delete if will not be used
  checkSelectSpecialOptions: function checkSelectSpecialOptions() {
    // check predefined property
    if (!this.get('triggeredOnSameValue') || !this.get('triggeredOnSameValue').length) return;
    // add custom additional observer that will handle property changes
    this.addObserver('value', this, 'valueCustomObserver');
    // get the full class attribute to find our select
    var classInlineAttr = this.get('fieldType').split(',').map(function (className) {
      return '.' + className.trim();
    }).join('');
    this.set('classInlineAttr', classInlineAttr);
    this.get('triggeredOnSameValue').forEach(function (triggeredValue) {
      triggeredValue.values.forEach(function (value, index) {
        // option with property `value`
        var $optionEl = $(this.get('element')).find(classInlineAttr).find('option[value="' + value + '"]');
        // should be displayed with `displayAs` caption
        $optionEl.text(triggeredValue.displayAs);
        // the second one option should be hidden
        // as the result, on init stage we show only one option that could be selected
        if (index === 1) {
          $optionEl.css('display', 'none');
        }
      }, this);
    }, this);
  },
  /**
   *
   * Custom observer that used for special case of Em.Select related to dispatching event
   * when option with same value selected more than one time.
   *
   **/
  //TODO delete if will not be used
  valueCustomObserver: function valueCustomObserver() {
    var hiddenValue;
    this.get('triggeredOnSameValue').forEach(function (triggeredValue) {
      var values = triggeredValue.values;
      // find current selected value from `values` list
      var currentValueIndex = values.indexOf(this.get('value'));
      if (currentValueIndex < 0) return;
      // value assigned to hidden option
      hiddenValue = values[Number(currentValueIndex === 0)];
    }, this);
    if (hiddenValue) {
      // our select
      var $select = $(this.get('element')).find(this.get('classInlineAttr'));
      // now hide option with current value
      $select.find('option[value="{0}"]'.format(this.get('value'))).css('display', 'none');
      // and show option that was hidden
      $select.find('option[value="{0}"]'.format(hiddenValue)).css('display', 'block');
    }
  }
});

/**
 * Simple input control for wrapperView
 */
var textFieldView = Ember.TextField.extend({
  classNames: ['input-sm', 'form-control'],
  type: 'text',
  placeholder: Em.I18n.t('any'),
  valueBinding: "parentView.value"
});

/**
 * Simple multiselect control for wrapperView.
 * Used to render blue button and popup, which opens on button click.
 * All content related logic should be implemented manually outside of it
 */
var componentFieldView = Ember.View.extend({
  classNames: ['btn-group'],
  classNameBindings: ['isFilterOpen:open:'],

  /**
   * Whether popup is shown or not
   */
  isFilterOpen: false,

  /**
   * We have <code>value</code> property similar to inputs <code>value</code> property
   */
  valueBinding: 'parentView.value',

  /**
   * Clear filter to initial state
   */
  clearFilter: function clearFilter() {
    this.set('value', '');
  },

  /**
   * Onclick handler for <code>cancel filter</code> button
   */
  closeFilter: function closeFilter() {
    $(document).unbind('click');
    this.set('isFilterOpen', false);
  },

  /**
   * Onclick handler for <code>apply filter</code> button
   */
  applyFilter: function applyFilter() {
    this.closeFilter();
  },

  /**
   * Onclick handler for <code>show component filter</code> button.
   * Also this function is used in some other places
   */
  clickFilterButton: function clickFilterButton() {
    var self = this;
    this.toggleProperty('isFilterOpen');
    if (this.get('isFilterOpen')) {

      var dropDown = this.$('.filter-components');
      var firstClick = true;
      $(document).bind('click', function (e) {
        if (!firstClick && !$(e.target).closest(dropDown).length) {
          self.set('isFilterOpen', false);
          $(document).unbind('click');
        }
        firstClick = false;
      });
    }
  }
});

/**
 * Simple select control for wrapperView
 */
var getSelectFieldView = function getSelectFieldView() {
  return App.DropdownView.extend({
    selectionBinding: 'parentView.selected',
    contentBinding: 'parentView.content',
    optionValuePath: "value",
    optionLabelPath: "label"
  });
};

/**
 * Result object, which will be accessible outside
 * @type {Object}
 */
module.exports = {
  /**
   * You can access wrapperView outside
   */
  wrapperView: wrapperView,

  /**
   * And also controls views if need it
   */
  textFieldView: textFieldView,
  componentFieldView: componentFieldView,

  /**
   * Quick create input filters
   * @param config parameters of <code>wrapperView</code>
   */
  createTextView: function createTextView(config) {
    config.fieldType = config.fieldType || 'input-medium';
    config.filterView = textFieldView.extend({
      classNames: [config.fieldType]
    });

    return wrapperView.extend(config);
  },

  /**
   * Quick create multiSelect filters
   * @param config parameters of <code>wrapperView</code>
   */
  createComponentView: function createComponentView(config) {
    config.clearFilter = function () {
      this.forEachChildView(function (item) {
        if (item.clearFilter) {
          item.clearFilter();
        }
      });
      return false;
    };

    return wrapperView.extend(config);
  },

  /**
   * Quick create select filters
   * @param config parameters of <code>wrapperView</code>
   */
  createSelectView: function createSelectView(config) {
    config.fieldType = config.fieldType || 'input-medium';
    config.filterView = getSelectFieldView().extend({
      classNames: config.fieldType.split(',').concat('display-inline-block'),
      attributeBindings: ['disabled', 'multiple'],
      disabled: false
    });

    config.valueBinding = 'selected.value';
    config.clearFilter = function () {
      this.set('selected', this.get('content').findProperty('value', this.get('emptyValue')));
    };
    config.setValue = function (value) {
      this.set('selected', this.get('content').findProperty('value', value));
    };
    return wrapperView.extend(config);
  },
  /**
   * returns the filter function, which depends on the type of property
   * @param type
   * @param isGlobal check is search global
   * @return {Function}
   */
  getFilterByType: function getFilterByType(type, isGlobal) {
    switch (type) {
      case 'ambari-bandwidth':
        return function (rowValue, rangeExp) {
          var compareChar = isNaN(rangeExp.charAt(0)) ? rangeExp.charAt(0) : false;
          var compareScale = rangeExp.charAt(rangeExp.length - 1);
          var compareValue = parseFloat(rangeExp.substr(compareChar ? 1 : 0));
          if (rangeExp.length === 1 && compareChar !== false) {
            // User types only '=' or '>' or '<', so don't filter column values
            return true;
          }
          var oneSymbolScales = {
            k: 1024,
            m: 1048576,
            g: 1073741824
          };
          var twoSymbolsScales = {
            KB: 1024,
            MB: 1048576,
            GB: 1073741824
          };

          compareValue *= oneSymbolScales[compareScale] ? oneSymbolScales[compareScale] : oneSymbolScales.g; // default value in GB
          rowValue = jQuery(rowValue).text() ? jQuery(rowValue).text() : rowValue;

          var convertedRowValue;
          if (rowValue === '<1KB') {
            convertedRowValue = 1;
          } else {
            var rowValueScale = rowValue.substr(rowValue.length - 2, 2);
            if (twoSymbolsScales[rowValueScale]) {
              convertedRowValue = parseFloat(rowValue) * twoSymbolsScales[rowValueScale];
            }
          }

          switch (compareChar) {
            case '<':
              return compareValue > convertedRowValue;
            case '>':
              return compareValue < convertedRowValue;
            case false:
            case '=':
              return compareValue === convertedRowValue;
            default:
              return false;
          }
        };
      case 'duration':
        return function (rowValue, rangeExp) {
          var compareChar = isNaN(rangeExp.charAt(0)) ? rangeExp.charAt(0) : false;
          var compareScale = rangeExp.charAt(rangeExp.length - 1);
          var compareValue = parseFloat(rangeExp.substr(compareChar ? 1 : 0));
          if (rangeExp.length === 1 && compareChar !== false) {
            // User types only '=' or '>' or '<', so don't filter column values
            return true;
          }
          var oneSymbolsScales = {
            s: 1000,
            m: 60000,
            h: 3600000
          };
          compareValue *= oneSymbolsScales[compareScale] ? oneSymbolsScales[compareScale] : oneSymbolsScales.s;
          rowValue = jQuery(rowValue).text() ? jQuery(rowValue).text() : rowValue;

          switch (compareChar) {
            case '<':
              return compareValue > rowValue;
            case '>':
              return compareValue < rowValue;
            case false:
            case '=':
              return compareValue == rowValue;
            default:
              return false;
          }
        };
      case 'date':
        return function (rowValue, rangeExp) {
          var timePassed = App.dateTime() - rowValue;
          switch (rangeExp) {
            case 'Past 1 hour':
              return timePassed <= 3600000;
            case 'Past 1 Day':
              return timePassed <= 86400000;
            case 'Past 2 Days':
              return timePassed <= 172800000;
            case 'Past 7 Days':
              return timePassed <= 604800000;
            case 'Past 14 Days':
              return timePassed <= 1209600000;
            case 'Past 30 Days':
              return timePassed <= 2592000000;
            case 'Any':
              return true;
            default:
              return false;
          }
        };
      case 'number':
        return function (rowValue, rangeExp) {
          var compareChar = rangeExp.charAt(0);
          var compareValue;
          if (rangeExp.length === 1) {
            if (isNaN(parseInt(compareChar, 10))) {
              // User types only '=' or '>' or '<', so don't filter column values
              return true;
            }
            compareValue = parseFloat(parseFloat(rangeExp).toFixed(2));
          } else {
            if (isNaN(parseInt(compareChar, 10))) {
              compareValue = parseFloat(parseFloat(rangeExp.substr(1, rangeExp.length)).toFixed(2));
            } else {
              compareValue = parseFloat(parseFloat(rangeExp.substr(0, rangeExp.length)).toFixed(2));
            }
          }
          rowValue = parseFloat(jQuery(rowValue).text() ? jQuery(rowValue).text() : rowValue);
          switch (compareChar) {
            case '<':
              return compareValue > rowValue;
            case '>':
              return compareValue < rowValue;
            case '=':
            default:
              return compareValue === rowValue;
          }
        };
      case 'sub-resource':
        return function (origin, compareValue) {
          if (!Array.isArray(compareValue) || !compareValue.length) {
            return true;
          }

          return origin.some(function (item) {
            for (var i = 0, l = compareValue.length; i < l; i++) {
              if (Array.isArray(compareValue[i].value)) {
                if (!compareValue[i].value.contains(item.get(compareValue[i].property))) return false;
              } else {
                if (item.get(compareValue[i].property) !== compareValue[i].value) return false;
              }
            }
            return true;
          });
        };
      case 'multiple':
        return function (origin, compareValue) {
          var options = compareValue.split(',');
          var rowValue = typeof origin === "string" ? origin : origin.mapProperty('componentName').join(" ");
          var str = new RegExp(compareValue, "i");
          for (var i = 0; i < options.length; i++) {
            if (!isGlobal) {
              str = new RegExp('(\\W|^)' + options[i] + '(\\W|$)');
            }
            if (rowValue.search(str) !== -1) {
              return true;
            }
          }
          return false;
        };
      case 'boolean':
      case 'select':
        return function (origin, compareValue) {
          return origin === compareValue;
        };
      case 'os':
        return function (origin, compareValue) {
          return origin.getEach('osType').contains(compareValue);
        };
      case 'range':
        return function (origin, compareValue) {
          if (compareValue[1] && compareValue[0]) {
            return origin >= compareValue[0] && origin <= compareValue[1];
          }
          if (compareValue[0]) {
            return origin >= compareValue[0];
          }
          if (compareValue[1]) {
            return origin <= compareValue[1];
          }
          return true;
        };
      case 'alert_status':
        /**
         * origin - alertDefinition.summary
         * compareValue - "OK|WARNING|CRITICAL|UNKNOWN|PENDING"
         * PENDING means that OK is 0, WARNING is 0, CRITICAL is 0 and UNKNOWN is 0
         */
        return function (origin, compareValue) {
          if ('PENDING' === compareValue) {
            var isPending = true;
            Em.keys(origin).forEach(function (state) {
              if (origin[state] && (origin[state].count > 0 || origin[state].maintenanceCount > 0)) {
                isPending = false;
              }
            });
            return isPending;
          }
          return !!origin[compareValue] && (origin[compareValue].count > 0 || origin[compareValue].maintenanceCount > 0);
        };
      case 'alert_group':
        return function (origin, compareValue) {
          return origin.mapProperty('id').contains(compareValue);
        };
      case 'enable_disable':
        return function (origin, compareValue) {
          return origin === (compareValue === 'enabled');
        };
      case 'file_extension':
        return function (origin, compareValue) {
          return origin.endsWith(compareValue);
        };
      case 'string':
      default:
        return function (origin, compareValue) {
          if (validator.isValidMatchesRegexp(compareValue)) {
            var escapedCompareValue = compareValue.replace("(", "\\(").replace(")", "\\)").trim();
            var regex = new RegExp(escapedCompareValue, "i");
            return regex.test(origin);
          }
          return false;
        };
    }
  },

  getComputedServicesList: function getComputedServicesList() {
    return Em.computed('App.router.clusterController.isLoaded', function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(App.Service.find().map(function (service) {
        return {
          value: service.get('serviceName'),
          label: service.get('displayName')
        };
      })).concat({
        value: 'AMBARI',
        label: Em.I18n.t('app.name')
      });
    });
  }

};

});

require.register("views/common/form/datepicker_view", function(exports, require, module) {
'use strict';

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DatepickerFieldView = Em.TextField.extend({
  value: '',
  dateFormat: 'dd/mm/yy',
  maxDate: new Date(),

  didInsertElement: function didInsertElement() {
    this.$().datetimepicker({
      format: this.get('dateFormat'),
      endDate: this.get('maxDate')
    });
    this.$().on('changeDate', this.onChangeDate.bind(this));
  },

  onChangeDate: function onChangeDate(e) {
    this.set('value', e.target.value);
  }
});

});

require.register("views/common/form/dropdown", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DropdownView = Em.View.extend({

  templateName: require('templates/common/form/dropdown'),

  qaAttr: '',

  selection: null,

  value: '',

  optionValuePath: '',

  optionLabelPath: '',

  /**
   * Used to prevent infinite loop because of cyclic updating of value and selection
   * @type {Boolean}
   */
  isUpdating: false,

  change: Em.K,

  didInsertElement: function didInsertElement() {
    this.observeEmptySelection();
  },

  /**
   * value should be updated after updating selection and vise versa
   */
  onValueOrSelectionUpdate: function (context, property) {
    var selection = this.get('selection');
    var value = this.get('value');
    var content = this.get('content');
    var optionValuePath = this.get('optionValuePath');
    this.set('isUpdating', true);
    if (property === 'value') {
      this.set('selection', optionValuePath ? content.findProperty(optionValuePath, value) : value);
    } else if (property === 'selection') {
      this.set('value', selection && Em.getWithDefault(selection, optionValuePath, selection) || '');
    }
    this.set('isUpdating', false);
  }.observes('selection', 'value'),

  selectOption: function selectOption(option) {
    this.set('selection', option.context);
    this.change();
  },

  /**
   * Set default selection
   */
  observeEmptySelection: function () {
    if (this.get('content.length') && !this.get('selection')) this.set('selection', this.get('content')[0]);
  }.observes('content')

});

App.DropdownOptionView = Em.View.extend({

  tagName: 'span',

  template: Em.Handlebars.compile('{{view.optionLabel}}'),

  optionLabel: function () {
    var optionLabelPath = this.get('optionLabelPath');
    var option = this.get('option');
    if (!option) return '';
    if (optionLabelPath) return Em.get(option, optionLabelPath);
    return option;
  }.property('option', 'optionLabelPath')

});

});

require.register("views/common/form/field", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FormFieldTemplate = Em.View.extend({
  templateDir: 'templates/common/form/',
  defaultTemplate: 'field',
  classNames: ["control-group"],
  init: function init() {
    this._super();
    this.updateTemplate();
  },
  updateTemplate: function () {
    var field = this.get('field');
    if (field) {
      var templateName = '';
      switch (field.get('displayType')) {
        case 'checkbox':
          templateName = field.get('displayType');
          break;
        default:
          templateName = this.get('defaultTemplate');
          break;
      }

      this.set('templateName', require(this.get('templateDir') + templateName));
    }
  }.observes('field')
});

});

require.register("views/common/form/manage_credentials_form_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var credentialsUtils = require('utils/credentials');

App.ManageCredentialsFormView = Em.View.extend({
  templateName: require('templates/common/form/manage_credentilas_form'),
  viewName: 'manageCredentialsForm',
  principal: "",
  password: "",

  /**
   * Store Admin credentials checkbox value
   *
   * @type {boolean}
   */
  storeCredentials: false,

  /**
   * Status of persistent storage. Returns <code>true</code> if persistent storage is available.
   * @type {boolean}
   */
  storePersisted: Em.computed.alias('App.isCredentialStorePersistent'),

  /**
   * Disable checkbox if persistent storage not available
   *
   * @type {boolean}
   */
  checkboxDisabled: Em.computed.not('storePersisted'),

  /**
   * Credentials can be removed, in case when they stored to persistent secure storage.
   *
   * @type {boolean}
   */
  isRemovable: false,

  /**
   * Remove button disabled status.
   *
   * @type {boolean}
   */
  isRemoveDisabled: true,

  /**
   * Signalize that action was performed and waiting for result.
   *
   * @type {boolean}
   */
  isActionInProgress: false,

  /**
   * Status message of performed action, e.g. remove or save credentials. Set to <code>false</code> to hide this message.
   *
   * @type {boolean|string}
   */
  actionStatus: false,

  /**
   * Validation message error regarding principal name.
   * When value is <code>false</code> error is hidden.
   *
   * @type {boolean|string}
   */
  passwordError: false,

  /**
   * Validation message error regarding principal name field.
   * When value is <code>false</code> error is hidden.
   *
   * @type {boolean|string}
   */
  principalError: false,

  isSubmitDisabled: true,

  principalNameObserver: function () {
    this.set('isPrincipalDirty', true);
    if (Em.isEmpty(this.get('principal'))) {
      this.set('principalError', Em.I18n.t('admin.users.editError.requiredField'));
    } else if (/\s+/.test(this.get('principal'))) {
      this.set('principalError', Em.I18n.t('host.spacesValidation'));
    } else {
      this.set('principalError', false);
    }
    this.toggleSubmitState();
  }.observes('principal'),

  passwordObserver: function () {
    this.set('isPasswordDirty', true);
    if (Em.isEmpty(this.get('password'))) {
      this.set('passwordError', Em.I18n.t('admin.users.editError.requiredField'));
    } else {
      this.set('passwordError', false);
    }
    this.toggleSubmitState();
  }.observes('password'),

  /**
   * Toggle submit disable state only when principal and password fields were modified.
   */
  toggleSubmitState: function toggleSubmitState() {
    if (this.get('isPrincipalDirty') && this.get('isPasswordDirty')) {
      this.set('isSubmitDisabled', !!this.get('principalError') || !!this.get('passwordError'));
    }
  },

  /**
   * Returns storage type used to save credentials e.g. <b>persistent</b>, <b>temporary</b> (default)
   *
   * @type {string}
   */
  storageType: function () {
    return credentialsUtils.STORE_TYPES.PERSISTENT;
  }.property('storeCredentials'),

  formHeader: Em.computed.ifThenElse('isRemovable', Em.I18n.t('admin.kerberos.credentials.form.header.stored'), Em.I18n.t('admin.kerberos.credentials.form.header.not.stored')),

  /**
   * Message to display in tooltip regarding persistent storage state.
   *
   * @type {string}
   */
  hintMessage: Em.computed.ifThenElse('storePersisted', Em.I18n.t('admin.kerberos.credentials.store.hint.supported'), Em.I18n.t('admin.kerberos.credentials.store.hint.not.supported')),

  /**
   * Observe changes for principal and password.
   * Hide status message and toggle action progress if performed.
   */
  formInputObserver: function () {
    if (this.get('actionStatus') || this.get('isActionInProgress')) {
      this.setInProgress(false);
      this.set('actionStatus', false);
    }
  }.observes('password', 'principal'),

  didInsertElement: function didInsertElement() {
    this._super();
    App.tooltip(this.$('[rel="tooltip"]'));
  },

  willInsertElement: function willInsertElement() {
    this._super();
    this.prepareContent();
  },

  prepareContent: function prepareContent() {
    var self = this;
    credentialsUtils.credentials(App.get('clusterName'), function (credentials) {
      Em.run.next(function () {
        self.set('isRemovable', credentialsUtils.isKDCCredentialsPersisted(credentials));
        self.set('isRemoveDisabled', !self.get('isRemovable'));
      });
    });
  },

  /**
   * Save credentials action.
   *
   * @returns {boolean|$.Deferred}
   */
  saveKDCCredentials: function saveKDCCredentials() {
    var self = this;
    var dfd = $.Deferred();

    this.setInProgress(true);
    credentialsUtils.createOrUpdateCredentials(App.get('clusterName'), credentialsUtils.ALIAS.KDC_CREDENTIALS, credentialsUtils.createCredentialResource(this.get('principal'), this.get('password'), this.get('storageType'))).always(function () {
      self.setInProgress(false);
      self.prepareContent();
      self.set('actionStatus', Em.I18n.t('common.success'));
      self.get('parentView').set('isCredentialsSaved', true);
      dfd.resolve();
    });
    return dfd.promise();
  },

  /**
   * Remove KDC credentials action.
   *
   * @returns {object} for better testing purpose returns object { deferred: $.Deferred, popup: App.ModalPopup }
   */
  removeKDCCredentials: function removeKDCCredentials() {
    var t = Em.I18n.t;
    var self = this;
    var dfd = $.Deferred();
    this.set('actionStatus', false);
    var popup = App.showConfirmationPopup(function () {
      self.setInProgress(true);
      credentialsUtils.removeCredentials(App.get('clusterName'), credentialsUtils.ALIAS.KDC_CREDENTIALS).always(function () {
        self.setInProgress(false);
        self.prepareContent();
        self.set('actionStatus', Em.I18n.t('common.success'));
        self.get('parentView').set('isCredentialsRemoved', true);
        dfd.resolve();
      });
    }, t('admin.kerberos.credentials.remove.confirmation.body'), function () {}, null, t('yes'));
    popup.set('secondary', t('no'));
    return {
      deferred: dfd,
      popup: popup
    };
  },

  /**
   * Toggle action status and disable/enable appropriate buttons.
   *
   * @param {boolean} [isInProgress=false] progress status
   */
  setInProgress: function setInProgress(isInProgress) {
    if (isInProgress) {
      this.set('actionStatus', false);
      if (this.get('isRemovable')) {
        this.set('isRemoveDisabled', true);
      }
      this.set('isSubmitDisabled', true);
      this.set('isActionInProgress', true);
    } else {
      if (this.get('isRemovable')) {
        this.set('isRemoveDisabled', false);
      }
      this.set('isSubmitDisabled', false);
      this.set('isActionInProgress', false);
    }
  }
});

});

require.register("views/common/form/spinner_input_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SpinnerInputView = Em.View.extend({
  templateName: require('templates/common/form/spinner_input'),
  classNames: ['spinner-input'],

  /**
   * Bind content directly from Handlebars template or any another way.
   * For example:
   * {{view App.SpinnerInputView contentBinding="someContent"}}
   *
   * @property content
   * @type {Object}
   */
  content: {

    /**
     * Minimal value that can be set. (not required)
     *
     * @property [minValue]
     * @type {Number}
     */
    minValue: null,

    /**
     * Maximum value that can be set. (not required)
     *
     * @property [maxValue]
     * @type {Number}
     */
    maxValue: null,

    /**
     * Input value. (required)
     *
     * @property value
     * @type {String|Number}
     */
    value: null,

    /**
     * Config display name. Will be shown beneath the input. (not required)
     *
     * @type {String}
     * @property [label]
     */
    label: null,

    /**
     * If set to true value will toggle with max|min on property exceed value.
     *
     * @property [invertOnOverflow]
     * @type {Boolean}
     */
    invertOnOverflow: false,

    /**
     * Determines if this control should be enabled or disabled
     * It's based on increment_step value
     *
     * @property [enabled]
     * @type {Boolean}
     */
    enabled: true,

    /**
     * Increment value
     *
     * @property [incrementStep]
     * @type {Number}
     */
    incrementStep: 1
  },

  /**
   * Input should be disabled, if whole widget is disabled or if its unit is less than step_increment value
   * @type {boolean}
   */
  computedDisabled: Em.computed.or('!content.enabled', 'disabled'),

  incrementValue: function incrementValue() {
    this.setValue(true);
  },

  decrementValue: function decrementValue() {
    this.setValue(false);
  },

  setValue: function setValue(increment) {
    var incrementStep = this.get('content.incrementStep');
    var value = parseInt(this.get('content.value')) + (increment ? incrementStep : -incrementStep);
    // if minimal required value is specified and decremented property value is less than minimal
    if (!Em.isEmpty(this.get('content.minValue')) && !increment && value < this.get('content.minValue')) {
      // set maximum value if value invert accepted
      value = this.get('content.invertOnOverflow') ? this.get('content.maxValue') + 1 - incrementStep : this.get('content.minValue');
    }
    // if maximum require value is specified and incremented value is more than maximum
    else if (!Em.isEmpty(this.get('content.maxValue')) && increment && value > this.get('content.maxValue')) {
        // set minimum if value invert accepted
        value = this.get('content.invertOnOverflow') ? this.get('content.minValue') : this.get('content.maxValue');
      }
    this.set('content.value', value);
  },

  /**
   * Handle keyboard event. Allow digits only and some key maps.
   */
  keyDown: function keyDown(e) {
    var charCode = e.charCode ? e.charCode : e.which;
    if ([46, 8, 9, 27, 13, 110].contains(charCode) || charCode == 65 && e.ctrlKey === true || charCode == 67 && e.ctrlKey === true || charCode == 88 && e.ctrlKey === true || charCode >= 35 && charCode <= 39) {
      return;
    }
    if (charCode == 190 || (e.shiftKey || charCode < 48 || charCode > 57) && (charCode < 96 || charCode > 105)) {
      e.preventDefault();
    }
  },

  /**
   * Change value to maximum if exceeded.
   */
  keyUp: function keyUp(e) {
    if (!Em.isEmpty(this.get('content.maxValue')) && this.get('content.value') > this.get('content.maxValue')) {
      this.set('content.value', this.get('content.maxValue'));
    }
  },

  /**
   * Set value to 0 if user removed it and changed focus to another element.
   */
  focusOut: function focusOut(e) {
    if (Em.isEmpty(this.get('content.value'))) {
      this.set('content.value', 0);
    }
  }
});

});

require.register("views/common/global/spinner", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SpinnerView = Em.View.extend({
  classNames: 'spinner',

  message: '',

  translated: function () {
    return this.get('message') ? Em.I18n.t(this.get('message')) : '';
  }.property('message'),

  template: Em.Handlebars.compile('<i class="icon-spinner icon-spin"></i>{{view.translated}}')

});

});

require.register("views/common/grid", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');

App.GridFilterObject = Em.Object.extend({
  checked: false
});

App.GridFilter = Em.View.extend({
  tagName: "ul",
  classNames: ['filter'],
  templateName: require('templates/common/grid/filter'),
  attributeBindings: ['style'],
  getHeader: function getHeader() {
    return this.get('header');
  },
  filters: Em.computed.alias('header._filters')
});

App.GridHeader = Em.View.extend({
  templateName: require('templates/common/grid/header'),
  tagName: 'th',
  filterable: true,
  showFilter: false,
  getGrid: function getGrid() {
    return this.get('grid');
  },
  _filters: [],
  doFilter: function doFilter() {},
  toggleFilter: function toggleFilter() {
    this.set('showFilter', 1 - this.get('showFilter'));
  },
  applyFilter: function applyFilter() {

    var filters = this.get('_filters');
    var filterValues = [];
    $.each(filters, function () {
      if (this.get('checked')) {
        filterValues.push(this.get('value'));
      }
    });

    var grid = this.get('grid');
    grid.addFilters(this.get('name'), filterValues);
    this.set('showFilter', false);
  },
  init: function init() {
    this._super();
    if (!this.get('_filters').length) {
      this.filterValues();
      var thisHeader = this;
      this.set('filter', App.GridFilter.extend({ header: thisHeader }));
    }
  },

  filterValues: function () {
    var gridFilters = this.get('grid._filters');
    if (gridFilters && gridFilters[this.get('name')]) {
      var filters = this.get('grid._filters')[this.get('name')];
      // there should be something like filter preparing
      var newFilters = [];
      $.each(filters, function (i, v) {
        newFilters.push(App.GridFilterObject.create({ label: v, value: v }));
      });

      this.set('_filters', newFilters);
    }
  }.observes('grid._filters')
});

App.GridRow = Em.View.extend({
  tagName: 'tr',
  init: function init(options) {
    var object = this.get('object');
    var grid = this.get('grid');
    var fieldNames = grid.get('fieldNames');
    var template = '';

    if (fieldNames) {
      $.each(grid.get('fieldNames'), function (i, field) {
        template += "<td>" + object.get(field) + "</td>";
      });

      this.set('template', Em.Handlebars.compile(template));
    }
    return this._super();
  }
});

App.GridPage = Em.Object.extend({
  activeClass: Em.computed.ifThenElse('active', 'active', ''),
  active: function () {
    return parseInt(this.get('number')) == parseInt(this.get('pager.grid.currentPage'));
  }.property('pager.grid.currentPage')
});

App.GridPager = Em.View.extend({

  pages: [],
  templateName: require('templates/common/grid/pager'),
  classNames: ['pagination'],

  activatePrevPage: function activatePrevPage() {
    var current = this.get('grid.currentPage');
    if (current > 1) this.set('grid.currentPage', current - 1);
  },
  activateNextPage: function activateNextPage() {
    var current = this.get('grid.currentPage');
    if (current < this.get('pages').length) this.set('grid.currentPage', current + 1);
  },

  prevPageDisabled: function () {
    return this.get('grid.currentPage') > 1 ? false : "disabled";
  }.property('grid.currentPage'),

  nextPageDisabled: function () {
    return this.get('grid.currentPage') < this.get('pages').length ? false : "disabled";
  }.property('grid.currentPage'),

  init: function init() {
    this._super();
    this.clearPages();
    this.pushPages();
  },

  activatePage: function activatePage(event) {
    var page = event.context;
    this.get('grid').set('currentPage', parseInt(event.context.get('number')));
  },

  clearPages: function clearPages() {
    this.set('pages', []);
  },

  pushPages: function () {
    var thisPager = this;
    var pages = this.get('grid._pager.pages');
    $.each(pages, function () {
      var thisNumber = this;
      thisPager.get('pages').push(App.GridPage.create({
        number: thisNumber,
        pager: thisPager
      }));
    });
  }.observes('grid._pager')
});

App.Grid = Em.View.extend({
  _columns: {}, // not used
  _filters: {}, // prepared filters from data values
  _pager: { pages: [1, 2, 3, 4, 5] }, // observed by pager to config it

  _collection: { className: false, staticOptions: {} }, // collection config
  currentPage: 1,
  fieldNames: [],
  appliedFilters: {},
  filteredArray: [],
  columns: [],
  collection: [],
  initComleted: false,
  rows: [],
  templateName: require('templates/main/admin/audit'),

  init: function init() {
    this._super();
    this.prepareColumns(); // should be the 1
    this.prepareCollection();
    this.preparePager();
  },

  preparePager: function preparePager() {
    //    this.set('pager', App.GridPager.extend({ grid:this })); ask to hide
  },

  addFilters: function addFilters(field, values) {
    var filters = this.get('appliedFilters');
    filters[field] = values;

    var collection = this.get('_collection.className');
    collection = collection.find();
    var arrayCollection = collection.filter(function (data) {
      var oneFilterFail = false;
      $.each(filters, function (fieldname, values) {
        if (values.length && values.indexOf(data.get(fieldname)) == -1) {
          return oneFilterFail = true;
        }
      });
      return !oneFilterFail;
    });

    this.set('filteredArray', arrayCollection);
  },

  prepareCollection: function prepareCollection() {
    if (validator.empty(this.get('_collection.className'))) {
      throw "_collection.className field is not defined";
    }
    var collection = this.get('_collection.className');
    this.set('collection', collection.find(this.get('_collection.staticOptions')));
  },

  addColumn: function addColumn(options) {
    options.grid = this;
    if (validator.empty(options.name)) {
      throw "define column name";
    }

    if (this.get('_columns.' + options.name)) {
      throw "column with this '" + options.name + "' already exists";
    }

    var field = App.GridHeader.extend(options);
    this.columns.push(field);

    if (field.filterable || 1) {
      // .filterable - field not working :(
      this.fieldNames.push(options.name);
    }
  },

  clearColumns: function clearColumns() {
    this.set('_columns', {});
    this.set('columns', []);
    this.set('fieldNames', []);
  },

  prepareColumns: function prepareColumns() {
    this.clearColumns();
  },

  prepareFilters: function () {
    var thisGrid = this;
    var collection = this.get('collection');
    var fieldNames = this.get('fieldNames');
    var options = {};

    if (collection && collection.content) {
      collection.forEach(function (object, i) {
        $.each(fieldNames, function (j, field) {
          if (!options[field]) {
            options[field] = [];
          }

          var filter = object.get(field);
          if (options[field].indexOf(filter) == -1) {
            options[field].push(filter);
          }
        });
      });

      thisGrid.set('_filters', options);
    }
  }.observes('collection.length'),

  clearRows: function clearRows() {
    this.set('rows', []);
  },

  prepareRows: function () {
    var collection = this.get('collection');
    var thisGrid = this;
    this.clearRows();
    var i = 1;

    if (collection && collection.content) {
      collection.forEach(function (object, i) {
        var row = App.GridRow.extend({ grid: thisGrid, object: object });
        thisGrid.rows.push(row);
      });
    }
  }.observes('collection.length'),

  filteredRows: function () {
    var collection = this.get('filteredArray');
    var thisGrid = this;
    this.clearRows();

    collection.forEach(function (object) {
      var row = App.GridRow.extend({ grid: thisGrid, object: object });
      thisGrid.rows.push(row);
    });
  }.observes('filteredArray')
});

});

require.register("views/common/helpers/format_null_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FormatNullView = Em.View.extend({
  tagName: 'span',
  template: Em.Handlebars.compile('{{view.result}}'),

  result: function () {
    var emptyValue = this.get('empty') ? this.get('empty') : Em.I18n.t('services.service.summary.notAvailable');
    emptyValue = emptyValue.startsWith('t:') ? Em.I18n.t(emptyValue.substr(2, emptyValue.length)) : emptyValue;
    return this.get('content') || this.get('content') == 0 ? this.get('content') : emptyValue;
  }.property('content')
});

});

require.register("views/common/helpers/format_role_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FormatRoleView = Em.View.extend({

  tagName: 'span',
  template: Em.Handlebars.compile('{{view.result}}'),

  result: Em.computed.formatRole('content', true)

});

});

require.register("views/common/helpers/format_word_break_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FormatWordBreakView = Em.View.extend({

  attributeBindings: ["data-original-title"],

  tagName: 'span',

  template: Em.Handlebars.compile('{{{view.result}}}'),

  _splitLongSubStrings: function _splitLongSubStrings(str) {
    var maxBlockLength = 20;
    if (str.length <= maxBlockLength || str.contains('-') || str.contains(' ')) {
      return str;
    }
    return str.split(/(?=[A-Z])/).join('<wbr>');
  },

  /**
   * @type {string}
   */
  result: function () {
    var content = Ember.Handlebars.Utils.escapeExpression(this.get('content')) || '';
    var self = this;
    ['.', '_', '/'].forEach(function (delimiter) {
      if (content.contains(delimiter)) {
        content = content.split(delimiter).map(function (substr) {
          return self._splitLongSubStrings(substr);
        }).join(delimiter + '<wbr>');
      }
    });
    return content.replace(/(<wbr>){2,}/g, '<wbr>'); // no need for double <wbr>
  }.property('content')
});

});

require.register("views/common/helpers/pluralize_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.PluralizeView = Em.View.extend({
  tagName: 'span',
  template: Em.Handlebars.compile('{{view.wordOut}}'),

  wordOut: function () {
    var count, singular, plural;
    count = this.get('content');
    singular = this.get('singular');
    plural = this.get('plural');
    return this.getWord(count, singular, plural);
  }.property('content'),
  /**
   * Get computed word.
   *
   * @param {Number} count
   * @param {String} singular
   * @param {String} [plural]
   * @return {String}
   * @method getWord
   */
  getWord: function getWord(count, singular, plural) {
    singular = this.parseValue(singular);
    // if plural not passed
    if (!plural) plural = singular + 's';else plural = this.parseValue(plural);
    if (singular && plural) {
      if (count == 1) {
        return singular;
      } else {
        return plural;
      }
    }
    return '';
  },
  /**
   * Detect and return value from its instance.
   *
   * @param {String} value
   * @return {*}
   * @method parseValue
   **/
  parseValue: function parseValue(value) {
    switch (value[0]) {
      case '@':
        value = this.getViewPropertyValue(value);
        break;
      case 't':
        value = this.tDetect(value);
        break;
      default:
        break;
    }
    return value;
  },
  /*
   * Detect for Em.I18n.t reference call
   * @params word {String}
   * return {String}
   */
  tDetect: function tDetect(word) {
    var splitted = word.split(':');
    if (splitted.length > 1 && splitted[0] == 't') {
      return Em.I18n.t(splitted[1]);
    } else {
      return splitted[0];
    }
  },
  /**
   * Get property value from view|controller by its key path.
   *
   * @param {String} value - key path
   * @return {*}
   * @method getViewPropertyValue
   **/
  getViewPropertyValue: function getViewPropertyValue(value) {
    value = value.substr(1);
    var keyword = value.split('.')[0]; // return 'controller' or 'view'
    switch (keyword) {
      case 'controller':
        return Em.get(this, value);
      case 'view':
        return Em.get(this, value.replace(/^view/, 'parentView'));
      default:
        break;
    }
  }
});

});

require.register("views/common/helpers/status_icon_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.StatusIconView = Em.View.extend({
  tagName: 'i',

  /**
   * relation map between status and icon class
   * @type {object}
   */
  statusIconMap: {
    'INIT': 'icon-cogs in_progress',
    'COMPLETED': 'glyphicon glyphicon-ok completed',
    'WARNING': 'glyphicon glyphicon-warning-sign',
    'FAILED': 'glyphicon glyphicon-exclamation-sign failed',
    'HOLDING_FAILED': 'glyphicon glyphicon-exclamation-sign failed',
    'SKIPPED_FAILED': 'glyphicon glyphicon-share-alt failed',
    'PENDING': 'glyphicon glyphicon-cog pending',
    'QUEUED': 'glyphicon glyphicon-cog queued',
    'IN_PROGRESS': 'icon-cogs in_progress',
    'HOLDING': 'glyphicon glyphicon-pause',
    'SUSPENDED': 'glyphicon glyphicon-pause',
    'ABORTED': 'glyphicon glyphicon-minus aborted',
    'TIMEDOUT': 'glyphicon glyphicon-time timedout',
    'HOLDING_TIMEDOUT': 'glyphicon glyphicon-time timedout',
    'SUBITEM_FAILED': 'glyphicon glyphicon-remove failed'
  },

  classNameBindings: ['iconClass'],
  attributeBindings: ['data-original-title'],

  didInsertElement: function didInsertElement() {
    App.tooltip($(this.get('element')));
  },

  'data-original-title': function () {
    return this.get('content').toCapital();
  }.property('content'),

  /**
   * @type {string}
   */
  iconClass: function () {
    return this.get('statusIconMap')[this.get('content')] || 'glyphicon glyphicon-question-sign';
  }.property('content')
});

});

require.register("views/common/host_progress_popup_body_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var fileUtils = require('utils/file_utils');

/**
 * @typedef {object} TaskRelationObject
 * @property {string} type relation type 'service', 'component'
 * @property {string} [value] optional value of relation e.g. name of component or service
 */

/**
 * Option for "filter by state" dropdown
 * @typedef {object} progressPopupCategoryObject
 * @property {string} value "all|pending|in progress|failed|completed|aborted|timedout"
 * @property {number} count number of items with <code>state</code> equal to <code>this.value</code>
 * @property {string} labelPath key in the messages.js
 * @property {string} label localized label
 */
var categoryObject = Em.Object.extend({
  value: '',
  count: 0,
  labelPath: '',
  label: function () {
    return Em.I18n.t(this.get('labelPath')).format(this.get('count'));
  }.property('count', 'labelPath')
});

/**
 * @class HostProgressPopupBodyView
 * @type {Em.View}
 */
App.HostProgressPopupBodyView = App.TableView.extend({

  templateName: require('templates/common/host_progress_popup'),

  /**
   * @type {boolean}
   */
  showTextArea: false,

  /**
   * @type {boolean}
   */
  isServiceEmptyList: true,

  /**
   * @type {boolean}
   */
  isTasksEmptyList: true,

  /**
   * @type {number}
   */
  sourceRequestScheduleId: -1,

  /**
   * @type {boolean}
   */
  sourceRequestScheduleRunning: false,

  /**
   * @type {boolean}
   */
  sourceRequestScheduleAborted: false,

  /**
   * @type {?string}
   */
  sourceRequestScheduleCommand: null,

  /**
   * @type {string}
   */
  clipBoardContent: null,

  /**
   * @type {boolean}
   */
  isClipBoardActive: false,

  /**
   * Determines that host information become loaded and mapped with <code>App.hostsMapper</code>.
   * This property play in only when Log Search service installed.
   *
   * @type {boolean}
   */
  hostInfoLoaded: true,

  /**
   * Clipboard for task logs
   * Used when user click "Copy" and textarea with task stderr and stdout is shown
   * Should be destroyed (call `destroy`) when used is moved out from task logs level or when BG-ops modal is closed
   *
   * @type {?Clipboard}
   */
  taskLogsClipboard: null,

  /**
   * Alias for <code>controller.hosts</code>
   *
   * @type {wrappedHost[]}
   */
  hosts: function () {
    return this.get('controller.hosts');
  }.property('controller.hosts.[]'),

  /**
   * Alias for <code>controller.servicesInfo</code>
   *
   * @type {wrappedService[]}
   */
  services: function () {
    return this.get('controller.servicesInfo');
  }.property('controller.servicesInfo.[]'),

  /**
   * @type {number}
   */
  openedTaskId: 0,

  /**
   * Return task detail info of opened task
   *
   * @type {wrappedTask}
   */
  openedTask: function () {
    if (!(this.get('openedTaskId') && this.get('tasks'))) {
      return Em.Object.create();
    }
    return this.get('tasks').findProperty('id', this.get('openedTaskId'));
  }.property('tasks', 'tasks.@each.stderr', 'tasks.@each.stdout', 'openedTaskId'),

  /**
   * stderr and stdout joined together for clipboard
   *
   * @type {string}
   */
  formattedLogsForOpenedTask: function () {
    var stderr = this.get('openedTask.stderr');
    var stdout = this.get('openedTask.stdout');
    return 'stderr: \n' + stderr + '\n stdout:\n' + stdout;
  }.property('openedTask.stderr', 'openedTask.stdout'),

  /**
   * @type {object}
   */
  filterMap: {
    pending: ["pending", "queued"],
    in_progress: ["in_progress", "upgrading"],
    failed: ["failed"],
    completed: ["completed", "success"],
    aborted: ["aborted"],
    timedout: ["timedout"]
  },

  /**
   * Determines if "Show More ..."-link should be shown
   * @type {boolean}
   */
  isShowMore: true,

  /**
   * @type {boolean}
   */
  pagination: true,

  /**
   * @type {boolean}
   */
  isPaginate: false,

  /**
   * Select box, display names and values
   *
   * @type {progressPopupCategoryObject[]}
   */
  categories: [categoryObject.create({ value: 'all', labelPath: 'hostPopup.status.category.all' }), categoryObject.create({ value: 'pending', labelPath: 'hostPopup.status.category.pending' }), categoryObject.create({ value: 'in_progress', labelPath: 'hostPopup.status.category.inProgress' }), categoryObject.create({ value: 'failed', labelPath: 'hostPopup.status.category.failed' }), categoryObject.create({ value: 'completed', labelPath: 'hostPopup.status.category.success' }), categoryObject.create({ value: 'aborted', labelPath: 'hostPopup.status.category.aborted' }), categoryObject.create({ value: 'timedout', labelPath: 'hostPopup.status.category.timedout' })],

  /**
   * Selected option is bound to this values
   * @type {?progressPopupCategoryObject}
   */
  serviceCategory: null,

  /**
   * @type {?progressPopupCategoryObject}
   */
  hostCategory: null,

  /**
   * @type {?progressPopupCategoryObject}
   */
  taskCategory: null,

  /**
   * Indicates current level displayed.
   *
   * @type {string}
   */
  currentLevel: "",

  /**
   * flag to indicate whether level data has already been loaded
   * applied only to HOSTS_LIST and TASK_DETAILS levels, whereas async query used to obtain data
   *
   * @type {boolean}
   */
  isLevelLoaded: true,

  /**
   * <code>switchLevel</code> for some <code>controller.dataSourceController</code> should be customized
   * So, this map contains information about customize-methods
   * Format: key - <code>dataSourceController.name</code>, value - method name in this view
   * Method is called with same arguments as <code>switchLevel</code> is
   *
   * @type {object}
   */
  customControllersSwitchLevelMap: {
    highAvailabilityProgressPopupController: '_switchLevelForHAProgressPopupController'
  },

  /**
   * @type {boolean}
   */
  isHostEmptyList: Em.computed.empty('pageContent'),

  /**
   * @type {wrappedHost}
   */
  currentHost: Em.computed.findByKey('hosts', 'name', 'controller.currentHostName'),

  /**
   * Tasks for current shown host (<code>currentHost</code>)
   *
   * @type {wrappedTask[]}
   */
  tasks: function () {
    var currentHost = this.get('currentHost');
    return currentHost ? currentHost.get('tasks') : [];
  }.property('currentHost.tasks', 'currentHost.tasks.@each.status'),

  /**
   * Message about aborting operation
   * Custom for Rolling Restart
   *
   * @type {string}
   */
  requestScheduleAbortLabel: function () {
    return 'ROLLING-RESTART' === this.get('sourceRequestScheduleCommand') ? Em.I18n.t("hostPopup.bgop.abort.rollingRestart") : Em.I18n.t("common.abort");
  }.property('sourceRequestScheduleCommand'),

  didInsertElement: function didInsertElement() {
    this.updateHostInfo();
    this.subscribeResize();
  },

  willDestroyElement: function willDestroyElement() {
    if (this.get('controller.dataSourceController.name') === 'highAvailabilityProgressPopupController') {
      this.set('controller.dataSourceController.isTaskPolling', false);
    }
    this.unsubscribeResize();
  },

  selectServiceCategory: function selectServiceCategory(e) {
    this.set('serviceCategory', e.context);
  },

  selectHostCategory: function selectHostCategory(e) {
    this.set('hostCategory', e.context);
  },

  selectTaskCategory: function selectTaskCategory(e) {
    this.set('taskCategory', e.context);
  },

  /**
   * Subscribe for window <code>resize</code> event.
   *
   * @method subscribeResize
   */
  subscribeResize: function subscribeResize() {
    var self = this;
    $(window).on('resize', this.resizeHandler.bind(this));
    Em.run.next(this, function () {
      self.resizeHandler();
    });
  },

  /**
   * Remove event listener for window <code>resize</code> event.
   */
  unsubscribeResize: function unsubscribeResize() {
    $(window).off('resize', this.resizeHandler.bind(this));
  },

  /**
   * This method handles window resize and fit modal body content according to visible items for each <code>level</code>.
   *
   * @method resizeHandler
   */
  resizeHandler: function resizeHandler() {
    var parentView = this.get('parentView');

    if (!parentView || !parentView.$ || !parentView.$() || this.get('state') === 'destroyed' || !parentView.get('isOpen')) return;

    var modal = parentView.$().find('.modal'),
        headerHeight = $(modal).find('.modal-header').outerHeight(true),
        modalFooterHeight = $(modal).find('.modal-footer').outerHeight(true),
        taskTopWrapHeight = $(modal).find('.top-wrap:visible').outerHeight(true),
        modalTopOffset = $(modal).offset().top,
        contentPaddingBottom = parseFloat($(modal).find('.modal-dialog').css('marginBottom')) || 0,
        hostsPageBarHeight = $(modal).find('#host-info tfoot').outerHeight(true),
        logComponentFileNameHeight = $(modal).find('#host-info tfoot').outerHeight(true),
        levelName = this.get('currentLevel'),
        boLevelHeightMap = {
      'REQUESTS_LIST': {
        height: window.innerHeight - 2 * modalTopOffset - headerHeight - taskTopWrapHeight - modalFooterHeight - contentPaddingBottom,
        target: '#service-info'
      },
      'HOSTS_LIST': {
        height: window.innerHeight - 2 * modalTopOffset - headerHeight - taskTopWrapHeight - modalFooterHeight - contentPaddingBottom - hostsPageBarHeight,
        target: '#host-info'
      },
      'TASKS_LIST': {
        height: window.innerHeight - 2 * modalTopOffset - headerHeight - taskTopWrapHeight - modalFooterHeight - contentPaddingBottom,
        target: '#host-log'
      },
      'TASK_DETAILS': {
        height: window.innerHeight - 2 * modalTopOffset - headerHeight - taskTopWrapHeight - modalFooterHeight - contentPaddingBottom,
        target: ['.task-detail-log-info', '.log-tail-content.pre-styled']
      }
    },
        currentLevelHeight,
        resizeTarget;

    if (levelName && levelName in boLevelHeightMap) {
      resizeTarget = boLevelHeightMap[levelName].target;
      currentLevelHeight = boLevelHeightMap[levelName].height;
      if (!Em.isArray(resizeTarget)) {
        resizeTarget = [resizeTarget];
      }
      resizeTarget.forEach(function (target) {
        if (target === '.log-tail-content.pre-styled') {
          currentLevelHeight -= logComponentFileNameHeight;
        }
        $(target).css('maxHeight', currentLevelHeight + 'px');
      });
    }
  },

  /**
   * Preset values on init
   *
   * @method setOnStart
   */
  setOnStart: function setOnStart() {
    this.set('serviceCategory', this.get('categories').findProperty('value', 'all'));

    if (this.get("controller.isBackgroundOperations")) {
      this.get('controller').setSelectCount(this.get("services"), this.get('categories'));
      this.updateHostInfo();
    } else {
      this.get('parentView').switchView("HOSTS_LIST");
      this.set('hostCategory', this.get('categories').findProperty('value', 'all'));
    }
  },

  /**
   * force popup to show list of operations
   *
   * @method resetState
   */
  resetState: function () {
    if (this.get('parentView.isOpen')) {
      this.get('parentView').switchView("OPS_LIST");
      this.get("controller").setBackgroundOperationHeader(false);
      this.get('controller.hosts').clear();
      this.setOnStart();
      this.rerender();
    }
  }.observes('parentView.isOpen'),

  /**
   * When popup is opened, and data after polling has changed, update this data in component
   *
   * @method updateHostInfo
   */
  updateHostInfo: function () {
    if (!this.get('parentView.isOpen')) {
      return;
    }
    this.set('parentView.isLoaded', false);
    this.get("controller").set("inputData", this.get("controller.dataSourceController.services"));
    this.get("controller").onServiceUpdate(this.get('parentView.isServiceListHidden'));
    this.get("controller").onHostUpdate();
    this.set('parentView.isLoaded', true);
    this.set("hosts", this.get("controller.hosts"));
    this.set("services", this.get("controller.servicesInfo"));
    this.set('isLevelLoaded', true);
  }.observes("controller.dataSourceController.serviceTimestamp"),

  /**
   * Depending on service filter, set which services should be shown
   *
   * @method visibleServices
   */
  visibleServices: function () {
    if (this.get("services")) {
      this.set("isServiceEmptyList", true);
      if (this.get('serviceCategory.value')) {
        var filter = this.get('serviceCategory.value');
        var services = this.get('services');
        this.set("isServiceEmptyList", this.setVisibility(filter, services));
      }
    }
  }.observes('serviceCategory', 'services', 'services.@each.status'),

  /**
   * Depending on hosts filter, set which hosts should be shown
   *
   * @method filter
   */
  filter: function () {
    var filter = this.get('hostCategory.value');
    var hosts = this.get('hosts') || [];
    var filterMap = this.get('filterMap');
    if (!filter || !hosts.length) {
      return;
    }
    if (filter === 'all') {
      this.set('filteredContent', hosts);
    } else {
      this.set('filteredContent', hosts.filter(function (item) {
        return filterMap[filter].contains(item.status);
      }));
    }
  }.observes('hosts.length', 'hostCategory.value'),

  /**
   * Reset startIndex property back to 1 when filter type has been changed.
   *
   * @method resetIndex
   */
  resetIndex: function () {
    if (this.get('hostCategory.value')) {
      this.set('startIndex', 1);
    }
  }.observes('hostCategory.value'),

  /**
   * Depending on tasks filter, set which tasks should be shown
   *
   * @method visibleTasks
   */
  visibleTasks: function () {
    this.set("isTasksEmptyList", true);
    if (this.get('taskCategory.value') && this.get('tasks')) {
      var filter = this.get('taskCategory.value');
      var tasks = this.get('tasks');
      this.set("isTasksEmptyList", this.setVisibility(filter, tasks));
    }
  }.observes('taskCategory', 'tasks', 'tasks.@each.status'),

  /**
   * Depending on selected filter type, set object visibility value
   *
   * @param filter
   * @param obj
   * @return {bool} isEmptyList
   * @method setVisibility
   */
  setVisibility: function setVisibility(filter, obj) {
    var isEmptyList = true;
    var filterMap = this.get('filterMap');
    if (filter === "all") {
      obj.setEach("isVisible", true);
      isEmptyList = !obj.length;
    } else {
      obj.forEach(function (item) {
        item.set('isVisible', filterMap[filter].contains(item.status));
        isEmptyList = isEmptyList ? !item.get('isVisible') : false;
      }, this);
    }
    return isEmptyList;
  },

  /**
   * Depending on currently viewed tab, call setSelectCount function
   *
   * @method updateSelectView
   */
  updateSelectView: function () {
    var isPaginate;
    if (this.get('parentView.isHostListHidden')) {
      if (this.get('parentView.isTaskListHidden')) {
        if (!this.get('parentView.isServiceListHidden')) {
          this.get('controller').setSelectCount(this.get("services"), this.get('categories'));
        }
      } else {
        this.get('controller').setSelectCount(this.get("tasks"), this.get('categories'));
      }
    } else {
      //since lazy loading used for hosts, we need to get hosts info directly from controller, that always contains entire array of data
      this.get('controller').setSelectCount(this.get("controller.hosts"), this.get('categories'));
      isPaginate = true;
    }
    this.set('isPaginate', !!isPaginate);
  }.observes('tasks.@each.status', 'hosts.@each.status', 'parentView.isTaskListHidden', 'parentView.isHostListHidden', 'services.@each.status'),

  setBreadcrumbs: function setBreadcrumbs(level) {
    var breadcrumbs = [];
    var self = this;
    var opsCrumb = this.get("controller.rootBreadcrumb");

    if (opsCrumb) {
      opsCrumb.action = function () {
        self.switchLevel("OPS_LIST");
      };

      var opCrumb = {
        label: this.get("controller.serviceName"),
        action: function action() {
          self.switchLevel("HOSTS_LIST", self.get('controller.servicesInfo').findProperty('id', self.get('controller.currentServiceId')));
        }
      };

      var hostCrumb = {
        label: this.get("controller.currentHostName"),
        action: function action() {
          self.switchLevel("TASKS_LIST", self.get('currentHost'));
        }
      };

      var taskCrumb = {
        itemView: Em.View.extend({
          tagName: "span",
          controller: self,
          template: Em.Handlebars.compile('<i style="margin-left: 20px;" {{bindAttr class="openedTask.status :task-detail-status-ico openedTask.icon"}}></i>{{openedTask.commandDetail}}')
        })
      };

      switch (level) {
        case "OPS_LIST":
          breadcrumbs.push(opsCrumb);
          break;
        case "HOSTS_LIST":
          breadcrumbs.push(opsCrumb);
          if (opCrumb.label === breadcrumbs[0].label) {
            breadcrumbs.pop();
          }
          breadcrumbs.push(opCrumb);
          break;
        case "TASKS_LIST":
          breadcrumbs.push(opsCrumb);
          if (opCrumb.label === breadcrumbs[0].label) {
            breadcrumbs.pop();
          }
          breadcrumbs.push(opCrumb);
          breadcrumbs.push(hostCrumb);
          break;
        case "TASK_DETAILS":
          breadcrumbs.push(opsCrumb);
          if (opCrumb.label === breadcrumbs[0].label) {
            breadcrumbs.pop();
          }
          breadcrumbs.push(opCrumb);
          breadcrumbs.push(hostCrumb);
          breadcrumbs.push(taskCrumb);
          break;
      }

      this.set('controller.breadcrumbs', breadcrumbs);
    }
  },

  /**
   * Sets up and tears down the different views in the modal when switching.
   * Calls switchView() on the controller to perform the actual view switch.
   *
   * @param {string} levelName
   * @method switchLevel
   */
  switchLevel: function switchLevel(levelName, context) {
    var prevLevel = this.get('controller.dataSourceController.levelInfo.name');

    //leaving level - do any cleanup here
    switch (prevLevel) {
      case "OPS_LIST":
        break;
      case "HOSTS_LIST":
        break;
      case "TASKS_LIST":
        break;
      case "TASK_DETAILS":
        this.destroyClipBoard();
        break;
    }

    //entering level - do any setup here
    switch (levelName) {
      case "OPS_LIST":
        this.gotoOps();
        break;
      case "HOSTS_LIST":
        this.gotoHosts(context);
        break;
      case "TASKS_LIST":
        this.gotoTasks(context);
        break;
      case "TASK_DETAILS":
        this.goToTaskDetails(context);
        break;
    }

    if (!this.get("controller.isBackgroundOperations")) {
      var customControllersSwitchLevelMap = this.get('customControllersSwitchLevelMap');
      var args = [].slice.call(arguments);
      Em.tryInvoke(this, customControllersSwitchLevelMap[this.get('controller.dataSourceController.name')], args);
    }
  },

  changeLevel: function changeLevel(levelName) {
    if (this.get("controller.isBackgroundOperations")) {
      var dataSourceController = this.get('controller.dataSourceController');
      var levelInfo = dataSourceController.get('levelInfo');

      levelInfo.set('taskId', this.get('openedTaskId'));
      levelInfo.set('requestId', this.get('controller.currentServiceId'));
      levelInfo.set('name', levelName);
      this.set('isLevelLoaded', dataSourceController.requestMostRecent());
    }

    this.set('currentLevel', levelName); //NOTE: setting this triggers levelDidChange() which updates the breadcrumbs
  },

  //This is triggered by changeLevel() -- (and probably some other things) --
  //when "controller.dataSourceController.levelInfo.name" is set
  levelDidChange: function () {
    var levelName = this.get('currentLevel'),
        self = this;

    if (this.get("parentView.isOpen")) {
      self.setBreadcrumbs(levelName);
    }

    if (levelName && this.get('isLevelLoaded')) {
      Em.run.next(this, function () {
        self.resizeHandler();
      });
    }
  }.observes('currentLevel', 'isLevelLoaded'),

  popupIsOpenDidChange: function () {
    var hostComponentLogs = this.get('hostComponentLogs');

    if (!this.get('parentView.isOpen') && hostComponentLogs) {
      hostComponentLogs.clear();
    }
  }.observes('parentView.isOpen'),

  /**
   * Switch-level custom method for <code>highAvailabilityProgressPopupController</code>
   *
   * @param {string} levelName
   * @private
   */
  _switchLevelForHAProgressPopupController: function _switchLevelForHAProgressPopupController(levelName) {
    var dataSourceController = this.get('controller.dataSourceController');
    if (levelName === 'TASK_DETAILS') {
      this.set('isLevelLoaded', false);
      dataSourceController.startTaskPolling(this.get('openedTask.request_id'), this.get('openedTask.id'));
      Em.keys(this.get('parentView.detailedProperties')).forEach(function (key) {
        dataSourceController.addObserver('taskInfo.' + this.get('parentView.detailedProperties')[key], this, 'updateTaskInfo');
      }, this);
    } else {
      dataSourceController.stopTaskPolling();
    }
  },

  /**
   * @method updateTaskInfo
   */
  updateTaskInfo: function updateTaskInfo() {
    var dataSourceController = this.get('controller.dataSourceController');
    var openedTask = this.get('openedTask');
    if (openedTask && openedTask.get('id') == dataSourceController.get('taskInfo.id')) {
      this.set('isLevelLoaded', true);
      Em.keys(this.get('parentView.detailedProperties')).forEach(function (key) {
        openedTask.set(key, dataSourceController.get('taskInfo.' + key));
      }, this);
    }
  },

  /**
   * Onclick handler for button <-Tasks
   *
   * @method backToTaskList
   */
  backToTaskList: function backToTaskList() {
    this.switchLevel("TASKS_LIST", true);
  },

  /**
   * Onclick handler for button <-Hosts
   *
   * @method backToHostList
   */
  backToHostList: function backToHostList() {
    this.switchLevel("HOSTS_LIST", true);
  },

  /**
   * Onclick handler for button <-Operations
   * TODO: This is still used somewhere outside the Background Operations heirarchy.
   *
   * @method backToServiceList
   */
  backToServiceList: function backToServiceList() {
    this.switchLevel("OPS_LIST", true);
  },

  /**
   * Onclick handler for Show more ..
   *
   * @method requestMoreOperations
   */
  requestMoreOperations: function requestMoreOperations() {
    var BGOController = App.router.get('backgroundOperationsController');
    var count = BGOController.get('operationsCount');
    BGOController.set('operationsCount', count + 10);
    BGOController.requestMostRecent();
  },

  /**
   * @method setShowMoreAvailable
   */
  setShowMoreAvailable: function () {
    if (this.get('parentView.isOpen')) {
      this.set('isShowMore', App.router.get("backgroundOperationsController.isShowMoreAvailable"));
    }
  }.observes('parentView.isOpen', 'App.router.backgroundOperationsController.isShowMoreAvailable'),

  gotoOps: function gotoOps() {
    this.get('controller.hosts').clear();
    var dataSourceController = this.get('controller.dataSourceController');
    dataSourceController.requestMostRecent();
    this.get("controller").setBackgroundOperationHeader(false);

    this.changeLevel("OPS_LIST");
    this.get("parentView").switchView("OPS_LIST");
  },

  /**
   * Onclick handler for selected Service (Operation)
   *
   * @param {{context: wrappedService}} event
   * @method gotoHosts
   */
  onOpClick: function onOpClick(event) {
    this.switchLevel("HOSTS_LIST", event.context);
  },

  gotoHosts: function gotoHosts(service) {
    this.get("controller").set("serviceName", service.get("name"));
    this.get("controller").set("currentServiceId", service.get("id"));
    this.get("controller").set("currentHostName", null);
    this.get("controller").onHostUpdate();
    this.get('hostComponentLogs').clear();

    this.changeLevel("HOSTS_LIST");

    var servicesInfo = this.get("controller.hosts");
    this.set("controller.operationInfo", service);

    //apply lazy loading on cluster with more than 100 nodes
    this.set('hosts', servicesInfo.length > 100 ? servicesInfo.slice(0, 50) : servicesInfo);
    $(".modal").scrollTop(0);
    $(".modal-body").scrollTop(0);
    if (servicesInfo.length > 100) {
      Em.run.next(this, function () {
        this.set('hosts', this.get('hosts').concat(servicesInfo.slice(50, servicesInfo.length)));
      });
    }
    // Determine if source request schedule is present
    this.set('sourceRequestScheduleId', service.get("sourceRequestScheduleId"));
    this.set('sourceRequestScheduleCommand', service.get('contextCommand'));
    this.refreshRequestScheduleInfo();

    this.set('hostCategory', this.get('categories').findProperty('value', 'all'));

    this.get("parentView").switchView("HOSTS_LIST");
  },

  /**
   * Navigate to host details logs tab with preset filter.
   */
  navigateToHostLogs: function navigateToHostLogs() {
    var relationType = this._determineRoleRelation(this.get('openedTask')),
        hostModel = App.Host.find().findProperty('hostName', this.get('openedTask.hostName')),
        queryParams = [],
        model;

    if (relationType.type === 'component') {
      model = App.StackServiceComponent.find().findProperty('componentName', relationType.value);
      queryParams.push('service_name=' + model.get('serviceName'));
      queryParams.push('component_name=' + relationType.value);
    }
    if (relationType.type === 'service') {
      queryParams.push('service_name=' + relationType.value);
    }
    App.router.transitionTo('main.hosts.hostDetails.logs', hostModel, { query: '' });
    if (this.get('parentView') && typeof this.get('parentView').onClose === 'function') this.get('parentView').onClose();
  },

  /**
  * Determines if opened task related to service or component.
  *
  * @return {boolean} <code>true</code> when relates to service or component
  */
  isLogsLinkVisible: function () {
    if (!this.get('openedTask') || !this.get('openedTask.id')) return false;
    return !!this._determineRoleRelation(this.get('openedTask'));
  }.property('openedTask'),

  /**
   * @param  {wrappedTask} taskInfo
   * @return {boolean|TaskRelationObject}
   */
  _determineRoleRelation: function _determineRoleRelation(taskInfo) {
    var foundComponentName,
        foundServiceName,
        componentNames = App.StackServiceComponent.find().mapProperty('componentName'),
        serviceNames = App.StackService.find().mapProperty('serviceName'),
        taskLog = this.get('currentHost.logTasks').findProperty('Tasks.id', Em.get(taskInfo, 'id')) || {},
        role = Em.getWithDefault(taskLog, 'Tasks.role', false),
        eqlFn = function eqlFn(compare) {
      return function (item) {
        return item === compare;
      };
    };

    if (!role) {
      return false;
    }
    // component service check
    if (role.endsWith('_SERVICE_CHECK')) {
      role = role.replace('_SERVICE_CHECK', '');
    }
    foundComponentName = componentNames.filter(eqlFn(role))[0];
    foundServiceName = serviceNames.filter(eqlFn(role))[0];
    if (foundComponentName || foundServiceName) {
      return {
        type: foundComponentName ? 'component' : 'service',
        value: foundComponentName || foundServiceName
      };
    }
    return false;
  },

  /**
   * @type {boolean}
   */
  isRequestSchedule: function () {
    var id = this.get('sourceRequestScheduleId');
    return id != null && !isNaN(id) && id > -1;
  }.property('sourceRequestScheduleId'),

  /**
   * @method refreshRequestScheduleInfo
   */
  refreshRequestScheduleInfo: function () {
    var self = this;
    var id = this.get('sourceRequestScheduleId');
    batchUtils.getRequestSchedule(id, function (data) {
      var status = Em.get(data || {}, 'RequestSchedule.status');
      if (status) {
        switch (status) {
          case 'DISABLED':
            self.set('sourceRequestScheduleRunning', false);
            self.set('sourceRequestScheduleAborted', true);
            break;
          case 'COMPLETED':
            self.set('sourceRequestScheduleRunning', false);
            self.set('sourceRequestScheduleAborted', false);
            break;
          case 'SCHEDULED':
            self.set('sourceRequestScheduleRunning', true);
            self.set('sourceRequestScheduleAborted', false);
            break;
        }
      } else {
        self.set('sourceRequestScheduleRunning', false);
        self.set('sourceRequestScheduleAborted', false);
      }
    }, function () {
      self.set('sourceRequestScheduleRunning', false);
      self.set('sourceRequestScheduleAborted', false);
    });
  }.observes('sourceRequestScheduleId'),

  /**
   * Attempts to abort the current request schedule
   *
   * @param {{context: number}} event
   * @method doAbortRequestSchedule
   */
  doAbortRequestSchedule: function doAbortRequestSchedule(event) {
    var self = this;
    var id = event.context;
    batchUtils.doAbortRequestSchedule(id, function () {
      self.refreshRequestScheduleInfo();
    });
  },

  /**
   * Onclick handler for selected Host
   *
   * @param {{context: wrappedHost}} event
   * @method gotoTasks
   */
  onHostClick: function onHostClick(event) {
    this.switchLevel("TASKS_LIST", event.context);
  },

  gotoTasks: function gotoTasks(host) {
    var tasksInfo = [];

    if (host.logTasks) {
      host.logTasks.forEach(function (_task) {
        tasksInfo.pushObject(this.get("controller").createTask(_task));
      }, this);
    }

    if (tasksInfo.length) {
      this.get("controller").set("currentHostName", host.publicName);
    }

    var currentHost = this.get("currentHost");
    if (currentHost) {
      currentHost.set("tasks", tasksInfo);
    }
    this.preloadHostModel(Em.getWithDefault(host || {}, 'name', false));
    this.set('taskCategory', this.get('categories').findProperty('value', 'all'));
    $(".modal").scrollTop(0);
    $(".modal-body").scrollTop(0);

    this.changeLevel("TASKS_LIST");
    this.get("parentView").switchView("TASKS_LIST");
  },

  /**
   * Will add <code>App.Host</code> model with relevant host info by specified host name only when it missed
   * and Log Search service installed.
   * This update needed to get actual status of logs for components located on this host and get appropriate model
   * for navigation to hosts/:host_id/logs route.
   *
   * @method preloadHostModel
   * @param {string} hostName host name to get info
   */
  preloadHostModel: function preloadHostModel(hostName) {
    var self = this,
        fields;
    if (!hostName) {
      self.set('hostInfoLoaded', true);
      return;
    }
    if (this.get('isLogSearchInstalled') && App.get('supports.logSearch') && !App.Host.find().someProperty('hostName', hostName)) {
      this.set('hostInfoLoaded', false);
      fields = ['stack_versions/repository_versions/RepositoryVersions/repository_version', 'host_components/HostRoles/host_name'];
      App.router.get('updateController').updateLogging(hostName, fields, function (data) {
        App.hostsMapper.map({ items: [data] });
      }).always(function () {
        self.set('hostInfoLoaded', true);
      });
    } else {
      self.set('hostInfoLoaded', true);
    }
  },

  /**
   * @method stopRebalanceHDFS
   * @returns {App.ModalPopup}
   */
  stopRebalanceHDFS: function stopRebalanceHDFS() {
    var hostPopup = this;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'cancel.background.operation',
        sender: hostPopup,
        data: {
          requestId: hostPopup.get('controller.currentServiceId')
        }
      });
      hostPopup.backToServiceList();
    });
  },

  /**
   * Onclick handler for selected Task
   *
   * @method openTaskLogInDialog
   */
  openTaskLogInDialog: function openTaskLogInDialog() {
    var target = ".task-detail-log-info",
        activeHostLog = this.get('hostComponentLogs').findProperty('isActive', true),
        activeHostLogSelector = activeHostLog ? activeHostLog.get('tabClassNameSelector') + '.active' : false;

    if (this.get('isClipBoardActive')) {
      this.destroyClipBoard();
    }
    if (activeHostLog && $(activeHostLogSelector).length) {
      target = activeHostLogSelector;
    }
    var newWindow = window.open();
    var newDocument = newWindow.document;
    newDocument.write('<pre>' + this.get('textAreaValue') + '</pre>');
    newDocument.close();
  },

  onTaskClick: function onTaskClick(event) {
    this.switchLevel("TASK_DETAILS", event.context);
  },

  /**
   * Onclick event for show task detail info
   *
   * @param {{context: wrappedTask}} event
   * @method goToTaskDetails
   */
  goToTaskDetails: function goToTaskDetails(taskInfo) {
    var self = this;
    var taskLogsClipboard = new Clipboard('.btn.copy-clipboard', {
      text: function text() {
        return self.get('textAreaValue');
      }
    });

    this.set('taskLogsClipboard', taskLogsClipboard);
    if (this.get('isClipBoardActive')) {
      this.destroyClipBoard();
    }

    this.set('openedTaskId', taskInfo.id);

    if (this.get("controller.isBackgroundOperations")) {
      var dataSourceController = this.get('controller.dataSourceController');
      dataSourceController.requestMostRecent();
      this.set('isLevelLoaded', false);
    }

    $(".modal").scrollTop(0);
    $(".modal-body").scrollTop(0);

    this.changeLevel("TASK_DETAILS");
    this.get("parentView").switchView("TASK_DETAILS");
  },

  /**
   * Onclick event for copy to clipboard button
   *
   * @method textTrigger
   */
  textTrigger: function textTrigger() {
    return this.get('isClipBoardActive') ? this.destroyClipBoard() : this.createClipBoard();
  },

  /**
   * @type {string}
   */
  textAreaValue: function () {
    return this.get('isLogComponentActive') ? this.get('clipBoardContent') : this.get('formattedLogsForOpenedTask');
  }.property('clipBoardContent', 'formattedLogsForOpenedTask', 'isLogComponentActive'),

  /**
   * Create Clip Board
   *
   * @method createClipBoard
   */
  createClipBoard: function createClipBoard() {
    this.set('isClipBoardActive', true);
  },

  /**
   * Destroy Clip Board
   *
   * @method destroyClipBoard
   */
  destroyClipBoard: function destroyClipBoard() {
    var logElement = this.get('isLogComponentActive') ? $('.log-component-tab.active .log-tail-content') : $(".task-detail-log-maintext");
    logElement.css('display', 'block');
    this.set('isClipBoardActive', false);
    var taskLogsClipboard = this.get('taskLogsClipboard');
    if (taskLogsClipboard) {
      Em.tryInvoke(taskLogsClipboard, 'destroy');
    }
  },

  /**
   * @type {boolean}
   */
  isLogSearchInstalled: function () {
    return App.Service.find().someProperty('serviceName', 'LOGSEARCH');
  }.property(),

  /**
   * Host component logs associated with selected component on 'TASK_DETAILS' level.
   *
   * @property {object[]}
   */
  hostComponentLogs: function () {
    var relationType,
        componentName,
        hostName,
        linkTailTpl = '/#/logs/serviceLogs;hosts={0};components={2};query=%5B%7B"id":0,"name":"path","label":"Path","value":"{1}","isExclude":false%7D%5D';

    if (this.get('openedTask.id')) {
      relationType = this._determineRoleRelation(this.get('openedTask'));
      if (relationType.type === 'component') {
        hostName = this.get('currentHost.name');
        componentName = relationType.value;
        return App.HostComponentLog.find().filterProperty('hostComponent.host.hostName', hostName).filterProperty('hostComponent.componentName', componentName).reduce(function (acc, item, index) {
          var logComponentName = item.get('name'),
              componentHostName = item.get('hostComponent.host.hostName');
          acc.pushObjects(item.get('logFileNames').map(function (logFileName, idx) {
            var tabClassName = logComponentName + '_' + index + '_' + idx;
            return Em.Object.create({
              hostName: componentHostName,
              logComponentName: logComponentName,
              fileName: logFileName,
              tabClassName: tabClassName,
              tabClassNameSelector: '.' + tabClassName,
              displayedFileName: fileUtils.fileNameFromPath(logFileName),
              linkTail: linkTailTpl.format(encodeURIComponent(hostName), encodeURIComponent(logFileName), encodeURIComponent(logComponentName)),
              isActive: false
            });
          }));
          return acc;
        }, []);
      }
    }
    return [];
  }.property('openedTaskId', 'isLevelLoaded'),

  /**
   * @type {boolean}
   */
  isLogComponentActive: Em.computed.someBy('hostComponentLogs', 'isActive', true),

  /**
   * Determines if there are component logs for selected component within 'TASK_DETAILS' level.
   *
   * @property {boolean}
   */
  hostComponentLogsExists: Em.computed.and('isLogSearchInstalled', 'hostComponentLogs.length', 'parentView.isOpen'),

  /**
   * Minimum required content to embed in App.LogTailView. This property observes current active host component log.
   *
   * @property {object}
   */
  logTailViewContent: function () {
    if (!this.get('hostComponentLog')) {
      return null;
    }
    return Em.Object.create({
      hostName: this.get('currentHost.name'),
      logComponentName: this.get('hostComponentLog.name')
    });
  }.property('hostComponentLogs.@each.isActive'),

  logTailView: App.LogTailView.extend({

    isActiveDidChange: function () {
      var self = this;
      if (this.get('content.isActive') === false) return;
      setTimeout(function () {
        self.scrollToBottom();
        self.storeToClipBoard();
      }, 500);
    }.observes('content.isActive'),

    logRowsLengthDidChange: function () {
      if (!this.get('content.isActive') || this.get('state') === 'destroyed') return;
      this.storeToClipBoard();
    }.observes('logRows.length'),

    /**
     * Stores log content to use for clip board.
     */
    storeToClipBoard: function storeToClipBoard() {
      this.get('parentView').set('clipBoardContent', this.get('logRows').map(function (i) {
        return i.get('logtimeFormatted') + ' ' + i.get('level') + ' ' + i.get('logMessage');
      }).join('\n'));
    }
  }),

  setActiveLogTab: function setActiveLogTab(e) {
    var content = e.context;
    this.set('clipBoardContent', null);
    this.get('hostComponentLogs').without(content).setEach('isActive', false);
    if (this.get('isClipBoardActive')) {
      this.destroyClipBoard();
    }
    content.set('isActive', true);
  },

  setActiveTaskLogTab: function setActiveTaskLogTab() {
    this.set('clipBoardContent', null);
    this.get('hostComponentLogs').setEach('isActive', false);
    if (this.get('isClipBoardActive')) {
      this.destroyClipBoard();
    }
  }
});

});

require.register("views/common/log_file_search_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view');

/**
* @augments App.InfiniteScrollMixin
* @type {Em.View}
*/
App.LogFileSearchView = Em.View.extend(App.InfiniteScrollMixin, {
  classNames: ['log-file-search'],
  templateName: require('templates/common/log_file_search'),
  logLevels: ['fatal', 'critical', 'error', 'warning', 'info', 'debug'],

  /**
  * @typedef {Em.Object} FilterKeyword
  * @property {Boolean} isIncluded determines include/exclude status of keyword
  * @property {String} id unique identifier
  * @property {String} value keyword value
  */

  /**
  * Stores all selected keywords.
  *
  * @type {FilterKeyword[]}
  */
  selectedKeywords: [],

  selectedKeywordsDidChange: function () {
    this.fetchContent();
  }.observes('selectedKeywords.length'),

  levelsContext: function () {
    var self = this;
    var levels = this.get('logLevels');

    return Em.A(levels.map(function (level) {
      return Em.Object.create({ name: level.toUpperCase(), counter: 0, displayName: level.capitalize(), checked: false });
    }));
  }.property(),

  levelsContextDidChange: function (e) {
    this.fetchContent();
  }.observes('levelsContext.@each.checked'),

  /** mock data **/
  content: function () {
    var data = [{
      message: 'java.lang.NullPointerException',
      date: '05.12.2016, 10:10:20',
      level: 'INFO'
    }, {
      message: 'java.lang.NullPointerException',
      date: '05.12.2016, 10:10:20',
      level: 'ERROR'
    }];

    var initialSize = 20;
    var ret = [];

    for (var i = 0; i < 20; i++) {
      ret.push(Em.Object.create(data[Math.ceil(Math.random() * 2) - 1]));
    }
    return ret;
  }.property(),

  contentDidChange: function () {
    this.refreshLevelCounters();
  }.observes('content.length'),

  dateFromValue: null,
  dateToValue: null,

  keywordsFilterView: filters.createTextView({
    layout: Em.Handlebars.compile('{{yield}}')
  }),

  keywordsFilterValue: null,

  didInsertElement: function didInsertElement() {
    this._super();
    this.infiniteScrollInit(this.$().find('.log-file-search-content'), {
      callback: this.loadMore.bind(this)
    });
    this.$().find('.log-file-search-content').contextmenu({
      target: '#log-file-search-item-context-menu'
    });
    this.refreshLevelCounters();
  },

  /** mock data **/
  loadMore: function loadMore() {
    var dfd = $.Deferred();
    var self = this;
    setTimeout(function () {
      var data = self.get('content');
      self.get('content').pushObjects(data.slice(0, 10));
      dfd.resolve();
    }, Math.ceil(Math.random() * 4000));
    return dfd.promise();
  },

  refreshLevelCounters: function refreshLevelCounters() {
    var self = this;
    this.get('logLevels').forEach(function (level) {
      var levelContext = self.get('levelsContext').findProperty('name', level.toUpperCase());
      levelContext.set('counter', self.get('content').filterProperty('level', level.toUpperCase()).length);
    });
  },

  /**
  * Make request and get content with applied filters.
  */
  fetchContent: function fetchContent() {
    console.debug('Make Request with params:', this.serializeFilters());
  },

  submitKeywordsValue: function submitKeywordsValue() {
    this.fetchContent();
  },

  serializeFilters: function serializeFilters() {
    var levels = this.serializeLevelFilters();
    var keywords = this.serializeKeywordsFilter();
    var date = this.serializeDateFilter();
    var includedExcludedKeywords = this.serializeIncludedExcludedKeywordFilter();

    return [levels, keywords, date, includedExcludedKeywords].compact().join('&');
  },

  serializeKeywordsFilter: function serializeKeywordsFilter() {
    return !!this.get('keywordsFilterValue') ? 'keywords=' + this.get('keywordsFilterValue') : null;
  },

  serializeDateFilter: function serializeDateFilter() {
    var dateFrom = !!this.get('dateFromValue') ? 'dateFrom=' + this.get('dateFromValue') : null;
    var dateTo = !!this.get('dateToValue') ? 'dateTo=' + this.get('dateFromValue') : null;
    var ret = [dateTo, dateFrom].compact();
    return ret.length ? ret.join('&') : null;
  },

  serializeLevelFilters: function serializeLevelFilters() {
    var selectedLevels = this.get('levelsContext').filterProperty('checked').mapProperty('name');
    return selectedLevels.length ? 'levels=' + selectedLevels.join(',') : null;
  },

  serializeIncludedExcludedKeywordFilter: function serializeIncludedExcludedKeywordFilter() {
    var self = this;
    var getValues = function getValues(included) {
      return self.get('selectedKeywords').filterProperty('isIncluded', included).mapProperty('value');
    };
    var included = getValues(true).join(',');
    var excluded = getValues(false).join(',');
    var ret = [];
    if (included.length) ret.push('include=' + included);
    if (excluded.length) ret.push('exclude=' + excluded);
    return ret.length ? ret.join('&') : null;
  },

  /** include/exclude keywords methods **/

  keywordToId: function keywordToId(keyword) {
    return keyword.toLowerCase().split(' ').join('_');
  },

  /**
  * Create keyword object
  * @param  {string} keyword keyword value
  * @param  {object} [opts]
  * @return {Em.Object}
  */
  createSelectedKeyword: function createSelectedKeyword(keyword, opts) {
    var defaultOpts = {
      isIncluded: false,
      id: this.keywordToId(keyword),
      value: keyword
    };
    return Em.Object.create($.extend({}, defaultOpts, opts));
  },

  /**
  * Adds keyword if not added.
  * @param  {FilterKeyword} keywordObject
  */
  addKeywordToList: function addKeywordToList(keywordObject) {
    if (!this.get('selectedKeywords').someProperty('id', keywordObject.get('id'))) {
      this.get('selectedKeywords').pushObject(keywordObject);
    }
  },

  /**
  * @param  {FilterKeyword} keyword
  */
  includeSelectedKeyword: function includeSelectedKeyword(keyword) {
    this.addKeywordToList(this.createSelectedKeyword(keyword, { isIncluded: true }));
  },

  /**
  * @param  {FilterKeyword} keyword
  */
  excludeSelectedKeyword: function excludeSelectedKeyword(keyword) {
    this.addKeywordToList(this.createSelectedKeyword(keyword, { isIncluded: false }));
  },

  /** view actions **/

  /** toolbar context menu actions **/
  moveTableTop: function moveTableTop(e) {
    var $el = $('.log-file-search-content');
    $el.scrollTop(0);
    $el = null;
  },

  moveTableBottom: function moveTableBottom(e) {
    var $el = $('.log-file-search-content');
    $el.scrollTop($el.get(0).scrollHeight);
    $el = null;
  },

  navigateToLogUI: function navigateToLogUI(e) {
    console.error('navigate to Log UI');
  },

  removeKeyword: function removeKeyword(e) {
    this.get('selectedKeywords').removeObject(e.context);
  },

  /** toolbar reset filter actions **/
  resetKeywordsDateFilter: function resetKeywordsDateFilter(e) {
    this.setProperties({
      keywordsFilterValue: '',
      dateFromValue: '',
      dateToValue: ''
    });
  },

  resetLevelsFilter: function resetLevelsFilter(e) {
    this.get('levelsContext').invoke('set', 'checked', false);
  },

  resetKeywordsFilter: function resetKeywordsFilter(e) {
    this.get('selectedKeywords').clear();
  },

  /** log search item context menu actions **/
  includeSelected: function includeSelected() {
    var selection = window.getSelection().toString();
    if (!!selection) this.includeSelectedKeyword(selection);
  },

  excludeSelected: function excludeSelected() {
    var selection = window.getSelection().toString();
    if (!!selection) this.excludeSelectedKeyword(selection);
  }
});

});

require.register("views/common/log_search_ui_link_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/common/quick_view_link_view');

/**
 * This view helps to get and format correct link to LogSearch UI with query params.
 * Used mostly for navigation to concrete log file.
 *
 * @augments App.QuickLinksView
 */
App.LogSearchUILinkView = App.QuickLinksView.extend({
  content: function () {
    return App.Service.find().findProperty('serviceName', 'LOGSEARCH');
  }.property(),

  formatedLink: function () {
    if (this.get('quickLinks.length')) {
      return Em.get(this.get('quickLinks').findProperty('label', 'Log Search UI'), 'url') + this.get('linkQueryParams');
    }
    return false;
  }.property('quickLinks.length', 'linkQueryParams')
});

});

require.register("views/common/log_tail_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var dateUtils = require('utils/date/date');

App.LogTailView = Em.View.extend(App.InfiniteScrollMixin, {
  startIndex: 0,
  selectedTailCount: 50,
  autoResize: false,
  logRows: Em.A([]),
  totalCount: 0,
  totalItems: 0,
  lastLogTime: 0,
  isDataReady: false,
  refreshEnd: true,
  pollLogs: true,
  pollLogInterval: 2000,
  pollLogTimeoutId: null,

  oldLogsAvailable: false,
  oldLogsIsFetching: false,

  templateName: require('templates/common/log_tail'),

  content: null,

  isVisible: Em.computed.not('App.isClusterUser'),

  /**
   * elements size are:
   * .modal margin 40px x 2
   * .modal inner padding 15px x 2
   * .top-wrap-header height + margin 60px
   * .modal-footer 60px
   * .log-tail-popup-info 45px
   *
   * @property {number} resizeDelta
   */
  resizeDelta: 15 * 2 + 60 + 60 + 40 + 45,

  didInsertElement: function didInsertElement() {
    var self = this;
    this.infiniteScrollInit(this.$('.log-tail-content'), {
      appendHtml: '<span id="empty-span"></span>'
    });
    this.fetchRows({
      startIndex: 0
    }).then(function (data) {
      var logs = self.fetchRowsSuccess(data);
      self.infiniteScrollSetDataAvailable(true);
      self.appendLogRows(logs.reverse());
      self.saveLastTimestamp(logs);
      self.set('isDataReady', true);
      self.scrollToBottom();
      self.startLogPolling();
    });
    this.subscribeResize();
  },

  scrollToBottom: function scrollToBottom() {
    Em.run.next(this, function () {
      this.$('.log-tail-content').scrollTop(this.$('.log-tail-content').prop('scrollHeight'));
    });
  },

  _infiniteScrollHandler: function _infiniteScrollHandler(e) {
    var self = this;
    this._super(e);
    if ($(e.target).scrollTop() === 0) {
      if (this.get('noOldLogs') && !this.get('oldLogsIsFetching')) return;
      self.set('oldLogsIsFetching', true);
      this.fetchRows(this.oldestLogs()).then(function (data) {
        var oldestLog = self.get('logRows.0.logtime');
        var logRows = self.fetchRowsSuccess(data).filter(function (i) {
          return parseInt(i.get('logtime'), 10) < parseInt(oldestLog, 10);
        });
        if (logRows.length) {
          self.get('logRows').unshiftObjects(logRows.reverse());
        } else {
          self.set('noOldLogs', true);
        }
        self.set('oldLogsIsFetching', false);
      });
    }
  },

  willDestroyElement: function willDestroyElement() {
    this._super();
    this.stopLogPolling();
    this.unsubscribeResize();
    this.get('logRows').clear();
  },

  resizeHandler: function resizeHandler() {
    // window.innerHeight * 0.1 = modal popup top 5% from both sides = 10%
    if (this.get('state') === 'destroyed') return;
    var newSize = $(window).height() - this.get('resizeDelta') - window.innerHeight * 0.08;
    this.$().find('.log-tail-content.pre-styled').css('maxHeight', newSize + 'px');
  },

  unsubscribeResize: function unsubscribeResize() {
    if (!this.get('autoResize')) return;
    $(window).off('resize', this.resizeHandler.bind(this));
  },

  subscribeResize: function subscribeResize() {
    if (!this.get('autoResize')) return;
    this.resizeHandler();
    $(window).on('resize', this.resizeHandler.bind(this));
  },

  fetchRows: function fetchRows(opts) {
    var options = $.extend({}, true, {
      startIndex: this.get('startIndex'),
      pageSize: this.get('selectedTailCount')
    }, opts || {});
    return App.ajax.send({
      sender: this,
      name: 'logtail.get',
      data: {
        hostName: this.get('content.hostName'),
        logComponentName: this.get('content.logComponentName'),
        pageSize: "" + options.pageSize,
        startIndex: "" + options.startIndex
      },
      error: 'fetchRowsError'
    });
  },

  fetchRowsSuccess: function fetchRowsSuccess(data) {
    var logRows = Em.getWithDefault(data, 'logList', []).map(function (logItem, i) {
      var item = App.keysUnderscoreToCamelCase(App.permit(logItem, ['log_message', 'logtime', 'level', 'id']));
      item.logtimeFormatted = dateUtils.dateFormat(parseInt(item.logtime, 10), 'YYYY-MM-DD HH:mm:ss,SSS');
      return Em.Object.create(item);
    });
    if (logRows.length === 0) {
      this.infiniteScrollSetDataAvailable(false);
      return [];
    }
    return logRows;
  },

  fetchRowsError: function fetchRowsError() {},

  /** actions **/

  openInLogSearch: function openInLogSearch() {},

  refreshContent: function refreshContent() {
    var self = this,
        allIds;
    if (!this.get('refreshEnd')) {
      return $.Deferred().resolve().promise();
    }
    this.set('refreshEnd', false);
    //this.infiniteScrollSetDataAvailable(true);
    allIds = this.get('logRows').mapProperty('id');
    return this.fetchRows(this.currentPage()).then(function (data) {
      var logRows = self.fetchRowsSuccess(data).filter(function (i) {
        return parseInt(i.logtime, 10) > parseInt(self.get('lastLogTime'), 10) && !allIds.contains(i.get('id'));
      });
      if (logRows.length) {
        self.appendLogRows(logRows.reverse());
        self.saveLastTimestamp(logRows);
      }
      self.set('refreshEnd', true);
    });
  },

  appendLogRows: function appendLogRows(logRows) {
    this.get('logRows').pushObjects(logRows);
  },

  saveLastTimestamp: function saveLastTimestamp(logRows) {
    this.set('lastLogTime', Em.getWithDefault(logRows, '0.logtime', 0));
    return this.get('lastLogTime');
  },

  currentPage: function currentPage() {
    return {
      startIndex: 0,
      pageSize: this.get('selectedTailCount')
    };
  },

  nextPage: function nextPage() {
    var newIndex = this.get('startIndex') + this.get('selectedTailCount');
    if (newIndex < 0) {
      newIndex = 0;
    }
    this.set('startIndex', newIndex);
    return {
      startIndex: newIndex,
      pageSize: this.get('selectedTailCount')
    };
  },

  oldestLogs: function oldestLogs() {
    return {
      startIndex: this.get('logRows.length'),
      pageSize: this.get('selectedTailCount')
    };
  },

  startLogPolling: function startLogPolling() {
    var self = this;
    if (!this.get('pollLogs') || this.get('state') === 'destroyed') return;
    this.set('pollLogTimeoutId', setTimeout(function () {
      self.stopLogPolling();
      self.refreshContent().then(self.startLogPolling.bind(self));
    }, this.get('pollLogInterval')));
  },

  stopLogPolling: function stopLogPolling() {
    if (!this.get('pollLogs') || this.get('pollLogTimeoutId') === null) return;
    clearTimeout(this.get('pollLogTimeoutId'));
  }

});

});

require.register("views/common/metric", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * use: {{view App.MetricFilteringWidget controllerBinding="App.router.mainChartsController"}}
 * set controller.metric field with metric value
 * widget assign itself to controller like metricWidget (controller.get('metricWidget'))
 * @type {*}
 */
App.MetricFilteringWidget = Em.View.extend({
  classNames: ['metric-filtering-widget'],
  /**
   * chosen metric value
   */
  chosenMetric: null,
  chosenMoreMetric: null,
  showMore: 0, // toggle more metrics indicator
  /**
   * metrics
   */
  metrics: [Em.Object.create({ label: Em.I18n.t('metric.default'), value: null }), Em.Object.create({ label: Em.I18n.t('metric.cpu'), value: 'cpu' }), Em.Object.create({ label: Em.I18n.t('metric.memory'), value: 'memory' }), Em.Object.create({ label: Em.I18n.t('metric.network'), value: 'network' }), Em.Object.create({ label: Em.I18n.t('metric.io'), value: 'io' })],

  moreMetrics: [Em.Object.create({ label: Em.I18n.t('metric.more.cpu'), code: 'cpu', items: [Em.Object.create({ value: "cpu_nice" }), Em.Object.create({ value: "cpu_wio" }), Em.Object.create({ value: "cpu_user" }), Em.Object.create({ value: "cpu_idle" }), Em.Object.create({ value: "cpu_system" }), Em.Object.create({ value: "cpu_aidle" })] }), Em.Object.create({ label: Em.I18n.t('metric.more.disk'), code: 'disk',
    items: [Em.Object.create({ value: 'disk_free' }), Em.Object.create({ value: 'disk_total' }), Em.Object.create({ value: 'part_max_used' })]
  }), Em.Object.create({ label: Em.I18n.t('metric.more.load'), code: 'load',
    items: [Em.Object.create({ value: 'load_one' }), Em.Object.create({ value: 'load_five' }), Em.Object.create({ value: 'load_fifteen' })]
  }), Em.Object.create({ label: Em.I18n.t('metric.more.memory'), code: 'memory',
    items: [Em.Object.create({ value: 'swap_free' }), Em.Object.create({ value: 'cpu' })]
  }), Em.Object.create({ label: Em.I18n.t('metric.more.network'), code: 'network',
    items: [Em.Object.create({ value: 'bytes_out' }), Em.Object.create({ value: 'bytes_in' }), Em.Object.create({ value: 'pkts_in' }), Em.Object.create({ value: 'pkts_out' })]
  }), Em.Object.create({ label: Em.I18n.t('metric.more.process'), code: 'process',
    items: [Em.Object.create({ value: 'proc_run' }), Em.Object.create({ value: 'proc_total' })]
  })],

  /**
   * return array of chosen metrics
   */
  chosenMetrics: function () {
    return this.get('chosenMetric') ? [this.get('chosenMetric')] : this.get('defaultMetrics');
  }.property('chosenMetric'),

  /**
   * metric item view
   */
  itemView: Em.View.extend({
    tagName: 'li',
    classNameBindings: ['disabled'],
    disabled: Em.computed.ifThenElse('isActive', 'disabled', false),
    isActive: Em.computed.equalProperties('metric.value', 'widget.chosenMetric'),
    label: Em.computed.alias('metric.label'),
    template: Em.Handlebars.compile('<a {{action activate view.metric.value target="view.widget" href="#" }}>{{unbound view.label}}</a>')
  }),

  moreItemView: function () {
    return this.get('itemView').extend({
      label: Em.computed.alias('metric.value')
    });
  }.property(),

  /**
   * return default selected metrics (currently - all)
   */
  defaultMetrics: function () {
    var values = [];
    $.each(this.get('metrics'), function () {
      if (this.value) {
        values.push(this.value);
      }
    });
    return values;
  }.property(),

  bindToController: function bindToController() {
    var thisW = this;
    var controller = this.get('controller');
    controller.set('metricWidget', thisW);
  },

  toggleMore: function toggleMore() {
    this.set('showMore', 1 - this.get('showMore'));
  },

  /**
   * assign this widget to controller, prepare items by metricsConfig
   */
  init: function init() {
    this._super();
    this.bindToController();
  },

  /**
   * write active metric to widget
   * @param event
   */
  activate: function activate(event) {
    this.set('chosenMetric', event.context);
  },

  templateName: require('templates/common/metric')
});

});

require.register("views/common/modal_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ModalPopup = Ember.View.extend({

  viewName: 'modalPopup',
  modalDialogClasses: [],
  templateName: require('templates/common/modal_popup'),
  header: '&nbsp;',
  body: '&nbsp;',
  encodeBody: true,
  // define bodyClass which extends Ember.View to use an arbitrary Handlebars template as the body
  primary: Em.I18n.t('ok'),
  secondary: Em.I18n.t('common.cancel'),
  third: null,
  autoHeight: true,
  marginBottom: 300,
  disablePrimary: false,
  disableSecondary: false,
  disableThird: false,
  primaryClass: 'btn-success',
  secondaryClass: 'btn-default',
  thirdClass: 'btn-default',
  modalDialogClassesStr: function () {
    var modalDialogClasses = this.get('modalDialogClasses');
    if (!Em.isArray(modalDialogClasses)) {
      return '';
    }
    return modalDialogClasses.join(' ');
  }.property('modalDialogClasses.[]'),
  primaryId: '',
  secondaryId: '',
  thirdId: '',
  'data-qa': 'modal',
  onPrimary: function onPrimary() {
    this.hide();
  },

  onSecondary: function onSecondary() {
    this.hide();
  },

  onThird: function onThird() {
    this.hide();
  },

  onClose: function onClose() {
    this.hide();
  },

  hide: function hide() {
    if (!$.mocho) {
      this.$('#modal').modal('hide');
    }
    this.destroy();
  },

  showFooter: true,

  /**
   * Hide or show 'X' button for closing popup
   */
  showCloseButton: true,

  didInsertElement: function didInsertElement() {
    this.$().find('#modal').on('enter-key-pressed', this.enterKeyPressed.bind(this)).on('escape-key-pressed', this.escapeKeyPressed.bind(this));
    this.fitZIndex();
    this.handleBackDrop();
    var firstInputElement = this.$('#modal').find(':input').not(':disabled, .no-autofocus').first();
    if (!$.mocho) {
      this.$('#modal').modal({
        keyboard: false,
        backdrop: false
      });
    }
    this.focusElement(firstInputElement);
    this.subscribeResize();
  },

  handleBackDrop: function handleBackDrop() {
    if (this.get('backdrop') === false) {
      $('.modal-backdrop').css('visibility', 'hidden');
    } else {
      $('.modal-backdrop').css('visibility', 'visible');
    }
  },

  subscribeResize: function subscribeResize() {
    if (this.get('autoHeight') && !$.mocho) {
      this.fitHeight();
      $(window).on('resize', this.fitHeight.bind(this));
    }
  },

  willDestroyElement: function willDestroyElement() {
    this.$().find('#modal').off('enter-key-pressed').off('escape-key-pressed');
    if (this.get('autoHeight') && !$.mocho) {
      $(window).off('resize', this.fitHeight);
    }
  },

  escapeKeyPressed: function escapeKeyPressed(event) {
    var closeButton = this.$().find('.modal-header > .close').last();
    if (closeButton.length > 0) {
      event.preventDefault();
      event.stopPropagation();
      closeButton.click();
      return false;
    }
  },

  enterKeyPressed: function enterKeyPressed(event) {
    var primaryButton = this.$().find('.modal-footer > .btn-success').last();
    if (!$("*:focus").is('textarea') && primaryButton.length > 0 && primaryButton.attr('disabled') !== 'disabled') {
      event.preventDefault();
      event.stopPropagation();
      primaryButton.click();
      return false;
    }
  },

  /**
   * If popup is opened from another popup it should be displayed above
   * @method fitZIndex
   */
  fitZIndex: function fitZIndex() {
    var existedPopups = $('.modal-backdrop');
    if (existedPopups && !$.mocho) {
      var maxZindex = 1;
      existedPopups.each(function (index, popup) {
        if ($(popup).css('z-index') > maxZindex) {
          maxZindex = $(popup).css('z-index');
        }
      });
      this.$().find('.modal-backdrop').css('z-index', maxZindex * 2);
      this.$().find('.modal').css('z-index', maxZindex * 2 + 1);
    }
  },

  focusElement: function focusElement(elem) {
    elem.focus();
  },

  fitHeight: function fitHeight() {
    if (this.get('state') === 'destroyed') return;
    var popup = this.$().find('#modal'),
        wrapper = $(popup).find('.modal-dialog'),
        block = $(popup).find('.modal-body'),
        wh = $(window).height(),
        ww = $(window).width(),
        topNavPaddingTop = 19,
        // from ambari-web/app/styles/common.less
    topNavFontSize = 20,
        // from ambari-web/app/styles/common.less
    topNavLineHeight = 1.3,
        // from ambari-web/app/styles/common.less
    modalMarginTopDefault = 10,
        // from ambari-web/app/styles/common.less
    modalMarginTopWide = 30,
        // from ambari-web/app/styles/common.less
    modalMarginTop = ww < 768 ? modalMarginTopDefault : modalMarginTopWide,
        // from ambari-web/vendor/styles/bootstrap.css
    top = topNavPaddingTop + topNavFontSize * topNavLineHeight - modalMarginTop;
    var newMaxHeight = wh - top * 2 - (wrapper.height() - block.height());

    popup.css({
      'top': top + 'px',
      'marginTop': 0
    });

    newMaxHeight = Math.max(newMaxHeight, 500);
    block.css('max-height', newMaxHeight);
  }
});

App.ModalPopup.reopenClass({

  show: function show(options) {
    var popup = this.create(options);
    popup.appendTo('#wrapper');
    return popup;
  }

});

});

require.register("views/common/modal_popups/alert_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show alert popup
 *
 * @param {String} header - header of the popup
 * @param {String} body - body of the popup
 * @param {Function} callback - function to call upon clicking the OK button, clicking "x" or pressing "Esc"
 * @return {*}
 */
App.showAlertPopup = function (header, body, callback) {
  return App.ModalPopup.show({
    'data-qa': 'alert-modal',
    primary: Em.I18n.t('ok'),
    secondary: null,
    header: header,
    body: body,
    hide: function hide() {
      this._super();
      if (callback) {
        callback();
      }
    }
  });
};

});

require.register("views/common/modal_popups/cluster_check_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function mapUpgradeChecks(items) {
  return items.map(function (item) {
    return Em.getProperties(item.UpgradeChecks, ['failed_on', 'reason', 'check']);
  });
}

/**
 * popup to display requirements that are not met
 * for current action
 * @param data
 * @param popup
 * @param configs
 * @returns {*|void}
 */
App.showClusterCheckPopup = function (data, popup, configs) {
  var fails = data.items.filterProperty('UpgradeChecks.status', 'FAIL'),
      warnings = data.items.filterProperty('UpgradeChecks.status', 'WARNING'),
      bypass = data.items.filterProperty('UpgradeChecks.status', 'BYPASS'),
      configsMergeConflicts = configs ? configs.filterProperty('wasModified', false) : [],
      configsRecommendations = configs ? configs.filterProperty('wasModified', true) : [],
      primary,
      secondary;
  popup = popup || {};

  if (Em.isNone(popup.primary)) {
    primary = fails.length ? Em.I18n.t('common.dismiss') : Em.I18n.t('common.proceedAnyway');
  } else {
    primary = popup.primary;
  }

  if (Em.isNone(popup.secondary)) {
    secondary = fails.length ? false : Em.I18n.t('common.cancel');
  } else {
    secondary = popup.secondary;
  }

  return App.ModalPopup.show({
    primary: primary,
    secondary: secondary,
    header: popup.header,
    classNames: ['cluster-check-popup'],
    bodyClass: Em.View.extend({
      failTitle: popup.failTitle,
      failAlert: popup.failAlert,
      warningTitle: popup.warningTitle,
      warningAlert: popup.warningAlert,
      templateName: require('templates/common/modal_popups/cluster_check_dialog'),
      warnings: mapUpgradeChecks(warnings),
      fails: mapUpgradeChecks(fails),
      bypass: mapUpgradeChecks(bypass), // errors that can be bypassed
      hasConfigsMergeConflicts: configsMergeConflicts.length > 0,
      hasConfigsRecommendations: configsRecommendations.length > 0,
      configsMergeTable: App.getMergeConflictsView(configsMergeConflicts),
      configsRecommendTable: App.getNewStackRecommendationsView(configsRecommendations),
      isAllPassed: !fails.length && !warnings.length && !bypass.length && !configsMergeConflicts.length && !configsRecommendations.length
    }),
    onPrimary: function onPrimary() {
      this._super();
      if (!popup.noCallbackCondition && popup.callback) {
        popup.callback();
      }
    },
    didInsertElement: function didInsertElement() {
      this._super();
      this.fitHeight();
    }
  });
};

App.getMergeConflictsView = function (configs) {
  return Em.View.extend({
    templateName: require('templates/main/admin/stack_upgrade/upgrade_configs_merge_table'),
    configs: configs
  });
};

App.getNewStackRecommendationsView = function (configs) {
  return Em.View.extend({
    templateName: require('templates/main/admin/stack_upgrade/upgrade_configs_recommend_table'),
    configs: configs
  });
};

});

require.register("views/common/modal_popups/config_validation/config_validation_failed_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show alert popup
 *
 * @return {*}
 */
App.showConfigValidationFailedPopup = function (primary, secondary) {
  return App.ModalPopup.show({
    header: Em.I18n.t('installer.step7.popup.validation.failed.header'),
    primary: Em.I18n.t('common.proceedAnyway'),
    primaryClass: 'btn-danger',
    marginBottom: 200,
    onPrimary: function onPrimary() {
      this._super();
      primary();
    },
    onSecondary: function onSecondary() {
      this._super();
      secondary();
    },
    onClose: function onClose() {
      this._super();
      secondary();
    },
    body: Em.I18n.t('installer.step7.popup.validation.request.failed.body')
  });
};

});

require.register("views/common/modal_popups/config_validation/config_validation_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show alert popup
 *
 * @return {*}
 */
App.showConfigValidationPopup = function (configErrors, primary, secondary, controller) {
  return App.ModalPopup.show({
    header: Em.I18n.t('installer.step7.popup.validation.warning.header'),
    classNames: ['common-modal-wrapper'],
    modalDialogClasses: ['modal-xlg'],
    primary: Em.I18n.t('common.proceedAnyway'),
    primaryClass: 'btn-danger',
    marginBottom: 200,
    onPrimary: function onPrimary() {
      this._super();
      primary();
    },
    onSecondary: function onSecondary() {
      this._super();
      secondary();
    },
    onClose: function onClose() {
      this._super();
      secondary();
    },
    disablePrimary: !!configErrors.get('criticalIssues.length'),
    bodyClass: App.ValidationsView.extend({
      controller: controller,
      configErrors: configErrors
    })
  });
};

App.ValidationsView = Em.View.extend({
  templateName: require('templates/common/modal_popups/config_recommendation_popup')
});

});

require.register("views/common/modal_popups/confirmation_feedback_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show confirmation popup
 * After sending command watch status of query,
 * and in case of failure provide ability to retry to launch an operation.
 *
 * @param {Function} primary - "OK" button click handler
 * @param {Object} bodyMessage - confirmMsg:{String},
 confirmButton:{String},
 additionalWarningMsg:{String},
 * @param {Function} secondary - "Cancel" button click handler
 * @return {*}
 */
App.showConfirmationFeedBackPopup = function (primary, bodyMessage, secondary) {
  if (!primary) {
    return false;
  }
  return App.ModalPopup.show({
    header: Em.I18n.t('popup.confirmation.commonHeader'),
    bodyClass: Em.View.extend({
      templateName: require('templates/common/modal_popups/confirmation_feedback')
    }),
    query: Em.Object.create({ status: "INIT" }),
    primary: bodyMessage ? bodyMessage.confirmButton : Em.I18n.t('ok'),
    onPrimary: function onPrimary() {
      this.set('query.status', "INIT");
      this.set('disablePrimary', true);
      this.set('disableSecondary', true);
      this.set('statusMessage', Em.I18n.t('popup.confirmationFeedBack.sending'));
      this.hide();
      primary(this.get('query'), this.get('runMmOperation'));
    },
    statusMessage: bodyMessage ? bodyMessage.confirmMsg : Em.I18n.t('question.sure'),
    additionalWarningMsg: bodyMessage ? bodyMessage.additionalWarningMsg : null,
    putInMaintenance: bodyMessage ? bodyMessage.putInMaintenance : null,
    runMmOperation: false,
    turnOnMmMsg: bodyMessage ? bodyMessage.turnOnMmMsg : null,
    watchStatus: function () {
      if (this.get('query.status') === "SUCCESS") {
        this.hide();
      } else if (this.get('query.status') === "FAIL") {
        this.set('primaryClass', 'btn-primary');
        this.set('primary', Em.I18n.t('common.retry'));
        this.set('disablePrimary', false);
        this.set('disableSecondary', false);
        this.set('statusMessage', Em.I18n.t('popup.confirmationFeedBack.query.fail'));
      }
    }.observes('query.status'),
    onSecondary: function onSecondary() {
      this.hide();
      if (secondary) {
        secondary();
      }
    }
  });
};

});

require.register("views/common/modal_popups/confirmation_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show confirmation popup
 *
 * @param {Function} primary - "OK" button click handler
 * @param {String} body - additional text constant. Will be placed in the popup-body
 * @param {Function} secondary
 * @param {String} header
 * @param {String} primaryText
 * @param {String} primaryStyle
 * @return {*}
 */
App.showConfirmationPopup = function (primary, body, secondary, header, primaryText) {
  var primaryStyle = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 'success';
  var staticId = arguments[6];

  var primaryClass = {
    'success': 'btn-success',
    'warning': 'btn-warning',
    'danger': 'btn-danger'
  }[primaryStyle];
  if (!primary) {
    return false;
  }
  return App.ModalPopup.show({
    'data-qa': 'confirmation-modal',
    encodeBody: false,
    primary: primaryText || Em.I18n.t('ok'),
    header: header || Em.I18n.t('popup.confirmation.commonHeader'),
    body: body || Em.I18n.t('question.sure'),
    primaryClass: primaryClass,
    primaryId: staticId ? staticId + '_primary' : '',
    secondaryId: staticId ? staticId + '_secondary' : '',
    thirdId: staticId ? staticId + '_third' : '',
    onPrimary: function onPrimary() {
      this.hide();
      primary();
    },
    onSecondary: function onSecondary() {
      this.hide();
      if (secondary) {
        secondary();
      }
    },
    onClose: function onClose() {
      this.hide();
      if (secondary) {
        secondary();
      }
    }
  });
};

});

require.register("views/common/modal_popups/dependent_configs_list_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DependentConfigsTableView = Em.View.extend({
  templateName: require('templates/common/modal_popups/dependent_configs_table'),
  recommendations: [],
  isClickable: false,
  showPopovers: true,
  elementsWithPopover: function () {
    return this.$('td.config-dependency-name');
  }.property(),
  hideMessage: function () {
    return this.get('controller.isInstallWizard');
  }.property('controller.isInstallWizard'),
  updateRecommendedDefault: function () {
    if (this.get('controller.isInstallWizard')) {
      var applyRecommendations = this.get('recommendations').filterProperty('saveRecommended');
      var dontApplyRecommendations = this.get('recommendations').filterProperty('saveRecommended', false);
      this.get('controller').undoRedoRecommended(applyRecommendations, true);
      this.get('controller').undoRedoRecommended(dontApplyRecommendations, false);
    }
  }.observes('recommendations.@each.saveRecommended'),
  isEditable: true,
  title: Em.computed.ifThenElse('isEditable', Em.I18n.t('popup.dependent.configs.table.recommended'), Em.I18n.t('popup.dependent.configs.table.required')),
  message: function () {
    var message = '';
    if (this.get('isEditable')) {
      if (this.get('parentView.isAfterRecommendation')) {
        message += Em.I18n.t('popup.dependent.configs.title.recommendation') + '<br>';
      }
      message += Em.I18n.t('popup.dependent.configs.title.values');
    } else {
      message += Em.I18n.t('popup.dependent.configs.title.required');
    }
    return message;
  }.property('isEditable'),
  didInsertElement: function didInsertElement() {
    if (this.get('showPopovers')) {
      App.popover(this.get('elementsWithPopover'), {
        placement: 'auto right',
        trigger: 'hover',
        html: true
      });
    }
  },
  willDestroyElement: function willDestroyElement() {
    if (this.get('showPopovers')) {
      this.get('elementsWithPopover').popover('destroy');
    }
  }
});

App.DependentConfigsListView = Em.View.extend({
  templateName: require('templates/common/modal_popups/dependent_configs_list'),
  isAfterRecommendation: true,
  isRecommendationsClickable: false,
  showRecommendationsPopovers: true,
  recommendations: [],
  requiredChanges: [],
  allConfigsWithErrors: [],
  toggleAllId: '',
  toggleAll: App.CheckboxView.extend({
    didInsertElement: function didInsertElement() {
      this.set('parentView.toggleAllId', this.get('elementId'));
      this.updateCheckbox();
    },
    click: function click() {
      Em.run.next(this, 'updateSaveRecommended');
    },
    updateCheckboxObserver: function () {
      Em.run.once(this, 'updateCheckbox');
    }.observes('parentView.recommendations.@each.saveRecommended'),

    updateCheckbox: function updateCheckbox() {
      this.set('checked', !(this.get('parentView.recommendations') || []).someProperty('saveRecommended', false));
    },
    updateSaveRecommended: function updateSaveRecommended() {
      this.get('parentView.recommendations').setEach('saveRecommended', this.get('checked'));
    }
  }),
  setAllConfigsWithErrors: function () {
    if (this.get('state') === 'inBuffer' || Em.isNone(this.get('controller.stepConfigs'))) {
      return;
    }
    this.set('allConfigsWithErrors', this.get('controller.stepConfigs').reduce(function (result, stepConfig) {
      if (stepConfig.get('configsWithErrors.length')) {
        result = result.concat(stepConfig.get('configsWithErrors'));
      }
      return result;
    }, []));
  }.observes('controller.stepConfigs.@each.configsWithErrors'),
  didInsertElement: function didInsertElement() {
    $('span.dropdown-toggle').dropdown();
    this.setAllConfigsWithErrors();
    this._super();
  }
});

/**
 * Show confirmation popup
 * @param {[Object]} recommendations
 * @param {[Object]} requiredChanges
 * @param {function} [primary=null]
 * @param {function} [secondary=null]
 * @param {boolean} [isRecommendationsClickable=false]
 * @param {boolean} [isRecommendationsClickable=true]
 * we use this parameter to defer saving configs before we make some decisions.
 * @return {App.ModalPopup}
 */
App.showDependentConfigsPopup = function (recommendations, requiredChanges, primary, secondary, controller) {
  var isRecommendationsClickable = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : false;
  var showRecommendationsPopovers = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : true;

  return App.ModalPopup.show({
    encodeBody: false,
    header: Em.I18n.t('popup.dependent.configs.header'),
    classNames: ['common-modal-wrapper'],
    modalDialogClasses: ['modal-xlg'],
    secondaryClass: 'cancel-button',
    bodyClass: App.DependentConfigsListView.extend({
      recommendations: recommendations,
      requiredChanges: requiredChanges,
      controller: controller,
      isRecommendationsClickable: isRecommendationsClickable,
      showRecommendationsPopovers: showRecommendationsPopovers
    }),
    saveChanges: function saveChanges() {
      recommendations.forEach(function (c) {
        Em.set(c, 'saveRecommendedDefault', Em.get(c, 'saveRecommended'));
      });
    },
    discardChanges: function discardChanges() {
      recommendations.forEach(function (c) {
        Em.set(c, 'saveRecommended', Em.get(c, 'saveRecommendedDefault'));
      });
    },
    onPrimary: function onPrimary() {
      this._super();
      if (primary) primary();
      this.saveChanges();
    },
    onSecondary: function onSecondary() {
      this._super();
      if (secondary) secondary();
      this.discardChanges();
    },
    onClose: function onClose() {
      this.onSecondary();
    }
  });
};

});

require.register("views/common/modal_popups/edit_dashboard_widget_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var _require = require('utils/validator'),
    isValidFloat = _require.isValidFloat;

/**
 * Thresholds manager for sliders with single value
 * Usage:
 * <pre>
 * SingleHandler.create({
 *    thresholdMin: 12,
 *    minValue: 10,
 *    maxValue: 100
 * });
 * </pre>
 *
 * @class SingleHandler
 */


var SingleHandler = Em.Object.extend({

  /**
   * @type {number}
   */
  thresholdMin: null,

  /**
   * @type {number}
   * @default 0
   */
  minValue: 0,

  /**
   * @type {number}
   */
  maxValue: null,

  /**
   * Is <code>thresholdMin</code> invalid
   * true - invalid
   * false - valid
   *
   * @type {boolean}
   */
  thresholdMinError: Em.computed.bool('thresholdMinErrorMessage'),

  /**
   * Alias for <code>thresholdMinError</code>
   *
   * @type {boolean}
   */
  hasErrors: Em.computed.alias('thresholdMinError'),

  /**
   * Error message for <code>thresholdMin</code>
   * <ul>
   *  <li>Value is not a number</li>
   *  <li>Value is out of range <code>(minValue - maxValue)</code></li>
   * </ul>
   * Empty message means that <code>thresholdMin</code> has valid value
   *
   * @type {string}
   */
  thresholdMinErrorMessage: function () {
    var thresholdMin = this.get('thresholdMin');
    var maxValue = this.get('maxValue');
    var minValue = this.get('minValue');
    if (!isValidFloat(thresholdMin) || thresholdMin > maxValue || thresholdMin < minValue) {
      return Em.I18n.t('dashboard.widgets.error.invalid').format(minValue, maxValue);
    }
    return '';
  }.property('thresholdMin', 'maxValue'),

  /**
   * Formatted threshold value
   *
   * @type {number[]}
   */
  preparedThresholds: function () {
    return [parseFloat(this.get('thresholdMin'))];
  }.property('thresholdMin'),

  /**
   * Force set new values for threshold
   *
   * @param {number[]} newValues
   */
  updateThresholds: function updateThresholds(newValues) {
    this.set('thresholdMin', newValues[0]);
  }
});

/**
 * Thresholds manager for sliders with double values
 * Usage:
 * <pre>
 * SingleHandler.create({
 *    thresholdMin: 12,
 *    thresholdMax: 40,
 *    minValue: 10,
 *    maxValue: 100
 * });
 * </pre>
 *
 * @class DoubleHandlers
 */
var DoubleHandlers = SingleHandler.extend({

  /**
   * @type {number}
   */
  thresholdMax: null,

  /**
   * Is <code>thresholdMax</code> invalid
   * true - invalid
   * false - valid
   *
   * @type {boolean}
   */
  thresholdMaxError: Em.computed.bool('thresholdMaxErrorMessage'),

  /**
   * Is some threshold invalid
   * true - some one is invalid
   * false - thresholds are valid
   *
   * @type {boolean}
   */
  hasErrors: Em.computed.or('thresholdMinError', 'thresholdMaxError'),

  /**
   * Error message for <code>thresholdMin</code>
   * <ul>
   *  <li>Value is not a number</li>
   *  <li>Value is out of range <code>(minValue - maxValue)</code></li>
   *  <li><code>thresholdMin</code>-value greater than <code>thresholdMax</code>-value</li>
   * </ul>
   * Empty message means that <code>thresholdMin</code> has valid value
   *
   * @type {string}
   */
  thresholdMinErrorMessage: function () {
    var thresholdMin = Number(this.get('thresholdMin'));
    var thresholdMax = Number(this.get('thresholdMax'));
    var maxValue = Number(this.get('maxValue'));
    var minValue = Number(this.get('minValue'));
    if (!isValidFloat(thresholdMin) || thresholdMin > maxValue || thresholdMin < minValue) {
      return Em.I18n.t('dashboard.widgets.error.invalid').format(minValue, maxValue);
    }
    if (this.get('thresholdMaxError') === false && thresholdMax <= thresholdMin) {
      return Em.I18n.t('dashboard.widgets.error.smaller');
    }
    return '';
  }.property('thresholdMin', 'thresholdMax'),

  /**
   * Error message for <code>thresholdMax</code>
   * <ul>
   *  <li>Value is not a number</li>
   *  <li>Value is out of range <code>(minValue - maxValue)</code></li>
   * </ul>
   * Empty message means that <code>thresholdMax</code> has valid value
   *
   * @type {string}
   */
  thresholdMaxErrorMessage: function () {
    var thresholdMax = this.get('thresholdMax');
    var maxValue = this.get('maxValue');
    var minValue = this.get('minValue');
    if (!isValidFloat(thresholdMax) || thresholdMax > maxValue || thresholdMax < minValue) {
      return Em.I18n.t('dashboard.widgets.error.invalid').format(minValue, maxValue);
    }
    return '';
  }.property('thresholdMax'),

  /**
   * Threshold values ready to save
   *
   * @type {number[]}
   */
  preparedThresholds: function () {
    return [parseFloat(this.get('thresholdMin')), parseFloat(this.get('thresholdMax'))];
  }.property('thresholdMin', 'thresholdMax'),

  /**
   * Force set new values for threshold
   *
   * @param {number[]} newValues
   */
  updateThresholds: function updateThresholds(newValues) {
    this.set('thresholdMin', newValues[0]);
    this.set('thresholdMax', newValues[1]);
  }
});

/**
 * Common body-view for popup with sliders
 *
 * @class EditDashboardWidgetPopupBody
 */
var EditDashboardWidgetPopupBody = Em.View.extend({
  templateName: require('templates/main/dashboard/edit_widget_popup')
});

/**
 * Popup with slider to edit dashboard widget
 * Usage:
 * <pre>
 *   App.EditDashboardWidgetPopup.show({
 *    widgetView: this,
 *    sliderHandlersManager: App.EditDashboardWidgetPopup.DoubleHandlers.create({
 *      maxValue: 100,
 *      thresholdMin: this.get('thresholdMin'),
 *      thresholdMax: this.get('thresholdMax')
 *    })
 *  });
 * </pre>
 *
 * <code>widgetView</code> should be set to view with widget.
 * Usually you will use <code>App.EditDashboardWidgetPopup</code> inside of some <code>App.DashboardWidgetView</code> instance,
 * so <code>widgetView</code> may be set to <code>this</code>
 * <code>sliderHandlersManager</code> should be set to some of the <code>App.EditDashboardWidgetPopup.SingleHandler</code>
 * or <code>App.EditDashboardWidgetPopup.DoubleHandler</code>
 *
 * You can't use <code>App.EditDashboardWidgetPopup</code> without setting this two properties!
 *
 * @class App.EditDashboardWidgetPopup
 */
App.EditDashboardWidgetPopup = App.ModalPopup.extend({

  header: Em.I18n.t('dashboard.widgets.popupHeader'),
  classNames: ['modal-edit-widget'],
  modalDialogClasses: ['modal-lg'],
  primary: Em.I18n.t('common.apply'),
  disablePrimary: Em.computed.alias('sliderHandlersManager.hasErrors'),

  /**
   * Can't be null or undefined
   *
   * @type {SingleHandler|DoubleHandlers}
   */
  sliderHandlersManager: null,

  /**
   * Widget view
   * Can't be not a view. Used to save Thresholds. Normally it's an instance of <code>App.DashboardWidgetView</code>
   *
   * @type {Em.View}
   */
  widgetView: null,

  /**
   * Determines if slider is enabled for slide
   * true - don't enabled
   * false - enabled
   *
   * Determines if slider handlers should be updated with Threshold values
   * true - don't update
   * false - update
   *
   * Used as option <code>disabled</code> for $.ui.slider
   *
   * @type {boolean}
   */
  sliderDisabled: false,

  /**
   * Slider "ticks"
   * Used as option <code>values</code> for $.ui.slider
   *
   * @type {number[]}
   */
  sliderHandlers: function () {
    return this.get('sliderHandlersManager.preparedThresholds');
  }.property('sliderHandlersManager.preparedThresholds.[]'),

  /**
   * Colors used for slider ranges
   * @type {string[]}
   */
  sliderColors: [App.healthStatusRed, App.healthStatusOrange, App.healthStatusGreen],

  /**
   * Maximum value for slider
   * Used as option <code>max</code> for $.ui.slider
   *
   * @type {number}
   */
  sliderMaxValue: Em.computed.alias('sliderHandlersManager.maxValue'),

  /**
   * Minimum value for slider
   * Used as option <code>min</code> for $.ui.slider
   *
   * @type {number}
   */
  sliderMinValue: 0,

  /**
   * Check how many handlers has slider
   * true - 2 handlers
   * false - 1 handler
   *
   * Used as option <code>range</code> for $.ui.slider
   *
   * @type {boolean}
   */
  sliderIsRange: true,

  bodyClass: EditDashboardWidgetPopupBody,

  init: function init() {
    Em.assert('`widgetView` should be valid view', this.get('widgetView.isView'));
    Em.assert('`sliderHandlersManager` should be set', !!this.get('sliderHandlersManager'));
    return this._super.apply(this, arguments);
  },


  /**
   * Save new threshold value on popup-close (means Primary click)
   * Use <code>widgetView</code> to get <code>widgetsView</code> and save new values
   * Current widget is updated too (without redrawing)
   */
  saveThreshold: function saveThreshold() {
    var preparedThresholds = this.get('sliderHandlersManager.preparedThresholds');
    this.get('widgetView').saveWidgetThresholds(preparedThresholds);
  },


  /**
   * Update slider values when new threshold values are provided
   * Don't do anything if some value is invalid or slider is disabled
   *
   * @private
   */
  _updateSliderValues: function () {
    var sliderHandlersManager = this.get('sliderHandlersManager');
    if (!sliderHandlersManager.get('hasErrors') && !this.get('sliderDisabled')) {
      $('#slider-range').slider('values', sliderHandlersManager.get('preparedThresholds'));
    }
  }.observes('sliderDisabled', 'sliderHandlersManager.preparedThresholds.[]', 'sliderHandlersManager.hasErrors'),

  onPrimary: function onPrimary() {
    var sliderHandlersManager = this.get('sliderHandlersManager');
    if (!sliderHandlersManager.get('hasErrors')) {
      this.saveThreshold();
      this.hide();
    }
  },


  /**
   * Create slider in the popup when it's opened
   */
  createSlider: function createSlider() {
    var self = this;
    var sliderHandlersManager = this.get('sliderHandlersManager');
    var handlers = this.get('sliderHandlers');

    $('#slider-range').slider({
      range: this.get('sliderIsRange'),
      min: this.get('sliderMinValue'),
      max: this.get('sliderMaxValue'),
      disabled: this.get('sliderDisabled'),
      values: handlers,
      create: function create() {
        self.updateSliderColors(handlers);
      },
      slide: function slide(event, ui) {
        self.updateSliderColors(ui.values);
        sliderHandlersManager.updateThresholds(ui.values);
      },
      change: function change(event, ui) {
        self.updateSliderColors(ui.values);
      }
    });
  },


  didInsertElement: function didInsertElement() {
    this._super();
    this.createSlider();
  },

  /**
   * Update colors on slider using <code>sliderColors</code> theme when user interacts with it
   *
   * @param {number[]} handlers
   */
  updateSliderColors: function updateSliderColors(handlers) {
    var gradient = this._getGradientStr(handlers);
    $('#slider-range').css('background-image', '-webkit-' + gradient).css('background-image', '-ms-' + gradient).css('background-image', '-moz-' + gradient).find('.ui-widget-header').css({
      'background-color': App.healthStatusOrange,
      'background-image': 'none'
    });
  },


  /**
   * @param {number[]} handlers
   * @returns {string}
   * @private
   */
  _getGradientStr: function _getGradientStr(handlers) {
    var maxValue = this.get('sliderMaxValue');
    var colors = this.get('sliderColors');
    var gradient = colors[0] + ', ' + handlers.map(function (handler, i) {
      return colors[i] + '  ' + handlers[i] * 100 / maxValue + '%, ' + colors[i + 1] + ' ' + handlers[i] * 100 / maxValue + '%,';
    }).join('') + colors[colors.length - 1];
    return 'linear-gradient(left,' + gradient + ')';
  }
});

App.EditDashboardWidgetPopup.reopenClass({
  SingleHandler: SingleHandler,
  DoubleHandlers: DoubleHandlers,
  EditDashboardWidgetPopupBody: EditDashboardWidgetPopupBody
});

});

require.register("views/common/modal_popups/hosts_table_list_popup", function(exports, require, module) {
'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show popup with list of stack versions/components installed on host
 * @param header
 * @param hostName
 * @param items
 * @returns {App.ModalPopup}
 */
App.showHostsTableListPopup = function (header, hostName, items) {
  var isObjectsList = items.map(function (item) {
    return typeof item === 'undefined' ? 'undefined' : _typeof(item);
  }).contains('object');
  return App.ModalPopup.show({
    header: header,
    secondary: null,
    bodyClass: Em.View.extend({
      templateName: require('templates/common/modal_popups/hosts_table_list_popup'),
      hostName: hostName,
      items: items,
      isObjectsList: isObjectsList
    })
  });
};

});

require.register("views/common/modal_popups/invalid_KDC_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var credentialsUtils = require('utils/credentials');

/**
 * @param {Object} ajaxOpt - callbek funciton when clicking save
 * @param {Object} message - warning message
 * @return {*}
 */
App.showInvalidKDCPopup = function (ajaxOpt, message) {
  return App.ModalPopup.show({
    primary: Em.I18n.t('common.save'),
    header: Em.I18n.t('popup.invalid.KDC.header'),
    principal: "",
    password: "",

    /**
     * Store Admin credentials checkbox value
     *
     * @type {boolean}
     */
    storeCredentials: false,

    /**
     * Status of persistent storage. Returns <code>true</code> if persistent storage is available.
     * @type {boolean}
     */
    storePersisted: Em.computed.alias('App.isCredentialStorePersistent'),

    /**
     * Disable checkbox if persistent storage not available
     *
     * @type {boolean}
     */
    checkboxDisabled: Em.computed.not('storePersisted'),

    /**
     * Returns storage type used to save credentials e.g. <b>persistent</b>, <b>temporary</b> (default)
     *
     * @type {string}
     */
    storageType: Em.computed.ifThenElse('storeCredentials', credentialsUtils.STORE_TYPES.PERSISTENT, credentialsUtils.STORE_TYPES.TEMPORARY),

    /**
     * Message to display in tooltip regarding persistent storage state.
     *
     * @type {string}
     */
    hintMessage: Em.computed.ifThenElse('storePersisted', Em.I18n.t('admin.kerberos.credentials.store.hint.supported'), Em.I18n.t('admin.kerberos.credentials.store.hint.not.supported')),

    bodyClass: Em.View.extend({
      warningMsg: message + Em.I18n.t('popup.invalid.KDC.msg'),
      templateName: require('templates/common/modal_popups/invalid_KDC_popup')
    }),

    didInsertElement: function didInsertElement() {
      this._super();
      App.tooltip(this.$('[rel="tooltip"]'));
    },

    onClose: function onClose() {
      this.hide();
      if (ajaxOpt.kdcCancelHandler) {
        ajaxOpt.kdcCancelHandler();
      }
    },

    onSecondary: function onSecondary() {
      this.hide();
      if (ajaxOpt.kdcCancelHandler) {
        ajaxOpt.kdcCancelHandler();
      }
    },

    onPrimary: function onPrimary() {
      this.hide();
      var resource = credentialsUtils.createCredentialResource(this.get('principal'), this.get('password'), this.get('storageType'));
      App.get('router.clusterController').createKerberosAdminSession(resource, ajaxOpt);
    }
  });
};

});

require.register("views/common/modal_popups/log_file_search_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.LogFileSearchPopup = function (header) {
  return App.ModalPopup.show({
    classNames: ['common-modal-wrapper', 'log-file-search-popup'],
    modalDialogClasses: ['modal-lg'],
    header: header,
    bodyView: null,
    bodyClass: App.LogFileSearchView.extend({
      didInsertElement: function didInsertElement() {
        this.set('parentView.bodyView', this);
        this._super();
      }
    }),
    enterKeyPressed: function enterKeyPressed() {
      this.get('bodyView').fetchContent();
    }
  });
};

});

require.register("views/common/modal_popups/log_tail_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var fileUtils = require('utils/file_utils');

App.showLogTailPopup = function (content) {
  return App.ModalPopup.show({
    classNames: ['log-tail-popup', 'wide-modal-wrapper', 'full-height-modal'],
    modalDialogClasses: ['modal-xlg'],
    header: fileUtils.fileNameFromPath(content.get('filePath')),
    primary: false,
    secondary: Em.I18n.t('common.dismiss'),
    secondaryClass: 'btn-success',
    showFooter: true,
    autoHeight: false,
    bodyClass: Em.View.extend({
      templateName: require('templates/common/modal_popups/log_tail_popup'),
      content: content,
      selectedTailCount: 50,
      isCopyActive: false,
      copyContent: null,

      logSearchUrl: function () {
        var quickLink = App.QuickLinks.find().findProperty('site', 'logsearch-env'),
            logSearchServerHost = App.HostComponent.find().findProperty('componentName', 'LOGSEARCH_SERVER').get('hostName'),
            params = '';
        if (quickLink) {
          params = 'hosts=' + encodeURIComponent(this.get('content.hostName')) + ';components=' + encodeURIComponent(this.get('content.logComponentName')) + ';query=%5B%7B"id":0,"name":"path","label":"Path","value":"' + encodeURIComponent(this.get('content.filePath')) + '","isExclude":false%7D%5D';
          return quickLink.get('template').fmt('http', logSearchServerHost, quickLink.get('default_http_port')) + '/#/logs/serviceLogs;' + params;
        }
        return '#';
      }.property('content'),

      logTailViewInstance: null,

      /** actions **/
      openInNewTab: function openInNewTab() {
        var newWindow = window.open();
        var newDocument = newWindow.document;
        newDocument.write('<pre>' + this.logsToString() + '</pre>');
        newDocument.close();
      },

      toggleCopy: function toggleCopy() {
        if (!this.get('isCopyActive')) {
          this.initCopy();
        } else {
          this.destroyCopy();
        }
      },

      initCopy: function initCopy() {
        var self = this;
        this.set('copyContent', this.logsToString());
        this.set('isCopyActive', true);
        Em.run.next(function () {
          self.$().find('.copy-textarea').select();
        });
      },

      destroyCopy: function destroyCopy() {
        this.set('copyContent', null);
        this.set('isCopyActive', false);
      },

      logsToString: function logsToString() {
        return this.get('logTailViewInstance.logRows').map(function (i) {
          return i.get('logtimeFormatted') + ' ' + i.get('level') + ' ' + i.get('logMessage');
        }).join('\n');
      },

      logTailContentView: App.LogTailView.extend({
        contentBinding: "parentView.content",
        autoResize: true,
        selectedTailCountBinding: "parentView.selectedTailCount",

        didInsertElement: function didInsertElement() {
          this._super();
          this.set('parentView.logTailViewInstance', this);
        },

        resizeHandler: function resizeHandler() {
          if (this.get('state') === 'destroyed') return;
          this._super();
          var newSize = $(window).height() - this.get('resizeDelta') - window.innerHeight * 0.08;
          this.get('parentView').$().find('.copy-textarea').css({
            height: newSize + 'px'
          });
        },

        willDestroyElement: function willDestroyElement() {
          this._super();
          this.set('parentView.logTailViewInstance', null);
        }
      })
    })
  });
};

});

require.register("views/common/modal_popups/logs_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.showLogsPopup = function (header, content) {
  return App.ModalPopup.show({
    header: header,
    primary: Em.I18n.t('ok'),
    secondary: null,
    bodyClass: Em.View.extend({
      templateName: require('templates/common/modal_popups/logs_popup'),

      /**
       * @type {string}
       */
      content: content,

      /**
       * Onclick handler for selected Task
       *
       * @method openTaskLogInDialog
       */
      openTaskLogInDialog: function openTaskLogInDialog() {
        if ($(".task-detail-log-clipboard").length) {
          this.destroyClipBoard();
        }
        var newWindow = window.open();
        var newDocument = newWindow.document;
        newDocument.write($(".task-detail-log-info").html());
        newDocument.close();
      },

      /**
       * Onclick event for copy to clipboard button
       *
       * @method textTrigger
       */
      textTrigger: function textTrigger() {
        $(".task-detail-log-clipboard").length ? this.destroyClipBoard() : this.createClipBoard();
      },

      /**
       * Create Clip Board
       *
       * @method createClipBoard
       */
      createClipBoard: function createClipBoard() {
        var logElement = $(".task-detail-log-maintext"),
            logElementRect = logElement[0].getBoundingClientRect(),
            textarea = $('<textarea class="task-detail-log-clipboard"></textarea>');
        $(".task-detail-log-clipboard-wrap").append(textarea);
        textarea.text(logElement.text()).css('display', 'block').width(logElementRect.width).height(logElementRect.height).select();

        logElement.css("display", "none");
      },

      /**
       * Destroy Clip Board
       *
       * @method destroyClipBoard
       */
      destroyClipBoard: function destroyClipBoard() {
        $(".task-detail-log-clipboard").remove();
        $(".task-detail-log-maintext").css("display", "block");
      }
    })
  });
};

});

require.register("views/common/modal_popups/manage_kdc_credentials_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @return {*}
 */
App.showManageCredentialsPopup = function () {
  return App.ModalPopup.show({
    header: Em.I18n.t('admin.kerberos.credentials.store.menu.label'),
    bodyClass: App.ManageCredentialsFormView,
    primary: Em.I18n.t('common.save'),

    thirdClass: 'pull-left btn-danger',
    third: Em.computed.ifThenElse('formView.isRemovable', Em.I18n.t('common.remove'), null),

    isCredentialsRemoved: false,

    disablePrimary: Em.computed.alias('formView.isSubmitDisabled'),

    formView: Em.computed.findBy('childViews', 'viewName', 'manageCredentialsForm'),

    onPrimary: function onPrimary() {
      var self = this;
      var formView = this.get('formView');
      if (formView) {
        formView.saveKDCCredentials().always(function () {
          self.hide();
        });
      } else {
        this.hide();
      }
    },

    onThird: function onThird() {
      var self = this;
      if (this.get('formView')) {
        this.get('formView').removeKDCCredentials().deferred.always(function () {
          self.hide();
        });
      }
    }
  });
};

});

require.register("views/common/modal_popups/select_groups_popup", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Show confirmation popup
 * @param {string} selectedServiceName
 * @param {App.ConfigGroup} selectedConfigGroup
 * @param {App.ServiceConfig[]} dependentStepConfigs
 * @param {Object[]} configs
 * @return {App.ModalPopup}
 */
App.showSelectGroupsPopup = function (selectedServiceName, selectedConfigGroup, dependentStepConfigs, configs) {
  return App.ModalPopup.show({
    encodeBody: false,
    primary: Em.I18n.t('common.save'),
    header: Em.I18n.t('popup.dependent.configs.select.config.group.header'),
    selectedServiceName: selectedServiceName,
    selectedConfigGroup: selectedConfigGroup,
    dependentStepConfigs: dependentStepConfigs,
    selectedGroups: {},
    didInsertElement: function didInsertElement() {
      this._super();
      this.set('selectedGroups', $.extend({}, selectedConfigGroup.get('dependentConfigGroups')));
    },
    bodyClass: Em.CollectionView.extend({
      content: dependentStepConfigs,
      itemViewClass: Em.View.extend({
        templateName: require('templates/common/modal_popups/select_groups_popup'),
        didInsertElement: function didInsertElement() {
          this.set('selectedGroup', this.get('parentView.parentView.selectedConfigGroup.dependentConfigGroups')[this.get('serviceName')]);
        },
        hasGroups: Em.computed.bool('groups.length'),
        serviceName: Em.computed.alias('content.serviceName'),
        selectedGroup: null,
        updateGroup: function () {
          var dependentGroups = $.extend({}, this.get('parentView.parentView.selectedConfigGroup.dependentConfigGroups'));
          dependentGroups[this.get('serviceName')] = this.get('selectedGroup');
          this.set('parentView.parentView.selectedConfigGroup.dependentConfigGroups', dependentGroups);
        }.observes('selectedGroup'),
        groups: function () {
          return this.get('content').get('configGroups').filterProperty('isDefault', false).mapProperty('name');
        }.property('content')
      })
    }),
    onPrimary: function onPrimary() {
      this._super();
      Object.keys(this.get('selectedConfigGroup.dependentConfigGroups')).forEach(function (serviceName) {
        var selectedGroupName = this.get('selectedConfigGroup.dependentConfigGroups')[serviceName];
        var currentGroupName = this.get('selectedGroups')[serviceName] || "";
        var configGroup = this.get('dependentStepConfigs').findProperty('serviceName', serviceName).get('configGroups').findProperty('name', selectedGroupName);
        if (!configGroup) return; //There can be no dependent config group.
        if (selectedGroupName != currentGroupName) {
          /** changing config group for recommendations **/
          configs.filterProperty('serviceName', serviceName).filterProperty('configGroup', selectedGroupName).forEach(function (c) {
            if (configs.filterProperty('serviceName', serviceName).filterProperty('configGroup', currentGroupName)) {
              configs.removeObject(c);
            }
          });
          configs.filterProperty('serviceName', serviceName).filterProperty('configGroup', currentGroupName).setEach('configGroup', selectedGroupName);
          /** danger part!!!! changing config group ***/
          this.get('dependentStepConfigs').findProperty('serviceName', serviceName).get('configs').forEach(function (cp) {
            var dependentConfig = configs.filterProperty('propertyName', cp.get('name')).filterProperty('fileName', App.config.getConfigTagFromFileName(cp.get('filename'))).findProperty('configGroup', selectedGroupName);
            var recommendedValue = dependentConfig && Em.get(dependentConfig, 'recommendedValue');
            if (cp.get('overrides')) {
              var currentGroupOverride = cp.get('overrides').findProperty('group.name', currentGroupName);
              if (currentGroupOverride && currentGroupOverride.get('initialValue') != currentGroupOverride.get('recommendedValue')) {
                currentGroupOverride.set('group', configGroup);
                currentGroupOverride.set('recommendedValue', recommendedValue);
                currentGroupOverride.set('value', recommendedValue);
              } else {
                var selectedGroupOverride = cp.get('overrides').findProperty('group.name', configGroup.get('name'));
                if (selectedGroupOverride) {
                  selectedGroupOverride.set('recommendedValue', recommendedValue);
                  selectedGroupOverride.set('value', recommendedValue);
                } else {
                  App.config.createOverride(cp, { "value": recommendedValue, "recommendedValue": recommendedValue, "isEditable": true }, configGroup);
                }
              }
            } else {
              App.config.createOverride(cp, { "value": recommendedValue, "recommendedValue": recommendedValue, "isEditable": true }, configGroup);
            }
          }, this);
        }
      }, this);
    },
    onSecondary: function onSecondary() {
      this._super();
      this.get('selectedConfigGroup').set('dependentConfigGroups', this.get('selectedGroups'));
    }
  });
};

});

require.register("views/common/not-scrollable-textarea", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Same as Em.TextArea
 * Update self height (to avoid scrolling) when value is changed
 *
 * @type {Em.View}
 */
App.NotScrollableTextArea = Em.TextArea.extend({

  didInsertElement: function didInsertElement() {
    this.fitHeight();
    this.$().select();
  },

  fitHeight: function () {
    var self = this.$();
    if (self) {
      Em.run.next(function () {
        self.height(1);
        self.height(self[0].scrollHeight);
      });
    }
  }.observes('value')

});

});

require.register("views/common/pagination_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Set of typical properties for Select View with list of "rows-per-page" options
 * @type {Ember.View}
 */
App.PaginationSelectMixin = Em.Mixin.create({

  content: ['10', '25', '50', '100'],

  attributeBindings: ['disabled'],

  disabled: false,

  change: function change() {
    this.get('parentView.dataView').saveDisplayLength();
  }

});

App.PaginationView = Em.View.extend({

  templateName: require('templates/common/pagination'),

  classNames: ['pagination-block', 'pull-right'],

  /**
   * Link to view corresponding to paginated data
   */
  dataViewBinding: 'parentView',

  /**
   * Determines whether links to the first and last pages should be displayed
   * @type {Boolean}
   */
  hasFirstAndLastPageLinks: false,

  /**
   * Determines whether paginated data is loaded
   * Should be bound to property from dataView if necessary
   * @type {Boolean}
   */
  isDataLoaded: true,

  /**
   * Determines whether link to previous page should be disabled
   * @type {Boolean}
   */
  isPreviousDisabled: Em.computed.or('dataView.isCurrentPageFirst', '!isDataLoaded'),

  /**
   * Determines whether link to next page should be disabled
   * @type {Boolean}
   */
  isNextDisabled: Em.computed.or('dataView.isCurrentPageLast', '!isDataLoaded'),

  /**
   * Select View with list of "rows-per-page" options
   * @type {Ember.View}
   */
  rowsPerPageSelectView: Em.Select.extend(App.PaginationSelectMixin)

});

});

require.register("views/common/progress_bar_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ProgressBarView = Em.View.extend({
  template: Ember.Handlebars.compile('<div {{bindAttr class="barClass :progress-bar" style="view.progressWidth"}}></div>'),
  classNameBindings: ['generalClass'],

  /**
   * @type {number}
   * @default null
   */
  progress: null,

  /**
   * @type {string}
   * @default null
   */
  status: null,

  /**
   * @type {string}
   */
  generalClass: 'progress',

  /**
   * string format: width:<number>%;
   * @type {string}
   */
  progressWidth: Em.computed.format('width:{0}%;', 'progress'),

  /**
   * @type {string}
   */
  barClass: function () {
    switch (this.get('status')) {
      case 'FAILED':
        return 'progress-bar-danger';
      case 'ABORTED':
      case 'TIMED_OUT':
        return 'progress-bar-warning';
      case 'COMPLETED':
        return 'progress-bar-success';
      case 'SUSPENDED':
        return 'progress-bar-info';
      case 'QUEUED':
      case 'PENDING':
      case 'IN_PROGRESS':
        return 'progress-bar-info active progress-bar-striped';
      default:
        return 'progress-bar-info';
    }
  }.property('status')
});

});

require.register("views/common/quick_view_link_view", function(exports, require, module) {
'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @typedef {object} hostForQuickLink
 * @property {string} hostName
 * @property {string} publicHostName
 * @property {string} componentName
 * @property {?string} status
 */

App.QuickLinksView = Em.View.extend({

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * @type {boolean}
   */
  showQuickLinks: false,

  /**
   * @type {boolean}
   */
  showNoLinks: false,

  /**
   * @type {string}
   */
  quickLinksErrorMessage: '',

  /**
   * Updated quick links. Here we put correct hostname to url
   *
   * @type {object[]}
   */
  quickLinks: [],

  /**
   * @type {object[]}
   */
  configProperties: [],

  /**
   * list of files that contains properties for enabling/disabling ssl
   *
   * @type {string[]}
   */
  requiredSiteNames: [],

  /**
   * services that supports security. this array is used to find out protocol.
   * besides YARN, MAPREDUCE2, ACCUMULO. These services use
   * their properties to know protocol
   */
  servicesSupportsHttps: ["HDFS", "HBASE"],

  masterGroups: [],

  tooltipAttribute: 'quick-links-title-tooltip',

  shouldSetGroupedLinks: false,

  /**
   * @type {object}
   */
  ambariProperties: function () {
    return App.router.get('clusterController.ambariProperties');
  }.property().volatile(),

  didInsertElement: function didInsertElement() {
    this.loadQuickLinksConfigurations();
  },

  willDestroyElement: function willDestroyElement() {
    this.get('configProperties').clear();
    this.get('quickLinks').clear();
    this.get('requiredSiteNames').clear();
  },

  /**
   * The flags responsible for data to build quick links:
   * - App.router.clusterController.isServiceMetricsLoaded
   *
   * The flags responsible for correct, up-to-date state of quick links:
   * - App.currentStackVersionNumber
   * - App.router.clusterController.isHostComponentMetricsLoaded
   */
  setQuickLinks: function () {
    var _this = this;

    if (App.get('router.clusterController.isServiceMetricsLoaded')) {
      this.setConfigProperties().done(function (configProperties) {
        _this.get('configProperties').pushObjects(configProperties);
        _this.getQuickLinksHosts();
      });
    }
  }.observes('App.currentStackVersionNumber', 'App.router.clusterController.isServiceMetricsLoaded', 'App.router.clusterController.isHostComponentMetricsLoaded', 'App.router.clusterController.quickLinksUpdateCounter'),

  /**
  * Request for quick-links config
  *
  * @returns {$.ajax}
  * @method loadQuickLinksConfigurations
  */
  loadQuickLinksConfigurations: function loadQuickLinksConfigurations() {
    var serviceName = this.get('content.serviceName');
    return App.ajax.send({
      name: 'configs.quicklinksconfig',
      sender: this,
      data: {
        serviceName: serviceName,
        stackVersionUrl: App.get('stackVersionURL')
      },
      success: 'loadQuickLinksConfigSuccessCallback'
    });
  },

  /**
   * Sucess-callback for quick-links config request
   *
   * @param {object} data
   * @method loadQuickLinksConfigSuccessCallback
   */
  loadQuickLinksConfigSuccessCallback: function loadQuickLinksConfigSuccessCallback(data) {
    App.quicklinksMapper.map(data);
    var quickLinksConfig = this.getQuickLinksConfiguration();
    if (!Em.isNone(quickLinksConfig)) {
      var protocolConfig = Em.get(quickLinksConfig, 'protocol');
      var checks = Em.get(protocolConfig, 'checks');
      var sites = ['core-site', 'hdfs-site', 'admin-properties'];
      if (checks) {
        checks.forEach(function (check) {
          var protocolConfigSiteProp = Em.get(check, 'site');
          if (sites.indexOf(protocolConfigSiteProp) < 0) {
            sites.push(protocolConfigSiteProp);
          }
        }, this);
      }

      var links = Em.get(quickLinksConfig, 'links');
      if (!Em.isEmpty(links)) {
        links.forEach(function (link) {
          if (!link.remove) {
            this.addSite(link, 'host', sites);
            this.addSite(link, 'port', sites);
          }
        }, this);
        this.set('requiredSiteNames', this.get('requiredSiteNames').pushObjects(sites).uniq());
        this.setQuickLinks();
      }
    } else {
      this.set('showNoLinks', true);
    }
  },

  addSite: function addSite(link, linkPropertyName, sites) {
    var config = Em.get(link, linkPropertyName);
    if (config) {
      var siteName = Em.get(config, 'site');
      if (!sites.contains(siteName)) {
        sites.push(siteName);
      }
    }
  },

  /**
   * call for public host names
   *
   * @returns {$.ajax}
   * @method getQuickLinksHosts
   */
  getQuickLinksHosts: function getQuickLinksHosts() {
    var links = App.QuickLinksConfig.find().mapProperty('links'),
        components = links.reduce(function (componentsArray, currentLinksArray) {
      var componentNames = currentLinksArray.mapProperty('component_name').uniq();
      return [].concat(_toConsumableArray(componentsArray), _toConsumableArray(componentNames));
    }, []),
        hosts = App.HostComponent.find().filter(function (component) {
      return components.contains(component.get('componentName'));
    }).mapProperty('hostName').uniq();

    if (hosts.length === 0) {
      return $.Deferred().reject().promise();
    }

    return App.ajax.send({
      name: 'hosts.for_quick_links',
      sender: this,
      data: {
        clusterName: App.get('clusterName'),
        hosts: hosts.join(','),
        urlParams: this.get('content.serviceName') === 'HBASE' ? ',host_components/metrics/hbase/master/IsActiveMaster' : ''
      },
      success: 'setQuickLinksSuccessCallback'
    });
  },

  /**
   * Success-callback for quick-links hosts request
   *
   * @param {object} response
   * @method setQuickLinksSuccessCallback
   */
  setQuickLinksSuccessCallback: function setQuickLinksSuccessCallback(response) {
    var serviceName = this.get('content.serviceName');
    var hosts = this.getHosts(response, serviceName);
    var hasQuickLinks = this.hasQuickLinksConfig(serviceName, hosts);
    var hasHosts = false;
    var componentNames = hosts.mapProperty('componentName');
    componentNames.forEach(function (_componentName) {
      var component = App.MasterComponent.find().findProperty('componentName', _componentName) || App.SlaveComponent.find().findProperty('componentName', _componentName);
      if (component) {
        hasHosts = hasHosts || !!component.get('totalCount');
      }
    });
    // no need to set quicklinks if
    // 1)current service does not have quick links configured
    // 2)No host component present for the configured quicklinks and has no overridden hosts
    if (hasQuickLinks && (hasHosts || this.hasOverriddenHost())) {
      this.set('showQuickLinks', true);
    } else {
      this.set('showNoLinks', true);
    }

    var isMultipleComponentsInLinks = componentNames.uniq().length > 1;

    if (hosts.length === 0 && !this.hasOverriddenHost()) {
      this.setEmptyLinks();
    } else if (hosts.length === 1 || isMultipleComponentsInLinks || this.hasOverriddenHost()) {
      this.setSingleHostLinks(hosts, response);
    } else if (this.get('masterGroups.length') > 1 || this.get('shouldSetGroupedLinks')) {
      this.setMultipleGroupLinks(hosts);
    } else {
      this.setMultipleHostLinks(hosts);
    }
  },

  hasOverriddenHost: function hasOverriddenHost() {
    var links = Em.get(this.getQuickLinksConfiguration(), 'links');
    return links && links.some(function (each) {
      return each.host;
    });
  },

  /**
   * Get public host name by its host name.
   *
   * @method getPublicHostName
   * @param {Object[]} hosts - list of hosts from response
   * @param {string} hostName
   * @return {?string}
   **/
  getPublicHostName: function getPublicHostName(hosts, hostName) {
    var host = hosts.findProperty('Hosts.host_name', hostName);
    return host ? Em.get(host, 'Hosts.public_host_name') : null;
  },

  /**
   * Get configs from `configurationController` for provided list of the tags
   *
   * @returns {$.Deferred}
   * @method setConfigProperties
   */
  setConfigProperties: function setConfigProperties() {
    this.get('configProperties').clear();
    return App.router.get('configurationController').getCurrentConfigsBySites(this.get('requiredSiteNames'));
  },

  /**
   * Get quick links config for <code>content.serviceName</code>
   *
   * @returns {?App.QuickLinksConfig}
   * @method getQuickLinksConfiguration
   */
  getQuickLinksConfiguration: function getQuickLinksConfiguration() {
    var serviceName = this.get('content.serviceName');
    var self = this;
    var quicklinks = {};
    if (self.hasQuickLinksConfig(serviceName)) {
      quicklinks = App.QuickLinksConfig.find().findProperty('id', serviceName);
      Em.set(quicklinks, 'links', Em.get(quicklinks, 'links').filterProperty('visible', true));
      return quicklinks;
    }
    return null;
  },

  /**
   * Check if <code>serviceName</code> has quick-links config
   *
   * @param {string} serviceName
   * @returns {boolean}
   * @method hasQuickLinksConfig
   */
  hasQuickLinksConfig: function hasQuickLinksConfig(serviceName) {
    var result = App.QuickLinksConfig.find().findProperty('id', serviceName);
    if (!result) {
      return false;
    }
    var links = result.get('links');
    return Em.isEmpty(links) ? false : links.length !== links.filterProperty('remove').length;
  },

  /**
   *
   * @param {object} link
   * @param {string} host
   * @param {string} protocol
   * @param {object[]} configProperties
   * @param {object} response
   * @returns {?object}
   * @method getHostLink
   */
  getHostLink: function getHostLink(link, host, protocol, configProperties, response) {
    var serviceName = this.get('content.serviceName');
    if (serviceName === 'MAPREDUCE2' && response) {
      var portConfig = Em.get(link, 'port');
      var siteName = Em.get(portConfig, 'site');
      var siteConfigs = this.get('configProperties').findProperty('type', siteName).properties;
      var hostPortConfigValue = siteConfigs[Em.get(portConfig, protocol + '_config')];
      if (!Em.isNone(hostPortConfigValue)) {
        var hostPortValue = hostPortConfigValue.match(new RegExp('([\\w\\d.-]*):(\\d+)'));
        var hostObj = response.items.findProperty('Hosts.host_name', hostPortValue[1]);
        if (!Em.isNone(hostObj)) {
          host = hostObj.Hosts.public_host_name;
        }
      }
    } else if (serviceName === 'RANGER') {
      var _siteConfigs = this.get('configProperties').findProperty('type', 'admin-properties').properties;
      if (_siteConfigs['policymgr_external_url']) {
        return {
          label: link.label,
          url: _siteConfigs['policymgr_external_url']
        };
      }
    }

    var linkPort = this.setPort(Em.get(link, 'port'), protocol, configProperties);
    if (Em.get(link, 'url') && !Em.get(link, 'removed')) {
      var newItem = {};
      var requiresUserName = Em.get(link, 'requires_user_name');
      var template = Em.get(link, 'url');
      if ('true' === requiresUserName) {
        newItem.url = template.fmt(protocol, host, linkPort, App.router.get('loginName'));
      } else {
        newItem.url = template.fmt(protocol, host, linkPort);
      }
      newItem.label = link.label;
      newItem.url = this.resolvePlaceholders(newItem.url);
      return newItem;
    }
    return null;
  },

  /**
   * Replace placeholders like ${config-type/property-name} in the given URL
   */
  resolvePlaceholders: function resolvePlaceholders(url) {
    return url.replace(/\$\{(\S+)\/(\S+)\}/g, function (match, configType, propertyName) {
      var config = this.get('configProperties').findProperty('type', configType);
      if (config) {
        return config.properties[propertyName] ? config.properties[propertyName] : match;
      } else {
        return match;
      }
    }.bind(this));
  },

  /**
   * set empty links
   *
   * @method setEmptyLinks
   */
  setEmptyLinks: function setEmptyLinks() {
    //display an error message
    var quickLinks = [{
      label: this.get('quickLinksErrorMessage')
    }];
    this.set('quickLinks', quickLinks);
    this.set('isLoaded', true);
  },

  /**
   * set links that contain only one host
   *
   * @param {hostForQuickLink[]} hosts
   * @param {object} response
   * @method setSingleHostLinks
   */
  setSingleHostLinks: function setSingleHostLinks(hosts, response) {
    var quickLinksConfig = this.getQuickLinksConfiguration();
    if (!Em.isNone(quickLinksConfig)) {
      var quickLinks = [];
      var configProperties = this.get('configProperties');
      var protocol = this.setProtocol(configProperties, quickLinksConfig.get('protocol'));

      var links = Em.get(quickLinksConfig, 'links');
      links.forEach(function (link) {
        var isRelatedComponentInstalled = this.isRelatedComponentInstalled(link);
        if (isRelatedComponentInstalled) {
          var publicHostName = this.publicHostName(link, hosts, protocol);
          if (publicHostName) {
            if (link.protocol) {
              protocol = this.setProtocol(configProperties, link.protocol);
            }
            var newItem = this.getHostLink(link, publicHostName, protocol, configProperties, response); //quicklink generated for the hbs template
            if (!Em.isNone(newItem)) {
              quickLinks.push(newItem);
            }
          }
        }
      }, this);
      this.set('quickLinks', quickLinks);
      this.set('isLoaded', true);
    } else {
      this.set('quickLinks', []);
      this.set('isLoaded', false);
    }
  },

  publicHostName: function publicHostName(link, hosts, protocol) {
    if (link.host) {
      // if quicklink overrides hostcomponent host name, get host from config
      var configProperties = this.get('configProperties');
      var hostProperty = Em.get(link.host, protocol + '_property');
      var site = configProperties.findProperty('type', Em.get(link.host, 'site'));
      return site && site.properties ? this.parseHostFromUri(site.properties[hostProperty]) : null;
    } else {
      var hostNameForComponent = hosts.findProperty('componentName', link.component_name);
      return hostNameForComponent ? hostNameForComponent.publicHostName : null;
    }
  },

  /**
   * @param {string} uri
   */
  parseHostFromUri: function parseHostFromUri(uri) {
    if (uri) {
      var match = uri.match(/:\/\/([^/:]+)/i);
      return match != null && match.length == 2 ? match[1] : uri;
    } else {
      return null;
    }
  },

  getMultipleHostLinks: function getMultipleHostLinks(hosts) {
    var quickLinksConfig = this.getQuickLinksConfiguration();
    if (Em.isNone(quickLinksConfig)) {
      return;
    }

    var quickLinksArray = [];
    hosts.forEach(function (host) {
      var publicHostName = host.publicHostName;
      var quickLinks = [];
      var configProperties = this.get('configProperties');

      var protocol = this.setProtocol(configProperties, quickLinksConfig.get('protocol'));
      var serviceName = Em.get(quickLinksConfig, 'serviceName');
      var links = Em.get(quickLinksConfig, 'links');
      links.forEach(function (link) {
        var linkRemoved = Em.get(link, 'removed');
        var url = Em.get(link, 'url');
        var isRelatedComponentInstalled = this.isRelatedComponentInstalled(link);
        if (url && !linkRemoved && isRelatedComponentInstalled) {
          var hostNameRegExp = new RegExp('([\\w\\W]*):\\d+');
          if (serviceName === 'HDFS') {
            var config;
            var configPropertiesObject = configProperties.findProperty('type', 'hdfs-site');
            if (configPropertiesObject && configPropertiesObject.properties) {
              var properties = configPropertiesObject.properties;
              var nnKeyRegex = new RegExp('^dfs\.namenode\.' + protocol + '-address\.');
              var nnProperties = Object.keys(properties).filter(function (key) {
                return nnKeyRegex.test(key);
              });
              var nnPropertiesLength = nnProperties.length;
              for (var i = nnPropertiesLength; i--;) {
                var propertyName = nnProperties[i];
                var hostNameMatch = properties[propertyName] && properties[propertyName].match(hostNameRegExp);
                if (hostNameMatch && hostNameMatch[1] === host.publicHostName) {
                  config = propertyName;
                  break;
                }
              }
            }
            var portConfig = Em.get(link, 'port');
            Em.set(portConfig, protocol + '_property', config);
            Em.set(link, 'port', portConfig);
          }

          var newItem = this.getHostLink(link, publicHostName, protocol, configProperties); //quicklink generated for the hbs template
          if (!Em.isNone(newItem)) {
            quickLinks.push(newItem);
          }
        }
      }, this);

      if (host.status) {
        quickLinks.set('publicHostNameLabel', Em.I18n.t('quick.links.publicHostName').format(host.publicHostName, host.status));
      } else {
        quickLinks.set('publicHostNameLabel', host.publicHostName);
      }
      quickLinksArray.push(quickLinks);
    }, this);
    return quickLinksArray;
  },

  /**
   * set links that contain multiple hosts
   *
   * @param {hostForQuickLink[]} hosts
   * @method setMultipleHostLinks
   */
  setMultipleHostLinks: function setMultipleHostLinks(hosts) {
    var quickLinks = this.getMultipleHostLinks(hosts);
    this.setProperties({
      quickLinksArray: [{
        links: quickLinks || []
      }],
      isLoaded: !!quickLinks
    });
  },

  /**
   * set links that contain for multiple grouped hosts
   *
   * @param {hostForQuickLink[]} hosts
   * @method setMultipleGroupLinks
   */
  setMultipleGroupLinks: function setMultipleGroupLinks(hosts) {
    var _this2 = this;

    var isLoaded = true;
    var quickLinksArray = this.get('masterGroups').map(function (group) {
      var groupHosts = hosts.filter(function (host) {
        return group.hosts.contains(host.hostName);
      }),
          links = _this2.getMultipleHostLinks(groupHosts);
      if (!links) {
        isLoaded = false;
      }
      return {
        title: group.title,
        links: links || []
      };
    });
    this.setProperties({
      quickLinksArray: quickLinksArray,
      isLoaded: isLoaded
    });
  },

  /**
   * set status to hosts with OOZIE_SERVER
   *
   * @param {object[]} hosts
   * @returns {object[]}
   * @method processOozieHosts
   */
  processOozieHosts: function processOozieHosts(hosts) {
    var activeOozieServers = this.get('content.hostComponents').filterProperty('componentName', 'OOZIE_SERVER').filterProperty('workStatus', 'STARTED').mapProperty('hostName');

    var oozieHostsArray = hosts.filter(function (host) {
      host.status = Em.I18n.t('quick.links.label.active');
      return activeOozieServers.contains(host.hostName);
    }, this);

    if (!oozieHostsArray.length) {
      this.set('quickLinksErrorMessage', Em.I18n.t('quick.links.error.oozie.label'));
    }
    return oozieHostsArray;
  },

  /**
   * set status to hosts with NAMENODE
   *
   * @param {hostForQuickLink[]} hosts
   * @returns {object[]}
   * @method hostForQuickLink
   */
  processHdfsHosts: function processHdfsHosts(hosts) {
    var _this3 = this;

    return hosts.map(function (host) {
      var hostName = host.hostName,
          isActiveNameNode = Em.get(_this3, 'content.activeNameNodes').someProperty('hostName', hostName),
          isStandbyNameNode = Em.get(_this3, 'content.standbyNameNodes').someProperty('hostName', hostName);

      if (isActiveNameNode) {
        host.status = Em.I18n.t('quick.links.label.active');
      }
      if (isStandbyNameNode) {
        host.status = Em.I18n.t('quick.links.label.standby');
      }
      return host;
    });
  },

  /**
   * set status to hosts with HBASE_MASTER
   *
   * @param {hostForQuickLink[]} hosts
   * @param {object} response
   * @returns {hostForQuickLink[]}
   * @method processHbaseHosts
   */
  processHbaseHosts: function processHbaseHosts(hosts, response) {
    return hosts.map(function (host) {
      var isActiveMaster;
      response.items.filterProperty('Hosts.host_name', host.hostName).filter(function (item) {
        var hbaseMaster = item.host_components.findProperty('HostRoles.component_name', 'HBASE_MASTER');
        isActiveMaster = hbaseMaster && Em.get(hbaseMaster, 'metrics.hbase.master.IsActiveMaster');
      });
      if (isActiveMaster === 'true') {
        host.status = Em.I18n.t('quick.links.label.active');
      } else if (isActiveMaster === 'false') {
        host.status = Em.I18n.t('quick.links.label.standby');
      }
      return host;
    }, this);
  },

  /**
   * set status to hosts with RESOURCEMANAGER
   *
   * @param {hostForQuickLink[]} hosts
   * @returns {hostForQuickLink[]}
   * @method processYarnHosts
   */
  processYarnHosts: function processYarnHosts(hosts) {
    return hosts.map(function (host) {
      var resourceManager = this.get('content.hostComponents').filterProperty('componentName', 'RESOURCEMANAGER').findProperty('hostName', host.hostName);
      var haStatus = resourceManager && resourceManager.get('haStatus');
      if (haStatus === 'ACTIVE') {
        host.status = Em.I18n.t('quick.links.label.active');
      } else if (haStatus === 'STANDBY') {
        host.status = Em.I18n.t('quick.links.label.standby');
      }
      return host;
    }, this);
  },

  /**
   * sets public host names for required masters of current service
   *
   * @param {object} response
   * @param {string} serviceName - selected serviceName
   * @returns {hostForQuickLink[]} containing hostName(s)
   * @method getHosts
   */
  getHosts: function getHosts(response, serviceName) {
    //The default error message when we cannot obtain the host information for the given service
    this.set('quickLinksErrorMessage', Em.I18n.t('quick.links.error.nohosts.label').format(serviceName));
    var hosts = [];
    var quickLinkConfigs = App.QuickLinksConfig.find().findProperty("id", serviceName);
    if (quickLinkConfigs) {
      var links = quickLinkConfigs.get('links');
      var componentNames = links.mapProperty('component_name').uniq();
      componentNames.forEach(function (_componentName) {
        var componentHosts = this.findHosts(_componentName, response);
        switch (serviceName) {
          case 'OOZIE':
            hosts = hosts.concat(this.processOozieHosts(componentHosts));
            break;
          case "HDFS":
            hosts = hosts.concat(this.processHdfsHosts(componentHosts));
            break;
          case "HBASE":
            hosts = hosts.concat(this.processHbaseHosts(componentHosts, response));
            break;
          case "YARN":
            hosts = hosts.concat(this.processYarnHosts(componentHosts));
            break;
          case "SMARTSENSE":
            if (!App.MasterComponent.find().findProperty('componentName', _componentName)) {
              hosts = [];
              break;
            }
          default:
            hosts = hosts.concat(componentHosts);
            break;
        }
      }, this);
    }
    return hosts;
  },

  /**
   * find host public names
   *
   * @param {string} componentName
   * @param {object} response
   * @returns {hostForQuickLink[]}
   */
  findHosts: function findHosts(componentName, response) {
    var hosts = [];
    var component = App.MasterComponent.find().findProperty('componentName', componentName) || App.SlaveComponent.find().findProperty('componentName', componentName);
    if (component) {
      var hostComponents = component.get('hostNames') || [];
      hostComponents.forEach(function (_hostName) {
        var host = this.getPublicHostName(response.items, _hostName);
        if (host) {
          hosts.push({
            hostName: _hostName,
            publicHostName: host,
            componentName: componentName
          });
        }
      }, this);
    }
    return hosts;
  },

  /**
   * 'http' for 'https'
   * 'https' for 'https'
   * empty string otherwise
   *
   * @param {string} type
   * @returns {string}
   * @method reverseType
   */
  reverseType: function reverseType(type) {
    if ('https' === type) {
      return 'http';
    }
    if ('http' === type) {
      return 'https';
    }
    return '';
  },

  /**
   *
   * @param {object[]} configProperties
   * @param {string} configType
   * @param {string} property
   * @param {string} desiredState
   * @returns {boolean}
   * @method meetDesired
   */
  meetDesired: function meetDesired(configProperties, configType, property, desiredState) {
    var currentConfig = configProperties.findProperty('type', configType);
    if (!currentConfig) {
      return false;
    }
    var currentPropertyValue = currentConfig.properties[property];
    if ('NOT_EXIST' === desiredState) {
      return Em.isNone(currentPropertyValue);
    }
    if ('EXIST' === desiredState) {
      return !Em.isNone(currentPropertyValue);
    }
    return desiredState === currentPropertyValue;
  },

  /**
   * setProtocol - if cluster is secure for some services (GANGLIA, MAPREDUCE2, YARN and servicesSupportsHttps)
   * protocol becomes "https" otherwise "http" (by default)
   *
   * @param {Object} configProperties
   * @param {Object} protocolConfig
   * @returns {string} "https" or "http" only!
   * @method setProtocol
   */
  setProtocol: function setProtocol(configProperties, protocolConfig) {
    var hadoopSslEnabled = false;

    if (!Em.isEmpty(configProperties)) {
      var hdfsSite = configProperties.findProperty('type', 'hdfs-site');
      hadoopSslEnabled = hdfsSite && Em.get(hdfsSite, 'properties') && hdfsSite.properties['dfs.http.policy'] === 'HTTPS_ONLY';
    }

    if (!protocolConfig) {
      return hadoopSslEnabled ? 'https' : 'http';
    }

    var protocolType = Em.get(protocolConfig, 'type');

    if ('HTTPS_ONLY' === protocolType) {
      return 'https';
    }
    if ('HTTP_ONLY' === protocolType) {
      return 'http';
    }

    protocolType = protocolType.toLowerCase();

    var count = 0;
    var checks = Em.get(protocolConfig, 'checks');
    if (!checks) {
      return hadoopSslEnabled ? 'https' : 'http';
    }
    checks.forEach(function (check) {
      var configType = Em.get(check, 'site');
      var property = Em.get(check, 'property');
      var desiredState = Em.get(check, 'desired');
      var checkMeet = this.meetDesired(configProperties, configType, property, desiredState);
      if (!checkMeet) {
        count++;
      }
    }, this);

    return count ? this.reverseType(protocolType) : protocolType;
  },

  /**
   * sets the port of quick link
   *
   * @param {object} portConfigs
   * @param {string} protocol
   * @param {object[]} configProperties
   * @returns {string}
   * @method setPort
   */
  setPort: function setPort(portConfigs, protocol, configProperties) {
    if (!portConfigs) {
      return '';
    }
    var defaultPort = Em.get(portConfigs, protocol + '_default_port');
    var portProperty = Em.get(portConfigs, protocol + '_property');
    var site = configProperties.findProperty('type', Em.get(portConfigs, 'site'));
    var propertyValue = site && site.properties && site.properties[portProperty];

    if (!propertyValue) {
      return defaultPort;
    }

    var regexValue = Em.get(portConfigs, 'regex');
    if (protocol === 'https') {
      var httpsRegex = Em.get(portConfigs, 'https_regex');
      if (httpsRegex) {
        regexValue = httpsRegex;
      }
    }
    regexValue = regexValue.trim();
    if (regexValue) {
      var re = new RegExp(regexValue);
      var portValue = propertyValue.match(re);
      try {
        return portValue[1];
      } catch (err) {
        return defaultPort;
      }
    } else {
      return propertyValue;
    }
  },

  setTooltip: function () {
    var _this4 = this;

    Em.run.next(function () {
      if (_this4.get('showQuickLinks') && _this4.get('isLoaded') && _this4.get('quickLinksArray.length')) {
        App.tooltip($('[rel="' + _this4.get('tooltipAttribute') + '"]'));
      }
    });
  }.observes('showQuickLinks', 'isLoaded', 'quickLinksArray.length'),

  /**
   * Check if host component related to quickLink is installed
   * @param quickLink
   * @returns {Boolean}
   */
  isRelatedComponentInstalled: function isRelatedComponentInstalled(quickLink) {
    return App.HostComponent.find().someProperty('componentName', quickLink.component_name);
  }
});

});

require.register("views/common/radio_button_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Create custom styled radio button. Extends from App.CheckboxView with few additional properties:
 * - selection
 * - value
 *
 * @extends App.CheckboxView
 * @see App.CheckboxView
 * @type {Ember.View}
 */
App.RadioButtonView = App.CheckboxView.extend({

  /**
   * Selected value
   * @type {*}
   */
  selection: null,

  classNames: ['radio'],

  /**
   * <code>value</code> html attribute of radio button
   * @type {[type]}
   */
  value: null,

  checked: Em.computed.equalProperties('selection', 'value'),

  checkboxView: Ember.RadioButton.extend({
    selectionBinding: 'parentView.selection',
    valueBinding: 'parentView.value',
    disabledBinding: 'parentView.disabled',
    checked: Em.computed.alias('parentView.checked'),
    nameBinding: 'parentView.name',
    didInsertElement: function didInsertElement() {
      this.set('parentView.checkboxId', this.get('elementId'));
      this._super();
    },
    change: function change() {
      if (typeof this.get('parentView.change') === 'function') {
        this.get('parentView.change').apply(this.get('parentView'), Array.prototype.slice.call(arguments));
      }
    }
  })
});

});

require.register("views/common/rolling_restart_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var numberUtils = require('utils/number_utils');

/**
 * View content of the rolling restart dialog.
 *
 * Callers provide the context in which this dialog is invoked.
 */
App.RollingRestartView = Em.View.extend({
  templateName: require('templates/common/rolling_restart_view'),

  /**
   * Component name for components that should be restarted
   * @type {String}
   */
  hostComponentName: null,

  /**
   * Service name for components that should be restarted
   * @type {String}
   */
  serviceName: null,

  /**
   * If service is in Maintenance Mode
   * @type {bool}
   */
  isServiceInMM: false,

  /**
   * If true service will be put in Maintenance mode before rolling restart
   * @type {bool}
   */
  turnOnMm: false,

  /**
   * Restart only components with <code>staleConfigs</code>
   * @type {bool}
   */
  staleConfigsOnly: false,

  /**
   * We should do rolling restart for components if we run
   * restart for service and service is in Maintenance mode
   * @type {bool}
   */
  skipMaintenance: false,

  /**
   * Count of host components in one batch
   * @type {Number}
   */
  batchSize: -1,

  /**
   * Delay between batches
   * @type {Number}
   */
  interBatchWaitTimeSeconds: -1,

  /**
   * @type {Number}
   */
  tolerateSize: -1,

  /**
   * Shows if all needed info is loaded and initialized
   * @type {Bool}
   */
  isLoaded: false,
  /**
   * List of error in batch-request properties
   * @type {Array}
   */
  errors: [],
  /**
   * List of warnings in batch-request properties, do not disable submit button
   * @type {Array}
   */
  warnings: [],
  /**
   * Set initial values for batch-request properties
   */
  initialize: function initialize() {
    if (this.get('batchSize') == -1 && this.get('interBatchWaitTimeSeconds') == -1 && this.get('tolerateSize') == -1) {
      var restartCount = this.get('restartHostComponents.length');
      var batchSize = 1;
      if (restartCount > 10 && this.get('hostComponentName') !== 'DATANODE') {
        batchSize = Math.ceil(restartCount / 10);
      }
      var tolerateCount = batchSize;
      this.set('batchSize', batchSize);
      this.set('tolerateSize', tolerateCount);
      this.set('interBatchWaitTimeSeconds', 120);
      this.set('isLoaded', true);
    }
  },

  /**
   * Validate batch-request properties
   * List of errors is saved to <code>errors</code>
   */
  validate: function () {
    var displayName = pluralize(this.get('hostComponentDisplayName'));
    var componentName = this.get('hostComponentName');
    var totalCount = this.get('restartHostComponents.length');
    var bs = this.get('batchSize');
    var ts = this.get('tolerateSize');
    var wait = this.get('interBatchWaitTimeSeconds');
    var errors = [];
    var warnings = [];
    var bsError, tsError, waitError;
    if (totalCount < 1) {
      errors.push(Em.I18n.t('rollingrestart.dialog.msg.noRestartHosts').format(displayName));
    } else {
      if (componentName === 'DATANODE') {
        // specific case for DataNodes batch size is more than 1
        if (bs > 1) {
          warnings.push(Em.I18n.t('rollingrestart.dialog.warn.datanode.batch.size'));
        }
        bsError = numberUtils.validateInteger(bs, 1, NaN);
      } else {
        bsError = numberUtils.validateInteger(bs, 1, totalCount);
      }
      tsError = numberUtils.validateInteger(ts, 0, totalCount);
      if (bsError != null) {
        errors.push(Em.I18n.t('rollingrestart.dialog.err.invalid.batchsize').format(bsError));
      }
      if (tsError != null) {
        errors.push(Em.I18n.t('rollingrestart.dialog.err.invalid.toleratesize').format(tsError));
      }
    }
    waitError = numberUtils.validateInteger(wait, 0, NaN);
    if (waitError != null) {
      errors.push(Em.I18n.t('rollingrestart.dialog.err.invalid.waitTime').format(waitError));
    }
    this.set('errors', errors);
    this.set('warnings', warnings);
  }.observes('batchSize', 'interBatchWaitTimeSeconds', 'tolerateSize', 'restartHostComponents', 'hostComponentDisplayName'),

  /**
   * Formatted <code>hostComponentName</code>
   * @type {String}
   */
  hostComponentDisplayName: Em.computed.formatRole('hostComponentName', false),

  /**
   * List of all host components
   * @type {Array}
   */
  allHostComponents: [],

  /**
   * List of host components without components in out-of-service state
   * @type {Array}
   */
  nonMaintainanceHostComponents: Em.computed.filterBy('allHostComponents', 'passiveState', 'OFF'),

  /**
   * List of host components with host in Maintenance mode
   * @type {Array}
   */
  componentsWithMaintenanceHost: Em.computed.filterBy('allHostComponents', 'hostPassiveState', 'ON'),

  /**
   * List of host components without components in out-of-service state
   * If <code>staleConfigsOnly</code> is true, components with <code>staleConfigs</code> = false are also filtered
   * @type {Array}
   */
  restartHostComponents: function () {
    var hostComponents = this.get('skipMaintenance') ? this.get('allHostComponents') : this.get('nonMaintainanceHostComponents');

    if (this.get('staleConfigsOnly')) {
      hostComponents = hostComponents.filterProperty('staleConfigs');
    }
    return hostComponents;
  }.property('nonMaintainanceHostComponents', 'staleConfigsOnly'),
  /**
   * @type {String}
   */
  suggestTurnOnMaintenanceMsg: function () {
    if (!this.get('isServiceInMM')) {
      return Em.I18n.t('rollingrestart.dialog.msg.serviceNotInMM').format(this.get('serviceName'));
    } else {
      return null;
    }
  }.property('isServiceInMM'),

  /**
   * @type {String}
   */
  restartMessage: function () {
    return Em.I18n.t('rollingrestart.dialog.msg.restart').format(pluralize(this.get('hostComponentDisplayName')));
  }.property('hostComponentDisplayName'),

  /**
   * @type {String}
   */
  maintainanceMessage: function () {
    var count = this.get('componentsWithMaintenanceHost.length');
    if (count > 0) {
      var name = this.get('hostComponentDisplayName');
      return Em.I18n.t('rollingrestart.dialog.msg.maintainance').format(count, pluralize(name));
    }
    return null;
  }.property('componentsWithMaintenanceHost', 'hostComponentDisplayName'),

  /**
   * @type {String}
   */
  batchSizeMessage: function () {
    return Em.I18n.t('rollingrestart.dialog.msg.componentsAtATime').format(pluralize(this.get('hostComponentDisplayName')));
  }.property('hostComponentDisplayName'),

  /**
   * @type {String}
   */
  staleConfigsOnlyMessage: function () {
    return Em.I18n.t('rollingrestart.dialog.msg.staleConfigsOnly').format(pluralize(this.get('hostComponentDisplayName')));
  }.property('hostComponentDisplayName')

});

});

require.register("views/common/search_box_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SearchBoxView = Em.View.extend({
  templateName: require('templates/main/host/combo_search_box'),
  errMsg: '',
  classNames: ['col-sm-12'],
  keyFilterMap: [],

  didInsertElement: function didInsertElement() {
    this.initVS();
    this.restoreComboFilterQuery();
    this.showHideClearButton();
    this.initOpenVSButton();
  },

  showErrMsg: function showErrMsg(category) {
    this.set('errMsg', category.attributes.value + " " + Em.I18n.t('hosts.combo.search.invalidCategory'));
  },

  clearErrMsg: function clearErrMsg() {
    this.set('errMsg', '');
  },

  initOpenVSButton: function initOpenVSButton() {
    $('.VS-open-box button').click(function () {
      $('.VS-open-box .popup-arrow-up, .search-box-row').toggleClass('hide');
    });
  },

  initVS: function initVS() {
    window.visualSearch = VS.init({
      container: $('#combo_search_box'),
      query: '',
      showFacets: true,
      delay: 500,
      placeholder: Em.I18n.t('common.search'),
      unquotable: ['text'],
      callbacks: {
        search: this.search.bind(this),
        facetMatches: this.facetMatches.bind(this),
        valueMatches: this.valueMatches.bind(this)
      }
    });
  },

  /**
   * 'search' callback for visualsearch.js
   * @param query
   * @param searchCollection
   */
  search: function search(query, searchCollection) {
    this.clearErrMsg();
    this.showHideClearButton();
    var invalidFacet = this.findInvalidFacet(searchCollection);
    if (invalidFacet) {
      this.showErrMsg(invalidFacet);
    }
    var tableView = this.get('parentView');
    App.db.setComboSearchQuery(tableView.get('controller.name'), query);
    var filterConditions = this.createFilterConditions(searchCollection);
    tableView.updateComboFilter(filterConditions);
  },

  /**
   * 'facetMatches' callback for visualsearch.js
   * @param callback
   */
  facetMatches: function facetMatches(callback) {
    callback(this.get('keyFilterMap').mapProperty('label'), { preserveOrder: true });
  },

  valueMatches: Em.K,

  /**
   *
   * @param {Array} values
   * @param {string} facetValue
   */
  rejectUsedValues: function rejectUsedValues(values, facetValue) {
    return values.reject(function (item) {
      return visualSearch.searchQuery.values(facetValue).indexOf(item) >= 0;
    });
  },

  /**
   *
   * @param {object} searchCollection
   * @returns {!object}
   */
  findInvalidFacet: function findInvalidFacet(searchCollection) {
    var map = this.get('keyFilterMap');
    return searchCollection.models.find(function (facet) {
      return !map.someProperty('label', facet.attributes.category);
    });
  },

  showHideClearButton: function showHideClearButton() {
    if (visualSearch.searchQuery.length > 0) {
      $('.VS-cancel-search-box').removeClass('hide');
    } else {
      $('.VS-cancel-search-box').addClass('hide');
    }
  },

  restoreComboFilterQuery: function restoreComboFilterQuery() {
    var query = App.db.getComboSearchQuery(this.get('parentView.controller.name'));
    if (query) {
      visualSearch.searchBox.setQuery(query);
    }
  },

  /**
   *
   * @param {object} searchCollection
   * @returns {Array}
   */
  createFilterConditions: function createFilterConditions(searchCollection) {
    var _this = this;

    var filterConditions = [];
    var map = this.get('keyFilterMap');

    searchCollection.models.forEach(function (model) {
      var filter = model.attributes;
      var category = map.findProperty('label', filter.category);
      if (category) {
        filterConditions.push({
          skipFilter: false,
          iColumn: category.column,
          value: _this.mapLabelToValue(category.key, filter.value),
          type: category.type
        });
      }
    });
    return filterConditions;
  },

  /**
   *
   * @param {string} category
   * @param {string} label
   * @returns {string}
   */
  mapLabelToValue: function mapLabelToValue(category, label) {
    return label;
  }
});

});

require.register("views/common/select_custom_date_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var stringUtils = require('utils/string_utils');

App.JobsCustomDatesSelectView = Em.View.extend({

  name: 'jobsCustomDatesSelectView',

  templateName: require('templates/common/custom_date_popup'),

  middayPeriodOptions: [Em.I18n.t('jobs.table.custom.date.am'), Em.I18n.t('jobs.table.custom.date.pm')],

  hourOptions: ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12'],

  minuteOptions: ['00', '05', '10', '15', '20', '25', '30', '35', '40', '45', '50', '55'],

  customDateFormFields: Ember.Object.create({
    startDate: null,
    hoursForStart: null,
    minutesForStart: null,
    middayPeriodForStart: null,
    duration: null,
    endDate: null,
    hoursForEnd: null,
    minutesForEnd: null,
    middayPeriodForEnd: null
  }),

  errors: Ember.Object.create({
    isStartDateError: false,
    isEndDateError: false
  }),

  errorMessages: Ember.Object.create({
    startDate: '',
    endDate: ''
  }),

  durationSelect: Em.Select.extend({
    classNames: ['input-medium'],
    content: [{
      value: 900000,
      label: Em.I18n.t('jobs.customDateFilter.duration.15min')
    }, {
      value: 1800000,
      label: Em.I18n.t('jobs.customDateFilter.duration.30min')
    }, {
      value: 3600000,
      label: Em.I18n.t('jobs.customDateFilter.duration.1hr')
    }, {
      value: 7200000,
      label: Em.I18n.t('jobs.customDateFilter.duration.2hr')
    }, {
      value: 14400000,
      label: Em.I18n.t('jobs.customDateFilter.duration.4hr')
    }, {
      value: 43200000,
      label: Em.I18n.t('jobs.customDateFilter.duration.12hr')
    }, {
      value: 86400000,
      label: Em.I18n.t('jobs.customDateFilter.duration.24hr')
    }, {
      value: 604800000,
      label: Em.I18n.t('jobs.customDateFilter.duration.1w')
    }, {
      value: 2592000000,
      label: Em.I18n.t('jobs.customDateFilter.duration.1m')
    }, {
      value: 31536000000,
      label: Em.I18n.t('jobs.customDateFilter.duration.1yr')
    }, {
      value: 0,
      label: Em.I18n.t('common.custom')
    }],
    optionValuePath: 'content.value',
    optionLabelPath: 'content.label',
    selectionBinding: 'parentView.customDateFormFields.duration',
    willInsertElement: function willInsertElement() {
      var duration = this.get('selection'),
          initialOption = this.get('content').find(function (item) {
        return duration === item.value || duration === item.label;
      }, this);
      if (!initialOption) {
        initialOption = this.get('content').findProperty('value', 0);
      }
      this.set('selection', initialOption);
    }
  }),

  isCustomEndDate: Em.computed.equal('customDateFormFields.duration.value', 0),

  didInsertElement: function didInsertElement() {
    $('.datepicker').datepicker({
      format: 'mm/dd/yyyy'
    }).on('changeDate', function () {
      $(this).datepicker('hide');
    });
  },

  createCustomStartDate: function createCustomStartDate() {
    var startDate = this.get('customDateFormFields.startDate');
    var hoursForStart = this.get('customDateFormFields.hoursForStart');
    var minutesForStart = this.get('customDateFormFields.minutesForStart');
    var middayPeriodForStart = this.get('customDateFormFields.middayPeriodForStart');
    if (startDate && hoursForStart && minutesForStart && middayPeriodForStart) {
      return App.getTimeStampFromLocalTime(new Date(startDate + ' ' + hoursForStart + ':' + minutesForStart + ' ' + middayPeriodForStart));
    }
    return null;
  },

  createCustomEndDate: function createCustomEndDate(startDate) {
    var duration = this.get('customDateFormFields.duration.value'),
        date;
    if (duration === 0) {
      var endDate = this.get('customDateFormFields.endDate');
      var hoursForEnd = this.get('customDateFormFields.hoursForEnd');
      var minutesForEnd = this.get('customDateFormFields.minutesForEnd');
      var middayPeriodForEnd = this.get('customDateFormFields.middayPeriodForEnd');
      if (endDate && hoursForEnd && minutesForEnd && middayPeriodForEnd) {
        date = App.getTimeStampFromLocalTime(new Date(endDate + ' ' + hoursForEnd + ':' + minutesForEnd + ' ' + middayPeriodForEnd));
      }
    } else if (!Em.isNone(startDate)) {
      date = startDate + duration;
    }
    return date;
  },

  setErrorMessage: function setErrorMessage(key, message) {
    var errors = this.get('errors'),
        errorMessages = this.get('errorMessages'),
        isError = !Em.isNone(message);
    message = isError ? message : '';
    errors.set('is' + key.capitalize() + 'Error', isError);
    errorMessages.set(key, message);
  },

  validate: function () {
    var hasErrors = false,
        formFields = this.get('customDateFormFields'),
        errors = this.get('errors'),
        errorMessages = this.get('errorMessages');

    // Check if fields are empty or invalid
    Em.keys(errorMessages).forEach(function (key) {
      var value = formFields.get(key),
          timestamp = App.getTimeStampFromLocalTime(value);
      if (key !== 'endDate' || this.get('isCustomEndDate')) {
        if (!formFields.get(key)) {
          hasErrors = true;
          this.setErrorMessage(key);
        } else if (isNaN(timestamp)) {
          this.setErrorMessage(key, Em.I18n.t('jobs.customDateFilter.error.incorrect'));
          hasErrors = true;
        } else {
          this.setErrorMessage(key);
        }
      }
    }, this);

    // Check that endDate is after startDate
    if (!hasErrors) {
      var startDate = this.createCustomStartDate(),
          endDate = this.createCustomEndDate(startDate);
      if (startDate && endDate) {
        if (startDate > endDate) {
          hasErrors = true;
          this.setErrorMessage('endDate', Em.I18n.t('jobs.customDateFilter.error.date.order'));
        }
        if (startDate > new Date().getTime()) {
          hasErrors = true;
          this.setErrorMessage('startDate', Em.I18n.t('jobs.customDateFilter.error.laterThanNow'));
        }
      }
    }

    this.set('parentView.disablePrimary', hasErrors);

    // Get customized time range if there are no errors
    if (!hasErrors) {
      var duration;
      if (this.get('isCustomEndDate')) {
        var values = [{
          label: 'year',
          max: Infinity
        }, {
          label: 'month',
          max: 12
        }, {
          label: 'week',
          max: 4
        }, {
          label: 'day',
          max: 7
        }, {
          label: 'hour',
          max: 24
        }, {
          label: 'minute',
          max: 60
        }],
            start = moment(startDate),
            end = moment(endDate);
        values.forEach(function (item) {
          var diff = end.diff(start, item.label);
          item.value = diff >= item.max ? diff % item.max : diff;
        });
        duration = values.filterProperty('value').map(function (item) {
          return item.value + ' ' + stringUtils.pluralize(item.value, item.label);
        }).join(' ');
      } else {
        duration = this.get('customDateFormFields.duration.label');
      }
      this.get('controller').setProperties({
        startTime: startDate,
        endTime: endDate,
        customDuration: duration
      });
    }
  }.observes('customDateFormFields.startDate', 'customDateFormFields.hoursForStart', 'customDateFormFields.minutesForStart', 'customDateFormFields.middayPeriodForStart', 'customDateFormFields.endDate', 'customDateFormFields.hoursForEnd', 'customDateFormFields.minutesForEnd', 'customDateFormFields.middayPeriodForEnd', 'customDateFormFields.duration.value')
});

});

require.register("views/common/sort_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var misc = require('utils/misc');
var stringUtils = require('utils/string_utils');
var App = require('app');

/**
 * Wrapper View for all sort components. Layout template and common actions are located inside of it.
 * Logic specific for sort fields
 * located in inner view - <code>fieldView</code>.
 *
 * @type {*}
 */
var wrapperView = Em.View.extend({
  tagName: 'tr',

  name: 'SortWrapperView',

  classNames: ['sort-wrapper'],

  willInsertElement: function willInsertElement() {
    if (this.get('parentView.tableFilteringComplete')) {
      this.get('parentView').set('filteringComplete', true);
    }
  },

  /**
   * Load sort statuses from local storage
   * Works only after finish filtering in the parent View
   */
  loadSortStatuses: function () {
    var statuses = App.db.getSortingStatuses(this.get('controller.name'));
    if (!this.get('parentView.filteringComplete')) return;
    if (statuses) {
      var childViews = this.get('childViews');
      var self = this;
      statuses.forEach(function (st) {
        if (st.status != 'sorting') {
          var sortOrder = false;
          if (st.status == 'sorting_desc') {
            sortOrder = true;
          }
          self.sort(childViews.findProperty('name', st.name), sortOrder);
          childViews.findProperty('name', st.name).set('status', sortOrder ? 'sorting_desc' : 'sorting_asc');
          self.get('controller').set('sortingColumn', childViews.findProperty('name', st.name));
        } else {
          childViews.findProperty('name', st.name).set('status', st.status);
        }
      });
    }
  }.observes('parentView.filteringComplete'),

  /**
   * Save sort statuses to local storage
   * Works only after finish filtering in the parent View
   */
  saveSortStatuses: function saveSortStatuses() {
    if (!this.get('parentView.filteringComplete')) return;

    var statuses = [];
    this.get('childViews').forEach(function (childView) {
      statuses.push({
        name: childView.get('name'),
        status: childView.get('status')
      });
    });
    App.db.setSortingStatuses(this.get('controller.name'), statuses);
  },

  /**
   * sort content by property
   * @param property {object}
   * @param order {Boolean} true - DESC, false - ASC
   * @param returnSorted {Boolean}
   * @param content {Array}
   */
  sort: function sort(property, order, returnSorted, content) {
    content = content || this.get('content').toArray();
    var sortFunc = this.getSortFunc(property, order);
    var status = order ? 'sorting_desc' : 'sorting_asc';

    this.resetSort();
    this.get('childViews').findProperty('name', property.get('name')).set('status', status);
    this.saveSortStatuses(property, order);
    content.sort(sortFunc);

    if (!!returnSorted) {
      return content;
    } else {
      this.set('content', content);
    }
  },

  isSorting: false,

  onContentChange: function () {
    if (!this.get('isSorting') && this.get('content.length')) {
      this.get('childViews').forEach(function (view) {
        if (view.status !== 'sorting') {
          var status = view.get('status');
          this.set('isSorting', true);
          this.sort(view, status == 'sorting_desc');
          this.set('isSorting', false);
          view.set('status', status);
        }
      }, this);
    }
  }.observes('controller.contentUpdater'),

  /**
   *
   * @param {Em.Object[]} content
   * @returns {Em.Object[]}
   */
  getSortedContent: function getSortedContent(content) {
    if (!this.get('isSorting') && content.get('length')) {
      var activeSortViews = this.get('childViews').rejectProperty('status', 'sorting');
      if (activeSortViews[0]) {
        var status = activeSortViews[0].get('status');
        this.set('isSorting', true);
        content = this.sort(activeSortViews[0], status === 'sorting_desc', true, content);
        this.set('isSorting', false);
        activeSortViews[0].set('status', status);
      }
    }
    return content;
  },

  /**
   * reset all sorts fields
   */
  resetSort: function resetSort() {
    this.get('childViews').setEach('status', 'sorting');
  },
  /**
   * determines sort function depending on the type of sort field
   * @param property
   * @param order
   * @return {*}
   */
  getSortFunc: function getSortFunc(property, order) {
    var func;
    switch (property.get('type')) {
      case 'ip':
        func = function func(a, b) {
          var a = misc.ipToInt(a.get(property.get('name')));
          var b = misc.ipToInt(b.get(property.get('name')));
          if (order) {
            return a - b;
          } else {
            return b - a;
          }
        };
        break;
      case 'number':
        func = function func(a, b) {
          var a_p = a.get(property.get('name'));
          var b_p = b.get(property.get('name'));
          a_p = Em.isNone(a_p) ? -Infinity : parseFloat(a_p);
          b_p = Em.isNone(b_p) ? -Infinity : parseFloat(b_p);
          if (order) {
            return a_p - b_p;
          } else {
            return b_p - a_p;
          }
        };
        break;
      case 'version':
        func = function func(a, b) {
          var res = stringUtils.compareVersions(a.get(property.get('name')), b.get(property.get('name')));
          if (order) {
            return -res;
          } else {
            return res;
          }
        };
        break;
      case 'alert_status':
        func = App.AlertDefinition.getSortDefinitionsByStatus(order);
        break;
      default:
        func = function func(a, b) {
          var a_p = a.get(property.get('name'));
          var b_p = b.get(property.get('name'));
          a_p = Em.isNone(a_p) ? '' : ('' + a_p).toLowerCase();
          b_p = Em.isNone(b_p) ? '' : ('' + b_p).toLowerCase();
          return order ? b_p.localeCompare(a_p) : a_p.localeCompare(b_p);
        };
    }
    return func;
  },

  /**
   * method that runs <code>contentWasChanged<code>
   *
   * @method onContentChangeOnce
   */
  onContentChangeOnce: function onContentChangeOnce() {
    var keys = arguments[1].match(/[a-zA-Z]+$/),
        key = keys.length ? keys[0] : null;
    if (key) {
      Em.run.once(this.get('controller'), 'contentWasChanged', key);
    }
  },

  /**
   * Add observer for key to call  <code>onContentChange</code>
   * @param key
   */
  addSortingObserver: function addSortingObserver(key) {
    this.addObserver('controller.content.@each.' + key, this, 'onContentChangeOnce');
  },

  /**
   * Remove observer for key to call  <code>onContentChange</code>
   * @param key
   */
  removeSortingObserver: function removeSortingObserver(key) {
    this.removeObserver('controller.content.@each.' + key, this, 'onContentChangeOnce');
  },

  willDestroyElement: function willDestroyElement() {
    this.removeSortingObserver(this.get('controller.sortingColumn.name'));
  }
});

/**
 * view that carry on sorting on server-side via <code>refresh()</code> in parentView
 * @type {*}
 */
var serverWrapperView = Em.View.extend({
  tagName: 'tr',

  classNames: ['sort-wrapper'],

  willInsertElement: function willInsertElement() {
    this.loadSortStatuses();
  },

  /**
   * Initialize and save sorting statuses: hostName sorting_asc
   */
  loadSortStatuses: function loadSortStatuses() {
    var childViews = this.get('childViews');
    var statuses = App.db.getSortingStatuses(this.get('controller.name'));
    if (statuses) {
      var sortingColumn = App.db.getSortingStatuses(this.get('controller.name')).find(function (column) {
        return column.status != 'sorting';
      });
      if (sortingColumn) {
        var sortingColumnView = childViews.findProperty('name', sortingColumn.name);
        sortingColumnView.set('status', sortingColumn.status);
        this.get('controller').set('sortingColumn', sortingColumnView);
      }
    } else {
      statuses = [];
      childViews.forEach(function (childView) {
        var sortStatus = childView.get('name') == 'hostName' && childView.get('status') == 'sorting' ? 'sorting_asc' : childView.get('status');
        statuses.push({
          name: childView.get('name'),
          status: sortStatus
        });
        childView.set('status', sortStatus);
      });
      App.db.setSortingStatuses(this.get('controller.name'), statuses);
      this.get('controller').set('sortingColumn', childViews.findProperty('name', 'hostName'));
    }
  },

  /**
   * Save sort statuses to local storage
   * Works only after finish filtering in the parent View
   */
  saveSortStatuses: function saveSortStatuses() {
    var statuses = [];
    this.get('childViews').forEach(function (childView) {
      statuses.push({
        name: childView.get('name'),
        status: childView.get('status')
      });
    });
    App.db.setSortingStatuses(this.get('controller.name'), statuses);
  },

  /**
   * sort content by property
   * @param property {object}
   * @param order {Boolean} true - DESC, false - ASC
   */
  sort: function sort(property, order) {
    var status = order ? 'sorting_desc' : 'sorting_asc';
    var self = this;

    this.resetSort();
    this.get('childViews').findProperty('name', property.get('name')).set('status', status);
    this.saveSortStatuses();
    if (!this.get('parentView.filteringComplete')) {
      clearTimeout(this.get('parentView.timeOut'));
      this.set('parentView.timeOut', setTimeout(function () {
        self.sort(property, order);
      }, this.get('parentView.filterWaitingTime')));
    } else {
      clearTimeout(this.get('parentView.timeOut'));
      this.get('parentView').refresh();
    }
  },

  /**
   * reset all sorts fields
   */
  resetSort: function resetSort() {
    this.get('childViews').setEach('status', 'sorting');
  }
});
/**
 * particular view that contain sort field properties:
 * name - name of property in content table
 * type(optional) - specific type to sort
 * displayName - label to display
 * @type {*}
 */
var fieldView = Em.View.extend({
  template: Em.Handlebars.compile('<span {{bindAttr class="view.status :column-name"}}>{{view.displayName}}</span>'),
  classNameBindings: ['viewNameClass'],
  tagName: 'th',
  name: null,
  displayName: null,
  status: 'sorting',
  viewNameClass: function () {
    return 'sort-view-' + this.get('column');
  }.property(),
  type: null,
  column: 0,
  /**
   * callback that run sorting and define order of sorting
   * @param event
   */
  click: function click(event) {
    var wrapperView = this.get('parentView');
    wrapperView.sort(this, this.get('status') !== 'sorting_desc');

    // add observer for sorting property key to apply sorting if some value will be changed
    if (wrapperView.addSortingObserver && wrapperView.removeSortingObserver) {
      var currentObserverProperty = this.get('controller.sortingColumn.name');
      if (currentObserverProperty) {
        wrapperView.removeSortingObserver(currentObserverProperty);
      }
      wrapperView.addSortingObserver(this.get('name'));
    }
    this.get('controller').set('sortingColumn', this);
  }
});

/**
 * Result object, which will be accessible outside
 * @type {Object}
 */
module.exports = {
  serverWrapperView: serverWrapperView,
  wrapperView: wrapperView,
  fieldView: fieldView
};

});

require.register("views/common/table_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view');

App.TableView = Em.View.extend(App.Persist, {

  init: function init() {
    this.set('filterConditions', []);
    this._super();
  },

  /**
   * Defines to show pagination or show all records
   * @type {Boolean}
   */
  pagination: true,

  /**
   * Shows if all data is loaded and filtered
   * @type {Boolean}
   */
  filteringComplete: false,

  /**
   * intermediary for filteringComplete
   * @type {Boolean}
   */
  tableFilteringComplete: false,

  /**
   * The number of rows to show on every page
   * The value should be a number converted into string type in order to support select element API
   * Example: "10", "25"
   * @type {String}
   */
  displayLength: null,

  /**
   * default value of display length
   * The value should be a number converted into string type in order to support select element API
   * Example: "10", "25"
   */
  defaultDisplayLength: "10",

  /**
   * number of items in table after applying filters
   */
  filteredCount: Em.computed.alias('filteredContent.length'),

  /**
   * total number of items in table before applying filters
   */
  totalCount: Em.computed.alias('content.length'),

  /**
   * Determines whether current page is the first one
   */
  isCurrentPageFirst: Em.computed.lte('startIndex', 1),

  /**
   * Determines whether current page is the last one
   */
  isCurrentPageLast: Em.computed.equalProperties('endIndex', 'filteredCount'),

  /**
   * Do filtering, using saved in the local storage filter conditions
   */
  willInsertElement: function willInsertElement() {
    var self = this;
    var name = this.get('controller.name');
    if (!this.get('displayLength')) {
      var displayLength = App.db.getDisplayLength(name);
      if (displayLength) {
        if (this.get('state') !== "inBuffer") {
          self.set('displayLength', displayLength);
          self.initFilters();
        } else {
          Em.run.next(function () {
            self.set('displayLength', displayLength);
            self.initFilters();
          });
        }
      } else {
        if (!$.mocho) {
          this.getUserPref(this.displayLengthKey()).complete(function () {
            self.initFilters();
          });
        }
      }
    }
  },

  /**
   * initialize filters
   * restore values from local DB
   * or clear filters in case there is no filters to restore
   */
  initFilters: function initFilters() {
    var name = this.get('controller.name');
    var self = this;
    var filterConditions = App.db.getFilterConditions(name);
    if (!Em.isEmpty(filterConditions)) {
      this.set('filterConditions', filterConditions);

      var childViews = this.get('childViews');

      filterConditions.forEach(function (condition, index, filteredConditions) {
        var view = !Em.isNone(condition.iColumn) && childViews.findProperty('column', condition.iColumn);
        if (view) {
          if (view.get('emptyValue')) {
            view.clearFilter();
            self.saveFilterConditions(condition.iColumn, view.get('appliedEmptyValue'), condition.type, false);
          } else {
            view.setValue(condition.value);
          }
          Em.run.next(function () {
            view.showClearFilter();
            // check if it is the last iteration
            if (filteredConditions.length - index - 1 === 0) {
              self.set('tableFilteringComplete', true);
            }
          });
        } else {
          self.set('tableFilteringComplete', true);
        }
      });
    } else {
      this.set('tableFilteringComplete', true);
    }
  },

  /**
   * Persist-key of current table displayLength property
   * @param {String} loginName current user login name
   * @returns {String}
   */
  displayLengthKey: function displayLengthKey(loginName) {
    if (App.get('testMode')) {
      return 'pagination_displayLength';
    }
    loginName = loginName ? loginName : App.router.get('loginName');
    return this.get('controller.name') + '-pagination-displayLength-' + loginName;
  },

  /**
   * Set received from server value to <code>displayLengthOnLoad</code>
   * @param {Number} response
   * @param {Object} request
   * @param {Object} data
   * @returns {*}
   */
  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response, request, data) {
    this.set('displayLength', response);
    return response;
  },

  /**
   * Set default value to <code>displayLength</code> (and send it on server) if value wasn't found on server
   * @returns {Number}
   */
  getUserPrefErrorCallback: function getUserPrefErrorCallback() {
    // this user is first time login
    var displayLengthDefault = this.get('defaultDisplayLength');
    this.set('displayLength', displayLengthDefault);
    if (App.isAuthorized('SERVICE.VIEW_METRICS')) {
      this.saveDisplayLength();
    }
    this.filter();
    return displayLengthDefault;
  },

  /**
   * Return pagination information displayed on the page
   * @type {String}
   */
  paginationInfo: Em.computed.i18nFormat('tableView.filters.paginationInfo', 'startIndex', 'endIndex', 'filteredCount'),

  /**
   * Start index for displayed content on the page
   */
  startIndex: 1,

  /**
   * Calculate end index for displayed content on the page
   */
  endIndex: function () {
    if (this.get('pagination') && this.get('displayLength')) {
      return Math.min(this.get('filteredCount'), this.get('startIndex') + parseInt(this.get('displayLength')) - 1);
    } else {
      return this.get('filteredCount') || 0;
    }
  }.property('startIndex', 'displayLength', 'filteredCount'),

  /**
   * Onclick handler for previous page button on the page
   */
  previousPage: function previousPage() {
    var result = this.get('startIndex') - parseInt(this.get('displayLength'));
    this.set('startIndex', result < 2 ? 1 : result);
  },

  /**
   * Onclick handler for next page button on the page
   */
  nextPage: function nextPage() {
    var result = this.get('startIndex') + parseInt(this.get('displayLength'));
    if (result - 1 < this.get('filteredCount')) {
      this.set('startIndex', result);
    }
  },
  /**
   * Onclick handler for first page button on the page
   */
  firstPage: function firstPage() {
    this.set('startIndex', 1);
  },
  /**
   * Onclick handler for last page button on the page
   */
  lastPage: function lastPage() {
    var pagesCount = this.get('filteredCount') / parseInt(this.get('displayLength'));
    var startIndex = this.get('filteredCount') % parseInt(this.get('displayLength')) === 0 ? (pagesCount - 1) * parseInt(this.get('displayLength')) : Math.floor(pagesCount) * parseInt(this.get('displayLength'));
    this.set('startIndex', ++startIndex);
  },

  /**
   * Calculates default value for startIndex property after applying filter or changing displayLength
   */
  updatePaging: function (controller, property) {
    var displayLength = this.get('displayLength');
    var filteredContentLength = this.get('filteredCount');
    if (property == 'displayLength') {
      this.set('startIndex', Math.min(1, filteredContentLength));
    } else if (!filteredContentLength) {
      this.set('startIndex', 0);
    } else if (this.get('startIndex') > filteredContentLength) {
      this.set('startIndex', Math.floor((filteredContentLength - 1) / displayLength) * displayLength + 1);
    } else if (!this.get('startIndex')) {
      this.set('startIndex', 1);
    }
  }.observes('displayLength', 'filteredCount'),

  /**
   * Apply each filter to each row
   *
   * @param {Number} iColumn number of column by which filter
   * @param {Object} value
   * @param {String} type
   */
  updateFilter: function updateFilter(iColumn, value, type) {
    this.saveFilterConditions(iColumn, value, type, false);
    this.filtersUsedCalc();
    this.filter();
  },

  /**
   *
   * @param {Array} filterConditions
   */
  updateComboFilter: function updateComboFilter(filterConditions) {
    this.set('controller.resetStartIndex', true);
    this.set('filterConditions', filterConditions);
    this.saveAllFilterConditions();
    this.filter();
  },

  /**
   * save filter conditions to local storage
   * @param iColumn {Number}
   * @param value {String|Array}
   * @param type {String}
   * @param skipFilter {Boolean}
   */
  saveFilterConditions: function saveFilterConditions(iColumn, value, type, skipFilter) {
    var filterCondition = this.get('filterConditions').findProperty('iColumn', iColumn);

    if (filterCondition) {
      filterCondition.value = value;
      filterCondition.skipFilter = skipFilter;
    } else {
      filterCondition = {
        skipFilter: skipFilter,
        iColumn: iColumn,
        value: value,
        type: type
      };
      this.get('filterConditions').push(filterCondition);
      this.propertyDidChange('showFilteredContent');
    }

    this.saveAllFilterConditions();
  },

  /**
   * Save not empty <code>filterConditions</code> to the localStorage
   *
   * @method saveAllFilterConditions
   */
  saveAllFilterConditions: function saveAllFilterConditions() {
    var filterConditions = this.get('filterConditions');
    // remove empty entries
    filterConditions = filterConditions.filter(function (item) {
      return !Em.isEmpty(item.value);
    });
    this.set('filterConditions', filterConditions);
    App.db.setFilterConditions(this.get('controller.name'), filterConditions);
  },

  saveDisplayLength: function saveDisplayLength() {
    var self = this;
    Em.run.next(function () {
      App.db.setDisplayLength(self.get('controller.name'), self.get('displayLength'));
      if (!App.get('testMode')) {
        if (App.isAuthorized('SERVICE.VIEW_METRICS')) {
          self.postUserPref(self.displayLengthKey(), self.get('displayLength'));
        }
      }
    });
  },

  clearFilterConditionsFromLocalStorage: function clearFilterConditionsFromLocalStorage() {
    var result = false;
    var currentFCs = App.db.getFilterConditions(this.get('controller.name'));
    if (currentFCs != null) {
      App.db.setFilterConditions(this.get('controller.name'), null);
      result = true;
    }
    var query = App.db.getComboSearchQuery(this.get('controller.name'));
    if (query != null) {
      App.db.setComboSearchQuery(this.get('controller.name'), null);
      result = true;
    }
    return result;
  },

  clearStartIndex: function clearStartIndex() {
    if (this.get('controller.startIndex') !== 1) {
      this.set('controller.resetStartIndex', true);
      return true;
    }
    return false;
  },

  /**
   * Contain filter conditions for each column
   * @type {Array}
   */
  filterConditions: [],

  /**
   * Contains content after implementing filters
   * @type {Array}
   */
  filteredContent: [],

  /**
   * Determine if <code>filteredContent</code> is empty or not
   * @type {Boolean}
   */
  hasFilteredItems: Em.computed.bool('filteredCount'),

  /**
   * Contains content to show on the current page of data page view
   * @type {Array}
   */
  pageContent: function () {
    return this.get('filteredContent').slice(this.get('startIndex') - 1, this.get('endIndex'));
  }.property('filteredCount', 'startIndex', 'endIndex'),

  /**
   * flag to toggle displaying filtered hosts counter
   */
  showFilteredContent: function () {
    var result = false;
    if (this.get('filterConditions.length') > 0) {
      this.get('filterConditions').forEach(function (f) {
        if (f.value) {
          if (Em.typeOf(f.value) == "array") {
            if (f.value[0] || f.value[1]) {
              result = true;
            }
          } else {
            result = true;
          }
        }
      });
    }
    return result;
  }.property('filteredContent.length'),
  /**
   * Filter table by filterConditions
   */
  filter: function () {
    var content = this.get('content');
    var filterConditions = this.get('filterConditions').filterProperty('value');
    var result;
    var assoc = this.get('colPropAssoc');
    if (filterConditions.length) {
      result = content.filter(function (item) {
        var match = true;
        filterConditions.forEach(function (condition) {
          var filterFunc = filters.getFilterByType(condition.type, false);
          if (match) {
            match = filterFunc(item.get(assoc[condition.iColumn]), condition.value);
          }
        });
        return match;
      });
      this.set('filteredContent', result);
    } else {
      this.set('filteredContent', content ? content.toArray() : []);
    }
  }.observes('content.length'),

  /**
   * Does any filter is used on the page
   * @type {Boolean}
   */
  filtersUsed: false,

  /**
   * Determine if some filters are used on the page
   * Set <code>filtersUsed</code> value
   */
  filtersUsedCalc: function filtersUsedCalc() {
    var filterConditions = this.get('filterConditions');
    if (!filterConditions.length) {
      this.set('filtersUsed', false);
      return;
    }
    var filtersUsed = false;
    filterConditions.forEach(function (filterCondition) {
      if (filterCondition.value.toString() !== '') {
        filtersUsed = true;
      }
    });
    this.set('filtersUsed', filtersUsed);
  },

  /**
   * Run <code>clearFilter</code> in the each child filterView
   */
  clearFilters: function clearFilters() {
    this.set('filterConditions', []);
    this.get('childViews').forEach(function (childView) {
      Em.tryInvoke(childView, 'clearFilter');
    });
  }

});

});

require.register("views/common/time_range", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * use: {{view App.TimeRangeWidget controllerBinding="App.router.mainChartsController"}}
 * set controller.preset field with preset value
 * widget assign itself to controller like presetWidget (controller.get('presetWidget'))
 * @type {*}
 */
App.TimeRangeWidget = Em.View.extend({
  classNames: ['time-range-widget'],
  templateName: require('templates/common/time_range'),
  dateFrom: null,
  dateTo: null,

  /**
   * presets
   */
  presets: [Em.Object.create({ label: Em.I18n.t('timeRange.presets.1hour'), value: '1h', period: 3600000, step: 300000 }), Em.Object.create({ label: Em.I18n.t('timeRange.presets.12hour'), value: '12h', period: 43200000, step: 3600000 }), Em.Object.create({ label: Em.I18n.t('timeRange.presets.1day'), value: '1d', period: 86400000, step: 3600000 }), Em.Object.create({ label: Em.I18n.t('timeRange.presets.1week'), value: '1wk', period: 604800000, step: 86400000 }), Em.Object.create({ label: Em.I18n.t('timeRange.presets.1month'), value: '1mo', period: 2592000000, step: 86400000 }), Em.Object.create({ label: Em.I18n.t('timeRange.presets.1year'), value: '1yr', period: 31536000000, step: 2592000000 })],
  /**
   * chosen preset value
   */
  chosenPreset: null,

  /**
   * return array of chosen presets
   */
  chosenPresets: function () {
    return this.get('chosenPreset') ? [this.get('chosenPreset')] : this.get('defaultPresets');
  }.property('chosenPreset'),

  /**
   * preset item view
   */
  presetView: Em.View.extend({
    tagName: 'li',
    classNameBindings: ['disabled'],
    disabled: Em.computed.ifThenElse('isActive', 'disabled', false),
    isActive: Em.computed.equalProperties('preset.value', 'widget.chosenPreset.value'),
    template: Em.Handlebars.compile('<a {{action activate view.preset target="view.widget" href="true" }}>{{unbound view.preset.label}}</a>')
  }),

  /**
   * return default selected presets (currently - all)
   */
  defaultPresets: function () {
    var values = [];
    $.each(this.get('presets'), function () {
      if (this.value) {
        values.push(this.value);
      }
    });
    return values;
  }.property(),

  bindToController: function bindToController() {
    var thisW = this;
    var controller = this.get('controller');
    controller.set('presetWidget', thisW);
  },

  /**
   * assign this widget to controller, prepare items by presetsConfig
   */
  init: function init() {
    this._super();
    this.bindToController();
  },

  /**
   * write active preset to widget
   * @param event
   */
  activate: function activate(event) {
    if (event.context == this.get('chosenPreset')) {
      this.set('chosenPreset', null);
    } else {
      this.set('chosenPreset', event.context);
    }
  },

  dateFromView: Ember.TextField.extend({
    elementId: 'timeRangeFrom',
    classNames: 'timeRangeFrom',
    attributeBindings: ['readonly'],
    readonly: true,
    didInsertElement: function didInsertElement() {
      var self = this;
      this.$().datetimepicker({
        dateFormat: 'dd/mm/yy',
        timeFormat: 'hh:mm',
        maxDate: new Date(),
        onClose: function onClose(dateText, inst) {
          var endDateTextBox = $('#timeRangeTo');
          if (endDateTextBox.val() != '') {
            var testStartDate = new Date(dateText);
            var testEndDate = new Date(endDateTextBox.val());
            if (testStartDate > testEndDate) endDateTextBox.val(dateText);
          } else {
            endDateTextBox.val(dateText);
          }
          self.set('dateFrom', dateText);
        },
        onSelect: function onSelect(selectedDateTime) {
          var start = $(this).datetimepicker('getDate');
          $('#timeRangeTo').datetimepicker('option', 'minDate', new Date(start.getTime()));
        }
      });
      self.set('dateFrom', this.get('value'));
    }
  }),

  dateToView: Ember.TextField.extend({
    elementId: 'timeRangeTo',
    classNames: 'timeRangeTo',
    attributeBindings: ['readonly'],
    readonly: true,
    didInsertElement: function didInsertElement() {
      var self = this;
      this.$().datetimepicker({
        dateFormat: 'dd/mm/yy',
        timeFormat: 'hh:mm',
        maxDate: new Date(),
        onClose: function onClose(dateText, inst) {
          var startDateTextBox = $('#timeRangeFrom');
          if (startDateTextBox.val() != '') {
            var testStartDate = new Date(startDateTextBox.val());
            var testEndDate = new Date(dateText);
            if (testStartDate > testEndDate) startDateTextBox.val(dateText);
          } else {
            startDateTextBox.val(dateText);
          }
          self.set('dateTo', dateText);
        },
        onSelect: function onSelect(selectedDateTime) {
          var end = $(this).datetimepicker('getDate');
          $('#timeRangeFrom').datetimepicker('option', 'maxDate', new Date(end.getTime()));
        }
      });
      self.set('dateTo', this.get('value'));
    }
  }),

  sliderOptions: Ember.Object.extend({
    end: null,
    period: null,
    start: function () {
      return this.get('end') - this.get('period');
    }.property('end', 'period')
  }),
  nowLabel: null,
  rangeLabel: null,
  buildSlider: function () {
    if (this.get('chosenPreset')) {
      var sliderOptions = this.sliderOptions.create({
        end: function () {
          var endDate = new Date();
          return endDate.getTime();
        }.property(),
        period: this.get('chosenPreset.period'),
        step: this.get('chosenPreset.step'),
        countTimeAgo: function countTimeAgo(stepValue) {
          var msAgo = this.get('end') - stepValue;
          return msAgo.toDaysHoursMinutes();
        }
      });
      this.set('nowLabel', 'Now');
      this.set('rangeLabel', new Date(sliderOptions.get('start')));

      var self = this;
      $('#slider').slider({
        range: "max",
        min: sliderOptions.get('start'),
        max: sliderOptions.get('end'),
        value: sliderOptions.get('start'),
        step: sliderOptions.get('step'),
        stop: function stop(event, ui) {
          self.set('rangeLabel', new Date(ui.value));
          //          self.set('rangeLabel', sliderOptions.countTimeAgo(ui.value).h);
        },
        slide: function slide(event, ui) {
          self.set('rangeLabel', new Date(ui.value));
          //          self.set('rangeLabel', sliderOptions.countTimeAgo(ui.value).h);
        }
      });
    } else {
      $("#slider").slider("destroy");
    }
  }.observes('chosenPreset')
});

});

require.register("views/common/time_range_list", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TimeRangeListView = Em.View.extend({

  templateName: require('templates/common/time_range_list'),

  classNameBindings: [':btn-group', 'parentView.timeRangeClassName']

});

});

require.register("views/common/widget/gauge_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.GaugeWidgetView = Em.View.extend(App.WidgetMixin, {
  templateName: require('templates/common/widget/gauge_widget'),

  /**
   * @type {string}
   */
  value: '',

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: [],

  /**
   * 1 - is maximum value of a gauge
   * @type {number}
   * @const
   */
  MAX_VALUE: 1,
  /**
   * 0 - is minimum value of a gauge
   * @type {number}
   * @const
   */
  MIN_VALUE: 0,

  /**
   * @type {boolean}
   */
  isUnavailable: function () {
    return isNaN(parseFloat(this.get('value'))) || this.get('isOverflowed');
  }.property('value', 'isOverflowed'),

  /**
   * @type {boolean}
   */
  isOverflowed: function () {
    return parseFloat(this.get('value')) > this.get('MAX_VALUE') || parseFloat(this.get('value')) < this.get('MIN_VALUE');
  }.property('value'),

  chartView: App.ChartPieView.extend({
    stroke: '#transparent',
    innerR: 40,

    /**
     * since chart widget using percentage values factor equal 100
     * @type {number}
     * @const
     */
    FACTOR: 100,

    /**
     * 100 - is maximum percentage value
     * @type {number}
     * @const
     */
    MAX_VALUE: 100,

    warningThreshold: Em.computed.alias('parentView.content.properties.warning_threshold'),

    errorThreshold: Em.computed.alias('parentView.content.properties.error_threshold'),

    id: Em.computed.format('gauge-widget-{0}', 'parentView.content.id'),

    existCenterText: true,
    centerTextColor: Em.computed.alias('contentColor'),

    palette: new Rickshaw.Color.Palette({
      scheme: ['#FFFFFF', '#D6DDDF'].reverse()
    }),

    data: function () {
      var data = parseFloat(this.get('parentView.value')) * this.get('FACTOR');
      if (isNaN(data) || data > this.get('MAX_VALUE')) return [0, this.get('MAX_VALUE')];
      return [data.toFixed(0), this.get('MAX_VALUE') - data.toFixed(0)];
    }.property('parentView.value'),

    contentColor: function () {
      var used = parseFloat(this.get('parentView.value')) * this.get('FACTOR');
      var threshold1 = parseFloat(this.get('warningThreshold')) * this.get('FACTOR');
      var threshold2 = parseFloat(this.get('errorThreshold')) * this.get('FACTOR');
      var color_green = App.healthStatusGreen;
      var color_red = App.healthStatusRed;
      var color_orange = App.healthStatusOrange;
      if (isNaN(threshold1) && isNaN(threshold2) || isNaN(threshold1) && used <= threshold2 || isNaN(threshold2) && used <= threshold1 || !isNaN(threshold2) && threshold1 > threshold2 && used > threshold1 || !isNaN(threshold2) && threshold1 < threshold2 && used <= threshold1) {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', color_green].reverse()
        }));
      } else if (!isNaN(threshold2) && used.isInRange(threshold1, threshold2) || isNaN(threshold2) && used > threshold1) {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', color_orange].reverse()
        }));
      } else {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', color_red].reverse()
        }));
      }
      return App.widgetContentColor;
    }.property('parentView.value', 'warningThreshold', 'errorThreshold'),

    // refresh text and color when data in model changed
    refreshSvg: function () {
      // remove old svg
      var old_svg = $("#" + this.get('id'));
      if (old_svg) {
        old_svg.remove();
      }

      // draw new svg
      this.appendSvg();
    }.observes('parentView.value', 'warningThreshold', 'errorThreshold')
  })
});

});

require.register("views/common/widget/graph_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var fileUtils = require('utils/file_utils');

var CUSTOM_TIME_INDEX = 8;

App.GraphWidgetView = Em.View.extend(App.WidgetMixin, App.ExportMetricsMixin, {
  templateName: require('templates/common/widget/graph_widget'),

  /**
   *  type of metric query from which the widget is comprised
   */

  metricType: 'TEMPORAL',

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: [],

  /**
   * 3600 sec in 1 hour
   * @const
   */
  TIME_FACTOR: 3600,

  /**
   * custom time range, set when graph opened in popup
   * @type {number|null}
   */
  customTimeRange: null,

  /**
   * value in seconds
   * @type {number}
   */
  timeRange: function () {
    var timeRange = parseInt(this.get('content.properties.time_range'));
    if (isNaN(timeRange)) {
      //1h - default time range
      timeRange = 1;
    }

    // Custom start and end time is specified by user
    if (this.get('exportTargetView.currentTimeIndex') === CUSTOM_TIME_INDEX) {
      return 0;
    }

    return this.get('customTimeRange') || timeRange * this.get('TIME_FACTOR');
  }.property('content.properties.time_range', 'customTimeRange'),

  /**
   * value in ms
   * @type {number}
   */
  timeStep: 15,

  /**
   * @type {Array}
   */
  data: [],

  /**
   * time range index for graph
   * @type {number}
   */
  timeIndex: 0,

  /**
   * custom start time for graph
   * @type {number|null}
   */
  startTime: null,

  /**
   * custom end time for graph
   * @type {number|null}
   */
  endTime: null,

  /**
   * graph time range duration in seconds
   * @type {number|null}
   */
  graphSeconds: null,

  /**
   * time range duration as string
   * @type {string|null}
   */
  durationFormatted: null,

  exportTargetView: Em.computed.alias('childViews.lastObject'),

  drawWidget: function drawWidget() {
    if (this.get('isLoaded')) {
      this.set('data', this.calculateValues());
    }
  },

  /**
   * calculate series datasets for graph widgets
   */
  calculateValues: function calculateValues() {
    var metrics = this.get('metrics');
    var seriesData = [];
    if (this.get('content.values')) {
      this.get('content.values').forEach(function (value) {
        var expression = this.extractExpressions(value)[0];
        var computedData;
        var datasetKey;

        if (expression) {
          datasetKey = value.value.match(this.get('EXPRESSION_REGEX'))[0];
          computedData = this.computeExpression(expression, metrics)[datasetKey];
          //exclude empty datasets
          if (computedData.length > 0) {
            seriesData.push({
              name: value.name,
              data: computedData
            });
          }
        }
      }, this);
    }
    return seriesData;
  },

  /**
   * compute expression
   *
   * @param {string} expression
   * @param {object} metrics
   * @returns {object}
   */
  computeExpression: function computeExpression(expression, metrics) {
    var validExpression = true,
        value = [],
        dataLinks = {},
        dataLength = -1,
        beforeCompute,
        result = {},
        isDataCorrupted = false,
        isPointNull = false;

    //replace values with metrics data
    expression.match(this.get('VALUE_NAME_REGEX')).forEach(function (match) {
      if (isNaN(match)) {
        if (metrics.someProperty('name', match)) {
          dataLinks[match] = metrics.findProperty('name', match).data;
          if (!isDataCorrupted) {
            isDataCorrupted = dataLength !== -1 && dataLength !== dataLinks[match].length;
          }
          dataLength = dataLinks[match].length > dataLength ? dataLinks[match].length : dataLength;
        } else {
          validExpression = false;
          console.warn('Metrics with name "' + match + '" not found to compute expression');
        }
      }
    });

    if (validExpression) {
      if (isDataCorrupted) {
        this.adjustData(dataLinks, dataLength);
      }
      for (var i = 0, timestamp; i < dataLength; i++) {
        isPointNull = false;
        beforeCompute = expression.replace(this.get('VALUE_NAME_REGEX'), function (match) {
          if (isNaN(match)) {
            timestamp = dataLinks[match][i][1];
            isPointNull = isPointNull ? true : Em.isNone(dataLinks[match][i][0]);
            return dataLinks[match][i][0];
          } else {
            return match;
          }
        });
        var dataLinkPointValue = isPointNull ? null : Number(window.eval(beforeCompute));
        // expression resulting into `0/0` will produce NaN Object which is not a valid series data value for RickShaw graphs
        if (isNaN(dataLinkPointValue)) {
          dataLinkPointValue = 0;
        }
        value.push([dataLinkPointValue, timestamp]);
      }
    }

    result['${' + expression + '}'] = value;
    return result;
  },

  /**
   *  add missing points, with zero value, to series
   *
   * @param {object} dataLinks
   * @param {number} length
   */
  adjustData: function adjustData(dataLinks, length) {
    //series with full data taken as original
    var original = [];
    var substituteValue = null;

    for (var i in dataLinks) {
      if (dataLinks[i].length === length) {
        original = dataLinks[i];
        break;
      }
    }
    original.forEach(function (point, index) {
      for (var i in dataLinks) {
        if (!dataLinks[i][index] || dataLinks[i][index][1] !== point[1]) {
          dataLinks[i].splice(index, 0, [substituteValue, point[1]]);
        }
      }
    }, this);
  },

  /**
   * add time properties
   * @param {Array} metricPaths
   * @returns {Array} result
   */
  addTimeProperties: function addTimeProperties(metricPaths) {
    var toSeconds,
        fromSeconds,
        step = this.get('timeStep'),
        timeRange = this.get('timeRange'),
        result = [],
        targetView = this.get('exportTargetView.isPopup') ? this.get('exportTargetView') : this.get('parentView');

    //if view destroyed then no metrics should be asked
    if (Em.isNone(targetView)) return result;

    if (timeRange === 0 && !Em.isNone(targetView.get('customStartTime')) && !Em.isNone(targetView.get('customEndTime'))) {
      // Custom start/end time is specified by user
      toSeconds = targetView.get('customEndTime') / 1000;
      fromSeconds = targetView.get('customStartTime') / 1000;
    } else {
      // Preset time range is specified by user
      toSeconds = Math.round(App.dateTime() / 1000);
      fromSeconds = toSeconds - timeRange;
    }

    metricPaths.forEach(function (metricPath) {
      result.push(metricPath + '[' + fromSeconds + ',' + toSeconds + ',' + step + ']');
    }, this);

    return result;
  },

  /**
   * @type {Em.View}
   * @class
   */
  graphView: App.ChartLinearTimeView.extend({

    noTitleUnderGraph: true,
    inWidget: true,
    description: Em.computed.alias('parentView.content.description'),
    isPreview: Em.computed.alias('parentView.isPreview'),
    displayUnit: Em.computed.alias('parentView.content.properties.display_unit'),
    setYAxisFormatter: function () {
      var displayUnit = this.get('displayUnit');
      if (displayUnit) {
        this.set('yAxisFormatter', function (value) {
          return App.ChartLinearTimeView.DisplayUnitFormatter(value, displayUnit);
        });
      }
    }.observes('displayUnit'),

    /**
     * set custom time range for graph widget
     */
    setTimeRange: function () {
      if (this.get('isPopup')) {
        if (this.get('currentTimeIndex') === CUSTOM_TIME_INDEX) {
          // Custom start and end time is specified by user
          this.get('parentView').propertyDidChange('customTimeRange');
        } else {
          // Preset time range is specified by user
          this.set('parentView.customTimeRange', this.get('timeUnitSeconds'));
        }
      } else {
        this.set('parentView.customTimeRange', null);
      }
    }.observes('isPopup', 'timeUnitSeconds'),

    /**
     * graph height
     * @type {number}
     */
    height: 95,

    /**
     * @type {string}
     */
    id: function () {
      return 'widget_' + this.get('parentView.content.id') + '_graph';
    }.property('parentView.content.id'),

    /**
     * @type {string}
     */
    renderer: function () {
      return this.get('parentView.content.properties.graph_type') === 'STACK' ? 'area' : 'line';
    }.property('parentView.content.properties.graph_type'),

    title: Em.computed.alias('parentView.content.widgetName'),

    transformToSeries: function transformToSeries(seriesData) {
      var seriesArray = [];

      seriesData.forEach(function (_series) {
        seriesArray.push(this.transformData(_series.data, _series.name));
      }, this);
      return seriesArray;
    },

    loadData: function loadData() {
      var self = this;
      Em.run.next(function () {
        self._refreshGraph(self.get('parentView.data'), self.get('parentView'));
      });
    },

    didInsertElement: function () {
      var self = this;
      this.$().closest('.graph-widget').on('mouseleave', function () {
        self.set('parentView.isExportMenuHidden', true);
      });
      this.setYAxisFormatter();
      if (!arguments.length || this.get('parentView.data.length')) {
        this.loadData();
      }
      Em.run.next(function () {
        if (self.get('isPreview')) {
          App.tooltip(self.$("[rel='ZoomInTooltip']"), 'disable');
        } else {
          App.tooltip(self.$("[rel='ZoomInTooltip']"), {
            placement: 'left',
            template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner graph-tooltip"></div></div>'
          });
        }
      });
    }.observes('parentView.data')
  }),

  getCustomFileName: function getCustomFileName() {
    // get current service name if it exists.
    var currentServiceName = Em.isEmpty(this.get('controller.content.serviceName')) ? "" : this.get('controller.content.serviceName') + '_';
    // serviceName_widgetName_metricName
    return (currentServiceName + this.get('content.widgetName').replace(/\s+/g, '_')).toLowerCase();
  },

  exportGraphData: function exportGraphData(event) {
    this.set('isExportMenuHidden', true);
    var data,
        isCSV = !!event.context,
        fileType = isCSV ? 'csv' : 'json',
        fileName = (Em.isEmpty(this.get('content.widgetName')) ? 'data' : this.getCustomFileName()) + '.' + fileType,
        metrics = this.get('data'),
        hasData = Em.isArray(metrics) && metrics.some(function (item) {
      return Em.isArray(item.data);
    });
    if (hasData) {
      data = isCSV ? this.prepareCSV(metrics) : JSON.stringify(metrics, this.jsonReplacer(), 4);
      fileUtils.downloadTextFile(data, fileType, fileName);
    } else {
      App.showAlertPopup(Em.I18n.t('graphs.noData.title'), Em.I18n.t('graphs.noData.tooltip.title'));
    }
  }
});

});

require.register("views/common/widget/heatmap_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var numberUtils = require('utils/number_utils');

App.HeatmapWidgetView = Em.View.extend(App.WidgetMixin, {
  templateName: require('templates/common/widget/heatmap_widget'),

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: [],

  /**
   *  racks container bound in the template
   *  @type {Array}
   */
  racks: [],

  /**
   * @type {$.ajax|null}
   * @default null
   */
  activeRequest: null,

  onMetricsLoaded: function onMetricsLoaded() {
    if (!this.get('isLoaded')) {
      this.set('controller.inputMaximum', this.get('content.properties.max_limit'));
    }
    this._super();
  },

  willDestroyElement: function willDestroyElement() {
    if ($.isPlainObject(this.get('activeRequest'))) {
      this.get('activeRequest').abort();
      this.set('activeRequest', null);
    }
  },

  getHostComponentsMetrics: function getHostComponentsMetrics(request) {
    var ajax = this._super(request);
    this.set('activeRequest', ajax);
    return ajax;
  },

  getHostsMetrics: function getHostsMetrics(request) {
    var ajax = this._super(request);
    this.set('activeRequest', ajax);
    return ajax;
  },

  /**
   * skip metrics loading if AMBARI METRICS service is not started
   */
  loadMetrics: function loadMetrics() {
    if (App.Service.find('AMBARI_METRICS').get('isStarted')) {
      this._super();
    } else {
      this.onMetricsLoaded();
    }
  },

  /**
   * draw widget
   */
  drawWidget: function () {
    if (this.get('isLoaded')) {
      var hostToValueMap = this.calculateValues();
      var hostNames = [];
      if (this.get('racks').everyProperty('isLoaded', true)) {
        this.get('racks').forEach(function (rack) {
          hostNames = hostNames.concat(rack.hosts.mapProperty('hostName'));
        });
      }

      var metricObject = App.MainChartHeatmapMetric.create({
        name: this.get('content.widgetName'),
        units: this.get('content.properties.display_unit'),
        maximumValue: this.get('controller.inputMaximum'),
        hostNames: hostNames,
        hostToValueMap: hostToValueMap
      });

      this.set('controller.selectedMetric', metricObject);
      App.loadTimer.finish('Heatmaps Page');
      App.loadTimer.finish('Service Heatmaps Page');
    }
  }.observes('racks.@each.isLoaded'),

  /**
   * calculate value for heatmap widgets
   * @returns {Object}
   */
  calculateValues: function calculateValues() {
    return this.computeExpression(this.extractExpressions(this.get('content.values')[0]), this.get('metrics'));
  },

  /**
   * compute expression
   * @param {Array} expressions
   * @param {Array} metrics
   * @returns {object}
   */
  computeExpression: function computeExpression(expressions, metrics) {
    var hostToValueMap = {};
    var hostNames = metrics.mapProperty('hostName');
    var metricsMap = {};

    metrics.forEach(function (_metric) {
      this.convertDataWhenMB(_metric);
      metricsMap[_metric.name + "_" + _metric.hostName] = _metric;
    }, this);

    hostNames.forEach(function (_hostName) {
      expressions.forEach(function (_expression) {
        var validExpression = true;

        //replace values with metrics data
        var beforeCompute = _expression.replace(this.get('VALUE_NAME_REGEX'), function (match) {
          var _metric;
          if (window.isNaN(match)) {
            _metric = metricsMap[match + "_" + _hostName];
            if (_metric) {
              return _metric.data;
            } else {
              validExpression = false;
              console.warn('Metrics with name "' + match + '" not found to compute expression');
            }
          } else {
            return match;
          }
        });

        if (validExpression && this.get('MATH_EXPRESSION_REGEX').test(beforeCompute)) {
          hostToValueMap[_hostName] = Number(window.eval(beforeCompute)).toString();
        } else if (beforeCompute.contains('undefined')) {
          // Data not available
          hostToValueMap[_hostName] = undefined;
        } else {
          console.error('Value for metric is not correct mathematical expression: ' + beforeCompute);
        }
      }, this);
    }, this);

    return hostToValueMap;
  },

  /**
   * If metric has value in MB convert it to bytes
   * @param _metric
   */
  convertDataWhenMB: function convertDataWhenMB(_metric) {
    if (_metric.metric_path.endsWith('M') && !Em.isNone(_metric.data) && isFinite(_metric.data)) {
      _metric.originalData = _metric.data;
      _metric.data = Math.round(_metric.data * numberUtils.BYTES_IN_MB);
    }
  }
});

});

require.register("views/common/widget/number_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NumberWidgetView = Em.View.extend(App.WidgetMixin, {
  templateName: require('templates/common/widget/number_widget'),

  /**
   * @type {string}
   */
  value: '',

  /**
   * @type {string}
   */
  displayValue: function () {
    var value = parseFloat(this.get('value'));
    if (isNaN(value)) return Em.I18n.t('common.na');
    return value + (this.get('content.properties.display_unit') || '');
  }.property('value', 'content.properties.display_unit'),

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: [],

  /**
   * color of content calculated by thresholds
   * @type {string}
   */
  contentColor: function () {
    var value = parseFloat(this.get('value'));
    var threshold1 = parseFloat(this.get('content.properties.warning_threshold'));
    var threshold2 = parseFloat(this.get('content.properties.error_threshold'));

    if (isNaN(value)) {
      return 'grey';
    } else if (isNaN(threshold1) && isNaN(threshold2) || isNaN(threshold1) && value <= threshold2 || isNaN(threshold2) && value <= threshold1 || !isNaN(threshold2) && threshold1 > threshold2 && value > threshold1 || !isNaN(threshold2) && threshold1 < threshold2 && value <= threshold1) {
      return 'green';
    } else if (!isNaN(threshold2) && value.isInRange(threshold1, threshold2) || isNaN(threshold2) && value > threshold1) {
      return 'orange';
    } else {
      return 'red';
    }
  }.property('value', 'content.properties.warning_threshold', 'content.properties.error_threshold')
});

});

require.register("views/common/widget/template_widget_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TemplateWidgetView = Em.View.extend(App.WidgetMixin, {
  templateName: require('templates/common/widget/template_widget'),

  /**
   * @type {string}
   */
  value: '',

  /**
   * @type {string}
   */
  displayValue: function () {
    return this.get('value') ? this.get('value') : Em.I18n.t('common.na');
  }.property('value'),

  /**
   * common metrics container
   * @type {Array}
   */
  metrics: []
});

});

require.register("views/experimental", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.ExperimentalView = Em.View.extend({
  templateName: require('templates/experimental')
});

});

require.register("views/installer", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.InstallerView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/installer')

});

});

require.register("views/loading", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.LoadingView = Em.View.extend({
  tagName: 'h2',
  classNames: ['col-md-9', 'align-center'],
  template: Ember.Handlebars.compile('{{t common.loading.eclipses}}')
});

});

require.register("views/login", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.LoginView = Em.View.extend({

  templateName: require('templates/login'),
  loginTextField: Em.TextField.extend({
    didInsertElement: function didInsertElement() {
      this._super();
      this.$().focus();
    }
  }),
  passTextField: Em.TextField.extend({
    insertNewline: function insertNewline() {
      this.get("controller").submit();
    },
    didInsertElement: function didInsertElement() {
      var self = this;
      this._super();
      this.$().attr('autocomplete', 'new-password');
      // Hack to off autocomplete
      this.$().on('focus', function () {
        self.$().attr('type', 'password');
      });
    }
  })
});

});

require.register("views/main", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainView = Em.View.extend({
  templateName: require('templates/main')
});

});

require.register("views/main/admin", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminView = Em.View.extend({
  templateName: require('templates/main/admin'),
  selectedBinding: 'controller.category',
  categories: function () {
    var items = [];
    if (App.isAuthorized('CLUSTER.VIEW_STACK_DETAILS, CLUSTER.UPGRADE_DOWNGRADE_STACK') || App.get('upgradeInProgress') || App.get('upgradeHolding')) {
      items.push({
        name: 'stackAndUpgrade',
        url: 'stackAndUpgrade.index',
        label: Em.I18n.t('admin.stackUpgrade.title')
      });
    }
    if (App.isAuthorized('SERVICE.SET_SERVICE_USERS_GROUPS') || App.get('upgradeInProgress') || App.get('upgradeHolding')) {
      items.push({
        name: 'adminServiceAccounts',
        url: 'adminServiceAccounts',
        label: Em.I18n.t('common.serviceAccounts'),
        disabled: App.get('upgradeInProgress') || App.get('upgradeHolding')
      });
    }
    if (!App.get('isHadoopWindowsStack') && App.isAuthorized('CLUSTER.TOGGLE_KERBEROS') || App.get('upgradeInProgress') || App.get('upgradeHolding')) {
      if (App.supports.enableToggleKerberos) {
        items.push({
          name: 'kerberos',
          url: 'adminKerberos.index',
          label: Em.I18n.t('common.kerberos'),
          disabled: App.get('upgradeInProgress') || App.get('upgradeHolding')
        });
      }
    }
    if (App.isAuthorized('SERVICE.START_STOP, CLUSTER.MODIFY_CONFIGS') && App.isAuthorized('SERVICE.MANAGE_AUTO_START, CLUSTER.MANAGE_AUTO_START') || App.get('upgradeInProgress') || App.get('upgradeHolding')) {
      if (App.supports.serviceAutoStart) {
        items.push({
          name: 'serviceAutoStart',
          url: 'adminServiceAutoStart',
          label: Em.I18n.t('admin.serviceAutoStart.title'),
          disabled: App.get('upgradeInProgress') || App.get('upgradeHolding')
        });
      }
    }
    return items;
  }.property(''),

  NavItemView: Ember.View.extend({
    tagName: 'li',
    classNameBindings: 'isActive:active isDisabled:disabled'.w(),
    isActive: Em.computed.equalProperties('item', 'parentView.selected'),
    isDisabled: function () {
      return !!this.get('parentView.categories').findProperty('name', this.get('item'))['disabled'];
    }.property('item', 'parentView.categories.@each.disabled')
  }),

  willDestroyElement: function willDestroyElement() {
    //reset selected category in Admin sub-menu after leaving admin section
    this.set('controller.category', null);
  }
});

});

require.register("views/main/admin/federation/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/federation/step1'),

  didInsertElement: function didInsertElement() {
    //todo: provide tooltip with correct message
    // App.popover($("input[type=text]"), {'placement': 'right', 'trigger': 'hover', 'title': this.t('admin.highAvailability.wizard.step1.nameserviceid.tooltip.title'), 'content': this.t('admin.highAvailability.wizard.step1.nameserviceid.tooltip.content')});
  }

});

});

require.register("views/main/admin/federation/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep2View = App.AssignMasterComponentsView.extend({
  title: Em.I18n.t('admin.nameNodeFederation.wizard.step2.header'),
  alertMessage: Em.computed.i18nFormat('admin.nameNodeFederation.wizard.step2.body', 'controller.content.nameServiceId')
});

});

require.register("views/main/admin/federation/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/federation/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },
  curNameNodes: function () {
    return this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').filterProperty('isInstalled', true).mapProperty('hostName');
  }.property('controller.content.masterComponentHosts'),
  addNameNodes: function () {
    return this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').filterProperty('isInstalled', false).mapProperty('hostName');
  }.property('controller.content.masterComponentHosts')
});

});

require.register("views/main/admin/federation/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardStep4View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/federation/step4'),

  headerTitle: Em.I18n.t('admin.nameNodeFederation.wizard.step4.header'),

  noticeInProgress: Em.I18n.t('admin.nameNodeFederation.wizard.step4.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.nameNodeFederation.wizard.step4.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete'),

  labelWidth: 'col-md-5'

});

});

require.register("views/main/admin/federation/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeFederationWizardView = Em.View.extend(App.WizardMenuMixin, App.WizardHostsLoading, {

  templateName: require('templates/main/admin/federation/wizard'),

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 3) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  }
});

});

require.register("views/main/admin/highAvailability/hawq/activateStandby/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/activateStandby/step1')

});

});

require.register("views/main/admin/highAvailability/hawq/activateStandby/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardStep2View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/activateStandby/step2'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  }
});

});

require.register("views/main/admin/highAvailability/hawq/activateStandby/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardStep3View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/activateStandby/step3'),

  headerTitle: Em.I18n.t('admin.activateHawqStandby.wizard.step3.header'),

  noticeInProgress: Em.I18n.t('admin.activateHawqStandby.wizard.step3.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.activateHawqStandby.wizard.step3.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete'),

  labelWidth: 'col-md-5'

});

});

require.register("views/main/admin/highAvailability/hawq/activateStandby/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ActivateHawqStandbyWizardView = Em.View.extend(App.WizardMenuMixin, App.WizardHostsLoading, {

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 3) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  },

  templateName: require('templates/main/admin/highAvailability/hawq/activateStandby/wizard')
});

});

require.register("views/main/admin/highAvailability/hawq/addStandby/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/addStandby/step1')

});

});

require.register("views/main/admin/highAvailability/hawq/addStandby/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep2View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('admin.addHawqStandby.wizard.step2.header'),

  alertMessage: Em.I18n.t('admin.addHawqStandby.wizard.step2.body')

});

});

require.register("views/main/admin/highAvailability/hawq/addStandby/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/addStandby/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  }
});

});

require.register("views/main/admin/highAvailability/hawq/addStandby/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardStep4View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/addStandby/step4'),

  headerTitle: Em.I18n.t('admin.addHawqStandby.wizard.step4.header'),

  noticeInProgress: Em.I18n.t('admin.addHawqStandby.wizard.step4.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.addHawqStandby.wizard.step4.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete'),

  labelWidth: 'col-md-5'

});

});

require.register("views/main/admin/highAvailability/hawq/addStandby/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHawqStandbyWizardView = Em.View.extend(App.WizardMenuMixin, App.WizardHostsLoading, {

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 3) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  },

  templateName: require('templates/main/admin/highAvailability/hawq/addStandby/wizard')
});

});

require.register("views/main/admin/highAvailability/hawq/removeStandby/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/removeStandby/step1')

});

});

require.register("views/main/admin/highAvailability/hawq/removeStandby/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardStep2View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/removeStandby/step2')

});

});

require.register("views/main/admin/highAvailability/hawq/removeStandby/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardStep3View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/hawq/removeStandby/step3'),

  headerTitle: Em.I18n.t('admin.removeHawqStandby.wizard.step3.header'),

  noticeInProgress: Em.I18n.t('admin.removeHawqStandby.wizard.step3.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.removeHawqStandby.wizard.step3.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete'),

  labelWidth: 'col-md-5'

});

});

require.register("views/main/admin/highAvailability/hawq/removeStandby/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RemoveHawqStandbyWizardView = Em.View.extend(App.WizardMenuMixin, App.WizardHostsLoading, {

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 3) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  },

  templateName: require('templates/main/admin/highAvailability/hawq/removeStandby/wizard')
});

});

require.register("views/main/admin/highAvailability/journalNode/progress_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeProgressPageView = Em.View.extend(App.wizardProgressPageViewMixin, {

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  headerTitle: function () {
    var currentStep = App.router.get('manageJournalNodeWizardController.currentStep');
    return Em.I18n.t('admin.manageJournalNode.wizard.step' + currentStep + '.header');
  }.property(),

  noticeInProgress: function () {
    var currentStep = App.router.get('manageJournalNodeWizardController.currentStep');
    return Em.I18n.t('admin.manageJournalNode.wizard.step' + currentStep + '.notice.inProgress');
  }.property(),

  notice: Em.I18n.t('admin.manageJournalNode.wizard.progressPage.notice.inProgress')
});

});

require.register("views/main/admin/highAvailability/journalNode/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
require('views/wizard/step5_view');

App.ManageJournalNodeWizardStep1View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('admin.manageJournalNode.wizard.step1.header'),

  alertMessage: Em.I18n.t('admin.manageJournalNode.wizard.step1.body'),

  isBackButtonVisible: false
});

});

require.register("views/main/admin/highAvailability/journalNode/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep2View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/journalNode/step2'),
  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  journalNodesToAdd: function () {
    return App.router.get('manageJournalNodeWizardController').getJournalNodesToAdd();
  }.property('App.router.clusterController.isHostsLoaded'),

  journalNodesToDelete: function () {
    return App.router.get('manageJournalNodeWizardController').getJournalNodesToDelete();
  }.property('App.router.clusterController.isHostsLoaded'),

  isBackButtonVisible: false
});

});

require.register("views/main/admin/highAvailability/journalNode/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep3View = App.HighAvailabilityWizardStep4View.extend({
  templateName: require('templates/main/admin/highAvailability/journalNode/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').pullCheckPointsStatuses();
  },

  step3BodyText: function () {
    var _Em$I18n$t;

    var nN = this.get('controller.content.activeNN'),
        hdfsUser = this.get('controller.content.hdfsUser'),
        nameSpaces = App.HDFSService.find('HDFS').get('masterComponentGroups').mapProperty('name');
    var formatArgs = [nN.host_name];
    if (nameSpaces.length > 1) {
      formatArgs.push(Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.safeModeText'), nameSpaces.map(function (ns) {
        return Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.safeModeCommand').format(hdfsUser, ns);
      }).join('<br>'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.checkPointText'), nameSpaces.map(function (ns) {
        return Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.checkPointCommand').format(hdfsUser, ns);
      }).join('<br>'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.proceed'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.multipleNameSpaces.recentCheckPoint'));
    } else {
      formatArgs.push(Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.safeModeText'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.safeModeCommand').format(hdfsUser), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.checkPointText'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.checkPointCommand').format(hdfsUser), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.proceed'), Em.I18n.t('admin.manageJournalNode.wizard.step3.body.singleNameSpace.recentCheckPoint'));
    }
    return (_Em$I18n$t = Em.I18n.t('admin.manageJournalNode.wizard.step3.body')).format.apply(_Em$I18n$t, formatArgs);
  }.property('controller.content.masterComponentHosts', 'controller.isHDFSNameSpacesLoaded'),

  nnCheckPointText: function () {
    if (this.get('controller.isHDFSNameSpacesLoaded')) {
      var nameSpaces = App.HDFSService.find('HDFS').get('masterComponentGroups');
      var key = void 0;
      if (nameSpaces.length > 1) {
        key = this.get('controller.isNextEnabled') ? 'admin.manageJournalNode.wizard.step3.checkPointsCreated' : 'admin.manageJournalNode.wizard.step3.checkPointsNotCreated';
      } else {
        key = this.get('controller.isNextEnabled') ? 'admin.highAvailability.wizard.step4.ckCreated' : 'admin.highAvailability.wizard.step4.ckNotCreated';
      }
      return Em.I18n.t(key);
    } else {
      return '';
    }
  }.property('controller.isHDFSNameSpacesLoaded', 'controller.isNextEnabled'),

  errorText: function () {
    if (this.get('controller.isHDFSNameSpacesLoaded')) {
      var nameSpaces = App.HDFSService.find('HDFS').get('masterComponentGroups');
      if (nameSpaces.length > 1) {
        return this.get('controller.isActiveNameNodesStarted') ? '' : Em.I18n.t('admin.manageJournalNode.wizard.step3.error.multipleNameSpaces.nameNodes');
      } else {
        return this.get('controller.isNameNodeStarted') ? '' : Em.I18n.t('admin.highAvailability.wizard.step4.error.nameNode');
      }
    } else {
      return '';
    }
  }.property('controller.isHDFSNameSpacesLoaded', 'controller.isNameNodeStarted', 'controller.isActiveNameNodesStarted')
});

});

require.register("views/main/admin/highAvailability/journalNode/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep4View = App.ManageJournalNodeProgressPageView.extend({
  templateName: require('templates/main/admin/highAvailability/journalNode/step4')
});

});

require.register("views/main/admin/highAvailability/journalNode/step5_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep5View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/journalNode/step5'),

  step5BodyText: function () {
    var existingJournalNode = this.get('controller.content.masterComponentHosts').find(function (hc) {
      return hc.component === 'JOURNALNODE' && hc.isInstalled;
    }),
        nameSpaces = App.HDFSService.find('HDFS').get('masterComponentGroups').mapProperty('name'),
        hdfsSiteConfigs = this.get('controller.content.serviceConfigProperties.items').findProperty('type', 'hdfs-site'),
        configProperties = hdfsSiteConfigs ? hdfsSiteConfigs.properties : {},
        directories = nameSpaces.length > 1 ? nameSpaces.map(function (ns) {
      return configProperties['dfs.journalnode.edits.dir.' + ns];
    }).uniq() : [configProperties['dfs.journalnode.edits.dir']],
        directoriesString = directories.map(function (dir) {
      return '<b>' + dir + '</b>';
    }).join(', ');
    return Em.I18n.t('admin.manageJournalNode.wizard.step5.body').format(existingJournalNode.hostName, directoriesString);
  }.property('controller.content.masterComponentHosts', 'controller.isHDFSNameSpacesLoaded')
});

});

require.register("views/main/admin/highAvailability/journalNode/step6_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep6View = App.ManageJournalNodeProgressPageView.extend({
  templateName: require('templates/main/admin/highAvailability/journalNode/step6')
});

});

require.register("views/main/admin/highAvailability/journalNode/step7_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardStep7View = App.ManageJournalNodeProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/journalNode/step7'),

  submitButtonText: Em.I18n.t('common.done'),

  noticeCompleted: Em.I18n.t('admin.manageJournalNode.wizard.step7.notice.completed')
});

});

require.register("views/main/admin/highAvailability/journalNode/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageJournalNodeWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/admin/highAvailability/journalNode/wizard')

});

});

require.register("views/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/admin/highAvailability/nameNode/rollbackHA/rollback_wizard'),

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 4) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  }

});

});

require.register("views/main/admin/highAvailability/nameNode/rollbackHA/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/rollbackHA/step1'),

  addNNHosts: null,
  sNNHosts: null,
  selectedSNNHost: null,
  selectedAddNNHost: null,

  isLoaded: false,

  loadHostsName: function loadHostsName() {
    App.ajax.send({
      name: 'hosts.all',
      sender: this,
      data: {},
      success: 'loadHostsNameSuccessCallback',
      error: 'loadHostsNameErrorCallback'
    });
  },

  loadHostsNameSuccessCallback: function loadHostsNameSuccessCallback(data) {
    var addNNHosts = App.HostComponent.find().filterProperty('componentName', 'NAMENODE');

    this.secondaryNNHosts = [];
    this.set('selectedSNNHost', this.get('controller.content.sNNHost'));
    this.set('selectedAddNNHost', this.get('controller.content.addNNHost'));

    if (addNNHosts.length == 2) {
      this.set('addNNHosts', addNNHosts.mapProperty('hostName'));
    }
    data.items.forEach(function (host) {
      this.secondaryNNHosts.push(host.Hosts.host_name);
    }, this);
    this.set('sNNHosts', this.secondaryNNHosts);
    this.set('isLoaded', true);
  },
  loadHostsNameErrorCallback: function loadHostsNameErrorCallback() {
    this.set('isLoaded', true);
  },

  didInsertElement: function didInsertElement() {
    this.loadHostsName();
  },

  tipAddNNHost: Em.computed.alias('controller.content.addNNHost'),

  tipSNNHost: Em.computed.alias('controller.content.sNNHost'),

  done: function done() {
    this.get('controller.content').set('selectedSNNHost', this.get('selectedSNNHost'));
    this.get('controller.content').set('selectedAddNNHost', this.get('selectedAddNNHost'));
    App.router.send('next');
  }

});

});

require.register("views/main/admin/highAvailability/nameNode/rollbackHA/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep2View = App.HighAvailabilityWizardStep4View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/rollbackHA/step2'),

  step2BodyText: function () {
    var activeNN = App.HostComponent.find().findProperty('displayNameAdvanced', 'Active NameNode');
    if (!activeNN) {
      activeNN = App.HostComponent.find().findProperty('componentName', 'NAMENODE');
    }
    activeNN = activeNN.get('hostName');
    this.get('controller.content').set('activeNNHost', activeNN);
    return Em.I18n.t('admin.highAvailability.rollback.step2.body').format(this.get('controller.content.hdfsUser'), activeNN);
  }.property()

});

});

require.register("views/main/admin/highAvailability/nameNode/rollbackHA/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RollbackHighAvailabilityWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/rollbackHA/step3'),

  didInsertElement: function didInsertElement() {}

});

});

require.register("views/main/admin/highAvailability/nameNode/rollback_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityRollbackView = App.HighAvailabilityProgressPageView.extend({
  templateName: require('templates/main/admin/highAvailability/nameNode/rollback'),

  notice: Em.I18n.t('admin.highAvailability.rollback.notice.inProgress'),
  submitButtonText: Em.I18n.t('common.done')

});

});

require.register("views/main/admin/highAvailability/nameNode/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step1'),

  didInsertElement: function didInsertElement() {
    App.popover($("input[type=text]"), { 'placement': 'right', 'trigger': 'hover', 'title': this.t('admin.highAvailability.wizard.step1.nameserviceid.tooltip.title'), 'content': this.t('admin.highAvailability.wizard.step1.nameserviceid.tooltip.content') });
  },

  showInputError: Em.computed.and('!controller.isNameServiceIdValid', 'controller.content.nameServiceId.length')

});

});

require.register("views/main/admin/highAvailability/nameNode/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/wizard/step5_view');

App.HighAvailabilityWizardStep2View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('admin.highAvailability.wizard.step2.header'),

  alertMessage: Em.I18n.t('admin.highAvailability.wizard.step2.body')

});

});

require.register("views/main/admin/highAvailability/nameNode/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step3'),
  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },
  curNameNode: function () {
    return this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true).hostName;
  }.property('controller.content.masterComponentHosts'),
  addNameNode: function () {
    return this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false).hostName;
  }.property('controller.content.masterComponentHosts'),
  secondaryNameNode: function () {
    return this.get('controller.content.masterComponentHosts').findProperty('component', "SECONDARY_NAMENODE").hostName;
  }.property('controller.content.masterComponentHosts'),

  journalNodes: Em.computed.filterBy('controller.content.masterComponentHosts', 'component', 'JOURNALNODE')

});

});

require.register("views/main/admin/highAvailability/nameNode/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep4View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step4'),

  didInsertElement: function didInsertElement() {
    this.get('controller').pullCheckPointStatus();
  },

  step4BodyText: function () {
    var nN = this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true);
    return Em.I18n.t('admin.highAvailability.wizard.step4.body').format(this.get('controller.content.hdfsUser'), nN.hostName);
  }.property('controller.content.masterComponentHosts'),

  nnCheckPointText: Em.computed.ifThenElse('controller.isNextEnabled', Em.I18n.t('admin.highAvailability.wizard.step4.ckCreated'), Em.I18n.t('admin.highAvailability.wizard.step4.ckNotCreated'))

});

});

require.register("views/main/admin/highAvailability/nameNode/step5_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep5View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step5')
});

});

require.register("views/main/admin/highAvailability/nameNode/step6_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep6View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step6'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  step6BodyText: function () {
    var nN = this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true);
    return Em.I18n.t('admin.highAvailability.wizard.step6.body').format(this.get('controller.content.hdfsUser'), nN.hostName);
  }.property('controller.content.masterComponentHosts'),

  jnCheckPointText: function () {
    switch (this.get('controller.status')) {
      case 'waiting':
        return Em.I18n.t('admin.highAvailability.wizard.step6.jsNoInit');
      case 'done':
        return Em.I18n.t('admin.highAvailability.wizard.step6.jsInit');
      case 'journalnode_stopped':
        return Em.I18n.t('admin.highAvailability.wizard.step6.jnStopped');
      default:
        return Em.I18n.t('admin.highAvailability.wizard.step6.jsNoInit');
    }
  }.property('controller.status')

});

});

require.register("views/main/admin/highAvailability/nameNode/step7_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep7View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step7')
});

});

require.register("views/main/admin/highAvailability/nameNode/step8_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep8View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step8'),

  step8BodyText: function () {
    var nN = this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', true);
    var addNN = this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').findProperty('isInstalled', false);
    return Em.I18n.t('admin.highAvailability.wizard.step8.body').format(this.get('controller.content.hdfsUser'), nN.hostName, addNN.hostName);
  }.property('controller.content.masterComponentHosts')

});

});

require.register("views/main/admin/highAvailability/nameNode/step9_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardStep9View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/nameNode/step9'),

  submitButtonText: Em.I18n.t('common.done'),

  noticeCompleted: Em.I18n.t('admin.highAvailability.wizard.step9.notice.completed')
});

});

require.register("views/main/admin/highAvailability/nameNode/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/admin/highAvailability/nameNode/wizard'),

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 4) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  }

});

});

require.register("views/main/admin/highAvailability/progress_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HighAvailabilityProgressPageView = Em.View.extend(App.wizardProgressPageViewMixin, {

  /**
   * @type {string}
   */
  notice: Em.I18n.t('admin.highAvailability.wizard.progressPage.notice.inProgress'),

  /**
   * @type {string}
   */
  noticeClass: 'alert alert-info',

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  /**
   * @type {string}
   */
  headerTitle: function () {
    var currentStep = App.router.get('highAvailabilityWizardController.currentStep');
    if (currentStep === 1) {
      return Em.I18n.t('admin.highAvailability.wizard.rollback.header.title');
    } else {
      return Em.I18n.t('admin.highAvailability.wizard.step' + currentStep + '.header');
    }
  }.property(),

  /**
   * @type {string}
   */
  noticeInProgress: function () {
    var currentStep = App.router.get('highAvailabilityWizardController.currentStep');
    if (currentStep === 1) {
      return Em.I18n.t('admin.highAvailability.rollback.notice.inProgress');
    } else {
      return Em.I18n.t('admin.highAvailability.wizard.progressPage.notice.inProgress');
    }
  }.property(),

  onStatusChange: function () {
    var status = this.get('controller.status');
    if (status === 'COMPLETED') {
      this.set('notice', this.get('noticeCompleted'));
      this.set('noticeClass', 'alert alert-success');
    } else if (status === 'FAILED') {
      this.set('notice', this.get('noticeFailed'));
      this.set('noticeClass', 'alert alert-danger');
    } else {
      this.set('notice', this.get('noticeInProgress'));
      this.set('noticeClass', 'alert alert-info');
    }
  }.observes('controller.status'),

  /**
   * @type {Em.View}
   */
  taskView: Em.View.extend({
    icon: '',
    iconColor: '',
    linkClass: '',

    didInsertElement: function didInsertElement() {
      this.onStatus();
      $('body').tooltip({
        selector: '[rel=tooltip]'
      });
    },

    barWidth: Em.computed.format('width: {0}%;', 'content.progress'),

    onStatus: function () {
      this.set('linkClass', Boolean(this.get('content.requestIds.length')) ? 'active-link' : 'active-text');
      if (this.get('content.status') === 'IN_PROGRESS') {
        this.set('icon', 'glyphicon glyphicon-cog');
        this.set('iconColor', 'text-info');
      } else if (this.get('content.status') === 'FAILED') {
        this.set('icon', 'glyphicon glyphicon-exclamation-sign');
        this.set('iconColor', 'text-danger');
      } else if (this.get('content.status') === 'COMPLETED') {
        this.set('icon', 'glyphicon glyphicon-ok');
        this.set('iconColor', 'text-success');
      } else {
        this.set('icon', 'glyphicon glyphicon-cog');
        this.set('iconColor', '');
        this.set('linkClass', 'not-active-link');
      }
    }.observes('content.status', 'content.hosts.length'),

    showProgressBar: Em.computed.equal('content.status', 'IN_PROGRESS'),

    hidePercent: Em.computed.equal('content.command', 'testDBConnection'),

    showDBTooltip: Em.computed.not('hidePercent')
  })
});

});

require.register("views/main/admin/highAvailability/rangerAdmin/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/rangerAdmin/step1')

});

});

require.register("views/main/admin/highAvailability/rangerAdmin/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep2View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('admin.ra_highAvailability.wizard.step2.header'),

  alertMessage: Em.I18n.t('admin.ra_highAvailability.wizard.step2.body')

});

});

require.register("views/main/admin/highAvailability/rangerAdmin/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/rangerAdmin/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  }

});

});

require.register("views/main/admin/highAvailability/rangerAdmin/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardStep4View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/rangerAdmin/step4'),

  headerTitle: Em.I18n.t('admin.ra_highAvailability.wizard.step4.header'),

  noticeInProgress: Em.I18n.t('admin.ra_highAvailability.wizard.step4.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.ra_highAvailability.wizard.step4.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete')

});

});

require.register("views/main/admin/highAvailability/rangerAdmin/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RAHighAvailabilityWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/admin/highAvailability/rangerAdmin/wizard')
});

});

require.register("views/main/admin/highAvailability/resourceManager/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/resourceManager/step1')

});

});

require.register("views/main/admin/highAvailability/resourceManager/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep2View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('admin.rm_highAvailability.wizard.step2.header'),

  alertMessage: Em.I18n.t('admin.rm_highAvailability.wizard.step2.body')

});

});

require.register("views/main/admin/highAvailability/resourceManager/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep3View = Em.View.extend({

  templateName: require('templates/main/admin/highAvailability/resourceManager/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  }
});

});

require.register("views/main/admin/highAvailability/resourceManager/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardStep4View = App.HighAvailabilityProgressPageView.extend({

  templateName: require('templates/main/admin/highAvailability/resourceManager/step4'),

  headerTitle: Em.I18n.t('admin.rm_highAvailability.wizard.step4.header'),

  noticeInProgress: Em.I18n.t('admin.rm_highAvailability.wizard.step4.notice.inProgress'),

  noticeCompleted: Em.I18n.t('admin.rm_highAvailability.wizard.step4.notice.completed'),

  submitButtonText: Em.I18n.t('common.complete'),

  labelWidth: 'col-md-5'

});

});

require.register("views/main/admin/highAvailability/resourceManager/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RMHighAvailabilityWizardView = Em.View.extend(App.WizardMenuMixin, App.WizardHostsLoading, {

  templateName: require('templates/main/admin/highAvailability/resourceManager/wizard'),

  didInsertElement: function didInsertElement() {
    var currentStep = this.get('controller.currentStep');
    if (currentStep > 3) {
      this.get('controller').setLowerStepsDisable(currentStep);
    }
  }
});

});

require.register("views/main/admin/kerberos", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminKerberosView = Em.View.extend({
  templateName: require('templates/main/admin/kerberos'),
  didInsertElement: function didInsertElement() {
    var controller = this.get('controller');
    controller.set('dataIsLoaded', false);
    controller.loadSecurityStatusFromServer();
  }
});

});

require.register("views/main/admin/kerberos/disable_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/main/admin/kerberos/progress_view');

App.KerberosDisableView = App.KerberosProgressPageView.extend({

  didInsertElement: function didInsertElement() {
    this._super();
    this.get('controller').loadStep();
  },

  headerTitle: '',

  noticeCompleted: Em.I18n.t('admin.kerberos.disable.notice.completed'),

  notice: Em.I18n.t('admin.kerberos.disable.inProgress'),

  templateName: require('templates/main/admin/kerberos/disable'),

  msgColor: 'alert-info',

  isSimpleModal: true,

  disableKerberos: true
});

});

require.register("views/main/admin/kerberos/progress_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosProgressPageView = Em.View.extend(App.wizardProgressPageViewMixin, {

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  headerTitle: function () {
    var currentStep = App.router.get('kerberosWizardController.currentStep');
    return Em.I18n.t('admin.kerberos.wizard.step' + currentStep + '.header');
  }.property(),

  noticeInProgress: function () {
    var currentStep = App.router.get('kerberosWizardController.currentStep');
    return Em.I18n.t('admin.kerberos.wizard.step' + currentStep + '.notice.inProgress');
  }.property(),

  notice: Em.I18n.t('admin.kerberos.wizard.progressPage.notice.inProgress')
});

});

require.register("views/main/admin/kerberos/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep1View = Em.View.extend({

  templateName: require('templates/main/admin/kerberos/step1'),

  didInsertElement: function didInsertElement() {
    var controller = this.get('controller');
    controller.loadStep();
  }

});

});

require.register("views/main/admin/kerberos/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

require('views/wizard/step7_view');

/**
 * controller.loadStep() called from internal view App.ServicesConfigView
 */
App.KerberosWizardStep2View = App.WizardStep7View.extend({

  templateName: require('templates/main/admin/kerberos/step2')
});

});

require.register("views/main/admin/kerberos/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep3View = App.KerberosProgressPageView.extend({

  templateName: require('templates/main/admin/kerberos/step3'),

  noticeCompleted: Em.I18n.t('admin.kerberos.wizard.step3.notice.completed'),

  submitButtonText: Em.I18n.t('common.next') + '&rarr;',

  showBackButton: true,

  isHostHeartbeatLost: function () {
    return !Em.isEmpty(this.get('controller.heartBeatLostHosts'));
  }.property('controller.heartBeatLostHosts.length'),

  resultMsg: function () {
    if (this.get('isHostHeartbeatLost')) {
      return Em.I18n.t('installer.step9.status.hosts.heartbeat_lost').format(this.get('controller.heartBeatLostHosts.length'));
    }
    return '';
  }.property('isHostHeartbeatLost'),

  showHostsWithLostHeartBeat: function showHostsWithLostHeartBeat() {
    var self = this;
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step9.host.heartbeat_lost.header'),
      autoHeight: false,
      secondary: null,
      bodyClass: Em.View.extend({
        hosts: self.get('controller.heartBeatLostHosts'),
        template: Em.Handlebars.compile('{{#each hostName in view.hosts}}<p>{{view.hostName}}</p>{{/each}}')
      })
    });
  }
});

});

require.register("views/main/admin/kerberos/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep4View = Em.View.extend({

  templateName: require('templates/main/admin/kerberos/step4')

});

});

require.register("views/main/admin/kerberos/step5_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep5View = App.KerberosProgressPageView.extend({

  templateName: require('templates/main/admin/kerberos/step5')
});

});

require.register("views/main/admin/kerberos/step6_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep6View = App.KerberosProgressPageView.extend({

  templateName: require('templates/main/admin/kerberos/step6'),

  noticeCompleted: Em.I18n.t('admin.kerberos.wizard.step6.notice.completed'),

  submitButtonText: Em.I18n.t('common.next') + '&rarr;',

  showBackButton: true
});

});

require.register("views/main/admin/kerberos/step7_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardStep7View = App.KerberosProgressPageView.extend({

  templateName: require('templates/main/admin/kerberos/step7'),

  noticeCompleted: Em.I18n.t('admin.kerberos.wizard.step7.notice.completed'),

  submitButtonText: Em.I18n.t('common.next') + '&rarr;',

  showBackButton: true

});

});

require.register("views/main/admin/kerberos/step8_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.KerberosWizardStep8View = App.KerberosProgressPageView.extend({

  templateName: require('templates/main/admin/kerberos/step8'),

  noticeCompleted: Em.I18n.t('admin.kerberos.wizard.step8.notice.completed'),

  noticeFailed: Em.I18n.t('admin.kerberos.wizard.step8.notice.failed'),

  submitButtonText: Em.I18n.t('common.complete'),

  showBackButton: false
});

});

require.register("views/main/admin/kerberos/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.KerberosWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/admin/kerberos/wizard')
});

});

require.register("views/main/admin/serviceAccounts_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminServiceAccountsView = Em.View.extend({
  templateName: require('templates/main/admin/serviceAccounts'),
  didInsertElement: function didInsertElement() {
    var self = this;
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isConfigsPropertiesLoaded').done(function () {
      self.get('controller').loadUsers();
    });
  }
});

});

require.register("views/main/admin/service_auto_start", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminServiceAutoStartView = Em.View.extend({
  templateName: require('templates/main/admin/service_auto_start'),

  /**
   * @type {boolean}
   */
  isSaveDisabled: Em.computed.or('!controller.isModified', 'controller.saveInProgress'),

  /**
   * @type {boolean}
   * @default false
   */
  isDisabled: false,

  /**
   * @type {boolean}
   */
  skipCyclicCall: false,

  /**
   * @type {boolean}
   */
  allComponentsChecked: false,

  /**
   * @type {?object}
   */
  switcher: null,

  didInsertElement: function didInsertElement() {
    this.set('isDisabled', !App.isAuthorized('CLUSTER.MANAGE_AUTO_START'));
    this.get('controller').load();
  },

  onValueChange: function () {
    if (this.get('switcher')) {
      this.get('switcher').bootstrapSwitch('state', this.get('controller.isGeneralRecoveryEnabled'));
    }
  }.observes('controller.isGeneralRecoveryEnabled'),

  /**
   * Init switcher plugin.
   *
   * @method initSwitcher
   */
  initSwitcher: function initSwitcher() {
    var self = this.get('parentView');
    if (self.get('controller.isLoaded')) {
      self.set('switcher', $(".general-auto-start>input").bootstrapSwitch({
        state: self.get('controller.isGeneralRecoveryEnabled'),
        onText: Em.I18n.t('common.enabled'),
        offText: Em.I18n.t('common.disabled'),
        offColor: 'default',
        onColor: 'success',
        disabled: self.get('isDisabled'),
        handleWidth: Math.max(Em.I18n.t('common.enabled').length, Em.I18n.t('common.disabled').length) * 8,
        onSwitchChange: function onSwitchChange(event, state) {
          self.set('controller.isGeneralRecoveryEnabled', state);
        }
      }));
    }
  },

  observeAllComponentsChecked: function () {
    if (this.get('skipCyclicCall')) {
      this.set('skipCyclicCall', false);
    } else {
      this.get('controller.componentsConfigsGrouped').setEach('recoveryEnabled', this.get('allComponentsChecked'));
    }
  }.observes('allComponentsChecked'),

  observesEachComponentChecked: function () {
    var components = this.get('controller.componentsConfigsGrouped');
    if (this.get('allComponentsChecked') && components.someProperty('recoveryEnabled', false)) {
      this.set('skipCyclicCall', true);
      this.set('allComponentsChecked', false);
    } else if (!this.get('allComponentsChecked') && components.everyProperty('recoveryEnabled', true)) {
      this.set('skipCyclicCall', true);
      this.set('allComponentsChecked', true);
    }
  }.observes('controller.componentsConfigsGrouped.@each.recoveryEnabled')

});

});

require.register("views/main/admin/stack_and_upgrade_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminStackAndUpgradeView = Em.View.extend({
  templateName: require('templates/main/admin/stack_and_upgrade')
});

});

require.register("views/main/admin/stack_upgrade/failed_hosts_modal_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FailedHostsPopupBodyView = Em.View.extend({

  templateName: require('templates/main/admin/stack_upgrade/failed_hosts_modal'),

  /**
   * @type {number}
   * @const
   */
  MAX_HOSTNAME_LENGTH: 50,

  /**
   * @type {string}
   */
  subHeader: Em.computed.i18nFormat('admin.stackUpgrade.failedHosts.subHeader', 'parentView.content.hosts.length'),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$("[rel='UsageTooltip']"));
    this.$(".panel").on("show hide", function (e) {
      $(e.target).siblings(".panel-heading").find("i.panel-toggle").toggleClass('icon-caret-right icon-caret-down');
    });
  },

  /**
   * @type {Array.<Em.Object>}
   */
  hosts: function () {
    var content = this.get('parentView.content');
    var result = [];

    content.hosts.forEach(function (hostName, index) {
      var hostComponents = [];

      if (content.host_detail[hostName]) {
        content.host_detail[hostName].forEach(function (details) {
          hostComponents.push(Em.Object.create({
            componentName: App.format.role(details.component, false),
            serviceName: App.format.role(details.service, true)
          }));
        }, this);
      }
      result.push(Em.Object.create({
        hostName: hostName,
        displayName: hostName.length > this.MAX_HOSTNAME_LENGTH ? hostName.substr(0, this.MAX_HOSTNAME_LENGTH) + '...' : hostName,
        collapseId: 'collapse' + index,
        collapseHref: '#collapse' + index,
        hostComponents: hostComponents
      }));
    }, this);
    return result;
  }.property('parentView.content'),

  /**
   * open hosts info in new window in JSON format
   */
  openDetails: function openDetails() {
    var newDocument = window.open().document;
    newDocument.write(JSON.stringify(this.get('parentView.content')));
    newDocument.close();
  }
});

});

require.register("views/main/admin/stack_upgrade/menu_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminStackMenuView = Em.CollectionView.extend({
  tagName: 'ul',
  classNames: ["nav", "nav-tabs", "background-text"],
  defaultRoute: 'services',

  content: function () {
    return [Em.Object.create({
      name: 'services',
      label: Em.I18n.t('common.stack'),
      routing: 'services'
    }), Em.Object.create({
      name: 'versions',
      label: Em.I18n.t('common.versions'),
      routing: 'versions',
      hidden: !App.get('stackVersionsAvailable')
    }), Em.Object.create({
      name: 'upgradeHistory',
      label: Em.I18n.t('common.upgrade.history'),
      routing: 'upgradeHistory',
      hidden: !App.get('upgradeHistoryAvailable')
    })];
  }.property('App.stackVersionsAvailable'),

  didInsertElement: function didInsertElement() {
    this.activateView();
  },

  activateView: function activateView() {
    var defaultRoute = App.router.get('currentState.name') || this.get('defaultRoute');
    $.each(this._childViews, function () {
      this.set('active', this.get('content.routing') == defaultRoute ? "active" : "");
    });
  },

  deactivateChildViews: function deactivateChildViews() {
    $.each(this._childViews, function () {
      this.set('active', "");
    });
  },

  itemViewClass: Em.View.extend({
    classNameBindings: ["active"],
    active: "",
    template: Ember.Handlebars.compile('{{#unless view.content.hidden}}<a {{action stackNavigate view.content.routing }} href="#"> {{unbound view.content.label}}</a>{{/unless}}')
  })
});

});

require.register("views/main/admin/stack_upgrade/services_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAdminStackServicesView = Em.View.extend({
  templateName: require('templates/main/admin/stack_upgrade/services'),

  isAddServiceAvailable: function () {
    return App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK');
  }.property('App.supports.opsDuringRollingUpgrade', 'App.upgradeState', 'App.isAdmin'),

  /**
   * @type {Array}
   */
  services: function () {
    var services = App.supports.installGanglia ? App.StackService.find() : App.StackService.find().without(App.StackService.find('GANGLIA'));
    var controller = this.get('controller');

    services.map(function (s) {
      s.set('serviceVersionDisplay', controller.get('serviceVersionsMap')[s.get('serviceName')]);
      s.set('isInstalled', App.Service.find().someProperty('serviceName', s.get('serviceName')));
      return s;
    });
    return services;
  }.property('App.router.clusterController.isLoaded', 'controller.serviceVersionsMap'),

  didInsertElement: function didInsertElement() {
    if (!App.get('stackVersionsAvailable')) {
      this.get('controller').loadStackVersionsToModel(true).done(function () {
        App.set('stackVersionsAvailable', App.StackVersion.find().content.length > 0);
      });
      this.get('controller').loadRepositories();
    }
  },

  /**
   * launch Add Service wizard
   * @param event
   */
  goToAddService: function goToAddService(event) {
    if (!App.isAuthorized('SERVICE.ADD_DELETE_SERVICES') || !App.supports.enableAddDeleteServices) {
      return;
    } else if (event.context == "KERBEROS") {
      App.router.get('mainAdminKerberosController').checkAndStartKerberosWizard();
      App.router.get('kerberosWizardController').setDBProperty('onClosePath', 'main.admin.stackAndUpgrade.services');
    } else {
      App.router.get('addServiceController').set('serviceToInstall', event.context);
      App.router.get('addServiceController').setDBProperty('onClosePath', 'main.admin.stackAndUpgrade.services');
      App.get('router').transitionTo('main.serviceAdd');
    }
  },

  /**
   * List of all repo-groups
   * @type {Object[][]}
   */
  allRepositoriesGroups: function () {
    var repos = this.get('controller.allRepos');
    var reposGroup = [];
    var repositories = [];
    reposGroup.set('stackVersion', App.get('currentStackVersionNumber'));
    if (repos) {
      repos.forEach(function (group) {
        group.repositories.forEach(function (repo) {
          var cur_repo = Em.Object.create({
            'repoId': repo.repoId,
            'id': repo.repoId + '-' + repo.osType,
            'repoName': repo.repoName,
            'stackName': repo.stackName,
            'stackVersion': repo.stackVersion,
            'baseUrl': repo.baseUrl,
            'originalBaseUrl': repo.baseUrl,
            'osType': repo.osType,
            'onEdit': false,
            'empty-error': !repo.baseUrl,
            'undo': false,
            'clearAll': repo.baseUrl
          });
          var cur_group = reposGroup.findProperty('name', group.name);
          if (!cur_group) {
            cur_group = Ember.Object.create({
              name: group.name,
              repositories: []
            });
            reposGroup.push(cur_group);
          }
          cur_group.repositories.push(cur_repo);
          repositories.push(cur_repo);
        });
      });
    }
    this.set('allRepos', repositories);
    return reposGroup;
  }.property('controller.allRepos'),

  /**
   * Onclick handler for edit action of each repo, enter edit mode
   * @param {object} event
   */
  onEditClick: function onEditClick(event) {
    var targetRepo = this.get('allRepos').findProperty('id', event.context.get('id'));
    if (targetRepo) {
      targetRepo.set('onEdit', true);
    }
  },

  /**
   * Onclick handler for undo action of each repo group
   * @method undoGroupLocalRepository
   * @param {object} event
   */
  undoGroupLocalRepository: function undoGroupLocalRepository(event) {
    this.doActionForGroupLocalRepository(event, 'originalBaseUrl');
  },

  /**
   * Handler for clear icon click
   * @method clearGroupLocalRepository
   * @param {object} event
   */
  clearGroupLocalRepository: function clearGroupLocalRepository(event) {
    this.doActionForGroupLocalRepository(event, '');
  },

  /**
   * Common handler for repo groups actions
   * @method doActionForGroupLocalRepository
   * @param {object} event
   * @param {string} newBaseUrlField
   */
  doActionForGroupLocalRepository: function doActionForGroupLocalRepository(event, newBaseUrlField) {
    var targetRepo = this.get('allRepos').findProperty('id', event.context.get('id'));
    if (targetRepo) {
      targetRepo.set('baseUrl', Em.isEmpty(newBaseUrlField) ? '' : Em.get(targetRepo, newBaseUrlField));
    }
  },

  /**
   * Handler when editing any repo group BaseUrl
   * @method editGroupLocalRepository
   */
  editGroupLocalRepository: function () {
    var repos = this.get('allRepos');
    repos.forEach(function (targetRepo) {
      targetRepo.set('undo', targetRepo.get('baseUrl') != targetRepo.get('originalBaseUrl'));
      targetRepo.set('clearAll', targetRepo.get('baseUrl'));
      targetRepo.set('empty-error', !targetRepo.get('baseUrl'));
    });
  }.observes('allRepos.@each.baseUrl'),

  /**
   * onSuccess callback for save Repo URL.
   */
  doSaveRepoUrlsSuccessCallback: function doSaveRepoUrlsSuccessCallback(response, request, data) {
    var id = data.repoId + '-' + data.osType;
    var targetRepo = this.get('allRepos').findProperty('id', id);
    if (!targetRepo) {
      return;
    } else {

      var modalCloseHandler = function modalCloseHandler() {
        this.hide();
        targetRepo.set('baseUrl', data.data.Repositories.base_url);
        targetRepo.set('originalBaseUrl', data.data.Repositories.base_url);
        targetRepo.set('onEdit', false);
      };

      App.ModalPopup.show({
        header: Em.I18n.t('admin.cluster.repositories.popup.header.success'),
        secondary: null,
        onPrimary: modalCloseHandler,
        onClose: modalCloseHandler,
        message: Em.I18n.t('admin.cluster.repositories.popup.body.success'),
        bodyClass: Em.View.extend({
          template: Em.Handlebars.compile('<div class="alert alert-success">{{{message}}}</div>')
        })
      });
    }
  },

  /**
   * onError callback for save Repo URL.
   */
  doSaveRepoUrlsErrorCallback: function doSaveRepoUrlsErrorCallback(request, ajaxOptions, error, data) {
    var self = this;
    var id = data.url.split('/')[10] + '-' + data.url.split('/')[8];
    var targetRepo = this.get('allRepos').findProperty('id', id);
    if (targetRepo) {
      App.ModalPopup.show({
        header: Em.I18n.t('admin.cluster.repositories.popup.header.fail'),
        primary: Em.I18n.t('common.saveAnyway'),
        secondary: Em.I18n.t('common.revert'),
        third: Em.I18n.t('common.cancel'),
        onPrimary: function onPrimary() {
          // save anyway: Go ahead and save with Repo URL validation turned off and close Dialog when done.
          this.hide();
          self.doSaveRepoUrls(id, false);
        },
        onSecondary: function onSecondary() {
          // Revert: Close dialog, revert URL value, go back to non-Edit mode
          this.hide();
          targetRepo.set('baseUrl', targetRepo.get('originalBaseUrl'));
          targetRepo.set('onEdit', false);
        },
        onThird: function onThird() {
          // cancel: Close dialog but stay in Edit mode
          this.hide();
        },
        message: Em.I18n.t('admin.cluster.repositories.popup.body.fail'),
        bodyClass: Em.View.extend({
          template: Em.Handlebars.compile('<div class="alert alert-warning">{{{message}}}</div>')
        })
      });
    }
  },

  /**
   * Check validation and Save the customized local urls
   */
  doSaveRepoUrls: function doSaveRepoUrls(id, verifyBaseUrl) {
    var targetRepo = this.get('allRepos').findProperty('id', id);
    App.ajax.send({
      name: 'wizard.advanced_repositories.valid_url',
      sender: this,
      data: {
        stackName: targetRepo.stackName,
        stackVersion: targetRepo.stackVersion,
        repoId: targetRepo.repoId,
        osType: targetRepo.osType,
        data: {
          'Repositories': {
            'base_url': targetRepo.baseUrl,
            "verify_base_url": verifyBaseUrl
          }
        }
      },
      success: 'doSaveRepoUrlsSuccessCallback',
      error: 'doSaveRepoUrlsErrorCallback'
    });
  },
  /**
   * Check validation and Save the customized local urls
   */
  saveRepoUrls: function saveRepoUrls(event) {
    this.doSaveRepoUrls(event.context.get('id'), true);
  },

  /**
   * on click handler 'Cancel' for current repo in edit mode
   */
  doCancel: function doCancel(event) {
    var targetRepo = this.get('allRepos').findProperty('id', event.context.get('id'));
    if (targetRepo) {
      targetRepo.set('baseUrl', targetRepo.get('originalBaseUrl'));
      targetRepo.set('onEdit', false);
    }
  }
});

});

require.register("views/main/admin/stack_upgrade/upgrade_group_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.upgradeGroupView = Em.View.extend({
  templateName: require('templates/main/admin/stack_upgrade/upgrade_group'),

  /**
   * customized view of progress bar that show completed/total counters
   * @class App.ProgressBarView
   */
  progressBarView: App.ProgressBarView.extend({
    classNames: ['progress-counter'],
    template: Ember.Handlebars.compile('<div class="progress-bar" {{bindAttr style="view.progressWidth"}}></div>' + '<div class="counters-label">{{view.completedTasks}}/{{view.totalTasks}}</div>')
  }),

  /**
   * Only one UpgradeGroup or UpgradeItem could be expanded at a time
   * @param {object} event
   */
  toggleExpanded: function toggleExpanded(event) {
    var isExpanded = event.context.get('isExpanded');
    event.contexts[1].filterProperty('isExpanded').forEach(function (item) {
      this.collapseLowerLevels(item);
      item.set('isExpanded', false);
    }, this);
    this.collapseLowerLevels(event.context);
    event.context.set('isExpanded', !isExpanded);
    if (!isExpanded && event.context.get('type') === 'ITEM') {
      this.doPolling(event.context);
    }
  },

  /**
   * collapse sub-entities of current
   * @param {App.upgradeEntity} entity
   */
  collapseLowerLevels: function collapseLowerLevels(entity) {
    if (entity.get('isExpanded')) {
      if (entity.type === 'ITEM') {
        entity.get('tasks').setEach('isExpanded', false);
      } else if (entity.type === 'GROUP') {
        entity.get('upgradeItems').forEach(function (item) {
          this.collapseLowerLevels(item);
          item.set('isExpanded', false);
        }, this);
      }
    }
  },

  /**
   * poll for tasks when item is expanded
   */
  doPolling: function doPolling(item) {
    var self = this;

    if (item && item.get('isExpanded')) {
      this.get('controller').getUpgradeItem(item).complete(function () {
        if (!item.get('isCompleted')) {
          self.set('timer', setTimeout(function () {
            self.doPolling(item);
          }, App.bgOperationsUpdateInterval));
        }
      });
    } else {
      clearTimeout(this.get('timer'));
    }
  },

  willDestroyElement: function willDestroyElement() {
    clearTimeout(this.get('timer'));
  }
});

});

require.register("views/main/admin/stack_upgrade/upgrade_history_details_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');

App.MainAdminStackUpgradeHistoryDetailsView = Em.View.extend({
  controllerBinding: 'App.router.mainAdminStackUpgradeHistoryController',
  templateName: require('templates/main/admin/stack_upgrade/upgrade_history_details'),
  isReady: false,

  willInsertElement: function willInsertElement() {
    var self = this;
    this.get('controller').loadStackUpgradeRecord().done(function () {
      self.populateUpgradeHistoryRecord();
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.set('isReady', false);
  },

  populateUpgradeHistoryRecord: function populateUpgradeHistoryRecord() {
    var upgradeData = this.get('controller').get('upgradeData');
    this.set('isReady', upgradeData != null);
  },

  /**
   * progress value is rounded to floor
   * @type {number}
   */
  overallProgress: function () {
    return Math.floor(this.get('controller.upgradeData.Upgrade.progress_percent'));
  }.property('controller.upgradeData.Upgrade.progress_percent'),

  /**
   * label of Upgrade status
   * @type {string}
   */
  upgradeStatusLabel: function () {
    var labelKey = null;
    switch (this.get('controller.upgradeData.Upgrade.request_status')) {
      case 'QUEUED':
      case 'PENDING':
      case 'IN_PROGRESS':
        labelKey = 'admin.stackUpgrade.state.inProgress';
        break;
      case 'COMPLETED':
        labelKey = 'admin.stackUpgrade.state.completed';
        break;
      case 'ABORTED':
        labelKey = 'admin.stackUpgrade.state.paused';
        break;
      case 'TIMEDOUT':
      case 'FAILED':
      case 'HOLDING_FAILED':
      case 'HOLDING_TIMEDOUT':
      case 'HOLDING':
        labelKey = 'admin.stackUpgrade.state.paused';
        break;
    }
    if (labelKey) {
      labelKey += this.get('controller.isDowngrade') ? '.downgrade' : "";
      return Em.I18n.t(labelKey);
    } else {
      return "";
    }
  }.property('controller.upgradeData.Upgrade.request_status', 'controller.isDowngrade')
});

});

require.register("views/main/admin/stack_upgrade/upgrade_history_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');
var arrayUtils = require('utils/array_utils');

App.MainAdminStackUpgradeHistoryView = App.TableView.extend(App.TableServerViewMixin, {

  controllerBinding: 'App.router.mainAdminStackUpgradeHistoryController',

  templateName: require('templates/main/admin/stack_upgrade/upgrade_history'),

  summary: [],

  isReady: false,

  categories: [Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.all',
    value: 'ALL',
    isSelected: true
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.upgrade',
    value: 'UPGRADE',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.downgrade',
    value: 'DOWNGRADE',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.successful.upgrade',
    value: 'UPGRADE_COMPLETED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.aborted.upgrade',
    value: 'UPGRADE_ABORTED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.failed.upgrade',
    value: 'UPGRADE_FAILED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.successful.downgrade',
    value: 'DOWNGRADE_COMPLETED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.aborted.downgrade',
    value: 'DOWNGRADE_ABORTED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.upgradeHistory.filter.failed.downgrade',
    value: 'DOWNGRADE_FAILED',
    isSelected: false
  })],
  upgradeMethods: [Em.Object.create({
    displayName: Em.I18n.t('common.rolling'),
    type: 'ROLLING'
  }), Em.Object.create({
    displayName: Em.I18n.t('common.express'),
    type: 'NON_ROLLING'
  }), Em.Object.create({
    displayName: Em.I18n.t('common.hostOrdered'),
    type: 'HOST_ORDERED'
  })],

  /**
   * @type {object}
   */
  selectedCategory: Em.computed.findBy('categories', 'isSelected', true),

  filteredCount: Em.computed.alias('filteredContent.length'),

  /**
   * displaying content filtered by upgrade type and upgrade status.
   */
  filteredContent: function () {
    var filterValue = 'ALL';
    var category = this.get('selectedCategory');
    if (category) {
      filterValue = category.get('value');
    }
    return this.filterBy(filterValue).reverse();
  }.property('selectedCategory'),

  /**
   * sort and slice received content by pagination parameters
   */
  pageContent: function () {
    var content = this.get('filteredContent').toArray();
    content = this.processForDisplay(content);
    content = content.slice(this.get('startIndex') - 1, this.get('endIndex'));
    return content;
  }.property('filteredContent', 'startIndex', 'endIndex'),

  processForDisplay: function processForDisplay(content) {
    var upgradeMethods = this.get('upgradeMethods');
    var repoVersions = App.RepositoryVersion.find();
    return content.map(function (item) {
      var versions = item.get('versions');
      var repoVersion = repoVersions.findProperty('repositoryVersion', item.get('associatedVersion'));
      var method = upgradeMethods.findProperty('type', item.get('upgradeType'));
      return {
        idHref: '#' + item.get('upgradeId'),
        id: item.get('upgradeId'),
        repositoryName: repoVersion.get('displayName'),
        repositoryType: repoVersion.get('type').toLowerCase().capitalize(),
        directionLabel: item.get('direction') === 'UPGRADE' ? Em.I18n.t('common.upgrade') : Em.I18n.t('common.downgrade'),
        upgradeTypeLabel: method ? method.get('displayName') : method,
        startTimeLabel: date.startTime(App.dateTimeWithTimeZone(item.get('startTime'))),
        endTimeLabel: date.endTime(App.dateTimeWithTimeZone(item.get('endTime'))),
        duration: date.durationSummary(item.get('startTime'), item.get('endTime')),
        displayStatus: item.get('displayStatus'),
        stackUpgradeHistoryItem: item,
        services: this.getRepoServicesForDisplay(versions)
      };
    }, this);
  },

  getRepoServicesForDisplay: function getRepoServicesForDisplay(versions) {
    return Object.keys(versions).map(function (serviceName) {
      var displayName = App.RepositoryVersion.find(versions[serviceName].from_repository_id).get('stackServices').findProperty('name', serviceName).get('displayName');
      return {
        name: serviceName,
        displayName: displayName,
        fromVersion: versions[serviceName].from_repository_version,
        toVersion: versions[serviceName].to_repository_version
      };
    });
  },

  paginationLeftClass: function () {
    if (this.get("startIndex") > 1) {
      return "paginate_previous";
    }
    return "paginate_disabled_previous";
  }.property("startIndex", 'filteredCount'),

  /**
   * Determines how display "next"-link - as link or text
   * @type {string}
   */
  paginationRightClass: function () {
    if (this.get("endIndex") < this.get("filteredCount")) {
      return "paginate_next";
    }
    return "paginate_disabled_next";
  }.property("endIndex", 'filteredCount'),

  /**
   * Show previous-page if user not in the first page
   * @method previousPage
   */
  previousPage: function previousPage() {
    if (this.get('paginationLeftClass') === 'paginate_previous') {
      this._super();
    }
  },

  /**
   * Show next-page if user not in the last page
   * @method nextPage
   */
  nextPage: function nextPage() {
    if (this.get('paginationRightClass') === 'paginate_next') {
      this._super();
    }
  },

  willInsertElement: function willInsertElement() {
    var self = this;
    this.get('controller').loadStackUpgradeHistoryToModel().done(function () {
      self.populateUpgradeHistorySummary();
    });
  },

  didInsertElement: function didInsertElement() {
    this.observesCategories();
    this.$(".accordion").on("show.bs.collapse hide.bs.collapse", function (e) {
      $(e.target).siblings(".accordion-heading").find("i.accordion-toggle").toggleClass('icon-caret-right icon-caret-down');
    });

    Em.run.later(this, function () {
      App.tooltip($('.widest-column span'));
    }, 1000);
  },

  observesCategories: function () {
    this.get('categories').forEach(function (category) {
      var label = Em.I18n.t(category.labelKey).format(this.filterBy(category.value).length);
      category.set('label', label);
    }, this);
  }.observes('isReady'),

  filterBy: function filterBy(filterValue) {
    if ('ALL' == filterValue) {
      var all_records = App.StackUpgradeHistory.find();
      return all_records.toArray();
    } else {
      var tokens = filterValue.split('_');
      var direction_token = null;
      var status_token = null;

      if (tokens.length == 1) {
        direction_token = tokens[0];
      } else if (tokens.length > 1) {
        direction_token = tokens[0];
        status_token = tokens[1];
      }

      var result = [];
      App.StackUpgradeHistory.find().forEach(function (item) {
        var direction = item.get('direction');
        if (direction == direction_token) {
          if (status_token != null) {
            //only for the given status
            var status = item.get('requestStatus');
            if (status == status_token) {
              result.push(item);
            }
          } else {
            //regardless status
            result.push(item);
          }
        }
      }, this);
      return result;
    }
  },

  selectCategory: function selectCategory(event) {
    this.get('categories').filterProperty('isSelected').setEach('isSelected', false);
    event.context.set('isSelected', true);
  },

  populateUpgradeHistorySummary: function populateUpgradeHistorySummary() {
    this.set('isReady', false);
    var result = [Em.Object.create({
      direction: 'UPGRADE',
      label: Em.I18n.t('common.upgrade'),
      hasSuccess: false,
      success: 0,
      hasAbort: false,
      abort: 0
    }), Em.Object.create({
      direction: 'DOWNGRADE',
      label: Em.I18n.t('common.downgrade'),
      hasSuccess: false,
      success: 0,
      hasAbort: false,
      abort: 0
    })];

    App.StackUpgradeHistory.find().forEach(function (item) {
      var direction = item.get('direction');
      var status = item.get('requestStatus');
      if ('UPGRADE' == direction) {
        if ('COMPLETED' == status) {
          result[0].set('success', result[0].get('success') + 1);
        } else if ('ABORTED' == status) {
          result[0].set('abort', result[0].get('abort') + 1);
        }
      } else if ('DOWNGRADE' == direction) {
        if ('COMPLETED' == status) {
          result[1].set('success', result[1].get('success') + 1);
        } else if ('ABORTED' == status) {
          result[1].set('abort', result[1].get('abort') + 1);
        }
      }
    }, this);

    result[0].set('hasSuccess', result[0].get('success') > 0);
    result[1].set('hasSuccess', result[1].get('success') > 0);
    result[0].set('hasAbort', result[0].get('abort') > 0);
    result[1].set('hasAbort', result[1].get('abort') > 0);

    this.set('summary', result);
    this.set('isReady', true);
  },

  showUpgradeHistoryRecord: function showUpgradeHistoryRecord(event) {
    var record = event.context;
    var direction = App.format.normalizeName(record.get('direction'));
    var associatedVersion = record.get('associatedVersion');
    var type = this.get('upgradeMethods').findProperty('type', record.get('upgradeType'));
    var displayName = type ? type.get('displayName') : App.format.normalizeName(record.get('upgradeType'));
    var i18nKeySuffix = direction.toLowerCase() === 'upgrade' ? 'upgrade' : 'downgrade';

    this.get('controller').set('currentUpgradeRecord', record);

    App.ModalPopup.show({
      classNames: ['wide-modal-wrapper'],
      modalDialogClasses: ['modal-xlg'],
      header: Em.I18n.t('admin.stackVersions.upgradeHistory.record.title.' + i18nKeySuffix).format(displayName, direction, associatedVersion),
      bodyClass: App.MainAdminStackUpgradeHistoryDetailsView,
      primary: Em.I18n.t('common.dismiss'),
      secondary: null,
      didInsertElement: function didInsertElement() {
        this._super();
        this.fitHeight();
        this.fitInnerHeight();
      },
      fitInnerHeight: function fitInnerHeight() {
        var block = this.$().find('.modal .modal-body');
        var scrollable = this.$().find('.modal .scrollable-block');
        scrollable.css('max-height', Number(block.css('max-height').slice(0, -2)) - block.height());
        block.css('max-height', 'none');
      }
    });
  }
});

});

require.register("views/main/admin/stack_upgrade/upgrade_task_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.upgradeTaskView = Em.View.extend({
  templateName: require('templates/main/admin/stack_upgrade/upgrade_task'),

  /**
   * view observed directly
   * @type {?Em.View}
   */
  outsideView: null,

  /**
   * @type {boolean}
   */
  isExpanded: false,

  /**
   * @type {boolean}
   */
  errorLogOpened: false,

  /**
   * @type {boolean}
   */
  outputLogOpened: false,

  /**
   * @type {App.upgradeEntity}
   * @default null
   */
  content: null,

  /**
   * @type {Array}
   */
  tasks: [],

  /**
   * @type {string}
   */
  logTabId: Em.computed.format('{0}-log-tab', 'elementId'),

  /**
   * @type {string}
   */
  errorTabId: Em.computed.format('{0}-error-tab', 'elementId'),

  /**
   * @type {string}
   */
  logTabIdLink: Em.computed.format('#{0}', 'logTabId'),

  /**
   * @type {string}
   */
  errorTabIdLInk: Em.computed.format('#{0}', 'errorTabId'),

  didInsertElement: function didInsertElement() {
    if (this.get('outsideView') && this.get('content')) {
      this.set('isExpanded', true);
      this.doPolling();
    }
  },

  toggleExpanded: function toggleExpanded() {
    var isExpanded = this.get('isExpanded');
    this.toggleProperty('isExpanded', !isExpanded);
    if (!isExpanded) {
      this.doPolling();
    } else {
      this.set('content.isContentLoaded', true);
    }
  },

  /**
   *
   */
  doPolling: function () {
    var self = this;
    var task = this.get('content');
    var isExpanded = this.get('outsideView') ? this.get('outsideView.isDetailsOpened') : this.get('isExpanded');

    if (task && isExpanded) {
      this.get('controller').getUpgradeTask(task).complete(function () {
        task.set('isContentLoaded', true);
        if (!task.get('isCompleted')) {
          self.set('timer', setTimeout(function () {
            self.doPolling(task);
          }, App.bgOperationsUpdateInterval));
        }
      });
    } else {
      clearTimeout(this.get('timer'));
    }
  }.observes('outsideView.isDetailsOpened'),

  willDestroyElement: function willDestroyElement() {
    clearTimeout(this.get('timer'));
  },

  /**
   * open error log in textarea to give ability to cope content
   * @param {object} event
   */
  copyErrLog: function copyErrLog(event) {
    this.toggleProperty('errorLogOpened');
  },

  /**
   * open stdout log in textarea to give ability to cope content
   * @param {object} event
   */
  copyOutLog: function copyOutLog(event) {
    this.toggleProperty('outputLogOpened');
  },

  /**
   * open error log in new window
   */
  openErrorLog: function openErrorLog() {
    this.openLogWindow(this.get('content.stderr'));
  },

  /**
   * open stdout log in new window
   */
  openOutLog: function openOutLog() {
    this.openLogWindow(this.get('content.stdout'));
  },

  /**
   * open logs in new window
   * @param {string} log
   */
  openLogWindow: function openLogWindow(log) {
    var newWindow = window.open(),
        newDocument = newWindow.document,
        outputWrapper = newDocument.createElement('pre'),
        output = newDocument.createTextNode(log);
    outputWrapper.appendChild(output);
    newDocument.body.appendChild(outputWrapper);
    newDocument.close();
  }
});

});

require.register("views/main/admin/stack_upgrade/upgrade_version_box_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.UpgradeVersionBoxView = Em.View.extend({
  templateName: require('templates/main/admin/stack_upgrade/upgrade_version_box'),
  classNames: ['version-box'],
  classNameBindings: ['versionClass'],

  /**
   * @type {string}
   * @constant
   */
  PROGRESS_STATUS: 'IN_PROGRESS',

  /**
   * @type {boolean}
   */
  upgradeCheckInProgress: false,

  /**
   * progress of version installation
   * @type {number}
   */
  installProgress: function () {
    if (App.get('testMode')) return 100;

    var installRequest,
        requestIds = this.get('controller').getRepoVersionInstallId();
    if (requestIds) {
      installRequest = App.router.get('backgroundOperationsController.services').findProperty('id', requestIds[0]);
    }
    return installRequest ? installRequest.get('progress') : 0;
  }.property('App.router.backgroundOperationsController.serviceTimestamp'),

  /**
   * version is upgrading or downgrading
   * @type {boolean}
   */
  isUpgrading: function () {
    return (this.get('controller.upgradeVersion') === this.get('content.displayName') || this.get('controller.fromVersion') === this.get('content.repositoryVersion')) && App.get('upgradeState') !== 'NOT_REQUIRED';
  }.property('App.upgradeState', 'content.displayName', 'controller.upgradeVersion'),

  isRepoUrlsEditDisabled: function () {
    return ['INSTALLING', 'UPGRADING'].contains(this.get('content.status')) || this.get('isUpgrading') || !App.isAuthorized('AMBARI.MANAGE_STACK_VERSIONS');
  }.property('content.status', 'isUpgrading'),

  /**
   * @type {string}
   */
  versionClass: function () {
    return this.get('content.status') === 'CURRENT' ? 'current-version-box' : '';
  }.property('content.status'),

  /**
   * @type {boolean}
   */
  isOutOfSync: Em.computed.equal('content.status', 'OUT_OF_SYNC'),

  /**
   * map containing version (id, label)
   * this is used as param for <code>showHosts<code> method
   * @type {Object}
   */
  versionStateMap: {
    'current': {
      'type': 'CURRENT',
      'value': ['CURRENT'],
      'property': 'currentHosts',
      'label': Em.I18n.t('admin.stackVersions.hosts.popup.header.current')
    },
    'installed': {
      'type': 'INSTALLED',
      'value': ['INSTALLED'],
      'property': 'installedHosts',
      'label': Em.I18n.t('admin.stackVersions.hosts.popup.header.installed')
    },
    'not_installed': {
      'type': 'NOT_INSTALLED',
      'value': ['INSTALLING', 'INSTALL_FAILED', 'OUT_OF_SYNC'],
      'property': 'notInstalledHosts',
      'label': Em.I18n.t('admin.stackVersions.hosts.popup.header.not_installed')
    }
  },

  /**
   * @type {object}
   * @default null
   */
  content: null,

  currentLabelClass: 'label label-success',

  /**
   * map of properties which correspond to particular state of Upgrade version
   * @type {object}
   */
  statePropertiesMap: function () {
    return {
      'CURRENT': {
        isLabel: true,
        text: Em.I18n.t('common.current'),
        class: this.get('currentLabelClass')
      },
      'NOT_REQUIRED': {
        isButton: true,
        text: Em.I18n.t('admin.stackVersions.version.installNow'),
        action: 'installRepoVersionPopup'
      },
      'LOADING': {
        isSpinner: true,
        class: 'spinner'
      },
      'INSTALLING': {
        iconClass: 'glyphicon glyphicon-cog',
        isLink: true,
        text: Em.I18n.t('hosts.host.stackVersions.status.installing'),
        action: 'showProgressPopup'
      },
      'INSTALLED': {
        iconClass: 'glyphicon glyphicon-ok',
        isButtonGroup: true,
        text: Em.I18n.t('common.installed'),
        action: null
      },
      'SUSPENDED': {
        isButton: true,
        text: Em.I18n.t('admin.stackUpgrade.dialog.resume'),
        action: 'resumeUpgrade'
      },
      'CURRENT_PATCH': {
        isLabel: true,
        text: Em.I18n.t('common.current'),
        class: this.get('currentLabelClass')
      },
      'CURRENT_PATCH_REVERTABLE': {
        isButtonGroup: true,
        text: Em.I18n.t('common.current'),
        action: null,
        buttons: [{
          text: Em.I18n.t('common.revert'),
          action: 'confirmRevertPatchUpgrade'
        }]
      }
    };
  }.property(),

  /**
   * object that describes how content should be displayed
   * @type {Em.Object}
   * TODO remove <code>isUpgrading</code> condition when transition of version states in API fixed
   */
  stateElement: function () {
    var statePropertiesMap = this.get('statePropertiesMap');
    var status = this.get('content.status');
    var element = Em.Object.create({
      status: status,
      isInstalling: Em.computed.equal('status', 'INSTALLING'),
      buttons: [],
      isDisabled: false
    });
    var isSuspended = App.get('upgradeSuspended');

    if (status === 'CURRENT' && this.get('content.isPatch') && !this.get('isUpgrading')) {
      if (this.get('content.stackVersion.supportsRevert')) {
        element.setProperties(statePropertiesMap['CURRENT_PATCH_REVERTABLE']);
      } else {
        element.setProperties(statePropertiesMap['CURRENT_PATCH']);
      }
    } else if (['INSTALLING', 'CURRENT'].contains(status)) {
      element.setProperties(statePropertiesMap[status]);
    } else if (status === 'NOT_REQUIRED') {
      this.processNotRequiredState(element);
    } else if (status === 'INSTALLED' && !this.get('isUpgrading') || ['INSTALL_FAILED', 'OUT_OF_SYNC'].contains(status)) {
      this.processPreUpgradeState(element);
    } else if (this.get('isUpgrading') && !isSuspended) {
      this.processUpgradingState(element);
    } else if (isSuspended) {
      this.processSuspendedState(element);
    }
    //For restricted upgrade wizard should be disabled in any state
    if (this.get('controller.isWizardRestricted') || !App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK')) {
      element.set('isDisabled', true);
    }
    return element;
  }.property('content.status', 'controller.isDowngrade', 'isUpgrading', 'controller.requestInProgress', 'controller.requestInProgressRepoId', 'parentView.repoVersions.@each.status', 'isCurrentStackPresent'),

  /**
   * check if actions of NOT_REQUIRED stack version disabled
   * @returns {boolean}
   */
  isDisabledOnInit: function isDisabledOnInit() {
    return this.get('controller.requestInProgress') || !this.get('isCurrentStackPresent') || !this.get('content.isCompatible') || App.get('upgradeIsRunning') && !App.get('upgradeSuspended') || this.get('parentView.repoVersions').someProperty('status', 'INSTALLING');
  },

  /**
   * @param {Em.Object} element
   */
  processSuspendedState: function processSuspendedState(element) {
    element.setProperties(this.get('statePropertiesMap')['SUSPENDED']);
    var text = this.get('controller.isDowngrade') ? Em.I18n.t('admin.stackUpgrade.dialog.resume.downgrade') : Em.I18n.t('admin.stackUpgrade.dialog.resume');
    element.set('text', this.get('isVersionColumnView') ? Em.I18n.t('common.resume') : text);
    element.set('isDisabled', this.get('controller.requestInProgress'));
  },

  /**
   * @param {Em.Object} element
   */
  processUpgradingState: function processUpgradingState(element) {
    element.set('isLink', true);
    element.set('action', 'openUpgradeDialog');
    if (['HOLDING', 'HOLDING_FAILED', 'HOLDING_TIMEDOUT', 'ABORTED'].contains(App.get('upgradeState'))) {
      element.set('iconClass', 'glyphicon glyphicon-pause');
      if (this.get('controller.isDowngrade')) {
        element.set('text', Em.I18n.t('admin.stackVersions.version.downgrade.pause'));
      } else {
        element.set('text', Em.I18n.t('admin.stackVersions.version.upgrade.pause'));
      }
    } else {
      element.set('iconClass', 'glyphicon glyphicon-cog');
      if (this.get('controller.isDowngrade')) {
        element.set('text', Em.I18n.t('admin.stackVersions.version.downgrade.running'));
      } else {
        element.set('text', Em.I18n.t('admin.stackVersions.version.upgrade.running'));
      }
    }
  },

  /**
   * @param {Em.Object} element
   */
  processPreUpgradeState: function processPreUpgradeState(element) {
    var currentVersion = this.get('controller.currentVersion');
    var status = this.get('content.status');
    var isVersionHigherThanCurrent = stringUtils.compareVersions(this.get('content.repositoryVersion'), Em.get(currentVersion, 'repository_version')) === 1;
    var isDisabled = this.isDisabledOnInstalled();
    if (Em.get(currentVersion, 'stack_name') !== this.get('content.stackVersionType') || isVersionHigherThanCurrent) {
      switch (status) {
        case 'OUT_OF_SYNC':
          if (Em.isNone(currentVersion)) {
            element.set('text', Em.I18n.t('admin.stackVersions.version.installError'));
            element.set('iconClass', 'glyphicon glyphicon-warning-sign');
            element.set('isLabel', true);
          } else {
            element.set('isButton', true);
            element.set('text', this.get('isVersionColumnView') ? Em.I18n.t('common.reinstall') : Em.I18n.t('admin.stackVersions.version.reinstall'));
            element.set('action', 'installRepoVersionPopup');
          }
          break;
        case 'INSTALL_FAILED':
          if (Em.isNone(currentVersion)) {
            element.set('text', Em.I18n.t('admin.stackVersions.version.installError'));
            element.set('iconClass', 'glyphicon glyphicon-warning-sign');
            element.set('isLabel', true);
          } else {
            element.set('isButton', true);
            element.set('text', this.get('isVersionColumnView') ? Em.I18n.t('common.reinstall') : Em.I18n.t('admin.stackVersions.version.reinstall'));
            element.set('action', 'installRepoVersionPopup');
          }
          break;
        default:
          var isVersionColumnView = this.get('isVersionColumnView');
          var stackServices = this.get('content.stackServices');
          var isUpgradable = stackServices && (this.get('content.isStandard') || stackServices.some(function (stackService) {
            return stackService.get('isUpgradable');
          }));
          var isPatch = this.get('content.isPatch');
          var isMaint = this.get('content.isMaint');

          element.set('isButtonGroup', true);
          if (isUpgradable) {
            element.set('text', isVersionColumnView ? Em.I18n.t('common.upgrade') : Em.I18n.t('admin.stackVersions.version.performUpgrade'));
            element.set('action', 'confirmUpgrade');
            element.get('buttons').pushObject({
              text: isVersionColumnView ? Em.I18n.t('common.reinstall') : Em.I18n.t('admin.stackVersions.version.reinstall'),
              action: 'installRepoVersionPopup',
              isDisabled: isDisabled
            });

            element.get('buttons').pushObject({
              text: Em.I18n.t('admin.stackVersions.version.preUpgradeCheck'),
              action: 'showUpgradeOptions',
              isDisabled: isDisabled
            });
          } else {
            element.set('iconClass', 'icon-ok');
            element.set('text', Em.I18n.t('common.installed'));
          }

          if (isPatch || isMaint) {
            element.get('buttons').pushObject({
              text: Em.I18n.t('common.hide'),
              action: 'confirmDiscardRepoVersion',
              isDisabled: isDisabled
            });
          }

      }
      element.set('isDisabled', isDisabled);
    } else {
      element.setProperties(this.get('statePropertiesMap')['INSTALLED']);
      if (this.get('content.isPatch') || this.get('content.isMaint')) {
        element.get('buttons').pushObject({
          text: Em.I18n.t('common.hide'),
          action: 'confirmDiscardRepoVersion',
          isDisabled: isDisabled
        });
      }
    }
  },

  /**
   * @param {Em.Object} element
   */
  processNotRequiredState: function processNotRequiredState(element) {
    var isDisabledOnInit = this.isDisabledOnInit();
    var requestInProgressRepoId = this.get('controller.requestInProgressRepoId');
    var status = this.get('content.status');

    if (requestInProgressRepoId && requestInProgressRepoId === this.get('content.id')) {
      element.setProperties(this.get('statePropertiesMap')['LOADING']);
    } else {
      element.setProperties(this.get('statePropertiesMap')[status]);
    }
    element.set('isDisabled', isDisabledOnInit);
    element.set('isButtonGroup', true);
    element.set('isButton', false);
    element.get('buttons').pushObject({
      text: Em.I18n.t('common.hide'),
      action: 'confirmDiscardRepoVersion',
      isDisabled: isDisabledOnInit
    });
  },

  /**
   * check if actions of INSTALLED stack version disabled
   * @returns {boolean}
   */
  isDisabledOnInstalled: function isDisabledOnInstalled() {
    return !this.get('isCurrentStackPresent') || !App.isAuthorized('CLUSTER.UPGRADE_DOWNGRADE_STACK') || this.get('controller.requestInProgress') || this.get('parentView.repoVersions').someProperty('status', 'INSTALLING') || this.get('controller.isDowngrade') && this.get('controller.currentVersion.repository_name') === this.get('controller.upgradeVersion');
  },

  didInsertElement: function didInsertElement() {
    App.tooltip($('.link-tooltip'), { title: Em.I18n.t('admin.stackVersions.version.linkTooltip') });
    App.tooltip($('.hosts-tooltip'));
    App.tooltip($('.out-of-sync-badge'), { title: Em.I18n.t('hosts.host.stackVersions.status.out_of_sync') });
    if (!this.get('content.isCompatible')) {
      App.tooltip(this.$(".repo-version-tooltip"), {
        title: Em.I18n.t('admin.stackVersions.version.noCompatible.tooltip')
      });
    }
    Em.run.later(this, function () {
      if (this.get('state') !== 'inDOM') {
        return;
      }
      if (this.get('maintenanceHosts').length + this.get('notRequiredHosts').length) {
        App.tooltip(this.$('.hosts-section'), { placement: 'bottom', title: Em.I18n.t('admin.stackVersions.version.hostsInfoTooltip').format(this.get('maintenanceHosts').length + this.get('notRequiredHosts').length, this.get('maintenanceHosts').length, this.get('notRequiredHosts').length) });
      }
    }, 1000);
  },

  willDestroyElement: function willDestroyElement() {
    $('.link-tooltip').tooltip('destroy');
    $('.hosts-tooltip').tooltip('destroy');
    $('.out-of-sync-badge').tooltip('destroy');
  },

  isCurrentStackPresent: Ember.computed('parentView.repoVersions.@each.stackVersion.state', function () {
    return this.get('parentView.repoVersions').someProperty('stackVersion.state', 'CURRENT');
  }),

  /**
   * run custom action of controller
   */
  runAction: function runAction(event) {
    var target = event && event.target,
        action = event && event.context || this.get('stateElement.action');
    if (target && ($(target).hasClass('disabled') || $(target).parent().hasClass('disabled'))) {
      return;
    }
    if (action) {
      this.get('controller')[action](this.get('content'));
    }
  },

  /**
   * @param App.RepositoryVersion
   * */
  getStackVersionNumber: function getStackVersionNumber(repository) {
    var stackVersion = null;
    var systems = repository.get('operatingSystems');

    systems.forEach(function (os) {
      os.get('repositories').forEach(function (repo) {
        stackVersion = repo.get('stackVersion');
        if (null != stackVersion) return stackVersion;
      });
    });

    return stackVersion;
  },

  /**
   * show popup with repositories to edit
   * @return {App.ModalPopup}
   */
  editRepositories: function editRepositories() {
    var self = this;
    var repoRecord = App.RepositoryVersion.find(this.get('content.id'));
    //make deep copy of repoRecord
    var repo = Em.Object.create({
      repoVersionId: repoRecord.get('id'),
      displayName: repoRecord.get('displayName'),
      repositoryVersion: repoRecord.get('displayName'),
      stackVersion: self.getStackVersionNumber(repoRecord),
      useRedhatSatellite: repoRecord.get('useRedhatSatellite'),
      operatingSystems: repoRecord.get('operatingSystems').map(function (os) {
        return Em.Object.create({
          osType: os.get('osType'),
          isSelected: true,
          repositories: os.get('repositories').map(function (repository) {
            return Em.Object.create({
              repoName: repository.get('repoName'),
              repoId: repository.get('repoId'),
              baseUrl: repository.get('baseUrl'),
              hasError: false,
              showRepo: repository.get('showRepo')
            });
          })
        });
      })
    });

    return this.get('isRepoUrlsEditDisabled') ? null : App.ModalPopup.show({
      classNames: ['repository-list', 'common-modal-wrapper'],
      modalDialogClasses: ['modal-lg'],
      skipValidation: false,
      autoHeight: false,
      /**
       * @type {boolean}
       */
      serverValidationFailed: false,
      bodyClass: Ember.View.extend({
        content: repo,
        skipCheckBox: App.CheckboxView.extend({
          repoBinding: 'parentView.content',
          checkboxClassNames: ["align-checkbox"],
          change: function change() {
            this.get('repo.operatingSystems').forEach(function (os) {
              os.get('repositories').forEach(function (repo) {
                repo.set('skipValidation', this.get('checked'));
              }, this);
            }, this);
          }
        }),

        /**
         * set <code>disablePrimary</code> of popup depending on base URL validation
         */
        uiValidation: function uiValidation() {
          var disablePrimary = !(App.get('isAdmin') && !App.get('isOperator'));

          this.get('content.operatingSystems').forEach(function (os) {
            os.get('repositories').forEach(function (repo) {
              disablePrimary = !disablePrimary ? repo.get('hasError') : disablePrimary;
            }, this);
          }, this);
          this.set('parentView.disablePrimary', disablePrimary);
        },
        templateName: require('templates/main/admin/stack_upgrade/edit_repositories'),
        didInsertElement: function didInsertElement() {
          App.tooltip($("[rel=skip-validation-tooltip], [rel=use-redhat-tooltip]"), { placement: 'right' });
        },
        willDestroyElement: function willDestroyElement() {
          $("[rel=skip-validation-tooltip], [rel=use-redhat-tooltip]").tooltip('destroy');
        }
      }),
      header: Em.I18n.t('common.repositories'),
      primary: Em.I18n.t('common.save'),
      disablePrimary: false,
      onPrimary: function onPrimary() {
        var self = this;
        App.get('router.mainAdminStackAndUpgradeController').saveRepoOS(repo, this.get('skipValidation')).done(function (data) {
          if (data.length > 0) {
            self.set('serverValidationFailed', true);
            self.set('disablePrimary', true);
          } else {
            self.hide();
          }
        });
      }
    });
  },

  /**
   * shows popup with listed hosts wich has current state of hostStackVersion
   * @param {object} event
   * @returns {App.ModalPopup}
   * @method showHostsListPopup
   */
  showHosts: function showHosts(event) {
    var status = event.contexts[0];
    var displayName = this.get('content.displayName');
    var hosts = this.get(status['property']);
    var self = this;
    var title = status.type === 'CURRENT' ? Em.I18n.t('admin.stackVersions.hosts.popup.current.title').format(displayName, hosts.length) : Em.I18n.t('admin.stackVersions.hosts.popup.title').format(displayName, status.label, hosts.length);
    hosts.sort();
    if (hosts.length) {
      return App.ModalPopup.show({
        bodyClass: Ember.View.extend({
          title: title,
          hosts: hosts,
          template: Em.Handlebars.compile('<h4>{{view.title}}</h4><div class="limited-height-2">{{#each view.hosts}}<div>{{this}}</div>{{/each}}</div>')
        }),
        header: Em.I18n.t('admin.stackVersions.hosts.popup.header').format(status.label),
        primary: Em.I18n.t('admin.stackVersions.hosts.popup.primary'),
        secondary: Em.I18n.t('common.close'),
        onPrimary: function onPrimary() {
          this.hide();
          if ($('.version-box-popup .modal')) {
            $('.version-box-popup .modal .modal-footer .btn-success').click();
          }
          self.filterHostsByStack(displayName, status.value);
        }
      });
    }
  },

  /**
   * goes to the hosts page with content filtered by repo_version_name and repo_version_state
   * @param {string} displayName
   * @param {Array} states
   * @method filterHostsByStack
   */
  filterHostsByStack: function filterHostsByStack(displayName, states) {
    if (Em.isNone(displayName) || Em.isNone(states) || !states.length) return;
    App.router.get('mainHostController').filterByStack(displayName, states);
    App.router.get('mainHostController').set('showFilterConditionsFirstLoad', true);
    App.router.get('mainHostController').set('filterChangeHappened', true);
    App.router.transitionTo('hosts.index');
  },

  /**
   * Not installed hosts should exclude 1.not required hosts 2. Maintenance Mode hosts,
   * or it maybe confusing to users
   * @type {Array}
   */
  notInstalledHosts: function () {
    var notInstalledHosts = this.get('content.notInstalledHosts') || App.get('allHostNames');
    var notRequiredHosts = this.get('notRequiredHosts');
    var maintenanceHosts = this.get('maintenanceHosts');
    if (notInstalledHosts.length && notRequiredHosts.length) {
      notRequiredHosts.forEach(function (not_required) {
        var index = notInstalledHosts.indexOf(not_required);
        if (index > -1) {
          notInstalledHosts.splice(index, 1);
        }
      });
    }
    if (notInstalledHosts.length && maintenanceHosts.length) {
      maintenanceHosts.forEach(function (mm_host) {
        var index = notInstalledHosts.indexOf(mm_host);
        if (index > -1) {
          notInstalledHosts.splice(index, 1);
        }
      });
    }
    return notInstalledHosts;
  }.property('content.notInstalledHosts', 'notRequiredHosts', 'maintenanceHosts'),

  /**
   * @type {Array}
   */
  maintenanceHosts: function () {
    return App.Host.find().filterProperty('passiveState', 'ON').mapProperty('hostName') || [];
  }.property(''),

  /**
   * Host with no HDP component is not required to install new version
   * @type {Array}
   */
  notRequiredHosts: function () {
    var notRequiredHosts = [];
    App.Host.find().forEach(function (host) {
      if (!host.get('hostComponents').someProperty('isHDPComponent')) {
        notRequiredHosts.push(host.get('hostName'));
      }
    });
    return notRequiredHosts.uniq() || [];
  }.property(''),

  /**
   * @type {Array}
   */
  installedHosts: function () {
    return this.get('content.installedHosts') || [];
  }.property('content.installedHosts'),

  /**
   * @type {Array}
   */
  currentHosts: function () {
    return this.get('content.currentHosts') || [];
  }.property('content.currentHosts')
});

});

require.register("views/main/admin/stack_upgrade/upgrade_version_column_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.UpgradeVersionColumnView = App.UpgradeVersionBoxView.extend({
  templateName: require('templates/main/admin/stack_upgrade/upgrade_version_column'),
  isVersionColumnView: true,
  classNames: ['version-column', 'col-md-4'],

  /**
   * Indicates whether block for version type should be displayed.
   * True if any of available versions is of PATCH, MAINT or SERVICE type
   * @type {Boolean}
   */
  displayVersionTypeBlock: false,

  currentLabelClass: 'btn btn-primary',

  didInsertElement: function didInsertElement() {
    App.tooltip($('.out-of-sync-badge'), { title: Em.I18n.t('hosts.host.stackVersions.status.out_of_sync') });
    App.tooltip($('.not-upgradable'));
    if (!this.get('content.isCompatible')) {
      App.tooltip(this.$(".repo-version-tooltip"), {
        title: Em.I18n.t('admin.stackVersions.version.noCompatible.tooltip')
      });
    }
    this.adjustColumnWidth();
  },

  adjustColumnWidth: function () {
    //set the width, height of each version column dynamically
    var reposCount = App.RepositoryVersion.find().filterProperty('isVisible').get('length');
    var widthFactor = reposCount > 3 ? 0.18 : 0.31;
    $('.version-column').width($('.versions-slides').width() * widthFactor);
    var height = App.Service.find().get('length') > 10 ? (App.Service.find().get('length') - 10) * 40 + 500 : 500;
    $('.version-column').height(height);

    // set the lines width of the table, line up the labels
    $('.border-extended-table').width(reposCount * 100 + 100 + "%");
    $('.border-extended-table').css("max-width", reposCount * 100 + 100 + "%");
  }.observes('parentView.repoVersions.@each.isVisible'),

  services: function () {
    var _this = this;

    var originalServices = this.get('content.stackServices');
    // sort the services in the order the same as service menu
    return App.Service.find().map(function (service) {

      var stackService = originalServices.findProperty('name', service.get('serviceName')),
          isAvailable = _this.isStackServiceAvailable(stackService);
      var notUpgradable = false,
          notUpgradableTitle = '';
      if (!stackService) {
        console.error(stackService + ' definition does not exist in the stack.');
        notUpgradable = true;
        notUpgradableTitle = Em.I18n.t('admin.stackVersions.version.service.notSupported');
      } else {
        notUpgradable = _this.getNotUpgradable(isAvailable, stackService.get('isUpgradable'));
        if (notUpgradable) {
          notUpgradableTitle = Em.I18n.t('admin.stackVersions.version.service.notUpgradable');
        }
      }

      return Em.Object.create({
        displayName: service.get('displayName'),
        name: service.get('serviceName'),
        latestVersion: stackService ? stackService.get('latestVersion') : '',
        isVersionInvisible: !stackService,
        notUpgradable: notUpgradable,
        notUpgradableTitle: notUpgradableTitle,
        isAvailable: isAvailable
      });
    });
  }.property(),

  getNotUpgradable: function getNotUpgradable(isAvailable, isUpgradable) {
    return this.get('content.isMaint') && !this.get('isUpgrading') && this.get('content.status') !== 'CURRENT' && isAvailable && !isUpgradable;
  },

  /**
   * @param {Em.Object} stackService
   * @returns {boolean}
   */
  isStackServiceAvailable: function isStackServiceAvailable(stackService) {
    var self = this;
    if (!stackService) {
      return false;
    }
    if (this.get('content.isCurrent')) {
      var originalService = App.Service.find(stackService.get('name'));
      return stackService.get('isAvailable') && originalService.get('desiredRepositoryVersionId') === this.get('content.id');
    } else {
      return stackService.get('isAvailable');
    }
  },

  /**
   * on click handler for "show details" link
   */
  openVersionBoxPopup: function openVersionBoxPopup(event) {
    var content = this.get('content');
    var parentView = this.get('parentView');

    return App.ModalPopup.show({
      classNames: ['version-box-popup'],
      bodyClass: App.UpgradeVersionBoxView.extend({
        classNames: ['version-box-in-popup'],
        content: content,
        parentView: parentView
      }),
      header: Em.I18n.t('admin.stackVersions.version.column.showDetails.title'),
      primary: Em.I18n.t('common.dismiss'),
      secondary: null
    });
  }
});

});

require.register("views/main/admin/stack_upgrade/upgrade_wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.upgradeWizardView = Em.View.extend({
  controllerBinding: 'App.router.mainAdminStackAndUpgradeController',
  templateName: require('templates/main/admin/stack_upgrade/stack_upgrade_wizard'),

  /**
   * @type {Array}
   */
  failedStatuses: ['HOLDING_FAILED', 'HOLDING_TIMEDOUT', 'FAILED', 'TIMED_OUT', 'ABORTED'],

  /**
   * @type {Array}
   */
  activeStatuses: ['HOLDING_FAILED', 'HOLDING_TIMEDOUT', 'FAILED', 'TIMED_OUT', 'HOLDING', 'IN_PROGRESS', 'ABORTED'],

  /**
   * update timer
   * @type {number|null}
   * @default null
   */
  updateTimer: null,

  /**
   * update timer of Upgrade Item
   * @type {number|null}
   * @default null
   */
  upgradeItemTimer: null,

  /**
   * @type {boolean}
   */
  isLoaded: false,

  /**
   * @type {boolean}
   */
  isDetailsOpened: false,

  /**
   * @type {boolean}
   */
  outsideView: true,

  /**
   * Downgrade should be available only if target version higher than current, so we can't downgrade
   * when downgrade already started
   * @type {boolean}
   */
  isDowngradeAvailable: Em.computed.and('!controller.isDowngrade', 'controller.downgradeAllowed', '!controller.cantBeStarted'),

  /**
   * progress value is rounded to floor
   * @type {number}
   */
  overallProgress: function () {
    return Math.floor(this.get('controller.upgradeData.Upgrade.progress_percent'));
  }.property('controller.upgradeData.Upgrade.progress_percent'),

  /**
   * upgrade groups, reversed and PENDING ones are hidden
   * @type {Array}
   */
  upgradeGroups: function () {
    return this.get('controller.upgradeData.upgradeGroups') || [];
  }.property('controller.upgradeData.upgradeGroups'),

  /**
   * currently active group
   * @type {object|undefined}
   */
  activeGroup: function () {
    if (App.get('upgradeSuspended')) return;
    return this.get('upgradeGroups').find(function (item) {
      return this.get('activeStatuses').contains(item.get('status'));
    }, this);
  }.property('upgradeGroups.@each.status'),

  /**
   * if upgrade group is in progress it should have currently running item
   * @type {object|undefined}
   */
  runningItem: Em.computed.findBy('activeGroup.upgradeItems', 'status', 'IN_PROGRESS'),

  /**
   * if upgrade group is failed it should have failed item
   * @type {object|undefined}
   */
  failedItem: function () {
    return this.get('activeGroup.upgradeItems') && this.get('activeGroup.upgradeItems').find(function (item) {
      return this.get('failedStatuses').contains(item.get('status'));
    }, this);
  }.property('activeGroup.upgradeItems.@each.status'),

  /**
   * can skip failed item or not
   * @type {boolean}
   */
  canSkipFailedItem: function () {
    var failedItem = this.get('failedItem');
    var associatedVersion = this.get('controller.upgradeData.Upgrade.associated_version');
    var version = associatedVersion && App.RepositoryVersion.find().findProperty('repositoryVersion', associatedVersion);
    var isPatchOrMaint = version && (version.get('isPatch') || version.get('isMaint'));
    return failedItem && failedItem.get('skippable') && !(this.get('isFinalizeItem') && isPatchOrMaint);
  }.property('failedItem'),

  /**
   * upgrade doesn't have any failed or manual or running item
   * @type {boolean}
   */
  noActiveItem: function () {
    return Em.isNone(this.get('failedItem')) && Em.isNone(this.get('runningItem')) && Em.isNone(this.get('manualItem')) && !['NOT_REQUIRED', 'COMPLETED', 'ABORTED'].contains(App.get('upgradeState'));
  }.property('failedItem', 'runningItem', 'manualItem', 'App.upgradeState'),

  /**
   * details of currently active task
   * @type {object|null}
   */
  taskDetails: function () {
    if (this.get('runningItem')) {
      return this.get('runningItem').get('tasks').findProperty('status', 'IN_PROGRESS');
    } else if (this.get('failedItem')) {
      return this.get('failedItem').get('tasks').find(function (task) {
        return this.get('failedStatuses').contains(task.get('status'));
      }, this);
    } else {
      return null;
    }
  }.property('failedItem.tasks.@each.status', 'runningItem.tasks.@each.status'),

  /**
   * indicate whether failed item can be skipped or retried in order to continue Upgrade
   * @type {boolean}
   */
  isHoldingState: function () {
    return Boolean(this.get('failedItem.status') && this.get('failedItem.status').contains('HOLDING') || this.get('failedItem.status') === 'ABORTED');
  }.property('failedItem.status'),

  /**
   * @type {boolean}
   */
  isManualDone: false,

  /**
   * if manualItem has been switched then isManualDone flag should be reset
   */
  resetManualDone: function () {
    this.set('isManualDone', false);
  }.observes('manualItem'),

  /**
   * @type {boolean}
   */
  isManualProceedDisabled: Em.computed.or('!isManualDone', 'controller.requestInProgress'),

  /**
   * if upgrade group is manual it should have manual item
   * @type {object|undefined}
   */
  manualItem: Em.computed.findBy('activeGroup.upgradeItems', 'status', 'HOLDING'),

  /**
   * plain manual item
   * @type {object|undefined}
   */
  plainManualItem: function () {
    return this.get('manualItem') && ![this.get('controller.finalizeContext'), this.get("controller.slaveFailuresContext"), this.get("controller.serviceCheckFailuresContext")].contains(this.get('manualItem.context'));
  }.property('manualItem.context'),

  /**
   * manualItem: indicate whether the step is "Slave component failures", a dialog with instructions will show up for manual steps
   * @type {boolean}
   */
  isSlaveComponentFailuresItem: function () {
    var item = this.get('activeGroup.upgradeItems') && this.get('activeGroup.upgradeItems').findProperty('context', this.get("controller.slaveFailuresContext"));
    var status = item && item.get('status');
    this.set('isOutOfSync', status === 'OUT_OF_SYNC');
    return ['HOLDING', 'HOLDING_FAILED', 'OUT_OF_SYNC'].contains(status);
  }.property('activeGroup.upgradeItems.@each.status', 'activeGroup.upgradeItems.@each.context'),

  /**
   * manualItem: indicate whether the step is "Service check failures", a dialog with instructions will show up for manual steps
   * @type {boolean}
   */
  isServiceCheckFailuresItem: Em.computed.equalProperties('manualItem.context', 'controller.serviceCheckFailuresContext'),

  /**
   * manualItem: indicate whether the step is Finalize
   * @type {boolean}
   */
  isFinalizeItem: Em.computed.equalProperties('manualItem.context', 'controller.finalizeContext'),

  /**
   * Upgrade of PATCH version is revertible
   */
  isRevertibleUpgrade: function () {
    var associatedVersion = this.get('controller.upgradeData.Upgrade.associated_version');
    var upgradeVersion = App.RepositoryVersion.find().findProperty('repositoryVersion', associatedVersion);
    return ['PATCH'].contains(upgradeVersion.get('type'));
  }.property('controller.upgradeData.Upgrade.associated_version'),

  revertibleFinalizeMessage: function () {
    var associatedVersion = this.get('controller.upgradeData.Upgrade.associated_version');
    var upgradeVersion = App.RepositoryVersion.find().findProperty('repositoryVersion', associatedVersion);
    return Em.I18n.t('admin.stackUpgrade.finalize.message.revertible').format(upgradeVersion.get('type'), upgradeVersion.get('displayName'));
  }.property('controller.upgradeData.Upgrade.associated_version'),

  /**
   * label of Upgrade status
   * @type {string}
   */
  upgradeStatusLabel: function () {
    var labelKey = null;
    switch (this.get('controller.upgradeData.Upgrade.request_status')) {
      case 'QUEUED':
      case 'PENDING':
      case 'IN_PROGRESS':
        labelKey = 'admin.stackUpgrade.state.inProgress';
        break;
      case 'COMPLETED':
        labelKey = 'admin.stackUpgrade.state.completed';
        break;
      case 'ABORTED':
        labelKey = 'admin.stackUpgrade.state.paused';
        break;
      case 'TIMEDOUT':
      case 'FAILED':
      case 'HOLDING_FAILED':
      case 'HOLDING_TIMEDOUT':
      case 'HOLDING':
        labelKey = 'admin.stackUpgrade.state.paused';
        break;
      default:
        labelKey = 'admin.stackUpgrade.state.init';
        break;
    }
    if (labelKey) {
      labelKey += this.get('controller.isDowngrade') ? '.downgrade' : "";
      return Em.I18n.t(labelKey);
    } else {
      return "";
    }
  }.property('controller.upgradeData.Upgrade.request_status', 'controller.isDowngrade'),

  /**
   * toggle details box
   */
  toggleDetails: function toggleDetails() {
    this.toggleProperty('isDetailsOpened');
  },

  /**
   * close details block if no active task present
   */
  closeDetails: function () {
    if (this.get('noActiveItem')) {
      this.set('isDetailsOpened', false);
    }
  }.observes('noActiveItem'),

  /**
   * start polling upgrade data
   */
  startPolling: function () {
    var self = this;
    if (App.get('clusterName')) {
      this.get('controller').loadUpgradeData().done(function () {
        self.set('isLoaded', true);
        self.doPolling();
      });
    }
  }.observes('App.clusterName'),

  getSkippedServiceChecks: function () {
    if (this.get('isFinalizeItem')) {
      if (!this.get('controller.areSkippedServiceChecksLoaded')) {
        var self = this;
        App.ajax.send({
          name: 'admin.upgrade.service_checks',
          sender: this,
          data: {
            upgradeId: this.get('controller.upgradeId')
          },
          success: 'getSkippedServiceChecksSuccessCallback'
        }).complete(function () {
          self.set('controller.areSkippedServiceChecksLoaded', true);
        });
      }
    } else {
      this.set('controller.areSkippedServiceChecksLoaded', false);
    }
  }.observes('isFinalizeItem'),

  getSkippedServiceChecksSuccessCallback: function getSkippedServiceChecksSuccessCallback(data) {
    if (data.items && data.items.length) {
      var lastItemWithChecks = data.items[data.items.length - 1];
      if (lastItemWithChecks && lastItemWithChecks.upgrade_items && lastItemWithChecks.upgrade_items.length) {
        var skippedServiceChecks = [];
        lastItemWithChecks.upgrade_items.forEach(function (item) {
          if (item.tasks && item.tasks.length) {
            item.tasks.forEach(function (task) {
              var detail = Em.get(task, 'Tasks.command_detail');
              if (detail && detail.startsWith('SERVICE_CHECK ')) {
                skippedServiceChecks.push(App.format.role(detail.replace('SERVICE_CHECK ', ''), true));
              }
            });
          }
        });
        skippedServiceChecks = skippedServiceChecks.uniq();
        this.set('controller.skippedServiceChecks', skippedServiceChecks);
      }
    }
  },

  /**
   * get slave-component failure hosts
   */
  getSlaveComponentItem: function () {
    var controller = this.get('controller');
    if (this.get('isSlaveComponentFailuresItem')) {
      if (!this.get('controller.areSlaveComponentFailuresHostsLoaded')) {
        var item = this.get('activeGroup.upgradeItems') && this.get('activeGroup.upgradeItems').findProperty('context', this.get("controller.slaveFailuresContext"));
        controller.getUpgradeItem(item, 'getSlaveComponentItemSuccessCallback').complete(function () {
          controller.set('areSlaveComponentFailuresHostsLoaded', true);
        });
      }
    } else {
      controller.set('areSlaveComponentFailuresHostsLoaded', false);
    }
  }.observes('isSlaveComponentFailuresItem'),

  /**
   * get service names of Service Check failures
   */
  getServiceCheckItem: function () {
    var controller = this.get('controller');
    if (this.get('isServiceCheckFailuresItem')) {
      if (!this.get('controller.areServiceCheckFailuresServicenamesLoaded')) {
        controller.getUpgradeItem(this.get('manualItem'), 'getServiceCheckItemSuccessCallback').complete(function () {
          controller.set('areServiceCheckFailuresServicenamesLoaded', true);
        });
      }
    } else {
      controller.set('areServiceCheckFailuresServicenamesLoaded', false);
    }
  }.observes('isServiceCheckFailuresItem'),

  /**
   * start polling upgrade data
   */
  willInsertElement: function willInsertElement() {
    this.startPolling();
  },

  /**
   * stop polling upgrade data
   */
  willDestroyElement: function willDestroyElement() {
    clearTimeout(this.get('updateTimer'));
    clearTimeout(this.get('upgradeItemTimer'));
    this.set('isLoaded', false);
  },

  /**
   * load upgrade data with time interval
   */
  doPolling: function doPolling() {
    var self = this;
    this.set('updateTimer', setTimeout(function () {
      self.get('controller').loadUpgradeData().done(function () {
        self.doPolling();
      });
    }, App.bgOperationsUpdateInterval));
  },

  /**
   * previous item request
   */
  prevItemRequest: null,

  /**
   * poll for tasks when item is expanded
   */
  doUpgradeItemPolling: function () {
    var self = this;
    var item = this.get('runningItem') || this.get('failedItem');
    var request = this.get('prevItemRequest');
    if (request) request.abort();
    if (item && this.get('isDetailsOpened')) {
      request = this.get('controller').getUpgradeItem(item).complete(function () {
        self.set('upgradeItemTimer', setTimeout(function () {
          self.doUpgradeItemPolling();
        }, App.bgOperationsUpdateInterval));
      });

      this.set('prevItemRequest', request);
    } else {
      clearTimeout(this.get('upgradeItemTimer'));
    }
  }.observes('isDetailsOpened', 'runningItem', 'failedItem'),

  /**
   * set current upgrade item state to FAILED (for HOLDING_FAILED) or TIMED_OUT (for HOLDING_TIMED_OUT)
   * in order to ignore fail and continue Upgrade
   * @param {object} event
   */
  continue: function _continue(event) {
    this.get('controller').setUpgradeItemStatus(event.context, event.context.get('status').slice(8));
    this.set('isDetailsOpened', false);
  },

  /**
   * set current upgrade item state to PENDING in order to retry Upgrade
   * @param {object} event
   */
  retry: function retry(event) {
    this.get('controller').setUpgradeItemStatus(event.context, 'PENDING');
    this.set('isDetailsOpened', false);
  },

  /**
   * set current upgrade item state to COMPLETED in order to proceed
   * @param {object} event
   */
  complete: function complete(event) {
    this.get('controller').setUpgradeItemStatus(event.context, 'COMPLETED');
    this.set('isManualDone', false);
  },

  pauseUpgrade: function pauseUpgrade() {
    var self = this;
    this.get('controller').suspendUpgrade().done(function () {
      self.get('parentView').closeWizard();
    });
  },

  /**
   * pause upgrade confirmation popup
   * @param {object} event
   */
  confirmPauseUpgrade: function confirmPauseUpgrade() {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.pauseUpgrade();
    }, Em.I18n.t('admin.stackUpgrade.pauseUpgrade.warning').format("upgrade"), null, Em.I18n.t('common.warning'), Em.I18n.t('admin.stackUpgrade.pauseUpgrade'));
  },

  /**
   * pause downgrade confirmation popup
   * @param {object} event
   */
  confirmPauseDowngrade: function confirmPauseDowngrade() {
    var self = this;
    return App.showConfirmationPopup(function () {
      self.pauseUpgrade();
    }, Em.I18n.t('admin.stackUpgrade.pauseUpgrade.warning').format("downgrade"), null, Em.I18n.t('common.warning'), Em.I18n.t('admin.stackUpgrade.pauseDowngrade'));
  },

  /**
   * @type {string}
   */
  failedHostsMessage: function () {
    var count = this.get('controller.slaveComponentStructuredInfo.hosts.length') || 0;
    return Em.I18n.t('admin.stackUpgrade.failedHosts.showHosts').format(count);
  }.property('controller.slaveComponentStructuredInfo.hosts'),

  showFailedHosts: function showFailedHosts() {
    return App.ModalPopup.show({
      header: Em.I18n.t('admin.stackUpgrade.failedHosts.header'),
      bodyClass: App.FailedHostsPopupBodyView,
      secondary: null,
      primary: Em.I18n.t('common.close'),
      content: this.get('controller.slaveComponentStructuredInfo')
    });
  }
});

});

require.register("views/main/admin/stack_upgrade/versions_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.MainAdminStackVersionsView = Em.View.extend({
  templateName: require('templates/main/admin/stack_upgrade/versions'),

  /**
   * update timer
   * @type {number|null}
   * @default null
   */
  updateTimer: null,

  /**
   * @type {Array}
   */
  services: App.Service.find(),

  /**
   * Not Installed = the version is not installed or out of sync
   * Upgrade Ready = the version is installed and ready for upgrade
   * Current = the version currently being used
   * Upgrade in Process = UPGRADING
   * Ready to Finalize = UPGRADED
   * Installed = All the versions that are installed BUT cannot be upgraded to (meaning: they are lower than the current)
   * @type {Array}
   */
  categories: [Em.Object.create({
    labelKey: 'admin.stackVersions.filter.all',
    value: '',
    isSelected: true
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.notInstalled',
    value: 'NOT_INSTALLED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.upgradeReady',
    value: 'UPGRADE_READY',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.current',
    value: 'CURRENT',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.installed',
    value: 'INSTALLED',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.upgrading',
    value: 'UPGRADING',
    isSelected: false
  }), Em.Object.create({
    labelKey: 'admin.stackVersions.filter.upgraded',
    value: 'UPGRADED',
    isSelected: false
  })],

  didInsertElement: function didInsertElement() {
    this.observesCategories();
  },

  /**
   * update categories labels
   */
  observesCategories: function () {
    this.get('categories').forEach(function (category) {
      category.set('label', Em.I18n.t(category.labelKey).format(this.filterBy(this.get('repoVersions'), category).length));
    }, this);
    this.filterVersions(this.get('selectedCategory'));
  }.observes('repoVersions.@each.stackVersion.state', 'controller.isLoaded'),

  /**
   * select category
   * @param event
   */
  selectCategory: function selectCategory(event) {
    this.get('categories').filterProperty('isSelected').setEach('isSelected', false);
    event.context.set('isSelected', true);
    this.filterVersions(event.context);
  },

  /**
   * filter versions that match category
   * @param {Em.Object} category
   */
  filterVersions: function filterVersions(category) {
    var filteredVersionIds = this.filterBy(this.get('repoVersions'), category).mapProperty('id');
    this.get('repoVersions').forEach(function (version) {
      version.set('isVisible', filteredVersionIds.contains(version.get('id')));
    });
  },

  /**
   * @type {object}
   */
  selectedCategory: Em.computed.findBy('categories', 'isSelected', true),

  /**
   * @type {Em.Array}
   */
  repoVersions: App.RepositoryVersion.find(),

  hasMaintRepoVersion: Em.computed.someBy('repoVersions', 'isMaint', true),

  hasPatchRepoVersion: Em.computed.someBy('repoVersions', 'isPatch', true),

  hasServiceRepoVersion: Em.computed.someBy('repoVersions', 'isService', true),

  hasSpecialTypeRepoVersion: Em.computed.or('hasMaintRepoVersion', 'hasPatchRepoVersion', 'hasServiceRepoVersion'),

  /**
   * @type {Em.Array}
   */
  stackVersions: App.StackVersion.find(),

  /**
   * @type {?Em.Object}
   */
  stackVersionError: function () {
    var errorStack = this.get('repoVersions').filterProperty('isVisible').filterProperty('status', 'OUT_OF_SYNC').findProperty('isStandard');
    if (errorStack) {
      return Em.Object.create({
        repoId: errorStack.get('id'),
        title: Em.I18n.t('admin.stackVersions.version.errors.outOfSync.title'),
        description: Em.I18n.t('admin.stackVersions.version.errors.outOfSync.desc'),
        stack: errorStack.get('displayNameFull')
      });
    }
    return null;
  }.property('repoVersions.@each.status'),

  /**
   * filter versions by category
   * @param versions
   * @param filter
   * @return {Array}
   */
  filterBy: function filterBy(versions, filter) {
    var currentVersion = this.get('controller.currentVersion');
    if (filter && filter.get('value')) {
      versions = versions.filter(function (version) {
        var status = version.get('status');
        var isUpgrading = this.isVersionUpgrading(version);
        if (status === 'INSTALLED' && ['UPGRADE_READY', 'INSTALLED', 'UPGRADING'].contains(filter.get('value'))) {
          if (filter.get('value') === 'UPGRADING') {
            return isUpgrading;
          } else if (filter.get('value') === 'UPGRADE_READY') {
            return !isUpgrading && stringUtils.compareVersions(version.get('repositoryVersion'), Em.get(currentVersion, 'repository_version')) === 1;
          } else if (filter.get('value') === 'INSTALLED') {
            return stringUtils.compareVersions(version.get('repositoryVersion'), Em.get(currentVersion, 'repository_version')) < 1;
          }
        } else if (filter.get('value') === 'NOT_INSTALLED') {
          return ['NOT_REQUIRED', 'INSTALL_FAILED', 'INSTALLING', 'OUT_OF_SYNC'].contains(status);
        } else {
          return status === filter.get('value');
        }
      }, this);
    }
    if (App.get('supports.displayOlderVersions') || Em.isNone(currentVersion)) {
      return versions.filterProperty('hidden', false).toArray();
    } else {
      return versions.filterProperty('hidden', false).filter(function (v) {
        if (v.get('stackVersionType') === Em.get(currentVersion, 'stack_name')) {
          // PATCH or MAINT version should be visible even if patch number lower than current
          return v.get('isPatch') || v.get('isMaint') || stringUtils.compareVersions(v.get('repositoryVersion'), Em.get(currentVersion, 'repository_version')) >= 0;
        }
        return v.get('isCompatible');
      }).toArray();
    }
  },

  /**
   * is version in upgrading or downgrading state
   * @param version
   */
  isVersionUpgrading: function isVersionUpgrading(version) {
    var upgradeController = App.router.get('mainAdminStackAndUpgradeController');
    return upgradeController.get('upgradeVersion') === version.get('displayName') || upgradeController.get('fromVersion') === version.get('repositoryVersion');
  },

  /**
   * route to versions in Admin View
   * @return {App.ModalPopup}
   */
  goToVersions: function goToVersions() {
    var self = this;
    return App.showConfirmationPopup(function () {
      App.ajax.send({
        name: 'ambari.service.load_server_version',
        sender: self
      }).then(function (data) {
        var components = Em.get(data, 'components');
        if (Em.isArray(components)) {
          var mappedVersions = components.map(function (component) {
            if (Em.get(component, 'RootServiceComponents.component_version')) {
              return Em.get(component, 'RootServiceComponents.component_version');
            }
          }),
              sortedMappedVersions = mappedVersions.sort(),
              latestVersion = sortedMappedVersions[sortedMappedVersions.length - 1].replace(/[^\d.-]/g, '');
          window.location.replace(App.appURLRoot + 'views/ADMIN_VIEW/' + latestVersion + '/INSTANCE/#/stackVersions');
        }
      });
    }, Em.I18n.t('admin.stackVersions.manageVersions.popup.body'), null, Em.I18n.t('admin.stackVersions.manageVersions'));
  },

  /**
   * load ClusterStackVersions data
   */
  willInsertElement: function willInsertElement() {
    this.poll();
  },

  /**
   * stop polling upgrade state
   */
  willDestroyElement: function willDestroyElement() {
    window.clearTimeout(this.get('updateTimer'));
    App.ajax.abortRequests(this.get('controller.runningCheckRequests'));
  },

  /**
   * set timer for polling
   */
  doPolling: function doPolling() {
    var self = this;
    this.set('updateTimer', window.setTimeout(function () {
      self.poll.apply(self);
    }, App.bgOperationsUpdateInterval));
  },

  /**
   * poll data
   */
  poll: function poll() {
    var self = this;
    //skip call if Upgrade wizard opened
    if (App.router.get('updateController').get('isWorking')) {
      this.get('controller').load().done(function () {
        self.set('controller.isLoaded', true);
        self.doPolling();
      });
    }
  }

});

});

require.register("views/main/alert_definitions_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var sort = require('views/common/sort_view');

App.MainAlertDefinitionsView = App.TableView.extend({

  templateName: require('templates/main/alerts'),

  content: [],

  contentObs: function () {
    Em.run.once(this, this.contentObsOnce);
  }.observes('controller.content.[]', 'App.router.clusterController.isAlertsLoaded'),

  contentObsOnce: function contentObsOnce() {
    var content = this.get('controller.content') && App.get('router.clusterController.isAlertsLoaded') ? this.get('controller.content').toArray() : [];
    if (this.get('childViews').someProperty('name', 'SortWrapperView')) {
      content = this.get('childViews').findProperty('name', 'SortWrapperView').getSortedContent(content);
    }
    this.set('content', content);
  },

  willInsertElement: function willInsertElement() {
    if (!this.get('controller.showFilterConditionsFirstLoad')) {
      this.clearFilterConditionsFromLocalStorage();
    }
    // on load alters should be sorted by status
    var controllerName = this.get('controller.name'),
        savedSortConditions = App.db.getSortingStatuses(controllerName) || [];
    if (savedSortConditions.everyProperty('status', 'sorting')) {
      savedSortConditions.push({
        name: "summary",
        status: "sorting_desc"
      });
      App.db.setSortingStatuses(controllerName, savedSortConditions);
    }
    this._super();
  },

  didInsertElement: function didInsertElement() {
    var self = this;
    Em.run.next(function () {
      self.set('isInitialRendering', false);
      self.contentObsOnce();
      self.tooltipsUpdater();
    });
  },

  willDestroyElement: function willDestroyElement() {
    $(".timeago").tooltip('destroy');
    this.removeObserver('pageContent.length', this, 'tooltipsUpdater');
  },

  /**
   * Save <code>startIndex</code> to the localStorage
   * @method saveStartIndex
   */
  saveStartIndex: function () {
    App.db.setStartIndex(this.get('controller.name'), this.get('startIndex'));
  }.observes('startIndex'),

  /**
   * Clear <code>startIndex</code> from the localStorage
   * @method clearStartIndex
   */
  clearStartIndex: function clearStartIndex() {
    App.db.setStartIndex(this.get('controller.name'), null);
  },

  /**
   * @type {number}
   */
  totalCount: Em.computed.alias('content.length'),

  colPropAssoc: ['', 'label', 'summary', 'serviceDisplayName', 'type', 'lastTriggered', 'enabled', 'groups'],

  sortView: sort.wrapperView,

  /**
   * Define whether initial view rendering has finished
   * @type {Boolean}
   */
  isInitialRendering: true,

  /**
   * Sorting header for <label>alertDefinition.label</label>
   * @type {Em.View}
   */
  nameSort: sort.fieldView.extend({
    column: 1,
    name: 'label',
    displayName: Em.I18n.t('alerts.table.header.definitionName')
  }),

  /**
   * Sorting header for <label>alertDefinition.status</label>
   * @type {Em.View}
   */
  statusSort: sort.fieldView.extend({
    column: 2,
    name: 'summary',
    displayName: Em.I18n.t('common.status'),
    type: 'alert_status',
    status: 'sorting_desc'
  }),

  /**
   * Sorting header for <label>alertDefinition.service.serviceName</label>
   * @type {Em.View}
   */
  serviceSort: sort.fieldView.extend({
    column: 3,
    name: 'serviceDisplayName',
    displayName: Em.I18n.t('common.service'),
    type: 'string'
  }),

  /**
   * Sorting header for <label>alertDefinition.type</label>
   * @type {Em.View}
   */
  typeSort: sort.fieldView.extend({
    column: 4,
    name: 'type',
    displayName: Em.I18n.t('common.type'),
    type: 'string'
  }),

  /**
   * Sorting header for <label>alertDefinition.lastTriggeredSort</label>
   * @type {Em.View}
   */
  lastTriggeredSort: sort.fieldView.extend({
    column: 5,
    name: 'lastTriggered',
    displayName: Em.I18n.t('alerts.table.header.lastTriggered'),
    type: 'number'
  }),

  /**
   * Sorting header for <label>alertDefinition.enabled</label>
   * @type {Em.View}
   */
  enabledSort: sort.fieldView.extend({
    template: Em.Handlebars.compile('<span {{bindAttr class="view.status :column-name"}}>{{t alerts.table.state}}</span>'),
    column: 6,
    name: 'enabled'
  }),

  /**
   * Filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('alerts.filters.filteredAlertsInfo', 'filteredCount', 'totalCount'),

  /**
   * Determines how display "back"-link - as link or text
   * @type {string}
   */
  paginationLeftClass: function () {
    if (this.get("startIndex") > 1) {
      return "paginate_previous";
    }
    return "paginate_disabled_previous";
  }.property("startIndex", 'filteredCount'),

  /**
   * Determines how display "next"-link - as link or text
   * @type {string}
   */
  paginationRightClass: function () {
    if (this.get("endIndex") < this.get("filteredCount")) {
      return "paginate_next";
    }
    return "paginate_disabled_next";
  }.property("endIndex", 'filteredCount'),

  /**
   * Show previous-page if user not in the first page
   * @method previousPage
   */
  previousPage: function previousPage() {
    if (this.get('paginationLeftClass') === 'paginate_previous') {
      this._super();
    }
    this.tooltipsUpdater();
  },

  /**
   * Show next-page if user not in the last page
   * @method nextPage
   */
  nextPage: function nextPage() {
    if (this.get('paginationRightClass') === 'paginate_next') {
      this._super();
    }
    this.tooltipsUpdater();
  },

  /**
   * Update tooltips when <code>pageContent</code> is changed
   * @method tooltipsUpdater
   */
  tooltipsUpdater: function tooltipsUpdater() {
    Em.run.next(this, function () {
      App.tooltip($(".timeago"));
    });
  },

  updateFilter: function updateFilter(iColumn, value, type) {
    if (!this.get('isInitialRendering')) {
      this._super(iColumn, value, type);
    }
    this.tooltipsUpdater();
  }

});

});

require.register("views/main/alerts", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAlertsView = App.TableView.extend(App.TableServerViewMixin, {
  templateName: require('templates/main/alerts'),

  tableName: 'Alerts'
});

});

require.register("views/main/alerts/add_alert_definition/add_alert_definition_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/alerts/add_alert_definition/add_alert_definition')

});

});

require.register("views/main/alerts/add_alert_definition/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep1View = Em.View.extend({

  templateName: require('templates/main/alerts/add_alert_definition/step1'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
    Em.run.later(this, function () {
      App.tooltip($("[rel='selectable-tooltip']"));
    }, 300);
  }

});

});

require.register("views/main/alerts/add_alert_definition/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep2View = Em.View.extend({

  templateName: require('templates/main/alerts/add_alert_definition/step2')

});

});

require.register("views/main/alerts/add_alert_definition/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddAlertDefinitionStep3View = Em.View.extend({

  /**
   * @type {string}
   */
  alertDefinitionToDisplay: '',

  willInsertElement: function willInsertElement() {
    this.set('alertDefinitionToDisplay', JSON.stringify(this.get('controller.content.formattedToRequestConfigs'), null, 4));
  },

  templateName: require('templates/main/alerts/add_alert_definition/step3')

});

});

require.register("views/main/alerts/alert_definition/alert_definition_state", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.AlertDefinitionState = Em.View.extend({

  templateName: require('templates/main/alerts/alert_definition/alert_definition_state'),

  labelText: Em.computed.ifThenElse('content.enabled', Em.I18n.t('alerts.table.state.enabled'), Em.I18n.t('alerts.table.state.disabled')),

  tooltipText: Em.computed.ifThenElse('content.enabled', Em.I18n.t('alerts.table.state.enabled.tooltip'), Em.I18n.t('alerts.table.state.disabled.tooltip')),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$(".enable-disable-button"));
  },
  willDestroyElement: function willDestroyElement() {
    this.$(".enable-disable-button").tooltip('destroy');
  }
});

});

require.register("views/main/alerts/alert_definition/alert_definition_summary", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.AlertDefinitionSummary = Em.View.extend({

  templateName: require('templates/main/alerts/alert_definition/alert_definition_summary'),

  /**
   * Bound from the template
   * @type {App.AlertDefinition}
   */
  content: null,

  hasMultipleCount: Em.computed.gt('content.hostCnt', 0),

  isActiveDefinitionState: Em.computed.and('hasMultipleCount', 'content.enabled'),

  definitionState: function () {
    var content = this.get('content');
    if (!content) {
      return [];
    }
    var order = content.get('order');
    var summary = content.get('summary');
    var hostCnt = content.get('hostCnt');
    var showCounts = hostCnt > 1;
    var ret = [];
    order.forEach(function (state) {
      var shortState = App.AlertDefinition.shortState[state];
      var _stateSummary = {
        state: 'alert-state-' + state,
        count: '',
        maintenanceCount: ''
      };
      if (summary[state]) {
        if (summary[state].count) {
          var count = shortState;
          if (showCounts) {
            count += ' (' + summary[state].count + ')';
          }
          _stateSummary.count = count;
        }
        // add status with maintenance mode icon
        if (summary[state].maintenanceCount) {
          var maintenanceCount = shortState;
          if (showCounts) {
            maintenanceCount += ' (' + summary[state].maintenanceCount + ')';
          }
          _stateSummary.maintenanceCount = maintenanceCount;
        }
      }
      ret.push(_stateSummary);
    });
    return ret;
  }.property('content.summary', 'content.hostCnt')

});

});

require.register("views/main/alerts/alert_definitions_actions_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.MainAlertDefinitionActionsView = Em.View.extend({

  classNames: ['btn-group', 'pull-right'],

  templateName: require('templates/main/alerts/alert_definitions_actions')

});

});

require.register("views/main/alerts/alert_instances_popup_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * Option for "filter by state" drop down
 * @typedef {object} categoryObject
 * @property {string} value "all|warning|critical"
 * @property {number} count number of items with <code>state</code> equal to <code>this.value</code>
 * @property {string} labelPath key in the messages.js
 * @property {string} label localized label
 */
var categoryObject = Em.Object.extend({
  value: '',
  count: 0,
  labelPath: '',
  label: function () {
    return Em.I18n.t(this.get('labelPath')).format(this.get('count'));
  }.property('count', 'labelPath')
});

App.AlertInstancesPopupView = App.TableView.extend(App.TableServerViewMixin, {

  templateName: require('templates/main/alerts/alert_notifications_popup'),

  updaterBinding: 'App.router.updateController',

  controllerBinding: 'App.router.mainAlertInstancesController',

  /**
   * Number of all critical and warning alert instances
   * @type {Boolean}
   */
  alertsNumberBinding: 'App.router.mainAlertDefinitionsController.unhealthyAlertInstancesCount',
  criticalNumberBinding: 'App.router.mainAlertDefinitionsController.criticalAlertInstancesCount',
  warningNumberBinding: 'App.router.mainAlertDefinitionsController.warningAlertInstancesCount',

  isPaginate: false,

  willInsertElement: function willInsertElement() {
    this._super();
    this.updateAlertInstances();
  },

  didInsertElement: function didInsertElement() {
    this.filter();
    this.addObserver('filteringComplete', this, this.overlayObserver);
    this.overlayObserver();
    this.$(".filter-select ").click(function (event) {
      return event.stopPropagation();
    });
    return this._super();
  },

  content: function () {
    return this.get('controller.unhealthyAlertInstances');
  }.property('controller.unhealthyAlertInstances.@each.state'),

  isLoaded: Em.computed.bool('controller.unhealthyAlertInstances'),

  isAlertEmptyList: Em.computed.or('isAlertContentEmptyList', 'isAlertFilterdEmptyList'),
  isAlertContentEmptyList: Em.computed.empty('content'),
  isAlertFilterdEmptyList: Em.computed.everyBy('content', 'isVisible', false),

  displayLength: 100,

  /**
   * Update list of shown alert instances
   * @method updateAlertInstances
   */
  updateAlertInstances: function () {
    var self = this,
        displayLength = this.get('displayLength'),
        startIndex = this.get('startIndex');
    if (!displayLength) return; // wait while table-info is loaded
    this.get('updater').set('queryParamsForUnhealthyAlertInstances', {
      from: startIndex - 1,
      page_size: displayLength
    });
    this.set('filteringComplete', false);
    this.get('updater').updateUnhealthyAlertInstances(function () {
      self.set('filteringComplete', true);
    });
  }.observes('displayLength', 'startIndex', 'alertsNumber'),

  /**
   * Show spinner when filter/sorting request is in processing
   * @method overlayObserver
   */
  overlayObserver: function overlayObserver() {
    var $tbody = this.$('.table.alerts-table'),
        $overlay = this.$('.table-overlay'),
        $spinner = $($overlay).find('.spinner');
    if (!this.get('filteringComplete')) {
      if (!$tbody) return;
      var tbodyPos = $tbody.position();
      if (!tbodyPos) return;
      $spinner.css('display', 'block');
      $overlay.css({
        top: tbodyPos.top + 1,
        left: tbodyPos.left + 1,
        width: $tbody.width() - 1,
        height: $tbody.height() - 1
      });
    }
  },

  categories: function () {
    var allCnt = this.get('alertsNumber');
    var criticalCnt = this.get('criticalNumber');
    var warningCnt = this.get('warningNumber');
    return [categoryObject.create({
      value: 'all',
      labelPath: 'alerts.dropdown.dialog.filters.all',
      count: allCnt
    }), categoryObject.create({
      value: 'alert-state-CRITICAL',
      labelPath: 'alerts.dropdown.dialog.filters.critical',
      count: criticalCnt
    }), categoryObject.create({
      value: 'alert-state-WARNING',
      labelPath: 'alerts.dropdown.dialog.filters.warning',
      count: warningCnt
    })];
  }.property('criticalNumber', 'warningNumber', 'alertsNumber'),

  selectedCategory: null,

  /**
   * Filter notifications by state
   * @method filter
   */
  filter: function () {
    var selectedState = this.get('selectedCategory.value');
    var content = this.get('content');
    if (selectedState == 'all') {
      content.setEach("isVisible", true);
      this.set('filteredContent', content);
    } else {
      this.set('filteredContent', content.filter(function (item) {
        if (item.get('stateClass') == selectedState) {
          item.set('isVisible', true);
          return true;
        } else {
          item.set('isVisible', false);
          return false;
        }
      }));
    }
  }.observes('content.length', 'selectedCategory'),

  /**
   * Router transition to alert definition details page
   * @param event
   */
  gotoAlertDetails: function gotoAlertDetails(event) {
    if (event && event.context) {
      var definition = App.AlertDefinition.find().findProperty('id', event.context.get('definitionId'));
      App.router.transitionTo('main.alerts.alertDetails', definition);
    }
  },

  /**
   * Router transition to alert definition page
   */
  gotoAllAlerts: function gotoAllAlerts() {
    App.router.transitionTo('main.alerts.index');
  }

});

});

require.register("views/main/alerts/alert_search_box", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAlertDefinitionSearchBoxView = App.SearchBoxView.extend({

  /**
   * describe filter columns
   * @type {Array}
   * @const
   */
  keyFilterMap: [{
    label: Em.I18n.t('common.status'),
    key: 'summary',
    type: 'alert_status',
    column: 2
  }, {
    label: Em.I18n.t('alerts.table.header.definitionName'),
    key: 'label',
    type: 'string',
    column: 1
  }, {
    label: Em.I18n.t('common.service'),
    key: 'serviceDisplayName',
    type: 'select',
    column: 3
  }, {
    label: Em.I18n.t('alerts.table.header.lastTriggered'),
    key: 'lastTriggered',
    type: 'date',
    column: 5
  }, {
    label: Em.I18n.t('alerts.table.state'),
    key: 'enabled',
    type: 'enable_disable',
    column: 6
  }, {
    label: Em.I18n.t('common.group'),
    key: 'groups',
    type: 'alert_group',
    column: 7
  }],

  enabledDisabledMap: {
    'enabled': Em.I18n.t('alerts.table.state.enabled'),
    'disabled': Em.I18n.t('alerts.table.state.disabled')
  },

  lastTriggeredOptions: ['Past 1 hour', 'Past 1 Day', 'Past 2 Days', 'Past 7 Days', 'Past 14 Days', 'Past 30 Days'],

  /**
   * populated dynamically in <code>getGroupsAvailableValues<code>
   * @type {object}
   */
  groupsNameIdMap: {},

  /**
   * 'valueMatches' callback for visualsearch.js
   * @param facetValue
   * @param searchTerm
   * @param callback
   */
  valueMatches: function valueMatches(facetValue, searchTerm, callback) {
    this.showHideClearButton();
    switch (this.get('keyFilterMap').findProperty('label', facetValue).key) {
      case 'summary':
        this.getSummaryAvailableValues(facetValue, callback);
        break;
      case 'label':
        this.getLabelAvailableValues(facetValue, callback);
        break;
      case 'serviceDisplayName':
        this.getServiceAvailableValues(facetValue, callback);
        break;
      case 'lastTriggered':
        this.getTriggeredAvailableValues(facetValue, callback);
        break;
      case 'enabled':
        this.getEnabledAvailableValues(facetValue, callback);
        break;
      case 'groups':
        this.getGroupsAvailableValues(facetValue, callback);
        break;
    }
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getSummaryAvailableValues: function getSummaryAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(Object.keys(App.AlertDefinition.shortState), facetValue), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getLabelAvailableValues: function getLabelAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(App.AlertDefinition.find().mapProperty('label').uniq(), facetValue));
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getServiceAvailableValues: function getServiceAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(App.AlertDefinition.find().mapProperty('serviceDisplayName').uniq(), facetValue));
  },

  /**
   *
   * @param {string} facetValue
   * @param {function} callback
   */
  getTriggeredAvailableValues: function getTriggeredAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(this.get('lastTriggeredOptions'), facetValue), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getEnabledAvailableValues: function getEnabledAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(Object.values(this.get('enabledDisabledMap')), facetValue), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getGroupsAvailableValues: function getGroupsAvailableValues(facetValue, callback) {
    var alertGroups = App.AlertGroup.find();
    var map = {};
    alertGroups.forEach(function (group) {
      map[group.get('displayName')] = group.get('id');
    });
    this.set('groupsNameIdMap', map);
    callback(this.rejectUsedValues(alertGroups.mapProperty('displayName'), facetValue));
  },

  /**
   *
   * @param {string} category
   * @param {string} label
   */
  mapLabelToValue: function mapLabelToValue(category, label) {
    var enabledDisabledMap = this.get('enabledDisabledMap');
    var groupsNameIdMap = this.get('groupsNameIdMap');

    switch (category) {
      case 'enabled':
        return Object.keys(enabledDisabledMap)[Object.values(enabledDisabledMap).indexOf(label)];
      case 'groups':
        return groupsNameIdMap[label];
      default:
        return label;
    }
  }
});

});

require.register("views/main/alerts/definition_configs_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AlertDefinitionConfigsView = Em.View.extend({

  controllerBinding: 'App.router.mainAlertDefinitionConfigsController',

  templateName: require('templates/main/alerts/configs'),

  /**
   * Define whether configs are editable
   * is set in template
   * @type {Boolean}
   */
  canEdit: true,

  /**
   * @type {string}
   */
  alertDefinitionType: '',

  formControlClass: 'form-control',

  init: function init() {
    this.set('controller.canEdit', this.get('canEdit'));
    this.set('controller.isWizard', this.get('isWizard'));
    this.set('controller.alertDefinitionType', this.get('alertDefinitionType'));
    this.set('controller.content', this.get('content'));
    this.get('controller').renderConfigs();
    this._super();
  },

  errorMessage: Em.I18n.t('alerts.definition.details.configs.thresholdsErrorMsg')

});

App.AlertConfigTextFieldView = Em.View.extend({
  templateName: require('templates/main/alerts/configs/alert_config_text_field'),
  classNameBindings: ['property.classNames'],
  classNames: ['row'],
  colWidth: function () {
    return this.get('property.colWidth') || 'col-md-12';
  }.property('property.colWidth')
});

App.AlertConfigTextAreaView = Em.View.extend({
  templateName: require('templates/main/alerts/configs/alert_config_text_area'),
  classNameBindings: ['property.classNames'],
  classNames: ['row']
});

App.AlertConfigSelectView = Em.Select.extend({
  attributeBindings: ['disabled'],
  selectionBinding: 'property.value',
  disabledBinding: 'property.isDisabled',
  contentBinding: 'property.options',
  classNameBindings: ['property.classNames']
});

App.AlertConfigThresholdView = Em.View.extend({
  templateName: require('templates/main/alerts/configs/alert_config_threshold'),
  classNameBindings: ['property.classNames'],
  classNames: ['row']
});

App.AlertConfigRadioButtonView = App.RadioButtonView.extend({
  nameBinding: 'property.group',
  checkedBinding: 'property.value',

  change: function change() {
    this.set('property.value', true);
    this.get('parentView.controller.configs').filterProperty('group', this.get('name')).without(this.get('property')).setEach('value', false);
    this.get('parentView.controller').changeType(this.get('property.name'));
  },

  classNameBindings: ['property.classNames']
});

App.AlertConfigParameterView = Em.View.extend({

  templateName: require('templates/main/alerts/configs/alert_config_parameter'),

  bigInput: Em.computed.equal('property.type', 'STRING'),

  classNameBindings: ['property.classNames'],
  classNames: ['row'],

  didInsertElement: function didInsertElement() {
    App.tooltip($("[rel='parameter-tooltip']"));
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='parameter-tooltip']").tooltip('destroy');
  }

});

});

require.register("views/main/alerts/definition_details_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var validator = require('utils/validator');
var filters = require('views/common/filter_view');
var sort = require('views/common/sort_view');

App.MainAlertDefinitionDetailsView = App.TableView.extend({

  templateName: require('templates/main/alerts/definition_details'),

  /**
   * Determines if <code>controller.content</code> is loaded
   * @type {bool}
   */
  isLoaded: false,

  /**
   * @type {string}
   */
  enabledDisplay: Em.I18n.t('alerts.table.state.enabled'),

  /**
   * @type {string}
   */
  disabledDisplay: Em.I18n.t('alerts.table.state.disabled'),

  colPropAssoc: ['serviceName', 'hostName', 'state'],

  /**
   * @type {string}
   */
  errorMessage: '',

  /**
   * return filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('tableView.filters.filteredAlertInstancesInfo', 'filteredCount', 'totalCount'),

  content: function () {
    return this.get('controller.alerts');
  }.property('controller.alerts.@each'),

  willInsertElement: function willInsertElement() {
    this._super();
    this.get('controller').clearStep();

    if (this.get('controller.content.isLoaded')) {
      this.set('isLoaded', true);
      this.get('controller').loadAlertInstances();
    } else {
      App.router.get('clusterController').addObserver('isAlertsLoaded', this, 'loadAfterModelLoaded');
    }
  },

  loadAfterModelLoaded: function loadAfterModelLoaded() {
    var clusterController = App.router.get('clusterController');
    if (clusterController.get('isAlertsLoaded')) {
      this.set('isLoaded', true);
      this.set('controller.content', App.AlertDefinition.find(parseInt(this.get('controller.content.id'))));
      this.get('controller').loadAlertInstances();
      clusterController.removeObserver('isAlertsLoaded', this, 'loadAfterModelLoaded');
    }
  },

  nameValidation: function () {
    var alertName = this.get('controller.editing.label.value').trim();
    var errorMessage = '';
    this.set('controller.editing.label.isError', false);
    if (alertName && !validator.isValidAlertName(alertName)) {
      errorMessage = Em.I18n.t("alert.definition.name.invalid");
      this.set('controller.editing.label.isError', true);
    }
    this.set('controller.errorMessage', errorMessage);
  }.observes('controller.editing.label.value'),

  didInsertElement: function didInsertElement() {
    this.filter();
    this.tooltipsUpdater();
  },

  /**
   * Update tooltips when <code>pageContent</code> is changed
   * @method tooltipsUpdater
   */
  tooltipsUpdater: function () {
    Em.run.next(function () {
      App.tooltip($(".enable-disable-button"));
    });
  }.observes('controller.content.enabled'),

  sortView: sort.wrapperView.extend({}),

  /**
   * Sorting header for <label>alertDefinition.label</label>
   * @type {Em.View}
   */
  serviceSort: sort.fieldView.extend({
    column: 0,
    name: 'serviceName',
    displayName: Em.I18n.t('common.service')
  }),

  /**
   * Sorting header for <label>alertDefinition.status</label>
   * @type {Em.View}
   */
  hostNameSort: sort.fieldView.extend({
    column: 1,
    name: 'hostName',
    displayName: Em.I18n.t('common.host')
  }),

  /**
   * Sorting header for <label>alertDefinition.service.serviceName</label>
   * @type {Em.View}
   */
  stateSort: sort.fieldView.extend({
    column: 2,
    name: 'state',
    displayName: Em.I18n.t('common.status')
  }),

  /**
   * Filtering header for <label>alertInstance.hostName</label>
   * @type {Em.View}
   */
  hostNameFilterView: filters.createTextView({
    column: 1,
    fieldType: 'input-medium',
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'string');
    }
  }),

  /**
   * Filtering header for <label>alertInstance.serviceName</label>
   * @type {Em.View}
   */
  serviceFilterView: filters.createSelectView({
    column: 0,
    fieldType: 'filter-input-width',
    content: filters.getComputedServicesList(),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * Filtering header for <label>alertInstance.state</label>
   * @type {Em.View}
   */
  stateFilterView: filters.createSelectView({
    column: 2,
    fieldType: 'filter-input-width',
    content: [{
      value: '',
      label: Em.I18n.t('common.all')
    }, {
      value: 'OK',
      label: 'OK'
    }, {
      value: 'WARNING',
      label: 'WARNING'
    }, {
      value: 'CRITICAL',
      label: 'CRITICAL'
    }, {
      value: 'UNKNOWN',
      label: 'UNKNOWN'
    }, {
      value: 'PENDING',
      label: 'NONE'
    }],
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * View calculates and represents count of alerts on appropriate host during last day
   */
  lastDayCount: Em.View.extend({
    hostName: '', // binding from template
    template: Ember.Handlebars.compile('<span>{{view.count}}</span>'),
    count: function () {
      var lastDayAlertsCount = this.get('parentView.controller.lastDayAlertsCount');
      return lastDayAlertsCount ? lastDayAlertsCount[this.get('hostName')] || 0 : Em.I18n.t('common.loading.eclipses');
    }.property('parentView.controller.lastDayAlertsCount', 'hostName')
  }),

  /**
   * View represents each row of instances table
   */
  instanceTableRow: Em.View.extend({
    tagName: 'tr',
    didInsertElement: function didInsertElement() {
      App.tooltip(this.$("[rel=tooltip]"));
      App.tooltip(this.$(".alert-text"), {
        placement: 'left',
        template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner alert-def-detail-tooltip"></div></div>'
      });
    },

    willDestroyElement: function willDestroyElement() {
      this.$("[rel=tooltip]").tooltip('destroy');
      this.$(".alert-text").tooltip('destroy');
    },

    /**
     * Router transition to service page
     * @param event
     */
    goToService: function goToService(event) {
      if (event && event.context) {
        App.router.transitionTo('main.services.service.summary', event.context);
      }
    },

    /**
     * Router transition to host level alerts page
     * @param {object} event
     * @method goToHostAlerts
     */
    goToHostAlerts: function goToHostAlerts(event) {
      if (event && event.context) {
        App.router.get('mainHostDetailsController').set('referer', App.router.location.lastSetURL);
        App.router.transitionTo('main.hosts.hostDetails.alerts', event.context);
      }
    },

    /**
     * open popup that contain full response of Alert Instance
     * @param {Object} event
     */
    openFullResponse: function openFullResponse(event) {
      App.showLogsPopup(Em.I18n.t('alerts.instance.fullLogPopup.header'), event.context.get('text'));
    }

  }),

  paginationLeftClass: function () {
    if (this.get("startIndex") > 1) {
      return "paginate_previous";
    }
    return "paginate_disabled_previous";
  }.property("startIndex", 'filteredCount'),

  /**
   * Determines how display "next"-link - as link or text
   * @type {string}
   */
  paginationRightClass: function () {
    if (this.get("endIndex") < this.get("filteredCount")) {
      return "paginate_next";
    }
    return "paginate_disabled_next";
  }.property("endIndex", 'filteredCount'),

  /**
   * Show previous-page if user not in the first page
   * @method previousPage
   */
  previousPage: function previousPage() {
    if (this.get('paginationLeftClass') === 'paginate_previous') {
      this._super();
    }
  },

  /**
   * Show next-page if user not in the last page
   * @method nextPage
   */
  nextPage: function nextPage() {
    if (this.get('paginationRightClass') === 'paginate_next') {
      this._super();
    }
  }

});

App.AlertInstanceServiceHostView = Em.View.extend({

  templateName: require('templates/main/alerts/instance_service_host'),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$("[rel='UsageTooltip']"));
  },

  willDestroyElement: function willDestroyElement() {
    this.$("[rel='UsageTooltip']").remove();
  },

  /**
   * Define whether show link for transition to service page
   */
  serviceIsLink: Em.computed.existsInByKey('instance.service.serviceName', 'App.services.all'),

  /**
   * Define whether show separator between service and hosts labels
   */
  showSeparator: Em.computed.and('instance.serviceDisplayName', 'instance.hostName')

});

App.AlertInstanceStateView = Em.View.extend({

  templateName: require('templates/main/alerts/alert_instance/status'),

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$("[rel='StateTooltip']"));
    App.tooltip(this.$("[rel='tooltip']"));
  },

  willDestroyElement: function willDestroyElement() {
    this.$("[rel='StateTooltip']").remove();
    this.$("[rel='tooltip']").remove();
  }

});

});

require.register("views/main/alerts/manage_alert_groups/select_definitions_popup_body_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SelectDefinitionsPopupBodyView = App.TableView.extend({

  templateName: require('templates/main/alerts/add_definition_to_group_popup'),

  controllerBinding: 'App.router.manageAlertGroupsController',

  isPaginate: true,

  filteredContent: [],

  filteredContentObs: function () {
    Em.run.once(this, this.filteredContentObsOnce);
  }.observes('parentView.availableDefs.@each.filtered'),

  filteredContentObsOnce: function filteredContentObsOnce() {
    var filtered = this.get('parentView.availableDefs').filterProperty('filtered') || [];
    this.set('filteredContent', filtered);
  },

  /**
   * @type {boolean}
   */
  showOnlySelectedDefs: false,

  filterComponent: null,

  filterService: null,

  /**
   * @type {boolean}
   */
  isDisabled: Em.computed.not('parentView.isLoaded'),

  didInsertElement: function didInsertElement() {
    var initialDefs = this.get('initialDefs');
    initialDefs.setEach('filtered', true);
    this.set('parentView.availableDefs', initialDefs);
    this.set('parentView.isLoaded', true);
    this.filteredContentObsOnce();
  },

  /**
   * Default filter isn't needed
   */
  filter: Em.K,

  /**
   * filter definitions by componentName and serviceName
   */
  filterDefs: function () {
    var showOnlySelectedDefs = this.get('showOnlySelectedDefs');
    var filterComponent = this.get('filterComponent');
    var filterService = this.get('filterService');

    this.get('parentView.availableDefs').forEach(function (defObj) {
      var matchComponent = filterComponent ? defObj.get('componentName') === filterComponent.get('componentName') : true,
          matchService = filterService ? defObj.get('serviceName') === filterService.get('serviceName') : true,
          filtered = matchComponent && matchService;

      defObj.set('filtered', showOnlySelectedDefs ? filtered && defObj.get('selected') : filtered);
    }, this);
    this.set('startIndex', 1);
  }.observes('parentView.availableDefs', 'filterService.serviceName', 'filterComponent.componentName', 'showOnlySelectedDefs'),

  defSelectMessage: function () {
    var defs = this.get('parentView.availableDefs');
    return this.t('alerts.actions.manage_alert_groups_popup.selectDefsDialog.selectedDefsLink').format(defs.filterProperty('selected').get('length'), defs.get('length'));
  }.property('parentView.availableDefs.@each.selected'),

  /**
   * apply component filter
   * @param {object|null} event
   */
  selectFilterComponent: function selectFilterComponent(event) {
    if (event && event.context.get('componentName')) {
      var currentFilter = this.get('filterComponent');
      if (currentFilter) {
        currentFilter.set('selected', false);
      }
      if (currentFilter && currentFilter.get('componentName') === event.context.get('componentName')) {
        // selecting the same filter deselects it.
        this.set('filterComponent', null);
      } else {
        this.set('filterComponent', event.context);
        event.context.set('selected', true);
      }
    }
  },

  /**
   * apply service filter
   * @param {object|null} event
   */
  selectFilterService: function selectFilterService(event) {
    if (event && event.context.get('serviceName')) {
      var currentFilter = this.get('filterService');
      if (currentFilter) {
        currentFilter.set('selected', false);
      }
      if (currentFilter && currentFilter.get('serviceName') === event.context.get('serviceName')) {
        // selecting the same filter deselects it.
        this.set('filterService', null);
      } else {
        this.set('filterService', event.context);
        event.context.set('selected', true);
      }
    }
  },

  /**
   * Determines if all alert definitions are selected
   * @type {boolean}
   */
  allDefsSelected: false,

  /**
   * Inverse selection for alert definitions
   * @method toggleSelectAllDefs
   */
  toggleSelectAllDefs: function () {
    this.get('parentView.availableDefs').filterProperty('filtered').setEach('selected', this.get('allDefsSelected'));
  }.observes('allDefsSelected'),

  toggleShowSelectedDefs: function toggleShowSelectedDefs() {
    if (this.get('filterComponent')) this.get('filterComponent').set('selected', false);
    if (this.get('filterService')) this.get('filterService').set('selected', false);
    this.set('filterComponent', null);
    this.set('filterService', null);
    this.toggleProperty('showOnlySelectedDefs');
  }
});

});

require.register("views/main/alerts/manage_alert_groups_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainAlertsManageAlertGroupView = Em.View.extend({

  templateName: require('templates/main/alerts/manage_alert_groups_popup'),

  /**
   * @type {object}
   */
  selectedAlertGroup: null,

  /**
   * Observer to prevent user select more than 1 alert group
   * @method onGroupSelect
   */
  onGroupSelect: function () {
    var selectedAlertGroup = this.get('selectedAlertGroup');
    // to unable user select more than one alert group at a time
    if (selectedAlertGroup && selectedAlertGroup.length) {
      this.set('controller.selectedAlertGroup', selectedAlertGroup[selectedAlertGroup.length - 1]);
    }
    if (selectedAlertGroup && selectedAlertGroup.length > 1) {
      this.set('selectedAlertGroup', selectedAlertGroup[selectedAlertGroup.length - 1]);
    }
    this.set('controller.selectedDefinitions', []);
  }.observes('selectedAlertGroup'),

  /**
   * Observer to provide two-way binding for group selection
   */
  setGroupInController: function () {
    if (!this.get('controller.isLoaded')) return;
    this.set('selectedAlertGroup', [this.get('controller.selectedAlertGroup')]);
  }.observes('controller.selectedAlertGroup'),

  /**
   * Select first alert group when all groups are loaded
   * @method onLoad
   */
  onLoad: function () {
    if (this.get('controller.isLoaded')) {
      this.set('selectedAlertGroup', [this.get('controller.alertGroups.firstObject')]);
      this.setTooltips();
    }
  }.observes('controller.isLoaded'),

  /**
   * Load alert groups, definitions and notification to have all new data
   * Useful if user delete some definition and immediately open "Manage Alert Groups" popup
   * @method willInsertElement
   */
  willInsertElement: function willInsertElement() {
    this.get('controller').loadAlertNotifications();
  },

  /**
   * Add tooltips and try to select first alert group
   * @method didInsertElement
   */
  didInsertElement: function didInsertElement() {
    this.onLoad();
  },

  /**
   * Setup tooltips for view elements
   */
  setTooltips: function setTooltips() {
    Em.run.next(function () {
      App.tooltip($("[rel='button-info']"));
      App.tooltip($("[rel='button-info-dropdown']"), { placement: 'left' });
    });
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='button-info']").tooltip('destroy');
    $("[rel='button-info-dropdown']").tooltip('destroy');
  },

  /**
   * Tooltip for "Add group"-button
   * @type {string}
   */
  addButtonTooltip: Em.I18n.t('alerts.actions.manage_alert_groups_popup.addButton'),

  /**
   * Tooltip for "Remove group"-button
   * @type {string}
   */
  removeButtonTooltip: Em.computed.ifThenElse('controller.isRemoveButtonDisabled', Em.I18n.t('alerts.actions.manage_alert_groups_popup.removeButtonDisabled'), Em.I18n.t('alerts.actions.manage_alert_groups_popup.removeButton')),

  /**
   * Tooltip for "Rename"-button
   * @type {string}
   */
  renameButtonTooltip: Em.I18n.t('alerts.actions.manage_alert_groups_popup.renameButton'),

  /**
   * Tooltip for "Duplicate group"-button
   * @type {string}
   */
  duplicateButtonTooltip: Em.I18n.t('alerts.actions.manage_alert_groups_popup.duplicateButton'),

  /**
   * Tooltip for "Add definition to group"-button
   * @type {string}
   */
  addDefinitionTooltip: function () {
    if (this.get('controller.selectedAlertGroup.default')) {
      return Em.I18n.t('alerts.actions.manage_alert_groups_popup.addDefinitionToDefault');
    }
    if (this.get('controller.selectedAlertGroup.isAddDefinitionsDisabled')) {
      return Em.I18n.t('alerts.actions.manage_alert_groups_popup.addDefinitionDisabled');
    }
    return Em.I18n.t('alerts.actions.manage_alert_groups_popup.addDefinition');
  }.property('controller.selectedAlertGroup.default', 'controller.selectedAlertGroup.isAddDefinitionsDisabled'),

  /**
   * Tooltip for "Remove definition from group"-button
   * @type {string}
   */
  removeDefinitionTooltip: function () {
    if (this.get('controller.selectedAlertGroup.default')) {
      return Em.I18n.t('alerts.actions.manage_alert_groups_popup.removeDefinitionDisabled');
    }
    if (this.get('controller.isDeleteDefinitionsDisabled')) {
      return Em.I18n.t('common.nothingToDelete');
    }
    return Em.I18n.t('alerts.actions.manage_alert_groups_popup.removeDefinition');
  }.property('controller.selectedAlertGroup.default', 'controller.isDeleteDefinitionsDisabled')

});

});

require.register("views/main/alerts/manage_alert_notifications_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ManageAlertNotificationsView = Em.View.extend({

  templateName: require('templates/main/alerts/manage_alert_notifications_popup'),

  /**
   * @type {App.AlertNotification}
   */
  selectedAlertNotification: null,

  selectedAlertNotificationGroups: function () {
    return this.get('controller.selectedAlertNotification.groups').toArray().mapProperty('displayName').join(', ');
  }.property('controller.selectedAlertNotification', 'controller.selectedAlertNotification.groups.@each.displayName', 'controller.isLoaded'),

  someAlertNotificationIsSelected: Em.computed.bool('controller.selectedAlertNotification'),

  /**
   * @type {boolean}
   */
  isEditButtonDisabled: Em.computed.or('!someAlertNotificationIsSelected', '!controller.selectedAlertNotification.enabled'),

  /**
   * @type {boolean}
   */
  isRemoveButtonDisabled: Em.computed.or('!someAlertNotificationIsSelected'),

  /**
   * @type {boolean}
   */
  isDuplicateButtonDisabled: Em.computed.or('!someAlertNotificationIsSelected', '!controller.selectedAlertNotification.enabled'),

  /**
   * @type {boolean}
   */
  isEnableOrDisableButtonDisabled: Em.computed.or('!someAlertNotificationIsSelected'),

  /**
   * Show EMAIL information if selected alert notification has type EMAIL
   * @type {boolean}
   */
  showEmailDetails: Em.computed.equal('controller.selectedAlertNotification.type', 'EMAIL'),

  /**
   * Show SNMP information if selected alert notification has type SNMP
   * @type {boolean}
   */
  showSNMPDetails: Em.computed.equal('controller.selectedAlertNotification.type', 'SNMP'),

  /**
   * Show Custom SNMP information if selected alert notification has type Custom SNMP
   * @type {boolean}
   */
  showCustomSNMPDetails: Em.computed.equal('controller.selectedAlertNotification.type', 'Custom SNMP'),

  email: function () {
    return this.get('controller.selectedAlertNotification.properties')['ambari.dispatch.recipients'];
  }.property('controller.selectedAlertNotification.properties'),

  /**
   * @type {string}
   */
  severities: function () {
    return this.get('controller.selectedAlertNotification.alertStates').join(', ');
  }.property('controller.selectedAlertNotification.alertStates'),

  selectedAlertNotificationTypeText: function () {
    return this.get('controller').getNotificationTypeText(this.get('controller.selectedAlertNotification.type'));
  }.property('controller.selectedAlertNotification', 'controller.isLoaded'),

  editAlertNotification: function editAlertNotification() {
    if (!this.get('isEditButtonDisabled')) {
      this.get('controller').editAlertNotification();
    }
  },

  duplicateAlertNotification: function duplicateAlertNotification() {
    if (!this.get('isDuplicateButtonDisabled')) {
      this.get('controller').duplicateAlertNotification();
    }
  },

  enableOrDisableAlertNotification: function enableOrDisableAlertNotification(e) {
    if (!this.get('isEnableOrDisableButtonDisabled')) {
      this.$("[rel='button-info-dropdown']").tooltip('destroy');
      this.get('controller').enableOrDisableAlertNotification(e);
    }
  },

  /**
   * Prevent user select more than 1 alert notification
   * @method onAlertNotificationSelect
   */
  onAlertNotificationSelect: function () {
    var selectedAlertNotification = this.get('selectedAlertNotification');
    if (selectedAlertNotification && selectedAlertNotification.length) {
      this.set('controller.selectedAlertNotification', selectedAlertNotification[selectedAlertNotification.length - 1]);
    }
    if (selectedAlertNotification && selectedAlertNotification.length > 1) {
      this.set('selectedAlertNotification', selectedAlertNotification[selectedAlertNotification.length - 1]);
    }
    if (this.$("[rel='button-info-dropdown']")) {
      this.$("[rel='button-info-dropdown']").tooltip('destroy');
    }
    Em.run.later(this, function () {
      App.tooltip(self.$("[rel='button-info-dropdown']").parent().not(".disabled").children(), { placement: 'left' });
    }, 50);
  }.observes('selectedAlertNotification'),

  /**
   * Set first alert notification as selected (if they are already loaded)
   * Add some tooltips on manage buttons
   * @method onLoad
   */
  onLoad: function () {
    var self = this;
    if (this.get('controller.isLoaded')) {
      var notifications = this.get('controller.alertNotifications');
      if (notifications && notifications.length) {
        this.set('selectedAlertNotification', this.get('controller.selectedAlertNotification') || notifications[0]);
      } else {
        this.set('selectedAlertNotification', null);
      }
      Em.run.later(this, function () {
        App.tooltip(self.$("[rel='button-info']"));
      }, 50);
    }
  }.observes('controller.isLoaded'),

  willDestroyElement: function willDestroyElement() {
    this.$("[rel='button-info']").tooltip('destroy');
    this.$("[rel='button-info-dropdown']").tooltip('destroy');
  },

  willInsertElement: function willInsertElement() {
    this.get('controller').loadAlertNotifications();
  },

  didInsertElement: function didInsertElement() {
    this.onLoad();
  }

});

});

require.register("views/main/charts", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainChartsView = Em.View.extend({
  templateName: require('templates/main/charts')
});

});

require.register("views/main/charts/heatmap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainChartsHeatmapView = Em.View.extend({
  templateName: require('templates/main/charts/heatmap'),
  didInsertElement: function didInsertElement() {
    // set default metric
    this.get('controller').loadPageData();
    $("#heatmapDetailsBlock").hide();
  },
  dropdownView: Em.View.extend({
    templateName: require('templates/main/charts/heatmap_dropdown')
  }),

  willDestroyElement: function willDestroyElement() {
    this._super();
    this.get('controller').clearActiveWidgetLayout();
  }
});

});

require.register("views/main/charts/heatmap/heatmap_host", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var date = require('utils/date/date');
var numberUtils = require('utils/number_utils');
var App = require('app');

App.MainChartsHeatmapHostView = Em.View.extend({
  templateName: require('templates/main/charts/heatmap/heatmap_host'),

  didInsertElement: function didInsertElement() {
    $("#heatmapDetailsBlock").hide();
  },

  /** @private */
  hostClass: 'hostBlock',

  /**
   * link to host record in App.Host model
   */
  hostModelLink: function () {
    return App.Host.find(this.get('content.hostName'));
  }.property('content.hostName'),

  /**
   * show Host details block and move it near the cursor
   *
   * @param {Object} event
   * @this App.MainChartsHeatmapHostView
   */
  mouseEnter: function mouseEnter(event) {
    var host = this.get('content'),
        view = App.MainChartsHeatmapHostDetailView.create();

    Object.keys(view.get('details')).forEach(function (i) {
      var val = host[i];

      switch (i) {
        case 'diskUsage':
          val = this.getUsage(host['diskTotal'], host['diskFree']);
          break;
        case 'cpuUsage':
          val = this.getCpuUsage(host['cpuSystem'], host['cpuUser']);
          break;
        case 'memoryUsage':
          val = this.getUsage(host['memTotal'], host['memFree']);
          break;
        case 'hostComponents':
          val = this.getHostComponents(host[i]);
      }

      view.set('details.' + i, val);
    }, this);
    this.setMetric(view, host);
    this.openDetailsBlock(event);
  },

  /**
   *
   * @param {string} number
   * @param {string} units
   * @returns {string}
   */
  convertValue: function convertValue(number, units) {
    if (Number.isFinite(Number(number)) && Number(number) > 0) {
      if (units === 'MB') {
        return (Number(number) / numberUtils.BYTES_IN_MB).toFixed(2) + units;
      } else if (units === 'ms') {
        return date.timingFormat(number);
      } else {
        return number + units;
      }
    }
    return number;
  },

  /**
   * get relative usage of metric in percents
   * @param {number} total
   * @param {number} free
   * @return {string}
   */
  getUsage: function getUsage(total, free) {
    var result = 0;

    if (Number.isFinite(total) && Number.isFinite(free) && total > 0) {
      result = (total - free) / total * 100;
    }
    return result.toFixed(1);
  },

  /**
   * get CPU usage
   * @param {number} cpuSystem
   * @param {number} cpuUser
   * @returns {string}
   */
  getCpuUsage: function getCpuUsage(cpuSystem, cpuUser) {
    var result = 0;

    if (Number.isFinite(cpuSystem) && Number.isFinite(cpuUser)) {
      result = cpuSystem + cpuUser;
    }
    return result.toFixed(1);
  },

  /**
   * get non-client host-components of host
   * @param {Array} components
   * @returns {string}
   */
  getHostComponents: function getHostComponents(components) {
    var nonClientComponents = App.get('components.slaves').concat(App.get('components.masters'));
    var result = [];

    components.forEach(function (componentName) {
      if (nonClientComponents.contains(componentName)) {
        result.push(App.format.role(componentName, false));
      }
    }, this);
    return result.join(', ');
  },

  /**
   * show tooltip with host's details
   */
  openDetailsBlock: function openDetailsBlock(event) {
    var detailsBlock = $("#heatmapDetailsBlock");

    detailsBlock.css('top', event.pageY + 10);
    detailsBlock.css('left', event.pageX + 10);
    detailsBlock.show();
  },

  /**
   * set name and value of selected metric
   * @param view
   * @param host
   */
  setMetric: function setMetric(view, host) {
    var selectedMetric = this.get('controller.selectedMetric');
    var hostToSlotMap = this.get('controller.hostToSlotMap');

    if (selectedMetric) {
      var slotDefinitions = selectedMetric.get('slotDefinitions');
      var metricName = selectedMetric.get('name');
      var h2vMap = selectedMetric.get('hostToValueMap');
      if (h2vMap && metricName) {
        var value = h2vMap[host.hostName];
        if (Em.isNone(value) || isNaN(Number(value))) {
          value = slotDefinitions[hostToSlotMap[host.hostName]].get('label');
        } else {
          value = this.convertValue(value, selectedMetric.get('units'));
        }
        view.set('details.metricName', metricName);
        view.set('details.metricValue', value);
      }
    }
  },

  /**
   * hide Host details block
   *
   * @param {Object} e
   * @this App.MainChartsHeatmapHostView
   */
  mouseLeave: function mouseLeave(e) {
    $("#heatmapDetailsBlock").hide();
  },

  hostTemperatureStyle: function () {
    var controller = this.get('controller');
    var h2sMap = controller.get('hostToSlotMap');
    if (h2sMap) {
      var hostname = this.get('content.hostName');
      if (hostname) {
        var slot = h2sMap[hostname];
        if (slot > -1) {
          var slotDefs = controller.get('selectedMetric.slotDefinitions');
          if (slotDefs && slotDefs.length > slot) {
            return slotDefs[slot].get('cssStyle');
          }
        }
      }
    }
    return '';
  }.property('controller.hostToSlotMap')
});

});

require.register("views/main/charts/heatmap/heatmap_host_detail", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainChartsHeatmapHostDetailView = Em.View.extend({
  templateName: require('templates/main/charts/heatmap/heatmap_host_detail'),
  /** @private */classNames: ['heatmap_host_details'],
  /** @private */elementId: 'heatmapDetailsBlock',
  /**
   * @private
   */
  details: {
    hostName: 'test node',
    publicHostName: 'test node',
    osType: 'OS',
    ip: '192.168.0.0',
    rack: '/default_rack',
    metricName: 'metric-name',
    metricValue: 'metric-value',
    diskUsage: '10',
    cpuUsage: '10',
    memoryUsage: '10',
    hostComponents: []
  }
});

});

require.register("views/main/charts/heatmap/heatmap_rack", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var lazyloading = require('utils/lazy_loading');

App.MainChartsHeatmapRackView = Em.View.extend({
  templateName: require('templates/main/charts/heatmap/heatmap_rack'),
  classNames: ['rack', 'panel', 'panel-default'],
  classNameBindings: ['visualSchema'],

  /** rack status block class */
  statusIndicator: 'statusIndicator',
  /** loaded hosts of rack */
  hosts: [],

  /**
   * display hosts of rack
   */
  displayHosts: function displayHosts() {
    var rackHosts = this.get('rack.hosts');
    var rackCount = this.get('controller.racks.length');

    if (this.get('hosts.length') === 0) {
      if (rackHosts.length > 100 && rackCount == 1) {
        lazyloading.run({
          initSize: 100,
          chunkSize: 200,
          delay: 100,
          destination: this.get('hosts'),
          source: rackHosts,
          context: this.get('rack')
        });
      } else {
        this.set('hosts', rackHosts);
        this.set('rack.isLoaded', true);
      }
    }
  },

  didInsertElement: function didInsertElement() {
    this.set('hosts', []);
    this.get('controller').addRackView(this);
  },
  /**
   * Provides the CSS style for an individual host.
   * This can be used as the 'style' attribute of element.
   */
  hostCssStyle: function () {
    var rack = this.get('rack');
    var hostCount = rack.get('hosts.length');
    if (hostCount >= 12) {
      return 'col-md-1';
    }
    if (hostCount === 1) {
      return 'col-md-12';
    }
    if (hostCount === 2) {
      return 'col-md-6';
    }
    if (hostCount === 3) {
      return 'col-md-4';
    }
    if (hostCount === 4) {
      return 'col-md-3';
    }
    return 'col-md-2';
  }.property('rack.isLoaded')
});

});

require.register("views/main/charts/menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainChartsMenuView = Em.CollectionView.extend({
  tagName: 'ul',
  classNames: ["nav", "nav-tabs"],
  content: [
    /*{ label:'Heatmap', routing:'heatmap', active:"active"},
    { label:'Horizon Chart', routing:'horizon_chart'}*/
  ],

  init: function init() {
    this._super();this.activateView();
  },

  activateView: function activateView() {
    $.each(this._childViews, function () {
      this.set('active', this.get('content.routing') == 'heatmap' ? "active" : "");
    });
  },

  deactivateChildViews: function deactivateChildViews() {
    $.each(this._childViews, function () {
      this.set('active', "");
    });
  },

  itemViewClass: Em.View.extend({
    classNameBindings: ["active"],
    active: "",
    template: Ember.Handlebars.compile('<a {{action showChart view.content.routing }} href="#"> {{unbound view.content.label}}</a>')
  })
});

});

require.register("views/main/dashboard", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainDashboardView = Em.View.extend({

  name: 'mainDashboardView',
  templateName: require('templates/main/dashboard'),

  selectedBinding: 'controller.selectedCategory',
  categories: [{
    name: 'widgets',
    url: 'dashboard.index',
    label: Em.I18n.t('dashboard.widgets.title')
  }, {
    name: 'charts',
    url: 'dashboard.charts.index',
    label: Em.I18n.t('dashboard.heatmaps.title')
  }, {
    name: 'configHistory',
    url: 'dashboard.configHistory',
    label: Em.I18n.t('dashboard.configHistory.title')
  }],

  NavItemView: Ember.View.extend({
    tagName: 'li',
    classNameBindings: 'isActive:active'.w(),
    isActive: Em.computed.equalProperties('item', 'parentView.selected'),
    elementId: Ember.computed(function () {
      var label = Em.get(this, 'templateData.keywords.category.label');
      return label ? 'dashboard-view-tab-' + label.toLowerCase().replace(/\s/g, '-') : "";
    })
  })
});

});

require.register("views/main/dashboard/cluster_metrics/cpu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing cluster CPU metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartClusterMetricsCPU = App.ChartLinearTimeView.extend({
  id: "cluster-metrics-cpu",

  ajaxIndex: 'dashboard.cluster_metrics.cpu',

  title: Em.I18n.t('dashboard.clusterMetrics.cpu'),
  displayUnit: '%',
  isTimePagingDisable: false,
  seriesTemplate: {
    path: 'metrics.cpu'
  },

  getData: function getData(jsonData) {
    var dataArray = [],
        idle = null,
        data = Em.get(jsonData, this.get('seriesTemplate.path'));
    if (data) {
      Object.keys(data).forEach(function (name) {
        var seriesData = data[name];
        if (seriesData) {
          var s = {
            name: name,
            data: seriesData
          };
          if (name.contains('Idle')) {
            //CPU idle metric should be the last in series array
            idle = s;
            return;
          }
          dataArray.push(s);
        }
      });
      if (idle) {
        dataArray.push(idle);
      }
    }
    return dataArray;
  },

  colorForSeries: function colorForSeries(series) {
    if (series.name && series.name.contains(Em.I18n.t('dashboard.clusterMetrics.cpu.displayNames.idle'))) {
      return '#CFECEC';
    }
    return null;
  }
});

});

require.register("views/main/dashboard/cluster_metrics/load", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing cluster load
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartClusterMetricsLoad = App.ChartLinearTimeView.extend({
  id: "cluster-metrics-load",

  ajaxIndex: 'dashboard.cluster_metrics.load',
  isTimePagingDisable: false,
  renderer: 'line',
  title: Em.I18n.t('dashboard.clusterMetrics.load'),
  seriesTemplate: {
    path: 'metrics.load'
  }
});

});

require.register("views/main/dashboard/cluster_metrics/memory", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing cluster memory metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartClusterMetricsMemory = App.ChartLinearTimeView.extend({
  id: "cluster-metrics-memory",

  ajaxIndex: 'dashboard.cluster_metrics.memory',

  isTimePagingDisable: false,
  title: Em.I18n.t('dashboard.clusterMetrics.memory'),
  displayUnit: 'B',
  renderer: 'line',
  seriesTemplate: {
    path: 'metrics.memory',
    factor: Math.pow(2, 10)
  }
});

});

require.register("views/main/dashboard/cluster_metrics/network", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing cluster network metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartClusterMetricsNetwork = App.ChartLinearTimeView.extend({
  id: "cluster-metrics-network",

  ajaxIndex: 'dashboard.cluster_metrics.network',

  isTimePagingDisable: false,
  title: Em.I18n.t('dashboard.clusterMetrics.network'),
  displayUnit: 'B',
  renderer: 'line',
  seriesTemplate: {
    path: 'metrics.network'
  }
});

});

require.register("views/main/dashboard/config_history_search_box", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainConfigHistorySearchBoxView = App.SearchBoxView.extend({

  /**
   * @type {Array}
   * @const
   */
  modifiedOptions: ['Past 1 hour', 'Past 1 Day', 'Past 2 Days', 'Past 7 Days', 'Past 14 Days', 'Past 30 Days'],

  /**
   * describe filter columns
   * @type {Array}
   * @const
   */
  keyFilterMap: [{
    label: Em.I18n.t('common.service'),
    key: 'serviceVersion',
    type: 'select',
    column: 1
  }, {
    label: Em.I18n.t('common.configGroup'),
    key: 'configGroup',
    type: 'select',
    column: 2
  }, {
    label: Em.I18n.t('dashboard.configHistory.table.created.title'),
    key: 'createTime',
    type: 'range',
    column: 3
  }, {
    label: Em.I18n.t('common.author'),
    key: 'author',
    type: 'string',
    column: 4
  }, {
    label: Em.I18n.t('common.notes'),
    key: 'notes',
    type: 'string',
    column: 5
  }],

  /**
   * populated dynamically in <code>getGroupsAvailableValues<code>
   * @type {object}
   */
  groupsNameIdMap: {},

  /**
   * 'valueMatches' callback for visualsearch.js
   * @param facetValue
   * @param searchTerm
   * @param callback
   */
  valueMatches: function valueMatches(facetValue, searchTerm, callback) {
    this.showHideClearButton();
    switch (this.get('keyFilterMap').findProperty('label', facetValue).key) {
      case 'serviceVersion':
        this.getServiceVersionAvailableValues(facetValue, callback);
        break;
      case 'configGroup':
        this.getConfigGroupAvailableValues(facetValue, callback);
        break;
      case 'createTime':
        this.getCreateTimeAvailableValues(facetValue, callback);
        break;
      case 'author':
        this.getAuthorAvailableValues(facetValue, callback);
        break;
      case 'notes':
        this.getNotesAvailableValues(facetValue, callback);
        break;
    }
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getServiceVersionAvailableValues: function getServiceVersionAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(App.StackService.find().mapProperty('displayName'), facetValue), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getConfigGroupAvailableValues: function getConfigGroupAvailableValues(facetValue, callback) {
    var configGroups = App.ServiceConfigVersion.find().mapProperty('groupName').uniq().compact();
    callback(this.rejectUsedValues(configGroups, facetValue));
    this.requestFacetSuggestions(facetValue, callback);
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getCreateTimeAvailableValues: function getCreateTimeAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(this.get('modifiedOptions'), facetValue), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getAuthorAvailableValues: function getAuthorAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(App.ServiceConfigVersion.find().mapProperty('author').uniq(), facetValue));
    this.requestFacetSuggestions(facetValue, callback);
  },

  /**
   *
   * @param {string} facetValue
   * @param {Function} callback
   */
  getNotesAvailableValues: function getNotesAvailableValues(facetValue, callback) {
    callback(this.rejectUsedValues(App.ServiceConfigVersion.find().mapProperty('notes').uniq(), facetValue));
    this.requestFacetSuggestions(facetValue, callback);
  },

  /**
   * request suggestions for facet from server
   * @param {string} facetValue
   * @param {Function} callback
   */
  requestFacetSuggestions: function requestFacetSuggestions(facetValue, callback) {
    var name = this.get('keyFilterMap').findProperty('label', facetValue).key;
    this.get('controller').getSearchBoxSuggestions(name).done(function (suggestions) {
      if (suggestions.length) {
        callback(suggestions);
      }
    });
  },

  /**
   *
   * @param {string} category
   * @param {string} label
   */
  mapLabelToValue: function mapLabelToValue(category, label) {
    switch (category) {
      case 'createTime':
        return this.computeCreateTimeRange(label);
      case 'serviceVersion':
        return App.StackService.find().findProperty('displayName', label).get('serviceName');
      default:
        return label;
    }
  },

  /**
   *
   * @param {string} value
   * @returns {Array} [startTime, endTime]
   */
  computeCreateTimeRange: function computeCreateTimeRange(value) {
    var time = "";
    var curTime = App.dateTime();

    switch (value) {
      case 'Past 1 hour':
        time = curTime - 3600000;
        break;
      case 'Past 1 Day':
        time = curTime - 86400000;
        break;
      case 'Past 2 Days':
        time = curTime - 172800000;
        break;
      case 'Past 7 Days':
        time = curTime - 604800000;
        break;
      case 'Past 14 Days':
        time = curTime - 1209600000;
        break;
      case 'Past 30 Days':
        time = curTime - 2592000000;
        break;
      case 'Any':
        time = "";
        break;
    }
    return [time, ''];
  }
});

});

require.register("views/main/dashboard/config_history_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view');
var sort = require('views/common/sort_view');

App.MainConfigHistoryView = App.TableView.extend(App.TableServerViewMixin, {
  templateName: require('templates/main/dashboard/config_history'),

  controllerBinding: 'App.router.mainConfigHistoryController',

  /**
   * @type {boolean}
   * @default false
   */
  filteringComplete: false,
  isInitialRendering: true,

  /**
   * return filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('tableView.filters.filteredConfigVersionInfo', 'filteredCount', 'totalCount'),

  willInsertElement: function willInsertElement() {
    var controllerName = this.get('controller.name'),
        savedSortConditions = App.db.getSortingStatuses(controllerName) || [];

    if (savedSortConditions.everyProperty('status', 'sorting')) {
      savedSortConditions.push({
        name: "createTime",
        status: "sorting_desc"
      });
      App.db.setSortingStatuses(controllerName, savedSortConditions);
    }
    if (!this.get('controller.showFilterConditionsFirstLoad')) {
      this.clearFilterConditionsFromLocalStorage();
    }
    this._super();
  },
  didInsertElement: function didInsertElement() {
    this.addObserver('startIndex', this, 'updatePagination');
    this.addObserver('displayLength', this, 'updatePagination');
    this.set('isInitialRendering', true);
    this.refresh(true);
    this.get('controller').subscribeToUpdates();
  },

  /**
   * stop polling after leaving config history page
   */
  willDestroyElement: function willDestroyElement() {
    this.get('controller').unsubscribeOfUpdates();
  },

  sortView: sort.serverWrapperView,
  versionSort: sort.fieldView.extend({
    column: 1,
    name: 'serviceVersion',
    displayName: Em.I18n.t('dashboard.configHistory.table.version.title'),
    classNames: ['first']
  }),
  configGroupSort: sort.fieldView.extend({
    column: 2,
    name: 'configGroup',
    displayName: Em.I18n.t('dashboard.configHistory.table.configGroup.title')
  }),
  modifiedSort: sort.fieldView.extend({
    column: 3,
    name: 'createTime',
    displayName: Em.I18n.t('dashboard.configHistory.table.created.title')
  }),
  authorSort: sort.fieldView.extend({
    column: 4,
    name: 'author',
    displayName: Em.I18n.t('common.author')
  }),
  notesSort: sort.fieldView.extend({
    column: 5,
    name: 'notes',
    displayName: Em.I18n.t('common.notes')
  }),

  ConfigVersionView: Em.View.extend({
    tagName: 'tr',
    showLessNotes: true,
    toggleShowLessStatus: function toggleShowLessStatus() {
      this.toggleProperty('showLessNotes');
    },
    didInsertElement: function didInsertElement() {
      App.tooltip(this.$("[rel='Tooltip']"), { html: false });
    },

    // Define if show plain text label or link
    isServiceLinkDisabled: function () {
      return this.get('content.serviceName') === 'KERBEROS' && !App.Service.find().someProperty('serviceName', 'KERBEROS') || this.get('content.isConfigGroupDeleted');
    }.property('content.serviceName', 'content.isConfigGroupDeleted')
  }),

  /**
   * refresh table content
   * @param {boolean} shouldUpdateCounter
   */
  refresh: function refresh(shouldUpdateCounter) {
    var self = this;
    this.set('filteringComplete', false);
    this.get('controller').load(shouldUpdateCounter).done(function () {
      self.refreshDone.apply(self);
    });
  },

  /**
   * callback executed after refresh call done
   * @method refreshDone
   */
  refreshDone: function refreshDone() {
    this.set('isInitialRendering', false);
    this.set('filteringComplete', true);
    this.propertyDidChange('pageContent');
    this.set('controller.resetStartIndex', false);
    App.loadTimer.finish('Config History Page');
  },

  saveStartIndex: function () {
    App.db.setStartIndex(this.get('controller.name'), this.get('startIndex'));
  }.observes('startIndex'),

  filter: Em.K
});

});

require.register("views/main/dashboard/widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DashboardWidgetView = Em.View.extend({

  /**
   * @type {string}
   * @default null
   */
  title: Em.computed.alias('widget.title'),

  templateName: null,

  sourceName: Em.computed.alias('widget.sourceName'),

  /**
   * Bound from template
   *
   * @type {object}
   */
  widget: null,

  /**
   * Indicates whether dropdown with Edit/Delete/Save actions should be displayed
   * @type {boolean}
   */
  showActions: true,

  /**
   * @type {Em.View}
   */
  widgetsView: Em.computed.alias('parentView'),

  /**
   * @type {object} - record from model that serve as data source
   */
  model: function () {
    var model = Em.Object.create();
    if (Em.isNone(this.get('sourceName'))) {
      return model;
    }
    return this.findModelBySource(this.get('sourceName'));
  }.property('sourceName'),

  /**
   * @type {number}
   * @default null
   */
  id: Em.computed.alias('widget.id'),

  /**
   * html id bind to view-class: widget-(1)
   * used by re-sort
   * @type {string}
   */
  viewID: Em.computed.format('widget-{0}', 'id'),

  classNames: ['span2p4'],

  attributeBindings: ['viewID'],

  /**
   * widget content pieChart/ text/ progress bar/links/ metrics. etc
   * @type {Array}
   * @default null
   */
  content: null,

  /**
   * more info details
   * @type {Array}
   */
  hiddenInfo: [],

  /**
   * @type {string}
   */
  hiddenInfoClass: "hidden-info-two-line",

  /**
   * @type {string}
   */
  hintInfo: Em.computed.i18nFormat('dashboard.widgets.hintInfo.common', 'maxValue'),

  /**
   * @type {number}
   * @default 0
   */
  thresholdMin: function () {
    return Em.isNone(this.get('widget.threshold')) ? 0 : this.get('widget.threshold')[0];
  }.property('widget.threshold'),

  /**
   * @type {number}
   * @default 0
   */
  thresholdMax: function () {
    return Em.isNone(this.get('widget.threshold')) ? 0 : this.get('widget.threshold')[1];
  }.property('widget.threshold'),

  /**
   * @type {Boolean}
   * @default false
   */
  isDataLoadedBinding: 'App.router.clusterController.isServiceContentFullyLoaded',

  didInsertElement: function didInsertElement() {
    App.tooltip(this.$("[rel='ZoomInTooltip']"), {
      placement: 'left',
      template: '<div class="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner graph-tooltip"></div></div>'
    });
  },

  /**
   *
   * @param {string} source
   * @returns {App.Service}
   */
  findModelBySource: function findModelBySource(source) {
    if (source === 'HOST_METRICS' && App.get('services.hostMetrics').length > 0) {
      return App.get('services.hostMetrics');
    }
    var extendedModel = App.Service.extendedModel[source];
    if (extendedModel) {
      return App[extendedModel].find(source);
    }
    return App.Service.find(source);
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='ZoomInTooltip']").tooltip('destroy');
  },

  /**
   * delete widget
   */
  deleteWidget: function deleteWidget() {
    this.get('parentView').hideWidget(this.get('id'), this.get('groupId'), this.get('subGroupId'), this.get('isAllItemsSubGroup'));
  },

  /**
   * Update thresholds for widget and save this them to persist
   *
   * @param {number[]} preparedThresholds
   */
  saveWidgetThresholds: function saveWidgetThresholds(preparedThresholds) {
    var widgetId = this.get('id');
    var widgetIdToNumber = Number(widgetId);
    var widgetsView = this.get('widgetsView');
    var userPreferences = widgetsView.get('userPreferences');
    var isGroupWidget = isNaN(widgetIdToNumber);
    if (isGroupWidget) {
      var parsedWidgetId = parseInt(widgetId);
      if (this.get('isAllItemsSubGroup')) {
        userPreferences.groups[this.get('groupId')]['*'].threshold[this.get('subGroupId')][parsedWidgetId] = preparedThresholds;
      } else {
        userPreferences.groups[this.get('groupId')][this.get('subGroupId')].threshold[parsedWidgetId] = preparedThresholds;
      }
    } else {
      userPreferences.threshold[widgetIdToNumber] = preparedThresholds;
      this.set('widget.threshold', userPreferences.threshold[widgetIdToNumber]);
    }
    widgetsView.saveWidgetsSettings(userPreferences);
  },


  /**
   * edit widget
   */
  editWidget: function editWidget() {
    return App.EditDashboardWidgetPopup.show({

      widgetView: this,

      sliderHandlersManager: App.EditDashboardWidgetPopup.DoubleHandlers.create({
        thresholdMin: this.get('thresholdMin'),
        thresholdMax: this.get('thresholdMax'),
        maxValue: parseFloat(this.get('maxValue'))
      }),

      sliderColors: [App.healthStatusGreen, App.healthStatusOrange, App.healthStatusRed]

    });
  }

});

});

require.register("views/main/dashboard/widgets", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

var WidgetObject = Em.Object.extend({
  id: '',
  threshold: '',
  viewClass: null,
  sourceName: '',
  title: '',
  checked: false,
  isVisible: true
});

var plusButtonFilterView = Ember.View.extend({
  tagName: 'ul',
  classNames: ['dropdown-menu'],
  templateName: require('templates/main/dashboard/plus_button_filter'),
  valueBinding: '',
  widgetCheckbox: App.CheckboxView.extend({
    didInsertElement: function didInsertElement() {
      $('.checkbox').click(function (event) {
        event.stopPropagation();
      });
    }
  }),
  applyFilter: function applyFilter() {
    var parent = this.get('parentView'),
        hiddenWidgets = this.get('hiddenWidgets');
    hiddenWidgets.filterProperty('checked').setEach('isVisible', true);
    parent.saveWidgetsSettings();
  }
});

App.MainDashboardWidgetsView = Em.View.extend(App.Persist, App.LocalStorage, App.TimeRangeMixin, {
  name: 'mainDashboardWidgetsView',
  templateName: require('templates/main/dashboard/widgets'),

  widgetsDefinition: require('data/dashboard_widgets'),

  widgetsDefinitionMap: function () {
    return this.get('widgetsDefinition').toMapByProperty('id');
  }.property('widgetsDefinition.[]'),

  widgetGroups: [],

  widgetGroupsDeferred: $.Deferred(),

  displayedWidgetGroups: Em.computed.filterBy('widgetGroups', 'isDisplayed', true),

  setWidgetGroups: function setWidgetGroups() {
    if (App.get('router.clusterController.isHDFSNameSpacesLoaded')) {
      var groups = [];
      var hdfsService = App.HDFSService.find().objectAt(0),
          hdfsMasterGroups = hdfsService ? hdfsService.get('masterComponentGroups') : [];
      this.removeObserver('App.router.clusterController.isHDFSNameSpacesLoaded', this, 'setWidgetGroups');
      if (hdfsMasterGroups.length) {
        var nameSpacesListItems = hdfsMasterGroups.map(function (nameSpace) {
          var name = nameSpace.name,
              title = nameSpace.title;

          return {
            name: name,
            title: title,
            isActive: false
          };
        });
        groups.push(Em.Object.create({
          name: 'nn',
          title: Em.I18n.t('dashboard.widgets.nameSpace'),
          serviceName: 'HDFS',
          subGroups: [{
            name: '*',
            title: Em.I18n.t('common.all'),
            isActive: true
          }].concat(_toConsumableArray(nameSpacesListItems)),
          activeSubGroup: Em.computed.findBy('subGroups', 'isActive', true),
          allWidgets: this.get('allNameNodeWidgets'),
          isDisplayed: App.get('hasNameNodeFederation')
        }));
      }
      this.set('widgetGroups', groups);
      this.get('widgetGroupsDeferred').resolve();
    }
  },

  /**
   * List of services
   * @type {Ember.Enumerable}
   */
  content: [],

  /**
   * Key-name to store data in Local Storage and Persist
   * @type {string}
   */
  persistKey: Em.computed.format('user-pref-{0}-dashboard', 'App.router.loginName'),

  /**
   * @type {boolean}
   */
  isDataLoaded: false,

  /**
   * Define if some widget is currently moving
   * @type {boolean}
   */
  isMoving: false,

  /**
   * @type {WidgetObject[]}
   */
  allWidgets: [],

  allNameNodeWidgets: [],

  /**
   * List of visible widgets
   *
   * @type {WidgetObject[]}
   */
  visibleWidgets: Em.computed.filterBy('allWidgets', 'isVisible', true),

  /**
   * List of hidden widgets
   *
   * @type {WidgetObject[]}
   */
  hiddenWidgets: Em.computed.filterBy('allWidgets', 'isVisible', false),

  timeRangeClassName: 'pull-left',

  /**
   * Example:
   * {
   *   visible: [1, 2, 4],
   *   hidden: [3, 5],
   *   threshold: {
   *     1: [80, 90],
   *     2: [],
   *     3: [1, 2]
   *   }
   * }
   * @type {Object|null}
   */
  userPreferences: null,

  didInsertElement: function didInsertElement() {
    var _this = this;

    this._super();
    if (App.get('router.clusterController.isHDFSNameSpacesLoaded')) {
      this.setWidgetGroups();
    } else {
      this.addObserver('App.router.clusterController.isHDFSNameSpacesLoaded', this, 'setWidgetGroups');
    }
    this.loadWidgetsSettings().complete(function () {
      _this.get('widgetGroupsDeferred').done(function () {
        if (_this.get('isDestroyed')) return;
        _this.checkServicesChange();
        _this.renderWidgets();
        _this.set('isDataLoaded', true);
        App.loadTimer.finish('Dashboard Metrics Page');
        Em.run.next(_this, 'makeSortable');
        if (_this.get('displayedWidgetGroups.length')) {
          Em.run.next(_this, 'makeGroupedWidgetsSortable');
        }
      });
    });
  },

  /**
   * Set visibility-status for widgets
   */
  loadWidgetsSettings: function loadWidgetsSettings() {
    return this.getUserPref(this.get('persistKey'));
  },

  /**
   * make POST call to save settings
   * format settings if they are not provided
   *
   * @param {object} [settings]
   */
  saveWidgetsSettings: function saveWidgetsSettings(settings) {
    var userPreferences = this.get('userPreferences');
    var newSettings = {
      visible: [],
      hidden: [],
      threshold: {},
      groups: {}
    };
    if (arguments.length === 1) {
      newSettings = settings;
    } else {
      newSettings.threshold = userPreferences.threshold;
      this.get('allWidgets').forEach(function (widget) {
        var key = widget.get('isVisible') ? 'visible' : 'hidden';
        newSettings[key].push(widget.get('id'));
      });
      this.get('widgetGroups').forEach(function (group) {
        var groupName = group.get('name');
        if (!newSettings.groups[groupName]) {
          newSettings.groups[groupName] = {};
        }
        group.get('allWidgets').forEach(function (widgetsSubGroup) {
          var subGroupName = widgetsSubGroup.get('subGroupName'),
              widgets = widgetsSubGroup.get('widgets'),
              subGroupSettings = {
            visible: [],
            hidden: [],
            threshold: {}
          };
          newSettings.groups[groupName][subGroupName] = subGroupSettings;
          widgets.forEach(function (widget) {
            var key = widget.get('isVisible') ? 'visible' : 'hidden',
                threshold = widget.get('threshold'),
                widgetId = widget.get('id'),
                id = parseInt(widgetId);
            if (subGroupName === '*') {
              var regExp = new RegExp('\\d+\\-' + groupName + '\\-([\\W\\w]+)\\-\\*'),
                  regExpMatch = widgetId.match(regExp),
                  subGroup = regExpMatch && regExpMatch[1];
              if (subGroup) {
                subGroupSettings[key].push({
                  id: id,
                  subGroup: subGroup
                });
                $.extend(true, subGroupSettings.threshold, _defineProperty({}, subGroup, _defineProperty({}, id, threshold)));
              }
            } else {
              subGroupSettings[key].push(id);
              subGroupSettings.threshold[id] = threshold;
            }
          });
        });
      });
    }
    this.get('widgetGroups').forEach(function (group) {
      var groupName = group.get('name');
      group.get('allWidgets').forEach(function (widgetsSubGroup) {
        var subGroupName = widgetsSubGroup.get('subGroupName'),
            widgets = widgetsSubGroup.get('widgets'),
            arrayFromSettings = newSettings.groups[groupName][subGroupName].visible,
            isSubGroupForAll = subGroupName === '*',
            orderedArray = isSubGroupForAll ? arrayFromSettings.map(function (widget) {
          return widget.id + '-' + groupName + '-' + widget.subGroup + '-*';
        }) : arrayFromSettings;
        widgetsSubGroup.set('widgets', widgets.sort(function (widgetA, widgetB) {
          var idA = isSubGroupForAll ? widgetA.get('id') : parseInt(widgetA.get('id')),
              idB = isSubGroupForAll ? widgetB.get('id') : parseInt(widgetB.get('id')),
              indexA = orderedArray.indexOf(idA),
              indexB = orderedArray.indexOf(idB);
          return indexA > -1 && indexB > -1 ? indexA - indexB : 0;
        }));
      });
    });
    this.set('userPreferences', newSettings);
    this.setDBProperty(this.get('persistKey'), newSettings);
    this.postUserPref(this.get('persistKey'), newSettings);
  },

  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response) {
    if (response) {
      this.set('userPreferences', response);
    } else {
      this.getUserPrefErrorCallback();
    }
  },

  getUserPrefErrorCallback: function getUserPrefErrorCallback() {
    var userPreferences = this.generateDefaultUserPreferences();
    this.saveWidgetsSettings(userPreferences);
  },

  resolveConfigDependencies: function resolveConfigDependencies(widgetsDefinition) {
    var clusterEnv = App.router.get('clusterController.clusterEnv.properties') || {};
    if (clusterEnv.hide_yarn_memory_widget === 'true') {
      widgetsDefinition.findProperty('id', 20).isHiddenByDefault = true;
    }
    return widgetsDefinition;
  },

  getWidgetSubGroupsObject: function getWidgetSubGroupsObject(subGroups) {
    return subGroups.reduce(function (current, subGroup) {
      return _extends({}, current, _defineProperty({}, subGroup.name, {
        visible: [],
        hidden: [],
        threshold: {}
      }));
    }, {});
  },

  generateDefaultUserPreferences: function generateDefaultUserPreferences() {
    var _this2 = this;

    var widgetsDefinition = this.get('widgetsDefinition');
    var preferences = {
      visible: [],
      hidden: [],
      threshold: {},
      groups: {}
    };

    this.resolveConfigDependencies(widgetsDefinition);
    widgetsDefinition.forEach(function (widget) {
      var sourceName = widget.sourceName,
          id = widget.id,
          groupName = widget.groupName,
          widgetGroups = _this2.get('displayedWidgetGroups');

      if (App.Service.find(sourceName).get('isLoaded') || sourceName === 'HOST_METRICS') {
        var state = widget.isHiddenByDefault ? 'hidden' : 'visible',
            threshold = widget.threshold,
            widgetGroup = widgetGroups.find(function (group) {
          return group.get('serviceName') === sourceName && group.get('name') === groupName;
        });

        if (widgetGroup) {
          var widgetGroupName = widgetGroup.get('name'),
              allSubGroups = widgetGroup.get('subGroups'),
              subGroupForAllItems = allSubGroups.findProperty('name', '*'),
              subGroups = allSubGroups.rejectProperty('name', '*'),
              existingEntry = preferences.groups[widgetGroupName],
              currentEntry = existingEntry || _this2.getWidgetSubGroupsObject(allSubGroups);
          subGroups.forEach(function (subGroup) {
            var name = subGroup.name;

            currentEntry[name][state].push(id);
            currentEntry[name].threshold[id] = threshold;
            if (subGroupForAllItems) {
              currentEntry['*'][state].push({
                id: id,
                subGroup: name
              });
              currentEntry['*'].threshold[name] = _extends({}, currentEntry['*'].threshold[name], _defineProperty({}, id, threshold));
            }
          });
          if (!existingEntry) {
            preferences.groups[widgetGroupName] = currentEntry;
          }
        } else {
          preferences[state].push(id);
        }
      }
      preferences.threshold[id] = widget.threshold;
    });

    return preferences;
  },

  /**
   * Don't show widget on the Dashboard
   *
   * @param {number|string} id
   * @param {string} [groupId]
   * @param {string} [subGroupId]
   * @param {boolean} [isAllItemsSubGroup]
   */
  hideWidget: function hideWidget(id, groupId, subGroupId, isAllItemsSubGroup) {
    var idToNumber = Number(id);
    if (isNaN(idToNumber)) {
      var subGroupToFilter = isAllItemsSubGroup ? '*' : subGroupId,
          groupWidgets = this.get('displayedWidgetGroups').findProperty('name', groupId).get('allWidgets'),
          subGroupWidgets = groupWidgets && groupWidgets.findProperty('subGroupName', subGroupToFilter).get('widgets'),
          targetWidget = subGroupWidgets && subGroupWidgets.findProperty('id', id);
      if (targetWidget) {
        targetWidget.set('isVisible', false);
      }
    } else {
      this.get('allWidgets').findProperty('id', id).set('isVisible', false);
    }
    this.saveWidgetsSettings();
  },


  /**
   *
   * @param {number} id
   * @param {boolean} isVisible
   * @param {string} subGroupId
   * @returns {WidgetObject}
   * @private
   */
  _createWidgetObj: function _createWidgetObj(id, isVisible, subGroupId) {
    var widget = this.get('widgetsDefinitionMap')[id];
    return WidgetObject.create({
      id: id,
      threshold: this.get('userPreferences.threshold')[id],
      viewClass: App[widget.viewName].extend({
        subGroupId: subGroupId
      }),
      sourceName: widget.sourceName,
      title: widget.title,
      isVisible: isVisible
    });
  },


  /**
   *
   * @param {number} id
   * @param {boolean} isVisible
   * @param {string} groupId
   * @param {string} subGroupId
   * @param {boolean} isAllSubGroupsDisplay
   * @returns {WidgetObject}
   * @private
   */
  _createGroupWidgetObj: function _createGroupWidgetObj(id, isVisible, groupId, subGroupId) {
    var isAllSubGroupsDisplay = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

    var widget = this.get('widgetsDefinitionMap')[id];
    subGroupId = subGroupId || 'default';
    return WidgetObject.create({
      id: id + '-' + groupId + '-' + subGroupId + (isAllSubGroupsDisplay ? '-*' : ''),
      threshold: isAllSubGroupsDisplay ? this.get('userPreferences.groups')[groupId]['*'].threshold[subGroupId][id] : this.get('userPreferences.groups')[groupId][subGroupId].threshold[id],
      viewClass: App[widget.viewName].extend({
        subGroupId: subGroupId,
        isAllItemsSubGroup: isAllSubGroupsDisplay
      }),
      sourceName: widget.sourceName,
      title: widget.title + ' - ' + subGroupId,
      isVisible: isVisible
    });
  },


  findWidgetInAllItemsSubGroup: function findWidgetInAllItemsSubGroup(id, subGroup) {
    return function (widget) {
      return widget.id === id && widget.subGroup === subGroup;
    };
  },

  /**
   * set widgets to view in order to render
   */
  renderWidgets: function renderWidgets() {
    var _this3 = this;

    var userPreferences = this.get('userPreferences'),
        widgetsDefinition = this.get('widgetsDefinition'),
        widgetGroups = this.get('widgetGroups');
    var newVisibleWidgets = [],
        newHiddenWidgets = [];
    widgetGroups.forEach(function (group) {
      return group.get('allWidgets').clear();
    });
    widgetsDefinition.forEach(function (widget) {
      var id = widget.id,
          groupName = widget.groupName,
          widgetGroup = widgetGroups.findProperty('name', groupName);

      if (groupName && widgetGroup && widgetGroup.get('isDisplayed')) {
        var groupPreferences = userPreferences.groups[groupName];
        if (groupPreferences) {
          var subGroupNames = widgetGroup.get('subGroups').mapProperty('name').without('*'),
              subGroupForAllItems = widgetGroup.get('subGroups').findProperty('name', '*');
          var allWidgets = widgetGroup.get('allWidgets');
          subGroupNames.forEach(function (subGroupName) {
            var subGroupPreferences = groupPreferences[subGroupName],
                existingSubGroup = allWidgets.findProperty('subGroupName', subGroupName),
                currentSubGroup = existingSubGroup || Em.Object.create({
              subGroupName: subGroupName,
              title: subGroupName,
              parentGroup: widgetGroup,
              isActive: Em.computed.equal('parentGroup.activeSubGroup.name', subGroupName),
              widgets: [],
              hiddenWidgets: Em.computed.filterBy('widgets', 'isVisible', false)
            }),
                visibleIndex = subGroupPreferences.visible.indexOf(id),
                hiddenIndex = subGroupPreferences.hidden.indexOf(id),
                visibleCount = subGroupPreferences.visible.length;
            if (!existingSubGroup) {
              allWidgets.pushObject(currentSubGroup);
            }
            if (visibleIndex > -1) {
              currentSubGroup.get('widgets')[visibleIndex] = _this3._createGroupWidgetObj(id, true, groupName, subGroupName);
            }
            if (hiddenIndex > -1) {
              currentSubGroup.get('widgets')[hiddenIndex + visibleCount] = _this3._createGroupWidgetObj(id, false, groupName, subGroupName);
            }
          });
          if (subGroupForAllItems) {
            var subGroupPreferences = groupPreferences['*'],
                existingSubGroup = allWidgets.findProperty('subGroupName', '*'),
                currentSubGroup = existingSubGroup || Em.Object.create({
              subGroupName: '*',
              title: Em.I18n.t('common.all'),
              parentGroup: widgetGroup,
              isActive: Em.computed.equal('parentGroup.activeSubGroup.name', '*'),
              widgets: [],
              hiddenWidgets: Em.computed.filterBy('widgets', 'isVisible', false)
            });
            if (!existingSubGroup) {
              allWidgets.pushObject(currentSubGroup);
            }
            var visibleItems = subGroupPreferences.visible.filterProperty('id', id),
                hiddenItems = subGroupPreferences.hidden.filterProperty('id', id),
                visibleCount = subGroupPreferences.visible.length;
            var widgets = [];
            visibleItems.forEach(function (widget) {
              var subgroupName = widget.subGroup,
                  findFunction = _this3.findWidgetInAllItemsSubGroup(id, subgroupName),
                  index = subGroupPreferences.visible.findIndex(findFunction);
              currentSubGroup.get('widgets')[index] = _this3._createGroupWidgetObj(id, true, groupName, subgroupName, true);
            });
            hiddenItems.forEach(function (widget) {
              var subgroupName = widget.subGroup,
                  findFunction = _this3.findWidgetInAllItemsSubGroup(id, subgroupName),
                  index = subGroupPreferences.hidden.findIndex(findFunction);
              currentSubGroup.get('widgets')[index + visibleCount] = _this3._createGroupWidgetObj(id, false, groupName, subgroupName, true);
            });
          }
          allWidgets.forEach(function (subGroup) {
            var widgets = subGroup.get('widgets');
          });
        }
      } else {
        var subGroupId = widgetGroup ? widgetGroup.get('subGroups.lastObject.name') : 'default',
            visibleIndex = userPreferences.visible.indexOf(id),
            hiddenIndex = userPreferences.hidden.indexOf(id);
        if (visibleIndex > -1) {
          newVisibleWidgets[visibleIndex] = _this3._createWidgetObj(id, true, subGroupId);
        }
        if (hiddenIndex > -1) {
          newHiddenWidgets[hiddenIndex] = _this3._createWidgetObj(id, false, subGroupId);
        }
      }
    });
    newVisibleWidgets = newVisibleWidgets.filter(function (widget) {
      return !Em.isNone(widget);
    });
    newHiddenWidgets = newHiddenWidgets.filter(function (widget) {
      return !Em.isNone(widget);
    });
    this.set('allWidgets', newVisibleWidgets.concat(newHiddenWidgets));
  },

  /**
   * check if stack has upgraded from HDP 1.0 to 2.0 OR add/delete services.
   * Update the value on server if true.
   */
  checkServicesChange: function checkServicesChange() {
    var _this4 = this;

    var userPreferences = this.get('userPreferences'),
        defaultPreferences = this.generateDefaultUserPreferences();
    var newValue = {
      visible: userPreferences.visible.slice(0),
      hidden: userPreferences.hidden.slice(0),
      threshold: userPreferences.threshold,
      groups: $.extend(true, {}, userPreferences.groups)
    },
        isChanged = false;

    ['visible', 'hidden'].forEach(function (state) {
      defaultPreferences[state].forEach(function (id) {
        if (!userPreferences.visible.contains(id) && !userPreferences.hidden.contains(id)) {
          isChanged = true;
          newValue[state].push(id);
        }
      });
      Object.keys(defaultPreferences.groups).forEach(function (groupName) {
        var groupPreferences = defaultPreferences.groups[groupName],
            subGroupForAllItems = groupPreferences['*'],
            subGroups = Object.keys(groupPreferences).without('*');
        subGroups.forEach(function (subGroupName) {
          groupPreferences[subGroupName][state].forEach(function (id) {
            if (!newValue.groups[groupName] || !newValue.groups[groupName][subGroupName]) {
              $.extend(true, newValue.groups, _defineProperty({}, groupName, _defineProperty({}, subGroupName, {
                visible: [],
                hidden: [],
                threshold: defaultPreferences.groups[groupName][subGroupName].threshold
              })));
            }
            var subGroupPreferences = newValue.groups[groupName][subGroupName];
            if (!subGroupPreferences.visible.contains(id) && !subGroupPreferences.hidden.contains(id)) {
              isChanged = true;
              subGroupPreferences[state].push(id);
            }
          });
        });
        if (subGroupForAllItems) {
          subGroupForAllItems[state].forEach(function (item) {
            var id = item.id,
                subGroup = item.subGroup;

            if (!newValue.groups[groupName]['*']) {
              $.extend(true, newValue.groups, _defineProperty({}, groupName, {
                '*': {
                  visible: [],
                  hidden: [],
                  threshold: defaultPreferences.groups[groupName]['*'].threshold
                }
              }));
            }
            var preferences = newValue.groups[groupName]['*'],
                checkFunction = _this4.findWidgetInAllItemsSubGroup(id, subGroup);
            if (!preferences.visible.some(checkFunction) && !preferences.hidden.some(checkFunction)) {
              isChanged = true;
              preferences[state].push({
                id: id,
                subGroup: subGroup
              });
            }
            if (!preferences.threshold[subGroup]) {
              isChanged = true;
              preferences.threshold[subGroup] = defaultPreferences.groups[groupName]['*'].threshold[subGroup];
            }
          });
        }
      });
    });
    if (isChanged) {
      this.saveWidgetsSettings(newValue);
    }
  },

  /**
   * Reset widgets visibility-status
   */
  resetAllWidgets: function resetAllWidgets() {
    var _this5 = this;

    App.showConfirmationPopup(function () {
      _this5.saveWidgetsSettings(_this5.generateDefaultUserPreferences());
      _this5.setProperties({
        currentTimeRangeIndex: 0,
        customStartTime: null,
        customEndTime: null
      });
      _this5.renderWidgets();
    });
  },

  /**
   * Make widgets' list sortable on New Dashboard style
   */
  makeSortable: function makeSortable() {
    var self = this;
    return $("#sortable").sortable({
      items: "> div",
      cursor: "move",
      tolerance: "pointer",
      scroll: false,
      update: function update() {
        var widgetsArray = $('#sortable div[viewid]');

        var userPreferences = self.get('userPreferences') || self.getDBProperty(self.get('persistKey'));
        var newValue = {
          visible: [],
          hidden: userPreferences.hidden,
          threshold: userPreferences.threshold,
          groups: userPreferences.groups
        };
        newValue.visible = userPreferences.visible.map(function (item, index) {
          var viewID = widgetsArray.get(index).getAttribute('viewid');
          return Number(viewID.split('-')[1]);
        });
        self.saveWidgetsSettings(newValue);
      },
      activate: function activate() {
        self.set('isMoving', true);
      },
      deactivate: function deactivate() {
        self.set('isMoving', false);
      }
    }).disableSelection();
  },

  makeGroupedWidgetsSortable: function makeGroupedWidgetsSortable() {
    var _this6 = this;

    this.get('displayedWidgetGroups').forEach(function (widgetGroup) {
      var selector = '#' + widgetGroup.get('name');
      $(selector).sortable({
        items: '> div',
        cursor: 'move',
        tolerance: 'pointer',
        scroll: false,
        update: function update() {
          var isSubGroupForAllItems = false;
          var widgetsArray = $(selector + ' div[viewid]'),
              userPreferences = _this6.get('userPreferences') || _this6.getDBProperty(self.get('persistKey')),
              currentWidgetsData = widgetsArray.toArray().map(function (widget) {
            var viewId = widget.getAttribute('viewid'),
                splittedViewId = viewId.split('-');
            if (splittedViewId.length > 4) {
              isSubGroupForAllItems = true;
            }
            return {
              id: Number(splittedViewId[1]),
              groupName: splittedViewId[2],
              subGroupName: splittedViewId.slice(3, isSubGroupForAllItems ? splittedViewId.length - 1 : splittedViewId.length).join('-')
            };
          }),
              groupName = currentWidgetsData[0].groupName,
              subGroupName = isSubGroupForAllItems ? '*' : currentWidgetsData[0].subGroupName,
              groupPreferences = userPreferences.groups[groupName][subGroupName],
              newSubGroupValue = {
            visible: groupPreferences.visible.map(function (item, index) {
              var widget = currentWidgetsData[index];
              if (isSubGroupForAllItems) {
                return {
                  id: widget.id,
                  subGroup: widget.subGroupName
                };
              } else {
                return widget.id;
              }
            })
          },
              newValue = {
            visible: userPreferences.visible,
            hidden: userPreferences.hidden,
            threshold: userPreferences.threshold,
            groups: $.extend(true, userPreferences.groups, _defineProperty({}, groupName, _defineProperty({}, subGroupName, newSubGroupValue)))
          };

          _this6.saveWidgetsSettings(newValue);
        },
        activate: function activate() {
          _this6.set('isMoving', true);
        },
        deactivate: function deactivate() {
          _this6.set('isMoving', false);
        }
      }).disableSelection();
    });
  },

  /**
   * Submenu view for New Dashboard style
   * @type {Ember.View}
   * @class
   */
  plusButtonFilterView: plusButtonFilterView.extend({
    hiddenWidgetsBinding: 'parentView.hiddenWidgets'
  }),

  groupWidgetsFilterView: plusButtonFilterView.extend(),

  showAlertsPopup: Em.K,

  setActiveSubGroup: function setActiveSubGroup(event) {
    var subGroups = event.contexts[0] || [];
    subGroups.forEach(function (subGroup) {
      return Em.set(subGroup, 'isActive', Em.get(subGroup, 'name') === event.contexts[1]);
    });
  }

});

});

require.register("views/main/dashboard/widgets/cluster_metrics_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ClusterMetricsDashboardWidgetView = App.DashboardWidgetView.extend(App.ExportMetricsMixin, {

  templateName: require('templates/main/dashboard/widgets/cluster_metrics'),

  isClusterMetricsWidget: true,

  exportTargetView: Em.computed.alias('childViews.lastObject'),

  didInsertElement: function didInsertElement() {
    var self = this;
    App.tooltip(this.$('.corner-icon > .glyphicon-save'), {
      title: Em.I18n.t('common.export')
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.$('.corner-icon > .glyphicon-save').tooltip('destroy');
  }

});

});

require.register("views/main/dashboard/widgets/datanode_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function counterOrNA(key) {
  var _key = 'model.{0}.length'.format(key);
  return Em.computed(_key, function () {
    if (Em.isNone(this.get('model.' + key)) || this.get('model.metricsNotAvailable')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get(_key);
  });
}

App.DataNodeUpView = App.TextDashboardWidgetView.extend(App.EditableWithLimitWidgetMixin, {

  hiddenInfo: function () {
    return [this.get('dataNodesLive') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.live'), this.get('dataNodesDead') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.dead'), this.get('dataNodesDecom') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.decom')];
  }.property('dataNodesLive', 'dataNodesDead', 'dataNodesDecom'),

  hiddenInfoClass: "hidden-info-three-line",

  maxValue: 100,

  dataNodesLive: counterOrNA('liveDataNodes'),

  dataNodesDead: counterOrNA('deadDataNodes'),

  dataNodesDecom: counterOrNA('decommissionDataNodes'),

  /**
   * @type {?number}
   */
  data: function () {
    if (this.get('someMetricsNA')) {
      return null;
    }
    return (this.get('dataNodesLive') / this.get('model.dataNodesTotal')).toFixed(2) * 100;
  }.property('model.dataNodesTotal', 'dataNodesLive', 'someMetricsNA'),

  /**
   * @type {string}
   */
  content: function () {
    if (this.get('someMetricsNA')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get('dataNodesLive') + "/" + this.get('model.dataNodesTotal');
  }.property('model.dataNodesTotal', 'dataNodesLive', 'someMetricsNA'),

  /**
   * @type {boolean}
   */
  someMetricsNA: function () {
    return Em.isNone(this.get('model.liveDataNodes')) || Em.isNone(this.get('model.dataNodesTotal')) || this.get('model.metricsNotAvailable');
  }.property('model.liveDataNodes.[]', 'model.dataNodesTotal.[]', 'model.metricsNotAvailable'),

  /**
   * @type {string}
   */
  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(maxTmp);
  }.property('maxValue')

});

});

require.register("views/main/dashboard/widgets/flume_agent_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.FlumeAgentUpView = App.TextDashboardWidgetView.extend(App.EditableWithLimitWidgetMixin, {

  hiddenInfo: function () {
    return [this.get('flumeAgentsLive.length') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.live'), this.get('flumeAgentsDead.length') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.dead')];
  }.property('flumeAgentsLive', 'flumeAgentsDead'),

  hiddenInfoClass: "hidden-info-two-line",

  maxValue: 100,

  flumeAgentComponents: Em.computed.filterBy('model.hostComponents', 'componentName', 'FLUME_HANDLER'),

  /**
   * @type {Array}
   */
  flumeAgentsLive: [],

  /**
   * @type {Array}
   */
  flumeAgentsDead: [],

  data: function () {
    if (!this.get('flumeAgentComponents.length')) {
      return -1;
    }
    return (this.get('flumeAgentsLive').length / this.get('model.hostComponents').filterProperty('componentName', 'FLUME_HANDLER').length).toFixed(2) * 100;
  }.property('model.hostComponents.length', 'flumeAgentsLive'),

  content: Em.computed.concat('/', 'flumeAgentsLive.length', 'flumeAgentComponents.length'),

  statusObserver: function () {
    Em.run.once(this, 'filterStatusOnce');
  }.observes('flumeAgentComponents.@each.workStatus'),

  filterStatusOnce: function filterStatusOnce() {
    this.set('flumeAgentsLive', this.get('flumeAgentComponents').filterProperty("workStatus", "STARTED"));
    this.set('flumeAgentsDead', this.get('flumeAgentComponents').filterProperty("workStatus", "INSTALLED"));
  },

  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(maxTmp);
  }.property('maxValue')

});

});

require.register("views/main/dashboard/widgets/hawqsegment_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function counterOrNA(key) {
  var _key = 'model.{0}'.format(key);
  return Em.computed(_key, function () {
    var value = this.get(_key);
    if (Em.isNone(value)) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return value;
  });
}

App.HawqSegmentUpView = App.TextDashboardWidgetView.extend(App.EditableWithLimitWidgetMixin, {

  hiddenInfo: function () {
    return [this.get('hawqSegmentsStarted') + ' ' + Em.I18n.t('dashboard.services.components.started'), this.get('hawqSegmentsInstalled') + ' ' + Em.I18n.t('dashboard.services.components.stopped'), this.get('hawqSegmentsTotal') + ' ' + Em.I18n.t('dashboard.services.components.total')];
  }.property('hawqSegmentsStarted', 'hawqSegmentsInstalled', 'hawqSegmentsTotal'),

  hiddenInfoClass: "hidden-info-three-line",

  maxValue: 100,

  hawqSegmentsStarted: counterOrNA('hawqSegmentsStarted'),

  hawqSegmentsInstalled: counterOrNA('hawqSegmentsInstalled'),

  hawqSegmentsTotal: counterOrNA('hawqSegmentsTotal'),

  /**
   * @type {?number}
   */
  data: function () {
    if (this.get('someMetricsNA')) {
      return null;
    }
    return (this.get('hawqSegmentsStarted') / this.get('model.hawqSegmentsTotal')).toFixed(2) * 100;
  }.property('model.hawqSegmentsTotal', 'hawqSegmentsStarted', 'someMetricsNA'),

  /**
   * @type {string}
   */
  content: function () {
    if (this.get('someMetricsNA')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get('hawqSegmentsStarted') + "/" + this.get('model.hawqSegmentsTotal');
  }.property('model.hawqSegmentsTotal', 'hawqSegmentsStarted', 'someMetricsNA'),

  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(maxTmp);
  }.property('maxValue'),

  /**
   * @type {boolean}
   */
  someMetricsNA: function () {
    return Em.isNone(this.get('model.hawqSegmentsStarted')) || Em.isNone(this.get('model.hawqSegmentsTotal'));
  }.property('model.hawqSegmentsStarted', 'model.hawqSegmentsTotal')

});

});

require.register("views/main/dashboard/widgets/hbase_average_load", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HBaseAverageLoadView = App.TextDashboardWidgetView.extend(App.EditableWidgetMixin, {

  hiddenInfo: function () {
    var avgLoad = this.get('model.averageLoad');
    if (isNaN(avgLoad)) {
      avgLoad = Em.I18n.t('services.service.summary.notAvailable');
    }
    return [Em.I18n.t('dashboard.services.hbase.averageLoadPerServer').format(avgLoad)];
  }.property("model.averageLoad"),

  isGreen: Em.computed.lteProperties('data', 'thresholdMin'),
  isRed: Em.computed.gtProperties('data', 'thresholdMax'),

  isNA: function () {
    return this.get('data') === null || isNaN(this.get('data'));
  }.property('data'),

  maxValue: 'infinity',

  data: Em.computed.alias('model.averageLoad'),

  content: function () {
    if (this.get('data') || this.get('data') == 0) {
      return this.get('data') + "";
    }
    return Em.I18n.t('services.service.summary.notAvailable');
  }.property('model.averageLoad'),

  hintInfo: Em.I18n.t('dashboard.widgets.hintInfo.hint2')

});

});

require.register("views/main/dashboard/widgets/hbase_links", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HBaseLinksView = App.LinkDashboardWidgetView.extend({

  templateName: require('templates/main/dashboard/widgets/hbase_links'),

  port: function () {
    return App.StackService.find('HBASE').compareCurrentVersion('1.1') > -1 ? '16010' : '60010';
  }.property(),

  componentName: 'HBASE_REGIONSERVER',

  /**
   * All master components
   */
  masters: Em.computed.filterBy('model.hostComponents', 'isMaster', true),
  /**
   * Passive master components
   */
  passiveMasters: Em.computed.filterBy('masters', 'haStatus', 'false'),
  /**
   * One(!) active master component
   */
  activeMaster: Em.computed.findBy('masters', 'haStatus', 'true'),

  activeMasterTitle: Em.I18n.t('service.hbase.activeMaster'),

  hbaseMasterWebUrl: function () {
    var hostName = this.get('activeMaster.host.publicHostName');
    return hostName ? 'http://' + hostName + ':' + this.get('port') : '';
  }.property('activeMaster'),

  calcWebUrl: function calcWebUrl() {
    return '';
  }

});

});

require.register("views/main/dashboard/widgets/hbase_master_heap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HBaseMasterHeapPieChartView = App.PieChartDashboardWidgetView.extend({

  modelValueMax: Em.computed.alias('model.heapMemoryMax'),
  modelValueUsed: Em.computed.alias('model.heapMemoryUsed'),
  widgetHtmlId: 'widget-hbase-heap',

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  },

  getUsed: function getUsed() {
    return this.get('modelValueUsed') / (1024 * 1024) || 0;
  },

  getMax: function getMax() {
    return this.get('modelValueMax') / (1024 * 1024) || 0;
  }
});

});

require.register("views/main/dashboard/widgets/hbase_master_uptime", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HBaseMasterUptimeView = App.UptimeTextDashboardWidgetView.extend({

  component: 'Hbase Master',
  modelValue: Em.computed.alias('model.masterStartTime')

});

});

require.register("views/main/dashboard/widgets/hbase_regions_in_transition", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HBaseRegionsInTransitionView = App.TextDashboardWidgetView.extend(App.EditableWidgetMixin, {

  hiddenInfo: function () {
    return [this.get("model.regionsInTransition") + " regions", "in transition"];
  }.property("model.regionsInTransition"),

  classNameBindings: ['isRed', 'isOrange', 'isGreen', 'isNA'],
  isGreen: Em.computed.lteProperties('data', 'thresholdMin'),
  isRed: Em.computed.gtProperties('data', 'thresholdMax'),
  isOrange: Em.computed.and('!isGreen', '!isRed'),
  isNA: function () {
    return this.get('data') === null;
  }.property('data'),

  maxValue: 'infinity',

  data: Em.computed.alias('model.regionsInTransition'),

  content: Em.computed.format('{0}', 'data'),

  hintInfo: Em.I18n.t('dashboard.widgets.hintInfo.hint2')

});

});

require.register("views/main/dashboard/widgets/hdfs_capacity", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var numberUtils = require('utils/number_utils');

App.NameNodeCapacityPieChartView = App.PieChartDashboardWidgetView.extend({

  modelValueMax: Em.computed.alias('model.capacityTotal'),
  /**
   * HDFS model has 'remaining' value, but not 'used'
   */
  modelValueUsed: Em.computed.alias('model.capacityRemaining'),
  modelValueCapacityUsed: Em.computed.alias('model.capacityUsed'),
  modelValueNonDfsUsed: Em.computed.alias('model.capacityNonDfsUsed'),
  widgetHtmlId: Em.computed.format('widget-nn-capacity'),
  hiddenInfoClass: "hidden-info-six-line",

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  },

  calcHiddenInfo: function calcHiddenInfo() {
    var total = this.get('modelValueMax') + 0;
    var remaining = this.get('modelValueUsed');
    var dfsUsed = this.get('modelValueCapacityUsed');
    var nonDfsUsed = this.get('modelValueNonDfsUsed');
    var dfsPercent = total > 0 ? (dfsUsed * 100 / total).toFixed(2) : 0;
    var nonDfsPercent = total > 0 ? (nonDfsUsed * 100 / total).toFixed(2) : 0;
    var remainingPercent = total > 0 ? (remaining * 100 / total).toFixed(2) : 0;
    if (dfsPercent === "NaN" || dfsPercent < 0) {
      dfsPercent = Em.I18n.t('services.service.summary.notAvailable') + " ";
    }
    if (nonDfsPercent === "NaN" || nonDfsPercent < 0) {
      nonDfsPercent = Em.I18n.t('services.service.summary.notAvailable') + " ";
    }
    if (remainingPercent === "NaN" || remainingPercent < 0) {
      remainingPercent = Em.I18n.t('services.service.summary.notAvailable') + " ";
    }
    return [Em.I18n.t('dashboard.widgets.HDFSDiskUsage.DFSused'), Em.I18n.t('dashboard.widgets.HDFSDiskUsage.info').format(numberUtils.bytesToSize(dfsUsed, 1, 'parseFloat'), dfsPercent), Em.I18n.t('dashboard.widgets.HDFSDiskUsage.nonDFSused'), Em.I18n.t('dashboard.widgets.HDFSDiskUsage.info').format(numberUtils.bytesToSize(nonDfsUsed, 1, 'parseFloat'), nonDfsPercent), Em.I18n.t('dashboard.widgets.HDFSDiskUsage.remaining'), Em.I18n.t('dashboard.widgets.HDFSDiskUsage.info').format(numberUtils.bytesToSize(remaining, 1, 'parseFloat'), remainingPercent)];
  },

  calcDataForPieChart: function calcDataForPieChart() {
    var total = this.get('modelValueMax') * 1024 * 1024;
    var used = total - this.get('modelValueUsed') * 1024 * 1024;
    var percent = total > 0 ? (used * 100 / total).toFixed() : 0;
    var percentPrecise = total > 0 ? (used * 100 / total).toFixed(1) : 0;
    return [percent, percentPrecise];
  }

});

});

require.register("views/main/dashboard/widgets/hdfs_links", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.HDFSLinksView = App.LinkDashboardWidgetView.extend(App.NameNodeWidgetMixin, {

  templateName: require('templates/main/dashboard/widgets/hdfs_links'),

  activeNameNode: Em.computed.findByKey('model.activeNameNodes', 'haNameSpace', 'subGroupId'),

  standbyNameNodes: Em.computed.filterByKey('model.standbyNameNodes', 'haNameSpace', 'subGroupId'),

  port: '50070',

  componentName: 'DATANODE',

  modelField: 'nameNode',

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  },

  isHAEnabled: Em.computed.not('model.snameNode'),

  isActiveNNValid: Em.computed.bool('activeNameNode'),

  isStandbyNNValid: Em.computed.bool('standbyNameNodes.length'),

  isTwoStandbyNN: Em.computed.equal('standbyNameNodes.length', 2),

  twoStandbyComponent: function () {
    return App.HostComponent.find().findProperty('componentName', 'NAMENODE');
  }.property(),

  masterGroupsArray: function () {
    var activeMasterGroup = this.get('model.masterComponentGroups').findProperty('name', this.get('subGroupId'));
    return [activeMasterGroup];
  }.property('model.masterComponentGroups', 'subGroupId')
});

});

require.register("views/main/dashboard/widgets/links_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.LinkDashboardWidgetView = App.DashboardWidgetView.extend({

  componentName: null,

  component: function () {
    return App.HostComponent.find().findProperty('componentName', this.get('componentName'));
  }.property(),

  port: null,

  modelField: null,

  webUrl: null,

  didInsertElement: function didInsertElement() {
    this._super();
    this.addObserver('model.' + this.get('modelField'), this, this.calc);
  },
  calc: function calc() {
    this.set('webUrl', this.calcWebUrl());
  },
  calcWebUrl: function calcWebUrl() {
    if (this.get('model') && this.get('model').get(this.get('modelField'))) {
      return "http://" + this.get('model').get(this.get('modelField')).get('publicHostName') + ':' + this.get('port');
    }
    return '';
  }
});

});

require.register("views/main/dashboard/widgets/metrics_cpu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartClusterMetricsCPUWidgetView = App.ClusterMetricsDashboardWidgetView.extend({

  content: App.ChartClusterMetricsCPU.extend({
    noTitleUnderGraph: true,
    inWidget: true
  })
});

});

require.register("views/main/dashboard/widgets/metrics_load", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartClusterMetricsLoadWidgetView = App.ClusterMetricsDashboardWidgetView.extend({

  content: App.ChartClusterMetricsLoad.extend({
    noTitleUnderGraph: true,
    inWidget: true
  })
});

});

require.register("views/main/dashboard/widgets/metrics_memory", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartClusterMetricsMemoryWidgetView = App.ClusterMetricsDashboardWidgetView.extend({

  content: App.ChartClusterMetricsMemory.extend({
    noTitleUnderGraph: true,
    inWidget: true
  })
});

});

require.register("views/main/dashboard/widgets/metrics_network", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ChartClusterMetricsNetworkWidgetView = App.ClusterMetricsDashboardWidgetView.extend({

  content: App.ChartClusterMetricsNetwork.extend({
    noTitleUnderGraph: true,
    inWidget: true
  })
});

});

require.register("views/main/dashboard/widgets/namenode_cpu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeCpuPieChartView = App.PieChartDashboardWidgetView.extend(App.NameNodeWidgetMixin, {

  widgetHtmlId: Em.computed.format('widget-nn-cpu-{0}', 'subGroupId'),
  cpuWio: null,
  nnHostName: "",
  intervalId: null,

  activeNameNodes: Em.computed.alias('model.activeNameNodes'),
  nameNode: Em.computed.alias('model.nameNode'),

  willDestroyElement: function willDestroyElement() {
    clearInterval(this.get("intervalId"));
  },

  didInsertElement: function didInsertElement() {
    var _this = this;

    this._super();
    var intervalId = void 0;
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isServiceContentFullyLoaded').done(function () {
      if (App.get('isHaEnabled')) {
        var nn = _this.get('activeNameNodes').findProperty('haNameSpace', _this.get('subGroupId'));
        if (nn) {
          _this.set('nnHostName', nn.get('hostName'));
        }
      } else {
        _this.set('nnHostName', _this.get('nameNode.hostName'));
      }
      if (_this.get('nnHostName')) {
        _this.getValue();
        intervalId = setInterval(function () {
          return _this.getValue();
        }, App.componentsUpdateInterval);
        _this.set('intervalId', intervalId);
      }
    });
  },

  getValue: function getValue() {
    App.ajax.send({
      name: 'namenode.cpu_wio',
      sender: this,
      data: {
        nnHost: this.get('nnHostName')
      },
      success: 'updateValueSuccess',
      error: 'updateValueError'
    });
  },

  updateValueError: function updateValueError() {
    this.calc();
  },

  updateValueSuccess: function updateValueSuccess(response) {
    this.set('cpuWio', Em.get(response, 'metrics.cpu.cpu_wio'));
    this.calc();
  },

  calcHiddenInfo: function calcHiddenInfo() {
    var value = this.get('cpuWio');
    var obj1;
    if (value) {
      value = value >= 100 ? 100 : value;
      obj1 = (value + 0).toFixed(2) + '%';
    } else {
      obj1 = Em.I18n.t('services.service.summary.notAvailable');
    }
    return [obj1, 'CPU wait I/O'];
  },

  calcIsPieExists: function calcIsPieExists() {
    return !Em.isNone(this.get('cpuWio'));
  },

  calcDataForPieChart: function calcDataForPieChart() {
    var value = this.get('cpuWio');
    value = value >= 100 ? 100 : value;
    var percent = (value + 0).toFixed(1);
    var percentPrecise = (value + 0).toFixed(2);
    return [percent, percentPrecise];
  }
});

});

require.register("views/main/dashboard/widgets/namenode_heap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeHeapPieChartView = App.PieChartDashboardWidgetView.extend(App.NameNodeWidgetMixin, {

  modelValueMax: Em.computed.getByKey('model.jvmMemoryHeapMaxValues', 'hostName'),
  modelValueUsed: Em.computed.getByKey('model.jvmMemoryHeapUsedValues', 'hostName'),
  widgetHtmlId: Em.computed.format('widget-nn-heap-{0}', 'subGroupId'),

  getUsed: function getUsed() {
    return this.get('modelValueUsed') / (1024 * 1024) || 0;
  },

  getMax: function getMax() {
    return this.get('modelValueMax') / (1024 * 1024) || 0;
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  }
});

});

require.register("views/main/dashboard/widgets/namenode_rpc", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeRpcView = App.TextDashboardWidgetView.extend(App.EditableWidgetMixin, App.NameNodeWidgetMixin, {

  hiddenInfo: function () {
    return [this.get('content') + ' average RPC', 'queue wait time'];
  }.property('content'),

  maxValue: 'infinity',

  isGreen: Em.computed.lteProperties('data', 'thresholdMin'),

  isRed: Em.computed.gtProperties('data', 'thresholdMax'),

  modelValue: Em.computed.getByKey('model.nameNodeRpcValues', 'hostName'),

  data: function () {
    var rpc = this.get('modelValue');
    if (rpc) {
      return rpc.toFixed(2);
    }
    if (rpc == 0) {
      return 0;
    }
    return null;
  }.property('modelValue'),

  content: function () {
    if (this.get('data') || this.get('data') == 0) {
      return this.get('data') + " ms";
    }
    return Em.I18n.t('services.service.summary.notAvailable');
  }.property('modelValue'),

  hintInfo: Em.I18n.t('dashboard.widgets.hintInfo.hint3')

});

});

require.register("views/main/dashboard/widgets/namenode_uptime", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NameNodeUptimeView = App.UptimeTextDashboardWidgetView.extend(App.NameNodeWidgetMixin, {

  component: 'NameNode',
  modelValue: Em.computed.getByKey('model.nameNodeStartTimeValues', 'hostName')

});

});

require.register("views/main/dashboard/widgets/node_managers_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NodeManagersLiveView = App.TextDashboardWidgetView.extend(App.EditableWithLimitWidgetMixin, {

  hiddenInfo: function () {
    var nmActive = this.get('model.nodeManagersCountActive') == null ? Em.I18n.t('services.service.summary.notAvailable') : this.get('model.nodeManagersCountActive');
    var nmLost = this.get('model.nodeManagersCountLost') == null ? Em.I18n.t('services.service.summary.notAvailable') : this.get('model.nodeManagersCountLost');
    var nmUnhealthy = this.get('model.nodeManagersCountUnhealthy') == null ? Em.I18n.t('services.service.summary.notAvailable') : this.get('model.nodeManagersCountUnhealthy');
    var nmRebooted = this.get('model.nodeManagersCountRebooted') == null ? Em.I18n.t('services.service.summary.notAvailable') : this.get('model.nodeManagersCountRebooted');
    var nmDecom = this.get('model.nodeManagersCountDecommissioned') == null ? Em.I18n.t('services.service.summary.notAvailable') : this.get('model.nodeManagersCountDecommissioned');
    return [nmActive + " active", nmLost + " lost", nmUnhealthy + " unhealthy", nmRebooted + " rebooted", nmDecom + " decommissioned"];
  }.property('model.nodeManagersCountActive', 'model.nodeManagersCountLost', 'model.nodeManagersCountUnhealthy', 'model.nodeManagersCountRebooted', 'model.nodeManagersCountDecommissioned'),

  hiddenInfoClass: "hidden-info-five-line",

  maxValue: 100,

  isDataAvailable: Em.computed.and('!model.metricsNotAvailable', 'App.router.clusterController.isComponentsStateLoaded'),

  nodeManagersLive: Em.computed.alias('model.nodeManagersCountActive'),

  data: function () {
    var nodeManagers = this.get('model.nodeManagersTotal');
    var nodeManagersLive = this.get('nodeManagersLive');
    if (!nodeManagersLive || !nodeManagers) {
      return null;
    }
    return (nodeManagersLive / nodeManagers).toFixed(2) * 100;
  }.property('model.nodeManagersTotal', 'nodeManagersLive'),

  content: function () {
    return !this.get('nodeManagersLive') || !this.get('model.nodeManagersTotal') ? Em.I18n.t('services.service.summary.notAvailable') : this.get('nodeManagersLive') + '/' + this.get('model.nodeManagersTotal');
  }.property('model.nodeManagersTotal', 'nodeManagersLive'),

  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(maxTmp);
  }.property('maxValue')

});

});

require.register("views/main/dashboard/widgets/pie_chart_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var numberUtils = require('utils/number_utils');

App.PieChartDashboardWidgetView = App.DashboardWidgetView.extend({

  templateName: require('templates/main/dashboard/widgets/pie_chart'),

  maxValue: 100,

  modelValueMax: null,
  modelValueUsed: null,

  hiddenInfo: null,

  isPieExist: null,

  dataForPieChart: null,

  widgetHtmlId: null,

  getUsed: function getUsed() {
    return this.get('modelValueUsed') || 0;
  },

  getMax: function getMax() {
    return this.get('modelValueMax') || 0;
  },

  calcHiddenInfo: function calcHiddenInfo() {
    var used = this.getUsed();
    var max = this.getMax();
    var percent = max > 0 ? 100 * used / max : 0;
    var result = [];
    result.pushObject(percent.toFixed(1) + '% used');
    result.pushObject(numberUtils.bytesToSize(used, 1, 'parseFloat', 1024 * 1024) + ' of ' + numberUtils.bytesToSize(max, 1, 'parseFloat', 1024 * 1024));
    return result;
  },

  calcIsPieExists: function calcIsPieExists() {
    return this.get('modelValueMax') > 0;
  },

  calcDataForPieChart: function calcDataForPieChart() {
    var used = this.getUsed();
    var total = this.getMax();
    var percent = total > 0 ? (used * 100 / total).toFixed() : 0;
    var percentPrecise = total > 0 ? (used * 100 / total).toFixed(1) : 0;
    return [percent, percentPrecise];
  },

  calc: function calc() {
    this.set('hiddenInfo', this.calcHiddenInfo());
    var isPieExists = this.calcIsPieExists();
    this.set('isPieExist', isPieExists);
    if (isPieExists) {
      this.set('dataForPieChart', this.calcDataForPieChart());
    }
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.addObserver('modelValueMax', this, this.calc);
    this.addObserver('modelValueUsed', this, this.calc);
  },

  content: App.ChartPieView.extend({
    model: null, //data bind here
    id: Em.computed.alias('parentView.widgetHtmlId'), // html id
    stroke: 'transparent',
    thresholdMin: null, //bind from parent
    thresholdMax: null,
    innerR: 40,

    existCenterText: true,
    centerTextColor: Em.computed.alias('contentColor'),

    palette: new Rickshaw.Color.Palette({
      scheme: ['#FFFFFF', '#D6DDDF'].reverse()
    }),

    data: function () {
      var oriData = this.get('parentView.dataForPieChart');
      return [oriData[0], 100 - oriData[0]];
    }.property(),

    setData: function () {
      this.set('data', this.get('parentView.dataForPieChart'));
    }.observes('parentView.dataForPieChart'),

    contentColor: function () {
      var used = parseFloat(this.get('parentView.dataForPieChart')[1]);
      var thresholdMin = parseFloat(this.get('thresholdMin'));
      var thresholdMax = parseFloat(this.get('thresholdMax'));
      if (used <= thresholdMin) {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', App.healthStatusGreen].reverse()
        }));
      } else if (used <= thresholdMax) {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', App.healthStatusOrange].reverse()
        }));
      } else {
        this.set('palette', new Rickshaw.Color.Palette({
          scheme: ['#DDDDDD', App.healthStatusRed].reverse()
        }));
      }
      return App.widgetContentColor;
    }.property('data', 'thresholdMin', 'thresholdMax'),

    // refresh text and color when data in model changed
    refreshSvg: function () {
      // remove old svg
      var oldSvg = $("#" + this.get('id'));
      if (oldSvg) {
        oldSvg.remove();
      }

      // draw new svg
      this.appendSvg();
    }.observes('data', 'thresholdMin', 'thresholdMax')
  })
});

});

require.register("views/main/dashboard/widgets/pxf_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function counterOrNA(key) {
  var _key = 'model.{0}'.format(key);
  return Em.computed(_key, function () {
    var value = this.get(_key);
    if (Em.isNone(value)) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return value;
  });
}

App.PxfUpView = App.TextDashboardSingleThresholdWidgetView.extend(App.SingleNumericThresholdMixin, {

  hiddenInfo: function () {
    return [this.get('pxfsStarted') + ' ' + Em.I18n.t('dashboard.services.components.started'), this.get('pxfsInstalled') + ' ' + Em.I18n.t('dashboard.services.components.stopped'), this.get('pxfsTotal') + ' ' + Em.I18n.t('dashboard.services.components.total')];
  }.property('pxfsStarted', 'pxfsInstalled', 'pxfsTotal'),

  hiddenInfoClass: "hidden-info-three-line",

  maxValue: counterOrNA('pxfsTotal'),

  pxfsStarted: counterOrNA('pxfsStarted'),

  pxfsInstalled: counterOrNA('pxfsInstalled'),

  pxfsTotal: counterOrNA('pxfsTotal'),

  /**
   * @type {?number}
   */
  data: function () {
    if (this.get('someMetricsNA')) {
      return null;
    }
    return this.get('pxfsTotal') - this.get('pxfsStarted');
  }.property('model.pxfsTotal', 'pxfsStarted', 'someMetricsNA'),

  /**
   * @type {string}
   */
  content: function () {
    if (this.get('someMetricsNA')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get('pxfsStarted') + "/" + this.get('model.pxfsTotal');
  }.property('model.pxfsTotal', 'pxfsStarted', 'someMetricsNA'),

  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint4').format(Em.I18n.t('dashboard.widgets.PXFAgents'), maxTmp);
  }.property('maxValue'),

  /**
   * @type {boolean}
   */
  someMetricsNA: function () {
    return Em.isNone(this.get('model.pxfsStarted')) || Em.isNone(this.get('model.pxfsTotal'));
  }.property('model.pxfsStarted', 'model.pxfsTotal')

});

});

require.register("views/main/dashboard/widgets/resource_manager_heap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ResourceManagerHeapPieChartView = App.PieChartDashboardWidgetView.extend({

  modelValueMax: Em.computed.alias('model.jvmMemoryHeapMax'),
  modelValueUsed: Em.computed.alias('model.jvmMemoryHeapUsed'),
  widgetHtmlId: 'widget-rm-heap',

  getUsed: function getUsed() {
    return this.get('modelValueUsed') / (1024 * 1024) || 0;
  },

  getMax: function getMax() {
    return this.get('modelValueMax') / (1024 * 1024) || 0;
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  }

});

});

require.register("views/main/dashboard/widgets/resource_manager_uptime", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ResourceManagerUptimeView = App.UptimeTextDashboardWidgetView.extend({

  component: 'ResourceManager',
  modelValue: Em.computed.alias('model.resourceManagerStartTime')

});

});

require.register("views/main/dashboard/widgets/supervisor_live", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SuperVisorUpView = App.TextDashboardWidgetView.extend(App.EditableWithLimitWidgetMixin, {

  hiddenInfo: function () {
    return [this.get('superVisorsLive') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.live'), this.get('superVisorsDead') + ' ' + Em.I18n.t('dashboard.services.hdfs.nodes.dead')];
  }.property('superVisorsLive', 'superVisorsDead'),

  hiddenInfoClass: "hidden-info-two-line",

  maxValue: 100,

  superVisorsLive: Em.computed.alias('model.superVisorsStarted'),

  superVisorsDead: Em.computed.alias('model.superVisorsInstalled'),

  superVisorsTotal: Em.computed.alias('model.superVisorsTotal'),

  data: function () {
    if (!this.get('superVisorsTotal') || Em.isNone(this.get('superVisorsLive'))) {
      return -1;
    }
    return (this.get('superVisorsLive') / this.get('superVisorsTotal')).toFixed(2) * 100;
  }.property('superVisorsTotal', 'superVisorsLive'),

  content: function () {
    if (!Em.isNone(this.get('superVisorsTotal')) && !Em.isNone(this.get('superVisorsLive'))) {
      return this.get('superVisorsLive') + "/" + this.get('superVisorsTotal');
    }
    return Em.I18n.t('services.service.summary.notAvailable');
  }.property('superVisorsLive', 'superVisorsTotal'),

  hintInfo: function () {
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(parseFloat(this.get('maxValue')));
  }.property('maxValue')
});

});

require.register("views/main/dashboard/widgets/text_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TextDashboardWidgetView = App.DashboardWidgetView.extend({

  templateName: require('templates/main/dashboard/widgets/simple_text'),

  classNameBindings: ['isRed', 'isOrange', 'isGreen', 'isNA'],

  isEditable: true,

  isRed: Em.computed.lteProperties('data', 'thresholdMin'),
  isOrange: Em.computed.and('!isGreen', '!isRed'),
  isGreen: Em.computed.gtProperties('data', 'thresholdMax'),

  isNA: function () {
    return this.get('data') === null;
  }.property('data'),

  hiddenInfo: []
});

});

require.register("views/main/dashboard/widgets/text_widget_single_threshold", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TextDashboardSingleThresholdWidgetView = App.DashboardWidgetView.extend({

  templateName: require('templates/main/dashboard/widgets/simple_text'),

  classNameBindings: ['isRed', 'isGreen', 'isNA'],
  isGreen: Em.computed.lteProperties('data', 'thresholdMin'),
  isRed: Em.computed.gtProperties('data', 'thresholdMin'),

  isNA: function () {
    return this.get('data') === null;
  }.property('data'),

  hiddenInfo: [],

  maxValue: null

});

});

require.register("views/main/dashboard/widgets/uptime_text_widget", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');

App.UptimeTextDashboardWidgetView = App.TextDashboardWidgetView.extend({

  templateName: require('templates/main/dashboard/widgets/uptime'),

  timeUnit: null,

  hiddenInfoClass: "hidden-info-three-line",

  maxValue: 'infinity',

  component: null,

  /**
   * Value received from model that's used to calculate data and content
   * Should be defined in every child
   */
  modelValue: null,

  data: null,

  content: Em.computed.alias('data'),

  isGreen: function () {
    return !Em.isNone(this.get('data'));
  }.property('data'),

  isOrange: false,

  isRed: false,

  timeConverter: function timeConverter(timestamp) {
    var m = moment(new Date(timestamp));
    return [m.format('ddd MMM DD YYYY'), m.format('HH:mm:ss')];
  },

  /**
   * All children should have such code
   * <code>
   * didInsertElement: function() {
   *   this._super();
   *   this.calc();
   * }
   * </code>
   */
  didInsertElement: function didInsertElement() {
    this._super();
    this.calculate();
    this.addObserver('modelValue', this, this.calculate);
  },

  calculate: function calculate() {
    var uptime = this.get('modelValue');
    if (uptime) {
      var diff = App.dateTimeWithTimeZone() - App.dateTimeWithTimeZone(uptime);
      diff = diff < 0 ? 0 : diff;
      var formatted = date.timingFormat(diff);
      var uptimeString = this.timeConverter(uptime);

      this.setProperties({
        data: formatted,
        hiddenInfo: [formatted, uptimeString[0], uptimeString[1]]
      });
    } else {
      this.setProperties({
        data: null,
        hiddenInfo: [this.get('component'), Em.I18n.t('services.service.summary.notRunning')]
      });
    }
  }
});

});

require.register("views/main/dashboard/widgets/yarn_containers", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

function counterOrNA(key) {
  var _key = 'model.{0}.length'.format(key);
  return Em.computed(_key, function () {
    if (Em.isNone(this.get('model.' + key)) || this.get('model.metricsNotAvailable')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get(_key);
  });
}

App.YarnContainersView = App.TextDashboardWidgetView.extend({

  hiddenInfo: function () {
    return [this.get('containersAllocated') + ' ' + Em.I18n.t('dashboard.services.yarn.containers.allocated'), this.get('containersPending') + ' ' + Em.I18n.t('dashboard.services.yarn.containers.pending'), this.get('containersReserved') + ' ' + Em.I18n.t('dashboard.services.yarn.containers.reserved')];
  }.property('containersAllocated', 'containersPending', 'containersReserved'),

  hiddenInfoClass: "hidden-info-three-line",

  maxValue: 'infinity',

  isEditable: false,

  containersAllocated: counterOrNA('containersAllocated'),

  containersPending: counterOrNA('containersPending'),

  containersReserved: counterOrNA('containersReserved'),

  isRed: Em.computed.alias('someMetricsNA'),

  isOrange: false,

  isGreen: Em.computed.equal('someMetricsNA', false),

  /**
   * @type {string}
   */
  content: function () {
    if (this.get('someMetricsNA')) {
      return Em.I18n.t('services.service.summary.notAvailable');
    }
    return this.get('containersAllocated') + "/" + this.get('containersPending') + "/" + this.get('containersReserved');
  }.property('containersAllocated', 'containersPending', 'containersReserved'),

  /**
   * @type {boolean}
   */
  someMetricsNA: function () {
    return Em.isNone(this.get('containersAllocated')) || Em.isNone(this.get('containersPending')) || Em.isNone(this.get('containersReserved'));
  }.property('containersAllocated', 'containersPending', 'containersReserved'),

  /**
   * @type {string}
   */
  hintInfo: function () {
    var maxTmp = parseFloat(this.get('maxValue'));
    return Em.I18n.t('dashboard.widgets.hintInfo.hint1').format(maxTmp);
  }.property('maxValue')
});

});

require.register("views/main/dashboard/widgets/yarn_links", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.YARNLinksView = App.LinkDashboardWidgetView.extend({
  templateName: require('templates/main/dashboard/widgets/yarn_links'),

  componentName: 'NODEMANAGER'
});

});

require.register("views/main/dashboard/widgets/yarn_memory", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.YARNMemoryPieChartView = App.PieChartDashboardWidgetView.extend({

  widgetHtmlId: 'widget-yarn-memory',
  modelValueUsed: Em.computed.alias('model.allocatedMemory'),
  modelValueMax: Em.computed.alias('model.maxMemory'),

  didInsertElement: function didInsertElement() {
    this._super();
    this.calc();
  }
});

});

require.register("views/main/host", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var sort = require('views/common/sort_view');

App.MainHostView = App.TableView.extend(App.TableServerViewMixin, {
  templateName: require('templates/main/host'),

  tableName: 'Hosts',
  updaterBinding: 'App.router.updateController',
  filterConditions: [],

  /**
   * Select/deselect all visible hosts flag
   * @property {bool}
   */
  selectAllHosts: false,

  /**
   * Contains all selected hosts on cluster
   */
  selectedHosts: [],

  /**
   * Request error data
   */
  requestError: null,

  /**
   * List of hosts in cluster
   * @type {Array}
   */
  contentBinding: 'controller.content',

  filteredContent: [],

  filteredContentObserver: function () {
    Em.run.once(this, this.setFilteredContentOnce);
  }.observes('content.@each'),

  setFilteredContentOnce: function setFilteredContentOnce() {
    this.set('filteredContent', this.get('content').filterProperty('isFiltered'));
  },

  onRequestErrorHandler: function () {
    this.set('requestError', null);
    this.set('filteringComplete', true);
    this.propertyDidChange('pageContent');
  }.observes('requestError'),

  /**
   * flag to toggle displaying selected hosts counter
   */
  showSelectedFilter: Em.computed.bool('selectedHosts.length'),

  /**
   * return filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('hosts.filters.filteredHostsInfo', 'filteredCount', 'totalCount'),

  /**
   * request latest data filtered by new parameters
   * called when trigger property(<code>refreshTriggers</code>) is changed
   */
  refresh: function refresh() {
    App.loadTimer.start('Hosts Page');
    this.set('filteringComplete', false);
    var updaterMethodName = this.get('updater.tableUpdaterMap')[this.get('tableName')];
    this.get('updater')[updaterMethodName](this.updaterSuccessCb.bind(this), this.updaterErrorCb.bind(this), true);
    return true;
  },

  /**
   * reset filters value by column to which filter belongs
   * @param columns {Array}
   */
  resetFilterByColumns: function resetFilterByColumns(columns) {
    var filterConditions = this.get('filterConditions');
    columns.forEach(function (iColumn) {
      var filterCondition = filterConditions.findProperty('iColumn', iColumn);

      if (filterCondition) {
        filterCondition.value = '';
        this.saveFilterConditions(filterCondition.iColumn, filterCondition.value, filterCondition.type, filterCondition.skipFilter);
      }
    }, this);
  },

  /**
   * Return pagination information displayed on the page
   * @type {String}
   */
  paginationInfo: Em.computed.i18nFormat('tableView.filters.paginationInfo', 'startIndex', 'endIndex', 'filteredCount'),

  paginationLeftClass: function () {
    if (this.get("startIndex") > 1 && this.get('filteringComplete')) {
      return "paginate_previous";
    }
    return "paginate_disabled_previous";
  }.property("startIndex", 'filteringComplete'),

  previousPage: function previousPage() {
    if (this.get('paginationLeftClass') === 'paginate_previous') {
      this._super();
    }
  },

  paginationRightClass: function () {
    if (this.get("endIndex") < this.get("filteredCount") && this.get('filteringComplete')) {
      return "paginate_next";
    }
    return "paginate_disabled_next";
  }.property("endIndex", 'filteredCount', 'filteringComplete'),

  nextPage: function nextPage() {
    if (this.get('paginationRightClass') === 'paginate_next') {
      this._super();
    }
  },

  /**
   * Select View with list of "rows-per-page" options
   * @type {Ember.View}
   */
  rowsPerPageSelectView: Em.Select.extend(App.PaginationSelectMixin, {

    disabled: true,

    disableView: function () {
      Em.run.next(this, function () {
        this.set('disabled', !this.get('parentView.dataView.filteringComplete'));
      });
    }.observes('parentView.dataView.filteringComplete'),

    change: function change() {
      this.get('parentView.dataView').saveDisplayLength();
      var self = this;
      if (this.get('parentView.dataView.startIndex') !== 1 && this.get('parentView.dataView.startIndex') !== 0) {
        Ember.run.next(function () {
          self.set('parentView.dataView.startIndex', 1);
        });
      }
    }
  }),

  saveStartIndex: function () {
    this.set('controller.startIndex', this.get('startIndex'));
  }.observes('startIndex'),

  clearFiltersObs: function clearFiltersObs() {
    var self = this;
    Em.run.next(function () {
      if (self.get('controller.clearFilters')) {
        self.clearFilters();
      }
    });
  },
  /**
   * Restore filter properties in view
   */
  willInsertElement: function willInsertElement() {
    if (!this.get('controller.showFilterConditionsFirstLoad')) {
      // filters and start index should be cleared when we enter Hosts page
      var didClearedFilters = this.clearFilterConditionsFromLocalStorage();
      var didClearStartIndex = this.clearStartIndex();
      this.set('controller.filterChangeHappened', didClearedFilters || didClearStartIndex);
    }
    if (!this.get('controller.saveSelection')) {
      this.refresh();
    }
    this._super();
    this.set('startIndex', this.get('controller.startIndex'));
    this.set('displayLength', this.get('controller.paginationProps').findProperty('name', 'displayLength').value);
    this.addObserver('pageContent.@each.selected', this, this.selectedHostsObserver);
  },

  /**
   * stub for filter function in TableView
   */
  filter: function filter() {
    //Since filtering moved to server side, function is empty
  },

  didInsertElement: function didInsertElement() {
    this.addObserver('controller.clearFilters', this, this.clearFiltersObs);
    this.clearFiltersObs();
    this.addObserver('selectAllHosts', this, this.toggleAllHosts);
    this.addObserver('filteringComplete', this, this.overlayObserver);
    this.addObserver('startIndex', this, 'updatePagination');
    this.addObserver('displayLength', this, 'updatePagination');
    this.addObserver('filteredCount', this, this.updatePaging);
    // should show overlay even when filtering has begun before observer was added
    this.overlayObserver();
    this.combineSelectedFilter();
  },

  willDestroyElement: function willDestroyElement() {
    $('.tooltip').remove();
  },

  onInitialLoad: function () {
    if (this.get('tableFilteringComplete')) {
      if (this.get('controller.filterChangeHappened')) {
        this.refresh();
      } else {
        // no refresh but still need to enable pagination controls
        this.propertyDidChange('filteringComplete');
      }
    }
    // reset filter change marker
    this.set('controller.filterChangeHappened', false);
  }.observes('tableFilteringComplete'),

  /**
   * Set <code>selected</code> property for each App.Host
   */
  toggleAllHosts: function toggleAllHosts() {
    this.get('pageContent').setEach('selected', this.get('selectAllHosts'));
  },

  /**
   * Trigger updating <code>selectedHostsCount</code> only 1 time
   */
  selectedHostsObserver: function selectedHostsObserver() {
    Ember.run.once(this, 'updateCheckedFlags');
  },

  /**
   * Update <code>selectAllHosts</code> value
   */
  updateCheckedFlags: function updateCheckedFlags() {
    this.removeObserver('selectAllHosts', this, this.toggleAllHosts);
    if (this.get('pageContent').length) {
      this.set('selectAllHosts', this.get('pageContent').everyProperty('selected', true));
    } else {
      this.set('selectAllHosts', false);
    }
    this.combineSelectedFilter();
    App.db.setSelectedHosts(this.get('selectedHosts'));

    this.addObserver('selectAllHosts', this, this.toggleAllHosts);
  },
  /**
   * combine selected hosts on page with selected hosts which are filtered out but added to cluster
   */
  combineSelectedFilter: function combineSelectedFilter() {
    var previouslySelectedHosts = App.db.getSelectedHosts();
    var selectedHosts = [];
    var hostsOnPage = this.get('pageContent').mapProperty('hostName');
    selectedHosts = this.get('pageContent').filterProperty('selected').mapProperty('hostName');

    previouslySelectedHosts.forEach(function (hostName) {
      if (!hostsOnPage.contains(hostName)) {
        selectedHosts.push(hostName);
      }
    }, this);
    this.set('selectedHosts', selectedHosts);
  },

  /**
   * filter selected hosts
   */
  filterSelected: function filterSelected() {
    //10 is an index of selected column
    this.updateFilter(10, this.get('selectedHosts'), 'multiple');
  },

  /**
   * Show spinner when filter/sorting request is in processing
   * @method overlayObserver
   */
  overlayObserver: function overlayObserver() {
    var $tbody = this.$('tbody'),
        $overlay = this.$('.table-overlay'),
        $spinner = $($overlay).find('.spinner');
    if (!this.get('filteringComplete')) {
      if (!$tbody) return;
      var tbodyPos = $tbody.position();
      if (!tbodyPos) return;
      $spinner.css('display', 'block');
      $overlay.css({
        top: tbodyPos.top + 1,
        left: tbodyPos.left + 1,
        width: $tbody.width() - 1,
        height: $tbody.height() - 1
      });
    }
  },

  /**
   * Clear selectedFilter
   * Set <code>selected</code> to false for each host
   */
  clearSelection: function clearSelection() {
    this.get('pageContent').setEach('selected', false);
    this.set('selectAllHosts', false);
    App.db.setSelectedHosts([]);
    this.get('selectedHosts').clear();
    this.filterSelected();
  },

  sortView: sort.serverWrapperView,
  nameSort: sort.fieldView.extend({
    column: 1,
    name: 'hostName',
    displayName: Em.I18n.t('common.name')
  }),
  ipSort: sort.fieldView.extend({
    column: 2,
    name: 'ip',
    displayName: Em.I18n.t('common.ipAddress'),
    type: 'ip'
  }),
  rackSort: sort.fieldView.extend({
    column: 12,
    name: 'rack',
    displayName: Em.I18n.t('common.rack'),
    type: 'rack'
  }),
  cpuSort: sort.fieldView.extend({
    column: 3,
    name: 'cpu',
    displayName: Em.I18n.t('common.cores'),
    type: 'number'
  }),
  memorySort: sort.fieldView.extend({
    column: 4,
    name: 'memoryFormatted',
    displayName: Em.I18n.t('common.ram'),
    type: 'number'
  }),
  diskUsageSort: sort.fieldView.extend({
    name: 'diskUsage',
    displayName: Em.I18n.t('common.diskUsage')
  }),
  loadAvgSort: sort.fieldView.extend({
    column: 5,
    name: 'loadAvg',
    displayName: Em.I18n.t('common.loadAvg'),
    type: 'number'
  }),

  HostView: Em.View.extend({
    content: null,
    tagName: 'tr',
    classNameBindings: ['content.selected:active'],
    didInsertElement: function didInsertElement() {
      App.tooltip(this.$("[rel='HealthTooltip'], [rel='UsageTooltip'], [rel='ComponentsTooltip']"));
    },

    willDestroyElement: function willDestroyElement() {
      this.$("[rel='HealthTooltip'], [rel='UsageTooltip'], [rel='ComponentsTooltip']").remove();
    },

    displayComponents: function displayComponents() {
      if (this.get('hasNoComponents')) {
        return;
      }
      var header = Em.I18n.t('common.components'),
          hostName = this.get('content.hostName'),
          items = this.get('content.hostComponents').getEach('displayName');
      App.showHostsTableListPopup(header, hostName, items);
    },

    displayVersions: function displayVersions() {
      if (this.get('hasSingleVersion')) {
        return;
      }
      var header = Em.I18n.t('common.versions'),
          hostName = this.get('content.hostName'),
          items = this.get('content.stackVersions').filterProperty('isVisible').map(function (stackVersion) {
        return {
          name: stackVersion.get('displayName'),
          status: App.format.role(stackVersion.get('status'), false)
        };
      });
      App.showHostsTableListPopup(header, hostName, items);
    },

    /**
     * Tooltip message for "Restart Required" icon
     * @returns {String}
     */
    restartRequiredComponentsMessage: function () {
      var restartRequiredComponents = this.get('content.componentsWithStaleConfigs');
      var count = this.get('content.componentsWithStaleConfigsCount');
      if (count <= 5) {
        var word = count == 1 ? Em.I18n.t('common.component') : Em.I18n.t('common.components');
        return Em.I18n.t('hosts.table.restartComponents.withNames').format(restartRequiredComponents.getEach('displayName').join(', ')) + ' ' + word.toLowerCase();
      }
      return Em.I18n.t('hosts.table.restartComponents.withoutNames').format(count);
    }.property('content.componentsWithStaleConfigs'),

    /**
     * Tooltip message for "Maintenance" icon
     * @returns {String}
     */
    componentsInPassiveStateMessage: function () {
      var componentsInPassiveState = this.get('content.componentsInPassiveState');
      var count = this.get('content.componentsInPassiveStateCount');
      if (count <= 5) {
        return Em.I18n.t('hosts.table.componentsInPassiveState.withNames').format(componentsInPassiveState.getEach('displayName').join(', '));
      }
      return Em.I18n.t('hosts.table.componentsInPassiveState.withoutNames').format(count);
    }.property('content.componentsInPassiveState'),

    /**
     * true if host has only one repoversion
     * in this case expander in version column is hidden
     * @returns {Boolean}
     */
    hasSingleVersion: function () {
      return this.get('content.stackVersions').filterProperty('isVisible', true).length < 2;
    }.property('content.stackVersions.length'),

    /**
     * true if host has no components
     * @returns {Boolean}
     */
    hasNoComponents: Em.computed.empty('content.hostComponents'),

    /**
     /**
     * this version is always shown others hidden unless expander is open
     * host may have no stack versions
     * @returns {String}
     */
    currentVersion: function () {
      var currentRepoVersion = this.get('content.stackVersions').findProperty('isCurrent') || this.get('content.stackVersions').objectAt(0);
      return currentRepoVersion ? currentRepoVersion.get('displayName') : "";
    }.property('content.stackVersions'),

    /**
     * CSS value for disk usage bar
     * @returns {String}
     */
    usageStyle: function () {
      return "width:" + this.get('content.diskUsage') + "%";
    }.property('content.diskUsage')

  }),

  /**
   * associations between host property and column index
   * @type {Array}
   */
  colPropAssoc: Em.computed.alias('controller.colPropAssoc'),

  /**
   * Run <code>clearFilter</code> in the each child filterView
   */
  clearFilters: function clearFilters() {
    // clean filters stored in-memory and local storage
    this.set('filterConditions', []);
    this.clearFilterConditionsFromLocalStorage();
    // force refresh
    this.refresh();
  }
});

});

require.register("views/main/host/addHost/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHostStep4View = Em.View.extend({
  templateName: require('templates/main/host/addHost/step4')
});

});

require.register("views/main/host/add_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddHostView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/host/add'),

  /**
   * @type {boolean}
   */
  isLoaded: false,

  willInsertElement: function willInsertElement() {
    if (this.get('controller').getDBProperty('hosts')) {
      this.set('isLoaded', true);
    } else {
      this.loadHosts();
    }
  },

  loadHosts: function loadHosts() {
    App.ajax.send({
      name: 'hosts.confirmed.minimal',
      sender: this,
      data: {},
      success: 'loadHostsSuccessCallback',
      error: 'loadHostsErrorCallback'
    });
  },

  loadHostsSuccessCallback: function loadHostsSuccessCallback(response) {
    var installedHosts = {};

    response.items.forEach(function (item) {
      installedHosts[item.Hosts.host_name] = {
        name: item.Hosts.host_name,
        bootStatus: "REGISTERED",
        isInstalled: true,
        hostComponents: item.host_components
      };
    });
    this.get('controller').setDBProperty('hosts', installedHosts);
    this.set('controller.content.hosts', installedHosts);
    this.set('isLoaded', true);
  },

  loadHostsErrorCallback: function loadHostsErrorCallback() {
    this.set('isLoaded', true);
  }

});

});

require.register("views/main/host/combo_search_box", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostComboSearchBoxView = App.SearchBoxView.extend({
  healthStatusCategories: require('data/host/categories'),
  serviceMap: {},
  controllerBinding: 'App.router.mainHostComboSearchBoxController',

  initVS: function initVS() {
    this.setupLabelMap();
    window.visualSearch = VS.init({
      container: $('#combo_search_box'),
      query: '',
      showFacets: true,
      delay: 1000,
      placeholder: Em.I18n.t('hosts.combo.search.placebolder'),
      unquotable: ['text'],
      remainder: 'Host Name',
      callbacks: {
        search: this.search.bind(this),
        facetMatches: this.facetMatches.bind(this),
        valueMatches: this.valueMatches.bind(this)
      }
    });
  },

  /**
   * 'search' call back for visualsearch.js
   * @param query
   * @param searchCollection
   */
  search: function search(query, searchCollection) {
    var tableView = this.get('parentView');
    App.db.setComboSearchQuery(tableView.get('controller.name'), query);
    var filterConditions = this.createFilterConditions(searchCollection);
    tableView.updateComboFilter(filterConditions);
  },

  /**
   * 'facetMatches' callback for visualsearch.js
   * @param callback
   */
  facetMatches: function facetMatches(callback) {
    this.showHideClearButton();
    var list = [{ label: 'Host Name', category: 'Host' }, { label: 'IP', category: 'Host' }, { label: 'Host Status', category: 'Host' }, { label: 'Cores', category: 'Host' }, { label: 'RAM', category: 'Host' }, { label: 'Stack Version', category: 'Host' }, { label: 'Version State', category: 'Host' }, { label: 'Rack', category: 'Host' }, { label: 'Service', category: 'Service' }];
    var hostComponentList = this.getHostComponentList();
    // Add host component facets only when there isn't any component filter
    // with value other than ALL yet
    var currentComponentFacets = this.getComponentStateFacets(hostComponentList, false);
    if (currentComponentFacets.length === 0) {
      list = list.concat(hostComponentList);
    }
    list = this.filterOutOneFacetOnlyOptions(list);
    // Append host components
    callback(list, { preserveOrder: true });
  },

  /**
   * 'valueMatches' callback for visualsearch.js
   * @param facet
   * @param searchTerm
   * @param callback
   */
  valueMatches: function valueMatches(facet, searchTerm, callback) {
    var controller = App.router.get('mainHostComboSearchBoxController');
    this.showHideClearButton();
    var map = App.router.get('mainHostController.labelValueMap');
    var facetValue = map[facet] || facet;
    if (controller.isComponentStateFacet(facetValue)) {
      facetValue = 'componentState';
    }
    switch (facetValue) {
      case 'hostName':
      case 'ip':
        this.searchByHostname(facetValue, searchTerm, facet, callback);
        break;
      case 'rack':
        this.searchByRack(facet, callback);
        break;
      case 'version':
        this.searchByVersion(facet, callback);
        break;
      case 'versionState':
        this.searchByVersionState(facet, callback, map);
        break;
      case 'healthClass':
        this.searchByHealthClass(facet, callback, map);
        break;
      case 'services':
        this.searchByServices(facet, callback);
        break;
      case 'componentState':
        this.searchByComponentState(facet, callback, map);
        break;
    }
  },

  /**
   *
   * @param {string} facetValue
   * @param {string} searchTerm
   * @param {string} facet
   * @param {Function} callback
   */
  searchByHostname: function searchByHostname(facetValue, searchTerm, facet, callback) {
    var controller = App.router.get('mainHostComboSearchBoxController');

    controller.getPropertySuggestions(facetValue, searchTerm).done(function () {
      callback(controller.get('currentSuggestion').reject(function (item) {
        return visualSearch.searchQuery.values(facet).indexOf(item) >= 0; // reject the ones already in search
      }), { preserveMatches: true });
    });
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   */
  searchByRack: function searchByRack(facet, callback) {
    callback(App.Host.find().toArray().mapProperty('rack').uniq().reject(function (item) {
      return visualSearch.searchQuery.values(facet).indexOf(item) >= 0;
    }));
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   */
  searchByVersion: function searchByVersion(facet, callback) {
    callback(App.HostStackVersion.find().toArray().filterProperty('isVisible', true).mapProperty('displayName').uniq().reject(function (item) {
      return visualSearch.searchQuery.values(facet).indexOf(item) >= 0;
    }));
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   * @param {object} map
   */
  searchByVersionState: function searchByVersionState(facet, callback, map) {
    callback(App.HostStackVersion.statusDefinition.map(function (status) {
      map[App.HostStackVersion.formatStatus(status)] = status;
      return App.HostStackVersion.formatStatus(status);
    }).reject(function (item) {
      return visualSearch.searchQuery.values(facet).indexOf(item) >= 0;
    }));
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   * @param {object} map
   */
  searchByHealthClass: function searchByHealthClass(facet, callback, map) {
    //exclude "All" category
    var category_mocks = this.get('healthStatusCategories').slice(1);

    callback(category_mocks.map(function (category) {
      map[category.value] = category.healthStatus;
      return category.value;
    }).reject(function (item) {
      return visualSearch.searchQuery.values(facet).indexOf(item) >= 0;
    }), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   */
  searchByServices: function searchByServices(facet, callback) {
    var serviceMap = this.get('serviceMap');

    callback(App.Service.find().toArray().map(function (service) {
      serviceMap[App.format.role(service.get('serviceName'), true)] = service.get('serviceName');
      return App.format.role(service.get('serviceName'), true);
    }).reject(function (item) {
      return visualSearch.searchQuery.values(facet).indexOf(item) >= 0;
    }), { preserveOrder: true });
  },

  /**
   *
   * @param {string} facet
   * @param {Function} callback
   * @param {object} map
   */
  searchByComponentState: function searchByComponentState(facet, callback, map) {
    var list = ["All"];
    // client only have "ALL" state
    if (facet.toLowerCase().indexOf("client") === -1) {
      var currentComponentFacets = this.getComponentStateFacets(null, true);
      if (currentComponentFacets.length === 0) {
        list = list.concat(App.HostComponentStatus.getStatusesList().reject(function (status) {
          return status === "UPGRADE_FAILED";
        }) // take out 'UPGRADE_FAILED'
        .map(function (status) {
          map[App.HostComponentStatus.getTextStatus(status)] = status;
          return App.HostComponentStatus.getTextStatus(status);
        })).concat(["Inservice", "Decommissioned", "Decommissioning", "RS Decommissioned", "Maintenance Mode On", "Maintenance Mode Off"]);
      }
    }
    callback(list, { preserveOrder: true });
  },

  getHostComponentList: function getHostComponentList() {
    var hostComponentList = [];
    App.MasterComponent.find().rejectProperty('totalCount', 0).toArray().concat(App.SlaveComponent.find().rejectProperty('totalCount', 0).toArray().concat(App.ClientComponent.find().rejectProperty('totalCount', 0).toArray())).forEach(function (component) {
      var displayName = component.get('displayName');
      if (displayName) {
        hostComponentList.push({ label: displayName, category: 'Component' });
        App.router.get('mainHostController.labelValueMap')[displayName] = component.get('componentName');
      }
    });
    hostComponentList = hostComponentList.sortProperty('label');
    return hostComponentList;
  },

  /**
   *
   * @param {Array} hostComponentList
   * @param {boolean} includeAllValue
   * @returns {Array}
   */
  getComponentStateFacets: function getComponentStateFacets(hostComponentList, includeAllValue) {
    if (!hostComponentList) {
      hostComponentList = this.getHostComponentList();
    }
    return visualSearch.searchQuery.toJSON().filter(function (facet) {
      var result = Boolean(hostComponentList.findProperty('label', facet.category) && facet.value);
      if (!includeAllValue) {
        result &= facet.value !== 'All';
      }
      return result;
    });
  },

  /**
   *
   * @param {string} name
   * @returns {Array}
   */
  getFacetsByName: function getFacetsByName(name) {
    return visualSearch.searchQuery.toJSON().filter(function (facet) {
      return facet.category === name;
    });
  },

  filterOutOneFacetOnlyOptions: function filterOutOneFacetOnlyOptions(options) {
    var self = this;
    var oneFacetOnlyLables = ['Cores', 'RAM'];
    oneFacetOnlyLables.forEach(function (label) {
      var facets = self.getFacetsByName(label);
      if (facets.length > 0) {
        options = options.rejectProperty('label', label);
      }
    });
    return options;
  },

  setupLabelMap: function setupLabelMap() {
    var map = App.router.get('mainHostController.labelValueMap');
    map['All'] = 'ALL';
    map['Host Name'] = 'hostName';
    map['IP'] = 'ip';
    map['Host Status'] = 'healthClass';
    map['Cores'] = 'cpu';
    map['RAM'] = 'memoryFormatted';
    map['Stack Version'] = 'version';
    map['Version State'] = 'versionState';
    map['Rack'] = 'rack';
    map['Service'] = 'services';

    map['Inservice'] = 'INSERVICE';
    map['Decommissioned'] = 'DECOMMISSIONED';
    map['Decommissioning'] = 'DECOMMISSIONING';
    map['RS Decommissioned'] = 'RS_DECOMMISSIONED';
    map['Maintenance Mode On'] = 'ON';
    map['Maintenance Mode Off'] = 'OFF';
  },

  createFilterConditions: function createFilterConditions(searchCollection) {
    var map = App.router.get('mainHostController.labelValueMap');
    var serviceMap = this.get('serviceMap');
    var filterConditions = Em.A();

    searchCollection.models.forEach(function (model) {
      var tag = model.attributes;
      var category = map[tag.category] || tag.category;
      var value = category === 'services' ? serviceMap[tag.value] || tag.value : map[tag.value] || tag.value;
      var iColumn = this.getFilterColumn(category, value);
      var filterValue = this.getFilterValue(category, value);
      var condition = filterConditions.findProperty('iColumn', iColumn);

      if (condition) {
        // handle multiple facets with same category
        if (typeof condition.value === 'string') {
          condition.value = [condition.value, filterValue];
        } else if (Em.isArray(condition.value) && condition.value.indexOf(filterValue) === -1) {
          condition.value.push(filterValue);
        }
      } else {
        filterConditions.push(this.createCondition(category, iColumn, filterValue));
      }
    }, this);
    return filterConditions;
  },

  /**
   *
   * @param {string} category
   * @param {number} iColumn
   * @param {string} filterValue
   * @returns {{skipFilter: boolean, iColumn: number, value: string, type: string}}
   */
  createCondition: function createCondition(category, iColumn, filterValue) {
    var type = 'string';
    if (category === 'cpu') {
      type = 'number';
    }
    if (category === 'memoryFormatted') {
      type = 'ambari-bandwidth';
    }
    return {
      skipFilter: false,
      iColumn: iColumn,
      value: filterValue,
      type: type
    };
  },

  /**
   *
   * @param {string} category
   * @param {string} value
   * @returns {number}
   */
  getFilterColumn: function getFilterColumn(category, value) {
    var iColumn;
    if (this.get('controller').isComponentStateFacet(category)) {
      iColumn = App.router.get('mainHostController.colPropAssoc').indexOf('componentState');
    } else if (this.get('controller').isComplexHealthStatusFacet(value)) {
      iColumn = this.get('healthStatusCategories').findProperty('healthStatus', value).column;
    } else {
      iColumn = App.router.get('mainHostController.colPropAssoc').indexOf(category);
    }
    return iColumn;
  },

  /**
   *
   * @param {string} category
   * @param {string} value
   * @returns {string}
   */
  getFilterValue: function getFilterValue(category, value) {
    var filterValue = value;
    if (this.get('controller').isComponentStateFacet(category)) {
      filterValue = category + ':' + value;
    } else if (this.get('controller').isComplexHealthStatusFacet(value)) {
      filterValue = this.get('healthStatusCategories').findProperty('healthStatus', value).filterValue;
    }
    return filterValue;
  }
});

});

require.register("views/main/host/configs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');

App.MainHostConfigsView = Em.View.extend({
  templateName: require('templates/main/host/configs'),

  content: Em.computed.alias('App.router.mainHostDetailsController.content'),

  isConfigAvailable: Em.computed.bool('content.hostComponents.length')

});

});

require.register("views/main/host/configs_service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainHostServiceConfigsView = Em.View.extend({

  templateName: require('templates/main/host/configs_service'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  /**
   * @type {boolean}
   */
  isConfigsEditable: false,

  content: Em.computed.alias('App.router.mainHostDetailsController.content')

});

});

require.register("views/main/host/configs_service_menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var misc = require('utils/misc');

App.MainHostServiceMenuView = Em.CollectionView.extend({
  content: function () {
    var host = this.get('host');
    var hostComponents = host.get('hostComponents');
    var services = Em.A([]);
    if (hostComponents) {
      hostComponents.forEach(function (hc) {
        var service = hc.get('service');
        if (service) {
          var serviceName = service.get('serviceName');
          if (!App.get('services.noConfigTypes').contains(serviceName)) {
            if (!services.findProperty('serviceName', serviceName)) {
              services.push(service);
            }
          }
        }
      });
    }
    var stackServices = App.StackService.find().mapProperty('serviceName');
    return misc.sortByOrder(stackServices, services);
  }.property('host'),

  host: Em.computed.alias('App.router.mainHostDetailsController.content'),

  selectedService: null,

  showHostService: function showHostService(event) {
    var service = event.contexts[0];
    if (!Em.isNone(service)) {
      this.set('selectedService', service);
      var context = service;
      context.host = this.get('host');
      this.get('controller').connectOutlet('service_config_outlet', 'mainHostServiceConfigs', context);
    }
  },

  didInsertElement: function didInsertElement() {
    var event = {
      contexts: [this.get('content').objectAt(0)]
    };
    this.showHostService(event);
  },

  activeServiceId: null,

  tagName: 'ul',
  classNames: ["nav", "nav-list", "nav-services"],

  itemViewClass: Em.View.extend({
    classNameBindings: ["active", "clients"],
    active: function () {
      return this.get('content.serviceName') == this.get('parentView.selectedService.serviceName') ? 'active' : '';
    }.property('parentView.selectedService.serviceName'),
    template: Ember.Handlebars.compile('<a href="#" {{action showHostService view.content target="view.parentView"}} >{{view.content.displayName}}</a>')
  })
});

});

require.register("views/main/host/details", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostDetailsView = Em.View.extend({
  templateName: require('templates/main/host/details'),

  /**
   * flag identify whether current host exist and loaded to model
   */
  isLoaded: false,

  content: Em.computed.alias('App.router.mainHostDetailsController.content'),

  clients: Em.computed.filterBy('content.hostComponents', 'isClient', true),

  clientsWithConfigs: function () {
    return this.get('content.hostComponents').filterProperty('isClient').filter(function (client) {
      return !App.get('services.noConfigTypes').contains(client.get('service.serviceName'));
    });
  }.property('content.hostComponents.@each'),

  hasManyClientsWithConfigs: Em.computed.gt('clientsWithConfigs.length', 1),

  maintenance: function () {
    var result = [];
    if (App.isAuthorized("SERVICE.START_STOP")) {
      result = result.concat([{
        action: 'startAllComponents',
        liClass: this.get('controller.content.isNotHeartBeating') ? 'disabled' : 'enabled',
        cssClass: 'glyphicon glyphicon-play',
        label: this.t('hosts.host.details.startAllComponents')
      }, {
        action: 'stopAllComponents',
        liClass: this.get('controller.content.isNotHeartBeating') ? 'disabled' : 'enabled',
        cssClass: 'glyphicon glyphicon-stop',
        label: this.t('hosts.host.details.stopAllComponents')
      }, {
        action: 'restartAllComponents',
        liClass: this.get('controller.content.isNotHeartBeating') ? 'disabled' : 'enabled',
        cssClass: 'glyphicon glyphicon-repeat',
        label: this.t('hosts.host.details.restartAllComponents')
      }]);
    }
    if (App.isAuthorized("HOST.TOGGLE_MAINTENANCE")) {
      result.push({
        action: 'setRackId',
        liClass: '',
        cssClass: 'glyphicon glyphicon-cog',
        label: this.t('hosts.host.details.setRackId')
      });
      result.push({
        action: 'onOffPassiveModeForHost',
        liClass: '',
        cssClass: 'icon-medkit',
        active: this.get('controller.content.isActive'),
        label: this.t('passiveState.turn' + (this.get('controller.content.isActive') ? "On" : "Off"))
      });
    }
    if (App.get('isKerberosEnabled') && App.get('supports.regenerateKeytabsOnSingleHost')) {
      var actionMap = App.HostComponentActionMap.getMap(this);
      result.push(actionMap.REGENERATE_KEYTAB_FILE_OPERATIONS);
    }
    if (App.isAuthorized("HOST.ADD_DELETE_HOSTS")) {
      result.push({
        action: 'deleteHost',
        liClass: '',
        cssClass: 'glyphicon glyphicon-remove',
        label: this.t('hosts.host.details.deleteHost')
      });
    }
    result.push({
      action: 'checkHost',
      liClass: '',
      cssClass: 'glyphicon glyphicon-check',
      label: this.t('host.host.details.checkHost')
    });
    return result;
  }.property('controller.content', 'controller.content.isActive', 'controller.content.isNotHeartBeating'),

  didInsertElement: function didInsertElement() {
    var self = this;
    var host = self.get('content');

    this.set('isLoaded', App.Host.find(host.get('id')).get('isLoaded'));
    App.router.get('updateController').updateHost(function () {
      self.set('isLoaded', true);
      App.tooltip($("[rel='HealthTooltip']"));
      if (!host.get('isLoaded')) {
        //if host is not existed then route to list of hosts
        App.router.transitionTo('main.hosts.index');
      }
    });
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='HealthTooltip']").tooltip('destroy');
  }

});

});

require.register("views/main/host/details/host_component_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var uiEffects = require('utils/ui_effects');

App.HostComponentView = Em.View.extend({

  templateName: require('templates/main/host/details/host_component'),
  tagName: 'tr',

  /**
   * @type {App.HostComponent}
   */
  content: null,

  /**
   * @type {Array}
   * @const
   */
  excludedMasterCommands: ['DECOMMISSION', 'RECOMMISSION'],

  /**
   * @type {string}
   * @const
   */
  upgradeFailedMessage: Em.I18n.t('hosts.host.details.upgradeFailed'),

  /**
   * @type {String}
   */
  workStatus: Em.computed.alias('content.workStatus'),

  /**
   * Return host component text status
   * @type {String}
   */
  componentTextStatus: Em.computed.alias('content.componentTextStatus'),

  /**
   * @type {string}
   */
  typeDisplay: function () {
    if (this.get('content.isMaster')) {
      return Em.I18n.t('common.master');
    } else if (this.get('content.isSlave')) {
      return Em.I18n.t('common.slave');
    } else if (this.get('content.isClient')) {
      return Em.I18n.t('common.client');
    }
  }.property('content'),

  /**
   * CSS-class for host component status
   * @type {String}
   */
  statusClass: function () {
    //Class when install failed
    if (this.get('workStatus') === App.HostComponentStatus.install_failed) {
      return 'health-status-color-red glyphicon glyphicon-cog';
    }

    //Class when installing
    if (this.get('workStatus') === App.HostComponentStatus.installing) {
      return 'health-status-color-blue glyphicon glyphicon-cog';
    }

    // Client can't be started, it has only INSTALLED as a working state, so show working/green icon
    if (this.get('workStatus') === App.HostComponentStatus.stopped && this.get('content.isClient')) {
      return 'health-status-started';
    }

    //For all other cases
    return 'health-status-' + App.HostComponentStatus.getKeyName(this.get('workStatus'));
  }.property('workStatus'),

  /**
   * CSS-icon-class for host component status
   * @type {String}
   */
  statusIconClass: function () {
    return Em.getWithDefault({
      'health-status-started': App.healthIconClassGreen,
      'health-status-starting': App.healthIconClassGreen,
      'health-status-installed': App.healthIconClassRed,
      'health-status-stopping': App.healthIconClassRed,
      'health-status-unknown': App.healthIconClassYellow,
      'health-status-DEAD-ORANGE': App.healthIconClassOrange
    }, this.get('statusClass'), '');
  }.property('statusClass'),

  /**
   * CSS-class for disabling drop-down menu with list of host component actions
   * Disabled if host's <code>healthClass</code> is health-status-DEAD-YELLOW (lost heartbeat)
   * Disabled if component's action list is empty
   * @type {boolean}
   */
  isDisabled: function () {
    return this.get('parentView.content.healthClass') === "health-status-DEAD-YELLOW" || this.get('noActionAvailable') === 'hidden' && this.get('isRestartComponentDisabled') || App.router.get('wizardWatcherController.isNonWizardUser');
  }.property('parentView.content.healthClass', 'noActionAvailable', 'isRestartComponentDisabled', 'App.router.wizardWatcherController.isNonWizardUser'),

  /**
   * For Upgrade failed state
   * @type {bool}
   */
  isUpgradeFailed: Em.computed.equal('workStatus', App.HostComponentStatus.upgrade_failed),

  /**
   * For Install failed state
   * @type {bool}
   */
  isInstallFailed: Em.computed.equal('workStatus', App.HostComponentStatus.install_failed),

  /**
   * For Started and Starting states
   * @type {bool}
   */
  isStart: Em.computed.existsIn('workStatus', [App.HostComponentStatus.started, App.HostComponentStatus.starting]),

  /**
   * For Installed state
   * @type {bool}
   */
  isStop: Em.computed.equal('workStatus', App.HostComponentStatus.stopped),

  /**
   * For Installing state
   * @type {bool}
   */
  isInstalling: Em.computed.equal('workStatus', App.HostComponentStatus.installing),

  /**
   * For Init state
   * @type {bool}
   */
  isInit: Em.computed.equal('workStatus', App.HostComponentStatus.init),

  /**
   * For Stopping or Starting states
   * @type {bool}
   */
  isInProgress: Em.computed.existsIn('workStatus', [App.HostComponentStatus.stopping, App.HostComponentStatus.starting]),

  withoutActions: Em.computed.existsIn('workStatus', [App.HostComponentStatus.starting, App.HostComponentStatus.stopping, App.HostComponentStatus.unknown, App.HostComponentStatus.disabled]),

  /**
   * No action available while component is starting/stopping/unknown
   * @type {String}
   */
  noActionAvailable: Em.computed.ifThenElse('withoutActions', 'hidden', ''),

  /**
   * For OFF <code>passiveState</code> of host component
   * @type {bool}
   */
  isActive: Em.computed.equal('content.passiveState', 'OFF'),

  /**
   *  Tooltip message for switch maintenance mode option
   *  @type {String}
   */
  maintenanceTooltip: function () {
    switch (this.get('content.passiveState')) {
      case 'IMPLIED_FROM_SERVICE':
        return Em.I18n.t('passiveState.disabled.impliedFromHighLevel').format(this.get('content.displayName'), this.get('content.service.serviceName'));
      case 'IMPLIED_FROM_HOST':
        return Em.I18n.t('passiveState.disabled.impliedFromHighLevel').format(this.get('content.displayName'), this.get('content.host.hostName'));
      case 'IMPLIED_FROM_SERVICE_AND_HOST':
        return Em.I18n.t('passiveState.disabled.impliedFromServiceAndHost').format(this.get('content.displayName'), this.get('content.service.serviceName'), this.get('content.host.hostName'));
      default:
        return '';
    }
  }.property('content.passiveState'),

  /**
   * Shows whether we need to show Delete button
   * @type {bool}
   */
  isDeletableComponent: Em.computed.existsInByKey('content.componentName', 'App.components.deletable'),

  /**
   * Host component with some <code>workStatus</code> can't be moved (so, disable such action in the dropdown list)
   * @type {boolean}
   */
  isMoveComponentDisabled: function () {
    return App.get('allHostNames').length === App.HostComponent.find().filterProperty('componentName', this.get('content.componentName')).mapProperty('hostName').length;
  }.property('content.componentName', 'App.allHostNames'),

  /**
   * Host component with some <code>workStatus</code> can't be deleted (so, disable such action in the dropdown list)
   * @type {bool}
   */
  isDeleteComponentDisabled: function () {
    var stackComponentCount = App.StackServiceComponent.find(this.get('content.componentName')).get('minToInstall');
    var installedCount = App.HostComponent.getCount(this.get('content.componentName'), 'totalCount');
    if (this.get('content.componentName') === 'MYSQL_SERVER' && this.get('content.serviceDisplayName') === 'Hive') {
      var db_type = App.db.getConfigs().findProperty('type', 'hive-env').properties['hive_database'];
      var status = [App.HostComponentStatus.stopped, App.HostComponentStatus.unknown, App.HostComponentStatus.install_failed, App.HostComponentStatus.upgrade_failed, App.HostComponentStatus.init].contains(this.get('workStatus'));
      if (db_type.indexOf('Existing') > -1 && status) return false;else return true;
    }
    if (this.get('content.componentName') === 'JOURNALNODE') {
      var count_JN = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').get('length');
      return count_JN <= 3; // TODO get 3 from stack
    }
    return installedCount <= stackComponentCount || ![App.HostComponentStatus.stopped, App.HostComponentStatus.unknown, App.HostComponentStatus.install_failed, App.HostComponentStatus.upgrade_failed, App.HostComponentStatus.init].contains(this.get('workStatus'));
  }.property('workStatus'),

  /**
   * Gets number of current running components that are applied to the cluster
   * @returns {Number}
   */
  runningComponentCounter: function runningComponentCounter() {
    return App.HostComponent.find().filter(function (component) {
      return component.get('componentName') === this.get('content.componentName') && [App.HostComponentStatus.started, App.HostComponentStatus.starting].contains(component.get('workStatus'));
    }, this).length;
  },

  /**
   * Check if component may be reassinged to another host
   * @type {bool}
   */
  isReassignable: function () {
    return App.get('components.reassignable').contains(this.get('content.componentName')) && App.get('allHostNames.length') > 1;
  }.property('content.componentName'),

  /**
   * Check if component is restartable
   * @type {bool}
   */
  isRestartableComponent: Em.computed.existsInByKey('content.componentName', 'App.components.restartable'),

  /**
   * Host component with some <code>workStatus</code> can't be restarted (so, disable such action in the dropdown list)
   * @type {bool}
   */
  isRestartComponentDisabled: Em.computed.notEqual('workStatus', App.HostComponentStatus.started),

  /**
   * Check if component configs can be refreshed
   * @type {bool}
   */
  isRefreshConfigsAllowed: Em.computed.existsInByKey('content.componentName', 'App.components.refreshConfigsAllowed'),

  willInsertElement: function willInsertElement() {
    //make link to view instance to get decommission state
    this.set('content.view', this);
  },

  didInsertElement: function didInsertElement() {
    App.tooltip($('[rel=componentHealthTooltip]'));
    App.tooltip($('[rel=passiveTooltip]'));
    App.tooltip($('[rel=componentNameTooltip]'));
    if (this.get('isInProgress')) {
      this.doBlinking();
    }
  },

  /**
   * Do blinking for 1 minute
   */
  doBlinking: function doBlinking() {
    var workStatus = this.get('workStatus');
    var self = this;
    var pulsate = [App.HostComponentStatus.starting, App.HostComponentStatus.stopping, App.HostComponentStatus.installing].contains(workStatus);
    if (pulsate && !self.get('isBlinking')) {
      self.set('isBlinking', true);
      uiEffects.pulsate(self.$('.components-health'), 1000, function () {
        self.set('isBlinking', false);
        self.doBlinking();
      });
    }
  },

  /**
   * Start blinking when host component is starting/stopping
   */
  startBlinking: function () {
    this.$('.components-health').stop(true, true);
    this.$('.components-health').css({ opacity: 1.0 });
    this.doBlinking();
  }.observes('workStatus'),

  /**
   * Get custom commands for slave components
   */
  customCommands: function () {
    var customCommands;
    var hostComponent = this.get('content');
    var component = App.StackServiceComponent.find(hostComponent.get('componentName'));

    customCommands = this.getCustomCommands(component, hostComponent, component.get('isSlave'));

    return customCommands;
  }.property('content', 'workStatus'),

  /**
   * Get a list of custom commands
   *
   * @param component
   * @param hostComponent
   * @param isSlave
   * @returns {Array}
   */
  getCustomCommands: function getCustomCommands(component, hostComponent, isSlave) {
    isSlave = isSlave || false;

    if (!component || !hostComponent) {
      return [];
    }

    var self = this;
    var commands = component.get('customCommands');
    var customCommands = [];

    commands.forEach(function (command) {
      if (!isSlave && !self.meetsCustomCommandReq(component, command)) {
        return;
      }

      var commandMap = App.HostComponentActionMap.getMap(self)[command];
      // push command if either there is no map or map is not instructing to hide command from this view
      if (!commandMap || !commandMap.hideFromComponentView) {
        customCommands.push({
          label: self.getCustomCommandLabel(command),
          service: component.get('serviceName'),
          hosts: hostComponent.get('hostName'),
          context: !!commandMap && !!commandMap.context ? commandMap.context : null,
          component: component.get('componentName'),
          command: command,
          disabled: !!commandMap ? !!commandMap.disabled : false
        });
      }
    });

    return customCommands;
  },

  /**
   * Get the Label of the custom command
   *
   * @param command
   * @returns {String}
   */
  getCustomCommandLabel: function getCustomCommandLabel(command) {
    if (command in App.HostComponentActionMap.getMap(this) && App.HostComponentActionMap.getMap(this)[command].label) return App.HostComponentActionMap.getMap(this)[command].label;

    return Em.I18n.t('services.service.actions.run.executeCustomCommand.menu').format(App.format.normalizeNameBySeparators(command, ["_", "-", " "]));
  },

  /**
   * The custom command meets the requirements to be active on master
   *
   * @param component
   * @param command
   *
   * @return {Boolean}
   */
  meetsCustomCommandReq: function meetsCustomCommandReq(component, command) {
    var excludedMasterCommands = this.get('excludedMasterCommands');

    if (excludedMasterCommands.indexOf(command) >= 0) {
      return false;
    }

    if (component.get('cardinality') !== '1') {
      if (!this.get('isStart')) {
        if (App.HostComponent.getCount(this.get('content.componentName'), 'totalCount') > 1) {
          if (this.runningComponentCounter()) {
            return false;
          }
        } else {
          return false;
        }
      }
    }

    return true;
  },

  /**
   * @type {Array}
   */
  clientCustomCommands: function () {
    var componentName = this.get('content.componentName');

    if (componentName === 'KERBEROS_CLIENT') {
      return [];
    }

    return App.StackServiceComponent.find(componentName).get('customCommands').map(function (command) {
      return {
        label: Em.I18n.t('services.service.actions.run.executeCustomCommand.menu').format(command),
        command: command
      };
    });
  }.property('controller'),

  installClient: function installClient() {
    this.get('controller').installClients([this.get('content')]);
  }

});

});

require.register("views/main/host/details/host_component_views/datanode_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DataNodeComponentView = App.HostComponentView.extend(App.Decommissionable, {

  componentForCheckDecommission: 'NAMENODE',

  /**
   * Get component decommission status from server
   * @returns {$.ajax}
   */
  getDNDecommissionStatus: function getDNDecommissionStatus() {
    // always get datanode decommission status from active namenodes (if NN HA or federation enabled)
    var hdfs = App.HDFSService.find().objectAt(0),
        activeNNHostNames = !hdfs.get('snameNode') && hdfs.get('activeNameNodes.length') ? hdfs.get('activeNameNodes').mapProperty('hostName').join(',') : hdfs.get('nameNode.hostName');
    return App.ajax.send({
      name: 'host.host_component.decommission_status_datanode',
      sender: this,
      data: {
        hostNames: activeNNHostNames,
        componentName: this.get('componentForCheckDecommission')
      },
      success: 'getDNDecommissionStatusSuccessCallback',
      error: 'getDNDecommissionStatusErrorCallback'
    });
  },

  /**
   * Set received value or null to <code>decommissionedStatusObject</code>
   * @param {Object} response
   * @returns {Object|null}
   */
  getDNDecommissionStatusSuccessCallback: function getDNDecommissionStatusSuccessCallback(response) {
    if (response && response.items) {
      var statusObjects = response.items.mapProperty('metrics.dfs.namenode');
      this.computeStatus(statusObjects);
    }
  },

  /**
   * Set null to <code>decommissionedStatusObject</code> if server returns error
   * @returns {null}
   */
  getDNDecommissionStatusErrorCallback: function getDNDecommissionStatusErrorCallback() {
    this.set('decommissionedStatusObject', null);
    return null;
  },

  /**
   * load Recommission/Decommission status from adminState of each live node
   */
  loadComponentDecommissionStatus: function loadComponentDecommissionStatus() {
    return this.getDNDecommissionStatus();
  },

  setDesiredAdminState: function setDesiredAdminState(desired_admin_state) {
    this.setStatusAs(desired_admin_state);
  },

  /**
   * compute and set decommission state by namenode metrics
   * @param metricObjects
   */
  computeStatus: function computeStatus(metricObjects) {
    var hostName = this.get('content.hostName');
    var inServiceCount = 0,
        decommissioningCount = 0,
        decommissionedCount = 0;
    metricObjects.forEach(function (curObj) {
      if (curObj) {
        var liveNodesJson = App.parseJSON(curObj.LiveNodes);
        for (var hostPort in liveNodesJson) {
          if (hostPort.indexOf(hostName) === 0) {
            switch (liveNodesJson[hostPort].adminState) {
              case 'In Service':
                inServiceCount++;
                break;
              case 'Decommission In Progress':
                decommissioningCount++;
                break;
              case 'Decommissioned':
                decommissionedCount++;
                break;
            }
            return;
          }
        }
      }
    });
    if (decommissioningCount) {
      this.setStatusAs('DECOMMISSIONING');
    } else if (inServiceCount && !decommissionedCount) {
      this.setStatusAs('INSERVICE');
    } else if (!inServiceCount && decommissionedCount) {
      this.setStatusAs('DECOMMISSIONED');
    } else {
      // if namenodes are down, get desired_admin_state to decide if the user had issued a decommission
      this.getDesiredAdminState();
    }
  }
});

});

require.register("views/main/host/details/host_component_views/nodemanager_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.NodeManagerComponentView = App.HostComponentView.extend(App.Decommissionable, {

  componentForCheckDecommission: 'RESOURCEMANAGER',

  setDesiredAdminState: function setDesiredAdminState(desiredAdminState) {
    var self = this;
    switch (desiredAdminState) {
      case "INSERVICE":
        // can be decommissioned if already started
        this.setStatusAs(desiredAdminState);
        break;
      case "DECOMMISSIONED":
        this.getDecommissionStatus();
        break;
    }
  },

  /**
   * set decommission status according to component status and resource manager metrics
   */
  setDecommissionStatus: function setDecommissionStatus(curObj) {
    var rmComponent = this.get('content.service.hostComponents').findProperty('componentName', this.get('componentForCheckDecommission')),
        hostName = this.get('content.hostName');

    rmComponent.set('workStatus', curObj.component_state);
    if (curObj.rm_metrics) {
      // Update RESOURCEMANAGER status
      var nodeManagersArray = App.parseJSON(curObj.rm_metrics.cluster.nodeManagers);
      if (nodeManagersArray.findProperty('HostName', hostName)) {
        // decommisioning ..
        this.setStatusAs('DECOMMISSIONING');
      } else {
        // decommissioned ..
        this.setStatusAs('DECOMMISSIONED');
      }
    } else {
      // in this case ResourceManager not started. Set status to Decommissioned
      this.setStatusAs('DECOMMISSIONED');
    }
  }
});

});

require.register("views/main/host/details/host_component_views/regionserver_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.RegionServerComponentView = App.HostComponentView.extend(App.Decommissionable, {

  componentForCheckDecommission: 'HBASE_MASTER',

  setDesiredAdminState: function setDesiredAdminState(desiredAdminState) {
    this.getRSDecommissionStatus(desiredAdminState);
  },

  getRSDecommissionStatus: function getRSDecommissionStatus(desiredAdminState) {
    var hostName = App.HBaseService.find('HBASE').get('master.hostName');
    App.ajax.send({
      name: 'host.host_component.decommission_status_regionserver',
      sender: this,
      data: {
        hostName: hostName,
        desiredAdminState: desiredAdminState
      },
      success: 'getRSDecommissionStatusSuccessCallback',
      error: 'getRSDecommissionStatusErrorCallback'
    });
  },

  getRSDecommissionStatusSuccessCallback: function getRSDecommissionStatusSuccessCallback(data, opt, params) {
    var desiredAdminState = params.desiredAdminState,
        hostName = this.get('content.hostName');

    if (data) {
      var liveRSHostsMetrics = Em.get(data, 'items.0.metrics.hbase.master.liveRegionServersHosts'),
          deadRSHostsMetrics = Em.get(data, 'items.0.metrics.hbase.master.deadRegionServersHosts'),
          liveRSHosts = this.parseRegionServersHosts(liveRSHostsMetrics),
          deadRSHosts = this.parseRegionServersHosts(deadRSHostsMetrics),
          isLiveRS = liveRSHosts.contains(hostName),
          isDeadRS = deadRSHosts.contains(hostName),
          isInServiceDesired = desiredAdminState === 'INSERVICE',
          isDecommissionedDesired = desiredAdminState === 'DECOMMISSIONED';
      if (liveRSHosts.length + deadRSHosts.length === 0 || isInServiceDesired && isLiveRS || isDecommissionedDesired && isDeadRS) {
        this.setDesiredAdminStateDefault(desiredAdminState);
      } else if (isInServiceDesired) {
        this.setStatusAs('RS_DECOMMISSIONED');
      } else if (isDecommissionedDesired) {
        this.setStatusAs('INSERVICE');
      }
    } else {
      this.setDesiredAdminStateDefault(desiredAdminState);
    }
  },

  getRSDecommissionStatusErrorCallback: function getRSDecommissionStatusErrorCallback(request, ajaxOptions, error, opt, params) {
    this.setDesiredAdminStateDefault(params.desiredAdminState);
  },

  parseRegionServersHosts: function parseRegionServersHosts(str) {
    var items = str ? str.split(';') : [],
        hosts = items.map(function (item) {
      return item.split(',')[0];
    });
    return hosts;
  },

  setDesiredAdminStateDefault: function setDesiredAdminStateDefault(desiredAdminState) {
    switch (desiredAdminState) {
      case 'INSERVICE':
        this.setStatusAs(desiredAdminState);
        break;
      case 'DECOMMISSIONED':
        this.setStatusAs('RS_DECOMMISSIONED');
        break;
    }
  }
});

});

require.register("views/main/host/details/host_component_views/tasktracker_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.TaskTrackerComponentView = App.HostComponentView.extend(App.Decommissionable, {

  componentForCheckDecommission: 'JOBTRACKER',

  setDesiredAdminState: function setDesiredAdminState(desiredAdminState) {
    switch (desiredAdminState) {
      case "INSERVICE":
        // can be decommissioned if already started
        this.setStatusAs('INSERVICE');
        break;
      case "DECOMMISSIONED":
        this.getDecommissionStatus();
        break;
    }
  },

  setDecommissionStatus: function setDecommissionStatus(curObj) {
    var hostName = this.get('content.hostName');

    if (curObj) {
      var aliveNodesArray = App.parseJSON(curObj.AliveNodes);
      if (!Em.isNone(aliveNodesArray)) {
        if (aliveNodesArray.findProperty('hostname', hostName)) {
          //decommissioning ..
          this.setStatusAs("DECOMMISSIONING");
        } else {
          //decommissioned
          this.setStatusAs("DECOMMISSIONED");
        }
      }
    }
  }
});

});

require.register("views/main/host/host_alerts_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view'),
    sort = require('views/common/sort_view');

App.MainHostAlertsView = App.TableView.extend({
  templateName: require('templates/main/host/host_alerts'),

  content: function () {
    var criticalAlerts = [],
        warningAlerts = [],
        okAlerts = [],
        otherAlerts = [];
    var content = this.get('controller.content');
    if (content) {
      content.forEach(function (alert) {
        switch (alert.get('state')) {
          case 'CRITICAL':
            criticalAlerts.push(alert);
            break;
          case 'WARNING':
            warningAlerts.push(alert);
            break;
          case 'OK':
            okAlerts.push(alert);
            break;
          default:
            otherAlerts.push(alert);
        }
      });
      return [].concat(criticalAlerts, warningAlerts, okAlerts, otherAlerts);
    }
    return [];
  }.property('controller.content.[]'),

  willInsertElement: function willInsertElement() {
    var hostName = this.get('parentView.controller.content.hostName');
    App.router.get('mainAlertInstancesController').loadAlertInstancesByHost(hostName);
    App.router.set('mainAlertInstancesController.isUpdating', true);

    // on load alters should be sorted by state
    var controllerName = this.get('controller.name'),
        savedSortConditions = App.db.getSortingStatuses(controllerName) || [];
    if (savedSortConditions.everyProperty('status', 'sorting')) {
      savedSortConditions.push({
        name: "state",
        status: "sorting_asc"
      });
      App.db.setSortingStatuses(controllerName, savedSortConditions);
    }

    this._super();
  },

  didInsertElement: function didInsertElement() {
    this.tooltipsUpdater();
  },

  /**
   * @type {number}
   */
  totalCount: Em.computed.alias('content.length'),

  colPropAssoc: ['', 'serviceName', 'label', 'latestTimestamp', 'state', 'text'],

  sortView: sort.wrapperView,

  /**
   * Sorting header for <label>alertDefinition.label</label>
   * @type {Em.View}
   */
  nameSort: sort.fieldView.extend({
    column: 2,
    name: 'label',
    displayName: Em.I18n.t('alerts.definition.name')
  }),

  /**
   * Sorting header for <label>alertDefinition.status</label>
   * @type {Em.View}
   */
  statusSort: sort.fieldView.extend({
    column: 4,
    name: 'state',
    displayName: Em.I18n.t('common.status'),
    type: 'select'
  }),

  /**
   * Sorting header for <label>alertDefinition.service.serviceName</label>
   * @type {Em.View}
   */
  serviceSort: sort.fieldView.extend({
    column: 1,
    name: 'serviceName',
    displayName: Em.I18n.t('common.service'),
    type: 'string'
  }),

  /**
   * Sorting header for <label>alertDefinition.label</label>
   * @type {Em.View}
   */
  textSort: sort.fieldView.extend({
    column: 5,
    name: 'text',
    displayName: Em.I18n.t('alerts.table.header.check.response')
  }),

  /**
   * Filtering header for <label>alertDefinition.label</label>
   * @type {Em.View}
   */
  nameFilterView: filters.createTextView({
    column: 2,
    fieldType: 'filter-input-width',
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'string');
    }
  }),

  /**
   * Filtering header for <label>alertDefinition.status</label>
   * @type {Em.View}
   */
  stateFilterView: filters.createSelectView({
    column: 4,
    fieldType: 'filter-input-width',
    content: [{
      value: '',
      label: Em.I18n.t('common.all')
    }, {
      value: 'OK',
      label: 'OK'
    }, {
      value: 'WARNING',
      label: 'WARNING'
    }, {
      value: 'CRITICAL',
      label: 'CRITICAL'
    }, {
      value: 'UNKNOWN',
      label: 'UNKNOWN'
    }],
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * Filtering header for <label>alertDefinition.service.serviceName</label>
   * @type {Em.View}
   */
  serviceFilterView: filters.createSelectView({
    column: 1,
    fieldType: 'filter-input-width',
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(App.Service.find().map(function (service) {
        return {
          value: service.get('serviceName'),
          label: service.get('displayName')
        };
      })).concat({
        value: 'AMBARI',
        label: Em.I18n.t('app.name')
      });
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * Filtering header for <label>alertDefinition.service.serviceName</label>
   * @type {Em.View}
   */
  textView: filters.createTextView({
    column: 5,
    fieldType: 'filter-input-width',
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'string');
    }
  }),

  /**
   * Filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('alerts.filters.filteredAlertsInfo', 'filteredCount', 'totalCount'),

  /**
   * Determines how display "back"-link - as link or text
   * @type {string}
   */
  paginationLeftClass: function () {
    return this.get('startIndex') > 1 ? 'paginate_previous' : 'paginate_disabled_previous';
  }.property('startIndex', 'filteredCount'),

  /**
   * Determines how display "next"-link - as link or text
   * @type {string}
   */
  paginationRightClass: function () {
    return this.get('endIndex') < this.get('filteredCount') ? 'paginate_next' : 'paginate_disabled_next';
  }.property('endIndex', 'filteredCount'),

  /**
   * Show previous-page if user not in the first page
   * @method previousPage
   */
  previousPage: function previousPage() {
    if (this.get('paginationLeftClass') === 'paginate_previous') {
      this._super();
    }
  },

  /**
   * Show next-page if user not in the last page
   * @method nextPage
   */
  nextPage: function nextPage() {
    if (this.get('paginationRightClass') === 'paginate_next') {
      this._super();
    }
  },

  /**
   * Update tooltips when <code>pageContent</code> is changed
   * @method tooltipsUpdater
   */
  tooltipsUpdater: function () {
    Em.run.once(this, this.tooltipsUpdaterOnce);
  }.observes('pageContent.[]'),

  tooltipsUpdaterOnce: function tooltipsUpdaterOnce() {
    var self = this;
    Em.run.next(this, function () {
      App.tooltip(self.$('.timeago, .alert-text'));
    });
  },

  /**
   * Run <code>clearFilter</code> in the each child filterView
   */
  clearFilters: function clearFilters() {
    this.set('filterConditions', []);
    this.get('childViews').forEach(function (childView) {
      Em.tryInvoke(childView, 'clearFilter');
    });
  },

  /**
   * Tooltips should be removed if some filter is applied or cleared
   *
   * @method clearTooltips
   */
  clearTooltips: function () {
    var $elements = this.$('.timeago, .alert-text');
    if ($elements) {
      $elements.tooltip('destroy');
    }
  }.observes('filteredCount'),

  willDestroyElement: function willDestroyElement() {
    this.clearTooltips();
  }

});

});

require.register("views/main/host/hosts_table_menu_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var O = Em.Object;

/**
 * @class HostTableMenuView
 */
App.HostTableMenuView = Em.View.extend({

  classNames: ['btn-group', 'pull-right', 'bulk-menu'],

  templateName: require('templates/main/host/bulk_operation_menu'),

  controllerBinding: 'App.router.bulkOperationsController',

  menuItems: function () {
    return {
      s: { label: Em.I18n.t('hosts.table.menu.l1.selectedHosts') },
      f: { label: Em.I18n.t('hosts.table.menu.l1.filteredHosts') },
      a: { label: Em.I18n.t('hosts.table.menu.l1.allHosts') }
    };
  }.property('App.router.clusterController.isLoaded'),

  components: function () {
    var serviceNames = App.Service.find().mapProperty('serviceName');
    var menuItems = this.getBulkMenuItemsPerServiceComponent();
    return menuItems.filter(function (item) {
      return serviceNames.contains(item.serviceName);
    });
  }.property(),

  getBulkMenuItemsPerServiceComponent: function getBulkMenuItemsPerServiceComponent() {
    var menuItems = [];
    App.StackServiceComponent.find().forEach(function (stackComponent) {
      if (stackComponent.get('hasBulkCommandsDefinition')) {
        var menuItem = O.create({
          serviceName: stackComponent.get('serviceName'),
          componentName: stackComponent.get('componentName'),
          masterComponentName: stackComponent.get('bulkCommandsMasterComponentName'),
          componentNameFormatted: stackComponent.get('bulkCommandsDisplayName')
        });
        menuItems.push(menuItem);
      }
    }, this);
    return menuItems;
  },

  /**
   * slaveItemView build second-level menu
   * for slave component
   */

  slaveItemView: Em.View.extend({

    tagName: 'li',
    classNames: ['dropdown-submenu', 'submenu-left'],

    /**
     * Get third-level menu items ingo for slave components
     * operationData format:
     * <code>
     *  {
     *    action: 'STARTED|INSTALLED|RESTART|DECOMMISSION|DECOMMISSION_OFF', // action for selected host components
     *    message: 'some text', // just text to BG popup
     *    componentName: 'DATANODE|NODEMANAGER...', //component name that should be processed
     *    realComponentName: 'DATANODE|NODEMANAGER...', // used only for decommission(_off) actions
     *    serviceName: 'HDFS|YARN|HBASE...', // service name of the processed component
     *    componentNameFormatted: 'DataNodes|NodeManagers...' // "user-friendly" string with component name (used in BG popup)
     *  }
     *  </code>
     *
     * @returns {Array}
     */

    operationsInfo: function () {
      var content = this.get('content');
      var menuItems = Em.A();
      if (App.isAuthorized("SERVICE.START_STOP")) {
        menuItems.pushObjects([O.create({
          label: Em.I18n.t('common.start'),
          operationData: O.create({
            action: App.HostComponentStatus.started,
            message: Em.I18n.t('common.start'),
            componentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        }), O.create({
          label: Em.I18n.t('common.stop'),
          operationData: O.create({
            action: App.HostComponentStatus.stopped,
            message: Em.I18n.t('common.stop'),
            componentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        }), O.create({
          label: Em.I18n.t('common.restart'),
          operationData: O.create({
            action: 'RESTART',
            message: Em.I18n.t('common.restart'),
            componentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        })]);
      }
      if (App.isAuthorized("HOST.ADD_DELETE_COMPONENTS")) {
        menuItems.pushObjects([O.create({
          label: Em.I18n.t('common.add'),
          operationData: O.create({
            action: 'ADD',
            message: Em.I18n.t('common.add'),
            componentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        }), O.create({
          label: Em.I18n.t('common.delete'),
          delete: true,
          operationData: O.create({
            action: 'DELETE',
            message: Em.I18n.t('common.delete'),
            componentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        })]);
      }
      if (App.isAuthorized("SERVICE.DECOMMISSION_RECOMMISSION") && App.get('components.decommissionAllowed').contains(content.componentName)) {
        menuItems.pushObjects([O.create({
          label: Em.I18n.t('common.decommission'),
          decommission: true,
          operationData: O.create({
            action: 'DECOMMISSION',
            message: Em.I18n.t('common.decommission'),
            componentName: content.masterComponentName,
            realComponentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        }), O.create({
          label: Em.I18n.t('common.recommission'),
          decommission: true,
          operationData: O.create({
            action: 'DECOMMISSION_OFF',
            message: Em.I18n.t('common.recommission'),
            componentName: content.masterComponentName,
            realComponentName: content.componentName,
            serviceName: content.serviceName,
            componentNameFormatted: content.componentNameFormatted
          })
        })]);
      }
      return menuItems;
    }.property("content"),

    /**
     * commonOperationView is used for third-level menu items
     * for simple operations ('START','STOP','RESTART')
     */
    commonOperationView: Em.View.extend({
      tagName: 'li',

      click: function click() {
        this.get('controller').bulkOperationConfirm(this.get('content'), this.get('selection'));
      }
    }),

    /**
     * advancedOperationView is used for third level menu item
     * for advanced operations ('RECOMMISSION','DECOMMISSION')
     */
    advancedOperationView: Em.View.extend({
      tagName: 'li',
      rel: 'menuTooltip',
      classNameBindings: ['disabledElement'],
      attributeBindings: ['tooltipMsg:data-original-title'],

      service: function () {
        return App.router.get('mainServiceController.content').findProperty('serviceName', this.get('content.serviceName'));
      }.property('App.router.mainServiceController.content.@each', 'content'),

      tooltipMsg: function () {
        var displayName = App.format.role(this.get('content.componentName'), false);
        return this.get('disabledElement') === 'disabled' ? Em.I18n.t('hosts.decommission.tooltip.warning').format(this.get('content.message'), displayName) : '';
      }.property('disabledElement', 'content.componentName'),

      disabledElement: function () {
        return this.get('service.workStatus') === 'STARTED' ? '' : 'disabled';
      }.property('service.workStatus'),

      click: function click() {
        if (this.get('disabledElement') === 'disabled') {
          return;
        }
        this.get('controller').bulkOperationConfirm(this.get('content'), this.get('selection'));
      },

      didInsertElement: function didInsertElement() {
        App.tooltip($(this.get('element')));
      }
    })
  }),

  /**
   * hostItemView build second-level menu
   * for host
   */

  hostItemView: Em.View.extend({

    tagName: 'li',
    classNames: ['dropdown-submenu', 'submenu-left'],

    label: Em.I18n.t('common.hosts'),

    /** Get third-level menu items for Hosts
     * operationData format:
     * <code>
     *  {
     *    action: 'STARTED|INSTALLED|RESTART..', // action for selected hosts (will be applied for each host component in selected hosts)
     *    actionToCheck: 'INSTALLED|STARTED..' // state to filter host components should be processed
     *    message: 'some text', // just text to BG popup
     *  }
     *  </code>
     * @returns {Array}
     */
    operationsInfo: function () {
      var result = [];
      if (App.isAuthorized("SERVICE.START_STOP")) {
        result = result.concat([O.create({
          label: Em.I18n.t('hosts.host.details.startAllComponents'),
          operationData: O.create({
            action: 'STARTED',
            actionToCheck: 'INSTALLED',
            message: Em.I18n.t('hosts.host.details.startAllComponents')
          })
        }), O.create({
          label: Em.I18n.t('hosts.host.details.stopAllComponents'),
          operationData: O.create({
            action: 'INSTALLED',
            actionToCheck: 'STARTED',
            message: Em.I18n.t('hosts.host.details.stopAllComponents')
          })
        }), O.create({
          label: Em.I18n.t('hosts.table.menu.l2.restartAllComponents'),
          operationData: O.create({
            action: 'RESTART',
            message: Em.I18n.t('hosts.table.menu.l2.restartAllComponents')
          })
        }), O.create({
          label: Em.I18n.t('hosts.table.menu.l2.reinstallFailedComponents'),
          operationData: O.create({
            action: 'REINSTALL',
            message: Em.I18n.t('hosts.table.menu.l2.reinstallFailedComponents')
          })
        }), O.create({
          label: Em.I18n.t('hosts.table.menu.l2.refreshConfigsClientComponents'),
          operationData: O.create({
            action: 'CONFIGURE',
            message: Em.I18n.t('hosts.table.menu.l2.refreshConfigsClientComponents')
          })
        })]);
      }

      if (App.isAuthorized("HOST.TOGGLE_MAINTENANCE")) {
        result = result.concat([O.create({
          label: Em.I18n.t('passiveState.turnOn'),
          operationData: O.create({
            state: 'ON',
            action: 'PASSIVE_STATE',
            message: Em.I18n.t('passiveState.turnOnFor').format('hosts')
          })
        }), O.create({
          label: Em.I18n.t('passiveState.turnOff'),
          operationData: O.create({
            state: 'OFF',
            action: 'PASSIVE_STATE',
            message: Em.I18n.t('passiveState.turnOffFor').format('hosts')
          })
        })]);
      }
      result = result.concat(O.create({
        label: Em.I18n.t('hosts.host.details.setRackId'),
        operationData: O.create({
          action: 'SET_RACK_INFO',
          message: Em.I18n.t('hosts.host.details.setRackId').format('hosts')
        })
      }));
      if (App.isAuthorized("HOST.ADD_DELETE_HOSTS")) {
        result = result.concat(O.create({
          label: Em.I18n.t('hosts.host.details.deleteHosts'),
          operationData: O.create({
            action: 'DELETE',
            message: Em.I18n.t('hosts.host.details.deleteHosts')
          })
        }));
      }
      return result;
    }.property(),

    /**
     * commonOperationView is used for third-level menu items
     * for all operations for host
     */
    operationView: Em.View.extend({
      tagName: 'li',

      click: function click() {
        this.get('controller').bulkOperationConfirm(this.get('content'), this.get('selection'));
      }
    })
  })

});

});

require.register("views/main/host/log_metrics", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

/**
 * @typedef {Ember.Object} LogLevelItemObject
 * @property {string} level level name
 * @property {number} counter
 */
/**
 * @typedef {Object} ServiceLogMetricsObject
 * @property {App.Service} service model instance
 * @property {LogLevelItemObject[]} logs
 */
App.MainHostLogMetrics = Em.View.extend({
  templateName: require('templates/main/host/log_metrics'),
  classNames: ['host-log-metrics'],

  /**
   * @type {ServiceLogMetricsObject[]}
   */
  logsData: function () {
    var services = this.get('content.hostComponents').mapProperty('service').uniq();
    var logLevels = ['fatal', 'critical', 'error', 'warning', 'info', 'debug'];
    return services.map(function (service) {
      var levels = logLevels.map(function (level) {
        return Em.Object.create({
          level: level,
          counter: Math.ceil(Math.random() * 10)
        });
      });
      return Em.Object.create({
        service: service,
        logs: levels
      });
    });
  }.property('content'),

  /**
   * @type  {Ember.View} Pie Chart view
   * @extends App.PieChartView
   */
  chartView: App.ChartPieView.extend({
    classNames: ['log-metrics-chart'],
    w: 150,
    h: 150,
    stroke: '#fff',
    strokeWidth: 1,
    levelColors: {
      FATAL: '#B10202',
      CRITICAL: '#E00505',
      ERROR: App.healthStatusRed,
      INFO: App.healthStatusGreen,
      WARNING: App.healthStatusOrange,
      DEBUG: '#1e61f7'
    },
    innerR: 36,
    donut: d3.layout.pie().sort(null).value(function (d) {
      return d.get('counter');
    }),

    prepareChartData: function prepareChartData(content) {
      this.set('data', content.get('logs'));
    },

    didInsertElement: function didInsertElement() {
      this.prepareChartData(this.get('content'));
      this._super();
      this.appendLabels();
      this.formatCenterText();
      this.attachArcEvents();
      this.colorizeArcs();
    },

    attachArcEvents: function attachArcEvents() {
      var self = this;
      this.get('svg').selectAll('.arc').on('mouseover', function (d) {
        self.get('svg').select('g.center-text').select('text').text(d.data.get('level').capitalize() + ": " + d.data.get('counter'));
      }).on('mouseout', function () {
        self.get('svg').select('g.center-text').select('text').text('');
      });
    },

    formatCenterText: function formatCenterText() {
      this.get('svg').append('svg:g').attr('class', 'center-text').attr('render-order', 1).append('svg:text').attr('transform', "translate(0,0)").attr('text-anchor', 'middle').attr('stroke', '#000').attr('stroke-width', 0);
    },

    appendLabels: function appendLabels() {
      var labelArc = d3.svg.arc().outerRadius(this.get('outerR') - 15).innerRadius(this.get('outerR') - 15);
      this.get('svg').selectAll('.arc').append('text').attr('transform', function (d) {
        return "translate(" + labelArc.centroid(d) + ")";
      }).attr('stroke', '#000').attr('stroke-width', 0).attr('font-size', '12px').attr('dy', '.50em').text(function (d) {
        return d.data.get('counter');
      });
    },

    colorizeArcs: function colorizeArcs() {
      var self = this;
      this.get('svg').selectAll('.arc path').attr('fill', function (d) {
        return self.get('levelColors')[d.data.get('level').toUpperCase()];
      });
    }
  }),

  transitionByService: function transitionByService(e) {
    var service = e.context;
    App.router.transitionTo('logs', { query: '?service_name=' + service.get('service.serviceName') });
  }
});

});

require.register("views/main/host/logs_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view');
var sort = require('views/common/sort_view');
var fileUtils = require('utils/file_utils');

App.MainHostLogsView = App.TableView.extend({
  templateName: require('templates/main/host/logs'),

  isVisible: Em.computed.not('App.isClusterUser'),
  classNames: ['logs-tab-content'],

  /**
   * Filtered number of all content number information displayed on the page footer bar
   * @returns {String}
   */
  filteredContentInfo: Em.computed.i18nFormat('tableView.filters.filteredLogsInfo', 'filteredCount', 'totalCount'),

  /**
   * @type {Ember.Object}
   */
  host: Em.computed.alias('App.router.mainHostDetailsController.content'),

  hostLogs: function () {
    return App.HostComponentLog.find().filterProperty('hostName', this.get('host.hostName'));
  }.property('App.HostComponentLog.length'),

  content: function () {
    var self = this,
        linkTailTpl = '/#/logs/serviceLogs;hosts={0};components={2};query=%5B%7B"id":0,"name":"path","label":"Path","value":"{1}","isExclude":false%7D%5D';

    return this.get('hostLogs').map(function (i) {
      return Em.Object.create({
        serviceName: i.get('hostComponent.service.serviceName'),
        serviceDisplayName: i.get('hostComponent.service.displayName'),
        componentName: i.get('hostComponent.componentName'),
        componentDisplayName: i.get('hostComponent.displayName'),
        hostName: self.get('host.hostName'),
        logComponentName: i.get('name'),
        fileNamesObject: i.get('logFileNames').map(function (filePath) {
          return {
            fileName: fileUtils.fileNameFromPath(filePath),
            filePath: filePath,
            linkTail: linkTailTpl.format(encodeURIComponent(i.get('hostName')), encodeURIComponent(filePath), encodeURIComponent(i.get('name')))
          };
        }),
        fileNamesFilterValue: i.get('logFileNames').join(',')
      });
    });
  }.property('hostLogs.length'),

  /**
   * @type {Ember.View}
   */
  sortView: sort.wrapperView,

  serviceNameSort: sort.fieldView.extend({
    column: 1,
    name: 'serviceName',
    displayName: Em.I18n.t('common.service')
  }),

  componentNameSort: sort.fieldView.extend({
    column: 2,
    name: 'componentName',
    displayName: Em.I18n.t('common.component')
  }),

  fileExtensionsSort: sort.fieldView.extend({
    column: 3,
    name: 'extension',
    displayName: Em.I18n.t('common.extension')
  }),

  serviceNameFilterView: filters.createSelectView({
    column: 1,
    fieldType: 'filter-input-width',
    didInsertElement: function didInsertElement() {
      this.setValue(Em.getWithDefault(this, 'controller.serializedQuery.service_name', ''));
      this._super();
    },
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(App.Service.find().mapProperty('serviceName').uniq().map(function (item) {
        return {
          value: item,
          label: App.Service.find().findProperty('serviceName', item).get('displayName')
        };
      }));
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  componentNameFilterView: filters.createSelectView({
    column: 2,
    fieldType: 'filter-input-width',
    didInsertElement: function didInsertElement() {
      this.setValue(Em.getWithDefault(this, 'controller.serializedQuery.component_name', ''));
      this._super();
    },
    content: function () {
      var hostName = this.get('parentView').get('host.hostName'),
          hostComponents = App.HostComponent.find().filterProperty('hostName', hostName),
          componentsMap = hostComponents.map(function (item) {
        return {
          value: item.get('componentName'),
          label: item.get('displayName')
        };
      });
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(componentsMap);
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  fileExtensionsFilter: filters.createSelectView({
    column: 3,
    fieldType: 'filter-input-width',
    type: 'string',
    didInsertElement: function didInsertElement() {
      this.setValue(Em.getWithDefault(this, 'controller.serializedQuery.file_extension', ''));
      this._super();
    },
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(['.out', '.log'].map(function (item) {
        return {
          value: item,
          label: item
        };
      }));
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'file_extension');
    }
  }),

  /**
   * @type {string[]}
   */
  colPropAssoc: function () {
    var ret = [];
    ret[1] = 'serviceName';
    ret[2] = 'componentName';
    ret[3] = 'fileNamesFilterValue';
    return ret;
  }.property(),

  logFileRowView: Em.View.extend({
    tagName: 'tr',

    didInsertElement: function didInsertElement() {
      this._super();
      App.tooltip(this.$('[rel="log-file-name-tooltip"]'));
    },

    willDestroyElement: function willDestroyElement() {
      this.$('[rel="log-file-name-tooltip"]').tooltip('destroy');
    }
  }),

  openLogFile: function openLogFile(e) {
    var content = e.contexts,
        filePath = content[1],
        componentLog = content[0];
    if (e.contexts.length) {
      App.showLogTailPopup(Em.Object.create({
        logComponentName: componentLog.get('logComponentName'),
        hostName: componentLog.get('hostName'),
        filePath: filePath
      }));
    }
  }
});

});

require.register("views/main/host/menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostMenuView = Em.CollectionView.extend({
  tagName: 'ul',
  classNames: ["nav", "nav-tabs", "background-text"],
  host: null,

  content: function () {
    return [Em.Object.create({
      name: 'summary',
      label: Em.I18n.t('common.summary'),
      routing: 'summary',
      id: 'host-details-summary-tab'
    }), Em.Object.create({
      name: 'configs',
      label: Em.I18n.t('common.configs'),
      routing: 'configs',
      id: 'host-details-summary-configs'
    }), Em.Object.create({
      name: 'alerts',
      label: Em.I18n.t('hosts.host.alerts.label'),
      routing: 'alerts',
      badgeText: '0',
      badgeClasses: 'label',
      id: 'host-details-summary-alerts'
    }), Em.Object.create({
      name: 'versions',
      label: Em.I18n.t('hosts.host.menu.stackVersions'),
      routing: 'stackVersions',
      hidden: !App.get('stackVersionsAvailable'),
      id: 'host-details-summary-version'
    }), Em.Object.create({
      name: 'logs',
      label: Em.I18n.t('hosts.host.menu.logs'),
      routing: 'logs',
      hidden: function () {
        if (App.get('supports.logSearch')) {
          return !(App.Service.find().someProperty('serviceName', 'LOGSEARCH') && App.isAuthorized('SERVICE.VIEW_OPERATIONAL_LOGS'));
        }
        return true;
      }.property('App.supports.logSearch'),
      id: 'host-details-summary-logs'
    })];
  }.property('App.stackVersionsAvailable'),

  /**
   * Update Alerts menu option counter text and class
   */
  updateAlertCounter: function () {
    var criticalWarningCount = this.get('host.criticalWarningAlertsCount');
    var criticalCount = this.get('host.alertsSummary.CRITICAL');
    var warningCount = this.get('host.alertsSummary.WARNING');
    var badgeText = "" + criticalWarningCount;
    var badgeClasses = "label";
    if (criticalCount > 0) {
      badgeClasses += " alerts-crit-count";
    } else if (warningCount > 0) {
      badgeClasses += " alerts-warn-count";
    }
    var alertOption = this.get('content').findProperty('name', 'alerts');
    alertOption.set('badgeText', badgeText);
    alertOption.set('badgeClasses', badgeClasses);
  }.observes('host.alertsSummary.CRITICAL', 'host.alertsSummary.WARNING', 'host.criticalWarningAlertsCount'),

  init: function init() {
    this._super();
    this.updateAlertCounter();
    this.activateView();
  },

  activateView: function () {
    var defaultRoute = App.router.get('currentState.name') || "summary";
    $.each(this._childViews, function () {
      this.set('active', this.get('content.routing') === defaultRoute ? 'active' : '');
    });
  }.observes('App.router.currentState.name'),

  deactivateChildViews: function deactivateChildViews() {
    this.get('_childViews').setEach('active', '');
  },

  itemViewClass: Em.View.extend({
    classNameBindings: ["active"],
    active: "",
    template: Ember.Handlebars.compile('{{#unless view.content.hidden}}<a {{action hostNavigate view.content.routing }} {{bindAttr id="view.content.id"}} href="#"> {{unbound view.content.label}} ' + '{{#if view.content.badgeText}} ' + '<span {{bindAttr class="view.content.badgeClasses"}}> ' + '{{view.content.badgeText}}' + '</span>  {{/if}}</a>{{/unless}}')
  })
});

});

require.register("views/main/host/metrics", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostMetricsView = Em.View.extend({
  templateName: require('templates/main/host/metrics'),
  content: Em.computed.alias('App.router.mainHostDetailsController.content')
});

});

require.register("views/main/host/metrics/cpu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing Host CPU metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsCPU = App.ChartLinearTimeView.extend({
  id: "host-metrics-cpu",
  title: Em.I18n.t('hosts.host.metrics.cpu'),
  displayUnit: '%',

  ajaxIndex: 'host.metrics.cpu',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/cpu/cpu_user', 'metrics/cpu/cpu_wio', 'metrics/cpu/cpu_nice', 'metrics/cpu/cpu_aidle', 'metrics/cpu/cpu_system', 'metrics/cpu/cpu_idle']
  },

  seriesTemplate: {
    path: 'metrics.cpu',
    displayName: function displayName(name) {
      var displayNameMap = {
        cpu_wio: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_wio'),
        cpu_idle: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_idle'),
        cpu_nice: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_nice'),
        cpu_aidle: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_aidle'),
        cpu_system: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_system'),
        cpu_user: Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_user')
      };
      return displayNameMap[name];
    }
  },

  getData: function getData(jsonData) {
    var dataArray = [],
        cpu_idle,
        template = this.get('seriesTemplate'),
        data = Em.get(jsonData, template.path);
    if (data) {
      for (var name in data) {
        var displayName = template.displayName(name),
            seriesData = data[name];
        if (seriesData) {
          var s = {
            name: displayName,
            data: seriesData
          };
          if (Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_idle') == displayName) {
            cpu_idle = s;
          } else {
            dataArray.push(s);
          }
        }
      }
      dataArray.push(cpu_idle);
    }
    return dataArray;
  },

  colorForSeries: function colorForSeries(series) {
    if (Em.I18n.t('hosts.host.metrics.cpu.displayNames.cpu_idle') == series.name) {
      return '#CFECEC';
    }
    return null;
  }
});

});

require.register("views/main/host/metrics/disk", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing host disk usage
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsDisk = App.ChartLinearTimeView.extend({
  id: "host-metrics-disk",
  title: Em.I18n.t('hosts.host.metrics.disk'),
  displayUnit: 'B',
  renderer: 'line',

  ajaxIndex: 'host.metrics.disk',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/disk/disk_total', 'metrics/disk/disk_free']
  },

  seriesTemplate: {
    path: 'metrics.disk',
    displayName: function displayName(name) {
      var displayNameMap = {
        disk_total: Em.I18n.t('hosts.host.metrics.disk.displayNames.disk_total'),
        disk_free: Em.I18n.t('hosts.host.metrics.disk.displayNames.disk_free')
      };
      return displayNameMap[name];
    },
    factor: Math.pow(2, 30)
  },

  getData: function getData(jsonData) {
    var partMaxUsed = Em.get(jsonData, 'metrics.part_max_used');
    if (!Em.isNone(partMaxUsed) && Em.get(jsonData, this.get('seriesTemplate.path'))) {
      Em.set(jsonData, this.get('seriesTemplate.path') + '.part_max_used', partMaxUsed);
    }
    return this._super(jsonData);
  }
});

});

require.register("views/main/host/metrics/load", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing host load
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsLoad = App.ChartLinearTimeView.extend({
  id: "host-metrics-load",
  title: Em.I18n.t('hosts.host.metrics.load'),
  renderer: 'line',

  ajaxIndex: 'host.metrics.load',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/load/load_fifteen', 'metrics/load/load_one', 'metrics/load/load_five']
  },

  seriesTemplate: {
    path: 'metrics.load',
    displayName: function displayName(name) {
      var displayNameMap = {
        load_fifteen: Em.I18n.t('hosts.host.metrics.load.displayNames.load_fifteen'),
        load_one: Em.I18n.t('hosts.host.metrics.load.displayNames.load_one'),
        load_five: Em.I18n.t('hosts.host.metrics.load.displayNames.load_five')
      };
      return displayNameMap[name];
    }
  }
});

});

require.register("views/main/host/metrics/memory", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing host memory metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsMemory = App.ChartLinearTimeView.extend({
  id: "host-metrics-memory",
  title: Em.I18n.t('hosts.host.metrics.memory'),
  displayUnit: 'B',
  renderer: 'line',

  ajaxIndex: 'host.metrics.memory',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/memory/swap_free', 'metrics/memory/mem_shared', 'metrics/memory/mem_free', 'metrics/memory/mem_cached', 'metrics/memory/mem_buffers']
  },

  seriesTemplate: {
    path: 'metrics.memory',
    displayName: function displayName(name) {
      var displayNameMap = {
        mem_shared: Em.I18n.t('hosts.host.metrics.memory.displayNames.mem_shared'),
        swap_free: Em.I18n.t('hosts.host.metrics.memory.displayNames.swap_free'),
        mem_buffers: Em.I18n.t('hosts.host.metrics.memory.displayNames.mem_buffers'),
        mem_free: Em.I18n.t('hosts.host.metrics.memory.displayNames.mem_free'),
        mem_cached: Em.I18n.t('hosts.host.metrics.memory.displayNames.mem_cached')
      };
      return displayNameMap[name];
    },
    factor: Math.pow(2, 10)
  }
});

});

require.register("views/main/host/metrics/network", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing host network metrics
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsNetwork = App.ChartLinearTimeView.extend({
  id: "host-metrics-network",
  title: Em.I18n.t('hosts.host.metrics.network'),
  displayUnit: 'B',
  renderer: 'line',

  ajaxIndex: 'host.metrics.network',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/network/bytes_in', 'metrics/network/bytes_out', 'metrics/network/pkts_in', 'metrics/network/pkts_out']
  },

  seriesTemplate: {
    path: 'metrics.network',
    displayName: function displayName(name) {
      var displayNameMap = {
        pkts_out: Em.I18n.t('hosts.host.metrics.network.displayNames.pkts_out'),
        bytes_in: Em.I18n.t('hosts.host.metrics.network.displayNames.bytes_in'),
        bytes_out: Em.I18n.t('hosts.host.metrics.network.displayNames.bytes_out'),
        pkts_in: Em.I18n.t('hosts.host.metrics.network.displayNames.pkts_in')
      };
      return displayNameMap[name];
    }
  }
});

});

require.register("views/main/host/metrics/processes", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing host process counts
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartHostMetricsProcesses = App.ChartLinearTimeView.extend({
  id: "host-metrics-processes",
  title: Em.I18n.t('hosts.host.metrics.processes'),
  renderer: 'line',

  ajaxIndex: 'host.metrics.processes',

  loadGroup: {
    name: 'host.metrics.aggregated',
    fields: ['metrics/process/proc_total', 'metrics/process/proc_run']
  },

  seriesTemplate: {
    path: 'metrics.process',
    displayName: function displayName(name) {
      var displayNameMap = {
        proc_total: Em.I18n.t('hosts.host.metrics.processes.displayNames.proc_total'),
        proc_run: Em.I18n.t('hosts.host.metrics.processes.displayNames.proc_run')
      };
      return displayNameMap[name];
    }
  }
});

});

require.register("views/main/host/stack_versions_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var filters = require('views/common/filter_view');
var sort = require('views/common/sort_view');

App.MainHostStackVersionsView = App.TableView.extend({
  templateName: require('templates/main/host/stack_versions'),
  classNames: ['host-tab-content', 'container-wrap-table'],

  /**
   * @type {Ember.Object}
   */
  host: Em.computed.alias('App.router.mainHostDetailsController.content'),

  /**
   * @type {Ember.Array}
   */
  content: Em.computed.filterBy('host.stackVersions', 'isVisible', true),

  /**
   * return filtered number of all content number information displayed on the page footer bar
   * @returns {string}
   */
  filteredContentInfo: Em.computed.i18nFormat('hosts.host.stackVersions.table.filteredInfo', 'filteredCount', 'totalCount'),

  /**
   * @type {Ember.View}
   */
  sortView: sort.wrapperView,

  /**
   * @type {Ember.View}
   */
  stackSort: sort.fieldView.extend({
    column: 1,
    name: 'stack',
    displayName: Em.I18n.t('common.stack'),
    type: 'version'
  }),

  /**
   * @type {Ember.View}
   */
  repoVersionSort: sort.fieldView.extend({
    column: 2,
    name: 'displayName',
    displayName: Em.I18n.t('common.name'),
    type: 'version'
  }),

  /**
   * @type {Ember.View}
   */
  statusSort: sort.fieldView.extend({
    column: 3,
    name: 'status',
    displayName: Em.I18n.t('common.status')
  }),

  /**
   * Filter view for stackName column
   * Based on <code>filters</code> library
   * @type {Ember.View}
   */
  stackFilterView: filters.createSelectView({
    column: 1,
    fieldType: 'filter-input-width',
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(this.get('parentView.content').mapProperty('stack').uniq().map(function (item) {
        return {
          value: item,
          label: item
        };
      }));
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * Filter view for version column
   * Based on <code>filters</code> library
   * @type {Ember.View}
   */
  repoVersionFilterView: filters.createSelectView({
    column: 2,
    fieldType: 'filter-input-width',
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(this.get('parentView.content').mapProperty('displayName').uniq().map(function (version) {
        return {
          value: version,
          label: version
        };
      }));
    }.property('App.router.clusterController.isLoaded'),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * Filter view for status column
   * Based on <code>filters</code> library
   * @type {Ember.View}
   */
  statusFilterView: filters.createSelectView({
    column: 3,
    fieldType: 'filter-input-width',
    content: function () {
      return [{
        value: '',
        label: Em.I18n.t('common.all')
      }].concat(App.HostStackVersion.statusDefinition.map(function (status) {
        return {
          value: status,
          label: App.HostStackVersion.formatStatus(status)
        };
      }));
    }.property(),
    onChangeValue: function onChangeValue() {
      this.get('parentView').updateFilter(this.get('column'), this.get('value'), 'select');
    }
  }),

  /**
   * show progress of installation of version on host
   */
  showInstallProgress: function showInstallProgress(event) {
    App.router.get('mainAdminStackAndUpgradeController').showProgressPopup(event.context);
  },

  outOfSyncInfo: Em.View.extend({
    tagName: 'i',
    classNames: ['glyphicon glyphicon-question-sign'],
    didInsertElement: function didInsertElement() {
      App.tooltip($(this.get('element')), {
        placement: "top",
        title: Em.I18n.t('hosts.host.stackVersions.outOfSync.tooltip')
      });
    },
    willDestroyElement: function willDestroyElement() {
      $(this.get('element')).tooltip('destroy');
    }
  }),

  /**
   * @type {Array}
   */
  colPropAssoc: function () {
    var associations = [];
    associations[1] = 'stack';
    associations[2] = 'displayName';
    associations[3] = 'status';
    return associations;
  }.property()

});

});

require.register("views/main/host/summary", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainHostSummaryView = Em.View.extend(App.HiveInteractiveCheck, App.TimeRangeMixin, {

  templateName: require('templates/main/host/summary'),

  /**
   * List of custom view for some host components
   * @type {Em.Object}
   * Format:
   *  <code>
   *    {
   *      COMPONENT_NAME1: VIEW1,
   *      COMPONENT_NAME2: VIEW2
   *      ....
   *    }
   *  </code>
   */
  hostComponentViewMap: Em.Object.create({
    'DATANODE': App.DataNodeComponentView,
    'NODEMANAGER': App.NodeManagerComponentView,
    'HBASE_REGIONSERVER': App.RegionServerComponentView,
    'TASKTRACKER': App.TaskTrackerComponentView
  }),

  /**
   * @type {bool}
   */
  isStopCommand: true,

  /**
   * @type {App.Host}
   */
  content: Em.computed.alias('App.router.mainHostDetailsController.content'),

  /**
   * Host metrics panel not displayed when Metrics service (ex:Ganglia) is not in stack definition.
   */
  hasHostMetricsService: Em.computed.gt('App.services.hostMetrics.length', 0),

  nameNodeComponent: Em.computed.findBy('content.hostComponents', 'componentName', 'NAMENODE'),

  hasNameNode: Em.computed.bool('nameNodeComponent'),

  showHostMetricsBlock: Em.computed.or('hasHostMetricsService', 'hasNameNode'),

  nameNodeWidgets: function () {
    var hasNameNode = this.get('hasNameNode');
    var widgets = [];
    if (hasNameNode) {
      var model = App.HDFSService.find('HDFS'),
          hostName = this.get('content.hostName'),
          widgetsDefinitions = require('data/dashboard_widgets').toMapByProperty('viewName');
      widgets.pushObjects([App.NameNodeHeapPieChartView.extend({
        model: model,
        hostName: hostName,
        widgetHtmlId: 'nn-heap',
        title: Em.I18n.t('dashboard.widgets.NameNodeHeap'),
        showActions: false,
        widget: {
          threshold: widgetsDefinitions.NameNodeHeapPieChartView.threshold
        }
      }), App.NameNodeCpuPieChartView.extend({
        widgetHtmlId: 'nn-cpu',
        title: Em.I18n.t('dashboard.widgets.NameNodeCpu'),
        showActions: false,
        widget: {
          threshold: widgetsDefinitions.NameNodeCpuPieChartView.threshold
        },
        subGroupId: this.get('nameNodeComponent.haNameSpace'),
        activeNameNodes: [this.get('nameNodeComponent')],
        nameNode: this.get('nameNodeComponent')
      }), App.NameNodeRpcView.extend({
        model: model,
        hostName: hostName,
        widgetHtmlId: 'nn-rpc',
        title: Em.I18n.t('dashboard.widgets.NameNodeRpc'),
        showActions: false,
        widget: {
          threshold: widgetsDefinitions.NameNodeRpcView.threshold
        }
      }), App.NameNodeUptimeView.extend({
        model: model,
        hostName: hostName,
        widgetHtmlId: 'nn-uptime',
        title: Em.I18n.t('dashboard.widgets.NameNodeUptime'),
        showActions: false,
        subGroupId: this.get('nameNodeComponent.haNameSpace')
      })]);
    }
    return widgets;
  }.property('hasNameNode'),

  /**
   * Message for "restart" block
   * @type {String}
   */
  needToRestartMessage: function () {
    var componentsCount, word;
    componentsCount = this.get('content.componentsWithStaleConfigsCount');
    if (componentsCount > 1) {
      word = Em.I18n.t('common.components').toLowerCase();
    } else {
      word = Em.I18n.t('common.component').toLowerCase();
    }
    return Em.I18n.t('hosts.host.details.needToRestart').format(this.get('content.componentsWithStaleConfigsCount'), word);
  }.property('content.componentsWithStaleConfigsCount'),

  willInsertElement: function willInsertElement() {
    this.sortedComponentsFormatter();
    this.addObserver('content.hostComponents.length', this, 'sortedComponentsFormatter');
    if (this.get('installedServices').indexOf('HIVE') !== -1) {
      this.loadHiveConfigs();
    }
  },

  didInsertElement: function didInsertElement() {
    this._super();
    this.addToolTip();
  },

  /**
   * Create tooltip for "Add" button if nothing to add to the current host
   */
  addToolTip: function () {
    if (this.get('addComponentDisabled')) {
      App.tooltip($('#add_component'), { title: Em.I18n.t('services.nothingToAdd') });
    }
  }.observes('addComponentDisabled'),

  /**
   * List of installed services
   * @type {String[]}
   */
  installedServices: function () {
    return App.Service.find().mapProperty('serviceName');
  }.property('App.router.clusterController.dataLoadList.services'),

  /**
   * List of installed masters and slaves
   * Masters first, then slaves
   * @type {App.HostComponent[]}
   */
  sortedComponents: [],

  /**
   * Update <code>sortedComponents</code>
   * Master components first, then slaves and clients
   */
  sortedComponentsFormatter: function sortedComponentsFormatter() {
    var updatebleProperties = Em.A(['workStatus', 'passiveState', 'staleConfigs', 'haStatus']);
    var hostComponentViewMap = this.get('hostComponentViewMap');
    var masters = [],
        slaves = [],
        clients = [];

    this.get('content.hostComponents').forEach(function (component) {
      component.set('viewClass', hostComponentViewMap[component.get('componentName')] ? hostComponentViewMap[component.get('componentName')] : App.HostComponentView);
      if (component.get('isMaster')) {
        masters.push(component);
      } else if (component.get('isSlave')) {
        slaves.push(component);
      } else if (component.get('isClient')) {
        component.set('isLast', true);
        component.set('isInstallFailed', ['INSTALL_FAILED', 'INIT'].contains(component.get('workStatus')));
        clients.pushObject(component);
      }
    }, this);
    this.set('sortedComponents', masters.concat(slaves, clients));
  },

  /**
   * Template for addable component
   * @type {Em.Object}
   */
  addableComponentObject: Em.Object.extend({
    componentName: '',
    displayName: function () {
      return App.format.role(this.get('componentName'), false);
    }.property('componentName')
  }),

  /**
   * If host lost heartbeat, components can't be added on it
   * @type {bool}
   */
  isAddComponent: Em.computed.notEqual('content.healthClass', 'health-status-DEAD-YELLOW'),

  /**
   * Disable "Add" button if components can't be added to the current host
   * @type {bool}
   */
  addComponentDisabled: Em.computed.or('!isAddComponent', '!addableComponents.length'),

  /**
   * List of components that may be added to the current host
   * @type {Em.Object[]}
   */
  addableComponents: function () {
    var components = [];
    var self = this;
    if (this.get('content.hostComponents')) {
      var installedComponents = this.get('content.hostComponents').mapProperty('componentName');
      var addableToHostComponents = App.StackServiceComponent.find().filterProperty('isAddableToHost');
      var installedServices = this.get('installedServices');

      addableToHostComponents.forEach(function (addableComponent) {
        if (installedServices.contains(addableComponent.get('serviceName')) && !installedComponents.contains(addableComponent.get('componentName')) && !this.hasCardinalityConflict(addableComponent.get('componentName'))) {
          if (addableComponent.get('componentName') === 'OOZIE_SERVER' && !App.router.get('mainHostDetailsController.isOozieServerAddable') || addableComponent.get('componentName') === 'HIVE_SERVER_INTERACTIVE' && !self.get('enableHiveInteractive')) {
            return;
          }
          components.pushObject(self.addableComponentObject.create({
            'componentName': addableComponent.get('componentName'),
            'serviceName': addableComponent.get('serviceName')
          }));
        }
      }, this);
    }
    return components;
  }.property('content.hostComponents.length', 'App.components.addableToHost.@each', 'enableHiveInteractive'),

  /**
   *
   * @param {string} componentName
   * @returns {boolean}
   */
  hasCardinalityConflict: function hasCardinalityConflict(componentName) {
    var totalCount = App.HostComponent.getCount(componentName, 'totalCount');
    var maxToInstall = App.StackServiceComponent.find(componentName).get('maxToInstall');
    return !(totalCount < maxToInstall);
  },

  /**
   * Formatted with <code>$.timeago</code> value of host's last heartbeat
   * @type {String}
   */
  timeSinceHeartBeat: function () {
    if (this.get('content.isNotHeartBeating')) {
      var d = this.get('content.lastHeartBeatTime');
      return d ? $.timeago(d) : '';
    }
    //when host hasn't lost heartbeat we assume that last heartbeat was a minute ago
    return Em.I18n.t('common.minute.ago');
  }.property('content.lastHeartBeatTime', 'content.isNotHeartBeating'),

  /**
   * Get clients with custom commands
   */
  clientsWithCustomCommands: function () {
    var clients = this.get('clients').rejectProperty('componentName', 'KERBEROS_CLIENT');
    var options = [];
    var clientWithCommands;
    clients.forEach(function (client) {
      var componentName = client.get('componentName');
      var customCommands = App.StackServiceComponent.find(componentName).get('customCommands');

      if (customCommands.length) {
        clientWithCommands = {
          label: client.get('displayName'),
          commands: []
        };
        customCommands.forEach(function (command) {
          clientWithCommands.commands.push({
            label: Em.I18n.t('services.service.actions.run.executeCustomCommand.menu').format(command),
            service: client.get('service.serviceName'),
            hosts: client.get('hostName'),
            component: componentName,
            command: command
          });
        });

        options.push(clientWithCommands);
      }
    });

    return options;
  }.property('controller'),

  /**
   * Call installClients method from controller for not installed client components
   */
  installClients: function installClients() {
    this.get('controller').installClients(this.get('notInstalledClientComponents'));
  },

  /**
   * Call installClients method from controller for not install failed client components
   */
  reinstallClients: function reinstallClients() {
    this.get('controller').installClients(this.get('installFailedClients'));
  }
});

});

require.register("views/main/menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainSideMenuView = Em.CollectionView.extend({
  tagName: 'ul',
  classNames: ['nav', 'side-nav-menu', 'nav-pills', 'nav-stacked'],

  views: function () {
    return App.router.get('mainViewsController.ambariViews');
  }.property('App.router.mainViewsController.ambariViews'),

  didInsertElement: function didInsertElement() {
    $('[data-toggle="collapse-side-nav"]').on('click', function () {
      if ($('.navigation-bar-container.collapsed').length > 0) {
        App.tooltip($('.navigation-bar-container.collapsed .mainmenu-li:not(.has-sub-menu)>a'), { placement: "right" });
        App.tooltip($('.navigation-bar-container.collapsed .mainmenu-li.has-sub-menu>a'), { placement: "top" });
      } else {
        $('.navigation-bar-container .mainmenu-li>a').tooltip('destroy');
      }
    });
  },

  content: function () {
    var result = [];
    var router = App.router;

    if (router.get('loggedIn')) {

      if (router.get('clusterController.isLoaded') && App.get('router.clusterInstallCompleted')) {
        if (!App.get('isOnlyViewUser')) {
          result.push({ label: Em.I18n.t('menu.item.dashboard'), iconClass: 'glyphicon glyphicon-home', routing: 'dashboard', active: 'active', href: router.urlFor('main.dashboard') }, { label: Em.I18n.t('menu.item.services'), iconClass: 'glyphicon glyphicon-briefcase', routing: 'services', href: router.urlFor('main.services') }, { label: Em.I18n.t('menu.item.hosts'), iconClass: 'icon-tasks', routing: 'hosts', href: router.urlFor('main.hosts') }, { label: Em.I18n.t('menu.item.alerts'), iconClass: 'glyphicon glyphicon-bell', routing: 'alerts', href: router.urlFor('main.alerts') });
        }
        if (App.isAuthorized('CLUSTER.TOGGLE_KERBEROS, CLUSTER.MODIFY_CONFIGS, SERVICE.START_STOP, SERVICE.SET_SERVICE_USERS_GROUPS, CLUSTER.UPGRADE_DOWNGRADE_STACK, CLUSTER.VIEW_STACK_DETAILS') || App.get('upgradeInProgress') || App.get('upgradeHolding')) {
          result.push({ label: Em.I18n.t('menu.item.admin'), iconClass: 'glyphicon glyphicon-wrench', routing: 'admin', href: router.urlFor('main.admin') });
        }
      }
    }
    return result;
  }.property('App.router.loggedIn', 'App.router.clusterController.isLoaded', 'App.router.clusterInstallCompleted', 'App.router.wizardWatcherController.isWizardRunning'),

  itemViewClass: Em.View.extend({

    classNameBindings: ['dropdownMenu:dropdown'],

    classNames: ['mainmenu-li'],

    templateName: require('templates/main/side-menu-item'),

    dropdownMenu: Em.computed.existsIn('content.routing', ['services', 'admin']),
    isAdminItem: Em.computed.equal('content.routing', 'admin'),
    isServicesItem: Em.computed.equal('content.routing', 'services'),
    isViewsItem: function () {
      return this.get('content').routing.contains('views');
    }.property(''),
    goToSection: function goToSection(event) {
      if (event.context === 'hosts') {
        App.router.set('mainHostController.showFilterConditionsFirstLoad', false);
      } else if (event.context === 'views') {
        App.router.route('views');
        return;
      } else if (event.context === 'alerts') {
        App.router.set('mainAlertDefinitionsController.showFilterConditionsFirstLoad', false);
      }
      App.router.route('main/' + event.context);
    },

    selectedAdminItemBinding: 'App.router.mainAdminController.category',

    dropdownCategories: function () {
      var itemName = this.get('content.routing');
      var categories = [];
      var upg = App.get('upgradeInProgress') || App.get('upgradeHolding');
      // create dropdown categories for each menu item
      var router = App.router;

      if (itemName === 'admin') {
        if (App.isAuthorized('CLUSTER.VIEW_STACK_DETAILS, CLUSTER.UPGRADE_DOWNGRADE_STACK') || upg) {
          categories.push({
            name: 'stackAndUpgrade',
            url: 'stack',
            label: Em.I18n.t('admin.stackUpgrade.title'),
            href: router.urlFor('main.admin.stackAndUpgrade')
          });
        }
        if (App.isAuthorized('SERVICE.SET_SERVICE_USERS_GROUPS') || upg) {
          categories.push({
            name: 'adminServiceAccounts',
            url: 'serviceAccounts',
            label: Em.I18n.t('common.serviceAccounts'),
            disabled: App.get('upgradeInProgress') || App.get('upgradeHolding'),
            href: router.urlFor('main.admin.adminServiceAccounts')
          });
        }
        if (!App.get('isHadoopWindowsStack') && App.isAuthorized('CLUSTER.TOGGLE_KERBEROS') || upg) {
          if (App.supports.enableToggleKerberos) {
            categories.push({
              name: 'kerberos',
              url: 'kerberos/',
              label: Em.I18n.t('common.kerberos'),
              disabled: App.get('upgradeInProgress') || App.get('upgradeHolding'),
              href: router.urlFor('main.admin.adminKerberos')
            });
          }
        }
        if (App.isAuthorized('SERVICE.START_STOP, CLUSTER.MODIFY_CONFIGS') && App.isAuthorized('SERVICE.MANAGE_AUTO_START, CLUSTER.MANAGE_AUTO_START') || upg) {
          if (App.supports.serviceAutoStart) {
            categories.push({
              name: 'serviceAutoStart',
              url: 'serviceAutoStart',
              label: Em.I18n.t('admin.serviceAutoStart.title'),
              href: router.urlFor('main.admin.adminServiceAutoStart')
            });
          }
        }
      }
      return categories;
    }.property('content.routing'),

    AdminDropdownItemView: Ember.View.extend({
      tagName: 'li',
      classNameBindings: ['isActive:active', 'isDisabled:disabled'],
      classNames: ['submenu-li'],
      isActive: Em.computed.equalProperties('item', 'parentView.selectedAdminItem'),
      isDisabled: function () {
        return !!this.get('parentView.dropdownCategories').findProperty('name', this.get('item')).disabled;
      }.property('item', 'parentView.dropdownCategories.@each.disabled'),
      goToCategory: function goToCategory(event) {
        var itemName = this.get('parentView').get('content').routing;
        // route to correct category of current menu item
        // skip routing to already selected category
        if (itemName === 'admin' && !this.get('isActive') && !this.get('isDisabled')) {
          App.router.route('main/admin/' + event.context);
        }
      }
    })
  })
});

App.SideNavServiceMenuView = Em.CollectionView.extend({
  disabledServices: [],

  content: function () {
    return App.router.get('mainServiceController.content').filter(function (item) {
      return !this.get('disabledServices').contains(item.get('id'));
    }, this);
  }.property('App.router.mainServiceController.content.length').volatile(),

  didInsertElement: function didInsertElement() {
    App.router.location.addObserver('lastSetURL', this, 'renderOnRoute');
    this.renderOnRoute();
    App.tooltip(this.$(".restart-required-service"), { html: true, placement: "right" });
    App.tooltip($("[rel='serviceHealthTooltip']"), { html: true, placement: "right" });
    App.tooltip(this.$(".passive-state-service"), { html: true, placement: "top" });
  },

  willDestroyElement: function willDestroyElement() {
    App.router.location.removeObserver('lastSetURL', this, 'renderOnRoute');
    this.$(".restart-required-service").tooltip('destroy');
    this.$(".passive-state-service").tooltip('destroy');
  },

  activeServiceId: null,
  /**
   *    Syncs navigation menu with requested URL
   */
  renderOnRoute: function renderOnRoute() {
    var lastUrl = App.router.location.lastSetURL || location.href.replace(/^[^#]*#/, '');
    if (lastUrl.substr(1, 4) !== 'main' || !this._childViews) {
      return;
    }
    var reg = /^\/main\/services\/(\S+)\//g;
    var subUrl = reg.exec(lastUrl);
    var serviceId = null != subUrl ? subUrl[1] : 1;
    this.set('activeServiceId', serviceId);
  },

  tagName: 'ul',
  classNames: ['sub-menu', 'nav', 'nav-pills', 'nav-stacked', 'services-submenu'],

  itemViewClass: Em.View.extend({

    classNameBindings: ["clients"],
    classNames: ["submenu-li"],
    templateName: require('templates/main/service/menu_item'),
    restartRequiredMessage: null,

    shouldBeRestarted: Em.computed.someBy('content.hostComponents', 'staleConfigs', true),

    alertsCountDisplay: function () {
      return this.get('content.alertsCount') > 99 ? "99+" : this.get('content.alertsCount');
    }.property('content.alertsCount'),

    noAlerts: Em.computed.equal('content.alertsCount', 0),

    hasCriticalAlerts: Em.computed.alias('content.hasCriticalAlerts'),

    isMasterDown: function () {
      return this.get('content.hostComponents').filterProperty('isMaster').some(function (component) {
        return !component.get('isRunning');
      });
    }.property('content.hostComponents.@each.workStatus'),

    isClientOnlyService: function () {
      return App.get('services.clientOnly').contains(this.get('content.serviceName'));
    }.property('content.serviceName'),

    isConfigurable: function () {
      return !App.get('services.noConfigTypes').contains(this.get('content.serviceName'));
    }.property('App.services.noConfigTypes', 'content.serviceName'),

    /**
     * '#/main/services/SERVICE_ID'
     *
     * @type {string}
     */
    dataHref: function () {
      return App.router.urlFor('main.services.service', { service_id: this.get('content.id') });
    }.property('content.id'),

    link: function () {
      var stateName = ['summary', 'configs'].contains(App.router.get('currentState.name')) ? this.get('isConfigurable') ? App.router.get('currentState.name') : 'summary' : 'summary';
      return "#/main/services/" + this.get('content.id') + "/" + stateName;
    }.property('App.router.currentState.name', 'parentView.activeServiceId', 'isConfigurable'),

    goToConfigs: function goToConfigs() {
      App.router.set('mainServiceItemController.routeToConfigs', true);
      App.router.transitionTo('services.service.configs', this.get('content'));
      App.router.set('mainServiceItemController.routeToConfigs', false);
    },

    refreshRestartRequiredMessage: function () {
      var restarted, componentsCount, hostsCount, message, tHosts, tComponents;
      restarted = this.get('content.restartRequiredHostsAndComponents');
      componentsCount = 0;
      hostsCount = 0;
      message = "";
      for (var host in restarted) {
        hostsCount++;
        componentsCount += restarted[host].length;
      }
      if (hostsCount > 1) {
        tHosts = Em.I18n.t('common.hosts');
      } else {
        tHosts = Em.I18n.t('common.host');
      }
      if (componentsCount > 1) {
        tComponents = Em.I18n.t('common.components');
      } else {
        tComponents = Em.I18n.t('common.component');
      }
      message += componentsCount + ' ' + tComponents + ' ' + Em.I18n.t('on') + ' ' + hostsCount + ' ' + tHosts + ' ' + Em.I18n.t('services.service.config.restartService.needToRestartEnd');
      this.set('restartRequiredMessage', message);
    }.observes('content.restartRequiredHostsAndComponents')
  })

});

});

require.register("views/main/service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceView = Em.View.extend({
  templateName: require('templates/main/service'),
  didInsertElement: function didInsertElement() {
    this.addToolTip();
  },
  addToolTip: function () {
    /* istanbul ignore next */
    if (this.get('controller.isAllServicesInstalled')) {
      App.tooltip($('.add-service-button a'), { title: Em.I18n.t('services.nothingToAdd') });
    }
  }.observes('controller.isAllServicesInstalled')
});

});

require.register("views/main/service/add_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AddServiceView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/service/add')

});

});

require.register("views/main/service/all_services_actions", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AllServicesActionView = Em.View.extend({
  templateName: require('templates/main/service/all_services_actions'),

  classNames: ['more-actions'],

  serviceController: function () {
    return App.get('router.mainServiceController');
  }.property('App.router.mainServiceController')
});

});

require.register("views/main/service/info/components_list_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SummaryMasterComponentsView = Em.View.extend({
  templateName: require('templates/main/service/info/summary/master_components'),

  // should be bound to App.MainDashboardServiceHdfsView instance
  summaryView: null,

  mastersComp: [{
    components: []
  }],

  mastersCompWillChange: function () {
    Em.run.next(this.removeTooltips);
  }.observesBefore('mastersComp.length', 'mastersComp.@each.components'),

  mastersCompDidChange: function () {
    Em.run.next(this.attachTooltip);
  }.observes('mastersComp.length', 'mastersComp.@each.components'),

  removeTooltips: function removeTooltips() {
    $('.tooltip').remove();
    $(App.SummaryMasterComponentsView.tooltipsSelector).tooltip('destroy');
  },

  attachTooltip: function attachTooltip() {
    App.tooltip($(App.SummaryMasterComponentsView.tooltipsSelector));
  },

  didInsertElement: function didInsertElement() {
    this.attachTooltip();
  },

  willDestroyElement: function willDestroyElement() {
    this.removeTooltips();
  }
});

App.SummaryMasterComponentsView.reopenClass({
  tooltipsSelector: '[rel=SummaryComponentHealthTooltip], [rel=UsageTooltip], [rel=MasterGroupTitleTooltip]'
});

App.SummaryClientComponentsView = Em.View.extend({
  templateName: require('templates/main/service/info/summary/client_components'),
  clientsObj: []
});

App.SummarySlaveComponentsView = Em.View.extend({
  templateName: require('templates/main/service/info/summary/slave_components'),
  slavesObj: []
});

});

require.register("views/main/service/info/configs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');

App.MainServiceInfoConfigsView = Em.View.extend({

  templateName: require('templates/main/service/info/configs'),

  didInsertElement: function didInsertElement() {
    var self = this;
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isConfigsPropertiesLoaded').done(function () {
      self.get('controller').loadStep();
    });
    this.resetConfigTabSelection();
  },

  /**
   * If user A is on the Service Configs page and B starts some Wizard, user A should be moved-out and then moved-in this page
   * It's done to properly disable "admin"-elements
   * This code can't be moved to the controller, because it should work only if user is in the configs page (this view exists)
   */
  simulateRefresh: function () {
    App.router.transitionTo('main.services.service.summary', this.get('controller.content'));
    App.router.transitionTo('main.services.service.configs', this.get('controller.content'));
  }.observes('App.router.wizardWatcherController.isWizardRunning'),

  willDestroyElement: function willDestroyElement() {
    this.get('controller').clearStep();
  },

  /**
   * reset selection flag of tabs on entering Configs page
   */
  resetConfigTabSelection: function resetConfigTabSelection() {
    App.Tab.find().filterProperty('serviceName', this.get('controller.content.serviceName')).setEach('isActive', false);
  },

  /**
   * Number of components that should be restarted
   * @type {number}
   */
  componentsCount: null,

  /**
   * Number of hosts with components that should be restarted
   * @type {number}
   */
  hostsCount: null,

  /**
   * @type {boolean}
   */
  isStopCommand: true,

  /**
   * @method updateComponentInformation
   */
  updateComponentInformation: function () {
    var hc = this.get('controller.content.restartRequiredHostsAndComponents');
    var componentsCount = 0;
    var hosts = Em.keys(hc);
    hosts.forEach(function (host) {
      componentsCount += hc[host].length;
    });
    this.setProperties({
      componentsCount: componentsCount,
      hostsCount: hosts.length
    });
  }.observes('controller.content.restartRequiredHostsAndComponents'),

  /**
   * @type {string}
   */
  rollingRestartSlaveComponentName: function () {
    return batchUtils.getRollingRestartComponentName(this.get('controller.content.serviceName'));
  }.property('controller.content.serviceName'),

  /**
   * @type {string}
   */
  rollingRestartActionName: function () {
    var componentName = this.get('rollingRestartSlaveComponentName');
    return componentName ? Em.I18n.t('rollingrestart.dialog.title').format(App.format.role(componentName, false)) : '';
  }.property('rollingRestartSlaveComponentName')

});

});

require.register("views/main/service/info/heatmap_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');
App.MainServiceInfoHeatmapView = App.MainChartsHeatmapView.extend({
  dropdownView: Em.View.extend({
    templateName: require('templates/main/service/info/heatmap_dropdown')
  })
});

});

require.register("views/main/service/info/menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceInfoMenuView = Em.CollectionView.extend({
  tagName: 'ul',
  classNames: ["nav", "nav-tabs", "background-text"],
  content: function () {
    var menuItems = [{
      label: Em.I18n.t('services.service.info.menu.summary'),
      id: 'summary-service-tab',
      routing: 'summary',
      active: "active"
    }];

    if (this.get('heatmapTab')) {
      menuItems.push({
        label: Em.I18n.t('services.service.info.menu.heatmaps'),
        id: 'heatmap-service-tab',
        routing: 'heatmaps'
      });
    }
    if (this.get('configTab')) {
      menuItems.push({
        label: Em.I18n.t('services.service.info.menu.configs'),
        id: 'configs-service-tab',
        routing: 'configs'
      });
    }
    if (this.get('metricTab')) {
      menuItems.push({
        label: Em.I18n.t('services.service.info.menu.metrics'),
        id: 'metrics-service-tab',
        routing: 'metrics'
      });
    }
    return menuItems;
  }.property(),

  init: function init() {
    this._super();
    this.activateView();
  },

  activateView: function () {
    this.get('_childViews').forEach(function (view) {
      view.set('active', document.URL.endsWith(view.get('content.routing')) ? "active" : "");
    }, this);
  }.observes('App.router.location.lastSetURL'),

  deactivateChildViews: function deactivateChildViews() {
    this.get('_childViews').setEach('active', '');
  },

  itemViewClass: Em.View.extend({
    classNameBindings: ["active"],
    active: "",
    template: Ember.Handlebars.compile('<a {{action showInfo view.content.routing}} {{bindAttr id="view.content.id"}} href="#"> {{unbound view.content.label}}</a>')
  })
});

});

require.register("views/main/service/info/metric_graphs_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
require('views/main/service/service');

App.MainServiceInfoSummaryMetricGraphsView = Em.View.extend({
  templateName: require('templates/main/service/info/metric_graphs'),

  /**
   * Data used by this view which is set from the template.
   */
  viewData: null,

  /**
   * Contains graphs for this particular service
   */
  serviceMetricGraphs: null
});

});

require.register("views/main/service/info/metrics/ambari_metrics/master_average_load", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase master avereage load
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_MasterAverageLoad = App.ChartLinearTimeView.extend({
  id: "service-metrics-ambari-metrics-master-average-load",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.master.averageLoad'),
  ajaxIndex: 'service.metrics.ambari_metrics.master.average_load',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/master/AverageLoad']
  },

  seriesTemplate: {
    path: 'metrics.hbase.master',
    displayName: function displayName(name) {
      var displayNameMap = {
        AverageLoad: Em.I18n.t('services.service.info.metrics.ambariMetrics.master.displayNames.averageLoad')
      };
      return displayNameMap[name];
    }
  }
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_base", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a a base for views showing data for HBase RegionServer
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerBaseView = App.ChartLinearTimeView.extend({
  displayName: '',
  regionServerName: '',
  seriesTemplate: {
    path: 'metrics.hbase.regionserver'
  },

  getData: function getData(jsonData) {
    var dataArray = [],
        template = this.get('seriesTemplate'),
        data = Em.get(jsonData, template.path);
    if (data) {
      for (var name in data) {
        if (name === this.get('regionServerName')) {
          var displayName = this.get('displayName'),
              currentData = data[name];
          if (currentData) {
            dataArray.push({
              name: displayName,
              data: currentData
            });
          }
        }
      }
    }
    return dataArray;
  }
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_block_cache_hit_percent", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase RegionServer Block cache hit percent
 *
 * @extends App.ChartServiceMetricsAMS_RegionServerBaseView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerBlockCacheHitPercent = App.ChartServiceMetricsAMS_RegionServerBaseView.extend({
  id: "service-metrics-ambari-metrics-region-server-block-cache-hit-percent",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.blockCacheHitPercent'),
  renderer: 'line',
  displayUnit: '%',
  ajaxIndex: 'service.metrics.ambari_metrics.region_server.block_cache_hit_percent',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/regionserver/blockCacheHitPercent']
  },

  displayName: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.displayNames.blockCacheHitPercent'),
  regionServerName: 'blockCacheHitPercent'
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_compaction_queue_size", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase RegionServer compaction queue size
 *
 * @extends App.ChartServiceMetricsAMS_RegionServerBaseView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerCompactionQueueSize = App.ChartServiceMetricsAMS_RegionServerBaseView.extend({
  id: "service-metrics-ambari-metrics-region-server-compaction-queue-size",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.compactionQueueSize'),
  ajaxIndex: 'service.metrics.ambari_metrics.region_server.compaction_queue_size',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/regionserver/compactionQueueSize']
  },

  displayName: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.displayNames.compactionQueueSize'),
  regionServerName: 'compactionQueueSize'
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_regions", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase RegionServer regions
 *
 * @extends App.ChartServiceMetricsAMS_RegionServerBaseView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerRegions = App.ChartServiceMetricsAMS_RegionServerBaseView.extend({
  id: "service-metrics-ambari-metrics-region-server-regions",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.regions'),
  ajaxIndex: 'service.metrics.ambari_metrics.region_server.regions',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/regionserver/regions']
  },

  displayName: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.displayNames.regionsCount'),
  regionServerName: 'regions'
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_requests", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase RegionServer requests
 *
 * @extends App.ChartServiceMetricsAMS_RegionServerBaseView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerRequests = App.ChartServiceMetricsAMS_RegionServerBaseView.extend({
  id: "service-metrics-ambari-metrics-region-server-requests",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.requests'),
  ajaxIndex: 'service.metrics.ambari_metrics.region_server.request',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/regionserver/requests._rate']
  },

  displayName: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.displayNames.requestCount'),
  regionServerName: 'requests._rate'
});

});

require.register("views/main/service/info/metrics/ambari_metrics/regionserver_store_files", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase RegionServer store files
 *
 * @extends App.ChartServiceMetricsAMS_RegionServerBaseView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsAMS_RegionServerStoreFiles = App.ChartServiceMetricsAMS_RegionServerBaseView.extend({
  id: "service-metrics-ambari-metrics-region-server-store-files",
  title: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.storeFiles'),
  renderer: 'line',
  ajaxIndex: 'service.metrics.ambari_metrics.region_server.store_files',

  loadGroup: {
    name: 'service.metrics.ambari_metrics.aggregated',
    fields: ['metrics/hbase/regionserver/storefiles']
  },

  regionServerName: 'storefiles',
  displayName: Em.I18n.t('services.service.info.metrics.ambariMetrics.regionServer.displayNames.storeFilesCount')
});

});

require.register("views/main/service/info/metrics/flume/channel_fill_pct", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase Cluster Requests
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_ChannelFillPercent = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-channel-fill-percent",
  title: Em.I18n.t('services.service.info.metrics.flume.channelFillPercent'),
  displayUnit: '%',

  ajaxIndex: 'service.metrics.flume.channel_fill_percent',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.channelName').format(name + ' (' + hostName + ')');
    },
    flumePropertyName: 'ChannelFillPercentage'
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/channel_size", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase Cluster Requests
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_ChannelSize = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-channel-size",
  title: Em.I18n.t('services.service.info.metrics.flume.channelSize'),

  ajaxIndex: 'service.metrics.flume.channel_size',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.channelName').format(name + ' (' + hostName + ')');
    },
    flumePropertyName: 'ChannelSize'
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/channel_size_mma", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing HBase Cluster Requests
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_ChannelSizeMMA = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-channel-size-mma",
  title: Em.I18n.t('services.service.info.metrics.flume.channelSizeMMA'),
  renderer: 'line',
  ajaxIndex: 'service.metrics.flume.channel_size_for_all.mma',
  displayUnit: '/s',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.ChannelSize.rate',
    displayName: function displayName(name) {
      return Em.I18n.t('services.service.info.metrics.flume.channelType').format(name);
    }
  },

  colorForSeries: function colorForSeries(series) {
    if (Em.I18n.t('services.service.info.metrics.flume.channelType').format("avg") == series.name) {
      return '#0066b3';
    } else if (Em.I18n.t('services.service.info.metrics.flume.channelType').format("min") == series.name) {
      return '#00CC00';
    } else if (Em.I18n.t('services.service.info.metrics.flume.channelType').format("max") == series.name) {
      return '#FF8000';
    }
    return null;
  }
});

});

require.register("views/main/service/info/metrics/flume/channel_sum", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 *
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_ChannelSizeSum = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-channel-size-sum",
  title: Em.I18n.t('services.service.info.metrics.flume.channelSizeSum'),
  displayUnit: 'B',

  ajaxIndex: 'service.metrics.flume.channel_size_for_all.sum',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.ChannelSize.rate',
    displayName: function displayName() {
      return Em.I18n.t('services.service.info.metrics.flume.channelSizeSum');
    }
  }
});

});

require.register("views/main/service/info/metrics/flume/cpu_user", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing HBase Cluster Requests
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_CPUUser = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-cpu-user",
  title: Em.I18n.t('services.service.info.metrics.flume.cpu.user'),
  displayUnit: '%',
  renderer: 'line',

  ajaxIndex: 'service.metrics.flume.cpu_user',

  seriesTemplate: {
    path: 'metrics.cpu.cpu_user',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.hostName').format(hostName);
    }
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/flume_agent_metrics_section", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.FlumeAgentMetricsSectionView = Em.View.extend(App.TimeRangeMixin, {

  templateName: require('templates/main/service/info/metrics/flume/flume_agent_metrics_section'),

  index: 0,

  metricTypeKey: '',

  metricView: null,

  metricViewData: null,

  id: Em.computed.format('metric{0}', 'index'),

  toggleIndex: Em.computed.format('#{0}', 'id'),

  header: function () {
    var metricType = Em.I18n.t('services.service.info.metrics.flume.' + this.get('metricTypeKey')).format(Em.I18n.t('common.metrics'));
    return metricType + ' - ' + this.get('metricViewData.agent.hostName');
  }.property('metricTypeKey', 'metricViewData.agent.hostName')

});

});

require.register("views/main/service/info/metrics/flume/flume_incoming_mma", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_IncommingMMA = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-incoming_mma",
  renderer: 'line',
  title: Em.I18n.t('services.service.info.metrics.flume.incoming.mma'),

  ajaxIndex: 'service.metrics.flume.incoming_event_put_successCount.mma',
  displayUnit: '/s',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.EventPutSuccessCount.rate',
    displayName: function displayName(name) {
      return Em.I18n.t('services.service.info.metrics.flume.incoming_mma').format(name);
    }
  },

  colorForSeries: function colorForSeries(series) {
    var name = Em.I18n.t('services.service.info.metrics.flume.incoming_mma');
    if (name.format("avg") == series.name) {
      return '#0066b3';
    } else if (name.format("min") == series.name) {
      return '#00CC00';
    } else if (name.format("max") == series.name) {
      return '#FF8000';
    }
    return null;
  }
});

});

require.register("views/main/service/info/metrics/flume/flume_incoming_sum", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_IncommingSum = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-incoming_sum",
  title: Em.I18n.t('services.service.info.metrics.flume.incoming.sum'),

  ajaxIndex: 'service.metrics.flume.incoming_event_put_successCount.sum',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.EventPutSuccessCount.rate',
    displayName: function displayName() {
      return Em.I18n.t('services.service.info.metrics.flume.incoming.sum');
    }
  }
});

});

require.register("views/main/service/info/metrics/flume/flume_metric_graph", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing individual graphs of Flume.
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceFlumeMetricGraph = App.ChartLinearTimeView.extend({
  /**
   * One of 'SOURCE', 'SINK' or 'CHANNEL'.
   */
  metricType: null,
  metricName: null,
  hostName: null,
  metricItems: null,

  id: Em.computed.format('service-metrics-flume-metric-graph-{0}-{1}', 'metricType', 'metricName'),

  title: Em.computed.alias('metricName'),

  ajaxIndex: 'host.host_component.flume.metrics.timeseries',

  getDataForAjaxRequest: function getDataForAjaxRequest() {
    var data = this._super();

    var urlFields = '';
    this.get('metricItems').forEach(function (metricItem, index) {
      urlFields += index === 0 ? '' : ',';
      urlFields += 'metrics/flume/flume/' + this.get('metricType') + '/' + metricItem + '/' + this.get('metricName') + '[' + data.fromSeconds + ',' + data.toSeconds + ',' + data.stepSeconds + ']';
    }, this);

    data.url = App.get('apiPrefix') + '/clusters/' + App.get('clusterName') + '/hosts/' + this.get('hostName') + '/host_components/FLUME_HANDLER?fields=' + urlFields;
    return data;
  },

  seriesTemplate: {
    path: 'metrics.flume.flume'
  },

  getData: function getData(jsonData) {

    var dataArray = [],
        data = Em.get(jsonData, this.get('seriesTemplate.path') + '.' + this.get('metricType'));
    if (data) {
      for (var componentName in data) {
        var seriesData = data[componentName][this.get('metricName')];
        if (seriesData) {
          dataArray.push({
            name: componentName,
            data: seriesData
          });
        }
      }
    }
    return dataArray;
  }
});

});

require.register("views/main/service/info/metrics/flume/flume_metric_graphs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
require('views/main/service/service');

App.MainServiceInfoFlumeGraphsView = App.MainServiceInfoSummaryMetricGraphsView.extend({

  serviceMetricGraphs: [],

  loadMetrics: function () {
    var graphRows = [];
    var viewData = this.get('viewData');
    if (viewData != null) {
      var metricType = viewData.metricType;
      var hostName = viewData.agent.get('hostName');
      App.ajax.send({
        'name': 'host.host_component.flume.metrics',
        'sender': this,
        'success': 'onLoadMetricsSuccess',
        'data': {
          hostName: hostName,
          flumeComponent: metricType
        }
      });
    }
    return graphRows;
  }.observes('viewData', 'metricType'),

  onLoadMetricsSuccess: function onLoadMetricsSuccess(data) {
    var graphRows = [];
    var viewData = this.get('viewData');
    var metricType = viewData.metricType;
    var hostName = viewData.agent.get('hostName');
    var metricNames = {};
    var metricItems = [];
    if (data != null && data.metrics != null && data.metrics.flume != null && data.metrics.flume.flume != null && data.metrics.flume.flume[metricType] != null) {
      for (var name in data.metrics.flume.flume[metricType]) {
        if (data.metrics.flume.flume[metricType].hasOwnProperty(name)) {
          for (var metricName in data.metrics.flume.flume[metricType][name]) {
            if (data.metrics.flume.flume[metricType][name].hasOwnProperty(metricName)) {
              metricNames[metricName] = name;
            }
          }
          metricItems.push(name);
        }
      }
    }
    // Now that we have collected all metric names, we create
    // views for each of them and store them 4 in a row.
    graphRows.push([]);
    var graphs = graphRows[0];
    for (var metricName in metricNames) {
      if (graphs.length > 1) {
        graphRows.push([]);
        graphs = graphRows[graphRows.length - 1];
      }
      graphs.push(App.ChartServiceFlumeMetricGraph.extend({
        metricType: metricType,
        metricName: metricName,
        hostName: hostName,
        metricItems: metricItems
      }));
    }
    this.set('serviceMetricGraphs', graphRows);
  },

  didInsertElement: function didInsertElement() {
    this.loadMetrics();
  }

});

});

require.register("views/main/service/info/metrics/flume/flume_outgoing_mma", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_OutgoingMMA = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-outgoing_mma",
  renderer: 'line',
  title: Em.I18n.t('services.service.info.metrics.flume.outgoing.mma'),

  ajaxIndex: 'service.metrics.flume.outgoing_event_take_success_count.mma',
  displayUnit: '/s',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.EventTakeSuccessCount.rate',
    displayName: function displayName(name) {
      return Em.I18n.t('services.service.info.metrics.flume.outgoing_mma').format(name);
    }
  },

  colorForSeries: function colorForSeries(series) {
    var name = Em.I18n.t('services.service.info.metrics.flume.outgoing_mma');
    if (name.format("avg") == series.name) {
      return '#0066b3';
    } else if (name.format("min") == series.name) {
      return '#00CC00';
    } else if (name.format("max") == series.name) {
      return '#FF8000';
    }
    return null;
  }
});

});

require.register("views/main/service/info/metrics/flume/flume_outgoing_sum", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_OutgoingSum = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-outgoing_sum",
  title: Em.I18n.t('services.service.info.metrics.flume.outgoing.sum'),

  ajaxIndex: 'service.metrics.flume.outgoing_event_take_success_count.sum',

  seriesTemplate: {
    path: 'metrics.flume.flume.CHANNEL.EventTakeSuccessCount.rate',
    displayName: function displayName() {
      return Em.I18n.t('services.service.info.metrics.flume.outgoing.sum');
    }
  }
});

});

require.register("views/main/service/info/metrics/flume/gc", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing HBase Cluster Requests
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_GarbageCollection = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-jvm-gc",
  title: Em.I18n.t('services.service.info.metrics.flume.gc'),
  displayUnit: 'ms',

  ajaxIndex: 'service.metrics.flume.gc',

  seriesTemplate: {
    path: 'metrics.jvm.gcTimeMillis',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.hostName').format(hostName);
    }
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/jvm_heap", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing HBase Cluster Requests
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_JVMHeapUsed = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-jvm-heap-used",
  title: Em.I18n.t('services.service.info.metrics.flume.jvmHeapUsed'),
  displayUnit: 'B',
  renderer: 'line',

  ajaxIndex: 'service.metrics.flume.jvm_heap_used',

  seriesTemplate: {
    path: 'metrics.jvm.memHeapUsedM',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.hostName').format(hostName);
    },
    factor: Math.pow(2, 20)
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/jvm_threads_runnable", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 * 
 * This is a view for showing HBase Cluster Requests
 * 
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_JVMThreadsRunnable = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-jvm-threads-runnable",
  title: Em.I18n.t('services.service.info.metrics.flume.jvmThreadsRunnable'),
  renderer: 'line',

  ajaxIndex: 'service.metrics.flume.jvm_threads_runnable',

  seriesTemplate: {
    path: 'metrics.jvm.threadsRunnable',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.hostName').format(hostName);
    }
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/sink_connection_failed", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase Cluster Requests
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_SinkConnectionFailedCount = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-sink-connection-failed",
  title: Em.I18n.t('services.service.info.metrics.flume.sinkConnectionFailed'),

  ajaxIndex: 'service.metrics.flume.sink_connection_failed',

  seriesTemplate: {
    path: 'metrics.flume.flume.SINK',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.sinkName').format(name + ' (' + hostName + ')');
    },
    flumePropertyName: 'ConnectionFailedCount'
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/sink_drain_success", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase Cluster Requests
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_SinkDrainSuccessCount = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-sink-drain-success",
  title: Em.I18n.t('services.service.info.metrics.flume.sinkDrainSuccess'),

  ajaxIndex: 'service.metrics.flume.sink_drain_success',

  seriesTemplate: {
    path: 'metrics.flume.flume.SINK',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.sinkName').format(name + ' (' + hostName + ')');
    },
    flumePropertyName: 'EventDrainSuccessCount'
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics/flume/source_accepted", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

/**
 * @class
 *
 * This is a view for showing HBase Cluster Requests
 *
 * @extends App.ChartLinearTimeView
 * @extends Ember.Object
 * @extends Ember.View
 */
App.ChartServiceMetricsFlume_SourceAcceptedCount = App.ChartLinearTimeView.extend({
  id: "service-metrics-flume-source-accepted",
  title: Em.I18n.t('services.service.info.metrics.flume.sourceAccepted'),

  ajaxIndex: 'service.metrics.flume.source_accepted',

  seriesTemplate: {
    path: 'metrics.flume.flume.SOURCE',
    displayName: function displayName(name, hostName) {
      return Em.I18n.t('services.service.info.metrics.flume.sourceName').format(name + ' (' + hostName + ')');
    },
    flumePropertyName: 'EventAcceptedCount'
  },

  getData: function getData(jsonData) {
    return this.getFlumeData(jsonData);
  }
});

});

require.register("views/main/service/info/metrics_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var misc = require('utils/misc');
require('views/main/service/service');
require('data/service_graph_config');

App.MainServiceInfoMetricsView = Em.View.extend(App.Persist, App.TimeRangeMixin, {
  templateName: require('templates/main/service/info/metrics'),
  /**
   * @property {Number} chunkSize - number of columns in Metrics section
   */
  chunkSize: 5,

  service: null,

  svc: function () {
    var svc = this.get('controller.content');
    var svcName = svc.get('serviceName');
    if (svcName) {
      switch (svcName.toLowerCase()) {
        case 'hdfs':
          svc = App.HDFSService.find().objectAt(0);
          break;
        case 'yarn':
          svc = App.YARNService.find().objectAt(0);
          break;
        case 'hbase':
          svc = App.HBaseService.find().objectAt(0);
          break;
        case 'flume':
          svc = App.FlumeService.find().objectAt(0);
          break;
        default:
          break;
      }
    }
    return svc;
  }.property('controller.content.serviceName').volatile(),

  getServiceModel: function getServiceModel(serviceName) {
    var extended = App.Service.extendedModel[serviceName];
    if (extended) {
      return App[extended].find().objectAt(0);
    }
    return App.Service.find(serviceName);
  },

  serviceName: Em.computed.alias('service.serviceName'),

  /**
   * Contains graphs for this particular service
   */
  serviceMetricGraphs: [],

  /**
   * @type {boolean}
   * @default false
   */
  serviceHasMetrics: false,

  /**
   * Key-name to store time range in Persist
   * @type {string}
   */
  persistKey: Em.computed.format('time-range-service-{0}', 'service.serviceName'),

  isLoading: false,

  didInsertElement: function didInsertElement() {
    var svcName = this.get('controller.content.serviceName');
    this.set('service', this.getServiceModel(svcName));
    var isMetricsSupported = svcName !== 'STORM' || App.get('isStormMetricsSupported');

    this.loadActiveWidgetLayout();

    if (App.get('supports.customizedWidgetLayout')) {
      this.get('controller').loadWidgetLayouts();
    }

    if (svcName && isMetricsSupported) {
      var allServices = require('data/service_graph_config');
      this.constructGraphObjects(allServices[svcName.toLowerCase()]);
    }
    this.makeSortable('#widget_layout');
    this.makeSortable('#ns_widget_layout', true);
    this.addWidgetTooltip();
    this._super();
  },

  loadActiveWidgetLayout: function () {
    var isHDFS = this.get('controller.content.serviceName') === 'HDFS';
    if (!this.get('isLoading') && (!isHDFS || isHDFS && App.router.get('clusterController.isHostComponentMetricsLoaded'))) {
      this.set('isLoading', true);
      this.get('controller').getActiveWidgetLayout();
    }
  }.observes('App.router.clusterController.isHostComponentMetricsLoaded'),

  addWidgetTooltip: function addWidgetTooltip() {
    Em.run.later(this, function () {
      App.tooltip($("[rel='add-widget-tooltip']"));
      // enable description show up on hover
      $('.img-thumbnail').hoverIntent(function () {
        if ($(this).is('hover')) {
          $(this).find('.hidden-description').delay(1000).fadeIn(200).end();
        }
      }, function () {
        $(this).find('.hidden-description').stop().hide().end();
      });
    }, 1000);
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='add-widget-tooltip']").tooltip('destroy');
    $('.img-thumbnail').off();
    $('#widget_layout').sortable('destroy');
    $('#ns_widget_layout').sortable('destroy');
    $('.widget.span2p4').detach().remove();
    this.get('serviceMetricGraphs').clear();
    this.set('service', null);
  },

  /*
  * Find the graph class associated with the graph name, and split
  * the array into sections of 5 for displaying on the page
  * (will only display rows with 5 items)
  */
  constructGraphObjects: function constructGraphObjects(graphNames) {
    var self = this,
        stackService = App.StackService.find(this.get('controller.content.serviceName'));

    if (!graphNames && !stackService.get('isServiceWithWidgets')) {
      this.get('serviceMetricGraphs').clear();
      this.set('serviceHasMetrics', false);
      return;
    }

    // load time range(currentTimeRangeIndex) for current service from server
    this.getUserPref(self.get('persistKey')).complete(function () {
      var result = [],
          graphObjects = [],
          chunkSize = self.get('chunkSize');
      if (graphNames) {
        graphNames.forEach(function (graphName) {
          graphObjects.push(App["ChartServiceMetrics" + graphName].extend());
        });
      }
      while (graphObjects.length) {
        result.push(graphObjects.splice(0, chunkSize));
      }
      self.set('serviceMetricGraphs', result);
      self.set('serviceHasMetrics', true);
    });
  },

  getUserPrefSuccessCallback: function getUserPrefSuccessCallback(response, request) {
    if (response) {
      this.set('currentTimeRangeIndex', response);
    }
  },

  getUserPrefErrorCallback: function getUserPrefErrorCallback(request) {
    if (request.status === 404) {
      this.postUserPref(this.get('persistKey'), 0);
      this.set('currentTimeRangeIndex', 0);
    }
  },

  /**
   * list of static actions of widget
   * @type {Array}
   */
  staticGeneralWidgetActions: [Em.Object.create({
    label: Em.I18n.t('dashboard.widgets.actions.browse'),
    class: 'glyphicon glyphicon-th',
    action: 'goToWidgetsBrowser',
    isAction: true
  })],

  /**
   *list of static actions of widget accessible to Admin/Operator privelege
   * @type {Array}
   */

  staticAdminPrivelegeWidgetActions: [Em.Object.create({
    label: Em.I18n.t('dashboard.widgets.create'),
    class: 'glyphicon glyphicon-plus',
    action: 'createWidget',
    isAction: true
  })],

  /**
   * List of static actions related to widget layout
   */
  staticWidgetLayoutActions: [Em.Object.create({
    label: Em.I18n.t('dashboard.widgets.layout.save'),
    class: 'glyphicon glyphicon-download-alt',
    action: 'saveLayout',
    isAction: true
  }), Em.Object.create({
    label: Em.I18n.t('dashboard.widgets.layout.import'),
    class: 'glyphicon glyphicon-file',
    isAction: true,
    layouts: App.WidgetLayout.find()
  })],

  /**
   * @type {Array}
   */
  widgetActions: function () {
    var options = [];
    if (App.isAuthorized('SERVICE.MODIFY_CONFIGS')) {
      if (App.supports.customizedWidgetLayout) {
        options.pushObjects(this.get('staticWidgetLayoutActions'));
      }
      options.pushObjects(this.get('staticAdminPrivelegeWidgetActions'));
    }
    options.pushObjects(this.get('staticGeneralWidgetActions'));
    return options;
  }.property(''),

  /**
   * call action function defined in controller
   * @param event
   */
  doWidgetAction: function doWidgetAction(event) {
    if ($.isFunction(this.get('controller')[event.context])) {
      this.get('controller')[event.context].apply(this.get('controller'));
    }
  },

  /**
   * onclick handler for a time range option
   * @param {object} event
   */
  setTimeRange: function setTimeRange(event) {
    var graphs = this.get('controller.widgets').filterProperty('widgetType', 'GRAPH'),
        callback = function callback() {
      graphs.forEach(function (widget) {
        widget.set('properties.time_range', event.context.value);
      });
    };
    this._super(event, callback);

    // Preset time range is specified by user
    if (event.context.value !== '0') {
      callback();
    }
  },

  /**
   * Define if some widget is currently moving
   * @type {boolean}
   */
  isMoving: false,

  /**
   * Make widgets' list sortable on New Dashboard style
   */
  makeSortable: function makeSortable(selector, isNSLayout) {
    var self = this;
    var controller = this.get('controller');
    $('html').on('DOMNodeInserted', selector, function () {
      $(this).sortable({
        items: "> div",
        cursor: "move",
        tolerance: "pointer",
        scroll: false,
        update: function update() {
          var layout = isNSLayout ? controller.get('selectedNSWidgetLayout') : controller.get('activeWidgetLayout');
          var widgets = misc.sortByOrder($(selector + " .widget").map(function () {
            return this.id;
          }), layout.get('widgets').toArray());
          controller.saveWidgetLayout(widgets, layout);
        },
        activate: function activate() {
          self.set('isMoving', true);
        },
        deactivate: function deactivate() {
          self.set('isMoving', false);
        }
      }).disableSelection();
      $('html').off('DOMNodeInserted', selector);
    });
  }
});

});

require.register("views/main/service/info/summary", function(exports, require, module) {
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var batchUtils = require('utils/batch_scheduled_requests');
var misc = require('utils/misc');
require('views/main/service/service');
require('data/service_graph_config');

App.MainServiceInfoSummaryView = Em.View.extend({
  templateName: require('templates/main/service/info/summary'),

  attributes: null,

  /**
   * Contain array with list of groups of master components from <code>App.Service.hostComponets</code> which are
   * <code>App.HostComponent</code> models.
   * @type {{title: String, hosts: String[], components: App.HostComponent[]}[]}
   */
  mastersObj: [{
    components: []
  }],
  mastersLength: 0,

  /**
   * Contain array with list of slave components models <code>App.SlaveComponent</code>.
   * @type {App.SlaveComponent[]}
   */
  slavesObj: [],
  slavesLength: 0,

  /**
   * Contain array with list of client components models <code>App.ClientComponent</code>.
   * @type {App.ClientComponent[]}
   */
  clientObj: [],
  clientsLength: 0,

  /**
   *  @property {String} templatePathPrefix - base path for custom templates
   *    if you want to add custom template, add <service_name>.hbs file to
   *    templates/main/service/info/summary folder.
   */
  templatePathPrefix: 'templates/main/service/info/summary/',
  /** @property {Ember.View} serviceSummaryView - view to embed, computed in
   *  <code>loadServiceSummary()</code>
   */
  serviceSummaryView: null,

  // TODO implement filtering of views related to current service
  ///**
  // * @type {App.ViewInstance}
  // */
  /*views: function () {
    return App.router.get('loggedIn') ? App.router.get('mainViewsController.visibleAmbariViews') : [];
  }.property('App.router.mainViewsController.visibleAmbariViews.[]', 'App.router.loggedIn'),*/

  /**
   * @property {Object} serviceCustomViewsMap - custom views to embed
   *
   */
  serviceCustomViewsMap: function () {
    return {
      HBASE: App.MainDashboardServiceHbaseView,
      HDFS: App.MainDashboardServiceHdfsView,
      ONEFS: App.MainDashboardServiceOnefsView,
      STORM: App.MainDashboardServiceStormView,
      YARN: App.MainDashboardServiceYARNView,
      RANGER: App.MainDashboardServiceRangerView,
      FLUME: App.MainDashboardServiceFlumeView,
      HIVE: App.MainDashboardServiceHiveView
    };
  }.property('serviceName'),
  /** @property collapsedMetrics {object[]} - metrics list for collapsed section
   *    structure of element from list:
   *      @property {string} header - title for section
   *      @property {string} id - id of section for toggling, like: metric1
   *      @property {string} toggleIndex - passed to `data-parent` attribute, like: #metric1
   *      @property {Em.View} metricView - metric view class
   */
  collapsedSections: null,

  servicesHaveClients: Em.computed.alias('App.services.hasClient'),

  hasManyServers: Em.computed.gt('servers.length', 1),

  clientsHostText: function () {
    if (this.get('controller.content.installedClients').length === 0) {
      return '';
    }
    if (this.get("hasManyClients")) {
      return Em.I18n.t('services.service.summary.viewHosts');
    }
    return Em.I18n.t('services.service.summary.viewHost');
  }.property("hasManyClients"),

  hasManyClients: Em.computed.gt('controller.content.installedClients.length', 1),

  servers: function () {
    var result = [];
    var service = this.get('controller.content');
    if (service.get("id") === "ZOOKEEPER" || service.get("id") === "FLUME") {
      var servers = service.get('hostComponents').filterProperty('isMaster');
      if (servers.length > 0) {
        result = [{
          'host': servers[0].get('displayName'),
          'isComma': false,
          'isAnd': false
        }];
      }
      if (servers.length > 1) {
        result[0].isComma = true;
        result.push({
          'host': servers[1].get('displayName'),
          'isComma': false,
          'isAnd': false
        });
      }
      if (servers.length > 2) {
        result[1].isAnd = true;
        result.push({
          'host': Em.I18n.t('services.service.info.summary.serversHostCount').format(servers.length - 2),
          'isComma': false,
          'isAnd': false
        });
      }
    }
    return result;
  }.property('controller.content'),

  /**
   * Property related to ZOOKEEPER service, is unused for other services
   * @return {Object}
   */
  serversHost: function () {
    var service = this.get('controller.content');
    if (service.get("id") === "ZOOKEEPER" || service.get("id") === "FLUME") {
      var servers = service.get('hostComponents').filterProperty('isMaster');
      if (servers.length > 0) {
        return servers[0];
      }
    }
    return {};
  }.property('controller.content'),

  showComponentsTitleForNonMasters: Em.computed.or('!mastersLength', 'hasMultipleMasterGroups'),

  componentsLengthDidChange: function () {
    var self = this;
    if (!this.get('service') || this.get('service.deleteInProgress')) return;
    Em.run.once(self, 'setComponentsContent');
  }.observes('service.hostComponents.length', 'service.slaveComponents.@each.totalCount', 'service.clientComponents.@each.totalCount', 'svc.masterComponentGroups.length'),

  loadServiceSummary: function () {
    var serviceName = this.get('serviceName');
    var serviceSummaryView = null;

    if (!serviceName) {
      return;
    }

    if (this.get('oldServiceName')) {
      // do not delete it!
      return;
    }

    var customServiceView = this.get('serviceCustomViewsMap')[serviceName];
    if (customServiceView) {
      serviceSummaryView = customServiceView.extend({
        service: this.get('service')
      });
    } else {
      serviceSummaryView = Em.View.extend(App.MainDashboardServiceViewWrapper, {
        templateName: this.get('templatePathPrefix') + 'base'
      });
    }
    this.set('serviceSummaryView', serviceSummaryView);
    this.set('oldServiceName', serviceName);
  }.observes('serviceName'),

  didInsertElement: function didInsertElement() {
    this._super();
    var svcName = this.get('controller.content.serviceName');
    this.set('service', this.getServiceModel(svcName));
    App.loadTimer.finish('Service Summary Page');
  },

  willDestroyElement: function willDestroyElement() {
    this.set('service', null);
    this.get('mastersObj').clear();
    this.get('slavesObj').clear();
    this.get('clientObj').clear();
  },

  setComponentsContent: function setComponentsContent() {
    Em.run.next(function () {
      if (Em.isNone(this.get('service'))) {
        return;
      }
      var masters = this.get('service.hostComponents').filterProperty('isMaster'),
          slaves = this.get('service.slaveComponents').toArray(),
          clients = this.get('service.clientComponents').toArray(),
          masterGroups = this.get('svc.masterComponentGroups') ? this.get('svc.masterComponentGroups').toArray() : [];

      if (this.get('mastersLength') !== masters.length || this.get('mastersObj.length') !== masterGroups.length) {
        var mastersInit = this.get('mastersObj').mapProperty('components').reduce(function (acc, group) {
          return [].concat(_toConsumableArray(acc), _toConsumableArray(group));
        }, []);
        this.updateComponentList(mastersInit, masters);
        this.set('mastersObj', this.getGroupedMasterComponents(mastersInit));
        this.set('mastersLength', masters.length);
      }
      if (this.get('slavesLength') !== slaves.length) {
        this.updateComponentList(this.get('slavesObj'), slaves);
        this.set('slavesLength', slaves.length);
      }
      if (this.get('clientsLength') !== clients.length) {
        this.updateComponentList(this.get('clientObj'), clients);
        this.set('clientsLength', clients.length);
      }
    }.bind(this));
  },

  updateComponentList: function updateComponentList(source, data) {
    var sourceIds = source.mapProperty('id');
    var dataIds = data.mapProperty('id');
    if (sourceIds.length === 0) {
      source.pushObjects(data);
    }
    if (source.length > data.length) {
      sourceIds.forEach(function (item) {
        if (!dataIds.contains(item)) {
          var sourceItem = source.findProperty('id', item);
          source.removeObject(sourceItem);
        }
      });
    } else if (source.length < data.length) {
      dataIds.forEach(function (item, index) {
        if (!sourceIds.contains(item)) {
          source.pushObject(data.objectAt(index));
        }
      });
    }
  },

  data: {
    hive: {
      "database": "PostgreSQL",
      "databaseName": "hive",
      "user": "hive"
    }
  },

  /**
   * @type {Em.View}
   * Wrapper for displayName. used to render correct display name for mysql_server
   */
  componentNameView: Ember.View.extend({
    template: Ember.Handlebars.compile('{{view.displayName}}'),
    comp: null,
    displayName: function () {
      if (this.get('comp.componentName') === 'MYSQL_SERVER') {
        return this.t('services.hive.databaseComponent');
      }
      return this.get('comp.displayName');
    }.property('comp')
  }),

  service: null,

  svc: function () {
    var svc = this.get('controller.content');
    var svcName = svc ? svc.get('serviceName') : null;
    if (svcName) {
      switch (svcName.toLowerCase()) {
        case 'hdfs':
          svc = App.HDFSService.find().objectAt(0);
          break;
        case 'yarn':
          svc = App.YARNService.find().objectAt(0);
          break;
        case 'hbase':
          svc = App.HBaseService.find().objectAt(0);
          break;
        case 'flume':
          svc = App.FlumeService.find().objectAt(0);
          break;
        default:
          break;
      }
    }
    return svc;
  }.property('controller.content.serviceName').volatile(),

  getServiceModel: function getServiceModel(serviceName) {
    var extended = App.Service.extendedModel[serviceName];
    if (extended) {
      return App[extended].find().objectAt(0);
    }
    return App.Service.find(serviceName);
  },

  /**
   * @type {boolean}
   * @default true
   */
  isHide: true,

  /**
   * @type {Em.View}
   */
  moreStatsView: Em.View.extend({
    tagName: "a",
    template: Ember.Handlebars.compile('{{t services.service.summary.moreStats}}'),
    attributeBindings: ['href'],
    classNames: ['more-stats'],
    click: function click() {
      this._parentView._parentView.set('isHide', false);
      this.remove();
    },
    href: 'javascript:void(null)'
  }),

  serviceName: Em.computed.alias('service.serviceName'),

  oldServiceName: '',

  /*
   * 'Restart Required bar' start
   */
  componentsCount: null,
  hostsCount: null,

  alertsCount: Em.computed.alias('controller.content.alertsCount'),

  hasNoAlerts: Em.computed.equal('alertsCount', 0),

  hasCriticalAlerts: Em.computed.alias('controller.content.hasCriticalAlerts'),

  hasNonCriticalAlertsOnly: Em.computed.and('!hasNoAlerts', '!hasCriticalAlerts'),

  /**
   * Define if service has alert definitions defined
   * @type {Boolean}
   */
  hasAlertDefinitions: function () {
    return App.AlertDefinition.find().someProperty('serviceName', this.get('controller.content.serviceName'));
  }.property('controller.content.serviceName'),

  updateComponentInformation: function () {
    var hc = this.get('controller.content.restartRequiredHostsAndComponents');
    var hostsCount = 0;
    var componentsCount = 0;
    for (var host in hc) {
      hostsCount++;
      componentsCount += hc[host].length;
    }
    this.set('componentsCount', componentsCount);
    this.set('hostsCount', hostsCount);
  }.observes('controller.content.restartRequiredHostsAndComponents'),

  rollingRestartSlaveComponentName: function () {
    return batchUtils.getRollingRestartComponentName(this.get('serviceName'));
  }.property('serviceName'),

  rollingRestartActionName: function () {
    var label = null;
    var componentName = this.get('rollingRestartSlaveComponentName');
    if (componentName) {
      label = Em.I18n.t('rollingrestart.dialog.title').format(App.format.role(componentName, false));
    }
    return label;
  }.property('rollingRestartSlaveComponentName'),

  restartAllStaleConfigComponents: function restartAllStaleConfigComponents() {
    var self = this;
    var serviceDisplayName = this.get('service.displayName');
    var bodyMessage = Em.Object.create({
      confirmMsg: Em.I18n.t('services.service.restartAll.confirmMsg').format(serviceDisplayName),
      confirmButton: Em.I18n.t('services.service.restartAll.confirmButton'),
      additionalWarningMsg: this.get('service.passiveState') === 'OFF' ? Em.I18n.t('services.service.restartAll.warningMsg.turnOnMM').format(serviceDisplayName) : null
    });

    var isNNAffected = false;
    var restartRequiredHostsAndComponents = this.get('controller.content.restartRequiredHostsAndComponents');
    for (var hostName in restartRequiredHostsAndComponents) {
      restartRequiredHostsAndComponents[hostName].forEach(function (hostComponent) {
        if (hostComponent === 'NameNode') isNNAffected = true;
      });
    }
    if (serviceDisplayName === 'HDFS' && isNNAffected && this.get('controller.content.hostComponents').filterProperty('componentName', 'NAMENODE').someProperty('workStatus', App.HostComponentStatus.started)) {
      App.router.get('mainServiceItemController').checkNnLastCheckpointTime(function () {
        return App.showConfirmationFeedBackPopup(function (query) {
          var selectedService = self.get('service.id');
          batchUtils.restartAllServiceHostComponents(serviceDisplayName, selectedService, true, query);
        }, bodyMessage);
      });
    } else {
      return App.showConfirmationFeedBackPopup(function (query) {
        var selectedService = self.get('service.id');
        batchUtils.restartAllServiceHostComponents(serviceDisplayName, selectedService, true, query);
      }, bodyMessage);
    }
  },

  rollingRestartStaleConfigSlaveComponents: function rollingRestartStaleConfigSlaveComponents(componentName) {
    batchUtils.launchHostComponentRollingRestart(componentName.context, this.get('service.displayName'), this.get('service.passiveState') === "ON", true);
  },

  hasMultipleMasterGroups: Em.computed.gt('mastersObj.length', 1),

  getGroupedMasterComponents: function getGroupedMasterComponents(components) {
    switch (this.get('serviceName')) {
      case 'HDFS':
        var masterComponentGroups = this.get('service.masterComponentGroups'),
            hostComponents = this.get('service.hostComponents'),
            zkfcs = hostComponents.filterProperty('componentName', 'ZKFC'),
            hasNameNodeFederation = this.get('service.hasMultipleMasterComponentGroups');
        var groups = [];
        hostComponents.forEach(function (component) {
          if (component.get('isMaster') && component.get('componentName') !== 'JOURNALNODE') {
            var hostName = component.get('hostName'),
                zkfc = zkfcs.findProperty('hostName', hostName);
            if (hasNameNodeFederation) {
              var name = component.get('haNameSpace'),
                  existingGroup = groups.findProperty('name', name),
                  currentGroup = existingGroup || _extends({}, masterComponentGroups.findProperty('name', name));
              if (!existingGroup) {
                groups.push(currentGroup);
                Em.setProperties(currentGroup, {
                  components: [],
                  componentWidgetsView: App.HDFSSummaryWidgetsView.extend({
                    nameSpace: name
                  })
                });
              }
              currentGroup.components.push(component);
              if (zkfc) {
                zkfc.set('isSubComponent', true);
                currentGroup.components.push(zkfc);
              }
            } else {
              if (!groups.length) {
                groups.push({
                  components: [],
                  componentWidgetsView: App.HDFSSummaryWidgetsView.extend({
                    nameSpace: component.get('haNameSpace') || 'default',
                    showSlaveComponents: true
                  })
                });
              }
              var defaultGroupComponents = groups[0].components;
              defaultGroupComponents.push(component);
              if (zkfc) {
                zkfc.set('isSubComponent', true);
                defaultGroupComponents.push(zkfc);
              }
            }
          }
        });
        return groups.sortProperty('name');
      default:
        return [{
          components: components
        }];
    }
  }
});

});

require.register("views/main/service/info/summary/hdfs/common_widgets", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var numberUtils = require('utils/number_utils');

function diskPart(i18nKey, totalKey, usedKey) {
  return Em.computed(totalKey, usedKey, function () {
    var text = Em.I18n.t(i18nKey),
        total = this.get(totalKey),
        used = this.get(usedKey);
    return text.format(numberUtils.bytesToSize(used, 1, 'parseFloat'), numberUtils.bytesToSize(total, 1, 'parseFloat'));
  });
}

function diskPartPercent(i18nKey, totalKey, usedKey) {
  return Em.computed(totalKey, usedKey, function () {
    var text = Em.I18n.t(i18nKey),
        total = this.get(totalKey),
        used = this.get(usedKey);
    var percent = total > 0 ? (used * 100 / total).toFixed(2) : 0;
    if (percent == 'NaN' || percent < 0) {
      percent = Em.I18n.t('services.service.summary.notAvailable') + ' ';
    }
    return text.format(percent);
  });
}

/**
 * Metric widgets which are common for all namespaces
 */
App.HDFSSummaryCommonWidgetsView = Em.View.extend(App.HDFSSummaryWidgetsMixin, {

  templateName: require('templates/main/service/info/summary/hdfs/common_widgets'),

  dfsUsedDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'model.capacityTotal', 'model.capacityUsed'),

  dfsUsedDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'model.capacityTotal', 'model.capacityUsed'),

  nonDfsUsed: function () {
    var total = this.get('model.capacityTotal'),
        remaining = this.get('model.capacityRemaining'),
        dfsUsed = this.get('model.capacityUsed');
    return Em.isNone(total) || Em.isNone(remaining) || Em.isNone(dfsUsed) ? null : total - remaining - dfsUsed;
  }.property('model.capacityTotal', 'model.capacityRemaining', 'model.capacityUsed'),

  nonDfsUsedDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'model.capacityTotal', 'nonDfsUsed'),

  nonDfsUsedDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'model.capacityTotal', 'nonDfsUsed'),

  remainingDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'model.capacityTotal', 'model.capacityRemaining'),

  remainingDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'model.capacityTotal', 'model.capacityRemaining')

});

});

require.register("views/main/service/info/summary/hdfs/slaves", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.HDFSSlaveComponentsView = Em.View.extend({
  templateName: require('templates/main/service/info/summary/hdfs/slaves'),

  showTitle: false,

  // should be bound to App.MainDashboardServiceHdfsView instance
  summaryView: null,

  dataNodeComponent: Em.Object.create({
    componentName: 'DATANODE'
  }),

  journalNodeComponent: Em.Object.create({
    componentName: 'JOURNALNODE'
  }),

  nfsGatewayComponent: Em.Object.create({
    componentName: 'NFS_GATEWAY'
  }),

  service: Em.computed.alias('summaryView.service'),

  isDataNodeCreated: function () {
    return this.get('summaryView') ? this.get('summaryView').isServiceComponentCreated('DATANODE') : false;
  }.property('App.router.clusterController.isComponentsStateLoaded'),

  isJournalNodeCreated: function () {
    return this.get('summaryView') ? this.get('summaryView').isServiceComponentCreated('JOURNALNODE') : false;
  }.property('App.router.clusterController.isComponentsStateLoaded'),

  journalNodesLive: function () {
    return this.get('service.journalNodes').filterProperty('workStatus', 'STARTED').get('length');
  }.property('service.journalNodes.@each.workStatus'),

  journalNodesTotal: Em.computed.alias('service.journalNodes.length'),

  /**
   * Define if NFS_GATEWAY is present in the installed stack
   * @type {Boolean}
   */
  isNfsInStack: function () {
    return App.StackServiceComponent.find().someProperty('componentName', 'NFS_GATEWAY');
  }.property()
});

});

require.register("views/main/service/info/summary/hdfs/widgets", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var date = require('utils/date/date');

/**
 * Metric widgets which are specific for different namespaces
 */
App.HDFSSummaryWidgetsView = Em.View.extend(App.NameNodeWidgetMixin, App.HDFSSummaryWidgetsMixin, {

  templateName: require('templates/main/service/info/summary/hdfs/widgets'),

  nameSpace: 'default',

  subGroupId: Em.computed.alias('nameSpace'),

  showSlaveComponents: false,

  // should be bound to App.HDFSSlaveComponentsView instance
  slaveComponentsView: null,

  nodeUptime: function () {
    var uptime = this.get('model.nameNodeStartTimeValues')[this.get('hostName')];
    if (uptime && uptime > 0) {
      var diff = App.dateTime() - uptime;
      if (diff < 0) {
        diff = 0;
      }
      var formatted = date.timingFormat(diff);
      return this.t('dashboard.services.uptime').format(formatted);
    }
    return this.t('services.service.summary.notRunning');
  }.property('model.nameNodeStartTimeValues'),

  jvmMemoryHeapUsed: Em.computed.getByKey('model.jvmMemoryHeapUsedValues', 'hostName'),

  jvmMemoryHeapMax: Em.computed.getByKey('model.jvmMemoryHeapMaxValues', 'hostName'),

  nodeHeapPercent: App.MainDashboardServiceView.formattedHeapPercent('dashboard.services.hdfs.nodes.heapUsedPercent', 'jvmMemoryHeapUsed', 'jvmMemoryHeapMax'),

  nodeHeap: App.MainDashboardServiceView.formattedHeap('dashboard.services.hdfs.nodes.heapUsed', 'jvmMemoryHeapUsed', 'jvmMemoryHeapMax'),

  dfsTotalBlocksValue: Em.computed.getByKey('model.dfsTotalBlocksValues', 'hostName'),

  dfsTotalBlocks: Em.computed.formatUnavailable('dfsTotalBlocksValue'),

  dfsCorruptBlocksValue: Em.computed.getByKey('model.dfsCorruptBlocksValues', 'hostName'),

  dfsCorruptBlocks: Em.computed.formatUnavailable('dfsCorruptBlocksValue'),

  dfsMissingBlocksValue: Em.computed.getByKey('model.dfsMissingBlocksValues', 'hostName'),

  dfsMissingBlocks: Em.computed.formatUnavailable('dfsMissingBlocksValue'),

  dfsUnderReplicatedBlocksValue: Em.computed.getByKey('model.dfsUnderReplicatedBlocksValues', 'hostName'),

  dfsUnderReplicatedBlocks: Em.computed.formatUnavailable('dfsUnderReplicatedBlocksValue'),

  dfsTotalFilesValue: Em.computed.getByKey('model.dfsTotalFilesValues', 'hostName'),

  dfsTotalFiles: Em.computed.formatUnavailable('dfsTotalFilesValue'),

  healthStatus: Em.computed.getByKey('model.healthStatusValues', 'hostName'),

  upgradeStatusValue: Em.computed.getByKey('model.upgradeStatusValues', 'hostName'),

  upgradeStatus: function () {
    var upgradeStatus = this.get('upgradeStatusValue'),
        healthStatus = this.get('healthStatus');
    if (upgradeStatus) {
      return Em.I18n.t('services.service.summary.pendingUpgradeStatus.notPending');
    } else if (upgradeStatus === false && healthStatus === 'green') {
      return Em.I18n.t('services.service.summary.pendingUpgradeStatus.notFinalized');
    } else {
      // upgrade status == null
      return Em.I18n.t('services.service.summary.notAvailable');
    }
  }.property('upgradeStatusValue', 'healthStatus'),

  isUpgradeStatusWarning: function () {
    return this.get('upgradeStatusValue') === false && this.get('healthStatus') === 'green';
  }.property('upgradeStatusValue', 'healthStatus'),

  safeModeStatusValue: Em.computed.getByKey('model.safeModeStatusValues', 'hostName'),

  safeModeStatus: function () {
    var safeMode = this.get('safeModeStatusValue');
    if (Em.isNone(safeMode)) {
      return Em.I18n.t('services.service.summary.notAvailable');
    } else if (safeMode.length === 0) {
      return Em.I18n.t('services.service.summary.safeModeStatus.notInSafeMode');
    } else {
      return Em.I18n.t('services.service.summary.safeModeStatus.inSafeMode');
    }
  }.property('safeModeStatusValue')

});

});

require.register("views/main/service/item", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceItemView = Em.View.extend(App.HiveInteractiveCheck, {
  templateName: require('templates/main/service/item'),

  serviceName: Em.computed.alias('controller.content.serviceName'),

  displayName: Em.computed.alias('controller.content.displayName'),

  isPassive: Em.computed.equal('controller.content.passiveState', 'ON'),

  /**
   * Some custom commands need custom logic to be execute.
   * Typically, these services already have Custom Commands, so we must exclude the default ones
   * in order to make more changes to them like icons and rules.
   */
  mastersExcludedCommands: {
    'NAMENODE': ['DECOMMISSION', 'REBALANCEHDFS'],
    'RESOURCEMANAGER': ['DECOMMISSION', 'REFRESHQUEUES'],
    'HBASE_MASTER': ['DECOMMISSION', 'UPDATE_REPLICATION', 'STOP_REPLICATION'],
    'KNOX_GATEWAY': ['STARTDEMOLDAP', 'STOPDEMOLDAP'],
    'HAWQMASTER': ['IMMEDIATE_STOP_HAWQ_SERVICE', 'RUN_HAWQ_CHECK', 'HAWQ_CLEAR_CACHE', 'REMOVE_HAWQ_STANDBY', 'RESYNC_HAWQ_STANDBY'],
    'HAWQSEGMENT': ['IMMEDIATE_STOP_HAWQ_SEGMENT'],
    'HAWQSTANDBY': ['ACTIVATE_HAWQ_STANDBY'],
    'HIVE_SERVER_INTERACTIVE': ["RESTART_LLAP"]
  },

  addActionMap: function addActionMap() {
    return [{
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.hbase.masterServer')),
      service: 'HBASE',
      component: 'HBASE_MASTER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.hive.metastore')),
      service: 'HIVE',
      component: 'HIVE_METASTORE'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.hive.server2')),
      service: 'HIVE',
      component: 'HIVE_SERVER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.hive.server2interactive')),
      service: 'HIVE',
      component: 'HIVE_SERVER_INTERACTIVE',
      dependsFromAnotherProperty: true,
      depend: this.get('enableHiveInteractive')
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.zookeeper.server')),
      service: 'ZOOKEEPER',
      component: 'ZOOKEEPER_SERVER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), Em.I18n.t('dashboard.services.flume.agentLabel')),
      service: 'FLUME',
      component: 'FLUME_HANDLER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('RANGER_KMS_SERVER', false)),
      service: 'RANGER_KMS',
      component: 'RANGER_KMS_SERVER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('NIMBUS', false)),
      service: 'STORM',
      component: 'NIMBUS'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('OOZIE_SERVER', false)),
      service: 'OOZIE',
      component: 'OOZIE_SERVER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('DRUID_BROKER', false)),
      service: 'DRUID',
      component: 'DRUID_BROKER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('DRUID_ROUTER', false)),
      service: 'DRUID',
      component: 'DRUID_ROUTER'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('DRUID_OVERLORD', false)),
      service: 'DRUID',
      component: 'DRUID_OVERLORD'
    }, {
      cssClass: 'glyphicon glyphicon-plus',
      'label': '{0} {1}'.format(Em.I18n.t('add'), App.format.role('DRUID_COORDINATOR', false)),
      service: 'DRUID',
      component: 'DRUID_COORDINATOR'
    }];
  },
  /**
   * Create option for MOVE_COMPONENT or ROLLING_RESTART task.
   *
   * @param {Object} option - one of the options that return by <code>App.HostComponentActionMap.getMap()</code>
   * @param {Object} fields - option fields to add/rewrite
   * @return {Object}
   */
  createOption: function createOption(option, fields) {
    return $.extend(true, {}, option, fields);
  },

  maintenance: [],

  isMaintenanceSet: false,

  tooltipAttribute: 'service-actions-tooltip',

  observeMaintenance: function observeMaintenance() {
    if (!this.get('isMaintenanceSet') && this.get('controller.isServicesInfoLoaded') && this.get('controller.isServiceConfigsLoaded')) {
      this.observeMaintenanceOnce();
    }
    Em.run.once(this, 'clearIsMaintenanceSet');
  },

  observeMaintenanceOnce: function observeMaintenanceOnce() {
    var self = this;
    var options = [];
    var service = this.get('controller.content');
    var allMasters = service.get('hostComponents').filterProperty('isMaster').mapProperty('componentName').uniq();
    var allSlaves = service.get('slaveComponents').rejectProperty('totalCount', 0).mapProperty('componentName');
    var actionMap = App.HostComponentActionMap.getMap(this);
    var serviceCheckSupported = App.get('services.supportsServiceCheck').contains(service.get('serviceName'));
    var hasConfigTab = this.get('hasConfigTab');
    var excludedCommands = this.get('mastersExcludedCommands');
    var serviceName = service.get('serviceName');
    var hasMultipleMasterComponentGroups = this.get('service.hasMultipleMasterComponentGroups');

    if (App.isAuthorized('SERVICE.START_STOP')) {
      options.push(actionMap.START_ALL);
      options.push(actionMap.STOP_ALL);
    }
    if (App.isAuthorized('SERVICE.RUN_CUSTOM_COMMAND, SERVICE.RUN_SERVICE_CHECK, SERVICE.TOGGLE_MAINTENANCE, SERVICE.ENABLE_HA')) {
      if (this.get('controller.isClientsOnlyService')) {
        if (serviceCheckSupported) {
          options.push(actionMap.RUN_SMOKE_TEST);
        }
        if (hasConfigTab) {
          options.push(actionMap.REFRESH_CONFIGS);
        }
      } else {
        if (this.get('serviceName') === 'FLUME') {
          options.push(actionMap.REFRESH_CONFIGS);
        }
        if (this.get('serviceName') === 'YARN') {
          options.push(actionMap.REFRESHQUEUES);
        }
        options.push(actionMap.RESTART_ALL);
        if (hasMultipleMasterComponentGroups && this.get('serviceName') === 'HDFS') {
          options.push(actionMap.RESTART_NAMENODES);
        }
        allSlaves.concat(allMasters).filter(function (_component) {
          return App.get('components.rollinRestartAllowed').contains(_component);
        }).forEach(function (_component) {
          var _componentNamePluralized = pluralize(App.format.role(_component, false));
          options.push(self.createOption(actionMap.ROLLING_RESTART, {
            context: _component,
            label: actionMap.ROLLING_RESTART.label.format(_componentNamePluralized)
          }));
        });
        allMasters.filter(function (master) {
          return App.get('components.reassignable').contains(master);
        }).forEach(function (master) {
          options.push(self.createOption(actionMap.MOVE_COMPONENT, {
            context: master,
            label: actionMap.MOVE_COMPONENT.label.format(App.format.role(master, false)),
            disabled: App.allHostNames.length === App.HostComponent.find().filterProperty('componentName', master).mapProperty('hostName').length
          }));
        });
        // add "Manage JournalNode" when NNHA is enabled and there is more hosts than JNs
        var JNCount = App.HostComponent.find().filterProperty('componentName', 'JOURNALNODE').get('length');
        if (App.get('supports.manageJournalNode') && service.get('serviceName') == 'HDFS' && service.get('serviceTypes').contains('HA_MODE') && (App.router.get('mainHostController.totalCount') > JNCount || JNCount > 3)) {
          options.push(actionMap.MANAGE_JN);
        }
        if (service.get('serviceTypes').contains('HA_MODE') && App.isAuthorized('SERVICE.ENABLE_HA')) {
          switch (service.get('serviceName')) {
            case 'HDFS':
              options.push(actionMap.TOGGLE_NN_HA);
              break;
            case 'YARN':
              options.push(actionMap.TOGGLE_RM_HA);
              break;
            case 'RANGER':
              options.push(actionMap.TOGGLE_RA_HA);
              break;
            case 'HAWQ':
              options.push(actionMap.TOGGLE_ADD_HAWQ_STANDBY);
              break;
          }
        }
        if (service.get('serviceTypes').contains('FEDERATION') && App.isAuthorized('SERVICE.ENABLE_HA')) {
          switch (service.get('serviceName')) {
            case 'HDFS':
              options.push(actionMap.TOGGLE_NN_FEDERATION);
              break;
          }
        }
        if (serviceCheckSupported) {
          options.push(actionMap.RUN_SMOKE_TEST);
        }
        options.push(actionMap.TOGGLE_PASSIVE);
        var nnComponent = App.StackServiceComponent.find().findProperty('componentName', 'NAMENODE');
        var knoxGatewayComponent = App.StackServiceComponent.find().findProperty('componentName', 'KNOX_GATEWAY');
        if (serviceName === 'HDFS' && nnComponent) {
          var namenodeCustomCommands = nnComponent.get('customCommands');
          if (namenodeCustomCommands && namenodeCustomCommands.contains('REBALANCEHDFS')) {
            options.push(actionMap.REBALANCEHDFS);
          }
        }

        if (serviceName === 'KNOX' && knoxGatewayComponent) {
          var knoxGatewayCustomCommands = knoxGatewayComponent.get('customCommands');
          knoxGatewayCustomCommands.forEach(function (command) {
            if (actionMap[command]) {
              options.push(actionMap[command]);
            }
          });
        }

        if (serviceName === 'HIVE') {
          var hiveServerInteractiveComponent = App.StackServiceComponent.find().findProperty('componentName', 'HIVE_SERVER_INTERACTIVE');
          var isHiveInteractiveServerPresent = allMasters.contains('HIVE_SERVER_INTERACTIVE');
          if (hiveServerInteractiveComponent && isHiveInteractiveServerPresent) {
            var LLAPCustomCommands = hiveServerInteractiveComponent.get('customCommands');
            LLAPCustomCommands.forEach(function (command) {
              if (actionMap[command]) {
                options.push(actionMap[command]);
              }
            });
          }
        }

        var hMasterComponent = App.StackServiceComponent.find().findProperty('componentName', 'HBASE_MASTER');
        if (serviceName === 'HBASE' && hMasterComponent) {
          var hMasterCustomCommands = hMasterComponent.get('customCommands');
          if (hMasterCustomCommands && hMasterCustomCommands.contains('UPDATE_REPLICATION')) {
            options.push(actionMap.UPDATE_REPLICATION);
          }
          if (hMasterCustomCommands && hMasterCustomCommands.contains('STOP_REPLICATION')) {
            options.push(actionMap.STOP_REPLICATION);
          }
        }

        /**
         * Display all custom commands of Master and StandBy on Service page.
         **/
        if (serviceName === 'HAWQ') {
          var hawqMasterComponent = App.StackServiceComponent.find().findProperty('componentName', 'HAWQMASTER');
          var hawqStandByComponent = App.StackServiceComponent.find().findProperty('componentName', 'HAWQSTANDBY');
          [hawqMasterComponent, hawqStandByComponent].forEach(function (component) {
            component.get('customCommands').forEach(function (command) {
              options.push(self.createOption(actionMap[command], {
                context: {
                  label: actionMap[command].context,
                  service: component.get('serviceName'),
                  component: component.get('componentName'),
                  command: command
                }
              }));
            });
          });
        }

        if (App.isAuthorized('HOST.ADD_DELETE_COMPONENTS')) {
          self.addActionMap().filterProperty('service', serviceName).forEach(function (item) {
            if (App.get('components.addableToHost').contains(item.component) && (!item.dependsFromAnotherProperty || item.depend)) {

              var isEnabled = App.HostComponent.find().filterProperty('componentName', item.component).length < App.get('allHostNames.length');

              if (item.component === 'OOZIE_SERVER') {
                isEnabled = isEnabled && !(Em.isEmpty(self.get('controller.configs.oozie-env.oozie_database')) || self.get('controller.configs.oozie-env.oozie_database') === 'New Derby Database');
              }

              item.action = 'addComponent';
              item.disabled = isEnabled ? '' : 'disabled';
              item.tooltip = isEnabled ? '' : Em.I18n.t('services.summary.allHostsAlreadyRunComponent').format(item.component);
              item.context = item.component;

              options.push(item);
            }
          });
        }

        if (App.get('isKerberosEnabled')) {
          options.push(actionMap.REGENERATE_KEYTAB_FILE_OPERATIONS);
        }

        allMasters.forEach(function (master) {
          var component = App.StackServiceComponent.find(master);
          var commands = component.get('customCommands');

          if (!commands.length) {
            return;
          }

          commands.forEach(function (command) {
            if (excludedCommands[master] && excludedCommands[master].contains(command)) {
              return;
            }

            options.push(self.createOption(actionMap.MASTER_CUSTOM_COMMAND, {
              label: Em.I18n.t('services.service.actions.run.executeCustomCommand.menu').format(App.format.normalizeNameBySeparators(command, ["_", "-", " "])),
              context: {
                label: Em.I18n.t('services.service.actions.run.executeCustomCommand.menu').format(App.format.normalizeNameBySeparators(command, ["_", "-", " "])),
                service: component.get('serviceName'),
                component: component.get('componentName'),
                command: command
              }
            }));
          });
        });
      }

      if (hasConfigTab) {
        options.push(actionMap.DOWNLOAD_CLIENT_CONFIGS);
      }

      if (App.isAuthorized("SERVICE.ADD_DELETE_SERVICES") && App.supports.enableAddDeleteServices) {
        options.push(actionMap.DELETE_SERVICE);
      }
    }

    if (this.get('maintenance.length')) {
      this.get('maintenance').forEach(function (option, index) {
        if (JSON.stringify(option) !== JSON.stringify(options[index])) {
          self.get('maintenance').removeAt(index).insertAt(index, options[index]);
        }
      });
      options.forEach(function (opt, index) {
        if (JSON.stringify(opt) !== JSON.stringify(self.get('maintenance')[index])) {
          self.get('maintenance').pushObject(opt);
        }
      });
    } else {
      this.set('maintenance', options);
    }
    this.set('isMaintenanceSet', true);
    App.tooltip($('[rel="' + this.get('tooltipAttribute') + '"]'));
  },

  clearIsMaintenanceSet: function clearIsMaintenanceSet() {
    this.set('isMaintenanceSet', false);
  },

  isMaintenanceActive: function () {
    return this.get('state') !== 'inDOM' || this.get('maintenance').length !== 0;
  }.property('maintenance'),

  hasConfigTab: function () {
    return App.havePermissions('CLUSTER.VIEW_CONFIGS') && !App.get('services.noConfigTypes').contains(this.get('controller.content.serviceName'));
  }.property('controller.content.serviceName', 'App.services.noConfigTypes'),

  hasHeatmapTab: function () {
    return App.get('services.servicesWithHeatmapTab').contains(this.get('controller.content.serviceName'));
  }.property('controller.content.serviceName', 'App.services.servicesWithHeatmapTab'),

  hasMetricTab: function () {
    var serviceName = this.get('controller.content.serviceName');
    var graphs = require('data/service_graph_config')[serviceName.toLowerCase()];
    return graphs || App.StackService.find(serviceName).get('isServiceWithWidgets');
  }.property('controller.content.serviceName'),

  didInsertElement: function didInsertElement() {
    this.get('controller').setStartStopState();
  },

  maintenanceObsFields: ['isStopDisabled', 'isClientsOnlyService', 'content.isRestartRequired', 'isServicesInfoLoaded', 'isServiceConfigsLoaded'],

  willInsertElement: function willInsertElement() {
    var self = this;
    this.get('controller').loadConfigs();
    if (this.get('controller.content.serviceName') === 'HIVE') {
      this.loadHiveConfigs();
    }

    this.get('maintenanceObsFields').forEach(function (field) {
      self.addObserver('controller.' + field, self, 'observeMaintenance');
    });
  },

  willDestroyElement: function willDestroyElement() {
    var self = this;
    this.get('maintenanceObsFields').forEach(function (field) {
      self.removeObserver('controller.' + field, self, 'observeMaintenance');
    });
  },

  service: function () {
    var svc = this.get('controller.content');
    var svcName = svc.get('serviceName');
    if (svcName) {
      switch (svcName.toLowerCase()) {
        case 'hdfs':
          svc = App.HDFSService.find().objectAt(0);
          break;
        case 'yarn':
          svc = App.YARNService.find().objectAt(0);
          break;
        case 'hbase':
          svc = App.HBaseService.find().objectAt(0);
          break;
        case 'flume':
          svc = App.FlumeService.find().objectAt(0);
          break;
        default:
          break;
      }
    }
    return svc;
  }.property('controller.content.serviceName').volatile()
});

});

require.register("views/main/service/manage_config_groups_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceManageConfigGroupView = Em.View.extend({

  templateName: require('templates/main/service/manage_configuration_groups_popup'),

  /**
   * Select Config Group
   * @type {App.ConfigGroup}
   */
  selectedConfigGroup: null,

  /**
   * Determines if "Delete Config Group" button is disabled
   * @type {boolean}
   */
  isRemoveButtonDisabled: true,

  /**
   * Determines if "Rename Config Group" button is disabled
   * @type {boolean}
   */
  isRenameButtonDisabled: true,

  /**
   * Determines if "Duplicate Config Group" button is disabled
   * @type {boolean}
   */
  isDuplicateButtonDisabled: true,

  /**
   * Tooltip for "Add Config Group" button
   * @type {string}
   */
  addButtonTooltip: Em.I18n.t('services.service.config_groups_popup.addButton'),

  /**
   * Tooltip for "Remove Config Group" button
   * @type {string}
   */
  removeButtonTooltip: Em.I18n.t('services.service.config_groups_popup.removeButton'),

  /**
   * Tooltip for "Rename Config Group" button
   * @type {string}
   */
  renameButtonTooltip: Em.I18n.t('services.service.config_groups_popup.renameButton'),

  /**
   * Tooltip for "Duplicate Config Group" button
   * @type {string}
   */
  duplicateButtonTooltip: Em.I18n.t('services.service.config_groups_popup.duplicateButton'),

  /**
   * Tooltip for "Remove Host From Config Group" button
   * @type {string}
   */
  removeHostTooltip: Em.I18n.t('services.service.config_groups_popup.removeHost'),

  /**
   * Tooltip for "Add Host To Config Group" button
   * @type {string}
   */
  addHostTooltip: function () {
    var selectedConfigGroup = this.get('controller.selectedConfigGroup');
    if (!selectedConfigGroup.get('isDefault') && selectedConfigGroup.get('isAddHostsDisabled')) {
      return Em.I18n.t('services.service.config_groups_popup.addHostDisabled');
    } else {
      return Em.I18n.t('services.service.config_groups_popup.addHost');
    }
  }.property('controller.selectedConfigGroup.isDefault', 'controller.selectedConfigGroup.isAddHostsDisabled'),

  willInsertElement: function willInsertElement() {
    this.get('controller').loadHosts();
  },

  willDestroyElement: function willDestroyElement() {
    this.get('controller.configGroups').clear();
    this.get('controller.originalConfigGroups').clear();
  },

  showTooltip: function () {
    if (!this.get('controller.isLoaded')) return false;
    Em.run.next(function () {
      App.tooltip($('.properties-link'), { html: true });
      App.tooltip($("[rel='button-info']"));
      App.tooltip($("[rel='button-info-dropdown']"), { placement: 'left' });
    });
  }.observes('controller.isLoaded'),

  /**
   * Disable actions remove and rename for Default config group
   * @method buttonObserver
   */
  buttonObserver: function () {
    var selectedConfigGroup = this.get('controller.selectedConfigGroup');
    this.set('isRemoveButtonDisabled', selectedConfigGroup.get('isDefault'));
    this.set('isRenameButtonDisabled', selectedConfigGroup.get('isDefault'));
    this.set('isDuplicateButtonDisabled', false);
  }.observes('controller.selectedConfigGroup'),

  /**
   * Prevent user to select more than 1 config group
   * Select last one of "selected"
   * Clean up <code>controller.selectedHosts</code>
   * @method onGroupSelect
   */
  onGroupSelect: function () {
    var selectedConfigGroup = this.get('selectedConfigGroup');
    // to unable user select more than one config group at a time
    if (selectedConfigGroup.length) {
      this.set('controller.selectedConfigGroup', selectedConfigGroup[selectedConfigGroup.length - 1]);
    }
    if (selectedConfigGroup.length > 1) {
      this.set('selectedConfigGroup', selectedConfigGroup[selectedConfigGroup.length - 1]);
    }
    this.set('controller.selectedHosts', []);
  }.observes('selectedConfigGroup'),

  /**
   * Select default config group after all config groups are loaded
   * @method selectDefaultGroup
   */
  selectDefaultGroup: function () {
    if (this.get('controller.isLoaded')) {
      this.set('selectedConfigGroup', [this.get('controller.configGroups').findProperty('isDefault')]);
    }
  }.observes('controller.isLoaded', 'controller.groupDeleteTrigger')

});

});

require.register("views/main/service/menu", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainServiceMenuView = Em.CollectionView.extend({
  disabledServices: [],

  content: function () {
    return App.router.get('mainServiceController.content').filter(function (item) {
      return !this.get('disabledServices').contains(item.get('id'));
    }, this);
  }.property('App.router.mainServiceController.content', 'App.router.mainServiceController.content.length'),

  didInsertElement: function didInsertElement() {
    App.router.location.addObserver('lastSetURL', this, 'renderOnRoute');
    this.renderOnRoute();
    App.tooltip(this.$(".restart-required-service"), { html: true, placement: "right" });
  },

  willDestroyElement: function willDestroyElement() {
    App.router.location.removeObserver('lastSetURL', this, 'renderOnRoute');
    this.$(".restart-required-service").tooltip('destroy');
  },

  activeServiceId: null,

  /**
   *    Syncs navigation menu with requested URL
   */
  renderOnRoute: function renderOnRoute() {
    var last_url = App.router.location.lastSetURL || location.href.replace(/^[^#]*#/, '');
    if (last_url.substr(1, 4) !== 'main' || !this._childViews) {
      return;
    }
    var reg = /^\/main\/services\/(\S+)\//g;
    var sub_url = reg.exec(last_url);
    var service_id = null != sub_url ? sub_url[1] : 1;
    this.set('activeServiceId', service_id);
  },

  tagName: 'ul',
  classNames: ["nav", "nav-list", "nav-services"],

  itemViewClass: Em.View.extend({

    classNameBindings: ["active", "clients"],
    templateName: require('templates/main/service/menu_item'),
    restartRequiredMessage: null,

    shouldBeRestarted: Em.computed.equal('content.isRestartRequired'),

    active: function () {
      return this.get('content.id') === this.get('parentView.activeServiceId') ? 'active' : '';
    }.property('parentView.activeServiceId'),

    hasCriticalAlerts: Em.computed.alias('content.hasCriticalAlerts'),

    isConfigurable: Em.computed.notExistsInByKey('content.serviceName', 'App.services.noConfigTypes'),

    link: function () {
      var currentState = App.router.get('currentState.name');
      var routeToNewState = this.get('parentView.activeServiceId') !== this.get('content.id');
      var stateName = ['summary', 'configs'].contains(currentState) ? this.get('isConfigurable') && routeToNewState ? currentState : 'summary' : 'summary';
      return "#/main/services/" + this.get('content.id') + "/" + stateName;
    }.property('App.router.currentState.name', 'parentView.activeServiceId', 'isConfigurable'),

    goToConfigs: function goToConfigs() {
      App.router.set('mainServiceItemController.routeToConfigs', true);
      App.router.transitionTo('services.service.configs', this.get('content'));
      App.router.set('mainServiceItemController.routeToConfigs', false);
    },

    refreshRestartRequiredMessage: function () {
      var restarted, componentsCount, hostsCount, message, tHosts, tComponents;
      restarted = this.get('content.restartRequiredHostsAndComponents');
      componentsCount = 0;
      hostsCount = 0;
      message = "";
      for (var host in restarted) {
        hostsCount++;
        componentsCount += restarted[host].length;
      }
      if (hostsCount > 1) {
        tHosts = Em.I18n.t('common.hosts');
      } else {
        tHosts = Em.I18n.t('common.host');
      }
      if (componentsCount > 1) {
        tComponents = Em.I18n.t('common.components');
      } else {
        tComponents = Em.I18n.t('common.component');
      }
      message += componentsCount + ' ' + tComponents + ' ' + Em.I18n.t('on') + ' ' + hostsCount + ' ' + tHosts + ' ' + Em.I18n.t('services.service.config.restartService.needToRestartEnd');
      this.set('restartRequiredMessage', message);
    }.observes('content.restartRequiredHostsAndComponents')
  })
});

});

require.register("views/main/service/reassign/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.ReassignMasterWizardStep1View = Em.View.extend({

  message: function () {
    var componentName = this.get('controller.content.reassign.component_name');
    var listOfServices;
    var installedServices = App.Service.find().mapProperty('serviceName');

    if (this.get('controller.content.componentsToStopAllServices').contains(componentName)) {
      listOfServices = installedServices;
    } else {
      listOfServices = this.get('controller.target.reassignMasterController.relatedServicesMap')[componentName];
      if (!listOfServices || !listOfServices.length) {
        listOfServices = installedServices.filter(function (service) {
          return service != 'HDFS';
        });
      } else {
        //not display any service which is not installed
        var installedServicesToStop = listOfServices.filter(function (service) {
          return installedServices.contains(service);
        });
        listOfServices = installedServicesToStop;
      }
    }

    var messages = [Em.I18n.t('services.reassign.step1.message1').format(this.get('controller.content.reassign.display_name')), Em.I18n.t('services.reassign.step1.message3').format(stringUtils.getFormattedStringFromArray(listOfServices), this.get('controller.content.reassign.display_name'))];
    if (this.get('controller.content.hasManualSteps')) {
      messages.splice(1, 0, Em.I18n.t('services.reassign.step1.message2').format(this.get('controller.content.reassign.display_name')));
    }

    return messages;
  }.property('controller.content.reassign.display_name', 'controller.content.hasManualSteps'),

  templateName: require('templates/main/service/reassign/step1')

});

});

require.register("views/main/service/reassign/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep2View = App.AssignMasterComponentsView.extend({

  title: Em.I18n.t('installer.step5.reassign.header'),

  alertMessage: function () {
    if (this.get('controller.content.reassign.component_name') === 'NAMENODE' && App.get('isHaEnabled')) {
      return Em.I18n.t('services.reassign.step2.body.namenodeHA').format(this.get('controller.content.reassign.display_name'));
    }
    return Em.I18n.t('services.reassign.step2.body').format(this.get('controller.content.reassign.display_name'));
  }.property('controller.content.reassign.component_name', 'controller.content.reassign.display_name')
});

});

require.register("views/main/service/reassign/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep3View = Em.View.extend({

  templateName: require('templates/main/service/reassign/step3'),

  sourceHost: Em.computed.alias('controller.content.reassignHosts.source'),

  targetHost: Em.computed.alias('controller.content.reassignHosts.target'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  jdbcSetupMessage: function () {
    if (['HIVE_SERVER', 'HIVE_METASTORE', 'OOZIE_SERVER'].contains(this.get('controller.content.reassign.component_name'))) {
      if (this.get('controller.content.reassign.component_name') === 'OOZIE_SERVER' && this.get('controller.content.databaseType') === 'derby') {
        return false;
      }

      return Em.I18n.t('services.service.config.database.msg.jdbcSetup').format(this.get('controller.content.databaseType'), this.get('controller.content.databaseType'));
    }

    return false;
  }.property('controller.content.reassign.display_name', 'controller.content.databaseType')
});

});

require.register("views/main/service/reassign/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep4View = App.HighAvailabilityProgressPageView.extend({

  headerTitle: Em.I18n.t('services.reassign.step4.header'),

  formattedComponentName: Em.computed.formatRole('controller.content.reassign.component_name', false),

  noticeInProgress: Em.computed.i18nFormat('services.reassign.step4.status.info', 'formattedComponentName'),

  noticeFailed: Em.computed.i18nFormat('services.reassign.step4.status.failed', 'formattedComponentName', 'controller.content.reassignHosts.source', 'controller.content.reassignHosts.target'),

  noticeCompleted: function () {
    if (this.get('controller.content.hasManualSteps')) {
      return Em.I18n.t('services.reassign.step4.status.success.withManualSteps').format(this.get('formattedComponentName'));
    } else {
      return Em.I18n.t('services.reassign.step4.status.success').format(this.get('formattedComponentName'), this.get('controller.content.reassignHosts.source'), this.get('controller.content.reassignHosts.target'));
    }
  }.property('controller.content.reassign.component_name', 'controller.content.reassignHosts.source', 'controller.content.reassignHosts.target'),

  submitButtonText: Em.computed.ifThenElse('controller.content.hasManualSteps', Em.I18n.t('common.next') + ' &rarr;', Em.I18n.t('common.complete')),

  templateName: require('templates/main/service/reassign/step4'),

  labelWidth: 'col-md-5'
});

});

require.register("views/main/service/reassign/step5_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.ReassignMasterWizardStep5View = Em.View.extend({

  templateName: require('templates/main/service/reassign/step5'),

  manualCommands: function () {
    if (!this.get('controller.content.componentsWithManualCommands').contains(this.get('controller.content.reassign.component_name'))) {
      return '';
    }
    var atsDir = App.StackService.find('YARN').compareCurrentVersion('2.7') > -1 ? "timeline-state-store.ldb" : "leveldb-timeline-store.ldb",
        componentDir = this.get('controller.content.componentDir') || '',
        componentDirCmd = componentDir.replace(/,/g, ' '),
        sourceHost = this.get('controller.content.reassignHosts.source'),
        targetHost = this.get('controller.content.reassignHosts.target'),
        ha = '',
        user = this.get('controller.content.hdfsUser'),
        path;

    if (this.get('controller.content.reassign.component_name') === 'NAMENODE' && App.get('isHaEnabled')) {
      ha = '_ha';
      var nnStartedHost = this.get('controller.content.masterComponentHosts').filterProperty('component', 'NAMENODE').mapProperty('hostName').without(sourceHost).without(targetHost);
    }

    if (this.get('controller.content.reassign.component_name') === 'APP_TIMELINE_SERVER') {
      user = this.get('controller.content.configs.yarn-env.yarn_user');
      path = this.get('controller.content.configs.yarn-site')['yarn.timeline-service.leveldb-timeline-store.path'];
    }

    return Em.I18n.t('services.reassign.step5.body.' + this.get('controller.content.reassign.component_name').toLowerCase() + ha).format(componentDir, sourceHost, targetHost, user, nnStartedHost, this.get('controller.content.group'), componentDirCmd, path, atsDir);
  }.property('controller.content.reassign.component_name', 'controller.content.componentDir', 'controller.content.masterComponentHosts', 'controller.content.reassign.host_id', 'controller.content.hdfsUser'),

  /**
   * security notice to generate keytab manually is not used any more
   */
  securityNotice: function () {
    var secureConfigs = this.get('controller.content.secureConfigs');
    var proceedMsg = Em.I18n.t('services.reassign.step5.body.proceedMsg');
    var hasSecureConfigs = !this.get('controller.content.componentsWithoutSecurityConfigs').contains(this.get('controller.content.reassign.component_name'));
    if (!hasSecureConfigs) {
      secureConfigs = [];
    }
    if (!App.get('isKerberosEnabled') || !secureConfigs.length) {
      return proceedMsg;
    }
    var formattedText = '<ul>';
    secureConfigs.forEach(function (config) {
      formattedText += '<li>' + Em.I18n.t('services.reassign.step5.body.securityConfigsList').format(config.keytab, config.principal.replace('_HOST', this.get('controller.content.reassignHosts.target')), this.get('controller.content.reassignHosts.target')) + '</li>';
    }, this);
    formattedText += '</ul>';
    return Em.I18n.t('services.reassign.step5.body.securityNotice').format(formattedText) + proceedMsg;
  }.property('App.isKerberosEnabled', 'controller.content.secureConfigs', 'controller.content.reassignHosts.target')
});

});

require.register("views/main/service/reassign/step6_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep6View = App.HighAvailabilityProgressPageView.extend({

  headerTitle: Em.I18n.t('services.reassign.step6.header'),

  formattedComponentName: Em.computed.formatRole('controller.content.reassign.component_name', false),

  noticeInProgress: Em.computed.i18nFormat('services.reassign.step6.status.info', 'formattedComponentName'),

  noticeFailed: Em.computed.i18nFormat('services.reassign.step6.status.failed', 'formattedComponentName', 'controller.content.reassignHosts.source', 'controller.content.reassignHosts.target'),

  noticeCompleted: Em.computed.i18nFormat('services.reassign.step6.status.success', 'formattedComponentName', 'controller.content.reassignHosts.source', 'controller.content.reassignHosts.target'),

  submitButtonText: Em.I18n.t('common.complete'),

  templateName: require('templates/main/service/reassign/step6'),

  labelWidth: 'col-md-5'
});

});

require.register("views/main/service/reassign/step7_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterWizardStep7View = App.HighAvailabilityProgressPageView.extend({

  headerTitle: Em.I18n.t('services.reassign.step7.header'),

  noticeInProgress: Em.I18n.t('services.reassign.step7.info'),

  noticeFailed: Em.I18n.t('services.reassign.step7.failed'),

  noticeCompleted: Em.I18n.t('services.reassign.step7.success'),

  submitButtonText: Em.I18n.t('common.complete') + ' &rarr;',

  templateName: require('templates/main/service/reassign/step7'),

  labelWidth: 'col-md-5'
});

});

require.register("views/main/service/reassign_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.ReassignMasterView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/service/reassign'),

  isLoaded: false,

  willInsertElement: function willInsertElement() {
    this.set('isLoaded', false);
    this.loadHosts();
  },

  /**
   * load hosts from server
   */
  loadHosts: function loadHosts() {
    App.ajax.send({
      name: 'hosts.high_availability.wizard',
      data: {},
      sender: this,
      success: 'loadHostsSuccessCallback',
      error: 'loadHostsErrorCallback'
    });
  },

  loadHostsSuccessCallback: function loadHostsSuccessCallback(data, opt, params) {
    var hosts = {};

    data.items.forEach(function (item) {
      hosts[item.Hosts.host_name] = {
        name: item.Hosts.host_name,
        bootStatus: "REGISTERED",
        isInstalled: true
      };
    });
    this.get('controller').setDBProperty('hosts', hosts);
    this.set('controller.content.hosts', hosts);
    this.set('isLoaded', true);
  },

  loadHostsErrorCallback: function loadHostsErrorCallback() {
    this.set('isLoaded', true);
  }

});

});

require.register("views/main/service/reconfigure", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var App = require('app');

App.MainServiceReconfigureView = Em.View.extend({

  templateName: require('templates/main/service/reconfigure')

});

App.StageLabelView = Em.View.extend({
  tagName: 'a',
  classNameBindings: ['removeLink'],
  attributeBindings: ['href'],
  href: '#',
  removeLink: null,
  didInsertElement: function didInsertElement() {
    this.onLink();
  },
  onLink: function () {
    this.set('removeLink', this.get('showLink') ? null : 'remove-link');
  }.observes('showLink'),
  command: null,
  click: function click() {
    if (this.get('command') && this.get('showLink')) {
      this.showHostPopup(this.get('command'));
    }
  },

  showHostPopup: function showHostPopup(command) {
    var controller = this.get("controller");
    App.router.get('userSettingsController').dataLoading('show_bg').done(function (initValue) {
      var popupView = App.HostPopup.initPopup(command.get('label'), controller, false, command.get('requestId'));
      popupView.set('isNotShowBgChecked', !initValue);
    });
  },

  isStarted: Em.computed.alias('command.isStarted'),

  showLink: Em.computed.alias('command.showLink')

});

App.StageSuccessView = Em.View.extend({
  layout: Ember.Handlebars.compile('<i class="glyphicon glyphicon-ok glyphicon-large"></i> {{t common.done}}')
});

App.StageFailureView = Em.View.extend({
  layout: Ember.Handlebars.compile('<i class="glyphicon glyphicon-remove iglyphicon-large"></i> {{t common.failed}}')
});

App.StageInProgressView = Em.View.extend({
  command: null,
  classNames: ['progress'],
  template: Ember.Handlebars.compile('<div class="progress-bar progress-bar-striped active" {{bindAttr style="command.barWidth"}}></div>'),

  isStageCompleted: function () {
    return this.get('obj.progress') === 100 || this.get('controller.isStepCompleted');
  }.property('controller.isStepCompleted', 'obj.progress')

});

});

require.register("views/main/service/service", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var uiEffects = require('utils/ui_effects');
var numberUtils = require('utils/number_utils');
require('views/main/service/info/components_list_view');

App.MainDashboardServiceHealthView = Em.View.extend({
  classNameBindings: ["healthStatus", "healthStatusClass"],
  //template: Em.Handlebars.compile(""),
  blink: false,
  tagName: 'span',
  attributeBindings: ['rel', 'title', 'data-original-title'],
  rel: 'HealthTooltip',
  'data-original-title': '',

  updateToolTip: function () {
    this.set('data-original-title', this.get('service.toolTipContent'));
  }.observes('service.toolTipContent'),

  startBlink: function startBlink() {
    this.set('blink', true);
  },

  doBlink: function () {
    var self = this;
    if (this.get('blink') && this.get("state") === "inDOM") {
      uiEffects.pulsate(self.$(), 1000, function () {
        self.doBlink();
      });
    }
  }.observes('blink'),

  stopBlink: function stopBlink() {
    this.set('blink', false);
  },

  healthStatus: function () {
    var isClientOnlyService = App.get('services.clientOnly').contains(this.get('service.serviceName'));
    if (isClientOnlyService) {
      return 'glyphicon glyphicon-blackboard';
    }
    if (this.get('service.passiveState') !== 'OFF') {
      return 'icon-medkit';
    }
    var status = this.get('service.healthStatus');
    switch (status) {
      case 'green':
        status = App.Service.Health.live;
        this.stopBlink();
        break;
      case 'green-blinking':
        status = App.Service.Health.live;
        this.startBlink();
        break;
      case 'red-blinking':
        status = App.Service.Health.dead;
        this.startBlink();
        break;
      case 'yellow':
        status = App.Service.Health.unknown;
        break;
      default:
        status = App.Service.Health.dead;
        this.stopBlink();
        break;
    }

    return 'health-status-' + status;
  }.property('service.healthStatus', 'service.passiveState', 'service.serviceName'),

  healthStatusClass: function () {
    if (this.get('service.passiveState') !== 'OFF' || App.get('services.clientOnly').contains(this.get('service.serviceName'))) {
      return '';
    }
    switch (this.get('service.healthStatus')) {
      case 'green':
      case 'green-blinking':
        return App.healthIconClassGreen;
        break;
      case 'red':
      case 'red-blinking':
        return App.healthIconClassRed;
        break;
      case 'yellow':
        return App.healthIconClassYellow;
      default:
        return '';
    }
  }.property('service.healthStatus', 'service.passiveState', 'service.serviceName'),

  didInsertElement: function didInsertElement() {
    this.updateToolTip();
    App.tooltip($("[rel='HealthTooltip']"), { html: true });
    this.doBlink(); // check for blink availability
  }
});

App.ComponentLiveTextView = Em.View.extend({
  liveComponents: null,
  totalComponents: null,
  color: function () {
    return this.get("liveComponents") === 0 && this.get('totalComponents') !== 0;
  }.property("liveComponents", 'totalComponents')
});

App.MainDashboardServiceViewWrapper = Em.Mixin.create({
  layoutName: require('templates/main/service/service')
});

App.MainDashboardServiceView = Em.View.extend(App.MainDashboardServiceViewWrapper, {
  classNames: ['service', 'clearfix'],

  data: function () {
    return this.get('controller.data.' + this.get('serviceName'));
  }.property('controller.data'),

  dashboardMasterComponentView: App.SummaryMasterComponentsView.extend({
    mastersComp: Em.computed.alias('parentView.parentView.mastersObj')
  }),

  alertsCount: Em.computed.alias('service.alertsCount'),

  hasCriticalAlerts: Em.computed.alias('service.hasCriticalAlerts'),

  isCollapsed: false,

  masters: Em.computed.filterBy('service.hostComponents', 'isMaster', true),

  clients: function () {
    var clients = this.get('service.hostComponents').filterProperty('isClient', true);
    var len = clients.length;
    var template = 'dashboard.services.{0}.client'.format(this.get('serviceName').toLowerCase());
    if (len > 1) {
      template += 's';
    }

    return {
      title: this.t(template).format(len),
      component: clients.objectAt(0)
    };
  }.property('service'),

  hasMultipleMasterGroups: Em.computed.gt('parentView.mastersObj.length', 1),

  /**
   * Check if service component is created
   * @param componentName
   * @returns {Boolean}
   */
  isServiceComponentCreated: function isServiceComponentCreated(componentName) {
    return App.MasterComponent.find().mapProperty('componentName').concat(App.ClientComponent.find().mapProperty('componentName'), App.SlaveComponent.find().mapProperty('componentName')).contains(componentName);
  }

});

App.MainDashboardServiceView.reopenClass({
  formattedHeapPercent: function formattedHeapPercent(i18nKey, heapUsedKey, heapMaxKey) {
    return Em.computed(heapUsedKey, heapMaxKey, function () {
      var memUsed = Em.get(this, heapUsedKey);
      var memMax = Em.get(this, heapMaxKey);
      var percent = memMax > 0 ? 100 * memUsed / memMax : 0;
      return Em.I18n.t(i18nKey).format(percent.toFixed(1));
    });
  },
  formattedHeap: function formattedHeap(i18nKey, heapUsedKey, heapMaxKey) {
    return Em.computed(heapUsedKey, heapMaxKey, function () {
      var memUsed = Em.get(this, heapUsedKey);
      var memMax = Em.get(this, heapMaxKey);
      return Em.I18n.t(i18nKey).format(numberUtils.bytesToSize(memUsed, 1, 'parseFloat'), numberUtils.bytesToSize(memMax, 1, 'parseFloat'));
    });
  }
});

});

require.register("views/main/service/services/flume", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var sort = require('views/common/sort_view');
require('views/main/service/info/metrics/flume/flume_agent_metrics_section');

App.MainDashboardServiceFlumeView = App.TableView.extend(App.MainDashboardServiceViewWrapper, {
  templateName: require('templates/main/service/services/flume'),

  pagination: false,

  selectedHost: null,

  flumeAgentsCount: 0,

  content: function () {
    var flumeAgents = this.get('service.agents'),
        content = [];

    flumeAgents.mapProperty('hostName').uniq().forEach(function (hostName) {
      var agents = flumeAgents.filterProperty('hostName', hostName);
      content.push(Em.Object.create({
        hostName: hostName,
        agents: agents
      }));
    });
    return content;
  }.property('App.FlumeAgent.@each.id', 'flumeAgentsCount'),

  summaryHeader: function () {
    var agentCount = App.FlumeService.find().objectAt(0).get('agents.length'),
        hostCount = this.get('service.flumeHandlersTotal');
    return this.t("dashboard.services.flume.summary.title").format(hostCount, hostCount > 1 ? 's' : '', agentCount, agentCount > 1 ? 's' : '');
  }.property('service.agents', 'service.hostComponents.length'),

  flumeHandlerComponent: Em.Object.create({
    componentName: 'FLUME_HANDLER'
  }),

  sortView: sort.wrapperView,

  hostSort: sort.fieldView.extend({
    column: '1',
    name: 'hostName',
    displayName: Em.I18n.t('common.host')
  }),

  didInsertElement: function didInsertElement() {
    var self = this;
    this.filter();
    this.$().on('click', '.flume-agents-actions .dropdown-toggle', function () {
      self.setDropdownPosition(this);
    });
  },

  selectHost: function selectHost(e) {
    var host = e && e.context;
    this.get('pageContent').setEach('isActive', false);
    if (host) {
      this.showAgentInfo(host);
    }
  },

  /**
   * Locate dropdown menu absolutely outside of scrollable block
   * @param element
   */
  setDropdownPosition: function setDropdownPosition(element) {
    var button = $(element);
    var dropdown = button.parent().find('.dropdown-menu');
    dropdown.css('top', button.offset().top + button.outerHeight() + "px");
    dropdown.css('left', button.offset().left + "px");
  },

  willDestroyElement: function willDestroyElement() {
    this.$().off();
  },

  /**
   * Change classes for dropdown DOM elements after status change of selected agent
   */
  setActionsDropdownClasses: function () {
    this.get('content').forEach(function (hosts) {
      hosts.agents.forEach(function (agent) {
        agent.set('isStartAgentDisabled', agent.get('status') !== 'NOT_RUNNING');
        agent.set('isStopAgentDisabled', agent.get('status') !== 'RUNNING');
      });
    });
  }.observes('service.agents.@each.status'),

  updateFlumeAgentsCount: function () {
    this.set('flumeAgentsCount', this.get('service.agents.length'));
  }.observes('service.agents.length'),

  /**
   * Action handler from flume template.
   * Highlight selected row and show metrics graphs of selected agent.
   *
   * @method showAgentInfo
   * @param {object} host
   */
  showAgentInfo: function showAgentInfo(host) {
    Em.set(host, 'isActive', true);
    this.set('selectedHost', host);
    this.setAgentMetrics(host);
  },
  /**
   * Show Flume agent metric.
   *
   * @method setFlumeAgentMetric
   * @param {object} host
   */
  setAgentMetrics: function setAgentMetrics(host) {
    var mockMetricData = [{
      header: 'channelName',
      metricType: 'CHANNEL'
    }, {
      header: 'sinkName',
      metricType: 'SINK'
    }, {
      header: 'sourceName',
      metricType: 'SOURCE'
    }];
    var metricViews = mockMetricData.map(function (mockData, index) {
      return App.FlumeAgentMetricsSectionView.extend({
        index: index,
        metricTypeKey: mockData.header,
        metricView: App.MainServiceInfoFlumeGraphsView.extend(),
        metricViewData: {
          agent: host,
          metricType: mockData.metricType
        }
      });
    });
    this.set('parentView.collapsedSections', metricViews);
  }
});

});

require.register("views/main/service/services/hbase", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var date = require('utils/date/date');

function getMasterTime(key) {
  var _k = 'service.' + key;
  return Em.computed(_k, function () {
    var uptime = this.get(_k);
    if (uptime && uptime > 0) {
      var diff = App.dateTime() - uptime;
      if (diff < 0) {
        diff = 0;
      }
      var formatted = date.timingFormat(diff);
      return this.t('dashboard.services.uptime').format(formatted);
    }
    return this.t('services.service.summary.notRunning');
  });
}

App.MainDashboardServiceHbaseView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/hbase'),
  serviceName: 'hbase',
  /**
   * All master components
   */
  masters: Em.computed.filterBy('service.hostComponents', 'isMaster', true),
  /**
   * Passive master components
   */
  passiveMasters: Em.computed.filterBy('masters', 'haStatus', 'false'),

  regionServesText: Em.computed.countBasedMessage('service.regionServersTotal', '', Em.I18n.t('services.service.summary.viewHost'), Em.I18n.t('services.service.summary.viewHosts')),

  phoenixServersText: Em.computed.countBasedMessage('service.phoenixServersTotal', '', Em.I18n.t('services.service.summary.viewHost'), Em.I18n.t('services.service.summary.viewHosts')),

  /**
   * One(!) active master component
   */
  activeMaster: Em.computed.findBy('masters', 'haStatus', 'true'),

  activeMasterTitle: Em.I18n.t('service.hbase.activeMaster'),

  masterServerHeapSummary: App.MainDashboardServiceView.formattedHeap('dashboard.services.hbase.masterServerHeap.summary', 'service.heapMemoryUsed', 'service.heapMemoryMax'),
  masterServerHeapSummaryPercent: App.MainDashboardServiceView.formattedHeapPercent('dashboard.services.hbase.masterServerHeap.percent', 'service.heapMemoryUsed', 'service.heapMemoryMax'),
  summaryHeader: function () {
    var avgLoad = this.get('service.averageLoad');
    if (isNaN(avgLoad)) {
      avgLoad = this.t("services.service.summary.unknown");
    }
    return this.t("dashboard.services.hbase.summary").format(this.get('service.regionServersTotal'), avgLoad);
  }.property('service.regionServersTotal', 'service.averageLoad'),

  averageLoad: function () {
    var avgLoad = this.get('service.averageLoad');
    if (isNaN(avgLoad)) {
      avgLoad = this.t('services.service.summary.notAvailable');
    }
    return this.t('dashboard.services.hbase.averageLoadPerServer').format(avgLoad);
  }.property("service.averageLoad"),

  masterStartedTime: getMasterTime('masterStartTime'),

  masterActivatedTime: getMasterTime('masterActiveTime'),

  regionServerComponent: Em.Object.create({
    componentName: 'HBASE_REGIONSERVER'
  }),

  phoenixServerComponent: Em.Object.create({
    componentName: 'PHOENIX_QUERY_SERVER'
  }),

  isRegionServerCreated: function () {
    return this.isServiceComponentCreated('HBASE_REGIONSERVER');
  }.property('App.router.clusterController.isComponentsStateLoaded'),

  isPhoenixQueryServerCreated: function () {
    return this.isServiceComponentCreated('PHOENIX_QUERY_SERVER');
  }.property('App.router.clusterController.isComponentsStateLoaded')
});

});

require.register("views/main/service/services/hdfs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var date = require('utils/date/date');
require('views/main/service/info/summary/hdfs/slaves');
require('views/main/service/info/summary/hdfs/common_widgets');

App.MainDashboardServiceHdfsView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/hdfs'),
  serviceName: 'HDFS',

  slaveComponentsView: App.HDFSSlaveComponentsView.extend(),

  componentCommonWidgetsView: App.HDFSSummaryCommonWidgetsView.extend(),

  metricsNotAvailableObserver: function () {
    if (!this.get("service.metricsNotAvailable")) {
      App.tooltip($("[rel='tooltip']"));
    }
  }.observes("service.metricsNotAvailable"),

  willDestroyElement: function willDestroyElement() {
    $("[rel='tooltip']").tooltip('destroy');
  }
});

});

require.register("views/main/service/services/hive", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainDashboardServiceHiveView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/hive'),
  serviceName: 'HIVE',

  viewsToShow: {},

  viewLinks: function () {
    var viewsToShow = this.get('viewsToShow');
    var links = [];
    App.router.get('mainViewsController.ambariViews').forEach(function (viewInstance) {
      var viewMeta = viewsToShow[viewInstance.get('instanceName')];
      if (viewMeta) {
        var link = {
          viewInstance: viewInstance,
          label: viewInstance.get('label')
        };
        if (viewMeta.overwriteLabel) {
          link.label = Em.I18n.t(viewMeta.overwriteLabel);
        }
        links.push(link);
      }
    });
    return links;
  }.property('App.router.mainViewController.ambariViews'),

  didInsertElement: function didInsertElement() {
    var controller = this.get('controller');
    this._super();
    App.router.get('mainController').isLoading.call(App.router.get('clusterController'), 'isComponentsStateLoaded').done(function () {
      controller.setHiveEndPointsValue();
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.get('controller.hiveServerEndPoints').clear();
  },

  /**
   * View for clipboard image that copies JDBC connection string
   */
  clipBoardView: Em.View.extend({
    tagName: 'a',
    classNames: ['clip-board'],
    href: "javascript:void(null)",
    attributeBindings: ['data-clipboard-text', 'data-clipboard-action', "href"],
    didInsertElement: function didInsertElement() {
      var $this = this.$();
      var id = "#" + $this.attr('id');
      var clipboard = new Clipboard(id);
      var options = {
        trigger: "click"
      };
      Em.run.next(function () {
        $("[rel=clipboard-tooltip]").tooltip(options);
      });
    },
    mouseLeave: function mouseLeave() {
      Em.run.next(function () {
        $("[rel=clipboard-tooltip]").tooltip("hide");
      });
    }
  })
});

});

require.register("views/main/service/services/mapreduce2", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainDashboardServiceMapreduce2View = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/mapreduce2'),
  serviceName: 'MAPREDUCE2',

  titleInfo: function () {
    var hs = this.get('service.hostComponents').findProperty('componentName', 'HISTORYSERVER');
    if (!hs) {
      return this.t('services.mapreduce2.history.unknown');
    } else if (hs.get('workStatus') === App.HostComponentStatus.started) {
      return this.t('services.mapreduce2.history.running');
    } else {
      return this.t('services.mapreduce2.history.stopped');
    }
  }.property('service')
});

});

require.register("views/main/service/services/onefs", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var date = require('utils/date/date');
var numberUtils = require('utils/number_utils');

function diskPart(i18nKey, totalKey, usedKey) {
  return Em.computed(totalKey, usedKey, function () {
    var text = Em.I18n.t(i18nKey);
    var total = this.get(totalKey);
    var used = this.get(usedKey);
    var percent = total > 0 ? (used * 100 / total).toFixed(2) : 0;
    if (percent == "NaN" || percent < 0) {
      percent = Em.I18n.t('services.service.summary.notAvailable') + " ";
    }
    return text.format(numberUtils.bytesToSize(used, 1, 'parseFloat'), numberUtils.bytesToSize(total, 1, 'parseFloat'));
  });
}

function diskPartPercent(i18nKey, totalKey, usedKey) {
  return Em.computed(totalKey, usedKey, function () {
    var text = Em.I18n.t(i18nKey);
    var total = this.get(totalKey);
    var used = this.get(usedKey);
    var percent = total > 0 ? (used * 100 / total).toFixed(2) : 0;
    if (percent == "NaN" || percent < 0) {
      percent = Em.I18n.t('services.service.summary.notAvailable') + " ";
    }
    return text.format(percent);
  });
}

App.MainDashboardServiceOnefsView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/onefs'),
  serviceName: 'ONEFS',
  Chart: App.ChartPieView.extend({
    service: null,
    color: '#0066B3',
    stroke: '#0066B3',
    palette: new Rickshaw.Color.Palette({
      scheme: ['rgba(0,102,179,1)', 'rgba(0,102,179,0)']
    }),
    data: function () {
      var remaining = Number(this.get('service.capacityRemaining'));
      var used = Number(this.get('service.capacityTotal')) - remaining;
      return [used, remaining];
    }.property('service.capacityUsed', 'service.capacityTotal')
  }),

  metricsNotAvailableObserver: function () {
    if (!this.get("service.metricsNotAvailable")) {
      App.tooltip($("[rel='tooltip']"));
    }
  }.observes("service.metricsNotAvailable"),

  willDestroyElement: function willDestroyElement() {
    $("[rel='tooltip']").tooltip('destroy');
  },

  dataNodesDead: Em.computed.alias('service.dataNodesInstalled'),

  journalNodesLive: function () {
    return this.get('service.journalNodes').filterProperty("workStatus", "STARTED").get("length");
  }.property("service.journalNodes.@each.workStatus"),

  journalNodesTotal: Em.computed.alias('service.journalNodes.length'),

  dfsTotalBlocks: Em.computed.formatUnavailable('service.dfsTotalBlocks'),

  dfsTotalFiles: Em.computed.formatUnavailable('service.dfsTotalFiles'),

  dfsCorruptBlocks: Em.computed.formatUnavailable('service.dfsCorruptBlocks'),

  dfsMissingBlocks: Em.computed.formatUnavailable('service.dfsMissingBlocks'),

  dfsUnderReplicatedBlocks: Em.computed.formatUnavailable('service.dfsUnderReplicatedBlocks'),

  nodeUptime: function () {
    var uptime = this.get('service.nameNodeStartTime');
    if (uptime && uptime > 0) {
      var diff = App.dateTime() - uptime;
      if (diff < 0) {
        diff = 0;
      }
      var formatted = date.timingFormat(diff);
      return this.t('dashboard.services.uptime').format(formatted);
    }
    return this.t('services.service.summary.notRunning');
  }.property("service.nameNodeStartTime"),

  nodeHeapPercent: App.MainDashboardServiceView.formattedHeapPercent('dashboard.services.hdfs.nodes.heapUsedPercent', 'service.jvmMemoryHeapUsed', 'service.jvmMemoryHeapMax'),
  nodeHeap: App.MainDashboardServiceView.formattedHeap('dashboard.services.hdfs.nodes.heapUsed', 'service.jvmMemoryHeapUsed', 'service.jvmMemoryHeapMax'),

  dfsUsedDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'service.capacityTotal', 'service.capacityUsed'),
  dfsUsedDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'service.capacityTotal', 'service.capacityUsed'),

  nonDfsUsed: function () {
    var total = this.get('service.capacityTotal');
    var remaining = this.get('service.capacityRemaining');
    var dfsUsed = this.get('service.capacityUsed');
    return Em.isNone(total) || Em.isNone(remaining) || Em.isNone(dfsUsed) ? null : total - remaining - dfsUsed;
  }.property('service.capacityTotal', 'service.capacityRemaining', 'service.capacityUsed'),

  nonDfsUsedDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'service.capacityTotal', 'nonDfsUsed'),
  nonDfsUsedDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'service.capacityTotal', 'nonDfsUsed'),

  remainingDisk: diskPart('dashboard.services.hdfs.capacityUsed', 'service.capacityTotal', 'service.capacityRemaining'),
  remainingDiskPercent: diskPartPercent('dashboard.services.hdfs.capacityUsedPercent', 'service.capacityTotal', 'service.capacityRemaining'),

  dataNodeComponent: Em.Object.create({
    componentName: 'DATANODE'
  }),

  nfsGatewayComponent: Em.Object.create({
    componentName: 'NFS_GATEWAY'
  }),

  /**
   * Define if NFS_GATEWAY is present in the installed stack
   * @type {Boolean}
   */
  isNfsInStack: function () {
    return App.StackServiceComponent.find().someProperty('componentName', 'NFS_GATEWAY');
  }.property(),

  journalNodeComponent: Em.Object.create({
    componentName: 'JOURNALNODE'
  }),

  /**
   * @type {string}
   */
  safeModeStatus: function () {
    var safeMode = this.get('service.safeModeStatus');
    if (Em.isNone(safeMode)) {
      return Em.I18n.t("services.service.summary.notAvailable");
    } else if (safeMode.length === 0) {
      return Em.I18n.t("services.service.summary.safeModeStatus.notInSafeMode");
    } else {
      return Em.I18n.t("services.service.summary.safeModeStatus.inSafeMode");
    }
  }.property('service.safeModeStatus'),

  /**
   * @type {string}
   */
  upgradeStatus: function () {
    var upgradeStatus = this.get('service.upgradeStatus');
    var healthStatus = this.get('service.healthStatus');
    if (upgradeStatus == 'true') {
      return Em.I18n.t('services.service.summary.pendingUpgradeStatus.notPending');
    } else if (upgradeStatus == 'false' && healthStatus == 'green') {
      return Em.I18n.t('services.service.summary.pendingUpgradeStatus.notFinalized');
    } else {
      // upgrade status == null
      return Em.I18n.t("services.service.summary.notAvailable");
    }
  }.property('service.upgradeStatus', 'service.healthStatus'),

  /**
   * @type {boolean}
   */
  isUpgradeStatusWarning: function () {
    return this.get('service.upgradeStatus') == 'false' && this.get('service.healthStatus') == 'green';
  }.property('service.upgradeStatus', 'service.healthStatus'),

  isDataNodeCreated: function () {
    return this.isServiceComponentCreated('DATANODE');
  }.property('App.router.clusterController.isComponentsStateLoaded'),

  isJournalNodeCreated: function () {
    return this.isServiceComponentCreated('JOURNALNODE');
  }.property('App.router.clusterController.isComponentsStateLoaded')
});

});

require.register("views/main/service/services/ranger", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

App.MainDashboardServiceRangerView = App.MainDashboardServiceView.extend({

  templateName: require('templates/main/service/services/ranger'),

  serviceName: 'RANGER',

  rangerTagsyncComponent: Em.Object.create({
    componentName: 'RANGER_TAGSYNC'
  }),

  isRangerTagsyncCreated: function () {
    return this.isServiceComponentCreated('RANGER_TAGSYNC');
  }.property('App.router.clusterController.isComponentsStateLoaded'),

  didInsertElement: function didInsertElement() {
    this.set('controller.isRangerUpdateWorking', true);
    App.updater.run(this.get('controller'), 'updateRangerPluginsStatus', 'isRangerUpdateWorking', App.bgOperationsUpdateInterval);
    App.updater.immediateRun('updateRangerPluginsStatus');
  },

  willDestroyElement: function willDestroyElement() {
    this.set('controller.isRangerUpdateWorking', false);
  }

});

});

require.register("views/main/service/services/storm", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainDashboardServiceStormView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/storm'),
  serviceName: 'STORM',

  /**
   * this parameter is used to fiter hosts by component name
   * used in mainHostController.filterByComponent() method
   */
  filterComponent: Em.Object.create({ componentName: 'SUPERVISOR' }),

  freeSlotsPercentage: Em.computed.percents('service.freeSlots', 'service.totalSlots'),

  superVisorsLive: Em.computed.alias('service.superVisorsStarted'),

  superVisorsTotal: Em.computed.alias('service.superVisorsTotal'),

  nimbusUptimeFormatted: function () {
    return this.get('service.nimbusUptime') || Em.I18n.t('services.service.summary.notRunning');
  }.property('service.nimbusUptime'),

  isSupervisorCreated: function () {
    return this.isServiceComponentCreated('SUPERVISOR');
  }.property('App.router.clusterController.isComponentsStateLoaded')
});

});

require.register("views/main/service/services/yarn", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');
var date = require('utils/date/date');
var numberUtils = require('utils/number_utils');

App.MainDashboardServiceYARNView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/yarn'),
  serviceName: 'YARN',

  nodeHeap: App.MainDashboardServiceView.formattedHeap('dashboard.services.yarn.nodes.heapUsed', 'service.jvmMemoryHeapUsed', 'service.jvmMemoryHeapMax'),
  nodeHeapPercent: App.MainDashboardServiceView.formattedHeapPercent('dashboard.services.yarn.nodes.heapUsedPercent', 'service.jvmMemoryHeapUsed', 'service.jvmMemoryHeapMax'),

  nodeManagerComponent: Em.Object.create({
    componentName: 'NODEMANAGER'
  }),

  yarnClientComponent: Em.Object.create({
    componentName: 'YARN_CLIENT'
  }),

  hasManyYarnClients: Em.computed.gt('service.installedClients', 1),

  nodeUptime: function () {
    var uptime = this.get('service.resourceManagerStartTime');
    if (uptime && uptime > 0) {
      var diff = App.dateTime() - uptime;
      if (diff < 0) {
        diff = 0;
      }
      var formatted = date.timingFormat(diff);
      return this.t('dashboard.services.uptime').format(formatted);
    }
    return this.t('services.service.summary.notRunning');
  }.property("service.resourceManagerStartTime"),

  nodeManagerText: Em.computed.countBasedMessage('service.nodeManagersTotal', '', Em.I18n.t('services.service.summary.viewHost'), Em.I18n.t('services.service.summary.viewHosts')),

  _nmActive: Em.computed.formatUnavailable('service.nodeManagersCountActive'),
  _nmLost: Em.computed.formatUnavailable('service.nodeManagersCountLost'),
  _nmUnhealthy: Em.computed.formatUnavailable('service.nodeManagersCountUnhealthy'),
  _nmRebooted: Em.computed.formatUnavailable('service.nodeManagersCountRebooted'),
  _nmDecom: Em.computed.formatUnavailable('service.nodeManagersCountDecommissioned'),

  _allocated: Em.computed.formatUnavailable('service.containersAllocated'),
  _pending: Em.computed.formatUnavailable('service.containersPending'),
  _reserved: Em.computed.formatUnavailable('service.containersReserved'),

  _appsSubmitted: Em.computed.formatUnavailable('service.appsSubmitted'),
  _appsRunning: Em.computed.formatUnavailable('service.appsRunning'),
  _appsPending: Em.computed.formatUnavailable('service.appsPending'),
  _appsCompleted: Em.computed.formatUnavailable('service.appsCompleted'),
  _appsKilled: Em.computed.formatUnavailable('service.appsKilled'),
  _appsFailed: Em.computed.formatUnavailable('service.appsFailed'),

  allocatedMemoryFormatted: function () {
    return numberUtils.bytesToSize(this.get('service.allocatedMemory'), 1, 'parseFloat', 1024 * 1024);
  }.property('service.allocatedMemory'),

  reservedMemoryFormatted: function () {
    return numberUtils.bytesToSize(this.get('service.reservedMemory'), 1, 'parseFloat', 1024 * 1024);
  }.property('service.reservedMemory'),

  availableMemoryFormatted: function () {
    return numberUtils.bytesToSize(this.get('service.availableMemory'), 1, 'parseFloat', 1024 * 1024);
  }.property('service.availableMemory'),

  _queuesCountFormatted: Em.computed.formatUnavailable('service.queuesCount'),
  queues: Em.computed.i18nFormat('dashboard.services.yarn.queues.msg', '_queuesCountFormatted'),

  didInsertElement: function didInsertElement() {
    App.tooltip($("[rel='queue-tooltip']"), { html: true, placement: "right" });
    App.tooltip($("[rel='tooltip']"));
  },

  willDestroyElement: function willDestroyElement() {
    $("[rel='queue-tooltip']").tooltip('destroy');
  },

  isNodeManagerCreated: function () {
    return this.isServiceComponentCreated('NODEMANAGER');
  }.property('App.router.clusterController.isComponentsStateLoaded')
});

});

require.register("views/main/service/services/zookeeper", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainDashboardServiceZookeperView = App.MainDashboardServiceView.extend({
  templateName: require('templates/main/service/services/zookeeper'),
  serviceName: 'zookeeper',

  titleInfo: function () {
    var components = this.get('service.hostComponents').filterProperty('componentName', 'ZOOKEEPER_SERVER');
    var running = 0;
    components.forEach(function (item) {
      if (item.get('workStatus') === App.HostComponentStatus.started) {
        running++;
      }
    });

    return {
      pre: this.t('services.zookeeper.prefix').format(running),
      title: this.t('services.zookeeper.title').format(components.length),
      component: components.objectAt(0)
    };
  }.property('service')
});

});

require.register("views/main/service/widgets/create/expression_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var misc = require('utils/misc');
var number_utils = require("utils/number_utils");

App.WidgetWizardExpressionView = Em.View.extend({
  templateName: require('templates/main/service/widgets/create/expression'),

  /**
   * @type {Array}
   */
  classNames: ['metric-container'],

  /**
   * @type {Array}
   */
  classNameBindings: ['isInvalid'],

  /**
   * list of operators that can be used in expression
   * @type {Array}
   * @constant
   */
  OPERATORS: ["+", "-", "*", "/", "(", ")"],

  /**
   * @type {Array}
   * @const
   */
  AGGREGATE_FUNCTIONS: ['avg', 'sum', 'min', 'max', 'rate'],

  /**
   * @type {RegExp}
   * @const
   */
  VALID_EXPRESSION_REGEX: /^((\(\s)*[\d]+)[\(\)\+\-\*\/\.\d\s]*[\d\)]*$/,

  /**
   * contains expression data before editing in order to restore previous state
   */
  dataBefore: [],

  /**
   * @type {Ember.Object}
   */
  expression: null,

  /**
   * @type {boolean}
   */
  isInvalid: false,

  /**
   * @type {boolean}
   */
  isInvalidTextfield: false,

  /**
   * contains value of number added to expression
   * @type {string}
   */
  numberValue: "",

  /**
   * @type {boolean}
   */
  isNumberValueInvalid: function () {
    return this.get('numberValue').trim() === "" || !number_utils.isPositiveNumber(this.get('numberValue').trim());
  }.property('numberValue'),

  /**
   * add operator to expression data
   * @param event
   */
  addOperator: function addOperator(event) {
    var data = this.get('expression.data');
    var lastId = data.length > 0 ? Math.max.apply(this, data.mapProperty('id')) : 0;

    data.pushObject(Em.Object.create({
      id: ++lastId,
      name: event.context,
      isOperator: true
    }));
  },

  /**
   * add operator to expression data
   */
  addNumber: function addNumber() {
    var data = this.get('expression.data');
    var lastId = data.length > 0 ? Math.max.apply(this, data.mapProperty('id')) : 0;

    data.pushObject(Em.Object.create({
      id: ++lastId,
      name: this.get('numberValue'),
      isNumber: true
    }));
    this.set('numberValue', "");
  },

  /**
   * redraw expression
   * NOTE: needed in order to avoid collision between scrollable lib and metric action event
   */
  redrawField: function redrawField() {
    this.set('expression.data', misc.sortByOrder($(this.get('element')).find('.metric-instance').map(function () {
      return this.id;
    }), this.get('expression.data')));
  },

  /**
   * enable metric edit area
   */
  didInsertElement: function didInsertElement() {
    var self = this;
    this.propertyDidChange('expression');
    Em.run.next(function () {
      $(self.get('element')).find('.metric-field').sortable({
        items: "> .metric-instance",
        tolerance: "pointer",
        scroll: false,
        update: function update() {
          self.redrawField();
        }
      }).disableSelection();
    });
  },

  /**
   * remove metric or operator from expression
   * @param {object} event
   */
  removeElement: function removeElement(event) {
    this.get('expression.data').removeObject(event.context);
  },

  validate: function () {
    //number 1 used as substitute to test expression to be mathematically correct
    var testNumber = 1;
    var isInvalid = true;
    var expression = this.get('expression.data').map(function (element) {
      if (element.isMetric) {
        return testNumber;
      } else {
        return element.name;
      }
    }, this).join(" ");

    if (expression.length > 0) {
      if (this.get('VALID_EXPRESSION_REGEX').test(expression)) {
        try {
          isInvalid = !isFinite(window.eval(expression));
        } catch (e) {
          isInvalid = true;
        }
      }
    } else {
      isInvalid = false;
    }

    this.set('isInvalid', isInvalid);
    this.set('expression.isInvalid', isInvalid);
    this.get('controller').propertyDidChange('isSubmitDisabled');
    if (!isInvalid) {
      this.get('controller').updateExpressions();
    }
  }.observes('expression.data.length')
});

/**
 * input used to add number to expression
 * @type {Em.TextField}
 * @class
 */
App.AddNumberExpressionView = Em.TextField.extend({
  classNameBindings: ['isInvalid'],

  /**
   * @type {boolean}
   */
  isInvalid: function () {
    return this.get('value').trim().length > 0 && !number_utils.isPositiveNumber(this.get('value').trim());
  }.property('value')
});

/**
 * show menu view that provide ability to add metric
 */
App.AddMetricExpressionView = Em.View.extend({
  templateName: require('templates/main/service/widgets/create/step2_add_metric'),
  controller: Em.computed.alias('parentView.controller'),
  elementId: Em.computed.format('add-metric-menu_{0}', 'parentView.expression.id'),
  didInsertElement: function didInsertElement() {
    //prevent dropdown closing on click select
    $('html').on('click.dropdown', '.dropdown-menu li', function (e) {
      $(this).hasClass('keep-open') && e.stopPropagation();
    });
  },

  metricsSelectionObj: function () {
    var self = this;
    return Em.Object.create({
      placeholder_text: Em.I18n.t('dashboard.widgets.wizard.step2.selectMetric'),
      no_results_text: Em.I18n.t('widget.create.wizard.step2.noMetricFound'),
      onChangeCallback: function onChangeCallback(event, obj) {
        var filteredComponentMetrics = self.get('controller.filteredMetrics').filterProperty('component_name', self.get('currentSelectedComponent.componentName')).filterProperty('level', self.get('currentSelectedComponent.level'));
        var filteredMetric = filteredComponentMetrics.findProperty('name', obj.selected);
        var selectedMetric = Em.Object.create({
          name: obj.selected,
          componentName: self.get('currentSelectedComponent.componentName'),
          serviceName: self.get('currentSelectedComponent.serviceName'),
          metricPath: filteredMetric.widget_id,
          isMetric: true
        });
        if (self.get('currentSelectedComponent.hostComponentCriteria')) {
          selectedMetric.hostComponentCriteria = self.get('currentSelectedComponent.hostComponentCriteria');
        }
        if (self.get('currentSelectedComponent.tag')) {
          selectedMetric.tag = self.get('currentSelectedComponent.tag');
        }
        self.set('currentSelectedComponent.selectedMetric', selectedMetric);
        if (self.get('currentSelectedComponent.selectedAggregation') == Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.scanOps')) {
          var defaultAggregator = self.get('parentView.AGGREGATE_FUNCTIONS')[0];
          self.set('currentSelectedComponent.selectedAggregation', defaultAggregator);
        }
      }
    });
  }.property(),

  aggregateFnSelectionObj: function () {
    var self = this;
    return Em.Object.create({
      placeholder_text: Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.scanOps'),
      no_results_text: Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.notFound'),
      onChangeCallback: function onChangeCallback(event, obj) {
        self.set('currentSelectedComponent.selectedAggregation', obj.selected);
      }
    });
  }.property(),

  /**
   * @type {Ember.Object}
   * @default null
   */
  currentSelectedComponent: null,

  /**
   * select component
   * @param {object} event
   */
  selectComponents: function selectComponents(event) {
    this.set('currentSelectedComponent', event.context);
    event.stopPropagation();
  },

  /**
   * add current metrics and aggregation to expression
   * @param event
   */
  addMetric: function addMetric(event) {
    var selectedMetric = event.context.get('selectedMetric'),
        aggregateFunction = event.context.get('selectedAggregation'),
        result = Em.Object.create(selectedMetric);

    if (event.context.get('isAddEnabled')) {
      var data = this.get('parentView').get('expression.data'),
          id = data.length > 0 ? Math.max.apply(this.get('parentView'), data.mapProperty('id')) + 1 : 1;
      result.set('id', id);
      if (event.context.get('showAggregateSelect')) {
        result.set('metricPath', result.get('metricPath') + '._' + aggregateFunction);
        result.set('name', result.get('name') + '._' + aggregateFunction);
      }
      data.pushObject(result);
      this.cancel();
    }
  },

  /**
   * cancel adding metric, close add metric menu
   */
  cancel: function cancel() {
    $(".service-level-dropdown").parent().removeClass('open');
    var id = "#" + this.get('currentSelectedComponent.id');
    var aggregatorId = "#" + this.get('currentSelectedComponent.aggregatorId');
    $(id).val('').trigger("chosen:updated");
    $(aggregatorId).val('').trigger("chosen:updated");
    this.set('currentSelectedComponent.selectedAggregation', Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.scanOps'));
    this.set('currentSelectedComponent.selectedMetric', null);
  },

  /**
   * map of components
   * has following hierarchy: service -> component -> metrics
   */
  componentMap: function () {
    var hasNameNodeFederation = App.get('hasNameNodeFederation');
    var servicesMap = {};
    var result = [];
    var nameServiceGroups = [];
    var masterNames = App.StackServiceComponent.find().filterProperty('isMaster').mapProperty('componentName');
    var parentView = this.get('parentView');
    var expressionId = "_" + parentView.get('expression.id');
    if (this.get('controller.filteredMetrics')) {
      this.get('controller.filteredMetrics').forEach(function (metric) {
        // ignore NameNode component level metrics on federated cluster
        if (hasNameNodeFederation && metric.component_name === 'NAMENODE' && metric.level === 'COMPONENT') return false;

        var service = servicesMap[metric.service_name];
        if (!service) {
          service = {
            count: 0,
            components: {}
          };
          servicesMap[metric.service_name] = service;
        }

        var componentId = masterNames.contains(metric.component_name) ? metric.component_name + '_' + metric.level : metric.component_name;
        service.count++;
        if (service.components[componentId]) {
          service.components[componentId].count++;
          service.components[componentId].metrics.push(metric.name);
        } else {
          service.components[componentId] = {
            component_name: metric.component_name,
            level: metric.level,
            count: 1,
            hostComponentCriteria: metric.host_component_criteria,
            metrics: [metric.name]
          };
        }
      }, this);
    }

    if (hasNameNodeFederation) {
      App.HDFSService.find().objectAt(0).get('masterComponentGroups').forEach(function (group) {
        nameServiceGroups.push({
          tag: group.name,
          displayName: Em.I18n.t('dashboard.widgets.wizard.step2.nameSpaceDropDownItem').format(group.name),
          component: null
        });
      });
    }

    for (var serviceName in servicesMap) {
      var components = [];
      for (var componentId in servicesMap[serviceName].components) {
        // Hide the option if none of the hostComponent is created in the cluster yet
        var componentName = servicesMap[serviceName].components[componentId].component_name;
        if (App.HostComponent.getCount(componentName, 'totalCount') === 0) continue;
        if (hasNameNodeFederation && componentName === 'NAMENODE') {
          nameServiceGroups.forEach(function (group) {
            group.component = this.createComponentItem(servicesMap[serviceName], serviceName, componentId, expressionId, group.tag);
          }, this);
        } else {
          components.push(this.createComponentItem(servicesMap[serviceName], serviceName, componentId, expressionId));
        }
      }
      if (hasNameNodeFederation && serviceName === 'HDFS') {
        components.push(Em.Object.create({
          displayName: 'NameNodes',
          isGroup: true,
          components: nameServiceGroups
        }));
      }
      result.push(Em.Object.create({
        serviceName: serviceName,
        //in order to support panel lists
        href: '#' + serviceName,
        displayName: App.StackService.find(serviceName).get('displayName'),
        count: servicesMap[serviceName].count,
        components: components
      }));
    }

    return this.putContextServiceOnTop(result);
  }.property('controller.filteredMetrics', 'App.router.clusterController.isComponentsStateLoaded'),

  createComponentItem: function createComponentItem(service, serviceName, componentId, expressionId, tag) {
    var stackComponent = App.StackServiceComponent.find(service.components[componentId].component_name);
    var component = service.components[componentId];
    tag = tag || '';
    return Em.Object.create({
      componentName: component.component_name,
      level: component.level,
      displayName: stackComponent.get('isMaster') && component.level === 'HOSTCOMPONENT' ? Em.I18n.t('widget.create.wizard.step2.activeComponents').format(stackComponent.get('displayName')) : Em.I18n.t('widget.create.wizard.step2.allComponents').format(pluralize(stackComponent.get('displayName'))),
      tag: tag,
      count: component.count,
      metrics: component.metrics.uniq().sort(),
      selected: false,
      id: componentId + expressionId + tag,
      aggregatorId: componentId + expressionId + '_aggregator',
      serviceName: serviceName,
      showAggregateSelect: Em.computed.equal('level', 'COMPONENT'),
      selectedMetric: null,
      selectedAggregation: Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.scanOps'),
      hostComponentCriteria: component.level === 'HOSTCOMPONENT' ? component.hostComponentCriteria : null,
      isAddEnabled: function () {
        var selectedMetric = this.get('selectedMetric'),
            aggregateFunction = this.get('selectedAggregation');
        if (this.get('showAggregateSelect')) {
          return !!selectedMetric && !!aggregateFunction && aggregateFunction != Em.I18n.t('dashboard.widgets.wizard.step2.aggregateFunction.scanOps');
        } else {
          return !!selectedMetric;
        }
      }.property('selectedMetric', 'selectedAggregation')
    });
  },

  /**
   * returns the input array with the context service (service from which widget browser is launched) as the first element of the array
   * @param serviceComponentMap {Array}
   * @return {Array}
   */
  putContextServiceOnTop: function putContextServiceOnTop(serviceComponentMap) {
    var contextService = this.get('controller.content.widgetService');
    var serviceIndex = serviceComponentMap.indexOf(serviceComponentMap.findProperty('serviceName', contextService));
    return serviceComponentMap.slice(serviceIndex, serviceComponentMap.length).concat(serviceComponentMap.slice(0, serviceIndex));
  }

});

App.InputCursorTextfieldView = Ember.TextField.extend({
  placeholder: "",
  classNameBindings: ['isInvalid'],
  isInvalid: false,

  didInsertElement: function didInsertElement() {
    this.focusCursor();
  },

  focusCursor: function () {
    var self = this;
    Em.run.next(function () {
      if (self.$()) {
        self.$().focus();
      }
    });
  }.observes('parentView.expression.data.length'),

  focusOut: function focusOut() {
    this.saveNumber();
  },

  validateInput: function () {
    var value = this.get('value');
    var parentView = this.get('parentView');
    var isInvalid = false,
        isInvalidTextfield = false;

    if (!number_utils.isPositiveNumber(value)) {
      if (value && parentView.get('OPERATORS').contains(value)) {
        // add operator
        var data = parentView.get('expression.data');
        var lastId = data.length > 0 ? Math.max.apply(parentView, data.mapProperty('id')) : 0;
        data.pushObject(Em.Object.create({
          id: ++lastId,
          name: value,
          isOperator: true
        }));
        this.set('value', '');
      } else if (value && value === 'm') {
        // open add metric menu
        var expressionId = "_" + parentView.get('expression.id');
        $('#add-metric-menu' + expressionId + '> div > a').click();
        this.set('value', '');
      } else if (value) {
        // invalid operator
        isInvalid = isInvalidTextfield = true;
      }
    }
    this.set('isInvalid', isInvalid);
    this.set('parentView.isInvalid', isInvalid);
    this.set('parentView.isInvalidTextfield', isInvalidTextfield);
  }.observes('value'),

  keyDown: function keyDown(event) {
    if ((event.keyCode === 8 || event.which === 8) && !this.get('value')) {
      // backspace
      var data = this.get('parentView.expression.data');
      if (data.length >= 1) {
        data.removeObject(data[data.length - 1]);
      }
    } else if (event.keyCode === 13) {
      //Enter
      this.saveNumber();
    }
  },

  saveNumber: function saveNumber() {
    if (number_utils.isPositiveNumber(this.get('value'))) {
      var data = this.get('parentView.expression.data');
      var lastId = data.length > 0 ? Math.max.apply(this, data.mapProperty('id')) : 0;
      data.pushObject(Em.Object.create({
        id: ++lastId,
        name: this.get('value'),
        isNumber: true
      }));
      this.set('numberValue', "");
      this.set('isInvalid', false);
      this.set('parentView.isInvalid', false);
      this.set('parentView.isInvalidTextfield', false);
      this.set('value', '');
    }
  }
});

});

require.register("views/main/service/widgets/create/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetWizardStep1View = Em.View.extend({

  templateName: require('templates/main/service/widgets/create/step1'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
    Em.run.later(this, function () {
      App.tooltip($("[rel='selectable-tooltip']"));
    }, 300);
  }

});

});

require.register("views/main/service/widgets/create/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.WidgetWizardStep2View = Em.View.extend({

  templateName: require('templates/main/service/widgets/create/step2'),
  templatePlaceholder: "{{Expression1}}",

  /**
   * calculate template by widget type
   * @type {object}
   */
  templateType: function () {
    switch (this.get('controller.content.widgetType')) {
      case 'GAUGE':
      case 'NUMBER':
        return {
          isNumber: true
        };
      case 'TEMPLATE':
        return {
          isTemplate: true
        };
      case 'GRAPH':
        return {
          isGraph: true
        };
      default:
        return {};
    }
  }.property('controller.content.widgetType'),

  ensureTooltip: function ensureTooltip() {
    Em.run.later(this, function () {
      App.tooltip($("[rel='threshold-tooltip']"));
    }, 500);
  },

  didInsertElement: function didInsertElement() {
    var controller = this.get('controller');
    this.ensureTooltip();
    controller.convertData();
    controller.initWidgetData();
    controller.renderProperties();
    controller.updateExpressions();
  }
});

App.WidgetPropertyTextFieldView = Em.TextField.extend({
  valueBinding: 'property.value',
  placeholderBinding: 'property.placeholder',
  classNames: ['form-control'],
  classNameBindings: ['property.classNames', 'parentView.basicClass']
});

App.WidgetPropertyThresholdView = Em.View.extend({
  templateName: require('templates/main/service/widgets/create/widget_property_threshold'),
  classNameBindings: ['property.classNames', 'parentView.basicClass']
});

App.WidgetPropertySelectView = Em.Select.extend({
  contentBinding: 'property.options',
  classNameBindings: ['property.classNames', 'parentView.basicClass'],
  classNames: ['form-control'],
  optionLabelPath: "content.label",
  optionValuePath: "content.value",
  didInsertElement: function didInsertElement() {
    var selection = this.get('content').findProperty('value', this.get('property.value'));
    if (selection) {
      this.set('selection', selection);
    }
    this.addObserver('selection.value', this, 'setValue');
    this.setValue();
  },
  setValue: function setValue() {
    this.set('property.value', this.get('selection.value'));
  }
});

App.WidgetWizardComponentDropdownView = Em.View.extend({

  templateName: require('templates/main/service/widgets/create/step2_component_dropdown'),

  tagName: 'li',

  classNames: ['dropdown-submenu'],

  displayName: ''

});

});

require.register("views/main/service/widgets/create/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

App.WidgetWizardStep3View = Em.View.extend({

  templateName: require('templates/main/service/widgets/create/step3'),

  didInsertElement: function didInsertElement() {
    this.get('controller').initPreviewData();
  }

});

});

require.register("views/main/service/widgets/create/wizard_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetWizardView = Em.View.extend(App.WizardMenuMixin, {

  templateName: require('templates/main/service/widgets/create/wizard'),

  /**
   * @type {App.Widget}
   */
  previewWidgetClass: function () {
    switch (this.get('controller.content.widgetType')) {
      case 'GRAPH':
        return App.GraphWidgetView.extend(App.WidgetPreviewMixin);
      case 'TEMPLATE':
        return App.TemplateWidgetView.extend(App.WidgetPreviewMixin);
      case 'NUMBER':
        return App.NumberWidgetView.extend(App.WidgetPreviewMixin);
      case 'GAUGE':
        return App.GaugeWidgetView.extend(App.WidgetPreviewMixin);
      default:
        return Em.View;
    }
  }.property('controller.content.widgetType'),

  /**
   * Widget preview should be shown on 2nd step of wizard
   * @type {boolean}
   */
  isStep2: function () {
    return this.get('controller.currentStep') == "2";
  }.property('controller.currentStep'),

  /**
   * Widget preview should be shown on 3rd step of wizard
   * @type {boolean}
   */
  isStep3: function () {
    return this.get('controller.currentStep') == "3";
  }.property('controller.currentStep')
});

});

require.register("views/main/service/widgets/edit_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WidgetEditView = App.WidgetWizardView.extend({

  templateName: require('templates/main/service/widgets/edit'),

  /**
   * Widget preview should be shown on 2nd step of wizard
   * @type {boolean}
   */
  isStep1: function () {
    return this.get('controller.currentStep') == "1";
  }.property('controller.currentStep'),

  /**
   * Widget preview should be shown on 3rd step of wizard
   * @type {boolean}
   */
  isStep2: function () {
    return this.get('controller.currentStep') == "2";
  }.property('controller.currentStep')
});

});

require.register("views/main/views/details", function(exports, require, module) {
"use strict";

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainViewsDetailsView = Em.View.extend({

  name: "mainViewsDetailsView",

  tagName: "iframe",
  classNames: ["views_sizes"],
  attributeBindings: ['src', 'seamless', 'allowfullscreen'],
  seamless: "seamless",
  allowfullscreen: "true",
  interval: null,

  /**
   * Drop autoHeight timer
   */
  willDestroyElement: function willDestroyElement() {
    var interval = this.get('interval');
    if (interval) {
      clearInterval(interval);
    }
  },

  /**
   * Updates iframe height every 5s
   */
  didInsertElement: function didInsertElement() {
    var interval,
        self = this;
    interval = setInterval(function () {
      self.resizeFunction();
    }, 5000);
    self.set('interval', interval);
    this.resizeFunction();
    App.router.get('mainController').monitorInactivity();
  },

  resizeFunction: function resizeFunction() {
    var body = $(document.body);
    var footer = $("footer", body);
    var header = $("#top-nav", body);
    var iframe = $("iframe", body);

    var bodyHeight = body.outerHeight();
    var footerHeight = footer != null ? footer.outerHeight() : 0;
    var headerHeight = header != null ? header.outerHeight() : 0;

    var defaultHeight = bodyHeight - footerHeight - headerHeight;

    if (iframe != null && iframe.length > 0) {
      var childrenHeight = 0;
      var iframeElement = iframe[0];
      var pageScrollTop = $(window).scrollTop();
      // set iframe height to 'auto' to get actual scrollHeight
      iframeElement.style.height = 'auto';
      if (iframeElement.contentWindow != null && iframeElement.contentWindow.document != null && iframeElement.contentWindow.document.body != null) {
        var iFrameContentBody = iframeElement.contentWindow.document.body;
        childrenHeight = iFrameContentBody.scrollHeight;
      }
      var iFrameHeight = Math.max(childrenHeight, defaultHeight);
      iframe.css('height', iFrameHeight);
      $(window).scrollTop(pageScrollTop);
    }
  },

  src: function () {
    // can't use window.location.origin because IE doesn't support it
    return window.location.protocol + '//' + window.location.host + this.get('controller.content.href') + this.get('controller.content.viewPath');
  }.property('controller.content')

});

});

require.register("views/main/views_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.MainViewsView = Em.View.extend({

  name: "mainViewsView",
  templateName: require('templates/main/views'),
  classNames: [""],

  views: function () {
    return App.router.get('mainViewsController.ambariViews');
  }.property('App.router.mainViewsController.ambariViews.[]'),

  selectedViewBinding: 'controller.selectedView'

});

});

require.register("views/wizard/step0_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep0View = Em.View.extend({

  tagName: "form",

  attributeBindings: ['autocomplete'],

  /**
   * Disable autocomplete for form's fields
   */
  autocomplete: 'off',

  templateName: require('templates/wizard/step0'),

  didInsertElement: function didInsertElement() {
    App.popover($("[rel=popover]"), { 'placement': 'right', 'trigger': 'hover' });
    this.get('controller').loadStep();
  },

  /**
   * Is some error with provided cluster name
   * @type {bool}
   */
  onError: function () {
    return !Em.isEmpty(this.get('controller.clusterNameError'));
  }.property('controller.clusterNameError')

});

/**
 * Field for cluster name
 * @type {Ember.TextField}
 */
App.WizardStep0ViewClusterNameInput = Em.TextField.extend({

  /**
   * Submit form if "Enter" pressed
   * @method keyPress
   * @param {object} event
   * @returns {bool}
   */
  keyPress: function keyPress(event) {
    if (event.keyCode == 13) {
      this.get('parentView.controller').submit();
      return false;
    }
    return true;
  }
});

});

require.register("views/wizard/step10_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep10View = Em.View.extend({

  templateName: require('templates/wizard/step10'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  serviceRestartText: function () {
    var controller = this.get('controller');
    if (controller.get('isAddServiceWizard')) return Em.I18n.t('common.important.strong') + Em.I18n.t('installer.step10.staleServicesRestartRequired');else return '';
  }.property('controller.isAddServiceWizard')
});

});

require.register("views/wizard/step1_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with this
 * work for additional information regarding copyright ownership. The ASF
 * licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

var App = require('app');

require('models/repository');

App.WizardStep1View = Em.View.extend({

  templateName: require('templates/wizard/step1'),

  didInsertElement: function () {
    var _this = this;

    if (this.get('controller.isLoadingComplete') && this.get('state') === 'inDOM') {
      Em.run.next(function () {
        $("[rel=skip-validation-tooltip]").tooltip({ placement: 'right' });
        $("[rel=use-redhat-tooltip]").tooltip({ placement: 'right' });
        $('.add-os-button,.redhat-label').tooltip();
        _this.$().on('mouseover', '.version-contents-body .table-hover > tbody > tr', function () {
          App.tooltip($(this).find('.action .icon'), { placement: 'bottom' });
          App.tooltip($(this).find('.icon-undo'), { placement: 'bottom' });
        });
        if (_this.get('controller.selectedStack.showAvailable')) {
          // first time load
          if (_this.get('controller.selectedStack.useRedhatSatellite')) {
            // restore `use local repo` on page refresh
            _this.get('controller').useLocalRepo();
          }
        } else {
          var selected = _this.get('controller.content.stacks') && _this.get('controller.content.stacks').findProperty('showAvailable');
          if (!selected) {
            // network disconnection
            Em.trySet(_this, 'controller.selectedStack.useLocalRepo', true);
            Em.trySet(_this, 'controller.selectedStack.usePublicRepo', false);
          }
        }
      });
    }
  }.observes('controller.isLoadingComplete'),

  willDestroyElement: function willDestroyElement() {
    $("[rel=skip-validation-tooltip]").tooltip('destroy');
    $("[rel=use-redhat-tooltip]").tooltip('destroy');
    $('.add-os-button,.redhat-label').tooltip('destroy');
    $('.icon-undo').tooltip('destroy');
    $('.action .icon').tooltip('destroy');
  },

  /**
   * Show possible reasons why Public Repo is disabled
   *
   * @returns {App.ModalPopup}
   */
  openPublicOptionDisabledWindow: function openPublicOptionDisabledWindow() {
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step1.selectUseRepoOptions.public.networkLost.popup.title'),
      bodyClass: Ember.View.extend({
        templateName: require('templates/wizard/step1/public_option_disabled_window_body')
      }),
      secondary: false
    });
  },

  removeOS: function removeOS() {
    var _get;

    $('.action .icon').tooltip('destroy');
    return (_get = this.get('controller')).removeOS.apply(_get, arguments);
  },


  editableRepoView: Em.View.extend({
    templateName: require('templates/wizard/step1/editable_repo'),
    classNames: ['editable-repo'],
    /**
     * @type {boolean}
     */
    isEditing: false,

    /**
     * @type {?App.Repository}
     */
    repository: null,

    /**
     * @type {boolean}
     */
    showEditIcon: Em.computed.and('controller.selectedStack.useRedhatSatellite', '!isEditing'),

    /**
     * @type {boolean}
     */
    showRevertIcon: function () {
      return this.get('isEditing') && this.get('repository.repoId') !== this.get('repository.originalRepoId');
    }.property('isEditing', 'repository.repoId'),

    didInsertElement: function () {
      this.set('isEditing', false);
    }.observes('controller.selectedStack.useRedhatSatellite'),

    revertToOriginal: function revertToOriginal() {
      this.set('repository.repoId', this.get('repository.originalRepoId'));
    },

    editRepoId: function editRepoId() {
      this.set('isEditing', true);
    }
  }),

  /**
   * Disable submit button flag
   *
   * @type {bool}
   */
  isSubmitDisabled: Em.computed.or('invalidFormatUrlExist', 'isNoOsChecked', 'isNoOsFilled', 'controller.content.isCheckInProgress', 'App.router.btnClickInProgress', '!controller.isLoadingComplete'),

  /**
   * Show warning message flag
   *
   * @type {bool}
   */
  warningExist: Em.computed.or('invalidFormatUrlExist', 'isNoOsChecked', 'isNoOsFilled'),

  skipVerifyBaseUrl: Em.computed.or('controller.selectedStack.skipValidationChecked', 'controller.selectedStack.useRedhatSatellite'),

  verifyBaseUrl: Em.computed.not('skipVerifyBaseUrl'),

  showWarning: Em.computed.and('warningExist', 'verifyBaseUrl'),

  /**
   * Onclick handler for recheck repos urls. Used in Advanced Repository Options.
   */
  retryRepoUrls: function retryRepoUrls() {
    App.router.get('installerController').checkRepoURL(this.get('controller'));
  },

  /**
   * Radio button for use Public repo
   *
   * @type {App.RadioButtonView}
   */
  usePublicRepoRadioButton: App.RadioButtonView.extend({
    labelTranslate: 'installer.step1.selectUseRepoOptions.public',
    checked: Em.computed.alias('controller.selectedStack.usePublicRepo'),

    change: function change() {
      this.get('controller').usePublicRepo();
    }
  }),

  /**
   * Checkbox for use Public repo
   *
   * @type {App.RadioButtonView}
   */
  useLocalRepoRadioButton: App.RadioButtonView.extend({
    labelTranslate: 'installer.step1.selectUseRepoOptions.local',
    checked: Em.computed.alias('controller.selectedStack.useLocalRepo'),

    change: function change() {
      this.get('controller').useLocalRepo();
    }
  }),

  /**
   * User already selected all OSes
   *
   * @type {boolean}
   */
  allOsesSelected: Em.computed.everyBy('controller.selectedStack.operatingSystems', 'isSelected', true),

  /**
   * Disallow adding OS if all OSes are already added or user select <code>useRedhatSatellite</code>
   *
   * @type {boolean}
   */
  isAddOsButtonDisabled: Em.computed.or('allOsesSelected', 'controller.selectedStack.useRedhatSatellite'),

  /**
   * Tooltip for Add OS Button
   * Empty if this button is enabled
   *
   * @type {string}
   */
  addOsButtonTooltip: Em.computed.ifThenElse('allOsesSelected', Em.I18n.t('installer.step1.addOs.disabled.tooltip'), ''),

  /**
   * Tooltip for useRedhatSatellite block
   * Empty if usage Redhat is enabled
   *
   * @type {string}
   */
  redhatDisabledTooltip: Em.computed.ifThenElse('controller.selectedStack.usePublicRepo', Em.I18n.t('installer.step1.advancedRepo.useRedhatSatellite.disabled.tooltip'), ''),
  /**
   * List of all repositories under selected stack operatingSystems
   *
   * @type {App.Repository[]}
   */
  allRepositories: function () {
    return this.getWithDefault('controller.selectedStack.repositories', []);
  }.property('controller.selectedStack.repositories.[]'),

  /**
   * Verify if some repo has invalid base-url
   * Ignore if <code>useRedhatSatellite</code> is true for selected stack
   *
   * @type {bool}
   */
  invalidFormatUrlExist: function () {
    var allRepositories = this.get('allRepositories');
    if (!allRepositories) {
      return false;
    }
    if (this.get('controller.selectedStack.useRedhatSatellite')) {
      allRepositories = allRepositories.filter(this.isRedhat);
    }
    return allRepositories.someProperty('invalidFormatError', true);
  }.property('controller.selectedStack.useRedhatSatellite', 'allRepositories.@each.invalidFormatError'),

  /**
   * Verify if some invalid repo-urls exist
   * @type {bool}
   */
  invalidUrlExist: Em.computed.someBy('allRepositories', 'validation', 'INVALID'),

  /**
   * If all repo links are unchecked
   * @type {bool}
   */
  isNoOsChecked: Em.computed.everyBy('controller.selectedStack.operatingSystems', 'isSelected', false),

  /**
   *
   * @param {App.Repository} item
   * @returns {boolean}
   */
  isRedhat: function isRedhat(item) {
    return Boolean(item.get('osType') && item.get('osType').contains('redhat'));
  },

  /**
   * If all OSes are empty
   * @type {bool}
   */
  isNoOsFilled: function () {
    var operatingSystems = this.get('controller.selectedStack.operatingSystems');
    if (this.get('controller.selectedStack.useRedhatSatellite') || Em.isNone(operatingSystems)) {
      return false;
    }
    var selectedOS = operatingSystems.filterProperty('isSelected', true);
    return selectedOS.everyProperty('isNotFilled', true);
  }.property('controller.selectedStack.operatingSystems.@each.isSelected', 'controller.selectedStack.operatingSystems.@each.isNotFilled', 'controller.selectedStack.useRedhatSatellite'),

  popoverView: Em.View.extend({
    tagName: 'i',
    classNameBindings: ['repository.validationClassName'],
    attributeBindings: ['repository.errorTitle:data-original-title', 'repository.errorContent:data-content'],
    didInsertElement: function didInsertElement() {
      App.popover($(this.get('element')), {
        template: '<div class="popover"><div class="arrow"></div><div class="popover-inner"><h3 class="popover-title"></h3><div class="popover-content"></div></div></div>',
        trigger: 'hover'
      });
    }
  }),

  /**
   * @type {App.CheckboxView}
   */
  redhatCheckBoxView: App.CheckboxView.extend({
    checkedBinding: 'controller.selectedStack.useRedhatSatellite',
    disabledBinding: 'controller.selectedStack.usePublicRepo',
    click: function click() {
      var _this2 = this;

      if (!this.get('disabled')) {
        if (this.get('controller.selectedStack.useRedhatSatellite')) {
          this.toggleProperty('controller.selectedStack.useRedhatSatellite');
        } else {
          App.showConfirmationPopup(function () {
            return _this2.toggleProperty('controller.selectedStack.useRedhatSatellite');
          }, Em.I18n.t('installer.step1.advancedRepo.useRedhatSatellite.warning'), Em.K, Em.I18n.t('installer.step1.advancedRepo.useRedhatSatellite.message'));
        }
      }
      return false;
    }
  }),

  repositoryTextField: Ember.TextField.extend({
    repository: null,
    placeholderBinding: "repository.placeholder",
    valueBinding: "repository.baseUrl"
  }),

  /**
   * Handler when editing any repo BaseUrl
   *
   * @method editLocalRepository
   */
  editLocalRepository: function () {
    //upload to content
    var repositories = this.get('allRepositories');
    if (!repositories) {
      return;
    }
    var changedRepo = repositories.find(function (repository) {
      return repository.get('lastBaseUrl') && repository.get('lastBaseUrl') !== repository.get('baseUrl');
    });
    repositories.forEach(function (repository) {
      if (repository.get('lastBaseUrl') !== repository.get('baseUrl')) {
        repository.setProperties({
          lastBaseUrl: repository.get('baseUrl'),
          validation: 'PENDING'
        });
      }
    }, this);
    if (changedRepo && !App.supports.disableCredentialsAutocompleteForRepoUrls) {
      try {
        var urlObject = new URL(changedRepo.get('baseUrl'));
      } catch (e) {
        return;
      }
      var username = urlObject.username;
      var password = urlObject.password;
      repositories.forEach(function (repository) {
        var currentUrl = repository.get('baseUrl');
        try {
          var currentUrlObject = new URL(currentUrl);
        } catch (e) {
          return;
        }
        currentUrlObject.username = username;
        currentUrlObject.password = password;
        var newUrl = currentUrlObject.toString();
        if (newUrl !== currentUrl) {
          setTimeout(function () {
            repository.set('baseUrl', newUrl);
          }, 0);
        }
      }, this);
    }
  }.observes('allRepositories.@each.baseUrl')

});

});

require.register("views/wizard/step2_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * 'License'); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.SshKeyFileUploader = Em.View.extend({
  //TODO: rewrite it using tagName and attribute binding
  //TODO: rewrite it as independent component and place it somewhere in utils
  // alternative is to move it to App.WizardStep2View
  template: Em.Handlebars.compile('<input class="inputfileUgly" type="file" name="file" id="file" {{bindAttr disabled="view.disabled"}} {{QAAttr "upload-ssh-input"}} />' + '<label class="btn btn-default" for="file" {{bindAttr disabled="view.disabled"}}>Choose file</label>' + '<span id="selectedFileName">No file selected</span>'),

  classNames: ['ssh-key-input-indentation'],

  change: function change(e) {
    var self = this;
    if (e.target.files && e.target.files.length == 1) {
      var file = e.target.files[0];
      var reader = new FileReader();

      reader.onload = function () {
        return function (e) {
          var fileNameArray = $("#file").val().toString().split("\\");
          var selectedFileName = fileNameArray[fileNameArray.length - 1];
          $('#selectedFileName').text(selectedFileName);
          $('#sshKey').text(e.target.result);
          self.get("controller").setSshKey(e.target.result);
        };
      }(file);
      reader.readAsText(file);
    }
  }

});

App.WizardStep2View = Em.View.extend({

  templateName: require('templates/wizard/step2'),

  didInsertElement: function didInsertElement() {
    App.popover($("[rel=popover]"), { 'placement': 'right', 'trigger': 'hover' });
    App.tooltip($("[rel=tooltip]"), { 'placement': 'top', 'trigger': 'hover' });
    //todo: move them to conroller
    this.set('controller.hostsError', null);
    this.set('controller.sshKeyError', null);
  },

  /**
   * Is manualInstall selected
   * @type {bool}
   */
  sshKeyState: Em.computed.alias('controller.content.installOptions.manualInstall'),

  /**
   * Is File API available
   * @type {bool}
   * TODO: incupsulate it inside of App.SshKeyFileUploader
   */
  isFileApi: function () {
    /* istanbul ignore next */
    return window.File && window.FileReader && window.FileList;
  }.property(),

  /**
   * Radio button for activate SSH fields
   * @type {App.RadioButtonView}
   * TODO: replace next 2 properties with new one used in both places
   */
  providingSSHKeyRadioButton: App.RadioButtonView.extend({
    classNames: ['radio'],
    checked: Em.computed.alias('controller.content.installOptions.useSsh'),

    click: function click() {
      this.set('controller.content.installOptions.useSsh', true);
      this.set('controller.content.installOptions.manualInstall', false);
    }
  }),

  /**
   * Radio button for manual registration
   * @type {App.RadioButtonView}
   */
  manualRegistrationRadioButton: App.RadioButtonView.extend({
    classNames: ['radio'],
    checked: Em.computed.alias('controller.content.installOptions.manualInstall'),

    click: function click() {
      this.set('controller.content.installOptions.manualInstall', true);
      this.set('controller.content.installOptions.useSsh', false);
    }
  }),

  /**
   * Checkbox to skip Host Checks
   * @type {App.CheckboxView}
   */
  skipHostsCheckBox: App.CheckboxView.extend({
    classNames: ['display-inline-block'],
    classNameBindings: ['containerClassName'],
    containerClassName: 'checkbox',

    showConfirmPopup: function () {
      if (this.get('controller.content.installOptions.skipHostChecks')) {
        App.ModalPopup.show({
          header: Em.I18n.t('installer.step2.skipHostChecks.popup.header'),
          body: Em.I18n.t('installer.step2.skipHostChecks.popup.body'),
          primary: Em.I18n.t('ok'),
          secondary: false
        });
      }
    }.observes('controller.content.installOptions.skipHostChecks')
  }),

  /**
   * Textarea with ssh-key
   * @type {Ember.TextField}
   */
  textFieldView: Em.TextField.extend({

    /**
     * Is textfield disabled
     * @type {bool}
     */
    disabled: Em.computed.not('isEnabled')
  })

});

});

require.register("views/wizard/step3/hostLogPopupBody_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep3HostLogPopupBody = Em.View.extend({

  templateName: require('templates/wizard/step3/step3_host_log_popup'),

  /**
   * Host's boot log
   * @type {string}
   */
  bootLog: Em.computed.alias('parentView.host.bootLog'),

  /**
   * Is textarea view active
   * @type {bool}
   */
  isTextArea: false,

  /**
   * Textbox with host's boot log
   * @type {Ember.TextArea}
   */
  textArea: Em.TextArea.extend({

    didInsertElement: function didInsertElement() {
      /* istanbul ignore next: simple DOM manipulations */
      $(this.get('element')).width($(this.get('parentView').get('element')).width() - 10).height($(this.get('parentView').get('element')).height()).select().css('resize', 'none');
    },

    /**
     * Edit disabled
     * @type {bool}
     */
    readOnly: true,

    /**
     * <code>parentView.bootLog</code>
     * @type {string}
     */
    value: Em.computed.alias('content')

  }),

  didInsertElement: function didInsertElement() {
    var self = this;
    var button = $(this.get('element')).find('.textTrigger');
    button.click(function () {
      $(this).text(self.get('isTextArea') ? Em.I18n.t('installer.step3.hostLogPopup.highlight') : Em.I18n.t('installer.step3.hostLogPopup.copy'));
      self.set('isTextArea', !self.get('isTextArea'));
    });
    /* istanbul ignore next: difficult to test */
    $(this.get('element')).find('.content-area').mouseenter(function () {
      button.css('visibility', 'visible');
    }).mouseleave(function () {
      button.css('visibility', 'hidden');
    });
  }

});

});

require.register("views/wizard/step3/hostWarningPopupBody_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var lazyloading = require('utils/lazy_loading');
var numberUtils = require('utils/number_utils');

App.WizardStep3HostWarningPopupBody = Em.View.extend({

  templateName: require('templates/wizard/step3/step3_host_warnings_popup'),

  classNames: ['host-check'],

  bodyControllerBinding: 'App.router.wizardStep3Controller',

  /**
   * Listbox with host filters
   * @type {Ember.Select}
   */
  hostSelectView: Em.Select.extend({
    elementId: 'hosts-check-select',

    selectionBinding: "parentView.category",

    /**
     * Content has default value "All Hosts" to bind selection to category
     * @type {string[]}
     */
    content: ['All Hosts'],

    /**
     * List of filtered hostnames
     * @type {string[]}
     */
    hosts: function () {
      var warningsByHost = this.get('parentView.warningsByHost');
      if (Em.isNone(warningsByHost)) return [];
      return warningsByHost.mapProperty('name');
    }.property('parentView.warningsByHost'),

    /**
     * Is data loaded
     * @type {bool}
     */
    isLoaded: false,

    didInsertElement: function didInsertElement() {
      this.initContent();
    },

    /**
     * Check browser and set content for listbox
     * @method initContent
     */
    initContent: function () {
      this.set('isLoaded', false);
      //The lazy loading for select elements supported only by Firefox and Chrome
      var isBrowserSupported = $.browser.mozilla || $.browser.safari && navigator.userAgent.indexOf('Chrome') !== -1;
      var isLazyLoading = isBrowserSupported && this.get('hosts').length > 100;
      this.set('isLazyLoading', isLazyLoading);
      if (isLazyLoading) {
        //select need at least 30 hosts to have scrollbar
        this.set('content', this.get('hosts').slice(0, 30));
      } else {
        this.set('content', this.get('hosts'));
        this.set('isLoaded', true);
      }
    }.observes('parentView.warningsByHost'),

    /**
     * On click start lazy loading
     * @method click
     */
    click: function click() {
      if (!this.get('isLoaded') && this.get('isLazyLoading')) {
        //filter out hosts, which already pushed in select
        var source = this.get('hosts').filter(function (_host) {
          return !this.get('content').contains(_host);
        }, this).slice();
        lazyloading.run({
          destination: this.get('content'),
          source: source,
          context: this,
          initSize: 30,
          chunkSize: 200,
          delay: 50
        });
      }
    }
  }),

  /**
   * List of warnings grouped by host
   * Same to <code>bodyController.warningsByHost</code>
   * @type {Ember.Enumerable}
   */
  warningsByHost: Em.computed.alias('bodyController.warningsByHost'),

  /**
   * List of all warnings
   * Same to <code>bodyController.warnings</code>
   * @type {Ember.Enumerable}
   */
  warnings: Em.computed.alias('bodyController.warnings'),

  /**
   * Selected category
   * @type {string}
   */
  category: 'All Hosts',

  /**
   * List of warnings for selected <code>category</code>
   * @type {Ember.Enumerable}
   */
  categoryWarnings: function () {
    var warningsByHost = this.get('warningsByHost');
    if (Em.isNone(warningsByHost)) return [];
    var category = warningsByHost.findProperty('name', this.get('category'));
    return Em.isNone(category) ? [] : category.warnings || [];
  }.property('warningsByHost', 'category'),

  /**
   * List of warnings grouped by <code>category</code>
   * @type {Ember.Object[]}
   */
  content: function () {
    var hostCheckWarnings = this.get('bodyController.hostCheckWarnings');
    var repoCategoryWarnings = this.get('bodyController.repoCategoryWarnings');
    var diskCategoryWarnings = this.get('bodyController.diskCategoryWarnings');
    var jdkCategoryWarnings = this.get('bodyController.jdkCategoryWarnings') || [];
    var thpCategoryWarnings = this.get('bodyController.thpCategoryWarnings');
    var categoryWarnings = this.get('categoryWarnings');
    var warningsArray = [Em.Object.create({
      warnings: thpCategoryWarnings,
      title: Em.I18n.t('installer.step3.hostWarningsPopup.thp'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.thp.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.thp')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.enabled'),
      category: 'thp'
    }), Em.Object.create({
      warnings: jdkCategoryWarnings,
      title: Em.I18n.t('installer.step3.hostWarningsPopup.jdk'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.jdk.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.jdk')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'jdk'
    }), Em.Object.create({
      warnings: diskCategoryWarnings,
      title: Em.I18n.t('installer.step3.hostWarningsPopup.disk'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.disk.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.disk')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'disk'
    }), Em.Object.create({
      warnings: repoCategoryWarnings,
      title: Em.I18n.t('installer.step3.hostWarningsPopup.repositories'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.repositories.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.repositories')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.invalid'),
      category: 'repositories'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'firewall'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.firewall'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.firewall.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.firewall')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.running'),
      category: 'firewall'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'processes'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.process'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.processes.message'),
      type: Em.I18n.t('common.process'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.processes')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.running'),
      category: 'process'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'packages'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.package'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.packages.message'),
      type: Em.I18n.t('common.package'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.packages')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.installed'),
      category: 'package'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'fileFolders'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.fileAndFolder'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.fileFolders.message'),
      type: Em.I18n.t('common.path'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.filesAndFolders')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'fileFolders'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'services'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.service'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.services.message'),
      type: Em.I18n.t('common.service'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.services')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.notRunning'),
      category: 'service'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'users'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.user'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.users.message'),
      type: Em.I18n.t('common.user'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.users')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'user'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'misc'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.misc'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.misc.message'),
      type: Em.I18n.t('installer.step3.hostWarningsPopup.misc.umask'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.empty.misc')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'misc'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'alternatives'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.alternatives'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.alternatives.message'),
      type: Em.I18n.t('installer.step3.hostWarningsPopup.alternatives.umask'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.alternatives.empty')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.exists'),
      category: 'alternatives'
    }), Em.Object.create({
      warnings: categoryWarnings.filterProperty('category', 'reverseLookup'),
      title: Em.I18n.t('installer.step3.hostWarningsPopup.reverseLookup'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.reverseLookup.message'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.reverseLookup.empty')),
      category: 'reverseLookup'
    }), Em.Object.create({
      warnings: hostCheckWarnings,
      title: Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.name'),
      message: Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.message'),
      type: Em.I18n.t('common.issues'),
      emptyName: Em.I18n.t('installer.step3.hostWarningsPopup.emptyMessage').format(Em.I18n.t('installer.step3.hostWarningsPopup.resolution.validation.empty')),
      action: Em.I18n.t('installer.step3.hostWarningsPopup.action.failed'),
      category: 'hostNameResolution'
    })];
    warningsArray.forEach(function (warningType) {
      warningType.set('isCollapsed', true);
      warningType.get('warnings').forEach(function (warn) {
        var hosts = Em.get(warn, 'hosts');
        if (hosts) {
          var hostsList = hosts.length < 11 ? hosts.join('<br>') : hosts.slice(0, 10).join('<br>') + '<br> ' + Em.I18n.t('installer.step3.hostWarningsPopup.moreHosts').format(warn.hosts.length - 10);
          Em.set(warn, 'hostsList', hostsList);
        }
      });
    });
    warningsArray.sort(function (a, b) {
      return b.warnings.length - a.warnings.length;
    });
    return warningsArray;
  }.property('category', 'warningsByHost', 'bodyController.jdkCategoryWarnings', 'bodyController.repoCategoryWarnings', 'bodyController.diskCategoryWarnings', 'bodyController.hostCheckWarnings', 'bodyController.thpCategoryWarnings'),

  /**
   * Message with info about warnings
   * @return {string}
   */
  warningsNotice: function () {
    var issuesNumber = this.get('warnings.length') + this.get('bodyController.repoCategoryWarnings.length') + this.get('bodyController.diskCategoryWarnings.length') + this.get('bodyController.jdkCategoryWarnings.length') + this.get('bodyController.hostCheckWarnings.length') + this.get('bodyController.thpCategoryWarnings.length');
    this.set('totalWarningsCount', issuesNumber);
    var issues = issuesNumber + ' ' + (issuesNumber.length === 1 ? Em.I18n.t('installer.step3.hostWarningsPopup.issue') : Em.I18n.t('installer.step3.hostWarningsPopup.issues'));
    var hostsCnt = this.warningHostsNamesCount();
    var hosts = hostsCnt + ' ' + (hostsCnt === 1 ? Em.I18n.t('installer.step3.hostWarningsPopup.host') : Em.I18n.t('installer.step3.hostWarningsPopup.hosts'));
    return Em.I18n.t('installer.step3.hostWarningsPopup.summary').format(issues, hosts);
  }.property('warnings', 'warningsByHost', 'bodyController.jdkCategoryWarnings', 'bodyController.repoCategoryWarnings', 'bodyController.diskCategoryWarnings', 'bodyController.hostCheckWarnings', 'bodyController.diskCategoryWarnings'),

  /**
   * Detailed content to show it in new window
   * @return {string}
   */
  contentInDetails: function () {
    var content = this.get('content');
    var newContent = '';
    newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.header') + new Date();
    newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.hosts');
    newContent += this.get('hostNamesWithWarnings').join(' ');
    if (content.findProperty('category', 'thp').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.thp');
      newContent += content.findProperty('category', 'thp').warnings[0].hostsLong.join(' ');
    }
    if (content.findProperty('category', 'jdk').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.jdk');
      newContent += content.findProperty('category', 'jdk').warnings[0].hostsLong.join('<br>');
    }
    if (content.findProperty('category', 'disk').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.disk');
      newContent += content.findProperty('category', 'disk').warnings[0].hostsLong.join('<br>');
    }
    if (content.findProperty('category', 'repositories').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.repositories');
      newContent += content.findProperty('category', 'repositories').warnings[0].hostsLong.join('<br>');
    }
    if (content.findProperty('category', 'hostNameResolution').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.hostNameResolution');
      newContent += content.findProperty('category', 'hostNameResolution').warnings[0].hostsLong.join('<br>');
    }
    if (content.findProperty('category', 'firewall').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.firewall');
      newContent += content.findProperty('category', 'firewall').warnings.mapProperty('name').join('<br>');
    }
    if (content.findProperty('category', 'fileFolders').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.fileFolders');
      newContent += content.findProperty('category', 'fileFolders').warnings.mapProperty('name').join(' ');
    }
    if (content.findProperty('category', 'reverseLookup').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.reverseLookup');
      newContent += content.findProperty('category', 'reverseLookup').warnings[0].hostsLong.join(' ');
    }
    if (content.findProperty('category', 'process').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.process');
      content.findProperty('category', 'process').warnings.forEach(function (process, i) {
        process.hosts.forEach(function (host, j) {
          if (!!i || !!j) {
            newContent += ',';
          }
          newContent += '(' + host + ',' + process.user + ',' + process.pid + ')';
        });
      });
    }
    if (content.findProperty('category', 'package').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.package');
      newContent += content.findProperty('category', 'package').warnings.mapProperty('name').join(' ');
    }
    if (content.findProperty('category', 'service').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.service');
      newContent += content.findProperty('category', 'service').warnings.mapProperty('name').join(' ');
    }
    if (content.findProperty('category', 'user').warnings.length) {
      newContent += Em.I18n.t('installer.step3.hostWarningsPopup.report.user');
      newContent += content.findProperty('category', 'user').warnings.mapProperty('name').join(' ');
    }
    newContent += '</p>';
    return newContent;
  }.property('content', 'warningsByHost', 'bodyController.jdkCategoryWarnings', 'bodyController.repoCategoryWarnings', 'bodyController.diskCategoryWarnings', 'bodyController.hostCheckWarnings'),

  didInsertElement: function () {
    Em.run.next(this, function () {
      App.tooltip(this.$("[rel='HostsListTooltip']"), { html: true, placement: "right" });
      App.tooltip(this.$('#process .warning-name'), { html: true, placement: "top" });
    });
  }.observes('content'),

  /**
   * Show popup with selected hostnames
   * @param {object} hosts
   * @method showHostsPopup
   * @return {App.ModalPopup}
   */
  showHostsPopup: function showHostsPopup(hosts) {
    $('.tooltip').hide();
    return App.ModalPopup.show({
      'data-qa': 'hosts-with-warnings-modal',
      header: Em.I18n.t('installer.step3.hostWarningsPopup.allHosts'),
      bodyClass: Em.View.extend({
        hosts: hosts.context,
        template: Em.Handlebars.compile('<ul>{{#each host in view.hosts}}<li {{QAAttr "host-with-warning-name"}}>{{host}}</li>{{/each}}</ul>')
      }),
      secondary: null
    });
  },

  /**
   * Open/Close selected category
   * @param {object} category
   * @method onToggleBlock
   */
  onToggleBlock: function onToggleBlock(category) {
    this.$('#' + category.context.category).toggle('blind', 500);
    category.context.set("isCollapsed", !category.context.get("isCollapsed"));
  },

  /**
   * Generate number of hosts which had warnings, avoid duplicated host names in different warnings.
   * @method warningHostsNamesCount
   * @return {number}
   */
  warningHostsNamesCount: function warningHostsNamesCount() {
    var hostNameMap = Em.Object.create();
    if (Em.isNone(this.get('bodyController.warningsByHost'))) return 0;
    var warningsByHost = this.get('bodyController.warningsByHost').slice();
    warningsByHost.shift();
    warningsByHost.forEach(function (_host) {
      if (_host.warnings.length) {
        hostNameMap[_host.name] = true;
      }
    });
    var repoCategoryWarnings = this.get('bodyController.repoCategoryWarnings');
    var diskCategoryWarnings = this.get('bodyController.diskCategoryWarnings');
    var jdkCategoryWarnings = this.get('bodyController.jdkCategoryWarnings');
    var thpCategoryWarnings = this.get('bodyController.thpCategoryWarnings');
    var hostResolutionWarnings = this.get('bodyController.hostCheckWarnings');

    if (repoCategoryWarnings.length) {
      repoCategoryWarnings[0].hostsNames.forEach(function (_hostName) {
        if (!hostNameMap[_hostName]) {
          hostNameMap[_hostName] = true;
        }
      });
    }
    if (diskCategoryWarnings.length) {
      diskCategoryWarnings[0].hostsNames.forEach(function (_hostName) {
        if (!hostNameMap[_hostName]) {
          hostNameMap[_hostName] = true;
        }
      });
    }
    if (jdkCategoryWarnings && jdkCategoryWarnings.length) {
      jdkCategoryWarnings[0].hostsNames.forEach(function (_hostName) {
        if (!hostNameMap[_hostName]) {
          hostNameMap[_hostName] = true;
        }
      });
    }
    if (hostResolutionWarnings && hostResolutionWarnings.length) {
      hostResolutionWarnings[0].hostsNames.forEach(function (_hostName) {
        if (!hostNameMap[_hostName]) {
          hostNameMap[_hostName] = true;
        }
      });
    }
    if (thpCategoryWarnings.length) {
      thpCategoryWarnings[0].hostsNames.forEach(function (_hostName) {
        if (!hostNameMap[_hostName]) {
          hostNameMap[_hostName] = true;
        }
      });
    }
    this.set('hostNamesWithWarnings', Em.keys(hostNameMap));
    return Em.keys(hostNameMap).length;
  },

  /**
   * Open new browser tab with detailed content
   * @method openWarningsInDialog
   */
  openWarningsInDialog: function openWarningsInDialog() {
    var newWindow = window.open('');
    var newDocument = newWindow.document;
    newDocument.write(this.get('contentInDetails'));
    newWindow.focus();
  }

});

});

require.register("views/wizard/step3/hostWarningPopupFooter_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep3HostWarningPopupFooter = Em.View.extend({

  templateName: require('templates/wizard/step3/step3_host_warning_popup_footer'),

  classNames: ['modal-footer', 'host-checks-update'],

  footerControllerBinding: 'App.router.wizardStep3Controller',

  progressWidth: Em.computed.format('width:{0}%', 'footerController.checksUpdateProgress'),

  isUpdateInProgress: function () {
    return !this.get('checkHostFinished') || this.get('footerController.checksUpdateProgress') > 0 && this.get('footerController.checksUpdateProgress') < 100;
  }.property('checkHostFinished', 'footerController.checksUpdateProgress'),

  updateStatusClass: function () {
    var status = this.get('footerController.checksUpdateStatus');
    if (status === 'SUCCESS') {
      return 'text-success';
    } else {
      if (status === 'FAILED') {
        return 'text-danger';
      } else {
        return null;
      }
    }
  }.property('footerController.checksUpdateStatus'),

  updateStatus: function () {
    var status = this.get('footerController.checksUpdateStatus');
    if (status === 'SUCCESS') {
      return Em.I18n.t('installer.step3.warnings.updateChecks.success');
    } else if (status === 'FAILED') {
      return Em.I18n.t('installer.step3.warnings.updateChecks.failed');
    } else {
      return null;
    }
  }.property('footerController.checksUpdateStatus')

});

});

require.register("views/wizard/step3_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep3View = App.TableView.extend({

  templateName: require('templates/wizard/step3'),

  /**
   * List of hosts
   * Same to <code>controller.hosts</code>
   * @type {Ember.Enumerable}
   */
  content: function () {
    return this.get('controller.hosts');
  }.property('controller.hosts.length'),

  /**
   * Message with info about registration result
   * @type {string}
   */
  message: '',

  /**
   * Text to link for hosts' warnings popup
   * @type {string}
   */
  linkText: '',

  /**
   * Registration status
   * @type {string}
   */
  status: '',

  /**
   * Active category
   * @type {string}
   */
  selectedCategory: Em.computed.findBy('categories', 'isActive', true),

  /**
   * Message about other registered hosts (not included in current registration)
   * @type {string}
   */
  registeredHostsMessage: '',

  /**
   * Number of visible hosts on page
   * @type {string}
   */
  displayLength: "25",

  /**
   * All checkboxes on page are checked
   * @type {bool}
   */
  pageChecked: false,

  /**
   * bootStatus category object
   * @type {Ember.Object}
   */
  categoryObject: Em.Object.extend({
    hostsCount: 0,
    label: Em.computed.format('{0} ({1})', 'value', 'hostsCount'),
    isActive: false,
    itemClass: Em.computed.ifThenElse('isActive', 'active', '')
  }),

  /**
   * List of bootStatus categories
   * @type {categoryObject[]}
   */
  categories: function () {
    return [this.categoryObject.create({ value: Em.I18n.t('common.all'), hostsBootStatus: 'ALL', isActive: true }), this.categoryObject.create({ value: Em.I18n.t('installer.step3.hosts.status.installing'), hostsBootStatus: 'RUNNING' }), this.categoryObject.create({ value: Em.I18n.t('installer.step3.hosts.status.registering'), hostsBootStatus: 'REGISTERING' }), this.categoryObject.create({ value: Em.I18n.t('common.success'), hostsBootStatus: 'REGISTERED' }), this.categoryObject.create({ value: Em.I18n.t('common.fail'), hostsBootStatus: 'FAILED', last: true })];
  }.property(),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
    this.$().on('mouseover', '#confirm-hosts-table tr', function () {
      App.tooltip($(this).find('.action .glyphicon'), { placement: 'bottom' });
    });
  },

  /**
   * Select checkboxes of hosts on page
   * @method onPageChecked
   */
  onPageChecked: function () {
    if (this.get('selectionInProgress')) return;
    this.get('pageContent').setEach('isChecked', this.get('pageChecked'));
  }.observes('pageChecked'),

  /**
   * Select checkboxes of all hosts
   * @method selectAll
   */
  selectAll: function selectAll() {
    this.get('content').setEach('isChecked', true);
  },

  /**
   * Reset checkbox of all hosts
   * @method unSelectAll
   */
  unSelectAll: function unSelectAll() {
    this.get('content').setEach('isChecked', false);
  },

  /**
   * Call <code>watchSelection</code> only once
   * @method watchSelectionOnce
   */
  watchSelectionOnce: function () {
    Em.run.once(this, 'watchSelection');
  }.observes('content.@each.isChecked', 'content.@each.bootStatus', 'pageContent'),

  /**
   * Watch selection and calculate such flags as:
   * <ul>
   *  <li>noHostsSelected</li>
   *  <li>selectedHostsCount</li>
   *  <li>pageChecked</li>
   * </ul>
   * @method watchSelection
   */
  watchSelection: function watchSelection() {
    this.set('selectionInProgress', true);
    this.set('pageChecked', !!this.get('pageContent.length') && this.get('pageContent').everyProperty('isChecked', true));
    this.set('selectionInProgress', false);
    var noNotRunningHostsSelected = true;
    var selectedNotRunningHostsCount = 0;
    var selectedHostsCount = 0;
    this.get('content').forEach(function (host) {
      var isSelectedAndNotRunning = host.get('isChecked') && host.get('bootStatus') !== 'RUNNING';
      selectedHostsCount += host.get('isChecked') ? 1 : 0;
      selectedNotRunningHostsCount += isSelectedAndNotRunning ? 1 : 0;
      noNotRunningHostsSelected = noNotRunningHostsSelected ? !isSelectedAndNotRunning : noNotRunningHostsSelected;
    });
    this.set('noNotRunningHostsSelected', noNotRunningHostsSelected);
    this.set('selectedNotRunningHostsCount', selectedNotRunningHostsCount);
    this.set('selectedHostsCount', selectedHostsCount);
  },

  /**
   * Update <code>registeredHostsMessage</code> according to <code>controller.registeredHots.length</code>
   * @method setRegisteredHosts
   */
  setRegisteredHosts: function () {
    this.set('registeredHostsMessage', Em.I18n.t('installer.step3.warning.registeredHosts').format(this.get('controller.registeredHosts').length));
  }.observes('controller.registeredHosts'),

  /**
   * Call filters and counters one time
   * @method hostBootStatusObserver
   */
  hostBootStatusObserver: function () {
    Em.run.once(this, 'countCategoryHosts');
    Em.run.once(this, 'filter');
    Em.run.once(this, 'monitorStatuses');
  }.observes('content.@each.bootStatus'),

  /**
   * Calculate host count grouped by <code>bootStatus</code>
   * @method countCategoryHosts
   */
  countCategoryHosts: function countCategoryHosts() {
    var counters = {
      "RUNNING": 0,
      "REGISTERING": 0,
      "REGISTERED": 0,
      "FAILED": 0
    };
    this.get('content').forEach(function (host) {
      if (!Em.isNone(counters[host.get('bootStatus')])) {
        counters[host.get('bootStatus')]++;
      }
    }, this);
    counters["ALL"] = this.get('content.length');
    this.get('categories').forEach(function (category) {
      category.set('hostsCount', counters[category.get('hostsBootStatus')]);
    }, this);
  },

  /**
   * Filter hosts by category
   * @method filter
   */
  filter: function () {
    var self = this;
    Em.run.next(function () {
      self.doFilter();
    });
  }.observes('selectedCategory'),

  /**
   * Real filter-method
   * Called from <code>filter</code> in Em.run.next-wrapper
   * @method doFilter
   */
  doFilter: function doFilter() {
    var result = [];
    var selectedCategory = this.get('selectedCategory');
    if (!selectedCategory || selectedCategory.get('hostsBootStatus') === 'ALL') {
      result = this.get('content');
    } else {
      result = this.get('content').filterProperty('bootStatus', this.get('selectedCategory.hostsBootStatus'));
    }
    this.set('filteredContent', result);
  },

  /**
   * Trigger on Category click
   * @param {Object} event
   * @method selectCategory
   */
  selectCategory: function selectCategory(event) {
    var categoryStatus = event.context.get('hostsBootStatus');
    this.get('categories').forEach(function (category) {
      category.set('isActive', category.get('hostsBootStatus') === categoryStatus);
    });
    this.watchSelection();
  },

  /**
   * Select "All" hosts category
   * run registration of failed hosts again
   * @method retrySelectedHosts
   */
  retrySelectedHosts: function retrySelectedHosts() {
    var eventObject = { context: Em.Object.create({ hostsBootStatus: 'ALL' }) };
    this.selectCategory(eventObject);
    this.get('controller').retrySelectedHosts();
  },

  /**
   * Update <code>status</code>, <code>linkText</code>, <code>message</code> according to hosts statuses
   * @method monitorStatuses
   */
  monitorStatuses: function () {
    var hosts = this.get('controller.bootHosts');
    var failedHosts = hosts.filterProperty('bootStatus', 'FAILED').length;

    if (hosts.length === 0) {
      this.set('status', 'alert-warning');
      this.set('linkText', '');
      this.set('message', Em.I18n.t('installer.step3.warnings.missingHosts'));
    } else {
      if (!this.get('controller.isWarningsLoaded')) {
        this.set('status', 'alert-info');
        this.set('linkText', '');
        this.set('message', Em.I18n.t('installer.step3.warning.loading'));
      } else {
        if (this.get('controller.isHostHaveWarnings') || this.get('controller.repoCategoryWarnings.length') || this.get('controller.diskCategoryWarnings.length') || this.get('controller.jdkCategoryWarnings.length') || this.get('controller.hostCheckWarnings.length') || this.get('controller.thpCategoryWarnings.length')) {
          this.set('status', 'alert-warning');
          this.set('linkText', Em.I18n.t('installer.step3.warnings.linkText'));
          this.set('message', Em.I18n.t('installer.step3.warnings.fails').format(hosts.length - failedHosts));
        } else {
          this.set('status', 'alert-success');
          this.set('linkText', Em.I18n.t('installer.step3.noWarnings.linkText'));
          if (failedHosts == 0) {
            // all are ok
            this.set('message', Em.I18n.t('installer.step3.warnings.noWarnings').format(hosts.length));
          } else {
            if (failedHosts == hosts.length) {
              // all failed
              this.set('status', 'alert-warning');
              this.set('linkText', '');
              this.set('message', Em.I18n.t('installer.step3.warnings.allFailed').format(failedHosts));
            } else {
              // some failed
              this.set('message', Em.I18n.t('installer.step3.warnings.someWarnings').format(hosts.length - failedHosts, failedHosts));
            }
          }
        }
      }
    }
  }.observes('controller.isWarningsLoaded', 'controller.hostCheckWarnings', 'controller.isHostHaveWarnings', 'controller.repoCategoryWarnings', 'controller.diskCategoryWarnings', 'controller.jdkCategoryWarnings', 'controller.thpCategoryWarnings')
});

//todo: move it inside WizardStep3View
App.WizardHostView = Em.View.extend({

  tagName: 'tr',

  classNameBindings: ['hostInfo.bootStatus', 'hostInfo.isChecked:active'],

  /**
   * Host from parent view
   * @type {Object}
   */
  hostInfo: null,

  /**
   * Remove selected host
   * @method remove
   */
  remove: function remove() {
    this.get('controller').removeHost(this.get('hostInfo'));
  },

  /**
   * Retry register selected host
   * @method retry
   */
  retry: function retry() {
    this.get('controller').retryHost(this.get('hostInfo'));
  }

});

});

require.register("views/wizard/step4_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep4View = Em.View.extend({

  templateName: require('templates/wizard/step4'),

  toggleCheckBox: function toggleCheckBox(event) {
    if (event.context.get('isDisabled')) {
      return;
    }
    var isSelected = event.context.get('isSelected');
    event.context.set('isSelected', !isSelected);
  }
});

});

require.register("views/wizard/step5_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

App.WizardStep5View = App.AssignMasterComponentsView.extend({

  isReassignWizard: Em.computed.equal('controller.content.controllerName', 'reassignMasterController'),

  title: Em.computed.ifThenElse('isReassignWizard', Em.I18n.t('installer.step5.reassign.header'), Em.I18n.t('installer.step5.header')),

  alertMessage: function () {
    var result = Em.I18n.t('installer.step5.body');
    result += this.get('coHostedComponentText') ? '\n' + this.get('coHostedComponentText') : '';
    return result;
  }.property('coHostedComponentText'),

  coHostedComponentText: '',

  didInsertElement: function didInsertElement() {
    this._super();
    this.setCoHostedComponentText();
  },

  setCoHostedComponentText: function setCoHostedComponentText() {
    var coHostedComponents = App.StackServiceComponent.find().filterProperty('isOtherComponentCoHosted').filterProperty('stackService.isSelected');
    var coHostedComponentsText = '';
    if (!this.get('controller').get('isReassignWizard')) {
      coHostedComponents.forEach(function (serviceComponent, index) {
        var coHostedComponentsDisplayNames = serviceComponent.get('coHostedComponents').map(function (item) {
          return App.StackServiceComponent.find().findProperty('componentName', item).get('displayName');
        });
        var componentTextArr = [serviceComponent.get('displayName')].concat(coHostedComponentsDisplayNames);
        coHostedComponents[index] = stringUtils.getFormattedStringFromArray(componentTextArr);
        coHostedComponentsText += '<br/>' + Em.I18n.t('installer.step5.body.coHostedComponents').format(coHostedComponents[index]);
      }, this);
    }

    this.set('coHostedComponentText', coHostedComponentsText);
  }

});

});

require.register("views/wizard/step6_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep6View = App.TableView.extend({

  templateName: require('templates/wizard/step6'),

  /**
   * Number of visible rows
   * @type {string}
   */
  displayLength: "25",

  /**
   * List of hosts
   * @type {object[]}
   */
  content: function () {
    return this.get('controller.hosts');
  }.property('controller.hosts'),

  /**
   * Synonym to <code>content</code> in this <code>App.TableView</code>
   * @type {object[]}
   */
  filteredContent: Em.computed.alias('content'),

  /**
   * Set <code>label</code> and do <code>loadStep</code>
   * @method didInsertElement
   */
  didInsertElement: function didInsertElement() {
    this.setLabel();
    this.get('controller').loadStep();
    Em.run.next(this, this.adjustColumnWidth);
  },

  adjustColumnWidth: function adjustColumnWidth() {
    var table = $('#component_assign_table');
    var columnsCount = this.get('controller.headers.length');
    if (table.width() > table.find('tbody').width()) {
      var columnWidth = Math.floor(table.width() / columnsCount);
      table.find("th:not('.freeze'), td:not('.freeze')").width(columnWidth);
    }
  },

  /**
   * Set <code>label</code> value
   * @method setLabel
   */
  setLabel: function setLabel() {
    var clients = this.get('controller.content.clients');
    var label = !!clients.length ? Em.I18n.t('installer.step6.body') + Em.I18n.t('installer.step6.body.clientText') : Em.I18n.t('installer.step6.body');

    clients.forEach(function (_client) {
      if (clients.length === 1) {
        label = label + ' ' + _client.display_name;
      } else {
        if (_client !== clients[clients.length - 1]) {
          // [clients.length - 1]
          label = label + ' ' + _client.display_name;
          if (_client !== clients[clients.length - 2]) {
            label = label + ',';
          }
        } else {
          label = label + ' ' + Em.I18n.t('and') + ' ' + _client.display_name + '.';
        }
      }
    }, this);
    this.set('label', label);
  },

  checkboxClick: function checkboxClick(e) {
    var checkbox = e.context;
    Em.set(checkbox, 'checked', !checkbox.checked);
    this.get('controller').checkCallback(checkbox.component);
    this.get('controller').callValidation();
  },

  columnCount: function () {
    var hosts = this.get('controller.hosts');
    return hosts && hosts.length > 0 ? hosts[0].get('checkboxes').length + 1 : 1;
  }.property('controller.hosts.@each.checkboxes')
});

App.WizardStep6HostView = Em.View.extend({

  /**
   * Bound <code>host</code> object
   * @type {object}
   */
  host: null,

  'data-qa': 'hostname-block',

  classNames: ['freeze'],

  tagName: 'td',

  /**
   * Create hover-labels for hostName with list of installed master-components
   * @method didInsertElement
   */
  didInsertElement: function didInsertElement() {
    var componentNames = this.get('controller').getMasterComponentsForHost(this.get('host.hostName')).map(function (_component) {
      return App.format.role(_component, false);
    }).join('<br />');
    App.popover(this.$(), {
      title: Em.I18n.t('installer.step6.wizardStep6Host.title').format(this.get('host.hostName')),
      content: '<div data-qa="master-component-popover">' + componentNames + '</div>',
      placement: 'right',
      trigger: 'hover'
    });
  },

  willDestroyElement: function willDestroyElement() {
    this.$().popover('destroy');
  }

});

});

require.register("views/wizard/step7/accounts_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.AccountsTabOnStep7View = Em.View.extend(App.WizardMiscPropertyChecker, {

  templateName: require('templates/wizard/step7/accounts_tab'),

  /**
   * Property objects to represent table rows
   */
  properties: function () {
    return this.get('controller.stepConfigs').findProperty('serviceName', 'MISC').get('configs').filterProperty('displayType', 'user');
  }.property('controller.stepConfigsCreated'),

  propertyChanged: function () {
    var changedProperty = this.get('properties').find(function (prop) {
      return prop.get('editDone');
    });
    if (!changedProperty) {
      return;
    }
    var stepConfigs = this.get('controller.stepConfigs');
    var serviceId = changedProperty.get('serviceName');
    return this.showConfirmationDialogIfShouldChangeProps(changedProperty, stepConfigs, serviceId);
  }.observes('properties.@each.editDone'),

  checkboxes: function () {
    var miscConfigs = this.get('controller.stepConfigs').findProperty('serviceName', 'MISC').get('configs');
    return [miscConfigs.findProperty('name', 'sysprep_skip_create_users_and_groups'), miscConfigs.findProperty('name', 'ignore_groupsusers_create'), miscConfigs.findProperty('name', 'override_uid')];
  }.property('controller.stepConfigsCreated')

});

});

require.register("views/wizard/step7/assign_master_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var stringUtils = require('utils/string_utils');

require('views/wizard/step5_view');

App.AssignMasterOnStep7View = App.AssignMasterComponentsView.extend({

  showTitle: false,

  alertMessage: '',

  isWizardStep: false,

  willInsertElement: function willInsertElement() {
    this._super();
    this.setAlertMessage();
  },

  /**
   * @method setAlertMessage
   */
  setAlertMessage: function setAlertMessage() {
    var mastersToCreate = this.get('controller.mastersToCreate'),
        mastersToCreateDisplayName = mastersToCreate.map(function (item) {
      return App.format.role(item);
    }),
        stringText1 = mastersToCreate.length > 1 ? Em.I18n.t('common.hosts').toLowerCase() : Em.I18n.t('common.host').toLowerCase(),
        stringText2 = mastersToCreate.length > 1 ? Em.I18n.t('then') : Em.I18n.t('it'),
        alertMessage = Em.I18n.t('installer.step7.assign.master.body').format(mastersToCreateDisplayName.join(), stringText1, stringText2),
        dependentComponents = this.getDependentComponents(mastersToCreate),
        isManualKerberos = App.get('router.mainAdminKerberosController.isManualKerberos');

    if (dependentComponents.length) {
      alertMessage += '<br/>' + Em.I18n.t('installer.step7.assign.master.dependent.component.body').format(stringUtils.getFormattedStringFromArray(dependentComponents));
    }

    if (isManualKerberos) {
      var warnMessage = Em.I18n.t('common.important.strong') + ': ' + Em.I18n.t('installer.step8.kerberors.warning');
      alertMessage += '<br/>' + Em.I18n.t('common.warn.message').format(warnMessage);
    }
    this.set('alertMessage', alertMessage);
  },

  /**
   * @method getDependentComponents
   * @param {Array} mastersToCreate
   * @returns {Array}
   */
  getDependentComponents: function getDependentComponents(mastersToCreate) {
    var dependentComponents = [];

    mastersToCreate.forEach(function (_component) {
      var dependencies = App.StackServiceComponent.find(_component).get('dependencies').filterProperty('scope', 'host').map(function (item) {
        return App.format.role(item.componentName);
      });
      dependentComponents = dependentComponents.concat(dependencies).uniq();
    }, this);
    return dependentComponents;
  }

});

});

require.register("views/wizard/step7/credentials_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.CredentialsTabOnStep7View = Em.View.extend({

  templateName: require('templates/wizard/step7/credentials_tab'),

  didInsertElement: function didInsertElement() {
    this.setRows();
  },

  /**
   * Row objects to represent table rows
   */
  rows: [],

  /**
   * Array of all used properties on tab to provide binding for enabling/disabling next button
   */
  properties: [],

  /**
   * Set rows in accordance to data fetched from themes API
   */
  setRows: function () {
    if (this.get('controller.stepConfigsCreated')) {
      var result = [];
      var properties = [];
      var self = this;
      var passwordProperty, usernameProperty;
      App.Tab.find().filterProperty('name', 'credentials').forEach(function (tab) {
        if (tab.get('isCategorized')) {
          tab.get('sections').findProperty('name', 'credentials').get('subSections').forEach(function (row) {
            passwordProperty = null;
            usernameProperty = null;
            row.get('configProperties').forEach(function (id) {
              var config = App.configsCollection.getConfig(id);
              var stepConfig = config && self.get('controller.stepConfigs').findProperty('serviceName', Em.get(config, 'serviceName')).get('configs').findProperty('name', Em.get(config, 'name'));
              if (stepConfig) {
                if (stepConfig.get('displayType') === 'password') {
                  passwordProperty = stepConfig;
                } else {
                  usernameProperty = stepConfig;
                }
                properties.push(stepConfig);
              }
            });
            if (passwordProperty) {
              result.push({
                displayName: row.get('displayName'),
                passwordProperty: passwordProperty,
                usernameProperty: usernameProperty
              });
            }
          });
        }
      });
      this.set('rows', result);
      this.set('properties', properties);
    }
  }.observes('controller.stepConfigsCreated'),

  /**
   * Update appropriate credentialsTabNextEnabled flag in accordance to properties validation results
   */
  updateNextDisabled: function () {
    var rows = this.get('rows');
    var result = !rows.someProperty('passwordProperty.error', true) && !rows.someProperty('usernameProperty.error', true);
    this.set('controller.credentialsTabNextEnabled', result);
  }.observes('properties.@each.error')

});

});

require.register("views/wizard/step7/databases_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DatabasesTabOnStep7View = Em.View.extend({

  templateName: require('templates/wizard/step7/databases_tab'),

  tabs: function () {
    return App.Tab.find().filterProperty('themeName', 'database');
  }.property(),

  /**
   * @type {Array}
   */
  properties: [],

  configsView: null,

  didInsertElement: function didInsertElement() {
    var controllerRoute = 'App.router.' + this.get('controller.name');
    this.set('configsView', App.ServiceConfigView.extend({
      templateName: require('templates/common/configs/service_config_wizard'),
      supportsHostOverrides: false,
      controllerBinding: controllerRoute,
      isNotEditableBinding: controllerRoute + '.isNotEditable',
      filterBinding: controllerRoute + '.filter',
      columnsBinding: controllerRoute + '.filterColumns',
      selectedServiceBinding: controllerRoute + '.selectedService',
      serviceConfigsByCategoryView: Em.ContainerView.create(),
      supportsConfigLayout: true,
      willDestroyElement: function willDestroyElement() {
        $('.loading').append(Em.I18n.t('app.loadingPlaceholder'));
        App.store.fastCommit();
      },
      didInsertElement: function didInsertElement() {
        $('.loading').empty();
        this._super();
      },
      tabs: Em.computed.alias('tabModels'),
      hideTabs: function () {
        this.get('tabs').forEach(function (tab) {
          tab.set('isHidden', tab.get('isConfigsPrepared') && tab.get('isHiddenByFilter'));
        });
      }.observes('tabs.@each.isHiddenByFilter'),
      setActiveTab: function setActiveTab(event) {
        if (event.context.get('isHiddenByFilter')) return false;
        this.get('tabs').forEach(function (tab) {
          tab.set('isActive', false);
        });
        var currentTab = event.context;
        currentTab.set('isActive', true);
      }
    }));
    this.setLocalProperties();
  },

  setLocalProperties: function setLocalProperties() {
    var _this = this;

    if (this.get('controller.stepConfigsCreated')) {
      var properties = [];
      this.get('tabs').forEach(function (tab) {
        if (tab.get('isCategorized')) {
          tab.get('sections').forEach(function (section) {
            section.get('subSections').forEach(function (row) {
              row.get('configProperties').forEach(function (id) {
                var config = App.configsCollection.getConfig(id);
                var stepConfig = config && _this.get('controller.stepConfigs').findProperty('serviceName', Em.get(config, 'serviceName')).get('configs').findProperty('name', Em.get(config, 'name'));
                if (stepConfig) {
                  properties.push(stepConfig);
                }
              });
            });
          });
        }
      });
      this.set('properties', properties);
    }
  },

  updateNextDisabled: function () {
    this.set('controller.databasesTabNextEnabled', !this.get('properties').filterProperty('isActive').someProperty('error'));
  }.observes('properties.@each.error', 'properties.@each.isActive')

});

});

require.register("views/wizard/step7/directories_tab_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.DirectoriesTabOnStep7View = Em.View.extend({

  templateName: require('templates/wizard/step7/directories_tab'),

  services: [],

  setServices: function () {
    var services = [];
    var directoriesTabs = App.Tab.find().filterProperty('themeName', 'directories');
    var stepConfigs = this.get('controller.stepConfigs');
    var lastSelectedService = this.get('controller.tabs').findProperty('isActive').get('selectedServiceName');
    stepConfigs.forEach(function (stepConfig) {
      var tab = directoriesTabs.findProperty('serviceName', stepConfig.get('serviceName'));
      if (tab) {
        services.push(tab);
      }
    });
    this.set('services', services);
    this.get('services').setEach('isActive', false);
    if (lastSelectedService) {
      this.set('controller.selectedService', stepConfigs.findProperty('serviceName', lastSelectedService));
    } else {
      this.set('controller.selectedService', stepConfigs[0]);
    }
    this.get('controller').selectedServiceObserver();
  }.observes('controller.stepConfigs'),

  selectService: function selectService(event) {
    this.set('controller.filter', '');
    this.get('controller.filterColumns').setEach('selected', false);
    this.get('services').setEach('isActive', false);
    event.context.set('isActive', true);
    this.set('controller.selectedService', this.get('controller.stepConfigs').findProperty('serviceName', event.context.get('serviceName')));
    this.get('controller.tabs').findProperty('isActive', true).set('selectedServiceName', event.context.get('serviceName'));
  },

  didInsertElement: function didInsertElement() {
    this.setServices();
    Em.run.next(this, function () {
      this.enableRightArrow();
    });
  },

  configsView: App.ServiceConfigView.extend({
    templateName: require('templates/common/configs/service_config_wizard'),
    selectedServiceBinding: 'controller.selectedService',
    filterBinding: 'controller.filter',
    columnsBinding: 'controller.filterColumns',
    hideTabHeaders: true,
    supportsConfigLayout: true,
    themeTemplate: require('templates/wizard/step7/directories_theme_layout'),
    tabs: function () {
      var tabs = App.Tab.find().filterProperty('themeName', 'directories').filterProperty('serviceName', this.get('controller.selectedService.serviceName'));
      return tabs;
    }.property('controller.selectedService.serviceName'),
    updateFilterCounters: function () {
      this.set('isAllConfigsHidden', this.get('tabs').everyProperty('allSectionsAreHiddenByFilter', true));
    }.observes('tabs.@each.allSectionsAreHiddenByFilter')
  }),

  isLeftArrowDisabled: true,

  isRightArrowDisabled: true,

  isNavArrowsHidden: Em.computed.and('isLeftArrowDisabled', 'isRightArrowDisabled'),

  enableRightArrow: function enableRightArrow() {
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    this.set('isRightArrowDisabled', container.width() >= content.width());
  },

  getScrollInterval: function getScrollInterval() {
    var INTERVAL = 300;
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    var gap = content.width() - container.width();
    var gapLeft = gap % INTERVAL;
    var totalScrollsNamber = Math.floor(gap / INTERVAL) || 1;
    return INTERVAL + Math.round(gapLeft / totalScrollsNamber) + 1;
  },

  scrollTabsLeft: function scrollTabsLeft() {
    if (!this.get('isLeftArrowDisabled')) this.scrollTabs('left');
  },

  scrollTabsRight: function scrollTabsRight() {
    if (!this.get('isRightArrowDisabled')) this.scrollTabs('right');
  },

  scrollTabs: function scrollTabs(dir) {
    var container = $(this.get('element')).find('.tabs-container');
    var content = container.find('ul');
    var interval = this.getScrollInterval();
    this.set('isLeftArrowDisabled', dir === 'left' && interval >= container.scrollLeft());
    this.set('isRightArrowDisabled', dir === 'right' && content.width() - container.width() <= container.scrollLeft() + interval);
    container.animate({
      scrollLeft: (dir === 'left' ? '-' : '+') + '=' + interval + 'px'
    });
  }

});

App.DirectoriesLayoutCategoryView = Em.View.extend(App.ConfigOverridable, {
  isCollapsed: false,
  toggleCollapsed: function toggleCollapsed() {
    this.$('.panel-body').toggle('blind', 500);
    this.toggleProperty('isCollapsed');
  }
});

});

require.register("views/wizard/step7_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep7View = Em.View.extend({

  templateName: function () {
    return require(this.get('controller.isInstallWizard') ? 'templates/wizard/step7_with_category_tabs' : 'templates/wizard/step7');
  }.property('controller.content.controllerName'),

  willInsertElement: function willInsertElement() {
    if (this.get('controller.isInstallWizard')) {
      this.get('controller').initTabs();
      this.get('controller').loadStep();
    }
  },

  willDestroyElement: function willDestroyElement() {
    this.get('controller').clearStep();
    this.get('controller').clearLastSelectedService();
  },

  /**
   * Link to model with credentials tab
   */
  credentialsTab: function () {
    return App.Tab.find().findProperty('name', 'credentials_tab');
  }.property()

});

});

require.register("views/wizard/step8_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');

App.WizardStep8View = Em.View.extend({

  templateName: require('templates/wizard/step8'),

  didInsertElement: function didInsertElement() {
    this.get('controller').loadStep();
  },

  /**
   * Print review-report
   * @method printReview
   */
  printReview: function printReview() {
    var o = $("#step8-info");
    o.jqprint();
  },

  repoInfo: function () {
    var repoInfo = this.get('controller.clusterInfo.repoInfo');
    if (!repoInfo) {
      return [];
    }
    return repoInfo.map(function (item) {
      var link = item.get('base_url');
      try {
        var urlObject = new URL(link);
        if (urlObject.username && urlObject.password) {
          urlObject.username = urlObject.username.replace(/./g, "*");
          urlObject.password = urlObject.password.replace(/./g, "*");
          link = urlObject.toString();
        }
      } catch (e) {}

      return {
        os_type: item.get('os_type'),
        repo_id: item.get('repo_id'),
        base_url: link
      };
    });
  }.property('controller.clusterInfo.repoInfo')
});

});

require.register("views/wizard/step9/hostLogPopupBody_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');

App.WizardStep9HostLogPopupBodyView = Em.View.extend({

  classNames: ['col-sm-12'],

  templateName: require('templates/wizard/step9/step9HostTasksLogPopup'),

  /**
   * Does host lost heartbeat
   * @type {bool}
   */
  isHeartbeatLost: Em.computed.equal('parentView.host.status', 'heartbeat_lost'),

  /**
   * Does host doesn't have scheduled tasks for install
   * @type {bool}
   */
  isNoTasksScheduled: Em.computed.alias('parentView.host.isNoTasksForInstall'),

  /**
   * Is log-box hidden
   * @type {bool}
   */
  isLogWrapHidden: true,

  /**
   * Is log-textarea visible
   * @type {bool}
   */
  showTextArea: false,

  /**
   * No tasks shown
   * @type {bool}
   */
  isEmptyList: true,

  /**
   * Is task logs loaded
   * @type {Boolean}
   */
  isTaskLoaded: true,

  /**
   * Checks if no visible tasks are in popup
   * @method visibleTasks
   */
  visibleTasks: function () {
    this.set("isEmptyList", true);
    if (this.get('category.value')) {
      var filter = this.get('category.value');
      var tasks = this.get('tasks');
      tasks.setEach("isVisible", false);

      if (filter == "all") {
        tasks.setEach("isVisible", true);
      } else if (filter == "pending") {
        tasks.filterProperty("status", "pending").setEach("isVisible", true);
        tasks.filterProperty("status", "queued").setEach("isVisible", true);
      } else if (filter == "in_progress") {
        tasks.filterProperty("status", "in_progress").setEach("isVisible", true);
      } else if (filter == "failed") {
        tasks.filterProperty("status", "failed").setEach("isVisible", true);
      } else if (filter == "completed") {
        tasks.filterProperty("status", "completed").setEach("isVisible", true);
      } else if (filter == "aborted") {
        tasks.filterProperty("status", "aborted").setEach("isVisible", true);
      } else if (filter == "timedout") {
        tasks.filterProperty("status", "timedout").setEach("isVisible", true);
      }

      if (tasks.filterProperty("isVisible", true).length > 0) {
        this.set("isEmptyList", false);
      }
    }
  }.observes('category', 'tasks'),

  /**
   * List categories (implements possible values for task status)
   * @type {Em.Object[]}
   */
  categories: [Em.Object.create({ value: 'all', label: Em.I18n.t('installer.step9.hostLog.popup.categories.all') }), Em.Object.create({ value: 'pending', label: Em.I18n.t('installer.step9.hostLog.popup.categories.pending') }), Em.Object.create({ value: 'in_progress', label: Em.I18n.t('installer.step9.hostLog.popup.categories.in_progress') }), Em.Object.create({ value: 'failed', label: Em.I18n.t('installer.step9.hostLog.popup.categories.failed') }), Em.Object.create({ value: 'completed', label: Em.I18n.t('installer.step9.hostLog.popup.categories.completed') }), Em.Object.create({ value: 'aborted', label: Em.I18n.t('installer.step9.hostLog.popup.categories.aborted') }), Em.Object.create({ value: 'timedout', label: Em.I18n.t('installer.step9.hostLog.popup.categories.timedout') })],

  /**
   * Current category
   * @type {Em.Object}
   */
  category: null,

  /**
   * List of tasks
   * @type {Em.Object[]}
   */
  tasks: function () {
    var tasksArr = [];
    var host = this.get('parentView.host');
    var tasks = this.getStartedTasks(host);
    tasks = tasks.sortProperty('Tasks.id');
    if (tasks.length) {
      tasks.forEach(function (_task) {
        var taskInfo = Em.Object.create({});
        taskInfo.set('id', _task.Tasks.id);
        taskInfo.set('requestId', _task.Tasks.request_id);
        taskInfo.set('command', _task.Tasks.command.toLowerCase() === 'service_check' ? '' : _task.Tasks.command.toLowerCase());
        taskInfo.set('commandDetail', App.format.commandDetail(_task.Tasks.command_detail, _task.Tasks.request_inputs, _task.Tasks.ops_display_name));
        taskInfo.set('status', App.format.taskStatus(_task.Tasks.status));
        taskInfo.set('role', App.format.role(_task.Tasks.role, false));
        taskInfo.set('stderr', _task.Tasks.stderr);
        taskInfo.set('stdout', _task.Tasks.stdout);
        taskInfo.set('outputLog', _task.Tasks.output_log);
        taskInfo.set('errorLog', _task.Tasks.error_log);
        taskInfo.set('startTime', date.startTime(_task.Tasks.start_time));
        taskInfo.set('duration', date.durationSummary(_task.Tasks.start_time, _task.Tasks.end_time));
        taskInfo.set('isVisible', true);
        taskInfo.set('icon', '');
        taskInfo.set('hostName', _task.Tasks.host_name);
        if (taskInfo.get('status') == 'pending' || taskInfo.get('status') == 'queued') {
          taskInfo.set('icon', 'glyphicon glyphicon-cog');
        } else if (taskInfo.get('status') == 'in_progress') {
          taskInfo.set('icon', 'icon-cogs');
        } else if (taskInfo.get('status') == 'completed') {
          taskInfo.set('icon', 'glyphicon glyphicon-ok');
        } else if (taskInfo.get('status') == 'failed') {
          taskInfo.set('icon', 'glyphicon glyphicon-exclamation-sign');
        } else if (taskInfo.get('status') == 'aborted') {
          taskInfo.set('icon', 'glyphicon glyphicon-minus');
        } else if (taskInfo.get('status') == 'timedout') {
          taskInfo.set('icon', 'glyphicon glyphicon-time');
        }
        tasksArr.push(taskInfo);
      }, this);
    }
    this.set('isTaskLoaded', true);
    return tasksArr;
  }.property('parentView.c.logTasksChangesCounter'),

  /**
   * Navigate to task list from task view
   * @method backToTaskList
   */
  backToTaskList: function backToTaskList() {
    this.destroyClipBoard();
    this.set("isLogWrapHidden", true);
  },

  /**
   * Get list of host's started tasks
   * @param {object} host
   * @returns {object[]}
   * @method getStartedTasks
   */
  getStartedTasks: function getStartedTasks(host) {
    return host.logTasks.filter(function (task) {
      return task.Tasks.status;
    });
  },

  /**
   * Open new window with task's log
   * @method openTaskLogInDialog
   */
  openTaskLogInDialog: function openTaskLogInDialog() {
    var newWindow = window.open();
    var newDocument = newWindow.document;
    newDocument.write('<pre>' + this.get('formattedLogsForOpenedTask') + '<pre>');
    newDocument.close();
  },

  /**
   * Currently open task
   * @type {Em.Object}
   */
  openedTask: function () {
    return this.get('tasks').findProperty('id', this.get('parentView.c.currentOpenTaskId'));
  }.property('parentView.c.currentOpenTaskId', 'tasks.[]'),

  /**
   * @type {string}
   */
  formattedLogsForOpenedTask: function () {
    var stderr = this.get('openedTask.stderr');
    var stdout = this.get('openedTask.stdout');
    return 'stderr: \n' + stderr + '\n stdout:\n' + stdout;
  }.property('openedTask.stderr', 'openedTask.stdout'),

  /**
   * Click-handler for toggle task's log view (textarea to box and back)
   * @param {object} event
   * @method toggleTaskLog
   */
  toggleTaskLog: function toggleTaskLog(event) {
    if (this.get('isLogWrapHidden')) {
      var taskInfo = event.context;
      this.set("isLogWrapHidden", false);
      this.set('parentView.c.currentOpenTaskId', taskInfo.id);
      this.set('parentView.c.currentOpenTaskRequestId', taskInfo.requestId);
      this.set('isTaskLoaded', false);
      this.get('parentView.c').loadCurrentTaskLog();
      $(".modal").scrollTop(0);
      $(".modal-body").scrollTop(0);
    } else {
      this.set("isLogWrapHidden", true);
      this.set('parentView.c.currentOpenTaskId', 0);
      this.set('parentView.c.currentOpenTaskRequestId', 0);
    }
  },

  /**
   * Create (if doesn't exist) or destroy (if exists) clipboard textarea
   * @method textTrigger
   */
  textTrigger: function textTrigger() {
    if (this.get('showClipBoard')) {
      this.destroyClipBoard();
    } else {
      this.createClipBoard();
    }
  },

  /**
   * @type {boolean}
   */
  showClipBoard: false,

  /**
   * Create clipboard with task's log
   * @method createClipBoard
   */
  createClipBoard: function createClipBoard() {
    this.set('showClipBoard', true);
    $('.task-detail-log-maintext').css('display', 'none');
  },

  /**
   * Destroy clipboard with task's log
   * @method destroyClipBoard
   */
  destroyClipBoard: function destroyClipBoard() {
    this.set('showClipBoard', false);
    $('.task-detail-log-maintext').css('display', 'block');
  }
});

});

require.register("views/wizard/step9_view", function(exports, require, module) {
'use strict';

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

var App = require('app');
var date = require('utils/date/date');

App.WizardStep9View = App.TableView.extend({

  templateName: require('templates/wizard/step9'),

  /**
   * Overall progress-bar color
   * @type {string}
   */
  barColor: '',

  /**
   * Overall result message
   * @type {string}
   */
  resultMsg: '',

  /**
   * Overall message color
   * @type {string}
   */
  resultMsgColor: '',

  /**
   * When progress is 100, step is completed
   * @type {bool}
   */
  isStepCompleted: Em.computed.equal('controller.progress', '100'),

  /**
   * Number of visible hosts
   * @type {string}
   */
  displayLength: "25",

  /**
   * Same to <code>controller.hosts</code>
   * @type {object[]}
   */
  content: function () {
    return this.get('controller.hosts');
  }.property('controller.hosts'),

  /**
   * Active category
   * @type {Ember.Object}
   */
  selectedCategory: Em.computed.findBy('categories', 'isActive', true),

  /**
   * Ember Object category. This object also contains
   * <code>
   *   hostStatus: {String} A valid status of a host.
   *               Used to filter hosts in that status.
   *   hostsCount: {Int} Dynamic count of hosts displayed in the category label
   *   label : {String} status and hosts in that status displayed which consists as a category on the page
   *   isActive: {boolean} Gets set when the category is selected/clicked by the user
   *   itemClass: {computed property} Binds the category link to active class when user clicks on the link
   * </code>
   */
  categoryObject: Em.Object.extend({
    hostsCount: 0,
    label: Em.computed.format('{0} ({1})', 'value', 'hostsCount'),
    isActive: false,
    itemClass: Em.computed.ifThenElse('isActive', 'active', '')
  }),

  /**
   * Domputed property creates the category objects on the load of the page and sets 'All' as the active category
   * @Returns {Em.Object[]} All created categories which are binded and iterated in the template
   */
  categories: function () {
    return [this.categoryObject.create({ value: Em.I18n.t('common.all'), hostStatus: 'all', isActive: true }), this.categoryObject.create({ value: Em.I18n.t('installer.step9.hosts.status.label.inProgress'), hostStatus: 'inProgress' }), this.categoryObject.create({ value: Em.I18n.t('installer.step9.hosts.status.label.warning'), hostStatus: 'warning' }), this.categoryObject.create({ value: Em.I18n.t('common.success'), hostStatus: 'success' }), this.categoryObject.create({ value: Em.I18n.t('common.fail'), hostStatus: 'failed', last: true })];
  }.property(),

  /**
   * True if <code>controller.hostsWithHeartbeatLost</code> contains some values
   * @type {bool}
   */
  isHostHeartbeatLost: Em.computed.bool('controller.hostsWithHeartbeatLost.length'),

  /**
   * Css-string to overall progress-bar width-property
   * @type {string}
   */
  barWidth: Em.computed.format('width: {0}%;', 'controller.progress'),

  /**
   * Filter hosts info shown up on bottom of the box. Set by filter function, when 'seletedCategory' changed
   * @type {string}
   */
  filteredHostsInfo: '',

  /**
   * Message for overall progress
   * @type {string}
   */
  progressMessage: Em.computed.i18nFormat('installer.step9.overallProgress', 'controller.progress'),

  /**
   * Run <code>countCategoryHosts</code>, <code>filter</code> only once
   * @method hostStatusObserver
   */
  hostStatusObserver: function () {
    Em.run.once(this, 'countCategoryHosts');
    Em.run.once(this, 'filter');
  }.observes('content.@each.status'),

  /**
   * Count each category hosts to update label
   * @method countCategoryHosts
   */
  countCategoryHosts: function countCategoryHosts() {
    var counters = {
      "info": 0,
      "pending": 0,
      "in_progress": 0,
      "heartbeat_lost": 0,
      "warning": 0,
      "success": 0,
      "failed": 0
    };

    this.get('content').forEach(function (host) {
      if (counters[host.get('status')] !== undefined) {
        counters[host.get('status')]++;
      }
    }, this);
    counters["all"] = this.get('content.length');
    counters["inProgress"] = counters["info"] + counters["pending"] + counters["in_progress"];
    counters["failed"] += counters["heartbeat_lost"];
    this.get('categories').forEach(function (category) {
      category.set('hostsCount', counters[category.get('hostStatus')]);
    }, this);
  },

  /**
   * Filter hosts by category
   * @method filter
   */
  filter: function () {
    var self = this;
    Em.run.next(function () {
      self.doFilter();
    });
  }.observes('selectedCategory'),

  /**
   * Real filter-method
   * Called from <code>filter</code> in Em.run.next-wrapper
   * @method doFilter
   */
  doFilter: function doFilter() {
    var result = [];
    var content = this.get('content');
    var selectedCategory = this.get('selectedCategory');
    if (!selectedCategory || selectedCategory.get('hostStatus') === 'all') {
      result = content;
    } else if (selectedCategory.get('hostStatus') == 'inProgress') {
      result = content.filter(function (_host) {
        return _host.get('status') == 'info' || _host.get('status') == 'pending' || _host.get('status') == 'in_progress';
      });
    } else if (selectedCategory.get('hostStatus') == 'failed') {
      result = content.filter(function (_host) {
        return _host.get('status') == 'failed' || _host.get('status') == 'heartbeat_lost';
      });
    } else {
      result = content.filterProperty('status', selectedCategory.get('hostStatus'));
    }
    this.set('filteredContent', result);
    this.set('filteredHostsInfo', Em.I18n.t('installer.step9.hosts.filteredHostsInfo').format(result.get('length'), content.get('length')));
  },

  /**
   * On click handler for 'show all' link
   * @method showAllHosts
   */
  showAllHosts: function showAllHosts() {
    this.get('categories').forEach(function (category) {
      category.set('isActive', category.get('hostStatus') === 'all');
    });
  },

  /**
   * Trigger on Category click
   * @param {Object} event
   * @method selectCategory
   */
  selectCategory: function selectCategory(event) {
    var categoryStatus = event.context.get('hostStatus');
    this.get('categories').forEach(function (category) {
      category.set('isActive', category.get('hostStatus') === categoryStatus);
    });
  },

  didInsertElement: function didInsertElement() {
    this.onStatus();
    this.get('controller').navigateStep();
  },

  /**
   * Set <code>resultMsg</code>, <code>resultMsg</code>, <code>resultMsgColor</code> according to
   * <code>controller.status</code>, <code>controller.startCallFailed</code>, <code>isHostHeartbeatLost</code>
   * @method onStatus
   */
  onStatus: function () {
    if (this.get('controller.status') === 'info') {
      this.set('resultMsg', '');
      this.set('barColor', 'progress-bar-info');
    } else if (this.get('controller.status') === 'warning') {
      this.set('barColor', 'progress-bar-warning');
      this.set('resultMsg', Em.I18n.t('installer.step9.status.warning'));
      this.set('resultMsgColor', 'alert-warning');
    } else if (this.get('controller.status') === 'failed') {
      this.set('barColor', 'progress-bar-danger');
      this.set('resultMsgColor', 'alert-danger');
      if (this.get('isHostHeartbeatLost')) {
        // When present requests succeeds but some host components are in UNKNOWN or INSTALL_FAILED state and
        // hosts are in HEARTBEAT_LOST state
        this.set('resultMsg', Em.I18n.t('installer.step9.status.hosts.heartbeat_lost').format(this.get('controller.hostsWithHeartbeatLost').length));
      } else if (this.get('controller.startCallFailed')) {
        this.set('resultMsg', Em.I18n.t('installer.step9.status.start.services.failed'));
      } else {
        this.set('resultMsg', Em.I18n.t('installer.step9.status.failed'));
      }
    } else if (this.get('controller.status') === 'success') {
      this.set('barColor', 'progress-bar-success');
      this.set('resultMsg', this.get('controller.content.cluster.status') === 'START_SKIPPED' ? Em.I18n.t('installer.step9.status.skipStartSuccess') : Em.I18n.t('installer.step9.status.success'));
      this.set('resultMsgColor', 'alert-success');
    }
  }.observes('controller.status', 'controller.content.cluster.status', 'controller.startCallFailed', 'isHostHeartbeatLost'),

  /**
   * Show popup with info about failed hosts
   * @return {App.ModalPopup}
   * @method hostWithInstallFailed
   */
  hostWithInstallFailed: function hostWithInstallFailed() {
    var controller = this.get('controller');
    return App.ModalPopup.show({
      header: Em.I18n.t('installer.step9.host.heartbeat_lost.header'),
      classNames: ['common-modal-wrapper'],
      modalDialogClasses: ['modal-lg'],
      autoHeight: false,
      secondary: null,

      bodyClass: Em.View.extend({
        templateName: require('templates/wizard/step9/step9_install_host_popup'),
        c: controller,
        failedHosts: function () {
          return controller.get('hostsWithHeartbeatLost');
        }.property()
      })
    });
  }

});

function hostStatus(statuses) {
  return Em.computed('isHostCompleted', 'obj.status', function () {
    statuses = Em.makeArray(statuses);
    return this.get('isHostCompleted') && statuses.contains(this.get('obj.status'));
  });
}

App.HostStatusView = Em.View.extend({

  tagName: 'tr',

  /**
   * Current host
   * @type {Em.Object}
   */
  obj: null,

  /**
   * wizardStep9Controller
   * @type {App.WizardStep9Controller}
   */
  controller: null,

  /**
   * Progress-bar color for current host
   * @type {string}
   */
  barColor: '',

  /**
   * Css-string to progress-bar width-property
   * @type {string}
   */
  barWidth: Em.computed.format('width: {0}%;', 'obj.progress'),

  /**
   * Is current host failed
   * @type {bool}
   */
  isFailed: hostStatus(['failed', 'heartbeat_lost']),

  /**
   * Is current host successfully installed
   * @type {bool}
   */
  isSuccess: hostStatus('success'),

  /**
   * Current host has warnings
   * @type {bool}
   */
  isWarning: hostStatus('warning'),

  /**
   * Current host completed all its tasks
   * @type {bool}
   */
  isHostCompleted: function () {
    return this.get('obj.progress') == 100;
  }.property('obj.progress'),

  didInsertElement: function didInsertElement() {
    this.onStatus();
  },

  /**
   * Set <code>barColor</code>, <code>obj.progress</code>, <code>obj.message</code> according to
   * <code>obj.status</code>, <code>obj.progress</code>, <code>controller.progress</code>
   * @method onStatus
   */
  onStatus: function () {
    if (this.get('obj.status') === 'info') {
      this.set('barColor', 'progress-bar-info');
    } else if (this.get('obj.status') === 'warning') {
      this.set('barColor', 'progress-bar-warning');
      if (this.get('obj.progress') === '100') {
        this.set('obj.message', Em.I18n.t('installer.step9.host.status.warning'));
      }
    } else if (this.get('obj.status') === 'failed') {
      this.set('barColor', 'progress-bar-danger');
      if (this.get('obj.progress') === '100') {
        this.set('obj.message', Em.I18n.t('installer.step9.host.status.failed'));
      }
    } else if (this.get('obj.status') === 'heartbeat_lost') {
      this.set('barColor', 'progress-bar-danger');
      if (this.get('obj.progress') === '100') {
        this.set('obj.message', Em.I18n.t('installer.step9.host.heartbeat_lost'));
      }
    } else if (this.get('obj.status') === 'success' && this.get('isHostCompleted') && parseInt(this.get('controller.progress')) > 34) {
      this.set('barColor', 'progress-bar-success');
      this.set('obj.message', Em.I18n.t('installer.step9.host.status.success'));
    }
  }.observes('obj.status', 'obj.progress', 'controller.progress'),

  /**
   * Show popup with host logs
   * @return {App.ModalPopup}
   * @method hostLogPopup
   */
  hostLogPopup: function hostLogPopup() {
    var controller = this.get('controller');
    var host = this.get('obj');

    return App.ModalPopup.show({

      header: host.get('name'),

      classNames: ['common-modal-wrapper'],
      modalDialogClasses: ['modal-lg'],
      'data-qa': 'host-log-modal',

      autoHeight: false,

      secondary: null,

      /**
       * Current host
       * @type {Em.Object}
       */
      host: host,

      /**
       * wizardStep9Controller
       * @type {App.WizardStep9Controller}
       */
      c: controller,

      onClose: function onClose() {
        this.set('c.currentOpenTaskId', 0);
        this.hide();
      },

      bodyClass: App.WizardStep9HostLogPopupBodyView
    });
  }

});

});

